package com.safran.morpho.cutting;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.json.simple.JSONValue;

import com.safran.morpho.utils.RequestUtils;


/**
 * decompose image servlet.
 * @author DIN
 *
 */
@WebServlet(urlPatterns = "/decompose", asyncSupported = true)
public class DecomposeServlet extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5349752248636959148L;
	
	private Logger LOG = Logger.getLogger(DecomposeServlet.class.getName());
	
	 /**
     * {@inheritDoc}
     */
    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        treatment(request, response);
    } 

    /**
     * {@inheritDoc}
     */
    @Override
    public void doPost( HttpServletRequest request,  HttpServletResponse response) throws ServletException, IOException { 
        treatment(request, response);
    }
        
    /**
     * {@inheritDoc}
     */
    @Override
    public void service( HttpServletRequest request,  HttpServletResponse response) throws ServletException, IOException {
        treatment(request, response);
    }
  
    /**
     * Treatment decompose.
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @throws IOException if failed
     */
    private void treatment(final HttpServletRequest request,  final HttpServletResponse response) throws IOException {
        
    	// extract data from request
    	String fileData = RequestUtils.getBody(request);
    	
    	// extract parameter data
    	int numberDatabase 	= Integer.valueOf(RequestUtils.getParameter(request, "numberDatabase"));
    	int threshold 		= Integer.valueOf(RequestUtils.getParameter(request, "threshold"));
    	int fileSize 		= Integer.valueOf(RequestUtils.getParameter(request, "fileSize"));
    	
    	LOG.info(" decompose parameter => numberDatabase : "+ numberDatabase);
    	LOG.info(" decompose parameter => threshold : " 	+ threshold);
    	LOG.info(" decompose parameter => fileSize : " 		+ fileSize);
    	
    	// treatment => JNI method
    	String[] returnValue = JNIDecompose.decompose(numberDatabase, threshold, fileData, fileSize);
    	
    	final Map<Integer,String> obj = new LinkedHashMap<Integer, String>();
    	
		for (int i = 0; i < returnValue.length; i++) {
			obj.put(i, returnValue[i]);
		}
    	
    	final String jsonText = JSONValue.toJSONString(obj);
    	
    	LOG.info("response from decompose method : " + jsonText);
    	
    	// add result to response
    	response.setContentType("text/x-json;charset=UTF-8");           
        response.setHeader("Cache-Control", "no-cache");
        response.getWriter().write(jsonText);
    }
  
 }
