package com.safran.morpho.cutting;

import java.io.IOException;
import java.util.logging.Logger;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.safran.morpho.utils.RequestUtils;

/**
 * recompose image servlet.
 * @author DIN
 *
 */
@WebServlet(urlPatterns = "/recompose", asyncSupported = true)
public class RecomposeServlet extends HttpServlet {

	private static final long serialVersionUID = 5349752248636959148L;
	
	private Logger LOG = Logger.getLogger(RecomposeServlet.class.getName());

	/**
     * {@inheritDoc}
     */
    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        treatment(request, response);
    } 

    /**
     * {@inheritDoc}
     */
    @Override
    public void doPost( HttpServletRequest request,  HttpServletResponse response) throws ServletException, IOException { 
        treatment(request, response);
    }
        
    /**
     * {@inheritDoc}
     */
    @Override
    public void service( HttpServletRequest request,  HttpServletResponse response) throws ServletException, IOException {
        treatment(request, response);
    }
  
    /**
     * Treatment recompose.
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @throws IOException if failed
     */
    private void treatment(final HttpServletRequest request,  final HttpServletResponse response) throws IOException {
        
    	// Get data
    	final String data = RequestUtils.getBody(request);
    	
    	// extract parameter data
    	int numberDatabase 	= Integer.valueOf(RequestUtils.getParameter(request, "numberDatabase"));
    	int threshold 		= Integer.valueOf(RequestUtils.getParameter(request, "threshold"));
    	
    	LOG.info(" decompose parameter => numberDatabase :" + numberDatabase);
    	LOG.info(" decompose parameter => threshold :" 		+ threshold);
    	
        String[] dataBlockList = data.split(",");
        
        // treatment => JNI method
    	String mergeFile = JNIRecompose.recompose(dataBlockList, numberDatabase, threshold);

    	LOG.info("response from recompose method : " + mergeFile);
    	
    	// add result to response
    	response.setContentType("text/x-json;charset=UTF-8");           
        response.setHeader("Cache-Control", "no-cache");
        response.getWriter().write(mergeFile);
    }


 }
