package com.safran.morpho.matching;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.AsyncContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.tomcat.util.http.fileupload.FileUtils;
import org.json.simple.JSONValue;

import com.safran.morpho.cutting.JNIRecompose;
import com.safran.morpho.utils.RequestUtils;
import com.sun.org.apache.xml.internal.security.exceptions.Base64DecodingException;
import com.sun.org.apache.xml.internal.security.utils.Base64;

public class AsyncRequestProcessor implements Runnable {
    
    private AsyncContext asyncContext;
    
    private Logger LOG = Logger.getLogger(AsyncRequestProcessor.class.getName());
 
    /**
     * Constructor default.
     */
    public AsyncRequestProcessor() {
    }
 
    /**
     * Constructor.
     * @param asyncCtx AsyncContext
     * @param data Image Match and Ref
     */
    public AsyncRequestProcessor(final AsyncContext asyncCtx) {
        this.asyncContext = asyncCtx;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void run() {
        try {
            process();
        } catch (final IOException exception) {
        }
        
        //complete the processing
        asyncContext.complete();
    }

    
    /**
     * Process matching when receive data from client.
     */
    private void process() throws IOException {
        int ret = -1;
  
        final HttpServletRequest request   = (HttpServletRequest) asyncContext.getRequest();
        final HttpServletResponse response = (HttpServletResponse) asyncContext.getResponse();
        
        // Test path to store image
        if (null == getPath()) {
            sendResponse(null, null, null, "pb getting path storing image", false, -1 ,response);
            return;
        }
        
        // Remove all images
        removeAllImages();
        
        // Get data
        final String data = RequestUtils.getBody(request);
        
         // 1. Split in two parts : Match + Ref -----------------------------------------------------
        if (null != data) {
            final String[] dataBase64 = data.split(";");
            LOG.info(" dataBase64 is received, number :" + dataBase64.length);
            
            final int numberDatabase    = Integer.valueOf(RequestUtils.getParameter(request,"numberDatabase"));
            final int threshold 		= Integer.valueOf(RequestUtils.getParameter(request,"threshold"));
            
            LOG.info("numberDatabase :" + numberDatabase +" threshold  " + threshold);
            
            if (null != dataBase64 && dataBase64.length == 2) {
            	final String dataMatchBase64            = dataBase64[0];
            	
            	LOG.info(" dataMatchBase64 :" + dataBase64[0]);
            	LOG.info(" before recompose dataBase64[1] :" + dataBase64[1]);
            	
                String[] blockList = dataBase64[1].split(",");
				final String dataRecompose = JNIRecompose.recompose(blockList, numberDatabase, threshold);
                final String dataRefBase64 = dataRecompose.replace("data:image/jpeg;base64,", "");
                LOG.info(" after recompose dataBase64 :" + dataRefBase64);
                //final String dataRefBase64              = dataBase64[1];
                final String fileinputMatch             = RequestUtils.getParameter(request,"fileinputMatch");
                final String fileinputRef               = RequestUtils.getParameter(request,"fileinputRef");
                
                final String widthMatch                 = RequestUtils.getParameter(request,"widthMatch");
                final String widthRef                   = RequestUtils.getParameter(request,"widthRef");
                final String heightMatch                = RequestUtils.getParameter(request,"heightMatch");
                final String heightRef                  = RequestUtils.getParameter(request,"heightRef");
                final String document_id                = RequestUtils.getParameter(request,"document_id");
                
                final String fileinputMatchWithoutExt   = getPath() + getFileNameWithoutExt(fileinputMatch);
                final String fileinputRefWithoutExt     = getPath() + getFileNameWithoutExt(fileinputRef);
                final String fileNameMatch              = saveFile(dataMatchBase64, fileinputMatch);
                final String fileNameRef                = saveFile(dataRefBase64  , fileinputRef);
                
                // Iterate two .....................................................................
                // 2. For Match
                // 2.1 Apply Djpeg
                ret = applyDjpeg(fileNameMatch, fileinputMatchWithoutExt);
                if ( ret == 0) {
                    // 2.2 Apply Cwsq
                    ret = applyCwsq(fileinputMatchWithoutExt, Integer.parseInt(widthMatch), Integer.parseInt(heightMatch));
                    if ( ret == 0) {
                        // 2.3 Apply mindtct
                        if (applyMindtct(fileinputMatchWithoutExt) != 0) {
                            sendResponse(fileinputMatch, fileinputRef, document_id, "pb applyMindtct", false, ret ,response);
                            return;
                        }
                    } else {
                        sendResponse(fileinputMatch, fileinputRef, document_id, "pb applyCwsq", false, ret ,response);
                        return;
                    }
                   
                } else {
                    sendResponse(fileinputMatch, fileinputRef, document_id, "pb applyDjpeg", false, ret ,response);
                    return;
                }
                
                // 3. For Reference
                // 3.1 Apply Djpeg
                ret = applyDjpeg(fileNameRef, fileinputRefWithoutExt);
                if ( ret == 0) {
                    // 3.2 Apply Cwsq
                    ret = applyCwsq(fileinputRefWithoutExt, Integer.parseInt(widthRef), Integer.parseInt(heightRef));
                    if ( ret == 0) {
                        // 3.3 Apply mindtct
                        if ( applyMindtct(fileinputRefWithoutExt) != 0) {
                            sendResponse(fileinputMatch, fileinputRef, document_id, "pb applyMindtct", false, ret ,response);
                         }
                    } else {
                        sendResponse(fileinputMatch, fileinputRef, document_id, "pb applyCwsq", false, ret ,response);
                    }
                   
                } else {
                    sendResponse(fileinputMatch, fileinputRef, document_id, "pb applyDjpeg", false, ret ,response);
                }
                
                //  ..................................................................... Iterate two
                
                // 4. Scoring ......................................................................
                if (ret == 0) {
                    int score = applyBozorth3(fileinputMatchWithoutExt, fileinputRefWithoutExt);
                    sendResponse(fileinputMatch, fileinputRef, document_id, "OK", true, score ,response);
                } else {
                    sendResponse(fileinputMatch, fileinputRef, document_id, "pb matching", false, ret ,response);
                }
                // ...................................................................... 4. Scoring
            }
        }
        //  ----------------------------------------------------- 1. Split in two parts : Match + Ref
    }

    
    /**
     * Get directory to store file.
     * @return directory to store file
     */
    private String getDirStore() {
        String dir = null;
      
        try {
            Context ctx = new InitialContext();
            ctx = (Context) ctx.lookup("java:comp/env");
            dir = (String) ctx.lookup("dirStoreFile");
        } catch (NamingException exception) {
            System.out.println("Impossible lookup dir to store file because we have : " + exception.getMessage());
        }        
        return dir;
    }
    

    /**
     * Get fileName without extension.
    * @return fileName without extension
     */
    private String getFileNameWithoutExt(String fileName) {
        
        if (fileName != null) {
            final String[] files = fileName.split("\\."); // escape character the dot
            if (null != files && files.length == 2) {
                fileName = files[0];
            }
        }
        return fileName;
    }
    
    /**
     * Save stream into file to server.
     * @param dataBase64
     * @return FileName with path
     */
    private String saveFile(final String dataBase64, final String fileName) {
        
        FileOutputStream fos = null;
        String fileNameWithPath = null;
        if (null != dataBase64) {
            try {
                final byte[] dataByte = Base64.decode(dataBase64);
                fileNameWithPath      = getPath() + fileName;
                fos = new FileOutputStream(fileNameWithPath);
                fos.write(dataByte);
            } catch (final Base64DecodingException exception) {
                System.out.println("PB decoding data because we have :" + exception.getMessage());
                fileNameWithPath = null;
            } catch (final FileNotFoundException exception) {
                System.out.println("PB file because we have :" + exception.getMessage());
                fileNameWithPath = null;
            } catch (final IOException exception) {
                System.out.println("PB exception because we have :" + exception.getMessage());
                fileNameWithPath = null;
            } finally {
                try {
                    fos.close();
                } catch (IOException exception) {
                }
            }
        }
        return fileNameWithPath;
    }
    
    /**
     * GetPath to store image.
     * @return path
     */
    private String getPath() {
        final String path = getDirStore() + File.separator;
        return path;
    }
    
    /**
     * Apply process Djpeg.
     * @param filename File name input
     * @param filenameWithoutExt File name output
     * @return 0 if OK
     */
    private int applyDjpeg(final String filename, final String filenameWithoutExt) {
        return JNIStartDjpeg.djpeg("greyscale", "ppm", filenameWithoutExt + ".ppm", filename);
    }

    /**
     * Apply process Cwsq.
     * @param filenameWithoutExt File name input
     * @param width With of Image
     *  @param height Height of Image
     * @return 0 if OK
     */
    private int applyCwsq(final String filenameWithoutExt, final int width, final int height) {
        return JNIStartCwsq.cwsq(0.75f, "wsq", filenameWithoutExt + ".ppm", 1, width,height,8);
    }

    /**
     * Apply process Mindtct.
     * @param filenameWithoutExt File name input
     * @return 0 if OK
     */
    private int applyMindtct(final String filenameWithoutExt) {
        return JNIStartMindtct.mindtct(filenameWithoutExt + ".wsq", filenameWithoutExt);
    }

    /**
     * Apply process Bozorth3.
     * Compare two images
     * @param filenameMatchWithoutExt Filename for matching
     * @param filenameRefWithoutExt Filename Ref use for matching
     * @return scoring of matching
     */
    private int applyBozorth3(final String filenameMatchWithoutExt, final String filenameRefWithoutExt) {
        return JNIStartBozorth3.bozorth3(filenameMatchWithoutExt + ".xyt", filenameRefWithoutExt + ".xyt");
    }
    
    /**
     * Response send to client.
     * @param fileinputMatch File Matching in input
     * @param fileinputRef File Reference in input
     * @param document_id ID image in database
     * @param message Message send
     * @param success Treatment with success
     * @param score Score of matching
     * @param response HttpServletResponse
     * @throws IOException if failed
     */
    private void sendResponse(final String fileinputMatch, final String fileinputRef, final String document_id, final String message, final boolean success, final int score, HttpServletResponse response) throws IOException {
        
        // Object response -------------------------------------------------------------------------
        final MatchingResult result = new MatchingResult();
        result.setFileinputMatch(fileinputMatch);
        result.setFileinputRef(fileinputRef);
        result.setDocument_id(document_id);
        result.setSuccess(success);
        result.setScore(score);
        result.setMessage(message);
        //  ------------------------------------------------------------------------- Object response
 
        // Convert to format Json ------------------------------------------------------------------
        final Map<String,String> obj = result.getJsonData();
        final String jsonText = JSONValue.toJSONString(obj);
        // ------------------------------------------------------------------ Convert to format Json
        
        // Prepare writer --------------------------------------------------------------------------
        response.setContentType("text/x-json;charset=UTF-8");           
        response.setHeader("Cache-Control", "no-cache");
        response.getWriter().write(jsonText);
        // -------------------------------------------------------------------------- Prepare writer
        
    }
    
    /**
     * Remove all images in directory save
     */
    private void removeAllImages() {
        final File directory = new File(getPath());
        
        try {
            FileUtils.cleanDirectory(directory);
        } catch (final IOException exception) {
            System.out.println("Delete images impossible because we have : " + exception.getMessage());
        } 
    }
    
}
