//     All Rights Reserved © Copyright 2013 - Morpho - Paris France
//
//     This module is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.

/* Declarations variables globales -------------------------------------------------------------- */
var confrest        = [];
var lstClsImageId   = [];
var width           = 0;
var height          = 0;
var seuilAcc        = 0;   // seuil acceptable
var seuilMatch      = 0;   // seuil matching
var seuilAccValue   = 250; // value by default
var seuilMatchValue = 400; // value by default

var shamirThreshold = 1;
/* -------------------------------------------------------------- Declarations variables globales */

/* Start Initialize ----------------------------------------------------------------------------- */
initialize();

/**
 * Initialize.
 */
function initialize() {
    
    // URL where store URL Images
    var url = "/restsqlConf/res/ConfRest";
    // Execute request
    return $.ajax({
        url: url,
        type: "GET",
        dataType: "json",
        success:function(data) {
            confrest = data;
            return data; 
        }
    });
}
/* ----------------------------------------------------------------------------- Start Initialize */

/* Functions gloables --------------------------------------------------------------------------- */
/**
 * Instantiate object JIO.
 * @param url Acces service Restsql
 * @param database Name database
 * @returns object JIO
 */
function getInstanceJIO(url, database) {
    
    var jio = null;
    var spec = {
        type: 'restsql', 
        url: url, 
        database: database
    };
      
    if (jio) { 
        jio.close(); 
    }
    jio = jIO.newJio(spec);
    
    return jio;
}

/**
 * Initialize opts.
 */
function initializeOpts() {
    var opts = {};

    opts.query              = "";
    opts.include_docs       = false;
    opts.wildcard_character = "%";
    opts.sort_on            = ["auteur","ascending"],["titre","descending"];
    opts.select_list        = ["auteur","titre"];
    opts.limit              = [0,100];
    opts.conflicts          = false;
    opts.revs               = false;
    opts.revs_info          = false;
    opts.max_retry          = 1;
    
    return opts;
}


// patch for android
/**
 * Is it an image reading?.
 * @param data Image
 * @return true if it's an image
 */
function isImageData(data) {
    return true;
}

/**
 * Clear panel cutting.
 */
function clearCutting() {
  
    // Clear file
    clearData("fileC");
    
    // Clear Title
    clearData("cutting-panel");

    // Clear author
    //clearData("author");

    // Clear Image
    var img = document.getElementById("img");
    img.src = "";
    
    // Number cut
    Ext.getCmp('cutting-panel').getForm().findField('decoupage').setValue(confrest.confrests.length);
    
}

/**
 * Clear panel matching.
 */
function clearMatching() {
  
    // Clear file
    clearData("file"); 
    
    // Reset seuil value
    var txtSeuilA = Ext.ComponentQuery.query('textfield[id=seuilA]')[0];
    txtSeuilA.setValue(seuilAccValue);
    var txtSeuilM = Ext.ComponentQuery.query('textfield[id=seuilM]')[0];
    txtSeuilM.setValue(seuilMatchValue);
    
    // Clean Image and Description
    clearMatchingStart();
    
}

/**
 * Clear panel matching only Image and Description.
 */
function clearMatchingStart() {
  
    // Clear Image
    //patch for android
    //var img = document.getElementById("imgM");
    //img.src = "";
    
    // Clear grid
    var gridMatching = Ext.getCmp('matching-panel').items.items[1].items.items[0];
    if (null != gridMatching) {
        gridMatching.store.clearData();
        gridMatching.view.refresh();
    }
    
}

/**
 * Clear value.
 * @param id Element in the form to be cleared
 */
function clearData(id){
    
    // Get the file upload element
    fileField = document.getElementById(id);
    
    // Get the file upload parent element
    parentNod = fileField.parentNode;
    
    // Create new element
    tmpForm   = document.createElement("form");
    parentNod.replaceChild(tmpForm,fileField);
    tmpForm.appendChild(fileField);
    tmpForm.reset();
    parentNod.replaceChild(fileField,tmpForm);
}

/**
 * Display data.
 * @param value Value displayed
 * @param idElt Element
 */
function setMsg(value, idElt) {
    var msg = Ext.getCmp(idElt);
    msg.setValue(msg.getValue() + "\n" + value);
}
/* --------------------------------------------------------------------------- Functions gloables */

/* Class image ---------------------------------------------------------------------------------- */
/**
 * Class with data image.
 */
function clsImageId() {
    this.document_id    = null;
    this.fileName       = null;
    this.width          = 0;
    this.height         = 0;
    this.lstPartImage   = [];
    this.countPartImage = 0;
    this.merge          = false;
}

clsImageId.prototype.getImageBase64  = function() {
    var value = "";
    
    for(var i=0;i<this.lstPartImage.length;i++) {
        value = value + this.lstPartImage[i];
    }
    return value;
};

clsImageId.prototype.getImageBase64Shamir  = function() {
    
    if(this.lstPartImage.length == 0){
        return "";
    }
    
    var value = "";
    for(var i=0;i<this.lstPartImage.length;i++) {
        if(this.lstPartImage[i] != null && this.lstPartImage[i] != undefined) {
            if(value.length == 0) {
                value = this.lstPartImage[i];
            } else {
                value = value + ',' + this.lstPartImage[i];
            }
        }  
    }
    return value;
};

clsImageId.prototype.setPartImageBase64  = function(colonne, data) {
    this.lstPartImage[colonne] = data;
};
clsImageId.prototype.setDocumentId  = function(documentId) {
    this.document_id = documentId;
};
clsImageId.prototype.getCountImage  = function() {
    return this.countPartImage;
};
clsImageId.prototype.incCountImage  = function() {
    return this.countPartImage++;
};
clsImageId.prototype.setFileName  = function(fileName) {
    this.fileName = fileName;
};
clsImageId.prototype.getFileName  = function() {
    return this.fileName;
};
clsImageId.prototype.setWidth  = function(width) {
    this.width = width;
};
clsImageId.prototype.getWidth  = function() {
    return this.width;
};
clsImageId.prototype.setHeight  = function(height) {
    this.height = height;
};
clsImageId.prototype.getHeight  = function() {
    return this.height;
};
clsImageId.prototype.setMerge  = function(merge) {
    this.merge = merge;
};
clsImageId.prototype.isMerge  = function() {
    return this.merge;
};
/* ---------------------------------------------------------------------------------- Class image */
