package org.resilience.fileutilities;

import java.io.File;
import java.io.IOException;
import java.lang.ProcessBuilder.Redirect;

import org.resilience.RateToolException;

public class Fortify {

	private ProcessBuilder _builder;
	private String _projectName;
	private String _scanDir;
	private String _resultFprFile;
	private String _build_log_file;
	
	public Fortify(String projectName, String uuid, String scanDir, String resultFprFile, String build_log_file){
		this._projectName=projectName+uuid;
		this._scanDir=scanDir+File.separator+projectName;
		this._resultFprFile=resultFprFile;
		this._build_log_file=build_log_file;
		this._builder = new ProcessBuilder( "sourceanalyzer","-v");
		this._builder.directory( new File(this._scanDir ) ); 
		this._builder.redirectErrorStream(true);
		File log = new File(this._build_log_file);
		this._builder.redirectOutput(Redirect.appendTo(log));
		this._builder.redirectError(Redirect.appendTo(log));
		try{
			Process process =  this._builder.start();
			int result = process.waitFor();
			if (result !=0)
				throw new RateToolException("Translate failed for project "+this._projectName);
		}catch(RateToolException e){
		}catch(IOException e){
		} catch (InterruptedException e) {
		}
	}
	

	public void clean() throws RateToolException {
		try{
			this._builder.command( "sourceanalyzer","-64", "-Xmx2048m", "-b", this._projectName, "-clean");
			Process process =  this._builder.start();
			int result = process.waitFor();
			if (result !=0)
				throw new RateToolException("Clean failed for project "+this._projectName);
		}catch(RateToolException e){
			throw e;
		}catch(IOException e){
			throw new RateToolException("Clean failed for project "+this._projectName);
		} catch (InterruptedException e) {
			throw new RateToolException("Clean failed for project "+this._projectName);
		}
	}

	public void translate_make() throws RateToolException {
		try{
			this._builder.command("mingw32-make");
			Process process =  this._builder.start();
			int result = process.waitFor();
			if (result !=0)
				throw new RateToolException("Translate failed for project "+this._projectName);
		}catch(RateToolException e){
			throw e;
		}catch(IOException e){
			throw new RateToolException("Translate failed for project "+this._projectName+" "+e.getMessage());
		} catch (InterruptedException e) {
			throw new RateToolException("Translate failed for project "+this._projectName+" "+e.getMessage());
		}
	}
	
	public void scan() throws RateToolException {
		try{
			this._builder.command( "sourceanalyzer","-64", "-Xmx2048m", "-b", this._projectName, "-scan", "-f", this._resultFprFile);
			Process process =  this._builder.start();
			int result = process.waitFor();
			if (result !=0)
				throw new RateToolException("scan failed for project.1 "+this._projectName);
		}catch(RateToolException e){
			throw e;
		}catch(IOException e){
			throw new RateToolException("scan failed for project.2 "+this._projectName);
		} catch (InterruptedException e) {
			throw new RateToolException("scan failed for project.3 "+this._projectName);
		}
	}
	
	
}
