package org.resilience.fileutilities;

import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.io.InputStream;

import org.resilience.RateToolException;

public class UnzipTool {

    public static void unzip(String zipFile, String targetDirectory) throws IOException, RateToolException, InterruptedException {
        ZipInputStream zipIn = null;
        
        try{
        	
        	zipIn= new ZipInputStream(new FileInputStream(zipFile));
        	ZipEntry entry = zipIn.getNextEntry();

	        while (entry != null) {
	            String filePath = targetDirectory + File.separator + entry.getName();
	            File newFile = new File(filePath);
	            if (entry.isDirectory())
	            	newFile.mkdirs();
	            else{
	            	newFile.getParentFile().mkdirs();
	            	extractFile(zipIn, newFile, safeLongToInt(entry.getSize())+10);
	            }
	            zipIn.closeEntry();
	            entry = zipIn.getNextEntry();
	        }
        }catch(IOException e){
        	throw e;
        }finally{
			try{
				if (zipIn!=null)
					zipIn.close();
			}catch(Exception ignore){}
		}
    }
    
    public static InputStream getAudit(String zipFile) throws RateToolException {
    	FileInputStream inputFile = null;
    	ZipInputStream zipIn = null;
    	boolean success = false;
    	try{
    		if (!UnzipTool.isZipFile(zipFile))
    			throw new Exception("Not a zip file " + zipFile);
    		inputFile = new FileInputStream(zipFile);
    		zipIn = new ZipInputStream(inputFile);
    		ZipEntry entry = zipIn.getNextEntry();

    		while (entry != null) {
    			if (entry.getName().equals("audit.fvdl")){
    				success = true;
    				return zipIn;
    			}
    			entry = zipIn.getNextEntry();
    		}
    		throw new Exception("Cannot find audit file");
    	}catch(Exception e){
    		throw new RateToolException(e.getMessage());
    	}finally{
    		if (!success){
    			try{
    				if (inputFile!=null)
    					inputFile.close();
    			}catch(Exception ignore){}
    			try{
    				if (zipIn!=null)
    					zipIn.close();
    			}catch(Exception ignore){}
    		}
    	}
    }

    private static void extractFile(ZipInputStream zipIn, File filePath, int filesize) throws IOException {
    	byte[] buf = new byte[1024];
    	FileOutputStream fos = null;
    	try{
	    	fos = new FileOutputStream(filePath);
	    	int len;
	        while ((len = zipIn.read(buf)) > 0) {
	        	fos.write(buf, 0, len);
	        }
    	}catch(IOException e){
    		throw e;
    	}finally{
    		try{
    			if (fos!=null)
    				fos.close();
    		}catch(Exception ignore){}
    	}
    }
    
    private static int safeLongToInt(long l) {
        if (l < Integer.MIN_VALUE || l > Integer.MAX_VALUE) {
            throw new IllegalArgumentException
                (l + " cannot be cast to int without changing its value.");
        }
        return (int) l;
    }
    
    public static boolean isZipFile(String filepath) throws IOException {
    	File zipFile = new File(filepath);
        if(zipFile.isDirectory()) {
            return false;
        }
        if(!zipFile.canRead()) {
            throw new IOException("Cannot read file "+zipFile.getAbsolutePath());
        }
        if(zipFile.length() < 4) {
            return false;
        }
        DataInputStream in = null;
        int test;
        try{
        	in = new DataInputStream(new BufferedInputStream(new FileInputStream(zipFile)));
            test = in.readInt();
        }catch(IOException e){
        	throw e;
	    }finally{
			try{
				if (in!=null)
					in.close();
			}catch(Exception ignore){}
		}
        return test == 0x504b0304;
    }
}
