package org.resilience.mark;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

import org.resilience.RateToolException;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

public class GenMark {
	SqualeConfigParser _conf;
	ArrayList<Owasp_practice> _practiceList;
	AuditParser _audit;
	
	public GenMark(String auditFile) throws RateToolException {
		try{
			this._conf=new SqualeConfigParser();
			this._practiceList=this._conf.getPracticeList();
			this._audit=new AuditParser(auditFile);
			this.loadIssues();
		}catch(RateToolException e){
			throw new RateToolException(e.getMessage());
		}
	}
	
	public GenMark(InputStream auditFile) throws RateToolException {
		try{
			this._conf=new SqualeConfigParser();
			this._practiceList=this._conf.getPracticeList();
			this._audit=new AuditParser(auditFile);
			this.loadIssues();
		}catch(RateToolException e){
			throw new RateToolException(e.getMessage());
		}
	}
	
	private void loadIssues(){
		NodeList nList = this._audit.getIssueList();
		for (int temp = 0; temp < nList.getLength(); temp++) {

			Node nNode = nList.item(temp);
		 		 
			if (nNode.getNodeType() == Node.ELEMENT_NODE) {
			 
				Element eElement = (Element) nNode;
				
				Fortify_issue issue = new Fortify_issue(eElement);	
				String issue_category_name=issue.getCategoryName();
				// Retrieve related practice
				String practice_name=this._conf.getAssociatedPractice(issue_category_name);
				for(Owasp_practice practice: this._practiceList){
					if (practice.getName().equals(practice_name)){
						// In case we are dealing with Discarded categories practice
						if ( practice.getName().equals(practice_name) && !practice.contains(issue_category_name))
							practice.addFortifyCategory(issue_category_name,0);

						// Add the issue
						practice.addFortifyIssue(issue);
					}
				}
			}
		}
	}
	
	public void writeResults(String zipFilename, String result_dir_path, String projectName) throws RateToolException{
		DateFormat shortDateFormatEN = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT, new Locale("EN","en"));
		Date now = new Date();
		//PrintWriter writer = new PrintWriter(result_dir_path+File.separator+projectName+"_results.txt", "UTF-8");
		PrintWriter writer = null;
		try {
			writer = new PrintWriter(result_dir_path+File.separator+projectName+"_results.txt", "UTF-8");
			writer.println("Zip File: " +zipFilename);
			writer.println("Results written at " +shortDateFormatEN.format(now));
			writer.println("Global Mark: " +this.getGlobalMark());
			for(Owasp_practice practice: this._practiceList){
				practice.displayStats(writer);
			}
		} catch (FileNotFoundException | UnsupportedEncodingException e) {
			throw new RateToolException("Failed at writing results");
		}finally{
			try{
				if (null!=writer)
					writer.close();
			}catch(Exception ignore){}
		}
		
	}

	private double getGlobalMark(){
		double sum=0;
		double sumWeight=0;
		for(Owasp_practice practice: this._practiceList){
			if (!practice.getName().equals("Discarded categories"))
				sumWeight+=practice.getWeight();
		}
		for(Owasp_practice practice: this._practiceList){
			if (!practice.getName().equals("Discarded categories"))
			 sum+=Math.pow(sumWeight, -practice.getMark());
		}
		return -Math.log10(sum/this._practiceList.size())/(Math.log10(sumWeight));
	}
}
