package org.resilience.mark;


import java.io.IOException;
import java.util.ArrayList;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.resilience.RateToolException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class SqualeConfigParser {

	private final NodeList _practiceList;
	private ArrayList<Owasp_practice> _list = new ArrayList<Owasp_practice>();
	
	public SqualeConfigParser() throws RateToolException {

		InputStream fXmlFile=null;
		try{
			fXmlFile=this.getClass().getClassLoader().getResourceAsStream("squale.xml");
			
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document config=dBuilder.parse(fXmlFile);
			config.getDocumentElement().normalize();
			this._practiceList=config.getElementsByTagName("practice");
			this.load();
		}catch(IOException e){
			throw new RateToolException("Failed at loading config1: "+e.getMessage());
		}catch(ParserConfigurationException e){
			throw new RateToolException("Failed at loading config2: "+e.getMessage());
		}catch(SAXException e){
			throw new RateToolException("Failed at loading config3: "+e.getMessage());
		}finally{
			try{
				if (fXmlFile!=null)
					fXmlFile.close();
			}catch(Exception ignore){}
		}
	}
	
	private void load(){
		for (int temp = 0; temp < this._practiceList.getLength(); temp++) {
			 
			Node nNode = this._practiceList.item(temp);
		 		 
			if (nNode.getNodeType() == Node.ELEMENT_NODE) {			
				Element eElement = (Element) nNode;
				Owasp_practice practice = new Owasp_practice(eElement.getAttribute("name"),Double.parseDouble(eElement.getAttribute("weight")));
				
				NodeList categoryList=eElement.getElementsByTagName("category");
				for (int temp2 = 0; temp2 < categoryList.getLength(); temp2++) {
					 
					Node nNode2 = categoryList.item(temp2);
				 		 
					if (nNode2.getNodeType() == Node.ELEMENT_NODE) {
						Element eElement2 = (Element) nNode2;
						practice.addFortifyCategory(eElement2.getAttribute("name"), Integer.parseInt(eElement2.getAttribute("threshold")));
					}
				}
				
				this._list.add(practice);
			}
		}
		//Adding discarded categories practice
		this._list.add(new Owasp_practice("Discarded categories",0));
	}
	
	
	public ArrayList<Owasp_practice> getPracticeList(){
		return this._list;
	}
	
		
	public String getAssociatedPractice(String categoryName){
		for (int temp = 0; temp < this._practiceList.getLength(); temp++) {
			 
			Node nNode = this._practiceList.item(temp);
		 		 
			if (nNode.getNodeType() == Node.ELEMENT_NODE) {
			 
				Element eElement = (Element) nNode;
				NodeList categoryList=eElement.getElementsByTagName("category");
				
				for (int temp2 = 0; temp2 < categoryList.getLength(); temp2++) {
					 
					Node nNode2 = categoryList.item(temp2);
				 		 
					if (nNode2.getNodeType() == Node.ELEMENT_NODE) {
						Element eElement2 = (Element) nNode2;
						if (eElement2.getAttribute("name").equals(categoryName)){
							return ((Element)eElement2.getParentNode()).getAttribute("name");
						}
					}
				}
			}
		}
		return "Discarded categories";
	}
	
	public boolean categoryExist(String categoryName){
		for (int temp = 0; temp < this._practiceList.getLength(); temp++) {
			 
			Node nNode = this._practiceList.item(temp);
		 		 
			if (nNode.getNodeType() == Node.ELEMENT_NODE) {
			 
				Element eElement = (Element) nNode;
				NodeList categoryList=eElement.getElementsByTagName("category");
				
				for (int temp2 = 0; temp2 < categoryList.getLength(); temp2++) {
					 
					Node nNode2 = categoryList.item(temp2);
				 		 
					if (nNode2.getNodeType() == Node.ELEMENT_NODE) {
						Element eElement2 = (Element) nNode2;
						if (eElement2.getAttribute("name").equals(categoryName)){
							return true;
						}
					}
				}
			}
		}
		return false;
	}
	
}
