/************************************************
* Utilisation de GMPLIB sous licence LGPL
************************************************/

#include <iostream>
#include <sstream>
#include <string>
#include <fstream>
#include <assert.h>
#include <stdio.h>
#include <vector>
#include <gmp.h>
#include <gmpxx.h>
#include <time.h>
#include "FieldElement.hpp"
#include "ShamirSharing.hpp"

using namespace std;

void usage(char * name){
	cerr << "Usage : " << name << " (Nombre de parties) (Seuil) [facultatif : Nom du fichier, 'entree' par dfaut]" << endl;
	cerr << "        Nombre de parties > Seuil > 0";
}

int main(int argc, char ** argv)
{
	// cf usage()
	if (argc < 3 || argc > 4){
		usage(argv[0]);
		return 1;
	}
    int num_players = atoi(argv[1]);
    int threshold = atoi(argv[2]);
	if (threshold >= num_players || num_players < 0){
		usage(argv[0]);
		return 3;
	}

	// Fichier  dcomposer
	FILE *secretFile;
	if (argc == 4){
		secretFile=fopen(argv[3],"rb");
		if (!secretFile){
			cerr << "Erreur lors de l'ouverture de " << argv[3] << endl;
			return 2;
		}		
	}else{
		secretFile=fopen("entree","rb");
		if (!secretFile){
			cerr << "Erreur lors de l'ouverture de " << "secret.txt" << endl;
			return 2;
		}
	}	

	// Lecture du fichier en binaire -> octets (fileC)
	char buffer[1024];
	int totalSize = 0;
	char* fileC = (char*) malloc(totalSize*sizeof(*fileC));
	size_t size = fread (buffer,1,1024,secretFile);
	while(size == 1024){
		totalSize += size;
		fileC = (char*) realloc(fileC,totalSize);
		for (unsigned int i = 0 ; i < size ; i++ ){
			fileC[i+totalSize-size] = buffer[i];
		}
		size = fread (buffer,1,1024,secretFile);
	}
	if (size){
		totalSize += size;
		fileC = (char*) realloc(fileC,totalSize);
		for (unsigned int i = 0 ; i < size ; i++ ){
			fileC[i+totalSize-size] = buffer[i];
		}
	}

	// Taille max d'une part
	unsigned int numBits = ShamirSharing::numBits;

	// prime fait 257 bits pour viter tout dbordement
	mpz_class prime = ShamirSharing::prime;

	// fileC (octets) -> entiers (cf ShamirSharing pour infos sur le dcoupage)
	vector <mpz_class> code = ShamirSharing::bytesToInt(fileC,(int)totalSize,numBits);

	// entiers (vector) -> shares (vector*vector)
	// un entier donne une liste de shares
	FieldElement secret;
	vector< vector< pair<int,FieldElement> > > shares;
	for (unsigned int i = 0 ; i < code.size() ; i++){
		secret = FieldElement(code.at(i),prime);
		shares.push_back(ShamirSharing::share(secret,threshold,num_players));
	}

	// Fichier  crire
	ofstream partyFile;

	// Pour chaque partie :
	// En-tte de fichier : "id - n - t"
	// Puis lignes : "id de la share - share"
	for (int i = 0 ; i < num_players ; i++){
		char partyFileName[256];
		sprintf(partyFileName,"partie%d.txt",i+1);
		partyFile.open(partyFileName);
		if (!partyFile.is_open()){
			cerr << "Erreur lors de l'ouverture de " << "partie" << (i+1) << ".txt" << endl;
			return 2;
		}
		partyFile << shares[0][i].first << "-" << num_players << "-" << threshold << endl;
		for (unsigned int j = 0 ; j < shares.size() ; j++){
			partyFile << j+1 << "-" << shares[j][i].second << endl;
		}
		partyFile.close();
	}

	return 0;
}
