/************************************************
* Utilisation de GMPLIB sous licence LGPL
************************************************/

#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <assert.h>
#include <vector>
#include <gmp.h>
#include <gmpxx.h>
#include "FieldElement.hpp"
#include "ShamirSharing.hpp"

using namespace std;

void usage(char * name){
	cerr << "Usage : " << name << " [facultatif : Nom du fichier, 'sortie' par dfaut]" << endl;
	cerr << "        Verifier que les fichiers 'partieX.txt' sont bien dans le mme rpertoire";
}

int main(int argc, char** argv)
{ 	
	vector < int > banned(1);
	int nbBanned = 0;
	int limit = (1 << 10);
	ifstream partyFile;
	int num_players;
    int real_num_players = 0;
    int threshold;
	string tmp;
	unsigned int delim;
	unsigned int delim2;
	int i;
	int error;
	// On cherche les fichiers partieX.txt pour 0 <= X < 2^10
	for ( i = 0 ; error || ((i < limit) && !partyFile.is_open()) ; i++){
		error = 0;
		char partyFileName[256];
		sprintf(partyFileName,"partie%d.txt",i+1);
		partyFile.open(partyFileName);
		if (partyFile.is_open()){
			// Des qu'on en trouve un, on prend l'en-tete et on parse
			getline(partyFile,tmp);
			delim = tmp.find('-');
			delim2 = tmp.find('-',delim+1);
			if (delim == string::npos || delim2 == string::npos){
				// -> Probleme de parsing : Ban du fichier
				error = 1;
				cerr << "Incohrence dans le fichier " << "partie" << i+1 << ".txt" << endl;
				partyFile.close();
				banned.push_back(i+1);
				nbBanned++;

			}else{
				stringstream(tmp.substr(delim+1,delim2)) >> num_players;
				stringstream(tmp.substr(delim2+1,tmp.length())) >> threshold;
				if (threshold >= num_players || num_players < 0){
					// -> Incohrence : Ban du fichier
					error = 1;
					cerr << "Incohrence dans le fichier " << "partie" << i+1 << ".txt" << endl;
					partyFile.close();
					banned.push_back(i+1);
					nbBanned++;
				}
			}
		}
	}

	if (!partyFile.is_open() && i == limit){
		cerr << "Impossible de trouver les fichiers ou incohrence des en-ttes (Nombre de parties/Seuil)" << endl;
		return 5;
	}

	partyFile.close();

	if (nbBanned >= (num_players - threshold)){
		cerr << "Trop peu de fichiers, impossible de recomposer le message" << endl;
		return 5;
	}

	unsigned int numBits = ShamirSharing::numBits;

	// Fichier ou ecrire le secret
	char* outputFile;
	if (argc == 2){
		outputFile = argv[1];
	}else if (argc > 2){
		usage(argv[0]);
		return 5;
	}else{
		outputFile = (char*) "sortie";
	}

	// Notre premier
	mpz_class prime = ShamirSharing::prime;

	// Sert a stocker les fichiers sans les analyser :
	// A chaque partie correspond un vector<string> qui est en fait la liste des lignes du fichier
	vector< vector<string> > stringShares(num_players);

	int nb_shunks = 0;

	// Rcuperation
	for (int i = 0 ; i < num_players ; i++){
		if ((find(banned.begin(),banned.end(),i+1)==banned.end())){
			char partyFileName[256];
			sprintf(partyFileName,"partie%d.txt",i+1);
			partyFile.open(partyFileName);
			if (!partyFile.is_open()){
				cerr << "Erreur lors de l'ouverture de " << "partie" << (i+1) << ".txt" << endl;
			}else{
				string tmp;
				int tcount = -1;
				// On ajoute les lignes au fichier une a une
				while(getline(partyFile,tmp)){
					tcount++;
					stringShares[real_num_players].push_back(tmp);
				}
				partyFile.close();
				nb_shunks = max(nb_shunks,tcount);
				if (nb_shunks != tcount){
					// S'il y a un probleme, on sort la partie des donnes rcupres 
					cerr << "Nombre de ligne incohrent pour le fichier " << "partie" << (i+1) << ".txt" << endl;
					stringShares[real_num_players].clear();
				}else {
					// Si tout se passe bien, on a une partie correcte
					real_num_players++;
				}
			}
		}
	}

	// Rcuperation du nombre de morceaux
	int id,file;
	int maxSize = 0;
	for (int part = 0 ; part < real_num_players ; part++){
		for (unsigned int i = 1 ; i < stringShares[part].size() ; i++){
			try{
				delim = stringShares[part].at(i).find('-');
				stringstream(stringShares[part].at(i).substr(0,delim)) >> file;
				if (file < (1 << 30)-1){
					maxSize = max(maxSize,file);
				}
			}catch(exception& x){}
		}
	}

	// A chaque morceaux correspond un vector< pair<int,FieldElement> > :
	// C'est une liste de couples (id,share)
	vector< vector< pair<int,FieldElement> > > shares(maxSize);

	// Parsing
	error = 1;

	// S'il y a une erreur, on recommence tout
	while (error){
		// On verifie qu'un id ne tombe qu'une seuel fois
		bool doneId[real_num_players+1];
		for (int i = 0 ; i < real_num_players+1 ; i++){
			doneId[i] = false;
		}

		// S'il y a eu trop d'erreur, on arrete
		if( real_num_players <= threshold ){
			cerr << "Trop peu de fichiers, impossible de recomposer le message" << endl;
			return 4;
		}


		error = 0;


		for (int part = 0 ; (part < real_num_players) && (error == 0) ; part++){
			shares.clear();

			// On verifie qu'un fichier ne tombe qu'une fois
			bool doneFile[maxSize];
			for (int i = 0 ; i < maxSize ; i++){
				doneFile[i] = false;
			}

			// On verifie l'en tte
			delim = stringShares[part].at(0).find('-');
			delim2 = stringShares[part].at(0).find('-',delim+1);
			if (delim == string::npos || delim2 == string::npos){
				error = 1;
				// On a une partie en moins
				real_num_players--;

				// On efface les donnes incohrentes
				stringShares.erase(stringShares.begin()+part);

				// On avertit l'utilisateur
				cerr << "Incohrence dans le fichier trait n" << part+1 << endl;
			}else{
				stringstream(stringShares[part].at(0).substr(0,delim)) >> id;

				// Si on a dj vu cet id, il y a un probleme : on efface le fichier
				if(doneId[id]){
					error = 1;
					real_num_players--;
					stringShares.erase(stringShares.begin()+part);
					cerr << "Partie " << (id) << " dj traite" << endl;					
				}else{
					// Pour l'instant tout se passe bien : On parse les donnes
					for (unsigned int i = 1 ; error == 0 && i < stringShares[part].size() ; i++){

						// On catch l'exeption du cas ou tmp = "" et on appelle mpz_class avec une chaine vide
						try{
							// les donnes sont de la forme "file-share"
							delim = stringShares[part].at(i).find('-');
							if (delim == string::npos){
								error = 1;
							}else{
								// On prend file
								file = 0;
								stringstream(stringShares[part].at(i).substr(0,delim)) >> file;
							}
							// Si on a dj vu ce fichier : erreur
							if (error || file == 0 || file > maxSize || doneFile[file-1]){
								error = 1;
							}
							else{
								// On prend share et on le stocke dans notre vector sous la forme (id,share)
								tmp = stringShares[part].at(i).substr(delim+1);
								shares[file-1].push_back(make_pair(id,FieldElement(mpz_class(tmp),prime)));

								// On a trait le fichier correctement
								doneFile[file-1] = true;
							}
						}catch(exception& e){
							error = 1;
						}
						if (error){					
							// S'il y a eu une erreur, on efface le fichier et les donnes parss correspondantes
							real_num_players--;
							for (int j = 0 ; j < maxSize ; j++ ){
								shares[j].clear();
							}
							stringShares.erase(stringShares.begin()+part);
							cerr << "Incohrence dans le fichier de la partie " << (id+1) << endl;
						}else{
							// Sinon tout s'est bien pass, on le note
							doneId[id] = true;
						}
					}
				}
			}
		}			
	}

	// Output
	FILE *resFile = fopen(outputFile,"wb");

	/*
	* On va recomposer le fichier :
	*  On va obtenir une liste d'entiers 
	*  On la convertit en octets
	*  Puis on les ecrit dans le fichier au fur et a mesure
	*/
	if (!resFile){
		cerr << "Erreur lors de l'ouverture de " << outputFile << endl;
	}else{
		int size = 0;
		for (int part = 0 ; part < nb_shunks ;  part++){
			char* res = ShamirSharing::intToBytes(ShamirSharing::recombine(shares[part]).getValue(),&size,numBits,part==0);
			fwrite (res , sizeof(char), size, resFile);
			free(res);
		}
	}

	fclose(resFile);

	return 0;
}