// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  Module['thisProgram'] = process['argv'][1];
  Module['arguments'] = process['argv'].slice(2);

  module['exports'] = Module;
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    return read(f, 'binary');
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  this['Module'] = Module;

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WEB) {
    window['Module'] = Module;
  } else {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] == 'undefined' && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Auto-generated preamble library stuff ===

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  forceAlign: function (target, quantum) {
    quantum = quantum || 4;
    if (quantum == 1) return target;
    if (isNumber(target) && isNumber(quantum)) {
      return Math.ceil(target/quantum)*quantum;
    } else if (isNumber(quantum) && isPowerOfTwo(quantum)) {
      return '(((' +target + ')+' + (quantum-1) + ')&' + -quantum + ')';
    }
    return 'Math.ceil((' + target + ')/' + quantum + ')*' + quantum;
  },
  isNumberType: function (type) {
    return type in Runtime.INT_TYPES || type in Runtime.FLOAT_TYPES;
  },
  isPointerType: function isPointerType(type) {
  return type[type.length-1] == '*';
},
  isStructType: function isStructType(type) {
  if (isPointerType(type)) return false;
  if (isArrayType(type)) return true;
  if (/<?\{ ?[^}]* ?\}>?/.test(type)) return true; // { i32, i8 } etc. - anonymous struct types
  // See comment in isStructPointerType()
  return type[0] == '%';
},
  INT_TYPES: {"i1":0,"i8":0,"i16":0,"i32":0,"i64":0},
  FLOAT_TYPES: {"float":0,"double":0},
  or64: function (x, y) {
    var l = (x | 0) | (y | 0);
    var h = (Math.round(x / 4294967296) | Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  and64: function (x, y) {
    var l = (x | 0) & (y | 0);
    var h = (Math.round(x / 4294967296) & Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  xor64: function (x, y) {
    var l = (x | 0) ^ (y | 0);
    var h = (Math.round(x / 4294967296) ^ Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  dedup: function dedup(items, ident) {
  var seen = {};
  if (ident) {
    return items.filter(function(item) {
      if (seen[item[ident]]) return false;
      seen[item[ident]] = true;
      return true;
    });
  } else {
    return items.filter(function(item) {
      if (seen[item]) return false;
      seen[item] = true;
      return true;
    });
  }
},
  set: function set() {
  var args = typeof arguments[0] === 'object' ? arguments[0] : arguments;
  var ret = {};
  for (var i = 0; i < args.length; i++) {
    ret[args[i]] = 0;
  }
  return ret;
},
  STACK_ALIGN: 8,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  calculateStructAlignment: function calculateStructAlignment(type) {
    type.flatSize = 0;
    type.alignSize = 0;
    var diffs = [];
    var prev = -1;
    var index = 0;
    type.flatIndexes = type.fields.map(function(field) {
      index++;
      var size, alignSize;
      if (Runtime.isNumberType(field) || Runtime.isPointerType(field)) {
        size = Runtime.getNativeTypeSize(field); // pack char; char; in structs, also char[X]s.
        alignSize = Runtime.getAlignSize(field, size);
      } else if (Runtime.isStructType(field)) {
        if (field[1] === '0') {
          // this is [0 x something]. When inside another structure like here, it must be at the end,
          // and it adds no size
          // XXX this happens in java-nbody for example... assert(index === type.fields.length, 'zero-length in the middle!');
          size = 0;
          if (Types.types[field]) {
            alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
          } else {
            alignSize = type.alignSize || QUANTUM_SIZE;
          }
        } else {
          size = Types.types[field].flatSize;
          alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
        }
      } else if (field[0] == 'b') {
        // bN, large number field, like a [N x i8]
        size = field.substr(1)|0;
        alignSize = 1;
      } else if (field[0] === '<') {
        // vector type
        size = alignSize = Types.types[field].flatSize; // fully aligned
      } else if (field[0] === 'i') {
        // illegal integer field, that could not be legalized because it is an internal structure field
        // it is ok to have such fields, if we just use them as markers of field size and nothing more complex
        size = alignSize = parseInt(field.substr(1))/8;
        assert(size % 1 === 0, 'cannot handle non-byte-size field ' + field);
      } else {
        assert(false, 'invalid type for calculateStructAlignment');
      }
      if (type.packed) alignSize = 1;
      type.alignSize = Math.max(type.alignSize, alignSize);
      var curr = Runtime.alignMemory(type.flatSize, alignSize); // if necessary, place this on aligned memory
      type.flatSize = curr + size;
      if (prev >= 0) {
        diffs.push(curr-prev);
      }
      prev = curr;
      return curr;
    });
    if (type.name_ && type.name_[0] === '[') {
      // arrays have 2 elements, so we get the proper difference. then we scale here. that way we avoid
      // allocating a potentially huge array for [999999 x i8] etc.
      type.flatSize = parseInt(type.name_.substr(1))*type.flatSize/2;
    }
    type.flatSize = Runtime.alignMemory(type.flatSize, type.alignSize);
    if (diffs.length == 0) {
      type.flatFactor = type.flatSize;
    } else if (Runtime.dedup(diffs).length == 1) {
      type.flatFactor = diffs[0];
    }
    type.needsFlattening = (type.flatFactor != 1);
    return type.flatIndexes;
  },
  generateStructInfo: function (struct, typeName, offset) {
    var type, alignment;
    if (typeName) {
      offset = offset || 0;
      type = (typeof Types === 'undefined' ? Runtime.typeInfo : Types.types)[typeName];
      if (!type) return null;
      if (type.fields.length != struct.length) {
        printErr('Number of named fields must match the type for ' + typeName + ': possibly duplicate struct names. Cannot return structInfo');
        return null;
      }
      alignment = type.flatIndexes;
    } else {
      var type = { fields: struct.map(function(item) { return item[0] }) };
      alignment = Runtime.calculateStructAlignment(type);
    }
    var ret = {
      __size__: type.flatSize
    };
    if (typeName) {
      struct.forEach(function(item, i) {
        if (typeof item === 'string') {
          ret[item] = alignment[i] + offset;
        } else {
          // embedded struct
          var key;
          for (var k in item) key = k;
          ret[key] = Runtime.generateStructInfo(item[key], type.fields[i], alignment[i]);
        }
      });
    } else {
      struct.forEach(function(item, i) {
        ret[item[1]] = alignment[i];
      });
    }
    return ret;
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      assert(args.length == sig.length-1);
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      assert(sig.length == 1);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  getAsmConst: function (code, numArgs) {
    // code is a constant string on the heap, so we can cache these
    if (!Runtime.asmConstCache) Runtime.asmConstCache = {};
    var func = Runtime.asmConstCache[code];
    if (func) return func;
    var args = [];
    for (var i = 0; i < numArgs; i++) {
      args.push(String.fromCharCode(36) + i); // $0, $1 etc
    }
    var source = Pointer_stringify(code);
    if (source[0] === '"') {
      // tolerate EM_ASM("..code..") even though EM_ASM(..code..) is correct
      if (source.indexOf('"', 1) === source.length-1) {
        source = source.substr(1, source.length-2);
      } else {
        // something invalid happened, e.g. EM_ASM("..code($0)..", input)
        abort('invalid EM_ASM input |' + source + '|. Please use EM_ASM(..code..) (no quotes) or EM_ASM({ ..code($0).. }, input) (to input values)');
      }
    }
    try {
      var evalled = eval('(function(' + args.join(',') + '){ ' + source + ' })'); // new Function does not allow upvars in node
    } catch(e) {
      Module.printErr('error in executing inline EM_ASM code: ' + e + ' on: \n\n' + source + '\n\nwith args |' + args + '| (make sure to use the right one out of EM_ASM, EM_ASM_ARGS, etc.)');
      throw e;
    }
    return Runtime.asmConstCache[code] = evalled;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[func]) {
      Runtime.funcWrappers[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return Runtime.funcWrappers[func];
  },
  UTF8Processor: function () {
    var buffer = [];
    var needed = 0;
    this.processCChar = function (code) {
      code = code & 0xFF;

      if (buffer.length == 0) {
        if ((code & 0x80) == 0x00) {        // 0xxxxxxx
          return String.fromCharCode(code);
        }
        buffer.push(code);
        if ((code & 0xE0) == 0xC0) {        // 110xxxxx
          needed = 1;
        } else if ((code & 0xF0) == 0xE0) { // 1110xxxx
          needed = 2;
        } else {                            // 11110xxx
          needed = 3;
        }
        return '';
      }

      if (needed) {
        buffer.push(code);
        needed--;
        if (needed > 0) return '';
      }

      var c1 = buffer[0];
      var c2 = buffer[1];
      var c3 = buffer[2];
      var c4 = buffer[3];
      var ret;
      if (buffer.length == 2) {
        ret = String.fromCharCode(((c1 & 0x1F) << 6)  | (c2 & 0x3F));
      } else if (buffer.length == 3) {
        ret = String.fromCharCode(((c1 & 0x0F) << 12) | ((c2 & 0x3F) << 6)  | (c3 & 0x3F));
      } else {
        // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        var codePoint = ((c1 & 0x07) << 18) | ((c2 & 0x3F) << 12) |
                        ((c3 & 0x3F) << 6)  | (c4 & 0x3F);
        ret = String.fromCharCode(
          Math.floor((codePoint - 0x10000) / 0x400) + 0xD800,
          (codePoint - 0x10000) % 0x400 + 0xDC00);
      }
      buffer.length = 0;
      return ret;
    }
    this.processJSString = function processJSString(string) {
      /* TODO: use TextEncoder when present,
        var encoder = new TextEncoder();
        encoder['encoding'] = "utf-8";
        var utf8Array = encoder['encode'](aMsg.data);
      */
      string = unescape(encodeURIComponent(string));
      var ret = [];
      for (var i = 0; i < string.length; i++) {
        ret.push(string.charCodeAt(i));
      }
      return ret;
    }
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+7)&-8);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+7)&-8); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+7)&-8); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 8))*(quantum ? quantum : 8); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;









//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var stack = 0;
  var JSfuncs = {
    'stackSave' : function() {
      stack = Runtime.stackSave();
    },
    'stackRestore' : function() {
      Runtime.stackRestore(stack);
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        ret = Runtime.stackAlloc(str.length + 1); // +1 for the trailing '\0'
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. A convenient way to call C functions (in C files, or
  // defined with extern "C").
  //
  // Note: ccall/cwrap use the C stack for temporary values. If you pass a string
  //       then it is only alive until the call is complete. If the code being
  //       called saves the pointer to be used later, it may point to invalid
  //       data. If you need a string to live forever, you can create it (and
  //       must later delete it manually!) using malloc and writeStringToMemory,
  //       for example.
  //
  // Note: LLVM optimizations can inline and remove functions, after which you will not be
  //       able to call them. Closure can also do so. To avoid that, add your function to
  //       the exports using something like
  //
  //         -s EXPORTED_FUNCTIONS='["_main", "_myfunc"]'
  //
  // @param ident      The name of the C function (note that C++ functions will be name-mangled - use extern "C")
  // @param returnType The return type of the function, one of the JS types 'number', 'string' or 'array' (use 'number' for any C pointer, and
  //                   'array' for JavaScript arrays and typed arrays; note that arrays are 8-bit).
  // @param argTypes   An array of the types of arguments for the function (if there are no arguments, this can be ommitted). Types are as in returnType,
  //                   except that 'array' is not possible (there is no way for us to know the length of the array)
  // @param args       An array of the arguments to the function, as native JS values (as in returnType)
  //                   Note that string arguments will be stored on the stack (the JS string will become a C string on the stack).
  // @return           The return value, as a native JS value (as in returnType)
  ccall = function ccallFunc(ident, returnType, argTypes, args) {
    assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
    var func = getCFunc(ident);
    var cArgs = [];
    assert(returnType !== 'array', 'Return type should not be "array".');
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) JSfuncs['stackRestore']();
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }
  // Returns a native JS wrapper for a C function. This is similar to ccall, but
  // returns a function you can call repeatedly in a normal way. For example:
  //
  //   var my_function = cwrap('my_c_function', 'number', ['number', 'number']);
  //   alert(my_function(5, 22));
  //   alert(my_function(99, 12));
  //
  cwrap = function cwrap(ident, returnType, argTypes) {
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    funcstr += "assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');\n";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["cwrap"] = cwrap;
Module["ccall"] = ccall;

// Sets a value in memory in a dynamic way at run-time. Uses the
// type data. This is the same as makeSetValue, except that
// makeSetValue is done at compile-time and generates the needed
// code then, whereas this function picks the right code at
// run-time.
// Note that setValue and getValue only do *aligned* writes and reads!
// Note that ccall uses JS types as for defining types, while setValue and
// getValue need LLVM types ('i8', 'i32') - this is a lower-level operation
function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;

// Parallel to setValue.
function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    assert(type, 'Must know what type to store in allocate!');

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

function Pointer_stringify(ptr, /* optional */ length) {
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = false;
  var t;
  var i = 0;
  while (1) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    if (t >= 128) hasUtf = true;
    else if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (!hasUtf) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }

  var utf8 = new Runtime.UTF8Processor();
  for (i = 0; i < length; i++) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    ret += utf8.processCChar(t);
  }
  return ret;
}
Module['Pointer_stringify'] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16LE form. The copy will require at most (str.length*2+1)*2 bytes of space in the HEAP.
function stringToUTF16(str, outPtr) {
  for(var i = 0; i < str.length; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[(((outPtr)+(i*2))>>1)]=codeUnit;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[(((outPtr)+(str.length*2))>>1)]=0;
}
Module['stringToUTF16'] = stringToUTF16;

// Given a pointer 'ptr' to a null-terminated UTF32LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32LE form. The copy will require at most (str.length+1)*4 bytes of space in the HEAP,
// but can use less, since str.length does not return the number of characters in the string, but the number of UTF-16 code units in the string.
function stringToUTF32(str, outPtr) {
  var iChar = 0;
  for(var iCodeUnit = 0; iCodeUnit < str.length; ++iCodeUnit) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    var codeUnit = str.charCodeAt(iCodeUnit); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++iCodeUnit);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[(((outPtr)+(iChar*4))>>2)]=codeUnit;
    ++iChar;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[(((outPtr)+(iChar*4))>>2)]=0;
}
Module['stringToUTF32'] = stringToUTF32;

function demangle(func) {
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    return parse();
  } catch(e) {
    return func;
  }
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function stackTrace() {
  var stack = new Error().stack;
  return stack ? demangleAll(stack) : '(no stack trace available)'; // Stack trace is not available at least on IE10 and Safari 6.
}

// Memory management

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk

function enlargeMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with ALLOW_MEMORY_GROWTH which adjusts the size at runtime but prevents some optimizations, or (3) set Module.TOTAL_MEMORY before the program runs.');
}

var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;

var totalMemory = 4096;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be more reasonable');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;

function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
    Module.printErr('Exiting runtime. Any attempt to access the compiled C code may fail from now. If you want to keep the runtime alive, set Module["noExitRuntime"] = true or build with -s NO_EXIT_RUNTIME=1');
  }
  callRuntimeCallbacks(__ATEXIT__);
  runtimeInitialized = false;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools

// This processes a JS string into a C-line array of numbers, 0-terminated.
// For LLVM-originating strings, see parser.js:parseLLVMString function
function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var ret = (new Runtime.UTF8Processor()).processJSString(stringy);
  if (length) {
    ret.length = length;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
        assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

// Write a Javascript array to somewhere in the heap
function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[(((buffer)+(i))>>0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; i++) {
    assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
    HEAP8[(((buffer)+(i))>>0)]=str.charCodeAt(i);
  }
  if (!dontAddNull) HEAP8[(((buffer)+(str.length))>>0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(!runDependencyTracking[id]);
    runDependencyTracking[id] = 1;
    if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
      // Check for missing dependencies every few seconds
      runDependencyWatcher = setInterval(function() {
        if (ABORT) {
          clearInterval(runDependencyWatcher);
          runDependencyWatcher = null;
          return;
        }
        var shown = false;
        for (var dep in runDependencyTracking) {
          if (!shown) {
            shown = true;
            Module.printErr('still waiting on run dependencies:');
          }
          Module.printErr('dependency: ' + dep);
        }
        if (shown) {
          Module.printErr('(end of list)');
        }
      }, 10000);
    }
  } else {
    Module.printErr('warning: run dependency added without ID');
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(runDependencyTracking[id]);
    delete runDependencyTracking[id];
  } else {
    Module.printErr('warning: run dependency removed without ID');
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data


var memoryInitializer = null;

// === Body ===





STATIC_BASE = 8;

STATICTOP = STATIC_BASE + Runtime.alignMemory(11875);
  /* global initializers */ __ATINIT__.push();
  

/* memory initializer */ allocate([0,0,0,0,0,0,0,0,100,106,112,101,103,0,0,0,0,0,0,0,40,8,0,0,72,8,0,0,120,8,0,0,160,8,0,0,208,8,0,0,248,8,0,0,40,9,0,0,80,9,0,0,104,9,0,0,144,9,0,0,176,9,0,0,216,9,0,0,240,9,0,0,8,10,0,0,48,10,0,0,80,10,0,0,96,10,0,0,120,10,0,0,168,10,0,0,216,10,0,0,0,11,0,0,32,11,0,0,72,11,0,0,104,11,0,0,128,11,0,0,168,11,0,0,200,11,0,0,224,11,0,0,240,11,0,0,8,12,0,0,24,12,0,0,48,12,0,0,88,12,0,0,128,12,0,0,168,12,0,0,192,12,0,0,224,12,0,0,0,13,0,0,56,13,0,0,112,13,0,0,128,13,0,0,192,13,0,0,0,0,0,0,37,115,58,32,111,110,108,121,32,111,110,101,32,105,110,112,117,116,32,102,105,108,101,10,0,0,0,0,0,0,0,0,114,98,0,0,0,0,0,0,37,115,58,32,99,97,110,39,116,32,111,112,101,110,32,37,115,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,119,98,0,0,0,0,0,0,0,0,0,0,0,0,0,0,117,115,97,103,101,58,32,37,115,32,91,115,119,105,116,99,104,101,115,93,32,0,0,0,91,105,110,112,117,116,102,105,108,101,93,10,0,0,0,0,83,119,105,116,99,104,101,115,32,40,110,97,109,101,115,32,109,97,121,32,98,101,32,97,98,98,114,101,118,105,97,116,101,100,41,58,10,0,0,0,32,32,45,99,111,108,111,114,115,32,78,32,32,32,32,32,32,82,101,100,117,99,101,32,105,109,97,103,101,32,116,111,32,110,111,32,109,111,114,101,32,116,104,97,110,32,78,32,99,111,108,111,114,115,10,0,32,32,45,102,97,115,116,32,32,32,32,32,32,32,32,32,32,70,97,115,116,44,32,108,111,119,45,113,117,97,108,105,116,121,32,112,114,111,99,101,115,115,105,110,103,10,0,0,32,32,45,103,114,97,121,115,99,97,108,101,32,32,32,32,32,70,111,114,99,101,32,103,114,97,121,115,99,97,108,101,32,111,117,116,112,117,116,10,0,0,0,0,0,0,0,0,32,32,45,115,99,97,108,101,32,77,47,78,32,32,32,32,32,83,99,97,108,101,32,111,117,116,112,117,116,32,105,109,97,103,101,32,98,121,32,102,114,97,99,116,105,111,110,32,77,47,78,44,32,101,103,44,32,49,47,56,10,0,0,0,32,32,45,98,109,112,32,32,32,32,32,32,32,32,32,32,32,83,101,108,101,99,116,32,66,77,80,32,111,117,116,112,117,116,32,102,111,114,109,97,116,32,40,87,105,110,100,111,119,115,32,115,116,121,108,101,41,37,115,10,0,0,0,0,0,0,0,0,0,0,0,0,32,32,45,103,105,102,32,32,32,32,32,32,32,32,32,32,32,83,101,108,101,99,116,32,71,73,70,32,111,117,116,112,117,116,32,102,111,114,109,97,116,37,115,10,0,0,0,0,32,32,45,111,115,50,32,32,32,32,32,32,32,32,32,32,32,83,101,108,101,99,116,32,66,77,80,32,111,117,116,112,117,116,32,102,111,114,109,97,116,32,40,79,83,47,50,32,115,116,121,108,101,41,37,115,10,0,0,0,0,0,0,0,32,32,45,112,110,109,32,32,32,32,32,32,32,32,32,32,32,83,101,108,101,99,116,32,80,66,77,80,76,85,83,32,40,80,80,77,47,80,71,77,41,32,111,117,116,112,117,116,32,102,111,114,109,97,116,37,115,10,0,0,0,0,0,0,32,40,100,101,102,97,117,108,116,41,0,0,0,0,0,0,32,32,45,116,97,114,103,97,32,32,32,32,32,32,32,32,32,83,101,108,101,99,116,32,84,97,114,103,97,32,111,117,116,112,117,116,32,102,111,114,109,97,116,37,115,10,0,0,83,119,105,116,99,104,101,115,32,102,111,114,32,97,100,118,97,110,99,101,100,32,117,115,101,114,115,58,10,0,0,0,32,32,45,100,99,116,32,105,110,116,32,32,32,32,32,32,32,85,115,101,32,105,110,116,101,103,101,114,32,68,67,84,32,109,101,116,104,111,100,37,115,10,0,0,0,0,0,0,32,32,45,100,99,116,32,102,97,115,116,32,32,32,32,32,32,85,115,101,32,102,97,115,116,32,105,110,116,101,103,101,114,32,68,67,84,32,40,108,101,115,115,32,97,99,99,117,114,97,116,101,41,37,115,10,0,0,0,0,0,0,0,0,32,32,45,100,99,116,32,102,108,111,97,116,32,32,32,32,32,85,115,101,32,102,108,111,97,116,105,110,103,45,112,111,105,110,116,32,68,67,84,32,109,101,116,104,111,100,37,115,10,0,0,0,0,0,0,0,32,32,45,100,105,116,104,101,114,32,102,115,32,32,32,32,32,85,115,101,32,70,45,83,32,100,105,116,104,101,114,105,110,103,32,40,100,101,102,97,117,108,116,41,10,0,0,0,32,32,45,100,105,116,104,101,114,32,110,111,110,101,32,32,32,68,111,110,39,116,32,117,115,101,32,100,105,116,104,101,114,105,110,103,32,105,110,32,113,117,97,110,116,105,122,97,116,105,111,110,10,0,0,0,32,32,45,100,105,116,104,101,114,32,111,114,100,101,114,101,100,32,32,85,115,101,32,111,114,100,101,114,101,100,32,100,105,116,104,101,114,32,40,109,101,100,105,117,109,32,115,112,101,101,100,44,32,113,117,97,108,105,116,121,41,10,0,0,32,32,45,109,97,112,32,70,73,76,69,32,32,32,32,32,32,77,97,112,32,116,111,32,99,111,108,111,114,115,32,117,115,101,100,32,105,110,32,110,97,109,101,100,32,105,109,97,103,101,32,102,105,108,101,10,0,0,0,0,0,0,0,0,32,32,45,110,111,115,109,111,111,116,104,32,32,32,32,32,32,68,111,110,39,116,32,117,115,101,32,104,105,103,104,45,113,117,97,108,105,116,121,32,117,112,115,97,109,112,108,105,110,103,10,0,0,0,0,0,32,32,45,111,110,101,112,97,115,115,32,32,32,32,32,32,32,85,115,101,32,49,45,112,97,115,115,32,113,117,97,110,116,105,122,97,116,105,111,110,32,40,102,97,115,116,44,32,108,111,119,32,113,117,97,108,105,116,121,41,10,0,0,0,32,32,45,109,97,120,109,101,109,111,114,121,32,78,32,32,32,77,97,120,105,109,117,109,32,109,101,109,111,114,121,32,116,111,32,117,115,101,32,40,105,110,32,107,98,121,116,101,115,41,10,0,0,0,0,0,32,32,45,111,117,116,102,105,108,101,32,110,97,109,101,32,32,83,112,101,99,105,102,121,32,110,97,109,101,32,102,111,114,32,111,117,116,112,117,116,32,102,105,108,101,10,0,0,32,32,45,118,101,114,98,111,115,101,32,32,111,114,32,32,45,100,101,98,117,103,32,32,32,69,109,105,116,32,100,101,98,117,103,32,111,117,116,112,117,116,10,0,0,0,0,0,98,109,112,0,0,0,0,0,99,111,108,111,114,115,0,0,99,111,108,111,117,114,115,0,113,117,97,110,116,105,122,101,0,0,0,0,0,0,0,0,113,117,97,110,116,105,115,101,0,0,0,0,0,0,0,0,37,100,0,0,0,0,0,0,100,99,116,0,0,0,0,0,105,110,116,0,0,0,0,0,102,97,115,116,0,0,0,0,102,108,111,97,116,0,0,0,100,105,116,104,101,114,0,0,102,115,0,0,0,0,0,0,110,111,110,101,0,0,0,0,111,114,100,101,114,101,100,0,100,101,98,117,103,0,0,0,118,101,114,98,111,115,101,0,0,0,0,0,0,0,0,0,73,110,100,101,112,101,110,100,101,110,116,32,74,80,69,71,32,71,114,111,117,112,39,115,32,68,74,80,69,71,44,32,118,101,114,115,105,111,110,32,37,115,10,37,115,10,0,0,54,98,32,32,50,55,45,77,97,114,45,49,57,57,56,0,67,111,112,121,114,105,103,104,116,32,40,67,41,32,49,57,57,56,44,32,84,104,111,109,97,115,32,71,46,32,76,97,110,101,0,0,0,0,0,0,103,105,102,0,0,0,0,0,103,114,97,121,115,99,97,108,101,0,0,0,0,0,0,0,103,114,101,121,115,99,97,108,101,0,0,0,0,0,0,0,109,97,112,0,0,0,0,0,109,97,120,109,101,109,111,114,121,0,0,0,0,0,0,0,37,108,100,37,99,0,0,0,110,111,115,109,111,111,116,104,0,0,0,0,0,0,0,0,111,110,101,112,97,115,115,0,111,115,50,0,0,0,0,0,111,117,116,102,105,108,101,0,112,110,109,0,0,0,0,0,112,112,109,0,0,0,0,0,114,108,101,0,0,0,0,0,115,99,97,108,101,0,0,0,37,100,47,37,100,0,0,0,116,97,114,103,97,0,0,0,67,111,109,109,101,110,116,44,32,108,101,110,103,116,104,32,37,108,100,58,10,0,0,0,65,80,80,37,100,44,32,108,101,110,103,116,104,32,37,108,100,58,10,0,0,0,0,0,10,0,0,0,0,0,0,0,92,92,0,0,0,0,0,0,92,37,48,51,111,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,66,77,80,32,99,111,108,111,114,109,97,112,32,102,111,114,109,97,116,0,79,110,108,121,32,56,45,32,97,110,100,32,50,52,45,98,105,116,32,66,77,80,32,102,105,108,101,115,32,97,114,101,32,115,117,112,112,111,114,116,101,100,0,0,0,0,0,0,73,110,118,97,108,105,100,32,66,77,80,32,102,105,108,101,58,32,98,97,100,32,104,101,97,100,101,114,32,108,101,110,103,116,104,0,0,0,0,0,73,110,118,97,108,105,100,32,66,77,80,32,102,105,108,101,58,32,98,105,80,108,97,110,101,115,32,110,111,116,32,101,113,117,97,108,32,116,111,32,49,0,0,0,0,0,0,0,66,77,80,32,111,117,116,112,117,116,32,109,117,115,116,32,98,101,32,103,114,97,121,115,99,97,108,101,32,111,114,32,82,71,66,0,0,0,0,0,83,111,114,114,121,44,32,99,111,109,112,114,101,115,115,101,100,32,66,77,80,115,32,110,111,116,32,121,101,116,32,115,117,112,112,111,114,116,101,100,0,0,0,0,0,0,0,0,78,111,116,32,97,32,66,77,80,32,102,105,108,101,32,45,32,100,111,101,115,32,110,111,116,32,115,116,97,114,116,32,119,105,116,104,32,66,77,0,37,117,120,37,117,32,50,52,45,98,105,116,32,66,77,80,32,105,109,97,103,101,0,0,37,117,120,37,117,32,56,45,98,105,116,32,99,111,108,111,114,109,97,112,112,101,100,32,66,77,80,32,105,109,97,103,101,0,0,0,0,0,0,0,37,117,120,37,117,32,50,52,45,98,105,116,32,79,83,50,32,66,77,80,32,105,109,97,103,101,0,0,0,0,0,0,37,117,120,37,117,32,56,45,98,105,116,32,99,111,108,111,114,109,97,112,112,101,100,32,79,83,50,32,66,77,80,32,105,109,97,103,101,0,0,0,71,73,70,32,111,117,116,112,117,116,32,103,111,116,32,99,111,110,102,117,115,101,100,0,66,111,103,117,115,32,71,73,70,32,99,111,100,101,115,105,122,101,32,37,100,0,0,0,71,73,70,32,111,117,116,112,117,116,32,109,117,115,116,32,98,101,32,103,114,97,121,115,99,97,108,101,32,111,114,32,82,71,66,0,0,0,0,0,84,111,111,32,102,101,119,32,105,109,97,103,101,115,32,105,110,32,71,73,70,32,102,105,108,101,0,0,0,0,0,0,78,111,116,32,97,32,71,73,70,32,102,105,108,101,0,0,37,117,120,37,117,120,37,100,32,71,73,70,32,105,109,97,103,101,0,0,0,0,0,0,87,97,114,110,105,110,103,58,32,117,110,101,120,112,101,99,116,101,100,32,71,73,70,32,118,101,114,115,105,111,110,32,110,117,109,98,101,114,32,39,37,99,37,99,37,99,39,0,73,103,110,111,114,105,110,103,32,71,73,70,32,101,120,116,101,110,115,105,111,110,32,98,108,111,99,107,32,111,102,32,116,121,112,101,32,48,120,37,48,50,120,0,0,0,0,0,67,97,117,116,105,111,110,58,32,110,111,110,115,113,117,97,114,101,32,112,105,120,101,108,115,32,105,110,32,105,110,112,117,116,0,0,0,0,0,0,67,111,114,114,117,112,116,32,100,97,116,97,32,105,110,32,71,73,70,32,102,105,108,101,0,0,0,0,0,0,0,0,66,111,103,117,115,32,99,104,97,114,32,48,120,37,48,50,120,32,105,110,32,71,73,70,32,102,105,108,101,44,32,105,103,110,111,114,105,110,103,0,80,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,71,73,70,32,105,109,97,103,101,0,0,0,0,0,0,82,97,110,32,111,117,116,32,111,102,32,71,73,70,32,98,105,116,115,0,0,0,0,0,80,80,77,32,111,117,116,112,117,116,32,109,117,115,116,32,98,101,32,103,114,97,121,115,99,97,108,101,32,111,114,32,82,71,66,0,0,0,0,0,78,111,110,110,117,109,101,114,105,99,32,100,97,116,97,32,105,110,32,80,80,77,32,102,105,108,101,0,0,0,0,0,78,111,116,32,97,32,80,80,77,47,80,71,77,32,102,105,108,101,0,0,0,0,0,0,37,117,120,37,117,32,80,71,77,32,105,109,97,103,101,0,37,117,120,37,117,32,116,101,120,116,32,80,71,77,32,105,109,97,103,101,0,0,0,0,37,117,120,37,117,32,80,80,77,32,105,109,97,103,101,0,37,117,120,37,117,32,116,101,120,116,32,80,80,77,32,105,109,97,103,101,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,84,97,114,103,97,32,99,111,108,111,114,109,97,112,32,102,111,114,109,97,116,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,32,84,97,114,103,97,32,102,105,108,101,0,0,0,0,0,0,0,84,97,114,103,97,32,111,117,116,112,117,116,32,109,117,115,116,32,98,101,32,103,114,97,121,115,99,97,108,101,32,111,114,32,82,71,66,0,0,0,37,117,120,37,117,32,82,71,66,32,84,97,114,103,97,32,105,109,97,103,101,0,0,0,37,117,120,37,117,32,103,114,97,121,115,99,97,108,101,32,84,97,114,103,97,32,105,109,97,103,101,0,0,0,0,0,37,117,120,37,117,32,99,111,108,111,114,109,97,112,112,101,100,32,84,97,114,103,97,32,105,109,97,103,101,0,0,0,67,111,108,111,114,32,109,97,112,32,102,105,108,101,32,105,115,32,105,110,118,97,108,105,100,32,111,114,32,111,102,32,117,110,115,117,112,112,111,114,116,101,100,32,102,111,114,109,97,116,0,0,0,0,0,0,79,117,116,112,117,116,32,102,105,108,101,32,102,111,114,109,97,116,32,99,97,110,110,111,116,32,104,97,110,100,108,101,32,37,100,32,99,111,108,111,114,109,97,112,32,101,110,116,114,105,101,115,0,0,0,0,117,110,103,101,116,99,32,102,97,105,108,101,100,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,105,110,112,117,116,32,102,105,108,101,32,102,111,114,109,97,116,32,45,45,45,32,112,101,114,104,97,112,115,32,121,111,117,32,110,101,101,100,32,45,116,97,114,103,97,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,111,117,116,112,117,116,32,102,105,108,101,32,102,111,114,109,97,116,0,0,80,53,10,37,108,100,32,37,108,100,10,37,100,10,0,0,80,54,10,37,108,100,32,37,108,100,10,37,100,10,0,0,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,128,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,0,0,0,255,255,255,255,253,255,255,255,249,255,255,255,241,255,255,255,225,255,255,255,193,255,255,255,129,255,255,255,1,255,255,255,1,254,255,255,1,252,255,255,1,248,255,255,1,240,255,255,1,224,255,255,1,192,255,255,1,128,255,255,66,111,103,117,115,32,109,101,115,115,97,103,101,32,99,111,100,101,32,37,100,0,0,0,83,111,114,114,121,44,32,116,104,101,114,101,32,97,114,101,32,108,101,103,97,108,32,114,101,115,116,114,105,99,116,105,111,110,115,32,111,110,32,97,114,105,116,104,109,101,116,105,99,32,99,111,100,105,110,103,0,0,0,0,0,0,0,0,65,76,73,71,78,95,84,89,80,69,32,105,115,32,119,114,111,110,103,44,32,112,108,101,97,115,101,32,102,105,120,0,77,65,88,95,65,76,76,79,67,95,67,72,85,78,75,32,105,115,32,119,114,111,110,103,44,32,112,108,101,97,115,101,32,102,105,120,0,0,0,0,66,111,103,117,115,32,98,117,102,102,101,114,32,99,111,110,116,114,111,108,32,109,111,100,101,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,111,109,112,111,110,101,110,116,32,73,68,32,37,100,32,105,110,32,83,79,83,0,0,68,67,84,32,99,111,101,102,102,105,99,105,101,110,116,32,111,117,116,32,111,102,32,114,97,110,103,101,0,0,0,0,73,68,67,84,32,111,117,116,112,117,116,32,98,108,111,99,107,32,115,105,122,101,32,37,100,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,66,111,103,117,115,32,72,117,102,102,109,97,110,32,116,97,98,108,101,32,100,101,102,105,110,105,116,105,111,110,0,0,66,111,103,117,115,32,105,110,112,117,116,32,99,111,108,111,114,115,112,97,99,101,0,0,66,111,103,117,115,32,74,80,69,71,32,99,111,108,111,114,115,112,97,99,101,0,0,0,66,111,103,117,115,32,109,97,114,107,101,114,32,108,101,110,103,116,104,0,0,0,0,0,87,114,111,110,103,32,74,80,69,71,32,108,105,98,114,97,114,121,32,118,101,114,115,105,111,110,58,32,108,105,98,114,97,114,121,32,105,115,32,37,100,44,32,99,97,108,108,101,114,32,101,120,112,101,99,116,115,32,37,100,0,0,0,0,83,97,109,112,108,105,110,103,32,102,97,99,116,111,114,115,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,105,110,116,101,114,108,101,97,118,101,100,32,115,99,97,110,0,73,110,118,97,108,105,100,32,109,101,109,111,114,121,32,112,111,111,108,32,99,111,100,101,32,37,100,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,74,80,69,71,32,100,97,116,97,32,112,114,101,99,105,115,105,111,110,32,37,100,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,114,111,103,114,101,115,115,105,118,101,32,112,97,114,97,109,101,116,101,114,115,32,83,115,61,37,100,32,83,101,61,37,100,32,65,104,61,37,100,32,65,108,61,37,100,0,0,73,110,118,97,108,105,100,32,112,114,111,103,114,101,115,115,105,118,101,32,112,97,114,97,109,101,116,101,114,115,32,97,116,32,115,99,97,110,32,115,99,114,105,112,116,32,101,110,116,114,121,32,37,100,0,0,66,111,103,117,115,32,115,97,109,112,108,105,110,103,32,102,97,99,116,111,114,115,0,0,73,110,118,97,108,105,100,32,115,99,97,110,32,115,99,114,105,112,116,32,97,116,32,101,110,116,114,121,32,37,100,0,73,109,112,114,111,112,101,114,32,99,97,108,108,32,116,111,32,74,80,69,71,32,108,105,98,114,97,114,121,32,105,110,32,115,116,97,116,101,32,37,100,0,0,0,0,0,0,0,74,80,69,71,32,112,97,114,97,109,101,116,101,114,32,115,116,114,117,99,116,32,109,105,115,109,97,116,99,104,58,32,108,105,98,114,97,114,121,32,116,104,105,110,107,115,32,115,105,122,101,32,105,115,32,37,117,44,32,99,97,108,108,101,114,32,101,120,112,101,99,116,115,32,37,117,0,0,0,0,66,111,103,117,115,32,118,105,114,116,117,97,108,32,97,114,114,97,121,32,97,99,99,101,115,115,0,0,0,0,0,0,66,117,102,102,101,114,32,112,97,115,115,101,100,32,116,111,32,74,80,69,71,32,108,105,98,114,97,114,121,32,105,115,32,116,111,111,32,115,109,97,108,108,0,0,0,0,0,0,83,117,115,112,101,110,115,105,111,110,32,110,111,116,32,97,108,108,111,119,101,100,32,104,101,114,101,0,0,0,0,0,67,67,73,82,54,48,49,32,115,97,109,112,108,105,110,103,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,0,0,0,0,84,111,111,32,109,97,110,121,32,99,111,108,111,114,32,99,111,109,112,111,110,101,110,116,115,58,32,37,100,44,32,109,97,120,32,37,100,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,99,111,108,111,114,32,99,111,110,118,101,114,115,105,111,110,32,114,101,113,117,101,115,116,0,0,0,0,66,111,103,117,115,32,68,65,67,32,105,110,100,101,120,32,37,100,0,0,0,0,0,0,66,111,103,117,115,32,68,65,67,32,118,97,108,117,101,32,48,120,37,120,0,0,0,0,66,111,103,117,115,32,68,72,84,32,105,110,100,101,120,32,37,100,0,0,0,0,0,0,66,111,103,117,115,32,68,81,84,32,105,110,100,101,120,32,37,100,0,0,0,0,0,0,69,109,112,116,121,32,74,80,69,71,32,105,109,97,103,101,32,40,68,78,76,32,110,111,116,32,115,117,112,112,111,114,116,101,100,41,0,0,0,0,82,101,97,100,32,102,114,111,109,32,69,77,83,32,102,97,105,108,101,100,0,0,0,0,87,114,105,116,101,32,116,111,32,69,77,83,32,102,97,105,108,101,100,0,0,0,0,0,68,105,100,110,39,116,32,101,120,112,101,99,116,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,115,99,97,110,0,0,0,0,0,0,0,0,73,110,112,117,116,32,102,105,108,101,32,114,101,97,100,32,101,114,114,111,114,0,0,0,79,117,116,112,117,116,32,102,105,108,101,32,119,114,105,116,101,32,101,114,114,111,114,32,45,45,45,32,111,117,116,32,111,102,32,100,105,115,107,32,115,112,97,99,101,63,0,0,70,114,97,99,116,105,111,110,97,108,32,115,97,109,112,108,105,110,103,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,0,72,117,102,102,109,97,110,32,99,111,100,101,32,115,105,122,101,32,116,97,98,108,101,32,111,118,101,114,102,108,111,119,0,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,72,117,102,102,109,97,110,32,99,111,100,101,32,116,97,98,108,101,32,101,110,116,114,121,0,0,0,0,0,0,0,0,77,97,120,105,109,117,109,32,115,117,112,112,111,114,116,101,100,32,105,109,97,103,101,32,100,105,109,101,110,115,105,111,110,32,105,115,32,37,117,32,112,105,120,101,108,115,0,0,69,109,112,116,121,32,105,110,112,117,116,32,102,105,108,101,0,0,0,0,0,0,0,0,80,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,105,110,112,117,116,32,102,105,108,101,0,0,0,0,0,67,97,110,110,111,116,32,116,114,97,110,115,99,111,100,101,32,100,117,101,32,116,111,32,109,117,108,116,105,112,108,101,32,117,115,101,32,111,102,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,32,37,100,0,0,0,83,99,97,110,32,115,99,114,105,112,116,32,100,111,101,115,32,110,111,116,32,116,114,97,110,115,109,105,116,32,97,108,108,32,100,97,116,97,0,0,73,110,118,97,108,105,100,32,99,111,108,111,114,32,113,117,97,110,116,105,122,97,116,105,111,110,32,109,111,100,101,32,99,104,97,110,103,101,0,0,78,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,0,0,0,0,0,82,101,113,117,101,115,116,101,100,32,102,101,97,116,117,114,101,32,119,97,115,32,111,109,105,116,116,101,100,32,97,116,32,99,111,109,112,105,108,101,32,116,105,109,101,0,0,0,66,97,99,107,105,110,103,32,115,116,111,114,101,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,0,0,0,0,72,117,102,102,109,97,110,32,116,97,98,108,101,32,48,120,37,48,50,120,32,119,97,115,32,110,111,116,32,100,101,102,105,110,101,100,0,0,0,0,74,80,69,71,32,100,97,116,97,115,116,114,101,97,109,32,99,111,110,116,97,105,110,115,32,110,111,32,105,109,97,103,101,0,0,0,0,0,0,0,81,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,32,48,120,37,48,50,120,32,119,97,115,32,110,111,116,32,100,101,102,105,110,101,100,0,0,0,0,0,0,0,78,111,116,32,97,32,74,80,69,71,32,102,105,108,101,58,32,115,116,97,114,116,115,32,119,105,116,104,32,48,120,37,48,50,120,32,48,120,37,48,50,120,0,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,40,99,97,115,101,32,37,100,41,0,0,0,67,97,110,110,111,116,32,113,117,97,110,116,105,122,101,32,109,111,114,101,32,116,104,97,110,32,37,100,32,99,111,108,111,114,32,99,111,109,112,111,110,101,110,116,115,0,0,0,67,97,110,110,111,116,32,113,117,97,110,116,105,122,101,32,116,111,32,102,101,119,101,114,32,116,104,97,110,32,37,100,32,99,111,108,111,114,115,0,67,97,110,110,111,116,32,113,117,97,110,116,105,122,101,32,116,111,32,109,111,114,101,32,116,104,97,110,32,37,100,32,99,111,108,111,114,115,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,116,119,111,32,83,79,70,32,109,97,114,107,101,114,115,0,0,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,109,105,115,115,105,110,103,32,83,79,83,32,109,97,114,107,101,114,0,85,110,115,117,112,112,111,114,116,101,100,32,74,80,69,71,32,112,114,111,99,101,115,115,58,32,83,79,70,32,116,121,112,101,32,48,120,37,48,50,120,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,116,119,111,32,83,79,73,32,109,97,114,107,101,114,115,0,0,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,83,79,83,32,98,101,102,111,114,101,32,83,79,70,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,99,114,101,97,116,101,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,37,115,0,0,0,0,0,0,82,101,97,100,32,102,97,105,108,101,100,32,111,110,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,0,0,0,83,101,101,107,32,102,97,105,108,101,100,32,111,110,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,0,0,0,87,114,105,116,101,32,102,97,105,108,101,100,32,111,110,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,45,45,45,32,111,117,116,32,111,102,32,100,105,115,107,32,115,112,97,99,101,63,0,0,0,65,112,112,108,105,99,97,116,105,111,110,32,116,114,97,110,115,102,101,114,114,101,100,32,116,111,111,32,102,101,119,32,115,99,97,110,108,105,110,101,115,0,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,109,97,114,107,101,114,32,116,121,112,101,32,48,120,37,48,50,120,0,0,86,105,114,116,117,97,108,32,97,114,114,97,121,32,99,111,110,116,114,111,108,108,101,114,32,109,101,115,115,101,100,32,117,112,0,0,0,0,0,0,73,109,97,103,101,32,116,111,111,32,119,105,100,101,32,102,111,114,32,116,104,105,115,32,105,109,112,108,101,109,101,110,116,97,116,105,111,110,0,0,82,101,97,100,32,102,114,111,109,32,88,77,83,32,102,97,105,108,101,100,0,0,0,0,87,114,105,116,101,32,116,111,32,88,77,83,32,102,97,105,108,101,100,0,0,0,0,0,67,111,112,121,114,105,103,104,116,32,40,67,41,32,49,57,57,56,44,32,84,104,111,109,97,115,32,71,46,32,76,97,110,101,0,0,0,0,0,0,54,98,32,32,50,55,45,77,97,114,45,49,57,57,56,0,67,97,117,116,105,111,110,58,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,115,32,97,114,101,32,116,111,111,32,99,111,97,114,115,101,32,102,111,114,32,98,97,115,101,108,105,110,101,32,74,80,69,71,0,0,0,65,100,111,98,101,32,65,80,80,49,52,32,109,97,114,107,101,114,58,32,118,101,114,115,105,111,110,32,37,100,44,32,102,108,97,103,115,32,48,120,37,48,52,120,32,48,120,37,48,52,120,44,32,116,114,97,110,115,102,111,114,109,32,37,100,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,65,80,80,48,32,109,97,114,107,101,114,32,40,110,111,116,32,74,70,73,70,41,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,65,80,80,49,52,32,109,97,114,107,101,114,32,40,110,111,116,32,65,100,111,98,101,41,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,68,101,102,105,110,101,32,65,114,105,116,104,109,101,116,105,99,32,84,97,98,108,101,32,48,120,37,48,50,120,58,32,48,120,37,48,50,120,0,0,68,101,102,105,110,101,32,72,117,102,102,109,97,110,32,84,97,98,108,101,32,48,120,37,48,50,120,0,0,0,0,0,68,101,102,105,110,101,32,81,117,97,110,116,105,122,97,116,105,111,110,32,84,97,98,108,101,32,37,100,32,32,112,114,101,99,105,115,105,111,110,32,37,100,0,0,0,0,0,0,68,101,102,105,110,101,32,82,101,115,116,97,114,116,32,73,110,116,101,114,118,97,108,32,37,117,0,0,0,0,0,0,70,114,101,101,100,32,69,77,83,32,104,97,110,100,108,101,32,37,117,0,0,0,0,0,79,98,116,97,105,110,101,100,32,69,77,83,32,104,97,110,100,108,101,32,37,117,0,0,69,110,100,32,79,102,32,73,109,97,103,101,0,0,0,0,32,32,32,32,32,32,32,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,0,74,70,73,70,32,65,80,80,48,32,109,97,114,107,101,114,58,32,118,101,114,115,105,111,110,32,37,100,46,37,48,50,100,44,32,100,101,110,115,105,116,121,32,37,100,120,37,100,32,32,37,100,0,0,0,0,87,97,114,110,105,110,103,58,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,32,115,105,122,101,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,100,97,116,97,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,116,121,112,101,32,48,120,37,48,50,120,44,32,108,101,110,103,116,104,32,37,117,0,0,0,32,32,32,32,119,105,116,104,32,37,100,32,120,32,37,100,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,0,0,0,0,0,0,0,0,77,105,115,99,101,108,108,97,110,101,111,117,115,32,109,97,114,107,101,114,32,48,120,37,48,50,120,44,32,108,101,110,103,116,104,32,37,117,0,0,85,110,101,120,112,101,99,116,101,100,32,109,97,114,107,101,114,32,48,120,37,48,50,120,0,0,0,0,0,0,0,0,32,32,32,32,32,32,32,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,0,81,117,97,110,116,105,122,105,110,103,32,116,111,32,37,100,32,61,32,37,100,42,37,100,42,37,100,32,99,111,108,111,114,115,0,0,0,0,0,0,81,117,97,110,116,105,122,105,110,103,32,116,111,32,37,100,32,99,111,108,111,114,115,0,83,101,108,101,99,116,101,100,32,37,100,32,99,111,108,111,114,115,32,102,111,114,32,113,117,97,110,116,105,122,97,116,105,111,110,0,0,0,0,0,65,116,32,109,97,114,107,101,114,32,48,120,37,48,50,120,44,32,114,101,99,111,118,101,114,121,32,97,99,116,105,111,110,32,37,100,0,0,0,0,82,83,84,37,100,0,0,0,83,109,111,111,116,104,105,110,103,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,119,105,116,104,32,110,111,110,115,116,97,110,100,97,114,100,32,115,97,109,112,108,105,110,103,32,114,97,116,105,111,115,0,0,0,0,0,0,0,0,83,116,97,114,116,32,79,102,32,70,114,97,109,101,32,48,120,37,48,50,120,58,32,119,105,100,116,104,61,37,117,44,32,104,101,105,103,104,116,61,37,117,44,32,99,111,109,112,111,110,101,110,116,115,61,37,100,0,0,0,0,0,0,0,32,32,32,32,67,111,109,112,111,110,101,110,116,32,37,100,58,32,37,100,104,120,37,100,118,32,113,61,37,100,0,0,83,116,97,114,116,32,111,102,32,73,109,97,103,101,0,0,83,116,97,114,116,32,79,102,32,83,99,97,110,58,32,37,100,32,99,111,109,112,111,110,101,110,116,115,0,0,0,0,32,32,32,32,67,111,109,112,111,110,101,110,116,32,37,100,58,32,100,99,61,37,100,32,97,99,61,37,100,0,0,0,32,32,83,115,61,37,100,44,32,83,101,61,37,100,44,32,65,104,61,37,100,44,32,65,108,61,37,100,0,0,0,0,67,108,111,115,101,100,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,37,115,0,0,0,0,0,0,0,0,79,112,101,110,101,100,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,37,115,0,0,0,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,74,80,69,71,45,99,111,109,112,114,101,115,115,101,100,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,112,97,108,101,116,116,101,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,82,71,66,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,44,32,108,101,110,103,116,104,32,37,117,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,99,111,109,112,111,110,101,110,116,32,73,68,115,32,37,100,32,37,100,32,37,100,44,32,97,115,115,117,109,105,110,103,32,89,67,98,67,114,0,0,0,0,0,70,114,101,101,100,32,88,77,83,32,104,97,110,100,108,101,32,37,117,0,0,0,0,0,79,98,116,97,105,110,101,100,32,88,77,83,32,104,97,110,100,108,101,32,37,117,0,0,85,110,107,110,111,119,110,32,65,100,111,98,101,32,99,111,108,111,114,32,116,114,97,110,115,102,111,114,109,32,99,111,100,101,32,37,100,0,0,0,73,110,99,111,110,115,105,115,116,101,110,116,32,112,114,111,103,114,101,115,115,105,111,110,32,115,101,113,117,101,110,99,101,32,102,111,114,32,99,111,109,112,111,110,101,110,116,32,37,100,32,99,111,101,102,102,105,99,105,101,110,116,32,37,100,0,0,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,37,117,32,101,120,116,114,97,110,101,111,117,115,32,98,121,116,101,115,32,98,101,102,111,114,101,32,109,97,114,107,101,114,32,48,120,37,48,50,120,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,112,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,100,97,116,97,32,115,101,103,109,101,110,116,0,0,0,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,98,97,100,32,72,117,102,102,109,97,110,32,99,111,100,101,0,0,0,0,0,87,97,114,110,105,110,103,58,32,117,110,107,110,111,119,110,32,74,70,73,70,32,114,101,118,105,115,105,111,110,32,110,117,109,98,101,114,32,37,100,46,37,48,50,100,0,0,0,80,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,74,80,69,71,32,102,105,108,101,0,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,102,111,117,110,100,32,109,97,114,107,101,114,32,48,120,37,48,50,120,32,105,110,115,116,101,97,100,32,111,102,32,82,83,84,37,100,0,73,110,118,97,108,105,100,32,83,79,83,32,112,97,114,97,109,101,116,101,114,115,32,102,111,114,32,115,101,113,117,101,110,116,105,97,108,32,74,80,69,71,0,0,0,0,0,0,65,112,112,108,105,99,97,116,105,111,110,32,116,114,97,110,115,102,101,114,114,101,100,32,116,111,111,32,109,97,110,121,32,115,99,97,110,108,105,110,101,115,0,0,0,0,0,0,128,14,0,0,152,14,0,0,216,14,0,0,248,14,0,0,32,15,0,0,64,15,0,0,96,15,0,0,128,15,0,0,168,15,0,0,200,15,0,0,224,15,0,0,248,15,0,0,16,16,0,0,80,16,0,0,128,16,0,0,160,16,0,0,200,16,0,0,0,17,0,0,56,17,0,0,80,17,0,0,112,17,0,0,160,17,0,0,240,17,0,0,16,18,0,0,64,18,0,0,96,18,0,0,136,18,0,0,176,18,0,0,216,18,0,0,240,18,0,0,8,19,0,0,32,19,0,0,56,19,0,0,96,19,0,0,120,19,0,0,144,19,0,0,184,19,0,0,208,19,0,0,0,20,0,0,40,20,0,0,80,20,0,0,120,20,0,0,168,20,0,0,192,20,0,0,224,20,0,0,32,21,0,0,72,21,0,0,112,21,0,0,136,21,0,0,184,21,0,0,216,21,0,0,0,22,0,0,40,22,0,0,88,22,0,0,136,22,0,0,168,22,0,0,216,22,0,0,0,23,0,0,40,23,0,0,88,23,0,0,136,23,0,0,184,23,0,0,232,23,0,0,24,24,0,0,64,24,0,0,96,24,0,0,128,24,0,0,184,24,0,0,232,24,0,0,8,25,0,0,48,25,0,0,88,25,0,0,112,25,0,0,136,25,0,0,176,25,0,0,192,25,0,0,0,26,0,0,72,26,0,0,120,26,0,0,168,26,0,0,208,26,0,0,240,26,0,0,32,27,0,0,64,27,0,0,88,27,0,0,112,27,0,0,128,27,0,0,168,27,0,0,224,27,0,0,32,28,0,0,80,28,0,0,120,28,0,0,160,28,0,0,192,28,0,0,232,28,0,0,16,29,0,0,40,29,0,0,80,29,0,0,120,29,0,0,128,29,0,0,192,29,0,0,0,30,0,0,32,30,0,0,48,30,0,0,80,30,0,0,112,30,0,0,144,30,0,0,176,30,0,0,208,30,0,0,24,31,0,0,88,31,0,0,144,31,0,0,200,31,0,0,224,31,0,0,248,31,0,0,32,32,0,0,104,32,0,0,168,32,0,0,224,32,0,0,8,33,0,0,56,33,0,0,88,33,0,0,144,33,0,0,192,33,0,0,0,0,0,0,0,0,0,0,37,115,10,0,0,0,0,0,74,80,69,71,77,69,77,0,37,108,100,37,99,0,0,0,64,6,0,0,128,62,0,0,0,0,0,0,136,19,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,192,48,240,12,204,60,252,3,195,51,243,15,207,63,255,128,64,176,112,140,76,188,124,131,67,179,115,143,79,191,127,32,224,16,208,44,236,28,220,35,227,19,211,47,239,31,223,160,96,144,80,172,108,156,92,163,99,147,83,175,111,159,95,8,200,56,248,4,196,52,244,11,203,59,251,7,199,55,247,136,72,184,120,132,68,180,116,139,75,187,123,135,71,183,119,40,232,24,216,36,228,20,212,43,235,27,219,39,231,23,215,168,104,152,88,164,100,148,84,171,107,155,91,167,103,151,87,2,194,50,242,14,206,62,254,1,193,49,241,13,205,61,253,130,66,178,114,142,78,190,126,129,65,177,113,141,77,189,125,34,226,18,210,46,238,30,222,33,225,17,209,45,237,29,221,162,98,146,82,174,110,158,94,161,97,145,81,173,109,157,93,10,202,58,250,6,198,54,246,9,201,57,249,5,197,53,245,138,74,186,122,134,70,182,118,137,73,185,121,133,69,181,117,42,234,26,218,38,230,22,214,41,233,25,217,37,229,21,213,170,106,154,90,166,102,150,86,169,105,153,89,165,101,149,85,0,0,0,0,1,0,0,0,8,0,0,0,16,0,0,0,9,0,0,0,2,0,0,0,3,0,0,0,10,0,0,0,17,0,0,0,24,0,0,0,32,0,0,0,25,0,0,0,18,0,0,0,11,0,0,0,4,0,0,0,5,0,0,0,12,0,0,0,19,0,0,0,26,0,0,0,33,0,0,0,40,0,0,0,48,0,0,0,41,0,0,0,34,0,0,0,27,0,0,0,20,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,14,0,0,0,21,0,0,0,28,0,0,0,35,0,0,0,42,0,0,0,49,0,0,0,56,0,0,0,57,0,0,0,50,0,0,0,43,0,0,0,36,0,0,0,29,0,0,0,22,0,0,0,15,0,0,0,23,0,0,0,30,0,0,0,37,0,0,0,44,0,0,0,51,0,0,0,58,0,0,0,59,0,0,0,52,0,0,0,45,0,0,0,38,0,0,0,31,0,0,0,39,0,0,0,46,0,0,0,53,0,0,0,60,0,0,0,61,0,0,0,54,0,0,0,47,0,0,0,55,0,0,0,62,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,0,64,197,88,159,83,66,75,0,64,73,50,163,34,168,17,197,88,33,123,252,115,98,104,197,88,191,69,11,48,126,24,159,83,252,115,65,109,84,98,159,83,179,65,65,45,18,23,66,75,98,104,84,98,126,88,66,75,33,59,186,40,195,20,0,64,197,88,159,83,66,75,0,64,73,50,163,34,168,17,73,50,191,69,179,65,33,59,73,50,130,39,55,27,224,13,163,34,11,48,65,45,186,40,163,34,55,27,191,18,142,9,168,17,126,24,18,23,195,20,168,17,224,13,142,9,223,4,0,0,0,0,0,0,240,63,239,97,72,177,80,49,246,63,202,111,77,145,174,231,244,63,170,17,108,239,98,208,242,63,0,0,0,0,0,0,240,63,59,191,167,192,105,36,233,63,187,32,199,123,122,81,225,63,93,171,114,222,85,168,209,63,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,128,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,0,0,0,255,255,255,255,253,255,255,255,249,255,255,255,241,255,255,255,225,255,255,255,193,255,255,255,129,255,255,255,1,255,255,255,1,254,255,255,1,252,255,255,1,248,255,255,1,240,255,255,1,224,255,255,1,192,255,255,1,128,255,255,2,0,0,192,3,0,0,192,4,0,0,192,5,0,0,192,6,0,0,192,7,0,0,192,8,0,0,192,9,0,0,192,10,0,0,192,11,0,0,192,12,0,0,192,13,0,0,192,14,0,0,192,15,0,0,192,16,0,0,192,17,0,0,192,18,0,0,192,19,0,0,192,20,0,0,192,21,0,0,192,22,0,0,192,23,0,0,192,24,0,0,192,25,0,0,192,26,0,0,192,27,0,0,192,28,0,0,192,29,0,0,192,30,0,0,192,31,0,0,192,0,0,0,179,1,0,0,195,2,0,0,195,3,0,0,195,4,0,0,195,5,0,0,195,6,0,0,195,7,0,0,195,8,0,0,195,9,0,0,195,10,0,0,195,11,0,0,195,12,0,0,195,13,0,0,211,14,0,0,195,15,0,0,195,0,0,12,187,1,0,12,195,2,0,12,195,3,0,12,195,4,0,12,211,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,1,2,3,4,5,6,7,8,9,255,255,255,255,255,255,255,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,255,255,255,255,255,255,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,1,2,4,7,3,6,5,0,0,0,0,0,0,0,0,105,110,102,105,110,105,116,121,0,0,0,0,0,0,0,0,110,97,110,0,0,0,0,0,95,112,137,0,255,9,47,15,10,0,0,0,100,0,0,0,232,3,0,0,16,39,0,0,160,134,1,0,64,66,15,0,128,150,152,0,0,225,245,5,17,0,10,0,17,17,17,0,0,0,0,5,0,0,0,0,0,0,9,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,15,10,17,17,17,3,10,7,0,1,19,9,11,11,0,0,9,6,11,0,0,11,0,6,17,0,0,0,17,17,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,10,10,17,17,17,0,10,0,0,2,0,9,11,0,0,0,9,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,4,13,0,0,0,0,9,14,0,0,0,0,0,14,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,15,0,0,0,0,9,16,0,0,0,0,0,16,0,0,16,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,10,0,0,0,0,9,11,0,0,0,0,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,45,43,32,32,32,48,88,48,120,0,0,0,0,0,0,0,40,110,117,108,108,41,0,0,45,48,88,43,48,88,32,48,88,45,48,120,43,48,120,32,48,120,0,0,0,0,0,0,105,110,102,0,0,0,0,0,73,78,70,0,0,0,0,0,110,97,110,0,0,0,0,0,78,65,78,0,0,0,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);




var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}


   
  Module["_i64Subtract"] = _i64Subtract;

  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 79:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

   
  Module["_memset"] = _memset;

  
  
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            continue;
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          if (stream.tty.output.length) {
            stream.tty.ops.put_char(stream.tty, 10);
          }
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            },
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (node.contents && node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
  
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
  
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              assert(position === 0, 'canOwn must imply no weird position inside the file');
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          stream.ungotten = [];
          stream.position = position;
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          fileStore.createIndex('timestamp', 'timestamp', { unique: false });
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function() {
          callback(this.error);
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function() { callback(this.error); };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function() { callback(this.error); };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function() { done(this.error); };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          stream.position = position;
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
      // we don't currently perform any user-space buffering of data
    }var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); },
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); },
            },
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        return FS.nodePermissions(dir, 'x');
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },getStreamFromPtr:function (ptr) {
        return FS.streams[ptr - 1];
      },getPtrForStream:function (stream) {
        return stream ? stream.fd + 1 : 0;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions
        var err = FS.mayOpen(node, flags);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        return stream.stream_ops.llseek(stream, offset, whence);
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = '';
          var utf8 = new Runtime.UTF8Processor();
          for (var i = 0; i < length; i++) {
            ret += utf8.processCChar(buf[i]);
          }
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var utf8 = new Runtime.UTF8Processor();
          var buf = new Uint8Array(utf8.processJSString(data));
          FS.write(stream, buf, 0, buf.length, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return Math.floor(Math.random()*256); };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=FS.getPtrForStream(stdin);
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=FS.getPtrForStream(stdout);
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=FS.getPtrForStream(stderr);
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno) {
          this.errno = errno;
          for (var key in ERRNO_CODES) {
            if (ERRNO_CODES[key] === errno) {
              this.code = key;
              break;
            }
          }
          this.message = ERRNO_MESSAGES[errno];
          if (this.stack) this.stack = demangleAll(this.stack);
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = Math.floor(idx / this.chunkSize);
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
            // Find length
            var xhr = new XMLHttpRequest();
            xhr.open('HEAD', url, false);
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            var datalength = Number(xhr.getResponseHeader("Content-length"));
            var header;
            var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
            var chunkSize = 1024*1024; // Chunk size in bytes
  
            if (!hasByteServing) chunkSize = datalength;
  
            // Function to get a range from the remote URL.
            var doXHR = (function(from, to) {
              if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
              if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
              // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
              var xhr = new XMLHttpRequest();
              xhr.open('GET', url, false);
              if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
              // Some hints to the browser that we want binary data.
              if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
              if (xhr.overrideMimeType) {
                xhr.overrideMimeType('text/plain; charset=x-user-defined');
              }
  
              xhr.send(null);
              if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
              if (xhr.response !== undefined) {
                return new Uint8Array(xhr.response || []);
              } else {
                return intArrayFromString(xhr.responseText || '', true);
              }
            });
            var lazyArray = this;
            lazyArray.setDataGetter(function(chunkNum) {
              var start = chunkNum * chunkSize;
              var end = (chunkNum+1) * chunkSize - 1; // including this byte
              end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
              if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
                lazyArray.chunks[chunkNum] = doXHR(start, end);
              }
              if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
              return lazyArray.chunks[chunkNum];
            });
  
            this._length = datalength;
            this._chunkSize = chunkSize;
            this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};function _close(fildes) {
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.close(stream);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _fsync(fildes) {
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }
  
  function _fileno(stream) {
      // int fileno(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fileno.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) return -1;
      return stream.fd;
    }function _fclose(stream) {
      // int fclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fclose.html
      var fd = _fileno(stream);
      _fsync(fd);
      return _close(fd);
    }

  
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          writeAsciiToMemory(msg, strerrbuf);
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }
  
  function _malloc(bytes) {
      /* Over-allocate to make sure it is byte-aligned by 8.
       * This will leak memory, but this is only the dummy
       * implementation (replaced by dlmalloc normally) so
       * not an issue.
       */
      var ptr = Runtime.dynamicAlloc(bytes + 8);
      return (ptr+8) & 0xFFFFFFF8;
    }
  Module["_malloc"] = _malloc;function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  
  
  
  
  
  function _mkport() { throw 'TODO' }var SOCKFS={mount:function (mount) {
        return FS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              // runtimeConfig gets set to true if WebSocket runtime configuration is available.
              var runtimeConfig = (Module['websocket'] && ('object' === typeof Module['websocket']));
  
              // The default value is 'ws://' the replace is needed because the compiler replaces "//" comments with '#'
              // comments without checking context, so we'd end up with ws:#, the replace swaps the "#" for "//" again.
              var url = 'ws:#'.replace('#', '//');
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['url']) {
                  url = Module['websocket']['url']; // Fetch runtime WebSocket URL config.
                }
              }
  
              if (url === 'ws://' || url === 'wss://') { // Is the supplied URL config just a prefix, if so complete it.
                url = url + addr + ':' + port;
              }
  
              // Make the WebSocket subprotocol (Sec-WebSocket-Protocol) default to binary if no configuration is set.
              var subProtocols = 'binary'; // The default value is 'binary'
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['subprotocol']) {
                  subProtocols = Module['websocket']['subprotocol']; // Fetch runtime WebSocket subprotocol config.
                }
              }
  
              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
  
              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
              var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
  
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('error', function() {
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODS.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
            }
          });
          sock.server.on('closed', function() {
            sock.server = null;
          });
          sock.server.on('error', function() {
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fwrite(ptr, size, nitems, stream) {
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var fd = _fileno(stream);
      var bytesWritten = _write(fd, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return 0;
      } else {
        return Math.floor(bytesWritten / size);
      }
    }
  
  
   
  Module["_strlen"] = _strlen;
  
  function __reallyNegative(x) {
      return x < 0 || (x === 0 && (1/x) === -Infinity);
    }function __formatString(format, varargs) {
      var textIndex = format;
      var argIndex = 0;
      function getNextArg(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        if (type === 'double') {
          ret = (HEAP32[((tempDoublePtr)>>2)]=HEAP32[(((varargs)+(argIndex))>>2)],HEAP32[(((tempDoublePtr)+(4))>>2)]=HEAP32[(((varargs)+((argIndex)+(4)))>>2)],(+(HEAPF64[(tempDoublePtr)>>3])));
        } else if (type == 'i64') {
          ret = [HEAP32[(((varargs)+(argIndex))>>2)],
                 HEAP32[(((varargs)+(argIndex+4))>>2)]];
  
        } else {
          type = 'i32'; // varargs are always i32, i64, or double
          ret = HEAP32[(((varargs)+(argIndex))>>2)];
        }
        argIndex += Runtime.getNativeFieldSize(type);
        return ret;
      }
  
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[((textIndex)>>0)];
        if (curr === 0) break;
        next = HEAP8[((textIndex+1)>>0)];
        if (curr == 37) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          var flagPadSign = false;
          flagsLoop: while (1) {
            switch (next) {
              case 43:
                flagAlwaysSigned = true;
                break;
              case 45:
                flagLeftAlign = true;
                break;
              case 35:
                flagAlternative = true;
                break;
              case 48:
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              case 32:
                flagPadSign = true;
                break;
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          }
  
          // Handle width.
          var width = 0;
          if (next == 42) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          } else {
            while (next >= 48 && next <= 57) {
              width = width * 10 + (next - 48);
              textIndex++;
              next = HEAP8[((textIndex+1)>>0)];
            }
          }
  
          // Handle precision.
          var precisionSet = false, precision = -1;
          if (next == 46) {
            precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
            if (next == 42) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[((textIndex+1)>>0)];
                if (precisionChr < 48 ||
                    precisionChr > 57) break;
                precision = precision * 10 + (precisionChr - 48);
                textIndex++;
              }
            }
            next = HEAP8[((textIndex+1)>>0)];
          }
          if (precision < 0) {
            precision = 6; // Standard default.
            precisionSet = false;
          }
  
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 104) {
                textIndex++;
                argSize = 1; // char (actually i32 in varargs)
              } else {
                argSize = 2; // short (actually i32 in varargs)
              }
              break;
            case 'l':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 108) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = null;
          }
          if (argSize) textIndex++;
          next = HEAP8[((textIndex+1)>>0)];
  
          // Handle type specifier.
          switch (String.fromCharCode(next)) {
            case 'd': case 'i': case 'u': case 'o': case 'x': case 'X': case 'p': {
              // Integer.
              var signed = next == 100 || next == 105;
              argSize = argSize || 4;
              var currArg = getNextArg('i' + (argSize * 8));
              var origArg = currArg;
              var argText;
              // Flatten i64-1 [low, high] into a (slightly rounded) double
              if (argSize == 8) {
                currArg = Runtime.makeBigInt(currArg[0], currArg[1], next == 117);
              }
              // Truncate to requested size.
              if (argSize <= 4) {
                var limit = Math.pow(256, argSize) - 1;
                currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
              }
              // Format the number.
              var currAbsArg = Math.abs(currArg);
              var prefix = '';
              if (next == 100 || next == 105) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], null); else
                argText = reSign(currArg, 8 * argSize, 1).toString(10);
              } else if (next == 117) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], true); else
                argText = unSign(currArg, 8 * argSize, 1).toString(10);
                currArg = Math.abs(currArg);
              } else if (next == 111) {
                argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
              } else if (next == 120 || next == 88) {
                prefix = (flagAlternative && currArg != 0) ? '0x' : '';
                if (argSize == 8 && i64Math) {
                  if (origArg[1]) {
                    argText = (origArg[1]>>>0).toString(16);
                    var lower = (origArg[0]>>>0).toString(16);
                    while (lower.length < 8) lower = '0' + lower;
                    argText += lower;
                  } else {
                    argText = (origArg[0]>>>0).toString(16);
                  }
                } else
                if (currArg < 0) {
                  // Represent negative numbers in hex as 2's complement.
                  currArg = -currArg;
                  argText = (currAbsArg - 1).toString(16);
                  var buffer = [];
                  for (var i = 0; i < argText.length; i++) {
                    buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                  }
                  argText = buffer.join('');
                  while (argText.length < argSize * 2) argText = 'f' + argText;
                } else {
                  argText = currAbsArg.toString(16);
                }
                if (next == 88) {
                  prefix = prefix.toUpperCase();
                  argText = argText.toUpperCase();
                }
              } else if (next == 112) {
                if (currAbsArg === 0) {
                  argText = '(nil)';
                } else {
                  prefix = '0x';
                  argText = currAbsArg.toString(16);
                }
              }
              if (precisionSet) {
                while (argText.length < precision) {
                  argText = '0' + argText;
                }
              }
  
              // Add sign if needed
              if (currArg >= 0) {
                if (flagAlwaysSigned) {
                  prefix = '+' + prefix;
                } else if (flagPadSign) {
                  prefix = ' ' + prefix;
                }
              }
  
              // Move sign to prefix so we zero-pad after the sign
              if (argText.charAt(0) == '-') {
                prefix = '-' + prefix;
                argText = argText.substr(1);
              }
  
              // Add padding.
              while (prefix.length + argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad) {
                    argText = '0' + argText;
                  } else {
                    prefix = ' ' + prefix;
                  }
                }
              }
  
              // Insert the result into the buffer.
              argText = prefix + argText;
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 'f': case 'F': case 'e': case 'E': case 'g': case 'G': {
              // Float.
              var currArg = getNextArg('double');
              var argText;
              if (isNaN(currArg)) {
                argText = 'nan';
                flagZeroPad = false;
              } else if (!isFinite(currArg)) {
                argText = (currArg < 0 ? '-' : '') + 'inf';
                flagZeroPad = false;
              } else {
                var isGeneral = false;
                var effectivePrecision = Math.min(precision, 20);
  
                // Convert g/G to f/F or e/E, as per:
                // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
                if (next == 103 || next == 71) {
                  isGeneral = true;
                  precision = precision || 1;
                  var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                  if (precision > exponent && exponent >= -4) {
                    next = ((next == 103) ? 'f' : 'F').charCodeAt(0);
                    precision -= exponent + 1;
                  } else {
                    next = ((next == 103) ? 'e' : 'E').charCodeAt(0);
                    precision--;
                  }
                  effectivePrecision = Math.min(precision, 20);
                }
  
                if (next == 101 || next == 69) {
                  argText = currArg.toExponential(effectivePrecision);
                  // Make sure the exponent has at least 2 digits.
                  if (/[eE][-+]\d$/.test(argText)) {
                    argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                  }
                } else if (next == 102 || next == 70) {
                  argText = currArg.toFixed(effectivePrecision);
                  if (currArg === 0 && __reallyNegative(currArg)) {
                    argText = '-' + argText;
                  }
                }
  
                var parts = argText.split('e');
                if (isGeneral && !flagAlternative) {
                  // Discard trailing zeros and periods.
                  while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                         (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                    parts[0] = parts[0].slice(0, -1);
                  }
                } else {
                  // Make sure we have a period in alternative mode.
                  if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                  // Zero pad until required precision.
                  while (precision > effectivePrecision++) parts[0] += '0';
                }
                argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
  
                // Capitalize 'E' if needed.
                if (next == 69) argText = argText.toUpperCase();
  
                // Add sign.
                if (currArg >= 0) {
                  if (flagAlwaysSigned) {
                    argText = '+' + argText;
                  } else if (flagPadSign) {
                    argText = ' ' + argText;
                  }
                }
              }
  
              // Add padding.
              while (argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                    argText = argText[0] + '0' + argText.slice(1);
                  } else {
                    argText = (flagZeroPad ? '0' : ' ') + argText;
                  }
                }
              }
  
              // Adjust case.
              if (next < 97) argText = argText.toUpperCase();
  
              // Insert the result into the buffer.
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 's': {
              // String.
              var arg = getNextArg('i8*');
              var argLength = arg ? _strlen(arg) : '(null)'.length;
              if (precisionSet) argLength = Math.min(argLength, precision);
              if (!flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              if (arg) {
                for (var i = 0; i < argLength; i++) {
                  ret.push(HEAPU8[((arg++)>>0)]);
                }
              } else {
                ret = ret.concat(intArrayFromString('(null)'.substr(0, argLength), true));
              }
              if (flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              break;
            }
            case 'c': {
              // Character.
              if (flagLeftAlign) ret.push(getNextArg('i8'));
              while (--width > 0) {
                ret.push(32);
              }
              if (!flagLeftAlign) ret.push(getNextArg('i8'));
              break;
            }
            case 'n': {
              // Write the length written so far to the next parameter.
              var ptr = getNextArg('i32*');
              HEAP32[((ptr)>>2)]=ret.length;
              break;
            }
            case '%': {
              // Literal percent sign.
              ret.push(curr);
              break;
            }
            default: {
              // Unknown specifiers remain untouched.
              for (var i = startTextIndex; i < textIndex + 2; i++) {
                ret.push(HEAP8[((i)>>0)]);
              }
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }


  
  function _open(path, oflag, varargs) {
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      var mode = HEAP32[((varargs)>>2)];
      path = Pointer_stringify(path);
      try {
        var stream = FS.open(path, oflag, mode);
        return stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fopen(filename, mode) {
      // FILE *fopen(const char *restrict filename, const char *restrict mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fopen.html
      var flags;
      mode = Pointer_stringify(mode);
      if (mode[0] == 'r') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 0;
        }
      } else if (mode[0] == 'w') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 512;
      } else if (mode[0] == 'a') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 1024;
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var fd = _open(filename, flags, allocate([0x1FF, 0, 0, 0], 'i32', ALLOC_STACK));  // All creation permissions.
      return fd === -1 ? 0 : FS.getPtrForStream(FS.getStream(fd));
    }


   
  Module["_i64Add"] = _i64Add;

  function _fabs() {
  return Math_abs.apply(null, arguments)
  }


  
  
  
  
  function _recv(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fread(ptr, size, nitems, stream) {
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) {
        return 0;
      }
      var bytesRead = 0;
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      while (streamObj.ungotten.length && bytesToRead > 0) {
        HEAP8[((ptr++)>>0)]=streamObj.ungotten.pop();
        bytesToRead--;
        bytesRead++;
      }
      var err = _read(streamObj.fd, ptr, bytesToRead);
      if (err == -1) {
        if (streamObj) streamObj.error = true;
        return 0;
      }
      bytesRead += err;
      if (bytesRead < bytesToRead) streamObj.eof = true;
      return Math.floor(bytesRead / size);
    }function _fgetc(stream) {
      // int fgetc(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetc.html
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) return -1;
      if (streamObj.eof || streamObj.error) return -1;
      var ret = _fread(_fgetc.ret, 1, 1, stream);
      if (ret == 0) {
        return -1;
      } else if (ret == -1) {
        streamObj.error = true;
        return -1;
      } else {
        return HEAPU8[((_fgetc.ret)>>0)];
      }
    }function _getc() {
  return _fgetc.apply(null, arguments)
  }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }

  
  function _fputc(c, stream) {
      // int fputc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputc.html
      var chr = unSign(c & 0xFF);
      HEAP8[((_fputc.ret)>>0)]=chr;
      var fd = _fileno(stream);
      var ret = _write(fd, _fputc.ret, 1);
      if (ret == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return -1;
      } else {
        return chr;
      }
    }function _putc() {
  return _fputc.apply(null, arguments)
  }

  function _time(ptr) {
      var ret = Math.floor(Date.now()/1000);
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  var Browser={mainLoop:{scheduler:null,method:"",shouldPause:false,paused:false,queue:[],pause:function () {
          Browser.mainLoop.shouldPause = true;
        },resume:function () {
          if (Browser.mainLoop.paused) {
            Browser.mainLoop.paused = false;
            Browser.mainLoop.scheduler();
          }
          Browser.mainLoop.shouldPause = false;
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          assert(typeof url == 'string', 'createObjectURL must return a url as a string');
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            assert(typeof url == 'string', 'createObjectURL must return a url as a string');
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx) return Module.ctx; // no need to recreate singleton GL context
  
        var ctx;
        var errorInfo = '?';
        function onContextCreationError(event) {
          errorInfo = event.statusMessage || errorInfo;
        }
        try {
          if (useWebGL) {
            var contextAttributes = {
              antialias: false,
              alpha: false
            };
  
            if (webGLContextAttributes) {
              for (var attribute in webGLContextAttributes) {
                contextAttributes[attribute] = webGLContextAttributes[attribute];
              }
            }
  
  
            canvas.addEventListener('webglcontextcreationerror', onContextCreationError, false);
            try {
              ['experimental-webgl', 'webgl'].some(function(webglId) {
                return ctx = canvas.getContext(webglId, contextAttributes);
              });
            } finally {
              canvas.removeEventListener('webglcontextcreationerror', onContextCreationError, false);
            }
          } else {
            ctx = canvas.getContext('2d');
          }
          if (!ctx) throw ':(';
        } catch (e) {
          Module.print('Could not create canvas: ' + [errorInfo, e]);
          return null;
        }
        if (useWebGL) {
          // possible GL_DEBUG entry point: ctx = wrapDebugGL(ctx);
  
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        }
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
          Module.ctx = ctx;
          if (useWebGL) GLctx = ctx;
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
        
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
        canvasContainer.requestFullScreen();
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = -event.wheelDelta;
            break;
          case 'wheel': 
            delta = event.deltaY;
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return Math.max(-1, Math.min(1, delta));
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
          // and we have no viable fallback.
          assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              Browser.lastTouches[touch.identifier] = Browser.touches[touch.identifier];
              Browser.touches[touch.identifier] = { x: adjustedX, y: adjustedY };
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      }};

  
  function _fmod(x, y) {
      return x % y;
    }function _fmodl() {
  return _fmod.apply(null, arguments)
  }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;


  var _BDtoILow=true;

  function _ferror(stream) {
      // int ferror(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ferror.html
      stream = FS.getStreamFromPtr(stream);
      return Number(stream && stream.error);
    }

  var _BDtoIHigh=true;

  
  
  
  
  var _environ=allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = 'root';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/emscripten';
        ENV['LANG'] = 'en_US.UTF-8';
        ENV['_'] = './this.program';
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        writeAsciiToMemory(line, poolPtr);
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }

  function ___errno_location() {
      return ___errno_state;
    }

  var _BItoD=true;

  
  function _copysign(a, b) {
      return __reallyNegative(a) === __reallyNegative(b) ? a : -a;
    }function _copysignl() {
  return _copysign.apply(null, arguments)
  }



___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });TTY.utf8 = new Runtime.UTF8Processor();
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
_fgetc.ret = allocate([0], "i8", ALLOC_STATIC);
_fputc.ret = allocate([0], "i8", ALLOC_STATIC);
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas) { Browser.requestFullScreen(lockPointer, resizeCanvas) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
___buildEnvironment(ENV);
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + 5242880;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var ctlz_i8 = allocate([8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_DYNAMIC);
 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);

  var Math_min = Math.min;
function nullFunc_iiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viiiiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'viiiiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'viiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_vi(x) { Module["printErr"]("Invalid function pointer called with signature 'vi'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_vii(x) { Module["printErr"]("Invalid function pointer called with signature 'vii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iiiiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_ii(x) { Module["printErr"]("Invalid function pointer called with signature 'ii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viii(x) { Module["printErr"]("Invalid function pointer called with signature 'viii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iii(x) { Module["printErr"]("Invalid function pointer called with signature 'iii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iiiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viiii(x) { Module["printErr"]("Invalid function pointer called with signature 'viiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiiiiii(index,a1,a2,a3,a4,a5,a6,a7) {
  try {
    Module["dynCall_viiiiiii"](index,a1,a2,a3,a4,a5,a6,a7);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiiii(index,a1,a2,a3,a4,a5) {
  try {
    Module["dynCall_viiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vii(index,a1,a2) {
  try {
    Module["dynCall_vii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    return Module["dynCall_iiiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viii(index,a1,a2,a3) {
  try {
    Module["dynCall_viii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiiii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiii(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

  function asmPrintInt(x, y) {
    Module.print('int ' + x + ',' + y);// + ' ' + new Error().stack);
  }
  function asmPrintFloat(x, y) {
    Module.print('float ' + x + ',' + y);// + ' ' + new Error().stack);
  }
  // EMSCRIPTEN_START_ASM
  var asm = (function(global, env, buffer) {
    'almost asm';
    var HEAP8 = new global.Int8Array(buffer);
    var HEAP16 = new global.Int16Array(buffer);
    var HEAP32 = new global.Int32Array(buffer);
    var HEAPU8 = new global.Uint8Array(buffer);
    var HEAPU16 = new global.Uint16Array(buffer);
    var HEAPU32 = new global.Uint32Array(buffer);
    var HEAPF32 = new global.Float32Array(buffer);
    var HEAPF64 = new global.Float64Array(buffer);
  
  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;
  var ctlz_i8=env.ctlz_i8|0;
  var _stderr=env._stderr|0;
  var _stdin=env._stdin|0;
  var _stdout=env._stdout|0;

    var __THREW__ = 0;
    var threwValue = 0;
    var setjmpId = 0;
    var undef = 0;
    var nan = +env.NaN, inf = +env.Infinity;
    var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;
  
    var tempRet0 = 0;
    var tempRet1 = 0;
    var tempRet2 = 0;
    var tempRet3 = 0;
    var tempRet4 = 0;
    var tempRet5 = 0;
    var tempRet6 = 0;
    var tempRet7 = 0;
    var tempRet8 = 0;
    var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var abort=env.abort;
  var assert=env.assert;
  var asmPrintInt=env.asmPrintInt;
  var asmPrintFloat=env.asmPrintFloat;
  var Math_min=env.min;
  var nullFunc_iiii=env.nullFunc_iiii;
  var nullFunc_viiiiiii=env.nullFunc_viiiiiii;
  var nullFunc_viiiii=env.nullFunc_viiiii;
  var nullFunc_vi=env.nullFunc_vi;
  var nullFunc_vii=env.nullFunc_vii;
  var nullFunc_iiiiiii=env.nullFunc_iiiiiii;
  var nullFunc_ii=env.nullFunc_ii;
  var nullFunc_viii=env.nullFunc_viii;
  var nullFunc_iiiii=env.nullFunc_iiiii;
  var nullFunc_iii=env.nullFunc_iii;
  var nullFunc_iiiiii=env.nullFunc_iiiiii;
  var nullFunc_viiii=env.nullFunc_viiii;
  var invoke_iiii=env.invoke_iiii;
  var invoke_viiiiiii=env.invoke_viiiiiii;
  var invoke_viiiii=env.invoke_viiiii;
  var invoke_vi=env.invoke_vi;
  var invoke_vii=env.invoke_vii;
  var invoke_iiiiiii=env.invoke_iiiiiii;
  var invoke_ii=env.invoke_ii;
  var invoke_viii=env.invoke_viii;
  var invoke_iiiii=env.invoke_iiiii;
  var invoke_iii=env.invoke_iii;
  var invoke_iiiiii=env.invoke_iiiiii;
  var invoke_viiii=env.invoke_viiii;
  var _fabs=env._fabs;
  var _putc=env._putc;
  var _send=env._send;
  var _fread=env._fread;
  var _fmod=env._fmod;
  var ___setErrNo=env.___setErrNo;
  var ___buildEnvironment=env.___buildEnvironment;
  var _fflush=env._fflush;
  var _pwrite=env._pwrite;
  var _strerror_r=env._strerror_r;
  var _open=env._open;
  var _sbrk=env._sbrk;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var _fileno=env._fileno;
  var _sysconf=env._sysconf;
  var _close=env._close;
  var _ferror=env._ferror;
  var _pread=env._pread;
  var _getenv=env._getenv;
  var _fclose=env._fclose;
  var __reallyNegative=env.__reallyNegative;
  var _getc=env._getc;
  var _write=env._write;
  var _fsync=env._fsync;
  var ___errno_location=env.___errno_location;
  var _recv=env._recv;
  var _copysign=env._copysign;
  var _fgetc=env._fgetc;
  var _fputc=env._fputc;
  var _mkport=env._mkport;
  var __exit=env.__exit;
  var _read=env._read;
  var _copysignl=env._copysignl;
  var _abort=env._abort;
  var _fwrite=env._fwrite;
  var _time=env._time;
  var _fprintf=env._fprintf;
  var _strerror=env._strerror;
  var __formatString=env.__formatString;
  var _fopen=env._fopen;
  var _exit=env._exit;
  var _fmodl=env._fmodl;
  var tempFloat = 0.0;

  // EMSCRIPTEN_START_FUNCS
  function stackAlloc(size) {
    size = size|0;
    var ret = 0;
    ret = STACKTOP;
    STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 7)&-8;
    return ret|0;
  }
  function stackSave() {
    return STACKTOP|0;
  }
  function stackRestore(top) {
    top = top|0;
    STACKTOP = top;
  }
  function setThrew(threw, value) {
    threw = threw|0;
    value = value|0;
    if ((__THREW__|0) == 0) {
      __THREW__ = threw;
      threwValue = value;
    }
  }
  function copyTempFloat(ptr) {
    ptr = ptr|0;
    HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
    HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
    HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
    HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  }
  function copyTempDouble(ptr) {
    ptr = ptr|0;
    HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
    HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
    HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
    HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
    HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
    HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
    HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
    HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
  }
  function setTempRet0(value) {
    value = value|0;
    tempRet0 = value;
  }
  function getTempRet0() {
    return tempRet0|0;
  }
  
function _main($argc,$argv) {
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $cinfo = 0, $dest_mgr = 0, $file_index = 0, $input_file = 0, $jerr = 0, $num_scanlines = 0, $output_file = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer5 = 0, $vararg_ptr4 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 656|0;
 $vararg_buffer5 = sp + 480|0;
 $vararg_buffer1 = sp + 472|0;
 $vararg_buffer = sp + 464|0;
 $cinfo = sp;
 $jerr = sp + 508|0;
 $0 = 0;
 $1 = $argc;
 $2 = $argv;
 $dest_mgr = 0;
 $3 = $2;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[8>>2] = $4;
 $5 = HEAP32[8>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  label = 3;
 } else {
  $7 = HEAP32[8>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)==(0);
  if ($10) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  HEAP32[8>>2] = 16;
 }
 $11 = (_jpeg_std_error($jerr)|0);
 HEAP32[$cinfo>>2] = $11;
 _jpeg_CreateDecompress($cinfo,62,464);
 $12 = (($jerr) + 120|0);
 HEAP32[$12>>2] = 24;
 $13 = (($jerr) + 124|0);
 HEAP32[$13>>2] = 1000;
 $14 = (($jerr) + 128|0);
 HEAP32[$14>>2] = 1043;
 _jpeg_set_marker_processor($cinfo,254,2);
 _jpeg_set_marker_processor($cinfo,236,2);
 $15 = $1;
 $16 = $2;
 $17 = (_parse_switches($cinfo,$15,$16,0,0)|0);
 $file_index = $17;
 $18 = $file_index;
 $19 = $1;
 $20 = (($19) - 1)|0;
 $21 = ($18|0)<($20|0);
 if ($21) {
  $22 = HEAP32[_stderr>>2]|0;
  $23 = HEAP32[8>>2]|0;
  HEAP32[$vararg_buffer>>2] = $23;
  (_fprintf(($22|0),(200|0),($vararg_buffer|0))|0);
  _usage();
 }
 $24 = $file_index;
 $25 = $1;
 $26 = ($24|0)<($25|0);
 do {
  if ($26) {
   $27 = $file_index;
   $28 = $2;
   $29 = (($28) + ($27<<2)|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (_fopen(($30|0),(232|0))|0);
   $input_file = $31;
   $32 = ($31|0)==(0|0);
   if ($32) {
    $33 = HEAP32[_stderr>>2]|0;
    $34 = HEAP32[8>>2]|0;
    $35 = $file_index;
    $36 = $2;
    $37 = (($36) + ($35<<2)|0);
    $38 = HEAP32[$37>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $34;
    $vararg_ptr4 = (($vararg_buffer1) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $38;
    (_fprintf(($33|0),(240|0),($vararg_buffer1|0))|0);
    _exit(1);
    // unreachable;
   } else {
    break;
   }
  } else {
   $39 = (_read_stdin()|0);
   $input_file = $39;
  }
 } while(0);
 $40 = HEAP32[264>>2]|0;
 $41 = ($40|0)!=(0|0);
 do {
  if ($41) {
   $42 = HEAP32[264>>2]|0;
   $43 = (_fopen(($42|0),(272|0))|0);
   $output_file = $43;
   $44 = ($43|0)==(0|0);
   if ($44) {
    $45 = HEAP32[_stderr>>2]|0;
    $46 = HEAP32[8>>2]|0;
    $47 = HEAP32[264>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $46;
    $vararg_ptr8 = (($vararg_buffer5) + 4|0);
    HEAP32[$vararg_ptr8>>2] = $47;
    (_fprintf(($45|0),(240|0),($vararg_buffer5|0))|0);
    _exit(1);
    // unreachable;
   } else {
    break;
   }
  } else {
   $48 = (_write_stdout()|0);
   $output_file = $48;
  }
 } while(0);
 $49 = $input_file;
 _jpeg_stdio_src($cinfo,$49);
 (_jpeg_read_header($cinfo,1)|0);
 $50 = $1;
 $51 = $2;
 $52 = (_parse_switches($cinfo,$50,$51,0,1)|0);
 $file_index = $52;
 $53 = HEAP32[280>>2]|0;
 switch ($53|0) {
 case 0:  {
  $54 = (_jinit_write_bmp($cinfo,0)|0);
  $dest_mgr = $54;
  break;
 }
 case 2:  {
  $55 = (_jinit_write_bmp($cinfo,1)|0);
  $dest_mgr = $55;
  break;
 }
 case 1:  {
  $56 = (_jinit_write_gif($cinfo)|0);
  $dest_mgr = $56;
  break;
 }
 case 3:  {
  $57 = (_jinit_write_ppm($cinfo)|0);
  $dest_mgr = $57;
  break;
 }
 case 5:  {
  $58 = (_jinit_write_targa($cinfo)|0);
  $dest_mgr = $58;
  break;
 }
 default: {
  $59 = HEAP32[$cinfo>>2]|0;
  $60 = (($59) + 20|0);
  HEAP32[$60>>2] = 1042;
  $61 = HEAP32[$cinfo>>2]|0;
  $62 = HEAP32[$61>>2]|0;
  FUNCTION_TABLE_vi[$62 & 127]($cinfo);
 }
 }
 $63 = $output_file;
 $64 = $dest_mgr;
 $65 = (($64) + 12|0);
 HEAP32[$65>>2] = $63;
 (_jpeg_start_decompress($cinfo)|0);
 $66 = $dest_mgr;
 $67 = HEAP32[$66>>2]|0;
 $68 = $dest_mgr;
 FUNCTION_TABLE_vii[$67 & 127]($cinfo,$68);
 while(1) {
  $69 = (($cinfo) + 140|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($cinfo) + 116|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($70>>>0)<($72>>>0);
  if (!($73)) {
   break;
  }
  $74 = $dest_mgr;
  $75 = (($74) + 16|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $dest_mgr;
  $78 = (($77) + 20|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (_jpeg_read_scanlines($cinfo,$76,$79)|0);
  $num_scanlines = $80;
  $81 = $dest_mgr;
  $82 = (($81) + 4|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $dest_mgr;
  $85 = $num_scanlines;
  FUNCTION_TABLE_viii[$83 & 31]($cinfo,$84,$85);
 }
 $86 = $dest_mgr;
 $87 = (($86) + 8|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $dest_mgr;
 FUNCTION_TABLE_vii[$88 & 127]($cinfo,$89);
 (_jpeg_finish_decompress($cinfo)|0);
 _jpeg_destroy_decompress($cinfo);
 $90 = $input_file;
 $91 = HEAP32[_stdin>>2]|0;
 $92 = ($90|0)!=($91|0);
 if ($92) {
  $93 = $input_file;
  (_fclose(($93|0))|0);
 }
 $94 = $output_file;
 $95 = HEAP32[_stdout>>2]|0;
 $96 = ($94|0)!=($95|0);
 if (!($96)) {
  $98 = (($jerr) + 108|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($99|0)!=(0);
  $101 = $100 ? 2 : 0;
  _exit(($101|0));
  // unreachable;
 }
 $97 = $output_file;
 (_fclose(($97|0))|0);
 $98 = (($jerr) + 108|0);
 $99 = HEAP32[$98>>2]|0;
 $100 = ($99|0)!=(0);
 $101 = $100 ? 2 : 0;
 _exit(($101|0));
 // unreachable;
 return 0|0;
}
function _print_text_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0;
 var $lastch = 0, $length = 0, $traceit = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer14 = sp + 48|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 104|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(1);
 $6 = $5&1;
 $traceit = $6;
 $lastch = 0;
 $7 = $0;
 $8 = (_jpeg_getc($7)|0);
 $9 = $8 << 8;
 $length = $9;
 $10 = $0;
 $11 = (_jpeg_getc($10)|0);
 $12 = $length;
 $13 = (($12) + ($11))|0;
 $length = $13;
 $14 = $length;
 $15 = (($14) - 2)|0;
 $length = $15;
 $16 = $traceit;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = $0;
  $19 = (($18) + 416|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(254);
  if ($21) {
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $length;
   HEAP32[$vararg_buffer>>2] = $23;
   (_fprintf(($22|0),(2016|0),($vararg_buffer|0))|0);
  } else {
   $24 = HEAP32[_stderr>>2]|0;
   $25 = $0;
   $26 = (($25) + 416|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) - 224)|0;
   $29 = $length;
   HEAP32[$vararg_buffer1>>2] = $28;
   $vararg_ptr4 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $29;
   (_fprintf(($24|0),(2040|0),($vararg_buffer1|0))|0);
  }
 }
 while(1) {
  $30 = $length;
  $31 = (($30) + -1)|0;
  $length = $31;
  $32 = ($31|0)>=(0);
  if (!($32)) {
   break;
  }
  $33 = $0;
  $34 = (_jpeg_getc($33)|0);
  $ch = $34;
  $35 = $traceit;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = $ch;
   $38 = ($37|0)==(13);
   if ($38) {
    $39 = HEAP32[_stderr>>2]|0;
    (_fprintf(($39|0),(2064|0),($vararg_buffer5|0))|0);
   } else {
    $40 = $ch;
    $41 = ($40|0)==(10);
    if ($41) {
     $42 = $lastch;
     $43 = ($42|0)!=(13);
     if ($43) {
      $44 = HEAP32[_stderr>>2]|0;
      (_fprintf(($44|0),(2064|0),($vararg_buffer7|0))|0);
     }
    } else {
     $45 = $ch;
     $46 = ($45|0)==(92);
     if ($46) {
      $47 = HEAP32[_stderr>>2]|0;
      (_fprintf(($47|0),(2072|0),($vararg_buffer9|0))|0);
     } else {
      $48 = $ch;
      $49 = (_isprint($48)|0);
      $50 = ($49|0)!=(0);
      if ($50) {
       $51 = $ch;
       $52 = HEAP32[_stderr>>2]|0;
       (_putc(($51|0),($52|0))|0);
      } else {
       $53 = HEAP32[_stderr>>2]|0;
       $54 = $ch;
       HEAP32[$vararg_buffer11>>2] = $54;
       (_fprintf(($53|0),(2080|0),($vararg_buffer11|0))|0);
      }
     }
    }
   }
   $55 = $ch;
   $lastch = $55;
  }
 }
 $56 = $traceit;
 $57 = ($56|0)!=(0);
 if (!($57)) {
  STACKTOP = sp;return 1;
 }
 $58 = HEAP32[_stderr>>2]|0;
 (_fprintf(($58|0),(2064|0),($vararg_buffer14|0))|0);
 STACKTOP = sp;return 1;
}
function _parse_switches($cinfo,$argc,$argv,$last_file_arg_seen,$for_real) {
 $cinfo = $cinfo|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $last_file_arg_seen = $last_file_arg_seen|0;
 $for_real = $for_real|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $arg = 0, $argn = 0, $ch = 0, $lval = 0, $mapfile = 0, $val = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer13 = 0, $vararg_buffer5 = 0, $vararg_buffer9 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0;
 var $vararg_ptr4 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer13 = sp + 32|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $val = sp + 48|0;
 $lval = sp + 40|0;
 $ch = sp + 80|0;
 $0 = $cinfo;
 $1 = $argc;
 $2 = $argv;
 $3 = $last_file_arg_seen;
 $4 = $for_real;
 HEAP32[280>>2] = 3;
 HEAP32[264>>2] = 0;
 $5 = $0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 104|0);
 HEAP32[$7>>2] = 0;
 $argn = 1;
 L1: while(1) {
  $8 = $argn;
  $9 = $1;
  $10 = ($8|0)<($9|0);
  if (!($10)) {
   label = 118;
   break;
  }
  $11 = $argn;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $arg = $14;
  $15 = $arg;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)!=(45);
  if ($18) {
   $19 = $argn;
   $20 = $3;
   $21 = ($19|0)<=($20|0);
   if (!($21)) {
    label = 6;
    break;
   }
   HEAP32[264>>2] = 0;
  } else {
   $22 = $arg;
   $23 = (($22) + 1|0);
   $arg = $23;
   $24 = $arg;
   $25 = (_keymatch($24,1600,1)|0);
   $26 = ($25|0)!=(0);
   if ($26) {
    HEAP32[280>>2] = 0;
   } else {
    $27 = $arg;
    $28 = (_keymatch($27,1608,1)|0);
    $29 = ($28|0)!=(0);
    if ($29) {
     label = 13;
    } else {
     $30 = $arg;
     $31 = (_keymatch($30,1616,1)|0);
     $32 = ($31|0)!=(0);
     if ($32) {
      label = 13;
     } else {
      $33 = $arg;
      $34 = (_keymatch($33,1624,1)|0);
      $35 = ($34|0)!=(0);
      if ($35) {
       label = 13;
      } else {
       $36 = $arg;
       $37 = (_keymatch($36,1640,1)|0);
       $38 = ($37|0)!=(0);
       if ($38) {
        label = 13;
       } else {
        $54 = $arg;
        $55 = (_keymatch($54,1664,2)|0);
        $56 = ($55|0)!=(0);
        if ($56) {
         $57 = $argn;
         $58 = (($57) + 1)|0;
         $argn = $58;
         $59 = $1;
         $60 = ($58|0)>=($59|0);
         if ($60) {
          _usage();
         }
         $61 = $argn;
         $62 = $2;
         $63 = (($62) + ($61<<2)|0);
         $64 = HEAP32[$63>>2]|0;
         $65 = (_keymatch($64,1672,1)|0);
         $66 = ($65|0)!=(0);
         if ($66) {
          $67 = $0;
          $68 = (($67) + 72|0);
          HEAP32[$68>>2] = 0;
         } else {
          $69 = $argn;
          $70 = $2;
          $71 = (($70) + ($69<<2)|0);
          $72 = HEAP32[$71>>2]|0;
          $73 = (_keymatch($72,1680,2)|0);
          $74 = ($73|0)!=(0);
          if ($74) {
           $75 = $0;
           $76 = (($75) + 72|0);
           HEAP32[$76>>2] = 1;
          } else {
           $77 = $argn;
           $78 = $2;
           $79 = (($78) + ($77<<2)|0);
           $80 = HEAP32[$79>>2]|0;
           $81 = (_keymatch($80,1688,2)|0);
           $82 = ($81|0)!=(0);
           if ($82) {
            $83 = $0;
            $84 = (($83) + 72|0);
            HEAP32[$84>>2] = 2;
           } else {
            _usage();
           }
          }
         }
        } else {
         $85 = $arg;
         $86 = (_keymatch($85,1696,2)|0);
         $87 = ($86|0)!=(0);
         if ($87) {
          $88 = $argn;
          $89 = (($88) + 1)|0;
          $argn = $89;
          $90 = $1;
          $91 = ($89|0)>=($90|0);
          if ($91) {
           _usage();
          }
          $92 = $argn;
          $93 = $2;
          $94 = (($93) + ($92<<2)|0);
          $95 = HEAP32[$94>>2]|0;
          $96 = (_keymatch($95,1704,2)|0);
          $97 = ($96|0)!=(0);
          if ($97) {
           $98 = $0;
           $99 = (($98) + 88|0);
           HEAP32[$99>>2] = 2;
          } else {
           $100 = $argn;
           $101 = $2;
           $102 = (($101) + ($100<<2)|0);
           $103 = HEAP32[$102>>2]|0;
           $104 = (_keymatch($103,1712,2)|0);
           $105 = ($104|0)!=(0);
           if ($105) {
            $106 = $0;
            $107 = (($106) + 88|0);
            HEAP32[$107>>2] = 0;
           } else {
            $108 = $argn;
            $109 = $2;
            $110 = (($109) + ($108<<2)|0);
            $111 = HEAP32[$110>>2]|0;
            $112 = (_keymatch($111,1720,2)|0);
            $113 = ($112|0)!=(0);
            if ($113) {
             $114 = $0;
             $115 = (($114) + 88|0);
             HEAP32[$115>>2] = 1;
            } else {
             _usage();
            }
           }
          }
         } else {
          $116 = $arg;
          $117 = (_keymatch($116,1728,1)|0);
          $118 = ($117|0)!=(0);
          if ($118) {
           label = 46;
          } else {
           $119 = $arg;
           $120 = (_keymatch($119,1736,1)|0);
           $121 = ($120|0)!=(0);
           if ($121) {
            label = 46;
           } else {
            $130 = $arg;
            $131 = (_keymatch($130,1680,1)|0);
            $132 = ($131|0)!=(0);
            if ($132) {
             $133 = $0;
             $134 = (($133) + 92|0);
             HEAP32[$134>>2] = 0;
             $135 = $0;
             $136 = (($135) + 88|0);
             HEAP32[$136>>2] = 1;
             $137 = $0;
             $138 = (($137) + 84|0);
             $139 = HEAP32[$138>>2]|0;
             $140 = ($139|0)!=(0);
             if (!($140)) {
              $141 = $0;
              $142 = (($141) + 96|0);
              HEAP32[$142>>2] = 216;
             }
             $143 = $0;
             $144 = (($143) + 72|0);
             HEAP32[$144>>2] = 1;
             $145 = $0;
             $146 = (($145) + 76|0);
             HEAP32[$146>>2] = 0;
            } else {
             $147 = $arg;
             $148 = (_keymatch($147,1856,1)|0);
             $149 = ($148|0)!=(0);
             if ($149) {
              HEAP32[280>>2] = 1;
             } else {
              $150 = $arg;
              $151 = (_keymatch($150,1864,2)|0);
              $152 = ($151|0)!=(0);
              do {
               if ($152) {
                label = 57;
               } else {
                $153 = $arg;
                $154 = (_keymatch($153,1880,2)|0);
                $155 = ($154|0)!=(0);
                if ($155) {
                 label = 57;
                 break;
                }
                $158 = $arg;
                $159 = (_keymatch($158,1896,3)|0);
                $160 = ($159|0)!=(0);
                if ($160) {
                 $161 = $argn;
                 $162 = (($161) + 1)|0;
                 $argn = $162;
                 $163 = $1;
                 $164 = ($162|0)>=($163|0);
                 if ($164) {
                  _usage();
                 }
                 $165 = $4;
                 $166 = ($165|0)!=(0);
                 if ($166) {
                  $167 = $argn;
                  $168 = $2;
                  $169 = (($168) + ($167<<2)|0);
                  $170 = HEAP32[$169>>2]|0;
                  $171 = (_fopen(($170|0),(232|0))|0);
                  $mapfile = $171;
                  $172 = ($171|0)==(0|0);
                  if ($172) {
                   label = 63;
                   break L1;
                  }
                  $179 = $0;
                  $180 = $mapfile;
                  _read_color_map($179,$180);
                  $181 = $mapfile;
                  (_fclose(($181|0))|0);
                  $182 = $0;
                  $183 = (($182) + 84|0);
                  HEAP32[$183>>2] = 1;
                 }
                } else {
                 $184 = $arg;
                 $185 = (_keymatch($184,1904,3)|0);
                 $186 = ($185|0)!=(0);
                 if ($186) {
                  HEAP8[$ch>>0] = 120;
                  $187 = $argn;
                  $188 = (($187) + 1)|0;
                  $argn = $188;
                  $189 = $1;
                  $190 = ($188|0)>=($189|0);
                  if ($190) {
                   _usage();
                  }
                  $191 = $argn;
                  $192 = $2;
                  $193 = (($192) + ($191<<2)|0);
                  $194 = HEAP32[$193>>2]|0;
                  HEAP32[$vararg_buffer9>>2] = $lval;
                  $vararg_ptr12 = (($vararg_buffer9) + 4|0);
                  HEAP32[$vararg_ptr12>>2] = $ch;
                  $195 = (_sscanf($194,1920,$vararg_buffer9)|0);
                  $196 = ($195|0)<(1);
                  if ($196) {
                   _usage();
                  }
                  $197 = HEAP8[$ch>>0]|0;
                  $198 = $197 << 24 >> 24;
                  $199 = ($198|0)==(109);
                  if ($199) {
                   label = 73;
                  } else {
                   $200 = HEAP8[$ch>>0]|0;
                   $201 = $200 << 24 >> 24;
                   $202 = ($201|0)==(77);
                   if ($202) {
                    label = 73;
                   }
                  }
                  if ((label|0) == 73) {
                   label = 0;
                   $203 = HEAP32[$lval>>2]|0;
                   $204 = ($203*1000)|0;
                   HEAP32[$lval>>2] = $204;
                  }
                  $205 = HEAP32[$lval>>2]|0;
                  $206 = ($205*1000)|0;
                  $207 = $0;
                  $208 = (($207) + 4|0);
                  $209 = HEAP32[$208>>2]|0;
                  $210 = (($209) + 44|0);
                  HEAP32[$210>>2] = $206;
                 } else {
                  $211 = $arg;
                  $212 = (_keymatch($211,1928,3)|0);
                  $213 = ($212|0)!=(0);
                  if ($213) {
                   $214 = $0;
                   $215 = (($214) + 76|0);
                   HEAP32[$215>>2] = 0;
                  } else {
                   $216 = $arg;
                   $217 = (_keymatch($216,1944,3)|0);
                   $218 = ($217|0)!=(0);
                   if ($218) {
                    $219 = $0;
                    $220 = (($219) + 92|0);
                    HEAP32[$220>>2] = 0;
                   } else {
                    $221 = $arg;
                    $222 = (_keymatch($221,1952,3)|0);
                    $223 = ($222|0)!=(0);
                    if ($223) {
                     HEAP32[280>>2] = 2;
                    } else {
                     $224 = $arg;
                     $225 = (_keymatch($224,1960,4)|0);
                     $226 = ($225|0)!=(0);
                     if ($226) {
                      $227 = $argn;
                      $228 = (($227) + 1)|0;
                      $argn = $228;
                      $229 = $1;
                      $230 = ($228|0)>=($229|0);
                      if ($230) {
                       _usage();
                      }
                      $231 = $argn;
                      $232 = $2;
                      $233 = (($232) + ($231<<2)|0);
                      $234 = HEAP32[$233>>2]|0;
                      HEAP32[264>>2] = $234;
                     } else {
                      $235 = $arg;
                      $236 = (_keymatch($235,1968,1)|0);
                      $237 = ($236|0)!=(0);
                      do {
                       if ($237) {
                        label = 87;
                       } else {
                        $238 = $arg;
                        $239 = (_keymatch($238,1976,1)|0);
                        $240 = ($239|0)!=(0);
                        if ($240) {
                         label = 87;
                         break;
                        }
                        $241 = $arg;
                        $242 = (_keymatch($241,1984,1)|0);
                        $243 = ($242|0)!=(0);
                        if ($243) {
                         HEAP32[280>>2] = 4;
                        } else {
                         $244 = $arg;
                         $245 = (_keymatch($244,1992,1)|0);
                         $246 = ($245|0)!=(0);
                         if ($246) {
                          $247 = $argn;
                          $248 = (($247) + 1)|0;
                          $argn = $248;
                          $249 = $1;
                          $250 = ($248|0)>=($249|0);
                          if ($250) {
                           _usage();
                          }
                          $251 = $argn;
                          $252 = $2;
                          $253 = (($252) + ($251<<2)|0);
                          $254 = HEAP32[$253>>2]|0;
                          $255 = $0;
                          $256 = (($255) + 48|0);
                          $257 = $0;
                          $258 = (($257) + 52|0);
                          HEAP32[$vararg_buffer13>>2] = $256;
                          $vararg_ptr16 = (($vararg_buffer13) + 4|0);
                          HEAP32[$vararg_ptr16>>2] = $258;
                          $259 = (_sscanf($254,2000,$vararg_buffer13)|0);
                          $260 = ($259|0)!=(2);
                          if ($260) {
                           _usage();
                          }
                         } else {
                          $261 = $arg;
                          $262 = (_keymatch($261,2008,1)|0);
                          $263 = ($262|0)!=(0);
                          if ($263) {
                           HEAP32[280>>2] = 5;
                          } else {
                           _usage();
                          }
                         }
                        }
                       }
                      } while(0);
                      if ((label|0) == 87) {
                       label = 0;
                       HEAP32[280>>2] = 3;
                      }
                     }
                    }
                   }
                  }
                 }
                }
               }
              } while(0);
              if ((label|0) == 57) {
               label = 0;
               $156 = $0;
               $157 = (($156) + 44|0);
               HEAP32[$157>>2] = 1;
              }
             }
            }
           }
          }
          if ((label|0) == 46) {
           label = 0;
           $122 = HEAP32[1744>>2]|0;
           $123 = ($122|0)!=(0);
           if (!($123)) {
            $124 = HEAP32[_stderr>>2]|0;
            HEAP32[$vararg_buffer1>>2] = 1800;
            $vararg_ptr4 = (($vararg_buffer1) + 4|0);
            HEAP32[$vararg_ptr4>>2] = 1816;
            (_fprintf(($124|0),(1752|0),($vararg_buffer1|0))|0);
            HEAP32[1744>>2] = 1;
           }
           $125 = $0;
           $126 = HEAP32[$125>>2]|0;
           $127 = (($126) + 104|0);
           $128 = HEAP32[$127>>2]|0;
           $129 = (($128) + 1)|0;
           HEAP32[$127>>2] = $129;
          }
         }
        }
       }
      }
     }
    }
    if ((label|0) == 13) {
     label = 0;
     $39 = $argn;
     $40 = (($39) + 1)|0;
     $argn = $40;
     $41 = $1;
     $42 = ($40|0)>=($41|0);
     if ($42) {
      _usage();
     }
     $43 = $argn;
     $44 = $2;
     $45 = (($44) + ($43<<2)|0);
     $46 = HEAP32[$45>>2]|0;
     HEAP32[$vararg_buffer>>2] = $val;
     $47 = (_sscanf($46,1656,$vararg_buffer)|0);
     $48 = ($47|0)!=(1);
     if ($48) {
      _usage();
     }
     $49 = HEAP32[$val>>2]|0;
     $50 = $0;
     $51 = (($50) + 96|0);
     HEAP32[$51>>2] = $49;
     $52 = $0;
     $53 = (($52) + 84|0);
     HEAP32[$53>>2] = 1;
    }
   }
  }
  $264 = $argn;
  $265 = (($264) + 1)|0;
  $argn = $265;
 }
 if ((label|0) == 6) {
  $266 = $argn;
  STACKTOP = sp;return ($266|0);
 }
 else if ((label|0) == 63) {
  $173 = HEAP32[_stderr>>2]|0;
  $174 = HEAP32[8>>2]|0;
  $175 = $argn;
  $176 = $2;
  $177 = (($176) + ($175<<2)|0);
  $178 = HEAP32[$177>>2]|0;
  HEAP32[$vararg_buffer5>>2] = $174;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $178;
  (_fprintf(($173|0),(240|0),($vararg_buffer5|0))|0);
  _exit(1);
  // unreachable;
 }
 else if ((label|0) == 118) {
  $266 = $argn;
  STACKTOP = sp;return ($266|0);
 }
 return 0|0;
}
function _usage() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer30 = 0, $vararg_buffer33 = 0, $vararg_buffer36 = 0, $vararg_buffer39 = 0;
 var $vararg_buffer41 = 0, $vararg_buffer43 = 0, $vararg_buffer45 = 0, $vararg_buffer47 = 0, $vararg_buffer49 = 0, $vararg_buffer5 = 0, $vararg_buffer51 = 0, $vararg_buffer53 = 0, $vararg_buffer55 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer55 = sp + 192|0;
 $vararg_buffer53 = sp + 184|0;
 $vararg_buffer51 = sp + 176|0;
 $vararg_buffer49 = sp + 168|0;
 $vararg_buffer47 = sp + 160|0;
 $vararg_buffer45 = sp + 152|0;
 $vararg_buffer43 = sp + 144|0;
 $vararg_buffer41 = sp + 136|0;
 $vararg_buffer39 = sp + 128|0;
 $vararg_buffer36 = sp + 120|0;
 $vararg_buffer33 = sp + 112|0;
 $vararg_buffer30 = sp + 104|0;
 $vararg_buffer28 = sp + 96|0;
 $vararg_buffer25 = sp + 88|0;
 $vararg_buffer22 = sp + 80|0;
 $vararg_buffer19 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer13 = sp + 56|0;
 $vararg_buffer11 = sp + 48|0;
 $vararg_buffer9 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = HEAP32[_stderr>>2]|0;
 $1 = HEAP32[8>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 (_fprintf(($0|0),(288|0),($vararg_buffer|0))|0);
 $2 = HEAP32[_stderr>>2]|0;
 (_fprintf(($2|0),(312|0),($vararg_buffer1|0))|0);
 $3 = HEAP32[_stderr>>2]|0;
 (_fprintf(($3|0),(328|0),($vararg_buffer3|0))|0);
 $4 = HEAP32[_stderr>>2]|0;
 (_fprintf(($4|0),(368|0),($vararg_buffer5|0))|0);
 $5 = HEAP32[_stderr>>2]|0;
 (_fprintf(($5|0),(424|0),($vararg_buffer7|0))|0);
 $6 = HEAP32[_stderr>>2]|0;
 (_fprintf(($6|0),(472|0),($vararg_buffer9|0))|0);
 $7 = HEAP32[_stderr>>2]|0;
 (_fprintf(($7|0),(520|0),($vararg_buffer11|0))|0);
 $8 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer13>>2] = 648;
 (_fprintf(($8|0),(584|0),($vararg_buffer13|0))|0);
 $9 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer16>>2] = 648;
 (_fprintf(($9|0),(656|0),($vararg_buffer16|0))|0);
 $10 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer19>>2] = 648;
 (_fprintf(($10|0),(704|0),($vararg_buffer19|0))|0);
 $11 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer22>>2] = 832;
 (_fprintf(($11|0),(768|0),($vararg_buffer22|0))|0);
 $12 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer25>>2] = 648;
 (_fprintf(($12|0),(848|0),($vararg_buffer25|0))|0);
 $13 = HEAP32[_stderr>>2]|0;
 (_fprintf(($13|0),(896|0),($vararg_buffer28|0))|0);
 $14 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer30>>2] = 832;
 (_fprintf(($14|0),(928|0),($vararg_buffer30|0))|0);
 $15 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer33>>2] = 648;
 (_fprintf(($15|0),(976|0),($vararg_buffer33|0))|0);
 $16 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer36>>2] = 648;
 (_fprintf(($16|0),(1040|0),($vararg_buffer36|0))|0);
 $17 = HEAP32[_stderr>>2]|0;
 (_fprintf(($17|0),(1096|0),($vararg_buffer39|0))|0);
 $18 = HEAP32[_stderr>>2]|0;
 (_fprintf(($18|0),(1144|0),($vararg_buffer41|0))|0);
 $19 = HEAP32[_stderr>>2]|0;
 (_fprintf(($19|0),(1200|0),($vararg_buffer43|0))|0);
 $20 = HEAP32[_stderr>>2]|0;
 (_fprintf(($20|0),(1264|0),($vararg_buffer45|0))|0);
 $21 = HEAP32[_stderr>>2]|0;
 (_fprintf(($21|0),(1328|0),($vararg_buffer47|0))|0);
 $22 = HEAP32[_stderr>>2]|0;
 (_fprintf(($22|0),(1384|0),($vararg_buffer49|0))|0);
 $23 = HEAP32[_stderr>>2]|0;
 (_fprintf(($23|0),(1448|0),($vararg_buffer51|0))|0);
 $24 = HEAP32[_stderr>>2]|0;
 (_fprintf(($24|0),(1504|0),($vararg_buffer53|0))|0);
 $25 = HEAP32[_stderr>>2]|0;
 (_fprintf(($25|0),(1552|0),($vararg_buffer55|0))|0);
 _exit(1);
 // unreachable;
}
function _jpeg_getc($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $datasrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $datasrc = $3;
 $4 = $datasrc;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = $datasrc;
  $9 = (($8) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = (FUNCTION_TABLE_ii[$10 & 127]($11)|0);
  $13 = ($12|0)!=(0);
  if (!($13)) {
   $14 = $0;
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + 20|0);
   HEAP32[$16>>2] = 24;
   $17 = $0;
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = $0;
   FUNCTION_TABLE_vi[$19 & 127]($20);
  }
 }
 $21 = $datasrc;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + -1)|0;
 HEAP32[$22>>2] = $24;
 $25 = $datasrc;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + 1|0);
 HEAP32[$25>>2] = $27;
 $28 = HEAP8[$26>>0]|0;
 $29 = $28&255;
 STACKTOP = sp;return ($29|0);
}
function _jinit_write_ppm($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $dest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,40)|0);
 $dest = $6;
 $7 = $dest;
 HEAP32[$7>>2] = 3;
 $8 = $dest;
 $9 = (($8) + 8|0);
 HEAP32[$9>>2] = 4;
 $10 = $0;
 _jpeg_calc_output_dimensions($10);
 $11 = $0;
 $12 = (($11) + 112|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $0;
 $15 = (($14) + 120|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = Math_imul($13, $16)|0;
 $18 = $dest;
 $19 = (($18) + 36|0);
 HEAP32[$19>>2] = $17;
 $20 = $dest;
 $21 = (($20) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $22;
 $24 = $dest;
 $25 = (($24) + 32|0);
 HEAP32[$25>>2] = $23;
 $26 = $0;
 $27 = (($26) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = $dest;
 $32 = (($31) + 32|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (FUNCTION_TABLE_iiii[$29 & 127]($30,1,$33)|0);
 $35 = $dest;
 $36 = (($35) + 24|0);
 HEAP32[$36>>2] = $34;
 $37 = $0;
 $38 = (($37) + 84|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = ($39|0)!=(0);
 if (!($40)) {
  $73 = $dest;
  $74 = (($73) + 24|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = $dest;
  $77 = (($76) + 28|0);
  HEAP32[$77>>2] = $75;
  $78 = $dest;
  $79 = (($78) + 28|0);
  $80 = $dest;
  $81 = (($80) + 16|0);
  HEAP32[$81>>2] = $79;
  $82 = $dest;
  $83 = (($82) + 20|0);
  HEAP32[$83>>2] = 1;
  $84 = $dest;
  $85 = (($84) + 4|0);
  HEAP32[$85>>2] = 8;
  $86 = $dest;
  STACKTOP = sp;return ($86|0);
 }
 $41 = $0;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = $0;
 $47 = $0;
 $48 = (($47) + 112|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $0;
 $51 = (($50) + 124|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = Math_imul($49, $52)|0;
 $54 = (FUNCTION_TABLE_iiiii[$45 & 127]($46,1,$53,1)|0);
 $55 = $dest;
 $56 = (($55) + 16|0);
 HEAP32[$56>>2] = $54;
 $57 = $dest;
 $58 = (($57) + 20|0);
 HEAP32[$58>>2] = 1;
 $59 = $0;
 $60 = (($59) + 84|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = ($61|0)!=(0);
 if ($62) {
  $65 = $0;
  $66 = (($65) + 44|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)==(1);
  if ($68) {
   $69 = $dest;
   $70 = (($69) + 4|0);
   HEAP32[$70>>2] = 6;
  } else {
   $71 = $dest;
   $72 = (($71) + 4|0);
   HEAP32[$72>>2] = 7;
  }
 } else {
  $63 = $dest;
  $64 = (($63) + 4|0);
  HEAP32[$64>>2] = 5;
 }
 $86 = $dest;
 STACKTOP = sp;return ($86|0);
}
function _start_output_ppm($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dest = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $dest = $2;
 $3 = $0;
 $4 = (($3) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 if ((($5|0) == 2)) {
  $15 = $dest;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  $19 = (($18) + 112|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  $22 = (($21) + 116|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $20;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $23;
  $vararg_ptr7 = (($vararg_buffer3) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 255;
  (_fprintf(($17|0),(3568|0),($vararg_buffer3|0))|0);
  STACKTOP = sp;return;
 } else if ((($5|0) == 1)) {
  $6 = $dest;
  $7 = (($6) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $0;
  $10 = (($9) + 112|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $0;
  $13 = (($12) + 116|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$vararg_buffer>>2] = $11;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $14;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 255;
  (_fprintf(($8|0),(3552|0),($vararg_buffer|0))|0);
  STACKTOP = sp;return;
 } else {
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 20|0);
  HEAP32[$26>>2] = 1025;
  $27 = $0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = $0;
  FUNCTION_TABLE_vi[$29 & 127]($30);
  STACKTOP = sp;return;
 }
}
function _finish_output_ppm($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 (_fflush(($4|0))|0);
 $5 = $1;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_ferror(($7|0))|0);
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 20|0);
 HEAP32[$12>>2] = 37;
 $13 = $0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 FUNCTION_TABLE_vi[$15 & 127]($16);
 STACKTOP = sp;return;
}
function _copy_pixel_rows($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bufferptr = 0, $col = 0, $dest = 0, $ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $dest;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $ptr = $7;
 $8 = $dest;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $bufferptr = $10;
 $11 = $dest;
 $12 = (($11) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $col = $13;
 while(1) {
  $14 = $col;
  $15 = ($14>>>0)>(0);
  if (!($15)) {
   break;
  }
  $16 = $ptr;
  $17 = (($16) + 1|0);
  $ptr = $17;
  $18 = HEAP8[$16>>0]|0;
  $19 = $18&255;
  $20 = $19&255;
  $21 = $bufferptr;
  $22 = (($21) + 1|0);
  $bufferptr = $22;
  HEAP8[$21>>0] = $20;
  $23 = $col;
  $24 = (($23) + -1)|0;
  $col = $24;
 }
 $25 = $dest;
 $26 = (($25) + 24|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $dest;
 $29 = (($28) + 32|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $dest;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 (_fwrite(($27|0),1,($30|0),($33|0))|0);
 STACKTOP = sp;return;
}
function _put_demapped_gray($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bufferptr = 0, $col = 0, $color_map = 0, $dest = 0, $ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $0;
 $5 = (($4) + 136|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $color_map = $7;
 $8 = $dest;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $ptr = $11;
 $12 = $dest;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $bufferptr = $14;
 $15 = $0;
 $16 = (($15) + 112|0);
 $17 = HEAP32[$16>>2]|0;
 $col = $17;
 while(1) {
  $18 = $col;
  $19 = ($18>>>0)>(0);
  if (!($19)) {
   break;
  }
  $20 = $ptr;
  $21 = (($20) + 1|0);
  $ptr = $21;
  $22 = HEAP8[$20>>0]|0;
  $23 = $22&255;
  $24 = $color_map;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = $27&255;
  $29 = $bufferptr;
  $30 = (($29) + 1|0);
  $bufferptr = $30;
  HEAP8[$29>>0] = $28;
  $31 = $col;
  $32 = (($31) + -1)|0;
  $col = $32;
 }
 $33 = $dest;
 $34 = (($33) + 24|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $dest;
 $37 = (($36) + 32|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = $dest;
 $40 = (($39) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 (_fwrite(($35|0),1,($38|0),($41|0))|0);
 STACKTOP = sp;return;
}
function _put_demapped_rgb($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $bufferptr = 0, $col = 0, $color_map0 = 0, $color_map1 = 0, $color_map2 = 0, $dest = 0, $pixval = 0, $ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $0;
 $5 = (($4) + 136|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $color_map0 = $7;
 $8 = $0;
 $9 = (($8) + 136|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $color_map1 = $12;
 $13 = $0;
 $14 = (($13) + 136|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $color_map2 = $17;
 $18 = $dest;
 $19 = (($18) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $ptr = $21;
 $22 = $dest;
 $23 = (($22) + 24|0);
 $24 = HEAP32[$23>>2]|0;
 $bufferptr = $24;
 $25 = $0;
 $26 = (($25) + 112|0);
 $27 = HEAP32[$26>>2]|0;
 $col = $27;
 while(1) {
  $28 = $col;
  $29 = ($28>>>0)>(0);
  if (!($29)) {
   break;
  }
  $30 = $ptr;
  $31 = (($30) + 1|0);
  $ptr = $31;
  $32 = HEAP8[$30>>0]|0;
  $33 = $32&255;
  $pixval = $33;
  $34 = $pixval;
  $35 = $color_map0;
  $36 = (($35) + ($34)|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = $37&255;
  $39 = $38&255;
  $40 = $bufferptr;
  $41 = (($40) + 1|0);
  $bufferptr = $41;
  HEAP8[$40>>0] = $39;
  $42 = $pixval;
  $43 = $color_map1;
  $44 = (($43) + ($42)|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46&255;
  $48 = $bufferptr;
  $49 = (($48) + 1|0);
  $bufferptr = $49;
  HEAP8[$48>>0] = $47;
  $50 = $pixval;
  $51 = $color_map2;
  $52 = (($51) + ($50)|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $54&255;
  $56 = $bufferptr;
  $57 = (($56) + 1|0);
  $bufferptr = $57;
  HEAP8[$56>>0] = $55;
  $58 = $col;
  $59 = (($58) + -1)|0;
  $col = $59;
 }
 $60 = $dest;
 $61 = (($60) + 24|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $dest;
 $64 = (($63) + 32|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = $dest;
 $67 = (($66) + 12|0);
 $68 = HEAP32[$67>>2]|0;
 (_fwrite(($62|0),1,($65|0),($68|0))|0);
 STACKTOP = sp;return;
}
function _put_pixel_rows($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $dest;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $dest;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $dest;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 (_fwrite(($6|0),1,($9|0),($12|0))|0);
 STACKTOP = sp;return;
}
function _jinit_write_gif($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $dest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,316)|0);
 $dest = $6;
 $7 = $0;
 $8 = $dest;
 $9 = (($8) + 24|0);
 HEAP32[$9>>2] = $7;
 $10 = $dest;
 HEAP32[$10>>2] = 9;
 $11 = $dest;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 10;
 $13 = $dest;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 11;
 $15 = $0;
 $16 = (($15) + 44|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(1);
 if ($18) {
  $19 = $0;
  $20 = (($19) + 44|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)!=(2);
  if ($22) {
   $23 = $0;
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + 20|0);
   HEAP32[$25>>2] = 1014;
   $26 = $0;
   $27 = HEAP32[$26>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   $29 = $0;
   FUNCTION_TABLE_vi[$28 & 127]($29);
  }
 }
 $30 = $0;
 $31 = (($30) + 44|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(1);
 if ($33) {
  label = 6;
 } else {
  $34 = $0;
  $35 = (($34) + 212|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)>(8);
  if ($37) {
   label = 6;
  }
 }
 if ((label|0) == 6) {
  $38 = $0;
  $39 = (($38) + 84|0);
  HEAP32[$39>>2] = 1;
  $40 = $0;
  $41 = (($40) + 96|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)>(256);
  if ($43) {
   $44 = $0;
   $45 = (($44) + 96|0);
   HEAP32[$45>>2] = 256;
  }
 }
 $46 = $0;
 _jpeg_calc_output_dimensions($46);
 $47 = $0;
 $48 = (($47) + 124|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($49|0)!=(1);
 if ($50) {
  $51 = $0;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 20|0);
  HEAP32[$53>>2] = 1012;
  $54 = $0;
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$55>>2]|0;
  $57 = $0;
  FUNCTION_TABLE_vi[$56 & 127]($57);
 }
 $58 = $0;
 $59 = (($58) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + 8|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $0;
 $64 = $0;
 $65 = (($64) + 112|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = (FUNCTION_TABLE_iiiii[$62 & 127]($63,1,$66,1)|0);
 $68 = $dest;
 $69 = (($68) + 16|0);
 HEAP32[$69>>2] = $67;
 $70 = $dest;
 $71 = (($70) + 20|0);
 HEAP32[$71>>2] = 1;
 $72 = $dest;
 STACKTOP = sp;return ($72|0);
}
function _start_output_gif($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $dest = $2;
 $3 = $0;
 $4 = (($3) + 84|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $dest;
  $8 = $0;
  $9 = (($8) + 132|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = (($11) + 136|0);
  $13 = HEAP32[$12>>2]|0;
  _emit_header($7,$10,$13);
  STACKTOP = sp;return;
 } else {
  $14 = $dest;
  _emit_header($14,256,0);
  STACKTOP = sp;return;
 }
}
function _put_pixel_rows120($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $col = 0, $dest = 0, $ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $dest;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $ptr = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $col = $10;
 while(1) {
  $11 = $col;
  $12 = ($11>>>0)>(0);
  if (!($12)) {
   break;
  }
  $13 = $dest;
  $14 = $ptr;
  $15 = (($14) + 1|0);
  $ptr = $15;
  $16 = HEAP8[$14>>0]|0;
  $17 = $16&255;
  _compress_pixel($13,$17);
  $18 = $col;
  $19 = (($18) + -1)|0;
  $col = $19;
 }
 STACKTOP = sp;return;
}
function _finish_output_gif($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $dest = $2;
 $3 = $dest;
 _compress_term($3);
 $4 = $dest;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 (_putc(0,($6|0))|0);
 $7 = $dest;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 (_putc(59,($9|0))|0);
 $10 = $dest;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 (_fflush(($12|0))|0);
 $13 = $dest;
 $14 = (($13) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_ferror(($15|0))|0);
 $17 = ($16|0)!=(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = $0;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 20|0);
 HEAP32[$20>>2] = 37;
 $21 = $0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 FUNCTION_TABLE_vi[$23 & 127]($24);
 STACKTOP = sp;return;
}
function _compress_term($dinfo) {
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $0;
 $2 = $0;
 $3 = (($2) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 _output($1,$4);
 $5 = $0;
 $6 = (($5) + 40|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)>(0);
 if (!($8)) {
  $26 = $0;
  _flush_packet($26);
  STACKTOP = sp;return;
 }
 $9 = $0;
 $10 = (($9) + 36|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 & 255;
 $13 = $12&255;
 $14 = $0;
 $15 = (($14) + 56|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$15>>2] = $17;
 $18 = $0;
 $19 = (($18) + 60|0);
 $20 = (($19) + ($17)|0);
 HEAP8[$20>>0] = $13;
 $21 = $0;
 $22 = (($21) + 56|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)>=(255);
 if ($24) {
  $25 = $0;
  _flush_packet($25);
 }
 $26 = $0;
 _flush_packet($26);
 STACKTOP = sp;return;
}
function _output($dinfo,$code) {
 $dinfo = $dinfo|0;
 $code = $code|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $code;
 $2 = $1;
 $3 = $0;
 $4 = (($3) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $2 << $5;
 $7 = $0;
 $8 = (($7) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 | $6;
 HEAP32[$8>>2] = $10;
 $11 = $0;
 $12 = (($11) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $0;
 $15 = (($14) + 40|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13))|0;
 HEAP32[$15>>2] = $17;
 while(1) {
  $18 = $0;
  $19 = (($18) + 40|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)>=(8);
  if (!($21)) {
   break;
  }
  $22 = $0;
  $23 = (($22) + 36|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 255;
  $26 = $25&255;
  $27 = $0;
  $28 = (($27) + 56|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$28>>2] = $30;
  $31 = $0;
  $32 = (($31) + 60|0);
  $33 = (($32) + ($30)|0);
  HEAP8[$33>>0] = $26;
  $34 = $0;
  $35 = (($34) + 56|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)>=(255);
  if ($37) {
   $38 = $0;
   _flush_packet($38);
  }
  $39 = $0;
  $40 = (($39) + 36|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $41 >> 8;
  HEAP32[$40>>2] = $42;
  $43 = $0;
  $44 = (($43) + 40|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) - 8)|0;
  HEAP32[$44>>2] = $46;
 }
 STACKTOP = sp;return;
}
function _flush_packet($dinfo) {
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $0;
 $2 = (($1) + 56|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = $0;
 $6 = (($5) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 1)|0;
 HEAP32[$6>>2] = $8;
 $9 = $7&255;
 $10 = $0;
 $11 = (($10) + 60|0);
 HEAP8[$11>>0] = $9;
 $12 = $0;
 $13 = (($12) + 60|0);
 $14 = $0;
 $15 = (($14) + 56|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_fwrite(($13|0),1,($16|0),($19|0))|0);
 $21 = $0;
 $22 = (($21) + 56|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)!=($23|0);
 if ($24) {
  $25 = $0;
  $26 = (($25) + 24|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 20|0);
  HEAP32[$29>>2] = 37;
  $30 = $0;
  $31 = (($30) + 24|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  $36 = (($35) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  FUNCTION_TABLE_vi[$34 & 127]($37);
 }
 $38 = $0;
 $39 = (($38) + 56|0);
 HEAP32[$39>>2] = 0;
 STACKTOP = sp;return;
}
function _compress_pixel($dinfo,$c) {
 $dinfo = $dinfo|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $c;
 $2 = $0;
 $3 = $1;
 _output($2,$3);
 $4 = $0;
 $5 = (($4) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)<($9|0);
 if ($10) {
  $11 = $0;
  $12 = (($11) + 52|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$12>>2] = $14;
  STACKTOP = sp;return;
 } else {
  $15 = $0;
  $16 = $0;
  $17 = (($16) + 44|0);
  $18 = HEAP32[$17>>2]|0;
  _output($15,$18);
  $19 = $0;
  $20 = (($19) + 44|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 2)|0;
  $23 = $0;
  $24 = (($23) + 52|0);
  HEAP32[$24>>2] = $22;
  STACKTOP = sp;return;
 }
}
function _emit_header($dinfo,$num_colors,$colormap) {
 $dinfo = $dinfo|0;
 $num_colors = $num_colors|0;
 $colormap = $colormap|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $BitsPerPixel = 0, $ColorMapSize = 0, $FlagByte = 0, $InitCodeSize = 0, $cshift = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $dinfo;
 $1 = $num_colors;
 $2 = $colormap;
 $3 = $0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 212|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 8)|0;
 $cshift = $8;
 $9 = $1;
 $10 = ($9|0)>(256);
 if ($10) {
  $11 = $0;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 20|0);
  HEAP32[$15>>2] = 1039;
  $16 = $1;
  $17 = $0;
  $18 = (($17) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 24|0);
  HEAP32[$21>>2] = $16;
  $22 = $0;
  $23 = (($22) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  $28 = (($27) + 24|0);
  $29 = HEAP32[$28>>2]|0;
  FUNCTION_TABLE_vi[$26 & 127]($29);
 }
 $BitsPerPixel = 1;
 while(1) {
  $30 = $1;
  $31 = $BitsPerPixel;
  $32 = 1 << $31;
  $33 = ($30|0)>($32|0);
  if (!($33)) {
   break;
  }
  $34 = $BitsPerPixel;
  $35 = (($34) + 1)|0;
  $BitsPerPixel = $35;
 }
 $36 = $BitsPerPixel;
 $37 = 1 << $36;
 $ColorMapSize = $37;
 $38 = $BitsPerPixel;
 $39 = ($38|0)<=(1);
 if ($39) {
  $InitCodeSize = 2;
 } else {
  $40 = $BitsPerPixel;
  $InitCodeSize = $40;
 }
 $41 = $0;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 (_putc(71,($43|0))|0);
 $44 = $0;
 $45 = (($44) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 (_putc(73,($46|0))|0);
 $47 = $0;
 $48 = (($47) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 (_putc(70,($49|0))|0);
 $50 = $0;
 $51 = (($50) + 12|0);
 $52 = HEAP32[$51>>2]|0;
 (_putc(56,($52|0))|0);
 $53 = $0;
 $54 = (($53) + 12|0);
 $55 = HEAP32[$54>>2]|0;
 (_putc(55,($55|0))|0);
 $56 = $0;
 $57 = (($56) + 12|0);
 $58 = HEAP32[$57>>2]|0;
 (_putc(97,($58|0))|0);
 $59 = $0;
 $60 = $0;
 $61 = (($60) + 24|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (($62) + 112|0);
 $64 = HEAP32[$63>>2]|0;
 _put_word($59,$64);
 $65 = $0;
 $66 = $0;
 $67 = (($66) + 24|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = (($68) + 116|0);
 $70 = HEAP32[$69>>2]|0;
 _put_word($65,$70);
 $FlagByte = 128;
 $71 = $BitsPerPixel;
 $72 = (($71) - 1)|0;
 $73 = $72 << 4;
 $74 = $FlagByte;
 $75 = $74 | $73;
 $FlagByte = $75;
 $76 = $BitsPerPixel;
 $77 = (($76) - 1)|0;
 $78 = $FlagByte;
 $79 = $78 | $77;
 $FlagByte = $79;
 $80 = $FlagByte;
 $81 = $0;
 $82 = (($81) + 12|0);
 $83 = HEAP32[$82>>2]|0;
 (_putc(($80|0),($83|0))|0);
 $84 = $0;
 $85 = (($84) + 12|0);
 $86 = HEAP32[$85>>2]|0;
 (_putc(0,($86|0))|0);
 $87 = $0;
 $88 = (($87) + 12|0);
 $89 = HEAP32[$88>>2]|0;
 (_putc(0,($89|0))|0);
 $i = 0;
 while(1) {
  $90 = $i;
  $91 = $ColorMapSize;
  $92 = ($90|0)<($91|0);
  if (!($92)) {
   break;
  }
  $93 = $i;
  $94 = $1;
  $95 = ($93|0)<($94|0);
  if ($95) {
   $96 = $2;
   $97 = ($96|0)!=(0|0);
   if ($97) {
    $98 = $0;
    $99 = (($98) + 24|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = (($100) + 44|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = ($102|0)==(2);
    if ($103) {
     $104 = $i;
     $105 = $2;
     $106 = HEAP32[$105>>2]|0;
     $107 = (($106) + ($104)|0);
     $108 = HEAP8[$107>>0]|0;
     $109 = $108&255;
     $110 = $cshift;
     $111 = $109 >> $110;
     $112 = $0;
     $113 = (($112) + 12|0);
     $114 = HEAP32[$113>>2]|0;
     (_putc(($111|0),($114|0))|0);
     $115 = $i;
     $116 = $2;
     $117 = (($116) + 4|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = (($118) + ($115)|0);
     $120 = HEAP8[$119>>0]|0;
     $121 = $120&255;
     $122 = $cshift;
     $123 = $121 >> $122;
     $124 = $0;
     $125 = (($124) + 12|0);
     $126 = HEAP32[$125>>2]|0;
     (_putc(($123|0),($126|0))|0);
     $127 = $i;
     $128 = $2;
     $129 = (($128) + 8|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = (($130) + ($127)|0);
     $132 = HEAP8[$131>>0]|0;
     $133 = $132&255;
     $134 = $cshift;
     $135 = $133 >> $134;
     $136 = $0;
     $137 = (($136) + 12|0);
     $138 = HEAP32[$137>>2]|0;
     (_putc(($135|0),($138|0))|0);
    } else {
     $139 = $0;
     $140 = $i;
     $141 = $2;
     $142 = HEAP32[$141>>2]|0;
     $143 = (($142) + ($140)|0);
     $144 = HEAP8[$143>>0]|0;
     $145 = $144&255;
     $146 = $cshift;
     $147 = $145 >> $146;
     _put_3bytes($139,$147);
    }
   } else {
    $148 = $0;
    $149 = $i;
    $150 = ($149*255)|0;
    $151 = $1;
    $152 = (($151) - 1)|0;
    $153 = (($152|0) / 2)&-1;
    $154 = (($150) + ($153))|0;
    $155 = $1;
    $156 = (($155) - 1)|0;
    $157 = (($154|0) / ($156|0))&-1;
    _put_3bytes($148,$157);
   }
  } else {
   $158 = $0;
   _put_3bytes($158,0);
  }
  $159 = $i;
  $160 = (($159) + 1)|0;
  $i = $160;
 }
 $161 = $0;
 $162 = (($161) + 12|0);
 $163 = HEAP32[$162>>2]|0;
 (_putc(44,($163|0))|0);
 $164 = $0;
 _put_word($164,0);
 $165 = $0;
 _put_word($165,0);
 $166 = $0;
 $167 = $0;
 $168 = (($167) + 24|0);
 $169 = HEAP32[$168>>2]|0;
 $170 = (($169) + 112|0);
 $171 = HEAP32[$170>>2]|0;
 _put_word($166,$171);
 $172 = $0;
 $173 = $0;
 $174 = (($173) + 24|0);
 $175 = HEAP32[$174>>2]|0;
 $176 = (($175) + 116|0);
 $177 = HEAP32[$176>>2]|0;
 _put_word($172,$177);
 $178 = $0;
 $179 = (($178) + 12|0);
 $180 = HEAP32[$179>>2]|0;
 (_putc(0,($180|0))|0);
 $181 = $InitCodeSize;
 $182 = $0;
 $183 = (($182) + 12|0);
 $184 = HEAP32[$183>>2]|0;
 (_putc(($181|0),($184|0))|0);
 $185 = $0;
 $186 = $InitCodeSize;
 $187 = (($186) + 1)|0;
 _compress_init($185,$187);
 STACKTOP = sp;return;
}
function _put_word($dinfo,$w) {
 $dinfo = $dinfo|0;
 $w = $w|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $w;
 $2 = $1;
 $3 = $2 & 255;
 $4 = $0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 (_putc(($3|0),($6|0))|0);
 $7 = $1;
 $8 = $7 >>> 8;
 $9 = $8 & 255;
 $10 = $0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 (_putc(($9|0),($12|0))|0);
 STACKTOP = sp;return;
}
function _put_3bytes($dinfo,$val) {
 $dinfo = $dinfo|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $val;
 $2 = $1;
 $3 = $0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 (_putc(($2|0),($5|0))|0);
 $6 = $1;
 $7 = $0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 (_putc(($6|0),($9|0))|0);
 $10 = $1;
 $11 = $0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 (_putc(($10|0),($13|0))|0);
 STACKTOP = sp;return;
}
function _compress_init($dinfo,$i_bits) {
 $dinfo = $dinfo|0;
 $i_bits = $i_bits|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $i_bits;
 $2 = $1;
 $3 = $0;
 $4 = (($3) + 28|0);
 HEAP32[$4>>2] = $2;
 $5 = $0;
 $6 = (($5) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = 1 << $7;
 $9 = (($8) - 1)|0;
 $10 = $0;
 $11 = (($10) + 32|0);
 HEAP32[$11>>2] = $9;
 $12 = $1;
 $13 = (($12) - 1)|0;
 $14 = 1 << $13;
 $15 = $0;
 $16 = (($15) + 44|0);
 HEAP32[$16>>2] = $14;
 $17 = $0;
 $18 = (($17) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 1)|0;
 $21 = $0;
 $22 = (($21) + 48|0);
 HEAP32[$22>>2] = $20;
 $23 = $0;
 $24 = (($23) + 44|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 2)|0;
 $27 = $0;
 $28 = (($27) + 52|0);
 HEAP32[$28>>2] = $26;
 $29 = $0;
 $30 = (($29) + 56|0);
 HEAP32[$30>>2] = 0;
 $31 = $0;
 $32 = (($31) + 36|0);
 HEAP32[$32>>2] = 0;
 $33 = $0;
 $34 = (($33) + 40|0);
 HEAP32[$34>>2] = 0;
 $35 = $0;
 $36 = $0;
 $37 = (($36) + 44|0);
 $38 = HEAP32[$37>>2]|0;
 _output($35,$38);
 STACKTOP = sp;return;
}
function _jinit_write_targa($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,32)|0);
 $dest = $6;
 $7 = $dest;
 HEAP32[$7>>2] = 12;
 $8 = $dest;
 $9 = (($8) + 8|0);
 HEAP32[$9>>2] = 13;
 $10 = $0;
 _jpeg_calc_output_dimensions($10);
 $11 = $0;
 $12 = (($11) + 112|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $0;
 $15 = (($14) + 124|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = Math_imul($13, $16)|0;
 $18 = $dest;
 $19 = (($18) + 28|0);
 HEAP32[$19>>2] = $17;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = $dest;
 $26 = (($25) + 28|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $27;
 $29 = (FUNCTION_TABLE_iiii[$23 & 127]($24,1,$28)|0);
 $30 = $dest;
 $31 = (($30) + 24|0);
 HEAP32[$31>>2] = $29;
 $32 = $0;
 $33 = (($32) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = $0;
 $38 = $dest;
 $39 = (($38) + 28|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (FUNCTION_TABLE_iiiii[$36 & 127]($37,1,$40,1)|0);
 $42 = $dest;
 $43 = (($42) + 16|0);
 HEAP32[$43>>2] = $41;
 $44 = $dest;
 $45 = (($44) + 20|0);
 HEAP32[$45>>2] = 1;
 $46 = $dest;
 STACKTOP = sp;return ($46|0);
}
function _start_output_tga($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $dest = 0, $i = 0, $num_colors = 0, $outfile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $dest = $2;
 $3 = $0;
 $4 = (($3) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(1);
 if ($6) {
  $7 = $0;
  $8 = $1;
  _write_header($7,$8,0);
  $9 = $0;
  $10 = (($9) + 84|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = $dest;
   $14 = (($13) + 4|0);
   HEAP32[$14>>2] = 14;
  } else {
   $15 = $dest;
   $16 = (($15) + 4|0);
   HEAP32[$16>>2] = 15;
  }
  STACKTOP = sp;return;
 }
 $17 = $0;
 $18 = (($17) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)==(2);
 if ($20) {
  $21 = $0;
  $22 = (($21) + 84|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0);
  if ($24) {
   $25 = $0;
   $26 = (($25) + 132|0);
   $27 = HEAP32[$26>>2]|0;
   $num_colors = $27;
   $28 = $num_colors;
   $29 = ($28|0)>(256);
   if ($29) {
    $30 = $0;
    $31 = HEAP32[$30>>2]|0;
    $32 = (($31) + 20|0);
    HEAP32[$32>>2] = 1039;
    $33 = $num_colors;
    $34 = $0;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($35) + 24|0);
    HEAP32[$36>>2] = $33;
    $37 = $0;
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = $0;
    FUNCTION_TABLE_vi[$39 & 127]($40);
   }
   $41 = $0;
   $42 = $1;
   $43 = $num_colors;
   _write_header($41,$42,$43);
   $44 = $dest;
   $45 = (($44) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $outfile = $46;
   $i = 0;
   while(1) {
    $47 = $i;
    $48 = $num_colors;
    $49 = ($47|0)<($48|0);
    if (!($49)) {
     break;
    }
    $50 = $i;
    $51 = $0;
    $52 = (($51) + 136|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (($53) + 8|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($50)|0);
    $57 = HEAP8[$56>>0]|0;
    $58 = $57&255;
    $59 = $outfile;
    (_putc(($58|0),($59|0))|0);
    $60 = $i;
    $61 = $0;
    $62 = (($61) + 136|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = (($63) + 4|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = (($65) + ($60)|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = $outfile;
    (_putc(($68|0),($69|0))|0);
    $70 = $i;
    $71 = $0;
    $72 = (($71) + 136|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + ($70)|0);
    $76 = HEAP8[$75>>0]|0;
    $77 = $76&255;
    $78 = $outfile;
    (_putc(($77|0),($78|0))|0);
    $79 = $i;
    $80 = (($79) + 1)|0;
    $i = $80;
   }
   $81 = $dest;
   $82 = (($81) + 4|0);
   HEAP32[$82>>2] = 15;
  } else {
   $83 = $0;
   $84 = $1;
   _write_header($83,$84,0);
   $85 = $dest;
   $86 = (($85) + 4|0);
   HEAP32[$86>>2] = 16;
  }
 } else {
  $87 = $0;
  $88 = HEAP32[$87>>2]|0;
  $89 = (($88) + 20|0);
  HEAP32[$89>>2] = 1034;
  $90 = $0;
  $91 = HEAP32[$90>>2]|0;
  $92 = HEAP32[$91>>2]|0;
  $93 = $0;
  FUNCTION_TABLE_vi[$92 & 127]($93);
 }
 STACKTOP = sp;return;
}
function _finish_output_tga($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 (_fflush(($4|0))|0);
 $5 = $1;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_ferror(($7|0))|0);
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 20|0);
 HEAP32[$12>>2] = 37;
 $13 = $0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 FUNCTION_TABLE_vi[$15 & 127]($16);
 STACKTOP = sp;return;
}
function _write_header($cinfo,$dinfo,$num_colors) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $num_colors = $num_colors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $targaheader = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $targaheader = sp + 12|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $num_colors;
 dest=$targaheader+0|0; stop=dest+18|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 $3 = $2;
 $4 = ($3|0)>(0);
 if ($4) {
  $5 = (($targaheader) + 1|0);
  HEAP8[$5>>0] = 1;
  $6 = $2;
  $7 = $6 & 255;
  $8 = $7&255;
  $9 = (($targaheader) + 5|0);
  HEAP8[$9>>0] = $8;
  $10 = $2;
  $11 = $10 >> 8;
  $12 = $11&255;
  $13 = (($targaheader) + 6|0);
  HEAP8[$13>>0] = $12;
  $14 = (($targaheader) + 7|0);
  HEAP8[$14>>0] = 24;
 }
 $15 = $0;
 $16 = (($15) + 112|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17 & 255;
 $19 = $18&255;
 $20 = (($targaheader) + 12|0);
 HEAP8[$20>>0] = $19;
 $21 = $0;
 $22 = (($21) + 112|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 >>> 8;
 $25 = $24&255;
 $26 = (($targaheader) + 13|0);
 HEAP8[$26>>0] = $25;
 $27 = $0;
 $28 = (($27) + 116|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $29 & 255;
 $31 = $30&255;
 $32 = (($targaheader) + 14|0);
 HEAP8[$32>>0] = $31;
 $33 = $0;
 $34 = (($33) + 116|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $35 >>> 8;
 $37 = $36&255;
 $38 = (($targaheader) + 15|0);
 HEAP8[$38>>0] = $37;
 $39 = (($targaheader) + 17|0);
 HEAP8[$39>>0] = 32;
 $40 = $0;
 $41 = (($40) + 44|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)==(1);
 if ($43) {
  $44 = (($targaheader) + 2|0);
  HEAP8[$44>>0] = 3;
  $45 = (($targaheader) + 16|0);
  HEAP8[$45>>0] = 8;
 } else {
  $46 = $2;
  $47 = ($46|0)>(0);
  if ($47) {
   $48 = (($targaheader) + 2|0);
   HEAP8[$48>>0] = 1;
   $49 = (($targaheader) + 16|0);
   HEAP8[$49>>0] = 8;
  } else {
   $50 = (($targaheader) + 2|0);
   HEAP8[$50>>0] = 2;
   $51 = (($targaheader) + 16|0);
   HEAP8[$51>>0] = 24;
  }
 }
 $52 = $1;
 $53 = (($52) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (_fwrite(($targaheader|0),1,18,($54|0))|0);
 $56 = ($55|0)!=(18);
 if (!($56)) {
  STACKTOP = sp;return;
 }
 $57 = $0;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($58) + 20|0);
 HEAP32[$59>>2] = 37;
 $60 = $0;
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$61>>2]|0;
 $63 = $0;
 FUNCTION_TABLE_vi[$62 & 127]($63);
 STACKTOP = sp;return;
}
function _put_demapped_gray123($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $col = 0, $color_map0 = 0, $dest = 0, $inptr = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $0;
 $5 = (($4) + 136|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $color_map0 = $7;
 $8 = $dest;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $inptr = $11;
 $12 = $dest;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $outptr = $14;
 $15 = $0;
 $16 = (($15) + 112|0);
 $17 = HEAP32[$16>>2]|0;
 $col = $17;
 while(1) {
  $18 = $col;
  $19 = ($18>>>0)>(0);
  if (!($19)) {
   break;
  }
  $20 = $inptr;
  $21 = (($20) + 1|0);
  $inptr = $21;
  $22 = HEAP8[$20>>0]|0;
  $23 = $22&255;
  $24 = $color_map0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = $27&255;
  $29 = $outptr;
  $30 = (($29) + 1|0);
  $outptr = $30;
  HEAP8[$29>>0] = $28;
  $31 = $col;
  $32 = (($31) + -1)|0;
  $col = $32;
 }
 $33 = $dest;
 $34 = (($33) + 24|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $dest;
 $37 = (($36) + 28|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = $dest;
 $40 = (($39) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 (_fwrite(($35|0),1,($38|0),($41|0))|0);
 STACKTOP = sp;return;
}
function _put_gray_rows($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $dest = 0, $inptr = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $dest;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $inptr = $7;
 $8 = $dest;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $outptr = $10;
 $11 = $0;
 $12 = (($11) + 112|0);
 $13 = HEAP32[$12>>2]|0;
 $col = $13;
 while(1) {
  $14 = $col;
  $15 = ($14>>>0)>(0);
  if (!($15)) {
   break;
  }
  $16 = $inptr;
  $17 = (($16) + 1|0);
  $inptr = $17;
  $18 = HEAP8[$16>>0]|0;
  $19 = $18&255;
  $20 = $19&255;
  $21 = $outptr;
  $22 = (($21) + 1|0);
  $outptr = $22;
  HEAP8[$21>>0] = $20;
  $23 = $col;
  $24 = (($23) + -1)|0;
  $col = $24;
 }
 $25 = $dest;
 $26 = (($25) + 24|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $dest;
 $29 = (($28) + 28|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $dest;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 (_fwrite(($27|0),1,($30|0),($33|0))|0);
 STACKTOP = sp;return;
}
function _put_pixel_rows124($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $dest = 0, $inptr = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $dest;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $inptr = $7;
 $8 = $dest;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $outptr = $10;
 $11 = $0;
 $12 = (($11) + 112|0);
 $13 = HEAP32[$12>>2]|0;
 $col = $13;
 while(1) {
  $14 = $col;
  $15 = ($14>>>0)>(0);
  if (!($15)) {
   break;
  }
  $16 = $inptr;
  $17 = (($16) + 2|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = $19&255;
  $21 = $outptr;
  HEAP8[$21>>0] = $20;
  $22 = $inptr;
  $23 = (($22) + 1|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = $25&255;
  $27 = $outptr;
  $28 = (($27) + 1|0);
  HEAP8[$28>>0] = $26;
  $29 = $inptr;
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = $31&255;
  $33 = $outptr;
  $34 = (($33) + 2|0);
  HEAP8[$34>>0] = $32;
  $35 = $inptr;
  $36 = (($35) + 3|0);
  $inptr = $36;
  $37 = $outptr;
  $38 = (($37) + 3|0);
  $outptr = $38;
  $39 = $col;
  $40 = (($39) + -1)|0;
  $col = $40;
 }
 $41 = $dest;
 $42 = (($41) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $dest;
 $45 = (($44) + 28|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $dest;
 $48 = (($47) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 (_fwrite(($43|0),1,($46|0),($49|0))|0);
 STACKTOP = sp;return;
}
function _jinit_write_bmp($cinfo,$is_os2) {
 $cinfo = $cinfo|0;
 $is_os2 = $is_os2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dest = 0, $progress = 0, $row_width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $is_os2;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 127]($6,1,48)|0);
 $dest = $7;
 $8 = $dest;
 HEAP32[$8>>2] = 17;
 $9 = $dest;
 $10 = (($9) + 8|0);
 HEAP32[$10>>2] = 18;
 $11 = $1;
 $12 = $dest;
 $13 = (($12) + 24|0);
 HEAP32[$13>>2] = $11;
 $14 = $0;
 $15 = (($14) + 44|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = $dest;
  $19 = (($18) + 4|0);
  HEAP32[$19>>2] = 19;
 } else {
  $20 = $0;
  $21 = (($20) + 44|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)==(2);
  if ($23) {
   $24 = $0;
   $25 = (($24) + 84|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0);
   if ($27) {
    $28 = $dest;
    $29 = (($28) + 4|0);
    HEAP32[$29>>2] = 19;
   } else {
    $30 = $dest;
    $31 = (($30) + 4|0);
    HEAP32[$31>>2] = 20;
   }
  } else {
   $32 = $0;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 20|0);
   HEAP32[$34>>2] = 1005;
   $35 = $0;
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   $38 = $0;
   FUNCTION_TABLE_vi[$37 & 127]($38);
  }
 }
 $39 = $0;
 _jpeg_calc_output_dimensions($39);
 $40 = $0;
 $41 = (($40) + 112|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $0;
 $44 = (($43) + 124|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = Math_imul($42, $45)|0;
 $row_width = $46;
 $47 = $row_width;
 $48 = $dest;
 $49 = (($48) + 32|0);
 HEAP32[$49>>2] = $47;
 while(1) {
  $50 = $row_width;
  $51 = $50 & 3;
  $52 = ($51|0)!=(0);
  if (!($52)) {
   break;
  }
  $53 = $row_width;
  $54 = (($53) + 1)|0;
  $row_width = $54;
 }
 $55 = $row_width;
 $56 = $dest;
 $57 = (($56) + 36|0);
 HEAP32[$57>>2] = $55;
 $58 = $row_width;
 $59 = $dest;
 $60 = (($59) + 32|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (($58) - ($61))|0;
 $63 = $dest;
 $64 = (($63) + 40|0);
 HEAP32[$64>>2] = $62;
 $65 = $0;
 $66 = (($65) + 4|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 16|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $0;
 $71 = $row_width;
 $72 = $0;
 $73 = (($72) + 116|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = (FUNCTION_TABLE_iiiiiii[$69 & 127]($70,1,0,$71,$74,1)|0);
 $76 = $dest;
 $77 = (($76) + 28|0);
 HEAP32[$77>>2] = $75;
 $78 = $dest;
 $79 = (($78) + 44|0);
 HEAP32[$79>>2] = 0;
 $80 = $0;
 $81 = (($80) + 8|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = ($82|0)!=(0|0);
 if (!($83)) {
  $91 = $0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + 8|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = $0;
  $97 = $row_width;
  $98 = (FUNCTION_TABLE_iiiii[$95 & 127]($96,1,$97,1)|0);
  $99 = $dest;
  $100 = (($99) + 16|0);
  HEAP32[$100>>2] = $98;
  $101 = $dest;
  $102 = (($101) + 20|0);
  HEAP32[$102>>2] = 1;
  $103 = $dest;
  STACKTOP = sp;return ($103|0);
 }
 $84 = $0;
 $85 = (($84) + 8|0);
 $86 = HEAP32[$85>>2]|0;
 $progress = $86;
 $87 = $progress;
 $88 = (($87) + 24|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + 1)|0;
 HEAP32[$88>>2] = $90;
 $91 = $0;
 $92 = (($91) + 4|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = (($93) + 8|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = $0;
 $97 = $row_width;
 $98 = (FUNCTION_TABLE_iiiii[$95 & 127]($96,1,$97,1)|0);
 $99 = $dest;
 $100 = (($99) + 16|0);
 HEAP32[$100>>2] = $98;
 $101 = $dest;
 $102 = (($101) + 20|0);
 HEAP32[$102>>2] = 1;
 $103 = $dest;
 STACKTOP = sp;return ($103|0);
}
function _start_output_bmp($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $dinfo;
 STACKTOP = sp;return;
}
function _finish_output_bmp($cinfo,$dinfo) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $col = 0, $data_ptr = 0, $dest = 0, $image_ptr = 0, $outfile = 0, $progress = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $1;
 $dest = $2;
 $3 = $dest;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $outfile = $5;
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $progress = $8;
 $9 = $dest;
 $10 = (($9) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $0;
  $14 = $dest;
  _write_os2_header($13,$14);
 } else {
  $15 = $0;
  $16 = $dest;
  _write_bmp_header($15,$16);
 }
 $17 = $0;
 $18 = (($17) + 116|0);
 $19 = HEAP32[$18>>2]|0;
 $row = $19;
 while(1) {
  $20 = $row;
  $21 = ($20>>>0)>(0);
  if (!($21)) {
   break;
  }
  $22 = $progress;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = $0;
   $25 = (($24) + 116|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $row;
   $28 = (($26) - ($27))|0;
   $29 = $progress;
   $30 = (($29) + 4|0);
   HEAP32[$30>>2] = $28;
   $31 = $0;
   $32 = (($31) + 116|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $progress;
   $35 = (($34) + 8|0);
   HEAP32[$35>>2] = $33;
   $36 = $progress;
   $37 = HEAP32[$36>>2]|0;
   $38 = $0;
   FUNCTION_TABLE_vi[$37 & 127]($38);
  }
  $39 = $0;
  $40 = (($39) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $0;
  $45 = $dest;
  $46 = (($45) + 28|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $row;
  $49 = (($48) - 1)|0;
  $50 = (FUNCTION_TABLE_iiiiii[$43 & 127]($44,$47,$49,1,0)|0);
  $image_ptr = $50;
  $51 = $image_ptr;
  $52 = HEAP32[$51>>2]|0;
  $data_ptr = $52;
  $53 = $dest;
  $54 = (($53) + 36|0);
  $55 = HEAP32[$54>>2]|0;
  $col = $55;
  while(1) {
   $56 = $col;
   $57 = ($56>>>0)>(0);
   if (!($57)) {
    break;
   }
   $58 = $data_ptr;
   $59 = HEAP8[$58>>0]|0;
   $60 = $59&255;
   $61 = $outfile;
   (_putc(($60|0),($61|0))|0);
   $62 = $data_ptr;
   $63 = (($62) + 1|0);
   $data_ptr = $63;
   $64 = $col;
   $65 = (($64) + -1)|0;
   $col = $65;
  }
  $66 = $row;
  $67 = (($66) + -1)|0;
  $row = $67;
 }
 $68 = $progress;
 $69 = ($68|0)!=(0|0);
 if ($69) {
  $70 = $progress;
  $71 = (($70) + 20|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$71>>2] = $73;
 }
 $74 = $outfile;
 (_fflush(($74|0))|0);
 $75 = $outfile;
 $76 = (_ferror(($75|0))|0);
 $77 = ($76|0)!=(0);
 if (!($77)) {
  STACKTOP = sp;return;
 }
 $78 = $0;
 $79 = HEAP32[$78>>2]|0;
 $80 = (($79) + 20|0);
 HEAP32[$80>>2] = 37;
 $81 = $0;
 $82 = HEAP32[$81>>2]|0;
 $83 = HEAP32[$82>>2]|0;
 $84 = $0;
 FUNCTION_TABLE_vi[$83 & 127]($84);
 STACKTOP = sp;return;
}
function _put_gray_rows127($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $dest = 0, $image_ptr = 0, $inptr = 0, $outptr = 0, $pad = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = $dest;
 $11 = (($10) + 28|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $dest;
 $14 = (($13) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (FUNCTION_TABLE_iiiiii[$8 & 127]($9,$12,$15,1,1)|0);
 $image_ptr = $16;
 $17 = $dest;
 $18 = (($17) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 1)|0;
 HEAP32[$18>>2] = $20;
 $21 = $dest;
 $22 = (($21) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $inptr = $24;
 $25 = $image_ptr;
 $26 = HEAP32[$25>>2]|0;
 $outptr = $26;
 $27 = $0;
 $28 = (($27) + 112|0);
 $29 = HEAP32[$28>>2]|0;
 $col = $29;
 while(1) {
  $30 = $col;
  $31 = ($30>>>0)>(0);
  if (!($31)) {
   break;
  }
  $32 = $inptr;
  $33 = (($32) + 1|0);
  $inptr = $33;
  $34 = HEAP8[$32>>0]|0;
  $35 = $outptr;
  $36 = (($35) + 1|0);
  $outptr = $36;
  HEAP8[$35>>0] = $34;
  $37 = $col;
  $38 = (($37) + -1)|0;
  $col = $38;
 }
 $39 = $dest;
 $40 = (($39) + 40|0);
 $41 = HEAP32[$40>>2]|0;
 $pad = $41;
 while(1) {
  $42 = $pad;
  $43 = (($42) + -1)|0;
  $pad = $43;
  $44 = ($43|0)>=(0);
  if (!($44)) {
   break;
  }
  $45 = $outptr;
  $46 = (($45) + 1|0);
  $outptr = $46;
  HEAP8[$45>>0] = 0;
 }
 STACKTOP = sp;return;
}
function _put_pixel_rows128($cinfo,$dinfo,$rows_supplied) {
 $cinfo = $cinfo|0;
 $dinfo = $dinfo|0;
 $rows_supplied = $rows_supplied|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $dest = 0;
 var $image_ptr = 0, $inptr = 0, $outptr = 0, $pad = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $dinfo;
 $2 = $rows_supplied;
 $3 = $1;
 $dest = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = $dest;
 $11 = (($10) + 28|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $dest;
 $14 = (($13) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (FUNCTION_TABLE_iiiiii[$8 & 127]($9,$12,$15,1,1)|0);
 $image_ptr = $16;
 $17 = $dest;
 $18 = (($17) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 1)|0;
 HEAP32[$18>>2] = $20;
 $21 = $dest;
 $22 = (($21) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $inptr = $24;
 $25 = $image_ptr;
 $26 = HEAP32[$25>>2]|0;
 $outptr = $26;
 $27 = $0;
 $28 = (($27) + 112|0);
 $29 = HEAP32[$28>>2]|0;
 $col = $29;
 while(1) {
  $30 = $col;
  $31 = ($30>>>0)>(0);
  if (!($31)) {
   break;
  }
  $32 = $inptr;
  $33 = (($32) + 1|0);
  $inptr = $33;
  $34 = HEAP8[$32>>0]|0;
  $35 = $outptr;
  $36 = (($35) + 2|0);
  HEAP8[$36>>0] = $34;
  $37 = $inptr;
  $38 = (($37) + 1|0);
  $inptr = $38;
  $39 = HEAP8[$37>>0]|0;
  $40 = $outptr;
  $41 = (($40) + 1|0);
  HEAP8[$41>>0] = $39;
  $42 = $inptr;
  $43 = (($42) + 1|0);
  $inptr = $43;
  $44 = HEAP8[$42>>0]|0;
  $45 = $outptr;
  HEAP8[$45>>0] = $44;
  $46 = $outptr;
  $47 = (($46) + 3|0);
  $outptr = $47;
  $48 = $col;
  $49 = (($48) + -1)|0;
  $col = $49;
 }
 $50 = $dest;
 $51 = (($50) + 40|0);
 $52 = HEAP32[$51>>2]|0;
 $pad = $52;
 while(1) {
  $53 = $pad;
  $54 = (($53) + -1)|0;
  $pad = $54;
  $55 = ($54|0)>=(0);
  if (!($55)) {
   break;
  }
  $56 = $outptr;
  $57 = (($56) + 1|0);
  $outptr = $57;
  HEAP8[$56>>0] = 0;
 }
 STACKTOP = sp;return;
}
function _write_os2_header($cinfo,$dest) {
 $cinfo = $cinfo|0;
 $dest = $dest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bfSize = 0, $bits_per_pixel = 0, $bmpcoreheader = 0, $bmpfileheader = 0, $cmap_entries = 0, $headersize = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $bmpfileheader = sp + 36|0;
 $bmpcoreheader = sp + 24|0;
 $0 = $cinfo;
 $1 = $dest;
 $2 = $0;
 $3 = (($2) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(2);
 if ($5) {
  $6 = $0;
  $7 = (($6) + 84|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0);
  if ($9) {
   $bits_per_pixel = 8;
   $cmap_entries = 256;
  } else {
   $bits_per_pixel = 24;
   $cmap_entries = 0;
  }
 } else {
  $bits_per_pixel = 8;
  $cmap_entries = 256;
 }
 $10 = $cmap_entries;
 $11 = ($10*3)|0;
 $12 = (26 + ($11))|0;
 $headersize = $12;
 $13 = $headersize;
 $14 = $1;
 $15 = (($14) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = (($17) + 116|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = Math_imul($16, $19)|0;
 $21 = (($13) + ($20))|0;
 $bfSize = $21;
 dest=$bmpfileheader+0|0; stop=dest+14|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 dest=$bmpcoreheader+0|0; stop=dest+12|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 HEAP8[$bmpfileheader>>0] = 66;
 $22 = (($bmpfileheader) + 1|0);
 HEAP8[$22>>0] = 77;
 $23 = $bfSize;
 $24 = $23 & 255;
 $25 = $24&255;
 $26 = (($bmpfileheader) + 2|0);
 HEAP8[$26>>0] = $25;
 $27 = $bfSize;
 $28 = $27 >> 8;
 $29 = $28 & 255;
 $30 = $29&255;
 $31 = (($bmpfileheader) + 3|0);
 HEAP8[$31>>0] = $30;
 $32 = $bfSize;
 $33 = $32 >> 16;
 $34 = $33 & 255;
 $35 = $34&255;
 $36 = (($bmpfileheader) + 4|0);
 HEAP8[$36>>0] = $35;
 $37 = $bfSize;
 $38 = $37 >> 24;
 $39 = $38 & 255;
 $40 = $39&255;
 $41 = (($bmpfileheader) + 5|0);
 HEAP8[$41>>0] = $40;
 $42 = $headersize;
 $43 = $42 & 255;
 $44 = $43&255;
 $45 = (($bmpfileheader) + 10|0);
 HEAP8[$45>>0] = $44;
 $46 = $headersize;
 $47 = $46 >> 8;
 $48 = $47 & 255;
 $49 = $48&255;
 $50 = (($bmpfileheader) + 11|0);
 HEAP8[$50>>0] = $49;
 $51 = $headersize;
 $52 = $51 >> 16;
 $53 = $52 & 255;
 $54 = $53&255;
 $55 = (($bmpfileheader) + 12|0);
 HEAP8[$55>>0] = $54;
 $56 = $headersize;
 $57 = $56 >> 24;
 $58 = $57 & 255;
 $59 = $58&255;
 $60 = (($bmpfileheader) + 13|0);
 HEAP8[$60>>0] = $59;
 HEAP8[$bmpcoreheader>>0] = 12;
 $61 = (($bmpcoreheader) + 1|0);
 HEAP8[$61>>0] = 0;
 $62 = $0;
 $63 = (($62) + 112|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = $64 & 255;
 $66 = $65&255;
 $67 = (($bmpcoreheader) + 4|0);
 HEAP8[$67>>0] = $66;
 $68 = $0;
 $69 = (($68) + 112|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $70 >>> 8;
 $72 = $71 & 255;
 $73 = $72&255;
 $74 = (($bmpcoreheader) + 5|0);
 HEAP8[$74>>0] = $73;
 $75 = $0;
 $76 = (($75) + 116|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = $77 & 255;
 $79 = $78&255;
 $80 = (($bmpcoreheader) + 6|0);
 HEAP8[$80>>0] = $79;
 $81 = $0;
 $82 = (($81) + 116|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = $83 >>> 8;
 $85 = $84 & 255;
 $86 = $85&255;
 $87 = (($bmpcoreheader) + 7|0);
 HEAP8[$87>>0] = $86;
 $88 = (($bmpcoreheader) + 8|0);
 HEAP8[$88>>0] = 1;
 $89 = (($bmpcoreheader) + 9|0);
 HEAP8[$89>>0] = 0;
 $90 = $bits_per_pixel;
 $91 = $90 & 255;
 $92 = $91&255;
 $93 = (($bmpcoreheader) + 10|0);
 HEAP8[$93>>0] = $92;
 $94 = $bits_per_pixel;
 $95 = $94 >> 8;
 $96 = $95 & 255;
 $97 = $96&255;
 $98 = (($bmpcoreheader) + 11|0);
 HEAP8[$98>>0] = $97;
 $99 = $1;
 $100 = (($99) + 12|0);
 $101 = HEAP32[$100>>2]|0;
 $102 = (_fwrite(($bmpfileheader|0),1,14,($101|0))|0);
 $103 = ($102|0)!=(14);
 if ($103) {
  $104 = $0;
  $105 = HEAP32[$104>>2]|0;
  $106 = (($105) + 20|0);
  HEAP32[$106>>2] = 37;
  $107 = $0;
  $108 = HEAP32[$107>>2]|0;
  $109 = HEAP32[$108>>2]|0;
  $110 = $0;
  FUNCTION_TABLE_vi[$109 & 127]($110);
 }
 $111 = $1;
 $112 = (($111) + 12|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = (_fwrite(($bmpcoreheader|0),1,12,($113|0))|0);
 $115 = ($114|0)!=(12);
 if ($115) {
  $116 = $0;
  $117 = HEAP32[$116>>2]|0;
  $118 = (($117) + 20|0);
  HEAP32[$118>>2] = 37;
  $119 = $0;
  $120 = HEAP32[$119>>2]|0;
  $121 = HEAP32[$120>>2]|0;
  $122 = $0;
  FUNCTION_TABLE_vi[$121 & 127]($122);
 }
 $123 = $cmap_entries;
 $124 = ($123|0)>(0);
 if (!($124)) {
  STACKTOP = sp;return;
 }
 $125 = $0;
 $126 = $1;
 $127 = $cmap_entries;
 _write_colormap($125,$126,$127,3);
 STACKTOP = sp;return;
}
function _write_bmp_header($cinfo,$dest) {
 $cinfo = $cinfo|0;
 $dest = $dest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $bfSize = 0, $bits_per_pixel = 0, $bmpfileheader = 0, $bmpinfoheader = 0, $cmap_entries = 0, $headersize = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $bmpfileheader = sp + 64|0;
 $bmpinfoheader = sp + 24|0;
 $0 = $cinfo;
 $1 = $dest;
 $2 = $0;
 $3 = (($2) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(2);
 if ($5) {
  $6 = $0;
  $7 = (($6) + 84|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0);
  if ($9) {
   $bits_per_pixel = 8;
   $cmap_entries = 256;
  } else {
   $bits_per_pixel = 24;
   $cmap_entries = 0;
  }
 } else {
  $bits_per_pixel = 8;
  $cmap_entries = 256;
 }
 $10 = $cmap_entries;
 $11 = $10<<2;
 $12 = (54 + ($11))|0;
 $headersize = $12;
 $13 = $headersize;
 $14 = $1;
 $15 = (($14) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = (($17) + 116|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = Math_imul($16, $19)|0;
 $21 = (($13) + ($20))|0;
 $bfSize = $21;
 dest=$bmpfileheader+0|0; stop=dest+14|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 dest=$bmpinfoheader+0|0; stop=dest+40|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 HEAP8[$bmpfileheader>>0] = 66;
 $22 = (($bmpfileheader) + 1|0);
 HEAP8[$22>>0] = 77;
 $23 = $bfSize;
 $24 = $23 & 255;
 $25 = $24&255;
 $26 = (($bmpfileheader) + 2|0);
 HEAP8[$26>>0] = $25;
 $27 = $bfSize;
 $28 = $27 >> 8;
 $29 = $28 & 255;
 $30 = $29&255;
 $31 = (($bmpfileheader) + 3|0);
 HEAP8[$31>>0] = $30;
 $32 = $bfSize;
 $33 = $32 >> 16;
 $34 = $33 & 255;
 $35 = $34&255;
 $36 = (($bmpfileheader) + 4|0);
 HEAP8[$36>>0] = $35;
 $37 = $bfSize;
 $38 = $37 >> 24;
 $39 = $38 & 255;
 $40 = $39&255;
 $41 = (($bmpfileheader) + 5|0);
 HEAP8[$41>>0] = $40;
 $42 = $headersize;
 $43 = $42 & 255;
 $44 = $43&255;
 $45 = (($bmpfileheader) + 10|0);
 HEAP8[$45>>0] = $44;
 $46 = $headersize;
 $47 = $46 >> 8;
 $48 = $47 & 255;
 $49 = $48&255;
 $50 = (($bmpfileheader) + 11|0);
 HEAP8[$50>>0] = $49;
 $51 = $headersize;
 $52 = $51 >> 16;
 $53 = $52 & 255;
 $54 = $53&255;
 $55 = (($bmpfileheader) + 12|0);
 HEAP8[$55>>0] = $54;
 $56 = $headersize;
 $57 = $56 >> 24;
 $58 = $57 & 255;
 $59 = $58&255;
 $60 = (($bmpfileheader) + 13|0);
 HEAP8[$60>>0] = $59;
 HEAP8[$bmpinfoheader>>0] = 40;
 $61 = (($bmpinfoheader) + 1|0);
 HEAP8[$61>>0] = 0;
 $62 = $0;
 $63 = (($62) + 112|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = $64 & 255;
 $66 = $65&255;
 $67 = (($bmpinfoheader) + 4|0);
 HEAP8[$67>>0] = $66;
 $68 = $0;
 $69 = (($68) + 112|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $70 >>> 8;
 $72 = $71 & 255;
 $73 = $72&255;
 $74 = (($bmpinfoheader) + 5|0);
 HEAP8[$74>>0] = $73;
 $75 = $0;
 $76 = (($75) + 112|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = $77 >>> 16;
 $79 = $78 & 255;
 $80 = $79&255;
 $81 = (($bmpinfoheader) + 6|0);
 HEAP8[$81>>0] = $80;
 $82 = $0;
 $83 = (($82) + 112|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = $84 >>> 24;
 $86 = $85 & 255;
 $87 = $86&255;
 $88 = (($bmpinfoheader) + 7|0);
 HEAP8[$88>>0] = $87;
 $89 = $0;
 $90 = (($89) + 116|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $91 & 255;
 $93 = $92&255;
 $94 = (($bmpinfoheader) + 8|0);
 HEAP8[$94>>0] = $93;
 $95 = $0;
 $96 = (($95) + 116|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = $97 >>> 8;
 $99 = $98 & 255;
 $100 = $99&255;
 $101 = (($bmpinfoheader) + 9|0);
 HEAP8[$101>>0] = $100;
 $102 = $0;
 $103 = (($102) + 116|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = $104 >>> 16;
 $106 = $105 & 255;
 $107 = $106&255;
 $108 = (($bmpinfoheader) + 10|0);
 HEAP8[$108>>0] = $107;
 $109 = $0;
 $110 = (($109) + 116|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = $111 >>> 24;
 $113 = $112 & 255;
 $114 = $113&255;
 $115 = (($bmpinfoheader) + 11|0);
 HEAP8[$115>>0] = $114;
 $116 = (($bmpinfoheader) + 12|0);
 HEAP8[$116>>0] = 1;
 $117 = (($bmpinfoheader) + 13|0);
 HEAP8[$117>>0] = 0;
 $118 = $bits_per_pixel;
 $119 = $118 & 255;
 $120 = $119&255;
 $121 = (($bmpinfoheader) + 14|0);
 HEAP8[$121>>0] = $120;
 $122 = $bits_per_pixel;
 $123 = $122 >> 8;
 $124 = $123 & 255;
 $125 = $124&255;
 $126 = (($bmpinfoheader) + 15|0);
 HEAP8[$126>>0] = $125;
 $127 = $0;
 $128 = (($127) + 286|0);
 $129 = HEAP8[$128>>0]|0;
 $130 = $129&255;
 $131 = ($130|0)==(2);
 if ($131) {
  $132 = $0;
  $133 = (($132) + 288|0);
  $134 = HEAP16[$133>>1]|0;
  $135 = $134&65535;
  $136 = ($135*100)|0;
  $137 = $136 & 255;
  $138 = $137&255;
  $139 = (($bmpinfoheader) + 24|0);
  HEAP8[$139>>0] = $138;
  $140 = $0;
  $141 = (($140) + 288|0);
  $142 = HEAP16[$141>>1]|0;
  $143 = $142&65535;
  $144 = ($143*100)|0;
  $145 = $144 >> 8;
  $146 = $145 & 255;
  $147 = $146&255;
  $148 = (($bmpinfoheader) + 25|0);
  HEAP8[$148>>0] = $147;
  $149 = $0;
  $150 = (($149) + 288|0);
  $151 = HEAP16[$150>>1]|0;
  $152 = $151&65535;
  $153 = ($152*100)|0;
  $154 = $153 >> 16;
  $155 = $154 & 255;
  $156 = $155&255;
  $157 = (($bmpinfoheader) + 26|0);
  HEAP8[$157>>0] = $156;
  $158 = $0;
  $159 = (($158) + 288|0);
  $160 = HEAP16[$159>>1]|0;
  $161 = $160&65535;
  $162 = ($161*100)|0;
  $163 = $162 >> 24;
  $164 = $163 & 255;
  $165 = $164&255;
  $166 = (($bmpinfoheader) + 27|0);
  HEAP8[$166>>0] = $165;
  $167 = $0;
  $168 = (($167) + 290|0);
  $169 = HEAP16[$168>>1]|0;
  $170 = $169&65535;
  $171 = ($170*100)|0;
  $172 = $171 & 255;
  $173 = $172&255;
  $174 = (($bmpinfoheader) + 28|0);
  HEAP8[$174>>0] = $173;
  $175 = $0;
  $176 = (($175) + 290|0);
  $177 = HEAP16[$176>>1]|0;
  $178 = $177&65535;
  $179 = ($178*100)|0;
  $180 = $179 >> 8;
  $181 = $180 & 255;
  $182 = $181&255;
  $183 = (($bmpinfoheader) + 29|0);
  HEAP8[$183>>0] = $182;
  $184 = $0;
  $185 = (($184) + 290|0);
  $186 = HEAP16[$185>>1]|0;
  $187 = $186&65535;
  $188 = ($187*100)|0;
  $189 = $188 >> 16;
  $190 = $189 & 255;
  $191 = $190&255;
  $192 = (($bmpinfoheader) + 30|0);
  HEAP8[$192>>0] = $191;
  $193 = $0;
  $194 = (($193) + 290|0);
  $195 = HEAP16[$194>>1]|0;
  $196 = $195&65535;
  $197 = ($196*100)|0;
  $198 = $197 >> 24;
  $199 = $198 & 255;
  $200 = $199&255;
  $201 = (($bmpinfoheader) + 31|0);
  HEAP8[$201>>0] = $200;
 }
 $202 = $cmap_entries;
 $203 = $202 & 255;
 $204 = $203&255;
 $205 = (($bmpinfoheader) + 32|0);
 HEAP8[$205>>0] = $204;
 $206 = $cmap_entries;
 $207 = $206 >> 8;
 $208 = $207 & 255;
 $209 = $208&255;
 $210 = (($bmpinfoheader) + 33|0);
 HEAP8[$210>>0] = $209;
 $211 = $1;
 $212 = (($211) + 12|0);
 $213 = HEAP32[$212>>2]|0;
 $214 = (_fwrite(($bmpfileheader|0),1,14,($213|0))|0);
 $215 = ($214|0)!=(14);
 if ($215) {
  $216 = $0;
  $217 = HEAP32[$216>>2]|0;
  $218 = (($217) + 20|0);
  HEAP32[$218>>2] = 37;
  $219 = $0;
  $220 = HEAP32[$219>>2]|0;
  $221 = HEAP32[$220>>2]|0;
  $222 = $0;
  FUNCTION_TABLE_vi[$221 & 127]($222);
 }
 $223 = $1;
 $224 = (($223) + 12|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (_fwrite(($bmpinfoheader|0),1,40,($225|0))|0);
 $227 = ($226|0)!=(40);
 if ($227) {
  $228 = $0;
  $229 = HEAP32[$228>>2]|0;
  $230 = (($229) + 20|0);
  HEAP32[$230>>2] = 37;
  $231 = $0;
  $232 = HEAP32[$231>>2]|0;
  $233 = HEAP32[$232>>2]|0;
  $234 = $0;
  FUNCTION_TABLE_vi[$233 & 127]($234);
 }
 $235 = $cmap_entries;
 $236 = ($235|0)>(0);
 if (!($236)) {
  STACKTOP = sp;return;
 }
 $237 = $0;
 $238 = $1;
 $239 = $cmap_entries;
 _write_colormap($237,$238,$239,4);
 STACKTOP = sp;return;
}
function _write_colormap($cinfo,$dest,$map_colors,$map_entry_size) {
 $cinfo = $cinfo|0;
 $dest = $dest|0;
 $map_colors = $map_colors|0;
 $map_entry_size = $map_entry_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $colormap = 0, $i = 0, $num_colors = 0;
 var $outfile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $dest;
 $2 = $map_colors;
 $3 = $map_entry_size;
 $4 = $0;
 $5 = (($4) + 136|0);
 $6 = HEAP32[$5>>2]|0;
 $colormap = $6;
 $7 = $0;
 $8 = (($7) + 132|0);
 $9 = HEAP32[$8>>2]|0;
 $num_colors = $9;
 $10 = $1;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $outfile = $12;
 $13 = $colormap;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = $0;
  $16 = (($15) + 120|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(3);
  if ($18) {
   $i = 0;
   while(1) {
    $19 = $i;
    $20 = $num_colors;
    $21 = ($19|0)<($20|0);
    if (!($21)) {
     break;
    }
    $22 = $i;
    $23 = $colormap;
    $24 = (($23) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + ($22)|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = $outfile;
    (_putc(($28|0),($29|0))|0);
    $30 = $i;
    $31 = $colormap;
    $32 = (($31) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + ($30)|0);
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = $outfile;
    (_putc(($36|0),($37|0))|0);
    $38 = $i;
    $39 = $colormap;
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + ($38)|0);
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $outfile;
    (_putc(($43|0),($44|0))|0);
    $45 = $3;
    $46 = ($45|0)==(4);
    if ($46) {
     $47 = $outfile;
     (_putc(0,($47|0))|0);
    }
    $48 = $i;
    $49 = (($48) + 1)|0;
    $i = $49;
   }
  } else {
   $i = 0;
   while(1) {
    $50 = $i;
    $51 = $num_colors;
    $52 = ($50|0)<($51|0);
    if (!($52)) {
     break;
    }
    $53 = $i;
    $54 = $colormap;
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($53)|0);
    $57 = HEAP8[$56>>0]|0;
    $58 = $57&255;
    $59 = $outfile;
    (_putc(($58|0),($59|0))|0);
    $60 = $i;
    $61 = $colormap;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($60)|0);
    $64 = HEAP8[$63>>0]|0;
    $65 = $64&255;
    $66 = $outfile;
    (_putc(($65|0),($66|0))|0);
    $67 = $i;
    $68 = $colormap;
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + ($67)|0);
    $71 = HEAP8[$70>>0]|0;
    $72 = $71&255;
    $73 = $outfile;
    (_putc(($72|0),($73|0))|0);
    $74 = $3;
    $75 = ($74|0)==(4);
    if ($75) {
     $76 = $outfile;
     (_putc(0,($76|0))|0);
    }
    $77 = $i;
    $78 = (($77) + 1)|0;
    $i = $78;
   }
  }
 } else {
  $i = 0;
  while(1) {
   $79 = $i;
   $80 = ($79|0)<(256);
   if (!($80)) {
    break;
   }
   $81 = $i;
   $82 = $outfile;
   (_putc(($81|0),($82|0))|0);
   $83 = $i;
   $84 = $outfile;
   (_putc(($83|0),($84|0))|0);
   $85 = $i;
   $86 = $outfile;
   (_putc(($85|0),($86|0))|0);
   $87 = $3;
   $88 = ($87|0)==(4);
   if ($88) {
    $89 = $outfile;
    (_putc(0,($89|0))|0);
   }
   $90 = $i;
   $91 = (($90) + 1)|0;
   $i = $91;
  }
 }
 $92 = $i;
 $93 = $2;
 $94 = ($92|0)>($93|0);
 if ($94) {
  $95 = $0;
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 20|0);
  HEAP32[$97>>2] = 1039;
  $98 = $i;
  $99 = $0;
  $100 = HEAP32[$99>>2]|0;
  $101 = (($100) + 24|0);
  HEAP32[$101>>2] = $98;
  $102 = $0;
  $103 = HEAP32[$102>>2]|0;
  $104 = HEAP32[$103>>2]|0;
  $105 = $0;
  FUNCTION_TABLE_vi[$104 & 127]($105);
 }
 while(1) {
  $106 = $i;
  $107 = $2;
  $108 = ($106|0)<($107|0);
  if (!($108)) {
   break;
  }
  $109 = $outfile;
  (_putc(0,($109|0))|0);
  $110 = $outfile;
  (_putc(0,($110|0))|0);
  $111 = $outfile;
  (_putc(0,($111|0))|0);
  $112 = $3;
  $113 = ($112|0)==(4);
  if ($113) {
   $114 = $outfile;
   (_putc(0,($114|0))|0);
  }
  $115 = $i;
  $116 = (($115) + 1)|0;
  $i = $116;
 }
 STACKTOP = sp;return;
}
function _read_color_map($cinfo,$infile) {
 $cinfo = $cinfo|0;
 $infile = $infile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $infile;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (FUNCTION_TABLE_iiiii[$6 & 127]($7,1,256,3)|0);
 $9 = $0;
 $10 = (($9) + 136|0);
 HEAP32[$10>>2] = $8;
 $11 = $0;
 $12 = (($11) + 132|0);
 HEAP32[$12>>2] = 0;
 $13 = $1;
 $14 = (_getc(($13|0))|0);
 if ((($14|0) == 71)) {
  $15 = $0;
  $16 = $1;
  _read_gif_map($15,$16);
  STACKTOP = sp;return;
 } else if ((($14|0) == 80)) {
  $17 = $0;
  $18 = $1;
  _read_ppm_map($17,$18);
  STACKTOP = sp;return;
 } else {
  $19 = $0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 1038;
  $22 = $0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = $0;
  FUNCTION_TABLE_vi[$24 & 127]($25);
  STACKTOP = sp;return;
 }
}
function _read_gif_map($cinfo,$infile) {
 $cinfo = $cinfo|0;
 $infile = $infile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $B = 0, $G = 0, $R = 0;
 var $colormaplen = 0, $header = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $header = sp + 20|0;
 $0 = $cinfo;
 $1 = $infile;
 $i = 1;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(13);
  if (!($3)) {
   break;
  }
  $4 = $1;
  $5 = (_getc(($4|0))|0);
  $6 = $i;
  $7 = (($header) + ($6<<2)|0);
  HEAP32[$7>>2] = $5;
  $8 = ($5|0)==(-1);
  if ($8) {
   $9 = $0;
   $10 = HEAP32[$9>>2]|0;
   $11 = (($10) + 20|0);
   HEAP32[$11>>2] = 1038;
   $12 = $0;
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = $0;
   FUNCTION_TABLE_vi[$14 & 127]($15);
  }
  $16 = $i;
  $17 = (($16) + 1)|0;
  $i = $17;
 }
 $18 = (($header) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(73);
 if ($20) {
  label = 9;
 } else {
  $21 = (($header) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(70);
  if ($23) {
   label = 9;
  }
 }
 if ((label|0) == 9) {
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 20|0);
  HEAP32[$26>>2] = 1038;
  $27 = $0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = $0;
  FUNCTION_TABLE_vi[$29 & 127]($30);
 }
 $31 = (($header) + 40|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = $32 & 128;
 $34 = ($33|0)==(0);
 if ($34) {
  $35 = $0;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + 20|0);
  HEAP32[$37>>2] = 1038;
  $38 = $0;
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = $0;
  FUNCTION_TABLE_vi[$40 & 127]($41);
 }
 $42 = (($header) + 40|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $43 & 7;
 $45 = 2 << $44;
 $colormaplen = $45;
 $i = 0;
 while(1) {
  $46 = $i;
  $47 = $colormaplen;
  $48 = ($46|0)<($47|0);
  if (!($48)) {
   break;
  }
  $49 = $1;
  $50 = (_getc(($49|0))|0);
  $R = $50;
  $51 = $1;
  $52 = (_getc(($51|0))|0);
  $G = $52;
  $53 = $1;
  $54 = (_getc(($53|0))|0);
  $B = $54;
  $55 = $R;
  $56 = ($55|0)==(-1);
  if ($56) {
   label = 17;
  } else {
   $57 = $G;
   $58 = ($57|0)==(-1);
   if ($58) {
    label = 17;
   } else {
    $59 = $B;
    $60 = ($59|0)==(-1);
    if ($60) {
     label = 17;
    }
   }
  }
  if ((label|0) == 17) {
   label = 0;
   $61 = $0;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + 20|0);
   HEAP32[$63>>2] = 1038;
   $64 = $0;
   $65 = HEAP32[$64>>2]|0;
   $66 = HEAP32[$65>>2]|0;
   $67 = $0;
   FUNCTION_TABLE_vi[$66 & 127]($67);
  }
  $68 = $0;
  $69 = $R;
  $70 = $69 << 0;
  $71 = $G;
  $72 = $71 << 0;
  $73 = $B;
  $74 = $73 << 0;
  _add_map_entry($68,$70,$72,$74);
  $75 = $i;
  $76 = (($75) + 1)|0;
  $i = $76;
 }
 STACKTOP = sp;return;
}
function _read_ppm_map($cinfo,$infile) {
 $cinfo = $cinfo|0;
 $infile = $infile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $B = 0;
 var $G = 0, $R = 0, $c = 0, $col = 0, $h = 0, $maxval = 0, $row = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $infile;
 $2 = $1;
 $3 = (_getc(($2|0))|0);
 $c = $3;
 $4 = $0;
 $5 = $1;
 $6 = (_read_pbm_integer($4,$5)|0);
 $w = $6;
 $7 = $0;
 $8 = $1;
 $9 = (_read_pbm_integer($7,$8)|0);
 $h = $9;
 $10 = $0;
 $11 = $1;
 $12 = (_read_pbm_integer($10,$11)|0);
 $maxval = $12;
 $13 = $w;
 $14 = ($13>>>0)<=(0);
 if ($14) {
  label = 4;
 } else {
  $15 = $h;
  $16 = ($15>>>0)<=(0);
  if ($16) {
   label = 4;
  } else {
   $17 = $maxval;
   $18 = ($17>>>0)<=(0);
   if ($18) {
    label = 4;
   }
  }
 }
 if ((label|0) == 4) {
  $19 = $0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 1038;
  $22 = $0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = $0;
  FUNCTION_TABLE_vi[$24 & 127]($25);
 }
 $26 = $maxval;
 $27 = ($26|0)!=(255);
 if ($27) {
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 20|0);
  HEAP32[$30>>2] = 1038;
  $31 = $0;
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = $0;
  FUNCTION_TABLE_vi[$33 & 127]($34);
 }
 $35 = $c;
 if ((($35|0) == 54)) {
  $row = 0;
  while(1) {
   $59 = $row;
   $60 = $h;
   $61 = ($59>>>0)<($60>>>0);
   if (!($61)) {
    break;
   }
   $col = 0;
   while(1) {
    $62 = $col;
    $63 = $w;
    $64 = ($62>>>0)<($63>>>0);
    if (!($64)) {
     break;
    }
    $65 = $1;
    $66 = (_getc(($65|0))|0);
    $R = $66;
    $67 = $1;
    $68 = (_getc(($67|0))|0);
    $G = $68;
    $69 = $1;
    $70 = (_getc(($69|0))|0);
    $B = $70;
    $71 = $R;
    $72 = ($71|0)==(-1);
    if ($72) {
     label = 24;
    } else {
     $73 = $G;
     $74 = ($73|0)==(-1);
     if ($74) {
      label = 24;
     } else {
      $75 = $B;
      $76 = ($75|0)==(-1);
      if ($76) {
       label = 24;
      }
     }
    }
    if ((label|0) == 24) {
     label = 0;
     $77 = $0;
     $78 = HEAP32[$77>>2]|0;
     $79 = (($78) + 20|0);
     HEAP32[$79>>2] = 1038;
     $80 = $0;
     $81 = HEAP32[$80>>2]|0;
     $82 = HEAP32[$81>>2]|0;
     $83 = $0;
     FUNCTION_TABLE_vi[$82 & 127]($83);
    }
    $84 = $0;
    $85 = $R;
    $86 = $G;
    $87 = $B;
    _add_map_entry($84,$85,$86,$87);
    $88 = $col;
    $89 = (($88) + 1)|0;
    $col = $89;
   }
   $90 = $row;
   $91 = (($90) + 1)|0;
   $row = $91;
  }
  STACKTOP = sp;return;
 } else if ((($35|0) == 51)) {
  $row = 0;
  while(1) {
   $36 = $row;
   $37 = $h;
   $38 = ($36>>>0)<($37>>>0);
   if (!($38)) {
    break;
   }
   $col = 0;
   while(1) {
    $39 = $col;
    $40 = $w;
    $41 = ($39>>>0)<($40>>>0);
    if (!($41)) {
     break;
    }
    $42 = $0;
    $43 = $1;
    $44 = (_read_pbm_integer($42,$43)|0);
    $R = $44;
    $45 = $0;
    $46 = $1;
    $47 = (_read_pbm_integer($45,$46)|0);
    $G = $47;
    $48 = $0;
    $49 = $1;
    $50 = (_read_pbm_integer($48,$49)|0);
    $B = $50;
    $51 = $0;
    $52 = $R;
    $53 = $G;
    $54 = $B;
    _add_map_entry($51,$52,$53,$54);
    $55 = $col;
    $56 = (($55) + 1)|0;
    $col = $56;
   }
   $57 = $row;
   $58 = (($57) + 1)|0;
   $row = $58;
  }
  STACKTOP = sp;return;
 } else {
  $92 = $0;
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + 20|0);
  HEAP32[$94>>2] = 1038;
  $95 = $0;
  $96 = HEAP32[$95>>2]|0;
  $97 = HEAP32[$96>>2]|0;
  $98 = $0;
  FUNCTION_TABLE_vi[$97 & 127]($98);
  STACKTOP = sp;return;
 }
}
function _read_pbm_integer($cinfo,$infile) {
 $cinfo = $cinfo|0;
 $infile = $infile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $infile;
 while(1) {
  $2 = $1;
  $3 = (_pbm_getc($2)|0);
  $ch = $3;
  $4 = $ch;
  $5 = ($4|0)==(-1);
  if ($5) {
   $6 = $0;
   $7 = HEAP32[$6>>2]|0;
   $8 = (($7) + 20|0);
   HEAP32[$8>>2] = 1038;
   $9 = $0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   $12 = $0;
   FUNCTION_TABLE_vi[$11 & 127]($12);
  }
  $13 = $ch;
  $14 = ($13|0)==(32);
  if ($14) {
   $46 = 1;
  } else {
   $15 = $ch;
   $16 = ($15|0)==(9);
   if ($16) {
    $46 = 1;
   } else {
    $17 = $ch;
    $18 = ($17|0)==(10);
    if ($18) {
     $46 = 1;
    } else {
     $19 = $ch;
     $20 = ($19|0)==(13);
     $46 = $20;
    }
   }
  }
  if (!($46)) {
   break;
  }
 }
 $21 = $ch;
 $22 = ($21|0)<(48);
 if ($22) {
  label = 12;
 } else {
  $23 = $ch;
  $24 = ($23|0)>(57);
  if ($24) {
   label = 12;
  }
 }
 if ((label|0) == 12) {
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 20|0);
  HEAP32[$27>>2] = 1038;
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vi[$30 & 127]($31);
 }
 $32 = $ch;
 $33 = (($32) - 48)|0;
 $val = $33;
 while(1) {
  $34 = $1;
  $35 = (_pbm_getc($34)|0);
  $ch = $35;
  $36 = ($35|0)>=(48);
  if ($36) {
   $37 = $ch;
   $38 = ($37|0)<=(57);
   $47 = $38;
  } else {
   $47 = 0;
  }
  if (!($47)) {
   break;
  }
  $39 = $val;
  $40 = ($39*10)|0;
  $val = $40;
  $41 = $ch;
  $42 = (($41) - 48)|0;
  $43 = $val;
  $44 = (($43) + ($42))|0;
  $val = $44;
 }
 $45 = $val;
 STACKTOP = sp;return ($45|0);
}
function _add_map_entry($cinfo,$R,$G,$B) {
 $cinfo = $cinfo|0;
 $R = $R|0;
 $G = $G|0;
 $B = $B|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $colormap0 = 0, $colormap1 = 0;
 var $colormap2 = 0, $index = 0, $ncolors = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $R;
 $2 = $G;
 $3 = $B;
 $4 = $0;
 $5 = (($4) + 136|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $colormap0 = $7;
 $8 = $0;
 $9 = (($8) + 136|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $colormap1 = $12;
 $13 = $0;
 $14 = (($13) + 136|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $colormap2 = $17;
 $18 = $0;
 $19 = (($18) + 132|0);
 $20 = HEAP32[$19>>2]|0;
 $ncolors = $20;
 $index = 0;
 while(1) {
  $21 = $index;
  $22 = $ncolors;
  $23 = ($21|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = $index;
  $25 = $colormap0;
  $26 = (($25) + ($24)|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  $29 = $1;
  $30 = ($28|0)==($29|0);
  if ($30) {
   $31 = $index;
   $32 = $colormap1;
   $33 = (($32) + ($31)|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = $34&255;
   $36 = $2;
   $37 = ($35|0)==($36|0);
   if ($37) {
    $38 = $index;
    $39 = $colormap2;
    $40 = (($39) + ($38)|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    $43 = $3;
    $44 = ($42|0)==($43|0);
    if ($44) {
     label = 6;
     break;
    }
   }
  }
  $45 = $index;
  $46 = (($45) + 1)|0;
  $index = $46;
 }
 if ((label|0) == 6) {
  STACKTOP = sp;return;
 }
 $47 = $ncolors;
 $48 = ($47|0)>=(256);
 if ($48) {
  $49 = $0;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 20|0);
  HEAP32[$51>>2] = 57;
  $52 = $0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + 24|0);
  HEAP32[$54>>2] = 256;
  $55 = $0;
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = $0;
  FUNCTION_TABLE_vi[$57 & 127]($58);
 }
 $59 = $1;
 $60 = $59&255;
 $61 = $ncolors;
 $62 = $colormap0;
 $63 = (($62) + ($61)|0);
 HEAP8[$63>>0] = $60;
 $64 = $2;
 $65 = $64&255;
 $66 = $ncolors;
 $67 = $colormap1;
 $68 = (($67) + ($66)|0);
 HEAP8[$68>>0] = $65;
 $69 = $3;
 $70 = $69&255;
 $71 = $ncolors;
 $72 = $colormap2;
 $73 = (($72) + ($71)|0);
 HEAP8[$73>>0] = $70;
 $74 = $0;
 $75 = (($74) + 132|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = (($76) + 1)|0;
 HEAP32[$75>>2] = $77;
 STACKTOP = sp;return;
}
function _pbm_getc($infile) {
 $infile = $infile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $infile;
 $1 = $0;
 $2 = (_getc(($1|0))|0);
 $ch = $2;
 $3 = $ch;
 $4 = ($3|0)==(35);
 if (!($4)) {
  $11 = $ch;
  STACKTOP = sp;return ($11|0);
 }
 while(1) {
  $5 = $0;
  $6 = (_getc(($5|0))|0);
  $ch = $6;
  $7 = $ch;
  $8 = ($7|0)!=(10);
  if ($8) {
   $9 = $ch;
   $10 = ($9|0)!=(-1);
   $12 = $10;
  } else {
   $12 = 0;
  }
  if (!($12)) {
   break;
  }
 }
 $11 = $ch;
 STACKTOP = sp;return ($11|0);
}
function _keymatch($arg,$keyword,$minchars) {
 $arg = $arg|0;
 $keyword = $keyword|0;
 $minchars = $minchars|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ca = 0, $ck = 0, $nmatched = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $arg;
 $2 = $keyword;
 $3 = $minchars;
 $nmatched = 0;
 while(1) {
  $4 = $1;
  $5 = (($4) + 1|0);
  $1 = $5;
  $6 = HEAP8[$4>>0]|0;
  $7 = $6 << 24 >> 24;
  $ca = $7;
  $8 = ($7|0)!=(0);
  if (!($8)) {
   label = 10;
   break;
  }
  $9 = $2;
  $10 = (($9) + 1|0);
  $2 = $10;
  $11 = HEAP8[$9>>0]|0;
  $12 = $11 << 24 >> 24;
  $ck = $12;
  $13 = ($12|0)==(0);
  if ($13) {
   label = 4;
   break;
  }
  $14 = $ca;
  $15 = (_isupper($14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = $ca;
   $18 = (_tolower($17)|0);
   $ca = $18;
  }
  $19 = $ca;
  $20 = $ck;
  $21 = ($19|0)!=($20|0);
  if ($21) {
   label = 8;
   break;
  }
  $22 = $nmatched;
  $23 = (($22) + 1)|0;
  $nmatched = $23;
 }
 if ((label|0) == 4) {
  $0 = 0;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 else if ((label|0) == 8) {
  $0 = 0;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 else if ((label|0) == 10) {
  $24 = $nmatched;
  $25 = $3;
  $26 = ($24|0)<($25|0);
  if ($26) {
   $0 = 0;
   $27 = $0;
   STACKTOP = sp;return ($27|0);
  } else {
   $0 = 1;
   $27 = $0;
   STACKTOP = sp;return ($27|0);
  }
 }
 return 0|0;
}
function _read_stdin() {
 var $0 = 0, $1 = 0, $input_file = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = HEAP32[_stdin>>2]|0;
 $input_file = $0;
 $1 = $input_file;
 STACKTOP = sp;return ($1|0);
}
function _write_stdout() {
 var $0 = 0, $1 = 0, $output_file = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = HEAP32[_stdout>>2]|0;
 $output_file = $0;
 $1 = $output_file;
 STACKTOP = sp;return ($1|0);
}
function _jpeg_CreateDecompress($cinfo,$version,$structsize) {
 $cinfo = $cinfo|0;
 $version = $version|0;
 $structsize = $structsize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $client_data = 0, $err = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $version;
 $2 = $structsize;
 $3 = $0;
 $4 = (($3) + 4|0);
 HEAP32[$4>>2] = 0;
 $5 = $1;
 $6 = ($5|0)!=(62);
 if ($6) {
  $7 = $0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 20|0);
  HEAP32[$9>>2] = 12;
  $10 = $0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 24|0);
  HEAP32[$12>>2] = 62;
  $13 = $1;
  $14 = $0;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 24|0);
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = $13;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 127]($21);
 }
 $22 = $2;
 $23 = ($22|0)!=(464);
 if ($23) {
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 20|0);
  HEAP32[$26>>2] = 21;
  $27 = $0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 24|0);
  HEAP32[$29>>2] = 464;
  $30 = $2;
  $31 = $0;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 24|0);
  $34 = (($33) + 4|0);
  HEAP32[$34>>2] = $30;
  $35 = $0;
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = $0;
  FUNCTION_TABLE_vi[$37 & 127]($38);
 }
 $39 = $0;
 $40 = HEAP32[$39>>2]|0;
 $err = $40;
 $41 = $0;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $client_data = $43;
 $44 = $0;
 _memset(($44|0),0,464)|0;
 $45 = $err;
 $46 = $0;
 HEAP32[$46>>2] = $45;
 $47 = $client_data;
 $48 = $0;
 $49 = (($48) + 12|0);
 HEAP32[$49>>2] = $47;
 $50 = $0;
 $51 = (($50) + 16|0);
 HEAP32[$51>>2] = 1;
 $52 = $0;
 _jinit_memory_mgr($52);
 $53 = $0;
 $54 = (($53) + 8|0);
 HEAP32[$54>>2] = 0;
 $55 = $0;
 $56 = (($55) + 24|0);
 HEAP32[$56>>2] = 0;
 $i = 0;
 while(1) {
  $57 = $i;
  $58 = ($57|0)<(4);
  if (!($58)) {
   break;
  }
  $59 = $i;
  $60 = $0;
  $61 = (($60) + 164|0);
  $62 = (($61) + ($59<<2)|0);
  HEAP32[$62>>2] = 0;
  $63 = $i;
  $64 = (($63) + 1)|0;
  $i = $64;
 }
 $i = 0;
 while(1) {
  $65 = $i;
  $66 = ($65|0)<(4);
  if (!($66)) {
   break;
  }
  $67 = $i;
  $68 = $0;
  $69 = (($68) + 180|0);
  $70 = (($69) + ($67<<2)|0);
  HEAP32[$70>>2] = 0;
  $71 = $i;
  $72 = $0;
  $73 = (($72) + 196|0);
  $74 = (($73) + ($71<<2)|0);
  HEAP32[$74>>2] = 0;
  $75 = $i;
  $76 = (($75) + 1)|0;
  $i = $76;
 }
 $77 = $0;
 $78 = (($77) + 304|0);
 HEAP32[$78>>2] = 0;
 $79 = $0;
 _jinit_marker_reader($79);
 $80 = $0;
 _jinit_input_controller($80);
 $81 = $0;
 $82 = (($81) + 20|0);
 HEAP32[$82>>2] = 200;
 STACKTOP = sp;return;
}
function _jpeg_destroy_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 _jpeg_destroy($1);
 STACKTOP = sp;return;
}
function _jpeg_read_header($cinfo,$require_image) {
 $cinfo = $cinfo|0;
 $require_image = $require_image|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $retcode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $require_image;
 $2 = $0;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(200);
 if ($5) {
  $6 = $0;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(201);
  if ($9) {
   $10 = $0;
   $11 = HEAP32[$10>>2]|0;
   $12 = (($11) + 20|0);
   HEAP32[$12>>2] = 20;
   $13 = $0;
   $14 = (($13) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + 24|0);
   HEAP32[$18>>2] = $15;
   $19 = $0;
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = $0;
   FUNCTION_TABLE_vi[$21 & 127]($22);
  }
 }
 $23 = $0;
 $24 = (_jpeg_consume_input($23)|0);
 $retcode = $24;
 $25 = $retcode;
 if ((($25|0) == 2)) {
  $26 = $1;
  $27 = ($26|0)!=(0);
  if ($27) {
   $28 = $0;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 20|0);
   HEAP32[$30>>2] = 51;
   $31 = $0;
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = $0;
   FUNCTION_TABLE_vi[$33 & 127]($34);
  }
  $35 = $0;
  _jpeg_abort($35);
  $retcode = 2;
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 } else if ((($25|0) == 1)) {
  $retcode = 1;
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 } else if ((($25|0) == 0)) {
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 } else {
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _jpeg_consume_input($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $retcode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $retcode = 0;
 $1 = $0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 switch ($3|0) {
 case 201:  {
  break;
 }
 case 202:  {
  $retcode = 1;
  $48 = $retcode;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 200:  {
  $4 = $0;
  $5 = (($4) + 436|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $0;
  FUNCTION_TABLE_vi[$8 & 127]($9);
  $10 = $0;
  $11 = (($10) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $0;
  FUNCTION_TABLE_vi[$14 & 127]($15);
  $16 = $0;
  $17 = (($16) + 20|0);
  HEAP32[$17>>2] = 201;
  break;
 }
 case 210: case 208: case 207: case 206: case 205: case 204: case 203:  {
  $29 = $0;
  $30 = (($29) + 436|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$31>>2]|0;
  $33 = $0;
  $34 = (FUNCTION_TABLE_ii[$32 & 127]($33)|0);
  $retcode = $34;
  $48 = $retcode;
  STACKTOP = sp;return ($48|0);
  break;
 }
 default: {
  $35 = $0;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + 20|0);
  HEAP32[$37>>2] = 20;
  $38 = $0;
  $39 = (($38) + 20|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 24|0);
  HEAP32[$43>>2] = $40;
  $44 = $0;
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $0;
  FUNCTION_TABLE_vi[$46 & 127]($47);
  $48 = $retcode;
  STACKTOP = sp;return ($48|0);
 }
 }
 $18 = $0;
 $19 = (($18) + 436|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = $0;
 $23 = (FUNCTION_TABLE_ii[$21 & 127]($22)|0);
 $retcode = $23;
 $24 = $retcode;
 $25 = ($24|0)==(1);
 if ($25) {
  $26 = $0;
  _default_decompress_parms($26);
  $27 = $0;
  $28 = (($27) + 20|0);
  HEAP32[$28>>2] = 202;
 }
 $48 = $retcode;
 STACKTOP = sp;return ($48|0);
}
function _default_decompress_parms($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $cid0 = 0, $cid1 = 0, $cid2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 36|0);
 $3 = HEAP32[$2>>2]|0;
 if ((($3|0) == 3)) {
  $8 = $0;
  $9 = (($8) + 280|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0);
  if ($11) {
   $12 = $0;
   $13 = (($12) + 40|0);
   HEAP32[$13>>2] = 3;
  } else {
   $14 = $0;
   $15 = (($14) + 292|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $0;
    $19 = (($18) + 296|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    if ((($21|0) == 1)) {
     $24 = $0;
     $25 = (($24) + 40|0);
     HEAP32[$25>>2] = 3;
    } else if ((($21|0) == 0)) {
     $22 = $0;
     $23 = (($22) + 40|0);
     HEAP32[$23>>2] = 2;
    } else {
     $26 = $0;
     $27 = HEAP32[$26>>2]|0;
     $28 = (($27) + 20|0);
     HEAP32[$28>>2] = 114;
     $29 = $0;
     $30 = (($29) + 296|0);
     $31 = HEAP8[$30>>0]|0;
     $32 = $31&255;
     $33 = $0;
     $34 = HEAP32[$33>>2]|0;
     $35 = (($34) + 24|0);
     HEAP32[$35>>2] = $32;
     $36 = $0;
     $37 = HEAP32[$36>>2]|0;
     $38 = (($37) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = $0;
     FUNCTION_TABLE_vii[$39 & 127]($40,-1);
     $41 = $0;
     $42 = (($41) + 40|0);
     HEAP32[$42>>2] = 3;
    }
   } else {
    $43 = $0;
    $44 = (($43) + 216|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$45>>2]|0;
    $cid0 = $46;
    $47 = $0;
    $48 = (($47) + 216|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 84|0);
    $51 = HEAP32[$50>>2]|0;
    $cid1 = $51;
    $52 = $0;
    $53 = (($52) + 216|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + 168|0);
    $56 = HEAP32[$55>>2]|0;
    $cid2 = $56;
    $57 = $cid0;
    $58 = ($57|0)==(1);
    if ($58) {
     $59 = $cid1;
     $60 = ($59|0)==(2);
     if ($60) {
      $61 = $cid2;
      $62 = ($61|0)==(3);
      if ($62) {
       $63 = $0;
       $64 = (($63) + 40|0);
       HEAP32[$64>>2] = 3;
      } else {
       label = 15;
      }
     } else {
      label = 15;
     }
    } else {
     label = 15;
    }
    if ((label|0) == 15) {
     $65 = $cid0;
     $66 = ($65|0)==(82);
     if ($66) {
      $67 = $cid1;
      $68 = ($67|0)==(71);
      if ($68) {
       $69 = $cid2;
       $70 = ($69|0)==(66);
       if ($70) {
        $71 = $0;
        $72 = (($71) + 40|0);
        HEAP32[$72>>2] = 2;
       } else {
        label = 19;
       }
      } else {
       label = 19;
      }
     } else {
      label = 19;
     }
     if ((label|0) == 19) {
      $73 = $0;
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + 24|0);
      $_mp = $75;
      $76 = $cid0;
      $77 = $_mp;
      HEAP32[$77>>2] = $76;
      $78 = $cid1;
      $79 = $_mp;
      $80 = (($79) + 4|0);
      HEAP32[$80>>2] = $78;
      $81 = $cid2;
      $82 = $_mp;
      $83 = (($82) + 8|0);
      HEAP32[$83>>2] = $81;
      $84 = $0;
      $85 = HEAP32[$84>>2]|0;
      $86 = (($85) + 20|0);
      HEAP32[$86>>2] = 111;
      $87 = $0;
      $88 = HEAP32[$87>>2]|0;
      $89 = (($88) + 4|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = $0;
      FUNCTION_TABLE_vii[$90 & 127]($91,1);
      $92 = $0;
      $93 = (($92) + 40|0);
      HEAP32[$93>>2] = 3;
     }
    }
   }
  }
  $94 = $0;
  $95 = (($94) + 44|0);
  HEAP32[$95>>2] = 2;
 } else if ((($3|0) == 4)) {
  $96 = $0;
  $97 = (($96) + 292|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = ($98|0)!=(0);
  if ($99) {
   $100 = $0;
   $101 = (($100) + 296|0);
   $102 = HEAP8[$101>>0]|0;
   $103 = $102&255;
   if ((($103|0) == 0)) {
    $104 = $0;
    $105 = (($104) + 40|0);
    HEAP32[$105>>2] = 4;
   } else if ((($103|0) == 2)) {
    $106 = $0;
    $107 = (($106) + 40|0);
    HEAP32[$107>>2] = 5;
   } else {
    $108 = $0;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 20|0);
    HEAP32[$110>>2] = 114;
    $111 = $0;
    $112 = (($111) + 296|0);
    $113 = HEAP8[$112>>0]|0;
    $114 = $113&255;
    $115 = $0;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($116) + 24|0);
    HEAP32[$117>>2] = $114;
    $118 = $0;
    $119 = HEAP32[$118>>2]|0;
    $120 = (($119) + 4|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = $0;
    FUNCTION_TABLE_vii[$121 & 127]($122,-1);
    $123 = $0;
    $124 = (($123) + 40|0);
    HEAP32[$124>>2] = 5;
   }
  } else {
   $125 = $0;
   $126 = (($125) + 40|0);
   HEAP32[$126>>2] = 4;
  }
  $127 = $0;
  $128 = (($127) + 44|0);
  HEAP32[$128>>2] = 4;
 } else if ((($3|0) == 1)) {
  $4 = $0;
  $5 = (($4) + 40|0);
  HEAP32[$5>>2] = 1;
  $6 = $0;
  $7 = (($6) + 44|0);
  HEAP32[$7>>2] = 1;
 } else {
  $129 = $0;
  $130 = (($129) + 40|0);
  HEAP32[$130>>2] = 0;
  $131 = $0;
  $132 = (($131) + 44|0);
  HEAP32[$132>>2] = 0;
 }
 $133 = $0;
 $134 = (($133) + 48|0);
 HEAP32[$134>>2] = 1;
 $135 = $0;
 $136 = (($135) + 52|0);
 HEAP32[$136>>2] = 1;
 $137 = $0;
 $138 = (($137) + 56|0);
 HEAPF64[$138>>3] = 1.0;
 $139 = $0;
 $140 = (($139) + 64|0);
 HEAP32[$140>>2] = 0;
 $141 = $0;
 $142 = (($141) + 68|0);
 HEAP32[$142>>2] = 0;
 $143 = $0;
 $144 = (($143) + 72|0);
 HEAP32[$144>>2] = 0;
 $145 = $0;
 $146 = (($145) + 76|0);
 HEAP32[$146>>2] = 1;
 $147 = $0;
 $148 = (($147) + 80|0);
 HEAP32[$148>>2] = 1;
 $149 = $0;
 $150 = (($149) + 84|0);
 HEAP32[$150>>2] = 0;
 $151 = $0;
 $152 = (($151) + 88|0);
 HEAP32[$152>>2] = 2;
 $153 = $0;
 $154 = (($153) + 92|0);
 HEAP32[$154>>2] = 1;
 $155 = $0;
 $156 = (($155) + 96|0);
 HEAP32[$156>>2] = 256;
 $157 = $0;
 $158 = (($157) + 136|0);
 HEAP32[$158>>2] = 0;
 $159 = $0;
 $160 = (($159) + 100|0);
 HEAP32[$160>>2] = 0;
 $161 = $0;
 $162 = (($161) + 104|0);
 HEAP32[$162>>2] = 0;
 $163 = $0;
 $164 = (($163) + 108|0);
 HEAP32[$164>>2] = 0;
 STACKTOP = sp;return;
}
function _jpeg_finish_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(205);
 if ($5) {
  label = 3;
 } else {
  $6 = $1;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(206);
  if ($9) {
   label = 3;
  } else {
   label = 7;
  }
 }
 if ((label|0) == 3) {
  $10 = $1;
  $11 = (($10) + 64|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   label = 7;
  } else {
   $14 = $1;
   $15 = (($14) + 140|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $1;
   $18 = (($17) + 116|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16>>>0)<($19>>>0);
   if ($20) {
    $21 = $1;
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + 20|0);
    HEAP32[$23>>2] = 67;
    $24 = $1;
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$25>>2]|0;
    $27 = $1;
    FUNCTION_TABLE_vi[$26 & 127]($27);
   }
   $28 = $1;
   $29 = (($28) + 420|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 4|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = $1;
   FUNCTION_TABLE_vi[$32 & 127]($33);
   $34 = $1;
   $35 = (($34) + 20|0);
   HEAP32[$35>>2] = 210;
  }
 }
 if ((label|0) == 7) {
  $36 = $1;
  $37 = (($36) + 20|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)==(207);
  if ($39) {
   $40 = $1;
   $41 = (($40) + 20|0);
   HEAP32[$41>>2] = 210;
  } else {
   $42 = $1;
   $43 = (($42) + 20|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(210);
   if ($45) {
    $46 = $1;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + 20|0);
    HEAP32[$48>>2] = 20;
    $49 = $1;
    $50 = (($49) + 20|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = $1;
    $53 = HEAP32[$52>>2]|0;
    $54 = (($53) + 24|0);
    HEAP32[$54>>2] = $51;
    $55 = $1;
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$56>>2]|0;
    $58 = $1;
    FUNCTION_TABLE_vi[$57 & 127]($58);
   }
  }
 }
 while(1) {
  $59 = $1;
  $60 = (($59) + 436|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)!=(0);
  $65 = $64 ^ 1;
  if (!($65)) {
   label = 18;
   break;
  }
  $66 = $1;
  $67 = (($66) + 436|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = $1;
  $71 = (FUNCTION_TABLE_ii[$69 & 127]($70)|0);
  $72 = ($71|0)==(0);
  if ($72) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 16) {
  $0 = 0;
  $80 = $0;
  STACKTOP = sp;return ($80|0);
 }
 else if ((label|0) == 18) {
  $73 = $1;
  $74 = (($73) + 24|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 24|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = $1;
  FUNCTION_TABLE_vi[$77 & 127]($78);
  $79 = $1;
  _jpeg_abort($79);
  $0 = 1;
  $80 = $0;
  STACKTOP = sp;return ($80|0);
 }
 return 0|0;
}
function _jpeg_start_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $retcode = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(202);
 do {
  if ($5) {
   $6 = $1;
   _jinit_master_decompress($6);
   $7 = $1;
   $8 = (($7) + 64|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0);
   if (!($10)) {
    $13 = $1;
    $14 = (($13) + 20|0);
    HEAP32[$14>>2] = 203;
    break;
   }
   $11 = $1;
   $12 = (($11) + 20|0);
   HEAP32[$12>>2] = 207;
   $0 = 1;
   $97 = $0;
   STACKTOP = sp;return ($97|0);
  }
 } while(0);
 $15 = $1;
 $16 = (($15) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(203);
 if ($18) {
  $19 = $1;
  $20 = (($19) + 436|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0);
  do {
   if ($24) {
    while(1) {
     $25 = $1;
     $26 = (($25) + 8|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = ($27|0)!=(0|0);
     if ($28) {
      $29 = $1;
      $30 = (($29) + 8|0);
      $31 = HEAP32[$30>>2]|0;
      $32 = HEAP32[$31>>2]|0;
      $33 = $1;
      FUNCTION_TABLE_vi[$32 & 127]($33);
     }
     $34 = $1;
     $35 = (($34) + 436|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = HEAP32[$36>>2]|0;
     $38 = $1;
     $39 = (FUNCTION_TABLE_ii[$37 & 127]($38)|0);
     $retcode = $39;
     $40 = $retcode;
     $41 = ($40|0)==(0);
     if ($41) {
      label = 11;
      break;
     }
     $42 = $retcode;
     $43 = ($42|0)==(2);
     if ($43) {
      label = 13;
      break;
     }
     $44 = $1;
     $45 = (($44) + 8|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)!=(0|0);
     do {
      if ($47) {
       $48 = $retcode;
       $49 = ($48|0)==(3);
       if (!($49)) {
        $50 = $retcode;
        $51 = ($50|0)==(1);
        if (!($51)) {
         break;
        }
       }
       $52 = $1;
       $53 = (($52) + 8|0);
       $54 = HEAP32[$53>>2]|0;
       $55 = (($54) + 4|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = (($56) + 1)|0;
       HEAP32[$55>>2] = $57;
       $58 = $1;
       $59 = (($58) + 8|0);
       $60 = HEAP32[$59>>2]|0;
       $61 = (($60) + 8|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = ($57|0)>=($62|0);
       if ($63) {
        $64 = $1;
        $65 = (($64) + 320|0);
        $66 = HEAP32[$65>>2]|0;
        $67 = $1;
        $68 = (($67) + 8|0);
        $69 = HEAP32[$68>>2]|0;
        $70 = (($69) + 8|0);
        $71 = HEAP32[$70>>2]|0;
        $72 = (($71) + ($66))|0;
        HEAP32[$70>>2] = $72;
       }
      }
     } while(0);
    }
    if ((label|0) == 11) {
     $0 = 0;
     $97 = $0;
     STACKTOP = sp;return ($97|0);
    }
    else if ((label|0) == 13) {
     break;
    }
   }
  } while(0);
  $73 = $1;
  $74 = (($73) + 144|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = $1;
  $77 = (($76) + 152|0);
  HEAP32[$77>>2] = $75;
 } else {
  $78 = $1;
  $79 = (($78) + 20|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = ($80|0)!=(204);
  if ($81) {
   $82 = $1;
   $83 = HEAP32[$82>>2]|0;
   $84 = (($83) + 20|0);
   HEAP32[$84>>2] = 20;
   $85 = $1;
   $86 = (($85) + 20|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = $1;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 24|0);
   HEAP32[$90>>2] = $87;
   $91 = $1;
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$92>>2]|0;
   $94 = $1;
   FUNCTION_TABLE_vi[$93 & 127]($94);
  }
 }
 $95 = $1;
 $96 = (_output_pass_setup($95)|0);
 $0 = $96;
 $97 = $0;
 STACKTOP = sp;return ($97|0);
}
function _output_pass_setup($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $last_scanline = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(204);
 if ($5) {
  $6 = $1;
  $7 = (($6) + 420|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = $1;
  FUNCTION_TABLE_vi[$9 & 127]($10);
  $11 = $1;
  $12 = (($11) + 140|0);
  HEAP32[$12>>2] = 0;
  $13 = $1;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = 204;
 }
 L4: while(1) {
  $15 = $1;
  $16 = (($15) + 420|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0);
  if (!($20)) {
   label = 13;
   break;
  }
  while(1) {
   $21 = $1;
   $22 = (($21) + 140|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = $1;
   $25 = (($24) + 116|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($23>>>0)<($26>>>0);
   if (!($27)) {
    break;
   }
   $28 = $1;
   $29 = (($28) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = $1;
    $33 = (($32) + 140|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = $1;
    $36 = (($35) + 8|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + 4|0);
    HEAP32[$38>>2] = $34;
    $39 = $1;
    $40 = (($39) + 116|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $1;
    $43 = (($42) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + 8|0);
    HEAP32[$45>>2] = $41;
    $46 = $1;
    $47 = (($46) + 8|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    $50 = $1;
    FUNCTION_TABLE_vi[$49 & 127]($50);
   }
   $51 = $1;
   $52 = (($51) + 140|0);
   $53 = HEAP32[$52>>2]|0;
   $last_scanline = $53;
   $54 = $1;
   $55 = (($54) + 424|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 4|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = $1;
   $60 = $1;
   $61 = (($60) + 140|0);
   FUNCTION_TABLE_viiii[$58 & 127]($59,0,$61,0);
   $62 = $1;
   $63 = (($62) + 140|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $last_scanline;
   $66 = ($64|0)==($65|0);
   if ($66) {
    label = 10;
    break L4;
   }
  }
  $67 = $1;
  $68 = (($67) + 420|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + 4|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = $1;
  FUNCTION_TABLE_vi[$71 & 127]($72);
  $73 = $1;
  $74 = (($73) + 420|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP32[$75>>2]|0;
  $77 = $1;
  FUNCTION_TABLE_vi[$76 & 127]($77);
  $78 = $1;
  $79 = (($78) + 140|0);
  HEAP32[$79>>2] = 0;
 }
 if ((label|0) == 10) {
  $0 = 0;
  $87 = $0;
  STACKTOP = sp;return ($87|0);
 }
 else if ((label|0) == 13) {
  $80 = $1;
  $81 = (($80) + 68|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($82|0)!=(0);
  $84 = $83 ? 206 : 205;
  $85 = $1;
  $86 = (($85) + 20|0);
  HEAP32[$86>>2] = $84;
  $0 = 1;
  $87 = $0;
  STACKTOP = sp;return ($87|0);
 }
 return 0|0;
}
function _jpeg_read_scanlines($cinfo,$scanlines,$max_lines) {
 $cinfo = $cinfo|0;
 $scanlines = $scanlines|0;
 $max_lines = $max_lines|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $row_ctr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $row_ctr = sp;
 $1 = $cinfo;
 $2 = $scanlines;
 $3 = $max_lines;
 $4 = $1;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(205);
 if ($7) {
  $8 = $1;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 20|0);
  HEAP32[$10>>2] = 20;
  $11 = $1;
  $12 = (($11) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $1;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 24|0);
  HEAP32[$16>>2] = $13;
  $17 = $1;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  FUNCTION_TABLE_vi[$19 & 127]($20);
 }
 $21 = $1;
 $22 = (($21) + 140|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = (($24) + 116|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($23>>>0)>=($26>>>0);
 if ($27) {
  $28 = $1;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 20|0);
  HEAP32[$30>>2] = 123;
  $31 = $1;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $1;
  FUNCTION_TABLE_vii[$34 & 127]($35,-1);
  $0 = 0;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 }
 $36 = $1;
 $37 = (($36) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = $1;
  $41 = (($40) + 140|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $1;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 4|0);
  HEAP32[$46>>2] = $42;
  $47 = $1;
  $48 = (($47) + 116|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = $1;
  $51 = (($50) + 8|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 8|0);
  HEAP32[$53>>2] = $49;
  $54 = $1;
  $55 = (($54) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = $1;
  FUNCTION_TABLE_vi[$57 & 127]($58);
 }
 HEAP32[$row_ctr>>2] = 0;
 $59 = $1;
 $60 = (($59) + 424|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (($61) + 4|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = $1;
 $65 = $2;
 $66 = $3;
 FUNCTION_TABLE_viiii[$63 & 127]($64,$65,$row_ctr,$66);
 $67 = HEAP32[$row_ctr>>2]|0;
 $68 = $1;
 $69 = (($68) + 140|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + ($67))|0;
 HEAP32[$69>>2] = $71;
 $72 = HEAP32[$row_ctr>>2]|0;
 $0 = $72;
 $73 = $0;
 STACKTOP = sp;return ($73|0);
}
function _jpeg_stdio_src($cinfo,$infile) {
 $cinfo = $cinfo|0;
 $infile = $infile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $infile;
 $2 = $0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = $0;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = $0;
  $11 = (FUNCTION_TABLE_iiii[$9 & 127]($10,0,40)|0);
  $12 = $0;
  $13 = (($12) + 24|0);
  HEAP32[$13>>2] = $11;
  $14 = $0;
  $15 = (($14) + 24|0);
  $16 = HEAP32[$15>>2]|0;
  $src = $16;
  $17 = $0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  $22 = (FUNCTION_TABLE_iiii[$20 & 127]($21,0,4096)|0);
  $23 = $src;
  $24 = (($23) + 32|0);
  HEAP32[$24>>2] = $22;
 }
 $25 = $0;
 $26 = (($25) + 24|0);
 $27 = HEAP32[$26>>2]|0;
 $src = $27;
 $28 = $src;
 $29 = (($28) + 8|0);
 HEAP32[$29>>2] = 21;
 $30 = $src;
 $31 = (($30) + 12|0);
 HEAP32[$31>>2] = 22;
 $32 = $src;
 $33 = (($32) + 16|0);
 HEAP32[$33>>2] = 23;
 $34 = $src;
 $35 = (($34) + 20|0);
 HEAP32[$35>>2] = 24;
 $36 = $src;
 $37 = (($36) + 24|0);
 HEAP32[$37>>2] = 25;
 $38 = $1;
 $39 = $src;
 $40 = (($39) + 28|0);
 HEAP32[$40>>2] = $38;
 $41 = $src;
 $42 = (($41) + 4|0);
 HEAP32[$42>>2] = 0;
 $43 = $src;
 HEAP32[$43>>2] = 0;
 STACKTOP = sp;return;
}
function _init_source($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $src = $3;
 $4 = $src;
 $5 = (($4) + 36|0);
 HEAP32[$5>>2] = 1;
 STACKTOP = sp;return;
}
function _fill_input_buffer($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nbytes = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $src = $3;
 $4 = $src;
 $5 = (($4) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $src;
 $8 = (($7) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_fread(($6|0),1,4096,($9|0))|0);
 $nbytes = $10;
 $11 = $nbytes;
 $12 = ($11>>>0)<=(0);
 if ($12) {
  $13 = $src;
  $14 = (($13) + 36|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = $0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (($18) + 20|0);
   HEAP32[$19>>2] = 42;
   $20 = $0;
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   $23 = $0;
   FUNCTION_TABLE_vi[$22 & 127]($23);
  }
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 20|0);
  HEAP32[$26>>2] = 120;
  $27 = $0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vii[$30 & 127]($31,-1);
  $32 = $src;
  $33 = (($32) + 32|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP8[$34>>0] = -1;
  $35 = $src;
  $36 = (($35) + 32|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + 1|0);
  HEAP8[$38>>0] = -39;
  $nbytes = 2;
 }
 $39 = $src;
 $40 = (($39) + 32|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $src;
 HEAP32[$42>>2] = $41;
 $43 = $nbytes;
 $44 = $src;
 $45 = (($44) + 4|0);
 HEAP32[$45>>2] = $43;
 $46 = $src;
 $47 = (($46) + 36|0);
 HEAP32[$47>>2] = 0;
 STACKTOP = sp;return 1;
}
function _skip_input_data($cinfo,$num_bytes) {
 $cinfo = $cinfo|0;
 $num_bytes = $num_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $num_bytes;
 $2 = $0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $src = $4;
 $5 = $1;
 $6 = ($5|0)>(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $7 = $1;
  $8 = $src;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)>($10|0);
  if (!($11)) {
   break;
  }
  $12 = $src;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $1;
  $16 = (($15) - ($14))|0;
  $1 = $16;
  $17 = $0;
  (_fill_input_buffer($17)|0);
 }
 $18 = $1;
 $19 = $src;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18)|0);
 HEAP32[$19>>2] = $21;
 $22 = $1;
 $23 = $src;
 $24 = (($23) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) - ($22))|0;
 HEAP32[$24>>2] = $26;
 STACKTOP = sp;return;
}
function _term_source($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _jinit_input_controller($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $inputctl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,0,28)|0);
 $inputctl = $6;
 $7 = $inputctl;
 $8 = $0;
 $9 = (($8) + 436|0);
 HEAP32[$9>>2] = $7;
 $10 = $inputctl;
 HEAP32[$10>>2] = 26;
 $11 = $inputctl;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 27;
 $13 = $inputctl;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 28;
 $15 = $inputctl;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = 29;
 $17 = $inputctl;
 $18 = (($17) + 16|0);
 HEAP32[$18>>2] = 0;
 $19 = $inputctl;
 $20 = (($19) + 20|0);
 HEAP32[$20>>2] = 0;
 $21 = $inputctl;
 $22 = (($21) + 24|0);
 HEAP32[$22>>2] = 1;
 STACKTOP = sp;return;
}
function _consume_markers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $inputctl = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 436|0);
 $4 = HEAP32[$3>>2]|0;
 $inputctl = $4;
 $5 = $inputctl;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $0 = 2;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $9 = $1;
 $10 = (($9) + 440|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $1;
 $15 = (FUNCTION_TABLE_ii[$13 & 127]($14)|0);
 $val = $15;
 $16 = $val;
 if ((($16|0) == 1)) {
  $17 = $inputctl;
  $18 = (($17) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = $1;
   _initial_setup($21);
   $22 = $inputctl;
   $23 = (($22) + 24|0);
   HEAP32[$23>>2] = 0;
  } else {
   $24 = $inputctl;
   $25 = (($24) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $28 = $1;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($29) + 20|0);
    HEAP32[$30>>2] = 35;
    $31 = $1;
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$32>>2]|0;
    $34 = $1;
    FUNCTION_TABLE_vi[$33 & 127]($34);
   }
   $35 = $1;
   _start_input_pass($35);
  }
 } else if ((($16|0) == 2)) {
  $36 = $inputctl;
  $37 = (($36) + 20|0);
  HEAP32[$37>>2] = 1;
  $38 = $inputctl;
  $39 = (($38) + 24|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = $1;
   $43 = (($42) + 440|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + 16|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = $1;
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 20|0);
    HEAP32[$50>>2] = 59;
    $51 = $1;
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    $54 = $1;
    FUNCTION_TABLE_vi[$53 & 127]($54);
   }
  } else {
   $55 = $1;
   $56 = (($55) + 152|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = $1;
   $59 = (($58) + 144|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ($57|0)>($60|0);
   if ($61) {
    $62 = $1;
    $63 = (($62) + 144|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = $1;
    $66 = (($65) + 152|0);
    HEAP32[$66>>2] = $64;
   }
  }
 } else if ((($16|0) == 0)) {
 }
 $67 = $val;
 $0 = $67;
 $68 = $0;
 STACKTOP = sp;return ($68|0);
}
function _reset_input_controller($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $inputctl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 436|0);
 $3 = HEAP32[$2>>2]|0;
 $inputctl = $3;
 $4 = $inputctl;
 HEAP32[$4>>2] = 26;
 $5 = $inputctl;
 $6 = (($5) + 16|0);
 HEAP32[$6>>2] = 0;
 $7 = $inputctl;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = 0;
 $9 = $inputctl;
 $10 = (($9) + 24|0);
 HEAP32[$10>>2] = 1;
 $11 = $0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $0;
 FUNCTION_TABLE_vi[$14 & 127]($15);
 $16 = $0;
 $17 = (($16) + 440|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = $0;
 FUNCTION_TABLE_vi[$19 & 127]($20);
 $21 = $0;
 $22 = (($21) + 160|0);
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _start_input_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 _per_scan_setup($1);
 $2 = $0;
 _latch_quant_tables($2);
 $3 = $0;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 FUNCTION_TABLE_vi[$6 & 127]($7);
 $8 = $0;
 $9 = (($8) + 428|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = $0;
 FUNCTION_TABLE_vi[$11 & 127]($12);
 $13 = $0;
 $14 = (($13) + 428|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $0;
 $19 = (($18) + 436|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$20>>2] = $17;
 STACKTOP = sp;return;
}
function _finish_input_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 436|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = 26;
 STACKTOP = sp;return;
}
function _per_scan_setup($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0, $mcublks = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 328|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(1);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 332|0);
  $7 = HEAP32[$6>>2]|0;
  $compptr = $7;
  $8 = $compptr;
  $9 = (($8) + 28|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = (($11) + 348|0);
  HEAP32[$12>>2] = $10;
  $13 = $compptr;
  $14 = (($13) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $0;
  $17 = (($16) + 352|0);
  HEAP32[$17>>2] = $15;
  $18 = $compptr;
  $19 = (($18) + 52|0);
  HEAP32[$19>>2] = 1;
  $20 = $compptr;
  $21 = (($20) + 56|0);
  HEAP32[$21>>2] = 1;
  $22 = $compptr;
  $23 = (($22) + 60|0);
  HEAP32[$23>>2] = 1;
  $24 = $compptr;
  $25 = (($24) + 36|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $compptr;
  $28 = (($27) + 64|0);
  HEAP32[$28>>2] = $26;
  $29 = $compptr;
  $30 = (($29) + 68|0);
  HEAP32[$30>>2] = 1;
  $31 = $compptr;
  $32 = (($31) + 32|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $compptr;
  $35 = (($34) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($33>>>0) % ($36>>>0))&-1;
  $tmp = $37;
  $38 = $tmp;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = $compptr;
   $41 = (($40) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $tmp = $42;
  }
  $43 = $tmp;
  $44 = $compptr;
  $45 = (($44) + 72|0);
  HEAP32[$45>>2] = $43;
  $46 = $0;
  $47 = (($46) + 356|0);
  HEAP32[$47>>2] = 1;
  $48 = $0;
  $49 = (($48) + 360|0);
  HEAP32[$49>>2] = 0;
  STACKTOP = sp;return;
 }
 $50 = $0;
 $51 = (($50) + 328|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)<=(0);
 if ($53) {
  label = 7;
 } else {
  $54 = $0;
  $55 = (($54) + 328|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)>(4);
  if ($57) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $58 = $0;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($59) + 20|0);
  HEAP32[$60>>2] = 26;
  $61 = $0;
  $62 = (($61) + 328|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $0;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 24|0);
  HEAP32[$66>>2] = $63;
  $67 = $0;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + 24|0);
  $70 = (($69) + 4|0);
  HEAP32[$70>>2] = 4;
  $71 = $0;
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  FUNCTION_TABLE_vi[$73 & 127]($74);
 }
 $75 = $0;
 $76 = (($75) + 28|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = $0;
 $79 = (($78) + 308|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $80<<3;
 $82 = (_jdiv_round_up($77,$81)|0);
 $83 = $0;
 $84 = (($83) + 348|0);
 HEAP32[$84>>2] = $82;
 $85 = $0;
 $86 = (($85) + 32|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $0;
 $89 = (($88) + 312|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = $90<<3;
 $92 = (_jdiv_round_up($87,$91)|0);
 $93 = $0;
 $94 = (($93) + 352|0);
 HEAP32[$94>>2] = $92;
 $95 = $0;
 $96 = (($95) + 356|0);
 HEAP32[$96>>2] = 0;
 $ci = 0;
 while(1) {
  $97 = $ci;
  $98 = $0;
  $99 = (($98) + 328|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($97|0)<($100|0);
  if (!($101)) {
   break;
  }
  $102 = $ci;
  $103 = $0;
  $104 = (($103) + 332|0);
  $105 = (($104) + ($102<<2)|0);
  $106 = HEAP32[$105>>2]|0;
  $compptr = $106;
  $107 = $compptr;
  $108 = (($107) + 8|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = $compptr;
  $111 = (($110) + 52|0);
  HEAP32[$111>>2] = $109;
  $112 = $compptr;
  $113 = (($112) + 12|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = $compptr;
  $116 = (($115) + 56|0);
  HEAP32[$116>>2] = $114;
  $117 = $compptr;
  $118 = (($117) + 52|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = $compptr;
  $121 = (($120) + 56|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = Math_imul($119, $122)|0;
  $124 = $compptr;
  $125 = (($124) + 60|0);
  HEAP32[$125>>2] = $123;
  $126 = $compptr;
  $127 = (($126) + 52|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = $compptr;
  $130 = (($129) + 36|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = Math_imul($128, $131)|0;
  $133 = $compptr;
  $134 = (($133) + 64|0);
  HEAP32[$134>>2] = $132;
  $135 = $compptr;
  $136 = (($135) + 28|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = $compptr;
  $139 = (($138) + 52|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = (($137>>>0) % ($140>>>0))&-1;
  $tmp = $141;
  $142 = $tmp;
  $143 = ($142|0)==(0);
  if ($143) {
   $144 = $compptr;
   $145 = (($144) + 52|0);
   $146 = HEAP32[$145>>2]|0;
   $tmp = $146;
  }
  $147 = $tmp;
  $148 = $compptr;
  $149 = (($148) + 68|0);
  HEAP32[$149>>2] = $147;
  $150 = $compptr;
  $151 = (($150) + 32|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = $compptr;
  $154 = (($153) + 56|0);
  $155 = HEAP32[$154>>2]|0;
  $156 = (($152>>>0) % ($155>>>0))&-1;
  $tmp = $156;
  $157 = $tmp;
  $158 = ($157|0)==(0);
  if ($158) {
   $159 = $compptr;
   $160 = (($159) + 56|0);
   $161 = HEAP32[$160>>2]|0;
   $tmp = $161;
  }
  $162 = $tmp;
  $163 = $compptr;
  $164 = (($163) + 72|0);
  HEAP32[$164>>2] = $162;
  $165 = $compptr;
  $166 = (($165) + 60|0);
  $167 = HEAP32[$166>>2]|0;
  $mcublks = $167;
  $168 = $0;
  $169 = (($168) + 356|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = $mcublks;
  $172 = (($170) + ($171))|0;
  $173 = ($172|0)>(10);
  if ($173) {
   $174 = $0;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($175) + 20|0);
   HEAP32[$176>>2] = 13;
   $177 = $0;
   $178 = HEAP32[$177>>2]|0;
   $179 = HEAP32[$178>>2]|0;
   $180 = $0;
   FUNCTION_TABLE_vi[$179 & 127]($180);
  }
  while(1) {
   $181 = $mcublks;
   $182 = (($181) + -1)|0;
   $mcublks = $182;
   $183 = ($181|0)>(0);
   if (!($183)) {
    break;
   }
   $184 = $ci;
   $185 = $0;
   $186 = (($185) + 356|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = (($187) + 1)|0;
   HEAP32[$186>>2] = $188;
   $189 = $0;
   $190 = (($189) + 360|0);
   $191 = (($190) + ($187<<2)|0);
   HEAP32[$191>>2] = $184;
  }
  $192 = $ci;
  $193 = (($192) + 1)|0;
  $ci = $193;
 }
 STACKTOP = sp;return;
}
function _latch_quant_tables($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $compptr = 0, $qtbl = 0, $qtblno = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $ci = 0;
 while(1) {
  $1 = $ci;
  $2 = $0;
  $3 = (($2) + 328|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $ci;
  $7 = $0;
  $8 = (($7) + 332|0);
  $9 = (($8) + ($6<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $compptr = $10;
  $11 = $compptr;
  $12 = (($11) + 76|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
  } else {
   $15 = $compptr;
   $16 = (($15) + 16|0);
   $17 = HEAP32[$16>>2]|0;
   $qtblno = $17;
   $18 = $qtblno;
   $19 = ($18|0)<(0);
   if ($19) {
    label = 8;
   } else {
    $20 = $qtblno;
    $21 = ($20|0)>=(4);
    if ($21) {
     label = 8;
    } else {
     $22 = $qtblno;
     $23 = $0;
     $24 = (($23) + 164|0);
     $25 = (($24) + ($22<<2)|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==(0|0);
     if ($27) {
      label = 8;
     }
    }
   }
   if ((label|0) == 8) {
    label = 0;
    $28 = $0;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($29) + 20|0);
    HEAP32[$30>>2] = 52;
    $31 = $qtblno;
    $32 = $0;
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 24|0);
    HEAP32[$34>>2] = $31;
    $35 = $0;
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = $0;
    FUNCTION_TABLE_vi[$37 & 127]($38);
   }
   $39 = $0;
   $40 = (($39) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = $0;
   $44 = (FUNCTION_TABLE_iiii[$42 & 127]($43,1,132)|0);
   $qtbl = $44;
   $45 = $qtbl;
   $46 = $qtblno;
   $47 = $0;
   $48 = (($47) + 164|0);
   $49 = (($48) + ($46<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   _memcpy(($45|0),($50|0),132)|0;
   $51 = $qtbl;
   $52 = $compptr;
   $53 = (($52) + 76|0);
   HEAP32[$53>>2] = $51;
  }
  $54 = $ci;
  $55 = (($54) + 1)|0;
  $ci = $55;
 }
 STACKTOP = sp;return;
}
function _initial_setup($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(65500);
 if ($4) {
  label = 3;
 } else {
  $5 = $0;
  $6 = (($5) + 28|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)>(65500);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = $0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 20|0);
  HEAP32[$11>>2] = 41;
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 24|0);
  HEAP32[$14>>2] = 65500;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 127]($18);
 }
 $19 = $0;
 $20 = (($19) + 212|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(8);
 if ($22) {
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 15;
  $26 = $0;
  $27 = (($26) + 212|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $0;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 24|0);
  HEAP32[$31>>2] = $28;
  $32 = $0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  FUNCTION_TABLE_vi[$34 & 127]($35);
 }
 $36 = $0;
 $37 = (($36) + 36|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)>(10);
 if ($39) {
  $40 = $0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 20|0);
  HEAP32[$42>>2] = 26;
  $43 = $0;
  $44 = (($43) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 24|0);
  HEAP32[$48>>2] = $45;
  $49 = $0;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 24|0);
  $52 = (($51) + 4|0);
  HEAP32[$52>>2] = 10;
  $53 = $0;
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = $0;
  FUNCTION_TABLE_vi[$55 & 127]($56);
 }
 $57 = $0;
 $58 = (($57) + 308|0);
 HEAP32[$58>>2] = 1;
 $59 = $0;
 $60 = (($59) + 312|0);
 HEAP32[$60>>2] = 1;
 $ci = 0;
 $61 = $0;
 $62 = (($61) + 216|0);
 $63 = HEAP32[$62>>2]|0;
 $compptr = $63;
 while(1) {
  $64 = $ci;
  $65 = $0;
  $66 = (($65) + 36|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($64|0)<($67|0);
  if (!($68)) {
   break;
  }
  $69 = $compptr;
  $70 = (($69) + 8|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = ($71|0)<=(0);
  if ($72) {
   label = 14;
  } else {
   $73 = $compptr;
   $74 = (($73) + 8|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75|0)>(4);
   if ($76) {
    label = 14;
   } else {
    $77 = $compptr;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)<=(0);
    if ($80) {
     label = 14;
    } else {
     $81 = $compptr;
     $82 = (($81) + 12|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)>(4);
     if ($84) {
      label = 14;
     }
    }
   }
  }
  if ((label|0) == 14) {
   label = 0;
   $85 = $0;
   $86 = HEAP32[$85>>2]|0;
   $87 = (($86) + 20|0);
   HEAP32[$87>>2] = 18;
   $88 = $0;
   $89 = HEAP32[$88>>2]|0;
   $90 = HEAP32[$89>>2]|0;
   $91 = $0;
   FUNCTION_TABLE_vi[$90 & 127]($91);
  }
  $92 = $0;
  $93 = (($92) + 308|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $compptr;
  $96 = (($95) + 8|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($94|0)>($97|0);
  if ($98) {
   $99 = $0;
   $100 = (($99) + 308|0);
   $101 = HEAP32[$100>>2]|0;
   $107 = $101;
  } else {
   $102 = $compptr;
   $103 = (($102) + 8|0);
   $104 = HEAP32[$103>>2]|0;
   $107 = $104;
  }
  $105 = $0;
  $106 = (($105) + 308|0);
  HEAP32[$106>>2] = $107;
  $108 = $0;
  $109 = (($108) + 312|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = $compptr;
  $112 = (($111) + 12|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($110|0)>($113|0);
  if ($114) {
   $115 = $0;
   $116 = (($115) + 312|0);
   $117 = HEAP32[$116>>2]|0;
   $123 = $117;
  } else {
   $118 = $compptr;
   $119 = (($118) + 12|0);
   $120 = HEAP32[$119>>2]|0;
   $123 = $120;
  }
  $121 = $0;
  $122 = (($121) + 312|0);
  HEAP32[$122>>2] = $123;
  $124 = $ci;
  $125 = (($124) + 1)|0;
  $ci = $125;
  $126 = $compptr;
  $127 = (($126) + 84|0);
  $compptr = $127;
 }
 $128 = $0;
 $129 = (($128) + 316|0);
 HEAP32[$129>>2] = 8;
 $ci = 0;
 $130 = $0;
 $131 = (($130) + 216|0);
 $132 = HEAP32[$131>>2]|0;
 $compptr = $132;
 while(1) {
  $133 = $ci;
  $134 = $0;
  $135 = (($134) + 36|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = ($133|0)<($136|0);
  if (!($137)) {
   break;
  }
  $138 = $compptr;
  $139 = (($138) + 36|0);
  HEAP32[$139>>2] = 8;
  $140 = $0;
  $141 = (($140) + 28|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = $compptr;
  $144 = (($143) + 8|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = Math_imul($142, $145)|0;
  $147 = $0;
  $148 = (($147) + 308|0);
  $149 = HEAP32[$148>>2]|0;
  $150 = $149<<3;
  $151 = (_jdiv_round_up($146,$150)|0);
  $152 = $compptr;
  $153 = (($152) + 28|0);
  HEAP32[$153>>2] = $151;
  $154 = $0;
  $155 = (($154) + 32|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = $compptr;
  $158 = (($157) + 12|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = Math_imul($156, $159)|0;
  $161 = $0;
  $162 = (($161) + 312|0);
  $163 = HEAP32[$162>>2]|0;
  $164 = $163<<3;
  $165 = (_jdiv_round_up($160,$164)|0);
  $166 = $compptr;
  $167 = (($166) + 32|0);
  HEAP32[$167>>2] = $165;
  $168 = $0;
  $169 = (($168) + 28|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = $compptr;
  $172 = (($171) + 8|0);
  $173 = HEAP32[$172>>2]|0;
  $174 = Math_imul($170, $173)|0;
  $175 = $0;
  $176 = (($175) + 308|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = (_jdiv_round_up($174,$177)|0);
  $179 = $compptr;
  $180 = (($179) + 40|0);
  HEAP32[$180>>2] = $178;
  $181 = $0;
  $182 = (($181) + 32|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = $compptr;
  $185 = (($184) + 12|0);
  $186 = HEAP32[$185>>2]|0;
  $187 = Math_imul($183, $186)|0;
  $188 = $0;
  $189 = (($188) + 312|0);
  $190 = HEAP32[$189>>2]|0;
  $191 = (_jdiv_round_up($187,$190)|0);
  $192 = $compptr;
  $193 = (($192) + 44|0);
  HEAP32[$193>>2] = $191;
  $194 = $compptr;
  $195 = (($194) + 48|0);
  HEAP32[$195>>2] = 1;
  $196 = $compptr;
  $197 = (($196) + 76|0);
  HEAP32[$197>>2] = 0;
  $198 = $ci;
  $199 = (($198) + 1)|0;
  $ci = $199;
  $200 = $compptr;
  $201 = (($200) + 84|0);
  $compptr = $201;
 }
 $202 = $0;
 $203 = (($202) + 32|0);
 $204 = HEAP32[$203>>2]|0;
 $205 = $0;
 $206 = (($205) + 312|0);
 $207 = HEAP32[$206>>2]|0;
 $208 = $207<<3;
 $209 = (_jdiv_round_up($204,$208)|0);
 $210 = $0;
 $211 = (($210) + 320|0);
 HEAP32[$211>>2] = $209;
 $212 = $0;
 $213 = (($212) + 328|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = $0;
 $216 = (($215) + 36|0);
 $217 = HEAP32[$216>>2]|0;
 $218 = ($214|0)<($217|0);
 if (!($218)) {
  $219 = $0;
  $220 = (($219) + 220|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = ($221|0)!=(0);
  if (!($222)) {
   $227 = $0;
   $228 = (($227) + 436|0);
   $229 = HEAP32[$228>>2]|0;
   $230 = (($229) + 16|0);
   HEAP32[$230>>2] = 0;
   STACKTOP = sp;return;
  }
 }
 $223 = $0;
 $224 = (($223) + 436|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + 16|0);
 HEAP32[$226>>2] = 1;
 STACKTOP = sp;return;
}
function _jpeg_resync_to_restart($cinfo,$desired) {
 $cinfo = $cinfo|0;
 $desired = $desired|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $action = 0, $marker = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $desired;
 $3 = $1;
 $4 = (($3) + 416|0);
 $5 = HEAP32[$4>>2]|0;
 $marker = $5;
 $action = 1;
 $6 = $1;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = 121;
 $9 = $marker;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 24|0);
 HEAP32[$12>>2] = $9;
 $13 = $2;
 $14 = $1;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 24|0);
 $17 = (($16) + 4|0);
 HEAP32[$17>>2] = $13;
 $18 = $1;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $1;
 FUNCTION_TABLE_vii[$21 & 127]($22,-1);
 while(1) {
  $23 = $marker;
  $24 = ($23|0)<(192);
  if ($24) {
   $action = 2;
  } else {
   $25 = $marker;
   $26 = ($25|0)<(208);
   if ($26) {
    label = 6;
   } else {
    $27 = $marker;
    $28 = ($27|0)>(215);
    if ($28) {
     label = 6;
    } else {
     $29 = $marker;
     $30 = $2;
     $31 = (($30) + 1)|0;
     $32 = $31 & 7;
     $33 = (208 + ($32))|0;
     $34 = ($29|0)==($33|0);
     if ($34) {
      label = 9;
     } else {
      $35 = $marker;
      $36 = $2;
      $37 = (($36) + 2)|0;
      $38 = $37 & 7;
      $39 = (208 + ($38))|0;
      $40 = ($35|0)==($39|0);
      if ($40) {
       label = 9;
      } else {
       $41 = $marker;
       $42 = $2;
       $43 = (($42) - 1)|0;
       $44 = $43 & 7;
       $45 = (208 + ($44))|0;
       $46 = ($41|0)==($45|0);
       if ($46) {
        label = 12;
       } else {
        $47 = $marker;
        $48 = $2;
        $49 = (($48) - 2)|0;
        $50 = $49 & 7;
        $51 = (208 + ($50))|0;
        $52 = ($47|0)==($51|0);
        if ($52) {
         label = 12;
        } else {
         $action = 1;
        }
       }
       if ((label|0) == 12) {
        label = 0;
        $action = 2;
       }
      }
     }
     if ((label|0) == 9) {
      label = 0;
      $action = 3;
     }
    }
   }
   if ((label|0) == 6) {
    label = 0;
    $action = 3;
   }
  }
  $53 = $1;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 20|0);
  HEAP32[$55>>2] = 97;
  $56 = $marker;
  $57 = $1;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 24|0);
  HEAP32[$59>>2] = $56;
  $60 = $action;
  $61 = $1;
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) + 24|0);
  $64 = (($63) + 4|0);
  HEAP32[$64>>2] = $60;
  $65 = $1;
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + 4|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $1;
  FUNCTION_TABLE_vii[$68 & 127]($69,4);
  $70 = $action;
  if ((($70|0) == 1)) {
   label = 18;
   break;
  } else if ((($70|0) == 2)) {
   $73 = $1;
   $74 = (_next_marker($73)|0);
   $75 = ($74|0)!=(0);
   if (!($75)) {
    label = 20;
    break;
   }
   $76 = $1;
   $77 = (($76) + 416|0);
   $78 = HEAP32[$77>>2]|0;
   $marker = $78;
  } else if ((($70|0) == 3)) {
   label = 22;
   break;
  }
 }
 if ((label|0) == 18) {
  $71 = $1;
  $72 = (($71) + 416|0);
  HEAP32[$72>>2] = 0;
  $0 = 1;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 20) {
  $0 = 0;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 22) {
  $0 = 1;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 return 0|0;
}
function _next_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytes_in_buffer = 0, $c = 0, $datasrc = 0, $next_input_byte = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 L1: while(1) {
  $10 = $bytes_in_buffer;
  $11 = ($10|0)==(0);
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if (!($17)) {
    label = 5;
    break;
   }
   $18 = $datasrc;
   $19 = HEAP32[$18>>2]|0;
   $next_input_byte = $19;
   $20 = $datasrc;
   $21 = (($20) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $bytes_in_buffer = $22;
  }
  $23 = $bytes_in_buffer;
  $24 = (($23) + -1)|0;
  $bytes_in_buffer = $24;
  $25 = $next_input_byte;
  $26 = (($25) + 1|0);
  $next_input_byte = $26;
  $27 = HEAP8[$25>>0]|0;
  $28 = $27&255;
  $c = $28;
  while(1) {
   $29 = $c;
   $30 = ($29|0)!=(255);
   if (!($30)) {
    break;
   }
   $31 = $1;
   $32 = (($31) + 440|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 24|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 1)|0;
   HEAP32[$34>>2] = $36;
   $37 = $next_input_byte;
   $38 = $datasrc;
   HEAP32[$38>>2] = $37;
   $39 = $bytes_in_buffer;
   $40 = $datasrc;
   $41 = (($40) + 4|0);
   HEAP32[$41>>2] = $39;
   $42 = $bytes_in_buffer;
   $43 = ($42|0)==(0);
   if ($43) {
    $44 = $datasrc;
    $45 = (($44) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = $1;
    $48 = (FUNCTION_TABLE_ii[$46 & 127]($47)|0);
    $49 = ($48|0)!=(0);
    if (!($49)) {
     label = 13;
     break L1;
    }
    $50 = $datasrc;
    $51 = HEAP32[$50>>2]|0;
    $next_input_byte = $51;
    $52 = $datasrc;
    $53 = (($52) + 4|0);
    $54 = HEAP32[$53>>2]|0;
    $bytes_in_buffer = $54;
   }
   $55 = $bytes_in_buffer;
   $56 = (($55) + -1)|0;
   $bytes_in_buffer = $56;
   $57 = $next_input_byte;
   $58 = (($57) + 1|0);
   $next_input_byte = $58;
   $59 = HEAP8[$57>>0]|0;
   $60 = $59&255;
   $c = $60;
  }
  while(1) {
   $61 = $bytes_in_buffer;
   $62 = ($61|0)==(0);
   if ($62) {
    $63 = $datasrc;
    $64 = (($63) + 12|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = $1;
    $67 = (FUNCTION_TABLE_ii[$65 & 127]($66)|0);
    $68 = ($67|0)!=(0);
    if (!($68)) {
     label = 21;
     break L1;
    }
    $69 = $datasrc;
    $70 = HEAP32[$69>>2]|0;
    $next_input_byte = $70;
    $71 = $datasrc;
    $72 = (($71) + 4|0);
    $73 = HEAP32[$72>>2]|0;
    $bytes_in_buffer = $73;
   }
   $74 = $bytes_in_buffer;
   $75 = (($74) + -1)|0;
   $bytes_in_buffer = $75;
   $76 = $next_input_byte;
   $77 = (($76) + 1|0);
   $next_input_byte = $77;
   $78 = HEAP8[$76>>0]|0;
   $79 = $78&255;
   $c = $79;
   $80 = $c;
   $81 = ($80|0)==(255);
   if (!($81)) {
    break;
   }
  }
  $82 = $c;
  $83 = ($82|0)!=(0);
  if ($83) {
   label = 27;
   break;
  }
  $84 = $1;
  $85 = (($84) + 440|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + 24|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($88) + 2)|0;
  HEAP32[$87>>2] = $89;
  $90 = $next_input_byte;
  $91 = $datasrc;
  HEAP32[$91>>2] = $90;
  $92 = $bytes_in_buffer;
  $93 = $datasrc;
  $94 = (($93) + 4|0);
  HEAP32[$94>>2] = $92;
 }
 if ((label|0) == 5) {
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 13) {
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 21) {
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 27) {
  $95 = $1;
  $96 = (($95) + 440|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = (($97) + 24|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = $1;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) + 20|0);
   HEAP32[$103>>2] = 116;
   $104 = $1;
   $105 = (($104) + 440|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($106) + 24|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = $1;
   $110 = HEAP32[$109>>2]|0;
   $111 = (($110) + 24|0);
   HEAP32[$111>>2] = $108;
   $112 = $c;
   $113 = $1;
   $114 = HEAP32[$113>>2]|0;
   $115 = (($114) + 24|0);
   $116 = (($115) + 4|0);
   HEAP32[$116>>2] = $112;
   $117 = $1;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($118) + 4|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = $1;
   FUNCTION_TABLE_vii[$120 & 127]($121,-1);
   $122 = $1;
   $123 = (($122) + 440|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = (($124) + 24|0);
   HEAP32[$125>>2] = 0;
  }
  $126 = $c;
  $127 = $1;
  $128 = (($127) + 416|0);
  HEAP32[$128>>2] = $126;
  $129 = $next_input_byte;
  $130 = $datasrc;
  HEAP32[$130>>2] = $129;
  $131 = $bytes_in_buffer;
  $132 = $datasrc;
  $133 = (($132) + 4|0);
  HEAP32[$133>>2] = $131;
  $0 = 1;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 return 0|0;
}
function _jinit_marker_reader($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $marker = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,0,172)|0);
 $marker = $6;
 $7 = $marker;
 $8 = $0;
 $9 = (($8) + 440|0);
 HEAP32[$9>>2] = $7;
 $10 = $marker;
 HEAP32[$10>>2] = 30;
 $11 = $marker;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 31;
 $13 = $marker;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 32;
 $15 = $marker;
 $16 = (($15) + 28|0);
 HEAP32[$16>>2] = 33;
 $17 = $marker;
 $18 = (($17) + 96|0);
 HEAP32[$18>>2] = 0;
 $i = 0;
 while(1) {
  $19 = $i;
  $20 = ($19|0)<(16);
  if (!($20)) {
   break;
  }
  $21 = $i;
  $22 = $marker;
  $23 = (($22) + 32|0);
  $24 = (($23) + ($21<<2)|0);
  HEAP32[$24>>2] = 33;
  $25 = $i;
  $26 = $marker;
  $27 = (($26) + 100|0);
  $28 = (($27) + ($25<<2)|0);
  HEAP32[$28>>2] = 0;
  $29 = $i;
  $30 = (($29) + 1)|0;
  $i = $30;
 }
 $31 = $marker;
 $32 = (($31) + 32|0);
 HEAP32[$32>>2] = 34;
 $33 = $marker;
 $34 = (($33) + 32|0);
 $35 = (($34) + 56|0);
 HEAP32[$35>>2] = 34;
 $36 = $0;
 _reset_marker_reader($36);
 STACKTOP = sp;return;
}
function _reset_marker_reader($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $marker = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 440|0);
 $3 = HEAP32[$2>>2]|0;
 $marker = $3;
 $4 = $0;
 $5 = (($4) + 216|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 $7 = (($6) + 144|0);
 HEAP32[$7>>2] = 0;
 $8 = $0;
 $9 = (($8) + 416|0);
 HEAP32[$9>>2] = 0;
 $10 = $marker;
 $11 = (($10) + 12|0);
 HEAP32[$11>>2] = 0;
 $12 = $marker;
 $13 = (($12) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = $marker;
 $15 = (($14) + 24|0);
 HEAP32[$15>>2] = 0;
 $16 = $marker;
 $17 = (($16) + 164|0);
 HEAP32[$17>>2] = 0;
 STACKTOP = sp;return;
}
function _read_markers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 L1: while(1) {
  $2 = $1;
  $3 = (($2) + 416|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)==(0);
  if ($5) {
   $6 = $1;
   $7 = (($6) + 440|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (($8) + 12|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0);
   if ($11) {
    $15 = $1;
    $16 = (_next_marker($15)|0);
    $17 = ($16|0)!=(0);
    if (!($17)) {
     label = 8;
     break;
    }
   } else {
    $12 = $1;
    $13 = (_first_marker($12)|0);
    $14 = ($13|0)!=(0);
    if (!($14)) {
     label = 5;
     break;
    }
   }
  }
  $18 = $1;
  $19 = (($18) + 416|0);
  $20 = HEAP32[$19>>2]|0;
  do {
   switch ($20|0) {
   case 194:  {
    $27 = $1;
    $28 = (_get_sof($27,1,0)|0);
    $29 = ($28|0)!=(0);
    if (!($29)) {
     label = 19;
     break L1;
    }
    break;
   }
   case 218:  {
    label = 28;
    break L1;
    break;
   }
   case 202:  {
    $33 = $1;
    $34 = (_get_sof($33,1,1)|0);
    $35 = ($34|0)!=(0);
    if (!($35)) {
     label = 25;
     break L1;
    }
    break;
   }
   case 219:  {
    $70 = $1;
    $71 = (_get_dqt($70)|0);
    $72 = ($71|0)!=(0);
    if (!($72)) {
     label = 39;
     break L1;
    }
    break;
   }
   case 217:  {
    label = 31;
    break L1;
    break;
   }
   case 204:  {
    $64 = $1;
    $65 = (_skip_variable($64)|0);
    $66 = ($65|0)!=(0);
    if (!($66)) {
     label = 33;
     break L1;
    }
    break;
   }
   case 201:  {
    $30 = $1;
    $31 = (_get_sof($30,0,1)|0);
    $32 = ($31|0)!=(0);
    if (!($32)) {
     label = 22;
     break L1;
    }
    break;
   }
   case 216:  {
    $21 = $1;
    $22 = (_get_soi($21)|0);
    $23 = ($22|0)!=(0);
    if (!($23)) {
     label = 13;
     break L1;
    }
    break;
   }
   case 221:  {
    $73 = $1;
    $74 = (_get_dri($73)|0);
    $75 = ($74|0)!=(0);
    if (!($75)) {
     label = 42;
     break L1;
    }
    break;
   }
   case 193: case 192:  {
    $24 = $1;
    $25 = (_get_sof($24,0,0)|0);
    $26 = ($25|0)!=(0);
    if (!($26)) {
     label = 16;
     break L1;
    }
    break;
   }
   case 196:  {
    $67 = $1;
    $68 = (_get_dht($67)|0);
    $69 = ($68|0)!=(0);
    if (!($69)) {
     label = 36;
     break L1;
    }
    break;
   }
   case 1: case 215: case 214: case 213: case 212: case 211: case 210: case 209: case 208:  {
    $97 = $1;
    $98 = HEAP32[$97>>2]|0;
    $99 = (($98) + 20|0);
    HEAP32[$99>>2] = 92;
    $100 = $1;
    $101 = (($100) + 416|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = $1;
    $104 = HEAP32[$103>>2]|0;
    $105 = (($104) + 24|0);
    HEAP32[$105>>2] = $102;
    $106 = $1;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($107) + 4|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = $1;
    FUNCTION_TABLE_vii[$109 & 127]($110,1);
    break;
   }
   case 220:  {
    $111 = $1;
    $112 = (_skip_variable($111)|0);
    $113 = ($112|0)!=(0);
    if (!($113)) {
     label = 52;
     break L1;
    }
    break;
   }
   case 239: case 238: case 237: case 236: case 235: case 234: case 233: case 232: case 231: case 230: case 229: case 228: case 227: case 226: case 225: case 224:  {
    $76 = $1;
    $77 = (($76) + 416|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) - 224)|0;
    $80 = $1;
    $81 = (($80) + 440|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (($82) + 32|0);
    $84 = (($83) + ($79<<2)|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = $1;
    $87 = (FUNCTION_TABLE_ii[$85 & 127]($86)|0);
    $88 = ($87|0)!=(0);
    if (!($88)) {
     label = 45;
     break L1;
    }
    break;
   }
   case 254:  {
    $89 = $1;
    $90 = (($89) + 440|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = (($91) + 28|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = $1;
    $95 = (FUNCTION_TABLE_ii[$93 & 127]($94)|0);
    $96 = ($95|0)!=(0);
    if (!($96)) {
     label = 48;
     break L1;
    }
    break;
   }
   case 207: case 206: case 205: case 203: case 200: case 199: case 198: case 197: case 195:  {
    $36 = $1;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + 20|0);
    HEAP32[$38>>2] = 60;
    $39 = $1;
    $40 = (($39) + 416|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $1;
    $43 = HEAP32[$42>>2]|0;
    $44 = (($43) + 24|0);
    HEAP32[$44>>2] = $41;
    $45 = $1;
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$46>>2]|0;
    $48 = $1;
    FUNCTION_TABLE_vi[$47 & 127]($48);
    break;
   }
   default: {
    $114 = $1;
    $115 = HEAP32[$114>>2]|0;
    $116 = (($115) + 20|0);
    HEAP32[$116>>2] = 68;
    $117 = $1;
    $118 = (($117) + 416|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = $1;
    $121 = HEAP32[$120>>2]|0;
    $122 = (($121) + 24|0);
    HEAP32[$122>>2] = $119;
    $123 = $1;
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP32[$124>>2]|0;
    $126 = $1;
    FUNCTION_TABLE_vi[$125 & 127]($126);
   }
   }
  } while(0);
  $127 = $1;
  $128 = (($127) + 416|0);
  HEAP32[$128>>2] = 0;
 }
 switch (label|0) {
  case 5: {
   $0 = 0;
   break;
  }
  case 8: {
   $0 = 0;
   break;
  }
  case 13: {
   $0 = 0;
   break;
  }
  case 16: {
   $0 = 0;
   break;
  }
  case 19: {
   $0 = 0;
   break;
  }
  case 22: {
   $0 = 0;
   break;
  }
  case 25: {
   $0 = 0;
   break;
  }
  case 28: {
   $49 = $1;
   $50 = (_get_sos($49)|0);
   $51 = ($50|0)!=(0);
   if ($51) {
    $52 = $1;
    $53 = (($52) + 416|0);
    HEAP32[$53>>2] = 0;
    $0 = 1;
    break;
   } else {
    $0 = 0;
    break;
   }
   break;
  }
  case 31: {
   $54 = $1;
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 20|0);
   HEAP32[$56>>2] = 85;
   $57 = $1;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($58) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $1;
   FUNCTION_TABLE_vii[$60 & 127]($61,1);
   $62 = $1;
   $63 = (($62) + 416|0);
   HEAP32[$63>>2] = 0;
   $0 = 2;
   break;
  }
  case 33: {
   $0 = 0;
   break;
  }
  case 36: {
   $0 = 0;
   break;
  }
  case 39: {
   $0 = 0;
   break;
  }
  case 42: {
   $0 = 0;
   break;
  }
  case 45: {
   $0 = 0;
   break;
  }
  case 48: {
   $0 = 0;
   break;
  }
  case 52: {
   $0 = 0;
   break;
  }
 }
 $129 = $0;
 STACKTOP = sp;return ($129|0);
}
function _read_restart_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 416|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = $1;
   $7 = (_next_marker($6)|0);
   $8 = ($7|0)!=(0);
   if ($8) {
    break;
   }
   $0 = 0;
   $61 = $0;
   STACKTOP = sp;return ($61|0);
  }
 } while(0);
 $9 = $1;
 $10 = (($9) + 416|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $1;
 $13 = (($12) + 440|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 20|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (208 + ($16))|0;
 $18 = ($11|0)==($17|0);
 do {
  if ($18) {
   $19 = $1;
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 20|0);
   HEAP32[$21>>2] = 98;
   $22 = $1;
   $23 = (($22) + 440|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + 20|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $1;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 24|0);
   HEAP32[$29>>2] = $26;
   $30 = $1;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $1;
   FUNCTION_TABLE_vii[$33 & 127]($34,3);
   $35 = $1;
   $36 = (($35) + 416|0);
   HEAP32[$36>>2] = 0;
  } else {
   $37 = $1;
   $38 = (($37) + 24|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + 20|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = $1;
   $43 = $1;
   $44 = (($43) + 440|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (($45) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (FUNCTION_TABLE_iii[$41 & 127]($42,$47)|0);
   $49 = ($48|0)!=(0);
   if ($49) {
    break;
   }
   $0 = 0;
   $61 = $0;
   STACKTOP = sp;return ($61|0);
  }
 } while(0);
 $50 = $1;
 $51 = (($50) + 440|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + 20|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 1)|0;
 $56 = $55 & 7;
 $57 = $1;
 $58 = (($57) + 440|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + 20|0);
 HEAP32[$60>>2] = $56;
 $0 = 1;
 $61 = $0;
 STACKTOP = sp;return ($61|0);
}
function _skip_variable($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $bytes_in_buffer = 0, $datasrc = 0, $length = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $86 = $0;
   STACKTOP = sp;return ($86|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $86 = $0;
   STACKTOP = sp;return ($86|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 $53 = $1;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 20|0);
 HEAP32[$55>>2] = 91;
 $56 = $1;
 $57 = (($56) + 416|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $1;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + 24|0);
 HEAP32[$61>>2] = $58;
 $62 = $length;
 $63 = $1;
 $64 = HEAP32[$63>>2]|0;
 $65 = (($64) + 24|0);
 $66 = (($65) + 4|0);
 HEAP32[$66>>2] = $62;
 $67 = $1;
 $68 = HEAP32[$67>>2]|0;
 $69 = (($68) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $1;
 FUNCTION_TABLE_vii[$70 & 127]($71,1);
 $72 = $next_input_byte;
 $73 = $datasrc;
 HEAP32[$73>>2] = $72;
 $74 = $bytes_in_buffer;
 $75 = $datasrc;
 $76 = (($75) + 4|0);
 HEAP32[$76>>2] = $74;
 $77 = $length;
 $78 = ($77|0)>(0);
 if ($78) {
  $79 = $1;
  $80 = (($79) + 24|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 16|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $1;
  $85 = $length;
  FUNCTION_TABLE_vii[$83 & 127]($84,$85);
 }
 $0 = 1;
 $86 = $0;
 STACKTOP = sp;return ($86|0);
}
function _get_interesting_appn($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $b = 0, $bytes_in_buffer = 0, $datasrc = 0, $i = 0, $length = 0, $next_input_byte = 0, $numtoread = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $b = sp + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $122 = $0;
   STACKTOP = sp;return ($122|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $122 = $0;
   STACKTOP = sp;return ($122|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 $53 = $length;
 $54 = ($53|0)>=(14);
 if ($54) {
  $numtoread = 14;
 } else {
  $55 = $length;
  $56 = ($55|0)>(0);
  if ($56) {
   $57 = $length;
   $numtoread = $57;
  } else {
   $numtoread = 0;
  }
 }
 $i = 0;
 while(1) {
  $58 = $i;
  $59 = $numtoread;
  $60 = ($58>>>0)<($59>>>0);
  if (!($60)) {
   break;
  }
  $61 = $bytes_in_buffer;
  $62 = ($61|0)==(0);
  if ($62) {
   $63 = $datasrc;
   $64 = (($63) + 12|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $1;
   $67 = (FUNCTION_TABLE_ii[$65 & 127]($66)|0);
   $68 = ($67|0)!=(0);
   if (!($68)) {
    label = 22;
    break;
   }
   $69 = $datasrc;
   $70 = HEAP32[$69>>2]|0;
   $next_input_byte = $70;
   $71 = $datasrc;
   $72 = (($71) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $bytes_in_buffer = $73;
  }
  $74 = $bytes_in_buffer;
  $75 = (($74) + -1)|0;
  $bytes_in_buffer = $75;
  $76 = $next_input_byte;
  $77 = (($76) + 1|0);
  $next_input_byte = $77;
  $78 = HEAP8[$76>>0]|0;
  $79 = $i;
  $80 = (($b) + ($79)|0);
  HEAP8[$80>>0] = $78;
  $81 = $i;
  $82 = (($81) + 1)|0;
  $i = $82;
 }
 if ((label|0) == 22) {
  $0 = 0;
  $122 = $0;
  STACKTOP = sp;return ($122|0);
 }
 $83 = $numtoread;
 $84 = $length;
 $85 = (($84) - ($83))|0;
 $length = $85;
 $86 = $1;
 $87 = (($86) + 416|0);
 $88 = HEAP32[$87>>2]|0;
 if ((($88|0) == 224)) {
  $89 = $1;
  $90 = $numtoread;
  $91 = $length;
  _examine_app0($89,$b,$90,$91);
 } else if ((($88|0) == 238)) {
  $92 = $1;
  $93 = $numtoread;
  $94 = $length;
  _examine_app14($92,$b,$93,$94);
 } else {
  $95 = $1;
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 20|0);
  HEAP32[$97>>2] = 68;
  $98 = $1;
  $99 = (($98) + 416|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = $1;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($102) + 24|0);
  HEAP32[$103>>2] = $100;
  $104 = $1;
  $105 = HEAP32[$104>>2]|0;
  $106 = HEAP32[$105>>2]|0;
  $107 = $1;
  FUNCTION_TABLE_vi[$106 & 127]($107);
 }
 $108 = $next_input_byte;
 $109 = $datasrc;
 HEAP32[$109>>2] = $108;
 $110 = $bytes_in_buffer;
 $111 = $datasrc;
 $112 = (($111) + 4|0);
 HEAP32[$112>>2] = $110;
 $113 = $length;
 $114 = ($113|0)>(0);
 if ($114) {
  $115 = $1;
  $116 = (($115) + 24|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = (($117) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = $1;
  $121 = $length;
  FUNCTION_TABLE_vii[$119 & 127]($120,$121);
 }
 $0 = 1;
 $122 = $0;
 STACKTOP = sp;return ($122|0);
}
function _jpeg_set_marker_processor($cinfo,$marker_code,$routine) {
 $cinfo = $cinfo|0;
 $marker_code = $marker_code|0;
 $routine = $routine|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $marker = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $marker_code;
 $2 = $routine;
 $3 = $0;
 $4 = (($3) + 440|0);
 $5 = HEAP32[$4>>2]|0;
 $marker = $5;
 $6 = $1;
 $7 = ($6|0)==(254);
 if ($7) {
  $8 = $2;
  $9 = $marker;
  $10 = (($9) + 28|0);
  HEAP32[$10>>2] = $8;
  STACKTOP = sp;return;
 }
 $11 = $1;
 $12 = ($11|0)>=(224);
 if ($12) {
  $13 = $1;
  $14 = ($13|0)<=(239);
  if ($14) {
   $15 = $2;
   $16 = $1;
   $17 = (($16) - 224)|0;
   $18 = $marker;
   $19 = (($18) + 32|0);
   $20 = (($19) + ($17<<2)|0);
   HEAP32[$20>>2] = $15;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 68;
  $24 = $1;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 24|0);
  HEAP32[$27>>2] = $24;
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vi[$30 & 127]($31);
 }
 STACKTOP = sp;return;
}
function _examine_app0($cinfo,$data,$datalen,$remaining) {
 $cinfo = $cinfo|0;
 $data = $data|0;
 $datalen = $datalen|0;
 $remaining = $remaining|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $_mp = 0, $totallen = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $data;
 $2 = $datalen;
 $3 = $remaining;
 $4 = $2;
 $5 = $3;
 $6 = (($4) + ($5))|0;
 $totallen = $6;
 $7 = $2;
 $8 = ($7>>>0)>=(14);
 if ($8) {
  $9 = $1;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)==(74);
  if ($12) {
   $13 = $1;
   $14 = (($13) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)==(70);
   if ($17) {
    $18 = $1;
    $19 = (($18) + 2|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    $22 = ($21|0)==(73);
    if ($22) {
     $23 = $1;
     $24 = (($23) + 3|0);
     $25 = HEAP8[$24>>0]|0;
     $26 = $25&255;
     $27 = ($26|0)==(70);
     if ($27) {
      $28 = $1;
      $29 = (($28) + 4|0);
      $30 = HEAP8[$29>>0]|0;
      $31 = $30&255;
      $32 = ($31|0)==(0);
      if ($32) {
       $33 = $0;
       $34 = (($33) + 280|0);
       HEAP32[$34>>2] = 1;
       $35 = $1;
       $36 = (($35) + 5|0);
       $37 = HEAP8[$36>>0]|0;
       $38 = $0;
       $39 = (($38) + 284|0);
       HEAP8[$39>>0] = $37;
       $40 = $1;
       $41 = (($40) + 6|0);
       $42 = HEAP8[$41>>0]|0;
       $43 = $0;
       $44 = (($43) + 285|0);
       HEAP8[$44>>0] = $42;
       $45 = $1;
       $46 = (($45) + 7|0);
       $47 = HEAP8[$46>>0]|0;
       $48 = $0;
       $49 = (($48) + 286|0);
       HEAP8[$49>>0] = $47;
       $50 = $1;
       $51 = (($50) + 8|0);
       $52 = HEAP8[$51>>0]|0;
       $53 = $52&255;
       $54 = $53 << 8;
       $55 = $1;
       $56 = (($55) + 9|0);
       $57 = HEAP8[$56>>0]|0;
       $58 = $57&255;
       $59 = (($54) + ($58))|0;
       $60 = $59&65535;
       $61 = $0;
       $62 = (($61) + 288|0);
       HEAP16[$62>>1] = $60;
       $63 = $1;
       $64 = (($63) + 10|0);
       $65 = HEAP8[$64>>0]|0;
       $66 = $65&255;
       $67 = $66 << 8;
       $68 = $1;
       $69 = (($68) + 11|0);
       $70 = HEAP8[$69>>0]|0;
       $71 = $70&255;
       $72 = (($67) + ($71))|0;
       $73 = $72&65535;
       $74 = $0;
       $75 = (($74) + 290|0);
       HEAP16[$75>>1] = $73;
       $76 = $0;
       $77 = (($76) + 284|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78&255;
       $80 = ($79|0)!=(1);
       if ($80) {
        $81 = $0;
        $82 = HEAP32[$81>>2]|0;
        $83 = (($82) + 20|0);
        HEAP32[$83>>2] = 119;
        $84 = $0;
        $85 = (($84) + 284|0);
        $86 = HEAP8[$85>>0]|0;
        $87 = $86&255;
        $88 = $0;
        $89 = HEAP32[$88>>2]|0;
        $90 = (($89) + 24|0);
        HEAP32[$90>>2] = $87;
        $91 = $0;
        $92 = (($91) + 285|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = $93&255;
        $95 = $0;
        $96 = HEAP32[$95>>2]|0;
        $97 = (($96) + 24|0);
        $98 = (($97) + 4|0);
        HEAP32[$98>>2] = $94;
        $99 = $0;
        $100 = HEAP32[$99>>2]|0;
        $101 = (($100) + 4|0);
        $102 = HEAP32[$101>>2]|0;
        $103 = $0;
        FUNCTION_TABLE_vii[$102 & 127]($103,-1);
       }
       $104 = $0;
       $105 = HEAP32[$104>>2]|0;
       $106 = (($105) + 24|0);
       $_mp = $106;
       $107 = $0;
       $108 = (($107) + 284|0);
       $109 = HEAP8[$108>>0]|0;
       $110 = $109&255;
       $111 = $_mp;
       HEAP32[$111>>2] = $110;
       $112 = $0;
       $113 = (($112) + 285|0);
       $114 = HEAP8[$113>>0]|0;
       $115 = $114&255;
       $116 = $_mp;
       $117 = (($116) + 4|0);
       HEAP32[$117>>2] = $115;
       $118 = $0;
       $119 = (($118) + 288|0);
       $120 = HEAP16[$119>>1]|0;
       $121 = $120&65535;
       $122 = $_mp;
       $123 = (($122) + 8|0);
       HEAP32[$123>>2] = $121;
       $124 = $0;
       $125 = (($124) + 290|0);
       $126 = HEAP16[$125>>1]|0;
       $127 = $126&65535;
       $128 = $_mp;
       $129 = (($128) + 12|0);
       HEAP32[$129>>2] = $127;
       $130 = $0;
       $131 = (($130) + 286|0);
       $132 = HEAP8[$131>>0]|0;
       $133 = $132&255;
       $134 = $_mp;
       $135 = (($134) + 16|0);
       HEAP32[$135>>2] = $133;
       $136 = $0;
       $137 = HEAP32[$136>>2]|0;
       $138 = (($137) + 20|0);
       HEAP32[$138>>2] = 87;
       $139 = $0;
       $140 = HEAP32[$139>>2]|0;
       $141 = (($140) + 4|0);
       $142 = HEAP32[$141>>2]|0;
       $143 = $0;
       FUNCTION_TABLE_vii[$142 & 127]($143,1);
       $144 = $1;
       $145 = (($144) + 12|0);
       $146 = HEAP8[$145>>0]|0;
       $147 = $146&255;
       $148 = $1;
       $149 = (($148) + 13|0);
       $150 = HEAP8[$149>>0]|0;
       $151 = $150&255;
       $152 = $147 | $151;
       $153 = ($152|0)!=(0);
       if ($153) {
        $154 = $0;
        $155 = HEAP32[$154>>2]|0;
        $156 = (($155) + 20|0);
        HEAP32[$156>>2] = 90;
        $157 = $1;
        $158 = (($157) + 12|0);
        $159 = HEAP8[$158>>0]|0;
        $160 = $159&255;
        $161 = $0;
        $162 = HEAP32[$161>>2]|0;
        $163 = (($162) + 24|0);
        HEAP32[$163>>2] = $160;
        $164 = $1;
        $165 = (($164) + 13|0);
        $166 = HEAP8[$165>>0]|0;
        $167 = $166&255;
        $168 = $0;
        $169 = HEAP32[$168>>2]|0;
        $170 = (($169) + 24|0);
        $171 = (($170) + 4|0);
        HEAP32[$171>>2] = $167;
        $172 = $0;
        $173 = HEAP32[$172>>2]|0;
        $174 = (($173) + 4|0);
        $175 = HEAP32[$174>>2]|0;
        $176 = $0;
        FUNCTION_TABLE_vii[$175 & 127]($176,1);
       }
       $177 = $totallen;
       $178 = (($177) - 14)|0;
       $totallen = $178;
       $179 = $totallen;
       $180 = $1;
       $181 = (($180) + 12|0);
       $182 = HEAP8[$181>>0]|0;
       $183 = $182&255;
       $184 = $1;
       $185 = (($184) + 13|0);
       $186 = HEAP8[$185>>0]|0;
       $187 = $186&255;
       $188 = Math_imul($183, $187)|0;
       $189 = ($188*3)|0;
       $190 = ($179|0)!=($189|0);
       if ($190) {
        $191 = $0;
        $192 = HEAP32[$191>>2]|0;
        $193 = (($192) + 20|0);
        HEAP32[$193>>2] = 88;
        $194 = $totallen;
        $195 = $0;
        $196 = HEAP32[$195>>2]|0;
        $197 = (($196) + 24|0);
        HEAP32[$197>>2] = $194;
        $198 = $0;
        $199 = HEAP32[$198>>2]|0;
        $200 = (($199) + 4|0);
        $201 = HEAP32[$200>>2]|0;
        $202 = $0;
        FUNCTION_TABLE_vii[$201 & 127]($202,1);
       }
       STACKTOP = sp;return;
      }
     }
    }
   }
  }
 }
 $203 = $2;
 $204 = ($203>>>0)>=(6);
 if ($204) {
  $205 = $1;
  $206 = HEAP8[$205>>0]|0;
  $207 = $206&255;
  $208 = ($207|0)==(74);
  if ($208) {
   $209 = $1;
   $210 = (($209) + 1|0);
   $211 = HEAP8[$210>>0]|0;
   $212 = $211&255;
   $213 = ($212|0)==(70);
   if ($213) {
    $214 = $1;
    $215 = (($214) + 2|0);
    $216 = HEAP8[$215>>0]|0;
    $217 = $216&255;
    $218 = ($217|0)==(88);
    if ($218) {
     $219 = $1;
     $220 = (($219) + 3|0);
     $221 = HEAP8[$220>>0]|0;
     $222 = $221&255;
     $223 = ($222|0)==(88);
     if ($223) {
      $224 = $1;
      $225 = (($224) + 4|0);
      $226 = HEAP8[$225>>0]|0;
      $227 = $226&255;
      $228 = ($227|0)==(0);
      if ($228) {
       $229 = $1;
       $230 = (($229) + 5|0);
       $231 = HEAP8[$230>>0]|0;
       $232 = $231&255;
       if ((($232|0) == 16)) {
        $233 = $0;
        $234 = HEAP32[$233>>2]|0;
        $235 = (($234) + 20|0);
        HEAP32[$235>>2] = 108;
        $236 = $totallen;
        $237 = $0;
        $238 = HEAP32[$237>>2]|0;
        $239 = (($238) + 24|0);
        HEAP32[$239>>2] = $236;
        $240 = $0;
        $241 = HEAP32[$240>>2]|0;
        $242 = (($241) + 4|0);
        $243 = HEAP32[$242>>2]|0;
        $244 = $0;
        FUNCTION_TABLE_vii[$243 & 127]($244,1);
       } else if ((($232|0) == 17)) {
        $245 = $0;
        $246 = HEAP32[$245>>2]|0;
        $247 = (($246) + 20|0);
        HEAP32[$247>>2] = 109;
        $248 = $totallen;
        $249 = $0;
        $250 = HEAP32[$249>>2]|0;
        $251 = (($250) + 24|0);
        HEAP32[$251>>2] = $248;
        $252 = $0;
        $253 = HEAP32[$252>>2]|0;
        $254 = (($253) + 4|0);
        $255 = HEAP32[$254>>2]|0;
        $256 = $0;
        FUNCTION_TABLE_vii[$255 & 127]($256,1);
       } else if ((($232|0) == 19)) {
        $257 = $0;
        $258 = HEAP32[$257>>2]|0;
        $259 = (($258) + 20|0);
        HEAP32[$259>>2] = 110;
        $260 = $totallen;
        $261 = $0;
        $262 = HEAP32[$261>>2]|0;
        $263 = (($262) + 24|0);
        HEAP32[$263>>2] = $260;
        $264 = $0;
        $265 = HEAP32[$264>>2]|0;
        $266 = (($265) + 4|0);
        $267 = HEAP32[$266>>2]|0;
        $268 = $0;
        FUNCTION_TABLE_vii[$267 & 127]($268,1);
       } else {
        $269 = $0;
        $270 = HEAP32[$269>>2]|0;
        $271 = (($270) + 20|0);
        HEAP32[$271>>2] = 89;
        $272 = $1;
        $273 = (($272) + 5|0);
        $274 = HEAP8[$273>>0]|0;
        $275 = $274&255;
        $276 = $0;
        $277 = HEAP32[$276>>2]|0;
        $278 = (($277) + 24|0);
        HEAP32[$278>>2] = $275;
        $279 = $totallen;
        $280 = $0;
        $281 = HEAP32[$280>>2]|0;
        $282 = (($281) + 24|0);
        $283 = (($282) + 4|0);
        HEAP32[$283>>2] = $279;
        $284 = $0;
        $285 = HEAP32[$284>>2]|0;
        $286 = (($285) + 4|0);
        $287 = HEAP32[$286>>2]|0;
        $288 = $0;
        FUNCTION_TABLE_vii[$287 & 127]($288,1);
       }
      } else {
       label = 28;
      }
     } else {
      label = 28;
     }
    } else {
     label = 28;
    }
   } else {
    label = 28;
   }
  } else {
   label = 28;
  }
 } else {
  label = 28;
 }
 if ((label|0) == 28) {
  $289 = $0;
  $290 = HEAP32[$289>>2]|0;
  $291 = (($290) + 20|0);
  HEAP32[$291>>2] = 77;
  $292 = $totallen;
  $293 = $0;
  $294 = HEAP32[$293>>2]|0;
  $295 = (($294) + 24|0);
  HEAP32[$295>>2] = $292;
  $296 = $0;
  $297 = HEAP32[$296>>2]|0;
  $298 = (($297) + 4|0);
  $299 = HEAP32[$298>>2]|0;
  $300 = $0;
  FUNCTION_TABLE_vii[$299 & 127]($300,1);
 }
 STACKTOP = sp;return;
}
function _examine_app14($cinfo,$data,$datalen,$remaining) {
 $cinfo = $cinfo|0;
 $data = $data|0;
 $datalen = $datalen|0;
 $remaining = $remaining|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $flags0 = 0, $flags1 = 0, $transform = 0, $version = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $data;
 $2 = $datalen;
 $3 = $remaining;
 $4 = $2;
 $5 = ($4>>>0)>=(12);
 if ($5) {
  $6 = $1;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(65);
  if ($9) {
   $10 = $1;
   $11 = (($10) + 1|0);
   $12 = HEAP8[$11>>0]|0;
   $13 = $12&255;
   $14 = ($13|0)==(100);
   if ($14) {
    $15 = $1;
    $16 = (($15) + 2|0);
    $17 = HEAP8[$16>>0]|0;
    $18 = $17&255;
    $19 = ($18|0)==(111);
    if ($19) {
     $20 = $1;
     $21 = (($20) + 3|0);
     $22 = HEAP8[$21>>0]|0;
     $23 = $22&255;
     $24 = ($23|0)==(98);
     if ($24) {
      $25 = $1;
      $26 = (($25) + 4|0);
      $27 = HEAP8[$26>>0]|0;
      $28 = $27&255;
      $29 = ($28|0)==(101);
      if ($29) {
       $30 = $1;
       $31 = (($30) + 5|0);
       $32 = HEAP8[$31>>0]|0;
       $33 = $32&255;
       $34 = $33 << 8;
       $35 = $1;
       $36 = (($35) + 6|0);
       $37 = HEAP8[$36>>0]|0;
       $38 = $37&255;
       $39 = (($34) + ($38))|0;
       $version = $39;
       $40 = $1;
       $41 = (($40) + 7|0);
       $42 = HEAP8[$41>>0]|0;
       $43 = $42&255;
       $44 = $43 << 8;
       $45 = $1;
       $46 = (($45) + 8|0);
       $47 = HEAP8[$46>>0]|0;
       $48 = $47&255;
       $49 = (($44) + ($48))|0;
       $flags0 = $49;
       $50 = $1;
       $51 = (($50) + 9|0);
       $52 = HEAP8[$51>>0]|0;
       $53 = $52&255;
       $54 = $53 << 8;
       $55 = $1;
       $56 = (($55) + 10|0);
       $57 = HEAP8[$56>>0]|0;
       $58 = $57&255;
       $59 = (($54) + ($58))|0;
       $flags1 = $59;
       $60 = $1;
       $61 = (($60) + 11|0);
       $62 = HEAP8[$61>>0]|0;
       $63 = $62&255;
       $transform = $63;
       $64 = $0;
       $65 = HEAP32[$64>>2]|0;
       $66 = (($65) + 24|0);
       $_mp = $66;
       $67 = $version;
       $68 = $_mp;
       HEAP32[$68>>2] = $67;
       $69 = $flags0;
       $70 = $_mp;
       $71 = (($70) + 4|0);
       HEAP32[$71>>2] = $69;
       $72 = $flags1;
       $73 = $_mp;
       $74 = (($73) + 8|0);
       HEAP32[$74>>2] = $72;
       $75 = $transform;
       $76 = $_mp;
       $77 = (($76) + 12|0);
       HEAP32[$77>>2] = $75;
       $78 = $0;
       $79 = HEAP32[$78>>2]|0;
       $80 = (($79) + 20|0);
       HEAP32[$80>>2] = 76;
       $81 = $0;
       $82 = HEAP32[$81>>2]|0;
       $83 = (($82) + 4|0);
       $84 = HEAP32[$83>>2]|0;
       $85 = $0;
       FUNCTION_TABLE_vii[$84 & 127]($85,1);
       $86 = $0;
       $87 = (($86) + 292|0);
       HEAP32[$87>>2] = 1;
       $88 = $transform;
       $89 = $88&255;
       $90 = $0;
       $91 = (($90) + 296|0);
       HEAP8[$91>>0] = $89;
       STACKTOP = sp;return;
      }
     }
    }
   }
  }
 }
 $92 = $0;
 $93 = HEAP32[$92>>2]|0;
 $94 = (($93) + 20|0);
 HEAP32[$94>>2] = 78;
 $95 = $2;
 $96 = $3;
 $97 = (($95) + ($96))|0;
 $98 = $0;
 $99 = HEAP32[$98>>2]|0;
 $100 = (($99) + 24|0);
 HEAP32[$100>>2] = $97;
 $101 = $0;
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) + 4|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = $0;
 FUNCTION_TABLE_vii[$104 & 127]($105,1);
 STACKTOP = sp;return;
}
function _first_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $bytes_in_buffer = 0, $c = 0, $c2 = 0;
 var $datasrc = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $76 = $0;
   STACKTOP = sp;return ($76|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $c = $28;
 $29 = $bytes_in_buffer;
 $30 = ($29|0)==(0);
 do {
  if ($30) {
   $31 = $datasrc;
   $32 = (($31) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $1;
   $35 = (FUNCTION_TABLE_ii[$33 & 127]($34)|0);
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = $datasrc;
    $38 = HEAP32[$37>>2]|0;
    $next_input_byte = $38;
    $39 = $datasrc;
    $40 = (($39) + 4|0);
    $41 = HEAP32[$40>>2]|0;
    $bytes_in_buffer = $41;
    break;
   }
   $0 = 0;
   $76 = $0;
   STACKTOP = sp;return ($76|0);
  }
 } while(0);
 $42 = $bytes_in_buffer;
 $43 = (($42) + -1)|0;
 $bytes_in_buffer = $43;
 $44 = $next_input_byte;
 $45 = (($44) + 1|0);
 $next_input_byte = $45;
 $46 = HEAP8[$44>>0]|0;
 $47 = $46&255;
 $c2 = $47;
 $48 = $c;
 $49 = ($48|0)!=(255);
 if ($49) {
  label = 15;
 } else {
  $50 = $c2;
  $51 = ($50|0)!=(216);
  if ($51) {
   label = 15;
  }
 }
 if ((label|0) == 15) {
  $52 = $1;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + 20|0);
  HEAP32[$54>>2] = 53;
  $55 = $c;
  $56 = $1;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 24|0);
  HEAP32[$58>>2] = $55;
  $59 = $c2;
  $60 = $1;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 24|0);
  $63 = (($62) + 4|0);
  HEAP32[$63>>2] = $59;
  $64 = $1;
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$65>>2]|0;
  $67 = $1;
  FUNCTION_TABLE_vi[$66 & 127]($67);
 }
 $68 = $c2;
 $69 = $1;
 $70 = (($69) + 416|0);
 HEAP32[$70>>2] = $68;
 $71 = $next_input_byte;
 $72 = $datasrc;
 HEAP32[$72>>2] = $71;
 $73 = $bytes_in_buffer;
 $74 = $datasrc;
 $75 = (($74) + 4|0);
 HEAP32[$75>>2] = $73;
 $0 = 1;
 $76 = $0;
 STACKTOP = sp;return ($76|0);
}
function _get_soi($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 HEAP32[$3>>2] = 102;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 FUNCTION_TABLE_vii[$7 & 127]($8,1);
 $9 = $0;
 $10 = (($9) + 440|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 20|0);
  HEAP32[$17>>2] = 61;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 127]($21);
 }
 $i = 0;
 while(1) {
  $22 = $i;
  $23 = ($22|0)<(16);
  if (!($23)) {
   break;
  }
  $24 = $i;
  $25 = $0;
  $26 = (($25) + 228|0);
  $27 = (($26) + ($24)|0);
  HEAP8[$27>>0] = 0;
  $28 = $i;
  $29 = $0;
  $30 = (($29) + 244|0);
  $31 = (($30) + ($28)|0);
  HEAP8[$31>>0] = 1;
  $32 = $i;
  $33 = $0;
  $34 = (($33) + 260|0);
  $35 = (($34) + ($32)|0);
  HEAP8[$35>>0] = 5;
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 $38 = $0;
 $39 = (($38) + 276|0);
 HEAP32[$39>>2] = 0;
 $40 = $0;
 $41 = (($40) + 40|0);
 HEAP32[$41>>2] = 0;
 $42 = $0;
 $43 = (($42) + 300|0);
 HEAP32[$43>>2] = 0;
 $44 = $0;
 $45 = (($44) + 280|0);
 HEAP32[$45>>2] = 0;
 $46 = $0;
 $47 = (($46) + 284|0);
 HEAP8[$47>>0] = 1;
 $48 = $0;
 $49 = (($48) + 285|0);
 HEAP8[$49>>0] = 1;
 $50 = $0;
 $51 = (($50) + 286|0);
 HEAP8[$51>>0] = 0;
 $52 = $0;
 $53 = (($52) + 288|0);
 HEAP16[$53>>1] = 1;
 $54 = $0;
 $55 = (($54) + 290|0);
 HEAP16[$55>>1] = 1;
 $56 = $0;
 $57 = (($56) + 292|0);
 HEAP32[$57>>2] = 0;
 $58 = $0;
 $59 = (($58) + 296|0);
 HEAP8[$59>>0] = 0;
 $60 = $0;
 $61 = (($60) + 440|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (($62) + 12|0);
 HEAP32[$63>>2] = 1;
 STACKTOP = sp;return 1;
}
function _get_sof($cinfo,$is_prog,$is_arith) {
 $cinfo = $cinfo|0;
 $is_prog = $is_prog|0;
 $is_arith = $is_arith|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $_mp1 = 0, $bytes_in_buffer = 0, $c = 0, $ci = 0, $compptr = 0, $datasrc = 0, $length = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $cinfo;
 $2 = $is_prog;
 $3 = $is_arith;
 $4 = $1;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $datasrc = $6;
 $7 = $datasrc;
 $8 = HEAP32[$7>>2]|0;
 $next_input_byte = $8;
 $9 = $datasrc;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $bytes_in_buffer = $11;
 $12 = $2;
 $13 = $1;
 $14 = (($13) + 220|0);
 HEAP32[$14>>2] = $12;
 $15 = $3;
 $16 = $1;
 $17 = (($16) + 224|0);
 HEAP32[$17>>2] = $15;
 $18 = $bytes_in_buffer;
 $19 = ($18|0)==(0);
 do {
  if ($19) {
   $20 = $datasrc;
   $21 = (($20) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = $1;
   $24 = (FUNCTION_TABLE_ii[$22 & 127]($23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $datasrc;
    $27 = HEAP32[$26>>2]|0;
    $next_input_byte = $27;
    $28 = $datasrc;
    $29 = (($28) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $bytes_in_buffer = $30;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $31 = $bytes_in_buffer;
 $32 = (($31) + -1)|0;
 $bytes_in_buffer = $32;
 $33 = $next_input_byte;
 $34 = (($33) + 1|0);
 $next_input_byte = $34;
 $35 = HEAP8[$33>>0]|0;
 $36 = $35&255;
 $37 = $36 << 8;
 $length = $37;
 $38 = $bytes_in_buffer;
 $39 = ($38|0)==(0);
 do {
  if ($39) {
   $40 = $datasrc;
   $41 = (($40) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = $1;
   $44 = (FUNCTION_TABLE_ii[$42 & 127]($43)|0);
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = $datasrc;
    $47 = HEAP32[$46>>2]|0;
    $next_input_byte = $47;
    $48 = $datasrc;
    $49 = (($48) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $bytes_in_buffer = $50;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $51 = $bytes_in_buffer;
 $52 = (($51) + -1)|0;
 $bytes_in_buffer = $52;
 $53 = $next_input_byte;
 $54 = (($53) + 1|0);
 $next_input_byte = $54;
 $55 = HEAP8[$53>>0]|0;
 $56 = $55&255;
 $57 = $length;
 $58 = (($57) + ($56))|0;
 $length = $58;
 $59 = $bytes_in_buffer;
 $60 = ($59|0)==(0);
 do {
  if ($60) {
   $61 = $datasrc;
   $62 = (($61) + 12|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $1;
   $65 = (FUNCTION_TABLE_ii[$63 & 127]($64)|0);
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = $datasrc;
    $68 = HEAP32[$67>>2]|0;
    $next_input_byte = $68;
    $69 = $datasrc;
    $70 = (($69) + 4|0);
    $71 = HEAP32[$70>>2]|0;
    $bytes_in_buffer = $71;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $72 = $bytes_in_buffer;
 $73 = (($72) + -1)|0;
 $bytes_in_buffer = $73;
 $74 = $next_input_byte;
 $75 = (($74) + 1|0);
 $next_input_byte = $75;
 $76 = HEAP8[$74>>0]|0;
 $77 = $76&255;
 $78 = $1;
 $79 = (($78) + 212|0);
 HEAP32[$79>>2] = $77;
 $80 = $bytes_in_buffer;
 $81 = ($80|0)==(0);
 do {
  if ($81) {
   $82 = $datasrc;
   $83 = (($82) + 12|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = $1;
   $86 = (FUNCTION_TABLE_ii[$84 & 127]($85)|0);
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = $datasrc;
    $89 = HEAP32[$88>>2]|0;
    $next_input_byte = $89;
    $90 = $datasrc;
    $91 = (($90) + 4|0);
    $92 = HEAP32[$91>>2]|0;
    $bytes_in_buffer = $92;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $93 = $bytes_in_buffer;
 $94 = (($93) + -1)|0;
 $bytes_in_buffer = $94;
 $95 = $next_input_byte;
 $96 = (($95) + 1|0);
 $next_input_byte = $96;
 $97 = HEAP8[$95>>0]|0;
 $98 = $97&255;
 $99 = $98 << 8;
 $100 = $1;
 $101 = (($100) + 32|0);
 HEAP32[$101>>2] = $99;
 $102 = $bytes_in_buffer;
 $103 = ($102|0)==(0);
 do {
  if ($103) {
   $104 = $datasrc;
   $105 = (($104) + 12|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = $1;
   $108 = (FUNCTION_TABLE_ii[$106 & 127]($107)|0);
   $109 = ($108|0)!=(0);
   if ($109) {
    $110 = $datasrc;
    $111 = HEAP32[$110>>2]|0;
    $next_input_byte = $111;
    $112 = $datasrc;
    $113 = (($112) + 4|0);
    $114 = HEAP32[$113>>2]|0;
    $bytes_in_buffer = $114;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $115 = $bytes_in_buffer;
 $116 = (($115) + -1)|0;
 $bytes_in_buffer = $116;
 $117 = $next_input_byte;
 $118 = (($117) + 1|0);
 $next_input_byte = $118;
 $119 = HEAP8[$117>>0]|0;
 $120 = $119&255;
 $121 = $1;
 $122 = (($121) + 32|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = (($123) + ($120))|0;
 HEAP32[$122>>2] = $124;
 $125 = $bytes_in_buffer;
 $126 = ($125|0)==(0);
 do {
  if ($126) {
   $127 = $datasrc;
   $128 = (($127) + 12|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = $1;
   $131 = (FUNCTION_TABLE_ii[$129 & 127]($130)|0);
   $132 = ($131|0)!=(0);
   if ($132) {
    $133 = $datasrc;
    $134 = HEAP32[$133>>2]|0;
    $next_input_byte = $134;
    $135 = $datasrc;
    $136 = (($135) + 4|0);
    $137 = HEAP32[$136>>2]|0;
    $bytes_in_buffer = $137;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $138 = $bytes_in_buffer;
 $139 = (($138) + -1)|0;
 $bytes_in_buffer = $139;
 $140 = $next_input_byte;
 $141 = (($140) + 1|0);
 $next_input_byte = $141;
 $142 = HEAP8[$140>>0]|0;
 $143 = $142&255;
 $144 = $143 << 8;
 $145 = $1;
 $146 = (($145) + 28|0);
 HEAP32[$146>>2] = $144;
 $147 = $bytes_in_buffer;
 $148 = ($147|0)==(0);
 do {
  if ($148) {
   $149 = $datasrc;
   $150 = (($149) + 12|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = $1;
   $153 = (FUNCTION_TABLE_ii[$151 & 127]($152)|0);
   $154 = ($153|0)!=(0);
   if ($154) {
    $155 = $datasrc;
    $156 = HEAP32[$155>>2]|0;
    $next_input_byte = $156;
    $157 = $datasrc;
    $158 = (($157) + 4|0);
    $159 = HEAP32[$158>>2]|0;
    $bytes_in_buffer = $159;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $160 = $bytes_in_buffer;
 $161 = (($160) + -1)|0;
 $bytes_in_buffer = $161;
 $162 = $next_input_byte;
 $163 = (($162) + 1|0);
 $next_input_byte = $163;
 $164 = HEAP8[$162>>0]|0;
 $165 = $164&255;
 $166 = $1;
 $167 = (($166) + 28|0);
 $168 = HEAP32[$167>>2]|0;
 $169 = (($168) + ($165))|0;
 HEAP32[$167>>2] = $169;
 $170 = $bytes_in_buffer;
 $171 = ($170|0)==(0);
 do {
  if ($171) {
   $172 = $datasrc;
   $173 = (($172) + 12|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = $1;
   $176 = (FUNCTION_TABLE_ii[$174 & 127]($175)|0);
   $177 = ($176|0)!=(0);
   if ($177) {
    $178 = $datasrc;
    $179 = HEAP32[$178>>2]|0;
    $next_input_byte = $179;
    $180 = $datasrc;
    $181 = (($180) + 4|0);
    $182 = HEAP32[$181>>2]|0;
    $bytes_in_buffer = $182;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $183 = $bytes_in_buffer;
 $184 = (($183) + -1)|0;
 $bytes_in_buffer = $184;
 $185 = $next_input_byte;
 $186 = (($185) + 1|0);
 $next_input_byte = $186;
 $187 = HEAP8[$185>>0]|0;
 $188 = $187&255;
 $189 = $1;
 $190 = (($189) + 36|0);
 HEAP32[$190>>2] = $188;
 $191 = $length;
 $192 = (($191) - 8)|0;
 $length = $192;
 $193 = $1;
 $194 = HEAP32[$193>>2]|0;
 $195 = (($194) + 24|0);
 $_mp = $195;
 $196 = $1;
 $197 = (($196) + 416|0);
 $198 = HEAP32[$197>>2]|0;
 $199 = $_mp;
 HEAP32[$199>>2] = $198;
 $200 = $1;
 $201 = (($200) + 28|0);
 $202 = HEAP32[$201>>2]|0;
 $203 = $_mp;
 $204 = (($203) + 4|0);
 HEAP32[$204>>2] = $202;
 $205 = $1;
 $206 = (($205) + 32|0);
 $207 = HEAP32[$206>>2]|0;
 $208 = $_mp;
 $209 = (($208) + 8|0);
 HEAP32[$209>>2] = $207;
 $210 = $1;
 $211 = (($210) + 36|0);
 $212 = HEAP32[$211>>2]|0;
 $213 = $_mp;
 $214 = (($213) + 12|0);
 HEAP32[$214>>2] = $212;
 $215 = $1;
 $216 = HEAP32[$215>>2]|0;
 $217 = (($216) + 20|0);
 HEAP32[$217>>2] = 100;
 $218 = $1;
 $219 = HEAP32[$218>>2]|0;
 $220 = (($219) + 4|0);
 $221 = HEAP32[$220>>2]|0;
 $222 = $1;
 FUNCTION_TABLE_vii[$221 & 127]($222,1);
 $223 = $1;
 $224 = (($223) + 440|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + 16|0);
 $227 = HEAP32[$226>>2]|0;
 $228 = ($227|0)!=(0);
 if ($228) {
  $229 = $1;
  $230 = HEAP32[$229>>2]|0;
  $231 = (($230) + 20|0);
  HEAP32[$231>>2] = 58;
  $232 = $1;
  $233 = HEAP32[$232>>2]|0;
  $234 = HEAP32[$233>>2]|0;
  $235 = $1;
  FUNCTION_TABLE_vi[$234 & 127]($235);
 }
 $236 = $1;
 $237 = (($236) + 32|0);
 $238 = HEAP32[$237>>2]|0;
 $239 = ($238>>>0)<=(0);
 if ($239) {
  label = 50;
 } else {
  $240 = $1;
  $241 = (($240) + 28|0);
  $242 = HEAP32[$241>>2]|0;
  $243 = ($242>>>0)<=(0);
  if ($243) {
   label = 50;
  } else {
   $244 = $1;
   $245 = (($244) + 36|0);
   $246 = HEAP32[$245>>2]|0;
   $247 = ($246|0)<=(0);
   if ($247) {
    label = 50;
   }
  }
 }
 if ((label|0) == 50) {
  $248 = $1;
  $249 = HEAP32[$248>>2]|0;
  $250 = (($249) + 20|0);
  HEAP32[$250>>2] = 32;
  $251 = $1;
  $252 = HEAP32[$251>>2]|0;
  $253 = HEAP32[$252>>2]|0;
  $254 = $1;
  FUNCTION_TABLE_vi[$253 & 127]($254);
 }
 $255 = $length;
 $256 = $1;
 $257 = (($256) + 36|0);
 $258 = HEAP32[$257>>2]|0;
 $259 = ($258*3)|0;
 $260 = ($255|0)!=($259|0);
 if ($260) {
  $261 = $1;
  $262 = HEAP32[$261>>2]|0;
  $263 = (($262) + 20|0);
  HEAP32[$263>>2] = 11;
  $264 = $1;
  $265 = HEAP32[$264>>2]|0;
  $266 = HEAP32[$265>>2]|0;
  $267 = $1;
  FUNCTION_TABLE_vi[$266 & 127]($267);
 }
 $268 = $1;
 $269 = (($268) + 216|0);
 $270 = HEAP32[$269>>2]|0;
 $271 = ($270|0)==(0|0);
 if ($271) {
  $272 = $1;
  $273 = (($272) + 4|0);
  $274 = HEAP32[$273>>2]|0;
  $275 = HEAP32[$274>>2]|0;
  $276 = $1;
  $277 = $1;
  $278 = (($277) + 36|0);
  $279 = HEAP32[$278>>2]|0;
  $280 = ($279*84)|0;
  $281 = (FUNCTION_TABLE_iiii[$275 & 127]($276,1,$280)|0);
  $282 = $1;
  $283 = (($282) + 216|0);
  HEAP32[$283>>2] = $281;
 }
 $ci = 0;
 $284 = $1;
 $285 = (($284) + 216|0);
 $286 = HEAP32[$285>>2]|0;
 $compptr = $286;
 while(1) {
  $287 = $ci;
  $288 = $1;
  $289 = (($288) + 36|0);
  $290 = HEAP32[$289>>2]|0;
  $291 = ($287|0)<($290|0);
  if (!($291)) {
   label = 79;
   break;
  }
  $292 = $ci;
  $293 = $compptr;
  $294 = (($293) + 4|0);
  HEAP32[$294>>2] = $292;
  $295 = $bytes_in_buffer;
  $296 = ($295|0)==(0);
  if ($296) {
   $297 = $datasrc;
   $298 = (($297) + 12|0);
   $299 = HEAP32[$298>>2]|0;
   $300 = $1;
   $301 = (FUNCTION_TABLE_ii[$299 & 127]($300)|0);
   $302 = ($301|0)!=(0);
   if (!($302)) {
    label = 60;
    break;
   }
   $303 = $datasrc;
   $304 = HEAP32[$303>>2]|0;
   $next_input_byte = $304;
   $305 = $datasrc;
   $306 = (($305) + 4|0);
   $307 = HEAP32[$306>>2]|0;
   $bytes_in_buffer = $307;
  }
  $308 = $bytes_in_buffer;
  $309 = (($308) + -1)|0;
  $bytes_in_buffer = $309;
  $310 = $next_input_byte;
  $311 = (($310) + 1|0);
  $next_input_byte = $311;
  $312 = HEAP8[$310>>0]|0;
  $313 = $312&255;
  $314 = $compptr;
  HEAP32[$314>>2] = $313;
  $315 = $bytes_in_buffer;
  $316 = ($315|0)==(0);
  if ($316) {
   $317 = $datasrc;
   $318 = (($317) + 12|0);
   $319 = HEAP32[$318>>2]|0;
   $320 = $1;
   $321 = (FUNCTION_TABLE_ii[$319 & 127]($320)|0);
   $322 = ($321|0)!=(0);
   if (!($322)) {
    label = 66;
    break;
   }
   $323 = $datasrc;
   $324 = HEAP32[$323>>2]|0;
   $next_input_byte = $324;
   $325 = $datasrc;
   $326 = (($325) + 4|0);
   $327 = HEAP32[$326>>2]|0;
   $bytes_in_buffer = $327;
  }
  $328 = $bytes_in_buffer;
  $329 = (($328) + -1)|0;
  $bytes_in_buffer = $329;
  $330 = $next_input_byte;
  $331 = (($330) + 1|0);
  $next_input_byte = $331;
  $332 = HEAP8[$330>>0]|0;
  $333 = $332&255;
  $c = $333;
  $334 = $c;
  $335 = $334 >> 4;
  $336 = $335 & 15;
  $337 = $compptr;
  $338 = (($337) + 8|0);
  HEAP32[$338>>2] = $336;
  $339 = $c;
  $340 = $339 & 15;
  $341 = $compptr;
  $342 = (($341) + 12|0);
  HEAP32[$342>>2] = $340;
  $343 = $bytes_in_buffer;
  $344 = ($343|0)==(0);
  if ($344) {
   $345 = $datasrc;
   $346 = (($345) + 12|0);
   $347 = HEAP32[$346>>2]|0;
   $348 = $1;
   $349 = (FUNCTION_TABLE_ii[$347 & 127]($348)|0);
   $350 = ($349|0)!=(0);
   if (!($350)) {
    label = 72;
    break;
   }
   $351 = $datasrc;
   $352 = HEAP32[$351>>2]|0;
   $next_input_byte = $352;
   $353 = $datasrc;
   $354 = (($353) + 4|0);
   $355 = HEAP32[$354>>2]|0;
   $bytes_in_buffer = $355;
  }
  $356 = $bytes_in_buffer;
  $357 = (($356) + -1)|0;
  $bytes_in_buffer = $357;
  $358 = $next_input_byte;
  $359 = (($358) + 1|0);
  $next_input_byte = $359;
  $360 = HEAP8[$358>>0]|0;
  $361 = $360&255;
  $362 = $compptr;
  $363 = (($362) + 16|0);
  HEAP32[$363>>2] = $361;
  $364 = $1;
  $365 = HEAP32[$364>>2]|0;
  $366 = (($365) + 24|0);
  $_mp1 = $366;
  $367 = $compptr;
  $368 = HEAP32[$367>>2]|0;
  $369 = $_mp1;
  HEAP32[$369>>2] = $368;
  $370 = $compptr;
  $371 = (($370) + 8|0);
  $372 = HEAP32[$371>>2]|0;
  $373 = $_mp1;
  $374 = (($373) + 4|0);
  HEAP32[$374>>2] = $372;
  $375 = $compptr;
  $376 = (($375) + 12|0);
  $377 = HEAP32[$376>>2]|0;
  $378 = $_mp1;
  $379 = (($378) + 8|0);
  HEAP32[$379>>2] = $377;
  $380 = $compptr;
  $381 = (($380) + 16|0);
  $382 = HEAP32[$381>>2]|0;
  $383 = $_mp1;
  $384 = (($383) + 12|0);
  HEAP32[$384>>2] = $382;
  $385 = $1;
  $386 = HEAP32[$385>>2]|0;
  $387 = (($386) + 20|0);
  HEAP32[$387>>2] = 101;
  $388 = $1;
  $389 = HEAP32[$388>>2]|0;
  $390 = (($389) + 4|0);
  $391 = HEAP32[$390>>2]|0;
  $392 = $1;
  FUNCTION_TABLE_vii[$391 & 127]($392,1);
  $393 = $ci;
  $394 = (($393) + 1)|0;
  $ci = $394;
  $395 = $compptr;
  $396 = (($395) + 84|0);
  $compptr = $396;
 }
 if ((label|0) == 60) {
  $0 = 0;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 else if ((label|0) == 66) {
  $0 = 0;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 else if ((label|0) == 72) {
  $0 = 0;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 else if ((label|0) == 79) {
  $397 = $1;
  $398 = (($397) + 440|0);
  $399 = HEAP32[$398>>2]|0;
  $400 = (($399) + 16|0);
  HEAP32[$400>>2] = 1;
  $401 = $next_input_byte;
  $402 = $datasrc;
  HEAP32[$402>>2] = $401;
  $403 = $bytes_in_buffer;
  $404 = $datasrc;
  $405 = (($404) + 4|0);
  HEAP32[$405>>2] = $403;
  $0 = 1;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 return 0|0;
}
function _get_sos($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $_mp1 = 0, $bytes_in_buffer = 0;
 var $c = 0, $cc = 0, $ci = 0, $compptr = 0, $datasrc = 0, $i = 0, $length = 0, $n = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $1;
 $11 = (($10) + 440|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0);
 if (!($15)) {
  $16 = $1;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 20|0);
  HEAP32[$18>>2] = 62;
  $19 = $1;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = $1;
  FUNCTION_TABLE_vi[$21 & 127]($22);
 }
 $23 = $bytes_in_buffer;
 $24 = ($23|0)==(0);
 do {
  if ($24) {
   $25 = $datasrc;
   $26 = (($25) + 12|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $1;
   $29 = (FUNCTION_TABLE_ii[$27 & 127]($28)|0);
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = $datasrc;
    $32 = HEAP32[$31>>2]|0;
    $next_input_byte = $32;
    $33 = $datasrc;
    $34 = (($33) + 4|0);
    $35 = HEAP32[$34>>2]|0;
    $bytes_in_buffer = $35;
    break;
   }
   $0 = 0;
   $336 = $0;
   STACKTOP = sp;return ($336|0);
  }
 } while(0);
 $36 = $bytes_in_buffer;
 $37 = (($36) + -1)|0;
 $bytes_in_buffer = $37;
 $38 = $next_input_byte;
 $39 = (($38) + 1|0);
 $next_input_byte = $39;
 $40 = HEAP8[$38>>0]|0;
 $41 = $40&255;
 $42 = $41 << 8;
 $length = $42;
 $43 = $bytes_in_buffer;
 $44 = ($43|0)==(0);
 do {
  if ($44) {
   $45 = $datasrc;
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $1;
   $49 = (FUNCTION_TABLE_ii[$47 & 127]($48)|0);
   $50 = ($49|0)!=(0);
   if ($50) {
    $51 = $datasrc;
    $52 = HEAP32[$51>>2]|0;
    $next_input_byte = $52;
    $53 = $datasrc;
    $54 = (($53) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $bytes_in_buffer = $55;
    break;
   }
   $0 = 0;
   $336 = $0;
   STACKTOP = sp;return ($336|0);
  }
 } while(0);
 $56 = $bytes_in_buffer;
 $57 = (($56) + -1)|0;
 $bytes_in_buffer = $57;
 $58 = $next_input_byte;
 $59 = (($58) + 1|0);
 $next_input_byte = $59;
 $60 = HEAP8[$58>>0]|0;
 $61 = $60&255;
 $62 = $length;
 $63 = (($62) + ($61))|0;
 $length = $63;
 $64 = $bytes_in_buffer;
 $65 = ($64|0)==(0);
 do {
  if ($65) {
   $66 = $datasrc;
   $67 = (($66) + 12|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = $1;
   $70 = (FUNCTION_TABLE_ii[$68 & 127]($69)|0);
   $71 = ($70|0)!=(0);
   if ($71) {
    $72 = $datasrc;
    $73 = HEAP32[$72>>2]|0;
    $next_input_byte = $73;
    $74 = $datasrc;
    $75 = (($74) + 4|0);
    $76 = HEAP32[$75>>2]|0;
    $bytes_in_buffer = $76;
    break;
   }
   $0 = 0;
   $336 = $0;
   STACKTOP = sp;return ($336|0);
  }
 } while(0);
 $77 = $bytes_in_buffer;
 $78 = (($77) + -1)|0;
 $bytes_in_buffer = $78;
 $79 = $next_input_byte;
 $80 = (($79) + 1|0);
 $next_input_byte = $80;
 $81 = HEAP8[$79>>0]|0;
 $82 = $81&255;
 $n = $82;
 $83 = $1;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($84) + 20|0);
 HEAP32[$85>>2] = 103;
 $86 = $n;
 $87 = $1;
 $88 = HEAP32[$87>>2]|0;
 $89 = (($88) + 24|0);
 HEAP32[$89>>2] = $86;
 $90 = $1;
 $91 = HEAP32[$90>>2]|0;
 $92 = (($91) + 4|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = $1;
 FUNCTION_TABLE_vii[$93 & 127]($94,1);
 $95 = $length;
 $96 = $n;
 $97 = $96<<1;
 $98 = (($97) + 6)|0;
 $99 = ($95|0)!=($98|0);
 if ($99) {
  label = 22;
 } else {
  $100 = $n;
  $101 = ($100|0)<(1);
  if ($101) {
   label = 22;
  } else {
   $102 = $n;
   $103 = ($102|0)>(4);
   if ($103) {
    label = 22;
   }
  }
 }
 if ((label|0) == 22) {
  $104 = $1;
  $105 = HEAP32[$104>>2]|0;
  $106 = (($105) + 20|0);
  HEAP32[$106>>2] = 11;
  $107 = $1;
  $108 = HEAP32[$107>>2]|0;
  $109 = HEAP32[$108>>2]|0;
  $110 = $1;
  FUNCTION_TABLE_vi[$109 & 127]($110);
 }
 $111 = $n;
 $112 = $1;
 $113 = (($112) + 328|0);
 HEAP32[$113>>2] = $111;
 $i = 0;
 while(1) {
  $114 = $i;
  $115 = $n;
  $116 = ($114|0)<($115|0);
  if (!($116)) {
   label = 48;
   break;
  }
  $117 = $bytes_in_buffer;
  $118 = ($117|0)==(0);
  if ($118) {
   $119 = $datasrc;
   $120 = (($119) + 12|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = $1;
   $123 = (FUNCTION_TABLE_ii[$121 & 127]($122)|0);
   $124 = ($123|0)!=(0);
   if (!($124)) {
    label = 28;
    break;
   }
   $125 = $datasrc;
   $126 = HEAP32[$125>>2]|0;
   $next_input_byte = $126;
   $127 = $datasrc;
   $128 = (($127) + 4|0);
   $129 = HEAP32[$128>>2]|0;
   $bytes_in_buffer = $129;
  }
  $130 = $bytes_in_buffer;
  $131 = (($130) + -1)|0;
  $bytes_in_buffer = $131;
  $132 = $next_input_byte;
  $133 = (($132) + 1|0);
  $next_input_byte = $133;
  $134 = HEAP8[$132>>0]|0;
  $135 = $134&255;
  $cc = $135;
  $136 = $bytes_in_buffer;
  $137 = ($136|0)==(0);
  if ($137) {
   $138 = $datasrc;
   $139 = (($138) + 12|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = $1;
   $142 = (FUNCTION_TABLE_ii[$140 & 127]($141)|0);
   $143 = ($142|0)!=(0);
   if (!($143)) {
    label = 34;
    break;
   }
   $144 = $datasrc;
   $145 = HEAP32[$144>>2]|0;
   $next_input_byte = $145;
   $146 = $datasrc;
   $147 = (($146) + 4|0);
   $148 = HEAP32[$147>>2]|0;
   $bytes_in_buffer = $148;
  }
  $149 = $bytes_in_buffer;
  $150 = (($149) + -1)|0;
  $bytes_in_buffer = $150;
  $151 = $next_input_byte;
  $152 = (($151) + 1|0);
  $next_input_byte = $152;
  $153 = HEAP8[$151>>0]|0;
  $154 = $153&255;
  $c = $154;
  $ci = 0;
  $155 = $1;
  $156 = (($155) + 216|0);
  $157 = HEAP32[$156>>2]|0;
  $compptr = $157;
  while(1) {
   $158 = $ci;
   $159 = $1;
   $160 = (($159) + 36|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = ($158|0)<($161|0);
   if (!($162)) {
    label = 43;
    break;
   }
   $163 = $cc;
   $164 = $compptr;
   $165 = HEAP32[$164>>2]|0;
   $166 = ($163|0)==($165|0);
   if ($166) {
    label = 40;
    break;
   }
   $167 = $ci;
   $168 = (($167) + 1)|0;
   $ci = $168;
   $169 = $compptr;
   $170 = (($169) + 84|0);
   $compptr = $170;
  }
  if ((label|0) == 40) {
   label = 0;
  }
  else if ((label|0) == 43) {
   label = 0;
   $171 = $1;
   $172 = HEAP32[$171>>2]|0;
   $173 = (($172) + 20|0);
   HEAP32[$173>>2] = 5;
   $174 = $cc;
   $175 = $1;
   $176 = HEAP32[$175>>2]|0;
   $177 = (($176) + 24|0);
   HEAP32[$177>>2] = $174;
   $178 = $1;
   $179 = HEAP32[$178>>2]|0;
   $180 = HEAP32[$179>>2]|0;
   $181 = $1;
   FUNCTION_TABLE_vi[$180 & 127]($181);
  }
  $182 = $compptr;
  $183 = $i;
  $184 = $1;
  $185 = (($184) + 332|0);
  $186 = (($185) + ($183<<2)|0);
  HEAP32[$186>>2] = $182;
  $187 = $c;
  $188 = $187 >> 4;
  $189 = $188 & 15;
  $190 = $compptr;
  $191 = (($190) + 20|0);
  HEAP32[$191>>2] = $189;
  $192 = $c;
  $193 = $192 & 15;
  $194 = $compptr;
  $195 = (($194) + 24|0);
  HEAP32[$195>>2] = $193;
  $196 = $1;
  $197 = HEAP32[$196>>2]|0;
  $198 = (($197) + 24|0);
  $_mp = $198;
  $199 = $cc;
  $200 = $_mp;
  HEAP32[$200>>2] = $199;
  $201 = $compptr;
  $202 = (($201) + 20|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = $_mp;
  $205 = (($204) + 4|0);
  HEAP32[$205>>2] = $203;
  $206 = $compptr;
  $207 = (($206) + 24|0);
  $208 = HEAP32[$207>>2]|0;
  $209 = $_mp;
  $210 = (($209) + 8|0);
  HEAP32[$210>>2] = $208;
  $211 = $1;
  $212 = HEAP32[$211>>2]|0;
  $213 = (($212) + 20|0);
  HEAP32[$213>>2] = 104;
  $214 = $1;
  $215 = HEAP32[$214>>2]|0;
  $216 = (($215) + 4|0);
  $217 = HEAP32[$216>>2]|0;
  $218 = $1;
  FUNCTION_TABLE_vii[$217 & 127]($218,1);
  $219 = $i;
  $220 = (($219) + 1)|0;
  $i = $220;
 }
 if ((label|0) == 28) {
  $0 = 0;
  $336 = $0;
  STACKTOP = sp;return ($336|0);
 }
 else if ((label|0) == 34) {
  $0 = 0;
  $336 = $0;
  STACKTOP = sp;return ($336|0);
 }
 else if ((label|0) == 48) {
  $221 = $bytes_in_buffer;
  $222 = ($221|0)==(0);
  do {
   if ($222) {
    $223 = $datasrc;
    $224 = (($223) + 12|0);
    $225 = HEAP32[$224>>2]|0;
    $226 = $1;
    $227 = (FUNCTION_TABLE_ii[$225 & 127]($226)|0);
    $228 = ($227|0)!=(0);
    if ($228) {
     $229 = $datasrc;
     $230 = HEAP32[$229>>2]|0;
     $next_input_byte = $230;
     $231 = $datasrc;
     $232 = (($231) + 4|0);
     $233 = HEAP32[$232>>2]|0;
     $bytes_in_buffer = $233;
     break;
    }
    $0 = 0;
    $336 = $0;
    STACKTOP = sp;return ($336|0);
   }
  } while(0);
  $234 = $bytes_in_buffer;
  $235 = (($234) + -1)|0;
  $bytes_in_buffer = $235;
  $236 = $next_input_byte;
  $237 = (($236) + 1|0);
  $next_input_byte = $237;
  $238 = HEAP8[$236>>0]|0;
  $239 = $238&255;
  $c = $239;
  $240 = $c;
  $241 = $1;
  $242 = (($241) + 400|0);
  HEAP32[$242>>2] = $240;
  $243 = $bytes_in_buffer;
  $244 = ($243|0)==(0);
  do {
   if ($244) {
    $245 = $datasrc;
    $246 = (($245) + 12|0);
    $247 = HEAP32[$246>>2]|0;
    $248 = $1;
    $249 = (FUNCTION_TABLE_ii[$247 & 127]($248)|0);
    $250 = ($249|0)!=(0);
    if ($250) {
     $251 = $datasrc;
     $252 = HEAP32[$251>>2]|0;
     $next_input_byte = $252;
     $253 = $datasrc;
     $254 = (($253) + 4|0);
     $255 = HEAP32[$254>>2]|0;
     $bytes_in_buffer = $255;
     break;
    }
    $0 = 0;
    $336 = $0;
    STACKTOP = sp;return ($336|0);
   }
  } while(0);
  $256 = $bytes_in_buffer;
  $257 = (($256) + -1)|0;
  $bytes_in_buffer = $257;
  $258 = $next_input_byte;
  $259 = (($258) + 1|0);
  $next_input_byte = $259;
  $260 = HEAP8[$258>>0]|0;
  $261 = $260&255;
  $c = $261;
  $262 = $c;
  $263 = $1;
  $264 = (($263) + 404|0);
  HEAP32[$264>>2] = $262;
  $265 = $bytes_in_buffer;
  $266 = ($265|0)==(0);
  do {
   if ($266) {
    $267 = $datasrc;
    $268 = (($267) + 12|0);
    $269 = HEAP32[$268>>2]|0;
    $270 = $1;
    $271 = (FUNCTION_TABLE_ii[$269 & 127]($270)|0);
    $272 = ($271|0)!=(0);
    if ($272) {
     $273 = $datasrc;
     $274 = HEAP32[$273>>2]|0;
     $next_input_byte = $274;
     $275 = $datasrc;
     $276 = (($275) + 4|0);
     $277 = HEAP32[$276>>2]|0;
     $bytes_in_buffer = $277;
     break;
    }
    $0 = 0;
    $336 = $0;
    STACKTOP = sp;return ($336|0);
   }
  } while(0);
  $278 = $bytes_in_buffer;
  $279 = (($278) + -1)|0;
  $bytes_in_buffer = $279;
  $280 = $next_input_byte;
  $281 = (($280) + 1|0);
  $next_input_byte = $281;
  $282 = HEAP8[$280>>0]|0;
  $283 = $282&255;
  $c = $283;
  $284 = $c;
  $285 = $284 >> 4;
  $286 = $285 & 15;
  $287 = $1;
  $288 = (($287) + 408|0);
  HEAP32[$288>>2] = $286;
  $289 = $c;
  $290 = $289 & 15;
  $291 = $1;
  $292 = (($291) + 412|0);
  HEAP32[$292>>2] = $290;
  $293 = $1;
  $294 = HEAP32[$293>>2]|0;
  $295 = (($294) + 24|0);
  $_mp1 = $295;
  $296 = $1;
  $297 = (($296) + 400|0);
  $298 = HEAP32[$297>>2]|0;
  $299 = $_mp1;
  HEAP32[$299>>2] = $298;
  $300 = $1;
  $301 = (($300) + 404|0);
  $302 = HEAP32[$301>>2]|0;
  $303 = $_mp1;
  $304 = (($303) + 4|0);
  HEAP32[$304>>2] = $302;
  $305 = $1;
  $306 = (($305) + 408|0);
  $307 = HEAP32[$306>>2]|0;
  $308 = $_mp1;
  $309 = (($308) + 8|0);
  HEAP32[$309>>2] = $307;
  $310 = $1;
  $311 = (($310) + 412|0);
  $312 = HEAP32[$311>>2]|0;
  $313 = $_mp1;
  $314 = (($313) + 12|0);
  HEAP32[$314>>2] = $312;
  $315 = $1;
  $316 = HEAP32[$315>>2]|0;
  $317 = (($316) + 20|0);
  HEAP32[$317>>2] = 105;
  $318 = $1;
  $319 = HEAP32[$318>>2]|0;
  $320 = (($319) + 4|0);
  $321 = HEAP32[$320>>2]|0;
  $322 = $1;
  FUNCTION_TABLE_vii[$321 & 127]($322,1);
  $323 = $1;
  $324 = (($323) + 440|0);
  $325 = HEAP32[$324>>2]|0;
  $326 = (($325) + 20|0);
  HEAP32[$326>>2] = 0;
  $327 = $1;
  $328 = (($327) + 144|0);
  $329 = HEAP32[$328>>2]|0;
  $330 = (($329) + 1)|0;
  HEAP32[$328>>2] = $330;
  $331 = $next_input_byte;
  $332 = $datasrc;
  HEAP32[$332>>2] = $331;
  $333 = $bytes_in_buffer;
  $334 = $datasrc;
  $335 = (($334) + 4|0);
  HEAP32[$335>>2] = $333;
  $0 = 1;
  $336 = $0;
  STACKTOP = sp;return ($336|0);
 }
 return 0|0;
}
function _get_dht($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $_mp1 = 0, $bits = 0, $bytes_in_buffer = 0, $count = 0, $datasrc = 0, $htblptr = 0, $huffval = 0;
 var $i = 0, $index = 0, $length = 0, $next_input_byte = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0;
 $bits = sp + 304|0;
 $huffval = sp + 48|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $311 = $0;
   STACKTOP = sp;return ($311|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $311 = $0;
   STACKTOP = sp;return ($311|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 L17: while(1) {
  $53 = $length;
  $54 = ($53|0)>(16);
  if (!($54)) {
   label = 55;
   break;
  }
  $55 = $bytes_in_buffer;
  $56 = ($55|0)==(0);
  if ($56) {
   $57 = $datasrc;
   $58 = (($57) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $1;
   $61 = (FUNCTION_TABLE_ii[$59 & 127]($60)|0);
   $62 = ($61|0)!=(0);
   if (!($62)) {
    label = 16;
    break;
   }
   $63 = $datasrc;
   $64 = HEAP32[$63>>2]|0;
   $next_input_byte = $64;
   $65 = $datasrc;
   $66 = (($65) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   $bytes_in_buffer = $67;
  }
  $68 = $bytes_in_buffer;
  $69 = (($68) + -1)|0;
  $bytes_in_buffer = $69;
  $70 = $next_input_byte;
  $71 = (($70) + 1|0);
  $next_input_byte = $71;
  $72 = HEAP8[$70>>0]|0;
  $73 = $72&255;
  $index = $73;
  $74 = $1;
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 20|0);
  HEAP32[$76>>2] = 80;
  $77 = $index;
  $78 = $1;
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + 24|0);
  HEAP32[$80>>2] = $77;
  $81 = $1;
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = $1;
  FUNCTION_TABLE_vii[$84 & 127]($85,1);
  HEAP8[$bits>>0] = 0;
  $count = 0;
  $i = 1;
  while(1) {
   $86 = $i;
   $87 = ($86|0)<=(16);
   if (!($87)) {
    break;
   }
   $88 = $bytes_in_buffer;
   $89 = ($88|0)==(0);
   if ($89) {
    $90 = $datasrc;
    $91 = (($90) + 12|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = $1;
    $94 = (FUNCTION_TABLE_ii[$92 & 127]($93)|0);
    $95 = ($94|0)!=(0);
    if (!($95)) {
     label = 24;
     break L17;
    }
    $96 = $datasrc;
    $97 = HEAP32[$96>>2]|0;
    $next_input_byte = $97;
    $98 = $datasrc;
    $99 = (($98) + 4|0);
    $100 = HEAP32[$99>>2]|0;
    $bytes_in_buffer = $100;
   }
   $101 = $bytes_in_buffer;
   $102 = (($101) + -1)|0;
   $bytes_in_buffer = $102;
   $103 = $next_input_byte;
   $104 = (($103) + 1|0);
   $next_input_byte = $104;
   $105 = HEAP8[$103>>0]|0;
   $106 = $i;
   $107 = (($bits) + ($106)|0);
   HEAP8[$107>>0] = $105;
   $108 = $i;
   $109 = (($bits) + ($108)|0);
   $110 = HEAP8[$109>>0]|0;
   $111 = $110&255;
   $112 = $count;
   $113 = (($112) + ($111))|0;
   $count = $113;
   $114 = $i;
   $115 = (($114) + 1)|0;
   $i = $115;
  }
  $116 = $length;
  $117 = (($116) - 17)|0;
  $length = $117;
  $118 = $1;
  $119 = HEAP32[$118>>2]|0;
  $120 = (($119) + 24|0);
  $_mp = $120;
  $121 = (($bits) + 1|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $_mp;
  HEAP32[$124>>2] = $123;
  $125 = (($bits) + 2|0);
  $126 = HEAP8[$125>>0]|0;
  $127 = $126&255;
  $128 = $_mp;
  $129 = (($128) + 4|0);
  HEAP32[$129>>2] = $127;
  $130 = (($bits) + 3|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = $131&255;
  $133 = $_mp;
  $134 = (($133) + 8|0);
  HEAP32[$134>>2] = $132;
  $135 = (($bits) + 4|0);
  $136 = HEAP8[$135>>0]|0;
  $137 = $136&255;
  $138 = $_mp;
  $139 = (($138) + 12|0);
  HEAP32[$139>>2] = $137;
  $140 = (($bits) + 5|0);
  $141 = HEAP8[$140>>0]|0;
  $142 = $141&255;
  $143 = $_mp;
  $144 = (($143) + 16|0);
  HEAP32[$144>>2] = $142;
  $145 = (($bits) + 6|0);
  $146 = HEAP8[$145>>0]|0;
  $147 = $146&255;
  $148 = $_mp;
  $149 = (($148) + 20|0);
  HEAP32[$149>>2] = $147;
  $150 = (($bits) + 7|0);
  $151 = HEAP8[$150>>0]|0;
  $152 = $151&255;
  $153 = $_mp;
  $154 = (($153) + 24|0);
  HEAP32[$154>>2] = $152;
  $155 = (($bits) + 8|0);
  $156 = HEAP8[$155>>0]|0;
  $157 = $156&255;
  $158 = $_mp;
  $159 = (($158) + 28|0);
  HEAP32[$159>>2] = $157;
  $160 = $1;
  $161 = HEAP32[$160>>2]|0;
  $162 = (($161) + 20|0);
  HEAP32[$162>>2] = 86;
  $163 = $1;
  $164 = HEAP32[$163>>2]|0;
  $165 = (($164) + 4|0);
  $166 = HEAP32[$165>>2]|0;
  $167 = $1;
  FUNCTION_TABLE_vii[$166 & 127]($167,2);
  $168 = $1;
  $169 = HEAP32[$168>>2]|0;
  $170 = (($169) + 24|0);
  $_mp1 = $170;
  $171 = (($bits) + 9|0);
  $172 = HEAP8[$171>>0]|0;
  $173 = $172&255;
  $174 = $_mp1;
  HEAP32[$174>>2] = $173;
  $175 = (($bits) + 10|0);
  $176 = HEAP8[$175>>0]|0;
  $177 = $176&255;
  $178 = $_mp1;
  $179 = (($178) + 4|0);
  HEAP32[$179>>2] = $177;
  $180 = (($bits) + 11|0);
  $181 = HEAP8[$180>>0]|0;
  $182 = $181&255;
  $183 = $_mp1;
  $184 = (($183) + 8|0);
  HEAP32[$184>>2] = $182;
  $185 = (($bits) + 12|0);
  $186 = HEAP8[$185>>0]|0;
  $187 = $186&255;
  $188 = $_mp1;
  $189 = (($188) + 12|0);
  HEAP32[$189>>2] = $187;
  $190 = (($bits) + 13|0);
  $191 = HEAP8[$190>>0]|0;
  $192 = $191&255;
  $193 = $_mp1;
  $194 = (($193) + 16|0);
  HEAP32[$194>>2] = $192;
  $195 = (($bits) + 14|0);
  $196 = HEAP8[$195>>0]|0;
  $197 = $196&255;
  $198 = $_mp1;
  $199 = (($198) + 20|0);
  HEAP32[$199>>2] = $197;
  $200 = (($bits) + 15|0);
  $201 = HEAP8[$200>>0]|0;
  $202 = $201&255;
  $203 = $_mp1;
  $204 = (($203) + 24|0);
  HEAP32[$204>>2] = $202;
  $205 = (($bits) + 16|0);
  $206 = HEAP8[$205>>0]|0;
  $207 = $206&255;
  $208 = $_mp1;
  $209 = (($208) + 28|0);
  HEAP32[$209>>2] = $207;
  $210 = $1;
  $211 = HEAP32[$210>>2]|0;
  $212 = (($211) + 20|0);
  HEAP32[$212>>2] = 86;
  $213 = $1;
  $214 = HEAP32[$213>>2]|0;
  $215 = (($214) + 4|0);
  $216 = HEAP32[$215>>2]|0;
  $217 = $1;
  FUNCTION_TABLE_vii[$216 & 127]($217,2);
  $218 = $count;
  $219 = ($218|0)>(256);
  if ($219) {
   label = 35;
  } else {
   $220 = $count;
   $221 = $length;
   $222 = ($220|0)>($221|0);
   if ($222) {
    label = 35;
   }
  }
  if ((label|0) == 35) {
   label = 0;
   $223 = $1;
   $224 = HEAP32[$223>>2]|0;
   $225 = (($224) + 20|0);
   HEAP32[$225>>2] = 8;
   $226 = $1;
   $227 = HEAP32[$226>>2]|0;
   $228 = HEAP32[$227>>2]|0;
   $229 = $1;
   FUNCTION_TABLE_vi[$228 & 127]($229);
  }
  $i = 0;
  while(1) {
   $230 = $i;
   $231 = $count;
   $232 = ($230|0)<($231|0);
   if (!($232)) {
    break;
   }
   $233 = $bytes_in_buffer;
   $234 = ($233|0)==(0);
   if ($234) {
    $235 = $datasrc;
    $236 = (($235) + 12|0);
    $237 = HEAP32[$236>>2]|0;
    $238 = $1;
    $239 = (FUNCTION_TABLE_ii[$237 & 127]($238)|0);
    $240 = ($239|0)!=(0);
    if (!($240)) {
     label = 41;
     break L17;
    }
    $241 = $datasrc;
    $242 = HEAP32[$241>>2]|0;
    $next_input_byte = $242;
    $243 = $datasrc;
    $244 = (($243) + 4|0);
    $245 = HEAP32[$244>>2]|0;
    $bytes_in_buffer = $245;
   }
   $246 = $bytes_in_buffer;
   $247 = (($246) + -1)|0;
   $bytes_in_buffer = $247;
   $248 = $next_input_byte;
   $249 = (($248) + 1|0);
   $next_input_byte = $249;
   $250 = HEAP8[$248>>0]|0;
   $251 = $i;
   $252 = (($huffval) + ($251)|0);
   HEAP8[$252>>0] = $250;
   $253 = $i;
   $254 = (($253) + 1)|0;
   $i = $254;
  }
  $255 = $count;
  $256 = $length;
  $257 = (($256) - ($255))|0;
  $length = $257;
  $258 = $index;
  $259 = $258 & 16;
  $260 = ($259|0)!=(0);
  if ($260) {
   $261 = $index;
   $262 = (($261) - 16)|0;
   $index = $262;
   $263 = $index;
   $264 = $1;
   $265 = (($264) + 196|0);
   $266 = (($265) + ($263<<2)|0);
   $htblptr = $266;
  } else {
   $267 = $index;
   $268 = $1;
   $269 = (($268) + 180|0);
   $270 = (($269) + ($267<<2)|0);
   $htblptr = $270;
  }
  $271 = $index;
  $272 = ($271|0)<(0);
  if ($272) {
   label = 51;
  } else {
   $273 = $index;
   $274 = ($273|0)>=(4);
   if ($274) {
    label = 51;
   }
  }
  if ((label|0) == 51) {
   label = 0;
   $275 = $1;
   $276 = HEAP32[$275>>2]|0;
   $277 = (($276) + 20|0);
   HEAP32[$277>>2] = 30;
   $278 = $index;
   $279 = $1;
   $280 = HEAP32[$279>>2]|0;
   $281 = (($280) + 24|0);
   HEAP32[$281>>2] = $278;
   $282 = $1;
   $283 = HEAP32[$282>>2]|0;
   $284 = HEAP32[$283>>2]|0;
   $285 = $1;
   FUNCTION_TABLE_vi[$284 & 127]($285);
  }
  $286 = $htblptr;
  $287 = HEAP32[$286>>2]|0;
  $288 = ($287|0)==(0|0);
  if ($288) {
   $289 = $1;
   $290 = (_jpeg_alloc_huff_table($289)|0);
   $291 = $htblptr;
   HEAP32[$291>>2] = $290;
  }
  $292 = $htblptr;
  $293 = HEAP32[$292>>2]|0;
  dest=$293+0|0; src=$bits+0|0; stop=dest+17|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $294 = $htblptr;
  $295 = HEAP32[$294>>2]|0;
  $296 = (($295) + 17|0);
  _memcpy(($296|0),($huffval|0),256)|0;
 }
 if ((label|0) == 16) {
  $0 = 0;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 else if ((label|0) == 24) {
  $0 = 0;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 else if ((label|0) == 41) {
  $0 = 0;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 else if ((label|0) == 55) {
  $297 = $length;
  $298 = ($297|0)!=(0);
  if ($298) {
   $299 = $1;
   $300 = HEAP32[$299>>2]|0;
   $301 = (($300) + 20|0);
   HEAP32[$301>>2] = 11;
   $302 = $1;
   $303 = HEAP32[$302>>2]|0;
   $304 = HEAP32[$303>>2]|0;
   $305 = $1;
   FUNCTION_TABLE_vi[$304 & 127]($305);
  }
  $306 = $next_input_byte;
  $307 = $datasrc;
  HEAP32[$307>>2] = $306;
  $308 = $bytes_in_buffer;
  $309 = $datasrc;
  $310 = (($309) + 4|0);
  HEAP32[$310>>2] = $308;
  $0 = 1;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 return 0|0;
}
function _get_dqt($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $_mp = 0, $bytes_in_buffer = 0, $datasrc = 0, $i = 0, $length = 0, $n = 0, $next_input_byte = 0, $prec = 0, $quant_ptr = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $300 = $0;
   STACKTOP = sp;return ($300|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $300 = $0;
   STACKTOP = sp;return ($300|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 L17: while(1) {
  $53 = $length;
  $54 = ($53|0)>(0);
  if (!($54)) {
   label = 57;
   break;
  }
  $55 = $bytes_in_buffer;
  $56 = ($55|0)==(0);
  if ($56) {
   $57 = $datasrc;
   $58 = (($57) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $1;
   $61 = (FUNCTION_TABLE_ii[$59 & 127]($60)|0);
   $62 = ($61|0)!=(0);
   if (!($62)) {
    label = 16;
    break;
   }
   $63 = $datasrc;
   $64 = HEAP32[$63>>2]|0;
   $next_input_byte = $64;
   $65 = $datasrc;
   $66 = (($65) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   $bytes_in_buffer = $67;
  }
  $68 = $bytes_in_buffer;
  $69 = (($68) + -1)|0;
  $bytes_in_buffer = $69;
  $70 = $next_input_byte;
  $71 = (($70) + 1|0);
  $next_input_byte = $71;
  $72 = HEAP8[$70>>0]|0;
  $73 = $72&255;
  $n = $73;
  $74 = $n;
  $75 = $74 >> 4;
  $prec = $75;
  $76 = $n;
  $77 = $76 & 15;
  $n = $77;
  $78 = $1;
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + 20|0);
  HEAP32[$80>>2] = 81;
  $81 = $n;
  $82 = $1;
  $83 = HEAP32[$82>>2]|0;
  $84 = (($83) + 24|0);
  HEAP32[$84>>2] = $81;
  $85 = $prec;
  $86 = $1;
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 24|0);
  $89 = (($88) + 4|0);
  HEAP32[$89>>2] = $85;
  $90 = $1;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = $1;
  FUNCTION_TABLE_vii[$93 & 127]($94,1);
  $95 = $n;
  $96 = ($95|0)>=(4);
  if ($96) {
   $97 = $1;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) + 20|0);
   HEAP32[$99>>2] = 31;
   $100 = $n;
   $101 = $1;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) + 24|0);
   HEAP32[$103>>2] = $100;
   $104 = $1;
   $105 = HEAP32[$104>>2]|0;
   $106 = HEAP32[$105>>2]|0;
   $107 = $1;
   FUNCTION_TABLE_vi[$106 & 127]($107);
  }
  $108 = $n;
  $109 = $1;
  $110 = (($109) + 164|0);
  $111 = (($110) + ($108<<2)|0);
  $112 = HEAP32[$111>>2]|0;
  $113 = ($112|0)==(0|0);
  if ($113) {
   $114 = $1;
   $115 = (_jpeg_alloc_quant_table($114)|0);
   $116 = $n;
   $117 = $1;
   $118 = (($117) + 164|0);
   $119 = (($118) + ($116<<2)|0);
   HEAP32[$119>>2] = $115;
  }
  $120 = $n;
  $121 = $1;
  $122 = (($121) + 164|0);
  $123 = (($122) + ($120<<2)|0);
  $124 = HEAP32[$123>>2]|0;
  $quant_ptr = $124;
  $i = 0;
  while(1) {
   $125 = $i;
   $126 = ($125|0)<(64);
   if (!($126)) {
    break;
   }
   $127 = $prec;
   $128 = ($127|0)!=(0);
   if ($128) {
    $129 = $bytes_in_buffer;
    $130 = ($129|0)==(0);
    if ($130) {
     $131 = $datasrc;
     $132 = (($131) + 12|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = $1;
     $135 = (FUNCTION_TABLE_ii[$133 & 127]($134)|0);
     $136 = ($135|0)!=(0);
     if (!($136)) {
      label = 29;
      break L17;
     }
     $137 = $datasrc;
     $138 = HEAP32[$137>>2]|0;
     $next_input_byte = $138;
     $139 = $datasrc;
     $140 = (($139) + 4|0);
     $141 = HEAP32[$140>>2]|0;
     $bytes_in_buffer = $141;
    }
    $142 = $bytes_in_buffer;
    $143 = (($142) + -1)|0;
    $bytes_in_buffer = $143;
    $144 = $next_input_byte;
    $145 = (($144) + 1|0);
    $next_input_byte = $145;
    $146 = HEAP8[$144>>0]|0;
    $147 = $146&255;
    $148 = $147 << 8;
    $tmp = $148;
    $149 = $bytes_in_buffer;
    $150 = ($149|0)==(0);
    if ($150) {
     $151 = $datasrc;
     $152 = (($151) + 12|0);
     $153 = HEAP32[$152>>2]|0;
     $154 = $1;
     $155 = (FUNCTION_TABLE_ii[$153 & 127]($154)|0);
     $156 = ($155|0)!=(0);
     if (!($156)) {
      label = 33;
      break L17;
     }
     $157 = $datasrc;
     $158 = HEAP32[$157>>2]|0;
     $next_input_byte = $158;
     $159 = $datasrc;
     $160 = (($159) + 4|0);
     $161 = HEAP32[$160>>2]|0;
     $bytes_in_buffer = $161;
    }
    $162 = $bytes_in_buffer;
    $163 = (($162) + -1)|0;
    $bytes_in_buffer = $163;
    $164 = $next_input_byte;
    $165 = (($164) + 1|0);
    $next_input_byte = $165;
    $166 = HEAP8[$164>>0]|0;
    $167 = $166&255;
    $168 = $tmp;
    $169 = (($168) + ($167))|0;
    $tmp = $169;
   } else {
    $170 = $bytes_in_buffer;
    $171 = ($170|0)==(0);
    if ($171) {
     $172 = $datasrc;
     $173 = (($172) + 12|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = $1;
     $176 = (FUNCTION_TABLE_ii[$174 & 127]($175)|0);
     $177 = ($176|0)!=(0);
     if (!($177)) {
      label = 40;
      break L17;
     }
     $178 = $datasrc;
     $179 = HEAP32[$178>>2]|0;
     $next_input_byte = $179;
     $180 = $datasrc;
     $181 = (($180) + 4|0);
     $182 = HEAP32[$181>>2]|0;
     $bytes_in_buffer = $182;
    }
    $183 = $bytes_in_buffer;
    $184 = (($183) + -1)|0;
    $bytes_in_buffer = $184;
    $185 = $next_input_byte;
    $186 = (($185) + 1|0);
    $next_input_byte = $186;
    $187 = HEAP8[$185>>0]|0;
    $188 = $187&255;
    $tmp = $188;
   }
   $189 = $tmp;
   $190 = $189&65535;
   $191 = $i;
   $192 = (9504 + ($191<<2)|0);
   $193 = HEAPU8[$192>>0]|(HEAPU8[$192+1>>0]<<8)|(HEAPU8[$192+2>>0]<<16)|(HEAPU8[$192+3>>0]<<24);
   $194 = $quant_ptr;
   $195 = (($194) + ($193<<1)|0);
   HEAP16[$195>>1] = $190;
   $196 = $i;
   $197 = (($196) + 1)|0;
   $i = $197;
  }
  $198 = $1;
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + 104|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = ($201|0)>=(2);
  if ($202) {
   $i = 0;
   while(1) {
    $203 = $i;
    $204 = ($203|0)<(64);
    if (!($204)) {
     break;
    }
    $205 = $1;
    $206 = HEAP32[$205>>2]|0;
    $207 = (($206) + 24|0);
    $_mp = $207;
    $208 = $i;
    $209 = $quant_ptr;
    $210 = (($209) + ($208<<1)|0);
    $211 = HEAP16[$210>>1]|0;
    $212 = $211&65535;
    $213 = $_mp;
    HEAP32[$213>>2] = $212;
    $214 = $i;
    $215 = (($214) + 1)|0;
    $216 = $quant_ptr;
    $217 = (($216) + ($215<<1)|0);
    $218 = HEAP16[$217>>1]|0;
    $219 = $218&65535;
    $220 = $_mp;
    $221 = (($220) + 4|0);
    HEAP32[$221>>2] = $219;
    $222 = $i;
    $223 = (($222) + 2)|0;
    $224 = $quant_ptr;
    $225 = (($224) + ($223<<1)|0);
    $226 = HEAP16[$225>>1]|0;
    $227 = $226&65535;
    $228 = $_mp;
    $229 = (($228) + 8|0);
    HEAP32[$229>>2] = $227;
    $230 = $i;
    $231 = (($230) + 3)|0;
    $232 = $quant_ptr;
    $233 = (($232) + ($231<<1)|0);
    $234 = HEAP16[$233>>1]|0;
    $235 = $234&65535;
    $236 = $_mp;
    $237 = (($236) + 12|0);
    HEAP32[$237>>2] = $235;
    $238 = $i;
    $239 = (($238) + 4)|0;
    $240 = $quant_ptr;
    $241 = (($240) + ($239<<1)|0);
    $242 = HEAP16[$241>>1]|0;
    $243 = $242&65535;
    $244 = $_mp;
    $245 = (($244) + 16|0);
    HEAP32[$245>>2] = $243;
    $246 = $i;
    $247 = (($246) + 5)|0;
    $248 = $quant_ptr;
    $249 = (($248) + ($247<<1)|0);
    $250 = HEAP16[$249>>1]|0;
    $251 = $250&65535;
    $252 = $_mp;
    $253 = (($252) + 20|0);
    HEAP32[$253>>2] = $251;
    $254 = $i;
    $255 = (($254) + 6)|0;
    $256 = $quant_ptr;
    $257 = (($256) + ($255<<1)|0);
    $258 = HEAP16[$257>>1]|0;
    $259 = $258&65535;
    $260 = $_mp;
    $261 = (($260) + 24|0);
    HEAP32[$261>>2] = $259;
    $262 = $i;
    $263 = (($262) + 7)|0;
    $264 = $quant_ptr;
    $265 = (($264) + ($263<<1)|0);
    $266 = HEAP16[$265>>1]|0;
    $267 = $266&65535;
    $268 = $_mp;
    $269 = (($268) + 28|0);
    HEAP32[$269>>2] = $267;
    $270 = $1;
    $271 = HEAP32[$270>>2]|0;
    $272 = (($271) + 20|0);
    HEAP32[$272>>2] = 93;
    $273 = $1;
    $274 = HEAP32[$273>>2]|0;
    $275 = (($274) + 4|0);
    $276 = HEAP32[$275>>2]|0;
    $277 = $1;
    FUNCTION_TABLE_vii[$276 & 127]($277,2);
    $278 = $i;
    $279 = (($278) + 8)|0;
    $i = $279;
   }
  }
  $280 = $length;
  $281 = (($280) - 65)|0;
  $length = $281;
  $282 = $prec;
  $283 = ($282|0)!=(0);
  if ($283) {
   $284 = $length;
   $285 = (($284) - 64)|0;
   $length = $285;
  }
 }
 if ((label|0) == 16) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 29) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 33) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 40) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 57) {
  $286 = $length;
  $287 = ($286|0)!=(0);
  if ($287) {
   $288 = $1;
   $289 = HEAP32[$288>>2]|0;
   $290 = (($289) + 20|0);
   HEAP32[$290>>2] = 11;
   $291 = $1;
   $292 = HEAP32[$291>>2]|0;
   $293 = HEAP32[$292>>2]|0;
   $294 = $1;
   FUNCTION_TABLE_vi[$293 & 127]($294);
  }
  $295 = $next_input_byte;
  $296 = $datasrc;
  HEAP32[$296>>2] = $295;
  $297 = $bytes_in_buffer;
  $298 = $datasrc;
  $299 = (($298) + 4|0);
  HEAP32[$299>>2] = $297;
  $0 = 1;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 return 0|0;
}
function _get_dri($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $bytes_in_buffer = 0, $datasrc = 0, $length = 0, $next_input_byte = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = ($51|0)!=(4);
 if ($52) {
  $53 = $1;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 20|0);
  HEAP32[$55>>2] = 11;
  $56 = $1;
  $57 = HEAP32[$56>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = $1;
  FUNCTION_TABLE_vi[$58 & 127]($59);
 }
 $60 = $bytes_in_buffer;
 $61 = ($60|0)==(0);
 do {
  if ($61) {
   $62 = $datasrc;
   $63 = (($62) + 12|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $1;
   $66 = (FUNCTION_TABLE_ii[$64 & 127]($65)|0);
   $67 = ($66|0)!=(0);
   if ($67) {
    $68 = $datasrc;
    $69 = HEAP32[$68>>2]|0;
    $next_input_byte = $69;
    $70 = $datasrc;
    $71 = (($70) + 4|0);
    $72 = HEAP32[$71>>2]|0;
    $bytes_in_buffer = $72;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $73 = $bytes_in_buffer;
 $74 = (($73) + -1)|0;
 $bytes_in_buffer = $74;
 $75 = $next_input_byte;
 $76 = (($75) + 1|0);
 $next_input_byte = $76;
 $77 = HEAP8[$75>>0]|0;
 $78 = $77&255;
 $79 = $78 << 8;
 $tmp = $79;
 $80 = $bytes_in_buffer;
 $81 = ($80|0)==(0);
 do {
  if ($81) {
   $82 = $datasrc;
   $83 = (($82) + 12|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = $1;
   $86 = (FUNCTION_TABLE_ii[$84 & 127]($85)|0);
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = $datasrc;
    $89 = HEAP32[$88>>2]|0;
    $next_input_byte = $89;
    $90 = $datasrc;
    $91 = (($90) + 4|0);
    $92 = HEAP32[$91>>2]|0;
    $bytes_in_buffer = $92;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $93 = $bytes_in_buffer;
 $94 = (($93) + -1)|0;
 $bytes_in_buffer = $94;
 $95 = $next_input_byte;
 $96 = (($95) + 1|0);
 $next_input_byte = $96;
 $97 = HEAP8[$95>>0]|0;
 $98 = $97&255;
 $99 = $tmp;
 $100 = (($99) + ($98))|0;
 $tmp = $100;
 $101 = $1;
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) + 20|0);
 HEAP32[$103>>2] = 82;
 $104 = $tmp;
 $105 = $1;
 $106 = HEAP32[$105>>2]|0;
 $107 = (($106) + 24|0);
 HEAP32[$107>>2] = $104;
 $108 = $1;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($109) + 4|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = $1;
 FUNCTION_TABLE_vii[$111 & 127]($112,1);
 $113 = $tmp;
 $114 = $1;
 $115 = (($114) + 276|0);
 HEAP32[$115>>2] = $113;
 $116 = $next_input_byte;
 $117 = $datasrc;
 HEAP32[$117>>2] = $116;
 $118 = $bytes_in_buffer;
 $119 = $datasrc;
 $120 = (($119) + 4|0);
 HEAP32[$120>>2] = $118;
 $0 = 1;
 $121 = $0;
 STACKTOP = sp;return ($121|0);
}
function _jpeg_calc_output_dimensions($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0, $ssize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(202);
 if ($4) {
  $5 = $0;
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) + 20|0);
  HEAP32[$7>>2] = 20;
  $8 = $0;
  $9 = (($8) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 24|0);
  HEAP32[$13>>2] = $10;
  $14 = $0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = $0;
  FUNCTION_TABLE_vi[$16 & 127]($17);
 }
 $18 = $0;
 $19 = (($18) + 48|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20<<3;
 $22 = $0;
 $23 = (($22) + 52|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($21>>>0)<=($24>>>0);
 if ($25) {
  $26 = $0;
  $27 = (($26) + 28|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (_jdiv_round_up($28,8)|0);
  $30 = $0;
  $31 = (($30) + 112|0);
  HEAP32[$31>>2] = $29;
  $32 = $0;
  $33 = (($32) + 32|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (_jdiv_round_up($34,8)|0);
  $36 = $0;
  $37 = (($36) + 116|0);
  HEAP32[$37>>2] = $35;
  $38 = $0;
  $39 = (($38) + 316|0);
  HEAP32[$39>>2] = 1;
 } else {
  $40 = $0;
  $41 = (($40) + 48|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $42<<2;
  $44 = $0;
  $45 = (($44) + 52|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($43>>>0)<=($46>>>0);
  if ($47) {
   $48 = $0;
   $49 = (($48) + 28|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_jdiv_round_up($50,4)|0);
   $52 = $0;
   $53 = (($52) + 112|0);
   HEAP32[$53>>2] = $51;
   $54 = $0;
   $55 = (($54) + 32|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (_jdiv_round_up($56,4)|0);
   $58 = $0;
   $59 = (($58) + 116|0);
   HEAP32[$59>>2] = $57;
   $60 = $0;
   $61 = (($60) + 316|0);
   HEAP32[$61>>2] = 2;
  } else {
   $62 = $0;
   $63 = (($62) + 48|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $64<<1;
   $66 = $0;
   $67 = (($66) + 52|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = ($65>>>0)<=($68>>>0);
   if ($69) {
    $70 = $0;
    $71 = (($70) + 28|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (_jdiv_round_up($72,2)|0);
    $74 = $0;
    $75 = (($74) + 112|0);
    HEAP32[$75>>2] = $73;
    $76 = $0;
    $77 = (($76) + 32|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (_jdiv_round_up($78,2)|0);
    $80 = $0;
    $81 = (($80) + 116|0);
    HEAP32[$81>>2] = $79;
    $82 = $0;
    $83 = (($82) + 316|0);
    HEAP32[$83>>2] = 4;
   } else {
    $84 = $0;
    $85 = (($84) + 28|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = $0;
    $88 = (($87) + 112|0);
    HEAP32[$88>>2] = $86;
    $89 = $0;
    $90 = (($89) + 32|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = $0;
    $93 = (($92) + 116|0);
    HEAP32[$93>>2] = $91;
    $94 = $0;
    $95 = (($94) + 316|0);
    HEAP32[$95>>2] = 8;
   }
  }
 }
 $ci = 0;
 $96 = $0;
 $97 = (($96) + 216|0);
 $98 = HEAP32[$97>>2]|0;
 $compptr = $98;
 while(1) {
  $99 = $ci;
  $100 = $0;
  $101 = (($100) + 36|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = ($99|0)<($102|0);
  if (!($103)) {
   break;
  }
  $104 = $0;
  $105 = (($104) + 316|0);
  $106 = HEAP32[$105>>2]|0;
  $ssize = $106;
  while(1) {
   $107 = $ssize;
   $108 = ($107|0)<(8);
   if ($108) {
    $109 = $compptr;
    $110 = (($109) + 8|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = $ssize;
    $113 = Math_imul($111, $112)|0;
    $114 = $113<<1;
    $115 = $0;
    $116 = (($115) + 308|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = $0;
    $119 = (($118) + 316|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = Math_imul($117, $120)|0;
    $122 = ($114|0)<=($121|0);
    if ($122) {
     $123 = $compptr;
     $124 = (($123) + 12|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = $ssize;
     $127 = Math_imul($125, $126)|0;
     $128 = $127<<1;
     $129 = $0;
     $130 = (($129) + 312|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = $0;
     $133 = (($132) + 316|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = Math_imul($131, $134)|0;
     $136 = ($128|0)<=($135|0);
     $228 = $136;
    } else {
     $228 = 0;
    }
   } else {
    $228 = 0;
   }
   if (!($228)) {
    break;
   }
   $137 = $ssize;
   $138 = $137<<1;
   $ssize = $138;
  }
  $139 = $ssize;
  $140 = $compptr;
  $141 = (($140) + 36|0);
  HEAP32[$141>>2] = $139;
  $142 = $ci;
  $143 = (($142) + 1)|0;
  $ci = $143;
  $144 = $compptr;
  $145 = (($144) + 84|0);
  $compptr = $145;
 }
 $ci = 0;
 $146 = $0;
 $147 = (($146) + 216|0);
 $148 = HEAP32[$147>>2]|0;
 $compptr = $148;
 while(1) {
  $149 = $ci;
  $150 = $0;
  $151 = (($150) + 36|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = ($149|0)<($152|0);
  if (!($153)) {
   break;
  }
  $154 = $0;
  $155 = (($154) + 28|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = $compptr;
  $158 = (($157) + 8|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = $compptr;
  $161 = (($160) + 36|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = Math_imul($159, $162)|0;
  $164 = Math_imul($156, $163)|0;
  $165 = $0;
  $166 = (($165) + 308|0);
  $167 = HEAP32[$166>>2]|0;
  $168 = $167<<3;
  $169 = (_jdiv_round_up($164,$168)|0);
  $170 = $compptr;
  $171 = (($170) + 40|0);
  HEAP32[$171>>2] = $169;
  $172 = $0;
  $173 = (($172) + 32|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = $compptr;
  $176 = (($175) + 12|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = $compptr;
  $179 = (($178) + 36|0);
  $180 = HEAP32[$179>>2]|0;
  $181 = Math_imul($177, $180)|0;
  $182 = Math_imul($174, $181)|0;
  $183 = $0;
  $184 = (($183) + 312|0);
  $185 = HEAP32[$184>>2]|0;
  $186 = $185<<3;
  $187 = (_jdiv_round_up($182,$186)|0);
  $188 = $compptr;
  $189 = (($188) + 44|0);
  HEAP32[$189>>2] = $187;
  $190 = $ci;
  $191 = (($190) + 1)|0;
  $ci = $191;
  $192 = $compptr;
  $193 = (($192) + 84|0);
  $compptr = $193;
 }
 $194 = $0;
 $195 = (($194) + 44|0);
 $196 = HEAP32[$195>>2]|0;
 switch ($196|0) {
 case 1:  {
  $197 = $0;
  $198 = (($197) + 120|0);
  HEAP32[$198>>2] = 1;
  break;
 }
 case 3: case 2:  {
  $199 = $0;
  $200 = (($199) + 120|0);
  HEAP32[$200>>2] = 3;
  break;
 }
 case 5: case 4:  {
  $201 = $0;
  $202 = (($201) + 120|0);
  HEAP32[$202>>2] = 4;
  break;
 }
 default: {
  $203 = $0;
  $204 = (($203) + 36|0);
  $205 = HEAP32[$204>>2]|0;
  $206 = $0;
  $207 = (($206) + 120|0);
  HEAP32[$207>>2] = $205;
 }
 }
 $208 = $0;
 $209 = (($208) + 84|0);
 $210 = HEAP32[$209>>2]|0;
 $211 = ($210|0)!=(0);
 if ($211) {
  $217 = 1;
 } else {
  $212 = $0;
  $213 = (($212) + 120|0);
  $214 = HEAP32[$213>>2]|0;
  $217 = $214;
 }
 $215 = $0;
 $216 = (($215) + 124|0);
 HEAP32[$216>>2] = $217;
 $218 = $0;
 $219 = (_use_merged_upsample($218)|0);
 $220 = ($219|0)!=(0);
 if ($220) {
  $221 = $0;
  $222 = (($221) + 312|0);
  $223 = HEAP32[$222>>2]|0;
  $224 = $0;
  $225 = (($224) + 128|0);
  HEAP32[$225>>2] = $223;
  STACKTOP = sp;return;
 } else {
  $226 = $0;
  $227 = (($226) + 128|0);
  HEAP32[$227>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _use_merged_upsample($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 76|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $6 = $1;
  $7 = (($6) + 300|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0);
  if (!($9)) {
   $10 = $1;
   $11 = (($10) + 40|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)!=(3);
   if (!($13)) {
    $14 = $1;
    $15 = (($14) + 36|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($16|0)!=(3);
    if (!($17)) {
     $18 = $1;
     $19 = (($18) + 44|0);
     $20 = HEAP32[$19>>2]|0;
     $21 = ($20|0)!=(2);
     if (!($21)) {
      $22 = $1;
      $23 = (($22) + 120|0);
      $24 = HEAP32[$23>>2]|0;
      $25 = ($24|0)!=(3);
      if (!($25)) {
       $26 = $1;
       $27 = (($26) + 216|0);
       $28 = HEAP32[$27>>2]|0;
       $29 = (($28) + 8|0);
       $30 = HEAP32[$29>>2]|0;
       $31 = ($30|0)!=(2);
       if (!($31)) {
        $32 = $1;
        $33 = (($32) + 216|0);
        $34 = HEAP32[$33>>2]|0;
        $35 = (($34) + 84|0);
        $36 = (($35) + 8|0);
        $37 = HEAP32[$36>>2]|0;
        $38 = ($37|0)!=(1);
        if (!($38)) {
         $39 = $1;
         $40 = (($39) + 216|0);
         $41 = HEAP32[$40>>2]|0;
         $42 = (($41) + 168|0);
         $43 = (($42) + 8|0);
         $44 = HEAP32[$43>>2]|0;
         $45 = ($44|0)!=(1);
         if (!($45)) {
          $46 = $1;
          $47 = (($46) + 216|0);
          $48 = HEAP32[$47>>2]|0;
          $49 = (($48) + 12|0);
          $50 = HEAP32[$49>>2]|0;
          $51 = ($50|0)>(2);
          if (!($51)) {
           $52 = $1;
           $53 = (($52) + 216|0);
           $54 = HEAP32[$53>>2]|0;
           $55 = (($54) + 84|0);
           $56 = (($55) + 12|0);
           $57 = HEAP32[$56>>2]|0;
           $58 = ($57|0)!=(1);
           if (!($58)) {
            $59 = $1;
            $60 = (($59) + 216|0);
            $61 = HEAP32[$60>>2]|0;
            $62 = (($61) + 168|0);
            $63 = (($62) + 12|0);
            $64 = HEAP32[$63>>2]|0;
            $65 = ($64|0)!=(1);
            if (!($65)) {
             $66 = $1;
             $67 = (($66) + 216|0);
             $68 = HEAP32[$67>>2]|0;
             $69 = (($68) + 36|0);
             $70 = HEAP32[$69>>2]|0;
             $71 = $1;
             $72 = (($71) + 316|0);
             $73 = HEAP32[$72>>2]|0;
             $74 = ($70|0)!=($73|0);
             if (!($74)) {
              $75 = $1;
              $76 = (($75) + 216|0);
              $77 = HEAP32[$76>>2]|0;
              $78 = (($77) + 84|0);
              $79 = (($78) + 36|0);
              $80 = HEAP32[$79>>2]|0;
              $81 = $1;
              $82 = (($81) + 316|0);
              $83 = HEAP32[$82>>2]|0;
              $84 = ($80|0)!=($83|0);
              if (!($84)) {
               $85 = $1;
               $86 = (($85) + 216|0);
               $87 = HEAP32[$86>>2]|0;
               $88 = (($87) + 168|0);
               $89 = (($88) + 36|0);
               $90 = HEAP32[$89>>2]|0;
               $91 = $1;
               $92 = (($91) + 316|0);
               $93 = HEAP32[$92>>2]|0;
               $94 = ($90|0)!=($93|0);
               if (!($94)) {
                $0 = 1;
                $95 = $0;
                STACKTOP = sp;return ($95|0);
               }
              }
             }
             $0 = 0;
             $95 = $0;
             STACKTOP = sp;return ($95|0);
            }
           }
          }
         }
        }
       }
       $0 = 0;
       $95 = $0;
       STACKTOP = sp;return ($95|0);
      }
     }
    }
   }
   $0 = 0;
   $95 = $0;
   STACKTOP = sp;return ($95|0);
  }
 }
 $0 = 0;
 $95 = $0;
 STACKTOP = sp;return ($95|0);
}
function _jinit_master_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $master = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,28)|0);
 $master = $6;
 $7 = $master;
 $8 = $0;
 $9 = (($8) + 420|0);
 HEAP32[$9>>2] = $7;
 $10 = $master;
 HEAP32[$10>>2] = 35;
 $11 = $master;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 36;
 $13 = $master;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 $15 = $0;
 _master_selection($15);
 STACKTOP = sp;return;
}
function _prepare_for_output_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $master = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 420|0);
 $3 = HEAP32[$2>>2]|0;
 $master = $3;
 $4 = $master;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $master;
  $9 = (($8) + 8|0);
  HEAP32[$9>>2] = 0;
  $10 = $0;
  $11 = (($10) + 460|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = $0;
  FUNCTION_TABLE_vii[$13 & 127]($14,0);
  $15 = $0;
  $16 = (($15) + 432|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  FUNCTION_TABLE_vii[$18 & 127]($19,2);
  $20 = $0;
  $21 = (($20) + 424|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = $0;
  FUNCTION_TABLE_vii[$23 & 127]($24,2);
 } else {
  $25 = $0;
  $26 = (($25) + 84|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = $0;
   $30 = (($29) + 136|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)==(0|0);
   if ($32) {
    $33 = $0;
    $34 = (($33) + 92|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)!=(0);
    if ($36) {
     $37 = $0;
     $38 = (($37) + 108|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = ($39|0)!=(0);
     if ($40) {
      $41 = $master;
      $42 = (($41) + 24|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = $0;
      $45 = (($44) + 460|0);
      HEAP32[$45>>2] = $43;
      $46 = $master;
      $47 = (($46) + 8|0);
      HEAP32[$47>>2] = 1;
     } else {
      label = 8;
     }
    } else {
     label = 8;
    }
    if ((label|0) == 8) {
     $48 = $0;
     $49 = (($48) + 100|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = ($50|0)!=(0);
     if ($51) {
      $52 = $master;
      $53 = (($52) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = $0;
      $56 = (($55) + 460|0);
      HEAP32[$56>>2] = $54;
     } else {
      $57 = $0;
      $58 = HEAP32[$57>>2]|0;
      $59 = (($58) + 20|0);
      HEAP32[$59>>2] = 46;
      $60 = $0;
      $61 = HEAP32[$60>>2]|0;
      $62 = HEAP32[$61>>2]|0;
      $63 = $0;
      FUNCTION_TABLE_vi[$62 & 127]($63);
     }
    }
   }
  }
  $64 = $0;
  $65 = (($64) + 448|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = HEAP32[$66>>2]|0;
  $68 = $0;
  FUNCTION_TABLE_vi[$67 & 127]($68);
  $69 = $0;
  $70 = (($69) + 428|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  FUNCTION_TABLE_vi[$73 & 127]($74);
  $75 = $0;
  $76 = (($75) + 68|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($77|0)!=(0);
  if (!($78)) {
   $79 = $master;
   $80 = (($79) + 16|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = ($81|0)!=(0);
   if (!($82)) {
    $83 = $0;
    $84 = (($83) + 456|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = HEAP32[$85>>2]|0;
    $87 = $0;
    FUNCTION_TABLE_vi[$86 & 127]($87);
   }
   $88 = $0;
   $89 = (($88) + 452|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$90>>2]|0;
   $92 = $0;
   FUNCTION_TABLE_vi[$91 & 127]($92);
   $93 = $0;
   $94 = (($93) + 84|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = ($95|0)!=(0);
   if ($96) {
    $97 = $0;
    $98 = (($97) + 460|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = HEAP32[$99>>2]|0;
    $101 = $0;
    $102 = $master;
    $103 = (($102) + 8|0);
    $104 = HEAP32[$103>>2]|0;
    FUNCTION_TABLE_vii[$100 & 127]($101,$104);
   }
   $105 = $0;
   $106 = (($105) + 432|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = HEAP32[$107>>2]|0;
   $109 = $0;
   $110 = $master;
   $111 = (($110) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = ($112|0)!=(0);
   $114 = $113 ? 3 : 0;
   FUNCTION_TABLE_vii[$108 & 127]($109,$114);
   $115 = $0;
   $116 = (($115) + 424|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = HEAP32[$117>>2]|0;
   $119 = $0;
   FUNCTION_TABLE_vii[$118 & 127]($119,0);
  }
 }
 $120 = $0;
 $121 = (($120) + 8|0);
 $122 = HEAP32[$121>>2]|0;
 $123 = ($122|0)!=(0|0);
 if (!($123)) {
  STACKTOP = sp;return;
 }
 $124 = $master;
 $125 = (($124) + 12|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = $0;
 $128 = (($127) + 8|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = (($129) + 12|0);
 HEAP32[$130>>2] = $126;
 $131 = $master;
 $132 = (($131) + 12|0);
 $133 = HEAP32[$132>>2]|0;
 $134 = $master;
 $135 = (($134) + 8|0);
 $136 = HEAP32[$135>>2]|0;
 $137 = ($136|0)!=(0);
 $138 = $137 ? 2 : 1;
 $139 = (($133) + ($138))|0;
 $140 = $0;
 $141 = (($140) + 8|0);
 $142 = HEAP32[$141>>2]|0;
 $143 = (($142) + 16|0);
 HEAP32[$143>>2] = $139;
 $144 = $0;
 $145 = (($144) + 64|0);
 $146 = HEAP32[$145>>2]|0;
 $147 = ($146|0)!=(0);
 if ($147) {
  $148 = $0;
  $149 = (($148) + 436|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = (($150) + 20|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = ($152|0)!=(0);
  if (!($153)) {
   $154 = $0;
   $155 = (($154) + 108|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = ($156|0)!=(0);
   $158 = $157 ? 2 : 1;
   $159 = $0;
   $160 = (($159) + 8|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = (($161) + 16|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = (($163) + ($158))|0;
   HEAP32[$162>>2] = $164;
  }
 }
 STACKTOP = sp;return;
}
function _finish_output_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $master = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 420|0);
 $3 = HEAP32[$2>>2]|0;
 $master = $3;
 $4 = $0;
 $5 = (($4) + 84|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $0;
  $9 = (($8) + 460|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $0;
  FUNCTION_TABLE_vi[$12 & 127]($13);
 }
 $14 = $master;
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$15>>2] = $17;
 STACKTOP = sp;return;
}
function _master_selection($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $jd_samplesperrow = 0, $master = 0, $nscans = 0, $samplesperrow = 0, $use_c_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 420|0);
 $3 = HEAP32[$2>>2]|0;
 $master = $3;
 $4 = $0;
 _jpeg_calc_output_dimensions($4);
 $5 = $0;
 _prepare_range_limit_table($5);
 $6 = $0;
 $7 = (($6) + 112|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = (($9) + 120|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = Math_imul($8, $11)|0;
 $samplesperrow = $12;
 $13 = $samplesperrow;
 $jd_samplesperrow = $13;
 $14 = $jd_samplesperrow;
 $15 = $samplesperrow;
 $16 = ($14|0)!=($15|0);
 if ($16) {
  $17 = $0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 20|0);
  HEAP32[$19>>2] = 70;
  $20 = $0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = $0;
  FUNCTION_TABLE_vi[$22 & 127]($23);
 }
 $24 = $master;
 $25 = (($24) + 12|0);
 HEAP32[$25>>2] = 0;
 $26 = $0;
 $27 = (_use_merged_upsample($26)|0);
 $28 = $master;
 $29 = (($28) + 16|0);
 HEAP32[$29>>2] = $27;
 $30 = $master;
 $31 = (($30) + 20|0);
 HEAP32[$31>>2] = 0;
 $32 = $master;
 $33 = (($32) + 24|0);
 HEAP32[$33>>2] = 0;
 $34 = $0;
 $35 = (($34) + 84|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = $0;
  $39 = (($38) + 64|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $42 = $0;
  $43 = (($42) + 100|0);
  HEAP32[$43>>2] = 0;
  $44 = $0;
  $45 = (($44) + 104|0);
  HEAP32[$45>>2] = 0;
  $46 = $0;
  $47 = (($46) + 108|0);
  HEAP32[$47>>2] = 0;
 }
 $48 = $0;
 $49 = (($48) + 84|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = $0;
  $53 = (($52) + 68|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0);
  if ($55) {
   $56 = $0;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 20|0);
   HEAP32[$58>>2] = 47;
   $59 = $0;
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$60>>2]|0;
   $62 = $0;
   FUNCTION_TABLE_vi[$61 & 127]($62);
  }
  $63 = $0;
  $64 = (($63) + 120|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)!=(3);
  if ($66) {
   $67 = $0;
   $68 = (($67) + 100|0);
   HEAP32[$68>>2] = 1;
   $69 = $0;
   $70 = (($69) + 104|0);
   HEAP32[$70>>2] = 0;
   $71 = $0;
   $72 = (($71) + 108|0);
   HEAP32[$72>>2] = 0;
   $73 = $0;
   $74 = (($73) + 136|0);
   HEAP32[$74>>2] = 0;
  } else {
   $75 = $0;
   $76 = (($75) + 136|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = ($77|0)!=(0|0);
   if ($78) {
    $79 = $0;
    $80 = (($79) + 104|0);
    HEAP32[$80>>2] = 1;
   } else {
    $81 = $0;
    $82 = (($81) + 92|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ($83|0)!=(0);
    if ($84) {
     $85 = $0;
     $86 = (($85) + 108|0);
     HEAP32[$86>>2] = 1;
    } else {
     $87 = $0;
     $88 = (($87) + 100|0);
     HEAP32[$88>>2] = 1;
    }
   }
  }
  $89 = $0;
  $90 = (($89) + 100|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = ($91|0)!=(0);
  if ($92) {
   $93 = $0;
   _jinit_1pass_quantizer($93);
   $94 = $0;
   $95 = (($94) + 460|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = $master;
   $98 = (($97) + 20|0);
   HEAP32[$98>>2] = $96;
  }
  $99 = $0;
  $100 = (($99) + 108|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = ($101|0)!=(0);
  if ($102) {
   label = 22;
  } else {
   $103 = $0;
   $104 = (($103) + 104|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = ($105|0)!=(0);
   if ($106) {
    label = 22;
   }
  }
  if ((label|0) == 22) {
   $107 = $0;
   _jinit_2pass_quantizer($107);
   $108 = $0;
   $109 = (($108) + 460|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = $master;
   $112 = (($111) + 24|0);
   HEAP32[$112>>2] = $110;
  }
 }
 $113 = $0;
 $114 = (($113) + 68|0);
 $115 = HEAP32[$114>>2]|0;
 $116 = ($115|0)!=(0);
 if (!($116)) {
  $117 = $master;
  $118 = (($117) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = ($119|0)!=(0);
  if ($120) {
   $121 = $0;
   _jinit_merged_upsampler($121);
  } else {
   $122 = $0;
   _jinit_color_deconverter($122);
   $123 = $0;
   _jinit_upsampler($123);
  }
  $124 = $0;
  $125 = $0;
  $126 = (($125) + 108|0);
  $127 = HEAP32[$126>>2]|0;
  _jinit_d_post_controller($124,$127);
 }
 $128 = $0;
 _jinit_inverse_dct($128);
 $129 = $0;
 $130 = (($129) + 224|0);
 $131 = HEAP32[$130>>2]|0;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = $0;
  $134 = HEAP32[$133>>2]|0;
  $135 = (($134) + 20|0);
  HEAP32[$135>>2] = 1;
  $136 = $0;
  $137 = HEAP32[$136>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = $0;
  FUNCTION_TABLE_vi[$138 & 127]($139);
 } else {
  $140 = $0;
  $141 = (($140) + 220|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = ($142|0)!=(0);
  if ($143) {
   $144 = $0;
   _jinit_phuff_decoder($144);
  } else {
   $145 = $0;
   _jinit_huff_decoder($145);
  }
 }
 $146 = $0;
 $147 = (($146) + 436|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = (($148) + 16|0);
 $150 = HEAP32[$149>>2]|0;
 $151 = ($150|0)!=(0);
 if ($151) {
  $157 = 1;
 } else {
  $152 = $0;
  $153 = (($152) + 64|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = ($154|0)!=(0);
  $157 = $155;
 }
 $156 = $157&1;
 $use_c_buffer = $156;
 $158 = $0;
 $159 = $use_c_buffer;
 _jinit_d_coef_controller($158,$159);
 $160 = $0;
 $161 = (($160) + 68|0);
 $162 = HEAP32[$161>>2]|0;
 $163 = ($162|0)!=(0);
 if (!($163)) {
  $164 = $0;
  _jinit_d_main_controller($164,0);
 }
 $165 = $0;
 $166 = (($165) + 4|0);
 $167 = HEAP32[$166>>2]|0;
 $168 = (($167) + 24|0);
 $169 = HEAP32[$168>>2]|0;
 $170 = $0;
 FUNCTION_TABLE_vi[$169 & 127]($170);
 $171 = $0;
 $172 = (($171) + 436|0);
 $173 = HEAP32[$172>>2]|0;
 $174 = (($173) + 8|0);
 $175 = HEAP32[$174>>2]|0;
 $176 = $0;
 FUNCTION_TABLE_vi[$175 & 127]($176);
 $177 = $0;
 $178 = (($177) + 8|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = ($179|0)!=(0|0);
 if (!($180)) {
  STACKTOP = sp;return;
 }
 $181 = $0;
 $182 = (($181) + 64|0);
 $183 = HEAP32[$182>>2]|0;
 $184 = ($183|0)!=(0);
 if ($184) {
  STACKTOP = sp;return;
 }
 $185 = $0;
 $186 = (($185) + 436|0);
 $187 = HEAP32[$186>>2]|0;
 $188 = (($187) + 16|0);
 $189 = HEAP32[$188>>2]|0;
 $190 = ($189|0)!=(0);
 if (!($190)) {
  STACKTOP = sp;return;
 }
 $191 = $0;
 $192 = (($191) + 220|0);
 $193 = HEAP32[$192>>2]|0;
 $194 = ($193|0)!=(0);
 if ($194) {
  $195 = $0;
  $196 = (($195) + 36|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = ($197*3)|0;
  $199 = (2 + ($198))|0;
  $nscans = $199;
 } else {
  $200 = $0;
  $201 = (($200) + 36|0);
  $202 = HEAP32[$201>>2]|0;
  $nscans = $202;
 }
 $203 = $0;
 $204 = (($203) + 8|0);
 $205 = HEAP32[$204>>2]|0;
 $206 = (($205) + 4|0);
 HEAP32[$206>>2] = 0;
 $207 = $0;
 $208 = (($207) + 320|0);
 $209 = HEAP32[$208>>2]|0;
 $210 = $nscans;
 $211 = Math_imul($209, $210)|0;
 $212 = $0;
 $213 = (($212) + 8|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = (($214) + 8|0);
 HEAP32[$215>>2] = $211;
 $216 = $0;
 $217 = (($216) + 8|0);
 $218 = HEAP32[$217>>2]|0;
 $219 = (($218) + 12|0);
 HEAP32[$219>>2] = 0;
 $220 = $0;
 $221 = (($220) + 108|0);
 $222 = HEAP32[$221>>2]|0;
 $223 = ($222|0)!=(0);
 $224 = $223 ? 3 : 2;
 $225 = $0;
 $226 = (($225) + 8|0);
 $227 = HEAP32[$226>>2]|0;
 $228 = (($227) + 16|0);
 HEAP32[$228>>2] = $224;
 $229 = $master;
 $230 = (($229) + 12|0);
 $231 = HEAP32[$230>>2]|0;
 $232 = (($231) + 1)|0;
 HEAP32[$230>>2] = $232;
 STACKTOP = sp;return;
}
function _prepare_range_limit_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $table = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,1408)|0);
 $table = $6;
 $7 = $table;
 $8 = (($7) + 256|0);
 $table = $8;
 $9 = $table;
 $10 = $0;
 $11 = (($10) + 324|0);
 HEAP32[$11>>2] = $9;
 $12 = $table;
 $13 = (($12) + -256|0);
 _memset(($13|0),0,256)|0;
 $i = 0;
 while(1) {
  $14 = $i;
  $15 = ($14|0)<=(255);
  if (!($15)) {
   break;
  }
  $16 = $i;
  $17 = $16&255;
  $18 = $i;
  $19 = $table;
  $20 = (($19) + ($18)|0);
  HEAP8[$20>>0] = $17;
  $21 = $i;
  $22 = (($21) + 1)|0;
  $i = $22;
 }
 $23 = $table;
 $24 = (($23) + 128|0);
 $table = $24;
 $i = 128;
 while(1) {
  $25 = $i;
  $26 = ($25|0)<(512);
  if (!($26)) {
   break;
  }
  $27 = $i;
  $28 = $table;
  $29 = (($28) + ($27)|0);
  HEAP8[$29>>0] = -1;
  $30 = $i;
  $31 = (($30) + 1)|0;
  $i = $31;
 }
 $32 = $table;
 $33 = (($32) + 512|0);
 _memset(($33|0),0,384)|0;
 $34 = $table;
 $35 = (($34) + 896|0);
 $36 = $0;
 $37 = (($36) + 324|0);
 $38 = HEAP32[$37>>2]|0;
 dest=$35+0|0; src=$38+0|0; stop=dest+128|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _jinit_merged_upsampler($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,48)|0);
 $upsample = $6;
 $7 = $upsample;
 $8 = $0;
 $9 = (($8) + 452|0);
 HEAP32[$9>>2] = $7;
 $10 = $upsample;
 HEAP32[$10>>2] = 37;
 $11 = $upsample;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = $0;
 $14 = (($13) + 112|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 $17 = (($16) + 120|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = Math_imul($15, $18)|0;
 $20 = $upsample;
 $21 = (($20) + 40|0);
 HEAP32[$21>>2] = $19;
 $22 = $0;
 $23 = (($22) + 312|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)==(2);
 if ($25) {
  $26 = $upsample;
  $27 = (($26) + 4|0);
  HEAP32[$27>>2] = 38;
  $28 = $upsample;
  $29 = (($28) + 12|0);
  HEAP32[$29>>2] = 39;
  $30 = $0;
  $31 = (($30) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  $36 = $upsample;
  $37 = (($36) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = $38;
  $40 = (FUNCTION_TABLE_iiii[$34 & 127]($35,1,$39)|0);
  $41 = $upsample;
  $42 = (($41) + 32|0);
  HEAP32[$42>>2] = $40;
  $49 = $0;
  _build_ycc_rgb_table($49);
  STACKTOP = sp;return;
 } else {
  $43 = $upsample;
  $44 = (($43) + 4|0);
  HEAP32[$44>>2] = 40;
  $45 = $upsample;
  $46 = (($45) + 12|0);
  HEAP32[$46>>2] = 41;
  $47 = $upsample;
  $48 = (($47) + 32|0);
  HEAP32[$48>>2] = 0;
  $49 = $0;
  _build_ycc_rgb_table($49);
  STACKTOP = sp;return;
 }
}
function _start_pass_merged_upsample($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $upsample = $3;
 $4 = $upsample;
 $5 = (($4) + 36|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 $7 = (($6) + 116|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $upsample;
 $10 = (($9) + 44|0);
 HEAP32[$10>>2] = $8;
 STACKTOP = sp;return;
}
function _merged_2v_upsample($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $num_rows = 0, $upsample = 0, $work_ptrs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $work_ptrs = sp + 8|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 452|0);
 $9 = HEAP32[$8>>2]|0;
 $upsample = $9;
 $10 = $upsample;
 $11 = (($10) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $upsample;
  $15 = (($14) + 32|0);
  $16 = $4;
  $17 = $5;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + ($18<<2)|0);
  $20 = $upsample;
  $21 = (($20) + 40|0);
  $22 = HEAP32[$21>>2]|0;
  _jcopy_sample_rows($15,0,$19,0,1,$22);
  $num_rows = 1;
  $23 = $upsample;
  $24 = (($23) + 36|0);
  HEAP32[$24>>2] = 0;
 } else {
  $num_rows = 2;
  $25 = $num_rows;
  $26 = $upsample;
  $27 = (($26) + 44|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25>>>0)>($28>>>0);
  if ($29) {
   $30 = $upsample;
   $31 = (($30) + 44|0);
   $32 = HEAP32[$31>>2]|0;
   $num_rows = $32;
  }
  $33 = $5;
  $34 = HEAP32[$33>>2]|0;
  $35 = $6;
  $36 = (($35) - ($34))|0;
  $6 = $36;
  $37 = $num_rows;
  $38 = $6;
  $39 = ($37>>>0)>($38>>>0);
  if ($39) {
   $40 = $6;
   $num_rows = $40;
  }
  $41 = $5;
  $42 = HEAP32[$41>>2]|0;
  $43 = $4;
  $44 = (($43) + ($42<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$work_ptrs>>2] = $45;
  $46 = $num_rows;
  $47 = ($46>>>0)>(1);
  if ($47) {
   $48 = $5;
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + 1)|0;
   $51 = $4;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = (($work_ptrs) + 4|0);
   HEAP32[$54>>2] = $53;
  } else {
   $55 = $upsample;
   $56 = (($55) + 32|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($work_ptrs) + 4|0);
   HEAP32[$58>>2] = $57;
   $59 = $upsample;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = 1;
  }
  $61 = $upsample;
  $62 = (($61) + 12|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $0;
  $65 = $1;
  $66 = $2;
  $67 = HEAP32[$66>>2]|0;
  FUNCTION_TABLE_viiii[$63 & 127]($64,$65,$67,$work_ptrs);
 }
 $68 = $num_rows;
 $69 = $5;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + ($68))|0;
 HEAP32[$69>>2] = $71;
 $72 = $num_rows;
 $73 = $upsample;
 $74 = (($73) + 44|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($75) - ($72))|0;
 HEAP32[$74>>2] = $76;
 $77 = $upsample;
 $78 = (($77) + 36|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = ($79|0)!=(0);
 if ($80) {
  STACKTOP = sp;return;
 }
 $81 = $2;
 $82 = HEAP32[$81>>2]|0;
 $83 = (($82) + 1)|0;
 HEAP32[$81>>2] = $83;
 STACKTOP = sp;return;
}
function _h2v2_merged_upsample($cinfo,$input_buf,$in_row_group_ctr,$output_buf) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cblue = 0, $cgreen = 0, $col = 0, $cr = 0, $cred = 0, $inptr00 = 0, $inptr01 = 0, $inptr1 = 0, $inptr2 = 0, $outptr0 = 0, $outptr1 = 0, $range_limit = 0;
 var $upsample = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $output_buf;
 $4 = $0;
 $5 = (($4) + 452|0);
 $6 = HEAP32[$5>>2]|0;
 $upsample = $6;
 $7 = $0;
 $8 = (($7) + 324|0);
 $9 = HEAP32[$8>>2]|0;
 $range_limit = $9;
 $10 = $upsample;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $Crrtab = $12;
 $13 = $upsample;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $Cbbtab = $15;
 $16 = $upsample;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $Crgtab = $18;
 $19 = $upsample;
 $20 = (($19) + 28|0);
 $21 = HEAP32[$20>>2]|0;
 $Cbgtab = $21;
 $22 = $2;
 $23 = $22<<1;
 $24 = $1;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($23<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 $inptr00 = $27;
 $28 = $2;
 $29 = $28<<1;
 $30 = (($29) + 1)|0;
 $31 = $1;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($30<<2)|0);
 $34 = HEAP32[$33>>2]|0;
 $inptr01 = $34;
 $35 = $2;
 $36 = $1;
 $37 = (($36) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35<<2)|0);
 $40 = HEAP32[$39>>2]|0;
 $inptr1 = $40;
 $41 = $2;
 $42 = $1;
 $43 = (($42) + 8|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<2)|0);
 $46 = HEAP32[$45>>2]|0;
 $inptr2 = $46;
 $47 = $3;
 $48 = HEAP32[$47>>2]|0;
 $outptr0 = $48;
 $49 = $3;
 $50 = (($49) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $outptr1 = $51;
 $52 = $0;
 $53 = (($52) + 112|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $54 >>> 1;
 $col = $55;
 while(1) {
  $56 = $col;
  $57 = ($56>>>0)>(0);
  if (!($57)) {
   break;
  }
  $58 = $inptr1;
  $59 = (($58) + 1|0);
  $inptr1 = $59;
  $60 = HEAP8[$58>>0]|0;
  $61 = $60&255;
  $cb = $61;
  $62 = $inptr2;
  $63 = (($62) + 1|0);
  $inptr2 = $63;
  $64 = HEAP8[$62>>0]|0;
  $65 = $64&255;
  $cr = $65;
  $66 = $cr;
  $67 = $Crrtab;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  $cred = $69;
  $70 = $cb;
  $71 = $Cbgtab;
  $72 = (($71) + ($70<<2)|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $cr;
  $75 = $Crgtab;
  $76 = (($75) + ($74<<2)|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (($73) + ($77))|0;
  $79 = $78 >> 16;
  $cgreen = $79;
  $80 = $cb;
  $81 = $Cbbtab;
  $82 = (($81) + ($80<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $cblue = $83;
  $84 = $inptr00;
  $85 = (($84) + 1|0);
  $inptr00 = $85;
  $86 = HEAP8[$84>>0]|0;
  $87 = $86&255;
  $y = $87;
  $88 = $y;
  $89 = $cred;
  $90 = (($88) + ($89))|0;
  $91 = $range_limit;
  $92 = (($91) + ($90)|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $outptr0;
  HEAP8[$94>>0] = $93;
  $95 = $y;
  $96 = $cgreen;
  $97 = (($95) + ($96))|0;
  $98 = $range_limit;
  $99 = (($98) + ($97)|0);
  $100 = HEAP8[$99>>0]|0;
  $101 = $outptr0;
  $102 = (($101) + 1|0);
  HEAP8[$102>>0] = $100;
  $103 = $y;
  $104 = $cblue;
  $105 = (($103) + ($104))|0;
  $106 = $range_limit;
  $107 = (($106) + ($105)|0);
  $108 = HEAP8[$107>>0]|0;
  $109 = $outptr0;
  $110 = (($109) + 2|0);
  HEAP8[$110>>0] = $108;
  $111 = $outptr0;
  $112 = (($111) + 3|0);
  $outptr0 = $112;
  $113 = $inptr00;
  $114 = (($113) + 1|0);
  $inptr00 = $114;
  $115 = HEAP8[$113>>0]|0;
  $116 = $115&255;
  $y = $116;
  $117 = $y;
  $118 = $cred;
  $119 = (($117) + ($118))|0;
  $120 = $range_limit;
  $121 = (($120) + ($119)|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $outptr0;
  HEAP8[$123>>0] = $122;
  $124 = $y;
  $125 = $cgreen;
  $126 = (($124) + ($125))|0;
  $127 = $range_limit;
  $128 = (($127) + ($126)|0);
  $129 = HEAP8[$128>>0]|0;
  $130 = $outptr0;
  $131 = (($130) + 1|0);
  HEAP8[$131>>0] = $129;
  $132 = $y;
  $133 = $cblue;
  $134 = (($132) + ($133))|0;
  $135 = $range_limit;
  $136 = (($135) + ($134)|0);
  $137 = HEAP8[$136>>0]|0;
  $138 = $outptr0;
  $139 = (($138) + 2|0);
  HEAP8[$139>>0] = $137;
  $140 = $outptr0;
  $141 = (($140) + 3|0);
  $outptr0 = $141;
  $142 = $inptr01;
  $143 = (($142) + 1|0);
  $inptr01 = $143;
  $144 = HEAP8[$142>>0]|0;
  $145 = $144&255;
  $y = $145;
  $146 = $y;
  $147 = $cred;
  $148 = (($146) + ($147))|0;
  $149 = $range_limit;
  $150 = (($149) + ($148)|0);
  $151 = HEAP8[$150>>0]|0;
  $152 = $outptr1;
  HEAP8[$152>>0] = $151;
  $153 = $y;
  $154 = $cgreen;
  $155 = (($153) + ($154))|0;
  $156 = $range_limit;
  $157 = (($156) + ($155)|0);
  $158 = HEAP8[$157>>0]|0;
  $159 = $outptr1;
  $160 = (($159) + 1|0);
  HEAP8[$160>>0] = $158;
  $161 = $y;
  $162 = $cblue;
  $163 = (($161) + ($162))|0;
  $164 = $range_limit;
  $165 = (($164) + ($163)|0);
  $166 = HEAP8[$165>>0]|0;
  $167 = $outptr1;
  $168 = (($167) + 2|0);
  HEAP8[$168>>0] = $166;
  $169 = $outptr1;
  $170 = (($169) + 3|0);
  $outptr1 = $170;
  $171 = $inptr01;
  $172 = (($171) + 1|0);
  $inptr01 = $172;
  $173 = HEAP8[$171>>0]|0;
  $174 = $173&255;
  $y = $174;
  $175 = $y;
  $176 = $cred;
  $177 = (($175) + ($176))|0;
  $178 = $range_limit;
  $179 = (($178) + ($177)|0);
  $180 = HEAP8[$179>>0]|0;
  $181 = $outptr1;
  HEAP8[$181>>0] = $180;
  $182 = $y;
  $183 = $cgreen;
  $184 = (($182) + ($183))|0;
  $185 = $range_limit;
  $186 = (($185) + ($184)|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $outptr1;
  $189 = (($188) + 1|0);
  HEAP8[$189>>0] = $187;
  $190 = $y;
  $191 = $cblue;
  $192 = (($190) + ($191))|0;
  $193 = $range_limit;
  $194 = (($193) + ($192)|0);
  $195 = HEAP8[$194>>0]|0;
  $196 = $outptr1;
  $197 = (($196) + 2|0);
  HEAP8[$197>>0] = $195;
  $198 = $outptr1;
  $199 = (($198) + 3|0);
  $outptr1 = $199;
  $200 = $col;
  $201 = (($200) + -1)|0;
  $col = $201;
 }
 $202 = $0;
 $203 = (($202) + 112|0);
 $204 = HEAP32[$203>>2]|0;
 $205 = $204 & 1;
 $206 = ($205|0)!=(0);
 if (!($206)) {
  STACKTOP = sp;return;
 }
 $207 = $inptr1;
 $208 = HEAP8[$207>>0]|0;
 $209 = $208&255;
 $cb = $209;
 $210 = $inptr2;
 $211 = HEAP8[$210>>0]|0;
 $212 = $211&255;
 $cr = $212;
 $213 = $cr;
 $214 = $Crrtab;
 $215 = (($214) + ($213<<2)|0);
 $216 = HEAP32[$215>>2]|0;
 $cred = $216;
 $217 = $cb;
 $218 = $Cbgtab;
 $219 = (($218) + ($217<<2)|0);
 $220 = HEAP32[$219>>2]|0;
 $221 = $cr;
 $222 = $Crgtab;
 $223 = (($222) + ($221<<2)|0);
 $224 = HEAP32[$223>>2]|0;
 $225 = (($220) + ($224))|0;
 $226 = $225 >> 16;
 $cgreen = $226;
 $227 = $cb;
 $228 = $Cbbtab;
 $229 = (($228) + ($227<<2)|0);
 $230 = HEAP32[$229>>2]|0;
 $cblue = $230;
 $231 = $inptr00;
 $232 = HEAP8[$231>>0]|0;
 $233 = $232&255;
 $y = $233;
 $234 = $y;
 $235 = $cred;
 $236 = (($234) + ($235))|0;
 $237 = $range_limit;
 $238 = (($237) + ($236)|0);
 $239 = HEAP8[$238>>0]|0;
 $240 = $outptr0;
 HEAP8[$240>>0] = $239;
 $241 = $y;
 $242 = $cgreen;
 $243 = (($241) + ($242))|0;
 $244 = $range_limit;
 $245 = (($244) + ($243)|0);
 $246 = HEAP8[$245>>0]|0;
 $247 = $outptr0;
 $248 = (($247) + 1|0);
 HEAP8[$248>>0] = $246;
 $249 = $y;
 $250 = $cblue;
 $251 = (($249) + ($250))|0;
 $252 = $range_limit;
 $253 = (($252) + ($251)|0);
 $254 = HEAP8[$253>>0]|0;
 $255 = $outptr0;
 $256 = (($255) + 2|0);
 HEAP8[$256>>0] = $254;
 $257 = $inptr01;
 $258 = HEAP8[$257>>0]|0;
 $259 = $258&255;
 $y = $259;
 $260 = $y;
 $261 = $cred;
 $262 = (($260) + ($261))|0;
 $263 = $range_limit;
 $264 = (($263) + ($262)|0);
 $265 = HEAP8[$264>>0]|0;
 $266 = $outptr1;
 HEAP8[$266>>0] = $265;
 $267 = $y;
 $268 = $cgreen;
 $269 = (($267) + ($268))|0;
 $270 = $range_limit;
 $271 = (($270) + ($269)|0);
 $272 = HEAP8[$271>>0]|0;
 $273 = $outptr1;
 $274 = (($273) + 1|0);
 HEAP8[$274>>0] = $272;
 $275 = $y;
 $276 = $cblue;
 $277 = (($275) + ($276))|0;
 $278 = $range_limit;
 $279 = (($278) + ($277)|0);
 $280 = HEAP8[$279>>0]|0;
 $281 = $outptr1;
 $282 = (($281) + 2|0);
 HEAP8[$282>>0] = $280;
 STACKTOP = sp;return;
}
function _merged_1v_upsample($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 452|0);
 $9 = HEAP32[$8>>2]|0;
 $upsample = $9;
 $10 = $upsample;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = $1;
 $15 = $2;
 $16 = HEAP32[$15>>2]|0;
 $17 = $4;
 $18 = $5;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + ($19<<2)|0);
 FUNCTION_TABLE_viiii[$12 & 127]($13,$14,$16,$20);
 $21 = $5;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 1)|0;
 HEAP32[$21>>2] = $23;
 $24 = $2;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$24>>2] = $26;
 STACKTOP = sp;return;
}
function _h2v1_merged_upsample($cinfo,$input_buf,$in_row_group_ctr,$output_buf) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cblue = 0, $cgreen = 0, $col = 0, $cr = 0, $cred = 0, $inptr0 = 0, $inptr1 = 0;
 var $inptr2 = 0, $outptr = 0, $range_limit = 0, $upsample = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $output_buf;
 $4 = $0;
 $5 = (($4) + 452|0);
 $6 = HEAP32[$5>>2]|0;
 $upsample = $6;
 $7 = $0;
 $8 = (($7) + 324|0);
 $9 = HEAP32[$8>>2]|0;
 $range_limit = $9;
 $10 = $upsample;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $Crrtab = $12;
 $13 = $upsample;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $Cbbtab = $15;
 $16 = $upsample;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $Crgtab = $18;
 $19 = $upsample;
 $20 = (($19) + 28|0);
 $21 = HEAP32[$20>>2]|0;
 $Cbgtab = $21;
 $22 = $2;
 $23 = $1;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + ($22<<2)|0);
 $26 = HEAP32[$25>>2]|0;
 $inptr0 = $26;
 $27 = $2;
 $28 = $1;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + ($27<<2)|0);
 $32 = HEAP32[$31>>2]|0;
 $inptr1 = $32;
 $33 = $2;
 $34 = $1;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + ($33<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $inptr2 = $38;
 $39 = $3;
 $40 = HEAP32[$39>>2]|0;
 $outptr = $40;
 $41 = $0;
 $42 = (($41) + 112|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $43 >>> 1;
 $col = $44;
 while(1) {
  $45 = $col;
  $46 = ($45>>>0)>(0);
  if (!($46)) {
   break;
  }
  $47 = $inptr1;
  $48 = (($47) + 1|0);
  $inptr1 = $48;
  $49 = HEAP8[$47>>0]|0;
  $50 = $49&255;
  $cb = $50;
  $51 = $inptr2;
  $52 = (($51) + 1|0);
  $inptr2 = $52;
  $53 = HEAP8[$51>>0]|0;
  $54 = $53&255;
  $cr = $54;
  $55 = $cr;
  $56 = $Crrtab;
  $57 = (($56) + ($55<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $cred = $58;
  $59 = $cb;
  $60 = $Cbgtab;
  $61 = (($60) + ($59<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = $cr;
  $64 = $Crgtab;
  $65 = (($64) + ($63<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($62) + ($66))|0;
  $68 = $67 >> 16;
  $cgreen = $68;
  $69 = $cb;
  $70 = $Cbbtab;
  $71 = (($70) + ($69<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $cblue = $72;
  $73 = $inptr0;
  $74 = (($73) + 1|0);
  $inptr0 = $74;
  $75 = HEAP8[$73>>0]|0;
  $76 = $75&255;
  $y = $76;
  $77 = $y;
  $78 = $cred;
  $79 = (($77) + ($78))|0;
  $80 = $range_limit;
  $81 = (($80) + ($79)|0);
  $82 = HEAP8[$81>>0]|0;
  $83 = $outptr;
  HEAP8[$83>>0] = $82;
  $84 = $y;
  $85 = $cgreen;
  $86 = (($84) + ($85))|0;
  $87 = $range_limit;
  $88 = (($87) + ($86)|0);
  $89 = HEAP8[$88>>0]|0;
  $90 = $outptr;
  $91 = (($90) + 1|0);
  HEAP8[$91>>0] = $89;
  $92 = $y;
  $93 = $cblue;
  $94 = (($92) + ($93))|0;
  $95 = $range_limit;
  $96 = (($95) + ($94)|0);
  $97 = HEAP8[$96>>0]|0;
  $98 = $outptr;
  $99 = (($98) + 2|0);
  HEAP8[$99>>0] = $97;
  $100 = $outptr;
  $101 = (($100) + 3|0);
  $outptr = $101;
  $102 = $inptr0;
  $103 = (($102) + 1|0);
  $inptr0 = $103;
  $104 = HEAP8[$102>>0]|0;
  $105 = $104&255;
  $y = $105;
  $106 = $y;
  $107 = $cred;
  $108 = (($106) + ($107))|0;
  $109 = $range_limit;
  $110 = (($109) + ($108)|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = $outptr;
  HEAP8[$112>>0] = $111;
  $113 = $y;
  $114 = $cgreen;
  $115 = (($113) + ($114))|0;
  $116 = $range_limit;
  $117 = (($116) + ($115)|0);
  $118 = HEAP8[$117>>0]|0;
  $119 = $outptr;
  $120 = (($119) + 1|0);
  HEAP8[$120>>0] = $118;
  $121 = $y;
  $122 = $cblue;
  $123 = (($121) + ($122))|0;
  $124 = $range_limit;
  $125 = (($124) + ($123)|0);
  $126 = HEAP8[$125>>0]|0;
  $127 = $outptr;
  $128 = (($127) + 2|0);
  HEAP8[$128>>0] = $126;
  $129 = $outptr;
  $130 = (($129) + 3|0);
  $outptr = $130;
  $131 = $col;
  $132 = (($131) + -1)|0;
  $col = $132;
 }
 $133 = $0;
 $134 = (($133) + 112|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = $135 & 1;
 $137 = ($136|0)!=(0);
 if (!($137)) {
  STACKTOP = sp;return;
 }
 $138 = $inptr1;
 $139 = HEAP8[$138>>0]|0;
 $140 = $139&255;
 $cb = $140;
 $141 = $inptr2;
 $142 = HEAP8[$141>>0]|0;
 $143 = $142&255;
 $cr = $143;
 $144 = $cr;
 $145 = $Crrtab;
 $146 = (($145) + ($144<<2)|0);
 $147 = HEAP32[$146>>2]|0;
 $cred = $147;
 $148 = $cb;
 $149 = $Cbgtab;
 $150 = (($149) + ($148<<2)|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = $cr;
 $153 = $Crgtab;
 $154 = (($153) + ($152<<2)|0);
 $155 = HEAP32[$154>>2]|0;
 $156 = (($151) + ($155))|0;
 $157 = $156 >> 16;
 $cgreen = $157;
 $158 = $cb;
 $159 = $Cbbtab;
 $160 = (($159) + ($158<<2)|0);
 $161 = HEAP32[$160>>2]|0;
 $cblue = $161;
 $162 = $inptr0;
 $163 = HEAP8[$162>>0]|0;
 $164 = $163&255;
 $y = $164;
 $165 = $y;
 $166 = $cred;
 $167 = (($165) + ($166))|0;
 $168 = $range_limit;
 $169 = (($168) + ($167)|0);
 $170 = HEAP8[$169>>0]|0;
 $171 = $outptr;
 HEAP8[$171>>0] = $170;
 $172 = $y;
 $173 = $cgreen;
 $174 = (($172) + ($173))|0;
 $175 = $range_limit;
 $176 = (($175) + ($174)|0);
 $177 = HEAP8[$176>>0]|0;
 $178 = $outptr;
 $179 = (($178) + 1|0);
 HEAP8[$179>>0] = $177;
 $180 = $y;
 $181 = $cblue;
 $182 = (($180) + ($181))|0;
 $183 = $range_limit;
 $184 = (($183) + ($182)|0);
 $185 = HEAP8[$184>>0]|0;
 $186 = $outptr;
 $187 = (($186) + 2|0);
 HEAP8[$187>>0] = $185;
 STACKTOP = sp;return;
}
function _build_ycc_rgb_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $i = 0, $upsample = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $upsample = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 127]($8,1,1024)|0);
 $10 = $upsample;
 $11 = (($10) + 16|0);
 HEAP32[$11>>2] = $9;
 $12 = $0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 $17 = (FUNCTION_TABLE_iiii[$15 & 127]($16,1,1024)|0);
 $18 = $upsample;
 $19 = (($18) + 20|0);
 HEAP32[$19>>2] = $17;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = (FUNCTION_TABLE_iiii[$23 & 127]($24,1,1024)|0);
 $26 = $upsample;
 $27 = (($26) + 24|0);
 HEAP32[$27>>2] = $25;
 $28 = $0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = $0;
 $33 = (FUNCTION_TABLE_iiii[$31 & 127]($32,1,1024)|0);
 $34 = $upsample;
 $35 = (($34) + 28|0);
 HEAP32[$35>>2] = $33;
 $i = 0;
 $x = -128;
 while(1) {
  $36 = $i;
  $37 = ($36|0)<=(255);
  if (!($37)) {
   break;
  }
  $38 = $x;
  $39 = ($38*91881)|0;
  $40 = (($39) + 32768)|0;
  $41 = $40 >> 16;
  $42 = $i;
  $43 = $upsample;
  $44 = (($43) + 16|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  HEAP32[$46>>2] = $41;
  $47 = $x;
  $48 = ($47*116130)|0;
  $49 = (($48) + 32768)|0;
  $50 = $49 >> 16;
  $51 = $i;
  $52 = $upsample;
  $53 = (($52) + 20|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  HEAP32[$55>>2] = $50;
  $56 = $x;
  $57 = Math_imul(-46802, $56)|0;
  $58 = $i;
  $59 = $upsample;
  $60 = (($59) + 24|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + ($58<<2)|0);
  HEAP32[$62>>2] = $57;
  $63 = $x;
  $64 = Math_imul(-22554, $63)|0;
  $65 = (($64) + 32768)|0;
  $66 = $i;
  $67 = $upsample;
  $68 = (($67) + 28|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + ($66<<2)|0);
  HEAP32[$70>>2] = $65;
  $71 = $i;
  $72 = (($71) + 1)|0;
  $i = $72;
  $73 = $x;
  $74 = (($73) + 1)|0;
  $x = $74;
 }
 STACKTOP = sp;return;
}
function _jinit_phuff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $coef_bit_ptr = 0, $entropy = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,64)|0);
 $entropy = $6;
 $7 = $entropy;
 $8 = $0;
 $9 = (($8) + 444|0);
 HEAP32[$9>>2] = $7;
 $10 = $entropy;
 HEAP32[$10>>2] = 42;
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = ($11|0)<(4);
  if (!($12)) {
   break;
  }
  $13 = $i;
  $14 = $entropy;
  $15 = (($14) + 44|0);
  $16 = (($15) + ($13<<2)|0);
  HEAP32[$16>>2] = 0;
  $17 = $i;
  $18 = (($17) + 1)|0;
  $i = $18;
 }
 $19 = $0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = $0;
 $25 = (($24) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $26<<6;
 $28 = $27<<2;
 $29 = (FUNCTION_TABLE_iiii[$22 & 127]($23,1,$28)|0);
 $30 = $0;
 $31 = (($30) + 160|0);
 HEAP32[$31>>2] = $29;
 $32 = $0;
 $33 = (($32) + 160|0);
 $34 = HEAP32[$33>>2]|0;
 $coef_bit_ptr = $34;
 $ci = 0;
 while(1) {
  $35 = $ci;
  $36 = $0;
  $37 = (($36) + 36|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($35|0)<($38|0);
  if (!($39)) {
   break;
  }
  $i = 0;
  while(1) {
   $40 = $i;
   $41 = ($40|0)<(64);
   if (!($41)) {
    break;
   }
   $42 = $coef_bit_ptr;
   $43 = (($42) + 4|0);
   $coef_bit_ptr = $43;
   HEAP32[$42>>2] = -1;
   $44 = $i;
   $45 = (($44) + 1)|0;
   $i = $45;
  }
  $46 = $ci;
  $47 = (($46) + 1)|0;
  $ci = $47;
 }
 STACKTOP = sp;return;
}
function _start_pass_phuff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bad = 0, $ci = 0, $cindex = 0, $coef_bit_ptr = 0, $coefi = 0, $compptr = 0, $entropy = 0, $expected = 0, $is_DC_band = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 444|0);
 $3 = HEAP32[$2>>2]|0;
 $entropy = $3;
 $4 = $0;
 $5 = (($4) + 400|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0);
 $8 = $7&1;
 $is_DC_band = $8;
 $bad = 0;
 $9 = $is_DC_band;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $0;
  $12 = (($11) + 404|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0);
  if ($14) {
   $bad = 1;
  }
 } else {
  $15 = $0;
  $16 = (($15) + 400|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  $19 = (($18) + 404|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)>($20|0);
  if ($21) {
   label = 7;
  } else {
   $22 = $0;
   $23 = (($22) + 404|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)>=(64);
   if ($25) {
    label = 7;
   }
  }
  if ((label|0) == 7) {
   $bad = 1;
  }
  $26 = $0;
  $27 = (($26) + 328|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(1);
  if ($29) {
   $bad = 1;
  }
 }
 $30 = $0;
 $31 = (($30) + 408|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = $0;
  $35 = (($34) + 412|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $0;
  $38 = (($37) + 408|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) - 1)|0;
  $41 = ($36|0)!=($40|0);
  if ($41) {
   $bad = 1;
  }
 }
 $42 = $0;
 $43 = (($42) + 412|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)>(13);
 if ($45) {
  $bad = 1;
 }
 $46 = $bad;
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = $0;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 20|0);
  HEAP32[$50>>2] = 16;
  $51 = $0;
  $52 = (($51) + 400|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $0;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + 24|0);
  HEAP32[$56>>2] = $53;
  $57 = $0;
  $58 = (($57) + 404|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $0;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 24|0);
  $63 = (($62) + 4|0);
  HEAP32[$63>>2] = $59;
  $64 = $0;
  $65 = (($64) + 408|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $0;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + 24|0);
  $70 = (($69) + 8|0);
  HEAP32[$70>>2] = $66;
  $71 = $0;
  $72 = (($71) + 412|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 24|0);
  $77 = (($76) + 12|0);
  HEAP32[$77>>2] = $73;
  $78 = $0;
  $79 = HEAP32[$78>>2]|0;
  $80 = HEAP32[$79>>2]|0;
  $81 = $0;
  FUNCTION_TABLE_vi[$80 & 127]($81);
 }
 $ci = 0;
 while(1) {
  $82 = $ci;
  $83 = $0;
  $84 = (($83) + 328|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($82|0)<($85|0);
  if (!($86)) {
   break;
  }
  $87 = $ci;
  $88 = $0;
  $89 = (($88) + 332|0);
  $90 = (($89) + ($87<<2)|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $cindex = $93;
  $94 = $cindex;
  $95 = $0;
  $96 = (($95) + 160|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = (($97) + ($94<<8)|0);
  $coef_bit_ptr = $98;
  $99 = $is_DC_band;
  $100 = ($99|0)!=(0);
  if (!($100)) {
   $101 = $coef_bit_ptr;
   $102 = HEAP32[$101>>2]|0;
   $103 = ($102|0)<(0);
   if ($103) {
    $104 = $0;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($105) + 20|0);
    HEAP32[$106>>2] = 115;
    $107 = $cindex;
    $108 = $0;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 24|0);
    HEAP32[$110>>2] = $107;
    $111 = $0;
    $112 = HEAP32[$111>>2]|0;
    $113 = (($112) + 24|0);
    $114 = (($113) + 4|0);
    HEAP32[$114>>2] = 0;
    $115 = $0;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($116) + 4|0);
    $118 = HEAP32[$117>>2]|0;
    $119 = $0;
    FUNCTION_TABLE_vii[$118 & 127]($119,-1);
   }
  }
  $120 = $0;
  $121 = (($120) + 400|0);
  $122 = HEAP32[$121>>2]|0;
  $coefi = $122;
  while(1) {
   $123 = $coefi;
   $124 = $0;
   $125 = (($124) + 404|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = ($123|0)<=($126|0);
   if (!($127)) {
    break;
   }
   $128 = $coefi;
   $129 = $coef_bit_ptr;
   $130 = (($129) + ($128<<2)|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = ($131|0)<(0);
   if ($132) {
    $137 = 0;
   } else {
    $133 = $coefi;
    $134 = $coef_bit_ptr;
    $135 = (($134) + ($133<<2)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = $136;
   }
   $expected = $137;
   $138 = $0;
   $139 = (($138) + 408|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = $expected;
   $142 = ($140|0)!=($141|0);
   if ($142) {
    $143 = $0;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($144) + 20|0);
    HEAP32[$145>>2] = 115;
    $146 = $cindex;
    $147 = $0;
    $148 = HEAP32[$147>>2]|0;
    $149 = (($148) + 24|0);
    HEAP32[$149>>2] = $146;
    $150 = $coefi;
    $151 = $0;
    $152 = HEAP32[$151>>2]|0;
    $153 = (($152) + 24|0);
    $154 = (($153) + 4|0);
    HEAP32[$154>>2] = $150;
    $155 = $0;
    $156 = HEAP32[$155>>2]|0;
    $157 = (($156) + 4|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = $0;
    FUNCTION_TABLE_vii[$158 & 127]($159,-1);
   }
   $160 = $0;
   $161 = (($160) + 412|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = $coefi;
   $164 = $coef_bit_ptr;
   $165 = (($164) + ($163<<2)|0);
   HEAP32[$165>>2] = $162;
   $166 = $coefi;
   $167 = (($166) + 1)|0;
   $coefi = $167;
  }
  $168 = $ci;
  $169 = (($168) + 1)|0;
  $ci = $169;
 }
 $170 = $0;
 $171 = (($170) + 408|0);
 $172 = HEAP32[$171>>2]|0;
 $173 = ($172|0)==(0);
 if ($173) {
  $174 = $is_DC_band;
  $175 = ($174|0)!=(0);
  if ($175) {
   $176 = $entropy;
   $177 = (($176) + 4|0);
   HEAP32[$177>>2] = 43;
  } else {
   $178 = $entropy;
   $179 = (($178) + 4|0);
   HEAP32[$179>>2] = 44;
  }
 } else {
  $180 = $is_DC_band;
  $181 = ($180|0)!=(0);
  if ($181) {
   $182 = $entropy;
   $183 = (($182) + 4|0);
   HEAP32[$183>>2] = 45;
  } else {
   $184 = $entropy;
   $185 = (($184) + 4|0);
   HEAP32[$185>>2] = 46;
  }
 }
 $ci = 0;
 while(1) {
  $186 = $ci;
  $187 = $0;
  $188 = (($187) + 328|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = ($186|0)<($189|0);
  if (!($190)) {
   break;
  }
  $191 = $ci;
  $192 = $0;
  $193 = (($192) + 332|0);
  $194 = (($193) + ($191<<2)|0);
  $195 = HEAP32[$194>>2]|0;
  $compptr = $195;
  $196 = $is_DC_band;
  $197 = ($196|0)!=(0);
  if ($197) {
   $198 = $0;
   $199 = (($198) + 408|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = ($200|0)==(0);
   if ($201) {
    $202 = $compptr;
    $203 = (($202) + 20|0);
    $204 = HEAP32[$203>>2]|0;
    $tbl = $204;
    $205 = $0;
    $206 = $tbl;
    $207 = $tbl;
    $208 = $entropy;
    $209 = (($208) + 44|0);
    $210 = (($209) + ($207<<2)|0);
    _jpeg_make_d_derived_tbl($205,1,$206,$210);
   }
  } else {
   $211 = $compptr;
   $212 = (($211) + 24|0);
   $213 = HEAP32[$212>>2]|0;
   $tbl = $213;
   $214 = $0;
   $215 = $tbl;
   $216 = $tbl;
   $217 = $entropy;
   $218 = (($217) + 44|0);
   $219 = (($218) + ($216<<2)|0);
   _jpeg_make_d_derived_tbl($214,0,$215,$219);
   $220 = $tbl;
   $221 = $entropy;
   $222 = (($221) + 44|0);
   $223 = (($222) + ($220<<2)|0);
   $224 = HEAP32[$223>>2]|0;
   $225 = $entropy;
   $226 = (($225) + 60|0);
   HEAP32[$226>>2] = $224;
  }
  $227 = $ci;
  $228 = $entropy;
  $229 = (($228) + 20|0);
  $230 = (($229) + 4|0);
  $231 = (($230) + ($227<<2)|0);
  HEAP32[$231>>2] = 0;
  $232 = $ci;
  $233 = (($232) + 1)|0;
  $ci = $233;
 }
 $234 = $entropy;
 $235 = (($234) + 12|0);
 $236 = (($235) + 4|0);
 HEAP32[$236>>2] = 0;
 $237 = $entropy;
 $238 = (($237) + 12|0);
 HEAP32[$238>>2] = 0;
 $239 = $entropy;
 $240 = (($239) + 8|0);
 HEAP32[$240>>2] = 0;
 $241 = $entropy;
 $242 = (($241) + 20|0);
 HEAP32[$242>>2] = 0;
 $243 = $0;
 $244 = (($243) + 276|0);
 $245 = HEAP32[$244>>2]|0;
 $246 = $entropy;
 $247 = (($246) + 40|0);
 HEAP32[$247>>2] = $245;
 STACKTOP = sp;return;
}
function _decode_mcu_DC_first($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Al = 0, $bits_left = 0, $blkn = 0, $block = 0, $br_state = 0, $ci = 0, $compptr = 0, $entropy = 0, $get_buffer = 0, $look = 0, $nb = 0;
 var $r = 0, $s = 0, $state = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $br_state = sp + 36|0;
 $state = sp + 16|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 412|0);
 $8 = HEAP32[$7>>2]|0;
 $Al = $8;
 $9 = $1;
 $10 = (($9) + 276|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $entropy;
  $14 = (($13) + 40|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)==(0);
  do {
   if ($16) {
    $17 = $1;
    $18 = (_process_restart($17)|0);
    $19 = ($18|0)!=(0);
    if ($19) {
     break;
    }
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
  } while(0);
 }
 $20 = $entropy;
 $21 = (($20) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0);
 do {
  if (!($23)) {
   $24 = $1;
   $25 = (($br_state) + 16|0);
   HEAP32[$25>>2] = $24;
   $26 = $1;
   $27 = (($26) + 24|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$br_state>>2] = $29;
   $30 = $1;
   $31 = (($30) + 24|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($br_state) + 4|0);
   HEAP32[$35>>2] = $34;
   $36 = $entropy;
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $get_buffer = $38;
   $39 = $entropy;
   $40 = (($39) + 12|0);
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $bits_left = $42;
   $43 = $entropy;
   $44 = (($43) + 20|0);
   ;HEAP32[$state+0>>2]=HEAP32[$44+0>>2]|0;HEAP32[$state+4>>2]=HEAP32[$44+4>>2]|0;HEAP32[$state+8>>2]=HEAP32[$44+8>>2]|0;HEAP32[$state+12>>2]=HEAP32[$44+12>>2]|0;HEAP32[$state+16>>2]=HEAP32[$44+16>>2]|0;
   $blkn = 0;
   L13: while(1) {
    $45 = $blkn;
    $46 = $1;
    $47 = (($46) + 356|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = ($45|0)<($48|0);
    if (!($49)) {
     label = 33;
     break;
    }
    $50 = $blkn;
    $51 = $2;
    $52 = (($51) + ($50<<2)|0);
    $53 = HEAP32[$52>>2]|0;
    $block = $53;
    $54 = $blkn;
    $55 = $1;
    $56 = (($55) + 360|0);
    $57 = (($56) + ($54<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $ci = $58;
    $59 = $ci;
    $60 = $1;
    $61 = (($60) + 332|0);
    $62 = (($61) + ($59<<2)|0);
    $63 = HEAP32[$62>>2]|0;
    $compptr = $63;
    $64 = $compptr;
    $65 = (($64) + 20|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = $entropy;
    $68 = (($67) + 44|0);
    $69 = (($68) + ($66<<2)|0);
    $70 = HEAP32[$69>>2]|0;
    $tbl = $70;
    $71 = $bits_left;
    $72 = ($71|0)<(8);
    do {
     if ($72) {
      $73 = $get_buffer;
      $74 = $bits_left;
      $75 = (_jpeg_fill_bit_buffer($br_state,$73,$74,0)|0);
      $76 = ($75|0)!=(0);
      if (!($76)) {
       label = 12;
       break L13;
      }
      $77 = (($br_state) + 8|0);
      $78 = HEAP32[$77>>2]|0;
      $get_buffer = $78;
      $79 = (($br_state) + 12|0);
      $80 = HEAP32[$79>>2]|0;
      $bits_left = $80;
      $81 = $bits_left;
      $82 = ($81|0)<(8);
      if ($82) {
       $nb = 1;
       label = 19;
       break;
      } else {
       label = 16;
       break;
      }
     } else {
      label = 16;
     }
    } while(0);
    do {
     if ((label|0) == 16) {
      label = 0;
      $83 = $get_buffer;
      $84 = $bits_left;
      $85 = (($84) - 8)|0;
      $86 = $83 >> $85;
      $87 = $86 & 255;
      $look = $87;
      $88 = $look;
      $89 = $tbl;
      $90 = (($89) + 144|0);
      $91 = (($90) + ($88<<2)|0);
      $92 = HEAP32[$91>>2]|0;
      $nb = $92;
      $93 = ($92|0)!=(0);
      if ($93) {
       $94 = $nb;
       $95 = $bits_left;
       $96 = (($95) - ($94))|0;
       $bits_left = $96;
       $97 = $look;
       $98 = $tbl;
       $99 = (($98) + 1168|0);
       $100 = (($99) + ($97)|0);
       $101 = HEAP8[$100>>0]|0;
       $102 = $101&255;
       $s = $102;
       break;
      } else {
       $nb = 9;
       label = 19;
       break;
      }
     }
    } while(0);
    if ((label|0) == 19) {
     label = 0;
     $103 = $get_buffer;
     $104 = $bits_left;
     $105 = $tbl;
     $106 = $nb;
     $107 = (_jpeg_huff_decode($br_state,$103,$104,$105,$106)|0);
     $s = $107;
     $108 = ($107|0)<(0);
     if ($108) {
      label = 20;
      break;
     }
     $109 = (($br_state) + 8|0);
     $110 = HEAP32[$109>>2]|0;
     $get_buffer = $110;
     $111 = (($br_state) + 12|0);
     $112 = HEAP32[$111>>2]|0;
     $bits_left = $112;
    }
    $113 = $s;
    $114 = ($113|0)!=(0);
    if ($114) {
     $115 = $bits_left;
     $116 = $s;
     $117 = ($115|0)<($116|0);
     if ($117) {
      $118 = $get_buffer;
      $119 = $bits_left;
      $120 = $s;
      $121 = (_jpeg_fill_bit_buffer($br_state,$118,$119,$120)|0);
      $122 = ($121|0)!=(0);
      if (!($122)) {
       label = 25;
       break;
      }
      $123 = (($br_state) + 8|0);
      $124 = HEAP32[$123>>2]|0;
      $get_buffer = $124;
      $125 = (($br_state) + 12|0);
      $126 = HEAP32[$125>>2]|0;
      $bits_left = $126;
     }
     $127 = $get_buffer;
     $128 = $s;
     $129 = $bits_left;
     $130 = (($129) - ($128))|0;
     $bits_left = $130;
     $131 = $127 >> $130;
     $132 = $s;
     $133 = 1 << $132;
     $134 = (($133) - 1)|0;
     $135 = $131 & $134;
     $r = $135;
     $136 = $r;
     $137 = $s;
     $138 = (3584 + ($137<<2)|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($136|0)<($139|0);
     if ($140) {
      $141 = $r;
      $142 = $s;
      $143 = (3648 + ($142<<2)|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = (($141) + ($144))|0;
      $147 = $145;
     } else {
      $146 = $r;
      $147 = $146;
     }
     $s = $147;
    }
    $148 = $ci;
    $149 = (($state) + 4|0);
    $150 = (($149) + ($148<<2)|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = $s;
    $153 = (($152) + ($151))|0;
    $s = $153;
    $154 = $s;
    $155 = $ci;
    $156 = (($state) + 4|0);
    $157 = (($156) + ($155<<2)|0);
    HEAP32[$157>>2] = $154;
    $158 = $s;
    $159 = $Al;
    $160 = $158 << $159;
    $161 = $160&65535;
    $162 = $block;
    HEAP16[$162>>1] = $161;
    $163 = $blkn;
    $164 = (($163) + 1)|0;
    $blkn = $164;
   }
   if ((label|0) == 12) {
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
   else if ((label|0) == 20) {
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
   else if ((label|0) == 25) {
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
   else if ((label|0) == 33) {
    $165 = HEAP32[$br_state>>2]|0;
    $166 = $1;
    $167 = (($166) + 24|0);
    $168 = HEAP32[$167>>2]|0;
    HEAP32[$168>>2] = $165;
    $169 = (($br_state) + 4|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = $1;
    $172 = (($171) + 24|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = (($173) + 4|0);
    HEAP32[$174>>2] = $170;
    $175 = $get_buffer;
    $176 = $entropy;
    $177 = (($176) + 12|0);
    HEAP32[$177>>2] = $175;
    $178 = $bits_left;
    $179 = $entropy;
    $180 = (($179) + 12|0);
    $181 = (($180) + 4|0);
    HEAP32[$181>>2] = $178;
    $182 = $entropy;
    $183 = (($182) + 20|0);
    ;HEAP32[$183+0>>2]=HEAP32[$state+0>>2]|0;HEAP32[$183+4>>2]=HEAP32[$state+4>>2]|0;HEAP32[$183+8>>2]=HEAP32[$state+8>>2]|0;HEAP32[$183+12>>2]=HEAP32[$state+12>>2]|0;HEAP32[$183+16>>2]=HEAP32[$state+16>>2]|0;
    break;
   }
  }
 } while(0);
 $184 = $entropy;
 $185 = (($184) + 40|0);
 $186 = HEAP32[$185>>2]|0;
 $187 = (($186) + -1)|0;
 HEAP32[$185>>2] = $187;
 $0 = 1;
 $188 = $0;
 STACKTOP = sp;return ($188|0);
}
function _decode_mcu_AC_first($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Al = 0, $EOBRUN = 0;
 var $Se = 0, $bits_left = 0, $block = 0, $br_state = 0, $entropy = 0, $get_buffer = 0, $k = 0, $look = 0, $nb = 0, $r = 0, $s = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $br_state = sp + 12|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 404|0);
 $8 = HEAP32[$7>>2]|0;
 $Se = $8;
 $9 = $1;
 $10 = (($9) + 412|0);
 $11 = HEAP32[$10>>2]|0;
 $Al = $11;
 $12 = $1;
 $13 = (($12) + 276|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $entropy;
  $17 = (($16) + 40|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)==(0);
  do {
   if ($19) {
    $20 = $1;
    $21 = (_process_restart($20)|0);
    $22 = ($21|0)!=(0);
    if ($22) {
     break;
    }
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
  } while(0);
 }
 $23 = $entropy;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $27 = $entropy;
  $28 = (($27) + 20|0);
  $29 = HEAP32[$28>>2]|0;
  $EOBRUN = $29;
  $30 = $EOBRUN;
  $31 = ($30>>>0)>(0);
  if ($31) {
   $32 = $EOBRUN;
   $33 = (($32) + -1)|0;
   $EOBRUN = $33;
  } else {
   $34 = $1;
   $35 = (($br_state) + 16|0);
   HEAP32[$35>>2] = $34;
   $36 = $1;
   $37 = (($36) + 24|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   HEAP32[$br_state>>2] = $39;
   $40 = $1;
   $41 = (($40) + 24|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($br_state) + 4|0);
   HEAP32[$45>>2] = $44;
   $46 = $entropy;
   $47 = (($46) + 12|0);
   $48 = HEAP32[$47>>2]|0;
   $get_buffer = $48;
   $49 = $entropy;
   $50 = (($49) + 12|0);
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $bits_left = $52;
   $53 = $2;
   $54 = HEAP32[$53>>2]|0;
   $block = $54;
   $55 = $entropy;
   $56 = (($55) + 60|0);
   $57 = HEAP32[$56>>2]|0;
   $tbl = $57;
   $58 = $1;
   $59 = (($58) + 400|0);
   $60 = HEAP32[$59>>2]|0;
   $k = $60;
   L16: while(1) {
    $61 = $k;
    $62 = $Se;
    $63 = ($61|0)<=($62|0);
    if (!($63)) {
     break;
    }
    $64 = $bits_left;
    $65 = ($64|0)<(8);
    do {
     if ($65) {
      $66 = $get_buffer;
      $67 = $bits_left;
      $68 = (_jpeg_fill_bit_buffer($br_state,$66,$67,0)|0);
      $69 = ($68|0)!=(0);
      if (!($69)) {
       label = 14;
       break L16;
      }
      $70 = (($br_state) + 8|0);
      $71 = HEAP32[$70>>2]|0;
      $get_buffer = $71;
      $72 = (($br_state) + 12|0);
      $73 = HEAP32[$72>>2]|0;
      $bits_left = $73;
      $74 = $bits_left;
      $75 = ($74|0)<(8);
      if ($75) {
       $nb = 1;
       label = 21;
       break;
      } else {
       label = 18;
       break;
      }
     } else {
      label = 18;
     }
    } while(0);
    do {
     if ((label|0) == 18) {
      label = 0;
      $76 = $get_buffer;
      $77 = $bits_left;
      $78 = (($77) - 8)|0;
      $79 = $76 >> $78;
      $80 = $79 & 255;
      $look = $80;
      $81 = $look;
      $82 = $tbl;
      $83 = (($82) + 144|0);
      $84 = (($83) + ($81<<2)|0);
      $85 = HEAP32[$84>>2]|0;
      $nb = $85;
      $86 = ($85|0)!=(0);
      if ($86) {
       $87 = $nb;
       $88 = $bits_left;
       $89 = (($88) - ($87))|0;
       $bits_left = $89;
       $90 = $look;
       $91 = $tbl;
       $92 = (($91) + 1168|0);
       $93 = (($92) + ($90)|0);
       $94 = HEAP8[$93>>0]|0;
       $95 = $94&255;
       $s = $95;
       break;
      } else {
       $nb = 9;
       label = 21;
       break;
      }
     }
    } while(0);
    if ((label|0) == 21) {
     label = 0;
     $96 = $get_buffer;
     $97 = $bits_left;
     $98 = $tbl;
     $99 = $nb;
     $100 = (_jpeg_huff_decode($br_state,$96,$97,$98,$99)|0);
     $s = $100;
     $101 = ($100|0)<(0);
     if ($101) {
      label = 22;
      break;
     }
     $102 = (($br_state) + 8|0);
     $103 = HEAP32[$102>>2]|0;
     $get_buffer = $103;
     $104 = (($br_state) + 12|0);
     $105 = HEAP32[$104>>2]|0;
     $bits_left = $105;
    }
    $106 = $s;
    $107 = $106 >> 4;
    $r = $107;
    $108 = $s;
    $109 = $108 & 15;
    $s = $109;
    $110 = $s;
    $111 = ($110|0)!=(0);
    if ($111) {
     $112 = $r;
     $113 = $k;
     $114 = (($113) + ($112))|0;
     $k = $114;
     $115 = $bits_left;
     $116 = $s;
     $117 = ($115|0)<($116|0);
     if ($117) {
      $118 = $get_buffer;
      $119 = $bits_left;
      $120 = $s;
      $121 = (_jpeg_fill_bit_buffer($br_state,$118,$119,$120)|0);
      $122 = ($121|0)!=(0);
      if (!($122)) {
       label = 27;
       break;
      }
      $123 = (($br_state) + 8|0);
      $124 = HEAP32[$123>>2]|0;
      $get_buffer = $124;
      $125 = (($br_state) + 12|0);
      $126 = HEAP32[$125>>2]|0;
      $bits_left = $126;
     }
     $127 = $get_buffer;
     $128 = $s;
     $129 = $bits_left;
     $130 = (($129) - ($128))|0;
     $bits_left = $130;
     $131 = $127 >> $130;
     $132 = $s;
     $133 = 1 << $132;
     $134 = (($133) - 1)|0;
     $135 = $131 & $134;
     $r = $135;
     $136 = $r;
     $137 = $s;
     $138 = (3584 + ($137<<2)|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($136|0)<($139|0);
     if ($140) {
      $141 = $r;
      $142 = $s;
      $143 = (3648 + ($142<<2)|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = (($141) + ($144))|0;
      $147 = $145;
     } else {
      $146 = $r;
      $147 = $146;
     }
     $s = $147;
     $148 = $s;
     $149 = $Al;
     $150 = $148 << $149;
     $151 = $150&65535;
     $152 = $k;
     $153 = (9504 + ($152<<2)|0);
     $154 = HEAPU8[$153>>0]|(HEAPU8[$153+1>>0]<<8)|(HEAPU8[$153+2>>0]<<16)|(HEAPU8[$153+3>>0]<<24);
     $155 = $block;
     $156 = (($155) + ($154<<1)|0);
     HEAP16[$156>>1] = $151;
    } else {
     $157 = $r;
     $158 = ($157|0)==(15);
     if (!($158)) {
      label = 35;
      break;
     }
     $159 = $k;
     $160 = (($159) + 15)|0;
     $k = $160;
    }
    $191 = $k;
    $192 = (($191) + 1)|0;
    $k = $192;
   }
   if ((label|0) == 14) {
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
   else if ((label|0) == 22) {
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
   else if ((label|0) == 27) {
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
   else if ((label|0) == 35) {
    $161 = $r;
    $162 = 1 << $161;
    $EOBRUN = $162;
    $163 = $r;
    $164 = ($163|0)!=(0);
    if ($164) {
     $165 = $bits_left;
     $166 = $r;
     $167 = ($165|0)<($166|0);
     do {
      if ($167) {
       $168 = $get_buffer;
       $169 = $bits_left;
       $170 = $r;
       $171 = (_jpeg_fill_bit_buffer($br_state,$168,$169,$170)|0);
       $172 = ($171|0)!=(0);
       if ($172) {
        $173 = (($br_state) + 8|0);
        $174 = HEAP32[$173>>2]|0;
        $get_buffer = $174;
        $175 = (($br_state) + 12|0);
        $176 = HEAP32[$175>>2]|0;
        $bits_left = $176;
        break;
       }
       $0 = 0;
       $217 = $0;
       STACKTOP = sp;return ($217|0);
      }
     } while(0);
     $177 = $get_buffer;
     $178 = $r;
     $179 = $bits_left;
     $180 = (($179) - ($178))|0;
     $bits_left = $180;
     $181 = $177 >> $180;
     $182 = $r;
     $183 = 1 << $182;
     $184 = (($183) - 1)|0;
     $185 = $181 & $184;
     $r = $185;
     $186 = $r;
     $187 = $EOBRUN;
     $188 = (($187) + ($186))|0;
     $EOBRUN = $188;
    }
    $189 = $EOBRUN;
    $190 = (($189) + -1)|0;
    $EOBRUN = $190;
   }
   $193 = HEAP32[$br_state>>2]|0;
   $194 = $1;
   $195 = (($194) + 24|0);
   $196 = HEAP32[$195>>2]|0;
   HEAP32[$196>>2] = $193;
   $197 = (($br_state) + 4|0);
   $198 = HEAP32[$197>>2]|0;
   $199 = $1;
   $200 = (($199) + 24|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = (($201) + 4|0);
   HEAP32[$202>>2] = $198;
   $203 = $get_buffer;
   $204 = $entropy;
   $205 = (($204) + 12|0);
   HEAP32[$205>>2] = $203;
   $206 = $bits_left;
   $207 = $entropy;
   $208 = (($207) + 12|0);
   $209 = (($208) + 4|0);
   HEAP32[$209>>2] = $206;
  }
  $210 = $EOBRUN;
  $211 = $entropy;
  $212 = (($211) + 20|0);
  HEAP32[$212>>2] = $210;
 }
 $213 = $entropy;
 $214 = (($213) + 40|0);
 $215 = HEAP32[$214>>2]|0;
 $216 = (($215) + -1)|0;
 HEAP32[$214>>2] = $216;
 $0 = 1;
 $217 = $0;
 STACKTOP = sp;return ($217|0);
}
function _decode_mcu_DC_refine($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $bits_left = 0, $blkn = 0, $block = 0, $br_state = 0, $entropy = 0;
 var $get_buffer = 0, $p1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $br_state = sp;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 412|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = 1 << $8;
 $p1 = $9;
 $10 = $1;
 $11 = (($10) + 276|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $entropy;
  $15 = (($14) + 40|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(0);
  do {
   if ($17) {
    $18 = $1;
    $19 = (_process_restart($18)|0);
    $20 = ($19|0)!=(0);
    if ($20) {
     break;
    }
    $0 = 0;
    $94 = $0;
    STACKTOP = sp;return ($94|0);
   }
  } while(0);
 }
 $21 = $1;
 $22 = (($br_state) + 16|0);
 HEAP32[$22>>2] = $21;
 $23 = $1;
 $24 = (($23) + 24|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$br_state>>2] = $26;
 $27 = $1;
 $28 = (($27) + 24|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($br_state) + 4|0);
 HEAP32[$32>>2] = $31;
 $33 = $entropy;
 $34 = (($33) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $get_buffer = $35;
 $36 = $entropy;
 $37 = (($36) + 12|0);
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $bits_left = $39;
 $blkn = 0;
 while(1) {
  $40 = $blkn;
  $41 = $1;
  $42 = (($41) + 356|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   label = 17;
   break;
  }
  $45 = $blkn;
  $46 = $2;
  $47 = (($46) + ($45<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $block = $48;
  $49 = $bits_left;
  $50 = ($49|0)<(1);
  if ($50) {
   $51 = $get_buffer;
   $52 = $bits_left;
   $53 = (_jpeg_fill_bit_buffer($br_state,$51,$52,1)|0);
   $54 = ($53|0)!=(0);
   if (!($54)) {
    label = 11;
    break;
   }
   $55 = (($br_state) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $get_buffer = $56;
   $57 = (($br_state) + 12|0);
   $58 = HEAP32[$57>>2]|0;
   $bits_left = $58;
  }
  $59 = $get_buffer;
  $60 = $bits_left;
  $61 = (($60) - 1)|0;
  $bits_left = $61;
  $62 = $59 >> $61;
  $63 = $62 & 1;
  $64 = ($63|0)!=(0);
  if ($64) {
   $65 = $p1;
   $66 = $block;
   $67 = HEAP16[$66>>1]|0;
   $68 = $67 << 16 >> 16;
   $69 = $68 | $65;
   $70 = $69&65535;
   HEAP16[$66>>1] = $70;
  }
  $71 = $blkn;
  $72 = (($71) + 1)|0;
  $blkn = $72;
 }
 if ((label|0) == 11) {
  $0 = 0;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 else if ((label|0) == 17) {
  $73 = HEAP32[$br_state>>2]|0;
  $74 = $1;
  $75 = (($74) + 24|0);
  $76 = HEAP32[$75>>2]|0;
  HEAP32[$76>>2] = $73;
  $77 = (($br_state) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = $1;
  $80 = (($79) + 24|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 4|0);
  HEAP32[$82>>2] = $78;
  $83 = $get_buffer;
  $84 = $entropy;
  $85 = (($84) + 12|0);
  HEAP32[$85>>2] = $83;
  $86 = $bits_left;
  $87 = $entropy;
  $88 = (($87) + 12|0);
  $89 = (($88) + 4|0);
  HEAP32[$89>>2] = $86;
  $90 = $entropy;
  $91 = (($90) + 40|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + -1)|0;
  HEAP32[$91>>2] = $93;
  $0 = 1;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 return 0|0;
}
function _decode_mcu_AC_refine($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $EOBRUN = 0, $Se = 0, $bits_left = 0, $block = 0, $br_state = 0, $entropy = 0, $get_buffer = 0;
 var $k = 0, $look = 0, $m1 = 0, $nb = 0, $newnz_pos = 0, $num_newnz = 0, $p1 = 0, $pos = 0, $r = 0, $s = 0, $tbl = 0, $thiscoef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $br_state = sp + 280|0;
 $newnz_pos = sp + 16|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 404|0);
 $8 = HEAP32[$7>>2]|0;
 $Se = $8;
 $9 = $1;
 $10 = (($9) + 412|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = 1 << $11;
 $p1 = $12;
 $13 = $1;
 $14 = (($13) + 412|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = -1 << $15;
 $m1 = $16;
 $17 = $1;
 $18 = (($17) + 276|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $entropy;
  $22 = (($21) + 40|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)==(0);
  do {
   if ($24) {
    $25 = $1;
    $26 = (_process_restart($25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     break;
    }
    $0 = 0;
    $332 = $0;
    STACKTOP = sp;return ($332|0);
   }
  } while(0);
 }
 $28 = $entropy;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0);
 L11: do {
  if (!($31)) {
   $32 = $1;
   $33 = (($br_state) + 16|0);
   HEAP32[$33>>2] = $32;
   $34 = $1;
   $35 = (($34) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$br_state>>2] = $37;
   $38 = $1;
   $39 = (($38) + 24|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($br_state) + 4|0);
   HEAP32[$43>>2] = $42;
   $44 = $entropy;
   $45 = (($44) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $get_buffer = $46;
   $47 = $entropy;
   $48 = (($47) + 12|0);
   $49 = (($48) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   $bits_left = $50;
   $51 = $entropy;
   $52 = (($51) + 20|0);
   $53 = HEAP32[$52>>2]|0;
   $EOBRUN = $53;
   $54 = $2;
   $55 = HEAP32[$54>>2]|0;
   $block = $55;
   $56 = $entropy;
   $57 = (($56) + 60|0);
   $58 = HEAP32[$57>>2]|0;
   $tbl = $58;
   $num_newnz = 0;
   $59 = $1;
   $60 = (($59) + 400|0);
   $61 = HEAP32[$60>>2]|0;
   $k = $61;
   $62 = $EOBRUN;
   $63 = ($62|0)==(0);
   L13: do {
    if ($63) {
     L15: while(1) {
      $64 = $k;
      $65 = $Se;
      $66 = ($64|0)<=($65|0);
      if (!($66)) {
       break;
      }
      $67 = $bits_left;
      $68 = ($67|0)<(8);
      do {
       if ($68) {
        $69 = $get_buffer;
        $70 = $bits_left;
        $71 = (_jpeg_fill_bit_buffer($br_state,$69,$70,0)|0);
        $72 = ($71|0)!=(0);
        if (!($72)) {
         label = 13;
         break L15;
        }
        $73 = (($br_state) + 8|0);
        $74 = HEAP32[$73>>2]|0;
        $get_buffer = $74;
        $75 = (($br_state) + 12|0);
        $76 = HEAP32[$75>>2]|0;
        $bits_left = $76;
        $77 = $bits_left;
        $78 = ($77|0)<(8);
        if ($78) {
         $nb = 1;
         label = 20;
         break;
        } else {
         label = 17;
         break;
        }
       } else {
        label = 17;
       }
      } while(0);
      do {
       if ((label|0) == 17) {
        label = 0;
        $79 = $get_buffer;
        $80 = $bits_left;
        $81 = (($80) - 8)|0;
        $82 = $79 >> $81;
        $83 = $82 & 255;
        $look = $83;
        $84 = $look;
        $85 = $tbl;
        $86 = (($85) + 144|0);
        $87 = (($86) + ($84<<2)|0);
        $88 = HEAP32[$87>>2]|0;
        $nb = $88;
        $89 = ($88|0)!=(0);
        if ($89) {
         $90 = $nb;
         $91 = $bits_left;
         $92 = (($91) - ($90))|0;
         $bits_left = $92;
         $93 = $look;
         $94 = $tbl;
         $95 = (($94) + 1168|0);
         $96 = (($95) + ($93)|0);
         $97 = HEAP8[$96>>0]|0;
         $98 = $97&255;
         $s = $98;
         break;
        } else {
         $nb = 9;
         label = 20;
         break;
        }
       }
      } while(0);
      if ((label|0) == 20) {
       label = 0;
       $99 = $get_buffer;
       $100 = $bits_left;
       $101 = $tbl;
       $102 = $nb;
       $103 = (_jpeg_huff_decode($br_state,$99,$100,$101,$102)|0);
       $s = $103;
       $104 = ($103|0)<(0);
       if ($104) {
        label = 21;
        break;
       }
       $105 = (($br_state) + 8|0);
       $106 = HEAP32[$105>>2]|0;
       $get_buffer = $106;
       $107 = (($br_state) + 12|0);
       $108 = HEAP32[$107>>2]|0;
       $bits_left = $108;
      }
      $109 = $s;
      $110 = $109 >> 4;
      $r = $110;
      $111 = $s;
      $112 = $111 & 15;
      $s = $112;
      $113 = $s;
      $114 = ($113|0)!=(0);
      if ($114) {
       $115 = $s;
       $116 = ($115|0)!=(1);
       if ($116) {
        $117 = $1;
        $118 = HEAP32[$117>>2]|0;
        $119 = (($118) + 20|0);
        HEAP32[$119>>2] = 118;
        $120 = $1;
        $121 = HEAP32[$120>>2]|0;
        $122 = (($121) + 4|0);
        $123 = HEAP32[$122>>2]|0;
        $124 = $1;
        FUNCTION_TABLE_vii[$123 & 127]($124,-1);
       }
       $125 = $bits_left;
       $126 = ($125|0)<(1);
       if ($126) {
        $127 = $get_buffer;
        $128 = $bits_left;
        $129 = (_jpeg_fill_bit_buffer($br_state,$127,$128,1)|0);
        $130 = ($129|0)!=(0);
        if (!($130)) {
         label = 28;
         break;
        }
        $131 = (($br_state) + 8|0);
        $132 = HEAP32[$131>>2]|0;
        $get_buffer = $132;
        $133 = (($br_state) + 12|0);
        $134 = HEAP32[$133>>2]|0;
        $bits_left = $134;
       }
       $135 = $get_buffer;
       $136 = $bits_left;
       $137 = (($136) - 1)|0;
       $bits_left = $137;
       $138 = $135 >> $137;
       $139 = $138 & 1;
       $140 = ($139|0)!=(0);
       if ($140) {
        $141 = $p1;
        $s = $141;
       } else {
        $142 = $m1;
        $s = $142;
       }
      } else {
       $143 = $r;
       $144 = ($143|0)!=(15);
       if ($144) {
        label = 35;
        break;
       }
      }
      while(1) {
       $173 = $block;
       $174 = $k;
       $175 = (9504 + ($174<<2)|0);
       $176 = HEAPU8[$175>>0]|(HEAPU8[$175+1>>0]<<8)|(HEAPU8[$175+2>>0]<<16)|(HEAPU8[$175+3>>0]<<24);
       $177 = (($173) + ($176<<1)|0);
       $thiscoef = $177;
       $178 = $thiscoef;
       $179 = HEAP16[$178>>1]|0;
       $180 = $179 << 16 >> 16;
       $181 = ($180|0)!=(0);
       if ($181) {
        $182 = $bits_left;
        $183 = ($182|0)<(1);
        if ($183) {
         $184 = $get_buffer;
         $185 = $bits_left;
         $186 = (_jpeg_fill_bit_buffer($br_state,$184,$185,1)|0);
         $187 = ($186|0)!=(0);
         if (!($187)) {
          label = 47;
          break L15;
         }
         $188 = (($br_state) + 8|0);
         $189 = HEAP32[$188>>2]|0;
         $get_buffer = $189;
         $190 = (($br_state) + 12|0);
         $191 = HEAP32[$190>>2]|0;
         $bits_left = $191;
        }
        $192 = $get_buffer;
        $193 = $bits_left;
        $194 = (($193) - 1)|0;
        $bits_left = $194;
        $195 = $192 >> $194;
        $196 = $195 & 1;
        $197 = ($196|0)!=(0);
        if ($197) {
         $198 = $thiscoef;
         $199 = HEAP16[$198>>1]|0;
         $200 = $199 << 16 >> 16;
         $201 = $p1;
         $202 = $200 & $201;
         $203 = ($202|0)==(0);
         if ($203) {
          $204 = $thiscoef;
          $205 = HEAP16[$204>>1]|0;
          $206 = $205 << 16 >> 16;
          $207 = ($206|0)>=(0);
          if ($207) {
           $208 = $p1;
           $209 = $thiscoef;
           $210 = HEAP16[$209>>1]|0;
           $211 = $210 << 16 >> 16;
           $212 = (($211) + ($208))|0;
           $213 = $212&65535;
           HEAP16[$209>>1] = $213;
          } else {
           $214 = $m1;
           $215 = $thiscoef;
           $216 = HEAP16[$215>>1]|0;
           $217 = $216 << 16 >> 16;
           $218 = (($217) + ($214))|0;
           $219 = $218&65535;
           HEAP16[$215>>1] = $219;
          }
         }
        }
       } else {
        $220 = $r;
        $221 = (($220) + -1)|0;
        $r = $221;
        $222 = ($221|0)<(0);
        if ($222) {
         label = 58;
         break;
        }
       }
       $223 = $k;
       $224 = (($223) + 1)|0;
       $k = $224;
       $225 = $k;
       $226 = $Se;
       $227 = ($225|0)<=($226|0);
       if (!($227)) {
        break;
       }
      }
      if ((label|0) == 58) {
       label = 0;
      }
      $228 = $s;
      $229 = ($228|0)!=(0);
      if ($229) {
       $230 = $k;
       $231 = (9504 + ($230<<2)|0);
       $232 = HEAPU8[$231>>0]|(HEAPU8[$231+1>>0]<<8)|(HEAPU8[$231+2>>0]<<16)|(HEAPU8[$231+3>>0]<<24);
       $pos = $232;
       $233 = $s;
       $234 = $233&65535;
       $235 = $pos;
       $236 = $block;
       $237 = (($236) + ($235<<1)|0);
       HEAP16[$237>>1] = $234;
       $238 = $pos;
       $239 = $num_newnz;
       $240 = (($239) + 1)|0;
       $num_newnz = $240;
       $241 = (($newnz_pos) + ($239<<2)|0);
       HEAP32[$241>>2] = $238;
      }
      $242 = $k;
      $243 = (($242) + 1)|0;
      $k = $243;
     }
     if ((label|0) == 13) {
      break;
     }
     else if ((label|0) == 21) {
      break;
     }
     else if ((label|0) == 28) {
      break;
     }
     else if ((label|0) == 35) {
      $145 = $r;
      $146 = 1 << $145;
      $EOBRUN = $146;
      $147 = $r;
      $148 = ($147|0)!=(0);
      if ($148) {
       $149 = $bits_left;
       $150 = $r;
       $151 = ($149|0)<($150|0);
       do {
        if ($151) {
         $152 = $get_buffer;
         $153 = $bits_left;
         $154 = $r;
         $155 = (_jpeg_fill_bit_buffer($br_state,$152,$153,$154)|0);
         $156 = ($155|0)!=(0);
         if ($156) {
          $157 = (($br_state) + 8|0);
          $158 = HEAP32[$157>>2]|0;
          $get_buffer = $158;
          $159 = (($br_state) + 12|0);
          $160 = HEAP32[$159>>2]|0;
          $bits_left = $160;
          break;
         } else {
          break L13;
         }
        }
       } while(0);
       $161 = $get_buffer;
       $162 = $r;
       $163 = $bits_left;
       $164 = (($163) - ($162))|0;
       $bits_left = $164;
       $165 = $161 >> $164;
       $166 = $r;
       $167 = 1 << $166;
       $168 = (($167) - 1)|0;
       $169 = $165 & $168;
       $r = $169;
       $170 = $r;
       $171 = $EOBRUN;
       $172 = (($171) + ($170))|0;
       $EOBRUN = $172;
      }
     }
     else if ((label|0) == 47) {
      break;
     }
     label = 67;
    } else {
     label = 67;
    }
   } while(0);
   L94: do {
    if ((label|0) == 67) {
     $244 = $EOBRUN;
     $245 = ($244>>>0)>(0);
     do {
      if ($245) {
       while(1) {
        $246 = $k;
        $247 = $Se;
        $248 = ($246|0)<=($247|0);
        if (!($248)) {
         label = 85;
         break;
        }
        $249 = $block;
        $250 = $k;
        $251 = (9504 + ($250<<2)|0);
        $252 = HEAPU8[$251>>0]|(HEAPU8[$251+1>>0]<<8)|(HEAPU8[$251+2>>0]<<16)|(HEAPU8[$251+3>>0]<<24);
        $253 = (($249) + ($252<<1)|0);
        $thiscoef = $253;
        $254 = $thiscoef;
        $255 = HEAP16[$254>>1]|0;
        $256 = $255 << 16 >> 16;
        $257 = ($256|0)!=(0);
        if ($257) {
         $258 = $bits_left;
         $259 = ($258|0)<(1);
         if ($259) {
          $260 = $get_buffer;
          $261 = $bits_left;
          $262 = (_jpeg_fill_bit_buffer($br_state,$260,$261,1)|0);
          $263 = ($262|0)!=(0);
          if (!($263)) {
           label = 73;
           break;
          }
          $264 = (($br_state) + 8|0);
          $265 = HEAP32[$264>>2]|0;
          $get_buffer = $265;
          $266 = (($br_state) + 12|0);
          $267 = HEAP32[$266>>2]|0;
          $bits_left = $267;
         }
         $268 = $get_buffer;
         $269 = $bits_left;
         $270 = (($269) - 1)|0;
         $bits_left = $270;
         $271 = $268 >> $270;
         $272 = $271 & 1;
         $273 = ($272|0)!=(0);
         if ($273) {
          $274 = $thiscoef;
          $275 = HEAP16[$274>>1]|0;
          $276 = $275 << 16 >> 16;
          $277 = $p1;
          $278 = $276 & $277;
          $279 = ($278|0)==(0);
          if ($279) {
           $280 = $thiscoef;
           $281 = HEAP16[$280>>1]|0;
           $282 = $281 << 16 >> 16;
           $283 = ($282|0)>=(0);
           if ($283) {
            $284 = $p1;
            $285 = $thiscoef;
            $286 = HEAP16[$285>>1]|0;
            $287 = $286 << 16 >> 16;
            $288 = (($287) + ($284))|0;
            $289 = $288&65535;
            HEAP16[$285>>1] = $289;
           } else {
            $290 = $m1;
            $291 = $thiscoef;
            $292 = HEAP16[$291>>1]|0;
            $293 = $292 << 16 >> 16;
            $294 = (($293) + ($290))|0;
            $295 = $294&65535;
            HEAP16[$291>>1] = $295;
           }
          }
         }
        }
        $296 = $k;
        $297 = (($296) + 1)|0;
        $k = $297;
       }
       if ((label|0) == 73) {
        break L94;
       }
       else if ((label|0) == 85) {
        $298 = $EOBRUN;
        $299 = (($298) + -1)|0;
        $EOBRUN = $299;
        break;
       }
      }
     } while(0);
     $300 = HEAP32[$br_state>>2]|0;
     $301 = $1;
     $302 = (($301) + 24|0);
     $303 = HEAP32[$302>>2]|0;
     HEAP32[$303>>2] = $300;
     $304 = (($br_state) + 4|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = $1;
     $307 = (($306) + 24|0);
     $308 = HEAP32[$307>>2]|0;
     $309 = (($308) + 4|0);
     HEAP32[$309>>2] = $305;
     $310 = $get_buffer;
     $311 = $entropy;
     $312 = (($311) + 12|0);
     HEAP32[$312>>2] = $310;
     $313 = $bits_left;
     $314 = $entropy;
     $315 = (($314) + 12|0);
     $316 = (($315) + 4|0);
     HEAP32[$316>>2] = $313;
     $317 = $EOBRUN;
     $318 = $entropy;
     $319 = (($318) + 20|0);
     HEAP32[$319>>2] = $317;
     break L11;
    }
   } while(0);
   while(1) {
    $324 = $num_newnz;
    $325 = ($324|0)>(0);
    if (!($325)) {
     break;
    }
    $326 = $num_newnz;
    $327 = (($326) + -1)|0;
    $num_newnz = $327;
    $328 = (($newnz_pos) + ($327<<2)|0);
    $329 = HEAP32[$328>>2]|0;
    $330 = $block;
    $331 = (($330) + ($329<<1)|0);
    HEAP16[$331>>1] = 0;
   }
   $0 = 0;
   $332 = $0;
   STACKTOP = sp;return ($332|0);
  }
 } while(0);
 $320 = $entropy;
 $321 = (($320) + 40|0);
 $322 = HEAP32[$321>>2]|0;
 $323 = (($322) + -1)|0;
 HEAP32[$321>>2] = $323;
 $0 = 1;
 $332 = $0;
 STACKTOP = sp;return ($332|0);
}
function _process_restart($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $entropy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 444|0);
 $4 = HEAP32[$3>>2]|0;
 $entropy = $4;
 $5 = $entropy;
 $6 = (($5) + 12|0);
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8|0) / 8)&-1;
 $10 = $1;
 $11 = (($10) + 440|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($9))|0;
 HEAP32[$13>>2] = $15;
 $16 = $entropy;
 $17 = (($16) + 12|0);
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = 0;
 $19 = $1;
 $20 = (($19) + 440|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = (FUNCTION_TABLE_ii[$23 & 127]($24)|0);
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $0 = 0;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 $ci = 0;
 while(1) {
  $27 = $ci;
  $28 = $1;
  $29 = (($28) + 328|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = $ci;
  $33 = $entropy;
  $34 = (($33) + 20|0);
  $35 = (($34) + 4|0);
  $36 = (($35) + ($32<<2)|0);
  HEAP32[$36>>2] = 0;
  $37 = $ci;
  $38 = (($37) + 1)|0;
  $ci = $38;
 }
 $39 = $entropy;
 $40 = (($39) + 20|0);
 HEAP32[$40>>2] = 0;
 $41 = $1;
 $42 = (($41) + 276|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $entropy;
 $45 = (($44) + 40|0);
 HEAP32[$45>>2] = $43;
 $46 = $1;
 $47 = (($46) + 416|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = ($48|0)==(0);
 if ($49) {
  $50 = $entropy;
  $51 = (($50) + 8|0);
  HEAP32[$51>>2] = 0;
 }
 $0 = 1;
 $52 = $0;
 STACKTOP = sp;return ($52|0);
}
function _jinit_d_post_controller($cinfo,$need_full_buffer) {
 $cinfo = $cinfo|0;
 $need_full_buffer = $need_full_buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $need_full_buffer;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 127]($6,1,28)|0);
 $post = $7;
 $8 = $post;
 $9 = $0;
 $10 = (($9) + 432|0);
 HEAP32[$10>>2] = $8;
 $11 = $post;
 HEAP32[$11>>2] = 47;
 $12 = $post;
 $13 = (($12) + 8|0);
 HEAP32[$13>>2] = 0;
 $14 = $post;
 $15 = (($14) + 12|0);
 HEAP32[$15>>2] = 0;
 $16 = $0;
 $17 = (($16) + 84|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 $20 = $0;
 $21 = (($20) + 312|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $post;
 $24 = (($23) + 16|0);
 HEAP32[$24>>2] = $22;
 $25 = $1;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $0;
  $28 = (($27) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $0;
  $33 = $0;
  $34 = (($33) + 112|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = $0;
  $37 = (($36) + 120|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = Math_imul($35, $38)|0;
  $40 = $0;
  $41 = (($40) + 116|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $post;
  $44 = (($43) + 16|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_jround_up($42,$45)|0);
  $47 = $post;
  $48 = (($47) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (FUNCTION_TABLE_iiiiiii[$31 & 127]($32,1,0,$39,$46,$49)|0);
  $51 = $post;
  $52 = (($51) + 8|0);
  HEAP32[$52>>2] = $50;
 } else {
  $53 = $0;
  $54 = (($53) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $0;
  $59 = $0;
  $60 = (($59) + 112|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = $0;
  $63 = (($62) + 120|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = Math_imul($61, $64)|0;
  $66 = $post;
  $67 = (($66) + 16|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (FUNCTION_TABLE_iiiii[$57 & 127]($58,1,$65,$68)|0);
  $70 = $post;
  $71 = (($70) + 12|0);
  HEAP32[$71>>2] = $69;
 }
 STACKTOP = sp;return;
}
function _start_pass_dpost($cinfo,$pass_mode) {
 $cinfo = $cinfo|0;
 $pass_mode = $pass_mode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $pass_mode;
 $2 = $0;
 $3 = (($2) + 432|0);
 $4 = HEAP32[$3>>2]|0;
 $post = $4;
 $5 = $1;
 if ((($5|0) == 2)) {
  $51 = $post;
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($53|0)==(0|0);
  if ($54) {
   $55 = $0;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 20|0);
   HEAP32[$57>>2] = 4;
   $58 = $0;
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$59>>2]|0;
   $61 = $0;
   FUNCTION_TABLE_vi[$60 & 127]($61);
  }
  $62 = $post;
  $63 = (($62) + 4|0);
  HEAP32[$63>>2] = 50;
 } else if ((($5|0) == 3)) {
  $38 = $post;
  $39 = (($38) + 8|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)==(0|0);
  if ($41) {
   $42 = $0;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($43) + 20|0);
   HEAP32[$44>>2] = 4;
   $45 = $0;
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$46>>2]|0;
   $48 = $0;
   FUNCTION_TABLE_vi[$47 & 127]($48);
  }
  $49 = $post;
  $50 = (($49) + 4|0);
  HEAP32[$50>>2] = 49;
 } else if ((($5|0) == 0)) {
  $6 = $0;
  $7 = (($6) + 84|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0);
  if ($9) {
   $10 = $post;
   $11 = (($10) + 4|0);
   HEAP32[$11>>2] = 48;
   $12 = $post;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(0|0);
   if ($15) {
    $16 = $0;
    $17 = (($16) + 4|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + 28|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $0;
    $22 = $post;
    $23 = (($22) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $post;
    $26 = (($25) + 16|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (FUNCTION_TABLE_iiiiii[$20 & 127]($21,$24,0,$27,1)|0);
    $29 = $post;
    $30 = (($29) + 12|0);
    HEAP32[$30>>2] = $28;
   }
  } else {
   $31 = $0;
   $32 = (($31) + 452|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $post;
   $37 = (($36) + 4|0);
   HEAP32[$37>>2] = $35;
  }
 } else {
  $64 = $0;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 20|0);
  HEAP32[$66>>2] = 4;
  $67 = $0;
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = $0;
  FUNCTION_TABLE_vi[$69 & 127]($70);
 }
 $71 = $post;
 $72 = (($71) + 24|0);
 HEAP32[$72>>2] = 0;
 $73 = $post;
 $74 = (($73) + 20|0);
 HEAP32[$74>>2] = 0;
 STACKTOP = sp;return;
}
function _post_process_1pass($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $max_rows = 0, $num_rows = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $num_rows = sp + 4|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 432|0);
 $9 = HEAP32[$8>>2]|0;
 $post = $9;
 $10 = $6;
 $11 = $5;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) - ($12))|0;
 $max_rows = $13;
 $14 = $max_rows;
 $15 = $post;
 $16 = (($15) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14>>>0)>($17>>>0);
 if ($18) {
  $19 = $post;
  $20 = (($19) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  $max_rows = $21;
 }
 HEAP32[$num_rows>>2] = 0;
 $22 = $0;
 $23 = (($22) + 452|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $0;
 $28 = $1;
 $29 = $2;
 $30 = $3;
 $31 = $post;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $max_rows;
 FUNCTION_TABLE_viiiiiii[$26 & 63]($27,$28,$29,$30,$33,$num_rows,$34);
 $35 = $0;
 $36 = (($35) + 460|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = $0;
 $41 = $post;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $4;
 $45 = $5;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + ($46<<2)|0);
 $48 = HEAP32[$num_rows>>2]|0;
 FUNCTION_TABLE_viiii[$39 & 127]($40,$43,$47,$48);
 $49 = HEAP32[$num_rows>>2]|0;
 $50 = $5;
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($49))|0;
 HEAP32[$50>>2] = $52;
 STACKTOP = sp;return;
}
function _post_process_prepass($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $num_rows = 0, $old_next_row = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 432|0);
 $9 = HEAP32[$8>>2]|0;
 $post = $9;
 $10 = $post;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  $20 = $post;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $post;
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $post;
  $27 = (($26) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (FUNCTION_TABLE_iiiiii[$18 & 127]($19,$22,$25,$28,1)|0);
  $30 = $post;
  $31 = (($30) + 12|0);
  HEAP32[$31>>2] = $29;
 }
 $32 = $post;
 $33 = (($32) + 24|0);
 $34 = HEAP32[$33>>2]|0;
 $old_next_row = $34;
 $35 = $0;
 $36 = (($35) + 452|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = $0;
 $41 = $1;
 $42 = $2;
 $43 = $3;
 $44 = $post;
 $45 = (($44) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $post;
 $48 = (($47) + 24|0);
 $49 = $post;
 $50 = (($49) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 FUNCTION_TABLE_viiiiiii[$39 & 63]($40,$41,$42,$43,$46,$48,$51);
 $52 = $post;
 $53 = (($52) + 24|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $old_next_row;
 $56 = ($54>>>0)>($55>>>0);
 if ($56) {
  $57 = $post;
  $58 = (($57) + 24|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $old_next_row;
  $61 = (($59) - ($60))|0;
  $num_rows = $61;
  $62 = $0;
  $63 = (($62) + 460|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($64) + 4|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $0;
  $68 = $post;
  $69 = (($68) + 12|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $old_next_row;
  $72 = (($70) + ($71<<2)|0);
  $73 = $num_rows;
  FUNCTION_TABLE_viiii[$66 & 127]($67,$72,0,$73);
  $74 = $num_rows;
  $75 = $5;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76) + ($74))|0;
  HEAP32[$75>>2] = $77;
 }
 $78 = $post;
 $79 = (($78) + 24|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $post;
 $82 = (($81) + 16|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = ($80>>>0)>=($83>>>0);
 if (!($84)) {
  STACKTOP = sp;return;
 }
 $85 = $post;
 $86 = (($85) + 16|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $post;
 $89 = (($88) + 20|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = (($90) + ($87))|0;
 HEAP32[$89>>2] = $91;
 $92 = $post;
 $93 = (($92) + 24|0);
 HEAP32[$93>>2] = 0;
 STACKTOP = sp;return;
}
function _post_process_2pass($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $max_rows = 0, $num_rows = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 432|0);
 $9 = HEAP32[$8>>2]|0;
 $post = $9;
 $10 = $post;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  $20 = $post;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $post;
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $post;
  $27 = (($26) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (FUNCTION_TABLE_iiiiii[$18 & 127]($19,$22,$25,$28,0)|0);
  $30 = $post;
  $31 = (($30) + 12|0);
  HEAP32[$31>>2] = $29;
 }
 $32 = $post;
 $33 = (($32) + 16|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $post;
 $36 = (($35) + 24|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($34) - ($37))|0;
 $num_rows = $38;
 $39 = $6;
 $40 = $5;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($39) - ($41))|0;
 $max_rows = $42;
 $43 = $num_rows;
 $44 = $max_rows;
 $45 = ($43>>>0)>($44>>>0);
 if ($45) {
  $46 = $max_rows;
  $num_rows = $46;
 }
 $47 = $0;
 $48 = (($47) + 116|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $post;
 $51 = (($50) + 20|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($49) - ($52))|0;
 $max_rows = $53;
 $54 = $num_rows;
 $55 = $max_rows;
 $56 = ($54>>>0)>($55>>>0);
 if ($56) {
  $57 = $max_rows;
  $num_rows = $57;
 }
 $58 = $0;
 $59 = (($58) + 460|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + 4|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $0;
 $64 = $post;
 $65 = (($64) + 12|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $post;
 $68 = (($67) + 24|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (($66) + ($69<<2)|0);
 $71 = $4;
 $72 = $5;
 $73 = HEAP32[$72>>2]|0;
 $74 = (($71) + ($73<<2)|0);
 $75 = $num_rows;
 FUNCTION_TABLE_viiii[$62 & 127]($63,$70,$74,$75);
 $76 = $num_rows;
 $77 = $5;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($78) + ($76))|0;
 HEAP32[$77>>2] = $79;
 $80 = $num_rows;
 $81 = $post;
 $82 = (($81) + 24|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (($83) + ($80))|0;
 HEAP32[$82>>2] = $84;
 $85 = $post;
 $86 = (($85) + 24|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $post;
 $89 = (($88) + 16|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = ($87>>>0)>=($90>>>0);
 if (!($91)) {
  STACKTOP = sp;return;
 }
 $92 = $post;
 $93 = (($92) + 16|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $post;
 $96 = (($95) + 20|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = (($97) + ($94))|0;
 HEAP32[$96>>2] = $98;
 $99 = $post;
 $100 = (($99) + 24|0);
 HEAP32[$100>>2] = 0;
 STACKTOP = sp;return;
}
function _jinit_upsampler($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $ci = 0, $compptr = 0, $do_fancy = 0, $h_in_group = 0, $h_out_group = 0, $need_buffer = 0, $upsample = 0, $v_in_group = 0, $v_out_group = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,160)|0);
 $upsample = $6;
 $7 = $upsample;
 $8 = $0;
 $9 = (($8) + 452|0);
 HEAP32[$9>>2] = $7;
 $10 = $upsample;
 HEAP32[$10>>2] = 51;
 $11 = $upsample;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 52;
 $13 = $upsample;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 $15 = $0;
 $16 = (($15) + 300|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = $0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 25;
  $22 = $0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = $0;
  FUNCTION_TABLE_vi[$24 & 127]($25);
 }
 $26 = $0;
 $27 = (($26) + 76|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = $0;
  $31 = (($30) + 316|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)>(1);
  $35 = $33;
 } else {
  $35 = 0;
 }
 $34 = $35&1;
 $do_fancy = $34;
 $ci = 0;
 $36 = $0;
 $37 = (($36) + 216|0);
 $38 = HEAP32[$37>>2]|0;
 $compptr = $38;
 while(1) {
  $39 = $ci;
  $40 = $0;
  $41 = (($40) + 36|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($39|0)<($42|0);
  if (!($43)) {
   break;
  }
  $44 = $compptr;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $compptr;
  $48 = (($47) + 36|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = Math_imul($46, $49)|0;
  $51 = $0;
  $52 = (($51) + 316|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($50|0) / ($53|0))&-1;
  $h_in_group = $54;
  $55 = $compptr;
  $56 = (($55) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $compptr;
  $59 = (($58) + 36|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = Math_imul($57, $60)|0;
  $62 = $0;
  $63 = (($62) + 316|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($61|0) / ($64|0))&-1;
  $v_in_group = $65;
  $66 = $0;
  $67 = (($66) + 308|0);
  $68 = HEAP32[$67>>2]|0;
  $h_out_group = $68;
  $69 = $0;
  $70 = (($69) + 312|0);
  $71 = HEAP32[$70>>2]|0;
  $v_out_group = $71;
  $72 = $v_in_group;
  $73 = $ci;
  $74 = $upsample;
  $75 = (($74) + 100|0);
  $76 = (($75) + ($73<<2)|0);
  HEAP32[$76>>2] = $72;
  $need_buffer = 1;
  $77 = $compptr;
  $78 = (($77) + 48|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0);
  if ($80) {
   $85 = $h_in_group;
   $86 = $h_out_group;
   $87 = ($85|0)==($86|0);
   if ($87) {
    $88 = $v_in_group;
    $89 = $v_out_group;
    $90 = ($88|0)==($89|0);
    if ($90) {
     $91 = $ci;
     $92 = $upsample;
     $93 = (($92) + 52|0);
     $94 = (($93) + ($91<<2)|0);
     HEAP32[$94>>2] = 54;
     $need_buffer = 0;
    } else {
     label = 12;
    }
   } else {
    label = 12;
   }
   if ((label|0) == 12) {
    label = 0;
    $95 = $h_in_group;
    $96 = $95<<1;
    $97 = $h_out_group;
    $98 = ($96|0)==($97|0);
    if ($98) {
     $99 = $v_in_group;
     $100 = $v_out_group;
     $101 = ($99|0)==($100|0);
     if ($101) {
      $102 = $do_fancy;
      $103 = ($102|0)!=(0);
      if ($103) {
       $104 = $compptr;
       $105 = (($104) + 40|0);
       $106 = HEAP32[$105>>2]|0;
       $107 = ($106>>>0)>(2);
       if ($107) {
        $108 = $ci;
        $109 = $upsample;
        $110 = (($109) + 52|0);
        $111 = (($110) + ($108<<2)|0);
        HEAP32[$111>>2] = 55;
       } else {
        label = 17;
       }
      } else {
       label = 17;
      }
      if ((label|0) == 17) {
       label = 0;
       $112 = $ci;
       $113 = $upsample;
       $114 = (($113) + 52|0);
       $115 = (($114) + ($112<<2)|0);
       HEAP32[$115>>2] = 56;
      }
     } else {
      label = 19;
     }
    } else {
     label = 19;
    }
    if ((label|0) == 19) {
     label = 0;
     $116 = $h_in_group;
     $117 = $116<<1;
     $118 = $h_out_group;
     $119 = ($117|0)==($118|0);
     if ($119) {
      $120 = $v_in_group;
      $121 = $120<<1;
      $122 = $v_out_group;
      $123 = ($121|0)==($122|0);
      if ($123) {
       $124 = $do_fancy;
       $125 = ($124|0)!=(0);
       if ($125) {
        $126 = $compptr;
        $127 = (($126) + 40|0);
        $128 = HEAP32[$127>>2]|0;
        $129 = ($128>>>0)>(2);
        if ($129) {
         $130 = $ci;
         $131 = $upsample;
         $132 = (($131) + 52|0);
         $133 = (($132) + ($130<<2)|0);
         HEAP32[$133>>2] = 57;
         $134 = $upsample;
         $135 = (($134) + 8|0);
         HEAP32[$135>>2] = 1;
        } else {
         label = 24;
        }
       } else {
        label = 24;
       }
       if ((label|0) == 24) {
        label = 0;
        $136 = $ci;
        $137 = $upsample;
        $138 = (($137) + 52|0);
        $139 = (($138) + ($136<<2)|0);
        HEAP32[$139>>2] = 58;
       }
      } else {
       label = 26;
      }
     } else {
      label = 26;
     }
     if ((label|0) == 26) {
      label = 0;
      $140 = $h_out_group;
      $141 = $h_in_group;
      $142 = (($140|0) % ($141|0))&-1;
      $143 = ($142|0)==(0);
      if ($143) {
       $144 = $v_out_group;
       $145 = $v_in_group;
       $146 = (($144|0) % ($145|0))&-1;
       $147 = ($146|0)==(0);
       if ($147) {
        $148 = $ci;
        $149 = $upsample;
        $150 = (($149) + 52|0);
        $151 = (($150) + ($148<<2)|0);
        HEAP32[$151>>2] = 59;
        $152 = $h_out_group;
        $153 = $h_in_group;
        $154 = (($152|0) / ($153|0))&-1;
        $155 = $154&255;
        $156 = $ci;
        $157 = $upsample;
        $158 = (($157) + 140|0);
        $159 = (($158) + ($156)|0);
        HEAP8[$159>>0] = $155;
        $160 = $v_out_group;
        $161 = $v_in_group;
        $162 = (($160|0) / ($161|0))&-1;
        $163 = $162&255;
        $164 = $ci;
        $165 = $upsample;
        $166 = (($165) + 150|0);
        $167 = (($166) + ($164)|0);
        HEAP8[$167>>0] = $163;
       } else {
        label = 29;
       }
      } else {
       label = 29;
      }
      if ((label|0) == 29) {
       label = 0;
       $168 = $0;
       $169 = HEAP32[$168>>2]|0;
       $170 = (($169) + 20|0);
       HEAP32[$170>>2] = 38;
       $171 = $0;
       $172 = HEAP32[$171>>2]|0;
       $173 = HEAP32[$172>>2]|0;
       $174 = $0;
       FUNCTION_TABLE_vi[$173 & 127]($174);
      }
     }
    }
   }
  } else {
   $81 = $ci;
   $82 = $upsample;
   $83 = (($82) + 52|0);
   $84 = (($83) + ($81<<2)|0);
   HEAP32[$84>>2] = 53;
   $need_buffer = 0;
  }
  $175 = $need_buffer;
  $176 = ($175|0)!=(0);
  if ($176) {
   $177 = $0;
   $178 = (($177) + 4|0);
   $179 = HEAP32[$178>>2]|0;
   $180 = (($179) + 8|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = $0;
   $183 = $0;
   $184 = (($183) + 112|0);
   $185 = HEAP32[$184>>2]|0;
   $186 = $0;
   $187 = (($186) + 308|0);
   $188 = HEAP32[$187>>2]|0;
   $189 = (_jround_up($185,$188)|0);
   $190 = $0;
   $191 = (($190) + 312|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = (FUNCTION_TABLE_iiiii[$181 & 127]($182,1,$189,$192)|0);
   $194 = $ci;
   $195 = $upsample;
   $196 = (($195) + 12|0);
   $197 = (($196) + ($194<<2)|0);
   HEAP32[$197>>2] = $193;
  }
  $198 = $ci;
  $199 = (($198) + 1)|0;
  $ci = $199;
  $200 = $compptr;
  $201 = (($200) + 84|0);
  $compptr = $201;
 }
 STACKTOP = sp;return;
}
function _start_pass_upsample($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $upsample = $3;
 $4 = $0;
 $5 = (($4) + 312|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $upsample;
 $8 = (($7) + 92|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = (($9) + 116|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $upsample;
 $13 = (($12) + 96|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _sep_upsample($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0;
 var $num_rows = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 452|0);
 $9 = HEAP32[$8>>2]|0;
 $upsample = $9;
 $10 = $upsample;
 $11 = (($10) + 92|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 312|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($12|0)>=($15|0);
 if ($16) {
  $ci = 0;
  $17 = $0;
  $18 = (($17) + 216|0);
  $19 = HEAP32[$18>>2]|0;
  $compptr = $19;
  while(1) {
   $20 = $ci;
   $21 = $0;
   $22 = (($21) + 36|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($20|0)<($23|0);
   if (!($24)) {
    break;
   }
   $25 = $ci;
   $26 = $upsample;
   $27 = (($26) + 52|0);
   $28 = (($27) + ($25<<2)|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $0;
   $31 = $compptr;
   $32 = $ci;
   $33 = $1;
   $34 = (($33) + ($32<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $2;
   $37 = HEAP32[$36>>2]|0;
   $38 = $ci;
   $39 = $upsample;
   $40 = (($39) + 100|0);
   $41 = (($40) + ($38<<2)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = Math_imul($37, $42)|0;
   $44 = (($35) + ($43<<2)|0);
   $45 = $upsample;
   $46 = (($45) + 12|0);
   $47 = $ci;
   $48 = (($46) + ($47<<2)|0);
   FUNCTION_TABLE_viiii[$29 & 127]($30,$31,$44,$48);
   $49 = $ci;
   $50 = (($49) + 1)|0;
   $ci = $50;
   $51 = $compptr;
   $52 = (($51) + 84|0);
   $compptr = $52;
  }
  $53 = $upsample;
  $54 = (($53) + 92|0);
  HEAP32[$54>>2] = 0;
 }
 $55 = $0;
 $56 = (($55) + 312|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $upsample;
 $59 = (($58) + 92|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($57) - ($60))|0;
 $num_rows = $61;
 $62 = $num_rows;
 $63 = $upsample;
 $64 = (($63) + 96|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = ($62>>>0)>($65>>>0);
 if ($66) {
  $67 = $upsample;
  $68 = (($67) + 96|0);
  $69 = HEAP32[$68>>2]|0;
  $num_rows = $69;
 }
 $70 = $5;
 $71 = HEAP32[$70>>2]|0;
 $72 = $6;
 $73 = (($72) - ($71))|0;
 $6 = $73;
 $74 = $num_rows;
 $75 = $6;
 $76 = ($74>>>0)>($75>>>0);
 if ($76) {
  $77 = $6;
  $num_rows = $77;
 }
 $78 = $0;
 $79 = (($78) + 456|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = (($80) + 4|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = $0;
 $84 = $upsample;
 $85 = (($84) + 12|0);
 $86 = $upsample;
 $87 = (($86) + 92|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $4;
 $90 = $5;
 $91 = HEAP32[$90>>2]|0;
 $92 = (($89) + ($91<<2)|0);
 $93 = $num_rows;
 FUNCTION_TABLE_viiiii[$82 & 127]($83,$85,$88,$92,$93);
 $94 = $num_rows;
 $95 = $5;
 $96 = HEAP32[$95>>2]|0;
 $97 = (($96) + ($94))|0;
 HEAP32[$95>>2] = $97;
 $98 = $num_rows;
 $99 = $upsample;
 $100 = (($99) + 96|0);
 $101 = HEAP32[$100>>2]|0;
 $102 = (($101) - ($98))|0;
 HEAP32[$100>>2] = $102;
 $103 = $num_rows;
 $104 = $upsample;
 $105 = (($104) + 92|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = (($106) + ($103))|0;
 HEAP32[$105>>2] = $107;
 $108 = $upsample;
 $109 = (($108) + 92|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = $0;
 $112 = (($111) + 312|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = ($110|0)>=($113|0);
 if (!($114)) {
  STACKTOP = sp;return;
 }
 $115 = $2;
 $116 = HEAP32[$115>>2]|0;
 $117 = (($116) + 1)|0;
 HEAP32[$115>>2] = $117;
 STACKTOP = sp;return;
}
function _noop_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 STACKTOP = sp;return;
}
function _fullsize_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $2;
 $5 = $3;
 HEAP32[$5>>2] = $4;
 STACKTOP = sp;return;
}
function _h2v1_fancy_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $colctr = 0, $inptr = 0, $inrow = 0, $invalue = 0, $outptr = 0, $output_data = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $inrow = 0;
 while(1) {
  $6 = $inrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $inrow;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $inptr = $14;
  $15 = $inrow;
  $16 = $output_data;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $outptr = $18;
  $19 = $inptr;
  $20 = (($19) + 1|0);
  $inptr = $20;
  $21 = HEAP8[$19>>0]|0;
  $22 = $21&255;
  $invalue = $22;
  $23 = $invalue;
  $24 = $23&255;
  $25 = $outptr;
  $26 = (($25) + 1|0);
  $outptr = $26;
  HEAP8[$25>>0] = $24;
  $27 = $invalue;
  $28 = ($27*3)|0;
  $29 = $inptr;
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = (($28) + ($31))|0;
  $33 = (($32) + 2)|0;
  $34 = $33 >> 2;
  $35 = $34&255;
  $36 = $outptr;
  $37 = (($36) + 1|0);
  $outptr = $37;
  HEAP8[$36>>0] = $35;
  $38 = $1;
  $39 = (($38) + 40|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) - 2)|0;
  $colctr = $41;
  while(1) {
   $42 = $colctr;
   $43 = ($42>>>0)>(0);
   if (!($43)) {
    break;
   }
   $44 = $inptr;
   $45 = (($44) + 1|0);
   $inptr = $45;
   $46 = HEAP8[$44>>0]|0;
   $47 = $46&255;
   $48 = ($47*3)|0;
   $invalue = $48;
   $49 = $invalue;
   $50 = $inptr;
   $51 = (($50) + -2|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = (($49) + ($53))|0;
   $55 = (($54) + 1)|0;
   $56 = $55 >> 2;
   $57 = $56&255;
   $58 = $outptr;
   $59 = (($58) + 1|0);
   $outptr = $59;
   HEAP8[$58>>0] = $57;
   $60 = $invalue;
   $61 = $inptr;
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = (($60) + ($63))|0;
   $65 = (($64) + 2)|0;
   $66 = $65 >> 2;
   $67 = $66&255;
   $68 = $outptr;
   $69 = (($68) + 1|0);
   $outptr = $69;
   HEAP8[$68>>0] = $67;
   $70 = $colctr;
   $71 = (($70) + -1)|0;
   $colctr = $71;
  }
  $72 = $inptr;
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $invalue = $74;
  $75 = $invalue;
  $76 = ($75*3)|0;
  $77 = $inptr;
  $78 = (($77) + -1|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = (($76) + ($80))|0;
  $82 = (($81) + 1)|0;
  $83 = $82 >> 2;
  $84 = $83&255;
  $85 = $outptr;
  $86 = (($85) + 1|0);
  $outptr = $86;
  HEAP8[$85>>0] = $84;
  $87 = $invalue;
  $88 = $87&255;
  $89 = $outptr;
  $90 = (($89) + 1|0);
  $outptr = $90;
  HEAP8[$89>>0] = $88;
  $91 = $inrow;
  $92 = (($91) + 1)|0;
  $inrow = $92;
 }
 STACKTOP = sp;return;
}
function _h2v1_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inptr = 0, $inrow = 0;
 var $invalue = 0, $outend = 0, $outptr = 0, $output_data = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $inrow = 0;
 while(1) {
  $6 = $inrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $inrow;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $inptr = $14;
  $15 = $inrow;
  $16 = $output_data;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $outptr = $18;
  $19 = $outptr;
  $20 = $0;
  $21 = (($20) + 112|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) + ($22)|0);
  $outend = $23;
  while(1) {
   $24 = $outptr;
   $25 = $outend;
   $26 = ($24>>>0)<($25>>>0);
   if (!($26)) {
    break;
   }
   $27 = $inptr;
   $28 = (($27) + 1|0);
   $inptr = $28;
   $29 = HEAP8[$27>>0]|0;
   $invalue = $29;
   $30 = $invalue;
   $31 = $outptr;
   $32 = (($31) + 1|0);
   $outptr = $32;
   HEAP8[$31>>0] = $30;
   $33 = $invalue;
   $34 = $outptr;
   $35 = (($34) + 1|0);
   $outptr = $35;
   HEAP8[$34>>0] = $33;
  }
  $36 = $inrow;
  $37 = (($36) + 1)|0;
  $inrow = $37;
 }
 STACKTOP = sp;return;
}
function _h2v2_fancy_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $colctr = 0, $inptr0 = 0, $inptr1 = 0, $inrow = 0, $lastcolsum = 0, $nextcolsum = 0, $outptr = 0, $output_data = 0, $outrow = 0, $thiscolsum = 0;
 var $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $outrow = 0;
 $inrow = 0;
 while(1) {
  $6 = $outrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $v = 0;
  while(1) {
   $11 = $v;
   $12 = ($11|0)<(2);
   if (!($12)) {
    break;
   }
   $13 = $inrow;
   $14 = $2;
   $15 = (($14) + ($13<<2)|0);
   $16 = HEAP32[$15>>2]|0;
   $inptr0 = $16;
   $17 = $v;
   $18 = ($17|0)==(0);
   if ($18) {
    $19 = $inrow;
    $20 = (($19) - 1)|0;
    $21 = $2;
    $22 = (($21) + ($20<<2)|0);
    $23 = HEAP32[$22>>2]|0;
    $inptr1 = $23;
   } else {
    $24 = $inrow;
    $25 = (($24) + 1)|0;
    $26 = $2;
    $27 = (($26) + ($25<<2)|0);
    $28 = HEAP32[$27>>2]|0;
    $inptr1 = $28;
   }
   $29 = $outrow;
   $30 = (($29) + 1)|0;
   $outrow = $30;
   $31 = $output_data;
   $32 = (($31) + ($29<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   $outptr = $33;
   $34 = $inptr0;
   $35 = (($34) + 1|0);
   $inptr0 = $35;
   $36 = HEAP8[$34>>0]|0;
   $37 = $36&255;
   $38 = ($37*3)|0;
   $39 = $inptr1;
   $40 = (($39) + 1|0);
   $inptr1 = $40;
   $41 = HEAP8[$39>>0]|0;
   $42 = $41&255;
   $43 = (($38) + ($42))|0;
   $thiscolsum = $43;
   $44 = $inptr0;
   $45 = (($44) + 1|0);
   $inptr0 = $45;
   $46 = HEAP8[$44>>0]|0;
   $47 = $46&255;
   $48 = ($47*3)|0;
   $49 = $inptr1;
   $50 = (($49) + 1|0);
   $inptr1 = $50;
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   $53 = (($48) + ($52))|0;
   $nextcolsum = $53;
   $54 = $thiscolsum;
   $55 = $54<<2;
   $56 = (($55) + 8)|0;
   $57 = $56 >> 4;
   $58 = $57&255;
   $59 = $outptr;
   $60 = (($59) + 1|0);
   $outptr = $60;
   HEAP8[$59>>0] = $58;
   $61 = $thiscolsum;
   $62 = ($61*3)|0;
   $63 = $nextcolsum;
   $64 = (($62) + ($63))|0;
   $65 = (($64) + 7)|0;
   $66 = $65 >> 4;
   $67 = $66&255;
   $68 = $outptr;
   $69 = (($68) + 1|0);
   $outptr = $69;
   HEAP8[$68>>0] = $67;
   $70 = $thiscolsum;
   $lastcolsum = $70;
   $71 = $nextcolsum;
   $thiscolsum = $71;
   $72 = $1;
   $73 = (($72) + 40|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) - 2)|0;
   $colctr = $75;
   while(1) {
    $76 = $colctr;
    $77 = ($76>>>0)>(0);
    if (!($77)) {
     break;
    }
    $78 = $inptr0;
    $79 = (($78) + 1|0);
    $inptr0 = $79;
    $80 = HEAP8[$78>>0]|0;
    $81 = $80&255;
    $82 = ($81*3)|0;
    $83 = $inptr1;
    $84 = (($83) + 1|0);
    $inptr1 = $84;
    $85 = HEAP8[$83>>0]|0;
    $86 = $85&255;
    $87 = (($82) + ($86))|0;
    $nextcolsum = $87;
    $88 = $thiscolsum;
    $89 = ($88*3)|0;
    $90 = $lastcolsum;
    $91 = (($89) + ($90))|0;
    $92 = (($91) + 8)|0;
    $93 = $92 >> 4;
    $94 = $93&255;
    $95 = $outptr;
    $96 = (($95) + 1|0);
    $outptr = $96;
    HEAP8[$95>>0] = $94;
    $97 = $thiscolsum;
    $98 = ($97*3)|0;
    $99 = $nextcolsum;
    $100 = (($98) + ($99))|0;
    $101 = (($100) + 7)|0;
    $102 = $101 >> 4;
    $103 = $102&255;
    $104 = $outptr;
    $105 = (($104) + 1|0);
    $outptr = $105;
    HEAP8[$104>>0] = $103;
    $106 = $thiscolsum;
    $lastcolsum = $106;
    $107 = $nextcolsum;
    $thiscolsum = $107;
    $108 = $colctr;
    $109 = (($108) + -1)|0;
    $colctr = $109;
   }
   $110 = $thiscolsum;
   $111 = ($110*3)|0;
   $112 = $lastcolsum;
   $113 = (($111) + ($112))|0;
   $114 = (($113) + 8)|0;
   $115 = $114 >> 4;
   $116 = $115&255;
   $117 = $outptr;
   $118 = (($117) + 1|0);
   $outptr = $118;
   HEAP8[$117>>0] = $116;
   $119 = $thiscolsum;
   $120 = $119<<2;
   $121 = (($120) + 7)|0;
   $122 = $121 >> 4;
   $123 = $122&255;
   $124 = $outptr;
   $125 = (($124) + 1|0);
   $outptr = $125;
   HEAP8[$124>>0] = $123;
   $126 = $v;
   $127 = (($126) + 1)|0;
   $v = $127;
  }
  $128 = $inrow;
  $129 = (($128) + 1)|0;
  $inrow = $129;
 }
 STACKTOP = sp;return;
}
function _h2v2_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inptr = 0, $inrow = 0, $invalue = 0, $outend = 0, $outptr = 0, $output_data = 0, $outrow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $outrow = 0;
 $inrow = 0;
 while(1) {
  $6 = $outrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $inrow;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $inptr = $14;
  $15 = $outrow;
  $16 = $output_data;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $outptr = $18;
  $19 = $outptr;
  $20 = $0;
  $21 = (($20) + 112|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) + ($22)|0);
  $outend = $23;
  while(1) {
   $24 = $outptr;
   $25 = $outend;
   $26 = ($24>>>0)<($25>>>0);
   if (!($26)) {
    break;
   }
   $27 = $inptr;
   $28 = (($27) + 1|0);
   $inptr = $28;
   $29 = HEAP8[$27>>0]|0;
   $invalue = $29;
   $30 = $invalue;
   $31 = $outptr;
   $32 = (($31) + 1|0);
   $outptr = $32;
   HEAP8[$31>>0] = $30;
   $33 = $invalue;
   $34 = $outptr;
   $35 = (($34) + 1|0);
   $outptr = $35;
   HEAP8[$34>>0] = $33;
  }
  $36 = $output_data;
  $37 = $outrow;
  $38 = $output_data;
  $39 = $outrow;
  $40 = (($39) + 1)|0;
  $41 = $0;
  $42 = (($41) + 112|0);
  $43 = HEAP32[$42>>2]|0;
  _jcopy_sample_rows($36,$37,$38,$40,1,$43);
  $44 = $inrow;
  $45 = (($44) + 1)|0;
  $inrow = $45;
  $46 = $outrow;
  $47 = (($46) + 2)|0;
  $outrow = $47;
 }
 STACKTOP = sp;return;
}
function _int_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $h = 0, $h_expand = 0, $inptr = 0, $inrow = 0, $invalue = 0, $outend = 0;
 var $outptr = 0, $output_data = 0, $outrow = 0, $upsample = 0, $v_expand = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $0;
 $5 = (($4) + 452|0);
 $6 = HEAP32[$5>>2]|0;
 $upsample = $6;
 $7 = $3;
 $8 = HEAP32[$7>>2]|0;
 $output_data = $8;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $upsample;
 $13 = (($12) + 140|0);
 $14 = (($13) + ($11)|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $h_expand = $16;
 $17 = $1;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $upsample;
 $21 = (($20) + 150|0);
 $22 = (($21) + ($19)|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $v_expand = $24;
 $outrow = 0;
 $inrow = 0;
 while(1) {
  $25 = $outrow;
  $26 = $0;
  $27 = (($26) + 312|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = $inrow;
  $31 = $2;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $inptr = $33;
  $34 = $outrow;
  $35 = $output_data;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $outptr = $37;
  $38 = $outptr;
  $39 = $0;
  $40 = (($39) + 112|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($38) + ($41)|0);
  $outend = $42;
  while(1) {
   $43 = $outptr;
   $44 = $outend;
   $45 = ($43>>>0)<($44>>>0);
   if (!($45)) {
    break;
   }
   $46 = $inptr;
   $47 = (($46) + 1|0);
   $inptr = $47;
   $48 = HEAP8[$46>>0]|0;
   $invalue = $48;
   $49 = $h_expand;
   $h = $49;
   while(1) {
    $50 = $h;
    $51 = ($50|0)>(0);
    if (!($51)) {
     break;
    }
    $52 = $invalue;
    $53 = $outptr;
    $54 = (($53) + 1|0);
    $outptr = $54;
    HEAP8[$53>>0] = $52;
    $55 = $h;
    $56 = (($55) + -1)|0;
    $h = $56;
   }
  }
  $57 = $v_expand;
  $58 = ($57|0)>(1);
  if ($58) {
   $59 = $output_data;
   $60 = $outrow;
   $61 = $output_data;
   $62 = $outrow;
   $63 = (($62) + 1)|0;
   $64 = $v_expand;
   $65 = (($64) - 1)|0;
   $66 = $0;
   $67 = (($66) + 112|0);
   $68 = HEAP32[$67>>2]|0;
   _jcopy_sample_rows($59,$60,$61,$63,$65,$68);
  }
  $69 = $inrow;
  $70 = (($69) + 1)|0;
  $inrow = $70;
  $71 = $v_expand;
  $72 = $outrow;
  $73 = (($72) + ($71))|0;
  $outrow = $73;
 }
 STACKTOP = sp;return;
}
function _jpeg_std_error($err) {
 $err = $err|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $err;
 $1 = $0;
 HEAP32[$1>>2] = 60;
 $2 = $0;
 $3 = (($2) + 4|0);
 HEAP32[$3>>2] = 61;
 $4 = $0;
 $5 = (($4) + 8|0);
 HEAP32[$5>>2] = 62;
 $6 = $0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 63;
 $8 = $0;
 $9 = (($8) + 16|0);
 HEAP32[$9>>2] = 64;
 $10 = $0;
 $11 = (($10) + 104|0);
 HEAP32[$11>>2] = 0;
 $12 = $0;
 $13 = (($12) + 108|0);
 HEAP32[$13>>2] = 0;
 $14 = $0;
 $15 = (($14) + 20|0);
 HEAP32[$15>>2] = 0;
 $16 = $0;
 $17 = (($16) + 112|0);
 HEAP32[$17>>2] = 8688;
 $18 = $0;
 $19 = (($18) + 116|0);
 HEAP32[$19>>2] = 123;
 $20 = $0;
 $21 = (($20) + 120|0);
 HEAP32[$21>>2] = 0;
 $22 = $0;
 $23 = (($22) + 124|0);
 HEAP32[$23>>2] = 0;
 $24 = $0;
 $25 = (($24) + 128|0);
 HEAP32[$25>>2] = 0;
 $26 = $0;
 STACKTOP = sp;return ($26|0);
}
function _error_exit($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 FUNCTION_TABLE_vi[$4 & 127]($5);
 $6 = $0;
 _jpeg_destroy($6);
 _exit(1);
 // unreachable;
}
function _emit_message($cinfo,$msg_level) {
 $cinfo = $cinfo|0;
 $msg_level = $msg_level|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $err = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $msg_level;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $err = $3;
 $4 = $1;
 $5 = ($4|0)<(0);
 if (!($5)) {
  $22 = $err;
  $23 = (($22) + 104|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $1;
  $26 = ($24|0)>=($25|0);
  if ($26) {
   $27 = $err;
   $28 = (($27) + 8|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $0;
   FUNCTION_TABLE_vi[$29 & 127]($30);
  }
  STACKTOP = sp;return;
 }
 $6 = $err;
 $7 = (($6) + 108|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  label = 4;
 } else {
  $10 = $err;
  $11 = (($10) + 104|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>=(3);
  if ($13) {
   label = 4;
  }
 }
 if ((label|0) == 4) {
  $14 = $err;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $0;
  FUNCTION_TABLE_vi[$16 & 127]($17);
 }
 $18 = $err;
 $19 = (($18) + 108|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _output_message($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buffer = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer = sp;
 $buffer = sp + 8|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 FUNCTION_TABLE_vii[$4 & 127]($5,$buffer);
 $6 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer>>2] = $buffer;
 (_fprintf(($6|0),(9192|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return;
}
function _format_message($cinfo,$buffer) {
 $cinfo = $cinfo|0;
 $buffer = $buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ch = 0, $err = 0, $isstring = 0, $msg_code = 0, $msgptr = 0, $msgtext = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr10 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $cinfo;
 $1 = $buffer;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $err = $3;
 $4 = $err;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $msg_code = $6;
 $msgtext = 0;
 $7 = $msg_code;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = $msg_code;
  $10 = $err;
  $11 = (($10) + 116|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<=($12|0);
  if ($13) {
   $14 = $msg_code;
   $15 = $err;
   $16 = (($15) + 112|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + ($14<<2)|0);
   $19 = HEAP32[$18>>2]|0;
   $msgtext = $19;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $20 = $err;
  $21 = (($20) + 120|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = $msg_code;
   $25 = $err;
   $26 = (($25) + 124|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($24|0)>=($27|0);
   if ($28) {
    $29 = $msg_code;
    $30 = $err;
    $31 = (($30) + 128|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($29|0)<=($32|0);
    if ($33) {
     $34 = $msg_code;
     $35 = $err;
     $36 = (($35) + 124|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = (($34) - ($37))|0;
     $39 = $err;
     $40 = (($39) + 120|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($38<<2)|0);
     $43 = HEAP32[$42>>2]|0;
     $msgtext = $43;
    }
   }
  }
 }
 $44 = $msgtext;
 $45 = ($44|0)==(0|0);
 if ($45) {
  $46 = $msg_code;
  $47 = $err;
  $48 = (($47) + 24|0);
  HEAP32[$48>>2] = $46;
  $49 = $err;
  $50 = (($49) + 112|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $msgtext = $52;
 }
 $isstring = 0;
 $53 = $msgtext;
 $msgptr = $53;
 while(1) {
  $54 = $msgptr;
  $55 = (($54) + 1|0);
  $msgptr = $55;
  $56 = HEAP8[$54>>0]|0;
  $ch = $56;
  $57 = $56 << 24 >> 24;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   break;
  }
  $59 = $ch;
  $60 = $59 << 24 >> 24;
  $61 = ($60|0)==(37);
  if ($61) {
   label = 14;
   break;
  }
 }
 if ((label|0) == 14) {
  $62 = $msgptr;
  $63 = HEAP8[$62>>0]|0;
  $64 = $63 << 24 >> 24;
  $65 = ($64|0)==(115);
  if ($65) {
   $isstring = 1;
  }
 }
 $66 = $isstring;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = $1;
  $69 = $msgtext;
  $70 = $err;
  $71 = (($70) + 24|0);
  HEAP32[$vararg_buffer>>2] = $71;
  (_sprintf($68,$69,$vararg_buffer)|0);
  STACKTOP = sp;return;
 } else {
  $72 = $1;
  $73 = $msgtext;
  $74 = $err;
  $75 = (($74) + 24|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $err;
  $78 = (($77) + 24|0);
  $79 = (($78) + 4|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = $err;
  $82 = (($81) + 24|0);
  $83 = (($82) + 8|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = $err;
  $86 = (($85) + 24|0);
  $87 = (($86) + 12|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $err;
  $90 = (($89) + 24|0);
  $91 = (($90) + 16|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = $err;
  $94 = (($93) + 24|0);
  $95 = (($94) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = $err;
  $98 = (($97) + 24|0);
  $99 = (($98) + 24|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = $err;
  $102 = (($101) + 24|0);
  $103 = (($102) + 28|0);
  $104 = HEAP32[$103>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $76;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $80;
  $vararg_ptr5 = (($vararg_buffer1) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $84;
  $vararg_ptr6 = (($vararg_buffer1) + 12|0);
  HEAP32[$vararg_ptr6>>2] = $88;
  $vararg_ptr7 = (($vararg_buffer1) + 16|0);
  HEAP32[$vararg_ptr7>>2] = $92;
  $vararg_ptr8 = (($vararg_buffer1) + 20|0);
  HEAP32[$vararg_ptr8>>2] = $96;
  $vararg_ptr9 = (($vararg_buffer1) + 24|0);
  HEAP32[$vararg_ptr9>>2] = $100;
  $vararg_ptr10 = (($vararg_buffer1) + 28|0);
  HEAP32[$vararg_ptr10>>2] = $104;
  (_sprintf($72,$73,$vararg_buffer1)|0);
  STACKTOP = sp;return;
 }
}
function _reset_error_mgr($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 108|0);
 HEAP32[$3>>2] = 0;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 20|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _jinit_memory_mgr($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $ch = 0, $max_to_use = 0, $mem = 0, $memenv = 0, $pool = 0, $test_mac = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $max_to_use = sp + 20|0;
 $ch = sp + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 HEAP32[$2>>2] = 0;
 $test_mac = 1000000000;
 $3 = $test_mac;
 $4 = ($3|0)!=(1000000000);
 if ($4) {
  $5 = $0;
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) + 20|0);
  HEAP32[$7>>2] = 3;
  $8 = $0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  FUNCTION_TABLE_vi[$10 & 127]($11);
 }
 $12 = $0;
 $13 = (_jpeg_mem_init($12)|0);
 HEAP32[$max_to_use>>2] = $13;
 $14 = $0;
 $15 = (_jpeg_get_small($14,84)|0);
 $mem = $15;
 $16 = $mem;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = $0;
  _jpeg_mem_term($18);
  $19 = $0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 54;
  $22 = $0;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 24|0);
  HEAP32[$24>>2] = 0;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  $28 = $0;
  FUNCTION_TABLE_vi[$27 & 127]($28);
 }
 $29 = $mem;
 HEAP32[$29>>2] = 65;
 $30 = $mem;
 $31 = (($30) + 4|0);
 HEAP32[$31>>2] = 66;
 $32 = $mem;
 $33 = (($32) + 8|0);
 HEAP32[$33>>2] = 67;
 $34 = $mem;
 $35 = (($34) + 12|0);
 HEAP32[$35>>2] = 68;
 $36 = $mem;
 $37 = (($36) + 16|0);
 HEAP32[$37>>2] = 69;
 $38 = $mem;
 $39 = (($38) + 20|0);
 HEAP32[$39>>2] = 70;
 $40 = $mem;
 $41 = (($40) + 24|0);
 HEAP32[$41>>2] = 71;
 $42 = $mem;
 $43 = (($42) + 28|0);
 HEAP32[$43>>2] = 72;
 $44 = $mem;
 $45 = (($44) + 32|0);
 HEAP32[$45>>2] = 73;
 $46 = $mem;
 $47 = (($46) + 36|0);
 HEAP32[$47>>2] = 74;
 $48 = $mem;
 $49 = (($48) + 40|0);
 HEAP32[$49>>2] = 75;
 $50 = $mem;
 $51 = (($50) + 48|0);
 HEAP32[$51>>2] = 1000000000;
 $52 = HEAP32[$max_to_use>>2]|0;
 $53 = $mem;
 $54 = (($53) + 44|0);
 HEAP32[$54>>2] = $52;
 $pool = 1;
 while(1) {
  $55 = $pool;
  $56 = ($55|0)>=(0);
  if (!($56)) {
   break;
  }
  $57 = $pool;
  $58 = $mem;
  $59 = (($58) + 52|0);
  $60 = (($59) + ($57<<2)|0);
  HEAP32[$60>>2] = 0;
  $61 = $pool;
  $62 = $mem;
  $63 = (($62) + 60|0);
  $64 = (($63) + ($61<<2)|0);
  HEAP32[$64>>2] = 0;
  $65 = $pool;
  $66 = (($65) + -1)|0;
  $pool = $66;
 }
 $67 = $mem;
 $68 = (($67) + 68|0);
 HEAP32[$68>>2] = 0;
 $69 = $mem;
 $70 = (($69) + 72|0);
 HEAP32[$70>>2] = 0;
 $71 = $mem;
 $72 = (($71) + 76|0);
 HEAP32[$72>>2] = 84;
 $73 = $mem;
 $74 = $0;
 $75 = (($74) + 4|0);
 HEAP32[$75>>2] = $73;
 $76 = (_getenv((9200|0))|0);
 $memenv = $76;
 $77 = ($76|0)!=(0|0);
 if (!($77)) {
  STACKTOP = sp;return;
 }
 HEAP8[$ch>>0] = 120;
 $78 = $memenv;
 HEAP32[$vararg_buffer>>2] = $max_to_use;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $ch;
 $79 = (_sscanf($78,9208,$vararg_buffer)|0);
 $80 = ($79|0)>(0);
 if ($80) {
  $81 = HEAP8[$ch>>0]|0;
  $82 = $81 << 24 >> 24;
  $83 = ($82|0)==(109);
  if ($83) {
   label = 13;
  } else {
   $84 = HEAP8[$ch>>0]|0;
   $85 = $84 << 24 >> 24;
   $86 = ($85|0)==(77);
   if ($86) {
    label = 13;
   }
  }
  if ((label|0) == 13) {
   $87 = HEAP32[$max_to_use>>2]|0;
   $88 = ($87*1000)|0;
   HEAP32[$max_to_use>>2] = $88;
  }
  $89 = HEAP32[$max_to_use>>2]|0;
  $90 = ($89*1000)|0;
  $91 = $mem;
  $92 = (($91) + 44|0);
  HEAP32[$92>>2] = $90;
 }
 STACKTOP = sp;return;
}
function _alloc_small($cinfo,$pool_id,$sizeofobject) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data_ptr = 0;
 var $hdr_ptr = 0, $mem = 0, $min_request = 0, $odd_bytes = 0, $prev_hdr_ptr = 0, $slop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $sizeofobject;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $mem = $5;
 $6 = $2;
 $7 = ($6>>>0)>(999999984);
 if ($7) {
  $8 = $0;
  _out_of_memory($8,1);
 }
 $9 = $2;
 $10 = (($9>>>0) % 8)&-1;
 $odd_bytes = $10;
 $11 = $odd_bytes;
 $12 = ($11>>>0)>(0);
 if ($12) {
  $13 = $odd_bytes;
  $14 = (8 - ($13))|0;
  $15 = $2;
  $16 = (($15) + ($14))|0;
  $2 = $16;
 }
 $17 = $1;
 $18 = ($17|0)<(0);
 if ($18) {
  label = 7;
 } else {
  $19 = $1;
  $20 = ($19|0)>=(2);
  if ($20) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 14;
  $24 = $1;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 24|0);
  HEAP32[$27>>2] = $24;
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vi[$30 & 127]($31);
 }
 $prev_hdr_ptr = 0;
 $32 = $1;
 $33 = $mem;
 $34 = (($33) + 52|0);
 $35 = (($34) + ($32<<2)|0);
 $36 = HEAP32[$35>>2]|0;
 $hdr_ptr = $36;
 while(1) {
  $37 = $hdr_ptr;
  $38 = ($37|0)!=(0|0);
  if (!($38)) {
   break;
  }
  $39 = $hdr_ptr;
  $40 = (($39) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $2;
  $43 = ($41>>>0)>=($42>>>0);
  if ($43) {
   label = 11;
   break;
  }
  $44 = $hdr_ptr;
  $prev_hdr_ptr = $44;
  $45 = $hdr_ptr;
  $46 = HEAP32[$45>>2]|0;
  $hdr_ptr = $46;
 }
 if ((label|0) == 11) {
 }
 $47 = $hdr_ptr;
 $48 = ($47|0)==(0|0);
 if (!($48)) {
  $101 = $hdr_ptr;
  $102 = (($101) + 16|0);
  $data_ptr = $102;
  $103 = $hdr_ptr;
  $104 = (($103) + 4|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = $data_ptr;
  $107 = (($106) + ($105)|0);
  $data_ptr = $107;
  $108 = $2;
  $109 = $hdr_ptr;
  $110 = (($109) + 4|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (($111) + ($108))|0;
  HEAP32[$110>>2] = $112;
  $113 = $2;
  $114 = $hdr_ptr;
  $115 = (($114) + 8|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (($116) - ($113))|0;
  HEAP32[$115>>2] = $117;
  $118 = $data_ptr;
  STACKTOP = sp;return ($118|0);
 }
 $49 = $2;
 $50 = (($49) + 16)|0;
 $min_request = $50;
 $51 = $prev_hdr_ptr;
 $52 = ($51|0)==(0|0);
 if ($52) {
  $53 = $1;
  $54 = (9216 + ($53<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $slop = $55;
 } else {
  $56 = $1;
  $57 = (9224 + ($56<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $slop = $58;
 }
 $59 = $slop;
 $60 = $min_request;
 $61 = (1000000000 - ($60))|0;
 $62 = ($59>>>0)>($61>>>0);
 if ($62) {
  $63 = $min_request;
  $64 = (1000000000 - ($63))|0;
  $slop = $64;
 }
 while(1) {
  $65 = $0;
  $66 = $min_request;
  $67 = $slop;
  $68 = (($66) + ($67))|0;
  $69 = (_jpeg_get_small($65,$68)|0);
  $hdr_ptr = $69;
  $70 = $hdr_ptr;
  $71 = ($70|0)!=(0|0);
  if ($71) {
   break;
  }
  $72 = $slop;
  $73 = (($72>>>0) / 2)&-1;
  $slop = $73;
  $74 = $slop;
  $75 = ($74>>>0)<(50);
  if ($75) {
   $76 = $0;
   _out_of_memory($76,2);
  }
 }
 $77 = $min_request;
 $78 = $slop;
 $79 = (($77) + ($78))|0;
 $80 = $mem;
 $81 = (($80) + 76|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = (($82) + ($79))|0;
 HEAP32[$81>>2] = $83;
 $84 = $hdr_ptr;
 HEAP32[$84>>2] = 0;
 $85 = $hdr_ptr;
 $86 = (($85) + 4|0);
 HEAP32[$86>>2] = 0;
 $87 = $2;
 $88 = $slop;
 $89 = (($87) + ($88))|0;
 $90 = $hdr_ptr;
 $91 = (($90) + 8|0);
 HEAP32[$91>>2] = $89;
 $92 = $prev_hdr_ptr;
 $93 = ($92|0)==(0|0);
 if ($93) {
  $94 = $hdr_ptr;
  $95 = $1;
  $96 = $mem;
  $97 = (($96) + 52|0);
  $98 = (($97) + ($95<<2)|0);
  HEAP32[$98>>2] = $94;
 } else {
  $99 = $hdr_ptr;
  $100 = $prev_hdr_ptr;
  HEAP32[$100>>2] = $99;
 }
 $101 = $hdr_ptr;
 $102 = (($101) + 16|0);
 $data_ptr = $102;
 $103 = $hdr_ptr;
 $104 = (($103) + 4|0);
 $105 = HEAP32[$104>>2]|0;
 $106 = $data_ptr;
 $107 = (($106) + ($105)|0);
 $data_ptr = $107;
 $108 = $2;
 $109 = $hdr_ptr;
 $110 = (($109) + 4|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = (($111) + ($108))|0;
 HEAP32[$110>>2] = $112;
 $113 = $2;
 $114 = $hdr_ptr;
 $115 = (($114) + 8|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = (($116) - ($113))|0;
 HEAP32[$115>>2] = $117;
 $118 = $data_ptr;
 STACKTOP = sp;return ($118|0);
}
function _alloc_large($cinfo,$pool_id,$sizeofobject) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $hdr_ptr = 0, $mem = 0, $odd_bytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $sizeofobject;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $mem = $5;
 $6 = $2;
 $7 = ($6>>>0)>(999999984);
 if ($7) {
  $8 = $0;
  _out_of_memory($8,3);
 }
 $9 = $2;
 $10 = (($9>>>0) % 8)&-1;
 $odd_bytes = $10;
 $11 = $odd_bytes;
 $12 = ($11>>>0)>(0);
 if ($12) {
  $13 = $odd_bytes;
  $14 = (8 - ($13))|0;
  $15 = $2;
  $16 = (($15) + ($14))|0;
  $2 = $16;
 }
 $17 = $1;
 $18 = ($17|0)<(0);
 if ($18) {
  label = 7;
 } else {
  $19 = $1;
  $20 = ($19|0)>=(2);
  if ($20) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 14;
  $24 = $1;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 24|0);
  HEAP32[$27>>2] = $24;
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vi[$30 & 127]($31);
 }
 $32 = $0;
 $33 = $2;
 $34 = (($33) + 16)|0;
 $35 = (_jpeg_get_large($32,$34)|0);
 $hdr_ptr = $35;
 $36 = $hdr_ptr;
 $37 = ($36|0)==(0|0);
 if ($37) {
  $38 = $0;
  _out_of_memory($38,4);
 }
 $39 = $2;
 $40 = (($39) + 16)|0;
 $41 = $mem;
 $42 = (($41) + 76|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + ($40))|0;
 HEAP32[$42>>2] = $44;
 $45 = $1;
 $46 = $mem;
 $47 = (($46) + 60|0);
 $48 = (($47) + ($45<<2)|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $hdr_ptr;
 HEAP32[$50>>2] = $49;
 $51 = $2;
 $52 = $hdr_ptr;
 $53 = (($52) + 4|0);
 HEAP32[$53>>2] = $51;
 $54 = $hdr_ptr;
 $55 = (($54) + 8|0);
 HEAP32[$55>>2] = 0;
 $56 = $hdr_ptr;
 $57 = $1;
 $58 = $mem;
 $59 = (($58) + 60|0);
 $60 = (($59) + ($57<<2)|0);
 HEAP32[$60>>2] = $56;
 $61 = $hdr_ptr;
 $62 = (($61) + 16|0);
 STACKTOP = sp;return ($62|0);
}
function _alloc_sarray($cinfo,$pool_id,$samplesperrow,$numrows) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $samplesperrow = $samplesperrow|0;
 $numrows = $numrows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $currow = 0, $i = 0, $ltemp = 0, $mem = 0, $result = 0, $rowsperchunk = 0, $workspace = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $samplesperrow;
 $3 = $numrows;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $mem = $6;
 $7 = $2;
 $8 = $7;
 $9 = (999999984 / ($8>>>0))&-1;
 $ltemp = $9;
 $10 = $ltemp;
 $11 = ($10|0)<=(0);
 if ($11) {
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = 70;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 127]($18);
 }
 $19 = $ltemp;
 $20 = $3;
 $21 = ($19|0)<($20|0);
 if ($21) {
  $22 = $ltemp;
  $rowsperchunk = $22;
 } else {
  $23 = $3;
  $rowsperchunk = $23;
 }
 $24 = $rowsperchunk;
 $25 = $mem;
 $26 = (($25) + 80|0);
 HEAP32[$26>>2] = $24;
 $27 = $0;
 $28 = $1;
 $29 = $3;
 $30 = $29<<2;
 $31 = (_alloc_small($27,$28,$30)|0);
 $result = $31;
 $currow = 0;
 while(1) {
  $32 = $currow;
  $33 = $3;
  $34 = ($32>>>0)<($33>>>0);
  if (!($34)) {
   break;
  }
  $35 = $rowsperchunk;
  $36 = $3;
  $37 = $currow;
  $38 = (($36) - ($37))|0;
  $39 = ($35>>>0)<($38>>>0);
  if ($39) {
   $40 = $rowsperchunk;
   $44 = $40;
  } else {
   $41 = $3;
   $42 = $currow;
   $43 = (($41) - ($42))|0;
   $44 = $43;
  }
  $rowsperchunk = $44;
  $45 = $0;
  $46 = $1;
  $47 = $rowsperchunk;
  $48 = $2;
  $49 = Math_imul($47, $48)|0;
  $50 = $49;
  $51 = (_alloc_large($45,$46,$50)|0);
  $workspace = $51;
  $52 = $rowsperchunk;
  $i = $52;
  while(1) {
   $53 = $i;
   $54 = ($53>>>0)>(0);
   if (!($54)) {
    break;
   }
   $55 = $workspace;
   $56 = $currow;
   $57 = (($56) + 1)|0;
   $currow = $57;
   $58 = $result;
   $59 = (($58) + ($56<<2)|0);
   HEAP32[$59>>2] = $55;
   $60 = $2;
   $61 = $workspace;
   $62 = (($61) + ($60)|0);
   $workspace = $62;
   $63 = $i;
   $64 = (($63) + -1)|0;
   $i = $64;
  }
 }
 $65 = $result;
 STACKTOP = sp;return ($65|0);
}
function _alloc_barray($cinfo,$pool_id,$blocksperrow,$numrows) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $blocksperrow = $blocksperrow|0;
 $numrows = $numrows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $currow = 0, $i = 0, $ltemp = 0, $mem = 0, $result = 0, $rowsperchunk = 0, $workspace = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $blocksperrow;
 $3 = $numrows;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $mem = $6;
 $7 = $2;
 $8 = $7<<7;
 $9 = (999999984 / ($8>>>0))&-1;
 $ltemp = $9;
 $10 = $ltemp;
 $11 = ($10|0)<=(0);
 if ($11) {
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = 70;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 127]($18);
 }
 $19 = $ltemp;
 $20 = $3;
 $21 = ($19|0)<($20|0);
 if ($21) {
  $22 = $ltemp;
  $rowsperchunk = $22;
 } else {
  $23 = $3;
  $rowsperchunk = $23;
 }
 $24 = $rowsperchunk;
 $25 = $mem;
 $26 = (($25) + 80|0);
 HEAP32[$26>>2] = $24;
 $27 = $0;
 $28 = $1;
 $29 = $3;
 $30 = $29<<2;
 $31 = (_alloc_small($27,$28,$30)|0);
 $result = $31;
 $currow = 0;
 while(1) {
  $32 = $currow;
  $33 = $3;
  $34 = ($32>>>0)<($33>>>0);
  if (!($34)) {
   break;
  }
  $35 = $rowsperchunk;
  $36 = $3;
  $37 = $currow;
  $38 = (($36) - ($37))|0;
  $39 = ($35>>>0)<($38>>>0);
  if ($39) {
   $40 = $rowsperchunk;
   $44 = $40;
  } else {
   $41 = $3;
   $42 = $currow;
   $43 = (($41) - ($42))|0;
   $44 = $43;
  }
  $rowsperchunk = $44;
  $45 = $0;
  $46 = $1;
  $47 = $rowsperchunk;
  $48 = $2;
  $49 = Math_imul($47, $48)|0;
  $50 = $49<<7;
  $51 = (_alloc_large($45,$46,$50)|0);
  $workspace = $51;
  $52 = $rowsperchunk;
  $i = $52;
  while(1) {
   $53 = $i;
   $54 = ($53>>>0)>(0);
   if (!($54)) {
    break;
   }
   $55 = $workspace;
   $56 = $currow;
   $57 = (($56) + 1)|0;
   $currow = $57;
   $58 = $result;
   $59 = (($58) + ($56<<2)|0);
   HEAP32[$59>>2] = $55;
   $60 = $2;
   $61 = $workspace;
   $62 = (($61) + ($60<<7)|0);
   $workspace = $62;
   $63 = $i;
   $64 = (($63) + -1)|0;
   $i = $64;
  }
 }
 $65 = $result;
 STACKTOP = sp;return ($65|0);
}
function _request_virt_sarray($cinfo,$pool_id,$pre_zero,$samplesperrow,$numrows,$maxaccess) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $pre_zero = $pre_zero|0;
 $samplesperrow = $samplesperrow|0;
 $numrows = $numrows|0;
 $maxaccess = $maxaccess|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mem = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $pre_zero;
 $3 = $samplesperrow;
 $4 = $numrows;
 $5 = $maxaccess;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mem = $8;
 $9 = $1;
 $10 = ($9|0)!=(1);
 if ($10) {
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 20|0);
  HEAP32[$13>>2] = 14;
  $14 = $1;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 24|0);
  HEAP32[$17>>2] = $14;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 127]($21);
 }
 $22 = $0;
 $23 = $1;
 $24 = (_alloc_small($22,$23,128)|0);
 $result = $24;
 $25 = $result;
 HEAP32[$25>>2] = 0;
 $26 = $4;
 $27 = $result;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = $3;
 $30 = $result;
 $31 = (($30) + 8|0);
 HEAP32[$31>>2] = $29;
 $32 = $5;
 $33 = $result;
 $34 = (($33) + 12|0);
 HEAP32[$34>>2] = $32;
 $35 = $2;
 $36 = $result;
 $37 = (($36) + 32|0);
 HEAP32[$37>>2] = $35;
 $38 = $result;
 $39 = (($38) + 40|0);
 HEAP32[$39>>2] = 0;
 $40 = $mem;
 $41 = (($40) + 68|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $result;
 $44 = (($43) + 44|0);
 HEAP32[$44>>2] = $42;
 $45 = $result;
 $46 = $mem;
 $47 = (($46) + 68|0);
 HEAP32[$47>>2] = $45;
 $48 = $result;
 STACKTOP = sp;return ($48|0);
}
function _request_virt_barray($cinfo,$pool_id,$pre_zero,$blocksperrow,$numrows,$maxaccess) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $pre_zero = $pre_zero|0;
 $blocksperrow = $blocksperrow|0;
 $numrows = $numrows|0;
 $maxaccess = $maxaccess|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mem = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $pre_zero;
 $3 = $blocksperrow;
 $4 = $numrows;
 $5 = $maxaccess;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mem = $8;
 $9 = $1;
 $10 = ($9|0)!=(1);
 if ($10) {
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 20|0);
  HEAP32[$13>>2] = 14;
  $14 = $1;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 24|0);
  HEAP32[$17>>2] = $14;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 127]($21);
 }
 $22 = $0;
 $23 = $1;
 $24 = (_alloc_small($22,$23,128)|0);
 $result = $24;
 $25 = $result;
 HEAP32[$25>>2] = 0;
 $26 = $4;
 $27 = $result;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = $3;
 $30 = $result;
 $31 = (($30) + 8|0);
 HEAP32[$31>>2] = $29;
 $32 = $5;
 $33 = $result;
 $34 = (($33) + 12|0);
 HEAP32[$34>>2] = $32;
 $35 = $2;
 $36 = $result;
 $37 = (($36) + 32|0);
 HEAP32[$37>>2] = $35;
 $38 = $result;
 $39 = (($38) + 40|0);
 HEAP32[$39>>2] = 0;
 $40 = $mem;
 $41 = (($40) + 72|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $result;
 $44 = (($43) + 44|0);
 HEAP32[$44>>2] = $42;
 $45 = $result;
 $46 = $mem;
 $47 = (($46) + 72|0);
 HEAP32[$47>>2] = $45;
 $48 = $result;
 STACKTOP = sp;return ($48|0);
}
function _realize_virt_arrays($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $avail_mem = 0;
 var $bptr = 0, $max_minheights = 0, $maximum_space = 0, $mem = 0, $minheights = 0, $space_per_minheight = 0, $sptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $mem = $3;
 $space_per_minheight = 0;
 $maximum_space = 0;
 $4 = $mem;
 $5 = (($4) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $sptr = $6;
 while(1) {
  $7 = $sptr;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = $sptr;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   $12 = $sptr;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $sptr;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = Math_imul($14, $17)|0;
   $19 = $18;
   $20 = $space_per_minheight;
   $21 = (($20) + ($19))|0;
   $space_per_minheight = $21;
   $22 = $sptr;
   $23 = (($22) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = $sptr;
   $26 = (($25) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = Math_imul($24, $27)|0;
   $29 = $28;
   $30 = $maximum_space;
   $31 = (($30) + ($29))|0;
   $maximum_space = $31;
  }
  $32 = $sptr;
  $33 = (($32) + 44|0);
  $34 = HEAP32[$33>>2]|0;
  $sptr = $34;
 }
 $35 = $mem;
 $36 = (($35) + 72|0);
 $37 = HEAP32[$36>>2]|0;
 $bptr = $37;
 while(1) {
  $38 = $bptr;
  $39 = ($38|0)!=(0|0);
  if (!($39)) {
   break;
  }
  $40 = $bptr;
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)==(0|0);
  if ($42) {
   $43 = $bptr;
   $44 = (($43) + 12|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = $bptr;
   $47 = (($46) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = Math_imul($45, $48)|0;
   $50 = $49<<7;
   $51 = $space_per_minheight;
   $52 = (($51) + ($50))|0;
   $space_per_minheight = $52;
   $53 = $bptr;
   $54 = (($53) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = $bptr;
   $57 = (($56) + 8|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = Math_imul($55, $58)|0;
   $60 = $59<<7;
   $61 = $maximum_space;
   $62 = (($61) + ($60))|0;
   $maximum_space = $62;
  }
  $63 = $bptr;
  $64 = (($63) + 44|0);
  $65 = HEAP32[$64>>2]|0;
  $bptr = $65;
 }
 $66 = $space_per_minheight;
 $67 = ($66|0)<=(0);
 if ($67) {
  STACKTOP = sp;return;
 }
 $68 = $0;
 $69 = $space_per_minheight;
 $70 = $maximum_space;
 $71 = $mem;
 $72 = (($71) + 76|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = (_jpeg_mem_available($68,$69,$70,$73)|0);
 $avail_mem = $74;
 $75 = $avail_mem;
 $76 = $maximum_space;
 $77 = ($75|0)>=($76|0);
 if ($77) {
  $max_minheights = 1000000000;
 } else {
  $78 = $avail_mem;
  $79 = $space_per_minheight;
  $80 = (($78|0) / ($79|0))&-1;
  $max_minheights = $80;
  $81 = $max_minheights;
  $82 = ($81|0)<=(0);
  if ($82) {
   $max_minheights = 1;
  }
 }
 $83 = $mem;
 $84 = (($83) + 68|0);
 $85 = HEAP32[$84>>2]|0;
 $sptr = $85;
 while(1) {
  $86 = $sptr;
  $87 = ($86|0)!=(0|0);
  if (!($87)) {
   break;
  }
  $88 = $sptr;
  $89 = HEAP32[$88>>2]|0;
  $90 = ($89|0)==(0|0);
  if ($90) {
   $91 = $sptr;
   $92 = (($91) + 4|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (($93) - 1)|0;
   $95 = $sptr;
   $96 = (($95) + 12|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = (($94>>>0) / ($97>>>0))&-1;
   $99 = (($98) + 1)|0;
   $minheights = $99;
   $100 = $minheights;
   $101 = $max_minheights;
   $102 = ($100|0)<=($101|0);
   if ($102) {
    $103 = $sptr;
    $104 = (($103) + 4|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $sptr;
    $107 = (($106) + 16|0);
    HEAP32[$107>>2] = $105;
   } else {
    $108 = $max_minheights;
    $109 = $sptr;
    $110 = (($109) + 12|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = Math_imul($108, $111)|0;
    $113 = $sptr;
    $114 = (($113) + 16|0);
    HEAP32[$114>>2] = $112;
    $115 = $0;
    $116 = $sptr;
    $117 = (($116) + 48|0);
    $118 = $sptr;
    $119 = (($118) + 4|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = $sptr;
    $122 = (($121) + 8|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = Math_imul($120, $123)|0;
    $125 = $124;
    _jpeg_open_backing_store($115,$117,$125);
    $126 = $sptr;
    $127 = (($126) + 40|0);
    HEAP32[$127>>2] = 1;
   }
   $128 = $0;
   $129 = $sptr;
   $130 = (($129) + 8|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = $sptr;
   $133 = (($132) + 16|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = (_alloc_sarray($128,1,$131,$134)|0);
   $136 = $sptr;
   HEAP32[$136>>2] = $135;
   $137 = $mem;
   $138 = (($137) + 80|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $sptr;
   $141 = (($140) + 20|0);
   HEAP32[$141>>2] = $139;
   $142 = $sptr;
   $143 = (($142) + 24|0);
   HEAP32[$143>>2] = 0;
   $144 = $sptr;
   $145 = (($144) + 28|0);
   HEAP32[$145>>2] = 0;
   $146 = $sptr;
   $147 = (($146) + 36|0);
   HEAP32[$147>>2] = 0;
  }
  $148 = $sptr;
  $149 = (($148) + 44|0);
  $150 = HEAP32[$149>>2]|0;
  $sptr = $150;
 }
 $151 = $mem;
 $152 = (($151) + 72|0);
 $153 = HEAP32[$152>>2]|0;
 $bptr = $153;
 while(1) {
  $154 = $bptr;
  $155 = ($154|0)!=(0|0);
  if (!($155)) {
   break;
  }
  $156 = $bptr;
  $157 = HEAP32[$156>>2]|0;
  $158 = ($157|0)==(0|0);
  if ($158) {
   $159 = $bptr;
   $160 = (($159) + 4|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = (($161) - 1)|0;
   $163 = $bptr;
   $164 = (($163) + 12|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = (($162>>>0) / ($165>>>0))&-1;
   $167 = (($166) + 1)|0;
   $minheights = $167;
   $168 = $minheights;
   $169 = $max_minheights;
   $170 = ($168|0)<=($169|0);
   if ($170) {
    $171 = $bptr;
    $172 = (($171) + 4|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = $bptr;
    $175 = (($174) + 16|0);
    HEAP32[$175>>2] = $173;
   } else {
    $176 = $max_minheights;
    $177 = $bptr;
    $178 = (($177) + 12|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = Math_imul($176, $179)|0;
    $181 = $bptr;
    $182 = (($181) + 16|0);
    HEAP32[$182>>2] = $180;
    $183 = $0;
    $184 = $bptr;
    $185 = (($184) + 48|0);
    $186 = $bptr;
    $187 = (($186) + 4|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = $bptr;
    $190 = (($189) + 8|0);
    $191 = HEAP32[$190>>2]|0;
    $192 = Math_imul($188, $191)|0;
    $193 = $192<<7;
    _jpeg_open_backing_store($183,$185,$193);
    $194 = $bptr;
    $195 = (($194) + 40|0);
    HEAP32[$195>>2] = 1;
   }
   $196 = $0;
   $197 = $bptr;
   $198 = (($197) + 8|0);
   $199 = HEAP32[$198>>2]|0;
   $200 = $bptr;
   $201 = (($200) + 16|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (_alloc_barray($196,1,$199,$202)|0);
   $204 = $bptr;
   HEAP32[$204>>2] = $203;
   $205 = $mem;
   $206 = (($205) + 80|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = $bptr;
   $209 = (($208) + 20|0);
   HEAP32[$209>>2] = $207;
   $210 = $bptr;
   $211 = (($210) + 24|0);
   HEAP32[$211>>2] = 0;
   $212 = $bptr;
   $213 = (($212) + 28|0);
   HEAP32[$213>>2] = 0;
   $214 = $bptr;
   $215 = (($214) + 36|0);
   HEAP32[$215>>2] = 0;
  }
  $216 = $bptr;
  $217 = (($216) + 44|0);
  $218 = HEAP32[$217>>2]|0;
  $bptr = $218;
 }
 STACKTOP = sp;return;
}
function _access_virt_sarray($cinfo,$ptr,$start_row,$num_rows,$writable) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $start_row = $start_row|0;
 $num_rows = $num_rows|0;
 $writable = $writable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytesperrow = 0;
 var $end_row = 0, $ltemp = 0, $undef_row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $start_row;
 $3 = $num_rows;
 $4 = $writable;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6))|0;
 $end_row = $7;
 $8 = $end_row;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 if ($12) {
  label = 4;
 } else {
  $13 = $3;
  $14 = $1;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13>>>0)>($16>>>0);
  if ($17) {
   label = 4;
  } else {
   $18 = $1;
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    label = 4;
   }
  }
 }
 if ((label|0) == 4) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 22;
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vi[$26 & 127]($27);
 }
 $28 = $2;
 $29 = $1;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($28>>>0)<($31>>>0);
 if ($32) {
  label = 7;
 } else {
  $33 = $end_row;
  $34 = $1;
  $35 = (($34) + 24|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $1;
  $38 = (($37) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($36) + ($39))|0;
  $41 = ($33>>>0)>($40>>>0);
  if ($41) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $42 = $1;
  $43 = (($42) + 40|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(0);
  if (!($45)) {
   $46 = $0;
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + 20|0);
   HEAP32[$48>>2] = 69;
   $49 = $0;
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$50>>2]|0;
   $52 = $0;
   FUNCTION_TABLE_vi[$51 & 127]($52);
  }
  $53 = $1;
  $54 = (($53) + 36|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0);
  if ($56) {
   $57 = $0;
   $58 = $1;
   _do_sarray_io($57,$58,1);
   $59 = $1;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = 0;
  }
  $61 = $2;
  $62 = $1;
  $63 = (($62) + 24|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($61>>>0)>($64>>>0);
  if ($65) {
   $66 = $2;
   $67 = $1;
   $68 = (($67) + 24|0);
   HEAP32[$68>>2] = $66;
  } else {
   $69 = $end_row;
   $70 = $1;
   $71 = (($70) + 16|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($69) - ($72))|0;
   $ltemp = $73;
   $74 = $ltemp;
   $75 = ($74|0)<(0);
   if ($75) {
    $ltemp = 0;
   }
   $76 = $ltemp;
   $77 = $1;
   $78 = (($77) + 24|0);
   HEAP32[$78>>2] = $76;
  }
  $79 = $0;
  $80 = $1;
  _do_sarray_io($79,$80,0);
 }
 $81 = $1;
 $82 = (($81) + 28|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = $end_row;
 $85 = ($83>>>0)<($84>>>0);
 if ($85) {
  $86 = $1;
  $87 = (($86) + 28|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $2;
  $90 = ($88>>>0)<($89>>>0);
  if ($90) {
   $91 = $4;
   $92 = ($91|0)!=(0);
   if ($92) {
    $93 = $0;
    $94 = HEAP32[$93>>2]|0;
    $95 = (($94) + 20|0);
    HEAP32[$95>>2] = 22;
    $96 = $0;
    $97 = HEAP32[$96>>2]|0;
    $98 = HEAP32[$97>>2]|0;
    $99 = $0;
    FUNCTION_TABLE_vi[$98 & 127]($99);
   }
   $100 = $2;
   $undef_row = $100;
  } else {
   $101 = $1;
   $102 = (($101) + 28|0);
   $103 = HEAP32[$102>>2]|0;
   $undef_row = $103;
  }
  $104 = $4;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = $end_row;
   $107 = $1;
   $108 = (($107) + 28|0);
   HEAP32[$108>>2] = $106;
  }
  $109 = $1;
  $110 = (($109) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ($111|0)!=(0);
  if ($112) {
   $113 = $1;
   $114 = (($113) + 8|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = $115;
   $bytesperrow = $116;
   $117 = $1;
   $118 = (($117) + 24|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = $undef_row;
   $121 = (($120) - ($119))|0;
   $undef_row = $121;
   $122 = $1;
   $123 = (($122) + 24|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = $end_row;
   $126 = (($125) - ($124))|0;
   $end_row = $126;
   while(1) {
    $127 = $undef_row;
    $128 = $end_row;
    $129 = ($127>>>0)<($128>>>0);
    if (!($129)) {
     break;
    }
    $130 = $undef_row;
    $131 = $1;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($132) + ($130<<2)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $bytesperrow;
    _jzero_far($134,$135);
    $136 = $undef_row;
    $137 = (($136) + 1)|0;
    $undef_row = $137;
   }
  } else {
   $138 = $4;
   $139 = ($138|0)!=(0);
   if (!($139)) {
    $140 = $0;
    $141 = HEAP32[$140>>2]|0;
    $142 = (($141) + 20|0);
    HEAP32[$142>>2] = 22;
    $143 = $0;
    $144 = HEAP32[$143>>2]|0;
    $145 = HEAP32[$144>>2]|0;
    $146 = $0;
    FUNCTION_TABLE_vi[$145 & 127]($146);
   }
  }
 }
 $147 = $4;
 $148 = ($147|0)!=(0);
 if (!($148)) {
  $151 = $1;
  $152 = HEAP32[$151>>2]|0;
  $153 = $2;
  $154 = $1;
  $155 = (($154) + 24|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = (($153) - ($156))|0;
  $158 = (($152) + ($157<<2)|0);
  STACKTOP = sp;return ($158|0);
 }
 $149 = $1;
 $150 = (($149) + 36|0);
 HEAP32[$150>>2] = 1;
 $151 = $1;
 $152 = HEAP32[$151>>2]|0;
 $153 = $2;
 $154 = $1;
 $155 = (($154) + 24|0);
 $156 = HEAP32[$155>>2]|0;
 $157 = (($153) - ($156))|0;
 $158 = (($152) + ($157<<2)|0);
 STACKTOP = sp;return ($158|0);
}
function _access_virt_barray($cinfo,$ptr,$start_row,$num_rows,$writable) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $start_row = $start_row|0;
 $num_rows = $num_rows|0;
 $writable = $writable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytesperrow = 0;
 var $end_row = 0, $ltemp = 0, $undef_row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $start_row;
 $3 = $num_rows;
 $4 = $writable;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6))|0;
 $end_row = $7;
 $8 = $end_row;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 if ($12) {
  label = 4;
 } else {
  $13 = $3;
  $14 = $1;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13>>>0)>($16>>>0);
  if ($17) {
   label = 4;
  } else {
   $18 = $1;
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    label = 4;
   }
  }
 }
 if ((label|0) == 4) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 22;
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vi[$26 & 127]($27);
 }
 $28 = $2;
 $29 = $1;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($28>>>0)<($31>>>0);
 if ($32) {
  label = 7;
 } else {
  $33 = $end_row;
  $34 = $1;
  $35 = (($34) + 24|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $1;
  $38 = (($37) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($36) + ($39))|0;
  $41 = ($33>>>0)>($40>>>0);
  if ($41) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $42 = $1;
  $43 = (($42) + 40|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(0);
  if (!($45)) {
   $46 = $0;
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + 20|0);
   HEAP32[$48>>2] = 69;
   $49 = $0;
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$50>>2]|0;
   $52 = $0;
   FUNCTION_TABLE_vi[$51 & 127]($52);
  }
  $53 = $1;
  $54 = (($53) + 36|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0);
  if ($56) {
   $57 = $0;
   $58 = $1;
   _do_barray_io($57,$58,1);
   $59 = $1;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = 0;
  }
  $61 = $2;
  $62 = $1;
  $63 = (($62) + 24|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($61>>>0)>($64>>>0);
  if ($65) {
   $66 = $2;
   $67 = $1;
   $68 = (($67) + 24|0);
   HEAP32[$68>>2] = $66;
  } else {
   $69 = $end_row;
   $70 = $1;
   $71 = (($70) + 16|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($69) - ($72))|0;
   $ltemp = $73;
   $74 = $ltemp;
   $75 = ($74|0)<(0);
   if ($75) {
    $ltemp = 0;
   }
   $76 = $ltemp;
   $77 = $1;
   $78 = (($77) + 24|0);
   HEAP32[$78>>2] = $76;
  }
  $79 = $0;
  $80 = $1;
  _do_barray_io($79,$80,0);
 }
 $81 = $1;
 $82 = (($81) + 28|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = $end_row;
 $85 = ($83>>>0)<($84>>>0);
 if ($85) {
  $86 = $1;
  $87 = (($86) + 28|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $2;
  $90 = ($88>>>0)<($89>>>0);
  if ($90) {
   $91 = $4;
   $92 = ($91|0)!=(0);
   if ($92) {
    $93 = $0;
    $94 = HEAP32[$93>>2]|0;
    $95 = (($94) + 20|0);
    HEAP32[$95>>2] = 22;
    $96 = $0;
    $97 = HEAP32[$96>>2]|0;
    $98 = HEAP32[$97>>2]|0;
    $99 = $0;
    FUNCTION_TABLE_vi[$98 & 127]($99);
   }
   $100 = $2;
   $undef_row = $100;
  } else {
   $101 = $1;
   $102 = (($101) + 28|0);
   $103 = HEAP32[$102>>2]|0;
   $undef_row = $103;
  }
  $104 = $4;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = $end_row;
   $107 = $1;
   $108 = (($107) + 28|0);
   HEAP32[$108>>2] = $106;
  }
  $109 = $1;
  $110 = (($109) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ($111|0)!=(0);
  if ($112) {
   $113 = $1;
   $114 = (($113) + 8|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = $115<<7;
   $bytesperrow = $116;
   $117 = $1;
   $118 = (($117) + 24|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = $undef_row;
   $121 = (($120) - ($119))|0;
   $undef_row = $121;
   $122 = $1;
   $123 = (($122) + 24|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = $end_row;
   $126 = (($125) - ($124))|0;
   $end_row = $126;
   while(1) {
    $127 = $undef_row;
    $128 = $end_row;
    $129 = ($127>>>0)<($128>>>0);
    if (!($129)) {
     break;
    }
    $130 = $undef_row;
    $131 = $1;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($132) + ($130<<2)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $bytesperrow;
    _jzero_far($134,$135);
    $136 = $undef_row;
    $137 = (($136) + 1)|0;
    $undef_row = $137;
   }
  } else {
   $138 = $4;
   $139 = ($138|0)!=(0);
   if (!($139)) {
    $140 = $0;
    $141 = HEAP32[$140>>2]|0;
    $142 = (($141) + 20|0);
    HEAP32[$142>>2] = 22;
    $143 = $0;
    $144 = HEAP32[$143>>2]|0;
    $145 = HEAP32[$144>>2]|0;
    $146 = $0;
    FUNCTION_TABLE_vi[$145 & 127]($146);
   }
  }
 }
 $147 = $4;
 $148 = ($147|0)!=(0);
 if (!($148)) {
  $151 = $1;
  $152 = HEAP32[$151>>2]|0;
  $153 = $2;
  $154 = $1;
  $155 = (($154) + 24|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = (($153) - ($156))|0;
  $158 = (($152) + ($157<<2)|0);
  STACKTOP = sp;return ($158|0);
 }
 $149 = $1;
 $150 = (($149) + 36|0);
 HEAP32[$150>>2] = 1;
 $151 = $1;
 $152 = HEAP32[$151>>2]|0;
 $153 = $2;
 $154 = $1;
 $155 = (($154) + 24|0);
 $156 = HEAP32[$155>>2]|0;
 $157 = (($153) - ($156))|0;
 $158 = (($152) + ($157<<2)|0);
 STACKTOP = sp;return ($158|0);
}
function _free_pool($cinfo,$pool_id) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bptr = 0, $lhdr_ptr = 0, $mem = 0, $next_lhdr_ptr = 0, $next_shdr_ptr = 0, $shdr_ptr = 0, $space_freed = 0, $sptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $mem = $4;
 $5 = $1;
 $6 = ($5|0)<(0);
 if ($6) {
  label = 3;
 } else {
  $7 = $1;
  $8 = ($7|0)>=(2);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = $0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 20|0);
  HEAP32[$11>>2] = 14;
  $12 = $1;
  $13 = $0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 24|0);
  HEAP32[$15>>2] = $12;
  $16 = $0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  FUNCTION_TABLE_vi[$18 & 127]($19);
 }
 $20 = $1;
 $21 = ($20|0)==(1);
 if ($21) {
  $22 = $mem;
  $23 = (($22) + 68|0);
  $24 = HEAP32[$23>>2]|0;
  $sptr = $24;
  while(1) {
   $25 = $sptr;
   $26 = ($25|0)!=(0|0);
   if (!($26)) {
    break;
   }
   $27 = $sptr;
   $28 = (($27) + 40|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = $sptr;
    $32 = (($31) + 40|0);
    HEAP32[$32>>2] = 0;
    $33 = $sptr;
    $34 = (($33) + 48|0);
    $35 = (($34) + 8|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = $0;
    $38 = $sptr;
    $39 = (($38) + 48|0);
    FUNCTION_TABLE_vii[$36 & 127]($37,$39);
   }
   $40 = $sptr;
   $41 = (($40) + 44|0);
   $42 = HEAP32[$41>>2]|0;
   $sptr = $42;
  }
  $43 = $mem;
  $44 = (($43) + 68|0);
  HEAP32[$44>>2] = 0;
  $45 = $mem;
  $46 = (($45) + 72|0);
  $47 = HEAP32[$46>>2]|0;
  $bptr = $47;
  while(1) {
   $48 = $bptr;
   $49 = ($48|0)!=(0|0);
   if (!($49)) {
    break;
   }
   $50 = $bptr;
   $51 = (($50) + 40|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)!=(0);
   if ($53) {
    $54 = $bptr;
    $55 = (($54) + 40|0);
    HEAP32[$55>>2] = 0;
    $56 = $bptr;
    $57 = (($56) + 48|0);
    $58 = (($57) + 8|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = $0;
    $61 = $bptr;
    $62 = (($61) + 48|0);
    FUNCTION_TABLE_vii[$59 & 127]($60,$62);
   }
   $63 = $bptr;
   $64 = (($63) + 44|0);
   $65 = HEAP32[$64>>2]|0;
   $bptr = $65;
  }
  $66 = $mem;
  $67 = (($66) + 72|0);
  HEAP32[$67>>2] = 0;
 }
 $68 = $1;
 $69 = $mem;
 $70 = (($69) + 60|0);
 $71 = (($70) + ($68<<2)|0);
 $72 = HEAP32[$71>>2]|0;
 $lhdr_ptr = $72;
 $73 = $1;
 $74 = $mem;
 $75 = (($74) + 60|0);
 $76 = (($75) + ($73<<2)|0);
 HEAP32[$76>>2] = 0;
 while(1) {
  $77 = $lhdr_ptr;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = $lhdr_ptr;
  $80 = HEAP32[$79>>2]|0;
  $next_lhdr_ptr = $80;
  $81 = $lhdr_ptr;
  $82 = (($81) + 4|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $lhdr_ptr;
  $85 = (($84) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($83) + ($86))|0;
  $88 = (($87) + 16)|0;
  $space_freed = $88;
  $89 = $0;
  $90 = $lhdr_ptr;
  $91 = $space_freed;
  _jpeg_free_large($89,$90,$91);
  $92 = $space_freed;
  $93 = $mem;
  $94 = (($93) + 76|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95) - ($92))|0;
  HEAP32[$94>>2] = $96;
  $97 = $next_lhdr_ptr;
  $lhdr_ptr = $97;
 }
 $98 = $1;
 $99 = $mem;
 $100 = (($99) + 52|0);
 $101 = (($100) + ($98<<2)|0);
 $102 = HEAP32[$101>>2]|0;
 $shdr_ptr = $102;
 $103 = $1;
 $104 = $mem;
 $105 = (($104) + 52|0);
 $106 = (($105) + ($103<<2)|0);
 HEAP32[$106>>2] = 0;
 while(1) {
  $107 = $shdr_ptr;
  $108 = ($107|0)!=(0|0);
  if (!($108)) {
   break;
  }
  $109 = $shdr_ptr;
  $110 = HEAP32[$109>>2]|0;
  $next_shdr_ptr = $110;
  $111 = $shdr_ptr;
  $112 = (($111) + 4|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = $shdr_ptr;
  $115 = (($114) + 8|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (($113) + ($116))|0;
  $118 = (($117) + 16)|0;
  $space_freed = $118;
  $119 = $0;
  $120 = $shdr_ptr;
  $121 = $space_freed;
  _jpeg_free_small($119,$120,$121);
  $122 = $space_freed;
  $123 = $mem;
  $124 = (($123) + 76|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = (($125) - ($122))|0;
  HEAP32[$124>>2] = $126;
  $127 = $next_shdr_ptr;
  $shdr_ptr = $127;
 }
 STACKTOP = sp;return;
}
function _self_destruct($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pool = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $pool = 1;
 while(1) {
  $1 = $pool;
  $2 = ($1|0)>=(0);
  if (!($2)) {
   break;
  }
  $3 = $0;
  $4 = $pool;
  _free_pool($3,$4);
  $5 = $pool;
  $6 = (($5) + -1)|0;
  $pool = $6;
 }
 $7 = $0;
 $8 = $0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 _jpeg_free_small($7,$10,84);
 $11 = $0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = $0;
 _jpeg_mem_term($13);
 STACKTOP = sp;return;
}
function _do_barray_io($cinfo,$ptr,$writing) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $writing = $writing|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $byte_count = 0, $bytesperrow = 0, $file_offset = 0, $i = 0, $rows = 0, $thisrow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $writing;
 $3 = $1;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<7;
 $bytesperrow = $6;
 $7 = $1;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $bytesperrow;
 $11 = Math_imul($9, $10)|0;
 $file_offset = $11;
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $1;
  $14 = (($13) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 19;
   break;
  }
  $17 = $1;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $i;
  $24 = (($22) - ($23))|0;
  $25 = ($19|0)<($24|0);
  if ($25) {
   $26 = $1;
   $27 = (($26) + 20|0);
   $28 = HEAP32[$27>>2]|0;
   $34 = $28;
  } else {
   $29 = $1;
   $30 = (($29) + 16|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $i;
   $33 = (($31) - ($32))|0;
   $34 = $33;
  }
  $rows = $34;
  $35 = $1;
  $36 = (($35) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $i;
  $39 = (($37) + ($38))|0;
  $thisrow = $39;
  $40 = $rows;
  $41 = $1;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $thisrow;
  $45 = (($43) - ($44))|0;
  $46 = ($40|0)<($45|0);
  if ($46) {
   $47 = $rows;
   $53 = $47;
  } else {
   $48 = $1;
   $49 = (($48) + 28|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $thisrow;
   $52 = (($50) - ($51))|0;
   $53 = $52;
  }
  $rows = $53;
  $54 = $rows;
  $55 = $1;
  $56 = (($55) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $thisrow;
  $59 = (($57) - ($58))|0;
  $60 = ($54|0)<($59|0);
  if ($60) {
   $61 = $rows;
   $67 = $61;
  } else {
   $62 = $1;
   $63 = (($62) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $thisrow;
   $66 = (($64) - ($65))|0;
   $67 = $66;
  }
  $rows = $67;
  $68 = $rows;
  $69 = ($68|0)<=(0);
  if ($69) {
   break;
  }
  $70 = $rows;
  $71 = $bytesperrow;
  $72 = Math_imul($70, $71)|0;
  $byte_count = $72;
  $73 = $2;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $1;
   $76 = (($75) + 48|0);
   $77 = (($76) + 4|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $0;
   $80 = $1;
   $81 = (($80) + 48|0);
   $82 = $i;
   $83 = $1;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + ($82<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $file_offset;
   $88 = $byte_count;
   FUNCTION_TABLE_viiiii[$78 & 127]($79,$81,$86,$87,$88);
  } else {
   $89 = $1;
   $90 = (($89) + 48|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = $0;
   $93 = $1;
   $94 = (($93) + 48|0);
   $95 = $i;
   $96 = $1;
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + ($95<<2)|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = $file_offset;
   $101 = $byte_count;
   FUNCTION_TABLE_viiiii[$91 & 127]($92,$94,$99,$100,$101);
  }
  $102 = $byte_count;
  $103 = $file_offset;
  $104 = (($103) + ($102))|0;
  $file_offset = $104;
  $105 = $1;
  $106 = (($105) + 20|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $i;
  $109 = (($108) + ($107))|0;
  $i = $109;
 }
 if ((label|0) == 19) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _do_sarray_io($cinfo,$ptr,$writing) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $writing = $writing|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $byte_count = 0, $bytesperrow = 0, $file_offset = 0, $i = 0, $rows = 0, $thisrow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $writing;
 $3 = $1;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5;
 $bytesperrow = $6;
 $7 = $1;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $bytesperrow;
 $11 = Math_imul($9, $10)|0;
 $file_offset = $11;
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $1;
  $14 = (($13) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 19;
   break;
  }
  $17 = $1;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $i;
  $24 = (($22) - ($23))|0;
  $25 = ($19|0)<($24|0);
  if ($25) {
   $26 = $1;
   $27 = (($26) + 20|0);
   $28 = HEAP32[$27>>2]|0;
   $34 = $28;
  } else {
   $29 = $1;
   $30 = (($29) + 16|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $i;
   $33 = (($31) - ($32))|0;
   $34 = $33;
  }
  $rows = $34;
  $35 = $1;
  $36 = (($35) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $i;
  $39 = (($37) + ($38))|0;
  $thisrow = $39;
  $40 = $rows;
  $41 = $1;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $thisrow;
  $45 = (($43) - ($44))|0;
  $46 = ($40|0)<($45|0);
  if ($46) {
   $47 = $rows;
   $53 = $47;
  } else {
   $48 = $1;
   $49 = (($48) + 28|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $thisrow;
   $52 = (($50) - ($51))|0;
   $53 = $52;
  }
  $rows = $53;
  $54 = $rows;
  $55 = $1;
  $56 = (($55) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $thisrow;
  $59 = (($57) - ($58))|0;
  $60 = ($54|0)<($59|0);
  if ($60) {
   $61 = $rows;
   $67 = $61;
  } else {
   $62 = $1;
   $63 = (($62) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $thisrow;
   $66 = (($64) - ($65))|0;
   $67 = $66;
  }
  $rows = $67;
  $68 = $rows;
  $69 = ($68|0)<=(0);
  if ($69) {
   break;
  }
  $70 = $rows;
  $71 = $bytesperrow;
  $72 = Math_imul($70, $71)|0;
  $byte_count = $72;
  $73 = $2;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $1;
   $76 = (($75) + 48|0);
   $77 = (($76) + 4|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $0;
   $80 = $1;
   $81 = (($80) + 48|0);
   $82 = $i;
   $83 = $1;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + ($82<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $file_offset;
   $88 = $byte_count;
   FUNCTION_TABLE_viiiii[$78 & 127]($79,$81,$86,$87,$88);
  } else {
   $89 = $1;
   $90 = (($89) + 48|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = $0;
   $93 = $1;
   $94 = (($93) + 48|0);
   $95 = $i;
   $96 = $1;
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + ($95<<2)|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = $file_offset;
   $101 = $byte_count;
   FUNCTION_TABLE_viiiii[$91 & 127]($92,$94,$99,$100,$101);
  }
  $102 = $byte_count;
  $103 = $file_offset;
  $104 = (($103) + ($102))|0;
  $file_offset = $104;
  $105 = $1;
  $106 = (($105) + 20|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $i;
  $109 = (($108) + ($107))|0;
  $i = $109;
 }
 if ((label|0) == 19) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _out_of_memory($cinfo,$which) {
 $cinfo = $cinfo|0;
 $which = $which|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $which;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 20|0);
 HEAP32[$4>>2] = 54;
 $5 = $1;
 $6 = $0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 24|0);
 HEAP32[$8>>2] = $5;
 $9 = $0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = $0;
 FUNCTION_TABLE_vi[$11 & 127]($12);
 STACKTOP = sp;return;
}
function _jpeg_get_small($cinfo,$sizeofobject) {
 $cinfo = $cinfo|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $sizeofobject;
 $2 = $1;
 $3 = (_malloc($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _jpeg_free_small($cinfo,$object,$sizeofobject) {
 $cinfo = $cinfo|0;
 $object = $object|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $object;
 $2 = $sizeofobject;
 $3 = $1;
 _free($3);
 STACKTOP = sp;return;
}
function _jpeg_get_large($cinfo,$sizeofobject) {
 $cinfo = $cinfo|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $sizeofobject;
 $2 = $1;
 $3 = (_malloc($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _jpeg_free_large($cinfo,$object,$sizeofobject) {
 $cinfo = $cinfo|0;
 $object = $object|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $object;
 $2 = $sizeofobject;
 $3 = $1;
 _free($3);
 STACKTOP = sp;return;
}
function _jpeg_mem_available($cinfo,$min_bytes_needed,$max_bytes_needed,$already_allocated) {
 $cinfo = $cinfo|0;
 $min_bytes_needed = $min_bytes_needed|0;
 $max_bytes_needed = $max_bytes_needed|0;
 $already_allocated = $already_allocated|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $min_bytes_needed;
 $2 = $max_bytes_needed;
 $3 = $already_allocated;
 $4 = $2;
 STACKTOP = sp;return ($4|0);
}
function _jpeg_open_backing_store($cinfo,$info,$total_bytes_needed) {
 $cinfo = $cinfo|0;
 $info = $info|0;
 $total_bytes_needed = $total_bytes_needed|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $info;
 $2 = $total_bytes_needed;
 $3 = $0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 20|0);
 HEAP32[$5>>2] = 49;
 $6 = $0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 FUNCTION_TABLE_vi[$8 & 127]($9);
 STACKTOP = sp;return;
}
function _jpeg_mem_init($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return 0;
}
function _jpeg_mem_term($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _jinit_1pass_quantizer($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,88)|0);
 $cquantize = $6;
 $7 = $cquantize;
 $8 = $0;
 $9 = (($8) + 460|0);
 HEAP32[$9>>2] = $7;
 $10 = $cquantize;
 HEAP32[$10>>2] = 76;
 $11 = $cquantize;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = 77;
 $13 = $cquantize;
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = 78;
 $15 = $cquantize;
 $16 = (($15) + 68|0);
 HEAP32[$16>>2] = 0;
 $17 = $cquantize;
 $18 = (($17) + 52|0);
 HEAP32[$18>>2] = 0;
 $19 = $0;
 $20 = (($19) + 120|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)>(4);
 if ($22) {
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 55;
  $26 = $0;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 24|0);
  HEAP32[$28>>2] = 4;
  $29 = $0;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = $0;
  FUNCTION_TABLE_vi[$31 & 127]($32);
 }
 $33 = $0;
 $34 = (($33) + 96|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)>(256);
 if ($36) {
  $37 = $0;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 20|0);
  HEAP32[$39>>2] = 57;
  $40 = $0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 24|0);
  HEAP32[$42>>2] = 256;
  $43 = $0;
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$44>>2]|0;
  $46 = $0;
  FUNCTION_TABLE_vi[$45 & 127]($46);
 }
 $47 = $0;
 _create_colormap($47);
 $48 = $0;
 _create_colorindex($48);
 $49 = $0;
 $50 = (($49) + 88|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)==(2);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = $0;
 _alloc_fs_workspace($53);
 STACKTOP = sp;return;
}
function _start_pass_1_quant($cinfo,$is_pre_scan) {
 $cinfo = $cinfo|0;
 $is_pre_scan = $is_pre_scan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $arraysize = 0, $cquantize = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $is_pre_scan;
 $2 = $0;
 $3 = (($2) + 460|0);
 $4 = HEAP32[$3>>2]|0;
 $cquantize = $4;
 $5 = $cquantize;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (($8) + 136|0);
 HEAP32[$9>>2] = $7;
 $10 = $cquantize;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 132|0);
 HEAP32[$14>>2] = $12;
 $15 = $0;
 $16 = (($15) + 88|0);
 $17 = HEAP32[$16>>2]|0;
 if ((($17|0) == 1)) {
  $26 = $0;
  $27 = (($26) + 120|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(3);
  if ($29) {
   $30 = $cquantize;
   $31 = (($30) + 4|0);
   HEAP32[$31>>2] = 81;
  } else {
   $32 = $cquantize;
   $33 = (($32) + 4|0);
   HEAP32[$33>>2] = 82;
  }
  $34 = $cquantize;
  $35 = (($34) + 48|0);
  HEAP32[$35>>2] = 0;
  $36 = $cquantize;
  $37 = (($36) + 28|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(0);
  if (!($39)) {
   $40 = $0;
   _create_colorindex($40);
  }
  $41 = $cquantize;
  $42 = (($41) + 52|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($43|0)==(0|0);
  if ($44) {
   $45 = $0;
   _create_odither_tables($45);
  }
  STACKTOP = sp;return;
 } else if ((($17|0) == 0)) {
  $18 = $0;
  $19 = (($18) + 120|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(3);
  if ($21) {
   $22 = $cquantize;
   $23 = (($22) + 4|0);
   HEAP32[$23>>2] = 79;
  } else {
   $24 = $cquantize;
   $25 = (($24) + 4|0);
   HEAP32[$25>>2] = 80;
  }
  STACKTOP = sp;return;
 } else if ((($17|0) == 2)) {
  $46 = $cquantize;
  $47 = (($46) + 4|0);
  HEAP32[$47>>2] = 83;
  $48 = $cquantize;
  $49 = (($48) + 84|0);
  HEAP32[$49>>2] = 0;
  $50 = $cquantize;
  $51 = (($50) + 68|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($52|0)==(0|0);
  if ($53) {
   $54 = $0;
   _alloc_fs_workspace($54);
  }
  $55 = $0;
  $56 = (($55) + 112|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 2)|0;
  $59 = $58<<1;
  $arraysize = $59;
  $i = 0;
  while(1) {
   $60 = $i;
   $61 = $0;
   $62 = (($61) + 120|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($60|0)<($63|0);
   if (!($64)) {
    break;
   }
   $65 = $i;
   $66 = $cquantize;
   $67 = (($66) + 68|0);
   $68 = (($67) + ($65<<2)|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = $arraysize;
   _jzero_far($69,$70);
   $71 = $i;
   $72 = (($71) + 1)|0;
   $i = $72;
  }
  STACKTOP = sp;return;
 } else {
  $73 = $0;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + 20|0);
  HEAP32[$75>>2] = 48;
  $76 = $0;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$77>>2]|0;
  $79 = $0;
  FUNCTION_TABLE_vi[$78 & 127]($79);
  STACKTOP = sp;return;
 }
}
function _finish_pass_1_quant($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _new_color_map_1_quant($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 HEAP32[$3>>2] = 46;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 FUNCTION_TABLE_vi[$6 & 127]($7);
 STACKTOP = sp;return;
}
function _create_colormap($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $blkdist = 0, $blksize = 0, $colormap = 0, $cquantize = 0, $i = 0, $j = 0, $k = 0, $nci = 0, $ptr = 0, $total_colors = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = $cquantize;
 $6 = (($5) + 32|0);
 $7 = (_select_ncolors($4,$6)|0);
 $total_colors = $7;
 $8 = $0;
 $9 = (($8) + 120|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(3);
 if ($11) {
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 24|0);
  $_mp = $14;
  $15 = $total_colors;
  $16 = $_mp;
  HEAP32[$16>>2] = $15;
  $17 = $cquantize;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $_mp;
  $21 = (($20) + 4|0);
  HEAP32[$21>>2] = $19;
  $22 = $cquantize;
  $23 = (($22) + 32|0);
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $_mp;
  $27 = (($26) + 8|0);
  HEAP32[$27>>2] = $25;
  $28 = $cquantize;
  $29 = (($28) + 32|0);
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $_mp;
  $33 = (($32) + 12|0);
  HEAP32[$33>>2] = $31;
  $34 = $0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 20|0);
  HEAP32[$36>>2] = 94;
  $37 = $0;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $0;
  FUNCTION_TABLE_vii[$40 & 127]($41,1);
 } else {
  $42 = $0;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 20|0);
  HEAP32[$44>>2] = 95;
  $45 = $total_colors;
  $46 = $0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 24|0);
  HEAP32[$48>>2] = $45;
  $49 = $0;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $0;
  FUNCTION_TABLE_vii[$52 & 127]($53,1);
 }
 $54 = $0;
 $55 = (($54) + 4|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = (($56) + 8|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $0;
 $60 = $total_colors;
 $61 = $0;
 $62 = (($61) + 120|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = (FUNCTION_TABLE_iiiii[$58 & 127]($59,1,$60,$63)|0);
 $colormap = $64;
 $65 = $total_colors;
 $blkdist = $65;
 $i = 0;
 while(1) {
  $66 = $i;
  $67 = $0;
  $68 = (($67) + 120|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = ($66|0)<($69|0);
  if (!($70)) {
   break;
  }
  $71 = $i;
  $72 = $cquantize;
  $73 = (($72) + 32|0);
  $74 = (($73) + ($71<<2)|0);
  $75 = HEAP32[$74>>2]|0;
  $nci = $75;
  $76 = $blkdist;
  $77 = $nci;
  $78 = (($76|0) / ($77|0))&-1;
  $blksize = $78;
  $j = 0;
  while(1) {
   $79 = $j;
   $80 = $nci;
   $81 = ($79|0)<($80|0);
   if (!($81)) {
    break;
   }
   $82 = $0;
   $83 = $i;
   $84 = $j;
   $85 = $nci;
   $86 = (($85) - 1)|0;
   $87 = (_output_value($82,$83,$84,$86)|0);
   $val = $87;
   $88 = $j;
   $89 = $blksize;
   $90 = Math_imul($88, $89)|0;
   $ptr = $90;
   while(1) {
    $91 = $ptr;
    $92 = $total_colors;
    $93 = ($91|0)<($92|0);
    if (!($93)) {
     break;
    }
    $k = 0;
    while(1) {
     $94 = $k;
     $95 = $blksize;
     $96 = ($94|0)<($95|0);
     if (!($96)) {
      break;
     }
     $97 = $val;
     $98 = $97&255;
     $99 = $ptr;
     $100 = $k;
     $101 = (($99) + ($100))|0;
     $102 = $i;
     $103 = $colormap;
     $104 = (($103) + ($102<<2)|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (($105) + ($101)|0);
     HEAP8[$106>>0] = $98;
     $107 = $k;
     $108 = (($107) + 1)|0;
     $k = $108;
    }
    $109 = $blkdist;
    $110 = $ptr;
    $111 = (($110) + ($109))|0;
    $ptr = $111;
   }
   $112 = $j;
   $113 = (($112) + 1)|0;
   $j = $113;
  }
  $114 = $blksize;
  $blkdist = $114;
  $115 = $i;
  $116 = (($115) + 1)|0;
  $i = $116;
 }
 $117 = $colormap;
 $118 = $cquantize;
 $119 = (($118) + 16|0);
 HEAP32[$119>>2] = $117;
 $120 = $total_colors;
 $121 = $cquantize;
 $122 = (($121) + 20|0);
 HEAP32[$122>>2] = $120;
 STACKTOP = sp;return;
}
function _create_colorindex($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $blksize = 0, $cquantize = 0, $i = 0, $indexptr = 0, $j = 0, $k = 0, $nci = 0, $pad = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = (($4) + 88|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(1);
 if ($7) {
  $pad = 510;
  $8 = $cquantize;
  $9 = (($8) + 28|0);
  HEAP32[$9>>2] = 1;
 } else {
  $pad = 0;
  $10 = $cquantize;
  $11 = (($10) + 28|0);
  HEAP32[$11>>2] = 0;
 }
 $12 = $0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = $pad;
 $19 = (256 + ($18))|0;
 $20 = $0;
 $21 = (($20) + 120|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiii[$16 & 127]($17,1,$19,$22)|0);
 $24 = $cquantize;
 $25 = (($24) + 24|0);
 HEAP32[$25>>2] = $23;
 $26 = $cquantize;
 $27 = (($26) + 20|0);
 $28 = HEAP32[$27>>2]|0;
 $blksize = $28;
 $i = 0;
 while(1) {
  $29 = $i;
  $30 = $0;
  $31 = (($30) + 120|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($29|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = $i;
  $35 = $cquantize;
  $36 = (($35) + 32|0);
  $37 = (($36) + ($34<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $nci = $38;
  $39 = $blksize;
  $40 = $nci;
  $41 = (($39|0) / ($40|0))&-1;
  $blksize = $41;
  $42 = $pad;
  $43 = ($42|0)!=(0);
  if ($43) {
   $44 = $i;
   $45 = $cquantize;
   $46 = (($45) + 24|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($44<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + 255|0);
   HEAP32[$48>>2] = $50;
  }
  $51 = $i;
  $52 = $cquantize;
  $53 = (($52) + 24|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $indexptr = $56;
  $val = 0;
  $57 = $0;
  $58 = $i;
  $59 = $nci;
  $60 = (($59) - 1)|0;
  $61 = (_largest_input_value($57,$58,0,$60)|0);
  $k = $61;
  $j = 0;
  while(1) {
   $62 = $j;
   $63 = ($62|0)<=(255);
   if (!($63)) {
    break;
   }
   while(1) {
    $64 = $j;
    $65 = $k;
    $66 = ($64|0)>($65|0);
    if (!($66)) {
     break;
    }
    $67 = $0;
    $68 = $i;
    $69 = $val;
    $70 = (($69) + 1)|0;
    $val = $70;
    $71 = $nci;
    $72 = (($71) - 1)|0;
    $73 = (_largest_input_value($67,$68,$70,$72)|0);
    $k = $73;
   }
   $74 = $val;
   $75 = $blksize;
   $76 = Math_imul($74, $75)|0;
   $77 = $76&255;
   $78 = $j;
   $79 = $indexptr;
   $80 = (($79) + ($78)|0);
   HEAP8[$80>>0] = $77;
   $81 = $j;
   $82 = (($81) + 1)|0;
   $j = $82;
  }
  $83 = $pad;
  $84 = ($83|0)!=(0);
  if ($84) {
   $j = 1;
   while(1) {
    $85 = $j;
    $86 = ($85|0)<=(255);
    if (!($86)) {
     break;
    }
    $87 = $indexptr;
    $88 = HEAP8[$87>>0]|0;
    $89 = $j;
    $90 = (0 - ($89))|0;
    $91 = $indexptr;
    $92 = (($91) + ($90)|0);
    HEAP8[$92>>0] = $88;
    $93 = $indexptr;
    $94 = (($93) + 255|0);
    $95 = HEAP8[$94>>0]|0;
    $96 = $j;
    $97 = (255 + ($96))|0;
    $98 = $indexptr;
    $99 = (($98) + ($97)|0);
    HEAP8[$99>>0] = $95;
    $100 = $j;
    $101 = (($100) + 1)|0;
    $j = $101;
   }
  }
  $102 = $i;
  $103 = (($102) + 1)|0;
  $i = $103;
 }
 STACKTOP = sp;return;
}
function _alloc_fs_workspace($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $arraysize = 0, $cquantize = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = (($4) + 112|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 2)|0;
 $8 = $7<<1;
 $arraysize = $8;
 $i = 0;
 while(1) {
  $9 = $i;
  $10 = $0;
  $11 = (($10) + 120|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  $20 = $arraysize;
  $21 = (FUNCTION_TABLE_iiii[$18 & 127]($19,1,$20)|0);
  $22 = $i;
  $23 = $cquantize;
  $24 = (($23) + 68|0);
  $25 = (($24) + ($22<<2)|0);
  HEAP32[$25>>2] = $21;
  $26 = $i;
  $27 = (($26) + 1)|0;
  $i = $27;
 }
 STACKTOP = sp;return;
}
function _largest_input_value($cinfo,$ci,$j,$maxj) {
 $cinfo = $cinfo|0;
 $ci = $ci|0;
 $j = $j|0;
 $maxj = $maxj|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $ci;
 $2 = $j;
 $3 = $maxj;
 $4 = $2;
 $5 = $4<<1;
 $6 = (($5) + 1)|0;
 $7 = ($6*255)|0;
 $8 = $3;
 $9 = (($7) + ($8))|0;
 $10 = $3;
 $11 = $10<<1;
 $12 = (($9|0) / ($11|0))&-1;
 STACKTOP = sp;return ($12|0);
}
function _select_ncolors($cinfo,$Ncolors) {
 $cinfo = $cinfo|0;
 $Ncolors = $Ncolors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $changed = 0, $i = 0, $iroot = 0, $j = 0, $max_colors = 0, $nc = 0, $temp = 0, $total_colors = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $Ncolors;
 $2 = $0;
 $3 = (($2) + 120|0);
 $4 = HEAP32[$3>>2]|0;
 $nc = $4;
 $5 = $0;
 $6 = (($5) + 96|0);
 $7 = HEAP32[$6>>2]|0;
 $max_colors = $7;
 $iroot = 1;
 while(1) {
  $8 = $iroot;
  $9 = (($8) + 1)|0;
  $iroot = $9;
  $10 = $iroot;
  $temp = $10;
  $i = 1;
  while(1) {
   $11 = $i;
   $12 = $nc;
   $13 = ($11|0)<($12|0);
   if (!($13)) {
    break;
   }
   $14 = $iroot;
   $15 = $temp;
   $16 = Math_imul($15, $14)|0;
   $temp = $16;
   $17 = $i;
   $18 = (($17) + 1)|0;
   $i = $18;
  }
  $19 = $temp;
  $20 = $max_colors;
  $21 = ($19|0)<=($20|0);
  if (!($21)) {
   break;
  }
 }
 $22 = $iroot;
 $23 = (($22) + -1)|0;
 $iroot = $23;
 $24 = $iroot;
 $25 = ($24|0)<(2);
 if ($25) {
  $26 = $0;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 20|0);
  HEAP32[$28>>2] = 56;
  $29 = $temp;
  $30 = $0;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 24|0);
  HEAP32[$32>>2] = $29;
  $33 = $0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = $0;
  FUNCTION_TABLE_vi[$35 & 127]($36);
 }
 $total_colors = 1;
 $i = 0;
 while(1) {
  $37 = $i;
  $38 = $nc;
  $39 = ($37|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = $iroot;
  $41 = $i;
  $42 = $1;
  $43 = (($42) + ($41<<2)|0);
  HEAP32[$43>>2] = $40;
  $44 = $iroot;
  $45 = $total_colors;
  $46 = Math_imul($45, $44)|0;
  $total_colors = $46;
  $47 = $i;
  $48 = (($47) + 1)|0;
  $i = $48;
 }
 while(1) {
  $changed = 0;
  $i = 0;
  while(1) {
   $49 = $i;
   $50 = $nc;
   $51 = ($49|0)<($50|0);
   if (!($51)) {
    break;
   }
   $52 = $0;
   $53 = (($52) + 44|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = ($54|0)==(2);
   if ($55) {
    $56 = $i;
    $57 = (9232 + ($56<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $60 = $58;
   } else {
    $59 = $i;
    $60 = $59;
   }
   $j = $60;
   $61 = $total_colors;
   $62 = $j;
   $63 = $1;
   $64 = (($63) + ($62<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($61|0) / ($65|0))&-1;
   $temp = $66;
   $67 = $j;
   $68 = $1;
   $69 = (($68) + ($67<<2)|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + 1)|0;
   $72 = $temp;
   $73 = Math_imul($72, $71)|0;
   $temp = $73;
   $74 = $temp;
   $75 = $max_colors;
   $76 = ($74|0)>($75|0);
   if ($76) {
    label = 21;
    break;
   }
   $77 = $j;
   $78 = $1;
   $79 = (($78) + ($77<<2)|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = (($80) + 1)|0;
   HEAP32[$79>>2] = $81;
   $82 = $temp;
   $total_colors = $82;
   $changed = 1;
   $83 = $i;
   $84 = (($83) + 1)|0;
   $i = $84;
  }
  if ((label|0) == 21) {
   label = 0;
  }
  $85 = $changed;
  $86 = ($85|0)!=(0);
  if (!($86)) {
   break;
  }
 }
 $87 = $total_colors;
 STACKTOP = sp;return ($87|0);
}
function _output_value($cinfo,$ci,$j,$maxj) {
 $cinfo = $cinfo|0;
 $ci = $ci|0;
 $j = $j|0;
 $maxj = $maxj|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $ci;
 $2 = $j;
 $3 = $maxj;
 $4 = $2;
 $5 = ($4*255)|0;
 $6 = $3;
 $7 = (($6|0) / 2)&-1;
 $8 = (($5) + ($7))|0;
 $9 = $3;
 $10 = (($8|0) / ($9|0))&-1;
 STACKTOP = sp;return ($10|0);
}
function _color_quantize3($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $col = 0, $colorindex0 = 0, $colorindex1 = 0, $colorindex2 = 0, $cquantize = 0, $pixcode = 0;
 var $ptrin = 0, $ptrout = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $colorindex0 = $10;
 $11 = $cquantize;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $colorindex1 = $15;
 $16 = $cquantize;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $colorindex2 = $20;
 $21 = $0;
 $22 = (($21) + 112|0);
 $23 = HEAP32[$22>>2]|0;
 $width = $23;
 $row = 0;
 while(1) {
  $24 = $row;
  $25 = $3;
  $26 = ($24|0)<($25|0);
  if (!($26)) {
   break;
  }
  $27 = $row;
  $28 = $1;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $ptrin = $30;
  $31 = $row;
  $32 = $2;
  $33 = (($32) + ($31<<2)|0);
  $34 = HEAP32[$33>>2]|0;
  $ptrout = $34;
  $35 = $width;
  $col = $35;
  while(1) {
   $36 = $col;
   $37 = ($36>>>0)>(0);
   if (!($37)) {
    break;
   }
   $38 = $ptrin;
   $39 = (($38) + 1|0);
   $ptrin = $39;
   $40 = HEAP8[$38>>0]|0;
   $41 = $40&255;
   $42 = $colorindex0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $pixcode = $45;
   $46 = $ptrin;
   $47 = (($46) + 1|0);
   $ptrin = $47;
   $48 = HEAP8[$46>>0]|0;
   $49 = $48&255;
   $50 = $colorindex1;
   $51 = (($50) + ($49)|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = $pixcode;
   $55 = (($54) + ($53))|0;
   $pixcode = $55;
   $56 = $ptrin;
   $57 = (($56) + 1|0);
   $ptrin = $57;
   $58 = HEAP8[$56>>0]|0;
   $59 = $58&255;
   $60 = $colorindex2;
   $61 = (($60) + ($59)|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = $pixcode;
   $65 = (($64) + ($63))|0;
   $pixcode = $65;
   $66 = $pixcode;
   $67 = $66&255;
   $68 = $ptrout;
   $69 = (($68) + 1|0);
   $ptrout = $69;
   HEAP8[$68>>0] = $67;
   $70 = $col;
   $71 = (($70) + -1)|0;
   $col = $71;
  }
  $72 = $row;
  $73 = (($72) + 1)|0;
  $row = $73;
 }
 STACKTOP = sp;return;
}
function _color_quantize($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $col = 0, $colorindex = 0, $cquantize = 0;
 var $nc = 0, $pixcode = 0, $ptrin = 0, $ptrout = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $colorindex = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $13 = $0;
 $14 = (($13) + 120|0);
 $15 = HEAP32[$14>>2]|0;
 $nc = $15;
 $row = 0;
 while(1) {
  $16 = $row;
  $17 = $3;
  $18 = ($16|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $row;
  $20 = $1;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $ptrin = $22;
  $23 = $row;
  $24 = $2;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $ptrout = $26;
  $27 = $width;
  $col = $27;
  while(1) {
   $28 = $col;
   $29 = ($28>>>0)>(0);
   if (!($29)) {
    break;
   }
   $pixcode = 0;
   $ci = 0;
   while(1) {
    $30 = $ci;
    $31 = $nc;
    $32 = ($30|0)<($31|0);
    if (!($32)) {
     break;
    }
    $33 = $ptrin;
    $34 = (($33) + 1|0);
    $ptrin = $34;
    $35 = HEAP8[$33>>0]|0;
    $36 = $35&255;
    $37 = $ci;
    $38 = $colorindex;
    $39 = (($38) + ($37<<2)|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + ($36)|0);
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $pixcode;
    $45 = (($44) + ($43))|0;
    $pixcode = $45;
    $46 = $ci;
    $47 = (($46) + 1)|0;
    $ci = $47;
   }
   $48 = $pixcode;
   $49 = $48&255;
   $50 = $ptrout;
   $51 = (($50) + 1|0);
   $ptrout = $51;
   HEAP8[$50>>0] = $49;
   $52 = $col;
   $53 = (($52) + -1)|0;
   $col = $53;
  }
  $54 = $row;
  $55 = (($54) + 1)|0;
  $row = $55;
 }
 STACKTOP = sp;return;
}
function _quantize3_ord_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $col = 0, $col_index = 0;
 var $colorindex0 = 0, $colorindex1 = 0, $colorindex2 = 0, $cquantize = 0, $dither0 = 0, $dither1 = 0, $dither2 = 0, $input_ptr = 0, $output_ptr = 0, $pixcode = 0, $row = 0, $row_index = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $colorindex0 = $10;
 $11 = $cquantize;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $colorindex1 = $15;
 $16 = $cquantize;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $colorindex2 = $20;
 $21 = $0;
 $22 = (($21) + 112|0);
 $23 = HEAP32[$22>>2]|0;
 $width = $23;
 $row = 0;
 while(1) {
  $24 = $row;
  $25 = $3;
  $26 = ($24|0)<($25|0);
  if (!($26)) {
   break;
  }
  $27 = $cquantize;
  $28 = (($27) + 48|0);
  $29 = HEAP32[$28>>2]|0;
  $row_index = $29;
  $30 = $row;
  $31 = $1;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $input_ptr = $33;
  $34 = $row;
  $35 = $2;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $output_ptr = $37;
  $38 = $row_index;
  $39 = $cquantize;
  $40 = (($39) + 52|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($38<<6)|0);
  $dither0 = $42;
  $43 = $row_index;
  $44 = $cquantize;
  $45 = (($44) + 52|0);
  $46 = (($45) + 4|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + ($43<<6)|0);
  $dither1 = $48;
  $49 = $row_index;
  $50 = $cquantize;
  $51 = (($50) + 52|0);
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($49<<6)|0);
  $dither2 = $54;
  $col_index = 0;
  $55 = $width;
  $col = $55;
  while(1) {
   $56 = $col;
   $57 = ($56>>>0)>(0);
   if (!($57)) {
    break;
   }
   $58 = $input_ptr;
   $59 = (($58) + 1|0);
   $input_ptr = $59;
   $60 = HEAP8[$58>>0]|0;
   $61 = $60&255;
   $62 = $col_index;
   $63 = $dither0;
   $64 = (($63) + ($62<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($61) + ($65))|0;
   $67 = $colorindex0;
   $68 = (($67) + ($66)|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = $69&255;
   $pixcode = $70;
   $71 = $input_ptr;
   $72 = (($71) + 1|0);
   $input_ptr = $72;
   $73 = HEAP8[$71>>0]|0;
   $74 = $73&255;
   $75 = $col_index;
   $76 = $dither1;
   $77 = (($76) + ($75<<2)|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = (($74) + ($78))|0;
   $80 = $colorindex1;
   $81 = (($80) + ($79)|0);
   $82 = HEAP8[$81>>0]|0;
   $83 = $82&255;
   $84 = $pixcode;
   $85 = (($84) + ($83))|0;
   $pixcode = $85;
   $86 = $input_ptr;
   $87 = (($86) + 1|0);
   $input_ptr = $87;
   $88 = HEAP8[$86>>0]|0;
   $89 = $88&255;
   $90 = $col_index;
   $91 = $dither2;
   $92 = (($91) + ($90<<2)|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (($89) + ($93))|0;
   $95 = $colorindex2;
   $96 = (($95) + ($94)|0);
   $97 = HEAP8[$96>>0]|0;
   $98 = $97&255;
   $99 = $pixcode;
   $100 = (($99) + ($98))|0;
   $pixcode = $100;
   $101 = $pixcode;
   $102 = $101&255;
   $103 = $output_ptr;
   $104 = (($103) + 1|0);
   $output_ptr = $104;
   HEAP8[$103>>0] = $102;
   $105 = $col_index;
   $106 = (($105) + 1)|0;
   $107 = $106 & 15;
   $col_index = $107;
   $108 = $col;
   $109 = (($108) + -1)|0;
   $col = $109;
  }
  $110 = $row_index;
  $111 = (($110) + 1)|0;
  $112 = $111 & 15;
  $row_index = $112;
  $113 = $row_index;
  $114 = $cquantize;
  $115 = (($114) + 48|0);
  HEAP32[$115>>2] = $113;
  $116 = $row;
  $117 = (($116) + 1)|0;
  $row = $117;
 }
 STACKTOP = sp;return;
}
function _quantize_ord_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $ci = 0, $col = 0, $col_index = 0, $colorindex_ci = 0, $cquantize = 0, $dither = 0, $input_ptr = 0, $nc = 0, $output_ptr = 0;
 var $row = 0, $row_index = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $0;
 $8 = (($7) + 120|0);
 $9 = HEAP32[$8>>2]|0;
 $nc = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $row = 0;
 while(1) {
  $13 = $row;
  $14 = $3;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $row;
  $17 = $2;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $width;
  $21 = $20;
  _jzero_far($19,$21);
  $22 = $cquantize;
  $23 = (($22) + 48|0);
  $24 = HEAP32[$23>>2]|0;
  $row_index = $24;
  $ci = 0;
  while(1) {
   $25 = $ci;
   $26 = $nc;
   $27 = ($25|0)<($26|0);
   if (!($27)) {
    break;
   }
   $28 = $row;
   $29 = $1;
   $30 = (($29) + ($28<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $ci;
   $33 = (($31) + ($32)|0);
   $input_ptr = $33;
   $34 = $row;
   $35 = $2;
   $36 = (($35) + ($34<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   $output_ptr = $37;
   $38 = $ci;
   $39 = $cquantize;
   $40 = (($39) + 24|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + ($38<<2)|0);
   $43 = HEAP32[$42>>2]|0;
   $colorindex_ci = $43;
   $44 = $row_index;
   $45 = $ci;
   $46 = $cquantize;
   $47 = (($46) + 52|0);
   $48 = (($47) + ($45<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + ($44<<6)|0);
   $dither = $50;
   $col_index = 0;
   $51 = $width;
   $col = $51;
   while(1) {
    $52 = $col;
    $53 = ($52>>>0)>(0);
    if (!($53)) {
     break;
    }
    $54 = $input_ptr;
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = $col_index;
    $58 = $dither;
    $59 = (($58) + ($57<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = (($56) + ($60))|0;
    $62 = $colorindex_ci;
    $63 = (($62) + ($61)|0);
    $64 = HEAP8[$63>>0]|0;
    $65 = $64&255;
    $66 = $output_ptr;
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = (($68) + ($65))|0;
    $70 = $69&255;
    HEAP8[$66>>0] = $70;
    $71 = $nc;
    $72 = $input_ptr;
    $73 = (($72) + ($71)|0);
    $input_ptr = $73;
    $74 = $output_ptr;
    $75 = (($74) + 1|0);
    $output_ptr = $75;
    $76 = $col_index;
    $77 = (($76) + 1)|0;
    $78 = $77 & 15;
    $col_index = $78;
    $79 = $col;
    $80 = (($79) + -1)|0;
    $col = $80;
   }
   $81 = $ci;
   $82 = (($81) + 1)|0;
   $ci = $82;
  }
  $83 = $row_index;
  $84 = (($83) + 1)|0;
  $85 = $84 & 15;
  $row_index = $85;
  $86 = $row_index;
  $87 = $cquantize;
  $88 = (($87) + 48|0);
  HEAP32[$88>>2] = $86;
  $89 = $row;
  $90 = (($89) + 1)|0;
  $row = $90;
 }
 STACKTOP = sp;return;
}
function _create_odither_tables($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, $i = 0, $j = 0, $nci = 0, $odither = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $0;
  $6 = (($5) + 120|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $i;
  $10 = $cquantize;
  $11 = (($10) + 32|0);
  $12 = (($11) + ($9<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $nci = $13;
  $odither = 0;
  $j = 0;
  while(1) {
   $14 = $j;
   $15 = $i;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $nci;
   $18 = $j;
   $19 = $cquantize;
   $20 = (($19) + 32|0);
   $21 = (($20) + ($18<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($17|0)==($22|0);
   if ($23) {
    label = 6;
    break;
   }
   $29 = $j;
   $30 = (($29) + 1)|0;
   $j = $30;
  }
  if ((label|0) == 6) {
   label = 0;
   $24 = $j;
   $25 = $cquantize;
   $26 = (($25) + 52|0);
   $27 = (($26) + ($24<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $odither = $28;
  }
  $31 = $odither;
  $32 = ($31|0)==(0|0);
  if ($32) {
   $33 = $0;
   $34 = $nci;
   $35 = (_make_odither_array($33,$34)|0);
   $odither = $35;
  }
  $36 = $odither;
  $37 = $i;
  $38 = $cquantize;
  $39 = (($38) + 52|0);
  $40 = (($39) + ($37<<2)|0);
  HEAP32[$40>>2] = $36;
  $41 = $i;
  $42 = (($41) + 1)|0;
  $i = $42;
 }
 STACKTOP = sp;return;
}
function _quantize_fs_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $belowerr = 0, $bnexterr = 0, $bpreverr = 0, $ci = 0, $col = 0, $colorindex_ci = 0, $colormap_ci = 0, $cquantize = 0, $cur = 0, $delta = 0, $dir = 0, $dirnc = 0;
 var $errorptr = 0, $input_ptr = 0, $nc = 0, $output_ptr = 0, $pixcode = 0, $range_limit = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $0;
 $8 = (($7) + 120|0);
 $9 = HEAP32[$8>>2]|0;
 $nc = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $13 = $0;
 $14 = (($13) + 324|0);
 $15 = HEAP32[$14>>2]|0;
 $range_limit = $15;
 $row = 0;
 while(1) {
  $16 = $row;
  $17 = $3;
  $18 = ($16|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $row;
  $20 = $2;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $width;
  $24 = $23;
  _jzero_far($22,$24);
  $ci = 0;
  while(1) {
   $25 = $ci;
   $26 = $nc;
   $27 = ($25|0)<($26|0);
   if (!($27)) {
    break;
   }
   $28 = $row;
   $29 = $1;
   $30 = (($29) + ($28<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $ci;
   $33 = (($31) + ($32)|0);
   $input_ptr = $33;
   $34 = $row;
   $35 = $2;
   $36 = (($35) + ($34<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   $output_ptr = $37;
   $38 = $cquantize;
   $39 = (($38) + 84|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = $width;
    $43 = (($42) - 1)|0;
    $44 = $nc;
    $45 = Math_imul($43, $44)|0;
    $46 = $input_ptr;
    $47 = (($46) + ($45)|0);
    $input_ptr = $47;
    $48 = $width;
    $49 = (($48) - 1)|0;
    $50 = $output_ptr;
    $51 = (($50) + ($49)|0);
    $output_ptr = $51;
    $dir = -1;
    $52 = $nc;
    $53 = (0 - ($52))|0;
    $dirnc = $53;
    $54 = $ci;
    $55 = $cquantize;
    $56 = (($55) + 68|0);
    $57 = (($56) + ($54<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $width;
    $60 = (($59) + 1)|0;
    $61 = (($58) + ($60<<1)|0);
    $errorptr = $61;
   } else {
    $dir = 1;
    $62 = $nc;
    $dirnc = $62;
    $63 = $ci;
    $64 = $cquantize;
    $65 = (($64) + 68|0);
    $66 = (($65) + ($63<<2)|0);
    $67 = HEAP32[$66>>2]|0;
    $errorptr = $67;
   }
   $68 = $ci;
   $69 = $cquantize;
   $70 = (($69) + 24|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + ($68<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $colorindex_ci = $73;
   $74 = $ci;
   $75 = $cquantize;
   $76 = (($75) + 16|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (($77) + ($74<<2)|0);
   $79 = HEAP32[$78>>2]|0;
   $colormap_ci = $79;
   $cur = 0;
   $bpreverr = 0;
   $belowerr = 0;
   $80 = $width;
   $col = $80;
   while(1) {
    $81 = $col;
    $82 = ($81>>>0)>(0);
    if (!($82)) {
     break;
    }
    $83 = $cur;
    $84 = $dir;
    $85 = $errorptr;
    $86 = (($85) + ($84<<1)|0);
    $87 = HEAP16[$86>>1]|0;
    $88 = $87 << 16 >> 16;
    $89 = (($83) + ($88))|0;
    $90 = (($89) + 8)|0;
    $91 = $90 >> 4;
    $cur = $91;
    $92 = $input_ptr;
    $93 = HEAP8[$92>>0]|0;
    $94 = $93&255;
    $95 = $cur;
    $96 = (($95) + ($94))|0;
    $cur = $96;
    $97 = $cur;
    $98 = $range_limit;
    $99 = (($98) + ($97)|0);
    $100 = HEAP8[$99>>0]|0;
    $101 = $100&255;
    $cur = $101;
    $102 = $cur;
    $103 = $colorindex_ci;
    $104 = (($103) + ($102)|0);
    $105 = HEAP8[$104>>0]|0;
    $106 = $105&255;
    $pixcode = $106;
    $107 = $pixcode;
    $108 = $107&255;
    $109 = $108&255;
    $110 = $output_ptr;
    $111 = HEAP8[$110>>0]|0;
    $112 = $111&255;
    $113 = (($112) + ($109))|0;
    $114 = $113&255;
    HEAP8[$110>>0] = $114;
    $115 = $pixcode;
    $116 = $colormap_ci;
    $117 = (($116) + ($115)|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = $118&255;
    $120 = $cur;
    $121 = (($120) - ($119))|0;
    $cur = $121;
    $122 = $cur;
    $bnexterr = $122;
    $123 = $cur;
    $124 = $123<<1;
    $delta = $124;
    $125 = $delta;
    $126 = $cur;
    $127 = (($126) + ($125))|0;
    $cur = $127;
    $128 = $bpreverr;
    $129 = $cur;
    $130 = (($128) + ($129))|0;
    $131 = $130&65535;
    $132 = $errorptr;
    HEAP16[$132>>1] = $131;
    $133 = $delta;
    $134 = $cur;
    $135 = (($134) + ($133))|0;
    $cur = $135;
    $136 = $belowerr;
    $137 = $cur;
    $138 = (($136) + ($137))|0;
    $bpreverr = $138;
    $139 = $bnexterr;
    $belowerr = $139;
    $140 = $delta;
    $141 = $cur;
    $142 = (($141) + ($140))|0;
    $cur = $142;
    $143 = $dirnc;
    $144 = $input_ptr;
    $145 = (($144) + ($143)|0);
    $input_ptr = $145;
    $146 = $dir;
    $147 = $output_ptr;
    $148 = (($147) + ($146)|0);
    $output_ptr = $148;
    $149 = $dir;
    $150 = $errorptr;
    $151 = (($150) + ($149<<1)|0);
    $errorptr = $151;
    $152 = $col;
    $153 = (($152) + -1)|0;
    $col = $153;
   }
   $154 = $bpreverr;
   $155 = $154&65535;
   $156 = $errorptr;
   HEAP16[$156>>1] = $155;
   $157 = $ci;
   $158 = (($157) + 1)|0;
   $ci = $158;
  }
  $159 = $cquantize;
  $160 = (($159) + 84|0);
  $161 = HEAP32[$160>>2]|0;
  $162 = ($161|0)!=(0);
  $163 = $162 ? 0 : 1;
  $164 = $cquantize;
  $165 = (($164) + 84|0);
  HEAP32[$165>>2] = $163;
  $166 = $row;
  $167 = (($166) + 1)|0;
  $row = $167;
 }
 STACKTOP = sp;return;
}
function _make_odither_array($cinfo,$ncolors) {
 $cinfo = $cinfo|0;
 $ncolors = $ncolors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $den = 0, $j = 0, $k = 0, $num = 0, $odither = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $ncolors;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 127]($6,1,1024)|0);
 $odither = $7;
 $8 = $1;
 $9 = (($8) - 1)|0;
 $10 = $9<<9;
 $den = $10;
 $j = 0;
 while(1) {
  $11 = $j;
  $12 = ($11|0)<(16);
  if (!($12)) {
   break;
  }
  $k = 0;
  while(1) {
   $13 = $k;
   $14 = ($13|0)<(16);
   if (!($14)) {
    break;
   }
   $15 = $k;
   $16 = $j;
   $17 = (9248 + ($16<<4)|0);
   $18 = (($17) + ($15)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = $20<<1;
   $22 = (255 - ($21))|0;
   $23 = ($22*255)|0;
   $num = $23;
   $24 = $num;
   $25 = ($24|0)<(0);
   if ($25) {
    $26 = $num;
    $27 = (0 - ($26))|0;
    $28 = $den;
    $29 = (($27|0) / ($28|0))&-1;
    $30 = (0 - ($29))|0;
    $39 = $30;
   } else {
    $31 = $num;
    $32 = $den;
    $33 = (($31|0) / ($32|0))&-1;
    $39 = $33;
   }
   $34 = $k;
   $35 = $j;
   $36 = $odither;
   $37 = (($36) + ($35<<6)|0);
   $38 = (($37) + ($34<<2)|0);
   HEAP32[$38>>2] = $39;
   $40 = $k;
   $41 = (($40) + 1)|0;
   $k = $41;
  }
  $42 = $j;
  $43 = (($42) + 1)|0;
  $j = $43;
 }
 $44 = $odither;
 STACKTOP = sp;return ($44|0);
}
function _jinit_2pass_quantizer($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cquantize = 0, $desired = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,44)|0);
 $cquantize = $6;
 $7 = $cquantize;
 $8 = $0;
 $9 = (($8) + 460|0);
 HEAP32[$9>>2] = $7;
 $10 = $cquantize;
 HEAP32[$10>>2] = 84;
 $11 = $cquantize;
 $12 = (($11) + 12|0);
 HEAP32[$12>>2] = 85;
 $13 = $cquantize;
 $14 = (($13) + 32|0);
 HEAP32[$14>>2] = 0;
 $15 = $cquantize;
 $16 = (($15) + 40|0);
 HEAP32[$16>>2] = 0;
 $17 = $0;
 $18 = (($17) + 120|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(3);
 if ($20) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 47;
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vi[$26 & 127]($27);
 }
 $28 = $0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = $0;
 $33 = (FUNCTION_TABLE_iiii[$31 & 127]($32,1,128)|0);
 $34 = $cquantize;
 $35 = (($34) + 24|0);
 HEAP32[$35>>2] = $33;
 $i = 0;
 while(1) {
  $36 = $i;
  $37 = ($36|0)<(32);
  if (!($37)) {
   break;
  }
  $38 = $0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $0;
  $44 = (FUNCTION_TABLE_iiii[$42 & 127]($43,1,4096)|0);
  $45 = $i;
  $46 = $cquantize;
  $47 = (($46) + 24|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($45<<2)|0);
  HEAP32[$49>>2] = $44;
  $50 = $i;
  $51 = (($50) + 1)|0;
  $i = $51;
 }
 $52 = $cquantize;
 $53 = (($52) + 28|0);
 HEAP32[$53>>2] = 1;
 $54 = $0;
 $55 = (($54) + 108|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = ($56|0)!=(0);
 if ($57) {
  $58 = $0;
  $59 = (($58) + 96|0);
  $60 = HEAP32[$59>>2]|0;
  $desired = $60;
  $61 = $desired;
  $62 = ($61|0)<(8);
  if ($62) {
   $63 = $0;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($64) + 20|0);
   HEAP32[$65>>2] = 56;
   $66 = $0;
   $67 = HEAP32[$66>>2]|0;
   $68 = (($67) + 24|0);
   HEAP32[$68>>2] = 8;
   $69 = $0;
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$70>>2]|0;
   $72 = $0;
   FUNCTION_TABLE_vi[$71 & 127]($72);
  }
  $73 = $desired;
  $74 = ($73|0)>(256);
  if ($74) {
   $75 = $0;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($76) + 20|0);
   HEAP32[$77>>2] = 57;
   $78 = $0;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + 24|0);
   HEAP32[$80>>2] = 256;
   $81 = $0;
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$82>>2]|0;
   $84 = $0;
   FUNCTION_TABLE_vi[$83 & 127]($84);
  }
  $85 = $0;
  $86 = (($85) + 4|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 8|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $0;
  $91 = $desired;
  $92 = (FUNCTION_TABLE_iiiii[$89 & 127]($90,1,$91,3)|0);
  $93 = $cquantize;
  $94 = (($93) + 16|0);
  HEAP32[$94>>2] = $92;
  $95 = $desired;
  $96 = $cquantize;
  $97 = (($96) + 20|0);
  HEAP32[$97>>2] = $95;
 } else {
  $98 = $cquantize;
  $99 = (($98) + 16|0);
  HEAP32[$99>>2] = 0;
 }
 $100 = $0;
 $101 = (($100) + 88|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = ($102|0)!=(0);
 if ($103) {
  $104 = $0;
  $105 = (($104) + 88|0);
  HEAP32[$105>>2] = 2;
 }
 $106 = $0;
 $107 = (($106) + 88|0);
 $108 = HEAP32[$107>>2]|0;
 $109 = ($108|0)==(2);
 if (!($109)) {
  STACKTOP = sp;return;
 }
 $110 = $0;
 $111 = (($110) + 4|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = (($112) + 4|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $0;
 $116 = $0;
 $117 = (($116) + 112|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = (($118) + 2)|0;
 $120 = ($119*6)|0;
 $121 = (FUNCTION_TABLE_iiii[$114 & 127]($115,1,$120)|0);
 $122 = $cquantize;
 $123 = (($122) + 32|0);
 HEAP32[$123>>2] = $121;
 $124 = $0;
 _init_error_limit($124);
 STACKTOP = sp;return;
}
function _start_pass_2_quant($cinfo,$is_pre_scan) {
 $cinfo = $cinfo|0;
 $is_pre_scan = $is_pre_scan|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $arraysize = 0, $cquantize = 0, $histogram = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $is_pre_scan;
 $2 = $0;
 $3 = (($2) + 460|0);
 $4 = HEAP32[$3>>2]|0;
 $cquantize = $4;
 $5 = $cquantize;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $histogram = $7;
 $8 = $0;
 $9 = (($8) + 88|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $0;
  $13 = (($12) + 88|0);
  HEAP32[$13>>2] = 2;
 }
 $14 = $1;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $cquantize;
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = 86;
  $18 = $cquantize;
  $19 = (($18) + 8|0);
  HEAP32[$19>>2] = 87;
  $20 = $cquantize;
  $21 = (($20) + 28|0);
  HEAP32[$21>>2] = 1;
 } else {
  $22 = $0;
  $23 = (($22) + 88|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)==(2);
  if ($25) {
   $26 = $cquantize;
   $27 = (($26) + 4|0);
   HEAP32[$27>>2] = 88;
  } else {
   $28 = $cquantize;
   $29 = (($28) + 4|0);
   HEAP32[$29>>2] = 89;
  }
  $30 = $cquantize;
  $31 = (($30) + 8|0);
  HEAP32[$31>>2] = 90;
  $32 = $0;
  $33 = (($32) + 132|0);
  $34 = HEAP32[$33>>2]|0;
  $i = $34;
  $35 = $i;
  $36 = ($35|0)<(1);
  if ($36) {
   $37 = $0;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + 20|0);
   HEAP32[$39>>2] = 56;
   $40 = $0;
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + 24|0);
   HEAP32[$42>>2] = 1;
   $43 = $0;
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $0;
   FUNCTION_TABLE_vi[$45 & 127]($46);
  }
  $47 = $i;
  $48 = ($47|0)>(256);
  if ($48) {
   $49 = $0;
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + 20|0);
   HEAP32[$51>>2] = 57;
   $52 = $0;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + 24|0);
   HEAP32[$54>>2] = 256;
   $55 = $0;
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$56>>2]|0;
   $58 = $0;
   FUNCTION_TABLE_vi[$57 & 127]($58);
  }
  $59 = $0;
  $60 = (($59) + 88|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($61|0)==(2);
  if ($62) {
   $63 = $0;
   $64 = (($63) + 112|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($65) + 2)|0;
   $67 = ($66*6)|0;
   $arraysize = $67;
   $68 = $cquantize;
   $69 = (($68) + 32|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ($70|0)==(0|0);
   if ($71) {
    $72 = $0;
    $73 = (($72) + 4|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + 4|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = $0;
    $78 = $arraysize;
    $79 = (FUNCTION_TABLE_iiii[$76 & 127]($77,1,$78)|0);
    $80 = $cquantize;
    $81 = (($80) + 32|0);
    HEAP32[$81>>2] = $79;
   }
   $82 = $cquantize;
   $83 = (($82) + 32|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = $arraysize;
   _jzero_far($84,$85);
   $86 = $cquantize;
   $87 = (($86) + 40|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)==(0|0);
   if ($89) {
    $90 = $0;
    _init_error_limit($90);
   }
   $91 = $cquantize;
   $92 = (($91) + 36|0);
   HEAP32[$92>>2] = 0;
  }
 }
 $93 = $cquantize;
 $94 = (($93) + 28|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = ($95|0)!=(0);
 if (!($96)) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $97 = $i;
  $98 = ($97|0)<(32);
  if (!($98)) {
   break;
  }
  $99 = $i;
  $100 = $histogram;
  $101 = (($100) + ($99<<2)|0);
  $102 = HEAP32[$101>>2]|0;
  _jzero_far($102,4096);
  $103 = $i;
  $104 = (($103) + 1)|0;
  $i = $104;
 }
 $105 = $cquantize;
 $106 = (($105) + 28|0);
 HEAP32[$106>>2] = 0;
 STACKTOP = sp;return;
}
function _new_color_map_2_quant($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $cquantize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $cquantize;
 $5 = (($4) + 28|0);
 HEAP32[$5>>2] = 1;
 STACKTOP = sp;return;
}
function _init_error_limit($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, $in = 0, $out = 0, $table = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 127]($8,1,2044)|0);
 $table = $9;
 $10 = $table;
 $11 = (($10) + 1020|0);
 $table = $11;
 $12 = $table;
 $13 = $cquantize;
 $14 = (($13) + 40|0);
 HEAP32[$14>>2] = $12;
 $out = 0;
 $in = 0;
 while(1) {
  $15 = $in;
  $16 = ($15|0)<(16);
  if (!($16)) {
   break;
  }
  $17 = $out;
  $18 = $in;
  $19 = $table;
  $20 = (($19) + ($18<<2)|0);
  HEAP32[$20>>2] = $17;
  $21 = $out;
  $22 = (0 - ($21))|0;
  $23 = $in;
  $24 = (0 - ($23))|0;
  $25 = $table;
  $26 = (($25) + ($24<<2)|0);
  HEAP32[$26>>2] = $22;
  $27 = $in;
  $28 = (($27) + 1)|0;
  $in = $28;
  $29 = $out;
  $30 = (($29) + 1)|0;
  $out = $30;
 }
 while(1) {
  $31 = $in;
  $32 = ($31|0)<(48);
  if (!($32)) {
   break;
  }
  $33 = $out;
  $34 = $in;
  $35 = $table;
  $36 = (($35) + ($34<<2)|0);
  HEAP32[$36>>2] = $33;
  $37 = $out;
  $38 = (0 - ($37))|0;
  $39 = $in;
  $40 = (0 - ($39))|0;
  $41 = $table;
  $42 = (($41) + ($40<<2)|0);
  HEAP32[$42>>2] = $38;
  $43 = $in;
  $44 = (($43) + 1)|0;
  $in = $44;
  $45 = $in;
  $46 = $45 & 1;
  $47 = ($46|0)!=(0);
  $48 = $47 ? 0 : 1;
  $49 = $out;
  $50 = (($49) + ($48))|0;
  $out = $50;
 }
 while(1) {
  $51 = $in;
  $52 = ($51|0)<=(255);
  if (!($52)) {
   break;
  }
  $53 = $out;
  $54 = $in;
  $55 = $table;
  $56 = (($55) + ($54<<2)|0);
  HEAP32[$56>>2] = $53;
  $57 = $out;
  $58 = (0 - ($57))|0;
  $59 = $in;
  $60 = (0 - ($59))|0;
  $61 = $table;
  $62 = (($61) + ($60<<2)|0);
  HEAP32[$62>>2] = $58;
  $63 = $in;
  $64 = (($63) + 1)|0;
  $in = $64;
 }
 STACKTOP = sp;return;
}
function _prescan_quantize($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $cquantize = 0, $histogram = 0, $histp = 0;
 var $ptr = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $row = 0;
 while(1) {
  $13 = $row;
  $14 = $3;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $row;
  $17 = $1;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $ptr = $19;
  $20 = $width;
  $col = $20;
  while(1) {
   $21 = $col;
   $22 = ($21>>>0)>(0);
   if (!($22)) {
    break;
   }
   $23 = $ptr;
   $24 = (($23) + 2|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = $26 >> 3;
   $28 = $ptr;
   $29 = (($28) + 1|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = $31 >> 2;
   $33 = $ptr;
   $34 = HEAP8[$33>>0]|0;
   $35 = $34&255;
   $36 = $35 >> 3;
   $37 = $histogram;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($32<<6)|0);
   $41 = (($40) + ($27<<1)|0);
   $histp = $41;
   $42 = $histp;
   $43 = HEAP16[$42>>1]|0;
   $44 = (($43) + 1)<<16>>16;
   HEAP16[$42>>1] = $44;
   $45 = $44&65535;
   $46 = ($45|0)<=(0);
   if ($46) {
    $47 = $histp;
    $48 = HEAP16[$47>>1]|0;
    $49 = (($48) + -1)<<16>>16;
    HEAP16[$47>>1] = $49;
   }
   $50 = $ptr;
   $51 = (($50) + 3|0);
   $ptr = $51;
   $52 = $col;
   $53 = (($52) + -1)|0;
   $col = $53;
  }
  $54 = $row;
  $55 = (($54) + 1)|0;
  $row = $55;
 }
 STACKTOP = sp;return;
}
function _finish_pass1($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $cquantize;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 136|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = $cquantize;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 _select_colors($9,$12);
 $13 = $cquantize;
 $14 = (($13) + 28|0);
 HEAP32[$14>>2] = 1;
 STACKTOP = sp;return;
}
function _pass2_fs_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $belowerr0 = 0, $belowerr1 = 0, $belowerr2 = 0, $bnexterr = 0, $bpreverr0 = 0, $bpreverr1 = 0, $bpreverr2 = 0, $cachep = 0, $col = 0, $colormap0 = 0, $colormap1 = 0, $colormap2 = 0, $cquantize = 0;
 var $cur0 = 0, $cur1 = 0, $cur2 = 0, $delta = 0, $dir = 0, $dir3 = 0, $error_limit = 0, $errorptr = 0, $histogram = 0, $inptr = 0, $outptr = 0, $pixcode = 0, $range_limit = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $13 = $0;
 $14 = (($13) + 324|0);
 $15 = HEAP32[$14>>2]|0;
 $range_limit = $15;
 $16 = $cquantize;
 $17 = (($16) + 40|0);
 $18 = HEAP32[$17>>2]|0;
 $error_limit = $18;
 $19 = $0;
 $20 = (($19) + 136|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $colormap0 = $22;
 $23 = $0;
 $24 = (($23) + 136|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $colormap1 = $27;
 $28 = $0;
 $29 = (($28) + 136|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $colormap2 = $32;
 $row = 0;
 while(1) {
  $33 = $row;
  $34 = $3;
  $35 = ($33|0)<($34|0);
  if (!($35)) {
   break;
  }
  $36 = $row;
  $37 = $1;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inptr = $39;
  $40 = $row;
  $41 = $2;
  $42 = (($41) + ($40<<2)|0);
  $43 = HEAP32[$42>>2]|0;
  $outptr = $43;
  $44 = $cquantize;
  $45 = (($44) + 36|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = $width;
   $49 = (($48) - 1)|0;
   $50 = ($49*3)|0;
   $51 = $inptr;
   $52 = (($51) + ($50)|0);
   $inptr = $52;
   $53 = $width;
   $54 = (($53) - 1)|0;
   $55 = $outptr;
   $56 = (($55) + ($54)|0);
   $outptr = $56;
   $dir = -1;
   $dir3 = -3;
   $57 = $cquantize;
   $58 = (($57) + 32|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $width;
   $61 = (($60) + 1)|0;
   $62 = ($61*3)|0;
   $63 = (($59) + ($62<<1)|0);
   $errorptr = $63;
   $64 = $cquantize;
   $65 = (($64) + 36|0);
   HEAP32[$65>>2] = 0;
  } else {
   $dir = 1;
   $dir3 = 3;
   $66 = $cquantize;
   $67 = (($66) + 32|0);
   $68 = HEAP32[$67>>2]|0;
   $errorptr = $68;
   $69 = $cquantize;
   $70 = (($69) + 36|0);
   HEAP32[$70>>2] = 1;
  }
  $cur2 = 0;
  $cur1 = 0;
  $cur0 = 0;
  $belowerr2 = 0;
  $belowerr1 = 0;
  $belowerr0 = 0;
  $bpreverr2 = 0;
  $bpreverr1 = 0;
  $bpreverr0 = 0;
  $71 = $width;
  $col = $71;
  while(1) {
   $72 = $col;
   $73 = ($72>>>0)>(0);
   if (!($73)) {
    break;
   }
   $74 = $cur0;
   $75 = $dir3;
   $76 = (($75) + 0)|0;
   $77 = $errorptr;
   $78 = (($77) + ($76<<1)|0);
   $79 = HEAP16[$78>>1]|0;
   $80 = $79 << 16 >> 16;
   $81 = (($74) + ($80))|0;
   $82 = (($81) + 8)|0;
   $83 = $82 >> 4;
   $cur0 = $83;
   $84 = $cur1;
   $85 = $dir3;
   $86 = (($85) + 1)|0;
   $87 = $errorptr;
   $88 = (($87) + ($86<<1)|0);
   $89 = HEAP16[$88>>1]|0;
   $90 = $89 << 16 >> 16;
   $91 = (($84) + ($90))|0;
   $92 = (($91) + 8)|0;
   $93 = $92 >> 4;
   $cur1 = $93;
   $94 = $cur2;
   $95 = $dir3;
   $96 = (($95) + 2)|0;
   $97 = $errorptr;
   $98 = (($97) + ($96<<1)|0);
   $99 = HEAP16[$98>>1]|0;
   $100 = $99 << 16 >> 16;
   $101 = (($94) + ($100))|0;
   $102 = (($101) + 8)|0;
   $103 = $102 >> 4;
   $cur2 = $103;
   $104 = $cur0;
   $105 = $error_limit;
   $106 = (($105) + ($104<<2)|0);
   $107 = HEAP32[$106>>2]|0;
   $cur0 = $107;
   $108 = $cur1;
   $109 = $error_limit;
   $110 = (($109) + ($108<<2)|0);
   $111 = HEAP32[$110>>2]|0;
   $cur1 = $111;
   $112 = $cur2;
   $113 = $error_limit;
   $114 = (($113) + ($112<<2)|0);
   $115 = HEAP32[$114>>2]|0;
   $cur2 = $115;
   $116 = $inptr;
   $117 = HEAP8[$116>>0]|0;
   $118 = $117&255;
   $119 = $cur0;
   $120 = (($119) + ($118))|0;
   $cur0 = $120;
   $121 = $inptr;
   $122 = (($121) + 1|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = $123&255;
   $125 = $cur1;
   $126 = (($125) + ($124))|0;
   $cur1 = $126;
   $127 = $inptr;
   $128 = (($127) + 2|0);
   $129 = HEAP8[$128>>0]|0;
   $130 = $129&255;
   $131 = $cur2;
   $132 = (($131) + ($130))|0;
   $cur2 = $132;
   $133 = $cur0;
   $134 = $range_limit;
   $135 = (($134) + ($133)|0);
   $136 = HEAP8[$135>>0]|0;
   $137 = $136&255;
   $cur0 = $137;
   $138 = $cur1;
   $139 = $range_limit;
   $140 = (($139) + ($138)|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141&255;
   $cur1 = $142;
   $143 = $cur2;
   $144 = $range_limit;
   $145 = (($144) + ($143)|0);
   $146 = HEAP8[$145>>0]|0;
   $147 = $146&255;
   $cur2 = $147;
   $148 = $cur2;
   $149 = $148 >> 3;
   $150 = $cur1;
   $151 = $150 >> 2;
   $152 = $cur0;
   $153 = $152 >> 3;
   $154 = $histogram;
   $155 = (($154) + ($153<<2)|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = (($156) + ($151<<6)|0);
   $158 = (($157) + ($149<<1)|0);
   $cachep = $158;
   $159 = $cachep;
   $160 = HEAP16[$159>>1]|0;
   $161 = $160&65535;
   $162 = ($161|0)==(0);
   if ($162) {
    $163 = $0;
    $164 = $cur0;
    $165 = $164 >> 3;
    $166 = $cur1;
    $167 = $166 >> 2;
    $168 = $cur2;
    $169 = $168 >> 3;
    _fill_inverse_cmap($163,$165,$167,$169);
   }
   $170 = $cachep;
   $171 = HEAP16[$170>>1]|0;
   $172 = $171&65535;
   $173 = (($172) - 1)|0;
   $pixcode = $173;
   $174 = $pixcode;
   $175 = $174&255;
   $176 = $outptr;
   HEAP8[$176>>0] = $175;
   $177 = $pixcode;
   $178 = $colormap0;
   $179 = (($178) + ($177)|0);
   $180 = HEAP8[$179>>0]|0;
   $181 = $180&255;
   $182 = $cur0;
   $183 = (($182) - ($181))|0;
   $cur0 = $183;
   $184 = $pixcode;
   $185 = $colormap1;
   $186 = (($185) + ($184)|0);
   $187 = HEAP8[$186>>0]|0;
   $188 = $187&255;
   $189 = $cur1;
   $190 = (($189) - ($188))|0;
   $cur1 = $190;
   $191 = $pixcode;
   $192 = $colormap2;
   $193 = (($192) + ($191)|0);
   $194 = HEAP8[$193>>0]|0;
   $195 = $194&255;
   $196 = $cur2;
   $197 = (($196) - ($195))|0;
   $cur2 = $197;
   $198 = $cur0;
   $bnexterr = $198;
   $199 = $cur0;
   $200 = $199<<1;
   $delta = $200;
   $201 = $delta;
   $202 = $cur0;
   $203 = (($202) + ($201))|0;
   $cur0 = $203;
   $204 = $bpreverr0;
   $205 = $cur0;
   $206 = (($204) + ($205))|0;
   $207 = $206&65535;
   $208 = $errorptr;
   HEAP16[$208>>1] = $207;
   $209 = $delta;
   $210 = $cur0;
   $211 = (($210) + ($209))|0;
   $cur0 = $211;
   $212 = $belowerr0;
   $213 = $cur0;
   $214 = (($212) + ($213))|0;
   $bpreverr0 = $214;
   $215 = $bnexterr;
   $belowerr0 = $215;
   $216 = $delta;
   $217 = $cur0;
   $218 = (($217) + ($216))|0;
   $cur0 = $218;
   $219 = $cur1;
   $bnexterr = $219;
   $220 = $cur1;
   $221 = $220<<1;
   $delta = $221;
   $222 = $delta;
   $223 = $cur1;
   $224 = (($223) + ($222))|0;
   $cur1 = $224;
   $225 = $bpreverr1;
   $226 = $cur1;
   $227 = (($225) + ($226))|0;
   $228 = $227&65535;
   $229 = $errorptr;
   $230 = (($229) + 2|0);
   HEAP16[$230>>1] = $228;
   $231 = $delta;
   $232 = $cur1;
   $233 = (($232) + ($231))|0;
   $cur1 = $233;
   $234 = $belowerr1;
   $235 = $cur1;
   $236 = (($234) + ($235))|0;
   $bpreverr1 = $236;
   $237 = $bnexterr;
   $belowerr1 = $237;
   $238 = $delta;
   $239 = $cur1;
   $240 = (($239) + ($238))|0;
   $cur1 = $240;
   $241 = $cur2;
   $bnexterr = $241;
   $242 = $cur2;
   $243 = $242<<1;
   $delta = $243;
   $244 = $delta;
   $245 = $cur2;
   $246 = (($245) + ($244))|0;
   $cur2 = $246;
   $247 = $bpreverr2;
   $248 = $cur2;
   $249 = (($247) + ($248))|0;
   $250 = $249&65535;
   $251 = $errorptr;
   $252 = (($251) + 4|0);
   HEAP16[$252>>1] = $250;
   $253 = $delta;
   $254 = $cur2;
   $255 = (($254) + ($253))|0;
   $cur2 = $255;
   $256 = $belowerr2;
   $257 = $cur2;
   $258 = (($256) + ($257))|0;
   $bpreverr2 = $258;
   $259 = $bnexterr;
   $belowerr2 = $259;
   $260 = $delta;
   $261 = $cur2;
   $262 = (($261) + ($260))|0;
   $cur2 = $262;
   $263 = $dir3;
   $264 = $inptr;
   $265 = (($264) + ($263)|0);
   $inptr = $265;
   $266 = $dir;
   $267 = $outptr;
   $268 = (($267) + ($266)|0);
   $outptr = $268;
   $269 = $dir3;
   $270 = $errorptr;
   $271 = (($270) + ($269<<1)|0);
   $errorptr = $271;
   $272 = $col;
   $273 = (($272) + -1)|0;
   $col = $273;
  }
  $274 = $bpreverr0;
  $275 = $274&65535;
  $276 = $errorptr;
  HEAP16[$276>>1] = $275;
  $277 = $bpreverr1;
  $278 = $277&65535;
  $279 = $errorptr;
  $280 = (($279) + 2|0);
  HEAP16[$280>>1] = $278;
  $281 = $bpreverr2;
  $282 = $281&65535;
  $283 = $errorptr;
  $284 = (($283) + 4|0);
  HEAP16[$284>>1] = $282;
  $285 = $row;
  $286 = (($285) + 1)|0;
  $row = $286;
 }
 STACKTOP = sp;return;
}
function _pass2_no_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $c0 = 0, $c1 = 0, $c2 = 0, $cachep = 0, $col = 0, $cquantize = 0, $histogram = 0, $inptr = 0, $outptr = 0, $row = 0, $width = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $row = 0;
 while(1) {
  $13 = $row;
  $14 = $3;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $row;
  $17 = $1;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $inptr = $19;
  $20 = $row;
  $21 = $2;
  $22 = (($21) + ($20<<2)|0);
  $23 = HEAP32[$22>>2]|0;
  $outptr = $23;
  $24 = $width;
  $col = $24;
  while(1) {
   $25 = $col;
   $26 = ($25>>>0)>(0);
   if (!($26)) {
    break;
   }
   $27 = $inptr;
   $28 = (($27) + 1|0);
   $inptr = $28;
   $29 = HEAP8[$27>>0]|0;
   $30 = $29&255;
   $31 = $30 >> 3;
   $c0 = $31;
   $32 = $inptr;
   $33 = (($32) + 1|0);
   $inptr = $33;
   $34 = HEAP8[$32>>0]|0;
   $35 = $34&255;
   $36 = $35 >> 2;
   $c1 = $36;
   $37 = $inptr;
   $38 = (($37) + 1|0);
   $inptr = $38;
   $39 = HEAP8[$37>>0]|0;
   $40 = $39&255;
   $41 = $40 >> 3;
   $c2 = $41;
   $42 = $c2;
   $43 = $c1;
   $44 = $c0;
   $45 = $histogram;
   $46 = (($45) + ($44<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($43<<6)|0);
   $49 = (($48) + ($42<<1)|0);
   $cachep = $49;
   $50 = $cachep;
   $51 = HEAP16[$50>>1]|0;
   $52 = $51&65535;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = $0;
    $55 = $c0;
    $56 = $c1;
    $57 = $c2;
    _fill_inverse_cmap($54,$55,$56,$57);
   }
   $58 = $cachep;
   $59 = HEAP16[$58>>1]|0;
   $60 = $59&65535;
   $61 = (($60) - 1)|0;
   $62 = $61&255;
   $63 = $outptr;
   $64 = (($63) + 1|0);
   $outptr = $64;
   HEAP8[$63>>0] = $62;
   $65 = $col;
   $66 = (($65) + -1)|0;
   $col = $66;
  }
  $67 = $row;
  $68 = (($67) + 1)|0;
  $row = $68;
 }
 STACKTOP = sp;return;
}
function _finish_pass2($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _fill_inverse_cmap($cinfo,$c0,$c1,$c2) {
 $cinfo = $cinfo|0;
 $c0 = $c0|0;
 $c1 = $c1|0;
 $c2 = $c2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $bestcolor = 0, $cachep = 0, $colorlist = 0, $cptr = 0, $cquantize = 0, $histogram = 0, $ic0 = 0;
 var $ic1 = 0, $ic2 = 0, $minc0 = 0, $minc1 = 0, $minc2 = 0, $numcolors = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 448|0;
 $colorlist = sp + 192|0;
 $bestcolor = sp + 64|0;
 $0 = $cinfo;
 $1 = $c0;
 $2 = $c1;
 $3 = $c2;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $1;
 $11 = $10 >> 2;
 $1 = $11;
 $12 = $2;
 $13 = $12 >> 3;
 $2 = $13;
 $14 = $3;
 $15 = $14 >> 2;
 $3 = $15;
 $16 = $1;
 $17 = $16 << 5;
 $18 = (($17) + 4)|0;
 $minc0 = $18;
 $19 = $2;
 $20 = $19 << 5;
 $21 = (($20) + 2)|0;
 $minc1 = $21;
 $22 = $3;
 $23 = $22 << 5;
 $24 = (($23) + 4)|0;
 $minc2 = $24;
 $25 = $0;
 $26 = $minc0;
 $27 = $minc1;
 $28 = $minc2;
 $29 = (_find_nearby_colors($25,$26,$27,$28,$colorlist)|0);
 $numcolors = $29;
 $30 = $0;
 $31 = $minc0;
 $32 = $minc1;
 $33 = $minc2;
 $34 = $numcolors;
 _find_best_colors($30,$31,$32,$33,$34,$colorlist,$bestcolor);
 $35 = $1;
 $36 = $35 << 2;
 $1 = $36;
 $37 = $2;
 $38 = $37 << 3;
 $2 = $38;
 $39 = $3;
 $40 = $39 << 2;
 $3 = $40;
 $cptr = $bestcolor;
 $ic0 = 0;
 while(1) {
  $41 = $ic0;
  $42 = ($41|0)<(4);
  if (!($42)) {
   break;
  }
  $ic1 = 0;
  while(1) {
   $43 = $ic1;
   $44 = ($43|0)<(8);
   if (!($44)) {
    break;
   }
   $45 = $3;
   $46 = $2;
   $47 = $ic1;
   $48 = (($46) + ($47))|0;
   $49 = $1;
   $50 = $ic0;
   $51 = (($49) + ($50))|0;
   $52 = $histogram;
   $53 = (($52) + ($51<<2)|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = (($54) + ($48<<6)|0);
   $56 = (($55) + ($45<<1)|0);
   $cachep = $56;
   $ic2 = 0;
   while(1) {
    $57 = $ic2;
    $58 = ($57|0)<(4);
    if (!($58)) {
     break;
    }
    $59 = $cptr;
    $60 = (($59) + 1|0);
    $cptr = $60;
    $61 = HEAP8[$59>>0]|0;
    $62 = $61&255;
    $63 = (($62) + 1)|0;
    $64 = $63&65535;
    $65 = $cachep;
    $66 = (($65) + 2|0);
    $cachep = $66;
    HEAP16[$65>>1] = $64;
    $67 = $ic2;
    $68 = (($67) + 1)|0;
    $ic2 = $68;
   }
   $69 = $ic1;
   $70 = (($69) + 1)|0;
   $ic1 = $70;
  }
  $71 = $ic0;
  $72 = (($71) + 1)|0;
  $ic0 = $72;
 }
 STACKTOP = sp;return;
}
function _find_nearby_colors($cinfo,$minc0,$minc1,$minc2,$colorlist) {
 $cinfo = $cinfo|0;
 $minc0 = $minc0|0;
 $minc1 = $minc1|0;
 $minc2 = $minc2|0;
 $colorlist = $colorlist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $centerc0 = 0, $centerc1 = 0;
 var $centerc2 = 0, $i = 0, $max_dist = 0, $maxc0 = 0, $maxc1 = 0, $maxc2 = 0, $min_dist = 0, $mindist = 0, $minmaxdist = 0, $ncolors = 0, $numcolors = 0, $tdist = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1104|0;
 $mindist = sp;
 $0 = $cinfo;
 $1 = $minc0;
 $2 = $minc1;
 $3 = $minc2;
 $4 = $colorlist;
 $5 = $0;
 $6 = (($5) + 132|0);
 $7 = HEAP32[$6>>2]|0;
 $numcolors = $7;
 $8 = $1;
 $9 = (($8) + 24)|0;
 $maxc0 = $9;
 $10 = $1;
 $11 = $maxc0;
 $12 = (($10) + ($11))|0;
 $13 = $12 >> 1;
 $centerc0 = $13;
 $14 = $2;
 $15 = (($14) + 28)|0;
 $maxc1 = $15;
 $16 = $2;
 $17 = $maxc1;
 $18 = (($16) + ($17))|0;
 $19 = $18 >> 1;
 $centerc1 = $19;
 $20 = $3;
 $21 = (($20) + 24)|0;
 $maxc2 = $21;
 $22 = $3;
 $23 = $maxc2;
 $24 = (($22) + ($23))|0;
 $25 = $24 >> 1;
 $centerc2 = $25;
 $minmaxdist = 2147483647;
 $i = 0;
 while(1) {
  $26 = $i;
  $27 = $numcolors;
  $28 = ($26|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = $i;
  $30 = $0;
  $31 = (($30) + 136|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($29)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  $x = $36;
  $37 = $x;
  $38 = $1;
  $39 = ($37|0)<($38|0);
  if ($39) {
   $40 = $x;
   $41 = $1;
   $42 = (($40) - ($41))|0;
   $43 = $42<<1;
   $tdist = $43;
   $44 = $tdist;
   $45 = $tdist;
   $46 = Math_imul($44, $45)|0;
   $min_dist = $46;
   $47 = $x;
   $48 = $maxc0;
   $49 = (($47) - ($48))|0;
   $50 = $49<<1;
   $tdist = $50;
   $51 = $tdist;
   $52 = $tdist;
   $53 = Math_imul($51, $52)|0;
   $max_dist = $53;
  } else {
   $54 = $x;
   $55 = $maxc0;
   $56 = ($54|0)>($55|0);
   if ($56) {
    $57 = $x;
    $58 = $maxc0;
    $59 = (($57) - ($58))|0;
    $60 = $59<<1;
    $tdist = $60;
    $61 = $tdist;
    $62 = $tdist;
    $63 = Math_imul($61, $62)|0;
    $min_dist = $63;
    $64 = $x;
    $65 = $1;
    $66 = (($64) - ($65))|0;
    $67 = $66<<1;
    $tdist = $67;
    $68 = $tdist;
    $69 = $tdist;
    $70 = Math_imul($68, $69)|0;
    $max_dist = $70;
   } else {
    $min_dist = 0;
    $71 = $x;
    $72 = $centerc0;
    $73 = ($71|0)<=($72|0);
    if ($73) {
     $74 = $x;
     $75 = $maxc0;
     $76 = (($74) - ($75))|0;
     $77 = $76<<1;
     $tdist = $77;
     $78 = $tdist;
     $79 = $tdist;
     $80 = Math_imul($78, $79)|0;
     $max_dist = $80;
    } else {
     $81 = $x;
     $82 = $1;
     $83 = (($81) - ($82))|0;
     $84 = $83<<1;
     $tdist = $84;
     $85 = $tdist;
     $86 = $tdist;
     $87 = Math_imul($85, $86)|0;
     $max_dist = $87;
    }
   }
  }
  $88 = $i;
  $89 = $0;
  $90 = (($89) + 136|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + ($88)|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $x = $96;
  $97 = $x;
  $98 = $2;
  $99 = ($97|0)<($98|0);
  if ($99) {
   $100 = $x;
   $101 = $2;
   $102 = (($100) - ($101))|0;
   $103 = ($102*3)|0;
   $tdist = $103;
   $104 = $tdist;
   $105 = $tdist;
   $106 = Math_imul($104, $105)|0;
   $107 = $min_dist;
   $108 = (($107) + ($106))|0;
   $min_dist = $108;
   $109 = $x;
   $110 = $maxc1;
   $111 = (($109) - ($110))|0;
   $112 = ($111*3)|0;
   $tdist = $112;
   $113 = $tdist;
   $114 = $tdist;
   $115 = Math_imul($113, $114)|0;
   $116 = $max_dist;
   $117 = (($116) + ($115))|0;
   $max_dist = $117;
  } else {
   $118 = $x;
   $119 = $maxc1;
   $120 = ($118|0)>($119|0);
   if ($120) {
    $121 = $x;
    $122 = $maxc1;
    $123 = (($121) - ($122))|0;
    $124 = ($123*3)|0;
    $tdist = $124;
    $125 = $tdist;
    $126 = $tdist;
    $127 = Math_imul($125, $126)|0;
    $128 = $min_dist;
    $129 = (($128) + ($127))|0;
    $min_dist = $129;
    $130 = $x;
    $131 = $2;
    $132 = (($130) - ($131))|0;
    $133 = ($132*3)|0;
    $tdist = $133;
    $134 = $tdist;
    $135 = $tdist;
    $136 = Math_imul($134, $135)|0;
    $137 = $max_dist;
    $138 = (($137) + ($136))|0;
    $max_dist = $138;
   } else {
    $139 = $x;
    $140 = $centerc1;
    $141 = ($139|0)<=($140|0);
    if ($141) {
     $142 = $x;
     $143 = $maxc1;
     $144 = (($142) - ($143))|0;
     $145 = ($144*3)|0;
     $tdist = $145;
     $146 = $tdist;
     $147 = $tdist;
     $148 = Math_imul($146, $147)|0;
     $149 = $max_dist;
     $150 = (($149) + ($148))|0;
     $max_dist = $150;
    } else {
     $151 = $x;
     $152 = $2;
     $153 = (($151) - ($152))|0;
     $154 = ($153*3)|0;
     $tdist = $154;
     $155 = $tdist;
     $156 = $tdist;
     $157 = Math_imul($155, $156)|0;
     $158 = $max_dist;
     $159 = (($158) + ($157))|0;
     $max_dist = $159;
    }
   }
  }
  $160 = $i;
  $161 = $0;
  $162 = (($161) + 136|0);
  $163 = HEAP32[$162>>2]|0;
  $164 = (($163) + 8|0);
  $165 = HEAP32[$164>>2]|0;
  $166 = (($165) + ($160)|0);
  $167 = HEAP8[$166>>0]|0;
  $168 = $167&255;
  $x = $168;
  $169 = $x;
  $170 = $3;
  $171 = ($169|0)<($170|0);
  if ($171) {
   $172 = $x;
   $173 = $3;
   $174 = (($172) - ($173))|0;
   $175 = $174;
   $tdist = $175;
   $176 = $tdist;
   $177 = $tdist;
   $178 = Math_imul($176, $177)|0;
   $179 = $min_dist;
   $180 = (($179) + ($178))|0;
   $min_dist = $180;
   $181 = $x;
   $182 = $maxc2;
   $183 = (($181) - ($182))|0;
   $184 = $183;
   $tdist = $184;
   $185 = $tdist;
   $186 = $tdist;
   $187 = Math_imul($185, $186)|0;
   $188 = $max_dist;
   $189 = (($188) + ($187))|0;
   $max_dist = $189;
  } else {
   $190 = $x;
   $191 = $maxc2;
   $192 = ($190|0)>($191|0);
   if ($192) {
    $193 = $x;
    $194 = $maxc2;
    $195 = (($193) - ($194))|0;
    $196 = $195;
    $tdist = $196;
    $197 = $tdist;
    $198 = $tdist;
    $199 = Math_imul($197, $198)|0;
    $200 = $min_dist;
    $201 = (($200) + ($199))|0;
    $min_dist = $201;
    $202 = $x;
    $203 = $3;
    $204 = (($202) - ($203))|0;
    $205 = $204;
    $tdist = $205;
    $206 = $tdist;
    $207 = $tdist;
    $208 = Math_imul($206, $207)|0;
    $209 = $max_dist;
    $210 = (($209) + ($208))|0;
    $max_dist = $210;
   } else {
    $211 = $x;
    $212 = $centerc2;
    $213 = ($211|0)<=($212|0);
    if ($213) {
     $214 = $x;
     $215 = $maxc2;
     $216 = (($214) - ($215))|0;
     $217 = $216;
     $tdist = $217;
     $218 = $tdist;
     $219 = $tdist;
     $220 = Math_imul($218, $219)|0;
     $221 = $max_dist;
     $222 = (($221) + ($220))|0;
     $max_dist = $222;
    } else {
     $223 = $x;
     $224 = $3;
     $225 = (($223) - ($224))|0;
     $226 = $225;
     $tdist = $226;
     $227 = $tdist;
     $228 = $tdist;
     $229 = Math_imul($227, $228)|0;
     $230 = $max_dist;
     $231 = (($230) + ($229))|0;
     $max_dist = $231;
    }
   }
  }
  $232 = $min_dist;
  $233 = $i;
  $234 = (($mindist) + ($233<<2)|0);
  HEAP32[$234>>2] = $232;
  $235 = $max_dist;
  $236 = $minmaxdist;
  $237 = ($235|0)<($236|0);
  if ($237) {
   $238 = $max_dist;
   $minmaxdist = $238;
  }
  $239 = $i;
  $240 = (($239) + 1)|0;
  $i = $240;
 }
 $ncolors = 0;
 $i = 0;
 while(1) {
  $241 = $i;
  $242 = $numcolors;
  $243 = ($241|0)<($242|0);
  if (!($243)) {
   break;
  }
  $244 = $i;
  $245 = (($mindist) + ($244<<2)|0);
  $246 = HEAP32[$245>>2]|0;
  $247 = $minmaxdist;
  $248 = ($246|0)<=($247|0);
  if ($248) {
   $249 = $i;
   $250 = $249&255;
   $251 = $ncolors;
   $252 = (($251) + 1)|0;
   $ncolors = $252;
   $253 = $4;
   $254 = (($253) + ($251)|0);
   HEAP8[$254>>0] = $250;
  }
  $255 = $i;
  $256 = (($255) + 1)|0;
  $i = $256;
 }
 $257 = $ncolors;
 STACKTOP = sp;return ($257|0);
}
function _find_best_colors($cinfo,$minc0,$minc1,$minc2,$numcolors,$colorlist,$bestcolor) {
 $cinfo = $cinfo|0;
 $minc0 = $minc0|0;
 $minc1 = $minc1|0;
 $minc2 = $minc2|0;
 $numcolors = $numcolors|0;
 $colorlist = $colorlist|0;
 $bestcolor = $bestcolor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bestdist = 0, $bptr = 0, $cptr = 0, $dist0 = 0, $dist1 = 0, $dist2 = 0, $i = 0, $ic0 = 0, $ic1 = 0, $ic2 = 0, $icolor = 0, $inc0 = 0, $inc1 = 0, $inc2 = 0;
 var $xx0 = 0, $xx1 = 0, $xx2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 608|0;
 $bestdist = sp;
 $0 = $cinfo;
 $1 = $minc0;
 $2 = $minc1;
 $3 = $minc2;
 $4 = $numcolors;
 $5 = $colorlist;
 $6 = $bestcolor;
 $bptr = $bestdist;
 $i = 127;
 while(1) {
  $7 = $i;
  $8 = ($7|0)>=(0);
  if (!($8)) {
   break;
  }
  $9 = $bptr;
  $10 = (($9) + 4|0);
  $bptr = $10;
  HEAP32[$9>>2] = 2147483647;
  $11 = $i;
  $12 = (($11) + -1)|0;
  $i = $12;
 }
 $i = 0;
 while(1) {
  $13 = $i;
  $14 = $4;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $i;
  $17 = $5;
  $18 = (($17) + ($16)|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $icolor = $20;
  $21 = $1;
  $22 = $icolor;
  $23 = $0;
  $24 = (($23) + 136|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($22)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = (($21) - ($29))|0;
  $31 = $30<<1;
  $inc0 = $31;
  $32 = $inc0;
  $33 = $inc0;
  $34 = Math_imul($32, $33)|0;
  $dist0 = $34;
  $35 = $2;
  $36 = $icolor;
  $37 = $0;
  $38 = (($37) + 136|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($36)|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = (($35) - ($44))|0;
  $46 = ($45*3)|0;
  $inc1 = $46;
  $47 = $inc1;
  $48 = $inc1;
  $49 = Math_imul($47, $48)|0;
  $50 = $dist0;
  $51 = (($50) + ($49))|0;
  $dist0 = $51;
  $52 = $3;
  $53 = $icolor;
  $54 = $0;
  $55 = (($54) + 136|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($53)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = (($52) - ($61))|0;
  $63 = $62;
  $inc2 = $63;
  $64 = $inc2;
  $65 = $inc2;
  $66 = Math_imul($64, $65)|0;
  $67 = $dist0;
  $68 = (($67) + ($66))|0;
  $dist0 = $68;
  $69 = $inc0;
  $70 = $69<<5;
  $71 = (($70) + 256)|0;
  $inc0 = $71;
  $72 = $inc1;
  $73 = ($72*24)|0;
  $74 = (($73) + 144)|0;
  $inc1 = $74;
  $75 = $inc2;
  $76 = $75<<4;
  $77 = (($76) + 64)|0;
  $inc2 = $77;
  $bptr = $bestdist;
  $78 = $6;
  $cptr = $78;
  $79 = $inc0;
  $xx0 = $79;
  $ic0 = 3;
  while(1) {
   $80 = $ic0;
   $81 = ($80|0)>=(0);
   if (!($81)) {
    break;
   }
   $82 = $dist0;
   $dist1 = $82;
   $83 = $inc1;
   $xx1 = $83;
   $ic1 = 7;
   while(1) {
    $84 = $ic1;
    $85 = ($84|0)>=(0);
    if (!($85)) {
     break;
    }
    $86 = $dist1;
    $dist2 = $86;
    $87 = $inc2;
    $xx2 = $87;
    $ic2 = 3;
    while(1) {
     $88 = $ic2;
     $89 = ($88|0)>=(0);
     if (!($89)) {
      break;
     }
     $90 = $dist2;
     $91 = $bptr;
     $92 = HEAP32[$91>>2]|0;
     $93 = ($90|0)<($92|0);
     if ($93) {
      $94 = $dist2;
      $95 = $bptr;
      HEAP32[$95>>2] = $94;
      $96 = $icolor;
      $97 = $96&255;
      $98 = $cptr;
      HEAP8[$98>>0] = $97;
     }
     $99 = $xx2;
     $100 = $dist2;
     $101 = (($100) + ($99))|0;
     $dist2 = $101;
     $102 = $xx2;
     $103 = (($102) + 128)|0;
     $xx2 = $103;
     $104 = $bptr;
     $105 = (($104) + 4|0);
     $bptr = $105;
     $106 = $cptr;
     $107 = (($106) + 1|0);
     $cptr = $107;
     $108 = $ic2;
     $109 = (($108) + -1)|0;
     $ic2 = $109;
    }
    $110 = $xx1;
    $111 = $dist1;
    $112 = (($111) + ($110))|0;
    $dist1 = $112;
    $113 = $xx1;
    $114 = (($113) + 288)|0;
    $xx1 = $114;
    $115 = $ic1;
    $116 = (($115) + -1)|0;
    $ic1 = $116;
   }
   $117 = $xx0;
   $118 = $dist0;
   $119 = (($118) + ($117))|0;
   $dist0 = $119;
   $120 = $xx0;
   $121 = (($120) + 512)|0;
   $xx0 = $121;
   $122 = $ic0;
   $123 = (($122) + -1)|0;
   $ic0 = $123;
  }
  $124 = $i;
  $125 = (($124) + 1)|0;
  $i = $125;
 }
 STACKTOP = sp;return;
}
function _select_colors($cinfo,$desired_colors) {
 $cinfo = $cinfo|0;
 $desired_colors = $desired_colors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $boxlist = 0, $i = 0, $numboxes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $desired_colors;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = $1;
 $8 = $7<<5;
 $9 = (FUNCTION_TABLE_iiii[$5 & 127]($6,1,$8)|0);
 $boxlist = $9;
 $numboxes = 1;
 $10 = $boxlist;
 HEAP32[$10>>2] = 0;
 $11 = $boxlist;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 31;
 $13 = $boxlist;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 $15 = $boxlist;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = 63;
 $17 = $boxlist;
 $18 = (($17) + 16|0);
 HEAP32[$18>>2] = 0;
 $19 = $boxlist;
 $20 = (($19) + 20|0);
 HEAP32[$20>>2] = 31;
 $21 = $0;
 $22 = $boxlist;
 _update_box($21,$22);
 $23 = $0;
 $24 = $boxlist;
 $25 = $numboxes;
 $26 = $1;
 $27 = (_median_cut($23,$24,$25,$26)|0);
 $numboxes = $27;
 $i = 0;
 while(1) {
  $28 = $i;
  $29 = $numboxes;
  $30 = ($28|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = $0;
  $32 = $i;
  $33 = $boxlist;
  $34 = (($33) + ($32<<5)|0);
  $35 = $i;
  _compute_color($31,$34,$35);
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 $38 = $numboxes;
 $39 = $0;
 $40 = (($39) + 132|0);
 HEAP32[$40>>2] = $38;
 $41 = $0;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + 20|0);
 HEAP32[$43>>2] = 96;
 $44 = $numboxes;
 $45 = $0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 24|0);
 HEAP32[$47>>2] = $44;
 $48 = $0;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = $0;
 FUNCTION_TABLE_vii[$51 & 127]($52,1);
 STACKTOP = sp;return;
}
function _update_box($cinfo,$boxp) {
 $cinfo = $cinfo|0;
 $boxp = $boxp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c0 = 0, $c0max = 0, $c0min = 0, $c1 = 0, $c1max = 0, $c1min = 0, $c2 = 0;
 var $c2max = 0, $c2min = 0, $ccount = 0, $cquantize = 0, $dist0 = 0, $dist1 = 0, $dist2 = 0, $histogram = 0, $histp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $boxp;
 $2 = $0;
 $3 = (($2) + 460|0);
 $4 = HEAP32[$3>>2]|0;
 $cquantize = $4;
 $5 = $cquantize;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $histogram = $7;
 $8 = $1;
 $9 = HEAP32[$8>>2]|0;
 $c0min = $9;
 $10 = $1;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $c0max = $12;
 $13 = $1;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $c1min = $15;
 $16 = $1;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $c1max = $18;
 $19 = $1;
 $20 = (($19) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $c2min = $21;
 $22 = $1;
 $23 = (($22) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $c2max = $24;
 $25 = $c0max;
 $26 = $c0min;
 $27 = ($25|0)>($26|0);
 do {
  if ($27) {
   $28 = $c0min;
   $c0 = $28;
   L3: while(1) {
    $29 = $c0;
    $30 = $c0max;
    $31 = ($29|0)<=($30|0);
    if (!($31)) {
     label = 16;
     break;
    }
    $32 = $c1min;
    $c1 = $32;
    while(1) {
     $33 = $c1;
     $34 = $c1max;
     $35 = ($33|0)<=($34|0);
     if (!($35)) {
      break;
     }
     $36 = $c2min;
     $37 = $c1;
     $38 = $c0;
     $39 = $histogram;
     $40 = (($39) + ($38<<2)|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($37<<6)|0);
     $43 = (($42) + ($36<<1)|0);
     $histp = $43;
     $44 = $c2min;
     $c2 = $44;
     while(1) {
      $45 = $c2;
      $46 = $c2max;
      $47 = ($45|0)<=($46|0);
      if (!($47)) {
       break;
      }
      $48 = $histp;
      $49 = (($48) + 2|0);
      $histp = $49;
      $50 = HEAP16[$48>>1]|0;
      $51 = $50&65535;
      $52 = ($51|0)!=(0);
      if ($52) {
       label = 9;
       break L3;
      }
      $55 = $c2;
      $56 = (($55) + 1)|0;
      $c2 = $56;
     }
     $57 = $c1;
     $58 = (($57) + 1)|0;
     $c1 = $58;
    }
    $59 = $c0;
    $60 = (($59) + 1)|0;
    $c0 = $60;
   }
   if ((label|0) == 9) {
    $53 = $c0;
    $c0min = $53;
    $54 = $1;
    HEAP32[$54>>2] = $53;
    break;
   }
   else if ((label|0) == 16) {
    label = 17;
    break;
   }
  } else {
   label = 17;
  }
 } while(0);
 if ((label|0) == 17) {
 }
 $61 = $c0max;
 $62 = $c0min;
 $63 = ($61|0)>($62|0);
 do {
  if ($63) {
   $64 = $c0max;
   $c0 = $64;
   L26: while(1) {
    $65 = $c0;
    $66 = $c0min;
    $67 = ($65|0)>=($66|0);
    if (!($67)) {
     label = 33;
     break;
    }
    $68 = $c1min;
    $c1 = $68;
    while(1) {
     $69 = $c1;
     $70 = $c1max;
     $71 = ($69|0)<=($70|0);
     if (!($71)) {
      break;
     }
     $72 = $c2min;
     $73 = $c1;
     $74 = $c0;
     $75 = $histogram;
     $76 = (($75) + ($74<<2)|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = (($77) + ($73<<6)|0);
     $79 = (($78) + ($72<<1)|0);
     $histp = $79;
     $80 = $c2min;
     $c2 = $80;
     while(1) {
      $81 = $c2;
      $82 = $c2max;
      $83 = ($81|0)<=($82|0);
      if (!($83)) {
       break;
      }
      $84 = $histp;
      $85 = (($84) + 2|0);
      $histp = $85;
      $86 = HEAP16[$84>>1]|0;
      $87 = $86&65535;
      $88 = ($87|0)!=(0);
      if ($88) {
       label = 26;
       break L26;
      }
      $92 = $c2;
      $93 = (($92) + 1)|0;
      $c2 = $93;
     }
     $94 = $c1;
     $95 = (($94) + 1)|0;
     $c1 = $95;
    }
    $96 = $c0;
    $97 = (($96) + -1)|0;
    $c0 = $97;
   }
   if ((label|0) == 26) {
    $89 = $c0;
    $c0max = $89;
    $90 = $1;
    $91 = (($90) + 4|0);
    HEAP32[$91>>2] = $89;
    break;
   }
   else if ((label|0) == 33) {
    label = 34;
    break;
   }
  } else {
   label = 34;
  }
 } while(0);
 if ((label|0) == 34) {
 }
 $98 = $c1max;
 $99 = $c1min;
 $100 = ($98|0)>($99|0);
 do {
  if ($100) {
   $101 = $c1min;
   $c1 = $101;
   L49: while(1) {
    $102 = $c1;
    $103 = $c1max;
    $104 = ($102|0)<=($103|0);
    if (!($104)) {
     label = 50;
     break;
    }
    $105 = $c0min;
    $c0 = $105;
    while(1) {
     $106 = $c0;
     $107 = $c0max;
     $108 = ($106|0)<=($107|0);
     if (!($108)) {
      break;
     }
     $109 = $c2min;
     $110 = $c1;
     $111 = $c0;
     $112 = $histogram;
     $113 = (($112) + ($111<<2)|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + ($110<<6)|0);
     $116 = (($115) + ($109<<1)|0);
     $histp = $116;
     $117 = $c2min;
     $c2 = $117;
     while(1) {
      $118 = $c2;
      $119 = $c2max;
      $120 = ($118|0)<=($119|0);
      if (!($120)) {
       break;
      }
      $121 = $histp;
      $122 = (($121) + 2|0);
      $histp = $122;
      $123 = HEAP16[$121>>1]|0;
      $124 = $123&65535;
      $125 = ($124|0)!=(0);
      if ($125) {
       label = 43;
       break L49;
      }
      $129 = $c2;
      $130 = (($129) + 1)|0;
      $c2 = $130;
     }
     $131 = $c0;
     $132 = (($131) + 1)|0;
     $c0 = $132;
    }
    $133 = $c1;
    $134 = (($133) + 1)|0;
    $c1 = $134;
   }
   if ((label|0) == 43) {
    $126 = $c1;
    $c1min = $126;
    $127 = $1;
    $128 = (($127) + 8|0);
    HEAP32[$128>>2] = $126;
    break;
   }
   else if ((label|0) == 50) {
    label = 51;
    break;
   }
  } else {
   label = 51;
  }
 } while(0);
 if ((label|0) == 51) {
 }
 $135 = $c1max;
 $136 = $c1min;
 $137 = ($135|0)>($136|0);
 do {
  if ($137) {
   $138 = $c1max;
   $c1 = $138;
   L72: while(1) {
    $139 = $c1;
    $140 = $c1min;
    $141 = ($139|0)>=($140|0);
    if (!($141)) {
     label = 67;
     break;
    }
    $142 = $c0min;
    $c0 = $142;
    while(1) {
     $143 = $c0;
     $144 = $c0max;
     $145 = ($143|0)<=($144|0);
     if (!($145)) {
      break;
     }
     $146 = $c2min;
     $147 = $c1;
     $148 = $c0;
     $149 = $histogram;
     $150 = (($149) + ($148<<2)|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = (($151) + ($147<<6)|0);
     $153 = (($152) + ($146<<1)|0);
     $histp = $153;
     $154 = $c2min;
     $c2 = $154;
     while(1) {
      $155 = $c2;
      $156 = $c2max;
      $157 = ($155|0)<=($156|0);
      if (!($157)) {
       break;
      }
      $158 = $histp;
      $159 = (($158) + 2|0);
      $histp = $159;
      $160 = HEAP16[$158>>1]|0;
      $161 = $160&65535;
      $162 = ($161|0)!=(0);
      if ($162) {
       label = 60;
       break L72;
      }
      $166 = $c2;
      $167 = (($166) + 1)|0;
      $c2 = $167;
     }
     $168 = $c0;
     $169 = (($168) + 1)|0;
     $c0 = $169;
    }
    $170 = $c1;
    $171 = (($170) + -1)|0;
    $c1 = $171;
   }
   if ((label|0) == 60) {
    $163 = $c1;
    $c1max = $163;
    $164 = $1;
    $165 = (($164) + 12|0);
    HEAP32[$165>>2] = $163;
    break;
   }
   else if ((label|0) == 67) {
    label = 68;
    break;
   }
  } else {
   label = 68;
  }
 } while(0);
 if ((label|0) == 68) {
 }
 $172 = $c2max;
 $173 = $c2min;
 $174 = ($172|0)>($173|0);
 do {
  if ($174) {
   $175 = $c2min;
   $c2 = $175;
   L95: while(1) {
    $176 = $c2;
    $177 = $c2max;
    $178 = ($176|0)<=($177|0);
    if (!($178)) {
     label = 84;
     break;
    }
    $179 = $c0min;
    $c0 = $179;
    while(1) {
     $180 = $c0;
     $181 = $c0max;
     $182 = ($180|0)<=($181|0);
     if (!($182)) {
      break;
     }
     $183 = $c2;
     $184 = $c1min;
     $185 = $c0;
     $186 = $histogram;
     $187 = (($186) + ($185<<2)|0);
     $188 = HEAP32[$187>>2]|0;
     $189 = (($188) + ($184<<6)|0);
     $190 = (($189) + ($183<<1)|0);
     $histp = $190;
     $191 = $c1min;
     $c1 = $191;
     while(1) {
      $192 = $c1;
      $193 = $c1max;
      $194 = ($192|0)<=($193|0);
      if (!($194)) {
       break;
      }
      $195 = $histp;
      $196 = HEAP16[$195>>1]|0;
      $197 = $196&65535;
      $198 = ($197|0)!=(0);
      if ($198) {
       label = 77;
       break L95;
      }
      $202 = $c1;
      $203 = (($202) + 1)|0;
      $c1 = $203;
      $204 = $histp;
      $205 = (($204) + 64|0);
      $histp = $205;
     }
     $206 = $c0;
     $207 = (($206) + 1)|0;
     $c0 = $207;
    }
    $208 = $c2;
    $209 = (($208) + 1)|0;
    $c2 = $209;
   }
   if ((label|0) == 77) {
    $199 = $c2;
    $c2min = $199;
    $200 = $1;
    $201 = (($200) + 16|0);
    HEAP32[$201>>2] = $199;
    break;
   }
   else if ((label|0) == 84) {
    label = 85;
    break;
   }
  } else {
   label = 85;
  }
 } while(0);
 if ((label|0) == 85) {
 }
 $210 = $c2max;
 $211 = $c2min;
 $212 = ($210|0)>($211|0);
 do {
  if ($212) {
   $213 = $c2max;
   $c2 = $213;
   L118: while(1) {
    $214 = $c2;
    $215 = $c2min;
    $216 = ($214|0)>=($215|0);
    if (!($216)) {
     label = 101;
     break;
    }
    $217 = $c0min;
    $c0 = $217;
    while(1) {
     $218 = $c0;
     $219 = $c0max;
     $220 = ($218|0)<=($219|0);
     if (!($220)) {
      break;
     }
     $221 = $c2;
     $222 = $c1min;
     $223 = $c0;
     $224 = $histogram;
     $225 = (($224) + ($223<<2)|0);
     $226 = HEAP32[$225>>2]|0;
     $227 = (($226) + ($222<<6)|0);
     $228 = (($227) + ($221<<1)|0);
     $histp = $228;
     $229 = $c1min;
     $c1 = $229;
     while(1) {
      $230 = $c1;
      $231 = $c1max;
      $232 = ($230|0)<=($231|0);
      if (!($232)) {
       break;
      }
      $233 = $histp;
      $234 = HEAP16[$233>>1]|0;
      $235 = $234&65535;
      $236 = ($235|0)!=(0);
      if ($236) {
       label = 94;
       break L118;
      }
      $240 = $c1;
      $241 = (($240) + 1)|0;
      $c1 = $241;
      $242 = $histp;
      $243 = (($242) + 64|0);
      $histp = $243;
     }
     $244 = $c0;
     $245 = (($244) + 1)|0;
     $c0 = $245;
    }
    $246 = $c2;
    $247 = (($246) + -1)|0;
    $c2 = $247;
   }
   if ((label|0) == 94) {
    $237 = $c2;
    $c2max = $237;
    $238 = $1;
    $239 = (($238) + 20|0);
    HEAP32[$239>>2] = $237;
    break;
   }
   else if ((label|0) == 101) {
    label = 102;
    break;
   }
  } else {
   label = 102;
  }
 } while(0);
 if ((label|0) == 102) {
 }
 $248 = $c0max;
 $249 = $c0min;
 $250 = (($248) - ($249))|0;
 $251 = $250 << 3;
 $252 = $251<<1;
 $dist0 = $252;
 $253 = $c1max;
 $254 = $c1min;
 $255 = (($253) - ($254))|0;
 $256 = $255 << 2;
 $257 = ($256*3)|0;
 $dist1 = $257;
 $258 = $c2max;
 $259 = $c2min;
 $260 = (($258) - ($259))|0;
 $261 = $260 << 3;
 $262 = $261;
 $dist2 = $262;
 $263 = $dist0;
 $264 = $dist0;
 $265 = Math_imul($263, $264)|0;
 $266 = $dist1;
 $267 = $dist1;
 $268 = Math_imul($266, $267)|0;
 $269 = (($265) + ($268))|0;
 $270 = $dist2;
 $271 = $dist2;
 $272 = Math_imul($270, $271)|0;
 $273 = (($269) + ($272))|0;
 $274 = $1;
 $275 = (($274) + 24|0);
 HEAP32[$275>>2] = $273;
 $ccount = 0;
 $276 = $c0min;
 $c0 = $276;
 while(1) {
  $277 = $c0;
  $278 = $c0max;
  $279 = ($277|0)<=($278|0);
  if (!($279)) {
   break;
  }
  $280 = $c1min;
  $c1 = $280;
  while(1) {
   $281 = $c1;
   $282 = $c1max;
   $283 = ($281|0)<=($282|0);
   if (!($283)) {
    break;
   }
   $284 = $c2min;
   $285 = $c1;
   $286 = $c0;
   $287 = $histogram;
   $288 = (($287) + ($286<<2)|0);
   $289 = HEAP32[$288>>2]|0;
   $290 = (($289) + ($285<<6)|0);
   $291 = (($290) + ($284<<1)|0);
   $histp = $291;
   $292 = $c2min;
   $c2 = $292;
   while(1) {
    $293 = $c2;
    $294 = $c2max;
    $295 = ($293|0)<=($294|0);
    if (!($295)) {
     break;
    }
    $296 = $histp;
    $297 = HEAP16[$296>>1]|0;
    $298 = $297&65535;
    $299 = ($298|0)!=(0);
    if ($299) {
     $300 = $ccount;
     $301 = (($300) + 1)|0;
     $ccount = $301;
    }
    $302 = $c2;
    $303 = (($302) + 1)|0;
    $c2 = $303;
    $304 = $histp;
    $305 = (($304) + 2|0);
    $histp = $305;
   }
   $306 = $c1;
   $307 = (($306) + 1)|0;
   $c1 = $307;
  }
  $308 = $c0;
  $309 = (($308) + 1)|0;
  $c0 = $309;
 }
 $310 = $ccount;
 $311 = $1;
 $312 = (($311) + 28|0);
 HEAP32[$312>>2] = $310;
 STACKTOP = sp;return;
}
function _median_cut($cinfo,$boxlist,$numboxes,$desired_colors) {
 $cinfo = $cinfo|0;
 $boxlist = $boxlist|0;
 $numboxes = $numboxes|0;
 $desired_colors = $desired_colors|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $b1 = 0, $b2 = 0, $c0 = 0, $c1 = 0, $c2 = 0;
 var $cmax = 0, $lb = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $boxlist;
 $2 = $numboxes;
 $3 = $desired_colors;
 while(1) {
  $4 = $2;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   label = 17;
   break;
  }
  $7 = $2;
  $8 = $7<<1;
  $9 = $3;
  $10 = ($8|0)<=($9|0);
  if ($10) {
   $11 = $1;
   $12 = $2;
   $13 = (_find_biggest_color_pop($11,$12)|0);
   $b1 = $13;
  } else {
   $14 = $1;
   $15 = $2;
   $16 = (_find_biggest_volume($14,$15)|0);
   $b1 = $16;
  }
  $17 = $b1;
  $18 = ($17|0)==(0|0);
  if ($18) {
   break;
  }
  $19 = $2;
  $20 = $1;
  $21 = (($20) + ($19<<5)|0);
  $b2 = $21;
  $22 = $b1;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $b2;
  $26 = (($25) + 4|0);
  HEAP32[$26>>2] = $24;
  $27 = $b1;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $b2;
  $31 = (($30) + 12|0);
  HEAP32[$31>>2] = $29;
  $32 = $b1;
  $33 = (($32) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $b2;
  $36 = (($35) + 20|0);
  HEAP32[$36>>2] = $34;
  $37 = $b1;
  $38 = HEAP32[$37>>2]|0;
  $39 = $b2;
  HEAP32[$39>>2] = $38;
  $40 = $b1;
  $41 = (($40) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $b2;
  $44 = (($43) + 8|0);
  HEAP32[$44>>2] = $42;
  $45 = $b1;
  $46 = (($45) + 16|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $b2;
  $49 = (($48) + 16|0);
  HEAP32[$49>>2] = $47;
  $50 = $b1;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $b1;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) - ($54))|0;
  $56 = $55 << 3;
  $57 = $56<<1;
  $c0 = $57;
  $58 = $b1;
  $59 = (($58) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = $b1;
  $62 = (($61) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($60) - ($63))|0;
  $65 = $64 << 2;
  $66 = ($65*3)|0;
  $c1 = $66;
  $67 = $b1;
  $68 = (($67) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = $b1;
  $71 = (($70) + 16|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($69) - ($72))|0;
  $74 = $73 << 3;
  $75 = $74;
  $c2 = $75;
  $76 = $c1;
  $cmax = $76;
  $n = 1;
  $77 = $c0;
  $78 = $cmax;
  $79 = ($77|0)>($78|0);
  if ($79) {
   $80 = $c0;
   $cmax = $80;
   $n = 0;
  }
  $81 = $c2;
  $82 = $cmax;
  $83 = ($81|0)>($82|0);
  if ($83) {
   $n = 2;
  }
  $84 = $n;
  if ((($84|0) == 2)) {
   $113 = $b1;
   $114 = (($113) + 20|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = $b1;
   $117 = (($116) + 16|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (($115) + ($118))|0;
   $120 = (($119|0) / 2)&-1;
   $lb = $120;
   $121 = $lb;
   $122 = $b1;
   $123 = (($122) + 20|0);
   HEAP32[$123>>2] = $121;
   $124 = $lb;
   $125 = (($124) + 1)|0;
   $126 = $b2;
   $127 = (($126) + 16|0);
   HEAP32[$127>>2] = $125;
  } else if ((($84|0) == 1)) {
   $98 = $b1;
   $99 = (($98) + 12|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = $b1;
   $102 = (($101) + 8|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (($100) + ($103))|0;
   $105 = (($104|0) / 2)&-1;
   $lb = $105;
   $106 = $lb;
   $107 = $b1;
   $108 = (($107) + 12|0);
   HEAP32[$108>>2] = $106;
   $109 = $lb;
   $110 = (($109) + 1)|0;
   $111 = $b2;
   $112 = (($111) + 8|0);
   HEAP32[$112>>2] = $110;
  } else if ((($84|0) == 0)) {
   $85 = $b1;
   $86 = (($85) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = $b1;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($87) + ($89))|0;
   $91 = (($90|0) / 2)&-1;
   $lb = $91;
   $92 = $lb;
   $93 = $b1;
   $94 = (($93) + 4|0);
   HEAP32[$94>>2] = $92;
   $95 = $lb;
   $96 = (($95) + 1)|0;
   $97 = $b2;
   HEAP32[$97>>2] = $96;
  }
  $128 = $0;
  $129 = $b1;
  _update_box($128,$129);
  $130 = $0;
  $131 = $b2;
  _update_box($130,$131);
  $132 = $2;
  $133 = (($132) + 1)|0;
  $2 = $133;
 }
 if ((label|0) == 17) {
  $134 = $2;
  STACKTOP = sp;return ($134|0);
 }
 $134 = $2;
 STACKTOP = sp;return ($134|0);
}
function _compute_color($cinfo,$boxp,$icolor) {
 $cinfo = $cinfo|0;
 $boxp = $boxp|0;
 $icolor = $icolor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $c0 = 0, $c0max = 0, $c0min = 0, $c0total = 0, $c1 = 0, $c1max = 0, $c1min = 0, $c1total = 0, $c2 = 0, $c2max = 0, $c2min = 0, $c2total = 0, $count = 0, $cquantize = 0, $histogram = 0, $histp = 0, $total = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $boxp;
 $2 = $icolor;
 $3 = $0;
 $4 = (($3) + 460|0);
 $5 = HEAP32[$4>>2]|0;
 $cquantize = $5;
 $6 = $cquantize;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $histogram = $8;
 $total = 0;
 $c0total = 0;
 $c1total = 0;
 $c2total = 0;
 $9 = $1;
 $10 = HEAP32[$9>>2]|0;
 $c0min = $10;
 $11 = $1;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $c0max = $13;
 $14 = $1;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $c1min = $16;
 $17 = $1;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $c1max = $19;
 $20 = $1;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $c2min = $22;
 $23 = $1;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $c2max = $25;
 $26 = $c0min;
 $c0 = $26;
 while(1) {
  $27 = $c0;
  $28 = $c0max;
  $29 = ($27|0)<=($28|0);
  if (!($29)) {
   break;
  }
  $30 = $c1min;
  $c1 = $30;
  while(1) {
   $31 = $c1;
   $32 = $c1max;
   $33 = ($31|0)<=($32|0);
   if (!($33)) {
    break;
   }
   $34 = $c2min;
   $35 = $c1;
   $36 = $c0;
   $37 = $histogram;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($35<<6)|0);
   $41 = (($40) + ($34<<1)|0);
   $histp = $41;
   $42 = $c2min;
   $c2 = $42;
   while(1) {
    $43 = $c2;
    $44 = $c2max;
    $45 = ($43|0)<=($44|0);
    if (!($45)) {
     break;
    }
    $46 = $histp;
    $47 = (($46) + 2|0);
    $histp = $47;
    $48 = HEAP16[$46>>1]|0;
    $49 = $48&65535;
    $count = $49;
    $50 = ($49|0)!=(0);
    if ($50) {
     $51 = $count;
     $52 = $total;
     $53 = (($52) + ($51))|0;
     $total = $53;
     $54 = $c0;
     $55 = $54 << 3;
     $56 = (($55) + 4)|0;
     $57 = $count;
     $58 = Math_imul($56, $57)|0;
     $59 = $c0total;
     $60 = (($59) + ($58))|0;
     $c0total = $60;
     $61 = $c1;
     $62 = $61 << 2;
     $63 = (($62) + 2)|0;
     $64 = $count;
     $65 = Math_imul($63, $64)|0;
     $66 = $c1total;
     $67 = (($66) + ($65))|0;
     $c1total = $67;
     $68 = $c2;
     $69 = $68 << 3;
     $70 = (($69) + 4)|0;
     $71 = $count;
     $72 = Math_imul($70, $71)|0;
     $73 = $c2total;
     $74 = (($73) + ($72))|0;
     $c2total = $74;
    }
    $75 = $c2;
    $76 = (($75) + 1)|0;
    $c2 = $76;
   }
   $77 = $c1;
   $78 = (($77) + 1)|0;
   $c1 = $78;
  }
  $79 = $c0;
  $80 = (($79) + 1)|0;
  $c0 = $80;
 }
 $81 = $c0total;
 $82 = $total;
 $83 = $82 >> 1;
 $84 = (($81) + ($83))|0;
 $85 = $total;
 $86 = (($84|0) / ($85|0))&-1;
 $87 = $86&255;
 $88 = $2;
 $89 = $0;
 $90 = (($89) + 136|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = HEAP32[$91>>2]|0;
 $93 = (($92) + ($88)|0);
 HEAP8[$93>>0] = $87;
 $94 = $c1total;
 $95 = $total;
 $96 = $95 >> 1;
 $97 = (($94) + ($96))|0;
 $98 = $total;
 $99 = (($97|0) / ($98|0))&-1;
 $100 = $99&255;
 $101 = $2;
 $102 = $0;
 $103 = (($102) + 136|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = (($104) + 4|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = (($106) + ($101)|0);
 HEAP8[$107>>0] = $100;
 $108 = $c2total;
 $109 = $total;
 $110 = $109 >> 1;
 $111 = (($108) + ($110))|0;
 $112 = $total;
 $113 = (($111|0) / ($112|0))&-1;
 $114 = $113&255;
 $115 = $2;
 $116 = $0;
 $117 = (($116) + 136|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = (($118) + 8|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = (($120) + ($115)|0);
 HEAP8[$121>>0] = $114;
 STACKTOP = sp;return;
}
function _find_biggest_color_pop($boxlist,$numboxes) {
 $boxlist = $boxlist|0;
 $numboxes = $numboxes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $boxp = 0, $i = 0, $maxc = 0, $which = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $boxlist;
 $1 = $numboxes;
 $maxc = 0;
 $which = 0;
 $i = 0;
 $2 = $0;
 $boxp = $2;
 while(1) {
  $3 = $i;
  $4 = $1;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $boxp;
  $7 = (($6) + 28|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $maxc;
  $10 = ($8|0)>($9|0);
  if ($10) {
   $11 = $boxp;
   $12 = (($11) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)>(0);
   if ($14) {
    $15 = $boxp;
    $which = $15;
    $16 = $boxp;
    $17 = (($16) + 28|0);
    $18 = HEAP32[$17>>2]|0;
    $maxc = $18;
   }
  }
  $19 = $i;
  $20 = (($19) + 1)|0;
  $i = $20;
  $21 = $boxp;
  $22 = (($21) + 32|0);
  $boxp = $22;
 }
 $23 = $which;
 STACKTOP = sp;return ($23|0);
}
function _find_biggest_volume($boxlist,$numboxes) {
 $boxlist = $boxlist|0;
 $numboxes = $numboxes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $boxp = 0, $i = 0, $maxv = 0, $which = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $boxlist;
 $1 = $numboxes;
 $maxv = 0;
 $which = 0;
 $i = 0;
 $2 = $0;
 $boxp = $2;
 while(1) {
  $3 = $i;
  $4 = $1;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $boxp;
  $7 = (($6) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $maxv;
  $10 = ($8|0)>($9|0);
  if ($10) {
   $11 = $boxp;
   $which = $11;
   $12 = $boxp;
   $13 = (($12) + 24|0);
   $14 = HEAP32[$13>>2]|0;
   $maxv = $14;
  }
  $15 = $i;
  $16 = (($15) + 1)|0;
  $i = $16;
  $17 = $boxp;
  $18 = (($17) + 32|0);
  $boxp = $18;
 }
 $19 = $which;
 STACKTOP = sp;return ($19|0);
}
function _jdiv_round_up($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = (($2) + ($3))|0;
 $5 = (($4) - 1)|0;
 $6 = $1;
 $7 = (($5|0) / ($6|0))&-1;
 STACKTOP = sp;return ($7|0);
}
function _jround_up($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $1;
 $3 = (($2) - 1)|0;
 $4 = $0;
 $5 = (($4) + ($3))|0;
 $0 = $5;
 $6 = $0;
 $7 = $0;
 $8 = $1;
 $9 = (($7|0) % ($8|0))&-1;
 $10 = (($6) - ($9))|0;
 STACKTOP = sp;return ($10|0);
}
function _jcopy_sample_rows($input_array,$source_row,$output_array,$dest_row,$num_rows,$num_cols) {
 $input_array = $input_array|0;
 $source_row = $source_row|0;
 $output_array = $output_array|0;
 $dest_row = $dest_row|0;
 $num_rows = $num_rows|0;
 $num_cols = $num_cols|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $count = 0, $inptr = 0, $outptr = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $input_array;
 $1 = $source_row;
 $2 = $output_array;
 $3 = $dest_row;
 $4 = $num_rows;
 $5 = $num_cols;
 $6 = $5;
 $7 = $6;
 $count = $7;
 $8 = $1;
 $9 = $0;
 $10 = (($9) + ($8<<2)|0);
 $0 = $10;
 $11 = $3;
 $12 = $2;
 $13 = (($12) + ($11<<2)|0);
 $2 = $13;
 $14 = $4;
 $row = $14;
 while(1) {
  $15 = $row;
  $16 = ($15|0)>(0);
  if (!($16)) {
   break;
  }
  $17 = $0;
  $18 = (($17) + 4|0);
  $0 = $18;
  $19 = HEAP32[$17>>2]|0;
  $inptr = $19;
  $20 = $2;
  $21 = (($20) + 4|0);
  $2 = $21;
  $22 = HEAP32[$20>>2]|0;
  $outptr = $22;
  $23 = $outptr;
  $24 = $inptr;
  $25 = $count;
  _memcpy(($23|0),($24|0),($25|0))|0;
  $26 = $row;
  $27 = (($26) + -1)|0;
  $row = $27;
 }
 STACKTOP = sp;return;
}
function _jcopy_block_row($input_row,$output_row,$num_blocks) {
 $input_row = $input_row|0;
 $output_row = $output_row|0;
 $num_blocks = $num_blocks|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $input_row;
 $1 = $output_row;
 $2 = $num_blocks;
 $3 = $1;
 $4 = $0;
 $5 = $2;
 $6 = $5<<7;
 _memcpy(($3|0),($4|0),($6|0))|0;
 STACKTOP = sp;return;
}
function _jzero_far($target,$bytestozero) {
 $target = $target|0;
 $bytestozero = $bytestozero|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $target;
 $1 = $bytestozero;
 $2 = $0;
 $3 = $1;
 _memset(($2|0),0,($3|0))|0;
 STACKTOP = sp;return;
}
function _jpeg_abort($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pool = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $pool = 1;
 while(1) {
  $5 = $pool;
  $6 = ($5|0)>(0);
  if (!($6)) {
   break;
  }
  $7 = $0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 36|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $0;
  $13 = $pool;
  FUNCTION_TABLE_vii[$11 & 127]($12,$13);
  $14 = $pool;
  $15 = (($14) + -1)|0;
  $pool = $15;
 }
 $16 = $0;
 $17 = (($16) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = $0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 200;
  $22 = $0;
  $23 = (($22) + 304|0);
  HEAP32[$23>>2] = 0;
  STACKTOP = sp;return;
 } else {
  $24 = $0;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 100;
  STACKTOP = sp;return;
 }
}
function _jpeg_destroy($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) + 40|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $0;
  FUNCTION_TABLE_vi[$9 & 127]($10);
 }
 $11 = $0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = $0;
 $14 = (($13) + 20|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _jpeg_alloc_quant_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,0,132)|0);
 $tbl = $6;
 $7 = $tbl;
 $8 = (($7) + 128|0);
 HEAP32[$8>>2] = 0;
 $9 = $tbl;
 STACKTOP = sp;return ($9|0);
}
function _jpeg_alloc_huff_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,0,280)|0);
 $tbl = $6;
 $7 = $tbl;
 $8 = (($7) + 276|0);
 HEAP32[$8>>2] = 0;
 $9 = $tbl;
 STACKTOP = sp;return ($9|0);
}
function _jinit_d_coef_controller($cinfo,$need_full_buffer) {
 $cinfo = $cinfo|0;
 $need_full_buffer = $need_full_buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $access_rows = 0, $buffer = 0, $ci = 0;
 var $coef = 0, $compptr = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $need_full_buffer;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 127]($6,1,116)|0);
 $coef = $7;
 $8 = $coef;
 $9 = $0;
 $10 = (($9) + 428|0);
 HEAP32[$10>>2] = $8;
 $11 = $coef;
 HEAP32[$11>>2] = 91;
 $12 = $coef;
 $13 = (($12) + 8|0);
 HEAP32[$13>>2] = 92;
 $14 = $coef;
 $15 = (($14) + 112|0);
 HEAP32[$15>>2] = 0;
 $16 = $1;
 $17 = ($16|0)!=(0);
 if (!($17)) {
  $73 = $0;
  $74 = (($73) + 4|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 4|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = $0;
  $79 = (FUNCTION_TABLE_iiii[$77 & 127]($78,1,1280)|0);
  $buffer = $79;
  $i = 0;
  while(1) {
   $80 = $i;
   $81 = ($80|0)<(10);
   if (!($81)) {
    break;
   }
   $82 = $buffer;
   $83 = $i;
   $84 = (($82) + ($83<<7)|0);
   $85 = $i;
   $86 = $coef;
   $87 = (($86) + 32|0);
   $88 = (($87) + ($85<<2)|0);
   HEAP32[$88>>2] = $84;
   $89 = $i;
   $90 = (($89) + 1)|0;
   $i = $90;
  }
  $91 = $coef;
  $92 = (($91) + 4|0);
  HEAP32[$92>>2] = 95;
  $93 = $coef;
  $94 = (($93) + 12|0);
  HEAP32[$94>>2] = 96;
  $95 = $coef;
  $96 = (($95) + 16|0);
  HEAP32[$96>>2] = 0;
  STACKTOP = sp;return;
 }
 $ci = 0;
 $18 = $0;
 $19 = (($18) + 216|0);
 $20 = HEAP32[$19>>2]|0;
 $compptr = $20;
 while(1) {
  $21 = $ci;
  $22 = $0;
  $23 = (($22) + 36|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($21|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = $compptr;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $access_rows = $28;
  $29 = $0;
  $30 = (($29) + 220|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = $access_rows;
   $34 = ($33*3)|0;
   $access_rows = $34;
  }
  $35 = $0;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + 20|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = $0;
  $41 = $compptr;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $compptr;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (_jround_up($43,$46)|0);
  $48 = $compptr;
  $49 = (($48) + 32|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = $compptr;
  $52 = (($51) + 12|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_jround_up($50,$53)|0);
  $55 = $access_rows;
  $56 = (FUNCTION_TABLE_iiiiiii[$39 & 127]($40,1,1,$47,$54,$55)|0);
  $57 = $ci;
  $58 = $coef;
  $59 = (($58) + 72|0);
  $60 = (($59) + ($57<<2)|0);
  HEAP32[$60>>2] = $56;
  $61 = $ci;
  $62 = (($61) + 1)|0;
  $ci = $62;
  $63 = $compptr;
  $64 = (($63) + 84|0);
  $compptr = $64;
 }
 $65 = $coef;
 $66 = (($65) + 4|0);
 HEAP32[$66>>2] = 93;
 $67 = $coef;
 $68 = (($67) + 12|0);
 HEAP32[$68>>2] = 94;
 $69 = $coef;
 $70 = (($69) + 72|0);
 $71 = $coef;
 $72 = (($71) + 16|0);
 HEAP32[$72>>2] = $70;
 STACKTOP = sp;return;
}
function _start_input_pass332($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 148|0);
 HEAP32[$2>>2] = 0;
 $3 = $0;
 _start_iMCU_row($3);
 STACKTOP = sp;return;
}
function _start_output_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $coef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 428|0);
 $3 = HEAP32[$2>>2]|0;
 $coef = $3;
 $4 = $coef;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $19 = $0;
  $20 = (($19) + 156|0);
  HEAP32[$20>>2] = 0;
  STACKTOP = sp;return;
 }
 $8 = $0;
 $9 = (($8) + 80|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $0;
  $13 = (_smoothing_ok($12)|0);
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = $coef;
   $16 = (($15) + 12|0);
   HEAP32[$16>>2] = 97;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $17 = $coef;
  $18 = (($17) + 12|0);
  HEAP32[$18>>2] = 94;
 }
 $19 = $0;
 $20 = (($19) + 156|0);
 HEAP32[$20>>2] = 0;
 STACKTOP = sp;return;
}
function _consume_data($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $MCU_col_num = 0, $blkn = 0, $buffer = 0, $buffer_ptr = 0, $ci = 0, $coef = 0, $compptr = 0, $start_col = 0, $xindex = 0, $yindex = 0, $yoffset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $buffer = sp + 8|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 428|0);
 $4 = HEAP32[$3>>2]|0;
 $coef = $4;
 $ci = 0;
 while(1) {
  $5 = $ci;
  $6 = $1;
  $7 = (($6) + 328|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $ci;
  $11 = $1;
  $12 = (($11) + 332|0);
  $13 = (($12) + ($10<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $compptr = $14;
  $15 = $1;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = $compptr;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = $coef;
  $25 = (($24) + 72|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $1;
  $29 = (($28) + 148|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $compptr;
  $32 = (($31) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = Math_imul($30, $33)|0;
  $35 = $compptr;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (FUNCTION_TABLE_iiiiii[$19 & 127]($20,$27,$34,$37,1)|0);
  $39 = $ci;
  $40 = (($buffer) + ($39<<2)|0);
  HEAP32[$40>>2] = $38;
  $41 = $ci;
  $42 = (($41) + 1)|0;
  $ci = $42;
 }
 $43 = $coef;
 $44 = (($43) + 24|0);
 $45 = HEAP32[$44>>2]|0;
 $yoffset = $45;
 L6: while(1) {
  $46 = $yoffset;
  $47 = $coef;
  $48 = (($47) + 28|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($46|0)<($49|0);
  if (!($50)) {
   break;
  }
  $51 = $coef;
  $52 = (($51) + 20|0);
  $53 = HEAP32[$52>>2]|0;
  $MCU_col_num = $53;
  while(1) {
   $54 = $MCU_col_num;
   $55 = $1;
   $56 = (($55) + 348|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = ($54>>>0)<($57>>>0);
   if (!($58)) {
    break;
   }
   $blkn = 0;
   $ci = 0;
   while(1) {
    $59 = $ci;
    $60 = $1;
    $61 = (($60) + 328|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($59|0)<($62|0);
    if (!($63)) {
     break;
    }
    $64 = $ci;
    $65 = $1;
    $66 = (($65) + 332|0);
    $67 = (($66) + ($64<<2)|0);
    $68 = HEAP32[$67>>2]|0;
    $compptr = $68;
    $69 = $MCU_col_num;
    $70 = $compptr;
    $71 = (($70) + 52|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = Math_imul($69, $72)|0;
    $start_col = $73;
    $yindex = 0;
    while(1) {
     $74 = $yindex;
     $75 = $compptr;
     $76 = (($75) + 56|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = ($74|0)<($77|0);
     if (!($78)) {
      break;
     }
     $79 = $yindex;
     $80 = $yoffset;
     $81 = (($79) + ($80))|0;
     $82 = $ci;
     $83 = (($buffer) + ($82<<2)|0);
     $84 = HEAP32[$83>>2]|0;
     $85 = (($84) + ($81<<2)|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = $start_col;
     $88 = (($86) + ($87<<7)|0);
     $buffer_ptr = $88;
     $xindex = 0;
     while(1) {
      $89 = $xindex;
      $90 = $compptr;
      $91 = (($90) + 52|0);
      $92 = HEAP32[$91>>2]|0;
      $93 = ($89|0)<($92|0);
      if (!($93)) {
       break;
      }
      $94 = $buffer_ptr;
      $95 = (($94) + 128|0);
      $buffer_ptr = $95;
      $96 = $blkn;
      $97 = (($96) + 1)|0;
      $blkn = $97;
      $98 = $coef;
      $99 = (($98) + 32|0);
      $100 = (($99) + ($96<<2)|0);
      HEAP32[$100>>2] = $94;
      $101 = $xindex;
      $102 = (($101) + 1)|0;
      $xindex = $102;
     }
     $103 = $yindex;
     $104 = (($103) + 1)|0;
     $yindex = $104;
    }
    $105 = $ci;
    $106 = (($105) + 1)|0;
    $ci = $106;
   }
   $107 = $1;
   $108 = (($107) + 444|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + 4|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $1;
   $113 = $coef;
   $114 = (($113) + 32|0);
   $115 = (FUNCTION_TABLE_iii[$111 & 127]($112,$114)|0);
   $116 = ($115|0)!=(0);
   if (!($116)) {
    label = 22;
    break L6;
   }
   $123 = $MCU_col_num;
   $124 = (($123) + 1)|0;
   $MCU_col_num = $124;
  }
  $125 = $coef;
  $126 = (($125) + 20|0);
  HEAP32[$126>>2] = 0;
  $127 = $yoffset;
  $128 = (($127) + 1)|0;
  $yoffset = $128;
 }
 if ((label|0) == 22) {
  $117 = $yoffset;
  $118 = $coef;
  $119 = (($118) + 24|0);
  HEAP32[$119>>2] = $117;
  $120 = $MCU_col_num;
  $121 = $coef;
  $122 = (($121) + 20|0);
  HEAP32[$122>>2] = $120;
  $0 = 0;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $129 = $1;
 $130 = (($129) + 148|0);
 $131 = HEAP32[$130>>2]|0;
 $132 = (($131) + 1)|0;
 HEAP32[$130>>2] = $132;
 $133 = $1;
 $134 = (($133) + 320|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = ($132>>>0)<($135>>>0);
 if ($136) {
  $137 = $1;
  _start_iMCU_row($137);
  $0 = 3;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 } else {
  $138 = $1;
  $139 = (($138) + 436|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = (($140) + 12|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = $1;
  FUNCTION_TABLE_vi[$142 & 127]($143);
  $0 = 4;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 return 0|0;
}
function _decompress_data($cinfo,$output_buf) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $block_num = 0, $block_row = 0, $block_rows = 0, $buffer = 0, $buffer_ptr = 0, $ci = 0, $coef = 0, $compptr = 0;
 var $inverse_DCT = 0, $last_iMCU_row = 0, $output_col = 0, $output_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $cinfo;
 $2 = $output_buf;
 $3 = $1;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$4>>2]|0;
 $coef = $5;
 $6 = $1;
 $7 = (($6) + 320|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $last_iMCU_row = $9;
 while(1) {
  $10 = $1;
  $11 = (($10) + 144|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $1;
  $14 = (($13) + 152|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if ($16) {
   $150 = 1;
  } else {
   $17 = $1;
   $18 = (($17) + 144|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = $1;
   $21 = (($20) + 152|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)==($22|0);
   if ($23) {
    $24 = $1;
    $25 = (($24) + 148|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = $1;
    $28 = (($27) + 156|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($26>>>0)<=($29>>>0);
    $151 = $30;
   } else {
    $151 = 0;
   }
   $150 = $151;
  }
  if (!($150)) {
   break;
  }
  $31 = $1;
  $32 = (($31) + 436|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $1;
  $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
  $37 = ($36|0)==(0);
  if ($37) {
   label = 8;
   break;
  }
 }
 if ((label|0) == 8) {
  $0 = 0;
  $149 = $0;
  STACKTOP = sp;return ($149|0);
 }
 $ci = 0;
 $38 = $1;
 $39 = (($38) + 216|0);
 $40 = HEAP32[$39>>2]|0;
 $compptr = $40;
 while(1) {
  $41 = $ci;
  $42 = $1;
  $43 = (($42) + 36|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ($41|0)<($44|0);
  if (!($45)) {
   break;
  }
  $46 = $compptr;
  $47 = (($46) + 48|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)!=(0);
  if ($49) {
   $50 = $1;
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 32|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = $1;
   $56 = $ci;
   $57 = $coef;
   $58 = (($57) + 72|0);
   $59 = (($58) + ($56<<2)|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $1;
   $62 = (($61) + 156|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $compptr;
   $65 = (($64) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = Math_imul($63, $66)|0;
   $68 = $compptr;
   $69 = (($68) + 12|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (FUNCTION_TABLE_iiiiii[$54 & 127]($55,$60,$67,$70,0)|0);
   $buffer = $71;
   $72 = $1;
   $73 = (($72) + 156|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = $last_iMCU_row;
   $76 = ($74>>>0)<($75>>>0);
   if ($76) {
    $77 = $compptr;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $block_rows = $79;
   } else {
    $80 = $compptr;
    $81 = (($80) + 32|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = $compptr;
    $84 = (($83) + 12|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = (($82>>>0) % ($85>>>0))&-1;
    $block_rows = $86;
    $87 = $block_rows;
    $88 = ($87|0)==(0);
    if ($88) {
     $89 = $compptr;
     $90 = (($89) + 12|0);
     $91 = HEAP32[$90>>2]|0;
     $block_rows = $91;
    }
   }
   $92 = $ci;
   $93 = $1;
   $94 = (($93) + 448|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (($95) + 4|0);
   $97 = (($96) + ($92<<2)|0);
   $98 = HEAP32[$97>>2]|0;
   $inverse_DCT = $98;
   $99 = $ci;
   $100 = $2;
   $101 = (($100) + ($99<<2)|0);
   $102 = HEAP32[$101>>2]|0;
   $output_ptr = $102;
   $block_row = 0;
   while(1) {
    $103 = $block_row;
    $104 = $block_rows;
    $105 = ($103|0)<($104|0);
    if (!($105)) {
     break;
    }
    $106 = $block_row;
    $107 = $buffer;
    $108 = (($107) + ($106<<2)|0);
    $109 = HEAP32[$108>>2]|0;
    $buffer_ptr = $109;
    $output_col = 0;
    $block_num = 0;
    while(1) {
     $110 = $block_num;
     $111 = $compptr;
     $112 = (($111) + 28|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = ($110>>>0)<($113>>>0);
     if (!($114)) {
      break;
     }
     $115 = $inverse_DCT;
     $116 = $1;
     $117 = $compptr;
     $118 = $buffer_ptr;
     $119 = $output_ptr;
     $120 = $output_col;
     FUNCTION_TABLE_viiiii[$115 & 127]($116,$117,$118,$119,$120);
     $121 = $buffer_ptr;
     $122 = (($121) + 128|0);
     $buffer_ptr = $122;
     $123 = $compptr;
     $124 = (($123) + 36|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = $output_col;
     $127 = (($126) + ($125))|0;
     $output_col = $127;
     $128 = $block_num;
     $129 = (($128) + 1)|0;
     $block_num = $129;
    }
    $130 = $compptr;
    $131 = (($130) + 36|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = $output_ptr;
    $134 = (($133) + ($132<<2)|0);
    $output_ptr = $134;
    $135 = $block_row;
    $136 = (($135) + 1)|0;
    $block_row = $136;
   }
  } else {
  }
  $137 = $ci;
  $138 = (($137) + 1)|0;
  $ci = $138;
  $139 = $compptr;
  $140 = (($139) + 84|0);
  $compptr = $140;
 }
 $141 = $1;
 $142 = (($141) + 156|0);
 $143 = HEAP32[$142>>2]|0;
 $144 = (($143) + 1)|0;
 HEAP32[$142>>2] = $144;
 $145 = $1;
 $146 = (($145) + 320|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = ($144>>>0)<($147>>>0);
 if ($148) {
  $0 = 3;
  $149 = $0;
  STACKTOP = sp;return ($149|0);
 } else {
  $0 = 4;
  $149 = $0;
  STACKTOP = sp;return ($149|0);
 }
 return 0|0;
}
function _dummy_consume_data($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return 0;
}
function _decompress_onepass($cinfo,$output_buf) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $MCU_col_num = 0, $blkn = 0, $ci = 0, $coef = 0, $compptr = 0, $inverse_DCT = 0, $last_MCU_col = 0, $last_iMCU_row = 0, $output_col = 0, $output_ptr = 0, $start_col = 0, $useful_width = 0, $xindex = 0, $yindex = 0;
 var $yoffset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $cinfo;
 $2 = $output_buf;
 $3 = $1;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$4>>2]|0;
 $coef = $5;
 $6 = $1;
 $7 = (($6) + 348|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $last_MCU_col = $9;
 $10 = $1;
 $11 = (($10) + 320|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) - 1)|0;
 $last_iMCU_row = $13;
 $14 = $coef;
 $15 = (($14) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $yoffset = $16;
 L1: while(1) {
  $17 = $yoffset;
  $18 = $coef;
  $19 = (($18) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = $coef;
  $23 = (($22) + 20|0);
  $24 = HEAP32[$23>>2]|0;
  $MCU_col_num = $24;
  while(1) {
   $25 = $MCU_col_num;
   $26 = $last_MCU_col;
   $27 = ($25>>>0)<=($26>>>0);
   if (!($27)) {
    break;
   }
   $28 = $coef;
   $29 = (($28) + 32|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $1;
   $32 = (($31) + 356|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $33<<7;
   _jzero_far($30,$34);
   $35 = $1;
   $36 = (($35) + 444|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $1;
   $41 = $coef;
   $42 = (($41) + 32|0);
   $43 = (FUNCTION_TABLE_iii[$39 & 127]($40,$42)|0);
   $44 = ($43|0)!=(0);
   if (!($44)) {
    label = 6;
    break L1;
   }
   $blkn = 0;
   $ci = 0;
   while(1) {
    $51 = $ci;
    $52 = $1;
    $53 = (($52) + 328|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = ($51|0)<($54|0);
    if (!($55)) {
     break;
    }
    $56 = $ci;
    $57 = $1;
    $58 = (($57) + 332|0);
    $59 = (($58) + ($56<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $compptr = $60;
    $61 = $compptr;
    $62 = (($61) + 48|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = ($63|0)!=(0);
    if ($64) {
     $70 = $compptr;
     $71 = (($70) + 4|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = $1;
     $74 = (($73) + 448|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + 4|0);
     $77 = (($76) + ($72<<2)|0);
     $78 = HEAP32[$77>>2]|0;
     $inverse_DCT = $78;
     $79 = $MCU_col_num;
     $80 = $last_MCU_col;
     $81 = ($79>>>0)<($80>>>0);
     if ($81) {
      $82 = $compptr;
      $83 = (($82) + 52|0);
      $84 = HEAP32[$83>>2]|0;
      $88 = $84;
     } else {
      $85 = $compptr;
      $86 = (($85) + 68|0);
      $87 = HEAP32[$86>>2]|0;
      $88 = $87;
     }
     $useful_width = $88;
     $89 = $compptr;
     $90 = (($89) + 4|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = $2;
     $93 = (($92) + ($91<<2)|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = $yoffset;
     $96 = $compptr;
     $97 = (($96) + 36|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = Math_imul($95, $98)|0;
     $100 = (($94) + ($99<<2)|0);
     $output_ptr = $100;
     $101 = $MCU_col_num;
     $102 = $compptr;
     $103 = (($102) + 64|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = Math_imul($101, $104)|0;
     $start_col = $105;
     $yindex = 0;
     while(1) {
      $106 = $yindex;
      $107 = $compptr;
      $108 = (($107) + 56|0);
      $109 = HEAP32[$108>>2]|0;
      $110 = ($106|0)<($109|0);
      if (!($110)) {
       break;
      }
      $111 = $1;
      $112 = (($111) + 148|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = $last_iMCU_row;
      $115 = ($113>>>0)<($114>>>0);
      if ($115) {
       label = 18;
      } else {
       $116 = $yoffset;
       $117 = $yindex;
       $118 = (($116) + ($117))|0;
       $119 = $compptr;
       $120 = (($119) + 72|0);
       $121 = HEAP32[$120>>2]|0;
       $122 = ($118|0)<($121|0);
       if ($122) {
        label = 18;
       }
      }
      if ((label|0) == 18) {
       label = 0;
       $123 = $start_col;
       $output_col = $123;
       $xindex = 0;
       while(1) {
        $124 = $xindex;
        $125 = $useful_width;
        $126 = ($124|0)<($125|0);
        if (!($126)) {
         break;
        }
        $127 = $inverse_DCT;
        $128 = $1;
        $129 = $compptr;
        $130 = $blkn;
        $131 = $xindex;
        $132 = (($130) + ($131))|0;
        $133 = $coef;
        $134 = (($133) + 32|0);
        $135 = (($134) + ($132<<2)|0);
        $136 = HEAP32[$135>>2]|0;
        $137 = $output_ptr;
        $138 = $output_col;
        FUNCTION_TABLE_viiiii[$127 & 127]($128,$129,$136,$137,$138);
        $139 = $compptr;
        $140 = (($139) + 36|0);
        $141 = HEAP32[$140>>2]|0;
        $142 = $output_col;
        $143 = (($142) + ($141))|0;
        $output_col = $143;
        $144 = $xindex;
        $145 = (($144) + 1)|0;
        $xindex = $145;
       }
      }
      $146 = $compptr;
      $147 = (($146) + 52|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = $blkn;
      $150 = (($149) + ($148))|0;
      $blkn = $150;
      $151 = $compptr;
      $152 = (($151) + 36|0);
      $153 = HEAP32[$152>>2]|0;
      $154 = $output_ptr;
      $155 = (($154) + ($153<<2)|0);
      $output_ptr = $155;
      $156 = $yindex;
      $157 = (($156) + 1)|0;
      $yindex = $157;
     }
    } else {
     $65 = $compptr;
     $66 = (($65) + 60|0);
     $67 = HEAP32[$66>>2]|0;
     $68 = $blkn;
     $69 = (($68) + ($67))|0;
     $blkn = $69;
    }
    $158 = $ci;
    $159 = (($158) + 1)|0;
    $ci = $159;
   }
   $160 = $MCU_col_num;
   $161 = (($160) + 1)|0;
   $MCU_col_num = $161;
  }
  $162 = $coef;
  $163 = (($162) + 20|0);
  HEAP32[$163>>2] = 0;
  $164 = $yoffset;
  $165 = (($164) + 1)|0;
  $yoffset = $165;
 }
 if ((label|0) == 6) {
  $45 = $yoffset;
  $46 = $coef;
  $47 = (($46) + 24|0);
  HEAP32[$47>>2] = $45;
  $48 = $MCU_col_num;
  $49 = $coef;
  $50 = (($49) + 20|0);
  HEAP32[$50>>2] = $48;
  $0 = 0;
  $185 = $0;
  STACKTOP = sp;return ($185|0);
 }
 $166 = $1;
 $167 = (($166) + 156|0);
 $168 = HEAP32[$167>>2]|0;
 $169 = (($168) + 1)|0;
 HEAP32[$167>>2] = $169;
 $170 = $1;
 $171 = (($170) + 148|0);
 $172 = HEAP32[$171>>2]|0;
 $173 = (($172) + 1)|0;
 HEAP32[$171>>2] = $173;
 $174 = $1;
 $175 = (($174) + 320|0);
 $176 = HEAP32[$175>>2]|0;
 $177 = ($173>>>0)<($176>>>0);
 if ($177) {
  $178 = $1;
  _start_iMCU_row($178);
  $0 = 3;
  $185 = $0;
  STACKTOP = sp;return ($185|0);
 } else {
  $179 = $1;
  $180 = (($179) + 436|0);
  $181 = HEAP32[$180>>2]|0;
  $182 = (($181) + 12|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = $1;
  FUNCTION_TABLE_vi[$183 & 127]($184);
  $0 = 4;
  $185 = $0;
  STACKTOP = sp;return ($185|0);
 }
 return 0|0;
}
function _start_iMCU_row($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $coef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 428|0);
 $3 = HEAP32[$2>>2]|0;
 $coef = $3;
 $4 = $0;
 $5 = (($4) + 328|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>(1);
 if ($7) {
  $8 = $coef;
  $9 = (($8) + 28|0);
  HEAP32[$9>>2] = 1;
  $32 = $coef;
  $33 = (($32) + 20|0);
  HEAP32[$33>>2] = 0;
  $34 = $coef;
  $35 = (($34) + 24|0);
  HEAP32[$35>>2] = 0;
  STACKTOP = sp;return;
 }
 $10 = $0;
 $11 = (($10) + 148|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 320|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = ($12>>>0)<($16>>>0);
 if ($17) {
  $18 = $0;
  $19 = (($18) + 332|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $coef;
  $24 = (($23) + 28|0);
  HEAP32[$24>>2] = $22;
 } else {
  $25 = $0;
  $26 = (($25) + 332|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 72|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $coef;
  $31 = (($30) + 28|0);
  HEAP32[$31>>2] = $29;
 }
 $32 = $coef;
 $33 = (($32) + 20|0);
 HEAP32[$33>>2] = 0;
 $34 = $coef;
 $35 = (($34) + 24|0);
 HEAP32[$35>>2] = 0;
 STACKTOP = sp;return;
}
function _smoothing_ok($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $coef = 0, $coef_bits = 0, $coef_bits_latch = 0, $coefi = 0, $compptr = 0, $qtable = 0, $smoothing_useful = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 428|0);
 $4 = HEAP32[$3>>2]|0;
 $coef = $4;
 $smoothing_useful = 0;
 $5 = $1;
 $6 = (($5) + 220|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $1;
  $10 = (($9) + 160|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)==(0|0);
  if (!($12)) {
   $13 = $coef;
   $14 = (($13) + 112|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)==(0|0);
   if ($16) {
    $17 = $1;
    $18 = (($17) + 4|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = HEAP32[$19>>2]|0;
    $21 = $1;
    $22 = $1;
    $23 = (($22) + 36|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($24*24)|0;
    $26 = (FUNCTION_TABLE_iiii[$20 & 127]($21,1,$25)|0);
    $27 = $coef;
    $28 = (($27) + 112|0);
    HEAP32[$28>>2] = $26;
   }
   $29 = $coef;
   $30 = (($29) + 112|0);
   $31 = HEAP32[$30>>2]|0;
   $coef_bits_latch = $31;
   $ci = 0;
   $32 = $1;
   $33 = (($32) + 216|0);
   $34 = HEAP32[$33>>2]|0;
   $compptr = $34;
   while(1) {
    $35 = $ci;
    $36 = $1;
    $37 = (($36) + 36|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ($35|0)<($38|0);
    if (!($39)) {
     label = 27;
     break;
    }
    $40 = $compptr;
    $41 = (($40) + 76|0);
    $42 = HEAP32[$41>>2]|0;
    $qtable = $42;
    $43 = ($42|0)==(0|0);
    if ($43) {
     label = 9;
     break;
    }
    $44 = $qtable;
    $45 = HEAP16[$44>>1]|0;
    $46 = $45&65535;
    $47 = ($46|0)==(0);
    if ($47) {
     label = 16;
     break;
    }
    $48 = $qtable;
    $49 = (($48) + 2|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = ($51|0)==(0);
    if ($52) {
     label = 16;
     break;
    }
    $53 = $qtable;
    $54 = (($53) + 16|0);
    $55 = HEAP16[$54>>1]|0;
    $56 = $55&65535;
    $57 = ($56|0)==(0);
    if ($57) {
     label = 16;
     break;
    }
    $58 = $qtable;
    $59 = (($58) + 32|0);
    $60 = HEAP16[$59>>1]|0;
    $61 = $60&65535;
    $62 = ($61|0)==(0);
    if ($62) {
     label = 16;
     break;
    }
    $63 = $qtable;
    $64 = (($63) + 18|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = $65&65535;
    $67 = ($66|0)==(0);
    if ($67) {
     label = 16;
     break;
    }
    $68 = $qtable;
    $69 = (($68) + 4|0);
    $70 = HEAP16[$69>>1]|0;
    $71 = $70&65535;
    $72 = ($71|0)==(0);
    if ($72) {
     label = 16;
     break;
    }
    $73 = $ci;
    $74 = $1;
    $75 = (($74) + 160|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = (($76) + ($73<<8)|0);
    $coef_bits = $77;
    $78 = $coef_bits;
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)<(0);
    if ($80) {
     label = 18;
     break;
    }
    $coefi = 1;
    while(1) {
     $81 = $coefi;
     $82 = ($81|0)<=(5);
     if (!($82)) {
      break;
     }
     $83 = $coefi;
     $84 = $coef_bits;
     $85 = (($84) + ($83<<2)|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = $coefi;
     $88 = $coef_bits_latch;
     $89 = (($88) + ($87<<2)|0);
     HEAP32[$89>>2] = $86;
     $90 = $coefi;
     $91 = $coef_bits;
     $92 = (($91) + ($90<<2)|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = ($93|0)!=(0);
     if ($94) {
      $smoothing_useful = 1;
     }
     $95 = $coefi;
     $96 = (($95) + 1)|0;
     $coefi = $96;
    }
    $97 = $coef_bits_latch;
    $98 = (($97) + 24|0);
    $coef_bits_latch = $98;
    $99 = $ci;
    $100 = (($99) + 1)|0;
    $ci = $100;
    $101 = $compptr;
    $102 = (($101) + 84|0);
    $compptr = $102;
   }
   if ((label|0) == 9) {
    $0 = 0;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   else if ((label|0) == 16) {
    $0 = 0;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   else if ((label|0) == 18) {
    $0 = 0;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   else if ((label|0) == 27) {
    $103 = $smoothing_useful;
    $0 = $103;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
  }
 }
 $0 = 0;
 $104 = $0;
 STACKTOP = sp;return ($104|0);
}
function _decompress_smooth_data($cinfo,$output_buf) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Al = 0;
 var $DC1 = 0, $DC2 = 0, $DC3 = 0, $DC4 = 0, $DC5 = 0, $DC6 = 0, $DC7 = 0, $DC8 = 0, $DC9 = 0, $Q00 = 0, $Q01 = 0, $Q02 = 0, $Q10 = 0, $Q11 = 0, $Q20 = 0, $access_rows = 0, $block_num = 0, $block_row = 0, $block_rows = 0, $buffer = 0;
 var $buffer_ptr = 0, $ci = 0, $coef = 0, $coef_bits = 0, $compptr = 0, $delta = 0, $first_row = 0, $inverse_DCT = 0, $last_block_column = 0, $last_iMCU_row = 0, $last_row = 0, $next_block_row = 0, $num = 0, $output_col = 0, $output_ptr = 0, $pred = 0, $prev_block_row = 0, $quanttbl = 0, $workspace = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $workspace = sp + 168|0;
 $1 = $cinfo;
 $2 = $output_buf;
 $3 = $1;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$4>>2]|0;
 $coef = $5;
 $6 = $1;
 $7 = (($6) + 320|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $last_iMCU_row = $9;
 while(1) {
  $10 = $1;
  $11 = (($10) + 144|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $1;
  $14 = (($13) + 152|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<=($15|0);
  if ($16) {
   $17 = $1;
   $18 = (($17) + 436|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0);
   $23 = $22 ^ 1;
   $558 = $23;
  } else {
   $558 = 0;
  }
  if (!($558)) {
   break;
  }
  $24 = $1;
  $25 = (($24) + 144|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $1;
  $28 = (($27) + 152|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)==($29|0);
  if ($30) {
   $31 = $1;
   $32 = (($31) + 400|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)==(0);
   $35 = $34 ? 1 : 0;
   $delta = $35;
   $36 = $1;
   $37 = (($36) + 148|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $1;
   $40 = (($39) + 156|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = $delta;
   $43 = (($41) + ($42))|0;
   $44 = ($38>>>0)>($43>>>0);
   if ($44) {
    label = 7;
    break;
   }
  }
  $45 = $1;
  $46 = (($45) + 436|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = $1;
  $50 = (FUNCTION_TABLE_ii[$48 & 127]($49)|0);
  $51 = ($50|0)==(0);
  if ($51) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 7) {
 }
 else if ((label|0) == 10) {
  $0 = 0;
  $557 = $0;
  STACKTOP = sp;return ($557|0);
 }
 $ci = 0;
 $52 = $1;
 $53 = (($52) + 216|0);
 $54 = HEAP32[$53>>2]|0;
 $compptr = $54;
 while(1) {
  $55 = $ci;
  $56 = $1;
  $57 = (($56) + 36|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ($55|0)<($58|0);
  if (!($59)) {
   break;
  }
  $60 = $compptr;
  $61 = (($60) + 48|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($62|0)!=(0);
  if ($63) {
   $64 = $1;
   $65 = (($64) + 156|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = $last_iMCU_row;
   $68 = ($66>>>0)<($67>>>0);
   if ($68) {
    $69 = $compptr;
    $70 = (($69) + 12|0);
    $71 = HEAP32[$70>>2]|0;
    $block_rows = $71;
    $72 = $block_rows;
    $73 = $72<<1;
    $access_rows = $73;
    $last_row = 0;
   } else {
    $74 = $compptr;
    $75 = (($74) + 32|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = $compptr;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = (($76>>>0) % ($79>>>0))&-1;
    $block_rows = $80;
    $81 = $block_rows;
    $82 = ($81|0)==(0);
    if ($82) {
     $83 = $compptr;
     $84 = (($83) + 12|0);
     $85 = HEAP32[$84>>2]|0;
     $block_rows = $85;
    }
    $86 = $block_rows;
    $access_rows = $86;
    $last_row = 1;
   }
   $87 = $1;
   $88 = (($87) + 156|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = ($89>>>0)>(0);
   if ($90) {
    $91 = $compptr;
    $92 = (($91) + 12|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = $access_rows;
    $95 = (($94) + ($93))|0;
    $access_rows = $95;
    $96 = $1;
    $97 = (($96) + 4|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = (($98) + 32|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $1;
    $102 = $ci;
    $103 = $coef;
    $104 = (($103) + 72|0);
    $105 = (($104) + ($102<<2)|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = $1;
    $108 = (($107) + 156|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) - 1)|0;
    $111 = $compptr;
    $112 = (($111) + 12|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = Math_imul($110, $113)|0;
    $115 = $access_rows;
    $116 = (FUNCTION_TABLE_iiiiii[$100 & 127]($101,$106,$114,$115,0)|0);
    $buffer = $116;
    $117 = $compptr;
    $118 = (($117) + 12|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = $buffer;
    $121 = (($120) + ($119<<2)|0);
    $buffer = $121;
    $first_row = 0;
   } else {
    $122 = $1;
    $123 = (($122) + 4|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = (($124) + 32|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = $1;
    $128 = $ci;
    $129 = $coef;
    $130 = (($129) + 72|0);
    $131 = (($130) + ($128<<2)|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = $access_rows;
    $134 = (FUNCTION_TABLE_iiiiii[$126 & 127]($127,$132,0,$133,0)|0);
    $buffer = $134;
    $first_row = 1;
   }
   $135 = $coef;
   $136 = (($135) + 112|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = $ci;
   $139 = ($138*6)|0;
   $140 = (($137) + ($139<<2)|0);
   $coef_bits = $140;
   $141 = $compptr;
   $142 = (($141) + 76|0);
   $143 = HEAP32[$142>>2]|0;
   $quanttbl = $143;
   $144 = $quanttbl;
   $145 = HEAP16[$144>>1]|0;
   $146 = $145&65535;
   $Q00 = $146;
   $147 = $quanttbl;
   $148 = (($147) + 2|0);
   $149 = HEAP16[$148>>1]|0;
   $150 = $149&65535;
   $Q01 = $150;
   $151 = $quanttbl;
   $152 = (($151) + 16|0);
   $153 = HEAP16[$152>>1]|0;
   $154 = $153&65535;
   $Q10 = $154;
   $155 = $quanttbl;
   $156 = (($155) + 32|0);
   $157 = HEAP16[$156>>1]|0;
   $158 = $157&65535;
   $Q20 = $158;
   $159 = $quanttbl;
   $160 = (($159) + 18|0);
   $161 = HEAP16[$160>>1]|0;
   $162 = $161&65535;
   $Q11 = $162;
   $163 = $quanttbl;
   $164 = (($163) + 4|0);
   $165 = HEAP16[$164>>1]|0;
   $166 = $165&65535;
   $Q02 = $166;
   $167 = $ci;
   $168 = $1;
   $169 = (($168) + 448|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = (($170) + 4|0);
   $172 = (($171) + ($167<<2)|0);
   $173 = HEAP32[$172>>2]|0;
   $inverse_DCT = $173;
   $174 = $ci;
   $175 = $2;
   $176 = (($175) + ($174<<2)|0);
   $177 = HEAP32[$176>>2]|0;
   $output_ptr = $177;
   $block_row = 0;
   while(1) {
    $178 = $block_row;
    $179 = $block_rows;
    $180 = ($178|0)<($179|0);
    if (!($180)) {
     break;
    }
    $181 = $block_row;
    $182 = $buffer;
    $183 = (($182) + ($181<<2)|0);
    $184 = HEAP32[$183>>2]|0;
    $buffer_ptr = $184;
    $185 = $first_row;
    $186 = ($185|0)!=(0);
    if ($186) {
     $187 = $block_row;
     $188 = ($187|0)==(0);
     if ($188) {
      $189 = $buffer_ptr;
      $prev_block_row = $189;
     } else {
      label = 29;
     }
    } else {
     label = 29;
    }
    if ((label|0) == 29) {
     label = 0;
     $190 = $block_row;
     $191 = (($190) - 1)|0;
     $192 = $buffer;
     $193 = (($192) + ($191<<2)|0);
     $194 = HEAP32[$193>>2]|0;
     $prev_block_row = $194;
    }
    $195 = $last_row;
    $196 = ($195|0)!=(0);
    if ($196) {
     $197 = $block_row;
     $198 = $block_rows;
     $199 = (($198) - 1)|0;
     $200 = ($197|0)==($199|0);
     if ($200) {
      $201 = $buffer_ptr;
      $next_block_row = $201;
     } else {
      label = 33;
     }
    } else {
     label = 33;
    }
    if ((label|0) == 33) {
     label = 0;
     $202 = $block_row;
     $203 = (($202) + 1)|0;
     $204 = $buffer;
     $205 = (($204) + ($203<<2)|0);
     $206 = HEAP32[$205>>2]|0;
     $next_block_row = $206;
    }
    $207 = $prev_block_row;
    $208 = HEAP16[$207>>1]|0;
    $209 = $208 << 16 >> 16;
    $DC3 = $209;
    $DC2 = $209;
    $DC1 = $209;
    $210 = $buffer_ptr;
    $211 = HEAP16[$210>>1]|0;
    $212 = $211 << 16 >> 16;
    $DC6 = $212;
    $DC5 = $212;
    $DC4 = $212;
    $213 = $next_block_row;
    $214 = HEAP16[$213>>1]|0;
    $215 = $214 << 16 >> 16;
    $DC9 = $215;
    $DC8 = $215;
    $DC7 = $215;
    $output_col = 0;
    $216 = $compptr;
    $217 = (($216) + 28|0);
    $218 = HEAP32[$217>>2]|0;
    $219 = (($218) - 1)|0;
    $last_block_column = $219;
    $block_num = 0;
    while(1) {
     $220 = $block_num;
     $221 = $last_block_column;
     $222 = ($220>>>0)<=($221>>>0);
     if (!($222)) {
      break;
     }
     $223 = $buffer_ptr;
     _jcopy_block_row($223,$workspace,1);
     $224 = $block_num;
     $225 = $last_block_column;
     $226 = ($224>>>0)<($225>>>0);
     if ($226) {
      $227 = $prev_block_row;
      $228 = (($227) + 128|0);
      $229 = HEAP16[$228>>1]|0;
      $230 = $229 << 16 >> 16;
      $DC3 = $230;
      $231 = $buffer_ptr;
      $232 = (($231) + 128|0);
      $233 = HEAP16[$232>>1]|0;
      $234 = $233 << 16 >> 16;
      $DC6 = $234;
      $235 = $next_block_row;
      $236 = (($235) + 128|0);
      $237 = HEAP16[$236>>1]|0;
      $238 = $237 << 16 >> 16;
      $DC9 = $238;
     }
     $239 = $coef_bits;
     $240 = (($239) + 4|0);
     $241 = HEAP32[$240>>2]|0;
     $Al = $241;
     $242 = ($241|0)!=(0);
     if ($242) {
      $243 = (($workspace) + 2|0);
      $244 = HEAP16[$243>>1]|0;
      $245 = $244 << 16 >> 16;
      $246 = ($245|0)==(0);
      if ($246) {
       $247 = $Q00;
       $248 = ($247*36)|0;
       $249 = $DC4;
       $250 = $DC6;
       $251 = (($249) - ($250))|0;
       $252 = Math_imul($248, $251)|0;
       $num = $252;
       $253 = $num;
       $254 = ($253|0)>=(0);
       if ($254) {
        $255 = $Q01;
        $256 = $255 << 7;
        $257 = $num;
        $258 = (($256) + ($257))|0;
        $259 = $Q01;
        $260 = $259 << 8;
        $261 = (($258|0) / ($260|0))&-1;
        $pred = $261;
        $262 = $Al;
        $263 = ($262|0)>(0);
        do {
         if ($263) {
          $264 = $pred;
          $265 = $Al;
          $266 = 1 << $265;
          $267 = ($264|0)>=($266|0);
          if (!($267)) {
           break;
          }
          $268 = $Al;
          $269 = 1 << $268;
          $270 = (($269) - 1)|0;
          $pred = $270;
         }
        } while(0);
       } else {
        $271 = $Q01;
        $272 = $271 << 7;
        $273 = $num;
        $274 = (($272) - ($273))|0;
        $275 = $Q01;
        $276 = $275 << 8;
        $277 = (($274|0) / ($276|0))&-1;
        $pred = $277;
        $278 = $Al;
        $279 = ($278|0)>(0);
        do {
         if ($279) {
          $280 = $pred;
          $281 = $Al;
          $282 = 1 << $281;
          $283 = ($280|0)>=($282|0);
          if (!($283)) {
           break;
          }
          $284 = $Al;
          $285 = 1 << $284;
          $286 = (($285) - 1)|0;
          $pred = $286;
         }
        } while(0);
        $287 = $pred;
        $288 = (0 - ($287))|0;
        $pred = $288;
       }
       $289 = $pred;
       $290 = $289&65535;
       $291 = (($workspace) + 2|0);
       HEAP16[$291>>1] = $290;
      }
     }
     $292 = $coef_bits;
     $293 = (($292) + 8|0);
     $294 = HEAP32[$293>>2]|0;
     $Al = $294;
     $295 = ($294|0)!=(0);
     if ($295) {
      $296 = (($workspace) + 16|0);
      $297 = HEAP16[$296>>1]|0;
      $298 = $297 << 16 >> 16;
      $299 = ($298|0)==(0);
      if ($299) {
       $300 = $Q00;
       $301 = ($300*36)|0;
       $302 = $DC2;
       $303 = $DC8;
       $304 = (($302) - ($303))|0;
       $305 = Math_imul($301, $304)|0;
       $num = $305;
       $306 = $num;
       $307 = ($306|0)>=(0);
       if ($307) {
        $308 = $Q10;
        $309 = $308 << 7;
        $310 = $num;
        $311 = (($309) + ($310))|0;
        $312 = $Q10;
        $313 = $312 << 8;
        $314 = (($311|0) / ($313|0))&-1;
        $pred = $314;
        $315 = $Al;
        $316 = ($315|0)>(0);
        do {
         if ($316) {
          $317 = $pred;
          $318 = $Al;
          $319 = 1 << $318;
          $320 = ($317|0)>=($319|0);
          if (!($320)) {
           break;
          }
          $321 = $Al;
          $322 = 1 << $321;
          $323 = (($322) - 1)|0;
          $pred = $323;
         }
        } while(0);
       } else {
        $324 = $Q10;
        $325 = $324 << 7;
        $326 = $num;
        $327 = (($325) - ($326))|0;
        $328 = $Q10;
        $329 = $328 << 8;
        $330 = (($327|0) / ($329|0))&-1;
        $pred = $330;
        $331 = $Al;
        $332 = ($331|0)>(0);
        do {
         if ($332) {
          $333 = $pred;
          $334 = $Al;
          $335 = 1 << $334;
          $336 = ($333|0)>=($335|0);
          if (!($336)) {
           break;
          }
          $337 = $Al;
          $338 = 1 << $337;
          $339 = (($338) - 1)|0;
          $pred = $339;
         }
        } while(0);
        $340 = $pred;
        $341 = (0 - ($340))|0;
        $pred = $341;
       }
       $342 = $pred;
       $343 = $342&65535;
       $344 = (($workspace) + 16|0);
       HEAP16[$344>>1] = $343;
      }
     }
     $345 = $coef_bits;
     $346 = (($345) + 12|0);
     $347 = HEAP32[$346>>2]|0;
     $Al = $347;
     $348 = ($347|0)!=(0);
     if ($348) {
      $349 = (($workspace) + 32|0);
      $350 = HEAP16[$349>>1]|0;
      $351 = $350 << 16 >> 16;
      $352 = ($351|0)==(0);
      if ($352) {
       $353 = $Q00;
       $354 = ($353*9)|0;
       $355 = $DC2;
       $356 = $DC8;
       $357 = (($355) + ($356))|0;
       $358 = $DC5;
       $359 = $358<<1;
       $360 = (($357) - ($359))|0;
       $361 = Math_imul($354, $360)|0;
       $num = $361;
       $362 = $num;
       $363 = ($362|0)>=(0);
       if ($363) {
        $364 = $Q20;
        $365 = $364 << 7;
        $366 = $num;
        $367 = (($365) + ($366))|0;
        $368 = $Q20;
        $369 = $368 << 8;
        $370 = (($367|0) / ($369|0))&-1;
        $pred = $370;
        $371 = $Al;
        $372 = ($371|0)>(0);
        do {
         if ($372) {
          $373 = $pred;
          $374 = $Al;
          $375 = 1 << $374;
          $376 = ($373|0)>=($375|0);
          if (!($376)) {
           break;
          }
          $377 = $Al;
          $378 = 1 << $377;
          $379 = (($378) - 1)|0;
          $pred = $379;
         }
        } while(0);
       } else {
        $380 = $Q20;
        $381 = $380 << 7;
        $382 = $num;
        $383 = (($381) - ($382))|0;
        $384 = $Q20;
        $385 = $384 << 8;
        $386 = (($383|0) / ($385|0))&-1;
        $pred = $386;
        $387 = $Al;
        $388 = ($387|0)>(0);
        do {
         if ($388) {
          $389 = $pred;
          $390 = $Al;
          $391 = 1 << $390;
          $392 = ($389|0)>=($391|0);
          if (!($392)) {
           break;
          }
          $393 = $Al;
          $394 = 1 << $393;
          $395 = (($394) - 1)|0;
          $pred = $395;
         }
        } while(0);
        $396 = $pred;
        $397 = (0 - ($396))|0;
        $pred = $397;
       }
       $398 = $pred;
       $399 = $398&65535;
       $400 = (($workspace) + 32|0);
       HEAP16[$400>>1] = $399;
      }
     }
     $401 = $coef_bits;
     $402 = (($401) + 16|0);
     $403 = HEAP32[$402>>2]|0;
     $Al = $403;
     $404 = ($403|0)!=(0);
     if ($404) {
      $405 = (($workspace) + 18|0);
      $406 = HEAP16[$405>>1]|0;
      $407 = $406 << 16 >> 16;
      $408 = ($407|0)==(0);
      if ($408) {
       $409 = $Q00;
       $410 = ($409*5)|0;
       $411 = $DC1;
       $412 = $DC3;
       $413 = (($411) - ($412))|0;
       $414 = $DC7;
       $415 = (($413) - ($414))|0;
       $416 = $DC9;
       $417 = (($415) + ($416))|0;
       $418 = Math_imul($410, $417)|0;
       $num = $418;
       $419 = $num;
       $420 = ($419|0)>=(0);
       if ($420) {
        $421 = $Q11;
        $422 = $421 << 7;
        $423 = $num;
        $424 = (($422) + ($423))|0;
        $425 = $Q11;
        $426 = $425 << 8;
        $427 = (($424|0) / ($426|0))&-1;
        $pred = $427;
        $428 = $Al;
        $429 = ($428|0)>(0);
        do {
         if ($429) {
          $430 = $pred;
          $431 = $Al;
          $432 = 1 << $431;
          $433 = ($430|0)>=($432|0);
          if (!($433)) {
           break;
          }
          $434 = $Al;
          $435 = 1 << $434;
          $436 = (($435) - 1)|0;
          $pred = $436;
         }
        } while(0);
       } else {
        $437 = $Q11;
        $438 = $437 << 7;
        $439 = $num;
        $440 = (($438) - ($439))|0;
        $441 = $Q11;
        $442 = $441 << 8;
        $443 = (($440|0) / ($442|0))&-1;
        $pred = $443;
        $444 = $Al;
        $445 = ($444|0)>(0);
        do {
         if ($445) {
          $446 = $pred;
          $447 = $Al;
          $448 = 1 << $447;
          $449 = ($446|0)>=($448|0);
          if (!($449)) {
           break;
          }
          $450 = $Al;
          $451 = 1 << $450;
          $452 = (($451) - 1)|0;
          $pred = $452;
         }
        } while(0);
        $453 = $pred;
        $454 = (0 - ($453))|0;
        $pred = $454;
       }
       $455 = $pred;
       $456 = $455&65535;
       $457 = (($workspace) + 18|0);
       HEAP16[$457>>1] = $456;
      }
     }
     $458 = $coef_bits;
     $459 = (($458) + 20|0);
     $460 = HEAP32[$459>>2]|0;
     $Al = $460;
     $461 = ($460|0)!=(0);
     if ($461) {
      $462 = (($workspace) + 4|0);
      $463 = HEAP16[$462>>1]|0;
      $464 = $463 << 16 >> 16;
      $465 = ($464|0)==(0);
      if ($465) {
       $466 = $Q00;
       $467 = ($466*9)|0;
       $468 = $DC4;
       $469 = $DC6;
       $470 = (($468) + ($469))|0;
       $471 = $DC5;
       $472 = $471<<1;
       $473 = (($470) - ($472))|0;
       $474 = Math_imul($467, $473)|0;
       $num = $474;
       $475 = $num;
       $476 = ($475|0)>=(0);
       if ($476) {
        $477 = $Q02;
        $478 = $477 << 7;
        $479 = $num;
        $480 = (($478) + ($479))|0;
        $481 = $Q02;
        $482 = $481 << 8;
        $483 = (($480|0) / ($482|0))&-1;
        $pred = $483;
        $484 = $Al;
        $485 = ($484|0)>(0);
        do {
         if ($485) {
          $486 = $pred;
          $487 = $Al;
          $488 = 1 << $487;
          $489 = ($486|0)>=($488|0);
          if (!($489)) {
           break;
          }
          $490 = $Al;
          $491 = 1 << $490;
          $492 = (($491) - 1)|0;
          $pred = $492;
         }
        } while(0);
       } else {
        $493 = $Q02;
        $494 = $493 << 7;
        $495 = $num;
        $496 = (($494) - ($495))|0;
        $497 = $Q02;
        $498 = $497 << 8;
        $499 = (($496|0) / ($498|0))&-1;
        $pred = $499;
        $500 = $Al;
        $501 = ($500|0)>(0);
        do {
         if ($501) {
          $502 = $pred;
          $503 = $Al;
          $504 = 1 << $503;
          $505 = ($502|0)>=($504|0);
          if (!($505)) {
           break;
          }
          $506 = $Al;
          $507 = 1 << $506;
          $508 = (($507) - 1)|0;
          $pred = $508;
         }
        } while(0);
        $509 = $pred;
        $510 = (0 - ($509))|0;
        $pred = $510;
       }
       $511 = $pred;
       $512 = $511&65535;
       $513 = (($workspace) + 4|0);
       HEAP16[$513>>1] = $512;
      }
     }
     $514 = $inverse_DCT;
     $515 = $1;
     $516 = $compptr;
     $517 = $output_ptr;
     $518 = $output_col;
     FUNCTION_TABLE_viiiii[$514 & 127]($515,$516,$workspace,$517,$518);
     $519 = $DC2;
     $DC1 = $519;
     $520 = $DC3;
     $DC2 = $520;
     $521 = $DC5;
     $DC4 = $521;
     $522 = $DC6;
     $DC5 = $522;
     $523 = $DC8;
     $DC7 = $523;
     $524 = $DC9;
     $DC8 = $524;
     $525 = $buffer_ptr;
     $526 = (($525) + 128|0);
     $buffer_ptr = $526;
     $527 = $prev_block_row;
     $528 = (($527) + 128|0);
     $prev_block_row = $528;
     $529 = $next_block_row;
     $530 = (($529) + 128|0);
     $next_block_row = $530;
     $531 = $compptr;
     $532 = (($531) + 36|0);
     $533 = HEAP32[$532>>2]|0;
     $534 = $output_col;
     $535 = (($534) + ($533))|0;
     $output_col = $535;
     $536 = $block_num;
     $537 = (($536) + 1)|0;
     $block_num = $537;
    }
    $538 = $compptr;
    $539 = (($538) + 36|0);
    $540 = HEAP32[$539>>2]|0;
    $541 = $output_ptr;
    $542 = (($541) + ($540<<2)|0);
    $output_ptr = $542;
    $543 = $block_row;
    $544 = (($543) + 1)|0;
    $block_row = $544;
   }
  } else {
  }
  $545 = $ci;
  $546 = (($545) + 1)|0;
  $ci = $546;
  $547 = $compptr;
  $548 = (($547) + 84|0);
  $compptr = $548;
 }
 $549 = $1;
 $550 = (($549) + 156|0);
 $551 = HEAP32[$550>>2]|0;
 $552 = (($551) + 1)|0;
 HEAP32[$550>>2] = $552;
 $553 = $1;
 $554 = (($553) + 320|0);
 $555 = HEAP32[$554>>2]|0;
 $556 = ($552>>>0)<($555>>>0);
 if ($556) {
  $0 = 3;
  $557 = $0;
  STACKTOP = sp;return ($557|0);
 } else {
  $0 = 4;
  $557 = $0;
  STACKTOP = sp;return ($557|0);
 }
 return 0|0;
}
function _jinit_color_deconverter($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cconvert = 0, $ci = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,24)|0);
 $cconvert = $6;
 $7 = $cconvert;
 $8 = $0;
 $9 = (($8) + 456|0);
 HEAP32[$9>>2] = $7;
 $10 = $cconvert;
 HEAP32[$10>>2] = 98;
 $11 = $0;
 $12 = (($11) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 switch ($13|0) {
 case 1:  {
  $14 = $0;
  $15 = (($14) + 36|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(1);
  if ($17) {
   $18 = $0;
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 20|0);
   HEAP32[$20>>2] = 10;
   $21 = $0;
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = $0;
   FUNCTION_TABLE_vi[$23 & 127]($24);
  }
  break;
 }
 case 3: case 2:  {
  $25 = $0;
  $26 = (($25) + 36|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(3);
  if ($28) {
   $29 = $0;
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 20|0);
   HEAP32[$31>>2] = 10;
   $32 = $0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   $35 = $0;
   FUNCTION_TABLE_vi[$34 & 127]($35);
  }
  break;
 }
 case 5: case 4:  {
  $36 = $0;
  $37 = (($36) + 36|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(4);
  if ($39) {
   $40 = $0;
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + 20|0);
   HEAP32[$42>>2] = 10;
   $43 = $0;
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $0;
   FUNCTION_TABLE_vi[$45 & 127]($46);
  }
  break;
 }
 default: {
  $47 = $0;
  $48 = (($47) + 36|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($49|0)<(1);
  if ($50) {
   $51 = $0;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 20|0);
   HEAP32[$53>>2] = 10;
   $54 = $0;
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$55>>2]|0;
   $57 = $0;
   FUNCTION_TABLE_vi[$56 & 127]($57);
  }
 }
 }
 $58 = $0;
 $59 = (($58) + 44|0);
 $60 = HEAP32[$59>>2]|0;
 if ((($60|0) == 1)) {
  $61 = $0;
  $62 = (($61) + 120|0);
  HEAP32[$62>>2] = 1;
  $63 = $0;
  $64 = (($63) + 40|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)==(1);
  if ($66) {
   label = 17;
  } else {
   $67 = $0;
   $68 = (($67) + 40|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)==(3);
   if ($70) {
    label = 17;
   } else {
    $86 = $0;
    $87 = HEAP32[$86>>2]|0;
    $88 = (($87) + 20|0);
    HEAP32[$88>>2] = 27;
    $89 = $0;
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = $0;
    FUNCTION_TABLE_vi[$91 & 127]($92);
   }
  }
  if ((label|0) == 17) {
   $71 = $cconvert;
   $72 = (($71) + 4|0);
   HEAP32[$72>>2] = 99;
   $ci = 1;
   while(1) {
    $73 = $ci;
    $74 = $0;
    $75 = (($74) + 36|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = ($73|0)<($76|0);
    if (!($77)) {
     break;
    }
    $78 = $ci;
    $79 = $0;
    $80 = (($79) + 216|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = (($81) + (($78*84)|0)|0);
    $83 = (($82) + 48|0);
    HEAP32[$83>>2] = 0;
    $84 = $ci;
    $85 = (($84) + 1)|0;
    $ci = $85;
   }
  }
 } else if ((($60|0) == 2)) {
  $93 = $0;
  $94 = (($93) + 120|0);
  HEAP32[$94>>2] = 3;
  $95 = $0;
  $96 = (($95) + 40|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($97|0)==(3);
  if ($98) {
   $99 = $cconvert;
   $100 = (($99) + 4|0);
   HEAP32[$100>>2] = 100;
   $101 = $0;
   _build_ycc_rgb_table335($101);
  } else {
   $102 = $0;
   $103 = (($102) + 40|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)==(1);
   if ($105) {
    $106 = $cconvert;
    $107 = (($106) + 4|0);
    HEAP32[$107>>2] = 101;
   } else {
    $108 = $0;
    $109 = (($108) + 40|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = ($110|0)==(2);
    if ($111) {
     $112 = $cconvert;
     $113 = (($112) + 4|0);
     HEAP32[$113>>2] = 102;
    } else {
     $114 = $0;
     $115 = HEAP32[$114>>2]|0;
     $116 = (($115) + 20|0);
     HEAP32[$116>>2] = 27;
     $117 = $0;
     $118 = HEAP32[$117>>2]|0;
     $119 = HEAP32[$118>>2]|0;
     $120 = $0;
     FUNCTION_TABLE_vi[$119 & 127]($120);
    }
   }
  }
 } else if ((($60|0) == 4)) {
  $121 = $0;
  $122 = (($121) + 120|0);
  HEAP32[$122>>2] = 4;
  $123 = $0;
  $124 = (($123) + 40|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = ($125|0)==(5);
  if ($126) {
   $127 = $cconvert;
   $128 = (($127) + 4|0);
   HEAP32[$128>>2] = 103;
   $129 = $0;
   _build_ycc_rgb_table335($129);
  } else {
   $130 = $0;
   $131 = (($130) + 40|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = ($132|0)==(4);
   if ($133) {
    $134 = $cconvert;
    $135 = (($134) + 4|0);
    HEAP32[$135>>2] = 102;
   } else {
    $136 = $0;
    $137 = HEAP32[$136>>2]|0;
    $138 = (($137) + 20|0);
    HEAP32[$138>>2] = 27;
    $139 = $0;
    $140 = HEAP32[$139>>2]|0;
    $141 = HEAP32[$140>>2]|0;
    $142 = $0;
    FUNCTION_TABLE_vi[$141 & 127]($142);
   }
  }
 } else {
  $143 = $0;
  $144 = (($143) + 44|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = $0;
  $147 = (($146) + 40|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = ($145|0)==($148|0);
  if ($149) {
   $150 = $0;
   $151 = (($150) + 36|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = $0;
   $154 = (($153) + 120|0);
   HEAP32[$154>>2] = $152;
   $155 = $cconvert;
   $156 = (($155) + 4|0);
   HEAP32[$156>>2] = 102;
  } else {
   $157 = $0;
   $158 = HEAP32[$157>>2]|0;
   $159 = (($158) + 20|0);
   HEAP32[$159>>2] = 27;
   $160 = $0;
   $161 = HEAP32[$160>>2]|0;
   $162 = HEAP32[$161>>2]|0;
   $163 = $0;
   FUNCTION_TABLE_vi[$162 & 127]($163);
  }
 }
 $164 = $0;
 $165 = (($164) + 84|0);
 $166 = HEAP32[$165>>2]|0;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = $0;
  $169 = (($168) + 124|0);
  HEAP32[$169>>2] = 1;
  STACKTOP = sp;return;
 } else {
  $170 = $0;
  $171 = (($170) + 120|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = $0;
  $174 = (($173) + 124|0);
  HEAP32[$174>>2] = $172;
  STACKTOP = sp;return;
 }
}
function _start_pass_dcolor($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _grayscale_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $1;
 $6 = HEAP32[$5>>2]|0;
 $7 = $2;
 $8 = $3;
 $9 = $4;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 _jcopy_sample_rows($6,$7,$8,0,$9,$12);
 STACKTOP = sp;return;
}
function _ycc_rgb_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cconvert = 0, $col = 0, $cr = 0, $inptr0 = 0;
 var $inptr1 = 0, $inptr2 = 0, $num_cols = 0, $outptr = 0, $range_limit = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 456|0);
 $7 = HEAP32[$6>>2]|0;
 $cconvert = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $num_cols = $10;
 $11 = $0;
 $12 = (($11) + 324|0);
 $13 = HEAP32[$12>>2]|0;
 $range_limit = $13;
 $14 = $cconvert;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $Crrtab = $16;
 $17 = $cconvert;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $Cbbtab = $19;
 $20 = $cconvert;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $Crgtab = $22;
 $23 = $cconvert;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $Cbgtab = $25;
 while(1) {
  $26 = $4;
  $27 = (($26) + -1)|0;
  $4 = $27;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   break;
  }
  $29 = $2;
  $30 = $1;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($29<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $inptr0 = $33;
  $34 = $2;
  $35 = $1;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inptr1 = $39;
  $40 = $2;
  $41 = $1;
  $42 = (($41) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $inptr2 = $45;
  $46 = $2;
  $47 = (($46) + 1)|0;
  $2 = $47;
  $48 = $3;
  $49 = (($48) + 4|0);
  $3 = $49;
  $50 = HEAP32[$48>>2]|0;
  $outptr = $50;
  $col = 0;
  while(1) {
   $51 = $col;
   $52 = $num_cols;
   $53 = ($51>>>0)<($52>>>0);
   if (!($53)) {
    break;
   }
   $54 = $col;
   $55 = $inptr0;
   $56 = (($55) + ($54)|0);
   $57 = HEAP8[$56>>0]|0;
   $58 = $57&255;
   $y = $58;
   $59 = $col;
   $60 = $inptr1;
   $61 = (($60) + ($59)|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $cb = $63;
   $64 = $col;
   $65 = $inptr2;
   $66 = (($65) + ($64)|0);
   $67 = HEAP8[$66>>0]|0;
   $68 = $67&255;
   $cr = $68;
   $69 = $y;
   $70 = $cr;
   $71 = $Crrtab;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($69) + ($73))|0;
   $75 = $range_limit;
   $76 = (($75) + ($74)|0);
   $77 = HEAP8[$76>>0]|0;
   $78 = $outptr;
   HEAP8[$78>>0] = $77;
   $79 = $y;
   $80 = $cb;
   $81 = $Cbgtab;
   $82 = (($81) + ($80<<2)|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = $cr;
   $85 = $Crgtab;
   $86 = (($85) + ($84<<2)|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (($83) + ($87))|0;
   $89 = $88 >> 16;
   $90 = (($79) + ($89))|0;
   $91 = $range_limit;
   $92 = (($91) + ($90)|0);
   $93 = HEAP8[$92>>0]|0;
   $94 = $outptr;
   $95 = (($94) + 1|0);
   HEAP8[$95>>0] = $93;
   $96 = $y;
   $97 = $cb;
   $98 = $Cbbtab;
   $99 = (($98) + ($97<<2)|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (($96) + ($100))|0;
   $102 = $range_limit;
   $103 = (($102) + ($101)|0);
   $104 = HEAP8[$103>>0]|0;
   $105 = $outptr;
   $106 = (($105) + 2|0);
   HEAP8[$106>>0] = $104;
   $107 = $outptr;
   $108 = (($107) + 3|0);
   $outptr = $108;
   $109 = $col;
   $110 = (($109) + 1)|0;
   $col = $110;
  }
 }
 STACKTOP = sp;return;
}
function _build_ycc_rgb_table335($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $cconvert = 0, $i = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 456|0);
 $3 = HEAP32[$2>>2]|0;
 $cconvert = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 127]($8,1,1024)|0);
 $10 = $cconvert;
 $11 = (($10) + 8|0);
 HEAP32[$11>>2] = $9;
 $12 = $0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 $17 = (FUNCTION_TABLE_iiii[$15 & 127]($16,1,1024)|0);
 $18 = $cconvert;
 $19 = (($18) + 12|0);
 HEAP32[$19>>2] = $17;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = (FUNCTION_TABLE_iiii[$23 & 127]($24,1,1024)|0);
 $26 = $cconvert;
 $27 = (($26) + 16|0);
 HEAP32[$27>>2] = $25;
 $28 = $0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = $0;
 $33 = (FUNCTION_TABLE_iiii[$31 & 127]($32,1,1024)|0);
 $34 = $cconvert;
 $35 = (($34) + 20|0);
 HEAP32[$35>>2] = $33;
 $i = 0;
 $x = -128;
 while(1) {
  $36 = $i;
  $37 = ($36|0)<=(255);
  if (!($37)) {
   break;
  }
  $38 = $x;
  $39 = ($38*91881)|0;
  $40 = (($39) + 32768)|0;
  $41 = $40 >> 16;
  $42 = $i;
  $43 = $cconvert;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  HEAP32[$46>>2] = $41;
  $47 = $x;
  $48 = ($47*116130)|0;
  $49 = (($48) + 32768)|0;
  $50 = $49 >> 16;
  $51 = $i;
  $52 = $cconvert;
  $53 = (($52) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  HEAP32[$55>>2] = $50;
  $56 = $x;
  $57 = Math_imul(-46802, $56)|0;
  $58 = $i;
  $59 = $cconvert;
  $60 = (($59) + 16|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + ($58<<2)|0);
  HEAP32[$62>>2] = $57;
  $63 = $x;
  $64 = Math_imul(-22554, $63)|0;
  $65 = (($64) + 32768)|0;
  $66 = $i;
  $67 = $cconvert;
  $68 = (($67) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + ($66<<2)|0);
  HEAP32[$70>>2] = $65;
  $71 = $i;
  $72 = (($71) + 1)|0;
  $i = $72;
  $73 = $x;
  $74 = (($73) + 1)|0;
  $x = $74;
 }
 STACKTOP = sp;return;
}
function _gray_rgb_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $inptr = 0, $num_cols = 0, $outptr = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 112|0);
 $7 = HEAP32[$6>>2]|0;
 $num_cols = $7;
 while(1) {
  $8 = $4;
  $9 = (($8) + -1)|0;
  $4 = $9;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = $2;
  $12 = (($11) + 1)|0;
  $2 = $12;
  $13 = $1;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $inptr = $16;
  $17 = $3;
  $18 = (($17) + 4|0);
  $3 = $18;
  $19 = HEAP32[$17>>2]|0;
  $outptr = $19;
  $col = 0;
  while(1) {
   $20 = $col;
   $21 = $num_cols;
   $22 = ($20>>>0)<($21>>>0);
   if (!($22)) {
    break;
   }
   $23 = $col;
   $24 = $inptr;
   $25 = (($24) + ($23)|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $outptr;
   $28 = (($27) + 2|0);
   HEAP8[$28>>0] = $26;
   $29 = $outptr;
   $30 = (($29) + 1|0);
   HEAP8[$30>>0] = $26;
   $31 = $outptr;
   HEAP8[$31>>0] = $26;
   $32 = $outptr;
   $33 = (($32) + 3|0);
   $outptr = $33;
   $34 = $col;
   $35 = (($34) + 1)|0;
   $col = $35;
  }
 }
 STACKTOP = sp;return;
}
function _null_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $count = 0, $inptr = 0, $num_cols = 0, $num_components = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $num_components = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $num_cols = $10;
 while(1) {
  $11 = $4;
  $12 = (($11) + -1)|0;
  $4 = $12;
  $13 = ($12|0)>=(0);
  if (!($13)) {
   break;
  }
  $ci = 0;
  while(1) {
   $14 = $ci;
   $15 = $num_components;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $2;
   $18 = $ci;
   $19 = $1;
   $20 = (($19) + ($18<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($17<<2)|0);
   $23 = HEAP32[$22>>2]|0;
   $inptr = $23;
   $24 = $3;
   $25 = HEAP32[$24>>2]|0;
   $26 = $ci;
   $27 = (($25) + ($26)|0);
   $outptr = $27;
   $28 = $num_cols;
   $count = $28;
   while(1) {
    $29 = $count;
    $30 = ($29>>>0)>(0);
    if (!($30)) {
     break;
    }
    $31 = $inptr;
    $32 = (($31) + 1|0);
    $inptr = $32;
    $33 = HEAP8[$31>>0]|0;
    $34 = $outptr;
    HEAP8[$34>>0] = $33;
    $35 = $num_components;
    $36 = $outptr;
    $37 = (($36) + ($35)|0);
    $outptr = $37;
    $38 = $count;
    $39 = (($38) + -1)|0;
    $count = $39;
   }
   $40 = $ci;
   $41 = (($40) + 1)|0;
   $ci = $41;
  }
  $42 = $2;
  $43 = (($42) + 1)|0;
  $2 = $43;
  $44 = $3;
  $45 = (($44) + 4|0);
  $3 = $45;
 }
 STACKTOP = sp;return;
}
function _ycck_cmyk_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cconvert = 0, $col = 0, $cr = 0, $inptr0 = 0, $inptr1 = 0, $inptr2 = 0, $inptr3 = 0, $num_cols = 0, $outptr = 0;
 var $range_limit = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 456|0);
 $7 = HEAP32[$6>>2]|0;
 $cconvert = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $num_cols = $10;
 $11 = $0;
 $12 = (($11) + 324|0);
 $13 = HEAP32[$12>>2]|0;
 $range_limit = $13;
 $14 = $cconvert;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $Crrtab = $16;
 $17 = $cconvert;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $Cbbtab = $19;
 $20 = $cconvert;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $Crgtab = $22;
 $23 = $cconvert;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $Cbgtab = $25;
 while(1) {
  $26 = $4;
  $27 = (($26) + -1)|0;
  $4 = $27;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   break;
  }
  $29 = $2;
  $30 = $1;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($29<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $inptr0 = $33;
  $34 = $2;
  $35 = $1;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inptr1 = $39;
  $40 = $2;
  $41 = $1;
  $42 = (($41) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $inptr2 = $45;
  $46 = $2;
  $47 = $1;
  $48 = (($47) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($46<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $inptr3 = $51;
  $52 = $2;
  $53 = (($52) + 1)|0;
  $2 = $53;
  $54 = $3;
  $55 = (($54) + 4|0);
  $3 = $55;
  $56 = HEAP32[$54>>2]|0;
  $outptr = $56;
  $col = 0;
  while(1) {
   $57 = $col;
   $58 = $num_cols;
   $59 = ($57>>>0)<($58>>>0);
   if (!($59)) {
    break;
   }
   $60 = $col;
   $61 = $inptr0;
   $62 = (($61) + ($60)|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = $63&255;
   $y = $64;
   $65 = $col;
   $66 = $inptr1;
   $67 = (($66) + ($65)|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = $68&255;
   $cb = $69;
   $70 = $col;
   $71 = $inptr2;
   $72 = (($71) + ($70)|0);
   $73 = HEAP8[$72>>0]|0;
   $74 = $73&255;
   $cr = $74;
   $75 = $y;
   $76 = $cr;
   $77 = $Crrtab;
   $78 = (($77) + ($76<<2)|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($75) + ($79))|0;
   $81 = (255 - ($80))|0;
   $82 = $range_limit;
   $83 = (($82) + ($81)|0);
   $84 = HEAP8[$83>>0]|0;
   $85 = $outptr;
   HEAP8[$85>>0] = $84;
   $86 = $y;
   $87 = $cb;
   $88 = $Cbgtab;
   $89 = (($88) + ($87<<2)|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = $cr;
   $92 = $Crgtab;
   $93 = (($92) + ($91<<2)|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($90) + ($94))|0;
   $96 = $95 >> 16;
   $97 = (($86) + ($96))|0;
   $98 = (255 - ($97))|0;
   $99 = $range_limit;
   $100 = (($99) + ($98)|0);
   $101 = HEAP8[$100>>0]|0;
   $102 = $outptr;
   $103 = (($102) + 1|0);
   HEAP8[$103>>0] = $101;
   $104 = $y;
   $105 = $cb;
   $106 = $Cbbtab;
   $107 = (($106) + ($105<<2)|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($104) + ($108))|0;
   $110 = (255 - ($109))|0;
   $111 = $range_limit;
   $112 = (($111) + ($110)|0);
   $113 = HEAP8[$112>>0]|0;
   $114 = $outptr;
   $115 = (($114) + 2|0);
   HEAP8[$115>>0] = $113;
   $116 = $col;
   $117 = $inptr3;
   $118 = (($117) + ($116)|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = $outptr;
   $121 = (($120) + 3|0);
   HEAP8[$121>>0] = $119;
   $122 = $outptr;
   $123 = (($122) + 4|0);
   $outptr = $123;
   $124 = $col;
   $125 = (($124) + 1)|0;
   $col = $125;
  }
 }
 STACKTOP = sp;return;
}
function _jinit_inverse_dct($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $compptr = 0;
 var $idct = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,84)|0);
 $idct = $6;
 $7 = $idct;
 $8 = $0;
 $9 = (($8) + 448|0);
 HEAP32[$9>>2] = $7;
 $10 = $idct;
 HEAP32[$10>>2] = 104;
 $ci = 0;
 $11 = $0;
 $12 = (($11) + 216|0);
 $13 = HEAP32[$12>>2]|0;
 $compptr = $13;
 while(1) {
  $14 = $ci;
  $15 = $0;
  $16 = (($15) + 36|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($14|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $0;
  $20 = (($19) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = $0;
  $24 = (FUNCTION_TABLE_iiii[$22 & 127]($23,1,256)|0);
  $25 = $compptr;
  $26 = (($25) + 80|0);
  HEAP32[$26>>2] = $24;
  $27 = $compptr;
  $28 = (($27) + 80|0);
  $29 = HEAP32[$28>>2]|0;
  _memset(($29|0),0,256)|0;
  $30 = $ci;
  $31 = $idct;
  $32 = (($31) + 44|0);
  $33 = (($32) + ($30<<2)|0);
  HEAP32[$33>>2] = -1;
  $34 = $ci;
  $35 = (($34) + 1)|0;
  $ci = $35;
  $36 = $compptr;
  $37 = (($36) + 84|0);
  $compptr = $37;
 }
 STACKTOP = sp;return;
}
function _start_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0.0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $ci = 0, $col = 0, $compptr = 0, $fmtbl = 0, $i = 0, $idct = 0, $ifmtbl = 0, $ismtbl = 0, $method = 0, $method_ptr = 0, $qtbl = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 448|0);
 $3 = HEAP32[$2>>2]|0;
 $idct = $3;
 $method = 0;
 $method_ptr = 0;
 $ci = 0;
 $4 = $0;
 $5 = (($4) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $compptr = $6;
 while(1) {
  $7 = $ci;
  $8 = $0;
  $9 = (($8) + 36|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = $compptr;
  $13 = (($12) + 36|0);
  $14 = HEAP32[$13>>2]|0;
  if ((($14|0) == 8)) {
   $15 = $0;
   $16 = (($15) + 72|0);
   $17 = HEAP32[$16>>2]|0;
   if ((($17|0) == 0)) {
    $method_ptr = 108;
    $method = 0;
   } else if ((($17|0) == 1)) {
    $method_ptr = 109;
    $method = 1;
   } else if ((($17|0) == 2)) {
    $method_ptr = 110;
    $method = 2;
   } else {
    $18 = $0;
    $19 = HEAP32[$18>>2]|0;
    $20 = (($19) + 20|0);
    HEAP32[$20>>2] = 48;
    $21 = $0;
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$22>>2]|0;
    $24 = $0;
    FUNCTION_TABLE_vi[$23 & 127]($24);
   }
  } else if ((($14|0) == 1)) {
   $method_ptr = 105;
   $method = 0;
  } else if ((($14|0) == 4)) {
   $method_ptr = 107;
   $method = 0;
  } else if ((($14|0) == 2)) {
   $method_ptr = 106;
   $method = 0;
  } else {
   $25 = $0;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + 20|0);
   HEAP32[$27>>2] = 7;
   $28 = $compptr;
   $29 = (($28) + 36|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $0;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 24|0);
   HEAP32[$33>>2] = $30;
   $34 = $0;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $0;
   FUNCTION_TABLE_vi[$36 & 127]($37);
  }
  $38 = $method_ptr;
  $39 = $ci;
  $40 = $idct;
  $41 = (($40) + 4|0);
  $42 = (($41) + ($39<<2)|0);
  HEAP32[$42>>2] = $38;
  $43 = $compptr;
  $44 = (($43) + 48|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)!=(0);
  do {
   if ($46) {
    $47 = $ci;
    $48 = $idct;
    $49 = (($48) + 44|0);
    $50 = (($49) + ($47<<2)|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = $method;
    $53 = ($51|0)==($52|0);
    if ($53) {
     label = 16;
    } else {
     $54 = $compptr;
     $55 = (($54) + 76|0);
     $56 = HEAP32[$55>>2]|0;
     $qtbl = $56;
     $57 = $qtbl;
     $58 = ($57|0)==(0|0);
     if ($58) {
      break;
     }
     $59 = $method;
     $60 = $ci;
     $61 = $idct;
     $62 = (($61) + 44|0);
     $63 = (($62) + ($60<<2)|0);
     HEAP32[$63>>2] = $59;
     $64 = $method;
     if ((($64|0) == 1)) {
      $80 = $compptr;
      $81 = (($80) + 80|0);
      $82 = HEAP32[$81>>2]|0;
      $ifmtbl = $82;
      $i = 0;
      while(1) {
       $83 = $i;
       $84 = ($83|0)<(64);
       if (!($84)) {
        break;
       }
       $85 = $i;
       $86 = $qtbl;
       $87 = (($86) + ($85<<1)|0);
       $88 = HEAP16[$87>>1]|0;
       $89 = $88&65535;
       $90 = $i;
       $91 = (9824 + ($90<<1)|0);
       $92 = HEAP16[$91>>1]|0;
       $93 = $92 << 16 >> 16;
       $94 = Math_imul($89, $93)|0;
       $95 = (($94) + 2048)|0;
       $96 = $95 >> 12;
       $97 = $i;
       $98 = $ifmtbl;
       $99 = (($98) + ($97<<2)|0);
       HEAP32[$99>>2] = $96;
       $100 = $i;
       $101 = (($100) + 1)|0;
       $i = $101;
      }
     } else if ((($64|0) == 0)) {
      $65 = $compptr;
      $66 = (($65) + 80|0);
      $67 = HEAP32[$66>>2]|0;
      $ismtbl = $67;
      $i = 0;
      while(1) {
       $68 = $i;
       $69 = ($68|0)<(64);
       if (!($69)) {
        break;
       }
       $70 = $i;
       $71 = $qtbl;
       $72 = (($71) + ($70<<1)|0);
       $73 = HEAP16[$72>>1]|0;
       $74 = $73&65535;
       $75 = $i;
       $76 = $ismtbl;
       $77 = (($76) + ($75<<2)|0);
       HEAP32[$77>>2] = $74;
       $78 = $i;
       $79 = (($78) + 1)|0;
       $i = $79;
      }
     } else if ((($64|0) == 2)) {
      $102 = $compptr;
      $103 = (($102) + 80|0);
      $104 = HEAP32[$103>>2]|0;
      $fmtbl = $104;
      $i = 0;
      $row = 0;
      while(1) {
       $105 = $row;
       $106 = ($105|0)<(8);
       if (!($106)) {
        break;
       }
       $col = 0;
       while(1) {
        $107 = $col;
        $108 = ($107|0)<(8);
        if (!($108)) {
         break;
        }
        $109 = $i;
        $110 = $qtbl;
        $111 = (($110) + ($109<<1)|0);
        $112 = HEAP16[$111>>1]|0;
        $113 = (+($112&65535));
        $114 = $row;
        $115 = (9952 + ($114<<3)|0);
        $116 = +HEAPF64[$115>>3];
        $117 = $113 * $116;
        $118 = $col;
        $119 = (9952 + ($118<<3)|0);
        $120 = +HEAPF64[$119>>3];
        $121 = $117 * $120;
        $122 = $121;
        $123 = $i;
        $124 = $fmtbl;
        $125 = (($124) + ($123<<2)|0);
        HEAPF32[$125>>2] = $122;
        $126 = $i;
        $127 = (($126) + 1)|0;
        $i = $127;
        $128 = $col;
        $129 = (($128) + 1)|0;
        $col = $129;
       }
       $130 = $row;
       $131 = (($130) + 1)|0;
       $row = $131;
      }
     } else {
      $132 = $0;
      $133 = HEAP32[$132>>2]|0;
      $134 = (($133) + 20|0);
      HEAP32[$134>>2] = 48;
      $135 = $0;
      $136 = HEAP32[$135>>2]|0;
      $137 = HEAP32[$136>>2]|0;
      $138 = $0;
      FUNCTION_TABLE_vi[$137 & 127]($138);
     }
    }
   } else {
    label = 16;
   }
  } while(0);
  if ((label|0) == 16) {
   label = 0;
  }
  $139 = $ci;
  $140 = (($139) + 1)|0;
  $ci = $140;
  $141 = $compptr;
  $142 = (($141) + 84|0);
  $compptr = $142;
 }
 STACKTOP = sp;return;
}
function _jpeg_make_d_derived_tbl($cinfo,$isDC,$tblno,$pdtbl) {
 $cinfo = $cinfo|0;
 $isDC = $isDC|0;
 $tblno = $tblno|0;
 $pdtbl = $pdtbl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $ctr = 0, $dtbl = 0, $htbl = 0, $huffcode = 0, $huffsize = 0, $i = 0;
 var $l = 0, $lookbits = 0, $numsymbols = 0, $p = 0, $si = 0, $sym = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1360|0;
 $huffsize = sp + 1088|0;
 $huffcode = sp + 8|0;
 $0 = $cinfo;
 $1 = $isDC;
 $2 = $tblno;
 $3 = $pdtbl;
 $4 = $2;
 $5 = ($4|0)<(0);
 if ($5) {
  label = 3;
 } else {
  $6 = $2;
  $7 = ($6|0)>=(4);
  if ($7) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $8 = $0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 20|0);
  HEAP32[$10>>2] = 50;
  $11 = $2;
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 24|0);
  HEAP32[$14>>2] = $11;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 127]($18);
 }
 $19 = $1;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $2;
  $22 = $0;
  $23 = (($22) + 180|0);
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $31 = $25;
 } else {
  $26 = $2;
  $27 = $0;
  $28 = (($27) + 196|0);
  $29 = (($28) + ($26<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30;
 }
 $htbl = $31;
 $32 = $htbl;
 $33 = ($32|0)==(0|0);
 if ($33) {
  $34 = $0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 20|0);
  HEAP32[$36>>2] = 50;
  $37 = $2;
  $38 = $0;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 24|0);
  HEAP32[$40>>2] = $37;
  $41 = $0;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = $0;
  FUNCTION_TABLE_vi[$43 & 127]($44);
 }
 $45 = $3;
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0|0);
 if ($47) {
  $48 = $0;
  $49 = (($48) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$50>>2]|0;
  $52 = $0;
  $53 = (FUNCTION_TABLE_iiii[$51 & 127]($52,1,1424)|0);
  $54 = $3;
  HEAP32[$54>>2] = $53;
 }
 $55 = $3;
 $56 = HEAP32[$55>>2]|0;
 $dtbl = $56;
 $57 = $htbl;
 $58 = $dtbl;
 $59 = (($58) + 140|0);
 HEAP32[$59>>2] = $57;
 $p = 0;
 $l = 1;
 while(1) {
  $60 = $l;
  $61 = ($60|0)<=(16);
  if (!($61)) {
   break;
  }
  $62 = $l;
  $63 = $htbl;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $i = $66;
  $67 = $i;
  $68 = ($67|0)<(0);
  if ($68) {
   label = 15;
  } else {
   $69 = $p;
   $70 = $i;
   $71 = (($69) + ($70))|0;
   $72 = ($71|0)>(256);
   if ($72) {
    label = 15;
   }
  }
  if ((label|0) == 15) {
   label = 0;
   $73 = $0;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + 20|0);
   HEAP32[$75>>2] = 8;
   $76 = $0;
   $77 = HEAP32[$76>>2]|0;
   $78 = HEAP32[$77>>2]|0;
   $79 = $0;
   FUNCTION_TABLE_vi[$78 & 127]($79);
  }
  while(1) {
   $80 = $i;
   $81 = (($80) + -1)|0;
   $i = $81;
   $82 = ($80|0)!=(0);
   if (!($82)) {
    break;
   }
   $83 = $l;
   $84 = $83&255;
   $85 = $p;
   $86 = (($85) + 1)|0;
   $p = $86;
   $87 = (($huffsize) + ($85)|0);
   HEAP8[$87>>0] = $84;
  }
  $88 = $l;
  $89 = (($88) + 1)|0;
  $l = $89;
 }
 $90 = $p;
 $91 = (($huffsize) + ($90)|0);
 HEAP8[$91>>0] = 0;
 $92 = $p;
 $numsymbols = $92;
 $code = 0;
 $93 = HEAP8[$huffsize>>0]|0;
 $94 = $93 << 24 >> 24;
 $si = $94;
 $p = 0;
 while(1) {
  $95 = $p;
  $96 = (($huffsize) + ($95)|0);
  $97 = HEAP8[$96>>0]|0;
  $98 = ($97<<24>>24)!=(0);
  if (!($98)) {
   break;
  }
  while(1) {
   $99 = $p;
   $100 = (($huffsize) + ($99)|0);
   $101 = HEAP8[$100>>0]|0;
   $102 = $101 << 24 >> 24;
   $103 = $si;
   $104 = ($102|0)==($103|0);
   if (!($104)) {
    break;
   }
   $105 = $code;
   $106 = $p;
   $107 = (($106) + 1)|0;
   $p = $107;
   $108 = (($huffcode) + ($106<<2)|0);
   HEAP32[$108>>2] = $105;
   $109 = $code;
   $110 = (($109) + 1)|0;
   $code = $110;
  }
  $111 = $code;
  $112 = $si;
  $113 = 1 << $112;
  $114 = ($111|0)>=($113|0);
  if ($114) {
   $115 = $0;
   $116 = HEAP32[$115>>2]|0;
   $117 = (($116) + 20|0);
   HEAP32[$117>>2] = 8;
   $118 = $0;
   $119 = HEAP32[$118>>2]|0;
   $120 = HEAP32[$119>>2]|0;
   $121 = $0;
   FUNCTION_TABLE_vi[$120 & 127]($121);
  }
  $122 = $code;
  $123 = $122 << 1;
  $code = $123;
  $124 = $si;
  $125 = (($124) + 1)|0;
  $si = $125;
 }
 $p = 0;
 $l = 1;
 while(1) {
  $126 = $l;
  $127 = ($126|0)<=(16);
  if (!($127)) {
   break;
  }
  $128 = $l;
  $129 = $htbl;
  $130 = (($129) + ($128)|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = ($131<<24>>24)!=(0);
  if ($132) {
   $133 = $p;
   $134 = $p;
   $135 = (($huffcode) + ($134<<2)|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($133) - ($136))|0;
   $138 = $l;
   $139 = $dtbl;
   $140 = (($139) + 72|0);
   $141 = (($140) + ($138<<2)|0);
   HEAP32[$141>>2] = $137;
   $142 = $l;
   $143 = $htbl;
   $144 = (($143) + ($142)|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = $145&255;
   $147 = $p;
   $148 = (($147) + ($146))|0;
   $p = $148;
   $149 = $p;
   $150 = (($149) - 1)|0;
   $151 = (($huffcode) + ($150<<2)|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = $l;
   $154 = $dtbl;
   $155 = (($154) + ($153<<2)|0);
   HEAP32[$155>>2] = $152;
  } else {
   $156 = $l;
   $157 = $dtbl;
   $158 = (($157) + ($156<<2)|0);
   HEAP32[$158>>2] = -1;
  }
  $159 = $l;
  $160 = (($159) + 1)|0;
  $l = $160;
 }
 $161 = $dtbl;
 $162 = (($161) + 68|0);
 HEAP32[$162>>2] = 1048575;
 $163 = $dtbl;
 $164 = (($163) + 144|0);
 _memset(($164|0),0,1024)|0;
 $p = 0;
 $l = 1;
 while(1) {
  $165 = $l;
  $166 = ($165|0)<=(8);
  if (!($166)) {
   break;
  }
  $i = 1;
  while(1) {
   $167 = $i;
   $168 = $l;
   $169 = $htbl;
   $170 = (($169) + ($168)|0);
   $171 = HEAP8[$170>>0]|0;
   $172 = $171&255;
   $173 = ($167|0)<=($172|0);
   if (!($173)) {
    break;
   }
   $174 = $p;
   $175 = (($huffcode) + ($174<<2)|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = $l;
   $178 = (8 - ($177))|0;
   $179 = $176 << $178;
   $lookbits = $179;
   $180 = $l;
   $181 = (8 - ($180))|0;
   $182 = 1 << $181;
   $ctr = $182;
   while(1) {
    $183 = $ctr;
    $184 = ($183|0)>(0);
    if (!($184)) {
     break;
    }
    $185 = $l;
    $186 = $lookbits;
    $187 = $dtbl;
    $188 = (($187) + 144|0);
    $189 = (($188) + ($186<<2)|0);
    HEAP32[$189>>2] = $185;
    $190 = $p;
    $191 = $htbl;
    $192 = (($191) + 17|0);
    $193 = (($192) + ($190)|0);
    $194 = HEAP8[$193>>0]|0;
    $195 = $lookbits;
    $196 = $dtbl;
    $197 = (($196) + 1168|0);
    $198 = (($197) + ($195)|0);
    HEAP8[$198>>0] = $194;
    $199 = $lookbits;
    $200 = (($199) + 1)|0;
    $lookbits = $200;
    $201 = $ctr;
    $202 = (($201) + -1)|0;
    $ctr = $202;
   }
   $203 = $i;
   $204 = (($203) + 1)|0;
   $i = $204;
   $205 = $p;
   $206 = (($205) + 1)|0;
   $p = $206;
  }
  $207 = $l;
  $208 = (($207) + 1)|0;
  $l = $208;
 }
 $209 = $1;
 $210 = ($209|0)!=(0);
 if (!($210)) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $211 = $i;
  $212 = $numsymbols;
  $213 = ($211|0)<($212|0);
  if (!($213)) {
   break;
  }
  $214 = $i;
  $215 = $htbl;
  $216 = (($215) + 17|0);
  $217 = (($216) + ($214)|0);
  $218 = HEAP8[$217>>0]|0;
  $219 = $218&255;
  $sym = $219;
  $220 = $sym;
  $221 = ($220|0)<(0);
  if ($221) {
   label = 53;
  } else {
   $222 = $sym;
   $223 = ($222|0)>(15);
   if ($223) {
    label = 53;
   }
  }
  if ((label|0) == 53) {
   label = 0;
   $224 = $0;
   $225 = HEAP32[$224>>2]|0;
   $226 = (($225) + 20|0);
   HEAP32[$226>>2] = 8;
   $227 = $0;
   $228 = HEAP32[$227>>2]|0;
   $229 = HEAP32[$228>>2]|0;
   $230 = $0;
   FUNCTION_TABLE_vi[$229 & 127]($230);
  }
  $231 = $i;
  $232 = (($231) + 1)|0;
  $i = $232;
 }
 STACKTOP = sp;return;
}
function _jpeg_fill_bit_buffer($state,$get_buffer,$bits_left,$nbits) {
 $state = $state|0;
 $get_buffer = $get_buffer|0;
 $bits_left = $bits_left|0;
 $nbits = $nbits|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $bytes_in_buffer = 0, $c = 0, $cinfo = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $state;
 $2 = $get_buffer;
 $3 = $bits_left;
 $4 = $nbits;
 $5 = $1;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $1;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $1;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $cinfo = $12;
 $13 = $cinfo;
 $14 = (($13) + 416|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0);
 do {
  if ($16) {
   L4: while(1) {
    $17 = $3;
    $18 = ($17|0)<(25);
    if (!($18)) {
     label = 21;
     break;
    }
    $19 = $bytes_in_buffer;
    $20 = ($19|0)==(0);
    if ($20) {
     $21 = $cinfo;
     $22 = (($21) + 24|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = (($23) + 12|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = $cinfo;
     $27 = (FUNCTION_TABLE_ii[$25 & 127]($26)|0);
     $28 = ($27|0)!=(0);
     if (!($28)) {
      label = 6;
      break;
     }
     $29 = $cinfo;
     $30 = (($29) + 24|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = HEAP32[$31>>2]|0;
     $next_input_byte = $32;
     $33 = $cinfo;
     $34 = (($33) + 24|0);
     $35 = HEAP32[$34>>2]|0;
     $36 = (($35) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $bytes_in_buffer = $37;
    }
    $38 = $bytes_in_buffer;
    $39 = (($38) + -1)|0;
    $bytes_in_buffer = $39;
    $40 = $next_input_byte;
    $41 = (($40) + 1|0);
    $next_input_byte = $41;
    $42 = HEAP8[$40>>0]|0;
    $43 = $42&255;
    $c = $43;
    $44 = $c;
    $45 = ($44|0)==(255);
    if ($45) {
     while(1) {
      $46 = $bytes_in_buffer;
      $47 = ($46|0)==(0);
      if ($47) {
       $48 = $cinfo;
       $49 = (($48) + 24|0);
       $50 = HEAP32[$49>>2]|0;
       $51 = (($50) + 12|0);
       $52 = HEAP32[$51>>2]|0;
       $53 = $cinfo;
       $54 = (FUNCTION_TABLE_ii[$52 & 127]($53)|0);
       $55 = ($54|0)!=(0);
       if (!($55)) {
        label = 12;
        break L4;
       }
       $56 = $cinfo;
       $57 = (($56) + 24|0);
       $58 = HEAP32[$57>>2]|0;
       $59 = HEAP32[$58>>2]|0;
       $next_input_byte = $59;
       $60 = $cinfo;
       $61 = (($60) + 24|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = (($62) + 4|0);
       $64 = HEAP32[$63>>2]|0;
       $bytes_in_buffer = $64;
      }
      $65 = $bytes_in_buffer;
      $66 = (($65) + -1)|0;
      $bytes_in_buffer = $66;
      $67 = $next_input_byte;
      $68 = (($67) + 1|0);
      $next_input_byte = $68;
      $69 = HEAP8[$67>>0]|0;
      $70 = $69&255;
      $c = $70;
      $71 = $c;
      $72 = ($71|0)==(255);
      if (!($72)) {
       break;
      }
     }
     $73 = $c;
     $74 = ($73|0)==(0);
     if (!($74)) {
      label = 18;
      break;
     }
     $c = 255;
    }
    $78 = $2;
    $79 = $78 << 8;
    $80 = $c;
    $81 = $79 | $80;
    $2 = $81;
    $82 = $3;
    $83 = (($82) + 8)|0;
    $3 = $83;
   }
   if ((label|0) == 6) {
    $0 = 0;
    $120 = $0;
    STACKTOP = sp;return ($120|0);
   }
   else if ((label|0) == 12) {
    $0 = 0;
    $120 = $0;
    STACKTOP = sp;return ($120|0);
   }
   else if ((label|0) == 18) {
    $75 = $c;
    $76 = $cinfo;
    $77 = (($76) + 416|0);
    HEAP32[$77>>2] = $75;
    label = 23;
    break;
   }
   else if ((label|0) == 21) {
    break;
   }
  } else {
   label = 23;
  }
 } while(0);
 if ((label|0) == 23) {
  $84 = $4;
  $85 = $3;
  $86 = ($84|0)>($85|0);
  if ($86) {
   $87 = $cinfo;
   $88 = (($87) + 444|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 8|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = ($91|0)!=(0);
   if (!($92)) {
    $93 = $cinfo;
    $94 = HEAP32[$93>>2]|0;
    $95 = (($94) + 20|0);
    HEAP32[$95>>2] = 117;
    $96 = $cinfo;
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97) + 4|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = $cinfo;
    FUNCTION_TABLE_vii[$99 & 127]($100,-1);
    $101 = $cinfo;
    $102 = (($101) + 444|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) + 8|0);
    HEAP32[$104>>2] = 1;
   }
   $105 = $3;
   $106 = (25 - ($105))|0;
   $107 = $2;
   $108 = $107 << $106;
   $2 = $108;
   $3 = 25;
  }
 }
 $109 = $next_input_byte;
 $110 = $1;
 HEAP32[$110>>2] = $109;
 $111 = $bytes_in_buffer;
 $112 = $1;
 $113 = (($112) + 4|0);
 HEAP32[$113>>2] = $111;
 $114 = $2;
 $115 = $1;
 $116 = (($115) + 8|0);
 HEAP32[$116>>2] = $114;
 $117 = $3;
 $118 = $1;
 $119 = (($118) + 12|0);
 HEAP32[$119>>2] = $117;
 $0 = 1;
 $120 = $0;
 STACKTOP = sp;return ($120|0);
}
function _jpeg_huff_decode($state,$get_buffer,$bits_left,$htbl,$min_bits) {
 $state = $state|0;
 $get_buffer = $get_buffer|0;
 $bits_left = $bits_left|0;
 $htbl = $htbl|0;
 $min_bits = $min_bits|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $code = 0, $l = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $state;
 $2 = $get_buffer;
 $3 = $bits_left;
 $4 = $htbl;
 $5 = $min_bits;
 $6 = $5;
 $l = $6;
 $7 = $3;
 $8 = $l;
 $9 = ($7|0)<($8|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = $2;
   $12 = $3;
   $13 = $l;
   $14 = (_jpeg_fill_bit_buffer($10,$11,$12,$13)|0);
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = $1;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $2 = $18;
    $19 = $1;
    $20 = (($19) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    $3 = $21;
    break;
   }
   $0 = -1;
   $97 = $0;
   STACKTOP = sp;return ($97|0);
  }
 } while(0);
 $22 = $2;
 $23 = $l;
 $24 = $3;
 $25 = (($24) - ($23))|0;
 $3 = $25;
 $26 = $22 >> $25;
 $27 = $l;
 $28 = 1 << $27;
 $29 = (($28) - 1)|0;
 $30 = $26 & $29;
 $code = $30;
 while(1) {
  $31 = $code;
  $32 = $l;
  $33 = $4;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($31|0)>($35|0);
  if (!($36)) {
   break;
  }
  $37 = $code;
  $38 = $37 << 1;
  $code = $38;
  $39 = $3;
  $40 = ($39|0)<(1);
  if ($40) {
   $41 = $1;
   $42 = $2;
   $43 = $3;
   $44 = (_jpeg_fill_bit_buffer($41,$42,$43,1)|0);
   $45 = ($44|0)!=(0);
   if (!($45)) {
    label = 9;
    break;
   }
   $46 = $1;
   $47 = (($46) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $2 = $48;
   $49 = $1;
   $50 = (($49) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   $3 = $51;
  }
  $52 = $2;
  $53 = $3;
  $54 = (($53) - 1)|0;
  $3 = $54;
  $55 = $52 >> $54;
  $56 = $55 & 1;
  $57 = $code;
  $58 = $57 | $56;
  $code = $58;
  $59 = $l;
  $60 = (($59) + 1)|0;
  $l = $60;
 }
 if ((label|0) == 9) {
  $0 = -1;
  $97 = $0;
  STACKTOP = sp;return ($97|0);
 }
 $61 = $2;
 $62 = $1;
 $63 = (($62) + 8|0);
 HEAP32[$63>>2] = $61;
 $64 = $3;
 $65 = $1;
 $66 = (($65) + 12|0);
 HEAP32[$66>>2] = $64;
 $67 = $l;
 $68 = ($67|0)>(16);
 if ($68) {
  $69 = $1;
  $70 = (($69) + 16|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + 20|0);
  HEAP32[$73>>2] = 118;
  $74 = $1;
  $75 = (($74) + 16|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($77) + 4|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $1;
  $81 = (($80) + 16|0);
  $82 = HEAP32[$81>>2]|0;
  FUNCTION_TABLE_vii[$79 & 127]($82,-1);
  $0 = 0;
  $97 = $0;
  STACKTOP = sp;return ($97|0);
 } else {
  $83 = $code;
  $84 = $l;
  $85 = $4;
  $86 = (($85) + 72|0);
  $87 = (($86) + ($84<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($83) + ($88))|0;
  $90 = $4;
  $91 = (($90) + 140|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + 17|0);
  $94 = (($93) + ($89)|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $0 = $96;
  $97 = $0;
  STACKTOP = sp;return ($97|0);
 }
 return 0|0;
}
function _jinit_huff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $entropy = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 127]($5,1,232)|0);
 $entropy = $6;
 $7 = $entropy;
 $8 = $0;
 $9 = (($8) + 444|0);
 HEAP32[$9>>2] = $7;
 $10 = $entropy;
 HEAP32[$10>>2] = 111;
 $11 = $entropy;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 112;
 $i = 0;
 while(1) {
  $13 = $i;
  $14 = ($13|0)<(4);
  if (!($14)) {
   break;
  }
  $15 = $i;
  $16 = $entropy;
  $17 = (($16) + 56|0);
  $18 = (($17) + ($15<<2)|0);
  HEAP32[$18>>2] = 0;
  $19 = $i;
  $20 = $entropy;
  $21 = (($20) + 40|0);
  $22 = (($21) + ($19<<2)|0);
  HEAP32[$22>>2] = 0;
  $23 = $i;
  $24 = (($23) + 1)|0;
  $i = $24;
 }
 STACKTOP = sp;return;
}
function _start_pass_huff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $actbl = 0, $blkn = 0;
 var $ci = 0, $compptr = 0, $dctbl = 0, $entropy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 444|0);
 $3 = HEAP32[$2>>2]|0;
 $entropy = $3;
 $4 = $0;
 $5 = (($4) + 400|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  label = 5;
 } else {
  $8 = $0;
  $9 = (($8) + 404|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(63);
  if ($11) {
   label = 5;
  } else {
   $12 = $0;
   $13 = (($12) + 408|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)!=(0);
   if ($15) {
    label = 5;
   } else {
    $16 = $0;
    $17 = (($16) + 412|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($18|0)!=(0);
    if ($19) {
     label = 5;
    }
   }
  }
 }
 if ((label|0) == 5) {
  $20 = $0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 20|0);
  HEAP32[$22>>2] = 122;
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vii[$26 & 127]($27,-1);
 }
 $ci = 0;
 while(1) {
  $28 = $ci;
  $29 = $0;
  $30 = (($29) + 328|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = $ci;
  $34 = $0;
  $35 = (($34) + 332|0);
  $36 = (($35) + ($33<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $compptr = $37;
  $38 = $compptr;
  $39 = (($38) + 20|0);
  $40 = HEAP32[$39>>2]|0;
  $dctbl = $40;
  $41 = $compptr;
  $42 = (($41) + 24|0);
  $43 = HEAP32[$42>>2]|0;
  $actbl = $43;
  $44 = $0;
  $45 = $dctbl;
  $46 = $dctbl;
  $47 = $entropy;
  $48 = (($47) + 40|0);
  $49 = (($48) + ($46<<2)|0);
  _jpeg_make_d_derived_tbl($44,1,$45,$49);
  $50 = $0;
  $51 = $actbl;
  $52 = $actbl;
  $53 = $entropy;
  $54 = (($53) + 56|0);
  $55 = (($54) + ($52<<2)|0);
  _jpeg_make_d_derived_tbl($50,0,$51,$55);
  $56 = $ci;
  $57 = $entropy;
  $58 = (($57) + 20|0);
  $59 = (($58) + ($56<<2)|0);
  HEAP32[$59>>2] = 0;
  $60 = $ci;
  $61 = (($60) + 1)|0;
  $ci = $61;
 }
 $blkn = 0;
 while(1) {
  $62 = $blkn;
  $63 = $0;
  $64 = (($63) + 356|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($62|0)<($65|0);
  if (!($66)) {
   break;
  }
  $67 = $blkn;
  $68 = $0;
  $69 = (($68) + 360|0);
  $70 = (($69) + ($67<<2)|0);
  $71 = HEAP32[$70>>2]|0;
  $ci = $71;
  $72 = $ci;
  $73 = $0;
  $74 = (($73) + 332|0);
  $75 = (($74) + ($72<<2)|0);
  $76 = HEAP32[$75>>2]|0;
  $compptr = $76;
  $77 = $compptr;
  $78 = (($77) + 20|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $entropy;
  $81 = (($80) + 40|0);
  $82 = (($81) + ($79<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $blkn;
  $85 = $entropy;
  $86 = (($85) + 72|0);
  $87 = (($86) + ($84<<2)|0);
  HEAP32[$87>>2] = $83;
  $88 = $compptr;
  $89 = (($88) + 24|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = $entropy;
  $92 = (($91) + 56|0);
  $93 = (($92) + ($90<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $blkn;
  $96 = $entropy;
  $97 = (($96) + 112|0);
  $98 = (($97) + ($95<<2)|0);
  HEAP32[$98>>2] = $94;
  $99 = $compptr;
  $100 = (($99) + 48|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = ($101|0)!=(0);
  if ($102) {
   $103 = $blkn;
   $104 = $entropy;
   $105 = (($104) + 152|0);
   $106 = (($105) + ($103<<2)|0);
   HEAP32[$106>>2] = 1;
   $107 = $compptr;
   $108 = (($107) + 36|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = ($109|0)>(1);
   $111 = $110&1;
   $112 = $blkn;
   $113 = $entropy;
   $114 = (($113) + 192|0);
   $115 = (($114) + ($112<<2)|0);
   HEAP32[$115>>2] = $111;
  } else {
   $116 = $blkn;
   $117 = $entropy;
   $118 = (($117) + 192|0);
   $119 = (($118) + ($116<<2)|0);
   HEAP32[$119>>2] = 0;
   $120 = $blkn;
   $121 = $entropy;
   $122 = (($121) + 152|0);
   $123 = (($122) + ($120<<2)|0);
   HEAP32[$123>>2] = 0;
  }
  $124 = $blkn;
  $125 = (($124) + 1)|0;
  $blkn = $125;
 }
 $126 = $entropy;
 $127 = (($126) + 12|0);
 $128 = (($127) + 4|0);
 HEAP32[$128>>2] = 0;
 $129 = $entropy;
 $130 = (($129) + 12|0);
 HEAP32[$130>>2] = 0;
 $131 = $entropy;
 $132 = (($131) + 8|0);
 HEAP32[$132>>2] = 0;
 $133 = $0;
 $134 = (($133) + 276|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = $entropy;
 $137 = (($136) + 36|0);
 HEAP32[$137>>2] = $135;
 STACKTOP = sp;return;
}
function _decode_mcu($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $actbl = 0, $bits_left = 0, $blkn = 0, $block = 0, $br_state = 0, $ci = 0, $dctbl = 0, $entropy = 0, $get_buffer = 0, $k = 0, $look = 0, $look2 = 0, $look4 = 0, $nb = 0, $nb1 = 0;
 var $nb3 = 0, $r = 0, $s = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $br_state = sp + 72|0;
 $state = sp + 56|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 276|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = $entropy;
  $11 = (($10) + 36|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)==(0);
  do {
   if ($13) {
    $14 = $1;
    $15 = (_process_restart348($14)|0);
    $16 = ($15|0)!=(0);
    if ($16) {
     break;
    }
    $0 = 0;
    $364 = $0;
    STACKTOP = sp;return ($364|0);
   }
  } while(0);
 }
 $17 = $entropy;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0);
 L11: do {
  if (!($20)) {
   $21 = $1;
   $22 = (($br_state) + 16|0);
   HEAP32[$22>>2] = $21;
   $23 = $1;
   $24 = (($23) + 24|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$br_state>>2] = $26;
   $27 = $1;
   $28 = (($27) + 24|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($br_state) + 4|0);
   HEAP32[$32>>2] = $31;
   $33 = $entropy;
   $34 = (($33) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $get_buffer = $35;
   $36 = $entropy;
   $37 = (($36) + 12|0);
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $bits_left = $39;
   $40 = $entropy;
   $41 = (($40) + 20|0);
   ;HEAP32[$state+0>>2]=HEAP32[$41+0>>2]|0;HEAP32[$state+4>>2]=HEAP32[$41+4>>2]|0;HEAP32[$state+8>>2]=HEAP32[$41+8>>2]|0;HEAP32[$state+12>>2]=HEAP32[$41+12>>2]|0;
   $blkn = 0;
   L13: while(1) {
    $42 = $blkn;
    $43 = $1;
    $44 = (($43) + 356|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($42|0)<($45|0);
    if (!($46)) {
     label = 91;
     break;
    }
    $47 = $blkn;
    $48 = $2;
    $49 = (($48) + ($47<<2)|0);
    $50 = HEAP32[$49>>2]|0;
    $block = $50;
    $51 = $blkn;
    $52 = $entropy;
    $53 = (($52) + 72|0);
    $54 = (($53) + ($51<<2)|0);
    $55 = HEAP32[$54>>2]|0;
    $dctbl = $55;
    $56 = $blkn;
    $57 = $entropy;
    $58 = (($57) + 112|0);
    $59 = (($58) + ($56<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $actbl = $60;
    $61 = $bits_left;
    $62 = ($61|0)<(8);
    do {
     if ($62) {
      $63 = $get_buffer;
      $64 = $bits_left;
      $65 = (_jpeg_fill_bit_buffer($br_state,$63,$64,0)|0);
      $66 = ($65|0)!=(0);
      if (!($66)) {
       label = 12;
       break L13;
      }
      $67 = (($br_state) + 8|0);
      $68 = HEAP32[$67>>2]|0;
      $get_buffer = $68;
      $69 = (($br_state) + 12|0);
      $70 = HEAP32[$69>>2]|0;
      $bits_left = $70;
      $71 = $bits_left;
      $72 = ($71|0)<(8);
      if ($72) {
       $nb = 1;
       label = 19;
       break;
      } else {
       label = 16;
       break;
      }
     } else {
      label = 16;
     }
    } while(0);
    do {
     if ((label|0) == 16) {
      label = 0;
      $73 = $get_buffer;
      $74 = $bits_left;
      $75 = (($74) - 8)|0;
      $76 = $73 >> $75;
      $77 = $76 & 255;
      $look = $77;
      $78 = $look;
      $79 = $dctbl;
      $80 = (($79) + 144|0);
      $81 = (($80) + ($78<<2)|0);
      $82 = HEAP32[$81>>2]|0;
      $nb = $82;
      $83 = ($82|0)!=(0);
      if ($83) {
       $84 = $nb;
       $85 = $bits_left;
       $86 = (($85) - ($84))|0;
       $bits_left = $86;
       $87 = $look;
       $88 = $dctbl;
       $89 = (($88) + 1168|0);
       $90 = (($89) + ($87)|0);
       $91 = HEAP8[$90>>0]|0;
       $92 = $91&255;
       $s = $92;
       break;
      } else {
       $nb = 9;
       label = 19;
       break;
      }
     }
    } while(0);
    if ((label|0) == 19) {
     label = 0;
     $93 = $get_buffer;
     $94 = $bits_left;
     $95 = $dctbl;
     $96 = $nb;
     $97 = (_jpeg_huff_decode($br_state,$93,$94,$95,$96)|0);
     $s = $97;
     $98 = ($97|0)<(0);
     if ($98) {
      label = 20;
      break;
     }
     $99 = (($br_state) + 8|0);
     $100 = HEAP32[$99>>2]|0;
     $get_buffer = $100;
     $101 = (($br_state) + 12|0);
     $102 = HEAP32[$101>>2]|0;
     $bits_left = $102;
    }
    $103 = $s;
    $104 = ($103|0)!=(0);
    if ($104) {
     $105 = $bits_left;
     $106 = $s;
     $107 = ($105|0)<($106|0);
     if ($107) {
      $108 = $get_buffer;
      $109 = $bits_left;
      $110 = $s;
      $111 = (_jpeg_fill_bit_buffer($br_state,$108,$109,$110)|0);
      $112 = ($111|0)!=(0);
      if (!($112)) {
       label = 25;
       break;
      }
      $113 = (($br_state) + 8|0);
      $114 = HEAP32[$113>>2]|0;
      $get_buffer = $114;
      $115 = (($br_state) + 12|0);
      $116 = HEAP32[$115>>2]|0;
      $bits_left = $116;
     }
     $117 = $get_buffer;
     $118 = $s;
     $119 = $bits_left;
     $120 = (($119) - ($118))|0;
     $bits_left = $120;
     $121 = $117 >> $120;
     $122 = $s;
     $123 = 1 << $122;
     $124 = (($123) - 1)|0;
     $125 = $121 & $124;
     $r = $125;
     $126 = $r;
     $127 = $s;
     $128 = (10016 + ($127<<2)|0);
     $129 = HEAP32[$128>>2]|0;
     $130 = ($126|0)<($129|0);
     if ($130) {
      $131 = $r;
      $132 = $s;
      $133 = (10080 + ($132<<2)|0);
      $134 = HEAP32[$133>>2]|0;
      $135 = (($131) + ($134))|0;
      $137 = $135;
     } else {
      $136 = $r;
      $137 = $136;
     }
     $s = $137;
    }
    $138 = $blkn;
    $139 = $entropy;
    $140 = (($139) + 152|0);
    $141 = (($140) + ($138<<2)|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ($142|0)!=(0);
    if ($143) {
     $144 = $blkn;
     $145 = $1;
     $146 = (($145) + 360|0);
     $147 = (($146) + ($144<<2)|0);
     $148 = HEAP32[$147>>2]|0;
     $ci = $148;
     $149 = $ci;
     $150 = (($state) + ($149<<2)|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = $s;
     $153 = (($152) + ($151))|0;
     $s = $153;
     $154 = $s;
     $155 = $ci;
     $156 = (($state) + ($155<<2)|0);
     HEAP32[$156>>2] = $154;
     $157 = $s;
     $158 = $157&65535;
     $159 = $block;
     HEAP16[$159>>1] = $158;
    }
    $160 = $blkn;
    $161 = $entropy;
    $162 = (($161) + 192|0);
    $163 = (($162) + ($160<<2)|0);
    $164 = HEAP32[$163>>2]|0;
    $165 = ($164|0)!=(0);
    if ($165) {
     $k = 1;
     while(1) {
      $166 = $k;
      $167 = ($166|0)<(64);
      if (!($167)) {
       break;
      }
      $168 = $bits_left;
      $169 = ($168|0)<(8);
      do {
       if ($169) {
        $170 = $get_buffer;
        $171 = $bits_left;
        $172 = (_jpeg_fill_bit_buffer($br_state,$170,$171,0)|0);
        $173 = ($172|0)!=(0);
        if (!($173)) {
         label = 38;
         break L13;
        }
        $174 = (($br_state) + 8|0);
        $175 = HEAP32[$174>>2]|0;
        $get_buffer = $175;
        $176 = (($br_state) + 12|0);
        $177 = HEAP32[$176>>2]|0;
        $bits_left = $177;
        $178 = $bits_left;
        $179 = ($178|0)<(8);
        if ($179) {
         $nb1 = 1;
         label = 45;
         break;
        } else {
         label = 42;
         break;
        }
       } else {
        label = 42;
       }
      } while(0);
      do {
       if ((label|0) == 42) {
        label = 0;
        $180 = $get_buffer;
        $181 = $bits_left;
        $182 = (($181) - 8)|0;
        $183 = $180 >> $182;
        $184 = $183 & 255;
        $look2 = $184;
        $185 = $look2;
        $186 = $actbl;
        $187 = (($186) + 144|0);
        $188 = (($187) + ($185<<2)|0);
        $189 = HEAP32[$188>>2]|0;
        $nb1 = $189;
        $190 = ($189|0)!=(0);
        if ($190) {
         $191 = $nb1;
         $192 = $bits_left;
         $193 = (($192) - ($191))|0;
         $bits_left = $193;
         $194 = $look2;
         $195 = $actbl;
         $196 = (($195) + 1168|0);
         $197 = (($196) + ($194)|0);
         $198 = HEAP8[$197>>0]|0;
         $199 = $198&255;
         $s = $199;
         break;
        } else {
         $nb1 = 9;
         label = 45;
         break;
        }
       }
      } while(0);
      if ((label|0) == 45) {
       label = 0;
       $200 = $get_buffer;
       $201 = $bits_left;
       $202 = $actbl;
       $203 = $nb1;
       $204 = (_jpeg_huff_decode($br_state,$200,$201,$202,$203)|0);
       $s = $204;
       $205 = ($204|0)<(0);
       if ($205) {
        label = 46;
        break L13;
       }
       $206 = (($br_state) + 8|0);
       $207 = HEAP32[$206>>2]|0;
       $get_buffer = $207;
       $208 = (($br_state) + 12|0);
       $209 = HEAP32[$208>>2]|0;
       $bits_left = $209;
      }
      $210 = $s;
      $211 = $210 >> 4;
      $r = $211;
      $212 = $s;
      $213 = $212 & 15;
      $s = $213;
      $214 = $s;
      $215 = ($214|0)!=(0);
      if ($215) {
       $216 = $r;
       $217 = $k;
       $218 = (($217) + ($216))|0;
       $k = $218;
       $219 = $bits_left;
       $220 = $s;
       $221 = ($219|0)<($220|0);
       if ($221) {
        $222 = $get_buffer;
        $223 = $bits_left;
        $224 = $s;
        $225 = (_jpeg_fill_bit_buffer($br_state,$222,$223,$224)|0);
        $226 = ($225|0)!=(0);
        if (!($226)) {
         label = 51;
         break L13;
        }
        $227 = (($br_state) + 8|0);
        $228 = HEAP32[$227>>2]|0;
        $get_buffer = $228;
        $229 = (($br_state) + 12|0);
        $230 = HEAP32[$229>>2]|0;
        $bits_left = $230;
       }
       $231 = $get_buffer;
       $232 = $s;
       $233 = $bits_left;
       $234 = (($233) - ($232))|0;
       $bits_left = $234;
       $235 = $231 >> $234;
       $236 = $s;
       $237 = 1 << $236;
       $238 = (($237) - 1)|0;
       $239 = $235 & $238;
       $r = $239;
       $240 = $r;
       $241 = $s;
       $242 = (10016 + ($241<<2)|0);
       $243 = HEAP32[$242>>2]|0;
       $244 = ($240|0)<($243|0);
       if ($244) {
        $245 = $r;
        $246 = $s;
        $247 = (10080 + ($246<<2)|0);
        $248 = HEAP32[$247>>2]|0;
        $249 = (($245) + ($248))|0;
        $251 = $249;
       } else {
        $250 = $r;
        $251 = $250;
       }
       $s = $251;
       $252 = $s;
       $253 = $252&65535;
       $254 = $k;
       $255 = (9504 + ($254<<2)|0);
       $256 = HEAPU8[$255>>0]|(HEAPU8[$255+1>>0]<<8)|(HEAPU8[$255+2>>0]<<16)|(HEAPU8[$255+3>>0]<<24);
       $257 = $block;
       $258 = (($257) + ($256<<1)|0);
       HEAP16[$258>>1] = $253;
      } else {
       $259 = $r;
       $260 = ($259|0)!=(15);
       if ($260) {
        label = 58;
        break;
       }
       $261 = $k;
       $262 = (($261) + 15)|0;
       $k = $262;
      }
      $263 = $k;
      $264 = (($263) + 1)|0;
      $k = $264;
     }
     if ((label|0) == 58) {
      label = 0;
     }
    } else {
     $k = 1;
     while(1) {
      $265 = $k;
      $266 = ($265|0)<(64);
      if (!($266)) {
       break;
      }
      $267 = $bits_left;
      $268 = ($267|0)<(8);
      do {
       if ($268) {
        $269 = $get_buffer;
        $270 = $bits_left;
        $271 = (_jpeg_fill_bit_buffer($br_state,$269,$270,0)|0);
        $272 = ($271|0)!=(0);
        if (!($272)) {
         label = 67;
         break L13;
        }
        $273 = (($br_state) + 8|0);
        $274 = HEAP32[$273>>2]|0;
        $get_buffer = $274;
        $275 = (($br_state) + 12|0);
        $276 = HEAP32[$275>>2]|0;
        $bits_left = $276;
        $277 = $bits_left;
        $278 = ($277|0)<(8);
        if ($278) {
         $nb3 = 1;
         label = 74;
         break;
        } else {
         label = 71;
         break;
        }
       } else {
        label = 71;
       }
      } while(0);
      do {
       if ((label|0) == 71) {
        label = 0;
        $279 = $get_buffer;
        $280 = $bits_left;
        $281 = (($280) - 8)|0;
        $282 = $279 >> $281;
        $283 = $282 & 255;
        $look4 = $283;
        $284 = $look4;
        $285 = $actbl;
        $286 = (($285) + 144|0);
        $287 = (($286) + ($284<<2)|0);
        $288 = HEAP32[$287>>2]|0;
        $nb3 = $288;
        $289 = ($288|0)!=(0);
        if ($289) {
         $290 = $nb3;
         $291 = $bits_left;
         $292 = (($291) - ($290))|0;
         $bits_left = $292;
         $293 = $look4;
         $294 = $actbl;
         $295 = (($294) + 1168|0);
         $296 = (($295) + ($293)|0);
         $297 = HEAP8[$296>>0]|0;
         $298 = $297&255;
         $s = $298;
         break;
        } else {
         $nb3 = 9;
         label = 74;
         break;
        }
       }
      } while(0);
      if ((label|0) == 74) {
       label = 0;
       $299 = $get_buffer;
       $300 = $bits_left;
       $301 = $actbl;
       $302 = $nb3;
       $303 = (_jpeg_huff_decode($br_state,$299,$300,$301,$302)|0);
       $s = $303;
       $304 = ($303|0)<(0);
       if ($304) {
        label = 75;
        break L13;
       }
       $305 = (($br_state) + 8|0);
       $306 = HEAP32[$305>>2]|0;
       $get_buffer = $306;
       $307 = (($br_state) + 12|0);
       $308 = HEAP32[$307>>2]|0;
       $bits_left = $308;
      }
      $309 = $s;
      $310 = $309 >> 4;
      $r = $310;
      $311 = $s;
      $312 = $311 & 15;
      $s = $312;
      $313 = $s;
      $314 = ($313|0)!=(0);
      if ($314) {
       $315 = $r;
       $316 = $k;
       $317 = (($316) + ($315))|0;
       $k = $317;
       $318 = $bits_left;
       $319 = $s;
       $320 = ($318|0)<($319|0);
       if ($320) {
        $321 = $get_buffer;
        $322 = $bits_left;
        $323 = $s;
        $324 = (_jpeg_fill_bit_buffer($br_state,$321,$322,$323)|0);
        $325 = ($324|0)!=(0);
        if (!($325)) {
         label = 80;
         break L13;
        }
        $326 = (($br_state) + 8|0);
        $327 = HEAP32[$326>>2]|0;
        $get_buffer = $327;
        $328 = (($br_state) + 12|0);
        $329 = HEAP32[$328>>2]|0;
        $bits_left = $329;
       }
       $330 = $s;
       $331 = $bits_left;
       $332 = (($331) - ($330))|0;
       $bits_left = $332;
      } else {
       $333 = $r;
       $334 = ($333|0)!=(15);
       if ($334) {
        label = 84;
        break;
       }
       $335 = $k;
       $336 = (($335) + 15)|0;
       $k = $336;
      }
      $337 = $k;
      $338 = (($337) + 1)|0;
      $k = $338;
     }
     if ((label|0) == 84) {
      label = 0;
     }
    }
    $339 = $blkn;
    $340 = (($339) + 1)|0;
    $blkn = $340;
   }
   switch (label|0) {
    case 12: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 20: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 25: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 38: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 46: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 51: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 67: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 75: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 80: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 91: {
     $341 = HEAP32[$br_state>>2]|0;
     $342 = $1;
     $343 = (($342) + 24|0);
     $344 = HEAP32[$343>>2]|0;
     HEAP32[$344>>2] = $341;
     $345 = (($br_state) + 4|0);
     $346 = HEAP32[$345>>2]|0;
     $347 = $1;
     $348 = (($347) + 24|0);
     $349 = HEAP32[$348>>2]|0;
     $350 = (($349) + 4|0);
     HEAP32[$350>>2] = $346;
     $351 = $get_buffer;
     $352 = $entropy;
     $353 = (($352) + 12|0);
     HEAP32[$353>>2] = $351;
     $354 = $bits_left;
     $355 = $entropy;
     $356 = (($355) + 12|0);
     $357 = (($356) + 4|0);
     HEAP32[$357>>2] = $354;
     $358 = $entropy;
     $359 = (($358) + 20|0);
     ;HEAP32[$359+0>>2]=HEAP32[$state+0>>2]|0;HEAP32[$359+4>>2]=HEAP32[$state+4>>2]|0;HEAP32[$359+8>>2]=HEAP32[$state+8>>2]|0;HEAP32[$359+12>>2]=HEAP32[$state+12>>2]|0;
     break L11;
     break;
    }
   }
  }
 } while(0);
 $360 = $entropy;
 $361 = (($360) + 36|0);
 $362 = HEAP32[$361>>2]|0;
 $363 = (($362) + -1)|0;
 HEAP32[$361>>2] = $363;
 $0 = 1;
 $364 = $0;
 STACKTOP = sp;return ($364|0);
}
function _process_restart348($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $entropy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 444|0);
 $4 = HEAP32[$3>>2]|0;
 $entropy = $4;
 $5 = $entropy;
 $6 = (($5) + 12|0);
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8|0) / 8)&-1;
 $10 = $1;
 $11 = (($10) + 440|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($9))|0;
 HEAP32[$13>>2] = $15;
 $16 = $entropy;
 $17 = (($16) + 12|0);
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = 0;
 $19 = $1;
 $20 = (($19) + 440|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = (FUNCTION_TABLE_ii[$23 & 127]($24)|0);
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $0 = 0;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 $ci = 0;
 while(1) {
  $27 = $ci;
  $28 = $1;
  $29 = (($28) + 328|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = $ci;
  $33 = $entropy;
  $34 = (($33) + 20|0);
  $35 = (($34) + ($32<<2)|0);
  HEAP32[$35>>2] = 0;
  $36 = $ci;
  $37 = (($36) + 1)|0;
  $ci = $37;
 }
 $38 = $1;
 $39 = (($38) + 276|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $entropy;
 $42 = (($41) + 36|0);
 HEAP32[$42>>2] = $40;
 $43 = $1;
 $44 = (($43) + 416|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  $47 = $entropy;
  $48 = (($47) + 8|0);
  HEAP32[$48>>2] = 0;
 }
 $0 = 1;
 $49 = $0;
 STACKTOP = sp;return ($49|0);
}
function _jinit_d_main_controller($cinfo,$need_full_buffer) {
 $cinfo = $cinfo|0;
 $need_full_buffer = $need_full_buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $ci = 0, $compptr = 0, $main = 0, $ngroups = 0, $rgroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $need_full_buffer;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 127]($6,1,80)|0);
 $main = $7;
 $8 = $main;
 $9 = $0;
 $10 = (($9) + 424|0);
 HEAP32[$10>>2] = $8;
 $11 = $main;
 HEAP32[$11>>2] = 113;
 $12 = $1;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $0;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 20|0);
  HEAP32[$16>>2] = 4;
  $17 = $0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = $0;
  FUNCTION_TABLE_vi[$19 & 127]($20);
 }
 $21 = $0;
 $22 = (($21) + 452|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $0;
  $28 = (($27) + 316|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)<(2);
  if ($30) {
   $31 = $0;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 20|0);
   HEAP32[$33>>2] = 47;
   $34 = $0;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $0;
   FUNCTION_TABLE_vi[$36 & 127]($37);
  }
  $38 = $0;
  _alloc_funny_pointers($38);
  $39 = $0;
  $40 = (($39) + 316|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 2)|0;
  $ngroups = $42;
 } else {
  $43 = $0;
  $44 = (($43) + 316|0);
  $45 = HEAP32[$44>>2]|0;
  $ngroups = $45;
 }
 $ci = 0;
 $46 = $0;
 $47 = (($46) + 216|0);
 $48 = HEAP32[$47>>2]|0;
 $compptr = $48;
 while(1) {
  $49 = $ci;
  $50 = $0;
  $51 = (($50) + 36|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($49|0)<($52|0);
  if (!($53)) {
   break;
  }
  $54 = $compptr;
  $55 = (($54) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $compptr;
  $58 = (($57) + 36|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = Math_imul($56, $59)|0;
  $61 = $0;
  $62 = (($61) + 316|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($60|0) / ($63|0))&-1;
  $rgroup = $64;
  $65 = $0;
  $66 = (($65) + 4|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = $0;
  $71 = $compptr;
  $72 = (($71) + 28|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $compptr;
  $75 = (($74) + 36|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = Math_imul($73, $76)|0;
  $78 = $rgroup;
  $79 = $ngroups;
  $80 = Math_imul($78, $79)|0;
  $81 = (FUNCTION_TABLE_iiiii[$69 & 127]($70,1,$77,$80)|0);
  $82 = $ci;
  $83 = $main;
  $84 = (($83) + 8|0);
  $85 = (($84) + ($82<<2)|0);
  HEAP32[$85>>2] = $81;
  $86 = $ci;
  $87 = (($86) + 1)|0;
  $ci = $87;
  $88 = $compptr;
  $89 = (($88) + 84|0);
  $compptr = $89;
 }
 STACKTOP = sp;return;
}
function _start_pass_main($cinfo,$pass_mode) {
 $cinfo = $cinfo|0;
 $pass_mode = $pass_mode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $main = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $pass_mode;
 $2 = $0;
 $3 = (($2) + 424|0);
 $4 = HEAP32[$3>>2]|0;
 $main = $4;
 $5 = $1;
 if ((($5|0) == 0)) {
  $6 = $0;
  $7 = (($6) + 452|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0);
  if ($11) {
   $12 = $main;
   $13 = (($12) + 4|0);
   HEAP32[$13>>2] = 114;
   $14 = $0;
   _make_funny_pointers($14);
   $15 = $main;
   $16 = (($15) + 64|0);
   HEAP32[$16>>2] = 0;
   $17 = $main;
   $18 = (($17) + 68|0);
   HEAP32[$18>>2] = 0;
   $19 = $main;
   $20 = (($19) + 76|0);
   HEAP32[$20>>2] = 0;
  } else {
   $21 = $main;
   $22 = (($21) + 4|0);
   HEAP32[$22>>2] = 115;
  }
  $23 = $main;
  $24 = (($23) + 48|0);
  HEAP32[$24>>2] = 0;
  $25 = $main;
  $26 = (($25) + 52|0);
  HEAP32[$26>>2] = 0;
  STACKTOP = sp;return;
 } else if ((($5|0) == 2)) {
  $27 = $main;
  $28 = (($27) + 4|0);
  HEAP32[$28>>2] = 116;
  STACKTOP = sp;return;
 } else {
  $29 = $0;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 20|0);
  HEAP32[$31>>2] = 4;
  $32 = $0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  FUNCTION_TABLE_vi[$34 & 127]($35);
  STACKTOP = sp;return;
 }
}
function _alloc_funny_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $M = 0, $ci = 0, $compptr = 0, $main = 0, $rgroup = 0, $xbuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $4 = $0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 $M = $6;
 $7 = $0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = $0;
 $12 = $0;
 $13 = (($12) + 36|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14<<1;
 $16 = $15<<2;
 $17 = (FUNCTION_TABLE_iiii[$10 & 127]($11,1,$16)|0);
 $18 = $main;
 $19 = (($18) + 56|0);
 HEAP32[$19>>2] = $17;
 $20 = $main;
 $21 = (($20) + 56|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = (($23) + 36|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($22) + ($25<<2)|0);
 $27 = $main;
 $28 = (($27) + 56|0);
 $29 = (($28) + 4|0);
 HEAP32[$29>>2] = $26;
 $ci = 0;
 $30 = $0;
 $31 = (($30) + 216|0);
 $32 = HEAP32[$31>>2]|0;
 $compptr = $32;
 while(1) {
  $33 = $ci;
  $34 = $0;
  $35 = (($34) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($33|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = $compptr;
  $39 = (($38) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $compptr;
  $42 = (($41) + 36|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = Math_imul($40, $43)|0;
  $45 = $0;
  $46 = (($45) + 316|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($44|0) / ($47|0))&-1;
  $rgroup = $48;
  $49 = $0;
  $50 = (($49) + 4|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $53 = $0;
  $54 = $rgroup;
  $55 = $M;
  $56 = (($55) + 4)|0;
  $57 = Math_imul($54, $56)|0;
  $58 = $57<<1;
  $59 = $58<<2;
  $60 = (FUNCTION_TABLE_iiii[$52 & 127]($53,1,$59)|0);
  $xbuf = $60;
  $61 = $rgroup;
  $62 = $xbuf;
  $63 = (($62) + ($61<<2)|0);
  $xbuf = $63;
  $64 = $xbuf;
  $65 = $ci;
  $66 = $main;
  $67 = (($66) + 56|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + ($65<<2)|0);
  HEAP32[$69>>2] = $64;
  $70 = $rgroup;
  $71 = $M;
  $72 = (($71) + 4)|0;
  $73 = Math_imul($70, $72)|0;
  $74 = $xbuf;
  $75 = (($74) + ($73<<2)|0);
  $xbuf = $75;
  $76 = $xbuf;
  $77 = $ci;
  $78 = $main;
  $79 = (($78) + 56|0);
  $80 = (($79) + 4|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + ($77<<2)|0);
  HEAP32[$82>>2] = $76;
  $83 = $ci;
  $84 = (($83) + 1)|0;
  $ci = $84;
  $85 = $compptr;
  $86 = (($85) + 84|0);
  $compptr = $86;
 }
 STACKTOP = sp;return;
}
function _process_data_context_main($cinfo,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $main = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $output_buf;
 $2 = $out_row_ctr;
 $3 = $out_rows_avail;
 $4 = $0;
 $5 = (($4) + 424|0);
 $6 = HEAP32[$5>>2]|0;
 $main = $6;
 $7 = $main;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 do {
  if (!($10)) {
   $11 = $0;
   $12 = (($11) + 428|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (($13) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $main;
   $18 = (($17) + 64|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = $main;
   $21 = (($20) + 56|0);
   $22 = (($21) + ($19<<2)|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (FUNCTION_TABLE_iii[$15 & 127]($16,$23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $main;
    $27 = (($26) + 48|0);
    HEAP32[$27>>2] = 1;
    $28 = $main;
    $29 = (($28) + 76|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$29>>2] = $31;
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 $32 = $main;
 $33 = (($32) + 68|0);
 $34 = HEAP32[$33>>2]|0;
 if ((($34|0) == 2)) {
  label = 6;
 } else if ((($34|0) == 0)) {
  label = 11;
 } else if (!((($34|0) == 1))) {
  STACKTOP = sp;return;
 }
 do {
  if ((label|0) == 6) {
   $35 = $0;
   $36 = (($35) + 432|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $0;
   $41 = $main;
   $42 = (($41) + 64|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = $main;
   $45 = (($44) + 56|0);
   $46 = (($45) + ($43<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $main;
   $49 = (($48) + 52|0);
   $50 = $main;
   $51 = (($50) + 72|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = $1;
   $54 = $2;
   $55 = $3;
   FUNCTION_TABLE_viiiiiii[$39 & 63]($40,$47,$49,$52,$53,$54,$55);
   $56 = $main;
   $57 = (($56) + 52|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = $main;
   $60 = (($59) + 72|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ($58>>>0)<($61>>>0);
   if ($62) {
    STACKTOP = sp;return;
   }
   $63 = $main;
   $64 = (($63) + 68|0);
   HEAP32[$64>>2] = 0;
   $65 = $2;
   $66 = HEAP32[$65>>2]|0;
   $67 = $3;
   $68 = ($66>>>0)>=($67>>>0);
   if (!($68)) {
    label = 11;
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 if ((label|0) == 11) {
  $69 = $main;
  $70 = (($69) + 52|0);
  HEAP32[$70>>2] = 0;
  $71 = $0;
  $72 = (($71) + 316|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) - 1)|0;
  $75 = $main;
  $76 = (($75) + 72|0);
  HEAP32[$76>>2] = $74;
  $77 = $main;
  $78 = (($77) + 76|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $0;
  $81 = (($80) + 320|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($79|0)==($82|0);
  if ($83) {
   $84 = $0;
   _set_bottom_pointers($84);
  }
  $85 = $main;
  $86 = (($85) + 68|0);
  HEAP32[$86>>2] = 1;
 }
 $87 = $0;
 $88 = (($87) + 432|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + 4|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $0;
 $93 = $main;
 $94 = (($93) + 64|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = $main;
 $97 = (($96) + 56|0);
 $98 = (($97) + ($95<<2)|0);
 $99 = HEAP32[$98>>2]|0;
 $100 = $main;
 $101 = (($100) + 52|0);
 $102 = $main;
 $103 = (($102) + 72|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = $1;
 $106 = $2;
 $107 = $3;
 FUNCTION_TABLE_viiiiiii[$91 & 63]($92,$99,$101,$104,$105,$106,$107);
 $108 = $main;
 $109 = (($108) + 52|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = $main;
 $112 = (($111) + 72|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = ($110>>>0)<($113>>>0);
 if ($114) {
  STACKTOP = sp;return;
 }
 $115 = $main;
 $116 = (($115) + 76|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = ($117|0)==(1);
 if ($118) {
  $119 = $0;
  _set_wraparound_pointers($119);
 }
 $120 = $main;
 $121 = (($120) + 64|0);
 $122 = HEAP32[$121>>2]|0;
 $123 = $122 ^ 1;
 HEAP32[$121>>2] = $123;
 $124 = $main;
 $125 = (($124) + 48|0);
 HEAP32[$125>>2] = 0;
 $126 = $0;
 $127 = (($126) + 316|0);
 $128 = HEAP32[$127>>2]|0;
 $129 = (($128) + 1)|0;
 $130 = $main;
 $131 = (($130) + 52|0);
 HEAP32[$131>>2] = $129;
 $132 = $0;
 $133 = (($132) + 316|0);
 $134 = HEAP32[$133>>2]|0;
 $135 = (($134) + 2)|0;
 $136 = $main;
 $137 = (($136) + 72|0);
 HEAP32[$137>>2] = $135;
 $138 = $main;
 $139 = (($138) + 68|0);
 HEAP32[$139>>2] = 2;
 STACKTOP = sp;return;
}
function _make_funny_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $M = 0, $buf = 0, $ci = 0, $compptr = 0;
 var $i = 0, $main = 0, $rgroup = 0, $xbuf0 = 0, $xbuf1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $4 = $0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 $M = $6;
 $ci = 0;
 $7 = $0;
 $8 = (($7) + 216|0);
 $9 = HEAP32[$8>>2]|0;
 $compptr = $9;
 while(1) {
  $10 = $ci;
  $11 = $0;
  $12 = (($11) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $compptr;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $compptr;
  $19 = (($18) + 36|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = Math_imul($17, $20)|0;
  $22 = $0;
  $23 = (($22) + 316|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($21|0) / ($24|0))&-1;
  $rgroup = $25;
  $26 = $ci;
  $27 = $main;
  $28 = (($27) + 56|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $xbuf0 = $31;
  $32 = $ci;
  $33 = $main;
  $34 = (($33) + 56|0);
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + ($32<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $xbuf1 = $38;
  $39 = $ci;
  $40 = $main;
  $41 = (($40) + 8|0);
  $42 = (($41) + ($39<<2)|0);
  $43 = HEAP32[$42>>2]|0;
  $buf = $43;
  $i = 0;
  while(1) {
   $44 = $i;
   $45 = $rgroup;
   $46 = $M;
   $47 = (($46) + 2)|0;
   $48 = Math_imul($45, $47)|0;
   $49 = ($44|0)<($48|0);
   if (!($49)) {
    break;
   }
   $50 = $i;
   $51 = $buf;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = $i;
   $55 = $xbuf1;
   $56 = (($55) + ($54<<2)|0);
   HEAP32[$56>>2] = $53;
   $57 = $i;
   $58 = $xbuf0;
   $59 = (($58) + ($57<<2)|0);
   HEAP32[$59>>2] = $53;
   $60 = $i;
   $61 = (($60) + 1)|0;
   $i = $61;
  }
  $i = 0;
  while(1) {
   $62 = $i;
   $63 = $rgroup;
   $64 = $63<<1;
   $65 = ($62|0)<($64|0);
   if (!($65)) {
    break;
   }
   $66 = $rgroup;
   $67 = $M;
   $68 = Math_imul($66, $67)|0;
   $69 = $i;
   $70 = (($68) + ($69))|0;
   $71 = $buf;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = $rgroup;
   $75 = $M;
   $76 = (($75) - 2)|0;
   $77 = Math_imul($74, $76)|0;
   $78 = $i;
   $79 = (($77) + ($78))|0;
   $80 = $xbuf1;
   $81 = (($80) + ($79<<2)|0);
   HEAP32[$81>>2] = $73;
   $82 = $rgroup;
   $83 = $M;
   $84 = (($83) - 2)|0;
   $85 = Math_imul($82, $84)|0;
   $86 = $i;
   $87 = (($85) + ($86))|0;
   $88 = $buf;
   $89 = (($88) + ($87<<2)|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = $rgroup;
   $92 = $M;
   $93 = Math_imul($91, $92)|0;
   $94 = $i;
   $95 = (($93) + ($94))|0;
   $96 = $xbuf1;
   $97 = (($96) + ($95<<2)|0);
   HEAP32[$97>>2] = $90;
   $98 = $i;
   $99 = (($98) + 1)|0;
   $i = $99;
  }
  $i = 0;
  while(1) {
   $100 = $i;
   $101 = $rgroup;
   $102 = ($100|0)<($101|0);
   if (!($102)) {
    break;
   }
   $103 = $xbuf0;
   $104 = HEAP32[$103>>2]|0;
   $105 = $i;
   $106 = $rgroup;
   $107 = (($105) - ($106))|0;
   $108 = $xbuf0;
   $109 = (($108) + ($107<<2)|0);
   HEAP32[$109>>2] = $104;
   $110 = $i;
   $111 = (($110) + 1)|0;
   $i = $111;
  }
  $112 = $ci;
  $113 = (($112) + 1)|0;
  $ci = $113;
  $114 = $compptr;
  $115 = (($114) + 84|0);
  $compptr = $115;
 }
 STACKTOP = sp;return;
}
function _process_data_simple_main($cinfo,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $main = 0, $rowgroups_avail = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $output_buf;
 $2 = $out_row_ctr;
 $3 = $out_rows_avail;
 $4 = $0;
 $5 = (($4) + 424|0);
 $6 = HEAP32[$5>>2]|0;
 $main = $6;
 $7 = $main;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 do {
  if (!($10)) {
   $11 = $0;
   $12 = (($11) + 428|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (($13) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $main;
   $18 = (($17) + 8|0);
   $19 = (FUNCTION_TABLE_iii[$15 & 127]($16,$18)|0);
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = $main;
    $22 = (($21) + 48|0);
    HEAP32[$22>>2] = 1;
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 $23 = $0;
 $24 = (($23) + 316|0);
 $25 = HEAP32[$24>>2]|0;
 $rowgroups_avail = $25;
 $26 = $0;
 $27 = (($26) + 432|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $0;
 $32 = $main;
 $33 = (($32) + 8|0);
 $34 = $main;
 $35 = (($34) + 52|0);
 $36 = $rowgroups_avail;
 $37 = $1;
 $38 = $2;
 $39 = $3;
 FUNCTION_TABLE_viiiiiii[$30 & 63]($31,$33,$35,$36,$37,$38,$39);
 $40 = $main;
 $41 = (($40) + 52|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $rowgroups_avail;
 $44 = ($42>>>0)>=($43>>>0);
 if (!($44)) {
  STACKTOP = sp;return;
 }
 $45 = $main;
 $46 = (($45) + 48|0);
 HEAP32[$46>>2] = 0;
 $47 = $main;
 $48 = (($47) + 52|0);
 HEAP32[$48>>2] = 0;
 STACKTOP = sp;return;
}
function _process_data_crank_post($cinfo,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $output_buf;
 $2 = $out_row_ctr;
 $3 = $out_rows_avail;
 $4 = $0;
 $5 = (($4) + 432|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = $1;
 $11 = $2;
 $12 = $3;
 FUNCTION_TABLE_viiiiiii[$8 & 63]($9,0,0,0,$10,$11,$12);
 STACKTOP = sp;return;
}
function _set_bottom_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $ci = 0, $compptr = 0, $i = 0, $iMCUheight = 0, $main = 0, $rgroup = 0, $rows_left = 0, $xbuf = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $ci = 0;
 $4 = $0;
 $5 = (($4) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $compptr = $6;
 while(1) {
  $7 = $ci;
  $8 = $0;
  $9 = (($8) + 36|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = $compptr;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $compptr;
  $16 = (($15) + 36|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = Math_imul($14, $17)|0;
  $iMCUheight = $18;
  $19 = $iMCUheight;
  $20 = $0;
  $21 = (($20) + 316|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19|0) / ($22|0))&-1;
  $rgroup = $23;
  $24 = $compptr;
  $25 = (($24) + 44|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $iMCUheight;
  $28 = (($26>>>0) % ($27>>>0))&-1;
  $rows_left = $28;
  $29 = $rows_left;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = $iMCUheight;
   $rows_left = $31;
  }
  $32 = $ci;
  $33 = ($32|0)==(0);
  if ($33) {
   $34 = $rows_left;
   $35 = (($34) - 1)|0;
   $36 = $rgroup;
   $37 = (($35|0) / ($36|0))&-1;
   $38 = (($37) + 1)|0;
   $39 = $main;
   $40 = (($39) + 72|0);
   HEAP32[$40>>2] = $38;
  }
  $41 = $ci;
  $42 = $main;
  $43 = (($42) + 64|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = $main;
  $46 = (($45) + 56|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($41<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $xbuf = $50;
  $i = 0;
  while(1) {
   $51 = $i;
   $52 = $rgroup;
   $53 = $52<<1;
   $54 = ($51|0)<($53|0);
   if (!($54)) {
    break;
   }
   $55 = $rows_left;
   $56 = (($55) - 1)|0;
   $57 = $xbuf;
   $58 = (($57) + ($56<<2)|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $rows_left;
   $61 = $i;
   $62 = (($60) + ($61))|0;
   $63 = $xbuf;
   $64 = (($63) + ($62<<2)|0);
   HEAP32[$64>>2] = $59;
   $65 = $i;
   $66 = (($65) + 1)|0;
   $i = $66;
  }
  $67 = $ci;
  $68 = (($67) + 1)|0;
  $ci = $68;
  $69 = $compptr;
  $70 = (($69) + 84|0);
  $compptr = $70;
 }
 STACKTOP = sp;return;
}
function _set_wraparound_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $M = 0, $ci = 0, $compptr = 0, $i = 0, $main = 0, $rgroup = 0, $xbuf0 = 0, $xbuf1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $4 = $0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 $M = $6;
 $ci = 0;
 $7 = $0;
 $8 = (($7) + 216|0);
 $9 = HEAP32[$8>>2]|0;
 $compptr = $9;
 while(1) {
  $10 = $ci;
  $11 = $0;
  $12 = (($11) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $compptr;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $compptr;
  $19 = (($18) + 36|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = Math_imul($17, $20)|0;
  $22 = $0;
  $23 = (($22) + 316|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($21|0) / ($24|0))&-1;
  $rgroup = $25;
  $26 = $ci;
  $27 = $main;
  $28 = (($27) + 56|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $xbuf0 = $31;
  $32 = $ci;
  $33 = $main;
  $34 = (($33) + 56|0);
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + ($32<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $xbuf1 = $38;
  $i = 0;
  while(1) {
   $39 = $i;
   $40 = $rgroup;
   $41 = ($39|0)<($40|0);
   if (!($41)) {
    break;
   }
   $42 = $rgroup;
   $43 = $M;
   $44 = (($43) + 1)|0;
   $45 = Math_imul($42, $44)|0;
   $46 = $i;
   $47 = (($45) + ($46))|0;
   $48 = $xbuf0;
   $49 = (($48) + ($47<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $i;
   $52 = $rgroup;
   $53 = (($51) - ($52))|0;
   $54 = $xbuf0;
   $55 = (($54) + ($53<<2)|0);
   HEAP32[$55>>2] = $50;
   $56 = $rgroup;
   $57 = $M;
   $58 = (($57) + 1)|0;
   $59 = Math_imul($56, $58)|0;
   $60 = $i;
   $61 = (($59) + ($60))|0;
   $62 = $xbuf1;
   $63 = (($62) + ($61<<2)|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $i;
   $66 = $rgroup;
   $67 = (($65) - ($66))|0;
   $68 = $xbuf1;
   $69 = (($68) + ($67<<2)|0);
   HEAP32[$69>>2] = $64;
   $70 = $i;
   $71 = $xbuf0;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = $rgroup;
   $75 = $M;
   $76 = (($75) + 2)|0;
   $77 = Math_imul($74, $76)|0;
   $78 = $i;
   $79 = (($77) + ($78))|0;
   $80 = $xbuf0;
   $81 = (($80) + ($79<<2)|0);
   HEAP32[$81>>2] = $73;
   $82 = $i;
   $83 = $xbuf1;
   $84 = (($83) + ($82<<2)|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $rgroup;
   $87 = $M;
   $88 = (($87) + 2)|0;
   $89 = Math_imul($86, $88)|0;
   $90 = $i;
   $91 = (($89) + ($90))|0;
   $92 = $xbuf1;
   $93 = (($92) + ($91<<2)|0);
   HEAP32[$93>>2] = $85;
   $94 = $i;
   $95 = (($94) + 1)|0;
   $i = $95;
  }
  $96 = $ci;
  $97 = (($96) + 1)|0;
  $ci = $97;
  $98 = $compptr;
  $99 = (($98) + 84|0);
  $compptr = $99;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_float($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0.0, $106 = 0.0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0.0, $124 = 0.0, $125 = 0.0, $126 = 0.0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0.0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0.0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0, $145 = 0.0, $146 = 0, $147 = 0, $148 = 0.0, $149 = 0.0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0.0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0.0, $162 = 0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0.0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0.0, $196 = 0.0, $197 = 0.0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0.0, $203 = 0.0, $204 = 0.0, $205 = 0.0;
 var $206 = 0.0, $207 = 0.0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0.0, $212 = 0.0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0.0, $225 = 0.0, $226 = 0, $227 = 0, $228 = 0.0, $229 = 0.0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0, $233 = 0.0, $234 = 0.0, $235 = 0.0, $236 = 0, $237 = 0, $238 = 0.0, $239 = 0.0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0.0, $245 = 0.0, $246 = 0, $247 = 0, $248 = 0.0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0.0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0.0, $275 = 0, $276 = 0.0, $277 = 0, $278 = 0;
 var $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0, $282 = 0, $283 = 0.0, $284 = 0, $285 = 0, $286 = 0.0, $287 = 0.0, $288 = 0, $289 = 0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0, $293 = 0.0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0.0, $302 = 0.0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0.0, $307 = 0.0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0;
 var $314 = 0, $315 = 0.0, $316 = 0.0, $317 = 0, $318 = 0, $319 = 0.0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0.0, $324 = 0, $325 = 0, $326 = 0.0, $327 = 0, $328 = 0, $329 = 0.0, $33 = 0, $330 = 0.0, $331 = 0;
 var $332 = 0, $333 = 0.0, $334 = 0, $335 = 0, $336 = 0.0, $337 = 0.0, $338 = 0.0, $339 = 0.0, $34 = 0, $340 = 0.0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0.0, $345 = 0.0, $346 = 0.0, $347 = 0.0, $348 = 0.0, $349 = 0.0, $35 = 0;
 var $350 = 0.0, $351 = 0.0, $352 = 0.0, $353 = 0.0, $354 = 0.0, $355 = 0.0, $356 = 0.0, $357 = 0.0, $358 = 0.0, $359 = 0.0, $36 = 0, $360 = 0.0, $361 = 0.0, $362 = 0.0, $363 = 0.0, $364 = 0.0, $365 = 0.0, $366 = 0.0, $367 = 0.0, $368 = 0.0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0.0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0.0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0.0, $402 = 0.0, $403 = 0.0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0.0, $414 = 0.0, $415 = 0.0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0.0, $426 = 0.0, $427 = 0.0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0.0, $438 = 0.0, $439 = 0.0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0.0, $45 = 0, $450 = 0.0, $451 = 0.0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0.0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0, $75 = 0, $76 = 0.0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0.0;
 var $95 = 0, $96 = 0, $97 = 0.0, $98 = 0.0, $99 = 0, $ctr = 0, $dcval = 0.0, $inptr = 0, $outptr = 0, $quantptr = 0, $range_limit = 0, $tmp0 = 0.0, $tmp1 = 0.0, $tmp10 = 0.0, $tmp11 = 0.0, $tmp12 = 0.0, $tmp13 = 0.0, $tmp2 = 0.0, $tmp3 = 0.0, $tmp4 = 0.0;
 var $tmp5 = 0.0, $tmp6 = 0.0, $tmp7 = 0.0, $workspace = 0, $wsptr = 0, $z10 = 0.0, $z11 = 0.0, $z12 = 0.0, $z13 = 0.0, $z5 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 384|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $inptr;
  $16 = (($15) + 16|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = $inptr;
   $21 = (($20) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $inptr;
    $26 = (($25) + 48|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27 << 16 >> 16;
    $29 = ($28|0)==(0);
    if ($29) {
     $30 = $inptr;
     $31 = (($30) + 64|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32 << 16 >> 16;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = $inptr;
      $36 = (($35) + 80|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37 << 16 >> 16;
      $39 = ($38|0)==(0);
      if ($39) {
       $40 = $inptr;
       $41 = (($40) + 96|0);
       $42 = HEAP16[$41>>1]|0;
       $43 = $42 << 16 >> 16;
       $44 = ($43|0)==(0);
       if ($44) {
        $45 = $inptr;
        $46 = (($45) + 112|0);
        $47 = HEAP16[$46>>1]|0;
        $48 = $47 << 16 >> 16;
        $49 = ($48|0)==(0);
        if ($49) {
         $50 = $inptr;
         $51 = HEAP16[$50>>1]|0;
         $52 = (+($51<<16>>16));
         $53 = $quantptr;
         $54 = +HEAPF32[$53>>2];
         $55 = $52 * $54;
         $dcval = $55;
         $56 = $dcval;
         $57 = $wsptr;
         HEAPF32[$57>>2] = $56;
         $58 = $dcval;
         $59 = $wsptr;
         $60 = (($59) + 32|0);
         HEAPF32[$60>>2] = $58;
         $61 = $dcval;
         $62 = $wsptr;
         $63 = (($62) + 64|0);
         HEAPF32[$63>>2] = $61;
         $64 = $dcval;
         $65 = $wsptr;
         $66 = (($65) + 96|0);
         HEAPF32[$66>>2] = $64;
         $67 = $dcval;
         $68 = $wsptr;
         $69 = (($68) + 128|0);
         HEAPF32[$69>>2] = $67;
         $70 = $dcval;
         $71 = $wsptr;
         $72 = (($71) + 160|0);
         HEAPF32[$72>>2] = $70;
         $73 = $dcval;
         $74 = $wsptr;
         $75 = (($74) + 192|0);
         HEAPF32[$75>>2] = $73;
         $76 = $dcval;
         $77 = $wsptr;
         $78 = (($77) + 224|0);
         HEAPF32[$78>>2] = $76;
         $79 = $inptr;
         $80 = (($79) + 2|0);
         $inptr = $80;
         $81 = $quantptr;
         $82 = (($81) + 4|0);
         $quantptr = $82;
         $83 = $wsptr;
         $84 = (($83) + 4|0);
         $wsptr = $84;
        } else {
         label = 11;
        }
       } else {
        label = 11;
       }
      } else {
       label = 11;
      }
     } else {
      label = 11;
     }
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   label = 0;
   $85 = $inptr;
   $86 = HEAP16[$85>>1]|0;
   $87 = (+($86<<16>>16));
   $88 = $quantptr;
   $89 = +HEAPF32[$88>>2];
   $90 = $87 * $89;
   $tmp0 = $90;
   $91 = $inptr;
   $92 = (($91) + 32|0);
   $93 = HEAP16[$92>>1]|0;
   $94 = (+($93<<16>>16));
   $95 = $quantptr;
   $96 = (($95) + 64|0);
   $97 = +HEAPF32[$96>>2];
   $98 = $94 * $97;
   $tmp1 = $98;
   $99 = $inptr;
   $100 = (($99) + 64|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = (+($101<<16>>16));
   $103 = $quantptr;
   $104 = (($103) + 128|0);
   $105 = +HEAPF32[$104>>2];
   $106 = $102 * $105;
   $tmp2 = $106;
   $107 = $inptr;
   $108 = (($107) + 96|0);
   $109 = HEAP16[$108>>1]|0;
   $110 = (+($109<<16>>16));
   $111 = $quantptr;
   $112 = (($111) + 192|0);
   $113 = +HEAPF32[$112>>2];
   $114 = $110 * $113;
   $tmp3 = $114;
   $115 = $tmp0;
   $116 = $tmp2;
   $117 = $115 + $116;
   $tmp10 = $117;
   $118 = $tmp0;
   $119 = $tmp2;
   $120 = $118 - $119;
   $tmp11 = $120;
   $121 = $tmp1;
   $122 = $tmp3;
   $123 = $121 + $122;
   $tmp13 = $123;
   $124 = $tmp1;
   $125 = $tmp3;
   $126 = $124 - $125;
   $127 = $126 * 1.41421353816986083984;
   $128 = $tmp13;
   $129 = $127 - $128;
   $tmp12 = $129;
   $130 = $tmp10;
   $131 = $tmp13;
   $132 = $130 + $131;
   $tmp0 = $132;
   $133 = $tmp10;
   $134 = $tmp13;
   $135 = $133 - $134;
   $tmp3 = $135;
   $136 = $tmp11;
   $137 = $tmp12;
   $138 = $136 + $137;
   $tmp1 = $138;
   $139 = $tmp11;
   $140 = $tmp12;
   $141 = $139 - $140;
   $tmp2 = $141;
   $142 = $inptr;
   $143 = (($142) + 16|0);
   $144 = HEAP16[$143>>1]|0;
   $145 = (+($144<<16>>16));
   $146 = $quantptr;
   $147 = (($146) + 32|0);
   $148 = +HEAPF32[$147>>2];
   $149 = $145 * $148;
   $tmp4 = $149;
   $150 = $inptr;
   $151 = (($150) + 48|0);
   $152 = HEAP16[$151>>1]|0;
   $153 = (+($152<<16>>16));
   $154 = $quantptr;
   $155 = (($154) + 96|0);
   $156 = +HEAPF32[$155>>2];
   $157 = $153 * $156;
   $tmp5 = $157;
   $158 = $inptr;
   $159 = (($158) + 80|0);
   $160 = HEAP16[$159>>1]|0;
   $161 = (+($160<<16>>16));
   $162 = $quantptr;
   $163 = (($162) + 160|0);
   $164 = +HEAPF32[$163>>2];
   $165 = $161 * $164;
   $tmp6 = $165;
   $166 = $inptr;
   $167 = (($166) + 112|0);
   $168 = HEAP16[$167>>1]|0;
   $169 = (+($168<<16>>16));
   $170 = $quantptr;
   $171 = (($170) + 224|0);
   $172 = +HEAPF32[$171>>2];
   $173 = $169 * $172;
   $tmp7 = $173;
   $174 = $tmp6;
   $175 = $tmp5;
   $176 = $174 + $175;
   $z13 = $176;
   $177 = $tmp6;
   $178 = $tmp5;
   $179 = $177 - $178;
   $z10 = $179;
   $180 = $tmp4;
   $181 = $tmp7;
   $182 = $180 + $181;
   $z11 = $182;
   $183 = $tmp4;
   $184 = $tmp7;
   $185 = $183 - $184;
   $z12 = $185;
   $186 = $z11;
   $187 = $z13;
   $188 = $186 + $187;
   $tmp7 = $188;
   $189 = $z11;
   $190 = $z13;
   $191 = $189 - $190;
   $192 = $191 * 1.41421353816986083984;
   $tmp11 = $192;
   $193 = $z10;
   $194 = $z12;
   $195 = $193 + $194;
   $196 = $195 * 1.84775900840759277344;
   $z5 = $196;
   $197 = $z12;
   $198 = 1.08239221572875976563 * $197;
   $199 = $z5;
   $200 = $198 - $199;
   $tmp10 = $200;
   $201 = $z10;
   $202 = -2.61312603950500488281 * $201;
   $203 = $z5;
   $204 = $202 + $203;
   $tmp12 = $204;
   $205 = $tmp12;
   $206 = $tmp7;
   $207 = $205 - $206;
   $tmp6 = $207;
   $208 = $tmp11;
   $209 = $tmp6;
   $210 = $208 - $209;
   $tmp5 = $210;
   $211 = $tmp10;
   $212 = $tmp5;
   $213 = $211 + $212;
   $tmp4 = $213;
   $214 = $tmp0;
   $215 = $tmp7;
   $216 = $214 + $215;
   $217 = $wsptr;
   HEAPF32[$217>>2] = $216;
   $218 = $tmp0;
   $219 = $tmp7;
   $220 = $218 - $219;
   $221 = $wsptr;
   $222 = (($221) + 224|0);
   HEAPF32[$222>>2] = $220;
   $223 = $tmp1;
   $224 = $tmp6;
   $225 = $223 + $224;
   $226 = $wsptr;
   $227 = (($226) + 32|0);
   HEAPF32[$227>>2] = $225;
   $228 = $tmp1;
   $229 = $tmp6;
   $230 = $228 - $229;
   $231 = $wsptr;
   $232 = (($231) + 192|0);
   HEAPF32[$232>>2] = $230;
   $233 = $tmp2;
   $234 = $tmp5;
   $235 = $233 + $234;
   $236 = $wsptr;
   $237 = (($236) + 64|0);
   HEAPF32[$237>>2] = $235;
   $238 = $tmp2;
   $239 = $tmp5;
   $240 = $238 - $239;
   $241 = $wsptr;
   $242 = (($241) + 160|0);
   HEAPF32[$242>>2] = $240;
   $243 = $tmp3;
   $244 = $tmp4;
   $245 = $243 + $244;
   $246 = $wsptr;
   $247 = (($246) + 128|0);
   HEAPF32[$247>>2] = $245;
   $248 = $tmp3;
   $249 = $tmp4;
   $250 = $248 - $249;
   $251 = $wsptr;
   $252 = (($251) + 96|0);
   HEAPF32[$252>>2] = $250;
   $253 = $inptr;
   $254 = (($253) + 2|0);
   $inptr = $254;
   $255 = $quantptr;
   $256 = (($255) + 4|0);
   $quantptr = $256;
   $257 = $wsptr;
   $258 = (($257) + 4|0);
   $wsptr = $258;
  }
  $259 = $ctr;
  $260 = (($259) + -1)|0;
  $ctr = $260;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $261 = $ctr;
  $262 = ($261|0)<(8);
  if (!($262)) {
   break;
  }
  $263 = $ctr;
  $264 = $3;
  $265 = (($264) + ($263<<2)|0);
  $266 = HEAP32[$265>>2]|0;
  $267 = $4;
  $268 = (($266) + ($267)|0);
  $outptr = $268;
  $269 = $wsptr;
  $270 = +HEAPF32[$269>>2];
  $271 = $wsptr;
  $272 = (($271) + 16|0);
  $273 = +HEAPF32[$272>>2];
  $274 = $270 + $273;
  $tmp10 = $274;
  $275 = $wsptr;
  $276 = +HEAPF32[$275>>2];
  $277 = $wsptr;
  $278 = (($277) + 16|0);
  $279 = +HEAPF32[$278>>2];
  $280 = $276 - $279;
  $tmp11 = $280;
  $281 = $wsptr;
  $282 = (($281) + 8|0);
  $283 = +HEAPF32[$282>>2];
  $284 = $wsptr;
  $285 = (($284) + 24|0);
  $286 = +HEAPF32[$285>>2];
  $287 = $283 + $286;
  $tmp13 = $287;
  $288 = $wsptr;
  $289 = (($288) + 8|0);
  $290 = +HEAPF32[$289>>2];
  $291 = $wsptr;
  $292 = (($291) + 24|0);
  $293 = +HEAPF32[$292>>2];
  $294 = $290 - $293;
  $295 = $294 * 1.41421353816986083984;
  $296 = $tmp13;
  $297 = $295 - $296;
  $tmp12 = $297;
  $298 = $tmp10;
  $299 = $tmp13;
  $300 = $298 + $299;
  $tmp0 = $300;
  $301 = $tmp10;
  $302 = $tmp13;
  $303 = $301 - $302;
  $tmp3 = $303;
  $304 = $tmp11;
  $305 = $tmp12;
  $306 = $304 + $305;
  $tmp1 = $306;
  $307 = $tmp11;
  $308 = $tmp12;
  $309 = $307 - $308;
  $tmp2 = $309;
  $310 = $wsptr;
  $311 = (($310) + 20|0);
  $312 = +HEAPF32[$311>>2];
  $313 = $wsptr;
  $314 = (($313) + 12|0);
  $315 = +HEAPF32[$314>>2];
  $316 = $312 + $315;
  $z13 = $316;
  $317 = $wsptr;
  $318 = (($317) + 20|0);
  $319 = +HEAPF32[$318>>2];
  $320 = $wsptr;
  $321 = (($320) + 12|0);
  $322 = +HEAPF32[$321>>2];
  $323 = $319 - $322;
  $z10 = $323;
  $324 = $wsptr;
  $325 = (($324) + 4|0);
  $326 = +HEAPF32[$325>>2];
  $327 = $wsptr;
  $328 = (($327) + 28|0);
  $329 = +HEAPF32[$328>>2];
  $330 = $326 + $329;
  $z11 = $330;
  $331 = $wsptr;
  $332 = (($331) + 4|0);
  $333 = +HEAPF32[$332>>2];
  $334 = $wsptr;
  $335 = (($334) + 28|0);
  $336 = +HEAPF32[$335>>2];
  $337 = $333 - $336;
  $z12 = $337;
  $338 = $z11;
  $339 = $z13;
  $340 = $338 + $339;
  $tmp7 = $340;
  $341 = $z11;
  $342 = $z13;
  $343 = $341 - $342;
  $344 = $343 * 1.41421353816986083984;
  $tmp11 = $344;
  $345 = $z10;
  $346 = $z12;
  $347 = $345 + $346;
  $348 = $347 * 1.84775900840759277344;
  $z5 = $348;
  $349 = $z12;
  $350 = 1.08239221572875976563 * $349;
  $351 = $z5;
  $352 = $350 - $351;
  $tmp10 = $352;
  $353 = $z10;
  $354 = -2.61312603950500488281 * $353;
  $355 = $z5;
  $356 = $354 + $355;
  $tmp12 = $356;
  $357 = $tmp12;
  $358 = $tmp7;
  $359 = $357 - $358;
  $tmp6 = $359;
  $360 = $tmp11;
  $361 = $tmp6;
  $362 = $360 - $361;
  $tmp5 = $362;
  $363 = $tmp10;
  $364 = $tmp5;
  $365 = $363 + $364;
  $tmp4 = $365;
  $366 = $tmp0;
  $367 = $tmp7;
  $368 = $366 + $367;
  $369 = (~~(($368)));
  $370 = (($369) + 4)|0;
  $371 = $370 >> 3;
  $372 = $371 & 1023;
  $373 = $range_limit;
  $374 = (($373) + ($372)|0);
  $375 = HEAP8[$374>>0]|0;
  $376 = $outptr;
  HEAP8[$376>>0] = $375;
  $377 = $tmp0;
  $378 = $tmp7;
  $379 = $377 - $378;
  $380 = (~~(($379)));
  $381 = (($380) + 4)|0;
  $382 = $381 >> 3;
  $383 = $382 & 1023;
  $384 = $range_limit;
  $385 = (($384) + ($383)|0);
  $386 = HEAP8[$385>>0]|0;
  $387 = $outptr;
  $388 = (($387) + 7|0);
  HEAP8[$388>>0] = $386;
  $389 = $tmp1;
  $390 = $tmp6;
  $391 = $389 + $390;
  $392 = (~~(($391)));
  $393 = (($392) + 4)|0;
  $394 = $393 >> 3;
  $395 = $394 & 1023;
  $396 = $range_limit;
  $397 = (($396) + ($395)|0);
  $398 = HEAP8[$397>>0]|0;
  $399 = $outptr;
  $400 = (($399) + 1|0);
  HEAP8[$400>>0] = $398;
  $401 = $tmp1;
  $402 = $tmp6;
  $403 = $401 - $402;
  $404 = (~~(($403)));
  $405 = (($404) + 4)|0;
  $406 = $405 >> 3;
  $407 = $406 & 1023;
  $408 = $range_limit;
  $409 = (($408) + ($407)|0);
  $410 = HEAP8[$409>>0]|0;
  $411 = $outptr;
  $412 = (($411) + 6|0);
  HEAP8[$412>>0] = $410;
  $413 = $tmp2;
  $414 = $tmp5;
  $415 = $413 + $414;
  $416 = (~~(($415)));
  $417 = (($416) + 4)|0;
  $418 = $417 >> 3;
  $419 = $418 & 1023;
  $420 = $range_limit;
  $421 = (($420) + ($419)|0);
  $422 = HEAP8[$421>>0]|0;
  $423 = $outptr;
  $424 = (($423) + 2|0);
  HEAP8[$424>>0] = $422;
  $425 = $tmp2;
  $426 = $tmp5;
  $427 = $425 - $426;
  $428 = (~~(($427)));
  $429 = (($428) + 4)|0;
  $430 = $429 >> 3;
  $431 = $430 & 1023;
  $432 = $range_limit;
  $433 = (($432) + ($431)|0);
  $434 = HEAP8[$433>>0]|0;
  $435 = $outptr;
  $436 = (($435) + 5|0);
  HEAP8[$436>>0] = $434;
  $437 = $tmp3;
  $438 = $tmp4;
  $439 = $437 + $438;
  $440 = (~~(($439)));
  $441 = (($440) + 4)|0;
  $442 = $441 >> 3;
  $443 = $442 & 1023;
  $444 = $range_limit;
  $445 = (($444) + ($443)|0);
  $446 = HEAP8[$445>>0]|0;
  $447 = $outptr;
  $448 = (($447) + 4|0);
  HEAP8[$448>>0] = $446;
  $449 = $tmp3;
  $450 = $tmp4;
  $451 = $449 - $450;
  $452 = (~~(($451)));
  $453 = (($452) + 4)|0;
  $454 = $453 >> 3;
  $455 = $454 & 1023;
  $456 = $range_limit;
  $457 = (($456) + ($455)|0);
  $458 = HEAP8[$457>>0]|0;
  $459 = $outptr;
  $460 = (($459) + 3|0);
  HEAP8[$460>>0] = $458;
  $461 = $wsptr;
  $462 = (($461) + 32|0);
  $wsptr = $462;
  $463 = $ctr;
  $464 = (($463) + 1)|0;
  $ctr = $464;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_ifast($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctr = 0;
 var $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0, $quantptr = 0, $range_limit = 0, $tmp0 = 0, $tmp1 = 0, $tmp10 = 0, $tmp11 = 0, $tmp12 = 0, $tmp13 = 0, $tmp2 = 0, $tmp3 = 0, $tmp4 = 0, $tmp5 = 0, $tmp6 = 0, $tmp7 = 0, $workspace = 0, $wsptr = 0;
 var $z10 = 0, $z11 = 0, $z12 = 0, $z13 = 0, $z5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 384|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $inptr;
  $16 = (($15) + 16|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = $inptr;
   $21 = (($20) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $inptr;
    $26 = (($25) + 48|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27 << 16 >> 16;
    $29 = ($28|0)==(0);
    if ($29) {
     $30 = $inptr;
     $31 = (($30) + 64|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32 << 16 >> 16;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = $inptr;
      $36 = (($35) + 80|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37 << 16 >> 16;
      $39 = ($38|0)==(0);
      if ($39) {
       $40 = $inptr;
       $41 = (($40) + 96|0);
       $42 = HEAP16[$41>>1]|0;
       $43 = $42 << 16 >> 16;
       $44 = ($43|0)==(0);
       if ($44) {
        $45 = $inptr;
        $46 = (($45) + 112|0);
        $47 = HEAP16[$46>>1]|0;
        $48 = $47 << 16 >> 16;
        $49 = ($48|0)==(0);
        if ($49) {
         $50 = $inptr;
         $51 = HEAP16[$50>>1]|0;
         $52 = $51 << 16 >> 16;
         $53 = $quantptr;
         $54 = HEAP32[$53>>2]|0;
         $55 = Math_imul($52, $54)|0;
         $dcval = $55;
         $56 = $dcval;
         $57 = $wsptr;
         HEAP32[$57>>2] = $56;
         $58 = $dcval;
         $59 = $wsptr;
         $60 = (($59) + 32|0);
         HEAP32[$60>>2] = $58;
         $61 = $dcval;
         $62 = $wsptr;
         $63 = (($62) + 64|0);
         HEAP32[$63>>2] = $61;
         $64 = $dcval;
         $65 = $wsptr;
         $66 = (($65) + 96|0);
         HEAP32[$66>>2] = $64;
         $67 = $dcval;
         $68 = $wsptr;
         $69 = (($68) + 128|0);
         HEAP32[$69>>2] = $67;
         $70 = $dcval;
         $71 = $wsptr;
         $72 = (($71) + 160|0);
         HEAP32[$72>>2] = $70;
         $73 = $dcval;
         $74 = $wsptr;
         $75 = (($74) + 192|0);
         HEAP32[$75>>2] = $73;
         $76 = $dcval;
         $77 = $wsptr;
         $78 = (($77) + 224|0);
         HEAP32[$78>>2] = $76;
         $79 = $inptr;
         $80 = (($79) + 2|0);
         $inptr = $80;
         $81 = $quantptr;
         $82 = (($81) + 4|0);
         $quantptr = $82;
         $83 = $wsptr;
         $84 = (($83) + 4|0);
         $wsptr = $84;
        } else {
         label = 11;
        }
       } else {
        label = 11;
       }
      } else {
       label = 11;
      }
     } else {
      label = 11;
     }
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   label = 0;
   $85 = $inptr;
   $86 = HEAP16[$85>>1]|0;
   $87 = $86 << 16 >> 16;
   $88 = $quantptr;
   $89 = HEAP32[$88>>2]|0;
   $90 = Math_imul($87, $89)|0;
   $tmp0 = $90;
   $91 = $inptr;
   $92 = (($91) + 32|0);
   $93 = HEAP16[$92>>1]|0;
   $94 = $93 << 16 >> 16;
   $95 = $quantptr;
   $96 = (($95) + 64|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = Math_imul($94, $97)|0;
   $tmp1 = $98;
   $99 = $inptr;
   $100 = (($99) + 64|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = $101 << 16 >> 16;
   $103 = $quantptr;
   $104 = (($103) + 128|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = Math_imul($102, $105)|0;
   $tmp2 = $106;
   $107 = $inptr;
   $108 = (($107) + 96|0);
   $109 = HEAP16[$108>>1]|0;
   $110 = $109 << 16 >> 16;
   $111 = $quantptr;
   $112 = (($111) + 192|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = Math_imul($110, $113)|0;
   $tmp3 = $114;
   $115 = $tmp0;
   $116 = $tmp2;
   $117 = (($115) + ($116))|0;
   $tmp10 = $117;
   $118 = $tmp0;
   $119 = $tmp2;
   $120 = (($118) - ($119))|0;
   $tmp11 = $120;
   $121 = $tmp1;
   $122 = $tmp3;
   $123 = (($121) + ($122))|0;
   $tmp13 = $123;
   $124 = $tmp1;
   $125 = $tmp3;
   $126 = (($124) - ($125))|0;
   $127 = ($126*362)|0;
   $128 = $127 >> 8;
   $129 = $tmp13;
   $130 = (($128) - ($129))|0;
   $tmp12 = $130;
   $131 = $tmp10;
   $132 = $tmp13;
   $133 = (($131) + ($132))|0;
   $tmp0 = $133;
   $134 = $tmp10;
   $135 = $tmp13;
   $136 = (($134) - ($135))|0;
   $tmp3 = $136;
   $137 = $tmp11;
   $138 = $tmp12;
   $139 = (($137) + ($138))|0;
   $tmp1 = $139;
   $140 = $tmp11;
   $141 = $tmp12;
   $142 = (($140) - ($141))|0;
   $tmp2 = $142;
   $143 = $inptr;
   $144 = (($143) + 16|0);
   $145 = HEAP16[$144>>1]|0;
   $146 = $145 << 16 >> 16;
   $147 = $quantptr;
   $148 = (($147) + 32|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = Math_imul($146, $149)|0;
   $tmp4 = $150;
   $151 = $inptr;
   $152 = (($151) + 48|0);
   $153 = HEAP16[$152>>1]|0;
   $154 = $153 << 16 >> 16;
   $155 = $quantptr;
   $156 = (($155) + 96|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = Math_imul($154, $157)|0;
   $tmp5 = $158;
   $159 = $inptr;
   $160 = (($159) + 80|0);
   $161 = HEAP16[$160>>1]|0;
   $162 = $161 << 16 >> 16;
   $163 = $quantptr;
   $164 = (($163) + 160|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = Math_imul($162, $165)|0;
   $tmp6 = $166;
   $167 = $inptr;
   $168 = (($167) + 112|0);
   $169 = HEAP16[$168>>1]|0;
   $170 = $169 << 16 >> 16;
   $171 = $quantptr;
   $172 = (($171) + 224|0);
   $173 = HEAP32[$172>>2]|0;
   $174 = Math_imul($170, $173)|0;
   $tmp7 = $174;
   $175 = $tmp6;
   $176 = $tmp5;
   $177 = (($175) + ($176))|0;
   $z13 = $177;
   $178 = $tmp6;
   $179 = $tmp5;
   $180 = (($178) - ($179))|0;
   $z10 = $180;
   $181 = $tmp4;
   $182 = $tmp7;
   $183 = (($181) + ($182))|0;
   $z11 = $183;
   $184 = $tmp4;
   $185 = $tmp7;
   $186 = (($184) - ($185))|0;
   $z12 = $186;
   $187 = $z11;
   $188 = $z13;
   $189 = (($187) + ($188))|0;
   $tmp7 = $189;
   $190 = $z11;
   $191 = $z13;
   $192 = (($190) - ($191))|0;
   $193 = ($192*362)|0;
   $194 = $193 >> 8;
   $tmp11 = $194;
   $195 = $z10;
   $196 = $z12;
   $197 = (($195) + ($196))|0;
   $198 = ($197*473)|0;
   $199 = $198 >> 8;
   $z5 = $199;
   $200 = $z12;
   $201 = ($200*277)|0;
   $202 = $201 >> 8;
   $203 = $z5;
   $204 = (($202) - ($203))|0;
   $tmp10 = $204;
   $205 = $z10;
   $206 = Math_imul($205, -669)|0;
   $207 = $206 >> 8;
   $208 = $z5;
   $209 = (($207) + ($208))|0;
   $tmp12 = $209;
   $210 = $tmp12;
   $211 = $tmp7;
   $212 = (($210) - ($211))|0;
   $tmp6 = $212;
   $213 = $tmp11;
   $214 = $tmp6;
   $215 = (($213) - ($214))|0;
   $tmp5 = $215;
   $216 = $tmp10;
   $217 = $tmp5;
   $218 = (($216) + ($217))|0;
   $tmp4 = $218;
   $219 = $tmp0;
   $220 = $tmp7;
   $221 = (($219) + ($220))|0;
   $222 = $wsptr;
   HEAP32[$222>>2] = $221;
   $223 = $tmp0;
   $224 = $tmp7;
   $225 = (($223) - ($224))|0;
   $226 = $wsptr;
   $227 = (($226) + 224|0);
   HEAP32[$227>>2] = $225;
   $228 = $tmp1;
   $229 = $tmp6;
   $230 = (($228) + ($229))|0;
   $231 = $wsptr;
   $232 = (($231) + 32|0);
   HEAP32[$232>>2] = $230;
   $233 = $tmp1;
   $234 = $tmp6;
   $235 = (($233) - ($234))|0;
   $236 = $wsptr;
   $237 = (($236) + 192|0);
   HEAP32[$237>>2] = $235;
   $238 = $tmp2;
   $239 = $tmp5;
   $240 = (($238) + ($239))|0;
   $241 = $wsptr;
   $242 = (($241) + 64|0);
   HEAP32[$242>>2] = $240;
   $243 = $tmp2;
   $244 = $tmp5;
   $245 = (($243) - ($244))|0;
   $246 = $wsptr;
   $247 = (($246) + 160|0);
   HEAP32[$247>>2] = $245;
   $248 = $tmp3;
   $249 = $tmp4;
   $250 = (($248) + ($249))|0;
   $251 = $wsptr;
   $252 = (($251) + 128|0);
   HEAP32[$252>>2] = $250;
   $253 = $tmp3;
   $254 = $tmp4;
   $255 = (($253) - ($254))|0;
   $256 = $wsptr;
   $257 = (($256) + 96|0);
   HEAP32[$257>>2] = $255;
   $258 = $inptr;
   $259 = (($258) + 2|0);
   $inptr = $259;
   $260 = $quantptr;
   $261 = (($260) + 4|0);
   $quantptr = $261;
   $262 = $wsptr;
   $263 = (($262) + 4|0);
   $wsptr = $263;
  }
  $264 = $ctr;
  $265 = (($264) + -1)|0;
  $ctr = $265;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $266 = $ctr;
  $267 = ($266|0)<(8);
  if (!($267)) {
   break;
  }
  $268 = $ctr;
  $269 = $3;
  $270 = (($269) + ($268<<2)|0);
  $271 = HEAP32[$270>>2]|0;
  $272 = $4;
  $273 = (($271) + ($272)|0);
  $outptr = $273;
  $274 = $wsptr;
  $275 = (($274) + 4|0);
  $276 = HEAP32[$275>>2]|0;
  $277 = ($276|0)==(0);
  if ($277) {
   $278 = $wsptr;
   $279 = (($278) + 8|0);
   $280 = HEAP32[$279>>2]|0;
   $281 = ($280|0)==(0);
   if ($281) {
    $282 = $wsptr;
    $283 = (($282) + 12|0);
    $284 = HEAP32[$283>>2]|0;
    $285 = ($284|0)==(0);
    if ($285) {
     $286 = $wsptr;
     $287 = (($286) + 16|0);
     $288 = HEAP32[$287>>2]|0;
     $289 = ($288|0)==(0);
     if ($289) {
      $290 = $wsptr;
      $291 = (($290) + 20|0);
      $292 = HEAP32[$291>>2]|0;
      $293 = ($292|0)==(0);
      if ($293) {
       $294 = $wsptr;
       $295 = (($294) + 24|0);
       $296 = HEAP32[$295>>2]|0;
       $297 = ($296|0)==(0);
       if ($297) {
        $298 = $wsptr;
        $299 = (($298) + 28|0);
        $300 = HEAP32[$299>>2]|0;
        $301 = ($300|0)==(0);
        if ($301) {
         $302 = $wsptr;
         $303 = HEAP32[$302>>2]|0;
         $304 = $303 >> 5;
         $305 = $304 & 1023;
         $306 = $range_limit;
         $307 = (($306) + ($305)|0);
         $308 = HEAP8[$307>>0]|0;
         $dcval1 = $308;
         $309 = $dcval1;
         $310 = $outptr;
         HEAP8[$310>>0] = $309;
         $311 = $dcval1;
         $312 = $outptr;
         $313 = (($312) + 1|0);
         HEAP8[$313>>0] = $311;
         $314 = $dcval1;
         $315 = $outptr;
         $316 = (($315) + 2|0);
         HEAP8[$316>>0] = $314;
         $317 = $dcval1;
         $318 = $outptr;
         $319 = (($318) + 3|0);
         HEAP8[$319>>0] = $317;
         $320 = $dcval1;
         $321 = $outptr;
         $322 = (($321) + 4|0);
         HEAP8[$322>>0] = $320;
         $323 = $dcval1;
         $324 = $outptr;
         $325 = (($324) + 5|0);
         HEAP8[$325>>0] = $323;
         $326 = $dcval1;
         $327 = $outptr;
         $328 = (($327) + 6|0);
         HEAP8[$328>>0] = $326;
         $329 = $dcval1;
         $330 = $outptr;
         $331 = (($330) + 7|0);
         HEAP8[$331>>0] = $329;
         $332 = $wsptr;
         $333 = (($332) + 32|0);
         $wsptr = $333;
        } else {
         label = 23;
        }
       } else {
        label = 23;
       }
      } else {
       label = 23;
      }
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $334 = $wsptr;
   $335 = HEAP32[$334>>2]|0;
   $336 = $wsptr;
   $337 = (($336) + 16|0);
   $338 = HEAP32[$337>>2]|0;
   $339 = (($335) + ($338))|0;
   $tmp10 = $339;
   $340 = $wsptr;
   $341 = HEAP32[$340>>2]|0;
   $342 = $wsptr;
   $343 = (($342) + 16|0);
   $344 = HEAP32[$343>>2]|0;
   $345 = (($341) - ($344))|0;
   $tmp11 = $345;
   $346 = $wsptr;
   $347 = (($346) + 8|0);
   $348 = HEAP32[$347>>2]|0;
   $349 = $wsptr;
   $350 = (($349) + 24|0);
   $351 = HEAP32[$350>>2]|0;
   $352 = (($348) + ($351))|0;
   $tmp13 = $352;
   $353 = $wsptr;
   $354 = (($353) + 8|0);
   $355 = HEAP32[$354>>2]|0;
   $356 = $wsptr;
   $357 = (($356) + 24|0);
   $358 = HEAP32[$357>>2]|0;
   $359 = (($355) - ($358))|0;
   $360 = ($359*362)|0;
   $361 = $360 >> 8;
   $362 = $tmp13;
   $363 = (($361) - ($362))|0;
   $tmp12 = $363;
   $364 = $tmp10;
   $365 = $tmp13;
   $366 = (($364) + ($365))|0;
   $tmp0 = $366;
   $367 = $tmp10;
   $368 = $tmp13;
   $369 = (($367) - ($368))|0;
   $tmp3 = $369;
   $370 = $tmp11;
   $371 = $tmp12;
   $372 = (($370) + ($371))|0;
   $tmp1 = $372;
   $373 = $tmp11;
   $374 = $tmp12;
   $375 = (($373) - ($374))|0;
   $tmp2 = $375;
   $376 = $wsptr;
   $377 = (($376) + 20|0);
   $378 = HEAP32[$377>>2]|0;
   $379 = $wsptr;
   $380 = (($379) + 12|0);
   $381 = HEAP32[$380>>2]|0;
   $382 = (($378) + ($381))|0;
   $z13 = $382;
   $383 = $wsptr;
   $384 = (($383) + 20|0);
   $385 = HEAP32[$384>>2]|0;
   $386 = $wsptr;
   $387 = (($386) + 12|0);
   $388 = HEAP32[$387>>2]|0;
   $389 = (($385) - ($388))|0;
   $z10 = $389;
   $390 = $wsptr;
   $391 = (($390) + 4|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = $wsptr;
   $394 = (($393) + 28|0);
   $395 = HEAP32[$394>>2]|0;
   $396 = (($392) + ($395))|0;
   $z11 = $396;
   $397 = $wsptr;
   $398 = (($397) + 4|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = $wsptr;
   $401 = (($400) + 28|0);
   $402 = HEAP32[$401>>2]|0;
   $403 = (($399) - ($402))|0;
   $z12 = $403;
   $404 = $z11;
   $405 = $z13;
   $406 = (($404) + ($405))|0;
   $tmp7 = $406;
   $407 = $z11;
   $408 = $z13;
   $409 = (($407) - ($408))|0;
   $410 = ($409*362)|0;
   $411 = $410 >> 8;
   $tmp11 = $411;
   $412 = $z10;
   $413 = $z12;
   $414 = (($412) + ($413))|0;
   $415 = ($414*473)|0;
   $416 = $415 >> 8;
   $z5 = $416;
   $417 = $z12;
   $418 = ($417*277)|0;
   $419 = $418 >> 8;
   $420 = $z5;
   $421 = (($419) - ($420))|0;
   $tmp10 = $421;
   $422 = $z10;
   $423 = Math_imul($422, -669)|0;
   $424 = $423 >> 8;
   $425 = $z5;
   $426 = (($424) + ($425))|0;
   $tmp12 = $426;
   $427 = $tmp12;
   $428 = $tmp7;
   $429 = (($427) - ($428))|0;
   $tmp6 = $429;
   $430 = $tmp11;
   $431 = $tmp6;
   $432 = (($430) - ($431))|0;
   $tmp5 = $432;
   $433 = $tmp10;
   $434 = $tmp5;
   $435 = (($433) + ($434))|0;
   $tmp4 = $435;
   $436 = $tmp0;
   $437 = $tmp7;
   $438 = (($436) + ($437))|0;
   $439 = $438 >> 5;
   $440 = $439 & 1023;
   $441 = $range_limit;
   $442 = (($441) + ($440)|0);
   $443 = HEAP8[$442>>0]|0;
   $444 = $outptr;
   HEAP8[$444>>0] = $443;
   $445 = $tmp0;
   $446 = $tmp7;
   $447 = (($445) - ($446))|0;
   $448 = $447 >> 5;
   $449 = $448 & 1023;
   $450 = $range_limit;
   $451 = (($450) + ($449)|0);
   $452 = HEAP8[$451>>0]|0;
   $453 = $outptr;
   $454 = (($453) + 7|0);
   HEAP8[$454>>0] = $452;
   $455 = $tmp1;
   $456 = $tmp6;
   $457 = (($455) + ($456))|0;
   $458 = $457 >> 5;
   $459 = $458 & 1023;
   $460 = $range_limit;
   $461 = (($460) + ($459)|0);
   $462 = HEAP8[$461>>0]|0;
   $463 = $outptr;
   $464 = (($463) + 1|0);
   HEAP8[$464>>0] = $462;
   $465 = $tmp1;
   $466 = $tmp6;
   $467 = (($465) - ($466))|0;
   $468 = $467 >> 5;
   $469 = $468 & 1023;
   $470 = $range_limit;
   $471 = (($470) + ($469)|0);
   $472 = HEAP8[$471>>0]|0;
   $473 = $outptr;
   $474 = (($473) + 6|0);
   HEAP8[$474>>0] = $472;
   $475 = $tmp2;
   $476 = $tmp5;
   $477 = (($475) + ($476))|0;
   $478 = $477 >> 5;
   $479 = $478 & 1023;
   $480 = $range_limit;
   $481 = (($480) + ($479)|0);
   $482 = HEAP8[$481>>0]|0;
   $483 = $outptr;
   $484 = (($483) + 2|0);
   HEAP8[$484>>0] = $482;
   $485 = $tmp2;
   $486 = $tmp5;
   $487 = (($485) - ($486))|0;
   $488 = $487 >> 5;
   $489 = $488 & 1023;
   $490 = $range_limit;
   $491 = (($490) + ($489)|0);
   $492 = HEAP8[$491>>0]|0;
   $493 = $outptr;
   $494 = (($493) + 5|0);
   HEAP8[$494>>0] = $492;
   $495 = $tmp3;
   $496 = $tmp4;
   $497 = (($495) + ($496))|0;
   $498 = $497 >> 5;
   $499 = $498 & 1023;
   $500 = $range_limit;
   $501 = (($500) + ($499)|0);
   $502 = HEAP8[$501>>0]|0;
   $503 = $outptr;
   $504 = (($503) + 4|0);
   HEAP8[$504>>0] = $502;
   $505 = $tmp3;
   $506 = $tmp4;
   $507 = (($505) - ($506))|0;
   $508 = $507 >> 5;
   $509 = $508 & 1023;
   $510 = $range_limit;
   $511 = (($510) + ($509)|0);
   $512 = HEAP8[$511>>0]|0;
   $513 = $outptr;
   $514 = (($513) + 3|0);
   HEAP8[$514>>0] = $512;
   $515 = $wsptr;
   $516 = (($515) + 32|0);
   $wsptr = $516;
  }
  $517 = $ctr;
  $518 = (($517) + 1)|0;
  $ctr = $518;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_islow($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctr = 0, $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0;
 var $quantptr = 0, $range_limit = 0, $tmp0 = 0, $tmp1 = 0, $tmp10 = 0, $tmp11 = 0, $tmp12 = 0, $tmp13 = 0, $tmp2 = 0, $tmp3 = 0, $workspace = 0, $wsptr = 0, $z1 = 0, $z2 = 0, $z3 = 0, $z4 = 0, $z5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $inptr;
  $16 = (($15) + 16|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = $inptr;
   $21 = (($20) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $inptr;
    $26 = (($25) + 48|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27 << 16 >> 16;
    $29 = ($28|0)==(0);
    if ($29) {
     $30 = $inptr;
     $31 = (($30) + 64|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32 << 16 >> 16;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = $inptr;
      $36 = (($35) + 80|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37 << 16 >> 16;
      $39 = ($38|0)==(0);
      if ($39) {
       $40 = $inptr;
       $41 = (($40) + 96|0);
       $42 = HEAP16[$41>>1]|0;
       $43 = $42 << 16 >> 16;
       $44 = ($43|0)==(0);
       if ($44) {
        $45 = $inptr;
        $46 = (($45) + 112|0);
        $47 = HEAP16[$46>>1]|0;
        $48 = $47 << 16 >> 16;
        $49 = ($48|0)==(0);
        if ($49) {
         $50 = $inptr;
         $51 = HEAP16[$50>>1]|0;
         $52 = $51 << 16 >> 16;
         $53 = $quantptr;
         $54 = HEAP32[$53>>2]|0;
         $55 = Math_imul($52, $54)|0;
         $56 = $55 << 2;
         $dcval = $56;
         $57 = $dcval;
         $58 = $wsptr;
         HEAP32[$58>>2] = $57;
         $59 = $dcval;
         $60 = $wsptr;
         $61 = (($60) + 32|0);
         HEAP32[$61>>2] = $59;
         $62 = $dcval;
         $63 = $wsptr;
         $64 = (($63) + 64|0);
         HEAP32[$64>>2] = $62;
         $65 = $dcval;
         $66 = $wsptr;
         $67 = (($66) + 96|0);
         HEAP32[$67>>2] = $65;
         $68 = $dcval;
         $69 = $wsptr;
         $70 = (($69) + 128|0);
         HEAP32[$70>>2] = $68;
         $71 = $dcval;
         $72 = $wsptr;
         $73 = (($72) + 160|0);
         HEAP32[$73>>2] = $71;
         $74 = $dcval;
         $75 = $wsptr;
         $76 = (($75) + 192|0);
         HEAP32[$76>>2] = $74;
         $77 = $dcval;
         $78 = $wsptr;
         $79 = (($78) + 224|0);
         HEAP32[$79>>2] = $77;
         $80 = $inptr;
         $81 = (($80) + 2|0);
         $inptr = $81;
         $82 = $quantptr;
         $83 = (($82) + 4|0);
         $quantptr = $83;
         $84 = $wsptr;
         $85 = (($84) + 4|0);
         $wsptr = $85;
        } else {
         label = 11;
        }
       } else {
        label = 11;
       }
      } else {
       label = 11;
      }
     } else {
      label = 11;
     }
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   label = 0;
   $86 = $inptr;
   $87 = (($86) + 32|0);
   $88 = HEAP16[$87>>1]|0;
   $89 = $88 << 16 >> 16;
   $90 = $quantptr;
   $91 = (($90) + 64|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = Math_imul($89, $92)|0;
   $z2 = $93;
   $94 = $inptr;
   $95 = (($94) + 96|0);
   $96 = HEAP16[$95>>1]|0;
   $97 = $96 << 16 >> 16;
   $98 = $quantptr;
   $99 = (($98) + 192|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = Math_imul($97, $100)|0;
   $z3 = $101;
   $102 = $z2;
   $103 = $z3;
   $104 = (($102) + ($103))|0;
   $105 = ($104*4433)|0;
   $z1 = $105;
   $106 = $z1;
   $107 = $z3;
   $108 = Math_imul($107, -15137)|0;
   $109 = (($106) + ($108))|0;
   $tmp2 = $109;
   $110 = $z1;
   $111 = $z2;
   $112 = ($111*6270)|0;
   $113 = (($110) + ($112))|0;
   $tmp3 = $113;
   $114 = $inptr;
   $115 = HEAP16[$114>>1]|0;
   $116 = $115 << 16 >> 16;
   $117 = $quantptr;
   $118 = HEAP32[$117>>2]|0;
   $119 = Math_imul($116, $118)|0;
   $z2 = $119;
   $120 = $inptr;
   $121 = (($120) + 64|0);
   $122 = HEAP16[$121>>1]|0;
   $123 = $122 << 16 >> 16;
   $124 = $quantptr;
   $125 = (($124) + 128|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = Math_imul($123, $126)|0;
   $z3 = $127;
   $128 = $z2;
   $129 = $z3;
   $130 = (($128) + ($129))|0;
   $131 = $130 << 13;
   $tmp0 = $131;
   $132 = $z2;
   $133 = $z3;
   $134 = (($132) - ($133))|0;
   $135 = $134 << 13;
   $tmp1 = $135;
   $136 = $tmp0;
   $137 = $tmp3;
   $138 = (($136) + ($137))|0;
   $tmp10 = $138;
   $139 = $tmp0;
   $140 = $tmp3;
   $141 = (($139) - ($140))|0;
   $tmp13 = $141;
   $142 = $tmp1;
   $143 = $tmp2;
   $144 = (($142) + ($143))|0;
   $tmp11 = $144;
   $145 = $tmp1;
   $146 = $tmp2;
   $147 = (($145) - ($146))|0;
   $tmp12 = $147;
   $148 = $inptr;
   $149 = (($148) + 112|0);
   $150 = HEAP16[$149>>1]|0;
   $151 = $150 << 16 >> 16;
   $152 = $quantptr;
   $153 = (($152) + 224|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = Math_imul($151, $154)|0;
   $tmp0 = $155;
   $156 = $inptr;
   $157 = (($156) + 80|0);
   $158 = HEAP16[$157>>1]|0;
   $159 = $158 << 16 >> 16;
   $160 = $quantptr;
   $161 = (($160) + 160|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = Math_imul($159, $162)|0;
   $tmp1 = $163;
   $164 = $inptr;
   $165 = (($164) + 48|0);
   $166 = HEAP16[$165>>1]|0;
   $167 = $166 << 16 >> 16;
   $168 = $quantptr;
   $169 = (($168) + 96|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = Math_imul($167, $170)|0;
   $tmp2 = $171;
   $172 = $inptr;
   $173 = (($172) + 16|0);
   $174 = HEAP16[$173>>1]|0;
   $175 = $174 << 16 >> 16;
   $176 = $quantptr;
   $177 = (($176) + 32|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = Math_imul($175, $178)|0;
   $tmp3 = $179;
   $180 = $tmp0;
   $181 = $tmp3;
   $182 = (($180) + ($181))|0;
   $z1 = $182;
   $183 = $tmp1;
   $184 = $tmp2;
   $185 = (($183) + ($184))|0;
   $z2 = $185;
   $186 = $tmp0;
   $187 = $tmp2;
   $188 = (($186) + ($187))|0;
   $z3 = $188;
   $189 = $tmp1;
   $190 = $tmp3;
   $191 = (($189) + ($190))|0;
   $z4 = $191;
   $192 = $z3;
   $193 = $z4;
   $194 = (($192) + ($193))|0;
   $195 = ($194*9633)|0;
   $z5 = $195;
   $196 = $tmp0;
   $197 = ($196*2446)|0;
   $tmp0 = $197;
   $198 = $tmp1;
   $199 = ($198*16819)|0;
   $tmp1 = $199;
   $200 = $tmp2;
   $201 = ($200*25172)|0;
   $tmp2 = $201;
   $202 = $tmp3;
   $203 = ($202*12299)|0;
   $tmp3 = $203;
   $204 = $z1;
   $205 = Math_imul($204, -7373)|0;
   $z1 = $205;
   $206 = $z2;
   $207 = Math_imul($206, -20995)|0;
   $z2 = $207;
   $208 = $z3;
   $209 = Math_imul($208, -16069)|0;
   $z3 = $209;
   $210 = $z4;
   $211 = Math_imul($210, -3196)|0;
   $z4 = $211;
   $212 = $z5;
   $213 = $z3;
   $214 = (($213) + ($212))|0;
   $z3 = $214;
   $215 = $z5;
   $216 = $z4;
   $217 = (($216) + ($215))|0;
   $z4 = $217;
   $218 = $z1;
   $219 = $z3;
   $220 = (($218) + ($219))|0;
   $221 = $tmp0;
   $222 = (($221) + ($220))|0;
   $tmp0 = $222;
   $223 = $z2;
   $224 = $z4;
   $225 = (($223) + ($224))|0;
   $226 = $tmp1;
   $227 = (($226) + ($225))|0;
   $tmp1 = $227;
   $228 = $z2;
   $229 = $z3;
   $230 = (($228) + ($229))|0;
   $231 = $tmp2;
   $232 = (($231) + ($230))|0;
   $tmp2 = $232;
   $233 = $z1;
   $234 = $z4;
   $235 = (($233) + ($234))|0;
   $236 = $tmp3;
   $237 = (($236) + ($235))|0;
   $tmp3 = $237;
   $238 = $tmp10;
   $239 = $tmp3;
   $240 = (($238) + ($239))|0;
   $241 = (($240) + 1024)|0;
   $242 = $241 >> 11;
   $243 = $wsptr;
   HEAP32[$243>>2] = $242;
   $244 = $tmp10;
   $245 = $tmp3;
   $246 = (($244) - ($245))|0;
   $247 = (($246) + 1024)|0;
   $248 = $247 >> 11;
   $249 = $wsptr;
   $250 = (($249) + 224|0);
   HEAP32[$250>>2] = $248;
   $251 = $tmp11;
   $252 = $tmp2;
   $253 = (($251) + ($252))|0;
   $254 = (($253) + 1024)|0;
   $255 = $254 >> 11;
   $256 = $wsptr;
   $257 = (($256) + 32|0);
   HEAP32[$257>>2] = $255;
   $258 = $tmp11;
   $259 = $tmp2;
   $260 = (($258) - ($259))|0;
   $261 = (($260) + 1024)|0;
   $262 = $261 >> 11;
   $263 = $wsptr;
   $264 = (($263) + 192|0);
   HEAP32[$264>>2] = $262;
   $265 = $tmp12;
   $266 = $tmp1;
   $267 = (($265) + ($266))|0;
   $268 = (($267) + 1024)|0;
   $269 = $268 >> 11;
   $270 = $wsptr;
   $271 = (($270) + 64|0);
   HEAP32[$271>>2] = $269;
   $272 = $tmp12;
   $273 = $tmp1;
   $274 = (($272) - ($273))|0;
   $275 = (($274) + 1024)|0;
   $276 = $275 >> 11;
   $277 = $wsptr;
   $278 = (($277) + 160|0);
   HEAP32[$278>>2] = $276;
   $279 = $tmp13;
   $280 = $tmp0;
   $281 = (($279) + ($280))|0;
   $282 = (($281) + 1024)|0;
   $283 = $282 >> 11;
   $284 = $wsptr;
   $285 = (($284) + 96|0);
   HEAP32[$285>>2] = $283;
   $286 = $tmp13;
   $287 = $tmp0;
   $288 = (($286) - ($287))|0;
   $289 = (($288) + 1024)|0;
   $290 = $289 >> 11;
   $291 = $wsptr;
   $292 = (($291) + 128|0);
   HEAP32[$292>>2] = $290;
   $293 = $inptr;
   $294 = (($293) + 2|0);
   $inptr = $294;
   $295 = $quantptr;
   $296 = (($295) + 4|0);
   $quantptr = $296;
   $297 = $wsptr;
   $298 = (($297) + 4|0);
   $wsptr = $298;
  }
  $299 = $ctr;
  $300 = (($299) + -1)|0;
  $ctr = $300;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $301 = $ctr;
  $302 = ($301|0)<(8);
  if (!($302)) {
   break;
  }
  $303 = $ctr;
  $304 = $3;
  $305 = (($304) + ($303<<2)|0);
  $306 = HEAP32[$305>>2]|0;
  $307 = $4;
  $308 = (($306) + ($307)|0);
  $outptr = $308;
  $309 = $wsptr;
  $310 = (($309) + 4|0);
  $311 = HEAP32[$310>>2]|0;
  $312 = ($311|0)==(0);
  if ($312) {
   $313 = $wsptr;
   $314 = (($313) + 8|0);
   $315 = HEAP32[$314>>2]|0;
   $316 = ($315|0)==(0);
   if ($316) {
    $317 = $wsptr;
    $318 = (($317) + 12|0);
    $319 = HEAP32[$318>>2]|0;
    $320 = ($319|0)==(0);
    if ($320) {
     $321 = $wsptr;
     $322 = (($321) + 16|0);
     $323 = HEAP32[$322>>2]|0;
     $324 = ($323|0)==(0);
     if ($324) {
      $325 = $wsptr;
      $326 = (($325) + 20|0);
      $327 = HEAP32[$326>>2]|0;
      $328 = ($327|0)==(0);
      if ($328) {
       $329 = $wsptr;
       $330 = (($329) + 24|0);
       $331 = HEAP32[$330>>2]|0;
       $332 = ($331|0)==(0);
       if ($332) {
        $333 = $wsptr;
        $334 = (($333) + 28|0);
        $335 = HEAP32[$334>>2]|0;
        $336 = ($335|0)==(0);
        if ($336) {
         $337 = $wsptr;
         $338 = HEAP32[$337>>2]|0;
         $339 = (($338) + 16)|0;
         $340 = $339 >> 5;
         $341 = $340 & 1023;
         $342 = $range_limit;
         $343 = (($342) + ($341)|0);
         $344 = HEAP8[$343>>0]|0;
         $dcval1 = $344;
         $345 = $dcval1;
         $346 = $outptr;
         HEAP8[$346>>0] = $345;
         $347 = $dcval1;
         $348 = $outptr;
         $349 = (($348) + 1|0);
         HEAP8[$349>>0] = $347;
         $350 = $dcval1;
         $351 = $outptr;
         $352 = (($351) + 2|0);
         HEAP8[$352>>0] = $350;
         $353 = $dcval1;
         $354 = $outptr;
         $355 = (($354) + 3|0);
         HEAP8[$355>>0] = $353;
         $356 = $dcval1;
         $357 = $outptr;
         $358 = (($357) + 4|0);
         HEAP8[$358>>0] = $356;
         $359 = $dcval1;
         $360 = $outptr;
         $361 = (($360) + 5|0);
         HEAP8[$361>>0] = $359;
         $362 = $dcval1;
         $363 = $outptr;
         $364 = (($363) + 6|0);
         HEAP8[$364>>0] = $362;
         $365 = $dcval1;
         $366 = $outptr;
         $367 = (($366) + 7|0);
         HEAP8[$367>>0] = $365;
         $368 = $wsptr;
         $369 = (($368) + 32|0);
         $wsptr = $369;
        } else {
         label = 23;
        }
       } else {
        label = 23;
       }
      } else {
       label = 23;
      }
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $370 = $wsptr;
   $371 = (($370) + 8|0);
   $372 = HEAP32[$371>>2]|0;
   $z2 = $372;
   $373 = $wsptr;
   $374 = (($373) + 24|0);
   $375 = HEAP32[$374>>2]|0;
   $z3 = $375;
   $376 = $z2;
   $377 = $z3;
   $378 = (($376) + ($377))|0;
   $379 = ($378*4433)|0;
   $z1 = $379;
   $380 = $z1;
   $381 = $z3;
   $382 = Math_imul($381, -15137)|0;
   $383 = (($380) + ($382))|0;
   $tmp2 = $383;
   $384 = $z1;
   $385 = $z2;
   $386 = ($385*6270)|0;
   $387 = (($384) + ($386))|0;
   $tmp3 = $387;
   $388 = $wsptr;
   $389 = HEAP32[$388>>2]|0;
   $390 = $wsptr;
   $391 = (($390) + 16|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = (($389) + ($392))|0;
   $394 = $393 << 13;
   $tmp0 = $394;
   $395 = $wsptr;
   $396 = HEAP32[$395>>2]|0;
   $397 = $wsptr;
   $398 = (($397) + 16|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = (($396) - ($399))|0;
   $401 = $400 << 13;
   $tmp1 = $401;
   $402 = $tmp0;
   $403 = $tmp3;
   $404 = (($402) + ($403))|0;
   $tmp10 = $404;
   $405 = $tmp0;
   $406 = $tmp3;
   $407 = (($405) - ($406))|0;
   $tmp13 = $407;
   $408 = $tmp1;
   $409 = $tmp2;
   $410 = (($408) + ($409))|0;
   $tmp11 = $410;
   $411 = $tmp1;
   $412 = $tmp2;
   $413 = (($411) - ($412))|0;
   $tmp12 = $413;
   $414 = $wsptr;
   $415 = (($414) + 28|0);
   $416 = HEAP32[$415>>2]|0;
   $tmp0 = $416;
   $417 = $wsptr;
   $418 = (($417) + 20|0);
   $419 = HEAP32[$418>>2]|0;
   $tmp1 = $419;
   $420 = $wsptr;
   $421 = (($420) + 12|0);
   $422 = HEAP32[$421>>2]|0;
   $tmp2 = $422;
   $423 = $wsptr;
   $424 = (($423) + 4|0);
   $425 = HEAP32[$424>>2]|0;
   $tmp3 = $425;
   $426 = $tmp0;
   $427 = $tmp3;
   $428 = (($426) + ($427))|0;
   $z1 = $428;
   $429 = $tmp1;
   $430 = $tmp2;
   $431 = (($429) + ($430))|0;
   $z2 = $431;
   $432 = $tmp0;
   $433 = $tmp2;
   $434 = (($432) + ($433))|0;
   $z3 = $434;
   $435 = $tmp1;
   $436 = $tmp3;
   $437 = (($435) + ($436))|0;
   $z4 = $437;
   $438 = $z3;
   $439 = $z4;
   $440 = (($438) + ($439))|0;
   $441 = ($440*9633)|0;
   $z5 = $441;
   $442 = $tmp0;
   $443 = ($442*2446)|0;
   $tmp0 = $443;
   $444 = $tmp1;
   $445 = ($444*16819)|0;
   $tmp1 = $445;
   $446 = $tmp2;
   $447 = ($446*25172)|0;
   $tmp2 = $447;
   $448 = $tmp3;
   $449 = ($448*12299)|0;
   $tmp3 = $449;
   $450 = $z1;
   $451 = Math_imul($450, -7373)|0;
   $z1 = $451;
   $452 = $z2;
   $453 = Math_imul($452, -20995)|0;
   $z2 = $453;
   $454 = $z3;
   $455 = Math_imul($454, -16069)|0;
   $z3 = $455;
   $456 = $z4;
   $457 = Math_imul($456, -3196)|0;
   $z4 = $457;
   $458 = $z5;
   $459 = $z3;
   $460 = (($459) + ($458))|0;
   $z3 = $460;
   $461 = $z5;
   $462 = $z4;
   $463 = (($462) + ($461))|0;
   $z4 = $463;
   $464 = $z1;
   $465 = $z3;
   $466 = (($464) + ($465))|0;
   $467 = $tmp0;
   $468 = (($467) + ($466))|0;
   $tmp0 = $468;
   $469 = $z2;
   $470 = $z4;
   $471 = (($469) + ($470))|0;
   $472 = $tmp1;
   $473 = (($472) + ($471))|0;
   $tmp1 = $473;
   $474 = $z2;
   $475 = $z3;
   $476 = (($474) + ($475))|0;
   $477 = $tmp2;
   $478 = (($477) + ($476))|0;
   $tmp2 = $478;
   $479 = $z1;
   $480 = $z4;
   $481 = (($479) + ($480))|0;
   $482 = $tmp3;
   $483 = (($482) + ($481))|0;
   $tmp3 = $483;
   $484 = $tmp10;
   $485 = $tmp3;
   $486 = (($484) + ($485))|0;
   $487 = (($486) + 131072)|0;
   $488 = $487 >> 18;
   $489 = $488 & 1023;
   $490 = $range_limit;
   $491 = (($490) + ($489)|0);
   $492 = HEAP8[$491>>0]|0;
   $493 = $outptr;
   HEAP8[$493>>0] = $492;
   $494 = $tmp10;
   $495 = $tmp3;
   $496 = (($494) - ($495))|0;
   $497 = (($496) + 131072)|0;
   $498 = $497 >> 18;
   $499 = $498 & 1023;
   $500 = $range_limit;
   $501 = (($500) + ($499)|0);
   $502 = HEAP8[$501>>0]|0;
   $503 = $outptr;
   $504 = (($503) + 7|0);
   HEAP8[$504>>0] = $502;
   $505 = $tmp11;
   $506 = $tmp2;
   $507 = (($505) + ($506))|0;
   $508 = (($507) + 131072)|0;
   $509 = $508 >> 18;
   $510 = $509 & 1023;
   $511 = $range_limit;
   $512 = (($511) + ($510)|0);
   $513 = HEAP8[$512>>0]|0;
   $514 = $outptr;
   $515 = (($514) + 1|0);
   HEAP8[$515>>0] = $513;
   $516 = $tmp11;
   $517 = $tmp2;
   $518 = (($516) - ($517))|0;
   $519 = (($518) + 131072)|0;
   $520 = $519 >> 18;
   $521 = $520 & 1023;
   $522 = $range_limit;
   $523 = (($522) + ($521)|0);
   $524 = HEAP8[$523>>0]|0;
   $525 = $outptr;
   $526 = (($525) + 6|0);
   HEAP8[$526>>0] = $524;
   $527 = $tmp12;
   $528 = $tmp1;
   $529 = (($527) + ($528))|0;
   $530 = (($529) + 131072)|0;
   $531 = $530 >> 18;
   $532 = $531 & 1023;
   $533 = $range_limit;
   $534 = (($533) + ($532)|0);
   $535 = HEAP8[$534>>0]|0;
   $536 = $outptr;
   $537 = (($536) + 2|0);
   HEAP8[$537>>0] = $535;
   $538 = $tmp12;
   $539 = $tmp1;
   $540 = (($538) - ($539))|0;
   $541 = (($540) + 131072)|0;
   $542 = $541 >> 18;
   $543 = $542 & 1023;
   $544 = $range_limit;
   $545 = (($544) + ($543)|0);
   $546 = HEAP8[$545>>0]|0;
   $547 = $outptr;
   $548 = (($547) + 5|0);
   HEAP8[$548>>0] = $546;
   $549 = $tmp13;
   $550 = $tmp0;
   $551 = (($549) + ($550))|0;
   $552 = (($551) + 131072)|0;
   $553 = $552 >> 18;
   $554 = $553 & 1023;
   $555 = $range_limit;
   $556 = (($555) + ($554)|0);
   $557 = HEAP8[$556>>0]|0;
   $558 = $outptr;
   $559 = (($558) + 3|0);
   HEAP8[$559>>0] = $557;
   $560 = $tmp13;
   $561 = $tmp0;
   $562 = (($560) - ($561))|0;
   $563 = (($562) + 131072)|0;
   $564 = $563 >> 18;
   $565 = $564 & 1023;
   $566 = $range_limit;
   $567 = (($566) + ($565)|0);
   $568 = HEAP8[$567>>0]|0;
   $569 = $outptr;
   $570 = (($569) + 4|0);
   HEAP8[$570>>0] = $568;
   $571 = $wsptr;
   $572 = (($571) + 32|0);
   $wsptr = $572;
  }
  $573 = $ctr;
  $574 = (($573) + 1)|0;
  $ctr = $574;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_4x4($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $ctr = 0, $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0, $quantptr = 0, $range_limit = 0, $tmp0 = 0, $tmp10 = 0, $tmp12 = 0, $tmp2 = 0, $workspace = 0, $wsptr = 0, $z1 = 0, $z2 = 0, $z3 = 0, $z4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $ctr;
  $16 = ($15|0)==(4);
  do {
   if ($16) {
   } else {
    $17 = $inptr;
    $18 = (($17) + 16|0);
    $19 = HEAP16[$18>>1]|0;
    $20 = $19 << 16 >> 16;
    $21 = ($20|0)==(0);
    if ($21) {
     $22 = $inptr;
     $23 = (($22) + 32|0);
     $24 = HEAP16[$23>>1]|0;
     $25 = $24 << 16 >> 16;
     $26 = ($25|0)==(0);
     if ($26) {
      $27 = $inptr;
      $28 = (($27) + 48|0);
      $29 = HEAP16[$28>>1]|0;
      $30 = $29 << 16 >> 16;
      $31 = ($30|0)==(0);
      if ($31) {
       $32 = $inptr;
       $33 = (($32) + 80|0);
       $34 = HEAP16[$33>>1]|0;
       $35 = $34 << 16 >> 16;
       $36 = ($35|0)==(0);
       if ($36) {
        $37 = $inptr;
        $38 = (($37) + 96|0);
        $39 = HEAP16[$38>>1]|0;
        $40 = $39 << 16 >> 16;
        $41 = ($40|0)==(0);
        if ($41) {
         $42 = $inptr;
         $43 = (($42) + 112|0);
         $44 = HEAP16[$43>>1]|0;
         $45 = $44 << 16 >> 16;
         $46 = ($45|0)==(0);
         if ($46) {
          $47 = $inptr;
          $48 = HEAP16[$47>>1]|0;
          $49 = $48 << 16 >> 16;
          $50 = $quantptr;
          $51 = HEAP32[$50>>2]|0;
          $52 = Math_imul($49, $51)|0;
          $53 = $52 << 2;
          $dcval = $53;
          $54 = $dcval;
          $55 = $wsptr;
          HEAP32[$55>>2] = $54;
          $56 = $dcval;
          $57 = $wsptr;
          $58 = (($57) + 32|0);
          HEAP32[$58>>2] = $56;
          $59 = $dcval;
          $60 = $wsptr;
          $61 = (($60) + 64|0);
          HEAP32[$61>>2] = $59;
          $62 = $dcval;
          $63 = $wsptr;
          $64 = (($63) + 96|0);
          HEAP32[$64>>2] = $62;
          break;
         }
        }
       }
      }
     }
    }
    $65 = $inptr;
    $66 = HEAP16[$65>>1]|0;
    $67 = $66 << 16 >> 16;
    $68 = $quantptr;
    $69 = HEAP32[$68>>2]|0;
    $70 = Math_imul($67, $69)|0;
    $tmp0 = $70;
    $71 = $tmp0;
    $72 = $71 << 14;
    $tmp0 = $72;
    $73 = $inptr;
    $74 = (($73) + 32|0);
    $75 = HEAP16[$74>>1]|0;
    $76 = $75 << 16 >> 16;
    $77 = $quantptr;
    $78 = (($77) + 64|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = Math_imul($76, $79)|0;
    $z2 = $80;
    $81 = $inptr;
    $82 = (($81) + 96|0);
    $83 = HEAP16[$82>>1]|0;
    $84 = $83 << 16 >> 16;
    $85 = $quantptr;
    $86 = (($85) + 192|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = Math_imul($84, $87)|0;
    $z3 = $88;
    $89 = $z2;
    $90 = ($89*15137)|0;
    $91 = $z3;
    $92 = Math_imul($91, -6270)|0;
    $93 = (($90) + ($92))|0;
    $tmp2 = $93;
    $94 = $tmp0;
    $95 = $tmp2;
    $96 = (($94) + ($95))|0;
    $tmp10 = $96;
    $97 = $tmp0;
    $98 = $tmp2;
    $99 = (($97) - ($98))|0;
    $tmp12 = $99;
    $100 = $inptr;
    $101 = (($100) + 112|0);
    $102 = HEAP16[$101>>1]|0;
    $103 = $102 << 16 >> 16;
    $104 = $quantptr;
    $105 = (($104) + 224|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = Math_imul($103, $106)|0;
    $z1 = $107;
    $108 = $inptr;
    $109 = (($108) + 80|0);
    $110 = HEAP16[$109>>1]|0;
    $111 = $110 << 16 >> 16;
    $112 = $quantptr;
    $113 = (($112) + 160|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = Math_imul($111, $114)|0;
    $z2 = $115;
    $116 = $inptr;
    $117 = (($116) + 48|0);
    $118 = HEAP16[$117>>1]|0;
    $119 = $118 << 16 >> 16;
    $120 = $quantptr;
    $121 = (($120) + 96|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = Math_imul($119, $122)|0;
    $z3 = $123;
    $124 = $inptr;
    $125 = (($124) + 16|0);
    $126 = HEAP16[$125>>1]|0;
    $127 = $126 << 16 >> 16;
    $128 = $quantptr;
    $129 = (($128) + 32|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = Math_imul($127, $130)|0;
    $z4 = $131;
    $132 = $z1;
    $133 = Math_imul($132, -1730)|0;
    $134 = $z2;
    $135 = ($134*11893)|0;
    $136 = (($133) + ($135))|0;
    $137 = $z3;
    $138 = Math_imul($137, -17799)|0;
    $139 = (($136) + ($138))|0;
    $140 = $z4;
    $141 = ($140*8697)|0;
    $142 = (($139) + ($141))|0;
    $tmp0 = $142;
    $143 = $z1;
    $144 = Math_imul($143, -4176)|0;
    $145 = $z2;
    $146 = Math_imul($145, -4926)|0;
    $147 = (($144) + ($146))|0;
    $148 = $z3;
    $149 = ($148*7373)|0;
    $150 = (($147) + ($149))|0;
    $151 = $z4;
    $152 = ($151*20995)|0;
    $153 = (($150) + ($152))|0;
    $tmp2 = $153;
    $154 = $tmp10;
    $155 = $tmp2;
    $156 = (($154) + ($155))|0;
    $157 = (($156) + 2048)|0;
    $158 = $157 >> 12;
    $159 = $wsptr;
    HEAP32[$159>>2] = $158;
    $160 = $tmp10;
    $161 = $tmp2;
    $162 = (($160) - ($161))|0;
    $163 = (($162) + 2048)|0;
    $164 = $163 >> 12;
    $165 = $wsptr;
    $166 = (($165) + 96|0);
    HEAP32[$166>>2] = $164;
    $167 = $tmp12;
    $168 = $tmp0;
    $169 = (($167) + ($168))|0;
    $170 = (($169) + 2048)|0;
    $171 = $170 >> 12;
    $172 = $wsptr;
    $173 = (($172) + 32|0);
    HEAP32[$173>>2] = $171;
    $174 = $tmp12;
    $175 = $tmp0;
    $176 = (($174) - ($175))|0;
    $177 = (($176) + 2048)|0;
    $178 = $177 >> 12;
    $179 = $wsptr;
    $180 = (($179) + 64|0);
    HEAP32[$180>>2] = $178;
   }
  } while(0);
  $181 = $inptr;
  $182 = (($181) + 2|0);
  $inptr = $182;
  $183 = $quantptr;
  $184 = (($183) + 4|0);
  $quantptr = $184;
  $185 = $wsptr;
  $186 = (($185) + 4|0);
  $wsptr = $186;
  $187 = $ctr;
  $188 = (($187) + -1)|0;
  $ctr = $188;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $189 = $ctr;
  $190 = ($189|0)<(4);
  if (!($190)) {
   break;
  }
  $191 = $ctr;
  $192 = $3;
  $193 = (($192) + ($191<<2)|0);
  $194 = HEAP32[$193>>2]|0;
  $195 = $4;
  $196 = (($194) + ($195)|0);
  $outptr = $196;
  $197 = $wsptr;
  $198 = (($197) + 4|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = ($199|0)==(0);
  if ($200) {
   $201 = $wsptr;
   $202 = (($201) + 8|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = ($203|0)==(0);
   if ($204) {
    $205 = $wsptr;
    $206 = (($205) + 12|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = ($207|0)==(0);
    if ($208) {
     $209 = $wsptr;
     $210 = (($209) + 20|0);
     $211 = HEAP32[$210>>2]|0;
     $212 = ($211|0)==(0);
     if ($212) {
      $213 = $wsptr;
      $214 = (($213) + 24|0);
      $215 = HEAP32[$214>>2]|0;
      $216 = ($215|0)==(0);
      if ($216) {
       $217 = $wsptr;
       $218 = (($217) + 28|0);
       $219 = HEAP32[$218>>2]|0;
       $220 = ($219|0)==(0);
       if ($220) {
        $221 = $wsptr;
        $222 = HEAP32[$221>>2]|0;
        $223 = (($222) + 16)|0;
        $224 = $223 >> 5;
        $225 = $224 & 1023;
        $226 = $range_limit;
        $227 = (($226) + ($225)|0);
        $228 = HEAP8[$227>>0]|0;
        $dcval1 = $228;
        $229 = $dcval1;
        $230 = $outptr;
        HEAP8[$230>>0] = $229;
        $231 = $dcval1;
        $232 = $outptr;
        $233 = (($232) + 1|0);
        HEAP8[$233>>0] = $231;
        $234 = $dcval1;
        $235 = $outptr;
        $236 = (($235) + 2|0);
        HEAP8[$236>>0] = $234;
        $237 = $dcval1;
        $238 = $outptr;
        $239 = (($238) + 3|0);
        HEAP8[$239>>0] = $237;
        $240 = $wsptr;
        $241 = (($240) + 32|0);
        $wsptr = $241;
       } else {
        label = 23;
       }
      } else {
       label = 23;
      }
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $242 = $wsptr;
   $243 = HEAP32[$242>>2]|0;
   $244 = $243 << 14;
   $tmp0 = $244;
   $245 = $wsptr;
   $246 = (($245) + 8|0);
   $247 = HEAP32[$246>>2]|0;
   $248 = ($247*15137)|0;
   $249 = $wsptr;
   $250 = (($249) + 24|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = Math_imul($251, -6270)|0;
   $253 = (($248) + ($252))|0;
   $tmp2 = $253;
   $254 = $tmp0;
   $255 = $tmp2;
   $256 = (($254) + ($255))|0;
   $tmp10 = $256;
   $257 = $tmp0;
   $258 = $tmp2;
   $259 = (($257) - ($258))|0;
   $tmp12 = $259;
   $260 = $wsptr;
   $261 = (($260) + 28|0);
   $262 = HEAP32[$261>>2]|0;
   $z1 = $262;
   $263 = $wsptr;
   $264 = (($263) + 20|0);
   $265 = HEAP32[$264>>2]|0;
   $z2 = $265;
   $266 = $wsptr;
   $267 = (($266) + 12|0);
   $268 = HEAP32[$267>>2]|0;
   $z3 = $268;
   $269 = $wsptr;
   $270 = (($269) + 4|0);
   $271 = HEAP32[$270>>2]|0;
   $z4 = $271;
   $272 = $z1;
   $273 = Math_imul($272, -1730)|0;
   $274 = $z2;
   $275 = ($274*11893)|0;
   $276 = (($273) + ($275))|0;
   $277 = $z3;
   $278 = Math_imul($277, -17799)|0;
   $279 = (($276) + ($278))|0;
   $280 = $z4;
   $281 = ($280*8697)|0;
   $282 = (($279) + ($281))|0;
   $tmp0 = $282;
   $283 = $z1;
   $284 = Math_imul($283, -4176)|0;
   $285 = $z2;
   $286 = Math_imul($285, -4926)|0;
   $287 = (($284) + ($286))|0;
   $288 = $z3;
   $289 = ($288*7373)|0;
   $290 = (($287) + ($289))|0;
   $291 = $z4;
   $292 = ($291*20995)|0;
   $293 = (($290) + ($292))|0;
   $tmp2 = $293;
   $294 = $tmp10;
   $295 = $tmp2;
   $296 = (($294) + ($295))|0;
   $297 = (($296) + 262144)|0;
   $298 = $297 >> 19;
   $299 = $298 & 1023;
   $300 = $range_limit;
   $301 = (($300) + ($299)|0);
   $302 = HEAP8[$301>>0]|0;
   $303 = $outptr;
   HEAP8[$303>>0] = $302;
   $304 = $tmp10;
   $305 = $tmp2;
   $306 = (($304) - ($305))|0;
   $307 = (($306) + 262144)|0;
   $308 = $307 >> 19;
   $309 = $308 & 1023;
   $310 = $range_limit;
   $311 = (($310) + ($309)|0);
   $312 = HEAP8[$311>>0]|0;
   $313 = $outptr;
   $314 = (($313) + 3|0);
   HEAP8[$314>>0] = $312;
   $315 = $tmp12;
   $316 = $tmp0;
   $317 = (($315) + ($316))|0;
   $318 = (($317) + 262144)|0;
   $319 = $318 >> 19;
   $320 = $319 & 1023;
   $321 = $range_limit;
   $322 = (($321) + ($320)|0);
   $323 = HEAP8[$322>>0]|0;
   $324 = $outptr;
   $325 = (($324) + 1|0);
   HEAP8[$325>>0] = $323;
   $326 = $tmp12;
   $327 = $tmp0;
   $328 = (($326) - ($327))|0;
   $329 = (($328) + 262144)|0;
   $330 = $329 >> 19;
   $331 = $330 & 1023;
   $332 = $range_limit;
   $333 = (($332) + ($331)|0);
   $334 = HEAP8[$333>>0]|0;
   $335 = $outptr;
   $336 = (($335) + 2|0);
   HEAP8[$336>>0] = $334;
   $337 = $wsptr;
   $338 = (($337) + 32|0);
   $wsptr = $338;
  }
  $339 = $ctr;
  $340 = (($339) + 1)|0;
  $ctr = $340;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_2x2($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctr = 0, $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0, $quantptr = 0;
 var $range_limit = 0, $tmp0 = 0, $tmp10 = 0, $workspace = 0, $wsptr = 0, $z1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $ctr;
  $16 = ($15|0)==(6);
  do {
   if ($16) {
    label = 6;
   } else {
    $17 = $ctr;
    $18 = ($17|0)==(4);
    if ($18) {
     label = 6;
    } else {
     $19 = $ctr;
     $20 = ($19|0)==(2);
     if ($20) {
      label = 6;
     } else {
      $21 = $inptr;
      $22 = (($21) + 16|0);
      $23 = HEAP16[$22>>1]|0;
      $24 = $23 << 16 >> 16;
      $25 = ($24|0)==(0);
      if ($25) {
       $26 = $inptr;
       $27 = (($26) + 48|0);
       $28 = HEAP16[$27>>1]|0;
       $29 = $28 << 16 >> 16;
       $30 = ($29|0)==(0);
       if ($30) {
        $31 = $inptr;
        $32 = (($31) + 80|0);
        $33 = HEAP16[$32>>1]|0;
        $34 = $33 << 16 >> 16;
        $35 = ($34|0)==(0);
        if ($35) {
         $36 = $inptr;
         $37 = (($36) + 112|0);
         $38 = HEAP16[$37>>1]|0;
         $39 = $38 << 16 >> 16;
         $40 = ($39|0)==(0);
         if ($40) {
          $41 = $inptr;
          $42 = HEAP16[$41>>1]|0;
          $43 = $42 << 16 >> 16;
          $44 = $quantptr;
          $45 = HEAP32[$44>>2]|0;
          $46 = Math_imul($43, $45)|0;
          $47 = $46 << 2;
          $dcval = $47;
          $48 = $dcval;
          $49 = $wsptr;
          HEAP32[$49>>2] = $48;
          $50 = $dcval;
          $51 = $wsptr;
          $52 = (($51) + 32|0);
          HEAP32[$52>>2] = $50;
          break;
         }
        }
       }
      }
      $53 = $inptr;
      $54 = HEAP16[$53>>1]|0;
      $55 = $54 << 16 >> 16;
      $56 = $quantptr;
      $57 = HEAP32[$56>>2]|0;
      $58 = Math_imul($55, $57)|0;
      $z1 = $58;
      $59 = $z1;
      $60 = $59 << 15;
      $tmp10 = $60;
      $61 = $inptr;
      $62 = (($61) + 112|0);
      $63 = HEAP16[$62>>1]|0;
      $64 = $63 << 16 >> 16;
      $65 = $quantptr;
      $66 = (($65) + 224|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = Math_imul($64, $67)|0;
      $z1 = $68;
      $69 = $z1;
      $70 = Math_imul($69, -5906)|0;
      $tmp0 = $70;
      $71 = $inptr;
      $72 = (($71) + 80|0);
      $73 = HEAP16[$72>>1]|0;
      $74 = $73 << 16 >> 16;
      $75 = $quantptr;
      $76 = (($75) + 160|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = Math_imul($74, $77)|0;
      $z1 = $78;
      $79 = $z1;
      $80 = ($79*6967)|0;
      $81 = $tmp0;
      $82 = (($81) + ($80))|0;
      $tmp0 = $82;
      $83 = $inptr;
      $84 = (($83) + 48|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85 << 16 >> 16;
      $87 = $quantptr;
      $88 = (($87) + 96|0);
      $89 = HEAP32[$88>>2]|0;
      $90 = Math_imul($86, $89)|0;
      $z1 = $90;
      $91 = $z1;
      $92 = Math_imul($91, -10426)|0;
      $93 = $tmp0;
      $94 = (($93) + ($92))|0;
      $tmp0 = $94;
      $95 = $inptr;
      $96 = (($95) + 16|0);
      $97 = HEAP16[$96>>1]|0;
      $98 = $97 << 16 >> 16;
      $99 = $quantptr;
      $100 = (($99) + 32|0);
      $101 = HEAP32[$100>>2]|0;
      $102 = Math_imul($98, $101)|0;
      $z1 = $102;
      $103 = $z1;
      $104 = ($103*29692)|0;
      $105 = $tmp0;
      $106 = (($105) + ($104))|0;
      $tmp0 = $106;
      $107 = $tmp10;
      $108 = $tmp0;
      $109 = (($107) + ($108))|0;
      $110 = (($109) + 4096)|0;
      $111 = $110 >> 13;
      $112 = $wsptr;
      HEAP32[$112>>2] = $111;
      $113 = $tmp10;
      $114 = $tmp0;
      $115 = (($113) - ($114))|0;
      $116 = (($115) + 4096)|0;
      $117 = $116 >> 13;
      $118 = $wsptr;
      $119 = (($118) + 32|0);
      HEAP32[$119>>2] = $117;
     }
    }
   }
  } while(0);
  if ((label|0) == 6) {
   label = 0;
  }
  $120 = $inptr;
  $121 = (($120) + 2|0);
  $inptr = $121;
  $122 = $quantptr;
  $123 = (($122) + 4|0);
  $quantptr = $123;
  $124 = $wsptr;
  $125 = (($124) + 4|0);
  $wsptr = $125;
  $126 = $ctr;
  $127 = (($126) + -1)|0;
  $ctr = $127;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $128 = $ctr;
  $129 = ($128|0)<(2);
  if (!($129)) {
   break;
  }
  $130 = $ctr;
  $131 = $3;
  $132 = (($131) + ($130<<2)|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = $4;
  $135 = (($133) + ($134)|0);
  $outptr = $135;
  $136 = $wsptr;
  $137 = (($136) + 4|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = ($138|0)==(0);
  if ($139) {
   $140 = $wsptr;
   $141 = (($140) + 12|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = ($142|0)==(0);
   if ($143) {
    $144 = $wsptr;
    $145 = (($144) + 20|0);
    $146 = HEAP32[$145>>2]|0;
    $147 = ($146|0)==(0);
    if ($147) {
     $148 = $wsptr;
     $149 = (($148) + 28|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = ($150|0)==(0);
     if ($151) {
      $152 = $wsptr;
      $153 = HEAP32[$152>>2]|0;
      $154 = (($153) + 16)|0;
      $155 = $154 >> 5;
      $156 = $155 & 1023;
      $157 = $range_limit;
      $158 = (($157) + ($156)|0);
      $159 = HEAP8[$158>>0]|0;
      $dcval1 = $159;
      $160 = $dcval1;
      $161 = $outptr;
      HEAP8[$161>>0] = $160;
      $162 = $dcval1;
      $163 = $outptr;
      $164 = (($163) + 1|0);
      HEAP8[$164>>0] = $162;
      $165 = $wsptr;
      $166 = (($165) + 32|0);
      $wsptr = $166;
     } else {
      label = 21;
     }
    } else {
     label = 21;
    }
   } else {
    label = 21;
   }
  } else {
   label = 21;
  }
  if ((label|0) == 21) {
   label = 0;
   $167 = $wsptr;
   $168 = HEAP32[$167>>2]|0;
   $169 = $168 << 15;
   $tmp10 = $169;
   $170 = $wsptr;
   $171 = (($170) + 28|0);
   $172 = HEAP32[$171>>2]|0;
   $173 = Math_imul($172, -5906)|0;
   $174 = $wsptr;
   $175 = (($174) + 20|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = ($176*6967)|0;
   $178 = (($173) + ($177))|0;
   $179 = $wsptr;
   $180 = (($179) + 12|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = Math_imul($181, -10426)|0;
   $183 = (($178) + ($182))|0;
   $184 = $wsptr;
   $185 = (($184) + 4|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = ($186*29692)|0;
   $188 = (($183) + ($187))|0;
   $tmp0 = $188;
   $189 = $tmp10;
   $190 = $tmp0;
   $191 = (($189) + ($190))|0;
   $192 = (($191) + 524288)|0;
   $193 = $192 >> 20;
   $194 = $193 & 1023;
   $195 = $range_limit;
   $196 = (($195) + ($194)|0);
   $197 = HEAP8[$196>>0]|0;
   $198 = $outptr;
   HEAP8[$198>>0] = $197;
   $199 = $tmp10;
   $200 = $tmp0;
   $201 = (($199) - ($200))|0;
   $202 = (($201) + 524288)|0;
   $203 = $202 >> 20;
   $204 = $203 & 1023;
   $205 = $range_limit;
   $206 = (($205) + ($204)|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $outptr;
   $209 = (($208) + 1|0);
   HEAP8[$209>>0] = $207;
   $210 = $wsptr;
   $211 = (($210) + 32|0);
   $wsptr = $211;
  }
  $212 = $ctr;
  $213 = (($212) + 1)|0;
  $ctr = $213;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_1x1($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dcval = 0, $quantptr = 0, $range_limit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $1;
 $10 = (($9) + 80|0);
 $11 = HEAP32[$10>>2]|0;
 $quantptr = $11;
 $12 = $2;
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = $quantptr;
 $16 = HEAP32[$15>>2]|0;
 $17 = Math_imul($14, $16)|0;
 $dcval = $17;
 $18 = $dcval;
 $19 = (($18) + 4)|0;
 $20 = $19 >> 3;
 $dcval = $20;
 $21 = $dcval;
 $22 = $21 & 1023;
 $23 = $range_limit;
 $24 = (($23) + ($22)|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $4;
 $27 = $3;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($26)|0);
 HEAP8[$29>>0] = $25;
 STACKTOP = sp;return;
}
function _isprint($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -32)|0;
 $1 = ($0>>>0)<(95);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _mbrtowc($wc,$src,$n,$st) {
 $wc = $wc|0;
 $src = $src|0;
 $n = $n|0;
 $st = $st|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c$0 = 0, $c$1 = 0, $c$2 = 0, $s$0 = 0, $s$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $wc;
 $1 = ($st|0)==(0|0);
 $$01 = $1 ? 10352 : $st;
 $2 = HEAP32[$$01>>2]|0;
 $3 = ($src|0)==(0|0);
 L1: do {
  if ($3) {
   $4 = ($2|0)==(0);
   if ($4) {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  } else {
   $5 = ($wc|0)==(0|0);
   if ($5) {
    HEAP32[$0>>2] = $0;
    $11 = $0;
   } else {
    $11 = $wc;
   }
   $6 = ($n|0)==(0);
   if ($6) {
    $$0 = -2;
    STACKTOP = sp;return ($$0|0);
   }
   $7 = ($2|0)==(0);
   do {
    if ($7) {
     $8 = HEAP8[$src>>0]|0;
     $9 = $8&255;
     $10 = ($8<<24>>24)>(-1);
     if ($10) {
      HEAP32[$11>>2] = $9;
      $12 = ($8<<24>>24)!=(0);
      $13 = $12&1;
      $$0 = $13;
      STACKTOP = sp;return ($$0|0);
     } else {
      $14 = (($9) + -194)|0;
      $15 = ($14>>>0)>(50);
      if ($15) {
       break L1;
      }
      $16 = (($src) + 1|0);
      $17 = (10144 + ($14<<2)|0);
      $18 = HEAP32[$17>>2]|0;
      $19 = (($n) + -1)|0;
      $$02 = $19;$c$0 = $18;$s$0 = $16;
      break;
     }
    } else {
     $$02 = $n;$c$0 = $2;$s$0 = $src;
    }
   } while(0);
   $20 = ($$02|0)==(0);
   L17: do {
    if ($20) {
     $c$2 = $c$0;
    } else {
     $21 = HEAP8[$s$0>>0]|0;
     $22 = $21&255;
     $23 = $22 >>> 3;
     $24 = (($23) + -16)|0;
     $25 = $c$0 >> 26;
     $26 = (($23) + ($25))|0;
     $27 = $24 | $26;
     $28 = ($27>>>0)>(7);
     if ($28) {
      break L1;
     } else {
      $$1 = $$02;$32 = $21;$c$1 = $c$0;$s$1 = $s$0;
     }
     while(1) {
      $29 = $c$1 << 6;
      $30 = (($s$1) + 1|0);
      $31 = $32&255;
      $33 = (($31) + -128)|0;
      $34 = $33 | $29;
      $35 = (($$1) + -1)|0;
      $36 = ($34|0)<(0);
      if (!($36)) {
       break;
      }
      $38 = ($35|0)==(0);
      if ($38) {
       $c$2 = $34;
       break L17;
      }
      $39 = HEAP8[$30>>0]|0;
      $40 = $39&255;
      $41 = (($40) + -128)|0;
      $42 = ($41>>>0)>(63);
      if ($42) {
       break L1;
      } else {
       $$1 = $35;$32 = $39;$c$1 = $34;$s$1 = $30;
      }
     }
     HEAP32[$$01>>2] = 0;
     HEAP32[$11>>2] = $34;
     $37 = (($n) - ($35))|0;
     $$0 = $37;
     STACKTOP = sp;return ($$0|0);
    }
   } while(0);
   HEAP32[$$01>>2] = $c$2;
   $$0 = -2;
   STACKTOP = sp;return ($$0|0);
  }
 } while(0);
 HEAP32[$$01>>2] = 0;
 $43 = (___errno_location()|0);
 HEAP32[$43>>2] = 84;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function _mbsinit($st) {
 $st = $st|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($st|0)==(0|0);
 if ($0) {
  $4 = 1;
 } else {
  $1 = HEAP32[$st>>2]|0;
  $2 = ($1|0)==(0);
  $4 = $2;
 }
 $3 = $4&1;
 STACKTOP = sp;return ($3|0);
}
function ___string_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $k$0 = 0, $k$0$len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 84|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($len) + 256)|0;
 $3 = (_memchr($1,0,$2)|0);
 $4 = ($3|0)==(0|0);
 if ($4) {
  $k$0 = $2;
 } else {
  $5 = $3;
  $6 = $1;
  $7 = (($5) - ($6))|0;
  $k$0 = $7;
 }
 $8 = ($k$0>>>0)<($len>>>0);
 $k$0$len = $8 ? $k$0 : $len;
 _memcpy(($buf|0),($1|0),($k$0$len|0))|0;
 $9 = (($1) + ($k$0$len)|0);
 $10 = (($f) + 4|0);
 HEAP32[$10>>2] = $9;
 $11 = (($1) + ($k$0)|0);
 $12 = (($f) + 8|0);
 HEAP32[$12>>2] = $11;
 HEAP32[$0>>2] = $11;
 STACKTOP = sp;return ($k$0$len|0);
}
function _sscanf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsscanf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfscanf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$10 = 0, $$4 = 0, $$7 = 0, $$8 = 0, $$9 = 0, $$lcssa18 = 0, $$matches$0 = 0, $$not = 0, $$pre = 0, $$pre$phi213Z2D = 0, $$pre193 = 0, $$pre206 = 0, $$pre207 = 0, $$pre208 = 0, $$pre209 = 0, $$pre210 = 0, $$pre211 = 0, $$pre212 = 0, $$size$0 = 0;
 var $$width$0 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0.0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $alloc$0 = 0, $alloc$1 = 0, $alloc$2 = 0, $ap2$i = 0, $arglist_current = 0, $arglist_current2 = 0, $arglist_next = 0, $arglist_next3 = 0, $base$0 = 0, $c$027 = 0, $dest$0 = 0, $factor = 0;
 var $factor14 = 0, $i$0$i = 0, $i$0$ph = 0, $i$0$ph46 = 0, $i$0$ph46$lcssa = 0, $i$0$ph46$us53 = 0, $i$1$lcssa = 0, $i$2 = 0, $i$2$ph = 0, $i$2$ph$phi = 0, $i$3 = 0, $i$4 = 0, $invert$0 = 0, $isdigit = 0, $isdigit3 = 0, $isdigit320 = 0, $isdigittmp = 0, $isdigittmp2 = 0, $isdigittmp219 = 0, $k$0$ph = 0;
 var $k$1$ph = 0, $matches$0$ = 0, $matches$0116 = 0, $matches$1 = 0, $matches$2 = 0, $matches$3 = 0, $not$ = 0, $or$cond$us = 0, $or$cond5 = 0, $or$cond6 = 0, $p$0122 = 0, $p$1 = 0, $p$10 = 0, $p$11 = 0, $p$2 = 0, $p$3$lcssa = 0, $p$321 = 0, $p$4 = 0, $p$5 = 0, $p$6 = 0;
 var $p$7 = 0, $p$7$ph = 0, $p$8 = 0, $p$9 = 0, $pos$0121 = 0, $pos$1 = 0, $pos$2 = 0, $s$0114 = 0, $s$1$ph = 0, $s$2 = 0, $s$3 = 0, $s$4 = 0, $s$5 = 0, $s$6 = 0, $scanset = 0, $size$0 = 0, $st = 0, $vacopy_currentptr = 0, $wc = 0, $wcs$0115 = 0;
 var $wcs$1 = 0, $wcs$2$ph = 0, $wcs$2$ph146 = 0, $wcs$3 = 0, $wcs$4 = 0, $wcs$5 = 0, $wcs$6 = 0, $wcs$7 = 0, $width$0$lcssa = 0, $width$022 = 0, $width$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $ap2$i = sp + 16|0;
 $st = sp + 8|0;
 $scanset = sp + 33|0;
 $wc = sp;
 $0 = sp + 32|0;
 $1 = HEAP8[$fmt>>0]|0;
 $2 = ($1<<24>>24)==(0);
 if ($2) {
  $matches$3 = 0;
  STACKTOP = sp;return ($matches$3|0);
 }
 $3 = (($f) + 4|0);
 $4 = (($f) + 100|0);
 $5 = (($f) + 108|0);
 $6 = (($f) + 8|0);
 $7 = (($scanset) + 10|0);
 $8 = (($scanset) + 33|0);
 $9 = (($st) + 4|0);
 $10 = (($scanset) + 46|0);
 $11 = (($scanset) + 94|0);
 $13 = $1;$matches$0116 = 0;$p$0122 = $fmt;$pos$0121 = 0;$s$0114 = 0;$wcs$0115 = 0;
 L4: while(1) {
  $12 = $13&255;
  $14 = (_isspace($12)|0);
  $15 = ($14|0)==(0);
  L6: do {
   if ($15) {
    $42 = HEAP8[$p$0122>>0]|0;
    $43 = ($42<<24>>24)==(37);
    L21: do {
     if ($43) {
      $44 = (($p$0122) + 1|0);
      $45 = HEAP8[$44>>0]|0;
      do {
       if ((($45<<24>>24) == 37)) {
        break L21;
       } else if ((($45<<24>>24) == 42)) {
        $65 = (($p$0122) + 2|0);
        $dest$0 = 0;$p$2 = $65;
       } else {
        $66 = $45&255;
        $isdigittmp = (($66) + -48)|0;
        $isdigit = ($isdigittmp>>>0)<(10);
        if ($isdigit) {
         $67 = (($p$0122) + 2|0);
         $68 = HEAP8[$67>>0]|0;
         $69 = ($68<<24>>24)==(36);
         if ($69) {
          $vacopy_currentptr = HEAP32[$ap>>2]|0;
          HEAP32[$ap2$i>>2] = $vacopy_currentptr;
          $i$0$i = $isdigittmp;
          while(1) {
           $70 = ($i$0$i>>>0)>(1);
           $arglist_current = HEAP32[$ap2$i>>2]|0;
           $71 = HEAP32[$arglist_current>>2]|0;
           $arglist_next = (($arglist_current) + 4|0);
           HEAP32[$ap2$i>>2] = $arglist_next;
           $72 = (($i$0$i) + -1)|0;
           if ($70) {
            $i$0$i = $72;
           } else {
            break;
           }
          }
          $73 = (($p$0122) + 3|0);
          $dest$0 = $71;$p$2 = $73;
          break;
         }
        }
        $arglist_current2 = HEAP32[$ap>>2]|0;
        $74 = HEAP32[$arglist_current2>>2]|0;
        $arglist_next3 = (($arglist_current2) + 4|0);
        HEAP32[$ap>>2] = $arglist_next3;
        $dest$0 = $74;$p$2 = $44;
       }
      } while(0);
      $75 = HEAP8[$p$2>>0]|0;
      $76 = $75&255;
      $isdigittmp219 = (($76) + -48)|0;
      $isdigit320 = ($isdigittmp219>>>0)<(10);
      if ($isdigit320) {
       $80 = $76;$p$321 = $p$2;$width$022 = 0;
       while(1) {
        $77 = ($width$022*10)|0;
        $78 = (($77) + -48)|0;
        $79 = (($78) + ($80))|0;
        $81 = (($p$321) + 1|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $isdigittmp2 = (($83) + -48)|0;
        $isdigit3 = ($isdigittmp2>>>0)<(10);
        if ($isdigit3) {
         $80 = $83;$p$321 = $81;$width$022 = $79;
        } else {
         $$lcssa18 = $82;$p$3$lcssa = $81;$width$0$lcssa = $79;
         break;
        }
       }
      } else {
       $$lcssa18 = $75;$p$3$lcssa = $p$2;$width$0$lcssa = 0;
      }
      $84 = ($$lcssa18<<24>>24)==(109);
      if ($84) {
       $85 = ($dest$0|0)!=(0|0);
       $86 = $85&1;
       $87 = (($p$3$lcssa) + 1|0);
       $$pre = HEAP8[$87>>0]|0;
       $90 = $$pre;$alloc$0 = $86;$p$4 = $87;
      } else {
       $90 = $$lcssa18;$alloc$0 = 0;$p$4 = $p$3$lcssa;
      }
      $88 = (($p$4) + 1|0);
      $89 = $90&255;
      switch ($89|0) {
      case 104:  {
       $91 = HEAP8[$88>>0]|0;
       $92 = ($91<<24>>24)==(104);
       $93 = (($p$4) + 2|0);
       $$7 = $92 ? $93 : $88;
       $$8 = $92 ? -2 : -1;
       $p$5 = $$7;$size$0 = $$8;
       break;
      }
      case 108:  {
       $94 = HEAP8[$88>>0]|0;
       $95 = ($94<<24>>24)==(108);
       $96 = (($p$4) + 2|0);
       $$9 = $95 ? $96 : $88;
       $$10 = $95 ? 3 : 1;
       $p$5 = $$9;$size$0 = $$10;
       break;
      }
      case 116: case 122:  {
       $p$5 = $88;$size$0 = 1;
       break;
      }
      case 76:  {
       $p$5 = $88;$size$0 = 2;
       break;
      }
      case 110: case 112: case 67: case 83: case 91: case 99: case 115: case 88: case 71: case 70: case 69: case 65: case 103: case 102: case 101: case 97: case 120: case 117: case 111: case 105: case 100:  {
       $p$5 = $p$4;$size$0 = 0;
       break;
      }
      case 106:  {
       $p$5 = $88;$size$0 = 3;
       break;
      }
      default: {
       $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
       label = 162;
       break L4;
      }
      }
      $97 = HEAP8[$p$5>>0]|0;
      $98 = $97&255;
      $99 = $98 & 47;
      $100 = ($99|0)==(3);
      $101 = $98 | 32;
      $$ = $100 ? $101 : $98;
      $$size$0 = $100 ? 1 : $size$0;
      if ((($$|0) == 91)) {
       $pos$1 = $pos$0121;$width$1 = $width$0$lcssa;
      } else if ((($$|0) == 110)) {
       $103 = ($pos$0121|0)<(0);
       $104 = $103 << 31 >> 31;
       $105 = ($dest$0|0)==(0|0);
       if ($105) {
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
       }
       switch ($$size$0|0) {
       case -2:  {
        $106 = $pos$0121&255;
        HEAP8[$dest$0>>0] = $106;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case -1:  {
        $107 = $pos$0121&65535;
        HEAP16[$dest$0>>1] = $107;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 0:  {
        HEAP32[$dest$0>>2] = $pos$0121;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 1:  {
        HEAP32[$dest$0>>2] = $pos$0121;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 3:  {
        $108 = $dest$0;
        $109 = $108;
        HEAP32[$109>>2] = $pos$0121;
        $110 = (($108) + 4)|0;
        $111 = $110;
        HEAP32[$111>>2] = $104;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       default: {
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
       }
       }
      } else if ((($$|0) == 99)) {
       $102 = ($width$0$lcssa|0)<(1);
       $$width$0 = $102 ? 1 : $width$0$lcssa;
       $pos$1 = $pos$0121;$width$1 = $$width$0;
      } else {
       ___shlim($f,0);
       while(1) {
        $112 = HEAP32[$3>>2]|0;
        $113 = HEAP32[$4>>2]|0;
        $114 = ($112>>>0)<($113>>>0);
        if ($114) {
         $115 = (($112) + 1|0);
         HEAP32[$3>>2] = $115;
         $116 = HEAP8[$112>>0]|0;
         $117 = $116&255;
         $119 = $117;
        } else {
         $118 = (___shgetc($f)|0);
         $119 = $118;
        }
        $120 = (_isspace($119)|0);
        $121 = ($120|0)==(0);
        if ($121) {
         break;
        }
       }
       $122 = HEAP32[$4>>2]|0;
       $123 = ($122|0)==(0|0);
       $$pre206 = HEAP32[$3>>2]|0;
       if ($123) {
        $128 = $$pre206;
       } else {
        $124 = (($$pre206) + -1|0);
        HEAP32[$3>>2] = $124;
        $128 = $124;
       }
       $125 = HEAP32[$5>>2]|0;
       $126 = HEAP32[$6>>2]|0;
       $127 = $128;
       $129 = $126;
       $130 = (($125) + ($pos$0121))|0;
       $131 = (($130) + ($127))|0;
       $132 = (($131) - ($129))|0;
       $pos$1 = $132;$width$1 = $width$0$lcssa;
      }
      ___shlim($f,$width$1);
      $133 = HEAP32[$3>>2]|0;
      $134 = HEAP32[$4>>2]|0;
      $135 = ($133>>>0)<($134>>>0);
      if ($135) {
       $136 = (($133) + 1|0);
       HEAP32[$3>>2] = $136;
       $140 = $134;
      } else {
       $137 = (___shgetc($f)|0);
       $138 = ($137|0)<(0);
       if ($138) {
        $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
        label = 162;
        break L4;
       }
       $$pre207 = HEAP32[$4>>2]|0;
       $140 = $$pre207;
      }
      $139 = ($140|0)==(0|0);
      if (!($139)) {
       $141 = HEAP32[$3>>2]|0;
       $142 = (($141) + -1|0);
       HEAP32[$3>>2] = $142;
      }
      L78: do {
       switch ($$|0) {
       case 111:  {
        $base$0 = 8;
        label = 144;
        break;
       }
       case 117: case 100:  {
        $base$0 = 10;
        label = 144;
        break;
       }
       case 105:  {
        $base$0 = 0;
        label = 144;
        break;
       }
       case 120: case 88: case 112:  {
        $base$0 = 16;
        label = 144;
        break;
       }
       case 71: case 103: case 70: case 102: case 69: case 101: case 65: case 97:  {
        $322 = (+___floatscan($f,$$size$0,0));
        $323 = HEAP32[$5>>2]|0;
        $324 = HEAP32[$3>>2]|0;
        $325 = HEAP32[$6>>2]|0;
        $326 = $324;
        $327 = $325;
        $328 = (($327) - ($326))|0;
        $329 = ($323|0)==($328|0);
        if ($329) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$0114;$wcs$7 = $wcs$0115;
         break L4;
        }
        $330 = ($dest$0|0)==(0|0);
        if ($330) {
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
        } else {
         if ((($$size$0|0) == 0)) {
          $331 = $322;
          HEAPF32[$dest$0>>2] = $331;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else if ((($$size$0|0) == 1)) {
          HEAPF64[$dest$0>>3] = $322;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else if ((($$size$0|0) == 2)) {
          HEAPF64[$dest$0>>3] = $322;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else {
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         }
        }
        break;
       }
       case 91: case 99: case 115:  {
        $143 = ($$|0)==(99);
        $144 = $$ & 239;
        $145 = ($144|0)==(99);
        L90: do {
         if ($145) {
          $146 = ($$|0)==(115);
          _memset(($scanset|0),-1,257)|0;
          HEAP8[$scanset>>0] = 0;
          if ($146) {
           HEAP8[$8>>0] = 0;
           ;HEAP8[$7+0>>0]=0|0;HEAP8[$7+1>>0]=0|0;HEAP8[$7+2>>0]=0|0;HEAP8[$7+3>>0]=0|0;HEAP8[$7+4>>0]=0|0;
           $p$9 = $p$5;
          } else {
           $p$9 = $p$5;
          }
         } else {
          $147 = (($p$5) + 1|0);
          $148 = HEAP8[$147>>0]|0;
          $149 = ($148<<24>>24)==(94);
          $150 = (($p$5) + 2|0);
          $invert$0 = $149&1;
          $p$6 = $149 ? $150 : $147;
          $151 = $149&1;
          _memset(($scanset|0),($151|0),257)|0;
          HEAP8[$scanset>>0] = 0;
          $152 = HEAP8[$p$6>>0]|0;
          if ((($152<<24>>24) == 45)) {
           $153 = (($p$6) + 1|0);
           $154 = $invert$0 ^ 1;
           $155 = $154&255;
           HEAP8[$10>>0] = $155;
           $$pre$phi213Z2D = $155;$p$7$ph = $153;
          } else if ((($152<<24>>24) == 93)) {
           $156 = (($p$6) + 1|0);
           $157 = $invert$0 ^ 1;
           $158 = $157&255;
           HEAP8[$11>>0] = $158;
           $$pre$phi213Z2D = $158;$p$7$ph = $156;
          } else {
           $$pre211 = $invert$0 ^ 1;
           $$pre212 = $$pre211&255;
           $$pre$phi213Z2D = $$pre212;$p$7$ph = $p$6;
          }
          $p$7 = $p$7$ph;
          while(1) {
           $159 = HEAP8[$p$7>>0]|0;
           if ((($159<<24>>24) == 93)) {
            $p$9 = $p$7;
            break L90;
           } else if ((($159<<24>>24) == 0)) {
            $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
            label = 162;
            break L4;
           } else if ((($159<<24>>24) == 45)) {
            $160 = (($p$7) + 1|0);
            $161 = HEAP8[$160>>0]|0;
            if ((($161<<24>>24) == 93) | (($161<<24>>24) == 0)) {
             $172 = 45;$p$8 = $p$7;
            } else {
             $162 = (($p$7) + -1|0);
             $163 = HEAP8[$162>>0]|0;
             $164 = ($163&255)<($161&255);
             if ($164) {
              $165 = $163&255;
              $c$027 = $165;
              while(1) {
               $166 = (($c$027) + 1)|0;
               $167 = (($scanset) + ($166)|0);
               HEAP8[$167>>0] = $$pre$phi213Z2D;
               $168 = HEAP8[$160>>0]|0;
               $169 = $168&255;
               $170 = ($166|0)<($169|0);
               if ($170) {
                $c$027 = $166;
               } else {
                $172 = $168;$p$8 = $160;
                break;
               }
              }
             } else {
              $172 = $161;$p$8 = $160;
             }
            }
           } else {
            $172 = $159;$p$8 = $p$7;
           }
           $171 = $172&255;
           $173 = (($171) + 1)|0;
           $174 = (($scanset) + ($173)|0);
           HEAP8[$174>>0] = $$pre$phi213Z2D;
           $175 = (($p$8) + 1|0);
           $p$7 = $175;
          }
         }
        } while(0);
        $176 = (($width$1) + 1)|0;
        $$4 = $143 ? $176 : 31;
        $177 = ($$size$0|0)==(1);
        $178 = ($alloc$0|0)!=(0);
        L109: do {
         if ($177) {
          if ($178) {
           $179 = $$4 << 2;
           $180 = (_malloc($179)|0);
           $181 = ($180|0)==(0|0);
           if ($181) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $180;
            label = 162;
            break L4;
           } else {
            $wcs$1 = $180;
           }
          } else {
           $wcs$1 = $dest$0;
          }
          HEAP32[$st>>2] = 0;
          HEAP32[$9>>2] = 0;
          $i$0$ph = 0;$k$0$ph = $$4;$wcs$2$ph = $wcs$1;
          L144: while(1) {
           $182 = ($wcs$2$ph|0)==(0|0);
           if ($182) {
            while(1) {
             $185 = HEAP32[$3>>2]|0;
             $196 = HEAP32[$4>>2]|0;
             $197 = ($185>>>0)<($196>>>0);
             if ($197) {
              $184 = (($185) + 1|0);
              HEAP32[$3>>2] = $184;
              $186 = HEAP8[$185>>0]|0;
              $187 = $186&255;
              $189 = $187;
             } else {
              $183 = (___shgetc($f)|0);
              $189 = $183;
             }
             $188 = (($189) + 1)|0;
             $190 = (($scanset) + ($188)|0);
             $191 = HEAP8[$190>>0]|0;
             $192 = ($191<<24>>24)==(0);
             if ($192) {
              $i$0$ph46$lcssa = $i$0$ph;$wcs$2$ph146 = 0;
              break L144;
             }
             $193 = $189&255;
             HEAP8[$0>>0] = $193;
             $194 = (_mbrtowc($wc,$0,1,$st)|0);
             if ((($194|0) == -2)) {
              continue;
             } else if ((($194|0) == -1)) {
              $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = 0;
              label = 162;
              break L4;
             }
             $195 = ($i$0$ph|0)==($k$0$ph|0);
             $or$cond$us = $178 & $195;
             if ($or$cond$us) {
              $i$1$lcssa = $i$0$ph;
              break;
             }
            }
           } else {
            if ($178) {
             $i$0$ph46$us53 = $i$0$ph;
            } else {
             $i$0$ph46 = $i$0$ph;
             label = 98;
             break;
            }
            while(1) {
             while(1) {
              $200 = HEAP32[$3>>2]|0;
              $214 = HEAP32[$4>>2]|0;
              $215 = ($200>>>0)<($214>>>0);
              if ($215) {
               $199 = (($200) + 1|0);
               HEAP32[$3>>2] = $199;
               $201 = HEAP8[$200>>0]|0;
               $202 = $201&255;
               $204 = $202;
              } else {
               $198 = (___shgetc($f)|0);
               $204 = $198;
              }
              $203 = (($204) + 1)|0;
              $205 = (($scanset) + ($203)|0);
              $206 = HEAP8[$205>>0]|0;
              $207 = ($206<<24>>24)==(0);
              if ($207) {
               $i$0$ph46$lcssa = $i$0$ph46$us53;$wcs$2$ph146 = $wcs$2$ph;
               break L144;
              }
              $208 = $204&255;
              HEAP8[$0>>0] = $208;
              $209 = (_mbrtowc($wc,$0,1,$st)|0);
              if ((($209|0) == -1)) {
               $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
               label = 162;
               break L4;
              } else if (!((($209|0) == -2))) {
               break;
              }
             }
             $210 = HEAP32[$wc>>2]|0;
             $211 = (($i$0$ph46$us53) + 1)|0;
             $212 = (($wcs$2$ph) + ($i$0$ph46$us53<<2)|0);
             HEAP32[$212>>2] = $210;
             $213 = ($211|0)==($k$0$ph|0);
             if ($213) {
              $i$1$lcssa = $k$0$ph;
              break;
             } else {
              $i$0$ph46$us53 = $211;
             }
            }
           }
           $factor = $k$0$ph << 1;
           $233 = $factor | 1;
           $234 = $233 << 2;
           $235 = (_realloc($wcs$2$ph,$234)|0);
           $236 = ($235|0)==(0|0);
           if ($236) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
            label = 162;
            break L4;
           }
           $i$0$ph = $i$1$lcssa;$k$0$ph = $233;$wcs$2$ph = $235;
          }
          L168: do {
           if ((label|0) == 98) {
            while(1) {
             label = 0;
             while(1) {
              $216 = HEAP32[$3>>2]|0;
              $217 = HEAP32[$4>>2]|0;
              $218 = ($216>>>0)<($217>>>0);
              if ($218) {
               $219 = (($216) + 1|0);
               HEAP32[$3>>2] = $219;
               $220 = HEAP8[$216>>0]|0;
               $221 = $220&255;
               $224 = $221;
              } else {
               $222 = (___shgetc($f)|0);
               $224 = $222;
              }
              $223 = (($224) + 1)|0;
              $225 = (($scanset) + ($223)|0);
              $226 = HEAP8[$225>>0]|0;
              $227 = ($226<<24>>24)==(0);
              if ($227) {
               $i$0$ph46$lcssa = $i$0$ph46;$wcs$2$ph146 = $wcs$2$ph;
               break L168;
              }
              $228 = $224&255;
              HEAP8[$0>>0] = $228;
              $229 = (_mbrtowc($wc,$0,1,$st)|0);
              if ((($229|0) == -1)) {
               $alloc$1 = 0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
               label = 162;
               break L4;
              } else if (!((($229|0) == -2))) {
               break;
              }
             }
             $230 = HEAP32[$wc>>2]|0;
             $231 = (($i$0$ph46) + 1)|0;
             $232 = (($wcs$2$ph) + ($i$0$ph46<<2)|0);
             HEAP32[$232>>2] = $230;
             $i$0$ph46 = $231;
             label = 98;
            }
           }
          } while(0);
          $237 = (_mbsinit($st)|0);
          $238 = ($237|0)==(0);
          if ($238) {
           $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph146;
           label = 162;
           break L4;
          } else {
           $i$4 = $i$0$ph46$lcssa;$s$2 = 0;$wcs$3 = $wcs$2$ph146;
          }
         } else {
          if ($178) {
           $239 = (_malloc($$4)|0);
           $240 = ($239|0)==(0|0);
           if ($240) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = 0;
            label = 162;
            break L4;
           } else {
            $i$2$ph = 0;$k$1$ph = $$4;$s$1$ph = $239;
           }
           while(1) {
            $i$2 = $i$2$ph;
            while(1) {
             $241 = HEAP32[$3>>2]|0;
             $242 = HEAP32[$4>>2]|0;
             $243 = ($241>>>0)<($242>>>0);
             if ($243) {
              $244 = (($241) + 1|0);
              HEAP32[$3>>2] = $244;
              $245 = HEAP8[$241>>0]|0;
              $246 = $245&255;
              $249 = $246;
             } else {
              $247 = (___shgetc($f)|0);
              $249 = $247;
             }
             $248 = (($249) + 1)|0;
             $250 = (($scanset) + ($248)|0);
             $251 = HEAP8[$250>>0]|0;
             $252 = ($251<<24>>24)==(0);
             if ($252) {
              $i$4 = $i$2;$s$2 = $s$1$ph;$wcs$3 = 0;
              break L109;
             }
             $253 = $249&255;
             $254 = (($i$2) + 1)|0;
             $255 = (($s$1$ph) + ($i$2)|0);
             HEAP8[$255>>0] = $253;
             $256 = ($254|0)==($k$1$ph|0);
             if ($256) {
              break;
             } else {
              $i$2 = $254;
             }
            }
            $factor14 = $k$1$ph << 1;
            $257 = $factor14 | 1;
            $258 = (_realloc($s$1$ph,$257)|0);
            $259 = ($258|0)==(0|0);
            if ($259) {
             $alloc$1 = $alloc$0;$s$5 = $s$1$ph;$wcs$6 = 0;
             label = 162;
             break L4;
            } else {
             $i$2$ph$phi = $k$1$ph;$k$1$ph = $257;$s$1$ph = $258;$i$2$ph = $i$2$ph$phi;
            }
           }
          }
          $260 = ($dest$0|0)==(0|0);
          if ($260) {
           $278 = $140;
           while(1) {
            $276 = HEAP32[$3>>2]|0;
            $277 = ($276>>>0)<($278>>>0);
            if ($277) {
             $279 = (($276) + 1|0);
             HEAP32[$3>>2] = $279;
             $280 = HEAP8[$276>>0]|0;
             $281 = $280&255;
             $284 = $281;
            } else {
             $282 = (___shgetc($f)|0);
             $284 = $282;
            }
            $283 = (($284) + 1)|0;
            $285 = (($scanset) + ($283)|0);
            $286 = HEAP8[$285>>0]|0;
            $287 = ($286<<24>>24)==(0);
            if ($287) {
             $i$4 = 0;$s$2 = 0;$wcs$3 = 0;
             break L109;
            }
            $$pre209 = HEAP32[$4>>2]|0;
            $278 = $$pre209;
           }
          } else {
           $263 = $140;$i$3 = 0;
           while(1) {
            $261 = HEAP32[$3>>2]|0;
            $262 = ($261>>>0)<($263>>>0);
            if ($262) {
             $264 = (($261) + 1|0);
             HEAP32[$3>>2] = $264;
             $265 = HEAP8[$261>>0]|0;
             $266 = $265&255;
             $269 = $266;
            } else {
             $267 = (___shgetc($f)|0);
             $269 = $267;
            }
            $268 = (($269) + 1)|0;
            $270 = (($scanset) + ($268)|0);
            $271 = HEAP8[$270>>0]|0;
            $272 = ($271<<24>>24)==(0);
            if ($272) {
             $i$4 = $i$3;$s$2 = $dest$0;$wcs$3 = 0;
             break L109;
            }
            $273 = $269&255;
            $274 = (($i$3) + 1)|0;
            $275 = (($dest$0) + ($i$3)|0);
            HEAP8[$275>>0] = $273;
            $$pre193 = HEAP32[$4>>2]|0;
            $263 = $$pre193;$i$3 = $274;
           }
          }
         }
        } while(0);
        $288 = HEAP32[$4>>2]|0;
        $289 = ($288|0)==(0|0);
        $$pre208 = HEAP32[$3>>2]|0;
        if ($289) {
         $294 = $$pre208;
        } else {
         $290 = (($$pre208) + -1|0);
         HEAP32[$3>>2] = $290;
         $294 = $290;
        }
        $291 = HEAP32[$5>>2]|0;
        $292 = HEAP32[$6>>2]|0;
        $293 = $294;
        $295 = $292;
        $296 = (($293) - ($295))|0;
        $297 = (($296) + ($291))|0;
        $298 = ($297|0)==(0);
        if ($298) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$2;$wcs$7 = $wcs$3;
         break L4;
        }
        $$not = $143 ^ 1;
        $299 = ($297|0)==($width$1|0);
        $or$cond5 = $299 | $$not;
        if (!($or$cond5)) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$2;$wcs$7 = $wcs$3;
         break L4;
        }
        do {
         if ($178) {
          if ($177) {
           HEAP32[$dest$0>>2] = $wcs$3;
           break;
          } else {
           HEAP32[$dest$0>>2] = $s$2;
           break;
          }
         }
        } while(0);
        if ($143) {
         $p$10 = $p$9;$s$3 = $s$2;$wcs$4 = $wcs$3;
        } else {
         $300 = ($wcs$3|0)==(0|0);
         if (!($300)) {
          $301 = (($wcs$3) + ($i$4<<2)|0);
          HEAP32[$301>>2] = 0;
         }
         $302 = ($s$2|0)==(0|0);
         if ($302) {
          $p$10 = $p$9;$s$3 = 0;$wcs$4 = $wcs$3;
         } else {
          $303 = (($s$2) + ($i$4)|0);
          HEAP8[$303>>0] = 0;
          $p$10 = $p$9;$s$3 = $s$2;$wcs$4 = $wcs$3;
         }
        }
        break;
       }
       default: {
        $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
       }
       }
      } while(0);
      L197: do {
       if ((label|0) == 144) {
        label = 0;
        $304 = (___intscan($f,$base$0,0,-1,-1)|0);
        $305 = tempRet0;
        $306 = HEAP32[$5>>2]|0;
        $307 = HEAP32[$3>>2]|0;
        $308 = HEAP32[$6>>2]|0;
        $309 = $307;
        $310 = $308;
        $311 = (($310) - ($309))|0;
        $312 = ($306|0)==($311|0);
        if ($312) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$0114;$wcs$7 = $wcs$0115;
         break L4;
        }
        $313 = ($$|0)!=(112);
        $314 = ($dest$0|0)==(0|0);
        $or$cond6 = $313 | $314;
        if (!($or$cond6)) {
         $315 = $304;
         HEAP32[$dest$0>>2] = $315;
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
         break;
        }
        if ($314) {
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
        } else {
         switch ($$size$0|0) {
         case -2:  {
          $316 = $304&255;
          HEAP8[$dest$0>>0] = $316;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case -1:  {
          $317 = $304&65535;
          HEAP16[$dest$0>>1] = $317;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 0:  {
          HEAP32[$dest$0>>2] = $304;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 1:  {
          HEAP32[$dest$0>>2] = $304;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 3:  {
          $318 = $dest$0;
          $319 = $318;
          HEAP32[$319>>2] = $304;
          $320 = (($318) + 4)|0;
          $321 = $320;
          HEAP32[$321>>2] = $305;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         default: {
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
         }
         }
        }
       }
      } while(0);
      $332 = HEAP32[$5>>2]|0;
      $333 = HEAP32[$3>>2]|0;
      $334 = HEAP32[$6>>2]|0;
      $335 = $333;
      $336 = $334;
      $337 = (($332) + ($pos$1))|0;
      $338 = (($337) + ($335))|0;
      $339 = (($338) - ($336))|0;
      $not$ = ($dest$0|0)!=(0|0);
      $340 = $not$&1;
      $matches$0$ = (($340) + ($matches$0116))|0;
      $matches$1 = $matches$0$;$p$11 = $p$10;$pos$2 = $339;$s$4 = $s$3;$wcs$5 = $wcs$4;
      break L6;
     }
    } while(0);
    $46 = $43&1;
    $47 = (($p$0122) + ($46)|0);
    ___shlim($f,0);
    $48 = HEAP32[$3>>2]|0;
    $49 = HEAP32[$4>>2]|0;
    $50 = ($48>>>0)<($49>>>0);
    if ($50) {
     $51 = (($48) + 1|0);
     HEAP32[$3>>2] = $51;
     $52 = HEAP8[$48>>0]|0;
     $53 = $52&255;
     $58 = $53;
    } else {
     $54 = (___shgetc($f)|0);
     $58 = $54;
    }
    $55 = HEAP8[$47>>0]|0;
    $56 = $55&255;
    $57 = ($58|0)==($56|0);
    if (!($57)) {
     label = 19;
     break L4;
    }
    $64 = (($pos$0121) + 1)|0;
    $matches$1 = $matches$0116;$p$11 = $47;$pos$2 = $64;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
   } else {
    $p$1 = $p$0122;
    while(1) {
     $16 = (($p$1) + 1|0);
     $17 = HEAP8[$16>>0]|0;
     $18 = $17&255;
     $19 = (_isspace($18)|0);
     $20 = ($19|0)==(0);
     if ($20) {
      break;
     } else {
      $p$1 = $16;
     }
    }
    ___shlim($f,0);
    while(1) {
     $21 = HEAP32[$3>>2]|0;
     $22 = HEAP32[$4>>2]|0;
     $23 = ($21>>>0)<($22>>>0);
     if ($23) {
      $24 = (($21) + 1|0);
      HEAP32[$3>>2] = $24;
      $25 = HEAP8[$21>>0]|0;
      $26 = $25&255;
      $28 = $26;
     } else {
      $27 = (___shgetc($f)|0);
      $28 = $27;
     }
     $29 = (_isspace($28)|0);
     $30 = ($29|0)==(0);
     if ($30) {
      break;
     }
    }
    $31 = HEAP32[$4>>2]|0;
    $32 = ($31|0)==(0|0);
    $$pre210 = HEAP32[$3>>2]|0;
    if ($32) {
     $37 = $$pre210;
    } else {
     $33 = (($$pre210) + -1|0);
     HEAP32[$3>>2] = $33;
     $37 = $33;
    }
    $34 = HEAP32[$5>>2]|0;
    $35 = HEAP32[$6>>2]|0;
    $36 = $37;
    $38 = $35;
    $39 = (($34) + ($pos$0121))|0;
    $40 = (($39) + ($36))|0;
    $41 = (($40) - ($38))|0;
    $matches$1 = $matches$0116;$p$11 = $p$1;$pos$2 = $41;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
   }
  } while(0);
  $341 = (($p$11) + 1|0);
  $342 = HEAP8[$341>>0]|0;
  $343 = ($342<<24>>24)==(0);
  if ($343) {
   $matches$3 = $matches$1;
   label = 165;
   break;
  } else {
   $13 = $342;$matches$0116 = $matches$1;$p$0122 = $341;$pos$0121 = $pos$2;$s$0114 = $s$4;$wcs$0115 = $wcs$5;
  }
 }
 if ((label|0) == 165) {
  STACKTOP = sp;return ($matches$3|0);
 }
 if ((label|0) == 19) {
  $59 = HEAP32[$4>>2]|0;
  $60 = ($59|0)==(0|0);
  if (!($60)) {
   $61 = HEAP32[$3>>2]|0;
   $62 = (($61) + -1|0);
   HEAP32[$3>>2] = $62;
  }
  $63 = ($58|0)<(0);
  if ($63) {
   $alloc$1 = 0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
   label = 162;
  } else {
   $matches$3 = $matches$0116;
   STACKTOP = sp;return ($matches$3|0);
  }
 }
 if ((label|0) == 162) {
  $344 = ($matches$0116|0)==(0);
  $345 = $344 << 31 >> 31;
  $$matches$0 = (($345) + ($matches$0116))|0;
  $alloc$2 = $alloc$1;$matches$2 = $$matches$0;$s$6 = $s$5;$wcs$7 = $wcs$6;
 }
 $346 = ($alloc$2|0)==(0);
 if ($346) {
  $matches$3 = $matches$2;
  STACKTOP = sp;return ($matches$3|0);
 }
 _free($s$6);
 _free($wcs$7);
 $matches$3 = $matches$2;
 STACKTOP = sp;return ($matches$3|0);
}
function _vsscanf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $f = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $f = sp;
 dest=$f+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $0 = (($f) + 32|0);
 HEAP32[$0>>2] = 117;
 $1 = (($f) + 44|0);
 HEAP32[$1>>2] = $s;
 $2 = (($f) + 76|0);
 HEAP32[$2>>2] = -1;
 $3 = (($f) + 84|0);
 HEAP32[$3>>2] = $s;
 $4 = (_MUSL_vfscanf($f,$fmt,$ap)|0);
 STACKTOP = sp;return ($4|0);
}
function _do_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___string_read($f,$buf,$len)|0);
 STACKTOP = sp;return ($0|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$$i = 0, $$3$i = 0, $$4$i = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i25 = 0, $$pre$i25$i = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i26$iZ2D = 0, $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre57$i$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0;
 var $$sum$i14$i = 0, $$sum$i15$i = 0, $$sum$i18$i = 0, $$sum$i21$i = 0, $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i16$i = 0, $$sum1$i22$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum10$pre$i$i = 0, $$sum107$i = 0, $$sum108$i = 0, $$sum109$i = 0;
 var $$sum11$i = 0, $$sum11$i$i = 0, $$sum11$i24$i = 0, $$sum110$i = 0, $$sum111$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0, $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0;
 var $$sum14$pre$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0, $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i17$i = 0, $$sum2$i19$i = 0, $$sum2$i23$i = 0, $$sum2$pre$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0;
 var $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum26$pre$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i$i = 0, $$sum3$i27 = 0, $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0;
 var $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0, $$sum8$pre = 0, $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0;
 var $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0;
 var $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0;
 var $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0;
 var $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0;
 var $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0;
 var $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0;
 var $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0;
 var $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0;
 var $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0;
 var $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0;
 var $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0;
 var $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0;
 var $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0;
 var $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0;
 var $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0;
 var $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0;
 var $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0;
 var $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0;
 var $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0;
 var $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0;
 var $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0;
 var $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0;
 var $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0;
 var $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0;
 var $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0;
 var $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0;
 var $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0;
 var $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0;
 var $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0;
 var $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0;
 var $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0;
 var $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0;
 var $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0;
 var $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0;
 var $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0;
 var $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0;
 var $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0;
 var $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0;
 var $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0;
 var $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0;
 var $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0;
 var $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0;
 var $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0;
 var $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0;
 var $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0;
 var $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0;
 var $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0;
 var $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0;
 var $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0;
 var $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0;
 var $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0;
 var $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0;
 var $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0;
 var $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0, $F5$0$i = 0;
 var $I1$0$c$i$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$025$i = 0, $K2$014$i$i = 0, $K8$052$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i18 = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i17 = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i28$i = 0, $T$013$i$i = 0;
 var $T$024$i = 0, $T$051$i$i = 0, $br$0$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0, $notlhs$i = 0, $notrhs$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i29 = 0, $or$cond1$i = 0, $or$cond10$i = 0, $or$cond19$i = 0, $or$cond2$i = 0;
 var $or$cond49$i = 0, $or$cond5$i = 0, $or$cond6$i = 0, $or$cond8$not$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i15 = 0, $rsize$1$i = 0, $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$329$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$075$i = 0, $sp$168$i = 0, $ssize$0$$i = 0;
 var $ssize$0$i = 0, $ssize$1$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0, $t$2$v$3$i = 0, $t$228$i = 0, $tbase$0$i = 0, $tbase$247$i = 0, $tsize$0$i = 0, $tsize$0323841$i = 0, $tsize$1$i = 0, $tsize$246$i = 0, $v$0$i = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0;
 var $v$330$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   if ($1) {
    $5 = 16;
   } else {
    $2 = (($bytes) + 11)|0;
    $3 = $2 & -8;
    $5 = $3;
   }
   $4 = $5 >>> 3;
   $6 = HEAP32[10360>>2]|0;
   $7 = $6 >>> $4;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($4))|0;
    $13 = $12 << 1;
    $14 = ((10360 + ($13<<2)|0) + 40|0);
    $$sum10 = (($13) + 2)|0;
    $15 = ((10360 + ($$sum10<<2)|0) + 40|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[10360>>2] = $22;
     } else {
      $23 = HEAP32[((10360 + 16|0))>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = (($18) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = (($16) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    STACKTOP = sp;return ($mem$0|0);
   }
   $34 = HEAP32[((10360 + 8|0))>>2]|0;
   $35 = ($5>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $4;
     $38 = 2 << $4;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = ((10360 + ($65<<2)|0) + 40|0);
     $$sum4 = (($65) + 2)|0;
     $67 = ((10360 + ($$sum4<<2)|0) + 40|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[10360>>2] = $74;
      } else {
       $75 = HEAP32[((10360 + 16|0))>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = (($70) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($5))|0;
     $82 = $5 | 3;
     $83 = (($68) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($5)|0);
     $85 = $81 | 1;
     $$sum56 = $5 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $88 = HEAP32[((10360 + 8|0))>>2]|0;
     $89 = ($88|0)==(0);
     if (!($89)) {
      $90 = HEAP32[((10360 + 20|0))>>2]|0;
      $91 = $88 >>> 3;
      $92 = $91 << 1;
      $93 = ((10360 + ($92<<2)|0) + 40|0);
      $94 = HEAP32[10360>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[10360>>2] = $98;
       $$sum8$pre = (($92) + 2)|0;
       $$pre = ((10360 + ($$sum8$pre<<2)|0) + 40|0);
       $$pre$phiZ2D = $$pre;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = ((10360 + ($$sum9<<2)|0) + 40|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[((10360 + 16|0))>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = (($F4$0) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = (($90) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = (($90) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[((10360 + 8|0))>>2] = $81;
     HEAP32[((10360 + 20|0))>>2] = $84;
     $mem$0 = $69;
     STACKTOP = sp;return ($mem$0|0);
    }
    $106 = HEAP32[((10360 + 4|0))>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $5;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = ((10360 + ($130<<2)|0) + 304|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (($132) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($5))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = (($t$0$i) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = (($t$0$i) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = (($144) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($5))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[((10360 + 16|0))>>2]|0;
     $150 = ($v$0$i>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i) + ($5)|0);
     $152 = ($v$0$i>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = (($v$0$i) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = (($v$0$i) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i|0);
     do {
      if ($157) {
       $167 = (($v$0$i) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = (($v$0$i) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = (($R$0$i) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = (($R$0$i) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i>>2] = 0;
        $R$1$i = $R$0$i;
        break;
       }
      } else {
       $158 = (($v$0$i) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = (($159) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = (($156) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = (($v$0$i) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ((10360 + ($182<<2)|0) + 304|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[((10360 + 4|0))>>2]|0;
         $189 = $188 & $187;
         HEAP32[((10360 + 4|0))>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[((10360 + 16|0))>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = (($154) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = (($154) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[((10360 + 16|0))>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = (($R$1$i) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = (($v$0$i) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = HEAP32[((10360 + 16|0))>>2]|0;
         $204 = ($201>>>0)<($203>>>0);
         if ($204) {
          _abort();
          // unreachable;
         } else {
          $205 = (($R$1$i) + 16|0);
          HEAP32[$205>>2] = $201;
          $206 = (($201) + 24|0);
          HEAP32[$206>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $207 = (($v$0$i) + 20|0);
       $208 = HEAP32[$207>>2]|0;
       $209 = ($208|0)==(0|0);
       if (!($209)) {
        $210 = HEAP32[((10360 + 16|0))>>2]|0;
        $211 = ($208>>>0)<($210>>>0);
        if ($211) {
         _abort();
         // unreachable;
        } else {
         $212 = (($R$1$i) + 20|0);
         HEAP32[$212>>2] = $208;
         $213 = (($208) + 24|0);
         HEAP32[$213>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $214 = ($rsize$0$i>>>0)<(16);
     if ($214) {
      $215 = (($rsize$0$i) + ($5))|0;
      $216 = $215 | 3;
      $217 = (($v$0$i) + 4|0);
      HEAP32[$217>>2] = $216;
      $$sum4$i = (($215) + 4)|0;
      $218 = (($v$0$i) + ($$sum4$i)|0);
      $219 = HEAP32[$218>>2]|0;
      $220 = $219 | 1;
      HEAP32[$218>>2] = $220;
     } else {
      $221 = $5 | 3;
      $222 = (($v$0$i) + 4|0);
      HEAP32[$222>>2] = $221;
      $223 = $rsize$0$i | 1;
      $$sum$i35 = $5 | 4;
      $224 = (($v$0$i) + ($$sum$i35)|0);
      HEAP32[$224>>2] = $223;
      $$sum1$i = (($rsize$0$i) + ($5))|0;
      $225 = (($v$0$i) + ($$sum1$i)|0);
      HEAP32[$225>>2] = $rsize$0$i;
      $226 = HEAP32[((10360 + 8|0))>>2]|0;
      $227 = ($226|0)==(0);
      if (!($227)) {
       $228 = HEAP32[((10360 + 20|0))>>2]|0;
       $229 = $226 >>> 3;
       $230 = $229 << 1;
       $231 = ((10360 + ($230<<2)|0) + 40|0);
       $232 = HEAP32[10360>>2]|0;
       $233 = 1 << $229;
       $234 = $232 & $233;
       $235 = ($234|0)==(0);
       if ($235) {
        $236 = $232 | $233;
        HEAP32[10360>>2] = $236;
        $$sum2$pre$i = (($230) + 2)|0;
        $$pre$i = ((10360 + ($$sum2$pre$i<<2)|0) + 40|0);
        $$pre$phi$iZ2D = $$pre$i;$F1$0$i = $231;
       } else {
        $$sum3$i = (($230) + 2)|0;
        $237 = ((10360 + ($$sum3$i<<2)|0) + 40|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = HEAP32[((10360 + 16|0))>>2]|0;
        $240 = ($238>>>0)<($239>>>0);
        if ($240) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $237;$F1$0$i = $238;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $228;
       $241 = (($F1$0$i) + 12|0);
       HEAP32[$241>>2] = $228;
       $242 = (($228) + 8|0);
       HEAP32[$242>>2] = $F1$0$i;
       $243 = (($228) + 12|0);
       HEAP32[$243>>2] = $231;
      }
      HEAP32[((10360 + 8|0))>>2] = $rsize$0$i;
      HEAP32[((10360 + 20|0))>>2] = $151;
     }
     $244 = (($v$0$i) + 8|0);
     $mem$0 = $244;
     STACKTOP = sp;return ($mem$0|0);
    }
   } else {
    $nb$0 = $5;
   }
  } else {
   $245 = ($bytes>>>0)>(4294967231);
   if ($245) {
    $nb$0 = -1;
   } else {
    $246 = (($bytes) + 11)|0;
    $247 = $246 & -8;
    $248 = HEAP32[((10360 + 4|0))>>2]|0;
    $249 = ($248|0)==(0);
    if ($249) {
     $nb$0 = $247;
    } else {
     $250 = (0 - ($247))|0;
     $251 = $246 >>> 8;
     $252 = ($251|0)==(0);
     if ($252) {
      $idx$0$i = 0;
     } else {
      $253 = ($247>>>0)>(16777215);
      if ($253) {
       $idx$0$i = 31;
      } else {
       $254 = (($251) + 1048320)|0;
       $255 = $254 >>> 16;
       $256 = $255 & 8;
       $257 = $251 << $256;
       $258 = (($257) + 520192)|0;
       $259 = $258 >>> 16;
       $260 = $259 & 4;
       $261 = $260 | $256;
       $262 = $257 << $260;
       $263 = (($262) + 245760)|0;
       $264 = $263 >>> 16;
       $265 = $264 & 2;
       $266 = $261 | $265;
       $267 = (14 - ($266))|0;
       $268 = $262 << $265;
       $269 = $268 >>> 15;
       $270 = (($267) + ($269))|0;
       $271 = $270 << 1;
       $272 = (($270) + 7)|0;
       $273 = $247 >>> $272;
       $274 = $273 & 1;
       $275 = $274 | $271;
       $idx$0$i = $275;
      }
     }
     $276 = ((10360 + ($idx$0$i<<2)|0) + 304|0);
     $277 = HEAP32[$276>>2]|0;
     $278 = ($277|0)==(0|0);
     L126: do {
      if ($278) {
       $rsize$2$i = $250;$t$1$i = 0;$v$2$i = 0;
      } else {
       $279 = ($idx$0$i|0)==(31);
       if ($279) {
        $283 = 0;
       } else {
        $280 = $idx$0$i >>> 1;
        $281 = (25 - ($280))|0;
        $283 = $281;
       }
       $282 = $247 << $283;
       $rsize$0$i15 = $250;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $277;$v$0$i16 = 0;
       while(1) {
        $284 = (($t$0$i14) + 4|0);
        $285 = HEAP32[$284>>2]|0;
        $286 = $285 & -8;
        $287 = (($286) - ($247))|0;
        $288 = ($287>>>0)<($rsize$0$i15>>>0);
        if ($288) {
         $289 = ($286|0)==($247|0);
         if ($289) {
          $rsize$2$i = $287;$t$1$i = $t$0$i14;$v$2$i = $t$0$i14;
          break L126;
         } else {
          $rsize$1$i = $287;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $290 = (($t$0$i14) + 20|0);
        $291 = HEAP32[$290>>2]|0;
        $292 = $sizebits$0$i >>> 31;
        $293 = ((($t$0$i14) + ($292<<2)|0) + 16|0);
        $294 = HEAP32[$293>>2]|0;
        $295 = ($291|0)==(0|0);
        $296 = ($291|0)==($294|0);
        $or$cond$i = $295 | $296;
        $rst$1$i = $or$cond$i ? $rst$0$i : $291;
        $297 = ($294|0)==(0|0);
        $298 = $sizebits$0$i << 1;
        if ($297) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $298;$t$0$i14 = $294;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     $299 = ($t$1$i|0)==(0|0);
     $300 = ($v$2$i|0)==(0|0);
     $or$cond19$i = $299 & $300;
     if ($or$cond19$i) {
      $301 = 2 << $idx$0$i;
      $302 = (0 - ($301))|0;
      $303 = $301 | $302;
      $304 = $248 & $303;
      $305 = ($304|0)==(0);
      if ($305) {
       $nb$0 = $247;
       break;
      }
      $306 = (0 - ($304))|0;
      $307 = $304 & $306;
      $308 = (($307) + -1)|0;
      $309 = $308 >>> 12;
      $310 = $309 & 16;
      $311 = $308 >>> $310;
      $312 = $311 >>> 5;
      $313 = $312 & 8;
      $314 = $313 | $310;
      $315 = $311 >>> $313;
      $316 = $315 >>> 2;
      $317 = $316 & 4;
      $318 = $314 | $317;
      $319 = $315 >>> $317;
      $320 = $319 >>> 1;
      $321 = $320 & 2;
      $322 = $318 | $321;
      $323 = $319 >>> $321;
      $324 = $323 >>> 1;
      $325 = $324 & 1;
      $326 = $322 | $325;
      $327 = $323 >>> $325;
      $328 = (($326) + ($327))|0;
      $329 = ((10360 + ($328<<2)|0) + 304|0);
      $330 = HEAP32[$329>>2]|0;
      $t$2$ph$i = $330;
     } else {
      $t$2$ph$i = $t$1$i;
     }
     $331 = ($t$2$ph$i|0)==(0|0);
     if ($331) {
      $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$2$i;
     } else {
      $rsize$329$i = $rsize$2$i;$t$228$i = $t$2$ph$i;$v$330$i = $v$2$i;
      while(1) {
       $332 = (($t$228$i) + 4|0);
       $333 = HEAP32[$332>>2]|0;
       $334 = $333 & -8;
       $335 = (($334) - ($247))|0;
       $336 = ($335>>>0)<($rsize$329$i>>>0);
       $$rsize$3$i = $336 ? $335 : $rsize$329$i;
       $t$2$v$3$i = $336 ? $t$228$i : $v$330$i;
       $337 = (($t$228$i) + 16|0);
       $338 = HEAP32[$337>>2]|0;
       $339 = ($338|0)==(0|0);
       if (!($339)) {
        $rsize$329$i = $$rsize$3$i;$t$228$i = $338;$v$330$i = $t$2$v$3$i;
        continue;
       }
       $340 = (($t$228$i) + 20|0);
       $341 = HEAP32[$340>>2]|0;
       $342 = ($341|0)==(0|0);
       if ($342) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$329$i = $$rsize$3$i;$t$228$i = $341;$v$330$i = $t$2$v$3$i;
       }
      }
     }
     $343 = ($v$3$lcssa$i|0)==(0|0);
     if ($343) {
      $nb$0 = $247;
     } else {
      $344 = HEAP32[((10360 + 8|0))>>2]|0;
      $345 = (($344) - ($247))|0;
      $346 = ($rsize$3$lcssa$i>>>0)<($345>>>0);
      if ($346) {
       $347 = HEAP32[((10360 + 16|0))>>2]|0;
       $348 = ($v$3$lcssa$i>>>0)<($347>>>0);
       if ($348) {
        _abort();
        // unreachable;
       }
       $349 = (($v$3$lcssa$i) + ($247)|0);
       $350 = ($v$3$lcssa$i>>>0)<($349>>>0);
       if (!($350)) {
        _abort();
        // unreachable;
       }
       $351 = (($v$3$lcssa$i) + 24|0);
       $352 = HEAP32[$351>>2]|0;
       $353 = (($v$3$lcssa$i) + 12|0);
       $354 = HEAP32[$353>>2]|0;
       $355 = ($354|0)==($v$3$lcssa$i|0);
       do {
        if ($355) {
         $365 = (($v$3$lcssa$i) + 20|0);
         $366 = HEAP32[$365>>2]|0;
         $367 = ($366|0)==(0|0);
         if ($367) {
          $368 = (($v$3$lcssa$i) + 16|0);
          $369 = HEAP32[$368>>2]|0;
          $370 = ($369|0)==(0|0);
          if ($370) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $369;$RP$0$i17 = $368;
          }
         } else {
          $R$0$i18 = $366;$RP$0$i17 = $365;
         }
         while(1) {
          $371 = (($R$0$i18) + 20|0);
          $372 = HEAP32[$371>>2]|0;
          $373 = ($372|0)==(0|0);
          if (!($373)) {
           $R$0$i18 = $372;$RP$0$i17 = $371;
           continue;
          }
          $374 = (($R$0$i18) + 16|0);
          $375 = HEAP32[$374>>2]|0;
          $376 = ($375|0)==(0|0);
          if ($376) {
           break;
          } else {
           $R$0$i18 = $375;$RP$0$i17 = $374;
          }
         }
         $377 = ($RP$0$i17>>>0)<($347>>>0);
         if ($377) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17>>2] = 0;
          $R$1$i20 = $R$0$i18;
          break;
         }
        } else {
         $356 = (($v$3$lcssa$i) + 8|0);
         $357 = HEAP32[$356>>2]|0;
         $358 = ($357>>>0)<($347>>>0);
         if ($358) {
          _abort();
          // unreachable;
         }
         $359 = (($357) + 12|0);
         $360 = HEAP32[$359>>2]|0;
         $361 = ($360|0)==($v$3$lcssa$i|0);
         if (!($361)) {
          _abort();
          // unreachable;
         }
         $362 = (($354) + 8|0);
         $363 = HEAP32[$362>>2]|0;
         $364 = ($363|0)==($v$3$lcssa$i|0);
         if ($364) {
          HEAP32[$359>>2] = $354;
          HEAP32[$362>>2] = $357;
          $R$1$i20 = $354;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $378 = ($352|0)==(0|0);
       do {
        if (!($378)) {
         $379 = (($v$3$lcssa$i) + 28|0);
         $380 = HEAP32[$379>>2]|0;
         $381 = ((10360 + ($380<<2)|0) + 304|0);
         $382 = HEAP32[$381>>2]|0;
         $383 = ($v$3$lcssa$i|0)==($382|0);
         if ($383) {
          HEAP32[$381>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $384 = 1 << $380;
           $385 = $384 ^ -1;
           $386 = HEAP32[((10360 + 4|0))>>2]|0;
           $387 = $386 & $385;
           HEAP32[((10360 + 4|0))>>2] = $387;
           break;
          }
         } else {
          $388 = HEAP32[((10360 + 16|0))>>2]|0;
          $389 = ($352>>>0)<($388>>>0);
          if ($389) {
           _abort();
           // unreachable;
          }
          $390 = (($352) + 16|0);
          $391 = HEAP32[$390>>2]|0;
          $392 = ($391|0)==($v$3$lcssa$i|0);
          if ($392) {
           HEAP32[$390>>2] = $R$1$i20;
          } else {
           $393 = (($352) + 20|0);
           HEAP32[$393>>2] = $R$1$i20;
          }
          $394 = ($R$1$i20|0)==(0|0);
          if ($394) {
           break;
          }
         }
         $395 = HEAP32[((10360 + 16|0))>>2]|0;
         $396 = ($R$1$i20>>>0)<($395>>>0);
         if ($396) {
          _abort();
          // unreachable;
         }
         $397 = (($R$1$i20) + 24|0);
         HEAP32[$397>>2] = $352;
         $398 = (($v$3$lcssa$i) + 16|0);
         $399 = HEAP32[$398>>2]|0;
         $400 = ($399|0)==(0|0);
         do {
          if (!($400)) {
           $401 = HEAP32[((10360 + 16|0))>>2]|0;
           $402 = ($399>>>0)<($401>>>0);
           if ($402) {
            _abort();
            // unreachable;
           } else {
            $403 = (($R$1$i20) + 16|0);
            HEAP32[$403>>2] = $399;
            $404 = (($399) + 24|0);
            HEAP32[$404>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $405 = (($v$3$lcssa$i) + 20|0);
         $406 = HEAP32[$405>>2]|0;
         $407 = ($406|0)==(0|0);
         if (!($407)) {
          $408 = HEAP32[((10360 + 16|0))>>2]|0;
          $409 = ($406>>>0)<($408>>>0);
          if ($409) {
           _abort();
           // unreachable;
          } else {
           $410 = (($R$1$i20) + 20|0);
           HEAP32[$410>>2] = $406;
           $411 = (($406) + 24|0);
           HEAP32[$411>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $412 = ($rsize$3$lcssa$i>>>0)<(16);
       L204: do {
        if ($412) {
         $413 = (($rsize$3$lcssa$i) + ($247))|0;
         $414 = $413 | 3;
         $415 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$415>>2] = $414;
         $$sum18$i = (($413) + 4)|0;
         $416 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $417 = HEAP32[$416>>2]|0;
         $418 = $417 | 1;
         HEAP32[$416>>2] = $418;
        } else {
         $419 = $247 | 3;
         $420 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$420>>2] = $419;
         $421 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $247 | 4;
         $422 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$422>>2] = $421;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($247))|0;
         $423 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$423>>2] = $rsize$3$lcssa$i;
         $424 = $rsize$3$lcssa$i >>> 3;
         $425 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($425) {
          $426 = $424 << 1;
          $427 = ((10360 + ($426<<2)|0) + 40|0);
          $428 = HEAP32[10360>>2]|0;
          $429 = 1 << $424;
          $430 = $428 & $429;
          $431 = ($430|0)==(0);
          do {
           if ($431) {
            $432 = $428 | $429;
            HEAP32[10360>>2] = $432;
            $$sum14$pre$i = (($426) + 2)|0;
            $$pre$i25 = ((10360 + ($$sum14$pre$i<<2)|0) + 40|0);
            $$pre$phi$i26Z2D = $$pre$i25;$F5$0$i = $427;
           } else {
            $$sum17$i = (($426) + 2)|0;
            $433 = ((10360 + ($$sum17$i<<2)|0) + 40|0);
            $434 = HEAP32[$433>>2]|0;
            $435 = HEAP32[((10360 + 16|0))>>2]|0;
            $436 = ($434>>>0)<($435>>>0);
            if (!($436)) {
             $$pre$phi$i26Z2D = $433;$F5$0$i = $434;
             break;
            }
            _abort();
            // unreachable;
           }
          } while(0);
          HEAP32[$$pre$phi$i26Z2D>>2] = $349;
          $437 = (($F5$0$i) + 12|0);
          HEAP32[$437>>2] = $349;
          $$sum15$i = (($247) + 8)|0;
          $438 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$438>>2] = $F5$0$i;
          $$sum16$i = (($247) + 12)|0;
          $439 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$439>>2] = $427;
          break;
         }
         $440 = $rsize$3$lcssa$i >>> 8;
         $441 = ($440|0)==(0);
         if ($441) {
          $I7$0$i = 0;
         } else {
          $442 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($442) {
           $I7$0$i = 31;
          } else {
           $443 = (($440) + 1048320)|0;
           $444 = $443 >>> 16;
           $445 = $444 & 8;
           $446 = $440 << $445;
           $447 = (($446) + 520192)|0;
           $448 = $447 >>> 16;
           $449 = $448 & 4;
           $450 = $449 | $445;
           $451 = $446 << $449;
           $452 = (($451) + 245760)|0;
           $453 = $452 >>> 16;
           $454 = $453 & 2;
           $455 = $450 | $454;
           $456 = (14 - ($455))|0;
           $457 = $451 << $454;
           $458 = $457 >>> 15;
           $459 = (($456) + ($458))|0;
           $460 = $459 << 1;
           $461 = (($459) + 7)|0;
           $462 = $rsize$3$lcssa$i >>> $461;
           $463 = $462 & 1;
           $464 = $463 | $460;
           $I7$0$i = $464;
          }
         }
         $465 = ((10360 + ($I7$0$i<<2)|0) + 304|0);
         $$sum2$i = (($247) + 28)|0;
         $466 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$466>>2] = $I7$0$i;
         $$sum3$i27 = (($247) + 16)|0;
         $467 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($247) + 20)|0;
         $468 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$468>>2] = 0;
         HEAP32[$467>>2] = 0;
         $469 = HEAP32[((10360 + 4|0))>>2]|0;
         $470 = 1 << $I7$0$i;
         $471 = $469 & $470;
         $472 = ($471|0)==(0);
         if ($472) {
          $473 = $469 | $470;
          HEAP32[((10360 + 4|0))>>2] = $473;
          HEAP32[$465>>2] = $349;
          $$sum5$i = (($247) + 24)|0;
          $474 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$474>>2] = $465;
          $$sum6$i = (($247) + 12)|0;
          $475 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$475>>2] = $349;
          $$sum7$i = (($247) + 8)|0;
          $476 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$476>>2] = $349;
          break;
         }
         $477 = HEAP32[$465>>2]|0;
         $478 = ($I7$0$i|0)==(31);
         if ($478) {
          $486 = 0;
         } else {
          $479 = $I7$0$i >>> 1;
          $480 = (25 - ($479))|0;
          $486 = $480;
         }
         $481 = (($477) + 4|0);
         $482 = HEAP32[$481>>2]|0;
         $483 = $482 & -8;
         $484 = ($483|0)==($rsize$3$lcssa$i|0);
         L224: do {
          if ($484) {
           $T$0$lcssa$i = $477;
          } else {
           $485 = $rsize$3$lcssa$i << $486;
           $K12$025$i = $485;$T$024$i = $477;
           while(1) {
            $493 = $K12$025$i >>> 31;
            $494 = ((($T$024$i) + ($493<<2)|0) + 16|0);
            $489 = HEAP32[$494>>2]|0;
            $495 = ($489|0)==(0|0);
            if ($495) {
             break;
            }
            $487 = $K12$025$i << 1;
            $488 = (($489) + 4|0);
            $490 = HEAP32[$488>>2]|0;
            $491 = $490 & -8;
            $492 = ($491|0)==($rsize$3$lcssa$i|0);
            if ($492) {
             $T$0$lcssa$i = $489;
             break L224;
            } else {
             $K12$025$i = $487;$T$024$i = $489;
            }
           }
           $496 = HEAP32[((10360 + 16|0))>>2]|0;
           $497 = ($494>>>0)<($496>>>0);
           if ($497) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$494>>2] = $349;
            $$sum11$i = (($247) + 24)|0;
            $498 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$498>>2] = $T$024$i;
            $$sum12$i = (($247) + 12)|0;
            $499 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$499>>2] = $349;
            $$sum13$i = (($247) + 8)|0;
            $500 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$500>>2] = $349;
            break L204;
           }
          }
         } while(0);
         $501 = (($T$0$lcssa$i) + 8|0);
         $502 = HEAP32[$501>>2]|0;
         $503 = HEAP32[((10360 + 16|0))>>2]|0;
         $504 = ($T$0$lcssa$i>>>0)<($503>>>0);
         if ($504) {
          _abort();
          // unreachable;
         }
         $505 = ($502>>>0)<($503>>>0);
         if ($505) {
          _abort();
          // unreachable;
         } else {
          $506 = (($502) + 12|0);
          HEAP32[$506>>2] = $349;
          HEAP32[$501>>2] = $349;
          $$sum8$i = (($247) + 8)|0;
          $507 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$507>>2] = $502;
          $$sum9$i = (($247) + 12)|0;
          $508 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$508>>2] = $T$0$lcssa$i;
          $$sum10$i = (($247) + 24)|0;
          $509 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$509>>2] = 0;
          break;
         }
        }
       } while(0);
       $510 = (($v$3$lcssa$i) + 8|0);
       $mem$0 = $510;
       STACKTOP = sp;return ($mem$0|0);
      } else {
       $nb$0 = $247;
      }
     }
    }
   }
  }
 } while(0);
 $511 = HEAP32[((10360 + 8|0))>>2]|0;
 $512 = ($nb$0>>>0)>($511>>>0);
 if (!($512)) {
  $513 = (($511) - ($nb$0))|0;
  $514 = HEAP32[((10360 + 20|0))>>2]|0;
  $515 = ($513>>>0)>(15);
  if ($515) {
   $516 = (($514) + ($nb$0)|0);
   HEAP32[((10360 + 20|0))>>2] = $516;
   HEAP32[((10360 + 8|0))>>2] = $513;
   $517 = $513 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $518 = (($514) + ($$sum2)|0);
   HEAP32[$518>>2] = $517;
   $519 = (($514) + ($511)|0);
   HEAP32[$519>>2] = $513;
   $520 = $nb$0 | 3;
   $521 = (($514) + 4|0);
   HEAP32[$521>>2] = $520;
  } else {
   HEAP32[((10360 + 8|0))>>2] = 0;
   HEAP32[((10360 + 20|0))>>2] = 0;
   $522 = $511 | 3;
   $523 = (($514) + 4|0);
   HEAP32[$523>>2] = $522;
   $$sum1 = (($511) + 4)|0;
   $524 = (($514) + ($$sum1)|0);
   $525 = HEAP32[$524>>2]|0;
   $526 = $525 | 1;
   HEAP32[$524>>2] = $526;
  }
  $527 = (($514) + 8|0);
  $mem$0 = $527;
  STACKTOP = sp;return ($mem$0|0);
 }
 $528 = HEAP32[((10360 + 12|0))>>2]|0;
 $529 = ($nb$0>>>0)<($528>>>0);
 if ($529) {
  $530 = (($528) - ($nb$0))|0;
  HEAP32[((10360 + 12|0))>>2] = $530;
  $531 = HEAP32[((10360 + 24|0))>>2]|0;
  $532 = (($531) + ($nb$0)|0);
  HEAP32[((10360 + 24|0))>>2] = $532;
  $533 = $530 | 1;
  $$sum = (($nb$0) + 4)|0;
  $534 = (($531) + ($$sum)|0);
  HEAP32[$534>>2] = $533;
  $535 = $nb$0 | 3;
  $536 = (($531) + 4|0);
  HEAP32[$536>>2] = $535;
  $537 = (($531) + 8|0);
  $mem$0 = $537;
  STACKTOP = sp;return ($mem$0|0);
 }
 $538 = HEAP32[10832>>2]|0;
 $539 = ($538|0)==(0);
 do {
  if ($539) {
   $540 = (_sysconf(30)|0);
   $541 = (($540) + -1)|0;
   $542 = $541 & $540;
   $543 = ($542|0)==(0);
   if ($543) {
    HEAP32[((10832 + 8|0))>>2] = $540;
    HEAP32[((10832 + 4|0))>>2] = $540;
    HEAP32[((10832 + 12|0))>>2] = -1;
    HEAP32[((10832 + 16|0))>>2] = -1;
    HEAP32[((10832 + 20|0))>>2] = 0;
    HEAP32[((10360 + 444|0))>>2] = 0;
    $544 = (_time((0|0))|0);
    $545 = $544 & -16;
    $546 = $545 ^ 1431655768;
    HEAP32[10832>>2] = $546;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $547 = (($nb$0) + 48)|0;
 $548 = HEAP32[((10832 + 8|0))>>2]|0;
 $549 = (($nb$0) + 47)|0;
 $550 = (($548) + ($549))|0;
 $551 = (0 - ($548))|0;
 $552 = $550 & $551;
 $553 = ($552>>>0)>($nb$0>>>0);
 if (!($553)) {
  $mem$0 = 0;
  STACKTOP = sp;return ($mem$0|0);
 }
 $554 = HEAP32[((10360 + 440|0))>>2]|0;
 $555 = ($554|0)==(0);
 if (!($555)) {
  $556 = HEAP32[((10360 + 432|0))>>2]|0;
  $557 = (($556) + ($552))|0;
  $558 = ($557>>>0)<=($556>>>0);
  $559 = ($557>>>0)>($554>>>0);
  $or$cond1$i = $558 | $559;
  if ($or$cond1$i) {
   $mem$0 = 0;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $560 = HEAP32[((10360 + 444|0))>>2]|0;
 $561 = $560 & 4;
 $562 = ($561|0)==(0);
 L269: do {
  if ($562) {
   $563 = HEAP32[((10360 + 24|0))>>2]|0;
   $564 = ($563|0)==(0|0);
   L271: do {
    if ($564) {
     label = 182;
    } else {
     $sp$0$i$i = ((10360 + 448|0));
     while(1) {
      $565 = HEAP32[$sp$0$i$i>>2]|0;
      $566 = ($565>>>0)>($563>>>0);
      if (!($566)) {
       $567 = (($sp$0$i$i) + 4|0);
       $568 = HEAP32[$567>>2]|0;
       $569 = (($565) + ($568)|0);
       $570 = ($569>>>0)>($563>>>0);
       if ($570) {
        break;
       }
      }
      $571 = (($sp$0$i$i) + 8|0);
      $572 = HEAP32[$571>>2]|0;
      $573 = ($572|0)==(0|0);
      if ($573) {
       label = 182;
       break L271;
      } else {
       $sp$0$i$i = $572;
      }
     }
     $574 = ($sp$0$i$i|0)==(0|0);
     if ($574) {
      label = 182;
     } else {
      $597 = HEAP32[((10360 + 12|0))>>2]|0;
      $598 = (($550) - ($597))|0;
      $599 = $598 & $551;
      $600 = ($599>>>0)<(2147483647);
      if ($600) {
       $601 = (_sbrk(($599|0))|0);
       $602 = HEAP32[$sp$0$i$i>>2]|0;
       $603 = HEAP32[$567>>2]|0;
       $604 = (($602) + ($603)|0);
       $605 = ($601|0)==($604|0);
       $$3$i = $605 ? $599 : 0;
       $$4$i = $605 ? $601 : (-1);
       $br$0$i = $601;$ssize$1$i = $599;$tbase$0$i = $$4$i;$tsize$0$i = $$3$i;
       label = 191;
      } else {
       $tsize$0323841$i = 0;
      }
     }
    }
   } while(0);
   do {
    if ((label|0) == 182) {
     $575 = (_sbrk(0)|0);
     $576 = ($575|0)==((-1)|0);
     if ($576) {
      $tsize$0323841$i = 0;
     } else {
      $577 = $575;
      $578 = HEAP32[((10832 + 4|0))>>2]|0;
      $579 = (($578) + -1)|0;
      $580 = $579 & $577;
      $581 = ($580|0)==(0);
      if ($581) {
       $ssize$0$i = $552;
      } else {
       $582 = (($579) + ($577))|0;
       $583 = (0 - ($578))|0;
       $584 = $582 & $583;
       $585 = (($552) - ($577))|0;
       $586 = (($585) + ($584))|0;
       $ssize$0$i = $586;
      }
      $587 = HEAP32[((10360 + 432|0))>>2]|0;
      $588 = (($587) + ($ssize$0$i))|0;
      $589 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $590 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i29 = $589 & $590;
      if ($or$cond$i29) {
       $591 = HEAP32[((10360 + 440|0))>>2]|0;
       $592 = ($591|0)==(0);
       if (!($592)) {
        $593 = ($588>>>0)<=($587>>>0);
        $594 = ($588>>>0)>($591>>>0);
        $or$cond2$i = $593 | $594;
        if ($or$cond2$i) {
         $tsize$0323841$i = 0;
         break;
        }
       }
       $595 = (_sbrk(($ssize$0$i|0))|0);
       $596 = ($595|0)==($575|0);
       $ssize$0$$i = $596 ? $ssize$0$i : 0;
       $$$i = $596 ? $575 : (-1);
       $br$0$i = $595;$ssize$1$i = $ssize$0$i;$tbase$0$i = $$$i;$tsize$0$i = $ssize$0$$i;
       label = 191;
      } else {
       $tsize$0323841$i = 0;
      }
     }
    }
   } while(0);
   L291: do {
    if ((label|0) == 191) {
     $606 = (0 - ($ssize$1$i))|0;
     $607 = ($tbase$0$i|0)==((-1)|0);
     if (!($607)) {
      $tbase$247$i = $tbase$0$i;$tsize$246$i = $tsize$0$i;
      label = 202;
      break L269;
     }
     $608 = ($br$0$i|0)!=((-1)|0);
     $609 = ($ssize$1$i>>>0)<(2147483647);
     $or$cond5$i = $608 & $609;
     $610 = ($ssize$1$i>>>0)<($547>>>0);
     $or$cond6$i = $or$cond5$i & $610;
     do {
      if ($or$cond6$i) {
       $611 = HEAP32[((10832 + 8|0))>>2]|0;
       $612 = (($549) - ($ssize$1$i))|0;
       $613 = (($612) + ($611))|0;
       $614 = (0 - ($611))|0;
       $615 = $613 & $614;
       $616 = ($615>>>0)<(2147483647);
       if ($616) {
        $617 = (_sbrk(($615|0))|0);
        $618 = ($617|0)==((-1)|0);
        if ($618) {
         (_sbrk(($606|0))|0);
         $tsize$0323841$i = $tsize$0$i;
         break L291;
        } else {
         $619 = (($615) + ($ssize$1$i))|0;
         $ssize$2$i = $619;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$i;
       }
      } else {
       $ssize$2$i = $ssize$1$i;
      }
     } while(0);
     $620 = ($br$0$i|0)==((-1)|0);
     if ($620) {
      $tsize$0323841$i = $tsize$0$i;
     } else {
      $tbase$247$i = $br$0$i;$tsize$246$i = $ssize$2$i;
      label = 202;
      break L269;
     }
    }
   } while(0);
   $621 = HEAP32[((10360 + 444|0))>>2]|0;
   $622 = $621 | 4;
   HEAP32[((10360 + 444|0))>>2] = $622;
   $tsize$1$i = $tsize$0323841$i;
   label = 199;
  } else {
   $tsize$1$i = 0;
   label = 199;
  }
 } while(0);
 if ((label|0) == 199) {
  $623 = ($552>>>0)<(2147483647);
  if ($623) {
   $624 = (_sbrk(($552|0))|0);
   $625 = (_sbrk(0)|0);
   $notlhs$i = ($624|0)!=((-1)|0);
   $notrhs$i = ($625|0)!=((-1)|0);
   $or$cond8$not$i = $notrhs$i & $notlhs$i;
   $626 = ($624>>>0)<($625>>>0);
   $or$cond9$i = $or$cond8$not$i & $626;
   if ($or$cond9$i) {
    $627 = $625;
    $628 = $624;
    $629 = (($627) - ($628))|0;
    $630 = (($nb$0) + 40)|0;
    $631 = ($629>>>0)>($630>>>0);
    $$tsize$1$i = $631 ? $629 : $tsize$1$i;
    if ($631) {
     $tbase$247$i = $624;$tsize$246$i = $$tsize$1$i;
     label = 202;
    }
   }
  }
 }
 if ((label|0) == 202) {
  $632 = HEAP32[((10360 + 432|0))>>2]|0;
  $633 = (($632) + ($tsize$246$i))|0;
  HEAP32[((10360 + 432|0))>>2] = $633;
  $634 = HEAP32[((10360 + 436|0))>>2]|0;
  $635 = ($633>>>0)>($634>>>0);
  if ($635) {
   HEAP32[((10360 + 436|0))>>2] = $633;
  }
  $636 = HEAP32[((10360 + 24|0))>>2]|0;
  $637 = ($636|0)==(0|0);
  L311: do {
   if ($637) {
    $638 = HEAP32[((10360 + 16|0))>>2]|0;
    $639 = ($638|0)==(0|0);
    $640 = ($tbase$247$i>>>0)<($638>>>0);
    $or$cond10$i = $639 | $640;
    if ($or$cond10$i) {
     HEAP32[((10360 + 16|0))>>2] = $tbase$247$i;
    }
    HEAP32[((10360 + 448|0))>>2] = $tbase$247$i;
    HEAP32[((10360 + 452|0))>>2] = $tsize$246$i;
    HEAP32[((10360 + 460|0))>>2] = 0;
    $641 = HEAP32[10832>>2]|0;
    HEAP32[((10360 + 36|0))>>2] = $641;
    HEAP32[((10360 + 32|0))>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $642 = $i$02$i$i << 1;
     $643 = ((10360 + ($642<<2)|0) + 40|0);
     $$sum$i$i = (($642) + 3)|0;
     $644 = ((10360 + ($$sum$i$i<<2)|0) + 40|0);
     HEAP32[$644>>2] = $643;
     $$sum1$i$i = (($642) + 2)|0;
     $645 = ((10360 + ($$sum1$i$i<<2)|0) + 40|0);
     HEAP32[$645>>2] = $643;
     $646 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($646|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $646;
     }
    }
    $647 = (($tsize$246$i) + -40)|0;
    $648 = (($tbase$247$i) + 8|0);
    $649 = $648;
    $650 = $649 & 7;
    $651 = ($650|0)==(0);
    if ($651) {
     $655 = 0;
    } else {
     $652 = (0 - ($649))|0;
     $653 = $652 & 7;
     $655 = $653;
    }
    $654 = (($tbase$247$i) + ($655)|0);
    $656 = (($647) - ($655))|0;
    HEAP32[((10360 + 24|0))>>2] = $654;
    HEAP32[((10360 + 12|0))>>2] = $656;
    $657 = $656 | 1;
    $$sum$i14$i = (($655) + 4)|0;
    $658 = (($tbase$247$i) + ($$sum$i14$i)|0);
    HEAP32[$658>>2] = $657;
    $$sum2$i$i = (($tsize$246$i) + -36)|0;
    $659 = (($tbase$247$i) + ($$sum2$i$i)|0);
    HEAP32[$659>>2] = 40;
    $660 = HEAP32[((10832 + 16|0))>>2]|0;
    HEAP32[((10360 + 28|0))>>2] = $660;
   } else {
    $sp$075$i = ((10360 + 448|0));
    while(1) {
     $661 = HEAP32[$sp$075$i>>2]|0;
     $662 = (($sp$075$i) + 4|0);
     $663 = HEAP32[$662>>2]|0;
     $664 = (($661) + ($663)|0);
     $665 = ($tbase$247$i|0)==($664|0);
     if ($665) {
      label = 214;
      break;
     }
     $666 = (($sp$075$i) + 8|0);
     $667 = HEAP32[$666>>2]|0;
     $668 = ($667|0)==(0|0);
     if ($668) {
      break;
     } else {
      $sp$075$i = $667;
     }
    }
    if ((label|0) == 214) {
     $669 = (($sp$075$i) + 12|0);
     $670 = HEAP32[$669>>2]|0;
     $671 = $670 & 8;
     $672 = ($671|0)==(0);
     if ($672) {
      $673 = ($636>>>0)>=($661>>>0);
      $674 = ($636>>>0)<($tbase$247$i>>>0);
      $or$cond49$i = $673 & $674;
      if ($or$cond49$i) {
       $675 = (($663) + ($tsize$246$i))|0;
       HEAP32[$662>>2] = $675;
       $676 = HEAP32[((10360 + 12|0))>>2]|0;
       $677 = (($676) + ($tsize$246$i))|0;
       $678 = (($636) + 8|0);
       $679 = $678;
       $680 = $679 & 7;
       $681 = ($680|0)==(0);
       if ($681) {
        $685 = 0;
       } else {
        $682 = (0 - ($679))|0;
        $683 = $682 & 7;
        $685 = $683;
       }
       $684 = (($636) + ($685)|0);
       $686 = (($677) - ($685))|0;
       HEAP32[((10360 + 24|0))>>2] = $684;
       HEAP32[((10360 + 12|0))>>2] = $686;
       $687 = $686 | 1;
       $$sum$i18$i = (($685) + 4)|0;
       $688 = (($636) + ($$sum$i18$i)|0);
       HEAP32[$688>>2] = $687;
       $$sum2$i19$i = (($677) + 4)|0;
       $689 = (($636) + ($$sum2$i19$i)|0);
       HEAP32[$689>>2] = 40;
       $690 = HEAP32[((10832 + 16|0))>>2]|0;
       HEAP32[((10360 + 28|0))>>2] = $690;
       break;
      }
     }
    }
    $691 = HEAP32[((10360 + 16|0))>>2]|0;
    $692 = ($tbase$247$i>>>0)<($691>>>0);
    if ($692) {
     HEAP32[((10360 + 16|0))>>2] = $tbase$247$i;
    }
    $693 = (($tbase$247$i) + ($tsize$246$i)|0);
    $sp$168$i = ((10360 + 448|0));
    while(1) {
     $694 = HEAP32[$sp$168$i>>2]|0;
     $695 = ($694|0)==($693|0);
     if ($695) {
      label = 224;
      break;
     }
     $696 = (($sp$168$i) + 8|0);
     $697 = HEAP32[$696>>2]|0;
     $698 = ($697|0)==(0|0);
     if ($698) {
      break;
     } else {
      $sp$168$i = $697;
     }
    }
    if ((label|0) == 224) {
     $699 = (($sp$168$i) + 12|0);
     $700 = HEAP32[$699>>2]|0;
     $701 = $700 & 8;
     $702 = ($701|0)==(0);
     if ($702) {
      HEAP32[$sp$168$i>>2] = $tbase$247$i;
      $703 = (($sp$168$i) + 4|0);
      $704 = HEAP32[$703>>2]|0;
      $705 = (($704) + ($tsize$246$i))|0;
      HEAP32[$703>>2] = $705;
      $706 = (($tbase$247$i) + 8|0);
      $707 = $706;
      $708 = $707 & 7;
      $709 = ($708|0)==(0);
      if ($709) {
       $713 = 0;
      } else {
       $710 = (0 - ($707))|0;
       $711 = $710 & 7;
       $713 = $711;
      }
      $712 = (($tbase$247$i) + ($713)|0);
      $$sum107$i = (($tsize$246$i) + 8)|0;
      $714 = (($tbase$247$i) + ($$sum107$i)|0);
      $715 = $714;
      $716 = $715 & 7;
      $717 = ($716|0)==(0);
      if ($717) {
       $720 = 0;
      } else {
       $718 = (0 - ($715))|0;
       $719 = $718 & 7;
       $720 = $719;
      }
      $$sum108$i = (($720) + ($tsize$246$i))|0;
      $721 = (($tbase$247$i) + ($$sum108$i)|0);
      $722 = $721;
      $723 = $712;
      $724 = (($722) - ($723))|0;
      $$sum$i21$i = (($713) + ($nb$0))|0;
      $725 = (($tbase$247$i) + ($$sum$i21$i)|0);
      $726 = (($724) - ($nb$0))|0;
      $727 = $nb$0 | 3;
      $$sum1$i22$i = (($713) + 4)|0;
      $728 = (($tbase$247$i) + ($$sum1$i22$i)|0);
      HEAP32[$728>>2] = $727;
      $729 = HEAP32[((10360 + 24|0))>>2]|0;
      $730 = ($721|0)==($729|0);
      L338: do {
       if ($730) {
        $731 = HEAP32[((10360 + 12|0))>>2]|0;
        $732 = (($731) + ($726))|0;
        HEAP32[((10360 + 12|0))>>2] = $732;
        HEAP32[((10360 + 24|0))>>2] = $725;
        $733 = $732 | 1;
        $$sum42$i$i = (($$sum$i21$i) + 4)|0;
        $734 = (($tbase$247$i) + ($$sum42$i$i)|0);
        HEAP32[$734>>2] = $733;
       } else {
        $735 = HEAP32[((10360 + 20|0))>>2]|0;
        $736 = ($721|0)==($735|0);
        if ($736) {
         $737 = HEAP32[((10360 + 8|0))>>2]|0;
         $738 = (($737) + ($726))|0;
         HEAP32[((10360 + 8|0))>>2] = $738;
         HEAP32[((10360 + 20|0))>>2] = $725;
         $739 = $738 | 1;
         $$sum40$i$i = (($$sum$i21$i) + 4)|0;
         $740 = (($tbase$247$i) + ($$sum40$i$i)|0);
         HEAP32[$740>>2] = $739;
         $$sum41$i$i = (($738) + ($$sum$i21$i))|0;
         $741 = (($tbase$247$i) + ($$sum41$i$i)|0);
         HEAP32[$741>>2] = $738;
         break;
        }
        $$sum2$i23$i = (($tsize$246$i) + 4)|0;
        $$sum109$i = (($$sum2$i23$i) + ($720))|0;
        $742 = (($tbase$247$i) + ($$sum109$i)|0);
        $743 = HEAP32[$742>>2]|0;
        $744 = $743 & 3;
        $745 = ($744|0)==(1);
        if ($745) {
         $746 = $743 & -8;
         $747 = $743 >>> 3;
         $748 = ($743>>>0)<(256);
         L346: do {
          if ($748) {
           $$sum3738$i$i = $720 | 8;
           $$sum119$i = (($$sum3738$i$i) + ($tsize$246$i))|0;
           $749 = (($tbase$247$i) + ($$sum119$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $$sum39$i$i = (($tsize$246$i) + 12)|0;
           $$sum120$i = (($$sum39$i$i) + ($720))|0;
           $751 = (($tbase$247$i) + ($$sum120$i)|0);
           $752 = HEAP32[$751>>2]|0;
           $753 = $747 << 1;
           $754 = ((10360 + ($753<<2)|0) + 40|0);
           $755 = ($750|0)==($754|0);
           do {
            if (!($755)) {
             $756 = HEAP32[((10360 + 16|0))>>2]|0;
             $757 = ($750>>>0)<($756>>>0);
             if ($757) {
              _abort();
              // unreachable;
             }
             $758 = (($750) + 12|0);
             $759 = HEAP32[$758>>2]|0;
             $760 = ($759|0)==($721|0);
             if ($760) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $761 = ($752|0)==($750|0);
           if ($761) {
            $762 = 1 << $747;
            $763 = $762 ^ -1;
            $764 = HEAP32[10360>>2]|0;
            $765 = $764 & $763;
            HEAP32[10360>>2] = $765;
            break;
           }
           $766 = ($752|0)==($754|0);
           do {
            if ($766) {
             $$pre57$i$i = (($752) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $767 = HEAP32[((10360 + 16|0))>>2]|0;
             $768 = ($752>>>0)<($767>>>0);
             if ($768) {
              _abort();
              // unreachable;
             }
             $769 = (($752) + 8|0);
             $770 = HEAP32[$769>>2]|0;
             $771 = ($770|0)==($721|0);
             if ($771) {
              $$pre$phi58$i$iZ2D = $769;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $772 = (($750) + 12|0);
           HEAP32[$772>>2] = $752;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $750;
          } else {
           $$sum34$i$i = $720 | 24;
           $$sum110$i = (($$sum34$i$i) + ($tsize$246$i))|0;
           $773 = (($tbase$247$i) + ($$sum110$i)|0);
           $774 = HEAP32[$773>>2]|0;
           $$sum5$i$i = (($tsize$246$i) + 12)|0;
           $$sum111$i = (($$sum5$i$i) + ($720))|0;
           $775 = (($tbase$247$i) + ($$sum111$i)|0);
           $776 = HEAP32[$775>>2]|0;
           $777 = ($776|0)==($721|0);
           do {
            if ($777) {
             $$sum67$i$i = $720 | 16;
             $$sum117$i = (($$sum2$i23$i) + ($$sum67$i$i))|0;
             $788 = (($tbase$247$i) + ($$sum117$i)|0);
             $789 = HEAP32[$788>>2]|0;
             $790 = ($789|0)==(0|0);
             if ($790) {
              $$sum118$i = (($$sum67$i$i) + ($tsize$246$i))|0;
              $791 = (($tbase$247$i) + ($$sum118$i)|0);
              $792 = HEAP32[$791>>2]|0;
              $793 = ($792|0)==(0|0);
              if ($793) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $792;$RP$0$i$i = $791;
              }
             } else {
              $R$0$i$i = $789;$RP$0$i$i = $788;
             }
             while(1) {
              $794 = (($R$0$i$i) + 20|0);
              $795 = HEAP32[$794>>2]|0;
              $796 = ($795|0)==(0|0);
              if (!($796)) {
               $R$0$i$i = $795;$RP$0$i$i = $794;
               continue;
              }
              $797 = (($R$0$i$i) + 16|0);
              $798 = HEAP32[$797>>2]|0;
              $799 = ($798|0)==(0|0);
              if ($799) {
               break;
              } else {
               $R$0$i$i = $798;$RP$0$i$i = $797;
              }
             }
             $800 = HEAP32[((10360 + 16|0))>>2]|0;
             $801 = ($RP$0$i$i>>>0)<($800>>>0);
             if ($801) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i>>2] = 0;
              $R$1$i$i = $R$0$i$i;
              break;
             }
            } else {
             $$sum3536$i$i = $720 | 8;
             $$sum112$i = (($$sum3536$i$i) + ($tsize$246$i))|0;
             $778 = (($tbase$247$i) + ($$sum112$i)|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = HEAP32[((10360 + 16|0))>>2]|0;
             $781 = ($779>>>0)<($780>>>0);
             if ($781) {
              _abort();
              // unreachable;
             }
             $782 = (($779) + 12|0);
             $783 = HEAP32[$782>>2]|0;
             $784 = ($783|0)==($721|0);
             if (!($784)) {
              _abort();
              // unreachable;
             }
             $785 = (($776) + 8|0);
             $786 = HEAP32[$785>>2]|0;
             $787 = ($786|0)==($721|0);
             if ($787) {
              HEAP32[$782>>2] = $776;
              HEAP32[$785>>2] = $779;
              $R$1$i$i = $776;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $802 = ($774|0)==(0|0);
           if ($802) {
            break;
           }
           $$sum30$i$i = (($tsize$246$i) + 28)|0;
           $$sum113$i = (($$sum30$i$i) + ($720))|0;
           $803 = (($tbase$247$i) + ($$sum113$i)|0);
           $804 = HEAP32[$803>>2]|0;
           $805 = ((10360 + ($804<<2)|0) + 304|0);
           $806 = HEAP32[$805>>2]|0;
           $807 = ($721|0)==($806|0);
           do {
            if ($807) {
             HEAP32[$805>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $808 = 1 << $804;
             $809 = $808 ^ -1;
             $810 = HEAP32[((10360 + 4|0))>>2]|0;
             $811 = $810 & $809;
             HEAP32[((10360 + 4|0))>>2] = $811;
             break L346;
            } else {
             $812 = HEAP32[((10360 + 16|0))>>2]|0;
             $813 = ($774>>>0)<($812>>>0);
             if ($813) {
              _abort();
              // unreachable;
             }
             $814 = (($774) + 16|0);
             $815 = HEAP32[$814>>2]|0;
             $816 = ($815|0)==($721|0);
             if ($816) {
              HEAP32[$814>>2] = $R$1$i$i;
             } else {
              $817 = (($774) + 20|0);
              HEAP32[$817>>2] = $R$1$i$i;
             }
             $818 = ($R$1$i$i|0)==(0|0);
             if ($818) {
              break L346;
             }
            }
           } while(0);
           $819 = HEAP32[((10360 + 16|0))>>2]|0;
           $820 = ($R$1$i$i>>>0)<($819>>>0);
           if ($820) {
            _abort();
            // unreachable;
           }
           $821 = (($R$1$i$i) + 24|0);
           HEAP32[$821>>2] = $774;
           $$sum3132$i$i = $720 | 16;
           $$sum114$i = (($$sum3132$i$i) + ($tsize$246$i))|0;
           $822 = (($tbase$247$i) + ($$sum114$i)|0);
           $823 = HEAP32[$822>>2]|0;
           $824 = ($823|0)==(0|0);
           do {
            if (!($824)) {
             $825 = HEAP32[((10360 + 16|0))>>2]|0;
             $826 = ($823>>>0)<($825>>>0);
             if ($826) {
              _abort();
              // unreachable;
             } else {
              $827 = (($R$1$i$i) + 16|0);
              HEAP32[$827>>2] = $823;
              $828 = (($823) + 24|0);
              HEAP32[$828>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum115$i = (($$sum2$i23$i) + ($$sum3132$i$i))|0;
           $829 = (($tbase$247$i) + ($$sum115$i)|0);
           $830 = HEAP32[$829>>2]|0;
           $831 = ($830|0)==(0|0);
           if ($831) {
            break;
           }
           $832 = HEAP32[((10360 + 16|0))>>2]|0;
           $833 = ($830>>>0)<($832>>>0);
           if ($833) {
            _abort();
            // unreachable;
           } else {
            $834 = (($R$1$i$i) + 20|0);
            HEAP32[$834>>2] = $830;
            $835 = (($830) + 24|0);
            HEAP32[$835>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $746 | $720;
         $$sum116$i = (($$sum9$i$i) + ($tsize$246$i))|0;
         $836 = (($tbase$247$i) + ($$sum116$i)|0);
         $837 = (($746) + ($726))|0;
         $oldfirst$0$i$i = $836;$qsize$0$i$i = $837;
        } else {
         $oldfirst$0$i$i = $721;$qsize$0$i$i = $726;
        }
        $838 = (($oldfirst$0$i$i) + 4|0);
        $839 = HEAP32[$838>>2]|0;
        $840 = $839 & -2;
        HEAP32[$838>>2] = $840;
        $841 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i21$i) + 4)|0;
        $842 = (($tbase$247$i) + ($$sum10$i$i)|0);
        HEAP32[$842>>2] = $841;
        $$sum11$i24$i = (($qsize$0$i$i) + ($$sum$i21$i))|0;
        $843 = (($tbase$247$i) + ($$sum11$i24$i)|0);
        HEAP32[$843>>2] = $qsize$0$i$i;
        $844 = $qsize$0$i$i >>> 3;
        $845 = ($qsize$0$i$i>>>0)<(256);
        if ($845) {
         $846 = $844 << 1;
         $847 = ((10360 + ($846<<2)|0) + 40|0);
         $848 = HEAP32[10360>>2]|0;
         $849 = 1 << $844;
         $850 = $848 & $849;
         $851 = ($850|0)==(0);
         do {
          if ($851) {
           $852 = $848 | $849;
           HEAP32[10360>>2] = $852;
           $$sum26$pre$i$i = (($846) + 2)|0;
           $$pre$i25$i = ((10360 + ($$sum26$pre$i$i<<2)|0) + 40|0);
           $$pre$phi$i26$iZ2D = $$pre$i25$i;$F4$0$i$i = $847;
          } else {
           $$sum29$i$i = (($846) + 2)|0;
           $853 = ((10360 + ($$sum29$i$i<<2)|0) + 40|0);
           $854 = HEAP32[$853>>2]|0;
           $855 = HEAP32[((10360 + 16|0))>>2]|0;
           $856 = ($854>>>0)<($855>>>0);
           if (!($856)) {
            $$pre$phi$i26$iZ2D = $853;$F4$0$i$i = $854;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i26$iZ2D>>2] = $725;
         $857 = (($F4$0$i$i) + 12|0);
         HEAP32[$857>>2] = $725;
         $$sum27$i$i = (($$sum$i21$i) + 8)|0;
         $858 = (($tbase$247$i) + ($$sum27$i$i)|0);
         HEAP32[$858>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i21$i) + 12)|0;
         $859 = (($tbase$247$i) + ($$sum28$i$i)|0);
         HEAP32[$859>>2] = $847;
         break;
        }
        $860 = $qsize$0$i$i >>> 8;
        $861 = ($860|0)==(0);
        do {
         if ($861) {
          $I7$0$i$i = 0;
         } else {
          $862 = ($qsize$0$i$i>>>0)>(16777215);
          if ($862) {
           $I7$0$i$i = 31;
           break;
          }
          $863 = (($860) + 1048320)|0;
          $864 = $863 >>> 16;
          $865 = $864 & 8;
          $866 = $860 << $865;
          $867 = (($866) + 520192)|0;
          $868 = $867 >>> 16;
          $869 = $868 & 4;
          $870 = $869 | $865;
          $871 = $866 << $869;
          $872 = (($871) + 245760)|0;
          $873 = $872 >>> 16;
          $874 = $873 & 2;
          $875 = $870 | $874;
          $876 = (14 - ($875))|0;
          $877 = $871 << $874;
          $878 = $877 >>> 15;
          $879 = (($876) + ($878))|0;
          $880 = $879 << 1;
          $881 = (($879) + 7)|0;
          $882 = $qsize$0$i$i >>> $881;
          $883 = $882 & 1;
          $884 = $883 | $880;
          $I7$0$i$i = $884;
         }
        } while(0);
        $885 = ((10360 + ($I7$0$i$i<<2)|0) + 304|0);
        $$sum12$i$i = (($$sum$i21$i) + 28)|0;
        $886 = (($tbase$247$i) + ($$sum12$i$i)|0);
        HEAP32[$886>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i21$i) + 16)|0;
        $887 = (($tbase$247$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i21$i) + 20)|0;
        $888 = (($tbase$247$i) + ($$sum14$i$i)|0);
        HEAP32[$888>>2] = 0;
        HEAP32[$887>>2] = 0;
        $889 = HEAP32[((10360 + 4|0))>>2]|0;
        $890 = 1 << $I7$0$i$i;
        $891 = $889 & $890;
        $892 = ($891|0)==(0);
        if ($892) {
         $893 = $889 | $890;
         HEAP32[((10360 + 4|0))>>2] = $893;
         HEAP32[$885>>2] = $725;
         $$sum15$i$i = (($$sum$i21$i) + 24)|0;
         $894 = (($tbase$247$i) + ($$sum15$i$i)|0);
         HEAP32[$894>>2] = $885;
         $$sum16$i$i = (($$sum$i21$i) + 12)|0;
         $895 = (($tbase$247$i) + ($$sum16$i$i)|0);
         HEAP32[$895>>2] = $725;
         $$sum17$i$i = (($$sum$i21$i) + 8)|0;
         $896 = (($tbase$247$i) + ($$sum17$i$i)|0);
         HEAP32[$896>>2] = $725;
         break;
        }
        $897 = HEAP32[$885>>2]|0;
        $898 = ($I7$0$i$i|0)==(31);
        if ($898) {
         $906 = 0;
        } else {
         $899 = $I7$0$i$i >>> 1;
         $900 = (25 - ($899))|0;
         $906 = $900;
        }
        $901 = (($897) + 4|0);
        $902 = HEAP32[$901>>2]|0;
        $903 = $902 & -8;
        $904 = ($903|0)==($qsize$0$i$i|0);
        L435: do {
         if ($904) {
          $T$0$lcssa$i28$i = $897;
         } else {
          $905 = $qsize$0$i$i << $906;
          $K8$052$i$i = $905;$T$051$i$i = $897;
          while(1) {
           $913 = $K8$052$i$i >>> 31;
           $914 = ((($T$051$i$i) + ($913<<2)|0) + 16|0);
           $909 = HEAP32[$914>>2]|0;
           $915 = ($909|0)==(0|0);
           if ($915) {
            break;
           }
           $907 = $K8$052$i$i << 1;
           $908 = (($909) + 4|0);
           $910 = HEAP32[$908>>2]|0;
           $911 = $910 & -8;
           $912 = ($911|0)==($qsize$0$i$i|0);
           if ($912) {
            $T$0$lcssa$i28$i = $909;
            break L435;
           } else {
            $K8$052$i$i = $907;$T$051$i$i = $909;
           }
          }
          $916 = HEAP32[((10360 + 16|0))>>2]|0;
          $917 = ($914>>>0)<($916>>>0);
          if ($917) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$914>>2] = $725;
           $$sum23$i$i = (($$sum$i21$i) + 24)|0;
           $918 = (($tbase$247$i) + ($$sum23$i$i)|0);
           HEAP32[$918>>2] = $T$051$i$i;
           $$sum24$i$i = (($$sum$i21$i) + 12)|0;
           $919 = (($tbase$247$i) + ($$sum24$i$i)|0);
           HEAP32[$919>>2] = $725;
           $$sum25$i$i = (($$sum$i21$i) + 8)|0;
           $920 = (($tbase$247$i) + ($$sum25$i$i)|0);
           HEAP32[$920>>2] = $725;
           break L338;
          }
         }
        } while(0);
        $921 = (($T$0$lcssa$i28$i) + 8|0);
        $922 = HEAP32[$921>>2]|0;
        $923 = HEAP32[((10360 + 16|0))>>2]|0;
        $924 = ($T$0$lcssa$i28$i>>>0)<($923>>>0);
        if ($924) {
         _abort();
         // unreachable;
        }
        $925 = ($922>>>0)<($923>>>0);
        if ($925) {
         _abort();
         // unreachable;
        } else {
         $926 = (($922) + 12|0);
         HEAP32[$926>>2] = $725;
         HEAP32[$921>>2] = $725;
         $$sum20$i$i = (($$sum$i21$i) + 8)|0;
         $927 = (($tbase$247$i) + ($$sum20$i$i)|0);
         HEAP32[$927>>2] = $922;
         $$sum21$i$i = (($$sum$i21$i) + 12)|0;
         $928 = (($tbase$247$i) + ($$sum21$i$i)|0);
         HEAP32[$928>>2] = $T$0$lcssa$i28$i;
         $$sum22$i$i = (($$sum$i21$i) + 24)|0;
         $929 = (($tbase$247$i) + ($$sum22$i$i)|0);
         HEAP32[$929>>2] = 0;
         break;
        }
       }
      } while(0);
      $$sum1819$i$i = $713 | 8;
      $930 = (($tbase$247$i) + ($$sum1819$i$i)|0);
      $mem$0 = $930;
      STACKTOP = sp;return ($mem$0|0);
     }
    }
    $sp$0$i$i$i = ((10360 + 448|0));
    while(1) {
     $931 = HEAP32[$sp$0$i$i$i>>2]|0;
     $932 = ($931>>>0)>($636>>>0);
     if (!($932)) {
      $933 = (($sp$0$i$i$i) + 4|0);
      $934 = HEAP32[$933>>2]|0;
      $935 = (($931) + ($934)|0);
      $936 = ($935>>>0)>($636>>>0);
      if ($936) {
       break;
      }
     }
     $937 = (($sp$0$i$i$i) + 8|0);
     $938 = HEAP32[$937>>2]|0;
     $sp$0$i$i$i = $938;
    }
    $$sum$i15$i = (($934) + -47)|0;
    $$sum1$i16$i = (($934) + -39)|0;
    $939 = (($931) + ($$sum1$i16$i)|0);
    $940 = $939;
    $941 = $940 & 7;
    $942 = ($941|0)==(0);
    if ($942) {
     $945 = 0;
    } else {
     $943 = (0 - ($940))|0;
     $944 = $943 & 7;
     $945 = $944;
    }
    $$sum2$i17$i = (($$sum$i15$i) + ($945))|0;
    $946 = (($931) + ($$sum2$i17$i)|0);
    $947 = (($636) + 16|0);
    $948 = ($946>>>0)<($947>>>0);
    $949 = $948 ? $636 : $946;
    $950 = (($949) + 8|0);
    $951 = (($tsize$246$i) + -40)|0;
    $952 = (($tbase$247$i) + 8|0);
    $953 = $952;
    $954 = $953 & 7;
    $955 = ($954|0)==(0);
    if ($955) {
     $959 = 0;
    } else {
     $956 = (0 - ($953))|0;
     $957 = $956 & 7;
     $959 = $957;
    }
    $958 = (($tbase$247$i) + ($959)|0);
    $960 = (($951) - ($959))|0;
    HEAP32[((10360 + 24|0))>>2] = $958;
    HEAP32[((10360 + 12|0))>>2] = $960;
    $961 = $960 | 1;
    $$sum$i$i$i = (($959) + 4)|0;
    $962 = (($tbase$247$i) + ($$sum$i$i$i)|0);
    HEAP32[$962>>2] = $961;
    $$sum2$i$i$i = (($tsize$246$i) + -36)|0;
    $963 = (($tbase$247$i) + ($$sum2$i$i$i)|0);
    HEAP32[$963>>2] = 40;
    $964 = HEAP32[((10832 + 16|0))>>2]|0;
    HEAP32[((10360 + 28|0))>>2] = $964;
    $965 = (($949) + 4|0);
    HEAP32[$965>>2] = 27;
    ;HEAP32[$950+0>>2]=HEAP32[((10360 + 448|0))+0>>2]|0;HEAP32[$950+4>>2]=HEAP32[((10360 + 448|0))+4>>2]|0;HEAP32[$950+8>>2]=HEAP32[((10360 + 448|0))+8>>2]|0;HEAP32[$950+12>>2]=HEAP32[((10360 + 448|0))+12>>2]|0;
    HEAP32[((10360 + 448|0))>>2] = $tbase$247$i;
    HEAP32[((10360 + 452|0))>>2] = $tsize$246$i;
    HEAP32[((10360 + 460|0))>>2] = 0;
    HEAP32[((10360 + 456|0))>>2] = $950;
    $966 = (($949) + 28|0);
    HEAP32[$966>>2] = 7;
    $967 = (($949) + 32|0);
    $968 = ($967>>>0)<($935>>>0);
    if ($968) {
     $970 = $966;
     while(1) {
      $969 = (($970) + 4|0);
      HEAP32[$969>>2] = 7;
      $971 = (($970) + 8|0);
      $972 = ($971>>>0)<($935>>>0);
      if ($972) {
       $970 = $969;
      } else {
       break;
      }
     }
    }
    $973 = ($949|0)==($636|0);
    if (!($973)) {
     $974 = $949;
     $975 = $636;
     $976 = (($974) - ($975))|0;
     $977 = (($636) + ($976)|0);
     $$sum3$i$i = (($976) + 4)|0;
     $978 = (($636) + ($$sum3$i$i)|0);
     $979 = HEAP32[$978>>2]|0;
     $980 = $979 & -2;
     HEAP32[$978>>2] = $980;
     $981 = $976 | 1;
     $982 = (($636) + 4|0);
     HEAP32[$982>>2] = $981;
     HEAP32[$977>>2] = $976;
     $983 = $976 >>> 3;
     $984 = ($976>>>0)<(256);
     if ($984) {
      $985 = $983 << 1;
      $986 = ((10360 + ($985<<2)|0) + 40|0);
      $987 = HEAP32[10360>>2]|0;
      $988 = 1 << $983;
      $989 = $987 & $988;
      $990 = ($989|0)==(0);
      do {
       if ($990) {
        $991 = $987 | $988;
        HEAP32[10360>>2] = $991;
        $$sum10$pre$i$i = (($985) + 2)|0;
        $$pre$i$i = ((10360 + ($$sum10$pre$i$i<<2)|0) + 40|0);
        $$pre$phi$i$iZ2D = $$pre$i$i;$F$0$i$i = $986;
       } else {
        $$sum11$i$i = (($985) + 2)|0;
        $992 = ((10360 + ($$sum11$i$i<<2)|0) + 40|0);
        $993 = HEAP32[$992>>2]|0;
        $994 = HEAP32[((10360 + 16|0))>>2]|0;
        $995 = ($993>>>0)<($994>>>0);
        if (!($995)) {
         $$pre$phi$i$iZ2D = $992;$F$0$i$i = $993;
         break;
        }
        _abort();
        // unreachable;
       }
      } while(0);
      HEAP32[$$pre$phi$i$iZ2D>>2] = $636;
      $996 = (($F$0$i$i) + 12|0);
      HEAP32[$996>>2] = $636;
      $997 = (($636) + 8|0);
      HEAP32[$997>>2] = $F$0$i$i;
      $998 = (($636) + 12|0);
      HEAP32[$998>>2] = $986;
      break;
     }
     $999 = $976 >>> 8;
     $1000 = ($999|0)==(0);
     if ($1000) {
      $I1$0$i$i = 0;
     } else {
      $1001 = ($976>>>0)>(16777215);
      if ($1001) {
       $I1$0$i$i = 31;
      } else {
       $1002 = (($999) + 1048320)|0;
       $1003 = $1002 >>> 16;
       $1004 = $1003 & 8;
       $1005 = $999 << $1004;
       $1006 = (($1005) + 520192)|0;
       $1007 = $1006 >>> 16;
       $1008 = $1007 & 4;
       $1009 = $1008 | $1004;
       $1010 = $1005 << $1008;
       $1011 = (($1010) + 245760)|0;
       $1012 = $1011 >>> 16;
       $1013 = $1012 & 2;
       $1014 = $1009 | $1013;
       $1015 = (14 - ($1014))|0;
       $1016 = $1010 << $1013;
       $1017 = $1016 >>> 15;
       $1018 = (($1015) + ($1017))|0;
       $1019 = $1018 << 1;
       $1020 = (($1018) + 7)|0;
       $1021 = $976 >>> $1020;
       $1022 = $1021 & 1;
       $1023 = $1022 | $1019;
       $I1$0$i$i = $1023;
      }
     }
     $1024 = ((10360 + ($I1$0$i$i<<2)|0) + 304|0);
     $1025 = (($636) + 28|0);
     $I1$0$c$i$i = $I1$0$i$i;
     HEAP32[$1025>>2] = $I1$0$c$i$i;
     $1026 = (($636) + 20|0);
     HEAP32[$1026>>2] = 0;
     $1027 = (($636) + 16|0);
     HEAP32[$1027>>2] = 0;
     $1028 = HEAP32[((10360 + 4|0))>>2]|0;
     $1029 = 1 << $I1$0$i$i;
     $1030 = $1028 & $1029;
     $1031 = ($1030|0)==(0);
     if ($1031) {
      $1032 = $1028 | $1029;
      HEAP32[((10360 + 4|0))>>2] = $1032;
      HEAP32[$1024>>2] = $636;
      $1033 = (($636) + 24|0);
      HEAP32[$1033>>2] = $1024;
      $1034 = (($636) + 12|0);
      HEAP32[$1034>>2] = $636;
      $1035 = (($636) + 8|0);
      HEAP32[$1035>>2] = $636;
      break;
     }
     $1036 = HEAP32[$1024>>2]|0;
     $1037 = ($I1$0$i$i|0)==(31);
     if ($1037) {
      $1045 = 0;
     } else {
      $1038 = $I1$0$i$i >>> 1;
      $1039 = (25 - ($1038))|0;
      $1045 = $1039;
     }
     $1040 = (($1036) + 4|0);
     $1041 = HEAP32[$1040>>2]|0;
     $1042 = $1041 & -8;
     $1043 = ($1042|0)==($976|0);
     L489: do {
      if ($1043) {
       $T$0$lcssa$i$i = $1036;
      } else {
       $1044 = $976 << $1045;
       $K2$014$i$i = $1044;$T$013$i$i = $1036;
       while(1) {
        $1052 = $K2$014$i$i >>> 31;
        $1053 = ((($T$013$i$i) + ($1052<<2)|0) + 16|0);
        $1048 = HEAP32[$1053>>2]|0;
        $1054 = ($1048|0)==(0|0);
        if ($1054) {
         break;
        }
        $1046 = $K2$014$i$i << 1;
        $1047 = (($1048) + 4|0);
        $1049 = HEAP32[$1047>>2]|0;
        $1050 = $1049 & -8;
        $1051 = ($1050|0)==($976|0);
        if ($1051) {
         $T$0$lcssa$i$i = $1048;
         break L489;
        } else {
         $K2$014$i$i = $1046;$T$013$i$i = $1048;
        }
       }
       $1055 = HEAP32[((10360 + 16|0))>>2]|0;
       $1056 = ($1053>>>0)<($1055>>>0);
       if ($1056) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$1053>>2] = $636;
        $1057 = (($636) + 24|0);
        HEAP32[$1057>>2] = $T$013$i$i;
        $1058 = (($636) + 12|0);
        HEAP32[$1058>>2] = $636;
        $1059 = (($636) + 8|0);
        HEAP32[$1059>>2] = $636;
        break L311;
       }
      }
     } while(0);
     $1060 = (($T$0$lcssa$i$i) + 8|0);
     $1061 = HEAP32[$1060>>2]|0;
     $1062 = HEAP32[((10360 + 16|0))>>2]|0;
     $1063 = ($T$0$lcssa$i$i>>>0)<($1062>>>0);
     if ($1063) {
      _abort();
      // unreachable;
     }
     $1064 = ($1061>>>0)<($1062>>>0);
     if ($1064) {
      _abort();
      // unreachable;
     } else {
      $1065 = (($1061) + 12|0);
      HEAP32[$1065>>2] = $636;
      HEAP32[$1060>>2] = $636;
      $1066 = (($636) + 8|0);
      HEAP32[$1066>>2] = $1061;
      $1067 = (($636) + 12|0);
      HEAP32[$1067>>2] = $T$0$lcssa$i$i;
      $1068 = (($636) + 24|0);
      HEAP32[$1068>>2] = 0;
      break;
     }
    }
   }
  } while(0);
  $1069 = HEAP32[((10360 + 12|0))>>2]|0;
  $1070 = ($1069>>>0)>($nb$0>>>0);
  if ($1070) {
   $1071 = (($1069) - ($nb$0))|0;
   HEAP32[((10360 + 12|0))>>2] = $1071;
   $1072 = HEAP32[((10360 + 24|0))>>2]|0;
   $1073 = (($1072) + ($nb$0)|0);
   HEAP32[((10360 + 24|0))>>2] = $1073;
   $1074 = $1071 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1075 = (($1072) + ($$sum$i32)|0);
   HEAP32[$1075>>2] = $1074;
   $1076 = $nb$0 | 3;
   $1077 = (($1072) + 4|0);
   HEAP32[$1077>>2] = $1076;
   $1078 = (($1072) + 8|0);
   $mem$0 = $1078;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $1079 = (___errno_location()|0);
 HEAP32[$1079>>2] = 12;
 $mem$0 = 0;
 STACKTOP = sp;return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$pre = 0, $$pre$phi68Z2D = 0, $$pre$phi70Z2D = 0, $$pre$phiZ2D = 0, $$pre67 = 0, $$pre69 = 0, $$sum = 0, $$sum16$pre = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum2324 = 0, $$sum25 = 0, $$sum26 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0;
 var $$sum31 = 0, $$sum32 = 0, $$sum33 = 0, $$sum34 = 0, $$sum35 = 0, $$sum36 = 0, $$sum37 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0;
 var $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $I18$0$c = 0, $K19$057 = 0;
 var $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$056 = 0, $cond = 0, $cond54 = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  STACKTOP = sp;return;
 }
 $1 = (($mem) + -8|0);
 $2 = HEAP32[((10360 + 16|0))>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = (($mem) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    STACKTOP = sp;return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[((10360 + 20|0))>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $104 = (($mem) + ($$sum3)|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $105 & 3;
    $107 = ($106|0)==(3);
    if (!($107)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[((10360 + 8|0))>>2] = $15;
    $108 = HEAP32[$104>>2]|0;
    $109 = $108 & -2;
    HEAP32[$104>>2] = $109;
    $110 = $15 | 1;
    $$sum26 = (($$sum2) + 4)|0;
    $111 = (($mem) + ($$sum26)|0);
    HEAP32[$111>>2] = $110;
    HEAP32[$9>>2] = $15;
    STACKTOP = sp;return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum36 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum36)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum37 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum37)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = ((10360 + ($25<<2)|0) + 40|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = (($22) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[10360>>2]|0;
     $36 = $35 & $34;
     HEAP32[10360>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre69 = (($24) + 8|0);
     $$pre$phi70Z2D = $$pre69;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = (($24) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi70Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = (($22) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi70Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum28 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum28)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum29 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum29)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum31 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum31)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum30 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum30)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = (($R$0) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = (($R$0) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum35 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum35)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = (($49) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = (($46) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum32 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum32)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = ((10360 + ($72<<2)|0) + 304|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[((10360 + 4|0))>>2]|0;
      $79 = $78 & $77;
      HEAP32[((10360 + 4|0))>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[((10360 + 16|0))>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = (($44) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = (($44) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[((10360 + 16|0))>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = (($R$1) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum33 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum33)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = HEAP32[((10360 + 16|0))>>2]|0;
      $94 = ($91>>>0)<($93>>>0);
      if ($94) {
       _abort();
       // unreachable;
      } else {
       $95 = (($R$1) + 16|0);
       HEAP32[$95>>2] = $91;
       $96 = (($91) + 24|0);
       HEAP32[$96>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum34 = (($$sum2) + 20)|0;
    $97 = (($mem) + ($$sum34)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0|0);
    if ($99) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $100 = HEAP32[((10360 + 16|0))>>2]|0;
     $101 = ($98>>>0)<($100>>>0);
     if ($101) {
      _abort();
      // unreachable;
     } else {
      $102 = (($R$1) + 20|0);
      HEAP32[$102>>2] = $98;
      $103 = (($98) + 24|0);
      HEAP32[$103>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $112 = ($p$0>>>0)<($9>>>0);
 if (!($112)) {
  _abort();
  // unreachable;
 }
 $$sum25 = (($8) + -4)|0;
 $113 = (($mem) + ($$sum25)|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $114 & 1;
 $116 = ($115|0)==(0);
 if ($116) {
  _abort();
  // unreachable;
 }
 $117 = $114 & 2;
 $118 = ($117|0)==(0);
 if ($118) {
  $119 = HEAP32[((10360 + 24|0))>>2]|0;
  $120 = ($9|0)==($119|0);
  if ($120) {
   $121 = HEAP32[((10360 + 12|0))>>2]|0;
   $122 = (($121) + ($psize$0))|0;
   HEAP32[((10360 + 12|0))>>2] = $122;
   HEAP32[((10360 + 24|0))>>2] = $p$0;
   $123 = $122 | 1;
   $124 = (($p$0) + 4|0);
   HEAP32[$124>>2] = $123;
   $125 = HEAP32[((10360 + 20|0))>>2]|0;
   $126 = ($p$0|0)==($125|0);
   if (!($126)) {
    STACKTOP = sp;return;
   }
   HEAP32[((10360 + 20|0))>>2] = 0;
   HEAP32[((10360 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $127 = HEAP32[((10360 + 20|0))>>2]|0;
  $128 = ($9|0)==($127|0);
  if ($128) {
   $129 = HEAP32[((10360 + 8|0))>>2]|0;
   $130 = (($129) + ($psize$0))|0;
   HEAP32[((10360 + 8|0))>>2] = $130;
   HEAP32[((10360 + 20|0))>>2] = $p$0;
   $131 = $130 | 1;
   $132 = (($p$0) + 4|0);
   HEAP32[$132>>2] = $131;
   $133 = (($p$0) + ($130)|0);
   HEAP32[$133>>2] = $130;
   STACKTOP = sp;return;
  }
  $134 = $114 & -8;
  $135 = (($134) + ($psize$0))|0;
  $136 = $114 >>> 3;
  $137 = ($114>>>0)<(256);
  do {
   if ($137) {
    $138 = (($mem) + ($8)|0);
    $139 = HEAP32[$138>>2]|0;
    $$sum2324 = $8 | 4;
    $140 = (($mem) + ($$sum2324)|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = $136 << 1;
    $143 = ((10360 + ($142<<2)|0) + 40|0);
    $144 = ($139|0)==($143|0);
    if (!($144)) {
     $145 = HEAP32[((10360 + 16|0))>>2]|0;
     $146 = ($139>>>0)<($145>>>0);
     if ($146) {
      _abort();
      // unreachable;
     }
     $147 = (($139) + 12|0);
     $148 = HEAP32[$147>>2]|0;
     $149 = ($148|0)==($9|0);
     if (!($149)) {
      _abort();
      // unreachable;
     }
    }
    $150 = ($141|0)==($139|0);
    if ($150) {
     $151 = 1 << $136;
     $152 = $151 ^ -1;
     $153 = HEAP32[10360>>2]|0;
     $154 = $153 & $152;
     HEAP32[10360>>2] = $154;
     break;
    }
    $155 = ($141|0)==($143|0);
    if ($155) {
     $$pre67 = (($141) + 8|0);
     $$pre$phi68Z2D = $$pre67;
    } else {
     $156 = HEAP32[((10360 + 16|0))>>2]|0;
     $157 = ($141>>>0)<($156>>>0);
     if ($157) {
      _abort();
      // unreachable;
     }
     $158 = (($141) + 8|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ($159|0)==($9|0);
     if ($160) {
      $$pre$phi68Z2D = $158;
     } else {
      _abort();
      // unreachable;
     }
    }
    $161 = (($139) + 12|0);
    HEAP32[$161>>2] = $141;
    HEAP32[$$pre$phi68Z2D>>2] = $139;
   } else {
    $$sum5 = (($8) + 16)|0;
    $162 = (($mem) + ($$sum5)|0);
    $163 = HEAP32[$162>>2]|0;
    $$sum67 = $8 | 4;
    $164 = (($mem) + ($$sum67)|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = ($165|0)==($9|0);
    do {
     if ($166) {
      $$sum9 = (($8) + 12)|0;
      $177 = (($mem) + ($$sum9)|0);
      $178 = HEAP32[$177>>2]|0;
      $179 = ($178|0)==(0|0);
      if ($179) {
       $$sum8 = (($8) + 8)|0;
       $180 = (($mem) + ($$sum8)|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = ($181|0)==(0|0);
       if ($182) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $181;$RP9$0 = $180;
       }
      } else {
       $R7$0 = $178;$RP9$0 = $177;
      }
      while(1) {
       $183 = (($R7$0) + 20|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($184|0)==(0|0);
       if (!($185)) {
        $R7$0 = $184;$RP9$0 = $183;
        continue;
       }
       $186 = (($R7$0) + 16|0);
       $187 = HEAP32[$186>>2]|0;
       $188 = ($187|0)==(0|0);
       if ($188) {
        break;
       } else {
        $R7$0 = $187;$RP9$0 = $186;
       }
      }
      $189 = HEAP32[((10360 + 16|0))>>2]|0;
      $190 = ($RP9$0>>>0)<($189>>>0);
      if ($190) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $167 = (($mem) + ($8)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = HEAP32[((10360 + 16|0))>>2]|0;
      $170 = ($168>>>0)<($169>>>0);
      if ($170) {
       _abort();
       // unreachable;
      }
      $171 = (($168) + 12|0);
      $172 = HEAP32[$171>>2]|0;
      $173 = ($172|0)==($9|0);
      if (!($173)) {
       _abort();
       // unreachable;
      }
      $174 = (($165) + 8|0);
      $175 = HEAP32[$174>>2]|0;
      $176 = ($175|0)==($9|0);
      if ($176) {
       HEAP32[$171>>2] = $165;
       HEAP32[$174>>2] = $168;
       $R7$1 = $165;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $191 = ($163|0)==(0|0);
    if (!($191)) {
     $$sum18 = (($8) + 20)|0;
     $192 = (($mem) + ($$sum18)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ((10360 + ($193<<2)|0) + 304|0);
     $195 = HEAP32[$194>>2]|0;
     $196 = ($9|0)==($195|0);
     if ($196) {
      HEAP32[$194>>2] = $R7$1;
      $cond54 = ($R7$1|0)==(0|0);
      if ($cond54) {
       $197 = 1 << $193;
       $198 = $197 ^ -1;
       $199 = HEAP32[((10360 + 4|0))>>2]|0;
       $200 = $199 & $198;
       HEAP32[((10360 + 4|0))>>2] = $200;
       break;
      }
     } else {
      $201 = HEAP32[((10360 + 16|0))>>2]|0;
      $202 = ($163>>>0)<($201>>>0);
      if ($202) {
       _abort();
       // unreachable;
      }
      $203 = (($163) + 16|0);
      $204 = HEAP32[$203>>2]|0;
      $205 = ($204|0)==($9|0);
      if ($205) {
       HEAP32[$203>>2] = $R7$1;
      } else {
       $206 = (($163) + 20|0);
       HEAP32[$206>>2] = $R7$1;
      }
      $207 = ($R7$1|0)==(0|0);
      if ($207) {
       break;
      }
     }
     $208 = HEAP32[((10360 + 16|0))>>2]|0;
     $209 = ($R7$1>>>0)<($208>>>0);
     if ($209) {
      _abort();
      // unreachable;
     }
     $210 = (($R7$1) + 24|0);
     HEAP32[$210>>2] = $163;
     $$sum19 = (($8) + 8)|0;
     $211 = (($mem) + ($$sum19)|0);
     $212 = HEAP32[$211>>2]|0;
     $213 = ($212|0)==(0|0);
     do {
      if (!($213)) {
       $214 = HEAP32[((10360 + 16|0))>>2]|0;
       $215 = ($212>>>0)<($214>>>0);
       if ($215) {
        _abort();
        // unreachable;
       } else {
        $216 = (($R7$1) + 16|0);
        HEAP32[$216>>2] = $212;
        $217 = (($212) + 24|0);
        HEAP32[$217>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum20 = (($8) + 12)|0;
     $218 = (($mem) + ($$sum20)|0);
     $219 = HEAP32[$218>>2]|0;
     $220 = ($219|0)==(0|0);
     if (!($220)) {
      $221 = HEAP32[((10360 + 16|0))>>2]|0;
      $222 = ($219>>>0)<($221>>>0);
      if ($222) {
       _abort();
       // unreachable;
      } else {
       $223 = (($R7$1) + 20|0);
       HEAP32[$223>>2] = $219;
       $224 = (($219) + 24|0);
       HEAP32[$224>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $225 = $135 | 1;
  $226 = (($p$0) + 4|0);
  HEAP32[$226>>2] = $225;
  $227 = (($p$0) + ($135)|0);
  HEAP32[$227>>2] = $135;
  $228 = HEAP32[((10360 + 20|0))>>2]|0;
  $229 = ($p$0|0)==($228|0);
  if ($229) {
   HEAP32[((10360 + 8|0))>>2] = $135;
   STACKTOP = sp;return;
  } else {
   $psize$1 = $135;
  }
 } else {
  $230 = $114 & -2;
  HEAP32[$113>>2] = $230;
  $231 = $psize$0 | 1;
  $232 = (($p$0) + 4|0);
  HEAP32[$232>>2] = $231;
  $233 = (($p$0) + ($psize$0)|0);
  HEAP32[$233>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $234 = $psize$1 >>> 3;
 $235 = ($psize$1>>>0)<(256);
 if ($235) {
  $236 = $234 << 1;
  $237 = ((10360 + ($236<<2)|0) + 40|0);
  $238 = HEAP32[10360>>2]|0;
  $239 = 1 << $234;
  $240 = $238 & $239;
  $241 = ($240|0)==(0);
  if ($241) {
   $242 = $238 | $239;
   HEAP32[10360>>2] = $242;
   $$sum16$pre = (($236) + 2)|0;
   $$pre = ((10360 + ($$sum16$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $237;
  } else {
   $$sum17 = (($236) + 2)|0;
   $243 = ((10360 + ($$sum17<<2)|0) + 40|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = HEAP32[((10360 + 16|0))>>2]|0;
   $246 = ($244>>>0)<($245>>>0);
   if ($246) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $243;$F16$0 = $244;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $247 = (($F16$0) + 12|0);
  HEAP32[$247>>2] = $p$0;
  $248 = (($p$0) + 8|0);
  HEAP32[$248>>2] = $F16$0;
  $249 = (($p$0) + 12|0);
  HEAP32[$249>>2] = $237;
  STACKTOP = sp;return;
 }
 $250 = $psize$1 >>> 8;
 $251 = ($250|0)==(0);
 if ($251) {
  $I18$0 = 0;
 } else {
  $252 = ($psize$1>>>0)>(16777215);
  if ($252) {
   $I18$0 = 31;
  } else {
   $253 = (($250) + 1048320)|0;
   $254 = $253 >>> 16;
   $255 = $254 & 8;
   $256 = $250 << $255;
   $257 = (($256) + 520192)|0;
   $258 = $257 >>> 16;
   $259 = $258 & 4;
   $260 = $259 | $255;
   $261 = $256 << $259;
   $262 = (($261) + 245760)|0;
   $263 = $262 >>> 16;
   $264 = $263 & 2;
   $265 = $260 | $264;
   $266 = (14 - ($265))|0;
   $267 = $261 << $264;
   $268 = $267 >>> 15;
   $269 = (($266) + ($268))|0;
   $270 = $269 << 1;
   $271 = (($269) + 7)|0;
   $272 = $psize$1 >>> $271;
   $273 = $272 & 1;
   $274 = $273 | $270;
   $I18$0 = $274;
  }
 }
 $275 = ((10360 + ($I18$0<<2)|0) + 304|0);
 $276 = (($p$0) + 28|0);
 $I18$0$c = $I18$0;
 HEAP32[$276>>2] = $I18$0$c;
 $277 = (($p$0) + 20|0);
 HEAP32[$277>>2] = 0;
 $278 = (($p$0) + 16|0);
 HEAP32[$278>>2] = 0;
 $279 = HEAP32[((10360 + 4|0))>>2]|0;
 $280 = 1 << $I18$0;
 $281 = $279 & $280;
 $282 = ($281|0)==(0);
 L199: do {
  if ($282) {
   $283 = $279 | $280;
   HEAP32[((10360 + 4|0))>>2] = $283;
   HEAP32[$275>>2] = $p$0;
   $284 = (($p$0) + 24|0);
   HEAP32[$284>>2] = $275;
   $285 = (($p$0) + 12|0);
   HEAP32[$285>>2] = $p$0;
   $286 = (($p$0) + 8|0);
   HEAP32[$286>>2] = $p$0;
  } else {
   $287 = HEAP32[$275>>2]|0;
   $288 = ($I18$0|0)==(31);
   if ($288) {
    $296 = 0;
   } else {
    $289 = $I18$0 >>> 1;
    $290 = (25 - ($289))|0;
    $296 = $290;
   }
   $291 = (($287) + 4|0);
   $292 = HEAP32[$291>>2]|0;
   $293 = $292 & -8;
   $294 = ($293|0)==($psize$1|0);
   L205: do {
    if ($294) {
     $T$0$lcssa = $287;
    } else {
     $295 = $psize$1 << $296;
     $K19$057 = $295;$T$056 = $287;
     while(1) {
      $303 = $K19$057 >>> 31;
      $304 = ((($T$056) + ($303<<2)|0) + 16|0);
      $299 = HEAP32[$304>>2]|0;
      $305 = ($299|0)==(0|0);
      if ($305) {
       break;
      }
      $297 = $K19$057 << 1;
      $298 = (($299) + 4|0);
      $300 = HEAP32[$298>>2]|0;
      $301 = $300 & -8;
      $302 = ($301|0)==($psize$1|0);
      if ($302) {
       $T$0$lcssa = $299;
       break L205;
      } else {
       $K19$057 = $297;$T$056 = $299;
      }
     }
     $306 = HEAP32[((10360 + 16|0))>>2]|0;
     $307 = ($304>>>0)<($306>>>0);
     if ($307) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$304>>2] = $p$0;
      $308 = (($p$0) + 24|0);
      HEAP32[$308>>2] = $T$056;
      $309 = (($p$0) + 12|0);
      HEAP32[$309>>2] = $p$0;
      $310 = (($p$0) + 8|0);
      HEAP32[$310>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $311 = (($T$0$lcssa) + 8|0);
   $312 = HEAP32[$311>>2]|0;
   $313 = HEAP32[((10360 + 16|0))>>2]|0;
   $314 = ($T$0$lcssa>>>0)<($313>>>0);
   if ($314) {
    _abort();
    // unreachable;
   }
   $315 = ($312>>>0)<($313>>>0);
   if ($315) {
    _abort();
    // unreachable;
   } else {
    $316 = (($312) + 12|0);
    HEAP32[$316>>2] = $p$0;
    HEAP32[$311>>2] = $p$0;
    $317 = (($p$0) + 8|0);
    HEAP32[$317>>2] = $312;
    $318 = (($p$0) + 12|0);
    HEAP32[$318>>2] = $T$0$lcssa;
    $319 = (($p$0) + 24|0);
    HEAP32[$319>>2] = 0;
    break;
   }
  }
 } while(0);
 $320 = HEAP32[((10360 + 32|0))>>2]|0;
 $321 = (($320) + -1)|0;
 HEAP32[((10360 + 32|0))>>2] = $321;
 $322 = ($321|0)==(0);
 if ($322) {
  $sp$0$in$i = ((10360 + 456|0));
 } else {
  STACKTOP = sp;return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $323 = ($sp$0$i|0)==(0|0);
  $324 = (($sp$0$i) + 8|0);
  if ($323) {
   break;
  } else {
   $sp$0$in$i = $324;
  }
 }
 HEAP32[((10360 + 32|0))>>2] = -1;
 STACKTOP = sp;return;
}
function _realloc($oldmem,$bytes) {
 $oldmem = $oldmem|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $mem$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($oldmem|0)==(0|0);
 do {
  if ($0) {
   $1 = (_malloc($bytes)|0);
   $mem$0 = $1;
  } else {
   $2 = ($bytes>>>0)>(4294967231);
   if ($2) {
    $3 = (___errno_location()|0);
    HEAP32[$3>>2] = 12;
    $mem$0 = 0;
    break;
   }
   $4 = ($bytes>>>0)<(11);
   if ($4) {
    $8 = 16;
   } else {
    $5 = (($bytes) + 11)|0;
    $6 = $5 & -8;
    $8 = $6;
   }
   $7 = (($oldmem) + -8|0);
   $9 = (_try_realloc_chunk($7,$8)|0);
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = (($9) + 8|0);
    $mem$0 = $11;
    break;
   }
   $12 = (_malloc($bytes)|0);
   $13 = ($12|0)==(0|0);
   if ($13) {
    $mem$0 = 0;
   } else {
    $14 = (($oldmem) + -4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = $15 & -8;
    $17 = $15 & 3;
    $18 = ($17|0)==(0);
    $19 = $18 ? 8 : 4;
    $20 = (($16) - ($19))|0;
    $21 = ($20>>>0)<($bytes>>>0);
    $22 = $21 ? $20 : $bytes;
    _memcpy(($12|0),($oldmem|0),($22|0))|0;
    _free($oldmem);
    $mem$0 = $12;
   }
  }
 } while(0);
 STACKTOP = sp;return ($mem$0|0);
}
function _try_realloc_chunk($p,$nb) {
 $p = $p|0;
 $nb = $nb|0;
 var $$pre = 0, $$pre$phiZ2D = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum2728 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum78 = 0;
 var $$sum910 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $R$0 = 0, $R$1 = 0, $RP$0 = 0;
 var $cond = 0, $newp$0 = 0, $or$cond = 0, $storemerge = 0, $storemerge21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 & -8;
 $3 = (($p) + ($2)|0);
 $4 = HEAP32[((10360 + 16|0))>>2]|0;
 $5 = ($p>>>0)<($4>>>0);
 if ($5) {
  _abort();
  // unreachable;
 }
 $6 = $1 & 3;
 $7 = ($6|0)!=(1);
 $8 = ($p>>>0)<($3>>>0);
 $or$cond = $7 & $8;
 if (!($or$cond)) {
  _abort();
  // unreachable;
 }
 $$sum2728 = $2 | 4;
 $9 = (($p) + ($$sum2728)|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 1;
 $12 = ($11|0)==(0);
 if ($12) {
  _abort();
  // unreachable;
 }
 $13 = ($6|0)==(0);
 if ($13) {
  $14 = ($nb>>>0)<(256);
  if ($14) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $15 = (($nb) + 4)|0;
  $16 = ($2>>>0)<($15>>>0);
  if (!($16)) {
   $17 = (($2) - ($nb))|0;
   $18 = HEAP32[((10832 + 8|0))>>2]|0;
   $19 = $18 << 1;
   $20 = ($17>>>0)>($19>>>0);
   if (!($20)) {
    $newp$0 = $p;
    STACKTOP = sp;return ($newp$0|0);
   }
  }
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $21 = ($2>>>0)<($nb>>>0);
 if (!($21)) {
  $22 = (($2) - ($nb))|0;
  $23 = ($22>>>0)>(15);
  if (!($23)) {
   $newp$0 = $p;
   STACKTOP = sp;return ($newp$0|0);
  }
  $24 = (($p) + ($nb)|0);
  $25 = $1 & 1;
  $26 = $25 | $nb;
  $27 = $26 | 2;
  HEAP32[$0>>2] = $27;
  $$sum23 = (($nb) + 4)|0;
  $28 = (($p) + ($$sum23)|0);
  $29 = $22 | 3;
  HEAP32[$28>>2] = $29;
  $30 = HEAP32[$9>>2]|0;
  $31 = $30 | 1;
  HEAP32[$9>>2] = $31;
  _dispose_chunk($24,$22);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $32 = HEAP32[((10360 + 24|0))>>2]|0;
 $33 = ($3|0)==($32|0);
 if ($33) {
  $34 = HEAP32[((10360 + 12|0))>>2]|0;
  $35 = (($34) + ($2))|0;
  $36 = ($35>>>0)>($nb>>>0);
  if (!($36)) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $37 = (($35) - ($nb))|0;
  $38 = (($p) + ($nb)|0);
  $39 = $1 & 1;
  $40 = $39 | $nb;
  $41 = $40 | 2;
  HEAP32[$0>>2] = $41;
  $$sum22 = (($nb) + 4)|0;
  $42 = (($p) + ($$sum22)|0);
  $43 = $37 | 1;
  HEAP32[$42>>2] = $43;
  HEAP32[((10360 + 24|0))>>2] = $38;
  HEAP32[((10360 + 12|0))>>2] = $37;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $44 = HEAP32[((10360 + 20|0))>>2]|0;
 $45 = ($3|0)==($44|0);
 if ($45) {
  $46 = HEAP32[((10360 + 8|0))>>2]|0;
  $47 = (($46) + ($2))|0;
  $48 = ($47>>>0)<($nb>>>0);
  if ($48) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $49 = (($47) - ($nb))|0;
  $50 = ($49>>>0)>(15);
  if ($50) {
   $51 = (($p) + ($nb)|0);
   $52 = (($p) + ($47)|0);
   $53 = $1 & 1;
   $54 = $53 | $nb;
   $55 = $54 | 2;
   HEAP32[$0>>2] = $55;
   $$sum19 = (($nb) + 4)|0;
   $56 = (($p) + ($$sum19)|0);
   $57 = $49 | 1;
   HEAP32[$56>>2] = $57;
   HEAP32[$52>>2] = $49;
   $$sum20 = (($47) + 4)|0;
   $58 = (($p) + ($$sum20)|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $59 & -2;
   HEAP32[$58>>2] = $60;
   $storemerge = $51;$storemerge21 = $49;
  } else {
   $61 = $1 & 1;
   $62 = $61 | $47;
   $63 = $62 | 2;
   HEAP32[$0>>2] = $63;
   $$sum17 = (($47) + 4)|0;
   $64 = (($p) + ($$sum17)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 | 1;
   HEAP32[$64>>2] = $66;
   $storemerge = 0;$storemerge21 = 0;
  }
  HEAP32[((10360 + 8|0))>>2] = $storemerge21;
  HEAP32[((10360 + 20|0))>>2] = $storemerge;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $67 = $10 & 2;
 $68 = ($67|0)==(0);
 if (!($68)) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $69 = $10 & -8;
 $70 = (($69) + ($2))|0;
 $71 = ($70>>>0)<($nb>>>0);
 if ($71) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $72 = (($70) - ($nb))|0;
 $73 = $10 >>> 3;
 $74 = ($10>>>0)<(256);
 do {
  if ($74) {
   $$sum15 = (($2) + 8)|0;
   $75 = (($p) + ($$sum15)|0);
   $76 = HEAP32[$75>>2]|0;
   $$sum16 = (($2) + 12)|0;
   $77 = (($p) + ($$sum16)|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $73 << 1;
   $80 = ((10360 + ($79<<2)|0) + 40|0);
   $81 = ($76|0)==($80|0);
   if (!($81)) {
    $82 = ($76>>>0)<($4>>>0);
    if ($82) {
     _abort();
     // unreachable;
    }
    $83 = (($76) + 12|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = ($84|0)==($3|0);
    if (!($85)) {
     _abort();
     // unreachable;
    }
   }
   $86 = ($78|0)==($76|0);
   if ($86) {
    $87 = 1 << $73;
    $88 = $87 ^ -1;
    $89 = HEAP32[10360>>2]|0;
    $90 = $89 & $88;
    HEAP32[10360>>2] = $90;
    break;
   }
   $91 = ($78|0)==($80|0);
   if ($91) {
    $$pre = (($78) + 8|0);
    $$pre$phiZ2D = $$pre;
   } else {
    $92 = ($78>>>0)<($4>>>0);
    if ($92) {
     _abort();
     // unreachable;
    }
    $93 = (($78) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==($3|0);
    if ($95) {
     $$pre$phiZ2D = $93;
    } else {
     _abort();
     // unreachable;
    }
   }
   $96 = (($76) + 12|0);
   HEAP32[$96>>2] = $78;
   HEAP32[$$pre$phiZ2D>>2] = $76;
  } else {
   $$sum = (($2) + 24)|0;
   $97 = (($p) + ($$sum)|0);
   $98 = HEAP32[$97>>2]|0;
   $$sum2 = (($2) + 12)|0;
   $99 = (($p) + ($$sum2)|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = ($100|0)==($3|0);
   do {
    if ($101) {
     $$sum4 = (($2) + 20)|0;
     $111 = (($p) + ($$sum4)|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = ($112|0)==(0|0);
     if ($113) {
      $$sum3 = (($2) + 16)|0;
      $114 = (($p) + ($$sum3)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = ($115|0)==(0|0);
      if ($116) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $115;$RP$0 = $114;
      }
     } else {
      $R$0 = $112;$RP$0 = $111;
     }
     while(1) {
      $117 = (($R$0) + 20|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = ($118|0)==(0|0);
      if (!($119)) {
       $R$0 = $118;$RP$0 = $117;
       continue;
      }
      $120 = (($R$0) + 16|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ($121|0)==(0|0);
      if ($122) {
       break;
      } else {
       $R$0 = $121;$RP$0 = $120;
      }
     }
     $123 = ($RP$0>>>0)<($4>>>0);
     if ($123) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum14 = (($2) + 8)|0;
     $102 = (($p) + ($$sum14)|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = ($103>>>0)<($4>>>0);
     if ($104) {
      _abort();
      // unreachable;
     }
     $105 = (($103) + 12|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = ($106|0)==($3|0);
     if (!($107)) {
      _abort();
      // unreachable;
     }
     $108 = (($100) + 8|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = ($109|0)==($3|0);
     if ($110) {
      HEAP32[$105>>2] = $100;
      HEAP32[$108>>2] = $103;
      $R$1 = $100;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $124 = ($98|0)==(0|0);
   if (!($124)) {
    $$sum11 = (($2) + 28)|0;
    $125 = (($p) + ($$sum11)|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ((10360 + ($126<<2)|0) + 304|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = ($3|0)==($128|0);
    if ($129) {
     HEAP32[$127>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $130 = 1 << $126;
      $131 = $130 ^ -1;
      $132 = HEAP32[((10360 + 4|0))>>2]|0;
      $133 = $132 & $131;
      HEAP32[((10360 + 4|0))>>2] = $133;
      break;
     }
    } else {
     $134 = HEAP32[((10360 + 16|0))>>2]|0;
     $135 = ($98>>>0)<($134>>>0);
     if ($135) {
      _abort();
      // unreachable;
     }
     $136 = (($98) + 16|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = ($137|0)==($3|0);
     if ($138) {
      HEAP32[$136>>2] = $R$1;
     } else {
      $139 = (($98) + 20|0);
      HEAP32[$139>>2] = $R$1;
     }
     $140 = ($R$1|0)==(0|0);
     if ($140) {
      break;
     }
    }
    $141 = HEAP32[((10360 + 16|0))>>2]|0;
    $142 = ($R$1>>>0)<($141>>>0);
    if ($142) {
     _abort();
     // unreachable;
    }
    $143 = (($R$1) + 24|0);
    HEAP32[$143>>2] = $98;
    $$sum12 = (($2) + 16)|0;
    $144 = (($p) + ($$sum12)|0);
    $145 = HEAP32[$144>>2]|0;
    $146 = ($145|0)==(0|0);
    do {
     if (!($146)) {
      $147 = HEAP32[((10360 + 16|0))>>2]|0;
      $148 = ($145>>>0)<($147>>>0);
      if ($148) {
       _abort();
       // unreachable;
      } else {
       $149 = (($R$1) + 16|0);
       HEAP32[$149>>2] = $145;
       $150 = (($145) + 24|0);
       HEAP32[$150>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum13 = (($2) + 20)|0;
    $151 = (($p) + ($$sum13)|0);
    $152 = HEAP32[$151>>2]|0;
    $153 = ($152|0)==(0|0);
    if (!($153)) {
     $154 = HEAP32[((10360 + 16|0))>>2]|0;
     $155 = ($152>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     } else {
      $156 = (($R$1) + 20|0);
      HEAP32[$156>>2] = $152;
      $157 = (($152) + 24|0);
      HEAP32[$157>>2] = $R$1;
      break;
     }
    }
   }
  }
 } while(0);
 $158 = ($72>>>0)<(16);
 if ($158) {
  $159 = HEAP32[$0>>2]|0;
  $160 = $159 & 1;
  $161 = $70 | $160;
  $162 = $161 | 2;
  HEAP32[$0>>2] = $162;
  $$sum910 = $70 | 4;
  $163 = (($p) + ($$sum910)|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = $164 | 1;
  HEAP32[$163>>2] = $165;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 } else {
  $166 = (($p) + ($nb)|0);
  $167 = HEAP32[$0>>2]|0;
  $168 = $167 & 1;
  $169 = $168 | $nb;
  $170 = $169 | 2;
  HEAP32[$0>>2] = $170;
  $$sum5 = (($nb) + 4)|0;
  $171 = (($p) + ($$sum5)|0);
  $172 = $72 | 3;
  HEAP32[$171>>2] = $172;
  $$sum78 = $70 | 4;
  $173 = (($p) + ($$sum78)|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = $174 | 1;
  HEAP32[$173>>2] = $175;
  _dispose_chunk($166,$72);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 return 0|0;
}
function _dispose_chunk($p,$psize) {
 $p = $p|0;
 $psize = $psize|0;
 var $$0 = 0, $$02 = 0, $$1 = 0, $$pre = 0, $$pre$phi63Z2D = 0, $$pre$phi65Z2D = 0, $$pre$phiZ2D = 0, $$pre62 = 0, $$pre64 = 0, $$sum = 0, $$sum1 = 0, $$sum12$pre = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0;
 var $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0, $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum4 = 0, $$sum5 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0;
 var $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0;
 var $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0;
 var $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0;
 var $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0;
 var $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0;
 var $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0;
 var $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0;
 var $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0;
 var $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0;
 var $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0;
 var $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0;
 var $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I19$0 = 0, $I19$0$c = 0, $K20$049 = 0, $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$048 = 0, $cond = 0, $cond46 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = (($p) + ($psize)|0);
 $1 = (($p) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = HEAP32[$p>>2]|0;
   $6 = $2 & 3;
   $7 = ($6|0)==(0);
   if ($7) {
    STACKTOP = sp;return;
   }
   $8 = (0 - ($5))|0;
   $9 = (($p) + ($8)|0);
   $10 = (($5) + ($psize))|0;
   $11 = HEAP32[((10360 + 16|0))>>2]|0;
   $12 = ($9>>>0)<($11>>>0);
   if ($12) {
    _abort();
    // unreachable;
   }
   $13 = HEAP32[((10360 + 20|0))>>2]|0;
   $14 = ($9|0)==($13|0);
   if ($14) {
    $$sum = (($psize) + 4)|0;
    $100 = (($p) + ($$sum)|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = $101 & 3;
    $103 = ($102|0)==(3);
    if (!($103)) {
     $$0 = $9;$$02 = $10;
     break;
    }
    HEAP32[((10360 + 8|0))>>2] = $10;
    $104 = HEAP32[$100>>2]|0;
    $105 = $104 & -2;
    HEAP32[$100>>2] = $105;
    $106 = $10 | 1;
    $$sum20 = (4 - ($5))|0;
    $107 = (($p) + ($$sum20)|0);
    HEAP32[$107>>2] = $106;
    HEAP32[$0>>2] = $10;
    STACKTOP = sp;return;
   }
   $15 = $5 >>> 3;
   $16 = ($5>>>0)<(256);
   if ($16) {
    $$sum30 = (8 - ($5))|0;
    $17 = (($p) + ($$sum30)|0);
    $18 = HEAP32[$17>>2]|0;
    $$sum31 = (12 - ($5))|0;
    $19 = (($p) + ($$sum31)|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $15 << 1;
    $22 = ((10360 + ($21<<2)|0) + 40|0);
    $23 = ($18|0)==($22|0);
    if (!($23)) {
     $24 = ($18>>>0)<($11>>>0);
     if ($24) {
      _abort();
      // unreachable;
     }
     $25 = (($18) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==($9|0);
     if (!($27)) {
      _abort();
      // unreachable;
     }
    }
    $28 = ($20|0)==($18|0);
    if ($28) {
     $29 = 1 << $15;
     $30 = $29 ^ -1;
     $31 = HEAP32[10360>>2]|0;
     $32 = $31 & $30;
     HEAP32[10360>>2] = $32;
     $$0 = $9;$$02 = $10;
     break;
    }
    $33 = ($20|0)==($22|0);
    if ($33) {
     $$pre64 = (($20) + 8|0);
     $$pre$phi65Z2D = $$pre64;
    } else {
     $34 = ($20>>>0)<($11>>>0);
     if ($34) {
      _abort();
      // unreachable;
     }
     $35 = (($20) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)==($9|0);
     if ($37) {
      $$pre$phi65Z2D = $35;
     } else {
      _abort();
      // unreachable;
     }
    }
    $38 = (($18) + 12|0);
    HEAP32[$38>>2] = $20;
    HEAP32[$$pre$phi65Z2D>>2] = $18;
    $$0 = $9;$$02 = $10;
    break;
   }
   $$sum22 = (24 - ($5))|0;
   $39 = (($p) + ($$sum22)|0);
   $40 = HEAP32[$39>>2]|0;
   $$sum23 = (12 - ($5))|0;
   $41 = (($p) + ($$sum23)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==($9|0);
   do {
    if ($43) {
     $$sum24 = (16 - ($5))|0;
     $$sum25 = (($$sum24) + 4)|0;
     $53 = (($p) + ($$sum25)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)==(0|0);
     if ($55) {
      $56 = (($p) + ($$sum24)|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==(0|0);
      if ($58) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $57;$RP$0 = $56;
      }
     } else {
      $R$0 = $54;$RP$0 = $53;
     }
     while(1) {
      $59 = (($R$0) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==(0|0);
      if (!($61)) {
       $R$0 = $60;$RP$0 = $59;
       continue;
      }
      $62 = (($R$0) + 16|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       break;
      } else {
       $R$0 = $63;$RP$0 = $62;
      }
     }
     $65 = ($RP$0>>>0)<($11>>>0);
     if ($65) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum29 = (8 - ($5))|0;
     $44 = (($p) + ($$sum29)|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($45>>>0)<($11>>>0);
     if ($46) {
      _abort();
      // unreachable;
     }
     $47 = (($45) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)==($9|0);
     if (!($49)) {
      _abort();
      // unreachable;
     }
     $50 = (($42) + 8|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)==($9|0);
     if ($52) {
      HEAP32[$47>>2] = $42;
      HEAP32[$50>>2] = $45;
      $R$1 = $42;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $66 = ($40|0)==(0|0);
   if ($66) {
    $$0 = $9;$$02 = $10;
   } else {
    $$sum26 = (28 - ($5))|0;
    $67 = (($p) + ($$sum26)|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ((10360 + ($68<<2)|0) + 304|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($9|0)==($70|0);
    if ($71) {
     HEAP32[$69>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $72 = 1 << $68;
      $73 = $72 ^ -1;
      $74 = HEAP32[((10360 + 4|0))>>2]|0;
      $75 = $74 & $73;
      HEAP32[((10360 + 4|0))>>2] = $75;
      $$0 = $9;$$02 = $10;
      break;
     }
    } else {
     $76 = HEAP32[((10360 + 16|0))>>2]|0;
     $77 = ($40>>>0)<($76>>>0);
     if ($77) {
      _abort();
      // unreachable;
     }
     $78 = (($40) + 16|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)==($9|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
     } else {
      $81 = (($40) + 20|0);
      HEAP32[$81>>2] = $R$1;
     }
     $82 = ($R$1|0)==(0|0);
     if ($82) {
      $$0 = $9;$$02 = $10;
      break;
     }
    }
    $83 = HEAP32[((10360 + 16|0))>>2]|0;
    $84 = ($R$1>>>0)<($83>>>0);
    if ($84) {
     _abort();
     // unreachable;
    }
    $85 = (($R$1) + 24|0);
    HEAP32[$85>>2] = $40;
    $$sum27 = (16 - ($5))|0;
    $86 = (($p) + ($$sum27)|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($87|0)==(0|0);
    do {
     if (!($88)) {
      $89 = HEAP32[((10360 + 16|0))>>2]|0;
      $90 = ($87>>>0)<($89>>>0);
      if ($90) {
       _abort();
       // unreachable;
      } else {
       $91 = (($R$1) + 16|0);
       HEAP32[$91>>2] = $87;
       $92 = (($87) + 24|0);
       HEAP32[$92>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum27) + 4)|0;
    $93 = (($p) + ($$sum28)|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==(0|0);
    if ($95) {
     $$0 = $9;$$02 = $10;
    } else {
     $96 = HEAP32[((10360 + 16|0))>>2]|0;
     $97 = ($94>>>0)<($96>>>0);
     if ($97) {
      _abort();
      // unreachable;
     } else {
      $98 = (($R$1) + 20|0);
      HEAP32[$98>>2] = $94;
      $99 = (($94) + 24|0);
      HEAP32[$99>>2] = $R$1;
      $$0 = $9;$$02 = $10;
      break;
     }
    }
   }
  } else {
   $$0 = $p;$$02 = $psize;
  }
 } while(0);
 $108 = HEAP32[((10360 + 16|0))>>2]|0;
 $109 = ($0>>>0)<($108>>>0);
 if ($109) {
  _abort();
  // unreachable;
 }
 $$sum1 = (($psize) + 4)|0;
 $110 = (($p) + ($$sum1)|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = $111 & 2;
 $113 = ($112|0)==(0);
 if ($113) {
  $114 = HEAP32[((10360 + 24|0))>>2]|0;
  $115 = ($0|0)==($114|0);
  if ($115) {
   $116 = HEAP32[((10360 + 12|0))>>2]|0;
   $117 = (($116) + ($$02))|0;
   HEAP32[((10360 + 12|0))>>2] = $117;
   HEAP32[((10360 + 24|0))>>2] = $$0;
   $118 = $117 | 1;
   $119 = (($$0) + 4|0);
   HEAP32[$119>>2] = $118;
   $120 = HEAP32[((10360 + 20|0))>>2]|0;
   $121 = ($$0|0)==($120|0);
   if (!($121)) {
    STACKTOP = sp;return;
   }
   HEAP32[((10360 + 20|0))>>2] = 0;
   HEAP32[((10360 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $122 = HEAP32[((10360 + 20|0))>>2]|0;
  $123 = ($0|0)==($122|0);
  if ($123) {
   $124 = HEAP32[((10360 + 8|0))>>2]|0;
   $125 = (($124) + ($$02))|0;
   HEAP32[((10360 + 8|0))>>2] = $125;
   HEAP32[((10360 + 20|0))>>2] = $$0;
   $126 = $125 | 1;
   $127 = (($$0) + 4|0);
   HEAP32[$127>>2] = $126;
   $128 = (($$0) + ($125)|0);
   HEAP32[$128>>2] = $125;
   STACKTOP = sp;return;
  }
  $129 = $111 & -8;
  $130 = (($129) + ($$02))|0;
  $131 = $111 >>> 3;
  $132 = ($111>>>0)<(256);
  do {
   if ($132) {
    $$sum18 = (($psize) + 8)|0;
    $133 = (($p) + ($$sum18)|0);
    $134 = HEAP32[$133>>2]|0;
    $$sum19 = (($psize) + 12)|0;
    $135 = (($p) + ($$sum19)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = $131 << 1;
    $138 = ((10360 + ($137<<2)|0) + 40|0);
    $139 = ($134|0)==($138|0);
    if (!($139)) {
     $140 = ($134>>>0)<($108>>>0);
     if ($140) {
      _abort();
      // unreachable;
     }
     $141 = (($134) + 12|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = ($142|0)==($0|0);
     if (!($143)) {
      _abort();
      // unreachable;
     }
    }
    $144 = ($136|0)==($134|0);
    if ($144) {
     $145 = 1 << $131;
     $146 = $145 ^ -1;
     $147 = HEAP32[10360>>2]|0;
     $148 = $147 & $146;
     HEAP32[10360>>2] = $148;
     break;
    }
    $149 = ($136|0)==($138|0);
    if ($149) {
     $$pre62 = (($136) + 8|0);
     $$pre$phi63Z2D = $$pre62;
    } else {
     $150 = ($136>>>0)<($108>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($136) + 8|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = ($152|0)==($0|0);
     if ($153) {
      $$pre$phi63Z2D = $151;
     } else {
      _abort();
      // unreachable;
     }
    }
    $154 = (($134) + 12|0);
    HEAP32[$154>>2] = $136;
    HEAP32[$$pre$phi63Z2D>>2] = $134;
   } else {
    $$sum2 = (($psize) + 24)|0;
    $155 = (($p) + ($$sum2)|0);
    $156 = HEAP32[$155>>2]|0;
    $$sum3 = (($psize) + 12)|0;
    $157 = (($p) + ($$sum3)|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = ($158|0)==($0|0);
    do {
     if ($159) {
      $$sum5 = (($psize) + 20)|0;
      $169 = (($p) + ($$sum5)|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==(0|0);
      if ($171) {
       $$sum4 = (($psize) + 16)|0;
       $172 = (($p) + ($$sum4)|0);
       $173 = HEAP32[$172>>2]|0;
       $174 = ($173|0)==(0|0);
       if ($174) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $173;$RP9$0 = $172;
       }
      } else {
       $R7$0 = $170;$RP9$0 = $169;
      }
      while(1) {
       $175 = (($R7$0) + 20|0);
       $176 = HEAP32[$175>>2]|0;
       $177 = ($176|0)==(0|0);
       if (!($177)) {
        $R7$0 = $176;$RP9$0 = $175;
        continue;
       }
       $178 = (($R7$0) + 16|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      }
      $181 = ($RP9$0>>>0)<($108>>>0);
      if ($181) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $$sum17 = (($psize) + 8)|0;
      $160 = (($p) + ($$sum17)|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = ($161>>>0)<($108>>>0);
      if ($162) {
       _abort();
       // unreachable;
      }
      $163 = (($161) + 12|0);
      $164 = HEAP32[$163>>2]|0;
      $165 = ($164|0)==($0|0);
      if (!($165)) {
       _abort();
       // unreachable;
      }
      $166 = (($158) + 8|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = ($167|0)==($0|0);
      if ($168) {
       HEAP32[$163>>2] = $158;
       HEAP32[$166>>2] = $161;
       $R7$1 = $158;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $182 = ($156|0)==(0|0);
    if (!($182)) {
     $$sum14 = (($psize) + 28)|0;
     $183 = (($p) + ($$sum14)|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = ((10360 + ($184<<2)|0) + 304|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = ($0|0)==($186|0);
     if ($187) {
      HEAP32[$185>>2] = $R7$1;
      $cond46 = ($R7$1|0)==(0|0);
      if ($cond46) {
       $188 = 1 << $184;
       $189 = $188 ^ -1;
       $190 = HEAP32[((10360 + 4|0))>>2]|0;
       $191 = $190 & $189;
       HEAP32[((10360 + 4|0))>>2] = $191;
       break;
      }
     } else {
      $192 = HEAP32[((10360 + 16|0))>>2]|0;
      $193 = ($156>>>0)<($192>>>0);
      if ($193) {
       _abort();
       // unreachable;
      }
      $194 = (($156) + 16|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = ($195|0)==($0|0);
      if ($196) {
       HEAP32[$194>>2] = $R7$1;
      } else {
       $197 = (($156) + 20|0);
       HEAP32[$197>>2] = $R7$1;
      }
      $198 = ($R7$1|0)==(0|0);
      if ($198) {
       break;
      }
     }
     $199 = HEAP32[((10360 + 16|0))>>2]|0;
     $200 = ($R7$1>>>0)<($199>>>0);
     if ($200) {
      _abort();
      // unreachable;
     }
     $201 = (($R7$1) + 24|0);
     HEAP32[$201>>2] = $156;
     $$sum15 = (($psize) + 16)|0;
     $202 = (($p) + ($$sum15)|0);
     $203 = HEAP32[$202>>2]|0;
     $204 = ($203|0)==(0|0);
     do {
      if (!($204)) {
       $205 = HEAP32[((10360 + 16|0))>>2]|0;
       $206 = ($203>>>0)<($205>>>0);
       if ($206) {
        _abort();
        // unreachable;
       } else {
        $207 = (($R7$1) + 16|0);
        HEAP32[$207>>2] = $203;
        $208 = (($203) + 24|0);
        HEAP32[$208>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum16 = (($psize) + 20)|0;
     $209 = (($p) + ($$sum16)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     if (!($211)) {
      $212 = HEAP32[((10360 + 16|0))>>2]|0;
      $213 = ($210>>>0)<($212>>>0);
      if ($213) {
       _abort();
       // unreachable;
      } else {
       $214 = (($R7$1) + 20|0);
       HEAP32[$214>>2] = $210;
       $215 = (($210) + 24|0);
       HEAP32[$215>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $216 = $130 | 1;
  $217 = (($$0) + 4|0);
  HEAP32[$217>>2] = $216;
  $218 = (($$0) + ($130)|0);
  HEAP32[$218>>2] = $130;
  $219 = HEAP32[((10360 + 20|0))>>2]|0;
  $220 = ($$0|0)==($219|0);
  if ($220) {
   HEAP32[((10360 + 8|0))>>2] = $130;
   STACKTOP = sp;return;
  } else {
   $$1 = $130;
  }
 } else {
  $221 = $111 & -2;
  HEAP32[$110>>2] = $221;
  $222 = $$02 | 1;
  $223 = (($$0) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($$0) + ($$02)|0);
  HEAP32[$224>>2] = $$02;
  $$1 = $$02;
 }
 $225 = $$1 >>> 3;
 $226 = ($$1>>>0)<(256);
 if ($226) {
  $227 = $225 << 1;
  $228 = ((10360 + ($227<<2)|0) + 40|0);
  $229 = HEAP32[10360>>2]|0;
  $230 = 1 << $225;
  $231 = $229 & $230;
  $232 = ($231|0)==(0);
  if ($232) {
   $233 = $229 | $230;
   HEAP32[10360>>2] = $233;
   $$sum12$pre = (($227) + 2)|0;
   $$pre = ((10360 + ($$sum12$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $228;
  } else {
   $$sum13 = (($227) + 2)|0;
   $234 = ((10360 + ($$sum13<<2)|0) + 40|0);
   $235 = HEAP32[$234>>2]|0;
   $236 = HEAP32[((10360 + 16|0))>>2]|0;
   $237 = ($235>>>0)<($236>>>0);
   if ($237) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $234;$F16$0 = $235;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $$0;
  $238 = (($F16$0) + 12|0);
  HEAP32[$238>>2] = $$0;
  $239 = (($$0) + 8|0);
  HEAP32[$239>>2] = $F16$0;
  $240 = (($$0) + 12|0);
  HEAP32[$240>>2] = $228;
  STACKTOP = sp;return;
 }
 $241 = $$1 >>> 8;
 $242 = ($241|0)==(0);
 if ($242) {
  $I19$0 = 0;
 } else {
  $243 = ($$1>>>0)>(16777215);
  if ($243) {
   $I19$0 = 31;
  } else {
   $244 = (($241) + 1048320)|0;
   $245 = $244 >>> 16;
   $246 = $245 & 8;
   $247 = $241 << $246;
   $248 = (($247) + 520192)|0;
   $249 = $248 >>> 16;
   $250 = $249 & 4;
   $251 = $250 | $246;
   $252 = $247 << $250;
   $253 = (($252) + 245760)|0;
   $254 = $253 >>> 16;
   $255 = $254 & 2;
   $256 = $251 | $255;
   $257 = (14 - ($256))|0;
   $258 = $252 << $255;
   $259 = $258 >>> 15;
   $260 = (($257) + ($259))|0;
   $261 = $260 << 1;
   $262 = (($260) + 7)|0;
   $263 = $$1 >>> $262;
   $264 = $263 & 1;
   $265 = $264 | $261;
   $I19$0 = $265;
  }
 }
 $266 = ((10360 + ($I19$0<<2)|0) + 304|0);
 $267 = (($$0) + 28|0);
 $I19$0$c = $I19$0;
 HEAP32[$267>>2] = $I19$0$c;
 $268 = (($$0) + 20|0);
 HEAP32[$268>>2] = 0;
 $269 = (($$0) + 16|0);
 HEAP32[$269>>2] = 0;
 $270 = HEAP32[((10360 + 4|0))>>2]|0;
 $271 = 1 << $I19$0;
 $272 = $270 & $271;
 $273 = ($272|0)==(0);
 if ($273) {
  $274 = $270 | $271;
  HEAP32[((10360 + 4|0))>>2] = $274;
  HEAP32[$266>>2] = $$0;
  $275 = (($$0) + 24|0);
  HEAP32[$275>>2] = $266;
  $276 = (($$0) + 12|0);
  HEAP32[$276>>2] = $$0;
  $277 = (($$0) + 8|0);
  HEAP32[$277>>2] = $$0;
  STACKTOP = sp;return;
 }
 $278 = HEAP32[$266>>2]|0;
 $279 = ($I19$0|0)==(31);
 if ($279) {
  $287 = 0;
 } else {
  $280 = $I19$0 >>> 1;
  $281 = (25 - ($280))|0;
  $287 = $281;
 }
 $282 = (($278) + 4|0);
 $283 = HEAP32[$282>>2]|0;
 $284 = $283 & -8;
 $285 = ($284|0)==($$1|0);
 L194: do {
  if ($285) {
   $T$0$lcssa = $278;
  } else {
   $286 = $$1 << $287;
   $K20$049 = $286;$T$048 = $278;
   while(1) {
    $294 = $K20$049 >>> 31;
    $295 = ((($T$048) + ($294<<2)|0) + 16|0);
    $290 = HEAP32[$295>>2]|0;
    $296 = ($290|0)==(0|0);
    if ($296) {
     break;
    }
    $288 = $K20$049 << 1;
    $289 = (($290) + 4|0);
    $291 = HEAP32[$289>>2]|0;
    $292 = $291 & -8;
    $293 = ($292|0)==($$1|0);
    if ($293) {
     $T$0$lcssa = $290;
     break L194;
    } else {
     $K20$049 = $288;$T$048 = $290;
    }
   }
   $297 = HEAP32[((10360 + 16|0))>>2]|0;
   $298 = ($295>>>0)<($297>>>0);
   if ($298) {
    _abort();
    // unreachable;
   }
   HEAP32[$295>>2] = $$0;
   $299 = (($$0) + 24|0);
   HEAP32[$299>>2] = $T$048;
   $300 = (($$0) + 12|0);
   HEAP32[$300>>2] = $$0;
   $301 = (($$0) + 8|0);
   HEAP32[$301>>2] = $$0;
   STACKTOP = sp;return;
  }
 } while(0);
 $302 = (($T$0$lcssa) + 8|0);
 $303 = HEAP32[$302>>2]|0;
 $304 = HEAP32[((10360 + 16|0))>>2]|0;
 $305 = ($T$0$lcssa>>>0)<($304>>>0);
 if ($305) {
  _abort();
  // unreachable;
 }
 $306 = ($303>>>0)<($304>>>0);
 if ($306) {
  _abort();
  // unreachable;
 }
 $307 = (($303) + 12|0);
 HEAP32[$307>>2] = $$0;
 HEAP32[$302>>2] = $$0;
 $308 = (($$0) + 8|0);
 HEAP32[$308>>2] = $303;
 $309 = (($$0) + 12|0);
 HEAP32[$309>>2] = $T$0$lcssa;
 $310 = (($$0) + 24|0);
 HEAP32[$310>>2] = 0;
 STACKTOP = sp;return;
}
function _isspace($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($c|0)==(32);
 if ($0) {
  $4 = 1;
 } else {
  $1 = (($c) + -9)|0;
  $2 = ($1>>>0)<(5);
  $4 = $2;
 }
 $3 = $4&1;
 STACKTOP = sp;return ($3|0);
}
function _isupper($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -65)|0;
 $1 = ($0>>>0)<(26);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _tolower($c) {
 $c = $c|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_isupper($c)|0);
 $1 = ($0|0)==(0);
 $2 = $c | 32;
 $$0 = $1 ? $c : $2;
 STACKTOP = sp;return ($$0|0);
}
function ___intscan($f,$base,$pok,$0,$1) {
 $f = $f|0;
 $base = $base|0;
 $pok = $pok|0;
 $0 = $0|0;
 $1 = $1|0;
 var $$ = 0, $$1 = 0, $$121 = 0, $$122 = 0, $$18 = 0, $$19 = 0, $$base = 0, $$base16 = 0, $$lcssa = 0, $$sum = 0, $$sum10 = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum1352 = 0, $$sum14 = 0, $$sum15 = 0, $$sum9 = 0, $$sum934 = 0, $10 = 0;
 var $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0;
 var $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0;
 var $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0;
 var $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0;
 var $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0;
 var $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0;
 var $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c$0 = 0, $c$1 = 0, $c$123 = 0, $c$2$be = 0, $c$2$lcssa = 0, $c$3$be = 0, $c$3$lcssa = 0, $c$359 = 0, $c$4$be = 0, $c$4$lcssa = 0, $c$5$be = 0, $c$6$be = 0, $c$6$lcssa = 0, $c$7$be = 0, $c$744 = 0;
 var $c$8 = 0, $c$9$be = 0, $neg$0 = 0, $or$cond = 0, $or$cond17 = 0, $or$cond20 = 0, $or$cond26 = 0, $or$cond2628 = 0, $phitmp93 = 0, $x$068 = 0, $x$135 = 0, $x$253 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $2 = ($base>>>0)>(36);
 if ($2) {
  $5 = (___errno_location()|0);
  HEAP32[$5>>2] = 22;
  $278 = 0;$279 = 0;
  tempRet0 = $278;
  STACKTOP = sp;return ($279|0);
 }
 $3 = (($f) + 4|0);
 $4 = (($f) + 100|0);
 while(1) {
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$4>>2]|0;
  $8 = ($6>>>0)<($7>>>0);
  if ($8) {
   $9 = (($6) + 1|0);
   HEAP32[$3>>2] = $9;
   $10 = HEAP8[$6>>0]|0;
   $11 = $10&255;
   $13 = $11;
  } else {
   $12 = (___shgetc($f)|0);
   $13 = $12;
  }
  $14 = (_isspace($13)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   break;
  }
 }
 $16 = ($13|0)==(45);
 do {
  if ((($13|0) == 43) | (($13|0) == 45)) {
   $17 = $16 << 31 >> 31;
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$4>>2]|0;
   $20 = ($18>>>0)<($19>>>0);
   if ($20) {
    $21 = (($18) + 1|0);
    HEAP32[$3>>2] = $21;
    $22 = HEAP8[$18>>0]|0;
    $23 = $22&255;
    $c$0 = $23;$neg$0 = $17;
    break;
   } else {
    $24 = (___shgetc($f)|0);
    $c$0 = $24;$neg$0 = $17;
    break;
   }
  } else {
   $c$0 = $13;$neg$0 = 0;
  }
 } while(0);
 $25 = ($base|0)==(0);
 $26 = $base & -17;
 $27 = ($26|0)==(0);
 $28 = ($c$0|0)==(48);
 $or$cond = $27 & $28;
 do {
  if ($or$cond) {
   $29 = HEAP32[$3>>2]|0;
   $30 = HEAP32[$4>>2]|0;
   $31 = ($29>>>0)<($30>>>0);
   if ($31) {
    $32 = (($29) + 1|0);
    HEAP32[$3>>2] = $32;
    $33 = HEAP8[$29>>0]|0;
    $34 = $33&255;
    $37 = $34;
   } else {
    $35 = (___shgetc($f)|0);
    $37 = $35;
   }
   $36 = $37 | 32;
   $38 = ($36|0)==(120);
   if (!($38)) {
    $$base = $25 ? 8 : $base;
    $$1 = $$base;$c$1 = $37;
    label = 32;
    break;
   }
   $39 = HEAP32[$3>>2]|0;
   $40 = HEAP32[$4>>2]|0;
   $41 = ($39>>>0)<($40>>>0);
   if ($41) {
    $42 = (($39) + 1|0);
    HEAP32[$3>>2] = $42;
    $43 = HEAP8[$39>>0]|0;
    $44 = $43&255;
    $46 = $44;
   } else {
    $45 = (___shgetc($f)|0);
    $46 = $45;
   }
   $$sum15 = (($46) + 1)|0;
   $47 = (10856 + ($$sum15)|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = ($48&255)>(15);
   if ($49) {
    $50 = HEAP32[$4>>2]|0;
    $51 = ($50|0)==(0|0);
    if (!($51)) {
     $52 = HEAP32[$3>>2]|0;
     $53 = (($52) + -1|0);
     HEAP32[$3>>2] = $53;
    }
    $54 = ($pok|0)==(0);
    if ($54) {
     ___shlim($f,0);
     $278 = 0;$279 = 0;
     tempRet0 = $278;
     STACKTOP = sp;return ($279|0);
    }
    if ($51) {
     $278 = 0;$279 = 0;
     tempRet0 = $278;
     STACKTOP = sp;return ($279|0);
    }
    $55 = HEAP32[$3>>2]|0;
    $56 = (($55) + -1|0);
    HEAP32[$3>>2] = $56;
    $278 = 0;$279 = 0;
    tempRet0 = $278;
    STACKTOP = sp;return ($279|0);
   } else {
    $$122 = 16;$c$123 = $46;
    label = 47;
   }
  } else {
   $$base16 = $25 ? 10 : $base;
   $$sum = (($c$0) + 1)|0;
   $57 = (10856 + ($$sum)|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = ($59>>>0)<($$base16>>>0);
   if ($60) {
    $$1 = $$base16;$c$1 = $c$0;
    label = 32;
   } else {
    $61 = HEAP32[$4>>2]|0;
    $62 = ($61|0)==(0|0);
    if (!($62)) {
     $63 = HEAP32[$3>>2]|0;
     $64 = (($63) + -1|0);
     HEAP32[$3>>2] = $64;
    }
    ___shlim($f,0);
    $65 = (___errno_location()|0);
    HEAP32[$65>>2] = 22;
    $278 = 0;$279 = 0;
    tempRet0 = $278;
    STACKTOP = sp;return ($279|0);
   }
  }
 } while(0);
 if ((label|0) == 32) {
  $66 = ($$1|0)==(10);
  if ($66) {
   $67 = (($c$1) + -48)|0;
   $68 = ($67>>>0)<(10);
   if ($68) {
    $70 = $67;$x$068 = 0;
    while(1) {
     $69 = (($x$068) + ($70))|0;
     $71 = HEAP32[$3>>2]|0;
     $72 = HEAP32[$4>>2]|0;
     $73 = ($71>>>0)<($72>>>0);
     if ($73) {
      $74 = (($71) + 1|0);
      HEAP32[$3>>2] = $74;
      $75 = HEAP8[$71>>0]|0;
      $76 = $75&255;
      $c$2$be = $76;
     } else {
      $77 = (___shgetc($f)|0);
      $c$2$be = $77;
     }
     $78 = (($c$2$be) + -48)|0;
     $79 = ($78>>>0)<(10);
     $80 = ($69>>>0)<(429496729);
     $$ = $79 & $80;
     if (!($$)) {
      break;
     }
     $phitmp93 = ($69*10)|0;
     $70 = $78;$x$068 = $phitmp93;
    }
    $280 = $69;$281 = 0;$c$2$lcssa = $c$2$be;
   } else {
    $280 = 0;$281 = 0;$c$2$lcssa = $c$1;
   }
   $81 = (($c$2$lcssa) + -48)|0;
   $82 = ($81>>>0)<(10);
   if ($82) {
    $83 = $280;$84 = $281;$88 = $81;$c$359 = $c$2$lcssa;
    while(1) {
     $85 = (___muldi3(($83|0),($84|0),10,0)|0);
     $86 = tempRet0;
     $87 = ($88|0)<(0);
     $89 = $87 << 31 >> 31;
     $90 = $88 ^ -1;
     $91 = $89 ^ -1;
     $92 = ($86>>>0)>($91>>>0);
     $93 = ($86|0)==($91|0);
     $94 = ($85>>>0)>($90>>>0);
     $95 = $93 & $94;
     $96 = $92 | $95;
     if ($96) {
      $$lcssa = $88;$282 = $83;$283 = $84;$c$3$lcssa = $c$359;
      break;
     }
     $97 = (_i64Add(($85|0),($86|0),($88|0),($89|0))|0);
     $98 = tempRet0;
     $99 = HEAP32[$3>>2]|0;
     $100 = HEAP32[$4>>2]|0;
     $101 = ($99>>>0)<($100>>>0);
     if ($101) {
      $102 = (($99) + 1|0);
      HEAP32[$3>>2] = $102;
      $103 = HEAP8[$99>>0]|0;
      $104 = $103&255;
      $c$3$be = $104;
     } else {
      $105 = (___shgetc($f)|0);
      $c$3$be = $105;
     }
     $106 = (($c$3$be) + -48)|0;
     $107 = ($106>>>0)<(10);
     $108 = ($98>>>0)<(429496729);
     $109 = ($98|0)==(429496729);
     $110 = ($97>>>0)<(2576980378);
     $111 = $109 & $110;
     $112 = $108 | $111;
     $or$cond17 = $107 & $112;
     if ($or$cond17) {
      $83 = $97;$84 = $98;$88 = $106;$c$359 = $c$3$be;
     } else {
      $$lcssa = $106;$282 = $97;$283 = $98;$c$3$lcssa = $c$3$be;
      break;
     }
    }
    $113 = ($$lcssa>>>0)>(9);
    if ($113) {
     $252 = $283;$255 = $282;
    } else {
     $$121 = 10;$284 = $282;$285 = $283;$c$8 = $c$3$lcssa;
     label = 73;
    }
   } else {
    $252 = $281;$255 = $280;
   }
  } else {
   $$122 = $$1;$c$123 = $c$1;
   label = 47;
  }
 }
 L70: do {
  if ((label|0) == 47) {
   $114 = (($$122) + -1)|0;
   $115 = $114 & $$122;
   $116 = ($115|0)==(0);
   if ($116) {
    $121 = ($$122*23)|0;
    $122 = $121 >>> 5;
    $123 = $122 & 7;
    $124 = (11120 + ($123)|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = $125 << 24 >> 24;
    $$sum934 = (($c$123) + 1)|0;
    $127 = (10856 + ($$sum934)|0);
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = ($129>>>0)<($$122>>>0);
    if ($130) {
     $133 = $129;$x$135 = 0;
     while(1) {
      $131 = $x$135 << $126;
      $132 = $133 | $131;
      $134 = HEAP32[$3>>2]|0;
      $135 = HEAP32[$4>>2]|0;
      $136 = ($134>>>0)<($135>>>0);
      if ($136) {
       $137 = (($134) + 1|0);
       HEAP32[$3>>2] = $137;
       $138 = HEAP8[$134>>0]|0;
       $139 = $138&255;
       $c$4$be = $139;
      } else {
       $140 = (___shgetc($f)|0);
       $c$4$be = $140;
      }
      $$sum9 = (($c$4$be) + 1)|0;
      $141 = (10856 + ($$sum9)|0);
      $142 = HEAP8[$141>>0]|0;
      $143 = $142&255;
      $144 = ($143>>>0)<($$122>>>0);
      $145 = ($132>>>0)<(134217728);
      $$18 = $144 & $145;
      if ($$18) {
       $133 = $143;$x$135 = $132;
      } else {
       break;
      }
     }
     $149 = $142;$152 = 0;$155 = $132;$c$4$lcssa = $c$4$be;
    } else {
     $149 = $128;$152 = 0;$155 = 0;$c$4$lcssa = $c$123;
    }
    $146 = (_bitshift64Lshr(-1,-1,($126|0))|0);
    $147 = tempRet0;
    $148 = $149&255;
    $150 = ($148>>>0)>=($$122>>>0);
    $151 = ($152>>>0)>($147>>>0);
    $153 = ($152|0)==($147|0);
    $154 = ($155>>>0)>($146>>>0);
    $156 = $153 & $154;
    $157 = $151 | $156;
    $or$cond2628 = $150 | $157;
    if ($or$cond2628) {
     $$121 = $$122;$284 = $155;$285 = $152;$c$8 = $c$4$lcssa;
     label = 73;
     break;
    } else {
     $158 = $155;$159 = $152;$163 = $149;
    }
    while(1) {
     $160 = (_bitshift64Shl(($158|0),($159|0),($126|0))|0);
     $161 = tempRet0;
     $162 = $163&255;
     $164 = $162 | $160;
     $165 = HEAP32[$3>>2]|0;
     $166 = HEAP32[$4>>2]|0;
     $167 = ($165>>>0)<($166>>>0);
     if ($167) {
      $168 = (($165) + 1|0);
      HEAP32[$3>>2] = $168;
      $169 = HEAP8[$165>>0]|0;
      $170 = $169&255;
      $c$5$be = $170;
     } else {
      $171 = (___shgetc($f)|0);
      $c$5$be = $171;
     }
     $$sum10 = (($c$5$be) + 1)|0;
     $172 = (10856 + ($$sum10)|0);
     $173 = HEAP8[$172>>0]|0;
     $174 = $173&255;
     $175 = ($174>>>0)>=($$122>>>0);
     $176 = ($161>>>0)>($147>>>0);
     $177 = ($161|0)==($147|0);
     $178 = ($164>>>0)>($146>>>0);
     $179 = $177 & $178;
     $180 = $176 | $179;
     $or$cond26 = $175 | $180;
     if ($or$cond26) {
      $$121 = $$122;$284 = $164;$285 = $161;$c$8 = $c$5$be;
      label = 73;
      break L70;
     } else {
      $158 = $164;$159 = $161;$163 = $173;
     }
    }
   }
   $$sum1352 = (($c$123) + 1)|0;
   $117 = (10856 + ($$sum1352)|0);
   $118 = HEAP8[$117>>0]|0;
   $119 = $118&255;
   $120 = ($119>>>0)<($$122>>>0);
   if ($120) {
    $183 = $119;$x$253 = 0;
    while(1) {
     $181 = Math_imul($x$253, $$122)|0;
     $182 = (($183) + ($181))|0;
     $184 = HEAP32[$3>>2]|0;
     $185 = HEAP32[$4>>2]|0;
     $186 = ($184>>>0)<($185>>>0);
     if ($186) {
      $187 = (($184) + 1|0);
      HEAP32[$3>>2] = $187;
      $188 = HEAP8[$184>>0]|0;
      $189 = $188&255;
      $c$6$be = $189;
     } else {
      $190 = (___shgetc($f)|0);
      $c$6$be = $190;
     }
     $$sum13 = (($c$6$be) + 1)|0;
     $191 = (10856 + ($$sum13)|0);
     $192 = HEAP8[$191>>0]|0;
     $193 = $192&255;
     $194 = ($193>>>0)<($$122>>>0);
     $195 = ($182>>>0)<(119304647);
     $$19 = $194 & $195;
     if ($$19) {
      $183 = $193;$x$253 = $182;
     } else {
      break;
     }
    }
    $197 = $192;$286 = $182;$287 = 0;$c$6$lcssa = $c$6$be;
   } else {
    $197 = $118;$286 = 0;$287 = 0;$c$6$lcssa = $c$123;
   }
   $196 = $197&255;
   $198 = ($196>>>0)<($$122>>>0);
   if ($198) {
    $199 = (___udivdi3(-1,-1,($$122|0),0)|0);
    $200 = tempRet0;
    $202 = $287;$205 = $286;$211 = $197;$c$744 = $c$6$lcssa;
    while(1) {
     $201 = ($202>>>0)>($200>>>0);
     $203 = ($202|0)==($200|0);
     $204 = ($205>>>0)>($199>>>0);
     $206 = $203 & $204;
     $207 = $201 | $206;
     if ($207) {
      $$121 = $$122;$284 = $205;$285 = $202;$c$8 = $c$744;
      label = 73;
      break L70;
     }
     $208 = (___muldi3(($205|0),($202|0),($$122|0),0)|0);
     $209 = tempRet0;
     $210 = $211&255;
     $212 = $210 ^ -1;
     $213 = ($209>>>0)>(4294967295);
     $214 = ($209|0)==(-1);
     $215 = ($208>>>0)>($212>>>0);
     $216 = $214 & $215;
     $217 = $213 | $216;
     if ($217) {
      $$121 = $$122;$284 = $205;$285 = $202;$c$8 = $c$744;
      label = 73;
      break L70;
     }
     $218 = (_i64Add(($210|0),0,($208|0),($209|0))|0);
     $219 = tempRet0;
     $220 = HEAP32[$3>>2]|0;
     $221 = HEAP32[$4>>2]|0;
     $222 = ($220>>>0)<($221>>>0);
     if ($222) {
      $223 = (($220) + 1|0);
      HEAP32[$3>>2] = $223;
      $224 = HEAP8[$220>>0]|0;
      $225 = $224&255;
      $c$7$be = $225;
     } else {
      $226 = (___shgetc($f)|0);
      $c$7$be = $226;
     }
     $$sum14 = (($c$7$be) + 1)|0;
     $227 = (10856 + ($$sum14)|0);
     $228 = HEAP8[$227>>0]|0;
     $229 = $228&255;
     $230 = ($229>>>0)<($$122>>>0);
     if ($230) {
      $202 = $219;$205 = $218;$211 = $228;$c$744 = $c$7$be;
     } else {
      $$121 = $$122;$284 = $218;$285 = $219;$c$8 = $c$7$be;
      label = 73;
      break;
     }
    }
   } else {
    $$121 = $$122;$284 = $286;$285 = $287;$c$8 = $c$6$lcssa;
    label = 73;
   }
  }
 } while(0);
 if ((label|0) == 73) {
  $$sum11 = (($c$8) + 1)|0;
  $231 = (10856 + ($$sum11)|0);
  $232 = HEAP8[$231>>0]|0;
  $233 = $232&255;
  $234 = ($233>>>0)<($$121>>>0);
  if ($234) {
   while(1) {
    $235 = HEAP32[$3>>2]|0;
    $236 = HEAP32[$4>>2]|0;
    $237 = ($235>>>0)<($236>>>0);
    if ($237) {
     $238 = (($235) + 1|0);
     HEAP32[$3>>2] = $238;
     $239 = HEAP8[$235>>0]|0;
     $240 = $239&255;
     $c$9$be = $240;
    } else {
     $241 = (___shgetc($f)|0);
     $c$9$be = $241;
    }
    $$sum12 = (($c$9$be) + 1)|0;
    $242 = (10856 + ($$sum12)|0);
    $243 = HEAP8[$242>>0]|0;
    $244 = $243&255;
    $245 = ($244>>>0)<($$121>>>0);
    if (!($245)) {
     break;
    }
   }
   $246 = (___errno_location()|0);
   HEAP32[$246>>2] = 34;
   $252 = $1;$255 = $0;
  } else {
   $252 = $285;$255 = $284;
  }
 }
 $247 = HEAP32[$4>>2]|0;
 $248 = ($247|0)==(0|0);
 if (!($248)) {
  $249 = HEAP32[$3>>2]|0;
  $250 = (($249) + -1|0);
  HEAP32[$3>>2] = $250;
 }
 $251 = ($252>>>0)<($1>>>0);
 $253 = ($252|0)==($1|0);
 $254 = ($255>>>0)<($0>>>0);
 $256 = $253 & $254;
 $257 = $251 | $256;
 if (!($257)) {
  $258 = $0 & 1;
  $259 = ($258|0)==(0);
  $260 = (0)==(0);
  $261 = $259 & $260;
  $262 = ($neg$0|0)==(0);
  $or$cond20 = $261 & $262;
  if ($or$cond20) {
   $263 = (___errno_location()|0);
   HEAP32[$263>>2] = 34;
   $264 = (_i64Add(($0|0),($1|0),-1,-1)|0);
   $265 = tempRet0;
   $278 = $265;$279 = $264;
   tempRet0 = $278;
   STACKTOP = sp;return ($279|0);
  }
  $266 = ($252>>>0)>($1>>>0);
  $267 = ($252|0)==($1|0);
  $268 = ($255>>>0)>($0>>>0);
  $269 = $267 & $268;
  $270 = $266 | $269;
  if ($270) {
   $271 = (___errno_location()|0);
   HEAP32[$271>>2] = 34;
   $278 = $1;$279 = $0;
   tempRet0 = $278;
   STACKTOP = sp;return ($279|0);
  }
 }
 $272 = ($neg$0|0)<(0);
 $273 = $272 << 31 >> 31;
 $274 = $255 ^ $neg$0;
 $275 = $252 ^ $273;
 $276 = (_i64Subtract(($274|0),($275|0),($neg$0|0),($273|0))|0);
 $277 = tempRet0;
 $278 = $277;$279 = $276;
 tempRet0 = $278;
 STACKTOP = sp;return ($279|0);
}
function ___floatscan($f,$prec,$pok) {
 $f = $f|0;
 $prec = $prec|0;
 $pok = $pok|0;
 var $$$i = 0, $$$i22 = 0, $$0 = 0.0, $$02$i = 0, $$08$i = 0, $$09$i = 0, $$1$be$i = 0, $$1$ph$i = 0, $$10$i = 0, $$14$i = 0, $$2$i = 0, $$3$be$i = 0, $$3$lcssa$i = 0, $$38$i = 0, $$388$i = 0, $$397$i = 0, $$in = 0, $$k$0$i = 0, $$lcssa43$i = 0, $$lnz$0$i = 0;
 var $$neg20$i = 0, $$pn$i = 0.0, $$pre$i = 0.0, $$pre$i17 = 0, $$pre$phi$iZ2D = 0.0, $$promoted$i = 0, $$sink$off0$us$i = 0, $$sink$off0$us53$i = 0, $$x$4$i = 0, $$y$3$i = 0.0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0;
 var $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0;
 var $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0;
 var $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0;
 var $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0;
 var $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0, $187 = 0, $188 = 0.0, $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0;
 var $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0;
 var $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0;
 var $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0;
 var $251 = 0, $252 = 0, $253 = 0, $254 = 0.0, $255 = 0.0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0.0, $266 = 0, $267 = 0, $268 = 0, $269 = 0;
 var $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0.0, $277 = 0.0, $278 = 0.0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0.0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0;
 var $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0.0;
 var $305 = 0.0, $306 = 0.0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0.0, $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0.0, $318 = 0, $319 = 0, $32 = 0, $320 = 0.0, $321 = 0, $322 = 0;
 var $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0;
 var $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0;
 var $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0;
 var $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0;
 var $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0;
 var $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0.0, $418 = 0.0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0;
 var $431 = 0, $432 = 0, $433 = 0, $434 = 0.0, $435 = 0.0, $436 = 0.0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0.0, $447 = 0.0, $448 = 0.0, $449 = 0;
 var $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0.0, $459 = 0.0, $46 = 0, $460 = 0.0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0;
 var $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0.0, $473 = 0, $474 = 0.0, $475 = 0.0, $476 = 0, $477 = 0.0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0, $482 = 0, $483 = 0, $484 = 0.0, $485 = 0.0;
 var $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0.0, $490 = 0, $491 = 0.0, $492 = 0.0, $493 = 0.0, $494 = 0, $495 = 0, $496 = 0, $497 = 0.0, $498 = 0.0, $499 = 0, $5 = 0, $50 = 0.0, $500 = 0, $501 = 0, $502 = 0;
 var $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0.0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0;
 var $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0;
 var $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0;
 var $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0;
 var $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0;
 var $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0;
 var $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0;
 var $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0.0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0;
 var $648 = 0.0, $649 = 0.0, $65 = 0, $650 = 0.0, $651 = 0, $652 = 0.0, $653 = 0.0, $654 = 0.0, $655 = 0.0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0;
 var $666 = 0.0, $667 = 0.0, $668 = 0.0, $669 = 0, $67 = 0, $670 = 0.0, $671 = 0.0, $672 = 0, $673 = 0, $674 = 0, $675 = 0.0, $676 = 0.0, $677 = 0.0, $678 = 0.0, $679 = 0, $68 = 0, $680 = 0, $681 = 0.0, $682 = 0, $683 = 0.0;
 var $684 = 0.0, $685 = 0.0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0.0, $691 = 0, $692 = 0, $693 = 0, $694 = 0.0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0;
 var $701 = 0.0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0.0, $707 = 0, $708 = 0, $709 = 0.0, $71 = 0, $710 = 0.0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0;
 var $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a$0$lcssa161$i = 0, $a$061$i = 0, $a$1$i = 0, $a$2$ph40$i = 0;
 var $a$3$i = 0, $a$3$i$ph = 0, $a$3$ph$i = 0, $a$427$i = 0, $a$5$i = 0, $bias$0$i = 0.0, $bias$07$i = 0.0, $bits$0$ph = 0, $brmerge = 0, $brmerge107 = 0, $c$0 = 0, $c$0$i = 0, $c$1$lcssa = 0, $c$1$ph$i = 0, $c$169 = 0, $c$2 = 0, $c$2$i = 0, $c$2$lcssa$i = 0, $c$364 = 0, $c$4 = 0;
 var $c$5 = 0, $c$6 = 0, $carry$063$i = 0, $carry1$0$us$i = 0, $carry1$0$us49$i = 0, $carry1$1$lcssa$lcssa$i = 0, $carry1$1$us$i = 0, $carry1$1$us54$i = 0, $carry3$030$i = 0, $cond$i = 0, $d$0$i = 0, $denormal$0$i = 0, $denormal$1$i = 0, $denormal$2$i = 0, $e2$0$ph$i = 0, $e2$0$us$i = 0, $e2$0$us44$i = 0, $e2$1$i = 0, $e2$1$i$ph = 0, $e2$1$ph$i = 0;
 var $e2$2$i = 0, $e2$3$i = 0, $emin$0$ph = 0, $exitcond$i = 0, $frac$0$i = 0.0, $frac$1$i = 0.0, $frac$2$i = 0.0, $gotdig$0$i = 0, $gotdig$0$i9 = 0, $gotdig$2$i = 0, $gotdig$2$i11 = 0, $gotdig$3$i = 0, $gotdig$3$lcssa$i = 0, $gotdig$381$i = 0, $gotdig$393$i = 0, $gotdig$4$i = 0, $gotrad$0$i = 0, $gotrad$0$i12 = 0, $gotrad$1$i = 0, $gotrad$1$lcssa$i = 0;
 var $gotrad$194$i = 0, $gotrad$2$i = 0, $gottail$0$i = 0, $gottail$1$i = 0, $gottail$2$i = 0, $i$0$lcssa = 0, $i$025$i = 0, $i$068 = 0, $i$1$i = 0, $i$166 = 0, $i$263 = 0, $i$3 = 0, $i$4 = 0, $j$0$lcssa$i = 0, $j$086$i = 0, $j$096$i = 0, $j$2$i = 0, $j$371$i = 0, $k$0$lcssa$i = 0, $k$084$i = 0;
 var $k$095$i = 0, $k$2$i = 0, $k$3$i = 0, $k$462$i = 0, $k$5$in$us$i = 0, $k$5$in$us48$i = 0, $k$5$us$i = 0, $k$5$us50$i = 0, $k$5$z$2$us$i = 0, $k$5$z$2$us56$i = 0, $k$628$i = 0, $lnz$0$lcssa$i = 0, $lnz$079$i = 0, $lnz$092$i = 0, $lnz$2$i = 0, $notlhs = 0, $notrhs = 0, $or$cond = 0, $or$cond$i = 0, $or$cond$i15 = 0;
 var $or$cond106$not = 0, $or$cond11$i = 0, $or$cond12$us$i = 0, $or$cond12$us55$i = 0, $or$cond13$i = 0, $or$cond15$i = 0, $or$cond16$i = 0, $or$cond17$i = 0, $or$cond3$i = 0, $or$cond3$i16 = 0, $or$cond4$i = 0, $or$cond5 = 0, $or$cond7$i = 0, $or$cond789$i = 0, $or$cond8 = 0, $rp$0$lcssa162$i = 0, $rp$060$i = 0, $rp$1$i18 = 0, $rp$2$ph38$i = 0, $rp$3$i$ph = 0;
 var $rp$3$ph33$i = 0, $rp$426$i = 0, $rp$5$i = 0, $scale$0$i = 0.0, $scale$1$i = 0.0, $scale$2$i = 0.0, $sign$0 = 0, $storemerge$i = 0, $sum$i = 0, $x$0$i = 0, $x$1$i = 0, $x$2$i = 0, $x$3$lcssa$i = 0, $x$313$i = 0, $x$4$lcssa$i = 0, $x$48$i = 0, $x$5$i = 0, $x$6$i = 0, $x$i = 0, $y$0$i = 0.0;
 var $y$1$i = 0.0, $y$1$i23 = 0.0, $y$2$i = 0.0, $y$2$i24 = 0.0, $y$3$i = 0.0, $y$3$lcssa$i = 0.0, $y$39$i = 0.0, $y$4$i = 0.0, $y$5$i = 0.0, $z$0$i = 0, $z$1$ph39$i = 0, $z$1$us$i = 0, $z$1$us45$i = 0, $z$2$us$i = 0, $z$2$us47$i = 0, $z$3$lcssa$lcssa$i = 0, $z$3$us$i = 0, $z$3$us57$i = 0, $z$4$i = 0, $z$5$ph$i = 0;
 var $z$7$1$i = 0, $z$7$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 512|0;
 $x$i = sp;
 if ((($prec|0) == 1)) {
  $bits$0$ph = 53;$emin$0$ph = -1074;
 } else if ((($prec|0) == 0)) {
  $bits$0$ph = 24;$emin$0$ph = -149;
 } else if ((($prec|0) == 2)) {
  $bits$0$ph = 53;$emin$0$ph = -1074;
 } else {
  $$0 = 0.0;
  STACKTOP = sp;return (+$$0);
 }
 $0 = (($f) + 4|0);
 $1 = (($f) + 100|0);
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = ($2>>>0)<($3>>>0);
  if ($4) {
   $5 = (($2) + 1|0);
   HEAP32[$0>>2] = $5;
   $6 = HEAP8[$2>>0]|0;
   $7 = $6&255;
   $9 = $7;
  } else {
   $8 = (___shgetc($f)|0);
   $9 = $8;
  }
  $10 = (_isspace($9)|0);
  $11 = ($10|0)==(0);
  if ($11) {
   break;
  }
 }
 $12 = ($9|0)==(45);
 do {
  if ((($9|0) == 43) | (($9|0) == 45)) {
   $13 = $12&1;
   $14 = $13 << 1;
   $15 = (1 - ($14))|0;
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ($16>>>0)<($17>>>0);
   if ($18) {
    $19 = (($16) + 1|0);
    HEAP32[$0>>2] = $19;
    $20 = HEAP8[$16>>0]|0;
    $21 = $20&255;
    $c$0 = $21;$sign$0 = $15;
    break;
   } else {
    $22 = (___shgetc($f)|0);
    $c$0 = $22;$sign$0 = $15;
    break;
   }
  } else {
   $c$0 = $9;$sign$0 = 1;
  }
 } while(0);
 $c$169 = $c$0;$i$068 = 0;
 while(1) {
  $23 = $c$169 | 32;
  $24 = (11136 + ($i$068)|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($23|0)==($26|0);
  if (!($27)) {
   $c$1$lcssa = $c$169;$i$0$lcssa = $i$068;
   break;
  }
  $28 = ($i$068>>>0)<(7);
  do {
   if ($28) {
    $29 = HEAP32[$0>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = ($29>>>0)<($30>>>0);
    if ($31) {
     $32 = (($29) + 1|0);
     HEAP32[$0>>2] = $32;
     $33 = HEAP8[$29>>0]|0;
     $34 = $33&255;
     $c$2 = $34;
     break;
    } else {
     $35 = (___shgetc($f)|0);
     $c$2 = $35;
     break;
    }
   } else {
    $c$2 = $c$169;
   }
  } while(0);
  $36 = (($i$068) + 1)|0;
  $37 = ($36>>>0)<(8);
  if ($37) {
   $c$169 = $c$2;$i$068 = $36;
  } else {
   $c$1$lcssa = $c$2;$i$0$lcssa = $36;
   break;
  }
 }
 do {
  if ((($i$0$lcssa|0) == 3)) {
   label = 23;
  } else if (!((($i$0$lcssa|0) == 8))) {
   $38 = ($i$0$lcssa>>>0)<(4);
   $39 = ($pok|0)==(0);
   $or$cond = $38 | $39;
   if (!($or$cond)) {
    $40 = ($i$0$lcssa|0)==(8);
    if ($40) {
     break;
    } else {
     label = 23;
     break;
    }
   }
   $52 = ($i$0$lcssa|0)==(0);
   L34: do {
    if ($52) {
     $c$364 = $c$1$lcssa;$i$263 = 0;
     while(1) {
      $53 = $c$364 | 32;
      $54 = (11152 + ($i$263)|0);
      $55 = HEAP8[$54>>0]|0;
      $56 = $55 << 24 >> 24;
      $57 = ($53|0)==($56|0);
      if (!($57)) {
       $c$5 = $c$364;$i$3 = $i$263;
       break L34;
      }
      $58 = ($i$263>>>0)<(2);
      do {
       if ($58) {
        $59 = HEAP32[$0>>2]|0;
        $60 = HEAP32[$1>>2]|0;
        $61 = ($59>>>0)<($60>>>0);
        if ($61) {
         $62 = (($59) + 1|0);
         HEAP32[$0>>2] = $62;
         $63 = HEAP8[$59>>0]|0;
         $64 = $63&255;
         $c$4 = $64;
         break;
        } else {
         $65 = (___shgetc($f)|0);
         $c$4 = $65;
         break;
        }
       } else {
        $c$4 = $c$364;
       }
      } while(0);
      $66 = (($i$263) + 1)|0;
      $67 = ($66>>>0)<(3);
      if ($67) {
       $c$364 = $c$4;$i$263 = $66;
      } else {
       $c$5 = $c$4;$i$3 = $66;
       break;
      }
     }
    } else {
     $c$5 = $c$1$lcssa;$i$3 = $i$0$lcssa;
    }
   } while(0);
   if ((($i$3|0) == 3)) {
    $68 = HEAP32[$0>>2]|0;
    $69 = HEAP32[$1>>2]|0;
    $70 = ($68>>>0)<($69>>>0);
    if ($70) {
     $71 = (($68) + 1|0);
     HEAP32[$0>>2] = $71;
     $72 = HEAP8[$68>>0]|0;
     $73 = $72&255;
     $76 = $73;
    } else {
     $74 = (___shgetc($f)|0);
     $76 = $74;
    }
    $75 = ($76|0)==(40);
    if ($75) {
     $i$4 = 1;
    } else {
     $77 = HEAP32[$1>>2]|0;
     $78 = ($77|0)==(0|0);
     if ($78) {
      $$0 = nan;
      STACKTOP = sp;return (+$$0);
     }
     $79 = HEAP32[$0>>2]|0;
     $80 = (($79) + -1|0);
     HEAP32[$0>>2] = $80;
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    }
    while(1) {
     $81 = HEAP32[$0>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = ($81>>>0)<($82>>>0);
     if ($83) {
      $84 = (($81) + 1|0);
      HEAP32[$0>>2] = $84;
      $85 = HEAP8[$81>>0]|0;
      $86 = $85&255;
      $89 = $86;
     } else {
      $87 = (___shgetc($f)|0);
      $89 = $87;
     }
     $88 = (($89) + -48)|0;
     $90 = ($88>>>0)<(10);
     $91 = (($89) + -65)|0;
     $92 = ($91>>>0)<(26);
     $or$cond8 = $90 | $92;
     if (!($or$cond8)) {
      $93 = (($89) + -97)|0;
      $94 = ($93>>>0)<(26);
      $95 = ($89|0)==(95);
      $or$cond5 = $94 | $95;
      if (!($or$cond5)) {
       break;
      }
     }
     $107 = (($i$4) + 1)|0;
     $i$4 = $107;
    }
    $96 = ($89|0)==(41);
    if ($96) {
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    }
    $97 = HEAP32[$1>>2]|0;
    $98 = ($97|0)==(0|0);
    if (!($98)) {
     $99 = HEAP32[$0>>2]|0;
     $100 = (($99) + -1|0);
     HEAP32[$0>>2] = $100;
    }
    if ($39) {
     $102 = (___errno_location()|0);
     HEAP32[$102>>2] = 22;
     ___shlim($f,0);
     $$0 = 0.0;
     STACKTOP = sp;return (+$$0);
    }
    $101 = ($i$4|0)==(0);
    $brmerge107 = $101 | $98;
    if ($brmerge107) {
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    } else {
     $$in = $i$4;
    }
    while(1) {
     $103 = (($$in) + -1)|0;
     $104 = HEAP32[$0>>2]|0;
     $105 = (($104) + -1|0);
     HEAP32[$0>>2] = $105;
     $106 = ($103|0)==(0);
     if ($106) {
      $$0 = nan;
      break;
     } else {
      $$in = $103;
     }
    }
    STACKTOP = sp;return (+$$0);
   } else if ((($i$3|0) == 0)) {
    $113 = ($c$5|0)==(48);
    do {
     if ($113) {
      $114 = HEAP32[$0>>2]|0;
      $115 = HEAP32[$1>>2]|0;
      $116 = ($114>>>0)<($115>>>0);
      if ($116) {
       $117 = (($114) + 1|0);
       HEAP32[$0>>2] = $117;
       $118 = HEAP8[$114>>0]|0;
       $119 = $118&255;
       $122 = $119;
      } else {
       $120 = (___shgetc($f)|0);
       $122 = $120;
      }
      $121 = $122 | 32;
      $123 = ($121|0)==(120);
      if (!($123)) {
       $321 = HEAP32[$1>>2]|0;
       $322 = ($321|0)==(0|0);
       if ($322) {
        $c$6 = 48;
        break;
       }
       $323 = HEAP32[$0>>2]|0;
       $324 = (($323) + -1|0);
       HEAP32[$0>>2] = $324;
       $c$6 = 48;
       break;
      }
      $124 = HEAP32[$0>>2]|0;
      $125 = HEAP32[$1>>2]|0;
      $126 = ($124>>>0)<($125>>>0);
      if ($126) {
       $127 = (($124) + 1|0);
       HEAP32[$0>>2] = $127;
       $128 = HEAP8[$124>>0]|0;
       $129 = $128&255;
       $c$0$i = $129;$gotdig$0$i = 0;
      } else {
       $130 = (___shgetc($f)|0);
       $c$0$i = $130;$gotdig$0$i = 0;
      }
      while(1) {
       if ((($c$0$i|0) == 46)) {
        label = 70;
        break;
       } else if (!((($c$0$i|0) == 48))) {
        $169 = 0;$172 = 0;$212 = 0;$214 = 0;$c$2$i = $c$0$i;$gotdig$2$i = $gotdig$0$i;$gotrad$0$i = 0;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
        break;
       }
       $131 = HEAP32[$0>>2]|0;
       $132 = HEAP32[$1>>2]|0;
       $133 = ($131>>>0)<($132>>>0);
       if ($133) {
        $134 = (($131) + 1|0);
        HEAP32[$0>>2] = $134;
        $135 = HEAP8[$131>>0]|0;
        $136 = $135&255;
        $c$0$i = $136;$gotdig$0$i = 1;
        continue;
       } else {
        $137 = (___shgetc($f)|0);
        $c$0$i = $137;$gotdig$0$i = 1;
        continue;
       }
      }
      L107: do {
       if ((label|0) == 70) {
        $138 = HEAP32[$0>>2]|0;
        $139 = HEAP32[$1>>2]|0;
        $140 = ($138>>>0)<($139>>>0);
        if ($140) {
         $141 = (($138) + 1|0);
         HEAP32[$0>>2] = $141;
         $142 = HEAP8[$138>>0]|0;
         $143 = $142&255;
         $c$1$ph$i = $143;
        } else {
         $144 = (___shgetc($f)|0);
         $c$1$ph$i = $144;
        }
        $145 = ($c$1$ph$i|0)==(48);
        if ($145) {
         $155 = -1;$156 = -1;
         while(1) {
          $146 = HEAP32[$0>>2]|0;
          $147 = HEAP32[$1>>2]|0;
          $148 = ($146>>>0)<($147>>>0);
          if ($148) {
           $149 = (($146) + 1|0);
           HEAP32[$0>>2] = $149;
           $150 = HEAP8[$146>>0]|0;
           $151 = $150&255;
           $154 = $151;
          } else {
           $152 = (___shgetc($f)|0);
           $154 = $152;
          }
          $153 = ($154|0)==(48);
          if (!($153)) {
           $169 = 0;$172 = 0;$212 = $155;$214 = $156;$c$2$i = $154;$gotdig$2$i = 1;$gotrad$0$i = 1;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
           break L107;
          }
          $157 = (_i64Add(($155|0),($156|0),-1,-1)|0);
          $158 = tempRet0;
          $155 = $157;$156 = $158;
         }
        } else {
         $169 = 0;$172 = 0;$212 = 0;$214 = 0;$c$2$i = $c$1$ph$i;$gotdig$2$i = $gotdig$0$i;$gotrad$0$i = 1;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
        }
       }
      } while(0);
      L120: while(1) {
       $159 = (($c$2$i) + -48)|0;
       $160 = ($159>>>0)<(10);
       do {
        if ($160) {
         $d$0$i = $159;
         label = 84;
        } else {
         $161 = $c$2$i | 32;
         $162 = (($161) + -97)|0;
         $163 = ($162>>>0)<(6);
         $164 = ($c$2$i|0)==(46);
         $or$cond$i = $163 | $164;
         if (!($or$cond$i)) {
          $c$2$lcssa$i = $c$2$i;
          break L120;
         }
         if ($164) {
          $165 = ($gotrad$0$i|0)==(0);
          if ($165) {
           $712 = $172;$713 = $169;$714 = $172;$715 = $169;$gotdig$3$i = $gotdig$2$i;$gotrad$1$i = 1;$gottail$2$i = $gottail$0$i;$scale$2$i = $scale$0$i;$x$2$i = $x$0$i;$y$2$i = $y$0$i;
           break;
          } else {
           $c$2$lcssa$i = 46;
           break L120;
          }
         } else {
          $166 = ($c$2$i|0)>(57);
          $167 = (($161) + -87)|0;
          $$38$i = $166 ? $167 : $159;
          $d$0$i = $$38$i;
          label = 84;
          break;
         }
        }
       } while(0);
       if ((label|0) == 84) {
        label = 0;
        $168 = ($169|0)<(0);
        $170 = ($169|0)==(0);
        $171 = ($172>>>0)<(8);
        $173 = $170 & $171;
        $174 = $168 | $173;
        do {
         if ($174) {
          $175 = $x$0$i << 4;
          $176 = (($d$0$i) + ($175))|0;
          $gottail$1$i = $gottail$0$i;$scale$1$i = $scale$0$i;$x$1$i = $176;$y$1$i = $y$0$i;
         } else {
          $177 = ($169|0)<(0);
          $178 = ($169|0)==(0);
          $179 = ($172>>>0)<(14);
          $180 = $178 & $179;
          $181 = $177 | $180;
          if ($181) {
           $182 = (+($d$0$i|0));
           $183 = $scale$0$i * 0.0625;
           $184 = $183 * $182;
           $185 = $y$0$i + $184;
           $gottail$1$i = $gottail$0$i;$scale$1$i = $183;$x$1$i = $x$0$i;$y$1$i = $185;
           break;
          }
          $186 = ($d$0$i|0)!=(0);
          $187 = ($gottail$0$i|0)==(0);
          $or$cond3$i = $186 & $187;
          if ($or$cond3$i) {
           $188 = $scale$0$i * 0.5;
           $189 = $y$0$i + $188;
           $gottail$1$i = 1;$scale$1$i = $scale$0$i;$x$1$i = $x$0$i;$y$1$i = $189;
          } else {
           $gottail$1$i = $gottail$0$i;$scale$1$i = $scale$0$i;$x$1$i = $x$0$i;$y$1$i = $y$0$i;
          }
         }
        } while(0);
        $190 = (_i64Add(($172|0),($169|0),1,0)|0);
        $191 = tempRet0;
        $712 = $212;$713 = $214;$714 = $190;$715 = $191;$gotdig$3$i = 1;$gotrad$1$i = $gotrad$0$i;$gottail$2$i = $gottail$1$i;$scale$2$i = $scale$1$i;$x$2$i = $x$1$i;$y$2$i = $y$1$i;
       }
       $192 = HEAP32[$0>>2]|0;
       $193 = HEAP32[$1>>2]|0;
       $194 = ($192>>>0)<($193>>>0);
       if ($194) {
        $195 = (($192) + 1|0);
        HEAP32[$0>>2] = $195;
        $196 = HEAP8[$192>>0]|0;
        $197 = $196&255;
        $169 = $715;$172 = $714;$212 = $712;$214 = $713;$c$2$i = $197;$gotdig$2$i = $gotdig$3$i;$gotrad$0$i = $gotrad$1$i;$gottail$0$i = $gottail$2$i;$scale$0$i = $scale$2$i;$x$0$i = $x$2$i;$y$0$i = $y$2$i;
        continue;
       } else {
        $198 = (___shgetc($f)|0);
        $169 = $715;$172 = $714;$212 = $712;$214 = $713;$c$2$i = $198;$gotdig$2$i = $gotdig$3$i;$gotrad$0$i = $gotrad$1$i;$gottail$0$i = $gottail$2$i;$scale$0$i = $scale$2$i;$x$0$i = $x$2$i;$y$0$i = $y$2$i;
        continue;
       }
      }
      $199 = ($gotdig$2$i|0)==(0);
      if ($199) {
       $200 = HEAP32[$1>>2]|0;
       $201 = ($200|0)==(0|0);
       if (!($201)) {
        $202 = HEAP32[$0>>2]|0;
        $203 = (($202) + -1|0);
        HEAP32[$0>>2] = $203;
       }
       if ($39) {
        ___shlim($f,0);
       } else {
        if (!($201)) {
         $204 = HEAP32[$0>>2]|0;
         $205 = (($204) + -1|0);
         HEAP32[$0>>2] = $205;
         $206 = ($gotrad$0$i|0)==(0);
         if (!($206)) {
          $207 = (($204) + -2|0);
          HEAP32[$0>>2] = $207;
         }
        }
       }
       $208 = (+($sign$0|0));
       $209 = $208 * 0.0;
       $$0 = $209;
       STACKTOP = sp;return (+$$0);
      }
      $210 = ($gotrad$0$i|0)==(0);
      $211 = $210 ? $172 : $212;
      $213 = $210 ? $169 : $214;
      $215 = ($169|0)<(0);
      $216 = ($169|0)==(0);
      $217 = ($172>>>0)<(8);
      $218 = $216 & $217;
      $219 = $215 | $218;
      if ($219) {
       $221 = $172;$222 = $169;$x$313$i = $x$0$i;
       while(1) {
        $220 = $x$313$i << 4;
        $223 = (_i64Add(($221|0),($222|0),1,0)|0);
        $224 = tempRet0;
        $225 = ($224|0)<(0);
        $226 = ($224|0)==(0);
        $227 = ($223>>>0)<(8);
        $228 = $226 & $227;
        $229 = $225 | $228;
        if ($229) {
         $221 = $223;$222 = $224;$x$313$i = $220;
        } else {
         $x$3$lcssa$i = $220;
         break;
        }
       }
      } else {
       $x$3$lcssa$i = $x$0$i;
      }
      $230 = $c$2$lcssa$i | 32;
      $231 = ($230|0)==(112);
      do {
       if ($231) {
        $232 = (_scanexp($f,$pok)|0);
        $233 = tempRet0;
        $234 = ($232|0)==(0);
        $235 = ($233|0)==(-2147483648);
        $236 = $234 & $235;
        if ($236) {
         if ($39) {
          ___shlim($f,0);
          $$0 = 0.0;
          STACKTOP = sp;return (+$$0);
         } else {
          $237 = HEAP32[$1>>2]|0;
          $238 = ($237|0)==(0|0);
          if ($238) {
           $249 = 0;$250 = 0;
           break;
          }
          $239 = HEAP32[$0>>2]|0;
          $240 = (($239) + -1|0);
          HEAP32[$0>>2] = $240;
          $249 = 0;$250 = 0;
          break;
         }
        } else {
         $249 = $232;$250 = $233;
        }
       } else {
        $241 = HEAP32[$1>>2]|0;
        $242 = ($241|0)==(0|0);
        if ($242) {
         $249 = 0;$250 = 0;
        } else {
         $243 = HEAP32[$0>>2]|0;
         $244 = (($243) + -1|0);
         HEAP32[$0>>2] = $244;
         $249 = 0;$250 = 0;
        }
       }
      } while(0);
      $245 = (_bitshift64Shl(($211|0),($213|0),2)|0);
      $246 = tempRet0;
      $247 = (_i64Add(($245|0),($246|0),-32,-1)|0);
      $248 = tempRet0;
      $251 = (_i64Add(($247|0),($248|0),($249|0),($250|0))|0);
      $252 = tempRet0;
      $253 = ($x$3$lcssa$i|0)==(0);
      if ($253) {
       $254 = (+($sign$0|0));
       $255 = $254 * 0.0;
       $$0 = $255;
       STACKTOP = sp;return (+$$0);
      }
      $256 = (0 - ($emin$0$ph))|0;
      $257 = ($252|0)>(0);
      $258 = ($252|0)==(0);
      $259 = ($251>>>0)>($256>>>0);
      $260 = $258 & $259;
      $261 = $257 | $260;
      if ($261) {
       $262 = (___errno_location()|0);
       HEAP32[$262>>2] = 34;
       $263 = (+($sign$0|0));
       $264 = $263 * 1.79769313486231570815E+308;
       $265 = $264 * 1.79769313486231570815E+308;
       $$0 = $265;
       STACKTOP = sp;return (+$$0);
      }
      $266 = (($emin$0$ph) + -106)|0;
      $267 = ($266|0)<(0);
      $268 = $267 << 31 >> 31;
      $269 = ($252|0)<($268|0);
      $270 = ($252|0)==($268|0);
      $271 = ($251>>>0)<($266>>>0);
      $272 = $270 & $271;
      $273 = $269 | $272;
      if ($273) {
       $275 = (___errno_location()|0);
       HEAP32[$275>>2] = 34;
       $276 = (+($sign$0|0));
       $277 = $276 * 2.22507385850720138309E-308;
       $278 = $277 * 2.22507385850720138309E-308;
       $$0 = $278;
       STACKTOP = sp;return (+$$0);
      }
      $274 = ($x$3$lcssa$i|0)>(-1);
      if ($274) {
       $283 = $251;$284 = $252;$x$48$i = $x$3$lcssa$i;$y$39$i = $y$0$i;
       while(1) {
        $279 = !($y$39$i >= 0.5);
        $280 = $x$48$i << 1;
        if ($279) {
         $$pn$i = $y$39$i;$x$5$i = $280;
        } else {
         $281 = $280 | 1;
         $282 = $y$39$i + -1.0;
         $$pn$i = $282;$x$5$i = $281;
        }
        $y$4$i = $y$39$i + $$pn$i;
        $285 = (_i64Add(($283|0),($284|0),-1,-1)|0);
        $286 = tempRet0;
        $287 = ($x$5$i|0)>(-1);
        if ($287) {
         $283 = $285;$284 = $286;$x$48$i = $x$5$i;$y$39$i = $y$4$i;
        } else {
         $292 = $285;$293 = $286;$x$4$lcssa$i = $x$5$i;$y$3$lcssa$i = $y$4$i;
         break;
        }
       }
      } else {
       $292 = $251;$293 = $252;$x$4$lcssa$i = $x$3$lcssa$i;$y$3$lcssa$i = $y$0$i;
      }
      $288 = ($emin$0$ph|0)<(0);
      $289 = $288 << 31 >> 31;
      $290 = (_i64Subtract(32,0,($emin$0$ph|0),($289|0))|0);
      $291 = tempRet0;
      $294 = (_i64Add(($292|0),($293|0),($290|0),($291|0))|0);
      $295 = tempRet0;
      $296 = (0)>($295|0);
      $297 = (0)==($295|0);
      $298 = ($bits$0$ph>>>0)>($294>>>0);
      $299 = $297 & $298;
      $300 = $296 | $299;
      if ($300) {
       $301 = ($294|0)<(0);
       $$$i = $301 ? 0 : $294;
       $$02$i = $$$i;
      } else {
       $$02$i = $bits$0$ph;
      }
      $302 = ($$02$i|0)<(53);
      do {
       if ($302) {
        $303 = (84 - ($$02$i))|0;
        $304 = (+_scalbn(1.0,$303));
        $305 = (+($sign$0|0));
        $306 = (+_copysignl((+$304),(+$305)));
        $307 = ($$02$i|0)<(32);
        $308 = $y$3$lcssa$i != 0.0;
        $or$cond4$i = $307 & $308;
        if (!($or$cond4$i)) {
         $$pre$phi$iZ2D = $305;$bias$07$i = $306;$x$6$i = $x$4$lcssa$i;$y$5$i = $y$3$lcssa$i;
         break;
        }
        $309 = $x$4$lcssa$i & 1;
        $310 = ($309|0)==(0);
        $311 = $309 ^ 1;
        $$x$4$i = (($311) + ($x$4$lcssa$i))|0;
        $$y$3$i = $310 ? 0.0 : $y$3$lcssa$i;
        $$pre$phi$iZ2D = $305;$bias$07$i = $306;$x$6$i = $$x$4$i;$y$5$i = $$y$3$i;
       } else {
        $$pre$i = (+($sign$0|0));
        $$pre$phi$iZ2D = $$pre$i;$bias$07$i = 0.0;$x$6$i = $x$4$lcssa$i;$y$5$i = $y$3$lcssa$i;
       }
      } while(0);
      $312 = (+($x$6$i>>>0));
      $313 = $$pre$phi$iZ2D * $312;
      $314 = $bias$07$i + $313;
      $315 = $$pre$phi$iZ2D * $y$5$i;
      $316 = $315 + $314;
      $317 = $316 - $bias$07$i;
      $318 = $317 != 0.0;
      if (!($318)) {
       $319 = (___errno_location()|0);
       HEAP32[$319>>2] = 34;
      }
      $320 = (+_scalbnl($317,$292));
      $$0 = $320;
      STACKTOP = sp;return (+$$0);
     } else {
      $c$6 = $c$5;
     }
    } while(0);
    $sum$i = (($emin$0$ph) + ($bits$0$ph))|0;
    $325 = (0 - ($sum$i))|0;
    $$08$i = $c$6;$gotdig$0$i9 = 0;
    while(1) {
     if ((($$08$i|0) == 46)) {
      label = 139;
      break;
     } else if (!((($$08$i|0) == 48))) {
      $$2$i = $$08$i;$716 = 0;$717 = 0;$gotdig$2$i11 = $gotdig$0$i9;$gotrad$0$i12 = 0;
      break;
     }
     $326 = HEAP32[$0>>2]|0;
     $327 = HEAP32[$1>>2]|0;
     $328 = ($326>>>0)<($327>>>0);
     if ($328) {
      $329 = (($326) + 1|0);
      HEAP32[$0>>2] = $329;
      $330 = HEAP8[$326>>0]|0;
      $331 = $330&255;
      $$08$i = $331;$gotdig$0$i9 = 1;
      continue;
     } else {
      $332 = (___shgetc($f)|0);
      $$08$i = $332;$gotdig$0$i9 = 1;
      continue;
     }
    }
    L209: do {
     if ((label|0) == 139) {
      $333 = HEAP32[$0>>2]|0;
      $334 = HEAP32[$1>>2]|0;
      $335 = ($333>>>0)<($334>>>0);
      if ($335) {
       $336 = (($333) + 1|0);
       HEAP32[$0>>2] = $336;
       $337 = HEAP8[$333>>0]|0;
       $338 = $337&255;
       $$1$ph$i = $338;
      } else {
       $339 = (___shgetc($f)|0);
       $$1$ph$i = $339;
      }
      $340 = ($$1$ph$i|0)==(48);
      if ($340) {
       $349 = -1;$350 = -1;
       while(1) {
        $341 = HEAP32[$0>>2]|0;
        $342 = HEAP32[$1>>2]|0;
        $343 = ($341>>>0)<($342>>>0);
        if ($343) {
         $344 = (($341) + 1|0);
         HEAP32[$0>>2] = $344;
         $345 = HEAP8[$341>>0]|0;
         $346 = $345&255;
         $$1$be$i = $346;
        } else {
         $347 = (___shgetc($f)|0);
         $$1$be$i = $347;
        }
        $348 = ($$1$be$i|0)==(48);
        if (!($348)) {
         $$2$i = $$1$be$i;$716 = $349;$717 = $350;$gotdig$2$i11 = 1;$gotrad$0$i12 = 1;
         break L209;
        }
        $351 = (_i64Add(($349|0),($350|0),-1,-1)|0);
        $352 = tempRet0;
        $349 = $351;$350 = $352;
       }
      } else {
       $$2$i = $$1$ph$i;$716 = 0;$717 = 0;$gotdig$2$i11 = $gotdig$0$i9;$gotrad$0$i12 = 1;
      }
     }
    } while(0);
    HEAP32[$x$i>>2] = 0;
    $353 = (($$2$i) + -48)|0;
    $354 = ($353>>>0)<(10);
    $355 = ($$2$i|0)==(46);
    $or$cond789$i = $354 | $355;
    L223: do {
     if ($or$cond789$i) {
      $356 = (($x$i) + 496|0);
      $$397$i = $$2$i;$358 = 0;$359 = 0;$718 = $355;$719 = $353;$720 = $716;$721 = $717;$gotdig$393$i = $gotdig$2$i11;$gotrad$194$i = $gotrad$0$i12;$j$096$i = 0;$k$095$i = 0;$lnz$092$i = 0;
      while(1) {
       do {
        if ($718) {
         $cond$i = ($gotrad$194$i|0)==(0);
         if ($cond$i) {
          $722 = $358;$723 = $359;$724 = $358;$725 = $359;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = 1;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
         } else {
          $$388$i = $$397$i;$405 = $720;$406 = $721;$421 = $358;$424 = $359;$gotdig$381$i = $gotdig$393$i;$j$086$i = $j$096$i;$k$084$i = $k$095$i;$lnz$079$i = $lnz$092$i;
          break L223;
         }
        } else {
         $357 = ($k$095$i|0)<(125);
         $360 = (_i64Add(($358|0),($359|0),1,0)|0);
         $361 = tempRet0;
         $362 = ($$397$i|0)!=(48);
         if (!($357)) {
          if (!($362)) {
           $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = $gotrad$194$i;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
           break;
          }
          $372 = HEAP32[$356>>2]|0;
          $373 = $372 | 1;
          HEAP32[$356>>2] = $373;
          $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = $gotrad$194$i;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
          break;
         }
         $$lnz$0$i = $362 ? $360 : $lnz$092$i;
         $363 = ($j$096$i|0)==(0);
         $364 = (($x$i) + ($k$095$i<<2)|0);
         if ($363) {
          $storemerge$i = $719;
         } else {
          $365 = HEAP32[$364>>2]|0;
          $366 = ($365*10)|0;
          $367 = (($$397$i) + -48)|0;
          $368 = (($367) + ($366))|0;
          $storemerge$i = $368;
         }
         HEAP32[$364>>2] = $storemerge$i;
         $369 = (($j$096$i) + 1)|0;
         $370 = ($369|0)==(9);
         $371 = $370&1;
         $$k$0$i = (($371) + ($k$095$i))|0;
         $$10$i = $370 ? 0 : $369;
         $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = 1;$gotrad$2$i = $gotrad$194$i;$j$2$i = $$10$i;$k$2$i = $$k$0$i;$lnz$2$i = $$lnz$0$i;
        }
       } while(0);
       $374 = HEAP32[$0>>2]|0;
       $375 = HEAP32[$1>>2]|0;
       $376 = ($374>>>0)<($375>>>0);
       if ($376) {
        $377 = (($374) + 1|0);
        HEAP32[$0>>2] = $377;
        $378 = HEAP8[$374>>0]|0;
        $379 = $378&255;
        $$3$be$i = $379;
       } else {
        $380 = (___shgetc($f)|0);
        $$3$be$i = $380;
       }
       $381 = (($$3$be$i) + -48)|0;
       $382 = ($381>>>0)<(10);
       $383 = ($$3$be$i|0)==(46);
       $or$cond7$i = $382 | $383;
       if ($or$cond7$i) {
        $$397$i = $$3$be$i;$358 = $724;$359 = $725;$718 = $383;$719 = $381;$720 = $722;$721 = $723;$gotdig$393$i = $gotdig$4$i;$gotrad$194$i = $gotrad$2$i;$j$096$i = $j$2$i;$k$095$i = $k$2$i;$lnz$092$i = $lnz$2$i;
       } else {
        $$3$lcssa$i = $$3$be$i;$386 = $724;$387 = $722;$389 = $725;$390 = $723;$gotdig$3$lcssa$i = $gotdig$4$i;$gotrad$1$lcssa$i = $gotrad$2$i;$j$0$lcssa$i = $j$2$i;$k$0$lcssa$i = $k$2$i;$lnz$0$lcssa$i = $lnz$2$i;
        label = 162;
        break;
       }
      }
     } else {
      $$3$lcssa$i = $$2$i;$386 = 0;$387 = $716;$389 = 0;$390 = $717;$gotdig$3$lcssa$i = $gotdig$2$i11;$gotrad$1$lcssa$i = $gotrad$0$i12;$j$0$lcssa$i = 0;$k$0$lcssa$i = 0;$lnz$0$lcssa$i = 0;
      label = 162;
     }
    } while(0);
    if ((label|0) == 162) {
     $384 = ($gotrad$1$lcssa$i|0)==(0);
     $385 = $384 ? $386 : $387;
     $388 = $384 ? $389 : $390;
     $$388$i = $$3$lcssa$i;$405 = $385;$406 = $388;$421 = $386;$424 = $389;$gotdig$381$i = $gotdig$3$lcssa$i;$j$086$i = $j$0$lcssa$i;$k$084$i = $k$0$lcssa$i;$lnz$079$i = $lnz$0$lcssa$i;
    }
    $391 = ($gotdig$381$i|0)!=(0);
    if ($391) {
     $392 = $$388$i | 32;
     $393 = ($392|0)==(101);
     if ($393) {
      $394 = (_scanexp($f,$pok)|0);
      $395 = tempRet0;
      $396 = ($394|0)==(0);
      $397 = ($395|0)==(-2147483648);
      $398 = $396 & $397;
      do {
       if ($398) {
        if ($39) {
         ___shlim($f,0);
         $$0 = 0.0;
         STACKTOP = sp;return (+$$0);
        } else {
         $399 = HEAP32[$1>>2]|0;
         $400 = ($399|0)==(0|0);
         if ($400) {
          $403 = 0;$404 = 0;
          break;
         }
         $401 = HEAP32[$0>>2]|0;
         $402 = (($401) + -1|0);
         HEAP32[$0>>2] = $402;
         $403 = 0;$404 = 0;
         break;
        }
       } else {
        $403 = $394;$404 = $395;
       }
      } while(0);
      $407 = (_i64Add(($403|0),($404|0),($405|0),($406|0))|0);
      $408 = tempRet0;
      $420 = $407;$423 = $408;
     } else {
      label = 171;
     }
    } else {
     label = 171;
    }
    if ((label|0) == 171) {
     $409 = ($$388$i|0)>(-1);
     if ($409) {
      $410 = HEAP32[$1>>2]|0;
      $411 = ($410|0)==(0|0);
      if ($411) {
       $420 = $405;$423 = $406;
      } else {
       $412 = HEAP32[$0>>2]|0;
       $413 = (($412) + -1|0);
       HEAP32[$0>>2] = $413;
       $420 = $405;$423 = $406;
      }
     } else {
      $420 = $405;$423 = $406;
     }
    }
    if (!($391)) {
     $414 = (___errno_location()|0);
     HEAP32[$414>>2] = 22;
     ___shlim($f,0);
     $$0 = 0.0;
     STACKTOP = sp;return (+$$0);
    }
    $415 = HEAP32[$x$i>>2]|0;
    $416 = ($415|0)==(0);
    if ($416) {
     $417 = (+($sign$0|0));
     $418 = $417 * 0.0;
     $$0 = $418;
     STACKTOP = sp;return (+$$0);
    }
    $419 = ($420|0)==($421|0);
    $422 = ($423|0)==($424|0);
    $425 = $419 & $422;
    $426 = ($424|0)<(0);
    $427 = ($424|0)==(0);
    $428 = ($421>>>0)<(10);
    $429 = $427 & $428;
    $430 = $426 | $429;
    $or$cond$i15 = $425 & $430;
    do {
     if ($or$cond$i15) {
      $431 = ($bits$0$ph>>>0)>(30);
      if (!($431)) {
       $432 = $415 >>> $bits$0$ph;
       $433 = ($432|0)==(0);
       if (!($433)) {
        break;
       }
      }
      $434 = (+($sign$0|0));
      $435 = (+($415>>>0));
      $436 = $434 * $435;
      $$0 = $436;
      STACKTOP = sp;return (+$$0);
     }
    } while(0);
    $437 = (($emin$0$ph|0) / -2)&-1;
    $438 = ($437|0)<(0);
    $439 = $438 << 31 >> 31;
    $440 = ($423|0)>($439|0);
    $441 = ($423|0)==($439|0);
    $442 = ($420>>>0)>($437>>>0);
    $443 = $441 & $442;
    $444 = $440 | $443;
    if ($444) {
     $445 = (___errno_location()|0);
     HEAP32[$445>>2] = 34;
     $446 = (+($sign$0|0));
     $447 = $446 * 1.79769313486231570815E+308;
     $448 = $447 * 1.79769313486231570815E+308;
     $$0 = $448;
     STACKTOP = sp;return (+$$0);
    }
    $449 = (($emin$0$ph) + -106)|0;
    $450 = ($449|0)<(0);
    $451 = $450 << 31 >> 31;
    $452 = ($423|0)<($451|0);
    $453 = ($423|0)==($451|0);
    $454 = ($420>>>0)<($449>>>0);
    $455 = $453 & $454;
    $456 = $452 | $455;
    if ($456) {
     $457 = (___errno_location()|0);
     HEAP32[$457>>2] = 34;
     $458 = (+($sign$0|0));
     $459 = $458 * 2.22507385850720138309E-308;
     $460 = $459 * 2.22507385850720138309E-308;
     $$0 = $460;
     STACKTOP = sp;return (+$$0);
    }
    $461 = ($j$086$i|0)==(0);
    if ($461) {
     $k$3$i = $k$084$i;
    } else {
     $462 = ($j$086$i|0)<(9);
     if ($462) {
      $463 = (($x$i) + ($k$084$i<<2)|0);
      $$promoted$i = HEAP32[$463>>2]|0;
      $465 = $$promoted$i;$j$371$i = $j$086$i;
      while(1) {
       $464 = ($465*10)|0;
       $466 = (($j$371$i) + 1)|0;
       $exitcond$i = ($466|0)==(9);
       if ($exitcond$i) {
        break;
       } else {
        $465 = $464;$j$371$i = $466;
       }
      }
      HEAP32[$463>>2] = $464;
     }
     $467 = (($k$084$i) + 1)|0;
     $k$3$i = $467;
    }
    $468 = ($lnz$079$i|0)<(9);
    do {
     if ($468) {
      $469 = ($lnz$079$i|0)<=($420|0);
      $470 = ($420|0)<(18);
      $or$cond3$i16 = $469 & $470;
      if ($or$cond3$i16) {
       $471 = ($420|0)==(9);
       if ($471) {
        $472 = (+($sign$0|0));
        $473 = HEAP32[$x$i>>2]|0;
        $474 = (+($473>>>0));
        $475 = $472 * $474;
        $$0 = $475;
        STACKTOP = sp;return (+$$0);
       }
       $476 = ($420|0)<(9);
       if ($476) {
        $477 = (+($sign$0|0));
        $478 = HEAP32[$x$i>>2]|0;
        $479 = (+($478>>>0));
        $480 = $477 * $479;
        $481 = (8 - ($420))|0;
        $482 = (11168 + ($481<<2)|0);
        $483 = HEAP32[$482>>2]|0;
        $484 = (+($483|0));
        $485 = $480 / $484;
        $$0 = $485;
        STACKTOP = sp;return (+$$0);
       }
       $486 = Math_imul($420, -3)|0;
       $$neg20$i = (($bits$0$ph) + 27)|0;
       $487 = (($$neg20$i) + ($486))|0;
       $488 = ($487|0)>(30);
       $$pre$i17 = HEAP32[$x$i>>2]|0;
       if (!($488)) {
        $489 = $$pre$i17 >>> $487;
        $490 = ($489|0)==(0);
        if (!($490)) {
         break;
        }
       }
       $491 = (+($sign$0|0));
       $492 = (+($$pre$i17>>>0));
       $493 = $491 * $492;
       $494 = (($420) + -10)|0;
       $495 = (11168 + ($494<<2)|0);
       $496 = HEAP32[$495>>2]|0;
       $497 = (+($496|0));
       $498 = $493 * $497;
       $$0 = $498;
       STACKTOP = sp;return (+$$0);
      }
     }
    } while(0);
    $499 = (($420|0) % 9)&-1;
    $500 = ($499|0)==(0);
    if ($500) {
     $a$2$ph40$i = 0;$e2$0$ph$i = 0;$rp$2$ph38$i = $420;$z$1$ph39$i = $k$3$i;
    } else {
     $501 = ($420|0)>(-1);
     $502 = (($499) + 9)|0;
     $503 = $501 ? $499 : $502;
     $504 = (8 - ($503))|0;
     $505 = (11168 + ($504<<2)|0);
     $506 = HEAP32[$505>>2]|0;
     $507 = ($k$3$i|0)==(0);
     if ($507) {
      $a$0$lcssa161$i = 0;$rp$0$lcssa162$i = $420;$z$0$i = 0;
     } else {
      $508 = (1000000000 / ($506|0))&-1;
      $a$061$i = 0;$carry$063$i = 0;$k$462$i = 0;$rp$060$i = $420;
      while(1) {
       $509 = (($x$i) + ($k$462$i<<2)|0);
       $510 = HEAP32[$509>>2]|0;
       $511 = (($510>>>0) % ($506>>>0))&-1;
       $512 = (($510>>>0) / ($506>>>0))&-1;
       $513 = (($512) + ($carry$063$i))|0;
       HEAP32[$509>>2] = $513;
       $514 = Math_imul($511, $508)|0;
       $515 = ($k$462$i|0)==($a$061$i|0);
       $516 = ($513|0)==(0);
       $or$cond11$i = $515 & $516;
       $517 = (($k$462$i) + 1)|0;
       if ($or$cond11$i) {
        $518 = $517 & 127;
        $519 = (($rp$060$i) + -9)|0;
        $a$1$i = $518;$rp$1$i18 = $519;
       } else {
        $a$1$i = $a$061$i;$rp$1$i18 = $rp$060$i;
       }
       $520 = ($517|0)==($k$3$i|0);
       if ($520) {
        break;
       } else {
        $a$061$i = $a$1$i;$carry$063$i = $514;$k$462$i = $517;$rp$060$i = $rp$1$i18;
       }
      }
      $521 = ($514|0)==(0);
      if ($521) {
       $a$0$lcssa161$i = $a$1$i;$rp$0$lcssa162$i = $rp$1$i18;$z$0$i = $k$3$i;
      } else {
       $522 = (($k$3$i) + 1)|0;
       $523 = (($x$i) + ($k$3$i<<2)|0);
       HEAP32[$523>>2] = $514;
       $a$0$lcssa161$i = $a$1$i;$rp$0$lcssa162$i = $rp$1$i18;$z$0$i = $522;
      }
     }
     $524 = (9 - ($503))|0;
     $525 = (($524) + ($rp$0$lcssa162$i))|0;
     $a$2$ph40$i = $a$0$lcssa161$i;$e2$0$ph$i = 0;$rp$2$ph38$i = $525;$z$1$ph39$i = $z$0$i;
    }
    L321: while(1) {
     $526 = ($rp$2$ph38$i|0)<(18);
     $527 = (($x$i) + ($a$2$ph40$i<<2)|0);
     if ($526) {
      $e2$0$us$i = $e2$0$ph$i;$z$1$us$i = $z$1$ph39$i;
      while(1) {
       $529 = (($z$1$us$i) + 127)|0;
       $carry1$0$us$i = 0;$k$5$in$us$i = $529;$z$2$us$i = $z$1$us$i;
       while(1) {
        $k$5$us$i = $k$5$in$us$i & 127;
        $530 = (($x$i) + ($k$5$us$i<<2)|0);
        $531 = HEAP32[$530>>2]|0;
        $532 = (_bitshift64Shl(($531|0),0,29)|0);
        $533 = tempRet0;
        $534 = (_i64Add(($532|0),($533|0),($carry1$0$us$i|0),0)|0);
        $535 = tempRet0;
        $536 = ($535>>>0)>(0);
        $537 = ($535|0)==(0);
        $538 = ($534>>>0)>(1000000000);
        $539 = $537 & $538;
        $540 = $536 | $539;
        if ($540) {
         $541 = (___udivdi3(($534|0),($535|0),1000000000,0)|0);
         $542 = tempRet0;
         $543 = (___uremdi3(($534|0),($535|0),1000000000,0)|0);
         $544 = tempRet0;
         $$sink$off0$us$i = $543;$carry1$1$us$i = $541;
        } else {
         $$sink$off0$us$i = $534;$carry1$1$us$i = 0;
        }
        HEAP32[$530>>2] = $$sink$off0$us$i;
        $545 = (($z$2$us$i) + 127)|0;
        $546 = $545 & 127;
        $547 = ($k$5$us$i|0)!=($546|0);
        $548 = ($k$5$us$i|0)==($a$2$ph40$i|0);
        $or$cond12$us$i = $547 | $548;
        if ($or$cond12$us$i) {
         $z$3$us$i = $z$2$us$i;
        } else {
         $549 = ($$sink$off0$us$i|0)==(0);
         $k$5$z$2$us$i = $549 ? $k$5$us$i : $z$2$us$i;
         $z$3$us$i = $k$5$z$2$us$i;
        }
        $550 = (($k$5$us$i) + -1)|0;
        if ($548) {
         break;
        } else {
         $carry1$0$us$i = $carry1$1$us$i;$k$5$in$us$i = $550;$z$2$us$i = $z$3$us$i;
        }
       }
       $551 = (($e2$0$us$i) + -29)|0;
       $552 = ($carry1$1$us$i|0)==(0);
       if ($552) {
        $e2$0$us$i = $551;$z$1$us$i = $z$3$us$i;
       } else {
        $$lcssa43$i = $551;$carry1$1$lcssa$lcssa$i = $carry1$1$us$i;$z$3$lcssa$lcssa$i = $z$3$us$i;
        break;
       }
      }
     } else {
      $528 = ($rp$2$ph38$i|0)==(18);
      if ($528) {
       $e2$0$us44$i = $e2$0$ph$i;$z$1$us45$i = $z$1$ph39$i;
      } else {
       $a$3$ph$i = $a$2$ph40$i;$e2$1$ph$i = $e2$0$ph$i;$rp$3$ph33$i = $rp$2$ph38$i;$z$5$ph$i = $z$1$ph39$i;
       break;
      }
      while(1) {
       $553 = HEAP32[$527>>2]|0;
       $554 = ($553>>>0)<(9007199);
       if (!($554)) {
        $a$3$ph$i = $a$2$ph40$i;$e2$1$ph$i = $e2$0$us44$i;$rp$3$ph33$i = 18;$z$5$ph$i = $z$1$us45$i;
        break L321;
       }
       $555 = (($z$1$us45$i) + 127)|0;
       $carry1$0$us49$i = 0;$k$5$in$us48$i = $555;$z$2$us47$i = $z$1$us45$i;
       while(1) {
        $k$5$us50$i = $k$5$in$us48$i & 127;
        $556 = (($x$i) + ($k$5$us50$i<<2)|0);
        $557 = HEAP32[$556>>2]|0;
        $558 = (_bitshift64Shl(($557|0),0,29)|0);
        $559 = tempRet0;
        $560 = (_i64Add(($558|0),($559|0),($carry1$0$us49$i|0),0)|0);
        $561 = tempRet0;
        $562 = ($561>>>0)>(0);
        $563 = ($561|0)==(0);
        $564 = ($560>>>0)>(1000000000);
        $565 = $563 & $564;
        $566 = $562 | $565;
        if ($566) {
         $567 = (___udivdi3(($560|0),($561|0),1000000000,0)|0);
         $568 = tempRet0;
         $569 = (___uremdi3(($560|0),($561|0),1000000000,0)|0);
         $570 = tempRet0;
         $$sink$off0$us53$i = $569;$carry1$1$us54$i = $567;
        } else {
         $$sink$off0$us53$i = $560;$carry1$1$us54$i = 0;
        }
        HEAP32[$556>>2] = $$sink$off0$us53$i;
        $571 = (($z$2$us47$i) + 127)|0;
        $572 = $571 & 127;
        $573 = ($k$5$us50$i|0)!=($572|0);
        $574 = ($k$5$us50$i|0)==($a$2$ph40$i|0);
        $or$cond12$us55$i = $573 | $574;
        if ($or$cond12$us55$i) {
         $z$3$us57$i = $z$2$us47$i;
        } else {
         $575 = ($$sink$off0$us53$i|0)==(0);
         $k$5$z$2$us56$i = $575 ? $k$5$us50$i : $z$2$us47$i;
         $z$3$us57$i = $k$5$z$2$us56$i;
        }
        $576 = (($k$5$us50$i) + -1)|0;
        if ($574) {
         break;
        } else {
         $carry1$0$us49$i = $carry1$1$us54$i;$k$5$in$us48$i = $576;$z$2$us47$i = $z$3$us57$i;
        }
       }
       $577 = (($e2$0$us44$i) + -29)|0;
       $578 = ($carry1$1$us54$i|0)==(0);
       if ($578) {
        $e2$0$us44$i = $577;$z$1$us45$i = $z$3$us57$i;
       } else {
        $$lcssa43$i = $577;$carry1$1$lcssa$lcssa$i = $carry1$1$us54$i;$z$3$lcssa$lcssa$i = $z$3$us57$i;
        break;
       }
      }
     }
     $579 = (($rp$2$ph38$i) + 9)|0;
     $580 = (($a$2$ph40$i) + 127)|0;
     $581 = $580 & 127;
     $582 = ($581|0)==($z$3$lcssa$lcssa$i|0);
     if ($582) {
      $583 = (($z$3$lcssa$lcssa$i) + 127)|0;
      $584 = $583 & 127;
      $585 = (($x$i) + ($584<<2)|0);
      $586 = HEAP32[$585>>2]|0;
      $587 = (($z$3$lcssa$lcssa$i) + 126)|0;
      $588 = $587 & 127;
      $589 = (($x$i) + ($588<<2)|0);
      $590 = HEAP32[$589>>2]|0;
      $591 = $590 | $586;
      HEAP32[$589>>2] = $591;
      $z$4$i = $584;
     } else {
      $z$4$i = $z$3$lcssa$lcssa$i;
     }
     $592 = (($x$i) + ($581<<2)|0);
     HEAP32[$592>>2] = $carry1$1$lcssa$lcssa$i;
     $a$2$ph40$i = $581;$e2$0$ph$i = $$lcssa43$i;$rp$2$ph38$i = $579;$z$1$ph39$i = $z$4$i;
    }
    L352: while(1) {
     $629 = (($z$5$ph$i) + 1)|0;
     $627 = $629 & 127;
     $630 = (($z$5$ph$i) + 127)|0;
     $631 = $630 & 127;
     $632 = (($x$i) + ($631<<2)|0);
     $a$3$i$ph = $a$3$ph$i;$e2$1$i$ph = $e2$1$ph$i;$rp$3$i$ph = $rp$3$ph33$i;
     while(1) {
      $605 = ($rp$3$i$ph|0)==(18);
      $633 = ($rp$3$i$ph|0)>(27);
      $$14$i = $633 ? 9 : 1;
      $a$3$i = $a$3$i$ph;$e2$1$i = $e2$1$i$ph;
      while(1) {
       $i$025$i = 0;
       while(1) {
        $595 = (($i$025$i) + ($a$3$i))|0;
        $596 = $595 & 127;
        $597 = ($596|0)==($z$5$ph$i|0);
        if ($597) {
         $i$1$i = 2;
         break;
        }
        $598 = (($x$i) + ($596<<2)|0);
        $599 = HEAP32[$598>>2]|0;
        $600 = (11160 + ($i$025$i<<2)|0);
        $601 = HEAP32[$600>>2]|0;
        $602 = ($599>>>0)<($601>>>0);
        if ($602) {
         $i$1$i = 2;
         break;
        }
        $603 = ($599>>>0)>($601>>>0);
        $594 = (($i$025$i) + 1)|0;
        if ($603) {
         $i$1$i = $i$025$i;
         break;
        }
        $593 = ($594|0)<(2);
        if ($593) {
         $i$025$i = $594;
        } else {
         $i$1$i = $594;
         break;
        }
       }
       $604 = ($i$1$i|0)==(2);
       $or$cond13$i = $604 & $605;
       if ($or$cond13$i) {
        break L352;
       }
       $608 = (($$14$i) + ($e2$1$i))|0;
       $609 = ($a$3$i|0)==($z$5$ph$i|0);
       if ($609) {
        $a$3$i = $z$5$ph$i;$e2$1$i = $608;
       } else {
        break;
       }
      }
      $610 = 1 << $$14$i;
      $611 = (($610) + -1)|0;
      $612 = 1000000000 >>> $$14$i;
      $a$427$i = $a$3$i;$carry3$030$i = 0;$k$628$i = $a$3$i;$rp$426$i = $rp$3$i$ph;
      while(1) {
       $613 = (($x$i) + ($k$628$i<<2)|0);
       $614 = HEAP32[$613>>2]|0;
       $615 = $614 & $611;
       $616 = $614 >>> $$14$i;
       $617 = (($616) + ($carry3$030$i))|0;
       HEAP32[$613>>2] = $617;
       $618 = Math_imul($615, $612)|0;
       $619 = ($k$628$i|0)==($a$427$i|0);
       $620 = ($617|0)==(0);
       $or$cond15$i = $619 & $620;
       $621 = (($k$628$i) + 1)|0;
       $622 = $621 & 127;
       $623 = (($rp$426$i) + -9)|0;
       $rp$5$i = $or$cond15$i ? $623 : $rp$426$i;
       $a$5$i = $or$cond15$i ? $622 : $a$427$i;
       $624 = ($622|0)==($z$5$ph$i|0);
       if ($624) {
        break;
       } else {
        $a$427$i = $a$5$i;$carry3$030$i = $618;$k$628$i = $622;$rp$426$i = $rp$5$i;
       }
      }
      $625 = ($618|0)==(0);
      if ($625) {
       $a$3$i$ph = $a$5$i;$e2$1$i$ph = $608;$rp$3$i$ph = $rp$5$i;
       continue;
      }
      $626 = ($627|0)==($a$5$i|0);
      if (!($626)) {
       break;
      }
      $634 = HEAP32[$632>>2]|0;
      $635 = $634 | 1;
      HEAP32[$632>>2] = $635;
      $a$3$i$ph = $a$5$i;$e2$1$i$ph = $608;$rp$3$i$ph = $rp$5$i;
     }
     $628 = (($x$i) + ($z$5$ph$i<<2)|0);
     HEAP32[$628>>2] = $618;
     $a$3$ph$i = $a$5$i;$e2$1$ph$i = $608;$rp$3$ph33$i = $rp$5$i;$z$5$ph$i = $627;
    }
    $606 = $a$3$i & 127;
    $607 = ($606|0)==($z$5$ph$i|0);
    if ($607) {
     $636 = (($627) + -1)|0;
     $637 = (($x$i) + ($636<<2)|0);
     HEAP32[$637>>2] = 0;
     $z$7$i = $627;
    } else {
     $z$7$i = $z$5$ph$i;
    }
    $638 = (($x$i) + ($606<<2)|0);
    $639 = HEAP32[$638>>2]|0;
    $640 = (+($639>>>0));
    $641 = (($a$3$i) + 1)|0;
    $642 = $641 & 127;
    $643 = ($642|0)==($z$7$i|0);
    if ($643) {
     $702 = (($z$7$i) + 1)|0;
     $703 = $702 & 127;
     $704 = (($703) + -1)|0;
     $705 = (($x$i) + ($704<<2)|0);
     HEAP32[$705>>2] = 0;
     $z$7$1$i = $703;
    } else {
     $z$7$1$i = $z$7$i;
    }
    $706 = $640 * 1.0E+9;
    $707 = (($x$i) + ($642<<2)|0);
    $708 = HEAP32[$707>>2]|0;
    $709 = (+($708>>>0));
    $710 = $706 + $709;
    $667 = (+($sign$0|0));
    $649 = $667 * $710;
    $687 = (($e2$1$i) + 53)|0;
    $645 = (($687) - ($emin$0$ph))|0;
    $711 = ($645|0)<($bits$0$ph|0);
    if ($711) {
     $644 = ($645|0)<(0);
     $$$i22 = $644 ? 0 : $645;
     $$09$i = $$$i22;$denormal$0$i = 1;
    } else {
     $$09$i = $bits$0$ph;$denormal$0$i = 0;
    }
    $646 = ($$09$i|0)<(53);
    if ($646) {
     $647 = (105 - ($$09$i))|0;
     $648 = (+_scalbn(1.0,$647));
     $650 = (+_copysignl((+$648),(+$649)));
     $651 = (53 - ($$09$i))|0;
     $652 = (+_scalbn(1.0,$651));
     $653 = (+_fmodl((+$649),(+$652)));
     $654 = $649 - $653;
     $655 = $650 + $654;
     $bias$0$i = $650;$frac$0$i = $653;$y$1$i23 = $655;
    } else {
     $bias$0$i = 0.0;$frac$0$i = 0.0;$y$1$i23 = $649;
    }
    $656 = (($a$3$i) + 2)|0;
    $657 = $656 & 127;
    $658 = ($657|0)==($z$7$1$i|0);
    do {
     if ($658) {
      $frac$2$i = $frac$0$i;
     } else {
      $659 = (($x$i) + ($657<<2)|0);
      $660 = HEAP32[$659>>2]|0;
      $661 = ($660>>>0)<(500000000);
      do {
       if ($661) {
        $662 = ($660|0)==(0);
        if ($662) {
         $663 = (($a$3$i) + 3)|0;
         $664 = $663 & 127;
         $665 = ($664|0)==($z$7$1$i|0);
         if ($665) {
          $frac$1$i = $frac$0$i;
          break;
         }
        }
        $666 = $667 * 0.25;
        $668 = $666 + $frac$0$i;
        $frac$1$i = $668;
       } else {
        $669 = ($660>>>0)>(500000000);
        if ($669) {
         $670 = $667 * 0.75;
         $671 = $670 + $frac$0$i;
         $frac$1$i = $671;
         break;
        }
        $672 = (($a$3$i) + 3)|0;
        $673 = $672 & 127;
        $674 = ($673|0)==($z$7$1$i|0);
        if ($674) {
         $675 = $667 * 0.5;
         $676 = $675 + $frac$0$i;
         $frac$1$i = $676;
         break;
        } else {
         $677 = $667 * 0.75;
         $678 = $677 + $frac$0$i;
         $frac$1$i = $678;
         break;
        }
       }
      } while(0);
      $679 = (53 - ($$09$i))|0;
      $680 = ($679|0)>(1);
      if (!($680)) {
       $frac$2$i = $frac$1$i;
       break;
      }
      $681 = (+_fmodl((+$frac$1$i),1.0));
      $682 = $681 != 0.0;
      if ($682) {
       $frac$2$i = $frac$1$i;
       break;
      }
      $683 = $frac$1$i + 1.0;
      $frac$2$i = $683;
     }
    } while(0);
    $684 = $y$1$i23 + $frac$2$i;
    $685 = $684 - $bias$0$i;
    $686 = $687 & 2147483647;
    $688 = (-2 - ($sum$i))|0;
    $689 = ($686|0)>($688|0);
    do {
     if ($689) {
      $690 = (+Math_abs((+$685)));
      $691 = !($690 >= 9007199254740992.0);
      if ($691) {
       $denormal$2$i = $denormal$0$i;$e2$2$i = $e2$1$i;$y$2$i24 = $685;
      } else {
       $692 = ($denormal$0$i|0)!=(0);
       $693 = ($$09$i|0)==($645|0);
       $or$cond16$i = $692 & $693;
       $denormal$1$i = $or$cond16$i ? 0 : $denormal$0$i;
       $694 = $685 * 0.5;
       $695 = (($e2$1$i) + 1)|0;
       $denormal$2$i = $denormal$1$i;$e2$2$i = $695;$y$2$i24 = $694;
      }
      $696 = (($e2$2$i) + 50)|0;
      $697 = ($696|0)>($325|0);
      if (!($697)) {
       $698 = ($denormal$2$i|0)!=(0);
       $699 = $frac$2$i != 0.0;
       $or$cond17$i = $698 & $699;
       if (!($or$cond17$i)) {
        $e2$3$i = $e2$2$i;$y$3$i = $y$2$i24;
        break;
       }
      }
      $700 = (___errno_location()|0);
      HEAP32[$700>>2] = 34;
      $e2$3$i = $e2$2$i;$y$3$i = $y$2$i24;
     } else {
      $e2$3$i = $e2$1$i;$y$3$i = $685;
     }
    } while(0);
    $701 = (+_scalbnl($y$3$i,$e2$3$i));
    $$0 = $701;
    STACKTOP = sp;return (+$$0);
   } else {
    $108 = HEAP32[$1>>2]|0;
    $109 = ($108|0)==(0|0);
    if (!($109)) {
     $110 = HEAP32[$0>>2]|0;
     $111 = (($110) + -1|0);
     HEAP32[$0>>2] = $111;
    }
    $112 = (___errno_location()|0);
    HEAP32[$112>>2] = 22;
    ___shlim($f,0);
    $$0 = 0.0;
    STACKTOP = sp;return (+$$0);
   }
  }
 } while(0);
 if ((label|0) == 23) {
  $41 = HEAP32[$1>>2]|0;
  $42 = ($41|0)==(0|0);
  if (!($42)) {
   $43 = HEAP32[$0>>2]|0;
   $44 = (($43) + -1|0);
   HEAP32[$0>>2] = $44;
  }
  $notlhs = ($pok|0)==(0);
  $notrhs = ($i$0$lcssa>>>0)<(4);
  $or$cond106$not = $notrhs | $notlhs;
  $brmerge = $or$cond106$not | $42;
  if (!($brmerge)) {
   $i$166 = $i$0$lcssa;
   while(1) {
    $45 = HEAP32[$0>>2]|0;
    $46 = (($45) + -1|0);
    HEAP32[$0>>2] = $46;
    $47 = (($i$166) + -1)|0;
    $48 = ($47>>>0)>(3);
    if ($48) {
     $i$166 = $47;
    } else {
     break;
    }
   }
  }
 }
 $49 = (+($sign$0|0));
 $50 = $49 * inf;
 $51 = $50;
 $$0 = $51;
 STACKTOP = sp;return (+$$0);
}
function _scanexp($f,$pok) {
 $f = $f|0;
 $pok = $pok|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $c$0 = 0, $c$1$be = 0;
 var $c$18 = 0, $c$2$be = 0, $c$2$lcssa = 0, $c$23 = 0, $c$3$be = 0, $neg$0 = 0, $or$cond = 0, $or$cond2 = 0, $phitmp16 = 0, $x$09 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($f) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($1>>>0)<($3>>>0);
 if ($4) {
  $5 = (($1) + 1|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP8[$1>>0]|0;
  $7 = $6&255;
  $10 = $7;
 } else {
  $8 = (___shgetc($f)|0);
  $10 = $8;
 }
 $9 = ($10|0)==(45);
 if ((($10|0) == 43) | (($10|0) == 45)) {
  $11 = $9&1;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if ($14) {
   $15 = (($12) + 1|0);
   HEAP32[$0>>2] = $15;
   $16 = HEAP8[$12>>0]|0;
   $17 = $16&255;
   $20 = $17;
  } else {
   $18 = (___shgetc($f)|0);
   $20 = $18;
  }
  $19 = (($20) + -48)|0;
  $21 = ($19>>>0)<(10);
  $22 = ($pok|0)==(0);
  $or$cond = $21 | $22;
  if ($or$cond) {
   $c$0 = $20;$neg$0 = $11;
  } else {
   $23 = HEAP32[$2>>2]|0;
   $24 = ($23|0)==(0|0);
   if ($24) {
    $c$0 = $20;$neg$0 = $11;
   } else {
    $25 = HEAP32[$0>>2]|0;
    $26 = (($25) + -1|0);
    HEAP32[$0>>2] = $26;
    $c$0 = $20;$neg$0 = $11;
   }
  }
 } else {
  $c$0 = $10;$neg$0 = 0;
 }
 $27 = (($c$0) + -48)|0;
 $28 = ($27>>>0)>(9);
 if ($28) {
  $29 = HEAP32[$2>>2]|0;
  $30 = ($29|0)==(0|0);
  if ($30) {
   $95 = -2147483648;$96 = 0;
   tempRet0 = $95;
   STACKTOP = sp;return ($96|0);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + -1|0);
  HEAP32[$0>>2] = $32;
  $95 = -2147483648;$96 = 0;
  tempRet0 = $95;
  STACKTOP = sp;return ($96|0);
 } else {
  $c$18 = $c$0;$x$09 = 0;
 }
 while(1) {
  $33 = (($c$18) + -48)|0;
  $34 = (($33) + ($x$09))|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ($35>>>0)<($36>>>0);
  if ($37) {
   $38 = (($35) + 1|0);
   HEAP32[$0>>2] = $38;
   $39 = HEAP8[$35>>0]|0;
   $40 = $39&255;
   $c$1$be = $40;
  } else {
   $41 = (___shgetc($f)|0);
   $c$1$be = $41;
  }
  $42 = (($c$1$be) + -48)|0;
  $43 = ($42>>>0)<(10);
  $44 = ($34|0)<(214748364);
  $$ = $43 & $44;
  if (!($$)) {
   break;
  }
  $phitmp16 = ($34*10)|0;
  $c$18 = $c$1$be;$x$09 = $phitmp16;
 }
 $45 = ($34|0)<(0);
 $46 = $45 << 31 >> 31;
 $47 = (($c$1$be) + -48)|0;
 $48 = ($47>>>0)<(10);
 if ($48) {
  $51 = $34;$52 = $46;$c$23 = $c$1$be;
  while(1) {
   $53 = (___muldi3(($51|0),($52|0),10,0)|0);
   $54 = tempRet0;
   $55 = ($c$23|0)<(0);
   $56 = $55 << 31 >> 31;
   $57 = (_i64Add(($c$23|0),($56|0),-48,-1)|0);
   $58 = tempRet0;
   $59 = (_i64Add(($57|0),($58|0),($53|0),($54|0))|0);
   $60 = tempRet0;
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$2>>2]|0;
   $63 = ($61>>>0)<($62>>>0);
   if ($63) {
    $64 = (($61) + 1|0);
    HEAP32[$0>>2] = $64;
    $65 = HEAP8[$61>>0]|0;
    $66 = $65&255;
    $c$2$be = $66;
   } else {
    $67 = (___shgetc($f)|0);
    $c$2$be = $67;
   }
   $68 = (($c$2$be) + -48)|0;
   $69 = ($68>>>0)<(10);
   $70 = ($60|0)<(21474836);
   $71 = ($60|0)==(21474836);
   $72 = ($59>>>0)<(2061584302);
   $73 = $71 & $72;
   $74 = $70 | $73;
   $or$cond2 = $69 & $74;
   if ($or$cond2) {
    $51 = $59;$52 = $60;$c$23 = $c$2$be;
   } else {
    $89 = $59;$90 = $60;$c$2$lcssa = $c$2$be;
    break;
   }
  }
 } else {
  $89 = $34;$90 = $46;$c$2$lcssa = $c$1$be;
 }
 $49 = (($c$2$lcssa) + -48)|0;
 $50 = ($49>>>0)<(10);
 if ($50) {
  while(1) {
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ($75>>>0)<($76>>>0);
   if ($77) {
    $78 = (($75) + 1|0);
    HEAP32[$0>>2] = $78;
    $79 = HEAP8[$75>>0]|0;
    $80 = $79&255;
    $c$3$be = $80;
   } else {
    $81 = (___shgetc($f)|0);
    $c$3$be = $81;
   }
   $82 = (($c$3$be) + -48)|0;
   $83 = ($82>>>0)<(10);
   if (!($83)) {
    break;
   }
  }
 }
 $84 = HEAP32[$2>>2]|0;
 $85 = ($84|0)==(0|0);
 if (!($85)) {
  $86 = HEAP32[$0>>2]|0;
  $87 = (($86) + -1|0);
  HEAP32[$0>>2] = $87;
 }
 $88 = ($neg$0|0)!=(0);
 $91 = (_i64Subtract(0,0,($89|0),($90|0))|0);
 $92 = tempRet0;
 $93 = $88 ? $91 : $89;
 $94 = $88 ? $92 : $90;
 $95 = $94;$96 = $93;
 tempRet0 = $95;
 STACKTOP = sp;return ($96|0);
}
function ___shlim($f,$lim) {
 $f = $f|0;
 $lim = $lim|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 104|0);
 HEAP32[$0>>2] = $lim;
 $1 = (($f) + 8|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = (($f) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $2;
 $6 = $4;
 $7 = (($5) - ($6))|0;
 $8 = (($f) + 108|0);
 HEAP32[$8>>2] = $7;
 $9 = ($lim|0)!=(0);
 $10 = ($7|0)>($lim|0);
 $or$cond = $9 & $10;
 if ($or$cond) {
  $11 = (($4) + ($lim)|0);
  $12 = (($f) + 100|0);
  HEAP32[$12>>2] = $11;
  STACKTOP = sp;return;
 } else {
  $13 = (($f) + 100|0);
  HEAP32[$13>>2] = $2;
  STACKTOP = sp;return;
 }
}
function ___shgetc($f) {
 $f = $f|0;
 var $$0 = 0, $$phi$trans$insert = 0, $$phi$trans$insert2 = 0, $$pre = 0, $$pre3 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 104|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  label = 3;
 } else {
  $3 = (($f) + 108|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)<($1|0);
  if ($5) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $6 = (___uflow($f)|0);
  $7 = ($6|0)<(0);
  if (!($7)) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)==(0);
   $$phi$trans$insert = (($f) + 8|0);
   $$pre = HEAP32[$$phi$trans$insert>>2]|0;
   if ($10) {
    label = 8;
   } else {
    $11 = (($f) + 4|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = $$pre;
    $14 = $12;
    $15 = (($13) - ($14))|0;
    $16 = (($f) + 108|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($9) - ($17))|0;
    $19 = (($18) + -1)|0;
    $20 = ($15|0)>($19|0);
    if ($20) {
     $21 = (($12) + ($19)|0);
     $22 = (($f) + 100|0);
     HEAP32[$22>>2] = $21;
    } else {
     label = 8;
    }
   }
   if ((label|0) == 8) {
    $23 = (($f) + 100|0);
    HEAP32[$23>>2] = $$pre;
   }
   $24 = ($$pre|0)==(0|0);
   $$phi$trans$insert2 = (($f) + 4|0);
   $$pre3 = HEAP32[$$phi$trans$insert2>>2]|0;
   if (!($24)) {
    $25 = $$pre;
    $26 = $$pre3;
    $27 = (($f) + 108|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (($25) + 1)|0;
    $30 = (($29) - ($26))|0;
    $31 = (($30) + ($28))|0;
    HEAP32[$27>>2] = $31;
   }
   $32 = (($$pre3) + -1|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = ($34|0)==($6|0);
   if ($35) {
    $$0 = $6;
    STACKTOP = sp;return ($$0|0);
   }
   $36 = $6&255;
   HEAP8[$32>>0] = $36;
   $$0 = $6;
   STACKTOP = sp;return ($$0|0);
  }
 }
 $8 = (($f) + 100|0);
 HEAP32[$8>>2] = 0;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function _frexp($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $$0 = 0.0, $$01 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0, $storemerge = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 if ((($4|0) == 2047)) {
  $$0 = $x;
  STACKTOP = sp;return (+$$0);
 } else if ((($4|0) == 0)) {
  $5 = $x != 0.0;
  if ($5) {
   $6 = $x * 18446744073709551616.0;
   $7 = (+_frexp($6,$e));
   $8 = HEAP32[$e>>2]|0;
   $9 = (($8) + -64)|0;
   $$01 = $7;$storemerge = $9;
  } else {
   $$01 = $x;$storemerge = 0;
  }
  HEAP32[$e>>2] = $storemerge;
  $$0 = $$01;
  STACKTOP = sp;return (+$$0);
 } else {
  $10 = (($4) + -1022)|0;
  HEAP32[$e>>2] = $10;
  $11 = $1 & -2146435073;
  $12 = $11 | 1071644672;
  HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $12;$13 = +HEAPF64[tempDoublePtr>>3];
  $$0 = $13;
  STACKTOP = sp;return (+$$0);
 }
 return +0;
}
function _frexpl($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_frexp($x,$e));
 STACKTOP = sp;return (+$0);
}
function _scalbn($x,$n) {
 $x = +$x;
 $n = $n|0;
 var $$ = 0, $$0 = 0, $$1 = 0, $0 = 0, $1 = 0.0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $2 = 0, $3 = 0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0.0, $9 = 0, $y$0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)>(1023);
 if ($0) {
  $1 = $x * 8.98846567431157953864E+307;
  $2 = (($n) + -1023)|0;
  $3 = ($2|0)>(1023);
  if ($3) {
   $4 = $1 * 8.98846567431157953864E+307;
   $5 = (($n) + -2046)|0;
   $6 = ($5|0)>(1023);
   $$ = $6 ? 1023 : $5;
   $$0 = $$;$y$0 = $4;
  } else {
   $$0 = $2;$y$0 = $1;
  }
 } else {
  $7 = ($n|0)<(-1022);
  if ($7) {
   $8 = $x * 2.22507385850720138309E-308;
   $9 = (($n) + 1022)|0;
   $10 = ($9|0)<(-1022);
   if ($10) {
    $11 = $8 * 2.22507385850720138309E-308;
    $12 = (($n) + 2044)|0;
    $13 = ($12|0)<(-1022);
    $$1 = $13 ? -1022 : $12;
    $$0 = $$1;$y$0 = $11;
   } else {
    $$0 = $9;$y$0 = $8;
   }
  } else {
   $$0 = $n;$y$0 = $x;
  }
 }
 $14 = (($$0) + 1023)|0;
 $15 = (_bitshift64Shl(($14|0),0,52)|0);
 $16 = tempRet0;
 HEAP32[tempDoublePtr>>2] = $15;HEAP32[tempDoublePtr+4>>2] = $16;$17 = +HEAPF64[tempDoublePtr>>3];
 $18 = $y$0 * $17;
 STACKTOP = sp;return (+$18);
}
function _scalbnl($x,$n) {
 $x = +$x;
 $n = $n|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_scalbn($x,$n));
 STACKTOP = sp;return (+$0);
}
function _wctomb($s,$wc) {
 $s = $s|0;
 $wc = $wc|0;
 var $$0 = 0, $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 0;
 } else {
  $1 = (_wcrtomb($s,$wc,0)|0);
  $$0 = $1;
 }
 STACKTOP = sp;return ($$0|0);
}
function _wcrtomb($s,$wc,$st) {
 $s = $s|0;
 $wc = $wc|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 1;
  STACKTOP = sp;return ($$0|0);
 }
 $1 = ($wc>>>0)<(128);
 if ($1) {
  $2 = $wc&255;
  HEAP8[$s>>0] = $2;
  $$0 = 1;
  STACKTOP = sp;return ($$0|0);
 }
 $3 = ($wc>>>0)<(2048);
 if ($3) {
  $4 = $wc >>> 6;
  $5 = $4 | 192;
  $6 = $5&255;
  $7 = (($s) + 1|0);
  HEAP8[$s>>0] = $6;
  $8 = $wc & 63;
  $9 = $8 | 128;
  $10 = $9&255;
  HEAP8[$7>>0] = $10;
  $$0 = 2;
  STACKTOP = sp;return ($$0|0);
 }
 $11 = ($wc>>>0)<(55296);
 $12 = (($wc) + -57344)|0;
 $13 = ($12>>>0)<(8192);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = $wc >>> 12;
  $15 = $14 | 224;
  $16 = $15&255;
  $17 = (($s) + 1|0);
  HEAP8[$s>>0] = $16;
  $18 = $wc >>> 6;
  $19 = $18 & 63;
  $20 = $19 | 128;
  $21 = $20&255;
  $22 = (($s) + 2|0);
  HEAP8[$17>>0] = $21;
  $23 = $wc & 63;
  $24 = $23 | 128;
  $25 = $24&255;
  HEAP8[$22>>0] = $25;
  $$0 = 3;
  STACKTOP = sp;return ($$0|0);
 }
 $26 = (($wc) + -65536)|0;
 $27 = ($26>>>0)<(1048576);
 if ($27) {
  $28 = $wc >>> 18;
  $29 = $28 | 240;
  $30 = $29&255;
  $31 = (($s) + 1|0);
  HEAP8[$s>>0] = $30;
  $32 = $wc >>> 12;
  $33 = $32 & 63;
  $34 = $33 | 128;
  $35 = $34&255;
  $36 = (($s) + 2|0);
  HEAP8[$31>>0] = $35;
  $37 = $wc >>> 6;
  $38 = $37 & 63;
  $39 = $38 | 128;
  $40 = $39&255;
  $41 = (($s) + 3|0);
  HEAP8[$36>>0] = $40;
  $42 = $wc & 63;
  $43 = $42 | 128;
  $44 = $43&255;
  HEAP8[$41>>0] = $44;
  $$0 = 4;
  STACKTOP = sp;return ($$0|0);
 } else {
  $45 = (___errno_location()|0);
  HEAP32[$45>>2] = 84;
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function ___toread($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = (($f) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($f) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($7>>>0)>($9>>>0);
 if ($10) {
  $11 = (($f) + 36|0);
  $12 = HEAP32[$11>>2]|0;
  (FUNCTION_TABLE_iiii[$12 & 127]($f,0,0)|0);
 }
 $13 = (($f) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = (($f) + 28|0);
 HEAP32[$14>>2] = 0;
 HEAP32[$6>>2] = 0;
 $15 = HEAP32[$f>>2]|0;
 $16 = $15 & 20;
 $17 = ($16|0)==(0);
 if ($17) {
  $21 = HEAP32[$8>>2]|0;
  $22 = (($f) + 8|0);
  HEAP32[$22>>2] = $21;
  $23 = (($f) + 4|0);
  HEAP32[$23>>2] = $21;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 $18 = $15 & 4;
 $19 = ($18|0)==(0);
 if ($19) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $20 = $15 | 32;
 HEAP32[$f>>2] = $20;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function ___towrite($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = HEAP32[$f>>2]|0;
 $7 = $6 & 8;
 $8 = ($7|0)==(0);
 if ($8) {
  $10 = (($f) + 8|0);
  HEAP32[$10>>2] = 0;
  $11 = (($f) + 4|0);
  HEAP32[$11>>2] = 0;
  $12 = (($f) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($f) + 28|0);
  HEAP32[$14>>2] = $13;
  $15 = (($f) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = (($f) + 48|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($13) + ($17)|0);
  $19 = (($f) + 16|0);
  HEAP32[$19>>2] = $18;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 } else {
  $9 = $6 | 32;
  HEAP32[$f>>2] = $9;
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function ___uflow($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $c = sp;
 $0 = (($f) + 8|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  $3 = (___toread($f)|0);
  $4 = ($3|0)==(0);
  if ($4) {
   label = 3;
  } else {
   $$0 = -1;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $5 = (($f) + 32|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = (FUNCTION_TABLE_iiii[$6 & 127]($f,$c,1)|0);
  $8 = ($7|0)==(1);
  if ($8) {
   $9 = HEAP8[$c>>0]|0;
   $10 = $9&255;
   $$0 = $10;
  } else {
   $$0 = -1;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___fwritex($s,$l,$f) {
 $s = $s|0;
 $l = $l|0;
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$pre = 0, $$pre6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$0 = 0, $i$1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 do {
  if ($2) {
   $3 = (___towrite($f)|0);
   $4 = ($3|0)==(0);
   if ($4) {
    $$pre6 = HEAP32[$0>>2]|0;
    $8 = $$pre6;
    break;
   } else {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  } else {
   $8 = $1;
  }
 } while(0);
 $5 = (($f) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $8;
 $9 = $6;
 $10 = (($7) - ($9))|0;
 $11 = ($10>>>0)<($l>>>0);
 if ($11) {
  $12 = (($f) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (FUNCTION_TABLE_iiii[$13 & 127]($f,$s,$l)|0);
  $$0 = $14;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = (($f) + 75|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)>(-1);
 L11: do {
  if ($17) {
   $i$0 = $l;
   while(1) {
    $18 = ($i$0|0)==(0);
    if ($18) {
     $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
     break L11;
    }
    $19 = (($i$0) + -1)|0;
    $20 = (($s) + ($19)|0);
    $21 = HEAP8[$20>>0]|0;
    $22 = ($21<<24>>24)==(10);
    if ($22) {
     break;
    } else {
     $i$0 = $19;
    }
   }
   $23 = (($f) + 36|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (FUNCTION_TABLE_iiii[$24 & 127]($f,$s,$i$0)|0);
   $26 = ($25>>>0)<($i$0>>>0);
   if ($26) {
    $$0 = $i$0;
    STACKTOP = sp;return ($$0|0);
   } else {
    $27 = (($s) + ($i$0)|0);
    $28 = (($l) - ($i$0))|0;
    $$pre = HEAP32[$5>>2]|0;
    $$01 = $28;$$02 = $27;$29 = $$pre;$i$1 = $i$0;
    break;
   }
  } else {
   $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
  }
 } while(0);
 _memcpy(($29|0),($$02|0),($$01|0))|0;
 $30 = HEAP32[$5>>2]|0;
 $31 = (($30) + ($$01)|0);
 HEAP32[$5>>2] = $31;
 $32 = (($i$1) + ($$01))|0;
 $$0 = $32;
 STACKTOP = sp;return ($$0|0);
}
function _sprintf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsprintf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfprintf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap2 = 0, $internal_buf = 0, $nl_arg = 0, $nl_type = 0, $ret$1 = 0, $vacopy_currentptr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $ap2 = sp + 120|0;
 $nl_type = sp + 80|0;
 $nl_arg = sp;
 $internal_buf = sp + 136|0;
 dest=$nl_type+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $vacopy_currentptr = HEAP32[$ap>>2]|0;
 HEAP32[$ap2>>2] = $vacopy_currentptr;
 $0 = (_printf_core(0,$fmt,$ap2,$nl_arg,$nl_type)|0);
 $1 = ($0|0)<(0);
 if ($1) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $2 = (($f) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $6 = (($f) + 44|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$6>>2] = $internal_buf;
  $8 = (($f) + 28|0);
  HEAP32[$8>>2] = $internal_buf;
  $9 = (($f) + 20|0);
  HEAP32[$9>>2] = $internal_buf;
  HEAP32[$2>>2] = 80;
  $10 = (($internal_buf) + 80|0);
  $11 = (($f) + 16|0);
  HEAP32[$11>>2] = $10;
  $12 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $13 = ($7|0)==(0|0);
  if ($13) {
   $ret$1 = $12;
  } else {
   $14 = (($f) + 36|0);
   $15 = HEAP32[$14>>2]|0;
   (FUNCTION_TABLE_iiii[$15 & 127]($f,0,0)|0);
   $16 = HEAP32[$9>>2]|0;
   $17 = ($16|0)==(0|0);
   $$ = $17 ? -1 : $12;
   HEAP32[$6>>2] = $7;
   HEAP32[$2>>2] = 0;
   HEAP32[$11>>2] = 0;
   HEAP32[$8>>2] = 0;
   HEAP32[$9>>2] = 0;
   $ret$1 = $$;
  }
 } else {
  $5 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $ret$1 = $5;
 }
 $$0 = $ret$1;
 STACKTOP = sp;return ($$0|0);
}
function _printf_core($f,$fmt,$ap,$nl_arg,$nl_type) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 $nl_arg = $nl_arg|0;
 $nl_type = $nl_type|0;
 var $$ = 0, $$$5$i = 0, $$$i = 0, $$$p$i = 0, $$0 = 0, $$0$lcssa$i = 0, $$0$lcssa$i$i = 0, $$0$lcssa$i104$i = 0, $$0$lcssa$i128$i = 0, $$0$lcssa$i143$i = 0, $$0$lcssa$i38 = 0, $$0$lcssa$i39$i = 0, $$0$lcssa$i44 = 0, $$0$lcssa$i46 = 0, $$0$lcssa$i46$i = 0, $$0$lcssa$i49$i = 0, $$0$lcssa$i53 = 0, $$0$lcssa$i56$i = 0, $$0$lcssa$i60 = 0, $$0$lcssa$i63$i = 0;
 var $$0$lcssa$i67 = 0, $$0$lcssa$i69$i = 0, $$0$lcssa$i76$i = 0, $$0$lcssa$i78 = 0, $$0$lcssa$i85$i = 0, $$0$lcssa$i97$i = 0, $$01$i = 0, $$01$i$i = 0, $$01$i102$i = 0, $$01$i126$i = 0, $$01$i141$i = 0, $$01$i36 = 0, $$01$i37$i = 0, $$01$i44$i = 0, $$01$i51 = 0, $$01$i54$i = 0, $$01$i58 = 0, $$01$i61$i = 0, $$01$i65 = 0, $$01$i67$i = 0;
 var $$01$i74$i = 0, $$01$i76 = 0, $$01$i95$i = 0, $$01$lcssa$off0$i = 0, $$01$lcssa$off0$i$i = 0, $$01$lcssa$off0$i86$i = 0, $$012$i = 0, $$012$i73 = 0, $$03$i41 = 0, $$05$i = 0, $$05$i$i = 0, $$05$i80$i = 0, $$06$i = 0.0, $$1$i = 0.0, $$1$lcssa$i$i = 0, $$1$lcssa$i113$i = 0, $$113$i = 0, $$12$i = 0, $$12$i$i = 0, $$12$i111$i = 0;
 var $$12$i120$i = 0, $$12$i135$i = 0, $$12$i88$i = 0, $$14 = 0, $$15 = 0, $$16 = 0, $$18 = 0, $$2$i = 0.0, $$2$us$i = 0.0, $$2$us$us$i = 0.0, $$2$us160$i = 0.0, $$20$i = 0, $$20$us$i = 0, $$21$i = 0, $$22$i = 0.0, $$24$i = 0, $$26$i = 0, $$29$$24$i = 0, $$29$$26$i = 0, $$29$i = 0;
 var $$3$i = 0.0, $$310$i = 0, $$32$i = 0, $$4$i = 0.0, $$411$lcssa$i = 0, $$411176$i = 0, $$5194$i = 0, $$a$3$i = 0, $$a$3$us$i = 0, $$a$3$us319$i = 0, $$a$3$us320$i = 0, $$a$3321$i = 0, $$a$3322$i = 0, $$fl$4 = 0, $$lcssa94 = 0, $$mask$i = 0, $$mask$i31 = 0, $$mask1$i = 0, $$mask1$i30 = 0, $$neg152$i = 0;
 var $$neg153$i = 0, $$not$i = 0, $$p$5 = 0, $$p$i = 0, $$pn$i = 0, $$pr$i = 0, $$pr147$i = 0, $$pre = 0, $$pre$i = 0, $$pre291 = 0, $$pre292 = 0, $$pre312$i = 0, $$sink = 0, $$sink300$off32 = 0, $$sum$i = 0, $$sum14$i = 0, $$sum15$i = 0, $$z$3$i = 0, $$z$4$us$i = 0, $0 = 0;
 var $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0;
 var $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0;
 var $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0.0, $104 = 0, $1040 = 0.0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1048$phi = 0, $1049 = 0, $1049$phi = 0, $105 = 0;
 var $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0.0, $369 = 0, $37 = 0, $370 = 0.0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0.0, $422 = 0, $423 = 0, $424 = 0, $425 = 0.0, $426 = 0.0, $427 = 0.0, $428 = 0.0, $429 = 0.0, $43 = 0, $430 = 0.0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0.0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0.0, $495 = 0.0, $496 = 0.0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0.0, $51 = 0, $510 = 0.0;
 var $511 = 0.0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0.0, $525 = 0.0, $526 = 0.0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0.0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0.0, $585 = 0.0, $586 = 0.0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0;
 var $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0;
 var $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0;
 var $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0.0;
 var $728 = 0.0, $729 = 0, $73 = 0, $730 = 0.0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0;
 var $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0;
 var $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0;
 var $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0;
 var $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0;
 var $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0;
 var $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0;
 var $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0;
 var $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0;
 var $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0;
 var $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0;
 var $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0;
 var $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0;
 var $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0;
 var $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0;
 var $999 = 0, $a$0 = 0, $a$1 = 0, $a$1$lcssa$i = 0, $a$1257$i = 0, $a$2 = 0, $a$2$i = 0, $a$3$lcssa$i = 0, $a$3245$i = 0, $a$3245$us$i = 0, $a$6$i = 0, $a$7$i = 0, $arglist_current = 0, $arglist_current11 = 0, $arglist_current14 = 0, $arglist_current17 = 0, $arglist_current2 = 0, $arglist_current20 = 0, $arglist_current23 = 0, $arglist_current26 = 0;
 var $arglist_current29 = 0, $arglist_current32 = 0, $arglist_current35 = 0, $arglist_current38 = 0, $arglist_current41 = 0, $arglist_current44 = 0, $arglist_current47 = 0, $arglist_current5 = 0, $arglist_current50 = 0, $arglist_current53 = 0, $arglist_current56 = 0, $arglist_current59 = 0, $arglist_current62 = 0, $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next15 = 0, $arglist_next18 = 0, $arglist_next21 = 0, $arglist_next24 = 0;
 var $arglist_next27 = 0, $arglist_next3 = 0, $arglist_next30 = 0, $arglist_next33 = 0, $arglist_next36 = 0, $arglist_next39 = 0, $arglist_next42 = 0, $arglist_next45 = 0, $arglist_next48 = 0, $arglist_next51 = 0, $arglist_next54 = 0, $arglist_next57 = 0, $arglist_next6 = 0, $arglist_next60 = 0, $arglist_next63 = 0, $arglist_next9 = 0, $argpos$0 = 0, $big$i = 0, $brmerge$i = 0, $buf = 0;
 var $buf$i = 0, $carry$0250$i = 0, $carry3$0239$i = 0, $carry3$0239$us$i = 0, $cnt$0 = 0, $cnt$1 = 0, $d$0251$i = 0, $d$1238$i = 0, $d$1238$us$i = 0, $d$2$a$3$i = 0, $d$2$lcssa$i = 0, $d$2220$i = 0, $d$3$i = 0, $d$4184$i = 0, $d$5175$i = 0, $d$6193$i = 0, $e$0233$i = 0, $e$1$i = 0, $e$2215$i = 0, $e$3$i = 0;
 var $e$4$i = 0, $e2$i = 0, $ebuf0$i = 0, $estr$0$i = 0, $estr$1$lcssa$i = 0, $estr$1$ph$i = 0, $estr$1201$i = 0, $estr$2$i = 0, $exitcond$i = 0, $fl$0102 = 0, $fl$0106 = 0, $fl$1 = 0, $fl$1$ = 0, $fl$3 = 0, $fl$4 = 0, $fl$6 = 0, $i$0$lcssa = 0, $i$0167 = 0, $i$0169 = 0, $i$0234$i = 0;
 var $i$03$i = 0, $i$03$i23 = 0, $i$1$lcssa$i = 0, $i$1175 = 0, $i$1226$i = 0, $i$2216$i = 0, $i$291 = 0, $i$3210$i = 0, $i$390 = 0, $isdigit = 0, $isdigit$i = 0, $isdigit$i25 = 0, $isdigit10 = 0, $isdigit12 = 0, $isdigit2$i = 0, $isdigit2$i22 = 0, $isdigittmp = 0, $isdigittmp$i = 0, $isdigittmp$i24 = 0, $isdigittmp1$i = 0;
 var $isdigittmp1$i21 = 0, $isdigittmp11 = 0, $isdigittmp9 = 0, $j$0$i = 0, $j$0225$i = 0, $j$0227$i = 0, $j$1211$i = 0, $j$2$i = 0, $l$0 = 0, $l$0$i = 0, $l$1$i = 0, $l$1$lcssa = 0, $l$1168 = 0, $l10n$0 = 0, $l10n$0$phi = 0, $l10n$1 = 0, $l10n$2 = 0, $l10n$3 = 0, $mb = 0, $or$cond = 0;
 var $or$cond$i = 0, $or$cond$i$i = 0, $or$cond$i101$i = 0, $or$cond$i36$i = 0, $or$cond$i43$i = 0, $or$cond$i50 = 0, $or$cond$i53$i = 0, $or$cond$i57 = 0, $or$cond$i60$i = 0, $or$cond$i64 = 0, $or$cond$i72 = 0, $or$cond$i73$i = 0, $or$cond$i75 = 0, $or$cond$i94$i = 0, $or$cond19$i = 0, $or$cond19315$i = 0, $or$cond23$i = 0, $or$cond29$i = 0, $or$cond29174$i = 0, $or$cond3$i = 0;
 var $or$cond30$i = 0, $p$0 = 0, $p$1 = 0, $p$2 = 0, $p$2$ = 0, $p$4296 = 0, $p$5 = 0, $pad$i = 0, $pl$0 = 0, $pl$0$i = 0, $pl$1 = 0, $pl$1$i = 0, $pl$2 = 0, $prefix$0 = 0, $prefix$0$$i = 0, $prefix$0$i = 0, $prefix$1 = 0, $prefix$2 = 0, $r$0$a$7$i = 0, $re$0$i = 0;
 var $re$1166$i = 0, $round$0165$i = 0.0, $round6$1$i = 0.0, $s$0$i = 0, $s$0$us$i = 0, $s$0$us$us$i = 0, $s$0$us159$i = 0, $s$1$i = 0, $s$1$lcssa$i = 0, $s$1$us$i = 0, $s$1$us$us$i = 0, $s$1$us161$i = 0, $s1$0$i = 0, $s7$0181$i = 0, $s7$1$i = 0, $s8$0$lcssa$i = 0, $s8$0170$i = 0, $s9$0$i = 0, $s9$1189$i = 0, $s9$2$i = 0;
 var $sext = 0, $sext86 = 0, $small$0$i = 0.0, $small$1$i = 0.0, $st$0 = 0, $storemerge = 0, $storemerge13 = 0, $storemerge8100 = 0, $storemerge8105 = 0, $t$0 = 0, $t$1 = 0, $w$$i = 0, $w$0 = 0, $w$1 = 0, $w$18$i = 0, $w$2 = 0, $w$31$i = 0, $wc = 0, $ws$0170 = 0, $ws$1176 = 0;
 var $y$03$i = 0, $y$03$i$i = 0, $y$03$i110$i = 0, $y$03$i119$i = 0, $y$03$i134$i = 0, $y$03$i87$i = 0, $z$0$i = 0, $z$0$lcssa = 0, $z$095 = 0, $z$1$lcssa$i = 0, $z$1256$i = 0, $z$2 = 0, $z$2$i = 0, $z$2316$i = 0, $z$2317$i = 0, $z$3$lcssa$i = 0, $z$3244$i = 0, $z$3244$us$i = 0, $z$4$i = 0, $z$4$us$i = 0;
 var $z$5$i = 0, $z$6$i = 0, $z$7$$i = 0, $z$7$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 848|0;
 $big$i = sp + 16|0;
 $e2$i = sp + 8|0;
 $buf$i = sp + 812|0;
 $0 = $buf$i;
 $ebuf0$i = sp + 800|0;
 $pad$i = sp + 544|0;
 $buf = sp + 504|0;
 $wc = sp;
 $mb = sp + 496|0;
 $1 = ($f|0)!=(0|0);
 $2 = (($buf) + 40|0);
 $3 = $2;
 $4 = (($buf) + 39|0);
 $5 = (($wc) + 4|0);
 $6 = (($ebuf0$i) + 12|0);
 $7 = (($ebuf0$i) + 11|0);
 $8 = $6;
 $9 = (($8) - ($0))|0;
 $10 = (-2 - ($0))|0;
 $11 = (($8) + 2)|0;
 $12 = (($big$i) + 264|0);
 $13 = (($buf$i) + 9|0);
 $14 = $13;
 $15 = (($buf$i) + 8|0);
 $1048 = 0;$1049 = 0;$22 = $fmt;$cnt$0 = 0;$l$0 = 0;$l10n$0 = 0;
 L1: while(1) {
  $16 = ($cnt$0|0)>(-1);
  do {
   if ($16) {
    $17 = (2147483647 - ($cnt$0))|0;
    $18 = ($l$0|0)>($17|0);
    if ($18) {
     $19 = (___errno_location()|0);
     HEAP32[$19>>2] = 75;
     $cnt$1 = -1;
     break;
    } else {
     $20 = (($l$0) + ($cnt$0))|0;
     $cnt$1 = $20;
     break;
    }
   } else {
    $cnt$1 = $cnt$0;
   }
  } while(0);
  $21 = HEAP8[$22>>0]|0;
  $23 = ($21<<24>>24)==(0);
  if ($23) {
   label = 339;
   break;
  } else {
   $1050 = $21;$25 = $22;
  }
  while(1) {
   if ((($1050<<24>>24) == 37)) {
    $27 = $25;$z$095 = $25;
    label = 9;
    break;
   } else if ((($1050<<24>>24) == 0)) {
    $$lcssa94 = $25;$z$0$lcssa = $25;
    break;
   }
   $24 = (($25) + 1|0);
   $$pre = HEAP8[$24>>0]|0;
   $1050 = $$pre;$25 = $24;
  }
  L12: do {
   if ((label|0) == 9) {
    while(1) {
     label = 0;
     $26 = (($27) + 1|0);
     $28 = HEAP8[$26>>0]|0;
     $29 = ($28<<24>>24)==(37);
     if (!($29)) {
      $$lcssa94 = $27;$z$0$lcssa = $z$095;
      break L12;
     }
     $30 = (($z$095) + 1|0);
     $31 = (($27) + 2|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = ($32<<24>>24)==(37);
     if ($33) {
      $27 = $31;$z$095 = $30;
      label = 9;
     } else {
      $$lcssa94 = $31;$z$0$lcssa = $30;
      break;
     }
    }
   }
  } while(0);
  $34 = $z$0$lcssa;
  $35 = $22;
  $36 = (($34) - ($35))|0;
  if ($1) {
   (___fwritex($22,$36,$f)|0);
  }
  $37 = ($z$0$lcssa|0)==($22|0);
  if (!($37)) {
   $l10n$0$phi = $l10n$0;$1049$phi = $1049;$1048$phi = $1048;$22 = $$lcssa94;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$0$phi;$1049 = $1049$phi;$1048 = $1048$phi;
   continue;
  }
  $38 = (($$lcssa94) + 1|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39 << 24 >> 24;
  $isdigittmp = (($40) + -48)|0;
  $isdigit = ($isdigittmp>>>0)<(10);
  if ($isdigit) {
   $41 = (($$lcssa94) + 2|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = ($42<<24>>24)==(36);
   if ($43) {
    $44 = (($$lcssa94) + 3|0);
    $$pre291 = HEAP8[$44>>0]|0;
    $46 = $$pre291;$argpos$0 = $isdigittmp;$l10n$1 = 1;$storemerge = $44;
   } else {
    $46 = $39;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $38;
   }
  } else {
   $46 = $39;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $38;
  }
  $45 = $46 << 24 >> 24;
  $47 = (($45) + -32)|0;
  $48 = ($47>>>0)<(32);
  L25: do {
   if ($48) {
    $50 = $45;$55 = $46;$fl$0106 = 0;$storemerge8105 = $storemerge;
    while(1) {
     $49 = (($50) + -32)|0;
     $51 = 1 << $49;
     $52 = $51 & 75913;
     $53 = ($52|0)==(0);
     if ($53) {
      $65 = $55;$fl$0102 = $fl$0106;$storemerge8100 = $storemerge8105;
      break L25;
     }
     $54 = $55 << 24 >> 24;
     $56 = (($54) + -32)|0;
     $57 = 1 << $56;
     $58 = $57 | $fl$0106;
     $59 = (($storemerge8105) + 1|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60 << 24 >> 24;
     $62 = (($61) + -32)|0;
     $63 = ($62>>>0)<(32);
     if ($63) {
      $50 = $61;$55 = $60;$fl$0106 = $58;$storemerge8105 = $59;
     } else {
      $65 = $60;$fl$0102 = $58;$storemerge8100 = $59;
      break;
     }
    }
   } else {
    $65 = $46;$fl$0102 = 0;$storemerge8100 = $storemerge;
   }
  } while(0);
  $64 = ($65<<24>>24)==(42);
  do {
   if ($64) {
    $66 = (($storemerge8100) + 1|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67 << 24 >> 24;
    $isdigittmp11 = (($68) + -48)|0;
    $isdigit12 = ($isdigittmp11>>>0)<(10);
    if ($isdigit12) {
     $69 = (($storemerge8100) + 2|0);
     $70 = HEAP8[$69>>0]|0;
     $71 = ($70<<24>>24)==(36);
     if ($71) {
      $72 = (($nl_type) + ($isdigittmp11<<2)|0);
      HEAP32[$72>>2] = 10;
      $73 = HEAP8[$66>>0]|0;
      $74 = $73 << 24 >> 24;
      $75 = (($74) + -48)|0;
      $76 = (($nl_arg) + ($75<<3)|0);
      $77 = $76;
      $78 = $77;
      $79 = HEAP32[$78>>2]|0;
      $80 = (($77) + 4)|0;
      $81 = $80;
      $82 = HEAP32[$81>>2]|0;
      $83 = (($storemerge8100) + 3|0);
      $l10n$2 = 1;$storemerge13 = $83;$w$0 = $79;
     } else {
      label = 24;
     }
    } else {
     label = 24;
    }
    if ((label|0) == 24) {
     label = 0;
     $84 = ($l10n$1|0)==(0);
     if (!($84)) {
      $$0 = -1;
      label = 357;
      break L1;
     }
     if (!($1)) {
      $100 = $66;$fl$1 = $fl$0102;$l10n$3 = 0;$w$1 = 0;
      break;
     }
     $arglist_current = HEAP32[$ap>>2]|0;
     $85 = HEAP32[$arglist_current>>2]|0;
     $arglist_next = (($arglist_current) + 4|0);
     HEAP32[$ap>>2] = $arglist_next;
     $l10n$2 = 0;$storemerge13 = $66;$w$0 = $85;
    }
    $86 = ($w$0|0)<(0);
    if ($86) {
     $87 = $fl$0102 | 8192;
     $88 = (0 - ($w$0))|0;
     $100 = $storemerge13;$fl$1 = $87;$l10n$3 = $l10n$2;$w$1 = $88;
    } else {
     $100 = $storemerge13;$fl$1 = $fl$0102;$l10n$3 = $l10n$2;$w$1 = $w$0;
    }
   } else {
    $89 = $65 << 24 >> 24;
    $isdigittmp1$i = (($89) + -48)|0;
    $isdigit2$i = ($isdigittmp1$i>>>0)<(10);
    if ($isdigit2$i) {
     $92 = $89;$95 = $storemerge8100;$i$03$i = 0;
     while(1) {
      $90 = ($i$03$i*10)|0;
      $91 = (($92) + -48)|0;
      $93 = (($91) + ($90))|0;
      $94 = (($95) + 1|0);
      $96 = HEAP8[$94>>0]|0;
      $97 = $96 << 24 >> 24;
      $isdigittmp$i = (($97) + -48)|0;
      $isdigit$i = ($isdigittmp$i>>>0)<(10);
      if ($isdigit$i) {
       $92 = $97;$95 = $94;$i$03$i = $93;
      } else {
       break;
      }
     }
     $98 = ($93|0)<(0);
     if ($98) {
      $$0 = -1;
      label = 357;
      break L1;
     } else {
      $100 = $94;$fl$1 = $fl$0102;$l10n$3 = $l10n$1;$w$1 = $93;
     }
    } else {
     $100 = $storemerge8100;$fl$1 = $fl$0102;$l10n$3 = $l10n$1;$w$1 = 0;
    }
   }
  } while(0);
  $99 = HEAP8[$100>>0]|0;
  $101 = ($99<<24>>24)==(46);
  L46: do {
   if ($101) {
    $102 = (($100) + 1|0);
    $103 = HEAP8[$102>>0]|0;
    $104 = ($103<<24>>24)==(42);
    if (!($104)) {
     $125 = $103 << 24 >> 24;
     $isdigittmp1$i21 = (($125) + -48)|0;
     $isdigit2$i22 = ($isdigittmp1$i21>>>0)<(10);
     if ($isdigit2$i22) {
      $128 = $125;$131 = $102;$i$03$i23 = 0;
     } else {
      $1051 = $102;$p$0 = 0;
      break;
     }
     while(1) {
      $126 = ($i$03$i23*10)|0;
      $127 = (($128) + -48)|0;
      $129 = (($127) + ($126))|0;
      $130 = (($131) + 1|0);
      $132 = HEAP8[$130>>0]|0;
      $133 = $132 << 24 >> 24;
      $isdigittmp$i24 = (($133) + -48)|0;
      $isdigit$i25 = ($isdigittmp$i24>>>0)<(10);
      if ($isdigit$i25) {
       $128 = $133;$131 = $130;$i$03$i23 = $129;
      } else {
       $1051 = $130;$p$0 = $129;
       break L46;
      }
     }
    }
    $105 = (($100) + 2|0);
    $106 = HEAP8[$105>>0]|0;
    $107 = $106 << 24 >> 24;
    $isdigittmp9 = (($107) + -48)|0;
    $isdigit10 = ($isdigittmp9>>>0)<(10);
    if ($isdigit10) {
     $108 = (($100) + 3|0);
     $109 = HEAP8[$108>>0]|0;
     $110 = ($109<<24>>24)==(36);
     if ($110) {
      $111 = (($nl_type) + ($isdigittmp9<<2)|0);
      HEAP32[$111>>2] = 10;
      $112 = HEAP8[$105>>0]|0;
      $113 = $112 << 24 >> 24;
      $114 = (($113) + -48)|0;
      $115 = (($nl_arg) + ($114<<3)|0);
      $116 = $115;
      $117 = $116;
      $118 = HEAP32[$117>>2]|0;
      $119 = (($116) + 4)|0;
      $120 = $119;
      $121 = HEAP32[$120>>2]|0;
      $122 = (($100) + 4|0);
      $1051 = $122;$p$0 = $118;
      break;
     }
    }
    $123 = ($l10n$3|0)==(0);
    if (!($123)) {
     $$0 = -1;
     label = 357;
     break L1;
    }
    if ($1) {
     $arglist_current2 = HEAP32[$ap>>2]|0;
     $124 = HEAP32[$arglist_current2>>2]|0;
     $arglist_next3 = (($arglist_current2) + 4|0);
     HEAP32[$ap>>2] = $arglist_next3;
     $1051 = $105;$p$0 = $124;
    } else {
     $1051 = $105;$p$0 = 0;
    }
   } else {
    $1051 = $100;$p$0 = -1;
   }
  } while(0);
  $135 = $1051;$st$0 = 0;
  while(1) {
   $134 = HEAP8[$135>>0]|0;
   $136 = $134 << 24 >> 24;
   $137 = (($136) + -65)|0;
   $138 = ($137>>>0)>(57);
   if ($138) {
    $$0 = -1;
    label = 357;
    break L1;
   }
   $139 = (($135) + 1|0);
   $140 = ((11200 + (($st$0*58)|0)|0) + ($137)|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141&255;
   $143 = (($142) + -1)|0;
   $144 = ($143>>>0)<(8);
   if ($144) {
    $135 = $139;$st$0 = $142;
   } else {
    break;
   }
  }
  $145 = ($141<<24>>24)==(0);
  if ($145) {
   $$0 = -1;
   label = 357;
   break;
  }
  $146 = ($141<<24>>24)==(19);
  $147 = ($argpos$0|0)>(-1);
  L65: do {
   if ($146) {
    if ($147) {
     $$0 = -1;
     label = 357;
     break L1;
    }
    if ($1) {
     $198 = $134;$206 = $1049;$228 = $1048;
    } else {
     $1049$phi = $1049;$1048$phi = $1048;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;$1049 = $1049$phi;$1048 = $1048$phi;
     continue L1;
    }
   } else {
    L69: do {
     if ($147) {
      $148 = (($nl_type) + ($argpos$0<<2)|0);
      HEAP32[$148>>2] = $142;
      $149 = (($nl_arg) + ($argpos$0<<3)|0);
      $150 = $149;
      $151 = $150;
      $152 = HEAP32[$151>>2]|0;
      $153 = (($150) + 4)|0;
      $154 = $153;
      $155 = HEAP32[$154>>2]|0;
      $$sink = $152;$$sink300$off32 = $155;
      label = 64;
     } else {
      if (!($1)) {
       $$0 = 0;
       label = 357;
       break L1;
      }
      $156 = ($141&255)>(20);
      if ($156) {
       $198 = $134;$206 = $1049;$228 = $1048;
       break L65;
      }
      do {
       switch ($142|0) {
       case 17:  {
        $arglist_current29 = HEAP32[$ap>>2]|0;
        HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current29>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current29+4>>2];$189 = +HEAPF64[tempDoublePtr>>3];
        $arglist_next30 = (($arglist_current29) + 8|0);
        HEAP32[$ap>>2] = $arglist_next30;
        HEAPF64[tempDoublePtr>>3] = $189;$190 = HEAP32[tempDoublePtr>>2]|0;
        $191 = HEAP32[tempDoublePtr+4>>2]|0;
        $192 = $190;
        $1052 = $192;$1053 = $191;
        break L69;
        break;
       }
       case 13:  {
        $arglist_current17 = HEAP32[$ap>>2]|0;
        $171 = HEAP32[$arglist_current17>>2]|0;
        $arglist_next18 = (($arglist_current17) + 4|0);
        HEAP32[$ap>>2] = $arglist_next18;
        $172 = $171&65535;
        $173 = $172 << 16 >> 16;
        $174 = ($173|0)<(0);
        $175 = $174 << 31 >> 31;
        $sext86 = $171 << 16;
        $176 = $sext86 >> 16;
        $177 = $176;
        $1052 = $177;$1053 = $175;
        break L69;
        break;
       }
       case 18:  {
        $arglist_current32 = HEAP32[$ap>>2]|0;
        HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current32>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current32+4>>2];$193 = +HEAPF64[tempDoublePtr>>3];
        $arglist_next33 = (($arglist_current32) + 8|0);
        HEAP32[$ap>>2] = $arglist_next33;
        HEAPF64[tempDoublePtr>>3] = $193;$194 = HEAP32[tempDoublePtr>>2]|0;
        $195 = HEAP32[tempDoublePtr+4>>2]|0;
        $$sink = $194;$$sink300$off32 = $195;
        label = 64;
        break L69;
        break;
       }
       case 11:  {
        $arglist_current11 = HEAP32[$ap>>2]|0;
        $162 = HEAP32[$arglist_current11>>2]|0;
        $arglist_next12 = (($arglist_current11) + 4|0);
        HEAP32[$ap>>2] = $arglist_next12;
        $163 = $162;
        $1052 = $163;$1053 = 0;
        break L69;
        break;
       }
       case 16:  {
        $arglist_current26 = HEAP32[$ap>>2]|0;
        $187 = HEAP32[$arglist_current26>>2]|0;
        $arglist_next27 = (($arglist_current26) + 4|0);
        HEAP32[$ap>>2] = $arglist_next27;
        $$mask$i31 = $187 & 255;
        $188 = $$mask$i31;
        $1052 = $188;$1053 = 0;
        break L69;
        break;
       }
       case 12:  {
        $arglist_current14 = HEAP32[$ap>>2]|0;
        $164 = $arglist_current14;
        $165 = $164;
        $166 = HEAP32[$165>>2]|0;
        $167 = (($164) + 4)|0;
        $168 = $167;
        $169 = HEAP32[$168>>2]|0;
        $arglist_next15 = (($arglist_current14) + 8|0);
        HEAP32[$ap>>2] = $arglist_next15;
        $170 = $166;
        $1052 = $170;$1053 = $169;
        break L69;
        break;
       }
       case 14:  {
        $arglist_current20 = HEAP32[$ap>>2]|0;
        $178 = HEAP32[$arglist_current20>>2]|0;
        $arglist_next21 = (($arglist_current20) + 4|0);
        HEAP32[$ap>>2] = $arglist_next21;
        $$mask1$i30 = $178 & 65535;
        $179 = $$mask1$i30;
        $1052 = $179;$1053 = 0;
        break L69;
        break;
       }
       case 15:  {
        $arglist_current23 = HEAP32[$ap>>2]|0;
        $180 = HEAP32[$arglist_current23>>2]|0;
        $arglist_next24 = (($arglist_current23) + 4|0);
        HEAP32[$ap>>2] = $arglist_next24;
        $181 = $180&255;
        $182 = $181 << 24 >> 24;
        $183 = ($182|0)<(0);
        $184 = $183 << 31 >> 31;
        $sext = $180 << 24;
        $185 = $sext >> 24;
        $186 = $185;
        $1052 = $186;$1053 = $184;
        break L69;
        break;
       }
       case 10:  {
        $arglist_current8 = HEAP32[$ap>>2]|0;
        $158 = HEAP32[$arglist_current8>>2]|0;
        $arglist_next9 = (($arglist_current8) + 4|0);
        HEAP32[$ap>>2] = $arglist_next9;
        $159 = ($158|0)<(0);
        $160 = $159 << 31 >> 31;
        $161 = $158;
        $1052 = $161;$1053 = $160;
        break L69;
        break;
       }
       case 9:  {
        $arglist_current5 = HEAP32[$ap>>2]|0;
        $157 = HEAP32[$arglist_current5>>2]|0;
        $arglist_next6 = (($arglist_current5) + 4|0);
        HEAP32[$ap>>2] = $arglist_next6;
        $1052 = $157;$1053 = $1048;
        break L69;
        break;
       }
       default: {
        $1052 = $1049;$1053 = $1048;
        break L69;
       }
       }
      } while(0);
     }
    } while(0);
    if ((label|0) == 64) {
     label = 0;
     $196 = $$sink;
     if ($1) {
      $1052 = $196;$1053 = $$sink300$off32;
     } else {
      $1048 = $$sink300$off32;$1049 = $196;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
      continue L1;
     }
    }
    $$pre292 = HEAP8[$135>>0]|0;
    $198 = $$pre292;$206 = $1052;$228 = $1053;
   }
  } while(0);
  $197 = $198 << 24 >> 24;
  $199 = ($st$0|0)==(0);
  if ($199) {
   $t$0 = $197;
  } else {
   $200 = $197 & 15;
   $201 = ($200|0)==(3);
   $202 = $197 & -33;
   $$ = $201 ? $202 : $197;
   $t$0 = $$;
  }
  $203 = $fl$1 & 8192;
  $204 = ($203|0)==(0);
  $205 = $fl$1 & -65537;
  $fl$1$ = $204 ? $fl$1 : $205;
  L92: do {
   switch ($t$0|0) {
   case 65: case 71: case 70: case 69: case 97: case 103: case 102: case 101:  {
    $367 = $206;
    HEAP32[tempDoublePtr>>2] = $367;HEAP32[tempDoublePtr+4>>2] = $228;$368 = +HEAPF64[tempDoublePtr>>3];
    HEAP32[$e2$i>>2] = 0;
    $369 = ($228|0)<(0);
    if ($369) {
     $370 = -$368;
     $$06$i = $370;$pl$0$i = 1;$prefix$0$i = 11688;
    } else {
     $371 = $fl$1$ & 2048;
     $372 = ($371|0)==(0);
     if ($372) {
      $373 = $fl$1$ & 1;
      $374 = ($373|0)==(0);
      $$$i = $374 ? ((11688 + 1|0)) : ((11688 + 6|0));
      $$06$i = $368;$pl$0$i = $373;$prefix$0$i = $$$i;
     } else {
      $$06$i = $368;$pl$0$i = 1;$prefix$0$i = ((11688 + 3|0));
     }
    }
    HEAPF64[tempDoublePtr>>3] = $$06$i;$375 = HEAP32[tempDoublePtr>>2]|0;
    $376 = HEAP32[tempDoublePtr+4>>2]|0;
    $377 = $376 & 2146435072;
    $378 = ($377>>>0)<(2146435072);
    $379 = ($377|0)==(2146435072);
    $380 = (0)<(0);
    $381 = $379 & $380;
    $382 = $378 | $381;
    if (!($382)) {
     $383 = $t$0 & 32;
     $384 = ($383|0)!=(0);
     $385 = $384 ? 11712 : 11720;
     $386 = ($$06$i != $$06$i) | (0.0 != 0.0);
     if ($386) {
      $387 = $384 ? 11728 : 11736;
      $pl$1$i = 0;$s1$0$i = $387;
     } else {
      $pl$1$i = $pl$0$i;$s1$0$i = $385;
     }
     $388 = (($pl$1$i) + 3)|0;
     $389 = $fl$1$ & 8192;
     $390 = ($389|0)==(0);
     $391 = ($388|0)<($w$1|0);
     $or$cond$i36$i = $390 & $391;
     if ($or$cond$i36$i) {
      $392 = (($w$1) - ($388))|0;
      $393 = ($392>>>0)>(256);
      $394 = $393 ? 256 : $392;
      _memset(($pad$i|0),32,($394|0))|0;
      $395 = ($392>>>0)>(255);
      if ($395) {
       $$01$i37$i = $392;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $396 = (($$01$i37$i) + -256)|0;
        $397 = ($396>>>0)>(255);
        if ($397) {
         $$01$i37$i = $396;
        } else {
         $$0$lcssa$i39$i = $396;
         break;
        }
       }
      } else {
       $$0$lcssa$i39$i = $392;
      }
      (___fwritex($pad$i,$$0$lcssa$i39$i,$f)|0);
     }
     (___fwritex($prefix$0$i,$pl$1$i,$f)|0);
     (___fwritex($s1$0$i,3,$f)|0);
     $398 = $fl$1$ & 73728;
     $399 = ($398|0)==(8192);
     $or$cond$i43$i = $399 & $391;
     if ($or$cond$i43$i) {
      $400 = (($w$1) - ($388))|0;
      $401 = ($400>>>0)>(256);
      $402 = $401 ? 256 : $400;
      _memset(($pad$i|0),32,($402|0))|0;
      $403 = ($400>>>0)>(255);
      if ($403) {
       $$01$i44$i = $400;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $404 = (($$01$i44$i) + -256)|0;
        $405 = ($404>>>0)>(255);
        if ($405) {
         $$01$i44$i = $404;
        } else {
         $$0$lcssa$i46$i = $404;
         break;
        }
       }
      } else {
       $$0$lcssa$i46$i = $400;
      }
      (___fwritex($pad$i,$$0$lcssa$i46$i,$f)|0);
     }
     $w$$i = $391 ? $w$1 : $388;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$$i;$l10n$0 = $l10n$3;
     continue L1;
    }
    $406 = (+_frexpl($$06$i,$e2$i));
    $407 = $406 * 2.0;
    $408 = $407 != 0.0;
    if ($408) {
     $409 = HEAP32[$e2$i>>2]|0;
     $410 = (($409) + -1)|0;
     HEAP32[$e2$i>>2] = $410;
    }
    $411 = $t$0 | 32;
    $412 = ($411|0)==(97);
    if ($412) {
     $413 = $t$0 & 32;
     $414 = ($413|0)==(0);
     $415 = (($prefix$0$i) + 9|0);
     $prefix$0$$i = $414 ? $prefix$0$i : $415;
     $416 = $pl$0$i | 2;
     $417 = ($p$0>>>0)>(11);
     $418 = (12 - ($p$0))|0;
     $re$0$i = $417 ? 0 : $418;
     $419 = ($re$0$i|0)==(0);
     do {
      if ($419) {
       $$1$i = $407;
      } else {
       $re$1166$i = $re$0$i;$round$0165$i = 8.0;
       while(1) {
        $420 = (($re$1166$i) + -1)|0;
        $421 = $round$0165$i * 16.0;
        $422 = ($420|0)==(0);
        if ($422) {
         break;
        } else {
         $re$1166$i = $420;$round$0165$i = $421;
        }
       }
       $423 = HEAP8[$prefix$0$$i>>0]|0;
       $424 = ($423<<24>>24)==(45);
       if ($424) {
        $425 = -$407;
        $426 = $425 - $421;
        $427 = $421 + $426;
        $428 = -$427;
        $$1$i = $428;
        break;
       } else {
        $429 = $407 + $421;
        $430 = $429 - $421;
        $$1$i = $430;
        break;
       }
      }
     } while(0);
     $431 = HEAP32[$e2$i>>2]|0;
     $432 = ($431|0)<(0);
     $433 = (0 - ($431))|0;
     $434 = $432 ? $433 : $431;
     $435 = ($434|0)<(0);
     if ($435) {
      $436 = ($434|0)<(0);
      $437 = $436 << 31 >> 31;
      $$05$i$i = $6;$438 = $434;$439 = $437;
      while(1) {
       $440 = (___uremdi3(($438|0),($439|0),10,0)|0);
       $441 = tempRet0;
       $442 = $440 | 48;
       $443 = $442&255;
       $444 = (($$05$i$i) + -1|0);
       HEAP8[$444>>0] = $443;
       $445 = (___udivdi3(($438|0),($439|0),10,0)|0);
       $446 = tempRet0;
       $447 = ($439>>>0)>(9);
       $448 = ($439|0)==(9);
       $449 = ($438>>>0)>(4294967295);
       $450 = $448 & $449;
       $451 = $447 | $450;
       if ($451) {
        $$05$i$i = $444;$438 = $445;$439 = $446;
       } else {
        break;
       }
      }
      $$0$lcssa$i49$i = $444;$$01$lcssa$off0$i$i = $445;
     } else {
      $$0$lcssa$i49$i = $6;$$01$lcssa$off0$i$i = $434;
     }
     $452 = ($$01$lcssa$off0$i$i|0)==(0);
     if ($452) {
      $$1$lcssa$i$i = $$0$lcssa$i49$i;
     } else {
      $$12$i$i = $$0$lcssa$i49$i;$y$03$i$i = $$01$lcssa$off0$i$i;
      while(1) {
       $453 = (($y$03$i$i>>>0) % 10)&-1;
       $454 = $453 | 48;
       $455 = $454&255;
       $456 = (($$12$i$i) + -1|0);
       HEAP8[$456>>0] = $455;
       $457 = (($y$03$i$i>>>0) / 10)&-1;
       $458 = ($y$03$i$i>>>0)<(10);
       if ($458) {
        $$1$lcssa$i$i = $456;
        break;
       } else {
        $$12$i$i = $456;$y$03$i$i = $457;
       }
      }
     }
     $459 = ($$1$lcssa$i$i|0)==($6|0);
     if ($459) {
      HEAP8[$7>>0] = 48;
      $estr$0$i = $7;
     } else {
      $estr$0$i = $$1$lcssa$i$i;
     }
     $460 = HEAP32[$e2$i>>2]|0;
     $461 = $460 >> 31;
     $462 = $461 & 2;
     $463 = (($462) + 43)|0;
     $464 = $463&255;
     $465 = (($estr$0$i) + -1|0);
     HEAP8[$465>>0] = $464;
     $466 = (($t$0) + 15)|0;
     $467 = $466&255;
     $468 = (($estr$0$i) + -2|0);
     HEAP8[$468>>0] = $467;
     $469 = ($p$0|0)>(0);
     $470 = $fl$1$ & 8;
     $471 = ($470|0)==(0);
     if ($469) {
      if ($471) {
       $$2$us$us$i = $$1$i;$s$0$us$us$i = $buf$i;
       while(1) {
        $472 = (~~(($$2$us$us$i)));
        $473 = (11744 + ($472)|0);
        $474 = HEAP8[$473>>0]|0;
        $475 = $474&255;
        $476 = $475 | $413;
        $477 = $476&255;
        $478 = (($s$0$us$us$i) + 1|0);
        HEAP8[$s$0$us$us$i>>0] = $477;
        $479 = (+($472|0));
        $480 = $$2$us$us$i - $479;
        $481 = $480 * 16.0;
        $482 = $478;
        $483 = (($482) - ($0))|0;
        $484 = ($483|0)==(1);
        if ($484) {
         $485 = (($s$0$us$us$i) + 2|0);
         HEAP8[$478>>0] = 46;
         $s$1$us$us$i = $485;
        } else {
         $s$1$us$us$i = $478;
        }
        $486 = $481 != 0.0;
        if ($486) {
         $$2$us$us$i = $481;$s$0$us$us$i = $s$1$us$us$i;
        } else {
         $s$1$lcssa$i = $s$1$us$us$i;
         break;
        }
       }
      } else {
       $$2$us$i = $$1$i;$s$0$us$i = $buf$i;
       while(1) {
        $487 = (~~(($$2$us$i)));
        $488 = (11744 + ($487)|0);
        $489 = HEAP8[$488>>0]|0;
        $490 = $489&255;
        $491 = $490 | $413;
        $492 = $491&255;
        $493 = (($s$0$us$i) + 1|0);
        HEAP8[$s$0$us$i>>0] = $492;
        $494 = (+($487|0));
        $495 = $$2$us$i - $494;
        $496 = $495 * 16.0;
        $497 = $493;
        $498 = (($497) - ($0))|0;
        $499 = ($498|0)==(1);
        if ($499) {
         $500 = (($s$0$us$i) + 2|0);
         HEAP8[$493>>0] = 46;
         $s$1$us$i = $500;
        } else {
         $s$1$us$i = $493;
        }
        $501 = $496 != 0.0;
        if ($501) {
         $$2$us$i = $496;$s$0$us$i = $s$1$us$i;
        } else {
         $s$1$lcssa$i = $s$1$us$i;
         break;
        }
       }
      }
     } else {
      if ($471) {
       $$2$us160$i = $$1$i;$s$0$us159$i = $buf$i;
       while(1) {
        $502 = (~~(($$2$us160$i)));
        $503 = (11744 + ($502)|0);
        $504 = HEAP8[$503>>0]|0;
        $505 = $504&255;
        $506 = $505 | $413;
        $507 = $506&255;
        $508 = (($s$0$us159$i) + 1|0);
        HEAP8[$s$0$us159$i>>0] = $507;
        $509 = (+($502|0));
        $510 = $$2$us160$i - $509;
        $511 = $510 * 16.0;
        $512 = $508;
        $513 = (($512) - ($0))|0;
        $514 = ($513|0)==(1);
        $515 = $511 != 0.0;
        $or$cond$i72 = $514 & $515;
        if ($or$cond$i72) {
         $516 = (($s$0$us159$i) + 2|0);
         HEAP8[$508>>0] = 46;
         $s$1$us161$i = $516;
        } else {
         $s$1$us161$i = $508;
        }
        if ($515) {
         $$2$us160$i = $511;$s$0$us159$i = $s$1$us161$i;
        } else {
         $s$1$lcssa$i = $s$1$us161$i;
         break;
        }
       }
      } else {
       $$2$i = $$1$i;$s$0$i = $buf$i;
       while(1) {
        $517 = (~~(($$2$i)));
        $518 = (11744 + ($517)|0);
        $519 = HEAP8[$518>>0]|0;
        $520 = $519&255;
        $521 = $520 | $413;
        $522 = $521&255;
        $523 = (($s$0$i) + 1|0);
        HEAP8[$s$0$i>>0] = $522;
        $524 = (+($517|0));
        $525 = $$2$i - $524;
        $526 = $525 * 16.0;
        $527 = $523;
        $528 = (($527) - ($0))|0;
        $529 = ($528|0)==(1);
        if ($529) {
         $530 = (($s$0$i) + 2|0);
         HEAP8[$523>>0] = 46;
         $s$1$i = $530;
        } else {
         $s$1$i = $523;
        }
        $531 = $526 != 0.0;
        if ($531) {
         $$2$i = $526;$s$0$i = $s$1$i;
        } else {
         $s$1$lcssa$i = $s$1$i;
         break;
        }
       }
      }
     }
     $532 = ($p$0|0)==(0);
     $$pre312$i = $s$1$lcssa$i;
     do {
      if ($532) {
       label = 174;
      } else {
       $533 = (($10) + ($$pre312$i))|0;
       $534 = ($533|0)<($p$0|0);
       if (!($534)) {
        label = 174;
        break;
       }
       $535 = $468;
       $536 = (($11) + ($p$0))|0;
       $537 = (($536) - ($535))|0;
       $l$0$i = $537;
      }
     } while(0);
     if ((label|0) == 174) {
      label = 0;
      $538 = $468;
      $539 = (($9) - ($538))|0;
      $540 = (($539) + ($$pre312$i))|0;
      $l$0$i = $540;
     }
     $541 = (($l$0$i) + ($416))|0;
     $542 = $fl$1$ & 73728;
     $543 = ($542|0)==(0);
     $544 = ($541|0)<($w$1|0);
     $or$cond$i53$i = $543 & $544;
     if ($or$cond$i53$i) {
      $545 = (($w$1) - ($541))|0;
      $546 = ($545>>>0)>(256);
      $547 = $546 ? 256 : $545;
      _memset(($pad$i|0),32,($547|0))|0;
      $548 = ($545>>>0)>(255);
      if ($548) {
       $$01$i54$i = $545;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $549 = (($$01$i54$i) + -256)|0;
        $550 = ($549>>>0)>(255);
        if ($550) {
         $$01$i54$i = $549;
        } else {
         $$0$lcssa$i56$i = $549;
         break;
        }
       }
      } else {
       $$0$lcssa$i56$i = $545;
      }
      (___fwritex($pad$i,$$0$lcssa$i56$i,$f)|0);
     }
     (___fwritex($prefix$0$$i,$416,$f)|0);
     $551 = ($542|0)==(65536);
     $or$cond$i60$i = $551 & $544;
     if ($or$cond$i60$i) {
      $552 = (($w$1) - ($541))|0;
      $553 = ($552>>>0)>(256);
      $554 = $553 ? 256 : $552;
      _memset(($pad$i|0),48,($554|0))|0;
      $555 = ($552>>>0)>(255);
      if ($555) {
       $$01$i61$i = $552;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $556 = (($$01$i61$i) + -256)|0;
        $557 = ($556>>>0)>(255);
        if ($557) {
         $$01$i61$i = $556;
        } else {
         $$0$lcssa$i63$i = $556;
         break;
        }
       }
      } else {
       $$0$lcssa$i63$i = $552;
      }
      (___fwritex($pad$i,$$0$lcssa$i63$i,$f)|0);
     }
     $558 = (($$pre312$i) - ($0))|0;
     (___fwritex($buf$i,$558,$f)|0);
     $559 = $468;
     $560 = (($8) - ($559))|0;
     $561 = (($l$0$i) - ($560))|0;
     $562 = (($561) - ($558))|0;
     $563 = ($562|0)>(0);
     if ($563) {
      $564 = ($562>>>0)>(256);
      $565 = $564 ? 256 : $562;
      _memset(($pad$i|0),48,($565|0))|0;
      $566 = ($562>>>0)>(255);
      if ($566) {
       $$01$i67$i = $562;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $567 = (($$01$i67$i) + -256)|0;
        $568 = ($567>>>0)>(255);
        if ($568) {
         $$01$i67$i = $567;
        } else {
         $$0$lcssa$i69$i = $567;
         break;
        }
       }
      } else {
       $$0$lcssa$i69$i = $562;
      }
      (___fwritex($pad$i,$$0$lcssa$i69$i,$f)|0);
     }
     (___fwritex($468,$560,$f)|0);
     $569 = ($542|0)==(8192);
     $or$cond$i73$i = $569 & $544;
     if ($or$cond$i73$i) {
      $570 = (($w$1) - ($541))|0;
      $571 = ($570>>>0)>(256);
      $572 = $571 ? 256 : $570;
      _memset(($pad$i|0),32,($572|0))|0;
      $573 = ($570>>>0)>(255);
      if ($573) {
       $$01$i74$i = $570;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $574 = (($$01$i74$i) + -256)|0;
        $575 = ($574>>>0)>(255);
        if ($575) {
         $$01$i74$i = $574;
        } else {
         $$0$lcssa$i76$i = $574;
         break;
        }
       }
      } else {
       $$0$lcssa$i76$i = $570;
      }
      (___fwritex($pad$i,$$0$lcssa$i76$i,$f)|0);
     }
     $w$18$i = $544 ? $w$1 : $541;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$18$i;$l10n$0 = $l10n$3;
     continue L1;
    }
    $576 = ($p$0|0)<(0);
    $$p$i = $576 ? 6 : $p$0;
    if ($408) {
     $577 = $407 * 268435456.0;
     $578 = HEAP32[$e2$i>>2]|0;
     $579 = (($578) + -28)|0;
     HEAP32[$e2$i>>2] = $579;
     $$3$i = $577;$581 = $579;
    } else {
     $$pre$i = HEAP32[$e2$i>>2]|0;
     $$3$i = $407;$581 = $$pre$i;
    }
    $580 = ($581|0)<(0);
    $$32$i = $580 ? $big$i : $12;
    $$4$i = $$3$i;$z$0$i = $$32$i;
    while(1) {
     $582 = (~~(($$4$i))>>>0);
     HEAP32[$z$0$i>>2] = $582;
     $583 = (($z$0$i) + 4|0);
     $584 = (+($582>>>0));
     $585 = $$4$i - $584;
     $586 = $585 * 1.0E+9;
     $587 = $586 != 0.0;
     if ($587) {
      $$4$i = $586;$z$0$i = $583;
     } else {
      break;
     }
    }
    $$pr$i = HEAP32[$e2$i>>2]|0;
    $588 = ($$pr$i|0)>(0);
    if ($588) {
     $590 = $$pr$i;$a$1257$i = $$32$i;$z$1256$i = $583;
     while(1) {
      $589 = ($590|0)>(29);
      $591 = $589 ? 29 : $590;
      $592 = (($z$1256$i) + -4|0);
      $593 = ($592>>>0)<($a$1257$i>>>0);
      do {
       if ($593) {
        $594 = HEAP32[$592>>2]|0;
        $595 = ($594|0)==(0);
        $596 = ($z$1256$i>>>0)>($a$1257$i>>>0);
        $or$cond19315$i = $595 & $596;
        $z$2316$i = $or$cond19315$i ? $592 : $z$1256$i;
        $a$2$i = $a$1257$i;$z$2317$i = $z$2316$i;
       } else {
        $carry$0250$i = 0;$d$0251$i = $592;
        while(1) {
         $597 = HEAP32[$d$0251$i>>2]|0;
         $598 = (_bitshift64Shl(($597|0),0,($591|0))|0);
         $599 = tempRet0;
         $600 = (_i64Add(($598|0),($599|0),($carry$0250$i|0),0)|0);
         $601 = tempRet0;
         $602 = (___uremdi3(($600|0),($601|0),1000000000,0)|0);
         $603 = tempRet0;
         HEAP32[$d$0251$i>>2] = $602;
         $604 = (___udivdi3(($600|0),($601|0),1000000000,0)|0);
         $605 = tempRet0;
         $606 = (($d$0251$i) + -4|0);
         $607 = ($606>>>0)<($a$1257$i>>>0);
         if ($607) {
          break;
         } else {
          $carry$0250$i = $604;$d$0251$i = $606;
         }
        }
        $608 = HEAP32[$592>>2]|0;
        $609 = ($608|0)==(0);
        $610 = ($z$1256$i>>>0)>($a$1257$i>>>0);
        $or$cond19$i = $609 & $610;
        $z$2$i = $or$cond19$i ? $592 : $z$1256$i;
        $611 = ($604|0)==(0);
        if ($611) {
         $a$2$i = $a$1257$i;$z$2317$i = $z$2$i;
         break;
        }
        $612 = (($a$1257$i) + -4|0);
        HEAP32[$612>>2] = $604;
        $a$2$i = $612;$z$2317$i = $z$2$i;
       }
      } while(0);
      $613 = HEAP32[$e2$i>>2]|0;
      $614 = (($613) - ($591))|0;
      HEAP32[$e2$i>>2] = $614;
      $615 = ($614|0)>(0);
      if ($615) {
       $590 = $614;$a$1257$i = $a$2$i;$z$1256$i = $z$2317$i;
      } else {
       $$pr147$i = $614;$a$1$lcssa$i = $a$2$i;$z$1$lcssa$i = $z$2317$i;
       break;
      }
     }
    } else {
     $$pr147$i = $$pr$i;$a$1$lcssa$i = $$32$i;$z$1$lcssa$i = $583;
    }
    $616 = ($$pr147$i|0)<(0);
    L225: do {
     if ($616) {
      $617 = ($411|0)==(102);
      $618 = (($$p$i|0) / 9)&-1;
      $619 = (($618) + 2)|0;
      if ($617) {
       $620 = $$32$i;
       $621 = (($$32$i) + ($619<<2)|0);
       $623 = $$pr147$i;$a$3245$us$i = $a$1$lcssa$i;$z$3244$us$i = $z$1$lcssa$i;
       while(1) {
        $622 = (0 - ($623))|0;
        $624 = ($622|0)>(9);
        $$20$us$i = $624 ? 9 : $622;
        $625 = ($a$3245$us$i>>>0)<($z$3244$us$i>>>0);
        do {
         if ($625) {
          $650 = 1 << $$20$us$i;
          $644 = (($650) + -1)|0;
          $647 = 1000000000 >>> $$20$us$i;
          $carry3$0239$us$i = 0;$d$1238$us$i = $a$3245$us$i;
          while(1) {
           $642 = HEAP32[$d$1238$us$i>>2]|0;
           $643 = $642 & $644;
           $645 = $642 >>> $$20$us$i;
           $646 = (($645) + ($carry3$0239$us$i))|0;
           HEAP32[$d$1238$us$i>>2] = $646;
           $633 = Math_imul($643, $647)|0;
           $648 = (($d$1238$us$i) + 4|0);
           $649 = ($648>>>0)<($z$3244$us$i>>>0);
           if ($649) {
            $carry3$0239$us$i = $633;$d$1238$us$i = $648;
           } else {
            break;
           }
          }
          $629 = HEAP32[$a$3245$us$i>>2]|0;
          $630 = ($629|0)==(0);
          $631 = (($a$3245$us$i) + 4|0);
          $$a$3$us$i = $630 ? $631 : $a$3245$us$i;
          $632 = ($633|0)==(0);
          if ($632) {
           $$a$3$us320$i = $$a$3$us$i;$z$4$us$i = $z$3244$us$i;
           break;
          }
          $634 = (($z$3244$us$i) + 4|0);
          HEAP32[$z$3244$us$i>>2] = $633;
          $$a$3$us320$i = $$a$3$us$i;$z$4$us$i = $634;
         } else {
          $626 = HEAP32[$a$3245$us$i>>2]|0;
          $627 = ($626|0)==(0);
          $628 = (($a$3245$us$i) + 4|0);
          $$a$3$us319$i = $627 ? $628 : $a$3245$us$i;
          $$a$3$us320$i = $$a$3$us319$i;$z$4$us$i = $z$3244$us$i;
         }
        } while(0);
        $635 = $z$4$us$i;
        $636 = (($635) - ($620))|0;
        $637 = $636 >> 2;
        $638 = ($637|0)>($619|0);
        $$z$4$us$i = $638 ? $621 : $z$4$us$i;
        $639 = HEAP32[$e2$i>>2]|0;
        $640 = (($639) + ($$20$us$i))|0;
        HEAP32[$e2$i>>2] = $640;
        $641 = ($640|0)<(0);
        if ($641) {
         $623 = $640;$a$3245$us$i = $$a$3$us320$i;$z$3244$us$i = $$z$4$us$i;
        } else {
         $a$3$lcssa$i = $$a$3$us320$i;$z$3$lcssa$i = $$z$4$us$i;
         break L225;
        }
       }
      } else {
       $652 = $$pr147$i;$a$3245$i = $a$1$lcssa$i;$z$3244$i = $z$1$lcssa$i;
      }
      while(1) {
       $651 = (0 - ($652))|0;
       $653 = ($651|0)>(9);
       $$20$i = $653 ? 9 : $651;
       $654 = ($a$3245$i>>>0)<($z$3244$i>>>0);
       do {
        if ($654) {
         $658 = 1 << $$20$i;
         $659 = (($658) + -1)|0;
         $660 = 1000000000 >>> $$20$i;
         $carry3$0239$i = 0;$d$1238$i = $a$3245$i;
         while(1) {
          $661 = HEAP32[$d$1238$i>>2]|0;
          $662 = $661 & $659;
          $663 = $661 >>> $$20$i;
          $664 = (($663) + ($carry3$0239$i))|0;
          HEAP32[$d$1238$i>>2] = $664;
          $665 = Math_imul($662, $660)|0;
          $666 = (($d$1238$i) + 4|0);
          $667 = ($666>>>0)<($z$3244$i>>>0);
          if ($667) {
           $carry3$0239$i = $665;$d$1238$i = $666;
          } else {
           break;
          }
         }
         $668 = HEAP32[$a$3245$i>>2]|0;
         $669 = ($668|0)==(0);
         $670 = (($a$3245$i) + 4|0);
         $$a$3$i = $669 ? $670 : $a$3245$i;
         $671 = ($665|0)==(0);
         if ($671) {
          $$a$3322$i = $$a$3$i;$z$4$i = $z$3244$i;
          break;
         }
         $672 = (($z$3244$i) + 4|0);
         HEAP32[$z$3244$i>>2] = $665;
         $$a$3322$i = $$a$3$i;$z$4$i = $672;
        } else {
         $655 = HEAP32[$a$3245$i>>2]|0;
         $656 = ($655|0)==(0);
         $657 = (($a$3245$i) + 4|0);
         $$a$3321$i = $656 ? $657 : $a$3245$i;
         $$a$3322$i = $$a$3321$i;$z$4$i = $z$3244$i;
        }
       } while(0);
       $673 = $z$4$i;
       $674 = $$a$3322$i;
       $675 = (($673) - ($674))|0;
       $676 = $675 >> 2;
       $677 = ($676|0)>($619|0);
       if ($677) {
        $678 = (($$a$3322$i) + ($619<<2)|0);
        $z$5$i = $678;
       } else {
        $z$5$i = $z$4$i;
       }
       $679 = HEAP32[$e2$i>>2]|0;
       $680 = (($679) + ($$20$i))|0;
       HEAP32[$e2$i>>2] = $680;
       $681 = ($680|0)<(0);
       if ($681) {
        $652 = $680;$a$3245$i = $$a$3322$i;$z$3244$i = $z$5$i;
       } else {
        $a$3$lcssa$i = $$a$3322$i;$z$3$lcssa$i = $z$5$i;
        break;
       }
      }
     } else {
      $a$3$lcssa$i = $a$1$lcssa$i;$z$3$lcssa$i = $z$1$lcssa$i;
     }
    } while(0);
    $682 = ($a$3$lcssa$i>>>0)<($z$3$lcssa$i>>>0);
    $683 = $$32$i;
    do {
     if ($682) {
      $684 = $a$3$lcssa$i;
      $685 = (($683) - ($684))|0;
      $686 = $685 >> 2;
      $687 = ($686*9)|0;
      $688 = HEAP32[$a$3$lcssa$i>>2]|0;
      $689 = ($688>>>0)<(10);
      if ($689) {
       $e$1$i = $687;
       break;
      } else {
       $e$0233$i = $687;$i$0234$i = 10;
      }
      while(1) {
       $690 = ($i$0234$i*10)|0;
       $691 = (($e$0233$i) + 1)|0;
       $692 = ($688>>>0)<($690>>>0);
       if ($692) {
        $e$1$i = $691;
        break;
       } else {
        $e$0233$i = $691;$i$0234$i = $690;
       }
      }
     } else {
      $e$1$i = 0;
     }
    } while(0);
    $693 = ($411|0)!=(102);
    $694 = $693 ? $e$1$i : 0;
    $695 = (($$p$i) - ($694))|0;
    $696 = ($411|0)==(103);
    $697 = ($$p$i|0)!=(0);
    $$21$i = $696 & $697;
    $$neg152$i = $$21$i << 31 >> 31;
    $698 = (($695) + ($$neg152$i))|0;
    $699 = $z$3$lcssa$i;
    $700 = (($699) - ($683))|0;
    $701 = $700 >> 2;
    $702 = ($701*9)|0;
    $703 = (($702) + -9)|0;
    $704 = ($698|0)<($703|0);
    if ($704) {
     $705 = (($698) + 9216)|0;
     $706 = (($705|0) / 9)&-1;
     $$sum$i = (($706) + -1023)|0;
     $707 = (($$32$i) + ($$sum$i<<2)|0);
     $708 = (($705|0) % 9)&-1;
     $j$0225$i = (($708) + 1)|0;
     $709 = ($j$0225$i|0)<(9);
     if ($709) {
      $i$1226$i = 10;$j$0227$i = $j$0225$i;
      while(1) {
       $710 = ($i$1226$i*10)|0;
       $j$0$i = (($j$0227$i) + 1)|0;
       $exitcond$i = ($j$0$i|0)==(9);
       if ($exitcond$i) {
        $i$1$lcssa$i = $710;
        break;
       } else {
        $i$1226$i = $710;$j$0227$i = $j$0$i;
       }
      }
     } else {
      $i$1$lcssa$i = 10;
     }
     $711 = HEAP32[$707>>2]|0;
     $712 = (($711>>>0) % ($i$1$lcssa$i>>>0))&-1;
     $713 = ($712|0)==(0);
     if ($713) {
      $$sum14$i = (($706) + -1022)|0;
      $714 = (($$32$i) + ($$sum14$i<<2)|0);
      $715 = ($714|0)==($z$3$lcssa$i|0);
      if ($715) {
       $a$6$i = $a$3$lcssa$i;$d$3$i = $707;$e$3$i = $e$1$i;
      } else {
       label = 232;
      }
     } else {
      label = 232;
     }
     do {
      if ((label|0) == 232) {
       label = 0;
       $716 = (($711>>>0) / ($i$1$lcssa$i>>>0))&-1;
       $717 = $716 & 1;
       $718 = ($717|0)==(0);
       $$22$i = $718 ? 9007199254740992.0 : 9007199254740994.0;
       $719 = (($i$1$lcssa$i|0) / 2)&-1;
       $720 = ($712>>>0)<($719>>>0);
       do {
        if ($720) {
         $small$0$i = 0.5;
        } else {
         $721 = ($712|0)==($719|0);
         if ($721) {
          $$sum15$i = (($706) + -1022)|0;
          $722 = (($$32$i) + ($$sum15$i<<2)|0);
          $723 = ($722|0)==($z$3$lcssa$i|0);
          if ($723) {
           $small$0$i = 1.0;
           break;
          }
         }
         $small$0$i = 1.5;
        }
       } while(0);
       $724 = ($pl$0$i|0)==(0);
       do {
        if ($724) {
         $round6$1$i = $$22$i;$small$1$i = $small$0$i;
        } else {
         $725 = HEAP8[$prefix$0$i>>0]|0;
         $726 = ($725<<24>>24)==(45);
         if (!($726)) {
          $round6$1$i = $$22$i;$small$1$i = $small$0$i;
          break;
         }
         $727 = $$22$i * -1.0;
         $728 = $small$0$i * -1.0;
         $round6$1$i = $727;$small$1$i = $728;
        }
       } while(0);
       $729 = (($711) - ($712))|0;
       HEAP32[$707>>2] = $729;
       $730 = $round6$1$i + $small$1$i;
       $731 = $730 != $round6$1$i;
       if (!($731)) {
        $a$6$i = $a$3$lcssa$i;$d$3$i = $707;$e$3$i = $e$1$i;
        break;
       }
       $732 = (($729) + ($i$1$lcssa$i))|0;
       HEAP32[$707>>2] = $732;
       $733 = ($732>>>0)>(999999999);
       if ($733) {
        $d$2220$i = $707;
        while(1) {
         $734 = (($d$2220$i) + -4|0);
         HEAP32[$d$2220$i>>2] = 0;
         $735 = HEAP32[$734>>2]|0;
         $736 = (($735) + 1)|0;
         HEAP32[$734>>2] = $736;
         $737 = ($736>>>0)>(999999999);
         if ($737) {
          $d$2220$i = $734;
         } else {
          $d$2$lcssa$i = $734;
          break;
         }
        }
       } else {
        $d$2$lcssa$i = $707;
       }
       $738 = ($d$2$lcssa$i>>>0)<($a$3$lcssa$i>>>0);
       $d$2$a$3$i = $738 ? $d$2$lcssa$i : $a$3$lcssa$i;
       $739 = $d$2$a$3$i;
       $740 = (($683) - ($739))|0;
       $741 = $740 >> 2;
       $742 = ($741*9)|0;
       $743 = HEAP32[$d$2$a$3$i>>2]|0;
       $744 = ($743>>>0)<(10);
       if ($744) {
        $a$6$i = $d$2$a$3$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $742;
        break;
       } else {
        $e$2215$i = $742;$i$2216$i = 10;
       }
       while(1) {
        $745 = ($i$2216$i*10)|0;
        $746 = (($e$2215$i) + 1)|0;
        $747 = ($743>>>0)<($745>>>0);
        if ($747) {
         $a$6$i = $d$2$a$3$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $746;
         break;
        } else {
         $e$2215$i = $746;$i$2216$i = $745;
        }
       }
      }
     } while(0);
     $748 = (($d$3$i) + 4|0);
     $749 = ($z$3$lcssa$i>>>0)>($748>>>0);
     $$z$3$i = $749 ? $748 : $z$3$lcssa$i;
     $z$6$i = $$z$3$i;
     while(1) {
      $750 = (($z$6$i) + -4|0);
      $751 = HEAP32[$750>>2]|0;
      $752 = ($751|0)==(0);
      $753 = ($z$6$i>>>0)>($a$6$i>>>0);
      $or$cond23$i = $752 & $753;
      if ($or$cond23$i) {
       $z$6$i = $750;
      } else {
       $a$7$i = $a$6$i;$e$4$i = $e$3$i;$z$7$i = $z$6$i;
       break;
      }
     }
    } else {
     $a$7$i = $a$3$lcssa$i;$e$4$i = $e$1$i;$z$7$i = $z$3$lcssa$i;
    }
    $754 = (0 - ($e$4$i))|0;
    do {
     if ($696) {
      $755 = ($$p$i|0)==(0);
      $756 = $755&1;
      $$$p$i = (($756) + ($$p$i))|0;
      $757 = ($$$p$i|0)>($e$4$i|0);
      $758 = ($e$4$i|0)>(-5);
      $or$cond3$i = $757 & $758;
      if ($or$cond3$i) {
       $759 = (($t$0) + -1)|0;
       $$neg153$i = (($$$p$i) + -1)|0;
       $760 = (($$neg153$i) - ($e$4$i))|0;
       $$012$i73 = $759;$$29$i = $760;
      } else {
       $761 = (($t$0) + -2)|0;
       $762 = (($$$p$i) + -1)|0;
       $$012$i73 = $761;$$29$i = $762;
      }
      $763 = $fl$1$ & 8;
      $764 = ($763|0)==(0);
      if (!($764)) {
       $$113$i = $$012$i73;$$310$i = $$29$i;
       break;
      }
      $765 = ($z$7$i>>>0)>($a$7$i>>>0);
      do {
       if ($765) {
        $766 = (($z$7$i) + -4|0);
        $767 = HEAP32[$766>>2]|0;
        $768 = ($767|0)==(0);
        if ($768) {
         $j$2$i = 9;
         break;
        }
        $769 = (($767>>>0) % 10)&-1;
        $770 = ($769|0)==(0);
        if ($770) {
         $i$3210$i = 10;$j$1211$i = 0;
        } else {
         $j$2$i = 0;
         break;
        }
        while(1) {
         $771 = ($i$3210$i*10)|0;
         $772 = (($j$1211$i) + 1)|0;
         $773 = (($767>>>0) % ($771>>>0))&-1;
         $774 = ($773|0)==(0);
         if ($774) {
          $i$3210$i = $771;$j$1211$i = $772;
         } else {
          $j$2$i = $772;
          break;
         }
        }
       } else {
        $j$2$i = 9;
       }
      } while(0);
      $775 = $$012$i73 | 32;
      $776 = ($775|0)==(102);
      $777 = $z$7$i;
      $778 = (($777) - ($683))|0;
      $779 = $778 >> 2;
      $780 = ($779*9)|0;
      $781 = (($780) + -9)|0;
      if ($776) {
       $782 = (($781) - ($j$2$i))|0;
       $783 = ($782|0)<(0);
       $$24$i = $783 ? 0 : $782;
       $784 = ($$29$i|0)<($$24$i|0);
       $$29$$24$i = $784 ? $$29$i : $$24$i;
       $$113$i = $$012$i73;$$310$i = $$29$$24$i;
       break;
      } else {
       $785 = (($781) + ($e$4$i))|0;
       $786 = (($785) - ($j$2$i))|0;
       $787 = ($786|0)<(0);
       $$26$i = $787 ? 0 : $786;
       $788 = ($$29$i|0)<($$26$i|0);
       $$29$$26$i = $788 ? $$29$i : $$26$i;
       $$113$i = $$012$i73;$$310$i = $$29$$26$i;
       break;
      }
     } else {
      $$113$i = $t$0;$$310$i = $$p$i;
     }
    } while(0);
    $789 = ($$310$i|0)!=(0);
    if ($789) {
     $793 = 1;
    } else {
     $790 = $fl$1$ & 8;
     $791 = ($790|0)!=(0);
     $793 = $791;
    }
    $792 = $793&1;
    $794 = $$113$i | 32;
    $795 = ($794|0)==(102);
    if ($795) {
     $796 = ($e$4$i|0)>(0);
     $797 = $796 ? $e$4$i : 0;
     $$pn$i = $797;$estr$2$i = 0;
    } else {
     $798 = ($e$4$i|0)<(0);
     $799 = $798 ? $754 : $e$4$i;
     $800 = ($799|0)<(0);
     if ($800) {
      $801 = ($799|0)<(0);
      $802 = $801 << 31 >> 31;
      $$05$i80$i = $6;$803 = $799;$804 = $802;
      while(1) {
       $805 = (___uremdi3(($803|0),($804|0),10,0)|0);
       $806 = tempRet0;
       $807 = $805 | 48;
       $808 = $807&255;
       $809 = (($$05$i80$i) + -1|0);
       HEAP8[$809>>0] = $808;
       $810 = (___udivdi3(($803|0),($804|0),10,0)|0);
       $811 = tempRet0;
       $812 = ($804>>>0)>(9);
       $813 = ($804|0)==(9);
       $814 = ($803>>>0)>(4294967295);
       $815 = $813 & $814;
       $816 = $812 | $815;
       if ($816) {
        $$05$i80$i = $809;$803 = $810;$804 = $811;
       } else {
        break;
       }
      }
      $$0$lcssa$i85$i = $809;$$01$lcssa$off0$i86$i = $810;
     } else {
      $$0$lcssa$i85$i = $6;$$01$lcssa$off0$i86$i = $799;
     }
     $817 = ($$01$lcssa$off0$i86$i|0)==(0);
     if ($817) {
      $estr$1$ph$i = $$0$lcssa$i85$i;
     } else {
      $$12$i88$i = $$0$lcssa$i85$i;$y$03$i87$i = $$01$lcssa$off0$i86$i;
      while(1) {
       $818 = (($y$03$i87$i>>>0) % 10)&-1;
       $819 = $818 | 48;
       $820 = $819&255;
       $821 = (($$12$i88$i) + -1|0);
       HEAP8[$821>>0] = $820;
       $822 = (($y$03$i87$i>>>0) / 10)&-1;
       $823 = ($y$03$i87$i>>>0)<(10);
       if ($823) {
        $estr$1$ph$i = $821;
        break;
       } else {
        $$12$i88$i = $821;$y$03$i87$i = $822;
       }
      }
     }
     $824 = $estr$1$ph$i;
     $825 = (($8) - ($824))|0;
     $826 = ($825|0)<(2);
     if ($826) {
      $estr$1201$i = $estr$1$ph$i;
      while(1) {
       $827 = (($estr$1201$i) + -1|0);
       HEAP8[$827>>0] = 48;
       $828 = $827;
       $829 = (($8) - ($828))|0;
       $830 = ($829|0)<(2);
       if ($830) {
        $estr$1201$i = $827;
       } else {
        $estr$1$lcssa$i = $827;
        break;
       }
      }
     } else {
      $estr$1$lcssa$i = $estr$1$ph$i;
     }
     $831 = $e$4$i >> 31;
     $832 = $831 & 2;
     $833 = (($832) + 43)|0;
     $834 = $833&255;
     $835 = (($estr$1$lcssa$i) + -1|0);
     HEAP8[$835>>0] = $834;
     $836 = $$113$i&255;
     $837 = (($estr$1$lcssa$i) + -2|0);
     HEAP8[$837>>0] = $836;
     $838 = $837;
     $839 = (($8) - ($838))|0;
     $$pn$i = $839;$estr$2$i = $837;
    }
    $840 = (($pl$0$i) + 1)|0;
    $841 = (($840) + ($$310$i))|0;
    $l$1$i = (($841) + ($792))|0;
    $842 = (($l$1$i) + ($$pn$i))|0;
    $843 = $fl$1$ & 73728;
    $844 = ($843|0)==(0);
    $845 = ($842|0)<($w$1|0);
    $or$cond$i94$i = $844 & $845;
    if ($or$cond$i94$i) {
     $846 = (($w$1) - ($842))|0;
     $847 = ($846>>>0)>(256);
     $848 = $847 ? 256 : $846;
     _memset(($pad$i|0),32,($848|0))|0;
     $849 = ($846>>>0)>(255);
     if ($849) {
      $$01$i95$i = $846;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $850 = (($$01$i95$i) + -256)|0;
       $851 = ($850>>>0)>(255);
       if ($851) {
        $$01$i95$i = $850;
       } else {
        $$0$lcssa$i97$i = $850;
        break;
       }
      }
     } else {
      $$0$lcssa$i97$i = $846;
     }
     (___fwritex($pad$i,$$0$lcssa$i97$i,$f)|0);
    }
    (___fwritex($prefix$0$i,$pl$0$i,$f)|0);
    $852 = ($843|0)==(65536);
    $or$cond$i101$i = $852 & $845;
    if ($or$cond$i101$i) {
     $853 = (($w$1) - ($842))|0;
     $854 = ($853>>>0)>(256);
     $855 = $854 ? 256 : $853;
     _memset(($pad$i|0),48,($855|0))|0;
     $856 = ($853>>>0)>(255);
     if ($856) {
      $$01$i102$i = $853;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $857 = (($$01$i102$i) + -256)|0;
       $858 = ($857>>>0)>(255);
       if ($858) {
        $$01$i102$i = $857;
       } else {
        $$0$lcssa$i104$i = $857;
        break;
       }
      }
     } else {
      $$0$lcssa$i104$i = $853;
     }
     (___fwritex($pad$i,$$0$lcssa$i104$i,$f)|0);
    }
    do {
     if ($795) {
      $859 = ($a$7$i>>>0)>($$32$i>>>0);
      $r$0$a$7$i = $859 ? $$32$i : $a$7$i;
      $d$4184$i = $r$0$a$7$i;
      while(1) {
       $860 = HEAP32[$d$4184$i>>2]|0;
       $861 = ($860|0)==(0);
       if ($861) {
        $$1$lcssa$i113$i = $13;
       } else {
        $$12$i111$i = $13;$y$03$i110$i = $860;
        while(1) {
         $862 = (($y$03$i110$i>>>0) % 10)&-1;
         $863 = $862 | 48;
         $864 = $863&255;
         $865 = (($$12$i111$i) + -1|0);
         HEAP8[$865>>0] = $864;
         $866 = (($y$03$i110$i>>>0) / 10)&-1;
         $867 = ($y$03$i110$i>>>0)<(10);
         if ($867) {
          $$1$lcssa$i113$i = $865;
          break;
         } else {
          $$12$i111$i = $865;$y$03$i110$i = $866;
         }
        }
       }
       $868 = ($d$4184$i|0)==($r$0$a$7$i|0);
       do {
        if ($868) {
         $872 = ($$1$lcssa$i113$i|0)==($13|0);
         if (!($872)) {
          $s7$1$i = $$1$lcssa$i113$i;
          break;
         }
         HEAP8[$15>>0] = 48;
         $s7$1$i = $15;
        } else {
         $869 = ($$1$lcssa$i113$i>>>0)>($buf$i>>>0);
         if ($869) {
          $s7$0181$i = $$1$lcssa$i113$i;
         } else {
          $s7$1$i = $$1$lcssa$i113$i;
          break;
         }
         while(1) {
          $870 = (($s7$0181$i) + -1|0);
          HEAP8[$870>>0] = 48;
          $871 = ($870>>>0)>($buf$i>>>0);
          if ($871) {
           $s7$0181$i = $870;
          } else {
           $s7$1$i = $870;
           break;
          }
         }
        }
       } while(0);
       $873 = $s7$1$i;
       $874 = (($14) - ($873))|0;
       (___fwritex($s7$1$i,$874,$f)|0);
       $875 = (($d$4184$i) + 4|0);
       $876 = ($875>>>0)>($$32$i>>>0);
       if ($876) {
        break;
       } else {
        $d$4184$i = $875;
       }
      }
      if (!($789)) {
       $877 = $fl$1$ & 8;
       $878 = ($877|0)==(0);
       if ($878) {
        break;
       }
      }
      (___fwritex(11760,1,$f)|0);
      $879 = ($875>>>0)<($z$7$i>>>0);
      $880 = ($$310$i|0)>(0);
      $or$cond29174$i = $879 & $880;
      if ($or$cond29174$i) {
       $$411176$i = $$310$i;$d$5175$i = $875;
       while(1) {
        $881 = HEAP32[$d$5175$i>>2]|0;
        $882 = ($881|0)==(0);
        if ($882) {
         $s8$0170$i = $13;
         label = 295;
        } else {
         $$12$i120$i = $13;$y$03$i119$i = $881;
         while(1) {
          $883 = (($y$03$i119$i>>>0) % 10)&-1;
          $884 = $883 | 48;
          $885 = $884&255;
          $886 = (($$12$i120$i) + -1|0);
          HEAP8[$886>>0] = $885;
          $887 = (($y$03$i119$i>>>0) / 10)&-1;
          $888 = ($y$03$i119$i>>>0)<(10);
          if ($888) {
           break;
          } else {
           $$12$i120$i = $886;$y$03$i119$i = $887;
          }
         }
         $889 = ($886>>>0)>($buf$i>>>0);
         if ($889) {
          $s8$0170$i = $886;
          label = 295;
         } else {
          $s8$0$lcssa$i = $886;
         }
        }
        if ((label|0) == 295) {
         while(1) {
          label = 0;
          $890 = (($s8$0170$i) + -1|0);
          HEAP8[$890>>0] = 48;
          $891 = ($890>>>0)>($buf$i>>>0);
          if ($891) {
           $s8$0170$i = $890;
           label = 295;
          } else {
           $s8$0$lcssa$i = $890;
           break;
          }
         }
        }
        $892 = ($$411176$i|0)>(9);
        $893 = $892 ? 9 : $$411176$i;
        (___fwritex($s8$0$lcssa$i,$893,$f)|0);
        $894 = (($d$5175$i) + 4|0);
        $895 = (($$411176$i) + -9)|0;
        $896 = ($894>>>0)<($z$7$i>>>0);
        $897 = ($895|0)>(0);
        $or$cond29$i = $896 & $897;
        if ($or$cond29$i) {
         $$411176$i = $895;$d$5175$i = $894;
        } else {
         $$411$lcssa$i = $895;
         break;
        }
       }
      } else {
       $$411$lcssa$i = $$310$i;
      }
      $898 = ($$411$lcssa$i|0)>(0);
      if (!($898)) {
       break;
      }
      $899 = ($$411$lcssa$i>>>0)>(256);
      $900 = $899 ? 256 : $$411$lcssa$i;
      _memset(($pad$i|0),48,($900|0))|0;
      $901 = ($$411$lcssa$i>>>0)>(255);
      if ($901) {
       $$01$i126$i = $$411$lcssa$i;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $902 = (($$01$i126$i) + -256)|0;
        $903 = ($902>>>0)>(255);
        if ($903) {
         $$01$i126$i = $902;
        } else {
         $$0$lcssa$i128$i = $902;
         break;
        }
       }
      } else {
       $$0$lcssa$i128$i = $$411$lcssa$i;
      }
      (___fwritex($pad$i,$$0$lcssa$i128$i,$f)|0);
     } else {
      $904 = ($z$7$i>>>0)>($a$7$i>>>0);
      $905 = (($a$7$i) + 4|0);
      $z$7$$i = $904 ? $z$7$i : $905;
      $906 = ($$310$i|0)>(-1);
      do {
       if ($906) {
        $907 = $fl$1$ & 8;
        $$not$i = ($907|0)!=(0);
        $$5194$i = $$310$i;$d$6193$i = $a$7$i;
        while(1) {
         $908 = HEAP32[$d$6193$i>>2]|0;
         $909 = ($908|0)==(0);
         if ($909) {
          label = 306;
         } else {
          $$12$i135$i = $13;$y$03$i134$i = $908;
          while(1) {
           $910 = (($y$03$i134$i>>>0) % 10)&-1;
           $911 = $910 | 48;
           $912 = $911&255;
           $913 = (($$12$i135$i) + -1|0);
           HEAP8[$913>>0] = $912;
           $914 = (($y$03$i134$i>>>0) / 10)&-1;
           $915 = ($y$03$i134$i>>>0)<(10);
           if ($915) {
            break;
           } else {
            $$12$i135$i = $913;$y$03$i134$i = $914;
           }
          }
          $916 = ($913|0)==($13|0);
          if ($916) {
           label = 306;
          } else {
           $s9$0$i = $913;
          }
         }
         if ((label|0) == 306) {
          label = 0;
          HEAP8[$15>>0] = 48;
          $s9$0$i = $15;
         }
         $917 = ($d$6193$i|0)==($a$7$i|0);
         do {
          if ($917) {
           $921 = (($s9$0$i) + 1|0);
           (___fwritex($s9$0$i,1,$f)|0);
           $922 = ($$5194$i|0)>(0);
           $brmerge$i = $922 | $$not$i;
           if (!($brmerge$i)) {
            $s9$2$i = $921;
            break;
           }
           (___fwritex(11760,1,$f)|0);
           $s9$2$i = $921;
          } else {
           $918 = ($s9$0$i>>>0)>($buf$i>>>0);
           if ($918) {
            $s9$1189$i = $s9$0$i;
           } else {
            $s9$2$i = $s9$0$i;
            break;
           }
           while(1) {
            $919 = (($s9$1189$i) + -1|0);
            HEAP8[$919>>0] = 48;
            $920 = ($919>>>0)>($buf$i>>>0);
            if ($920) {
             $s9$1189$i = $919;
            } else {
             $s9$2$i = $919;
             break;
            }
           }
          }
         } while(0);
         $923 = $s9$2$i;
         $924 = (($14) - ($923))|0;
         $925 = ($924|0)<($$5194$i|0);
         $$$5$i = $925 ? $924 : $$5194$i;
         (___fwritex($s9$2$i,$$$5$i,$f)|0);
         $926 = (($$5194$i) - ($924))|0;
         $927 = (($d$6193$i) + 4|0);
         $928 = ($927>>>0)<($z$7$$i>>>0);
         $929 = ($926|0)>(-1);
         $or$cond30$i = $928 & $929;
         if ($or$cond30$i) {
          $$5194$i = $926;$d$6193$i = $927;
         } else {
          break;
         }
        }
        $930 = ($926|0)>(0);
        if (!($930)) {
         break;
        }
        $931 = ($926>>>0)>(256);
        $932 = $931 ? 256 : $926;
        _memset(($pad$i|0),48,($932|0))|0;
        $933 = ($926>>>0)>(255);
        if ($933) {
         $$01$i141$i = $926;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $934 = (($$01$i141$i) + -256)|0;
          $935 = ($934>>>0)>(255);
          if ($935) {
           $$01$i141$i = $934;
          } else {
           $$0$lcssa$i143$i = $934;
           break;
          }
         }
        } else {
         $$0$lcssa$i143$i = $926;
        }
        (___fwritex($pad$i,$$0$lcssa$i143$i,$f)|0);
       }
      } while(0);
      $936 = $estr$2$i;
      $937 = (($8) - ($936))|0;
      (___fwritex($estr$2$i,$937,$f)|0);
     }
    } while(0);
    $938 = ($843|0)==(8192);
    $or$cond$i$i = $938 & $845;
    if ($or$cond$i$i) {
     $939 = (($w$1) - ($842))|0;
     $940 = ($939>>>0)>(256);
     $941 = $940 ? 256 : $939;
     _memset(($pad$i|0),32,($941|0))|0;
     $942 = ($939>>>0)>(255);
     if ($942) {
      $$01$i$i = $939;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $943 = (($$01$i$i) + -256)|0;
       $944 = ($943>>>0)>(255);
       if ($944) {
        $$01$i$i = $943;
       } else {
        $$0$lcssa$i$i = $943;
        break;
       }
      }
     } else {
      $$0$lcssa$i$i = $939;
     }
     (___fwritex($pad$i,$$0$lcssa$i$i,$f)|0);
    }
    $w$31$i = $845 ? $w$1 : $842;
    $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$31$i;$l10n$0 = $l10n$3;
    continue L1;
    break;
   }
   case 99:  {
    $318 = $206;
    $319 = $318&255;
    HEAP8[$4>>0] = $319;
    $1054 = $228;$1055 = $206;$a$2 = $4;$fl$6 = $205;$p$5 = 1;$pl$2 = 0;$prefix$2 = 11664;$z$2 = $2;
    break;
   }
   case 67:  {
    $330 = $206;
    HEAP32[$wc>>2] = $330;
    HEAP32[$5>>2] = 0;
    $1056 = $wc;$1057 = $wc;$p$4296 = -1;
    label = 105;
    break;
   }
   case 83:  {
    $331 = ($p$0|0)==(0);
    if ($331) {
     $1058 = $206;$1059 = $206;$i$0167 = 0;
     label = 111;
    } else {
     $1056 = $206;$1057 = $206;$p$4296 = $p$0;
     label = 105;
    }
    break;
   }
   case 110:  {
    switch ($st$0|0) {
    case 3:  {
     $213 = $cnt$1&65535;
     HEAP16[$206>>1] = $213;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 4:  {
     $214 = $cnt$1&255;
     HEAP8[$206>>0] = $214;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 6:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 0:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 1:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 7:  {
     $215 = ($cnt$1|0)<(0);
     $216 = $215 << 31 >> 31;
     $217 = $206;
     $218 = $217;
     HEAP32[$218>>2] = $cnt$1;
     $219 = (($217) + 4)|0;
     $220 = $219;
     HEAP32[$220>>2] = $216;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 2:  {
     $207 = ($cnt$1|0)<(0);
     $208 = $207 << 31 >> 31;
     $209 = $206;
     $210 = $209;
     HEAP32[$210>>2] = $cnt$1;
     $211 = (($209) + 4)|0;
     $212 = $211;
     HEAP32[$212>>2] = $208;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    default: {
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
    }
    }
    break;
   }
   case 111:  {
    $248 = $206;
    $249 = ($248|0)==(0);
    $250 = ($228|0)==(0);
    $251 = $249 & $250;
    if ($251) {
     $$0$lcssa$i44 = $2;
    } else {
     $$03$i41 = $2;$253 = $248;$257 = $228;
     while(1) {
      $252 = $253 & 7;
      $254 = $252 | 48;
      $255 = $254&255;
      $256 = (($$03$i41) + -1|0);
      HEAP8[$256>>0] = $255;
      $258 = (_bitshift64Lshr(($253|0),($257|0),3)|0);
      $259 = tempRet0;
      $260 = ($258|0)==(0);
      $261 = ($259|0)==(0);
      $262 = $260 & $261;
      if ($262) {
       $$0$lcssa$i44 = $256;
       break;
      } else {
       $$03$i41 = $256;$253 = $258;$257 = $259;
      }
     }
    }
    $263 = $fl$1$ & 8;
    $264 = ($263|0)==(0);
    if ($264) {
     $307 = $206;$310 = $228;$a$0 = $$0$lcssa$i44;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = 0;$prefix$1 = 11664;
     label = 95;
    } else {
     $$14 = $251 ? 11664 : ((11664 + 5|0));
     $265 = $251&1;
     $$15 = $265 ^ 1;
     $307 = $206;$310 = $228;$a$0 = $$0$lcssa$i44;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $$15;$prefix$1 = $$14;
     label = 95;
    }
    break;
   }
   case 109:  {
    $320 = (___errno_location()|0);
    $321 = HEAP32[$320>>2]|0;
    $322 = (_strerror(($321|0))|0);
    $a$1 = $322;
    label = 100;
    break;
   }
   case 115:  {
    $323 = ($206|0)==(0|0);
    $$18 = $323 ? 11680 : $206;
    $a$1 = $$18;
    label = 100;
    break;
   }
   case 112:  {
    $221 = ($p$0>>>0)>(8);
    $222 = $221 ? $p$0 : 8;
    $223 = $fl$1$ | 8;
    $fl$3 = $223;$p$1 = $222;$t$1 = 120;
    label = 78;
    break;
   }
   case 105: case 100:  {
    $266 = $206;
    $267 = ($228|0)<(0);
    if ($267) {
     $268 = (_i64Subtract(0,0,($266|0),($228|0))|0);
     $269 = tempRet0;
     $270 = $268;
     $276 = $270;$278 = $269;$pl$0 = 1;$prefix$0 = 11664;
     label = 90;
     break L92;
    }
    $271 = $fl$1$ & 2048;
    $272 = ($271|0)==(0);
    if ($272) {
     $273 = $fl$1$ & 1;
     $274 = ($273|0)==(0);
     $$16 = $274 ? 11664 : ((11664 + 2|0));
     $276 = $206;$278 = $228;$pl$0 = $273;$prefix$0 = $$16;
     label = 90;
    } else {
     $276 = $206;$278 = $228;$pl$0 = 1;$prefix$0 = ((11664 + 1|0));
     label = 90;
    }
    break;
   }
   case 88: case 120:  {
    $fl$3 = $fl$1$;$p$1 = $p$0;$t$1 = $t$0;
    label = 78;
    break;
   }
   case 117:  {
    $276 = $206;$278 = $228;$pl$0 = 0;$prefix$0 = 11664;
    label = 90;
    break;
   }
   default: {
    $1054 = $228;$1055 = $206;$a$2 = $22;$fl$6 = $fl$1$;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 11664;$z$2 = $2;
   }
   }
  } while(0);
  L437: do {
   if ((label|0) == 78) {
    label = 0;
    $224 = $206;
    $225 = $t$1 & 32;
    $226 = ($224|0)==(0);
    $227 = ($228|0)==(0);
    $229 = $226 & $227;
    if ($229) {
     $307 = $206;$310 = $228;$a$0 = $2;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 11664;
     label = 95;
    } else {
     $$012$i = $2;$231 = $224;$238 = $228;
     while(1) {
      $230 = $231 & 15;
      $232 = (11744 + ($230)|0);
      $233 = HEAP8[$232>>0]|0;
      $234 = $233&255;
      $235 = $234 | $225;
      $236 = $235&255;
      $237 = (($$012$i) + -1|0);
      HEAP8[$237>>0] = $236;
      $239 = (_bitshift64Lshr(($231|0),($238|0),4)|0);
      $240 = tempRet0;
      $241 = ($239|0)==(0);
      $242 = ($240|0)==(0);
      $243 = $241 & $242;
      if ($243) {
       break;
      } else {
       $$012$i = $237;$231 = $239;$238 = $240;
      }
     }
     $244 = $fl$3 & 8;
     $245 = ($244|0)==(0);
     if ($245) {
      $307 = $206;$310 = $228;$a$0 = $237;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 11664;
      label = 95;
     } else {
      $246 = $t$1 >> 4;
      $247 = (11664 + ($246)|0);
      $307 = $206;$310 = $228;$a$0 = $237;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 2;$prefix$1 = $247;
      label = 95;
     }
    }
   }
   else if ((label|0) == 90) {
    label = 0;
    $275 = $276;
    $277 = ($278>>>0)>(0);
    $279 = ($278|0)==(0);
    $280 = ($275>>>0)>(4294967295);
    $281 = $279 & $280;
    $282 = $277 | $281;
    if ($282) {
     $$05$i = $2;$283 = $275;$284 = $278;
     while(1) {
      $285 = (___uremdi3(($283|0),($284|0),10,0)|0);
      $286 = tempRet0;
      $287 = $285 | 48;
      $288 = $287&255;
      $289 = (($$05$i) + -1|0);
      HEAP8[$289>>0] = $288;
      $290 = (___udivdi3(($283|0),($284|0),10,0)|0);
      $291 = tempRet0;
      $292 = ($284>>>0)>(9);
      $293 = ($284|0)==(9);
      $294 = ($283>>>0)>(4294967295);
      $295 = $293 & $294;
      $296 = $292 | $295;
      if ($296) {
       $$05$i = $289;$283 = $290;$284 = $291;
      } else {
       break;
      }
     }
     $$0$lcssa$i46 = $289;$$01$lcssa$off0$i = $290;
    } else {
     $$0$lcssa$i46 = $2;$$01$lcssa$off0$i = $275;
    }
    $297 = ($$01$lcssa$off0$i|0)==(0);
    if ($297) {
     $307 = $276;$310 = $278;$a$0 = $$0$lcssa$i46;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
     label = 95;
    } else {
     $$12$i = $$0$lcssa$i46;$y$03$i = $$01$lcssa$off0$i;
     while(1) {
      $298 = (($y$03$i>>>0) % 10)&-1;
      $299 = $298 | 48;
      $300 = $299&255;
      $301 = (($$12$i) + -1|0);
      HEAP8[$301>>0] = $300;
      $302 = (($y$03$i>>>0) / 10)&-1;
      $303 = ($y$03$i>>>0)<(10);
      if ($303) {
       $307 = $276;$310 = $278;$a$0 = $301;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
       label = 95;
       break;
      } else {
       $$12$i = $301;$y$03$i = $302;
      }
     }
    }
   }
   else if ((label|0) == 100) {
    label = 0;
    $324 = (_memchr($a$1,0,$p$0)|0);
    $325 = ($324|0)==(0|0);
    if ($325) {
     $326 = (($a$1) + ($p$0)|0);
     $1054 = $228;$1055 = $206;$a$2 = $a$1;$fl$6 = $205;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 11664;$z$2 = $326;
     break;
    } else {
     $327 = $324;
     $328 = $a$1;
     $329 = (($327) - ($328))|0;
     $1054 = $228;$1055 = $206;$a$2 = $a$1;$fl$6 = $205;$p$5 = $329;$pl$2 = 0;$prefix$2 = 11664;$z$2 = $324;
     break;
    }
   }
   else if ((label|0) == 105) {
    label = 0;
    $i$0169 = 0;$l$1168 = 0;$ws$0170 = $1056;
    while(1) {
     $335 = HEAP32[$ws$0170>>2]|0;
     $336 = ($335|0)==(0);
     if ($336) {
      $i$0$lcssa = $i$0169;$l$1$lcssa = $l$1168;
      break;
     }
     $337 = (_wctomb($mb,$335)|0);
     $338 = ($337|0)>(-1);
     if (!($338)) {
      $$0 = -1;
      label = 357;
      break L1;
     }
     $339 = (($p$4296) - ($i$0169))|0;
     $340 = ($337>>>0)>($339>>>0);
     $334 = (($337) + ($i$0169))|0;
     if ($340) {
      $1058 = $1056;$1059 = $1057;$i$0167 = $i$0169;
      label = 111;
      break L437;
     }
     $332 = (($ws$0170) + 4|0);
     $333 = ($334>>>0)<($p$4296>>>0);
     if ($333) {
      $i$0169 = $334;$l$1168 = $337;$ws$0170 = $332;
     } else {
      $i$0$lcssa = $334;$l$1$lcssa = $337;
      break;
     }
    }
    $341 = ($l$1$lcssa|0)<(0);
    if ($341) {
     $$0 = -1;
     label = 357;
     break L1;
    } else {
     $1058 = $1056;$1059 = $1057;$i$0167 = $i$0$lcssa;
     label = 111;
    }
   }
  } while(0);
  if ((label|0) == 95) {
   label = 0;
   $304 = ($p$2|0)>(-1);
   $305 = $fl$4 & -65537;
   $$fl$4 = $304 ? $305 : $fl$4;
   $306 = $307;
   $308 = ($306|0)==(0);
   $309 = ($310|0)==(0);
   $311 = $308 & $309;
   $312 = ($p$2|0)==(0);
   $or$cond = $311 & $312;
   if ($or$cond) {
    $1054 = $310;$1055 = $307;$a$2 = $2;$fl$6 = $$fl$4;$p$5 = 0;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   } else {
    $313 = $a$0;
    $314 = (($3) - ($313))|0;
    $315 = $311&1;
    $316 = (($315) + ($314))|0;
    $317 = ($p$2|0)>($316|0);
    $p$2$ = $317 ? $p$2 : $316;
    $1054 = $310;$1055 = $307;$a$2 = $a$0;$fl$6 = $$fl$4;$p$5 = $p$2$;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   }
  }
  else if ((label|0) == 111) {
   label = 0;
   $342 = $fl$1$ & 73728;
   $343 = ($342|0)==(0);
   $344 = ($i$0167|0)<($w$1|0);
   $or$cond$i57 = $343 & $344;
   if ($or$cond$i57) {
    $345 = (($w$1) - ($i$0167))|0;
    $346 = ($345>>>0)>(256);
    $347 = $346 ? 256 : $345;
    _memset(($pad$i|0),32,($347|0))|0;
    $348 = ($345>>>0)>(255);
    if ($348) {
     $$01$i58 = $345;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $349 = (($$01$i58) + -256)|0;
      $350 = ($349>>>0)>(255);
      if ($350) {
       $$01$i58 = $349;
      } else {
       $$0$lcssa$i60 = $349;
       break;
      }
     }
    } else {
     $$0$lcssa$i60 = $345;
    }
    (___fwritex($pad$i,$$0$lcssa$i60,$f)|0);
   }
   $351 = ($i$0167|0)==(0);
   L471: do {
    if (!($351)) {
     $i$1175 = 0;$ws$1176 = $1058;
     while(1) {
      $352 = HEAP32[$ws$1176>>2]|0;
      $353 = ($352|0)==(0);
      if ($353) {
       break L471;
      }
      $354 = (_wctomb($mb,$352)|0);
      $355 = (($354) + ($i$1175))|0;
      $356 = ($355|0)>($i$0167|0);
      if ($356) {
       break L471;
      }
      $357 = (($ws$1176) + 4|0);
      (___fwritex($mb,$354,$f)|0);
      $358 = ($355>>>0)<($i$0167>>>0);
      if ($358) {
       $i$1175 = $355;$ws$1176 = $357;
      } else {
       break;
      }
     }
    }
   } while(0);
   $359 = ($342|0)==(8192);
   $or$cond$i64 = $359 & $344;
   if ($or$cond$i64) {
    $360 = (($w$1) - ($i$0167))|0;
    $361 = ($360>>>0)>(256);
    $362 = $361 ? 256 : $360;
    _memset(($pad$i|0),32,($362|0))|0;
    $363 = ($360>>>0)>(255);
    if ($363) {
     $$01$i65 = $360;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $364 = (($$01$i65) + -256)|0;
      $365 = ($364>>>0)>(255);
      if ($365) {
       $$01$i65 = $364;
      } else {
       $$0$lcssa$i67 = $364;
       break;
      }
     }
    } else {
     $$0$lcssa$i67 = $360;
    }
    (___fwritex($pad$i,$$0$lcssa$i67,$f)|0);
   }
   $366 = $344 ? $w$1 : $i$0167;
   $1048 = $228;$1049 = $1059;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $366;$l10n$0 = $l10n$3;
   continue;
  }
  $945 = $z$2;
  $946 = $a$2;
  $947 = (($945) - ($946))|0;
  $948 = ($p$5|0)<($947|0);
  $$p$5 = $948 ? $947 : $p$5;
  $949 = (($pl$2) + ($$p$5))|0;
  $950 = ($w$1|0)<($949|0);
  $w$2 = $950 ? $949 : $w$1;
  $951 = $fl$6 & 73728;
  $952 = ($951|0)==(0);
  $953 = ($949|0)<($w$2|0);
  $or$cond$i75 = $952 & $953;
  if ($or$cond$i75) {
   $954 = (($w$2) - ($949))|0;
   $955 = ($954>>>0)>(256);
   $956 = $955 ? 256 : $954;
   _memset(($pad$i|0),32,($956|0))|0;
   $957 = ($954>>>0)>(255);
   if ($957) {
    $$01$i76 = $954;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $958 = (($$01$i76) + -256)|0;
     $959 = ($958>>>0)>(255);
     if ($959) {
      $$01$i76 = $958;
     } else {
      $$0$lcssa$i78 = $958;
      break;
     }
    }
   } else {
    $$0$lcssa$i78 = $954;
   }
   (___fwritex($pad$i,$$0$lcssa$i78,$f)|0);
  }
  (___fwritex($prefix$2,$pl$2,$f)|0);
  $960 = ($951|0)==(65536);
  $or$cond$i50 = $960 & $953;
  if ($or$cond$i50) {
   $961 = (($w$2) - ($949))|0;
   $962 = ($961>>>0)>(256);
   $963 = $962 ? 256 : $961;
   _memset(($pad$i|0),48,($963|0))|0;
   $964 = ($961>>>0)>(255);
   if ($964) {
    $$01$i51 = $961;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $965 = (($$01$i51) + -256)|0;
     $966 = ($965>>>0)>(255);
     if ($966) {
      $$01$i51 = $965;
     } else {
      $$0$lcssa$i53 = $965;
      break;
     }
    }
   } else {
    $$0$lcssa$i53 = $961;
   }
   (___fwritex($pad$i,$$0$lcssa$i53,$f)|0);
  }
  $967 = ($947|0)<($$p$5|0);
  if ($967) {
   $968 = (($$p$5) - ($947))|0;
   $969 = ($968>>>0)>(256);
   $970 = $969 ? 256 : $968;
   _memset(($pad$i|0),48,($970|0))|0;
   $971 = ($968>>>0)>(255);
   if ($971) {
    $$01$i36 = $968;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $972 = (($$01$i36) + -256)|0;
     $973 = ($972>>>0)>(255);
     if ($973) {
      $$01$i36 = $972;
     } else {
      $$0$lcssa$i38 = $972;
      break;
     }
    }
   } else {
    $$0$lcssa$i38 = $968;
   }
   (___fwritex($pad$i,$$0$lcssa$i38,$f)|0);
  }
  (___fwritex($a$2,$947,$f)|0);
  $974 = ($951|0)==(8192);
  $or$cond$i = $974 & $953;
  if (!($or$cond$i)) {
   $1048 = $1054;$1049 = $1055;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
   continue;
  }
  $975 = (($w$2) - ($949))|0;
  $976 = ($975>>>0)>(256);
  $977 = $976 ? 256 : $975;
  _memset(($pad$i|0),32,($977|0))|0;
  $978 = ($975>>>0)>(255);
  if ($978) {
   $$01$i = $975;
   while(1) {
    (___fwritex($pad$i,256,$f)|0);
    $979 = (($$01$i) + -256)|0;
    $980 = ($979>>>0)>(255);
    if ($980) {
     $$01$i = $979;
    } else {
     $$0$lcssa$i = $979;
     break;
    }
   }
  } else {
   $$0$lcssa$i = $975;
  }
  (___fwritex($pad$i,$$0$lcssa$i,$f)|0);
  $1048 = $1054;$1049 = $1055;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
 }
 if ((label|0) == 339) {
  $981 = ($f|0)==(0|0);
  if (!($981)) {
   $$0 = $cnt$1;
   STACKTOP = sp;return ($$0|0);
  }
  $982 = ($l10n$0|0)==(0);
  if ($982) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  } else {
   $i$291 = 1;
  }
  while(1) {
   $983 = (($nl_type) + ($i$291<<2)|0);
   $984 = HEAP32[$983>>2]|0;
   $985 = ($984|0)==(0);
   if ($985) {
    $i$390 = $i$291;
    break;
   }
   $986 = (($nl_arg) + ($i$291<<3)|0);
   $987 = ($984>>>0)>(20);
   L523: do {
    if (!($987)) {
     do {
      switch ($984|0) {
      case 9:  {
       $arglist_current35 = HEAP32[$ap>>2]|0;
       $988 = HEAP32[$arglist_current35>>2]|0;
       $arglist_next36 = (($arglist_current35) + 4|0);
       HEAP32[$ap>>2] = $arglist_next36;
       HEAP32[$986>>2] = $988;
       break L523;
       break;
      }
      case 10:  {
       $arglist_current38 = HEAP32[$ap>>2]|0;
       $989 = HEAP32[$arglist_current38>>2]|0;
       $arglist_next39 = (($arglist_current38) + 4|0);
       HEAP32[$ap>>2] = $arglist_next39;
       $990 = ($989|0)<(0);
       $991 = $990 << 31 >> 31;
       $992 = $986;
       $993 = $992;
       HEAP32[$993>>2] = $989;
       $994 = (($992) + 4)|0;
       $995 = $994;
       HEAP32[$995>>2] = $991;
       break L523;
       break;
      }
      case 11:  {
       $arglist_current41 = HEAP32[$ap>>2]|0;
       $996 = HEAP32[$arglist_current41>>2]|0;
       $arglist_next42 = (($arglist_current41) + 4|0);
       HEAP32[$ap>>2] = $arglist_next42;
       $997 = $986;
       $998 = $997;
       HEAP32[$998>>2] = $996;
       $999 = (($997) + 4)|0;
       $1000 = $999;
       HEAP32[$1000>>2] = 0;
       break L523;
       break;
      }
      case 12:  {
       $arglist_current44 = HEAP32[$ap>>2]|0;
       $1001 = $arglist_current44;
       $1002 = $1001;
       $1003 = HEAP32[$1002>>2]|0;
       $1004 = (($1001) + 4)|0;
       $1005 = $1004;
       $1006 = HEAP32[$1005>>2]|0;
       $arglist_next45 = (($arglist_current44) + 8|0);
       HEAP32[$ap>>2] = $arglist_next45;
       $1007 = $986;
       $1008 = $1007;
       HEAP32[$1008>>2] = $1003;
       $1009 = (($1007) + 4)|0;
       $1010 = $1009;
       HEAP32[$1010>>2] = $1006;
       break L523;
       break;
      }
      case 13:  {
       $arglist_current47 = HEAP32[$ap>>2]|0;
       $1011 = HEAP32[$arglist_current47>>2]|0;
       $arglist_next48 = (($arglist_current47) + 4|0);
       HEAP32[$ap>>2] = $arglist_next48;
       $1012 = $1011&65535;
       $1013 = $1012 << 16 >> 16;
       $1014 = ($1013|0)<(0);
       $1015 = $1014 << 31 >> 31;
       $1016 = $986;
       $1017 = $1016;
       HEAP32[$1017>>2] = $1013;
       $1018 = (($1016) + 4)|0;
       $1019 = $1018;
       HEAP32[$1019>>2] = $1015;
       break L523;
       break;
      }
      case 14:  {
       $arglist_current50 = HEAP32[$ap>>2]|0;
       $1020 = HEAP32[$arglist_current50>>2]|0;
       $arglist_next51 = (($arglist_current50) + 4|0);
       HEAP32[$ap>>2] = $arglist_next51;
       $$mask1$i = $1020 & 65535;
       $1021 = $986;
       $1022 = $1021;
       HEAP32[$1022>>2] = $$mask1$i;
       $1023 = (($1021) + 4)|0;
       $1024 = $1023;
       HEAP32[$1024>>2] = 0;
       break L523;
       break;
      }
      case 15:  {
       $arglist_current53 = HEAP32[$ap>>2]|0;
       $1025 = HEAP32[$arglist_current53>>2]|0;
       $arglist_next54 = (($arglist_current53) + 4|0);
       HEAP32[$ap>>2] = $arglist_next54;
       $1026 = $1025&255;
       $1027 = $1026 << 24 >> 24;
       $1028 = ($1027|0)<(0);
       $1029 = $1028 << 31 >> 31;
       $1030 = $986;
       $1031 = $1030;
       HEAP32[$1031>>2] = $1027;
       $1032 = (($1030) + 4)|0;
       $1033 = $1032;
       HEAP32[$1033>>2] = $1029;
       break L523;
       break;
      }
      case 16:  {
       $arglist_current56 = HEAP32[$ap>>2]|0;
       $1034 = HEAP32[$arglist_current56>>2]|0;
       $arglist_next57 = (($arglist_current56) + 4|0);
       HEAP32[$ap>>2] = $arglist_next57;
       $$mask$i = $1034 & 255;
       $1035 = $986;
       $1036 = $1035;
       HEAP32[$1036>>2] = $$mask$i;
       $1037 = (($1035) + 4)|0;
       $1038 = $1037;
       HEAP32[$1038>>2] = 0;
       break L523;
       break;
      }
      case 17:  {
       $arglist_current59 = HEAP32[$ap>>2]|0;
       HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current59>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current59+4>>2];$1039 = +HEAPF64[tempDoublePtr>>3];
       $arglist_next60 = (($arglist_current59) + 8|0);
       HEAP32[$ap>>2] = $arglist_next60;
       HEAPF64[$986>>3] = $1039;
       break L523;
       break;
      }
      case 18:  {
       $arglist_current62 = HEAP32[$ap>>2]|0;
       HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current62>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current62+4>>2];$1040 = +HEAPF64[tempDoublePtr>>3];
       $arglist_next63 = (($arglist_current62) + 8|0);
       HEAP32[$ap>>2] = $arglist_next63;
       HEAPF64[$986>>3] = $1040;
       break L523;
       break;
      }
      default: {
       break L523;
      }
      }
     } while(0);
    }
   } while(0);
   $1041 = (($i$291) + 1)|0;
   $1042 = ($1041|0)<(10);
   if ($1042) {
    $i$291 = $1041;
   } else {
    $$0 = 1;
    label = 357;
    break;
   }
  }
  if ((label|0) == 357) {
   STACKTOP = sp;return ($$0|0);
  }
  while(1) {
   $1045 = (($nl_type) + ($i$390<<2)|0);
   $1046 = HEAP32[$1045>>2]|0;
   $1047 = ($1046|0)==(0);
   $1044 = (($i$390) + 1)|0;
   if (!($1047)) {
    $$0 = -1;
    label = 357;
    break;
   }
   $1043 = ($1044|0)<(10);
   if ($1043) {
    $i$390 = $1044;
   } else {
    $$0 = 1;
    label = 357;
    break;
   }
  }
  if ((label|0) == 357) {
   STACKTOP = sp;return ($$0|0);
  }
 }
 else if ((label|0) == 357) {
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _vsnprintf($s,$n,$fmt,$ap) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$$02 = 0, $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $f = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $b = sp + 112|0;
 $f = sp;
 dest=$f+0|0; src=11768+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $0 = (($n) + -1)|0;
 $1 = ($0>>>0)>(2147483646);
 if ($1) {
  $2 = ($n|0)==(0);
  if ($2) {
   $$01 = $b;$$02 = 1;
  } else {
   $3 = (___errno_location()|0);
   HEAP32[$3>>2] = 75;
   $$0 = -1;
   STACKTOP = sp;return ($$0|0);
  }
 } else {
  $$01 = $s;$$02 = $n;
 }
 $4 = $$01;
 $5 = (-2 - ($4))|0;
 $6 = ($$02>>>0)>($5>>>0);
 $$$02 = $6 ? $5 : $$02;
 $7 = (($f) + 48|0);
 HEAP32[$7>>2] = $$$02;
 $8 = (($f) + 20|0);
 HEAP32[$8>>2] = $$01;
 $9 = (($f) + 44|0);
 HEAP32[$9>>2] = $$01;
 $10 = (($$01) + ($$$02)|0);
 $11 = (($f) + 16|0);
 HEAP32[$11>>2] = $10;
 $12 = (($f) + 28|0);
 HEAP32[$12>>2] = $10;
 $13 = (_MUSL_vfprintf($f,$fmt,$ap)|0);
 $14 = ($$$02|0)==(0);
 if ($14) {
  $$0 = $13;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = HEAP32[$8>>2]|0;
 $16 = HEAP32[$11>>2]|0;
 $17 = ($15|0)==($16|0);
 $18 = $17 << 31 >> 31;
 $19 = (($15) + ($18)|0);
 HEAP8[$19>>0] = 0;
 $$0 = $13;
 STACKTOP = sp;return ($$0|0);
}
function _sn_write($f,$s,$l) {
 $f = $f|0;
 $s = $s|0;
 $l = $l|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($f) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $1;
 $5 = $3;
 $6 = (($4) - ($5))|0;
 $7 = ($6>>>0)>($l>>>0);
 $l$ = $7 ? $l : $6;
 _memcpy(($3|0),($s|0),($l$|0))|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($l$)|0);
 HEAP32[$2>>2] = $9;
 STACKTOP = sp;return ($l|0);
}
function _vsprintf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_vsnprintf($s,2147483647,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _memchr($src,$c,$n) {
 $src = $src|0;
 $c = $c|0;
 $n = $n|0;
 var $$0$lcssa = 0, $$0$lcssa34 = 0, $$013 = 0, $$1$lcssa = 0, $$17 = 0, $$24 = 0, $$3 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond12 = 0, $s$0$lcssa = 0, $s$0$lcssa33 = 0, $s$014 = 0, $s$15 = 0, $s$2 = 0, $w$0$lcssa = 0, $w$08 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = $src;
 $2 = $1 & 3;
 $3 = ($2|0)==(0);
 $4 = ($n|0)==(0);
 $or$cond12 = $3 | $4;
 L1: do {
  if ($or$cond12) {
   $$0$lcssa = $n;$$lcssa = $4;$s$0$lcssa = $src;
   label = 5;
  } else {
   $5 = $c&255;
   $$013 = $n;$s$014 = $src;
   while(1) {
    $6 = HEAP8[$s$014>>0]|0;
    $7 = ($6<<24>>24)==($5<<24>>24);
    if ($7) {
     $$0$lcssa34 = $$013;$s$0$lcssa33 = $s$014;
     label = 6;
     break L1;
    }
    $8 = (($s$014) + 1|0);
    $9 = (($$013) + -1)|0;
    $10 = $8;
    $11 = $10 & 3;
    $12 = ($11|0)==(0);
    $13 = ($9|0)==(0);
    $or$cond = $12 | $13;
    if ($or$cond) {
     $$0$lcssa = $9;$$lcssa = $13;$s$0$lcssa = $8;
     label = 5;
     break;
    } else {
     $$013 = $9;$s$014 = $8;
    }
   }
  }
 } while(0);
 if ((label|0) == 5) {
  if ($$lcssa) {
   $$3 = 0;$s$2 = $s$0$lcssa;
  } else {
   $$0$lcssa34 = $$0$lcssa;$s$0$lcssa33 = $s$0$lcssa;
   label = 6;
  }
 }
 L8: do {
  if ((label|0) == 6) {
   $14 = HEAP8[$s$0$lcssa33>>0]|0;
   $15 = $c&255;
   $16 = ($14<<24>>24)==($15<<24>>24);
   if ($16) {
    $$3 = $$0$lcssa34;$s$2 = $s$0$lcssa33;
   } else {
    $17 = Math_imul($0, 16843009)|0;
    $18 = ($$0$lcssa34>>>0)>(3);
    L11: do {
     if ($18) {
      $$17 = $$0$lcssa34;$w$08 = $s$0$lcssa33;
      while(1) {
       $19 = HEAP32[$w$08>>2]|0;
       $20 = $19 ^ $17;
       $21 = (($20) + -16843009)|0;
       $22 = $20 & -2139062144;
       $23 = $22 ^ -2139062144;
       $24 = $23 & $21;
       $25 = ($24|0)==(0);
       if (!($25)) {
        $$1$lcssa = $$17;$w$0$lcssa = $w$08;
        break L11;
       }
       $26 = (($w$08) + 4|0);
       $27 = (($$17) + -4)|0;
       $28 = ($27>>>0)>(3);
       if ($28) {
        $$17 = $27;$w$08 = $26;
       } else {
        $$1$lcssa = $27;$w$0$lcssa = $26;
        break;
       }
      }
     } else {
      $$1$lcssa = $$0$lcssa34;$w$0$lcssa = $s$0$lcssa33;
     }
    } while(0);
    $29 = ($$1$lcssa|0)==(0);
    if ($29) {
     $$3 = 0;$s$2 = $w$0$lcssa;
    } else {
     $$24 = $$1$lcssa;$s$15 = $w$0$lcssa;
     while(1) {
      $30 = HEAP8[$s$15>>0]|0;
      $31 = ($30<<24>>24)==($15<<24>>24);
      if ($31) {
       $$3 = $$24;$s$2 = $s$15;
       break L8;
      }
      $32 = (($s$15) + 1|0);
      $33 = (($$24) + -1)|0;
      $34 = ($33|0)==(0);
      if ($34) {
       $$3 = 0;$s$2 = $32;
       break;
      } else {
       $$24 = $33;$s$15 = $32;
      }
     }
    }
   }
  }
 } while(0);
 $35 = ($$3|0)!=(0);
 $36 = $35 ? $s$2 : 0;
 STACKTOP = sp;return ($36|0);
}
function runPostSets() {
 
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _strlen(ptr) {
    ptr = ptr|0;
    var curr = 0;
    curr = ptr;
    while (((HEAP8[((curr)>>0)])|0)) {
      curr = (curr + 1)|0;
    }
    return (curr - ptr)|0;
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _memcpy(dest, src, num) {

    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_ctlz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((ctlz_i8)+(x >>> 24))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((ctlz_i8)+(x&0xff))>>0)])|0) + 24)|0;
  }

function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return (tempRet0 = tempRet0, $10$0) | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return (tempRet0 = tempRet0, $1$0) | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (_llvm_ctlz_i32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0, $d_sroa_0_0_insert_insert99$1, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



// EMSCRIPTEN_END_FUNCS

    
    function dynCall_iiii(index,a1,a2,a3) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0;
      return FUNCTION_TABLE_iiii[index&127](a1|0,a2|0,a3|0)|0;
    }
  

    function dynCall_viiiiiii(index,a1,a2,a3,a4,a5,a6,a7) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0; a7=a7|0;
      FUNCTION_TABLE_viiiiiii[index&63](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0,a7|0);
    }
  

    function dynCall_viiiii(index,a1,a2,a3,a4,a5) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
      FUNCTION_TABLE_viiiii[index&127](a1|0,a2|0,a3|0,a4|0,a5|0);
    }
  

    function dynCall_vi(index,a1) {
      index = index|0;
      a1=a1|0;
      FUNCTION_TABLE_vi[index&127](a1|0);
    }
  

    function dynCall_vii(index,a1,a2) {
      index = index|0;
      a1=a1|0; a2=a2|0;
      FUNCTION_TABLE_vii[index&127](a1|0,a2|0);
    }
  

    function dynCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
      return FUNCTION_TABLE_iiiiiii[index&127](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
    }
  

    function dynCall_ii(index,a1) {
      index = index|0;
      a1=a1|0;
      return FUNCTION_TABLE_ii[index&127](a1|0)|0;
    }
  

    function dynCall_viii(index,a1,a2,a3) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0;
      FUNCTION_TABLE_viii[index&31](a1|0,a2|0,a3|0);
    }
  

    function dynCall_iiiii(index,a1,a2,a3,a4) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
      return FUNCTION_TABLE_iiiii[index&127](a1|0,a2|0,a3|0,a4|0)|0;
    }
  

    function dynCall_iii(index,a1,a2) {
      index = index|0;
      a1=a1|0; a2=a2|0;
      return FUNCTION_TABLE_iii[index&127](a1|0,a2|0)|0;
    }
  

    function dynCall_iiiiii(index,a1,a2,a3,a4,a5) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
      return FUNCTION_TABLE_iiiiii[index&127](a1|0,a2|0,a3|0,a4|0,a5|0)|0;
    }
  

    function dynCall_viiii(index,a1,a2,a3,a4) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
      FUNCTION_TABLE_viiii[index&127](a1|0,a2|0,a3|0,a4|0);
    }
  
function b0(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_iiii(0);return 0; }
  function b1(p0,p1,p2,p3,p4,p5,p6) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0;p6 = p6|0; nullFunc_viiiiiii(1); }
  function b2(p0,p1,p2,p3,p4) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; nullFunc_viiiii(2); }
  function b3(p0) { p0 = p0|0; nullFunc_vi(3); }
  function b4(p0,p1) { p0 = p0|0;p1 = p1|0; nullFunc_vii(4); }
  function b5(p0,p1,p2,p3,p4,p5) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; nullFunc_iiiiiii(5);return 0; }
  function b6(p0) { p0 = p0|0; nullFunc_ii(6);return 0; }
  function b7(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_viii(7); }
  function b8(p0,p1,p2,p3) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; nullFunc_iiiii(8);return 0; }
  function b9(p0,p1) { p0 = p0|0;p1 = p1|0; nullFunc_iii(9);return 0; }
  function b10(p0,p1,p2,p3,p4) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; nullFunc_iiiiii(10);return 0; }
  function b11(p0,p1,p2,p3) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; nullFunc_viiii(11); }
  // EMSCRIPTEN_END_FUNCS
  var FUNCTION_TABLE_iiii = [b0,_sn_write,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,_alloc_small,_alloc_large,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,_do_read,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0];
  var FUNCTION_TABLE_viiiiiii = [b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
  ,b1,b1,b1,b1,b1,b1,b1,b1,b1,_merged_2v_upsample,b1,_merged_1v_upsample,b1,b1,b1,b1,b1,b1,b1,_post_process_1pass,_post_process_prepass,_post_process_2pass,b1,_sep_upsample,b1,b1,b1,b1,b1,b1
  ,b1,b1,b1,b1,b1];
  var FUNCTION_TABLE_viiiii = [b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,_grayscale_convert,_ycc_rgb_convert,_gray_rgb_convert,_null_convert,_ycck_cmyk_convert,b2,_jpeg_idct_1x1,_jpeg_idct_2x2,_jpeg_idct_4x4,_jpeg_idct_islow,_jpeg_idct_ifast,_jpeg_idct_float,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2];
  var FUNCTION_TABLE_vi = [b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,_init_source,b3,b3,b3,_term_source,b3,_reset_input_controller,_start_input_pass
  ,_finish_input_pass,_reset_marker_reader,b3,b3,b3,b3,_prepare_for_output_pass,_finish_output_pass,_start_pass_merged_upsample,b3,b3,b3,b3,_start_pass_phuff_decoder,b3,b3,b3,b3,b3,b3,b3,b3,_start_pass_upsample,b3,b3,b3,b3,b3,b3,b3
  ,b3,_error_exit,b3,_output_message,b3,_reset_error_mgr,b3,b3,b3,b3,b3,b3,_realize_virt_arrays,b3,b3,b3,_self_destruct,b3,_finish_pass_1_quant,_new_color_map_1_quant,b3,b3,b3,b3,b3,b3,_new_color_map_2_quant,b3,_finish_pass1,b3
  ,b3,_finish_pass2,_start_input_pass332,_start_output_pass,b3,b3,b3,b3,b3,_start_pass_dcolor,b3,b3,b3,b3,b3,_start_pass,b3,b3,b3,b3,b3,b3,_start_pass_huff_decoder,b3,b3,b3,b3,b3,b3,b3
  ,b3,b3,b3,b3,b3,b3,b3,b3,b3];
  var FUNCTION_TABLE_vii = [b4,b4,b4,_start_output_ppm,_finish_output_ppm,b4,b4,b4,b4,_start_output_gif,b4,_finish_output_gif,_start_output_tga,_finish_output_tga,b4,b4,b4,_start_output_bmp,_finish_output_bmp,b4,b4,b4,b4,_skip_input_data,b4,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_start_pass_dpost,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
  ,b4,b4,_emit_message,b4,_format_message,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_free_pool,b4,_start_pass_1_quant,b4,b4,b4,b4,b4,b4,b4,_start_pass_2_quant,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_start_pass_main,b4,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4];
  var FUNCTION_TABLE_iiiiiii = [b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,_request_virt_sarray,_request_virt_barray,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5];
  var FUNCTION_TABLE_ii = [b6,b6,_print_text_marker,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,_fill_input_buffer,b6,b6,b6,_consume_markers,b6,b6
  ,b6,b6,_read_markers,_read_restart_marker,_skip_variable,_get_interesting_appn,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,_consume_data,b6,_dummy_consume_data,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,b6,b6,b6,b6,b6];
  var FUNCTION_TABLE_viii = [b7,b7,b7,b7,b7,_copy_pixel_rows,_put_demapped_gray,_put_demapped_rgb,_put_pixel_rows,b7,_put_pixel_rows120,b7,b7,b7,_put_demapped_gray123,_put_gray_rows,_put_pixel_rows124,b7,b7,_put_gray_rows127,_put_pixel_rows128,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7];
  var FUNCTION_TABLE_iiiii = [b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,_alloc_sarray,_alloc_barray,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8];
  var FUNCTION_TABLE_iii = [b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,_jpeg_resync_to_restart,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,_decode_mcu_DC_first,_decode_mcu_AC_first,_decode_mcu_DC_refine,_decode_mcu_AC_refine,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,_decompress_data,b9,_decompress_onepass,_decompress_smooth_data,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,_decode_mcu,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9];
  var FUNCTION_TABLE_iiiiii = [b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,_access_virt_sarray,_access_virt_barray,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10];
  var FUNCTION_TABLE_viiii = [b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
  ,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,_h2v2_merged_upsample,b11,_h2v1_merged_upsample,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,_noop_upsample,_fullsize_upsample,_h2v1_fancy_upsample,_h2v1_upsample,_h2v2_fancy_upsample,_h2v2_upsample
  ,_int_upsample,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,_color_quantize3,_color_quantize,_quantize3_ord_dither,_quantize_ord_dither,_quantize_fs_dither,b11,b11,_prescan_quantize,b11,_pass2_fs_dither
  ,_pass2_no_dither,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,_process_data_context_main,_process_data_simple_main,_process_data_crank_post,b11,b11
  ,b11,b11,b11,b11,b11,b11,b11,b11,b11];

    return { _i64Subtract: _i64Subtract, _free: _free, _main: _main, _realloc: _realloc, _i64Add: _i64Add, _strlen: _strlen, _memset: _memset, _malloc: _malloc, _memcpy: _memcpy, _bitshift64Lshr: _bitshift64Lshr, _bitshift64Shl: _bitshift64Shl, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii, dynCall_viiiiiii: dynCall_viiiiiii, dynCall_viiiii: dynCall_viiiii, dynCall_vi: dynCall_vi, dynCall_vii: dynCall_vii, dynCall_iiiiiii: dynCall_iiiiiii, dynCall_ii: dynCall_ii, dynCall_viii: dynCall_viii, dynCall_iiiii: dynCall_iiiii, dynCall_iii: dynCall_iii, dynCall_iiiiii: dynCall_iiiiii, dynCall_viiii: dynCall_viiii };
  })
  // EMSCRIPTEN_END_ASM
  ({ "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array }, { "abort": abort, "assert": assert, "asmPrintInt": asmPrintInt, "asmPrintFloat": asmPrintFloat, "min": Math_min, "nullFunc_iiii": nullFunc_iiii, "nullFunc_viiiiiii": nullFunc_viiiiiii, "nullFunc_viiiii": nullFunc_viiiii, "nullFunc_vi": nullFunc_vi, "nullFunc_vii": nullFunc_vii, "nullFunc_iiiiiii": nullFunc_iiiiiii, "nullFunc_ii": nullFunc_ii, "nullFunc_viii": nullFunc_viii, "nullFunc_iiiii": nullFunc_iiiii, "nullFunc_iii": nullFunc_iii, "nullFunc_iiiiii": nullFunc_iiiiii, "nullFunc_viiii": nullFunc_viiii, "invoke_iiii": invoke_iiii, "invoke_viiiiiii": invoke_viiiiiii, "invoke_viiiii": invoke_viiiii, "invoke_vi": invoke_vi, "invoke_vii": invoke_vii, "invoke_iiiiiii": invoke_iiiiiii, "invoke_ii": invoke_ii, "invoke_viii": invoke_viii, "invoke_iiiii": invoke_iiiii, "invoke_iii": invoke_iii, "invoke_iiiiii": invoke_iiiiii, "invoke_viiii": invoke_viiii, "_fabs": _fabs, "_putc": _putc, "_send": _send, "_fread": _fread, "_fmod": _fmod, "___setErrNo": ___setErrNo, "___buildEnvironment": ___buildEnvironment, "_fflush": _fflush, "_pwrite": _pwrite, "_strerror_r": _strerror_r, "_open": _open, "_sbrk": _sbrk, "_emscripten_memcpy_big": _emscripten_memcpy_big, "_fileno": _fileno, "_sysconf": _sysconf, "_close": _close, "_ferror": _ferror, "_pread": _pread, "_getenv": _getenv, "_fclose": _fclose, "__reallyNegative": __reallyNegative, "_getc": _getc, "_write": _write, "_fsync": _fsync, "___errno_location": ___errno_location, "_recv": _recv, "_copysign": _copysign, "_fgetc": _fgetc, "_fputc": _fputc, "_mkport": _mkport, "__exit": __exit, "_read": _read, "_copysignl": _copysignl, "_abort": _abort, "_fwrite": _fwrite, "_time": _time, "_fprintf": _fprintf, "_strerror": _strerror, "__formatString": __formatString, "_fopen": _fopen, "_exit": _exit, "_fmodl": _fmodl, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "ctlz_i8": ctlz_i8, "NaN": NaN, "Infinity": Infinity, "_stderr": _stderr, "_stdin": _stdin, "_stdout": _stdout }, buffer);
  var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _free = Module["_free"] = asm["_free"];
var _main = Module["_main"] = asm["_main"];
var _realloc = Module["_realloc"] = asm["_realloc"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _strlen = Module["_strlen"] = asm["_strlen"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_viiiiiii = Module["dynCall_viiiiiii"] = asm["dynCall_viiiiiii"];
var dynCall_viiiii = Module["dynCall_viiiii"] = asm["dynCall_viiiii"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_vii = Module["dynCall_vii"] = asm["dynCall_vii"];
var dynCall_iiiiiii = Module["dynCall_iiiiiii"] = asm["dynCall_iiiiiii"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_viii = Module["dynCall_viii"] = asm["dynCall_viii"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];
var dynCall_iiiiii = Module["dynCall_iiiiii"] = asm["dynCall_iiiiii"];
var dynCall_viiii = Module["dynCall_viiii"] = asm["dynCall_viiii"];
  
  Runtime.stackAlloc = asm['stackAlloc'];
  Runtime.stackSave = asm['stackSave'];
  Runtime.stackRestore = asm['stackRestore'];
  Runtime.setTempRet0 = asm['setTempRet0'];
  Runtime.getTempRet0 = asm['getTempRet0'];
  

// TODO: strip out parts of this we do not need

//======= begin closure i64 code =======

// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */

var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };


  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.

    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };


  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.


  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};


  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }

    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };


  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };


  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };


  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }

    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }

    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));

    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };


  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.


  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;


  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);


  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);


  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);


  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);


  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);


  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);


  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };


  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };


  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (this.isZero()) {
      return '0';
    }

    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }

    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));

    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);

      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };


  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };


  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };


  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };


  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };


  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };


  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };


  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };


  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }

    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }

    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };


  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };


  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };


  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }

    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }

    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }

    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));

      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);

      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }

      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }

      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };


  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };


  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };


  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };


  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };


  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };


  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };

  //======= begin jsbn =======

  var navigator = { appName: 'Modern Browser' }; // polyfill a little

  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/

  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */

  // Basic JavaScript BN library - subset useful for RSA encryption.

  // Bits per digit
  var dbits;

  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);

  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }

  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }

  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.

  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }

  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);

  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;

  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }

  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }

  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }

  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }

  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }

  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }

  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }

  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }

  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }

  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }

  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }

  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }

  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }

  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }

  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }

  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }

  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }

  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }

  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;

  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }

  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }

  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }

  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }

  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }

  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;

  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }

  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }

  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;

  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;

  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);

  // jsbn2 stuff

  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }

  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }

  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }

  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }

  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }

  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }

  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }

  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;

  //======= end jsbn =======

  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();

//======= end closure i64 code =======



// === Auto-generated postamble setup entry stuff ===

if (memoryInitializer) {
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    var data = Module['readBinary'](memoryInitializer);
    HEAPU8.set(data, STATIC_BASE);
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      for (var i = 0; i < data.length; i++) {
        assert(HEAPU8[STATIC_BASE + i] === 0, "area for memory initializer should not have been touched before it's loaded");
      }
      HEAPU8.set(data, STATIC_BASE);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}

function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun'] && shouldRunNow) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram'] || '/bin/this.program'), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    if (!Module['noExitRuntime']) {
      exit(ret);
    }
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['_main'] && shouldRunNow) {
      Module['callMain'](args);
    }

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      if (!ABORT) doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status) {
  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;

  // exit the runtime
  exitRuntime();

  // TODO We should handle this differently based on environment.
  // In the browser, the best we can do is throw an exception
  // to halt execution, but in node we could process.exit and
  // I'd imagine SM shell would have something equivalent.
  // This would let us set a proper exit status (which
  // would be great for checking test exit statuses).
  // https://github.com/kripken/emscripten/issues/1371

  // throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

function abort(text) {
  if (text) {
    Module.print(text);
    Module.printErr(text);
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '';

  throw 'abort() at ' + stackTrace() + extra;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



//# sourceMappingURL=djpeg.js.map