// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  Module['thisProgram'] = process['argv'][1];
  Module['arguments'] = process['argv'].slice(2);

  module['exports'] = Module;
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    return read(f, 'binary');
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  this['Module'] = Module;

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WEB) {
    window['Module'] = Module;
  } else {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] == 'undefined' && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Auto-generated preamble library stuff ===

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  forceAlign: function (target, quantum) {
    quantum = quantum || 4;
    if (quantum == 1) return target;
    if (isNumber(target) && isNumber(quantum)) {
      return Math.ceil(target/quantum)*quantum;
    } else if (isNumber(quantum) && isPowerOfTwo(quantum)) {
      return '(((' +target + ')+' + (quantum-1) + ')&' + -quantum + ')';
    }
    return 'Math.ceil((' + target + ')/' + quantum + ')*' + quantum;
  },
  isNumberType: function (type) {
    return type in Runtime.INT_TYPES || type in Runtime.FLOAT_TYPES;
  },
  isPointerType: function isPointerType(type) {
  return type[type.length-1] == '*';
},
  isStructType: function isStructType(type) {
  if (isPointerType(type)) return false;
  if (isArrayType(type)) return true;
  if (/<?\{ ?[^}]* ?\}>?/.test(type)) return true; // { i32, i8 } etc. - anonymous struct types
  // See comment in isStructPointerType()
  return type[0] == '%';
},
  INT_TYPES: {"i1":0,"i8":0,"i16":0,"i32":0,"i64":0},
  FLOAT_TYPES: {"float":0,"double":0},
  or64: function (x, y) {
    var l = (x | 0) | (y | 0);
    var h = (Math.round(x / 4294967296) | Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  and64: function (x, y) {
    var l = (x | 0) & (y | 0);
    var h = (Math.round(x / 4294967296) & Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  xor64: function (x, y) {
    var l = (x | 0) ^ (y | 0);
    var h = (Math.round(x / 4294967296) ^ Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  dedup: function dedup(items, ident) {
  var seen = {};
  if (ident) {
    return items.filter(function(item) {
      if (seen[item[ident]]) return false;
      seen[item[ident]] = true;
      return true;
    });
  } else {
    return items.filter(function(item) {
      if (seen[item]) return false;
      seen[item] = true;
      return true;
    });
  }
},
  set: function set() {
  var args = typeof arguments[0] === 'object' ? arguments[0] : arguments;
  var ret = {};
  for (var i = 0; i < args.length; i++) {
    ret[args[i]] = 0;
  }
  return ret;
},
  STACK_ALIGN: 8,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  calculateStructAlignment: function calculateStructAlignment(type) {
    type.flatSize = 0;
    type.alignSize = 0;
    var diffs = [];
    var prev = -1;
    var index = 0;
    type.flatIndexes = type.fields.map(function(field) {
      index++;
      var size, alignSize;
      if (Runtime.isNumberType(field) || Runtime.isPointerType(field)) {
        size = Runtime.getNativeTypeSize(field); // pack char; char; in structs, also char[X]s.
        alignSize = Runtime.getAlignSize(field, size);
      } else if (Runtime.isStructType(field)) {
        if (field[1] === '0') {
          // this is [0 x something]. When inside another structure like here, it must be at the end,
          // and it adds no size
          // XXX this happens in java-nbody for example... assert(index === type.fields.length, 'zero-length in the middle!');
          size = 0;
          if (Types.types[field]) {
            alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
          } else {
            alignSize = type.alignSize || QUANTUM_SIZE;
          }
        } else {
          size = Types.types[field].flatSize;
          alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
        }
      } else if (field[0] == 'b') {
        // bN, large number field, like a [N x i8]
        size = field.substr(1)|0;
        alignSize = 1;
      } else if (field[0] === '<') {
        // vector type
        size = alignSize = Types.types[field].flatSize; // fully aligned
      } else if (field[0] === 'i') {
        // illegal integer field, that could not be legalized because it is an internal structure field
        // it is ok to have such fields, if we just use them as markers of field size and nothing more complex
        size = alignSize = parseInt(field.substr(1))/8;
        assert(size % 1 === 0, 'cannot handle non-byte-size field ' + field);
      } else {
        assert(false, 'invalid type for calculateStructAlignment');
      }
      if (type.packed) alignSize = 1;
      type.alignSize = Math.max(type.alignSize, alignSize);
      var curr = Runtime.alignMemory(type.flatSize, alignSize); // if necessary, place this on aligned memory
      type.flatSize = curr + size;
      if (prev >= 0) {
        diffs.push(curr-prev);
      }
      prev = curr;
      return curr;
    });
    if (type.name_ && type.name_[0] === '[') {
      // arrays have 2 elements, so we get the proper difference. then we scale here. that way we avoid
      // allocating a potentially huge array for [999999 x i8] etc.
      type.flatSize = parseInt(type.name_.substr(1))*type.flatSize/2;
    }
    type.flatSize = Runtime.alignMemory(type.flatSize, type.alignSize);
    if (diffs.length == 0) {
      type.flatFactor = type.flatSize;
    } else if (Runtime.dedup(diffs).length == 1) {
      type.flatFactor = diffs[0];
    }
    type.needsFlattening = (type.flatFactor != 1);
    return type.flatIndexes;
  },
  generateStructInfo: function (struct, typeName, offset) {
    var type, alignment;
    if (typeName) {
      offset = offset || 0;
      type = (typeof Types === 'undefined' ? Runtime.typeInfo : Types.types)[typeName];
      if (!type) return null;
      if (type.fields.length != struct.length) {
        printErr('Number of named fields must match the type for ' + typeName + ': possibly duplicate struct names. Cannot return structInfo');
        return null;
      }
      alignment = type.flatIndexes;
    } else {
      var type = { fields: struct.map(function(item) { return item[0] }) };
      alignment = Runtime.calculateStructAlignment(type);
    }
    var ret = {
      __size__: type.flatSize
    };
    if (typeName) {
      struct.forEach(function(item, i) {
        if (typeof item === 'string') {
          ret[item] = alignment[i] + offset;
        } else {
          // embedded struct
          var key;
          for (var k in item) key = k;
          ret[key] = Runtime.generateStructInfo(item[key], type.fields[i], alignment[i]);
        }
      });
    } else {
      struct.forEach(function(item, i) {
        ret[item[1]] = alignment[i];
      });
    }
    return ret;
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      assert(args.length == sig.length-1);
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      assert(sig.length == 1);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  getAsmConst: function (code, numArgs) {
    // code is a constant string on the heap, so we can cache these
    if (!Runtime.asmConstCache) Runtime.asmConstCache = {};
    var func = Runtime.asmConstCache[code];
    if (func) return func;
    var args = [];
    for (var i = 0; i < numArgs; i++) {
      args.push(String.fromCharCode(36) + i); // $0, $1 etc
    }
    var source = Pointer_stringify(code);
    if (source[0] === '"') {
      // tolerate EM_ASM("..code..") even though EM_ASM(..code..) is correct
      if (source.indexOf('"', 1) === source.length-1) {
        source = source.substr(1, source.length-2);
      } else {
        // something invalid happened, e.g. EM_ASM("..code($0)..", input)
        abort('invalid EM_ASM input |' + source + '|. Please use EM_ASM(..code..) (no quotes) or EM_ASM({ ..code($0).. }, input) (to input values)');
      }
    }
    try {
      var evalled = eval('(function(' + args.join(',') + '){ ' + source + ' })'); // new Function does not allow upvars in node
    } catch(e) {
      Module.printErr('error in executing inline EM_ASM code: ' + e + ' on: \n\n' + source + '\n\nwith args |' + args + '| (make sure to use the right one out of EM_ASM, EM_ASM_ARGS, etc.)');
      throw e;
    }
    return Runtime.asmConstCache[code] = evalled;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[func]) {
      Runtime.funcWrappers[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return Runtime.funcWrappers[func];
  },
  UTF8Processor: function () {
    var buffer = [];
    var needed = 0;
    this.processCChar = function (code) {
      code = code & 0xFF;

      if (buffer.length == 0) {
        if ((code & 0x80) == 0x00) {        // 0xxxxxxx
          return String.fromCharCode(code);
        }
        buffer.push(code);
        if ((code & 0xE0) == 0xC0) {        // 110xxxxx
          needed = 1;
        } else if ((code & 0xF0) == 0xE0) { // 1110xxxx
          needed = 2;
        } else {                            // 11110xxx
          needed = 3;
        }
        return '';
      }

      if (needed) {
        buffer.push(code);
        needed--;
        if (needed > 0) return '';
      }

      var c1 = buffer[0];
      var c2 = buffer[1];
      var c3 = buffer[2];
      var c4 = buffer[3];
      var ret;
      if (buffer.length == 2) {
        ret = String.fromCharCode(((c1 & 0x1F) << 6)  | (c2 & 0x3F));
      } else if (buffer.length == 3) {
        ret = String.fromCharCode(((c1 & 0x0F) << 12) | ((c2 & 0x3F) << 6)  | (c3 & 0x3F));
      } else {
        // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        var codePoint = ((c1 & 0x07) << 18) | ((c2 & 0x3F) << 12) |
                        ((c3 & 0x3F) << 6)  | (c4 & 0x3F);
        ret = String.fromCharCode(
          Math.floor((codePoint - 0x10000) / 0x400) + 0xD800,
          (codePoint - 0x10000) % 0x400 + 0xDC00);
      }
      buffer.length = 0;
      return ret;
    }
    this.processJSString = function processJSString(string) {
      /* TODO: use TextEncoder when present,
        var encoder = new TextEncoder();
        encoder['encoding'] = "utf-8";
        var utf8Array = encoder['encode'](aMsg.data);
      */
      string = unescape(encodeURIComponent(string));
      var ret = [];
      for (var i = 0; i < string.length; i++) {
        ret.push(string.charCodeAt(i));
      }
      return ret;
    }
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+7)&-8);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+7)&-8); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+7)&-8); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 8))*(quantum ? quantum : 8); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;









//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var stack = 0;
  var JSfuncs = {
    'stackSave' : function() {
      stack = Runtime.stackSave();
    },
    'stackRestore' : function() {
      Runtime.stackRestore(stack);
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        ret = Runtime.stackAlloc(str.length + 1); // +1 for the trailing '\0'
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. A convenient way to call C functions (in C files, or
  // defined with extern "C").
  //
  // Note: ccall/cwrap use the C stack for temporary values. If you pass a string
  //       then it is only alive until the call is complete. If the code being
  //       called saves the pointer to be used later, it may point to invalid
  //       data. If you need a string to live forever, you can create it (and
  //       must later delete it manually!) using malloc and writeStringToMemory,
  //       for example.
  //
  // Note: LLVM optimizations can inline and remove functions, after which you will not be
  //       able to call them. Closure can also do so. To avoid that, add your function to
  //       the exports using something like
  //
  //         -s EXPORTED_FUNCTIONS='["_main", "_myfunc"]'
  //
  // @param ident      The name of the C function (note that C++ functions will be name-mangled - use extern "C")
  // @param returnType The return type of the function, one of the JS types 'number', 'string' or 'array' (use 'number' for any C pointer, and
  //                   'array' for JavaScript arrays and typed arrays; note that arrays are 8-bit).
  // @param argTypes   An array of the types of arguments for the function (if there are no arguments, this can be ommitted). Types are as in returnType,
  //                   except that 'array' is not possible (there is no way for us to know the length of the array)
  // @param args       An array of the arguments to the function, as native JS values (as in returnType)
  //                   Note that string arguments will be stored on the stack (the JS string will become a C string on the stack).
  // @return           The return value, as a native JS value (as in returnType)
  ccall = function ccallFunc(ident, returnType, argTypes, args) {
    assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
    var func = getCFunc(ident);
    var cArgs = [];
    assert(returnType !== 'array', 'Return type should not be "array".');
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) JSfuncs['stackRestore']();
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }
  // Returns a native JS wrapper for a C function. This is similar to ccall, but
  // returns a function you can call repeatedly in a normal way. For example:
  //
  //   var my_function = cwrap('my_c_function', 'number', ['number', 'number']);
  //   alert(my_function(5, 22));
  //   alert(my_function(99, 12));
  //
  cwrap = function cwrap(ident, returnType, argTypes) {
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    funcstr += "assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');\n";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["cwrap"] = cwrap;
Module["ccall"] = ccall;

// Sets a value in memory in a dynamic way at run-time. Uses the
// type data. This is the same as makeSetValue, except that
// makeSetValue is done at compile-time and generates the needed
// code then, whereas this function picks the right code at
// run-time.
// Note that setValue and getValue only do *aligned* writes and reads!
// Note that ccall uses JS types as for defining types, while setValue and
// getValue need LLVM types ('i8', 'i32') - this is a lower-level operation
function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;

// Parallel to setValue.
function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    assert(type, 'Must know what type to store in allocate!');

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

function Pointer_stringify(ptr, /* optional */ length) {
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = false;
  var t;
  var i = 0;
  while (1) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    if (t >= 128) hasUtf = true;
    else if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (!hasUtf) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }

  var utf8 = new Runtime.UTF8Processor();
  for (i = 0; i < length; i++) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    ret += utf8.processCChar(t);
  }
  return ret;
}
Module['Pointer_stringify'] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16LE form. The copy will require at most (str.length*2+1)*2 bytes of space in the HEAP.
function stringToUTF16(str, outPtr) {
  for(var i = 0; i < str.length; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[(((outPtr)+(i*2))>>1)]=codeUnit;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[(((outPtr)+(str.length*2))>>1)]=0;
}
Module['stringToUTF16'] = stringToUTF16;

// Given a pointer 'ptr' to a null-terminated UTF32LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32LE form. The copy will require at most (str.length+1)*4 bytes of space in the HEAP,
// but can use less, since str.length does not return the number of characters in the string, but the number of UTF-16 code units in the string.
function stringToUTF32(str, outPtr) {
  var iChar = 0;
  for(var iCodeUnit = 0; iCodeUnit < str.length; ++iCodeUnit) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    var codeUnit = str.charCodeAt(iCodeUnit); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++iCodeUnit);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[(((outPtr)+(iChar*4))>>2)]=codeUnit;
    ++iChar;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[(((outPtr)+(iChar*4))>>2)]=0;
}
Module['stringToUTF32'] = stringToUTF32;

function demangle(func) {
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    return parse();
  } catch(e) {
    return func;
  }
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function stackTrace() {
  var stack = new Error().stack;
  return stack ? demangleAll(stack) : '(no stack trace available)'; // Stack trace is not available at least on IE10 and Safari 6.
}

// Memory management

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk

function enlargeMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with ALLOW_MEMORY_GROWTH which adjusts the size at runtime but prevents some optimizations, or (3) set Module.TOTAL_MEMORY before the program runs.');
}

var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;

var totalMemory = 4096;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be more reasonable');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;

function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
    Module.printErr('Exiting runtime. Any attempt to access the compiled C code may fail from now. If you want to keep the runtime alive, set Module["noExitRuntime"] = true or build with -s NO_EXIT_RUNTIME=1');
  }
  callRuntimeCallbacks(__ATEXIT__);
  runtimeInitialized = false;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools

// This processes a JS string into a C-line array of numbers, 0-terminated.
// For LLVM-originating strings, see parser.js:parseLLVMString function
function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var ret = (new Runtime.UTF8Processor()).processJSString(stringy);
  if (length) {
    ret.length = length;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
        assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

// Write a Javascript array to somewhere in the heap
function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[(((buffer)+(i))>>0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; i++) {
    assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
    HEAP8[(((buffer)+(i))>>0)]=str.charCodeAt(i);
  }
  if (!dontAddNull) HEAP8[(((buffer)+(str.length))>>0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(!runDependencyTracking[id]);
    runDependencyTracking[id] = 1;
    if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
      // Check for missing dependencies every few seconds
      runDependencyWatcher = setInterval(function() {
        if (ABORT) {
          clearInterval(runDependencyWatcher);
          runDependencyWatcher = null;
          return;
        }
        var shown = false;
        for (var dep in runDependencyTracking) {
          if (!shown) {
            shown = true;
            Module.printErr('still waiting on run dependencies:');
          }
          Module.printErr('dependency: ' + dep);
        }
        if (shown) {
          Module.printErr('(end of list)');
        }
      }, 10000);
    }
  } else {
    Module.printErr('warning: run dependency added without ID');
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(runDependencyTracking[id]);
    delete runDependencyTracking[id];
  } else {
    Module.printErr('warning: run dependency removed without ID');
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data


var memoryInitializer = null;

// === Body ===





STATIC_BASE = 8;

STATICTOP = STATIC_BASE + Runtime.alignMemory(77955);
  /* global initializers */ __ATINIT__.push();
  

/* memory initializer */ allocate([0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,109,97,105,110,32,58,32,0,103,114,97,121,115,99,97,108,101,32,105,109,97,103,101,32,114,101,99,111,114,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,37,115,10,0,37,115,46,37,115,0,0,0,109,100,116,0,0,0,0,0,98,114,119,0,0,0,0,0,45,118,101,114,115,105,111,110,0,0,0,0,0,0,0,0,83,116,97,110,100,97,114,100,32,86,101,114,115,105,111,110,58,32,37,115,10,78,66,73,83,32,78,111,110,45,69,120,112,111,114,116,32,67,111,110,116,114,111,108,32,83,111,102,116,119,97,114,101,32,86,101,114,115,105,111,110,58,32,37,115,10,0,0,0,0,0,0,65,78,83,73,47,78,73,83,84,45,73,84,76,32,49,45,50,48,48,55,0,0,0,0,82,101,108,101,97,115,101,32,52,46,49,46,48,0,0,0,45,98,0,0,0,0,0,0,45,109,49,0,0,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,102,108,97,103,32,34,37,115,34,10,0,85,115,97,103,101,32,58,32,37,115,32,91,45,98,93,32,91,45,109,49,93,32,60,102,105,110,103,101,114,95,105,109,103,95,105,110,62,32,60,111,114,111,111,116,62,10,0,0,32,32,32,32,32,32,32,32,45,98,32,32,61,32,99,111,110,116,114,97,115,116,32,98,111,111,115,116,32,105,109,97,103,101,10,0,0,0,0,0,32,32,32,32,32,32,32,32,45,109,49,32,61,32,111,117,116,112,117,116,32,34,42,46,120,121,116,34,32,97,99,99,111,114,100,105,110,103,32,116,111,32,65,78,83,73,32,73,78,67,73,84,83,32,51,55,56,45,50,48,48,52,10,0,73,110,118,97,108,105,100,32,110,117,109,98,101,114,32,111,102,32,97,114,103,117,109,101,110,116,115,32,111,110,32,99,111,109,109,97,110,100,32,108,105,110,101,10,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,109,105,110,117,116,105,97,101,32,58,32,105,110,112,117,116,32,105,109,97,103,101,32,112,105,120,101,108,32,0,0,0,0,0,0,0,100,101,112,116,104,32,61,32,37,100,32,33,61,32,56,46,10,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,64,0,0,0,0,0,0,8,64,0,0,0,0,0,0,16,64,128,0,0,0,1,0,0,0,8,0,0,0,24,0,0,0,8,0,0,0,16,0,0,0,24,45,68,84,251,33,249,63,3,0,0,0,0,0,0,0,154,153,153,153,153,153,201,63,3,0,0,0,7,0,0,0,7,0,0,0,5,0,0,0,5,0,0,0,2,0,0,0,10,0,0,0,5,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,106,248,64,102,102,102,102,102,102,14,64,0,0,0,0,132,215,135,65,2,0,0,0,0,0,0,0,102,102,102,102,102,102,230,63,0,0,0,0,0,0,232,63,7,0,0,0,9,0,0,0,0,0,0,0,3,0,0,0,10,0,0,0,0,0,0,0,101,115,45,56,82,193,240,63,14,0,0,0,20,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,2,64,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,30,0,0,0,30,0,0,0,4,0,0,0,15,0,0,0,7,0,0,0,4,0,0,0,7,0,0,0,8,0,0,0,6,0,0,0,10,0,0,0,20,0,0,0,0,0,0,0,0,0,0,64,20,0,0,0,3,0,0,0,12,0,0,0,10,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,0,0,224,63,0,0,0,0,0,0,2,64,5,0,0,0,10,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,255,255,255,255,3,0,0,0,2,0,0,0,1,0,0,0,4,0,0,0,255,255,255,255,0,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,109,105,110,117,116,105,97,101,32,58,32,109,97,108,108,111,99,32,58,32,109,105,110,117,116,105,97,101,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,109,105,110,117,116,105,97,101,32,58,32,109,97,108,108,111,99,32,58,32,109,105,110,117,116,105,97,101,45,62,108,105,115,116,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,108,108,111,99,95,109,105,110,117,116,105,97,101,32,58,32,114,101,97,108,108,111,99,32,58,32,109,105,110,117,116,105,97,101,45,62,108,105,115,116,10,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,109,105,110,117,116,105,97,101,95,121,95,120,32,58,32,109,97,108,108,111,99,32,58,32,114,97,110,107,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,109,105,110,117,116,105,97,101,95,121,95,120,32,58,32,109,97,108,108,111,99,32,58,32,110,101,119,108,105,115,116,10,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,109,105,110,117,116,105,97,101,95,120,95,121,32,58,32,109,97,108,108,111,99,32,58,32,114,97,110,107,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,109,105,110,117,116,105,97,101,95,120,95,121,32,58,32,109,97,108,108,111,99,32,58,32,110,101,119,108,105,115,116,10,0,0,0,10,37,100,32,77,105,110,117,116,105,97,101,32,68,101,116,101,99,116,101,100,10,10,0,37,52,100,32,58,32,37,52,100,44,32,37,52,100,32,58,32,37,50,100,32,58,32,37,54,46,51,102,32,58,0,0,82,73,71,32,58,32,0,0,66,73,70,32,58,32,0,0,65,80,80,32,58,32,0,0,68,73,83,32,58,32,0,0,37,50,100,32,0,0,0,0,58,32,37,52,100,44,37,52,100,59,32,37,50,100,32,0,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,99,114,101,97,116,101,95,109,105,110,117,116,105,97,32,58,32,109,97,108,108,111,99,32,58,32,109,105,110,117,116,105,97,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,109,105,110,117,116,105,97,32,58,32,105,110,100,101,120,32,111,117,116,32,111,102,32,114,97,110,103,101,10,0,0,0,0,69,82,82,79,82,32,58,32,105,115,95,109,105,110,117,116,105,97,95,97,112,112,101,97,114,105,110,103,32,58,32,98,97,100,32,99,111,110,102,105,103,117,114,97,116,105,111,110,32,111,102,32,112,105,120,101,108,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,110,95,113,117,97,108,105,116,121,95,109,97,112,32,58,32,109,97,108,108,111,99,32,58,32,81,117,97,108,77,97,112,10,0,0,0,0,0,69,82,82,79,82,32,58,32,99,111,109,98,105,110,101,100,95,109,105,117,116,105,97,95,113,117,97,108,105,116,121,32,58,32,0,0,0,0,0,0,105,109,97,103,101,32,109,117,115,116,32,112,105,120,101,108,32,100,101,112,116,104,32,61,32,37,100,32,109,117,115,116,32,98,101,32,56,32,0,0,116,111,32,99,111,109,112,117,116,101,32,114,101,108,105,97,98,105,108,105,116,121,10,0,117,110,101,120,112,101,99,116,101,100,32,113,117,97,108,105,116,121,32,109,97,112,32,118,97,108,117,101,32,37,100,32,0,0,0,0,0,0,0,0,110,111,116,32,105,110,32,114,97,110,103,101,32,91,48,46,46,52,93,10,0,0,0,0,37,115,46,37,115,0,0,0,109,105,110,0,0,0,0,0,119,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,116,101,120,116,95,114,101,115,117,108,116,115,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,0,0,0,0,0,73,109,97,103,101,32,40,119,44,104,41,32,37,100,32,37,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,116,101,120,116,95,114,101,115,117,108,116,115,32,58,32,102,99,108,111,115,101,32,58,32,37,115,10,0,0,0,0,0,0,0,120,121,116,0,0,0,0,0,113,109,0,0,0,0,0,0,100,109,0,0,0,0,0,0,108,99,109,0,0,0,0,0,108,102,109,0,0,0,0,0,104,99,109,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,109,105,110,117,116,105,97,101,95,88,89,84,81,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,109,105,110,117,116,105,97,101,95,88,89,84,81,32,58,32,0,0,73,110,118,97,108,105,100,32,88,89,84,32,114,101,112,114,101,115,101,110,116,97,116,105,111,110,32,116,121,112,101,32,61,32,37,100,10,0,0,0,37,100,32,37,100,32,37,100,32,37,100,10,0,0,0,0,37,50,100,32,0,0,0,0,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,105,110,100,105,99,101,115,95,105,110,116,95,105,110,99,32,58,32,109,97,108,108,111,99,32,58,32,111,114,100,101,114,10,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,108,102,115,95,114,101,115,117,108,116,115,32,58,32,0,105,109,97,103,101,32,112,105,120,101,108,32,100,101,112,116,104,32,61,32,37,100,32,33,61,32,123,49,44,56,44,50,52,125,10,0,0,0,0,0,78,79,78,69,0,0,0,0,78,73,83,84,95,76,70,83,95,86,69,82,50,0,0,0,69,82,82,79,82,32,58,32,109,105,110,109,97,120,115,32,58,32,109,97,108,108,111,99,32,58,32,109,105,110,109,97,120,95,118,97,108,10,0,0,69,82,82,79,82,32,58,32,109,105,110,109,97,120,115,32,58,32,109,97,108,108,111,99,32,58,32,109,105,110,109,97,120,95,116,121,112,101,10,0,69,82,82,79,82,32,58,32,109,105,110,109,97,120,115,32,58,32,109,97,108,108,111,99,32,58,32,109,105,110,109,97,120,95,105,10,0,0,0,0,24,45,68,84,251,33,25,64,69,82,82,79,82,32,58,32,98,108,111,99,107,95,111,102,102,115,101,116,115,32,58,32,105,109,97,103,101,32,109,117,115,116,32,98,101,32,97,116,32,108,101,97,115,116,32,37,100,32,98,121,32,37,100,32,105,110,32,115,105,122,101,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,98,108,111,99,107,95,111,102,102,115,101,116,115,32,58,32,109,97,108,108,111,99,32,58,32,98,108,107,111,102,102,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,108,111,119,95,99,111,110,116,114,97,115,116,95,98,108,111,99,107,32,58,32,109,105,110,32,112,101,114,99,101,110,116,105,108,101,32,112,105,120,101,108,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,69,82,82,79,82,32,58,32,108,111,119,95,99,111,110,116,114,97,115,116,95,98,108,111,99,107,32,58,32,109,97,120,32,112,101,114,99,101,110,116,105,108,101,32,112,105,120,101,108,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,97,116,101,95,99,111,110,116,111,117,114,32,58,32,109,97,108,108,111,99,32,58,32,99,111,110,116,111,117,114,95,120,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,97,116,101,95,99,111,110,116,111,117,114,32,58,32,109,97,108,108,111,99,32,58,32,99,111,110,116,111,117,114,95,121,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,97,116,101,95,99,111,110,116,111,117,114,32,58,32,109,97,108,108,111,99,32,58,32,99,111,110,116,111,117,114,95,101,120,10,0,69,82,82,79,82,32,58,32,97,108,108,111,99,97,116,101,95,99,111,110,116,111,117,114,32,58,32,109,97,108,108,111,99,32,58,32,99,111,110,116,111,117,114,95,101,121,10,0,10,73,78,73,84,73,65,76,73,90,65,84,73,79,78,32,65,78,68,32,80,65,68,68,73,78,71,32,68,79,78,69,10,0,0,0,0,0,0,0,10,66,73,78,65,82,73,90,65,84,73,79,78,32,68,79,78,69,10,0,0,0,0,0,10,77,73,78,85,84,73,65,32,68,69,84,69,67,84,73,79,78,32,68,79,78,69,10,0,0,0,0,0,0,0,0,10,78,69,73,71,72,66,79,82,32,82,73,68,71,69,32,67,79,85,78,84,32,68,79,78,69,10,0,0,0,0,0,69,82,82,79,82,32,58,32,108,102,115,95,100,101,116,101,99,116,95,109,105,110,117,116,105,97,101,95,86,50,32,58,32,109,97,108,108,111,99,32,58,32,112,100,97,116,97,10,0,0,0,0,0,0,0,0,10,77,65,80,83,32,68,79,78,69,10,0,0,0,0,0,69,82,82,79,82,32,58,32,108,102,115,95,100,101,116,101,99,116,95,109,105,110,117,116,105,97,101,95,86,50,32,58,0,0,0,0,0,0,0,0,98,105,110,97,114,121,32,105,109,97,103,101,32,104,97,115,32,98,97,100,32,100,105,109,101,110,115,105,111,110,115,32,58,32,37,100,44,32,37,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,112,97,100,95,117,99,104,97,114,95,105,109,97,103,101,32,58,32,109,97,108,108,111,99,32,58,32,112,100,97,116,97,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,105,114,50,114,97,100,32,58,32,109,97,108,108,111,99,32,58,32,100,105,114,50,114,97,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,105,114,50,114,97,100,32,58,32,109,97,108,108,111,99,32,58,32,100,105,114,50,114,97,100,45,62,99,111,115,10,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,105,114,50,114,97,100,32,58,32,109,97,108,108,111,99,32,58,32,100,105,114,50,114,97,100,45,62,115,105,110,10,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,102,116,119,97,118,101,115,32,58,32,109,97,108,108,111,99,32,58,32,100,102,116,119,97,118,101,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,102,116,119,97,118,101,115,32,58,32,109,97,108,108,111,99,32,58,32,100,102,116,119,97,118,101,115,45,62,119,97,118,101,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,102,116,119,97,118,101,115,32,58,32,109,97,108,108,111,99,32,58,32,100,102,116,119,97,118,101,115,45,62,119,97,118,101,115,91,105,93,10,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,102,116,119,97,118,101,115,32,58,32,109,97,108,108,111,99,32,58,32,100,102,116,119,97,118,101,115,45,62,119,97,118,101,115,91,105,93,45,62,99,111,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,100,102,116,119,97,118,101,115,32,58,32,109,97,108,108,111,99,32,58,32,100,102,116,119,97,118,101,115,45,62,119,97,118,101,115,91,105,93,45,62,115,105,110,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,114,111,116,103,114,105,100,115,32,58,32,109,97,108,108,111,99,32,58,32,114,111,116,103,114,105,100,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,114,111,116,103,114,105,100,115,32,58,32,73,108,108,101,103,97,108,32,114,101,108,97,116,105,118,101,32,102,108,97,103,32,58,32,37,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,114,111,116,103,114,105,100,115,32,58,32,80,97,100,32,112,97,115,115,101,100,32,105,115,32,116,111,111,32,115,109,97,108,108,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,114,111,116,103,114,105,100,115,32,58,32,109,97,108,108,111,99,32,58,32,114,111,116,103,114,105,100,115,45,62,103,114,105,100,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,105,116,95,114,111,116,103,114,105,100,115,32,58,32,109,97,108,108,111,99,32,58,32,114,111,116,103,114,105,100,115,45,62,103,114,105,100,115,91,100,105,114,93,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,100,105,114,95,112,111,119,101,114,115,32,58,32,109,97,108,108,111,99,32,58,32,112,111,119,101,114,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,100,105,114,95,112,111,119,101,114,115,32,58,32,109,97,108,108,111,99,32,58,32,112,111,119,101,114,115,91,119,93,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,112,111,119,101,114,95,115,116,97,116,115,32,58,32,109,97,108,108,111,99,32,58,32,119,105,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,112,111,119,101,114,95,115,116,97,116,115,32,58,32,109,97,108,108,111,99,32,58,32,112,111,119,109,97,120,115,10,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,112,111,119,101,114,95,115,116,97,116,115,32,58,32,109,97,108,108,111,99,32,58,32,112,111,119,109,97,120,95,100,105,114,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,112,111,119,101,114,95,115,116,97,116,115,32,58,32,109,97,108,108,111,99,32,58,32,112,111,119,110,111,114,109,115,10,0,0,69,82,82,79,82,32,58,32,108,105,110,101,95,112,111,105,110,116,115,32,58,32,109,97,108,108,111,99,32,58,32,120,95,108,105,115,116,10,0,0,69,82,82,79,82,32,58,32,108,105,110,101,95,112,111,105,110,116,115,32,58,32,109,97,108,108,111,99,32,58,32,121,95,108,105,115,116,10,0,0,69,82,82,79,82,32,58,32,108,105,110,101,95,112,111,105,110,116,115,32,58,32,99,111,111,114,100,32,108,105,115,116,32,111,118,101,114,102,108,111,119,10,0,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,102,105,108,108,95,108,111,111,112,32,58,32,117,110,101,120,112,101,99,116,101,100,32,115,104,97,112,101,44,32,112,114,101,101,109,112,116,105,110,103,32,108,111,111,112,32,102,105,108,108,10,0,0,0,69,82,82,79,82,32,58,32,103,101,110,95,105,109,97,103,101,95,109,97,112,115,32,58,32,68,70,84,32,103,114,105,100,115,32,109,117,115,116,32,98,101,32,115,113,117,97,114,101,10,0,0,0,0,0,0,73,78,73,84,73,65,76,32,77,65,80,10,0,0,0,0,69,82,82,79,82,32,58,32,103,101,110,95,105,110,105,116,105,97,108,95,109,97,112,115,32,58,32,109,97,108,108,111,99,32,58,32,100,105,114,101,99,116,105,111,110,95,109,97,112,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,110,95,105,110,105,116,105,97,108,95,109,97,112,115,32,58,32,109,97,108,108,111,99,32,58,32,108,111,119,95,99,111,110,116,114,97,115,116,95,109,97,112,10,0,0,0,69,82,82,79,82,32,58,32,103,101,110,95,105,110,105,116,105,97,108,95,109,97,112,115,32,58,32,109,97,108,108,111,99,32,58,32,108,111,119,95,102,108,111,119,95,109,97,112,10,0,0,0,0,0,0,0,32,32,32,66,76,79,67,75,32,37,50,100,32,40,37,50,100,44,32,37,50,100,41,32,0,0,0,0,0,0,0,0,76,79,87,32,67,79,78,84,82,65,83,84,10,0,0,0,10,0,0,0,0,0,0,0,73,78,84,69,82,80,79,76,65,84,69,32,68,73,82,69,67,84,73,79,78,32,77,65,80,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,116,101,114,112,111,108,97,116,101,95,100,105,114,101,99,116,105,111,110,95,109,97,112,32,58,32,109,97,108,108,111,99,32,58,32,111,109,97,112,10,0,0,0,0,0,0,32,32,32,66,108,111,99,107,32,37,100,44,37,100,32,73,78,84,69,82,80,32,110,117,109,110,98,115,61,37,100,32,110,101,119,100,105,114,61,37,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,109,111,114,112,104,95,84,70,95,109,97,112,32,58,32,109,97,108,108,111,99,32,58,32,99,105,109,97,103,101,10,0,69,82,82,79,82,32,58,32,109,111,114,112,104,95,84,70,95,109,97,112,32,58,32,109,97,108,108,111,99,32,58,32,109,105,109,97,103,101,10,0,69,82,82,79,82,32,58,32,112,105,120,101,108,105,122,101,95,109,97,112,32,58,32,109,97,108,108,111,99,32,58,32,112,109,97,112,10,0,0,0,69,82,82,79,82,32,58,32,112,105,120,101,108,105,122,101,95,109,97,112,32,58,32,98,108,111,99,107,32,100,105,109,101,110,115,105,111,110,115,32,100,111,32,110,111,116,32,109,97,116,99,104,10,0,0,0,83,77,79,79,84,72,32,68,73,82,69,67,84,73,79,78,32,77,65,80,10,0,0,0,69,82,82,79,82,58,32,103,101,110,95,104,105,103,104,95,99,117,114,118,101,95,109,97,112,32,58,32,109,97,108,108,111,99,32,58,32,104,105,103,104,95,99,117,114,118,101,95,109,97,112,10,0,0,0,0,32,32,32,32,32,32,80,114,105,109,97,114,121,10,0,0,32,32,32,32,32,32,32,32,32,76,101,102,116,32,61,32,37,100,44,32,67,117,114,114,101,110,116,32,61,32,37,100,44,32,82,105,103,104,116,32,61,32,37,100,10,0,0,0,10,82,69,77,79,86,73,78,71,32,72,79,76,69,83,58,10,0,0,0,0,0,0,0,37,100,44,37,100,32,82,77,10,0,0,0,0,0,0,0,10,82,69,77,79,86,73,78,71,32,72,79,79,75,83,58,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,104,111,111,107,115,32,58,32,99,97,108,108,111,99,32,58,32,116,111,95,114,101,109,111,118,101,10,0,0,0,0,0,0,10,0,0,0,0,0,0,0,49,58,37,100,40,37,100,44,37,100,41,37,100,32,50,58,37,100,40,37,100,44,37,100,41,37,100,32,0,0,0,0,49,68,89,32,0,0,0,0,50,68,83,32,0,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,104,111,111,107,115,32,58,32,73,78,86,65,76,73,68,32,100,105,114,101,99,116,105,111,110,10,0,0,0,0,0,0,0,51,68,68,32,0,0,0,0,52,72,75,32,82,77,10,0,82,77,10,0,0,0,0,0,52,73,76,32,82,77,10,0,52,79,86,32,82,77,10,0,10,82,69,77,79,86,73,78,71,32,73,83,76,65,78,68,83,32,65,78,68,32,76,65,75,69,83,58,10,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,105,115,108,97,110,100,115,95,97,110,100,95,108,97,107,101,115,32,58,32,99,97,108,108,111,99,32,58,32,116,111,95,114,101,109,111,118,101,10,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,105,115,108,97,110,100,115,95,97,110,100,95,108,97,107,101,115,32,58,32,73,78,86,65,76,73,68,32,100,105,114,101,99,116,105,111,110,10,0,0,0,10,82,69,77,79,86,73,78,71,32,77,65,76,70,79,82,77,65,84,73,79,78,83,58,10,0,0,0,0,0,0,0,37,100,44,37,100,32,82,77,65,10,0,0,0,0,0,0,37,100,44,37,100,32,82,77,66,10,0,0,0,0,0,0,37,100,44,37,100,32,82,77,77,65,76,49,10,0,0,0,37,100,44,37,100,32,82,77,77,65,76,50,10,0,0,0,37,100,44,37,100,32,82,77,77,65,76,51,32,40,37,102,41,10,0,0,0,0,0,0,10,82,69,77,79,86,73,78,71,32,77,73,78,85,84,73,65,32,78,69,65,82,32,73,78,86,65,76,73,68,32,66,76,79,67,75,83,58,10,0,37,100,44,37,100,32,82,77,49,10,0,0,0,0,0,0,37,100,44,37,100,32,82,77,50,10,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,255,255,255,255,2,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,2,0,0,0,6,0,0,0,255,255,255,255,2,0,0,0,6,0,0,0,4,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,0,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,110,101,97,114,95,105,110,118,98,108,111,99,107,95,86,50,32,58,32,109,97,114,103,105,110,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,98,108,111,99,107,115,105,122,101,10,0,0,0,0,0,0,0,10,82,69,77,79,86,73,78,71,32,77,73,78,85,84,73,65,32,80,79,73,78,84,73,78,71,32,84,79,32,73,78,86,65,76,73,68,32,66,76,79,67,75,83,58,10,0,0,10,82,69,77,79,86,73,78,71,32,79,86,69,82,76,65,80,83,58,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,111,118,101,114,108,97,112,115,32,58,32,99,97,108,108,111,99,32,58,32,116,111,95,114,101,109,111,118,101,10,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,111,118,101,114,108,97,112,115,32,58,32,73,78,86,65,76,73,68,32,100,105,114,101,99,116,105,111,110,10,0,0,0,0,106,111,105,110,100,105,114,61,37,100,32,100,105,115,116,61,37,102,32,0,0,0,0,0,10,82,69,77,79,86,73,78,71,32,80,79,82,69,83,58,10,0,0,0,0,0,0,0,37,100,44,37,100,32,82,77,68,10,0,0,0,0,0,0,37,100,44,37,100,32,82,77,67,10,0,0,0,0,0,0,37,100,44,37,100,32,0,0,82,61,37,100,44,37,100,32,80,61,37,100,44,37,100,32,66,61,37,100,44,37,100,32,68,61,37,100,44,37,100,32,81,61,37,100,44,37,100,32,65,61,37,100,44,37,100,32,67,61,37,100,44,37,100,32,0,0,0,0,0,0,0,0,37,100,44,37,100,32,82,77,81,10,0,0,0,0,0,0,37,100,44,37,100,32,82,77,80,10,0,0,0,0,0,0,82,77,82,65,84,73,79,32,37,102,10,0,0,0,0,0,10,65,68,74,85,83,84,73,78,71,32,83,73,68,69,32,77,73,78,85,84,73,65,58,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,109,111,118,101,95,111,114,95,97,100,106,117,115,116,95,115,105,100,101,95,109,105,110,117,116,105,97,101,95,86,50,32,58,32,109,97,108,108,111,99,32,58,32,114,111,116,95,121,10,0,0,0,0,0,82,77,50,10,0,0,0,0,65,68,49,32,37,100,44,37,100,10,0,0,0,0,0,0,82,77,51,10,0,0,0,0,65,68,50,32,37,100,44,37,100,10,0,0,0,0,0,0,37,100,44,37,100,32,82,77,52,10,0,0,0,0,0,0,10,70,73,78,68,73,78,71,32,78,66,82,83,32,65,78,68,32,67,79,85,78,84,73,78,71,32,82,73,68,71,69,83,58,10,0,0,0,0,0,78,66,82,83,32,70,79,85,78,68,58,32,37,100,44,37,100,32,61,32,37,100,10,0,69,82,82,79,82,32,58,32,99,111,117,110,116,95,109,105,110,117,116,105,97,95,114,105,100,103,101,115,32,58,32,109,97,108,108,111,99,32,58,32,110,98,114,95,110,114,105,100,103,101,115,10,0,0,0,0,69,82,82,79,82,32,58,32,102,105,110,100,95,110,101,105,103,104,98,111,114,115,32,58,32,109,97,108,108,111,99,32,58,32,110,98,114,95,108,105,115,116,10,0,0,0,0,0,69,82,82,79,82,32,58,32,102,105,110,100,95,110,101,105,103,104,98,111,114,115,32,58,32,109,97,108,108,111,99,32,58,32,110,98,114,95,115,113,114,95,100,105,115,116,115,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,110,98,114,95,100,105,115,116,115,32,58,32,105,108,108,101,103,97,108,32,112,111,115,105,116,105,111,110,32,102,111,114,32,110,101,119,32,110,101,105,103,104,98,111,114,10,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,110,101,105,103,104,98,111,114,32,58,32,105,110,115,101,114,116,105,111,110,32,112,111,105,110,116,32,101,120,99,101,101,100,115,32,108,105,115,116,115,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,110,101,105,103,104,98,111,114,32,58,32,111,118,101,114,102,108,111,119,32,105,110,32,110,101,105,103,104,98,111,114,32,108,105,115,116,115,10,0,0,0,24,45,68,84,251,33,25,64,69,82,82,79,82,32,58,32,115,111,114,116,95,110,101,105,103,104,98,111,114,115,32,58,32,109,97,108,108,111,99,32,58,32,106,111,105,110,95,116,104,101,116,97,115,10,0,0,82,73,68,71,69,32,67,79,85,78,84,58,32,37,100,44,37,100,32,116,111,32,37,100,44,37,100,32,0,0,0,0,10,0,0,0,0,0,0,0,58,32,82,83,32,37,100,44,37,100,32,0,0,0,0,0,59,32,82,69,32,37,100,44,37,100,32,0,0,0,0,0,59,32,86,37,100,32,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,115,104,97,112,101,32,58,32,109,97,108,108,111,99,32,58,32,115,104,97,112,101,10,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,115,104,97,112,101,32,58,32,109,97,108,108,111,99,32,58,32,115,104,97,112,101,45,62,114,111,119,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,115,104,97,112,101,32,58,32,109,97,108,108,111,99,32,58,32,115,104,97,112,101,45,62,114,111,119,115,91,105,93,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,115,104,97,112,101,32,58,32,109,97,108,108,111,99,32,58,32,115,104,97,112,101,45,62,114,111,119,115,91,105,93,45,62,120,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,115,104,97,112,101,95,102,114,111,109,95,99,111,110,116,111,117,114,32,58,32,114,111,119,32,111,118,101,114,102,108,111,119,10,0,0,0,0,0,0,48,0,0,0,0,0,0,0,37,48,50,100,0,0,0,0,52,0,0,0,0,0,0,0,83,0,0,0,0,0,0,0,78,73,83,84,95,76,70,83,95,86,69,82,50,0,0,0,65,0,0,0,0,0,0,0,48,48,0,0,0,0,0,0,84,0,0,0,0,0,0,0,85,78,0,0,0,0,0,0,37,100,0,0,0,0,0,0,49,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,109,105,110,117,116,105,97,101,50,102,105,101,108,100,95,49,50,32,58,32,0,0,0,0,110,111,32,109,105,110,117,116,105,97,101,32,114,101,99,111,114,100,101,100,10,0,0,0,37,48,51,100,0,0,0,0,37,48,52,100,37,48,52,100,37,48,51,100,0,0,0,0,109,105,110,117,116,105,97,32,37,100,32,88,88,88,88,89,89,89,89,84,84,84,32,118,97,108,117,101,32,61,32,37,115,32,62,32,49,49,32,99,104,97,114,115,10,0,0,0,66,0,0,0,0,0,0,0,117,110,115,117,112,112,111,114,116,101,100,32,109,105,110,117,116,105,97,32,116,121,112,101,32,61,32,37,100,10,0,0,37,48,51,100,44,37,48,50,100,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,98,105,110,97,114,105,122,101,95,105,109,97,103,101,95,86,50,32,58,32,109,97,108,108,111,99,32,58,32,98,100,97,116,97,10,0,0,0,0,0,69,82,82,79,82,32,58,32,99,104,97,105,110,95,99,111,100,101,95,108,111,111,112,32,58,32,109,97,108,108,111,99,32,58,32,99,104,97,105,110,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,102,116,95,100,105,114,95,112,111,119,101,114,115,32,58,32,68,70,84,32,103,114,105,100,115,32,109,117,115,116,32,98,101,32,115,113,117,97,114,101,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,102,116,95,100,105,114,95,112,111,119,101,114,115,32,58,32,109,97,108,108,111,99,32,58,32,114,111,119,115,117,109,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,100,102,116,95,119,97,118,101,115,32,58,32,109,97,108,108,111,99,32,58,32,112,111,119,110,111,114,109,115,50,10,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,32,58,32,109,97,108,108,111,99,32,58,32,97,110,115,105,95,110,105,115,116,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,32,58,32,109,97,108,108,111,99,32,58,32,37,100,32,114,101,99,111,114,100,115,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,32,58,32,109,97,108,108,111,99,32,58,32,114,101,99,111,114,100,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,32,58,32,109,97,108,108,111,99,32,58,32,37,100,32,102,105,101,108,100,115,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,110,101,119,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,99,97,108,108,111,99,32,58,32,102,105,101,108,100,45,62,105,100,32,91,37,100,46,37,48,51,100,93,32,40,37,100,32,98,121,116,101,115,41,10,0,0,37,100,46,37,48,51,100,58,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,110,101,119,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,115,110,112,114,105,110,116,102,32,37,100,32,98,121,116,101,32,115,116,114,105,110,103,32,111,118,101,114,102,108,111,119,115,32,37,100,32,98,121,116,101,32,98,117,102,102,101,114,10,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,109,97,108,108,111,99,32,58,32,102,105,101,108,100,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,109,97,108,108,111,99,32,58,32,37,100,32,115,117,98,102,105,101,108,100,115,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,115,117,98,102,105,101,108,100,32,58,32,109,97,108,108,111,99,32,58,32,115,117,98,102,105,101,108,100,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,115,117,98,102,105,101,108,100,32,58,32,109,97,108,108,111,99,32,58,32,37,100,32,105,116,101,109,115,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,109,97,108,108,111,99,32,58,32,105,116,101,109,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,109,97,108,108,111,99,32,58,32,105,116,101,109,45,62,118,97,108,117,101,32,40,37,100,32,98,121,116,101,115,41,10,0,114,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,108,101,32,58,32,102,111,112,101,110,32,39,37,115,39,58,32,37,115,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,108,101,32,58,32,102,99,108,111,115,101,32,39,37,115,39,58,32,37,115,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,108,101,32,58,32,101,120,116,114,97,32,100,97,116,97,32,115,116,97,114,116,105,110,103,32,97,116,32,37,115,58,37,108,117,10,0,0,0,0,69,82,82,79,82,32,58,32,112,97,114,115,101,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,99,97,108,108,111,99,32,102,105,101,108,100,95,105,100,32,40,37,100,32,98,121,116,101,115,41,10,0,0,119,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,32,58,32,102,99,108,111,115,101,32,58,32,37,115,10,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,32,58,117,110,107,111,119,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,116,97,103,103,101,100,95,102,105,101,108,100,32,58,102,105,101,108,100,32,73,68,32,101,109,112,116,121,32,105,110,32,116,97,103,103,101,100,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,102,119,114,105,116,101,32,58,32,105,100,32,61,32,37,115,44,32,37,115,10,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,116,97,103,103,101,100,95,105,116,101,109,32,58,32,102,119,114,105,116,101,32,58,32,119,114,111,116,101,32,111,110,108,121,32,37,100,32,111,102,32,37,100,32,105,116,101,109,32,98,121,116,101,115,44,32,37,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,115,101,112,97,114,97,116,111,114,32,58,32,105,108,108,101,103,97,108,32,115,101,112,97,114,97,116,111,114,32,61,32,48,120,37,48,50,120,10,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,115,101,112,97,114,97,116,111,114,32,58,32,102,119,114,105,116,101,32,58,32,48,120,37,48,50,120,44,32,37,115,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,110,111,32,98,121,116,101,115,32,105,110,32,105,116,101,109,32,111,102,32,115,105,122,101,32,37,100,10,0,0,0,0,0,0,37,117,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,115,115,99,97,110,102,32,58,32,102,97,105,108,101,100,32,116,111,32,112,97,114,115,101,32,117,105,110,116,32,105,116,101,109,32,37,52,115,10,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,102,119,114,105,116,101,32,58,32,102,97,105,108,101,100,32,116,111,32,119,114,105,116,101,32,117,105,110,116,32,105,116,101,109,32,37,117,44,32,37,115,10,0,0,0,0,0,0,0,0,37,104,117,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,115,115,99,97,110,102,32,58,32,102,97,105,108,101,100,32,116,111,32,112,97,114,115,101,32,117,115,104,111,114,116,32,105,116,101,109,32,37,50,115,10,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,102,119,114,105,116,101,32,58,32,102,97,105,108,101,100,32,116,111,32,119,114,105,116,101,32,117,115,104,111,114,116,32,105,116,101,109,32,37,104,117,44,32,37,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,115,115,99,97,110,102,32,58,32,102,97,105,108,101,100,32,116,111,32,112,97,114,115,101,32,117,99,104,97,114,32,105,116,101,109,32,37,49,115,10,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,117,110,115,105,103,110,101,100,32,99,104,97,114,97,99,116,101,114,32,118,97,108,117,101,32,37,104,117,32,101,120,99,101,101,100,115,32,37,104,117,32,40,85,67,72,65,82,95,77,65,88,41,10,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,102,119,114,105,116,101,32,58,32,102,97,105,108,101,100,32,116,111,32,119,114,105,116,101,32,117,99,104,97,114,32,105,116,101,109,32,37,99,32,40,48,120,37,48,50,120,41,44,32,37,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,114,105,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,116,101,109,32,58,32,102,119,114,105,116,101,32,58,32,119,114,111,116,101,32,111,110,108,121,32,37,100,32,111,102,32,37,100,32,105,116,101,109,32,98,121,116,101,115,44,32,37,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,76,69,78,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,73,68,67,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,83,73,71,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,83,82,84,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,73,83,82,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,72,76,76,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,86,76,76,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,45,62,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,44,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,44,32,97,116,32,37,108,100,10,0,0,112,114,101,109,97,116,117,114,101,32,69,79,70,0,0,0,98,117,102,102,101,114,32,101,120,104,97,117,115,116,101,100,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,114,101,97,100,32,58,32,111,110,108,121,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,111,102,32,37,100,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,115,105,103,110,97,116,117,114,101,95,114,101,99,111,114,100,32,58,32,111,110,108,121,32,37,100,32,111,102,32,116,111,116,97,108,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,44,32,97,116,32,37,108,100,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,76,69,78,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,73,68,67,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,73,77,80,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,70,71,80,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,73,83,82,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,72,76,76,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,86,76,76,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,67,65,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,110,111,116,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,45,62,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,44,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,114,101,97,100,32,114,101,99,111,114,100,32,58,32,111,110,108,121,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,111,102,32,37,100,44,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,111,110,108,121,32,37,100,32,111,102,32,116,111,116,97,108,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,103,101,116,99,32,105,116,101,109,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,99,97,108,108,111,99,32,58,32,102,105,101,108,100,95,105,100,32,40,37,100,32,98,121,116,101,115,41,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,103,101,116,99,32,114,101,99,111,114,100,95,116,121,112,101,32,100,105,103,105,116,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,114,101,99,111,114,100,95,116,121,112,101,32,99,104,97,114,97,99,116,101,114,32,39,37,99,39,32,40,48,120,37,48,50,120,41,32,110,111,116,32,110,117,109,101,114,105,99,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,114,101,99,111,114,100,32,105,110,116,101,103,101,114,32,110,111,116,32,102,111,117,110,100,32,105,110,32,102,105,101,108,100,32,73,68,44,32,97,116,32,37,108,100,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,103,101,116,99,32,102,105,101,108,100,95,105,110,116,32,100,105,103,105,116,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,102,105,101,108,100,95,105,110,116,32,99,104,97,114,97,99,116,101,114,32,39,37,99,39,32,40,48,120,37,48,50,120,41,32,110,111,116,32,110,117,109,101,114,105,99,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,95,73,68,32,58,32,102,105,101,108,100,32,105,110,116,101,103,101,114,32,110,111,116,32,102,111,117,110,100,32,105,110,32,102,105,101,108,100,32,73,68,44,32,97,116,32,37,108,100,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,95,102,105,101,108,100,32,58,32,114,101,109,97,105,110,105,110,103,32,98,121,116,101,115,32,105,110,32,114,101,99,111,114,100,32,117,110,115,112,101,99,105,102,105,101,100,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,95,102,105,101,108,100,32,58,32,116,111,111,32,102,101,119,32,98,121,116,101,115,32,37,100,32,114,101,109,97,105,110,105,110,103,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,44,32,105,109,97,103,101,32,115,105,122,101,32,37,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,95,102,105,101,108,100,32,58,32,114,101,97,108,108,111,99,32,58,32,105,109,97,103,101,95,105,116,101,109,45,62,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,44,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,95,102,105,101,108,100,32,58,32,114,101,97,100,32,105,109,97,103,101,95,105,116,101,109,45,62,118,97,108,117,101,58,32,111,110,108,121,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,111,102,32,37,100,44,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,95,102,105,101,108,100,32,58,32,103,101,116,99,32,100,101,108,105,109,105,116,101,114,58,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,58,32,37,115,44,32,97,116,32,37,108,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,95,102,105,101,108,100,32,58,32,105,109,97,103,101,32,114,101,99,111,114,100,32,116,101,114,109,105,110,97,116,101,100,32,119,105,116,104,32,48,120,37,48,50,120,32,110,111,116,32,70,83,95,67,72,65,82,32,40,48,120,37,48,50,120,41,44,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,116,111,111,32,102,101,119,32,98,121,116,101,115,32,37,100,32,114,101,109,97,105,110,105,110,103,32,105,110,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,110,116,101,103,101,114,95,102,105,101,108,100,32,58,32,98,97,100,32,105,110,116,101,103,101,114,32,102,105,101,108,100,32,102,111,114,109,97,116,44,32,37,100,32,115,117,98,102,105,101,108,100,115,44,32,115,104,111,117,108,100,32,98,101,32,49,44,32,110,101,97,114,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,105,110,116,101,103,101,114,95,102,105,101,108,100,32,58,32,98,97,100,32,105,110,116,101,103,101,114,32,102,105,101,108,100,32,102,111,114,109,97,116,44,32,37,100,32,105,116,101,109,115,44,32,115,104,111,117,108,100,32,98,101,32,49,44,32,110,101,97,114,32,37,108,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,118,101,114,115,105,111,110,32,58,32,84,121,112,101,45,49,32,86,69,82,32,70,105,101,108,100,32,40,49,46,37,48,51,100,41,32,118,101,114,115,105,111,110,32,110,111,116,32,112,97,114,115,101,100,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,118,101,114,115,105,111,110,32,58,32,102,105,101,108,100,32,105,110,116,32,37,100,32,110,111,116,32,37,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,108,101,110,103,116,104,32,58,32,114,101,99,111,114,100,32,108,101,110,103,116,104,32,110,111,116,32,112,97,114,115,101,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,108,101,110,103,116,104,32,58,32,102,105,101,108,100,32,110,117,109,32,37,100,44,32,110,111,116,32,37,100,32,97,115,32,114,101,113,117,105,114,101,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,116,97,103,103,101,100,95,114,101,99,111,114,100,32,58,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,32,102,111,117,110,100,44,32,110,111,116,32,84,121,112,101,45,37,100,32,97,115,32,101,120,112,101,99,116,101,100,44,32,97,116,32,37,108,100,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,116,97,103,103,101,100,95,114,101,99,111,114,100,32,58,32,114,101,99,111,114,100,32,116,101,114,109,105,110,97,116,101,100,32,98,121,32,48,120,37,48,50,120,32,110,111,116,32,70,83,95,67,72,65,82,32,40,48,120,37,48,50,120,41,44,32,97,116,32,98,121,116,101,32,37,100,32,111,102,32,97,32,37,100,32,98,121,116,101,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,116,97,103,103,101,100,95,114,101,99,111,114,100,32,58,32,111,110,108,121,32,37,100,32,111,102,32,116,111,116,97,108,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,105,110,32,84,121,112,101,45,37,100,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,32,58,32,117,110,115,117,112,112,111,114,116,101,100,32,114,101,99,111,114,100,32,116,121,112,101,32,37,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,109,97,105,110,105,110,103,95,114,101,99,111,114,100,115,32,58,32,110,111,32,114,101,99,111,114,100,115,32,102,111,117,110,100,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,109,97,105,110,105,110,103,95,114,101,99,111,114,100,115,32,58,32,102,105,114,115,116,32,114,101,99,111,114,100,32,116,121,112,101,32,37,100,44,32,109,117,115,116,32,98,101,32,49,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,109,97,105,110,105,110,103,95,114,101,99,111,114,100,115,32,58,32,84,121,112,101,45,49,32,67,78,84,32,70,105,101,108,100,32,40,49,46,48,48,51,41,32,110,111,116,32,102,111,117,110,100,44,32,97,116,32,37,108,100,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,95,114,101,109,97,105,110,105,110,103,95,114,101,99,111,114,100,115,32,58,32,84,121,112,101,45,49,32,67,78,84,32,70,105,101,108,100,32,40,49,46,48,48,51,41,32,83,117,98,102,105,101,108,100,32,37,100,32,98,97,100,32,102,111,114,109,97,116,58,32,110,117,109,98,101,114,32,111,102,32,105,116,101,109,115,32,37,100,44,32,110,111,116,32,50,32,97,115,32,114,101,113,117,105,114,101,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,102,105,114,115,116,32,114,101,99,111,114,100,32,116,121,112,101,32,37,100,44,32,109,117,115,116,32,98,101,32,49,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,108,101,110,103,116,104,32,102,105,101,108,100,32,40,49,46,48,48,49,41,32,116,101,114,109,105,110,97,116,101,100,32,119,105,116,104,32,48,120,37,48,50,120,44,32,110,111,116,32,71,83,95,67,72,65,82,32,40,48,120,37,48,50,120,41,44,32,97,116,32,37,108,100,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,118,101,114,115,105,111,110,32,102,105,101,108,100,32,40,49,46,48,48,50,41,32,116,101,114,109,105,110,97,116,101,100,32,119,105,116,104,32,48,120,37,48,50,120,44,32,110,111,116,32,71,83,95,67,72,65,82,32,40,48,120,37,48,50,120,41,44,32,97,116,32,37,108,100,10,0,87,65,82,78,73,78,71,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,65,78,83,73,47,78,73,83,84,32,86,101,114,115,105,111,110,32,61,32,37,100,32,85,110,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,0,32,32,32,32,32,32,32,32,32,32,65,116,116,101,109,112,116,105,110,103,32,116,111,32,114,101,97,100,32,46,46,46,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,114,101,99,111,114,100,32,116,101,114,109,105,110,97,116,101,100,32,98,121,32,48,120,37,48,50,120,32,110,111,116,32,70,83,95,67,72,65,82,32,40,48,120,37,48,50,120,41,44,32,98,121,116,101,32,37,100,32,111,102,32,97,32,37,100,32,98,121,116,101,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,114,101,97,100,32,37,100,32,98,121,116,101,115,32,111,102,32,97,32,37,100,32,98,121,116,101,32,114,101,99,111,114,100,44,32,97,116,32,37,108,100,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,84,121,112,101,49,95,114,101,99,111,114,100,32,58,32,114,101,99,111,114,100,32,109,105,115,115,105,110,103,32,37,100,32,109,97,110,100,97,116,111,114,121,32,102,105,101,108,100,115,44,32,97,116,32,37,108,100,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,65,78,83,73,95,78,73,83,84,32,58,32,68,67,83,32,102,105,101,108,100,32,40,49,46,48,49,53,41,32,102,111,117,110,100,58,32,97,108,116,101,114,110,97,116,101,32,99,104,97,114,97,99,116,101,114,32,115,101,116,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,102,105,114,115,116,95,103,114,97,121,112,114,105,110,116,32,58,32,73,68,67,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,69,82,82,79,82,32,58,32,103,101,116,95,102,105,114,115,116,95,103,114,97,121,112,114,105,110,116,32,58,32,73,77,80,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,73,110,115,101,114,116,101,100,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,10,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,99,111,114,101,32,58,32,105,110,115,101,114,116,105,110,103,32,97,32,84,121,112,101,45,49,32,114,101,99,111,114,100,32,110,111,116,32,112,101,114,109,105,116,116,101,100,10,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,99,111,114,101,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,99,111,114,101,32,58,32,114,101,97,108,108,111,99,32,58,32,114,101,99,111,114,100,32,108,105,115,116,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,0,0,0,0,0,0,85,112,100,97,116,105,110,103,32,67,78,84,32,102,105,101,108,100,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,84,121,112,101,45,49,32,114,101,99,111,114,100,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,67,78,84,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,49,93,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,111,102,32,115,117,98,102,105,101,108,100,115,32,105,110,32,67,78,84,32,102,105,101,108,100,32,105,110,100,101,120,32,91,49,46,37,100,93,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,73,68,67,32,105,116,101,109,32,105,110,100,101,120,32,91,49,46,37,100,46,37,100,46,50,93,32,110,111,116,32,102,111,117,110,100,32,105,110,32,67,78,84,32,102,105,101,108,100,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,73,68,67,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,105,110,115,101,114,116,101,100,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,114,101,97,108,108,111,99,32,58,32,67,78,84,32,115,117,98,102,105,101,108,100,32,108,105,115,116,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,0,0,0,0,0,0,37,100,0,0,0,0,0,0,37,48,50,100,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,98,97,100,32,102,111,114,109,97,116,32,111,102,32,67,78,84,32,115,117,98,102,105,101,108,100,32,105,110,100,101,120,32,91,49,46,37,100,46,49,93,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,73,68,67,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,0,0,0,65,100,106,117,115,116,105,110,103,32,112,114,101,118,105,111,117,115,32,73,68,67,115,32,105,110,32,67,78,84,32,102,105,101,108,100,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,73,68,67,32,105,116,101,109,32,105,110,100,101,120,32,91,49,46,37,100,46,50,93,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,49,46,37,48,51,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,100,106,117,115,116,95,105,110,115,114,101,99,95,67,78,84,95,73,68,67,115,32,58,32,98,97,100,32,102,111,114,109,97,116,32,111,102,32,73,68,67,32,102,105,101,108,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,105,110,115,101,114,116,105,111,110,32,111,102,32,105,116,101,109,32,105,110,32,98,105,110,97,114,121,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,32,110,111,116,32,112,101,114,109,105,116,116,101,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,105,110,115,101,114,116,105,111,110,32,111,102,32,105,109,97,103,101,32,105,116,101,109,32,105,110,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,46,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,32,110,111,116,32,112,101,114,109,105,116,116,101,100,10,0,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,115,117,98,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,105,116,101,109,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,46,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,32,108,105,115,116,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,115,101,114,116,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,32,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,73,110,115,101,114,116,101,100,32,105,116,101,109,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,46,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,32,61,32,37,115,10,0,0,114,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,115,95,65,78,83,73,95,78,73,83,84,95,102,105,108,101,32,58,32,102,111,112,101,110,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,105,115,95,65,78,83,73,95,78,73,83,84,95,102,105,108,101,32,58,32,102,114,101,97,100,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,112,114,101,109,97,116,117,114,101,32,69,79,70,0,0,0,87,65,82,78,73,78,71,32,58,32,108,111,111,107,117,112,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,32,58,32,84,121,112,101,45,56,32,114,101,99,111,114,100,32,91,37,100,93,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,73,109,97,103,101,32,114,101,99,111,114,100,32,105,103,110,111,114,101,100,46,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,32,10,110,111,116,32,97,32,98,105,110,97,114,121,32,102,105,101,108,100,32,105,109,97,103,101,32,114,101,99,111,114,100,10,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,78,84,82,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,49,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,0,0,0,0,37,102,0,0,0,0,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,72,80,83,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,105,109,97,103,101,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,86,80,83,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,105,109,97,103,101,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,108,111,111,107,117,112,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,112,105,120,101,108,32,97,115,112,101,99,116,32,114,97,116,105,111,32,33,61,32,49,32,110,111,116,32,100,105,114,101,99,116,108,121,32,115,117,112,112,111,114,116,101,100,32,105,110,32,105,109,97,103,101,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,87,105,108,108,32,99,111,110,116,105,110,117,101,32,119,105,116,104,32,111,112,101,114,97,116,105,110,103,32,97,115,115,117,109,112,105,116,111,110,32,116,104,97,116,32,97,115,112,101,99,116,32,114,97,116,105,111,32,61,32,49,46,10,0,0,0,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,83,76,67,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,105,109,97,103,101,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,95,112,112,109,109,32,58,32,105,108,108,101,103,97,108,32,83,76,67,32,99,111,100,101,32,61,32,37,100,32,105,110,32,105,109,97,103,101,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,95,65,78,83,73,95,78,73,83,84,95,103,114,97,121,112,114,105,110,116,32,58,32,66,80,88,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,109,97,103,101,95,100,97,116,97,32,58,32,115,116,97,116,32,102,97,105,108,101,100,32,58,32,37,115,10,0,0,0,0,0,0,114,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,109,97,103,101,95,100,97,116,97,32,58,32,102,111,112,101,110,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,109,97,103,101,95,100,97,116,97,32,58,32,109,97,108,108,111,99,32,58,32,98,105,110,100,97,116,97,32,40,37,100,32,98,121,116,101,115,41,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,109,97,103,101,95,100,97,116,97,32,58,32,102,114,101,97,100,32,58,32,111,110,108,121,32,37,100,32,98,121,116,101,115,32,111,102,32,37,100,32,114,101,97,100,44,32,97,116,32,37,108,100,58,32,37,115,10,0,0,112,114,101,109,97,116,117,114,101,32,69,79,70,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,109,97,103,101,95,100,97,116,97,32,58,32,102,99,108,111,115,101,32,39,37,115,39,58,32,37,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,117,99,104,97,114,32,58,32,114,101,97,100,32,58,32,117,99,104,97,114,32,110,111,116,32,114,101,97,100,44,32,97,116,32,37,108,100,58,32,37,115,10,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,117,115,104,111,114,116,32,58,32,114,101,97,100,32,58,32,117,115,104,111,114,116,32,110,111,116,32,114,101,97,100,44,32,97,116,32,37,108,100,58,32,37,115,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,117,105,110,116,32,58,32,114,101,97,100,32,58,32,117,105,110,116,32,110,111,116,32,114,101,97,100,44,32,97,116,32,37,108,100,58,32,37,115,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,116,101,109,95,100,97,116,97,32,58,32,109,97,108,108,111,99,32,58,32,117,105,110,116,32,115,116,114,105,110,103,32,118,97,108,117,101,32,40,37,100,32,98,121,116,101,115,41,10,0,0,37,100,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,116,101,109,95,100,97,116,97,32,58,32,109,97,108,108,111,99,32,58,32,117,115,104,111,114,116,32,115,116,114,105,110,103,32,118,97,108,117,101,32,40,37,100,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,116,101,109,95,100,97,116,97,32,58,32,109,97,108,108,111,99,32,58,32,117,99,104,97,114,32,115,116,114,105,110,103,32,118,97,108,117,101,32,40,37,100,32,98,121,116,101,115,41,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,98,105,110,97,114,121,95,105,116,101,109,95,100,97,116,97,32,58,32,110,117,109,98,101,114,32,111,102,32,98,121,116,101,115,32,37,100,32,116,111,32,98,101,32,114,101,97,100,32,117,110,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,115,117,98,115,116,105,116,117,116,101,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,115,117,98,115,116,105,116,117,116,101,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,93,10,0,0,69,82,82,79,82,32,58,32,115,117,98,115,116,105,116,117,116,101,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,115,117,98,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,115,117,98,115,116,105,116,117,116,101,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,105,116,101,109,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,46,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,0,83,117,98,115,116,105,116,117,116,101,100,32,98,105,110,97,114,121,32,105,109,97,103,101,32,105,116,101,109,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,46,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,32,119,105,116,104,32,99,111,110,116,101,110,116,115,32,105,110,32,37,115,0,0,0,0,69,82,82,79,82,32,58,32,115,117,98,115,116,105,116,117,116,101,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,32,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,0,0,0,83,117,98,115,116,105,116,117,116,101,100,32,105,116,101,109,32,105,110,100,101,120,32,91,37,100,46,37,100,46,37,100,46,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,32,32,32,79,108,100,32,118,97,108,117,101,32,61,32,37,115,10,0,0,32,32,32,78,101,119,32,118,97,108,117,101,32,61,32,37,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,102,105,110,103,101,114,112,114,105,110,116,50,116,97,103,103,101,100,95,105,109,97,103,101,95,114,101,99,111,114,100,32,58,32,105,108,108,101,103,97,108,32,73,109,112,114,101,115,115,105,111,110,32,84,121,112,101,32,61,32,37,100,10,0,69,82,82,79,82,32,58,32,105,109,97,103,101,50,116,121,112,101,95,49,51,32,58,32,105,109,97,103,101,32,112,105,120,101,108,32,100,101,112,116,104,32,61,32,37,100,32,110,111,116,32,99,117,114,114,101,110,116,108,121,32,115,117,112,112,111,114,116,101,100,32,105,110,32,84,121,112,101,45,49,51,32,114,101,99,111,114,100,10,0,0,0,0,0,0,0,78,79,78,69,0,0,0,0,69,82,82,79,82,32,58,32,105,109,97,103,101,50,116,121,112,101,95,49,51,32,58,32,105,109,97,103,101,32,99,111,109,112,114,101,115,115,105,111,110,32,34,37,115,34,110,111,116,32,112,101,114,109,105,116,116,101,100,32,105,110,32,84,121,112,101,45,49,51,32,114,101,99,111,114,100,10,67,117,114,114,101,110,116,108,121,32,110,111,32,99,111,109,112,114,101,115,115,105,111,110,32,105,115,32,97,108,108,111,119,101,100,10,0,0,0,0,0,0,48,0,0,0,0,0,0,0,37,48,50,100,0,0,0,0,37,100,0,0,0,0,0,0,50,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,109,97,103,101,50,116,121,112,101,95,49,51,32,58,32,114,101,97,108,108,111,99,32,58,32,68,65,84,32,105,116,101,109,32,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,0,0,0,69,82,82,79,82,32,58,32,105,109,97,103,101,50,116,121,112,101,95,49,52,32,58,32,105,109,97,103,101,32,112,105,120,101,108,32,100,101,112,116,104,32,61,32,37,100,32,110,111,116,32,99,117,114,114,101,110,116,108,121,32,115,117,112,112,111,114,116,101,100,32,105,110,32,84,121,112,101,45,49,52,32,114,101,99,111,114,100,10,0,0,0,0,0,0,0,87,83,81,50,48,0,0,0,69,82,82,79,82,32,58,32,105,109,97,103,101,50,116,121,112,101,95,49,52,32,58,32,105,109,97,103,101,32,99,111,109,112,114,101,115,115,105,111,110,32,34,37,115,34,110,111,116,32,112,101,114,109,105,116,116,101,100,32,105,110,32,84,121,112,101,45,49,52,32,114,101,99,111,114,100,10,79,110,108,121,32,87,83,81,32,99,111,109,112,114,101,115,115,105,111,110,32,105,115,32,97,108,108,111,119,101,100,10,0,0,69,82,82,79,82,32,58,32,105,109,97,103,101,50,116,121,112,101,95,49,52,32,58,32,114,101,97,108,108,111,99,32,58,32,68,65,84,32,105,116,101,109,32,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,32,58,32,114,101,97,108,108,111,99,32,58,32,114,101,99,111,114,100,115,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,32,58,32,114,101,97,108,108,111,99,32,58,32,102,105,101,108,100,115,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,102,105,101,108,100,32,58,32,114,101,97,108,108,111,99,32,58,32,115,117,98,102,105,101,108,100,115,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,115,117,98,102,105,101,108,100,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,115,32,40,105,110,99,114,101,97,115,101,32,37,108,117,32,98,121,116,101,115,32,116,111,32,37,108,117,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,105,116,101,109,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,45,62,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,76,69,78,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,114,101,99,111,114,100,95,76,69,78,32,58,32,76,69,78,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,46,37,100,93,32,102,111,114,109,97,116,32,101,114,114,111,114,32,105,110,32,114,101,99,111,114,100,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,76,69,78,32,102,105,101,108,100,32,105,110,100,101,120,32,91,37,100,46,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,32,117,112,100,97,116,101,100,32,40,37,100,32,110,111,119,32,37,100,41,10,0,0,0,0,0,37,100,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,98,105,110,97,114,121,95,114,101,99,111,114,100,95,76,69,78,32,58,32,114,101,97,108,108,111,99,32,58,32,76,69,78,32,105,116,101,109,32,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,65,78,83,73,95,78,73,83,84,95,116,97,103,103,101,100,95,114,101,99,111,114,100,95,76,69,78,32,58,32,114,101,97,108,108,111,99,32,58,32,76,69,78,32,105,116,101,109,32,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,99,114,101,109,101,110,116,95,110,117,109,101,114,105,99,95,105,116,101,109,32,58,32,109,97,108,108,111,99,32,58,32,105,116,101,109,118,97,108,117,101,32,40,37,100,32,98,121,116,101,115,41,10,0,37,100,0,0,0,0,0,0,69,82,82,79,82,32,58,32,118,97,108,117,101,50,102,105,101,108,100,32,58,32,114,101,97,108,108,111,99,32,58,32,105,116,101,109,45,62,118,97,108,117,101,32,40,105,110,99,114,101,97,115,101,32,37,100,32,98,121,116,101,115,32,116,111,32,37,100,41,10,0,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,100,97,121,32,111,102,32,119,101,101,107,32,110,111,116,32,102,111,117,110,100,32,105,110,32,100,97,116,101,32,115,116,114,105,110,103,32,39,37,115,39,10,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,109,111,110,116,104,32,110,111,116,32,102,111,117,110,100,32,105,110,32,100,97,116,101,32,115,116,114,105,110,103,32,39,37,115,39,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,110,117,109,101,114,105,99,32,100,97,121,32,110,111,116,32,102,111,117,110,100,32,105,110,32,100,97,116,101,32,115,116,114,105,110,103,32,39,37,115,39,10,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,116,105,109,101,32,111,102,32,100,97,121,32,110,111,116,32,102,111,117,110,100,32,105,110,32,100,97,116,101,32,115,116,114,105,110,103,32,39,37,115,39,10,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,121,101,97,114,32,110,111,116,32,102,111,117,110,100,32,105,110,32,100,97,116,101,32,115,116,114,105,110,103,32,39,37,115,39,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,99,97,108,108,111,99,32,58,32,100,97,116,101,95,115,116,114,32,40,37,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,121,101,97,114,32,115,116,114,105,110,103,32,37,115,32,110,111,116,32,52,32,99,104,97,114,115,32,108,111,110,103,10,0,0,0,74,97,110,0,0,0,0,0,48,49,0,0,0,0,0,0,70,101,98,0,0,0,0,0,48,50,0,0,0,0,0,0,77,97,114,0,0,0,0,0,48,51,0,0,0,0,0,0,65,112,114,0,0,0,0,0,48,52,0,0,0,0,0,0,77,97,121,0,0,0,0,0,48,53,0,0,0,0,0,0,74,117,110,0,0,0,0,0,48,54,0,0,0,0,0,0,74,117,108,0,0,0,0,0,48,55,0,0,0,0,0,0,65,117,103,0,0,0,0,0,48,56,0,0,0,0,0,0,83,101,112,0,0,0,0,0,48,57,0,0,0,0,0,0,79,99,116,0,0,0,0,0,49,48,0,0,0,0,0,0,78,111,118,0,0,0,0,0,49,49,0,0,0,0,0,0,68,101,99,0,0,0,0,0,49,50,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,105,108,108,101,103,97,108,32,109,111,110,116,104,32,115,116,114,105,110,103,32,61,32,37,115,10,0,37,48,50,100,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,65,78,83,73,95,78,73,83,84,95,100,97,116,101,32,58,32,110,117,109,101,114,105,99,32,100,97,121,32,115,116,114,105,110,103,32,37,115,32,110,111,116,32,50,32,99,104,97,114,115,32,108,111,110,103,10,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,65,78,83,73,95,78,73,83,84,95,105,109,97,103,101,32,58,32,82,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,32,110,111,116,32,97,110,32,105,109,97,103,101,32,114,101,99,111,114,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,84,121,112,101,45,56,32,114,101,99,111,114,100,32,91,37,100,93,32,110,111,116,32,115,117,112,112,111,114,116,101,100,46,10,73,109,97,103,101,32,114,101,99,111,114,100,32,105,103,110,111,114,101,100,46,10,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,66,73,78,95,67,65,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,48,0,0,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,98,105,110,97,114,121,32,105,109,97,103,101,32,99,111,109,112,114,101,115,115,105,111,110,32,111,102,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,32,105,115,32,117,110,115,117,112,112,111,114,116,101,100,46,10,73,109,97,103,101,32,114,101,99,111,114,100,32,105,103,110,111,114,101,100,46,10,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,72,76,76,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,86,76,76,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,105,108,108,101,103,97,108,32,98,105,110,97,114,121,32,105,109,97,103,101,32,114,101,99,111,114,100,32,116,121,112,101,32,61,32,37,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,109,97,108,108,111,99,32,58,32,105,100,97,116,97,50,32,40,37,100,32,98,121,116,101,115,41,10,0,49,0,0,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,117,110,115,117,112,112,111,114,116,101,100,32,99,111,109,112,114,101,115,115,105,111,110,32,97,108,103,111,114,105,116,104,109,32,37,46,49,48,115,37,115,32,105,110,32,105,109,97,103,101,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,46,10,73,109,97,103,101,32,114,101,99,111,114,100,32,105,103,110,111,114,101,100,46,10,0,0,0,0,46,46,46,0,0,0,0,0,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,91,72,76,76,32,102,105,101,108,100,32,40,102,114,111,109,32,102,105,108,101,41,32,61,32,37,100,93,32,33,61,32,91,105,109,97,103,101,32,119,105,100,116,104,32,40,102,114,111,109,32,100,101,99,111,100,101,114,41,32,61,32,37,100,93,10,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,91,86,76,76,32,102,105,101,108,100,32,40,102,114,111,109,32,102,105,108,101,41,32,61,32,37,100,93,32,33,61,32,91,105,109,97,103,101,32,104,101,105,103,104,116,32,40,102,114,111,109,32,100,101,99,111,100,101,114,41,32,61,32,37,100,93,10,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,98,105,110,97,114,121,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,91,112,105,120,101,108,32,100,101,112,116,104,32,40,102,114,111,109,32,114,101,99,111,114,100,32,84,121,112,101,45,37,100,41,32,61,32,37,100,93,32,33,61,32,91,112,105,120,101,108,32,100,101,112,116,104,32,40,102,114,111,109,32,100,101,99,111,100,101,114,41,32,61,32,37,100,93,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,111,117,116,32,111,102,32,114,97,110,103,101,32,91,49,46,46,37,100,93,10,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,32,105,115,32,110,111,116,32,97,32,116,97,103,103,101,100,32,102,105,108,101,32,105,109,97,103,101,32,114,101,99,111,114,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,84,65,71,95,67,65,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,72,76,76,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,86,76,76,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,66,80,88,32,102,105,101,108,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,100,112,121,97,110,50,107,95,116,97,103,103,101,100,95,114,101,99,111,114,100,32,58,32,67,83,80,32,102,105,101,108,100,32,105,110,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,0,0,0,0,71,82,65,89,0,0,0,0,82,71,66,0,0,0,0,0,89,67,67,0,0,0,0,0,83,82,71,66,0,0,0,0,83,89,67,67,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,99,111,108,111,114,115,112,97,99,101,32,34,37,46,49,48,115,37,115,34,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,105,110,32,105,110,32,114,101,99,111,114,100,32,91,37,100,93,32,91,84,121,112,101,45,37,100,46,37,48,51,100,93,46,10,73,109,97,103,101,32,114,101,99,111,114,100,32,105,103,110,111,114,101,100,46,10,0,0,0,0,0,0,0,78,79,78,69,0,0,0,0,69,82,82,79,82,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,109,97,108,108,111,99,32,58,32,105,100,97,116,97,50,32,40,37,100,32,98,121,116,101,115,41,10,0,87,83,81,50,48,0,0,0,74,80,69,71,66,0,0,0,74,80,69,71,76,0,0,0,74,80,50,0,0,0,0,0,74,80,50,76,0,0,0,0,80,78,71,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,117,110,115,117,112,112,111,114,116,101,100,32,99,111,109,112,114,101,115,115,105,111,110,32,97,108,103,111,114,105,116,104,109,32,37,46,49,48,115,37,115,32,105,110,32,105,109,97,103,101,32,114,101,99,111,114,100,32,105,110,100,101,120,32,91,37,100,93,32,91,84,121,112,101,45,37,100,93,46,10,73,109,97,103,101,32,114,101,99,111,114,100,32,105,103,110,111,114,101,100,46,10,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,91,72,76,76,32,102,105,101,108,100,32,40,102,114,111,109,32,102,105,108,101,41,32,61,32,37,100,93,32,33,61,32,91,105,109,97,103,101,32,119,105,100,116,104,32,40,102,114,111,109,32,100,101,99,111,100,101,114,41,32,61,32,37,100,93,10,87,105,108,108,32,99,111,110,116,105,110,117,101,32,119,105,116,104,32,111,112,101,114,97,116,105,110,103,32,97,115,115,117,109,112,116,105,111,110,32,116,104,97,116,32,105,109,97,103,101,32,119,105,100,116,104,32,105,115,32,37,100,10,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,91,86,76,76,32,102,105,101,108,100,32,40,102,114,111,109,32,102,105,108,101,41,32,61,32,37,100,93,32,33,61,32,91,105,109,97,103,101,32,104,101,105,103,104,116,32,40,102,114,111,109,32,100,101,99,111,100,101,114,41,32,61,32,37,100,93,10,87,105,108,108,32,99,111,110,116,105,110,117,101,32,119,105,116,104,32,111,112,101,114,97,116,105,110,103,32,97,115,115,117,109,112,116,105,111,110,32,116,104,97,116,32,105,109,97,103,101,32,104,101,105,103,104,116,32,105,115,32,37,100,10,0,0,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,100,101,99,111,100,101,95,116,97,103,103,101,100,95,102,105,101,108,100,95,105,109,97,103,101,32,58,32,91,112,105,120,101,108,32,100,101,112,116,104,32,40,102,114,111,109,32,102,105,101,108,100,32,118,97,108,117,101,41,32,61,32,37,100,93,32,33,61,32,91,112,105,120,101,108,32,100,101,112,116,104,32,40,102,114,111,109,32,100,101,99,111,100,101,114,41,32,61,32,37,100,93,10,87,105,108,108,32,99,111,110,116,105,110,117,101,32,119,105,116,104,32,111,112,101,114,97,116,105,110,103,32,97,115,115,117,109,112,116,105,111,110,32,116,104,97,116,32,105,109,97,103,101,32,100,101,112,116,104,32,105,115,32,37,100,10,0,0,0,0,0,0,0,0,1,0,0,0,2,0,0,0,9,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,99,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,8,0,0,0,10,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,17,0,0,0,99,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,98,46,106,112,50,0,0,0,97,46,114,97,119,0,0,0,69,82,82,79,82,32,45,62,32,102,97,105,108,101,100,32,116,111,32,100,101,99,111,100,101,32,105,109,97,103,101,33,10,0,0,0,0,0,0,0,69,82,82,79,82,32,45,62,32,67,97,110,39,116,32,100,101,99,111,100,101,32,115,105,103,110,101,100,32,98,117,102,102,101,114,33,10,0,0,0,69,82,82,79,82,32,45,62,32,105,109,97,103,101,95,116,111,95,114,97,119,33,10,0,69,82,82,79,82,32,58,32,111,112,101,110,106,112,101,103,50,107,95,100,101,99,111,100,101,95,109,101,109,58,32,102,97,105,108,101,100,32,116,111,32,103,101,110,101,114,97,116,101,32,105,109,103,95,100,97,116,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,109,103,95,100,97,116,95,103,101,110,101,114,97,116,101,32,58,32,99,97,108,108,111,99,32,58,32,105,109,103,95,100,97,116,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,105,109,103,95,100,97,116,95,103,101,110,101,114,97,116,101,58,32,117,110,101,113,117,97,108,32,99,111,109,112,111,110,101,110,116,32,100,101,112,116,104,115,44,32,48,58,32,37,100,32,97,110,100,32,37,100,58,32,37,100,10,0,69,82,82,79,82,32,58,32,103,101,116,95,114,97,119,95,105,109,97,103,101,58,32,99,97,108,108,111,99,32,58,32,105,109,103,95,100,97,116,45,62,105,109,97,103,101,91,37,100,93,32,40,37,108,117,32,98,121,116,101,115,41,10,0,10,69,114,114,111,114,58,32,105,110,118,97,108,105,100,32,114,97,119,32,105,109,97,103,101,32,112,97,114,97,109,101,116,101,114,115,10,0,0,0,82,97,119,32,105,109,97,103,101,32,99,104,97,114,97,99,116,101,114,105,115,116,105,99,115,58,32,37,100,32,99,111,109,112,111,110,101,110,116,115,10,0,0,0,0,0,0,0,67,111,109,112,111,110,101,110,116,32,37,100,32,99,104,97,114,97,99,116,101,114,105,115,116,105,99,115,58,32,37,100,120,37,100,120,37,100,32,37,115,10,0,0,0,0,0,0,115,105,103,110,101,100,0,0,117,110,115,105,103,110,101,100,0,0,0,0,0,0,0,0,77,111,114,101,32,116,104,97,110,32,56,32,98,105,116,115,32,112,101,114,32,99,111,109,112,111,110,101,110,116,32,110,111,32,104,97,110,100,108,101,100,32,121,101,116,10,0,0,77,111,114,101,32,116,104,97,110,32,49,54,32,98,105,116,115,32,112,101,114,32,99,111,109,112,111,110,101,110,116,32,110,111,32,104,97,110,100,108,101,100,32,121,101,116,10,0,69,114,114,111,114,58,32,105,110,118,97,108,105,100,32,112,114,101,99,105,115,105,111,110,58,32,37,100,10,0,0,0,64,108,0,0,72,108,0,0,80,108,0,0,88,108,0,0,96,108,0,0,104,108,0,0,112,108,0,0,120,108,0,0,128,108,0,0,136,108,0,0,144,108,0,0,0,0,0,0,112,103,120,0,0,0,0,0,112,110,109,0,0,0,0,0,112,103,109,0,0,0,0,0,112,112,109,0,0,0,0,0,98,109,112,0,0,0,0,0,116,105,102,0,0,0,0,0,114,97,119,0,0,0,0,0,116,103,97,0,0,0,0,0,106,50,107,0,0,0,0,0,106,112,50,0,0,0,0,0,106,50,99,0,0,0,0,0,11,0,0,0,10,0,0,0,10,0,0,0,10,0,0,0,12,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,82,101,97,100,32,69,114,114,111,114,0,0,0,0,0,0,49,46,50,46,50,51,0,0,69,82,82,79,82,32,58,32,105,109,103,95,100,97,116,95,103,101,110,101,114,97,116,101,58,32,99,97,108,108,111,99,32,58,32,105,109,103,95,100,97,116,32,40,37,108,117,32,98,121,116,101,115,41,10,0,69,82,82,79,82,32,58,32,103,101,116,95,114,97,119,95,105,109,97,103,101,32,58,32,99,97,108,108,111,99,32,58,32,105,109,103,95,100,97,116,45,62,105,109,97,103,101,91,37,100,93,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,112,110,103,95,115,116,114,101,97,109,32,58,115,101,116,106,109,112,32,58,32,101,114,114,111,114,32,100,117,114,105,110,103,32,74,65,83,80,69,82,32,100,101,99,111,100,105,110,103,32,111,102,32,80,78,71,32,105,109,97,103,101,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,112,110,103,95,115,116,114,101,97,109,32,58,32,109,97,108,108,111,99,32,37,117,32,114,111,119,95,112,111,105,110,116,101,114,115,32,40,37,108,117,32,98,121,116,101,115,41,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,112,110,103,95,115,116,114,101,97,109,32,58,32,109,97,108,108,111,99,32,114,111,119,32,37,100,32,40,37,117,32,98,121,116,101,115,41,10,0,0,0,0,0,114,98,0,0,0,0,0,0,69,82,79,82,82,32,58,32,114,101,97,100,95,114,97,119,95,102,114,111,109,95,102,105,108,101,115,105,122,101,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,0,69,82,79,82,82,32,58,32,114,101,97,100,95,114,97,119,95,102,114,111,109,95,102,105,108,101,115,105,122,101,32,58,32,109,97,108,108,111,99,32,58,32,105,100,97,116,97,10,0,0,0,0,0,0,0,0,69,82,79,82,82,32,58,32,109,97,105,110,32,58,32,114,101,97,100,95,114,97,119,95,102,114,111,109,95,102,105,108,101,115,105,122,101,32,58,32,0,0,0,0,0,0,0,0,37,100,32,111,102,32,37,100,32,98,121,116,101,115,32,114,101,97,100,32,102,114,111,109,32,37,115,10,0,0,0,0,119,98,0,0,0,0,0,0,69,82,82,79,82,58,32,119,114,105,116,101,95,114,97,119,95,102,114,111,109,95,109,101,109,115,105,122,101,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,0,0,69,82,82,79,82,58,32,119,114,105,116,101,95,114,97,119,95,102,114,111,109,95,109,101,109,115,105,122,101,32,58,32,102,119,114,105,116,101,32,58,32,0,0,0,0,0,0,0,111,110,108,121,32,37,100,32,111,102,32,37,100,32,98,121,116,101,115,32,119,114,105,116,116,101,110,32,102,114,111,109,32,102,105,108,101,32,37,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,97,110,100,95,100,101,99,111,100,101,95,103,114,97,121,115,99,97,108,101,95,105,109,97,103,101,32,58,32,0,0,0,0,0,0,103,114,97,121,115,99,97,108,101,32,105,109,97,103,101,32,114,101,99,111,114,100,32,110,111,116,32,102,111,117,110,100,32,105,110,32,37,115,10,0,37,115,32,58,32,105,109,97,103,101,32,116,121,112,101,32,85,78,75,78,79,87,78,32,58,32,110,111,116,32,115,117,112,112,111,114,116,101,100,10,0,0,0,0,0,0,0,0,37,115,32,58,32,105,109,97,103,101,32,100,101,112,116,104,32,58,32,37,100,32,33,61,32,56,10,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,97,110,100,95,100,101,99,111,100,101,95,105,109,97,103,101,32,58,32,0,0,0,0,0,0,0,0,74,80,69,71,66,32,100,101,99,111,100,101,114,32,114,101,116,117,114,110,101,100,32,100,61,37,100,32,0,0,0,0,110,111,116,32,101,113,117,97,108,32,116,111,32,56,32,111,114,32,50,52,10,0,0,0,73,72,101,97,100,32,100,101,99,111,100,101,114,32,114,101,116,117,114,110,101,100,32,100,61,37,100,32,0,0,0,0,110,111,116,32,101,113,117,97,108,32,116,111,32,123,49,44,56,44,50,52,125,10,0,0,105,108,108,101,103,97,108,32,105,109,97,103,101,32,116,121,112,101,32,61,32,37,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,104,101,97,100,95,100,101,99,111,100,101,95,109,101,109,32,58,32,109,97,108,108,111,99,32,58,32,111,100,97,116,97,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,104,101,97,100,95,100,101,99,111,100,101,95,109,101,109,32,58,32,0,0,0,0,0,105,110,118,97,108,105,100,32,99,111,109,112,114,101,115,115,105,111,110,32,99,111,100,101,32,61,32,37,100,10,0,0,37,100,0,0,0,0,0,0,69,82,82,79,82,32,58,32,106,112,101,103,95,116,121,112,101,32,58,32,67,111,117,108,100,32,110,111,116,32,100,101,116,101,114,109,105,110,101,32,74,80,69,71,32,116,121,112,101,32,0,0,0,0,0,0,40,105,101,46,32,98,97,115,101,108,105,110,101,32,111,114,32,108,111,115,115,108,101,115,115,41,10,0,0,0,0,0,106,80,32,32,0,0,0,0,80,105,120,80,101,114,66,121,116,101,0,0,0,0,0,0,100,101,112,116,104,32,33,61,32,112,111,119,101,114,32,111,102,32,50,32,98,101,116,119,101,101,110,32,49,32,97,110,100,32,54,52,32,40,105,110,99,108,117,115,105,118,101,41,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,116,114,108,118,50,110,111,116,95,109,101,109,32,58,32,110,117,109,98,101,114,32,111,102,32,99,111,109,112,111,110,101,110,116,115,32,61,32,37,100,32,62,32,37,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,105,110,116,114,108,118,50,110,111,116,95,109,101,109,32,58,32,109,97,108,108,111,99,32,58,32,111,100,97,116,97,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,110,111,116,50,105,110,116,114,108,118,95,109,101,109,32,58,32,110,117,109,98,101,114,32,111,102,32,99,111,109,112,111,110,101,110,116,115,32,61,32,37,100,32,62,32,37,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,110,111,116,50,105,110,116,114,108,118,95,109,101,109,32,58,32,109,97,108,108,111,99,32,58,32,111,100,97,116,97,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,112,97,100,95,99,111,109,112,111,110,101,110,116,95,112,108,97,110,101,115,32,58,32,114,101,97,108,108,111,99,32,58,32,105,100,97,116,97,10,0,0,0,0,0,0,0,0,0,79,117,116,112,117,116,32,66,117,102,102,101,114,32,79,118,101,114,102,108,111,119,32,105,110,32,82,76,76,95,112,117,116,99,46,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,67,97,110,110,111,116,32,97,108,108,111,99,97,116,101,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,102,111,114,32,99,111,109,112,114,101,115,115,101,100,32,102,105,108,101,46,10,0,0,0,0,10,78,111,32,109,101,109,111,114,121,32,97,108,108,111,99,97,116,101,100,32,102,111,114,32,105,110,112,117,116,32,100,97,116,97,33,10,0,0,0,10,78,111,32,109,101,109,111,114,121,32,97,108,108,111,99,97,116,101,100,32,102,111,114,32,100,101,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,33,10,0,0,0,0,84,104,105,115,32,119,105,108,108,32,107,105,108,108,32,116,104,101,32,112,114,111,103,114,97,109,32,97,110,100,32,99,114,101,97,116,101,32,97,32,99,111,114,101,32,102,105,108,101,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,64,32,16,8,4,2,1,127,191,223,239,247,251,253,254,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,64,32,16,8,4,2,1,255,255,255,255,184,116,0,0,172,116,0,0,255,255,255,255,208,116,0,0,196,116,0,0,255,255,255,255,232,116,0,0,220,116,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,117,0,0,244,116,0,0,255,255,255,255,24,117,0,0,12,117,0,0,4,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,48,117,0,0,36,117,0,0,255,255,255,255,72,117,0,0,60,117,0,0,5,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,96,117,0,0,84,117,0,0,255,255,255,255,120,117,0,0,108,117,0,0,7,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,144,117,0,0,132,117,0,0,255,255,255,255,168,117,0,0,156,117,0,0,255,255,255,255,192,117,0,0,180,117,0,0,8,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,216,117,0,0,204,117,0,0,255,255,255,255,240,117,0,0,228,117,0,0,255,255,255,255,8,118,0,0,252,117,0,0,255,255,255,255,32,118,0,0,20,118,0,0,12,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,56,118,0,0,44,118,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,80,118,0,0,68,118,0,0,255,255,255,255,104,118,0,0,92,118,0,0,255,255,255,255,128,118,0,0,116,118,0,0,255,255,255,255,0,0,0,0,140,118,0,0,255,255,255,255,164,118,0,0,152,118,0,0,255,255,255,255,188,118,0,0,176,118,0,0,14,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,212,118,0,0,200,118,0,0,255,255,255,255,236,118,0,0,224,118,0,0,13,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,4,119,0,0,248,118,0,0,255,255,255,255,28,119,0,0,16,119,0,0,255,255,255,255,52,119,0,0,40,119,0,0,255,255,255,255,76,119,0,0,64,119,0,0,255,255,255,255,100,119,0,0,88,119,0,0,255,255,255,255,124,119,0,0,112,119,0,0,15,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,148,119,0,0,136,119,0,0,255,255,255,255,172,119,0,0,160,119,0,0,255,255,255,255,196,119,0,0,184,119,0,0,255,255,255,255,220,119,0,0,208,119,0,0,255,255,255,255,244,119,0,0,232,119,0,0,255,255,255,255,12,120,0,0,0,120,0,0,255,255,255,255,36,120,0,0,24,120,0,0,255,255,255,255,60,120,0,0,48,120,0,0,255,255,255,255,84,120,0,0,72,120,0,0,255,255,255,255,108,120,0,0,96,120,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,132,120,0,0,120,120,0,0,255,255,255,255,156,120,0,0,144,120,0,0,255,255,255,255,180,120,0,0,168,120,0,0,255,255,255,255,204,120,0,0,192,120,0,0,255,255,255,255,228,120,0,0,216,120,0,0,255,255,255,255,252,120,0,0,240,120,0,0,255,255,255,255,20,121,0,0,8,121,0,0,255,255,255,255,44,121,0,0,32,121,0,0,17,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,68,121,0,0,56,121,0,0,255,255,255,255,92,121,0,0,80,121,0,0,255,255,255,255,116,121,0,0,104,121,0,0,64,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,140,121,0,0,128,121,0,0,255,255,255,255,164,121,0,0,152,121,0,0,255,255,255,255,188,121,0,0,176,121,0,0,255,255,255,255,212,121,0,0,200,121,0,0,255,255,255,255,236,121,0,0,224,121,0,0,255,255,255,255,4,122,0,0,248,121,0,0,18,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,28,122,0,0,16,122,0,0,255,255,255,255,52,122,0,0,40,122,0,0,255,255,255,255,76,122,0,0,64,122,0,0,255,255,255,255,100,122,0,0,88,122,0,0,255,255,255,255,124,122,0,0,112,122,0,0,21,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,148,122,0,0,136,122,0,0,255,255,255,255,172,122,0,0,160,122,0,0,255,255,255,255,196,122,0,0,184,122,0,0,20,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,220,122,0,0,208,122,0,0,255,255,255,255,244,122,0,0,232,122,0,0,255,255,255,255,12,123,0,0,0,123,0,0,22,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,36,123,0,0,24,123,0,0,255,255,255,255,60,123,0,0,48,123,0,0,255,255,255,255,84,123,0,0,72,123,0,0,255,255,255,255,108,123,0,0,96,123,0,0,255,255,255,255,132,123,0,0,120,123,0,0,255,255,255,255,156,123,0,0,144,123,0,0,255,255,255,255,180,123,0,0,168,123,0,0,255,255,255,255,204,123,0,0,192,123,0,0,255,255,255,255,228,123,0,0,216,123,0,0,255,255,255,255,252,123,0,0,240,123,0,0,23,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,20,124,0,0,8,124,0,0,255,255,255,255,44,124,0,0,32,124,0,0,255,255,255,255,68,124,0,0,56,124,0,0,255,255,255,255,92,124,0,0,80,124,0,0,255,255,255,255,116,124,0,0,104,124,0,0,25,0,0,0,0,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,140,124,0,0,128,124,0,0,255,255,255,255,164,124,0,0,152,124,0,0,255,255,255,255,188,124,0,0,176,124,0,0,255,255,255,255,212,124,0,0,200,124,0,0,255,255,255,255,236,124,0,0,224,124,0,0,255,255,255,255,4,125,0,0,248,124,0,0,255,255,255,255,28,125,0,0,16,125,0,0,128,7,0,0,0,0,0,0,0,0,0,0,64,7,0,0,0,0,0,0,0,0,0,0,255,255,255,255,52,125,0,0,40,125,0,0,255,255,255,255,76,125,0,0,64,125,0,0,255,255,255,255,100,125,0,0,88,125,0,0,0,7,0,0,0,0,0,0,0,0,0,0,43,0,0,0,0,0,0,0,0,0,0,0,42,0,0,0,0,0,0,0,0,0,0,0,39,0,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,37,0,0,0,0,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,35,0,0,0,0,0,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,29,0,0,0,0,0,0,0,0,0,0,0,28,0,0,0,0,0,0,0,0,0,0,0,27,0,0,0,0,0,0,0,0,0,0,0,26,0,0,0,0,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,41,0,0,0,0,0,0,0,0,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,0,0,0,0,62,0,0,0,0,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,61,0,0,0,0,0,0,0,0,0,0,0,58,0,0,0,0,0,0,0,0,0,0,0,57,0,0,0,0,0,0,0,0,0,0,0,47,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,0,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,51,0,0,0,0,0,0,0,0,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,124,125,0,0,112,125,0,0,255,255,255,255,148,125,0,0,136,125,0,0,255,255,255,255,172,125,0,0,160,125,0,0,54,0,0,0,0,0,0,0,0,0,0,0,53,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,196,125,0,0,184,125,0,0,192,1,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,0,0,0,0,0,0,0,64,1,0,0,0,0,0,0,0,0,0,0,255,255,255,255,220,125,0,0,208,125,0,0,255,255,255,255,244,125,0,0,232,125,0,0,60,0,0,0,0,0,0,0,0,0,0,0,59,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,12,126,0,0,0,126,0,0,255,255,255,255,36,126,0,0,24,126,0,0,56,0,0,0,0,0,0,0,0,0,0,0,55,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,60,126,0,0,48,126,0,0,255,255,255,255,84,126,0,0,72,126,0,0,52,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,192,9,0,0,0,0,0,0,0,0,0,0,128,9,0,0,0,0,0,0,0,0,0,0,64,9,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,192,8,0,0,0,0,0,0,0,0,0,0,128,8,0,0,0,0,0,0,0,0,0,0,64,8,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,192,7,0,0,0,0,0,0,0,0,0,0,192,4,0,0,0,0,0,0,0,0,0,0,128,4,0,0,0,0,0,0,0,0,0,0,64,4,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,192,3,0,0,0,0,0,0,0,0,0,0,128,3,0,0,0,0,0,0,0,0,0,0,64,2,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,192,6,0,0,0,0,0,0,0,0,0,0,128,6,0,0,0,0,0,0,0,0,0,0,64,6,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,192,5,0,0,0,0,0,0,0,0,0,0,128,5,0,0,0,0,0,0,0,0,0,0,64,5,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,64,3,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,192,2,0,0,0,0,0,0,0,0,0,0,128,2,0,0,0,0,0,0,0,0,0,0,255,255,255,255,120,126,0,0,108,126,0,0,255,255,255,255,144,126,0,0,132,126,0,0,255,255,255,255,168,126,0,0,156,126,0,0,255,255,255,255,192,126,0,0,180,126,0,0,255,255,255,255,216,126,0,0,204,126,0,0,255,255,255,255,240,126,0,0,228,126,0,0,255,255,255,255,8,127,0,0,252,126,0,0,255,255,255,255,32,127,0,0,20,127,0,0,255,255,255,255,56,127,0,0,44,127,0,0,255,255,255,255,80,127,0,0,68,127,0,0,255,255,255,255,104,127,0,0,92,127,0,0,255,255,255,255,128,127,0,0,116,127,0,0,255,255,255,255,152,127,0,0,140,127,0,0,255,255,255,255,176,127,0,0,164,127,0,0,255,255,255,255,200,127,0,0,188,127,0,0,7,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,224,127,0,0,212,127,0,0,5,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,248,127,0,0,236,127,0,0,255,255,255,255,16,128,0,0,4,128,0,0,3,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,40,128,0,0,28,128,0,0,255,255,255,255,64,128,0,0,52,128,0,0,255,255,255,255,88,128,0,0,76,128,0,0,255,255,255,255,112,128,0,0,100,128,0,0,255,255,255,255,136,128,0,0,124,128,0,0,255,255,255,255,160,128,0,0,148,128,0,0,255,255,255,255,184,128,0,0,172,128,0,0,64,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,208,128,0,0,196,128,0,0,255,255,255,255,232,128,0,0,220,128,0,0,9,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,129,0,0,244,128,0,0,255,255,255,255,24,129,0,0,12,129,0,0,255,255,255,255,48,129,0,0,36,129,0,0,255,255,255,255,72,129,0,0,60,129,0,0,255,255,255,255,96,129,0,0,84,129,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,120,129,0,0,108,129,0,0,255,255,255,255,144,129,0,0,132,129,0,0,255,255,255,255,168,129,0,0,156,129,0,0,255,255,255,255,192,129,0,0,180,129,0,0,255,255,255,255,216,129,0,0,204,129,0,0,255,255,255,255,240,129,0,0,228,129,0,0,255,255,255,255,8,130,0,0,252,129,0,0,15,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,32,130,0,0,20,130,0,0,255,255,255,255,56,130,0,0,44,130,0,0,255,255,255,255,80,130,0,0,68,130,0,0,128,6,0,0,0,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,104,130,0,0,92,130,0,0,255,255,255,255,128,130,0,0,116,130,0,0,255,255,255,255,152,130,0,0,140,130,0,0,255,255,255,255,176,130,0,0,164,130,0,0,255,255,255,255,200,130,0,0,188,130,0,0,255,255,255,255,224,130,0,0,212,130,0,0,255,255,255,255,248,130,0,0,236,130,0,0,255,255,255,255,16,131,0,0,4,131,0,0,255,255,255,255,40,131,0,0,28,131,0,0,255,255,255,255,64,131,0,0,52,131,0,0,12,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,88,131,0,0,76,131,0,0,255,255,255,255,112,131,0,0,100,131,0,0,255,255,255,255,136,131,0,0,124,131,0,0,13,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,160,131,0,0,148,131,0,0,255,255,255,255,184,131,0,0,172,131,0,0,255,255,255,255,208,131,0,0,196,131,0,0,0,1,0,0,0,0,0,0,0,0,0,0,255,255,255,255,232,131,0,0,220,131,0,0,255,255,255,255,0,132,0,0,244,131,0,0,255,255,255,255,24,132,0,0,12,132,0,0,255,255,255,255,48,132,0,0,36,132,0,0,255,255,255,255,72,132,0,0,60,132,0,0,255,255,255,255,96,132,0,0,84,132,0,0,255,255,255,255,120,132,0,0,108,132,0,0,25,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,144,132,0,0,132,132,0,0,255,255,255,255,168,132,0,0,156,132,0,0,24,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,192,132,0,0,180,132,0,0,255,255,255,255,216,132,0,0,204,132,0,0,27,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,240,132,0,0,228,132,0,0,255,255,255,255,8,133,0,0,252,132,0,0,255,255,255,255,32,133,0,0,20,133,0,0,28,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,56,133,0,0,44,133,0,0,255,255,255,255,80,133,0,0,68,133,0,0,255,255,255,255,104,133,0,0,92,133,0,0,26,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,128,133,0,0,116,133,0,0,255,255,255,255,152,133,0,0,140,133,0,0,19,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,176,133,0,0,164,133,0,0,255,255,255,255,200,133,0,0,188,133,0,0,255,255,255,255,224,133,0,0,212,133,0,0,20,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,248,133,0,0,236,133,0,0,23,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,16,134,0,0,4,134,0,0,255,255,255,255,40,134,0,0,28,134,0,0,255,255,255,255,0,0,0,0,52,134,0,0,255,255,255,255,76,134,0,0,64,134,0,0,255,255,255,255,100,134,0,0,88,134,0,0,255,255,255,255,124,134,0,0,112,134,0,0,255,255,255,255,148,134,0,0,136,134,0,0,255,255,255,255,172,134,0,0,160,134,0,0,64,2,0,0,0,0,0,0,0,0,0,0,128,2,0,0,0,0,0,0,0,0,0,0,255,255,255,255,196,134,0,0,184,134,0,0,0,2,0,0,0,0,0,0,0,0,0,0,192,1,0,0,0,0,0,0,0,0,0,0,58,0,0,0,0,0,0,0,0,0,0,0,57,0,0,0,0,0,0,0,0,0,0,0,56,0,0,0,0,0,0,0,0,0,0,0,55,0,0,0,0,0,0,0,0,0,0,0,52,0,0,0,0,0,0,0,0,0,0,0,51,0,0,0,0,0,0,0,0,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,220,134,0,0,208,134,0,0,255,255,255,255,244,134,0,0,232,134,0,0,60,0,0,0,0,0,0,0,0,0,0,0,59,0,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,0,0,0,0,0,0,0,64,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,0,0,0,0,62,0,0,0,0,0,0,0,0,0,0,0,61,0,0,0,0,0,0,0,0,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,43,0,0,0,0,0,0,0,0,0,0,0,42,0,0,0,0,0,0,0,0,0,0,0,41,0,0,0,0,0,0,0,0,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,39,0,0,0,0,0,0,0,0,0,0,0,54,0,0,0,0,0,0,0,0,0,0,0,53,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,37,0,0,0,0,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,35,0,0,0,0,0,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,47,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,0,0,0,0,29,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,12,135,0,0,0,135,0,0,128,5,0,0,0,0,0,0,0,0,0,0,64,5,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,192,4,0,0,0,0,0,0,0,0,0,0,128,4,0,0,0,0,0,0,0,0,0,0,64,4,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,192,3,0,0,0,0,0,0,0,0,0,0,128,3,0,0,0,0,0,0,0,0,0,0,64,3,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,192,2,0,0,0,0,0,0,0,0,0,0,192,6,0,0,0,0,0,0,0,0,0,0,64,6,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,192,5,0,0,0,0,0,0,0,0,0,0,255,255,255,255,36,135,0,0,24,135,0,0,255,255,255,255,60,135,0,0,48,135,0,0,255,255,255,255,84,135,0,0,72,135,0,0,255,255,255,255,108,135,0,0,96,135,0,0,255,255,255,255,132,135,0,0,120,135,0,0,255,255,255,255,156,135,0,0,144,135,0,0,255,255,255,255,180,135,0,0,168,135,0,0,255,255,255,255,204,135,0,0,192,135,0,0,128,7,0,0,0,0,0,0,0,0,0,0,64,7,0,0,0,0,0,0,0,0,0,0,255,255,255,255,228,135,0,0,216,135,0,0,255,255,255,255,252,135,0,0,240,135,0,0,255,255,255,255,20,136,0,0,8,136,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,192,9,0,0,0,0,0,0,0,0,0,0,128,9,0,0,0,0,0,0,0,0,0,0,64,9,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,192,8,0,0,0,0,0,0,0,0,0,0,128,8,0,0,0,0,0,0,0,0,0,0,64,8,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,192,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,111,105,110,116,101,114,32,104,101,97,100,32,105,115,32,78,85,76,76,0,0,0,0,103,101,116,95,119,105,100,116,104,0,0,0,0,0,0,0,37,100,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,103,101,116,32,116,104,101,32,112,105,120,101,108,32,119,105,100,116,104,0,0,0,103,101,116,95,104,101,105,103,104,116,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,103,101,116,32,116,104,101,32,112,105,120,101,108,32,104,101,105,103,104,116,0,0,103,101,116,95,100,101,112,116,104,0,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,103,101,116,32,116,104,101,32,100,101,112,116,104,32,102,114,111,109,32,116,104,101,32,105,104,101,97,100,32,115,116,117,114,99,116,117,114,101,0,0,0,0,0,0,0,0,103,101,116,95,100,101,110,115,105,116,121,0,0,0,0,0,103,101,116,95,99,111,109,112,114,101,115,115,105,111,110,0,70,97,105,108,101,100,32,116,111,32,103,101,116,32,116,104,101,32,99,111,109,112,114,101,115,115,105,111,110,32,116,121,112,101,0,0,0,0,0,0,115,101,116,95,99,111,109,112,114,101,115,115,105,111,110,0,103,101,116,95,99,111,109,112,108,101,110,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,103,101,116,32,116,104,101,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,32,108,101,110,103,116,104,0,0,0,0,0,0,0,0,115,101,116,95,99,111,109,112,108,101,110,0,0,0,0,0,83,79,73,44,32,116,97,98,108,101,115,44,32,97,110,100,32,102,114,97,109,101,32,104,101,97,100,101,114,32,114,101,97,100,10,10,0,0,0,0,84,97,98,108,101,115,32,102,111,114,32,119,97,118,101,108,101,116,32,100,101,99,111,109,112,111,115,105,116,105,111,110,32,102,105,110,105,115,104,101,100,10,10,0,0,0,0,0,69,82,82,79,82,58,32,119,115,113,95,100,101,99,111,100,101,95,109,101,109,32,58,32,109,97,108,108,111,99,32,58,32,113,100,97,116,97,49,10,0,0,0,0,0,0,0,0,81,117,97,110,116,105,122,101,100,32,87,83,81,32,115,117,98,98,97,110,100,32,100,97,116,97,32,98,108,111,99,107,115,32,114,101,97,100,32,97,110,100,32,72,117,102,102,109,97,110,32,100,101,99,111,100,101,100,10,10,0,0,0,0,87,83,81,32,115,117,98,98,97,110,100,32,100,97,116,97,32,98,108,111,99,107,115,32,117,110,113,117,97,110,116,105,122,101,100,10,10,0,0,0,87,83,81,32,114,101,99,111,110,115,116,114,117,99,116,105,111,110,32,111,102,32,105,109,97,103,101,32,102,105,110,105,115,104,101,100,10,10,0,0,69,82,82,79,82,58,32,119,115,113,95,100,101,99,111,100,101,95,109,101,109,32,58,32,109,97,108,108,111,99,32,58,32,99,100,97,116,97,10,0,68,111,117,98,108,101,105,110,103,32,112,111,105,110,116,32,112,105,120,101,108,115,32,99,111,110,118,101,114,116,101,100,32,116,111,32,117,110,115,105,103,110,101,100,32,99,104,97,114,10,10,0,0,0,0,0,69,82,82,79,82,32,58,32,104,117,102,102,109,97,110,95,100,101,99,111,100,101,95,100,97,116,97,95,109,101,109,32,58,32,0,0,0,0,0,0,104,117,102,102,109,97,110,32,116,97,98,108,101,32,123,37,100,125,32,117,110,100,101,102,105,110,101,100,46,10,0,0,32,32,32,32,32,32,32,32,32,104,117,102,102,116,97,98,108,101,95,105,100,32,61,32,37,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,104,117,102,102,109,97,110,95,100,101,99,111,100,101,95,100,97,116,97,95,109,101,109,32,91,49,93,58,32,0,0,0,68,101,99,111,100,101,100,32,100,97,116,97,32,101,120,116,101,110,100,115,32,112,97,115,116,32,105,109,97,103,101,32,98,117,102,102,101,114,46,32,0,0,0,0,0,0,0,0,69,110,99,111,100,101,100,32,100,97,116,97,32,97,112,112,101,97,114,115,32,99,111,114,114,117,112,116,32,111,114,32,110,111,110,45,115,116,97,110,100,97,114,100,46,10,0,0,69,82,82,79,82,32,58,32,104,117,102,102,109,97,110,95,100,101,99,111,100,101,95,100,97,116,97,95,109,101,109,32,91,50,93,58,32,0,0,0,69,82,82,79,82,32,58,32,104,117,102,102,109,97,110,95,100,101,99,111,100,101,95,100,97,116,97,95,109,101,109,32,91,51,93,58,32,0,0,0,69,82,82,79,82,32,58,32,104,117,102,102,109,97,110,95,100,101,99,111,100,101,95,100,97,116,97,95,109,101,109,32,91,52,93,58,32,0,0,0,69,82,82,79,82,58,32,104,117,102,102,109,97,110,95,100,101,99,111,100,101,95,100,97,116,97,95,109,101,109,32,58,32,73,110,118,97,108,105,100,32,99,111,100,101,32,37,100,32,40,37,120,41,46,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,3,7,15,31,63,127,255,0,0,0,0,0,0,0,69,82,82,79,82,58,32,103,101,116,99,95,110,101,120,116,98,105,116,115,95,119,115,113,32,58,32,78,111,32,115,116,117,102,102,101,100,32,122,101,114,111,115,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,87,65,82,78,73,78,71,58,32,65,32,99,111,100,101,32,105,110,32,116,104,101,32,104,117,102,102,116,97,98,108,101,32,99,111,110,116,97,105,110,115,32,97,110,32,0,0,0,97,108,108,32,49,39,115,32,99,111,100,101,46,10,32,32,32,32,32,32,32,32,32,84,104,105,115,32,105,109,97,103,101,32,109,97,121,32,115,116,105,108,108,32,98,101,32,0,100,101,99,111,100,97,98,108,101,46,10,32,32,32,32,32,32,32,32,32,73,116,32,105,115,32,110,111,116,32,99,111,109,112,108,105,97,110,116,32,119,105,116,104,32,0,0,0,116,104,101,32,87,83,81,32,115,112,101,99,105,102,105,99,97,116,105,111,110,46,10,0,80,80,73,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,119,115,113,32,58,32,78,111,32,83,79,73,32,109,97,114,107,101,114,46,32,123,37,48,52,88,125,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,119,115,113,32,58,32,78,111,32,83,79,70,44,32,84,97,98,108,101,44,32,111,114,32,99,111,109,109,101,110,116,32,109,97,114,107,101,114,115,46,10,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,119,115,113,32,58,32,78,111,32,83,79,66,44,32,84,97,98,108,101,44,32,111,114,32,99,111,109,109,101,110,116,32,109,97,114,107,101,114,115,46,123,37,48,52,88,125,10,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,119,115,113,32,58,32,110,111,32,109,97,114,107,101,114,32,102,111,117,110,100,32,123,37,48,52,88,125,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,119,115,113,32,58,32,123,37,48,52,88,125,32,110,111,116,32,97,32,118,97,108,105,100,32,109,97,114,107,101,114,10,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,119,115,113,32,58,32,73,110,118,97,108,105,100,32,109,97,114,107,101,114,32,45,62,32,123,37,52,88,125,10,0,0,0,0,0,0,69,82,82,79,82,58,32,103,101,116,99,95,116,97,98,108,101,95,119,115,113,32,58,32,73,110,118,97,108,105,100,32,116,97,98,108,101,32,100,101,102,105,110,101,100,32,45,62,32,123,37,117,125,10,0,0,82,101,97,100,105,110,103,32,116,114,97,110,115,102,111,114,109,32,116,97,98,108,101,46,10,0,0,0,0,0,0,0,108,111,115,105,122,101,32,61,32,37,100,10,0,0,0,0,104,105,115,105,122,101,32,61,32,37,100,10,0,0,0,0,108,111,102,105,108,116,91,37,100,93,32,61,32,37,46,49,53,102,10,0,0,0,0,0,104,105,102,105,108,116,91,37,100,93,32,61,32,37,46,49,53,102,10,0,0,0,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,116,114,97,110,115,102,111,114,109,32,116,97,98,108,101,46,10,10,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,116,114,97,110,115,102,111,114,109,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,108,111,102,105,108,116,10,0,69,82,82,79,82,32,58,32,103,101,116,99,95,116,114,97,110,115,102,111,114,109,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,104,105,102,105,108,116,10,0,69,82,82,79,82,32,58,32,103,101,116,99,95,116,114,97,110,115,102,111,114,109,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,97,95,108,111,102,105,108,116,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,116,114,97,110,115,102,111,114,109,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,97,95,104,105,102,105,108,116,10,0,0,0,0,0,0,0,82,101,97,100,105,110,103,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,46,10,0,0,0,0,113,91,37,100,93,32,61,32,37,102,32,58,58,32,122,91,37,100,93,32,61,32,37,102,10,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,46,10,10,0,0,104,117,102,102,109,97,110,32,116,97,98,108,101,32,73,68,32,61,32,37,100,32,97,108,114,101,97,100,121,32,100,101,102,105,110,101,100,10,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,104,117,102,102,109,97,110,95,116,97,98,108,101,95,119,115,113,32,58,32,0,0,0,0,0,0,0,82,101,97,100,105,110,103,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,0,0,98,108,97,99,107,32,61,32,37,100,32,58,58,32,119,104,105,116,101,32,61,32,37,117,10,0,0,0,0,0,0,0,119,32,61,32,37,100,32,58,58,32,104,32,61,32,37,100,10,0,0,0,0,0,0,0,109,95,115,104,105,102,116,32,61,32,37,102,32,58,58,32,114,95,115,99,97,108,101,32,61,32,37,102,10,0,0,0,87,83,81,95,101,110,99,111,100,101,114,32,61,32,37,100,10,0,0,0,0,0,0,0,83,111,102,116,119,97,114,101,32,61,32,37,100,10,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,10,0,0,0,0,0,0,0,0,82,101,97,100,105,110,103,32,98,108,111,99,107,32,104,101,97,100,101,114,46,10,0,0,104,117,102,102,95,116,97,98,108,101,32,61,32,37,100,10,0,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,98,108,111,99,107,32,104,101,97,100,101,114,46,10,10,0,0,0,0,0,0,0,0,78,73,83,84,95,67,79,77,0,0,0,0,0,0,0,0,116,37,100,32,45,62,32,120,32,61,32,37,100,32,32,121,32,61,32,37,100,32,58,32,100,120,32,61,32,37,100,32,32,100,121,32,61,32,37,100,32,58,32,105,114,32,61,32,37,100,32,32,105,99,32,61,32,37,100,10,0,0,0,0,10,10,0,0,0,0,0,0,116,37,100,32,45,62,32,120,32,61,32,37,100,32,32,121,32,61,32,37,100,32,58,32,108,120,32,61,32,37,100,32,32,108,121,32,61,32,37,100,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,110,113,117,97,110,116,105,122,101,32,58,32,99,97,108,108,111,99,32,58,32,102,105,112,10,0,0,0,0,0,0,69,82,82,79,82,58,32,117,110,113,117,97,110,116,105,122,101,32,58,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,32,112,97,114,97,109,101,116,101,114,115,32,110,111,116,32,100,101,102,105,110,101,100,33,10,0,69,82,82,79,82,32,58,32,117,110,113,117,97,110,116,105,122,101,32,58,32,105,110,118,97,108,105,100,32,113,117,97,110,116,105,122,97,116,105,111,110,32,112,105,120,101,108,32,118,97,108,117,101,10,0,0,69,82,82,79,82,58,32,119,115,113,95,114,101,99,111,110,115,116,114,117,99,116,32,58,32,76,111,112,97,115,115,32,102,105,108,116,101,114,32,99,111,101,102,102,105,99,105,101,110,116,115,32,110,111,116,32,100,101,102,105,110,101,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,58,32,119,115,113,95,114,101,99,111,110,115,116,114,117,99,116,32,58,32,72,105,112,97,115,115,32,102,105,108,116,101,114,32,99,111,101,102,102,105,99,105,101,110,116,115,32,110,111,116,32,100,101,102,105,110,101,100,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,115,113,95,114,101,99,111,110,115,116,114,117,99,116,32,58,32,109,97,108,108,111,99,32,58,32,102,100,97,116,97,49,10,0,0,0,0,0,0,69,82,82,79,82,58,32,106,112,101,103,108,95,100,101,99,111,100,101,95,109,101,109,32,58,32,0,0,0,0,0,0,83,111,114,114,121,44,32,116,104,105,115,32,100,101,99,111,100,101,114,32,100,111,101,115,32,110,111,116,32,104,97,110,100,108,101,32,0,0,0,0,101,110,99,111,100,101,100,32,100,97,116,97,32,116,104,97,116,32,105,115,32,105,110,116,101,114,108,101,97,118,101,100,46,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,3,7,15,31,63,127,255,0,0,0,0,0,0,0,69,82,82,79,82,58,32,103,101,116,99,95,110,101,120,116,98,105,116,115,95,106,112,101,103,108,32,58,32,110,111,32,115,116,117,102,102,101,100,32,122,101,114,111,115,10,0,0,83,116,97,114,116,32,114,101,97,100,105,110,103,32,104,117,102,102,109,97,110,32,116,97,98,108,101,46,10,0,0,0,110,111,32,104,117,102,102,109,97,110,32,116,97,98,108,101,32,98,121,116,101,115,32,114,101,109,97,105,110,105,110,103,10,0,0,0,0,0,0,0,110,117,109,95,104,117,102,118,97,108,115,32,40,37,100,41,32,105,115,32,108,97,114,103,101,114,0,0,0,0,0,0,116,104,97,110,32,77,65,88,95,72,85,70,70,67,79,85,78,84,83,32,40,37,100,41,10,0,0,0,0,0,0,0,84,97,98,108,101,32,76,101,110,32,61,32,37,100,10,0,84,97,98,108,101,32,73,68,32,61,32,37,100,10,0,0,98,105,116,115,91,37,100,93,32,61,32,37,100,10,0,0,118,97,108,117,101,115,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,104,117,102,102,109,97,110,32,116,97,98,108,101,46,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,104,117,102,102,109,97,110,95,116,97,98,108,101,32,58,32,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,104,117,102,102,109,97,110,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,104,117,102,102,98,105,116,115,10,0,69,82,82,79,82,32,58,32,103,101,116,99,95,104,117,102,102,109,97,110,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,104,117,102,102,118,97,108,117,101,115,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,98,117,105,108,100,95,104,117,102,102,115,105,122,101,115,32,58,32,99,97,108,108,111,99,32,58,32,104,117,102,102,99,111,100,101,95,116,97,98,108,101,10,0,0,0,0,0,0,73,110,32,98,117,105,108,100,95,104,117,102,102,115,105,122,101,115,58,10,0,0,0,0,104,102,95,115,122,91,37,100,93,32,61,32,37,100,10,0,99,97,108,108,111,99,32,58,32,116,104,117,102,95,116,97,98,108,101,10,0,0,0,0,101,120,116,114,97,32,98,121,116,101,115,32,97,102,116,101,114,32,104,117,102,102,109,97,110,32,116,97,98,108,101,32,73,68,32,61,32,37,100,10,0,0,0,0,0,0,0,0,104,117,102,102,109,97,110,32,116,97,98,108,101,32,105,110,100,101,120,32,37,100,32,110,111,116,32,105,110,32,114,97,110,103,101,32,37,100,32,45,32,37,100,10,0,0,0,0,65,116,116,101,109,112,116,105,110,103,32,116,111,32,100,101,99,111,100,101,32,119,105,116,104,32,103,105,118,101,110,32,116,97,98,108,101,32,105,110,100,101,120,46,0,0,0,0,32,65,115,115,117,109,105,110,103,32,105,110,100,101,120,32,118,97,108,117,101,115,32,48,45,51,32,97,114,101,32,98,101,105,110,103,32,117,115,101,100,46,10,0,0,0,0,0,69,82,82,79,82,32,58,32,106,112,101,103,108,95,100,101,99,111,100,101,95,109,101,109,32,58,32,104,117,102,102,109,97,110,32,116,97,98,108,101,32,37,100,32,105,108,108,101,103,97,108,108,121,32,114,101,100,101,102,105,110,101,100,10,0,0,0,0,0,0,0,0,99,97,108,108,111,99,32,58,32,109,97,120,99,111,100,101,10,0,0,0,0,0,0,0,99,97,108,108,111,99,32,58,32,109,105,110,99,111,100,101,10,0,0,0,0,0,0,0,99,97,108,108,111,99,32,58,32,118,97,108,112,116,114,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,104,117,102,102,109,97,110,95,116,97,98,108,101,95,106,112,101,103,108,32,58,32,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,103,101,116,99,95,104,117,102,102,109,97,110,95,116,97,98,108,101,95,106,112,101,103,108,32,58,32,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,73,77,71,95,68,65,84,95,105,109,97,103,101,32,58,32,109,97,108,108,111,99,32,58,32,110,100,97,116,97,10,0,0,0,0,0,109,97,108,108,111,99,32,58,32,105,109,103,95,100,97,116,45,62,105,109,97,103,101,91,37,100,93,10,0,0,0,0,69,82,82,79,82,32,58,32,115,101,116,117,112,95,73,77,71,95,68,65,84,95,100,101,99,111,100,101,32,58,32,99,97,108,108,111,99,32,58,32,105,109,103,95,100,97,116,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,95,73,77,71,95,68,65,84,95,100,101,99,111,100,101,32,58,32,0,0,0,0,0,0,0,0,104,117,102,102,109,97,110,32,116,97,98,108,101,32,37,100,32,110,111,116,32,100,101,102,105,110,101,100,10,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,112,112,105,95,106,112,101,103,108,32,58,32,0,0,0,0,0,0,0,0,105,108,108,101,103,97,108,32,100,101,110,115,105,116,121,32,117,110,105,116,32,61,32,37,100,10,0,0,0,0,0,0,82,101,97,100,32,77,97,114,107,101,114,32,61,32,37,100,44,32,116,121,112,101,32,37,100,10,0,0,0,0,0,0,78,111,32,68,72,84,44,32,67,79,77,44,32,111,114,32,83,79,70,51,32,109,97,114,107,101,114,115,46,10,0,0,78,111,32,68,72,84,44,32,67,79,77,44,32,111,114,32,83,79,83,32,109,97,114,107,101,114,115,46,10,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,106,112,101,103,108,32,58,32,78,111,32,83,79,73,32,109,97,114,107,101,114,46,32,123,37,100,125,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,106,112,101,103,108,32,58,32,78,111,32,65,80,80,48,32,40,74,70,73,70,41,32,109,97,114,107,101,114,46,32,123,37,100,125,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,106,112,101,103,108,32,58,32,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,106,112,101,103,108,32,58,32,110,111,32,109,97,114,107,101,114,32,102,111,117,110,100,32,123,37,48,52,88,125,10,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,109,97,114,107,101,114,95,106,112,101,103,108,32,58,32,73,110,118,97,108,105,100,32,109,97,114,107,101,114,32,45,62,32,123,37,52,88,125,10,0,0,0,0,74,70,73,70,0,0,0,0,83,116,97,114,116,32,114,101,97,100,105,110,103,32,74,70,73,70,32,104,101,97,100,101,114,46,10,0,0,0,0,0,84,97,98,108,101,32,76,101,110,32,61,32,37,100,10,0,73,100,101,110,116,32,61,32,37,115,10,0,0,0,0,0,118,101,114,115,105,111,110,32,61,32,37,100,46,0,0,0,37,48,50,100,10,0,0,0,117,110,105,116,115,32,61,32,37,100,10,0,0,0,0,0,100,120,32,61,32,37,100,10,0,0,0,0,0,0,0,0,100,121,32,61,32,37,100,10,0,0,0,0,0,0,0,0,116,120,32,61,32,37,100,10,0,0,0,0,0,0,0,0,116,121,32,61,32,37,100,10,0,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,74,70,73,70,32,104,101,97,100,101,114,46,10,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,106,102,105,102,95,104,101,97,100,101,114,32,58,32,109,97,108,108,111,99,32,58,32,106,102,105,102,95,104,101,97,100,101,114,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,106,102,105,102,95,104,101,97,100,101,114,32,58,32,78,111,116,32,97,32,74,70,73,70,32,72,101,97,100,101,114,10,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,106,102,105,102,95,104,101,97,100,101,114,32,58,32,67,97,110,39,116,32,104,97,110,100,108,101,32,116,104,117,109,98,110,97,105,108,115,10,0,0,0,0,0,69,82,82,79,82,58,32,103,101,116,99,95,116,97,98,108,101,95,106,112,101,103,108,32,58,32,73,110,118,97,108,105,100,32,116,97,98,108,101,32,100,101,102,105,110,101,100,32,45,62,32,123,37,117,125,10,0,0,0,0,0,0,0,0,109,97,108,108,111,99,32,58,32,102,114,109,95,104,101,97,100,101,114,10,0,0,0,0,83,116,97,114,116,32,114,101,97,100,105,110,103,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,0,0,0,0,76,102,32,61,32,37,100,10,0,0,0,0,0,0,0,0,80,32,61,32,37,100,10,0,89,32,61,32,37,100,10,0,88,32,61,32,37,100,10,0,78,102,32,61,32,37,100,10,0,0,0,0,0,0,0,0,67,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,72,86,91,37,100,93,32,61,32,37,100,10,0,0,0,0,84,113,91,37,100,93,32,61,32,37,100,10,0,0,0,0,70,105,110,105,115,104,101,100,32,102,114,97,109,101,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,10,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,102,114,97,109,101,95,104,101,97,100,101,114,95,106,112,101,103,108,32,58,32,0,0,0,0,0,0,83,116,97,114,116,32,114,101,97,100,105,110,103,32,115,99,97,110,32,104,101,97,100,101,114,10,0,0,0,0,0,0,76,115,32,61,32,37,100,10,0,0,0,0,0,0,0,0,78,115,32,61,32,37,100,10,0,0,0,0,0,0,0,0,67,115,91,37,100,93,32,61,32,37,100,10,0,0,0,0,84,100,97,91,37,100,93,32,61,32,37,100,10,0,0,0,83,115,32,61,32,37,100,10,0,0,0,0,0,0,0,0,83,101,32,61,32,37,100,10,0,0,0,0,0,0,0,0,65,104,108,32,61,32,37,100,10,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,114,101,97,100,105,110,103,32,115,99,97,110,32,104,101,97,100,101,114,10,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,115,99,97,110,95,104,101,97,100,101,114,32,58,32,109,97,108,108,111,99,32,58,32,115,99,110,95,104,101,97,100,101,114,10,0,82,101,97,100,105,110,103,32,67,111,109,109,101,110,116,32,70,105,101,108,100,46,10,0,67,111,109,109,101,110,116,32,61,32,32,37,115,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,99,111,109,109,101,110,116,32,58,32,109,97,108,108,111,99,32,58,32,99,111,109,109,101,110,116,10,0,0,0,0,0,0,0,0,69,82,79,82,82,32,58,32,112,114,101,100,105,99,116,32,58,32,105,110,118,97,108,105,100,32,112,114,101,100,105,99,116,105,111,110,32,116,121,112,101,32,0,0,0,0,0,0,37,100,32,110,111,116,32,105,110,32,114,97,110,103,101,32,91,37,100,46,46,37,100,93,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,106,112,101,103,95,100,101,99,111,100,101,95,109,101,109,32,58,32,109,97,108,108,111,99,32,58,32,111,117,116,95,98,117,102,102,101,114,10,0,0,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,128,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,0,0,0,255,255,255,255,253,255,255,255,249,255,255,255,241,255,255,255,225,255,255,255,193,255,255,255,129,255,255,255,1,255,255,255,1,254,255,255,1,252,255,255,1,248,255,255,1,240,255,255,1,224,255,255,1,192,255,255,1,128,255,255,66,111,103,117,115,32,109,101,115,115,97,103,101,32,99,111,100,101,32,37,100,0,0,0,83,111,114,114,121,44,32,116,104,101,114,101,32,97,114,101,32,108,101,103,97,108,32,114,101,115,116,114,105,99,116,105,111,110,115,32,111,110,32,97,114,105,116,104,109,101,116,105,99,32,99,111,100,105,110,103,0,0,0,0,0,0,0,0,65,76,73,71,78,95,84,89,80,69,32,105,115,32,119,114,111,110,103,44,32,112,108,101,97,115,101,32,102,105,120,0,77,65,88,95,65,76,76,79,67,95,67,72,85,78,75,32,105,115,32,119,114,111,110,103,44,32,112,108,101,97,115,101,32,102,105,120,0,0,0,0,66,111,103,117,115,32,98,117,102,102,101,114,32,99,111,110,116,114,111,108,32,109,111,100,101,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,111,109,112,111,110,101,110,116,32,73,68,32,37,100,32,105,110,32,83,79,83,0,0,68,67,84,32,99,111,101,102,102,105,99,105,101,110,116,32,111,117,116,32,111,102,32,114,97,110,103,101,0,0,0,0,73,68,67,84,32,111,117,116,112,117,116,32,98,108,111,99,107,32,115,105,122,101,32,37,100,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,66,111,103,117,115,32,72,117,102,102,109,97,110,32,116,97,98,108,101,32,100,101,102,105,110,105,116,105,111,110,0,0,66,111,103,117,115,32,105,110,112,117,116,32,99,111,108,111,114,115,112,97,99,101,0,0,66,111,103,117,115,32,74,80,69,71,32,99,111,108,111,114,115,112,97,99,101,0,0,0,66,111,103,117,115,32,109,97,114,107,101,114,32,108,101,110,103,116,104,0,0,0,0,0,87,114,111,110,103,32,74,80,69,71,32,108,105,98,114,97,114,121,32,118,101,114,115,105,111,110,58,32,108,105,98,114,97,114,121,32,105,115,32,37,100,44,32,99,97,108,108,101,114,32,101,120,112,101,99,116,115,32,37,100,0,0,0,0,83,97,109,112,108,105,110,103,32,102,97,99,116,111,114,115,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,105,110,116,101,114,108,101,97,118,101,100,32,115,99,97,110,0,73,110,118,97,108,105,100,32,109,101,109,111,114,121,32,112,111,111,108,32,99,111,100,101,32,37,100,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,74,80,69,71,32,100,97,116,97,32,112,114,101,99,105,115,105,111,110,32,37,100,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,114,111,103,114,101,115,115,105,118,101,32,112,97,114,97,109,101,116,101,114,115,32,83,115,61,37,100,32,83,101,61,37,100,32,65,104,61,37,100,32,65,108,61,37,100,0,0,73,110,118,97,108,105,100,32,112,114,111,103,114,101,115,115,105,118,101,32,112,97,114,97,109,101,116,101,114,115,32,97,116,32,115,99,97,110,32,115,99,114,105,112,116,32,101,110,116,114,121,32,37,100,0,0,66,111,103,117,115,32,115,97,109,112,108,105,110,103,32,102,97,99,116,111,114,115,0,0,73,110,118,97,108,105,100,32,115,99,97,110,32,115,99,114,105,112,116,32,97,116,32,101,110,116,114,121,32,37,100,0,73,109,112,114,111,112,101,114,32,99,97,108,108,32,116,111,32,74,80,69,71,32,108,105,98,114,97,114,121,32,105,110,32,115,116,97,116,101,32,37,100,0,0,0,0,0,0,0,74,80,69,71,32,112,97,114,97,109,101,116,101,114,32,115,116,114,117,99,116,32,109,105,115,109,97,116,99,104,58,32,108,105,98,114,97,114,121,32,116,104,105,110,107,115,32,115,105,122,101,32,105,115,32,37,117,44,32,99,97,108,108,101,114,32,101,120,112,101,99,116,115,32,37,117,0,0,0,0,66,111,103,117,115,32,118,105,114,116,117,97,108,32,97,114,114,97,121,32,97,99,99,101,115,115,0,0,0,0,0,0,66,117,102,102,101,114,32,112,97,115,115,101,100,32,116,111,32,74,80,69,71,32,108,105,98,114,97,114,121,32,105,115,32,116,111,111,32,115,109,97,108,108,0,0,0,0,0,0,83,117,115,112,101,110,115,105,111,110,32,110,111,116,32,97,108,108,111,119,101,100,32,104,101,114,101,0,0,0,0,0,67,67,73,82,54,48,49,32,115,97,109,112,108,105,110,103,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,0,0,0,0,84,111,111,32,109,97,110,121,32,99,111,108,111,114,32,99,111,109,112,111,110,101,110,116,115,58,32,37,100,44,32,109,97,120,32,37,100,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,99,111,108,111,114,32,99,111,110,118,101,114,115,105,111,110,32,114,101,113,117,101,115,116,0,0,0,0,66,111,103,117,115,32,68,65,67,32,105,110,100,101,120,32,37,100,0,0,0,0,0,0,66,111,103,117,115,32,68,65,67,32,118,97,108,117,101,32,48,120,37,120,0,0,0,0,66,111,103,117,115,32,68,72,84,32,105,110,100,101,120,32,37,100,0,0,0,0,0,0,66,111,103,117,115,32,68,81,84,32,105,110,100,101,120,32,37,100,0,0,0,0,0,0,69,109,112,116,121,32,74,80,69,71,32,105,109,97,103,101,32,40,68,78,76,32,110,111,116,32,115,117,112,112,111,114,116,101,100,41,0,0,0,0,82,101,97,100,32,102,114,111,109,32,69,77,83,32,102,97,105,108,101,100,0,0,0,0,87,114,105,116,101,32,116,111,32,69,77,83,32,102,97,105,108,101,100,0,0,0,0,0,68,105,100,110,39,116,32,101,120,112,101,99,116,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,115,99,97,110,0,0,0,0,0,0,0,0,73,110,112,117,116,32,102,105,108,101,32,114,101,97,100,32,101,114,114,111,114,0,0,0,79,117,116,112,117,116,32,102,105,108,101,32,119,114,105,116,101,32,101,114,114,111,114,32,45,45,45,32,111,117,116,32,111,102,32,100,105,115,107,32,115,112,97,99,101,63,0,0,70,114,97,99,116,105,111,110,97,108,32,115,97,109,112,108,105,110,103,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,0,72,117,102,102,109,97,110,32,99,111,100,101,32,115,105,122,101,32,116,97,98,108,101,32,111,118,101,114,102,108,111,119,0,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,72,117,102,102,109,97,110,32,99,111,100,101,32,116,97,98,108,101,32,101,110,116,114,121,0,0,0,0,0,0,0,0,77,97,120,105,109,117,109,32,115,117,112,112,111,114,116,101,100,32,105,109,97,103,101,32,100,105,109,101,110,115,105,111,110,32,105,115,32,37,117,32,112,105,120,101,108,115,0,0,69,109,112,116,121,32,105,110,112,117,116,32,102,105,108,101,0,0,0,0,0,0,0,0,80,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,105,110,112,117,116,32,102,105,108,101,0,0,0,0,0,67,97,110,110,111,116,32,116,114,97,110,115,99,111,100,101,32,100,117,101,32,116,111,32,109,117,108,116,105,112,108,101,32,117,115,101,32,111,102,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,32,37,100,0,0,0,83,99,97,110,32,115,99,114,105,112,116,32,100,111,101,115,32,110,111,116,32,116,114,97,110,115,109,105,116,32,97,108,108,32,100,97,116,97,0,0,73,110,118,97,108,105,100,32,99,111,108,111,114,32,113,117,97,110,116,105,122,97,116,105,111,110,32,109,111,100,101,32,99,104,97,110,103,101,0,0,78,111,116,32,105,109,112,108,101,109,101,110,116,101,100,32,121,101,116,0,0,0,0,0,82,101,113,117,101,115,116,101,100,32,102,101,97,116,117,114,101,32,119,97,115,32,111,109,105,116,116,101,100,32,97,116,32,99,111,109,112,105,108,101,32,116,105,109,101,0,0,0,66,97,99,107,105,110,103,32,115,116,111,114,101,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,0,0,0,0,72,117,102,102,109,97,110,32,116,97,98,108,101,32,48,120,37,48,50,120,32,119,97,115,32,110,111,116,32,100,101,102,105,110,101,100,0,0,0,0,74,80,69,71,32,100,97,116,97,115,116,114,101,97,109,32,99,111,110,116,97,105,110,115,32,110,111,32,105,109,97,103,101,0,0,0,0,0,0,0,81,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,32,48,120,37,48,50,120,32,119,97,115,32,110,111,116,32,100,101,102,105,110,101,100,0,0,0,0,0,0,0,78,111,116,32,97,32,74,80,69,71,32,102,105,108,101,58,32,115,116,97,114,116,115,32,119,105,116,104,32,48,120,37,48,50,120,32,48,120,37,48,50,120,0,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,40,99,97,115,101,32,37,100,41,0,0,0,67,97,110,110,111,116,32,113,117,97,110,116,105,122,101,32,109,111,114,101,32,116,104,97,110,32,37,100,32,99,111,108,111,114,32,99,111,109,112,111,110,101,110,116,115,0,0,0,67,97,110,110,111,116,32,113,117,97,110,116,105,122,101,32,116,111,32,102,101,119,101,114,32,116,104,97,110,32,37,100,32,99,111,108,111,114,115,0,67,97,110,110,111,116,32,113,117,97,110,116,105,122,101,32,116,111,32,109,111,114,101,32,116,104,97,110,32,37,100,32,99,111,108,111,114,115,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,116,119,111,32,83,79,70,32,109,97,114,107,101,114,115,0,0,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,109,105,115,115,105,110,103,32,83,79,83,32,109,97,114,107,101,114,0,85,110,115,117,112,112,111,114,116,101,100,32,74,80,69,71,32,112,114,111,99,101,115,115,58,32,83,79,70,32,116,121,112,101,32,48,120,37,48,50,120,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,116,119,111,32,83,79,73,32,109,97,114,107,101,114,115,0,0,0,0,73,110,118,97,108,105,100,32,74,80,69,71,32,102,105,108,101,32,115,116,114,117,99,116,117,114,101,58,32,83,79,83,32,98,101,102,111,114,101,32,83,79,70,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,99,114,101,97,116,101,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,37,115,0,0,0,0,0,0,82,101,97,100,32,102,97,105,108,101,100,32,111,110,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,0,0,0,83,101,101,107,32,102,97,105,108,101,100,32,111,110,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,0,0,0,87,114,105,116,101,32,102,97,105,108,101,100,32,111,110,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,45,45,45,32,111,117,116,32,111,102,32,100,105,115,107,32,115,112,97,99,101,63,0,0,0,65,112,112,108,105,99,97,116,105,111,110,32,116,114,97,110,115,102,101,114,114,101,100,32,116,111,111,32,102,101,119,32,115,99,97,110,108,105,110,101,115,0,0,0,0,0,0,0,85,110,115,117,112,112,111,114,116,101,100,32,109,97,114,107,101,114,32,116,121,112,101,32,48,120,37,48,50,120,0,0,86,105,114,116,117,97,108,32,97,114,114,97,121,32,99,111,110,116,114,111,108,108,101,114,32,109,101,115,115,101,100,32,117,112,0,0,0,0,0,0,73,109,97,103,101,32,116,111,111,32,119,105,100,101,32,102,111,114,32,116,104,105,115,32,105,109,112,108,101,109,101,110,116,97,116,105,111,110,0,0,82,101,97,100,32,102,114,111,109,32,88,77,83,32,102,97,105,108,101,100,0,0,0,0,87,114,105,116,101,32,116,111,32,88,77,83,32,102,97,105,108,101,100,0,0,0,0,0,67,111,112,121,114,105,103,104,116,32,40,67,41,32,49,57,57,56,44,32,84,104,111,109,97,115,32,71,46,32,76,97,110,101,0,0,0,0,0,0,54,98,32,32,50,55,45,77,97,114,45,49,57,57,56,0,67,97,117,116,105,111,110,58,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,115,32,97,114,101,32,116,111,111,32,99,111,97,114,115,101,32,102,111,114,32,98,97,115,101,108,105,110,101,32,74,80,69,71,0,0,0,65,100,111,98,101,32,65,80,80,49,52,32,109,97,114,107,101,114,58,32,118,101,114,115,105,111,110,32,37,100,44,32,102,108,97,103,115,32,48,120,37,48,52,120,32,48,120,37,48,52,120,44,32,116,114,97,110,115,102,111,114,109,32,37,100,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,65,80,80,48,32,109,97,114,107,101,114,32,40,110,111,116,32,74,70,73,70,41,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,65,80,80,49,52,32,109,97,114,107,101,114,32,40,110,111,116,32,65,100,111,98,101,41,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,68,101,102,105,110,101,32,65,114,105,116,104,109,101,116,105,99,32,84,97,98,108,101,32,48,120,37,48,50,120,58,32,48,120,37,48,50,120,0,0,68,101,102,105,110,101,32,72,117,102,102,109,97,110,32,84,97,98,108,101,32,48,120,37,48,50,120,0,0,0,0,0,68,101,102,105,110,101,32,81,117,97,110,116,105,122,97,116,105,111,110,32,84,97,98,108,101,32,37,100,32,32,112,114,101,99,105,115,105,111,110,32,37,100,0,0,0,0,0,0,68,101,102,105,110,101,32,82,101,115,116,97,114,116,32,73,110,116,101,114,118,97,108,32,37,117,0,0,0,0,0,0,70,114,101,101,100,32,69,77,83,32,104,97,110,100,108,101,32,37,117,0,0,0,0,0,79,98,116,97,105,110,101,100,32,69,77,83,32,104,97,110,100,108,101,32,37,117,0,0,69,110,100,32,79,102,32,73,109,97,103,101,0,0,0,0,32,32,32,32,32,32,32,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,32,37,51,100,0,74,70,73,70,32,65,80,80,48,32,109,97,114,107,101,114,58,32,118,101,114,115,105,111,110,32,37,100,46,37,48,50,100,44,32,100,101,110,115,105,116,121,32,37,100,120,37,100,32,32,37,100,0,0,0,0,87,97,114,110,105,110,103,58,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,32,115,105,122,101,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,100,97,116,97,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,116,121,112,101,32,48,120,37,48,50,120,44,32,108,101,110,103,116,104,32,37,117,0,0,0,32,32,32,32,119,105,116,104,32,37,100,32,120,32,37,100,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,0,0,0,0,0,0,0,0,77,105,115,99,101,108,108,97,110,101,111,117,115,32,109,97,114,107,101,114,32,48,120,37,48,50,120,44,32,108,101,110,103,116,104,32,37,117,0,0,85,110,101,120,112,101,99,116,101,100,32,109,97,114,107,101,114,32,48,120,37,48,50,120,0,0,0,0,0,0,0,0,32,32,32,32,32,32,32,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,32,37,52,117,0,81,117,97,110,116,105,122,105,110,103,32,116,111,32,37,100,32,61,32,37,100,42,37,100,42,37,100,32,99,111,108,111,114,115,0,0,0,0,0,0,81,117,97,110,116,105,122,105,110,103,32,116,111,32,37,100,32,99,111,108,111,114,115,0,83,101,108,101,99,116,101,100,32,37,100,32,99,111,108,111,114,115,32,102,111,114,32,113,117,97,110,116,105,122,97,116,105,111,110,0,0,0,0,0,65,116,32,109,97,114,107,101,114,32,48,120,37,48,50,120,44,32,114,101,99,111,118,101,114,121,32,97,99,116,105,111,110,32,37,100,0,0,0,0,82,83,84,37,100,0,0,0,83,109,111,111,116,104,105,110,103,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,119,105,116,104,32,110,111,110,115,116,97,110,100,97,114,100,32,115,97,109,112,108,105,110,103,32,114,97,116,105,111,115,0,0,0,0,0,0,0,0,83,116,97,114,116,32,79,102,32,70,114,97,109,101,32,48,120,37,48,50,120,58,32,119,105,100,116,104,61,37,117,44,32,104,101,105,103,104,116,61,37,117,44,32,99,111,109,112,111,110,101,110,116,115,61,37,100,0,0,0,0,0,0,0,32,32,32,32,67,111,109,112,111,110,101,110,116,32,37,100,58,32,37,100,104,120,37,100,118,32,113,61,37,100,0,0,83,116,97,114,116,32,111,102,32,73,109,97,103,101,0,0,83,116,97,114,116,32,79,102,32,83,99,97,110,58,32,37,100,32,99,111,109,112,111,110,101,110,116,115,0,0,0,0,32,32,32,32,67,111,109,112,111,110,101,110,116,32,37,100,58,32,100,99,61,37,100,32,97,99,61,37,100,0,0,0,32,32,83,115,61,37,100,44,32,83,101,61,37,100,44,32,65,104,61,37,100,44,32,65,108,61,37,100,0,0,0,0,67,108,111,115,101,100,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,37,115,0,0,0,0,0,0,0,0,79,112,101,110,101,100,32,116,101,109,112,111,114,97,114,121,32,102,105,108,101,32,37,115,0,0,0,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,74,80,69,71,45,99,111,109,112,114,101,115,115,101,100,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,112,97,108,101,116,116,101,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,44,32,108,101,110,103,116,104,32,37,117,0,0,0,0,0,0,0,74,70,73,70,32,101,120,116,101,110,115,105,111,110,32,109,97,114,107,101,114,58,32,82,71,66,32,116,104,117,109,98,110,97,105,108,32,105,109,97,103,101,44,32,108,101,110,103,116,104,32,37,117,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,99,111,109,112,111,110,101,110,116,32,73,68,115,32,37,100,32,37,100,32,37,100,44,32,97,115,115,117,109,105,110,103,32,89,67,98,67,114,0,0,0,0,0,70,114,101,101,100,32,88,77,83,32,104,97,110,100,108,101,32,37,117,0,0,0,0,0,79,98,116,97,105,110,101,100,32,88,77,83,32,104,97,110,100,108,101,32,37,117,0,0,85,110,107,110,111,119,110,32,65,100,111,98,101,32,99,111,108,111,114,32,116,114,97,110,115,102,111,114,109,32,99,111,100,101,32,37,100,0,0,0,73,110,99,111,110,115,105,115,116,101,110,116,32,112,114,111,103,114,101,115,115,105,111,110,32,115,101,113,117,101,110,99,101,32,102,111,114,32,99,111,109,112,111,110,101,110,116,32,37,100,32,99,111,101,102,102,105,99,105,101,110,116,32,37,100,0,0,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,37,117,32,101,120,116,114,97,110,101,111,117,115,32,98,121,116,101,115,32,98,101,102,111,114,101,32,109,97,114,107,101,114,32,48,120,37,48,50,120,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,112,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,100,97,116,97,32,115,101,103,109,101,110,116,0,0,0,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,98,97,100,32,72,117,102,102,109,97,110,32,99,111,100,101,0,0,0,0,0,87,97,114,110,105,110,103,58,32,117,110,107,110,111,119,110,32,74,70,73,70,32,114,101,118,105,115,105,111,110,32,110,117,109,98,101,114,32,37,100,46,37,48,50,100,0,0,0,80,114,101,109,97,116,117,114,101,32,101,110,100,32,111,102,32,74,80,69,71,32,102,105,108,101,0,0,0,0,0,0,67,111,114,114,117,112,116,32,74,80,69,71,32,100,97,116,97,58,32,102,111,117,110,100,32,109,97,114,107,101,114,32,48,120,37,48,50,120,32,105,110,115,116,101,97,100,32,111,102,32,82,83,84,37,100,0,73,110,118,97,108,105,100,32,83,79,83,32,112,97,114,97,109,101,116,101,114,115,32,102,111,114,32,115,101,113,117,101,110,116,105,97,108,32,74,80,69,71,0,0,0,0,0,0,65,112,112,108,105,99,97,116,105,111,110,32,116,114,97,110,115,102,101,114,114,101,100,32,116,111,111,32,109,97,110,121,32,115,99,97,110,108,105,110,101,115,0,0,0,0,0,0,168,174,0,0,192,174,0,0,0,175,0,0,32,175,0,0,72,175,0,0,104,175,0,0,136,175,0,0,168,175,0,0,208,175,0,0,240,175,0,0,8,176,0,0,32,176,0,0,56,176,0,0,120,176,0,0,168,176,0,0,200,176,0,0,240,176,0,0,40,177,0,0,96,177,0,0,120,177,0,0,152,177,0,0,200,177,0,0,24,178,0,0,56,178,0,0,104,178,0,0,136,178,0,0,176,178,0,0,216,178,0,0,0,179,0,0,24,179,0,0,48,179,0,0,72,179,0,0,96,179,0,0,136,179,0,0,160,179,0,0,184,179,0,0,224,179,0,0,248,179,0,0,40,180,0,0,80,180,0,0,120,180,0,0,160,180,0,0,208,180,0,0,232,180,0,0,8,181,0,0,72,181,0,0,112,181,0,0,152,181,0,0,176,181,0,0,224,181,0,0,0,182,0,0,40,182,0,0,80,182,0,0,128,182,0,0,176,182,0,0,208,182,0,0,0,183,0,0,40,183,0,0,80,183,0,0,128,183,0,0,176,183,0,0,224,183,0,0,16,184,0,0,64,184,0,0,104,184,0,0,136,184,0,0,168,184,0,0,224,184,0,0,16,185,0,0,48,185,0,0,88,185,0,0,128,185,0,0,152,185,0,0,176,185,0,0,216,185,0,0,232,185,0,0,40,186,0,0,112,186,0,0,160,186,0,0,208,186,0,0,248,186,0,0,24,187,0,0,72,187,0,0,104,187,0,0,128,187,0,0,152,187,0,0,168,187,0,0,208,187,0,0,8,188,0,0,72,188,0,0,120,188,0,0,160,188,0,0,200,188,0,0,232,188,0,0,16,189,0,0,56,189,0,0,80,189,0,0,120,189,0,0,160,189,0,0,168,189,0,0,232,189,0,0,40,190,0,0,72,190,0,0,88,190,0,0,120,190,0,0,152,190,0,0,184,190,0,0,216,190,0,0,248,190,0,0,64,191,0,0,128,191,0,0,184,191,0,0,240,191,0,0,8,192,0,0,32,192,0,0,72,192,0,0,144,192,0,0,208,192,0,0,8,193,0,0,48,193,0,0,96,193,0,0,128,193,0,0,184,193,0,0,232,193,0,0,0,0,0,0,0,0,0,0,37,115,10,0,0,0,0,0,74,80,69,71,77,69,77,0,37,108,100,37,99,0,0,0,64,6,0,0,128,62,0,0,0,0,0,0,136,19,0,0,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,192,48,240,12,204,60,252,3,195,51,243,15,207,63,255,128,64,176,112,140,76,188,124,131,67,179,115,143,79,191,127,32,224,16,208,44,236,28,220,35,227,19,211,47,239,31,223,160,96,144,80,172,108,156,92,163,99,147,83,175,111,159,95,8,200,56,248,4,196,52,244,11,203,59,251,7,199,55,247,136,72,184,120,132,68,180,116,139,75,187,123,135,71,183,119,40,232,24,216,36,228,20,212,43,235,27,219,39,231,23,215,168,104,152,88,164,100,148,84,171,107,155,91,167,103,151,87,2,194,50,242,14,206,62,254,1,193,49,241,13,205,61,253,130,66,178,114,142,78,190,126,129,65,177,113,141,77,189,125,34,226,18,210,46,238,30,222,33,225,17,209,45,237,29,221,162,98,146,82,174,110,158,94,161,97,145,81,173,109,157,93,10,202,58,250,6,198,54,246,9,201,57,249,5,197,53,245,138,74,186,122,134,70,182,118,137,73,185,121,133,69,181,117,42,234,26,218,38,230,22,214,41,233,25,217,37,229,21,213,170,106,154,90,166,102,150,86,169,105,153,89,165,101,149,85,0,0,0,0,1,0,0,0,8,0,0,0,16,0,0,0,9,0,0,0,2,0,0,0,3,0,0,0,10,0,0,0,17,0,0,0,24,0,0,0,32,0,0,0,25,0,0,0,18,0,0,0,11,0,0,0,4,0,0,0,5,0,0,0,12,0,0,0,19,0,0,0,26,0,0,0,33,0,0,0,40,0,0,0,48,0,0,0,41,0,0,0,34,0,0,0,27,0,0,0,20,0,0,0,13,0,0,0,6,0,0,0,7,0,0,0,14,0,0,0,21,0,0,0,28,0,0,0,35,0,0,0,42,0,0,0,49,0,0,0,56,0,0,0,57,0,0,0,50,0,0,0,43,0,0,0,36,0,0,0,29,0,0,0,22,0,0,0,15,0,0,0,23,0,0,0,30,0,0,0,37,0,0,0,44,0,0,0,51,0,0,0,58,0,0,0,59,0,0,0,52,0,0,0,45,0,0,0,38,0,0,0,31,0,0,0,39,0,0,0,46,0,0,0,53,0,0,0,60,0,0,0,61,0,0,0,54,0,0,0,47,0,0,0,55,0,0,0,62,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,63,0,0,0,69,82,82,79,82,32,58,32,103,101,116,95,112,112,105,95,106,112,101,103,98,32,58,32,0,0,0,0,0,0,0,0,105,108,108,101,103,97,108,32,100,101,110,115,105,116,121,32,117,110,105,116,32,61,32,37,100,10,0,0,0,0,0,0,0,64,197,88,159,83,66,75,0,64,73,50,163,34,168,17,197,88,33,123,252,115,98,104,197,88,191,69,11,48,126,24,159,83,252,115,65,109,84,98,159,83,179,65,65,45,18,23,66,75,98,104,84,98,126,88,66,75,33,59,186,40,195,20,0,64,197,88,159,83,66,75,0,64,73,50,163,34,168,17,73,50,191,69,179,65,33,59,73,50,130,39,55,27,224,13,163,34,11,48,65,45,186,40,163,34,55,27,191,18,142,9,168,17,126,24,18,23,195,20,168,17,224,13,142,9,223,4,0,0,0,0,0,0,240,63,239,97,72,177,80,49,246,63,202,111,77,145,174,231,244,63,170,17,108,239,98,208,242,63,0,0,0,0,0,0,240,63,59,191,167,192,105,36,233,63,187,32,199,123,122,81,225,63,93,171,114,222,85,168,209,63,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,128,0,0,0,0,1,0,0,0,2,0,0,0,4,0,0,0,8,0,0,0,16,0,0,0,32,0,0,0,64,0,0,0,0,0,0,255,255,255,255,253,255,255,255,249,255,255,255,241,255,255,255,225,255,255,255,193,255,255,255,129,255,255,255,1,255,255,255,1,254,255,255,1,252,255,255,1,248,255,255,1,240,255,255,1,224,255,255,1,192,255,255,1,128,255,255,69,82,82,79,82,32,58,32,101,120,116,114,97,99,116,102,101,116,95,114,101,116,32,58,32,102,101,97,116,117,114,101,32,37,115,32,110,111,116,32,102,111,117,110,100,10,0,0,69,82,82,79,82,32,58,32,101,120,116,114,97,99,116,102,101,116,95,114,101,116,32,58,32,115,116,114,100,117,112,32,58,32,118,97,108,117,101,10,0,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,115,116,114,105,110,103,50,102,101,116,32,58,32,101,109,112,116,121,32,110,97,109,101,32,115,116,114,105,110,103,32,102,111,117,110,100,10,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,102,101,116,95,114,101,116,32,58,32,115,116,114,100,117,112,32,58,32,102,101,116,45,62,118,97,108,117,101,115,91,93,10,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,102,101,116,95,114,101,116,32,58,32,115,116,114,100,117,112,32,58,32,102,101,116,45,62,110,97,109,101,115,91,93,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,109,97,108,108,111,99,32,58,32,102,101,116,10,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,99,97,108,108,111,99,32,58,32,102,101,116,45,62,110,97,109,101,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,99,97,108,108,111,99,32,58,32,102,101,116,45,62,118,97,108,117,101,115,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,114,101,97,108,108,111,99,32,58,32,102,101,116,45,62,110,97,109,101,115,10,0,0,69,82,82,79,82,32,58,32,114,101,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,114,101,97,108,108,111,99,32,58,32,102,101,116,45,62,118,97,108,117,101,115,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,98,121,116,101,32,58,32,112,114,101,109,97,116,117,114,101,32,69,110,100,32,79,102,32,66,117,102,102,101,114,10,0,0,0,0,69,82,82,79,82,32,58,32,103,101,116,99,95,98,121,116,101,115,32,58,32,112,114,101,109,97,116,117,114,101,32,69,110,100,32,79,102,32,66,117,102,102,101,114,10,0,0,0,114,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,102,105,108,101,115,105,122,101,32,58,32,102,105,108,101,32,37,115,32,99,111,117,108,100,32,110,111,116,32,98,101,32,111,112,101,110,101,100,10,0,69,82,82,79,82,32,58,32,102,105,108,101,115,105,122,101,32,58,32,115,101,101,107,105,110,103,32,116,111,32,69,79,70,32,111,102,32,102,105,108,101,32,37,115,32,102,97,105,108,101,100,10,0,0,0,0,69,82,82,79,82,32,58,32,102,105,108,101,115,105,122,101,32,58,32,102,116,101,108,108,32,97,116,32,69,79,70,32,111,102,32,102,105,108,101,32,37,115,32,102,97,105,108,101,100,10,0,0,0,0,0,0,109,97,114,107,101,114,32,37,100,32,115,101,103,109,101,110,116,32,111,102,32,108,101,110,103,116,104,32,37,100,10,0,69,82,82,79,82,32,58,32,103,101,116,99,95,115,107,105,112,95,109,97,114,107,101,114,95,115,101,103,109,101,110,116,32,58,32,0,0,0,0,0,117,110,101,120,112,101,99,116,101,100,32,101,110,100,32,111,102,32,98,117,102,102,101,114,32,119,104,101,110,32,112,97,114,115,105,110,103,32,0,0,69,82,82,79,82,58,32,37,115,10,0,0,0,0,0,0,69,82,82,79,82,58,32,37,115,58,32,37,115,10,0,0,69,82,82,79,82,58,32,37,115,58,32,37,115,58,32,37,115,10,0,0,0,0,0,0,0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,1,129,65,193,33,161,97,225,17,145,81,209,49,177,113,241,9,137,73,201,41,169,105,233,25,153,89,217,57,185,121,249,5,133,69,197,37,165,101,229,21,149,85,213,53,181,117,245,13,141,77,205,45,173,109,237,29,157,93,221,61,189,125,253,3,131,67,195,35,163,99,227,19,147,83,211,51,179,115,243,11,139,75,203,43,171,107,235,27,155,91,219,59,187,123,251,7,135,71,199,39,167,103,231,23,151,87,215,55,183,119,247,15,143,79,207,47,175,111,239,31,159,95,223,63,191,127,255,69,114,114,111,114,32,97,108,108,111,99,97,116,105,110,103,32,109,101,109,111,114,121,32,102,111,114,32,99,111,109,112,114,101,115,115,101,100,32,98,105,116,115,116,114,101,97,109,10,0,0,0,0,0,0,0,114,101,97,100,32,101,114,114,111,114,58,32,112,97,115,115,101,100,32,116,104,101,32,101,110,100,32,111,102,32,116,104,101,32,99,111,100,101,115,116,114,101,97,109,32,40,115,116,97,114,116,32,61,32,37,100,44,32,99,117,114,114,101,110,116,32,61,32,37,100,44,32,101,110,100,32,61,32,37,100,10,0,0,0,0,0,0,0,79,255,0,0,1,0,0,0,1,0,0,0,144,255,0,0,12,0,0,0,2,0,0,0,147,255,0,0,16,0,0,0,3,0,0,0,217,255,0,0,8,0,0,0,4,0,0,0,81,255,0,0,2,0,0,0,5,0,0,0,82,255,0,0,20,0,0,0,6,0,0,0,83,255,0,0,20,0,0,0,7,0,0,0,94,255,0,0,20,0,0,0,8,0,0,0,92,255,0,0,20,0,0,0,9,0,0,0,93,255,0,0,20,0,0,0,10,0,0,0,95,255,0,0,20,0,0,0,11,0,0,0,85,255,0,0,4,0,0,0,12,0,0,0,87,255,0,0,4,0,0,0,13,0,0,0,88,255,0,0,16,0,0,0,14,0,0,0,96,255,0,0,4,0,0,0,15,0,0,0,97,255,0,0,16,0,0,0,16,0,0,0,145,255,0,0,0,0,0,0,0,0,0,0,99,255,0,0,4,0,0,0,17,0,0,0,100,255,0,0,20,0,0,0,18,0,0,0,0,0,0,0,20,0,0,0,19,0,0,0,37,46,56,120,58,32,101,120,112,101,99,116,101,100,32,97,32,109,97,114,107,101,114,32,105,110,115,116,101,97,100,32,111,102,32,37,120,10,0,0,37,46,56,120,58,32,117,110,101,120,112,101,99,116,101,100,32,109,97,114,107,101,114,32,37,120,10,0,0,0,0,0,77,97,105,110,32,72,101,97,100,101,114,32,100,101,99,111,100,101,100,46,10,0,0,0,73,110,99,111,109,112,108,101,116,101,32,98,105,116,115,116,114,101,97,109,10,0,0,0,91,74,80,84,45,115,116,114,101,97,109,93,32,58,32,69,120,112,101,99,116,105,110,103,32,77,97,105,110,32,104,101,97,100,101,114,32,102,105,114,115,116,32,91,99,108,97,115,115,95,73,100,32,37,100,93,32,33,10,0,0,0,0,0,91,74,80,84,45,115,116,114,101,97,109,93,32,58,32,69,120,112,101,99,116,105,110,103,32,84,105,108,101,32,105,110,102,111,32,33,10,0,0,0,85,110,107,110,111,119,110,32,109,97,114,107,101,114,10,0,69,114,114,111,114,32,100,101,99,111,100,105,110,103,32,99,111,109,112,111,110,101,110,116,32,37,100,46,10,84,104,101,32,110,117,109,98,101,114,32,111,102,32,114,101,115,111,108,117,116,105,111,110,115,32,116,111,32,114,101,109,111,118,101,32,105,115,32,104,105,103,104,101,114,32,116,104,97,110,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,114,101,115,111,108,117,116,105,111,110,115,32,111,102,32,116,104,105,115,32,99,111,109,112,111,110,101,110,116,10,77,111,100,105,102,121,32,116,104,101,32,99,112,95,114,101,100,117,99,101,32,112,97,114,97,109,101,116,101,114,46,10,10,0,0,0,37,115,58,32,105,110,118,97,108,105,100,32,105,109,97,103,101,32,115,105,122,101,32,40,120,48,58,37,100,44,32,120,49,58,37,100,44,32,121,48,58,37,100,44,32,121,49,58,37,100,41,10,0,0,0,0,69,120,112,101,99,116,101,100,32,74,80,50,72,32,77,97,114,107,101,114,10,0,0,0,70,97,105,108,101,100,32,116,111,32,100,101,99,111,100,101,32,106,112,50,32,115,116,114,117,99,116,117,114,101,10,0,70,97,105,108,101,100,32,116,111,32,100,101,99,111,100,101,32,74,50,75,32,105,109,97,103,101,10,0,0,0,0,0,69,120,112,101,99,116,101,100,32,70,84,89,80,32,77,97,114,107,101,114,10,0,0,0,69,114,114,111,114,32,119,105,116,104,32,70,84,89,80,32,66,111,120,10,0,0,0,0,69,120,112,101,99,116,101,100,32,74,80,32,77,97,114,107,101,114,10,0,0,0,0,0,69,114,114,111,114,32,119,105,116,104,32,74,80,32,77,97,114,107,101,114,10,0,0,0,69,114,114,111,114,32,119,105,116,104,32,74,80,32,66,111,120,32,115,105,122,101,10,0,69,114,114,111,114,32,119,105,116,104,32,67,79,76,82,32,98,111,120,32,115,105,122,101,10,0,0,0,0,0,0,0,69,114,114,111,114,32,119,105,116,104,32,67,79,76,82,32,66,111,120,10,0,0,0,0,69,120,112,101,99,116,101,100,32,66,80,67,67,32,77,97,114,107,101,114,10,0,0,0,69,114,114,111,114,32,119,105,116,104,32,66,80,67,67,32,66,111,120,10,0,0,0,0,69,120,112,101,99,116,101,100,32,73,72,68,82,32,77,97,114,107,101,114,10,0,0,0,69,114,114,111,114,32,119,105,116,104,32,73,72,68,82,32,66,111,120,10,0,0,0,0,67,97,110,110,111,116,32,104,97,110,100,108,101,32,98,111,120,32,115,105,122,101,115,32,104,105,103,104,101,114,32,116,104,97,110,32,50,94,51,50,10,0,0,0,0,0,0,0,70,111,114,98,105,100,100,101,110,32,118,97,108,117,101,32,101,110,99,111,117,110,116,101,114,32,105,110,32,109,101,115,115,97,103,101,32,104,101,97,100,101,114,32,33,33,10,0,1,86,0,0,0,0,0,0,192,209,0,0,208,209,0,0,1,86,0,0,1,0,0,0,208,209,0,0,192,209,0,0,1,52,0,0,0,0,0,0,224,209,0,0,96,210,0,0,1,52,0,0,1,0,0,0,240,209,0,0,112,210,0,0,1,24,0,0,0,0,0,0,0,210,0,0,192,210,0,0,1,24,0,0,1,0,0,0,16,210,0,0,208,210,0,0,193,10,0,0,0,0,0,0,32,210,0,0,32,211,0,0,193,10,0,0,1,0,0,0,48,210,0,0,48,211,0,0,33,5,0,0,0,0,0,0,64,210,0,0,64,213,0,0,33,5,0,0,1,0,0,0,80,210,0,0,80,213,0,0,33,2,0,0,0,0,0,0,96,214,0,0,192,213,0,0,33,2,0,0,1,0,0,0,112,214,0,0,208,213,0,0,1,86,0,0,0,0,0,0,128,210,0,0,112,210,0,0,1,86,0,0,1,0,0,0,144,210,0,0,96,210,0,0,1,84,0,0,0,0,0,0,160,210,0,0,96,211,0,0,1,84,0,0,1,0,0,0,176,210,0,0,112,211,0,0,1,72,0,0,0,0,0,0,192,210,0,0,96,211,0,0,1,72,0,0,1,0,0,0,208,210,0,0,112,211,0,0,1,56,0,0,0,0,0,0,224,210,0,0,96,211,0,0,1,56,0,0,1,0,0,0,240,210,0,0,112,211,0,0,1,48,0,0,0,0,0,0,0,211,0,0,192,211,0,0,1,48,0,0,1,0,0,0,16,211,0,0,208,211,0,0,1,36,0,0,0,0,0,0,32,211,0,0,224,211,0,0,1,36,0,0,1,0,0,0,48,211,0,0,240,211,0,0,1,28,0,0,0,0,0,0,64,211,0,0,32,212,0,0,1,28,0,0,1,0,0,0,80,211,0,0,48,212,0,0,1,22,0,0,0,0,0,0,64,213,0,0,64,212,0,0,1,22,0,0,1,0,0,0,80,213,0,0,80,212,0,0,1,86,0,0,0,0,0,0,128,211,0,0,112,211,0,0,1,86,0,0,1,0,0,0,144,211,0,0,96,211,0,0,1,84,0,0,0,0,0,0,160,211,0,0,96,211,0,0,1,84,0,0,1,0,0,0,176,211,0,0,112,211,0,0,1,81,0,0,0,0,0,0,192,211,0,0,128,211,0,0,1,81,0,0,1,0,0,0,208,211,0,0,144,211,0,0,1,72,0,0,0,0,0,0,224,211,0,0,160,211,0,0,1,72,0,0,1,0,0,0,240,211,0,0,176,211,0,0,1,56,0,0,0,0,0,0,0,212,0,0,192,211,0,0,1,56,0,0,1,0,0,0,16,212,0,0,208,211,0,0,1,52,0,0,0,0,0,0,32,212,0,0,224,211,0,0,1,52,0,0,1,0,0,0,48,212,0,0,240,211,0,0,1,48,0,0,0,0,0,0,64,212,0,0,0,212,0,0,1,48,0,0,1,0,0,0,80,212,0,0,16,212,0,0,1,40,0,0,0,0,0,0,96,212,0,0,0,212,0,0,1,40,0,0,1,0,0,0,112,212,0,0,16,212,0,0,1,36,0,0,0,0,0,0,128,212,0,0,32,212,0,0,1,36,0,0,1,0,0,0,144,212,0,0,48,212,0,0,1,34,0,0,0,0,0,0,160,212,0,0,64,212,0,0,1,34,0,0,1,0,0,0,176,212,0,0,80,212,0,0,1,28,0,0,0,0,0,0,192,212,0,0,96,212,0,0,1,28,0,0,1,0,0,0,208,212,0,0,112,212,0,0,1,24,0,0,0,0,0,0,224,212,0,0,128,212,0,0,1,24,0,0,1,0,0,0,240,212,0,0,144,212,0,0,1,22,0,0,0,0,0,0,0,213,0,0,160,212,0,0,1,22,0,0,1,0,0,0,16,213,0,0,176,212,0,0,1,20,0,0,0,0,0,0,32,213,0,0,192,212,0,0,1,20,0,0,1,0,0,0,48,213,0,0,208,212,0,0,1,18,0,0,0,0,0,0,64,213,0,0,224,212,0,0,1,18,0,0,1,0,0,0,80,213,0,0,240,212,0,0,1,17,0,0,0,0,0,0,96,213,0,0,0,213,0,0,1,17,0,0,1,0,0,0,112,213,0,0,16,213,0,0,193,10,0,0,0,0,0,0,128,213,0,0,32,213,0,0,193,10,0,0,1,0,0,0,144,213,0,0,48,213,0,0,193,9,0,0,0,0,0,0,160,213,0,0,64,213,0,0,193,9,0,0,1,0,0,0,176,213,0,0,80,213,0,0,161,8,0,0,0,0,0,0,192,213,0,0,96,213,0,0,161,8,0,0,1,0,0,0,208,213,0,0,112,213,0,0,33,5,0,0,0,0,0,0,224,213,0,0,128,213,0,0,33,5,0,0,1,0,0,0,240,213,0,0,144,213,0,0,65,4,0,0,0,0,0,0,0,214,0,0,160,213,0,0,65,4,0,0,1,0,0,0,16,214,0,0,176,213,0,0,161,2,0,0,0,0,0,0,32,214,0,0,192,213,0,0,161,2,0,0,1,0,0,0,48,214,0,0,208,213,0,0,33,2,0,0,0,0,0,0,64,214,0,0,224,213,0,0,33,2,0,0,1,0,0,0,80,214,0,0,240,213,0,0,65,1,0,0,0,0,0,0,96,214,0,0,0,214,0,0,65,1,0,0,1,0,0,0,112,214,0,0,16,214,0,0,17,1,0,0,0,0,0,0,128,214,0,0,32,214,0,0,17,1,0,0,1,0,0,0,144,214,0,0,48,214,0,0,133,0,0,0,0,0,0,0,160,214,0,0,64,214,0,0,133,0,0,0,1,0,0,0,176,214,0,0,80,214,0,0,73,0,0,0,0,0,0,0,192,214,0,0,96,214,0,0,73,0,0,0,1,0,0,0,208,214,0,0,112,214,0,0,37,0,0,0,0,0,0,0,224,214,0,0,128,214,0,0,37,0,0,0,1,0,0,0,240,214,0,0,144,214,0,0,21,0,0,0,0,0,0,0,0,215,0,0,160,214,0,0,21,0,0,0,1,0,0,0,16,215,0,0,176,214,0,0,9,0,0,0,0,0,0,0,32,215,0,0,192,214,0,0,9,0,0,0,1,0,0,0,48,215,0,0,208,214,0,0,5,0,0,0,0,0,0,0,64,215,0,0,224,214,0,0,5,0,0,0,1,0,0,0,80,215,0,0,240,214,0,0,1,0,0,0,0,0,0,0,64,215,0,0,0,215,0,0,1,0,0,0,1,0,0,0,80,215,0,0,16,215,0,0,1,86,0,0,0,0,0,0,96,215,0,0,96,215,0,0,1,86,0,0,1,0,0,0,112,215,0,0,112,215,0,0,64,0,64,4,32,0,32,2,128,0,128,8,16,0,16,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,1,1,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,1,0,0,0,0,0,0,1,0,0,1,1,1,1,1,1,1,0,0,0,1,0,0,1,1,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,0,1,0,0,0,0,0,0,1,1,0,1,1,1,0,0,0,0,1,1,0,0,1,1,1,1,1,1,1,1,1,1,0,1,1,1,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,1,0,0,0,1,1,0,0,1,1,1,0,0,1,0,0,1,1,0,0,1,1,0,1,1,1,1,1,0,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,9,10,12,13,10,10,13,13,12,13,12,13,13,13,13,13,9,10,12,11,10,9,13,12,12,11,12,11,13,12,13,12,9,10,12,11,10,10,11,11,12,13,9,10,13,13,10,10,9,10,12,13,10,9,11,12,12,11,9,10,13,12,10,9,9,10,12,13,10,9,11,12,12,13,12,13,11,12,11,12,9,10,12,11,10,10,11,11,12,11,12,11,11,11,11,11,9,10,12,11,10,9,13,12,12,13,9,10,11,12,10,9,9,10,12,13,10,10,13,13,12,11,9,10,11,11,10,10,9,10,12,13,10,10,13,13,12,11,9,10,11,11,10,10,9,10,12,11,10,9,13,12,12,13,9,10,11,12,10,9,9,10,12,11,10,10,11,11,12,11,12,11,11,11,11,11,9,10,12,13,10,9,11,12,12,13,12,13,11,12,11,12,9,10,12,13,10,9,11,12,12,11,9,10,13,12,10,9,9,10,12,11,10,10,11,11,12,13,9,10,13,13,10,10,9,10,12,11,10,9,13,12,12,11,12,11,13,12,13,12,9,10,12,13,10,10,13,13,12,13,12,13,13,13,13,13,0,1,1,2,1,2,2,2,1,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,0,1,1,2,1,2,2,2,1,2,2,2,2,2,2,2,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,0,1,1,2,1,2,2,2,1,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,0,3,3,6,3,6,6,8,3,6,6,8,6,8,8,8,1,4,4,7,4,7,7,8,4,7,7,8,7,8,8,8,1,4,4,7,4,7,7,8,4,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,1,4,4,7,4,7,7,8,4,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,1,4,4,7,4,7,7,8,4,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,2,5,5,7,5,7,7,8,5,7,7,8,7,8,8,8,69,120,112,101,99,116,101,100,32,83,79,80,32,109,97,114,107,101,114,10,0,0,0,0,69,114,114,111,114,32,58,32,101,120,112,101,99,116,101,100,32,69,80,72,32,109,97,114,107,101,114,10,0,0,0,0,69,120,112,101,99,116,101,100,32,69,80,72,32,109,97,114,107,101,114,10,0,0,0,0,116,105,108,101,32,37,100,32,111,102,32,37,100,10,0,0,116,99,100,95,100,101,99,111,100,101,58,32,105,110,99,111,109,112,108,101,116,101,32,98,105,115,116,114,101,97,109,10,0,0,0,0,0,0,0,0,45,32,116,105,101,114,115,45,49,32,116,111,111,107,32,37,102,32,115,10,0,0,0,0,69,114,114,111,114,32,100,101,99,111,100,105,110,103,32,116,105,108,101,46,32,84,104,101,32,110,117,109,98,101,114,32,111,102,32,114,101,115,111,108,117,116,105,111,110,115,32,116,111,32,114,101,109,111,118,101,32,91,37,100,43,49,93,32,105,115,32,104,105,103,104,101,114,32,116,104,97,110,32,116,104,101,32,110,117,109,98,101,114,32,32,111,102,32,114,101,115,111,108,117,116,105,111,110,115,32,105,110,32,116,104,101,32,111,114,105,103,105,110,97,108,32,99,111,100,101,115,116,114,101,97,109,32,91,37,100,93,10,77,111,100,105,102,121,32,116,104,101,32,99,112,95,114,101,100,117,99,101,32,112,97,114,97,109,101,116,101,114,46,10,0,0,0,0,0,0,45,32,100,119,116,32,116,111,111,107,32,37,102,32,115,10,0,0,0,0,0,0,0,0,45,32,116,105,108,101,32,100,101,99,111,100,101,100,32,105,110,32,37,102,32,115,10,0,49,46,50,46,50,51,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,73,72,68,82,0,0,0,0,73,68,65,84,0,0,0,0,73,69,78,68,0,0,0,0,80,76,84,69,0,0,0,0,98,75,71,68,0,0,0,0,99,72,82,77,0,0,0,0,103,65,77,65,0,0,0,0,104,73,83,84,0,0,0,0,105,67,67,80,0,0,0,0,111,70,70,115,0,0,0,0,112,67,65,76,0,0,0,0,115,67,65,76,0,0,0,0,112,72,89,115,0,0,0,0,115,66,73,84,0,0,0,0,115,80,76,84,0,0,0,0,115,82,71,66,0,0,0,0,116,69,88,116,0,0,0,0,116,73,77,69,0,0,0,0,116,82,78,83,0,0,0,0,122,84,88,116,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,2,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,8,0,0,0,8,0,0,0,8,0,0,0,4,0,0,0,4,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,128,0,0,0,8,0,0,0,136,0,0,0,34,0,0,0,170,0,0,0,85,0,0,0,255,0,0,0,0,0,0,0,255,0,0,0,15,0,0,0,255,0,0,0,51,0,0,0,255,0,0,0,85,0,0,0,255,0,0,0,0,0,0,0,137,80,78,71,13,10,26,10,80,111,116,101,110,116,105,97,108,32,111,118,101,114,102,108,111,119,32,105,110,32,112,110,103,95,122,97,108,108,111,99,40,41,0,0,0,0,0,0,108,105,98,112,110,103,32,119,97,114,110,105,110,103,32,110,111,46,32,37,115,58,32,37,115,10,0,0,0,0,0,0,108,105,98,112,110,103,32,119,97,114,110,105,110,103,58,32,37,115,10,0,0,0,0,0,108,105,98,112,110,103,32,101,114,114,111,114,32,110,111,46,32,37,115,58,32,37,115,10,0,0,0,0,0,0,0,0,108,105,98,112,110,103,32,101,114,114,111,114,58,32,37,115,44,32,111,102,102,115,101,116,61,37,100,10,0,0,0,0,108,105,98,112,110,103,32,101,114,114,111,114,58,32,37,115,10,0,0,0,0,0,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,79,117,116,32,111,102,32,77,101,109,111,114,121,33,0,0,79,118,101,114,102,108,111,119,32,105,110,32,112,110,103,95,109,101,109,99,112,121,95,99,104,101,99,107,46,0,0,0,79,118,101,114,102,108,111,119,32,105,110,32,112,110,103,95,109,101,109,115,101,116,95,99,104,101,99,107,46,0,0,0,65,112,112,108,105,99,97,116,105,111,110,32,119,97,115,32,99,111,109,112,105,108,101,100,32,119,105,116,104,32,112,110,103,46,104,32,102,114,111,109,32,108,105,98,112,110,103,45,37,46,50,48,115,0,0,0,65,112,112,108,105,99,97,116,105,111,110,32,32,105,115,32,32,114,117,110,110,105,110,103,32,119,105,116,104,32,112,110,103,46,99,32,102,114,111,109,32,108,105,98,112,110,103,45,37,46,50,48,115,0,0,0,73,110,99,111,109,112,97,116,105,98,108,101,32,108,105,98,112,110,103,32,118,101,114,115,105,111,110,32,105,110,32,97,112,112,108,105,99,97,116,105,111,110,32,97,110,100,32,108,105,98,114,97,114,121,0,0,49,46,50,46,53,0,0,0,122,108,105,98,32,109,101,109,111,114,121,32,101,114,114,111,114,0,0,0,0,0,0,0,122,108,105,98,32,118,101,114,115,105,111,110,32,101,114,114,111,114,0,0,0,0,0,0,85,110,107,110,111,119,110,32,122,108,105,98,32,101,114,114,111,114,0,0,0,0,0,0,78,111,116,32,97,32,80,78,71,32,102,105,108,101,0,0,80,78,71,32,102,105,108,101,32,99,111,114,114,117,112,116,101,100,32,98,121,32,65,83,67,73,73,32,99,111,110,118,101,114,115,105,111,110,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,73,68,65,84,0,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,80,76,84,69,32,98,101,102,111,114,101,32,73,68,65,84,0,0,0,0,0,0,0,0,73,103,110,111,114,105,110,103,32,101,120,116,114,97,32,112,110,103,95,114,101,97,100,95,117,112,100,97,116,101,95,105,110,102,111,40,41,32,99,97,108,108,59,32,114,111,119,32,98,117,102,102,101,114,32,110,111,116,32,114,101,97,108,108,111,99,97,116,101,100,0,0,73,110,118,97,108,105,100,32,97,116,116,101,109,112,116,32,116,111,32,114,101,97,100,32,114,111,119,32,100,97,116,97,0,0,0,0,0,0,0,0,78,111,116,32,101,110,111,117,103,104,32,105,109,97,103,101,32,100,97,116,97,0,0,0,69,120,116,114,97,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,0,0,0,68,101,99,111,109,112,114,101,115,115,105,111,110,32,101,114,114,111,114,0,0,0,0,0,67,97,108,108,32,116,111,32,78,85,76,76,32,114,101,97,100,32,102,117,110,99,116,105,111,110,0,0,0,0,0,0,82,101,97,100,32,69,114,114,111,114,0,0,0,0,0,0,73,116,39,115,32,97,110,32,101,114,114,111,114,32,116,111,32,115,101,116,32,98,111,116,104,32,114,101,97,100,95,100,97,116,97,95,102,110,32,97,110,100,32,119,114,105,116,101,95,100,97,116,97,95,102,110,32,105,110,32,116,104,101,32,0,0,0,0,0,0,0,0,115,97,109,101,32,115,116,114,117,99,116,117,114,101,46,32,32,82,101,115,101,116,116,105,110,103,32,119,114,105,116,101,95,100,97,116,97,95,102,110,32,116,111,32,78,85,76,76,46,0,0,0,0,0,0,0,78,85,76,76,32,114,111,119,32,98,117,102,102,101,114,32,102,111,114,32,114,111,119,32,37,108,100,44,32,112,97,115,115,32,37,100,0,0,0,0,85,110,105,110,105,116,105,97,108,105,122,101,100,32,114,111,119,0,0,0,0,0,0,0,112,110,103,95,100,111,95,114,103,98,95,116,111,95,103,114,97,121,32,102,111,117,110,100,32,110,111,110,103,114,97,121,32,112,105,120,101,108,0,0,112,110,103,95,100,111,95,100,105,116,104,101,114,32,114,101,116,117,114,110,101,100,32,114,111,119,98,121,116,101,115,61,48,0,0,0,0,0,0,0,16,0,0,0,33,0,0,0,66,0,0,0,132,0,0,0,16,1,0,0,72,2,0,0,80,5,0,0,240,15,0,0,0,0,0,0,0,0,0,0,80,78,71,32,117,110,115,105,103,110,101,100,32,105,110,116,101,103,101,114,32,111,117,116,32,111,102,32,114,97,110,103,101,46,0,0,0,0,0,0,67,82,67,32,101,114,114,111,114,0,0,0,0,0,0,0,69,114,114,111,114,32,100,101,99,111,100,105,110,103,32,99,111,109,112,114,101,115,115,101,100,32,116,101,120,116,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,116,111,32,100,101,99,111,109,112,114,101,115,115,32,99,104,117,110,107,0,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,116,111,32,100,101,99,111,109,112,114,101,115,115,32,99,104,117,110,107,46,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,116,111,32,100,101,99,111,109,112,114,101,115,115,32,99,104,117,110,107,46,46,0,66,117,102,102,101,114,32,101,114,114,111,114,32,105,110,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,115,116,114,101,97,109,32,105,110,32,37,115,32,99,104,117,110,107,0,0,0,0,0,0,0,68,97,116,97,32,101,114,114,111,114,32,105,110,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,115,116,114,101,97,109,32,105,110,32,37,115,32,99,104,117,110,107,0,73,110,99,111,109,112,108,101,116,101,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,115,116,114,101,97,109,32,105,110,32,37,115,32,99,104,117,110,107,0,0,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,102,111,114,32,116,101,120,116,46,0,0,0,0,0,85,110,107,110,111,119,110,32,122,84,88,116,32,99,111,109,112,114,101,115,115,105,111,110,32,116,121,112,101,32,37,100,0,0,0,0,0,0,0,0,79,117,116,32,111,102,32,112,108,97,99,101,32,73,72,68,82,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,73,72,68,82,32,99,104,117,110,107,0,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,80,76,84,69,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,80,76,84,69,32,97,102,116,101,114,32,73,68,65,84,0,68,117,112,108,105,99,97,116,101,32,80,76,84,69,32,99,104,117,110,107,0,0,0,0,73,103,110,111,114,105,110,103,32,80,76,84,69,32,99,104,117,110,107,32,105,110,32,103,114,97,121,115,99,97,108,101,32,80,78,71,0,0,0,0,73,110,118,97,108,105,100,32,112,97,108,101,116,116,101,32,99,104,117,110,107,0,0,0,84,114,117,110,99,97,116,105,110,103,32,105,110,99,111,114,114,101,99,116,32,116,82,78,83,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,84,114,117,110,99,97,116,105,110,103,32,105,110,99,111,114,114,101,99,116,32,105,110,102,111,32,116,82,78,83,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,78,111,32,105,109,97,103,101,32,105,110,32,102,105,108,101,0,0,0,0,0,0,0,0,73,110,99,111,114,114,101,99,116,32,73,69,78,68,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,103,65,77,65,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,103,65,77,65,32,97,102,116,101,114,32,73,68,65,84,0,79,117,116,32,111,102,32,112,108,97,99,101,32,103,65,77,65,32,99,104,117,110,107,0,68,117,112,108,105,99,97,116,101,32,103,65,77,65,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,103,65,77,65,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,73,103,110,111,114,105,110,103,32,103,65,77,65,32,99,104,117,110,107,32,119,105,116,104,32,103,97,109,109,97,61,48,0,0,0,0,0,0,0,0,73,103,110,111,114,105,110,103,32,105,110,99,111,114,114,101,99,116,32,103,65,77,65,32,118,97,108,117,101,32,119,104,101,110,32,115,82,71,66,32,105,115,32,97,108,115,111,32,112,114,101,115,101,110,116,0,103,97,109,109,97,32,61,32,40,37,100,47,49,48,48,48,48,48,41,10,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,115,66,73,84,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,66,73,84,32,97,102,116,101,114,32,73,68,65,84,0,79,117,116,32,111,102,32,112,108,97,99,101,32,115,66,73,84,32,99,104,117,110,107,0,68,117,112,108,105,99,97,116,101,32,115,66,73,84,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,115,66,73,84,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,99,72,82,77,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,72,82,77,32,97,102,116,101,114,32,73,68,65,84,0,77,105,115,115,105,110,103,32,80,76,84,69,32,98,101,102,111,114,101,32,99,72,82,77,0,0,0,0,0,0,0,0,68,117,112,108,105,99,97,116,101,32,99,72,82,77,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,99,72,82,77,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,73,110,118,97,108,105,100,32,99,72,82,77,32,119,104,105,116,101,32,112,111,105,110,116,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,72,82,77,32,114,101,100,32,112,111,105,110,116,0,0,73,110,118,97,108,105,100,32,99,72,82,77,32,103,114,101,101,110,32,112,111,105,110,116,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,72,82,77,32,98,108,117,101,32,112,111,105,110,116,0,73,103,110,111,114,105,110,103,32,105,110,99,111,114,114,101,99,116,32,99,72,82,77,32,118,97,108,117,101,32,119,104,101,110,32,115,82,71,66,32,105,115,32,97,108,115,111,32,112,114,101,115,101,110,116,0,119,120,61,37,102,44,32,119,121,61,37,102,44,32,114,120,61,37,102,44,32,114,121,61,37,102,10,0,0,0,0,0,103,120,61,37,102,44,32,103,121,61,37,102,44,32,98,120,61,37,102,44,32,98,121,61,37,102,10,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,115,82,71,66,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,82,71,66,32,97,102,116,101,114,32,73,68,65,84,0,79,117,116,32,111,102,32,112,108,97,99,101,32,115,82,71,66,32,99,104,117,110,107,0,68,117,112,108,105,99,97,116,101,32,115,82,71,66,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,115,82,71,66,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,85,110,107,110,111,119,110,32,115,82,71,66,32,105,110,116,101,110,116,0,0,0,0,0,105,110,99,111,114,114,101,99,116,32,103,97,109,109,97,61,40,37,100,47,49,48,48,48,48,48,41,10,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,105,67,67,80,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,105,67,67,80,32,97,102,116,101,114,32,73,68,65,84,0,79,117,116,32,111,102,32,112,108,97,99,101,32,105,67,67,80,32,99,104,117,110,107,0,68,117,112,108,105,99,97,116,101,32,105,67,67,80,32,99,104,117,110,107,0,0,0,0,77,97,108,102,111,114,109,101,100,32,105,67,67,80,32,99,104,117,110,107,0,0,0,0,73,103,110,111,114,105,110,103,32,110,111,110,122,101,114,111,32,99,111,109,112,114,101,115,115,105,111,110,32,116,121,112,101,32,105,110,32,105,67,67,80,32,99,104,117,110,107,0,80,114,111,102,105,108,101,32,115,105,122,101,32,102,105,101,108,100,32,109,105,115,115,105,110,103,32,102,114,111,109,32,105,67,67,80,32,99,104,117,110,107,0,0,0,0,0,0,73,103,110,111,114,105,110,103,32,116,114,117,110,99,97,116,101,100,32,105,67,67,80,32,112,114,111,102,105,108,101,46,0,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,115,80,76,84,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,80,76,84,32,97,102,116,101,114,32,73,68,65,84,0,109,97,108,102,111,114,109,101,100,32,115,80,76,84,32,99,104,117,110,107,0,0,0,0,115,80,76,84,32,99,104,117,110,107,32,104,97,115,32,98,97,100,32,108,101,110,103,116,104,0,0,0,0,0,0,0,115,80,76,84,32,99,104,117,110,107,32,116,111,111,32,108,111,110,103,0,0,0,0,0,115,80,76,84,32,99,104,117,110,107,32,114,101,113,117,105,114,101,115,32,116,111,111,32,109,117,99,104,32,109,101,109,111,114,121,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,116,82,78,83,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,116,82,78,83,32,97,102,116,101,114,32,73,68,65,84,0,68,117,112,108,105,99,97,116,101,32,116,82,78,83,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,116,82,78,83,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,80,76,84,69,32,98,101,102,111,114,101,32,116,82,78,83,0,0,0,0,0,0,0,0,90,101,114,111,32,108,101,110,103,116,104,32,116,82,78,83,32,99,104,117,110,107,0,0,116,82,78,83,32,99,104,117,110,107,32,110,111,116,32,97,108,108,111,119,101,100,32,119,105,116,104,32,97,108,112,104,97,32,99,104,97,110,110,101,108,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,98,75,71,68,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,98,75,71,68,32,97,102,116,101,114,32,73,68,65,84,0,77,105,115,115,105,110,103,32,80,76,84,69,32,98,101,102,111,114,101,32,98,75,71,68,0,0,0,0,0,0,0,0,68,117,112,108,105,99,97,116,101,32,98,75,71,68,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,98,75,71,68,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,73,110,99,111,114,114,101,99,116,32,98,75,71,68,32,99,104,117,110,107,32,105,110,100,101,120,32,118,97,108,117,101,0,0,0,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,104,73,83,84,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,104,73,83,84,32,97,102,116,101,114,32,73,68,65,84,0,77,105,115,115,105,110,103,32,80,76,84,69,32,98,101,102,111,114,101,32,104,73,83,84,0,0,0,0,0,0,0,0,68,117,112,108,105,99,97,116,101,32,104,73,83,84,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,104,73,83,84,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,112,72,89,115,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,72,89,115,32,97,102,116,101,114,32,73,68,65,84,0,68,117,112,108,105,99,97,116,101,32,112,72,89,115,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,112,72,89,115,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,111,70,70,115,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,111,70,70,115,32,97,102,116,101,114,32,73,68,65,84,0,68,117,112,108,105,99,97,116,101,32,111,70,70,115,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,111,70,70,115,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,112,67,65,76,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,67,65,76,32,97,102,116,101,114,32,73,68,65,84,0,68,117,112,108,105,99,97,116,101,32,112,67,65,76,32,99,104,117,110,107,0,0,0,0,78,111,32,109,101,109,111,114,121,32,102,111,114,32,112,67,65,76,32,112,117,114,112,111,115,101,46,0,0,0,0,0,73,110,118,97,108,105,100,32,112,67,65,76,32,100,97,116,97,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,112,67,65,76,32,112,97,114,97,109,101,116,101,114,115,32,102,111,114,32,101,113,117,97,116,105,111,110,32,116,121,112,101,0,0,0,0,0,0,0,85,110,114,101,99,111,103,110,105,122,101,100,32,101,113,117,97,116,105,111,110,32,116,121,112,101,32,102,111,114,32,112,67,65,76,32,99,104,117,110,107,0,0,0,0,0,0,0,78,111,32,109,101,109,111,114,121,32,102,111,114,32,112,67,65,76,32,112,97,114,97,109,115,46,0,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,115,67,65,76,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,115,67,65,76,32,97,102,116,101,114,32,73,68,65,84,0,68,117,112,108,105,99,97,116,101,32,115,67,65,76,32,99,104,117,110,107,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,32,119,104,105,108,101,32,112,114,111,99,101,115,115,105,110,103,32,115,67,65,76,32,99,104,117,110,107,0,0,0,0,0,0,0,109,97,108,102,111,114,109,101,100,32,119,105,100,116,104,32,115,116,114,105,110,103,32,105,110,32,115,67,65,76,32,99,104,117,110,107,0,0,0,0,84,114,117,110,99,97,116,101,100,32,115,67,65,76,32,99,104,117,110,107,0,0,0,0,109,97,108,102,111,114,109,101,100,32,104,101,105,103,104,116,32,115,116,114,105,110,103,32,105,110,32,115,67,65,76,32,99,104,117,110,107,0,0,0,73,110,118,97,108,105,100,32,115,67,65,76,32,100,97,116,97,0,0,0,0,0,0,0,79,117,116,32,111,102,32,112,108,97,99,101,32,116,73,77,69,32,99,104,117,110,107,0,68,117,112,108,105,99,97,116,101,32,116,73,77,69,32,99,104,117,110,107,0,0,0,0,73,110,99,111,114,114,101,99,116,32,116,73,77,69,32,99,104,117,110,107,32,108,101,110,103,116,104,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,116,69,88,116,0,0,0,0,0,0,0,0,78,111,32,109,101,109,111,114,121,32,116,111,32,112,114,111,99,101,115,115,32,116,101,120,116,32,99,104,117,110,107,46,0,0,0,0,0,0,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,116,111,32,112,114,111,99,101,115,115,32,116,101,120,116,32,99,104,117,110,107,46,0,0,0,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,116,111,32,112,114,111,99,101,115,115,32,116,101,120,116,32,99,104,117,110,107,46,0,0,0,0,0,0,77,105,115,115,105,110,103,32,73,72,68,82,32,98,101,102,111,114,101,32,122,84,88,116,0,0,0,0,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,32,112,114,111,99,101,115,115,105,110,103,32,122,84,88,116,32,99,104,117,110,107,46,0,0,0,0,84,114,117,110,99,97,116,101,100,32,122,84,88,116,32,99,104,117,110,107,0,0,0,0,85,110,107,110,111,119,110,32,99,111,109,112,114,101,115,115,105,111,110,32,116,121,112,101,32,105,110,32,122,84,88,116,32,99,104,117,110,107,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,116,111,32,112,114,111,99,101,115,115,32,122,84,88,116,32,99,104,117,110,107,46,0,0,0,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,116,111,32,115,116,111,114,101,32,122,84,88,116,32,99,104,117,110,107,46,0,0,0,0,0,0,0,0,117,110,107,110,111,119,110,32,99,114,105,116,105,99,97,108,32,99,104,117,110,107,0,0,101,114,114,111,114,32,105,110,32,117,115,101,114,32,99,104,117,110,107,0,0,0,0,0,105,110,118,97,108,105,100,32,99,104,117,110,107,32,116,121,112,101,0,0,0,0,0,0,73,103,110,111,114,105,110,103,32,98,97,100,32,97,100,97,112,116,105,118,101,32,102,105,108,116,101,114,32,116,121,112,101,0,0,0,0,0,0,0,78,111,116,32,101,110,111,117,103,104,32,105,109,97,103,101,32,100,97,116,97,0,0,0,69,120,116,114,97,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,0,0,0,68,101,99,111,109,112,114,101,115,115,105,111,110,32,69,114,114,111,114,0,0,0,0,0,69,120,116,114,97,32,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,46,0,0,69,120,116,114,97,32,99,111,109,112,114,101,115,115,105,111,110,32,100,97,116,97,0,0,82,111,119,98,121,116,101,115,32,111,118,101,114,102,108,111,119,32,105,110,32,112,110,103,95,114,101,97,100,95,115,116,97,114,116,95,114,111,119,0,82,111,119,32,104,97,115,32,116,111,111,32,109,97,110,121,32,98,121,116,101,115,32,116,111,32,97,108,108,111,99,97,116,101,32,105,110,32,109,101,109,111,114,121,46,0,0,0,73,103,110,111,114,105,110,103,32,97,116,116,101,109,112,116,32,116,111,32,115,101,116,32,97,108,108,45,122,101,114,111,32,99,104,114,111,109,97,116,105,99,105,116,121,32,118,97,108,117,101,115,0,0,0,0,73,103,110,111,114,105,110,103,32,97,116,116,101,109,112,116,32,116,111,32,115,101,116,32,110,101,103,97,116,105,118,101,32,99,104,114,111,109,97,116,105,99,105,116,121,32,118,97,108,117,101,0,0,0,0,0,73,103,110,111,114,105,110,103,32,97,116,116,101,109,112,116,32,116,111,32,115,101,116,32,99,104,114,111,109,97,116,105,99,105,116,121,32,118,97,108,117,101,32,101,120,99,101,101,100,105,110,103,32,50,49,52,55,52,46,56,51,0,0,0,76,105,109,105,116,105,110,103,32,103,97,109,109,97,32,116,111,32,50,49,52,55,52,46,56,51,0,0,0,0,0,0,83,101,116,116,105,110,103,32,103,97,109,109,97,61,48,0,83,101,116,116,105,110,103,32,110,101,103,97,116,105,118,101,32,103,97,109,109,97,32,116,111,32,122,101,114,111,0,0,73,110,118,97,108,105,100,32,112,97,108,101,116,116,101,32,115,105,122,101,44,32,104,73,83,84,32,97,108,108,111,99,97,116,105,111,110,32,115,107,105,112,112,101,100,46,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,102,111,114,32,104,73,83,84,32,99,104,117,110,107,32,100,97,116,97,46,0,0,0,0,0,0,0,0,73,109,97,103,101,32,119,105,100,116,104,32,111,114,32,104,101,105,103,104,116,32,105,115,32,122,101,114,111,32,105,110,32,73,72,68,82,0,0,0,105,109,97,103,101,32,115,105,122,101,32,101,120,99,101,101,100,115,32,117,115,101,114,32,108,105,109,105,116,115,32,105,110,32,73,72,68,82,0,0,73,110,118,97,108,105,100,32,105,109,97,103,101,32,115,105,122,101,32,105,110,32,73,72,68,82,0,0,0,0,0,0,87,105,100,116,104,32,105,115,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,108,105,98,112,110,103,32,116,111,32,112,114,111,99,101,115,115,32,112,105,120,101,108,115,0,73,110,118,97,108,105,100,32,98,105,116,32,100,101,112,116,104,32,105,110,32,73,72,68,82,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,111,108,111,114,32,116,121,112,101,32,105,110,32,73,72,68,82,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,111,108,111,114,32,116,121,112,101,47,98,105,116,32,100,101,112,116,104,32,99,111,109,98,105,110,97,116,105,111,110,32,105,110,32,73,72,68,82,0,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,105,110,116,101,114,108,97,99,101,32,109,101,116,104,111,100,32,105,110,32,73,72,68,82,0,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,99,111,109,112,114,101,115,115,105,111,110,32,109,101,116,104,111,100,32,105,110,32,73,72,68,82,0,0,0,0,0,0,77,78,71,32,102,101,97,116,117,114,101,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,97,32,80,78,71,32,100,97,116,97,115,116,114,101,97,109,0,0,0,0,0,0,0,0,85,110,107,110,111,119,110,32,102,105,108,116,101,114,32,109,101,116,104,111,100,32,105,110,32,73,72,68,82,0,0,0,73,110,118,97,108,105,100,32,102,105,108,116,101,114,32,109,101,116,104,111,100,32,105,110,32,73,72,68,82,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,102,111,114,32,112,67,65,76,32,112,117,114,112,111,115,101,46,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,102,111,114,32,112,67,65,76,32,117,110,105,116,115,46,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,102,111,114,32,112,67,65,76,32,112,97,114,97,109,115,46,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,102,111,114,32,112,67,65,76,32,112,97,114,97,109,101,116,101,114,46,0,73,110,118,97,108,105,100,32,112,97,108,101,116,116,101,32,108,101,110,103,116,104,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,116,111,32,112,114,111,99,101,115,115,32,105,67,67,80,32,99,104,117,110,107,46,0,0,0,0,0,0,73,110,115,117,102,102,105,99,105,101,110,116,32,109,101,109,111,114,121,32,116,111,32,112,114,111,99,101,115,115,32,105,67,67,80,32,112,114,111,102,105,108,101,46,0,0,0,0,105,84,88,116,32,99,104,117,110,107,32,110,111,116,32,115,117,112,112,111,114,116,101,100,46,0,0,0,0,0,0,0,78,111,32,109,101,109,111,114,121,32,102,111,114,32,115,80,76,84,32,112,97,108,101,116,116,101,115,46,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,32,119,104,105,108,101,32,112,114,111,99,101,115,115,105,110,103,32,115,80,76,84,32,99,104,117,110,107,0,0,0,0,0,0,0,79,117,116,32,111,102,32,109,101,109,111,114,121,32,119,104,105,108,101,32,112,114,111,99,101,115,115,105,110,103,32,117,110,107,110,111,119,110,32,99,104,117,110,107,46,0,0,0,0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,1,129,65,193,33,161,97,225,17,145,81,209,49,177,113,241,9,137,73,201,41,169,105,233,25,153,89,217,57,185,121,249,5,133,69,197,37,165,101,229,21,149,85,213,53,181,117,245,13,141,77,205,45,173,109,237,29,157,93,221,61,189,125,253,3,131,67,195,35,163,99,227,19,147,83,211,51,179,115,243,11,139,75,203,43,171,107,235,27,155,91,219,59,187,123,251,7,135,71,199,39,167,103,231,23,151,87,215,55,183,119,247,15,143,79,207,47,175,111,239,31,159,95,223,63,191,127,255,0,64,128,192,16,80,144,208,32,96,160,224,48,112,176,240,4,68,132,196,20,84,148,212,36,100,164,228,52,116,180,244,8,72,136,200,24,88,152,216,40,104,168,232,56,120,184,248,12,76,140,204,28,92,156,220,44,108,172,236,60,124,188,252,1,65,129,193,17,81,145,209,33,97,161,225,49,113,177,241,5,69,133,197,21,85,149,213,37,101,165,229,53,117,181,245,9,73,137,201,25,89,153,217,41,105,169,233,57,121,185,249,13,77,141,205,29,93,157,221,45,109,173,237,61,125,189,253,2,66,130,194,18,82,146,210,34,98,162,226,50,114,178,242,6,70,134,198,22,86,150,214,38,102,166,230,54,118,182,246,10,74,138,202,26,90,154,218,42,106,170,234,58,122,186,250,14,78,142,206,30,94,158,222,46,110,174,238,62,126,190,254,3,67,131,195,19,83,147,211,35,99,163,227,51,115,179,243,7,71,135,199,23,87,151,215,39,103,167,231,55,119,183,247,11,75,139,203,27,91,155,219,43,107,171,235,59,123,187,251,15,79,143,207,31,95,159,223,47,111,175,239,63,127,191,255,0,16,32,48,64,80,96,112,128,144,160,176,192,208,224,240,1,17,33,49,65,81,97,113,129,145,161,177,193,209,225,241,2,18,34,50,66,82,98,114,130,146,162,178,194,210,226,242,3,19,35,51,67,83,99,115,131,147,163,179,195,211,227,243,4,20,36,52,68,84,100,116,132,148,164,180,196,212,228,244,5,21,37,53,69,85,101,117,133,149,165,181,197,213,229,245,6,22,38,54,70,86,102,118,134,150,166,182,198,214,230,246,7,23,39,55,71,87,103,119,135,151,167,183,199,215,231,247,8,24,40,56,72,88,104,120,136,152,168,184,200,216,232,248,9,25,41,57,73,89,105,121,137,153,169,185,201,217,233,249,10,26,42,58,74,90,106,122,138,154,170,186,202,218,234,250,11,27,43,59,75,91,107,123,139,155,171,187,203,219,235,251,12,28,44,60,76,92,108,124,140,156,172,188,204,220,236,252,13,29,45,61,77,93,109,125,141,157,173,189,205,221,237,253,14,30,46,62,78,94,110,126,142,158,174,190,206,222,238,254,15,31,47,63,79,95,111,127,143,159,175,191,207,223,239,255,0,0,0,0,150,48,7,119,44,97,14,238,186,81,9,153,25,196,109,7,143,244,106,112,53,165,99,233,163,149,100,158,50,136,219,14,164,184,220,121,30,233,213,224,136,217,210,151,43,76,182,9,189,124,177,126,7,45,184,231,145,29,191,144,100,16,183,29,242,32,176,106,72,113,185,243,222,65,190,132,125,212,218,26,235,228,221,109,81,181,212,244,199,133,211,131,86,152,108,19,192,168,107,100,122,249,98,253,236,201,101,138,79,92,1,20,217,108,6,99,99,61,15,250,245,13,8,141,200,32,110,59,94,16,105,76,228,65,96,213,114,113,103,162,209,228,3,60,71,212,4,75,253,133,13,210,107,181,10,165,250,168,181,53,108,152,178,66,214,201,187,219,64,249,188,172,227,108,216,50,117,92,223,69,207,13,214,220,89,61,209,171,172,48,217,38,58,0,222,81,128,81,215,200,22,97,208,191,181,244,180,33,35,196,179,86,153,149,186,207,15,165,189,184,158,184,2,40,8,136,5,95,178,217,12,198,36,233,11,177,135,124,111,47,17,76,104,88,171,29,97,193,61,45,102,182,144,65,220,118,6,113,219,1,188,32,210,152,42,16,213,239,137,133,177,113,31,181,182,6,165,228,191,159,51,212,184,232,162,201,7,120,52,249,0,15,142,168,9,150,24,152,14,225,187,13,106,127,45,61,109,8,151,108,100,145,1,92,99,230,244,81,107,107,98,97,108,28,216,48,101,133,78,0,98,242,237,149,6,108,123,165,1,27,193,244,8,130,87,196,15,245,198,217,176,101,80,233,183,18,234,184,190,139,124,136,185,252,223,29,221,98,73,45,218,21,243,124,211,140,101,76,212,251,88,97,178,77,206,81,181,58,116,0,188,163,226,48,187,212,65,165,223,74,215,149,216,61,109,196,209,164,251,244,214,211,106,233,105,67,252,217,110,52,70,136,103,173,208,184,96,218,115,45,4,68,229,29,3,51,95,76,10,170,201,124,13,221,60,113,5,80,170,65,2,39,16,16,11,190,134,32,12,201,37,181,104,87,179,133,111,32,9,212,102,185,159,228,97,206,14,249,222,94,152,201,217,41,34,152,208,176,180,168,215,199,23,61,179,89,129,13,180,46,59,92,189,183,173,108,186,192,32,131,184,237,182,179,191,154,12,226,182,3,154,210,177,116,57,71,213,234,175,119,210,157,21,38,219,4,131,22,220,115,18,11,99,227,132,59,100,148,62,106,109,13,168,90,106,122,11,207,14,228,157,255,9,147,39,174,0,10,177,158,7,125,68,147,15,240,210,163,8,135,104,242,1,30,254,194,6,105,93,87,98,247,203,103,101,128,113,54,108,25,231,6,107,110,118,27,212,254,224,43,211,137,90,122,218,16,204,74,221,103,111,223,185,249,249,239,190,142,67,190,183,23,213,142,176,96,232,163,214,214,126,147,209,161,196,194,216,56,82,242,223,79,241,103,187,209,103,87,188,166,221,6,181,63,75,54,178,72,218,43,13,216,76,27,10,175,246,74,3,54,96,122,4,65,195,239,96,223,85,223,103,168,239,142,110,49,121,190,105,70,140,179,97,203,26,131,102,188,160,210,111,37,54,226,104,82,149,119,12,204,3,71,11,187,185,22,2,34,47,38,5,85,190,59,186,197,40,11,189,178,146,90,180,43,4,106,179,92,167,255,215,194,49,207,208,181,139,158,217,44,29,174,222,91,176,194,100,155,38,242,99,236,156,163,106,117,10,147,109,2,169,6,9,156,63,54,14,235,133,103,7,114,19,87,0,5,130,74,191,149,20,122,184,226,174,43,177,123,56,27,182,12,155,142,210,146,13,190,213,229,183,239,220,124,33,223,219,11,212,210,211,134,66,226,212,241,248,179,221,104,110,131,218,31,205,22,190,129,91,38,185,246,225,119,176,111,119,71,183,24,230,90,8,136,112,106,15,255,202,59,6,102,92,11,1,17,255,158,101,143,105,174,98,248,211,255,107,97,69,207,108,22,120,226,10,160,238,210,13,215,84,131,4,78,194,179,3,57,97,38,103,167,247,22,96,208,77,71,105,73,219,119,110,62,74,106,209,174,220,90,214,217,102,11,223,64,240,59,216,55,83,174,188,169,197,158,187,222,127,207,178,71,233,255,181,48,28,242,189,189,138,194,186,202,48,147,179,83,166,163,180,36,5,54,208,186,147,6,215,205,41,87,222,84,191,103,217,35,46,122,102,179,184,74,97,196,2,27,104,93,148,43,111,42,55,190,11,180,161,142,12,195,27,223,5,90,141,239,2,45,0,0,0,0,65,49,27,25,130,98,54,50,195,83,45,43,4,197,108,100,69,244,119,125,134,167,90,86,199,150,65,79,8,138,217,200,73,187,194,209,138,232,239,250,203,217,244,227,12,79,181,172,77,126,174,181,142,45,131,158,207,28,152,135,81,18,194,74,16,35,217,83,211,112,244,120,146,65,239,97,85,215,174,46,20,230,181,55,215,181,152,28,150,132,131,5,89,152,27,130,24,169,0,155,219,250,45,176,154,203,54,169,93,93,119,230,28,108,108,255,223,63,65,212,158,14,90,205,162,36,132,149,227,21,159,140,32,70,178,167,97,119,169,190,166,225,232,241,231,208,243,232,36,131,222,195,101,178,197,218,170,174,93,93,235,159,70,68,40,204,107,111,105,253,112,118,174,107,49,57,239,90,42,32,44,9,7,11,109,56,28,18,243,54,70,223,178,7,93,198,113,84,112,237,48,101,107,244,247,243,42,187,182,194,49,162,117,145,28,137,52,160,7,144,251,188,159,23,186,141,132,14,121,222,169,37,56,239,178,60,255,121,243,115,190,72,232,106,125,27,197,65,60,42,222,88,5,79,121,240,68,126,98,233,135,45,79,194,198,28,84,219,1,138,21,148,64,187,14,141,131,232,35,166,194,217,56,191,13,197,160,56,76,244,187,33,143,167,150,10,206,150,141,19,9,0,204,92,72,49,215,69,139,98,250,110,202,83,225,119,84,93,187,186,21,108,160,163,214,63,141,136,151,14,150,145,80,152,215,222,17,169,204,199,210,250,225,236,147,203,250,245,92,215,98,114,29,230,121,107,222,181,84,64,159,132,79,89,88,18,14,22,25,35,21,15,218,112,56,36,155,65,35,61,167,107,253,101,230,90,230,124,37,9,203,87,100,56,208,78,163,174,145,1,226,159,138,24,33,204,167,51,96,253,188,42,175,225,36,173,238,208,63,180,45,131,18,159,108,178,9,134,171,36,72,201,234,21,83,208,41,70,126,251,104,119,101,226,246,121,63,47,183,72,36,54,116,27,9,29,53,42,18,4,242,188,83,75,179,141,72,82,112,222,101,121,49,239,126,96,254,243,230,231,191,194,253,254,124,145,208,213,61,160,203,204,250,54,138,131,187,7,145,154,120,84,188,177,57,101,167,168,75,152,131,59,10,169,152,34,201,250,181,9,136,203,174,16,79,93,239,95,14,108,244,70,205,63,217,109,140,14,194,116,67,18,90,243,2,35,65,234,193,112,108,193,128,65,119,216,71,215,54,151,6,230,45,142,197,181,0,165,132,132,27,188,26,138,65,113,91,187,90,104,152,232,119,67,217,217,108,90,30,79,45,21,95,126,54,12,156,45,27,39,221,28,0,62,18,0,152,185,83,49,131,160,144,98,174,139,209,83,181,146,22,197,244,221,87,244,239,196,148,167,194,239,213,150,217,246,233,188,7,174,168,141,28,183,107,222,49,156,42,239,42,133,237,121,107,202,172,72,112,211,111,27,93,248,46,42,70,225,225,54,222,102,160,7,197,127,99,84,232,84,34,101,243,77,229,243,178,2,164,194,169,27,103,145,132,48,38,160,159,41,184,174,197,228,249,159,222,253,58,204,243,214,123,253,232,207,188,107,169,128,253,90,178,153,62,9,159,178,127,56,132,171,176,36,28,44,241,21,7,53,50,70,42,30,115,119,49,7,180,225,112,72,245,208,107,81,54,131,70,122,119,178,93,99,78,215,250,203,15,230,225,210,204,181,204,249,141,132,215,224,74,18,150,175,11,35,141,182,200,112,160,157,137,65,187,132,70,93,35,3,7,108,56,26,196,63,21,49,133,14,14,40,66,152,79,103,3,169,84,126,192,250,121,85,129,203,98,76,31,197,56,129,94,244,35,152,157,167,14,179,220,150,21,170,27,0,84,229,90,49,79,252,153,98,98,215,216,83,121,206,23,79,225,73,86,126,250,80,149,45,215,123,212,28,204,98,19,138,141,45,82,187,150,52,145,232,187,31,208,217,160,6,236,243,126,94,173,194,101,71,110,145,72,108,47,160,83,117,232,54,18,58,169,7,9,35,106,84,36,8,43,101,63,17,228,121,167,150,165,72,188,143,102,27,145,164,39,42,138,189,224,188,203,242,161,141,208,235,98,222,253,192,35,239,230,217,189,225,188,20,252,208,167,13,63,131,138,38,126,178,145,63,185,36,208,112,248,21,203,105,59,70,230,66,122,119,253,91,181,107,101,220,244,90,126,197,55,9,83,238,118,56,72,247,177,174,9,184,240,159,18,161,51,204,63,138,114,253,36,147,0,0,0,0,55,106,194,1,110,212,132,3,89,190,70,2,220,168,9,7,235,194,203,6,178,124,141,4,133,22,79,5,184,81,19,14,143,59,209,15,214,133,151,13,225,239,85,12,100,249,26,9,83,147,216,8,10,45,158,10,61,71,92,11,112,163,38,28,71,201,228,29,30,119,162,31,41,29,96,30,172,11,47,27,155,97,237,26,194,223,171,24,245,181,105,25,200,242,53,18,255,152,247,19,166,38,177,17,145,76,115,16,20,90,60,21,35,48,254,20,122,142,184,22,77,228,122,23,224,70,77,56,215,44,143,57,142,146,201,59,185,248,11,58,60,238,68,63,11,132,134,62,82,58,192,60,101,80,2,61,88,23,94,54,111,125,156,55,54,195,218,53,1,169,24,52,132,191,87,49,179,213,149,48,234,107,211,50,221,1,17,51,144,229,107,36,167,143,169,37,254,49,239,39,201,91,45,38,76,77,98,35,123,39,160,34,34,153,230,32,21,243,36,33,40,180,120,42,31,222,186,43,70,96,252,41,113,10,62,40,244,28,113,45,195,118,179,44,154,200,245,46,173,162,55,47,192,141,154,112,247,231,88,113,174,89,30,115,153,51,220,114,28,37,147,119,43,79,81,118,114,241,23,116,69,155,213,117,120,220,137,126,79,182,75,127,22,8,13,125,33,98,207,124,164,116,128,121,147,30,66,120,202,160,4,122,253,202,198,123,176,46,188,108,135,68,126,109,222,250,56,111,233,144,250,110,108,134,181,107,91,236,119,106,2,82,49,104,53,56,243,105,8,127,175,98,63,21,109,99,102,171,43,97,81,193,233,96,212,215,166,101,227,189,100,100,186,3,34,102,141,105,224,103,32,203,215,72,23,161,21,73,78,31,83,75,121,117,145,74,252,99,222,79,203,9,28,78,146,183,90,76,165,221,152,77,152,154,196,70,175,240,6,71,246,78,64,69,193,36,130,68,68,50,205,65,115,88,15,64,42,230,73,66,29,140,139,67,80,104,241,84,103,2,51,85,62,188,117,87,9,214,183,86,140,192,248,83,187,170,58,82,226,20,124,80,213,126,190,81,232,57,226,90,223,83,32,91,134,237,102,89,177,135,164,88,52,145,235,93,3,251,41,92,90,69,111,94,109,47,173,95,128,27,53,225,183,113,247,224,238,207,177,226,217,165,115,227,92,179,60,230,107,217,254,231,50,103,184,229,5,13,122,228,56,74,38,239,15,32,228,238,86,158,162,236,97,244,96,237,228,226,47,232,211,136,237,233,138,54,171,235,189,92,105,234,240,184,19,253,199,210,209,252,158,108,151,254,169,6,85,255,44,16,26,250,27,122,216,251,66,196,158,249,117,174,92,248,72,233,0,243,127,131,194,242,38,61,132,240,17,87,70,241,148,65,9,244,163,43,203,245,250,149,141,247,205,255,79,246,96,93,120,217,87,55,186,216,14,137,252,218,57,227,62,219,188,245,113,222,139,159,179,223,210,33,245,221,229,75,55,220,216,12,107,215,239,102,169,214,182,216,239,212,129,178,45,213,4,164,98,208,51,206,160,209,106,112,230,211,93,26,36,210,16,254,94,197,39,148,156,196,126,42,218,198,73,64,24,199,204,86,87,194,251,60,149,195,162,130,211,193,149,232,17,192,168,175,77,203,159,197,143,202,198,123,201,200,241,17,11,201,116,7,68,204,67,109,134,205,26,211,192,207,45,185,2,206,64,150,175,145,119,252,109,144,46,66,43,146,25,40,233,147,156,62,166,150,171,84,100,151,242,234,34,149,197,128,224,148,248,199,188,159,207,173,126,158,150,19,56,156,161,121,250,157,36,111,181,152,19,5,119,153,74,187,49,155,125,209,243,154,48,53,137,141,7,95,75,140,94,225,13,142,105,139,207,143,236,157,128,138,219,247,66,139,130,73,4,137,181,35,198,136,136,100,154,131,191,14,88,130,230,176,30,128,209,218,220,129,84,204,147,132,99,166,81,133,58,24,23,135,13,114,213,134,160,208,226,169,151,186,32,168,206,4,102,170,249,110,164,171,124,120,235,174,75,18,41,175,18,172,111,173,37,198,173,172,24,129,241,167,47,235,51,166,118,85,117,164,65,63,183,165,196,41,248,160,243,67,58,161,170,253,124,163,157,151,190,162,208,115,196,181,231,25,6,180,190,167,64,182,137,205,130,183,12,219,205,178,59,177,15,179,98,15,73,177,85,101,139,176,104,34,215,187,95,72,21,186,6,246,83,184,49,156,145,185,180,138,222,188,131,224,28,189,218,94,90,191,237,52,152,190,0,0,0,0,101,103,188,184,139,200,9,170,238,175,181,18,87,151,98,143,50,240,222,55,220,95,107,37,185,56,215,157,239,40,180,197,138,79,8,125,100,224,189,111,1,135,1,215,184,191,214,74,221,216,106,242,51,119,223,224,86,16,99,88,159,87,25,80,250,48,165,232,20,159,16,250,113,248,172,66,200,192,123,223,173,167,199,103,67,8,114,117,38,111,206,205,112,127,173,149,21,24,17,45,251,183,164,63,158,208,24,135,39,232,207,26,66,143,115,162,172,32,198,176,201,71,122,8,62,175,50,160,91,200,142,24,181,103,59,10,208,0,135,178,105,56,80,47,12,95,236,151,226,240,89,133,135,151,229,61,209,135,134,101,180,224,58,221,90,79,143,207,63,40,51,119,134,16,228,234,227,119,88,82,13,216,237,64,104,191,81,248,161,248,43,240,196,159,151,72,42,48,34,90,79,87,158,226,246,111,73,127,147,8,245,199,125,167,64,213,24,192,252,109,78,208,159,53,43,183,35,141,197,24,150,159,160,127,42,39,25,71,253,186,124,32,65,2,146,143,244,16,247,232,72,168,61,88,20,155,88,63,168,35,182,144,29,49,211,247,161,137,106,207,118,20,15,168,202,172,225,7,127,190,132,96,195,6,210,112,160,94,183,23,28,230,89,184,169,244,60,223,21,76,133,231,194,209,224,128,126,105,14,47,203,123,107,72,119,195,162,15,13,203,199,104,177,115,41,199,4,97,76,160,184,217,245,152,111,68,144,255,211,252,126,80,102,238,27,55,218,86,77,39,185,14,40,64,5,182,198,239,176,164,163,136,12,28,26,176,219,129,127,215,103,57,145,120,210,43,244,31,110,147,3,247,38,59,102,144,154,131,136,63,47,145,237,88,147,41,84,96,68,180,49,7,248,12,223,168,77,30,186,207,241,166,236,223,146,254,137,184,46,70,103,23,155,84,2,112,39,236,187,72,240,113,222,47,76,201,48,128,249,219,85,231,69,99,156,160,63,107,249,199,131,211,23,104,54,193,114,15,138,121,203,55,93,228,174,80,225,92,64,255,84,78,37,152,232,246,115,136,139,174,22,239,55,22,248,64,130,4,157,39,62,188,36,31,233,33,65,120,85,153,175,215,224,139,202,176,92,51,59,182,89,237,94,209,229,85,176,126,80,71,213,25,236,255,108,33,59,98,9,70,135,218,231,233,50,200,130,142,142,112,212,158,237,40,177,249,81,144,95,86,228,130,58,49,88,58,131,9,143,167,230,110,51,31,8,193,134,13,109,166,58,181,164,225,64,189,193,134,252,5,47,41,73,23,74,78,245,175,243,118,34,50,150,17,158,138,120,190,43,152,29,217,151,32,75,201,244,120,46,174,72,192,192,1,253,210,165,102,65,106,28,94,150,247,121,57,42,79,151,150,159,93,242,241,35,229,5,25,107,77,96,126,215,245,142,209,98,231,235,182,222,95,82,142,9,194,55,233,181,122,217,70,0,104,188,33,188,208,234,49,223,136,143,86,99,48,97,249,214,34,4,158,106,154,189,166,189,7,216,193,1,191,54,110,180,173,83,9,8,21,154,78,114,29,255,41,206,165,17,134,123,183,116,225,199,15,205,217,16,146,168,190,172,42,70,17,25,56,35,118,165,128,117,102,198,216,16,1,122,96,254,174,207,114,155,201,115,202,34,241,164,87,71,150,24,239,169,57,173,253,204,94,17,69,6,238,77,118,99,137,241,206,141,38,68,220,232,65,248,100,81,121,47,249,52,30,147,65,218,177,38,83,191,214,154,235,233,198,249,179,140,161,69,11,98,14,240,25,7,105,76,161,190,81,155,60,219,54,39,132,53,153,146,150,80,254,46,46,153,185,84,38,252,222,232,158,18,113,93,140,119,22,225,52,206,46,54,169,171,73,138,17,69,230,63,3,32,129,131,187,118,145,224,227,19,246,92,91,253,89,233,73,152,62,85,241,33,6,130,108,68,97,62,212,170,206,139,198,207,169,55,126,56,65,127,214,93,38,195,110,179,137,118,124,214,238,202,196,111,214,29,89,10,177,161,225,228,30,20,243,129,121,168,75,215,105,203,19,178,14,119,171,92,161,194,185,57,198,126,1,128,254,169,156,229,153,21,36,11,54,160,54,110,81,28,142,167,22,102,134,194,113,218,62,44,222,111,44,73,185,211,148,240,129,4,9,149,230,184,177,123,73,13,163,30,46,177,27,72,62,210,67,45,89,110,251,195,246,219,233,166,145,103,81,31,169,176,204,122,206,12,116,148,97,185,102,241,6,5,222,0,0,0,0,119,7,48,150,238,14,97,44,153,9,81,186,7,109,196,25,112,106,244,143,233,99,165,53,158,100,149,163,14,219,136,50,121,220,184,164,224,213,233,30,151,210,217,136,9,182,76,43,126,177,124,189,231,184,45,7,144,191,29,145,29,183,16,100,106,176,32,242,243,185,113,72,132,190,65,222,26,218,212,125,109,221,228,235,244,212,181,81,131,211,133,199,19,108,152,86,100,107,168,192,253,98,249,122,138,101,201,236,20,1,92,79,99,6,108,217,250,15,61,99,141,8,13,245,59,110,32,200,76,105,16,94,213,96,65,228,162,103,113,114,60,3,228,209,75,4,212,71,210,13,133,253,165,10,181,107,53,181,168,250,66,178,152,108,219,187,201,214,172,188,249,64,50,216,108,227,69,223,92,117,220,214,13,207,171,209,61,89,38,217,48,172,81,222,0,58,200,215,81,128,191,208,97,22,33,180,244,181,86,179,196,35,207,186,149,153,184,189,165,15,40,2,184,158,95,5,136,8,198,12,217,178,177,11,233,36,47,111,124,135,88,104,76,17,193,97,29,171,182,102,45,61,118,220,65,144,1,219,113,6,152,210,32,188,239,213,16,42,113,177,133,137,6,182,181,31,159,191,228,165,232,184,212,51,120,7,201,162,15,0,249,52,150,9,168,142,225,14,152,24,127,106,13,187,8,109,61,45,145,100,108,151,230,99,92,1,107,107,81,244,28,108,97,98,133,101,48,216,242,98,0,78,108,6,149,237,27,1,165,123,130,8,244,193,245,15,196,87,101,176,217,198,18,183,233,80,139,190,184,234,252,185,136,124,98,221,29,223,21,218,45,73,140,211,124,243,251,212,76,101,77,178,97,88,58,181,81,206,163,188,0,116,212,187,48,226,74,223,165,65,61,216,149,215,164,209,196,109,211,214,244,251,67,105,233,106,52,110,217,252,173,103,136,70,218,96,184,208,68,4,45,115,51,3,29,229,170,10,76,95,221,13,124,201,80,5,113,60,39,2,65,170,190,11,16,16,201,12,32,134,87,104,181,37,32,111,133,179,185,102,212,9,206,97,228,159,94,222,249,14,41,217,201,152,176,208,152,34,199,215,168,180,89,179,61,23,46,180,13,129,183,189,92,59,192,186,108,173,237,184,131,32,154,191,179,182,3,182,226,12,116,177,210,154,234,213,71,57,157,210,119,175,4,219,38,21,115,220,22,131,227,99,11,18,148,100,59,132,13,109,106,62,122,106,90,168,228,14,207,11,147,9,255,157,10,0,174,39,125,7,158,177,240,15,147,68,135,8,163,210,30,1,242,104,105,6,194,254,247,98,87,93,128,101,103,203,25,108,54,113,110,107,6,231,254,212,27,118,137,211,43,224,16,218,122,90,103,221,74,204,249,185,223,111,142,190,239,249,23,183,190,67,96,176,142,213,214,214,163,232,161,209,147,126,56,216,194,196,79,223,242,82,209,187,103,241,166,188,87,103,63,181,6,221,72,178,54,75,216,13,43,218,175,10,27,76,54,3,74,246,65,4,122,96,223,96,239,195,168,103,223,85,49,110,142,239,70,105,190,121,203,97,179,140,188,102,131,26,37,111,210,160,82,104,226,54,204,12,119,149,187,11,71,3,34,2,22,185,85,5,38,47,197,186,59,190,178,189,11,40,43,180,90,146,92,179,106,4,194,215,255,167,181,208,207,49,44,217,158,139,91,222,174,29,155,100,194,176,236,99,242,38,117,106,163,156,2,109,147,10,156,9,6,169,235,14,54,63,114,7,103,133,5,0,87,19,149,191,74,130,226,184,122,20,123,177,43,174,12,182,27,56,146,210,142,155,229,213,190,13,124,220,239,183,11,219,223,33,134,211,210,212,241,212,226,66,104,221,179,248,31,218,131,110,129,190,22,205,246,185,38,91,111,176,119,225,24,183,71,119,136,8,90,230,255,15,106,112,102,6,59,202,17,1,11,92,143,101,158,255,248,98,174,105,97,107,255,211,22,108,207,69,160,10,226,120,215,13,210,238,78,4,131,84,57,3,179,194,167,103,38,97,208,96,22,247,73,105,71,77,62,110,119,219,174,209,106,74,217,214,90,220,64,223,11,102,55,216,59,240,169,188,174,83,222,187,158,197,71,178,207,127,48,181,255,233,189,189,242,28,202,186,194,138,83,179,147,48,36,180,163,166,186,208,54,5,205,215,6,147,84,222,87,41,35,217,103,191,179,102,122,46,196,97,74,184,93,104,27,2,42,111,43,148,180,11,190,55,195,12,142,161,90,5,223,27,45,2,239,141,0,0,0,0,25,27,49,65,50,54,98,130,43,45,83,195,100,108,197,4,125,119,244,69,86,90,167,134,79,65,150,199,200,217,138,8,209,194,187,73,250,239,232,138,227,244,217,203,172,181,79,12,181,174,126,77,158,131,45,142,135,152,28,207,74,194,18,81,83,217,35,16,120,244,112,211,97,239,65,146,46,174,215,85,55,181,230,20,28,152,181,215,5,131,132,150,130,27,152,89,155,0,169,24,176,45,250,219,169,54,203,154,230,119,93,93,255,108,108,28,212,65,63,223,205,90,14,158,149,132,36,162,140,159,21,227,167,178,70,32,190,169,119,97,241,232,225,166,232,243,208,231,195,222,131,36,218,197,178,101,93,93,174,170,68,70,159,235,111,107,204,40,118,112,253,105,57,49,107,174,32,42,90,239,11,7,9,44,18,28,56,109,223,70,54,243,198,93,7,178,237,112,84,113,244,107,101,48,187,42,243,247,162,49,194,182,137,28,145,117,144,7,160,52,23,159,188,251,14,132,141,186,37,169,222,121,60,178,239,56,115,243,121,255,106,232,72,190,65,197,27,125,88,222,42,60,240,121,79,5,233,98,126,68,194,79,45,135,219,84,28,198,148,21,138,1,141,14,187,64,166,35,232,131,191,56,217,194,56,160,197,13,33,187,244,76,10,150,167,143,19,141,150,206,92,204,0,9,69,215,49,72,110,250,98,139,119,225,83,202,186,187,93,84,163,160,108,21,136,141,63,214,145,150,14,151,222,215,152,80,199,204,169,17,236,225,250,210,245,250,203,147,114,98,215,92,107,121,230,29,64,84,181,222,89,79,132,159,22,14,18,88,15,21,35,25,36,56,112,218,61,35,65,155,101,253,107,167,124,230,90,230,87,203,9,37,78,208,56,100,1,145,174,163,24,138,159,226,51,167,204,33,42,188,253,96,173,36,225,175,180,63,208,238,159,18,131,45,134,9,178,108,201,72,36,171,208,83,21,234,251,126,70,41,226,101,119,104,47,63,121,246,54,36,72,183,29,9,27,116,4,18,42,53,75,83,188,242,82,72,141,179,121,101,222,112,96,126,239,49,231,230,243,254,254,253,194,191,213,208,145,124,204,203,160,61,131,138,54,250,154,145,7,187,177,188,84,120,168,167,101,57,59,131,152,75,34,152,169,10,9,181,250,201,16,174,203,136,95,239,93,79,70,244,108,14,109,217,63,205,116,194,14,140,243,90,18,67,234,65,35,2,193,108,112,193,216,119,65,128,151,54,215,71,142,45,230,6,165,0,181,197,188,27,132,132,113,65,138,26,104,90,187,91,67,119,232,152,90,108,217,217,21,45,79,30,12,54,126,95,39,27,45,156,62,0,28,221,185,152,0,18,160,131,49,83,139,174,98,144,146,181,83,209,221,244,197,22,196,239,244,87,239,194,167,148,246,217,150,213,174,7,188,233,183,28,141,168,156,49,222,107,133,42,239,42,202,107,121,237,211,112,72,172,248,93,27,111,225,70,42,46,102,222,54,225,127,197,7,160,84,232,84,99,77,243,101,34,2,178,243,229,27,169,194,164,48,132,145,103,41,159,160,38,228,197,174,184,253,222,159,249,214,243,204,58,207,232,253,123,128,169,107,188,153,178,90,253,178,159,9,62,171,132,56,127,44,28,36,176,53,7,21,241,30,42,70,50,7,49,119,115,72,112,225,180,81,107,208,245,122,70,131,54,99,93,178,119,203,250,215,78,210,225,230,15,249,204,181,204,224,215,132,141,175,150,18,74,182,141,35,11,157,160,112,200,132,187,65,137,3,35,93,70,26,56,108,7,49,21,63,196,40,14,14,133,103,79,152,66,126,84,169,3,85,121,250,192,76,98,203,129,129,56,197,31,152,35,244,94,179,14,167,157,170,21,150,220,229,84,0,27,252,79,49,90,215,98,98,153,206,121,83,216,73,225,79,23,80,250,126,86,123,215,45,149,98,204,28,212,45,141,138,19,52,150,187,82,31,187,232,145,6,160,217,208,94,126,243,236,71,101,194,173,108,72,145,110,117,83,160,47,58,18,54,232,35,9,7,169,8,36,84,106,17,63,101,43,150,167,121,228,143,188,72,165,164,145,27,102,189,138,42,39,242,203,188,224,235,208,141,161,192,253,222,98,217,230,239,35,20,188,225,189,13,167,208,252,38,138,131,63,63,145,178,126,112,208,36,185,105,203,21,248,66,230,70,59,91,253,119,122,220,101,107,181,197,126,90,244,238,83,9,55,247,72,56,118,184,9,174,177,161,18,159,240,138,63,204,51,147,36,253,114,0,0,0,0,1,194,106,55,3,132,212,110,2,70,190,89,7,9,168,220,6,203,194,235,4,141,124,178,5,79,22,133,14,19,81,184,15,209,59,143,13,151,133,214,12,85,239,225,9,26,249,100,8,216,147,83,10,158,45,10,11,92,71,61,28,38,163,112,29,228,201,71,31,162,119,30,30,96,29,41,27,47,11,172,26,237,97,155,24,171,223,194,25,105,181,245,18,53,242,200,19,247,152,255,17,177,38,166,16,115,76,145,21,60,90,20,20,254,48,35,22,184,142,122,23,122,228,77,56,77,70,224,57,143,44,215,59,201,146,142,58,11,248,185,63,68,238,60,62,134,132,11,60,192,58,82,61,2,80,101,54,94,23,88,55,156,125,111,53,218,195,54,52,24,169,1,49,87,191,132,48,149,213,179,50,211,107,234,51,17,1,221,36,107,229,144,37,169,143,167,39,239,49,254,38,45,91,201,35,98,77,76,34,160,39,123,32,230,153,34,33,36,243,21,42,120,180,40,43,186,222,31,41,252,96,70,40,62,10,113,45,113,28,244,44,179,118,195,46,245,200,154,47,55,162,173,112,154,141,192,113,88,231,247,115,30,89,174,114,220,51,153,119,147,37,28,118,81,79,43,116,23,241,114,117,213,155,69,126,137,220,120,127,75,182,79,125,13,8,22,124,207,98,33,121,128,116,164,120,66,30,147,122,4,160,202,123,198,202,253,108,188,46,176,109,126,68,135,111,56,250,222,110,250,144,233,107,181,134,108,106,119,236,91,104,49,82,2,105,243,56,53,98,175,127,8,99,109,21,63,97,43,171,102,96,233,193,81,101,166,215,212,100,100,189,227,102,34,3,186,103,224,105,141,72,215,203,32,73,21,161,23,75,83,31,78,74,145,117,121,79,222,99,252,78,28,9,203,76,90,183,146,77,152,221,165,70,196,154,152,71,6,240,175,69,64,78,246,68,130,36,193,65,205,50,68,64,15,88,115,66,73,230,42,67,139,140,29,84,241,104,80,85,51,2,103,87,117,188,62,86,183,214,9,83,248,192,140,82,58,170,187,80,124,20,226,81,190,126,213,90,226,57,232,91,32,83,223,89,102,237,134,88,164,135,177,93,235,145,52,92,41,251,3,94,111,69,90,95,173,47,109,225,53,27,128,224,247,113,183,226,177,207,238,227,115,165,217,230,60,179,92,231,254,217,107,229,184,103,50,228,122,13,5,239,38,74,56,238,228,32,15,236,162,158,86,237,96,244,97,232,47,226,228,233,237,136,211,235,171,54,138,234,105,92,189,253,19,184,240,252,209,210,199,254,151,108,158,255,85,6,169,250,26,16,44,251,216,122,27,249,158,196,66,248,92,174,117,243,0,233,72,242,194,131,127,240,132,61,38,241,70,87,17,244,9,65,148,245,203,43,163,247,141,149,250,246,79,255,205,217,120,93,96,216,186,55,87,218,252,137,14,219,62,227,57,222,113,245,188,223,179,159,139,221,245,33,210,220,55,75,229,215,107,12,216,214,169,102,239,212,239,216,182,213,45,178,129,208,98,164,4,209,160,206,51,211,230,112,106,210,36,26,93,197,94,254,16,196,156,148,39,198,218,42,126,199,24,64,73,194,87,86,204,195,149,60,251,193,211,130,162,192,17,232,149,203,77,175,168,202,143,197,159,200,201,123,198,201,11,17,241,204,68,7,116,205,134,109,67,207,192,211,26,206,2,185,45,145,175,150,64,144,109,252,119,146,43,66,46,147,233,40,25,150,166,62,156,151,100,84,171,149,34,234,242,148,224,128,197,159,188,199,248,158,126,173,207,156,56,19,150,157,250,121,161,152,181,111,36,153,119,5,19,155,49,187,74,154,243,209,125,141,137,53,48,140,75,95,7,142,13,225,94,143,207,139,105,138,128,157,236,139,66,247,219,137,4,73,130,136,198,35,181,131,154,100,136,130,88,14,191,128,30,176,230,129,220,218,209,132,147,204,84,133,81,166,99,135,23,24,58,134,213,114,13,169,226,208,160,168,32,186,151,170,102,4,206,171,164,110,249,174,235,120,124,175,41,18,75,173,111,172,18,172,173,198,37,167,241,129,24,166,51,235,47,164,117,85,118,165,183,63,65,160,248,41,196,161,58,67,243,163,124,253,170,162,190,151,157,181,196,115,208,180,6,25,231,182,64,167,190,183,130,205,137,178,205,219,12,179,15,177,59,177,73,15,98,176,139,101,85,187,215,34,104,186,21,72,95,184,83,246,6,185,145,156,49,188,222,138,180,189,28,224,131,191,90,94,218,190,152,52,237,0,0,0,0,184,188,103,101,170,9,200,139,18,181,175,238,143,98,151,87,55,222,240,50,37,107,95,220,157,215,56,185,197,180,40,239,125,8,79,138,111,189,224,100,215,1,135,1,74,214,191,184,242,106,216,221,224,223,119,51,88,99,16,86,80,25,87,159,232,165,48,250,250,16,159,20,66,172,248,113,223,123,192,200,103,199,167,173,117,114,8,67,205,206,111,38,149,173,127,112,45,17,24,21,63,164,183,251,135,24,208,158,26,207,232,39,162,115,143,66,176,198,32,172,8,122,71,201,160,50,175,62,24,142,200,91,10,59,103,181,178,135,0,208,47,80,56,105,151,236,95,12,133,89,240,226,61,229,151,135,101,134,135,209,221,58,224,180,207,143,79,90,119,51,40,63,234,228,16,134,82,88,119,227,64,237,216,13,248,81,191,104,240,43,248,161,72,151,159,196,90,34,48,42,226,158,87,79,127,73,111,246,199,245,8,147,213,64,167,125,109,252,192,24,53,159,208,78,141,35,183,43,159,150,24,197,39,42,127,160,186,253,71,25,2,65,32,124,16,244,143,146,168,72,232,247,155,20,88,61,35,168,63,88,49,29,144,182,137,161,247,211,20,118,207,106,172,202,168,15,190,127,7,225,6,195,96,132,94,160,112,210,230,28,23,183,244,169,184,89,76,21,223,60,209,194,231,133,105,126,128,224,123,203,47,14,195,119,72,107,203,13,15,162,115,177,104,199,97,4,199,41,217,184,160,76,68,111,152,245,252,211,255,144,238,102,80,126,86,218,55,27,14,185,39,77,182,5,64,40,164,176,239,198,28,12,136,163,129,219,176,26,57,103,215,127,43,210,120,145,147,110,31,244,59,38,247,3,131,154,144,102,145,47,63,136,41,147,88,237,180,68,96,84,12,248,7,49,30,77,168,223,166,241,207,186,254,146,223,236,70,46,184,137,84,155,23,103,236,39,112,2,113,240,72,187,201,76,47,222,219,249,128,48,99,69,231,85,107,63,160,156,211,131,199,249,193,54,104,23,121,138,15,114,228,93,55,203,92,225,80,174,78,84,255,64,246,232,152,37,174,139,136,115,22,55,239,22,4,130,64,248,188,62,39,157,33,233,31,36,153,85,120,65,139,224,215,175,51,92,176,202,237,89,182,59,85,229,209,94,71,80,126,176,255,236,25,213,98,59,33,108,218,135,70,9,200,50,233,231,112,142,142,130,40,237,158,212,144,81,249,177,130,228,86,95,58,88,49,58,167,143,9,131,31,51,110,230,13,134,193,8,181,58,166,109,189,64,225,164,5,252,134,193,23,73,41,47,175,245,78,74,50,34,118,243,138,158,17,150,152,43,190,120,32,151,217,29,120,244,201,75,192,72,174,46,210,253,1,192,106,65,102,165,247,150,94,28,79,42,57,121,93,159,150,151,229,35,241,242,77,107,25,5,245,215,126,96,231,98,209,142,95,222,182,235,194,9,142,82,122,181,233,55,104,0,70,217,208,188,33,188,136,223,49,234,48,99,86,143,34,214,249,97,154,106,158,4,7,189,166,189,191,1,193,216,173,180,110,54,21,8,9,83,29,114,78,154,165,206,41,255,183,123,134,17,15,199,225,116,146,16,217,205,42,172,190,168,56,25,17,70,128,165,118,35,216,198,102,117,96,122,1,16,114,207,174,254,202,115,201,155,87,164,241,34,239,24,150,71,253,173,57,169,69,17,94,204,118,77,238,6,206,241,137,99,220,68,38,141,100,248,65,232,249,47,121,81,65,147,30,52,83,38,177,218,235,154,214,191,179,249,198,233,11,69,161,140,25,240,14,98,161,76,105,7,60,155,81,190,132,39,54,219,150,146,153,53,46,46,254,80,38,84,185,153,158,232,222,252,140,93,113,18,52,225,22,119,169,54,46,206,17,138,73,171,3,63,230,69,187,131,129,32,227,224,145,118,91,92,246,19,73,233,89,253,241,85,62,152,108,130,6,33,212,62,97,68,198,139,206,170,126,55,169,207,214,127,65,56,110,195,38,93,124,118,137,179,196,202,238,214,89,29,214,111,225,161,177,10,243,20,30,228,75,168,121,129,19,203,105,215,171,119,14,178,185,194,161,92,1,126,198,57,156,169,254,128,36,21,153,229,54,160,54,11,142,28,81,110,134,102,22,167,62,218,113,194,44,111,222,44,148,211,185,73,9,4,129,240,177,184,230,149,163,13,73,123,27,177,46,30,67,210,62,72,251,110,89,45,233,219,246,195,81,103,145,166,204,176,169,31,116,12,206,122,102,185,97,148,222,5,6,241,49,46,50,46,51,0,0,0,16,0,17,0,18,0,0,0,8,0,7,0,9,0,6,0,10,0,5,0,11,0,4,0,12,0,3,0,13,0,2,0,14,0,1,0,15,0,0,0,105,110,99,111,114,114,101,99,116,32,104,101,97,100,101,114,32,99,104,101,99,107,0,0,117,110,107,110,111,119,110,32,99,111,109,112,114,101,115,115,105,111,110,32,109,101,116,104,111,100,0,0,0,0,0,0,105,110,118,97,108,105,100,32,119,105,110,100,111,119,32,115,105,122,101,0,0,0,0,0,117,110,107,110,111,119,110,32,104,101,97,100,101,114,32,102,108,97,103,115,32,115,101,116,0,0,0,0,0,0,0,0,104,101,97,100,101,114,32,99,114,99,32,109,105,115,109,97,116,99,104,0,0,0,0,0,105,110,118,97,108,105,100,32,98,108,111,99,107,32,116,121,112,101,0,0,0,0,0,0,105,110,118,97,108,105,100,32,115,116,111,114,101,100,32,98,108,111,99,107,32,108,101,110,103,116,104,115,0,0,0,0,116,111,111,32,109,97,110,121,32,108,101,110,103,116,104,32,111,114,32,100,105,115,116,97,110,99,101,32,115,121,109,98,111,108,115,0,0,0,0,0,105,110,118,97,108,105,100,32,99,111,100,101,32,108,101,110,103,116,104,115,32,115,101,116,0,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,98,105,116,32,108,101,110,103,116,104,32,114,101,112,101,97,116,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,108,105,116,101,114,97,108,47,108,101,110,103,116,104,115,32,115,101,116,0,0,0,0,0,105,110,118,97,108,105,100,32,100,105,115,116,97,110,99,101,115,32,115,101,116,0,0,0,105,110,118,97,108,105,100,32,108,105,116,101,114,97,108,47,108,101,110,103,116,104,32,99,111,100,101,0,0,0,0,0,105,110,118,97,108,105,100,32,100,105,115,116,97,110,99,101,32,99,111,100,101,0,0,0,105,110,118,97,108,105,100,32,100,105,115,116,97,110,99,101,32,116,111,111,32,102,97,114,32,98,97,99,107,0,0,0,105,110,99,111,114,114,101,99,116,32,100,97,116,97,32,99,104,101,99,107,0,0,0,0,105,110,99,111,114,114,101,99,116,32,108,101,110,103,116,104,32,99,104,101,99,107,0,0,96,7,0,0,0,8,80,0,0,8,16,0,20,8,115,0,18,7,31,0,0,8,112,0,0,8,48,0,0,9,192,0,16,7,10,0,0,8,96,0,0,8,32,0,0,9,160,0,0,8,0,0,0,8,128,0,0,8,64,0,0,9,224,0,16,7,6,0,0,8,88,0,0,8,24,0,0,9,144,0,19,7,59,0,0,8,120,0,0,8,56,0,0,9,208,0,17,7,17,0,0,8,104,0,0,8,40,0,0,9,176,0,0,8,8,0,0,8,136,0,0,8,72,0,0,9,240,0,16,7,4,0,0,8,84,0,0,8,20,0,21,8,227,0,19,7,43,0,0,8,116,0,0,8,52,0,0,9,200,0,17,7,13,0,0,8,100,0,0,8,36,0,0,9,168,0,0,8,4,0,0,8,132,0,0,8,68,0,0,9,232,0,16,7,8,0,0,8,92,0,0,8,28,0,0,9,152,0,20,7,83,0,0,8,124,0,0,8,60,0,0,9,216,0,18,7,23,0,0,8,108,0,0,8,44,0,0,9,184,0,0,8,12,0,0,8,140,0,0,8,76,0,0,9,248,0,16,7,3,0,0,8,82,0,0,8,18,0,21,8,163,0,19,7,35,0,0,8,114,0,0,8,50,0,0,9,196,0,17,7,11,0,0,8,98,0,0,8,34,0,0,9,164,0,0,8,2,0,0,8,130,0,0,8,66,0,0,9,228,0,16,7,7,0,0,8,90,0,0,8,26,0,0,9,148,0,20,7,67,0,0,8,122,0,0,8,58,0,0,9,212,0,18,7,19,0,0,8,106,0,0,8,42,0,0,9,180,0,0,8,10,0,0,8,138,0,0,8,74,0,0,9,244,0,16,7,5,0,0,8,86,0,0,8,22,0,64,8,0,0,19,7,51,0,0,8,118,0,0,8,54,0,0,9,204,0,17,7,15,0,0,8,102,0,0,8,38,0,0,9,172,0,0,8,6,0,0,8,134,0,0,8,70,0,0,9,236,0,16,7,9,0,0,8,94,0,0,8,30,0,0,9,156,0,20,7,99,0,0,8,126,0,0,8,62,0,0,9,220,0,18,7,27,0,0,8,110,0,0,8,46,0,0,9,188,0,0,8,14,0,0,8,142,0,0,8,78,0,0,9,252,0,96,7,0,0,0,8,81,0,0,8,17,0,21,8,131,0,18,7,31,0,0,8,113,0,0,8,49,0,0,9,194,0,16,7,10,0,0,8,97,0,0,8,33,0,0,9,162,0,0,8,1,0,0,8,129,0,0,8,65,0,0,9,226,0,16,7,6,0,0,8,89,0,0,8,25,0,0,9,146,0,19,7,59,0,0,8,121,0,0,8,57,0,0,9,210,0,17,7,17,0,0,8,105,0,0,8,41,0,0,9,178,0,0,8,9,0,0,8,137,0,0,8,73,0,0,9,242,0,16,7,4,0,0,8,85,0,0,8,21,0,16,8,2,1,19,7,43,0,0,8,117,0,0,8,53,0,0,9,202,0,17,7,13,0,0,8,101,0,0,8,37,0,0,9,170,0,0,8,5,0,0,8,133,0,0,8,69,0,0,9,234,0,16,7,8,0,0,8,93,0,0,8,29,0,0,9,154,0,20,7,83,0,0,8,125,0,0,8,61,0,0,9,218,0,18,7,23,0,0,8,109,0,0,8,45,0,0,9,186,0,0,8,13,0,0,8,141,0,0,8,77,0,0,9,250,0,16,7,3,0,0,8,83,0,0,8,19,0,21,8,195,0,19,7,35,0,0,8,115,0,0,8,51,0,0,9,198,0,17,7,11,0,0,8,99,0,0,8,35,0,0,9,166,0,0,8,3,0,0,8,131,0,0,8,67,0,0,9,230,0,16,7,7,0,0,8,91,0,0,8,27,0,0,9,150,0,20,7,67,0,0,8,123,0,0,8,59,0,0,9,214,0,18,7,19,0,0,8,107,0,0,8,43,0,0,9,182,0,0,8,11,0,0,8,139,0,0,8,75,0,0,9,246,0,16,7,5,0,0,8,87,0,0,8,23,0,64,8,0,0,19,7,51,0,0,8,119,0,0,8,55,0,0,9,206,0,17,7,15,0,0,8,103,0,0,8,39,0,0,9,174,0,0,8,7,0,0,8,135,0,0,8,71,0,0,9,238,0,16,7,9,0,0,8,95,0,0,8,31,0,0,9,158,0,20,7,99,0,0,8,127,0,0,8,63,0,0,9,222,0,18,7,27,0,0,8,111,0,0,8,47,0,0,9,190,0,0,8,15,0,0,8,143,0,0,8,79,0,0,9,254,0,96,7,0,0,0,8,80,0,0,8,16,0,20,8,115,0,18,7,31,0,0,8,112,0,0,8,48,0,0,9,193,0,16,7,10,0,0,8,96,0,0,8,32,0,0,9,161,0,0,8,0,0,0,8,128,0,0,8,64,0,0,9,225,0,16,7,6,0,0,8,88,0,0,8,24,0,0,9,145,0,19,7,59,0,0,8,120,0,0,8,56,0,0,9,209,0,17,7,17,0,0,8,104,0,0,8,40,0,0,9,177,0,0,8,8,0,0,8,136,0,0,8,72,0,0,9,241,0,16,7,4,0,0,8,84,0,0,8,20,0,21,8,227,0,19,7,43,0,0,8,116,0,0,8,52,0,0,9,201,0,17,7,13,0,0,8,100,0,0,8,36,0,0,9,169,0,0,8,4,0,0,8,132,0,0,8,68,0,0,9,233,0,16,7,8,0,0,8,92,0,0,8,28,0,0,9,153,0,20,7,83,0,0,8,124,0,0,8,60,0,0,9,217,0,18,7,23,0,0,8,108,0,0,8,44,0,0,9,185,0,0,8,12,0,0,8,140,0,0,8,76,0,0,9,249,0,16,7,3,0,0,8,82,0,0,8,18,0,21,8,163,0,19,7,35,0,0,8,114,0,0,8,50,0,0,9,197,0,17,7,11,0,0,8,98,0,0,8,34,0,0,9,165,0,0,8,2,0,0,8,130,0,0,8,66,0,0,9,229,0,16,7,7,0,0,8,90,0,0,8,26,0,0,9,149,0,20,7,67,0,0,8,122,0,0,8,58,0,0,9,213,0,18,7,19,0,0,8,106,0,0,8,42,0,0,9,181,0,0,8,10,0,0,8,138,0,0,8,74,0,0,9,245,0,16,7,5,0,0,8,86,0,0,8,22,0,64,8,0,0,19,7,51,0,0,8,118,0,0,8,54,0,0,9,205,0,17,7,15,0,0,8,102,0,0,8,38,0,0,9,173,0,0,8,6,0,0,8,134,0,0,8,70,0,0,9,237,0,16,7,9,0,0,8,94,0,0,8,30,0,0,9,157,0,20,7,99,0,0,8,126,0,0,8,62,0,0,9,221,0,18,7,27,0,0,8,110,0,0,8,46,0,0,9,189,0,0,8,14,0,0,8,142,0,0,8,78,0,0,9,253,0,96,7,0,0,0,8,81,0,0,8,17,0,21,8,131,0,18,7,31,0,0,8,113,0,0,8,49,0,0,9,195,0,16,7,10,0,0,8,97,0,0,8,33,0,0,9,163,0,0,8,1,0,0,8,129,0,0,8,65,0,0,9,227,0,16,7,6,0,0,8,89,0,0,8,25,0,0,9,147,0,19,7,59,0,0,8,121,0,0,8,57,0,0,9,211,0,17,7,17,0,0,8,105,0,0,8,41,0,0,9,179,0,0,8,9,0,0,8,137,0,0,8,73,0,0,9,243,0,16,7,4,0,0,8,85,0,0,8,21,0,16,8,2,1,19,7,43,0,0,8,117,0,0,8,53,0,0,9,203,0,17,7,13,0,0,8,101,0,0,8,37,0,0,9,171,0,0,8,5,0,0,8,133,0,0,8,69,0,0,9,235,0,16,7,8,0,0,8,93,0,0,8,29,0,0,9,155,0,20,7,83,0,0,8,125,0,0,8,61,0,0,9,219,0,18,7,23,0,0,8,109,0,0,8,45,0,0,9,187,0,0,8,13,0,0,8,141,0,0,8,77,0,0,9,251,0,16,7,3,0,0,8,83,0,0,8,19,0,21,8,195,0,19,7,35,0,0,8,115,0,0,8,51,0,0,9,199,0,17,7,11,0,0,8,99,0,0,8,35,0,0,9,167,0,0,8,3,0,0,8,131,0,0,8,67,0,0,9,231,0,16,7,7,0,0,8,91,0,0,8,27,0,0,9,151,0,20,7,67,0,0,8,123,0,0,8,59,0,0,9,215,0,18,7,19,0,0,8,107,0,0,8,43,0,0,9,183,0,0,8,11,0,0,8,139,0,0,8,75,0,0,9,247,0,16,7,5,0,0,8,87,0,0,8,23,0,64,8,0,0,19,7,51,0,0,8,119,0,0,8,55,0,0,9,207,0,17,7,15,0,0,8,103,0,0,8,39,0,0,9,175,0,0,8,7,0,0,8,135,0,0,8,71,0,0,9,239,0,16,7,9,0,0,8,95,0,0,8,31,0,0,9,159,0,20,7,99,0,0,8,127,0,0,8,63,0,0,9,223,0,18,7,27,0,0,8,111,0,0,8,47,0,0,9,191,0,0,8,15,0,0,8,143,0,0,8,79,0,0,9,255,0,16,5,1,0,23,5,1,1,19,5,17,0,27,5,1,16,17,5,5,0,25,5,1,4,21,5,65,0,29,5,1,64,16,5,3,0,24,5,1,2,20,5,33,0,28,5,1,32,18,5,9,0,26,5,1,8,22,5,129,0,64,5,0,0,16,5,2,0,23,5,129,1,19,5,25,0,27,5,1,24,17,5,7,0,25,5,1,6,21,5,97,0,29,5,1,96,16,5,4,0,24,5,1,3,20,5,49,0,28,5,1,48,18,5,13,0,26,5,1,12,22,5,193,0,64,5,0,0,3,0,4,0,5,0,6,0,7,0,8,0,9,0,10,0,11,0,13,0,15,0,17,0,19,0,23,0,27,0,31,0,35,0,43,0,51,0,59,0,67,0,83,0,99,0,115,0,131,0,163,0,195,0,227,0,2,1,0,0,0,0,0,0,16,0,16,0,16,0,16,0,16,0,16,0,16,0,16,0,17,0,17,0,17,0,17,0,18,0,18,0,18,0,18,0,19,0,19,0,19,0,19,0,20,0,20,0,20,0,20,0,21,0,21,0,21,0,21,0,16,0,201,0,196,0,0,0,1,0,2,0,3,0,4,0,5,0,7,0,9,0,13,0,17,0,25,0,33,0,49,0,65,0,97,0,129,0,193,0,1,1,129,1,1,2,1,3,1,4,1,6,1,8,1,12,1,16,1,24,1,32,1,48,1,64,1,96,0,0,0,0,16,0,16,0,16,0,16,0,17,0,17,0,18,0,18,0,19,0,19,0,20,0,20,0,21,0,21,0,22,0,22,0,23,0,23,0,24,0,24,0,25,0,25,0,26,0,26,0,27,0,27,0,28,0,28,0,29,0,29,0,64,0,64,0,105,110,118,97,108,105,100,32,100,105,115,116,97,110,99,101,32,116,111,111,32,102,97,114,32,98,97,99,107,0,0,0,105,110,118,97,108,105,100,32,100,105,115,116,97,110,99,101,32,99,111,100,101,0,0,0,105,110,118,97,108,105,100,32,108,105,116,101,114,97,108,47,108,101,110,103,116,104,32,99,111,100,101,0,0,0,0,0,2,0,0,192,3,0,0,192,4,0,0,192,5,0,0,192,6,0,0,192,7,0,0,192,8,0,0,192,9,0,0,192,10,0,0,192,11,0,0,192,12,0,0,192,13,0,0,192,14,0,0,192,15,0,0,192,16,0,0,192,17,0,0,192,18,0,0,192,19,0,0,192,20,0,0,192,21,0,0,192,22,0,0,192,23,0,0,192,24,0,0,192,25,0,0,192,26,0,0,192,27,0,0,192,28,0,0,192,29,0,0,192,30,0,0,192,31,0,0,192,0,0,0,179,1,0,0,195,2,0,0,195,3,0,0,195,4,0,0,195,5,0,0,195,6,0,0,195,7,0,0,195,8,0,0,195,9,0,0,195,10,0,0,195,11,0,0,195,12,0,0,195,13,0,0,211,14,0,0,195,15,0,0,195,0,0,12,187,1,0,12,195,2,0,12,195,3,0,12,195,4,0,12,211,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,1,2,3,4,5,6,7,8,9,255,255,255,255,255,255,255,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,255,255,255,255,255,255,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,1,2,4,7,3,6,5,0,0,0,0,0,0,0,0,105,110,102,105,110,105,116,121,0,0,0,0,0,0,0,0,110,97,110,0,0,0,0,0,95,112,137,0,255,9,47,15,10,0,0,0,100,0,0,0,232,3,0,0,16,39,0,0,160,134,1,0,64,66,15,0,128,150,152,0,0,225,245,5,17,0,10,0,17,17,17,0,0,0,0,5,0,0,0,0,0,0,9,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,15,10,17,17,17,3,10,7,0,1,19,9,11,11,0,0,9,6,11,0,0,11,0,6,17,0,0,0,17,17,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,10,10,17,17,17,0,10,0,0,2,0,9,11,0,0,0,9,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,4,13,0,0,0,0,9,14,0,0,0,0,0,14,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,15,0,0,0,0,9,16,0,0,0,0,0,16,0,0,16,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,10,0,0,0,0,9,11,0,0,0,0,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,45,43,32,32,32,48,88,48,120,0,0,0,0,0,0,0,40,110,117,108,108,41,0,0,45,48,88,43,48,88,32,48,88,45,48,120,43,48,120,32,48,120,0,0,0,0,0,0,105,110,102,0,0,0,0,0,73,78,70,0,0,0,0,0,110,97,110,0,0,0,0,0,78,65,78,0,0,0,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);




var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}


  
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            continue;
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          if (stream.tty.output.length) {
            stream.tty.ops.put_char(stream.tty, 10);
          }
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            },
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (node.contents && node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
  
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
  
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              assert(position === 0, 'canOwn must imply no weird position inside the file');
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          stream.ungotten = [];
          stream.position = position;
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          fileStore.createIndex('timestamp', 'timestamp', { unique: false });
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function() {
          callback(this.error);
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function() { callback(this.error); };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function() { callback(this.error); };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function() { done(this.error); };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          stream.position = position;
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
      // we don't currently perform any user-space buffering of data
    }var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); },
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); },
            },
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        return FS.nodePermissions(dir, 'x');
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },getStreamFromPtr:function (ptr) {
        return FS.streams[ptr - 1];
      },getPtrForStream:function (stream) {
        return stream ? stream.fd + 1 : 0;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions
        var err = FS.mayOpen(node, flags);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        return stream.stream_ops.llseek(stream, offset, whence);
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = '';
          var utf8 = new Runtime.UTF8Processor();
          for (var i = 0; i < length; i++) {
            ret += utf8.processCChar(buf[i]);
          }
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var utf8 = new Runtime.UTF8Processor();
          var buf = new Uint8Array(utf8.processJSString(data));
          FS.write(stream, buf, 0, buf.length, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return Math.floor(Math.random()*256); };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=FS.getPtrForStream(stdin);
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=FS.getPtrForStream(stdout);
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=FS.getPtrForStream(stderr);
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno) {
          this.errno = errno;
          for (var key in ERRNO_CODES) {
            if (ERRNO_CODES[key] === errno) {
              this.code = key;
              break;
            }
          }
          this.message = ERRNO_MESSAGES[errno];
          if (this.stack) this.stack = demangleAll(this.stack);
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = Math.floor(idx / this.chunkSize);
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
            // Find length
            var xhr = new XMLHttpRequest();
            xhr.open('HEAD', url, false);
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            var datalength = Number(xhr.getResponseHeader("Content-length"));
            var header;
            var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
            var chunkSize = 1024*1024; // Chunk size in bytes
  
            if (!hasByteServing) chunkSize = datalength;
  
            // Function to get a range from the remote URL.
            var doXHR = (function(from, to) {
              if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
              if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
              // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
              var xhr = new XMLHttpRequest();
              xhr.open('GET', url, false);
              if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
              // Some hints to the browser that we want binary data.
              if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
              if (xhr.overrideMimeType) {
                xhr.overrideMimeType('text/plain; charset=x-user-defined');
              }
  
              xhr.send(null);
              if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
              if (xhr.response !== undefined) {
                return new Uint8Array(xhr.response || []);
              } else {
                return intArrayFromString(xhr.responseText || '', true);
              }
            });
            var lazyArray = this;
            lazyArray.setDataGetter(function(chunkNum) {
              var start = chunkNum * chunkSize;
              var end = (chunkNum+1) * chunkSize - 1; // including this byte
              end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
              if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
                lazyArray.chunks[chunkNum] = doXHR(start, end);
              }
              if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
              return lazyArray.chunks[chunkNum];
            });
  
            this._length = datalength;
            this._chunkSize = chunkSize;
            this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};
  
  function _lseek(fildes, offset, whence) {
      // off_t lseek(int fildes, off_t offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lseek.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        return FS.llseek(stream, offset, whence);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _fileno(stream) {
      // int fileno(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fileno.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) return -1;
      return stream.fd;
    }function _fseek(stream, offset, whence) {
      // int fseek(FILE *stream, long offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fseek.html
      var fd = _fileno(stream);
      var ret = _lseek(fd, offset, whence);
      if (ret == -1) {
        return -1;
      }
      stream = FS.getStreamFromPtr(stream);
      stream.eof = false;
      return 0;
    }

   
  Module["_i64Subtract"] = _i64Subtract;

  function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 79:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  
  
  
  
  
  function _mkport() { throw 'TODO' }var SOCKFS={mount:function (mount) {
        return FS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              // runtimeConfig gets set to true if WebSocket runtime configuration is available.
              var runtimeConfig = (Module['websocket'] && ('object' === typeof Module['websocket']));
  
              // The default value is 'ws://' the replace is needed because the compiler replaces "//" comments with '#'
              // comments without checking context, so we'd end up with ws:#, the replace swaps the "#" for "//" again.
              var url = 'ws:#'.replace('#', '//');
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['url']) {
                  url = Module['websocket']['url']; // Fetch runtime WebSocket URL config.
                }
              }
  
              if (url === 'ws://' || url === 'wss://') { // Is the supplied URL config just a prefix, if so complete it.
                url = url + addr + ':' + port;
              }
  
              // Make the WebSocket subprotocol (Sec-WebSocket-Protocol) default to binary if no configuration is set.
              var subProtocols = 'binary'; // The default value is 'binary'
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['subprotocol']) {
                  subProtocols = Module['websocket']['subprotocol']; // Fetch runtime WebSocket subprotocol config.
                }
              }
  
              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
  
              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
              var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
  
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('error', function() {
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODS.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
            }
          });
          sock.server.on('closed', function() {
            sock.server = null;
          });
          sock.server.on('error', function() {
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _recv(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fread(ptr, size, nitems, stream) {
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) {
        return 0;
      }
      var bytesRead = 0;
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      while (streamObj.ungotten.length && bytesToRead > 0) {
        HEAP8[((ptr++)>>0)]=streamObj.ungotten.pop();
        bytesToRead--;
        bytesRead++;
      }
      var err = _read(streamObj.fd, ptr, bytesToRead);
      if (err == -1) {
        if (streamObj) streamObj.error = true;
        return 0;
      }
      bytesRead += err;
      if (bytesRead < bytesToRead) streamObj.eof = true;
      return Math.floor(bytesRead / size);
    }function _fgetc(stream) {
      // int fgetc(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetc.html
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) return -1;
      if (streamObj.eof || streamObj.error) return -1;
      var ret = _fread(_fgetc.ret, 1, 1, stream);
      if (ret == 0) {
        return -1;
      } else if (ret == -1) {
        streamObj.error = true;
        return -1;
      } else {
        return HEAPU8[((_fgetc.ret)>>0)];
      }
    }

  function ___errno_location() {
      return ___errno_state;
    }

   
  Module["_memset"] = _memset;

  
  function _close(fildes) {
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.close(stream);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _fsync(fildes) {
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }function _fclose(stream) {
      // int fclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fclose.html
      var fd = _fileno(stream);
      _fsync(fd);
      return _close(fd);
    }

  
   
  Module["_strlen"] = _strlen; 
  Module["_strcat"] = _strcat;

  
  
  function _malloc(bytes) {
      /* Over-allocate to make sure it is byte-aligned by 8.
       * This will leak memory, but this is only the dummy
       * implementation (replaced by dlmalloc normally) so
       * not an issue.
       */
      var ptr = Runtime.dynamicAlloc(bytes + 8);
      return (ptr+8) & 0xFFFFFFF8;
    }
  Module["_malloc"] = _malloc;
  
  var ___tm_current=allocate(44, "i8", ALLOC_STATIC);
  
  
  var ___tm_timezone=allocate(intArrayFromString("GMT"), "i8", ALLOC_STATIC);
  
  
  var _tzname=allocate(8, "i32*", ALLOC_STATIC);
  
  var _daylight=allocate(1, "i32*", ALLOC_STATIC);
  
  var _timezone=allocate(1, "i32*", ALLOC_STATIC);function _tzset() {
      // TODO: Use (malleable) environment variables instead of system settings.
      if (_tzset.called) return;
      _tzset.called = true;
  
      HEAP32[((_timezone)>>2)]=-(new Date()).getTimezoneOffset() * 60;
  
      var winter = new Date(2000, 0, 1);
      var summer = new Date(2000, 6, 1);
      HEAP32[((_daylight)>>2)]=Number(winter.getTimezoneOffset() != summer.getTimezoneOffset());
  
      var winterName = 'GMT'; // XXX do not rely on browser timezone info, it is very unpredictable | winter.toString().match(/\(([A-Z]+)\)/)[1];
      var summerName = 'GMT'; // XXX do not rely on browser timezone info, it is very unpredictable | summer.toString().match(/\(([A-Z]+)\)/)[1];
      var winterNamePtr = allocate(intArrayFromString(winterName), 'i8', ALLOC_NORMAL);
      var summerNamePtr = allocate(intArrayFromString(summerName), 'i8', ALLOC_NORMAL);
      HEAP32[((_tzname)>>2)]=winterNamePtr;
      HEAP32[(((_tzname)+(4))>>2)]=summerNamePtr;
    }function _localtime_r(time, tmPtr) {
      _tzset();
      var date = new Date(HEAP32[((time)>>2)]*1000);
      HEAP32[((tmPtr)>>2)]=date.getSeconds();
      HEAP32[(((tmPtr)+(4))>>2)]=date.getMinutes();
      HEAP32[(((tmPtr)+(8))>>2)]=date.getHours();
      HEAP32[(((tmPtr)+(12))>>2)]=date.getDate();
      HEAP32[(((tmPtr)+(16))>>2)]=date.getMonth();
      HEAP32[(((tmPtr)+(20))>>2)]=date.getFullYear()-1900;
      HEAP32[(((tmPtr)+(24))>>2)]=date.getDay();
  
      var start = new Date(date.getFullYear(), 0, 1);
      var yday = Math.floor((date.getTime() - start.getTime()) / (1000 * 60 * 60 * 24));
      HEAP32[(((tmPtr)+(28))>>2)]=yday;
      HEAP32[(((tmPtr)+(36))>>2)]=start.getTimezoneOffset() * 60;
  
      var dst = Number(start.getTimezoneOffset() != date.getTimezoneOffset());
      HEAP32[(((tmPtr)+(32))>>2)]=dst;
  
      HEAP32[(((tmPtr)+(40))>>2)]=___tm_timezone;
  
      return tmPtr;
    }function _localtime(time) {
      return _localtime_r(time, ___tm_current);
    }
  
  
  var ___tm_formatted=allocate(44, "i8", ALLOC_STATIC);
  
  
  function _mktime(tmPtr) {
      _tzset();
      var year = HEAP32[(((tmPtr)+(20))>>2)];
      var timestamp = new Date(year >= 1900 ? year : year + 1900,
                               HEAP32[(((tmPtr)+(16))>>2)],
                               HEAP32[(((tmPtr)+(12))>>2)],
                               HEAP32[(((tmPtr)+(8))>>2)],
                               HEAP32[(((tmPtr)+(4))>>2)],
                               HEAP32[((tmPtr)>>2)],
                               0).getTime() / 1000;
      HEAP32[(((tmPtr)+(24))>>2)]=new Date(timestamp).getDay();
      var yday = Math.round((timestamp - (new Date(year, 0, 1)).getTime()) / (1000 * 60 * 60 * 24));
      HEAP32[(((tmPtr)+(28))>>2)]=yday;
      return timestamp;
    }function _asctime_r(tmPtr, buf) {
      var date = new Date(_mktime(tmPtr)*1000);
      var formatted = date.toString();
      var datePart = formatted.replace(/\d{4}.*/, '').replace(/ 0/, '  ');
      var timePart = formatted.match(/\d{2}:\d{2}:\d{2}/)[0];
      formatted = datePart + timePart + ' ' + date.getFullYear() + '\n';
      formatted.split('').forEach(function(chr, index) {
        HEAP8[(((buf)+(index))>>0)]=chr.charCodeAt(0);
      });
      HEAP8[(((buf)+(25))>>0)]=0;
      return buf;
    }function _asctime(tmPtr) {
      return _asctime_r(tmPtr, ___tm_formatted);
    }function _ctime(timer) {
      return _asctime(_localtime(timer));
    }

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  
  
  
  function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fwrite(ptr, size, nitems, stream) {
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var fd = _fileno(stream);
      var bytesWritten = _write(fd, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return 0;
      } else {
        return Math.floor(bytesWritten / size);
      }
    }
  
  
  function __reallyNegative(x) {
      return x < 0 || (x === 0 && (1/x) === -Infinity);
    }function __formatString(format, varargs) {
      var textIndex = format;
      var argIndex = 0;
      function getNextArg(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        if (type === 'double') {
          ret = (HEAP32[((tempDoublePtr)>>2)]=HEAP32[(((varargs)+(argIndex))>>2)],HEAP32[(((tempDoublePtr)+(4))>>2)]=HEAP32[(((varargs)+((argIndex)+(4)))>>2)],(+(HEAPF64[(tempDoublePtr)>>3])));
        } else if (type == 'i64') {
          ret = [HEAP32[(((varargs)+(argIndex))>>2)],
                 HEAP32[(((varargs)+(argIndex+4))>>2)]];
  
        } else {
          type = 'i32'; // varargs are always i32, i64, or double
          ret = HEAP32[(((varargs)+(argIndex))>>2)];
        }
        argIndex += Runtime.getNativeFieldSize(type);
        return ret;
      }
  
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[((textIndex)>>0)];
        if (curr === 0) break;
        next = HEAP8[((textIndex+1)>>0)];
        if (curr == 37) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          var flagPadSign = false;
          flagsLoop: while (1) {
            switch (next) {
              case 43:
                flagAlwaysSigned = true;
                break;
              case 45:
                flagLeftAlign = true;
                break;
              case 35:
                flagAlternative = true;
                break;
              case 48:
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              case 32:
                flagPadSign = true;
                break;
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          }
  
          // Handle width.
          var width = 0;
          if (next == 42) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          } else {
            while (next >= 48 && next <= 57) {
              width = width * 10 + (next - 48);
              textIndex++;
              next = HEAP8[((textIndex+1)>>0)];
            }
          }
  
          // Handle precision.
          var precisionSet = false, precision = -1;
          if (next == 46) {
            precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
            if (next == 42) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[((textIndex+1)>>0)];
                if (precisionChr < 48 ||
                    precisionChr > 57) break;
                precision = precision * 10 + (precisionChr - 48);
                textIndex++;
              }
            }
            next = HEAP8[((textIndex+1)>>0)];
          }
          if (precision < 0) {
            precision = 6; // Standard default.
            precisionSet = false;
          }
  
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 104) {
                textIndex++;
                argSize = 1; // char (actually i32 in varargs)
              } else {
                argSize = 2; // short (actually i32 in varargs)
              }
              break;
            case 'l':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 108) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = null;
          }
          if (argSize) textIndex++;
          next = HEAP8[((textIndex+1)>>0)];
  
          // Handle type specifier.
          switch (String.fromCharCode(next)) {
            case 'd': case 'i': case 'u': case 'o': case 'x': case 'X': case 'p': {
              // Integer.
              var signed = next == 100 || next == 105;
              argSize = argSize || 4;
              var currArg = getNextArg('i' + (argSize * 8));
              var origArg = currArg;
              var argText;
              // Flatten i64-1 [low, high] into a (slightly rounded) double
              if (argSize == 8) {
                currArg = Runtime.makeBigInt(currArg[0], currArg[1], next == 117);
              }
              // Truncate to requested size.
              if (argSize <= 4) {
                var limit = Math.pow(256, argSize) - 1;
                currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
              }
              // Format the number.
              var currAbsArg = Math.abs(currArg);
              var prefix = '';
              if (next == 100 || next == 105) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], null); else
                argText = reSign(currArg, 8 * argSize, 1).toString(10);
              } else if (next == 117) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], true); else
                argText = unSign(currArg, 8 * argSize, 1).toString(10);
                currArg = Math.abs(currArg);
              } else if (next == 111) {
                argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
              } else if (next == 120 || next == 88) {
                prefix = (flagAlternative && currArg != 0) ? '0x' : '';
                if (argSize == 8 && i64Math) {
                  if (origArg[1]) {
                    argText = (origArg[1]>>>0).toString(16);
                    var lower = (origArg[0]>>>0).toString(16);
                    while (lower.length < 8) lower = '0' + lower;
                    argText += lower;
                  } else {
                    argText = (origArg[0]>>>0).toString(16);
                  }
                } else
                if (currArg < 0) {
                  // Represent negative numbers in hex as 2's complement.
                  currArg = -currArg;
                  argText = (currAbsArg - 1).toString(16);
                  var buffer = [];
                  for (var i = 0; i < argText.length; i++) {
                    buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                  }
                  argText = buffer.join('');
                  while (argText.length < argSize * 2) argText = 'f' + argText;
                } else {
                  argText = currAbsArg.toString(16);
                }
                if (next == 88) {
                  prefix = prefix.toUpperCase();
                  argText = argText.toUpperCase();
                }
              } else if (next == 112) {
                if (currAbsArg === 0) {
                  argText = '(nil)';
                } else {
                  prefix = '0x';
                  argText = currAbsArg.toString(16);
                }
              }
              if (precisionSet) {
                while (argText.length < precision) {
                  argText = '0' + argText;
                }
              }
  
              // Add sign if needed
              if (currArg >= 0) {
                if (flagAlwaysSigned) {
                  prefix = '+' + prefix;
                } else if (flagPadSign) {
                  prefix = ' ' + prefix;
                }
              }
  
              // Move sign to prefix so we zero-pad after the sign
              if (argText.charAt(0) == '-') {
                prefix = '-' + prefix;
                argText = argText.substr(1);
              }
  
              // Add padding.
              while (prefix.length + argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad) {
                    argText = '0' + argText;
                  } else {
                    prefix = ' ' + prefix;
                  }
                }
              }
  
              // Insert the result into the buffer.
              argText = prefix + argText;
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 'f': case 'F': case 'e': case 'E': case 'g': case 'G': {
              // Float.
              var currArg = getNextArg('double');
              var argText;
              if (isNaN(currArg)) {
                argText = 'nan';
                flagZeroPad = false;
              } else if (!isFinite(currArg)) {
                argText = (currArg < 0 ? '-' : '') + 'inf';
                flagZeroPad = false;
              } else {
                var isGeneral = false;
                var effectivePrecision = Math.min(precision, 20);
  
                // Convert g/G to f/F or e/E, as per:
                // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
                if (next == 103 || next == 71) {
                  isGeneral = true;
                  precision = precision || 1;
                  var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                  if (precision > exponent && exponent >= -4) {
                    next = ((next == 103) ? 'f' : 'F').charCodeAt(0);
                    precision -= exponent + 1;
                  } else {
                    next = ((next == 103) ? 'e' : 'E').charCodeAt(0);
                    precision--;
                  }
                  effectivePrecision = Math.min(precision, 20);
                }
  
                if (next == 101 || next == 69) {
                  argText = currArg.toExponential(effectivePrecision);
                  // Make sure the exponent has at least 2 digits.
                  if (/[eE][-+]\d$/.test(argText)) {
                    argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                  }
                } else if (next == 102 || next == 70) {
                  argText = currArg.toFixed(effectivePrecision);
                  if (currArg === 0 && __reallyNegative(currArg)) {
                    argText = '-' + argText;
                  }
                }
  
                var parts = argText.split('e');
                if (isGeneral && !flagAlternative) {
                  // Discard trailing zeros and periods.
                  while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                         (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                    parts[0] = parts[0].slice(0, -1);
                  }
                } else {
                  // Make sure we have a period in alternative mode.
                  if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                  // Zero pad until required precision.
                  while (precision > effectivePrecision++) parts[0] += '0';
                }
                argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
  
                // Capitalize 'E' if needed.
                if (next == 69) argText = argText.toUpperCase();
  
                // Add sign.
                if (currArg >= 0) {
                  if (flagAlwaysSigned) {
                    argText = '+' + argText;
                  } else if (flagPadSign) {
                    argText = ' ' + argText;
                  }
                }
              }
  
              // Add padding.
              while (argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                    argText = argText[0] + '0' + argText.slice(1);
                  } else {
                    argText = (flagZeroPad ? '0' : ' ') + argText;
                  }
                }
              }
  
              // Adjust case.
              if (next < 97) argText = argText.toUpperCase();
  
              // Insert the result into the buffer.
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 's': {
              // String.
              var arg = getNextArg('i8*');
              var argLength = arg ? _strlen(arg) : '(null)'.length;
              if (precisionSet) argLength = Math.min(argLength, precision);
              if (!flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              if (arg) {
                for (var i = 0; i < argLength; i++) {
                  ret.push(HEAPU8[((arg++)>>0)]);
                }
              } else {
                ret = ret.concat(intArrayFromString('(null)'.substr(0, argLength), true));
              }
              if (flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              break;
            }
            case 'c': {
              // Character.
              if (flagLeftAlign) ret.push(getNextArg('i8'));
              while (--width > 0) {
                ret.push(32);
              }
              if (!flagLeftAlign) ret.push(getNextArg('i8'));
              break;
            }
            case 'n': {
              // Write the length written so far to the next parameter.
              var ptr = getNextArg('i32*');
              HEAP32[((ptr)>>2)]=ret.length;
              break;
            }
            case '%': {
              // Literal percent sign.
              ret.push(curr);
              break;
            }
            default: {
              // Unknown specifiers remain untouched.
              for (var i = startTextIndex; i < textIndex + 2; i++) {
                ret.push(HEAP8[((i)>>0)]);
              }
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }

  function _printf(format, varargs) {
      // int printf(const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var stdout = HEAP32[((_stdout)>>2)];
      return _fprintf(stdout, format, varargs);
    }

  var _emscripten_prep_setjmp=true;


  
  function _open(path, oflag, varargs) {
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      var mode = HEAP32[((varargs)>>2)];
      path = Pointer_stringify(path);
      try {
        var stream = FS.open(path, oflag, mode);
        return stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fopen(filename, mode) {
      // FILE *fopen(const char *restrict filename, const char *restrict mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fopen.html
      var flags;
      mode = Pointer_stringify(mode);
      if (mode[0] == 'r') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 0;
        }
      } else if (mode[0] == 'w') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 512;
      } else if (mode[0] == 'a') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 1024;
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var fd = _open(filename, flags, allocate([0x1FF, 0, 0, 0], 'i32', ALLOC_STACK));  // All creation permissions.
      return fd === -1 ? 0 : FS.getPtrForStream(FS.getStream(fd));
    }


  var _emscripten_check_longjmp=true;

   
  Module["_i64Add"] = _i64Add;

  var _emscripten_postinvoke=true;


  
  function _rint(x) {
      if (Math.abs(x % 1) !== 0.5) return Math.round(x);
      return x + x % 2 + ((x < 0) ? 1 : -1);
    }function _lrintf() {
  return _rint.apply(null, arguments)
  }

  function _sqrt() {
  return Math_sqrt.apply(null, arguments)
  }

  
  
  function _fputc(c, stream) {
      // int fputc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputc.html
      var chr = unSign(c & 0xFF);
      HEAP8[((_fputc.ret)>>0)]=chr;
      var fd = _fileno(stream);
      var ret = _write(fd, _fputc.ret, 1);
      if (ret == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return -1;
      } else {
        return chr;
      }
    }function _putchar(c) {
      // int putchar(int c);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/putchar.html
      return _fputc(c, HEAP32[((_stdout)>>2)]);
    } 
  Module["_saveSetjmp"] = _saveSetjmp;

  function _ftell(stream) {
      // long ftell(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftell.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (FS.isChrdev(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.ESPIPE);
        return -1;
      } else {
        return stream.position;
      }
    }

  function _abs() {
  return Math_abs.apply(null, arguments)
  }

  var Browser={mainLoop:{scheduler:null,method:"",shouldPause:false,paused:false,queue:[],pause:function () {
          Browser.mainLoop.shouldPause = true;
        },resume:function () {
          if (Browser.mainLoop.paused) {
            Browser.mainLoop.paused = false;
            Browser.mainLoop.scheduler();
          }
          Browser.mainLoop.shouldPause = false;
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          assert(typeof url == 'string', 'createObjectURL must return a url as a string');
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            assert(typeof url == 'string', 'createObjectURL must return a url as a string');
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx) return Module.ctx; // no need to recreate singleton GL context
  
        var ctx;
        var errorInfo = '?';
        function onContextCreationError(event) {
          errorInfo = event.statusMessage || errorInfo;
        }
        try {
          if (useWebGL) {
            var contextAttributes = {
              antialias: false,
              alpha: false
            };
  
            if (webGLContextAttributes) {
              for (var attribute in webGLContextAttributes) {
                contextAttributes[attribute] = webGLContextAttributes[attribute];
              }
            }
  
  
            canvas.addEventListener('webglcontextcreationerror', onContextCreationError, false);
            try {
              ['experimental-webgl', 'webgl'].some(function(webglId) {
                return ctx = canvas.getContext(webglId, contextAttributes);
              });
            } finally {
              canvas.removeEventListener('webglcontextcreationerror', onContextCreationError, false);
            }
          } else {
            ctx = canvas.getContext('2d');
          }
          if (!ctx) throw ':(';
        } catch (e) {
          Module.print('Could not create canvas: ' + [errorInfo, e]);
          return null;
        }
        if (useWebGL) {
          // possible GL_DEBUG entry point: ctx = wrapDebugGL(ctx);
  
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        }
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
          Module.ctx = ctx;
          if (useWebGL) GLctx = ctx;
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
        
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
        canvasContainer.requestFullScreen();
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = -event.wheelDelta;
            break;
          case 'wheel': 
            delta = event.deltaY;
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return Math.max(-1, Math.min(1, delta));
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
          // and we have no viable fallback.
          assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              Browser.lastTouches[touch.identifier] = Browser.touches[touch.identifier];
              Browser.touches[touch.identifier] = { x: adjustedX, y: adjustedY };
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      }};

  var _emscripten_get_longjmp_result=true;

  function _time(ptr) {
      var ret = Math.floor(Date.now()/1000);
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }

  function _sin() {
  return Math_sin.apply(null, arguments)
  }

  
  function _fmod(x, y) {
      return x % y;
    }function _fmodl() {
  return _fmod.apply(null, arguments)
  }

  function _stat(path, buf, dontResolveLastLink) {
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/stat.html
      // int stat(const char *path, struct stat *buf);
      // NOTE: dontResolveLastLink is a shortcut for lstat(). It should never be
      //       used in client code.
      path = typeof path !== 'string' ? Pointer_stringify(path) : path;
      try {
        var stat = dontResolveLastLink ? FS.lstat(path) : FS.stat(path);
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode;
        HEAP32[(((buf)+(16))>>2)]=stat.nlink;
        HEAP32[(((buf)+(20))>>2)]=stat.uid;
        HEAP32[(((buf)+(24))>>2)]=stat.gid;
        HEAP32[(((buf)+(28))>>2)]=stat.rdev;
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size;
        HEAP32[(((buf)+(40))>>2)]=4096;
        HEAP32[(((buf)+(44))>>2)]=stat.blocks;
        HEAP32[(((buf)+(48))>>2)]=Math.floor(stat.atime.getTime() / 1000);
        HEAP32[(((buf)+(52))>>2)]=0;
        HEAP32[(((buf)+(56))>>2)]=Math.floor(stat.mtime.getTime() / 1000);
        HEAP32[(((buf)+(60))>>2)]=0;
        HEAP32[(((buf)+(64))>>2)]=Math.floor(stat.ctime.getTime() / 1000);
        HEAP32[(((buf)+(68))>>2)]=0;
        HEAP32[(((buf)+(72))>>2)]=stat.ino;
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;


  
   
  Module["_testSetjmp"] = _testSetjmp;function _longjmp(env, value) {
      asm['setThrew'](env, value || 1);
      throw 'longjmp';
    }

  var _BDtoILow=true;

  function _ferror(stream) {
      // int ferror(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ferror.html
      stream = FS.getStreamFromPtr(stream);
      return Number(stream && stream.error);
    }

  var _BDtoIHigh=true;

  function _ceil() {
  return Math_ceil.apply(null, arguments)
  }

  function _emscripten_longjmp(env, value) {
      _longjmp(env, value);
    }

  
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          writeAsciiToMemory(msg, strerrbuf);
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }

  
  
  
  
  var _environ=allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = 'root';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/emscripten';
        ENV['LANG'] = 'en_US.UTF-8';
        ENV['_'] = './this.program';
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        writeAsciiToMemory(line, poolPtr);
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  function _llvm_pow_f64() {
  return Math_pow.apply(null, arguments)
  }

  var _emscripten_setjmp=true;

  function _cos() {
  return Math_cos.apply(null, arguments)
  }

  function _fabs() {
  return Math_abs.apply(null, arguments)
  }

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }

  var _emscripten_preinvoke=true;

  var _BItoD=true;

  function _getrusage(resource, rlp) {
      // int getrusage(int resource, struct rusage *rlp);
      HEAP32[((rlp)>>2)]=1;
      HEAP32[(((rlp)+(4))>>2)]=2;
      HEAP32[(((rlp)+(8))>>2)]=3;
      HEAP32[(((rlp)+(12))>>2)]=4;
      return 0;
    }

  function _atan2() {
  return Math_atan2.apply(null, arguments)
  }

   
  Module["_strcpy"] = _strcpy;


  
  function _copysign(a, b) {
      return __reallyNegative(a) === __reallyNegative(b) ? a : -a;
    }function _copysignl() {
  return _copysign.apply(null, arguments)
  }


FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });TTY.utf8 = new Runtime.UTF8Processor();
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
_fgetc.ret = allocate([0], "i8", ALLOC_STATIC);
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
_fputc.ret = allocate([0], "i8", ALLOC_STATIC);
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas) { Browser.requestFullScreen(lockPointer, resizeCanvas) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
___buildEnvironment(ENV);
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + 5242880;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var ctlz_i8 = allocate([8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_DYNAMIC);
 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);

  var Math_min = Math.min;
function nullFunc_iiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viiiiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'viiiiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'viiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_vi(x) { Module["printErr"]("Invalid function pointer called with signature 'vi'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_vii(x) { Module["printErr"]("Invalid function pointer called with signature 'vii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iiiiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_ii(x) { Module["printErr"]("Invalid function pointer called with signature 'ii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viii(x) { Module["printErr"]("Invalid function pointer called with signature 'viii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_v(x) { Module["printErr"]("Invalid function pointer called with signature 'v'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iii(x) { Module["printErr"]("Invalid function pointer called with signature 'iii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_iiiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function nullFunc_viiii(x) { Module["printErr"]("Invalid function pointer called with signature 'viiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiiiiii(index,a1,a2,a3,a4,a5,a6,a7) {
  try {
    Module["dynCall_viiiiiii"](index,a1,a2,a3,a4,a5,a6,a7);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiiii(index,a1,a2,a3,a4,a5) {
  try {
    Module["dynCall_viiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vii(index,a1,a2) {
  try {
    Module["dynCall_vii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    return Module["dynCall_iiiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viii(index,a1,a2,a3) {
  try {
    Module["dynCall_viii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiiii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiii(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

  function asmPrintInt(x, y) {
    Module.print('int ' + x + ',' + y);// + ' ' + new Error().stack);
  }
  function asmPrintFloat(x, y) {
    Module.print('float ' + x + ',' + y);// + ' ' + new Error().stack);
  }
  // EMSCRIPTEN_START_ASM
  var asm = (function(global, env, buffer) {
    'almost asm';
    var HEAP8 = new global.Int8Array(buffer);
    var HEAP16 = new global.Int16Array(buffer);
    var HEAP32 = new global.Int32Array(buffer);
    var HEAPU8 = new global.Uint8Array(buffer);
    var HEAPU16 = new global.Uint16Array(buffer);
    var HEAPU32 = new global.Uint32Array(buffer);
    var HEAPF32 = new global.Float32Array(buffer);
    var HEAPF64 = new global.Float64Array(buffer);
  
  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;
  var ctlz_i8=env.ctlz_i8|0;
  var _stderr=env._stderr|0;
  var _stdout=env._stdout|0;

    var __THREW__ = 0;
    var threwValue = 0;
    var setjmpId = 0;
    var undef = 0;
    var nan = +env.NaN, inf = +env.Infinity;
    var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;
  
    var tempRet0 = 0;
    var tempRet1 = 0;
    var tempRet2 = 0;
    var tempRet3 = 0;
    var tempRet4 = 0;
    var tempRet5 = 0;
    var tempRet6 = 0;
    var tempRet7 = 0;
    var tempRet8 = 0;
    var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var abort=env.abort;
  var assert=env.assert;
  var asmPrintInt=env.asmPrintInt;
  var asmPrintFloat=env.asmPrintFloat;
  var Math_min=env.min;
  var nullFunc_iiii=env.nullFunc_iiii;
  var nullFunc_viiiiiii=env.nullFunc_viiiiiii;
  var nullFunc_viiiii=env.nullFunc_viiiii;
  var nullFunc_vi=env.nullFunc_vi;
  var nullFunc_vii=env.nullFunc_vii;
  var nullFunc_iiiiiii=env.nullFunc_iiiiiii;
  var nullFunc_ii=env.nullFunc_ii;
  var nullFunc_viii=env.nullFunc_viii;
  var nullFunc_v=env.nullFunc_v;
  var nullFunc_iiiii=env.nullFunc_iiiii;
  var nullFunc_iii=env.nullFunc_iii;
  var nullFunc_iiiiii=env.nullFunc_iiiiii;
  var nullFunc_viiii=env.nullFunc_viiii;
  var invoke_iiii=env.invoke_iiii;
  var invoke_viiiiiii=env.invoke_viiiiiii;
  var invoke_viiiii=env.invoke_viiiii;
  var invoke_vi=env.invoke_vi;
  var invoke_vii=env.invoke_vii;
  var invoke_iiiiiii=env.invoke_iiiiiii;
  var invoke_ii=env.invoke_ii;
  var invoke_viii=env.invoke_viii;
  var invoke_v=env.invoke_v;
  var invoke_iiiii=env.invoke_iiiii;
  var invoke_iii=env.invoke_iii;
  var invoke_iiiiii=env.invoke_iiiiii;
  var invoke_viiii=env.invoke_viiii;
  var _fabs=env._fabs;
  var _sin=env._sin;
  var _printf=env._printf;
  var _llvm_pow_f64=env._llvm_pow_f64;
  var _send=env._send;
  var _fread=env._fread;
  var _ctime=env._ctime;
  var _fmod=env._fmod;
  var _lseek=env._lseek;
  var _open=env._open;
  var _ceil=env._ceil;
  var _asctime=env._asctime;
  var _fputc=env._fputc;
  var _fsync=env._fsync;
  var ___buildEnvironment=env.___buildEnvironment;
  var _longjmp=env._longjmp;
  var _fflush=env._fflush;
  var _pwrite=env._pwrite;
  var _strerror_r=env._strerror_r;
  var _localtime_r=env._localtime_r;
  var _tzset=env._tzset;
  var __reallyNegative=env.__reallyNegative;
  var _sbrk=env._sbrk;
  var _atan2=env._atan2;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var _fileno=env._fileno;
  var _sysconf=env._sysconf;
  var ___setErrNo=env.___setErrNo;
  var _ferror=env._ferror;
  var _putchar=env._putchar;
  var _cos=env._cos;
  var _fseek=env._fseek;
  var _asctime_r=env._asctime_r;
  var _lrintf=env._lrintf;
  var _pread=env._pread;
  var _getenv=env._getenv;
  var _fclose=env._fclose;
  var _localtime=env._localtime;
  var _write=env._write;
  var _ftell=env._ftell;
  var ___errno_location=env.___errno_location;
  var _stat=env._stat;
  var _recv=env._recv;
  var _copysign=env._copysign;
  var _fgetc=env._fgetc;
  var _mkport=env._mkport;
  var _getrusage=env._getrusage;
  var __exit=env.__exit;
  var _read=env._read;
  var _copysignl=env._copysignl;
  var _abort=env._abort;
  var _fwrite=env._fwrite;
  var _time=env._time;
  var _fprintf=env._fprintf;
  var _strerror=env._strerror;
  var _emscripten_longjmp=env._emscripten_longjmp;
  var _abs=env._abs;
  var __formatString=env.__formatString;
  var _rint=env._rint;
  var _sqrt=env._sqrt;
  var _fopen=env._fopen;
  var _exit=env._exit;
  var _close=env._close;
  var _mktime=env._mktime;
  var _fmodl=env._fmodl;
  var tempFloat = 0.0;

  // EMSCRIPTEN_START_FUNCS
  function stackAlloc(size) {
    size = size|0;
    var ret = 0;
    ret = STACKTOP;
    STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 7)&-8;
    return ret|0;
  }
  function stackSave() {
    return STACKTOP|0;
  }
  function stackRestore(top) {
    top = top|0;
    STACKTOP = top;
  }
  function setThrew(threw, value) {
    threw = threw|0;
    value = value|0;
    if ((__THREW__|0) == 0) {
      __THREW__ = threw;
      threwValue = value;
    }
  }
  function copyTempFloat(ptr) {
    ptr = ptr|0;
    HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
    HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
    HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
    HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  }
  function copyTempDouble(ptr) {
    ptr = ptr|0;
    HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
    HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
    HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
    HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
    HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
    HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
    HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
    HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
  }
  function setTempRet0(value) {
    value = value|0;
    tempRet0 = value;
  }
  function getTempRet0() {
    return tempRet0|0;
  }
  
function _main($argc,$argv) {
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0.0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ansi_nist = 0, $bd = 0, $bdata = 0, $bh = 0, $boostflag = 0, $bw = 0, $direction_map = 0, $high_curve_map = 0, $id = 0, $idata = 0, $ifile = 0, $ih = 0, $ilen = 0, $img_idc = 0, $img_imp = 0;
 var $img_type = 0, $imgrecord = 0, $imgrecord_i = 0, $ippi = 0, $ippmm = 0, $iw = 0, $low_contrast_map = 0, $low_flow_map = 0, $m1flag = 0, $map_h = 0, $map_w = 0, $minutiae = 0, $ofile = 0, $oroot = 0, $quality_map = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0;
 var $vararg_ptr10 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 4272|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $boostflag = sp + 152|0;
 $m1flag = sp + 148|0;
 $ifile = sp + 144|0;
 $oroot = sp + 140|0;
 $ofile = sp + 168|0;
 $idata = sp + 136|0;
 $bdata = sp + 132|0;
 $img_type = sp + 128|0;
 $ilen = sp + 124|0;
 $iw = sp + 120|0;
 $ih = sp + 116|0;
 $id = sp + 112|0;
 $ippi = sp + 108|0;
 $bw = sp + 104|0;
 $bh = sp + 100|0;
 $bd = sp + 96|0;
 $ippmm = sp;
 $img_idc = sp + 92|0;
 $img_imp = sp + 88|0;
 $direction_map = sp + 84|0;
 $low_contrast_map = sp + 80|0;
 $low_flow_map = sp + 76|0;
 $high_curve_map = sp + 72|0;
 $quality_map = sp + 68|0;
 $map_w = sp + 64|0;
 $map_h = sp + 60|0;
 $minutiae = sp + 52|0;
 $ansi_nist = sp + 48|0;
 $imgrecord = sp + 44|0;
 $imgrecord_i = sp + 40|0;
 $0 = 0;
 $1 = $argc;
 $2 = $argv;
 $3 = $1;
 $4 = $2;
 _procargs($3,$4,$boostflag,$m1flag,$ifile,$oroot);
 $5 = HEAP32[$ifile>>2]|0;
 $6 = (_is_ANSI_NIST_file($5)|0);
 $ret = $6;
 $7 = ($6|0)<(0);
 if ($7) {
  $8 = $ret;
  _exit(($8|0));
  // unreachable;
 }
 $9 = $ret;
 $10 = ($9|0)!=(0);
 do {
  if ($10) {
   HEAP32[$img_type>>2] = 5;
   $11 = HEAP32[$ifile>>2]|0;
   $12 = (_read_ANSI_NIST_file($11,$ansi_nist)|0);
   $ret = $12;
   $13 = ($12|0)!=(0);
   if ($13) {
    $14 = $ret;
    _exit(($14|0));
    // unreachable;
   }
   $15 = HEAP32[$ansi_nist>>2]|0;
   $16 = (_get_first_grayprint($idata,$iw,$ih,$id,$ippmm,$img_idc,$img_imp,$imgrecord,$imgrecord_i,$15)|0);
   $ret = $16;
   $17 = ($16|0)<(0);
   if ($17) {
    $18 = HEAP32[$ansi_nist>>2]|0;
    _free_ANSI_NIST($18);
    $19 = $ret;
    _exit(($19|0));
    // unreachable;
   }
   $20 = $ret;
   $21 = ($20|0)!=(0);
   if ($21) {
    break;
   } else {
    $22 = HEAP32[$ansi_nist>>2]|0;
    _free_ANSI_NIST($22);
    $23 = HEAP32[_stderr>>2]|0;
    (_fprintf(($23|0),(16|0),($vararg_buffer|0))|0);
    $24 = HEAP32[_stderr>>2]|0;
    $25 = HEAP32[$ifile>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $25;
    (_fprintf(($24|0),(32|0),($vararg_buffer1|0))|0);
    _exit(-2);
    // unreachable;
   }
  } else {
   $26 = HEAP32[$ifile>>2]|0;
   $27 = (_read_and_decode_grayscale_image($26,$img_type,$idata,$ilen,$iw,$ih,$id,$ippi)|0);
   $ret = $27;
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = $ret;
    _exit(($29|0));
    // unreachable;
   }
   $30 = HEAP32[$ippi>>2]|0;
   $31 = ($30|0)==(-1);
   if ($31) {
    HEAPF64[$ippmm>>3] = 19.685039370078740717;
   } else {
    $32 = HEAP32[$ippi>>2]|0;
    $33 = (+($32|0));
    $34 = $33 / 25.3999999999999985789;
    HEAPF64[$ippmm>>3] = $34;
   }
  }
 } while(0);
 $35 = HEAP32[$boostflag>>2]|0;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$idata>>2]|0;
  $38 = HEAP32[$iw>>2]|0;
  $39 = HEAP32[$ih>>2]|0;
  _trim_histtails_contrast_boost($37,$38,$39);
 }
 $40 = HEAP32[$idata>>2]|0;
 $41 = HEAP32[$iw>>2]|0;
 $42 = HEAP32[$ih>>2]|0;
 $43 = HEAP32[$id>>2]|0;
 $44 = +HEAPF64[$ippmm>>3];
 $45 = (_get_minutiae($minutiae,$quality_map,$direction_map,$low_contrast_map,$low_flow_map,$high_curve_map,$map_w,$map_h,$bdata,$bw,$bh,$bd,$40,$41,$42,$43,$44,568)|0);
 $ret = $45;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = HEAP32[$img_type>>2]|0;
  $48 = ($47|0)==(5);
  if (!($48)) {
   $50 = HEAP32[$idata>>2]|0;
   _free($50);
   $51 = $ret;
   _exit(($51|0));
   // unreachable;
  }
  $49 = HEAP32[$ansi_nist>>2]|0;
  _free_ANSI_NIST($49);
  $50 = HEAP32[$idata>>2]|0;
  _free($50);
  $51 = $ret;
  _exit(($51|0));
  // unreachable;
 }
 $52 = HEAP32[$idata>>2]|0;
 _free($52);
 $53 = HEAP32[$oroot>>2]|0;
 $54 = HEAP32[$m1flag>>2]|0;
 $55 = HEAP32[$bw>>2]|0;
 $56 = HEAP32[$bh>>2]|0;
 $57 = HEAP32[$minutiae>>2]|0;
 $58 = HEAP32[$quality_map>>2]|0;
 $59 = HEAP32[$direction_map>>2]|0;
 $60 = HEAP32[$low_contrast_map>>2]|0;
 $61 = HEAP32[$low_flow_map>>2]|0;
 $62 = HEAP32[$high_curve_map>>2]|0;
 $63 = HEAP32[$map_w>>2]|0;
 $64 = HEAP32[$map_h>>2]|0;
 $65 = (_write_text_results($53,$54,$55,$56,$57,$58,$59,$60,$61,$62,$63,$64)|0);
 $ret = $65;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$img_type>>2]|0;
  $68 = ($67|0)==(5);
  if (!($68)) {
   $70 = HEAP32[$minutiae>>2]|0;
   _free_minutiae($70);
   $71 = HEAP32[$quality_map>>2]|0;
   _free($71);
   $72 = HEAP32[$direction_map>>2]|0;
   _free($72);
   $73 = HEAP32[$low_contrast_map>>2]|0;
   _free($73);
   $74 = HEAP32[$low_flow_map>>2]|0;
   _free($74);
   $75 = HEAP32[$high_curve_map>>2]|0;
   _free($75);
   $76 = HEAP32[$bdata>>2]|0;
   _free($76);
   $77 = $ret;
   _exit(($77|0));
   // unreachable;
  }
  $69 = HEAP32[$ansi_nist>>2]|0;
  _free_ANSI_NIST($69);
  $70 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($70);
  $71 = HEAP32[$quality_map>>2]|0;
  _free($71);
  $72 = HEAP32[$direction_map>>2]|0;
  _free($72);
  $73 = HEAP32[$low_contrast_map>>2]|0;
  _free($73);
  $74 = HEAP32[$low_flow_map>>2]|0;
  _free($74);
  $75 = HEAP32[$high_curve_map>>2]|0;
  _free($75);
  $76 = HEAP32[$bdata>>2]|0;
  _free($76);
  $77 = $ret;
  _exit(($77|0));
  // unreachable;
 }
 $78 = HEAP32[$quality_map>>2]|0;
 _free($78);
 $79 = HEAP32[$direction_map>>2]|0;
 _free($79);
 $80 = HEAP32[$low_contrast_map>>2]|0;
 _free($80);
 $81 = HEAP32[$low_flow_map>>2]|0;
 _free($81);
 $82 = HEAP32[$high_curve_map>>2]|0;
 _free($82);
 $83 = HEAP32[$img_type>>2]|0;
 $84 = ($83|0)==(5);
 if (!($84)) {
  $113 = HEAP32[$oroot>>2]|0;
  HEAP32[$vararg_buffer7>>2] = $113;
  $vararg_ptr10 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr10>>2] = 88;
  (_sprintf($ofile,72,$vararg_buffer7)|0);
  $114 = HEAP32[$bdata>>2]|0;
  $115 = HEAP32[$bw>>2]|0;
  $116 = HEAP32[$bh>>2]|0;
  $117 = Math_imul($115, $116)|0;
  $118 = (_write_raw_from_memsize($ofile,$114,$117)|0);
  $ret = $118;
  $119 = ($118|0)!=(0);
  if ($119) {
   $120 = HEAP32[$minutiae>>2]|0;
   _free_minutiae($120);
   $121 = HEAP32[$bdata>>2]|0;
   _free($121);
   $122 = $ret;
   _exit(($122|0));
   // unreachable;
  }
  $123 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($123);
  $124 = HEAP32[$bdata>>2]|0;
  _free($124);
  _exit(0);
  // unreachable;
 }
 $85 = HEAP32[$ansi_nist>>2]|0;
 $86 = HEAP32[$minutiae>>2]|0;
 $87 = HEAP32[$bdata>>2]|0;
 $88 = HEAP32[$bw>>2]|0;
 $89 = HEAP32[$bh>>2]|0;
 $90 = HEAP32[$bd>>2]|0;
 $91 = +HEAPF64[$ippmm>>3];
 $92 = HEAP32[$img_idc>>2]|0;
 $93 = HEAP32[$img_imp>>2]|0;
 $94 = (_update_ANSI_NIST_lfs_results($85,$86,$87,$88,$89,$90,$91,$92,$93)|0);
 $ret = $94;
 $95 = ($94|0)!=(0);
 if ($95) {
  $96 = HEAP32[$img_type>>2]|0;
  $97 = ($96|0)==(5);
  if (!($97)) {
   $99 = HEAP32[$minutiae>>2]|0;
   _free_minutiae($99);
   $100 = HEAP32[$bdata>>2]|0;
   _free($100);
   $101 = $ret;
   _exit(($101|0));
   // unreachable;
  }
  $98 = HEAP32[$ansi_nist>>2]|0;
  _free_ANSI_NIST($98);
  $99 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($99);
  $100 = HEAP32[$bdata>>2]|0;
  _free($100);
  $101 = $ret;
  _exit(($101|0));
  // unreachable;
 }
 $102 = HEAP32[$oroot>>2]|0;
 HEAP32[$vararg_buffer3>>2] = $102;
 $vararg_ptr6 = (($vararg_buffer3) + 4|0);
 HEAP32[$vararg_ptr6>>2] = 80;
 (_sprintf($ofile,72,$vararg_buffer3)|0);
 $103 = HEAP32[$ansi_nist>>2]|0;
 $104 = (_write_ANSI_NIST_file($ofile,$103)|0);
 $ret = $104;
 $105 = ($104|0)!=(0);
 if (!($105)) {
  $112 = HEAP32[$ansi_nist>>2]|0;
  _free_ANSI_NIST($112);
  $123 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($123);
  $124 = HEAP32[$bdata>>2]|0;
  _free($124);
  _exit(0);
  // unreachable;
 }
 $106 = HEAP32[$img_type>>2]|0;
 $107 = ($106|0)==(5);
 if (!($107)) {
  $109 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($109);
  $110 = HEAP32[$bdata>>2]|0;
  _free($110);
  $111 = $ret;
  _exit(($111|0));
  // unreachable;
 }
 $108 = HEAP32[$ansi_nist>>2]|0;
 _free_ANSI_NIST($108);
 $109 = HEAP32[$minutiae>>2]|0;
 _free_minutiae($109);
 $110 = HEAP32[$bdata>>2]|0;
 _free($110);
 $111 = $ret;
 _exit(($111|0));
 // unreachable;
 return 0|0;
}
function _procargs($argc,$argv,$boostflag,$m1flag,$ifile,$oroot) {
 $argc = $argc|0;
 $argv = $argv|0;
 $boostflag = $boostflag|0;
 $m1flag = $m1flag|0;
 $ifile = $ifile|0;
 $oroot = $oroot|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $a = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0;
 var $vararg_buffer14 = 0, $vararg_buffer17 = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer17 = sp + 56|0;
 $vararg_buffer14 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $argc;
 $1 = $argv;
 $2 = $boostflag;
 $3 = $m1flag;
 $4 = $ifile;
 $5 = $oroot;
 $6 = $2;
 HEAP32[$6>>2] = 0;
 $7 = $3;
 HEAP32[$7>>2] = 0;
 $8 = $0;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = $1;
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_strcmp($12,96)|0);
  $14 = ($13|0)==(0);
  if ($14) {
   HEAP32[$vararg_buffer>>2] = 184;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 208;
   (_printf((112|0),($vararg_buffer|0))|0);
   _exit(0);
   // unreachable;
  }
 }
 $15 = $0;
 $16 = ($15|0)==(3);
 if ($16) {
  $17 = $1;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $4;
  HEAP32[$20>>2] = $19;
  $21 = $1;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = $5;
  HEAP32[$24>>2] = $23;
  STACKTOP = sp;return;
 }
 $25 = $0;
 $26 = ($25|0)==(4);
 if (!($26)) {
  $27 = $0;
  $28 = ($27|0)==(5);
  if (!($28)) {
   $59 = HEAP32[_stderr>>2]|0;
   (_fprintf(($59|0),(416|0),($vararg_buffer12|0))|0);
   $60 = HEAP32[_stderr>>2]|0;
   $61 = $1;
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$vararg_buffer14>>2] = $62;
   (_fprintf(($60|0),(264|0),($vararg_buffer14|0))|0);
   $63 = HEAP32[_stderr>>2]|0;
   (_fprintf(($63|0),(312|0),($vararg_buffer17|0))|0);
   $64 = HEAP32[_stderr>>2]|0;
   (_fprintf(($64|0),(352|0),($vararg_buffer19|0))|0);
   _exit(2);
   // unreachable;
  }
 }
 $a = 1;
 while(1) {
  $29 = $a;
  $30 = $0;
  $31 = (($30) - 2)|0;
  $32 = ($29|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = $a;
  $34 = $1;
  $35 = (($34) + ($33<<2)|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (_strcmp($36,224)|0);
  $38 = ($37|0)==(0);
  if ($38) {
   $39 = $2;
   HEAP32[$39>>2] = 1;
  } else {
   $40 = $a;
   $41 = $1;
   $42 = (($41) + ($40<<2)|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_strcmp($43,232)|0);
   $45 = ($44|0)==(0);
   if (!($45)) {
    label = 14;
    break;
   }
   $46 = $3;
   HEAP32[$46>>2] = 1;
  }
  $57 = $a;
  $58 = (($57) + 1)|0;
  $a = $58;
 }
 if ((label|0) == 14) {
  $47 = HEAP32[_stderr>>2]|0;
  $48 = $a;
  $49 = $1;
  $50 = (($49) + ($48<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $51;
  (_fprintf(($47|0),(240|0),($vararg_buffer2|0))|0);
  $52 = HEAP32[_stderr>>2]|0;
  $53 = $1;
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$vararg_buffer5>>2] = $54;
  (_fprintf(($52|0),(264|0),($vararg_buffer5|0))|0);
  $55 = HEAP32[_stderr>>2]|0;
  (_fprintf(($55|0),(312|0),($vararg_buffer8|0))|0);
  $56 = HEAP32[_stderr>>2]|0;
  (_fprintf(($56|0),(352|0),($vararg_buffer10|0))|0);
  _exit(1);
  // unreachable;
 }
 $65 = $a;
 $66 = (($65) + 1)|0;
 $a = $66;
 $67 = $1;
 $68 = (($67) + ($65<<2)|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $4;
 HEAP32[$70>>2] = $69;
 $71 = $a;
 $72 = $1;
 $73 = (($72) + ($71<<2)|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = $5;
 HEAP32[$75>>2] = $74;
 STACKTOP = sp;return;
}
function _get_minutiae($ominutiae,$oquality_map,$odirection_map,$olow_contrast_map,$olow_flow_map,$ohigh_curve_map,$omap_w,$omap_h,$obdata,$obw,$obh,$obd,$idata,$iw,$ih,$id,$ppmm,$lfsparms) {
 $ominutiae = $ominutiae|0;
 $oquality_map = $oquality_map|0;
 $odirection_map = $odirection_map|0;
 $olow_contrast_map = $olow_contrast_map|0;
 $olow_flow_map = $olow_flow_map|0;
 $ohigh_curve_map = $ohigh_curve_map|0;
 $omap_w = $omap_w|0;
 $omap_h = $omap_h|0;
 $obdata = $obdata|0;
 $obw = $obw|0;
 $obh = $obh|0;
 $obd = $obd|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $id = $id|0;
 $ppmm = +$ppmm;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $bdata = 0, $bh = 0, $bw = 0, $direction_map = 0, $high_curve_map = 0, $low_contrast_map = 0, $low_flow_map = 0;
 var $map_h = 0, $map_w = 0, $minutiae = 0, $quality_map = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $minutiae = sp + 60|0;
 $direction_map = sp + 56|0;
 $low_contrast_map = sp + 52|0;
 $low_flow_map = sp + 48|0;
 $high_curve_map = sp + 44|0;
 $quality_map = sp + 40|0;
 $map_w = sp + 36|0;
 $map_h = sp + 32|0;
 $bdata = sp + 28|0;
 $bw = sp + 24|0;
 $bh = sp + 20|0;
 $1 = $ominutiae;
 $2 = $oquality_map;
 $3 = $odirection_map;
 $4 = $olow_contrast_map;
 $5 = $olow_flow_map;
 $6 = $ohigh_curve_map;
 $7 = $omap_w;
 $8 = $omap_h;
 $9 = $obdata;
 $10 = $obw;
 $11 = $obh;
 $12 = $obd;
 $13 = $idata;
 $14 = $iw;
 $15 = $ih;
 $16 = $id;
 $17 = $ppmm;
 $18 = $lfsparms;
 $19 = $16;
 $20 = ($19|0)!=(8);
 if ($20) {
  $21 = HEAP32[_stderr>>2]|0;
  (_fprintf(($21|0),(464|0),($vararg_buffer|0))|0);
  $22 = HEAP32[_stderr>>2]|0;
  $23 = $16;
  HEAP32[$vararg_buffer1>>2] = $23;
  (_fprintf(($22|0),(512|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $92 = $0;
  STACKTOP = sp;return ($92|0);
 }
 $24 = $13;
 $25 = $14;
 $26 = $15;
 $27 = $18;
 $28 = (_lfs_detect_minutiae_V2($minutiae,$direction_map,$low_contrast_map,$low_flow_map,$high_curve_map,$map_w,$map_h,$bdata,$bw,$bh,$24,$25,$26,$27)|0);
 $ret = $28;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = $ret;
  $0 = $30;
  $92 = $0;
  STACKTOP = sp;return ($92|0);
 }
 $31 = HEAP32[$direction_map>>2]|0;
 $32 = HEAP32[$low_contrast_map>>2]|0;
 $33 = HEAP32[$low_flow_map>>2]|0;
 $34 = HEAP32[$high_curve_map>>2]|0;
 $35 = HEAP32[$map_w>>2]|0;
 $36 = HEAP32[$map_h>>2]|0;
 $37 = (_gen_quality_map($quality_map,$31,$32,$33,$34,$35,$36)|0);
 $ret = $37;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($39);
  $40 = HEAP32[$direction_map>>2]|0;
  _free($40);
  $41 = HEAP32[$low_contrast_map>>2]|0;
  _free($41);
  $42 = HEAP32[$low_flow_map>>2]|0;
  _free($42);
  $43 = HEAP32[$high_curve_map>>2]|0;
  _free($43);
  $44 = HEAP32[$bdata>>2]|0;
  _free($44);
  $45 = $ret;
  $0 = $45;
  $92 = $0;
  STACKTOP = sp;return ($92|0);
 }
 $46 = HEAP32[$minutiae>>2]|0;
 $47 = HEAP32[$quality_map>>2]|0;
 $48 = HEAP32[$map_w>>2]|0;
 $49 = HEAP32[$map_h>>2]|0;
 $50 = $18;
 $51 = (($50) + 8|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = $13;
 $54 = $14;
 $55 = $15;
 $56 = $16;
 $57 = $17;
 $58 = (_combined_minutia_quality($46,$47,$48,$49,$52,$53,$54,$55,$56,$57)|0);
 $ret = $58;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$minutiae>>2]|0;
  _free_minutiae($60);
  $61 = HEAP32[$direction_map>>2]|0;
  _free($61);
  $62 = HEAP32[$low_contrast_map>>2]|0;
  _free($62);
  $63 = HEAP32[$low_flow_map>>2]|0;
  _free($63);
  $64 = HEAP32[$high_curve_map>>2]|0;
  _free($64);
  $65 = HEAP32[$quality_map>>2]|0;
  _free($65);
  $66 = HEAP32[$bdata>>2]|0;
  _free($66);
  $67 = $ret;
  $0 = $67;
  $92 = $0;
  STACKTOP = sp;return ($92|0);
 } else {
  $68 = HEAP32[$minutiae>>2]|0;
  $69 = $1;
  HEAP32[$69>>2] = $68;
  $70 = HEAP32[$quality_map>>2]|0;
  $71 = $2;
  HEAP32[$71>>2] = $70;
  $72 = HEAP32[$direction_map>>2]|0;
  $73 = $3;
  HEAP32[$73>>2] = $72;
  $74 = HEAP32[$low_contrast_map>>2]|0;
  $75 = $4;
  HEAP32[$75>>2] = $74;
  $76 = HEAP32[$low_flow_map>>2]|0;
  $77 = $5;
  HEAP32[$77>>2] = $76;
  $78 = HEAP32[$high_curve_map>>2]|0;
  $79 = $6;
  HEAP32[$79>>2] = $78;
  $80 = HEAP32[$map_w>>2]|0;
  $81 = $7;
  HEAP32[$81>>2] = $80;
  $82 = HEAP32[$map_h>>2]|0;
  $83 = $8;
  HEAP32[$83>>2] = $82;
  $84 = HEAP32[$bdata>>2]|0;
  $85 = $9;
  HEAP32[$85>>2] = $84;
  $86 = HEAP32[$bw>>2]|0;
  $87 = $10;
  HEAP32[$87>>2] = $86;
  $88 = HEAP32[$bh>>2]|0;
  $89 = $11;
  HEAP32[$89>>2] = $88;
  $90 = $16;
  $91 = $12;
  HEAP32[$91>>2] = $90;
  $0 = 0;
  $92 = $0;
  STACKTOP = sp;return ($92|0);
 }
 return 0|0;
}
function _alloc_minutiae($ominutiae,$max_minutiae) {
 $ominutiae = $ominutiae|0;
 $max_minutiae = $max_minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $minutiae = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $ominutiae;
 $1 = $max_minutiae;
 $2 = (_malloc(12)|0);
 $minutiae = $2;
 $3 = $minutiae;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  (_fprintf(($5|0),(1336|0),($vararg_buffer|0))|0);
  _exit(-430);
  // unreachable;
 }
 $6 = $1;
 $7 = $6<<2;
 $8 = (_malloc($7)|0);
 $9 = $minutiae;
 $10 = (($9) + 8|0);
 HEAP32[$10>>2] = $8;
 $11 = $minutiae;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(1384|0),($vararg_buffer1|0))|0);
  _exit(-431);
  // unreachable;
 } else {
  $16 = $1;
  $17 = $minutiae;
  HEAP32[$17>>2] = $16;
  $18 = $minutiae;
  $19 = (($18) + 4|0);
  HEAP32[$19>>2] = 0;
  $20 = $minutiae;
  $21 = $0;
  HEAP32[$21>>2] = $20;
  STACKTOP = sp;return 0;
 }
 return 0|0;
}
function _realloc_minutiae($minutiae,$incr_minutiae) {
 $minutiae = $minutiae|0;
 $incr_minutiae = $incr_minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = $minutiae;
 $1 = $incr_minutiae;
 $2 = $1;
 $3 = $0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + ($2))|0;
 HEAP32[$3>>2] = $5;
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = HEAP32[$9>>2]|0;
 $11 = $10<<2;
 $12 = (_realloc($8,$11)|0);
 $13 = $0;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = $12;
 $15 = $0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(1440|0),($vararg_buffer|0))|0);
  _exit(-432);
  // unreachable;
 } else {
  STACKTOP = sp;return 0;
 }
 return 0|0;
}
function _choose_scan_direction($imapval,$ndirs) {
 $imapval = $imapval|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $qtr_ndirs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $imapval;
 $2 = $ndirs;
 $3 = $2;
 $4 = $3 >> 2;
 $qtr_ndirs = $4;
 $5 = $1;
 $6 = $qtr_ndirs;
 $7 = ($5|0)<=($6|0);
 if (!($7)) {
  $8 = $1;
  $9 = $qtr_ndirs;
  $10 = ($9*3)|0;
  $11 = ($8|0)>($10|0);
  if (!($11)) {
   $0 = 1;
   $12 = $0;
   STACKTOP = sp;return ($12|0);
  }
 }
 $0 = 0;
 $12 = $0;
 STACKTOP = sp;return ($12|0);
}
function _detect_minutiae_V2($minutiae,$bdata,$iw,$ih,$direction_map,$low_flow_map,$high_curve_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $direction_map = $direction_map|0;
 $low_flow_map = $low_flow_map|0;
 $high_curve_map = $high_curve_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $pdirection_map = 0;
 var $phigh_curve_map = 0, $plow_flow_map = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $pdirection_map = sp + 8|0;
 $plow_flow_map = sp + 4|0;
 $phigh_curve_map = sp;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $direction_map;
 $6 = $low_flow_map;
 $7 = $high_curve_map;
 $8 = $mw;
 $9 = $mh;
 $10 = $lfsparms;
 $11 = $3;
 $12 = $4;
 $13 = $5;
 $14 = $8;
 $15 = $9;
 $16 = $10;
 $17 = (($16) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (_pixelize_map($pdirection_map,$11,$12,$13,$14,$15,$18)|0);
 $ret = $19;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $ret;
  $0 = $21;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 $22 = $3;
 $23 = $4;
 $24 = $6;
 $25 = $8;
 $26 = $9;
 $27 = $10;
 $28 = (($27) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_pixelize_map($plow_flow_map,$22,$23,$24,$25,$26,$29)|0);
 $ret = $30;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$pdirection_map>>2]|0;
  _free($32);
  $33 = $ret;
  $0 = $33;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 $34 = $3;
 $35 = $4;
 $36 = $7;
 $37 = $8;
 $38 = $9;
 $39 = $10;
 $40 = (($39) + 8|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (_pixelize_map($phigh_curve_map,$34,$35,$36,$37,$38,$41)|0);
 $ret = $42;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = HEAP32[$pdirection_map>>2]|0;
  _free($44);
  $45 = HEAP32[$plow_flow_map>>2]|0;
  _free($45);
  $46 = $ret;
  $0 = $46;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 $47 = $1;
 $48 = $2;
 $49 = $3;
 $50 = $4;
 $51 = HEAP32[$pdirection_map>>2]|0;
 $52 = HEAP32[$plow_flow_map>>2]|0;
 $53 = HEAP32[$phigh_curve_map>>2]|0;
 $54 = $10;
 $55 = (_scan4minutiae_horizontally_V2($47,$48,$49,$50,$51,$52,$53,$54)|0);
 $ret = $55;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$pdirection_map>>2]|0;
  _free($57);
  $58 = HEAP32[$plow_flow_map>>2]|0;
  _free($58);
  $59 = HEAP32[$phigh_curve_map>>2]|0;
  _free($59);
  $60 = $ret;
  $0 = $60;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 $61 = $1;
 $62 = $2;
 $63 = $3;
 $64 = $4;
 $65 = HEAP32[$pdirection_map>>2]|0;
 $66 = HEAP32[$plow_flow_map>>2]|0;
 $67 = HEAP32[$phigh_curve_map>>2]|0;
 $68 = $10;
 $69 = (_scan4minutiae_vertically_V2($61,$62,$63,$64,$65,$66,$67,$68)|0);
 $ret = $69;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$pdirection_map>>2]|0;
  _free($71);
  $72 = HEAP32[$plow_flow_map>>2]|0;
  _free($72);
  $73 = HEAP32[$phigh_curve_map>>2]|0;
  _free($73);
  $74 = $ret;
  $0 = $74;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 } else {
  $75 = HEAP32[$pdirection_map>>2]|0;
  _free($75);
  $76 = HEAP32[$plow_flow_map>>2]|0;
  _free($76);
  $77 = HEAP32[$phigh_curve_map>>2]|0;
  _free($77);
  $0 = 0;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 return 0|0;
}
function _scan4minutiae_horizontally_V2($minutiae,$bdata,$iw,$ih,$pdirection_map,$plow_flow_map,$phigh_curve_map,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $pdirection_map = $pdirection_map|0;
 $plow_flow_map = $plow_flow_map|0;
 $phigh_curve_map = $phigh_curve_map|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $cx = 0, $cy = 0, $ex = 0, $ey = 0, $nposs = 0, $p1ptr = 0, $p2ptr = 0, $possible = 0, $ret = 0, $sx = 0, $sy = 0, $x2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $cx = sp + 64|0;
 $p1ptr = sp + 52|0;
 $p2ptr = sp + 48|0;
 $possible = sp + 8|0;
 $nposs = sp + 4|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $pdirection_map;
 $6 = $plow_flow_map;
 $7 = $phigh_curve_map;
 $8 = $lfsparms;
 $sx = 0;
 $9 = $3;
 $ex = $9;
 $sy = 0;
 $10 = $4;
 $ey = $10;
 $11 = $sy;
 $cy = $11;
 L1: while(1) {
  $12 = $cy;
  $13 = (($12) + 1)|0;
  $14 = $ey;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   label = 24;
   break;
  }
  $16 = $sx;
  HEAP32[$cx>>2] = $16;
  while(1) {
   $17 = HEAP32[$cx>>2]|0;
   $18 = $ex;
   $19 = ($17|0)<($18|0);
   if (!($19)) {
    break;
   }
   $20 = $2;
   $21 = $cy;
   $22 = $3;
   $23 = Math_imul($21, $22)|0;
   $24 = (($20) + ($23)|0);
   $25 = HEAP32[$cx>>2]|0;
   $26 = (($24) + ($25)|0);
   HEAP32[$p1ptr>>2] = $26;
   $27 = $2;
   $28 = $cy;
   $29 = (($28) + 1)|0;
   $30 = $3;
   $31 = Math_imul($29, $30)|0;
   $32 = (($27) + ($31)|0);
   $33 = HEAP32[$cx>>2]|0;
   $34 = (($32) + ($33)|0);
   HEAP32[$p2ptr>>2] = $34;
   $35 = HEAP32[$p1ptr>>2]|0;
   $36 = HEAP8[$35>>0]|0;
   $37 = HEAP32[$p2ptr>>2]|0;
   $38 = HEAP8[$37>>0]|0;
   $39 = (_match_1st_pair($36,$38,$possible,$nposs)|0);
   $40 = ($39|0)!=(0);
   if ($40) {
    $41 = HEAP32[$cx>>2]|0;
    $42 = (($41) + 1)|0;
    HEAP32[$cx>>2] = $42;
    $43 = HEAP32[$p1ptr>>2]|0;
    $44 = (($43) + 1|0);
    HEAP32[$p1ptr>>2] = $44;
    $45 = HEAP32[$p2ptr>>2]|0;
    $46 = (($45) + 1|0);
    HEAP32[$p2ptr>>2] = $46;
    $47 = HEAP32[$cx>>2]|0;
    $48 = $ex;
    $49 = ($47|0)<($48|0);
    if ($49) {
     $50 = HEAP32[$p1ptr>>2]|0;
     $51 = HEAP8[$50>>0]|0;
     $52 = HEAP32[$p2ptr>>2]|0;
     $53 = HEAP8[$52>>0]|0;
     $54 = (_match_2nd_pair($51,$53,$possible,$nposs)|0);
     $55 = ($54|0)!=(0);
     if ($55) {
      $56 = HEAP32[$cx>>2]|0;
      $x2 = $56;
      $57 = $ex;
      $58 = $3;
      $59 = $4;
      _skip_repeated_horizontal_pair($cx,$57,$p1ptr,$p2ptr,$58,$59);
      $60 = HEAP32[$cx>>2]|0;
      $61 = $ex;
      $62 = ($60|0)<($61|0);
      if ($62) {
       $63 = HEAP32[$p1ptr>>2]|0;
       $64 = HEAP8[$63>>0]|0;
       $65 = HEAP32[$p2ptr>>2]|0;
       $66 = HEAP8[$65>>0]|0;
       $67 = (_match_3rd_pair($64,$66,$possible,$nposs)|0);
       $68 = ($67|0)!=(0);
       if ($68) {
        $69 = $1;
        $70 = HEAP32[$cx>>2]|0;
        $71 = $cy;
        $72 = $x2;
        $73 = HEAP32[$possible>>2]|0;
        $74 = $2;
        $75 = $3;
        $76 = $4;
        $77 = $5;
        $78 = $6;
        $79 = $7;
        $80 = $8;
        $81 = (_process_horizontal_scan_minutia_V2($69,$70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$80)|0);
        $ret = $81;
        $82 = ($81|0)!=(0);
        if ($82) {
         $83 = $ret;
         $84 = ($83|0)<(0);
         if ($84) {
          label = 12;
          break L1;
         }
        }
       }
       $86 = HEAP32[$p1ptr>>2]|0;
       $87 = HEAP8[$86>>0]|0;
       $88 = $87&255;
       $89 = HEAP32[$p2ptr>>2]|0;
       $90 = HEAP8[$89>>0]|0;
       $91 = $90&255;
       $92 = ($88|0)!=($91|0);
       if ($92) {
        $93 = HEAP32[$cx>>2]|0;
        $94 = (($93) + -1)|0;
        HEAP32[$cx>>2] = $94;
       }
      }
     }
    }
   } else {
    $95 = HEAP32[$cx>>2]|0;
    $96 = (($95) + 1)|0;
    HEAP32[$cx>>2] = $96;
   }
  }
  $97 = $cy;
  $98 = (($97) + 1)|0;
  $cy = $98;
 }
 if ((label|0) == 12) {
  $85 = $ret;
  $0 = $85;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 else if ((label|0) == 24) {
  $0 = 0;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 return 0|0;
}
function _scan4minutiae_vertically_V2($minutiae,$bdata,$iw,$ih,$pdirection_map,$plow_flow_map,$phigh_curve_map,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $pdirection_map = $pdirection_map|0;
 $plow_flow_map = $plow_flow_map|0;
 $phigh_curve_map = $phigh_curve_map|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $cx = 0, $cy = 0, $ex = 0, $ey = 0;
 var $nposs = 0, $p1ptr = 0, $p2ptr = 0, $possible = 0, $ret = 0, $sx = 0, $sy = 0, $y2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $cy = sp + 60|0;
 $p1ptr = sp + 52|0;
 $p2ptr = sp + 48|0;
 $possible = sp + 8|0;
 $nposs = sp + 4|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $pdirection_map;
 $6 = $plow_flow_map;
 $7 = $phigh_curve_map;
 $8 = $lfsparms;
 $sx = 0;
 $9 = $3;
 $ex = $9;
 $sy = 0;
 $10 = $4;
 $ey = $10;
 $11 = $sx;
 $cx = $11;
 L1: while(1) {
  $12 = $cx;
  $13 = (($12) + 1)|0;
  $14 = $ex;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   label = 24;
   break;
  }
  $16 = $sy;
  HEAP32[$cy>>2] = $16;
  while(1) {
   $17 = HEAP32[$cy>>2]|0;
   $18 = $ey;
   $19 = ($17|0)<($18|0);
   if (!($19)) {
    break;
   }
   $20 = $2;
   $21 = HEAP32[$cy>>2]|0;
   $22 = $3;
   $23 = Math_imul($21, $22)|0;
   $24 = (($20) + ($23)|0);
   $25 = $cx;
   $26 = (($24) + ($25)|0);
   HEAP32[$p1ptr>>2] = $26;
   $27 = HEAP32[$p1ptr>>2]|0;
   $28 = (($27) + 1|0);
   HEAP32[$p2ptr>>2] = $28;
   $29 = HEAP32[$p1ptr>>2]|0;
   $30 = HEAP8[$29>>0]|0;
   $31 = HEAP32[$p2ptr>>2]|0;
   $32 = HEAP8[$31>>0]|0;
   $33 = (_match_1st_pair($30,$32,$possible,$nposs)|0);
   $34 = ($33|0)!=(0);
   if ($34) {
    $35 = HEAP32[$cy>>2]|0;
    $36 = (($35) + 1)|0;
    HEAP32[$cy>>2] = $36;
    $37 = $3;
    $38 = HEAP32[$p1ptr>>2]|0;
    $39 = (($38) + ($37)|0);
    HEAP32[$p1ptr>>2] = $39;
    $40 = $3;
    $41 = HEAP32[$p2ptr>>2]|0;
    $42 = (($41) + ($40)|0);
    HEAP32[$p2ptr>>2] = $42;
    $43 = HEAP32[$cy>>2]|0;
    $44 = $ey;
    $45 = ($43|0)<($44|0);
    if ($45) {
     $46 = HEAP32[$p1ptr>>2]|0;
     $47 = HEAP8[$46>>0]|0;
     $48 = HEAP32[$p2ptr>>2]|0;
     $49 = HEAP8[$48>>0]|0;
     $50 = (_match_2nd_pair($47,$49,$possible,$nposs)|0);
     $51 = ($50|0)!=(0);
     if ($51) {
      $52 = HEAP32[$cy>>2]|0;
      $y2 = $52;
      $53 = $ey;
      $54 = $3;
      $55 = $4;
      _skip_repeated_vertical_pair($cy,$53,$p1ptr,$p2ptr,$54,$55);
      $56 = HEAP32[$cy>>2]|0;
      $57 = $ey;
      $58 = ($56|0)<($57|0);
      if ($58) {
       $59 = HEAP32[$p1ptr>>2]|0;
       $60 = HEAP8[$59>>0]|0;
       $61 = HEAP32[$p2ptr>>2]|0;
       $62 = HEAP8[$61>>0]|0;
       $63 = (_match_3rd_pair($60,$62,$possible,$nposs)|0);
       $64 = ($63|0)!=(0);
       if ($64) {
        $65 = $1;
        $66 = $cx;
        $67 = HEAP32[$cy>>2]|0;
        $68 = $y2;
        $69 = HEAP32[$possible>>2]|0;
        $70 = $2;
        $71 = $3;
        $72 = $4;
        $73 = $5;
        $74 = $6;
        $75 = $7;
        $76 = $8;
        $77 = (_process_vertical_scan_minutia_V2($65,$66,$67,$68,$69,$70,$71,$72,$73,$74,$75,$76)|0);
        $ret = $77;
        $78 = ($77|0)!=(0);
        if ($78) {
         $79 = $ret;
         $80 = ($79|0)<(0);
         if ($80) {
          label = 12;
          break L1;
         }
        }
       }
       $82 = HEAP32[$p1ptr>>2]|0;
       $83 = HEAP8[$82>>0]|0;
       $84 = $83&255;
       $85 = HEAP32[$p2ptr>>2]|0;
       $86 = HEAP8[$85>>0]|0;
       $87 = $86&255;
       $88 = ($84|0)!=($87|0);
       if ($88) {
        $89 = HEAP32[$cy>>2]|0;
        $90 = (($89) + -1)|0;
        HEAP32[$cy>>2] = $90;
       }
      }
     }
    }
   } else {
    $91 = HEAP32[$cy>>2]|0;
    $92 = (($91) + 1)|0;
    HEAP32[$cy>>2] = $92;
   }
  }
  $93 = $cx;
  $94 = (($93) + 1)|0;
  $cx = $94;
 }
 if ((label|0) == 12) {
  $81 = $ret;
  $0 = $81;
  $95 = $0;
  STACKTOP = sp;return ($95|0);
 }
 else if ((label|0) == 24) {
  $0 = 0;
  $95 = $0;
  STACKTOP = sp;return ($95|0);
 }
 return 0|0;
}
function _update_minutiae($minutiae,$minutia,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $minutia = $minutia|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $delta_dir = 0, $dx = 0, $dy = 0, $full_ndirs = 0, $i = 0, $qtr_ndirs = 0, $ret = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $minutiae;
 $2 = $minutia;
 $3 = $bdata;
 $4 = $iw;
 $5 = $ih;
 $6 = $lfsparms;
 $7 = $1;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($9|0)>=($11|0);
 do {
  if ($12) {
   $13 = $1;
   $14 = (_realloc_minutiae($13,1000)|0);
   $ret = $14;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    break;
   }
   $16 = $ret;
   $0 = $16;
   $212 = $0;
   STACKTOP = sp;return ($212|0);
  }
 } while(0);
 $17 = $6;
 $18 = (($17) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 >> 2;
 $qtr_ndirs = $20;
 $21 = $6;
 $22 = (($21) + 20|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 << 1;
 $full_ndirs = $24;
 $25 = $1;
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)>(0);
 do {
  if ($28) {
   $i = 0;
   while(1) {
    $29 = $i;
    $30 = $1;
    $31 = (($30) + 4|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($29|0)<($32|0);
    if (!($33)) {
     label = 28;
     break;
    }
    $34 = $i;
    $35 = $1;
    $36 = (($35) + 8|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + ($34<<2)|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$39>>2]|0;
    $41 = $2;
    $42 = HEAP32[$41>>2]|0;
    $43 = (($40) - ($42))|0;
    $44 = (Math_abs(($43|0))|0);
    $dx = $44;
    $45 = $dx;
    $46 = $6;
    $47 = (($46) + 152|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = ($45|0)<($48|0);
    if ($49) {
     $50 = $i;
     $51 = $1;
     $52 = (($51) + 8|0);
     $53 = HEAP32[$52>>2]|0;
     $54 = (($53) + ($50<<2)|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (($55) + 4|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = $2;
     $59 = (($58) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = (($57) - ($60))|0;
     $62 = (Math_abs(($61|0))|0);
     $dy = $62;
     $63 = $dy;
     $64 = $6;
     $65 = (($64) + 152|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = ($63|0)<($66|0);
     if ($67) {
      $68 = $i;
      $69 = $1;
      $70 = (($69) + 8|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = (($71) + ($68<<2)|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = (($73) + 32|0);
      $75 = HEAP32[$74>>2]|0;
      $76 = $2;
      $77 = (($76) + 32|0);
      $78 = HEAP32[$77>>2]|0;
      $79 = ($75|0)==($78|0);
      if ($79) {
       $80 = $i;
       $81 = $1;
       $82 = (($81) + 8|0);
       $83 = HEAP32[$82>>2]|0;
       $84 = (($83) + ($80<<2)|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = (($85) + 16|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = $2;
       $89 = (($88) + 16|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = (($87) - ($90))|0;
       $92 = (Math_abs(($91|0))|0);
       $delta_dir = $92;
       $93 = $delta_dir;
       $94 = $full_ndirs;
       $95 = $delta_dir;
       $96 = (($94) - ($95))|0;
       $97 = ($93|0)<($96|0);
       if ($97) {
        $98 = $delta_dir;
        $102 = $98;
       } else {
        $99 = $full_ndirs;
        $100 = $delta_dir;
        $101 = (($99) - ($100))|0;
        $102 = $101;
       }
       $delta_dir = $102;
       $103 = $delta_dir;
       $104 = $qtr_ndirs;
       $105 = ($103|0)<=($104|0);
       if ($105) {
        $106 = $dx;
        $107 = ($106|0)==(0);
        if ($107) {
         $108 = $dy;
         $109 = ($108|0)==(0);
         if ($109) {
          label = 17;
          break;
         }
        }
        $110 = $2;
        $111 = HEAP32[$110>>2]|0;
        $112 = $2;
        $113 = (($112) + 4|0);
        $114 = HEAP32[$113>>2]|0;
        $115 = $6;
        $116 = (($115) + 152|0);
        $117 = HEAP32[$116>>2]|0;
        $118 = $i;
        $119 = $1;
        $120 = (($119) + 8|0);
        $121 = HEAP32[$120>>2]|0;
        $122 = (($121) + ($118<<2)|0);
        $123 = HEAP32[$122>>2]|0;
        $124 = HEAP32[$123>>2]|0;
        $125 = $i;
        $126 = $1;
        $127 = (($126) + 8|0);
        $128 = HEAP32[$127>>2]|0;
        $129 = (($128) + ($125<<2)|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = (($130) + 4|0);
        $132 = HEAP32[$131>>2]|0;
        $133 = $i;
        $134 = $1;
        $135 = (($134) + 8|0);
        $136 = HEAP32[$135>>2]|0;
        $137 = (($136) + ($133<<2)|0);
        $138 = HEAP32[$137>>2]|0;
        $139 = (($138) + 8|0);
        $140 = HEAP32[$139>>2]|0;
        $141 = $i;
        $142 = $1;
        $143 = (($142) + 8|0);
        $144 = HEAP32[$143>>2]|0;
        $145 = (($144) + ($141<<2)|0);
        $146 = HEAP32[$145>>2]|0;
        $147 = (($146) + 12|0);
        $148 = HEAP32[$147>>2]|0;
        $149 = $3;
        $150 = $4;
        $151 = $5;
        $152 = (_search_contour($111,$114,$117,$124,$132,$140,$148,0,$149,$150,$151)|0);
        $153 = ($152|0)!=(0);
        if ($153) {
         label = 19;
         break;
        }
        $154 = $2;
        $155 = HEAP32[$154>>2]|0;
        $156 = $2;
        $157 = (($156) + 4|0);
        $158 = HEAP32[$157>>2]|0;
        $159 = $6;
        $160 = (($159) + 152|0);
        $161 = HEAP32[$160>>2]|0;
        $162 = $i;
        $163 = $1;
        $164 = (($163) + 8|0);
        $165 = HEAP32[$164>>2]|0;
        $166 = (($165) + ($162<<2)|0);
        $167 = HEAP32[$166>>2]|0;
        $168 = HEAP32[$167>>2]|0;
        $169 = $i;
        $170 = $1;
        $171 = (($170) + 8|0);
        $172 = HEAP32[$171>>2]|0;
        $173 = (($172) + ($169<<2)|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = (($174) + 4|0);
        $176 = HEAP32[$175>>2]|0;
        $177 = $i;
        $178 = $1;
        $179 = (($178) + 8|0);
        $180 = HEAP32[$179>>2]|0;
        $181 = (($180) + ($177<<2)|0);
        $182 = HEAP32[$181>>2]|0;
        $183 = (($182) + 8|0);
        $184 = HEAP32[$183>>2]|0;
        $185 = $i;
        $186 = $1;
        $187 = (($186) + 8|0);
        $188 = HEAP32[$187>>2]|0;
        $189 = (($188) + ($185<<2)|0);
        $190 = HEAP32[$189>>2]|0;
        $191 = (($190) + 12|0);
        $192 = HEAP32[$191>>2]|0;
        $193 = $3;
        $194 = $4;
        $195 = $5;
        $196 = (_search_contour($155,$158,$161,$168,$176,$184,$192,1,$193,$194,$195)|0);
        $197 = ($196|0)!=(0);
        if ($197) {
         label = 21;
         break;
        }
       }
      }
     }
    }
    $198 = $i;
    $199 = (($198) + 1)|0;
    $i = $199;
   }
   if ((label|0) == 17) {
    $0 = 2;
    $212 = $0;
    STACKTOP = sp;return ($212|0);
   }
   else if ((label|0) == 19) {
    $0 = 2;
    $212 = $0;
    STACKTOP = sp;return ($212|0);
   }
   else if ((label|0) == 21) {
    $0 = 2;
    $212 = $0;
    STACKTOP = sp;return ($212|0);
   }
   else if ((label|0) == 28) {
    break;
   }
  }
 } while(0);
 $200 = $2;
 $201 = $1;
 $202 = (($201) + 4|0);
 $203 = HEAP32[$202>>2]|0;
 $204 = $1;
 $205 = (($204) + 8|0);
 $206 = HEAP32[$205>>2]|0;
 $207 = (($206) + ($203<<2)|0);
 HEAP32[$207>>2] = $200;
 $208 = $1;
 $209 = (($208) + 4|0);
 $210 = HEAP32[$209>>2]|0;
 $211 = (($210) + 1)|0;
 HEAP32[$209>>2] = $211;
 $0 = 0;
 $212 = $0;
 STACKTOP = sp;return ($212|0);
}
function _update_minutiae_V2($minutiae,$minutia,$scan_dir,$dmapval,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $minutia = $minutia|0;
 $scan_dir = $scan_dir|0;
 $dmapval = $dmapval|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $delta_dir = 0, $dx = 0, $dy = 0, $full_ndirs = 0, $i = 0, $map_scan_dir = 0, $qtr_ndirs = 0, $ret = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $minutiae;
 $2 = $minutia;
 $3 = $scan_dir;
 $4 = $dmapval;
 $5 = $bdata;
 $6 = $iw;
 $7 = $ih;
 $8 = $lfsparms;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $1;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($11|0)>=($13|0);
 do {
  if ($14) {
   $15 = $1;
   $16 = (_realloc_minutiae($15,1000)|0);
   $ret = $16;
   $17 = ($16|0)!=(0);
   if (!($17)) {
    break;
   }
   $18 = $ret;
   $0 = $18;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
 } while(0);
 $19 = $8;
 $20 = (($19) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $21 >> 2;
 $qtr_ndirs = $22;
 $23 = $8;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 << 1;
 $full_ndirs = $26;
 $27 = $1;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>(0);
 do {
  if ($30) {
   $31 = $1;
   $32 = (($31) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) - 1)|0;
   $i = $34;
   while(1) {
    $35 = $i;
    $36 = ($35|0)>=(0);
    if (!($36)) {
     label = 35;
     break;
    }
    $37 = $i;
    $38 = $1;
    $39 = (($38) + 8|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + ($37<<2)|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$42>>2]|0;
    $44 = $2;
    $45 = HEAP32[$44>>2]|0;
    $46 = (($43) - ($45))|0;
    $47 = (Math_abs(($46|0))|0);
    $dx = $47;
    $48 = $dx;
    $49 = $8;
    $50 = (($49) + 152|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ($48|0)<($51|0);
    if ($52) {
     $53 = $i;
     $54 = $1;
     $55 = (($54) + 8|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (($56) + ($53<<2)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = (($58) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = $2;
     $62 = (($61) + 4|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = (($60) - ($63))|0;
     $65 = (Math_abs(($64|0))|0);
     $dy = $65;
     $66 = $dy;
     $67 = $8;
     $68 = (($67) + 152|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ($66|0)<($69|0);
     if ($70) {
      $71 = $i;
      $72 = $1;
      $73 = (($72) + 8|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + ($71<<2)|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = (($76) + 32|0);
      $78 = HEAP32[$77>>2]|0;
      $79 = $2;
      $80 = (($79) + 32|0);
      $81 = HEAP32[$80>>2]|0;
      $82 = ($78|0)==($81|0);
      if ($82) {
       $83 = $i;
       $84 = $1;
       $85 = (($84) + 8|0);
       $86 = HEAP32[$85>>2]|0;
       $87 = (($86) + ($83<<2)|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = (($88) + 16|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = $2;
       $92 = (($91) + 16|0);
       $93 = HEAP32[$92>>2]|0;
       $94 = (($90) - ($93))|0;
       $95 = (Math_abs(($94|0))|0);
       $delta_dir = $95;
       $96 = $delta_dir;
       $97 = $full_ndirs;
       $98 = $delta_dir;
       $99 = (($97) - ($98))|0;
       $100 = ($96|0)<($99|0);
       if ($100) {
        $101 = $delta_dir;
        $105 = $101;
       } else {
        $102 = $full_ndirs;
        $103 = $delta_dir;
        $104 = (($102) - ($103))|0;
        $105 = $104;
       }
       $delta_dir = $105;
       $106 = $delta_dir;
       $107 = $qtr_ndirs;
       $108 = ($106|0)<=($107|0);
       if ($108) {
        $109 = $dx;
        $110 = ($109|0)==(0);
        if ($110) {
         $111 = $dy;
         $112 = ($111|0)==(0);
         if ($112) {
          label = 17;
          break;
         }
        }
        $113 = $2;
        $114 = HEAP32[$113>>2]|0;
        $115 = $2;
        $116 = (($115) + 4|0);
        $117 = HEAP32[$116>>2]|0;
        $118 = $8;
        $119 = (($118) + 152|0);
        $120 = HEAP32[$119>>2]|0;
        $121 = $i;
        $122 = $1;
        $123 = (($122) + 8|0);
        $124 = HEAP32[$123>>2]|0;
        $125 = (($124) + ($121<<2)|0);
        $126 = HEAP32[$125>>2]|0;
        $127 = HEAP32[$126>>2]|0;
        $128 = $i;
        $129 = $1;
        $130 = (($129) + 8|0);
        $131 = HEAP32[$130>>2]|0;
        $132 = (($131) + ($128<<2)|0);
        $133 = HEAP32[$132>>2]|0;
        $134 = (($133) + 4|0);
        $135 = HEAP32[$134>>2]|0;
        $136 = $i;
        $137 = $1;
        $138 = (($137) + 8|0);
        $139 = HEAP32[$138>>2]|0;
        $140 = (($139) + ($136<<2)|0);
        $141 = HEAP32[$140>>2]|0;
        $142 = (($141) + 8|0);
        $143 = HEAP32[$142>>2]|0;
        $144 = $i;
        $145 = $1;
        $146 = (($145) + 8|0);
        $147 = HEAP32[$146>>2]|0;
        $148 = (($147) + ($144<<2)|0);
        $149 = HEAP32[$148>>2]|0;
        $150 = (($149) + 12|0);
        $151 = HEAP32[$150>>2]|0;
        $152 = $5;
        $153 = $6;
        $154 = $7;
        $155 = (_search_contour($114,$117,$120,$127,$135,$143,$151,0,$152,$153,$154)|0);
        $156 = ($155|0)!=(0);
        if ($156) {
         label = 20;
        } else {
         $157 = $2;
         $158 = HEAP32[$157>>2]|0;
         $159 = $2;
         $160 = (($159) + 4|0);
         $161 = HEAP32[$160>>2]|0;
         $162 = $8;
         $163 = (($162) + 152|0);
         $164 = HEAP32[$163>>2]|0;
         $165 = $i;
         $166 = $1;
         $167 = (($166) + 8|0);
         $168 = HEAP32[$167>>2]|0;
         $169 = (($168) + ($165<<2)|0);
         $170 = HEAP32[$169>>2]|0;
         $171 = HEAP32[$170>>2]|0;
         $172 = $i;
         $173 = $1;
         $174 = (($173) + 8|0);
         $175 = HEAP32[$174>>2]|0;
         $176 = (($175) + ($172<<2)|0);
         $177 = HEAP32[$176>>2]|0;
         $178 = (($177) + 4|0);
         $179 = HEAP32[$178>>2]|0;
         $180 = $i;
         $181 = $1;
         $182 = (($181) + 8|0);
         $183 = HEAP32[$182>>2]|0;
         $184 = (($183) + ($180<<2)|0);
         $185 = HEAP32[$184>>2]|0;
         $186 = (($185) + 8|0);
         $187 = HEAP32[$186>>2]|0;
         $188 = $i;
         $189 = $1;
         $190 = (($189) + 8|0);
         $191 = HEAP32[$190>>2]|0;
         $192 = (($191) + ($188<<2)|0);
         $193 = HEAP32[$192>>2]|0;
         $194 = (($193) + 12|0);
         $195 = HEAP32[$194>>2]|0;
         $196 = $5;
         $197 = $6;
         $198 = $7;
         $199 = (_search_contour($158,$161,$164,$171,$179,$187,$195,1,$196,$197,$198)|0);
         $200 = ($199|0)!=(0);
         if ($200) {
          label = 20;
         }
        }
        if ((label|0) == 20) {
         label = 0;
         $201 = $4;
         $202 = ($201|0)>=(0);
         if (!($202)) {
          label = 27;
          break;
         }
         $203 = $4;
         $204 = $8;
         $205 = (($204) + 20|0);
         $206 = HEAP32[$205>>2]|0;
         $207 = (_choose_scan_direction($203,$206)|0);
         $map_scan_dir = $207;
         $208 = $map_scan_dir;
         $209 = $3;
         $210 = ($208|0)==($209|0);
         if (!($210)) {
          label = 25;
          break;
         }
         $211 = $i;
         $212 = $1;
         $213 = (_remove_minutia($211,$212)|0);
         $ret = $213;
         $214 = ($213|0)!=(0);
         if ($214) {
          label = 23;
          break;
         }
        }
       }
      }
     }
    }
    $216 = $i;
    $217 = (($216) + -1)|0;
    $i = $217;
   }
   if ((label|0) == 17) {
    $0 = 2;
    $230 = $0;
    STACKTOP = sp;return ($230|0);
   }
   else if ((label|0) == 23) {
    $215 = $ret;
    $0 = $215;
    $230 = $0;
    STACKTOP = sp;return ($230|0);
   }
   else if ((label|0) == 25) {
    $0 = 2;
    $230 = $0;
    STACKTOP = sp;return ($230|0);
   }
   else if ((label|0) == 27) {
    $0 = 2;
    $230 = $0;
    STACKTOP = sp;return ($230|0);
   }
   else if ((label|0) == 35) {
    break;
   }
  }
 } while(0);
 $218 = $2;
 $219 = $1;
 $220 = (($219) + 4|0);
 $221 = HEAP32[$220>>2]|0;
 $222 = $1;
 $223 = (($222) + 8|0);
 $224 = HEAP32[$223>>2]|0;
 $225 = (($224) + ($221<<2)|0);
 HEAP32[$225>>2] = $218;
 $226 = $1;
 $227 = (($226) + 4|0);
 $228 = HEAP32[$227>>2]|0;
 $229 = (($228) + 1)|0;
 HEAP32[$227>>2] = $229;
 $0 = 0;
 $230 = $0;
 STACKTOP = sp;return ($230|0);
}
function _remove_minutia($index,$minutiae) {
 $index = $index|0;
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fr = 0, $to = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $index;
 $2 = $minutiae;
 $3 = $1;
 $4 = ($3|0)<(0);
 if ($4) {
  $5 = $1;
  $6 = $2;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)>=($8|0);
  if ($9) {
   $10 = HEAP32[_stderr>>2]|0;
   (_fprintf(($10|0),(1856|0),($vararg_buffer|0))|0);
   $0 = -380;
   $44 = $0;
   STACKTOP = sp;return ($44|0);
  }
 }
 $11 = $1;
 $12 = $2;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 _free_minutia($16);
 $17 = $1;
 $to = $17;
 $18 = $1;
 $19 = (($18) + 1)|0;
 $fr = $19;
 while(1) {
  $20 = $fr;
  $21 = $2;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = $fr;
  $26 = $2;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $to;
  $32 = $2;
  $33 = (($32) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31<<2)|0);
  HEAP32[$35>>2] = $30;
  $36 = $to;
  $37 = (($36) + 1)|0;
  $to = $37;
  $38 = $fr;
  $39 = (($38) + 1)|0;
  $fr = $39;
 }
 $40 = $2;
 $41 = (($40) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + -1)|0;
 HEAP32[$41>>2] = $43;
 $0 = 0;
 $44 = $0;
 STACKTOP = sp;return ($44|0);
}
function _sort_minutiae_y_x($minutiae,$iw,$ih) {
 $minutiae = $minutiae|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $i = 0, $newlist = 0, $order = 0, $ranks = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $order = sp + 24|0;
 $1 = $minutiae;
 $2 = $iw;
 $3 = $ih;
 $4 = $1;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6<<2;
 $8 = (_malloc($7)|0);
 $ranks = $8;
 $9 = $ranks;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(1496|0),($vararg_buffer|0))|0);
  $0 = -310;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $1;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   break;
  }
  $17 = $i;
  $18 = $1;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + ($17<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $2;
  $26 = Math_imul($24, $25)|0;
  $27 = $i;
  $28 = $1;
  $29 = (($28) + 8|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<2)|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($26) + ($33))|0;
  $35 = $i;
  $36 = $ranks;
  $37 = (($36) + ($35<<2)|0);
  HEAP32[$37>>2] = $34;
  $38 = $i;
  $39 = (($38) + 1)|0;
  $i = $39;
 }
 $40 = $ranks;
 $41 = $1;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sort_indices_int_inc($order,$40,$43)|0);
 $ret = $44;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = $ranks;
  _free($46);
  $47 = $ret;
  $0 = $47;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $48 = $1;
 $49 = (($48) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = $50<<2;
 $52 = (_malloc($51)|0);
 $newlist = $52;
 $53 = $newlist;
 $54 = ($53|0)==(0|0);
 if ($54) {
  $55 = $ranks;
  _free($55);
  $56 = HEAP32[$order>>2]|0;
  _free($56);
  $57 = HEAP32[_stderr>>2]|0;
  (_fprintf(($57|0),(1544|0),($vararg_buffer1|0))|0);
  $0 = -311;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $i = 0;
 while(1) {
  $58 = $i;
  $59 = $1;
  $60 = (($59) + 4|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($58|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = $i;
  $64 = HEAP32[$order>>2]|0;
  $65 = (($64) + ($63<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $1;
  $68 = (($67) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + ($66<<2)|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = $i;
  $73 = $newlist;
  $74 = (($73) + ($72<<2)|0);
  HEAP32[$74>>2] = $71;
  $75 = $i;
  $76 = (($75) + 1)|0;
  $i = $76;
 }
 $77 = $1;
 $78 = (($77) + 8|0);
 $79 = HEAP32[$78>>2]|0;
 _free($79);
 $80 = $newlist;
 $81 = $1;
 $82 = (($81) + 8|0);
 HEAP32[$82>>2] = $80;
 $83 = HEAP32[$order>>2]|0;
 _free($83);
 $84 = $ranks;
 _free($84);
 $0 = 0;
 $85 = $0;
 STACKTOP = sp;return ($85|0);
}
function _sort_minutiae_x_y($minutiae,$iw,$ih) {
 $minutiae = $minutiae|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $i = 0, $newlist = 0, $order = 0, $ranks = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $order = sp + 24|0;
 $1 = $minutiae;
 $2 = $iw;
 $3 = $ih;
 $4 = $1;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6<<2;
 $8 = (_malloc($7)|0);
 $ranks = $8;
 $9 = $ranks;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(1592|0),($vararg_buffer|0))|0);
  $0 = -440;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $1;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   break;
  }
  $17 = $i;
  $18 = $1;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + ($17<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = $2;
  $25 = Math_imul($23, $24)|0;
  $26 = $i;
  $27 = $1;
  $28 = (($27) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($25) + ($33))|0;
  $35 = $i;
  $36 = $ranks;
  $37 = (($36) + ($35<<2)|0);
  HEAP32[$37>>2] = $34;
  $38 = $i;
  $39 = (($38) + 1)|0;
  $i = $39;
 }
 $40 = $ranks;
 $41 = $1;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sort_indices_int_inc($order,$40,$43)|0);
 $ret = $44;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = $ranks;
  _free($46);
  $47 = $ret;
  $0 = $47;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $48 = $1;
 $49 = (($48) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = $50<<2;
 $52 = (_malloc($51)|0);
 $newlist = $52;
 $53 = $newlist;
 $54 = ($53|0)==(0|0);
 if ($54) {
  $55 = $ranks;
  _free($55);
  $56 = HEAP32[$order>>2]|0;
  _free($56);
  $57 = HEAP32[_stderr>>2]|0;
  (_fprintf(($57|0),(1640|0),($vararg_buffer1|0))|0);
  $0 = -441;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $i = 0;
 while(1) {
  $58 = $i;
  $59 = $1;
  $60 = (($59) + 4|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($58|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = $i;
  $64 = HEAP32[$order>>2]|0;
  $65 = (($64) + ($63<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $1;
  $68 = (($67) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + ($66<<2)|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = $i;
  $73 = $newlist;
  $74 = (($73) + ($72<<2)|0);
  HEAP32[$74>>2] = $71;
  $75 = $i;
  $76 = (($75) + 1)|0;
  $i = $76;
 }
 $77 = $1;
 $78 = (($77) + 8|0);
 $79 = HEAP32[$78>>2]|0;
 _free($79);
 $80 = $newlist;
 $81 = $1;
 $82 = (($81) + 8|0);
 HEAP32[$82>>2] = $80;
 $83 = HEAP32[$order>>2]|0;
 _free($83);
 $84 = $ranks;
 _free($84);
 $0 = 0;
 $85 = $0;
 STACKTOP = sp;return ($85|0);
}
function _rm_dup_minutiae($minutiae) {
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $minutia1 = 0, $minutia2 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $minutiae;
 $2 = $1;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 $i = $5;
 while(1) {
  $6 = $i;
  $7 = ($6|0)>(0);
  if (!($7)) {
   label = 10;
   break;
  }
  $8 = $i;
  $9 = $1;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $minutia1 = $13;
  $14 = $i;
  $15 = (($14) - 1)|0;
  $16 = $1;
  $17 = (($16) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $minutia2 = $20;
  $21 = $minutia1;
  $22 = HEAP32[$21>>2]|0;
  $23 = $minutia2;
  $24 = HEAP32[$23>>2]|0;
  $25 = ($22|0)==($24|0);
  if ($25) {
   $26 = $minutia1;
   $27 = (($26) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = $minutia2;
   $30 = (($29) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($28|0)==($31|0);
   if ($32) {
    $33 = $i;
    $34 = (($33) - 1)|0;
    $35 = $1;
    $36 = (_remove_minutia($34,$35)|0);
    $ret = $36;
    $37 = ($36|0)!=(0);
    if ($37) {
     label = 6;
     break;
    }
   }
  }
  $39 = $i;
  $40 = (($39) + -1)|0;
  $i = $40;
 }
 if ((label|0) == 6) {
  $38 = $ret;
  $0 = $38;
  $41 = $0;
  STACKTOP = sp;return ($41|0);
 }
 else if ((label|0) == 10) {
  $0 = 0;
  $41 = $0;
  STACKTOP = sp;return ($41|0);
 }
 return 0|0;
}
function _dump_minutiae($fpout,$minutiae) {
 $fpout = $fpout|0;
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer24 = 0, $vararg_buffer8 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer24 = sp + 88|0;
 $vararg_buffer19 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $fpout;
 $1 = $minutiae;
 $2 = $0;
 $3 = $1;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 (_fprintf(($2|0),(1688|0),($vararg_buffer|0))|0);
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $1;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $0;
  $12 = $i;
  $13 = $i;
  $14 = $1;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = $i;
  $21 = $1;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $i;
  $29 = $1;
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + 16|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = $i;
  $37 = $1;
  $38 = (($37) + 8|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + ($36<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 24|0);
  $43 = +HEAPF64[$42>>3];
  HEAP32[$vararg_buffer1>>2] = $12;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $19;
  $vararg_ptr5 = (($vararg_buffer1) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $27;
  $vararg_ptr6 = (($vararg_buffer1) + 12|0);
  HEAP32[$vararg_ptr6>>2] = $35;
  $vararg_ptr7 = (($vararg_buffer1) + 16|0);
  HEAPF64[tempDoublePtr>>3]=$43;HEAP32[$vararg_ptr7>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr7+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($11|0),(1712|0),($vararg_buffer1|0))|0);
  $44 = $i;
  $45 = $1;
  $46 = (($45) + 8|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + ($44<<2)|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 32|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ($51|0)==(1);
  if ($52) {
   $53 = $0;
   (_fprintf(($53|0),(1744|0),($vararg_buffer8|0))|0);
  } else {
   $54 = $0;
   (_fprintf(($54|0),(1752|0),($vararg_buffer10|0))|0);
  }
  $55 = $i;
  $56 = $1;
  $57 = (($56) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($55<<2)|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + 36|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($62|0)!=(0);
  if ($63) {
   $64 = $0;
   (_fprintf(($64|0),(1760|0),($vararg_buffer12|0))|0);
  } else {
   $65 = $0;
   (_fprintf(($65|0),(1768|0),($vararg_buffer14|0))|0);
  }
  $66 = $0;
  $67 = $i;
  $68 = $1;
  $69 = (($68) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($70) + ($67<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + 40|0);
  $74 = HEAP32[$73>>2]|0;
  HEAP32[$vararg_buffer16>>2] = $74;
  (_fprintf(($66|0),(1776|0),($vararg_buffer16|0))|0);
  $j = 0;
  while(1) {
   $75 = $j;
   $76 = $i;
   $77 = $1;
   $78 = (($77) + 8|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76<<2)|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = (($81) + 52|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = ($75|0)<($83|0);
   if (!($84)) {
    break;
   }
   $85 = $0;
   $86 = $j;
   $87 = $i;
   $88 = $1;
   $89 = (($88) + 8|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (($90) + ($87<<2)|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = (($92) + 44|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + ($86<<2)|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = $1;
   $98 = (($97) + 8|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = (($99) + ($96<<2)|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = HEAP32[$101>>2]|0;
   $103 = $j;
   $104 = $i;
   $105 = $1;
   $106 = (($105) + 8|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + ($104<<2)|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + 44|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = (($111) + ($103<<2)|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = $1;
   $115 = (($114) + 8|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = (($116) + ($113<<2)|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (($118) + 4|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = $j;
   $122 = $i;
   $123 = $1;
   $124 = (($123) + 8|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = (($125) + ($122<<2)|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) + 48|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = (($129) + ($121<<2)|0);
   $131 = HEAP32[$130>>2]|0;
   HEAP32[$vararg_buffer19>>2] = $102;
   $vararg_ptr22 = (($vararg_buffer19) + 4|0);
   HEAP32[$vararg_ptr22>>2] = $120;
   $vararg_ptr23 = (($vararg_buffer19) + 8|0);
   HEAP32[$vararg_ptr23>>2] = $131;
   (_fprintf(($85|0),(1784|0),($vararg_buffer19|0))|0);
   $132 = $j;
   $133 = (($132) + 1)|0;
   $j = $133;
  }
  $134 = $0;
  (_fprintf(($134|0),(1800|0),($vararg_buffer24|0))|0);
  $135 = $i;
  $136 = (($135) + 1)|0;
  $i = $136;
 }
 STACKTOP = sp;return;
}
function _create_minutia($ominutia,$x_loc,$y_loc,$x_edge,$y_edge,$idir,$reliability,$type,$appearing,$feature_id) {
 $ominutia = $ominutia|0;
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 $idir = $idir|0;
 $reliability = +$reliability;
 $type = $type|0;
 $appearing = $appearing|0;
 $feature_id = $feature_id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0, $9 = 0, $minutia = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $1 = $ominutia;
 $2 = $x_loc;
 $3 = $y_loc;
 $4 = $x_edge;
 $5 = $y_edge;
 $6 = $idir;
 $7 = $reliability;
 $8 = $type;
 $9 = $appearing;
 $10 = $feature_id;
 $11 = (_malloc(56)|0);
 $minutia = $11;
 $12 = $minutia;
 $13 = ($12|0)==(0|0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  (_fprintf(($14|0),(1808|0),($vararg_buffer|0))|0);
  $0 = -230;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 } else {
  $15 = $2;
  $16 = $minutia;
  HEAP32[$16>>2] = $15;
  $17 = $3;
  $18 = $minutia;
  $19 = (($18) + 4|0);
  HEAP32[$19>>2] = $17;
  $20 = $4;
  $21 = $minutia;
  $22 = (($21) + 8|0);
  HEAP32[$22>>2] = $20;
  $23 = $5;
  $24 = $minutia;
  $25 = (($24) + 12|0);
  HEAP32[$25>>2] = $23;
  $26 = $6;
  $27 = $minutia;
  $28 = (($27) + 16|0);
  HEAP32[$28>>2] = $26;
  $29 = $7;
  $30 = $minutia;
  $31 = (($30) + 24|0);
  HEAPF64[$31>>3] = $29;
  $32 = $8;
  $33 = $minutia;
  $34 = (($33) + 32|0);
  HEAP32[$34>>2] = $32;
  $35 = $9;
  $36 = $minutia;
  $37 = (($36) + 36|0);
  HEAP32[$37>>2] = $35;
  $38 = $10;
  $39 = $minutia;
  $40 = (($39) + 40|0);
  HEAP32[$40>>2] = $38;
  $41 = $minutia;
  $42 = (($41) + 44|0);
  HEAP32[$42>>2] = 0;
  $43 = $minutia;
  $44 = (($43) + 48|0);
  HEAP32[$44>>2] = 0;
  $45 = $minutia;
  $46 = (($45) + 52|0);
  HEAP32[$46>>2] = 0;
  $47 = $minutia;
  $48 = $1;
  HEAP32[$48>>2] = $47;
  $0 = 0;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 return 0|0;
}
function _free_minutiae($minutiae) {
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $minutiae;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 4|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free_minutia($11);
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $0;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 _free($16);
 $17 = $0;
 _free($17);
 STACKTOP = sp;return;
}
function _free_minutia($minutia) {
 $minutia = $minutia|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $minutia;
 $1 = $0;
 $2 = (($1) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 44|0);
  $7 = HEAP32[$6>>2]|0;
  _free($7);
 }
 $8 = $0;
 $9 = (($8) + 48|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $15 = $0;
  _free($15);
  STACKTOP = sp;return;
 }
 $12 = $0;
 $13 = (($12) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 _free($14);
 $15 = $0;
 _free($15);
 STACKTOP = sp;return;
}
function _minutia_type($feature_pix) {
 $feature_pix = $feature_pix|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $type = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $feature_pix;
 $1 = $0;
 $2 = ($1|0)==(0);
 if ($2) {
  $type = 0;
 } else {
  $type = 1;
 }
 $3 = $type;
 STACKTOP = sp;return ($3|0);
}
function _is_minutia_appearing($x_loc,$y_loc,$x_edge,$y_edge) {
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $x_loc;
 $2 = $y_loc;
 $3 = $x_edge;
 $4 = $y_edge;
 $5 = $3;
 $6 = $1;
 $7 = ($5|0)<($6|0);
 do {
  if ($7) {
   $0 = 1;
  } else {
   $8 = $3;
   $9 = $1;
   $10 = ($8|0)>($9|0);
   if ($10) {
    $0 = 0;
    break;
   }
   $11 = $4;
   $12 = $2;
   $13 = ($11|0)<($12|0);
   if ($13) {
    $0 = 1;
    break;
   }
   $14 = $4;
   $15 = $2;
   $16 = ($14|0)>($15|0);
   if ($16) {
    $0 = 0;
    break;
   } else {
    $17 = HEAP32[_stderr>>2]|0;
    (_fprintf(($17|0),(1904|0),($vararg_buffer|0))|0);
    $0 = -240;
    break;
   }
  }
 } while(0);
 $18 = $0;
 STACKTOP = sp;return ($18|0);
}
function _process_horizontal_scan_minutia_V2($minutiae,$cx,$cy,$x2,$feature_id,$bdata,$iw,$ih,$pdirection_map,$plow_flow_map,$phigh_curve_map,$lfsparms) {
 $minutiae = $minutiae|0;
 $cx = $cx|0;
 $cy = $cy|0;
 $x2 = $x2|0;
 $feature_id = $feature_id|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $pdirection_map = $pdirection_map|0;
 $plow_flow_map = $plow_flow_map|0;
 $phigh_curve_map = $phigh_curve_map|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cmapval = 0, $dmapval = 0, $fmapval = 0, $idir = 0, $minutia = 0, $reliability = 0.0, $ret = 0, $x_edge = 0, $x_loc = 0, $y_edge = 0;
 var $y_loc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $minutia = sp + 44|0;
 $x_loc = sp + 40|0;
 $y_loc = sp + 36|0;
 $x_edge = sp + 32|0;
 $y_edge = sp + 28|0;
 $idir = sp + 24|0;
 $1 = $minutiae;
 $2 = $cx;
 $3 = $cy;
 $4 = $x2;
 $5 = $feature_id;
 $6 = $bdata;
 $7 = $iw;
 $8 = $ih;
 $9 = $pdirection_map;
 $10 = $plow_flow_map;
 $11 = $phigh_curve_map;
 $12 = $lfsparms;
 $13 = $2;
 $14 = $4;
 $15 = (($13) + ($14))|0;
 $16 = $15 >> 1;
 HEAP32[$x_loc>>2] = $16;
 $17 = HEAP32[$x_loc>>2]|0;
 HEAP32[$x_edge>>2] = $17;
 $18 = $5;
 $19 = (1016 + ($18<<5)|0);
 $20 = (($19) + 4|0);
 $21 = HEAPU8[$20>>0]|(HEAPU8[$20+1>>0]<<8)|(HEAPU8[$20+2>>0]<<16)|(HEAPU8[$20+3>>0]<<24);
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $3;
  $24 = (($23) + 1)|0;
  HEAP32[$y_loc>>2] = $24;
  $25 = $3;
  HEAP32[$y_edge>>2] = $25;
 } else {
  $26 = $3;
  HEAP32[$y_loc>>2] = $26;
  $27 = $3;
  $28 = (($27) + 1)|0;
  HEAP32[$y_edge>>2] = $28;
 }
 $29 = $9;
 $30 = HEAP32[$y_loc>>2]|0;
 $31 = $7;
 $32 = Math_imul($30, $31)|0;
 $33 = (($29) + ($32<<2)|0);
 $34 = HEAP32[$x_loc>>2]|0;
 $35 = (($33) + ($34<<2)|0);
 $36 = HEAP32[$35>>2]|0;
 $dmapval = $36;
 $37 = $10;
 $38 = HEAP32[$y_loc>>2]|0;
 $39 = $7;
 $40 = Math_imul($38, $39)|0;
 $41 = (($37) + ($40<<2)|0);
 $42 = HEAP32[$x_loc>>2]|0;
 $43 = (($41) + ($42<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 $fmapval = $44;
 $45 = $11;
 $46 = HEAP32[$y_loc>>2]|0;
 $47 = $7;
 $48 = Math_imul($46, $47)|0;
 $49 = (($45) + ($48<<2)|0);
 $50 = HEAP32[$x_loc>>2]|0;
 $51 = (($49) + ($50<<2)|0);
 $52 = HEAP32[$51>>2]|0;
 $cmapval = $52;
 $53 = $dmapval;
 $54 = ($53|0)==(-1);
 if ($54) {
  $0 = 2;
  $109 = $0;
  STACKTOP = sp;return ($109|0);
 }
 $55 = $cmapval;
 $56 = ($55|0)!=(0);
 do {
  if ($56) {
   $57 = HEAP32[$x_loc>>2]|0;
   $58 = HEAP32[$y_loc>>2]|0;
   $59 = HEAP32[$x_edge>>2]|0;
   $60 = HEAP32[$y_edge>>2]|0;
   $61 = $6;
   $62 = $7;
   $63 = $8;
   $64 = $10;
   $65 = $1;
   $66 = $12;
   $67 = (_adjust_high_curvature_minutia_V2($idir,$x_loc,$y_loc,$x_edge,$y_edge,$57,$58,$59,$60,$61,$62,$63,$64,$65,$66)|0);
   $ret = $67;
   $68 = ($67|0)!=(0);
   if (!($68)) {
    break;
   }
   $69 = $ret;
   $0 = $69;
   $109 = $0;
   STACKTOP = sp;return ($109|0);
  } else {
   $70 = $5;
   $71 = (1016 + ($70<<5)|0);
   $72 = (($71) + 4|0);
   $73 = HEAPU8[$72>>0]|(HEAPU8[$72+1>>0]<<8)|(HEAPU8[$72+2>>0]<<16)|(HEAPU8[$72+3>>0]<<24);
   $74 = $dmapval;
   $75 = $12;
   $76 = (($75) + 20|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (_get_low_curvature_direction(0,$73,$74,$77)|0);
   HEAP32[$idir>>2] = $78;
  }
 } while(0);
 $79 = $fmapval;
 $80 = ($79|0)!=(0);
 if ($80) {
  $reliability = 0.5;
 } else {
  $reliability = 0.989999999999999991118;
 }
 $81 = HEAP32[$x_loc>>2]|0;
 $82 = HEAP32[$y_loc>>2]|0;
 $83 = HEAP32[$x_edge>>2]|0;
 $84 = HEAP32[$y_edge>>2]|0;
 $85 = HEAP32[$idir>>2]|0;
 $86 = $reliability;
 $87 = $5;
 $88 = (1016 + ($87<<5)|0);
 $89 = HEAPU8[$88>>0]|(HEAPU8[$88+1>>0]<<8)|(HEAPU8[$88+2>>0]<<16)|(HEAPU8[$88+3>>0]<<24);
 $90 = $5;
 $91 = (1016 + ($90<<5)|0);
 $92 = (($91) + 4|0);
 $93 = HEAPU8[$92>>0]|(HEAPU8[$92+1>>0]<<8)|(HEAPU8[$92+2>>0]<<16)|(HEAPU8[$92+3>>0]<<24);
 $94 = $5;
 $95 = (_create_minutia($minutia,$81,$82,$83,$84,$85,$86,$89,$93,$94)|0);
 $ret = $95;
 $96 = ($95|0)!=(0);
 if ($96) {
  $97 = $ret;
  $0 = $97;
  $109 = $0;
  STACKTOP = sp;return ($109|0);
 }
 $98 = $1;
 $99 = HEAP32[$minutia>>2]|0;
 $100 = $dmapval;
 $101 = $6;
 $102 = $7;
 $103 = $8;
 $104 = $12;
 $105 = (_update_minutiae_V2($98,$99,0,$100,$101,$102,$103,$104)|0);
 $ret = $105;
 $106 = $ret;
 $107 = ($106|0)==(2);
 if ($107) {
  $108 = HEAP32[$minutia>>2]|0;
  _free_minutia($108);
 }
 $0 = 0;
 $109 = $0;
 STACKTOP = sp;return ($109|0);
}
function _process_vertical_scan_minutia_V2($minutiae,$cx,$cy,$y2,$feature_id,$bdata,$iw,$ih,$pdirection_map,$plow_flow_map,$phigh_curve_map,$lfsparms) {
 $minutiae = $minutiae|0;
 $cx = $cx|0;
 $cy = $cy|0;
 $y2 = $y2|0;
 $feature_id = $feature_id|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $pdirection_map = $pdirection_map|0;
 $plow_flow_map = $plow_flow_map|0;
 $phigh_curve_map = $phigh_curve_map|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cmapval = 0, $dmapval = 0, $fmapval = 0, $idir = 0, $minutia = 0, $reliability = 0.0, $ret = 0, $x_edge = 0, $x_loc = 0, $y_edge = 0;
 var $y_loc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $minutia = sp + 44|0;
 $x_loc = sp + 40|0;
 $y_loc = sp + 36|0;
 $x_edge = sp + 32|0;
 $y_edge = sp + 28|0;
 $idir = sp + 24|0;
 $1 = $minutiae;
 $2 = $cx;
 $3 = $cy;
 $4 = $y2;
 $5 = $feature_id;
 $6 = $bdata;
 $7 = $iw;
 $8 = $ih;
 $9 = $pdirection_map;
 $10 = $plow_flow_map;
 $11 = $phigh_curve_map;
 $12 = $lfsparms;
 $13 = $5;
 $14 = (1016 + ($13<<5)|0);
 $15 = (($14) + 4|0);
 $16 = HEAPU8[$15>>0]|(HEAPU8[$15+1>>0]<<8)|(HEAPU8[$15+2>>0]<<16)|(HEAPU8[$15+3>>0]<<24);
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = $2;
  $19 = (($18) + 1)|0;
  HEAP32[$x_loc>>2] = $19;
  $20 = $2;
  HEAP32[$x_edge>>2] = $20;
 } else {
  $21 = $2;
  HEAP32[$x_loc>>2] = $21;
  $22 = $2;
  $23 = (($22) + 1)|0;
  HEAP32[$x_edge>>2] = $23;
 }
 $24 = $3;
 $25 = $4;
 $26 = (($24) + ($25))|0;
 $27 = $26 >> 1;
 HEAP32[$y_loc>>2] = $27;
 $28 = HEAP32[$y_loc>>2]|0;
 HEAP32[$y_edge>>2] = $28;
 $29 = $9;
 $30 = HEAP32[$y_loc>>2]|0;
 $31 = $7;
 $32 = Math_imul($30, $31)|0;
 $33 = (($29) + ($32<<2)|0);
 $34 = HEAP32[$x_loc>>2]|0;
 $35 = (($33) + ($34<<2)|0);
 $36 = HEAP32[$35>>2]|0;
 $dmapval = $36;
 $37 = $10;
 $38 = HEAP32[$y_loc>>2]|0;
 $39 = $7;
 $40 = Math_imul($38, $39)|0;
 $41 = (($37) + ($40<<2)|0);
 $42 = HEAP32[$x_loc>>2]|0;
 $43 = (($41) + ($42<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 $fmapval = $44;
 $45 = $11;
 $46 = HEAP32[$y_loc>>2]|0;
 $47 = $7;
 $48 = Math_imul($46, $47)|0;
 $49 = (($45) + ($48<<2)|0);
 $50 = HEAP32[$x_loc>>2]|0;
 $51 = (($49) + ($50<<2)|0);
 $52 = HEAP32[$51>>2]|0;
 $cmapval = $52;
 $53 = $dmapval;
 $54 = ($53|0)==(-1);
 if ($54) {
  $0 = 2;
  $109 = $0;
  STACKTOP = sp;return ($109|0);
 }
 $55 = $cmapval;
 $56 = ($55|0)!=(0);
 do {
  if ($56) {
   $57 = HEAP32[$x_loc>>2]|0;
   $58 = HEAP32[$y_loc>>2]|0;
   $59 = HEAP32[$x_edge>>2]|0;
   $60 = HEAP32[$y_edge>>2]|0;
   $61 = $6;
   $62 = $7;
   $63 = $8;
   $64 = $10;
   $65 = $1;
   $66 = $12;
   $67 = (_adjust_high_curvature_minutia_V2($idir,$x_loc,$y_loc,$x_edge,$y_edge,$57,$58,$59,$60,$61,$62,$63,$64,$65,$66)|0);
   $ret = $67;
   $68 = ($67|0)!=(0);
   if (!($68)) {
    break;
   }
   $69 = $ret;
   $0 = $69;
   $109 = $0;
   STACKTOP = sp;return ($109|0);
  } else {
   $70 = $5;
   $71 = (1016 + ($70<<5)|0);
   $72 = (($71) + 4|0);
   $73 = HEAPU8[$72>>0]|(HEAPU8[$72+1>>0]<<8)|(HEAPU8[$72+2>>0]<<16)|(HEAPU8[$72+3>>0]<<24);
   $74 = $dmapval;
   $75 = $12;
   $76 = (($75) + 20|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (_get_low_curvature_direction(1,$73,$74,$77)|0);
   HEAP32[$idir>>2] = $78;
  }
 } while(0);
 $79 = $fmapval;
 $80 = ($79|0)!=(0);
 if ($80) {
  $reliability = 0.5;
 } else {
  $reliability = 0.989999999999999991118;
 }
 $81 = HEAP32[$x_loc>>2]|0;
 $82 = HEAP32[$y_loc>>2]|0;
 $83 = HEAP32[$x_edge>>2]|0;
 $84 = HEAP32[$y_edge>>2]|0;
 $85 = HEAP32[$idir>>2]|0;
 $86 = $reliability;
 $87 = $5;
 $88 = (1016 + ($87<<5)|0);
 $89 = HEAPU8[$88>>0]|(HEAPU8[$88+1>>0]<<8)|(HEAPU8[$88+2>>0]<<16)|(HEAPU8[$88+3>>0]<<24);
 $90 = $5;
 $91 = (1016 + ($90<<5)|0);
 $92 = (($91) + 4|0);
 $93 = HEAPU8[$92>>0]|(HEAPU8[$92+1>>0]<<8)|(HEAPU8[$92+2>>0]<<16)|(HEAPU8[$92+3>>0]<<24);
 $94 = $5;
 $95 = (_create_minutia($minutia,$81,$82,$83,$84,$85,$86,$89,$93,$94)|0);
 $ret = $95;
 $96 = ($95|0)!=(0);
 if ($96) {
  $97 = $ret;
  $0 = $97;
  $109 = $0;
  STACKTOP = sp;return ($109|0);
 }
 $98 = $1;
 $99 = HEAP32[$minutia>>2]|0;
 $100 = $dmapval;
 $101 = $6;
 $102 = $7;
 $103 = $8;
 $104 = $12;
 $105 = (_update_minutiae_V2($98,$99,1,$100,$101,$102,$103,$104)|0);
 $ret = $105;
 $106 = $ret;
 $107 = ($106|0)==(2);
 if ($107) {
  $108 = HEAP32[$minutia>>2]|0;
  _free_minutia($108);
 }
 $0 = 0;
 $109 = $0;
 STACKTOP = sp;return ($109|0);
}
function _get_low_curvature_direction($scan_dir,$appearing,$imapval,$ndirs) {
 $scan_dir = $scan_dir|0;
 $appearing = $appearing|0;
 $imapval = $imapval|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $idir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $scan_dir;
 $1 = $appearing;
 $2 = $imapval;
 $3 = $ndirs;
 $4 = $2;
 $idir = $4;
 $5 = $2;
 $6 = $3;
 $7 = $6 >> 1;
 $8 = ($5|0)<=($7|0);
 if ($8) {
  $9 = $0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = $1;
   $12 = ($11|0)!=(0);
   if ($12) {
    $13 = $3;
    $14 = $idir;
    $15 = (($14) + ($13))|0;
    $idir = $15;
   }
  } else {
   $16 = $1;
   $17 = ($16|0)!=(0);
   if (!($17)) {
    $18 = $3;
    $19 = $idir;
    $20 = (($19) + ($18))|0;
    $idir = $20;
   }
  }
  $33 = $idir;
  STACKTOP = sp;return ($33|0);
 } else {
  $21 = $0;
  $22 = ($21|0)==(0);
  if ($22) {
   $23 = $1;
   $24 = ($23|0)!=(0);
   if (!($24)) {
    $25 = $3;
    $26 = $idir;
    $27 = (($26) + ($25))|0;
    $idir = $27;
   }
  } else {
   $28 = $1;
   $29 = ($28|0)!=(0);
   if (!($29)) {
    $30 = $3;
    $31 = $idir;
    $32 = (($31) + ($30))|0;
    $idir = $32;
   }
  }
  $33 = $idir;
  STACKTOP = sp;return ($33|0);
 }
 return 0|0;
}
function _adjust_high_curvature_minutia_V2($oidir,$ox_loc,$oy_loc,$ox_edge,$oy_edge,$x_loc,$y_loc,$x_edge,$y_edge,$bdata,$iw,$ih,$plow_flow_map,$minutiae,$lfsparms) {
 $oidir = $oidir|0;
 $ox_loc = $ox_loc|0;
 $oy_loc = $oy_loc|0;
 $ox_edge = $ox_edge|0;
 $oy_edge = $oy_edge|0;
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $plow_flow_map = $plow_flow_map|0;
 $minutiae = $minutiae|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $angle_edge = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $feature_pix = 0, $half_contour = 0, $idir = 0, $mid_pix = 0, $mid_x = 0, $mid_y = 0, $min_i = 0, $min_theta = 0, $ncontour = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $contour_x = sp + 56|0;
 $contour_y = sp + 52|0;
 $contour_ex = sp + 48|0;
 $contour_ey = sp + 44|0;
 $ncontour = sp + 40|0;
 $min_i = sp + 36|0;
 $min_theta = sp;
 $1 = $oidir;
 $2 = $ox_loc;
 $3 = $oy_loc;
 $4 = $ox_edge;
 $5 = $oy_edge;
 $6 = $x_loc;
 $7 = $y_loc;
 $8 = $x_edge;
 $9 = $y_edge;
 $10 = $bdata;
 $11 = $iw;
 $12 = $ih;
 $13 = $plow_flow_map;
 $14 = $minutiae;
 $15 = $lfsparms;
 $16 = $15;
 $17 = (($16) + 168|0);
 $18 = HEAP32[$17>>2]|0;
 $half_contour = $18;
 $19 = $half_contour;
 $20 = $19 >> 1;
 $angle_edge = $20;
 $21 = $10;
 $22 = $7;
 $23 = $11;
 $24 = Math_imul($22, $23)|0;
 $25 = (($21) + ($24)|0);
 $26 = $6;
 $27 = (($25) + ($26)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $feature_pix = $29;
 $30 = $half_contour;
 $31 = $6;
 $32 = $7;
 $33 = $8;
 $34 = $9;
 $35 = $10;
 $36 = $11;
 $37 = $12;
 $38 = (_get_high_curvature_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$30,$31,$32,$33,$34,$35,$36,$37)|0);
 $ret = $38;
 $39 = ($38|0)!=(0);
 if ($39) {
  $40 = $ret;
  $41 = ($40|0)==(1);
  if (!($41)) {
   $73 = $ret;
   $0 = $73;
   $180 = $0;
   STACKTOP = sp;return ($180|0);
  }
  $42 = HEAP32[$contour_x>>2]|0;
  $43 = HEAP32[$contour_y>>2]|0;
  $44 = HEAP32[$ncontour>>2]|0;
  $45 = (_is_loop_clockwise($42,$43,$44,1)|0);
  $ret = $45;
  $46 = ($45|0)!=(0);
  if ($46) {
   $47 = HEAP32[$contour_x>>2]|0;
   $48 = HEAP32[$contour_y>>2]|0;
   $49 = HEAP32[$contour_ex>>2]|0;
   $50 = HEAP32[$contour_ey>>2]|0;
   _free_contour($47,$48,$49,$50);
   $51 = $ret;
   $52 = ($51|0)<(0);
   if ($52) {
    $53 = $ret;
    $0 = $53;
    $180 = $0;
    STACKTOP = sp;return ($180|0);
   } else {
    $0 = 2;
    $180 = $0;
    STACKTOP = sp;return ($180|0);
   }
  } else {
   $54 = $14;
   $55 = HEAP32[$contour_x>>2]|0;
   $56 = HEAP32[$contour_y>>2]|0;
   $57 = HEAP32[$contour_ex>>2]|0;
   $58 = HEAP32[$contour_ey>>2]|0;
   $59 = HEAP32[$ncontour>>2]|0;
   $60 = $10;
   $61 = $11;
   $62 = $12;
   $63 = $13;
   $64 = $15;
   $65 = (_process_loop_V2($54,$55,$56,$57,$58,$59,$60,$61,$62,$63,$64)|0);
   $ret = $65;
   $66 = HEAP32[$contour_x>>2]|0;
   $67 = HEAP32[$contour_y>>2]|0;
   $68 = HEAP32[$contour_ex>>2]|0;
   $69 = HEAP32[$contour_ey>>2]|0;
   _free_contour($66,$67,$68,$69);
   $70 = $ret;
   $71 = ($70|0)==(0);
   if ($71) {
    $0 = 2;
    $180 = $0;
    STACKTOP = sp;return ($180|0);
   } else {
    $72 = $ret;
    $0 = $72;
    $180 = $0;
    STACKTOP = sp;return ($180|0);
   }
  }
 } else {
  $74 = HEAP32[$ncontour>>2]|0;
  $75 = ($74|0)==(0);
  if ($75) {
   $0 = 2;
   $180 = $0;
   STACKTOP = sp;return ($180|0);
  }
  $76 = $angle_edge;
  $77 = HEAP32[$contour_x>>2]|0;
  $78 = HEAP32[$contour_y>>2]|0;
  $79 = HEAP32[$ncontour>>2]|0;
  $80 = (_min_contour_theta($min_i,$min_theta,$76,$77,$78,$79)|0);
  $ret = $80;
  $81 = ($80|0)!=(0);
  if ($81) {
   $82 = HEAP32[$contour_x>>2]|0;
   $83 = HEAP32[$contour_y>>2]|0;
   $84 = HEAP32[$contour_ex>>2]|0;
   $85 = HEAP32[$contour_ey>>2]|0;
   _free_contour($82,$83,$84,$85);
   $86 = $ret;
   $0 = $86;
   $180 = $0;
   STACKTOP = sp;return ($180|0);
  }
  $87 = +HEAPF64[$min_theta>>3];
  $88 = $15;
  $89 = (($88) + 160|0);
  $90 = +HEAPF64[$89>>3];
  $91 = $87 >= $90;
  if ($91) {
   $92 = HEAP32[$contour_x>>2]|0;
   $93 = HEAP32[$contour_y>>2]|0;
   $94 = HEAP32[$contour_ex>>2]|0;
   $95 = HEAP32[$contour_ey>>2]|0;
   _free_contour($92,$93,$94,$95);
   $0 = 2;
   $180 = $0;
   STACKTOP = sp;return ($180|0);
  }
  $96 = HEAP32[$min_i>>2]|0;
  $97 = $angle_edge;
  $98 = (($96) - ($97))|0;
  $99 = HEAP32[$contour_x>>2]|0;
  $100 = (($99) + ($98<<2)|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = HEAP32[$min_i>>2]|0;
  $103 = $angle_edge;
  $104 = (($102) + ($103))|0;
  $105 = HEAP32[$contour_x>>2]|0;
  $106 = (($105) + ($104<<2)|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($101) + ($107))|0;
  $109 = $108 >> 1;
  $mid_x = $109;
  $110 = HEAP32[$min_i>>2]|0;
  $111 = $angle_edge;
  $112 = (($110) - ($111))|0;
  $113 = HEAP32[$contour_y>>2]|0;
  $114 = (($113) + ($112<<2)|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = HEAP32[$min_i>>2]|0;
  $117 = $angle_edge;
  $118 = (($116) + ($117))|0;
  $119 = HEAP32[$contour_y>>2]|0;
  $120 = (($119) + ($118<<2)|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = (($115) + ($121))|0;
  $123 = $122 >> 1;
  $mid_y = $123;
  $124 = $10;
  $125 = $mid_y;
  $126 = $11;
  $127 = Math_imul($125, $126)|0;
  $128 = (($124) + ($127)|0);
  $129 = $mid_x;
  $130 = (($128) + ($129)|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = $131&255;
  $mid_pix = $132;
  $133 = $mid_pix;
  $134 = $feature_pix;
  $135 = ($133|0)!=($134|0);
  if ($135) {
   $136 = HEAP32[$contour_x>>2]|0;
   $137 = HEAP32[$contour_y>>2]|0;
   $138 = HEAP32[$contour_ex>>2]|0;
   $139 = HEAP32[$contour_ey>>2]|0;
   _free_contour($136,$137,$138,$139);
   $0 = 2;
   $180 = $0;
   STACKTOP = sp;return ($180|0);
  } else {
   $140 = HEAP32[$min_i>>2]|0;
   $141 = HEAP32[$contour_x>>2]|0;
   $142 = (($141) + ($140<<2)|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = HEAP32[$min_i>>2]|0;
   $145 = HEAP32[$contour_y>>2]|0;
   $146 = (($145) + ($144<<2)|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = $mid_x;
   $149 = $mid_y;
   $150 = $15;
   $151 = (($150) + 20|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = (_line2direction($143,$147,$148,$149,$152)|0);
   $idir = $153;
   $154 = $idir;
   $155 = $1;
   HEAP32[$155>>2] = $154;
   $156 = HEAP32[$min_i>>2]|0;
   $157 = HEAP32[$contour_x>>2]|0;
   $158 = (($157) + ($156<<2)|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = $2;
   HEAP32[$160>>2] = $159;
   $161 = HEAP32[$min_i>>2]|0;
   $162 = HEAP32[$contour_y>>2]|0;
   $163 = (($162) + ($161<<2)|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = $3;
   HEAP32[$165>>2] = $164;
   $166 = HEAP32[$min_i>>2]|0;
   $167 = HEAP32[$contour_ex>>2]|0;
   $168 = (($167) + ($166<<2)|0);
   $169 = HEAP32[$168>>2]|0;
   $170 = $4;
   HEAP32[$170>>2] = $169;
   $171 = HEAP32[$min_i>>2]|0;
   $172 = HEAP32[$contour_ey>>2]|0;
   $173 = (($172) + ($171<<2)|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = $5;
   HEAP32[$175>>2] = $174;
   $176 = HEAP32[$contour_x>>2]|0;
   $177 = HEAP32[$contour_y>>2]|0;
   $178 = HEAP32[$contour_ex>>2]|0;
   $179 = HEAP32[$contour_ey>>2]|0;
   _free_contour($176,$177,$178,$179);
   $0 = 0;
   $180 = $0;
   STACKTOP = sp;return ($180|0);
  }
 }
 return 0|0;
}
function _gen_quality_map($oqmap,$direction_map,$low_contrast_map,$low_flow_map,$high_curve_map,$map_w,$map_h) {
 $oqmap = $oqmap|0;
 $direction_map = $direction_map|0;
 $low_contrast_map = $low_contrast_map|0;
 $low_flow_map = $low_flow_map|0;
 $high_curve_map = $high_curve_map|0;
 $map_w = $map_w|0;
 $map_h = $map_h|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $QualMap = 0, $QualOffset = 0, $arrayPos = 0, $arrayPos2 = 0, $compX = 0, $compY = 0, $thisX = 0, $thisY = 0, $vararg_buffer = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $1 = $oqmap;
 $2 = $direction_map;
 $3 = $low_contrast_map;
 $4 = $low_flow_map;
 $5 = $high_curve_map;
 $6 = $map_w;
 $7 = $map_h;
 $8 = $6;
 $9 = $7;
 $10 = Math_imul($8, $9)|0;
 $11 = $10<<2;
 $12 = (_malloc($11)|0);
 $QualMap = $12;
 $13 = $QualMap;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(1968|0),($vararg_buffer|0))|0);
  $0 = -2;
  $130 = $0;
  STACKTOP = sp;return ($130|0);
 }
 $thisY = 0;
 while(1) {
  $16 = $thisY;
  $17 = $7;
  $18 = ($16|0)<($17|0);
  if (!($18)) {
   break;
  }
  $thisX = 0;
  while(1) {
   $19 = $thisX;
   $20 = $6;
   $21 = ($19|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = $thisY;
   $23 = $6;
   $24 = Math_imul($22, $23)|0;
   $25 = $thisX;
   $26 = (($24) + ($25))|0;
   $arrayPos = $26;
   $27 = $arrayPos;
   $28 = $3;
   $29 = (($28) + ($27<<2)|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0);
   if ($31) {
    label = 9;
   } else {
    $32 = $arrayPos;
    $33 = $2;
    $34 = (($33) + ($32<<2)|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)<(0);
    if ($36) {
     label = 9;
    } else {
     $40 = $arrayPos;
     $41 = $4;
     $42 = (($41) + ($40<<2)|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = ($43|0)!=(0);
     if ($44) {
      label = 12;
     } else {
      $45 = $arrayPos;
      $46 = $5;
      $47 = (($46) + ($45<<2)|0);
      $48 = HEAP32[$47>>2]|0;
      $49 = ($48|0)!=(0);
      if ($49) {
       label = 12;
      } else {
       $53 = $arrayPos;
       $54 = $QualMap;
       $55 = (($54) + ($53<<2)|0);
       HEAP32[$55>>2] = 4;
      }
     }
     if ((label|0) == 12) {
      label = 0;
      $50 = $arrayPos;
      $51 = $QualMap;
      $52 = (($51) + ($50<<2)|0);
      HEAP32[$52>>2] = 3;
     }
     $56 = $thisY;
     $57 = ($56|0)<(2);
     if ($57) {
      label = 18;
     } else {
      $58 = $thisY;
      $59 = $7;
      $60 = (($59) - 1)|0;
      $61 = (($60) - 2)|0;
      $62 = ($58|0)>($61|0);
      if ($62) {
       label = 18;
      } else {
       $63 = $thisX;
       $64 = ($63|0)<(2);
       if ($64) {
        label = 18;
       } else {
        $65 = $thisX;
        $66 = $6;
        $67 = (($66) - 1)|0;
        $68 = (($67) - 2)|0;
        $69 = ($65|0)>($68|0);
        if ($69) {
         label = 18;
        } else {
         $QualOffset = 0;
         $73 = $thisY;
         $74 = (($73) - 2)|0;
         $compY = $74;
         while(1) {
          $75 = $compY;
          $76 = $thisY;
          $77 = (($76) + 2)|0;
          $78 = ($75|0)<=($77|0);
          if (!($78)) {
           break;
          }
          $79 = $thisX;
          $80 = (($79) - 2)|0;
          $compX = $80;
          while(1) {
           $81 = $compX;
           $82 = $thisX;
           $83 = (($82) + 2)|0;
           $84 = ($81|0)<=($83|0);
           if (!($84)) {
            break;
           }
           $85 = $compY;
           $86 = $6;
           $87 = Math_imul($85, $86)|0;
           $88 = $compX;
           $89 = (($87) + ($88))|0;
           $arrayPos2 = $89;
           $90 = $arrayPos2;
           $91 = $3;
           $92 = (($91) + ($90<<2)|0);
           $93 = HEAP32[$92>>2]|0;
           $94 = ($93|0)!=(0);
           if ($94) {
            label = 25;
            break;
           }
           $95 = $arrayPos2;
           $96 = $2;
           $97 = (($96) + ($95<<2)|0);
           $98 = HEAP32[$97>>2]|0;
           $99 = ($98|0)<(0);
           if ($99) {
            label = 25;
            break;
           }
           $100 = $arrayPos2;
           $101 = $4;
           $102 = (($101) + ($100<<2)|0);
           $103 = HEAP32[$102>>2]|0;
           $104 = ($103|0)!=(0);
           if ($104) {
            label = 28;
           } else {
            $105 = $arrayPos2;
            $106 = $5;
            $107 = (($106) + ($105<<2)|0);
            $108 = HEAP32[$107>>2]|0;
            $109 = ($108|0)!=(0);
            if ($109) {
             label = 28;
            }
           }
           if ((label|0) == 28) {
            label = 0;
            $110 = $QualOffset;
            $111 = ($110|0)<(-1);
            if ($111) {
             $112 = $QualOffset;
             $113 = $112;
            } else {
             $113 = -1;
            }
            $QualOffset = $113;
           }
           $114 = $compX;
           $115 = (($114) + 1)|0;
           $compX = $115;
          }
          if ((label|0) == 25) {
           label = 0;
           $QualOffset = -2;
          }
          $116 = $compY;
          $117 = (($116) + 1)|0;
          $compY = $117;
         }
         $118 = $QualOffset;
         $119 = $arrayPos;
         $120 = $QualMap;
         $121 = (($120) + ($119<<2)|0);
         $122 = HEAP32[$121>>2]|0;
         $123 = (($122) + ($118))|0;
         HEAP32[$121>>2] = $123;
        }
       }
      }
     }
     if ((label|0) == 18) {
      label = 0;
      $70 = $arrayPos;
      $71 = $QualMap;
      $72 = (($71) + ($70<<2)|0);
      HEAP32[$72>>2] = 1;
     }
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $37 = $arrayPos;
    $38 = $QualMap;
    $39 = (($38) + ($37<<2)|0);
    HEAP32[$39>>2] = 0;
   }
   $124 = $thisX;
   $125 = (($124) + 1)|0;
   $thisX = $125;
  }
  $126 = $thisY;
  $127 = (($126) + 1)|0;
  $thisY = $127;
 }
 $128 = $QualMap;
 $129 = $1;
 HEAP32[$129>>2] = $128;
 $0 = 0;
 $130 = $0;
 STACKTOP = sp;return ($130|0);
}
function _combined_minutia_quality($minutiae,$quality_map,$mw,$mh,$blocksize,$idata,$iw,$ih,$id,$ppmm) {
 $minutiae = $minutiae|0;
 $quality_map = $quality_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $blocksize = $blocksize|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $id = $id|0;
 $ppmm = +$ppmm;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $gs_reliability = 0.0, $i = 0, $index = 0, $minutia = 0, $pquality_map = 0, $qmap_value = 0, $radius_pix = 0, $reliability = 0.0, $ret = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer10 = sp + 64|0;
 $vararg_buffer7 = sp + 56|0;
 $vararg_buffer5 = sp + 48|0;
 $vararg_buffer3 = sp + 40|0;
 $vararg_buffer1 = sp + 32|0;
 $vararg_buffer = sp + 24|0;
 $pquality_map = sp + 76|0;
 $1 = $minutiae;
 $2 = $quality_map;
 $3 = $mw;
 $4 = $mh;
 $5 = $blocksize;
 $6 = $idata;
 $7 = $iw;
 $8 = $ih;
 $9 = $id;
 $10 = $ppmm;
 $11 = $9;
 $12 = ($11|0)!=(8);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  (_fprintf(($13|0),(2016|0),($vararg_buffer|0))|0);
  $14 = HEAP32[_stderr>>2]|0;
  $15 = $9;
  HEAP32[$vararg_buffer1>>2] = $15;
  (_fprintf(($14|0),(2056|0),($vararg_buffer1|0))|0);
  $16 = HEAP32[_stderr>>2]|0;
  (_fprintf(($16|0),(2096|0),($vararg_buffer3|0))|0);
  $0 = -2;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 $17 = $10;
 $18 = 0.558659217877094937954 * $17;
 $19 = $18 < 0.0;
 if ($19) {
  $20 = $10;
  $21 = 0.558659217877094937954 * $20;
  $22 = $21 - 0.5;
  $27 = $22;
 } else {
  $23 = $10;
  $24 = 0.558659217877094937954 * $23;
  $25 = $24 + 0.5;
  $27 = $25;
 }
 $26 = (~~(($27)));
 $radius_pix = $26;
 $28 = $7;
 $29 = $8;
 $30 = $2;
 $31 = $3;
 $32 = $4;
 $33 = $5;
 $34 = (_pixelize_map($pquality_map,$28,$29,$30,$31,$32,$33)|0);
 $ret = $34;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = $ret;
  $0 = $36;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 $i = 0;
 L13: while(1) {
  $37 = $i;
  $38 = $1;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($37|0)<($40|0);
  if (!($41)) {
   label = 19;
   break;
  }
  $42 = $i;
  $43 = $1;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  $minutia = $47;
  $48 = $minutia;
  $49 = $6;
  $50 = $7;
  $51 = $8;
  $52 = $radius_pix;
  $53 = (+_grayscale_reliability($48,$49,$50,$51,$52));
  $gs_reliability = $53;
  $54 = $minutia;
  $55 = (($54) + 4|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $7;
  $58 = Math_imul($56, $57)|0;
  $59 = $minutia;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + ($60))|0;
  $index = $61;
  $62 = $index;
  $63 = HEAP32[$pquality_map>>2]|0;
  $64 = (($63) + ($62<<2)|0);
  $65 = HEAP32[$64>>2]|0;
  $qmap_value = $65;
  $66 = $qmap_value;
  switch ($66|0) {
  case 3:  {
   $70 = $gs_reliability;
   $71 = 0.239999999999999991118 * $70;
   $72 = 0.25 + $71;
   $reliability = $72;
   break;
  }
  case 2:  {
   $73 = $gs_reliability;
   $74 = 0.140000000000000013323 * $73;
   $75 = 0.100000000000000005551 + $74;
   $reliability = $75;
   break;
  }
  case 4:  {
   $67 = $gs_reliability;
   $68 = 0.489999999999999991118 * $67;
   $69 = 0.5 + $68;
   $reliability = $69;
   break;
  }
  case 1:  {
   $76 = $gs_reliability;
   $77 = 0.0400000000000000008327 * $76;
   $78 = 0.0500000000000000027756 + $77;
   $reliability = $78;
   break;
  }
  case 0:  {
   $reliability = 0.0100000000000000002082;
   break;
  }
  default: {
   label = 16;
   break L13;
  }
  }
  $84 = $reliability;
  $85 = $minutia;
  $86 = (($85) + 24|0);
  HEAPF64[$86>>3] = $84;
  $87 = $i;
  $88 = (($87) + 1)|0;
  $i = $88;
 }
 if ((label|0) == 16) {
  $79 = HEAP32[_stderr>>2]|0;
  (_fprintf(($79|0),(2016|0),($vararg_buffer5|0))|0);
  $80 = HEAP32[_stderr>>2]|0;
  $81 = $qmap_value;
  HEAP32[$vararg_buffer7>>2] = $81;
  (_fprintf(($80|0),(2120|0),($vararg_buffer7|0))|0);
  $82 = HEAP32[_stderr>>2]|0;
  (_fprintf(($82|0),(2160|0),($vararg_buffer10|0))|0);
  $83 = HEAP32[$pquality_map>>2]|0;
  _free($83);
  $0 = -3;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 else if ((label|0) == 19) {
  $89 = HEAP32[$pquality_map>>2]|0;
  _free($89);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 return 0|0;
}
function _grayscale_reliability($minutia,$idata,$iw,$ih,$radius_pix) {
 $minutia = $minutia|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $radius_pix = $radius_pix|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mean = 0, $reliability = 0.0, $stdev = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $mean = sp + 16|0;
 $stdev = sp + 8|0;
 $0 = $minutia;
 $1 = $idata;
 $2 = $iw;
 $3 = $ih;
 $4 = $radius_pix;
 $5 = $0;
 $6 = $1;
 $7 = $2;
 $8 = $3;
 $9 = $4;
 _get_neighborhood_stats($mean,$stdev,$5,$6,$7,$8,$9);
 $10 = +HEAPF64[$stdev>>3];
 $11 = $10 > 64.0;
 if ($11) {
  $20 = 1.0;
 } else {
  $12 = +HEAPF64[$stdev>>3];
  $13 = $12 / 64.0;
  $20 = $13;
 }
 $14 = +HEAPF64[$mean>>3];
 $15 = $14 - 127.0;
 $16 = (+Math_abs((+$15)));
 $17 = $16 / 127.0;
 $18 = 1.0 - $17;
 $19 = $20 < $18;
 if (!($19)) {
  $25 = +HEAPF64[$mean>>3];
  $26 = $25 - 127.0;
  $27 = (+Math_abs((+$26)));
  $28 = $27 / 127.0;
  $29 = 1.0 - $28;
  $30 = $29;
  $reliability = $30;
  $31 = $reliability;
  STACKTOP = sp;return (+$31);
 }
 $21 = +HEAPF64[$stdev>>3];
 $22 = $21 > 64.0;
 if ($22) {
  $32 = 1.0;
 } else {
  $23 = +HEAPF64[$stdev>>3];
  $24 = $23 / 64.0;
  $32 = $24;
 }
 $30 = $32;
 $reliability = $30;
 $31 = $reliability;
 STACKTOP = sp;return (+$31);
}
function _get_neighborhood_stats($mean,$stdev,$minutia,$idata,$iw,$ih,$radius_pix) {
 $mean = $mean|0;
 $stdev = $stdev|0;
 $minutia = $minutia|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $radius_pix = $radius_pix|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0.0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0.0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0, $99 = 0, $cols = 0, $histogram = 0, $i = 0, $n = 0, $rows = 0, $sumX = 0, $sumXX = 0, $x = 0;
 var $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1088|0;
 $histogram = sp;
 $0 = $mean;
 $1 = $stdev;
 $2 = $minutia;
 $3 = $idata;
 $4 = $iw;
 $5 = $ih;
 $6 = $radius_pix;
 $n = 0;
 $sumX = 0;
 $sumXX = 0;
 _memset(($histogram|0),0,1024)|0;
 $7 = $2;
 $8 = HEAP32[$7>>2]|0;
 $x = $8;
 $9 = $2;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $y = $11;
 $12 = $x;
 $13 = $6;
 $14 = ($12|0)<($13|0);
 if (!($14)) {
  $15 = $x;
  $16 = $4;
  $17 = $6;
  $18 = (($16) - ($17))|0;
  $19 = (($18) - 1)|0;
  $20 = ($15|0)>($19|0);
  if (!($20)) {
   $21 = $y;
   $22 = $6;
   $23 = ($21|0)<($22|0);
   if (!($23)) {
    $24 = $y;
    $25 = $5;
    $26 = $6;
    $27 = (($25) - ($26))|0;
    $28 = (($27) - 1)|0;
    $29 = ($24|0)>($28|0);
    if (!($29)) {
     $32 = $y;
     $33 = $6;
     $34 = (($32) - ($33))|0;
     $rows = $34;
     while(1) {
      $35 = $rows;
      $36 = $y;
      $37 = $6;
      $38 = (($36) + ($37))|0;
      $39 = ($35|0)<=($38|0);
      if (!($39)) {
       break;
      }
      $40 = $x;
      $41 = $6;
      $42 = (($40) - ($41))|0;
      $cols = $42;
      while(1) {
       $43 = $cols;
       $44 = $x;
       $45 = $6;
       $46 = (($44) + ($45))|0;
       $47 = ($43|0)<=($46|0);
       if (!($47)) {
        break;
       }
       $48 = $3;
       $49 = $rows;
       $50 = $4;
       $51 = Math_imul($49, $50)|0;
       $52 = (($48) + ($51)|0);
       $53 = $cols;
       $54 = (($52) + ($53)|0);
       $55 = HEAP8[$54>>0]|0;
       $56 = $55&255;
       $57 = (($histogram) + ($56<<2)|0);
       $58 = HEAP32[$57>>2]|0;
       $59 = (($58) + 1)|0;
       HEAP32[$57>>2] = $59;
       $60 = $cols;
       $61 = (($60) + 1)|0;
       $cols = $61;
      }
      $62 = $rows;
      $63 = (($62) + 1)|0;
      $rows = $63;
     }
     $i = 0;
     while(1) {
      $64 = $i;
      $65 = ($64|0)<(256);
      if (!($65)) {
       break;
      }
      $66 = $i;
      $67 = (($histogram) + ($66<<2)|0);
      $68 = HEAP32[$67>>2]|0;
      $69 = ($68|0)!=(0);
      if ($69) {
       $70 = $i;
       $71 = $i;
       $72 = (($histogram) + ($71<<2)|0);
       $73 = HEAP32[$72>>2]|0;
       $74 = Math_imul($70, $73)|0;
       $75 = $sumX;
       $76 = (($75) + ($74))|0;
       $sumX = $76;
       $77 = $i;
       $78 = $i;
       $79 = Math_imul($77, $78)|0;
       $80 = $i;
       $81 = (($histogram) + ($80<<2)|0);
       $82 = HEAP32[$81>>2]|0;
       $83 = Math_imul($79, $82)|0;
       $84 = $sumXX;
       $85 = (($84) + ($83))|0;
       $sumXX = $85;
       $86 = $i;
       $87 = (($histogram) + ($86<<2)|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = $n;
       $90 = (($89) + ($88))|0;
       $n = $90;
      }
      $91 = $i;
      $92 = (($91) + 1)|0;
      $i = $92;
     }
     $93 = $sumX;
     $94 = (+($93|0));
     $95 = $n;
     $96 = (+($95|0));
     $97 = $94 / $96;
     $98 = $0;
     HEAPF64[$98>>3] = $97;
     $99 = $sumXX;
     $100 = (+($99|0));
     $101 = $n;
     $102 = (+($101|0));
     $103 = $100 / $102;
     $104 = $0;
     $105 = +HEAPF64[$104>>3];
     $106 = $0;
     $107 = +HEAPF64[$106>>3];
     $108 = $105 * $107;
     $109 = $103 - $108;
     $110 = (+Math_sqrt((+$109)));
     $111 = $1;
     HEAPF64[$111>>3] = $110;
     STACKTOP = sp;return;
    }
   }
  }
 }
 $30 = $0;
 HEAPF64[$30>>3] = 0.0;
 $31 = $1;
 HEAPF64[$31>>3] = 0.0;
 STACKTOP = sp;return;
}
function _write_text_results($oroot,$m1flag,$iw,$ih,$minutiae,$quality_map,$direction_map,$low_contrast_map,$low_flow_map,$high_curve_map,$map_w,$map_h) {
 $oroot = $oroot|0;
 $m1flag = $m1flag|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $minutiae = $minutiae|0;
 $quality_map = $quality_map|0;
 $direction_map = $direction_map|0;
 $low_contrast_map = $low_contrast_map|0;
 $low_flow_map = $low_flow_map|0;
 $high_curve_map = $high_curve_map|0;
 $map_w = $map_w|0;
 $map_h = $map_h|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $fp = 0, $ofile = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer2 = 0, $vararg_buffer20 = 0, $vararg_buffer23 = 0, $vararg_buffer26 = 0, $vararg_buffer30 = 0, $vararg_buffer33 = 0, $vararg_buffer36 = 0, $vararg_buffer40 = 0, $vararg_buffer43 = 0, $vararg_buffer46 = 0, $vararg_buffer5 = 0, $vararg_buffer50 = 0;
 var $vararg_buffer53 = 0, $vararg_buffer56 = 0, $vararg_buffer60 = 0, $vararg_buffer63 = 0, $vararg_buffer9 = 0, $vararg_ptr1 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr29 = 0, $vararg_ptr39 = 0, $vararg_ptr49 = 0, $vararg_ptr59 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 4320|0;
 $vararg_buffer63 = sp + 152|0;
 $vararg_buffer60 = sp + 144|0;
 $vararg_buffer56 = sp + 136|0;
 $vararg_buffer53 = sp + 128|0;
 $vararg_buffer50 = sp + 120|0;
 $vararg_buffer46 = sp + 112|0;
 $vararg_buffer43 = sp + 104|0;
 $vararg_buffer40 = sp + 96|0;
 $vararg_buffer36 = sp + 88|0;
 $vararg_buffer33 = sp + 80|0;
 $vararg_buffer30 = sp + 72|0;
 $vararg_buffer26 = sp + 64|0;
 $vararg_buffer23 = sp + 56|0;
 $vararg_buffer20 = sp + 48|0;
 $vararg_buffer16 = sp + 40|0;
 $vararg_buffer12 = sp + 32|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $ofile = sp + 216|0;
 $1 = $oroot;
 $2 = $m1flag;
 $3 = $iw;
 $4 = $ih;
 $5 = $minutiae;
 $6 = $quality_map;
 $7 = $direction_map;
 $8 = $low_contrast_map;
 $9 = $low_flow_map;
 $10 = $high_curve_map;
 $11 = $map_w;
 $12 = $map_h;
 $13 = $1;
 HEAP32[$vararg_buffer>>2] = $13;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 2192;
 (_sprintf($ofile,2184,$vararg_buffer)|0);
 $14 = (_fopen(($ofile|0),(2200|0))|0);
 $fp = $14;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $ofile;
  (_fprintf(($16|0),(2208|0),($vararg_buffer2|0))|0);
  $0 = -2;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $17 = $fp;
 $18 = $3;
 $19 = $4;
 HEAP32[$vararg_buffer5>>2] = $18;
 $vararg_ptr8 = (($vararg_buffer5) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $19;
 (_fprintf(($17|0),(2256|0),($vararg_buffer5|0))|0);
 $20 = $fp;
 $21 = $5;
 _dump_minutiae($20,$21);
 $22 = $fp;
 $23 = (_fclose(($22|0))|0);
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $ofile;
  (_fprintf(($25|0),(2280|0),($vararg_buffer9|0))|0);
  $0 = -3;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $26 = $1;
 HEAP32[$vararg_buffer12>>2] = $26;
 $vararg_ptr15 = (($vararg_buffer12) + 4|0);
 HEAP32[$vararg_ptr15>>2] = 2328;
 (_sprintf($ofile,2184,$vararg_buffer12)|0);
 $27 = $2;
 $28 = ($27|0)!=(0);
 do {
  if ($28) {
   $29 = $5;
   $30 = $3;
   $31 = $4;
   $32 = (_write_minutiae_XYTQ($ofile,1,$29,$30,$31)|0);
   $ret = $32;
   $33 = ($32|0)!=(0);
   if (!($33)) {
    break;
   }
   $34 = $ret;
   $0 = $34;
   $101 = $0;
   STACKTOP = sp;return ($101|0);
  } else {
   $35 = $5;
   $36 = $3;
   $37 = $4;
   $38 = (_write_minutiae_XYTQ($ofile,0,$35,$36,$37)|0);
   $ret = $38;
   $39 = ($38|0)!=(0);
   if (!($39)) {
    break;
   }
   $40 = $ret;
   $0 = $40;
   $101 = $0;
   STACKTOP = sp;return ($101|0);
  }
 } while(0);
 $41 = $1;
 HEAP32[$vararg_buffer16>>2] = $41;
 $vararg_ptr19 = (($vararg_buffer16) + 4|0);
 HEAP32[$vararg_ptr19>>2] = 2336;
 (_sprintf($ofile,2184,$vararg_buffer16)|0);
 $42 = (_fopen(($ofile|0),(2200|0))|0);
 $fp = $42;
 $43 = ($42|0)==(0|0);
 if ($43) {
  $44 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer20>>2] = $ofile;
  (_fprintf(($44|0),(2208|0),($vararg_buffer20|0))|0);
  $0 = -4;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $45 = $fp;
 $46 = $6;
 $47 = $11;
 $48 = $12;
 _dump_map($45,$46,$47,$48);
 $49 = $fp;
 $50 = (_fclose(($49|0))|0);
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer23>>2] = $ofile;
  (_fprintf(($52|0),(2280|0),($vararg_buffer23|0))|0);
  $0 = -5;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $53 = $1;
 HEAP32[$vararg_buffer26>>2] = $53;
 $vararg_ptr29 = (($vararg_buffer26) + 4|0);
 HEAP32[$vararg_ptr29>>2] = 2344;
 (_sprintf($ofile,2184,$vararg_buffer26)|0);
 $54 = (_fopen(($ofile|0),(2200|0))|0);
 $fp = $54;
 $55 = ($54|0)==(0|0);
 if ($55) {
  $56 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer30>>2] = $ofile;
  (_fprintf(($56|0),(2208|0),($vararg_buffer30|0))|0);
  $0 = -6;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $57 = $fp;
 $58 = $7;
 $59 = $11;
 $60 = $12;
 _dump_map($57,$58,$59,$60);
 $61 = $fp;
 $62 = (_fclose(($61|0))|0);
 $63 = ($62|0)!=(0);
 if ($63) {
  $64 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer33>>2] = $ofile;
  (_fprintf(($64|0),(2280|0),($vararg_buffer33|0))|0);
  $0 = -7;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $65 = $1;
 HEAP32[$vararg_buffer36>>2] = $65;
 $vararg_ptr39 = (($vararg_buffer36) + 4|0);
 HEAP32[$vararg_ptr39>>2] = 2352;
 (_sprintf($ofile,2184,$vararg_buffer36)|0);
 $66 = (_fopen(($ofile|0),(2200|0))|0);
 $fp = $66;
 $67 = ($66|0)==(0|0);
 if ($67) {
  $68 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer40>>2] = $ofile;
  (_fprintf(($68|0),(2208|0),($vararg_buffer40|0))|0);
  $0 = -8;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $69 = $fp;
 $70 = $8;
 $71 = $11;
 $72 = $12;
 _dump_map($69,$70,$71,$72);
 $73 = $fp;
 $74 = (_fclose(($73|0))|0);
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer43>>2] = $ofile;
  (_fprintf(($76|0),(2280|0),($vararg_buffer43|0))|0);
  $0 = -9;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $77 = $1;
 HEAP32[$vararg_buffer46>>2] = $77;
 $vararg_ptr49 = (($vararg_buffer46) + 4|0);
 HEAP32[$vararg_ptr49>>2] = 2360;
 (_sprintf($ofile,2184,$vararg_buffer46)|0);
 $78 = (_fopen(($ofile|0),(2200|0))|0);
 $fp = $78;
 $79 = ($78|0)==(0|0);
 if ($79) {
  $80 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer50>>2] = $ofile;
  (_fprintf(($80|0),(2208|0),($vararg_buffer50|0))|0);
  $0 = -10;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $81 = $fp;
 $82 = $9;
 $83 = $11;
 $84 = $12;
 _dump_map($81,$82,$83,$84);
 $85 = $fp;
 $86 = (_fclose(($85|0))|0);
 $87 = ($86|0)!=(0);
 if ($87) {
  $88 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer53>>2] = $ofile;
  (_fprintf(($88|0),(2280|0),($vararg_buffer53|0))|0);
  $0 = -11;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $89 = $1;
 HEAP32[$vararg_buffer56>>2] = $89;
 $vararg_ptr59 = (($vararg_buffer56) + 4|0);
 HEAP32[$vararg_ptr59>>2] = 2368;
 (_sprintf($ofile,2184,$vararg_buffer56)|0);
 $90 = (_fopen(($ofile|0),(2200|0))|0);
 $fp = $90;
 $91 = ($90|0)==(0|0);
 if ($91) {
  $92 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer60>>2] = $ofile;
  (_fprintf(($92|0),(2208|0),($vararg_buffer60|0))|0);
  $0 = -12;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $93 = $fp;
 $94 = $10;
 $95 = $11;
 $96 = $12;
 _dump_map($93,$94,$95,$96);
 $97 = $fp;
 $98 = (_fclose(($97|0))|0);
 $99 = ($98|0)!=(0);
 if ($99) {
  $100 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer63>>2] = $ofile;
  (_fprintf(($100|0),(2280|0),($vararg_buffer63|0))|0);
  $0 = -13;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 } else {
  $0 = 0;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 return 0|0;
}
function _write_minutiae_XYTQ($ofile,$reptype,$minutiae,$iw,$ih) {
 $ofile = $ofile|0;
 $reptype = $reptype|0;
 $minutiae = $minutiae|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0.0;
 var $45 = 0.0, $46 = 0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $fp = 0, $i = 0, $minutia = 0, $oq = 0, $ot = 0, $ox = 0, $oy = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $ox = sp + 60|0;
 $oy = sp + 56|0;
 $ot = sp + 52|0;
 $1 = $ofile;
 $2 = $reptype;
 $3 = $minutiae;
 $4 = $iw;
 $5 = $ih;
 $6 = $1;
 $7 = (_fopen(($6|0),(2200|0))|0);
 $fp = $7;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  $10 = $1;
  HEAP32[$vararg_buffer>>2] = $10;
  (_fprintf(($9|0),(2376|0),($vararg_buffer|0))|0);
  $0 = -2;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $3;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $i;
  $17 = $3;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $minutia = $21;
  $22 = $2;
  if ((($22|0) == 1)) {
   $23 = $minutia;
   _lfs2m1_minutia_XYT($ox,$oy,$ot,$23);
  } else if ((($22|0) == 0)) {
   $24 = $minutia;
   $25 = $4;
   $26 = $5;
   _lfs2nist_minutia_XYT($ox,$oy,$ot,$24,$25,$26);
  } else {
   label = 8;
   break;
  }
  $31 = $minutia;
  $32 = (($31) + 24|0);
  $33 = +HEAPF64[$32>>3];
  $34 = $33 * 100.0;
  $35 = $34 < 0.0;
  if ($35) {
   $36 = $minutia;
   $37 = (($36) + 24|0);
   $38 = +HEAPF64[$37>>3];
   $39 = $38 * 100.0;
   $40 = $39 - 0.5;
   $47 = $40;
  } else {
   $41 = $minutia;
   $42 = (($41) + 24|0);
   $43 = +HEAPF64[$42>>3];
   $44 = $43 * 100.0;
   $45 = $44 + 0.5;
   $47 = $45;
  }
  $46 = (~~(($47)));
  $oq = $46;
  $48 = $fp;
  $49 = HEAP32[$ox>>2]|0;
  $50 = HEAP32[$oy>>2]|0;
  $51 = HEAP32[$ot>>2]|0;
  $52 = $oq;
  HEAP32[$vararg_buffer6>>2] = $49;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $50;
  $vararg_ptr10 = (($vararg_buffer6) + 8|0);
  HEAP32[$vararg_ptr10>>2] = $51;
  $vararg_ptr11 = (($vararg_buffer6) + 12|0);
  HEAP32[$vararg_ptr11>>2] = $52;
  (_fprintf(($48|0),(2496|0),($vararg_buffer6|0))|0);
  $53 = $i;
  $54 = (($53) + 1)|0;
  $i = $54;
 }
 if ((label|0) == 8) {
  $27 = HEAP32[_stderr>>2]|0;
  (_fprintf(($27|0),(2424|0),($vararg_buffer1|0))|0);
  $28 = HEAP32[_stderr>>2]|0;
  $29 = $2;
  HEAP32[$vararg_buffer3>>2] = $29;
  (_fprintf(($28|0),(2456|0),($vararg_buffer3|0))|0);
  $30 = $fp;
  (_fclose(($30|0))|0);
  $0 = -4;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 $55 = $fp;
 $56 = (_fclose(($55|0))|0);
 $57 = ($56|0)!=(0);
 if ($57) {
  $58 = HEAP32[_stderr>>2]|0;
  $59 = $1;
  HEAP32[$vararg_buffer12>>2] = $59;
  (_fprintf(($58|0),(2376|0),($vararg_buffer12|0))|0);
  $0 = -5;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 } else {
  $0 = 0;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 return 0|0;
}
function _dump_map($fpout,$map,$mw,$mh) {
 $fpout = $fpout|0;
 $map = $map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iptr = 0, $mx = 0, $my = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $fpout;
 $1 = $map;
 $2 = $mw;
 $3 = $mh;
 $4 = $1;
 $iptr = $4;
 $my = 0;
 while(1) {
  $5 = $my;
  $6 = $3;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $mx = 0;
  while(1) {
   $8 = $mx;
   $9 = $2;
   $10 = ($8|0)<($9|0);
   if (!($10)) {
    break;
   }
   $11 = $0;
   $12 = $iptr;
   $13 = (($12) + 4|0);
   $iptr = $13;
   $14 = HEAP32[$12>>2]|0;
   HEAP32[$vararg_buffer>>2] = $14;
   (_fprintf(($11|0),(2512|0),($vararg_buffer|0))|0);
   $15 = $mx;
   $16 = (($15) + 1)|0;
   $mx = $16;
  }
  $17 = $0;
  (_fprintf(($17|0),(2520|0),($vararg_buffer1|0))|0);
  $18 = $my;
  $19 = (($18) + 1)|0;
  $my = $19;
 }
 STACKTOP = sp;return;
}
function _sort_indices_int_inc($optr,$ranks,$num) {
 $optr = $optr|0;
 $ranks = $ranks|0;
 $num = $num|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $order = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $optr;
 $2 = $ranks;
 $3 = $num;
 $4 = $3;
 $5 = $4<<2;
 $6 = (_malloc($5)|0);
 $order = $6;
 $7 = $order;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(2528|0),($vararg_buffer|0))|0);
  $0 = -390;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $i = 0;
 while(1) {
  $10 = $i;
  $11 = $3;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = $i;
  $14 = $i;
  $15 = $order;
  $16 = (($15) + ($14<<2)|0);
  HEAP32[$16>>2] = $13;
  $17 = $i;
  $18 = (($17) + 1)|0;
  $i = $18;
 }
 $19 = $2;
 $20 = $order;
 $21 = $3;
 _bubble_sort_int_inc_2($19,$20,$21);
 $22 = $order;
 $23 = $1;
 HEAP32[$23>>2] = $22;
 $0 = 0;
 $24 = $0;
 STACKTOP = sp;return ($24|0);
}
function _bubble_sort_int_inc_2($ranks,$items,$len) {
 $ranks = $ranks|0;
 $items = $items|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $done = 0, $i = 0, $n = 0, $p = 0, $titem = 0;
 var $trank = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $ranks;
 $1 = $items;
 $2 = $len;
 $done = 0;
 $3 = $2;
 $n = $3;
 while(1) {
  $4 = $done;
  $5 = ($4|0)!=(0);
  $6 = $5 ^ 1;
  if (!($6)) {
   break;
  }
  $done = 1;
  $i = 1;
  $p = 0;
  while(1) {
   $7 = $i;
   $8 = $n;
   $9 = ($7|0)<($8|0);
   if (!($9)) {
    break;
   }
   $10 = $p;
   $11 = $0;
   $12 = (($11) + ($10<<2)|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = $i;
   $15 = $0;
   $16 = (($15) + ($14<<2)|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ($13|0)>($17|0);
   if ($18) {
    $19 = $i;
    $20 = $0;
    $21 = (($20) + ($19<<2)|0);
    $22 = HEAP32[$21>>2]|0;
    $trank = $22;
    $23 = $p;
    $24 = $0;
    $25 = (($24) + ($23<<2)|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = $i;
    $28 = $0;
    $29 = (($28) + ($27<<2)|0);
    HEAP32[$29>>2] = $26;
    $30 = $trank;
    $31 = $p;
    $32 = $0;
    $33 = (($32) + ($31<<2)|0);
    HEAP32[$33>>2] = $30;
    $34 = $i;
    $35 = $1;
    $36 = (($35) + ($34<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    $titem = $37;
    $38 = $p;
    $39 = $1;
    $40 = (($39) + ($38<<2)|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $i;
    $43 = $1;
    $44 = (($43) + ($42<<2)|0);
    HEAP32[$44>>2] = $41;
    $45 = $titem;
    $46 = $p;
    $47 = $1;
    $48 = (($47) + ($46<<2)|0);
    HEAP32[$48>>2] = $45;
    $done = 0;
   }
   $49 = $i;
   $50 = (($49) + 1)|0;
   $i = $50;
   $51 = $p;
   $52 = (($51) + 1)|0;
   $p = $52;
  }
  $53 = $n;
  $54 = (($53) + -1)|0;
  $n = $54;
 }
 STACKTOP = sp;return;
}
function _bubble_sort_double_inc_2($ranks,$items,$len) {
 $ranks = $ranks|0;
 $items = $items|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $done = 0, $i = 0, $n = 0, $p = 0, $titem = 0;
 var $trank = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $ranks;
 $1 = $items;
 $2 = $len;
 $done = 0;
 $3 = $2;
 $n = $3;
 while(1) {
  $4 = $done;
  $5 = ($4|0)!=(0);
  $6 = $5 ^ 1;
  if (!($6)) {
   break;
  }
  $done = 1;
  $i = 1;
  $p = 0;
  while(1) {
   $7 = $i;
   $8 = $n;
   $9 = ($7|0)<($8|0);
   if (!($9)) {
    break;
   }
   $10 = $p;
   $11 = $0;
   $12 = (($11) + ($10<<3)|0);
   $13 = +HEAPF64[$12>>3];
   $14 = $i;
   $15 = $0;
   $16 = (($15) + ($14<<3)|0);
   $17 = +HEAPF64[$16>>3];
   $18 = $13 > $17;
   if ($18) {
    $19 = $i;
    $20 = $0;
    $21 = (($20) + ($19<<3)|0);
    $22 = +HEAPF64[$21>>3];
    $trank = $22;
    $23 = $p;
    $24 = $0;
    $25 = (($24) + ($23<<3)|0);
    $26 = +HEAPF64[$25>>3];
    $27 = $i;
    $28 = $0;
    $29 = (($28) + ($27<<3)|0);
    HEAPF64[$29>>3] = $26;
    $30 = $trank;
    $31 = $p;
    $32 = $0;
    $33 = (($32) + ($31<<3)|0);
    HEAPF64[$33>>3] = $30;
    $34 = $i;
    $35 = $1;
    $36 = (($35) + ($34<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    $titem = $37;
    $38 = $p;
    $39 = $1;
    $40 = (($39) + ($38<<2)|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $i;
    $43 = $1;
    $44 = (($43) + ($42<<2)|0);
    HEAP32[$44>>2] = $41;
    $45 = $titem;
    $46 = $p;
    $47 = $1;
    $48 = (($47) + ($46<<2)|0);
    HEAP32[$48>>2] = $45;
    $done = 0;
   }
   $49 = $i;
   $50 = (($49) + 1)|0;
   $i = $50;
   $51 = $p;
   $52 = (($51) + 1)|0;
   $p = $52;
  }
  $53 = $n;
  $54 = (($53) + -1)|0;
  $n = $54;
 }
 STACKTOP = sp;return;
}
function _bubble_sort_double_dec_2($ranks,$items,$len) {
 $ranks = $ranks|0;
 $items = $items|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $done = 0, $i = 0, $n = 0, $p = 0, $titem = 0;
 var $trank = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $ranks;
 $1 = $items;
 $2 = $len;
 $done = 0;
 $3 = $2;
 $n = $3;
 while(1) {
  $4 = $done;
  $5 = ($4|0)!=(0);
  $6 = $5 ^ 1;
  if (!($6)) {
   break;
  }
  $done = 1;
  $i = 1;
  $p = 0;
  while(1) {
   $7 = $i;
   $8 = $n;
   $9 = ($7|0)<($8|0);
   if (!($9)) {
    break;
   }
   $10 = $p;
   $11 = $0;
   $12 = (($11) + ($10<<3)|0);
   $13 = +HEAPF64[$12>>3];
   $14 = $i;
   $15 = $0;
   $16 = (($15) + ($14<<3)|0);
   $17 = +HEAPF64[$16>>3];
   $18 = $13 < $17;
   if ($18) {
    $19 = $i;
    $20 = $0;
    $21 = (($20) + ($19<<3)|0);
    $22 = +HEAPF64[$21>>3];
    $trank = $22;
    $23 = $p;
    $24 = $0;
    $25 = (($24) + ($23<<3)|0);
    $26 = +HEAPF64[$25>>3];
    $27 = $i;
    $28 = $0;
    $29 = (($28) + ($27<<3)|0);
    HEAPF64[$29>>3] = $26;
    $30 = $trank;
    $31 = $p;
    $32 = $0;
    $33 = (($32) + ($31<<3)|0);
    HEAPF64[$33>>3] = $30;
    $34 = $i;
    $35 = $1;
    $36 = (($35) + ($34<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    $titem = $37;
    $38 = $p;
    $39 = $1;
    $40 = (($39) + ($38<<2)|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $i;
    $43 = $1;
    $44 = (($43) + ($42<<2)|0);
    HEAP32[$44>>2] = $41;
    $45 = $titem;
    $46 = $p;
    $47 = $1;
    $48 = (($47) + ($46<<2)|0);
    HEAP32[$48>>2] = $45;
    $done = 0;
   }
   $49 = $i;
   $50 = (($49) + 1)|0;
   $i = $50;
   $51 = $p;
   $52 = (($51) + 1)|0;
   $p = $52;
  }
  $53 = $n;
  $54 = (($53) + -1)|0;
  $n = $54;
 }
 STACKTOP = sp;return;
}
function _bubble_sort_int_inc($ranks,$len) {
 $ranks = $ranks|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $done = 0;
 var $i = 0, $n = 0, $p = 0, $trank = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $ranks;
 $1 = $len;
 $done = 0;
 $2 = $1;
 $n = $2;
 while(1) {
  $3 = $done;
  $4 = ($3|0)!=(0);
  $5 = $4 ^ 1;
  if (!($5)) {
   break;
  }
  $done = 1;
  $i = 1;
  $p = 0;
  while(1) {
   $6 = $i;
   $7 = $n;
   $8 = ($6|0)<($7|0);
   if (!($8)) {
    break;
   }
   $9 = $p;
   $10 = $0;
   $11 = (($10) + ($9<<2)|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = $i;
   $14 = $0;
   $15 = (($14) + ($13<<2)|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($12|0)>($16|0);
   if ($17) {
    $18 = $i;
    $19 = $0;
    $20 = (($19) + ($18<<2)|0);
    $21 = HEAP32[$20>>2]|0;
    $trank = $21;
    $22 = $p;
    $23 = $0;
    $24 = (($23) + ($22<<2)|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = $i;
    $27 = $0;
    $28 = (($27) + ($26<<2)|0);
    HEAP32[$28>>2] = $25;
    $29 = $trank;
    $30 = $p;
    $31 = $0;
    $32 = (($31) + ($30<<2)|0);
    HEAP32[$32>>2] = $29;
    $done = 0;
   }
   $33 = $i;
   $34 = (($33) + 1)|0;
   $i = $34;
   $35 = $p;
   $36 = (($35) + 1)|0;
   $p = $36;
  }
  $37 = $n;
  $38 = (($37) + -1)|0;
  $n = $38;
 }
 STACKTOP = sp;return;
}
function _update_ANSI_NIST_lfs_results($ansi_nist,$minutiae,$idata,$iw,$ih,$id,$ppmm,$img_idc,$img_imp) {
 $ansi_nist = $ansi_nist|0;
 $minutiae = $minutiae|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $id = $id|0;
 $ppmm = +$ppmm;
 $img_idc = $img_idc|0;
 $img_imp = $img_imp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0.0, $8 = 0, $9 = 0, $ilen = 0, $imgrecord = 0, $ret = 0, $type_9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $type_9 = sp + 24|0;
 $imgrecord = sp + 20|0;
 $1 = $ansi_nist;
 $2 = $minutiae;
 $3 = $idata;
 $4 = $iw;
 $5 = $ih;
 $6 = $id;
 $7 = $ppmm;
 $8 = $img_idc;
 $9 = $img_imp;
 $10 = $2;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 do {
  if ($13) {
   $14 = $8;
   $15 = $2;
   $16 = $4;
   $17 = $5;
   $18 = $7;
   $19 = (_minutiae2type_9($type_9,$14,$15,$16,$17,$18)|0);
   $ret = $19;
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = $ret;
    $0 = $21;
    $66 = $0;
    STACKTOP = sp;return ($66|0);
   }
   $22 = $1;
   $23 = (($22) + 8|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$type_9>>2]|0;
   $26 = $1;
   $27 = (_insert_ANSI_NIST_record_frmem($24,$25,$26)|0);
   $ret = $27;
   $28 = ($27|0)!=(0);
   if (!($28)) {
    break;
   }
   $29 = HEAP32[$type_9>>2]|0;
   _free_ANSI_NIST_record($29);
   $30 = $ret;
   $0 = $30;
   $66 = $0;
   STACKTOP = sp;return ($66|0);
  }
 } while(0);
 $31 = $6;
 if ((($31|0) == 24)) {
  $39 = $4;
  $40 = $5;
  $41 = Math_imul($39, $40)|0;
  $42 = ($41*3)|0;
  $ilen = $42;
 } else if ((($31|0) == 1)) {
  $32 = $4;
  $33 = $32 >> 3;
  $34 = $5;
  $35 = Math_imul($33, $34)|0;
  $ilen = $35;
 } else if ((($31|0) == 8)) {
  $36 = $4;
  $37 = $5;
  $38 = Math_imul($36, $37)|0;
  $ilen = $38;
 } else {
  $43 = HEAP32[_stderr>>2]|0;
  (_fprintf(($43|0),(2576|0),($vararg_buffer|0))|0);
  $44 = HEAP32[_stderr>>2]|0;
  $45 = $6;
  HEAP32[$vararg_buffer1>>2] = $45;
  (_fprintf(($44|0),(2616|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 }
 $46 = $3;
 $47 = $ilen;
 $48 = $4;
 $49 = $5;
 $50 = $6;
 $51 = $7;
 $52 = $8;
 $53 = $9;
 $54 = (_fingerprint2tagged_field_image($imgrecord,$46,$47,$48,$49,$50,$51,2656,$52,$53,2664)|0);
 $ret = $54;
 $55 = ($54|0)!=(0);
 if ($55) {
  $56 = $ret;
  $0 = $56;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 }
 $57 = $1;
 $58 = (($57) + 8|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = HEAP32[$imgrecord>>2]|0;
 $61 = $1;
 $62 = (_insert_ANSI_NIST_record_frmem($59,$60,$61)|0);
 $ret = $62;
 $63 = ($62|0)!=(0);
 if ($63) {
  $64 = HEAP32[$imgrecord>>2]|0;
  _free_ANSI_NIST_record($64);
  $65 = $ret;
  $0 = $65;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 } else {
  $0 = 0;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 }
 return 0|0;
}
function _maxv($list,$num) {
 $list = $list|0;
 $num = $num|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $maxval = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $list;
 $1 = $num;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $maxval = $3;
 $i = 1;
 while(1) {
  $4 = $i;
  $5 = $1;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $0;
  $9 = (($8) + ($7<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $maxval;
  $12 = ($10|0)>($11|0);
  if ($12) {
   $13 = $i;
   $14 = $0;
   $15 = (($14) + ($13<<2)|0);
   $16 = HEAP32[$15>>2]|0;
   $maxval = $16;
  }
  $17 = $i;
  $18 = (($17) + 1)|0;
  $i = $18;
 }
 $19 = $maxval;
 STACKTOP = sp;return ($19|0);
}
function _minv($list,$num) {
 $list = $list|0;
 $num = $num|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $minval = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $list;
 $1 = $num;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $minval = $3;
 $i = 1;
 while(1) {
  $4 = $i;
  $5 = $1;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $0;
  $9 = (($8) + ($7<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $minval;
  $12 = ($10|0)<($11|0);
  if ($12) {
   $13 = $i;
   $14 = $0;
   $15 = (($14) + ($13<<2)|0);
   $16 = HEAP32[$15>>2]|0;
   $minval = $16;
  }
  $17 = $i;
  $18 = (($17) + 1)|0;
  $i = $18;
 }
 $19 = $minval;
 STACKTOP = sp;return ($19|0);
}
function _minmaxs($ominmax_val,$ominmax_type,$ominmax_i,$ominmax_alloc,$ominmax_num,$items,$num) {
 $ominmax_val = $ominmax_val|0;
 $ominmax_type = $ominmax_type|0;
 $ominmax_i = $ominmax_i|0;
 $ominmax_alloc = $ominmax_alloc|0;
 $ominmax_num = $ominmax_num|0;
 $items = $items|0;
 $num = $num|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $diff = 0, $i = 0;
 var $loc = 0, $minmax_alloc = 0, $minmax_i = 0, $minmax_num = 0, $minmax_type = 0, $minmax_val = 0, $start = 0, $state = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ominmax_val;
 $2 = $ominmax_type;
 $3 = $ominmax_i;
 $4 = $ominmax_alloc;
 $5 = $ominmax_num;
 $6 = $items;
 $7 = $num;
 $8 = $7;
 $9 = ($8|0)<(3);
 if ($9) {
  $10 = $4;
  HEAP32[$10>>2] = 0;
  $11 = $5;
  HEAP32[$11>>2] = 0;
  $0 = 0;
  $177 = $0;
  STACKTOP = sp;return ($177|0);
 }
 $12 = $7;
 $13 = (($12) - 2)|0;
 $minmax_alloc = $13;
 $14 = $minmax_alloc;
 $15 = $14<<2;
 $16 = (_malloc($15)|0);
 $minmax_val = $16;
 $17 = $minmax_val;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(2680|0),($vararg_buffer|0))|0);
  $0 = -290;
  $177 = $0;
  STACKTOP = sp;return ($177|0);
 }
 $20 = $minmax_alloc;
 $21 = $20<<2;
 $22 = (_malloc($21)|0);
 $minmax_type = $22;
 $23 = $minmax_type;
 $24 = ($23|0)==(0|0);
 if ($24) {
  $25 = $minmax_val;
  _free($25);
  $26 = HEAP32[_stderr>>2]|0;
  (_fprintf(($26|0),(2720|0),($vararg_buffer1|0))|0);
  $0 = -291;
  $177 = $0;
  STACKTOP = sp;return ($177|0);
 }
 $27 = $minmax_alloc;
 $28 = $27<<2;
 $29 = (_malloc($28)|0);
 $minmax_i = $29;
 $30 = $minmax_i;
 $31 = ($30|0)==(0|0);
 if ($31) {
  $32 = $minmax_val;
  _free($32);
  $33 = $minmax_type;
  _free($33);
  $34 = HEAP32[_stderr>>2]|0;
  (_fprintf(($34|0),(2760|0),($vararg_buffer3|0))|0);
  $0 = -292;
  $177 = $0;
  STACKTOP = sp;return ($177|0);
 }
 $minmax_num = 0;
 $i = 0;
 $35 = $6;
 $36 = (($35) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $6;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) - ($39))|0;
 $diff = $40;
 $41 = $diff;
 $42 = ($41|0)>(0);
 if ($42) {
  $state = 1;
 } else {
  $43 = $diff;
  $44 = ($43|0)<(0);
  if ($44) {
   $state = -1;
  } else {
   $state = 0;
  }
 }
 $start = 0;
 $45 = $i;
 $46 = (($45) + 1)|0;
 $i = $46;
 while(1) {
  $47 = $i;
  $48 = $7;
  $49 = (($48) - 1)|0;
  $50 = ($47|0)<($49|0);
  if (!($50)) {
   break;
  }
  $51 = $i;
  $52 = (($51) + 1)|0;
  $53 = $6;
  $54 = (($53) + ($52<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = $i;
  $57 = $6;
  $58 = (($57) + ($56<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (($55) - ($59))|0;
  $diff = $60;
  $61 = $diff;
  $62 = ($61|0)>(0);
  if ($62) {
   $63 = $state;
   $64 = ($63|0)==(1);
   if ($64) {
    $65 = $i;
    $start = $65;
   } else {
    $66 = $state;
    $67 = ($66|0)==(-1);
    if ($67) {
     $68 = $start;
     $69 = $i;
     $70 = (($68) + ($69))|0;
     $71 = (($70|0) / 2)&-1;
     $loc = $71;
     $72 = $loc;
     $73 = $6;
     $74 = (($73) + ($72<<2)|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = $minmax_num;
     $77 = $minmax_val;
     $78 = (($77) + ($76<<2)|0);
     HEAP32[$78>>2] = $75;
     $79 = $minmax_num;
     $80 = $minmax_type;
     $81 = (($80) + ($79<<2)|0);
     HEAP32[$81>>2] = -1;
     $82 = $loc;
     $83 = $minmax_num;
     $84 = (($83) + 1)|0;
     $minmax_num = $84;
     $85 = $minmax_i;
     $86 = (($85) + ($83<<2)|0);
     HEAP32[$86>>2] = $82;
     $state = 1;
     $87 = $i;
     $start = $87;
    } else {
     $88 = $i;
     $89 = $start;
     $90 = (($88) - ($89))|0;
     $91 = ($90|0)>(1);
     if ($91) {
      $92 = $start;
      $93 = $i;
      $94 = (($92) + ($93))|0;
      $95 = (($94|0) / 2)&-1;
      $loc = $95;
      $96 = $loc;
      $97 = $6;
      $98 = (($97) + ($96<<2)|0);
      $99 = HEAP32[$98>>2]|0;
      $100 = $minmax_num;
      $101 = $minmax_val;
      $102 = (($101) + ($100<<2)|0);
      HEAP32[$102>>2] = $99;
      $103 = $minmax_num;
      $104 = $minmax_type;
      $105 = (($104) + ($103<<2)|0);
      HEAP32[$105>>2] = -1;
      $106 = $loc;
      $107 = $minmax_num;
      $108 = (($107) + 1)|0;
      $minmax_num = $108;
      $109 = $minmax_i;
      $110 = (($109) + ($107<<2)|0);
      HEAP32[$110>>2] = $106;
      $state = 1;
      $111 = $i;
      $start = $111;
     } else {
      $state = 1;
      $112 = $i;
      $start = $112;
     }
    }
   }
  } else {
   $113 = $diff;
   $114 = ($113|0)<(0);
   if ($114) {
    $115 = $state;
    $116 = ($115|0)==(-1);
    if ($116) {
     $117 = $i;
     $start = $117;
    } else {
     $118 = $state;
     $119 = ($118|0)==(1);
     if ($119) {
      $120 = $start;
      $121 = $i;
      $122 = (($120) + ($121))|0;
      $123 = (($122|0) / 2)&-1;
      $loc = $123;
      $124 = $loc;
      $125 = $6;
      $126 = (($125) + ($124<<2)|0);
      $127 = HEAP32[$126>>2]|0;
      $128 = $minmax_num;
      $129 = $minmax_val;
      $130 = (($129) + ($128<<2)|0);
      HEAP32[$130>>2] = $127;
      $131 = $minmax_num;
      $132 = $minmax_type;
      $133 = (($132) + ($131<<2)|0);
      HEAP32[$133>>2] = 1;
      $134 = $loc;
      $135 = $minmax_num;
      $136 = (($135) + 1)|0;
      $minmax_num = $136;
      $137 = $minmax_i;
      $138 = (($137) + ($135<<2)|0);
      HEAP32[$138>>2] = $134;
      $state = -1;
      $139 = $i;
      $start = $139;
     } else {
      $140 = $i;
      $141 = $start;
      $142 = (($140) - ($141))|0;
      $143 = ($142|0)>(1);
      if ($143) {
       $144 = $start;
       $145 = $i;
       $146 = (($144) + ($145))|0;
       $147 = (($146|0) / 2)&-1;
       $loc = $147;
       $148 = $loc;
       $149 = $6;
       $150 = (($149) + ($148<<2)|0);
       $151 = HEAP32[$150>>2]|0;
       $152 = $minmax_num;
       $153 = $minmax_val;
       $154 = (($153) + ($152<<2)|0);
       HEAP32[$154>>2] = $151;
       $155 = $minmax_num;
       $156 = $minmax_type;
       $157 = (($156) + ($155<<2)|0);
       HEAP32[$157>>2] = 1;
       $158 = $loc;
       $159 = $minmax_num;
       $160 = (($159) + 1)|0;
       $minmax_num = $160;
       $161 = $minmax_i;
       $162 = (($161) + ($159<<2)|0);
       HEAP32[$162>>2] = $158;
       $state = -1;
       $163 = $i;
       $start = $163;
      } else {
       $state = -1;
       $164 = $i;
       $start = $164;
      }
     }
    }
   }
  }
  $165 = $i;
  $166 = (($165) + 1)|0;
  $i = $166;
 }
 $167 = $minmax_val;
 $168 = $1;
 HEAP32[$168>>2] = $167;
 $169 = $minmax_type;
 $170 = $2;
 HEAP32[$170>>2] = $169;
 $171 = $minmax_i;
 $172 = $3;
 HEAP32[$172>>2] = $171;
 $173 = $minmax_alloc;
 $174 = $4;
 HEAP32[$174>>2] = $173;
 $175 = $minmax_num;
 $176 = $5;
 HEAP32[$176>>2] = $175;
 $0 = 0;
 $177 = $0;
 STACKTOP = sp;return ($177|0);
}
function _distance($x1,$y1,$x2,$y2) {
 $x1 = $x1|0;
 $y1 = $y1|0;
 $x2 = $x2|0;
 $y2 = $y2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0;
 var $8 = 0, $9 = 0, $dist = 0.0, $dx = 0.0, $dy = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $x1;
 $1 = $y1;
 $2 = $x2;
 $3 = $y2;
 $4 = $0;
 $5 = $2;
 $6 = (($4) - ($5))|0;
 $7 = (+($6|0));
 $dx = $7;
 $8 = $1;
 $9 = $3;
 $10 = (($8) - ($9))|0;
 $11 = (+($10|0));
 $dy = $11;
 $12 = $dx;
 $13 = $dx;
 $14 = $12 * $13;
 $15 = $dy;
 $16 = $dy;
 $17 = $15 * $16;
 $18 = $14 + $17;
 $dist = $18;
 $19 = $dist;
 $20 = (+Math_sqrt((+$19)));
 $dist = $20;
 $21 = $dist;
 STACKTOP = sp;return (+$21);
}
function _squared_distance($x1,$y1,$x2,$y2) {
 $x1 = $x1|0;
 $y1 = $y1|0;
 $x2 = $x2|0;
 $y2 = $y2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0, $9 = 0;
 var $dist = 0.0, $dx = 0.0, $dy = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $x1;
 $1 = $y1;
 $2 = $x2;
 $3 = $y2;
 $4 = $0;
 $5 = $2;
 $6 = (($4) - ($5))|0;
 $7 = (+($6|0));
 $dx = $7;
 $8 = $1;
 $9 = $3;
 $10 = (($8) - ($9))|0;
 $11 = (+($10|0));
 $dy = $11;
 $12 = $dx;
 $13 = $dx;
 $14 = $12 * $13;
 $15 = $dy;
 $16 = $dy;
 $17 = $15 * $16;
 $18 = $14 + $17;
 $dist = $18;
 $19 = $dist;
 STACKTOP = sp;return (+$19);
}
function _in_int_list($item,$list,$len) {
 $item = $item|0;
 $list = $list|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $item;
 $2 = $list;
 $3 = $len;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   label = 7;
   break;
  }
  $7 = $i;
  $8 = $2;
  $9 = (($8) + ($7<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $1;
  $12 = ($10|0)==($11|0);
  if ($12) {
   label = 4;
   break;
  }
  $14 = $i;
  $15 = (($14) + 1)|0;
  $i = $15;
 }
 if ((label|0) == 4) {
  $13 = $i;
  $0 = $13;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 else if ((label|0) == 7) {
  $0 = -1;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _find_incr_position_dbl($val,$list,$num) {
 $val = +$val;
 $list = $list|0;
 $num = $num|0;
 var $0 = 0, $1 = 0.0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $val;
 $2 = $list;
 $3 = $num;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   label = 7;
   break;
  }
  $7 = $1;
  $8 = $i;
  $9 = $2;
  $10 = (($9) + ($8<<3)|0);
  $11 = +HEAPF64[$10>>3];
  $12 = $7 < $11;
  if ($12) {
   label = 4;
   break;
  }
  $14 = $i;
  $15 = (($14) + 1)|0;
  $i = $15;
 }
 if ((label|0) == 4) {
  $13 = $i;
  $0 = $13;
  $17 = $0;
  STACKTOP = sp;return ($17|0);
 }
 else if ((label|0) == 7) {
  $16 = $i;
  $0 = $16;
  $17 = $0;
  STACKTOP = sp;return ($17|0);
 }
 return 0|0;
}
function _angle2line($fx,$fy,$tx,$ty) {
 $fx = $fx|0;
 $fy = $fy|0;
 $tx = $tx|0;
 $ty = $ty|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0;
 var $8 = 0, $9 = 0, $dx = 0.0, $dy = 0.0, $theta = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $fx;
 $1 = $fy;
 $2 = $tx;
 $3 = $ty;
 $4 = $1;
 $5 = $3;
 $6 = (($4) - ($5))|0;
 $7 = (+($6|0));
 $dy = $7;
 $8 = $2;
 $9 = $0;
 $10 = (($8) - ($9))|0;
 $11 = (+($10|0));
 $dx = $11;
 $12 = $dx;
 $13 = (+Math_abs((+$12)));
 $14 = $13 < 0.5;
 if ($14) {
  $15 = $dy;
  $16 = (+Math_abs((+$15)));
  $17 = $16 < 0.5;
  if ($17) {
   $theta = 0.0;
   $21 = $theta;
   STACKTOP = sp;return (+$21);
  }
 }
 $18 = $dy;
 $19 = $dx;
 $20 = (+Math_atan2((+$18),(+$19)));
 $theta = $20;
 $21 = $theta;
 STACKTOP = sp;return (+$21);
}
function _line2direction($fx,$fy,$tx,$ty,$ndirs) {
 $fx = $fx|0;
 $fy = $fy|0;
 $tx = $tx|0;
 $ty = $ty|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0.0, $43 = 0.0, $44 = 0.0;
 var $45 = 0.0, $46 = 0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0.0, $full_ndirs = 0, $idir = 0, $pi_factor = 0.0, $theta = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $fx;
 $1 = $fy;
 $2 = $tx;
 $3 = $ty;
 $4 = $ndirs;
 $5 = $3;
 $6 = $2;
 $7 = $1;
 $8 = $0;
 $9 = (+_angle2line($5,$6,$7,$8));
 $theta = $9;
 $10 = +HEAPF64[2800>>3];
 $11 = $theta;
 $12 = $11 + $10;
 $theta = $12;
 $13 = $theta;
 $14 = +HEAPF64[2800>>3];
 $15 = (+_fmod((+$13),(+$14)));
 $theta = $15;
 $16 = $4;
 $17 = $16 << 1;
 $full_ndirs = $17;
 $18 = $full_ndirs;
 $19 = (+($18|0));
 $20 = +HEAPF64[2800>>3];
 $21 = $19 / $20;
 $pi_factor = $21;
 $22 = $pi_factor;
 $23 = $theta;
 $24 = $23 * $22;
 $theta = $24;
 $25 = $theta;
 $26 = $25 < 0.0;
 if ($26) {
  $27 = $theta;
  $28 = $27 * 16384.0;
  $29 = $28 - 0.5;
  $30 = (~~(($29)));
  $31 = (+($30|0));
  $32 = $31 / 16384.0;
  $39 = $32;
 } else {
  $33 = $theta;
  $34 = $33 * 16384.0;
  $35 = $34 + 0.5;
  $36 = (~~(($35)));
  $37 = (+($36|0));
  $38 = $37 / 16384.0;
  $39 = $38;
 }
 $theta = $39;
 $40 = $theta;
 $41 = $40 < 0.0;
 if ($41) {
  $42 = $theta;
  $43 = $42 - 0.5;
  $47 = $43;
  $46 = (~~(($47)));
  $idir = $46;
  $48 = $full_ndirs;
  $49 = $idir;
  $50 = (($49|0) % ($48|0))&-1;
  $idir = $50;
  $51 = $idir;
  STACKTOP = sp;return ($51|0);
 } else {
  $44 = $theta;
  $45 = $44 + 0.5;
  $47 = $45;
  $46 = (~~(($47)));
  $idir = $46;
  $48 = $full_ndirs;
  $49 = $idir;
  $50 = (($49|0) % ($48|0))&-1;
  $idir = $50;
  $51 = $idir;
  STACKTOP = sp;return ($51|0);
 }
 return 0|0;
}
function _closest_dir_dist($dir1,$dir2,$ndirs) {
 $dir1 = $dir1|0;
 $dir2 = $dir2|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $d1 = 0, $d2 = 0, $dist = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $dir1;
 $1 = $dir2;
 $2 = $ndirs;
 $dist = -1;
 $3 = $0;
 $4 = ($3|0)>=(0);
 if ($4) {
  $5 = $1;
  $6 = ($5|0)>=(0);
  if ($6) {
   $7 = $1;
   $8 = $0;
   $9 = (($7) - ($8))|0;
   $10 = (Math_abs(($9|0))|0);
   $d1 = $10;
   $11 = $2;
   $12 = $d1;
   $13 = (($11) - ($12))|0;
   $d2 = $13;
   $14 = $d1;
   $15 = $d2;
   $16 = ($14|0)<($15|0);
   if ($16) {
    $17 = $d1;
    $19 = $17;
   } else {
    $18 = $d2;
    $19 = $18;
   }
   $dist = $19;
  }
 }
 $20 = $dist;
 STACKTOP = sp;return ($20|0);
}
function _lfs2nist_minutia_XYT($ox,$oy,$ot,$minutia,$iw,$ih) {
 $ox = $ox|0;
 $oy = $oy|0;
 $ot = $ot|0;
 $minutia = $minutia|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $degrees_per_unit = 0.0, $t = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $ox;
 $1 = $oy;
 $2 = $ot;
 $3 = $minutia;
 $4 = $iw;
 $5 = $ih;
 $6 = $3;
 $7 = HEAP32[$6>>2]|0;
 $x = $7;
 $8 = $5;
 $9 = $3;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($8) - ($11))|0;
 $y = $12;
 $degrees_per_unit = 11.25;
 $13 = $3;
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (+($15|0));
 $17 = $degrees_per_unit;
 $18 = $16 * $17;
 $19 = $18 < 0.0;
 if ($19) {
  $20 = $3;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (+($22|0));
  $24 = $degrees_per_unit;
  $25 = $23 * $24;
  $26 = $25;
  $27 = $26 - 0.5;
  $37 = $27;
 } else {
  $28 = $3;
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (+($30|0));
  $32 = $degrees_per_unit;
  $33 = $31 * $32;
  $34 = $33;
  $35 = $34 + 0.5;
  $37 = $35;
 }
 $36 = (~~(($37)));
 $38 = (270 - ($36))|0;
 $39 = (($38|0) % 360)&-1;
 $t = $39;
 $40 = $t;
 $41 = ($40|0)<(0);
 if (!($41)) {
  $44 = $x;
  $45 = $0;
  HEAP32[$45>>2] = $44;
  $46 = $y;
  $47 = $1;
  HEAP32[$47>>2] = $46;
  $48 = $t;
  $49 = $2;
  HEAP32[$49>>2] = $48;
  STACKTOP = sp;return;
 }
 $42 = $t;
 $43 = (($42) + 360)|0;
 $t = $43;
 $44 = $x;
 $45 = $0;
 HEAP32[$45>>2] = $44;
 $46 = $y;
 $47 = $1;
 HEAP32[$47>>2] = $46;
 $48 = $t;
 $49 = $2;
 HEAP32[$49>>2] = $48;
 STACKTOP = sp;return;
}
function _lfs2m1_minutia_XYT($ox,$oy,$ot,$minutia) {
 $ox = $ox|0;
 $oy = $oy|0;
 $ot = $ot|0;
 $minutia = $minutia|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0.0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $degrees_per_unit = 0.0, $t = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $ox;
 $1 = $oy;
 $2 = $ot;
 $3 = $minutia;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $x = $5;
 $6 = $3;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $y = $8;
 $degrees_per_unit = 11.25;
 $9 = $3;
 $10 = (($9) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (+($11|0));
 $13 = $degrees_per_unit;
 $14 = $12 * $13;
 $15 = $14 < 0.0;
 if ($15) {
  $16 = $3;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (+($18|0));
  $20 = $degrees_per_unit;
  $21 = $19 * $20;
  $22 = $21;
  $23 = $22 - 0.5;
  $33 = $23;
 } else {
  $24 = $3;
  $25 = (($24) + 16|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (+($26|0));
  $28 = $degrees_per_unit;
  $29 = $27 * $28;
  $30 = $29;
  $31 = $30 + 0.5;
  $33 = $31;
 }
 $32 = (~~(($33)));
 $34 = (90 - ($32))|0;
 $35 = (($34|0) % 360)&-1;
 $t = $35;
 $36 = $t;
 $37 = ($36|0)<(0);
 if ($37) {
  $38 = $t;
  $39 = (($38) + 360)|0;
  $t = $39;
 }
 $40 = $t;
 $41 = (($40|0) / 2)&-1;
 $t = $41;
 $42 = $x;
 $43 = $0;
 HEAP32[$43>>2] = $42;
 $44 = $y;
 $45 = $1;
 HEAP32[$45>>2] = $44;
 $46 = $t;
 $47 = $2;
 HEAP32[$47>>2] = $46;
 STACKTOP = sp;return;
}
function _block_offsets($optr,$ow,$oh,$iw,$ih,$pad,$blocksize) {
 $optr = $optr|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $pad = $pad|0;
 $blocksize = $blocksize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0, $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0, $35 = 0.0;
 var $36 = 0.0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bh = 0, $bi = 0, $blkoffs = 0, $blkrow_size = 0, $blkrow_start = 0, $bsize = 0, $bw = 0, $bx = 0, $by = 0, $lastbh = 0;
 var $lastbw = 0, $offset = 0, $pad2 = 0, $ph = 0, $pw = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $optr;
 $2 = $ow;
 $3 = $oh;
 $4 = $iw;
 $5 = $ih;
 $6 = $pad;
 $7 = $blocksize;
 $8 = $4;
 $9 = $7;
 $10 = ($8|0)<($9|0);
 if (!($10)) {
  $11 = $5;
  $12 = $7;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   $17 = $6;
   $18 = $17 << 1;
   $pad2 = $18;
   $19 = $4;
   $20 = $pad2;
   $21 = (($19) + ($20))|0;
   $pw = $21;
   $22 = $5;
   $23 = $pad2;
   $24 = (($22) + ($23))|0;
   $ph = $24;
   $25 = $4;
   $26 = (+($25|0));
   $27 = $7;
   $28 = (+($27|0));
   $29 = $26 / $28;
   $30 = (+Math_ceil((+$29)));
   $31 = (~~(($30)));
   $bw = $31;
   $32 = $5;
   $33 = (+($32|0));
   $34 = $7;
   $35 = (+($34|0));
   $36 = $33 / $35;
   $37 = (+Math_ceil((+$36)));
   $38 = (~~(($37)));
   $bh = $38;
   $39 = $bw;
   $40 = $bh;
   $41 = Math_imul($39, $40)|0;
   $bsize = $41;
   $42 = $bw;
   $43 = (($42) - 1)|0;
   $lastbw = $43;
   $44 = $bh;
   $45 = (($44) - 1)|0;
   $lastbh = $45;
   $46 = $bsize;
   $47 = $46<<2;
   $48 = (_malloc($47)|0);
   $blkoffs = $48;
   $49 = $blkoffs;
   $50 = ($49|0)==(0|0);
   if ($50) {
    $51 = HEAP32[_stderr>>2]|0;
    (_fprintf(($51|0),(2880|0),($vararg_buffer2|0))|0);
    $0 = -81;
    $129 = $0;
    STACKTOP = sp;return ($129|0);
   }
   $bi = 0;
   $52 = $6;
   $53 = $pw;
   $54 = Math_imul($52, $53)|0;
   $55 = $6;
   $56 = (($54) + ($55))|0;
   $blkrow_start = $56;
   $57 = $pw;
   $58 = $7;
   $59 = Math_imul($57, $58)|0;
   $blkrow_size = $59;
   $by = 0;
   while(1) {
    $60 = $by;
    $61 = $lastbh;
    $62 = ($60|0)<($61|0);
    if (!($62)) {
     break;
    }
    $63 = $blkrow_start;
    $offset = $63;
    $bx = 0;
    while(1) {
     $64 = $bx;
     $65 = $lastbw;
     $66 = ($64|0)<($65|0);
     if (!($66)) {
      break;
     }
     $67 = $offset;
     $68 = $bi;
     $69 = (($68) + 1)|0;
     $bi = $69;
     $70 = $blkoffs;
     $71 = (($70) + ($68<<2)|0);
     HEAP32[$71>>2] = $67;
     $72 = $7;
     $73 = $offset;
     $74 = (($73) + ($72))|0;
     $offset = $74;
     $75 = $bx;
     $76 = (($75) + 1)|0;
     $bx = $76;
    }
    $77 = $blkrow_start;
    $78 = $4;
    $79 = (($77) + ($78))|0;
    $80 = $7;
    $81 = (($79) - ($80))|0;
    $82 = $bi;
    $83 = (($82) + 1)|0;
    $bi = $83;
    $84 = $blkoffs;
    $85 = (($84) + ($82<<2)|0);
    HEAP32[$85>>2] = $81;
    $86 = $blkrow_size;
    $87 = $blkrow_start;
    $88 = (($87) + ($86))|0;
    $blkrow_start = $88;
    $89 = $by;
    $90 = (($89) + 1)|0;
    $by = $90;
   }
   $91 = $6;
   $92 = $5;
   $93 = (($91) + ($92))|0;
   $94 = $7;
   $95 = (($93) - ($94))|0;
   $96 = $pw;
   $97 = Math_imul($95, $96)|0;
   $98 = $6;
   $99 = (($97) + ($98))|0;
   $blkrow_start = $99;
   $100 = $blkrow_start;
   $offset = $100;
   $bx = 0;
   while(1) {
    $101 = $bx;
    $102 = $lastbw;
    $103 = ($101|0)<($102|0);
    if (!($103)) {
     break;
    }
    $104 = $offset;
    $105 = $bi;
    $106 = (($105) + 1)|0;
    $bi = $106;
    $107 = $blkoffs;
    $108 = (($107) + ($105<<2)|0);
    HEAP32[$108>>2] = $104;
    $109 = $7;
    $110 = $offset;
    $111 = (($110) + ($109))|0;
    $offset = $111;
    $112 = $bx;
    $113 = (($112) + 1)|0;
    $bx = $113;
   }
   $114 = $blkrow_start;
   $115 = $4;
   $116 = (($114) + ($115))|0;
   $117 = $7;
   $118 = (($116) - ($117))|0;
   $119 = $bi;
   $120 = (($119) + 1)|0;
   $bi = $120;
   $121 = $blkoffs;
   $122 = (($121) + ($119<<2)|0);
   HEAP32[$122>>2] = $118;
   $123 = $blkoffs;
   $124 = $1;
   HEAP32[$124>>2] = $123;
   $125 = $bw;
   $126 = $2;
   HEAP32[$126>>2] = $125;
   $127 = $bh;
   $128 = $3;
   HEAP32[$128>>2] = $127;
   $0 = 0;
   $129 = $0;
   STACKTOP = sp;return ($129|0);
  }
 }
 $14 = HEAP32[_stderr>>2]|0;
 $15 = $7;
 $16 = $7;
 HEAP32[$vararg_buffer>>2] = $15;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $16;
 (_fprintf(($14|0),(2808|0),($vararg_buffer|0))|0);
 $0 = -80;
 $129 = $0;
 STACKTOP = sp;return ($129|0);
}
function _low_contrast_block($blkoffset,$blocksize,$pdata,$pw,$ph,$lfsparms) {
 $blkoffset = $blkoffset|0;
 $blocksize = $blocksize|0;
 $pdata = $pdata|0;
 $pw = $pw|0;
 $ph = $ph|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $19 = 0.0;
 var $2 = 0, $20 = 0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0, $25 = 0.0, $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0.0, $37 = 0.0;
 var $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $delta = 0, $found = 0, $numpix = 0, $pi = 0, $pixsum = 0, $pixtable = 0, $pptr = 0, $prctmax = 0, $prctmin = 0, $prctthresh = 0, $px = 0, $py = 0;
 var $sptr = 0, $tdbl = 0.0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $pixtable = sp + 72|0;
 $1 = $blkoffset;
 $2 = $blocksize;
 $3 = $pdata;
 $4 = $pw;
 $5 = $ph;
 $6 = $lfsparms;
 $prctmin = 0;
 $prctmax = 0;
 $7 = $2;
 $8 = $2;
 $9 = Math_imul($7, $8)|0;
 $numpix = $9;
 _memset(($pixtable|0),0,256)|0;
 $10 = $6;
 $11 = (($10) + 72|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($12|0));
 $14 = $13 / 100.0;
 $15 = $numpix;
 $16 = (($15) - 1)|0;
 $17 = (+($16|0));
 $18 = $14 * $17;
 $tdbl = $18;
 $19 = $tdbl;
 $20 = $19 < 0.0;
 if ($20) {
  $21 = $tdbl;
  $22 = $21 * 16384.0;
  $23 = $22 - 0.5;
  $24 = (~~(($23)));
  $25 = (+($24|0));
  $26 = $25 / 16384.0;
  $33 = $26;
 } else {
  $27 = $tdbl;
  $28 = $27 * 16384.0;
  $29 = $28 + 0.5;
  $30 = (~~(($29)));
  $31 = (+($30|0));
  $32 = $31 / 16384.0;
  $33 = $32;
 }
 $tdbl = $33;
 $34 = $tdbl;
 $35 = $34 < 0.0;
 if ($35) {
  $36 = $tdbl;
  $37 = $36 - 0.5;
  $41 = $37;
 } else {
  $38 = $tdbl;
  $39 = $38 + 0.5;
  $41 = $39;
 }
 $40 = (~~(($41)));
 $prctthresh = $40;
 $42 = $3;
 $43 = $1;
 $44 = (($42) + ($43)|0);
 $sptr = $44;
 $py = 0;
 while(1) {
  $45 = $py;
  $46 = $2;
  $47 = ($45|0)<($46|0);
  if (!($47)) {
   break;
  }
  $48 = $sptr;
  $pptr = $48;
  $px = 0;
  while(1) {
   $49 = $px;
   $50 = $2;
   $51 = ($49|0)<($50|0);
   if (!($51)) {
    break;
   }
   $52 = $pptr;
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = (($pixtable) + ($54<<2)|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 1)|0;
   HEAP32[$55>>2] = $57;
   $58 = $pptr;
   $59 = (($58) + 1|0);
   $pptr = $59;
   $60 = $px;
   $61 = (($60) + 1)|0;
   $px = $61;
  }
  $62 = $4;
  $63 = $sptr;
  $64 = (($63) + ($62)|0);
  $sptr = $64;
  $65 = $py;
  $66 = (($65) + 1)|0;
  $py = $66;
 }
 $pi = 0;
 $pixsum = 0;
 $found = 0;
 while(1) {
  $67 = $pi;
  $68 = ($67|0)<(64);
  if (!($68)) {
   break;
  }
  $69 = $pi;
  $70 = (($pixtable) + ($69<<2)|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = $pixsum;
  $73 = (($72) + ($71))|0;
  $pixsum = $73;
  $74 = $pixsum;
  $75 = $prctthresh;
  $76 = ($74|0)>=($75|0);
  if ($76) {
   label = 18;
   break;
  }
  $78 = $pi;
  $79 = (($78) + 1)|0;
  $pi = $79;
 }
 if ((label|0) == 18) {
  $77 = $pi;
  $prctmin = $77;
  $found = 1;
 }
 $80 = $found;
 $81 = ($80|0)!=(0);
 if (!($81)) {
  $82 = HEAP32[_stderr>>2]|0;
  (_fprintf(($82|0),(2928|0),($vararg_buffer|0))|0);
  $0 = -510;
  $107 = $0;
  STACKTOP = sp;return ($107|0);
 }
 $pi = 63;
 $pixsum = 0;
 $found = 0;
 while(1) {
  $83 = $pi;
  $84 = ($83|0)>=(0);
  if (!($84)) {
   break;
  }
  $85 = $pi;
  $86 = (($pixtable) + ($85<<2)|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = $pixsum;
  $89 = (($88) + ($87))|0;
  $pixsum = $89;
  $90 = $pixsum;
  $91 = $prctthresh;
  $92 = ($90|0)>=($91|0);
  if ($92) {
   label = 25;
   break;
  }
  $94 = $pi;
  $95 = (($94) + -1)|0;
  $pi = $95;
 }
 if ((label|0) == 25) {
  $93 = $pi;
  $prctmax = $93;
  $found = 1;
 }
 $96 = $found;
 $97 = ($96|0)!=(0);
 if (!($97)) {
  $98 = HEAP32[_stderr>>2]|0;
  (_fprintf(($98|0),(2992|0),($vararg_buffer1|0))|0);
  $0 = -511;
  $107 = $0;
  STACKTOP = sp;return ($107|0);
 }
 $99 = $prctmax;
 $100 = $prctmin;
 $101 = (($99) - ($100))|0;
 $delta = $101;
 $102 = $delta;
 $103 = $6;
 $104 = (($103) + 76|0);
 $105 = HEAP32[$104>>2]|0;
 $106 = ($102|0)<($105|0);
 if ($106) {
  $0 = 1;
  $107 = $0;
  STACKTOP = sp;return ($107|0);
 } else {
  $0 = 0;
  $107 = $0;
  STACKTOP = sp;return ($107|0);
 }
 return 0|0;
}
function _find_valid_block($nbr_dir,$nbr_x,$nbr_y,$direction_map,$low_contrast_map,$sx,$sy,$mw,$mh,$x_incr,$y_incr) {
 $nbr_dir = $nbr_dir|0;
 $nbr_x = $nbr_x|0;
 $nbr_y = $nbr_y|0;
 $direction_map = $direction_map|0;
 $low_contrast_map = $low_contrast_map|0;
 $sx = $sx|0;
 $sy = $sy|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $x_incr = $x_incr|0;
 $y_incr = $y_incr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $dir = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $nbr_dir;
 $2 = $nbr_x;
 $3 = $nbr_y;
 $4 = $direction_map;
 $5 = $low_contrast_map;
 $6 = $sx;
 $7 = $sy;
 $8 = $mw;
 $9 = $mh;
 $10 = $x_incr;
 $11 = $y_incr;
 $12 = $6;
 $13 = $10;
 $14 = (($12) + ($13))|0;
 $x = $14;
 $15 = $7;
 $16 = $11;
 $17 = (($15) + ($16))|0;
 $y = $17;
 while(1) {
  $18 = $x;
  $19 = ($18|0)>=(0);
  if ($19) {
   $20 = $x;
   $21 = $8;
   $22 = ($20|0)<($21|0);
   if ($22) {
    $23 = $y;
    $24 = ($23|0)>=(0);
    if ($24) {
     $25 = $y;
     $26 = $9;
     $27 = ($25|0)<($26|0);
     $59 = $27;
    } else {
     $59 = 0;
    }
   } else {
    $59 = 0;
   }
  } else {
   $59 = 0;
  }
  if (!($59)) {
   label = 12;
   break;
  }
  $28 = $5;
  $29 = $y;
  $30 = $8;
  $31 = Math_imul($29, $30)|0;
  $32 = (($28) + ($31<<2)|0);
  $33 = $x;
  $34 = (($32) + ($33<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   label = 8;
   break;
  }
  $37 = $4;
  $38 = $y;
  $39 = $8;
  $40 = Math_imul($38, $39)|0;
  $41 = (($37) + ($40<<2)|0);
  $42 = $x;
  $43 = (($41) + ($42<<2)|0);
  $44 = HEAP32[$43>>2]|0;
  $dir = $44;
  $45 = ($44|0)>=(0);
  if ($45) {
   label = 10;
   break;
  }
  $52 = $10;
  $53 = $x;
  $54 = (($53) + ($52))|0;
  $x = $54;
  $55 = $11;
  $56 = $y;
  $57 = (($56) + ($55))|0;
  $y = $57;
 }
 if ((label|0) == 8) {
  $0 = 0;
  $58 = $0;
  STACKTOP = sp;return ($58|0);
 }
 else if ((label|0) == 10) {
  $46 = $dir;
  $47 = $1;
  HEAP32[$47>>2] = $46;
  $48 = $x;
  $49 = $2;
  HEAP32[$49>>2] = $48;
  $50 = $y;
  $51 = $3;
  HEAP32[$51>>2] = $50;
  $0 = 1;
  $58 = $0;
  STACKTOP = sp;return ($58|0);
 }
 else if ((label|0) == 12) {
  $0 = 0;
  $58 = $0;
  STACKTOP = sp;return ($58|0);
 }
 return 0|0;
}
function _set_margin_blocks($map,$mw,$mh,$margin_value) {
 $map = $map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $margin_value = $margin_value|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ptr1 = 0, $ptr2 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $map;
 $1 = $mw;
 $2 = $mh;
 $3 = $margin_value;
 $4 = $0;
 $ptr1 = $4;
 $5 = $0;
 $6 = $2;
 $7 = (($6) - 1)|0;
 $8 = $1;
 $9 = Math_imul($7, $8)|0;
 $10 = (($5) + ($9<<2)|0);
 $ptr2 = $10;
 $x = 0;
 while(1) {
  $11 = $x;
  $12 = $1;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $3;
  $15 = $ptr1;
  $16 = (($15) + 4|0);
  $ptr1 = $16;
  HEAP32[$15>>2] = $14;
  $17 = $3;
  $18 = $ptr2;
  $19 = (($18) + 4|0);
  $ptr2 = $19;
  HEAP32[$18>>2] = $17;
  $20 = $x;
  $21 = (($20) + 1)|0;
  $x = $21;
 }
 $22 = $0;
 $23 = $1;
 $24 = (($22) + ($23<<2)|0);
 $ptr1 = $24;
 $25 = $0;
 $26 = $1;
 $27 = (($25) + ($26<<2)|0);
 $28 = $1;
 $29 = (($27) + ($28<<2)|0);
 $30 = (($29) + -4|0);
 $ptr2 = $30;
 $y = 1;
 while(1) {
  $31 = $y;
  $32 = $2;
  $33 = (($32) - 1)|0;
  $34 = ($31|0)<($33|0);
  if (!($34)) {
   break;
  }
  $35 = $3;
  $36 = $ptr1;
  HEAP32[$36>>2] = $35;
  $37 = $3;
  $38 = $ptr2;
  HEAP32[$38>>2] = $37;
  $39 = $1;
  $40 = $ptr1;
  $41 = (($40) + ($39<<2)|0);
  $ptr1 = $41;
  $42 = $1;
  $43 = $ptr2;
  $44 = (($43) + ($42<<2)|0);
  $ptr2 = $44;
  $45 = $y;
  $46 = (($45) + 1)|0;
  $y = $46;
 }
 STACKTOP = sp;return;
}
function _allocate_contour($ocontour_x,$ocontour_y,$ocontour_ex,$ocontour_ey,$ncontour) {
 $ocontour_x = $ocontour_x|0;
 $ocontour_y = $ocontour_y|0;
 $ocontour_ex = $ocontour_ex|0;
 $ocontour_ey = $ocontour_ey|0;
 $ncontour = $ncontour|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ocontour_x;
 $2 = $ocontour_y;
 $3 = $ocontour_ex;
 $4 = $ocontour_ey;
 $5 = $ncontour;
 $6 = $5;
 $7 = $6<<2;
 $8 = (_malloc($7)|0);
 $contour_x = $8;
 $9 = $contour_x;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(3056|0),($vararg_buffer|0))|0);
  $0 = -180;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $12 = $5;
 $13 = $12<<2;
 $14 = (_malloc($13)|0);
 $contour_y = $14;
 $15 = $contour_y;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = $contour_x;
  _free($17);
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(3104|0),($vararg_buffer1|0))|0);
  $0 = -181;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $19 = $5;
 $20 = $19<<2;
 $21 = (_malloc($20)|0);
 $contour_ex = $21;
 $22 = $contour_ex;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = $contour_x;
  _free($24);
  $25 = $contour_y;
  _free($25);
  $26 = HEAP32[_stderr>>2]|0;
  (_fprintf(($26|0),(3152|0),($vararg_buffer3|0))|0);
  $0 = -182;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $27 = $5;
 $28 = $27<<2;
 $29 = (_malloc($28)|0);
 $contour_ey = $29;
 $30 = $contour_ey;
 $31 = ($30|0)==(0|0);
 if ($31) {
  $32 = $contour_x;
  _free($32);
  $33 = $contour_y;
  _free($33);
  $34 = $contour_ex;
  _free($34);
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(3200|0),($vararg_buffer5|0))|0);
  $0 = -183;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 } else {
  $36 = $contour_x;
  $37 = $1;
  HEAP32[$37>>2] = $36;
  $38 = $contour_y;
  $39 = $2;
  HEAP32[$39>>2] = $38;
  $40 = $contour_ex;
  $41 = $3;
  HEAP32[$41>>2] = $40;
  $42 = $contour_ey;
  $43 = $4;
  HEAP32[$43>>2] = $42;
  $0 = 0;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 return 0|0;
}
function _free_contour($contour_x,$contour_y,$contour_ex,$contour_ey) {
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $contour_ex = $contour_ex|0;
 $contour_ey = $contour_ey|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $contour_x;
 $1 = $contour_y;
 $2 = $contour_ex;
 $3 = $contour_ey;
 $4 = $0;
 _free($4);
 $5 = $1;
 _free($5);
 $6 = $2;
 _free($6);
 $7 = $3;
 _free($7);
 STACKTOP = sp;return;
}
function _get_high_curvature_contour($ocontour_x,$ocontour_y,$ocontour_ex,$ocontour_ey,$oncontour,$half_contour,$x_loc,$y_loc,$x_edge,$y_edge,$bdata,$iw,$ih) {
 $ocontour_x = $ocontour_x|0;
 $ocontour_y = $ocontour_y|0;
 $ocontour_ex = $ocontour_ex|0;
 $ocontour_ey = $ocontour_ey|0;
 $oncontour = $oncontour|0;
 $half_contour = $half_contour|0;
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $half1_ex = 0, $half1_ey = 0, $half1_x = 0, $half1_y = 0, $half2_ex = 0, $half2_ey = 0, $half2_x = 0, $half2_y = 0, $i = 0, $j = 0, $max_contour = 0;
 var $ncontour = 0, $nhalf1 = 0, $nhalf2 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $half1_x = sp + 68|0;
 $half1_y = sp + 64|0;
 $half1_ex = sp + 60|0;
 $half1_ey = sp + 56|0;
 $nhalf1 = sp + 52|0;
 $half2_x = sp + 48|0;
 $half2_y = sp + 44|0;
 $half2_ex = sp + 40|0;
 $half2_ey = sp + 36|0;
 $nhalf2 = sp + 32|0;
 $contour_x = sp + 28|0;
 $contour_y = sp + 24|0;
 $contour_ex = sp + 20|0;
 $contour_ey = sp + 16|0;
 $1 = $ocontour_x;
 $2 = $ocontour_y;
 $3 = $ocontour_ex;
 $4 = $ocontour_ey;
 $5 = $oncontour;
 $6 = $half_contour;
 $7 = $x_loc;
 $8 = $y_loc;
 $9 = $x_edge;
 $10 = $y_edge;
 $11 = $bdata;
 $12 = $iw;
 $13 = $ih;
 $14 = $6;
 $15 = $14 << 1;
 $16 = (($15) + 1)|0;
 $max_contour = $16;
 $17 = $5;
 HEAP32[$17>>2] = 0;
 $18 = $6;
 $19 = $7;
 $20 = $8;
 $21 = $7;
 $22 = $8;
 $23 = $9;
 $24 = $10;
 $25 = $11;
 $26 = $12;
 $27 = $13;
 $28 = (_trace_contour($half1_x,$half1_y,$half1_ex,$half1_ey,$nhalf1,$18,$19,$20,$21,$22,$23,$24,0,$25,$26,$27)|0);
 $ret = $28;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = $ret;
  $31 = ($30|0)==(2);
  if ($31) {
   $0 = 0;
   $284 = $0;
   STACKTOP = sp;return ($284|0);
  }
  $32 = $ret;
  $33 = ($32|0)==(1);
  if (!($33)) {
   $103 = $ret;
   $0 = $103;
   $284 = $0;
   STACKTOP = sp;return ($284|0);
  }
  $34 = HEAP32[$nhalf1>>2]|0;
  $35 = (($34) + 1)|0;
  $ncontour = $35;
  $36 = $ncontour;
  $37 = (_allocate_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$36)|0);
  $ret = $37;
  $38 = ($37|0)!=(0);
  if ($38) {
   $39 = HEAP32[$half1_x>>2]|0;
   $40 = HEAP32[$half1_y>>2]|0;
   $41 = HEAP32[$half1_ex>>2]|0;
   $42 = HEAP32[$half1_ey>>2]|0;
   _free_contour($39,$40,$41,$42);
   $43 = $ret;
   $0 = $43;
   $284 = $0;
   STACKTOP = sp;return ($284|0);
  }
  $44 = $7;
  $45 = HEAP32[$contour_x>>2]|0;
  HEAP32[$45>>2] = $44;
  $46 = $8;
  $47 = HEAP32[$contour_y>>2]|0;
  HEAP32[$47>>2] = $46;
  $48 = $9;
  $49 = HEAP32[$contour_ex>>2]|0;
  HEAP32[$49>>2] = $48;
  $50 = $10;
  $51 = HEAP32[$contour_ey>>2]|0;
  HEAP32[$51>>2] = $50;
  $i = 1;
  $52 = HEAP32[$nhalf1>>2]|0;
  $53 = (($52) - 1)|0;
  $j = $53;
  while(1) {
   $54 = $i;
   $55 = $ncontour;
   $56 = ($54|0)<($55|0);
   if (!($56)) {
    break;
   }
   $57 = $j;
   $58 = HEAP32[$half1_x>>2]|0;
   $59 = (($58) + ($57<<2)|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $i;
   $62 = HEAP32[$contour_x>>2]|0;
   $63 = (($62) + ($61<<2)|0);
   HEAP32[$63>>2] = $60;
   $64 = $j;
   $65 = HEAP32[$half1_y>>2]|0;
   $66 = (($65) + ($64<<2)|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = $i;
   $69 = HEAP32[$contour_y>>2]|0;
   $70 = (($69) + ($68<<2)|0);
   HEAP32[$70>>2] = $67;
   $71 = $j;
   $72 = HEAP32[$half1_ex>>2]|0;
   $73 = (($72) + ($71<<2)|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = $i;
   $76 = HEAP32[$contour_ex>>2]|0;
   $77 = (($76) + ($75<<2)|0);
   HEAP32[$77>>2] = $74;
   $78 = $j;
   $79 = HEAP32[$half1_ey>>2]|0;
   $80 = (($79) + ($78<<2)|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = $i;
   $83 = HEAP32[$contour_ey>>2]|0;
   $84 = (($83) + ($82<<2)|0);
   HEAP32[$84>>2] = $81;
   $85 = $i;
   $86 = (($85) + 1)|0;
   $i = $86;
   $87 = $j;
   $88 = (($87) + -1)|0;
   $j = $88;
  }
  $89 = HEAP32[$half1_x>>2]|0;
  $90 = HEAP32[$half1_y>>2]|0;
  $91 = HEAP32[$half1_ex>>2]|0;
  $92 = HEAP32[$half1_ey>>2]|0;
  _free_contour($89,$90,$91,$92);
  $93 = HEAP32[$contour_x>>2]|0;
  $94 = $1;
  HEAP32[$94>>2] = $93;
  $95 = HEAP32[$contour_y>>2]|0;
  $96 = $2;
  HEAP32[$96>>2] = $95;
  $97 = HEAP32[$contour_ex>>2]|0;
  $98 = $3;
  HEAP32[$98>>2] = $97;
  $99 = HEAP32[$contour_ey>>2]|0;
  $100 = $4;
  HEAP32[$100>>2] = $99;
  $101 = $ncontour;
  $102 = $5;
  HEAP32[$102>>2] = $101;
  $0 = 1;
  $284 = $0;
  STACKTOP = sp;return ($284|0);
 }
 $104 = HEAP32[$nhalf1>>2]|0;
 $105 = $6;
 $106 = ($104|0)<($105|0);
 if ($106) {
  $107 = HEAP32[$half1_x>>2]|0;
  $108 = HEAP32[$half1_y>>2]|0;
  $109 = HEAP32[$half1_ex>>2]|0;
  $110 = HEAP32[$half1_ey>>2]|0;
  _free_contour($107,$108,$109,$110);
  $0 = 0;
  $284 = $0;
  STACKTOP = sp;return ($284|0);
 }
 $111 = $6;
 $112 = HEAP32[$nhalf1>>2]|0;
 $113 = (($112) - 1)|0;
 $114 = HEAP32[$half1_x>>2]|0;
 $115 = (($114) + ($113<<2)|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = HEAP32[$nhalf1>>2]|0;
 $118 = (($117) - 1)|0;
 $119 = HEAP32[$half1_y>>2]|0;
 $120 = (($119) + ($118<<2)|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = $7;
 $123 = $8;
 $124 = $9;
 $125 = $10;
 $126 = $11;
 $127 = $12;
 $128 = $13;
 $129 = (_trace_contour($half2_x,$half2_y,$half2_ex,$half2_ey,$nhalf2,$111,$116,$121,$122,$123,$124,$125,1,$126,$127,$128)|0);
 $ret = $129;
 $130 = ($129|0)!=(0);
 do {
  if ($130) {
   $131 = $ret;
   $132 = ($131|0)==(2);
   if ($132) {
    $133 = HEAP32[$half1_x>>2]|0;
    $134 = HEAP32[$half1_y>>2]|0;
    $135 = HEAP32[$half1_ex>>2]|0;
    $136 = HEAP32[$half1_ey>>2]|0;
    _free_contour($133,$134,$135,$136);
    $0 = 0;
    $284 = $0;
    STACKTOP = sp;return ($284|0);
   }
   $137 = $ret;
   $138 = ($137|0)!=(1);
   if (!($138)) {
    break;
   }
   $139 = HEAP32[$half1_x>>2]|0;
   $140 = HEAP32[$half1_y>>2]|0;
   $141 = HEAP32[$half1_ex>>2]|0;
   $142 = HEAP32[$half1_ey>>2]|0;
   _free_contour($139,$140,$141,$142);
   $143 = $ret;
   $0 = $143;
   $284 = $0;
   STACKTOP = sp;return ($284|0);
  }
 } while(0);
 $144 = $ret;
 $145 = ($144|0)!=(1);
 if ($145) {
  $146 = HEAP32[$nhalf2>>2]|0;
  $147 = $6;
  $148 = ($146|0)<($147|0);
  if ($148) {
   $149 = HEAP32[$half1_x>>2]|0;
   $150 = HEAP32[$half1_y>>2]|0;
   $151 = HEAP32[$half1_ex>>2]|0;
   $152 = HEAP32[$half1_ey>>2]|0;
   _free_contour($149,$150,$151,$152);
   $153 = HEAP32[$half2_x>>2]|0;
   $154 = HEAP32[$half2_y>>2]|0;
   $155 = HEAP32[$half2_ex>>2]|0;
   $156 = HEAP32[$half2_ey>>2]|0;
   _free_contour($153,$154,$155,$156);
   $0 = 0;
   $284 = $0;
   STACKTOP = sp;return ($284|0);
  }
 }
 $157 = $max_contour;
 $158 = (_allocate_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$157)|0);
 $ret = $158;
 $159 = ($158|0)!=(0);
 if ($159) {
  $160 = HEAP32[$half1_x>>2]|0;
  $161 = HEAP32[$half1_y>>2]|0;
  $162 = HEAP32[$half1_ex>>2]|0;
  $163 = HEAP32[$half1_ey>>2]|0;
  _free_contour($160,$161,$162,$163);
  $164 = HEAP32[$half2_x>>2]|0;
  $165 = HEAP32[$half2_y>>2]|0;
  $166 = HEAP32[$half2_ex>>2]|0;
  $167 = HEAP32[$half2_ey>>2]|0;
  _free_contour($164,$165,$166,$167);
  $168 = $ret;
  $0 = $168;
  $284 = $0;
  STACKTOP = sp;return ($284|0);
 }
 $ncontour = 0;
 $i = 0;
 $169 = HEAP32[$nhalf1>>2]|0;
 $170 = (($169) - 1)|0;
 $j = $170;
 while(1) {
  $171 = $i;
  $172 = HEAP32[$nhalf1>>2]|0;
  $173 = ($171|0)<($172|0);
  if (!($173)) {
   break;
  }
  $174 = $j;
  $175 = HEAP32[$half1_x>>2]|0;
  $176 = (($175) + ($174<<2)|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = $i;
  $179 = HEAP32[$contour_x>>2]|0;
  $180 = (($179) + ($178<<2)|0);
  HEAP32[$180>>2] = $177;
  $181 = $j;
  $182 = HEAP32[$half1_y>>2]|0;
  $183 = (($182) + ($181<<2)|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = $i;
  $186 = HEAP32[$contour_y>>2]|0;
  $187 = (($186) + ($185<<2)|0);
  HEAP32[$187>>2] = $184;
  $188 = $j;
  $189 = HEAP32[$half1_ex>>2]|0;
  $190 = (($189) + ($188<<2)|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = $i;
  $193 = HEAP32[$contour_ex>>2]|0;
  $194 = (($193) + ($192<<2)|0);
  HEAP32[$194>>2] = $191;
  $195 = $j;
  $196 = HEAP32[$half1_ey>>2]|0;
  $197 = (($196) + ($195<<2)|0);
  $198 = HEAP32[$197>>2]|0;
  $199 = $i;
  $200 = HEAP32[$contour_ey>>2]|0;
  $201 = (($200) + ($199<<2)|0);
  HEAP32[$201>>2] = $198;
  $202 = $ncontour;
  $203 = (($202) + 1)|0;
  $ncontour = $203;
  $204 = $i;
  $205 = (($204) + 1)|0;
  $i = $205;
  $206 = $j;
  $207 = (($206) + -1)|0;
  $j = $207;
 }
 $208 = HEAP32[$half1_x>>2]|0;
 $209 = HEAP32[$half1_y>>2]|0;
 $210 = HEAP32[$half1_ex>>2]|0;
 $211 = HEAP32[$half1_ey>>2]|0;
 _free_contour($208,$209,$210,$211);
 $212 = $7;
 $213 = HEAP32[$nhalf1>>2]|0;
 $214 = HEAP32[$contour_x>>2]|0;
 $215 = (($214) + ($213<<2)|0);
 HEAP32[$215>>2] = $212;
 $216 = $8;
 $217 = HEAP32[$nhalf1>>2]|0;
 $218 = HEAP32[$contour_y>>2]|0;
 $219 = (($218) + ($217<<2)|0);
 HEAP32[$219>>2] = $216;
 $220 = $9;
 $221 = HEAP32[$nhalf1>>2]|0;
 $222 = HEAP32[$contour_ex>>2]|0;
 $223 = (($222) + ($221<<2)|0);
 HEAP32[$223>>2] = $220;
 $224 = $10;
 $225 = HEAP32[$nhalf1>>2]|0;
 $226 = HEAP32[$contour_ey>>2]|0;
 $227 = (($226) + ($225<<2)|0);
 HEAP32[$227>>2] = $224;
 $228 = $ncontour;
 $229 = (($228) + 1)|0;
 $ncontour = $229;
 $i = 0;
 $230 = HEAP32[$nhalf1>>2]|0;
 $231 = (($230) + 1)|0;
 $j = $231;
 while(1) {
  $232 = $i;
  $233 = HEAP32[$nhalf2>>2]|0;
  $234 = ($232|0)<($233|0);
  if (!($234)) {
   break;
  }
  $235 = $i;
  $236 = HEAP32[$half2_x>>2]|0;
  $237 = (($236) + ($235<<2)|0);
  $238 = HEAP32[$237>>2]|0;
  $239 = $j;
  $240 = HEAP32[$contour_x>>2]|0;
  $241 = (($240) + ($239<<2)|0);
  HEAP32[$241>>2] = $238;
  $242 = $i;
  $243 = HEAP32[$half2_y>>2]|0;
  $244 = (($243) + ($242<<2)|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = $j;
  $247 = HEAP32[$contour_y>>2]|0;
  $248 = (($247) + ($246<<2)|0);
  HEAP32[$248>>2] = $245;
  $249 = $i;
  $250 = HEAP32[$half2_ex>>2]|0;
  $251 = (($250) + ($249<<2)|0);
  $252 = HEAP32[$251>>2]|0;
  $253 = $j;
  $254 = HEAP32[$contour_ex>>2]|0;
  $255 = (($254) + ($253<<2)|0);
  HEAP32[$255>>2] = $252;
  $256 = $i;
  $257 = HEAP32[$half2_ey>>2]|0;
  $258 = (($257) + ($256<<2)|0);
  $259 = HEAP32[$258>>2]|0;
  $260 = $j;
  $261 = HEAP32[$contour_ey>>2]|0;
  $262 = (($261) + ($260<<2)|0);
  HEAP32[$262>>2] = $259;
  $263 = $ncontour;
  $264 = (($263) + 1)|0;
  $ncontour = $264;
  $265 = $i;
  $266 = (($265) + 1)|0;
  $i = $266;
  $267 = $j;
  $268 = (($267) + 1)|0;
  $j = $268;
 }
 $269 = HEAP32[$half2_x>>2]|0;
 $270 = HEAP32[$half2_y>>2]|0;
 $271 = HEAP32[$half2_ex>>2]|0;
 $272 = HEAP32[$half2_ey>>2]|0;
 _free_contour($269,$270,$271,$272);
 $273 = HEAP32[$contour_x>>2]|0;
 $274 = $1;
 HEAP32[$274>>2] = $273;
 $275 = HEAP32[$contour_y>>2]|0;
 $276 = $2;
 HEAP32[$276>>2] = $275;
 $277 = HEAP32[$contour_ex>>2]|0;
 $278 = $3;
 HEAP32[$278>>2] = $277;
 $279 = HEAP32[$contour_ey>>2]|0;
 $280 = $4;
 HEAP32[$280>>2] = $279;
 $281 = $ncontour;
 $282 = $5;
 HEAP32[$282>>2] = $281;
 $283 = $ret;
 $0 = $283;
 $284 = $0;
 STACKTOP = sp;return ($284|0);
}
function _trace_contour($ocontour_x,$ocontour_y,$ocontour_ex,$ocontour_ey,$oncontour,$max_len,$x_loop,$y_loop,$x_loc,$y_loc,$x_edge,$y_edge,$scan_clock,$bdata,$iw,$ih) {
 $ocontour_x = $ocontour_x|0;
 $ocontour_y = $ocontour_y|0;
 $ocontour_ex = $ocontour_ex|0;
 $ocontour_ey = $ocontour_ey|0;
 $oncontour = $oncontour|0;
 $max_len = $max_len|0;
 $x_loop = $x_loop|0;
 $y_loop = $y_loop|0;
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 $scan_clock = $scan_clock|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $contour_ex = 0, $contour_ey = 0;
 var $contour_x = 0, $contour_y = 0, $cur_x_edge = 0, $cur_x_loc = 0, $cur_y_edge = 0, $cur_y_loc = 0, $i = 0, $ncontour = 0, $next_x_edge = 0, $next_x_loc = 0, $next_y_edge = 0, $next_y_loc = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $contour_x = sp + 56|0;
 $contour_y = sp + 52|0;
 $contour_ex = sp + 48|0;
 $contour_ey = sp + 44|0;
 $next_x_loc = sp + 20|0;
 $next_y_loc = sp + 16|0;
 $next_x_edge = sp + 12|0;
 $next_y_edge = sp + 8|0;
 $1 = $ocontour_x;
 $2 = $ocontour_y;
 $3 = $ocontour_ex;
 $4 = $ocontour_ey;
 $5 = $oncontour;
 $6 = $max_len;
 $7 = $x_loop;
 $8 = $y_loop;
 $9 = $x_loc;
 $10 = $y_loc;
 $11 = $x_edge;
 $12 = $y_edge;
 $13 = $scan_clock;
 $14 = $bdata;
 $15 = $iw;
 $16 = $ih;
 $17 = $14;
 $18 = $10;
 $19 = $15;
 $20 = Math_imul($18, $19)|0;
 $21 = (($17) + ($20)|0);
 $22 = $9;
 $23 = (($21) + ($22)|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $26 = $14;
 $27 = $12;
 $28 = $15;
 $29 = Math_imul($27, $28)|0;
 $30 = (($26) + ($29)|0);
 $31 = $11;
 $32 = (($30) + ($31)|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = ($25|0)==($34|0);
 if ($35) {
  $0 = 2;
  $117 = $0;
  STACKTOP = sp;return ($117|0);
 }
 $36 = $6;
 $37 = (_allocate_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$36)|0);
 $ret = $37;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = $ret;
  $0 = $39;
  $117 = $0;
  STACKTOP = sp;return ($117|0);
 }
 $ncontour = 0;
 $40 = $9;
 $cur_x_loc = $40;
 $41 = $10;
 $cur_y_loc = $41;
 $42 = $11;
 $cur_x_edge = $42;
 $43 = $12;
 $cur_y_edge = $43;
 $i = 0;
 while(1) {
  $44 = $i;
  $45 = $6;
  $46 = ($44|0)<($45|0);
  if (!($46)) {
   label = 15;
   break;
  }
  $47 = $cur_x_loc;
  $48 = $cur_y_loc;
  $49 = $cur_x_edge;
  $50 = $cur_y_edge;
  $51 = $13;
  $52 = $14;
  $53 = $15;
  $54 = $16;
  $55 = (_next_contour_pixel($next_x_loc,$next_y_loc,$next_x_edge,$next_y_edge,$47,$48,$49,$50,$51,$52,$53,$54)|0);
  $56 = ($55|0)!=(0);
  if (!($56)) {
   label = 12;
   break;
  }
  $57 = HEAP32[$next_x_loc>>2]|0;
  $58 = $7;
  $59 = ($57|0)==($58|0);
  if ($59) {
   $60 = HEAP32[$next_y_loc>>2]|0;
   $61 = $8;
   $62 = ($60|0)==($61|0);
   if ($62) {
    label = 10;
    break;
   }
  }
  $73 = HEAP32[$next_x_loc>>2]|0;
  $74 = $i;
  $75 = HEAP32[$contour_x>>2]|0;
  $76 = (($75) + ($74<<2)|0);
  HEAP32[$76>>2] = $73;
  $77 = HEAP32[$next_y_loc>>2]|0;
  $78 = $i;
  $79 = HEAP32[$contour_y>>2]|0;
  $80 = (($79) + ($78<<2)|0);
  HEAP32[$80>>2] = $77;
  $81 = HEAP32[$next_x_edge>>2]|0;
  $82 = $i;
  $83 = HEAP32[$contour_ex>>2]|0;
  $84 = (($83) + ($82<<2)|0);
  HEAP32[$84>>2] = $81;
  $85 = HEAP32[$next_y_edge>>2]|0;
  $86 = $i;
  $87 = HEAP32[$contour_ey>>2]|0;
  $88 = (($87) + ($86<<2)|0);
  HEAP32[$88>>2] = $85;
  $89 = $ncontour;
  $90 = (($89) + 1)|0;
  $ncontour = $90;
  $91 = HEAP32[$next_x_loc>>2]|0;
  $cur_x_loc = $91;
  $92 = HEAP32[$next_y_loc>>2]|0;
  $cur_y_loc = $92;
  $93 = HEAP32[$next_x_edge>>2]|0;
  $cur_x_edge = $93;
  $94 = HEAP32[$next_y_edge>>2]|0;
  $cur_y_edge = $94;
  $105 = $i;
  $106 = (($105) + 1)|0;
  $i = $106;
 }
 if ((label|0) == 10) {
  $63 = HEAP32[$contour_x>>2]|0;
  $64 = $1;
  HEAP32[$64>>2] = $63;
  $65 = HEAP32[$contour_y>>2]|0;
  $66 = $2;
  HEAP32[$66>>2] = $65;
  $67 = HEAP32[$contour_ex>>2]|0;
  $68 = $3;
  HEAP32[$68>>2] = $67;
  $69 = HEAP32[$contour_ey>>2]|0;
  $70 = $4;
  HEAP32[$70>>2] = $69;
  $71 = $ncontour;
  $72 = $5;
  HEAP32[$72>>2] = $71;
  $0 = 1;
  $117 = $0;
  STACKTOP = sp;return ($117|0);
 }
 else if ((label|0) == 12) {
  $95 = HEAP32[$contour_x>>2]|0;
  $96 = $1;
  HEAP32[$96>>2] = $95;
  $97 = HEAP32[$contour_y>>2]|0;
  $98 = $2;
  HEAP32[$98>>2] = $97;
  $99 = HEAP32[$contour_ex>>2]|0;
  $100 = $3;
  HEAP32[$100>>2] = $99;
  $101 = HEAP32[$contour_ey>>2]|0;
  $102 = $4;
  HEAP32[$102>>2] = $101;
  $103 = $ncontour;
  $104 = $5;
  HEAP32[$104>>2] = $103;
  $0 = 0;
  $117 = $0;
  STACKTOP = sp;return ($117|0);
 }
 else if ((label|0) == 15) {
  $107 = HEAP32[$contour_x>>2]|0;
  $108 = $1;
  HEAP32[$108>>2] = $107;
  $109 = HEAP32[$contour_y>>2]|0;
  $110 = $2;
  HEAP32[$110>>2] = $109;
  $111 = HEAP32[$contour_ex>>2]|0;
  $112 = $3;
  HEAP32[$112>>2] = $111;
  $113 = HEAP32[$contour_ey>>2]|0;
  $114 = $4;
  HEAP32[$114>>2] = $113;
  $115 = $ncontour;
  $116 = $5;
  HEAP32[$116>>2] = $115;
  $0 = 0;
  $117 = $0;
  STACKTOP = sp;return ($117|0);
 }
 return 0|0;
}
function _get_centered_contour($ocontour_x,$ocontour_y,$ocontour_ex,$ocontour_ey,$oncontour,$half_contour,$x_loc,$y_loc,$x_edge,$y_edge,$bdata,$iw,$ih) {
 $ocontour_x = $ocontour_x|0;
 $ocontour_y = $ocontour_y|0;
 $ocontour_ex = $ocontour_ex|0;
 $ocontour_ey = $ocontour_ey|0;
 $oncontour = $oncontour|0;
 $half_contour = $half_contour|0;
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $half1_ex = 0, $half1_ey = 0, $half1_x = 0, $half1_y = 0, $half2_ex = 0, $half2_ey = 0, $half2_x = 0, $half2_y = 0, $i = 0, $j = 0, $max_contour = 0, $ncontour = 0, $nhalf1 = 0;
 var $nhalf2 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $half1_x = sp + 68|0;
 $half1_y = sp + 64|0;
 $half1_ex = sp + 60|0;
 $half1_ey = sp + 56|0;
 $nhalf1 = sp + 52|0;
 $half2_x = sp + 48|0;
 $half2_y = sp + 44|0;
 $half2_ex = sp + 40|0;
 $half2_ey = sp + 36|0;
 $nhalf2 = sp + 32|0;
 $contour_x = sp + 28|0;
 $contour_y = sp + 24|0;
 $contour_ex = sp + 20|0;
 $contour_ey = sp + 16|0;
 $1 = $ocontour_x;
 $2 = $ocontour_y;
 $3 = $ocontour_ex;
 $4 = $ocontour_ey;
 $5 = $oncontour;
 $6 = $half_contour;
 $7 = $x_loc;
 $8 = $y_loc;
 $9 = $x_edge;
 $10 = $y_edge;
 $11 = $bdata;
 $12 = $iw;
 $13 = $ih;
 $14 = $6;
 $15 = $14 << 1;
 $16 = (($15) + 1)|0;
 $max_contour = $16;
 $17 = $5;
 HEAP32[$17>>2] = 0;
 $18 = $6;
 $19 = $7;
 $20 = $8;
 $21 = $7;
 $22 = $8;
 $23 = $9;
 $24 = $10;
 $25 = $11;
 $26 = $12;
 $27 = $13;
 $28 = (_trace_contour($half1_x,$half1_y,$half1_ex,$half1_ey,$nhalf1,$18,$19,$20,$21,$22,$23,$24,0,$25,$26,$27)|0);
 $ret = $28;
 $29 = $ret;
 $30 = ($29|0)<(0);
 if ($30) {
  $31 = $ret;
  $0 = $31;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $32 = $ret;
 $33 = ($32|0)==(2);
 if ($33) {
  $0 = 2;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $34 = $ret;
 $35 = ($34|0)==(1);
 if ($35) {
  $36 = HEAP32[$half1_x>>2]|0;
  $37 = HEAP32[$half1_y>>2]|0;
  $38 = HEAP32[$half1_ex>>2]|0;
  $39 = HEAP32[$half1_ey>>2]|0;
  _free_contour($36,$37,$38,$39);
  $0 = 1;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $40 = HEAP32[$nhalf1>>2]|0;
 $41 = $6;
 $42 = ($40|0)<($41|0);
 if ($42) {
  $43 = HEAP32[$half1_x>>2]|0;
  $44 = HEAP32[$half1_y>>2]|0;
  $45 = HEAP32[$half1_ex>>2]|0;
  $46 = HEAP32[$half1_ey>>2]|0;
  _free_contour($43,$44,$45,$46);
  $0 = 3;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $47 = $6;
 $48 = HEAP32[$nhalf1>>2]|0;
 $49 = (($48) - 1)|0;
 $50 = HEAP32[$half1_x>>2]|0;
 $51 = (($50) + ($49<<2)|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$nhalf1>>2]|0;
 $54 = (($53) - 1)|0;
 $55 = HEAP32[$half1_y>>2]|0;
 $56 = (($55) + ($54<<2)|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $7;
 $59 = $8;
 $60 = $9;
 $61 = $10;
 $62 = $11;
 $63 = $12;
 $64 = $13;
 $65 = (_trace_contour($half2_x,$half2_y,$half2_ex,$half2_ey,$nhalf2,$47,$52,$57,$58,$59,$60,$61,1,$62,$63,$64)|0);
 $ret = $65;
 $66 = $ret;
 $67 = ($66|0)<(0);
 if ($67) {
  $68 = $ret;
  $0 = $68;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $69 = $ret;
 $70 = ($69|0)==(2);
 if ($70) {
  $71 = HEAP32[$half1_x>>2]|0;
  $72 = HEAP32[$half1_y>>2]|0;
  $73 = HEAP32[$half1_ex>>2]|0;
  $74 = HEAP32[$half1_ey>>2]|0;
  _free_contour($71,$72,$73,$74);
  $0 = 2;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $75 = $ret;
 $76 = ($75|0)==(1);
 if ($76) {
  $77 = HEAP32[$half1_x>>2]|0;
  $78 = HEAP32[$half1_y>>2]|0;
  $79 = HEAP32[$half1_ex>>2]|0;
  $80 = HEAP32[$half1_ey>>2]|0;
  _free_contour($77,$78,$79,$80);
  $81 = HEAP32[$half2_x>>2]|0;
  $82 = HEAP32[$half2_y>>2]|0;
  $83 = HEAP32[$half2_ex>>2]|0;
  $84 = HEAP32[$half2_ey>>2]|0;
  _free_contour($81,$82,$83,$84);
  $0 = 1;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $85 = HEAP32[$nhalf2>>2]|0;
 $86 = $6;
 $87 = ($85|0)<($86|0);
 if ($87) {
  $88 = HEAP32[$half1_x>>2]|0;
  $89 = HEAP32[$half1_y>>2]|0;
  $90 = HEAP32[$half1_ex>>2]|0;
  $91 = HEAP32[$half1_ey>>2]|0;
  _free_contour($88,$89,$90,$91);
  $92 = HEAP32[$half2_x>>2]|0;
  $93 = HEAP32[$half2_y>>2]|0;
  $94 = HEAP32[$half2_ex>>2]|0;
  $95 = HEAP32[$half2_ey>>2]|0;
  _free_contour($92,$93,$94,$95);
  $0 = 3;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $96 = $max_contour;
 $97 = (_allocate_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$96)|0);
 $ret = $97;
 $98 = ($97|0)!=(0);
 if ($98) {
  $99 = HEAP32[$half1_x>>2]|0;
  $100 = HEAP32[$half1_y>>2]|0;
  $101 = HEAP32[$half1_ex>>2]|0;
  $102 = HEAP32[$half1_ey>>2]|0;
  _free_contour($99,$100,$101,$102);
  $103 = HEAP32[$half2_x>>2]|0;
  $104 = HEAP32[$half2_y>>2]|0;
  $105 = HEAP32[$half2_ex>>2]|0;
  $106 = HEAP32[$half2_ey>>2]|0;
  _free_contour($103,$104,$105,$106);
  $107 = $ret;
  $0 = $107;
  $222 = $0;
  STACKTOP = sp;return ($222|0);
 }
 $ncontour = 0;
 $i = 0;
 $108 = HEAP32[$nhalf1>>2]|0;
 $109 = (($108) - 1)|0;
 $j = $109;
 while(1) {
  $110 = $i;
  $111 = HEAP32[$nhalf1>>2]|0;
  $112 = ($110|0)<($111|0);
  if (!($112)) {
   break;
  }
  $113 = $j;
  $114 = HEAP32[$half1_x>>2]|0;
  $115 = (($114) + ($113<<2)|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = $i;
  $118 = HEAP32[$contour_x>>2]|0;
  $119 = (($118) + ($117<<2)|0);
  HEAP32[$119>>2] = $116;
  $120 = $j;
  $121 = HEAP32[$half1_y>>2]|0;
  $122 = (($121) + ($120<<2)|0);
  $123 = HEAP32[$122>>2]|0;
  $124 = $i;
  $125 = HEAP32[$contour_y>>2]|0;
  $126 = (($125) + ($124<<2)|0);
  HEAP32[$126>>2] = $123;
  $127 = $j;
  $128 = HEAP32[$half1_ex>>2]|0;
  $129 = (($128) + ($127<<2)|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = $i;
  $132 = HEAP32[$contour_ex>>2]|0;
  $133 = (($132) + ($131<<2)|0);
  HEAP32[$133>>2] = $130;
  $134 = $j;
  $135 = HEAP32[$half1_ey>>2]|0;
  $136 = (($135) + ($134<<2)|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = $i;
  $139 = HEAP32[$contour_ey>>2]|0;
  $140 = (($139) + ($138<<2)|0);
  HEAP32[$140>>2] = $137;
  $141 = $ncontour;
  $142 = (($141) + 1)|0;
  $ncontour = $142;
  $143 = $i;
  $144 = (($143) + 1)|0;
  $i = $144;
  $145 = $j;
  $146 = (($145) + -1)|0;
  $j = $146;
 }
 $147 = HEAP32[$half1_x>>2]|0;
 $148 = HEAP32[$half1_y>>2]|0;
 $149 = HEAP32[$half1_ex>>2]|0;
 $150 = HEAP32[$half1_ey>>2]|0;
 _free_contour($147,$148,$149,$150);
 $151 = $7;
 $152 = HEAP32[$nhalf1>>2]|0;
 $153 = HEAP32[$contour_x>>2]|0;
 $154 = (($153) + ($152<<2)|0);
 HEAP32[$154>>2] = $151;
 $155 = $8;
 $156 = HEAP32[$nhalf1>>2]|0;
 $157 = HEAP32[$contour_y>>2]|0;
 $158 = (($157) + ($156<<2)|0);
 HEAP32[$158>>2] = $155;
 $159 = $9;
 $160 = HEAP32[$nhalf1>>2]|0;
 $161 = HEAP32[$contour_ex>>2]|0;
 $162 = (($161) + ($160<<2)|0);
 HEAP32[$162>>2] = $159;
 $163 = $10;
 $164 = HEAP32[$nhalf1>>2]|0;
 $165 = HEAP32[$contour_ey>>2]|0;
 $166 = (($165) + ($164<<2)|0);
 HEAP32[$166>>2] = $163;
 $167 = $ncontour;
 $168 = (($167) + 1)|0;
 $ncontour = $168;
 $i = 0;
 $169 = HEAP32[$nhalf1>>2]|0;
 $170 = (($169) + 1)|0;
 $j = $170;
 while(1) {
  $171 = $i;
  $172 = HEAP32[$nhalf2>>2]|0;
  $173 = ($171|0)<($172|0);
  if (!($173)) {
   break;
  }
  $174 = $i;
  $175 = HEAP32[$half2_x>>2]|0;
  $176 = (($175) + ($174<<2)|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = $j;
  $179 = HEAP32[$contour_x>>2]|0;
  $180 = (($179) + ($178<<2)|0);
  HEAP32[$180>>2] = $177;
  $181 = $i;
  $182 = HEAP32[$half2_y>>2]|0;
  $183 = (($182) + ($181<<2)|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = $j;
  $186 = HEAP32[$contour_y>>2]|0;
  $187 = (($186) + ($185<<2)|0);
  HEAP32[$187>>2] = $184;
  $188 = $i;
  $189 = HEAP32[$half2_ex>>2]|0;
  $190 = (($189) + ($188<<2)|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = $j;
  $193 = HEAP32[$contour_ex>>2]|0;
  $194 = (($193) + ($192<<2)|0);
  HEAP32[$194>>2] = $191;
  $195 = $i;
  $196 = HEAP32[$half2_ey>>2]|0;
  $197 = (($196) + ($195<<2)|0);
  $198 = HEAP32[$197>>2]|0;
  $199 = $j;
  $200 = HEAP32[$contour_ey>>2]|0;
  $201 = (($200) + ($199<<2)|0);
  HEAP32[$201>>2] = $198;
  $202 = $ncontour;
  $203 = (($202) + 1)|0;
  $ncontour = $203;
  $204 = $i;
  $205 = (($204) + 1)|0;
  $i = $205;
  $206 = $j;
  $207 = (($206) + 1)|0;
  $j = $207;
 }
 $208 = HEAP32[$half2_x>>2]|0;
 $209 = HEAP32[$half2_y>>2]|0;
 $210 = HEAP32[$half2_ex>>2]|0;
 $211 = HEAP32[$half2_ey>>2]|0;
 _free_contour($208,$209,$210,$211);
 $212 = HEAP32[$contour_x>>2]|0;
 $213 = $1;
 HEAP32[$213>>2] = $212;
 $214 = HEAP32[$contour_y>>2]|0;
 $215 = $2;
 HEAP32[$215>>2] = $214;
 $216 = HEAP32[$contour_ex>>2]|0;
 $217 = $3;
 HEAP32[$217>>2] = $216;
 $218 = HEAP32[$contour_ey>>2]|0;
 $219 = $4;
 HEAP32[$219>>2] = $218;
 $220 = $ncontour;
 $221 = $5;
 HEAP32[$221>>2] = $220;
 $0 = 0;
 $222 = $0;
 STACKTOP = sp;return ($222|0);
}
function _next_contour_pixel($next_x_loc,$next_y_loc,$next_x_edge,$next_y_edge,$cur_x_loc,$cur_y_loc,$cur_x_edge,$cur_y_edge,$scan_clock,$bdata,$iw,$ih) {
 $next_x_loc = $next_x_loc|0;
 $next_y_loc = $next_y_loc|0;
 $next_x_edge = $next_x_edge|0;
 $next_y_edge = $next_y_edge|0;
 $cur_x_loc = $cur_x_loc|0;
 $cur_y_loc = $cur_y_loc|0;
 $cur_x_edge = $cur_x_edge|0;
 $cur_y_edge = $cur_y_edge|0;
 $scan_clock = $scan_clock|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cur_nbr_pix = 0, $cur_nbr_x = 0, $cur_nbr_y = 0, $edge_pix = 0, $feature_pix = 0, $i = 0, $nbr_i = 0, $ni = 0, $npix = 0, $nx = 0, $ny = 0, $prev_nbr_pix = 0, $prev_nbr_x = 0, $prev_nbr_y = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $1 = $next_x_loc;
 $2 = $next_y_loc;
 $3 = $next_x_edge;
 $4 = $next_y_edge;
 $5 = $cur_x_loc;
 $6 = $cur_y_loc;
 $7 = $cur_x_edge;
 $8 = $cur_y_edge;
 $9 = $scan_clock;
 $10 = $bdata;
 $11 = $iw;
 $12 = $ih;
 $13 = $10;
 $14 = $6;
 $15 = $11;
 $16 = Math_imul($14, $15)|0;
 $17 = (($13) + ($16)|0);
 $18 = $5;
 $19 = (($17) + ($18)|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $feature_pix = $21;
 $22 = $10;
 $23 = $8;
 $24 = $11;
 $25 = Math_imul($23, $24)|0;
 $26 = (($22) + ($25)|0);
 $27 = $7;
 $28 = (($26) + ($27)|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = $29&255;
 $edge_pix = $30;
 $31 = $5;
 $32 = $6;
 $33 = $7;
 $34 = $8;
 $35 = (_start_scan_nbr($31,$32,$33,$34)|0);
 $nbr_i = $35;
 $36 = $7;
 $cur_nbr_x = $36;
 $37 = $8;
 $cur_nbr_y = $37;
 $38 = $edge_pix;
 $cur_nbr_pix = $38;
 $i = 0;
 while(1) {
  $39 = $i;
  $40 = ($39|0)<(8);
  if (!($40)) {
   label = 24;
   break;
  }
  $41 = $cur_nbr_x;
  $prev_nbr_x = $41;
  $42 = $cur_nbr_y;
  $prev_nbr_y = $42;
  $43 = $cur_nbr_pix;
  $prev_nbr_pix = $43;
  $44 = $nbr_i;
  $45 = $9;
  $46 = (_next_scan_nbr($44,$45)|0);
  $nbr_i = $46;
  $47 = $5;
  $48 = $nbr_i;
  $49 = (912 + ($48<<2)|0);
  $50 = HEAPU8[$49>>0]|(HEAPU8[$49+1>>0]<<8)|(HEAPU8[$49+2>>0]<<16)|(HEAPU8[$49+3>>0]<<24);
  $51 = (($47) + ($50))|0;
  $cur_nbr_x = $51;
  $52 = $6;
  $53 = $nbr_i;
  $54 = (944 + ($53<<2)|0);
  $55 = HEAPU8[$54>>0]|(HEAPU8[$54+1>>0]<<8)|(HEAPU8[$54+2>>0]<<16)|(HEAPU8[$54+3>>0]<<24);
  $56 = (($52) + ($55))|0;
  $cur_nbr_y = $56;
  $57 = $cur_nbr_x;
  $58 = ($57|0)<(0);
  if ($58) {
   label = 7;
   break;
  }
  $59 = $cur_nbr_x;
  $60 = $11;
  $61 = ($59|0)>=($60|0);
  if ($61) {
   label = 7;
   break;
  }
  $62 = $cur_nbr_y;
  $63 = ($62|0)<(0);
  if ($63) {
   label = 7;
   break;
  }
  $64 = $cur_nbr_y;
  $65 = $12;
  $66 = ($64|0)>=($65|0);
  if ($66) {
   label = 7;
   break;
  }
  $67 = $10;
  $68 = $cur_nbr_y;
  $69 = $11;
  $70 = Math_imul($68, $69)|0;
  $71 = (($67) + ($70)|0);
  $72 = $cur_nbr_x;
  $73 = (($71) + ($72)|0);
  $74 = HEAP8[$73>>0]|0;
  $75 = $74&255;
  $cur_nbr_pix = $75;
  $76 = $cur_nbr_pix;
  $77 = $feature_pix;
  $78 = ($76|0)==($77|0);
  if ($78) {
   $79 = $prev_nbr_pix;
   $80 = $edge_pix;
   $81 = ($79|0)==($80|0);
   if ($81) {
    $82 = $nbr_i;
    $83 = (($82|0) % 2)&-1;
    $84 = ($83|0)!=(0);
    if (!($84)) {
     label = 20;
     break;
    }
    $85 = $nbr_i;
    $86 = $9;
    $87 = (_next_scan_nbr($85,$86)|0);
    $ni = $87;
    $88 = $5;
    $89 = $ni;
    $90 = (912 + ($89<<2)|0);
    $91 = HEAPU8[$90>>0]|(HEAPU8[$90+1>>0]<<8)|(HEAPU8[$90+2>>0]<<16)|(HEAPU8[$90+3>>0]<<24);
    $92 = (($88) + ($91))|0;
    $nx = $92;
    $93 = $6;
    $94 = $ni;
    $95 = (944 + ($94<<2)|0);
    $96 = HEAPU8[$95>>0]|(HEAPU8[$95+1>>0]<<8)|(HEAPU8[$95+2>>0]<<16)|(HEAPU8[$95+3>>0]<<24);
    $97 = (($93) + ($96))|0;
    $ny = $97;
    $98 = $nx;
    $99 = ($98|0)<(0);
    if ($99) {
     label = 15;
     break;
    }
    $100 = $nx;
    $101 = $11;
    $102 = ($100|0)>=($101|0);
    if ($102) {
     label = 15;
     break;
    }
    $103 = $ny;
    $104 = ($103|0)<(0);
    if ($104) {
     label = 15;
     break;
    }
    $105 = $ny;
    $106 = $12;
    $107 = ($105|0)>=($106|0);
    if ($107) {
     label = 15;
     break;
    }
    $108 = $10;
    $109 = $ny;
    $110 = $11;
    $111 = Math_imul($109, $110)|0;
    $112 = (($108) + ($111)|0);
    $113 = $nx;
    $114 = (($112) + ($113)|0);
    $115 = HEAP8[$114>>0]|0;
    $116 = $115&255;
    $npix = $116;
    $117 = $npix;
    $118 = $feature_pix;
    $119 = ($117|0)==($118|0);
    if ($119) {
     label = 17;
     break;
    }
    $128 = $nx;
    $cur_nbr_x = $128;
    $129 = $ny;
    $cur_nbr_y = $129;
    $130 = $npix;
    $cur_nbr_pix = $130;
    $131 = $ni;
    $nbr_i = $131;
    $132 = $i;
    $133 = (($132) + 1)|0;
    $i = $133;
   }
  }
  $142 = $i;
  $143 = (($142) + 1)|0;
  $i = $143;
 }
 if ((label|0) == 7) {
  $0 = 0;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 else if ((label|0) == 15) {
  $0 = 0;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 else if ((label|0) == 17) {
  $120 = $cur_nbr_x;
  $121 = $1;
  HEAP32[$121>>2] = $120;
  $122 = $cur_nbr_y;
  $123 = $2;
  HEAP32[$123>>2] = $122;
  $124 = $prev_nbr_x;
  $125 = $3;
  HEAP32[$125>>2] = $124;
  $126 = $prev_nbr_y;
  $127 = $4;
  HEAP32[$127>>2] = $126;
  $0 = 1;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 else if ((label|0) == 20) {
  $134 = $cur_nbr_x;
  $135 = $1;
  HEAP32[$135>>2] = $134;
  $136 = $cur_nbr_y;
  $137 = $2;
  HEAP32[$137>>2] = $136;
  $138 = $prev_nbr_x;
  $139 = $3;
  HEAP32[$139>>2] = $138;
  $140 = $prev_nbr_y;
  $141 = $4;
  HEAP32[$141>>2] = $140;
  $0 = 1;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 else if ((label|0) == 24) {
  $0 = 0;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 return 0|0;
}
function _search_contour($x_search,$y_search,$search_len,$x_loc,$y_loc,$x_edge,$y_edge,$scan_clock,$bdata,$iw,$ih) {
 $x_search = $x_search|0;
 $y_search = $y_search|0;
 $search_len = $search_len|0;
 $x_loc = $x_loc|0;
 $y_loc = $y_loc|0;
 $x_edge = $x_edge|0;
 $y_edge = $y_edge|0;
 $scan_clock = $scan_clock|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $cur_x_edge = 0, $cur_x_loc = 0, $cur_y_edge = 0, $cur_y_loc = 0, $i = 0, $next_x_edge = 0, $next_x_loc = 0, $next_y_edge = 0, $next_y_loc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $next_x_loc = sp + 16|0;
 $next_y_loc = sp + 12|0;
 $next_x_edge = sp + 8|0;
 $next_y_edge = sp + 4|0;
 $1 = $x_search;
 $2 = $y_search;
 $3 = $search_len;
 $4 = $x_loc;
 $5 = $y_loc;
 $6 = $x_edge;
 $7 = $y_edge;
 $8 = $scan_clock;
 $9 = $bdata;
 $10 = $iw;
 $11 = $ih;
 $12 = $4;
 $cur_x_loc = $12;
 $13 = $5;
 $cur_y_loc = $13;
 $14 = $6;
 $cur_x_edge = $14;
 $15 = $7;
 $cur_y_edge = $15;
 $i = 0;
 while(1) {
  $16 = $i;
  $17 = $3;
  $18 = ($16|0)<($17|0);
  if (!($18)) {
   label = 11;
   break;
  }
  $19 = $cur_x_loc;
  $20 = $cur_y_loc;
  $21 = $cur_x_edge;
  $22 = $cur_y_edge;
  $23 = $8;
  $24 = $9;
  $25 = $10;
  $26 = $11;
  $27 = (_next_contour_pixel($next_x_loc,$next_y_loc,$next_x_edge,$next_y_edge,$19,$20,$21,$22,$23,$24,$25,$26)|0);
  $28 = ($27|0)!=(0);
  if (!($28)) {
   label = 8;
   break;
  }
  $29 = HEAP32[$next_x_loc>>2]|0;
  $30 = $1;
  $31 = ($29|0)==($30|0);
  if ($31) {
   $32 = HEAP32[$next_y_loc>>2]|0;
   $33 = $2;
   $34 = ($32|0)==($33|0);
   if ($34) {
    label = 6;
    break;
   }
  }
  $35 = HEAP32[$next_x_loc>>2]|0;
  $cur_x_loc = $35;
  $36 = HEAP32[$next_y_loc>>2]|0;
  $cur_y_loc = $36;
  $37 = HEAP32[$next_x_edge>>2]|0;
  $cur_x_edge = $37;
  $38 = HEAP32[$next_y_edge>>2]|0;
  $cur_y_edge = $38;
  $39 = $i;
  $40 = (($39) + 1)|0;
  $i = $40;
 }
 if ((label|0) == 6) {
  $0 = 1;
  $41 = $0;
  STACKTOP = sp;return ($41|0);
 }
 else if ((label|0) == 8) {
  $0 = 0;
  $41 = $0;
  STACKTOP = sp;return ($41|0);
 }
 else if ((label|0) == 11) {
  $0 = 0;
  $41 = $0;
  STACKTOP = sp;return ($41|0);
 }
 return 0|0;
}
function _start_scan_nbr($x_prev,$y_prev,$x_next,$y_next) {
 $x_prev = $x_prev|0;
 $y_prev = $y_prev|0;
 $x_next = $x_next|0;
 $y_next = $y_next|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $x_prev;
 $2 = $y_prev;
 $3 = $x_next;
 $4 = $y_next;
 $5 = $1;
 $6 = $3;
 $7 = ($5|0)==($6|0);
 if ($7) {
  $8 = $4;
  $9 = $2;
  $10 = ($8|0)>($9|0);
  if ($10) {
   $0 = 4;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 do {
  if ((label|0) == 4) {
   $11 = $1;
   $12 = $3;
   $13 = ($11|0)==($12|0);
   if ($13) {
    $14 = $4;
    $15 = $2;
    $16 = ($14|0)<($15|0);
    if ($16) {
     $0 = 0;
     break;
    }
   }
   $17 = $3;
   $18 = $1;
   $19 = ($17|0)>($18|0);
   if ($19) {
    $20 = $2;
    $21 = $4;
    $22 = ($20|0)==($21|0);
    if ($22) {
     $0 = 2;
     break;
    }
   }
   $23 = $3;
   $24 = $1;
   $25 = ($23|0)<($24|0);
   if ($25) {
    $26 = $2;
    $27 = $4;
    $28 = ($26|0)==($27|0);
    if ($28) {
     $0 = 6;
     break;
    }
   }
   $0 = -1;
  }
 } while(0);
 $29 = $0;
 STACKTOP = sp;return ($29|0);
}
function _next_scan_nbr($nbr_i,$scan_clock) {
 $nbr_i = $nbr_i|0;
 $scan_clock = $scan_clock|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $new_i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $nbr_i;
 $1 = $scan_clock;
 $2 = $1;
 $3 = ($2|0)==(0);
 if ($3) {
  $4 = $0;
  $5 = (($4) + 1)|0;
  $6 = (($5|0) % 8)&-1;
  $new_i = $6;
  $10 = $new_i;
  STACKTOP = sp;return ($10|0);
 } else {
  $7 = $0;
  $8 = (($7) + 7)|0;
  $9 = (($8|0) % 8)&-1;
  $new_i = $9;
  $10 = $new_i;
  STACKTOP = sp;return ($10|0);
 }
 return 0|0;
}
function _min_contour_theta($omin_i,$omin_theta,$angle_edge,$contour_x,$contour_y,$ncontour) {
 $omin_i = $omin_i|0;
 $omin_theta = $omin_theta|0;
 $angle_edge = $angle_edge|0;
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0, $117 = 0, $12 = 0.0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0, $24 = 0.0, $25 = 0.0, $26 = 0.0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0;
 var $83 = 0.0, $84 = 0.0, $85 = 0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0.0, $93 = 0.0, $94 = 0.0, $95 = 0.0, $96 = 0.0, $97 = 0, $98 = 0, $99 = 0.0, $dtheta = 0.0, $min_i = 0;
 var $min_theta = 0.0, $pcenter = 0, $pleft = 0, $pright = 0, $theta1 = 0.0, $theta2 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $omin_i;
 $2 = $omin_theta;
 $3 = $angle_edge;
 $4 = $contour_x;
 $5 = $contour_y;
 $6 = $ncontour;
 $7 = $6;
 $8 = $3;
 $9 = $8 << 1;
 $10 = (($9) + 1)|0;
 $11 = ($7|0)<($10|0);
 if ($11) {
  $0 = 2;
  $117 = $0;
  STACKTOP = sp;return ($117|0);
 }
 $min_theta = 3.141592653589793116;
 $12 = $min_theta;
 $13 = $12 < 0.0;
 if ($13) {
  $14 = $min_theta;
  $15 = $14 * 16384.0;
  $16 = $15 - 0.5;
  $17 = (~~(($16)));
  $18 = (+($17|0));
  $19 = $18 / 16384.0;
  $26 = $19;
 } else {
  $20 = $min_theta;
  $21 = $20 * 16384.0;
  $22 = $21 + 0.5;
  $23 = (~~(($22)));
  $24 = (+($23|0));
  $25 = $24 / 16384.0;
  $26 = $25;
 }
 $min_theta = $26;
 $min_i = -1;
 $pleft = 0;
 $27 = $3;
 $pcenter = $27;
 $28 = $pcenter;
 $29 = $3;
 $30 = (($28) + ($29))|0;
 $pright = $30;
 while(1) {
  $31 = $pright;
  $32 = $6;
  $33 = ($31|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = $pcenter;
  $35 = $4;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $pcenter;
  $39 = $5;
  $40 = (($39) + ($38<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $pleft;
  $43 = $4;
  $44 = (($43) + ($42<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $pleft;
  $47 = $5;
  $48 = (($47) + ($46<<2)|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (+_angle2line($37,$41,$45,$49));
  $theta1 = $50;
  $51 = $pcenter;
  $52 = $4;
  $53 = (($52) + ($51<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = $pcenter;
  $56 = $5;
  $57 = (($56) + ($55<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = $pright;
  $60 = $4;
  $61 = (($60) + ($59<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = $pright;
  $64 = $5;
  $65 = (($64) + ($63<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (+_angle2line($54,$58,$62,$66));
  $theta2 = $67;
  $68 = $theta2;
  $69 = $theta1;
  $70 = $68 - $69;
  $71 = (+Math_abs((+$70)));
  $dtheta = $71;
  $72 = $dtheta;
  $73 = $dtheta;
  $74 = 6.283185307179586232 - $73;
  $75 = $72 < $74;
  if ($75) {
   $76 = $dtheta;
   $79 = $76;
  } else {
   $77 = $dtheta;
   $78 = 6.283185307179586232 - $77;
   $79 = $78;
  }
  $dtheta = $79;
  $80 = $dtheta;
  $81 = $80 < 0.0;
  if ($81) {
   $82 = $dtheta;
   $83 = $82 * 16384.0;
   $84 = $83 - 0.5;
   $85 = (~~(($84)));
   $86 = (+($85|0));
   $87 = $86 / 16384.0;
   $94 = $87;
  } else {
   $88 = $dtheta;
   $89 = $88 * 16384.0;
   $90 = $89 + 0.5;
   $91 = (~~(($90)));
   $92 = (+($91|0));
   $93 = $92 / 16384.0;
   $94 = $93;
  }
  $dtheta = $94;
  $95 = $dtheta;
  $96 = $min_theta;
  $97 = $95 < $96;
  if ($97) {
   $98 = $pcenter;
   $min_i = $98;
   $99 = $dtheta;
   $min_theta = $99;
  }
  $100 = $pleft;
  $101 = (($100) + 1)|0;
  $pleft = $101;
  $102 = $pcenter;
  $103 = (($102) + 1)|0;
  $pcenter = $103;
  $104 = $pright;
  $105 = (($104) + 1)|0;
  $pright = $105;
 }
 $106 = $min_i;
 $107 = ($106|0)==(-1);
 if ($107) {
  $108 = $6;
  $109 = $108 >> 1;
  $110 = $1;
  HEAP32[$110>>2] = $109;
  $111 = $min_theta;
  $112 = $2;
  HEAPF64[$112>>3] = $111;
 } else {
  $113 = $min_i;
  $114 = $1;
  HEAP32[$114>>2] = $113;
  $115 = $min_theta;
  $116 = $2;
  HEAPF64[$116>>3] = $115;
 }
 $0 = 0;
 $117 = $0;
 STACKTOP = sp;return ($117|0);
}
function _contour_limits($xmin,$ymin,$xmax,$ymax,$contour_x,$contour_y,$ncontour) {
 $xmin = $xmin|0;
 $ymin = $ymin|0;
 $xmax = $xmax|0;
 $ymax = $ymax|0;
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $xmin;
 $1 = $ymin;
 $2 = $xmax;
 $3 = $ymax;
 $4 = $contour_x;
 $5 = $contour_y;
 $6 = $ncontour;
 $7 = $4;
 $8 = $6;
 $9 = (_minv($7,$8)|0);
 $10 = $0;
 HEAP32[$10>>2] = $9;
 $11 = $5;
 $12 = $6;
 $13 = (_minv($11,$12)|0);
 $14 = $1;
 HEAP32[$14>>2] = $13;
 $15 = $4;
 $16 = $6;
 $17 = (_maxv($15,$16)|0);
 $18 = $2;
 HEAP32[$18>>2] = $17;
 $19 = $5;
 $20 = $6;
 $21 = (_maxv($19,$20)|0);
 $22 = $3;
 HEAP32[$22>>2] = $21;
 STACKTOP = sp;return;
}
function _fix_edge_pixel_pair($feat_x,$feat_y,$edge_x,$edge_y,$bdata,$iw,$ih) {
 $feat_x = $feat_x|0;
 $feat_y = $feat_y|0;
 $edge_x = $edge_x|0;
 $edge_y = $edge_y|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $cx = 0, $cy = 0, $dx = 0, $dy = 0, $feature_pix = 0, $px = 0, $py = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $feat_x;
 $1 = $feat_y;
 $2 = $edge_x;
 $3 = $edge_y;
 $4 = $bdata;
 $5 = $iw;
 $6 = $ih;
 $7 = $4;
 $8 = $1;
 $9 = HEAP32[$8>>2]|0;
 $10 = $5;
 $11 = Math_imul($9, $10)|0;
 $12 = (($7) + ($11)|0);
 $13 = $0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + ($14)|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $feature_pix = $17;
 $18 = $0;
 $19 = HEAP32[$18>>2]|0;
 $cx = $19;
 $20 = $1;
 $21 = HEAP32[$20>>2]|0;
 $cy = $21;
 $22 = $2;
 $23 = HEAP32[$22>>2]|0;
 $px = $23;
 $24 = $3;
 $25 = HEAP32[$24>>2]|0;
 $py = $25;
 $26 = $px;
 $27 = $cx;
 $28 = (($26) - ($27))|0;
 $dx = $28;
 $29 = $py;
 $30 = $cy;
 $31 = (($29) - ($30))|0;
 $dy = $31;
 $32 = $dx;
 $33 = (Math_abs(($32|0))|0);
 $34 = ($33|0)==(1);
 if (!($34)) {
  STACKTOP = sp;return;
 }
 $35 = $dy;
 $36 = (Math_abs(($35|0))|0);
 $37 = ($36|0)==(1);
 if (!($37)) {
  STACKTOP = sp;return;
 }
 $38 = $4;
 $39 = $py;
 $40 = $5;
 $41 = Math_imul($39, $40)|0;
 $42 = (($38) + ($41)|0);
 $43 = $px;
 $44 = $dx;
 $45 = (($43) - ($44))|0;
 $46 = (($42) + ($45)|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47&255;
 $49 = $feature_pix;
 $50 = ($48|0)!=($49|0);
 if ($50) {
  $51 = $dx;
  $52 = $px;
  $53 = (($52) - ($51))|0;
  $px = $53;
 } else {
  $54 = $4;
  $55 = $py;
  $56 = $dy;
  $57 = (($55) - ($56))|0;
  $58 = $5;
  $59 = Math_imul($57, $58)|0;
  $60 = (($54) + ($59)|0);
  $61 = $px;
  $62 = (($60) + ($61)|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = $63&255;
  $65 = $feature_pix;
  $66 = ($64|0)!=($65|0);
  if ($66) {
   $67 = $dy;
   $68 = $py;
   $69 = (($68) - ($67))|0;
   $py = $69;
  } else {
   $70 = $dy;
   $71 = $cy;
   $72 = (($71) + ($70))|0;
   $cy = $72;
  }
 }
 $73 = $cx;
 $74 = $0;
 HEAP32[$74>>2] = $73;
 $75 = $cy;
 $76 = $1;
 HEAP32[$76>>2] = $75;
 $77 = $px;
 $78 = $2;
 HEAP32[$78>>2] = $77;
 $79 = $py;
 $80 = $3;
 HEAP32[$80>>2] = $79;
 STACKTOP = sp;return;
}
function _lfs_detect_minutiae_V2($ominutiae,$odmap,$olcmap,$olfmap,$ohcmap,$omw,$omh,$obdata,$obw,$obh,$idata,$iw,$ih,$lfsparms) {
 $ominutiae = $ominutiae|0;
 $odmap = $odmap|0;
 $olcmap = $olcmap|0;
 $olfmap = $olfmap|0;
 $ohcmap = $ohcmap|0;
 $omw = $omw|0;
 $omh = $omh|0;
 $obdata = $obdata|0;
 $obw = $obw|0;
 $obh = $obh|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $bdata = 0, $bh = 0, $bw = 0, $dftgrids = 0, $dftwaves = 0, $dir2rad = 0, $dirbingrids = 0, $direction_map = 0, $high_curve_map = 0, $low_contrast_map = 0, $low_flow_map = 0, $maxpad = 0, $mh = 0, $minutiae = 0, $mw = 0, $pdata = 0, $ph = 0, $pw = 0;
 var $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $pdata = sp + 132|0;
 $bdata = sp + 128|0;
 $pw = sp + 124|0;
 $ph = sp + 120|0;
 $bw = sp + 116|0;
 $bh = sp + 112|0;
 $dir2rad = sp + 108|0;
 $dftwaves = sp + 104|0;
 $dftgrids = sp + 100|0;
 $dirbingrids = sp + 96|0;
 $direction_map = sp + 92|0;
 $low_contrast_map = sp + 88|0;
 $low_flow_map = sp + 84|0;
 $high_curve_map = sp + 80|0;
 $mw = sp + 76|0;
 $mh = sp + 72|0;
 $minutiae = sp + 60|0;
 $1 = $ominutiae;
 $2 = $odmap;
 $3 = $olcmap;
 $4 = $olfmap;
 $5 = $ohcmap;
 $6 = $omw;
 $7 = $omh;
 $8 = $obdata;
 $9 = $obw;
 $10 = $obh;
 $11 = $idata;
 $12 = $iw;
 $13 = $ih;
 $14 = $lfsparms;
 $15 = (_open_logfile()|0);
 $ret = $15;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = $ret;
  $0 = $17;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $18 = $14;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $14;
 $22 = (($21) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $14;
 $25 = (($24) + 136|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $14;
 $28 = (($27) + 140|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_get_max_padding_V2($20,$23,$26,$29)|0);
 $maxpad = $30;
 $31 = $14;
 $32 = (($31) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (_init_dir2rad($dir2rad,$33)|0);
 $ret = $34;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = $ret;
  $0 = $36;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $37 = $14;
 $38 = (($37) + 80|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = $14;
 $41 = (($40) + 12|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (_init_dftwaves($dftwaves,536,$39,$42)|0);
 $ret = $43;
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = HEAP32[$dir2rad>>2]|0;
  _free_dir2rad($45);
  $46 = $ret;
  $0 = $46;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $47 = $12;
 $48 = $13;
 $49 = $maxpad;
 $50 = $14;
 $51 = (($50) + 24|0);
 $52 = +HEAPF64[$51>>3];
 $53 = $14;
 $54 = (($53) + 20|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = $14;
 $57 = (($56) + 12|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $14;
 $60 = (($59) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (_init_rotgrids($dftgrids,$47,$48,$49,$52,$55,$58,$61,1)|0);
 $ret = $62;
 $63 = ($62|0)!=(0);
 if ($63) {
  $64 = HEAP32[$dir2rad>>2]|0;
  _free_dir2rad($64);
  $65 = HEAP32[$dftwaves>>2]|0;
  _free_dftwaves($65);
  $66 = $ret;
  $0 = $66;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $67 = $maxpad;
 $68 = ($67|0)>(0);
 do {
  if ($68) {
   $69 = $11;
   $70 = $12;
   $71 = $13;
   $72 = $maxpad;
   $73 = $14;
   $74 = HEAP32[$73>>2]|0;
   $75 = (_pad_uchar_image($pdata,$pw,$ph,$69,$70,$71,$72,$74)|0);
   $ret = $75;
   $76 = ($75|0)!=(0);
   if (!($76)) {
    break;
   }
   $77 = HEAP32[$dir2rad>>2]|0;
   _free_dir2rad($77);
   $78 = HEAP32[$dftwaves>>2]|0;
   _free_dftwaves($78);
   $79 = HEAP32[$dftgrids>>2]|0;
   _free_rotgrids($79);
   $80 = $ret;
   $0 = $80;
   $261 = $0;
   STACKTOP = sp;return ($261|0);
  } else {
   $81 = $12;
   $82 = $13;
   $83 = Math_imul($81, $82)|0;
   $84 = (_malloc($83)|0);
   HEAP32[$pdata>>2] = $84;
   $85 = HEAP32[$pdata>>2]|0;
   $86 = ($85|0)==(0|0);
   if (!($86)) {
    $91 = HEAP32[$pdata>>2]|0;
    $92 = $11;
    $93 = $12;
    $94 = $13;
    $95 = Math_imul($93, $94)|0;
    _memcpy(($91|0),($92|0),($95|0))|0;
    $96 = $12;
    HEAP32[$pw>>2] = $96;
    $97 = $13;
    HEAP32[$ph>>2] = $97;
    break;
   }
   $87 = HEAP32[$dir2rad>>2]|0;
   _free_dir2rad($87);
   $88 = HEAP32[$dftwaves>>2]|0;
   _free_dftwaves($88);
   $89 = HEAP32[$dftgrids>>2]|0;
   _free_rotgrids($89);
   $90 = HEAP32[_stderr>>2]|0;
   (_fprintf(($90|0),(3376|0),($vararg_buffer|0))|0);
   $0 = -580;
   $261 = $0;
   STACKTOP = sp;return ($261|0);
  }
 } while(0);
 $98 = HEAP32[$pdata>>2]|0;
 $99 = HEAP32[$pw>>2]|0;
 $100 = HEAP32[$ph>>2]|0;
 _bits_8to6($98,$99,$100);
 _print2log(3248,$vararg_buffer1);
 $101 = HEAP32[$pdata>>2]|0;
 $102 = HEAP32[$pw>>2]|0;
 $103 = HEAP32[$ph>>2]|0;
 $104 = HEAP32[$dir2rad>>2]|0;
 $105 = HEAP32[$dftwaves>>2]|0;
 $106 = HEAP32[$dftgrids>>2]|0;
 $107 = $14;
 $108 = (_gen_image_maps($direction_map,$low_contrast_map,$low_flow_map,$high_curve_map,$mw,$mh,$101,$102,$103,$104,$105,$106,$107)|0);
 $ret = $108;
 $109 = ($108|0)!=(0);
 if ($109) {
  $110 = HEAP32[$dir2rad>>2]|0;
  _free_dir2rad($110);
  $111 = HEAP32[$dftwaves>>2]|0;
  _free_dftwaves($111);
  $112 = HEAP32[$dftgrids>>2]|0;
  _free_rotgrids($112);
  $113 = HEAP32[$pdata>>2]|0;
  _free($113);
  $114 = $ret;
  $0 = $114;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $115 = HEAP32[$dir2rad>>2]|0;
 _free_dir2rad($115);
 $116 = HEAP32[$dftwaves>>2]|0;
 _free_dftwaves($116);
 $117 = HEAP32[$dftgrids>>2]|0;
 _free_rotgrids($117);
 _print2log(3432,$vararg_buffer3);
 $118 = $12;
 $119 = $13;
 $120 = $maxpad;
 $121 = $14;
 $122 = (($121) + 24|0);
 $123 = +HEAPF64[$122>>3];
 $124 = $14;
 $125 = (($124) + 20|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = $14;
 $128 = (($127) + 136|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = $14;
 $131 = (($130) + 140|0);
 $132 = HEAP32[$131>>2]|0;
 $133 = (_init_rotgrids($dirbingrids,$118,$119,$120,$123,$126,$129,$132,0)|0);
 $ret = $133;
 $134 = ($133|0)!=(0);
 if ($134) {
  $135 = HEAP32[$pdata>>2]|0;
  _free($135);
  $136 = HEAP32[$direction_map>>2]|0;
  _free($136);
  $137 = HEAP32[$low_contrast_map>>2]|0;
  _free($137);
  $138 = HEAP32[$low_flow_map>>2]|0;
  _free($138);
  $139 = HEAP32[$high_curve_map>>2]|0;
  _free($139);
  $140 = $ret;
  $0 = $140;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $141 = HEAP32[$pdata>>2]|0;
 $142 = HEAP32[$pw>>2]|0;
 $143 = HEAP32[$ph>>2]|0;
 $144 = HEAP32[$direction_map>>2]|0;
 $145 = HEAP32[$mw>>2]|0;
 $146 = HEAP32[$mh>>2]|0;
 $147 = HEAP32[$dirbingrids>>2]|0;
 $148 = $14;
 $149 = (_binarize_V2($bdata,$bw,$bh,$141,$142,$143,$144,$145,$146,$147,$148)|0);
 $ret = $149;
 $150 = ($149|0)!=(0);
 if ($150) {
  $151 = HEAP32[$pdata>>2]|0;
  _free($151);
  $152 = HEAP32[$direction_map>>2]|0;
  _free($152);
  $153 = HEAP32[$low_contrast_map>>2]|0;
  _free($153);
  $154 = HEAP32[$low_flow_map>>2]|0;
  _free($154);
  $155 = HEAP32[$high_curve_map>>2]|0;
  _free($155);
  $156 = HEAP32[$dirbingrids>>2]|0;
  _free_rotgrids($156);
  $157 = $ret;
  $0 = $157;
  $261 = $0;
  STACKTOP = sp;return ($261|0);
 }
 $158 = HEAP32[$dirbingrids>>2]|0;
 _free_rotgrids($158);
 $159 = $12;
 $160 = HEAP32[$bw>>2]|0;
 $161 = ($159|0)!=($160|0);
 if (!($161)) {
  $162 = $13;
  $163 = HEAP32[$bh>>2]|0;
  $164 = ($162|0)!=($163|0);
  if (!($164)) {
   _print2log(3288,$vararg_buffer10);
   $175 = HEAP32[$bdata>>2]|0;
   $176 = $12;
   $177 = $13;
   _gray2bin(1,1,0,$175,$176,$177);
   $178 = (_alloc_minutiae($minutiae,1000)|0);
   $ret = $178;
   $179 = ($178|0)!=(0);
   if ($179) {
    $180 = $ret;
    $0 = $180;
    $261 = $0;
    STACKTOP = sp;return ($261|0);
   }
   $181 = HEAP32[$minutiae>>2]|0;
   $182 = HEAP32[$bdata>>2]|0;
   $183 = $12;
   $184 = $13;
   $185 = HEAP32[$direction_map>>2]|0;
   $186 = HEAP32[$low_flow_map>>2]|0;
   $187 = HEAP32[$high_curve_map>>2]|0;
   $188 = HEAP32[$mw>>2]|0;
   $189 = HEAP32[$mh>>2]|0;
   $190 = $14;
   $191 = (_detect_minutiae_V2($181,$182,$183,$184,$185,$186,$187,$188,$189,$190)|0);
   $ret = $191;
   $192 = ($191|0)!=(0);
   if ($192) {
    $193 = HEAP32[$pdata>>2]|0;
    _free($193);
    $194 = HEAP32[$direction_map>>2]|0;
    _free($194);
    $195 = HEAP32[$low_contrast_map>>2]|0;
    _free($195);
    $196 = HEAP32[$low_flow_map>>2]|0;
    _free($196);
    $197 = HEAP32[$high_curve_map>>2]|0;
    _free($197);
    $198 = HEAP32[$bdata>>2]|0;
    _free($198);
    $199 = $ret;
    $0 = $199;
    $261 = $0;
    STACKTOP = sp;return ($261|0);
   }
   $200 = HEAP32[$minutiae>>2]|0;
   $201 = HEAP32[$bdata>>2]|0;
   $202 = $12;
   $203 = $13;
   $204 = HEAP32[$direction_map>>2]|0;
   $205 = HEAP32[$low_flow_map>>2]|0;
   $206 = HEAP32[$high_curve_map>>2]|0;
   $207 = HEAP32[$mw>>2]|0;
   $208 = HEAP32[$mh>>2]|0;
   $209 = $14;
   $210 = (_remove_false_minutia_V2($200,$201,$202,$203,$204,$205,$206,$207,$208,$209)|0);
   $ret = $210;
   $211 = ($210|0)!=(0);
   if ($211) {
    $212 = HEAP32[$pdata>>2]|0;
    _free($212);
    $213 = HEAP32[$direction_map>>2]|0;
    _free($213);
    $214 = HEAP32[$low_contrast_map>>2]|0;
    _free($214);
    $215 = HEAP32[$low_flow_map>>2]|0;
    _free($215);
    $216 = HEAP32[$high_curve_map>>2]|0;
    _free($216);
    $217 = HEAP32[$bdata>>2]|0;
    _free($217);
    $218 = HEAP32[$minutiae>>2]|0;
    _free_minutiae($218);
    $219 = $ret;
    $0 = $219;
    $261 = $0;
    STACKTOP = sp;return ($261|0);
   }
   _print2log(3312,$vararg_buffer12);
   $220 = HEAP32[$minutiae>>2]|0;
   $221 = HEAP32[$bdata>>2]|0;
   $222 = $12;
   $223 = $13;
   $224 = $14;
   $225 = (_count_minutiae_ridges($220,$221,$222,$223,$224)|0);
   $ret = $225;
   $226 = ($225|0)!=(0);
   if ($226) {
    $227 = HEAP32[$pdata>>2]|0;
    _free($227);
    $228 = HEAP32[$direction_map>>2]|0;
    _free($228);
    $229 = HEAP32[$low_contrast_map>>2]|0;
    _free($229);
    $230 = HEAP32[$low_flow_map>>2]|0;
    _free($230);
    $231 = HEAP32[$high_curve_map>>2]|0;
    _free($231);
    $232 = HEAP32[$minutiae>>2]|0;
    _free_minutiae($232);
    $233 = $ret;
    $0 = $233;
    $261 = $0;
    STACKTOP = sp;return ($261|0);
   }
   _print2log(3344,$vararg_buffer14);
   $234 = HEAP32[$bdata>>2]|0;
   $235 = $12;
   $236 = $13;
   _gray2bin(1,255,0,$234,$235,$236);
   $237 = HEAP32[$pdata>>2]|0;
   _free($237);
   $238 = HEAP32[$direction_map>>2]|0;
   $239 = $2;
   HEAP32[$239>>2] = $238;
   $240 = HEAP32[$low_contrast_map>>2]|0;
   $241 = $3;
   HEAP32[$241>>2] = $240;
   $242 = HEAP32[$low_flow_map>>2]|0;
   $243 = $4;
   HEAP32[$243>>2] = $242;
   $244 = HEAP32[$high_curve_map>>2]|0;
   $245 = $5;
   HEAP32[$245>>2] = $244;
   $246 = HEAP32[$mw>>2]|0;
   $247 = $6;
   HEAP32[$247>>2] = $246;
   $248 = HEAP32[$mh>>2]|0;
   $249 = $7;
   HEAP32[$249>>2] = $248;
   $250 = HEAP32[$bdata>>2]|0;
   $251 = $8;
   HEAP32[$251>>2] = $250;
   $252 = HEAP32[$bw>>2]|0;
   $253 = $9;
   HEAP32[$253>>2] = $252;
   $254 = HEAP32[$bh>>2]|0;
   $255 = $10;
   HEAP32[$255>>2] = $254;
   $256 = HEAP32[$minutiae>>2]|0;
   $257 = $1;
   HEAP32[$257>>2] = $256;
   $258 = (_close_logfile()|0);
   $ret = $258;
   $259 = ($258|0)!=(0);
   if ($259) {
    $260 = $ret;
    $0 = $260;
    $261 = $0;
    STACKTOP = sp;return ($261|0);
   } else {
    $0 = 0;
    $261 = $0;
    STACKTOP = sp;return ($261|0);
   }
  }
 }
 $165 = HEAP32[$pdata>>2]|0;
 _free($165);
 $166 = HEAP32[$direction_map>>2]|0;
 _free($166);
 $167 = HEAP32[$low_contrast_map>>2]|0;
 _free($167);
 $168 = HEAP32[$low_flow_map>>2]|0;
 _free($168);
 $169 = HEAP32[$high_curve_map>>2]|0;
 _free($169);
 $170 = HEAP32[$bdata>>2]|0;
 _free($170);
 $171 = HEAP32[_stderr>>2]|0;
 (_fprintf(($171|0),(3448|0),($vararg_buffer5|0))|0);
 $172 = HEAP32[_stderr>>2]|0;
 $173 = HEAP32[$bw>>2]|0;
 $174 = HEAP32[$bh>>2]|0;
 HEAP32[$vararg_buffer7>>2] = $173;
 $vararg_ptr9 = (($vararg_buffer7) + 4|0);
 HEAP32[$vararg_ptr9>>2] = $174;
 (_fprintf(($172|0),(3488|0),($vararg_buffer7|0))|0);
 $0 = -581;
 $261 = $0;
 STACKTOP = sp;return ($261|0);
}
function _free_dir2rad($dir2rad) {
 $dir2rad = $dir2rad|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dir2rad;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 _free($3);
 $4 = $0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 _free($6);
 $7 = $0;
 _free($7);
 STACKTOP = sp;return;
}
function _free_dftwaves($dftwaves) {
 $dftwaves = $dftwaves|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dftwaves;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = HEAP32[$2>>2]|0;
  $4 = ($1|0)<($3|0);
  if (!($4)) {
   break;
  }
  $5 = $i;
  $6 = $0;
  $7 = (($6) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + ($5<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  _free($11);
  $12 = $i;
  $13 = $0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  _free($19);
  $20 = $i;
  $21 = $0;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  _free($25);
  $26 = $i;
  $27 = (($26) + 1)|0;
  $i = $27;
 }
 $28 = $0;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 _free($30);
 $31 = $0;
 _free($31);
 STACKTOP = sp;return;
}
function _free_rotgrids($rotgrids) {
 $rotgrids = $rotgrids|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $rotgrids;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 16|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 28|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free($11);
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $0;
 $15 = (($14) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 _free($16);
 $17 = $0;
 _free($17);
 STACKTOP = sp;return;
}
function _free_dir_powers($powers,$nwaves) {
 $powers = $powers|0;
 $nwaves = $nwaves|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $powers;
 $1 = $nwaves;
 $w = 0;
 while(1) {
  $2 = $w;
  $3 = $1;
  $4 = ($2|0)<($3|0);
  if (!($4)) {
   break;
  }
  $5 = $w;
  $6 = $0;
  $7 = (($6) + ($5<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  _free($8);
  $9 = $w;
  $10 = (($9) + 1)|0;
  $w = $10;
 }
 $11 = $0;
 _free($11);
 STACKTOP = sp;return;
}
function _bits_8to6($idata,$iw,$ih) {
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iptr = 0;
 var $isize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $idata;
 $1 = $iw;
 $2 = $ih;
 $3 = $1;
 $4 = $2;
 $5 = Math_imul($3, $4)|0;
 $isize = $5;
 $6 = $0;
 $iptr = $6;
 $i = 0;
 while(1) {
  $7 = $i;
  $8 = $isize;
  $9 = ($7|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $iptr;
  $11 = (($10) + 1|0);
  $iptr = $11;
  $12 = HEAP8[$10>>0]|0;
  $13 = $12&255;
  $14 = $13 >> 2;
  $15 = $14&255;
  HEAP8[$10>>0] = $15;
  $16 = $i;
  $17 = (($16) + 1)|0;
  $i = $17;
 }
 STACKTOP = sp;return;
}
function _gray2bin($thresh,$less_pix,$greater_pix,$bdata,$iw,$ih) {
 $thresh = $thresh|0;
 $less_pix = $less_pix|0;
 $greater_pix = $greater_pix|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $thresh;
 $1 = $less_pix;
 $2 = $greater_pix;
 $3 = $bdata;
 $4 = $iw;
 $5 = $ih;
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $4;
  $8 = $5;
  $9 = Math_imul($7, $8)|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $i;
  $12 = $3;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = $0;
  $17 = ($15|0)>=($16|0);
  if ($17) {
   $18 = $2;
   $19 = $18&255;
   $20 = $i;
   $21 = $3;
   $22 = (($21) + ($20)|0);
   HEAP8[$22>>0] = $19;
  } else {
   $23 = $1;
   $24 = $23&255;
   $25 = $i;
   $26 = $3;
   $27 = (($26) + ($25)|0);
   HEAP8[$27>>0] = $24;
  }
  $28 = $i;
  $29 = (($28) + 1)|0;
  $i = $29;
 }
 STACKTOP = sp;return;
}
function _pad_uchar_image($optr,$ow,$oh,$idata,$iw,$ih,$pad,$pad_value) {
 $optr = $optr|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $pad = $pad|0;
 $pad_value = $pad_value|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $iptr = 0, $pad2 = 0, $pdata = 0, $ph = 0, $pptr = 0, $psize = 0, $pw = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $1 = $optr;
 $2 = $ow;
 $3 = $oh;
 $4 = $idata;
 $5 = $iw;
 $6 = $ih;
 $7 = $pad;
 $8 = $pad_value;
 $9 = $7;
 $10 = $9 << 1;
 $pad2 = $10;
 $11 = $5;
 $12 = $pad2;
 $13 = (($11) + ($12))|0;
 $pw = $13;
 $14 = $6;
 $15 = $pad2;
 $16 = (($14) + ($15))|0;
 $ph = $16;
 $17 = $pw;
 $18 = $ph;
 $19 = Math_imul($17, $18)|0;
 $psize = $19;
 $20 = $psize;
 $21 = $20;
 $22 = (_malloc($21)|0);
 $pdata = $22;
 $23 = $pdata;
 $24 = ($23|0)==(0|0);
 if ($24) {
  $25 = HEAP32[_stderr>>2]|0;
  (_fprintf(($25|0),(3536|0),($vararg_buffer|0))|0);
  $0 = -160;
  $58 = $0;
  STACKTOP = sp;return ($58|0);
 }
 $26 = $pdata;
 $27 = $8;
 $28 = $27&255;
 $29 = $psize;
 _memset(($26|0),($28|0),($29|0))|0;
 $30 = $4;
 $iptr = $30;
 $31 = $pdata;
 $32 = $7;
 $33 = $pw;
 $34 = Math_imul($32, $33)|0;
 $35 = (($31) + ($34)|0);
 $36 = $7;
 $37 = (($35) + ($36)|0);
 $pptr = $37;
 $i = 0;
 while(1) {
  $38 = $i;
  $39 = $6;
  $40 = ($38|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = $pptr;
  $42 = $iptr;
  $43 = $5;
  _memcpy(($41|0),($42|0),($43|0))|0;
  $44 = $5;
  $45 = $iptr;
  $46 = (($45) + ($44)|0);
  $iptr = $46;
  $47 = $pw;
  $48 = $pptr;
  $49 = (($48) + ($47)|0);
  $pptr = $49;
  $50 = $i;
  $51 = (($50) + 1)|0;
  $i = $51;
 }
 $52 = $pdata;
 $53 = $1;
 HEAP32[$53>>2] = $52;
 $54 = $pw;
 $55 = $2;
 HEAP32[$55>>2] = $54;
 $56 = $ph;
 $57 = $3;
 HEAP32[$57>>2] = $56;
 $0 = 0;
 $58 = $0;
 STACKTOP = sp;return ($58|0);
}
function _fill_holes($bdata,$iw,$ih) {
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bptr = 0, $iw2 = 0;
 var $ix = 0, $iy = 0, $lptr = 0, $mptr = 0, $rptr = 0, $sptr = 0, $tptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $bdata;
 $1 = $iw;
 $2 = $ih;
 $3 = $0;
 $4 = (($3) + 1|0);
 $sptr = $4;
 $iy = 0;
 while(1) {
  $5 = $iy;
  $6 = $2;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $sptr;
  $9 = (($8) + -1|0);
  $lptr = $9;
  $10 = $sptr;
  $mptr = $10;
  $11 = $sptr;
  $12 = (($11) + 1|0);
  $rptr = $12;
  $ix = 1;
  while(1) {
   $13 = $ix;
   $14 = $1;
   $15 = (($14) - 1)|0;
   $16 = ($13|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $lptr;
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $mptr;
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = ($19|0)!=($22|0);
   if ($23) {
    $24 = $lptr;
    $25 = HEAP8[$24>>0]|0;
    $26 = $25&255;
    $27 = $rptr;
    $28 = HEAP8[$27>>0]|0;
    $29 = $28&255;
    $30 = ($26|0)==($29|0);
    if ($30) {
     $31 = $lptr;
     $32 = HEAP8[$31>>0]|0;
     $33 = $mptr;
     HEAP8[$33>>0] = $32;
     $34 = $lptr;
     $35 = (($34) + 2|0);
     $lptr = $35;
     $36 = $mptr;
     $37 = (($36) + 2|0);
     $mptr = $37;
     $38 = $rptr;
     $39 = (($38) + 2|0);
     $rptr = $39;
     $40 = $ix;
     $41 = (($40) + 1)|0;
     $ix = $41;
    } else {
     label = 8;
    }
   } else {
    label = 8;
   }
   if ((label|0) == 8) {
    label = 0;
    $42 = $lptr;
    $43 = (($42) + 1|0);
    $lptr = $43;
    $44 = $mptr;
    $45 = (($44) + 1|0);
    $mptr = $45;
    $46 = $rptr;
    $47 = (($46) + 1|0);
    $rptr = $47;
   }
   $48 = $ix;
   $49 = (($48) + 1)|0;
   $ix = $49;
  }
  $50 = $1;
  $51 = $sptr;
  $52 = (($51) + ($50)|0);
  $sptr = $52;
  $53 = $iy;
  $54 = (($53) + 1)|0;
  $iy = $54;
 }
 $55 = $1;
 $56 = $55 << 1;
 $iw2 = $56;
 $57 = $0;
 $58 = $1;
 $59 = (($57) + ($58)|0);
 $sptr = $59;
 $ix = 0;
 while(1) {
  $60 = $ix;
  $61 = $1;
  $62 = ($60|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = $sptr;
  $64 = $1;
  $65 = (0 - ($64))|0;
  $66 = (($63) + ($65)|0);
  $tptr = $66;
  $67 = $sptr;
  $mptr = $67;
  $68 = $sptr;
  $69 = $1;
  $70 = (($68) + ($69)|0);
  $bptr = $70;
  $iy = 1;
  while(1) {
   $71 = $iy;
   $72 = $2;
   $73 = (($72) - 1)|0;
   $74 = ($71|0)<($73|0);
   if (!($74)) {
    break;
   }
   $75 = $tptr;
   $76 = HEAP8[$75>>0]|0;
   $77 = $76&255;
   $78 = $mptr;
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = ($77|0)!=($80|0);
   if ($81) {
    $82 = $tptr;
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = $bptr;
    $86 = HEAP8[$85>>0]|0;
    $87 = $86&255;
    $88 = ($84|0)==($87|0);
    if ($88) {
     $89 = $tptr;
     $90 = HEAP8[$89>>0]|0;
     $91 = $mptr;
     HEAP8[$91>>0] = $90;
     $92 = $iw2;
     $93 = $tptr;
     $94 = (($93) + ($92)|0);
     $tptr = $94;
     $95 = $iw2;
     $96 = $mptr;
     $97 = (($96) + ($95)|0);
     $mptr = $97;
     $98 = $iw2;
     $99 = $bptr;
     $100 = (($99) + ($98)|0);
     $bptr = $100;
     $101 = $iy;
     $102 = (($101) + 1)|0;
     $iy = $102;
    } else {
     label = 20;
    }
   } else {
    label = 20;
   }
   if ((label|0) == 20) {
    label = 0;
    $103 = $1;
    $104 = $tptr;
    $105 = (($104) + ($103)|0);
    $tptr = $105;
    $106 = $1;
    $107 = $mptr;
    $108 = (($107) + ($106)|0);
    $mptr = $108;
    $109 = $1;
    $110 = $bptr;
    $111 = (($110) + ($109)|0);
    $bptr = $111;
   }
   $112 = $iy;
   $113 = (($112) + 1)|0;
   $iy = $113;
  }
  $114 = $sptr;
  $115 = (($114) + 1|0);
  $sptr = $115;
  $116 = $ix;
  $117 = (($116) + 1)|0;
  $ix = $117;
 }
 STACKTOP = sp;return;
}
function _free_path($x1,$y1,$x2,$y2,$bdata,$iw,$ih,$lfsparms) {
 $x1 = $x1|0;
 $y1 = $y1|0;
 $x2 = $x2|0;
 $y2 = $y2|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $nextval = 0, $num = 0, $preval = 0, $ret = 0, $trans = 0, $x_list = 0, $y_list = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $x_list = sp + 28|0;
 $y_list = sp + 24|0;
 $num = sp + 20|0;
 $1 = $x1;
 $2 = $y1;
 $3 = $x2;
 $4 = $y2;
 $5 = $bdata;
 $6 = $iw;
 $7 = $ih;
 $8 = $lfsparms;
 $9 = $1;
 $10 = $2;
 $11 = $3;
 $12 = $4;
 $13 = (_line_points($x_list,$y_list,$num,$9,$10,$11,$12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $ret;
  $0 = $15;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 $trans = 0;
 $16 = $5;
 $17 = $2;
 $18 = $6;
 $19 = Math_imul($17, $18)|0;
 $20 = (($16) + ($19)|0);
 $21 = $1;
 $22 = (($20) + ($21)|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $preval = $24;
 $i = 1;
 while(1) {
  $25 = $i;
  $26 = HEAP32[$num>>2]|0;
  $27 = ($25|0)<($26|0);
  if (!($27)) {
   label = 11;
   break;
  }
  $28 = $5;
  $29 = $i;
  $30 = HEAP32[$y_list>>2]|0;
  $31 = (($30) + ($29<<2)|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = $6;
  $34 = Math_imul($32, $33)|0;
  $35 = (($28) + ($34)|0);
  $36 = $i;
  $37 = HEAP32[$x_list>>2]|0;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($35) + ($39)|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41&255;
  $nextval = $42;
  $43 = $nextval;
  $44 = $preval;
  $45 = ($43|0)!=($44|0);
  if ($45) {
   $46 = $trans;
   $47 = (($46) + 1)|0;
   $trans = $47;
   $48 = $trans;
   $49 = $8;
   $50 = (($49) + 204|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($48|0)>($51|0);
   if ($52) {
    label = 7;
    break;
   }
   $55 = $nextval;
   $preval = $55;
  }
  $56 = $i;
  $57 = (($56) + 1)|0;
  $i = $57;
 }
 if ((label|0) == 7) {
  $53 = HEAP32[$x_list>>2]|0;
  _free($53);
  $54 = HEAP32[$y_list>>2]|0;
  _free($54);
  $0 = 0;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 else if ((label|0) == 11) {
  $58 = HEAP32[$x_list>>2]|0;
  _free($58);
  $59 = HEAP32[$y_list>>2]|0;
  _free($59);
  $0 = 1;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 return 0|0;
}
function _search_in_direction($ox,$oy,$oex,$oey,$pix,$strt_x,$strt_y,$delta_x,$delta_y,$maxsteps,$bdata,$iw,$ih) {
 $ox = $ox|0;
 $oy = $oy|0;
 $oex = $oex|0;
 $oey = $oey|0;
 $pix = $pix|0;
 $strt_x = $strt_x|0;
 $strt_y = $strt_y|0;
 $delta_x = +$delta_x;
 $delta_y = +$delta_y;
 $maxsteps = $maxsteps|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0.0, $37 = 0.0, $38 = 0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0.0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0.0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0.0, $fx = 0.0, $fy = 0.0, $i = 0, $px = 0, $py = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $x = sp + 44|0;
 $y = sp + 40|0;
 $px = sp + 36|0;
 $py = sp + 32|0;
 $1 = $ox;
 $2 = $oy;
 $3 = $oex;
 $4 = $oey;
 $5 = $pix;
 $6 = $strt_x;
 $7 = $strt_y;
 $8 = $delta_x;
 $9 = $delta_y;
 $10 = $maxsteps;
 $11 = $bdata;
 $12 = $iw;
 $13 = $ih;
 $14 = $6;
 HEAP32[$px>>2] = $14;
 $15 = $7;
 HEAP32[$py>>2] = $15;
 $16 = $6;
 $17 = (+($16|0));
 $fx = $17;
 $18 = $7;
 $19 = (+($18|0));
 $fy = $19;
 $i = 0;
 while(1) {
  $20 = $i;
  $21 = $10;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   label = 18;
   break;
  }
  $23 = $8;
  $24 = $fx;
  $25 = $24 + $23;
  $fx = $25;
  $26 = $9;
  $27 = $fy;
  $28 = $27 + $26;
  $fy = $28;
  $29 = $fx;
  $30 = $29 < 0.0;
  if ($30) {
   $31 = $fx;
   $32 = $31 - 0.5;
   $36 = $32;
  } else {
   $33 = $fx;
   $34 = $33 + 0.5;
   $36 = $34;
  }
  $35 = (~~(($36)));
  HEAP32[$x>>2] = $35;
  $37 = $fy;
  $38 = $37 < 0.0;
  if ($38) {
   $39 = $fy;
   $40 = $39 - 0.5;
   $44 = $40;
  } else {
   $41 = $fy;
   $42 = $41 + 0.5;
   $44 = $42;
  }
  $43 = (~~(($44)));
  HEAP32[$y>>2] = $43;
  $45 = HEAP32[$x>>2]|0;
  $46 = ($45|0)<(0);
  if ($46) {
   label = 13;
   break;
  }
  $47 = HEAP32[$x>>2]|0;
  $48 = $12;
  $49 = ($47|0)>=($48|0);
  if ($49) {
   label = 13;
   break;
  }
  $50 = HEAP32[$y>>2]|0;
  $51 = ($50|0)<(0);
  if ($51) {
   label = 13;
   break;
  }
  $52 = HEAP32[$y>>2]|0;
  $53 = $13;
  $54 = ($52|0)>=($53|0);
  if ($54) {
   label = 13;
   break;
  }
  $59 = $11;
  $60 = HEAP32[$y>>2]|0;
  $61 = $12;
  $62 = Math_imul($60, $61)|0;
  $63 = (($59) + ($62)|0);
  $64 = HEAP32[$x>>2]|0;
  $65 = (($63) + ($64)|0);
  $66 = HEAP8[$65>>0]|0;
  $67 = $66&255;
  $68 = $5;
  $69 = ($67|0)==($68|0);
  if ($69) {
   label = 15;
   break;
  }
  $81 = HEAP32[$x>>2]|0;
  HEAP32[$px>>2] = $81;
  $82 = HEAP32[$y>>2]|0;
  HEAP32[$py>>2] = $82;
  $83 = $i;
  $84 = (($83) + 1)|0;
  $i = $84;
 }
 if ((label|0) == 13) {
  $55 = $1;
  HEAP32[$55>>2] = -1;
  $56 = $2;
  HEAP32[$56>>2] = -1;
  $57 = $3;
  HEAP32[$57>>2] = -1;
  $58 = $4;
  HEAP32[$58>>2] = -1;
  $0 = 0;
  $89 = $0;
  STACKTOP = sp;return ($89|0);
 }
 else if ((label|0) == 15) {
  $70 = $11;
  $71 = $12;
  $72 = $13;
  _fix_edge_pixel_pair($x,$y,$px,$py,$70,$71,$72);
  $73 = HEAP32[$x>>2]|0;
  $74 = $1;
  HEAP32[$74>>2] = $73;
  $75 = HEAP32[$y>>2]|0;
  $76 = $2;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$px>>2]|0;
  $78 = $3;
  HEAP32[$78>>2] = $77;
  $79 = HEAP32[$py>>2]|0;
  $80 = $4;
  HEAP32[$80>>2] = $79;
  $0 = 1;
  $89 = $0;
  STACKTOP = sp;return ($89|0);
 }
 else if ((label|0) == 18) {
  $85 = $1;
  HEAP32[$85>>2] = -1;
  $86 = $2;
  HEAP32[$86>>2] = -1;
  $87 = $3;
  HEAP32[$87>>2] = -1;
  $88 = $4;
  HEAP32[$88>>2] = -1;
  $0 = 0;
  $89 = $0;
  STACKTOP = sp;return ($89|0);
 }
 return 0|0;
}
function _init_dir2rad($optr,$ndirs) {
 $optr = $optr|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0.0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0.0, $43 = 0.0, $44 = 0.0;
 var $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0.0;
 var $63 = 0.0, $64 = 0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $cs = 0.0, $dir2rad = 0, $i = 0, $pi_factor = 0.0, $sn = 0.0;
 var $theta = 0.0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 48|0;
 $vararg_buffer1 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $1 = $optr;
 $2 = $ndirs;
 $3 = (_malloc(12)|0);
 $dir2rad = $3;
 $4 = $dir2rad;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(3584|0),($vararg_buffer|0))|0);
  $0 = -10;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 $7 = $2;
 $8 = $dir2rad;
 HEAP32[$8>>2] = $7;
 $9 = $2;
 $10 = $9<<3;
 $11 = (_malloc($10)|0);
 $12 = $dir2rad;
 $13 = (($12) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = $dir2rad;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = $dir2rad;
  _free($18);
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(3632|0),($vararg_buffer1|0))|0);
  $0 = -11;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 $20 = $2;
 $21 = $20<<3;
 $22 = (_malloc($21)|0);
 $23 = $dir2rad;
 $24 = (($23) + 8|0);
 HEAP32[$24>>2] = $22;
 $25 = $dir2rad;
 $26 = (($25) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = $dir2rad;
  $30 = (($29) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  _free($31);
  $32 = $dir2rad;
  _free($32);
  $33 = HEAP32[_stderr>>2]|0;
  (_fprintf(($33|0),(3680|0),($vararg_buffer3|0))|0);
  $0 = -12;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 $34 = $2;
 $35 = (+($34|0));
 $36 = 6.283185307179586232 / $35;
 $pi_factor = $36;
 $i = 0;
 while(1) {
  $37 = $i;
  $38 = $2;
  $39 = ($37|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = $i;
  $41 = (+($40|0));
  $42 = $pi_factor;
  $43 = $41 * $42;
  $theta = $43;
  $44 = $theta;
  $45 = (+Math_cos((+$44)));
  $cs = $45;
  $46 = $theta;
  $47 = (+Math_sin((+$46)));
  $sn = $47;
  $48 = $cs;
  $49 = $48 < 0.0;
  if ($49) {
   $50 = $cs;
   $51 = $50 * 16384.0;
   $52 = $51 - 0.5;
   $53 = (~~(($52)));
   $54 = (+($53|0));
   $55 = $54 / 16384.0;
   $62 = $55;
  } else {
   $56 = $cs;
   $57 = $56 * 16384.0;
   $58 = $57 + 0.5;
   $59 = (~~(($58)));
   $60 = (+($59|0));
   $61 = $60 / 16384.0;
   $62 = $61;
  }
  $cs = $62;
  $63 = $sn;
  $64 = $63 < 0.0;
  if ($64) {
   $65 = $sn;
   $66 = $65 * 16384.0;
   $67 = $66 - 0.5;
   $68 = (~~(($67)));
   $69 = (+($68|0));
   $70 = $69 / 16384.0;
   $77 = $70;
  } else {
   $71 = $sn;
   $72 = $71 * 16384.0;
   $73 = $72 + 0.5;
   $74 = (~~(($73)));
   $75 = (+($74|0));
   $76 = $75 / 16384.0;
   $77 = $76;
  }
  $sn = $77;
  $78 = $cs;
  $79 = $i;
  $80 = $dir2rad;
  $81 = (($80) + 4|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + ($79<<3)|0);
  HEAPF64[$83>>3] = $78;
  $84 = $sn;
  $85 = $i;
  $86 = $dir2rad;
  $87 = (($86) + 8|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($88) + ($85<<3)|0);
  HEAPF64[$89>>3] = $84;
  $90 = $i;
  $91 = (($90) + 1)|0;
  $i = $91;
 }
 $92 = $dir2rad;
 $93 = $1;
 HEAP32[$93>>2] = $92;
 $0 = 0;
 $94 = $0;
 STACKTOP = sp;return ($94|0);
}
function _init_dftwaves($optr,$dft_coefs,$nwaves,$blocksize) {
 $optr = $optr|0;
 $dft_coefs = $dft_coefs|0;
 $nwaves = $nwaves|0;
 $blocksize = $blocksize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0.0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0, $214 = 0.0, $215 = 0, $216 = 0.0, $217 = 0.0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0, $221 = 0, $222 = 0.0, $223 = 0.0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0.0, $25 = 0.0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_j = 0, $_j1 = 0, $_j2 = 0, $cptr = 0, $dftwaves = 0, $freq = 0.0, $i = 0;
 var $j = 0, $pi_factor = 0.0, $sptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $x = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer7 = sp + 56|0;
 $vararg_buffer5 = sp + 48|0;
 $vararg_buffer3 = sp + 40|0;
 $vararg_buffer1 = sp + 32|0;
 $vararg_buffer = sp + 24|0;
 $1 = $optr;
 $2 = $dft_coefs;
 $3 = $nwaves;
 $4 = $blocksize;
 $5 = (_malloc(12)|0);
 $dftwaves = $5;
 $6 = $dftwaves;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[_stderr>>2]|0;
  (_fprintf(($8|0),(3728|0),($vararg_buffer|0))|0);
  $0 = -20;
  $232 = $0;
  STACKTOP = sp;return ($232|0);
 }
 $9 = $3;
 $10 = $dftwaves;
 HEAP32[$10>>2] = $9;
 $11 = $4;
 $12 = $dftwaves;
 $13 = (($12) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = $3;
 $15 = $14<<2;
 $16 = (_malloc($15)|0);
 $17 = $dftwaves;
 $18 = (($17) + 8|0);
 HEAP32[$18>>2] = $16;
 $19 = $dftwaves;
 $20 = ($19|0)==(0|0);
 if ($20) {
  $21 = $dftwaves;
  _free($21);
  $22 = HEAP32[_stderr>>2]|0;
  (_fprintf(($22|0),(3776|0),($vararg_buffer1|0))|0);
  $0 = -21;
  $232 = $0;
  STACKTOP = sp;return ($232|0);
 }
 $23 = $4;
 $24 = (+($23|0));
 $25 = 6.283185307179586232 / $24;
 $pi_factor = $25;
 $i = 0;
 while(1) {
  $26 = $i;
  $27 = $3;
  $28 = ($26|0)<($27|0);
  if (!($28)) {
   label = 31;
   break;
  }
  $29 = (_malloc(8)|0);
  $30 = $i;
  $31 = $dftwaves;
  $32 = (($31) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30<<2)|0);
  HEAP32[$34>>2] = $29;
  $35 = $i;
  $36 = $dftwaves;
  $37 = (($36) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + ($35<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)==(0|0);
  if ($41) {
   label = 8;
   break;
  }
  $73 = $4;
  $74 = $73<<3;
  $75 = (_malloc($74)|0);
  $76 = $i;
  $77 = $dftwaves;
  $78 = (($77) + 8|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + ($76<<2)|0);
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$81>>2] = $75;
  $82 = $i;
  $83 = $dftwaves;
  $84 = (($83) + 8|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (($85) + ($82<<2)|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = HEAP32[$87>>2]|0;
  $89 = ($88|0)==(0|0);
  if ($89) {
   label = 14;
   break;
  }
  $127 = $4;
  $128 = $127<<3;
  $129 = (_malloc($128)|0);
  $130 = $i;
  $131 = $dftwaves;
  $132 = (($131) + 8|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = (($133) + ($130<<2)|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (($135) + 4|0);
  HEAP32[$136>>2] = $129;
  $137 = $i;
  $138 = $dftwaves;
  $139 = (($138) + 8|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = (($140) + ($137<<2)|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = (($142) + 4|0);
  $144 = HEAP32[$143>>2]|0;
  $145 = ($144|0)==(0|0);
  if ($145) {
   label = 20;
   break;
  }
  $190 = $i;
  $191 = $dftwaves;
  $192 = (($191) + 8|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = (($193) + ($190<<2)|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = HEAP32[$195>>2]|0;
  $cptr = $196;
  $197 = $i;
  $198 = $dftwaves;
  $199 = (($198) + 8|0);
  $200 = HEAP32[$199>>2]|0;
  $201 = (($200) + ($197<<2)|0);
  $202 = HEAP32[$201>>2]|0;
  $203 = (($202) + 4|0);
  $204 = HEAP32[$203>>2]|0;
  $sptr = $204;
  $205 = $pi_factor;
  $206 = $i;
  $207 = $2;
  $208 = (($207) + ($206<<3)|0);
  $209 = +HEAPF64[$208>>3];
  $210 = $205 * $209;
  $freq = $210;
  $j = 0;
  while(1) {
   $211 = $j;
   $212 = $4;
   $213 = ($211|0)<($212|0);
   if (!($213)) {
    break;
   }
   $214 = $freq;
   $215 = $j;
   $216 = (+($215|0));
   $217 = $214 * $216;
   $x = $217;
   $218 = $x;
   $219 = (+Math_cos((+$218)));
   $220 = $cptr;
   $221 = (($220) + 8|0);
   $cptr = $221;
   HEAPF64[$220>>3] = $219;
   $222 = $x;
   $223 = (+Math_sin((+$222)));
   $224 = $sptr;
   $225 = (($224) + 8|0);
   $sptr = $225;
   HEAPF64[$224>>3] = $223;
   $226 = $j;
   $227 = (($226) + 1)|0;
   $j = $227;
  }
  $228 = $i;
  $229 = (($228) + 1)|0;
  $i = $229;
 }
 if ((label|0) == 8) {
  $_j = 0;
  while(1) {
   $42 = $_j;
   $43 = $i;
   $44 = ($42|0)<($43|0);
   if (!($44)) {
    break;
   }
   $45 = $_j;
   $46 = $dftwaves;
   $47 = (($46) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + ($45<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$50>>2]|0;
   _free($51);
   $52 = $_j;
   $53 = $dftwaves;
   $54 = (($53) + 8|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + ($52<<2)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   _free($59);
   $60 = $_j;
   $61 = $dftwaves;
   $62 = (($61) + 8|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (($63) + ($60<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   _free($65);
   $66 = $_j;
   $67 = (($66) + 1)|0;
   $_j = $67;
  }
  $68 = $dftwaves;
  $69 = (($68) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  _free($70);
  $71 = $dftwaves;
  _free($71);
  $72 = HEAP32[_stderr>>2]|0;
  (_fprintf(($72|0),(3832|0),($vararg_buffer3|0))|0);
  $0 = -22;
  $232 = $0;
  STACKTOP = sp;return ($232|0);
 }
 else if ((label|0) == 14) {
  $_j1 = 0;
  while(1) {
   $90 = $_j1;
   $91 = $i;
   $92 = ($90|0)<($91|0);
   if (!($92)) {
    break;
   }
   $93 = $_j1;
   $94 = $dftwaves;
   $95 = (($94) + 8|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = (($96) + ($93<<2)|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$98>>2]|0;
   _free($99);
   $100 = $_j1;
   $101 = $dftwaves;
   $102 = (($101) + 8|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (($103) + ($100<<2)|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = (($105) + 4|0);
   $107 = HEAP32[$106>>2]|0;
   _free($107);
   $108 = $_j1;
   $109 = $dftwaves;
   $110 = (($109) + 8|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = (($111) + ($108<<2)|0);
   $113 = HEAP32[$112>>2]|0;
   _free($113);
   $114 = $_j1;
   $115 = (($114) + 1)|0;
   $_j1 = $115;
  }
  $116 = $i;
  $117 = $dftwaves;
  $118 = (($117) + 8|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = (($119) + ($116<<2)|0);
  $121 = HEAP32[$120>>2]|0;
  _free($121);
  $122 = $dftwaves;
  $123 = (($122) + 8|0);
  $124 = HEAP32[$123>>2]|0;
  _free($124);
  $125 = $dftwaves;
  _free($125);
  $126 = HEAP32[_stderr>>2]|0;
  (_fprintf(($126|0),(3888|0),($vararg_buffer5|0))|0);
  $0 = -23;
  $232 = $0;
  STACKTOP = sp;return ($232|0);
 }
 else if ((label|0) == 20) {
  $_j2 = 0;
  while(1) {
   $146 = $_j2;
   $147 = $i;
   $148 = ($146|0)<($147|0);
   if (!($148)) {
    break;
   }
   $149 = $_j2;
   $150 = $dftwaves;
   $151 = (($150) + 8|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = (($152) + ($149<<2)|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = HEAP32[$154>>2]|0;
   _free($155);
   $156 = $_j2;
   $157 = $dftwaves;
   $158 = (($157) + 8|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = (($159) + ($156<<2)|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = (($161) + 4|0);
   $163 = HEAP32[$162>>2]|0;
   _free($163);
   $164 = $_j2;
   $165 = $dftwaves;
   $166 = (($165) + 8|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = (($167) + ($164<<2)|0);
   $169 = HEAP32[$168>>2]|0;
   _free($169);
   $170 = $_j2;
   $171 = (($170) + 1)|0;
   $_j2 = $171;
  }
  $172 = $i;
  $173 = $dftwaves;
  $174 = (($173) + 8|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = (($175) + ($172<<2)|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = HEAP32[$177>>2]|0;
  _free($178);
  $179 = $i;
  $180 = $dftwaves;
  $181 = (($180) + 8|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = (($182) + ($179<<2)|0);
  $184 = HEAP32[$183>>2]|0;
  _free($184);
  $185 = $dftwaves;
  $186 = (($185) + 8|0);
  $187 = HEAP32[$186>>2]|0;
  _free($187);
  $188 = $dftwaves;
  _free($188);
  $189 = HEAP32[_stderr>>2]|0;
  (_fprintf(($189|0),(3952|0),($vararg_buffer7|0))|0);
  $0 = -24;
  $232 = $0;
  STACKTOP = sp;return ($232|0);
 }
 else if ((label|0) == 31) {
  $230 = $dftwaves;
  $231 = $1;
  HEAP32[$231>>2] = $230;
  $0 = 0;
  $232 = $0;
  STACKTOP = sp;return ($232|0);
 }
 return 0|0;
}
function _get_max_padding_V2($map_windowsize,$map_windowoffset,$dirbin_grid_w,$dirbin_grid_h) {
 $map_windowsize = $map_windowsize|0;
 $map_windowoffset = $map_windowoffset|0;
 $dirbin_grid_w = $dirbin_grid_w|0;
 $dirbin_grid_h = $dirbin_grid_h|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0.0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0.0, $60 = 0.0, $61 = 0.0, $62 = 0.0;
 var $63 = 0.0, $64 = 0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0.0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0.0, $dft_pad = 0, $diag = 0.0, $dirbin_pad = 0, $max_pad = 0, $pad = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $map_windowsize;
 $1 = $map_windowoffset;
 $2 = $dirbin_grid_w;
 $3 = $dirbin_grid_h;
 $4 = $0;
 $5 = (+($4|0));
 $6 = 2.0 * $5;
 $7 = $0;
 $8 = (+($7|0));
 $9 = $6 * $8;
 $10 = (+Math_sqrt((+$9)));
 $diag = $10;
 $11 = $diag;
 $12 = $0;
 $13 = (+($12|0));
 $14 = $11 - $13;
 $15 = $14 / 2.0;
 $pad = $15;
 $16 = $pad;
 $17 = $16 < 0.0;
 if ($17) {
  $18 = $pad;
  $19 = $18 * 16384.0;
  $20 = $19 - 0.5;
  $21 = (~~(($20)));
  $22 = (+($21|0));
  $23 = $22 / 16384.0;
  $30 = $23;
 } else {
  $24 = $pad;
  $25 = $24 * 16384.0;
  $26 = $25 + 0.5;
  $27 = (~~(($26)));
  $28 = (+($27|0));
  $29 = $28 / 16384.0;
  $30 = $29;
 }
 $pad = $30;
 $31 = $pad;
 $32 = $31 < 0.0;
 if ($32) {
  $33 = $pad;
  $34 = $33 - 0.5;
  $38 = $34;
 } else {
  $35 = $pad;
  $36 = $35 + 0.5;
  $38 = $36;
 }
 $37 = (~~(($38)));
 $39 = $1;
 $40 = (($37) + ($39))|0;
 $dft_pad = $40;
 $41 = $2;
 $42 = $2;
 $43 = Math_imul($41, $42)|0;
 $44 = $3;
 $45 = $3;
 $46 = Math_imul($44, $45)|0;
 $47 = (($43) + ($46))|0;
 $48 = (+($47|0));
 $49 = (+Math_sqrt((+$48)));
 $diag = $49;
 $50 = $diag;
 $51 = $50 - 1.0;
 $52 = $51 / 2.0;
 $pad = $52;
 $53 = $pad;
 $54 = $53 < 0.0;
 if ($54) {
  $55 = $pad;
  $56 = $55 * 16384.0;
  $57 = $56 - 0.5;
  $58 = (~~(($57)));
  $59 = (+($58|0));
  $60 = $59 / 16384.0;
  $67 = $60;
 } else {
  $61 = $pad;
  $62 = $61 * 16384.0;
  $63 = $62 + 0.5;
  $64 = (~~(($63)));
  $65 = (+($64|0));
  $66 = $65 / 16384.0;
  $67 = $66;
 }
 $pad = $67;
 $68 = $pad;
 $69 = $68 < 0.0;
 if ($69) {
  $70 = $pad;
  $71 = $70 - 0.5;
  $75 = $71;
 } else {
  $72 = $pad;
  $73 = $72 + 0.5;
  $75 = $73;
 }
 $74 = (~~(($75)));
 $dirbin_pad = $74;
 $76 = $dft_pad;
 $77 = $dirbin_pad;
 $78 = ($76|0)>($77|0);
 if ($78) {
  $79 = $dft_pad;
  $81 = $79;
  $max_pad = $81;
  $82 = $max_pad;
  STACKTOP = sp;return ($82|0);
 } else {
  $80 = $dirbin_pad;
  $81 = $80;
  $max_pad = $81;
  $82 = $max_pad;
  STACKTOP = sp;return ($82|0);
 }
 return 0|0;
}
function _init_rotgrids($optr,$iw,$ih,$ipad,$start_dir_angle,$ndirs,$grid_w,$grid_h,$relative2) {
 $optr = $optr|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $ipad = $ipad|0;
 $start_dir_angle = +$start_dir_angle;
 $ndirs = $ndirs|0;
 $grid_w = $grid_w|0;
 $grid_h = $grid_h|0;
 $relative2 = $relative2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0.0, $143 = 0.0, $144 = 0.0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0.0, $196 = 0, $197 = 0.0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0, $203 = 0, $204 = 0.0, $205 = 0.0;
 var $206 = 0.0, $207 = 0.0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0, $215 = 0.0, $216 = 0.0, $217 = 0.0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0, $223 = 0.0;
 var $224 = 0.0, $225 = 0.0, $226 = 0.0, $227 = 0.0, $228 = 0.0, $229 = 0.0, $23 = 0.0, $230 = 0, $231 = 0.0, $232 = 0.0, $233 = 0.0, $234 = 0, $235 = 0.0, $236 = 0.0, $237 = 0.0, $238 = 0.0, $239 = 0.0, $24 = 0, $240 = 0, $241 = 0.0;
 var $242 = 0.0, $243 = 0.0, $244 = 0.0, $245 = 0, $246 = 0.0, $247 = 0.0, $248 = 0.0, $249 = 0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0.0, $253 = 0.0, $254 = 0.0, $255 = 0, $256 = 0.0, $257 = 0.0, $258 = 0.0, $259 = 0.0, $26 = 0;
 var $260 = 0, $261 = 0.0, $262 = 0.0, $263 = 0.0, $264 = 0.0, $265 = 0, $266 = 0.0, $267 = 0.0, $268 = 0, $269 = 0.0, $27 = 0, $270 = 0.0, $271 = 0.0, $272 = 0.0, $273 = 0, $274 = 0.0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0.0, $289 = 0.0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0.0, $37 = 0.0, $38 = 0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0.0;
 var $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0.0, $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0, $73 = 0.0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0.0;
 var $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0, $93 = 0.0, $94 = 0.0, $95 = 0.0, $96 = 0.0, $97 = 0, $98 = 0.0, $99 = 0, $_j = 0, $cs = 0.0, $cx = 0.0, $cy = 0.0, $diag = 0.0, $dir = 0;
 var $fx = 0.0, $fxm = 0.0, $fy = 0.0, $fym = 0.0, $grid = 0, $grid_pad = 0, $grid_size = 0, $ix = 0, $ixt = 0, $iy = 0, $iyt = 0, $min_dim = 0, $pad = 0.0, $pi_incr = 0.0, $pi_offset = 0.0, $pw = 0, $rotgrids = 0, $sn = 0.0, $theta = 0.0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer7 = sp + 144|0;
 $vararg_buffer5 = sp + 136|0;
 $vararg_buffer3 = sp + 128|0;
 $vararg_buffer1 = sp + 120|0;
 $vararg_buffer = sp + 112|0;
 $1 = $optr;
 $2 = $iw;
 $3 = $ih;
 $4 = $ipad;
 $5 = $start_dir_angle;
 $6 = $ndirs;
 $7 = $grid_w;
 $8 = $grid_h;
 $9 = $relative2;
 $10 = (_malloc(32)|0);
 $rotgrids = $10;
 $11 = $rotgrids;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  (_fprintf(($13|0),(4016|0),($vararg_buffer|0))|0);
  $0 = -30;
  $293 = $0;
  STACKTOP = sp;return ($293|0);
 }
 $14 = $6;
 $15 = $rotgrids;
 $16 = (($15) + 16|0);
 HEAP32[$16>>2] = $14;
 $17 = $7;
 $18 = $rotgrids;
 $19 = (($18) + 20|0);
 HEAP32[$19>>2] = $17;
 $20 = $8;
 $21 = $rotgrids;
 $22 = (($21) + 24|0);
 HEAP32[$22>>2] = $20;
 $23 = $5;
 $24 = $rotgrids;
 $25 = (($24) + 8|0);
 HEAPF64[$25>>3] = $23;
 $26 = $9;
 $27 = $rotgrids;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = $7;
 $30 = $7;
 $31 = Math_imul($29, $30)|0;
 $32 = $8;
 $33 = $8;
 $34 = Math_imul($32, $33)|0;
 $35 = (($31) + ($34))|0;
 $36 = (+($35|0));
 $37 = (+Math_sqrt((+$36)));
 $diag = $37;
 $38 = $9;
 if ((($38|0) == 0)) {
  $39 = $diag;
  $40 = $39 - 1.0;
  $41 = $40 / 2.0;
  $pad = $41;
  $42 = $pad;
  $43 = $42 < 0.0;
  if ($43) {
   $44 = $pad;
   $45 = $44 * 16384.0;
   $46 = $45 - 0.5;
   $47 = (~~(($46)));
   $48 = (+($47|0));
   $49 = $48 / 16384.0;
   $56 = $49;
  } else {
   $50 = $pad;
   $51 = $50 * 16384.0;
   $52 = $51 + 0.5;
   $53 = (~~(($52)));
   $54 = (+($53|0));
   $55 = $54 / 16384.0;
   $56 = $55;
  }
  $pad = $56;
  $57 = $pad;
  $58 = $57 < 0.0;
  if ($58) {
   $59 = $pad;
   $60 = $59 - 0.5;
   $64 = $60;
  } else {
   $61 = $pad;
   $62 = $61 + 0.5;
   $64 = $62;
  }
  $63 = (~~(($64)));
  $grid_pad = $63;
 } else if ((($38|0) == 1)) {
  $65 = $7;
  $66 = $8;
  $67 = ($65|0)<($66|0);
  if ($67) {
   $68 = $7;
   $70 = $68;
  } else {
   $69 = $8;
   $70 = $69;
  }
  $min_dim = $70;
  $71 = $diag;
  $72 = $min_dim;
  $73 = (+($72|0));
  $74 = $71 - $73;
  $75 = $74 / 2.0;
  $pad = $75;
  $76 = $pad;
  $77 = $76 < 0.0;
  if ($77) {
   $78 = $pad;
   $79 = $78 * 16384.0;
   $80 = $79 - 0.5;
   $81 = (~~(($80)));
   $82 = (+($81|0));
   $83 = $82 / 16384.0;
   $90 = $83;
  } else {
   $84 = $pad;
   $85 = $84 * 16384.0;
   $86 = $85 + 0.5;
   $87 = (~~(($86)));
   $88 = (+($87|0));
   $89 = $88 / 16384.0;
   $90 = $89;
  }
  $pad = $90;
  $91 = $pad;
  $92 = $91 < 0.0;
  if ($92) {
   $93 = $pad;
   $94 = $93 - 0.5;
   $98 = $94;
  } else {
   $95 = $pad;
   $96 = $95 + 0.5;
   $98 = $96;
  }
  $97 = (~~(($98)));
  $grid_pad = $97;
 } else {
  $99 = HEAP32[_stderr>>2]|0;
  $100 = $9;
  HEAP32[$vararg_buffer1>>2] = $100;
  (_fprintf(($99|0),(4064|0),($vararg_buffer1|0))|0);
  $101 = $rotgrids;
  _free($101);
  $0 = -31;
  $293 = $0;
  STACKTOP = sp;return ($293|0);
 }
 $102 = $4;
 $103 = ($102|0)==(-1);
 do {
  if ($103) {
   $104 = $grid_pad;
   $105 = $rotgrids;
   HEAP32[$105>>2] = $104;
  } else {
   $106 = $4;
   $107 = $grid_pad;
   $108 = ($106|0)<($107|0);
   if (!($108)) {
    $111 = $4;
    $112 = $rotgrids;
    HEAP32[$112>>2] = $111;
    break;
   }
   $109 = HEAP32[_stderr>>2]|0;
   (_fprintf(($109|0),(4120|0),($vararg_buffer3|0))|0);
   $110 = $rotgrids;
   _free($110);
   $0 = -32;
   $293 = $0;
   STACKTOP = sp;return ($293|0);
  }
 } while(0);
 $113 = $7;
 $114 = $8;
 $115 = Math_imul($113, $114)|0;
 $grid_size = $115;
 $116 = $2;
 $117 = $rotgrids;
 $118 = HEAP32[$117>>2]|0;
 $119 = $118 << 1;
 $120 = (($116) + ($119))|0;
 $pw = $120;
 $121 = $7;
 $122 = (($121) - 1)|0;
 $123 = (+($122|0));
 $124 = $123 / 2.0;
 $cx = $124;
 $125 = $8;
 $126 = (($125) - 1)|0;
 $127 = (+($126|0));
 $128 = $127 / 2.0;
 $cy = $128;
 $129 = $6;
 $130 = $129<<2;
 $131 = (_malloc($130)|0);
 $132 = $rotgrids;
 $133 = (($132) + 28|0);
 HEAP32[$133>>2] = $131;
 $134 = $rotgrids;
 $135 = (($134) + 28|0);
 $136 = HEAP32[$135>>2]|0;
 $137 = ($136|0)==(0|0);
 if ($137) {
  $138 = $rotgrids;
  _free($138);
  $139 = HEAP32[_stderr>>2]|0;
  (_fprintf(($139|0),(4176|0),($vararg_buffer5|0))|0);
  $0 = -33;
  $293 = $0;
  STACKTOP = sp;return ($293|0);
 }
 $140 = $5;
 $pi_offset = $140;
 $141 = $6;
 $142 = (+($141|0));
 $143 = 3.141592653589793116 / $142;
 $pi_incr = $143;
 $dir = 0;
 $144 = $pi_offset;
 $theta = $144;
 while(1) {
  $145 = $dir;
  $146 = $6;
  $147 = ($145|0)<($146|0);
  if (!($147)) {
   label = 61;
   break;
  }
  $148 = $grid_size;
  $149 = $148<<2;
  $150 = (_malloc($149)|0);
  $151 = $dir;
  $152 = $rotgrids;
  $153 = (($152) + 28|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = (($154) + ($151<<2)|0);
  HEAP32[$155>>2] = $150;
  $156 = $dir;
  $157 = $rotgrids;
  $158 = (($157) + 28|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = (($159) + ($156<<2)|0);
  $161 = HEAP32[$160>>2]|0;
  $162 = ($161|0)==(0|0);
  if ($162) {
   break;
  }
  $176 = $dir;
  $177 = $rotgrids;
  $178 = (($177) + 28|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = (($179) + ($176<<2)|0);
  $181 = HEAP32[$180>>2]|0;
  $grid = $181;
  $182 = $theta;
  $183 = (+Math_cos((+$182)));
  $cs = $183;
  $184 = $theta;
  $185 = (+Math_sin((+$184)));
  $sn = $185;
  $iy = 0;
  while(1) {
   $186 = $iy;
   $187 = $8;
   $188 = ($186|0)<($187|0);
   if (!($188)) {
    break;
   }
   $189 = $iy;
   $190 = (+($189|0));
   $191 = $cy;
   $192 = $190 - $191;
   $193 = $sn;
   $194 = $192 * $193;
   $195 = -1.0 * $194;
   $fxm = $195;
   $196 = $iy;
   $197 = (+($196|0));
   $198 = $cy;
   $199 = $197 - $198;
   $200 = $cs;
   $201 = $199 * $200;
   $fym = $201;
   $202 = $9;
   $203 = ($202|0)==(1);
   if ($203) {
    $204 = $cx;
    $205 = $fxm;
    $206 = $205 + $204;
    $fxm = $206;
    $207 = $cy;
    $208 = $fym;
    $209 = $208 + $207;
    $fym = $209;
   }
   $ix = 0;
   while(1) {
    $210 = $ix;
    $211 = $7;
    $212 = ($210|0)<($211|0);
    if (!($212)) {
     break;
    }
    $213 = $fxm;
    $214 = $ix;
    $215 = (+($214|0));
    $216 = $cx;
    $217 = $215 - $216;
    $218 = $cs;
    $219 = $217 * $218;
    $220 = $213 + $219;
    $fx = $220;
    $221 = $fym;
    $222 = $ix;
    $223 = (+($222|0));
    $224 = $cx;
    $225 = $223 - $224;
    $226 = $sn;
    $227 = $225 * $226;
    $228 = $221 + $227;
    $fy = $228;
    $229 = $fx;
    $230 = $229 < 0.0;
    if ($230) {
     $231 = $fx;
     $232 = $231 * 16384.0;
     $233 = $232 - 0.5;
     $234 = (~~(($233)));
     $235 = (+($234|0));
     $236 = $235 / 16384.0;
     $243 = $236;
    } else {
     $237 = $fx;
     $238 = $237 * 16384.0;
     $239 = $238 + 0.5;
     $240 = (~~(($239)));
     $241 = (+($240|0));
     $242 = $241 / 16384.0;
     $243 = $242;
    }
    $fx = $243;
    $244 = $fy;
    $245 = $244 < 0.0;
    if ($245) {
     $246 = $fy;
     $247 = $246 * 16384.0;
     $248 = $247 - 0.5;
     $249 = (~~(($248)));
     $250 = (+($249|0));
     $251 = $250 / 16384.0;
     $258 = $251;
    } else {
     $252 = $fy;
     $253 = $252 * 16384.0;
     $254 = $253 + 0.5;
     $255 = (~~(($254)));
     $256 = (+($255|0));
     $257 = $256 / 16384.0;
     $258 = $257;
    }
    $fy = $258;
    $259 = $fx;
    $260 = $259 < 0.0;
    if ($260) {
     $261 = $fx;
     $262 = $261 - 0.5;
     $266 = $262;
    } else {
     $263 = $fx;
     $264 = $263 + 0.5;
     $266 = $264;
    }
    $265 = (~~(($266)));
    $ixt = $265;
    $267 = $fy;
    $268 = $267 < 0.0;
    if ($268) {
     $269 = $fy;
     $270 = $269 - 0.5;
     $274 = $270;
    } else {
     $271 = $fy;
     $272 = $271 + 0.5;
     $274 = $272;
    }
    $273 = (~~(($274)));
    $iyt = $273;
    $275 = $ixt;
    $276 = $iyt;
    $277 = $pw;
    $278 = Math_imul($276, $277)|0;
    $279 = (($275) + ($278))|0;
    $280 = $grid;
    $281 = (($280) + 4|0);
    $grid = $281;
    HEAP32[$280>>2] = $279;
    $282 = $ix;
    $283 = (($282) + 1)|0;
    $ix = $283;
   }
   $284 = $iy;
   $285 = (($284) + 1)|0;
   $iy = $285;
  }
  $286 = $dir;
  $287 = (($286) + 1)|0;
  $dir = $287;
  $288 = $pi_incr;
  $289 = $theta;
  $290 = $289 + $288;
  $theta = $290;
 }
 if ((label|0) == 61) {
  $291 = $rotgrids;
  $292 = $1;
  HEAP32[$292>>2] = $291;
  $0 = 0;
  $293 = $0;
  STACKTOP = sp;return ($293|0);
 }
 $_j = 0;
 while(1) {
  $163 = $_j;
  $164 = $dir;
  $165 = ($163|0)<($164|0);
  if (!($165)) {
   break;
  }
  $166 = $_j;
  $167 = $rotgrids;
  $168 = (($167) + 28|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = (($169) + ($166<<2)|0);
  $171 = HEAP32[$170>>2]|0;
  _free($171);
  $172 = $_j;
  $173 = (($172) + 1)|0;
  $_j = $173;
 }
 $174 = $rotgrids;
 _free($174);
 $175 = HEAP32[_stderr>>2]|0;
 (_fprintf(($175|0),(4232|0),($vararg_buffer7|0))|0);
 $0 = -34;
 $293 = $0;
 STACKTOP = sp;return ($293|0);
}
function _alloc_dir_powers($opowers,$nwaves,$ndirs) {
 $opowers = $opowers|0;
 $nwaves = $nwaves|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $_j = 0, $powers = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $opowers;
 $2 = $nwaves;
 $3 = $ndirs;
 $4 = $2;
 $5 = $4<<2;
 $6 = (_malloc($5)|0);
 $powers = $6;
 $7 = $powers;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(4288|0),($vararg_buffer|0))|0);
  $0 = -40;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 $w = 0;
 while(1) {
  $10 = $w;
  $11 = $2;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   label = 13;
   break;
  }
  $13 = $3;
  $14 = $13<<3;
  $15 = (_malloc($14)|0);
  $16 = $w;
  $17 = $powers;
  $18 = (($17) + ($16<<2)|0);
  HEAP32[$18>>2] = $15;
  $19 = $w;
  $20 = $powers;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)==(0|0);
  if ($23) {
   break;
  }
  $35 = $w;
  $36 = (($35) + 1)|0;
  $w = $36;
 }
 if ((label|0) == 13) {
  $37 = $powers;
  $38 = $1;
  HEAP32[$38>>2] = $37;
  $0 = 0;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 $_j = 0;
 while(1) {
  $24 = $_j;
  $25 = $w;
  $26 = ($24|0)<($25|0);
  if (!($26)) {
   break;
  }
  $27 = $_j;
  $28 = $powers;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  _free($30);
  $31 = $_j;
  $32 = (($31) + 1)|0;
  $_j = $32;
 }
 $33 = $powers;
 _free($33);
 $34 = HEAP32[_stderr>>2]|0;
 (_fprintf(($34|0),(4336|0),($vararg_buffer1|0))|0);
 $0 = -41;
 $39 = $0;
 STACKTOP = sp;return ($39|0);
}
function _alloc_power_stats($owis,$opowmaxs,$opowmax_dirs,$opownorms,$nstats) {
 $owis = $owis|0;
 $opowmaxs = $opowmaxs|0;
 $opowmax_dirs = $opowmax_dirs|0;
 $opownorms = $opownorms|0;
 $nstats = $nstats|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $powmax_dirs = 0, $powmaxs = 0, $pownorms = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $wis = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $owis;
 $2 = $opowmaxs;
 $3 = $opowmax_dirs;
 $4 = $opownorms;
 $5 = $nstats;
 $6 = $5;
 $7 = $6<<2;
 $8 = (_malloc($7)|0);
 $wis = $8;
 $9 = $wis;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(4384|0),($vararg_buffer|0))|0);
  $0 = -50;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $12 = $5;
 $13 = $12<<3;
 $14 = (_malloc($13)|0);
 $powmaxs = $14;
 $15 = $powmaxs;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = $wis;
  _free($17);
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(4432|0),($vararg_buffer1|0))|0);
  $0 = -51;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $19 = $5;
 $20 = $19<<2;
 $21 = (_malloc($20)|0);
 $powmax_dirs = $21;
 $22 = $powmax_dirs;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = $wis;
  _free($24);
  $25 = $powmaxs;
  _free($25);
  $26 = HEAP32[_stderr>>2]|0;
  (_fprintf(($26|0),(4480|0),($vararg_buffer3|0))|0);
  $0 = -52;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $27 = $5;
 $28 = $27<<3;
 $29 = (_malloc($28)|0);
 $pownorms = $29;
 $30 = $pownorms;
 $31 = ($30|0)==(0|0);
 if ($31) {
  $32 = $wis;
  _free($32);
  $33 = $powmaxs;
  _free($33);
  $34 = $pownorms;
  _free($34);
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(4536|0),($vararg_buffer5|0))|0);
  $0 = -53;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 } else {
  $36 = $wis;
  $37 = $1;
  HEAP32[$37>>2] = $36;
  $38 = $powmaxs;
  $39 = $2;
  HEAP32[$39>>2] = $38;
  $40 = $powmax_dirs;
  $41 = $3;
  HEAP32[$41>>2] = $40;
  $42 = $pownorms;
  $43 = $4;
  HEAP32[$43>>2] = $42;
  $0 = 0;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 return 0|0;
}
function _line_points($ox_list,$oy_list,$onum,$x1,$y1,$x2,$y2) {
 $ox_list = $ox_list|0;
 $oy_list = $oy_list|0;
 $onum = $onum|0;
 $x1 = $x1|0;
 $y1 = $y1|0;
 $x2 = $x2|0;
 $y2 = $y2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0.0, $143 = 0, $144 = 0.0, $145 = 0.0, $146 = 0.0, $147 = 0.0, $148 = 0, $149 = 0.0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0, $153 = 0.0, $154 = 0.0, $155 = 0.0, $156 = 0.0, $157 = 0.0, $158 = 0, $159 = 0.0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0.0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0.0, $73 = 0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0, $78 = 0.0, $79 = 0, $8 = 0;
 var $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0.0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0.0;
 var $99 = 0, $adx = 0, $ady = 0, $asize = 0, $dx = 0, $dy = 0, $i = 0, $intx = 0, $inty = 0, $inx = 0, $iny = 0, $ix = 0, $iy = 0, $rx = 0.0, $ry = 0.0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $x_factor = 0.0, $x_incr = 0;
 var $x_list = 0, $y_factor = 0.0, $y_incr = 0, $y_list = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer3 = sp + 48|0;
 $vararg_buffer1 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $1 = $ox_list;
 $2 = $oy_list;
 $3 = $onum;
 $4 = $x1;
 $5 = $y1;
 $6 = $x2;
 $7 = $y2;
 $8 = $6;
 $9 = $4;
 $10 = (($8) - ($9))|0;
 $11 = (Math_abs(($10|0))|0);
 $12 = (($11) + 2)|0;
 $13 = $7;
 $14 = $5;
 $15 = (($13) - ($14))|0;
 $16 = (Math_abs(($15|0))|0);
 $17 = (($16) + 2)|0;
 $18 = ($12|0)>($17|0);
 if ($18) {
  $19 = $6;
  $20 = $4;
  $21 = (($19) - ($20))|0;
  $22 = (Math_abs(($21|0))|0);
  $23 = (($22) + 2)|0;
  $29 = $23;
 } else {
  $24 = $7;
  $25 = $5;
  $26 = (($24) - ($25))|0;
  $27 = (Math_abs(($26|0))|0);
  $28 = (($27) + 2)|0;
  $29 = $28;
 }
 $asize = $29;
 $30 = $asize;
 $31 = $30<<2;
 $32 = (_malloc($31)|0);
 $x_list = $32;
 $33 = $x_list;
 $34 = ($33|0)==(0|0);
 if ($34) {
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(4584|0),($vararg_buffer|0))|0);
  $0 = -410;
  $199 = $0;
  STACKTOP = sp;return ($199|0);
 }
 $36 = $asize;
 $37 = $36<<2;
 $38 = (_malloc($37)|0);
 $y_list = $38;
 $39 = $y_list;
 $40 = ($39|0)==(0|0);
 if ($40) {
  $41 = $x_list;
  _free($41);
  $42 = HEAP32[_stderr>>2]|0;
  (_fprintf(($42|0),(4624|0),($vararg_buffer1|0))|0);
  $0 = -411;
  $199 = $0;
  STACKTOP = sp;return ($199|0);
 }
 $43 = $6;
 $44 = $4;
 $45 = (($43) - ($44))|0;
 $dx = $45;
 $46 = $7;
 $47 = $5;
 $48 = (($46) - ($47))|0;
 $dy = $48;
 $49 = $dx;
 $50 = ($49|0)>=(0);
 if ($50) {
  $x_incr = 1;
 } else {
  $x_incr = -1;
 }
 $51 = $dy;
 $52 = ($51|0)>=(0);
 if ($52) {
  $y_incr = 1;
 } else {
  $y_incr = -1;
 }
 $53 = $dx;
 $54 = (Math_abs(($53|0))|0);
 $adx = $54;
 $55 = $dy;
 $56 = (Math_abs(($55|0))|0);
 $ady = $56;
 $57 = $adx;
 $58 = $ady;
 $59 = ($57|0)>($58|0);
 if ($59) {
  $inx = 1;
 } else {
  $inx = 0;
 }
 $60 = $ady;
 $61 = $adx;
 $62 = ($60|0)>($61|0);
 if ($62) {
  $iny = 1;
 } else {
  $iny = 0;
 }
 $63 = $iny;
 $64 = (1 - ($63))|0;
 $intx = $64;
 $65 = $inx;
 $66 = (1 - ($65))|0;
 $inty = $66;
 $67 = $inx;
 $68 = $x_incr;
 $69 = Math_imul($67, $68)|0;
 $70 = (+($69|0));
 $71 = $iny;
 $72 = (+($71|0));
 $73 = $dx;
 $74 = (+($73|0));
 $75 = $ady;
 $76 = (1)>($75|0);
 if ($76) {
  $79 = 1;
 } else {
  $77 = $ady;
  $79 = $77;
 }
 $78 = (+($79|0));
 $80 = $74 / $78;
 $81 = $72 * $80;
 $82 = $70 + $81;
 $x_factor = $82;
 $83 = $iny;
 $84 = $y_incr;
 $85 = Math_imul($83, $84)|0;
 $86 = (+($85|0));
 $87 = $inx;
 $88 = (+($87|0));
 $89 = $dy;
 $90 = (+($89|0));
 $91 = $adx;
 $92 = (1)>($91|0);
 if ($92) {
  $95 = 1;
 } else {
  $93 = $adx;
  $95 = $93;
 }
 $94 = (+($95|0));
 $96 = $90 / $94;
 $97 = $88 * $96;
 $98 = $86 + $97;
 $y_factor = $98;
 $99 = $4;
 $ix = $99;
 $100 = $5;
 $iy = $100;
 $101 = $4;
 $102 = (+($101|0));
 $rx = $102;
 $103 = $5;
 $104 = (+($103|0));
 $ry = $104;
 $i = 0;
 $105 = $4;
 $106 = $i;
 $107 = $x_list;
 $108 = (($107) + ($106<<2)|0);
 HEAP32[$108>>2] = $105;
 $109 = $5;
 $110 = $i;
 $111 = (($110) + 1)|0;
 $i = $111;
 $112 = $y_list;
 $113 = (($112) + ($110<<2)|0);
 HEAP32[$113>>2] = $109;
 while(1) {
  $114 = $ix;
  $115 = $6;
  $116 = ($114|0)!=($115|0);
  if ($116) {
   $200 = 1;
  } else {
   $117 = $iy;
   $118 = $7;
   $119 = ($117|0)!=($118|0);
   $200 = $119;
  }
  if (!($200)) {
   label = 39;
   break;
  }
  $120 = $i;
  $121 = $asize;
  $122 = ($120|0)>=($121|0);
  if ($122) {
   label = 31;
   break;
  }
  $126 = $x_factor;
  $127 = $rx;
  $128 = $127 + $126;
  $rx = $128;
  $129 = $y_factor;
  $130 = $ry;
  $131 = $130 + $129;
  $ry = $131;
  $132 = $rx;
  $133 = $132 < 0.0;
  if ($133) {
   $134 = $rx;
   $135 = $134 * 16384.0;
   $136 = $135 - 0.5;
   $137 = (~~(($136)));
   $138 = (+($137|0));
   $139 = $138 / 16384.0;
   $146 = $139;
  } else {
   $140 = $rx;
   $141 = $140 * 16384.0;
   $142 = $141 + 0.5;
   $143 = (~~(($142)));
   $144 = (+($143|0));
   $145 = $144 / 16384.0;
   $146 = $145;
  }
  $rx = $146;
  $147 = $ry;
  $148 = $147 < 0.0;
  if ($148) {
   $149 = $ry;
   $150 = $149 * 16384.0;
   $151 = $150 - 0.5;
   $152 = (~~(($151)));
   $153 = (+($152|0));
   $154 = $153 / 16384.0;
   $161 = $154;
  } else {
   $155 = $ry;
   $156 = $155 * 16384.0;
   $157 = $156 + 0.5;
   $158 = (~~(($157)));
   $159 = (+($158|0));
   $160 = $159 / 16384.0;
   $161 = $160;
  }
  $ry = $161;
  $162 = $intx;
  $163 = $ix;
  $164 = $x_incr;
  $165 = (($163) + ($164))|0;
  $166 = Math_imul($162, $165)|0;
  $167 = $iny;
  $168 = $rx;
  $169 = $168 + 0.5;
  $170 = (~~(($169)));
  $171 = Math_imul($167, $170)|0;
  $172 = (($166) + ($171))|0;
  $ix = $172;
  $173 = $inty;
  $174 = $iy;
  $175 = $y_incr;
  $176 = (($174) + ($175))|0;
  $177 = Math_imul($173, $176)|0;
  $178 = $inx;
  $179 = $ry;
  $180 = $179 + 0.5;
  $181 = (~~(($180)));
  $182 = Math_imul($178, $181)|0;
  $183 = (($177) + ($182))|0;
  $iy = $183;
  $184 = $ix;
  $185 = $i;
  $186 = $x_list;
  $187 = (($186) + ($185<<2)|0);
  HEAP32[$187>>2] = $184;
  $188 = $iy;
  $189 = $i;
  $190 = (($189) + 1)|0;
  $i = $190;
  $191 = $y_list;
  $192 = (($191) + ($189<<2)|0);
  HEAP32[$192>>2] = $188;
 }
 if ((label|0) == 31) {
  $123 = HEAP32[_stderr>>2]|0;
  (_fprintf(($123|0),(4664|0),($vararg_buffer3|0))|0);
  $124 = $x_list;
  _free($124);
  $125 = $y_list;
  _free($125);
  $0 = -412;
  $199 = $0;
  STACKTOP = sp;return ($199|0);
 }
 else if ((label|0) == 39) {
  $193 = $x_list;
  $194 = $1;
  HEAP32[$194>>2] = $193;
  $195 = $y_list;
  $196 = $2;
  HEAP32[$196>>2] = $195;
  $197 = $i;
  $198 = $3;
  HEAP32[$198>>2] = $197;
  $0 = 0;
  $199 = $0;
  STACKTOP = sp;return ($199|0);
 }
 return 0|0;
}
function _open_logfile() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 0;
}
function _print2log($fmt,$varargs) {
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $fmt;
 STACKTOP = sp;return;
}
function _close_logfile() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 0;
}
function _on_loop($minutia,$max_loop_len,$bdata,$iw,$ih) {
 $minutia = $minutia|0;
 $max_loop_len = $max_loop_len|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $ncontour = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $contour_x = sp + 16|0;
 $contour_y = sp + 12|0;
 $contour_ex = sp + 8|0;
 $contour_ey = sp + 4|0;
 $ncontour = sp;
 $1 = $minutia;
 $2 = $max_loop_len;
 $3 = $bdata;
 $4 = $iw;
 $5 = $ih;
 $6 = $2;
 $7 = $1;
 $8 = HEAP32[$7>>2]|0;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $1;
 $13 = HEAP32[$12>>2]|0;
 $14 = $1;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $1;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $1;
 $21 = (($20) + 12|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $3;
 $24 = $4;
 $25 = $5;
 $26 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$6,$8,$11,$13,$16,$19,$22,0,$23,$24,$25)|0);
 $ret = $26;
 $27 = $ret;
 $28 = ($27|0)==(2);
 if ($28) {
  $29 = $ret;
  $0 = $29;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 }
 $30 = $ret;
 $31 = ($30|0)==(1);
 if ($31) {
  $32 = HEAP32[$contour_x>>2]|0;
  $33 = HEAP32[$contour_y>>2]|0;
  $34 = HEAP32[$contour_ex>>2]|0;
  $35 = HEAP32[$contour_ey>>2]|0;
  _free_contour($32,$33,$34,$35);
  $0 = 1;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 }
 $36 = $ret;
 $37 = ($36|0)==(0);
 if ($37) {
  $38 = HEAP32[$contour_x>>2]|0;
  $39 = HEAP32[$contour_y>>2]|0;
  $40 = HEAP32[$contour_ex>>2]|0;
  $41 = HEAP32[$contour_ey>>2]|0;
  _free_contour($38,$39,$40,$41);
  $0 = 0;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 } else {
  $42 = $ret;
  $0 = $42;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 }
 return 0|0;
}
function _on_island_lake($ocontour_x,$ocontour_y,$ocontour_ex,$ocontour_ey,$oncontour,$minutia1,$minutia2,$max_half_loop,$bdata,$iw,$ih) {
 $ocontour_x = $ocontour_x|0;
 $ocontour_y = $ocontour_y|0;
 $ocontour_ex = $ocontour_ex|0;
 $ocontour_ey = $ocontour_ey|0;
 $oncontour = $oncontour|0;
 $minutia1 = $minutia1|0;
 $minutia2 = $minutia2|0;
 $max_half_loop = $max_half_loop|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $contour1_ex = 0, $contour1_ey = 0, $contour1_x = 0, $contour1_y = 0, $contour2_ex = 0, $contour2_ey = 0, $contour2_x = 0, $contour2_y = 0, $i = 0, $l = 0, $loop_ex = 0, $loop_ey = 0, $loop_x = 0, $loop_y = 0, $ncontour1 = 0, $ncontour2 = 0, $nloop = 0, $ret = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $contour1_x = sp + 56|0;
 $contour1_y = sp + 52|0;
 $contour1_ex = sp + 48|0;
 $contour1_ey = sp + 44|0;
 $ncontour1 = sp + 40|0;
 $contour2_x = sp + 36|0;
 $contour2_y = sp + 32|0;
 $contour2_ex = sp + 28|0;
 $contour2_ey = sp + 24|0;
 $ncontour2 = sp + 20|0;
 $loop_x = sp + 16|0;
 $loop_y = sp + 12|0;
 $loop_ex = sp + 8|0;
 $loop_ey = sp + 4|0;
 $1 = $ocontour_x;
 $2 = $ocontour_y;
 $3 = $ocontour_ex;
 $4 = $ocontour_ey;
 $5 = $oncontour;
 $6 = $minutia1;
 $7 = $minutia2;
 $8 = $max_half_loop;
 $9 = $bdata;
 $10 = $iw;
 $11 = $ih;
 $12 = $8;
 $13 = $7;
 $14 = HEAP32[$13>>2]|0;
 $15 = $7;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $6;
 $19 = HEAP32[$18>>2]|0;
 $20 = $6;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $6;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $6;
 $27 = (($26) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = $9;
 $30 = $10;
 $31 = $11;
 $32 = (_trace_contour($contour1_x,$contour1_y,$contour1_ex,$contour1_ey,$ncontour1,$12,$14,$17,$19,$22,$25,$28,0,$29,$30,$31)|0);
 $ret = $32;
 $33 = $ret;
 $34 = ($33|0)==(2);
 if ($34) {
  $35 = $ret;
  $0 = $35;
  $240 = $0;
  STACKTOP = sp;return ($240|0);
 }
 $36 = $ret;
 $37 = ($36|0)==(1);
 if (!($37)) {
  $233 = $ret;
  $234 = ($233|0)==(0);
  if ($234) {
   $235 = HEAP32[$contour1_x>>2]|0;
   $236 = HEAP32[$contour1_y>>2]|0;
   $237 = HEAP32[$contour1_ex>>2]|0;
   $238 = HEAP32[$contour1_ey>>2]|0;
   _free_contour($235,$236,$237,$238);
   $0 = 0;
   $240 = $0;
   STACKTOP = sp;return ($240|0);
  } else {
   $239 = $ret;
   $0 = $239;
   $240 = $0;
   STACKTOP = sp;return ($240|0);
  }
 }
 $38 = $8;
 $39 = $6;
 $40 = HEAP32[$39>>2]|0;
 $41 = $6;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $7;
 $45 = HEAP32[$44>>2]|0;
 $46 = $7;
 $47 = (($46) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = $7;
 $50 = (($49) + 8|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = $7;
 $53 = (($52) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $9;
 $56 = $10;
 $57 = $11;
 $58 = (_trace_contour($contour2_x,$contour2_y,$contour2_ex,$contour2_ey,$ncontour2,$38,$40,$43,$45,$48,$51,$54,0,$55,$56,$57)|0);
 $ret = $58;
 $59 = $ret;
 $60 = ($59|0)==(2);
 if ($60) {
  $61 = HEAP32[$contour1_x>>2]|0;
  $62 = HEAP32[$contour1_y>>2]|0;
  $63 = HEAP32[$contour1_ex>>2]|0;
  $64 = HEAP32[$contour1_ey>>2]|0;
  _free_contour($61,$62,$63,$64);
  $65 = $ret;
  $0 = $65;
  $240 = $0;
  STACKTOP = sp;return ($240|0);
 }
 $66 = $ret;
 $67 = ($66|0)==(1);
 if (!($67)) {
  $218 = $ret;
  $219 = ($218|0)==(0);
  if ($219) {
   $220 = HEAP32[$contour1_x>>2]|0;
   $221 = HEAP32[$contour1_y>>2]|0;
   $222 = HEAP32[$contour1_ex>>2]|0;
   $223 = HEAP32[$contour1_ey>>2]|0;
   _free_contour($220,$221,$222,$223);
   $224 = HEAP32[$contour2_x>>2]|0;
   $225 = HEAP32[$contour2_y>>2]|0;
   $226 = HEAP32[$contour2_ex>>2]|0;
   $227 = HEAP32[$contour2_ey>>2]|0;
   _free_contour($224,$225,$226,$227);
   $0 = 0;
   $240 = $0;
   STACKTOP = sp;return ($240|0);
  } else {
   $228 = HEAP32[$contour1_x>>2]|0;
   $229 = HEAP32[$contour1_y>>2]|0;
   $230 = HEAP32[$contour1_ex>>2]|0;
   $231 = HEAP32[$contour1_ey>>2]|0;
   _free_contour($228,$229,$230,$231);
   $232 = $ret;
   $0 = $232;
   $240 = $0;
   STACKTOP = sp;return ($240|0);
  }
 }
 $68 = HEAP32[$ncontour1>>2]|0;
 $69 = HEAP32[$ncontour2>>2]|0;
 $70 = (($68) + ($69))|0;
 $71 = (($70) + 2)|0;
 $nloop = $71;
 $72 = $nloop;
 $73 = (_allocate_contour($loop_x,$loop_y,$loop_ex,$loop_ey,$72)|0);
 $ret = $73;
 $74 = ($73|0)!=(0);
 if ($74) {
  $75 = HEAP32[$contour1_x>>2]|0;
  $76 = HEAP32[$contour1_y>>2]|0;
  $77 = HEAP32[$contour1_ex>>2]|0;
  $78 = HEAP32[$contour1_ey>>2]|0;
  _free_contour($75,$76,$77,$78);
  $79 = HEAP32[$contour2_x>>2]|0;
  $80 = HEAP32[$contour2_y>>2]|0;
  $81 = HEAP32[$contour2_ex>>2]|0;
  $82 = HEAP32[$contour2_ey>>2]|0;
  _free_contour($79,$80,$81,$82);
  $83 = $ret;
  $0 = $83;
  $240 = $0;
  STACKTOP = sp;return ($240|0);
 }
 $l = 0;
 $84 = $6;
 $85 = HEAP32[$84>>2]|0;
 $86 = $l;
 $87 = HEAP32[$loop_x>>2]|0;
 $88 = (($87) + ($86<<2)|0);
 HEAP32[$88>>2] = $85;
 $89 = $6;
 $90 = (($89) + 4|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $l;
 $93 = HEAP32[$loop_y>>2]|0;
 $94 = (($93) + ($92<<2)|0);
 HEAP32[$94>>2] = $91;
 $95 = $6;
 $96 = (($95) + 8|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = $l;
 $99 = HEAP32[$loop_ex>>2]|0;
 $100 = (($99) + ($98<<2)|0);
 HEAP32[$100>>2] = $97;
 $101 = $6;
 $102 = (($101) + 12|0);
 $103 = HEAP32[$102>>2]|0;
 $104 = $l;
 $105 = (($104) + 1)|0;
 $l = $105;
 $106 = HEAP32[$loop_ey>>2]|0;
 $107 = (($106) + ($104<<2)|0);
 HEAP32[$107>>2] = $103;
 $i = 0;
 while(1) {
  $108 = $i;
  $109 = HEAP32[$ncontour1>>2]|0;
  $110 = ($108|0)<($109|0);
  if (!($110)) {
   break;
  }
  $111 = $i;
  $112 = HEAP32[$contour1_x>>2]|0;
  $113 = (($112) + ($111<<2)|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = $l;
  $116 = HEAP32[$loop_x>>2]|0;
  $117 = (($116) + ($115<<2)|0);
  HEAP32[$117>>2] = $114;
  $118 = $i;
  $119 = HEAP32[$contour1_y>>2]|0;
  $120 = (($119) + ($118<<2)|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = $l;
  $123 = HEAP32[$loop_y>>2]|0;
  $124 = (($123) + ($122<<2)|0);
  HEAP32[$124>>2] = $121;
  $125 = $i;
  $126 = HEAP32[$contour1_ex>>2]|0;
  $127 = (($126) + ($125<<2)|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = $l;
  $130 = HEAP32[$loop_ex>>2]|0;
  $131 = (($130) + ($129<<2)|0);
  HEAP32[$131>>2] = $128;
  $132 = $i;
  $133 = HEAP32[$contour1_ey>>2]|0;
  $134 = (($133) + ($132<<2)|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = $l;
  $137 = (($136) + 1)|0;
  $l = $137;
  $138 = HEAP32[$loop_ey>>2]|0;
  $139 = (($138) + ($136<<2)|0);
  HEAP32[$139>>2] = $135;
  $140 = $i;
  $141 = (($140) + 1)|0;
  $i = $141;
 }
 $142 = $7;
 $143 = HEAP32[$142>>2]|0;
 $144 = $l;
 $145 = HEAP32[$loop_x>>2]|0;
 $146 = (($145) + ($144<<2)|0);
 HEAP32[$146>>2] = $143;
 $147 = $7;
 $148 = (($147) + 4|0);
 $149 = HEAP32[$148>>2]|0;
 $150 = $l;
 $151 = HEAP32[$loop_y>>2]|0;
 $152 = (($151) + ($150<<2)|0);
 HEAP32[$152>>2] = $149;
 $153 = $7;
 $154 = (($153) + 8|0);
 $155 = HEAP32[$154>>2]|0;
 $156 = $l;
 $157 = HEAP32[$loop_ex>>2]|0;
 $158 = (($157) + ($156<<2)|0);
 HEAP32[$158>>2] = $155;
 $159 = $7;
 $160 = (($159) + 12|0);
 $161 = HEAP32[$160>>2]|0;
 $162 = $l;
 $163 = (($162) + 1)|0;
 $l = $163;
 $164 = HEAP32[$loop_ey>>2]|0;
 $165 = (($164) + ($162<<2)|0);
 HEAP32[$165>>2] = $161;
 $i = 0;
 while(1) {
  $166 = $i;
  $167 = HEAP32[$ncontour2>>2]|0;
  $168 = ($166|0)<($167|0);
  if (!($168)) {
   break;
  }
  $169 = $i;
  $170 = HEAP32[$contour2_x>>2]|0;
  $171 = (($170) + ($169<<2)|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = $l;
  $174 = HEAP32[$loop_x>>2]|0;
  $175 = (($174) + ($173<<2)|0);
  HEAP32[$175>>2] = $172;
  $176 = $i;
  $177 = HEAP32[$contour2_y>>2]|0;
  $178 = (($177) + ($176<<2)|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = $l;
  $181 = HEAP32[$loop_y>>2]|0;
  $182 = (($181) + ($180<<2)|0);
  HEAP32[$182>>2] = $179;
  $183 = $i;
  $184 = HEAP32[$contour2_ex>>2]|0;
  $185 = (($184) + ($183<<2)|0);
  $186 = HEAP32[$185>>2]|0;
  $187 = $l;
  $188 = HEAP32[$loop_ex>>2]|0;
  $189 = (($188) + ($187<<2)|0);
  HEAP32[$189>>2] = $186;
  $190 = $i;
  $191 = HEAP32[$contour2_ey>>2]|0;
  $192 = (($191) + ($190<<2)|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = $l;
  $195 = (($194) + 1)|0;
  $l = $195;
  $196 = HEAP32[$loop_ey>>2]|0;
  $197 = (($196) + ($194<<2)|0);
  HEAP32[$197>>2] = $193;
  $198 = $i;
  $199 = (($198) + 1)|0;
  $i = $199;
 }
 $200 = HEAP32[$contour1_x>>2]|0;
 $201 = HEAP32[$contour1_y>>2]|0;
 $202 = HEAP32[$contour1_ex>>2]|0;
 $203 = HEAP32[$contour1_ey>>2]|0;
 _free_contour($200,$201,$202,$203);
 $204 = HEAP32[$contour2_x>>2]|0;
 $205 = HEAP32[$contour2_y>>2]|0;
 $206 = HEAP32[$contour2_ex>>2]|0;
 $207 = HEAP32[$contour2_ey>>2]|0;
 _free_contour($204,$205,$206,$207);
 $208 = HEAP32[$loop_x>>2]|0;
 $209 = $1;
 HEAP32[$209>>2] = $208;
 $210 = HEAP32[$loop_y>>2]|0;
 $211 = $2;
 HEAP32[$211>>2] = $210;
 $212 = HEAP32[$loop_ex>>2]|0;
 $213 = $3;
 HEAP32[$213>>2] = $212;
 $214 = HEAP32[$loop_ey>>2]|0;
 $215 = $4;
 HEAP32[$215>>2] = $214;
 $216 = $nloop;
 $217 = $5;
 HEAP32[$217>>2] = $216;
 $0 = 1;
 $240 = $0;
 STACKTOP = sp;return ($240|0);
}
function _on_hook($minutia1,$minutia2,$max_hook_len,$bdata,$iw,$ih) {
 $minutia1 = $minutia1|0;
 $minutia2 = $minutia2|0;
 $max_hook_len = $max_hook_len|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $ncontour = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $contour_x = sp + 16|0;
 $contour_y = sp + 12|0;
 $contour_ex = sp + 8|0;
 $contour_ey = sp + 4|0;
 $ncontour = sp;
 $1 = $minutia1;
 $2 = $minutia2;
 $3 = $max_hook_len;
 $4 = $bdata;
 $5 = $iw;
 $6 = $ih;
 $7 = $3;
 $8 = $2;
 $9 = HEAP32[$8>>2]|0;
 $10 = $2;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $1;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $1;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $1;
 $20 = HEAP32[$19>>2]|0;
 $21 = $1;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $4;
 $25 = $5;
 $26 = $6;
 $27 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$7,$9,$12,$15,$18,$20,$23,0,$24,$25,$26)|0);
 $ret = $27;
 $28 = $ret;
 $29 = ($28|0)==(2);
 if ($29) {
  $30 = $ret;
  $0 = $30;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $31 = $ret;
 $32 = ($31|0)==(1);
 if ($32) {
  $33 = HEAP32[$contour_x>>2]|0;
  $34 = HEAP32[$contour_y>>2]|0;
  $35 = HEAP32[$contour_ex>>2]|0;
  $36 = HEAP32[$contour_ey>>2]|0;
  _free_contour($33,$34,$35,$36);
  $0 = 1;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $37 = $ret;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = $ret;
  $0 = $39;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $40 = HEAP32[$contour_x>>2]|0;
 $41 = HEAP32[$contour_y>>2]|0;
 $42 = HEAP32[$contour_ex>>2]|0;
 $43 = HEAP32[$contour_ey>>2]|0;
 _free_contour($40,$41,$42,$43);
 $44 = $3;
 $45 = $2;
 $46 = HEAP32[$45>>2]|0;
 $47 = $2;
 $48 = (($47) + 4|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $1;
 $51 = (($50) + 8|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = $1;
 $54 = (($53) + 12|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = $1;
 $57 = HEAP32[$56>>2]|0;
 $58 = $1;
 $59 = (($58) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = $4;
 $62 = $5;
 $63 = $6;
 $64 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$44,$46,$49,$52,$55,$57,$60,1,$61,$62,$63)|0);
 $ret = $64;
 $65 = $ret;
 $66 = ($65|0)==(2);
 if ($66) {
  $67 = $ret;
  $0 = $67;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $68 = $ret;
 $69 = ($68|0)==(1);
 if ($69) {
  $70 = HEAP32[$contour_x>>2]|0;
  $71 = HEAP32[$contour_y>>2]|0;
  $72 = HEAP32[$contour_ex>>2]|0;
  $73 = HEAP32[$contour_ey>>2]|0;
  _free_contour($70,$71,$72,$73);
  $0 = 1;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $74 = $ret;
 $75 = ($74|0)==(0);
 if ($75) {
  $76 = HEAP32[$contour_x>>2]|0;
  $77 = HEAP32[$contour_y>>2]|0;
  $78 = HEAP32[$contour_ex>>2]|0;
  $79 = HEAP32[$contour_ey>>2]|0;
  _free_contour($76,$77,$78,$79);
  $0 = 0;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 } else {
  $80 = $ret;
  $0 = $80;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 return 0|0;
}
function _is_loop_clockwise($contour_x,$contour_y,$ncontour,$default_ret) {
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 $default_ret = $default_ret|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $chain = 0, $nchain = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $chain = sp + 4|0;
 $nchain = sp;
 $1 = $contour_x;
 $2 = $contour_y;
 $3 = $ncontour;
 $4 = $default_ret;
 $5 = $1;
 $6 = $2;
 $7 = $3;
 $8 = (_chain_code_loop($chain,$nchain,$5,$6,$7)|0);
 $ret = $8;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = $ret;
  $0 = $10;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 $11 = HEAP32[$nchain>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = $4;
  $0 = $13;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 } else {
  $14 = HEAP32[$chain>>2]|0;
  $15 = HEAP32[$nchain>>2]|0;
  $16 = $4;
  $17 = (_is_chain_clockwise($14,$15,$16)|0);
  $ret = $17;
  $18 = HEAP32[$chain>>2]|0;
  _free($18);
  $19 = $ret;
  $0 = $19;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _get_loop_aspect($omin_fr,$omin_to,$omin_dist,$omax_fr,$omax_to,$omax_dist,$contour_x,$contour_y,$ncontour) {
 $omin_fr = $omin_fr|0;
 $omin_to = $omin_to|0;
 $omin_dist = $omin_dist|0;
 $omax_fr = $omax_fr|0;
 $omax_to = $omax_to|0;
 $omax_dist = $omax_dist|0;
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0.0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0, $dist = 0.0, $halfway = 0;
 var $i = 0, $j = 0, $limit = 0, $max_dist = 0.0, $max_i = 0, $max_j = 0, $min_dist = 0.0, $min_i = 0, $min_j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $omin_fr;
 $1 = $omin_to;
 $2 = $omin_dist;
 $3 = $omax_fr;
 $4 = $omax_to;
 $5 = $omax_dist;
 $6 = $contour_x;
 $7 = $contour_y;
 $8 = $ncontour;
 $9 = $8;
 $10 = $9 >> 1;
 $halfway = $10;
 $i = 0;
 $11 = $halfway;
 $j = $11;
 $12 = $i;
 $13 = $6;
 $14 = (($13) + ($12<<2)|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $i;
 $17 = $7;
 $18 = (($17) + ($16<<2)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $j;
 $21 = $6;
 $22 = (($21) + ($20<<2)|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $j;
 $25 = $7;
 $26 = (($25) + ($24<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (+_squared_distance($15,$19,$23,$27));
 $dist = $28;
 $29 = $dist;
 $min_dist = $29;
 $30 = $i;
 $min_i = $30;
 $31 = $j;
 $min_j = $31;
 $32 = $dist;
 $max_dist = $32;
 $33 = $i;
 $max_i = $33;
 $34 = $j;
 $max_j = $34;
 $35 = $i;
 $36 = (($35) + 1)|0;
 $i = $36;
 $37 = $j;
 $38 = (($37) + 1)|0;
 $j = $38;
 $39 = $8;
 $40 = $j;
 $41 = (($40|0) % ($39|0))&-1;
 $j = $41;
 $42 = $8;
 $43 = (($42|0) % 2)&-1;
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = $8;
  $limit = $45;
 } else {
  $46 = $halfway;
  $limit = $46;
 }
 while(1) {
  $47 = $i;
  $48 = $limit;
  $49 = ($47|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = $i;
  $51 = $6;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $i;
  $55 = $7;
  $56 = (($55) + ($54<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $j;
  $59 = $6;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = $j;
  $63 = $7;
  $64 = (($63) + ($62<<2)|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (+_squared_distance($53,$57,$61,$65));
  $dist = $66;
  $67 = $dist;
  $68 = $min_dist;
  $69 = $67 < $68;
  if ($69) {
   $70 = $dist;
   $min_dist = $70;
   $71 = $i;
   $min_i = $71;
   $72 = $j;
   $min_j = $72;
  }
  $73 = $dist;
  $74 = $max_dist;
  $75 = $73 > $74;
  if ($75) {
   $76 = $dist;
   $max_dist = $76;
   $77 = $i;
   $max_i = $77;
   $78 = $j;
   $max_j = $78;
  }
  $79 = $i;
  $80 = (($79) + 1)|0;
  $i = $80;
  $81 = $j;
  $82 = (($81) + 1)|0;
  $j = $82;
  $83 = $8;
  $84 = $j;
  $85 = (($84|0) % ($83|0))&-1;
  $j = $85;
 }
 $86 = $min_i;
 $87 = $0;
 HEAP32[$87>>2] = $86;
 $88 = $min_j;
 $89 = $1;
 HEAP32[$89>>2] = $88;
 $90 = $min_dist;
 $91 = $2;
 HEAPF64[$91>>3] = $90;
 $92 = $max_i;
 $93 = $3;
 HEAP32[$93>>2] = $92;
 $94 = $max_j;
 $95 = $4;
 HEAP32[$95>>2] = $94;
 $96 = $max_dist;
 $97 = $5;
 HEAPF64[$97>>3] = $96;
 STACKTOP = sp;return;
}
function _fill_loop($contour_x,$contour_y,$ncontour,$bdata,$iw,$ih) {
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $edge_pix = 0, $feature_pix = 0, $i = 0, $j = 0, $lastj = 0, $next_pix = 0, $nx = 0, $ret = 0, $shape = 0, $vararg_buffer = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $shape = sp + 44|0;
 $1 = $contour_x;
 $2 = $contour_y;
 $3 = $ncontour;
 $4 = $bdata;
 $5 = $iw;
 $6 = $ih;
 $7 = $1;
 $8 = $2;
 $9 = $3;
 $10 = (_shape_from_contour($shape,$7,$8,$9)|0);
 $ret = $10;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $ret;
  $0 = $12;
  $140 = $0;
  STACKTOP = sp;return ($140|0);
 }
 $13 = $4;
 $14 = $2;
 $15 = HEAP32[$14>>2]|0;
 $16 = $5;
 $17 = Math_imul($15, $16)|0;
 $18 = (($13) + ($17)|0);
 $19 = $1;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + ($20)|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $feature_pix = $23;
 $24 = $feature_pix;
 $25 = ($24|0)!=(0);
 if ($25) {
  $edge_pix = 0;
 } else {
  $edge_pix = 1;
 }
 $i = 0;
 while(1) {
  $26 = $i;
  $27 = HEAP32[$shape>>2]|0;
  $28 = (($27) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)<($29|0);
  if (!($30)) {
   label = 18;
   break;
  }
  $31 = $i;
  $32 = HEAP32[$shape>>2]|0;
  $33 = (($32) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31<<2)|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $y = $37;
  $38 = $i;
  $39 = HEAP32[$shape>>2]|0;
  $40 = (($39) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($38<<2)|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 12|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)<(1);
  if ($46) {
   label = 9;
   break;
  }
  $j = 0;
  $49 = $j;
  $50 = $i;
  $51 = HEAP32[$shape>>2]|0;
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($50<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($49<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $x = $59;
  $60 = $edge_pix;
  $61 = $60&255;
  $62 = $4;
  $63 = $y;
  $64 = $5;
  $65 = Math_imul($63, $64)|0;
  $66 = (($62) + ($65)|0);
  $67 = $x;
  $68 = (($66) + ($67)|0);
  HEAP8[$68>>0] = $61;
  $69 = $i;
  $70 = HEAP32[$shape>>2]|0;
  $71 = (($70) + 8|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + ($69<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76) - 1)|0;
  $lastj = $77;
  while(1) {
   $78 = $j;
   $79 = $lastj;
   $80 = ($78|0)<($79|0);
   if (!($80)) {
    break;
   }
   $81 = $x;
   $82 = (($81) + 1)|0;
   $x = $82;
   $83 = $4;
   $84 = $y;
   $85 = $5;
   $86 = Math_imul($84, $85)|0;
   $87 = (($83) + ($86)|0);
   $88 = $x;
   $89 = (($87) + ($88)|0);
   $90 = HEAP8[$89>>0]|0;
   $91 = $90&255;
   $next_pix = $91;
   $92 = $next_pix;
   $93 = $edge_pix;
   $94 = ($92|0)==($93|0);
   if ($94) {
    $95 = $j;
    $96 = (($95) + 1)|0;
    $j = $96;
    $97 = $j;
    $98 = $i;
    $99 = HEAP32[$shape>>2]|0;
    $100 = (($99) + 8|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = (($101) + ($98<<2)|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) + 4|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = (($105) + ($97<<2)|0);
    $107 = HEAP32[$106>>2]|0;
    $x = $107;
    $108 = $edge_pix;
    $109 = $108&255;
    $110 = $4;
    $111 = $y;
    $112 = $5;
    $113 = Math_imul($111, $112)|0;
    $114 = (($110) + ($113)|0);
    $115 = $x;
    $116 = (($114) + ($115)|0);
    HEAP8[$116>>0] = $109;
   } else {
    $117 = $j;
    $118 = (($117) + 1)|0;
    $j = $118;
    $119 = $j;
    $120 = $i;
    $121 = HEAP32[$shape>>2]|0;
    $122 = (($121) + 8|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = (($123) + ($120<<2)|0);
    $125 = HEAP32[$124>>2]|0;
    $126 = (($125) + 4|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = (($127) + ($119<<2)|0);
    $129 = HEAP32[$128>>2]|0;
    $nx = $129;
    $130 = $edge_pix;
    $131 = $x;
    $132 = $nx;
    $133 = $y;
    $134 = $4;
    $135 = $5;
    $136 = $6;
    _fill_partial_row($130,$131,$132,$133,$134,$135,$136);
   }
  }
  $137 = $i;
  $138 = (($137) + 1)|0;
  $i = $138;
 }
 if ((label|0) == 9) {
  $47 = HEAP32[$shape>>2]|0;
  _free_shape($47);
  $48 = HEAP32[_stderr>>2]|0;
  (_fprintf(($48|0),(4712|0),($vararg_buffer|0))|0);
  $0 = 0;
  $140 = $0;
  STACKTOP = sp;return ($140|0);
 }
 else if ((label|0) == 18) {
  $139 = HEAP32[$shape>>2]|0;
  _free_shape($139);
  $0 = 0;
  $140 = $0;
  STACKTOP = sp;return ($140|0);
 }
 return 0|0;
}
function _process_loop_V2($minutiae,$contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$bdata,$iw,$ih,$plow_flow_map,$lfsparms) {
 $minutiae = $minutiae|0;
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $contour_ex = $contour_ex|0;
 $contour_ey = $contour_ey|0;
 $ncontour = $ncontour|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $plow_flow_map = $plow_flow_map|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0.0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0.0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0, $45 = 0.0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $appearing = 0, $feature_pix = 0, $fmapval = 0;
 var $halfway = 0, $idir = 0, $max_dist = 0, $max_fr = 0, $max_to = 0, $mid_pix = 0, $mid_x = 0, $mid_y = 0, $min_dist = 0, $min_fr = 0, $min_to = 0, $minutia = 0, $reliability = 0.0, $ret = 0, $type = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $min_dist = sp + 16|0;
 $max_dist = sp + 8|0;
 $min_fr = sp + 64|0;
 $max_fr = sp + 60|0;
 $min_to = sp + 56|0;
 $max_to = sp + 52|0;
 $minutia = sp + 28|0;
 $1 = $minutiae;
 $2 = $contour_x;
 $3 = $contour_y;
 $4 = $contour_ex;
 $5 = $contour_ey;
 $6 = $ncontour;
 $7 = $bdata;
 $8 = $iw;
 $9 = $ih;
 $10 = $plow_flow_map;
 $11 = $lfsparms;
 $12 = $6;
 $13 = ($12|0)<=(0);
 if ($13) {
  $0 = 0;
  $256 = $0;
  STACKTOP = sp;return ($256|0);
 }
 $14 = $6;
 $15 = $11;
 $16 = (($15) + 172|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14|0)>($17|0);
 if ($18) {
  $19 = $7;
  $20 = $3;
  $21 = HEAP32[$20>>2]|0;
  $22 = $8;
  $23 = Math_imul($21, $22)|0;
  $24 = (($19) + ($23)|0);
  $25 = $2;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($24) + ($26)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $feature_pix = $29;
  $30 = $6;
  $31 = $30 >> 1;
  $halfway = $31;
  $32 = $2;
  $33 = $3;
  $34 = $6;
  _get_loop_aspect($min_fr,$min_to,$min_dist,$max_fr,$max_to,$max_dist,$32,$33,$34);
  $35 = +HEAPF64[$min_dist>>3];
  $36 = $11;
  $37 = (($36) + 176|0);
  $38 = +HEAPF64[$37>>3];
  $39 = $35 < $38;
  if ($39) {
   label = 6;
  } else {
   $40 = +HEAPF64[$max_dist>>3];
   $41 = +HEAPF64[$min_dist>>3];
   $42 = $40 / $41;
   $43 = $11;
   $44 = (($43) + 184|0);
   $45 = +HEAPF64[$44>>3];
   $46 = $42 >= $45;
   if ($46) {
    label = 6;
   }
  }
  do {
   if ((label|0) == 6) {
    $47 = HEAP32[$max_fr>>2]|0;
    $48 = $2;
    $49 = (($48) + ($47<<2)|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$max_to>>2]|0;
    $52 = $2;
    $53 = (($52) + ($51<<2)|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($50) + ($54))|0;
    $56 = $55 >> 1;
    $mid_x = $56;
    $57 = HEAP32[$max_fr>>2]|0;
    $58 = $3;
    $59 = (($58) + ($57<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$max_to>>2]|0;
    $62 = $3;
    $63 = (($62) + ($61<<2)|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (($60) + ($64))|0;
    $66 = $65 >> 1;
    $mid_y = $66;
    $67 = $7;
    $68 = $mid_y;
    $69 = $8;
    $70 = Math_imul($68, $69)|0;
    $71 = (($67) + ($70)|0);
    $72 = $mid_x;
    $73 = (($71) + ($72)|0);
    $74 = HEAP8[$73>>0]|0;
    $75 = $74&255;
    $mid_pix = $75;
    $76 = $mid_pix;
    $77 = $feature_pix;
    $78 = ($76|0)==($77|0);
    if (!($78)) {
     break;
    }
    $79 = HEAP32[$max_fr>>2]|0;
    $80 = $2;
    $81 = (($80) + ($79<<2)|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$max_fr>>2]|0;
    $84 = $3;
    $85 = (($84) + ($83<<2)|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = HEAP32[$max_to>>2]|0;
    $88 = $2;
    $89 = (($88) + ($87<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$max_to>>2]|0;
    $92 = $3;
    $93 = (($92) + ($91<<2)|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = $11;
    $96 = (($95) + 20|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = (_line2direction($82,$86,$90,$94,$97)|0);
    $idir = $98;
    $99 = $feature_pix;
    $100 = (_minutia_type($99)|0);
    $type = $100;
    $101 = HEAP32[$max_fr>>2]|0;
    $102 = $2;
    $103 = (($102) + ($101<<2)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = HEAP32[$max_fr>>2]|0;
    $106 = $3;
    $107 = (($106) + ($105<<2)|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = HEAP32[$max_fr>>2]|0;
    $110 = $4;
    $111 = (($110) + ($109<<2)|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$max_fr>>2]|0;
    $114 = $5;
    $115 = (($114) + ($113<<2)|0);
    $116 = HEAP32[$115>>2]|0;
    $117 = (_is_minutia_appearing($104,$108,$112,$116)|0);
    $appearing = $117;
    $118 = ($117|0)<(0);
    if ($118) {
     $119 = $appearing;
     $0 = $119;
     $256 = $0;
     STACKTOP = sp;return ($256|0);
    }
    $120 = $10;
    $121 = HEAP32[$max_fr>>2]|0;
    $122 = $3;
    $123 = (($122) + ($121<<2)|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = $8;
    $126 = Math_imul($124, $125)|0;
    $127 = (($120) + ($126<<2)|0);
    $128 = HEAP32[$max_fr>>2]|0;
    $129 = $2;
    $130 = (($129) + ($128<<2)|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = (($127) + ($131<<2)|0);
    $133 = HEAP32[$132>>2]|0;
    $fmapval = $133;
    $134 = $fmapval;
    $135 = ($134|0)!=(0);
    if ($135) {
     $reliability = 0.5;
    } else {
     $reliability = 0.989999999999999991118;
    }
    $136 = HEAP32[$max_fr>>2]|0;
    $137 = $2;
    $138 = (($137) + ($136<<2)|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = HEAP32[$max_fr>>2]|0;
    $141 = $3;
    $142 = (($141) + ($140<<2)|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = HEAP32[$max_fr>>2]|0;
    $145 = $4;
    $146 = (($145) + ($144<<2)|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = HEAP32[$max_fr>>2]|0;
    $149 = $5;
    $150 = (($149) + ($148<<2)|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = $idir;
    $153 = $reliability;
    $154 = $type;
    $155 = $appearing;
    $156 = (_create_minutia($minutia,$139,$143,$147,$151,$152,$153,$154,$155,10)|0);
    $ret = $156;
    $157 = ($156|0)!=(0);
    if ($157) {
     $158 = $ret;
     $0 = $158;
     $256 = $0;
     STACKTOP = sp;return ($256|0);
    }
    $159 = $1;
    $160 = HEAP32[$minutia>>2]|0;
    $161 = $7;
    $162 = $8;
    $163 = $9;
    $164 = $11;
    $165 = (_update_minutiae($159,$160,$161,$162,$163,$164)|0);
    $ret = $165;
    $166 = $ret;
    $167 = ($166|0)==(2);
    if ($167) {
     $168 = HEAP32[$minutia>>2]|0;
     _free_minutia($168);
    }
    $169 = $11;
    $170 = (($169) + 20|0);
    $171 = HEAP32[$170>>2]|0;
    $172 = $idir;
    $173 = (($172) + ($171))|0;
    $idir = $173;
    $174 = $11;
    $175 = (($174) + 20|0);
    $176 = HEAP32[$175>>2]|0;
    $177 = $176 << 1;
    $178 = $idir;
    $179 = (($178|0) % ($177|0))&-1;
    $idir = $179;
    $180 = HEAP32[$max_to>>2]|0;
    $181 = $2;
    $182 = (($181) + ($180<<2)|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = HEAP32[$max_to>>2]|0;
    $185 = $3;
    $186 = (($185) + ($184<<2)|0);
    $187 = HEAP32[$186>>2]|0;
    $188 = HEAP32[$max_to>>2]|0;
    $189 = $4;
    $190 = (($189) + ($188<<2)|0);
    $191 = HEAP32[$190>>2]|0;
    $192 = HEAP32[$max_to>>2]|0;
    $193 = $5;
    $194 = (($193) + ($192<<2)|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = (_is_minutia_appearing($183,$187,$191,$195)|0);
    $appearing = $196;
    $197 = ($196|0)<(0);
    if ($197) {
     $198 = $appearing;
     $0 = $198;
     $256 = $0;
     STACKTOP = sp;return ($256|0);
    }
    $199 = $10;
    $200 = HEAP32[$max_to>>2]|0;
    $201 = $3;
    $202 = (($201) + ($200<<2)|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = $8;
    $205 = Math_imul($203, $204)|0;
    $206 = (($199) + ($205<<2)|0);
    $207 = HEAP32[$max_to>>2]|0;
    $208 = $2;
    $209 = (($208) + ($207<<2)|0);
    $210 = HEAP32[$209>>2]|0;
    $211 = (($206) + ($210<<2)|0);
    $212 = HEAP32[$211>>2]|0;
    $fmapval = $212;
    $213 = $fmapval;
    $214 = ($213|0)!=(0);
    if ($214) {
     $reliability = 0.5;
    } else {
     $reliability = 0.989999999999999991118;
    }
    $215 = HEAP32[$max_to>>2]|0;
    $216 = $2;
    $217 = (($216) + ($215<<2)|0);
    $218 = HEAP32[$217>>2]|0;
    $219 = HEAP32[$max_to>>2]|0;
    $220 = $3;
    $221 = (($220) + ($219<<2)|0);
    $222 = HEAP32[$221>>2]|0;
    $223 = HEAP32[$max_to>>2]|0;
    $224 = $4;
    $225 = (($224) + ($223<<2)|0);
    $226 = HEAP32[$225>>2]|0;
    $227 = HEAP32[$max_to>>2]|0;
    $228 = $5;
    $229 = (($228) + ($227<<2)|0);
    $230 = HEAP32[$229>>2]|0;
    $231 = $idir;
    $232 = $reliability;
    $233 = $type;
    $234 = $appearing;
    $235 = (_create_minutia($minutia,$218,$222,$226,$230,$231,$232,$233,$234,10)|0);
    $ret = $235;
    $236 = ($235|0)!=(0);
    if ($236) {
     $237 = $ret;
     $0 = $237;
     $256 = $0;
     STACKTOP = sp;return ($256|0);
    }
    $238 = $1;
    $239 = HEAP32[$minutia>>2]|0;
    $240 = $7;
    $241 = $8;
    $242 = $9;
    $243 = $11;
    $244 = (_update_minutiae($238,$239,$240,$241,$242,$243)|0);
    $ret = $244;
    $245 = $ret;
    $246 = ($245|0)==(2);
    if ($246) {
     $247 = HEAP32[$minutia>>2]|0;
     _free_minutia($247);
    }
    $0 = 0;
    $256 = $0;
    STACKTOP = sp;return ($256|0);
   }
  } while(0);
 }
 $248 = $2;
 $249 = $3;
 $250 = $6;
 $251 = $7;
 $252 = $8;
 $253 = $9;
 $254 = (_fill_loop($248,$249,$250,$251,$252,$253)|0);
 $ret = $254;
 $255 = $ret;
 $0 = $255;
 $256 = $0;
 STACKTOP = sp;return ($256|0);
}
function _fill_partial_row($fill_pix,$frx,$tox,$y,$bdata,$iw,$ih) {
 $fill_pix = $fill_pix|0;
 $frx = $frx|0;
 $tox = $tox|0;
 $y = $y|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bptr = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $fill_pix;
 $1 = $frx;
 $2 = $tox;
 $3 = $y;
 $4 = $bdata;
 $5 = $iw;
 $6 = $ih;
 $7 = $4;
 $8 = $3;
 $9 = $5;
 $10 = Math_imul($8, $9)|0;
 $11 = (($7) + ($10)|0);
 $12 = $1;
 $13 = (($11) + ($12)|0);
 $bptr = $13;
 $14 = $1;
 $x = $14;
 while(1) {
  $15 = $x;
  $16 = $2;
  $17 = ($15|0)<=($16|0);
  if (!($17)) {
   break;
  }
  $18 = $0;
  $19 = $18&255;
  $20 = $bptr;
  HEAP8[$20>>0] = $19;
  $21 = $bptr;
  $22 = (($21) + 1|0);
  $bptr = $22;
  $23 = $x;
  $24 = (($23) + 1)|0;
  $x = $24;
 }
 STACKTOP = sp;return;
}
function _gen_image_maps($odmap,$olcmap,$olfmap,$ohcmap,$omw,$omh,$pdata,$pw,$ph,$dir2rad,$dftwaves,$dftgrids,$lfsparms) {
 $odmap = $odmap|0;
 $olcmap = $olcmap|0;
 $olfmap = $olfmap|0;
 $ohcmap = $ohcmap|0;
 $omw = $omw|0;
 $omh = $omh|0;
 $pdata = $pdata|0;
 $pw = $pw|0;
 $ph = $ph|0;
 $dir2rad = $dir2rad|0;
 $dftwaves = $dftwaves|0;
 $dftgrids = $dftgrids|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $blkoffs = 0, $direction_map = 0, $high_curve_map = 0, $ih = 0;
 var $iw = 0, $low_contrast_map = 0, $low_flow_map = 0, $mh = 0, $mw = 0, $ret = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp;
 $direction_map = sp + 40|0;
 $low_contrast_map = sp + 36|0;
 $low_flow_map = sp + 32|0;
 $high_curve_map = sp + 28|0;
 $mw = sp + 24|0;
 $mh = sp + 20|0;
 $blkoffs = sp + 8|0;
 $1 = $odmap;
 $2 = $olcmap;
 $3 = $olfmap;
 $4 = $ohcmap;
 $5 = $omw;
 $6 = $omh;
 $7 = $pdata;
 $8 = $pw;
 $9 = $ph;
 $10 = $dir2rad;
 $11 = $dftwaves;
 $12 = $dftgrids;
 $13 = $lfsparms;
 $14 = $12;
 $15 = (($14) + 20|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $12;
 $18 = (($17) + 24|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($16|0)!=($19|0);
 if ($20) {
  $21 = HEAP32[_stderr>>2]|0;
  (_fprintf(($21|0),(4776|0),($vararg_buffer|0))|0);
  $0 = -540;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $22 = $8;
 $23 = $12;
 $24 = HEAP32[$23>>2]|0;
 $25 = $24 << 1;
 $26 = (($22) - ($25))|0;
 $iw = $26;
 $27 = $9;
 $28 = $12;
 $29 = HEAP32[$28>>2]|0;
 $30 = $29 << 1;
 $31 = (($27) - ($30))|0;
 $ih = $31;
 $32 = $iw;
 $33 = $ih;
 $34 = $12;
 $35 = HEAP32[$34>>2]|0;
 $36 = $13;
 $37 = (($36) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (_block_offsets($blkoffs,$mw,$mh,$32,$33,$35,$38)|0);
 $ret = $39;
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = $ret;
  $0 = $41;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $42 = HEAP32[$blkoffs>>2]|0;
 $43 = HEAP32[$mw>>2]|0;
 $44 = HEAP32[$mh>>2]|0;
 $45 = $7;
 $46 = $8;
 $47 = $9;
 $48 = $11;
 $49 = $12;
 $50 = $13;
 $51 = (_gen_initial_maps($direction_map,$low_contrast_map,$low_flow_map,$42,$43,$44,$45,$46,$47,$48,$49,$50)|0);
 $ret = $51;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$blkoffs>>2]|0;
  _free($53);
  $54 = $ret;
  $0 = $54;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $55 = HEAP32[$low_flow_map>>2]|0;
 $56 = HEAP32[$mw>>2]|0;
 $57 = HEAP32[$mh>>2]|0;
 $58 = $13;
 $59 = (_morph_TF_map($55,$56,$57,$58)|0);
 $ret = $59;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = $ret;
  $0 = $61;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $62 = HEAP32[$direction_map>>2]|0;
 $63 = HEAP32[$mw>>2]|0;
 $64 = HEAP32[$mh>>2]|0;
 $65 = $10;
 $66 = $13;
 _remove_incon_dirs($62,$63,$64,$65,$66);
 $67 = HEAP32[$direction_map>>2]|0;
 $68 = HEAP32[$low_contrast_map>>2]|0;
 $69 = HEAP32[$mw>>2]|0;
 $70 = HEAP32[$mh>>2]|0;
 $71 = $10;
 $72 = $13;
 _smooth_direction_map($67,$68,$69,$70,$71,$72);
 $73 = HEAP32[$direction_map>>2]|0;
 $74 = HEAP32[$low_contrast_map>>2]|0;
 $75 = HEAP32[$mw>>2]|0;
 $76 = HEAP32[$mh>>2]|0;
 $77 = $13;
 $78 = (_interpolate_direction_map($73,$74,$75,$76,$77)|0);
 $ret = $78;
 $79 = ($78|0)!=(0);
 if ($79) {
  $80 = $ret;
  $0 = $80;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $81 = HEAP32[$direction_map>>2]|0;
 $82 = HEAP32[$mw>>2]|0;
 $83 = HEAP32[$mh>>2]|0;
 $84 = $10;
 $85 = $13;
 _remove_incon_dirs($81,$82,$83,$84,$85);
 $86 = HEAP32[$direction_map>>2]|0;
 $87 = HEAP32[$low_contrast_map>>2]|0;
 $88 = HEAP32[$mw>>2]|0;
 $89 = HEAP32[$mh>>2]|0;
 $90 = $10;
 $91 = $13;
 _smooth_direction_map($86,$87,$88,$89,$90,$91);
 $92 = HEAP32[$direction_map>>2]|0;
 $93 = HEAP32[$mw>>2]|0;
 $94 = HEAP32[$mh>>2]|0;
 _set_margin_blocks($92,$93,$94,-1);
 $95 = HEAP32[$direction_map>>2]|0;
 $96 = HEAP32[$mw>>2]|0;
 $97 = HEAP32[$mh>>2]|0;
 $98 = $13;
 $99 = (_gen_high_curve_map($high_curve_map,$95,$96,$97,$98)|0);
 $ret = $99;
 $100 = ($99|0)!=(0);
 if ($100) {
  $101 = $ret;
  $0 = $101;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 } else {
  $102 = HEAP32[$blkoffs>>2]|0;
  _free($102);
  $103 = HEAP32[$direction_map>>2]|0;
  $104 = $1;
  HEAP32[$104>>2] = $103;
  $105 = HEAP32[$low_contrast_map>>2]|0;
  $106 = $2;
  HEAP32[$106>>2] = $105;
  $107 = HEAP32[$low_flow_map>>2]|0;
  $108 = $3;
  HEAP32[$108>>2] = $107;
  $109 = HEAP32[$high_curve_map>>2]|0;
  $110 = $4;
  HEAP32[$110>>2] = $109;
  $111 = HEAP32[$mw>>2]|0;
  $112 = $5;
  HEAP32[$112>>2] = $111;
  $113 = HEAP32[$mh>>2]|0;
  $114 = $6;
  HEAP32[$114>>2] = $113;
  $0 = 0;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 return 0|0;
}
function _gen_initial_maps($odmap,$olcmap,$olfmap,$blkoffs,$mw,$mh,$pdata,$pw,$ph,$dftwaves,$dftgrids,$lfsparms) {
 $odmap = $odmap|0;
 $olcmap = $olcmap|0;
 $olfmap = $olfmap|0;
 $blkoffs = $blkoffs|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $pdata = $pdata|0;
 $pw = $pw|0;
 $ph = $ph|0;
 $dftwaves = $dftwaves|0;
 $dftgrids = $dftgrids|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bi = 0, $blkdir = 0, $bsize = 0, $dft_offset = 0, $direction_map = 0, $low_contrast_map = 0, $low_contrast_offset = 0, $low_flow_map = 0, $nstats = 0, $powers = 0, $powmax_dirs = 0, $powmaxs = 0, $pownorms = 0;
 var $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr9 = 0, $win_x = 0, $win_y = 0, $wis = 0, $xmaxlimit = 0, $xminlimit = 0, $ymaxlimit = 0, $yminlimit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer13 = sp + 56|0;
 $vararg_buffer11 = sp + 48|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $wis = sp + 116|0;
 $powmax_dirs = sp + 112|0;
 $powers = sp + 108|0;
 $powmaxs = sp + 104|0;
 $pownorms = sp + 100|0;
 $1 = $odmap;
 $2 = $olcmap;
 $3 = $olfmap;
 $4 = $blkoffs;
 $5 = $mw;
 $6 = $mh;
 $7 = $pdata;
 $8 = $pw;
 $9 = $ph;
 $10 = $dftwaves;
 $11 = $dftgrids;
 $12 = $lfsparms;
 _print2log(4832,$vararg_buffer);
 $13 = $5;
 $14 = $6;
 $15 = Math_imul($13, $14)|0;
 $bsize = $15;
 $16 = $bsize;
 $17 = $16<<2;
 $18 = (_malloc($17)|0);
 $direction_map = $18;
 $19 = $direction_map;
 $20 = ($19|0)==(0|0);
 if ($20) {
  $21 = HEAP32[_stderr>>2]|0;
  (_fprintf(($21|0),(4848|0),($vararg_buffer1|0))|0);
  $0 = -550;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 $22 = $direction_map;
 $23 = $bsize;
 $24 = $23<<2;
 _memset(($22|0),-1,($24|0))|0;
 $25 = $bsize;
 $26 = $25<<2;
 $27 = (_malloc($26)|0);
 $low_contrast_map = $27;
 $28 = $low_contrast_map;
 $29 = ($28|0)==(0|0);
 if ($29) {
  $30 = $direction_map;
  _free($30);
  $31 = HEAP32[_stderr>>2]|0;
  (_fprintf(($31|0),(4904|0),($vararg_buffer3|0))|0);
  $0 = -551;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 $32 = $low_contrast_map;
 $33 = $bsize;
 $34 = $33<<2;
 _memset(($32|0),0,($34|0))|0;
 $35 = $bsize;
 $36 = $35<<2;
 $37 = (_malloc($36)|0);
 $low_flow_map = $37;
 $38 = $low_flow_map;
 $39 = ($38|0)==(0|0);
 if ($39) {
  $40 = $direction_map;
  _free($40);
  $41 = $low_contrast_map;
  _free($41);
  $42 = HEAP32[_stderr>>2]|0;
  (_fprintf(($42|0),(4960|0),($vararg_buffer5|0))|0);
  $0 = -552;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 $43 = $low_flow_map;
 $44 = $bsize;
 $45 = $44<<2;
 _memset(($43|0),0,($45|0))|0;
 $46 = $10;
 $47 = HEAP32[$46>>2]|0;
 $48 = $11;
 $49 = (($48) + 16|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (_alloc_dir_powers($powers,$47,$50)|0);
 $ret = $51;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = $direction_map;
  _free($53);
  $54 = $low_contrast_map;
  _free($54);
  $55 = $low_flow_map;
  _free($55);
  $56 = $ret;
  $0 = $56;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 $57 = $10;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($58) - 1)|0;
 $nstats = $59;
 $60 = $nstats;
 $61 = (_alloc_power_stats($wis,$powmaxs,$powmax_dirs,$pownorms,$60)|0);
 $ret = $61;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = $direction_map;
  _free($63);
  $64 = $low_contrast_map;
  _free($64);
  $65 = $low_flow_map;
  _free($65);
  $66 = HEAP32[$powers>>2]|0;
  $67 = $10;
  $68 = HEAP32[$67>>2]|0;
  _free_dir_powers($66,$68);
  $69 = $ret;
  $0 = $69;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 $70 = $11;
 $71 = HEAP32[$70>>2]|0;
 $xminlimit = $71;
 $72 = $11;
 $73 = HEAP32[$72>>2]|0;
 $yminlimit = $73;
 $74 = $8;
 $75 = $11;
 $76 = HEAP32[$75>>2]|0;
 $77 = (($74) - ($76))|0;
 $78 = $12;
 $79 = (($78) + 12|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = (($77) - ($80))|0;
 $82 = (($81) - 1)|0;
 $xmaxlimit = $82;
 $83 = $9;
 $84 = $11;
 $85 = HEAP32[$84>>2]|0;
 $86 = (($83) - ($85))|0;
 $87 = $12;
 $88 = (($87) + 12|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($86) - ($89))|0;
 $91 = (($90) - 1)|0;
 $ymaxlimit = $91;
 $bi = 0;
 while(1) {
  $92 = $bi;
  $93 = $bsize;
  $94 = ($92|0)<($93|0);
  if (!($94)) {
   label = 42;
   break;
  }
  $95 = $bi;
  $96 = $4;
  $97 = (($96) + ($95<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = $12;
  $100 = (($99) + 16|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = $8;
  $103 = Math_imul($101, $102)|0;
  $104 = (($98) - ($103))|0;
  $105 = $12;
  $106 = (($105) + 16|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($104) - ($107))|0;
  $dft_offset = $108;
  $109 = $dft_offset;
  $110 = $8;
  $111 = (($109|0) % ($110|0))&-1;
  $win_x = $111;
  $112 = $dft_offset;
  $113 = $8;
  $114 = (($112|0) / ($113|0))&-1;
  $win_y = $114;
  $115 = $xminlimit;
  $116 = $win_x;
  $117 = ($115|0)>($116|0);
  if ($117) {
   $118 = $xminlimit;
   $120 = $118;
  } else {
   $119 = $win_x;
   $120 = $119;
  }
  $win_x = $120;
  $121 = $xmaxlimit;
  $122 = $win_x;
  $123 = ($121|0)<($122|0);
  if ($123) {
   $124 = $xmaxlimit;
   $126 = $124;
  } else {
   $125 = $win_x;
   $126 = $125;
  }
  $win_x = $126;
  $127 = $yminlimit;
  $128 = $win_y;
  $129 = ($127|0)>($128|0);
  if ($129) {
   $130 = $yminlimit;
   $132 = $130;
  } else {
   $131 = $win_y;
   $132 = $131;
  }
  $win_y = $132;
  $133 = $ymaxlimit;
  $134 = $win_y;
  $135 = ($133|0)<($134|0);
  if ($135) {
   $136 = $ymaxlimit;
   $138 = $136;
  } else {
   $137 = $win_y;
   $138 = $137;
  }
  $win_y = $138;
  $139 = $win_y;
  $140 = $8;
  $141 = Math_imul($139, $140)|0;
  $142 = $win_x;
  $143 = (($141) + ($142))|0;
  $low_contrast_offset = $143;
  $144 = $bi;
  $145 = $bi;
  $146 = $5;
  $147 = (($145|0) % ($146|0))&-1;
  $148 = $bi;
  $149 = $5;
  $150 = (($148|0) / ($149|0))&-1;
  HEAP32[$vararg_buffer7>>2] = $144;
  $vararg_ptr9 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $147;
  $vararg_ptr10 = (($vararg_buffer7) + 8|0);
  HEAP32[$vararg_ptr10>>2] = $150;
  _print2log(5016,$vararg_buffer7);
  $151 = $low_contrast_offset;
  $152 = $12;
  $153 = (($152) + 12|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = $7;
  $156 = $8;
  $157 = $9;
  $158 = $12;
  $159 = (_low_contrast_block($151,$154,$155,$156,$157,$158)|0);
  $ret = $159;
  $160 = ($159|0)!=(0);
  if ($160) {
   $161 = $ret;
   $162 = ($161|0)<(0);
   if ($162) {
    label = 27;
    break;
   }
   _print2log(5048,$vararg_buffer11);
   $174 = $bi;
   $175 = $low_contrast_map;
   $176 = (($175) + ($174<<2)|0);
   HEAP32[$176>>2] = 1;
  } else {
   _print2log(5064,$vararg_buffer13);
   $177 = HEAP32[$powers>>2]|0;
   $178 = $7;
   $179 = $low_contrast_offset;
   $180 = $8;
   $181 = $9;
   $182 = $10;
   $183 = $11;
   $184 = (_dft_dir_powers($177,$178,$179,$180,$181,$182,$183)|0);
   $ret = $184;
   $185 = ($184|0)!=(0);
   if ($185) {
    label = 30;
    break;
   }
   $197 = HEAP32[$wis>>2]|0;
   $198 = HEAP32[$powmaxs>>2]|0;
   $199 = HEAP32[$powmax_dirs>>2]|0;
   $200 = HEAP32[$pownorms>>2]|0;
   $201 = HEAP32[$powers>>2]|0;
   $202 = $10;
   $203 = HEAP32[$202>>2]|0;
   $204 = $11;
   $205 = (($204) + 16|0);
   $206 = HEAP32[$205>>2]|0;
   $207 = (_dft_power_stats($197,$198,$199,$200,$201,1,$203,$206)|0);
   $ret = $207;
   $208 = ($207|0)!=(0);
   if ($208) {
    label = 32;
    break;
   }
   $220 = HEAP32[$powers>>2]|0;
   $221 = HEAP32[$wis>>2]|0;
   $222 = HEAP32[$powmaxs>>2]|0;
   $223 = HEAP32[$powmax_dirs>>2]|0;
   $224 = HEAP32[$pownorms>>2]|0;
   $225 = $nstats;
   $226 = $12;
   $227 = (_primary_dir_test($220,$221,$222,$223,$224,$225,$226)|0);
   $blkdir = $227;
   $228 = $blkdir;
   $229 = ($228|0)!=(-1);
   if ($229) {
    $230 = $blkdir;
    $231 = $bi;
    $232 = $direction_map;
    $233 = (($232) + ($231<<2)|0);
    HEAP32[$233>>2] = $230;
   } else {
    $234 = HEAP32[$powers>>2]|0;
    $235 = HEAP32[$wis>>2]|0;
    $236 = HEAP32[$powmaxs>>2]|0;
    $237 = HEAP32[$powmax_dirs>>2]|0;
    $238 = HEAP32[$pownorms>>2]|0;
    $239 = $nstats;
    $240 = $12;
    $241 = (_secondary_fork_test($234,$235,$236,$237,$238,$239,$240)|0);
    $blkdir = $241;
    $242 = $blkdir;
    $243 = ($242|0)!=(-1);
    if ($243) {
     $244 = $blkdir;
     $245 = $bi;
     $246 = $direction_map;
     $247 = (($246) + ($245<<2)|0);
     HEAP32[$247>>2] = $244;
    } else {
     $248 = $bi;
     $249 = $low_flow_map;
     $250 = (($249) + ($248<<2)|0);
     HEAP32[$250>>2] = 1;
    }
   }
  }
  $251 = $bi;
  $252 = (($251) + 1)|0;
  $bi = $252;
 }
 if ((label|0) == 27) {
  $163 = $direction_map;
  _free($163);
  $164 = $low_contrast_map;
  _free($164);
  $165 = $low_flow_map;
  _free($165);
  $166 = HEAP32[$powers>>2]|0;
  $167 = $10;
  $168 = HEAP32[$167>>2]|0;
  _free_dir_powers($166,$168);
  $169 = HEAP32[$wis>>2]|0;
  _free($169);
  $170 = HEAP32[$powmaxs>>2]|0;
  _free($170);
  $171 = HEAP32[$powmax_dirs>>2]|0;
  _free($171);
  $172 = HEAP32[$pownorms>>2]|0;
  _free($172);
  $173 = $ret;
  $0 = $173;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 else if ((label|0) == 30) {
  $186 = $direction_map;
  _free($186);
  $187 = $low_contrast_map;
  _free($187);
  $188 = $low_flow_map;
  _free($188);
  $189 = HEAP32[$powers>>2]|0;
  $190 = $10;
  $191 = HEAP32[$190>>2]|0;
  _free_dir_powers($189,$191);
  $192 = HEAP32[$wis>>2]|0;
  _free($192);
  $193 = HEAP32[$powmaxs>>2]|0;
  _free($193);
  $194 = HEAP32[$powmax_dirs>>2]|0;
  _free($194);
  $195 = HEAP32[$pownorms>>2]|0;
  _free($195);
  $196 = $ret;
  $0 = $196;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 else if ((label|0) == 32) {
  $209 = $direction_map;
  _free($209);
  $210 = $low_contrast_map;
  _free($210);
  $211 = $low_flow_map;
  _free($211);
  $212 = HEAP32[$powers>>2]|0;
  $213 = $10;
  $214 = HEAP32[$213>>2]|0;
  _free_dir_powers($212,$214);
  $215 = HEAP32[$wis>>2]|0;
  _free($215);
  $216 = HEAP32[$powmaxs>>2]|0;
  _free($216);
  $217 = HEAP32[$powmax_dirs>>2]|0;
  _free($217);
  $218 = HEAP32[$pownorms>>2]|0;
  _free($218);
  $219 = $ret;
  $0 = $219;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 else if ((label|0) == 42) {
  $253 = HEAP32[$powers>>2]|0;
  $254 = $10;
  $255 = HEAP32[$254>>2]|0;
  _free_dir_powers($253,$255);
  $256 = HEAP32[$wis>>2]|0;
  _free($256);
  $257 = HEAP32[$powmaxs>>2]|0;
  _free($257);
  $258 = HEAP32[$powmax_dirs>>2]|0;
  _free($258);
  $259 = HEAP32[$pownorms>>2]|0;
  _free($259);
  $260 = $direction_map;
  $261 = $1;
  HEAP32[$261>>2] = $260;
  $262 = $low_contrast_map;
  $263 = $2;
  HEAP32[$263>>2] = $262;
  $264 = $low_flow_map;
  $265 = $3;
  HEAP32[$265>>2] = $264;
  $0 = 0;
  $266 = $0;
  STACKTOP = sp;return ($266|0);
 }
 return 0|0;
}
function _morph_TF_map($tfmap,$mw,$mh,$lfsparms) {
 $tfmap = $tfmap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $cimage = 0, $cptr = 0, $i = 0, $mimage = 0, $mptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $tfmap;
 $2 = $mw;
 $3 = $mh;
 $4 = $lfsparms;
 $5 = $2;
 $6 = $3;
 $7 = Math_imul($5, $6)|0;
 $8 = (_malloc($7)|0);
 $cimage = $8;
 $9 = $cimage;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(5208|0),($vararg_buffer|0))|0);
  $0 = -660;
  $67 = $0;
  STACKTOP = sp;return ($67|0);
 }
 $12 = $2;
 $13 = $3;
 $14 = Math_imul($12, $13)|0;
 $15 = (_malloc($14)|0);
 $mimage = $15;
 $16 = $mimage;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(5248|0),($vararg_buffer1|0))|0);
  $0 = -661;
  $67 = $0;
  STACKTOP = sp;return ($67|0);
 }
 $19 = $cimage;
 $cptr = $19;
 $20 = $1;
 $mptr = $20;
 $i = 0;
 while(1) {
  $21 = $i;
  $22 = $2;
  $23 = $3;
  $24 = Math_imul($22, $23)|0;
  $25 = ($21|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = $mptr;
  $27 = (($26) + 4|0);
  $mptr = $27;
  $28 = HEAP32[$26>>2]|0;
  $29 = $28&255;
  $30 = $cptr;
  $31 = (($30) + 1|0);
  $cptr = $31;
  HEAP8[$30>>0] = $29;
  $32 = $i;
  $33 = (($32) + 1)|0;
  $i = $33;
 }
 $34 = $cimage;
 $35 = $mimage;
 $36 = $2;
 $37 = $3;
 _dilate_charimage_2($34,$35,$36,$37);
 $38 = $mimage;
 $39 = $cimage;
 $40 = $2;
 $41 = $3;
 _dilate_charimage_2($38,$39,$40,$41);
 $42 = $cimage;
 $43 = $mimage;
 $44 = $2;
 $45 = $3;
 _erode_charimage_2($42,$43,$44,$45);
 $46 = $mimage;
 $47 = $cimage;
 $48 = $2;
 $49 = $3;
 _erode_charimage_2($46,$47,$48,$49);
 $50 = $cimage;
 $cptr = $50;
 $51 = $1;
 $mptr = $51;
 $i = 0;
 while(1) {
  $52 = $i;
  $53 = $2;
  $54 = $3;
  $55 = Math_imul($53, $54)|0;
  $56 = ($52|0)<($55|0);
  if (!($56)) {
   break;
  }
  $57 = $cptr;
  $58 = (($57) + 1|0);
  $cptr = $58;
  $59 = HEAP8[$57>>0]|0;
  $60 = $59&255;
  $61 = $mptr;
  $62 = (($61) + 4|0);
  $mptr = $62;
  HEAP32[$61>>2] = $60;
  $63 = $i;
  $64 = (($63) + 1)|0;
  $i = $64;
 }
 $65 = $cimage;
 _free($65);
 $66 = $mimage;
 _free($66);
 $0 = 0;
 $67 = $0;
 STACKTOP = sp;return ($67|0);
}
function _remove_incon_dirs($imap,$mw,$mh,$dir2rad,$lfsparms) {
 $imap = $imap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bbox = 0, $cx = 0;
 var $cy = 0, $iptr = 0, $lbox = 0, $nremoved = 0, $rbox = 0, $tbox = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $imap;
 $1 = $mw;
 $2 = $mh;
 $3 = $dir2rad;
 $4 = $lfsparms;
 $5 = $1;
 $6 = $5 >> 1;
 $cx = $6;
 $7 = $2;
 $8 = $7 >> 1;
 $cy = $8;
 while(1) {
  $nremoved = 0;
  $9 = $0;
  $10 = $cy;
  $11 = $1;
  $12 = Math_imul($10, $11)|0;
  $13 = (($9) + ($12<<2)|0);
  $14 = $cx;
  $15 = (($13) + ($14<<2)|0);
  $iptr = $15;
  $16 = $iptr;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(-1);
  if ($18) {
   $19 = $0;
   $20 = $cx;
   $21 = $cy;
   $22 = $1;
   $23 = $2;
   $24 = $3;
   $25 = $4;
   $26 = (_remove_dir($19,$20,$21,$22,$23,$24,$25)|0);
   $27 = ($26|0)!=(0);
   if ($27) {
    $28 = $iptr;
    HEAP32[$28>>2] = -1;
    $29 = $nremoved;
    $30 = (($29) + 1)|0;
    $nremoved = $30;
   }
  }
  $31 = $cx;
  $32 = (($31) - 1)|0;
  $lbox = $32;
  $33 = $cy;
  $34 = (($33) - 1)|0;
  $tbox = $34;
  $35 = $cx;
  $36 = (($35) + 1)|0;
  $rbox = $36;
  $37 = $cy;
  $38 = (($37) + 1)|0;
  $bbox = $38;
  while(1) {
   $39 = $lbox;
   $40 = ($39|0)>=(0);
   if ($40) {
    $117 = 1;
   } else {
    $41 = $rbox;
    $42 = $1;
    $43 = ($41|0)<($42|0);
    if ($43) {
     $117 = 1;
    } else {
     $44 = $tbox;
     $45 = ($44|0)>=(0);
     if ($45) {
      $117 = 1;
     } else {
      $46 = $bbox;
      $47 = $2;
      $48 = ($46|0)<($47|0);
      $117 = $48;
     }
    }
   }
   if (!($117)) {
    break;
   }
   $49 = $tbox;
   $50 = ($49|0)>=(0);
   if ($50) {
    $51 = $lbox;
    $52 = $tbox;
    $53 = $rbox;
    $54 = $bbox;
    $55 = $0;
    $56 = $1;
    $57 = $2;
    $58 = $3;
    $59 = $4;
    $60 = (_test_top_edge($51,$52,$53,$54,$55,$56,$57,$58,$59)|0);
    $61 = $nremoved;
    $62 = (($61) + ($60))|0;
    $nremoved = $62;
   }
   $63 = $rbox;
   $64 = $1;
   $65 = ($63|0)<($64|0);
   if ($65) {
    $66 = $lbox;
    $67 = $tbox;
    $68 = $rbox;
    $69 = $bbox;
    $70 = $0;
    $71 = $1;
    $72 = $2;
    $73 = $3;
    $74 = $4;
    $75 = (_test_right_edge($66,$67,$68,$69,$70,$71,$72,$73,$74)|0);
    $76 = $nremoved;
    $77 = (($76) + ($75))|0;
    $nremoved = $77;
   }
   $78 = $bbox;
   $79 = $2;
   $80 = ($78|0)<($79|0);
   if ($80) {
    $81 = $lbox;
    $82 = $tbox;
    $83 = $rbox;
    $84 = $bbox;
    $85 = $0;
    $86 = $1;
    $87 = $2;
    $88 = $3;
    $89 = $4;
    $90 = (_test_bottom_edge($81,$82,$83,$84,$85,$86,$87,$88,$89)|0);
    $91 = $nremoved;
    $92 = (($91) + ($90))|0;
    $nremoved = $92;
   }
   $93 = $lbox;
   $94 = ($93|0)>=(0);
   if ($94) {
    $95 = $lbox;
    $96 = $tbox;
    $97 = $rbox;
    $98 = $bbox;
    $99 = $0;
    $100 = $1;
    $101 = $2;
    $102 = $3;
    $103 = $4;
    $104 = (_test_left_edge($95,$96,$97,$98,$99,$100,$101,$102,$103)|0);
    $105 = $nremoved;
    $106 = (($105) + ($104))|0;
    $nremoved = $106;
   }
   $107 = $lbox;
   $108 = (($107) + -1)|0;
   $lbox = $108;
   $109 = $tbox;
   $110 = (($109) + -1)|0;
   $tbox = $110;
   $111 = $rbox;
   $112 = (($111) + 1)|0;
   $rbox = $112;
   $113 = $bbox;
   $114 = (($113) + 1)|0;
   $bbox = $114;
  }
  $115 = $nremoved;
  $116 = ($115|0)!=(0);
  if (!($116)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _smooth_direction_map($direction_map,$low_contrast_map,$mw,$mh,$dir2rad,$lfsparms) {
 $direction_map = $direction_map|0;
 $low_contrast_map = $low_contrast_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $avrdir = 0, $cptr = 0, $dir_strength = 0, $dptr = 0, $mx = 0, $my = 0, $nvalid = 0;
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $avrdir = sp + 16|0;
 $nvalid = sp + 12|0;
 $dir_strength = sp;
 $0 = $direction_map;
 $1 = $low_contrast_map;
 $2 = $mw;
 $3 = $mh;
 $4 = $dir2rad;
 $5 = $lfsparms;
 _print2log(5384,$vararg_buffer);
 $6 = $0;
 $dptr = $6;
 $7 = $1;
 $cptr = $7;
 $my = 0;
 while(1) {
  $8 = $my;
  $9 = $3;
  $10 = ($8|0)<($9|0);
  if (!($10)) {
   break;
  }
  $mx = 0;
  while(1) {
   $11 = $mx;
   $12 = $2;
   $13 = ($11|0)<($12|0);
   if (!($13)) {
    break;
   }
   $14 = $cptr;
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $17 = $0;
    $18 = $mx;
    $19 = $my;
    $20 = $2;
    $21 = $3;
    $22 = $4;
    _average_8nbr_dir($avrdir,$dir_strength,$nvalid,$17,$18,$19,$20,$21,$22);
    $23 = +HEAPF64[$dir_strength>>3];
    $24 = $5;
    $25 = (($24) + 40|0);
    $26 = +HEAPF64[$25>>3];
    $27 = $23 >= $26;
    if ($27) {
     $28 = $dptr;
     $29 = HEAP32[$28>>2]|0;
     $30 = ($29|0)!=(-1);
     if ($30) {
      $31 = HEAP32[$nvalid>>2]|0;
      $32 = $5;
      $33 = (($32) + 32|0);
      $34 = HEAP32[$33>>2]|0;
      $35 = ($31|0)>=($34|0);
      if ($35) {
       $36 = HEAP32[$avrdir>>2]|0;
       $37 = $dptr;
       HEAP32[$37>>2] = $36;
      }
     } else {
      $38 = HEAP32[$nvalid>>2]|0;
      $39 = $5;
      $40 = (($39) + 52|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = ($38|0)>=($41|0);
      if ($42) {
       $43 = HEAP32[$avrdir>>2]|0;
       $44 = $dptr;
       HEAP32[$44>>2] = $43;
      }
     }
    }
   }
   $45 = $dptr;
   $46 = (($45) + 4|0);
   $dptr = $46;
   $47 = $cptr;
   $48 = (($47) + 4|0);
   $cptr = $48;
   $49 = $mx;
   $50 = (($49) + 1)|0;
   $mx = $50;
  }
  $51 = $my;
  $52 = (($51) + 1)|0;
  $my = $52;
 }
 STACKTOP = sp;return;
}
function _interpolate_direction_map($direction_map,$low_contrast_map,$mw,$mh,$lfsparms) {
 $direction_map = $direction_map|0;
 $low_contrast_map = $low_contrast_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0, $135 = 0.0, $136 = 0, $137 = 0.0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0, $145 = 0.0, $146 = 0, $147 = 0.0, $148 = 0, $149 = 0.0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0.0, $153 = 0.0, $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0, $159 = 0.0, $16 = 0, $160 = 0, $161 = 0.0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0.0, $172 = 0, $173 = 0.0, $174 = 0.0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0.0, $179 = 0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0, $184 = 0.0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0.0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0, $195 = 0.0, $196 = 0.0, $197 = 0.0, $198 = 0.0, $199 = 0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $avr_dir = 0.0, $cptr = 0, $dptr = 0, $e_delta = 0, $e_dir = 0, $e_dist = 0, $e_found = 0, $n_delta = 0, $n_dir = 0;
 var $n_dist = 0, $n_found = 0, $nbr_x = 0, $nbr_y = 0, $new_dir = 0, $omap = 0, $optr = 0, $s_delta = 0, $s_dir = 0, $s_dist = 0, $s_found = 0, $total_delta = 0, $total_dist = 0, $total_found = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0;
 var $w_delta = 0, $w_dir = 0, $w_dist = 0, $w_found = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $n_dir = sp + 136|0;
 $e_dir = sp + 132|0;
 $s_dir = sp + 128|0;
 $w_dir = sp + 124|0;
 $nbr_x = sp + 60|0;
 $nbr_y = sp + 56|0;
 $1 = $direction_map;
 $2 = $low_contrast_map;
 $3 = $mw;
 $4 = $mh;
 $5 = $lfsparms;
 $n_dist = 0;
 $e_dist = 0;
 $s_dist = 0;
 $w_dist = 0;
 $n_delta = 0;
 $e_delta = 0;
 $s_delta = 0;
 $w_delta = 0;
 _print2log(5072,$vararg_buffer);
 $6 = $3;
 $7 = $4;
 $8 = Math_imul($6, $7)|0;
 $9 = $8<<2;
 $10 = (_malloc($9)|0);
 $omap = $10;
 $11 = $omap;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  (_fprintf(($13|0),(5104|0),($vararg_buffer1|0))|0);
  $0 = -520;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 $14 = $1;
 $dptr = $14;
 $15 = $2;
 $cptr = $15;
 $16 = $omap;
 $optr = $16;
 $y = 0;
 while(1) {
  $17 = $y;
  $18 = $4;
  $19 = ($17|0)<($18|0);
  if (!($19)) {
   break;
  }
  $x = 0;
  while(1) {
   $20 = $x;
   $21 = $3;
   $22 = ($20|0)<($21|0);
   if (!($22)) {
    break;
   }
   $23 = $cptr;
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0);
   if ($25) {
    label = 43;
   } else {
    $26 = $dptr;
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27|0)==(-1);
    if ($28) {
     $total_found = 0;
     $total_dist = 0;
     $29 = $1;
     $30 = $2;
     $31 = $x;
     $32 = $y;
     $33 = $3;
     $34 = $4;
     $35 = (_find_valid_block($n_dir,$nbr_x,$nbr_y,$29,$30,$31,$32,$33,$34,0,-1)|0);
     $n_found = $35;
     $36 = ($35|0)==(1);
     if ($36) {
      $37 = $y;
      $38 = HEAP32[$nbr_y>>2]|0;
      $39 = (($37) - ($38))|0;
      $n_dist = $39;
      $40 = $n_dist;
      $41 = $total_dist;
      $42 = (($41) + ($40))|0;
      $total_dist = $42;
      $43 = $total_found;
      $44 = (($43) + 1)|0;
      $total_found = $44;
     }
     $45 = $1;
     $46 = $2;
     $47 = $x;
     $48 = $y;
     $49 = $3;
     $50 = $4;
     $51 = (_find_valid_block($e_dir,$nbr_x,$nbr_y,$45,$46,$47,$48,$49,$50,1,0)|0);
     $e_found = $51;
     $52 = ($51|0)==(1);
     if ($52) {
      $53 = HEAP32[$nbr_x>>2]|0;
      $54 = $x;
      $55 = (($53) - ($54))|0;
      $e_dist = $55;
      $56 = $e_dist;
      $57 = $total_dist;
      $58 = (($57) + ($56))|0;
      $total_dist = $58;
      $59 = $total_found;
      $60 = (($59) + 1)|0;
      $total_found = $60;
     }
     $61 = $1;
     $62 = $2;
     $63 = $x;
     $64 = $y;
     $65 = $3;
     $66 = $4;
     $67 = (_find_valid_block($s_dir,$nbr_x,$nbr_y,$61,$62,$63,$64,$65,$66,0,1)|0);
     $s_found = $67;
     $68 = ($67|0)==(1);
     if ($68) {
      $69 = HEAP32[$nbr_y>>2]|0;
      $70 = $y;
      $71 = (($69) - ($70))|0;
      $s_dist = $71;
      $72 = $s_dist;
      $73 = $total_dist;
      $74 = (($73) + ($72))|0;
      $total_dist = $74;
      $75 = $total_found;
      $76 = (($75) + 1)|0;
      $total_found = $76;
     }
     $77 = $1;
     $78 = $2;
     $79 = $x;
     $80 = $y;
     $81 = $3;
     $82 = $4;
     $83 = (_find_valid_block($w_dir,$nbr_x,$nbr_y,$77,$78,$79,$80,$81,$82,-1,0)|0);
     $w_found = $83;
     $84 = ($83|0)==(1);
     if ($84) {
      $85 = $x;
      $86 = HEAP32[$nbr_x>>2]|0;
      $87 = (($85) - ($86))|0;
      $w_dist = $87;
      $88 = $w_dist;
      $89 = $total_dist;
      $90 = (($89) + ($88))|0;
      $total_dist = $90;
      $91 = $total_found;
      $92 = (($91) + 1)|0;
      $total_found = $92;
     }
     $93 = $total_found;
     $94 = $5;
     $95 = (($94) + 68|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = ($93|0)>=($96|0);
     if ($97) {
      $total_delta = 0;
      $98 = $n_found;
      $99 = ($98|0)!=(0);
      if ($99) {
       $100 = $total_dist;
       $101 = $n_dist;
       $102 = (($100) - ($101))|0;
       $n_delta = $102;
       $103 = $n_delta;
       $104 = $total_delta;
       $105 = (($104) + ($103))|0;
       $total_delta = $105;
      }
      $106 = $e_found;
      $107 = ($106|0)!=(0);
      if ($107) {
       $108 = $total_dist;
       $109 = $e_dist;
       $110 = (($108) - ($109))|0;
       $e_delta = $110;
       $111 = $e_delta;
       $112 = $total_delta;
       $113 = (($112) + ($111))|0;
       $total_delta = $113;
      }
      $114 = $s_found;
      $115 = ($114|0)!=(0);
      if ($115) {
       $116 = $total_dist;
       $117 = $s_dist;
       $118 = (($116) - ($117))|0;
       $s_delta = $118;
       $119 = $s_delta;
       $120 = $total_delta;
       $121 = (($120) + ($119))|0;
       $total_delta = $121;
      }
      $122 = $w_found;
      $123 = ($122|0)!=(0);
      if ($123) {
       $124 = $total_dist;
       $125 = $w_dist;
       $126 = (($124) - ($125))|0;
       $w_delta = $126;
       $127 = $w_delta;
       $128 = $total_delta;
       $129 = (($128) + ($127))|0;
       $total_delta = $129;
      }
      $avr_dir = 0.0;
      $130 = $n_found;
      $131 = ($130|0)!=(0);
      if ($131) {
       $132 = HEAP32[$n_dir>>2]|0;
       $133 = (+($132|0));
       $134 = $n_delta;
       $135 = (+($134|0));
       $136 = $total_delta;
       $137 = (+($136|0));
       $138 = $135 / $137;
       $139 = $133 * $138;
       $140 = $avr_dir;
       $141 = $140 + $139;
       $avr_dir = $141;
      }
      $142 = $e_found;
      $143 = ($142|0)!=(0);
      if ($143) {
       $144 = HEAP32[$e_dir>>2]|0;
       $145 = (+($144|0));
       $146 = $e_delta;
       $147 = (+($146|0));
       $148 = $total_delta;
       $149 = (+($148|0));
       $150 = $147 / $149;
       $151 = $145 * $150;
       $152 = $avr_dir;
       $153 = $152 + $151;
       $avr_dir = $153;
      }
      $154 = $s_found;
      $155 = ($154|0)!=(0);
      if ($155) {
       $156 = HEAP32[$s_dir>>2]|0;
       $157 = (+($156|0));
       $158 = $s_delta;
       $159 = (+($158|0));
       $160 = $total_delta;
       $161 = (+($160|0));
       $162 = $159 / $161;
       $163 = $157 * $162;
       $164 = $avr_dir;
       $165 = $164 + $163;
       $avr_dir = $165;
      }
      $166 = $w_found;
      $167 = ($166|0)!=(0);
      if ($167) {
       $168 = HEAP32[$w_dir>>2]|0;
       $169 = (+($168|0));
       $170 = $w_delta;
       $171 = (+($170|0));
       $172 = $total_delta;
       $173 = (+($172|0));
       $174 = $171 / $173;
       $175 = $169 * $174;
       $176 = $avr_dir;
       $177 = $176 + $175;
       $avr_dir = $177;
      }
      $178 = $avr_dir;
      $179 = $178 < 0.0;
      if ($179) {
       $180 = $avr_dir;
       $181 = $180 * 16384.0;
       $182 = $181 - 0.5;
       $183 = (~~(($182)));
       $184 = (+($183|0));
       $185 = $184 / 16384.0;
       $192 = $185;
      } else {
       $186 = $avr_dir;
       $187 = $186 * 16384.0;
       $188 = $187 + 0.5;
       $189 = (~~(($188)));
       $190 = (+($189|0));
       $191 = $190 / 16384.0;
       $192 = $191;
      }
      $avr_dir = $192;
      $193 = $avr_dir;
      $194 = $193 < 0.0;
      if ($194) {
       $195 = $avr_dir;
       $196 = $195 - 0.5;
       $200 = $196;
      } else {
       $197 = $avr_dir;
       $198 = $197 + 0.5;
       $200 = $198;
      }
      $199 = (~~(($200)));
      $new_dir = $199;
      $201 = $x;
      $202 = $y;
      $203 = $total_found;
      $204 = $new_dir;
      HEAP32[$vararg_buffer3>>2] = $201;
      $vararg_ptr5 = (($vararg_buffer3) + 4|0);
      HEAP32[$vararg_ptr5>>2] = $202;
      $vararg_ptr6 = (($vararg_buffer3) + 8|0);
      HEAP32[$vararg_ptr6>>2] = $203;
      $vararg_ptr7 = (($vararg_buffer3) + 12|0);
      HEAP32[$vararg_ptr7>>2] = $204;
      _print2log(5160,$vararg_buffer3);
      $205 = $new_dir;
      $206 = $optr;
      HEAP32[$206>>2] = $205;
     } else {
      $207 = $dptr;
      $208 = HEAP32[$207>>2]|0;
      $209 = $optr;
      HEAP32[$209>>2] = $208;
     }
    } else {
     label = 43;
    }
   }
   if ((label|0) == 43) {
    label = 0;
    $210 = $dptr;
    $211 = HEAP32[$210>>2]|0;
    $212 = $optr;
    HEAP32[$212>>2] = $211;
   }
   $213 = $dptr;
   $214 = (($213) + 4|0);
   $dptr = $214;
   $215 = $cptr;
   $216 = (($215) + 4|0);
   $cptr = $216;
   $217 = $optr;
   $218 = (($217) + 4|0);
   $optr = $218;
   $219 = $x;
   $220 = (($219) + 1)|0;
   $x = $220;
  }
  $221 = $y;
  $222 = (($221) + 1)|0;
  $y = $222;
 }
 $223 = $1;
 $224 = $omap;
 $225 = $3;
 $226 = $4;
 $227 = Math_imul($225, $226)|0;
 $228 = $227<<2;
 _memcpy(($223|0),($224|0),($228|0))|0;
 $229 = $omap;
 _free($229);
 $0 = 0;
 $230 = $0;
 STACKTOP = sp;return ($230|0);
}
function _gen_high_curve_map($ohcmap,$direction_map,$mw,$mh,$lfsparms) {
 $ohcmap = $ohcmap|0;
 $direction_map = $direction_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $bx = 0, $by = 0, $cmeasure = 0, $dptr = 0, $high_curve_map = 0, $hptr = 0, $mapsize = 0, $nvalid = 0, $vararg_buffer = 0, $vmeasure = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $1 = $ohcmap;
 $2 = $direction_map;
 $3 = $mw;
 $4 = $mh;
 $5 = $lfsparms;
 $6 = $3;
 $7 = $4;
 $8 = Math_imul($6, $7)|0;
 $mapsize = $8;
 $9 = $mapsize;
 $10 = $9<<2;
 $11 = (_malloc($10)|0);
 $high_curve_map = $11;
 $12 = $high_curve_map;
 $13 = ($12|0)==(0|0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  (_fprintf(($14|0),(5408|0),($vararg_buffer|0))|0);
  $0 = -530;
  $82 = $0;
  STACKTOP = sp;return ($82|0);
 }
 $15 = $high_curve_map;
 $16 = $mapsize;
 $17 = $16<<2;
 _memset(($15|0),0,($17|0))|0;
 $18 = $high_curve_map;
 $hptr = $18;
 $19 = $2;
 $dptr = $19;
 $by = 0;
 while(1) {
  $20 = $by;
  $21 = $4;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $bx = 0;
  while(1) {
   $23 = $bx;
   $24 = $3;
   $25 = ($23|0)<($24|0);
   if (!($25)) {
    break;
   }
   $26 = $2;
   $27 = $bx;
   $28 = $by;
   $29 = $3;
   $30 = $4;
   $31 = (_num_valid_8nbrs($26,$27,$28,$29,$30)|0);
   $nvalid = $31;
   $32 = $nvalid;
   $33 = ($32|0)>(0);
   if ($33) {
    $34 = $dptr;
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)==(-1);
    if ($36) {
     $37 = $nvalid;
     $38 = $5;
     $39 = (($38) + 56|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($37|0)>=($40|0);
     if ($41) {
      $42 = $2;
      $43 = $bx;
      $44 = $by;
      $45 = $3;
      $46 = $4;
      $47 = $5;
      $48 = (($47) + 20|0);
      $49 = HEAP32[$48>>2]|0;
      $50 = (_vorticity($42,$43,$44,$45,$46,$49)|0);
      $vmeasure = $50;
      $51 = $vmeasure;
      $52 = $5;
      $53 = (($52) + 60|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($51|0)>=($54|0);
      if ($55) {
       $56 = $hptr;
       HEAP32[$56>>2] = 1;
      }
     }
    } else {
     $57 = $2;
     $58 = $bx;
     $59 = $by;
     $60 = $3;
     $61 = $4;
     $62 = $5;
     $63 = (($62) + 20|0);
     $64 = HEAP32[$63>>2]|0;
     $65 = (_curvature($57,$58,$59,$60,$61,$64)|0);
     $cmeasure = $65;
     $66 = $cmeasure;
     $67 = $5;
     $68 = (($67) + 64|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ($66|0)>=($69|0);
     if ($70) {
      $71 = $hptr;
      HEAP32[$71>>2] = 1;
     }
    }
   }
   $72 = $dptr;
   $73 = (($72) + 4|0);
   $dptr = $73;
   $74 = $hptr;
   $75 = (($74) + 4|0);
   $hptr = $75;
   $76 = $bx;
   $77 = (($76) + 1)|0;
   $bx = $77;
  }
  $78 = $by;
  $79 = (($78) + 1)|0;
  $by = $79;
 }
 $80 = $high_curve_map;
 $81 = $1;
 HEAP32[$81>>2] = $80;
 $0 = 0;
 $82 = $0;
 STACKTOP = sp;return ($82|0);
}
function _primary_dir_test($powers,$wis,$powmaxs,$powmax_dirs,$pownorms,$nstats,$lfsparms) {
 $powers = $powers|0;
 $wis = $wis|0;
 $powmaxs = $powmaxs|0;
 $powmax_dirs = $powmax_dirs|0;
 $pownorms = $pownorms|0;
 $nstats = $nstats|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0.0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $w = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $1 = $powers;
 $2 = $wis;
 $3 = $powmaxs;
 $4 = $powmax_dirs;
 $5 = $pownorms;
 $6 = $nstats;
 $7 = $lfsparms;
 _print2log(5464,$vararg_buffer);
 $w = 0;
 while(1) {
  $8 = $w;
  $9 = $6;
  $10 = ($8|0)<($9|0);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = $w;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $3;
  $16 = (($15) + ($14<<3)|0);
  $17 = +HEAPF64[$16>>3];
  $18 = $7;
  $19 = (($18) + 88|0);
  $20 = +HEAPF64[$19>>3];
  $21 = $17 > $20;
  if ($21) {
   $22 = $w;
   $23 = $2;
   $24 = (($23) + ($22<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $5;
   $27 = (($26) + ($25<<3)|0);
   $28 = +HEAPF64[$27>>3];
   $29 = $7;
   $30 = (($29) + 96|0);
   $31 = +HEAPF64[$30>>3];
   $32 = $28 > $31;
   if ($32) {
    $33 = $w;
    $34 = $2;
    $35 = (($34) + ($33<<2)|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = $4;
    $38 = (($37) + ($36<<2)|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = $1;
    $41 = HEAP32[$40>>2]|0;
    $42 = (($41) + ($39<<3)|0);
    $43 = +HEAPF64[$42>>3];
    $44 = $7;
    $45 = (($44) + 104|0);
    $46 = +HEAPF64[$45>>3];
    $47 = $43 <= $46;
    if ($47) {
     label = 6;
     break;
    }
   }
  }
  $55 = $w;
  $56 = (($55) + 1)|0;
  $w = $56;
 }
 if ((label|0) == 6) {
  $48 = $w;
  $49 = $2;
  $50 = (($49) + ($48<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = $4;
  $53 = (($52) + ($51<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $0 = $54;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 9) {
  $0 = -1;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 }
 return 0|0;
}
function _secondary_fork_test($powers,$wis,$powmaxs,$powmax_dirs,$pownorms,$nstats,$lfsparms) {
 $powers = $powers|0;
 $wis = $wis|0;
 $powmaxs = $powmaxs|0;
 $powmax_dirs = $powmax_dirs|0;
 $pownorms = $pownorms|0;
 $nstats = $nstats|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0, $131 = 0.0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0.0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0.0, $86 = 0, $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0.0;
 var $fork_pow_thresh = 0.0, $fork_pownorm_min = 0.0, $ldir = 0, $rdir = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp + 16|0;
 $1 = $powers;
 $2 = $wis;
 $3 = $powmaxs;
 $4 = $powmax_dirs;
 $5 = $pownorms;
 $6 = $nstats;
 $7 = $lfsparms;
 $8 = $7;
 $9 = (($8) + 128|0);
 $10 = +HEAPF64[$9>>3];
 $11 = $7;
 $12 = (($11) + 96|0);
 $13 = +HEAPF64[$12>>3];
 $14 = $10 * $13;
 $fork_pownorm_min = $14;
 $15 = $2;
 $16 = HEAP32[$15>>2]|0;
 $17 = $3;
 $18 = (($17) + ($16<<3)|0);
 $19 = +HEAPF64[$18>>3];
 $20 = $7;
 $21 = (($20) + 88|0);
 $22 = +HEAPF64[$21>>3];
 $23 = $19 > $22;
 if ($23) {
  $24 = $2;
  $25 = HEAP32[$24>>2]|0;
  $26 = $5;
  $27 = (($26) + ($25<<3)|0);
  $28 = +HEAPF64[$27>>3];
  $29 = $fork_pownorm_min;
  $30 = $28 >= $29;
  if ($30) {
   $31 = $2;
   $32 = HEAP32[$31>>2]|0;
   $33 = $4;
   $34 = (($33) + ($32<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $1;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + ($35<<3)|0);
   $39 = +HEAPF64[$38>>3];
   $40 = $7;
   $41 = (($40) + 104|0);
   $42 = +HEAPF64[$41>>3];
   $43 = $39 <= $42;
   if ($43) {
    $44 = $2;
    $45 = HEAP32[$44>>2]|0;
    $46 = $4;
    $47 = (($46) + ($45<<2)|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = $7;
    $50 = (($49) + 112|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($48) + ($51))|0;
    $53 = $7;
    $54 = (($53) + 20|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($52|0) % ($55|0))&-1;
    $rdir = $56;
    $57 = $2;
    $58 = HEAP32[$57>>2]|0;
    $59 = $4;
    $60 = (($59) + ($58<<2)|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = $7;
    $63 = (($62) + 20|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (($61) + ($64))|0;
    $66 = $7;
    $67 = (($66) + 112|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($65) - ($68))|0;
    $70 = $7;
    $71 = (($70) + 20|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (($69|0) % ($72|0))&-1;
    $ldir = $73;
    $74 = $ldir;
    $75 = $2;
    $76 = HEAP32[$75>>2]|0;
    $77 = $4;
    $78 = (($77) + ($76<<2)|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = $rdir;
    HEAP32[$vararg_buffer>>2] = $74;
    $vararg_ptr1 = (($vararg_buffer) + 4|0);
    HEAP32[$vararg_ptr1>>2] = $79;
    $vararg_ptr2 = (($vararg_buffer) + 8|0);
    HEAP32[$vararg_ptr2>>2] = $80;
    _print2log(5480,$vararg_buffer);
    $81 = $2;
    $82 = HEAP32[$81>>2]|0;
    $83 = $3;
    $84 = (($83) + ($82<<3)|0);
    $85 = +HEAPF64[$84>>3];
    $86 = $7;
    $87 = (($86) + 120|0);
    $88 = +HEAPF64[$87>>3];
    $89 = $85 * $88;
    $fork_pow_thresh = $89;
    $90 = $ldir;
    $91 = $2;
    $92 = HEAP32[$91>>2]|0;
    $93 = (($92) + 1)|0;
    $94 = $1;
    $95 = (($94) + ($93<<2)|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = (($96) + ($90<<3)|0);
    $98 = +HEAPF64[$97>>3];
    $99 = $fork_pow_thresh;
    $100 = $98 <= $99;
    if ($100) {
     label = 6;
    } else {
     $101 = $rdir;
     $102 = $2;
     $103 = HEAP32[$102>>2]|0;
     $104 = (($103) + 1)|0;
     $105 = $1;
     $106 = (($105) + ($104<<2)|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = (($107) + ($101<<3)|0);
     $109 = +HEAPF64[$108>>3];
     $110 = $fork_pow_thresh;
     $111 = $109 <= $110;
     if ($111) {
      label = 6;
     }
    }
    do {
     if ((label|0) == 6) {
      $112 = $ldir;
      $113 = $2;
      $114 = HEAP32[$113>>2]|0;
      $115 = (($114) + 1)|0;
      $116 = $1;
      $117 = (($116) + ($115<<2)|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = (($118) + ($112<<3)|0);
      $120 = +HEAPF64[$119>>3];
      $121 = $fork_pow_thresh;
      $122 = $120 > $121;
      if (!($122)) {
       $123 = $rdir;
       $124 = $2;
       $125 = HEAP32[$124>>2]|0;
       $126 = (($125) + 1)|0;
       $127 = $1;
       $128 = (($127) + ($126<<2)|0);
       $129 = HEAP32[$128>>2]|0;
       $130 = (($129) + ($123<<3)|0);
       $131 = +HEAPF64[$130>>3];
       $132 = $fork_pow_thresh;
       $133 = $131 > $132;
       if (!($133)) {
        break;
       }
      }
      $134 = $2;
      $135 = HEAP32[$134>>2]|0;
      $136 = $4;
      $137 = (($136) + ($135<<2)|0);
      $138 = HEAP32[$137>>2]|0;
      $0 = $138;
      $139 = $0;
      STACKTOP = sp;return ($139|0);
     }
    } while(0);
   }
  }
 }
 $0 = -1;
 $139 = $0;
 STACKTOP = sp;return ($139|0);
}
function _pixelize_map($omap,$iw,$ih,$imap,$mw,$mh,$blocksize) {
 $omap = $omap|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $imap = $imap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $blocksize = $blocksize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $bh = 0, $bi = 0, $blkoffs = 0, $bw = 0, $pmap = 0, $pptr = 0, $ret = 0, $spptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $x = 0, $y = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $blkoffs = sp + 32|0;
 $bw = sp + 28|0;
 $bh = sp + 24|0;
 $1 = $omap;
 $2 = $iw;
 $3 = $ih;
 $4 = $imap;
 $5 = $mw;
 $6 = $mh;
 $7 = $blocksize;
 $8 = $2;
 $9 = $3;
 $10 = Math_imul($8, $9)|0;
 $11 = $10<<2;
 $12 = (_malloc($11)|0);
 $pmap = $12;
 $13 = $pmap;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(5288|0),($vararg_buffer|0))|0);
  $0 = -590;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 }
 $16 = $2;
 $17 = $3;
 $18 = $7;
 $19 = (_block_offsets($blkoffs,$bw,$bh,$16,$17,0,$18)|0);
 $ret = $19;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $ret;
  $0 = $21;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 }
 $22 = HEAP32[$bw>>2]|0;
 $23 = $5;
 $24 = ($22|0)!=($23|0);
 if (!($24)) {
  $25 = HEAP32[$bh>>2]|0;
  $26 = $6;
  $27 = ($25|0)!=($26|0);
  if (!($27)) {
   $bi = 0;
   while(1) {
    $30 = $bi;
    $31 = $5;
    $32 = $6;
    $33 = Math_imul($31, $32)|0;
    $34 = ($30|0)<($33|0);
    if (!($34)) {
     break;
    }
    $35 = $pmap;
    $36 = $bi;
    $37 = HEAP32[$blkoffs>>2]|0;
    $38 = (($37) + ($36<<2)|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($35) + ($39<<2)|0);
    $spptr = $40;
    $y = 0;
    while(1) {
     $41 = $y;
     $42 = $7;
     $43 = ($41|0)<($42|0);
     if (!($43)) {
      break;
     }
     $44 = $spptr;
     $pptr = $44;
     $x = 0;
     while(1) {
      $45 = $x;
      $46 = $7;
      $47 = ($45|0)<($46|0);
      if (!($47)) {
       break;
      }
      $48 = $bi;
      $49 = $4;
      $50 = (($49) + ($48<<2)|0);
      $51 = HEAP32[$50>>2]|0;
      $52 = $pptr;
      $53 = (($52) + 4|0);
      $pptr = $53;
      HEAP32[$52>>2] = $51;
      $54 = $x;
      $55 = (($54) + 1)|0;
      $x = $55;
     }
     $56 = $2;
     $57 = $spptr;
     $58 = (($57) + ($56<<2)|0);
     $spptr = $58;
     $59 = $y;
     $60 = (($59) + 1)|0;
     $y = $60;
    }
    $61 = $bi;
    $62 = (($61) + 1)|0;
    $bi = $62;
   }
   $63 = HEAP32[$blkoffs>>2]|0;
   _free($63);
   $64 = $pmap;
   $65 = $1;
   HEAP32[$65>>2] = $64;
   $0 = 0;
   $66 = $0;
   STACKTOP = sp;return ($66|0);
  }
 }
 $28 = HEAP32[$blkoffs>>2]|0;
 _free($28);
 $29 = HEAP32[_stderr>>2]|0;
 (_fprintf(($29|0),(5328|0),($vararg_buffer1|0))|0);
 $0 = -591;
 $66 = $0;
 STACKTOP = sp;return ($66|0);
}
function _average_8nbr_dir($avrdir,$dir_strength,$nvalid,$imap,$mx,$my,$mw,$mh,$dir2rad) {
 $avrdir = $avrdir|0;
 $dir_strength = $dir_strength|0;
 $nvalid = $nvalid|0;
 $imap = $imap|0;
 $mx = $mx|0;
 $my = $my|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0.0, $142 = 0.0, $143 = 0.0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0.0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0.0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0.0, $213 = 0.0, $214 = 0.0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0.0, $222 = 0.0, $223 = 0.0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0.0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0.0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0.0, $282 = 0.0, $283 = 0.0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0.0, $291 = 0.0, $292 = 0.0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0, $307 = 0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0.0, $311 = 0.0, $312 = 0.0, $313 = 0.0;
 var $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0.0, $318 = 0, $319 = 0, $32 = 0, $320 = 0.0, $321 = 0, $322 = 0, $323 = 0.0, $324 = 0.0, $325 = 0.0, $326 = 0, $327 = 0.0, $328 = 0.0, $329 = 0, $33 = 0, $330 = 0.0, $331 = 0.0;
 var $332 = 0.0, $333 = 0, $334 = 0.0, $335 = 0.0, $336 = 0, $337 = 0.0, $338 = 0, $339 = 0.0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0.0, $344 = 0.0, $345 = 0.0, $346 = 0.0, $347 = 0.0, $348 = 0.0, $349 = 0.0, $35 = 0;
 var $350 = 0.0, $351 = 0.0, $352 = 0.0, $353 = 0, $354 = 0, $355 = 0.0, $356 = 0.0, $357 = 0.0, $358 = 0.0, $359 = 0.0, $36 = 0, $360 = 0.0, $361 = 0, $362 = 0.0, $363 = 0.0, $364 = 0.0, $365 = 0, $366 = 0.0, $367 = 0.0, $368 = 0.0;
 var $369 = 0.0, $37 = 0, $370 = 0.0, $371 = 0, $372 = 0.0, $373 = 0.0, $374 = 0.0, $375 = 0.0, $376 = 0, $377 = 0.0, $378 = 0.0, $379 = 0.0, $38 = 0.0, $380 = 0.0, $381 = 0, $382 = 0.0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0;
 var $73 = 0.0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $avr = 0.0, $cospart = 0.0, $e = 0, $iptr = 0, $n = 0, $pi2 = 0.0, $pi_factor = 0.0, $s = 0, $sinpart = 0.0, $theta = 0.0, $w = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = $avrdir;
 $1 = $dir_strength;
 $2 = $nvalid;
 $3 = $imap;
 $4 = $mx;
 $5 = $my;
 $6 = $mw;
 $7 = $mh;
 $8 = $dir2rad;
 $9 = $4;
 $10 = (($9) + 1)|0;
 $e = $10;
 $11 = $4;
 $12 = (($11) - 1)|0;
 $w = $12;
 $13 = $5;
 $14 = (($13) - 1)|0;
 $n = $14;
 $15 = $5;
 $16 = (($15) + 1)|0;
 $s = $16;
 $17 = $2;
 HEAP32[$17>>2] = 0;
 $cospart = 0.0;
 $sinpart = 0.0;
 $18 = $w;
 $19 = ($18|0)>=(0);
 if ($19) {
  $20 = $n;
  $21 = ($20|0)>=(0);
  if ($21) {
   $22 = $3;
   $23 = $n;
   $24 = $6;
   $25 = Math_imul($23, $24)|0;
   $26 = (($22) + ($25<<2)|0);
   $27 = $w;
   $28 = (($26) + ($27<<2)|0);
   $iptr = $28;
   $29 = $iptr;
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(-1);
   if ($31) {
    $32 = $iptr;
    $33 = HEAP32[$32>>2]|0;
    $34 = $8;
    $35 = (($34) + 4|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = (($36) + ($33<<3)|0);
    $38 = +HEAPF64[$37>>3];
    $39 = $cospart;
    $40 = $39 + $38;
    $cospart = $40;
    $41 = $iptr;
    $42 = HEAP32[$41>>2]|0;
    $43 = $8;
    $44 = (($43) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (($45) + ($42<<3)|0);
    $47 = +HEAPF64[$46>>3];
    $48 = $sinpart;
    $49 = $48 + $47;
    $sinpart = $49;
    $50 = $2;
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + 1)|0;
    HEAP32[$50>>2] = $52;
   }
  }
 }
 $53 = $n;
 $54 = ($53|0)>=(0);
 if ($54) {
  $55 = $3;
  $56 = $n;
  $57 = $6;
  $58 = Math_imul($56, $57)|0;
  $59 = (($55) + ($58<<2)|0);
  $60 = $4;
  $61 = (($59) + ($60<<2)|0);
  $iptr = $61;
  $62 = $iptr;
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)!=(-1);
  if ($64) {
   $65 = $iptr;
   $66 = HEAP32[$65>>2]|0;
   $67 = $8;
   $68 = (($67) + 4|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = (($69) + ($66<<3)|0);
   $71 = +HEAPF64[$70>>3];
   $72 = $cospart;
   $73 = $72 + $71;
   $cospart = $73;
   $74 = $iptr;
   $75 = HEAP32[$74>>2]|0;
   $76 = $8;
   $77 = (($76) + 8|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = (($78) + ($75<<3)|0);
   $80 = +HEAPF64[$79>>3];
   $81 = $sinpart;
   $82 = $81 + $80;
   $sinpart = $82;
   $83 = $2;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + 1)|0;
   HEAP32[$83>>2] = $85;
  }
 }
 $86 = $e;
 $87 = $6;
 $88 = ($86|0)<($87|0);
 if ($88) {
  $89 = $n;
  $90 = ($89|0)>=(0);
  if ($90) {
   $91 = $3;
   $92 = $n;
   $93 = $6;
   $94 = Math_imul($92, $93)|0;
   $95 = (($91) + ($94<<2)|0);
   $96 = $e;
   $97 = (($95) + ($96<<2)|0);
   $iptr = $97;
   $98 = $iptr;
   $99 = HEAP32[$98>>2]|0;
   $100 = ($99|0)!=(-1);
   if ($100) {
    $101 = $iptr;
    $102 = HEAP32[$101>>2]|0;
    $103 = $8;
    $104 = (($103) + 4|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = (($105) + ($102<<3)|0);
    $107 = +HEAPF64[$106>>3];
    $108 = $cospart;
    $109 = $108 + $107;
    $cospart = $109;
    $110 = $iptr;
    $111 = HEAP32[$110>>2]|0;
    $112 = $8;
    $113 = (($112) + 8|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = (($114) + ($111<<3)|0);
    $116 = +HEAPF64[$115>>3];
    $117 = $sinpart;
    $118 = $117 + $116;
    $sinpart = $118;
    $119 = $2;
    $120 = HEAP32[$119>>2]|0;
    $121 = (($120) + 1)|0;
    HEAP32[$119>>2] = $121;
   }
  }
 }
 $122 = $e;
 $123 = $6;
 $124 = ($122|0)<($123|0);
 if ($124) {
  $125 = $3;
  $126 = $5;
  $127 = $6;
  $128 = Math_imul($126, $127)|0;
  $129 = (($125) + ($128<<2)|0);
  $130 = $e;
  $131 = (($129) + ($130<<2)|0);
  $iptr = $131;
  $132 = $iptr;
  $133 = HEAP32[$132>>2]|0;
  $134 = ($133|0)!=(-1);
  if ($134) {
   $135 = $iptr;
   $136 = HEAP32[$135>>2]|0;
   $137 = $8;
   $138 = (($137) + 4|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = (($139) + ($136<<3)|0);
   $141 = +HEAPF64[$140>>3];
   $142 = $cospart;
   $143 = $142 + $141;
   $cospart = $143;
   $144 = $iptr;
   $145 = HEAP32[$144>>2]|0;
   $146 = $8;
   $147 = (($146) + 8|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = (($148) + ($145<<3)|0);
   $150 = +HEAPF64[$149>>3];
   $151 = $sinpart;
   $152 = $151 + $150;
   $sinpart = $152;
   $153 = $2;
   $154 = HEAP32[$153>>2]|0;
   $155 = (($154) + 1)|0;
   HEAP32[$153>>2] = $155;
  }
 }
 $156 = $e;
 $157 = $6;
 $158 = ($156|0)<($157|0);
 if ($158) {
  $159 = $s;
  $160 = $7;
  $161 = ($159|0)<($160|0);
  if ($161) {
   $162 = $3;
   $163 = $s;
   $164 = $6;
   $165 = Math_imul($163, $164)|0;
   $166 = (($162) + ($165<<2)|0);
   $167 = $e;
   $168 = (($166) + ($167<<2)|0);
   $iptr = $168;
   $169 = $iptr;
   $170 = HEAP32[$169>>2]|0;
   $171 = ($170|0)!=(-1);
   if ($171) {
    $172 = $iptr;
    $173 = HEAP32[$172>>2]|0;
    $174 = $8;
    $175 = (($174) + 4|0);
    $176 = HEAP32[$175>>2]|0;
    $177 = (($176) + ($173<<3)|0);
    $178 = +HEAPF64[$177>>3];
    $179 = $cospart;
    $180 = $179 + $178;
    $cospart = $180;
    $181 = $iptr;
    $182 = HEAP32[$181>>2]|0;
    $183 = $8;
    $184 = (($183) + 8|0);
    $185 = HEAP32[$184>>2]|0;
    $186 = (($185) + ($182<<3)|0);
    $187 = +HEAPF64[$186>>3];
    $188 = $sinpart;
    $189 = $188 + $187;
    $sinpart = $189;
    $190 = $2;
    $191 = HEAP32[$190>>2]|0;
    $192 = (($191) + 1)|0;
    HEAP32[$190>>2] = $192;
   }
  }
 }
 $193 = $s;
 $194 = $7;
 $195 = ($193|0)<($194|0);
 if ($195) {
  $196 = $3;
  $197 = $s;
  $198 = $6;
  $199 = Math_imul($197, $198)|0;
  $200 = (($196) + ($199<<2)|0);
  $201 = $4;
  $202 = (($200) + ($201<<2)|0);
  $iptr = $202;
  $203 = $iptr;
  $204 = HEAP32[$203>>2]|0;
  $205 = ($204|0)!=(-1);
  if ($205) {
   $206 = $iptr;
   $207 = HEAP32[$206>>2]|0;
   $208 = $8;
   $209 = (($208) + 4|0);
   $210 = HEAP32[$209>>2]|0;
   $211 = (($210) + ($207<<3)|0);
   $212 = +HEAPF64[$211>>3];
   $213 = $cospart;
   $214 = $213 + $212;
   $cospart = $214;
   $215 = $iptr;
   $216 = HEAP32[$215>>2]|0;
   $217 = $8;
   $218 = (($217) + 8|0);
   $219 = HEAP32[$218>>2]|0;
   $220 = (($219) + ($216<<3)|0);
   $221 = +HEAPF64[$220>>3];
   $222 = $sinpart;
   $223 = $222 + $221;
   $sinpart = $223;
   $224 = $2;
   $225 = HEAP32[$224>>2]|0;
   $226 = (($225) + 1)|0;
   HEAP32[$224>>2] = $226;
  }
 }
 $227 = $w;
 $228 = ($227|0)>=(0);
 if ($228) {
  $229 = $s;
  $230 = $7;
  $231 = ($229|0)<($230|0);
  if ($231) {
   $232 = $3;
   $233 = $s;
   $234 = $6;
   $235 = Math_imul($233, $234)|0;
   $236 = (($232) + ($235<<2)|0);
   $237 = $w;
   $238 = (($236) + ($237<<2)|0);
   $iptr = $238;
   $239 = $iptr;
   $240 = HEAP32[$239>>2]|0;
   $241 = ($240|0)!=(-1);
   if ($241) {
    $242 = $iptr;
    $243 = HEAP32[$242>>2]|0;
    $244 = $8;
    $245 = (($244) + 4|0);
    $246 = HEAP32[$245>>2]|0;
    $247 = (($246) + ($243<<3)|0);
    $248 = +HEAPF64[$247>>3];
    $249 = $cospart;
    $250 = $249 + $248;
    $cospart = $250;
    $251 = $iptr;
    $252 = HEAP32[$251>>2]|0;
    $253 = $8;
    $254 = (($253) + 8|0);
    $255 = HEAP32[$254>>2]|0;
    $256 = (($255) + ($252<<3)|0);
    $257 = +HEAPF64[$256>>3];
    $258 = $sinpart;
    $259 = $258 + $257;
    $sinpart = $259;
    $260 = $2;
    $261 = HEAP32[$260>>2]|0;
    $262 = (($261) + 1)|0;
    HEAP32[$260>>2] = $262;
   }
  }
 }
 $263 = $w;
 $264 = ($263|0)>=(0);
 if ($264) {
  $265 = $3;
  $266 = $5;
  $267 = $6;
  $268 = Math_imul($266, $267)|0;
  $269 = (($265) + ($268<<2)|0);
  $270 = $w;
  $271 = (($269) + ($270<<2)|0);
  $iptr = $271;
  $272 = $iptr;
  $273 = HEAP32[$272>>2]|0;
  $274 = ($273|0)!=(-1);
  if ($274) {
   $275 = $iptr;
   $276 = HEAP32[$275>>2]|0;
   $277 = $8;
   $278 = (($277) + 4|0);
   $279 = HEAP32[$278>>2]|0;
   $280 = (($279) + ($276<<3)|0);
   $281 = +HEAPF64[$280>>3];
   $282 = $cospart;
   $283 = $282 + $281;
   $cospart = $283;
   $284 = $iptr;
   $285 = HEAP32[$284>>2]|0;
   $286 = $8;
   $287 = (($286) + 8|0);
   $288 = HEAP32[$287>>2]|0;
   $289 = (($288) + ($285<<3)|0);
   $290 = +HEAPF64[$289>>3];
   $291 = $sinpart;
   $292 = $291 + $290;
   $sinpart = $292;
   $293 = $2;
   $294 = HEAP32[$293>>2]|0;
   $295 = (($294) + 1)|0;
   HEAP32[$293>>2] = $295;
  }
 }
 $296 = $2;
 $297 = HEAP32[$296>>2]|0;
 $298 = ($297|0)==(0);
 if ($298) {
  $299 = $1;
  HEAPF64[$299>>3] = 0.0;
  $300 = $0;
  HEAP32[$300>>2] = -1;
  STACKTOP = sp;return;
 }
 $301 = $2;
 $302 = HEAP32[$301>>2]|0;
 $303 = (+($302|0));
 $304 = $cospart;
 $305 = $304 / $303;
 $cospart = $305;
 $306 = $2;
 $307 = HEAP32[$306>>2]|0;
 $308 = (+($307|0));
 $309 = $sinpart;
 $310 = $309 / $308;
 $sinpart = $310;
 $311 = $cospart;
 $312 = $cospart;
 $313 = $311 * $312;
 $314 = $sinpart;
 $315 = $sinpart;
 $316 = $314 * $315;
 $317 = $313 + $316;
 $318 = $1;
 HEAPF64[$318>>3] = $317;
 $319 = $1;
 $320 = +HEAPF64[$319>>3];
 $321 = $320 < 0.0;
 if ($321) {
  $322 = $1;
  $323 = +HEAPF64[$322>>3];
  $324 = $323 * 16384.0;
  $325 = $324 - 0.5;
  $326 = (~~(($325)));
  $327 = (+($326|0));
  $328 = $327 / 16384.0;
  $337 = $328;
 } else {
  $329 = $1;
  $330 = +HEAPF64[$329>>3];
  $331 = $330 * 16384.0;
  $332 = $331 + 0.5;
  $333 = (~~(($332)));
  $334 = (+($333|0));
  $335 = $334 / 16384.0;
  $337 = $335;
 }
 $336 = $1;
 HEAPF64[$336>>3] = $337;
 $338 = $1;
 $339 = +HEAPF64[$338>>3];
 $340 = $339 < 0.200000000000000011102;
 if ($340) {
  $341 = $1;
  HEAPF64[$341>>3] = 0.0;
  $342 = $0;
  HEAP32[$342>>2] = -1;
  STACKTOP = sp;return;
 }
 $343 = $sinpart;
 $344 = $cospart;
 $345 = (+Math_atan2((+$343),(+$344)));
 $theta = $345;
 $pi2 = 6.283185307179586232;
 $346 = $pi2;
 $347 = $theta;
 $348 = $347 + $346;
 $theta = $348;
 $349 = $theta;
 $350 = $pi2;
 $351 = (+_fmod((+$349),(+$350)));
 $theta = $351;
 $352 = $pi2;
 $353 = $8;
 $354 = HEAP32[$353>>2]|0;
 $355 = (+($354|0));
 $356 = $352 / $355;
 $pi_factor = $356;
 $357 = $theta;
 $358 = $pi_factor;
 $359 = $357 / $358;
 $avr = $359;
 $360 = $avr;
 $361 = $360 < 0.0;
 if ($361) {
  $362 = $avr;
  $363 = $362 * 16384.0;
  $364 = $363 - 0.5;
  $365 = (~~(($364)));
  $366 = (+($365|0));
  $367 = $366 / 16384.0;
  $374 = $367;
 } else {
  $368 = $avr;
  $369 = $368 * 16384.0;
  $370 = $369 + 0.5;
  $371 = (~~(($370)));
  $372 = (+($371|0));
  $373 = $372 / 16384.0;
  $374 = $373;
 }
 $avr = $374;
 $375 = $avr;
 $376 = $375 < 0.0;
 if ($376) {
  $377 = $avr;
  $378 = $377 - 0.5;
  $382 = $378;
 } else {
  $379 = $avr;
  $380 = $379 + 0.5;
  $382 = $380;
 }
 $381 = (~~(($382)));
 $383 = $0;
 HEAP32[$383>>2] = $381;
 $384 = $8;
 $385 = HEAP32[$384>>2]|0;
 $386 = $0;
 $387 = HEAP32[$386>>2]|0;
 $388 = (($387|0) % ($385|0))&-1;
 HEAP32[$386>>2] = $388;
 STACKTOP = sp;return;
}
function _num_valid_8nbrs($imap,$mx,$my,$mw,$mh) {
 $imap = $imap|0;
 $mx = $mx|0;
 $my = $my|0;
 $mw = $mw|0;
 $mh = $mh|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $e_ind = 0, $n_ind = 0, $nvalid = 0, $s_ind = 0, $w_ind = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $imap;
 $1 = $mx;
 $2 = $my;
 $3 = $mw;
 $4 = $mh;
 $nvalid = 0;
 $5 = $1;
 $6 = (($5) + 1)|0;
 $e_ind = $6;
 $7 = $1;
 $8 = (($7) - 1)|0;
 $w_ind = $8;
 $9 = $2;
 $10 = (($9) - 1)|0;
 $n_ind = $10;
 $11 = $2;
 $12 = (($11) + 1)|0;
 $s_ind = $12;
 $13 = $w_ind;
 $14 = ($13|0)>=(0);
 if ($14) {
  $15 = $n_ind;
  $16 = ($15|0)>=(0);
  if ($16) {
   $17 = $0;
   $18 = $n_ind;
   $19 = $3;
   $20 = Math_imul($18, $19)|0;
   $21 = (($17) + ($20<<2)|0);
   $22 = $w_ind;
   $23 = (($21) + ($22<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)>=(0);
   if ($25) {
    $26 = $nvalid;
    $27 = (($26) + 1)|0;
    $nvalid = $27;
   }
  }
 }
 $28 = $n_ind;
 $29 = ($28|0)>=(0);
 if ($29) {
  $30 = $0;
  $31 = $n_ind;
  $32 = $3;
  $33 = Math_imul($31, $32)|0;
  $34 = (($30) + ($33<<2)|0);
  $35 = $1;
  $36 = (($34) + ($35<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)>=(0);
  if ($38) {
   $39 = $nvalid;
   $40 = (($39) + 1)|0;
   $nvalid = $40;
  }
 }
 $41 = $n_ind;
 $42 = ($41|0)>=(0);
 if ($42) {
  $43 = $e_ind;
  $44 = $3;
  $45 = ($43|0)<($44|0);
  if ($45) {
   $46 = $0;
   $47 = $n_ind;
   $48 = $3;
   $49 = Math_imul($47, $48)|0;
   $50 = (($46) + ($49<<2)|0);
   $51 = $e_ind;
   $52 = (($50) + ($51<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ($53|0)>=(0);
   if ($54) {
    $55 = $nvalid;
    $56 = (($55) + 1)|0;
    $nvalid = $56;
   }
  }
 }
 $57 = $e_ind;
 $58 = $3;
 $59 = ($57|0)<($58|0);
 if ($59) {
  $60 = $0;
  $61 = $2;
  $62 = $3;
  $63 = Math_imul($61, $62)|0;
  $64 = (($60) + ($63<<2)|0);
  $65 = $e_ind;
  $66 = (($64) + ($65<<2)|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)>=(0);
  if ($68) {
   $69 = $nvalid;
   $70 = (($69) + 1)|0;
   $nvalid = $70;
  }
 }
 $71 = $e_ind;
 $72 = $3;
 $73 = ($71|0)<($72|0);
 if ($73) {
  $74 = $s_ind;
  $75 = $4;
  $76 = ($74|0)<($75|0);
  if ($76) {
   $77 = $0;
   $78 = $s_ind;
   $79 = $3;
   $80 = Math_imul($78, $79)|0;
   $81 = (($77) + ($80<<2)|0);
   $82 = $e_ind;
   $83 = (($81) + ($82<<2)|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)>=(0);
   if ($85) {
    $86 = $nvalid;
    $87 = (($86) + 1)|0;
    $nvalid = $87;
   }
  }
 }
 $88 = $s_ind;
 $89 = $4;
 $90 = ($88|0)<($89|0);
 if ($90) {
  $91 = $0;
  $92 = $s_ind;
  $93 = $3;
  $94 = Math_imul($92, $93)|0;
  $95 = (($91) + ($94<<2)|0);
  $96 = $1;
  $97 = (($95) + ($96<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = ($98|0)>=(0);
  if ($99) {
   $100 = $nvalid;
   $101 = (($100) + 1)|0;
   $nvalid = $101;
  }
 }
 $102 = $w_ind;
 $103 = ($102|0)>=(0);
 if ($103) {
  $104 = $s_ind;
  $105 = $4;
  $106 = ($104|0)<($105|0);
  if ($106) {
   $107 = $0;
   $108 = $s_ind;
   $109 = $3;
   $110 = Math_imul($108, $109)|0;
   $111 = (($107) + ($110<<2)|0);
   $112 = $w_ind;
   $113 = (($111) + ($112<<2)|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = ($114|0)>=(0);
   if ($115) {
    $116 = $nvalid;
    $117 = (($116) + 1)|0;
    $nvalid = $117;
   }
  }
 }
 $118 = $w_ind;
 $119 = ($118|0)>=(0);
 if (!($119)) {
  $131 = $nvalid;
  STACKTOP = sp;return ($131|0);
 }
 $120 = $0;
 $121 = $2;
 $122 = $3;
 $123 = Math_imul($121, $122)|0;
 $124 = (($120) + ($123<<2)|0);
 $125 = $w_ind;
 $126 = (($124) + ($125<<2)|0);
 $127 = HEAP32[$126>>2]|0;
 $128 = ($127|0)>=(0);
 if (!($128)) {
  $131 = $nvalid;
  STACKTOP = sp;return ($131|0);
 }
 $129 = $nvalid;
 $130 = (($129) + 1)|0;
 $nvalid = $130;
 $131 = $nvalid;
 STACKTOP = sp;return ($131|0);
}
function _vorticity($imap,$mx,$my,$mw,$mh,$ndirs) {
 $imap = $imap|0;
 $mx = $mx|0;
 $my = $my|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $e_ind = 0, $e_val = 0, $n_ind = 0, $n_val = 0, $ne_val = 0, $nw_val = 0, $s_ind = 0;
 var $s_val = 0, $se_val = 0, $sw_val = 0, $vmeasure = 0, $w_ind = 0, $w_val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vmeasure = sp;
 $0 = $imap;
 $1 = $mx;
 $2 = $my;
 $3 = $mw;
 $4 = $mh;
 $5 = $ndirs;
 $6 = $1;
 $7 = (($6) + 1)|0;
 $e_ind = $7;
 $8 = $1;
 $9 = (($8) - 1)|0;
 $w_ind = $9;
 $10 = $2;
 $11 = (($10) - 1)|0;
 $n_ind = $11;
 $12 = $2;
 $13 = (($12) + 1)|0;
 $s_ind = $13;
 $14 = $w_ind;
 $15 = ($14|0)>=(0);
 if ($15) {
  $16 = $n_ind;
  $17 = ($16|0)>=(0);
  if ($17) {
   $18 = $0;
   $19 = $n_ind;
   $20 = $3;
   $21 = Math_imul($19, $20)|0;
   $22 = (($18) + ($21<<2)|0);
   $23 = $w_ind;
   $24 = (($22) + ($23<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $nw_val = $25;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $nw_val = -1;
 }
 $26 = $n_ind;
 $27 = ($26|0)>=(0);
 if ($27) {
  $28 = $0;
  $29 = $n_ind;
  $30 = $3;
  $31 = Math_imul($29, $30)|0;
  $32 = (($28) + ($31<<2)|0);
  $33 = $1;
  $34 = (($32) + ($33<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $n_val = $35;
 } else {
  $n_val = -1;
 }
 $36 = $n_ind;
 $37 = ($36|0)>=(0);
 if ($37) {
  $38 = $e_ind;
  $39 = $3;
  $40 = ($38|0)<($39|0);
  if ($40) {
   $41 = $0;
   $42 = $n_ind;
   $43 = $3;
   $44 = Math_imul($42, $43)|0;
   $45 = (($41) + ($44<<2)|0);
   $46 = $e_ind;
   $47 = (($45) + ($46<<2)|0);
   $48 = HEAP32[$47>>2]|0;
   $ne_val = $48;
  } else {
   label = 11;
  }
 } else {
  label = 11;
 }
 if ((label|0) == 11) {
  $ne_val = -1;
 }
 $49 = $e_ind;
 $50 = $3;
 $51 = ($49|0)<($50|0);
 if ($51) {
  $52 = $0;
  $53 = $2;
  $54 = $3;
  $55 = Math_imul($53, $54)|0;
  $56 = (($52) + ($55<<2)|0);
  $57 = $e_ind;
  $58 = (($56) + ($57<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $e_val = $59;
 } else {
  $e_val = -1;
 }
 $60 = $e_ind;
 $61 = $3;
 $62 = ($60|0)<($61|0);
 if ($62) {
  $63 = $s_ind;
  $64 = $4;
  $65 = ($63|0)<($64|0);
  if ($65) {
   $66 = $0;
   $67 = $s_ind;
   $68 = $3;
   $69 = Math_imul($67, $68)|0;
   $70 = (($66) + ($69<<2)|0);
   $71 = $e_ind;
   $72 = (($70) + ($71<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $se_val = $73;
  } else {
   label = 18;
  }
 } else {
  label = 18;
 }
 if ((label|0) == 18) {
  $se_val = -1;
 }
 $74 = $s_ind;
 $75 = $4;
 $76 = ($74|0)<($75|0);
 if ($76) {
  $77 = $0;
  $78 = $s_ind;
  $79 = $3;
  $80 = Math_imul($78, $79)|0;
  $81 = (($77) + ($80<<2)|0);
  $82 = $1;
  $83 = (($81) + ($82<<2)|0);
  $84 = HEAP32[$83>>2]|0;
  $s_val = $84;
 } else {
  $s_val = -1;
 }
 $85 = $w_ind;
 $86 = ($85|0)>=(0);
 if ($86) {
  $87 = $s_ind;
  $88 = $4;
  $89 = ($87|0)<($88|0);
  if ($89) {
   $90 = $0;
   $91 = $s_ind;
   $92 = $3;
   $93 = Math_imul($91, $92)|0;
   $94 = (($90) + ($93<<2)|0);
   $95 = $w_ind;
   $96 = (($94) + ($95<<2)|0);
   $97 = HEAP32[$96>>2]|0;
   $sw_val = $97;
  } else {
   label = 25;
  }
 } else {
  label = 25;
 }
 if ((label|0) == 25) {
  $sw_val = -1;
 }
 $98 = $w_ind;
 $99 = ($98|0)>=(0);
 if ($99) {
  $100 = $0;
  $101 = $2;
  $102 = $3;
  $103 = Math_imul($101, $102)|0;
  $104 = (($100) + ($103<<2)|0);
  $105 = $w_ind;
  $106 = (($104) + ($105<<2)|0);
  $107 = HEAP32[$106>>2]|0;
  $w_val = $107;
 } else {
  $w_val = -1;
 }
 HEAP32[$vmeasure>>2] = 0;
 $108 = $nw_val;
 $109 = $n_val;
 $110 = $5;
 _accum_nbr_vorticity($vmeasure,$108,$109,$110);
 $111 = $n_val;
 $112 = $ne_val;
 $113 = $5;
 _accum_nbr_vorticity($vmeasure,$111,$112,$113);
 $114 = $ne_val;
 $115 = $e_val;
 $116 = $5;
 _accum_nbr_vorticity($vmeasure,$114,$115,$116);
 $117 = $e_val;
 $118 = $se_val;
 $119 = $5;
 _accum_nbr_vorticity($vmeasure,$117,$118,$119);
 $120 = $se_val;
 $121 = $s_val;
 $122 = $5;
 _accum_nbr_vorticity($vmeasure,$120,$121,$122);
 $123 = $s_val;
 $124 = $sw_val;
 $125 = $5;
 _accum_nbr_vorticity($vmeasure,$123,$124,$125);
 $126 = $sw_val;
 $127 = $w_val;
 $128 = $5;
 _accum_nbr_vorticity($vmeasure,$126,$127,$128);
 $129 = $w_val;
 $130 = $nw_val;
 $131 = $5;
 _accum_nbr_vorticity($vmeasure,$129,$130,$131);
 $132 = HEAP32[$vmeasure>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _curvature($imap,$mx,$my,$mw,$mh,$ndirs) {
 $imap = $imap|0;
 $mx = $mx|0;
 $my = $my|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cmeasure = 0, $dist = 0, $e_ind = 0, $e_val = 0, $iptr = 0, $n_ind = 0, $n_val = 0, $ne_val = 0, $nw_val = 0, $s_ind = 0, $s_val = 0, $se_val = 0;
 var $sw_val = 0, $w_ind = 0, $w_val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $imap;
 $1 = $mx;
 $2 = $my;
 $3 = $mw;
 $4 = $mh;
 $5 = $ndirs;
 $6 = $1;
 $7 = (($6) + 1)|0;
 $e_ind = $7;
 $8 = $1;
 $9 = (($8) - 1)|0;
 $w_ind = $9;
 $10 = $2;
 $11 = (($10) - 1)|0;
 $n_ind = $11;
 $12 = $2;
 $13 = (($12) + 1)|0;
 $s_ind = $13;
 $14 = $w_ind;
 $15 = ($14|0)>=(0);
 if ($15) {
  $16 = $n_ind;
  $17 = ($16|0)>=(0);
  if ($17) {
   $18 = $0;
   $19 = $n_ind;
   $20 = $3;
   $21 = Math_imul($19, $20)|0;
   $22 = (($18) + ($21<<2)|0);
   $23 = $w_ind;
   $24 = (($22) + ($23<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $nw_val = $25;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $nw_val = -1;
 }
 $26 = $n_ind;
 $27 = ($26|0)>=(0);
 if ($27) {
  $28 = $0;
  $29 = $n_ind;
  $30 = $3;
  $31 = Math_imul($29, $30)|0;
  $32 = (($28) + ($31<<2)|0);
  $33 = $1;
  $34 = (($32) + ($33<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $n_val = $35;
 } else {
  $n_val = -1;
 }
 $36 = $n_ind;
 $37 = ($36|0)>=(0);
 if ($37) {
  $38 = $e_ind;
  $39 = $3;
  $40 = ($38|0)<($39|0);
  if ($40) {
   $41 = $0;
   $42 = $n_ind;
   $43 = $3;
   $44 = Math_imul($42, $43)|0;
   $45 = (($41) + ($44<<2)|0);
   $46 = $e_ind;
   $47 = (($45) + ($46<<2)|0);
   $48 = HEAP32[$47>>2]|0;
   $ne_val = $48;
  } else {
   label = 11;
  }
 } else {
  label = 11;
 }
 if ((label|0) == 11) {
  $ne_val = -1;
 }
 $49 = $e_ind;
 $50 = $3;
 $51 = ($49|0)<($50|0);
 if ($51) {
  $52 = $0;
  $53 = $2;
  $54 = $3;
  $55 = Math_imul($53, $54)|0;
  $56 = (($52) + ($55<<2)|0);
  $57 = $e_ind;
  $58 = (($56) + ($57<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $e_val = $59;
 } else {
  $e_val = -1;
 }
 $60 = $e_ind;
 $61 = $3;
 $62 = ($60|0)<($61|0);
 if ($62) {
  $63 = $s_ind;
  $64 = $4;
  $65 = ($63|0)<($64|0);
  if ($65) {
   $66 = $0;
   $67 = $s_ind;
   $68 = $3;
   $69 = Math_imul($67, $68)|0;
   $70 = (($66) + ($69<<2)|0);
   $71 = $e_ind;
   $72 = (($70) + ($71<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $se_val = $73;
  } else {
   label = 18;
  }
 } else {
  label = 18;
 }
 if ((label|0) == 18) {
  $se_val = -1;
 }
 $74 = $s_ind;
 $75 = $4;
 $76 = ($74|0)<($75|0);
 if ($76) {
  $77 = $0;
  $78 = $s_ind;
  $79 = $3;
  $80 = Math_imul($78, $79)|0;
  $81 = (($77) + ($80<<2)|0);
  $82 = $1;
  $83 = (($81) + ($82<<2)|0);
  $84 = HEAP32[$83>>2]|0;
  $s_val = $84;
 } else {
  $s_val = -1;
 }
 $85 = $w_ind;
 $86 = ($85|0)>=(0);
 if ($86) {
  $87 = $s_ind;
  $88 = $4;
  $89 = ($87|0)<($88|0);
  if ($89) {
   $90 = $0;
   $91 = $s_ind;
   $92 = $3;
   $93 = Math_imul($91, $92)|0;
   $94 = (($90) + ($93<<2)|0);
   $95 = $w_ind;
   $96 = (($94) + ($95<<2)|0);
   $97 = HEAP32[$96>>2]|0;
   $sw_val = $97;
  } else {
   label = 25;
  }
 } else {
  label = 25;
 }
 if ((label|0) == 25) {
  $sw_val = -1;
 }
 $98 = $w_ind;
 $99 = ($98|0)>=(0);
 if ($99) {
  $100 = $0;
  $101 = $2;
  $102 = $3;
  $103 = Math_imul($101, $102)|0;
  $104 = (($100) + ($103<<2)|0);
  $105 = $w_ind;
  $106 = (($104) + ($105<<2)|0);
  $107 = HEAP32[$106>>2]|0;
  $w_val = $107;
 } else {
  $w_val = -1;
 }
 $108 = $0;
 $109 = $2;
 $110 = $3;
 $111 = Math_imul($109, $110)|0;
 $112 = (($108) + ($111<<2)|0);
 $113 = $1;
 $114 = (($112) + ($113<<2)|0);
 $iptr = $114;
 $cmeasure = -1;
 $115 = $iptr;
 $116 = HEAP32[$115>>2]|0;
 $117 = $nw_val;
 $118 = $5;
 $119 = (_closest_dir_dist($116,$117,$118)|0);
 $dist = $119;
 $120 = $dist;
 $121 = $cmeasure;
 $122 = ($120|0)>($121|0);
 if ($122) {
  $123 = $dist;
  $cmeasure = $123;
 }
 $124 = $iptr;
 $125 = HEAP32[$124>>2]|0;
 $126 = $n_val;
 $127 = $5;
 $128 = (_closest_dir_dist($125,$126,$127)|0);
 $dist = $128;
 $129 = $dist;
 $130 = $cmeasure;
 $131 = ($129|0)>($130|0);
 if ($131) {
  $132 = $dist;
  $cmeasure = $132;
 }
 $133 = $iptr;
 $134 = HEAP32[$133>>2]|0;
 $135 = $ne_val;
 $136 = $5;
 $137 = (_closest_dir_dist($134,$135,$136)|0);
 $dist = $137;
 $138 = $dist;
 $139 = $cmeasure;
 $140 = ($138|0)>($139|0);
 if ($140) {
  $141 = $dist;
  $cmeasure = $141;
 }
 $142 = $iptr;
 $143 = HEAP32[$142>>2]|0;
 $144 = $e_val;
 $145 = $5;
 $146 = (_closest_dir_dist($143,$144,$145)|0);
 $dist = $146;
 $147 = $dist;
 $148 = $cmeasure;
 $149 = ($147|0)>($148|0);
 if ($149) {
  $150 = $dist;
  $cmeasure = $150;
 }
 $151 = $iptr;
 $152 = HEAP32[$151>>2]|0;
 $153 = $se_val;
 $154 = $5;
 $155 = (_closest_dir_dist($152,$153,$154)|0);
 $dist = $155;
 $156 = $dist;
 $157 = $cmeasure;
 $158 = ($156|0)>($157|0);
 if ($158) {
  $159 = $dist;
  $cmeasure = $159;
 }
 $160 = $iptr;
 $161 = HEAP32[$160>>2]|0;
 $162 = $s_val;
 $163 = $5;
 $164 = (_closest_dir_dist($161,$162,$163)|0);
 $dist = $164;
 $165 = $dist;
 $166 = $cmeasure;
 $167 = ($165|0)>($166|0);
 if ($167) {
  $168 = $dist;
  $cmeasure = $168;
 }
 $169 = $iptr;
 $170 = HEAP32[$169>>2]|0;
 $171 = $sw_val;
 $172 = $5;
 $173 = (_closest_dir_dist($170,$171,$172)|0);
 $dist = $173;
 $174 = $dist;
 $175 = $cmeasure;
 $176 = ($174|0)>($175|0);
 if ($176) {
  $177 = $dist;
  $cmeasure = $177;
 }
 $178 = $iptr;
 $179 = HEAP32[$178>>2]|0;
 $180 = $w_val;
 $181 = $5;
 $182 = (_closest_dir_dist($179,$180,$181)|0);
 $dist = $182;
 $183 = $dist;
 $184 = $cmeasure;
 $185 = ($183|0)>($184|0);
 if (!($185)) {
  $187 = $cmeasure;
  STACKTOP = sp;return ($187|0);
 }
 $186 = $dist;
 $cmeasure = $186;
 $187 = $cmeasure;
 STACKTOP = sp;return ($187|0);
}
function _remove_dir($imap,$mx,$my,$mw,$mh,$dir2rad,$lfsparms) {
 $imap = $imap|0;
 $mx = $mx|0;
 $my = $my|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $avrdir = 0, $dir_strength = 0, $dist = 0, $nvalid = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $avrdir = sp + 16|0;
 $nvalid = sp + 12|0;
 $dir_strength = sp;
 $1 = $imap;
 $2 = $mx;
 $3 = $my;
 $4 = $mw;
 $5 = $mh;
 $6 = $dir2rad;
 $7 = $lfsparms;
 $8 = $1;
 $9 = $2;
 $10 = $3;
 $11 = $4;
 $12 = $5;
 $13 = $6;
 _average_8nbr_dir($avrdir,$dir_strength,$nvalid,$8,$9,$10,$11,$12,$13);
 $14 = HEAP32[$nvalid>>2]|0;
 $15 = $7;
 $16 = (($15) + 32|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14|0)<($17|0);
 if ($18) {
  $0 = 1;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 $19 = +HEAPF64[$dir_strength>>3];
 $20 = $7;
 $21 = (($20) + 40|0);
 $22 = +HEAPF64[$21>>3];
 $23 = $19 >= $22;
 do {
  if ($23) {
   $24 = HEAP32[$avrdir>>2]|0;
   $25 = $1;
   $26 = $3;
   $27 = $4;
   $28 = Math_imul($26, $27)|0;
   $29 = (($25) + ($28<<2)|0);
   $30 = $2;
   $31 = (($29) + ($30<<2)|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($24) - ($32))|0;
   $34 = (Math_abs(($33|0))|0);
   $dist = $34;
   $35 = $dist;
   $36 = $6;
   $37 = HEAP32[$36>>2]|0;
   $38 = $dist;
   $39 = (($37) - ($38))|0;
   $40 = ($35|0)<($39|0);
   if ($40) {
    $41 = $dist;
    $46 = $41;
   } else {
    $42 = $6;
    $43 = HEAP32[$42>>2]|0;
    $44 = $dist;
    $45 = (($43) - ($44))|0;
    $46 = $45;
   }
   $dist = $46;
   $47 = $dist;
   $48 = $7;
   $49 = (($48) + 48|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($47|0)>($50|0);
   if (!($51)) {
    break;
   }
   $0 = 2;
   $52 = $0;
   STACKTOP = sp;return ($52|0);
  }
 } while(0);
 $0 = 0;
 $52 = $0;
 STACKTOP = sp;return ($52|0);
}
function _test_top_edge($lbox,$tbox,$rbox,$bbox,$imap,$mw,$mh,$dir2rad,$lfsparms) {
 $lbox = $lbox|0;
 $tbox = $tbox|0;
 $rbox = $rbox|0;
 $bbox = $bbox|0;
 $imap = $imap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $bx = 0, $by = 0, $eptr = 0, $ex = 0, $iptr = 0, $nremoved = 0, $sptr = 0, $sx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $lbox;
 $1 = $tbox;
 $2 = $rbox;
 $3 = $bbox;
 $4 = $imap;
 $5 = $mw;
 $6 = $mh;
 $7 = $dir2rad;
 $8 = $lfsparms;
 $nremoved = 0;
 $9 = $0;
 $10 = ($9|0)>(0);
 if ($10) {
  $11 = $0;
  $12 = $11;
 } else {
  $12 = 0;
 }
 $sx = $12;
 $13 = $4;
 $14 = $1;
 $15 = $5;
 $16 = Math_imul($14, $15)|0;
 $17 = (($13) + ($16<<2)|0);
 $18 = $sx;
 $19 = (($17) + ($18<<2)|0);
 $sptr = $19;
 $20 = $2;
 $21 = (($20) - 1)|0;
 $22 = $5;
 $23 = (($22) - 1)|0;
 $24 = ($21|0)<($23|0);
 if ($24) {
  $25 = $2;
  $26 = (($25) - 1)|0;
  $29 = $26;
 } else {
  $27 = $5;
  $28 = (($27) - 1)|0;
  $29 = $28;
 }
 $ex = $29;
 $30 = $4;
 $31 = $1;
 $32 = $5;
 $33 = Math_imul($31, $32)|0;
 $34 = (($30) + ($33<<2)|0);
 $35 = $ex;
 $36 = (($34) + ($35<<2)|0);
 $eptr = $36;
 $37 = $sptr;
 $iptr = $37;
 $38 = $sx;
 $bx = $38;
 $39 = $1;
 $by = $39;
 while(1) {
  $40 = $iptr;
  $41 = $eptr;
  $42 = ($40>>>0)<=($41>>>0);
  if (!($42)) {
   break;
  }
  $43 = $iptr;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(-1);
  if ($45) {
   $46 = $4;
   $47 = $bx;
   $48 = $by;
   $49 = $5;
   $50 = $6;
   $51 = $7;
   $52 = $8;
   $53 = (_remove_dir($46,$47,$48,$49,$50,$51,$52)|0);
   $54 = ($53|0)!=(0);
   if ($54) {
    $55 = $iptr;
    HEAP32[$55>>2] = -1;
    $56 = $nremoved;
    $57 = (($56) + 1)|0;
    $nremoved = $57;
   }
  }
  $58 = $iptr;
  $59 = (($58) + 4|0);
  $iptr = $59;
  $60 = $bx;
  $61 = (($60) + 1)|0;
  $bx = $61;
 }
 $62 = $nremoved;
 STACKTOP = sp;return ($62|0);
}
function _test_right_edge($lbox,$tbox,$rbox,$bbox,$imap,$mw,$mh,$dir2rad,$lfsparms) {
 $lbox = $lbox|0;
 $tbox = $tbox|0;
 $rbox = $rbox|0;
 $bbox = $bbox|0;
 $imap = $imap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $bx = 0, $by = 0, $eptr = 0, $ey = 0, $iptr = 0, $nremoved = 0, $sptr = 0, $sy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $lbox;
 $1 = $tbox;
 $2 = $rbox;
 $3 = $bbox;
 $4 = $imap;
 $5 = $mw;
 $6 = $mh;
 $7 = $dir2rad;
 $8 = $lfsparms;
 $nremoved = 0;
 $9 = $1;
 $10 = ($9|0)>(0);
 if ($10) {
  $11 = $1;
  $12 = $11;
 } else {
  $12 = 0;
 }
 $sy = $12;
 $13 = $4;
 $14 = $sy;
 $15 = $5;
 $16 = Math_imul($14, $15)|0;
 $17 = (($13) + ($16<<2)|0);
 $18 = $2;
 $19 = (($17) + ($18<<2)|0);
 $sptr = $19;
 $20 = $3;
 $21 = (($20) - 1)|0;
 $22 = $6;
 $23 = (($22) - 1)|0;
 $24 = ($21|0)<($23|0);
 if ($24) {
  $25 = $3;
  $26 = (($25) - 1)|0;
  $29 = $26;
 } else {
  $27 = $6;
  $28 = (($27) - 1)|0;
  $29 = $28;
 }
 $ey = $29;
 $30 = $4;
 $31 = $ey;
 $32 = $5;
 $33 = Math_imul($31, $32)|0;
 $34 = (($30) + ($33<<2)|0);
 $35 = $2;
 $36 = (($34) + ($35<<2)|0);
 $eptr = $36;
 $37 = $sptr;
 $iptr = $37;
 $38 = $2;
 $bx = $38;
 $39 = $sy;
 $by = $39;
 while(1) {
  $40 = $iptr;
  $41 = $eptr;
  $42 = ($40>>>0)<=($41>>>0);
  if (!($42)) {
   break;
  }
  $43 = $iptr;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(-1);
  if ($45) {
   $46 = $4;
   $47 = $bx;
   $48 = $by;
   $49 = $5;
   $50 = $6;
   $51 = $7;
   $52 = $8;
   $53 = (_remove_dir($46,$47,$48,$49,$50,$51,$52)|0);
   $54 = ($53|0)!=(0);
   if ($54) {
    $55 = $iptr;
    HEAP32[$55>>2] = -1;
    $56 = $nremoved;
    $57 = (($56) + 1)|0;
    $nremoved = $57;
   }
  }
  $58 = $5;
  $59 = $iptr;
  $60 = (($59) + ($58<<2)|0);
  $iptr = $60;
  $61 = $by;
  $62 = (($61) + 1)|0;
  $by = $62;
 }
 $63 = $nremoved;
 STACKTOP = sp;return ($63|0);
}
function _test_bottom_edge($lbox,$tbox,$rbox,$bbox,$imap,$mw,$mh,$dir2rad,$lfsparms) {
 $lbox = $lbox|0;
 $tbox = $tbox|0;
 $rbox = $rbox|0;
 $bbox = $bbox|0;
 $imap = $imap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $bx = 0, $by = 0, $eptr = 0, $ex = 0, $iptr = 0, $nremoved = 0, $sptr = 0, $sx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $lbox;
 $1 = $tbox;
 $2 = $rbox;
 $3 = $bbox;
 $4 = $imap;
 $5 = $mw;
 $6 = $mh;
 $7 = $dir2rad;
 $8 = $lfsparms;
 $nremoved = 0;
 $9 = $2;
 $10 = $5;
 $11 = (($10) - 1)|0;
 $12 = ($9|0)<($11|0);
 if ($12) {
  $13 = $2;
  $16 = $13;
 } else {
  $14 = $5;
  $15 = (($14) - 1)|0;
  $16 = $15;
 }
 $sx = $16;
 $17 = $4;
 $18 = $3;
 $19 = $5;
 $20 = Math_imul($18, $19)|0;
 $21 = (($17) + ($20<<2)|0);
 $22 = $sx;
 $23 = (($21) + ($22<<2)|0);
 $sptr = $23;
 $24 = $0;
 $25 = (($24) - 1)|0;
 $26 = ($25|0)>(0);
 if ($26) {
  $27 = $0;
  $28 = (($27) - 1)|0;
  $29 = $28;
 } else {
  $29 = 0;
 }
 $ex = $29;
 $30 = $4;
 $31 = $3;
 $32 = $5;
 $33 = Math_imul($31, $32)|0;
 $34 = (($30) + ($33<<2)|0);
 $35 = $ex;
 $36 = (($34) + ($35<<2)|0);
 $eptr = $36;
 $37 = $sptr;
 $iptr = $37;
 $38 = $sx;
 $bx = $38;
 $39 = $3;
 $by = $39;
 while(1) {
  $40 = $iptr;
  $41 = $eptr;
  $42 = ($40>>>0)>=($41>>>0);
  if (!($42)) {
   break;
  }
  $43 = $iptr;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(-1);
  if ($45) {
   $46 = $4;
   $47 = $bx;
   $48 = $by;
   $49 = $5;
   $50 = $6;
   $51 = $7;
   $52 = $8;
   $53 = (_remove_dir($46,$47,$48,$49,$50,$51,$52)|0);
   $54 = ($53|0)!=(0);
   if ($54) {
    $55 = $iptr;
    HEAP32[$55>>2] = -1;
    $56 = $nremoved;
    $57 = (($56) + 1)|0;
    $nremoved = $57;
   }
  }
  $58 = $iptr;
  $59 = (($58) + -4|0);
  $iptr = $59;
  $60 = $bx;
  $61 = (($60) + -1)|0;
  $bx = $61;
 }
 $62 = $nremoved;
 STACKTOP = sp;return ($62|0);
}
function _test_left_edge($lbox,$tbox,$rbox,$bbox,$imap,$mw,$mh,$dir2rad,$lfsparms) {
 $lbox = $lbox|0;
 $tbox = $tbox|0;
 $rbox = $rbox|0;
 $bbox = $bbox|0;
 $imap = $imap|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dir2rad = $dir2rad|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $bx = 0, $by = 0, $eptr = 0, $ey = 0, $iptr = 0, $nremoved = 0, $sptr = 0, $sy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $lbox;
 $1 = $tbox;
 $2 = $rbox;
 $3 = $bbox;
 $4 = $imap;
 $5 = $mw;
 $6 = $mh;
 $7 = $dir2rad;
 $8 = $lfsparms;
 $nremoved = 0;
 $9 = $3;
 $10 = $6;
 $11 = (($10) - 1)|0;
 $12 = ($9|0)<($11|0);
 if ($12) {
  $13 = $3;
  $16 = $13;
 } else {
  $14 = $6;
  $15 = (($14) - 1)|0;
  $16 = $15;
 }
 $sy = $16;
 $17 = $4;
 $18 = $sy;
 $19 = $5;
 $20 = Math_imul($18, $19)|0;
 $21 = (($17) + ($20<<2)|0);
 $22 = $0;
 $23 = (($21) + ($22<<2)|0);
 $sptr = $23;
 $24 = $1;
 $25 = (($24) - 1)|0;
 $26 = ($25|0)>(0);
 if ($26) {
  $27 = $1;
  $28 = (($27) - 1)|0;
  $29 = $28;
 } else {
  $29 = 0;
 }
 $ey = $29;
 $30 = $4;
 $31 = $ey;
 $32 = $5;
 $33 = Math_imul($31, $32)|0;
 $34 = (($30) + ($33<<2)|0);
 $35 = $0;
 $36 = (($34) + ($35<<2)|0);
 $eptr = $36;
 $37 = $sptr;
 $iptr = $37;
 $38 = $0;
 $bx = $38;
 $39 = $sy;
 $by = $39;
 while(1) {
  $40 = $iptr;
  $41 = $eptr;
  $42 = ($40>>>0)>=($41>>>0);
  if (!($42)) {
   break;
  }
  $43 = $iptr;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(-1);
  if ($45) {
   $46 = $4;
   $47 = $bx;
   $48 = $by;
   $49 = $5;
   $50 = $6;
   $51 = $7;
   $52 = $8;
   $53 = (_remove_dir($46,$47,$48,$49,$50,$51,$52)|0);
   $54 = ($53|0)!=(0);
   if ($54) {
    $55 = $iptr;
    HEAP32[$55>>2] = -1;
    $56 = $nremoved;
    $57 = (($56) + 1)|0;
    $nremoved = $57;
   }
  }
  $58 = $5;
  $59 = $iptr;
  $60 = (0 - ($58))|0;
  $61 = (($59) + ($60<<2)|0);
  $iptr = $61;
  $62 = $by;
  $63 = (($62) + -1)|0;
  $by = $63;
 }
 $64 = $nremoved;
 STACKTOP = sp;return ($64|0);
}
function _accum_nbr_vorticity($vmeasure,$dir1,$dir2,$ndirs) {
 $vmeasure = $vmeasure|0;
 $dir1 = $dir1|0;
 $dir2 = $dir2|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dist = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $vmeasure;
 $1 = $dir1;
 $2 = $dir2;
 $3 = $ndirs;
 $4 = $1;
 $5 = $2;
 $6 = ($4|0)!=($5|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $1;
 $8 = ($7|0)>=(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = $2;
 $10 = ($9|0)>=(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = $2;
 $12 = $1;
 $13 = (($11) - ($12))|0;
 $dist = $13;
 $14 = $dist;
 $15 = ($14|0)<(0);
 if ($15) {
  $16 = $3;
  $17 = $dist;
  $18 = (($17) + ($16))|0;
  $dist = $18;
 }
 $19 = $dist;
 $20 = $3;
 $21 = $20 >> 1;
 $22 = ($19|0)>($21|0);
 if ($22) {
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + -1)|0;
  HEAP32[$23>>2] = $25;
 } else {
  $26 = $0;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$26>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _match_1st_pair($p1,$p2,$possible,$nposs) {
 $p1 = $p1|0;
 $p2 = $p2|0;
 $possible = $possible|0;
 $nposs = $nposs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $p1;
 $1 = $p2;
 $2 = $possible;
 $3 = $nposs;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = ($5|0)<(10);
  if (!($6)) {
   break;
  }
  $7 = $0;
  $8 = $7&255;
  $9 = $i;
  $10 = (1016 + ($9<<5)|0);
  $11 = (($10) + 8|0);
  $12 = HEAPU8[$11>>0]|(HEAPU8[$11+1>>0]<<8)|(HEAPU8[$11+2>>0]<<16)|(HEAPU8[$11+3>>0]<<24);
  $13 = ($8|0)==($12|0);
  if ($13) {
   $14 = $1;
   $15 = $14&255;
   $16 = $i;
   $17 = (1016 + ($16<<5)|0);
   $18 = (($17) + 8|0);
   $19 = (($18) + 4|0);
   $20 = HEAPU8[$19>>0]|(HEAPU8[$19+1>>0]<<8)|(HEAPU8[$19+2>>0]<<16)|(HEAPU8[$19+3>>0]<<24);
   $21 = ($15|0)==($20|0);
   if ($21) {
    $22 = $i;
    $23 = $3;
    $24 = HEAP32[$23>>2]|0;
    $25 = $2;
    $26 = (($25) + ($24<<2)|0);
    HEAP32[$26>>2] = $22;
    $27 = $3;
    $28 = HEAP32[$27>>2]|0;
    $29 = (($28) + 1)|0;
    HEAP32[$27>>2] = $29;
   }
  }
  $30 = $i;
  $31 = (($30) + 1)|0;
  $i = $31;
 }
 $32 = $3;
 $33 = HEAP32[$32>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _match_2nd_pair($p1,$p2,$possible,$nposs) {
 $p1 = $p1|0;
 $p2 = $p2|0;
 $possible = $possible|0;
 $nposs = $nposs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $tnposs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $p1;
 $2 = $p2;
 $3 = $possible;
 $4 = $nposs;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $tnposs = $6;
 $7 = $4;
 HEAP32[$7>>2] = 0;
 $8 = $1;
 $9 = $8&255;
 $10 = $2;
 $11 = $10&255;
 $12 = ($9|0)==($11|0);
 if ($12) {
  $13 = $4;
  $14 = HEAP32[$13>>2]|0;
  $0 = $14;
  $54 = $0;
  STACKTOP = sp;return ($54|0);
 }
 $i = 0;
 while(1) {
  $15 = $i;
  $16 = $tnposs;
  $17 = ($15|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = $1;
  $19 = $18&255;
  $20 = $i;
  $21 = $3;
  $22 = (($21) + ($20<<2)|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (1016 + ($23<<5)|0);
  $25 = (($24) + 16|0);
  $26 = HEAPU8[$25>>0]|(HEAPU8[$25+1>>0]<<8)|(HEAPU8[$25+2>>0]<<16)|(HEAPU8[$25+3>>0]<<24);
  $27 = ($19|0)==($26|0);
  if ($27) {
   $28 = $2;
   $29 = $28&255;
   $30 = $i;
   $31 = $3;
   $32 = (($31) + ($30<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (1016 + ($33<<5)|0);
   $35 = (($34) + 16|0);
   $36 = (($35) + 4|0);
   $37 = HEAPU8[$36>>0]|(HEAPU8[$36+1>>0]<<8)|(HEAPU8[$36+2>>0]<<16)|(HEAPU8[$36+3>>0]<<24);
   $38 = ($29|0)==($37|0);
   if ($38) {
    $39 = $i;
    $40 = $3;
    $41 = (($40) + ($39<<2)|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = $4;
    $44 = HEAP32[$43>>2]|0;
    $45 = $3;
    $46 = (($45) + ($44<<2)|0);
    HEAP32[$46>>2] = $42;
    $47 = $4;
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + 1)|0;
    HEAP32[$47>>2] = $49;
   }
  }
  $50 = $i;
  $51 = (($50) + 1)|0;
  $i = $51;
 }
 $52 = $4;
 $53 = HEAP32[$52>>2]|0;
 $0 = $53;
 $54 = $0;
 STACKTOP = sp;return ($54|0);
}
function _match_3rd_pair($p1,$p2,$possible,$nposs) {
 $p1 = $p1|0;
 $p2 = $p2|0;
 $possible = $possible|0;
 $nposs = $nposs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $tnposs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $p1;
 $1 = $p2;
 $2 = $possible;
 $3 = $nposs;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $tnposs = $5;
 $6 = $3;
 HEAP32[$6>>2] = 0;
 $i = 0;
 while(1) {
  $7 = $i;
  $8 = $tnposs;
  $9 = ($7|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $0;
  $11 = $10&255;
  $12 = $i;
  $13 = $2;
  $14 = (($13) + ($12<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (1016 + ($15<<5)|0);
  $17 = (($16) + 24|0);
  $18 = HEAPU8[$17>>0]|(HEAPU8[$17+1>>0]<<8)|(HEAPU8[$17+2>>0]<<16)|(HEAPU8[$17+3>>0]<<24);
  $19 = ($11|0)==($18|0);
  if ($19) {
   $20 = $1;
   $21 = $20&255;
   $22 = $i;
   $23 = $2;
   $24 = (($23) + ($22<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (1016 + ($25<<5)|0);
   $27 = (($26) + 24|0);
   $28 = (($27) + 4|0);
   $29 = HEAPU8[$28>>0]|(HEAPU8[$28+1>>0]<<8)|(HEAPU8[$28+2>>0]<<16)|(HEAPU8[$28+3>>0]<<24);
   $30 = ($21|0)==($29|0);
   if ($30) {
    $31 = $i;
    $32 = $2;
    $33 = (($32) + ($31<<2)|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = $3;
    $36 = HEAP32[$35>>2]|0;
    $37 = $2;
    $38 = (($37) + ($36<<2)|0);
    HEAP32[$38>>2] = $34;
    $39 = $3;
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + 1)|0;
    HEAP32[$39>>2] = $41;
   }
  }
  $42 = $i;
  $43 = (($42) + 1)|0;
  $i = $43;
 }
 $44 = $3;
 $45 = HEAP32[$44>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _skip_repeated_horizontal_pair($cx,$ex,$p1ptr,$p2ptr,$iw,$ih) {
 $cx = $cx|0;
 $ex = $ex|0;
 $p1ptr = $p1ptr|0;
 $p2ptr = $p2ptr|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $old1 = 0, $old2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cx;
 $1 = $ex;
 $2 = $p1ptr;
 $3 = $p2ptr;
 $4 = $iw;
 $5 = $ih;
 $6 = $2;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $old1 = $9;
 $10 = $3;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $old2 = $13;
 $14 = $0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 1)|0;
 HEAP32[$14>>2] = $16;
 $17 = $2;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 1|0);
 HEAP32[$17>>2] = $19;
 $20 = $3;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1|0);
 HEAP32[$20>>2] = $22;
 while(1) {
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = $1;
  $26 = ($24|0)<($25|0);
  if (!($26)) {
   label = 7;
   break;
  }
  $27 = $2;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP8[$28>>0]|0;
  $30 = $29&255;
  $31 = $old1;
  $32 = ($30|0)!=($31|0);
  if ($32) {
   break;
  }
  $33 = $3;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  $37 = $old2;
  $38 = ($36|0)!=($37|0);
  if ($38) {
   break;
  }
  $39 = $0;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$39>>2] = $41;
  $42 = $2;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1|0);
  HEAP32[$42>>2] = $44;
  $45 = $3;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + 1|0);
  HEAP32[$45>>2] = $47;
 }
 if ((label|0) == 7) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _skip_repeated_vertical_pair($cy,$ey,$p1ptr,$p2ptr,$iw,$ih) {
 $cy = $cy|0;
 $ey = $ey|0;
 $p1ptr = $p1ptr|0;
 $p2ptr = $p2ptr|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $old1 = 0, $old2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cy;
 $1 = $ey;
 $2 = $p1ptr;
 $3 = $p2ptr;
 $4 = $iw;
 $5 = $ih;
 $6 = $2;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $old1 = $9;
 $10 = $3;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $old2 = $13;
 $14 = $0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 1)|0;
 HEAP32[$14>>2] = $16;
 $17 = $4;
 $18 = $2;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + ($17)|0);
 HEAP32[$18>>2] = $20;
 $21 = $4;
 $22 = $3;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($21)|0);
 HEAP32[$22>>2] = $24;
 while(1) {
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $1;
  $28 = ($26|0)<($27|0);
  if (!($28)) {
   label = 7;
   break;
  }
  $29 = $2;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = $old1;
  $34 = ($32|0)!=($33|0);
  if ($34) {
   break;
  }
  $35 = $3;
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP8[$36>>0]|0;
  $38 = $37&255;
  $39 = $old2;
  $40 = ($38|0)!=($39|0);
  if ($40) {
   break;
  }
  $41 = $0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$41>>2] = $43;
  $44 = $4;
  $45 = $2;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + ($44)|0);
  HEAP32[$45>>2] = $47;
  $48 = $4;
  $49 = $3;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + ($48)|0);
  HEAP32[$49>>2] = $51;
 }
 if ((label|0) == 7) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _erode_charimage_2($inp,$out,$iw,$ih) {
 $inp = $inp|0;
 $out = $out|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $itr = 0, $otr = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $inp;
 $1 = $out;
 $2 = $iw;
 $3 = $ih;
 $4 = $0;
 $itr = $4;
 $5 = $1;
 $otr = $5;
 $6 = $1;
 $7 = $0;
 $8 = $2;
 $9 = $3;
 $10 = Math_imul($8, $9)|0;
 _memcpy(($6|0),($7|0),($10|0))|0;
 $row = 0;
 while(1) {
  $11 = $row;
  $12 = $3;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $col = 0;
  while(1) {
   $14 = $col;
   $15 = $2;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $itr;
   $18 = HEAP8[$17>>0]|0;
   $19 = ($18<<24>>24)!=(0);
   if ($19) {
    $20 = $itr;
    $21 = $col;
    $22 = (_get_west8_2($20,$21,1)|0);
    $23 = $22 << 24 >> 24;
    $24 = ($23|0)!=(0);
    if ($24) {
     $25 = $itr;
     $26 = $col;
     $27 = $2;
     $28 = (_get_east8_2($25,$26,$27,1)|0);
     $29 = $28 << 24 >> 24;
     $30 = ($29|0)!=(0);
     if ($30) {
      $31 = $itr;
      $32 = $row;
      $33 = $2;
      $34 = (_get_north8_2($31,$32,$33,1)|0);
      $35 = $34 << 24 >> 24;
      $36 = ($35|0)!=(0);
      if ($36) {
       $37 = $itr;
       $38 = $row;
       $39 = $2;
       $40 = $3;
       $41 = (_get_south8_2($37,$38,$39,$40,1)|0);
       $42 = $41 << 24 >> 24;
       $43 = ($42|0)!=(0);
       if (!($43)) {
        label = 10;
       }
      } else {
       label = 10;
      }
     } else {
      label = 10;
     }
    } else {
     label = 10;
    }
    if ((label|0) == 10) {
     label = 0;
     $44 = $otr;
     HEAP8[$44>>0] = 0;
    }
   }
   $45 = $itr;
   $46 = (($45) + 1|0);
   $itr = $46;
   $47 = $otr;
   $48 = (($47) + 1|0);
   $otr = $48;
   $49 = $col;
   $50 = (($49) + 1)|0;
   $col = $50;
  }
  $51 = $row;
  $52 = (($51) + 1)|0;
  $row = $52;
 }
 STACKTOP = sp;return;
}
function _get_west8_2($ptr,$col,$failcode) {
 $ptr = $ptr|0;
 $col = $col|0;
 $failcode = $failcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $ptr;
 $2 = $col;
 $3 = $failcode;
 $4 = $2;
 $5 = ($4|0)<(1);
 if ($5) {
  $6 = $3;
  $7 = $6&255;
  $0 = $7;
 } else {
  $8 = $1;
  $9 = (($8) + -1|0);
  $10 = HEAP8[$9>>0]|0;
  $0 = $10;
 }
 $11 = $0;
 STACKTOP = sp;return ($11|0);
}
function _get_east8_2($ptr,$col,$iw,$failcode) {
 $ptr = $ptr|0;
 $col = $col|0;
 $iw = $iw|0;
 $failcode = $failcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $ptr;
 $2 = $col;
 $3 = $iw;
 $4 = $failcode;
 $5 = $2;
 $6 = $3;
 $7 = (($6) - 1)|0;
 $8 = ($5|0)>=($7|0);
 if ($8) {
  $9 = $4;
  $10 = $9&255;
  $0 = $10;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 } else {
  $11 = $1;
  $12 = (($11) + 1|0);
  $13 = HEAP8[$12>>0]|0;
  $0 = $13;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 }
 return 0|0;
}
function _get_north8_2($ptr,$row,$iw,$failcode) {
 $ptr = $ptr|0;
 $row = $row|0;
 $iw = $iw|0;
 $failcode = $failcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $ptr;
 $2 = $row;
 $3 = $iw;
 $4 = $failcode;
 $5 = $2;
 $6 = ($5|0)<(1);
 if ($6) {
  $7 = $4;
  $8 = $7&255;
  $0 = $8;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 } else {
  $9 = $1;
  $10 = $3;
  $11 = (0 - ($10))|0;
  $12 = (($9) + ($11)|0);
  $13 = HEAP8[$12>>0]|0;
  $0 = $13;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 }
 return 0|0;
}
function _get_south8_2($ptr,$row,$iw,$ih,$failcode) {
 $ptr = $ptr|0;
 $row = $row|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $failcode = $failcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $ptr;
 $2 = $row;
 $3 = $iw;
 $4 = $ih;
 $5 = $failcode;
 $6 = $2;
 $7 = $4;
 $8 = (($7) - 1)|0;
 $9 = ($6|0)>=($8|0);
 if ($9) {
  $10 = $5;
  $11 = $10&255;
  $0 = $11;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 } else {
  $12 = $1;
  $13 = $3;
  $14 = (($12) + ($13)|0);
  $15 = HEAP8[$14>>0]|0;
  $0 = $15;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _dilate_charimage_2($inp,$out,$iw,$ih) {
 $inp = $inp|0;
 $out = $out|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $itr = 0, $otr = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $inp;
 $1 = $out;
 $2 = $iw;
 $3 = $ih;
 $4 = $0;
 $itr = $4;
 $5 = $1;
 $otr = $5;
 $6 = $1;
 $7 = $0;
 $8 = $2;
 $9 = $3;
 $10 = Math_imul($8, $9)|0;
 _memcpy(($6|0),($7|0),($10|0))|0;
 $row = 0;
 while(1) {
  $11 = $row;
  $12 = $3;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $col = 0;
  while(1) {
   $14 = $col;
   $15 = $2;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $itr;
   $18 = HEAP8[$17>>0]|0;
   $19 = ($18<<24>>24)!=(0);
   if (!($19)) {
    $20 = $itr;
    $21 = $col;
    $22 = (_get_west8_2($20,$21,0)|0);
    $23 = $22 << 24 >> 24;
    $24 = ($23|0)!=(0);
    if ($24) {
     label = 10;
    } else {
     $25 = $itr;
     $26 = $col;
     $27 = $2;
     $28 = (_get_east8_2($25,$26,$27,0)|0);
     $29 = $28 << 24 >> 24;
     $30 = ($29|0)!=(0);
     if ($30) {
      label = 10;
     } else {
      $31 = $itr;
      $32 = $row;
      $33 = $2;
      $34 = (_get_north8_2($31,$32,$33,0)|0);
      $35 = $34 << 24 >> 24;
      $36 = ($35|0)!=(0);
      if ($36) {
       label = 10;
      } else {
       $37 = $itr;
       $38 = $row;
       $39 = $2;
       $40 = $3;
       $41 = (_get_south8_2($37,$38,$39,$40,0)|0);
       $42 = $41 << 24 >> 24;
       $43 = ($42|0)!=(0);
       if ($43) {
        label = 10;
       }
      }
     }
    }
    if ((label|0) == 10) {
     label = 0;
     $44 = $otr;
     HEAP8[$44>>0] = 1;
    }
   }
   $45 = $itr;
   $46 = (($45) + 1|0);
   $itr = $46;
   $47 = $otr;
   $48 = (($47) + 1|0);
   $otr = $48;
   $49 = $col;
   $50 = (($49) + 1)|0;
   $col = $50;
  }
  $51 = $row;
  $52 = (($51) + 1)|0;
  $row = $52;
 }
 STACKTOP = sp;return;
}
function _remove_holes($minutiae,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $minutia = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $lfsparms;
 _print2log(5528,$vararg_buffer);
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $1;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 16;
   break;
  }
  $11 = $i;
  $12 = $1;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $minutia = $16;
  $17 = $minutia;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)==(0);
  if ($20) {
   $21 = $minutia;
   $22 = $5;
   $23 = (($22) + 256|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = $2;
   $26 = $3;
   $27 = $4;
   $28 = (_on_loop($21,$24,$25,$26,$27)|0);
   $ret = $28;
   $29 = $ret;
   $30 = ($29|0)==(1);
   if ($30) {
    label = 6;
   } else {
    $31 = $ret;
    $32 = ($31|0)==(2);
    if ($32) {
     label = 6;
    } else {
     $43 = $ret;
     $44 = ($43|0)==(0);
     if (!($44)) {
      label = 11;
      break;
     }
     $45 = $i;
     $46 = (($45) + 1)|0;
     $i = $46;
    }
   }
   if ((label|0) == 6) {
    label = 0;
    $33 = $minutia;
    $34 = HEAP32[$33>>2]|0;
    $35 = $minutia;
    $36 = (($35) + 4|0);
    $37 = HEAP32[$36>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $34;
    $vararg_ptr3 = (($vararg_buffer1) + 4|0);
    HEAP32[$vararg_ptr3>>2] = $37;
    _print2log(5552,$vararg_buffer1);
    $38 = $i;
    $39 = $1;
    $40 = (_remove_minutia($38,$39)|0);
    $ret = $40;
    $41 = ($40|0)!=(0);
    if ($41) {
     label = 7;
     break;
    }
   }
  } else {
   $48 = $i;
   $49 = (($48) + 1)|0;
   $i = $49;
  }
 }
 if ((label|0) == 7) {
  $42 = $ret;
  $0 = $42;
  $50 = $0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 11) {
  $47 = $ret;
  $0 = $47;
  $50 = $0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 16) {
  $0 = 0;
  $50 = $0;
  STACKTOP = sp;return ($50|0);
 }
 return 0|0;
}
function _remove_false_minutia_V2($minutiae,$bdata,$iw,$ih,$direction_map,$low_flow_map,$high_curve_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $direction_map = $direction_map|0;
 $low_flow_map = $low_flow_map|0;
 $high_curve_map = $high_curve_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $direction_map;
 $6 = $low_flow_map;
 $7 = $high_curve_map;
 $8 = $mw;
 $9 = $mh;
 $10 = $lfsparms;
 $11 = $1;
 $12 = $3;
 $13 = $4;
 $14 = (_sort_minutiae_y_x($11,$12,$13)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 do {
  if ($15) {
   $16 = $ret;
   $0 = $16;
  } else {
   $17 = $1;
   $18 = $2;
   $19 = $3;
   $20 = $4;
   $21 = $10;
   $22 = (_remove_islands_and_lakes($17,$18,$19,$20,$21)|0);
   $ret = $22;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = $ret;
    $0 = $24;
    break;
   }
   $25 = $1;
   $26 = $2;
   $27 = $3;
   $28 = $4;
   $29 = $10;
   $30 = (_remove_holes($25,$26,$27,$28,$29)|0);
   $ret = $30;
   $31 = ($30|0)!=(0);
   if ($31) {
    $32 = $ret;
    $0 = $32;
    break;
   }
   $33 = $1;
   $34 = $5;
   $35 = $8;
   $36 = $9;
   $37 = $10;
   $38 = (_remove_pointing_invblock_V2($33,$34,$35,$36,$37)|0);
   $ret = $38;
   $39 = ($38|0)!=(0);
   if ($39) {
    $40 = $ret;
    $0 = $40;
    break;
   }
   $41 = $1;
   $42 = $5;
   $43 = $8;
   $44 = $9;
   $45 = $10;
   $46 = (_remove_near_invblock_V2($41,$42,$43,$44,$45)|0);
   $ret = $46;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = $ret;
    $0 = $48;
    break;
   }
   $49 = $1;
   $50 = $2;
   $51 = $3;
   $52 = $4;
   $53 = $5;
   $54 = $8;
   $55 = $9;
   $56 = $10;
   $57 = (_remove_or_adjust_side_minutiae_V2($49,$50,$51,$52,$53,$54,$55,$56)|0);
   $ret = $57;
   $58 = ($57|0)!=(0);
   if ($58) {
    $59 = $ret;
    $0 = $59;
    break;
   }
   $60 = $1;
   $61 = $2;
   $62 = $3;
   $63 = $4;
   $64 = $10;
   $65 = (_remove_hooks($60,$61,$62,$63,$64)|0);
   $ret = $65;
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = $ret;
    $0 = $67;
    break;
   }
   $68 = $1;
   $69 = $2;
   $70 = $3;
   $71 = $4;
   $72 = $10;
   $73 = (_remove_overlaps($68,$69,$70,$71,$72)|0);
   $ret = $73;
   $74 = ($73|0)!=(0);
   if ($74) {
    $75 = $ret;
    $0 = $75;
    break;
   }
   $76 = $1;
   $77 = $2;
   $78 = $3;
   $79 = $4;
   $80 = $6;
   $81 = $8;
   $82 = $9;
   $83 = $10;
   $84 = (_remove_malformations($76,$77,$78,$79,$80,$81,$82,$83)|0);
   $ret = $84;
   $85 = ($84|0)!=(0);
   if ($85) {
    $86 = $ret;
    $0 = $86;
    break;
   }
   $87 = $1;
   $88 = $2;
   $89 = $3;
   $90 = $4;
   $91 = $5;
   $92 = $6;
   $93 = $7;
   $94 = $8;
   $95 = $9;
   $96 = $10;
   $97 = (_remove_pores_V2($87,$88,$89,$90,$91,$92,$93,$94,$95,$96)|0);
   $ret = $97;
   $98 = ($97|0)!=(0);
   if ($98) {
    $99 = $ret;
    $0 = $99;
    break;
   } else {
    $0 = 0;
    break;
   }
  }
 } while(0);
 $100 = $0;
 STACKTOP = sp;return ($100|0);
}
function _remove_islands_and_lakes($minutiae,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0.0, $146 = 0.0, $147 = 0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $delta_y = 0, $deltadir = 0, $dist = 0.0, $dist_thresh = 0, $f = 0, $full_ndirs = 0, $half_loop = 0, $i = 0, $loop_ex = 0;
 var $loop_ey = 0, $loop_x = 0, $loop_y = 0, $min_deltadir = 0, $minutia1 = 0, $minutia2 = 0, $nloop = 0, $qtr_ndirs = 0, $ret = 0, $s = 0, $to_remove = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer24 = 0, $vararg_buffer26 = 0;
 var $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer30 = 0, $vararg_buffer32 = 0, $vararg_buffer34 = 0, $vararg_buffer36 = 0, $vararg_buffer5 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer36 = sp + 152|0;
 $vararg_buffer34 = sp + 144|0;
 $vararg_buffer32 = sp + 136|0;
 $vararg_buffer30 = sp + 128|0;
 $vararg_buffer28 = sp + 120|0;
 $vararg_buffer26 = sp + 112|0;
 $vararg_buffer24 = sp + 104|0;
 $vararg_buffer22 = sp + 96|0;
 $vararg_buffer20 = sp + 88|0;
 $vararg_buffer18 = sp + 80|0;
 $vararg_buffer16 = sp + 72|0;
 $vararg_buffer14 = sp + 64|0;
 $vararg_buffer5 = sp + 32|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $loop_x = sp + 188|0;
 $loop_y = sp + 184|0;
 $loop_ex = sp + 180|0;
 $loop_ey = sp + 176|0;
 $nloop = sp + 172|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $lfsparms;
 _print2log(5784,$vararg_buffer);
 $6 = $5;
 $7 = (($6) + 240|0);
 $8 = HEAP32[$7>>2]|0;
 $dist_thresh = $8;
 $9 = $5;
 $10 = (($9) + 248|0);
 $11 = HEAP32[$10>>2]|0;
 $half_loop = $11;
 $12 = $1;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_calloc($14,4)|0);
 $to_remove = $15;
 $16 = $to_remove;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(5816|0),($vararg_buffer1|0))|0);
  $0 = -610;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 $19 = $5;
 $20 = (($19) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $21 << 1;
 $full_ndirs = $22;
 $23 = $5;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 >> 2;
 $qtr_ndirs = $26;
 $27 = $qtr_ndirs;
 $28 = ($27*3)|0;
 $29 = (($28) - 1)|0;
 $min_deltadir = $29;
 $f = 0;
 L5: while(1) {
  $30 = $f;
  $31 = $1;
  $32 = (($31) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) - 1)|0;
  $35 = ($30|0)<($34|0);
  if (!($35)) {
   label = 42;
   break;
  }
  $36 = $f;
  $37 = $to_remove;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0);
  if (!($40)) {
   _print2log(5640,$vararg_buffer3);
   $41 = $f;
   $42 = $1;
   $43 = (($42) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($41<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $minutia1 = $46;
   $47 = $f;
   $48 = (($47) + 1)|0;
   $s = $48;
   while(1) {
    $49 = $s;
    $50 = $1;
    $51 = (($50) + 4|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = ($49|0)<($52|0);
    if (!($53)) {
     break;
    }
    $54 = $s;
    $55 = $1;
    $56 = (($55) + 8|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + ($54<<2)|0);
    $59 = HEAP32[$58>>2]|0;
    $minutia2 = $59;
    $60 = $minutia2;
    $61 = (($60) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = $minutia1;
    $64 = (($63) + 32|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = ($62|0)==($65|0);
    if ($66) {
     $67 = $f;
     $68 = $minutia1;
     $69 = HEAP32[$68>>2]|0;
     $70 = $minutia1;
     $71 = (($70) + 4|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = $minutia1;
     $74 = (($73) + 32|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = $s;
     $77 = $minutia2;
     $78 = HEAP32[$77>>2]|0;
     $79 = $minutia2;
     $80 = (($79) + 4|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = $minutia2;
     $83 = (($82) + 32|0);
     $84 = HEAP32[$83>>2]|0;
     HEAP32[$vararg_buffer5>>2] = $67;
     $vararg_ptr7 = (($vararg_buffer5) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $69;
     $vararg_ptr8 = (($vararg_buffer5) + 8|0);
     HEAP32[$vararg_ptr8>>2] = $72;
     $vararg_ptr9 = (($vararg_buffer5) + 12|0);
     HEAP32[$vararg_ptr9>>2] = $75;
     $vararg_ptr10 = (($vararg_buffer5) + 16|0);
     HEAP32[$vararg_ptr10>>2] = $76;
     $vararg_ptr11 = (($vararg_buffer5) + 20|0);
     HEAP32[$vararg_ptr11>>2] = $78;
     $vararg_ptr12 = (($vararg_buffer5) + 24|0);
     HEAP32[$vararg_ptr12>>2] = $81;
     $vararg_ptr13 = (($vararg_buffer5) + 28|0);
     HEAP32[$vararg_ptr13>>2] = $84;
     _print2log(5648,$vararg_buffer5);
     $85 = $2;
     $86 = $minutia1;
     $87 = (($86) + 4|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = $3;
     $90 = Math_imul($88, $89)|0;
     $91 = (($85) + ($90)|0);
     $92 = $minutia1;
     $93 = HEAP32[$92>>2]|0;
     $94 = (($91) + ($93)|0);
     $95 = HEAP8[$94>>0]|0;
     $96 = $95&255;
     $97 = $minutia1;
     $98 = (($97) + 32|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = ($96|0)!=($99|0);
     if ($100) {
      label = 10;
      break;
     }
     $101 = $2;
     $102 = $minutia2;
     $103 = (($102) + 4|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = $3;
     $106 = Math_imul($104, $105)|0;
     $107 = (($101) + ($106)|0);
     $108 = $minutia2;
     $109 = HEAP32[$108>>2]|0;
     $110 = (($107) + ($109)|0);
     $111 = HEAP8[$110>>0]|0;
     $112 = $111&255;
     $113 = $minutia2;
     $114 = (($113) + 32|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = ($112|0)!=($115|0);
     if ($116) {
      $117 = $s;
      $118 = $to_remove;
      $119 = (($118) + ($117<<2)|0);
      HEAP32[$119>>2] = 1;
     }
     $120 = $s;
     $121 = $to_remove;
     $122 = (($121) + ($120<<2)|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = ($123|0)!=(0);
     if ($124) {
      _print2log(5640,$vararg_buffer36);
     } else {
      $125 = $minutia2;
      $126 = (($125) + 4|0);
      $127 = HEAP32[$126>>2]|0;
      $128 = $minutia1;
      $129 = (($128) + 4|0);
      $130 = HEAP32[$129>>2]|0;
      $131 = (($127) - ($130))|0;
      $delta_y = $131;
      $132 = $delta_y;
      $133 = $dist_thresh;
      $134 = ($132|0)<=($133|0);
      if (!($134)) {
       label = 35;
       break;
      }
      _print2log(5680,$vararg_buffer16);
      $135 = $minutia1;
      $136 = HEAP32[$135>>2]|0;
      $137 = $minutia1;
      $138 = (($137) + 4|0);
      $139 = HEAP32[$138>>2]|0;
      $140 = $minutia2;
      $141 = HEAP32[$140>>2]|0;
      $142 = $minutia2;
      $143 = (($142) + 4|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = (+_distance($136,$139,$141,$144));
      $dist = $145;
      $146 = $dist;
      $147 = $dist_thresh;
      $148 = (+($147|0));
      $149 = $146 <= $148;
      if ($149) {
       _print2log(5688,$vararg_buffer18);
       $150 = $minutia1;
       $151 = (($150) + 16|0);
       $152 = HEAP32[$151>>2]|0;
       $153 = $minutia2;
       $154 = (($153) + 16|0);
       $155 = HEAP32[$154>>2]|0;
       $156 = $full_ndirs;
       $157 = (_closest_dir_dist($152,$155,$156)|0);
       $deltadir = $157;
       $158 = ($157|0)==(-1);
       if ($158) {
        label = 17;
        break L5;
       }
       $161 = $deltadir;
       $162 = $min_deltadir;
       $163 = ($161|0)>($162|0);
       if ($163) {
        _print2log(5744,$vararg_buffer22);
        $164 = $minutia1;
        $165 = $minutia2;
        $166 = $half_loop;
        $167 = $2;
        $168 = $3;
        $169 = $4;
        $170 = (_on_island_lake($loop_x,$loop_y,$loop_ex,$loop_ey,$nloop,$164,$165,$166,$167,$168,$169)|0);
        $ret = $170;
        $171 = $ret;
        $172 = ($171|0)==(1);
        if ($172) {
         _print2log(5768,$vararg_buffer24);
         $173 = HEAP32[$loop_x>>2]|0;
         $174 = HEAP32[$loop_y>>2]|0;
         $175 = HEAP32[$nloop>>2]|0;
         $176 = $2;
         $177 = $3;
         $178 = $4;
         $179 = (_fill_loop($173,$174,$175,$176,$177,$178)|0);
         $ret = $179;
         $180 = ($179|0)!=(0);
         if ($180) {
          label = 21;
          break L5;
         }
         $187 = $f;
         $188 = $to_remove;
         $189 = (($188) + ($187<<2)|0);
         HEAP32[$189>>2] = 1;
         $190 = $s;
         $191 = $to_remove;
         $192 = (($191) + ($190<<2)|0);
         HEAP32[$192>>2] = 1;
         $193 = HEAP32[$loop_x>>2]|0;
         $194 = HEAP32[$loop_y>>2]|0;
         $195 = HEAP32[$loop_ex>>2]|0;
         $196 = HEAP32[$loop_ey>>2]|0;
         _free_contour($193,$194,$195,$196);
        } else {
         $197 = $ret;
         $198 = ($197|0)==(2);
         if ($198) {
          label = 24;
          break;
         }
         $202 = $ret;
         $203 = ($202|0)<(0);
         if ($203) {
          label = 26;
          break L5;
         }
         _print2log(5640,$vararg_buffer28);
        }
       } else {
        _print2log(5640,$vararg_buffer30);
       }
      } else {
       _print2log(5640,$vararg_buffer32);
      }
     }
    }
    $206 = $s;
    $207 = (($206) + 1)|0;
    $s = $207;
   }
   if ((label|0) == 10) {
    label = 0;
    _print2log(5640,$vararg_buffer14);
   }
   else if ((label|0) == 24) {
    label = 0;
    _print2log(5760,$vararg_buffer26);
    $199 = $f;
    $200 = $to_remove;
    $201 = (($200) + ($199<<2)|0);
    HEAP32[$201>>2] = 1;
   }
   else if ((label|0) == 35) {
    label = 0;
    _print2log(5640,$vararg_buffer34);
   }
  }
  $208 = $f;
  $209 = (($208) + 1)|0;
  $f = $209;
 }
 if ((label|0) == 17) {
  $159 = $to_remove;
  _free($159);
  $160 = HEAP32[_stderr>>2]|0;
  (_fprintf(($160|0),(5872|0),($vararg_buffer20|0))|0);
  $0 = -611;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 21) {
  $181 = HEAP32[$loop_x>>2]|0;
  $182 = HEAP32[$loop_y>>2]|0;
  $183 = HEAP32[$loop_ex>>2]|0;
  $184 = HEAP32[$loop_ey>>2]|0;
  _free_contour($181,$182,$183,$184);
  $185 = $to_remove;
  _free($185);
  $186 = $ret;
  $0 = $186;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 26) {
  $204 = $to_remove;
  _free($204);
  $205 = $ret;
  $0 = $205;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 42) {
  $210 = $1;
  $211 = (($210) + 4|0);
  $212 = HEAP32[$211>>2]|0;
  $213 = (($212) - 1)|0;
  $i = $213;
  while(1) {
   $214 = $i;
   $215 = ($214|0)>=(0);
   if (!($215)) {
    label = 50;
    break;
   }
   $216 = $i;
   $217 = $to_remove;
   $218 = (($217) + ($216<<2)|0);
   $219 = HEAP32[$218>>2]|0;
   $220 = ($219|0)!=(0);
   if ($220) {
    $221 = $i;
    $222 = $1;
    $223 = (_remove_minutia($221,$222)|0);
    $ret = $223;
    $224 = ($223|0)!=(0);
    if ($224) {
     label = 46;
     break;
    }
   }
   $227 = $i;
   $228 = (($227) + -1)|0;
   $i = $228;
  }
  if ((label|0) == 46) {
   $225 = $to_remove;
   _free($225);
   $226 = $ret;
   $0 = $226;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
  else if ((label|0) == 50) {
   $229 = $to_remove;
   _free($229);
   $0 = 0;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
 }
 return 0|0;
}
function _remove_pointing_invblock_V2($minutiae,$direction_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $direction_map = $direction_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0, $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0;
 var $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0, $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0;
 var $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0.0, $78 = 0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0;
 var $87 = 0.0, $88 = 0, $89 = 0, $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bx = 0, $by = 0, $delta_x = 0, $delta_y = 0, $dmapval = 0, $dx = 0.0;
 var $dy = 0.0, $i = 0, $minutia = 0, $nx = 0, $ny = 0, $pi_factor = 0.0, $ret = 0, $theta = 0.0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer1 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $1 = $minutiae;
 $2 = $direction_map;
 $3 = $mw;
 $4 = $mh;
 $5 = $lfsparms;
 _print2log(6352,$vararg_buffer);
 $6 = $5;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (+($8|0));
 $10 = 3.141592653589793116 / $9;
 $pi_factor = $10;
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $1;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   label = 33;
   break;
  }
  $16 = $i;
  $17 = $1;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $minutia = $21;
  $22 = $minutia;
  $23 = (($22) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (+($24|0));
  $26 = $pi_factor;
  $27 = $25 * $26;
  $theta = $27;
  $28 = $theta;
  $29 = (+Math_sin((+$28)));
  $30 = $5;
  $31 = (($30) + 252|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (+($32|0));
  $34 = $29 * $33;
  $dx = $34;
  $35 = $theta;
  $36 = (+Math_cos((+$35)));
  $37 = $5;
  $38 = (($37) + 252|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (+($39|0));
  $41 = $36 * $40;
  $dy = $41;
  $42 = $dx;
  $43 = $42 < 0.0;
  if ($43) {
   $44 = $dx;
   $45 = $44 * 16384.0;
   $46 = $45 - 0.5;
   $47 = (~~(($46)));
   $48 = (+($47|0));
   $49 = $48 / 16384.0;
   $56 = $49;
  } else {
   $50 = $dx;
   $51 = $50 * 16384.0;
   $52 = $51 + 0.5;
   $53 = (~~(($52)));
   $54 = (+($53|0));
   $55 = $54 / 16384.0;
   $56 = $55;
  }
  $dx = $56;
  $57 = $dy;
  $58 = $57 < 0.0;
  if ($58) {
   $59 = $dy;
   $60 = $59 * 16384.0;
   $61 = $60 - 0.5;
   $62 = (~~(($61)));
   $63 = (+($62|0));
   $64 = $63 / 16384.0;
   $71 = $64;
  } else {
   $65 = $dy;
   $66 = $65 * 16384.0;
   $67 = $66 + 0.5;
   $68 = (~~(($67)));
   $69 = (+($68|0));
   $70 = $69 / 16384.0;
   $71 = $70;
  }
  $dy = $71;
  $72 = $dx;
  $73 = $72 < 0.0;
  if ($73) {
   $74 = $dx;
   $75 = $74 - 0.5;
   $79 = $75;
  } else {
   $76 = $dx;
   $77 = $76 + 0.5;
   $79 = $77;
  }
  $78 = (~~(($79)));
  $delta_x = $78;
  $80 = $dy;
  $81 = $80 < 0.0;
  if ($81) {
   $82 = $dy;
   $83 = $82 - 0.5;
   $87 = $83;
  } else {
   $84 = $dy;
   $85 = $84 + 0.5;
   $87 = $85;
  }
  $86 = (~~(($87)));
  $delta_y = $86;
  $88 = $minutia;
  $89 = HEAP32[$88>>2]|0;
  $90 = $delta_x;
  $91 = (($89) - ($90))|0;
  $nx = $91;
  $92 = $minutia;
  $93 = (($92) + 4|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $delta_y;
  $96 = (($94) + ($95))|0;
  $ny = $96;
  $97 = $nx;
  $98 = $5;
  $99 = (($98) + 8|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = (($97|0) / ($100|0))&-1;
  $bx = $101;
  $102 = $ny;
  $103 = $5;
  $104 = (($103) + 8|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = (($102|0) / ($105|0))&-1;
  $by = $106;
  $107 = $bx;
  $108 = (0)>($107|0);
  if ($108) {
   $110 = 0;
  } else {
   $109 = $bx;
   $110 = $109;
  }
  $bx = $110;
  $111 = $3;
  $112 = (($111) - 1)|0;
  $113 = $bx;
  $114 = ($112|0)<($113|0);
  if ($114) {
   $115 = $3;
   $116 = (($115) - 1)|0;
   $118 = $116;
  } else {
   $117 = $bx;
   $118 = $117;
  }
  $bx = $118;
  $119 = $by;
  $120 = (0)>($119|0);
  if ($120) {
   $122 = 0;
  } else {
   $121 = $by;
   $122 = $121;
  }
  $by = $122;
  $123 = $4;
  $124 = (($123) - 1)|0;
  $125 = $by;
  $126 = ($124|0)<($125|0);
  if ($126) {
   $127 = $4;
   $128 = (($127) - 1)|0;
   $130 = $128;
  } else {
   $129 = $by;
   $130 = $129;
  }
  $by = $130;
  $131 = $2;
  $132 = $by;
  $133 = $3;
  $134 = Math_imul($132, $133)|0;
  $135 = (($131) + ($134<<2)|0);
  $136 = $bx;
  $137 = (($135) + ($136<<2)|0);
  $138 = HEAP32[$137>>2]|0;
  $dmapval = $138;
  $139 = $dmapval;
  $140 = ($139|0)==(-1);
  if ($140) {
   $141 = $minutia;
   $142 = HEAP32[$141>>2]|0;
   $143 = $minutia;
   $144 = (($143) + 4|0);
   $145 = HEAP32[$144>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $142;
   $vararg_ptr3 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $145;
   _print2log(5552,$vararg_buffer1);
   $146 = $i;
   $147 = $1;
   $148 = (_remove_minutia($146,$147)|0);
   $ret = $148;
   $149 = ($148|0)!=(0);
   if ($149) {
    label = 29;
    break;
   }
  } else {
   $151 = $i;
   $152 = (($151) + 1)|0;
   $i = $152;
  }
 }
 if ((label|0) == 29) {
  $150 = $ret;
  $0 = $150;
  $153 = $0;
  STACKTOP = sp;return ($153|0);
 }
 else if ((label|0) == 33) {
  $0 = 0;
  $153 = $0;
  STACKTOP = sp;return ($153|0);
 }
 return 0|0;
}
function _remove_near_invblock_V2($minutiae,$direction_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $direction_map = $direction_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $bx = 0, $by = 0, $ebi = 0, $hi_margin = 0, $i = 0, $ix = 0, $iy = 0, $lo_margin = 0, $minutia = 0, $nbx = 0, $nby = 0, $ni = 0, $nvalid = 0, $px = 0, $py = 0, $removed = 0, $ret = 0, $sbi = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $minutiae;
 $2 = $direction_map;
 $3 = $mw;
 $4 = $mh;
 $5 = $lfsparms;
 _print2log(6048,$vararg_buffer);
 $6 = $5;
 $7 = (($6) + 264|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $5;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 >> 1;
 $13 = ($8|0)>($12|0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  (_fprintf(($14|0),(6280|0),($vararg_buffer1|0))|0);
  $0 = -620;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $15 = $5;
 $16 = (($15) + 264|0);
 $17 = HEAP32[$16>>2]|0;
 $lo_margin = $17;
 $18 = $5;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $5;
 $22 = (($21) + 264|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($20) - ($23))|0;
 $25 = (($24) - 1)|0;
 $hi_margin = $25;
 $i = 0;
 while(1) {
  $26 = $i;
  $27 = $1;
  $28 = (($27) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)<($29|0);
  if (!($30)) {
   label = 41;
   break;
  }
  $31 = $i;
  $32 = $1;
  $33 = (($32) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31<<2)|0);
  $36 = HEAP32[$35>>2]|0;
  $minutia = $36;
  $37 = $minutia;
  $38 = HEAP32[$37>>2]|0;
  $39 = $5;
  $40 = (($39) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($38|0) / ($41|0))&-1;
  $bx = $42;
  $43 = $minutia;
  $44 = (($43) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $5;
  $47 = (($46) + 8|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($45|0) / ($48|0))&-1;
  $by = $49;
  $50 = $minutia;
  $51 = HEAP32[$50>>2]|0;
  $52 = $5;
  $53 = (($52) + 8|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($51|0) % ($54|0))&-1;
  $px = $55;
  $56 = $minutia;
  $57 = (($56) + 4|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = $5;
  $60 = (($59) + 8|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($58|0) % ($61|0))&-1;
  $py = $62;
  $63 = $px;
  $64 = $lo_margin;
  $65 = ($63|0)<($64|0);
  if ($65) {
   $ix = 0;
  } else {
   $66 = $px;
   $67 = $hi_margin;
   $68 = ($66|0)>($67|0);
   if ($68) {
    $ix = 2;
   } else {
    $ix = 1;
   }
  }
  $69 = $py;
  $70 = $lo_margin;
  $71 = ($69|0)<($70|0);
  if ($71) {
   $iy = 0;
  } else {
   $72 = $py;
   $73 = $hi_margin;
   $74 = ($72|0)>($73|0);
   if ($74) {
    $iy = 2;
   } else {
    $iy = 1;
   }
  }
  $removed = 0;
  $75 = $ix;
  $76 = ($75|0)!=(1);
  if ($76) {
   label = 19;
  } else {
   $77 = $iy;
   $78 = ($77|0)!=(1);
   if ($78) {
    label = 19;
   }
  }
  if ((label|0) == 19) {
   label = 0;
   $79 = $iy;
   $80 = ($79*3)|0;
   $81 = (6120 + ($80<<2)|0);
   $82 = $ix;
   $83 = (($81) + ($82<<2)|0);
   $84 = HEAP32[$83>>2]|0;
   $sbi = $84;
   $85 = $iy;
   $86 = ($85*3)|0;
   $87 = (6160 + ($86<<2)|0);
   $88 = $ix;
   $89 = (($87) + ($88<<2)|0);
   $90 = HEAP32[$89>>2]|0;
   $ebi = $90;
   $91 = $sbi;
   $ni = $91;
   while(1) {
    $92 = $ni;
    $93 = $ebi;
    $94 = ($92|0)<=($93|0);
    if (!($94)) {
     break;
    }
    $95 = $bx;
    $96 = $ni;
    $97 = (6200 + ($96<<2)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = (($95) + ($98))|0;
    $nbx = $99;
    $100 = $by;
    $101 = $ni;
    $102 = (6240 + ($101<<2)|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($100) + ($103))|0;
    $nby = $104;
    $105 = $nbx;
    $106 = ($105|0)<(0);
    if ($106) {
     label = 25;
     break;
    }
    $107 = $nbx;
    $108 = $3;
    $109 = ($107|0)>=($108|0);
    if ($109) {
     label = 25;
     break;
    }
    $110 = $nby;
    $111 = ($110|0)<(0);
    if ($111) {
     label = 25;
     break;
    }
    $112 = $nby;
    $113 = $4;
    $114 = ($112|0)>=($113|0);
    if ($114) {
     label = 25;
     break;
    }
    $125 = $2;
    $126 = $nby;
    $127 = $3;
    $128 = Math_imul($126, $127)|0;
    $129 = (($125) + ($128<<2)|0);
    $130 = $nbx;
    $131 = (($129) + ($130<<2)|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = ($132|0)==(-1);
    if ($133) {
     $134 = $2;
     $135 = $nbx;
     $136 = $nby;
     $137 = $3;
     $138 = $4;
     $139 = (_num_valid_8nbrs($134,$135,$136,$137,$138)|0);
     $nvalid = $139;
     $140 = $nvalid;
     $141 = $5;
     $142 = (($141) + 268|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = ($140|0)<($143|0);
     if ($144) {
      label = 30;
      break;
     }
    }
    $155 = $ni;
    $156 = (($155) + 1)|0;
    $ni = $156;
   }
   if ((label|0) == 25) {
    label = 0;
    $115 = $minutia;
    $116 = HEAP32[$115>>2]|0;
    $117 = $minutia;
    $118 = (($117) + 4|0);
    $119 = HEAP32[$118>>2]|0;
    HEAP32[$vararg_buffer3>>2] = $116;
    $vararg_ptr5 = (($vararg_buffer3) + 4|0);
    HEAP32[$vararg_ptr5>>2] = $119;
    _print2log(6088,$vararg_buffer3);
    $120 = $i;
    $121 = $1;
    $122 = (_remove_minutia($120,$121)|0);
    $ret = $122;
    $123 = ($122|0)!=(0);
    if ($123) {
     label = 26;
     break;
    }
    $removed = 1;
   }
   else if ((label|0) == 30) {
    label = 0;
    $145 = $minutia;
    $146 = HEAP32[$145>>2]|0;
    $147 = $minutia;
    $148 = (($147) + 4|0);
    $149 = HEAP32[$148>>2]|0;
    HEAP32[$vararg_buffer6>>2] = $146;
    $vararg_ptr9 = (($vararg_buffer6) + 4|0);
    HEAP32[$vararg_ptr9>>2] = $149;
    _print2log(6104,$vararg_buffer6);
    $150 = $i;
    $151 = $1;
    $152 = (_remove_minutia($150,$151)|0);
    $ret = $152;
    $153 = ($152|0)!=(0);
    if ($153) {
     label = 31;
     break;
    }
    $removed = 1;
   }
  }
  $157 = $removed;
  $158 = ($157|0)!=(0);
  if (!($158)) {
   $159 = $i;
   $160 = (($159) + 1)|0;
   $i = $160;
  }
 }
 if ((label|0) == 26) {
  $124 = $ret;
  $0 = $124;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 else if ((label|0) == 31) {
  $154 = $ret;
  $0 = $154;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 else if ((label|0) == 41) {
  $0 = 0;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 return 0|0;
}
function _remove_or_adjust_side_minutiae_V2($minutiae,$bdata,$iw,$ih,$direction_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $direction_map = $direction_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0.0, $103 = 0, $104 = 0.0, $105 = 0.0, $106 = 0.0, $107 = 0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0.0, $113 = 0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0.0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0.0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0;
 var $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0.0;
 var $bx = 0, $by = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $cos_theta = 0.0, $drot_y = 0.0, $i = 0, $j = 0, $minloc = 0, $minmax_alloc = 0, $minmax_i = 0, $minmax_num = 0, $minmax_type = 0, $minmax_val = 0, $minutia = 0, $ncontour = 0, $pi_factor = 0.0, $ret = 0;
 var $rot_y = 0, $sin_theta = 0.0, $theta = 0.0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer26 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer26 = sp + 112|0;
 $vararg_buffer22 = sp + 104|0;
 $vararg_buffer20 = sp + 96|0;
 $vararg_buffer16 = sp + 88|0;
 $vararg_buffer12 = sp + 80|0;
 $vararg_buffer10 = sp + 72|0;
 $vararg_buffer6 = sp + 64|0;
 $vararg_buffer3 = sp + 56|0;
 $vararg_buffer1 = sp + 48|0;
 $vararg_buffer = sp + 40|0;
 $contour_x = sp + 172|0;
 $contour_y = sp + 168|0;
 $contour_ex = sp + 164|0;
 $contour_ey = sp + 160|0;
 $ncontour = sp + 156|0;
 $minmax_val = sp + 144|0;
 $minmax_i = sp + 140|0;
 $minmax_type = sp + 136|0;
 $minmax_alloc = sp + 132|0;
 $minmax_num = sp + 128|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $direction_map;
 $6 = $mw;
 $7 = $mh;
 $8 = $lfsparms;
 _print2log(6720,$vararg_buffer);
 $9 = $8;
 $10 = (($9) + 260|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 << 1;
 $13 = (($12) + 1)|0;
 $14 = $13<<2;
 $15 = (_malloc($14)|0);
 $rot_y = $15;
 $16 = $rot_y;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(6752|0),($vararg_buffer1|0))|0);
  $0 = -630;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 $19 = $8;
 $20 = (($19) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (+($21|0));
 $23 = 3.141592653589793116 / $22;
 $pi_factor = $23;
 $i = 0;
 while(1) {
  $24 = $i;
  $25 = $1;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($24|0)<($27|0);
  if (!($28)) {
   label = 58;
   break;
  }
  $29 = $i;
  $30 = $1;
  $31 = (($30) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29<<2)|0);
  $34 = HEAP32[$33>>2]|0;
  $minutia = $34;
  $35 = $8;
  $36 = (($35) + 260|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $minutia;
  $39 = HEAP32[$38>>2]|0;
  $40 = $minutia;
  $41 = (($40) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $minutia;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $minutia;
  $47 = (($46) + 12|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $2;
  $50 = $3;
  $51 = $4;
  $52 = (_get_centered_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$37,$39,$42,$45,$48,$49,$50,$51)|0);
  $ret = $52;
  $53 = $ret;
  $54 = ($53|0)<(0);
  if ($54) {
   label = 6;
   break;
  }
  $57 = $ret;
  $58 = ($57|0)==(1);
  if ($58) {
   label = 10;
  } else {
   $59 = $ret;
   $60 = ($59|0)==(2);
   if ($60) {
    label = 10;
   } else {
    $61 = $ret;
    $62 = ($61|0)==(3);
    if ($62) {
     label = 10;
    } else {
     $74 = $minutia;
     $75 = (($74) + 16|0);
     $76 = HEAP32[$75>>2]|0;
     $77 = (+($76|0));
     $78 = $pi_factor;
     $79 = $77 * $78;
     $theta = $79;
     $80 = $theta;
     $81 = (+Math_sin((+$80)));
     $sin_theta = $81;
     $82 = $theta;
     $83 = (+Math_cos((+$82)));
     $cos_theta = $83;
     $j = 0;
     while(1) {
      $84 = $j;
      $85 = HEAP32[$ncontour>>2]|0;
      $86 = ($84|0)<($85|0);
      if (!($86)) {
       break;
      }
      $87 = $j;
      $88 = HEAP32[$contour_x>>2]|0;
      $89 = (($88) + ($87<<2)|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (+($90|0));
      $92 = $sin_theta;
      $93 = $91 * $92;
      $94 = $j;
      $95 = HEAP32[$contour_y>>2]|0;
      $96 = (($95) + ($94<<2)|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = (+($97|0));
      $99 = $cos_theta;
      $100 = $98 * $99;
      $101 = $93 - $100;
      $drot_y = $101;
      $102 = $drot_y;
      $103 = $102 < 0.0;
      if ($103) {
       $104 = $drot_y;
       $105 = $104 * 16384.0;
       $106 = $105 - 0.5;
       $107 = (~~(($106)));
       $108 = (+($107|0));
       $109 = $108 / 16384.0;
       $116 = $109;
      } else {
       $110 = $drot_y;
       $111 = $110 * 16384.0;
       $112 = $111 + 0.5;
       $113 = (~~(($112)));
       $114 = (+($113|0));
       $115 = $114 / 16384.0;
       $116 = $115;
      }
      $drot_y = $116;
      $117 = $drot_y;
      $118 = $117 < 0.0;
      if ($118) {
       $119 = $drot_y;
       $120 = $119 - 0.5;
       $124 = $120;
      } else {
       $121 = $drot_y;
       $122 = $121 + 0.5;
       $124 = $122;
      }
      $123 = (~~(($124)));
      $125 = $j;
      $126 = $rot_y;
      $127 = (($126) + ($125<<2)|0);
      HEAP32[$127>>2] = $123;
      $128 = $j;
      $129 = (($128) + 1)|0;
      $j = $129;
     }
     $130 = $rot_y;
     $131 = HEAP32[$ncontour>>2]|0;
     $132 = (_minmaxs($minmax_val,$minmax_type,$minmax_i,$minmax_alloc,$minmax_num,$130,$131)|0);
     $ret = $132;
     $133 = ($132|0)!=(0);
     if ($133) {
      label = 24;
      break;
     }
     $140 = HEAP32[$minmax_num>>2]|0;
     $141 = ($140|0)==(1);
     if ($141) {
      $142 = HEAP32[$minmax_type>>2]|0;
      $143 = HEAP32[$142>>2]|0;
      $144 = ($143|0)==(-1);
      if ($144) {
       $145 = $minutia;
       $146 = HEAP32[$145>>2]|0;
       $147 = $minutia;
       $148 = (($147) + 4|0);
       $149 = HEAP32[$148>>2]|0;
       HEAP32[$vararg_buffer6>>2] = $146;
       $vararg_ptr9 = (($vararg_buffer6) + 4|0);
       HEAP32[$vararg_ptr9>>2] = $149;
       _print2log(6600,$vararg_buffer6);
       $150 = HEAP32[$minmax_i>>2]|0;
       $151 = HEAP32[$150>>2]|0;
       $152 = HEAP32[$contour_x>>2]|0;
       $153 = (($152) + ($151<<2)|0);
       $154 = HEAP32[$153>>2]|0;
       $155 = $minutia;
       HEAP32[$155>>2] = $154;
       $156 = HEAP32[$minmax_i>>2]|0;
       $157 = HEAP32[$156>>2]|0;
       $158 = HEAP32[$contour_y>>2]|0;
       $159 = (($158) + ($157<<2)|0);
       $160 = HEAP32[$159>>2]|0;
       $161 = $minutia;
       $162 = (($161) + 4|0);
       HEAP32[$162>>2] = $160;
       $163 = HEAP32[$minmax_i>>2]|0;
       $164 = HEAP32[$163>>2]|0;
       $165 = HEAP32[$contour_ex>>2]|0;
       $166 = (($165) + ($164<<2)|0);
       $167 = HEAP32[$166>>2]|0;
       $168 = $minutia;
       $169 = (($168) + 8|0);
       HEAP32[$169>>2] = $167;
       $170 = HEAP32[$minmax_i>>2]|0;
       $171 = HEAP32[$170>>2]|0;
       $172 = HEAP32[$contour_ey>>2]|0;
       $173 = (($172) + ($171<<2)|0);
       $174 = HEAP32[$173>>2]|0;
       $175 = $minutia;
       $176 = (($175) + 12|0);
       HEAP32[$176>>2] = $174;
       $177 = $minutia;
       $178 = HEAP32[$177>>2]|0;
       $179 = $8;
       $180 = (($179) + 8|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = (($178|0) / ($181|0))&-1;
       $bx = $182;
       $183 = $minutia;
       $184 = (($183) + 4|0);
       $185 = HEAP32[$184>>2]|0;
       $186 = $8;
       $187 = (($186) + 8|0);
       $188 = HEAP32[$187>>2]|0;
       $189 = (($185|0) / ($188|0))&-1;
       $by = $189;
       $190 = $5;
       $191 = $by;
       $192 = $6;
       $193 = Math_imul($191, $192)|0;
       $194 = (($190) + ($193<<2)|0);
       $195 = $bx;
       $196 = (($194) + ($195<<2)|0);
       $197 = HEAP32[$196>>2]|0;
       $198 = ($197|0)==(-1);
       if ($198) {
        $199 = $i;
        $200 = $1;
        $201 = (_remove_minutia($199,$200)|0);
        $ret = $201;
        $202 = ($201|0)!=(0);
        if ($202) {
         label = 29;
         break;
        }
        _print2log(6816,$vararg_buffer10);
       } else {
        $214 = $i;
        $215 = (($214) + 1)|0;
        $i = $215;
        $216 = $minutia;
        $217 = HEAP32[$216>>2]|0;
        $218 = $minutia;
        $219 = (($218) + 4|0);
        $220 = HEAP32[$219>>2]|0;
        HEAP32[$vararg_buffer12>>2] = $217;
        $vararg_ptr15 = (($vararg_buffer12) + 4|0);
        HEAP32[$vararg_ptr15>>2] = $220;
        _print2log(6824,$vararg_buffer12);
       }
      } else {
       label = 35;
      }
     } else {
      label = 35;
     }
     if ((label|0) == 35) {
      label = 0;
      $221 = HEAP32[$minmax_num>>2]|0;
      $222 = ($221|0)==(3);
      if ($222) {
       $223 = HEAP32[$minmax_type>>2]|0;
       $224 = HEAP32[$223>>2]|0;
       $225 = ($224|0)==(-1);
       if ($225) {
        $226 = HEAP32[$minmax_val>>2]|0;
        $227 = HEAP32[$226>>2]|0;
        $228 = HEAP32[$minmax_val>>2]|0;
        $229 = (($228) + 8|0);
        $230 = HEAP32[$229>>2]|0;
        $231 = ($227|0)<($230|0);
        if ($231) {
         $232 = HEAP32[$minmax_i>>2]|0;
         $233 = HEAP32[$232>>2]|0;
         $minloc = $233;
        } else {
         $234 = HEAP32[$minmax_i>>2]|0;
         $235 = (($234) + 8|0);
         $236 = HEAP32[$235>>2]|0;
         $minloc = $236;
        }
        $237 = $minutia;
        $238 = HEAP32[$237>>2]|0;
        $239 = $minutia;
        $240 = (($239) + 4|0);
        $241 = HEAP32[$240>>2]|0;
        HEAP32[$vararg_buffer16>>2] = $238;
        $vararg_ptr19 = (($vararg_buffer16) + 4|0);
        HEAP32[$vararg_ptr19>>2] = $241;
        _print2log(6600,$vararg_buffer16);
        $242 = $minloc;
        $243 = HEAP32[$contour_x>>2]|0;
        $244 = (($243) + ($242<<2)|0);
        $245 = HEAP32[$244>>2]|0;
        $246 = $minutia;
        HEAP32[$246>>2] = $245;
        $247 = $minloc;
        $248 = HEAP32[$contour_y>>2]|0;
        $249 = (($248) + ($247<<2)|0);
        $250 = HEAP32[$249>>2]|0;
        $251 = $minutia;
        $252 = (($251) + 4|0);
        HEAP32[$252>>2] = $250;
        $253 = $minloc;
        $254 = HEAP32[$contour_ex>>2]|0;
        $255 = (($254) + ($253<<2)|0);
        $256 = HEAP32[$255>>2]|0;
        $257 = $minutia;
        $258 = (($257) + 8|0);
        HEAP32[$258>>2] = $256;
        $259 = $minloc;
        $260 = HEAP32[$contour_ey>>2]|0;
        $261 = (($260) + ($259<<2)|0);
        $262 = HEAP32[$261>>2]|0;
        $263 = $minutia;
        $264 = (($263) + 12|0);
        HEAP32[$264>>2] = $262;
        $265 = $minutia;
        $266 = HEAP32[$265>>2]|0;
        $267 = $8;
        $268 = (($267) + 8|0);
        $269 = HEAP32[$268>>2]|0;
        $270 = (($266|0) / ($269|0))&-1;
        $bx = $270;
        $271 = $minutia;
        $272 = (($271) + 4|0);
        $273 = HEAP32[$272>>2]|0;
        $274 = $8;
        $275 = (($274) + 8|0);
        $276 = HEAP32[$275>>2]|0;
        $277 = (($273|0) / ($276|0))&-1;
        $by = $277;
        $278 = $5;
        $279 = $by;
        $280 = $6;
        $281 = Math_imul($279, $280)|0;
        $282 = (($278) + ($281<<2)|0);
        $283 = $bx;
        $284 = (($282) + ($283<<2)|0);
        $285 = HEAP32[$284>>2]|0;
        $286 = ($285|0)==(-1);
        if ($286) {
         $287 = $i;
         $288 = $1;
         $289 = (_remove_minutia($287,$288)|0);
         $ret = $289;
         $290 = ($289|0)!=(0);
         if ($290) {
          label = 42;
          break;
         }
         _print2log(6840,$vararg_buffer20);
        } else {
         $302 = $i;
         $303 = (($302) + 1)|0;
         $i = $303;
         $304 = $minutia;
         $305 = HEAP32[$304>>2]|0;
         $306 = $minutia;
         $307 = (($306) + 4|0);
         $308 = HEAP32[$307>>2]|0;
         HEAP32[$vararg_buffer22>>2] = $305;
         $vararg_ptr25 = (($vararg_buffer22) + 4|0);
         HEAP32[$vararg_ptr25>>2] = $308;
         _print2log(6848,$vararg_buffer22);
        }
       } else {
        label = 48;
       }
      } else {
       label = 48;
      }
      if ((label|0) == 48) {
       label = 0;
       $309 = $minutia;
       $310 = HEAP32[$309>>2]|0;
       $311 = $minutia;
       $312 = (($311) + 4|0);
       $313 = HEAP32[$312>>2]|0;
       HEAP32[$vararg_buffer26>>2] = $310;
       $vararg_ptr29 = (($vararg_buffer26) + 4|0);
       HEAP32[$vararg_ptr29>>2] = $313;
       _print2log(6864,$vararg_buffer26);
       $314 = $i;
       $315 = $1;
       $316 = (_remove_minutia($314,$315)|0);
       $ret = $316;
       $317 = ($316|0)!=(0);
       if ($317) {
        label = 49;
        break;
       }
      }
     }
     $329 = HEAP32[$contour_x>>2]|0;
     $330 = HEAP32[$contour_y>>2]|0;
     $331 = HEAP32[$contour_ex>>2]|0;
     $332 = HEAP32[$contour_ey>>2]|0;
     _free_contour($329,$330,$331,$332);
     $333 = HEAP32[$minmax_alloc>>2]|0;
     $334 = ($333|0)>(0);
     if ($334) {
      $335 = HEAP32[$minmax_val>>2]|0;
      _free($335);
      $336 = HEAP32[$minmax_type>>2]|0;
      _free($336);
      $337 = HEAP32[$minmax_i>>2]|0;
      _free($337);
     }
    }
   }
  }
  if ((label|0) == 10) {
   label = 0;
   $63 = $minutia;
   $64 = HEAP32[$63>>2]|0;
   $65 = $minutia;
   $66 = (($65) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $64;
   $vararg_ptr5 = (($vararg_buffer3) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $67;
   _print2log(6088,$vararg_buffer3);
   $68 = $i;
   $69 = $1;
   $70 = (_remove_minutia($68,$69)|0);
   $ret = $70;
   $71 = ($70|0)!=(0);
   if ($71) {
    label = 11;
    break;
   }
  }
 }
 if ((label|0) == 6) {
  $55 = $rot_y;
  _free($55);
  $56 = $ret;
  $0 = $56;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 else if ((label|0) == 11) {
  $72 = $rot_y;
  _free($72);
  $73 = $ret;
  $0 = $73;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 else if ((label|0) == 24) {
  $134 = $rot_y;
  _free($134);
  $135 = HEAP32[$contour_x>>2]|0;
  $136 = HEAP32[$contour_y>>2]|0;
  $137 = HEAP32[$contour_ex>>2]|0;
  $138 = HEAP32[$contour_ey>>2]|0;
  _free_contour($135,$136,$137,$138);
  $139 = $ret;
  $0 = $139;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 else if ((label|0) == 29) {
  $203 = $rot_y;
  _free($203);
  $204 = HEAP32[$contour_x>>2]|0;
  $205 = HEAP32[$contour_y>>2]|0;
  $206 = HEAP32[$contour_ex>>2]|0;
  $207 = HEAP32[$contour_ey>>2]|0;
  _free_contour($204,$205,$206,$207);
  $208 = HEAP32[$minmax_alloc>>2]|0;
  $209 = ($208|0)>(0);
  if ($209) {
   $210 = HEAP32[$minmax_val>>2]|0;
   _free($210);
   $211 = HEAP32[$minmax_type>>2]|0;
   _free($211);
   $212 = HEAP32[$minmax_i>>2]|0;
   _free($212);
  }
  $213 = $ret;
  $0 = $213;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 else if ((label|0) == 42) {
  $291 = $rot_y;
  _free($291);
  $292 = HEAP32[$contour_x>>2]|0;
  $293 = HEAP32[$contour_y>>2]|0;
  $294 = HEAP32[$contour_ex>>2]|0;
  $295 = HEAP32[$contour_ey>>2]|0;
  _free_contour($292,$293,$294,$295);
  $296 = HEAP32[$minmax_alloc>>2]|0;
  $297 = ($296|0)>(0);
  if ($297) {
   $298 = HEAP32[$minmax_val>>2]|0;
   _free($298);
   $299 = HEAP32[$minmax_type>>2]|0;
   _free($299);
   $300 = HEAP32[$minmax_i>>2]|0;
   _free($300);
  }
  $301 = $ret;
  $0 = $301;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 else if ((label|0) == 49) {
  $318 = $rot_y;
  _free($318);
  $319 = HEAP32[$contour_x>>2]|0;
  $320 = HEAP32[$contour_y>>2]|0;
  $321 = HEAP32[$contour_ex>>2]|0;
  $322 = HEAP32[$contour_ey>>2]|0;
  _free_contour($319,$320,$321,$322);
  $323 = HEAP32[$minmax_alloc>>2]|0;
  $324 = ($323|0)>(0);
  if ($324) {
   $325 = HEAP32[$minmax_val>>2]|0;
   _free($325);
   $326 = HEAP32[$minmax_type>>2]|0;
   _free($326);
   $327 = HEAP32[$minmax_i>>2]|0;
   _free($327);
  }
  $328 = $ret;
  $0 = $328;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 else if ((label|0) == 58) {
  $338 = $rot_y;
  _free($338);
  $0 = 0;
  $339 = $0;
  STACKTOP = sp;return ($339|0);
 }
 return 0|0;
}
function _remove_hooks($minutiae,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0.0, $135 = 0.0, $136 = 0, $137 = 0, $138 = 0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $delta_y = 0, $deltadir = 0, $dist = 0.0, $f = 0, $full_ndirs = 0, $i = 0, $min_deltadir = 0;
 var $minutia1 = 0, $minutia2 = 0, $qtr_ndirs = 0, $ret = 0, $s = 0, $to_remove = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer24 = 0, $vararg_buffer26 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer30 = 0, $vararg_buffer32 = 0, $vararg_buffer34 = 0;
 var $vararg_buffer36 = 0, $vararg_buffer38 = 0, $vararg_buffer5 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer38 = sp + 160|0;
 $vararg_buffer36 = sp + 152|0;
 $vararg_buffer34 = sp + 144|0;
 $vararg_buffer32 = sp + 136|0;
 $vararg_buffer30 = sp + 128|0;
 $vararg_buffer28 = sp + 120|0;
 $vararg_buffer26 = sp + 112|0;
 $vararg_buffer24 = sp + 104|0;
 $vararg_buffer22 = sp + 96|0;
 $vararg_buffer20 = sp + 88|0;
 $vararg_buffer18 = sp + 80|0;
 $vararg_buffer16 = sp + 72|0;
 $vararg_buffer14 = sp + 64|0;
 $vararg_buffer5 = sp + 32|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $lfsparms;
 _print2log(5568,$vararg_buffer);
 $6 = $1;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_calloc($8,4)|0);
 $to_remove = $9;
 $10 = $to_remove;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  (_fprintf(($12|0),(5592|0),($vararg_buffer1|0))|0);
  $0 = -640;
  $212 = $0;
  STACKTOP = sp;return ($212|0);
 }
 $13 = $5;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 << 1;
 $full_ndirs = $16;
 $17 = $5;
 $18 = (($17) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 >> 2;
 $qtr_ndirs = $20;
 $21 = $qtr_ndirs;
 $22 = ($21*3)|0;
 $23 = (($22) - 1)|0;
 $min_deltadir = $23;
 $f = 0;
 L5: while(1) {
  $24 = $f;
  $25 = $1;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) - 1)|0;
  $29 = ($24|0)<($28|0);
  if (!($29)) {
   label = 41;
   break;
  }
  $30 = $f;
  $31 = $to_remove;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)!=(0);
  if (!($34)) {
   _print2log(5640,$vararg_buffer3);
   $35 = $f;
   $36 = $1;
   $37 = (($36) + 8|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + ($35<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $minutia1 = $40;
   $41 = $f;
   $42 = (($41) + 1)|0;
   $s = $42;
   while(1) {
    $43 = $s;
    $44 = $1;
    $45 = (($44) + 4|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = ($43|0)<($46|0);
    if (!($47)) {
     break;
    }
    $48 = $s;
    $49 = $1;
    $50 = (($49) + 8|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + ($48<<2)|0);
    $53 = HEAP32[$52>>2]|0;
    $minutia2 = $53;
    $54 = $f;
    $55 = $minutia1;
    $56 = HEAP32[$55>>2]|0;
    $57 = $minutia1;
    $58 = (($57) + 4|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = $minutia1;
    $61 = (($60) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = $s;
    $64 = $minutia2;
    $65 = HEAP32[$64>>2]|0;
    $66 = $minutia2;
    $67 = (($66) + 4|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = $minutia2;
    $70 = (($69) + 32|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $54;
    $vararg_ptr7 = (($vararg_buffer5) + 4|0);
    HEAP32[$vararg_ptr7>>2] = $56;
    $vararg_ptr8 = (($vararg_buffer5) + 8|0);
    HEAP32[$vararg_ptr8>>2] = $59;
    $vararg_ptr9 = (($vararg_buffer5) + 12|0);
    HEAP32[$vararg_ptr9>>2] = $62;
    $vararg_ptr10 = (($vararg_buffer5) + 16|0);
    HEAP32[$vararg_ptr10>>2] = $63;
    $vararg_ptr11 = (($vararg_buffer5) + 20|0);
    HEAP32[$vararg_ptr11>>2] = $65;
    $vararg_ptr12 = (($vararg_buffer5) + 24|0);
    HEAP32[$vararg_ptr12>>2] = $68;
    $vararg_ptr13 = (($vararg_buffer5) + 28|0);
    HEAP32[$vararg_ptr13>>2] = $71;
    _print2log(5648,$vararg_buffer5);
    $72 = $2;
    $73 = $minutia1;
    $74 = (($73) + 4|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = $3;
    $77 = Math_imul($75, $76)|0;
    $78 = (($72) + ($77)|0);
    $79 = $minutia1;
    $80 = HEAP32[$79>>2]|0;
    $81 = (($78) + ($80)|0);
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $minutia1;
    $85 = (($84) + 32|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = ($83|0)!=($86|0);
    if ($87) {
     label = 9;
     break;
    }
    $88 = $2;
    $89 = $minutia2;
    $90 = (($89) + 4|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = $3;
    $93 = Math_imul($91, $92)|0;
    $94 = (($88) + ($93)|0);
    $95 = $minutia2;
    $96 = HEAP32[$95>>2]|0;
    $97 = (($94) + ($96)|0);
    $98 = HEAP8[$97>>0]|0;
    $99 = $98&255;
    $100 = $minutia2;
    $101 = (($100) + 32|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = ($99|0)!=($102|0);
    if ($103) {
     $104 = $s;
     $105 = $to_remove;
     $106 = (($105) + ($104<<2)|0);
     HEAP32[$106>>2] = 1;
    }
    $107 = $s;
    $108 = $to_remove;
    $109 = (($108) + ($107<<2)|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = ($110|0)!=(0);
    if ($111) {
     _print2log(5640,$vararg_buffer38);
    } else {
     $112 = $minutia2;
     $113 = (($112) + 4|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = $minutia1;
     $116 = (($115) + 4|0);
     $117 = HEAP32[$116>>2]|0;
     $118 = (($114) - ($117))|0;
     $delta_y = $118;
     $119 = $delta_y;
     $120 = $5;
     $121 = (($120) + 240|0);
     $122 = HEAP32[$121>>2]|0;
     $123 = ($119|0)<=($122|0);
     if (!($123)) {
      label = 35;
      break;
     }
     _print2log(5680,$vararg_buffer16);
     $124 = $minutia1;
     $125 = HEAP32[$124>>2]|0;
     $126 = $minutia1;
     $127 = (($126) + 4|0);
     $128 = HEAP32[$127>>2]|0;
     $129 = $minutia2;
     $130 = HEAP32[$129>>2]|0;
     $131 = $minutia2;
     $132 = (($131) + 4|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = (+_distance($125,$128,$130,$133));
     $dist = $134;
     $135 = $dist;
     $136 = $5;
     $137 = (($136) + 240|0);
     $138 = HEAP32[$137>>2]|0;
     $139 = (+($138|0));
     $140 = $135 <= $139;
     if ($140) {
      _print2log(5688,$vararg_buffer18);
      $141 = $minutia1;
      $142 = (($141) + 16|0);
      $143 = HEAP32[$142>>2]|0;
      $144 = $minutia2;
      $145 = (($144) + 16|0);
      $146 = HEAP32[$145>>2]|0;
      $147 = $full_ndirs;
      $148 = (_closest_dir_dist($143,$146,$147)|0);
      $deltadir = $148;
      $149 = ($148|0)==(-1);
      if ($149) {
       label = 16;
       break L5;
      }
      $152 = $deltadir;
      $153 = $min_deltadir;
      $154 = ($152|0)>($153|0);
      if ($154) {
       _print2log(5744,$vararg_buffer22);
       $155 = $minutia1;
       $156 = (($155) + 32|0);
       $157 = HEAP32[$156>>2]|0;
       $158 = $minutia2;
       $159 = (($158) + 32|0);
       $160 = HEAP32[$159>>2]|0;
       $161 = ($157|0)!=($160|0);
       if ($161) {
        $162 = $minutia1;
        $163 = $minutia2;
        $164 = $5;
        $165 = (($164) + 244|0);
        $166 = HEAP32[$165>>2]|0;
        $167 = $2;
        $168 = $3;
        $169 = $4;
        $170 = (_on_hook($162,$163,$166,$167,$168,$169)|0);
        $ret = $170;
        $171 = $ret;
        $172 = ($171|0)==(1);
        if ($172) {
         _print2log(5752,$vararg_buffer24);
         $173 = $f;
         $174 = $to_remove;
         $175 = (($174) + ($173<<2)|0);
         HEAP32[$175>>2] = 1;
         $176 = $s;
         $177 = $to_remove;
         $178 = (($177) + ($176<<2)|0);
         HEAP32[$178>>2] = 1;
        } else {
         $179 = $ret;
         $180 = ($179|0)==(2);
         if ($180) {
          label = 22;
          break;
         }
         $184 = $ret;
         $185 = ($184|0)<(0);
         if ($185) {
          label = 24;
          break L5;
         }
         _print2log(5640,$vararg_buffer28);
        }
       } else {
        _print2log(5640,$vararg_buffer30);
       }
      } else {
       _print2log(5640,$vararg_buffer32);
      }
     } else {
      _print2log(5640,$vararg_buffer34);
     }
    }
    $188 = $s;
    $189 = (($188) + 1)|0;
    $s = $189;
   }
   if ((label|0) == 9) {
    label = 0;
    _print2log(5640,$vararg_buffer14);
   }
   else if ((label|0) == 22) {
    label = 0;
    _print2log(5760,$vararg_buffer26);
    $181 = $f;
    $182 = $to_remove;
    $183 = (($182) + ($181<<2)|0);
    HEAP32[$183>>2] = 1;
   }
   else if ((label|0) == 35) {
    label = 0;
    _print2log(5640,$vararg_buffer36);
   }
  }
  $190 = $f;
  $191 = (($190) + 1)|0;
  $f = $191;
 }
 if ((label|0) == 16) {
  $150 = $to_remove;
  _free($150);
  $151 = HEAP32[_stderr>>2]|0;
  (_fprintf(($151|0),(5696|0),($vararg_buffer20|0))|0);
  $0 = -641;
  $212 = $0;
  STACKTOP = sp;return ($212|0);
 }
 else if ((label|0) == 24) {
  $186 = $to_remove;
  _free($186);
  $187 = $ret;
  $0 = $187;
  $212 = $0;
  STACKTOP = sp;return ($212|0);
 }
 else if ((label|0) == 41) {
  $192 = $1;
  $193 = (($192) + 4|0);
  $194 = HEAP32[$193>>2]|0;
  $195 = (($194) - 1)|0;
  $i = $195;
  while(1) {
   $196 = $i;
   $197 = ($196|0)>=(0);
   if (!($197)) {
    label = 49;
    break;
   }
   $198 = $i;
   $199 = $to_remove;
   $200 = (($199) + ($198<<2)|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = ($201|0)!=(0);
   if ($202) {
    $203 = $i;
    $204 = $1;
    $205 = (_remove_minutia($203,$204)|0);
    $ret = $205;
    $206 = ($205|0)!=(0);
    if ($206) {
     label = 45;
     break;
    }
   }
   $209 = $i;
   $210 = (($209) + -1)|0;
   $i = $210;
  }
  if ((label|0) == 45) {
   $207 = $to_remove;
   _free($207);
   $208 = $ret;
   $0 = $208;
   $212 = $0;
   STACKTOP = sp;return ($212|0);
  }
  else if ((label|0) == 49) {
   $211 = $to_remove;
   _free($211);
   $0 = 0;
   $212 = $0;
   STACKTOP = sp;return ($212|0);
  }
 }
 return 0|0;
}
function _remove_overlaps($minutiae,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0.0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0.0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0.0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0.0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $delta_y = 0, $deltadir = 0, $dist = 0.0, $f = 0, $full_ndirs = 0, $half_ndirs = 0, $i = 0, $joindir = 0, $min_deltadir = 0, $minutia1 = 0, $minutia2 = 0, $opp1dir = 0, $qtr_ndirs = 0, $ret = 0, $s = 0, $to_remove = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer14 = 0;
 var $vararg_buffer16 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer24 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer30 = 0, $vararg_buffer32 = 0, $vararg_buffer34 = 0, $vararg_buffer36 = 0, $vararg_buffer38 = 0, $vararg_buffer40 = 0, $vararg_buffer5 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr27 = 0, $vararg_ptr7 = 0;
 var $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer40 = sp + 168|0;
 $vararg_buffer38 = sp + 160|0;
 $vararg_buffer36 = sp + 152|0;
 $vararg_buffer34 = sp + 144|0;
 $vararg_buffer32 = sp + 136|0;
 $vararg_buffer30 = sp + 128|0;
 $vararg_buffer28 = sp + 120|0;
 $vararg_buffer24 = sp + 104|0;
 $vararg_buffer22 = sp + 96|0;
 $vararg_buffer20 = sp + 88|0;
 $vararg_buffer18 = sp + 80|0;
 $vararg_buffer16 = sp + 72|0;
 $vararg_buffer14 = sp + 64|0;
 $vararg_buffer5 = sp + 32|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $lfsparms;
 _print2log(6400,$vararg_buffer);
 $6 = $1;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_calloc($8,4)|0);
 $to_remove = $9;
 $10 = $to_remove;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  (_fprintf(($12|0),(6424|0),($vararg_buffer1|0))|0);
  $0 = -650;
  $260 = $0;
  STACKTOP = sp;return ($260|0);
 }
 $13 = $5;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 << 1;
 $full_ndirs = $16;
 $17 = $5;
 $18 = (($17) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 >> 2;
 $qtr_ndirs = $20;
 $21 = $5;
 $22 = (($21) + 20|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 >> 1;
 $half_ndirs = $24;
 $25 = $qtr_ndirs;
 $26 = ($25*3)|0;
 $27 = (($26) - 1)|0;
 $min_deltadir = $27;
 $f = 0;
 L5: while(1) {
  $28 = $f;
  $29 = $1;
  $30 = (($29) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) - 1)|0;
  $33 = ($28|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = $f;
  $35 = $to_remove;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)!=(0);
  if (!($38)) {
   _print2log(5640,$vararg_buffer3);
   $39 = $f;
   $40 = $1;
   $41 = (($40) + 8|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + ($39<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $minutia1 = $44;
   $45 = $f;
   $46 = (($45) + 1)|0;
   $s = $46;
   while(1) {
    $47 = $s;
    $48 = $1;
    $49 = (($48) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = ($47|0)<($50|0);
    if (!($51)) {
     break;
    }
    $52 = $s;
    $53 = $1;
    $54 = (($53) + 8|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($52<<2)|0);
    $57 = HEAP32[$56>>2]|0;
    $minutia2 = $57;
    $58 = $f;
    $59 = $minutia1;
    $60 = HEAP32[$59>>2]|0;
    $61 = $minutia1;
    $62 = (($61) + 4|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = $minutia1;
    $65 = (($64) + 32|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = $s;
    $68 = $minutia2;
    $69 = HEAP32[$68>>2]|0;
    $70 = $minutia2;
    $71 = (($70) + 4|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $minutia2;
    $74 = (($73) + 32|0);
    $75 = HEAP32[$74>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $58;
    $vararg_ptr7 = (($vararg_buffer5) + 4|0);
    HEAP32[$vararg_ptr7>>2] = $60;
    $vararg_ptr8 = (($vararg_buffer5) + 8|0);
    HEAP32[$vararg_ptr8>>2] = $63;
    $vararg_ptr9 = (($vararg_buffer5) + 12|0);
    HEAP32[$vararg_ptr9>>2] = $66;
    $vararg_ptr10 = (($vararg_buffer5) + 16|0);
    HEAP32[$vararg_ptr10>>2] = $67;
    $vararg_ptr11 = (($vararg_buffer5) + 20|0);
    HEAP32[$vararg_ptr11>>2] = $69;
    $vararg_ptr12 = (($vararg_buffer5) + 24|0);
    HEAP32[$vararg_ptr12>>2] = $72;
    $vararg_ptr13 = (($vararg_buffer5) + 28|0);
    HEAP32[$vararg_ptr13>>2] = $75;
    _print2log(5648,$vararg_buffer5);
    $76 = $2;
    $77 = $minutia1;
    $78 = (($77) + 4|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = $3;
    $81 = Math_imul($79, $80)|0;
    $82 = (($76) + ($81)|0);
    $83 = $minutia1;
    $84 = HEAP32[$83>>2]|0;
    $85 = (($82) + ($84)|0);
    $86 = HEAP8[$85>>0]|0;
    $87 = $86&255;
    $88 = $minutia1;
    $89 = (($88) + 32|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($87|0)!=($90|0);
    if ($91) {
     label = 9;
     break;
    }
    $92 = $2;
    $93 = $minutia2;
    $94 = (($93) + 4|0);
    $95 = HEAP32[$94>>2]|0;
    $96 = $3;
    $97 = Math_imul($95, $96)|0;
    $98 = (($92) + ($97)|0);
    $99 = $minutia2;
    $100 = HEAP32[$99>>2]|0;
    $101 = (($98) + ($100)|0);
    $102 = HEAP8[$101>>0]|0;
    $103 = $102&255;
    $104 = $minutia2;
    $105 = (($104) + 32|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = ($103|0)!=($106|0);
    if ($107) {
     $108 = $s;
     $109 = $to_remove;
     $110 = (($109) + ($108<<2)|0);
     HEAP32[$110>>2] = 1;
    }
    $111 = $s;
    $112 = $to_remove;
    $113 = (($112) + ($111<<2)|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = ($114|0)!=(0);
    if ($115) {
     _print2log(5640,$vararg_buffer40);
    } else {
     $116 = $minutia2;
     $117 = (($116) + 4|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = $minutia1;
     $120 = (($119) + 4|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = (($118) - ($121))|0;
     $delta_y = $122;
     $123 = $delta_y;
     $124 = $5;
     $125 = (($124) + 272|0);
     $126 = HEAP32[$125>>2]|0;
     $127 = ($123|0)<=($126|0);
     if (!($127)) {
      label = 34;
      break;
     }
     _print2log(5680,$vararg_buffer16);
     $128 = $minutia1;
     $129 = HEAP32[$128>>2]|0;
     $130 = $minutia1;
     $131 = (($130) + 4|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = $minutia2;
     $134 = HEAP32[$133>>2]|0;
     $135 = $minutia2;
     $136 = (($135) + 4|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = (+_distance($129,$132,$134,$137));
     $dist = $138;
     $139 = $dist;
     $140 = $5;
     $141 = (($140) + 272|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = (+($142|0));
     $144 = $139 <= $143;
     if ($144) {
      _print2log(5688,$vararg_buffer18);
      $145 = $minutia1;
      $146 = (($145) + 16|0);
      $147 = HEAP32[$146>>2]|0;
      $148 = $minutia2;
      $149 = (($148) + 16|0);
      $150 = HEAP32[$149>>2]|0;
      $151 = $full_ndirs;
      $152 = (_closest_dir_dist($147,$150,$151)|0);
      $deltadir = $152;
      $153 = ($152|0)==(-1);
      if ($153) {
       label = 16;
       break L5;
      }
      $156 = $deltadir;
      $157 = $min_deltadir;
      $158 = ($156|0)>($157|0);
      if ($158) {
       _print2log(5744,$vararg_buffer22);
       $159 = $minutia1;
       $160 = (($159) + 32|0);
       $161 = HEAP32[$160>>2]|0;
       $162 = $minutia2;
       $163 = (($162) + 32|0);
       $164 = HEAP32[$163>>2]|0;
       $165 = ($161|0)==($164|0);
       if ($165) {
        $166 = $minutia1;
        $167 = HEAP32[$166>>2]|0;
        $168 = $minutia1;
        $169 = (($168) + 4|0);
        $170 = HEAP32[$169>>2]|0;
        $171 = $minutia2;
        $172 = HEAP32[$171>>2]|0;
        $173 = $minutia2;
        $174 = (($173) + 4|0);
        $175 = HEAP32[$174>>2]|0;
        $176 = $5;
        $177 = (($176) + 20|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = (_line2direction($167,$170,$172,$175,$178)|0);
        $joindir = $179;
        $180 = $minutia1;
        $181 = (($180) + 16|0);
        $182 = HEAP32[$181>>2]|0;
        $183 = $5;
        $184 = (($183) + 20|0);
        $185 = HEAP32[$184>>2]|0;
        $186 = (($182) + ($185))|0;
        $187 = $full_ndirs;
        $188 = (($186|0) % ($187|0))&-1;
        $opp1dir = $188;
        $189 = $opp1dir;
        $190 = $joindir;
        $191 = (($189) - ($190))|0;
        $192 = (Math_abs(($191|0))|0);
        $joindir = $192;
        $193 = $joindir;
        $194 = $full_ndirs;
        $195 = $joindir;
        $196 = (($194) - ($195))|0;
        $197 = ($193|0)<($196|0);
        if ($197) {
         $198 = $joindir;
         $202 = $198;
        } else {
         $199 = $full_ndirs;
         $200 = $joindir;
         $201 = (($199) - ($200))|0;
         $202 = $201;
        }
        $joindir = $202;
        $203 = $joindir;
        $204 = $dist;
        HEAP32[$vararg_buffer24>>2] = $203;
        $vararg_ptr27 = (($vararg_buffer24) + 4|0);
        HEAPF64[tempDoublePtr>>3]=$204;HEAP32[$vararg_ptr27>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr27+4>>2]=HEAP32[tempDoublePtr+4>>2];
        _print2log(6520,$vararg_buffer24);
        $205 = $joindir;
        $206 = $half_ndirs;
        $207 = ($205|0)<=($206|0);
        if ($207) {
         label = 24;
        } else {
         $208 = $dist;
         $209 = $5;
         $210 = (($209) + 276|0);
         $211 = HEAP32[$210>>2]|0;
         $212 = (+($211|0));
         $213 = $208 <= $212;
         if ($213) {
          label = 24;
         } else {
          label = 26;
         }
        }
        do {
         if ((label|0) == 24) {
          label = 0;
          $214 = $minutia1;
          $215 = HEAP32[$214>>2]|0;
          $216 = $minutia1;
          $217 = (($216) + 4|0);
          $218 = HEAP32[$217>>2]|0;
          $219 = $minutia2;
          $220 = HEAP32[$219>>2]|0;
          $221 = $minutia2;
          $222 = (($221) + 4|0);
          $223 = HEAP32[$222>>2]|0;
          $224 = $2;
          $225 = $3;
          $226 = $4;
          $227 = $5;
          $228 = (_free_path($215,$218,$220,$223,$224,$225,$226,$227)|0);
          $229 = ($228|0)!=(0);
          if (!($229)) {
           label = 26;
           break;
          }
          _print2log(5776,$vararg_buffer28);
          $230 = $f;
          $231 = $to_remove;
          $232 = (($231) + ($230<<2)|0);
          HEAP32[$232>>2] = 1;
          $233 = $s;
          $234 = $to_remove;
          $235 = (($234) + ($233<<2)|0);
          HEAP32[$235>>2] = 1;
         }
        } while(0);
        if ((label|0) == 26) {
         label = 0;
         _print2log(5640,$vararg_buffer30);
        }
       } else {
        _print2log(5640,$vararg_buffer32);
       }
      } else {
       _print2log(5640,$vararg_buffer34);
      }
     } else {
      _print2log(5640,$vararg_buffer36);
     }
    }
    $236 = $s;
    $237 = (($236) + 1)|0;
    $s = $237;
   }
   if ((label|0) == 9) {
    label = 0;
    _print2log(5640,$vararg_buffer14);
   }
   else if ((label|0) == 34) {
    label = 0;
    _print2log(5640,$vararg_buffer38);
   }
  }
  $238 = $f;
  $239 = (($238) + 1)|0;
  $f = $239;
 }
 if ((label|0) == 16) {
  $154 = $to_remove;
  _free($154);
  $155 = HEAP32[_stderr>>2]|0;
  (_fprintf(($155|0),(6472|0),($vararg_buffer20|0))|0);
  $0 = -651;
  $260 = $0;
  STACKTOP = sp;return ($260|0);
 }
 $240 = $1;
 $241 = (($240) + 4|0);
 $242 = HEAP32[$241>>2]|0;
 $243 = (($242) - 1)|0;
 $i = $243;
 while(1) {
  $244 = $i;
  $245 = ($244|0)>=(0);
  if (!($245)) {
   label = 48;
   break;
  }
  $246 = $i;
  $247 = $to_remove;
  $248 = (($247) + ($246<<2)|0);
  $249 = HEAP32[$248>>2]|0;
  $250 = ($249|0)!=(0);
  if ($250) {
   $251 = $i;
   $252 = $1;
   $253 = (_remove_minutia($251,$252)|0);
   $ret = $253;
   $254 = ($253|0)!=(0);
   if ($254) {
    label = 44;
    break;
   }
  }
  $257 = $i;
  $258 = (($257) + -1)|0;
  $i = $258;
 }
 if ((label|0) == 44) {
  $255 = $to_remove;
  _free($255);
  $256 = $ret;
  $0 = $256;
  $260 = $0;
  STACKTOP = sp;return ($260|0);
 }
 else if ((label|0) == 48) {
  $259 = $to_remove;
  _free($259);
  $0 = 0;
  $260 = $0;
  STACKTOP = sp;return ($260|0);
 }
 return 0|0;
}
function _remove_malformations($minutiae,$bdata,$iw,$ih,$low_flow_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $low_flow_map = $low_flow_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0.0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0.0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0, $222 = 0.0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0.0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0.0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0, $302 = 0.0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0.0, $307 = 0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0.0, $311 = 0.0, $312 = 0, $313 = 0;
 var $314 = 0.0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0.0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_dist = 0.0, $ax1 = 0, $ax2 = 0, $ay1 = 0;
 var $ay2 = 0, $b_dist = 0.0, $blk_x = 0, $blk_y = 0, $bx1 = 0, $bx2 = 0, $by1 = 0, $by2 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $fmapval = 0, $i = 0, $j = 0, $minutia = 0, $ncontour = 0, $num = 0, $ratio = 0.0, $removed = 0;
 var $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, $x_list = 0, $y_list = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer12 = sp + 56|0;
 $vararg_buffer8 = sp + 48|0;
 $vararg_buffer4 = sp + 40|0;
 $vararg_buffer1 = sp + 32|0;
 $vararg_buffer = sp + 24|0;
 $contour_x = sp + 156|0;
 $contour_y = sp + 152|0;
 $contour_ex = sp + 148|0;
 $contour_ey = sp + 144|0;
 $ncontour = sp + 140|0;
 $x_list = sp + 104|0;
 $y_list = sp + 100|0;
 $num = sp + 96|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $low_flow_map;
 $6 = $mw;
 $7 = $mh;
 $8 = $lfsparms;
 _print2log(5928,$vararg_buffer);
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) - 1)|0;
 $i = $12;
 while(1) {
  $13 = $i;
  $14 = ($13|0)>=(0);
  if (!($14)) {
   label = 59;
   break;
  }
  $15 = $i;
  $16 = $1;
  $17 = (($16) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $minutia = $20;
  $21 = $8;
  $22 = (($21) + 284|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = $minutia;
  $25 = HEAP32[$24>>2]|0;
  $26 = $minutia;
  $27 = (($26) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $minutia;
  $30 = HEAP32[$29>>2]|0;
  $31 = $minutia;
  $32 = (($31) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $minutia;
  $35 = (($34) + 8|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $minutia;
  $38 = (($37) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = $2;
  $41 = $3;
  $42 = $4;
  $43 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$23,$25,$28,$30,$33,$36,$39,1,$40,$41,$42)|0);
  $ret = $43;
  $44 = $ret;
  $45 = ($44|0)<(0);
  if ($45) {
   label = 4;
   break;
  }
  $47 = $ret;
  $48 = ($47|0)==(2);
  if ($48) {
   label = 8;
  } else {
   $49 = $ret;
   $50 = ($49|0)==(1);
   if ($50) {
    label = 8;
   } else {
    $51 = HEAP32[$ncontour>>2]|0;
    $52 = $8;
    $53 = (($52) + 284|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = ($51|0)<($54|0);
    if ($55) {
     label = 8;
    } else {
     $77 = $8;
     $78 = (($77) + 280|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = (($79) - 1)|0;
     $81 = HEAP32[$contour_x>>2]|0;
     $82 = (($81) + ($80<<2)|0);
     $83 = HEAP32[$82>>2]|0;
     $ax1 = $83;
     $84 = $8;
     $85 = (($84) + 280|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = (($86) - 1)|0;
     $88 = HEAP32[$contour_y>>2]|0;
     $89 = (($88) + ($87<<2)|0);
     $90 = HEAP32[$89>>2]|0;
     $ay1 = $90;
     $91 = $8;
     $92 = (($91) + 284|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = (($93) - 1)|0;
     $95 = HEAP32[$contour_x>>2]|0;
     $96 = (($95) + ($94<<2)|0);
     $97 = HEAP32[$96>>2]|0;
     $bx1 = $97;
     $98 = $8;
     $99 = (($98) + 284|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = (($100) - 1)|0;
     $102 = HEAP32[$contour_y>>2]|0;
     $103 = (($102) + ($101<<2)|0);
     $104 = HEAP32[$103>>2]|0;
     $by1 = $104;
     $105 = HEAP32[$contour_x>>2]|0;
     $106 = HEAP32[$contour_y>>2]|0;
     $107 = HEAP32[$contour_ex>>2]|0;
     $108 = HEAP32[$contour_ey>>2]|0;
     _free_contour($105,$106,$107,$108);
     $109 = $8;
     $110 = (($109) + 284|0);
     $111 = HEAP32[$110>>2]|0;
     $112 = $minutia;
     $113 = HEAP32[$112>>2]|0;
     $114 = $minutia;
     $115 = (($114) + 4|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = $minutia;
     $118 = HEAP32[$117>>2]|0;
     $119 = $minutia;
     $120 = (($119) + 4|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = $minutia;
     $123 = (($122) + 8|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = $minutia;
     $126 = (($125) + 12|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = $2;
     $129 = $3;
     $130 = $4;
     $131 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$111,$113,$116,$118,$121,$124,$127,0,$128,$129,$130)|0);
     $ret = $131;
     $132 = $ret;
     $133 = ($132|0)<(0);
     if ($133) {
      label = 15;
      break;
     }
     $135 = $ret;
     $136 = ($135|0)==(2);
     if ($136) {
      label = 19;
     } else {
      $137 = $ret;
      $138 = ($137|0)==(1);
      if ($138) {
       label = 19;
      } else {
       $139 = HEAP32[$ncontour>>2]|0;
       $140 = $8;
       $141 = (($140) + 284|0);
       $142 = HEAP32[$141>>2]|0;
       $143 = ($139|0)<($142|0);
       if ($143) {
        label = 19;
       } else {
        $165 = $8;
        $166 = (($165) + 280|0);
        $167 = HEAP32[$166>>2]|0;
        $168 = (($167) - 1)|0;
        $169 = HEAP32[$contour_x>>2]|0;
        $170 = (($169) + ($168<<2)|0);
        $171 = HEAP32[$170>>2]|0;
        $ax2 = $171;
        $172 = $8;
        $173 = (($172) + 280|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = (($174) - 1)|0;
        $176 = HEAP32[$contour_y>>2]|0;
        $177 = (($176) + ($175<<2)|0);
        $178 = HEAP32[$177>>2]|0;
        $ay2 = $178;
        $179 = $8;
        $180 = (($179) + 284|0);
        $181 = HEAP32[$180>>2]|0;
        $182 = (($181) - 1)|0;
        $183 = HEAP32[$contour_x>>2]|0;
        $184 = (($183) + ($182<<2)|0);
        $185 = HEAP32[$184>>2]|0;
        $bx2 = $185;
        $186 = $8;
        $187 = (($186) + 284|0);
        $188 = HEAP32[$187>>2]|0;
        $189 = (($188) - 1)|0;
        $190 = HEAP32[$contour_y>>2]|0;
        $191 = (($190) + ($189<<2)|0);
        $192 = HEAP32[$191>>2]|0;
        $by2 = $192;
        $193 = HEAP32[$contour_x>>2]|0;
        $194 = HEAP32[$contour_y>>2]|0;
        $195 = HEAP32[$contour_ex>>2]|0;
        $196 = HEAP32[$contour_ey>>2]|0;
        _free_contour($193,$194,$195,$196);
        $197 = $ax1;
        $198 = $ay1;
        $199 = $ax2;
        $200 = $ay2;
        $201 = (+_distance($197,$198,$199,$200));
        $a_dist = $201;
        $202 = $bx1;
        $203 = $by1;
        $204 = $bx2;
        $205 = $by2;
        $206 = (+_distance($202,$203,$204,$205));
        $b_dist = $206;
        $207 = $minutia;
        $208 = HEAP32[$207>>2]|0;
        $209 = $8;
        $210 = (($209) + 8|0);
        $211 = HEAP32[$210>>2]|0;
        $212 = (($208|0) / ($211|0))&-1;
        $blk_x = $212;
        $213 = $minutia;
        $214 = (($213) + 4|0);
        $215 = HEAP32[$214>>2]|0;
        $216 = $8;
        $217 = (($216) + 8|0);
        $218 = HEAP32[$217>>2]|0;
        $219 = (($215|0) / ($218|0))&-1;
        $blk_y = $219;
        $removed = 0;
        $220 = $a_dist;
        $221 = $220 == 0.0;
        if ($221) {
         label = 27;
        } else {
         $222 = $b_dist;
         $223 = $222 == 0.0;
         if ($223) {
          label = 27;
         }
        }
        if ((label|0) == 27) {
         label = 0;
         $224 = $minutia;
         $225 = HEAP32[$224>>2]|0;
         $226 = $minutia;
         $227 = (($226) + 4|0);
         $228 = HEAP32[$227>>2]|0;
         HEAP32[$vararg_buffer8>>2] = $225;
         $vararg_ptr11 = (($vararg_buffer8) + 4|0);
         HEAP32[$vararg_ptr11>>2] = $228;
         _print2log(5992,$vararg_buffer8);
         $229 = $i;
         $230 = $1;
         $231 = (_remove_minutia($229,$230)|0);
         $ret = $231;
         $232 = ($231|0)!=(0);
         if ($232) {
          label = 28;
          break;
         }
         $removed = 1;
        }
        $234 = $removed;
        $235 = ($234|0)!=(0);
        if (!($235)) {
         $236 = $5;
         $237 = $blk_y;
         $238 = $6;
         $239 = Math_imul($237, $238)|0;
         $240 = (($236) + ($239<<2)|0);
         $241 = $blk_x;
         $242 = (($240) + ($241<<2)|0);
         $243 = HEAP32[$242>>2]|0;
         $fmapval = $243;
         $244 = $fmapval;
         $245 = ($244|0)!=(0);
         if ($245) {
          $246 = $b_dist;
          $247 = $8;
          $248 = (($247) + 296|0);
          $249 = HEAP32[$248>>2]|0;
          $250 = (+($249|0));
          $251 = $246 > $250;
          if ($251) {
           $252 = $minutia;
           $253 = HEAP32[$252>>2]|0;
           $254 = $minutia;
           $255 = (($254) + 4|0);
           $256 = HEAP32[$255>>2]|0;
           HEAP32[$vararg_buffer12>>2] = $253;
           $vararg_ptr15 = (($vararg_buffer12) + 4|0);
           HEAP32[$vararg_ptr15>>2] = $256;
           _print2log(6008,$vararg_buffer12);
           $257 = $i;
           $258 = $1;
           $259 = (_remove_minutia($257,$258)|0);
           $ret = $259;
           $260 = ($259|0)!=(0);
           if ($260) {
            label = 34;
            break;
           }
           $removed = 1;
          }
         }
        }
        $262 = $removed;
        $263 = ($262|0)!=(0);
        if (!($263)) {
         $264 = $bx1;
         $265 = $by1;
         $266 = $bx2;
         $267 = $by2;
         $268 = (_line_points($x_list,$y_list,$num,$264,$265,$266,$267)|0);
         $ret = $268;
         $269 = ($268|0)!=(0);
         if ($269) {
          label = 40;
          break;
         }
         $j = 0;
         while(1) {
          $271 = $j;
          $272 = HEAP32[$num>>2]|0;
          $273 = ($271|0)<($272|0);
          if (!($273)) {
           break;
          }
          $274 = $2;
          $275 = $j;
          $276 = HEAP32[$y_list>>2]|0;
          $277 = (($276) + ($275<<2)|0);
          $278 = HEAP32[$277>>2]|0;
          $279 = $3;
          $280 = Math_imul($278, $279)|0;
          $281 = (($274) + ($280)|0);
          $282 = $j;
          $283 = HEAP32[$x_list>>2]|0;
          $284 = (($283) + ($282<<2)|0);
          $285 = HEAP32[$284>>2]|0;
          $286 = (($281) + ($285)|0);
          $287 = HEAP8[$286>>0]|0;
          $288 = $287&255;
          $289 = $minutia;
          $290 = (($289) + 32|0);
          $291 = HEAP32[$290>>2]|0;
          $292 = ($288|0)!=($291|0);
          if ($292) {
           $293 = $b_dist;
           $294 = $a_dist;
           $295 = $293 / $294;
           $ratio = $295;
           $296 = $ratio;
           $297 = $296 < 0.0;
           if ($297) {
            $298 = $ratio;
            $299 = $298 * 16384.0;
            $300 = $299 - 0.5;
            $301 = (~~(($300)));
            $302 = (+($301|0));
            $303 = $302 / 16384.0;
            $310 = $303;
           } else {
            $304 = $ratio;
            $305 = $304 * 16384.0;
            $306 = $305 + 0.5;
            $307 = (~~(($306)));
            $308 = (+($307|0));
            $309 = $308 / 16384.0;
            $310 = $309;
           }
           $ratio = $310;
           $311 = $ratio;
           $312 = $8;
           $313 = (($312) + 288|0);
           $314 = +HEAPF64[$313>>3];
           $315 = $311 > $314;
           if ($315) {
            label = 48;
            break;
           }
          }
          $329 = $j;
          $330 = (($329) + 1)|0;
          $j = $330;
         }
         if ((label|0) == 48) {
          label = 0;
          $316 = $minutia;
          $317 = HEAP32[$316>>2]|0;
          $318 = $minutia;
          $319 = (($318) + 4|0);
          $320 = HEAP32[$319>>2]|0;
          $321 = $ratio;
          HEAP32[$vararg_buffer16>>2] = $317;
          $vararg_ptr19 = (($vararg_buffer16) + 4|0);
          HEAP32[$vararg_ptr19>>2] = $320;
          $vararg_ptr20 = (($vararg_buffer16) + 8|0);
          HEAPF64[tempDoublePtr>>3]=$321;HEAP32[$vararg_ptr20>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr20+4>>2]=HEAP32[tempDoublePtr+4>>2];
          _print2log(6024,$vararg_buffer16);
          $322 = $i;
          $323 = $1;
          $324 = (_remove_minutia($322,$323)|0);
          $ret = $324;
          $325 = ($324|0)!=(0);
          if ($325) {
           label = 49;
           break;
          }
         }
         $331 = HEAP32[$x_list>>2]|0;
         _free($331);
         $332 = HEAP32[$y_list>>2]|0;
         _free($332);
        }
       }
      }
     }
     if ((label|0) == 19) {
      label = 0;
      $144 = $ret;
      $145 = ($144|0)==(1);
      if ($145) {
       label = 21;
      } else {
       $146 = HEAP32[$ncontour>>2]|0;
       $147 = $8;
       $148 = (($147) + 284|0);
       $149 = HEAP32[$148>>2]|0;
       $150 = ($146|0)<($149|0);
       if ($150) {
        label = 21;
       }
      }
      if ((label|0) == 21) {
       label = 0;
       $151 = HEAP32[$contour_x>>2]|0;
       $152 = HEAP32[$contour_y>>2]|0;
       $153 = HEAP32[$contour_ex>>2]|0;
       $154 = HEAP32[$contour_ey>>2]|0;
       _free_contour($151,$152,$153,$154);
      }
      $155 = $minutia;
      $156 = HEAP32[$155>>2]|0;
      $157 = $minutia;
      $158 = (($157) + 4|0);
      $159 = HEAP32[$158>>2]|0;
      HEAP32[$vararg_buffer4>>2] = $156;
      $vararg_ptr7 = (($vararg_buffer4) + 4|0);
      HEAP32[$vararg_ptr7>>2] = $159;
      _print2log(5976,$vararg_buffer4);
      $160 = $i;
      $161 = $1;
      $162 = (_remove_minutia($160,$161)|0);
      $ret = $162;
      $163 = ($162|0)!=(0);
      if ($163) {
       label = 23;
       break;
      }
     }
    }
   }
  }
  if ((label|0) == 8) {
   label = 0;
   $56 = $ret;
   $57 = ($56|0)==(1);
   if ($57) {
    label = 10;
   } else {
    $58 = HEAP32[$ncontour>>2]|0;
    $59 = $8;
    $60 = (($59) + 284|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($58|0)<($61|0);
    if ($62) {
     label = 10;
    }
   }
   if ((label|0) == 10) {
    label = 0;
    $63 = HEAP32[$contour_x>>2]|0;
    $64 = HEAP32[$contour_y>>2]|0;
    $65 = HEAP32[$contour_ex>>2]|0;
    $66 = HEAP32[$contour_ey>>2]|0;
    _free_contour($63,$64,$65,$66);
   }
   $67 = $minutia;
   $68 = HEAP32[$67>>2]|0;
   $69 = $minutia;
   $70 = (($69) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $68;
   $vararg_ptr3 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $71;
   _print2log(5960,$vararg_buffer1);
   $72 = $i;
   $73 = $1;
   $74 = (_remove_minutia($72,$73)|0);
   $ret = $74;
   $75 = ($74|0)!=(0);
   if ($75) {
    label = 12;
    break;
   }
  }
  $333 = $i;
  $334 = (($333) + -1)|0;
  $i = $334;
 }
 if ((label|0) == 4) {
  $46 = $ret;
  $0 = $46;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 12) {
  $76 = $ret;
  $0 = $76;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 15) {
  $134 = $ret;
  $0 = $134;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 23) {
  $164 = $ret;
  $0 = $164;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 28) {
  $233 = $ret;
  $0 = $233;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 34) {
  $261 = $ret;
  $0 = $261;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 40) {
  $270 = $ret;
  $0 = $270;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 49) {
  $326 = HEAP32[$x_list>>2]|0;
  _free($326);
  $327 = HEAP32[$y_list>>2]|0;
  _free($327);
  $328 = $ret;
  $0 = $328;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 else if ((label|0) == 59) {
  $0 = 0;
  $335 = $0;
  STACKTOP = sp;return ($335|0);
 }
 return 0|0;
}
function _remove_pores_V2($minutiae,$bdata,$iw,$ih,$direction_map,$low_flow_map,$high_curve_map,$mw,$mh,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $direction_map = $direction_map|0;
 $low_flow_map = $low_flow_map|0;
 $high_curve_map = $high_curve_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0.0, $103 = 0, $104 = 0.0, $105 = 0.0, $106 = 0.0, $107 = 0.0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0.0, $113 = 0.0, $114 = 0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0.0, $124 = 0, $125 = 0.0, $126 = 0.0, $127 = 0.0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0.0;
 var $134 = 0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0.0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0.0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0.0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0.0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0.0, $439 = 0.0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0.0, $443 = 0, $444 = 0.0, $445 = 0.0, $446 = 0.0, $447 = 0.0, $448 = 0, $449 = 0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0.0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0, $79 = 0.0;
 var $8 = 0, $80 = 0.0, $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0, $93 = 0, $94 = 0, $95 = 0.0, $96 = 0.0, $97 = 0.0;
 var $98 = 0.0, $99 = 0, $ab2 = 0.0, $ax = 0, $ay = 0, $blk_x = 0, $blk_y = 0, $bx = 0, $by = 0, $cd2 = 0.0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0, $cos_theta = 0.0, $cx = 0, $cy = 0, $drx = 0.0, $dry = 0.0, $dx = 0;
 var $dy = 0, $i = 0, $minutia = 0, $ncontour = 0, $pex = 0, $pey = 0, $pi_factor = 0.0, $px = 0, $py = 0, $qex = 0, $qey = 0, $qx = 0, $qy = 0, $ratio = 0.0, $removed = 0, $ret = 0, $rx = 0, $ry = 0, $sin_theta = 0.0, $theta = 0.0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer20 = 0, $vararg_buffer36 = 0, $vararg_buffer39 = 0, $vararg_buffer4 = 0, $vararg_buffer43 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0, $vararg_ptr24 = 0, $vararg_ptr25 = 0, $vararg_ptr26 = 0, $vararg_ptr27 = 0, $vararg_ptr28 = 0, $vararg_ptr29 = 0;
 var $vararg_ptr3 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr33 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr42 = 0, $vararg_ptr46 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $vararg_buffer43 = sp + 192|0;
 $vararg_buffer39 = sp + 184|0;
 $vararg_buffer36 = sp + 176|0;
 $vararg_buffer20 = sp + 120|0;
 $vararg_buffer16 = sp + 112|0;
 $vararg_buffer12 = sp + 104|0;
 $vararg_buffer8 = sp + 96|0;
 $vararg_buffer4 = sp + 88|0;
 $vararg_buffer1 = sp + 80|0;
 $vararg_buffer = sp + 72|0;
 $px = sp + 284|0;
 $py = sp + 280|0;
 $pex = sp + 276|0;
 $pey = sp + 272|0;
 $qx = sp + 252|0;
 $qy = sp + 248|0;
 $qex = sp + 244|0;
 $qey = sp + 240|0;
 $contour_x = sp + 216|0;
 $contour_y = sp + 212|0;
 $contour_ex = sp + 208|0;
 $contour_ey = sp + 204|0;
 $ncontour = sp + 200|0;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $direction_map;
 $6 = $low_flow_map;
 $7 = $high_curve_map;
 $8 = $mw;
 $9 = $mh;
 $10 = $lfsparms;
 _print2log(6544,$vararg_buffer);
 $11 = $10;
 $12 = (($11) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (+($13|0));
 $15 = 3.141592653589793116 / $14;
 $pi_factor = $15;
 $i = 0;
 L1: while(1) {
  $16 = $i;
  $17 = $1;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   label = 88;
   break;
  }
  $21 = $i;
  $22 = $1;
  $23 = (($22) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $minutia = $26;
  $removed = 0;
  $27 = $minutia;
  $28 = HEAP32[$27>>2]|0;
  $29 = $10;
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($28|0) / ($31|0))&-1;
  $blk_x = $32;
  $33 = $minutia;
  $34 = (($33) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = $10;
  $37 = (($36) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($35|0) / ($38|0))&-1;
  $blk_y = $39;
  $40 = $6;
  $41 = $blk_y;
  $42 = $8;
  $43 = Math_imul($41, $42)|0;
  $44 = (($40) + ($43<<2)|0);
  $45 = $blk_x;
  $46 = (($44) + ($45<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = ($47|0)!=(0);
  if ($48) {
   label = 5;
  } else {
   $49 = $7;
   $50 = $blk_y;
   $51 = $8;
   $52 = Math_imul($50, $51)|0;
   $53 = (($49) + ($52<<2)|0);
   $54 = $blk_x;
   $55 = (($53) + ($54<<2)|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = ($56|0)!=(0);
   if ($57) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $58 = $5;
   $59 = $blk_y;
   $60 = $8;
   $61 = Math_imul($59, $60)|0;
   $62 = (($58) + ($61<<2)|0);
   $63 = $blk_x;
   $64 = (($62) + ($63<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($65|0)>=(0);
   if ($66) {
    $67 = $minutia;
    $68 = (($67) + 16|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (+($69|0));
    $71 = $pi_factor;
    $72 = $70 * $71;
    $theta = $72;
    $73 = $theta;
    $74 = (+Math_sin((+$73)));
    $sin_theta = $74;
    $75 = $theta;
    $76 = (+Math_cos((+$75)));
    $cos_theta = $76;
    $77 = $minutia;
    $78 = HEAP32[$77>>2]|0;
    $79 = (+($78|0));
    $80 = $sin_theta;
    $81 = $10;
    $82 = (($81) + 300|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = (+($83|0));
    $85 = $80 * $84;
    $86 = $79 - $85;
    $drx = $86;
    $87 = $minutia;
    $88 = (($87) + 4|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = (+($89|0));
    $91 = $cos_theta;
    $92 = $10;
    $93 = (($92) + 300|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = (+($94|0));
    $96 = $91 * $95;
    $97 = $90 + $96;
    $dry = $97;
    $98 = $drx;
    $99 = $98 < 0.0;
    if ($99) {
     $100 = $drx;
     $101 = $100 * 16384.0;
     $102 = $101 - 0.5;
     $103 = (~~(($102)));
     $104 = (+($103|0));
     $105 = $104 / 16384.0;
     $112 = $105;
    } else {
     $106 = $drx;
     $107 = $106 * 16384.0;
     $108 = $107 + 0.5;
     $109 = (~~(($108)));
     $110 = (+($109|0));
     $111 = $110 / 16384.0;
     $112 = $111;
    }
    $drx = $112;
    $113 = $dry;
    $114 = $113 < 0.0;
    if ($114) {
     $115 = $dry;
     $116 = $115 * 16384.0;
     $117 = $116 - 0.5;
     $118 = (~~(($117)));
     $119 = (+($118|0));
     $120 = $119 / 16384.0;
     $127 = $120;
    } else {
     $121 = $dry;
     $122 = $121 * 16384.0;
     $123 = $122 + 0.5;
     $124 = (~~(($123)));
     $125 = (+($124|0));
     $126 = $125 / 16384.0;
     $127 = $126;
    }
    $dry = $127;
    $128 = $drx;
    $129 = $128 < 0.0;
    if ($129) {
     $130 = $drx;
     $131 = $130 - 0.5;
     $135 = $131;
    } else {
     $132 = $drx;
     $133 = $132 + 0.5;
     $135 = $133;
    }
    $134 = (~~(($135)));
    $rx = $134;
    $136 = $dry;
    $137 = $136 < 0.0;
    if ($137) {
     $138 = $dry;
     $139 = $138 - 0.5;
     $143 = $139;
    } else {
     $140 = $dry;
     $141 = $140 + 0.5;
     $143 = $141;
    }
    $142 = (~~(($143)));
    $ry = $142;
    $144 = $2;
    $145 = $ry;
    $146 = $3;
    $147 = Math_imul($145, $146)|0;
    $148 = (($144) + ($147)|0);
    $149 = $rx;
    $150 = (($148) + ($149)|0);
    $151 = HEAP8[$150>>0]|0;
    $152 = $151&255;
    $153 = $minutia;
    $154 = (($153) + 32|0);
    $155 = HEAP32[$154>>2]|0;
    $156 = ($152|0)!=($155|0);
    if ($156) {
     $157 = $minutia;
     $158 = (($157) + 32|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = $rx;
     $161 = $ry;
     $162 = $cos_theta;
     $163 = -$162;
     $164 = $sin_theta;
     $165 = -$164;
     $166 = $10;
     $167 = (($166) + 304|0);
     $168 = HEAP32[$167>>2]|0;
     $169 = $2;
     $170 = $3;
     $171 = $4;
     $172 = (_search_in_direction($px,$py,$pex,$pey,$159,$160,$161,$163,$165,$168,$169,$170,$171)|0);
     $173 = ($172|0)!=(0);
     if ($173) {
      $174 = $10;
      $175 = (($174) + 308|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = HEAP32[$px>>2]|0;
      $178 = HEAP32[$py>>2]|0;
      $179 = HEAP32[$px>>2]|0;
      $180 = HEAP32[$py>>2]|0;
      $181 = HEAP32[$pex>>2]|0;
      $182 = HEAP32[$pey>>2]|0;
      $183 = $2;
      $184 = $3;
      $185 = $4;
      $186 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$176,$177,$178,$179,$180,$181,$182,1,$183,$184,$185)|0);
      $ret = $186;
      $187 = $ret;
      $188 = ($187|0)<(0);
      if ($188) {
       label = 21;
       break;
      }
      $190 = $ret;
      $191 = ($190|0)==(2);
      if ($191) {
       label = 25;
      } else {
       $192 = $ret;
       $193 = ($192|0)==(1);
       if ($193) {
        label = 25;
       } else {
        $194 = HEAP32[$ncontour>>2]|0;
        $195 = $10;
        $196 = (($195) + 308|0);
        $197 = HEAP32[$196>>2]|0;
        $198 = ($194|0)<($197|0);
        if ($198) {
         label = 25;
        } else {
         $220 = HEAP32[$ncontour>>2]|0;
         $221 = (($220) - 1)|0;
         $222 = HEAP32[$contour_x>>2]|0;
         $223 = (($222) + ($221<<2)|0);
         $224 = HEAP32[$223>>2]|0;
         $bx = $224;
         $225 = HEAP32[$ncontour>>2]|0;
         $226 = (($225) - 1)|0;
         $227 = HEAP32[$contour_y>>2]|0;
         $228 = (($227) + ($226<<2)|0);
         $229 = HEAP32[$228>>2]|0;
         $by = $229;
         $230 = HEAP32[$contour_x>>2]|0;
         $231 = HEAP32[$contour_y>>2]|0;
         $232 = HEAP32[$contour_ex>>2]|0;
         $233 = HEAP32[$contour_ey>>2]|0;
         _free_contour($230,$231,$232,$233);
         $234 = $10;
         $235 = (($234) + 312|0);
         $236 = HEAP32[$235>>2]|0;
         $237 = HEAP32[$px>>2]|0;
         $238 = HEAP32[$py>>2]|0;
         $239 = HEAP32[$px>>2]|0;
         $240 = HEAP32[$py>>2]|0;
         $241 = HEAP32[$pex>>2]|0;
         $242 = HEAP32[$pey>>2]|0;
         $243 = $2;
         $244 = $3;
         $245 = $4;
         $246 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$236,$237,$238,$239,$240,$241,$242,0,$243,$244,$245)|0);
         $ret = $246;
         $247 = $ret;
         $248 = ($247|0)<(0);
         if ($248) {
          label = 32;
          break;
         }
         $250 = $ret;
         $251 = ($250|0)==(2);
         do {
          if ($251) {
           label = 36;
          } else {
           $252 = $ret;
           $253 = ($252|0)==(1);
           if ($253) {
            label = 36;
            break;
           }
           $254 = HEAP32[$ncontour>>2]|0;
           $255 = $10;
           $256 = (($255) + 312|0);
           $257 = HEAP32[$256>>2]|0;
           $258 = ($254|0)<($257|0);
           if ($258) {
            label = 36;
            break;
           }
           $280 = HEAP32[$ncontour>>2]|0;
           $281 = (($280) - 1)|0;
           $282 = HEAP32[$contour_x>>2]|0;
           $283 = (($282) + ($281<<2)|0);
           $284 = HEAP32[$283>>2]|0;
           $dx = $284;
           $285 = HEAP32[$ncontour>>2]|0;
           $286 = (($285) - 1)|0;
           $287 = HEAP32[$contour_y>>2]|0;
           $288 = (($287) + ($286<<2)|0);
           $289 = HEAP32[$288>>2]|0;
           $dy = $289;
           $290 = HEAP32[$contour_x>>2]|0;
           $291 = HEAP32[$contour_y>>2]|0;
           $292 = HEAP32[$contour_ex>>2]|0;
           $293 = HEAP32[$contour_ey>>2]|0;
           _free_contour($290,$291,$292,$293);
           $294 = $minutia;
           $295 = (($294) + 32|0);
           $296 = HEAP32[$295>>2]|0;
           $297 = $rx;
           $298 = $ry;
           $299 = $cos_theta;
           $300 = $sin_theta;
           $301 = $10;
           $302 = (($301) + 304|0);
           $303 = HEAP32[$302>>2]|0;
           $304 = $2;
           $305 = $3;
           $306 = $4;
           $307 = (_search_in_direction($qx,$qy,$qex,$qey,$296,$297,$298,$299,$300,$303,$304,$305,$306)|0);
           $308 = ($307|0)!=(0);
           if ($308) {
            $309 = $10;
            $310 = (($309) + 308|0);
            $311 = HEAP32[$310>>2]|0;
            $312 = HEAP32[$qx>>2]|0;
            $313 = HEAP32[$qy>>2]|0;
            $314 = HEAP32[$qx>>2]|0;
            $315 = HEAP32[$qy>>2]|0;
            $316 = HEAP32[$qex>>2]|0;
            $317 = HEAP32[$qey>>2]|0;
            $318 = $2;
            $319 = $3;
            $320 = $4;
            $321 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$311,$312,$313,$314,$315,$316,$317,0,$318,$319,$320)|0);
            $ret = $321;
            $322 = $ret;
            $323 = ($322|0)<(0);
            if ($323) {
             label = 44;
             break L1;
            }
            $325 = $ret;
            $326 = ($325|0)==(2);
            do {
             if ($326) {
              label = 48;
             } else {
              $327 = $ret;
              $328 = ($327|0)==(1);
              if ($328) {
               label = 48;
               break;
              }
              $329 = HEAP32[$ncontour>>2]|0;
              $330 = $10;
              $331 = (($330) + 308|0);
              $332 = HEAP32[$331>>2]|0;
              $333 = ($329|0)<($332|0);
              if ($333) {
               label = 48;
               break;
              }
              $355 = HEAP32[$ncontour>>2]|0;
              $356 = (($355) - 1)|0;
              $357 = HEAP32[$contour_x>>2]|0;
              $358 = (($357) + ($356<<2)|0);
              $359 = HEAP32[$358>>2]|0;
              $ax = $359;
              $360 = HEAP32[$ncontour>>2]|0;
              $361 = (($360) - 1)|0;
              $362 = HEAP32[$contour_y>>2]|0;
              $363 = (($362) + ($361<<2)|0);
              $364 = HEAP32[$363>>2]|0;
              $ay = $364;
              $365 = HEAP32[$contour_x>>2]|0;
              $366 = HEAP32[$contour_y>>2]|0;
              $367 = HEAP32[$contour_ex>>2]|0;
              $368 = HEAP32[$contour_ey>>2]|0;
              _free_contour($365,$366,$367,$368);
              $369 = $10;
              $370 = (($369) + 312|0);
              $371 = HEAP32[$370>>2]|0;
              $372 = HEAP32[$qx>>2]|0;
              $373 = HEAP32[$qy>>2]|0;
              $374 = HEAP32[$qx>>2]|0;
              $375 = HEAP32[$qy>>2]|0;
              $376 = HEAP32[$qex>>2]|0;
              $377 = HEAP32[$qey>>2]|0;
              $378 = $2;
              $379 = $3;
              $380 = $4;
              $381 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$371,$372,$373,$374,$375,$376,$377,1,$378,$379,$380)|0);
              $ret = $381;
              $382 = $ret;
              $383 = ($382|0)<(0);
              if ($383) {
               label = 55;
               break L1;
              }
              $385 = $ret;
              $386 = ($385|0)==(2);
              do {
               if ($386) {
                label = 59;
               } else {
                $387 = $ret;
                $388 = ($387|0)==(1);
                if ($388) {
                 label = 59;
                 break;
                }
                $389 = HEAP32[$ncontour>>2]|0;
                $390 = $10;
                $391 = (($390) + 312|0);
                $392 = HEAP32[$391>>2]|0;
                $393 = ($389|0)<($392|0);
                if ($393) {
                 label = 59;
                 break;
                }
                $415 = HEAP32[$ncontour>>2]|0;
                $416 = (($415) - 1)|0;
                $417 = HEAP32[$contour_x>>2]|0;
                $418 = (($417) + ($416<<2)|0);
                $419 = HEAP32[$418>>2]|0;
                $cx = $419;
                $420 = HEAP32[$ncontour>>2]|0;
                $421 = (($420) - 1)|0;
                $422 = HEAP32[$contour_y>>2]|0;
                $423 = (($422) + ($421<<2)|0);
                $424 = HEAP32[$423>>2]|0;
                $cy = $424;
                $425 = HEAP32[$contour_x>>2]|0;
                $426 = HEAP32[$contour_y>>2]|0;
                $427 = HEAP32[$contour_ex>>2]|0;
                $428 = HEAP32[$contour_ey>>2]|0;
                _free_contour($425,$426,$427,$428);
                $429 = $ax;
                $430 = $ay;
                $431 = $bx;
                $432 = $by;
                $433 = (+_squared_distance($429,$430,$431,$432));
                $ab2 = $433;
                $434 = $cx;
                $435 = $cy;
                $436 = $dx;
                $437 = $dy;
                $438 = (+_squared_distance($434,$435,$436,$437));
                $cd2 = $438;
                $439 = $cd2;
                $440 = $10;
                $441 = (($440) + 320|0);
                $442 = +HEAPF64[$441>>3];
                $443 = $439 > $442;
                if ($443) {
                 $444 = $ab2;
                 $445 = $cd2;
                 $446 = $444 / $445;
                 $ratio = $446;
                 $447 = $ratio;
                 $448 = $10;
                 $449 = (($448) + 328|0);
                 $450 = +HEAPF64[$449>>3];
                 $451 = $447 <= $450;
                 if ($451) {
                  $452 = $minutia;
                  $453 = HEAP32[$452>>2]|0;
                  $454 = $minutia;
                  $455 = (($454) + 4|0);
                  $456 = HEAP32[$455>>2]|0;
                  HEAP32[$vararg_buffer16>>2] = $453;
                  $vararg_ptr19 = (($vararg_buffer16) + 4|0);
                  HEAP32[$vararg_ptr19>>2] = $456;
                  _print2log(6600,$vararg_buffer16);
                  $457 = $rx;
                  $458 = $ry;
                  $459 = HEAP32[$px>>2]|0;
                  $460 = HEAP32[$py>>2]|0;
                  $461 = $bx;
                  $462 = $by;
                  $463 = $dx;
                  $464 = $dy;
                  $465 = HEAP32[$qx>>2]|0;
                  $466 = HEAP32[$qy>>2]|0;
                  $467 = $ax;
                  $468 = $ay;
                  $469 = $cx;
                  $470 = $cy;
                  HEAP32[$vararg_buffer20>>2] = $457;
                  $vararg_ptr23 = (($vararg_buffer20) + 4|0);
                  HEAP32[$vararg_ptr23>>2] = $458;
                  $vararg_ptr24 = (($vararg_buffer20) + 8|0);
                  HEAP32[$vararg_ptr24>>2] = $459;
                  $vararg_ptr25 = (($vararg_buffer20) + 12|0);
                  HEAP32[$vararg_ptr25>>2] = $460;
                  $vararg_ptr26 = (($vararg_buffer20) + 16|0);
                  HEAP32[$vararg_ptr26>>2] = $461;
                  $vararg_ptr27 = (($vararg_buffer20) + 20|0);
                  HEAP32[$vararg_ptr27>>2] = $462;
                  $vararg_ptr28 = (($vararg_buffer20) + 24|0);
                  HEAP32[$vararg_ptr28>>2] = $463;
                  $vararg_ptr29 = (($vararg_buffer20) + 28|0);
                  HEAP32[$vararg_ptr29>>2] = $464;
                  $vararg_ptr30 = (($vararg_buffer20) + 32|0);
                  HEAP32[$vararg_ptr30>>2] = $465;
                  $vararg_ptr31 = (($vararg_buffer20) + 36|0);
                  HEAP32[$vararg_ptr31>>2] = $466;
                  $vararg_ptr32 = (($vararg_buffer20) + 40|0);
                  HEAP32[$vararg_ptr32>>2] = $467;
                  $vararg_ptr33 = (($vararg_buffer20) + 44|0);
                  HEAP32[$vararg_ptr33>>2] = $468;
                  $vararg_ptr34 = (($vararg_buffer20) + 48|0);
                  HEAP32[$vararg_ptr34>>2] = $469;
                  $vararg_ptr35 = (($vararg_buffer20) + 52|0);
                  HEAP32[$vararg_ptr35>>2] = $470;
                  _print2log(6608,$vararg_buffer20);
                  $471 = $ratio;
                  HEAPF64[tempDoublePtr>>3]=$471;HEAP32[$vararg_buffer36>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer36+4>>2]=HEAP32[tempDoublePtr+4>>2];
                  _print2log(6704,$vararg_buffer36);
                  $472 = $i;
                  $473 = $1;
                  $474 = (_remove_minutia($472,$473)|0);
                  $ret = $474;
                  $475 = ($474|0)!=(0);
                  if ($475) {
                   label = 68;
                   break L1;
                  }
                  $removed = 1;
                 }
                }
               }
              } while(0);
              if ((label|0) == 59) {
               label = 0;
               $394 = $ret;
               $395 = ($394|0)==(1);
               if ($395) {
                label = 61;
               } else {
                $396 = HEAP32[$ncontour>>2]|0;
                $397 = $10;
                $398 = (($397) + 312|0);
                $399 = HEAP32[$398>>2]|0;
                $400 = ($396|0)<($399|0);
                if ($400) {
                 label = 61;
                }
               }
               if ((label|0) == 61) {
                label = 0;
                $401 = HEAP32[$contour_x>>2]|0;
                $402 = HEAP32[$contour_y>>2]|0;
                $403 = HEAP32[$contour_ex>>2]|0;
                $404 = HEAP32[$contour_ey>>2]|0;
                _free_contour($401,$402,$403,$404);
               }
               $405 = $minutia;
               $406 = HEAP32[$405>>2]|0;
               $407 = $minutia;
               $408 = (($407) + 4|0);
               $409 = HEAP32[$408>>2]|0;
               HEAP32[$vararg_buffer12>>2] = $406;
               $vararg_ptr15 = (($vararg_buffer12) + 4|0);
               HEAP32[$vararg_ptr15>>2] = $409;
               _print2log(6584,$vararg_buffer12);
               $410 = $i;
               $411 = $1;
               $412 = (_remove_minutia($410,$411)|0);
               $ret = $412;
               $413 = ($412|0)!=(0);
               if ($413) {
                label = 63;
                break L1;
               }
               $removed = 1;
              }
             }
            } while(0);
            if ((label|0) == 48) {
             label = 0;
             $334 = $ret;
             $335 = ($334|0)==(1);
             if ($335) {
              label = 50;
             } else {
              $336 = HEAP32[$ncontour>>2]|0;
              $337 = $10;
              $338 = (($337) + 308|0);
              $339 = HEAP32[$338>>2]|0;
              $340 = ($336|0)<($339|0);
              if ($340) {
               label = 50;
              }
             }
             if ((label|0) == 50) {
              label = 0;
              $341 = HEAP32[$contour_x>>2]|0;
              $342 = HEAP32[$contour_y>>2]|0;
              $343 = HEAP32[$contour_ex>>2]|0;
              $344 = HEAP32[$contour_ey>>2]|0;
              _free_contour($341,$342,$343,$344);
             }
             $345 = $minutia;
             $346 = HEAP32[$345>>2]|0;
             $347 = $minutia;
             $348 = (($347) + 4|0);
             $349 = HEAP32[$348>>2]|0;
             HEAP32[$vararg_buffer8>>2] = $346;
             $vararg_ptr11 = (($vararg_buffer8) + 4|0);
             HEAP32[$vararg_ptr11>>2] = $349;
             _print2log(5960,$vararg_buffer8);
             $350 = $i;
             $351 = $1;
             $352 = (_remove_minutia($350,$351)|0);
             $ret = $352;
             $353 = ($352|0)!=(0);
             if ($353) {
              label = 52;
              break L1;
             }
             $removed = 1;
            }
           } else {
            $477 = $minutia;
            $478 = HEAP32[$477>>2]|0;
            $479 = $minutia;
            $480 = (($479) + 4|0);
            $481 = HEAP32[$480>>2]|0;
            HEAP32[$vararg_buffer39>>2] = $478;
            $vararg_ptr42 = (($vararg_buffer39) + 4|0);
            HEAP32[$vararg_ptr42>>2] = $481;
            _print2log(6672,$vararg_buffer39);
            $482 = $i;
            $483 = $1;
            $484 = (_remove_minutia($482,$483)|0);
            $ret = $484;
            $485 = ($484|0)!=(0);
            if ($485) {
             label = 75;
             break L1;
            }
            $removed = 1;
           }
          }
         } while(0);
         if ((label|0) == 36) {
          label = 0;
          $259 = $ret;
          $260 = ($259|0)==(1);
          if ($260) {
           label = 38;
          } else {
           $261 = HEAP32[$ncontour>>2]|0;
           $262 = $10;
           $263 = (($262) + 312|0);
           $264 = HEAP32[$263>>2]|0;
           $265 = ($261|0)<($264|0);
           if ($265) {
            label = 38;
           }
          }
          if ((label|0) == 38) {
           label = 0;
           $266 = HEAP32[$contour_x>>2]|0;
           $267 = HEAP32[$contour_y>>2]|0;
           $268 = HEAP32[$contour_ex>>2]|0;
           $269 = HEAP32[$contour_ey>>2]|0;
           _free_contour($266,$267,$268,$269);
          }
          $270 = $minutia;
          $271 = HEAP32[$270>>2]|0;
          $272 = $minutia;
          $273 = (($272) + 4|0);
          $274 = HEAP32[$273>>2]|0;
          HEAP32[$vararg_buffer4>>2] = $271;
          $vararg_ptr7 = (($vararg_buffer4) + 4|0);
          HEAP32[$vararg_ptr7>>2] = $274;
          _print2log(6568,$vararg_buffer4);
          $275 = $i;
          $276 = $1;
          $277 = (_remove_minutia($275,$276)|0);
          $ret = $277;
          $278 = ($277|0)!=(0);
          if ($278) {
           label = 40;
           break;
          }
          $removed = 1;
         }
        }
       }
      }
      if ((label|0) == 25) {
       label = 0;
       $199 = $ret;
       $200 = ($199|0)==(1);
       if ($200) {
        label = 27;
       } else {
        $201 = HEAP32[$ncontour>>2]|0;
        $202 = $10;
        $203 = (($202) + 308|0);
        $204 = HEAP32[$203>>2]|0;
        $205 = ($201|0)<($204|0);
        if ($205) {
         label = 27;
        }
       }
       if ((label|0) == 27) {
        label = 0;
        $206 = HEAP32[$contour_x>>2]|0;
        $207 = HEAP32[$contour_y>>2]|0;
        $208 = HEAP32[$contour_ex>>2]|0;
        $209 = HEAP32[$contour_ey>>2]|0;
        _free_contour($206,$207,$208,$209);
       }
       $210 = $minutia;
       $211 = HEAP32[$210>>2]|0;
       $212 = $minutia;
       $213 = (($212) + 4|0);
       $214 = HEAP32[$213>>2]|0;
       HEAP32[$vararg_buffer1>>2] = $211;
       $vararg_ptr3 = (($vararg_buffer1) + 4|0);
       HEAP32[$vararg_ptr3>>2] = $214;
       _print2log(5976,$vararg_buffer1);
       $215 = $i;
       $216 = $1;
       $217 = (_remove_minutia($215,$216)|0);
       $ret = $217;
       $218 = ($217|0)!=(0);
       if ($218) {
        label = 29;
        break;
       }
       $removed = 1;
      }
     } else {
      $487 = $minutia;
      $488 = HEAP32[$487>>2]|0;
      $489 = $minutia;
      $490 = (($489) + 4|0);
      $491 = HEAP32[$490>>2]|0;
      HEAP32[$vararg_buffer43>>2] = $488;
      $vararg_ptr46 = (($vararg_buffer43) + 4|0);
      HEAP32[$vararg_ptr46>>2] = $491;
      _print2log(6688,$vararg_buffer43);
      $492 = $i;
      $493 = $1;
      $494 = (_remove_minutia($492,$493)|0);
      $ret = $494;
      $495 = ($494|0)!=(0);
      if ($495) {
       label = 81;
       break;
      }
      $removed = 1;
     }
    }
   }
  }
  $497 = $removed;
  $498 = ($497|0)!=(0);
  if (!($498)) {
   $499 = $i;
   $500 = (($499) + 1)|0;
   $i = $500;
  }
 }
 switch (label|0) {
  case 21: {
   $189 = $ret;
   $0 = $189;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 29: {
   $219 = $ret;
   $0 = $219;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 32: {
   $249 = $ret;
   $0 = $249;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 40: {
   $279 = $ret;
   $0 = $279;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 44: {
   $324 = $ret;
   $0 = $324;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 52: {
   $354 = $ret;
   $0 = $354;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 55: {
   $384 = $ret;
   $0 = $384;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 63: {
   $414 = $ret;
   $0 = $414;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 68: {
   $476 = $ret;
   $0 = $476;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 75: {
   $486 = $ret;
   $0 = $486;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 81: {
   $496 = $ret;
   $0 = $496;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
  case 88: {
   $0 = 0;
   $501 = $0;
   STACKTOP = sp;return ($501|0);
   break;
  }
 }
 return 0|0;
}
function _count_minutiae_ridges($minutiae,$bdata,$iw,$ih,$lfsparms) {
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $ret = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $1 = $minutiae;
 $2 = $bdata;
 $3 = $iw;
 $4 = $ih;
 $5 = $lfsparms;
 _print2log(6880,$vararg_buffer);
 $6 = $1;
 $7 = $3;
 $8 = $4;
 $9 = (_sort_minutiae_x_y($6,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 $12 = $1;
 $13 = (_rm_dup_minutiae($12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $ret;
  $0 = $15;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 $i = 0;
 while(1) {
  $16 = $i;
  $17 = $1;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - 1)|0;
  $21 = ($16|0)<($20|0);
  if (!($21)) {
   label = 11;
   break;
  }
  $22 = $i;
  $23 = $1;
  $24 = $2;
  $25 = $3;
  $26 = $4;
  $27 = $5;
  $28 = (_count_minutia_ridges($22,$23,$24,$25,$26,$27)|0);
  $ret = $28;
  $29 = ($28|0)!=(0);
  if ($29) {
   label = 8;
   break;
  }
  $31 = $i;
  $32 = (($31) + 1)|0;
  $i = $32;
 }
 if ((label|0) == 8) {
  $30 = $ret;
  $0 = $30;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 else if ((label|0) == 11) {
  $0 = 0;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 return 0|0;
}
function _count_minutia_ridges($first,$minutiae,$bdata,$iw,$ih,$lfsparms) {
 $first = $first|0;
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $i = 0;
 var $nbr_list = 0, $nbr_nridges = 0, $nnbrs = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $nbr_list = sp + 28|0;
 $nnbrs = sp + 20|0;
 $1 = $first;
 $2 = $minutiae;
 $3 = $bdata;
 $4 = $iw;
 $5 = $ih;
 $6 = $lfsparms;
 $7 = $6;
 $8 = (($7) + 336|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $1;
 $11 = $2;
 $12 = (_find_neighbors($nbr_list,$nnbrs,$9,$10,$11)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$nbr_list>>2]|0;
  _free($14);
  $15 = $ret;
  $0 = $15;
  $98 = $0;
  STACKTOP = sp;return ($98|0);
 }
 $16 = $1;
 $17 = $2;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + ($16<<2)|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = $1;
 $24 = $2;
 $25 = (($24) + 8|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($23<<2)|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$nnbrs>>2]|0;
 HEAP32[$vararg_buffer>>2] = $22;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $30;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $31;
 _print2log(6920,$vararg_buffer);
 $32 = HEAP32[$nnbrs>>2]|0;
 $33 = ($32|0)==(0);
 if ($33) {
  $0 = 0;
  $98 = $0;
  STACKTOP = sp;return ($98|0);
 }
 $34 = HEAP32[$nbr_list>>2]|0;
 $35 = HEAP32[$nnbrs>>2]|0;
 $36 = $1;
 $37 = $2;
 $38 = (_sort_neighbors($34,$35,$36,$37)|0);
 $ret = $38;
 $39 = ($38|0)!=(0);
 if ($39) {
  $40 = HEAP32[$nbr_list>>2]|0;
  _free($40);
  $41 = $ret;
  $0 = $41;
  $98 = $0;
  STACKTOP = sp;return ($98|0);
 }
 $42 = HEAP32[$nnbrs>>2]|0;
 $43 = $42<<2;
 $44 = (_malloc($43)|0);
 $nbr_nridges = $44;
 $45 = $nbr_nridges;
 $46 = ($45|0)==(0|0);
 if ($46) {
  $47 = HEAP32[$nbr_list>>2]|0;
  _free($47);
  $48 = HEAP32[_stderr>>2]|0;
  (_fprintf(($48|0),(6944|0),($vararg_buffer3|0))|0);
  $0 = -450;
  $98 = $0;
  STACKTOP = sp;return ($98|0);
 }
 $i = 0;
 while(1) {
  $49 = $i;
  $50 = HEAP32[$nnbrs>>2]|0;
  $51 = ($49|0)<($50|0);
  if (!($51)) {
   label = 15;
   break;
  }
  $52 = $1;
  $53 = $i;
  $54 = HEAP32[$nbr_list>>2]|0;
  $55 = (($54) + ($53<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $2;
  $58 = $3;
  $59 = $4;
  $60 = $5;
  $61 = $6;
  $62 = (_ridge_count($52,$56,$57,$58,$59,$60,$61)|0);
  $ret = $62;
  $63 = $ret;
  $64 = ($63|0)<(0);
  if ($64) {
   label = 12;
   break;
  }
  $68 = $ret;
  $69 = $i;
  $70 = $nbr_nridges;
  $71 = (($70) + ($69<<2)|0);
  HEAP32[$71>>2] = $68;
  $72 = $i;
  $73 = (($72) + 1)|0;
  $i = $73;
 }
 if ((label|0) == 12) {
  $65 = HEAP32[$nbr_list>>2]|0;
  _free($65);
  $66 = $nbr_nridges;
  _free($66);
  $67 = $ret;
  $0 = $67;
  $98 = $0;
  STACKTOP = sp;return ($98|0);
 }
 else if ((label|0) == 15) {
  $74 = HEAP32[$nbr_list>>2]|0;
  $75 = $1;
  $76 = $2;
  $77 = (($76) + 8|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (($78) + ($75<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = (($80) + 44|0);
  HEAP32[$81>>2] = $74;
  $82 = $nbr_nridges;
  $83 = $1;
  $84 = $2;
  $85 = (($84) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + ($83<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($88) + 48|0);
  HEAP32[$89>>2] = $82;
  $90 = HEAP32[$nnbrs>>2]|0;
  $91 = $1;
  $92 = $2;
  $93 = (($92) + 8|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 52|0);
  HEAP32[$97>>2] = $90;
  $0 = 0;
  $98 = $0;
  STACKTOP = sp;return ($98|0);
 }
 return 0|0;
}
function _find_neighbors($onbr_list,$onnbrs,$max_nbrs,$first,$minutiae) {
 $onbr_list = $onbr_list|0;
 $onnbrs = $onnbrs|0;
 $max_nbrs = $max_nbrs|0;
 $first = $first|0;
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $last_nbr = 0, $minutia1 = 0, $minutia2 = 0, $nbr_list = 0, $nbr_sqr_dists = 0, $nnbrs = 0, $ret = 0, $second = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $xdist = 0.0, $xdist2 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer1 = sp + 24|0;
 $vararg_buffer = sp + 16|0;
 $nnbrs = sp + 32|0;
 $1 = $onbr_list;
 $2 = $onnbrs;
 $3 = $max_nbrs;
 $4 = $first;
 $5 = $minutiae;
 $6 = $3;
 $7 = $6<<2;
 $8 = (_malloc($7)|0);
 $nbr_list = $8;
 $9 = $nbr_list;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(7000|0),($vararg_buffer|0))|0);
  $0 = -460;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 $12 = $3;
 $13 = $12<<3;
 $14 = (_malloc($13)|0);
 $nbr_sqr_dists = $14;
 $15 = $nbr_sqr_dists;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = $nbr_list;
  _free($17);
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(7048|0),($vararg_buffer1|0))|0);
  $0 = -461;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 HEAP32[$nnbrs>>2] = 0;
 $19 = $4;
 $20 = (($19) + 1)|0;
 $second = $20;
 $21 = $3;
 $22 = (($21) - 1)|0;
 $last_nbr = $22;
 while(1) {
  $23 = $second;
  $24 = $5;
  $25 = (($24) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = $4;
  $29 = $5;
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $minutia1 = $33;
  $34 = $second;
  $35 = $5;
  $36 = (($35) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $minutia2 = $39;
  $40 = $minutia2;
  $41 = HEAP32[$40>>2]|0;
  $42 = $minutia1;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) - ($43))|0;
  $45 = (+($44|0));
  $xdist = $45;
  $46 = $xdist;
  $47 = $xdist;
  $48 = $46 * $47;
  $xdist2 = $48;
  $49 = HEAP32[$nnbrs>>2]|0;
  $50 = $3;
  $51 = ($49|0)<($50|0);
  if (!($51)) {
   $52 = $xdist2;
   $53 = $last_nbr;
   $54 = $nbr_sqr_dists;
   $55 = (($54) + ($53<<3)|0);
   $56 = +HEAPF64[$55>>3];
   $57 = $52 < $56;
   if (!($57)) {
    label = 12;
    break;
   }
  }
  $58 = $nbr_list;
  $59 = $nbr_sqr_dists;
  $60 = $3;
  $61 = $4;
  $62 = $second;
  $63 = $5;
  $64 = (_update_nbr_dists($58,$59,$nnbrs,$60,$61,$62,$63)|0);
  $ret = $64;
  $65 = ($64|0)!=(0);
  if ($65) {
   label = 10;
   break;
  }
  $68 = $second;
  $69 = (($68) + 1)|0;
  $second = $69;
 }
 if ((label|0) == 10) {
  $66 = $nbr_sqr_dists;
  _free($66);
  $67 = $ret;
  $0 = $67;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 12) {
 }
 $70 = $nbr_sqr_dists;
 _free($70);
 $71 = HEAP32[$nnbrs>>2]|0;
 $72 = ($71|0)==(0);
 if ($72) {
  $73 = $nbr_list;
  _free($73);
  $74 = $2;
  HEAP32[$74>>2] = 0;
 } else {
  $75 = $nbr_list;
  $76 = $1;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$nnbrs>>2]|0;
  $78 = $2;
  HEAP32[$78>>2] = $77;
 }
 $0 = 0;
 $79 = $0;
 STACKTOP = sp;return ($79|0);
}
function _sort_neighbors($nbr_list,$nnbrs,$first,$minutiae) {
 $nbr_list = $nbr_list|0;
 $nnbrs = $nnbrs|0;
 $first = $first|0;
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $join_thetas = 0, $theta = 0.0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $1 = $nbr_list;
 $2 = $nnbrs;
 $3 = $first;
 $4 = $minutiae;
 $5 = $2;
 $6 = $5<<3;
 $7 = (_malloc($6)|0);
 $join_thetas = $7;
 $8 = $join_thetas;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(7296|0),($vararg_buffer|0))|0);
  $0 = -490;
  $67 = $0;
  STACKTOP = sp;return ($67|0);
 }
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $2;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $i;
  $15 = $1;
  $16 = (($15) + ($14<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $4;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + ($17<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $i;
  $26 = $1;
  $27 = (($26) + ($25<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $4;
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $3;
  $36 = $4;
  $37 = (($36) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + ($35<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $3;
  $44 = $4;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + ($43<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$48>>2]|0;
  $50 = (+_angle2line($24,$34,$42,$49));
  $theta = $50;
  $51 = +HEAPF64[7288>>3];
  $52 = $theta;
  $53 = $52 + $51;
  $theta = $53;
  $54 = $theta;
  $55 = +HEAPF64[7288>>3];
  $56 = (+_fmod((+$54),(+$55)));
  $theta = $56;
  $57 = $theta;
  $58 = $i;
  $59 = $join_thetas;
  $60 = (($59) + ($58<<3)|0);
  HEAPF64[$60>>3] = $57;
  $61 = $i;
  $62 = (($61) + 1)|0;
  $i = $62;
 }
 $63 = $join_thetas;
 $64 = $1;
 $65 = $2;
 _bubble_sort_double_inc_2($63,$64,$65);
 $66 = $join_thetas;
 _free($66);
 $0 = 0;
 $67 = $0;
 STACKTOP = sp;return ($67|0);
}
function _ridge_count($first,$second,$minutiae,$bdata,$iw,$ih,$lfsparms) {
 $first = $first|0;
 $second = $second|0;
 $minutiae = $minutiae|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $curpix = 0, $found = 0, $i = 0, $minutia1 = 0, $minutia2 = 0, $num = 0, $prevpix = 0, $ret = 0, $ridge_count = 0, $ridge_end = 0, $ridge_start = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0;
 var $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer4 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr15 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr9 = 0, $xlist = 0, $ylist = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer19 = sp + 56|0;
 $vararg_buffer16 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer = sp;
 $i = sp + 100|0;
 $xlist = sp + 88|0;
 $ylist = sp + 84|0;
 $num = sp + 80|0;
 $1 = $first;
 $2 = $second;
 $3 = $minutiae;
 $4 = $bdata;
 $5 = $iw;
 $6 = $ih;
 $7 = $lfsparms;
 $8 = $1;
 $9 = $3;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($8<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 $minutia1 = $13;
 $14 = $2;
 $15 = $3;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<2)|0);
 $19 = HEAP32[$18>>2]|0;
 $minutia2 = $19;
 $20 = $minutia1;
 $21 = HEAP32[$20>>2]|0;
 $22 = $minutia2;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($21|0)==($23|0);
 if ($24) {
  $25 = $minutia1;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $minutia2;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)==($30|0);
  if ($31) {
   $0 = 0;
   $165 = $0;
   STACKTOP = sp;return ($165|0);
  }
 }
 $32 = $minutia1;
 $33 = HEAP32[$32>>2]|0;
 $34 = $minutia1;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = $minutia2;
 $38 = HEAP32[$37>>2]|0;
 $39 = $minutia2;
 $40 = (($39) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (_line_points($xlist,$ylist,$num,$33,$36,$38,$41)|0);
 $ret = $42;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = $ret;
  $0 = $44;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 $45 = HEAP32[$num>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  $47 = HEAP32[$xlist>>2]|0;
  _free($47);
  $48 = HEAP32[$ylist>>2]|0;
  _free($48);
  $0 = 0;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 $49 = $4;
 $50 = HEAP32[$ylist>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = $5;
 $53 = Math_imul($51, $52)|0;
 $54 = (($49) + ($53)|0);
 $55 = HEAP32[$xlist>>2]|0;
 $56 = HEAP32[$55>>2]|0;
 $57 = (($54) + ($56)|0);
 $58 = HEAP8[$57>>0]|0;
 $59 = $58&255;
 $prevpix = $59;
 HEAP32[$i>>2] = 1;
 $found = 0;
 while(1) {
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$num>>2]|0;
  $62 = ($60|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = $4;
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$ylist>>2]|0;
  $66 = (($65) + ($64<<2)|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = $5;
  $69 = Math_imul($67, $68)|0;
  $70 = (($63) + ($69)|0);
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$xlist>>2]|0;
  $73 = (($72) + ($71<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($70) + ($74)|0);
  $76 = HEAP8[$75>>0]|0;
  $77 = $76&255;
  $curpix = $77;
  $78 = $curpix;
  $79 = $prevpix;
  $80 = ($78|0)!=($79|0);
  if ($80) {
   label = 11;
   break;
  }
  $81 = HEAP32[$i>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$i>>2] = $82;
 }
 if ((label|0) == 11) {
  $found = 1;
 }
 $83 = $found;
 $84 = ($83|0)!=(0);
 if (!($84)) {
  $85 = HEAP32[$xlist>>2]|0;
  _free($85);
  $86 = HEAP32[$ylist>>2]|0;
  _free($86);
  $0 = 0;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 $ridge_count = 0;
 $87 = $minutia1;
 $88 = HEAP32[$87>>2]|0;
 $89 = $minutia1;
 $90 = (($89) + 4|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $minutia2;
 $93 = HEAP32[$92>>2]|0;
 $94 = $minutia2;
 $95 = (($94) + 4|0);
 $96 = HEAP32[$95>>2]|0;
 HEAP32[$vararg_buffer>>2] = $88;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $91;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $93;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $96;
 _print2log(7344,$vararg_buffer);
 while(1) {
  $97 = HEAP32[$i>>2]|0;
  $98 = HEAP32[$num>>2]|0;
  $99 = ($97|0)<($98|0);
  if (!($99)) {
   label = 26;
   break;
  }
  $100 = HEAP32[$xlist>>2]|0;
  $101 = HEAP32[$ylist>>2]|0;
  $102 = HEAP32[$num>>2]|0;
  $103 = $4;
  $104 = $5;
  $105 = $6;
  $106 = (_find_transition($i,0,1,$100,$101,$102,$103,$104,$105)|0);
  $107 = ($106|0)!=(0);
  if (!($107)) {
   label = 18;
   break;
  }
  $111 = HEAP32[$i>>2]|0;
  $ridge_start = $111;
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$xlist>>2]|0;
  $114 = (($113) + ($112<<2)|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = HEAP32[$i>>2]|0;
  $117 = HEAP32[$ylist>>2]|0;
  $118 = (($117) + ($116<<2)|0);
  $119 = HEAP32[$118>>2]|0;
  HEAP32[$vararg_buffer6>>2] = $115;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $119;
  _print2log(7384,$vararg_buffer6);
  $120 = HEAP32[$xlist>>2]|0;
  $121 = HEAP32[$ylist>>2]|0;
  $122 = HEAP32[$num>>2]|0;
  $123 = $4;
  $124 = $5;
  $125 = $6;
  $126 = (_find_transition($i,1,0,$120,$121,$122,$123,$124,$125)|0);
  $127 = ($126|0)!=(0);
  if (!($127)) {
   label = 20;
   break;
  }
  $131 = HEAP32[$i>>2]|0;
  $ridge_end = $131;
  $132 = HEAP32[$i>>2]|0;
  $133 = HEAP32[$xlist>>2]|0;
  $134 = (($133) + ($132<<2)|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = HEAP32[$i>>2]|0;
  $137 = HEAP32[$ylist>>2]|0;
  $138 = (($137) + ($136<<2)|0);
  $139 = HEAP32[$138>>2]|0;
  HEAP32[$vararg_buffer12>>2] = $135;
  $vararg_ptr15 = (($vararg_buffer12) + 4|0);
  HEAP32[$vararg_ptr15>>2] = $139;
  _print2log(7400,$vararg_buffer12);
  $140 = $ridge_start;
  $141 = $ridge_end;
  $142 = HEAP32[$xlist>>2]|0;
  $143 = HEAP32[$ylist>>2]|0;
  $144 = HEAP32[$num>>2]|0;
  $145 = $4;
  $146 = $5;
  $147 = $6;
  $148 = $7;
  $149 = (($148) + 340|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = (_validate_ridge_crossing($140,$141,$142,$143,$144,$145,$146,$147,$150)|0);
  $ret = $151;
  $152 = $ret;
  $153 = ($152|0)<(0);
  if ($153) {
   label = 22;
   break;
  }
  $157 = $ret;
  HEAP32[$vararg_buffer16>>2] = $157;
  _print2log(7416,$vararg_buffer16);
  $158 = $ret;
  $159 = ($158|0)!=(0);
  if ($159) {
   $160 = $ridge_count;
   $161 = (($160) + 1)|0;
   $ridge_count = $161;
  }
 }
 if ((label|0) == 18) {
  $108 = HEAP32[$xlist>>2]|0;
  _free($108);
  $109 = HEAP32[$ylist>>2]|0;
  _free($109);
  _print2log(7376,$vararg_buffer4);
  $110 = $ridge_count;
  $0 = $110;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 else if ((label|0) == 20) {
  $128 = HEAP32[$xlist>>2]|0;
  _free($128);
  $129 = HEAP32[$ylist>>2]|0;
  _free($129);
  _print2log(7376,$vararg_buffer10);
  $130 = $ridge_count;
  $0 = $130;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 else if ((label|0) == 22) {
  $154 = HEAP32[$xlist>>2]|0;
  _free($154);
  $155 = HEAP32[$ylist>>2]|0;
  _free($155);
  $156 = $ret;
  $0 = $156;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 else if ((label|0) == 26) {
  $162 = HEAP32[$xlist>>2]|0;
  _free($162);
  $163 = HEAP32[$ylist>>2]|0;
  _free($163);
  _print2log(7376,$vararg_buffer19);
  $164 = $ridge_count;
  $0 = $164;
  $165 = $0;
  STACKTOP = sp;return ($165|0);
 }
 return 0|0;
}
function _update_nbr_dists($nbr_list,$nbr_sqr_dists,$nnbrs,$max_nbrs,$first,$second,$minutiae) {
 $nbr_list = $nbr_list|0;
 $nbr_sqr_dists = $nbr_sqr_dists|0;
 $nnbrs = $nnbrs|0;
 $max_nbrs = $max_nbrs|0;
 $first = $first|0;
 $second = $second|0;
 $minutiae = $minutiae|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $dist2 = 0.0, $last_nbr = 0, $minutia1 = 0, $minutia2 = 0, $pos = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $1 = $nbr_list;
 $2 = $nbr_sqr_dists;
 $3 = $nnbrs;
 $4 = $max_nbrs;
 $5 = $first;
 $6 = $second;
 $7 = $minutiae;
 $8 = $4;
 $9 = (($8) - 1)|0;
 $last_nbr = $9;
 $10 = $5;
 $11 = $7;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<2)|0);
 $15 = HEAP32[$14>>2]|0;
 $minutia1 = $15;
 $16 = $6;
 $17 = $7;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + ($16<<2)|0);
 $21 = HEAP32[$20>>2]|0;
 $minutia2 = $21;
 $22 = $minutia1;
 $23 = HEAP32[$22>>2]|0;
 $24 = $minutia1;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $minutia2;
 $28 = HEAP32[$27>>2]|0;
 $29 = $minutia2;
 $30 = (($29) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (+_squared_distance($23,$26,$28,$31));
 $dist2 = $32;
 $33 = $3;
 $34 = HEAP32[$33>>2]|0;
 $35 = $4;
 $36 = ($34|0)<($35|0);
 if (!($36)) {
  $37 = $dist2;
  $38 = $last_nbr;
  $39 = $2;
  $40 = (($39) + ($38<<3)|0);
  $41 = +HEAPF64[$40>>3];
  $42 = $37 < $41;
  if (!($42)) {
   $0 = 0;
   $61 = $0;
   STACKTOP = sp;return ($61|0);
  }
 }
 $43 = $dist2;
 $44 = $2;
 $45 = $3;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_find_incr_position_dbl($43,$44,$46)|0);
 $pos = $47;
 $48 = $pos;
 $49 = $4;
 $50 = ($48|0)>=($49|0);
 if ($50) {
  $51 = HEAP32[_stderr>>2]|0;
  (_fprintf(($51|0),(7104|0),($vararg_buffer|0))|0);
  $0 = -470;
  $61 = $0;
  STACKTOP = sp;return ($61|0);
 }
 $52 = $pos;
 $53 = $6;
 $54 = $dist2;
 $55 = $1;
 $56 = $2;
 $57 = $3;
 $58 = $4;
 $59 = (_insert_neighbor($52,$53,$54,$55,$56,$57,$58)|0);
 $60 = ($59|0)!=(0);
 if ($60) {
  $0 = -471;
  $61 = $0;
  STACKTOP = sp;return ($61|0);
 } else {
  $0 = 0;
  $61 = $0;
  STACKTOP = sp;return ($61|0);
 }
 return 0|0;
}
function _insert_neighbor($pos,$nbr_index,$nbr_dist2,$nbr_list,$nbr_sqr_dists,$nnbrs,$max_nbrs) {
 $pos = $pos|0;
 $nbr_index = $nbr_index|0;
 $nbr_dist2 = +$nbr_dist2;
 $nbr_list = $nbr_list|0;
 $nbr_sqr_dists = $nbr_sqr_dists|0;
 $nnbrs = $nnbrs|0;
 $max_nbrs = $max_nbrs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0.0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $1 = $pos;
 $2 = $nbr_index;
 $3 = $nbr_dist2;
 $4 = $nbr_list;
 $5 = $nbr_sqr_dists;
 $6 = $nnbrs;
 $7 = $max_nbrs;
 $8 = $1;
 $9 = $6;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($8|0)>($10|0);
 if (!($11)) {
  $12 = $1;
  $13 = $7;
  $14 = ($12|0)>=($13|0);
  if (!($14)) {
   $16 = $6;
   $17 = HEAP32[$16>>2]|0;
   $18 = $7;
   $19 = ($17|0)<($18|0);
   do {
    if ($19) {
     $20 = $6;
     $21 = HEAP32[$20>>2]|0;
     $22 = (($21) - 1)|0;
     $i = $22;
     $23 = $6;
     $24 = HEAP32[$23>>2]|0;
     $25 = (($24) + 1)|0;
     HEAP32[$23>>2] = $25;
    } else {
     $26 = $6;
     $27 = HEAP32[$26>>2]|0;
     $28 = $7;
     $29 = ($27|0)==($28|0);
     if ($29) {
      $30 = $6;
      $31 = HEAP32[$30>>2]|0;
      $32 = (($31) - 2)|0;
      $i = $32;
      break;
     } else {
      $33 = HEAP32[_stderr>>2]|0;
      (_fprintf(($33|0),(7232|0),($vararg_buffer1|0))|0);
      $0 = -481;
      $63 = $0;
      STACKTOP = sp;return ($63|0);
     }
    }
   } while(0);
   while(1) {
    $34 = $i;
    $35 = $1;
    $36 = ($34|0)>=($35|0);
    if (!($36)) {
     break;
    }
    $37 = $i;
    $38 = $4;
    $39 = (($38) + ($37<<2)|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = $i;
    $42 = (($41) + 1)|0;
    $43 = $4;
    $44 = (($43) + ($42<<2)|0);
    HEAP32[$44>>2] = $40;
    $45 = $i;
    $46 = $5;
    $47 = (($46) + ($45<<3)|0);
    $48 = +HEAPF64[$47>>3];
    $49 = $i;
    $50 = (($49) + 1)|0;
    $51 = $5;
    $52 = (($51) + ($50<<3)|0);
    HEAPF64[$52>>3] = $48;
    $53 = $i;
    $54 = (($53) + -1)|0;
    $i = $54;
   }
   $55 = $2;
   $56 = $1;
   $57 = $4;
   $58 = (($57) + ($56<<2)|0);
   HEAP32[$58>>2] = $55;
   $59 = $3;
   $60 = $1;
   $61 = $5;
   $62 = (($61) + ($60<<3)|0);
   HEAPF64[$62>>3] = $59;
   $0 = 0;
   $63 = $0;
   STACKTOP = sp;return ($63|0);
  }
 }
 $15 = HEAP32[_stderr>>2]|0;
 (_fprintf(($15|0),(7168|0),($vararg_buffer|0))|0);
 $0 = -480;
 $63 = $0;
 STACKTOP = sp;return ($63|0);
}
function _find_transition($iptr,$pix1,$pix2,$xlist,$ylist,$num,$bdata,$iw,$ih) {
 $iptr = $iptr|0;
 $pix1 = $pix1|0;
 $pix2 = $pix2|0;
 $xlist = $xlist|0;
 $ylist = $ylist|0;
 $num = $num|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $iptr;
 $2 = $pix1;
 $3 = $pix2;
 $4 = $xlist;
 $5 = $ylist;
 $6 = $num;
 $7 = $bdata;
 $8 = $iw;
 $9 = $ih;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $i = $11;
 $12 = $i;
 $13 = (($12) + 1)|0;
 $j = $13;
 while(1) {
  $14 = $i;
  $15 = $6;
  $16 = (($15) - 1)|0;
  $17 = ($14|0)<($16|0);
  if (!($17)) {
   label = 7;
   break;
  }
  $18 = $7;
  $19 = $i;
  $20 = $5;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $8;
  $24 = Math_imul($22, $23)|0;
  $25 = (($18) + ($24)|0);
  $26 = $i;
  $27 = $4;
  $28 = (($27) + ($26<<2)|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($25) + ($29)|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = $2;
  $34 = ($32|0)==($33|0);
  if ($34) {
   $35 = $7;
   $36 = $j;
   $37 = $5;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $8;
   $41 = Math_imul($39, $40)|0;
   $42 = (($35) + ($41)|0);
   $43 = $j;
   $44 = $4;
   $45 = (($44) + ($43<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($42) + ($46)|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   $50 = $3;
   $51 = ($49|0)==($50|0);
   if ($51) {
    label = 5;
    break;
   }
  }
  $54 = $i;
  $55 = (($54) + 1)|0;
  $i = $55;
  $56 = $j;
  $57 = (($56) + 1)|0;
  $j = $57;
 }
 if ((label|0) == 5) {
  $52 = $j;
  $53 = $1;
  HEAP32[$53>>2] = $52;
  $0 = 1;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 else if ((label|0) == 7) {
  $58 = $6;
  $59 = $1;
  HEAP32[$59>>2] = $58;
  $0 = 0;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 return 0|0;
}
function _validate_ridge_crossing($ridge_start,$ridge_end,$xlist,$ylist,$num,$bdata,$iw,$ih,$max_ridge_steps) {
 $ridge_start = $ridge_start|0;
 $ridge_end = $ridge_end|0;
 $xlist = $xlist|0;
 $ylist = $ylist|0;
 $num = $num|0;
 $bdata = $bdata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $max_ridge_steps = $max_ridge_steps|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $contour_ex = 0, $contour_ey = 0, $contour_x = 0, $contour_y = 0;
 var $edge_x = 0, $edge_y = 0, $feat_x = 0, $feat_y = 0, $ncontour = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $feat_x = sp + 32|0;
 $feat_y = sp + 28|0;
 $edge_x = sp + 24|0;
 $edge_y = sp + 20|0;
 $contour_x = sp + 16|0;
 $contour_y = sp + 12|0;
 $contour_ex = sp + 8|0;
 $contour_ey = sp + 4|0;
 $ncontour = sp;
 $1 = $ridge_start;
 $2 = $ridge_end;
 $3 = $xlist;
 $4 = $ylist;
 $5 = $num;
 $6 = $bdata;
 $7 = $iw;
 $8 = $ih;
 $9 = $max_ridge_steps;
 $10 = $2;
 $11 = $3;
 $12 = (($11) + ($10<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$feat_x>>2] = $13;
 $14 = $2;
 $15 = $4;
 $16 = (($15) + ($14<<2)|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$feat_y>>2] = $17;
 $18 = $2;
 $19 = (($18) - 1)|0;
 $20 = $3;
 $21 = (($20) + ($19<<2)|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$edge_x>>2] = $22;
 $23 = $2;
 $24 = (($23) - 1)|0;
 $25 = $4;
 $26 = (($25) + ($24<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$edge_y>>2] = $27;
 $28 = $6;
 $29 = $7;
 $30 = $8;
 _fix_edge_pixel_pair($feat_x,$feat_y,$edge_x,$edge_y,$28,$29,$30);
 $31 = $9;
 $32 = $1;
 $33 = (($32) - 1)|0;
 $34 = $3;
 $35 = (($34) + ($33<<2)|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = $1;
 $38 = (($37) - 1)|0;
 $39 = $4;
 $40 = (($39) + ($38<<2)|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$feat_x>>2]|0;
 $43 = HEAP32[$feat_y>>2]|0;
 $44 = HEAP32[$edge_x>>2]|0;
 $45 = HEAP32[$edge_y>>2]|0;
 $46 = $6;
 $47 = $7;
 $48 = $8;
 $49 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$31,$36,$41,$42,$43,$44,$45,0,$46,$47,$48)|0);
 $ret = $49;
 $50 = $ret;
 $51 = ($50|0)<(0);
 if ($51) {
  $52 = $ret;
  $0 = $52;
  $95 = $0;
  STACKTOP = sp;return ($95|0);
 }
 $53 = $ret;
 $54 = ($53|0)!=(2);
 if ($54) {
  $55 = HEAP32[$contour_x>>2]|0;
  $56 = HEAP32[$contour_y>>2]|0;
  $57 = HEAP32[$contour_ex>>2]|0;
  $58 = HEAP32[$contour_ey>>2]|0;
  _free_contour($55,$56,$57,$58);
 }
 $59 = $ret;
 $60 = ($59|0)!=(2);
 if ($60) {
  $61 = $ret;
  $62 = ($61|0)!=(1);
  if ($62) {
   $63 = $9;
   $64 = $1;
   $65 = (($64) - 1)|0;
   $66 = $3;
   $67 = (($66) + ($65<<2)|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = $1;
   $70 = (($69) - 1)|0;
   $71 = $4;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = HEAP32[$feat_x>>2]|0;
   $75 = HEAP32[$feat_y>>2]|0;
   $76 = HEAP32[$edge_x>>2]|0;
   $77 = HEAP32[$edge_y>>2]|0;
   $78 = $6;
   $79 = $7;
   $80 = $8;
   $81 = (_trace_contour($contour_x,$contour_y,$contour_ex,$contour_ey,$ncontour,$63,$68,$73,$74,$75,$76,$77,1,$78,$79,$80)|0);
   $ret = $81;
   $82 = $ret;
   $83 = ($82|0)<(0);
   if ($83) {
    $84 = $ret;
    $0 = $84;
    $95 = $0;
    STACKTOP = sp;return ($95|0);
   }
   $85 = $ret;
   $86 = ($85|0)!=(2);
   if ($86) {
    $87 = HEAP32[$contour_x>>2]|0;
    $88 = HEAP32[$contour_y>>2]|0;
    $89 = HEAP32[$contour_ex>>2]|0;
    $90 = HEAP32[$contour_ey>>2]|0;
    _free_contour($87,$88,$89,$90);
   }
   $91 = $ret;
   $92 = ($91|0)!=(2);
   if ($92) {
    $93 = $ret;
    $94 = ($93|0)!=(1);
    if ($94) {
     $0 = 1;
     $95 = $0;
     STACKTOP = sp;return ($95|0);
    }
   }
  }
 }
 $0 = 0;
 $95 = $0;
 STACKTOP = sp;return ($95|0);
}
function _alloc_shape($oshape,$xmin,$ymin,$xmax,$ymax) {
 $oshape = $oshape|0;
 $xmin = $xmin|0;
 $ymin = $ymin|0;
 $xmax = $xmax|0;
 $ymax = $ymax|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $alloc_pts = 0;
 var $alloc_rows = 0, $i = 0, $j = 0, $shape = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $oshape;
 $2 = $xmin;
 $3 = $ymin;
 $4 = $xmax;
 $5 = $ymax;
 $6 = $5;
 $7 = $3;
 $8 = (($6) - ($7))|0;
 $9 = (($8) + 1)|0;
 $alloc_rows = $9;
 $10 = $4;
 $11 = $2;
 $12 = (($10) - ($11))|0;
 $13 = (($12) + 1)|0;
 $alloc_pts = $13;
 $14 = (_malloc(20)|0);
 $shape = $14;
 $15 = $shape;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = HEAP32[_stderr>>2]|0;
  (_fprintf(($17|0),(7424|0),($vararg_buffer|0))|0);
  $0 = -250;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 $18 = $alloc_rows;
 $19 = $18<<2;
 $20 = (_malloc($19)|0);
 $21 = $shape;
 $22 = (($21) + 8|0);
 HEAP32[$22>>2] = $20;
 $23 = $shape;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)==(0|0);
 if ($26) {
  $27 = $shape;
  _free($27);
  $28 = HEAP32[_stderr>>2]|0;
  (_fprintf(($28|0),(7464|0),($vararg_buffer1|0))|0);
  $0 = -251;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 $29 = $3;
 $30 = $shape;
 HEAP32[$30>>2] = $29;
 $31 = $5;
 $32 = $shape;
 $33 = (($32) + 4|0);
 HEAP32[$33>>2] = $31;
 $34 = $alloc_rows;
 $35 = $shape;
 $36 = (($35) + 12|0);
 HEAP32[$36>>2] = $34;
 $37 = $alloc_rows;
 $38 = $shape;
 $39 = (($38) + 16|0);
 HEAP32[$39>>2] = $37;
 $i = 0;
 $40 = $3;
 $y = $40;
 while(1) {
  $41 = $i;
  $42 = $alloc_rows;
  $43 = ($41|0)<($42|0);
  if (!($43)) {
   label = 21;
   break;
  }
  $44 = (_malloc(16)|0);
  $45 = $i;
  $46 = $shape;
  $47 = (($46) + 8|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($45<<2)|0);
  HEAP32[$49>>2] = $44;
  $50 = $i;
  $51 = $shape;
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($50<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)==(0|0);
  if ($56) {
   label = 8;
   break;
  }
  $81 = $alloc_pts;
  $82 = $81<<2;
  $83 = (_malloc($82)|0);
  $84 = $i;
  $85 = $shape;
  $86 = (($85) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + ($84<<2)|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = (($89) + 4|0);
  HEAP32[$90>>2] = $83;
  $91 = $i;
  $92 = $shape;
  $93 = (($92) + 8|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 4|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = ($98|0)==(0|0);
  if ($99) {
   label = 14;
   break;
  }
  $130 = $y;
  $131 = $i;
  $132 = $shape;
  $133 = (($132) + 8|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = (($134) + ($131<<2)|0);
  $136 = HEAP32[$135>>2]|0;
  HEAP32[$136>>2] = $130;
  $137 = $alloc_pts;
  $138 = $i;
  $139 = $shape;
  $140 = (($139) + 8|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = (($141) + ($138<<2)|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = (($143) + 8|0);
  HEAP32[$144>>2] = $137;
  $145 = $i;
  $146 = $shape;
  $147 = (($146) + 8|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = (($148) + ($145<<2)|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = (($150) + 12|0);
  HEAP32[$151>>2] = 0;
  $152 = $i;
  $153 = (($152) + 1)|0;
  $i = $153;
  $154 = $y;
  $155 = (($154) + 1)|0;
  $y = $155;
 }
 if ((label|0) == 8) {
  $j = 0;
  while(1) {
   $57 = $j;
   $58 = $i;
   $59 = ($57|0)<($58|0);
   if (!($59)) {
    break;
   }
   $60 = $j;
   $61 = $shape;
   $62 = (($61) + 8|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (($63) + ($60<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($65) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   _free($67);
   $68 = $j;
   $69 = $shape;
   $70 = (($69) + 8|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + ($68<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   _free($73);
   $74 = $j;
   $75 = (($74) + 1)|0;
   $j = $75;
  }
  $76 = $shape;
  $77 = (($76) + 8|0);
  $78 = HEAP32[$77>>2]|0;
  _free($78);
  $79 = $shape;
  _free($79);
  $80 = HEAP32[_stderr>>2]|0;
  (_fprintf(($80|0),(7512|0),($vararg_buffer3|0))|0);
  $0 = -252;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 else if ((label|0) == 14) {
  $j = 0;
  while(1) {
   $100 = $j;
   $101 = $i;
   $102 = ($100|0)<($101|0);
   if (!($102)) {
    break;
   }
   $103 = $j;
   $104 = $shape;
   $105 = (($104) + 8|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($106) + ($103<<2)|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($108) + 4|0);
   $110 = HEAP32[$109>>2]|0;
   _free($110);
   $111 = $j;
   $112 = $shape;
   $113 = (($112) + 8|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = (($114) + ($111<<2)|0);
   $116 = HEAP32[$115>>2]|0;
   _free($116);
   $117 = $j;
   $118 = (($117) + 1)|0;
   $j = $118;
  }
  $119 = $i;
  $120 = $shape;
  $121 = (($120) + 8|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = (($122) + ($119<<2)|0);
  $124 = HEAP32[$123>>2]|0;
  _free($124);
  $125 = $shape;
  $126 = (($125) + 8|0);
  $127 = HEAP32[$126>>2]|0;
  _free($127);
  $128 = $shape;
  _free($128);
  $129 = HEAP32[_stderr>>2]|0;
  (_fprintf(($129|0),(7560|0),($vararg_buffer5|0))|0);
  $0 = -253;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 else if ((label|0) == 21) {
  $156 = $shape;
  $157 = $1;
  HEAP32[$157>>2] = $156;
  $0 = 0;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 return 0|0;
}
function _free_shape($shape) {
 $shape = $shape|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $shape;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 12|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  _free($13);
  $14 = $i;
  $15 = $0;
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  _free($19);
  $20 = $i;
  $21 = (($20) + 1)|0;
  $i = $21;
 }
 $22 = $0;
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 _free($24);
 $25 = $0;
 _free($25);
 STACKTOP = sp;return;
}
function _shape_from_contour($oshape,$contour_x,$contour_y,$ncontour) {
 $oshape = $oshape|0;
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $i = 0;
 var $ret = 0, $row = 0, $shape = 0, $vararg_buffer = 0, $xmax = 0, $xmin = 0, $ymax = 0, $ymin = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $shape = sp + 32|0;
 $xmin = sp + 16|0;
 $ymin = sp + 12|0;
 $xmax = sp + 8|0;
 $ymax = sp + 4|0;
 $1 = $oshape;
 $2 = $contour_x;
 $3 = $contour_y;
 $4 = $ncontour;
 $5 = $2;
 $6 = $3;
 $7 = $4;
 _contour_limits($xmin,$ymin,$xmax,$ymax,$5,$6,$7);
 $8 = HEAP32[$xmin>>2]|0;
 $9 = HEAP32[$ymin>>2]|0;
 $10 = HEAP32[$xmax>>2]|0;
 $11 = HEAP32[$ymax>>2]|0;
 $12 = (_alloc_shape($shape,$8,$9,$10,$11)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $ret;
  $0 = $14;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 $i = 0;
 while(1) {
  $15 = $i;
  $16 = $4;
  $17 = ($15|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = $i;
  $19 = $3;
  $20 = (($19) + ($18<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$ymin>>2]|0;
  $23 = (($21) - ($22))|0;
  $24 = HEAP32[$shape>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($23<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $row = $28;
  $29 = $i;
  $30 = $2;
  $31 = (($30) + ($29<<2)|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = $row;
  $34 = (($33) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = $row;
  $37 = (($36) + 12|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (_in_int_list($32,$35,$38)|0);
  $40 = ($39|0)<(0);
  if ($40) {
   $41 = $row;
   $42 = (($41) + 12|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = $row;
   $45 = (($44) + 8|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($43|0)>=($46|0);
   if ($47) {
    label = 7;
    break;
   }
   $49 = $i;
   $50 = $2;
   $51 = (($50) + ($49<<2)|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = $row;
   $54 = (($53) + 12|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$54>>2] = $56;
   $57 = $row;
   $58 = (($57) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + ($55<<2)|0);
   HEAP32[$60>>2] = $52;
  }
  $61 = $i;
  $62 = (($61) + 1)|0;
  $i = $62;
 }
 if ((label|0) == 7) {
  $48 = HEAP32[_stderr>>2]|0;
  (_fprintf(($48|0),(7616|0),($vararg_buffer|0))|0);
  $0 = -260;
  $78 = $0;
  STACKTOP = sp;return ($78|0);
 }
 $i = 0;
 while(1) {
  $63 = $i;
  $64 = HEAP32[$shape>>2]|0;
  $65 = (($64) + 16|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = ($63|0)<($66|0);
  if (!($67)) {
   break;
  }
  $68 = $i;
  $69 = HEAP32[$shape>>2]|0;
  $70 = (($69) + 8|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + ($68<<2)|0);
  $73 = HEAP32[$72>>2]|0;
  _sort_row_on_x($73);
  $74 = $i;
  $75 = (($74) + 1)|0;
  $i = $75;
 }
 $76 = HEAP32[$shape>>2]|0;
 $77 = $1;
 HEAP32[$77>>2] = $76;
 $0 = 0;
 $78 = $0;
 STACKTOP = sp;return ($78|0);
}
function _sort_row_on_x($row) {
 $row = $row|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $row;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 _bubble_sort_int_inc($3,$6);
 STACKTOP = sp;return;
}
function _minutiae2type_9($otype_9,$img_idc,$minutiae,$iw,$ih,$ppmm) {
 $otype_9 = $otype_9|0;
 $img_idc = $img_idc|0;
 $minutiae = $minutiae|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $ppmm = +$ppmm;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0.0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $field = 0, $item = 0, $record = 0, $ret = 0, $subfield = 0, $uint_str = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $record = sp + 32|0;
 $field = sp + 28|0;
 $subfield = sp + 24|0;
 $item = sp + 20|0;
 $uint_str = sp + 64|0;
 $1 = $otype_9;
 $2 = $img_idc;
 $3 = $minutiae;
 $4 = $iw;
 $5 = $ih;
 $6 = $ppmm;
 $7 = (_new_ANSI_NIST_record($record,9)|0);
 $ret = $7;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $ret;
  $0 = $9;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $10 = (_value2field($field,9,1,7664)|0);
 $ret = $10;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($12);
  $13 = $ret;
  $0 = $13;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $14 = HEAP32[$record>>2]|0;
 $15 = HEAP32[$field>>2]|0;
 $16 = (_append_ANSI_NIST_record($14,$15)|0);
 $ret = $16;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($18);
  $19 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($19);
  $20 = $ret;
  $0 = $20;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $21 = $2;
 HEAP32[$vararg_buffer>>2] = $21;
 (_sprintf($uint_str,7672,$vararg_buffer)|0);
 $22 = (_value2field($field,9,2,$uint_str)|0);
 $ret = $22;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($24);
  $25 = $ret;
  $0 = $25;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $26 = HEAP32[$record>>2]|0;
 $27 = HEAP32[$field>>2]|0;
 $28 = (_append_ANSI_NIST_record($26,$27)|0);
 $ret = $28;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($30);
  $31 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($31);
  $32 = $ret;
  $0 = $32;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $33 = (_value2field($field,9,3,7680)|0);
 $ret = $33;
 $34 = ($33|0)!=(0);
 if ($34) {
  $35 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($35);
  $36 = $ret;
  $0 = $36;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $37 = HEAP32[$record>>2]|0;
 $38 = HEAP32[$field>>2]|0;
 $39 = (_append_ANSI_NIST_record($37,$38)|0);
 $ret = $39;
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($41);
  $42 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($42);
  $43 = $ret;
  $0 = $43;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $44 = (_value2field($field,9,4,7688)|0);
 $ret = $44;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($46);
  $47 = $ret;
  $0 = $47;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $48 = HEAP32[$record>>2]|0;
 $49 = HEAP32[$field>>2]|0;
 $50 = (_append_ANSI_NIST_record($48,$49)|0);
 $ret = $50;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($52);
  $53 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($53);
  $54 = $ret;
  $0 = $54;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $55 = (_value2subfield($subfield,7696)|0);
 $ret = $55;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($57);
  $58 = $ret;
  $0 = $58;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $59 = (_value2item($item,7712)|0);
 $ret = $59;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($61);
  $62 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($62);
  $63 = $ret;
  $0 = $63;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $64 = HEAP32[$subfield>>2]|0;
 $65 = HEAP32[$item>>2]|0;
 $66 = (_append_ANSI_NIST_subfield($64,$65)|0);
 $ret = $66;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($68);
  $69 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($69);
  $70 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($70);
  $71 = $ret;
  $0 = $71;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $72 = (_new_ANSI_NIST_field($field,9,5)|0);
 $ret = $72;
 $73 = ($72|0)!=(0);
 if ($73) {
  $74 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($74);
  $75 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($75);
  $76 = $ret;
  $0 = $76;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $77 = HEAP32[$field>>2]|0;
 $78 = HEAP32[$subfield>>2]|0;
 $79 = (_append_ANSI_NIST_field($77,$78)|0);
 $ret = $79;
 $80 = ($79|0)!=(0);
 if ($80) {
  $81 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($81);
  $82 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($82);
  $83 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($83);
  $84 = $ret;
  $0 = $84;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $85 = HEAP32[$record>>2]|0;
 $86 = HEAP32[$field>>2]|0;
 $87 = (_append_ANSI_NIST_record($85,$86)|0);
 $ret = $87;
 $88 = ($87|0)!=(0);
 if ($88) {
  $89 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($89);
  $90 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($90);
  $91 = $ret;
  $0 = $91;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $92 = (_value2field($field,9,6,7720)|0);
 $ret = $92;
 $93 = ($92|0)!=(0);
 if ($93) {
  $94 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($94);
  $95 = $ret;
  $0 = $95;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $96 = HEAP32[$record>>2]|0;
 $97 = HEAP32[$field>>2]|0;
 $98 = (_append_ANSI_NIST_record($96,$97)|0);
 $ret = $98;
 $99 = ($98|0)!=(0);
 if ($99) {
  $100 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($100);
  $101 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($101);
  $102 = $ret;
  $0 = $102;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $103 = (_value2subfield($subfield,7728)|0);
 $ret = $103;
 $104 = ($103|0)!=(0);
 if ($104) {
  $105 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($105);
  $106 = $ret;
  $0 = $106;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $107 = (_value2item($item,7736)|0);
 $ret = $107;
 $108 = ($107|0)!=(0);
 if ($108) {
  $109 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($109);
  $110 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($110);
  $111 = $ret;
  $0 = $111;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $112 = HEAP32[$subfield>>2]|0;
 $113 = HEAP32[$item>>2]|0;
 $114 = (_append_ANSI_NIST_subfield($112,$113)|0);
 $ret = $114;
 $115 = ($114|0)!=(0);
 if ($115) {
  $116 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($116);
  $117 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($117);
  $118 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($118);
  $119 = $ret;
  $0 = $119;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $120 = (_new_ANSI_NIST_field($field,9,7)|0);
 $ret = $120;
 $121 = ($120|0)!=(0);
 if ($121) {
  $122 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($122);
  $123 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($123);
  $124 = $ret;
  $0 = $124;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $125 = HEAP32[$field>>2]|0;
 $126 = HEAP32[$subfield>>2]|0;
 $127 = (_append_ANSI_NIST_field($125,$126)|0);
 $ret = $127;
 $128 = ($127|0)!=(0);
 if ($128) {
  $129 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($129);
  $130 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($130);
  $131 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($131);
  $132 = $ret;
  $0 = $132;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $133 = HEAP32[$record>>2]|0;
 $134 = HEAP32[$field>>2]|0;
 $135 = (_append_ANSI_NIST_record($133,$134)|0);
 $ret = $135;
 $136 = ($135|0)!=(0);
 if ($136) {
  $137 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($137);
  $138 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($138);
  $139 = $ret;
  $0 = $139;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $140 = $3;
 $141 = (($140) + 4|0);
 $142 = HEAP32[$141>>2]|0;
 HEAP32[$vararg_buffer1>>2] = $142;
 (_sprintf($uint_str,7744,$vararg_buffer1)|0);
 $143 = (_value2field($field,9,10,$uint_str)|0);
 $ret = $143;
 $144 = ($143|0)!=(0);
 if ($144) {
  $145 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($145);
  $146 = $ret;
  $0 = $146;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $147 = HEAP32[$record>>2]|0;
 $148 = HEAP32[$field>>2]|0;
 $149 = (_append_ANSI_NIST_record($147,$148)|0);
 $ret = $149;
 $150 = ($149|0)!=(0);
 if ($150) {
  $151 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($151);
  $152 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($152);
  $153 = $ret;
  $0 = $153;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $154 = (_value2field($field,9,11,7752)|0);
 $ret = $154;
 $155 = ($154|0)!=(0);
 if ($155) {
  $156 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($156);
  $157 = $ret;
  $0 = $157;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $158 = HEAP32[$record>>2]|0;
 $159 = HEAP32[$field>>2]|0;
 $160 = (_append_ANSI_NIST_record($158,$159)|0);
 $ret = $160;
 $161 = ($160|0)!=(0);
 if ($161) {
  $162 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($162);
  $163 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($163);
  $164 = $ret;
  $0 = $164;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 $165 = $3;
 $166 = (($165) + 4|0);
 $167 = HEAP32[$166>>2]|0;
 $168 = ($167|0)>(0);
 do {
  if ($168) {
   $169 = $3;
   $170 = $4;
   $171 = $5;
   $172 = $6;
   $173 = (_mintiae2field_12($field,$169,$170,$171,$172)|0);
   $ret = $173;
   $174 = ($173|0)!=(0);
   if ($174) {
    $175 = HEAP32[$record>>2]|0;
    _free_ANSI_NIST_record($175);
    $176 = $ret;
    $0 = $176;
    $191 = $0;
    STACKTOP = sp;return ($191|0);
   }
   $177 = HEAP32[$record>>2]|0;
   $178 = HEAP32[$field>>2]|0;
   $179 = (_append_ANSI_NIST_record($177,$178)|0);
   $ret = $179;
   $180 = ($179|0)!=(0);
   if (!($180)) {
    break;
   }
   $181 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($181);
   $182 = HEAP32[$record>>2]|0;
   _free_ANSI_NIST_record($182);
   $183 = $ret;
   $0 = $183;
   $191 = $0;
   STACKTOP = sp;return ($191|0);
  }
 } while(0);
 $184 = HEAP32[$record>>2]|0;
 $185 = (_update_ANSI_NIST_tagged_record_LEN($184)|0);
 $ret = $185;
 $186 = ($185|0)!=(0);
 if ($186) {
  $187 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($187);
  $188 = $ret;
  $0 = $188;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 } else {
  $189 = HEAP32[$record>>2]|0;
  $190 = $1;
  HEAP32[$190>>2] = $189;
  $0 = 0;
  $191 = $0;
  STACKTOP = sp;return ($191|0);
 }
 return 0|0;
}
function _mintiae2field_12($ofield,$minutiae,$iw,$ih,$ppmm) {
 $ofield = $ofield|0;
 $minutiae = $minutiae|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $ppmm = +$ppmm;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0.0, $123 = 0.0, $124 = 0.0, $125 = 0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0.0, $163 = 0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0.0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0.0, $175 = 0.0, $176 = 0, $177 = 0.0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0.0, $43 = 0.0, $44 = 0.0, $45 = 0.0;
 var $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0.0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0, $63 = 0.0;
 var $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0.0;
 var $82 = 0.0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0.0, $99 = 0.0, $deg_per_idir = 0.0;
 var $field = 0, $h_units = 0, $i = 0, $item = 0, $j = 0, $min_qual = 0, $minutia = 0, $native_deg = 0, $ret = 0, $std_deg = 0, $std_type = 0, $subfield = 0, $value = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer21 = 0;
 var $vararg_buffer24 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_ptr15 = 0, $vararg_ptr27 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $x_units = 0, $y_units = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer24 = sp + 96|0;
 $vararg_buffer21 = sp + 88|0;
 $vararg_buffer19 = sp + 80|0;
 $vararg_buffer16 = sp + 72|0;
 $vararg_buffer12 = sp + 64|0;
 $vararg_buffer10 = sp + 56|0;
 $vararg_buffer5 = sp + 40|0;
 $vararg_buffer3 = sp + 32|0;
 $vararg_buffer1 = sp + 24|0;
 $vararg_buffer = sp + 16|0;
 $item = sp + 144|0;
 $subfield = sp + 140|0;
 $field = sp + 136|0;
 $value = sp + 180|0;
 $1 = $ofield;
 $2 = $minutiae;
 $3 = $iw;
 $4 = $ih;
 $5 = $ppmm;
 $6 = $2;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)<(1);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(7760|0),($vararg_buffer|0))|0);
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(7792|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $258 = $0;
  STACKTOP = sp;return ($258|0);
 }
 $12 = (_new_ANSI_NIST_field($field,9,12)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $ret;
  $0 = $14;
  $258 = $0;
  STACKTOP = sp;return ($258|0);
 }
 $deg_per_idir = 11.25;
 $i = 0;
 L9: while(1) {
  $15 = $i;
  $16 = $2;
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   label = 56;
   break;
  }
  $20 = $i;
  $21 = $2;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $minutia = $25;
  $26 = $i;
  $27 = (($26) + 1)|0;
  HEAP32[$vararg_buffer3>>2] = $27;
  (_sprintf($value,7816,$vararg_buffer3)|0);
  $28 = (_value2subfield($subfield,$value)|0);
  $ret = $28;
  $29 = ($28|0)!=(0);
  if ($29) {
   label = 8;
   break;
  }
  $32 = $minutia;
  $33 = HEAP32[$32>>2]|0;
  $34 = (+($33|0));
  $35 = $5;
  $36 = $34 / $35;
  $37 = $36 * 100.0;
  $38 = $37 < 0.0;
  if ($38) {
   $39 = $minutia;
   $40 = HEAP32[$39>>2]|0;
   $41 = (+($40|0));
   $42 = $5;
   $43 = $41 / $42;
   $44 = $43 * 100.0;
   $45 = $44 - 0.5;
   $54 = $45;
  } else {
   $46 = $minutia;
   $47 = HEAP32[$46>>2]|0;
   $48 = (+($47|0));
   $49 = $5;
   $50 = $48 / $49;
   $51 = $50 * 100.0;
   $52 = $51 + 0.5;
   $54 = $52;
  }
  $53 = (~~(($54)));
  $x_units = $53;
  $55 = $4;
  $56 = (+($55|0));
  $57 = $5;
  $58 = $56 / $57;
  $59 = $58 * 100.0;
  $60 = $59 < 0.0;
  if ($60) {
   $61 = $4;
   $62 = (+($61|0));
   $63 = $5;
   $64 = $62 / $63;
   $65 = $64 * 100.0;
   $66 = $65 - 0.5;
   $74 = $66;
  } else {
   $67 = $4;
   $68 = (+($67|0));
   $69 = $5;
   $70 = $68 / $69;
   $71 = $70 * 100.0;
   $72 = $71 + 0.5;
   $74 = $72;
  }
  $73 = (~~(($74)));
  $h_units = $73;
  $75 = $h_units;
  $76 = (($75) - 1)|0;
  $77 = $minutia;
  $78 = (($77) + 4|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (+($79|0));
  $81 = $5;
  $82 = $80 / $81;
  $83 = $82 * 100.0;
  $84 = $83 < 0.0;
  if ($84) {
   $85 = $minutia;
   $86 = (($85) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (+($87|0));
   $89 = $5;
   $90 = $88 / $89;
   $91 = $90 * 100.0;
   $92 = $91 - 0.5;
   $102 = $92;
  } else {
   $93 = $minutia;
   $94 = (($93) + 4|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (+($95|0));
   $97 = $5;
   $98 = $96 / $97;
   $99 = $98 * 100.0;
   $100 = $99 + 0.5;
   $102 = $100;
  }
  $101 = (~~(($102)));
  $103 = (($76) - ($101))|0;
  $y_units = $103;
  $104 = $minutia;
  $105 = (($104) + 16|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (+($106|0));
  $108 = $deg_per_idir;
  $109 = $107 * $108;
  $110 = $109 < 0.0;
  if ($110) {
   $111 = $minutia;
   $112 = (($111) + 16|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = (+($113|0));
   $115 = $deg_per_idir;
   $116 = $114 * $115;
   $117 = $116 - 0.5;
   $126 = $117;
  } else {
   $118 = $minutia;
   $119 = (($118) + 16|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (+($120|0));
   $122 = $deg_per_idir;
   $123 = $121 * $122;
   $124 = $123 + 0.5;
   $126 = $124;
  }
  $125 = (~~(($126)));
  $native_deg = $125;
  $127 = $native_deg;
  $128 = (270 - ($127))|0;
  $129 = (($128|0) % 360)&-1;
  $std_deg = $129;
  $130 = $std_deg;
  $131 = ($130|0)<(0);
  if ($131) {
   $132 = $std_deg;
   $133 = (($132) + 360)|0;
   $std_deg = $133;
  }
  $134 = $x_units;
  $135 = $y_units;
  $136 = $std_deg;
  HEAP32[$vararg_buffer5>>2] = $134;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $135;
  $vararg_ptr9 = (($vararg_buffer5) + 8|0);
  HEAP32[$vararg_ptr9>>2] = $136;
  (_sprintf($value,7824,$vararg_buffer5)|0);
  $137 = (_strlen(($value|0))|0);
  $138 = ($137>>>0)>(11);
  if ($138) {
   label = 24;
   break;
  }
  $145 = (_value2item($item,$value)|0);
  $ret = $145;
  $146 = ($145|0)!=(0);
  if ($146) {
   label = 26;
   break;
  }
  $150 = HEAP32[$subfield>>2]|0;
  $151 = HEAP32[$item>>2]|0;
  $152 = (_append_ANSI_NIST_subfield($150,$151)|0);
  $ret = $152;
  $153 = ($152|0)!=(0);
  if ($153) {
   label = 28;
   break;
  }
  $158 = $minutia;
  $159 = (($158) + 24|0);
  $160 = +HEAPF64[$159>>3];
  $161 = $160 * 61.0;
  $162 = 63.0 - $161;
  $163 = $162 < 0.0;
  if ($163) {
   $164 = $minutia;
   $165 = (($164) + 24|0);
   $166 = +HEAPF64[$165>>3];
   $167 = $166 * 61.0;
   $168 = 63.0 - $167;
   $169 = $168 - 0.5;
   $177 = $169;
  } else {
   $170 = $minutia;
   $171 = (($170) + 24|0);
   $172 = +HEAPF64[$171>>3];
   $173 = $172 * 61.0;
   $174 = 63.0 - $173;
   $175 = $174 + 0.5;
   $177 = $175;
  }
  $176 = (~~(($177)));
  $min_qual = $176;
  $178 = $min_qual;
  HEAP32[$vararg_buffer16>>2] = $178;
  (_sprintf($value,7672,$vararg_buffer16)|0);
  $179 = (_value2item($item,$value)|0);
  $ret = $179;
  $180 = ($179|0)!=(0);
  if ($180) {
   label = 33;
   break;
  }
  $184 = HEAP32[$subfield>>2]|0;
  $185 = HEAP32[$item>>2]|0;
  $186 = (_append_ANSI_NIST_subfield($184,$185)|0);
  $ret = $186;
  $187 = ($186|0)!=(0);
  if ($187) {
   label = 35;
   break;
  }
  $192 = $minutia;
  $193 = (($192) + 32|0);
  $194 = HEAP32[$193>>2]|0;
  if ((($194|0) == 1)) {
   $std_type = 7712;
  } else if ((($194|0) == 0)) {
   $std_type = 7888;
  } else {
   label = 39;
   break;
  }
  $201 = $std_type;
  $202 = (_value2item($item,$201)|0);
  $ret = $202;
  $203 = ($202|0)!=(0);
  if ($203) {
   label = 41;
   break;
  }
  $207 = HEAP32[$subfield>>2]|0;
  $208 = HEAP32[$item>>2]|0;
  $209 = (_append_ANSI_NIST_subfield($207,$208)|0);
  $ret = $209;
  $210 = ($209|0)!=(0);
  if ($210) {
   label = 43;
   break;
  }
  $j = 0;
  while(1) {
   $215 = $j;
   $216 = $minutia;
   $217 = (($216) + 52|0);
   $218 = HEAP32[$217>>2]|0;
   $219 = ($215|0)<($218|0);
   if (!($219)) {
    break;
   }
   $220 = $j;
   $221 = $minutia;
   $222 = (($221) + 44|0);
   $223 = HEAP32[$222>>2]|0;
   $224 = (($223) + ($220<<2)|0);
   $225 = HEAP32[$224>>2]|0;
   $226 = $j;
   $227 = $minutia;
   $228 = (($227) + 48|0);
   $229 = HEAP32[$228>>2]|0;
   $230 = (($229) + ($226<<2)|0);
   $231 = HEAP32[$230>>2]|0;
   HEAP32[$vararg_buffer24>>2] = $225;
   $vararg_ptr27 = (($vararg_buffer24) + 4|0);
   HEAP32[$vararg_ptr27>>2] = $231;
   (_sprintf($value,7928,$vararg_buffer24)|0);
   $232 = (_value2item($item,$value)|0);
   $ret = $232;
   $233 = ($232|0)!=(0);
   if ($233) {
    label = 47;
    break L9;
   }
   $237 = HEAP32[$subfield>>2]|0;
   $238 = HEAP32[$item>>2]|0;
   $239 = (_append_ANSI_NIST_subfield($237,$238)|0);
   $ret = $239;
   $240 = ($239|0)!=(0);
   if ($240) {
    label = 49;
    break L9;
   }
   $245 = $j;
   $246 = (($245) + 1)|0;
   $j = $246;
  }
  $247 = HEAP32[$field>>2]|0;
  $248 = HEAP32[$subfield>>2]|0;
  $249 = (_append_ANSI_NIST_field($247,$248)|0);
  $ret = $249;
  $250 = ($249|0)!=(0);
  if ($250) {
   label = 53;
   break;
  }
  $254 = $i;
  $255 = (($254) + 1)|0;
  $i = $255;
 }
 switch (label|0) {
  case 8: {
   $30 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($30);
   $31 = $ret;
   $0 = $31;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 24: {
   $139 = HEAP32[_stderr>>2]|0;
   (_fprintf(($139|0),(7760|0),($vararg_buffer10|0))|0);
   $140 = HEAP32[_stderr>>2]|0;
   $141 = $i;
   $142 = (($141) + 1)|0;
   HEAP32[$vararg_buffer12>>2] = $142;
   $vararg_ptr15 = (($vararg_buffer12) + 4|0);
   HEAP32[$vararg_ptr15>>2] = $value;
   (_fprintf(($140|0),(7840|0),($vararg_buffer12|0))|0);
   $143 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($143);
   $144 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($144);
   $0 = -3;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 26: {
   $147 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($147);
   $148 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($148);
   $149 = $ret;
   $0 = $149;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 28: {
   $154 = HEAP32[$item>>2]|0;
   _free_ANSI_NIST_item($154);
   $155 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($155);
   $156 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($156);
   $157 = $ret;
   $0 = $157;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 33: {
   $181 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($181);
   $182 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($182);
   $183 = $ret;
   $0 = $183;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 35: {
   $188 = HEAP32[$item>>2]|0;
   _free_ANSI_NIST_item($188);
   $189 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($189);
   $190 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($190);
   $191 = $ret;
   $0 = $191;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 39: {
   $195 = HEAP32[_stderr>>2]|0;
   (_fprintf(($195|0),(7760|0),($vararg_buffer19|0))|0);
   $196 = HEAP32[_stderr>>2]|0;
   $197 = $minutia;
   $198 = (($197) + 32|0);
   $199 = HEAP32[$198>>2]|0;
   HEAP32[$vararg_buffer21>>2] = $199;
   (_fprintf(($196|0),(7896|0),($vararg_buffer21|0))|0);
   $200 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($200);
   $0 = -4;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 41: {
   $204 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($204);
   $205 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($205);
   $206 = $ret;
   $0 = $206;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 43: {
   $211 = HEAP32[$item>>2]|0;
   _free_ANSI_NIST_item($211);
   $212 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($212);
   $213 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($213);
   $214 = $ret;
   $0 = $214;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 47: {
   $234 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($234);
   $235 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($235);
   $236 = $ret;
   $0 = $236;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 49: {
   $241 = HEAP32[$item>>2]|0;
   _free_ANSI_NIST_item($241);
   $242 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($242);
   $243 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($243);
   $244 = $ret;
   $0 = $244;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 53: {
   $251 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($251);
   $252 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($252);
   $253 = $ret;
   $0 = $253;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
  case 56: {
   $256 = HEAP32[$field>>2]|0;
   $257 = $1;
   HEAP32[$257>>2] = $256;
   $0 = 0;
   $258 = $0;
   STACKTOP = sp;return ($258|0);
   break;
  }
 }
 return 0|0;
}
function _binarize_V2($odata,$ow,$oh,$pdata,$pw,$ph,$direction_map,$mw,$mh,$dirbingrids,$lfsparms) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $pdata = $pdata|0;
 $pw = $pw|0;
 $ph = $ph|0;
 $direction_map = $direction_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $dirbingrids = $dirbingrids|0;
 $lfsparms = $lfsparms|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bdata = 0, $bh = 0, $bw = 0, $i = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $bdata = sp + 16|0;
 $bw = sp + 8|0;
 $bh = sp + 4|0;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $pdata;
 $5 = $pw;
 $6 = $ph;
 $7 = $direction_map;
 $8 = $mw;
 $9 = $mh;
 $10 = $dirbingrids;
 $11 = $lfsparms;
 $12 = $4;
 $13 = $5;
 $14 = $6;
 $15 = $7;
 $16 = $8;
 $17 = $9;
 $18 = $11;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $10;
 $22 = (_binarize_image_V2($bdata,$bw,$bh,$12,$13,$14,$15,$16,$17,$20,$21)|0);
 $ret = $22;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = $ret;
  $0 = $24;
  $41 = $0;
  STACKTOP = sp;return ($41|0);
 }
 $i = 0;
 while(1) {
  $25 = $i;
  $26 = $11;
  $27 = (($26) + 148|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$bdata>>2]|0;
  $31 = HEAP32[$bw>>2]|0;
  $32 = HEAP32[$bh>>2]|0;
  _fill_holes($30,$31,$32);
  $33 = $i;
  $34 = (($33) + 1)|0;
  $i = $34;
 }
 $35 = HEAP32[$bdata>>2]|0;
 $36 = $1;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$bw>>2]|0;
 $38 = $2;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$bh>>2]|0;
 $40 = $3;
 HEAP32[$40>>2] = $39;
 $0 = 0;
 $41 = $0;
 STACKTOP = sp;return ($41|0);
}
function _binarize_image_V2($odata,$ow,$oh,$pdata,$pw,$ph,$direction_map,$mw,$mh,$blocksize,$dirbingrids) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $pdata = $pdata|0;
 $pw = $pw|0;
 $ph = $ph|0;
 $direction_map = $direction_map|0;
 $mw = $mw|0;
 $mh = $mh|0;
 $blocksize = $blocksize|0;
 $dirbingrids = $dirbingrids|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $bdata = 0, $bh = 0, $bptr = 0, $bw = 0, $bx = 0, $by = 0, $ix = 0, $iy = 0, $mapval = 0, $pptr = 0, $spptr = 0, $vararg_buffer = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $pdata;
 $5 = $pw;
 $6 = $ph;
 $7 = $direction_map;
 $8 = $mw;
 $9 = $mh;
 $10 = $blocksize;
 $11 = $dirbingrids;
 $12 = $5;
 $13 = $11;
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 << 1;
 $16 = (($12) - ($15))|0;
 $bw = $16;
 $17 = $6;
 $18 = $11;
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 << 1;
 $21 = (($17) - ($20))|0;
 $bh = $21;
 $22 = $bw;
 $23 = $bh;
 $24 = Math_imul($22, $23)|0;
 $25 = $24;
 $26 = (_malloc($25)|0);
 $bdata = $26;
 $27 = $bdata;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[_stderr>>2]|0;
  (_fprintf(($29|0),(7944|0),($vararg_buffer|0))|0);
  $0 = -600;
  $87 = $0;
  STACKTOP = sp;return ($87|0);
 }
 $30 = $bdata;
 $bptr = $30;
 $31 = $4;
 $32 = $11;
 $33 = HEAP32[$32>>2]|0;
 $34 = $5;
 $35 = Math_imul($33, $34)|0;
 $36 = (($31) + ($35)|0);
 $37 = $11;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + ($38)|0);
 $spptr = $39;
 $iy = 0;
 while(1) {
  $40 = $iy;
  $41 = $bh;
  $42 = ($40|0)<($41|0);
  if (!($42)) {
   break;
  }
  $43 = $spptr;
  $pptr = $43;
  $ix = 0;
  while(1) {
   $44 = $ix;
   $45 = $bw;
   $46 = ($44|0)<($45|0);
   if (!($46)) {
    break;
   }
   $47 = $ix;
   $48 = $10;
   $49 = (($47|0) / ($48|0))&-1;
   $bx = $49;
   $50 = $iy;
   $51 = $10;
   $52 = (($50|0) / ($51|0))&-1;
   $by = $52;
   $53 = $7;
   $54 = $by;
   $55 = $8;
   $56 = Math_imul($54, $55)|0;
   $57 = (($53) + ($56<<2)|0);
   $58 = $bx;
   $59 = (($57) + ($58<<2)|0);
   $60 = HEAP32[$59>>2]|0;
   $mapval = $60;
   $61 = $mapval;
   $62 = ($61|0)==(-1);
   if ($62) {
    $63 = $bptr;
    HEAP8[$63>>0] = -1;
   } else {
    $64 = $pptr;
    $65 = $mapval;
    $66 = $11;
    $67 = (_dirbinarize($64,$65,$66)|0);
    $68 = $67&255;
    $69 = $bptr;
    HEAP8[$69>>0] = $68;
   }
   $70 = $pptr;
   $71 = (($70) + 1|0);
   $pptr = $71;
   $72 = $bptr;
   $73 = (($72) + 1|0);
   $bptr = $73;
   $74 = $ix;
   $75 = (($74) + 1)|0;
   $ix = $75;
  }
  $76 = $5;
  $77 = $spptr;
  $78 = (($77) + ($76)|0);
  $spptr = $78;
  $79 = $iy;
  $80 = (($79) + 1)|0;
  $iy = $80;
 }
 $81 = $bdata;
 $82 = $1;
 HEAP32[$82>>2] = $81;
 $83 = $bw;
 $84 = $2;
 HEAP32[$84>>2] = $83;
 $85 = $bh;
 $86 = $3;
 HEAP32[$86>>2] = $85;
 $0 = 0;
 $87 = $0;
 STACKTOP = sp;return ($87|0);
}
function _dirbinarize($pptr,$idir,$dirbingrids) {
 $pptr = $pptr|0;
 $idir = $idir|0;
 $dirbingrids = $dirbingrids|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $csum = 0, $cy = 0, $dcy = 0.0, $gi = 0, $grid = 0, $gsum = 0, $gx = 0, $gy = 0, $rsum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $pptr;
 $2 = $idir;
 $3 = $dirbingrids;
 $csum = 0;
 $4 = $2;
 $5 = $3;
 $6 = (($5) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $grid = $9;
 $10 = $3;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) - 1)|0;
 $14 = (+($13|0));
 $15 = $14 / 2.0;
 $dcy = $15;
 $16 = $dcy;
 $17 = $16 < 0.0;
 if ($17) {
  $18 = $dcy;
  $19 = $18 * 16384.0;
  $20 = $19 - 0.5;
  $21 = (~~(($20)));
  $22 = (+($21|0));
  $23 = $22 / 16384.0;
  $30 = $23;
 } else {
  $24 = $dcy;
  $25 = $24 * 16384.0;
  $26 = $25 + 0.5;
  $27 = (~~(($26)));
  $28 = (+($27|0));
  $29 = $28 / 16384.0;
  $30 = $29;
 }
 $dcy = $30;
 $31 = $dcy;
 $32 = $31 < 0.0;
 if ($32) {
  $33 = $dcy;
  $34 = $33 - 0.5;
  $38 = $34;
 } else {
  $35 = $dcy;
  $36 = $35 + 0.5;
  $38 = $36;
 }
 $37 = (~~(($38)));
 $cy = $37;
 $gi = 0;
 $gsum = 0;
 $gy = 0;
 while(1) {
  $39 = $gy;
  $40 = $3;
  $41 = (($40) + 24|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($39|0)<($42|0);
  if (!($43)) {
   break;
  }
  $rsum = 0;
  $gx = 0;
  while(1) {
   $44 = $gx;
   $45 = $3;
   $46 = (($45) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ($44|0)<($47|0);
   if (!($48)) {
    break;
   }
   $49 = $1;
   $50 = $gi;
   $51 = $grid;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = (($49) + ($53)|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55&255;
   $57 = $rsum;
   $58 = (($57) + ($56))|0;
   $rsum = $58;
   $59 = $gi;
   $60 = (($59) + 1)|0;
   $gi = $60;
   $61 = $gx;
   $62 = (($61) + 1)|0;
   $gx = $62;
  }
  $63 = $rsum;
  $64 = $gsum;
  $65 = (($64) + ($63))|0;
  $gsum = $65;
  $66 = $gy;
  $67 = $cy;
  $68 = ($66|0)==($67|0);
  if ($68) {
   $69 = $rsum;
   $csum = $69;
  }
  $70 = $gy;
  $71 = (($70) + 1)|0;
  $gy = $71;
 }
 $72 = $csum;
 $73 = $3;
 $74 = (($73) + 24|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = Math_imul($72, $75)|0;
 $77 = $gsum;
 $78 = ($76|0)<($77|0);
 if ($78) {
  $0 = 0;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 } else {
  $0 = 255;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 return 0|0;
}
function _chain_code_loop($ochain,$onchain,$contour_x,$contour_y,$ncontour) {
 $ochain = $ochain|0;
 $onchain = $onchain|0;
 $contour_x = $contour_x|0;
 $contour_y = $contour_y|0;
 $ncontour = $ncontour|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $chain = 0, $dx = 0, $dy = 0, $i = 0, $j = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $1 = $ochain;
 $2 = $onchain;
 $3 = $contour_x;
 $4 = $contour_y;
 $5 = $ncontour;
 $6 = $5;
 $7 = ($6|0)<=(3);
 if ($7) {
  $8 = $2;
  HEAP32[$8>>2] = 0;
  $0 = 0;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $9 = $5;
 $10 = $9<<2;
 $11 = (_malloc($10)|0);
 $chain = $11;
 $12 = $chain;
 $13 = ($12|0)==(0|0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  (_fprintf(($14|0),(7992|0),($vararg_buffer|0))|0);
  $0 = -170;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $i = 0;
 $j = 1;
 while(1) {
  $15 = $i;
  $16 = $5;
  $17 = (($16) - 1)|0;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $j;
  $20 = $3;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $i;
  $24 = $3;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($22) - ($26))|0;
  $dx = $27;
  $28 = $j;
  $29 = $4;
  $30 = (($29) + ($28<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $i;
  $33 = $4;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($31) - ($35))|0;
  $dy = $36;
  $37 = $dy;
  $38 = (($37) + 1)|0;
  $39 = ($38*3)|0;
  $40 = (976 + ($39<<2)|0);
  $41 = $dx;
  $42 = (($40) + ($41<<2)|0);
  $43 = (($42) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = $i;
  $46 = $chain;
  $47 = (($46) + ($45<<2)|0);
  HEAP32[$47>>2] = $44;
  $48 = $i;
  $49 = (($48) + 1)|0;
  $i = $49;
  $50 = $j;
  $51 = (($50) + 1)|0;
  $j = $51;
 }
 $52 = $3;
 $53 = HEAP32[$52>>2]|0;
 $54 = $i;
 $55 = $3;
 $56 = (($55) + ($54<<2)|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = (($53) - ($57))|0;
 $dx = $58;
 $59 = $4;
 $60 = HEAP32[$59>>2]|0;
 $61 = $i;
 $62 = $4;
 $63 = (($62) + ($61<<2)|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = (($60) - ($64))|0;
 $dy = $65;
 $66 = $dy;
 $67 = (($66) + 1)|0;
 $68 = ($67*3)|0;
 $69 = (976 + ($68<<2)|0);
 $70 = $dx;
 $71 = (($69) + ($70<<2)|0);
 $72 = (($71) + 4|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $i;
 $75 = $chain;
 $76 = (($75) + ($74<<2)|0);
 HEAP32[$76>>2] = $73;
 $77 = $chain;
 $78 = $1;
 HEAP32[$78>>2] = $77;
 $79 = $5;
 $80 = $2;
 HEAP32[$80>>2] = $79;
 $0 = 0;
 $81 = $0;
 STACKTOP = sp;return ($81|0);
}
function _is_chain_clockwise($chain,$nchain,$default_ret) {
 $chain = $chain|0;
 $nchain = $nchain|0;
 $default_ret = $default_ret|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, $i = 0, $j = 0, $sum = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $chain;
 $2 = $nchain;
 $3 = $default_ret;
 $sum = 0;
 $i = 0;
 $j = 1;
 while(1) {
  $4 = $i;
  $5 = $2;
  $6 = (($5) - 1)|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $j;
  $9 = $1;
  $10 = (($9) + ($8<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $i;
  $13 = $1;
  $14 = (($13) + ($12<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($11) - ($15))|0;
  $d = $16;
  $17 = $d;
  $18 = ($17|0)>=(4);
  if ($18) {
   $19 = $d;
   $20 = (($19) - 8)|0;
   $d = $20;
  } else {
   $21 = $d;
   $22 = ($21|0)<=(-4);
   if ($22) {
    $23 = $d;
    $24 = (($23) + 8)|0;
    $d = $24;
   }
  }
  $25 = $d;
  $26 = $sum;
  $27 = (($26) + ($25))|0;
  $sum = $27;
  $28 = $i;
  $29 = (($28) + 1)|0;
  $i = $29;
  $30 = $j;
  $31 = (($30) + 1)|0;
  $j = $31;
 }
 $32 = $1;
 $33 = HEAP32[$32>>2]|0;
 $34 = $i;
 $35 = $1;
 $36 = (($35) + ($34<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($33) - ($37))|0;
 $d = $38;
 $39 = $d;
 $40 = ($39|0)>=(4);
 if ($40) {
  $41 = $d;
  $42 = (($41) - 8)|0;
  $d = $42;
 } else {
  $43 = $d;
  $44 = ($43|0)<=(-4);
  if ($44) {
   $45 = $d;
   $46 = (($45) + 8)|0;
   $d = $46;
  }
 }
 $47 = $d;
 $48 = $sum;
 $49 = (($48) + ($47))|0;
 $sum = $49;
 $50 = $sum;
 $51 = ($50|0)==(0);
 if ($51) {
  $52 = $3;
  $0 = $52;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $53 = $sum;
 $54 = ($53|0)>(0);
 if ($54) {
  $0 = 0;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 } else {
  $0 = 1;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 return 0|0;
}
function _dft_dir_powers($powers,$pdata,$blkoffset,$pw,$ph,$dftwaves,$dftgrids) {
 $powers = $powers|0;
 $pdata = $pdata|0;
 $blkoffset = $blkoffset|0;
 $pw = $pw|0;
 $ph = $ph|0;
 $dftwaves = $dftwaves|0;
 $dftgrids = $dftgrids|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $blkptr = 0, $dir = 0, $rowsums = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $powers;
 $2 = $pdata;
 $3 = $blkoffset;
 $4 = $pw;
 $5 = $ph;
 $6 = $dftwaves;
 $7 = $dftgrids;
 $8 = $7;
 $9 = (($8) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $7;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(8040|0),($vararg_buffer|0))|0);
  $0 = -90;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $16 = $7;
 $17 = (($16) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $18<<2;
 $20 = (_malloc($19)|0);
 $rowsums = $20;
 $21 = $rowsums;
 $22 = ($21|0)==(0|0);
 if ($22) {
  $23 = HEAP32[_stderr>>2]|0;
  (_fprintf(($23|0),(8096|0),($vararg_buffer1|0))|0);
  $0 = -91;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $dir = 0;
 while(1) {
  $24 = $dir;
  $25 = $7;
  $26 = (($25) + 16|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($24|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = $2;
  $30 = $3;
  $31 = (($29) + ($30)|0);
  $blkptr = $31;
  $32 = $rowsums;
  $33 = $blkptr;
  $34 = $dir;
  $35 = $7;
  $36 = (($35) + 28|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = $7;
  $41 = (($40) + 20|0);
  $42 = HEAP32[$41>>2]|0;
  _sum_rot_block_rows($32,$33,$39,$42);
  $w = 0;
  while(1) {
   $43 = $w;
   $44 = $6;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($43|0)<($45|0);
   if (!($46)) {
    break;
   }
   $47 = $dir;
   $48 = $w;
   $49 = $1;
   $50 = (($49) + ($48<<2)|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (($51) + ($47<<3)|0);
   $53 = $rowsums;
   $54 = $w;
   $55 = $6;
   $56 = (($55) + 8|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + ($54<<2)|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $6;
   $61 = (($60) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   _dft_power($52,$53,$59,$62);
   $63 = $w;
   $64 = (($63) + 1)|0;
   $w = $64;
  }
  $65 = $dir;
  $66 = (($65) + 1)|0;
  $dir = $66;
 }
 $67 = $rowsums;
 _free($67);
 $0 = 0;
 $68 = $0;
 STACKTOP = sp;return ($68|0);
}
function _sum_rot_block_rows($rowsums,$blkptr,$grid_offsets,$blocksize) {
 $rowsums = $rowsums|0;
 $blkptr = $blkptr|0;
 $grid_offsets = $grid_offsets|0;
 $blocksize = $blocksize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $gi = 0, $ix = 0, $iy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $rowsums;
 $1 = $blkptr;
 $2 = $grid_offsets;
 $3 = $blocksize;
 $gi = 0;
 $iy = 0;
 while(1) {
  $4 = $iy;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $iy;
  $8 = $0;
  $9 = (($8) + ($7<<2)|0);
  HEAP32[$9>>2] = 0;
  $ix = 0;
  while(1) {
   $10 = $ix;
   $11 = $3;
   $12 = ($10|0)<($11|0);
   if (!($12)) {
    break;
   }
   $13 = $1;
   $14 = $gi;
   $15 = $2;
   $16 = (($15) + ($14<<2)|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($13) + ($17)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = $iy;
   $22 = $0;
   $23 = (($22) + ($21<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($20))|0;
   HEAP32[$23>>2] = $25;
   $26 = $gi;
   $27 = (($26) + 1)|0;
   $gi = $27;
   $28 = $ix;
   $29 = (($28) + 1)|0;
   $ix = $29;
  }
  $30 = $iy;
  $31 = (($30) + 1)|0;
  $iy = $31;
 }
 STACKTOP = sp;return;
}
function _dft_power($power,$rowsums,$wave,$wavelen) {
 $power = $power|0;
 $rowsums = $rowsums|0;
 $wave = $wave|0;
 $wavelen = $wavelen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0, $35 = 0, $36 = 0.0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cospart = 0.0, $i = 0, $sinpart = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $power;
 $1 = $rowsums;
 $2 = $wave;
 $3 = $wavelen;
 $cospart = 0.0;
 $sinpart = 0.0;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $1;
  $9 = (($8) + ($7<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (+($10|0));
  $12 = $i;
  $13 = $2;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($12<<3)|0);
  $16 = +HEAPF64[$15>>3];
  $17 = $11 * $16;
  $18 = $cospart;
  $19 = $18 + $17;
  $cospart = $19;
  $20 = $i;
  $21 = $1;
  $22 = (($21) + ($20<<2)|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (+($23|0));
  $25 = $i;
  $26 = $2;
  $27 = (($26) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<3)|0);
  $30 = +HEAPF64[$29>>3];
  $31 = $24 * $30;
  $32 = $sinpart;
  $33 = $32 + $31;
  $sinpart = $33;
  $34 = $i;
  $35 = (($34) + 1)|0;
  $i = $35;
 }
 $36 = $cospart;
 $37 = $cospart;
 $38 = $36 * $37;
 $39 = $sinpart;
 $40 = $sinpart;
 $41 = $39 * $40;
 $42 = $38 + $41;
 $43 = $0;
 HEAPF64[$43>>3] = $42;
 STACKTOP = sp;return;
}
function _dft_power_stats($wis,$powmaxs,$powmax_dirs,$pownorms,$powers,$fw,$tw,$ndirs) {
 $wis = $wis|0;
 $powmaxs = $powmaxs|0;
 $powmax_dirs = $powmax_dirs|0;
 $pownorms = $pownorms|0;
 $powers = $powers|0;
 $fw = $fw|0;
 $tw = $tw|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $ret = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $wis;
 $2 = $powmaxs;
 $3 = $powmax_dirs;
 $4 = $pownorms;
 $5 = $powers;
 $6 = $fw;
 $7 = $tw;
 $8 = $ndirs;
 $9 = $6;
 $w = $9;
 $i = 0;
 while(1) {
  $10 = $w;
  $11 = $7;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = $i;
  $14 = $2;
  $15 = (($14) + ($13<<3)|0);
  $16 = $i;
  $17 = $3;
  $18 = (($17) + ($16<<2)|0);
  $19 = $i;
  $20 = $4;
  $21 = (($20) + ($19<<3)|0);
  $22 = $w;
  $23 = $5;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $8;
  _get_max_norm($15,$18,$21,$25,$26);
  $27 = $w;
  $28 = (($27) + 1)|0;
  $w = $28;
  $29 = $i;
  $30 = (($29) + 1)|0;
  $i = $30;
 }
 $31 = $1;
 $32 = $2;
 $33 = $4;
 $34 = $7;
 $35 = $6;
 $36 = (($34) - ($35))|0;
 $37 = (_sort_dft_waves($31,$32,$33,$36)|0);
 $ret = $37;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = $ret;
  $0 = $39;
  $40 = $0;
  STACKTOP = sp;return ($40|0);
 } else {
  $0 = 0;
  $40 = $0;
  STACKTOP = sp;return ($40|0);
 }
 return 0|0;
}
function _get_max_norm($powmax,$powmax_dir,$pownorm,$power_vector,$ndirs) {
 $powmax = $powmax|0;
 $powmax_dir = $powmax_dir|0;
 $pownorm = $pownorm|0;
 $power_vector = $power_vector|0;
 $ndirs = $ndirs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0, $37 = 0.0, $38 = 0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0, $43 = 0.0, $44 = 0.0;
 var $45 = 0.0, $46 = 0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0.0, $9 = 0, $dir = 0, $max_i = 0, $max_v = 0.0, $powmean = 0.0, $powsum = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $powmax;
 $1 = $powmax_dir;
 $2 = $pownorm;
 $3 = $power_vector;
 $4 = $ndirs;
 $5 = $3;
 $6 = +HEAPF64[$5>>3];
 $max_v = $6;
 $max_i = 0;
 $7 = $3;
 $8 = +HEAPF64[$7>>3];
 $powsum = $8;
 $dir = 1;
 while(1) {
  $9 = $dir;
  $10 = $4;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = $dir;
  $13 = $3;
  $14 = (($13) + ($12<<3)|0);
  $15 = +HEAPF64[$14>>3];
  $16 = $powsum;
  $17 = $16 + $15;
  $powsum = $17;
  $18 = $dir;
  $19 = $3;
  $20 = (($19) + ($18<<3)|0);
  $21 = +HEAPF64[$20>>3];
  $22 = $max_v;
  $23 = $21 > $22;
  if ($23) {
   $24 = $dir;
   $25 = $3;
   $26 = (($25) + ($24<<3)|0);
   $27 = +HEAPF64[$26>>3];
   $max_v = $27;
   $28 = $dir;
   $max_i = $28;
  }
  $29 = $dir;
  $30 = (($29) + 1)|0;
  $dir = $30;
 }
 $31 = $max_v;
 $32 = $0;
 HEAPF64[$32>>3] = $31;
 $33 = $max_i;
 $34 = $1;
 HEAP32[$34>>2] = $33;
 $35 = $powsum;
 $36 = $35 > 10.0;
 if ($36) {
  $37 = $powsum;
  $41 = $37;
  $38 = $4;
  $39 = (+($38|0));
  $40 = $41 / $39;
  $powmean = $40;
  $42 = $0;
  $43 = +HEAPF64[$42>>3];
  $44 = $powmean;
  $45 = $43 / $44;
  $46 = $2;
  HEAPF64[$46>>3] = $45;
  STACKTOP = sp;return;
 } else {
  $41 = 10.0;
  $38 = $4;
  $39 = (+($38|0));
  $40 = $41 / $39;
  $powmean = $40;
  $42 = $0;
  $43 = +HEAPF64[$42>>3];
  $44 = $powmean;
  $45 = $43 / $44;
  $46 = $2;
  HEAPF64[$46>>3] = $45;
  STACKTOP = sp;return;
 }
}
function _sort_dft_waves($wis,$powmaxs,$pownorms,$nstats) {
 $wis = $wis|0;
 $powmaxs = $powmaxs|0;
 $pownorms = $pownorms|0;
 $nstats = $nstats|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0, $26 = 0.0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pownorms2 = 0, $vararg_buffer = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $wis;
 $2 = $powmaxs;
 $3 = $pownorms;
 $4 = $nstats;
 $5 = $4;
 $6 = $5<<3;
 $7 = (_malloc($6)|0);
 $pownorms2 = $7;
 $8 = $pownorms2;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(8144|0),($vararg_buffer|0))|0);
  $0 = -100;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $4;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $i;
  $15 = $i;
  $16 = $1;
  $17 = (($16) + ($15<<2)|0);
  HEAP32[$17>>2] = $14;
  $18 = $i;
  $19 = $2;
  $20 = (($19) + ($18<<3)|0);
  $21 = +HEAPF64[$20>>3];
  $22 = $i;
  $23 = $3;
  $24 = (($23) + ($22<<3)|0);
  $25 = +HEAPF64[$24>>3];
  $26 = $21 * $25;
  $27 = $i;
  $28 = $pownorms2;
  $29 = (($28) + ($27<<3)|0);
  HEAPF64[$29>>3] = $26;
  $30 = $i;
  $31 = (($30) + 1)|0;
  $i = $31;
 }
 $32 = $pownorms2;
 $33 = $1;
 $34 = $4;
 _bubble_sort_double_dec_2($32,$33,$34);
 $35 = $pownorms2;
 _free($35);
 $0 = 0;
 $36 = $0;
 STACKTOP = sp;return ($36|0);
}
function _alloc_ANSI_NIST($oansi_nist) {
 $oansi_nist = $oansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ansi_nist = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $oansi_nist;
 $2 = (_malloc(20)|0);
 $ansi_nist = $2;
 $3 = $ansi_nist;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 20;
  (_fprintf(($5|0),(8192|0),($vararg_buffer|0))|0);
  $0 = -2;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $6 = $ansi_nist;
 HEAP32[$6>>2] = -1;
 $7 = $ansi_nist;
 $8 = (($7) + 4|0);
 HEAP32[$8>>2] = 0;
 $9 = $ansi_nist;
 $10 = (($9) + 8|0);
 HEAP32[$10>>2] = 0;
 $11 = $ansi_nist;
 $12 = (($11) + 12|0);
 HEAP32[$12>>2] = 100;
 $13 = (_malloc(400)|0);
 $14 = $ansi_nist;
 $15 = (($14) + 16|0);
 HEAP32[$15>>2] = $13;
 $16 = $ansi_nist;
 $17 = (($16) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  $20 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer1>>2] = 100;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 400;
  (_fprintf(($20|0),(8256|0),($vararg_buffer1|0))|0);
  $21 = $ansi_nist;
  _free($21);
  $0 = -3;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 } else {
  $22 = $ansi_nist;
  $23 = $1;
  HEAP32[$23>>2] = $22;
  $0 = 0;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _new_ANSI_NIST_record($orecord,$record_type) {
 $orecord = $orecord|0;
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $record = 0, $ret = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $record = sp;
 $1 = $orecord;
 $2 = $record_type;
 $3 = (_alloc_ANSI_NIST_record($record)|0);
 $ret = $3;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = $ret;
  $0 = $5;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 } else {
  $6 = $2;
  $7 = HEAP32[$record>>2]|0;
  HEAP32[$7>>2] = $6;
  $8 = HEAP32[$record>>2]|0;
  $9 = (($8) + 24|0);
  HEAP32[$9>>2] = 1;
  $10 = HEAP32[$record>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$11>>2] = $13;
  $14 = HEAP32[$record>>2]|0;
  $15 = $1;
  HEAP32[$15>>2] = $14;
  $0 = 0;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _alloc_ANSI_NIST_record($orecord) {
 $orecord = $orecord|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $record = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $orecord;
 $2 = (_malloc(28)|0);
 $record = $2;
 $3 = $record;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 28;
  (_fprintf(($5|0),(8320|0),($vararg_buffer|0))|0);
  $0 = -2;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 $6 = (_malloc(400)|0);
 $7 = $record;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = $6;
 $9 = $record;
 $10 = (($9) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer1>>2] = 100;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 400;
  (_fprintf(($13|0),(8384|0),($vararg_buffer1|0))|0);
  $14 = $record;
  _free($14);
  $0 = -3;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 } else {
  $15 = $record;
  HEAP32[$15>>2] = -1;
  $16 = $record;
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = -1;
  $18 = $record;
  $19 = (($18) + 8|0);
  HEAP32[$19>>2] = 0;
  $20 = $record;
  $21 = (($20) + 12|0);
  HEAP32[$21>>2] = 0;
  $22 = $record;
  $23 = (($22) + 16|0);
  HEAP32[$23>>2] = 100;
  $24 = $record;
  $25 = (($24) + 24|0);
  HEAP32[$25>>2] = 0;
  $26 = $record;
  $27 = $1;
  HEAP32[$27>>2] = $26;
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _new_ANSI_NIST_field($ofield,$record_type,$field_int) {
 $ofield = $ofield|0;
 $record_type = $record_type|0;
 $field_int = $field_int|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $field_id_bytes = 0, $id_strlen = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $field = sp + 36|0;
 $1 = $ofield;
 $2 = $record_type;
 $3 = $field_int;
 $4 = (_alloc_ANSI_NIST_field($field)|0);
 $ret = $4;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = $ret;
  $0 = $6;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 $7 = $2;
 $8 = HEAP32[$field>>2]|0;
 $9 = (($8) + 4|0);
 HEAP32[$9>>2] = $7;
 $10 = $3;
 $11 = HEAP32[$field>>2]|0;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = $10;
 $field_id_bytes = 21;
 $13 = $field_id_bytes;
 $14 = (_calloc($13,1)|0);
 $15 = HEAP32[$field>>2]|0;
 HEAP32[$15>>2] = $14;
 $16 = HEAP32[$field>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  $20 = $2;
  $21 = $3;
  $22 = $field_id_bytes;
  HEAP32[$vararg_buffer>>2] = $20;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $21;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $22;
  (_fprintf(($19|0),(8456|0),($vararg_buffer|0))|0);
  $23 = HEAP32[$field>>2]|0;
  _free($23);
  $0 = -2;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 $24 = HEAP32[$field>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = $2;
 $27 = $3;
 HEAP32[$vararg_buffer3>>2] = $26;
 $vararg_ptr6 = (($vararg_buffer3) + 4|0);
 HEAP32[$vararg_ptr6>>2] = $27;
 $28 = (_sprintf($25,8528,$vararg_buffer3)|0);
 $id_strlen = $28;
 $29 = $id_strlen;
 $30 = $field_id_bytes;
 $31 = ($29|0)>=($30|0);
 if ($31) {
  $32 = HEAP32[_stderr>>2]|0;
  $33 = $id_strlen;
  $34 = $field_id_bytes;
  HEAP32[$vararg_buffer7>>2] = $33;
  $vararg_ptr10 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr10>>2] = $34;
  (_fprintf(($32|0),(8544|0),($vararg_buffer7|0))|0);
  $35 = HEAP32[$field>>2]|0;
  $36 = HEAP32[$35>>2]|0;
  _free($36);
  $37 = HEAP32[$field>>2]|0;
  _free($37);
  $0 = -3;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 } else {
  $38 = $id_strlen;
  $39 = HEAP32[$field>>2]|0;
  $40 = (($39) + 12|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($38))|0;
  HEAP32[$40>>2] = $42;
  $43 = HEAP32[$field>>2]|0;
  $44 = $1;
  HEAP32[$44>>2] = $43;
  $0 = 0;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 return 0|0;
}
function _alloc_ANSI_NIST_field($ofield) {
 $ofield = $ofield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofield;
 $2 = (_malloc(32)|0);
 $field = $2;
 $3 = $field;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 32;
  (_fprintf(($5|0),(8624|0),($vararg_buffer|0))|0);
  $0 = -2;
  $30 = $0;
  STACKTOP = sp;return ($30|0);
 }
 $6 = (_malloc(400)|0);
 $7 = $field;
 $8 = (($7) + 24|0);
 HEAP32[$8>>2] = $6;
 $9 = $field;
 $10 = (($9) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer1>>2] = 100;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 400;
  (_fprintf(($13|0),(8688|0),($vararg_buffer1|0))|0);
  $14 = $field;
  _free($14);
  $0 = -3;
  $30 = $0;
  STACKTOP = sp;return ($30|0);
 } else {
  $15 = $field;
  HEAP32[$15>>2] = 0;
  $16 = $field;
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = -1;
  $18 = $field;
  $19 = (($18) + 8|0);
  HEAP32[$19>>2] = -1;
  $20 = $field;
  $21 = (($20) + 12|0);
  HEAP32[$21>>2] = 0;
  $22 = $field;
  $23 = (($22) + 16|0);
  HEAP32[$23>>2] = 0;
  $24 = $field;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 100;
  $26 = $field;
  $27 = (($26) + 28|0);
  HEAP32[$27>>2] = 0;
  $28 = $field;
  $29 = $1;
  HEAP32[$29>>2] = $28;
  $0 = 0;
  $30 = $0;
  STACKTOP = sp;return ($30|0);
 }
 return 0|0;
}
function _alloc_ANSI_NIST_subfield($osubfield) {
 $osubfield = $osubfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $subfield = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $osubfield;
 $2 = (_malloc(20)|0);
 $subfield = $2;
 $3 = $subfield;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 20;
  (_fprintf(($5|0),(8760|0),($vararg_buffer|0))|0);
  $0 = -2;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $6 = $subfield;
 $7 = (($6) + 8|0);
 HEAP32[$7>>2] = 100;
 $8 = (_malloc(400)|0);
 $9 = $subfield;
 $10 = (($9) + 12|0);
 HEAP32[$10>>2] = $8;
 $11 = $subfield;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer1>>2] = 100;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 400;
  (_fprintf(($15|0),(8832|0),($vararg_buffer1|0))|0);
  $16 = $subfield;
  _free($16);
  $0 = -3;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 } else {
  $17 = $subfield;
  HEAP32[$17>>2] = 0;
  $18 = $subfield;
  $19 = (($18) + 4|0);
  HEAP32[$19>>2] = 0;
  $20 = $subfield;
  $21 = (($20) + 16|0);
  HEAP32[$21>>2] = 0;
  $22 = $subfield;
  $23 = $1;
  HEAP32[$23>>2] = $22;
  $0 = 0;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _alloc_ANSI_NIST_item($oitem) {
 $oitem = $oitem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $oitem;
 $2 = (_malloc(20)|0);
 $item = $2;
 $3 = $item;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 20;
  (_fprintf(($5|0),(8904|0),($vararg_buffer|0))|0);
  $0 = -2;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 $6 = $item;
 $7 = (($6) + 8|0);
 HEAP32[$7>>2] = 100;
 $8 = (_malloc(100)|0);
 $9 = $item;
 $10 = (($9) + 12|0);
 HEAP32[$10>>2] = $8;
 $11 = $item;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer1>>2] = 100;
  (_fprintf(($15|0),(8968|0),($vararg_buffer1|0))|0);
  $16 = $item;
  _free($16);
  $0 = -3;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 } else {
  $17 = $item;
  HEAP32[$17>>2] = 0;
  $18 = $item;
  $19 = (($18) + 4|0);
  HEAP32[$19>>2] = 0;
  $20 = $item;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP8[$22>>0] = 0;
  $23 = $item;
  $24 = (($23) + 16|0);
  HEAP32[$24>>2] = 0;
  $25 = $item;
  $26 = $1;
  HEAP32[$26>>2] = $25;
  $0 = 0;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 return 0|0;
}
function _free_ANSI_NIST($ansi_nist) {
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $ansi_nist;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 8|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free_ANSI_NIST_record($11);
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $0;
 $15 = (($14) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 _free($16);
 $17 = $0;
 _free($17);
 STACKTOP = sp;return;
}
function _free_ANSI_NIST_record($record) {
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $record;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 12|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free_ANSI_NIST_field($11);
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $0;
 $15 = (($14) + 20|0);
 $16 = HEAP32[$15>>2]|0;
 _free($16);
 $17 = $0;
 _free($17);
 STACKTOP = sp;return;
}
function _free_ANSI_NIST_field($field) {
 $field = $field|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $field;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 16|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 24|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free_ANSI_NIST_subfield($11);
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $0;
 $15 = (($14) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 _free($16);
 $17 = $0;
 $18 = HEAP32[$17>>2]|0;
 _free($18);
 $19 = $0;
 _free($19);
 STACKTOP = sp;return;
}
function _free_ANSI_NIST_subfield($subfield) {
 $subfield = $subfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $subfield;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = $0;
  $3 = (($2) + 4|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $0;
  $8 = (($7) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free_ANSI_NIST_item($11);
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $0;
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 _free($16);
 $17 = $0;
 _free($17);
 STACKTOP = sp;return;
}
function _free_ANSI_NIST_item($item) {
 $item = $item|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $item;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 _free($3);
 $4 = $0;
 _free($4);
 STACKTOP = sp;return;
}
function _append_ANSI_NIST_record($record,$field) {
 $record = $record|0;
 $field = $field|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lastfield = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $record;
 $2 = $field;
 $3 = $1;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = $1;
  $8 = (($7) + 24|0);
  HEAP32[$8>>2] = 1;
  $9 = $1;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 1)|0;
  HEAP32[$10>>2] = $12;
 }
 $13 = $1;
 $14 = (($13) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)>(0);
 if ($16) {
  $17 = $1;
  $18 = (($17) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - 1)|0;
  $21 = $1;
  $22 = (($21) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $lastfield = $25;
  $26 = $lastfield;
  $27 = (($26) + 28|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0);
  if (!($29)) {
   $30 = $lastfield;
   $31 = (($30) + 28|0);
   HEAP32[$31>>2] = 1;
   $32 = $lastfield;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$33>>2] = $35;
   $36 = $1;
   $37 = (($36) + 8|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + 1)|0;
   HEAP32[$37>>2] = $39;
  }
 }
 $40 = $1;
 $41 = $2;
 $42 = (_update_ANSI_NIST_record($40,$41)|0);
 $ret = $42;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = $ret;
  $0 = $44;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 } else {
  $0 = 0;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 return 0|0;
}
function _append_ANSI_NIST_field($field,$subfield) {
 $field = $field|0;
 $subfield = $subfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lastsubfield = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $field;
 $2 = $subfield;
 $3 = $1;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = $1;
  $8 = (($7) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) - 1)|0;
  $11 = $1;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $lastsubfield = $15;
  $16 = $lastsubfield;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)!=(0);
  if (!($19)) {
   $20 = $lastsubfield;
   $21 = (($20) + 16|0);
   HEAP32[$21>>2] = 1;
   $22 = $lastsubfield;
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$22>>2] = $24;
   $25 = $1;
   $26 = (($25) + 12|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + 1)|0;
   HEAP32[$26>>2] = $28;
  }
 }
 $29 = $1;
 $30 = $2;
 $31 = (_update_ANSI_NIST_field($29,$30)|0);
 $ret = $31;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = $ret;
  $0 = $33;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 } else {
  $0 = 0;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 return 0|0;
}
function _append_ANSI_NIST_subfield($subfield,$item) {
 $subfield = $subfield|0;
 $item = $item|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lastitem = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $subfield;
 $2 = $item;
 $3 = $1;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = $1;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) - 1)|0;
  $11 = $1;
  $12 = (($11) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $lastitem = $15;
  $16 = $lastitem;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)!=(0);
  if (!($19)) {
   $20 = $lastitem;
   $21 = (($20) + 16|0);
   HEAP32[$21>>2] = 1;
   $22 = $lastitem;
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$22>>2] = $24;
   $25 = $1;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$25>>2] = $27;
  }
 }
 $28 = $1;
 $29 = $2;
 $30 = (_update_ANSI_NIST_subfield($28,$29)|0);
 $ret = $30;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = $ret;
  $0 = $32;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 } else {
  $0 = 0;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 return 0|0;
}
function _read_ANSI_NIST_file($ifile,$oansi_nist) {
 $ifile = $ifile|0;
 $oansi_nist = $oansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ansi_nist = 0, $fpin = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer14 = 0;
 var $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer14 = sp + 32|0;
 $vararg_buffer10 = sp + 24|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $ansi_nist = sp + 40|0;
 $1 = $ifile;
 $2 = $oansi_nist;
 $3 = $1;
 $4 = (_fopen(($3|0),(9032|0))|0);
 $fpin = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  $7 = $1;
  $8 = (___errno_location()|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_strerror(($9|0))|0);
  HEAP32[$vararg_buffer>>2] = $7;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $10;
  (_fprintf(($6|0),(9040|0),($vararg_buffer|0))|0);
  $0 = -2;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 $11 = (_alloc_ANSI_NIST($ansi_nist)|0);
 $ret = $11;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $fpin;
  $14 = (_fclose(($13|0))|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[_stderr>>2]|0;
   $17 = $1;
   $18 = (___errno_location()|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (_strerror(($19|0))|0);
   HEAP32[$vararg_buffer2>>2] = $17;
   $vararg_ptr5 = (($vararg_buffer2) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $20;
   (_fprintf(($16|0),(9088|0),($vararg_buffer2|0))|0);
  }
  $21 = $ret;
  $0 = $21;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 $22 = $fpin;
 $23 = HEAP32[$ansi_nist>>2]|0;
 $24 = (_read_ANSI_NIST($22,$23)|0);
 $ret = $24;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP32[$ansi_nist>>2]|0;
  _free_ANSI_NIST($26);
  $27 = $fpin;
  $28 = (_fclose(($27|0))|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[_stderr>>2]|0;
   $31 = $1;
   $32 = (___errno_location()|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (_strerror(($33|0))|0);
   HEAP32[$vararg_buffer6>>2] = $31;
   $vararg_ptr9 = (($vararg_buffer6) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $34;
   (_fprintf(($30|0),(9088|0),($vararg_buffer6|0))|0);
   $0 = -4;
   $53 = $0;
   STACKTOP = sp;return ($53|0);
  } else {
   $35 = $ret;
   $0 = $35;
   $53 = $0;
   STACKTOP = sp;return ($53|0);
  }
 }
 $36 = $fpin;
 $37 = (_fgetc(($36|0))|0);
 $38 = ($37|0)!=(-1);
 if ($38) {
  $39 = HEAP32[_stderr>>2]|0;
  $40 = $1;
  $41 = $fpin;
  $42 = (_ftell(($41|0))|0);
  HEAP32[$vararg_buffer10>>2] = $40;
  $vararg_ptr13 = (($vararg_buffer10) + 4|0);
  HEAP32[$vararg_ptr13>>2] = $42;
  (_fprintf(($39|0),(9136|0),($vararg_buffer10|0))|0);
 }
 $43 = $fpin;
 $44 = (_fclose(($43|0))|0);
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[_stderr>>2]|0;
  $47 = $1;
  $48 = (___errno_location()|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_strerror(($49|0))|0);
  HEAP32[$vararg_buffer14>>2] = $47;
  $vararg_ptr17 = (($vararg_buffer14) + 4|0);
  HEAP32[$vararg_ptr17>>2] = $50;
  (_fprintf(($46|0),(9088|0),($vararg_buffer14|0))|0);
  $0 = -5;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 } else {
  $51 = HEAP32[$ansi_nist>>2]|0;
  $52 = $2;
  HEAP32[$52>>2] = $51;
  $0 = 0;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 return 0|0;
}
function _read_ANSI_NIST($fpin,$ansi_nist) {
 $fpin = $fpin|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $fpin;
 $1 = $ansi_nist;
 $2 = $0;
 $3 = $1;
 $4 = (_i_read_ANSI_NIST($2,0,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _i_read_ANSI_NIST($fpin,$buf,$ansi_nist) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dcsfield = 0, $dcsfield_i = 0, $record = 0, $ret = 0, $vararg_buffer = 0, $version = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $version = sp + 16|0;
 $record = sp + 12|0;
 $dcsfield = sp + 8|0;
 $dcsfield_i = sp + 4|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ansi_nist;
 $4 = $1;
 $5 = $2;
 $6 = (_i_read_Type1_record($4,$5,$record,$version)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 }
 $9 = HEAP32[$version>>2]|0;
 $10 = $3;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$record>>2]|0;
 $12 = (_lookup_ANSI_NIST_field($dcsfield,$dcsfield_i,15,$11)|0);
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($14);
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(16104|0),($vararg_buffer|0))|0);
  $0 = -2;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 }
 $16 = $3;
 $17 = HEAP32[$record>>2]|0;
 $18 = (_update_ANSI_NIST($16,$17)|0);
 $ret = $18;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($20);
  $21 = $ret;
  $0 = $21;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 } else {
  $22 = $1;
  $23 = $2;
  $24 = $3;
  $25 = (_i_read_ANSI_NIST_remaining_records($22,$23,$24)|0);
  $0 = $25;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 }
 return 0|0;
}
function _i_read_Type1_record($fpin,$buf,$orecord,$oversion) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $orecord = $orecord|0;
 $oversion = $oversion|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $field = 0, $record = 0, $record_bytes = 0, $ret = 0, $ret_delimiter = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer23 = 0, $vararg_buffer28 = 0;
 var $vararg_buffer6 = 0, $vararg_ptr10 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr27 = 0, $vararg_ptr31 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $version = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer28 = sp + 96|0;
 $vararg_buffer23 = sp + 80|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer14 = sp + 48|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $record_bytes = sp + 116|0;
 $version = sp + 112|0;
 $record = sp + 108|0;
 $field = sp + 104|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $orecord;
 $4 = $oversion;
 $5 = (_alloc_ANSI_NIST_record($record)|0);
 $ret = $5;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $8 = $1;
 $9 = $2;
 $10 = (_i_read_ANSI_NIST_record_length($8,$9,$record_bytes,$field)|0);
 $ret_delimiter = $10;
 $11 = $ret_delimiter;
 $12 = ($11|0)<(0);
 if ($12) {
  $13 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($13);
  $14 = $ret_delimiter;
  $0 = $14;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $15 = HEAP32[$field>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(1);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  $20 = HEAP32[$field>>2]|0;
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  (_fprintf(($19|0),(15456|0),($vararg_buffer|0))|0);
  $23 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($23);
  $24 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($24);
  $0 = -2;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $25 = HEAP32[$record>>2]|0;
 HEAP32[$25>>2] = 1;
 $26 = HEAP32[$record_bytes>>2]|0;
 $27 = HEAP32[$record>>2]|0;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = $ret_delimiter;
 $30 = ($29|0)!=(29);
 if ($30) {
  $31 = HEAP32[_stderr>>2]|0;
  $32 = $ret_delimiter;
  $33 = $1;
  $34 = $2;
  $35 = (_fbtell($33,$34)|0);
  HEAP32[$vararg_buffer1>>2] = $32;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 29;
  $vararg_ptr5 = (($vararg_buffer1) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $35;
  (_fprintf(($31|0),(15520|0),($vararg_buffer1|0))|0);
  $36 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($36);
  $0 = -3;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $37 = HEAP32[$record>>2]|0;
 $38 = HEAP32[$field>>2]|0;
 $39 = (_update_ANSI_NIST_record($37,$38)|0);
 $ret = $39;
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($41);
  $42 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($42);
  $43 = $ret;
  $0 = $43;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $44 = $1;
 $45 = $2;
 $46 = (_i_read_ANSI_NIST_version($44,$45,$version,$field)|0);
 $ret_delimiter = $46;
 $47 = $ret_delimiter;
 $48 = ($47|0)<(0);
 if ($48) {
  $49 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($49);
  $50 = $ret_delimiter;
  $0 = $50;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $51 = $ret_delimiter;
 $52 = ($51|0)!=(29);
 if ($52) {
  $53 = HEAP32[_stderr>>2]|0;
  $54 = $ret_delimiter;
  $55 = $1;
  $56 = $2;
  $57 = (_fbtell($55,$56)|0);
  HEAP32[$vararg_buffer6>>2] = $54;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = 29;
  $vararg_ptr10 = (($vararg_buffer6) + 8|0);
  HEAP32[$vararg_ptr10>>2] = $57;
  (_fprintf(($53|0),(15624|0),($vararg_buffer6|0))|0);
  $58 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($58);
  $0 = -4;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $59 = HEAP32[$record>>2]|0;
 $60 = HEAP32[$field>>2]|0;
 $61 = (_update_ANSI_NIST_record($59,$60)|0);
 $ret = $61;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($63);
  $64 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($64);
  $65 = $ret;
  $0 = $65;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $66 = HEAP32[$version>>2]|0;
 if ((($66|0) == 400) | (($66|0) == 300) | (($66|0) == 201) | (($66|0) == 200)) {
 } else {
  $67 = HEAP32[_stderr>>2]|0;
  $68 = HEAP32[$version>>2]|0;
  HEAP32[$vararg_buffer11>>2] = $68;
  (_fprintf(($67|0),(15728|0),($vararg_buffer11|0))|0);
  $69 = HEAP32[_stderr>>2]|0;
  (_fprintf(($69|0),(15800|0),($vararg_buffer14|0))|0);
 }
 $70 = $1;
 $71 = $2;
 $72 = HEAP32[$record>>2]|0;
 $73 = (_i_read_ANSI_NIST_remaining_fields($70,$71,$72)|0);
 $ret_delimiter = $73;
 $74 = $ret_delimiter;
 $75 = ($74|0)<(0);
 if ($75) {
  $76 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($76);
  $77 = $ret_delimiter;
  $0 = $77;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $78 = $ret_delimiter;
 $79 = ($78|0)!=(28);
 if ($79) {
  $80 = HEAP32[_stderr>>2]|0;
  $81 = $ret_delimiter;
  $82 = HEAP32[$record>>2]|0;
  $83 = (($82) + 8|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = HEAP32[$record>>2]|0;
  $86 = (($85) + 4|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = $1;
  $89 = $2;
  $90 = (_fbtell($88,$89)|0);
  HEAP32[$vararg_buffer16>>2] = $81;
  $vararg_ptr19 = (($vararg_buffer16) + 4|0);
  HEAP32[$vararg_ptr19>>2] = 28;
  $vararg_ptr20 = (($vararg_buffer16) + 8|0);
  HEAP32[$vararg_ptr20>>2] = $84;
  $vararg_ptr21 = (($vararg_buffer16) + 12|0);
  HEAP32[$vararg_ptr21>>2] = $87;
  $vararg_ptr22 = (($vararg_buffer16) + 16|0);
  HEAP32[$vararg_ptr22>>2] = $90;
  (_fprintf(($80|0),(15840|0),($vararg_buffer16|0))|0);
  $91 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($91);
  $0 = -6;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $92 = HEAP32[$record>>2]|0;
 $93 = (($92) + 4|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = HEAP32[$record>>2]|0;
 $96 = (($95) + 8|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = ($94|0)!=($97|0);
 if ($98) {
  $99 = HEAP32[_stderr>>2]|0;
  $100 = HEAP32[$record>>2]|0;
  $101 = (($100) + 8|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = HEAP32[$record>>2]|0;
  $104 = (($103) + 4|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = $1;
  $107 = $2;
  $108 = (_fbtell($106,$107)|0);
  HEAP32[$vararg_buffer23>>2] = $102;
  $vararg_ptr26 = (($vararg_buffer23) + 4|0);
  HEAP32[$vararg_ptr26>>2] = $105;
  $vararg_ptr27 = (($vararg_buffer23) + 8|0);
  HEAP32[$vararg_ptr27>>2] = $108;
  (_fprintf(($99|0),(15960|0),($vararg_buffer23|0))|0);
  $109 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($109);
  $0 = -7;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 $110 = HEAP32[$record>>2]|0;
 $111 = (($110) + 12|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)<(9);
 if ($113) {
  $114 = HEAP32[_stderr>>2]|0;
  $115 = HEAP32[$record>>2]|0;
  $116 = (($115) + 12|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = (9 - ($117))|0;
  $119 = $1;
  $120 = $2;
  $121 = (_fbtell($119,$120)|0);
  HEAP32[$vararg_buffer28>>2] = $118;
  $vararg_ptr31 = (($vararg_buffer28) + 4|0);
  HEAP32[$vararg_ptr31>>2] = $121;
  (_fprintf(($114|0),(16032|0),($vararg_buffer28|0))|0);
  $122 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($122);
  $0 = -8;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 } else {
  $123 = HEAP32[$record>>2]|0;
  $124 = $3;
  HEAP32[$124>>2] = $123;
  $125 = HEAP32[$version>>2]|0;
  $126 = $4;
  HEAP32[$126>>2] = $125;
  $0 = 0;
  $127 = $0;
  STACKTOP = sp;return ($127|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_remaining_records($fpin,$buf,$ansi_nist) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $cntfield = 0, $cntfield_i = 0, $i = 0, $num_records = 0, $record = 0, $record_type = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $cntfield = sp + 48|0;
 $cntfield_i = sp + 40|0;
 $record = sp + 32|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ansi_nist;
 $4 = $3;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)<(1);
 if ($7) {
  $8 = HEAP32[_stderr>>2]|0;
  (_fprintf(($8|0),(15088|0),($vararg_buffer|0))|0);
  $0 = -2;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $9 = $3;
 $10 = (($9) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(1);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = $3;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $20;
  (_fprintf(($15|0),(15152|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $21 = $3;
 $22 = (($21) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $25 = (_lookup_ANSI_NIST_field($cntfield,$cntfield_i,3,$24)|0);
 $26 = ($25|0)==(0);
 if ($26) {
  $27 = HEAP32[_stderr>>2]|0;
  $28 = $1;
  $29 = $2;
  $30 = (_fbtell($28,$29)|0);
  HEAP32[$vararg_buffer3>>2] = $30;
  (_fprintf(($27|0),(15232|0),($vararg_buffer3|0))|0);
  $0 = -4;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $31 = HEAP32[$cntfield>>2]|0;
 $32 = (($31) + 16|0);
 $33 = HEAP32[$32>>2]|0;
 $num_records = $33;
 $34 = $3;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $i = $36;
 while(1) {
  $37 = $i;
  $38 = $num_records;
  $39 = ($37|0)<($38|0);
  if (!($39)) {
   label = 17;
   break;
  }
  $40 = $i;
  $41 = HEAP32[$cntfield>>2]|0;
  $42 = (($41) + 24|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 4|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = ($47|0)!=(2);
  if ($48) {
   label = 10;
   break;
  }
  $59 = $i;
  $60 = HEAP32[$cntfield>>2]|0;
  $61 = (($60) + 24|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) + ($59<<2)|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($64) + 12|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + 12|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (_atoi($69)|0);
  $record_type = $70;
  $71 = $1;
  $72 = $2;
  $73 = $record_type;
  $74 = (_i_read_ANSI_NIST_record($71,$72,$record,$73)|0);
  $ret = $74;
  $75 = ($74|0)!=(0);
  if ($75) {
   label = 12;
   break;
  }
  $77 = $3;
  $78 = HEAP32[$record>>2]|0;
  $79 = (_update_ANSI_NIST($77,$78)|0);
  $ret = $79;
  $80 = ($79|0)!=(0);
  if ($80) {
   label = 14;
   break;
  }
  $83 = $i;
  $84 = (($83) + 1)|0;
  $i = $84;
 }
 if ((label|0) == 10) {
  $49 = HEAP32[_stderr>>2]|0;
  $50 = $i;
  $51 = $i;
  $52 = HEAP32[$cntfield>>2]|0;
  $53 = (($52) + 24|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + 4|0);
  $58 = HEAP32[$57>>2]|0;
  HEAP32[$vararg_buffer6>>2] = $50;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $58;
  (_fprintf(($49|0),(15320|0),($vararg_buffer6|0))|0);
  $0 = -5;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 else if ((label|0) == 12) {
  $76 = $ret;
  $0 = $76;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 else if ((label|0) == 14) {
  $81 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($81);
  $82 = $ret;
  $0 = $82;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 else if ((label|0) == 17) {
  $0 = 0;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_record($fpin,$buf,$orecord,$record_type) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $orecord = $orecord|0;
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $record = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $record = sp + 12|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $orecord;
 $4 = $record_type;
 $5 = $4;
 $6 = (_tagged_record($5)|0);
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = $1;
   $9 = $2;
   $10 = $4;
   $11 = (_i_read_ANSI_NIST_tagged_record($8,$9,$record,$10)|0);
   $ret = $11;
   $12 = ($11|0)!=(0);
   if (!($12)) {
    break;
   }
   $13 = $ret;
   $0 = $13;
   $39 = $0;
   STACKTOP = sp;return ($39|0);
  } else {
   $14 = $4;
   $15 = (_binary_image_record($14)|0);
   $16 = ($15|0)!=(0);
   do {
    if ($16) {
     $17 = $1;
     $18 = $2;
     $19 = $4;
     $20 = (_i_read_ANSI_NIST_binary_image_record($17,$18,$record,$19)|0);
     $ret = $20;
     $21 = ($20|0)!=(0);
     if (!($21)) {
      break;
     }
     $22 = $ret;
     $0 = $22;
     $39 = $0;
     STACKTOP = sp;return ($39|0);
    } else {
     $23 = $4;
     $24 = (_binary_signature_record($23)|0);
     $25 = ($24|0)!=(0);
     if (!($25)) {
      $32 = HEAP32[_stderr>>2]|0;
      $33 = $4;
      $34 = $1;
      $35 = $2;
      $36 = (_fbtell($34,$35)|0);
      HEAP32[$vararg_buffer>>2] = $33;
      $vararg_ptr1 = (($vararg_buffer) + 4|0);
      HEAP32[$vararg_ptr1>>2] = $36;
      (_fprintf(($32|0),(15016|0),($vararg_buffer|0))|0);
      $0 = -2;
      $39 = $0;
      STACKTOP = sp;return ($39|0);
     }
     $26 = $1;
     $27 = $2;
     $28 = $4;
     $29 = (_i_read_ANSI_NIST_binary_signature_record($26,$27,$record,$28)|0);
     $ret = $29;
     $30 = ($29|0)!=(0);
     if ($30) {
      $31 = $ret;
      $0 = $31;
      $39 = $0;
      STACKTOP = sp;return ($39|0);
     } else {
      break;
     }
    }
   } while(0);
  }
 } while(0);
 $37 = HEAP32[$record>>2]|0;
 $38 = $3;
 HEAP32[$38>>2] = $37;
 $0 = 0;
 $39 = $0;
 STACKTOP = sp;return ($39|0);
}
function _i_read_ANSI_NIST_tagged_record($fpin,$buf,$orecord,$record_type) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $orecord = $orecord|0;
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $field = 0, $record = 0, $record_bytes = 0, $ret = 0, $ret_delimiter = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $field = sp + 72|0;
 $record = sp + 68|0;
 $record_bytes = sp + 56|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $orecord;
 $4 = $record_type;
 $5 = (_alloc_ANSI_NIST_record($record)|0);
 $ret = $5;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $102 = $0;
  STACKTOP = sp;return ($102|0);
 }
 $8 = $1;
 $9 = $2;
 $10 = (_i_read_ANSI_NIST_record_length($8,$9,$record_bytes,$field)|0);
 $ret_delimiter = $10;
 $11 = $ret_delimiter;
 $12 = ($11|0)<(0);
 if ($12) {
  $13 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($13);
  $14 = $ret_delimiter;
  $0 = $14;
  $102 = $0;
  STACKTOP = sp;return ($102|0);
 }
 $15 = $4;
 $16 = ($15|0)!=(-1);
 if ($16) {
  $17 = HEAP32[$field>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $4;
  $21 = ($19|0)!=($20|0);
  if ($21) {
   $22 = HEAP32[_stderr>>2]|0;
   $23 = HEAP32[$field>>2]|0;
   $24 = (($23) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $4;
   $27 = $1;
   $28 = $2;
   $29 = (_fbtell($27,$28)|0);
   HEAP32[$vararg_buffer>>2] = $25;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $26;
   $vararg_ptr2 = (($vararg_buffer) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $29;
   (_fprintf(($22|0),(14680|0),($vararg_buffer|0))|0);
   $30 = HEAP32[$record>>2]|0;
   _free_ANSI_NIST_record($30);
   $31 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($31);
   $0 = -2;
   $102 = $0;
   STACKTOP = sp;return ($102|0);
  }
 }
 $32 = $4;
 $33 = HEAP32[$record>>2]|0;
 HEAP32[$33>>2] = $32;
 $34 = HEAP32[$record_bytes>>2]|0;
 $35 = HEAP32[$record>>2]|0;
 $36 = (($35) + 4|0);
 HEAP32[$36>>2] = $34;
 $37 = $ret_delimiter;
 $38 = ($37|0)==(28);
 if ($38) {
  $39 = HEAP32[$record>>2]|0;
  $40 = (($39) + 24|0);
  HEAP32[$40>>2] = 1;
  $41 = HEAP32[$record>>2]|0;
  $42 = (($41) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$42>>2] = $44;
 }
 $45 = HEAP32[$record>>2]|0;
 $46 = HEAP32[$field>>2]|0;
 $47 = (_update_ANSI_NIST_record($45,$46)|0);
 $ret = $47;
 $48 = ($47|0)!=(0);
 if ($48) {
  $49 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($49);
  $50 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($50);
  $51 = $ret;
  $0 = $51;
  $102 = $0;
  STACKTOP = sp;return ($102|0);
 }
 $52 = $ret_delimiter;
 $53 = ($52|0)==(29);
 do {
  if ($53) {
   $54 = $1;
   $55 = $2;
   $56 = HEAP32[$record>>2]|0;
   $57 = (_i_read_ANSI_NIST_remaining_fields($54,$55,$56)|0);
   $ret_delimiter = $57;
   $58 = $ret_delimiter;
   $59 = ($58|0)<(0);
   if (!($59)) {
    break;
   }
   $60 = HEAP32[$record>>2]|0;
   _free_ANSI_NIST_record($60);
   $61 = $ret_delimiter;
   $0 = $61;
   $102 = $0;
   STACKTOP = sp;return ($102|0);
  }
 } while(0);
 $62 = $ret_delimiter;
 $63 = ($62|0)!=(28);
 if ($63) {
  $64 = HEAP32[_stderr>>2]|0;
  $65 = $ret_delimiter;
  $66 = HEAP32[$record>>2]|0;
  $67 = (($66) + 8|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$record>>2]|0;
  $70 = (($69) + 4|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = $1;
  $73 = $2;
  $74 = (_fbtell($72,$73)|0);
  HEAP32[$vararg_buffer3>>2] = $65;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 28;
  $vararg_ptr7 = (($vararg_buffer3) + 8|0);
  HEAP32[$vararg_ptr7>>2] = $68;
  $vararg_ptr8 = (($vararg_buffer3) + 12|0);
  HEAP32[$vararg_ptr8>>2] = $71;
  $vararg_ptr9 = (($vararg_buffer3) + 16|0);
  HEAP32[$vararg_ptr9>>2] = $74;
  (_fprintf(($64|0),(14776|0),($vararg_buffer3|0))|0);
  $75 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($75);
  $0 = -3;
  $102 = $0;
  STACKTOP = sp;return ($102|0);
 }
 $76 = HEAP32[$record>>2]|0;
 $77 = (($76) + 4|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = ($78|0)!=(-1);
 if ($79) {
  $80 = HEAP32[$record>>2]|0;
  $81 = (($80) + 4|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = HEAP32[$record>>2]|0;
  $84 = (($83) + 8|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($82|0)!=($85|0);
  if ($86) {
   $87 = HEAP32[_stderr>>2]|0;
   $88 = HEAP32[$record>>2]|0;
   $89 = (($88) + 8|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$record>>2]|0;
   $92 = (($91) + 4|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = HEAP32[$record>>2]|0;
   $95 = HEAP32[$94>>2]|0;
   $96 = $1;
   $97 = $2;
   $98 = (_fbtell($96,$97)|0);
   HEAP32[$vararg_buffer10>>2] = $90;
   $vararg_ptr13 = (($vararg_buffer10) + 4|0);
   HEAP32[$vararg_ptr13>>2] = $93;
   $vararg_ptr14 = (($vararg_buffer10) + 8|0);
   HEAP32[$vararg_ptr14>>2] = $95;
   $vararg_ptr15 = (($vararg_buffer10) + 12|0);
   HEAP32[$vararg_ptr15>>2] = $98;
   (_fprintf(($87|0),(14912|0),($vararg_buffer10|0))|0);
   $99 = HEAP32[$record>>2]|0;
   _free_ANSI_NIST_record($99);
   $0 = -4;
   $102 = $0;
   STACKTOP = sp;return ($102|0);
  }
 }
 $100 = HEAP32[$record>>2]|0;
 $101 = $3;
 HEAP32[$101>>2] = $100;
 $0 = 0;
 $102 = $0;
 STACKTOP = sp;return ($102|0);
}
function _i_read_ANSI_NIST_record_length($fpin,$buf,$orecord_bytes,$ofield) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $orecord_bytes = $orecord_bytes|0;
 $ofield = $ofield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $record_bytes = 0, $ret_delimiter = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $record_bytes = sp + 24|0;
 $field = sp + 20|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $orecord_bytes;
 $4 = $ofield;
 $5 = $1;
 $6 = $2;
 $7 = (_i_read_ANSI_NIST_integer_field($5,$6,$record_bytes,$field)|0);
 $ret_delimiter = $7;
 $8 = $ret_delimiter;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(14520|0),($vararg_buffer|0))|0);
  $11 = $ret_delimiter;
  $0 = $11;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 }
 $12 = HEAP32[$field>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(1);
 if ($15) {
  $16 = HEAP32[_stderr>>2]|0;
  $17 = HEAP32[$field>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = $2;
  $22 = (_fbtell($20,$21)|0);
  HEAP32[$vararg_buffer1>>2] = $19;
  $vararg_ptr3 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr3>>2] = 1;
  $vararg_ptr4 = (($vararg_buffer1) + 8|0);
  HEAP32[$vararg_ptr4>>2] = $22;
  (_fprintf(($16|0),(14592|0),($vararg_buffer1|0))|0);
  $23 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($23);
  $0 = -2;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 } else {
  $24 = HEAP32[$record_bytes>>2]|0;
  $25 = $3;
  HEAP32[$25>>2] = $24;
  $26 = HEAP32[$field>>2]|0;
  $27 = $4;
  HEAP32[$27>>2] = $26;
  $28 = $ret_delimiter;
  $0 = $28;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_version($fpin,$buf,$oversion,$ofield) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $oversion = $oversion|0;
 $ofield = $ofield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $ret_delimiter = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $version = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $version = sp + 20|0;
 $field = sp + 16|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $oversion;
 $4 = $ofield;
 $5 = $1;
 $6 = $2;
 $7 = (_i_read_ANSI_NIST_integer_field($5,$6,$version,$field)|0);
 $ret_delimiter = $7;
 $8 = $ret_delimiter;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 2;
  (_fprintf(($10|0),(14384|0),($vararg_buffer|0))|0);
  $11 = $ret_delimiter;
  $0 = $11;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 }
 $12 = HEAP32[$field>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(2);
 if ($15) {
  $16 = HEAP32[_stderr>>2]|0;
  $17 = HEAP32[$field>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $19;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 2;
  (_fprintf(($16|0),(14464|0),($vararg_buffer1|0))|0);
  $20 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($20);
  $0 = -2;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 } else {
  $21 = HEAP32[$version>>2]|0;
  $22 = $3;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$field>>2]|0;
  $24 = $4;
  HEAP32[$24>>2] = $23;
  $25 = $ret_delimiter;
  $0 = $25;
  $26 = $0;
  STACKTOP = sp;return ($26|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_integer_field($fpin,$buf,$ofield_value,$ofield) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ofield_value = $ofield_value|0;
 $ofield = $ofield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $ret_delimiter = 0, $vararg_buffer = 0, $vararg_buffer2 = 0;
 var $vararg_ptr1 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $field = sp + 16|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ofield_value;
 $4 = $ofield;
 $5 = $1;
 $6 = $2;
 $7 = (_i_read_ANSI_NIST_field($5,$6,$field,-1)|0);
 $ret_delimiter = $7;
 $8 = $ret_delimiter;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = $ret_delimiter;
  $0 = $10;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $11 = HEAP32[$field>>2]|0;
 $12 = (($11) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(1);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = HEAP32[$field>>2]|0;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $1;
  $20 = $2;
  $21 = (_fbtell($19,$20)|0);
  HEAP32[$vararg_buffer>>2] = $18;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $21;
  (_fprintf(($15|0),(14176|0),($vararg_buffer|0))|0);
  $22 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($22);
  $0 = -1;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $23 = HEAP32[$field>>2]|0;
 $24 = (($23) + 24|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(1);
 if ($29) {
  $30 = HEAP32[_stderr>>2]|0;
  $31 = HEAP32[$field>>2]|0;
  $32 = (($31) + 24|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $1;
  $38 = $2;
  $39 = (_fbtell($37,$38)|0);
  HEAP32[$vararg_buffer2>>2] = $36;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $39;
  (_fprintf(($30|0),(14280|0),($vararg_buffer2|0))|0);
  $40 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($40);
  $0 = -2;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 } else {
  $41 = HEAP32[$field>>2]|0;
  $42 = (($41) + 24|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + 12|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_atoi($49)|0);
  $51 = $3;
  HEAP32[$51>>2] = $50;
  $52 = HEAP32[$field>>2]|0;
  $53 = $4;
  HEAP32[$53>>2] = $52;
  $54 = $ret_delimiter;
  $0 = $54;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_remaining_fields($fpin,$buf,$record) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $ret = 0, $ret_delimiter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $field = sp + 8|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $record;
 while(1) {
  $4 = $1;
  $5 = $2;
  $6 = $3;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $3;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($8) - ($11))|0;
  $13 = (_i_read_ANSI_NIST_field($4,$5,$field,$12)|0);
  $ret_delimiter = $13;
  $14 = $ret_delimiter;
  $15 = ($14|0)<(0);
  if ($15) {
   label = 3;
   break;
  }
  $17 = $3;
  $18 = HEAP32[$field>>2]|0;
  $19 = (_update_ANSI_NIST_record($17,$18)|0);
  $ret = $19;
  $20 = ($19|0)!=(0);
  if ($20) {
   label = 5;
   break;
  }
  $23 = $ret_delimiter;
  $24 = ($23|0)!=(29);
  if ($24) {
   label = 7;
   break;
  }
 }
 if ((label|0) == 3) {
  $16 = $ret_delimiter;
  $0 = $16;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 else if ((label|0) == 5) {
  $21 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($21);
  $22 = $ret;
  $0 = $22;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 else if ((label|0) == 7) {
  $25 = $ret_delimiter;
  $26 = ($25|0)==(28);
  if ($26) {
   $27 = $3;
   $28 = (($27) + 24|0);
   HEAP32[$28>>2] = 1;
   $29 = $3;
   $30 = (($29) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + 1)|0;
   HEAP32[$30>>2] = $32;
  }
  $33 = $ret_delimiter;
  $0 = $33;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_field($fpin,$buf,$ofield,$record_bytes) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ofield = $ofield|0;
 $record_bytes = $record_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $field_id = 0, $field_int = 0, $record_type = 0, $ret = 0, $ret_delimiter = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $field_id = sp + 20|0;
 $field = sp + 16|0;
 $record_type = sp + 12|0;
 $field_int = sp + 8|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ofield;
 $4 = $record_bytes;
 $5 = $4;
 $6 = ($5|0)<(-1);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $4;
  $9 = $1;
  $10 = $2;
  $11 = (_fbtell($9,$10)|0);
  HEAP32[$vararg_buffer>>2] = $8;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $11;
  (_fprintf(($7|0),(14096|0),($vararg_buffer|0))|0);
  $0 = -1;
  $46 = $0;
  STACKTOP = sp;return ($46|0);
 }
 $12 = $1;
 $13 = $2;
 $14 = (_i_read_ANSI_NIST_field_ID($12,$13,$field_id,$record_type,$field_int)|0);
 $ret = $14;
 $15 = $ret;
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = $ret;
  $0 = $17;
  $46 = $0;
  STACKTOP = sp;return ($46|0);
 }
 $18 = HEAP32[$record_type>>2]|0;
 $19 = (_tagged_image_record($18)|0);
 $20 = ($19|0)!=(0);
 do {
  if ($20) {
   $21 = HEAP32[$field_int>>2]|0;
   $22 = ($21|0)==(999);
   if ($22) {
    $23 = $1;
    $24 = $2;
    $25 = HEAP32[$field_id>>2]|0;
    $26 = HEAP32[$record_type>>2]|0;
    $27 = HEAP32[$field_int>>2]|0;
    $28 = $4;
    $29 = (_i_read_ANSI_NIST_image_field($23,$24,$field,$25,$26,$27,$28)|0);
    $ret_delimiter = $29;
    $30 = $ret_delimiter;
    $31 = ($30|0)<(0);
    if (!($31)) {
     break;
    }
    $32 = $ret_delimiter;
    $0 = $32;
    $46 = $0;
    STACKTOP = sp;return ($46|0);
   } else {
    label = 10;
   }
  } else {
   label = 10;
  }
 } while(0);
 do {
  if ((label|0) == 10) {
   $33 = $1;
   $34 = $2;
   $35 = HEAP32[$field_id>>2]|0;
   $36 = HEAP32[$record_type>>2]|0;
   $37 = HEAP32[$field_int>>2]|0;
   $38 = $4;
   $39 = (_i_read_ANSI_NIST_tagged_field($33,$34,$field,$35,$36,$37,$38)|0);
   $ret_delimiter = $39;
   $40 = $ret_delimiter;
   $41 = ($40|0)<(0);
   if (!($41)) {
    break;
   }
   $42 = $ret_delimiter;
   $0 = $42;
   $46 = $0;
   STACKTOP = sp;return ($46|0);
  }
 } while(0);
 $43 = HEAP32[$field>>2]|0;
 $44 = $3;
 HEAP32[$44>>2] = $43;
 $45 = $ret_delimiter;
 $0 = $45;
 $46 = $0;
 STACKTOP = sp;return ($46|0);
}
function _i_read_ANSI_NIST_image_field($fpin,$buf,$ofield,$field_id,$record_type,$field_int,$record_bytes) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ofield = $ofield|0;
 $field_id = $field_id|0;
 $record_type = $record_type|0;
 $field_int = $field_int|0;
 $record_bytes = $record_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $delimiter = 0, $errmsg = 0, $field = 0, $image_item = 0, $image_size = 0, $new_ptr = 0, $nread = 0, $ret = 0, $subfield = 0, $vararg_buffer = 0, $vararg_buffer14 = 0, $vararg_buffer2 = 0, $vararg_buffer21 = 0, $vararg_buffer26 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0;
 var $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr24 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer26 = sp + 80|0;
 $vararg_buffer21 = sp + 64|0;
 $vararg_buffer14 = sp + 40|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $field = sp + 120|0;
 $subfield = sp + 116|0;
 $image_item = sp + 112|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ofield;
 $4 = $field_id;
 $5 = $record_type;
 $6 = $field_int;
 $7 = $record_bytes;
 $8 = $7;
 $9 = ($8|0)==(-1);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $5;
  $12 = $1;
  $13 = $2;
  $14 = (_fbtell($12,$13)|0);
  HEAP32[$vararg_buffer>>2] = $11;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $14;
  (_fprintf(($10|0),(13424|0),($vararg_buffer|0))|0);
  $0 = -2;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $15 = $7;
 $16 = $4;
 $17 = (_strlen(($16|0))|0);
 $18 = (($15) - ($17))|0;
 $19 = (($18) - 1)|0;
 $image_size = $19;
 $20 = $image_size;
 $21 = ($20|0)<(0);
 if ($21) {
  $22 = HEAP32[_stderr>>2]|0;
  $23 = $7;
  $24 = $5;
  $25 = $image_size;
  $26 = $1;
  $27 = $2;
  $28 = (_fbtell($26,$27)|0);
  HEAP32[$vararg_buffer2>>2] = $23;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $24;
  $vararg_ptr6 = (($vararg_buffer2) + 8|0);
  HEAP32[$vararg_ptr6>>2] = $25;
  $vararg_ptr7 = (($vararg_buffer2) + 12|0);
  HEAP32[$vararg_ptr7>>2] = $28;
  (_fprintf(($22|0),(13528|0),($vararg_buffer2|0))|0);
  $0 = -21;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $29 = (_alloc_ANSI_NIST_item($image_item)|0);
 $ret = $29;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = $ret;
  $0 = $31;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $32 = HEAP32[$image_item>>2]|0;
 $33 = (($32) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $image_size;
 $36 = ($34|0)<($35|0);
 do {
  if ($36) {
   $37 = HEAP32[$image_item>>2]|0;
   $38 = (($37) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $image_size;
   $41 = (_realloc($39,$40)|0);
   $new_ptr = $41;
   $42 = $new_ptr;
   $43 = ($42|0)==(0|0);
   if (!($43)) {
    $54 = $new_ptr;
    $55 = HEAP32[$image_item>>2]|0;
    $56 = (($55) + 12|0);
    HEAP32[$56>>2] = $54;
    $57 = $image_size;
    $58 = HEAP32[$image_item>>2]|0;
    $59 = (($58) + 8|0);
    HEAP32[$59>>2] = $57;
    break;
   }
   $44 = HEAP32[_stderr>>2]|0;
   $45 = HEAP32[$image_item>>2]|0;
   $46 = (($45) + 8|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $image_size;
   $49 = $5;
   $50 = $1;
   $51 = $2;
   $52 = (_fbtell($50,$51)|0);
   HEAP32[$vararg_buffer8>>2] = $47;
   $vararg_ptr11 = (($vararg_buffer8) + 4|0);
   HEAP32[$vararg_ptr11>>2] = $48;
   $vararg_ptr12 = (($vararg_buffer8) + 8|0);
   HEAP32[$vararg_ptr12>>2] = $49;
   $vararg_ptr13 = (($vararg_buffer8) + 12|0);
   HEAP32[$vararg_ptr13>>2] = $52;
   (_fprintf(($44|0),(13640|0),($vararg_buffer8|0))|0);
   $53 = HEAP32[$image_item>>2]|0;
   _free_ANSI_NIST_item($53);
   $0 = -3;
   $161 = $0;
   STACKTOP = sp;return ($161|0);
  }
 } while(0);
 $60 = HEAP32[$image_item>>2]|0;
 $61 = (($60) + 12|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $image_size;
 $64 = $1;
 $65 = $2;
 $66 = (_fbread($62,1,$63,$64,$65)|0);
 $nread = $66;
 $67 = $nread;
 $68 = $image_size;
 $69 = ($67|0)!=($68|0);
 if ($69) {
  $70 = $1;
  $71 = ($70|0)!=(0|0);
  if ($71) {
   $72 = $1;
   $73 = (_ferror(($72|0))|0);
   $74 = ($73|0)!=(0);
   if ($74) {
    $75 = (___errno_location()|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = (_strerror(($76|0))|0);
    $162 = $77;
   } else {
    $162 = 11416;
   }
   $78 = $162;
  } else {
   $78 = 11432;
  }
  $errmsg = $78;
  $79 = HEAP32[_stderr>>2]|0;
  $80 = $nread;
  $81 = $image_size;
  $82 = $5;
  $83 = $errmsg;
  $84 = $1;
  $85 = $2;
  $86 = (_fbtell($84,$85)|0);
  HEAP32[$vararg_buffer14>>2] = $80;
  $vararg_ptr17 = (($vararg_buffer14) + 4|0);
  HEAP32[$vararg_ptr17>>2] = $81;
  $vararg_ptr18 = (($vararg_buffer14) + 8|0);
  HEAP32[$vararg_ptr18>>2] = $82;
  $vararg_ptr19 = (($vararg_buffer14) + 12|0);
  HEAP32[$vararg_ptr19>>2] = $83;
  $vararg_ptr20 = (($vararg_buffer14) + 16|0);
  HEAP32[$vararg_ptr20>>2] = $86;
  (_fprintf(($79|0),(13760|0),($vararg_buffer14|0))|0);
  $87 = HEAP32[$image_item>>2]|0;
  _free_ANSI_NIST_item($87);
  $0 = -4;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $88 = $image_size;
 $89 = HEAP32[$image_item>>2]|0;
 HEAP32[$89>>2] = $88;
 $90 = $image_size;
 $91 = HEAP32[$image_item>>2]|0;
 $92 = (($91) + 4|0);
 HEAP32[$92>>2] = $90;
 $93 = $1;
 $94 = $2;
 $95 = (_fbgetc($93,$94)|0);
 $delimiter = $95;
 $96 = ($95|0)==(-1);
 if ($96) {
  $97 = $1;
  $98 = ($97|0)!=(0|0);
  if ($98) {
   $99 = $1;
   $100 = (_ferror(($99|0))|0);
   $101 = ($100|0)!=(0);
   if ($101) {
    $102 = (___errno_location()|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (_strerror(($103|0))|0);
    $163 = $104;
   } else {
    $163 = 11416;
   }
   $105 = $163;
  } else {
   $105 = 11432;
  }
  $errmsg = $105;
  $106 = HEAP32[_stderr>>2]|0;
  $107 = $5;
  $108 = $errmsg;
  $109 = $1;
  $110 = $2;
  $111 = (_fbtell($109,$110)|0);
  HEAP32[$vararg_buffer21>>2] = $107;
  $vararg_ptr24 = (($vararg_buffer21) + 4|0);
  HEAP32[$vararg_ptr24>>2] = $108;
  $vararg_ptr25 = (($vararg_buffer21) + 8|0);
  HEAP32[$vararg_ptr25>>2] = $111;
  (_fprintf(($106|0),(13880|0),($vararg_buffer21|0))|0);
  $112 = HEAP32[$image_item>>2]|0;
  _free_ANSI_NIST_item($112);
  $0 = -5;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $113 = $delimiter;
 $114 = ($113|0)!=(28);
 if ($114) {
  $115 = HEAP32[_stderr>>2]|0;
  $116 = $delimiter;
  $117 = $5;
  $118 = $1;
  $119 = $2;
  $120 = (_fbtell($118,$119)|0);
  HEAP32[$vararg_buffer26>>2] = $116;
  $vararg_ptr29 = (($vararg_buffer26) + 4|0);
  HEAP32[$vararg_ptr29>>2] = 28;
  $vararg_ptr30 = (($vararg_buffer26) + 8|0);
  HEAP32[$vararg_ptr30>>2] = $117;
  $vararg_ptr31 = (($vararg_buffer26) + 12|0);
  HEAP32[$vararg_ptr31>>2] = $120;
  (_fprintf(($115|0),(13968|0),($vararg_buffer26|0))|0);
  $121 = HEAP32[$image_item>>2]|0;
  _free_ANSI_NIST_item($121);
  $0 = -6;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $122 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $122;
 $123 = ($122|0)!=(0);
 if ($123) {
  $124 = HEAP32[$image_item>>2]|0;
  _free_ANSI_NIST_item($124);
  $125 = $ret;
  $0 = $125;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $126 = HEAP32[$subfield>>2]|0;
 $127 = HEAP32[$image_item>>2]|0;
 $128 = (_update_ANSI_NIST_subfield($126,$127)|0);
 $ret = $128;
 $129 = ($128|0)!=(0);
 if ($129) {
  $130 = HEAP32[$image_item>>2]|0;
  _free_ANSI_NIST_item($130);
  $131 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($131);
  $132 = $ret;
  $0 = $132;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $133 = (_alloc_ANSI_NIST_field($field)|0);
 $ret = $133;
 $134 = ($133|0)!=(0);
 if ($134) {
  $135 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($135);
  $136 = $ret;
  $0 = $136;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $137 = $4;
 $138 = HEAP32[$field>>2]|0;
 HEAP32[$138>>2] = $137;
 $139 = $5;
 $140 = HEAP32[$field>>2]|0;
 $141 = (($140) + 4|0);
 HEAP32[$141>>2] = $139;
 $142 = $6;
 $143 = HEAP32[$field>>2]|0;
 $144 = (($143) + 8|0);
 HEAP32[$144>>2] = $142;
 $145 = $4;
 $146 = (_strlen(($145|0))|0);
 $147 = HEAP32[$field>>2]|0;
 $148 = (($147) + 12|0);
 $149 = HEAP32[$148>>2]|0;
 $150 = (($149) + ($146))|0;
 HEAP32[$148>>2] = $150;
 $151 = HEAP32[$field>>2]|0;
 $152 = HEAP32[$subfield>>2]|0;
 $153 = (_update_ANSI_NIST_field($151,$152)|0);
 $ret = $153;
 $154 = ($153|0)!=(0);
 if ($154) {
  $155 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($155);
  $156 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($156);
  $157 = $ret;
  $0 = $157;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 } else {
  $158 = HEAP32[$field>>2]|0;
  $159 = $3;
  HEAP32[$159>>2] = $158;
  $160 = $delimiter;
  $0 = $160;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_tagged_field($fpin,$buf,$ofield,$field_id,$record_type,$field_int,$record_bytes) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ofield = $ofield|0;
 $field_id = $field_id|0;
 $record_type = $record_type|0;
 $field_int = $field_int|0;
 $record_bytes = $record_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $ret = 0, $ret_delimiter = 0, $subfield = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $field = sp + 4|0;
 $subfield = sp;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ofield;
 $4 = $field_id;
 $5 = $record_type;
 $6 = $field_int;
 $7 = $record_bytes;
 $8 = (_alloc_ANSI_NIST_field($field)|0);
 $ret = $8;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = $ret;
  $0 = $10;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 $11 = $4;
 $12 = HEAP32[$field>>2]|0;
 HEAP32[$12>>2] = $11;
 $13 = $5;
 $14 = HEAP32[$field>>2]|0;
 $15 = (($14) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = $6;
 $17 = HEAP32[$field>>2]|0;
 $18 = (($17) + 8|0);
 HEAP32[$18>>2] = $16;
 $19 = $4;
 $20 = (_strlen(($19|0))|0);
 $21 = HEAP32[$field>>2]|0;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20))|0;
 HEAP32[$22>>2] = $24;
 while(1) {
  $25 = $1;
  $26 = $2;
  $27 = (_i_read_ANSI_NIST_subfield($25,$26,$subfield)|0);
  $ret_delimiter = $27;
  $28 = $ret_delimiter;
  $29 = ($28|0)<(0);
  if ($29) {
   label = 5;
   break;
  }
  $32 = HEAP32[$field>>2]|0;
  $33 = HEAP32[$subfield>>2]|0;
  $34 = (_update_ANSI_NIST_field($32,$33)|0);
  $ret = $34;
  $35 = ($34|0)!=(0);
  if ($35) {
   label = 7;
   break;
  }
  $39 = $ret_delimiter;
  $40 = ($39|0)!=(30);
  if ($40) {
   label = 9;
   break;
  }
 }
 if ((label|0) == 5) {
  $30 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($30);
  $31 = $ret_delimiter;
  $0 = $31;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 else if ((label|0) == 7) {
  $36 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($36);
  $37 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($37);
  $38 = $ret;
  $0 = $38;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 else if ((label|0) == 9) {
  $41 = $ret_delimiter;
  $42 = ($41|0)==(29);
  if ($42) {
   $43 = HEAP32[$field>>2]|0;
   $44 = (($43) + 28|0);
   HEAP32[$44>>2] = 1;
   $45 = HEAP32[$field>>2]|0;
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + 1)|0;
   HEAP32[$46>>2] = $48;
  }
  $49 = HEAP32[$field>>2]|0;
  $50 = $3;
  HEAP32[$50>>2] = $49;
  $51 = $ret_delimiter;
  $0 = $51;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_field_ID($fpin,$buf,$ofield_id,$orecord_type,$ofield_int) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ofield_id = $ofield_id|0;
 $orecord_type = $orecord_type|0;
 $ofield_int = $ofield_int|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $errmsg = 0, $field_id = 0, $field_id_bytes = 0;
 var $field_int = 0, $fptr = 0, $i = 0, $iptr = 0, $nextchar = 0, $record_type = 0, $rptr = 0, $sep_found = 0, $vararg_buffer = 0, $vararg_buffer12 = 0, $vararg_buffer15 = 0, $vararg_buffer19 = 0, $vararg_buffer24 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr18 = 0, $vararg_ptr2 = 0;
 var $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer24 = sp + 72|0;
 $vararg_buffer19 = sp + 56|0;
 $vararg_buffer15 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ofield_id;
 $4 = $orecord_type;
 $5 = $ofield_int;
 $record_type = 0;
 $field_int = 0;
 $field_id_bytes = 21;
 $6 = $field_id_bytes;
 $7 = (_calloc($6,1)|0);
 $field_id = $7;
 $8 = $field_id;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = (___errno_location()|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_strerror(($11|0))|0);
  $errmsg = $12;
  $13 = HEAP32[_stderr>>2]|0;
  $14 = $field_id_bytes;
  $15 = $errmsg;
  $16 = $1;
  $17 = $2;
  $18 = (_fbtell($16,$17)|0);
  HEAP32[$vararg_buffer>>2] = $14;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $15;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $18;
  (_fprintf(($13|0),(12848|0),($vararg_buffer|0))|0);
  $0 = -2;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 $i = 0;
 $19 = $field_id;
 $iptr = $19;
 $20 = $field_id;
 $rptr = $20;
 $sep_found = 0;
 while(1) {
  $21 = $i;
  $22 = ($21|0)<(10);
  if (!($22)) {
   break;
  }
  $23 = $1;
  $24 = $2;
  $25 = (_fbgetc($23,$24)|0);
  $nextchar = $25;
  $26 = ($25|0)==(-1);
  if ($26) {
   label = 6;
   break;
  }
  $42 = $i;
  $43 = (($42) + 1)|0;
  $i = $43;
  $44 = $nextchar;
  $45 = ($44|0)==(46);
  if ($45) {
   label = 14;
   break;
  }
  $52 = $nextchar;
  $53 = ($52|0)>=(48);
  if (!($53)) {
   label = 18;
   break;
  }
  $54 = $nextchar;
  $55 = ($54|0)<=(57);
  if (!($55)) {
   label = 18;
   break;
  }
  $56 = $nextchar;
  $57 = $56&255;
  $58 = $iptr;
  $59 = (($58) + 1|0);
  $iptr = $59;
  HEAP8[$58>>0] = $57;
 }
 if ((label|0) == 6) {
  $27 = $1;
  $28 = ($27|0)!=(0|0);
  if ($28) {
   $29 = $1;
   $30 = (_ferror(($29|0))|0);
   $31 = ($30|0)!=(0);
   if ($31) {
    $32 = (___errno_location()|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (_strerror(($33|0))|0);
    $135 = $34;
   } else {
    $135 = 11416;
   }
   $35 = $135;
  } else {
   $35 = 11432;
  }
  $errmsg = $35;
  $36 = HEAP32[_stderr>>2]|0;
  $37 = $errmsg;
  $38 = $1;
  $39 = $2;
  $40 = (_fbtell($38,$39)|0);
  HEAP32[$vararg_buffer3>>2] = $37;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $40;
  (_fprintf(($36|0),(12928|0),($vararg_buffer3|0))|0);
  $41 = $field_id;
  _free($41);
  $0 = -3;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 14) {
  $46 = $rptr;
  $47 = (_atoi($46)|0);
  $record_type = $47;
  $48 = $nextchar;
  $49 = $48&255;
  $50 = $iptr;
  $51 = (($50) + 1|0);
  $iptr = $51;
  HEAP8[$50>>0] = $49;
  $sep_found = 1;
 }
 else if ((label|0) == 18) {
  $60 = HEAP32[_stderr>>2]|0;
  $61 = $nextchar;
  $62 = $nextchar;
  $63 = $1;
  $64 = $2;
  $65 = (_fbtell($63,$64)|0);
  HEAP32[$vararg_buffer7>>2] = $61;
  $vararg_ptr10 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr10>>2] = $62;
  $vararg_ptr11 = (($vararg_buffer7) + 8|0);
  HEAP32[$vararg_ptr11>>2] = $65;
  (_fprintf(($60|0),(13000|0),($vararg_buffer7|0))|0);
  $66 = $field_id;
  _free($66);
  $0 = -4;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 $67 = $sep_found;
 $68 = ($67|0)==(0);
 if ($68) {
  $69 = HEAP32[_stderr>>2]|0;
  $70 = $1;
  $71 = $2;
  $72 = (_fbtell($70,$71)|0);
  HEAP32[$vararg_buffer12>>2] = $72;
  (_fprintf(($69|0),(13096|0),($vararg_buffer12|0))|0);
  $73 = $field_id;
  _free($73);
  $0 = -5;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 $i = 0;
 $74 = $iptr;
 $fptr = $74;
 $sep_found = 0;
 while(1) {
  $75 = $i;
  $76 = ($75|0)<(10);
  if (!($76)) {
   break;
  }
  $77 = $1;
  $78 = $2;
  $79 = (_fbgetc($77,$78)|0);
  $nextchar = $79;
  $80 = ($79|0)==(-1);
  if ($80) {
   label = 25;
   break;
  }
  $96 = $i;
  $97 = (($96) + 1)|0;
  $i = $97;
  $98 = $nextchar;
  $99 = ($98|0)==(58);
  if ($99) {
   label = 33;
   break;
  }
  $106 = $nextchar;
  $107 = ($106|0)>=(48);
  if (!($107)) {
   label = 37;
   break;
  }
  $108 = $nextchar;
  $109 = ($108|0)<=(57);
  if (!($109)) {
   label = 37;
   break;
  }
  $110 = $nextchar;
  $111 = $110&255;
  $112 = $iptr;
  $113 = (($112) + 1|0);
  $iptr = $113;
  HEAP8[$112>>0] = $111;
 }
 if ((label|0) == 25) {
  $81 = $1;
  $82 = ($81|0)!=(0|0);
  if ($82) {
   $83 = $1;
   $84 = (_ferror(($83|0))|0);
   $85 = ($84|0)!=(0);
   if ($85) {
    $86 = (___errno_location()|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = (_strerror(($87|0))|0);
    $136 = $88;
   } else {
    $136 = 11416;
   }
   $89 = $136;
  } else {
   $89 = 11432;
  }
  $errmsg = $89;
  $90 = HEAP32[_stderr>>2]|0;
  $91 = $errmsg;
  $92 = $1;
  $93 = $2;
  $94 = (_fbtell($92,$93)|0);
  HEAP32[$vararg_buffer15>>2] = $91;
  $vararg_ptr18 = (($vararg_buffer15) + 4|0);
  HEAP32[$vararg_ptr18>>2] = $94;
  (_fprintf(($90|0),(13176|0),($vararg_buffer15|0))|0);
  $95 = $field_id;
  _free($95);
  $0 = -6;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 33) {
  $100 = $fptr;
  $101 = (_atoi($100)|0);
  $field_int = $101;
  $102 = $nextchar;
  $103 = $102&255;
  $104 = $iptr;
  $105 = (($104) + 1|0);
  $iptr = $105;
  HEAP8[$104>>0] = $103;
  $sep_found = 1;
 }
 else if ((label|0) == 37) {
  $114 = HEAP32[_stderr>>2]|0;
  $115 = $nextchar;
  $116 = $nextchar;
  $117 = $1;
  $118 = $2;
  $119 = (_fbtell($117,$118)|0);
  HEAP32[$vararg_buffer19>>2] = $115;
  $vararg_ptr22 = (($vararg_buffer19) + 4|0);
  HEAP32[$vararg_ptr22>>2] = $116;
  $vararg_ptr23 = (($vararg_buffer19) + 8|0);
  HEAP32[$vararg_ptr23>>2] = $119;
  (_fprintf(($114|0),(13248|0),($vararg_buffer19|0))|0);
  $120 = $field_id;
  _free($120);
  $0 = -7;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 $121 = $sep_found;
 $122 = ($121|0)==(0);
 if ($122) {
  $123 = HEAP32[_stderr>>2]|0;
  $124 = $1;
  $125 = $2;
  $126 = (_fbtell($124,$125)|0);
  HEAP32[$vararg_buffer24>>2] = $126;
  (_fprintf(($123|0),(13344|0),($vararg_buffer24|0))|0);
  $127 = $field_id;
  _free($127);
  $0 = -8;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 } else {
  $128 = $field_id;
  $129 = $3;
  HEAP32[$129>>2] = $128;
  $130 = $record_type;
  $131 = $4;
  HEAP32[$131>>2] = $130;
  $132 = $field_int;
  $133 = $5;
  HEAP32[$133>>2] = $132;
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 return 0|0;
}
function _parse_ANSI_NIST_field_ID($oibufptr,$ebufptr,$ofield_id,$orecord_type,$ofield_int) {
 $oibufptr = $oibufptr|0;
 $ebufptr = $ebufptr|0;
 $ofield_id = $ofield_id|0;
 $orecord_type = $orecord_type|0;
 $ofield_int = $ofield_int|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $cptr = 0, $field_id = 0, $field_id_bytes = 0, $field_int = 0, $fptr = 0, $i = 0, $iptr = 0, $nextchar = 0, $record_type = 0;
 var $rptr = 0, $sep_found = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $1 = $oibufptr;
 $2 = $ebufptr;
 $3 = $ofield_id;
 $4 = $orecord_type;
 $5 = $ofield_int;
 $record_type = 0;
 $field_int = 0;
 $field_id_bytes = 21;
 $6 = $field_id_bytes;
 $7 = (_calloc($6,1)|0);
 $field_id = $7;
 $8 = $field_id;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $field_id_bytes;
  HEAP32[$vararg_buffer>>2] = $11;
  (_fprintf(($10|0),(9200|0),($vararg_buffer|0))|0);
  $0 = -2;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 $i = 0;
 $12 = $1;
 $13 = HEAP32[$12>>2]|0;
 $cptr = $13;
 $14 = $field_id;
 $iptr = $14;
 $15 = $field_id;
 $rptr = $15;
 $sep_found = 0;
 while(1) {
  $16 = $i;
  $17 = ($16|0)<(10);
  if (!($17)) {
   break;
  }
  $18 = $cptr;
  $19 = $2;
  $20 = ($18>>>0)>=($19>>>0);
  if ($20) {
   label = 6;
   break;
  }
  $22 = $cptr;
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  $nextchar = $24;
  $25 = $cptr;
  $26 = (($25) + 1|0);
  $cptr = $26;
  $27 = $i;
  $28 = (($27) + 1)|0;
  $i = $28;
  $29 = $nextchar;
  $30 = ($29|0)==(46);
  if ($30) {
   label = 8;
   break;
  }
  $37 = $nextchar;
  $38 = ($37|0)>=(48);
  if (!($38)) {
   label = 12;
   break;
  }
  $39 = $nextchar;
  $40 = ($39|0)<=(57);
  if (!($40)) {
   label = 12;
   break;
  }
  $41 = $nextchar;
  $42 = $41&255;
  $43 = $iptr;
  $44 = (($43) + 1|0);
  $iptr = $44;
  HEAP8[$43>>0] = $42;
 }
 if ((label|0) == 6) {
  $21 = $field_id;
  _free($21);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 else if ((label|0) == 8) {
  $31 = $rptr;
  $32 = (_atoi($31)|0);
  $record_type = $32;
  $33 = $nextchar;
  $34 = $33&255;
  $35 = $iptr;
  $36 = (($35) + 1|0);
  $iptr = $36;
  HEAP8[$35>>0] = $34;
  $sep_found = 1;
 }
 else if ((label|0) == 12) {
  $45 = $field_id;
  _free($45);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 $46 = $sep_found;
 $47 = ($46|0)==(0);
 if ($47) {
  $48 = $field_id;
  _free($48);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 $i = 0;
 $49 = $iptr;
 $fptr = $49;
 $sep_found = 0;
 while(1) {
  $50 = $i;
  $51 = ($50|0)<(10);
  if (!($51)) {
   break;
  }
  $52 = $cptr;
  $53 = $2;
  $54 = ($52>>>0)>=($53>>>0);
  if ($54) {
   label = 19;
   break;
  }
  $56 = $cptr;
  $57 = (($56) + 1|0);
  $cptr = $57;
  $58 = HEAP8[$56>>0]|0;
  $59 = $58&255;
  $nextchar = $59;
  $60 = $i;
  $61 = (($60) + 1)|0;
  $i = $61;
  $62 = $nextchar;
  $63 = ($62|0)==(58);
  if ($63) {
   label = 21;
   break;
  }
  $70 = $nextchar;
  $71 = ($70|0)>=(48);
  if (!($71)) {
   label = 25;
   break;
  }
  $72 = $nextchar;
  $73 = ($72|0)<=(57);
  if (!($73)) {
   label = 25;
   break;
  }
  $74 = $nextchar;
  $75 = $74&255;
  $76 = $iptr;
  $77 = (($76) + 1|0);
  $iptr = $77;
  HEAP8[$76>>0] = $75;
 }
 if ((label|0) == 19) {
  $55 = $field_id;
  _free($55);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 else if ((label|0) == 21) {
  $64 = $fptr;
  $65 = (_atoi($64)|0);
  $field_int = $65;
  $66 = $nextchar;
  $67 = $66&255;
  $68 = $iptr;
  $69 = (($68) + 1|0);
  $iptr = $69;
  HEAP8[$68>>0] = $67;
  $sep_found = 1;
 }
 else if ((label|0) == 25) {
  $78 = $field_id;
  _free($78);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 $79 = $sep_found;
 $80 = ($79|0)==(0);
 if ($80) {
  $81 = $field_id;
  _free($81);
  $0 = 0;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 } else {
  $82 = $cptr;
  $83 = $1;
  HEAP32[$83>>2] = $82;
  $84 = $field_id;
  $85 = $3;
  HEAP32[$85>>2] = $84;
  $86 = $record_type;
  $87 = $4;
  HEAP32[$87>>2] = $86;
  $88 = $field_int;
  $89 = $5;
  HEAP32[$89>>2] = $88;
  $0 = 1;
  $90 = $0;
  STACKTOP = sp;return ($90|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_subfield($fpin,$buf,$osubfield) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $osubfield = $osubfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, $ret = 0, $ret_delimiter = 0, $subfield = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $subfield = sp + 4|0;
 $item = sp;
 $1 = $fpin;
 $2 = $buf;
 $3 = $osubfield;
 $4 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $4;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = $ret;
  $0 = $6;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 while(1) {
  $7 = $1;
  $8 = $2;
  $9 = (_i_read_ANSI_NIST_item($7,$8,$item)|0);
  $ret_delimiter = $9;
  $10 = $ret_delimiter;
  $11 = ($10|0)<(0);
  if ($11) {
   label = 5;
   break;
  }
  $14 = HEAP32[$subfield>>2]|0;
  $15 = HEAP32[$item>>2]|0;
  $16 = (_update_ANSI_NIST_subfield($14,$15)|0);
  $ret = $16;
  $17 = ($16|0)!=(0);
  if ($17) {
   label = 7;
   break;
  }
  $21 = $ret_delimiter;
  $22 = ($21|0)!=(31);
  if ($22) {
   label = 9;
   break;
  }
 }
 if ((label|0) == 5) {
  $12 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($12);
  $13 = $ret_delimiter;
  $0 = $13;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 else if ((label|0) == 7) {
  $18 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($18);
  $19 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($19);
  $20 = $ret;
  $0 = $20;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 else if ((label|0) == 9) {
  $23 = $ret_delimiter;
  $24 = ($23|0)==(30);
  if ($24) {
   $25 = HEAP32[$subfield>>2]|0;
   $26 = (($25) + 16|0);
   HEAP32[$26>>2] = 1;
   $27 = HEAP32[$subfield>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$27>>2] = $29;
  }
  $30 = HEAP32[$subfield>>2]|0;
  $31 = $3;
  HEAP32[$31>>2] = $30;
  $32 = $ret_delimiter;
  $0 = $32;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_item($fpin,$buf,$oitem) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $oitem = $oitem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $errmsg = 0, $item = 0, $nextchar = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $item = sp + 20|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $oitem;
 $4 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $4;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = $ret;
  $0 = $6;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 while(1) {
  $7 = $1;
  $8 = $2;
  $9 = (_fbgetc($7,$8)|0);
  $nextchar = $9;
  $10 = ($9|0)==(-1);
  if ($10) {
   label = 5;
   break;
  }
  $26 = $nextchar;
  $27 = (_is_delimiter($26)|0);
  $28 = ($27|0)!=(0);
  if ($28) {
   label = 13;
   break;
  }
  $36 = HEAP32[$item>>2]|0;
  $37 = $nextchar;
  $38 = (_update_ANSI_NIST_item($36,$37)|0);
  $ret = $38;
  $39 = ($38|0)!=(0);
  if ($39) {
   label = 17;
   break;
  }
 }
 if ((label|0) == 5) {
  $11 = $1;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = $1;
   $14 = (_ferror(($13|0))|0);
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = (___errno_location()|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (_strerror(($17|0))|0);
    $46 = $18;
   } else {
    $46 = 11416;
   }
   $19 = $46;
  } else {
   $19 = 11432;
  }
  $errmsg = $19;
  $20 = HEAP32[_stderr>>2]|0;
  $21 = $errmsg;
  $22 = $1;
  $23 = $2;
  $24 = (_fbtell($22,$23)|0);
  HEAP32[$vararg_buffer>>2] = $21;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $24;
  (_fprintf(($20|0),(12792|0),($vararg_buffer|0))|0);
  $25 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($25);
  $0 = -2;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 else if ((label|0) == 13) {
  $29 = $nextchar;
  $30 = ($29|0)==(31);
  if ($30) {
   $31 = HEAP32[$item>>2]|0;
   $32 = (($31) + 16|0);
   HEAP32[$32>>2] = 1;
   $33 = HEAP32[$item>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$33>>2] = $35;
  }
  $42 = HEAP32[$item>>2]|0;
  $43 = $3;
  HEAP32[$43>>2] = $42;
  $44 = $nextchar;
  $0 = $44;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 else if ((label|0) == 17) {
  $40 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($40);
  $41 = $ret;
  $0 = $41;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_binary_image_record($fpin,$buf,$orecord,$record_type) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $orecord = $orecord|0;
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $errmsg = 0;
 var $field = 0, $item = 0, $new_ptr = 0, $nread = 0, $record = 0, $record_bytes = 0, $ret = 0, $subfield = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer14 = 0, $vararg_buffer18 = 0, $vararg_buffer2 = 0, $vararg_buffer22 = 0, $vararg_buffer26 = 0, $vararg_buffer30 = 0, $vararg_buffer36 = 0, $vararg_buffer43 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0;
 var $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr21 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr33 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, $vararg_ptr41 = 0, $vararg_ptr42 = 0, $vararg_ptr46 = 0, $vararg_ptr47 = 0, $vararg_ptr48 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer43 = sp + 104|0;
 $vararg_buffer36 = sp + 80|0;
 $vararg_buffer30 = sp + 64|0;
 $vararg_buffer26 = sp + 56|0;
 $vararg_buffer22 = sp + 48|0;
 $vararg_buffer18 = sp + 40|0;
 $vararg_buffer14 = sp + 32|0;
 $vararg_buffer10 = sp + 24|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $record = sp + 152|0;
 $field = sp + 148|0;
 $subfield = sp + 144|0;
 $item = sp + 140|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $orecord;
 $4 = $record_type;
 $5 = (_alloc_ANSI_NIST_record($record)|0);
 $ret = $5;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $8 = $4;
 $9 = HEAP32[$record>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = $1;
 $11 = $2;
 $12 = (_i_read_ANSI_NIST_binary_field($10,$11,$field,4)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  $15 = HEAP32[$record>>2]|0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 1)|0;
  $19 = $4;
  HEAP32[$vararg_buffer>>2] = $18;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $19;
  (_fprintf(($14|0),(11664|0),($vararg_buffer|0))|0);
  $0 = -2;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $20 = $4;
 $21 = HEAP32[$field>>2]|0;
 $22 = (($21) + 4|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$record>>2]|0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 $27 = HEAP32[$field>>2]|0;
 $28 = (($27) + 8|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$field>>2]|0;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (_atoi($37)|0);
 $39 = HEAP32[$record>>2]|0;
 $40 = (($39) + 4|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$record>>2]|0;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) - 4)|0;
 $record_bytes = $44;
 $45 = HEAP32[$record>>2]|0;
 $46 = HEAP32[$field>>2]|0;
 $47 = (_update_ANSI_NIST_record($45,$46)|0);
 $ret = $47;
 $48 = ($47|0)!=(0);
 if ($48) {
  $49 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($49);
  $50 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($50);
  $51 = $ret;
  $0 = $51;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $52 = $1;
 $53 = $2;
 $54 = (_i_read_ANSI_NIST_binary_field($52,$53,$field,1)|0);
 $ret = $54;
 $55 = ($54|0)!=(0);
 if ($55) {
  $56 = HEAP32[_stderr>>2]|0;
  $57 = HEAP32[$record>>2]|0;
  $58 = (($57) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (($59) + 1)|0;
  $61 = $4;
  HEAP32[$vararg_buffer2>>2] = $60;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $61;
  (_fprintf(($56|0),(11760|0),($vararg_buffer2|0))|0);
  $0 = -3;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $62 = $4;
 $63 = HEAP32[$field>>2]|0;
 $64 = (($63) + 4|0);
 HEAP32[$64>>2] = $62;
 $65 = HEAP32[$record>>2]|0;
 $66 = (($65) + 12|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 $69 = HEAP32[$field>>2]|0;
 $70 = (($69) + 8|0);
 HEAP32[$70>>2] = $68;
 $71 = $record_bytes;
 $72 = (($71) - 1)|0;
 $record_bytes = $72;
 $73 = HEAP32[$record>>2]|0;
 $74 = HEAP32[$field>>2]|0;
 $75 = (_update_ANSI_NIST_record($73,$74)|0);
 $ret = $75;
 $76 = ($75|0)!=(0);
 if ($76) {
  $77 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($77);
  $78 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($78);
  $79 = $ret;
  $0 = $79;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $80 = $1;
 $81 = $2;
 $82 = (_i_read_ANSI_NIST_binary_field($80,$81,$field,1)|0);
 $ret = $82;
 $83 = ($82|0)!=(0);
 if ($83) {
  $84 = HEAP32[_stderr>>2]|0;
  $85 = HEAP32[$record>>2]|0;
  $86 = (($85) + 12|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 1)|0;
  $89 = $4;
  HEAP32[$vararg_buffer6>>2] = $88;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $89;
  (_fprintf(($84|0),(11856|0),($vararg_buffer6|0))|0);
  $0 = -4;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $90 = $4;
 $91 = HEAP32[$field>>2]|0;
 $92 = (($91) + 4|0);
 HEAP32[$92>>2] = $90;
 $93 = HEAP32[$record>>2]|0;
 $94 = (($93) + 12|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = (($95) + 1)|0;
 $97 = HEAP32[$field>>2]|0;
 $98 = (($97) + 8|0);
 HEAP32[$98>>2] = $96;
 $99 = $record_bytes;
 $100 = (($99) - 1)|0;
 $record_bytes = $100;
 $101 = HEAP32[$record>>2]|0;
 $102 = HEAP32[$field>>2]|0;
 $103 = (_update_ANSI_NIST_record($101,$102)|0);
 $ret = $103;
 $104 = ($103|0)!=(0);
 if ($104) {
  $105 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($105);
  $106 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($106);
  $107 = $ret;
  $0 = $107;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $108 = $1;
 $109 = $2;
 $110 = (_i_read_ANSI_NIST_binary_field($108,$109,$field,6)|0);
 $ret = $110;
 $111 = ($110|0)!=(0);
 if ($111) {
  $112 = HEAP32[_stderr>>2]|0;
  $113 = HEAP32[$record>>2]|0;
  $114 = (($113) + 12|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) + 1)|0;
  $117 = $4;
  HEAP32[$vararg_buffer10>>2] = $116;
  $vararg_ptr13 = (($vararg_buffer10) + 4|0);
  HEAP32[$vararg_ptr13>>2] = $117;
  (_fprintf(($112|0),(11952|0),($vararg_buffer10|0))|0);
  $0 = -5;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $118 = $4;
 $119 = HEAP32[$field>>2]|0;
 $120 = (($119) + 4|0);
 HEAP32[$120>>2] = $118;
 $121 = HEAP32[$record>>2]|0;
 $122 = (($121) + 12|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = (($123) + 1)|0;
 $125 = HEAP32[$field>>2]|0;
 $126 = (($125) + 8|0);
 HEAP32[$126>>2] = $124;
 $127 = $record_bytes;
 $128 = (($127) - 6)|0;
 $record_bytes = $128;
 $129 = HEAP32[$record>>2]|0;
 $130 = HEAP32[$field>>2]|0;
 $131 = (_update_ANSI_NIST_record($129,$130)|0);
 $ret = $131;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($133);
  $134 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($134);
  $135 = $ret;
  $0 = $135;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $136 = $1;
 $137 = $2;
 $138 = (_i_read_ANSI_NIST_binary_field($136,$137,$field,1)|0);
 $ret = $138;
 $139 = ($138|0)!=(0);
 if ($139) {
  $140 = HEAP32[_stderr>>2]|0;
  $141 = HEAP32[$record>>2]|0;
  $142 = (($141) + 12|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = (($143) + 1)|0;
  $145 = $4;
  HEAP32[$vararg_buffer14>>2] = $144;
  $vararg_ptr17 = (($vararg_buffer14) + 4|0);
  HEAP32[$vararg_ptr17>>2] = $145;
  (_fprintf(($140|0),(12048|0),($vararg_buffer14|0))|0);
  $0 = -6;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $146 = $4;
 $147 = HEAP32[$field>>2]|0;
 $148 = (($147) + 4|0);
 HEAP32[$148>>2] = $146;
 $149 = HEAP32[$record>>2]|0;
 $150 = (($149) + 12|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = (($151) + 1)|0;
 $153 = HEAP32[$field>>2]|0;
 $154 = (($153) + 8|0);
 HEAP32[$154>>2] = $152;
 $155 = $record_bytes;
 $156 = (($155) - 1)|0;
 $record_bytes = $156;
 $157 = HEAP32[$record>>2]|0;
 $158 = HEAP32[$field>>2]|0;
 $159 = (_update_ANSI_NIST_record($157,$158)|0);
 $ret = $159;
 $160 = ($159|0)!=(0);
 if ($160) {
  $161 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($161);
  $162 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($162);
  $163 = $ret;
  $0 = $163;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $164 = $1;
 $165 = $2;
 $166 = (_i_read_ANSI_NIST_binary_field($164,$165,$field,2)|0);
 $ret = $166;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = HEAP32[_stderr>>2]|0;
  $169 = HEAP32[$record>>2]|0;
  $170 = (($169) + 12|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = (($171) + 1)|0;
  $173 = $4;
  HEAP32[$vararg_buffer18>>2] = $172;
  $vararg_ptr21 = (($vararg_buffer18) + 4|0);
  HEAP32[$vararg_ptr21>>2] = $173;
  (_fprintf(($168|0),(12144|0),($vararg_buffer18|0))|0);
  $0 = -7;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $174 = $4;
 $175 = HEAP32[$field>>2]|0;
 $176 = (($175) + 4|0);
 HEAP32[$176>>2] = $174;
 $177 = HEAP32[$record>>2]|0;
 $178 = (($177) + 12|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = (($179) + 1)|0;
 $181 = HEAP32[$field>>2]|0;
 $182 = (($181) + 8|0);
 HEAP32[$182>>2] = $180;
 $183 = $record_bytes;
 $184 = (($183) - 2)|0;
 $record_bytes = $184;
 $185 = HEAP32[$record>>2]|0;
 $186 = HEAP32[$field>>2]|0;
 $187 = (_update_ANSI_NIST_record($185,$186)|0);
 $ret = $187;
 $188 = ($187|0)!=(0);
 if ($188) {
  $189 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($189);
  $190 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($190);
  $191 = $ret;
  $0 = $191;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $192 = $1;
 $193 = $2;
 $194 = (_i_read_ANSI_NIST_binary_field($192,$193,$field,2)|0);
 $ret = $194;
 $195 = ($194|0)!=(0);
 if ($195) {
  $196 = HEAP32[_stderr>>2]|0;
  $197 = HEAP32[$record>>2]|0;
  $198 = (($197) + 12|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + 1)|0;
  $201 = $4;
  HEAP32[$vararg_buffer22>>2] = $200;
  $vararg_ptr25 = (($vararg_buffer22) + 4|0);
  HEAP32[$vararg_ptr25>>2] = $201;
  (_fprintf(($196|0),(12240|0),($vararg_buffer22|0))|0);
  $0 = -8;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $202 = $4;
 $203 = HEAP32[$field>>2]|0;
 $204 = (($203) + 4|0);
 HEAP32[$204>>2] = $202;
 $205 = HEAP32[$record>>2]|0;
 $206 = (($205) + 12|0);
 $207 = HEAP32[$206>>2]|0;
 $208 = (($207) + 1)|0;
 $209 = HEAP32[$field>>2]|0;
 $210 = (($209) + 8|0);
 HEAP32[$210>>2] = $208;
 $211 = $record_bytes;
 $212 = (($211) - 2)|0;
 $record_bytes = $212;
 $213 = HEAP32[$record>>2]|0;
 $214 = HEAP32[$field>>2]|0;
 $215 = (_update_ANSI_NIST_record($213,$214)|0);
 $ret = $215;
 $216 = ($215|0)!=(0);
 if ($216) {
  $217 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($217);
  $218 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($218);
  $219 = $ret;
  $0 = $219;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $220 = $1;
 $221 = $2;
 $222 = (_i_read_ANSI_NIST_binary_field($220,$221,$field,1)|0);
 $ret = $222;
 $223 = ($222|0)!=(0);
 if ($223) {
  $224 = HEAP32[_stderr>>2]|0;
  $225 = HEAP32[$record>>2]|0;
  $226 = (($225) + 12|0);
  $227 = HEAP32[$226>>2]|0;
  $228 = (($227) + 1)|0;
  $229 = $4;
  HEAP32[$vararg_buffer26>>2] = $228;
  $vararg_ptr29 = (($vararg_buffer26) + 4|0);
  HEAP32[$vararg_ptr29>>2] = $229;
  (_fprintf(($224|0),(12336|0),($vararg_buffer26|0))|0);
  $0 = -9;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $230 = $4;
 $231 = HEAP32[$field>>2]|0;
 $232 = (($231) + 4|0);
 HEAP32[$232>>2] = $230;
 $233 = HEAP32[$record>>2]|0;
 $234 = (($233) + 12|0);
 $235 = HEAP32[$234>>2]|0;
 $236 = (($235) + 1)|0;
 $237 = HEAP32[$field>>2]|0;
 $238 = (($237) + 8|0);
 HEAP32[$238>>2] = $236;
 $239 = $record_bytes;
 $240 = (($239) - 1)|0;
 $record_bytes = $240;
 $241 = HEAP32[$record>>2]|0;
 $242 = HEAP32[$field>>2]|0;
 $243 = (_update_ANSI_NIST_record($241,$242)|0);
 $ret = $243;
 $244 = ($243|0)!=(0);
 if ($244) {
  $245 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($245);
  $246 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($246);
  $247 = $ret;
  $0 = $247;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $248 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $248;
 $249 = ($248|0)!=(0);
 if ($249) {
  $250 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($250);
  $251 = $ret;
  $0 = $251;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $252 = HEAP32[$item>>2]|0;
 $253 = (($252) + 8|0);
 $254 = HEAP32[$253>>2]|0;
 $255 = $record_bytes;
 $256 = ($254|0)<($255|0);
 do {
  if ($256) {
   $257 = HEAP32[$item>>2]|0;
   $258 = (($257) + 12|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = $record_bytes;
   $261 = (_realloc($259,$260)|0);
   $new_ptr = $261;
   $262 = $new_ptr;
   $263 = ($262|0)==(0|0);
   if (!($263)) {
    $275 = $new_ptr;
    $276 = HEAP32[$item>>2]|0;
    $277 = (($276) + 12|0);
    HEAP32[$277>>2] = $275;
    $278 = $record_bytes;
    $279 = HEAP32[$item>>2]|0;
    $280 = (($279) + 8|0);
    HEAP32[$280>>2] = $278;
    break;
   }
   $264 = HEAP32[_stderr>>2]|0;
   $265 = HEAP32[$item>>2]|0;
   $266 = (($265) + 8|0);
   $267 = HEAP32[$266>>2]|0;
   $268 = $record_bytes;
   $269 = $4;
   $270 = $1;
   $271 = $2;
   $272 = (_fbtell($270,$271)|0);
   HEAP32[$vararg_buffer30>>2] = $267;
   $vararg_ptr33 = (($vararg_buffer30) + 4|0);
   HEAP32[$vararg_ptr33>>2] = $268;
   $vararg_ptr34 = (($vararg_buffer30) + 8|0);
   HEAP32[$vararg_ptr34>>2] = $269;
   $vararg_ptr35 = (($vararg_buffer30) + 12|0);
   HEAP32[$vararg_ptr35>>2] = $272;
   (_fprintf(($264|0),(12432|0),($vararg_buffer30|0))|0);
   $273 = HEAP32[$item>>2]|0;
   _free_ANSI_NIST_item($273);
   $274 = HEAP32[$record>>2]|0;
   _free_ANSI_NIST_record($274);
   $0 = -2;
   $377 = $0;
   STACKTOP = sp;return ($377|0);
  }
 } while(0);
 $281 = HEAP32[$item>>2]|0;
 $282 = (($281) + 12|0);
 $283 = HEAP32[$282>>2]|0;
 $284 = $record_bytes;
 $285 = $1;
 $286 = $2;
 $287 = (_fbread($283,1,$284,$285,$286)|0);
 $nread = $287;
 $288 = $nread;
 $289 = $record_bytes;
 $290 = ($288|0)!=($289|0);
 if ($290) {
  $291 = $1;
  $292 = ($291|0)!=(0|0);
  if ($292) {
   $293 = $1;
   $294 = (_ferror(($293|0))|0);
   $295 = ($294|0)!=(0);
   if ($295) {
    $296 = (___errno_location()|0);
    $297 = HEAP32[$296>>2]|0;
    $298 = (_strerror(($297|0))|0);
    $378 = $298;
   } else {
    $378 = 11416;
   }
   $299 = $378;
  } else {
   $299 = 11432;
  }
  $errmsg = $299;
  $300 = HEAP32[_stderr>>2]|0;
  $301 = $nread;
  $302 = $record_bytes;
  $303 = $4;
  $304 = $errmsg;
  $305 = $1;
  $306 = $2;
  $307 = (_fbtell($305,$306)|0);
  HEAP32[$vararg_buffer36>>2] = $301;
  $vararg_ptr39 = (($vararg_buffer36) + 4|0);
  HEAP32[$vararg_ptr39>>2] = $302;
  $vararg_ptr40 = (($vararg_buffer36) + 8|0);
  HEAP32[$vararg_ptr40>>2] = $303;
  $vararg_ptr41 = (($vararg_buffer36) + 12|0);
  HEAP32[$vararg_ptr41>>2] = $304;
  $vararg_ptr42 = (($vararg_buffer36) + 16|0);
  HEAP32[$vararg_ptr42>>2] = $307;
  (_fprintf(($300|0),(12560|0),($vararg_buffer36|0))|0);
  $308 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($308);
  $309 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($309);
  $0 = -3;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $310 = $record_bytes;
 $311 = HEAP32[$item>>2]|0;
 HEAP32[$311>>2] = $310;
 $312 = $record_bytes;
 $313 = HEAP32[$item>>2]|0;
 $314 = (($313) + 4|0);
 HEAP32[$314>>2] = $312;
 $315 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $315;
 $316 = ($315|0)!=(0);
 if ($316) {
  $317 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($317);
  $318 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($318);
  $319 = $ret;
  $0 = $319;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $320 = HEAP32[$subfield>>2]|0;
 $321 = HEAP32[$item>>2]|0;
 $322 = (_update_ANSI_NIST_subfield($320,$321)|0);
 $ret = $322;
 $323 = ($322|0)!=(0);
 if ($323) {
  $324 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($324);
  $325 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($325);
  $326 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($326);
  $327 = $ret;
  $0 = $327;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $328 = (_alloc_ANSI_NIST_field($field)|0);
 $ret = $328;
 $329 = ($328|0)!=(0);
 if ($329) {
  $330 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($330);
  $331 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($331);
  $332 = $ret;
  $0 = $332;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $333 = $4;
 $334 = HEAP32[$field>>2]|0;
 $335 = (($334) + 4|0);
 HEAP32[$335>>2] = $333;
 $336 = HEAP32[$record>>2]|0;
 $337 = (($336) + 12|0);
 $338 = HEAP32[$337>>2]|0;
 $339 = (($338) + 1)|0;
 $340 = HEAP32[$field>>2]|0;
 $341 = (($340) + 8|0);
 HEAP32[$341>>2] = $339;
 $342 = HEAP32[$field>>2]|0;
 $343 = HEAP32[$subfield>>2]|0;
 $344 = (_update_ANSI_NIST_field($342,$343)|0);
 $ret = $344;
 $345 = ($344|0)!=(0);
 if ($345) {
  $346 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($346);
  $347 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($347);
  $348 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($348);
  $349 = $ret;
  $0 = $349;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $350 = HEAP32[$record>>2]|0;
 $351 = HEAP32[$field>>2]|0;
 $352 = (_update_ANSI_NIST_record($350,$351)|0);
 $ret = $352;
 $353 = ($352|0)!=(0);
 if ($353) {
  $354 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($354);
  $355 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($355);
  $356 = $ret;
  $0 = $356;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 $357 = HEAP32[$record>>2]|0;
 $358 = (($357) + 4|0);
 $359 = HEAP32[$358>>2]|0;
 $360 = HEAP32[$record>>2]|0;
 $361 = (($360) + 8|0);
 $362 = HEAP32[$361>>2]|0;
 $363 = ($359|0)!=($362|0);
 if ($363) {
  $364 = HEAP32[_stderr>>2]|0;
  $365 = HEAP32[$record>>2]|0;
  $366 = (($365) + 8|0);
  $367 = HEAP32[$366>>2]|0;
  $368 = HEAP32[$record>>2]|0;
  $369 = (($368) + 4|0);
  $370 = HEAP32[$369>>2]|0;
  $371 = $4;
  $372 = $1;
  $373 = $2;
  $374 = (_fbtell($372,$373)|0);
  HEAP32[$vararg_buffer43>>2] = $367;
  $vararg_ptr46 = (($vararg_buffer43) + 4|0);
  HEAP32[$vararg_ptr46>>2] = $370;
  $vararg_ptr47 = (($vararg_buffer43) + 8|0);
  HEAP32[$vararg_ptr47>>2] = $371;
  $vararg_ptr48 = (($vararg_buffer43) + 12|0);
  HEAP32[$vararg_ptr48>>2] = $374;
  (_fprintf(($364|0),(12680|0),($vararg_buffer43|0))|0);
  $0 = -4;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 } else {
  $375 = HEAP32[$record>>2]|0;
  $376 = $3;
  HEAP32[$376>>2] = $375;
  $0 = 0;
  $377 = $0;
  STACKTOP = sp;return ($377|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_binary_signature_record($fpin,$buf,$orecord,$record_type) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $orecord = $orecord|0;
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $errmsg = 0, $field = 0, $item = 0, $new_ptr = 0, $nread = 0, $record = 0, $record_bytes = 0, $ret = 0, $subfield = 0, $vararg_buffer = 0;
 var $vararg_buffer10 = 0, $vararg_buffer14 = 0, $vararg_buffer18 = 0, $vararg_buffer2 = 0, $vararg_buffer22 = 0, $vararg_buffer26 = 0, $vararg_buffer32 = 0, $vararg_buffer38 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr21 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr35 = 0, $vararg_ptr36 = 0, $vararg_ptr37 = 0;
 var $vararg_ptr41 = 0, $vararg_ptr42 = 0, $vararg_ptr43 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer38 = sp + 88|0;
 $vararg_buffer32 = sp + 72|0;
 $vararg_buffer26 = sp + 56|0;
 $vararg_buffer22 = sp + 48|0;
 $vararg_buffer18 = sp + 40|0;
 $vararg_buffer14 = sp + 32|0;
 $vararg_buffer10 = sp + 24|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $record = sp + 136|0;
 $field = sp + 132|0;
 $subfield = sp + 128|0;
 $item = sp + 124|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $orecord;
 $4 = $record_type;
 $5 = (_alloc_ANSI_NIST_record($record)|0);
 $ret = $5;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $8 = $4;
 $9 = HEAP32[$record>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = $1;
 $11 = $2;
 $12 = (_i_read_ANSI_NIST_binary_field($10,$11,$field,4)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  $15 = HEAP32[$record>>2]|0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 1)|0;
  $19 = $4;
  HEAP32[$vararg_buffer>>2] = $18;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $19;
  (_fprintf(($14|0),(10560|0),($vararg_buffer|0))|0);
  $0 = -2;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $20 = $4;
 $21 = HEAP32[$field>>2]|0;
 $22 = (($21) + 4|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$record>>2]|0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 $27 = HEAP32[$field>>2]|0;
 $28 = (($27) + 8|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$field>>2]|0;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (_atoi($37)|0);
 $39 = HEAP32[$record>>2]|0;
 $40 = (($39) + 4|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$record>>2]|0;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) - 4)|0;
 $record_bytes = $44;
 $45 = HEAP32[$record>>2]|0;
 $46 = HEAP32[$field>>2]|0;
 $47 = (_update_ANSI_NIST_record($45,$46)|0);
 $ret = $47;
 $48 = ($47|0)!=(0);
 if ($48) {
  $49 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($49);
  $50 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($50);
  $51 = $ret;
  $0 = $51;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $52 = $1;
 $53 = $2;
 $54 = (_i_read_ANSI_NIST_binary_field($52,$53,$field,1)|0);
 $ret = $54;
 $55 = ($54|0)!=(0);
 if ($55) {
  $56 = HEAP32[_stderr>>2]|0;
  $57 = HEAP32[$record>>2]|0;
  $58 = (($57) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (($59) + 1)|0;
  $61 = $4;
  HEAP32[$vararg_buffer2>>2] = $60;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $61;
  (_fprintf(($56|0),(10664|0),($vararg_buffer2|0))|0);
  $0 = -3;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $62 = $4;
 $63 = HEAP32[$field>>2]|0;
 $64 = (($63) + 4|0);
 HEAP32[$64>>2] = $62;
 $65 = HEAP32[$record>>2]|0;
 $66 = (($65) + 12|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 $69 = HEAP32[$field>>2]|0;
 $70 = (($69) + 8|0);
 HEAP32[$70>>2] = $68;
 $71 = $record_bytes;
 $72 = (($71) - 1)|0;
 $record_bytes = $72;
 $73 = HEAP32[$record>>2]|0;
 $74 = HEAP32[$field>>2]|0;
 $75 = (_update_ANSI_NIST_record($73,$74)|0);
 $ret = $75;
 $76 = ($75|0)!=(0);
 if ($76) {
  $77 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($77);
  $78 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($78);
  $79 = $ret;
  $0 = $79;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $80 = $1;
 $81 = $2;
 $82 = (_i_read_ANSI_NIST_binary_field($80,$81,$field,1)|0);
 $ret = $82;
 $83 = ($82|0)!=(0);
 if ($83) {
  $84 = HEAP32[_stderr>>2]|0;
  $85 = HEAP32[$record>>2]|0;
  $86 = (($85) + 12|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 1)|0;
  $89 = $4;
  HEAP32[$vararg_buffer6>>2] = $88;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $89;
  (_fprintf(($84|0),(10768|0),($vararg_buffer6|0))|0);
  $0 = -4;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $90 = $4;
 $91 = HEAP32[$field>>2]|0;
 $92 = (($91) + 4|0);
 HEAP32[$92>>2] = $90;
 $93 = HEAP32[$record>>2]|0;
 $94 = (($93) + 12|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = (($95) + 1)|0;
 $97 = HEAP32[$field>>2]|0;
 $98 = (($97) + 8|0);
 HEAP32[$98>>2] = $96;
 $99 = $record_bytes;
 $100 = (($99) - 1)|0;
 $record_bytes = $100;
 $101 = HEAP32[$record>>2]|0;
 $102 = HEAP32[$field>>2]|0;
 $103 = (_update_ANSI_NIST_record($101,$102)|0);
 $ret = $103;
 $104 = ($103|0)!=(0);
 if ($104) {
  $105 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($105);
  $106 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($106);
  $107 = $ret;
  $0 = $107;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $108 = $1;
 $109 = $2;
 $110 = (_i_read_ANSI_NIST_binary_field($108,$109,$field,1)|0);
 $ret = $110;
 $111 = ($110|0)!=(0);
 if ($111) {
  $112 = HEAP32[_stderr>>2]|0;
  $113 = HEAP32[$record>>2]|0;
  $114 = (($113) + 12|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) + 1)|0;
  $117 = $4;
  HEAP32[$vararg_buffer10>>2] = $116;
  $vararg_ptr13 = (($vararg_buffer10) + 4|0);
  HEAP32[$vararg_ptr13>>2] = $117;
  (_fprintf(($112|0),(10872|0),($vararg_buffer10|0))|0);
  $0 = -5;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $118 = $4;
 $119 = HEAP32[$field>>2]|0;
 $120 = (($119) + 4|0);
 HEAP32[$120>>2] = $118;
 $121 = HEAP32[$record>>2]|0;
 $122 = (($121) + 12|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = (($123) + 1)|0;
 $125 = HEAP32[$field>>2]|0;
 $126 = (($125) + 8|0);
 HEAP32[$126>>2] = $124;
 $127 = $record_bytes;
 $128 = (($127) - 1)|0;
 $record_bytes = $128;
 $129 = HEAP32[$record>>2]|0;
 $130 = HEAP32[$field>>2]|0;
 $131 = (_update_ANSI_NIST_record($129,$130)|0);
 $ret = $131;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($133);
  $134 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($134);
  $135 = $ret;
  $0 = $135;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $136 = $1;
 $137 = $2;
 $138 = (_i_read_ANSI_NIST_binary_field($136,$137,$field,1)|0);
 $ret = $138;
 $139 = ($138|0)!=(0);
 if ($139) {
  $140 = HEAP32[_stderr>>2]|0;
  $141 = HEAP32[$record>>2]|0;
  $142 = (($141) + 12|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = (($143) + 1)|0;
  $145 = $4;
  HEAP32[$vararg_buffer14>>2] = $144;
  $vararg_ptr17 = (($vararg_buffer14) + 4|0);
  HEAP32[$vararg_ptr17>>2] = $145;
  (_fprintf(($140|0),(10976|0),($vararg_buffer14|0))|0);
  $0 = -6;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $146 = $4;
 $147 = HEAP32[$field>>2]|0;
 $148 = (($147) + 4|0);
 HEAP32[$148>>2] = $146;
 $149 = HEAP32[$record>>2]|0;
 $150 = (($149) + 12|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = (($151) + 1)|0;
 $153 = HEAP32[$field>>2]|0;
 $154 = (($153) + 8|0);
 HEAP32[$154>>2] = $152;
 $155 = $record_bytes;
 $156 = (($155) - 1)|0;
 $record_bytes = $156;
 $157 = HEAP32[$record>>2]|0;
 $158 = HEAP32[$field>>2]|0;
 $159 = (_update_ANSI_NIST_record($157,$158)|0);
 $ret = $159;
 $160 = ($159|0)!=(0);
 if ($160) {
  $161 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($161);
  $162 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($162);
  $163 = $ret;
  $0 = $163;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $164 = $1;
 $165 = $2;
 $166 = (_i_read_ANSI_NIST_binary_field($164,$165,$field,2)|0);
 $ret = $166;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = HEAP32[_stderr>>2]|0;
  $169 = HEAP32[$record>>2]|0;
  $170 = (($169) + 12|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = (($171) + 1)|0;
  $173 = $4;
  HEAP32[$vararg_buffer18>>2] = $172;
  $vararg_ptr21 = (($vararg_buffer18) + 4|0);
  HEAP32[$vararg_ptr21>>2] = $173;
  (_fprintf(($168|0),(11080|0),($vararg_buffer18|0))|0);
  $0 = -7;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $174 = $4;
 $175 = HEAP32[$field>>2]|0;
 $176 = (($175) + 4|0);
 HEAP32[$176>>2] = $174;
 $177 = HEAP32[$record>>2]|0;
 $178 = (($177) + 12|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = (($179) + 1)|0;
 $181 = HEAP32[$field>>2]|0;
 $182 = (($181) + 8|0);
 HEAP32[$182>>2] = $180;
 $183 = $record_bytes;
 $184 = (($183) - 2)|0;
 $record_bytes = $184;
 $185 = HEAP32[$record>>2]|0;
 $186 = HEAP32[$field>>2]|0;
 $187 = (_update_ANSI_NIST_record($185,$186)|0);
 $ret = $187;
 $188 = ($187|0)!=(0);
 if ($188) {
  $189 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($189);
  $190 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($190);
  $191 = $ret;
  $0 = $191;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $192 = $1;
 $193 = $2;
 $194 = (_i_read_ANSI_NIST_binary_field($192,$193,$field,2)|0);
 $ret = $194;
 $195 = ($194|0)!=(0);
 if ($195) {
  $196 = HEAP32[_stderr>>2]|0;
  $197 = HEAP32[$record>>2]|0;
  $198 = (($197) + 12|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + 1)|0;
  $201 = $4;
  HEAP32[$vararg_buffer22>>2] = $200;
  $vararg_ptr25 = (($vararg_buffer22) + 4|0);
  HEAP32[$vararg_ptr25>>2] = $201;
  (_fprintf(($196|0),(11184|0),($vararg_buffer22|0))|0);
  $0 = -8;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $202 = $4;
 $203 = HEAP32[$field>>2]|0;
 $204 = (($203) + 4|0);
 HEAP32[$204>>2] = $202;
 $205 = HEAP32[$record>>2]|0;
 $206 = (($205) + 12|0);
 $207 = HEAP32[$206>>2]|0;
 $208 = (($207) + 1)|0;
 $209 = HEAP32[$field>>2]|0;
 $210 = (($209) + 8|0);
 HEAP32[$210>>2] = $208;
 $211 = $record_bytes;
 $212 = (($211) - 2)|0;
 $record_bytes = $212;
 $213 = HEAP32[$record>>2]|0;
 $214 = HEAP32[$field>>2]|0;
 $215 = (_update_ANSI_NIST_record($213,$214)|0);
 $ret = $215;
 $216 = ($215|0)!=(0);
 if ($216) {
  $217 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($217);
  $218 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($218);
  $219 = $ret;
  $0 = $219;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $220 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $220;
 $221 = ($220|0)!=(0);
 if ($221) {
  $222 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($222);
  $223 = $ret;
  $0 = $223;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $224 = HEAP32[$item>>2]|0;
 $225 = (($224) + 8|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = $record_bytes;
 $228 = ($226|0)<($227|0);
 do {
  if ($228) {
   $229 = HEAP32[$item>>2]|0;
   $230 = (($229) + 12|0);
   $231 = HEAP32[$230>>2]|0;
   $232 = $record_bytes;
   $233 = (_realloc($231,$232)|0);
   $new_ptr = $233;
   $234 = $new_ptr;
   $235 = ($234|0)==(0|0);
   if (!($235)) {
    $247 = $new_ptr;
    $248 = HEAP32[$item>>2]|0;
    $249 = (($248) + 12|0);
    HEAP32[$249>>2] = $247;
    $250 = $record_bytes;
    $251 = HEAP32[$item>>2]|0;
    $252 = (($251) + 8|0);
    HEAP32[$252>>2] = $250;
    break;
   }
   $236 = HEAP32[$item>>2]|0;
   _free_ANSI_NIST_item($236);
   $237 = HEAP32[$record>>2]|0;
   _free_ANSI_NIST_record($237);
   $238 = HEAP32[_stderr>>2]|0;
   $239 = HEAP32[$item>>2]|0;
   $240 = (($239) + 8|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = $record_bytes;
   $243 = $4;
   $244 = $1;
   $245 = $2;
   $246 = (_fbtell($244,$245)|0);
   HEAP32[$vararg_buffer26>>2] = $241;
   $vararg_ptr29 = (($vararg_buffer26) + 4|0);
   HEAP32[$vararg_ptr29>>2] = $242;
   $vararg_ptr30 = (($vararg_buffer26) + 8|0);
   HEAP32[$vararg_ptr30>>2] = $243;
   $vararg_ptr31 = (($vararg_buffer26) + 12|0);
   HEAP32[$vararg_ptr31>>2] = $246;
   (_fprintf(($238|0),(11288|0),($vararg_buffer26|0))|0);
   $0 = -2;
   $348 = $0;
   STACKTOP = sp;return ($348|0);
  }
 } while(0);
 $253 = HEAP32[$item>>2]|0;
 $254 = (($253) + 12|0);
 $255 = HEAP32[$254>>2]|0;
 $256 = $record_bytes;
 $257 = $1;
 $258 = $2;
 $259 = (_fbread($255,1,$256,$257,$258)|0);
 $nread = $259;
 $260 = $nread;
 $261 = $record_bytes;
 $262 = ($260|0)!=($261|0);
 if ($262) {
  $263 = $1;
  $264 = ($263|0)!=(0|0);
  if ($264) {
   $265 = $1;
   $266 = (_ferror(($265|0))|0);
   $267 = ($266|0)!=(0);
   if ($267) {
    $268 = (___errno_location()|0);
    $269 = HEAP32[$268>>2]|0;
    $270 = (_strerror(($269|0))|0);
    $349 = $270;
   } else {
    $349 = 11416;
   }
   $271 = $349;
  } else {
   $271 = 11432;
  }
  $errmsg = $271;
  $272 = HEAP32[_stderr>>2]|0;
  $273 = $nread;
  $274 = $record_bytes;
  $275 = $errmsg;
  $276 = $1;
  $277 = $2;
  $278 = (_fbtell($276,$277)|0);
  HEAP32[$vararg_buffer32>>2] = $273;
  $vararg_ptr35 = (($vararg_buffer32) + 4|0);
  HEAP32[$vararg_ptr35>>2] = $274;
  $vararg_ptr36 = (($vararg_buffer32) + 8|0);
  HEAP32[$vararg_ptr36>>2] = $275;
  $vararg_ptr37 = (($vararg_buffer32) + 12|0);
  HEAP32[$vararg_ptr37>>2] = $278;
  (_fprintf(($272|0),(11456|0),($vararg_buffer32|0))|0);
  $279 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($279);
  $280 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($280);
  $0 = -3;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $281 = $record_bytes;
 $282 = HEAP32[$item>>2]|0;
 HEAP32[$282>>2] = $281;
 $283 = $record_bytes;
 $284 = HEAP32[$item>>2]|0;
 $285 = (($284) + 4|0);
 HEAP32[$285>>2] = $283;
 $286 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $286;
 $287 = ($286|0)!=(0);
 if ($287) {
  $288 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($288);
  $289 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($289);
  $290 = $ret;
  $0 = $290;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $291 = HEAP32[$subfield>>2]|0;
 $292 = HEAP32[$item>>2]|0;
 $293 = (_update_ANSI_NIST_subfield($291,$292)|0);
 $ret = $293;
 $294 = ($293|0)!=(0);
 if ($294) {
  $295 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($295);
  $296 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($296);
  $297 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($297);
  $298 = $ret;
  $0 = $298;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $299 = (_alloc_ANSI_NIST_field($field)|0);
 $ret = $299;
 $300 = ($299|0)!=(0);
 if ($300) {
  $301 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($301);
  $302 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($302);
  $303 = $ret;
  $0 = $303;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $304 = $4;
 $305 = HEAP32[$field>>2]|0;
 $306 = (($305) + 4|0);
 HEAP32[$306>>2] = $304;
 $307 = HEAP32[$record>>2]|0;
 $308 = (($307) + 12|0);
 $309 = HEAP32[$308>>2]|0;
 $310 = (($309) + 1)|0;
 $311 = HEAP32[$field>>2]|0;
 $312 = (($311) + 8|0);
 HEAP32[$312>>2] = $310;
 $313 = HEAP32[$field>>2]|0;
 $314 = HEAP32[$subfield>>2]|0;
 $315 = (_update_ANSI_NIST_field($313,$314)|0);
 $ret = $315;
 $316 = ($315|0)!=(0);
 if ($316) {
  $317 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($317);
  $318 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($318);
  $319 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($319);
  $320 = $ret;
  $0 = $320;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $321 = HEAP32[$record>>2]|0;
 $322 = HEAP32[$field>>2]|0;
 $323 = (_update_ANSI_NIST_record($321,$322)|0);
 $ret = $323;
 $324 = ($323|0)!=(0);
 if ($324) {
  $325 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($325);
  $326 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($326);
  $327 = $ret;
  $0 = $327;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 $328 = HEAP32[$record>>2]|0;
 $329 = (($328) + 4|0);
 $330 = HEAP32[$329>>2]|0;
 $331 = HEAP32[$record>>2]|0;
 $332 = (($331) + 8|0);
 $333 = HEAP32[$332>>2]|0;
 $334 = ($330|0)!=($333|0);
 if ($334) {
  $335 = HEAP32[_stderr>>2]|0;
  $336 = HEAP32[$record>>2]|0;
  $337 = (($336) + 8|0);
  $338 = HEAP32[$337>>2]|0;
  $339 = HEAP32[$record>>2]|0;
  $340 = (($339) + 4|0);
  $341 = HEAP32[$340>>2]|0;
  $342 = $4;
  $343 = $1;
  $344 = $2;
  $345 = (_fbtell($343,$344)|0);
  HEAP32[$vararg_buffer38>>2] = $338;
  $vararg_ptr41 = (($vararg_buffer38) + 4|0);
  HEAP32[$vararg_ptr41>>2] = $341;
  $vararg_ptr42 = (($vararg_buffer38) + 8|0);
  HEAP32[$vararg_ptr42>>2] = $342;
  $vararg_ptr43 = (($vararg_buffer38) + 12|0);
  HEAP32[$vararg_ptr43>>2] = $345;
  (_fprintf(($335|0),(11552|0),($vararg_buffer38|0))|0);
  $0 = -4;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 } else {
  $346 = HEAP32[$record>>2]|0;
  $347 = $3;
  HEAP32[$347>>2] = $346;
  $0 = 0;
  $348 = $0;
  STACKTOP = sp;return ($348|0);
 }
 return 0|0;
}
function _i_read_ANSI_NIST_binary_field($fpin,$buf,$ofield,$num_bytes) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ofield = $ofield|0;
 $num_bytes = $num_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $field = 0, $i = 0, $item = 0, $itemvalue = 0, $ret = 0, $subfield = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $itemvalue = sp + 12|0;
 $item = sp + 8|0;
 $subfield = sp + 4|0;
 $field = sp;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ofield;
 $4 = $num_bytes;
 $5 = (_alloc_ANSI_NIST_field($field)|0);
 $ret = $5;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $8 = $4;
 $9 = ($8>>>0)<=(4);
 do {
  if ($9) {
   $10 = $1;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = $1;
    $13 = $4;
    $14 = (_read_binary_item_data($12,$itemvalue,$13)|0);
    $ret = $14;
   } else {
    $15 = $2;
    $16 = $4;
    $17 = (_scan_binary_item_data($15,$itemvalue,$16)|0);
    $ret = $17;
   }
   $18 = $ret;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($20);
    $21 = $ret;
    $0 = $21;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   $22 = (_alloc_ANSI_NIST_item($item)|0);
   $ret = $22;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = HEAP32[$itemvalue>>2]|0;
    _free($24);
    $25 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($25);
    $26 = $ret;
    $0 = $26;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   $27 = HEAP32[$item>>2]|0;
   $28 = (($27) + 12|0);
   $29 = HEAP32[$28>>2]|0;
   _free($29);
   $30 = HEAP32[$itemvalue>>2]|0;
   $31 = HEAP32[$item>>2]|0;
   $32 = (($31) + 12|0);
   HEAP32[$32>>2] = $30;
   $33 = HEAP32[$itemvalue>>2]|0;
   $34 = (_strlen(($33|0))|0);
   $35 = (($34) + 1)|0;
   $36 = HEAP32[$item>>2]|0;
   $37 = (($36) + 8|0);
   HEAP32[$37>>2] = $35;
   $38 = HEAP32[$item>>2]|0;
   $39 = (($38) + 8|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$item>>2]|0;
   $42 = (($41) + 4|0);
   HEAP32[$42>>2] = $40;
   $43 = $4;
   $44 = HEAP32[$item>>2]|0;
   HEAP32[$44>>2] = $43;
   $45 = (_alloc_ANSI_NIST_subfield($subfield)|0);
   $ret = $45;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = HEAP32[$item>>2]|0;
    _free_ANSI_NIST_item($47);
    $48 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($48);
    $49 = $ret;
    $0 = $49;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   $50 = HEAP32[$subfield>>2]|0;
   $51 = HEAP32[$item>>2]|0;
   $52 = (_update_ANSI_NIST_subfield($50,$51)|0);
   $ret = $52;
   $53 = ($52|0)!=(0);
   if ($53) {
    $54 = HEAP32[$item>>2]|0;
    _free_ANSI_NIST_item($54);
    $55 = HEAP32[$subfield>>2]|0;
    _free_ANSI_NIST_subfield($55);
    $56 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($56);
    $57 = $ret;
    $0 = $57;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   $58 = HEAP32[$field>>2]|0;
   $59 = HEAP32[$subfield>>2]|0;
   $60 = (_update_ANSI_NIST_field($58,$59)|0);
   $ret = $60;
   $61 = ($60|0)!=(0);
   if (!($61)) {
    break;
   }
   $62 = HEAP32[$subfield>>2]|0;
   _free_ANSI_NIST_subfield($62);
   $63 = HEAP32[$field>>2]|0;
   _free_ANSI_NIST_field($63);
   $64 = $ret;
   $0 = $64;
   $128 = $0;
   STACKTOP = sp;return ($128|0);
  } else {
   $65 = (_alloc_ANSI_NIST_subfield($subfield)|0);
   $ret = $65;
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($67);
    $68 = $ret;
    $0 = $68;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   $69 = HEAP32[$field>>2]|0;
   $70 = HEAP32[$subfield>>2]|0;
   $71 = (_update_ANSI_NIST_field($69,$70)|0);
   $ret = $71;
   $72 = ($71|0)!=(0);
   if ($72) {
    $73 = HEAP32[$subfield>>2]|0;
    _free_ANSI_NIST_subfield($73);
    $74 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($74);
    $75 = $ret;
    $0 = $75;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   $i = 0;
   while(1) {
    $76 = $i;
    $77 = $4;
    $78 = ($76|0)<($77|0);
    if (!($78)) {
     label = 35;
     break;
    }
    $79 = $1;
    $80 = ($79|0)!=(0|0);
    if ($80) {
     $81 = $1;
     $82 = (_read_binary_item_data($81,$itemvalue,1)|0);
     $ret = $82;
    } else {
     $83 = $2;
     $84 = (_scan_binary_item_data($83,$itemvalue,1)|0);
     $ret = $84;
    }
    $85 = $ret;
    $86 = ($85|0)!=(0);
    if ($86) {
     label = 28;
     break;
    }
    $89 = (_alloc_ANSI_NIST_item($item)|0);
    $ret = $89;
    $90 = ($89|0)!=(0);
    if ($90) {
     label = 30;
     break;
    }
    $94 = HEAP32[$item>>2]|0;
    $95 = (($94) + 12|0);
    $96 = HEAP32[$95>>2]|0;
    _free($96);
    $97 = HEAP32[$itemvalue>>2]|0;
    $98 = HEAP32[$item>>2]|0;
    $99 = (($98) + 12|0);
    HEAP32[$99>>2] = $97;
    $100 = HEAP32[$itemvalue>>2]|0;
    $101 = (_strlen(($100|0))|0);
    $102 = HEAP32[$item>>2]|0;
    $103 = (($102) + 8|0);
    HEAP32[$103>>2] = $101;
    $104 = HEAP32[$item>>2]|0;
    $105 = (($104) + 8|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = HEAP32[$item>>2]|0;
    $108 = (($107) + 4|0);
    HEAP32[$108>>2] = $106;
    $109 = HEAP32[$item>>2]|0;
    HEAP32[$109>>2] = 1;
    $110 = HEAP32[$subfield>>2]|0;
    $111 = HEAP32[$item>>2]|0;
    $112 = (_update_ANSI_NIST_subfield($110,$111)|0);
    $ret = $112;
    $113 = ($112|0)!=(0);
    if ($113) {
     label = 32;
     break;
    }
    $118 = HEAP32[$item>>2]|0;
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP32[$field>>2]|0;
    $121 = (($120) + 12|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = (($122) + ($119))|0;
    HEAP32[$121>>2] = $123;
    $124 = $i;
    $125 = (($124) + 1)|0;
    $i = $125;
   }
   if ((label|0) == 28) {
    $87 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($87);
    $88 = $ret;
    $0 = $88;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   else if ((label|0) == 30) {
    $91 = HEAP32[$itemvalue>>2]|0;
    _free($91);
    $92 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($92);
    $93 = $ret;
    $0 = $93;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   else if ((label|0) == 32) {
    $114 = HEAP32[$item>>2]|0;
    _free_ANSI_NIST_item($114);
    $115 = HEAP32[$subfield>>2]|0;
    _free_ANSI_NIST_subfield($115);
    $116 = HEAP32[$field>>2]|0;
    _free_ANSI_NIST_field($116);
    $117 = $ret;
    $0 = $117;
    $128 = $0;
    STACKTOP = sp;return ($128|0);
   }
   else if ((label|0) == 35) {
    break;
   }
  }
 } while(0);
 $126 = HEAP32[$field>>2]|0;
 $127 = $3;
 HEAP32[$127>>2] = $126;
 $0 = 0;
 $128 = $0;
 STACKTOP = sp;return ($128|0);
}
function _write_ANSI_NIST_file($ofile,$ansi_nist) {
 $ofile = $ofile|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fpout = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofile;
 $2 = $ansi_nist;
 $3 = $1;
 $4 = (_fopen(($3|0),(9264|0))|0);
 $fpout = $4;
 $5 = $fpout;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $1;
  HEAP32[$vararg_buffer>>2] = $8;
  (_fprintf(($7|0),(9272|0),($vararg_buffer|0))|0);
  $0 = -2;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $9 = $fpout;
 $10 = $2;
 $11 = (_write_ANSI_NIST($9,$10)|0);
 $ret = $11;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $fpout;
  $14 = (_fclose(($13|0))|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[_stderr>>2]|0;
   $17 = $1;
   HEAP32[$vararg_buffer1>>2] = $17;
   (_fprintf(($16|0),(9312|0),($vararg_buffer1|0))|0);
   $0 = -3;
   $24 = $0;
   STACKTOP = sp;return ($24|0);
  } else {
   $18 = $ret;
   $0 = $18;
   $24 = $0;
   STACKTOP = sp;return ($24|0);
  }
 } else {
  $19 = $fpout;
  $20 = (_fclose(($19|0))|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $1;
   HEAP32[$vararg_buffer4>>2] = $23;
   (_fprintf(($22|0),(9312|0),($vararg_buffer4|0))|0);
   $0 = -4;
   $24 = $0;
   STACKTOP = sp;return ($24|0);
  } else {
   $0 = 0;
   $24 = $0;
   STACKTOP = sp;return ($24|0);
  }
 }
 return 0|0;
}
function _write_ANSI_NIST($fpout,$ansi_nist) {
 $fpout = $fpout|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $record_i = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $fpout;
 $2 = $ansi_nist;
 $record_i = 0;
 while(1) {
  $3 = $record_i;
  $4 = $2;
  $5 = (($4) + 8|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = $1;
  $9 = $record_i;
  $10 = $2;
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_write_ANSI_NIST_record($8,$14)|0);
  $ret = $15;
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 4;
   break;
  }
  $18 = $record_i;
  $19 = (($18) + 1)|0;
  $record_i = $19;
 }
 if ((label|0) == 4) {
  $17 = $ret;
  $0 = $17;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _write_ANSI_NIST_record($fpout,$record) {
 $fpout = $fpout|0;
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field_i = 0, $ret = 0, $vararg_buffer = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $fpout;
 $2 = $record;
 $3 = $2;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_tagged_record($4)|0);
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   $field_i = 0;
   while(1) {
    $7 = $field_i;
    $8 = $2;
    $9 = (($8) + 12|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = ($7|0)<($10|0);
    if (!($11)) {
     break;
    }
    $12 = $1;
    $13 = $field_i;
    $14 = $2;
    $15 = (($14) + 20|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + ($13<<2)|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (_write_ANSI_NIST_tagged_field($12,$18)|0);
    $ret = $19;
    $20 = ($19|0)!=(0);
    if ($20) {
     label = 5;
     break;
    }
    $22 = $field_i;
    $23 = (($22) + 1)|0;
    $field_i = $23;
   }
   if ((label|0) == 5) {
    $21 = $ret;
    $0 = $21;
    $56 = $0;
    STACKTOP = sp;return ($56|0);
   }
   $24 = $2;
   $25 = (($24) + 24|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0);
   do {
    if ($27) {
     $28 = $1;
     $29 = (_write_ANSI_NIST_separator($28,28)|0);
     $ret = $29;
     $30 = ($29|0)!=(0);
     if (!($30)) {
      break;
     }
     $31 = $ret;
     $0 = $31;
     $56 = $0;
     STACKTOP = sp;return ($56|0);
    }
   } while(0);
  } else {
   $32 = $2;
   $33 = HEAP32[$32>>2]|0;
   $34 = (_binary_record($33)|0);
   $35 = ($34|0)!=(0);
   if (!($35)) {
    $53 = HEAP32[_stderr>>2]|0;
    $54 = $2;
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$vararg_buffer>>2] = $55;
    (_fprintf(($53|0),(9352|0),($vararg_buffer|0))|0);
    $0 = -2;
    $56 = $0;
    STACKTOP = sp;return ($56|0);
   }
   $field_i = 0;
   while(1) {
    $36 = $field_i;
    $37 = $2;
    $38 = (($37) + 12|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)<($39|0);
    if (!($40)) {
     label = 20;
     break;
    }
    $41 = $1;
    $42 = $field_i;
    $43 = $2;
    $44 = (($43) + 20|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (($45) + ($42<<2)|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = (_write_ANSI_NIST_binary_field($41,$47)|0);
    $ret = $48;
    $49 = ($48|0)!=(0);
    if ($49) {
     label = 17;
     break;
    }
    $51 = $field_i;
    $52 = (($51) + 1)|0;
    $field_i = $52;
   }
   if ((label|0) == 17) {
    $50 = $ret;
    $0 = $50;
    $56 = $0;
    STACKTOP = sp;return ($56|0);
   }
   else if ((label|0) == 20) {
    break;
   }
  }
 } while(0);
 $0 = 0;
 $56 = $0;
 STACKTOP = sp;return ($56|0);
}
function _write_ANSI_NIST_tagged_field($fpout,$field) {
 $fpout = $fpout|0;
 $field = $field|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ret = 0, $subfield_i = 0, $vararg_buffer = 0, $vararg_buffer2 = 0;
 var $vararg_ptr1 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $fpout;
 $2 = $field;
 $3 = $2;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  $7 = $2;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $2;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$vararg_buffer>>2] = $9;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $12;
  (_fprintf(($6|0),(9416|0),($vararg_buffer|0))|0);
  $0 = -2;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $13 = $2;
 $14 = HEAP32[$13>>2]|0;
 $15 = $2;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_strlen(($16|0))|0);
 $18 = $1;
 $19 = (_fwrite(($14|0),1,($17|0),($18|0))|0);
 $20 = $2;
 $21 = HEAP32[$20>>2]|0;
 $22 = (_strlen(($21|0))|0);
 $23 = ($19|0)!=($22|0);
 if ($23) {
  $24 = HEAP32[_stderr>>2]|0;
  $25 = $2;
  $26 = HEAP32[$25>>2]|0;
  $27 = (___errno_location()|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (_strerror(($28|0))|0);
  HEAP32[$vararg_buffer2>>2] = $26;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $29;
  (_fprintf(($24|0),(9504|0),($vararg_buffer2|0))|0);
  $0 = -2;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $subfield_i = 0;
 while(1) {
  $30 = $subfield_i;
  $31 = $2;
  $32 = (($31) + 16|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($30|0)<($33|0);
  if (!($34)) {
   break;
  }
  $35 = $1;
  $36 = $subfield_i;
  $37 = $2;
  $38 = (($37) + 24|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + ($36<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (_write_ANSI_NIST_tagged_subfield($35,$41)|0);
  $ret = $42;
  $43 = ($42|0)!=(0);
  if ($43) {
   label = 8;
   break;
  }
  $45 = $subfield_i;
  $46 = (($45) + 1)|0;
  $subfield_i = $46;
 }
 if ((label|0) == 8) {
  $44 = $ret;
  $0 = $44;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $47 = $2;
 $48 = (($47) + 28|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($49|0)!=(0);
 do {
  if ($50) {
   $51 = $1;
   $52 = (_write_ANSI_NIST_separator($51,29)|0);
   $ret = $52;
   $53 = ($52|0)!=(0);
   if (!($53)) {
    break;
   }
   $54 = $ret;
   $0 = $54;
   $55 = $0;
   STACKTOP = sp;return ($55|0);
  }
 } while(0);
 $0 = 0;
 $55 = $0;
 STACKTOP = sp;return ($55|0);
}
function _write_ANSI_NIST_separator($fpout,$sep_char) {
 $fpout = $fpout|0;
 $sep_char = $sep_char|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $2 = sp + 24|0;
 $1 = $fpout;
 HEAP8[$2>>0] = $sep_char;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = (_is_delimiter($4)|0);
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = HEAP8[$2>>0]|0;
  $9 = $8 << 24 >> 24;
  HEAP32[$vararg_buffer>>2] = $9;
  (_fprintf(($7|0),(9648|0),($vararg_buffer|0))|0);
  $0 = -2;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 $10 = $1;
 $11 = (_fwrite(($2|0),1,1,($10|0))|0);
 $12 = ($11|0)!=(1);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  $14 = HEAP8[$2>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = (___errno_location()|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (_strerror(($17|0))|0);
  HEAP32[$vararg_buffer1>>2] = $15;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $18;
  (_fprintf(($13|0),(9712|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 } else {
  $0 = 0;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _write_ANSI_NIST_binary_field($fpout,$field) {
 $fpout = $fpout|0;
 $field = $field|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ret = 0, $subfield_i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $fpout;
 $2 = $field;
 $subfield_i = 0;
 while(1) {
  $3 = $subfield_i;
  $4 = $2;
  $5 = (($4) + 16|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = $1;
  $9 = $subfield_i;
  $10 = $2;
  $11 = (($10) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_write_ANSI_NIST_binary_subfield($8,$14)|0);
  $ret = $15;
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 4;
   break;
  }
  $18 = $subfield_i;
  $19 = (($18) + 1)|0;
  $subfield_i = $19;
 }
 if ((label|0) == 4) {
  $17 = $ret;
  $0 = $17;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _write_ANSI_NIST_tagged_subfield($fpout,$subfield) {
 $fpout = $fpout|0;
 $subfield = $subfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item_i = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $fpout;
 $2 = $subfield;
 $item_i = 0;
 while(1) {
  $3 = $item_i;
  $4 = $2;
  $5 = (($4) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $1;
  $9 = $item_i;
  $10 = $2;
  $11 = (($10) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_write_ANSI_NIST_tagged_item($8,$14)|0);
  $ret = $15;
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 4;
   break;
  }
  $18 = $item_i;
  $19 = (($18) + 1)|0;
  $item_i = $19;
 }
 if ((label|0) == 4) {
  $17 = $ret;
  $0 = $17;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 $20 = $2;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0);
 do {
  if ($23) {
   $24 = $1;
   $25 = (_write_ANSI_NIST_separator($24,30)|0);
   $ret = $25;
   $26 = ($25|0)!=(0);
   if (!($26)) {
    break;
   }
   $27 = $ret;
   $0 = $27;
   $28 = $0;
   STACKTOP = sp;return ($28|0);
  }
 } while(0);
 $0 = 0;
 $28 = $0;
 STACKTOP = sp;return ($28|0);
}
function _write_ANSI_NIST_tagged_item($fpout,$item) {
 $fpout = $fpout|0;
 $item = $item|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $fpout;
 $2 = $item;
 $3 = $2;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $2;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $1;
 $10 = (_fwrite(($5|0),1,($8|0),($9|0))|0);
 $n = $10;
 $11 = $n;
 $12 = $2;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)!=($14|0);
 if ($15) {
  $16 = HEAP32[_stderr>>2]|0;
  $17 = $n;
  $18 = $2;
  $19 = (($18) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (___errno_location()|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (_strerror(($22|0))|0);
  HEAP32[$vararg_buffer>>2] = $17;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $20;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $23;
  (_fprintf(($16|0),(9560|0),($vararg_buffer|0))|0);
  $0 = -2;
  $32 = $0;
  STACKTOP = sp;return ($32|0);
 }
 $24 = $2;
 $25 = (($24) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($26|0)!=(0);
 do {
  if ($27) {
   $28 = $1;
   $29 = (_write_ANSI_NIST_separator($28,31)|0);
   $ret = $29;
   $30 = ($29|0)!=(0);
   if (!($30)) {
    break;
   }
   $31 = $ret;
   $0 = $31;
   $32 = $0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 $0 = 0;
 $32 = $0;
 STACKTOP = sp;return ($32|0);
}
function _write_ANSI_NIST_binary_subfield($fpout,$subfield) {
 $fpout = $fpout|0;
 $subfield = $subfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $item_i = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $fpout;
 $2 = $subfield;
 $item_i = 0;
 while(1) {
  $3 = $item_i;
  $4 = $2;
  $5 = (($4) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = $1;
  $9 = $item_i;
  $10 = $2;
  $11 = (($10) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_write_ANSI_NIST_binary_item($8,$14)|0);
  $ret = $15;
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 4;
   break;
  }
  $18 = $item_i;
  $19 = (($18) + 1)|0;
  $item_i = $19;
 }
 if ((label|0) == 4) {
  $17 = $ret;
  $0 = $17;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _write_ANSI_NIST_binary_item($fpout,$item) {
 $fpout = $fpout|0;
 $item = $item|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_b_ = 0, $_b_1 = 0, $_f_ = 0, $_f_2 = 0, $_t_ = 0, $_t_3 = 0, $n = 0, $uchar_val = 0, $uint_val = 0, $ushort_val = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer17 = 0, $vararg_buffer21 = 0;
 var $vararg_buffer24 = 0, $vararg_buffer27 = 0, $vararg_buffer31 = 0, $vararg_buffer36 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr20 = 0, $vararg_ptr30 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer36 = sp + 96|0;
 $vararg_buffer31 = sp + 80|0;
 $vararg_buffer27 = sp + 72|0;
 $vararg_buffer24 = sp + 64|0;
 $vararg_buffer21 = sp + 56|0;
 $vararg_buffer17 = sp + 48|0;
 $vararg_buffer14 = sp + 40|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $uint_val = sp + 128|0;
 $ushort_val = sp + 150|0;
 $uchar_val = sp + 152|0;
 $_b_ = sp + 124|0;
 $_b_1 = sp + 148|0;
 $1 = $fpout;
 $2 = $item;
 $3 = $2;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  $7 = $2;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$vararg_buffer>>2] = $8;
  (_fprintf(($6|0),(9776|0),($vararg_buffer|0))|0);
  $0 = -2;
  $123 = $0;
  STACKTOP = sp;return ($123|0);
 }
 $9 = $2;
 $10 = HEAP32[$9>>2]|0;
 do {
  if ((($10|0) == 2)) {
   $47 = $2;
   $48 = (($47) + 12|0);
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$vararg_buffer11>>2] = $ushort_val;
   $50 = (_sscanf($49,10024,$vararg_buffer11)|0);
   $51 = ($50|0)!=(1);
   if ($51) {
    $52 = HEAP32[_stderr>>2]|0;
    $53 = $2;
    $54 = (($53) + 12|0);
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$vararg_buffer14>>2] = $55;
    (_fprintf(($52|0),(10032|0),($vararg_buffer14|0))|0);
    $0 = -31;
    $123 = $0;
    STACKTOP = sp;return ($123|0);
   }
   $56 = HEAP16[$ushort_val>>1]|0;
   HEAP16[$_b_1>>1] = $56;
   $_f_2 = $_b_1;
   $_t_3 = $ushort_val;
   $57 = $_f_2;
   $58 = HEAP8[$57>>0]|0;
   $59 = $_t_3;
   $60 = (($59) + 1|0);
   HEAP8[$60>>0] = $58;
   $61 = $_f_2;
   $62 = (($61) + 1|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = $_t_3;
   HEAP8[$64>>0] = $63;
   $65 = $1;
   $66 = (_fwrite(($ushort_val|0),2,1,($65|0))|0);
   $67 = ($66|0)!=(1);
   if (!($67)) {
    break;
   }
   $68 = HEAP32[_stderr>>2]|0;
   $69 = HEAP16[$ushort_val>>1]|0;
   $70 = $69&65535;
   $71 = (___errno_location()|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (_strerror(($72|0))|0);
   HEAP32[$vararg_buffer17>>2] = $70;
   $vararg_ptr20 = (($vararg_buffer17) + 4|0);
   HEAP32[$vararg_ptr20>>2] = $73;
   (_fprintf(($68|0),(10112|0),($vararg_buffer17|0))|0);
   $0 = -4;
   $123 = $0;
   STACKTOP = sp;return ($123|0);
  } else if ((($10|0) == 1)) {
   $74 = $2;
   $75 = (($74) + 12|0);
   $76 = HEAP32[$75>>2]|0;
   HEAP32[$vararg_buffer21>>2] = $ushort_val;
   $77 = (_sscanf($76,10024,$vararg_buffer21)|0);
   $78 = ($77|0)!=(1);
   if ($78) {
    $79 = HEAP32[_stderr>>2]|0;
    $80 = $2;
    $81 = (($80) + 12|0);
    $82 = HEAP32[$81>>2]|0;
    HEAP32[$vararg_buffer24>>2] = $82;
    (_fprintf(($79|0),(10200|0),($vararg_buffer24|0))|0);
    $0 = -41;
    $123 = $0;
    STACKTOP = sp;return ($123|0);
   }
   $83 = HEAP16[$ushort_val>>1]|0;
   $84 = $83&65535;
   $85 = ($84|0)>(255);
   if ($85) {
    $86 = HEAP32[_stderr>>2]|0;
    $87 = HEAP16[$ushort_val>>1]|0;
    $88 = $87&65535;
    HEAP32[$vararg_buffer27>>2] = $88;
    $vararg_ptr30 = (($vararg_buffer27) + 4|0);
    HEAP32[$vararg_ptr30>>2] = 255;
    (_fprintf(($86|0),(10280|0),($vararg_buffer27|0))|0);
   }
   $89 = HEAP16[$ushort_val>>1]|0;
   $90 = $89&255;
   HEAP8[$uchar_val>>0] = $90;
   $91 = $1;
   $92 = (_fwrite(($uchar_val|0),1,1,($91|0))|0);
   $93 = ($92|0)!=(1);
   if (!($93)) {
    break;
   }
   $94 = HEAP32[_stderr>>2]|0;
   $95 = HEAP8[$uchar_val>>0]|0;
   $96 = $95&255;
   $97 = HEAP8[$uchar_val>>0]|0;
   $98 = $97&255;
   $99 = (___errno_location()|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (_strerror(($100|0))|0);
   HEAP32[$vararg_buffer31>>2] = $96;
   $vararg_ptr34 = (($vararg_buffer31) + 4|0);
   HEAP32[$vararg_ptr34>>2] = $98;
   $vararg_ptr35 = (($vararg_buffer31) + 8|0);
   HEAP32[$vararg_ptr35>>2] = $101;
   (_fprintf(($94|0),(10376|0),($vararg_buffer31|0))|0);
   $0 = -5;
   $123 = $0;
   STACKTOP = sp;return ($123|0);
  } else if ((($10|0) == 4)) {
   $11 = $2;
   $12 = (($11) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $uint_val;
   $14 = (_sscanf($13,9848,$vararg_buffer1)|0);
   $15 = ($14|0)!=(1);
   if ($15) {
    $16 = HEAP32[_stderr>>2]|0;
    $17 = $2;
    $18 = (($17) + 12|0);
    $19 = HEAP32[$18>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $19;
    (_fprintf(($16|0),(9856|0),($vararg_buffer4|0))|0);
    $0 = -21;
    $123 = $0;
    STACKTOP = sp;return ($123|0);
   }
   $20 = HEAP32[$uint_val>>2]|0;
   HEAP32[$_b_>>2] = $20;
   $_f_ = $_b_;
   $_t_ = $uint_val;
   $21 = $_f_;
   $22 = HEAP8[$21>>0]|0;
   $23 = $_t_;
   $24 = (($23) + 3|0);
   HEAP8[$24>>0] = $22;
   $25 = $_f_;
   $26 = (($25) + 1|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $_t_;
   $29 = (($28) + 2|0);
   HEAP8[$29>>0] = $27;
   $30 = $_f_;
   $31 = (($30) + 2|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $_t_;
   $34 = (($33) + 1|0);
   HEAP8[$34>>0] = $32;
   $35 = $_f_;
   $36 = (($35) + 3|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $_t_;
   HEAP8[$38>>0] = $37;
   $39 = $1;
   $40 = (_fwrite(($uint_val|0),4,1,($39|0))|0);
   $41 = ($40|0)!=(1);
   if (!($41)) {
    break;
   }
   $42 = HEAP32[_stderr>>2]|0;
   $43 = HEAP32[$uint_val>>2]|0;
   $44 = (___errno_location()|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (_strerror(($45|0))|0);
   HEAP32[$vararg_buffer7>>2] = $43;
   $vararg_ptr10 = (($vararg_buffer7) + 4|0);
   HEAP32[$vararg_ptr10>>2] = $46;
   (_fprintf(($42|0),(9936|0),($vararg_buffer7|0))|0);
   $0 = -3;
   $123 = $0;
   STACKTOP = sp;return ($123|0);
  } else {
   $102 = $2;
   $103 = (($102) + 12|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = $2;
   $106 = (($105) + 4|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = $1;
   $109 = (_fwrite(($104|0),1,($107|0),($108|0))|0);
   $n = $109;
   $110 = $n;
   $111 = $2;
   $112 = (($111) + 4|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = ($110|0)!=($113|0);
   if (!($114)) {
    break;
   }
   $115 = HEAP32[_stderr>>2]|0;
   $116 = $n;
   $117 = $2;
   $118 = (($117) + 4|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = (___errno_location()|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = (_strerror(($121|0))|0);
   HEAP32[$vararg_buffer36>>2] = $116;
   $vararg_ptr39 = (($vararg_buffer36) + 4|0);
   HEAP32[$vararg_ptr39>>2] = $119;
   $vararg_ptr40 = (($vararg_buffer36) + 8|0);
   HEAP32[$vararg_ptr40>>2] = $122;
   (_fprintf(($115|0),(10472|0),($vararg_buffer36|0))|0);
   $0 = -6;
   $123 = $0;
   STACKTOP = sp;return ($123|0);
  }
 } while(0);
 $0 = 0;
 $123 = $0;
 STACKTOP = sp;return ($123|0);
}
function _get_first_grayprint($odata,$ow,$oh,$od,$oppmm,$oimg_idc,$oimg_imp,$oimgrecord,$oimgrecord_i,$ansi_nist) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppmm = $oppmm|0;
 $oimg_idc = $oimg_idc|0;
 $oimg_imp = $oimg_imp|0;
 $oimgrecord = $oimgrecord|0;
 $oimgrecord_i = $oimgrecord_i|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $id = 0, $idata = 0, $idcfield = 0, $idcfield_i = 0, $ih = 0;
 var $img_idc = 0, $img_imp = 0, $imgrecord = 0, $imgrecord_i = 0, $impfield = 0, $impfield_i = 0, $iw = 0, $ppmm = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer = sp + 8|0;
 $idata = sp + 80|0;
 $iw = sp + 76|0;
 $ih = sp + 72|0;
 $id = sp + 68|0;
 $ppmm = sp;
 $imgrecord = sp + 56|0;
 $idcfield = sp + 52|0;
 $impfield = sp + 48|0;
 $imgrecord_i = sp + 44|0;
 $idcfield_i = sp + 40|0;
 $impfield_i = sp + 36|0;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppmm;
 $6 = $oimg_idc;
 $7 = $oimg_imp;
 $8 = $oimgrecord;
 $9 = $oimgrecord_i;
 $10 = $ansi_nist;
 $11 = $10;
 $12 = (_lookup_ANSI_NIST_grayprint($imgrecord,$imgrecord_i,1,$11)|0);
 $ret = $12;
 $13 = $ret;
 $14 = ($13|0)<(0);
 if ($14) {
  $15 = $ret;
  $0 = $15;
  $74 = $0;
  STACKTOP = sp;return ($74|0);
 }
 $16 = $ret;
 $17 = ($16|0)!=(0);
 if (!($17)) {
  $0 = 0;
  $74 = $0;
  STACKTOP = sp;return ($74|0);
 }
 $18 = $10;
 $19 = HEAP32[$imgrecord_i>>2]|0;
 $20 = (_decode_ANSI_NIST_image($idata,$iw,$ih,$id,$ppmm,$18,$19,1)|0);
 $ret = $20;
 $21 = $ret;
 $22 = ($21|0)<=(0);
 if ($22) {
  $23 = $ret;
  $0 = $23;
  $74 = $0;
  STACKTOP = sp;return ($74|0);
 }
 $24 = HEAP32[$imgrecord>>2]|0;
 $25 = (_lookup_ANSI_NIST_field($idcfield,$idcfield_i,2,$24)|0);
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $27 = HEAP32[_stderr>>2]|0;
  $28 = HEAP32[$imgrecord_i>>2]|0;
  $29 = (($28) + 1)|0;
  $30 = HEAP32[$imgrecord>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$vararg_buffer>>2] = $29;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $31;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2;
  (_fprintf(($27|0),(16200|0),($vararg_buffer|0))|0);
  $0 = -2;
  $74 = $0;
  STACKTOP = sp;return ($74|0);
 }
 $32 = HEAP32[$idcfield>>2]|0;
 $33 = (($32) + 24|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (_atoi($40)|0);
 $img_idc = $41;
 $42 = HEAP32[$imgrecord>>2]|0;
 $43 = (_lookup_ANSI_NIST_field($impfield,$impfield_i,3,$42)|0);
 $44 = ($43|0)!=(0);
 if ($44) {
  $50 = HEAP32[$impfield>>2]|0;
  $51 = (($50) + 24|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + 12|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + 12|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (_atoi($58)|0);
  $img_imp = $59;
  $60 = HEAP32[$idata>>2]|0;
  $61 = $1;
  HEAP32[$61>>2] = $60;
  $62 = HEAP32[$iw>>2]|0;
  $63 = $2;
  HEAP32[$63>>2] = $62;
  $64 = HEAP32[$ih>>2]|0;
  $65 = $3;
  HEAP32[$65>>2] = $64;
  $66 = HEAP32[$id>>2]|0;
  $67 = $4;
  HEAP32[$67>>2] = $66;
  $68 = +HEAPF64[$ppmm>>3];
  $69 = $5;
  HEAPF64[$69>>3] = $68;
  $70 = $img_idc;
  $71 = $6;
  HEAP32[$71>>2] = $70;
  $72 = $img_imp;
  $73 = $7;
  HEAP32[$73>>2] = $72;
  $0 = 1;
  $74 = $0;
  STACKTOP = sp;return ($74|0);
 } else {
  $45 = HEAP32[_stderr>>2]|0;
  $46 = HEAP32[$imgrecord_i>>2]|0;
  $47 = (($46) + 1)|0;
  $48 = HEAP32[$imgrecord>>2]|0;
  $49 = HEAP32[$48>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $47;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $49;
  $vararg_ptr7 = (($vararg_buffer3) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 3;
  (_fprintf(($45|0),(16288|0),($vararg_buffer3|0))|0);
  $0 = -3;
  $74 = $0;
  STACKTOP = sp;return ($74|0);
 }
 return 0|0;
}
function _insert_ANSI_NIST_item($record_i,$field_i,$subfield_i,$item_i,$itemvalue,$ansi_nist) {
 $record_i = $record_i|0;
 $field_i = $field_i|0;
 $subfield_i = $subfield_i|0;
 $item_i = $item_i|0;
 $itemvalue = $itemvalue|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $alloc_chars = 0, $byte_adjust = 0, $field = 0, $item = 0, $j = 0, $k = 0, $new_ptr = 0, $new_ptr1 = 0, $new_size = 0, $record = 0, $ret = 0;
 var $subfield = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer16 = 0, $vararg_buffer2 = 0, $vararg_buffer24 = 0, $vararg_buffer33 = 0, $vararg_buffer37 = 0, $vararg_buffer41 = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr27 = 0;
 var $vararg_ptr28 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr40 = 0, $vararg_ptr44 = 0, $vararg_ptr45 = 0, $vararg_ptr46 = 0, $vararg_ptr47 = 0, $vararg_ptr48 = 0, $vararg_ptr49 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer41 = sp + 120|0;
 $vararg_buffer37 = sp + 112|0;
 $vararg_buffer33 = sp + 104|0;
 $vararg_buffer24 = sp + 72|0;
 $vararg_buffer16 = sp + 48|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $item = sp + 168|0;
 $1 = $record_i;
 $2 = $field_i;
 $3 = $subfield_i;
 $4 = $item_i;
 $5 = $itemvalue;
 $6 = $ansi_nist;
 $7 = $1;
 $8 = ($7|0)<(0);
 if (!($8)) {
  $9 = $1;
  $10 = $6;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)>=($12|0);
  if (!($13)) {
   $20 = $1;
   $21 = $6;
   $22 = (($21) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + ($20<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $record = $25;
   $26 = $record;
   $27 = HEAP32[$26>>2]|0;
   $28 = (_binary_record($27)|0);
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[_stderr>>2]|0;
    $31 = $record;
    $32 = HEAP32[$31>>2]|0;
    HEAP32[$vararg_buffer2>>2] = $32;
    (_fprintf(($30|0),(17816|0),($vararg_buffer2|0))|0);
    $0 = -3;
    $308 = $0;
    STACKTOP = sp;return ($308|0);
   }
   $33 = $2;
   $34 = ($33|0)<(0);
   if (!($34)) {
    $35 = $2;
    $36 = $record;
    $37 = (($36) + 12|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ($35|0)>=($38|0);
    if (!($39)) {
     $48 = $2;
     $49 = $record;
     $50 = (($49) + 20|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + ($48<<2)|0);
     $53 = HEAP32[$52>>2]|0;
     $field = $53;
     $54 = $field;
     $55 = (_image_field($54)|0);
     $56 = ($55|0)!=(0);
     if ($56) {
      $57 = HEAP32[_stderr>>2]|0;
      $58 = $1;
      $59 = (($58) + 1)|0;
      $60 = $2;
      $61 = (($60) + 1)|0;
      $62 = $record;
      $63 = HEAP32[$62>>2]|0;
      $64 = $field;
      $65 = (($64) + 8|0);
      $66 = HEAP32[$65>>2]|0;
      HEAP32[$vararg_buffer10>>2] = $59;
      $vararg_ptr13 = (($vararg_buffer10) + 4|0);
      HEAP32[$vararg_ptr13>>2] = $61;
      $vararg_ptr14 = (($vararg_buffer10) + 8|0);
      HEAP32[$vararg_ptr14>>2] = $63;
      $vararg_ptr15 = (($vararg_buffer10) + 12|0);
      HEAP32[$vararg_ptr15>>2] = $66;
      (_fprintf(($57|0),(18008|0),($vararg_buffer10|0))|0);
      $0 = -5;
      $308 = $0;
      STACKTOP = sp;return ($308|0);
     }
     $67 = $3;
     $68 = ($67|0)<(0);
     if (!($68)) {
      $69 = $3;
      $70 = $field;
      $71 = (($70) + 16|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ($69|0)>=($72|0);
      if (!($73)) {
       $89 = $3;
       $90 = $field;
       $91 = (($90) + 24|0);
       $92 = HEAP32[$91>>2]|0;
       $93 = (($92) + ($89<<2)|0);
       $94 = HEAP32[$93>>2]|0;
       $subfield = $94;
       $95 = $4;
       $96 = ($95|0)<(0);
       if (!($96)) {
        $97 = $4;
        $98 = $subfield;
        $99 = (($98) + 4|0);
        $100 = HEAP32[$99>>2]|0;
        $101 = ($97|0)>($100|0);
        if (!($101)) {
         $120 = $subfield;
         $121 = (($120) + 4|0);
         $122 = HEAP32[$121>>2]|0;
         $123 = $subfield;
         $124 = (($123) + 8|0);
         $125 = HEAP32[$124>>2]|0;
         $126 = ($122|0)>=($125|0);
         do {
          if ($126) {
           $127 = $subfield;
           $128 = (($127) + 8|0);
           $129 = HEAP32[$128>>2]|0;
           $130 = (($129) + 100)|0;
           $131 = $130<<2;
           $new_size = $131;
           $132 = $subfield;
           $133 = (($132) + 12|0);
           $134 = HEAP32[$133>>2]|0;
           $135 = $new_size;
           $136 = (_realloc($134,$135)|0);
           $new_ptr = $136;
           $137 = $new_ptr;
           $138 = ($137|0)==(0|0);
           if (!($138)) {
            $145 = $new_ptr;
            $146 = $subfield;
            $147 = (($146) + 12|0);
            HEAP32[$147>>2] = $145;
            $148 = $subfield;
            $149 = (($148) + 8|0);
            $150 = HEAP32[$149>>2]|0;
            $151 = (($150) + 100)|0;
            HEAP32[$149>>2] = $151;
            break;
           }
           $139 = HEAP32[_stderr>>2]|0;
           $140 = $subfield;
           $141 = (($140) + 8|0);
           $142 = HEAP32[$141>>2]|0;
           $143 = $142<<2;
           $144 = $new_size;
           HEAP32[$vararg_buffer33>>2] = $143;
           $vararg_ptr36 = (($vararg_buffer33) + 4|0);
           HEAP32[$vararg_ptr36>>2] = $144;
           (_fprintf(($139|0),(18336|0),($vararg_buffer33|0))|0);
           $0 = -8;
           $308 = $0;
           STACKTOP = sp;return ($308|0);
          }
         } while(0);
         $152 = (_alloc_ANSI_NIST_item($item)|0);
         $ret = $152;
         $153 = ($152|0)!=(0);
         if ($153) {
          $154 = $ret;
          $0 = $154;
          $308 = $0;
          STACKTOP = sp;return ($308|0);
         }
         $155 = $5;
         $156 = (_strlen(($155|0))|0);
         $157 = (($156) + 1)|0;
         $alloc_chars = $157;
         $158 = $alloc_chars;
         $159 = HEAP32[$item>>2]|0;
         $160 = (($159) + 8|0);
         $161 = HEAP32[$160>>2]|0;
         $162 = ($158|0)>=($161|0);
         do {
          if ($162) {
           $163 = HEAP32[$item>>2]|0;
           $164 = (($163) + 12|0);
           $165 = HEAP32[$164>>2]|0;
           $166 = $alloc_chars;
           $167 = (_realloc($165,$166)|0);
           $new_ptr1 = $167;
           $168 = $new_ptr1;
           $169 = ($168|0)==(0|0);
           if (!($169)) {
            $175 = $new_ptr1;
            $176 = HEAP32[$item>>2]|0;
            $177 = (($176) + 12|0);
            HEAP32[$177>>2] = $175;
            $178 = $alloc_chars;
            $179 = HEAP32[$item>>2]|0;
            $180 = (($179) + 8|0);
            HEAP32[$180>>2] = $178;
            break;
           }
           $170 = HEAP32[_stderr>>2]|0;
           $171 = HEAP32[$item>>2]|0;
           $172 = (($171) + 8|0);
           $173 = HEAP32[$172>>2]|0;
           $174 = $alloc_chars;
           HEAP32[$vararg_buffer37>>2] = $173;
           $vararg_ptr40 = (($vararg_buffer37) + 4|0);
           HEAP32[$vararg_ptr40>>2] = $174;
           (_fprintf(($170|0),(18424|0),($vararg_buffer37|0))|0);
           $0 = -9;
           $308 = $0;
           STACKTOP = sp;return ($308|0);
          }
         } while(0);
         $181 = HEAP32[$item>>2]|0;
         $182 = (($181) + 12|0);
         $183 = HEAP32[$182>>2]|0;
         $184 = $5;
         (_strcpy(($183|0),($184|0))|0);
         $185 = HEAP32[$item>>2]|0;
         $186 = (($185) + 12|0);
         $187 = HEAP32[$186>>2]|0;
         $188 = (_strlen(($187|0))|0);
         $189 = HEAP32[$item>>2]|0;
         $190 = (($189) + 4|0);
         HEAP32[$190>>2] = $188;
         $191 = HEAP32[$item>>2]|0;
         $192 = (($191) + 4|0);
         $193 = HEAP32[$192>>2]|0;
         $194 = HEAP32[$item>>2]|0;
         HEAP32[$194>>2] = $193;
         $195 = $subfield;
         $196 = (($195) + 4|0);
         $197 = HEAP32[$196>>2]|0;
         $198 = (($197) - 1)|0;
         $j = $198;
         $199 = $subfield;
         $200 = (($199) + 4|0);
         $201 = HEAP32[$200>>2]|0;
         $k = $201;
         while(1) {
          $202 = $j;
          $203 = $4;
          $204 = ($202|0)>=($203|0);
          if (!($204)) {
           break;
          }
          $205 = $j;
          $206 = $subfield;
          $207 = (($206) + 12|0);
          $208 = HEAP32[$207>>2]|0;
          $209 = (($208) + ($205<<2)|0);
          $210 = HEAP32[$209>>2]|0;
          $211 = $k;
          $212 = $subfield;
          $213 = (($212) + 12|0);
          $214 = HEAP32[$213>>2]|0;
          $215 = (($214) + ($211<<2)|0);
          HEAP32[$215>>2] = $210;
          $216 = $j;
          $217 = (($216) + -1)|0;
          $j = $217;
          $218 = $k;
          $219 = (($218) + -1)|0;
          $k = $219;
         }
         $220 = HEAP32[$item>>2]|0;
         $221 = $4;
         $222 = $subfield;
         $223 = (($222) + 12|0);
         $224 = HEAP32[$223>>2]|0;
         $225 = (($224) + ($221<<2)|0);
         HEAP32[$225>>2] = $220;
         $226 = $subfield;
         $227 = (($226) + 4|0);
         $228 = HEAP32[$227>>2]|0;
         $229 = (($228) + 1)|0;
         HEAP32[$227>>2] = $229;
         $230 = HEAP32[$item>>2]|0;
         $231 = (($230) + 4|0);
         $232 = HEAP32[$231>>2]|0;
         $byte_adjust = $232;
         $233 = $subfield;
         $234 = (($233) + 4|0);
         $235 = HEAP32[$234>>2]|0;
         $236 = ($235|0)>(1);
         if ($236) {
          $237 = $4;
          $238 = $subfield;
          $239 = (($238) + 4|0);
          $240 = HEAP32[$239>>2]|0;
          $241 = (($240) - 1)|0;
          $242 = ($237|0)==($241|0);
          if ($242) {
           $243 = $4;
           $244 = (($243) - 1)|0;
           $245 = $subfield;
           $246 = (($245) + 12|0);
           $247 = HEAP32[$246>>2]|0;
           $248 = (($247) + ($244<<2)|0);
           $249 = HEAP32[$248>>2]|0;
           $250 = (($249) + 16|0);
           HEAP32[$250>>2] = 1;
           $251 = $4;
           $252 = (($251) - 1)|0;
           $253 = $subfield;
           $254 = (($253) + 12|0);
           $255 = HEAP32[$254>>2]|0;
           $256 = (($255) + ($252<<2)|0);
           $257 = HEAP32[$256>>2]|0;
           $258 = HEAP32[$257>>2]|0;
           $259 = (($258) + 1)|0;
           HEAP32[$257>>2] = $259;
          } else {
           $260 = HEAP32[$item>>2]|0;
           $261 = (($260) + 16|0);
           HEAP32[$261>>2] = 1;
           $262 = HEAP32[$item>>2]|0;
           $263 = HEAP32[$262>>2]|0;
           $264 = (($263) + 1)|0;
           HEAP32[$262>>2] = $264;
          }
          $265 = $byte_adjust;
          $266 = (($265) + 1)|0;
          $byte_adjust = $266;
         }
         $267 = $byte_adjust;
         $268 = $subfield;
         $269 = HEAP32[$268>>2]|0;
         $270 = (($269) + ($267))|0;
         HEAP32[$268>>2] = $270;
         $271 = $byte_adjust;
         $272 = $field;
         $273 = (($272) + 12|0);
         $274 = HEAP32[$273>>2]|0;
         $275 = (($274) + ($271))|0;
         HEAP32[$273>>2] = $275;
         $276 = $byte_adjust;
         $277 = $record;
         $278 = (($277) + 8|0);
         $279 = HEAP32[$278>>2]|0;
         $280 = (($279) + ($276))|0;
         HEAP32[$278>>2] = $280;
         $281 = $byte_adjust;
         $282 = $6;
         $283 = (($282) + 4|0);
         $284 = HEAP32[$283>>2]|0;
         $285 = (($284) + ($281))|0;
         HEAP32[$283>>2] = $285;
         $286 = $6;
         $287 = $1;
         $288 = (_update_ANSI_NIST_record_LEN($286,$287)|0);
         $ret = $288;
         $289 = ($288|0)!=(0);
         if ($289) {
          $290 = $ret;
          $0 = $290;
          $308 = $0;
          STACKTOP = sp;return ($308|0);
         } else {
          $291 = HEAP32[_stderr>>2]|0;
          $292 = $1;
          $293 = (($292) + 1)|0;
          $294 = $2;
          $295 = (($294) + 1)|0;
          $296 = $3;
          $297 = (($296) + 1)|0;
          $298 = $4;
          $299 = (($298) + 1)|0;
          $300 = $record;
          $301 = HEAP32[$300>>2]|0;
          $302 = $field;
          $303 = (($302) + 8|0);
          $304 = HEAP32[$303>>2]|0;
          $305 = HEAP32[$item>>2]|0;
          $306 = (($305) + 12|0);
          $307 = HEAP32[$306>>2]|0;
          HEAP32[$vararg_buffer41>>2] = $293;
          $vararg_ptr44 = (($vararg_buffer41) + 4|0);
          HEAP32[$vararg_ptr44>>2] = $295;
          $vararg_ptr45 = (($vararg_buffer41) + 8|0);
          HEAP32[$vararg_ptr45>>2] = $297;
          $vararg_ptr46 = (($vararg_buffer41) + 12|0);
          HEAP32[$vararg_ptr46>>2] = $299;
          $vararg_ptr47 = (($vararg_buffer41) + 16|0);
          HEAP32[$vararg_ptr47>>2] = $301;
          $vararg_ptr48 = (($vararg_buffer41) + 20|0);
          HEAP32[$vararg_ptr48>>2] = $304;
          $vararg_ptr49 = (($vararg_buffer41) + 24|0);
          HEAP32[$vararg_ptr49>>2] = $307;
          (_fprintf(($291|0),(18504|0),($vararg_buffer41|0))|0);
          $0 = 0;
          $308 = $0;
          STACKTOP = sp;return ($308|0);
         }
        }
       }
       $102 = HEAP32[_stderr>>2]|0;
       $103 = $1;
       $104 = (($103) + 1)|0;
       $105 = $2;
       $106 = (($105) + 1)|0;
       $107 = $3;
       $108 = (($107) + 1)|0;
       $109 = $4;
       $110 = (($109) + 1)|0;
       $111 = $subfield;
       $112 = (($111) + 4|0);
       $113 = HEAP32[$112>>2]|0;
       $114 = (($113) + 1)|0;
       $115 = $record;
       $116 = HEAP32[$115>>2]|0;
       $117 = $field;
       $118 = (($117) + 8|0);
       $119 = HEAP32[$118>>2]|0;
       HEAP32[$vararg_buffer24>>2] = $104;
       $vararg_ptr27 = (($vararg_buffer24) + 4|0);
       HEAP32[$vararg_ptr27>>2] = $106;
       $vararg_ptr28 = (($vararg_buffer24) + 8|0);
       HEAP32[$vararg_ptr28>>2] = $108;
       $vararg_ptr29 = (($vararg_buffer24) + 12|0);
       HEAP32[$vararg_ptr29>>2] = $110;
       $vararg_ptr30 = (($vararg_buffer24) + 16|0);
       HEAP32[$vararg_ptr30>>2] = $114;
       $vararg_ptr31 = (($vararg_buffer24) + 20|0);
       HEAP32[$vararg_ptr31>>2] = $116;
       $vararg_ptr32 = (($vararg_buffer24) + 24|0);
       HEAP32[$vararg_ptr32>>2] = $119;
       (_fprintf(($102|0),(18232|0),($vararg_buffer24|0))|0);
       $0 = -7;
       $308 = $0;
       STACKTOP = sp;return ($308|0);
      }
     }
     $74 = HEAP32[_stderr>>2]|0;
     $75 = $1;
     $76 = (($75) + 1)|0;
     $77 = $2;
     $78 = (($77) + 1)|0;
     $79 = $3;
     $80 = (($79) + 1)|0;
     $81 = $field;
     $82 = (($81) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = $record;
     $85 = HEAP32[$84>>2]|0;
     $86 = $field;
     $87 = (($86) + 8|0);
     $88 = HEAP32[$87>>2]|0;
     HEAP32[$vararg_buffer16>>2] = $76;
     $vararg_ptr19 = (($vararg_buffer16) + 4|0);
     HEAP32[$vararg_ptr19>>2] = $78;
     $vararg_ptr20 = (($vararg_buffer16) + 8|0);
     HEAP32[$vararg_ptr20>>2] = $80;
     $vararg_ptr21 = (($vararg_buffer16) + 12|0);
     HEAP32[$vararg_ptr21>>2] = $83;
     $vararg_ptr22 = (($vararg_buffer16) + 16|0);
     HEAP32[$vararg_ptr22>>2] = $85;
     $vararg_ptr23 = (($vararg_buffer16) + 20|0);
     HEAP32[$vararg_ptr23>>2] = $88;
     (_fprintf(($74|0),(18120|0),($vararg_buffer16|0))|0);
     $0 = -6;
     $308 = $0;
     STACKTOP = sp;return ($308|0);
    }
   }
   $40 = HEAP32[_stderr>>2]|0;
   $41 = $2;
   $42 = (($41) + 1)|0;
   $43 = $record;
   $44 = (($43) + 12|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = $record;
   $47 = HEAP32[$46>>2]|0;
   HEAP32[$vararg_buffer5>>2] = $42;
   $vararg_ptr8 = (($vararg_buffer5) + 4|0);
   HEAP32[$vararg_ptr8>>2] = $45;
   $vararg_ptr9 = (($vararg_buffer5) + 8|0);
   HEAP32[$vararg_ptr9>>2] = $47;
   (_fprintf(($40|0),(17912|0),($vararg_buffer5|0))|0);
   $0 = -4;
   $308 = $0;
   STACKTOP = sp;return ($308|0);
  }
 }
 $14 = HEAP32[_stderr>>2]|0;
 $15 = $1;
 $16 = (($15) + 1)|0;
 $17 = $6;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer>>2] = $16;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $19;
 (_fprintf(($14|0),(17744|0),($vararg_buffer|0))|0);
 $0 = -2;
 $308 = $0;
 STACKTOP = sp;return ($308|0);
}
function _insert_ANSI_NIST_record_core($record_i,$record,$new_idc,$ansi_nist) {
 $record_i = $record_i|0;
 $record = $record|0;
 $new_idc = $new_idc|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $j = 0, $k = 0, $new_ptr = 0, $new_size = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $record_i;
 $2 = $record;
 $3 = $new_idc;
 $4 = $ansi_nist;
 $5 = $2;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(1);
 if ($7) {
  $8 = HEAP32[_stderr>>2]|0;
  (_fprintf(($8|0),(16416|0),($vararg_buffer|0))|0);
  $0 = -3;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 $9 = $1;
 $10 = ($9|0)<(0);
 if (!($10)) {
  $11 = $1;
  $12 = $4;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)>($14|0);
  if (!($15)) {
   $23 = $4;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $4;
   $27 = (($26) + 12|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($25|0)>=($28|0);
   do {
    if ($29) {
     $30 = $4;
     $31 = (($30) + 12|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = (($32) + 100)|0;
     $34 = $33<<2;
     $new_size = $34;
     $35 = $4;
     $36 = (($35) + 16|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = $new_size;
     $39 = (_realloc($37,$38)|0);
     $new_ptr = $39;
     $40 = $new_ptr;
     $41 = ($40|0)==(0|0);
     if (!($41)) {
      $48 = $new_ptr;
      $49 = $4;
      $50 = (($49) + 16|0);
      HEAP32[$50>>2] = $48;
      $51 = $4;
      $52 = (($51) + 12|0);
      $53 = HEAP32[$52>>2]|0;
      $54 = (($53) + 100)|0;
      HEAP32[$52>>2] = $54;
      break;
     }
     $42 = HEAP32[_stderr>>2]|0;
     $43 = $4;
     $44 = (($43) + 12|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = $45<<2;
     $47 = $new_size;
     HEAP32[$vararg_buffer4>>2] = $46;
     $vararg_ptr7 = (($vararg_buffer4) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $47;
     (_fprintf(($42|0),(16576|0),($vararg_buffer4|0))|0);
     $0 = -5;
     $101 = $0;
     STACKTOP = sp;return ($101|0);
    }
   } while(0);
   $55 = $4;
   $56 = (($55) + 8|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) - 1)|0;
   $j = $58;
   $59 = $4;
   $60 = (($59) + 8|0);
   $61 = HEAP32[$60>>2]|0;
   $k = $61;
   while(1) {
    $62 = $j;
    $63 = $1;
    $64 = ($62|0)>=($63|0);
    if (!($64)) {
     break;
    }
    $65 = $j;
    $66 = $4;
    $67 = (($66) + 16|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + ($65<<2)|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = $k;
    $72 = $4;
    $73 = (($72) + 16|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + ($71<<2)|0);
    HEAP32[$75>>2] = $70;
    $76 = $j;
    $77 = (($76) + -1)|0;
    $j = $77;
    $78 = $k;
    $79 = (($78) + -1)|0;
    $k = $79;
   }
   $80 = $2;
   $81 = $1;
   $82 = $4;
   $83 = (($82) + 16|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + ($81<<2)|0);
   HEAP32[$85>>2] = $80;
   $86 = $4;
   $87 = (($86) + 8|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = (($88) + 1)|0;
   HEAP32[$87>>2] = $89;
   $90 = $2;
   $91 = (($90) + 8|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = $4;
   $94 = (($93) + 4|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (($95) + ($92))|0;
   HEAP32[$94>>2] = $96;
   $97 = $1;
   $98 = $3;
   $99 = $4;
   $100 = (_adjust_insrec_CNT_IDCs($97,$98,$99)|0);
   $0 = $100;
   $101 = $0;
   STACKTOP = sp;return ($101|0);
  }
 }
 $16 = HEAP32[_stderr>>2]|0;
 $17 = $1;
 $18 = (($17) + 1)|0;
 $19 = $4;
 $20 = (($19) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$vararg_buffer1>>2] = $18;
 $vararg_ptr3 = (($vararg_buffer1) + 4|0);
 HEAP32[$vararg_ptr3>>2] = $22;
 (_fprintf(($16|0),(16496|0),($vararg_buffer1|0))|0);
 $0 = -4;
 $101 = $0;
 STACKTOP = sp;return ($101|0);
}
function _insert_ANSI_NIST_record_frmem($record_i,$record,$ansi_nist) {
 $record_i = $record_i|0;
 $record = $record|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $record_i;
 $2 = $record;
 $3 = $ansi_nist;
 $4 = $1;
 $5 = $2;
 $6 = $3;
 $7 = (_insert_ANSI_NIST_record_core($4,$5,0,$6)|0);
 $ret = $7;
 $8 = ($7|0)<(0);
 if ($8) {
  $9 = $ret;
  $0 = $9;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 } else {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $1;
  $12 = (($11) + 1)|0;
  $13 = $1;
  $14 = $3;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$vararg_buffer>>2] = $12;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $19;
  (_fprintf(($10|0),(16376|0),($vararg_buffer|0))|0);
  $0 = 0;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _adjust_insrec_CNT_IDCs($record_i,$new_idc,$ansi_nist) {
 $record_i = $record_i|0;
 $new_idc = $new_idc|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $byte_adjust = 0, $cntfield = 0, $cntfield_i = 0, $idcfield = 0, $idcfield_i = 0, $insidc = 0, $item = 0, $itemint = 0, $j = 0, $k = 0, $maxv = 0, $new_ptr = 0, $new_size = 0, $numstr = 0, $ret = 0, $subfield = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer17 = 0;
 var $vararg_buffer21 = 0, $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer31 = 0, $vararg_buffer35 = 0, $vararg_buffer40 = 0, $vararg_buffer43 = 0, $vararg_buffer47 = 0, $vararg_buffer52 = 0, $vararg_buffer6 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr20 = 0, $vararg_ptr24 = 0, $vararg_ptr34 = 0, $vararg_ptr38 = 0, $vararg_ptr39 = 0;
 var $vararg_ptr46 = 0, $vararg_ptr50 = 0, $vararg_ptr51 = 0, $vararg_ptr55 = 0, $vararg_ptr56 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer52 = sp + 144|0;
 $vararg_buffer47 = sp + 128|0;
 $vararg_buffer43 = sp + 120|0;
 $vararg_buffer40 = sp + 112|0;
 $vararg_buffer35 = sp + 96|0;
 $vararg_buffer31 = sp + 88|0;
 $vararg_buffer28 = sp + 80|0;
 $vararg_buffer25 = sp + 72|0;
 $vararg_buffer21 = sp + 64|0;
 $vararg_buffer17 = sp + 56|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $cntfield = sp + 208|0;
 $idcfield = sp + 204|0;
 $subfield = sp + 200|0;
 $item = sp + 196|0;
 $cntfield_i = sp + 192|0;
 $idcfield_i = sp + 188|0;
 $numstr = sp + 232|0;
 $1 = $record_i;
 $2 = $new_idc;
 $3 = $ansi_nist;
 $4 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer>>2] = 3;
 (_fprintf(($4|0),(16672|0),($vararg_buffer|0))|0);
 $5 = $3;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<=(0);
 if (!($8)) {
  $9 = $3;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(1);
  if (!($14)) {
   $16 = $3;
   $17 = (($16) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = (_lookup_ANSI_NIST_field($cntfield,$cntfield_i,3,$19)|0);
   $21 = ($20|0)!=(0);
   if (!($21)) {
    $22 = HEAP32[_stderr>>2]|0;
    HEAP32[$vararg_buffer3>>2] = 3;
    (_fprintf(($22|0),(16776|0),($vararg_buffer3|0))|0);
    $0 = -3;
    $359 = $0;
    STACKTOP = sp;return ($359|0);
   }
   $23 = $1;
   $24 = ($23|0)<(0);
   if (!($24)) {
    $25 = $1;
    $26 = HEAP32[$cntfield>>2]|0;
    $27 = (($26) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = ($25|0)>($28|0);
    if (!($29)) {
     $39 = $2;
     $40 = ($39|0)!=(0);
     do {
      if ($40) {
       $maxv = -1;
       $j = 1;
       while(1) {
        $41 = $j;
        $42 = $1;
        $43 = ($41|0)<($42|0);
        if (!($43)) {
         break;
        }
        $44 = $j;
        $45 = HEAP32[$cntfield>>2]|0;
        $46 = (($45) + 24|0);
        $47 = HEAP32[$46>>2]|0;
        $48 = (($47) + ($44<<2)|0);
        $49 = HEAP32[$48>>2]|0;
        $50 = (_lookup_ANSI_NIST_item($item,1,$49)|0);
        $51 = ($50|0)!=(0);
        if (!($51)) {
         label = 13;
         break;
        }
        $57 = HEAP32[$item>>2]|0;
        $58 = (($57) + 12|0);
        $59 = HEAP32[$58>>2]|0;
        $60 = (_atoi($59)|0);
        $itemint = $60;
        $61 = $itemint;
        $62 = $maxv;
        $63 = ($61|0)>($62|0);
        if ($63) {
         $64 = $itemint;
         $maxv = $64;
        }
        $65 = $j;
        $66 = (($65) + 1)|0;
        $j = $66;
       }
       if ((label|0) == 13) {
        $52 = HEAP32[_stderr>>2]|0;
        $53 = HEAP32[$cntfield_i>>2]|0;
        $54 = (($53) + 1)|0;
        $55 = $j;
        $56 = (($55) + 1)|0;
        HEAP32[$vararg_buffer12>>2] = $54;
        $vararg_ptr15 = (($vararg_buffer12) + 4|0);
        HEAP32[$vararg_ptr15>>2] = $56;
        $vararg_ptr16 = (($vararg_buffer12) + 8|0);
        HEAP32[$vararg_ptr16>>2] = 3;
        (_fprintf(($52|0),(16992|0),($vararg_buffer12|0))|0);
        $0 = -5;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       $67 = $maxv;
       $68 = ($67|0)==(-1);
       if ($68) {
        $insidc = 0;
       } else {
        $69 = $maxv;
        $70 = (($69) + 1)|0;
        $insidc = $70;
       }
      } else {
       $71 = $1;
       $72 = $3;
       $73 = (($72) + 16|0);
       $74 = HEAP32[$73>>2]|0;
       $75 = (($74) + ($71<<2)|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = (_lookup_ANSI_NIST_field($idcfield,$idcfield_i,2,$76)|0);
       $78 = ($77|0)!=(0);
       if ($78) {
        $87 = HEAP32[$idcfield>>2]|0;
        $88 = (($87) + 24|0);
        $89 = HEAP32[$88>>2]|0;
        $90 = HEAP32[$89>>2]|0;
        $91 = (($90) + 12|0);
        $92 = HEAP32[$91>>2]|0;
        $93 = HEAP32[$92>>2]|0;
        $94 = (($93) + 12|0);
        $95 = HEAP32[$94>>2]|0;
        $96 = (_atoi($95)|0);
        $insidc = $96;
        break;
       }
       $79 = HEAP32[_stderr>>2]|0;
       $80 = $1;
       $81 = $3;
       $82 = (($81) + 16|0);
       $83 = HEAP32[$82>>2]|0;
       $84 = (($83) + ($80<<2)|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = HEAP32[$85>>2]|0;
       HEAP32[$vararg_buffer17>>2] = $86;
       $vararg_ptr20 = (($vararg_buffer17) + 4|0);
       HEAP32[$vararg_ptr20>>2] = 2;
       (_fprintf(($79|0),(17096|0),($vararg_buffer17|0))|0);
       $0 = -6;
       $359 = $0;
       STACKTOP = sp;return ($359|0);
      }
     } while(0);
     $97 = HEAP32[$cntfield>>2]|0;
     $98 = (($97) + 16|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$cntfield>>2]|0;
     $101 = (($100) + 20|0);
     $102 = HEAP32[$101>>2]|0;
     $103 = ($99|0)>=($102|0);
     do {
      if ($103) {
       $104 = HEAP32[$cntfield>>2]|0;
       $105 = (($104) + 20|0);
       $106 = HEAP32[$105>>2]|0;
       $107 = (($106) + 100)|0;
       $108 = $107<<2;
       $new_size = $108;
       $109 = HEAP32[$cntfield>>2]|0;
       $110 = (($109) + 24|0);
       $111 = HEAP32[$110>>2]|0;
       $112 = $new_size;
       $113 = (_realloc($111,$112)|0);
       $new_ptr = $113;
       $114 = $new_ptr;
       $115 = ($114|0)==(0|0);
       if (!($115)) {
        $122 = $new_ptr;
        $123 = HEAP32[$cntfield>>2]|0;
        $124 = (($123) + 24|0);
        HEAP32[$124>>2] = $122;
        $125 = HEAP32[$cntfield>>2]|0;
        $126 = (($125) + 20|0);
        $127 = HEAP32[$126>>2]|0;
        $128 = (($127) + 100)|0;
        HEAP32[$126>>2] = $128;
        break;
       }
       $116 = HEAP32[_stderr>>2]|0;
       $117 = HEAP32[$cntfield>>2]|0;
       $118 = (($117) + 20|0);
       $119 = HEAP32[$118>>2]|0;
       $120 = $119<<2;
       $121 = $new_size;
       HEAP32[$vararg_buffer21>>2] = $120;
       $vararg_ptr24 = (($vararg_buffer21) + 4|0);
       HEAP32[$vararg_ptr24>>2] = $121;
       (_fprintf(($116|0),(17184|0),($vararg_buffer21|0))|0);
       $0 = -6;
       $359 = $0;
       STACKTOP = sp;return ($359|0);
      }
     } while(0);
     $129 = (_alloc_ANSI_NIST_subfield($subfield)|0);
     $ret = $129;
     $130 = ($129|0)!=(0);
     if ($130) {
      $131 = $ret;
      $0 = $131;
      $359 = $0;
      STACKTOP = sp;return ($359|0);
     }
     $132 = HEAP32[$cntfield>>2]|0;
     $133 = (($132) + 16|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = (($134) - 1)|0;
     $j = $135;
     $136 = HEAP32[$cntfield>>2]|0;
     $137 = (($136) + 16|0);
     $138 = HEAP32[$137>>2]|0;
     $k = $138;
     while(1) {
      $139 = $j;
      $140 = $1;
      $141 = ($139|0)>=($140|0);
      if (!($141)) {
       break;
      }
      $142 = $j;
      $143 = HEAP32[$cntfield>>2]|0;
      $144 = (($143) + 24|0);
      $145 = HEAP32[$144>>2]|0;
      $146 = (($145) + ($142<<2)|0);
      $147 = HEAP32[$146>>2]|0;
      $148 = $k;
      $149 = HEAP32[$cntfield>>2]|0;
      $150 = (($149) + 24|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = (($151) + ($148<<2)|0);
      HEAP32[$152>>2] = $147;
      $153 = $j;
      $154 = (($153) + -1)|0;
      $j = $154;
      $155 = $k;
      $156 = (($155) + -1)|0;
      $k = $156;
     }
     $157 = HEAP32[$subfield>>2]|0;
     $158 = $1;
     $159 = HEAP32[$cntfield>>2]|0;
     $160 = (($159) + 24|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = (($161) + ($158<<2)|0);
     HEAP32[$162>>2] = $157;
     $163 = HEAP32[$cntfield>>2]|0;
     $164 = (($163) + 16|0);
     $165 = HEAP32[$164>>2]|0;
     $166 = (($165) + 1)|0;
     HEAP32[$164>>2] = $166;
     $byte_adjust = 0;
     $167 = HEAP32[$cntfield>>2]|0;
     $168 = (($167) + 16|0);
     $169 = HEAP32[$168>>2]|0;
     $170 = ($169|0)>(1);
     if ($170) {
      $171 = $1;
      $172 = HEAP32[$cntfield>>2]|0;
      $173 = (($172) + 16|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = (($174) - 1)|0;
      $176 = ($171|0)==($175|0);
      if ($176) {
       $177 = $1;
       $178 = (($177) - 1)|0;
       $179 = HEAP32[$cntfield>>2]|0;
       $180 = (($179) + 24|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = (($181) + ($178<<2)|0);
       $183 = HEAP32[$182>>2]|0;
       $184 = (($183) + 16|0);
       HEAP32[$184>>2] = 1;
       $185 = $1;
       $186 = (($185) - 1)|0;
       $187 = HEAP32[$cntfield>>2]|0;
       $188 = (($187) + 24|0);
       $189 = HEAP32[$188>>2]|0;
       $190 = (($189) + ($186<<2)|0);
       $191 = HEAP32[$190>>2]|0;
       $192 = HEAP32[$191>>2]|0;
       $193 = (($192) + 1)|0;
       HEAP32[$191>>2] = $193;
      } else {
       $194 = HEAP32[$subfield>>2]|0;
       $195 = (($194) + 16|0);
       HEAP32[$195>>2] = 1;
       $196 = HEAP32[$subfield>>2]|0;
       $197 = HEAP32[$196>>2]|0;
       $198 = (($197) + 1)|0;
       HEAP32[$196>>2] = $198;
      }
      $199 = $byte_adjust;
      $200 = (($199) + 1)|0;
      $byte_adjust = $200;
     }
     $201 = $byte_adjust;
     $202 = HEAP32[$cntfield>>2]|0;
     $203 = (($202) + 12|0);
     $204 = HEAP32[$203>>2]|0;
     $205 = (($204) + ($201))|0;
     HEAP32[$203>>2] = $205;
     $206 = $byte_adjust;
     $207 = $3;
     $208 = (($207) + 16|0);
     $209 = HEAP32[$208>>2]|0;
     $210 = HEAP32[$209>>2]|0;
     $211 = (($210) + 8|0);
     $212 = HEAP32[$211>>2]|0;
     $213 = (($212) + ($206))|0;
     HEAP32[$211>>2] = $213;
     $214 = $byte_adjust;
     $215 = $3;
     $216 = (($215) + 4|0);
     $217 = HEAP32[$216>>2]|0;
     $218 = (($217) + ($214))|0;
     HEAP32[$216>>2] = $218;
     $219 = $1;
     $220 = $3;
     $221 = (($220) + 16|0);
     $222 = HEAP32[$221>>2]|0;
     $223 = (($222) + ($219<<2)|0);
     $224 = HEAP32[$223>>2]|0;
     $225 = HEAP32[$224>>2]|0;
     HEAP32[$vararg_buffer25>>2] = $225;
     (_sprintf($numstr,17280,$vararg_buffer25)|0);
     $226 = HEAP32[$cntfield_i>>2]|0;
     $227 = $1;
     $228 = $3;
     $229 = (_insert_ANSI_NIST_item(0,$226,$227,0,$numstr,$228)|0);
     $ret = $229;
     $230 = ($229|0)!=(0);
     if ($230) {
      $231 = $ret;
      $0 = $231;
      $359 = $0;
      STACKTOP = sp;return ($359|0);
     }
     $232 = $insidc;
     HEAP32[$vararg_buffer28>>2] = $232;
     (_sprintf($numstr,17288,$vararg_buffer28)|0);
     $233 = HEAP32[$cntfield_i>>2]|0;
     $234 = $1;
     $235 = $3;
     $236 = (_insert_ANSI_NIST_item(0,$233,$234,1,$numstr,$235)|0);
     $ret = $236;
     $237 = ($236|0)!=(0);
     if ($237) {
      $238 = $ret;
      $0 = $238;
      $359 = $0;
      STACKTOP = sp;return ($359|0);
     }
     $239 = HEAP32[$cntfield>>2]|0;
     $240 = (($239) + 24|0);
     $241 = HEAP32[$240>>2]|0;
     $242 = HEAP32[$241>>2]|0;
     $243 = (($242) + 4|0);
     $244 = HEAP32[$243>>2]|0;
     $245 = ($244|0)!=(2);
     if ($245) {
      $246 = HEAP32[_stderr>>2]|0;
      $247 = HEAP32[$cntfield_i>>2]|0;
      $248 = (($247) + 1)|0;
      HEAP32[$vararg_buffer31>>2] = $248;
      $vararg_ptr34 = (($vararg_buffer31) + 4|0);
      HEAP32[$vararg_ptr34>>2] = 3;
      (_fprintf(($246|0),(17296|0),($vararg_buffer31|0))|0);
      $0 = -7;
      $359 = $0;
      STACKTOP = sp;return ($359|0);
     }
     $249 = HEAP32[$cntfield_i>>2]|0;
     $250 = $3;
     $251 = (_increment_numeric_item(0,$249,0,1,$250,0)|0);
     $ret = $251;
     $252 = ($251|0)!=(0);
     if ($252) {
      $253 = $ret;
      $0 = $253;
      $359 = $0;
      STACKTOP = sp;return ($359|0);
     }
     $254 = $2;
     $255 = ($254|0)!=(0);
     do {
      if ($255) {
       $256 = $1;
       $257 = $3;
       $258 = (($257) + 16|0);
       $259 = HEAP32[$258>>2]|0;
       $260 = (($259) + ($256<<2)|0);
       $261 = HEAP32[$260>>2]|0;
       $262 = (_lookup_ANSI_NIST_field($idcfield,$idcfield_i,2,$261)|0);
       $263 = ($262|0)!=(0);
       if (!($263)) {
        $264 = HEAP32[_stderr>>2]|0;
        $265 = $1;
        $266 = (($265) + 1)|0;
        $267 = $1;
        $268 = $3;
        $269 = (($268) + 16|0);
        $270 = HEAP32[$269>>2]|0;
        $271 = (($270) + ($267<<2)|0);
        $272 = HEAP32[$271>>2]|0;
        $273 = HEAP32[$272>>2]|0;
        HEAP32[$vararg_buffer35>>2] = $266;
        $vararg_ptr38 = (($vararg_buffer35) + 4|0);
        HEAP32[$vararg_ptr38>>2] = $273;
        $vararg_ptr39 = (($vararg_buffer35) + 8|0);
        HEAP32[$vararg_ptr39>>2] = 2;
        (_fprintf(($264|0),(17400|0),($vararg_buffer35|0))|0);
        $0 = -8;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       $274 = $1;
       $275 = HEAP32[$idcfield_i>>2]|0;
       $276 = $3;
       $277 = (_substitute_ANSI_NIST_item($274,$275,0,0,$numstr,$276)|0);
       $ret = $277;
       $278 = ($277|0)!=(0);
       if ($278) {
        $279 = $ret;
        $0 = $279;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       $280 = HEAP32[_stderr>>2]|0;
       HEAP32[$vararg_buffer40>>2] = 3;
       (_fprintf(($280|0),(17496|0),($vararg_buffer40|0))|0);
       $281 = $1;
       $282 = (($281) + 1)|0;
       $j = $282;
       while(1) {
        $283 = $j;
        $284 = HEAP32[$cntfield>>2]|0;
        $285 = (($284) + 16|0);
        $286 = HEAP32[$285>>2]|0;
        $287 = ($283|0)<($286|0);
        if (!($287)) {
         label = 70;
         break;
        }
        $288 = $j;
        $289 = HEAP32[$cntfield>>2]|0;
        $290 = (($289) + 24|0);
        $291 = HEAP32[$290>>2]|0;
        $292 = (($291) + ($288<<2)|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = (_lookup_ANSI_NIST_item($item,1,$293)|0);
        $295 = ($294|0)!=(0);
        if (!($295)) {
         label = 56;
         break;
        }
        $299 = HEAP32[$item>>2]|0;
        $300 = (($299) + 12|0);
        $301 = HEAP32[$300>>2]|0;
        $302 = (_atoi($301)|0);
        $itemint = $302;
        $303 = $itemint;
        $304 = $insidc;
        $305 = ($303|0)>=($304|0);
        if ($305) {
         $306 = HEAP32[$cntfield_i>>2]|0;
         $307 = $j;
         $308 = $3;
         $309 = (_increment_numeric_item(0,$306,$307,1,$308,17288)|0);
         $ret = $309;
         $310 = ($309|0)!=(0);
         if ($310) {
          label = 59;
          break;
         }
         $312 = $j;
         $313 = $3;
         $314 = (($313) + 16|0);
         $315 = HEAP32[$314>>2]|0;
         $316 = (($315) + ($312<<2)|0);
         $317 = HEAP32[$316>>2]|0;
         $318 = (_lookup_ANSI_NIST_field($idcfield,$idcfield_i,2,$317)|0);
         $319 = ($318|0)!=(0);
         if (!($319)) {
          label = 61;
          break;
         }
         $330 = HEAP32[$idcfield>>2]|0;
         $331 = (($330) + 16|0);
         $332 = HEAP32[$331>>2]|0;
         $333 = ($332|0)!=(1);
         if ($333) {
          label = 64;
          break;
         }
         $334 = HEAP32[$idcfield>>2]|0;
         $335 = (($334) + 24|0);
         $336 = HEAP32[$335>>2]|0;
         $337 = HEAP32[$336>>2]|0;
         $338 = (($337) + 4|0);
         $339 = HEAP32[$338>>2]|0;
         $340 = ($339|0)!=(1);
         if ($340) {
          label = 64;
          break;
         }
         $351 = $j;
         $352 = HEAP32[$idcfield_i>>2]|0;
         $353 = $3;
         $354 = (_increment_numeric_item($351,$352,0,0,$353,17288)|0);
         $ret = $354;
         $355 = ($354|0)!=(0);
         if ($355) {
          label = 66;
          break;
         }
        }
        $357 = $j;
        $358 = (($357) + 1)|0;
        $j = $358;
       }
       if ((label|0) == 56) {
        $296 = HEAP32[_stderr>>2]|0;
        $297 = $j;
        $298 = (($297) + 1)|0;
        HEAP32[$vararg_buffer43>>2] = $298;
        $vararg_ptr46 = (($vararg_buffer43) + 4|0);
        HEAP32[$vararg_ptr46>>2] = 3;
        (_fprintf(($296|0),(17552|0),($vararg_buffer43|0))|0);
        $0 = -9;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       else if ((label|0) == 59) {
        $311 = $ret;
        $0 = $311;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       else if ((label|0) == 61) {
        $320 = HEAP32[_stderr>>2]|0;
        $321 = $j;
        $322 = (($321) + 1)|0;
        $323 = $j;
        $324 = $3;
        $325 = (($324) + 16|0);
        $326 = HEAP32[$325>>2]|0;
        $327 = (($326) + ($323<<2)|0);
        $328 = HEAP32[$327>>2]|0;
        $329 = HEAP32[$328>>2]|0;
        HEAP32[$vararg_buffer47>>2] = $322;
        $vararg_ptr50 = (($vararg_buffer47) + 4|0);
        HEAP32[$vararg_ptr50>>2] = $329;
        $vararg_ptr51 = (($vararg_buffer47) + 8|0);
        HEAP32[$vararg_ptr51>>2] = 2;
        (_fprintf(($320|0),(17400|0),($vararg_buffer47|0))|0);
        $0 = -10;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       else if ((label|0) == 64) {
        $341 = HEAP32[_stderr>>2]|0;
        $342 = $j;
        $343 = (($342) + 1)|0;
        $344 = $j;
        $345 = $3;
        $346 = (($345) + 16|0);
        $347 = HEAP32[$346>>2]|0;
        $348 = (($347) + ($344<<2)|0);
        $349 = HEAP32[$348>>2]|0;
        $350 = HEAP32[$349>>2]|0;
        HEAP32[$vararg_buffer52>>2] = $343;
        $vararg_ptr55 = (($vararg_buffer52) + 4|0);
        HEAP32[$vararg_ptr55>>2] = $350;
        $vararg_ptr56 = (($vararg_buffer52) + 8|0);
        HEAP32[$vararg_ptr56>>2] = 2;
        (_fprintf(($341|0),(17648|0),($vararg_buffer52|0))|0);
        $0 = -11;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       else if ((label|0) == 66) {
        $356 = $ret;
        $0 = $356;
        $359 = $0;
        STACKTOP = sp;return ($359|0);
       }
       else if ((label|0) == 70) {
        break;
       }
      }
     } while(0);
     $0 = 0;
     $359 = $0;
     STACKTOP = sp;return ($359|0);
    }
   }
   $30 = HEAP32[_stderr>>2]|0;
   $31 = $1;
   $32 = (($31) + 1)|0;
   $33 = HEAP32[$cntfield>>2]|0;
   $34 = (($33) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 1)|0;
   $37 = HEAP32[$cntfield_i>>2]|0;
   $38 = (($37) + 1)|0;
   HEAP32[$vararg_buffer6>>2] = $32;
   $vararg_ptr9 = (($vararg_buffer6) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $36;
   $vararg_ptr10 = (($vararg_buffer6) + 8|0);
   HEAP32[$vararg_ptr10>>2] = $38;
   $vararg_ptr11 = (($vararg_buffer6) + 12|0);
   HEAP32[$vararg_ptr11>>2] = 3;
   (_fprintf(($30|0),(16864|0),($vararg_buffer6|0))|0);
   $0 = -4;
   $359 = $0;
   STACKTOP = sp;return ($359|0);
  }
 }
 $15 = HEAP32[_stderr>>2]|0;
 (_fprintf(($15|0),(16712|0),($vararg_buffer1|0))|0);
 $0 = -2;
 $359 = $0;
 STACKTOP = sp;return ($359|0);
}
function _is_ANSI_NIST_file($ifile) {
 $ifile = $ifile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $buffer = 0, $cbufptr = 0, $ebufptr = 0, $field_id = 0, $field_int = 0, $fp = 0, $n = 0, $record_type = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $buffer = sp + 56|0;
 $cbufptr = sp + 32|0;
 $field_id = sp + 24|0;
 $record_type = sp + 20|0;
 $field_int = sp + 16|0;
 $1 = $ifile;
 $2 = $1;
 $3 = (_fopen(($2|0),(18560|0))|0);
 $fp = $3;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  $6 = $1;
  $7 = (___errno_location()|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_strerror(($8|0))|0);
  HEAP32[$vararg_buffer>>2] = $6;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $9;
  (_fprintf(($5|0),(18568|0),($vararg_buffer|0))|0);
  $0 = -2;
  $40 = $0;
  STACKTOP = sp;return ($40|0);
 }
 $10 = $fp;
 $11 = (_fread(($buffer|0),1,20,($10|0))|0);
 $n = $11;
 $12 = $fp;
 $13 = (_ferror(($12|0))|0);
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = $1;
  $17 = $fp;
  $18 = (_ferror(($17|0))|0);
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = (___errno_location()|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (_strerror(($21|0))|0);
   $23 = $22;
  } else {
   $23 = 18664;
  }
  HEAP32[$vararg_buffer2>>2] = $16;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $23;
  (_fprintf(($15|0),(18616|0),($vararg_buffer2|0))|0);
  $24 = $fp;
  (_fclose(($24|0))|0);
  $0 = -4;
  $40 = $0;
  STACKTOP = sp;return ($40|0);
 }
 $25 = $fp;
 (_fclose(($25|0))|0);
 HEAP32[$cbufptr>>2] = $buffer;
 $26 = $n;
 $27 = (($buffer) + ($26)|0);
 $ebufptr = $27;
 $28 = $ebufptr;
 $29 = (_parse_ANSI_NIST_field_ID($cbufptr,$28,$field_id,$record_type,$field_int)|0);
 $ret = $29;
 $30 = $ret;
 $31 = ($30|0)<(0);
 if ($31) {
  $32 = $ret;
  $0 = $32;
  $40 = $0;
  STACKTOP = sp;return ($40|0);
 }
 $33 = $ret;
 $34 = ($33|0)==(1);
 if ($34) {
  $35 = HEAP32[$record_type>>2]|0;
  $36 = ($35|0)==(1);
  if ($36) {
   $37 = HEAP32[$field_int>>2]|0;
   $38 = ($37|0)==(1);
   if ($38) {
    $39 = HEAP32[$field_id>>2]|0;
    _free($39);
    $0 = 1;
    $40 = $0;
    STACKTOP = sp;return ($40|0);
   }
  }
 }
 $0 = 0;
 $40 = $0;
 STACKTOP = sp;return ($40|0);
}
function _is_ANSI_NIST($idata,$ilen) {
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $cbufptr = 0, $ebufptr = 0, $field_id = 0, $field_int = 0, $record_type = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $cbufptr = sp + 16|0;
 $field_id = sp + 8|0;
 $record_type = sp + 4|0;
 $field_int = sp;
 $1 = $idata;
 $2 = $ilen;
 $3 = $1;
 HEAP32[$cbufptr>>2] = $3;
 $4 = $1;
 $5 = $2;
 $6 = (($4) + ($5)|0);
 $ebufptr = $6;
 $7 = $ebufptr;
 $8 = (_parse_ANSI_NIST_field_ID($cbufptr,$7,$field_id,$record_type,$field_int)|0);
 $ret = $8;
 $9 = $ret;
 $10 = ($9|0)<(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 $12 = $ret;
 $13 = ($12|0)==(1);
 if ($13) {
  $14 = HEAP32[$record_type>>2]|0;
  $15 = ($14|0)==(1);
  if ($15) {
   $16 = HEAP32[$field_int>>2]|0;
   $17 = ($16|0)==(1);
   if ($17) {
    $18 = HEAP32[$field_id>>2]|0;
    _free($18);
    $0 = 1;
    $19 = $0;
    STACKTOP = sp;return ($19|0);
   }
  }
 }
 $0 = 0;
 $19 = $0;
 STACKTOP = sp;return ($19|0);
}
function _lookup_ANSI_NIST_field($ofield,$ofield_i,$field_int,$record) {
 $ofield = $ofield|0;
 $ofield_i = $ofield_i|0;
 $field_int = $field_int|0;
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $ofield;
 $2 = $ofield_i;
 $3 = $field_int;
 $4 = $record;
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = $4;
  $7 = (($6) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 6;
   break;
  }
  $10 = $i;
  $11 = $4;
  $12 = (($11) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $3;
  $19 = ($17|0)==($18|0);
  if ($19) {
   label = 4;
   break;
  }
  $29 = $i;
  $30 = (($29) + 1)|0;
  $i = $30;
 }
 if ((label|0) == 4) {
  $20 = $i;
  $21 = $4;
  $22 = (($21) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $1;
  HEAP32[$26>>2] = $25;
  $27 = $i;
  $28 = $2;
  HEAP32[$28>>2] = $27;
  $0 = 1;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 }
 else if ((label|0) == 6) {
  $0 = 0;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 }
 return 0|0;
}
function _lookup_ANSI_NIST_item($oitem,$item_index,$subfield) {
 $oitem = $oitem|0;
 $item_index = $item_index|0;
 $subfield = $subfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $oitem;
 $2 = $item_index;
 $3 = $subfield;
 $4 = $2;
 $5 = ($4>>>0)>=(0);
 if ($5) {
  $6 = $2;
  $7 = $3;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6>>>0)<($9>>>0);
  if ($10) {
   $11 = $2;
   $12 = $3;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = (($14) + ($11<<2)|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $1;
   HEAP32[$17>>2] = $16;
   $0 = 1;
   $18 = $0;
   STACKTOP = sp;return ($18|0);
  }
 }
 $0 = 0;
 $18 = $0;
 STACKTOP = sp;return ($18|0);
}
function _lookup_ANSI_NIST_image($oimgrecord,$oimgrecord_i,$strt_record,$ansi_nist) {
 $oimgrecord = $oimgrecord|0;
 $oimgrecord_i = $oimgrecord_i|0;
 $strt_record = $strt_record|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $found = 0, $i = 0, $record = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $1 = $oimgrecord;
 $2 = $oimgrecord_i;
 $3 = $strt_record;
 $4 = $ansi_nist;
 $record = 0;
 $5 = $3;
 $i = $5;
 $found = 0;
 while(1) {
  $6 = $i;
  $7 = $4;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $i;
  $12 = $4;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $record = $16;
  $17 = $record;
  $18 = HEAP32[$17>>2]|0;
  $19 = (_binary_image_record($18)|0);
  $20 = ($19|0)!=(0);
  if ($20) {
   label = 4;
   break;
  }
  $21 = $i;
  $22 = $4;
  $23 = (($22) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  $28 = (_tagged_image_record($27)|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   label = 6;
   break;
  }
  $30 = $record;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)==(8);
  if ($32) {
   $33 = HEAP32[_stderr>>2]|0;
   $34 = $i;
   $35 = (($34) + 1)|0;
   HEAP32[$vararg_buffer>>2] = $35;
   (_fprintf(($33|0),(18680|0),($vararg_buffer|0))|0);
  }
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 if ((label|0) == 4) {
  $found = 1;
 }
 else if ((label|0) == 6) {
  $found = 1;
 }
 $38 = $found;
 $39 = ($38|0)!=(0);
 if ($39) {
  $40 = $record;
  $41 = $1;
  HEAP32[$41>>2] = $40;
  $42 = $i;
  $43 = $2;
  HEAP32[$43>>2] = $42;
  $0 = 1;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 } else {
  $0 = 0;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 return 0|0;
}
function _lookup_binary_field_image_ppmm($oppmm,$ansi_nist,$imgrecord_i) {
 $oppmm = $oppmm|0;
 $ansi_nist = $ansi_nist|0;
 $imgrecord_i = $imgrecord_i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $field_i = 0, $imgrecord = 0, $ppmm = 0.0, $tfloat = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer2 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $tfloat = sp + 48|0;
 $field = sp + 40|0;
 $field_i = sp + 36|0;
 $1 = $oppmm;
 $2 = $ansi_nist;
 $3 = $imgrecord_i;
 $4 = $3;
 $5 = ($4|0)<(1);
 if (!($5)) {
  $6 = $3;
  $7 = $2;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)>($9|0);
  if (!($10)) {
   $18 = $3;
   $19 = $2;
   $20 = (($19) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18<<2)|0);
   $23 = HEAP32[$22>>2]|0;
   $imgrecord = $23;
   $24 = $imgrecord;
   $25 = HEAP32[$24>>2]|0;
   $26 = (_binary_image_record($25)|0);
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $28 = HEAP32[_stderr>>2]|0;
    $29 = $3;
    $30 = (($29) + 1)|0;
    $31 = $imgrecord;
    $32 = HEAP32[$31>>2]|0;
    HEAP32[$vararg_buffer2>>2] = $30;
    $vararg_ptr5 = (($vararg_buffer2) + 4|0);
    HEAP32[$vararg_ptr5>>2] = $32;
    (_fprintf(($28|0),(18864|0),($vararg_buffer2|0))|0);
    $0 = -3;
    $63 = $0;
    STACKTOP = sp;return ($63|0);
   }
   $33 = $2;
   $34 = (($33) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = (_lookup_ANSI_NIST_field($field,$field_i,12,$36)|0);
   $38 = ($37|0)!=(0);
   if (!($38)) {
    $39 = HEAP32[_stderr>>2]|0;
    $40 = $imgrecord;
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$vararg_buffer6>>2] = $41;
    $vararg_ptr9 = (($vararg_buffer6) + 4|0);
    HEAP32[$vararg_ptr9>>2] = 12;
    (_fprintf(($39|0),(18968|0),($vararg_buffer6|0))|0);
    $0 = -4;
    $63 = $0;
    STACKTOP = sp;return ($63|0);
   }
   $42 = HEAP32[$field>>2]|0;
   $43 = (($42) + 24|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$vararg_buffer10>>2] = $tfloat;
   (_sscanf($50,19072,$vararg_buffer10)|0);
   $51 = +HEAPF32[$tfloat>>2];
   $52 = $51;
   $ppmm = $52;
   $53 = $imgrecord;
   $54 = HEAP32[$53>>2]|0;
   $55 = ($54|0)==(3);
   if ($55) {
    label = 10;
   } else {
    $56 = $imgrecord;
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)==(5);
    if ($58) {
     label = 10;
    }
   }
   if ((label|0) == 10) {
    $59 = $ppmm;
    $60 = $59 * 0.5;
    $ppmm = $60;
   }
   $61 = $ppmm;
   $62 = $1;
   HEAPF64[$62>>3] = $61;
   $0 = 0;
   $63 = $0;
   STACKTOP = sp;return ($63|0);
  }
 }
 $11 = HEAP32[_stderr>>2]|0;
 $12 = $3;
 $13 = (($12) + 1)|0;
 $14 = $2;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$vararg_buffer>>2] = $13;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $17;
 (_fprintf(($11|0),(18776|0),($vararg_buffer|0))|0);
 $0 = -2;
 $63 = $0;
 STACKTOP = sp;return ($63|0);
}
function _lookup_tagged_field_image_ppmm($oppmm,$record) {
 $oppmm = $oppmm|0;
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0;
 var $63 = 0.0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0.0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $field = 0, $field_i = 0, $hps = 0, $ppmm = 0.0, $slc = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer2 = 0;
 var $vararg_buffer6 = 0, $vararg_buffer9 = 0, $vararg_ptr1 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr5 = 0, $vps = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer2 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $field = sp + 68|0;
 $field_i = sp + 64|0;
 $1 = $oppmm;
 $2 = $record;
 $3 = $1;
 HEAPF64[$3>>3] = -1.0;
 $4 = $2;
 $5 = (_lookup_ANSI_NIST_field($field,$field_i,9,$4)|0);
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $2;
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$vararg_buffer>>2] = $9;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 9;
  (_fprintf(($7|0),(19080|0),($vararg_buffer|0))|0);
  $0 = -2;
  $71 = $0;
  STACKTOP = sp;return ($71|0);
 }
 $10 = HEAP32[$field>>2]|0;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (_atoi($18)|0);
 $hps = $19;
 $20 = $2;
 $21 = (_lookup_ANSI_NIST_field($field,$field_i,10,$20)|0);
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $23 = HEAP32[_stderr>>2]|0;
  $24 = $2;
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $25;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = 10;
  (_fprintf(($23|0),(19176|0),($vararg_buffer2|0))|0);
  $0 = -3;
  $71 = $0;
  STACKTOP = sp;return ($71|0);
 }
 $26 = HEAP32[$field>>2]|0;
 $27 = (($26) + 24|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (_atoi($34)|0);
 $vps = $35;
 $36 = $hps;
 $37 = $vps;
 $38 = ($36|0)!=($37|0);
 if ($38) {
  $39 = HEAP32[_stderr>>2]|0;
  $40 = $2;
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$vararg_buffer6>>2] = $41;
  (_fprintf(($39|0),(19272|0),($vararg_buffer6|0))|0);
 }
 $42 = $2;
 $43 = (_lookup_ANSI_NIST_field($field,$field_i,8,$42)|0);
 $44 = ($43|0)!=(0);
 if (!($44)) {
  $45 = HEAP32[_stderr>>2]|0;
  $46 = $2;
  $47 = HEAP32[$46>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $47;
  $vararg_ptr12 = (($vararg_buffer9) + 4|0);
  HEAP32[$vararg_ptr12>>2] = 8;
  (_fprintf(($45|0),(19456|0),($vararg_buffer9|0))|0);
  $0 = -4;
  $71 = $0;
  STACKTOP = sp;return ($71|0);
 }
 $48 = HEAP32[$field>>2]|0;
 $49 = (($48) + 24|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + 12|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 12|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = (_atoi($56)|0);
 $slc = $57;
 $58 = $slc;
 if ((($58|0) == 0)) {
  $ppmm = -1.0;
 } else if ((($58|0) == 1)) {
  $59 = $vps;
  $60 = (+($59|0));
  $61 = $60 / 25.3999999999999985789;
  $ppmm = $61;
 } else if ((($58|0) == 2)) {
  $62 = $vps;
  $63 = (+($62|0));
  $64 = $63 / 10.0;
  $ppmm = $64;
 } else {
  $65 = HEAP32[_stderr>>2]|0;
  $66 = $slc;
  $67 = $2;
  $68 = HEAP32[$67>>2]|0;
  HEAP32[$vararg_buffer13>>2] = $66;
  $vararg_ptr16 = (($vararg_buffer13) + 4|0);
  HEAP32[$vararg_ptr16>>2] = $68;
  $vararg_ptr17 = (($vararg_buffer13) + 8|0);
  HEAP32[$vararg_ptr17>>2] = 8;
  (_fprintf(($65|0),(19552|0),($vararg_buffer13|0))|0);
  $0 = -5;
  $71 = $0;
  STACKTOP = sp;return ($71|0);
 }
 $69 = $ppmm;
 $70 = $1;
 HEAPF64[$70>>3] = $69;
 $0 = 1;
 $71 = $0;
 STACKTOP = sp;return ($71|0);
}
function _lookup_ANSI_NIST_grayprint($oimgrecord,$oimgrecord_i,$strt_record,$ansi_nist) {
 $oimgrecord = $oimgrecord|0;
 $oimgrecord_i = $oimgrecord_i|0;
 $strt_record = $strt_record|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $field_i = 0, $i = 0, $id = 0, $imgrecord = 0, $imgrecord_i = 0, $ret = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $imgrecord = sp + 24|0;
 $imgrecord_i = sp + 20|0;
 $field = sp + 16|0;
 $field_i = sp + 12|0;
 $1 = $oimgrecord;
 $2 = $oimgrecord_i;
 $3 = $strt_record;
 $4 = $ansi_nist;
 $5 = $3;
 $i = $5;
 while(1) {
  $6 = $i;
  $7 = $4;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 15;
   break;
  }
  $11 = $i;
  $12 = $4;
  $13 = (_lookup_ANSI_NIST_image($imgrecord,$imgrecord_i,$11,$12)|0);
  $ret = $13;
  $14 = $ret;
  $15 = ($14|0)<(0);
  if ($15) {
   label = 4;
   break;
  }
  $17 = $ret;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   label = 6;
   break;
  }
  $19 = HEAP32[$imgrecord>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  if ((($20|0) == 14) | (($20|0) == 13)) {
   $25 = HEAP32[$imgrecord>>2]|0;
   $26 = (_lookup_ANSI_NIST_field($field,$field_i,12,$25)|0);
   $27 = ($26|0)!=(0);
   if (!($27)) {
    label = 10;
    break;
   }
   $33 = HEAP32[$field>>2]|0;
   $34 = (($33) + 24|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + 12|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (_atoi($41)|0);
   $id = $42;
   $43 = $id;
   $44 = ($43|0)==(8);
   if ($44) {
    label = 12;
    break;
   }
  } else if ((($20|0) == 4)) {
   label = 8;
   break;
  }
  $49 = HEAP32[$imgrecord_i>>2]|0;
  $50 = (($49) + 1)|0;
  $i = $50;
 }
 if ((label|0) == 4) {
  $16 = $ret;
  $0 = $16;
  $51 = $0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 6) {
  $0 = 0;
  $51 = $0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 8) {
  $21 = HEAP32[$imgrecord>>2]|0;
  $22 = $1;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$imgrecord_i>>2]|0;
  $24 = $2;
  HEAP32[$24>>2] = $23;
  $0 = 1;
  $51 = $0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 10) {
  $28 = HEAP32[_stderr>>2]|0;
  $29 = HEAP32[$imgrecord_i>>2]|0;
  $30 = (($29) + 1)|0;
  $31 = HEAP32[$imgrecord>>2]|0;
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$vararg_buffer>>2] = $30;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $32;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 12;
  (_fprintf(($28|0),(19648|0),($vararg_buffer|0))|0);
  $0 = -2;
  $51 = $0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 12) {
  $45 = HEAP32[$imgrecord>>2]|0;
  $46 = $1;
  HEAP32[$46>>2] = $45;
  $47 = HEAP32[$imgrecord_i>>2]|0;
  $48 = $2;
  HEAP32[$48>>2] = $47;
  $0 = 1;
  $51 = $0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 15) {
  $0 = 0;
  $51 = $0;
  STACKTOP = sp;return ($51|0);
 }
 return 0|0;
}
function _fbgetc($stream,$bdb) {
 $stream = $stream|0;
 $bdb = $bdb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rval = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $stream;
 $2 = $bdb;
 $3 = $1;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $1;
  $6 = (_fgetc(($5|0))|0);
  $0 = $6;
  $25 = $0;
  STACKTOP = sp;return ($25|0);
 }
 $7 = $2;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1|0);
 $11 = $2;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10>>>0)>($13>>>0);
 if ($14) {
  $0 = -1;
  $25 = $0;
  STACKTOP = sp;return ($25|0);
 } else {
  $15 = $2;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP8[$17>>0]|0;
  $rval = $18;
  $19 = $2;
  $20 = (($19) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 1|0);
  HEAP32[$20>>2] = $22;
  $23 = $rval;
  $24 = $23 << 24 >> 24;
  $0 = $24;
  $25 = $0;
  STACKTOP = sp;return ($25|0);
 }
 return 0|0;
}
function _fbread($ptr,$size,$nitems,$stream,$bdb) {
 $ptr = $ptr|0;
 $size = $size|0;
 $nitems = $nitems|0;
 $stream = $stream|0;
 $bdb = $bdb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $ptr;
 $2 = $size;
 $3 = $nitems;
 $4 = $stream;
 $5 = $bdb;
 $6 = $4;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = $1;
  $9 = $2;
  $10 = $3;
  $11 = $4;
  $12 = (_fread(($8|0),($9|0),($10|0),($11|0))|0);
  $0 = $12;
  $42 = $0;
  STACKTOP = sp;return ($42|0);
 }
 $n = 0;
 while(1) {
  $13 = $n;
  $14 = $3;
  $15 = ($13>>>0)<($14>>>0);
  if (!($15)) {
   label = 9;
   break;
  }
  $16 = $5;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $2;
  $20 = (($18) + ($19)|0);
  $21 = $5;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20>>>0)>($23>>>0);
  if ($24) {
   label = 6;
   break;
  }
  $26 = $1;
  $27 = $5;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $2;
  _memcpy(($26|0),($29|0),($30|0))|0;
  $31 = $2;
  $32 = $1;
  $33 = (($32) + ($31)|0);
  $1 = $33;
  $34 = $2;
  $35 = $5;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34)|0);
  HEAP32[$36>>2] = $38;
  $39 = $n;
  $40 = (($39) + 1)|0;
  $n = $40;
 }
 if ((label|0) == 6) {
  $25 = $n;
  $0 = $25;
  $42 = $0;
  STACKTOP = sp;return ($42|0);
 }
 else if ((label|0) == 9) {
  $41 = $n;
  $0 = $41;
  $42 = $0;
  STACKTOP = sp;return ($42|0);
 }
 return 0|0;
}
function _fbtell($stream,$bdb) {
 $stream = $stream|0;
 $bdb = $bdb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $stream;
 $2 = $bdb;
 $3 = $1;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $1;
  $6 = (_ftell(($5|0))|0);
  $0 = $6;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 } else {
  $7 = $2;
  $8 = (($7) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $2;
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $9;
  $14 = $12;
  $15 = (($13) - ($14))|0;
  $0 = $15;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _read_binary_item_data($fpin,$ovalue,$num_bytes) {
 $fpin = $fpin|0;
 $ovalue = $ovalue|0;
 $num_bytes = $num_bytes|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $fpin;
 $1 = $ovalue;
 $2 = $num_bytes;
 $3 = $0;
 $4 = $1;
 $5 = $2;
 $6 = (_i_read_binary_item_data($3,0,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _i_read_binary_item_data($fpin,$buf,$ovalue,$num_bytes) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ovalue = $ovalue|0;
 $num_bytes = $num_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ret = 0, $uchar_val = 0, $uint_val = 0, $ushort_val = 0, $value = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer16 = sp + 48|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $uint_val = sp + 56|0;
 $ushort_val = sp + 84|0;
 $uchar_val = sp + 86|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ovalue;
 $4 = $num_bytes;
 $5 = $4;
 do {
  if ((($5|0) == 2)) {
   $17 = $1;
   $18 = $2;
   $19 = (_i_read_binary_ushort($17,$18,$ushort_val)|0);
   $ret = $19;
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = $ret;
    $0 = $21;
    $45 = $0;
    STACKTOP = sp;return ($45|0);
   }
   $22 = (_malloc(6)|0);
   $value = $22;
   $23 = $value;
   $24 = ($23|0)==(0|0);
   if (!($24)) {
    $26 = $value;
    $27 = HEAP16[$ushort_val>>1]|0;
    $28 = $27&65535;
    HEAP32[$vararg_buffer7>>2] = $28;
    (_sprintf($26,20352,$vararg_buffer7)|0);
    break;
   }
   $25 = HEAP32[_stderr>>2]|0;
   HEAP32[$vararg_buffer4>>2] = 6;
   (_fprintf(($25|0),(20360|0),($vararg_buffer4|0))|0);
   $0 = -3;
   $45 = $0;
   STACKTOP = sp;return ($45|0);
  } else if ((($5|0) == 1)) {
   $29 = $1;
   $30 = $2;
   $31 = (_i_read_binary_uchar($29,$30,$uchar_val)|0);
   $ret = $31;
   $32 = ($31|0)!=(0);
   if ($32) {
    $33 = $ret;
    $0 = $33;
    $45 = $0;
    STACKTOP = sp;return ($45|0);
   }
   $34 = (_malloc(4)|0);
   $value = $34;
   $35 = $value;
   $36 = ($35|0)==(0|0);
   if (!($36)) {
    $38 = $value;
    $39 = HEAP8[$uchar_val>>0]|0;
    $40 = $39&255;
    HEAP32[$vararg_buffer13>>2] = $40;
    (_sprintf($38,20352,$vararg_buffer13)|0);
    break;
   }
   $37 = HEAP32[_stderr>>2]|0;
   HEAP32[$vararg_buffer10>>2] = 4;
   (_fprintf(($37|0),(20440|0),($vararg_buffer10|0))|0);
   $0 = -4;
   $45 = $0;
   STACKTOP = sp;return ($45|0);
  } else if ((($5|0) == 4)) {
   $6 = $1;
   $7 = $2;
   $8 = (_i_read_binary_uint($6,$7,$uint_val)|0);
   $ret = $8;
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = $ret;
    $0 = $10;
    $45 = $0;
    STACKTOP = sp;return ($45|0);
   }
   $11 = (_malloc(11)|0);
   $value = $11;
   $12 = $value;
   $13 = ($12|0)==(0|0);
   if (!($13)) {
    $15 = $value;
    $16 = HEAP32[$uint_val>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $16;
    (_sprintf($15,20352,$vararg_buffer1)|0);
    break;
   }
   $14 = HEAP32[_stderr>>2]|0;
   HEAP32[$vararg_buffer>>2] = 11;
   (_fprintf(($14|0),(20280|0),($vararg_buffer|0))|0);
   $0 = -2;
   $45 = $0;
   STACKTOP = sp;return ($45|0);
  } else {
   $41 = HEAP32[_stderr>>2]|0;
   $42 = $4;
   HEAP32[$vararg_buffer16>>2] = $42;
   (_fprintf(($41|0),(20512|0),($vararg_buffer16|0))|0);
   $0 = -5;
   $45 = $0;
   STACKTOP = sp;return ($45|0);
  }
 } while(0);
 $43 = $value;
 $44 = $3;
 HEAP32[$44>>2] = $43;
 $0 = 0;
 $45 = $0;
 STACKTOP = sp;return ($45|0);
}
function _scan_binary_item_data($buf,$ovalue,$num_bytes) {
 $buf = $buf|0;
 $ovalue = $ovalue|0;
 $num_bytes = $num_bytes|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $buf;
 $1 = $ovalue;
 $2 = $num_bytes;
 $3 = $0;
 $4 = $1;
 $5 = $2;
 $6 = (_i_read_binary_item_data(0,$3,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _i_read_binary_uint($fpin,$buf,$ouint_val) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ouint_val = $ouint_val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $_b_ = 0, $_f_ = 0, $_t_ = 0, $uint_val = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $uint_val = sp + 20|0;
 $_b_ = sp + 16|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ouint_val;
 $4 = $1;
 $5 = $2;
 $6 = (_fbread($uint_val,4,1,$4,$5)|0);
 $7 = ($6|0)!=(1);
 if (!($7)) {
  $19 = HEAP32[$uint_val>>2]|0;
  HEAP32[$_b_>>2] = $19;
  $_f_ = $_b_;
  $_t_ = $uint_val;
  $20 = $_f_;
  $21 = HEAP8[$20>>0]|0;
  $22 = $_t_;
  $23 = (($22) + 3|0);
  HEAP8[$23>>0] = $21;
  $24 = $_f_;
  $25 = (($24) + 1|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $_t_;
  $28 = (($27) + 2|0);
  HEAP8[$28>>0] = $26;
  $29 = $_f_;
  $30 = (($29) + 2|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $_t_;
  $33 = (($32) + 1|0);
  HEAP8[$33>>0] = $31;
  $34 = $_f_;
  $35 = (($34) + 3|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $_t_;
  HEAP8[$37>>0] = $36;
  $38 = HEAP32[$uint_val>>2]|0;
  $39 = $3;
  HEAP32[$39>>2] = $38;
  $0 = 0;
  $40 = $0;
  STACKTOP = sp;return ($40|0);
 }
 $8 = HEAP32[_stderr>>2]|0;
 $9 = $1;
 $10 = $2;
 $11 = (_fbtell($9,$10)|0);
 $12 = $1;
 $13 = (_ferror(($12|0))|0);
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = (___errno_location()|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_strerror(($16|0))|0);
  $18 = $17;
 } else {
  $18 = 20008;
 }
 HEAP32[$vararg_buffer>>2] = $11;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $18;
 (_fprintf(($8|0),(20216|0),($vararg_buffer|0))|0);
 $0 = -2;
 $40 = $0;
 STACKTOP = sp;return ($40|0);
}
function _i_read_binary_ushort($fpin,$buf,$oushort_val) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $oushort_val = $oushort_val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $_b_ = 0, $_f_ = 0, $_t_ = 0, $ushort_val = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $ushort_val = sp + 34|0;
 $_b_ = sp + 32|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $oushort_val;
 $4 = $1;
 $5 = $2;
 $6 = (_fbread($ushort_val,2,1,$4,$5)|0);
 $7 = ($6|0)!=(1);
 if (!($7)) {
  $19 = HEAP16[$ushort_val>>1]|0;
  HEAP16[$_b_>>1] = $19;
  $_f_ = $_b_;
  $_t_ = $ushort_val;
  $20 = $_f_;
  $21 = HEAP8[$20>>0]|0;
  $22 = $_t_;
  $23 = (($22) + 1|0);
  HEAP8[$23>>0] = $21;
  $24 = $_f_;
  $25 = (($24) + 1|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $_t_;
  HEAP8[$27>>0] = $26;
  $28 = HEAP16[$ushort_val>>1]|0;
  $29 = $3;
  HEAP16[$29>>1] = $28;
  $0 = 0;
  $30 = $0;
  STACKTOP = sp;return ($30|0);
 }
 $8 = HEAP32[_stderr>>2]|0;
 $9 = $1;
 $10 = $2;
 $11 = (_fbtell($9,$10)|0);
 $12 = $1;
 $13 = (_ferror(($12|0))|0);
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = (___errno_location()|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_strerror(($16|0))|0);
  $18 = $17;
 } else {
  $18 = 20008;
 }
 HEAP32[$vararg_buffer>>2] = $11;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $18;
 (_fprintf(($8|0),(20144|0),($vararg_buffer|0))|0);
 $0 = -2;
 $30 = $0;
 STACKTOP = sp;return ($30|0);
}
function _i_read_binary_uchar($fpin,$buf,$ouchar_val) {
 $fpin = $fpin|0;
 $buf = $buf|0;
 $ouchar_val = $ouchar_val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $uchar_val = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $uchar_val = sp + 24|0;
 $1 = $fpin;
 $2 = $buf;
 $3 = $ouchar_val;
 $4 = $1;
 $5 = $2;
 $6 = (_fbread($uchar_val,1,1,$4,$5)|0);
 $7 = ($6|0)!=(1);
 if (!($7)) {
  $19 = HEAP8[$uchar_val>>0]|0;
  $20 = $3;
  HEAP8[$20>>0] = $19;
  $0 = 0;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 $8 = HEAP32[_stderr>>2]|0;
 $9 = $1;
 $10 = $2;
 $11 = (_fbtell($9,$10)|0);
 $12 = $1;
 $13 = (_ferror(($12|0))|0);
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = (___errno_location()|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_strerror(($16|0))|0);
  $18 = $17;
 } else {
  $18 = 20008;
 }
 HEAP32[$vararg_buffer>>2] = $11;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $18;
 (_fprintf(($8|0),(20080|0),($vararg_buffer|0))|0);
 $0 = -2;
 $21 = $0;
 STACKTOP = sp;return ($21|0);
}
function _read_binary_image_data($bfile,$obindata,$obinbytes) {
 $bfile = $bfile|0;
 $obindata = $obindata|0;
 $obinbytes = $obinbytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $bfp = 0, $binbytes = 0, $bindata = 0, $binstat = 0, $nread = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer14 = 0, $vararg_buffer18 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0;
 var $vararg_ptr17 = 0, $vararg_ptr21 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer18 = sp + 48|0;
 $vararg_buffer14 = sp + 40|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $binstat = sp + 56|0;
 $1 = $bfile;
 $2 = $obindata;
 $3 = $obinbytes;
 $4 = $1;
 $5 = (_stat(($4|0),($binstat|0))|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $1;
  HEAP32[$vararg_buffer>>2] = $8;
  (_fprintf(($7|0),(19744|0),($vararg_buffer|0))|0);
  $0 = -2;
  $65 = $0;
  STACKTOP = sp;return ($65|0);
 }
 $9 = (($binstat) + 36|0);
 $10 = HEAP32[$9>>2]|0;
 $binbytes = $10;
 $11 = $1;
 $12 = (_fopen(($11|0),(19800|0))|0);
 $bfp = $12;
 $13 = $bfp;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = $1;
  $17 = (___errno_location()|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_strerror(($18|0))|0);
  HEAP32[$vararg_buffer1>>2] = $16;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $19;
  (_fprintf(($15|0),(19808|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $65 = $0;
  STACKTOP = sp;return ($65|0);
 }
 $20 = $binbytes;
 $21 = (_malloc($20)|0);
 $bindata = $21;
 $22 = $bindata;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[_stderr>>2]|0;
  $25 = $binbytes;
  HEAP32[$vararg_buffer5>>2] = $25;
  (_fprintf(($24|0),(19864|0),($vararg_buffer5|0))|0);
  $0 = -4;
  $65 = $0;
  STACKTOP = sp;return ($65|0);
 }
 $26 = $bindata;
 $27 = $binbytes;
 $28 = $bfp;
 $29 = (_fread(($26|0),1,($27|0),($28|0))|0);
 $nread = $29;
 $30 = $nread;
 $31 = $binbytes;
 $32 = ($30|0)!=($31|0);
 if (!($32)) {
  $53 = $bfp;
  $54 = (_fclose(($53|0))|0);
  $55 = ($54|0)!=(0);
  if ($55) {
   $56 = HEAP32[_stderr>>2]|0;
   $57 = $1;
   $58 = (___errno_location()|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_strerror(($59|0))|0);
   HEAP32[$vararg_buffer18>>2] = $57;
   $vararg_ptr21 = (($vararg_buffer18) + 4|0);
   HEAP32[$vararg_ptr21>>2] = $60;
   (_fprintf(($56|0),(20024|0),($vararg_buffer18|0))|0);
   $0 = -7;
   $65 = $0;
   STACKTOP = sp;return ($65|0);
  } else {
   $61 = $bindata;
   $62 = $2;
   HEAP32[$62>>2] = $61;
   $63 = $binbytes;
   $64 = $3;
   HEAP32[$64>>2] = $63;
   $0 = 0;
   $65 = $0;
   STACKTOP = sp;return ($65|0);
  }
 }
 $33 = HEAP32[_stderr>>2]|0;
 $34 = $nread;
 $35 = $binbytes;
 $36 = $bfp;
 $37 = (_ftell(($36|0))|0);
 $38 = $bfp;
 $39 = (_ferror(($38|0))|0);
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = (___errno_location()|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_strerror(($42|0))|0);
  $44 = $43;
 } else {
  $44 = 20008;
 }
 HEAP32[$vararg_buffer8>>2] = $34;
 $vararg_ptr11 = (($vararg_buffer8) + 4|0);
 HEAP32[$vararg_ptr11>>2] = $35;
 $vararg_ptr12 = (($vararg_buffer8) + 8|0);
 HEAP32[$vararg_ptr12>>2] = $37;
 $vararg_ptr13 = (($vararg_buffer8) + 12|0);
 HEAP32[$vararg_ptr13>>2] = $44;
 (_fprintf(($33|0),(19928|0),($vararg_buffer8|0))|0);
 $45 = $bfp;
 $46 = (_fclose(($45|0))|0);
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = HEAP32[_stderr>>2]|0;
  $49 = $1;
  $50 = (___errno_location()|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (_strerror(($51|0))|0);
  HEAP32[$vararg_buffer14>>2] = $49;
  $vararg_ptr17 = (($vararg_buffer14) + 4|0);
  HEAP32[$vararg_ptr17>>2] = $52;
  (_fprintf(($48|0),(20024|0),($vararg_buffer14|0))|0);
  $0 = -5;
  $65 = $0;
  STACKTOP = sp;return ($65|0);
 } else {
  $0 = -6;
  $65 = $0;
  STACKTOP = sp;return ($65|0);
 }
 return 0|0;
}
function _substitute_ANSI_NIST_item($record_i,$field_i,$subfield_i,$item_i,$newvalue,$ansi_nist) {
 $record_i = $record_i|0;
 $field_i = $field_i|0;
 $subfield_i = $subfield_i|0;
 $item_i = $item_i|0;
 $newvalue = $newvalue|0;
 $ansi_nist = $ansi_nist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $byte_adjust = 0;
 var $field = 0, $image_value = 0, $item = 0, $new_ptr = 0, $newalloc = 0, $newlen = 0, $oldlen = 0, $record = 0, $ret = 0, $subfield = 0, $vararg_buffer = 0, $vararg_buffer15 = 0, $vararg_buffer2 = 0, $vararg_buffer24 = 0, $vararg_buffer33 = 0, $vararg_buffer37 = 0, $vararg_buffer46 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0;
 var $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr27 = 0, $vararg_ptr28 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr40 = 0, $vararg_ptr41 = 0, $vararg_ptr42 = 0;
 var $vararg_ptr43 = 0, $vararg_ptr44 = 0, $vararg_ptr45 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer46 = sp + 152|0;
 $vararg_buffer37 = sp + 120|0;
 $vararg_buffer33 = sp + 112|0;
 $vararg_buffer24 = sp + 80|0;
 $vararg_buffer15 = sp + 48|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $newalloc = sp + 164|0;
 $image_value = sp + 160|0;
 $1 = $record_i;
 $2 = $field_i;
 $3 = $subfield_i;
 $4 = $item_i;
 $5 = $newvalue;
 $6 = $ansi_nist;
 $7 = $1;
 $8 = ($7|0)<(0);
 if (!($8)) {
  $9 = $1;
  $10 = $6;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)>=($12|0);
  if (!($13)) {
   $20 = $1;
   $21 = $6;
   $22 = (($21) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + ($20<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $record = $25;
   $26 = $2;
   $27 = ($26|0)<(0);
   if (!($27)) {
    $28 = $2;
    $29 = $record;
    $30 = (($29) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = ($28|0)>=($31|0);
    if (!($32)) {
     $41 = $2;
     $42 = $record;
     $43 = (($42) + 20|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = (($44) + ($41<<2)|0);
     $46 = HEAP32[$45>>2]|0;
     $field = $46;
     $47 = $3;
     $48 = ($47|0)<(0);
     if (!($48)) {
      $49 = $3;
      $50 = $field;
      $51 = (($50) + 16|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = ($49|0)>=($52|0);
      if (!($53)) {
       $69 = $3;
       $70 = $field;
       $71 = (($70) + 24|0);
       $72 = HEAP32[$71>>2]|0;
       $73 = (($72) + ($69<<2)|0);
       $74 = HEAP32[$73>>2]|0;
       $subfield = $74;
       $75 = $4;
       $76 = ($75|0)<(0);
       if (!($76)) {
        $77 = $4;
        $78 = $subfield;
        $79 = (($78) + 4|0);
        $80 = HEAP32[$79>>2]|0;
        $81 = ($77|0)>=($80|0);
        if (!($81)) {
         $99 = $4;
         $100 = $subfield;
         $101 = (($100) + 12|0);
         $102 = HEAP32[$101>>2]|0;
         $103 = (($102) + ($99<<2)|0);
         $104 = HEAP32[$103>>2]|0;
         $item = $104;
         $105 = $field;
         $106 = (_image_field($105)|0);
         $107 = ($106|0)!=(0);
         do {
          if ($107) {
           $108 = $5;
           (_read_binary_image_data($108,$image_value,$newalloc)|0);
           $109 = $item;
           $110 = HEAP32[$109>>2]|0;
           $oldlen = $110;
           $111 = HEAP32[$newalloc>>2]|0;
           $newlen = $111;
           $112 = $item;
           $113 = (($112) + 12|0);
           $114 = HEAP32[$113>>2]|0;
           _free($114);
           $115 = HEAP32[$image_value>>2]|0;
           $116 = $item;
           $117 = (($116) + 12|0);
           HEAP32[$117>>2] = $115;
           $118 = HEAP32[$newalloc>>2]|0;
           $119 = $item;
           $120 = (($119) + 8|0);
           HEAP32[$120>>2] = $118;
           $121 = HEAP32[$newalloc>>2]|0;
           $122 = $item;
           $123 = (($122) + 4|0);
           HEAP32[$123>>2] = $121;
           $124 = HEAP32[$newalloc>>2]|0;
           $125 = $item;
           HEAP32[$125>>2] = $124;
           $126 = $newlen;
           $127 = $oldlen;
           $128 = (($126) - ($127))|0;
           $byte_adjust = $128;
           $129 = $byte_adjust;
           $130 = $subfield;
           $131 = HEAP32[$130>>2]|0;
           $132 = (($131) + ($129))|0;
           HEAP32[$130>>2] = $132;
           $133 = $byte_adjust;
           $134 = $field;
           $135 = (($134) + 12|0);
           $136 = HEAP32[$135>>2]|0;
           $137 = (($136) + ($133))|0;
           HEAP32[$135>>2] = $137;
           $138 = $byte_adjust;
           $139 = $record;
           $140 = (($139) + 8|0);
           $141 = HEAP32[$140>>2]|0;
           $142 = (($141) + ($138))|0;
           HEAP32[$140>>2] = $142;
           $143 = $byte_adjust;
           $144 = $6;
           $145 = (($144) + 4|0);
           $146 = HEAP32[$145>>2]|0;
           $147 = (($146) + ($143))|0;
           HEAP32[$145>>2] = $147;
           $148 = $6;
           $149 = $1;
           $150 = (_update_ANSI_NIST_record_LEN($148,$149)|0);
           $ret = $150;
           $151 = ($150|0)!=(0);
           if (!($151)) {
            $153 = HEAP32[_stderr>>2]|0;
            $154 = $1;
            $155 = (($154) + 1)|0;
            $156 = $2;
            $157 = (($156) + 1)|0;
            $158 = $3;
            $159 = (($158) + 1)|0;
            $160 = $4;
            $161 = (($160) + 1)|0;
            $162 = $record;
            $163 = HEAP32[$162>>2]|0;
            $164 = $field;
            $165 = (($164) + 8|0);
            $166 = HEAP32[$165>>2]|0;
            $167 = $5;
            HEAP32[$vararg_buffer24>>2] = $155;
            $vararg_ptr27 = (($vararg_buffer24) + 4|0);
            HEAP32[$vararg_ptr27>>2] = $157;
            $vararg_ptr28 = (($vararg_buffer24) + 8|0);
            HEAP32[$vararg_ptr28>>2] = $159;
            $vararg_ptr29 = (($vararg_buffer24) + 12|0);
            HEAP32[$vararg_ptr29>>2] = $161;
            $vararg_ptr30 = (($vararg_buffer24) + 16|0);
            HEAP32[$vararg_ptr30>>2] = $163;
            $vararg_ptr31 = (($vararg_buffer24) + 20|0);
            HEAP32[$vararg_ptr31>>2] = $166;
            $vararg_ptr32 = (($vararg_buffer24) + 24|0);
            HEAP32[$vararg_ptr32>>2] = $167;
            (_fprintf(($153|0),(20992|0),($vararg_buffer24|0))|0);
            break;
           }
           $152 = $ret;
           $0 = $152;
           $278 = $0;
           STACKTOP = sp;return ($278|0);
          } else {
           $168 = $item;
           $169 = (($168) + 4|0);
           $170 = HEAP32[$169>>2]|0;
           $oldlen = $170;
           $171 = $5;
           $172 = (_strlen(($171|0))|0);
           $newlen = $172;
           $173 = $newlen;
           $174 = (($173) + 1)|0;
           HEAP32[$newalloc>>2] = $174;
           $175 = HEAP32[$newalloc>>2]|0;
           $176 = $item;
           $177 = (($176) + 8|0);
           $178 = HEAP32[$177>>2]|0;
           $179 = ($175|0)>=($178|0);
           do {
            if ($179) {
             $180 = $item;
             $181 = (($180) + 12|0);
             $182 = HEAP32[$181>>2]|0;
             $183 = HEAP32[$newalloc>>2]|0;
             $184 = (_realloc($182,$183)|0);
             $new_ptr = $184;
             $185 = $new_ptr;
             $186 = ($185|0)==(0|0);
             if (!($186)) {
              $192 = $new_ptr;
              $193 = $item;
              $194 = (($193) + 12|0);
              HEAP32[$194>>2] = $192;
              $195 = HEAP32[$newalloc>>2]|0;
              $196 = $item;
              $197 = (($196) + 8|0);
              HEAP32[$197>>2] = $195;
              break;
             }
             $187 = HEAP32[_stderr>>2]|0;
             $188 = $item;
             $189 = (($188) + 8|0);
             $190 = HEAP32[$189>>2]|0;
             $191 = HEAP32[$newalloc>>2]|0;
             HEAP32[$vararg_buffer33>>2] = $190;
             $vararg_ptr36 = (($vararg_buffer33) + 4|0);
             HEAP32[$vararg_ptr36>>2] = $191;
             (_fprintf(($187|0),(21080|0),($vararg_buffer33|0))|0);
             $0 = -8;
             $278 = $0;
             STACKTOP = sp;return ($278|0);
            }
           } while(0);
           $198 = HEAP32[_stderr>>2]|0;
           $199 = $1;
           $200 = (($199) + 1)|0;
           $201 = $2;
           $202 = (($201) + 1)|0;
           $203 = $3;
           $204 = (($203) + 1)|0;
           $205 = $4;
           $206 = (($205) + 1)|0;
           $207 = $record;
           $208 = HEAP32[$207>>2]|0;
           $209 = $field;
           $210 = (($209) + 8|0);
           $211 = HEAP32[$210>>2]|0;
           $212 = $item;
           $213 = (($212) + 12|0);
           $214 = HEAP32[$213>>2]|0;
           HEAP32[$vararg_buffer37>>2] = $200;
           $vararg_ptr40 = (($vararg_buffer37) + 4|0);
           HEAP32[$vararg_ptr40>>2] = $202;
           $vararg_ptr41 = (($vararg_buffer37) + 8|0);
           HEAP32[$vararg_ptr41>>2] = $204;
           $vararg_ptr42 = (($vararg_buffer37) + 12|0);
           HEAP32[$vararg_ptr42>>2] = $206;
           $vararg_ptr43 = (($vararg_buffer37) + 16|0);
           HEAP32[$vararg_ptr43>>2] = $208;
           $vararg_ptr44 = (($vararg_buffer37) + 20|0);
           HEAP32[$vararg_ptr44>>2] = $211;
           $vararg_ptr45 = (($vararg_buffer37) + 24|0);
           HEAP32[$vararg_ptr45>>2] = $214;
           (_fprintf(($198|0),(21168|0),($vararg_buffer37|0))|0);
           $215 = $item;
           $216 = (($215) + 12|0);
           $217 = HEAP32[$216>>2]|0;
           $218 = $5;
           (_strcpy(($217|0),($218|0))|0);
           $219 = $item;
           $220 = (($219) + 12|0);
           $221 = HEAP32[$220>>2]|0;
           $222 = (_strlen(($221|0))|0);
           $223 = $item;
           $224 = (($223) + 4|0);
           HEAP32[$224>>2] = $222;
           $225 = $record;
           $226 = HEAP32[$225>>2]|0;
           $227 = (_tagged_record($226)|0);
           $228 = ($227|0)!=(0);
           if ($228) {
            $229 = $item;
            $230 = (($229) + 4|0);
            $231 = HEAP32[$230>>2]|0;
            $232 = $item;
            HEAP32[$232>>2] = $231;
           }
           $233 = $item;
           $234 = (($233) + 16|0);
           $235 = HEAP32[$234>>2]|0;
           $236 = ($235|0)!=(0);
           if ($236) {
            $237 = $item;
            $238 = HEAP32[$237>>2]|0;
            $239 = (($238) + 1)|0;
            HEAP32[$237>>2] = $239;
           }
           $240 = HEAP32[_stderr>>2]|0;
           $241 = $item;
           $242 = (($241) + 12|0);
           $243 = HEAP32[$242>>2]|0;
           HEAP32[$vararg_buffer46>>2] = $243;
           (_fprintf(($240|0),(21240|0),($vararg_buffer46|0))|0);
           $244 = $record;
           $245 = HEAP32[$244>>2]|0;
           $246 = (_tagged_record($245)|0);
           $247 = ($246|0)!=(0);
           do {
            if ($247) {
             $248 = $oldlen;
             $249 = $newlen;
             $250 = ($248|0)!=($249|0);
             if ($250) {
              $251 = $newlen;
              $252 = $oldlen;
              $253 = (($251) - ($252))|0;
              $byte_adjust = $253;
              $254 = $byte_adjust;
              $255 = $subfield;
              $256 = HEAP32[$255>>2]|0;
              $257 = (($256) + ($254))|0;
              HEAP32[$255>>2] = $257;
              $258 = $byte_adjust;
              $259 = $field;
              $260 = (($259) + 12|0);
              $261 = HEAP32[$260>>2]|0;
              $262 = (($261) + ($258))|0;
              HEAP32[$260>>2] = $262;
              $263 = $byte_adjust;
              $264 = $record;
              $265 = (($264) + 8|0);
              $266 = HEAP32[$265>>2]|0;
              $267 = (($266) + ($263))|0;
              HEAP32[$265>>2] = $267;
              $268 = $byte_adjust;
              $269 = $6;
              $270 = (($269) + 4|0);
              $271 = HEAP32[$270>>2]|0;
              $272 = (($271) + ($268))|0;
              HEAP32[$270>>2] = $272;
              $273 = $6;
              $274 = $1;
              $275 = (_update_ANSI_NIST_record_LEN($273,$274)|0);
              $ret = $275;
              $276 = ($275|0)!=(0);
              if (!($276)) {
               break;
              }
              $277 = $ret;
              $0 = $277;
              $278 = $0;
              STACKTOP = sp;return ($278|0);
             }
            }
           } while(0);
          }
         } while(0);
         $0 = 0;
         $278 = $0;
         STACKTOP = sp;return ($278|0);
        }
       }
       $82 = HEAP32[_stderr>>2]|0;
       $83 = $1;
       $84 = (($83) + 1)|0;
       $85 = $2;
       $86 = (($85) + 1)|0;
       $87 = $3;
       $88 = (($87) + 1)|0;
       $89 = $4;
       $90 = (($89) + 1)|0;
       $91 = $subfield;
       $92 = (($91) + 4|0);
       $93 = HEAP32[$92>>2]|0;
       $94 = $record;
       $95 = HEAP32[$94>>2]|0;
       $96 = $field;
       $97 = (($96) + 8|0);
       $98 = HEAP32[$97>>2]|0;
       HEAP32[$vararg_buffer15>>2] = $84;
       $vararg_ptr18 = (($vararg_buffer15) + 4|0);
       HEAP32[$vararg_ptr18>>2] = $86;
       $vararg_ptr19 = (($vararg_buffer15) + 8|0);
       HEAP32[$vararg_ptr19>>2] = $88;
       $vararg_ptr20 = (($vararg_buffer15) + 12|0);
       HEAP32[$vararg_ptr20>>2] = $90;
       $vararg_ptr21 = (($vararg_buffer15) + 16|0);
       HEAP32[$vararg_ptr21>>2] = $93;
       $vararg_ptr22 = (($vararg_buffer15) + 20|0);
       HEAP32[$vararg_ptr22>>2] = $95;
       $vararg_ptr23 = (($vararg_buffer15) + 24|0);
       HEAP32[$vararg_ptr23>>2] = $98;
       (_fprintf(($82|0),(20880|0),($vararg_buffer15|0))|0);
       $0 = -6;
       $278 = $0;
       STACKTOP = sp;return ($278|0);
      }
     }
     $54 = HEAP32[_stderr>>2]|0;
     $55 = $1;
     $56 = (($55) + 1)|0;
     $57 = $2;
     $58 = (($57) + 1)|0;
     $59 = $3;
     $60 = (($59) + 1)|0;
     $61 = $field;
     $62 = (($61) + 16|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = $record;
     $65 = HEAP32[$64>>2]|0;
     $66 = $field;
     $67 = (($66) + 8|0);
     $68 = HEAP32[$67>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $56;
     $vararg_ptr10 = (($vararg_buffer7) + 4|0);
     HEAP32[$vararg_ptr10>>2] = $58;
     $vararg_ptr11 = (($vararg_buffer7) + 8|0);
     HEAP32[$vararg_ptr11>>2] = $60;
     $vararg_ptr12 = (($vararg_buffer7) + 12|0);
     HEAP32[$vararg_ptr12>>2] = $63;
     $vararg_ptr13 = (($vararg_buffer7) + 16|0);
     HEAP32[$vararg_ptr13>>2] = $65;
     $vararg_ptr14 = (($vararg_buffer7) + 20|0);
     HEAP32[$vararg_ptr14>>2] = $68;
     (_fprintf(($54|0),(20768|0),($vararg_buffer7|0))|0);
     $0 = -5;
     $278 = $0;
     STACKTOP = sp;return ($278|0);
    }
   }
   $33 = HEAP32[_stderr>>2]|0;
   $34 = $2;
   $35 = (($34) + 1)|0;
   $36 = $record;
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $record;
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$vararg_buffer2>>2] = $35;
   $vararg_ptr5 = (($vararg_buffer2) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $38;
   $vararg_ptr6 = (($vararg_buffer2) + 8|0);
   HEAP32[$vararg_ptr6>>2] = $40;
   (_fprintf(($33|0),(20672|0),($vararg_buffer2|0))|0);
   $0 = -3;
   $278 = $0;
   STACKTOP = sp;return ($278|0);
  }
 }
 $14 = HEAP32[_stderr>>2]|0;
 $15 = $1;
 $16 = (($15) + 1)|0;
 $17 = $6;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer>>2] = $16;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $19;
 (_fprintf(($14|0),(20592|0),($vararg_buffer|0))|0);
 $0 = -2;
 $278 = $0;
 STACKTOP = sp;return ($278|0);
}
function _tagged_record($record_type) {
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $record_type;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(10);
  if (!($3)) {
   label = 7;
   break;
  }
  $4 = $1;
  $5 = $i;
  $6 = (26864 + ($5<<2)|0);
  $7 = HEAPU8[$6>>0]|(HEAPU8[$6+1>>0]<<8)|(HEAPU8[$6+2>>0]<<16)|(HEAPU8[$6+3>>0]<<24);
  $8 = ($4|0)==($7|0);
  if ($8) {
   label = 4;
   break;
  }
  $9 = $i;
  $10 = (($9) + 1)|0;
  $i = $10;
 }
 if ((label|0) == 4) {
  $0 = 1;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _binary_record($record_type) {
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $record_type;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(6);
  if (!($3)) {
   label = 7;
   break;
  }
  $4 = $1;
  $5 = $i;
  $6 = (26904 + ($5<<2)|0);
  $7 = HEAPU8[$6>>0]|(HEAPU8[$6+1>>0]<<8)|(HEAPU8[$6+2>>0]<<16)|(HEAPU8[$6+3>>0]<<24);
  $8 = ($4|0)==($7|0);
  if ($8) {
   label = 4;
   break;
  }
  $9 = $i;
  $10 = (($9) + 1)|0;
  $i = $10;
 }
 if ((label|0) == 4) {
  $0 = 1;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _tagged_image_record($record_type) {
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $record_type;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(7);
  if (!($3)) {
   label = 7;
   break;
  }
  $4 = $1;
  $5 = $i;
  $6 = (26928 + ($5<<2)|0);
  $7 = HEAPU8[$6>>0]|(HEAPU8[$6+1>>0]<<8)|(HEAPU8[$6+2>>0]<<16)|(HEAPU8[$6+3>>0]<<24);
  $8 = ($4|0)==($7|0);
  if ($8) {
   label = 4;
   break;
  }
  $9 = $i;
  $10 = (($9) + 1)|0;
  $i = $10;
 }
 if ((label|0) == 4) {
  $0 = 1;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _binary_image_record($record_type) {
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $record_type;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(5);
  if (!($3)) {
   label = 7;
   break;
  }
  $4 = $1;
  $5 = $i;
  $6 = (26960 + ($5<<2)|0);
  $7 = HEAPU8[$6>>0]|(HEAPU8[$6+1>>0]<<8)|(HEAPU8[$6+2>>0]<<16)|(HEAPU8[$6+3>>0]<<24);
  $8 = ($4|0)==($7|0);
  if ($8) {
   label = 4;
   break;
  }
  $9 = $i;
  $10 = (($9) + 1)|0;
  $i = $10;
 }
 if ((label|0) == 4) {
  $0 = 1;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _binary_signature_record($record_type) {
 $record_type = $record_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $record_type;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(1);
  if (!($3)) {
   label = 7;
   break;
  }
  $4 = $1;
  $5 = $i;
  $6 = (26984 + ($5<<2)|0);
  $7 = HEAPU8[$6>>0]|(HEAPU8[$6+1>>0]<<8)|(HEAPU8[$6+2>>0]<<16)|(HEAPU8[$6+3>>0]<<24);
  $8 = ($4|0)==($7|0);
  if ($8) {
   label = 4;
   break;
  }
  $9 = $i;
  $10 = (($9) + 1)|0;
  $i = $10;
 }
 if ((label|0) == 4) {
  $0 = 1;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 else if ((label|0) == 7) {
  $0 = 0;
  $11 = $0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _image_field($field) {
 $field = $field|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $field;
 $2 = $1;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (_binary_image_record($4)|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $1;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(9);
  if (!($10)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $11 = $1;
   $12 = (($11) + 4|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (_binary_signature_record($13)|0);
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = $1;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($18|0)==(8);
    if ($19) {
     break;
    }
   }
   $20 = $1;
   $21 = (($20) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_tagged_image_record($22)|0);
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = $1;
    $26 = (($25) + 8|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27|0)==(999);
    if ($28) {
     break;
    }
   }
   $0 = 0;
   $29 = $0;
   STACKTOP = sp;return ($29|0);
  }
 } while(0);
 $0 = 1;
 $29 = $0;
 STACKTOP = sp;return ($29|0);
}
function _is_delimiter($nextchar) {
 $nextchar = $nextchar|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $nextchar;
 $2 = $1;
 if ((($2|0) == 31) | (($2|0) == 30) | (($2|0) == 29) | (($2|0) == 28)) {
  $0 = 1;
 } else {
  $0 = 0;
 }
 $3 = $0;
 STACKTOP = sp;return ($3|0);
}
function _fingerprint2tagged_field_image($orecord,$idata,$ilen,$iw,$ih,$id,$ppmm,$img_comp,$img_idc,$img_imp,$source_str) {
 $orecord = $orecord|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $id = $id|0;
 $ppmm = +$ppmm;
 $img_comp = $img_comp|0;
 $img_idc = $img_idc|0;
 $img_imp = $img_imp|0;
 $source_str = $source_str|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0.0, $8 = 0, $9 = 0, $record = 0, $ret = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $record = sp + 12|0;
 $1 = $orecord;
 $2 = $idata;
 $3 = $ilen;
 $4 = $iw;
 $5 = $ih;
 $6 = $id;
 $7 = $ppmm;
 $8 = $img_comp;
 $9 = $img_idc;
 $10 = $img_imp;
 $11 = $source_str;
 $12 = $10;
 L1: do {
  switch ($12|0) {
  case 3: case 2: case 1: case 0:  {
   $13 = $2;
   $14 = $3;
   $15 = $4;
   $16 = $5;
   $17 = $6;
   $18 = $7;
   $19 = $8;
   $20 = $9;
   $21 = $10;
   $22 = $11;
   $23 = (_image2type_14($record,$13,$14,$15,$16,$17,$18,$19,$20,$21,$22)|0);
   $ret = $23;
   $24 = ($23|0)!=(0);
   if (!($24)) {
    break L1;
   }
   $25 = $ret;
   $0 = $25;
   $43 = $0;
   STACKTOP = sp;return ($43|0);
   break;
  }
  case 7: case 6: case 5: case 4:  {
   $26 = $2;
   $27 = $3;
   $28 = $4;
   $29 = $5;
   $30 = $6;
   $31 = $7;
   $32 = $8;
   $33 = $9;
   $34 = $10;
   $35 = $11;
   $36 = (_image2type_13($record,$26,$27,$28,$29,$30,$31,$32,$33,$34,$35)|0);
   $ret = $36;
   $37 = ($36|0)!=(0);
   if (!($37)) {
    break L1;
   }
   $38 = $ret;
   $0 = $38;
   $43 = $0;
   STACKTOP = sp;return ($43|0);
   break;
  }
  default: {
   $39 = HEAP32[_stderr>>2]|0;
   $40 = $10;
   HEAP32[$vararg_buffer>>2] = $40;
   (_fprintf(($39|0),(21264|0),($vararg_buffer|0))|0);
   $0 = -2;
   $43 = $0;
   STACKTOP = sp;return ($43|0);
  }
  }
 } while(0);
 $41 = HEAP32[$record>>2]|0;
 $42 = $1;
 HEAP32[$42>>2] = $41;
 $0 = 0;
 $43 = $0;
 STACKTOP = sp;return ($43|0);
}
function _image2type_14($orecord,$idata,$ilen,$iw,$ih,$id,$ppmm,$img_comp,$img_idc,$img_imp,$source_str) {
 $orecord = $orecord|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $id = $id|0;
 $ppmm = +$ppmm;
 $img_comp = $img_comp|0;
 $img_idc = $img_idc|0;
 $img_imp = $img_imp|0;
 $source_str = $source_str|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0.0, $143 = 0.0, $144 = 0.0, $145 = 0, $146 = 0.0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0.0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $date_str = 0, $field = 0, $item = 0, $new_val = 0;
 var $ppcm = 0, $record = 0, $record_type = 0, $ret = 0, $subfield = 0, $uint_str = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr25 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer22 = sp + 72|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $record = sp + 100|0;
 $field = sp + 96|0;
 $subfield = sp + 92|0;
 $item = sp + 88|0;
 $date_str = sp + 84|0;
 $uint_str = sp + 160|0;
 $1 = $orecord;
 $2 = $idata;
 $3 = $ilen;
 $4 = $iw;
 $5 = $ih;
 $6 = $id;
 $7 = $ppmm;
 $8 = $img_comp;
 $9 = $img_idc;
 $10 = $img_imp;
 $11 = $source_str;
 $12 = $6;
 $13 = ($12|0)!=(8);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  $15 = $6;
  HEAP32[$vararg_buffer>>2] = $15;
  (_fprintf(($14|0),(21672|0),($vararg_buffer|0))|0);
  $0 = -2;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $16 = $8;
 $17 = (_strcmp($16,21432)|0);
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = $8;
  $20 = (_strcmp($19,21768)|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $8;
   HEAP32[$vararg_buffer1>>2] = $23;
   (_fprintf(($22|0),(21776|0),($vararg_buffer1|0))|0);
   $0 = -3;
   $295 = $0;
   STACKTOP = sp;return ($295|0);
  }
 }
 $record_type = 14;
 $24 = $record_type;
 $25 = (_new_ANSI_NIST_record($record,$24)|0);
 $ret = $25;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $ret;
  $0 = $27;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $28 = $record_type;
 $29 = (_value2field($field,$28,1,21560)|0);
 $ret = $29;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($31);
  $32 = $ret;
  $0 = $32;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $33 = HEAP32[$record>>2]|0;
 $34 = HEAP32[$field>>2]|0;
 $35 = (_append_ANSI_NIST_record($33,$34)|0);
 $ret = $35;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($37);
  $38 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($38);
  $39 = $ret;
  $0 = $39;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $40 = $9;
 HEAP32[$vararg_buffer4>>2] = $40;
 (_sprintf($uint_str,21568,$vararg_buffer4)|0);
 $41 = $record_type;
 $42 = (_value2field($field,$41,2,$uint_str)|0);
 $ret = $42;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($44);
  $45 = $ret;
  $0 = $45;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $46 = HEAP32[$record>>2]|0;
 $47 = HEAP32[$field>>2]|0;
 $48 = (_append_ANSI_NIST_record($46,$47)|0);
 $ret = $48;
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($50);
  $51 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($51);
  $52 = $ret;
  $0 = $52;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $53 = $10;
 HEAP32[$vararg_buffer7>>2] = $53;
 (_sprintf($uint_str,21568,$vararg_buffer7)|0);
 $54 = $record_type;
 $55 = (_value2field($field,$54,3,$uint_str)|0);
 $ret = $55;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($57);
  $58 = $ret;
  $0 = $58;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $59 = HEAP32[$record>>2]|0;
 $60 = HEAP32[$field>>2]|0;
 $61 = (_append_ANSI_NIST_record($59,$60)|0);
 $ret = $61;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($63);
  $64 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($64);
  $65 = $ret;
  $0 = $65;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $66 = $record_type;
 $67 = $11;
 $68 = (_value2field($field,$66,4,$67)|0);
 $ret = $68;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($70);
  $71 = $ret;
  $0 = $71;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $72 = HEAP32[$record>>2]|0;
 $73 = HEAP32[$field>>2]|0;
 $74 = (_append_ANSI_NIST_record($72,$73)|0);
 $ret = $74;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($76);
  $77 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($77);
  $78 = $ret;
  $0 = $78;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $79 = (_get_ANSI_NIST_date($date_str)|0);
 $ret = $79;
 $80 = ($79|0)!=(0);
 if ($80) {
  $81 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($81);
  $82 = $ret;
  $0 = $82;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $83 = $record_type;
 $84 = HEAP32[$date_str>>2]|0;
 $85 = (_value2field($field,$83,5,$84)|0);
 $ret = $85;
 $86 = ($85|0)!=(0);
 if ($86) {
  $87 = HEAP32[$date_str>>2]|0;
  _free($87);
  $88 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($88);
  $89 = $ret;
  $0 = $89;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $90 = HEAP32[$date_str>>2]|0;
 _free($90);
 $91 = HEAP32[$record>>2]|0;
 $92 = HEAP32[$field>>2]|0;
 $93 = (_append_ANSI_NIST_record($91,$92)|0);
 $ret = $93;
 $94 = ($93|0)!=(0);
 if ($94) {
  $95 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($95);
  $96 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($96);
  $97 = $ret;
  $0 = $97;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $98 = $4;
 HEAP32[$vararg_buffer10>>2] = $98;
 (_sprintf($uint_str,21576,$vararg_buffer10)|0);
 $99 = $record_type;
 $100 = (_value2field($field,$99,6,$uint_str)|0);
 $ret = $100;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($102);
  $103 = $ret;
  $0 = $103;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $104 = HEAP32[$record>>2]|0;
 $105 = HEAP32[$field>>2]|0;
 $106 = (_append_ANSI_NIST_record($104,$105)|0);
 $ret = $106;
 $107 = ($106|0)!=(0);
 if ($107) {
  $108 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($108);
  $109 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($109);
  $110 = $ret;
  $0 = $110;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $111 = $5;
 HEAP32[$vararg_buffer13>>2] = $111;
 (_sprintf($uint_str,21576,$vararg_buffer13)|0);
 $112 = $record_type;
 $113 = (_value2field($field,$112,7,$uint_str)|0);
 $ret = $113;
 $114 = ($113|0)!=(0);
 if ($114) {
  $115 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($115);
  $116 = $ret;
  $0 = $116;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $117 = HEAP32[$record>>2]|0;
 $118 = HEAP32[$field>>2]|0;
 $119 = (_append_ANSI_NIST_record($117,$118)|0);
 $ret = $119;
 $120 = ($119|0)!=(0);
 if ($120) {
  $121 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($121);
  $122 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($122);
  $123 = $ret;
  $0 = $123;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $124 = $record_type;
 $125 = (_value2field($field,$124,8,21584)|0);
 $ret = $125;
 $126 = ($125|0)!=(0);
 if ($126) {
  $127 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($127);
  $128 = $ret;
  $0 = $128;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $129 = HEAP32[$record>>2]|0;
 $130 = HEAP32[$field>>2]|0;
 $131 = (_append_ANSI_NIST_record($129,$130)|0);
 $ret = $131;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($133);
  $134 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($134);
  $135 = $ret;
  $0 = $135;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $136 = $7;
 $137 = $136 * 10.0;
 $138 = $137 < 0.0;
 if ($138) {
  $139 = $7;
  $140 = $139 * 10.0;
  $141 = $140 - 0.5;
  $146 = $141;
 } else {
  $142 = $7;
  $143 = $142 * 10.0;
  $144 = $143 + 0.5;
  $146 = $144;
 }
 $145 = (~~(($146)));
 $ppcm = $145;
 $147 = $ppcm;
 HEAP32[$vararg_buffer16>>2] = $147;
 (_sprintf($uint_str,21576,$vararg_buffer16)|0);
 $148 = $record_type;
 $149 = (_value2field($field,$148,9,$uint_str)|0);
 $ret = $149;
 $150 = ($149|0)!=(0);
 if ($150) {
  $151 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($151);
  $152 = $ret;
  $0 = $152;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $153 = HEAP32[$record>>2]|0;
 $154 = HEAP32[$field>>2]|0;
 $155 = (_append_ANSI_NIST_record($153,$154)|0);
 $ret = $155;
 $156 = ($155|0)!=(0);
 if ($156) {
  $157 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($157);
  $158 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($158);
  $159 = $ret;
  $0 = $159;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $160 = $record_type;
 $161 = (_value2field($field,$160,10,$uint_str)|0);
 $ret = $161;
 $162 = ($161|0)!=(0);
 if ($162) {
  $163 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($163);
  $164 = $ret;
  $0 = $164;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $165 = HEAP32[$record>>2]|0;
 $166 = HEAP32[$field>>2]|0;
 $167 = (_append_ANSI_NIST_record($165,$166)|0);
 $ret = $167;
 $168 = ($167|0)!=(0);
 if ($168) {
  $169 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($169);
  $170 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($170);
  $171 = $ret;
  $0 = $171;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $172 = $record_type;
 $173 = $8;
 $174 = (_value2field($field,$172,11,$173)|0);
 $ret = $174;
 $175 = ($174|0)!=(0);
 if ($175) {
  $176 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($176);
  $177 = $ret;
  $0 = $177;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $178 = HEAP32[$record>>2]|0;
 $179 = HEAP32[$field>>2]|0;
 $180 = (_append_ANSI_NIST_record($178,$179)|0);
 $ret = $180;
 $181 = ($180|0)!=(0);
 if ($181) {
  $182 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($182);
  $183 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($183);
  $184 = $ret;
  $0 = $184;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $185 = $6;
 HEAP32[$vararg_buffer19>>2] = $185;
 (_sprintf($uint_str,21576,$vararg_buffer19)|0);
 $186 = $record_type;
 $187 = (_value2field($field,$186,12,$uint_str)|0);
 $ret = $187;
 $188 = ($187|0)!=(0);
 if ($188) {
  $189 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($189);
  $190 = $ret;
  $0 = $190;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $191 = HEAP32[$record>>2]|0;
 $192 = HEAP32[$field>>2]|0;
 $193 = (_append_ANSI_NIST_record($191,$192)|0);
 $ret = $193;
 $194 = ($193|0)!=(0);
 if ($194) {
  $195 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($195);
  $196 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($196);
  $197 = $ret;
  $0 = $197;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $198 = $record_type;
 $199 = (_value2field($field,$198,13,21560)|0);
 $ret = $199;
 $200 = ($199|0)!=(0);
 if ($200) {
  $201 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($201);
  $202 = $ret;
  $0 = $202;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $203 = HEAP32[$record>>2]|0;
 $204 = HEAP32[$field>>2]|0;
 $205 = (_append_ANSI_NIST_record($203,$204)|0);
 $ret = $205;
 $206 = ($205|0)!=(0);
 if ($206) {
  $207 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($207);
  $208 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($208);
  $209 = $ret;
  $0 = $209;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $210 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $210;
 $211 = ($210|0)!=(0);
 if ($211) {
  $212 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($212);
  $213 = $ret;
  $0 = $213;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $214 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $214;
 $215 = ($214|0)!=(0);
 if ($215) {
  $216 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($216);
  $217 = $ret;
  $0 = $217;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $218 = HEAP32[$item>>2]|0;
 $219 = (($218) + 12|0);
 $220 = HEAP32[$219>>2]|0;
 $221 = $3;
 $222 = (_realloc($220,$221)|0);
 $new_val = $222;
 $223 = $new_val;
 $224 = ($223|0)==(0|0);
 if ($224) {
  $225 = HEAP32[_stderr>>2]|0;
  $226 = HEAP32[$item>>2]|0;
  $227 = (($226) + 8|0);
  $228 = HEAP32[$227>>2]|0;
  $229 = $3;
  HEAP32[$vararg_buffer22>>2] = $228;
  $vararg_ptr25 = (($vararg_buffer22) + 4|0);
  HEAP32[$vararg_ptr25>>2] = $229;
  (_fprintf(($225|0),(21888|0),($vararg_buffer22|0))|0);
  $230 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($230);
  $231 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($231);
  $0 = -4;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $232 = $new_val;
 $233 = HEAP32[$item>>2]|0;
 $234 = (($233) + 12|0);
 HEAP32[$234>>2] = $232;
 $235 = $3;
 $236 = HEAP32[$item>>2]|0;
 $237 = (($236) + 8|0);
 HEAP32[$237>>2] = $235;
 $238 = HEAP32[$item>>2]|0;
 $239 = (($238) + 12|0);
 $240 = HEAP32[$239>>2]|0;
 $241 = $2;
 $242 = HEAP32[$item>>2]|0;
 $243 = (($242) + 8|0);
 $244 = HEAP32[$243>>2]|0;
 _memcpy(($240|0),($241|0),($244|0))|0;
 $245 = HEAP32[$item>>2]|0;
 $246 = (($245) + 8|0);
 $247 = HEAP32[$246>>2]|0;
 $248 = HEAP32[$item>>2]|0;
 HEAP32[$248>>2] = $247;
 $249 = HEAP32[$item>>2]|0;
 $250 = (($249) + 8|0);
 $251 = HEAP32[$250>>2]|0;
 $252 = HEAP32[$item>>2]|0;
 $253 = (($252) + 4|0);
 HEAP32[$253>>2] = $251;
 $254 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $254;
 $255 = ($254|0)!=(0);
 if ($255) {
  $256 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($256);
  $257 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($257);
  $258 = $ret;
  $0 = $258;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $259 = HEAP32[$subfield>>2]|0;
 $260 = HEAP32[$item>>2]|0;
 $261 = (_append_ANSI_NIST_subfield($259,$260)|0);
 $ret = $261;
 $262 = ($261|0)!=(0);
 if ($262) {
  $263 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($263);
  $264 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($264);
  $265 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($265);
  $266 = $ret;
  $0 = $266;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $267 = $record_type;
 $268 = (_new_ANSI_NIST_field($field,$267,999)|0);
 $ret = $268;
 $269 = ($268|0)!=(0);
 if ($269) {
  $270 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($270);
  $271 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($271);
  $272 = $ret;
  $0 = $272;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $273 = HEAP32[$field>>2]|0;
 $274 = HEAP32[$subfield>>2]|0;
 $275 = (_append_ANSI_NIST_field($273,$274)|0);
 $ret = $275;
 $276 = ($275|0)!=(0);
 if ($276) {
  $277 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($277);
  $278 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($278);
  $279 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($279);
  $280 = $ret;
  $0 = $280;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $281 = HEAP32[$record>>2]|0;
 $282 = HEAP32[$field>>2]|0;
 $283 = (_append_ANSI_NIST_record($281,$282)|0);
 $ret = $283;
 $284 = ($283|0)!=(0);
 if ($284) {
  $285 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($285);
  $286 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($286);
  $287 = $ret;
  $0 = $287;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 $288 = HEAP32[$record>>2]|0;
 $289 = (_update_ANSI_NIST_tagged_record_LEN($288)|0);
 $ret = $289;
 $290 = ($289|0)!=(0);
 if ($290) {
  $291 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($291);
  $292 = $ret;
  $0 = $292;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 } else {
  $293 = HEAP32[$record>>2]|0;
  $294 = $1;
  HEAP32[$294>>2] = $293;
  $0 = 0;
  $295 = $0;
  STACKTOP = sp;return ($295|0);
 }
 return 0|0;
}
function _image2type_13($orecord,$idata,$ilen,$iw,$ih,$id,$ppmm,$img_comp,$img_idc,$img_imp,$source_str) {
 $orecord = $orecord|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 $iw = $iw|0;
 $ih = $ih|0;
 $id = $id|0;
 $ppmm = +$ppmm;
 $img_comp = $img_comp|0;
 $img_idc = $img_idc|0;
 $img_imp = $img_imp|0;
 $source_str = $source_str|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0.0, $135 = 0, $136 = 0.0, $137 = 0.0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0.0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0.0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $date_str = 0, $field = 0, $item = 0, $new_val = 0, $ppcm = 0, $record = 0, $record_type = 0, $ret = 0, $subfield = 0, $uint_str = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr25 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer22 = sp + 72|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $record = sp + 100|0;
 $field = sp + 96|0;
 $subfield = sp + 92|0;
 $item = sp + 88|0;
 $date_str = sp + 84|0;
 $uint_str = sp + 160|0;
 $1 = $orecord;
 $2 = $idata;
 $3 = $ilen;
 $4 = $iw;
 $5 = $ih;
 $6 = $id;
 $7 = $ppmm;
 $8 = $img_comp;
 $9 = $img_idc;
 $10 = $img_imp;
 $11 = $source_str;
 $12 = $6;
 $13 = ($12|0)!=(8);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  $15 = $6;
  HEAP32[$vararg_buffer>>2] = $15;
  (_fprintf(($14|0),(21336|0),($vararg_buffer|0))|0);
  $0 = -2;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $16 = $8;
 $17 = (_strcmp($16,21432)|0);
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  $20 = $8;
  HEAP32[$vararg_buffer1>>2] = $20;
  (_fprintf(($19|0),(21440|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $record_type = 13;
 $21 = $record_type;
 $22 = (_new_ANSI_NIST_record($record,$21)|0);
 $ret = $22;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = $ret;
  $0 = $24;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $25 = $record_type;
 $26 = (_value2field($field,$25,1,21560)|0);
 $ret = $26;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($28);
  $29 = $ret;
  $0 = $29;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $30 = HEAP32[$record>>2]|0;
 $31 = HEAP32[$field>>2]|0;
 $32 = (_append_ANSI_NIST_record($30,$31)|0);
 $ret = $32;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($34);
  $35 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($35);
  $36 = $ret;
  $0 = $36;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $37 = $9;
 HEAP32[$vararg_buffer4>>2] = $37;
 (_sprintf($uint_str,21568,$vararg_buffer4)|0);
 $38 = $record_type;
 $39 = (_value2field($field,$38,2,$uint_str)|0);
 $ret = $39;
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($41);
  $42 = $ret;
  $0 = $42;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $43 = HEAP32[$record>>2]|0;
 $44 = HEAP32[$field>>2]|0;
 $45 = (_append_ANSI_NIST_record($43,$44)|0);
 $ret = $45;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($47);
  $48 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($48);
  $49 = $ret;
  $0 = $49;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $50 = $10;
 HEAP32[$vararg_buffer7>>2] = $50;
 (_sprintf($uint_str,21568,$vararg_buffer7)|0);
 $51 = $record_type;
 $52 = (_value2field($field,$51,3,$uint_str)|0);
 $ret = $52;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($54);
  $55 = $ret;
  $0 = $55;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $56 = HEAP32[$record>>2]|0;
 $57 = HEAP32[$field>>2]|0;
 $58 = (_append_ANSI_NIST_record($56,$57)|0);
 $ret = $58;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($60);
  $61 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($61);
  $62 = $ret;
  $0 = $62;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $63 = $record_type;
 $64 = $11;
 $65 = (_value2field($field,$63,4,$64)|0);
 $ret = $65;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($67);
  $68 = $ret;
  $0 = $68;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $69 = HEAP32[$record>>2]|0;
 $70 = HEAP32[$field>>2]|0;
 $71 = (_append_ANSI_NIST_record($69,$70)|0);
 $ret = $71;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($73);
  $74 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($74);
  $75 = $ret;
  $0 = $75;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $76 = (_get_ANSI_NIST_date($date_str)|0);
 $ret = $76;
 $77 = ($76|0)!=(0);
 if ($77) {
  $78 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($78);
  $79 = $ret;
  $0 = $79;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $80 = $record_type;
 $81 = HEAP32[$date_str>>2]|0;
 $82 = (_value2field($field,$80,5,$81)|0);
 $ret = $82;
 $83 = ($82|0)!=(0);
 if ($83) {
  $84 = HEAP32[$date_str>>2]|0;
  _free($84);
  $85 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($85);
  $86 = $ret;
  $0 = $86;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $87 = HEAP32[$date_str>>2]|0;
 _free($87);
 $88 = HEAP32[$record>>2]|0;
 $89 = HEAP32[$field>>2]|0;
 $90 = (_append_ANSI_NIST_record($88,$89)|0);
 $ret = $90;
 $91 = ($90|0)!=(0);
 if ($91) {
  $92 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($92);
  $93 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($93);
  $94 = $ret;
  $0 = $94;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $95 = $4;
 HEAP32[$vararg_buffer10>>2] = $95;
 (_sprintf($uint_str,21576,$vararg_buffer10)|0);
 $96 = $record_type;
 $97 = (_value2field($field,$96,6,$uint_str)|0);
 $ret = $97;
 $98 = ($97|0)!=(0);
 if ($98) {
  $99 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($99);
  $100 = $ret;
  $0 = $100;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $101 = HEAP32[$record>>2]|0;
 $102 = HEAP32[$field>>2]|0;
 $103 = (_append_ANSI_NIST_record($101,$102)|0);
 $ret = $103;
 $104 = ($103|0)!=(0);
 if ($104) {
  $105 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($105);
  $106 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($106);
  $107 = $ret;
  $0 = $107;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $108 = $5;
 HEAP32[$vararg_buffer13>>2] = $108;
 (_sprintf($uint_str,21576,$vararg_buffer13)|0);
 $109 = $record_type;
 $110 = (_value2field($field,$109,7,$uint_str)|0);
 $ret = $110;
 $111 = ($110|0)!=(0);
 if ($111) {
  $112 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($112);
  $113 = $ret;
  $0 = $113;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $114 = HEAP32[$record>>2]|0;
 $115 = HEAP32[$field>>2]|0;
 $116 = (_append_ANSI_NIST_record($114,$115)|0);
 $ret = $116;
 $117 = ($116|0)!=(0);
 if ($117) {
  $118 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($118);
  $119 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($119);
  $120 = $ret;
  $0 = $120;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $121 = $record_type;
 $122 = (_value2field($field,$121,8,21584)|0);
 $ret = $122;
 $123 = ($122|0)!=(0);
 if ($123) {
  $124 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($124);
  $125 = $ret;
  $0 = $125;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $126 = HEAP32[$record>>2]|0;
 $127 = HEAP32[$field>>2]|0;
 $128 = (_append_ANSI_NIST_record($126,$127)|0);
 $ret = $128;
 $129 = ($128|0)!=(0);
 if ($129) {
  $130 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($130);
  $131 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($131);
  $132 = $ret;
  $0 = $132;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $133 = $7;
 $134 = $133 * 10.0;
 $135 = $134 < 0.0;
 if ($135) {
  $136 = $7;
  $137 = $136 * 10.0;
  $138 = $137 - 0.5;
  $143 = $138;
 } else {
  $139 = $7;
  $140 = $139 * 10.0;
  $141 = $140 + 0.5;
  $143 = $141;
 }
 $142 = (~~(($143)));
 $ppcm = $142;
 $144 = $ppcm;
 HEAP32[$vararg_buffer16>>2] = $144;
 (_sprintf($uint_str,21576,$vararg_buffer16)|0);
 $145 = $record_type;
 $146 = (_value2field($field,$145,9,$uint_str)|0);
 $ret = $146;
 $147 = ($146|0)!=(0);
 if ($147) {
  $148 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($148);
  $149 = $ret;
  $0 = $149;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $150 = HEAP32[$record>>2]|0;
 $151 = HEAP32[$field>>2]|0;
 $152 = (_append_ANSI_NIST_record($150,$151)|0);
 $ret = $152;
 $153 = ($152|0)!=(0);
 if ($153) {
  $154 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($154);
  $155 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($155);
  $156 = $ret;
  $0 = $156;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $157 = $record_type;
 $158 = (_value2field($field,$157,10,$uint_str)|0);
 $ret = $158;
 $159 = ($158|0)!=(0);
 if ($159) {
  $160 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($160);
  $161 = $ret;
  $0 = $161;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $162 = HEAP32[$record>>2]|0;
 $163 = HEAP32[$field>>2]|0;
 $164 = (_append_ANSI_NIST_record($162,$163)|0);
 $ret = $164;
 $165 = ($164|0)!=(0);
 if ($165) {
  $166 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($166);
  $167 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($167);
  $168 = $ret;
  $0 = $168;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $169 = $record_type;
 $170 = $8;
 $171 = (_value2field($field,$169,11,$170)|0);
 $ret = $171;
 $172 = ($171|0)!=(0);
 if ($172) {
  $173 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($173);
  $174 = $ret;
  $0 = $174;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $175 = HEAP32[$record>>2]|0;
 $176 = HEAP32[$field>>2]|0;
 $177 = (_append_ANSI_NIST_record($175,$176)|0);
 $ret = $177;
 $178 = ($177|0)!=(0);
 if ($178) {
  $179 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($179);
  $180 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($180);
  $181 = $ret;
  $0 = $181;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $182 = $6;
 HEAP32[$vararg_buffer19>>2] = $182;
 (_sprintf($uint_str,21576,$vararg_buffer19)|0);
 $183 = $record_type;
 $184 = (_value2field($field,$183,12,$uint_str)|0);
 $ret = $184;
 $185 = ($184|0)!=(0);
 if ($185) {
  $186 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($186);
  $187 = $ret;
  $0 = $187;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $188 = HEAP32[$record>>2]|0;
 $189 = HEAP32[$field>>2]|0;
 $190 = (_append_ANSI_NIST_record($188,$189)|0);
 $ret = $190;
 $191 = ($190|0)!=(0);
 if ($191) {
  $192 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($192);
  $193 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($193);
  $194 = $ret;
  $0 = $194;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $195 = $record_type;
 $196 = (_value2field($field,$195,13,21560)|0);
 $ret = $196;
 $197 = ($196|0)!=(0);
 if ($197) {
  $198 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($198);
  $199 = $ret;
  $0 = $199;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $200 = HEAP32[$record>>2]|0;
 $201 = HEAP32[$field>>2]|0;
 $202 = (_append_ANSI_NIST_record($200,$201)|0);
 $ret = $202;
 $203 = ($202|0)!=(0);
 if ($203) {
  $204 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($204);
  $205 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($205);
  $206 = $ret;
  $0 = $206;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $207 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $207;
 $208 = ($207|0)!=(0);
 if ($208) {
  $209 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($209);
  $210 = $ret;
  $0 = $210;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $211 = HEAP32[$item>>2]|0;
 $212 = (($211) + 12|0);
 $213 = HEAP32[$212>>2]|0;
 $214 = $3;
 $215 = (_realloc($213,$214)|0);
 $new_val = $215;
 $216 = $new_val;
 $217 = ($216|0)==(0|0);
 if ($217) {
  $218 = HEAP32[_stderr>>2]|0;
  $219 = HEAP32[$item>>2]|0;
  $220 = (($219) + 8|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = $3;
  HEAP32[$vararg_buffer22>>2] = $221;
  $vararg_ptr25 = (($vararg_buffer22) + 4|0);
  HEAP32[$vararg_ptr25>>2] = $222;
  (_fprintf(($218|0),(21592|0),($vararg_buffer22|0))|0);
  $223 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($223);
  $224 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($224);
  $0 = -4;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $225 = $new_val;
 $226 = HEAP32[$item>>2]|0;
 $227 = (($226) + 12|0);
 HEAP32[$227>>2] = $225;
 $228 = $3;
 $229 = HEAP32[$item>>2]|0;
 $230 = (($229) + 8|0);
 HEAP32[$230>>2] = $228;
 $231 = HEAP32[$item>>2]|0;
 $232 = (($231) + 12|0);
 $233 = HEAP32[$232>>2]|0;
 $234 = $2;
 $235 = HEAP32[$item>>2]|0;
 $236 = (($235) + 8|0);
 $237 = HEAP32[$236>>2]|0;
 _memcpy(($233|0),($234|0),($237|0))|0;
 $238 = HEAP32[$item>>2]|0;
 $239 = (($238) + 8|0);
 $240 = HEAP32[$239>>2]|0;
 $241 = HEAP32[$item>>2]|0;
 HEAP32[$241>>2] = $240;
 $242 = HEAP32[$item>>2]|0;
 $243 = (($242) + 8|0);
 $244 = HEAP32[$243>>2]|0;
 $245 = HEAP32[$item>>2]|0;
 $246 = (($245) + 4|0);
 HEAP32[$246>>2] = $244;
 $247 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $247;
 $248 = ($247|0)!=(0);
 if ($248) {
  $249 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($249);
  $250 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($250);
  $251 = $ret;
  $0 = $251;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $252 = HEAP32[$subfield>>2]|0;
 $253 = HEAP32[$item>>2]|0;
 $254 = (_append_ANSI_NIST_subfield($252,$253)|0);
 $ret = $254;
 $255 = ($254|0)!=(0);
 if ($255) {
  $256 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($256);
  $257 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($257);
  $258 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($258);
  $259 = $ret;
  $0 = $259;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $260 = $record_type;
 $261 = (_new_ANSI_NIST_field($field,$260,999)|0);
 $ret = $261;
 $262 = ($261|0)!=(0);
 if ($262) {
  $263 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($263);
  $264 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($264);
  $265 = $ret;
  $0 = $265;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $266 = HEAP32[$field>>2]|0;
 $267 = HEAP32[$subfield>>2]|0;
 $268 = (_append_ANSI_NIST_field($266,$267)|0);
 $ret = $268;
 $269 = ($268|0)!=(0);
 if ($269) {
  $270 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($270);
  $271 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($271);
  $272 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($272);
  $273 = $ret;
  $0 = $273;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $274 = HEAP32[$record>>2]|0;
 $275 = HEAP32[$field>>2]|0;
 $276 = (_append_ANSI_NIST_record($274,$275)|0);
 $ret = $276;
 $277 = ($276|0)!=(0);
 if ($277) {
  $278 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($278);
  $279 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($279);
  $280 = $ret;
  $0 = $280;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 $281 = HEAP32[$record>>2]|0;
 $282 = (_update_ANSI_NIST_tagged_record_LEN($281)|0);
 $ret = $282;
 $283 = ($282|0)!=(0);
 if ($283) {
  $284 = HEAP32[$record>>2]|0;
  _free_ANSI_NIST_record($284);
  $285 = $ret;
  $0 = $285;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 } else {
  $286 = HEAP32[$record>>2]|0;
  $287 = $1;
  HEAP32[$287>>2] = $286;
  $0 = 0;
  $288 = $0;
  STACKTOP = sp;return ($288|0);
 }
 return 0|0;
}
function _update_ANSI_NIST($ansi_nist,$record) {
 $ansi_nist = $ansi_nist|0;
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $new_ptr = 0, $new_size = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $ansi_nist;
 $2 = $record;
 $3 = $1;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $1;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5|0)>=($8|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = (($10) + 12|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + 100)|0;
   $14 = $13<<2;
   $new_size = $14;
   $15 = $1;
   $16 = (($15) + 16|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $new_size;
   $19 = (_realloc($17,$18)|0);
   $new_ptr = $19;
   $20 = $new_ptr;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $28 = $new_ptr;
    $29 = $1;
    $30 = (($29) + 16|0);
    HEAP32[$30>>2] = $28;
    $31 = $1;
    $32 = (($31) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 100)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $1;
   $24 = (($23) + 12|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25<<2;
   $27 = $new_size;
   HEAP32[$vararg_buffer>>2] = $26;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $27;
   (_fprintf(($22|0),(21968|0),($vararg_buffer|0))|0);
   $0 = -2;
   $51 = $0;
   STACKTOP = sp;return ($51|0);
  }
 } while(0);
 $35 = $2;
 $36 = $1;
 $37 = (($36) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = $1;
 $41 = (($40) + 16|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($38<<2)|0);
 HEAP32[$43>>2] = $35;
 $44 = $2;
 $45 = (($44) + 8|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $1;
 $48 = (($47) + 4|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + ($46))|0;
 HEAP32[$48>>2] = $50;
 $0 = 0;
 $51 = $0;
 STACKTOP = sp;return ($51|0);
}
function _update_ANSI_NIST_record($record,$field) {
 $record = $record|0;
 $field = $field|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $new_ptr = 0, $new_size = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $record;
 $2 = $field;
 $3 = $1;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $1;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5|0)>=($8|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = (($10) + 16|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + 100)|0;
   $14 = $13<<2;
   $new_size = $14;
   $15 = $1;
   $16 = (($15) + 20|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $new_size;
   $19 = (_realloc($17,$18)|0);
   $new_ptr = $19;
   $20 = $new_ptr;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $28 = $new_ptr;
    $29 = $1;
    $30 = (($29) + 20|0);
    HEAP32[$30>>2] = $28;
    $31 = $1;
    $32 = (($31) + 16|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 100)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $1;
   $24 = (($23) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25<<2;
   $27 = $new_size;
   HEAP32[$vararg_buffer>>2] = $26;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $27;
   (_fprintf(($22|0),(22048|0),($vararg_buffer|0))|0);
   $0 = -2;
   $51 = $0;
   STACKTOP = sp;return ($51|0);
  }
 } while(0);
 $35 = $2;
 $36 = $1;
 $37 = (($36) + 12|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = $1;
 $41 = (($40) + 20|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($38<<2)|0);
 HEAP32[$43>>2] = $35;
 $44 = $2;
 $45 = (($44) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $1;
 $48 = (($47) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + ($46))|0;
 HEAP32[$48>>2] = $50;
 $0 = 0;
 $51 = $0;
 STACKTOP = sp;return ($51|0);
}
function _update_ANSI_NIST_field($field,$subfield) {
 $field = $field|0;
 $subfield = $subfield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $new_ptr = 0, $new_size = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $field;
 $2 = $subfield;
 $3 = $1;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $1;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5|0)>=($8|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = (($10) + 20|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + 100)|0;
   $14 = $13<<2;
   $new_size = $14;
   $15 = $1;
   $16 = (($15) + 24|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $new_size;
   $19 = (_realloc($17,$18)|0);
   $new_ptr = $19;
   $20 = $new_ptr;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $28 = $new_ptr;
    $29 = $1;
    $30 = (($29) + 24|0);
    HEAP32[$30>>2] = $28;
    $31 = $1;
    $32 = (($31) + 20|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 100)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $1;
   $24 = (($23) + 20|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25<<2;
   $27 = $new_size;
   HEAP32[$vararg_buffer>>2] = $26;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $27;
   (_fprintf(($22|0),(22128|0),($vararg_buffer|0))|0);
   $0 = -2;
   $50 = $0;
   STACKTOP = sp;return ($50|0);
  }
 } while(0);
 $35 = $2;
 $36 = $1;
 $37 = (($36) + 16|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = $1;
 $41 = (($40) + 24|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($38<<2)|0);
 HEAP32[$43>>2] = $35;
 $44 = $2;
 $45 = HEAP32[$44>>2]|0;
 $46 = $1;
 $47 = (($46) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + ($45))|0;
 HEAP32[$47>>2] = $49;
 $0 = 0;
 $50 = $0;
 STACKTOP = sp;return ($50|0);
}
function _update_ANSI_NIST_subfield($subfield,$item) {
 $subfield = $subfield|0;
 $item = $item|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $new_ptr = 0, $new_size = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $subfield;
 $2 = $item;
 $3 = $1;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $1;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5|0)>=($8|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + 100)|0;
   $14 = $13<<2;
   $new_size = $14;
   $15 = $1;
   $16 = (($15) + 12|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $new_size;
   $19 = (_realloc($17,$18)|0);
   $new_ptr = $19;
   $20 = $new_ptr;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $28 = $new_ptr;
    $29 = $1;
    $30 = (($29) + 12|0);
    HEAP32[$30>>2] = $28;
    $31 = $1;
    $32 = (($31) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 100)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $22 = HEAP32[_stderr>>2]|0;
   $23 = $1;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25<<2;
   $27 = $new_size;
   HEAP32[$vararg_buffer>>2] = $26;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $27;
   (_fprintf(($22|0),(22216|0),($vararg_buffer|0))|0);
   $0 = -2;
   $49 = $0;
   STACKTOP = sp;return ($49|0);
  }
 } while(0);
 $35 = $2;
 $36 = $1;
 $37 = (($36) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = $1;
 $41 = (($40) + 12|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($38<<2)|0);
 HEAP32[$43>>2] = $35;
 $44 = $2;
 $45 = HEAP32[$44>>2]|0;
 $46 = $1;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($45))|0;
 HEAP32[$46>>2] = $48;
 $0 = 0;
 $49 = $0;
 STACKTOP = sp;return ($49|0);
}
function _update_ANSI_NIST_item($item,$nextchar) {
 $item = $item|0;
 $nextchar = $nextchar|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $alloc_chars = 0, $new_ptr = 0, $new_size = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $item;
 $2 = $nextchar;
 $3 = $1;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 $alloc_chars = $6;
 $7 = $alloc_chars;
 $8 = $1;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7|0)>=($10|0);
 do {
  if ($11) {
   $12 = $1;
   $13 = (($12) + 8|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = (($14) + 100)|0;
   $new_size = $15;
   $16 = $1;
   $17 = (($16) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = $new_size;
   $20 = (_realloc($18,$19)|0);
   $new_ptr = $20;
   $21 = $new_ptr;
   $22 = ($21|0)==(0|0);
   if (!($22)) {
    $28 = $new_ptr;
    $29 = $1;
    $30 = (($29) + 12|0);
    HEAP32[$30>>2] = $28;
    $31 = $1;
    $32 = (($31) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 100)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $23 = HEAP32[_stderr>>2]|0;
   $24 = $1;
   $25 = (($24) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $new_size;
   HEAP32[$vararg_buffer>>2] = $26;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $27;
   (_fprintf(($23|0),(22304|0),($vararg_buffer|0))|0);
   $0 = -2;
   $55 = $0;
   STACKTOP = sp;return ($55|0);
  }
 } while(0);
 $35 = $2;
 $36 = $35&255;
 $37 = $1;
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + 1)|0;
 HEAP32[$38>>2] = $40;
 $41 = $1;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + ($39)|0);
 HEAP8[$44>>0] = $36;
 $45 = $1;
 $46 = (($45) + 4|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = $1;
 $49 = (($48) + 12|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + ($47)|0);
 HEAP8[$51>>0] = 0;
 $52 = $1;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) + 1)|0;
 HEAP32[$52>>2] = $54;
 $0 = 0;
 $55 = $0;
 STACKTOP = sp;return ($55|0);
}
function _update_ANSI_NIST_record_LEN($ansi_nist,$record_i) {
 $ansi_nist = $ansi_nist|0;
 $record_i = $record_i|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $lenfield = 0, $lenfield_i = 0, $lenitem = 0, $orig_recordlen = 0, $record = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer16 = 0;
 var $vararg_buffer2 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer16 = sp + 48|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $lenfield = sp + 84|0;
 $lenfield_i = sp + 80|0;
 $1 = $ansi_nist;
 $2 = $record_i;
 $3 = $2;
 $4 = ($3|0)<(0);
 if (!($4)) {
  $5 = $2;
  $6 = $1;
  $7 = (($6) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)>=($8|0);
  if (!($9)) {
   $16 = $2;
   $17 = $1;
   $18 = (($17) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + ($16<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $record = $21;
   $22 = $record;
   $23 = (_lookup_ANSI_NIST_field($lenfield,$lenfield_i,1,$22)|0);
   $24 = ($23|0)!=(0);
   if (!($24)) {
    $0 = 0;
    $111 = $0;
    STACKTOP = sp;return ($111|0);
   }
   $25 = HEAP32[$lenfield>>2]|0;
   $26 = (($25) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)!=(1);
   if (!($28)) {
    $29 = HEAP32[$lenfield>>2]|0;
    $30 = (($29) + 24|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) + 4|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = ($34|0)!=(1);
    if (!($35)) {
     $46 = HEAP32[$lenfield>>2]|0;
     $47 = (($46) + 24|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = HEAP32[$48>>2]|0;
     $50 = (($49) + 12|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = HEAP32[$51>>2]|0;
     $lenitem = $52;
     $53 = $lenitem;
     $54 = (($53) + 12|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (_atoi($55)|0);
     $orig_recordlen = $56;
     $57 = $orig_recordlen;
     $58 = $record;
     $59 = (($58) + 8|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($57|0)!=($60|0);
     do {
      if ($61) {
       $62 = $record;
       $63 = HEAP32[$62>>2]|0;
       $64 = (_binary_record($63)|0);
       $65 = ($64|0)!=(0);
       if (!($65)) {
        $84 = $record;
        $85 = (_update_ANSI_NIST_tagged_record_LEN($84)|0);
        $ret = $85;
        $86 = ($85|0)!=(0);
        if (!($86)) {
         $88 = $record;
         $89 = (($88) + 8|0);
         $90 = HEAP32[$89>>2]|0;
         $91 = $orig_recordlen;
         $92 = (($90) - ($91))|0;
         $93 = $1;
         $94 = (($93) + 4|0);
         $95 = HEAP32[$94>>2]|0;
         $96 = (($95) + ($92))|0;
         HEAP32[$94>>2] = $96;
         $97 = HEAP32[_stderr>>2]|0;
         $98 = $2;
         $99 = (($98) + 1)|0;
         $100 = HEAP32[$lenfield_i>>2]|0;
         $101 = (($100) + 1)|0;
         $102 = $record;
         $103 = HEAP32[$102>>2]|0;
         $104 = HEAP32[$lenfield>>2]|0;
         $105 = (($104) + 8|0);
         $106 = HEAP32[$105>>2]|0;
         $107 = $orig_recordlen;
         $108 = $record;
         $109 = (($108) + 8|0);
         $110 = HEAP32[$109>>2]|0;
         HEAP32[$vararg_buffer16>>2] = $99;
         $vararg_ptr19 = (($vararg_buffer16) + 4|0);
         HEAP32[$vararg_ptr19>>2] = $101;
         $vararg_ptr20 = (($vararg_buffer16) + 8|0);
         HEAP32[$vararg_ptr20>>2] = $103;
         $vararg_ptr21 = (($vararg_buffer16) + 12|0);
         HEAP32[$vararg_ptr21>>2] = $106;
         $vararg_ptr22 = (($vararg_buffer16) + 16|0);
         HEAP32[$vararg_ptr22>>2] = $107;
         $vararg_ptr23 = (($vararg_buffer16) + 20|0);
         HEAP32[$vararg_ptr23>>2] = $110;
         (_fprintf(($97|0),(22576|0),($vararg_buffer16|0))|0);
         break;
        }
        $87 = $ret;
        $0 = $87;
        $111 = $0;
        STACKTOP = sp;return ($111|0);
       }
       $66 = $record;
       $67 = (_update_ANSI_NIST_binary_record_LEN($66)|0);
       $ret = $67;
       $68 = ($67|0)!=(0);
       if ($68) {
        $69 = $ret;
        $0 = $69;
        $111 = $0;
        STACKTOP = sp;return ($111|0);
       } else {
        $70 = HEAP32[_stderr>>2]|0;
        $71 = $2;
        $72 = (($71) + 1)|0;
        $73 = HEAP32[$lenfield_i>>2]|0;
        $74 = (($73) + 1)|0;
        $75 = $record;
        $76 = HEAP32[$75>>2]|0;
        $77 = HEAP32[$lenfield>>2]|0;
        $78 = (($77) + 8|0);
        $79 = HEAP32[$78>>2]|0;
        $80 = $orig_recordlen;
        $81 = $record;
        $82 = (($81) + 8|0);
        $83 = HEAP32[$82>>2]|0;
        HEAP32[$vararg_buffer8>>2] = $72;
        $vararg_ptr11 = (($vararg_buffer8) + 4|0);
        HEAP32[$vararg_ptr11>>2] = $74;
        $vararg_ptr12 = (($vararg_buffer8) + 8|0);
        HEAP32[$vararg_ptr12>>2] = $76;
        $vararg_ptr13 = (($vararg_buffer8) + 12|0);
        HEAP32[$vararg_ptr13>>2] = $79;
        $vararg_ptr14 = (($vararg_buffer8) + 16|0);
        HEAP32[$vararg_ptr14>>2] = $80;
        $vararg_ptr15 = (($vararg_buffer8) + 20|0);
        HEAP32[$vararg_ptr15>>2] = $83;
        (_fprintf(($70|0),(22576|0),($vararg_buffer8|0))|0);
        $0 = 0;
        $111 = $0;
        STACKTOP = sp;return ($111|0);
       }
      }
     } while(0);
     $0 = 0;
     $111 = $0;
     STACKTOP = sp;return ($111|0);
    }
   }
   $36 = HEAP32[_stderr>>2]|0;
   $37 = $2;
   $38 = (($37) + 1)|0;
   $39 = HEAP32[$lenfield_i>>2]|0;
   $40 = (($39) + 1)|0;
   $41 = $record;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$lenfield>>2]|0;
   $44 = (($43) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   HEAP32[$vararg_buffer2>>2] = $38;
   $vararg_ptr5 = (($vararg_buffer2) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $40;
   $vararg_ptr6 = (($vararg_buffer2) + 8|0);
   HEAP32[$vararg_ptr6>>2] = $42;
   $vararg_ptr7 = (($vararg_buffer2) + 12|0);
   HEAP32[$vararg_ptr7>>2] = $45;
   (_fprintf(($36|0),(22472|0),($vararg_buffer2|0))|0);
   $0 = -4;
   $111 = $0;
   STACKTOP = sp;return ($111|0);
  }
 }
 $10 = HEAP32[_stderr>>2]|0;
 $11 = $2;
 $12 = (($11) + 1)|0;
 $13 = $1;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$vararg_buffer>>2] = $12;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $15;
 (_fprintf(($10|0),(22392|0),($vararg_buffer|0))|0);
 $0 = -2;
 $111 = $0;
 STACKTOP = sp;return ($111|0);
}
function _update_ANSI_NIST_binary_record_LEN($record) {
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $field = 0, $item = 0, $new_ptr = 0, $value = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $value = sp + 36|0;
 $1 = $record;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $field = $5;
 $6 = $field;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $item = $12;
 $13 = $1;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$vararg_buffer>>2] = $15;
 (_sprintf($value,22640,$vararg_buffer)|0);
 $16 = (_strlen(($value|0))|0);
 $17 = $item;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($16>>>0)>=($19>>>0);
 do {
  if ($20) {
   $21 = $item;
   $22 = (($21) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_realloc($23,11)|0);
   $new_ptr = $24;
   $25 = $new_ptr;
   $26 = ($25|0)==(0|0);
   if (!($26)) {
    $31 = $new_ptr;
    $32 = $item;
    $33 = (($32) + 12|0);
    HEAP32[$33>>2] = $31;
    $34 = $item;
    $35 = (($34) + 8|0);
    HEAP32[$35>>2] = 11;
    break;
   }
   $27 = HEAP32[_stderr>>2]|0;
   $28 = $item;
   $29 = (($28) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $30;
   $vararg_ptr4 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr4>>2] = 11;
   (_fprintf(($27|0),(22648|0),($vararg_buffer1|0))|0);
   $0 = -2;
   $42 = $0;
   STACKTOP = sp;return ($42|0);
  }
 } while(0);
 $36 = $item;
 $37 = (($36) + 12|0);
 $38 = HEAP32[$37>>2]|0;
 (_strcpy(($38|0),($value|0))|0);
 $39 = (_strlen(($value|0))|0);
 $40 = $item;
 $41 = (($40) + 4|0);
 HEAP32[$41>>2] = $39;
 $0 = 0;
 $42 = $0;
 STACKTOP = sp;return ($42|0);
}
function _update_ANSI_NIST_tagged_record_LEN($record) {
 $record = $record|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $byte_adjust = 0, $field = 0, $item = 0, $new_ptr = 0, $old_recordchars = 0, $old_recordlen = 0, $recordchars = 0, $recordlen = 0;
 var $value = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $value = sp + 64|0;
 $1 = $record;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $field = $5;
 $6 = $field;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $item = $12;
 $13 = $item;
 $14 = (($13) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_atoi($15)|0);
 $recordlen = $16;
 $17 = $recordlen;
 HEAP32[$vararg_buffer>>2] = $17;
 (_sprintf($value,22640,$vararg_buffer)|0);
 $18 = (_strlen(($value|0))|0);
 $recordchars = $18;
 while(1) {
  $19 = $recordlen;
  $20 = $1;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($19|0)!=($22|0);
  if (!($23)) {
   label = 12;
   break;
  }
  $24 = $recordlen;
  $old_recordlen = $24;
  $25 = $recordchars;
  $old_recordchars = $25;
  $26 = $1;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $recordlen = $28;
  $29 = $recordlen;
  HEAP32[$vararg_buffer1>>2] = $29;
  (_sprintf($value,22640,$vararg_buffer1)|0);
  $30 = (_strlen(($value|0))|0);
  $recordchars = $30;
  $31 = $old_recordchars;
  $32 = $recordchars;
  $33 = ($31|0)!=($32|0);
  if ($33) {
   $34 = $recordchars;
   $35 = $old_recordchars;
   $36 = (($34) - ($35))|0;
   $byte_adjust = $36;
   $37 = $byte_adjust;
   $38 = $field;
   $39 = (($38) + 24|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + ($37))|0;
   HEAP32[$41>>2] = $43;
   $44 = $byte_adjust;
   $45 = $field;
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($44))|0;
   HEAP32[$46>>2] = $48;
   $49 = $byte_adjust;
   $50 = $1;
   $51 = (($50) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + ($49))|0;
   HEAP32[$51>>2] = $53;
  }
  $54 = (_strlen(($value|0))|0);
  $55 = $item;
  $56 = (($55) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ($54>>>0)>=($57>>>0);
  if ($58) {
   $59 = $item;
   $60 = (($59) + 12|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (_realloc($61,11)|0);
   $new_ptr = $62;
   $63 = $new_ptr;
   $64 = ($63|0)==(0|0);
   if ($64) {
    label = 7;
    break;
   }
   $69 = $new_ptr;
   $70 = $item;
   $71 = (($70) + 12|0);
   HEAP32[$71>>2] = $69;
   $72 = $item;
   $73 = (($72) + 8|0);
   HEAP32[$73>>2] = 11;
  }
  $74 = $item;
  $75 = (($74) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  (_strcpy(($76|0),($value|0))|0);
  $77 = (_strlen(($value|0))|0);
  $78 = $item;
  $79 = (($78) + 4|0);
  HEAP32[$79>>2] = $77;
  $80 = $item;
  $81 = (($80) + 4|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = $item;
  HEAP32[$83>>2] = $82;
  $84 = $item;
  $85 = (($84) + 16|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = $item;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 1)|0;
   HEAP32[$88>>2] = $90;
  }
 }
 if ((label|0) == 7) {
  $65 = HEAP32[_stderr>>2]|0;
  $66 = $item;
  $67 = (($66) + 8|0);
  $68 = HEAP32[$67>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $68;
  $vararg_ptr7 = (($vararg_buffer4) + 4|0);
  HEAP32[$vararg_ptr7>>2] = 11;
  (_fprintf(($65|0),(22752|0),($vararg_buffer4|0))|0);
  $0 = -2;
  $91 = $0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 12) {
  $0 = 0;
  $91 = $0;
  STACKTOP = sp;return ($91|0);
 }
 return 0|0;
}
function _increment_numeric_item($record_i,$field_i,$subfield_i,$item_i,$ansi_nist,$format) {
 $record_i = $record_i|0;
 $field_i = $field_i|0;
 $subfield_i = $subfield_i|0;
 $item_i = $item_i|0;
 $ansi_nist = $ansi_nist|0;
 $format = $format|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, $itemint = 0, $itemvalue = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $record_i;
 $2 = $field_i;
 $3 = $subfield_i;
 $4 = $item_i;
 $5 = $ansi_nist;
 $6 = $format;
 $7 = $4;
 $8 = $3;
 $9 = $2;
 $10 = $1;
 $11 = $5;
 $12 = (($11) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<2)|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($9<<2)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + ($8<<2)|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($7<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 $item = $27;
 $28 = $item;
 $29 = (($28) + 12|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (_atoi($30)|0);
 $itemint = $31;
 $32 = $itemint;
 $33 = (($32) + 1)|0;
 $itemint = $33;
 $34 = $item;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + 2)|0;
 $38 = (_malloc($37)|0);
 $itemvalue = $38;
 $39 = $itemvalue;
 $40 = ($39|0)==(0|0);
 if ($40) {
  $41 = HEAP32[_stderr>>2]|0;
  $42 = $item;
  $43 = (($42) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + 2)|0;
  HEAP32[$vararg_buffer>>2] = $45;
  (_fprintf(($41|0),(22856|0),($vararg_buffer|0))|0);
  $0 = -2;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 $46 = $6;
 $47 = ($46|0)!=(0|0);
 if ($47) {
  $48 = $itemvalue;
  $49 = $6;
  $50 = $itemint;
  HEAP32[$vararg_buffer1>>2] = $50;
  (_sprintf($48,$49,$vararg_buffer1)|0);
 } else {
  $51 = $itemvalue;
  $52 = $itemint;
  HEAP32[$vararg_buffer4>>2] = $52;
  (_sprintf($51,22920,$vararg_buffer4)|0);
 }
 $53 = $1;
 $54 = $2;
 $55 = $3;
 $56 = $4;
 $57 = $itemvalue;
 $58 = $5;
 $59 = (_substitute_ANSI_NIST_item($53,$54,$55,$56,$57,$58)|0);
 $ret = $59;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = $itemvalue;
  _free($61);
  $62 = $ret;
  $0 = $62;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 } else {
  $63 = $itemvalue;
  _free($63);
  $0 = 0;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 return 0|0;
}
function _value2field($ofield,$record_type,$field_int,$value) {
 $ofield = $ofield|0;
 $record_type = $record_type|0;
 $field_int = $field_int|0;
 $value = $value|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $field = 0, $ret = 0, $subfield = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $subfield = sp + 4|0;
 $field = sp;
 $1 = $ofield;
 $2 = $record_type;
 $3 = $field_int;
 $4 = $value;
 $5 = $4;
 $6 = (_value2subfield($subfield,$5)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $9 = $2;
 $10 = $3;
 $11 = (_new_ANSI_NIST_field($field,$9,$10)|0);
 $ret = $11;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($13);
  $14 = $ret;
  $0 = $14;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $15 = HEAP32[$field>>2]|0;
 $16 = HEAP32[$subfield>>2]|0;
 $17 = (_update_ANSI_NIST_field($15,$16)|0);
 $ret = $17;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($19);
  $20 = HEAP32[$field>>2]|0;
  _free_ANSI_NIST_field($20);
  $21 = $ret;
  $0 = $21;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 } else {
  $22 = HEAP32[$field>>2]|0;
  $23 = $1;
  HEAP32[$23>>2] = $22;
  $0 = 0;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _value2subfield($osubfield,$value) {
 $osubfield = $osubfield|0;
 $value = $value|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $item = 0, $ret = 0, $subfield = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $item = sp + 4|0;
 $subfield = sp;
 $1 = $osubfield;
 $2 = $value;
 $3 = $2;
 $4 = (_value2item($item,$3)|0);
 $ret = $4;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = $ret;
  $0 = $6;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 $7 = (_alloc_ANSI_NIST_subfield($subfield)|0);
 $ret = $7;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($9);
  $10 = $ret;
  $0 = $10;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 $11 = HEAP32[$subfield>>2]|0;
 $12 = HEAP32[$item>>2]|0;
 $13 = (_update_ANSI_NIST_subfield($11,$12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$item>>2]|0;
  _free_ANSI_NIST_item($15);
  $16 = HEAP32[$subfield>>2]|0;
  _free_ANSI_NIST_subfield($16);
  $17 = $ret;
  $0 = $17;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 } else {
  $18 = HEAP32[$subfield>>2]|0;
  $19 = $1;
  HEAP32[$19>>2] = $18;
  $0 = 0;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _value2item($oitem,$value) {
 $oitem = $oitem|0;
 $value = $value|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $alloc_len = 0, $item = 0, $new_ptr = 0, $ret = 0, $value_len = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $item = sp + 20|0;
 $1 = $oitem;
 $2 = $value;
 $3 = $2;
 $4 = (_strlen(($3|0))|0);
 $value_len = $4;
 $5 = $value_len;
 $6 = (($5) + 1)|0;
 $alloc_len = $6;
 $7 = (_alloc_ANSI_NIST_item($item)|0);
 $ret = $7;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $ret;
  $0 = $9;
  $48 = $0;
  STACKTOP = sp;return ($48|0);
 }
 $10 = $alloc_len;
 $11 = HEAP32[$item>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)>=($13|0);
 do {
  if ($14) {
   $15 = HEAP32[$item>>2]|0;
   $16 = (($15) + 12|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $alloc_len;
   $19 = (_realloc($17,$18)|0);
   $new_ptr = $19;
   $20 = $new_ptr;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $27 = $new_ptr;
    $28 = HEAP32[$item>>2]|0;
    $29 = (($28) + 12|0);
    HEAP32[$29>>2] = $27;
    $30 = $alloc_len;
    $31 = HEAP32[$item>>2]|0;
    $32 = (($31) + 8|0);
    HEAP32[$32>>2] = $30;
    break;
   }
   $22 = HEAP32[_stderr>>2]|0;
   $23 = HEAP32[$item>>2]|0;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $alloc_len;
   HEAP32[$vararg_buffer>>2] = $25;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $26;
   (_fprintf(($22|0),(22928|0),($vararg_buffer|0))|0);
   $0 = -2;
   $48 = $0;
   STACKTOP = sp;return ($48|0);
  }
 } while(0);
 $33 = HEAP32[$item>>2]|0;
 $34 = (($33) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $2;
 (_strcpy(($35|0),($36|0))|0);
 $37 = $value_len;
 $38 = HEAP32[$item>>2]|0;
 $39 = (($38) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + ($37))|0;
 HEAP32[$39>>2] = $41;
 $42 = $value_len;
 $43 = HEAP32[$item>>2]|0;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($42))|0;
 HEAP32[$43>>2] = $45;
 $46 = HEAP32[$item>>2]|0;
 $47 = $1;
 HEAP32[$47>>2] = $46;
 $0 = 0;
 $48 = $0;
 STACKTOP = sp;return ($48|0);
}
function _get_ANSI_NIST_date($odate_str) {
 $odate_str = $odate_str|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cptr = 0, $date_str = 0, $day_i = 0, $dayptr = 0, $monthptr = 0, $tdate_str = 0, $tm = 0, $uint_str = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer25 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $yearptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer25 = sp + 72|0;
 $vararg_buffer22 = sp + 64|0;
 $vararg_buffer19 = sp + 56|0;
 $vararg_buffer16 = sp + 48|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $uint_str = sp + 116|0;
 $tm = sp + 80|0;
 $1 = $odate_str;
 $2 = (_time((0|0))|0);
 HEAP32[$tm>>2] = $2;
 $3 = (_ctime(($tm|0))|0);
 $tdate_str = $3;
 $4 = $tdate_str;
 $5 = (_strchr($4,32)|0);
 $cptr = $5;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $tdate_str;
  HEAP32[$vararg_buffer>>2] = $8;
  (_fprintf(($7|0),(23000|0),($vararg_buffer|0))|0);
  $0 = -2;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $9 = $cptr;
 $10 = (($9) + 1|0);
 $cptr = $10;
 $11 = $cptr;
 $monthptr = $11;
 $12 = $cptr;
 $13 = (_strchr($12,32)|0);
 $cptr = $13;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = $tdate_str;
  HEAP32[$vararg_buffer1>>2] = $16;
  (_fprintf(($15|0),(23072|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $17 = $cptr;
 $18 = (($17) + 1|0);
 $cptr = $18;
 HEAP8[$17>>0] = 0;
 while(1) {
  $19 = $cptr;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($21|0)==(32);
  if (!($22)) {
   break;
  }
  $23 = $cptr;
  $24 = (($23) + 1|0);
  $cptr = $24;
 }
 $25 = $cptr;
 $dayptr = $25;
 $26 = $cptr;
 $27 = (_strchr($26,32)|0);
 $cptr = $27;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[_stderr>>2]|0;
  $30 = $tdate_str;
  HEAP32[$vararg_buffer4>>2] = $30;
  (_fprintf(($29|0),(23144|0),($vararg_buffer4|0))|0);
  $0 = -4;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $31 = $cptr;
 $32 = (($31) + 1|0);
 $cptr = $32;
 HEAP8[$31>>0] = 0;
 while(1) {
  $33 = $cptr;
  $34 = HEAP8[$33>>0]|0;
  $35 = $34 << 24 >> 24;
  $36 = ($35|0)==(32);
  if (!($36)) {
   break;
  }
  $37 = $cptr;
  $38 = (($37) + 1|0);
  $cptr = $38;
 }
 $39 = $cptr;
 $40 = (_strchr($39,32)|0);
 $cptr = $40;
 $41 = ($40|0)==(0|0);
 if ($41) {
  $42 = HEAP32[_stderr>>2]|0;
  $43 = $tdate_str;
  HEAP32[$vararg_buffer7>>2] = $43;
  (_fprintf(($42|0),(23216|0),($vararg_buffer7|0))|0);
  $0 = -5;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $44 = $cptr;
 $45 = (($44) + 1|0);
 $cptr = $45;
 while(1) {
  $46 = $cptr;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47 << 24 >> 24;
  $49 = ($48|0)==(32);
  if (!($49)) {
   break;
  }
  $50 = $cptr;
  $51 = (($50) + 1|0);
  $cptr = $51;
 }
 $52 = $cptr;
 $yearptr = $52;
 $53 = $cptr;
 $54 = (_strchr($53,10)|0);
 $cptr = $54;
 $55 = ($54|0)==(0|0);
 if ($55) {
  $56 = HEAP32[_stderr>>2]|0;
  $57 = $tdate_str;
  HEAP32[$vararg_buffer10>>2] = $57;
  (_fprintf(($56|0),(23288|0),($vararg_buffer10|0))|0);
  $0 = -6;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $58 = $cptr;
 HEAP8[$58>>0] = 0;
 $59 = (_calloc(9,1)|0);
 $date_str = $59;
 $60 = ($59|0)==(0|0);
 if ($60) {
  $61 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer13>>2] = 9;
  (_fprintf(($61|0),(23360|0),($vararg_buffer13|0))|0);
  $0 = -7;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $62 = $yearptr;
 $63 = (_strlen(($62|0))|0);
 $64 = ($63|0)!=(4);
 if ($64) {
  $65 = HEAP32[_stderr>>2]|0;
  $66 = $yearptr;
  HEAP32[$vararg_buffer16>>2] = $66;
  (_fprintf(($65|0),(23424|0),($vararg_buffer16|0))|0);
  $67 = $date_str;
  _free($67);
  $0 = -8;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 $68 = $date_str;
 $69 = $yearptr;
 (_strcpy(($68|0),($69|0))|0);
 $70 = $monthptr;
 $71 = (_strcmp($70,23488)|0);
 $72 = ($71|0)==(0);
 if ($72) {
  $73 = $date_str;
  (_strcat(($73|0),(23496|0))|0);
 } else {
  $74 = $monthptr;
  $75 = (_strcmp($74,23504)|0);
  $76 = ($75|0)==(0);
  if ($76) {
   $77 = $date_str;
   (_strcat(($77|0),(23512|0))|0);
  } else {
   $78 = $monthptr;
   $79 = (_strcmp($78,23520)|0);
   $80 = ($79|0)==(0);
   if ($80) {
    $81 = $date_str;
    (_strcat(($81|0),(23528|0))|0);
   } else {
    $82 = $monthptr;
    $83 = (_strcmp($82,23536)|0);
    $84 = ($83|0)==(0);
    if ($84) {
     $85 = $date_str;
     (_strcat(($85|0),(23544|0))|0);
    } else {
     $86 = $monthptr;
     $87 = (_strcmp($86,23552)|0);
     $88 = ($87|0)==(0);
     if ($88) {
      $89 = $date_str;
      (_strcat(($89|0),(23560|0))|0);
     } else {
      $90 = $monthptr;
      $91 = (_strcmp($90,23568)|0);
      $92 = ($91|0)==(0);
      if ($92) {
       $93 = $date_str;
       (_strcat(($93|0),(23576|0))|0);
      } else {
       $94 = $monthptr;
       $95 = (_strcmp($94,23584)|0);
       $96 = ($95|0)==(0);
       if ($96) {
        $97 = $date_str;
        (_strcat(($97|0),(23592|0))|0);
       } else {
        $98 = $monthptr;
        $99 = (_strcmp($98,23600)|0);
        $100 = ($99|0)==(0);
        if ($100) {
         $101 = $date_str;
         (_strcat(($101|0),(23608|0))|0);
        } else {
         $102 = $monthptr;
         $103 = (_strcmp($102,23616)|0);
         $104 = ($103|0)==(0);
         if ($104) {
          $105 = $date_str;
          (_strcat(($105|0),(23624|0))|0);
         } else {
          $106 = $monthptr;
          $107 = (_strcmp($106,23632)|0);
          $108 = ($107|0)==(0);
          if ($108) {
           $109 = $date_str;
           (_strcat(($109|0),(23640|0))|0);
          } else {
           $110 = $monthptr;
           $111 = (_strcmp($110,23648)|0);
           $112 = ($111|0)==(0);
           do {
            if ($112) {
             $113 = $date_str;
             (_strcat(($113|0),(23656|0))|0);
            } else {
             $114 = $monthptr;
             $115 = (_strcmp($114,23664)|0);
             $116 = ($115|0)==(0);
             if ($116) {
              $117 = $date_str;
              (_strcat(($117|0),(23672|0))|0);
              break;
             } else {
              $118 = HEAP32[_stderr>>2]|0;
              $119 = $monthptr;
              HEAP32[$vararg_buffer19>>2] = $119;
              (_fprintf(($118|0),(23680|0),($vararg_buffer19|0))|0);
              $120 = $date_str;
              _free($120);
              $0 = -9;
              $131 = $0;
              STACKTOP = sp;return ($131|0);
             }
            }
           } while(0);
          }
         }
        }
       }
      }
     }
    }
   }
  }
 }
 $121 = $dayptr;
 $122 = (_atoi($121)|0);
 $day_i = $122;
 $123 = $day_i;
 HEAP32[$vararg_buffer22>>2] = $123;
 $124 = (_sprintf($uint_str,23736,$vararg_buffer22)|0);
 $125 = ($124|0)!=(2);
 if ($125) {
  $126 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer25>>2] = $uint_str;
  (_fprintf(($126|0),(23744|0),($vararg_buffer25|0))|0);
  $127 = $date_str;
  _free($127);
  $0 = -10;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 } else {
  $128 = $date_str;
  (_strcat(($128|0),($uint_str|0))|0);
  $129 = $date_str;
  $130 = $1;
  HEAP32[$130>>2] = $129;
  $0 = 0;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 return 0|0;
}
function _decode_ANSI_NIST_image($odata,$ow,$oh,$od,$oppmm,$ansi_nist,$imgrecord_i,$intrlvflag) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppmm = $oppmm|0;
 $ansi_nist = $ansi_nist|0;
 $imgrecord_i = $imgrecord_i|0;
 $intrlvflag = $intrlvflag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $imgrecord = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppmm;
 $6 = $ansi_nist;
 $7 = $imgrecord_i;
 $8 = $intrlvflag;
 $9 = $7;
 $10 = ($9|0)<(1);
 if (!($10)) {
  $11 = $7;
  $12 = $6;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)>($14|0);
  if (!($15)) {
   $23 = $7;
   $24 = $6;
   $25 = (($24) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($23<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $imgrecord = $28;
   $29 = $imgrecord;
   $30 = HEAP32[$29>>2]|0;
   $31 = (_binary_image_record($30)|0);
   $32 = ($31|0)!=(0);
   if ($32) {
    $33 = $1;
    $34 = $2;
    $35 = $3;
    $36 = $4;
    $37 = $5;
    $38 = $6;
    $39 = $7;
    $40 = (_decode_binary_field_image($33,$34,$35,$36,$37,$38,$39)|0);
    $ret = $40;
    $41 = $ret;
    $0 = $41;
    $61 = $0;
    STACKTOP = sp;return ($61|0);
   }
   $42 = $imgrecord;
   $43 = HEAP32[$42>>2]|0;
   $44 = (_tagged_image_record($43)|0);
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = $1;
    $47 = $2;
    $48 = $3;
    $49 = $4;
    $50 = $5;
    $51 = $6;
    $52 = $7;
    $53 = $8;
    $54 = (_decode_tagged_field_image($46,$47,$48,$49,$50,$51,$52,$53)|0);
    $ret = $54;
    $55 = $ret;
    $0 = $55;
    $61 = $0;
    STACKTOP = sp;return ($61|0);
   } else {
    $56 = HEAP32[_stderr>>2]|0;
    $57 = $7;
    $58 = (($57) + 1)|0;
    $59 = $imgrecord;
    $60 = HEAP32[$59>>2]|0;
    HEAP32[$vararg_buffer2>>2] = $58;
    $vararg_ptr5 = (($vararg_buffer2) + 4|0);
    HEAP32[$vararg_ptr5>>2] = $60;
    (_fprintf(($56|0),(23896|0),($vararg_buffer2|0))|0);
    $0 = -2;
    $61 = $0;
    STACKTOP = sp;return ($61|0);
   }
  }
 }
 $16 = HEAP32[_stderr>>2]|0;
 $17 = $7;
 $18 = (($17) + 1)|0;
 $19 = $6;
 $20 = (($19) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$vararg_buffer>>2] = $18;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $22;
 (_fprintf(($16|0),(23816|0),($vararg_buffer|0))|0);
 $0 = -2;
 $61 = $0;
 STACKTOP = sp;return ($61|0);
}
function _decode_binary_field_image($odata,$ow,$oh,$od,$oppmm,$ansi_nist,$imgrecord_i) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppmm = $oppmm|0;
 $ansi_nist = $ansi_nist|0;
 $imgrecord_i = $imgrecord_i|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0.0, $209 = 0, $21 = 0, $210 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $field = 0, $field_i = 0, $id1 = 0, $id2 = 0, $idata1 = 0, $idata2 = 0, $ih1 = 0, $ih2 = 0, $ilen = 0;
 var $img_comp = 0, $imgrecord = 0, $iw1 = 0, $iw2 = 0, $lossyflag = 0, $ppi = 0, $ppmm = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer14 = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer24 = 0, $vararg_buffer27 = 0, $vararg_buffer30 = 0, $vararg_buffer36 = 0, $vararg_buffer40 = 0, $vararg_buffer44 = 0, $vararg_buffer5 = 0;
 var $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr33 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr43 = 0, $vararg_ptr47 = 0, $vararg_ptr48 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer44 = sp + 128|0;
 $vararg_buffer40 = sp + 120|0;
 $vararg_buffer36 = sp + 112|0;
 $vararg_buffer30 = sp + 96|0;
 $vararg_buffer27 = sp + 88|0;
 $vararg_buffer24 = sp + 80|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer14 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer2 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $field = sp + 192|0;
 $field_i = sp + 188|0;
 $idata2 = sp + 176|0;
 $ppmm = sp;
 $iw2 = sp + 156|0;
 $ih2 = sp + 152|0;
 $id2 = sp + 148|0;
 $ppi = sp + 144|0;
 $lossyflag = sp + 140|0;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppmm;
 $6 = $ansi_nist;
 $7 = $imgrecord_i;
 $8 = $7;
 $9 = ($8|0)<(1);
 if (!($9)) {
  $10 = $7;
  $11 = $6;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)>($13|0);
  if (!($14)) {
   $22 = $7;
   $23 = $6;
   $24 = (($23) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + ($22<<2)|0);
   $27 = HEAP32[$26>>2]|0;
   $imgrecord = $27;
   $28 = $imgrecord;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(8);
   if ($30) {
    $31 = HEAP32[_stderr>>2]|0;
    $32 = $7;
    $33 = (($32) + 1)|0;
    HEAP32[$vararg_buffer2>>2] = $33;
    (_fprintf(($31|0),(24064|0),($vararg_buffer2|0))|0);
    $0 = 0;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $34 = $imgrecord;
   $35 = (_lookup_ANSI_NIST_field($field,$field_i,8,$34)|0);
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[_stderr>>2]|0;
    $38 = $7;
    $39 = (($38) + 1)|0;
    $40 = $imgrecord;
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $39;
    $vararg_ptr8 = (($vararg_buffer5) + 4|0);
    HEAP32[$vararg_ptr8>>2] = $41;
    $vararg_ptr9 = (($vararg_buffer5) + 8|0);
    HEAP32[$vararg_ptr9>>2] = 8;
    (_fprintf(($37|0),(24160|0),($vararg_buffer5|0))|0);
    $0 = -3;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $42 = HEAP32[$field>>2]|0;
   $43 = (($42) + 24|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $img_comp = $50;
   $51 = $imgrecord;
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)==(5);
   if ($53) {
    label = 10;
   } else {
    $54 = $imgrecord;
    $55 = HEAP32[$54>>2]|0;
    $56 = ($55|0)==(6);
    if ($56) {
     label = 10;
    }
   }
   do {
    if ((label|0) == 10) {
     $57 = $img_comp;
     $58 = (_strcmp($57,24256)|0);
     $59 = ($58|0)!=(0);
     if (!($59)) {
      break;
     }
     $60 = HEAP32[_stderr>>2]|0;
     $61 = $7;
     $62 = (($61) + 1)|0;
     $63 = $imgrecord;
     $64 = HEAP32[$63>>2]|0;
     HEAP32[$vararg_buffer10>>2] = $62;
     $vararg_ptr13 = (($vararg_buffer10) + 4|0);
     HEAP32[$vararg_ptr13>>2] = $64;
     (_fprintf(($60|0),(24264|0),($vararg_buffer10|0))|0);
     $0 = 0;
     $210 = $0;
     STACKTOP = sp;return ($210|0);
    }
   } while(0);
   $65 = $imgrecord;
   $66 = (_lookup_ANSI_NIST_field($field,$field_i,6,$65)|0);
   $67 = ($66|0)==(0);
   if ($67) {
    $68 = HEAP32[_stderr>>2]|0;
    $69 = $7;
    $70 = (($69) + 1)|0;
    $71 = $imgrecord;
    $72 = HEAP32[$71>>2]|0;
    HEAP32[$vararg_buffer14>>2] = $70;
    $vararg_ptr17 = (($vararg_buffer14) + 4|0);
    HEAP32[$vararg_ptr17>>2] = $72;
    $vararg_ptr18 = (($vararg_buffer14) + 8|0);
    HEAP32[$vararg_ptr18>>2] = 6;
    (_fprintf(($68|0),(24400|0),($vararg_buffer14|0))|0);
    $0 = -4;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $73 = HEAP32[$field>>2]|0;
   $74 = (($73) + 24|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($76) + 12|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + 12|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = (_atoi($81)|0);
   $iw1 = $82;
   $83 = $imgrecord;
   $84 = (_lookup_ANSI_NIST_field($field,$field_i,7,$83)|0);
   $85 = ($84|0)==(0);
   if ($85) {
    $86 = HEAP32[_stderr>>2]|0;
    $87 = $7;
    $88 = (($87) + 1)|0;
    $89 = $imgrecord;
    $90 = HEAP32[$89>>2]|0;
    HEAP32[$vararg_buffer19>>2] = $88;
    $vararg_ptr22 = (($vararg_buffer19) + 4|0);
    HEAP32[$vararg_ptr22>>2] = $90;
    $vararg_ptr23 = (($vararg_buffer19) + 8|0);
    HEAP32[$vararg_ptr23>>2] = 7;
    (_fprintf(($86|0),(24496|0),($vararg_buffer19|0))|0);
    $0 = -5;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $91 = HEAP32[$field>>2]|0;
   $92 = (($91) + 24|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + 12|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + 12|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = (_atoi($99)|0);
   $ih1 = $100;
   $101 = $imgrecord;
   $102 = HEAP32[$101>>2]|0;
   if ((($102|0) == 6) | (($102|0) == 5)) {
    $id1 = 1;
   } else if ((($102|0) == 4) | (($102|0) == 3)) {
    $id1 = 8;
   } else {
    $103 = HEAP32[_stderr>>2]|0;
    $104 = $imgrecord;
    $105 = HEAP32[$104>>2]|0;
    HEAP32[$vararg_buffer24>>2] = $105;
    (_fprintf(($103|0),(24592|0),($vararg_buffer24|0))|0);
    $0 = -6;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $106 = $6;
   $107 = $7;
   $108 = (_lookup_binary_field_image_ppmm($ppmm,$106,$107)|0);
   $ret = $108;
   $109 = ($108|0)!=(0);
   if ($109) {
    $110 = $ret;
    $0 = $110;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $111 = $imgrecord;
   $112 = (($111) + 12|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = (($113) - 1)|0;
   $115 = $imgrecord;
   $116 = (($115) + 20|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = (($117) + ($114<<2)|0);
   $119 = HEAP32[$118>>2]|0;
   HEAP32[$field>>2] = $119;
   $120 = HEAP32[$field>>2]|0;
   $121 = (($120) + 24|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = HEAP32[$122>>2]|0;
   $124 = (($123) + 12|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($126) + 12|0);
   $128 = HEAP32[$127>>2]|0;
   $idata1 = $128;
   $129 = HEAP32[$field>>2]|0;
   $130 = (($129) + 24|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = HEAP32[$131>>2]|0;
   $133 = (($132) + 12|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = HEAP32[$134>>2]|0;
   $136 = HEAP32[$135>>2]|0;
   $ilen = $136;
   $137 = $img_comp;
   $138 = (_strcmp($137,24256)|0);
   $139 = ($138|0)==(0);
   if ($139) {
    $140 = $ilen;
    $141 = (_malloc($140)|0);
    HEAP32[$idata2>>2] = $141;
    $142 = HEAP32[$idata2>>2]|0;
    $143 = ($142|0)==(0|0);
    if ($143) {
     $144 = HEAP32[_stderr>>2]|0;
     $145 = $ilen;
     HEAP32[$vararg_buffer27>>2] = $145;
     (_fprintf(($144|0),(24672|0),($vararg_buffer27|0))|0);
     $0 = -7;
     $210 = $0;
     STACKTOP = sp;return ($210|0);
    } else {
     $146 = HEAP32[$idata2>>2]|0;
     $147 = $idata1;
     $148 = $ilen;
     _memcpy(($146|0),($147|0),($148|0))|0;
     $149 = HEAP32[$idata2>>2]|0;
     $150 = $1;
     HEAP32[$150>>2] = $149;
     $151 = $iw1;
     $152 = $2;
     HEAP32[$152>>2] = $151;
     $153 = $ih1;
     $154 = $3;
     HEAP32[$154>>2] = $153;
     $155 = $id1;
     $156 = $4;
     HEAP32[$156>>2] = $155;
     $157 = +HEAPF64[$ppmm>>3];
     $158 = $5;
     HEAPF64[$158>>3] = $157;
     $0 = 1;
     $210 = $0;
     STACKTOP = sp;return ($210|0);
    }
   }
   $159 = $img_comp;
   $160 = (_strcmp($159,24736)|0);
   $161 = ($160|0)==(0);
   if (!($161)) {
    $167 = HEAP32[_stderr>>2]|0;
    $168 = $img_comp;
    $169 = $img_comp;
    $170 = (_strlen(($169|0))|0);
    $171 = ($170>>>0)>(10);
    $172 = $171 ? 24888 : 24896;
    $173 = $7;
    $174 = (($173) + 1)|0;
    $175 = $imgrecord;
    $176 = HEAP32[$175>>2]|0;
    HEAP32[$vararg_buffer30>>2] = $168;
    $vararg_ptr33 = (($vararg_buffer30) + 4|0);
    HEAP32[$vararg_ptr33>>2] = $172;
    $vararg_ptr34 = (($vararg_buffer30) + 8|0);
    HEAP32[$vararg_ptr34>>2] = $174;
    $vararg_ptr35 = (($vararg_buffer30) + 12|0);
    HEAP32[$vararg_ptr35>>2] = $176;
    (_fprintf(($167|0),(24744|0),($vararg_buffer30|0))|0);
    $0 = 0;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $162 = $idata1;
   $163 = $ilen;
   $164 = (_wsq_decode_mem($idata2,$iw2,$ih2,$id2,$ppi,$lossyflag,$162,$163)|0);
   $ret = $164;
   $165 = ($164|0)!=(0);
   if ($165) {
    $166 = $ret;
    $0 = $166;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $177 = $iw1;
   $178 = HEAP32[$iw2>>2]|0;
   $179 = ($177|0)!=($178|0);
   if ($179) {
    $180 = HEAP32[_stderr>>2]|0;
    $181 = $iw1;
    $182 = HEAP32[$iw2>>2]|0;
    HEAP32[$vararg_buffer36>>2] = $181;
    $vararg_ptr39 = (($vararg_buffer36) + 4|0);
    HEAP32[$vararg_ptr39>>2] = $182;
    (_fprintf(($180|0),(24904|0),($vararg_buffer36|0))|0);
    $183 = HEAP32[$idata2>>2]|0;
    _free($183);
    $0 = -8;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $184 = $ih1;
   $185 = HEAP32[$ih2>>2]|0;
   $186 = ($184|0)!=($185|0);
   if ($186) {
    $187 = HEAP32[_stderr>>2]|0;
    $188 = $ih1;
    $189 = HEAP32[$ih2>>2]|0;
    HEAP32[$vararg_buffer40>>2] = $188;
    $vararg_ptr43 = (($vararg_buffer40) + 4|0);
    HEAP32[$vararg_ptr43>>2] = $189;
    (_fprintf(($187|0),(25008|0),($vararg_buffer40|0))|0);
    $190 = HEAP32[$idata2>>2]|0;
    _free($190);
    $0 = -9;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
   $191 = $id1;
   $192 = HEAP32[$id2>>2]|0;
   $193 = ($191|0)!=($192|0);
   if ($193) {
    $194 = HEAP32[_stderr>>2]|0;
    $195 = HEAP32[$id2>>2]|0;
    $196 = $imgrecord;
    $197 = HEAP32[$196>>2]|0;
    $198 = $id1;
    HEAP32[$vararg_buffer44>>2] = $195;
    $vararg_ptr47 = (($vararg_buffer44) + 4|0);
    HEAP32[$vararg_ptr47>>2] = $197;
    $vararg_ptr48 = (($vararg_buffer44) + 8|0);
    HEAP32[$vararg_ptr48>>2] = $198;
    (_fprintf(($194|0),(25112|0),($vararg_buffer44|0))|0);
    $199 = HEAP32[$idata2>>2]|0;
    _free($199);
    $0 = -10;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   } else {
    $200 = HEAP32[$idata2>>2]|0;
    $201 = $1;
    HEAP32[$201>>2] = $200;
    $202 = $iw1;
    $203 = $2;
    HEAP32[$203>>2] = $202;
    $204 = $ih1;
    $205 = $3;
    HEAP32[$205>>2] = $204;
    $206 = $id1;
    $207 = $4;
    HEAP32[$207>>2] = $206;
    $208 = +HEAPF64[$ppmm>>3];
    $209 = $5;
    HEAPF64[$209>>3] = $208;
    $0 = 1;
    $210 = $0;
    STACKTOP = sp;return ($210|0);
   }
  }
 }
 $15 = HEAP32[_stderr>>2]|0;
 $16 = $7;
 $17 = (($16) + 1)|0;
 $18 = $6;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$vararg_buffer>>2] = $17;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $21;
 (_fprintf(($15|0),(23984|0),($vararg_buffer|0))|0);
 $0 = -2;
 $210 = $0;
 STACKTOP = sp;return ($210|0);
}
function _decode_tagged_field_image($odata,$ow,$oh,$od,$oppmm,$ansi_nist,$imgrecord_i,$intrlvflag) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppmm = $oppmm|0;
 $ansi_nist = $ansi_nist|0;
 $imgrecord_i = $imgrecord_i|0;
 $intrlvflag = $intrlvflag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0.0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0.0, $467 = 0, $468 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $field = 0, $field_i = 0, $hor_sampfctr = 0, $i = 0, $id1 = 0, $id2 = 0, $idata1 = 0, $idata2 = 0, $ih1 = 0, $ih2 = 0, $ilen1 = 0;
 var $ilen2 = 0, $img_comp = 0, $img_csp = 0, $img_dat = 0, $imgrecord = 0, $iw1 = 0, $iw2 = 0, $lossyflag = 0, $n_cmpnts = 0, $ppi = 0, $ppmm = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer16 = 0, $vararg_buffer2 = 0, $vararg_buffer21 = 0, $vararg_buffer26 = 0, $vararg_buffer31 = 0, $vararg_buffer36 = 0;
 var $vararg_buffer43 = 0, $vararg_buffer46 = 0, $vararg_buffer52 = 0, $vararg_buffer57 = 0, $vararg_buffer6 = 0, $vararg_buffer62 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr24 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0;
 var $vararg_ptr41 = 0, $vararg_ptr42 = 0, $vararg_ptr49 = 0, $vararg_ptr5 = 0, $vararg_ptr50 = 0, $vararg_ptr51 = 0, $vararg_ptr55 = 0, $vararg_ptr56 = 0, $vararg_ptr60 = 0, $vararg_ptr61 = 0, $vararg_ptr65 = 0, $vararg_ptr66 = 0, $vararg_ptr9 = 0, $vrt_sampfctr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $vararg_buffer62 = sp + 200|0;
 $vararg_buffer57 = sp + 184|0;
 $vararg_buffer52 = sp + 168|0;
 $vararg_buffer46 = sp + 152|0;
 $vararg_buffer43 = sp + 144|0;
 $vararg_buffer36 = sp + 120|0;
 $vararg_buffer31 = sp + 104|0;
 $vararg_buffer26 = sp + 88|0;
 $vararg_buffer21 = sp + 72|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer2 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $field = sp + 312|0;
 $field_i = sp + 308|0;
 $idata1 = sp + 296|0;
 $idata2 = sp + 292|0;
 $ilen1 = sp + 288|0;
 $ppmm = sp;
 $ilen2 = sp + 272|0;
 $iw2 = sp + 268|0;
 $ih2 = sp + 264|0;
 $id2 = sp + 260|0;
 $ppi = sp + 256|0;
 $lossyflag = sp + 252|0;
 $img_dat = sp + 248|0;
 $hor_sampfctr = sp + 232|0;
 $vrt_sampfctr = sp + 216|0;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppmm;
 $6 = $ansi_nist;
 $7 = $imgrecord_i;
 $8 = $intrlvflag;
 $9 = $7;
 $10 = ($9|0)<(1);
 if (!($10)) {
  $11 = $7;
  $12 = $6;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)>($14|0);
  if (!($15)) {
   $23 = $7;
   $24 = $6;
   $25 = (($24) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($23<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $imgrecord = $28;
   $29 = $7;
   $30 = $6;
   $31 = (($30) + 16|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + ($29<<2)|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = (_tagged_image_record($35)|0);
   $37 = ($36|0)==(0);
   if ($37) {
    $38 = HEAP32[_stderr>>2]|0;
    $39 = $7;
    $40 = (($39) + 1)|0;
    $41 = $imgrecord;
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$vararg_buffer2>>2] = $40;
    $vararg_ptr5 = (($vararg_buffer2) + 4|0);
    HEAP32[$vararg_ptr5>>2] = $42;
    (_fprintf(($38|0),(25312|0),($vararg_buffer2|0))|0);
    $0 = -3;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $43 = $imgrecord;
   $44 = (_lookup_ANSI_NIST_field($field,$field_i,11,$43)|0);
   $45 = ($44|0)==(0);
   if ($45) {
    $46 = HEAP32[_stderr>>2]|0;
    $47 = $7;
    $48 = (($47) + 1)|0;
    $49 = $imgrecord;
    $50 = HEAP32[$49>>2]|0;
    HEAP32[$vararg_buffer6>>2] = $48;
    $vararg_ptr9 = (($vararg_buffer6) + 4|0);
    HEAP32[$vararg_ptr9>>2] = $50;
    $vararg_ptr10 = (($vararg_buffer6) + 8|0);
    HEAP32[$vararg_ptr10>>2] = 11;
    (_fprintf(($46|0),(25416|0),($vararg_buffer6|0))|0);
    $0 = -4;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $51 = HEAP32[$field>>2]|0;
   $52 = (($51) + 24|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($54) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $img_comp = $59;
   $60 = $imgrecord;
   $61 = (_lookup_ANSI_NIST_field($field,$field_i,6,$60)|0);
   $62 = ($61|0)==(0);
   if ($62) {
    $63 = HEAP32[_stderr>>2]|0;
    $64 = $7;
    $65 = (($64) + 1)|0;
    $66 = $imgrecord;
    $67 = HEAP32[$66>>2]|0;
    HEAP32[$vararg_buffer11>>2] = $65;
    $vararg_ptr14 = (($vararg_buffer11) + 4|0);
    HEAP32[$vararg_ptr14>>2] = $67;
    $vararg_ptr15 = (($vararg_buffer11) + 8|0);
    HEAP32[$vararg_ptr15>>2] = 6;
    (_fprintf(($63|0),(25512|0),($vararg_buffer11|0))|0);
    $0 = -5;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $68 = HEAP32[$field>>2]|0;
   $69 = (($68) + 24|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + 12|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + 12|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = (_atoi($76)|0);
   $iw1 = $77;
   $78 = $imgrecord;
   $79 = (_lookup_ANSI_NIST_field($field,$field_i,7,$78)|0);
   $80 = ($79|0)==(0);
   if ($80) {
    $81 = HEAP32[_stderr>>2]|0;
    $82 = $7;
    $83 = (($82) + 1)|0;
    $84 = $imgrecord;
    $85 = HEAP32[$84>>2]|0;
    HEAP32[$vararg_buffer16>>2] = $83;
    $vararg_ptr19 = (($vararg_buffer16) + 4|0);
    HEAP32[$vararg_ptr19>>2] = $85;
    $vararg_ptr20 = (($vararg_buffer16) + 8|0);
    HEAP32[$vararg_ptr20>>2] = 7;
    (_fprintf(($81|0),(25608|0),($vararg_buffer16|0))|0);
    $0 = -6;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $86 = HEAP32[$field>>2]|0;
   $87 = (($86) + 24|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 12|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = HEAP32[$91>>2]|0;
   $93 = (($92) + 12|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (_atoi($94)|0);
   $ih1 = $95;
   $96 = $imgrecord;
   $97 = (_lookup_ANSI_NIST_field($field,$field_i,12,$96)|0);
   $98 = ($97|0)==(0);
   if ($98) {
    $99 = HEAP32[_stderr>>2]|0;
    $100 = $7;
    $101 = (($100) + 1)|0;
    $102 = $imgrecord;
    $103 = HEAP32[$102>>2]|0;
    HEAP32[$vararg_buffer21>>2] = $101;
    $vararg_ptr24 = (($vararg_buffer21) + 4|0);
    HEAP32[$vararg_ptr24>>2] = $103;
    $vararg_ptr25 = (($vararg_buffer21) + 8|0);
    HEAP32[$vararg_ptr25>>2] = 12;
    (_fprintf(($99|0),(25704|0),($vararg_buffer21|0))|0);
    $0 = -7;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $104 = $imgrecord;
   $105 = HEAP32[$104>>2]|0;
   $106 = ($105|0)==(10);
   if ($106) {
    label = 16;
   } else {
    $107 = $imgrecord;
    $108 = HEAP32[$107>>2]|0;
    $109 = ($108|0)==(17);
    if ($109) {
     label = 16;
    } else {
     $166 = HEAP32[$field>>2]|0;
     $167 = (($166) + 24|0);
     $168 = HEAP32[$167>>2]|0;
     $169 = HEAP32[$168>>2]|0;
     $170 = (($169) + 12|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = HEAP32[$171>>2]|0;
     $173 = (($172) + 12|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = (_atoi($174)|0);
     $id1 = $175;
    }
   }
   if ((label|0) == 16) {
    $110 = $imgrecord;
    $111 = HEAP32[$110>>2]|0;
    $112 = ($111|0)==(10);
    do {
     if ($112) {
      $113 = $imgrecord;
      $114 = (_lookup_ANSI_NIST_field($field,$field_i,12,$113)|0);
      $115 = ($114|0)==(0);
      if (!($115)) {
       break;
      }
      $116 = HEAP32[_stderr>>2]|0;
      $117 = $7;
      $118 = (($117) + 1)|0;
      $119 = $imgrecord;
      $120 = HEAP32[$119>>2]|0;
      HEAP32[$vararg_buffer26>>2] = $118;
      $vararg_ptr29 = (($vararg_buffer26) + 4|0);
      HEAP32[$vararg_ptr29>>2] = $120;
      $vararg_ptr30 = (($vararg_buffer26) + 8|0);
      HEAP32[$vararg_ptr30>>2] = 12;
      (_fprintf(($116|0),(25800|0),($vararg_buffer26|0))|0);
      $0 = -2;
      $468 = $0;
      STACKTOP = sp;return ($468|0);
     }
    } while(0);
    $121 = $imgrecord;
    $122 = HEAP32[$121>>2]|0;
    $123 = ($122|0)==(17);
    do {
     if ($123) {
      $124 = $imgrecord;
      $125 = (_lookup_ANSI_NIST_field($field,$field_i,13,$124)|0);
      $126 = ($125|0)==(0);
      if (!($126)) {
       break;
      }
      $127 = HEAP32[_stderr>>2]|0;
      $128 = $7;
      $129 = (($128) + 1)|0;
      $130 = $imgrecord;
      $131 = HEAP32[$130>>2]|0;
      HEAP32[$vararg_buffer31>>2] = $129;
      $vararg_ptr34 = (($vararg_buffer31) + 4|0);
      HEAP32[$vararg_ptr34>>2] = $131;
      $vararg_ptr35 = (($vararg_buffer31) + 8|0);
      HEAP32[$vararg_ptr35>>2] = 13;
      (_fprintf(($127|0),(25800|0),($vararg_buffer31|0))|0);
      $0 = -2;
      $468 = $0;
      STACKTOP = sp;return ($468|0);
     }
    } while(0);
    $132 = HEAP32[$field>>2]|0;
    $133 = (($132) + 24|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = HEAP32[$134>>2]|0;
    $136 = (($135) + 12|0);
    $137 = HEAP32[$136>>2]|0;
    $138 = HEAP32[$137>>2]|0;
    $139 = (($138) + 12|0);
    $140 = HEAP32[$139>>2]|0;
    $img_csp = $140;
    $141 = $img_csp;
    $142 = (_strcmp($141,25896)|0);
    $143 = ($142|0)==(0);
    if ($143) {
     $id1 = 8;
    } else {
     $144 = $img_csp;
     $145 = (_strcmp($144,25904)|0);
     $146 = ($145|0)==(0);
     if (!($146)) {
      $147 = $img_csp;
      $148 = (_strcmp($147,25912)|0);
      $149 = ($148|0)==(0);
      if (!($149)) {
       $150 = $img_csp;
       $151 = (_strcmp($150,25920)|0);
       $152 = ($151|0)==(0);
       if (!($152)) {
        $153 = $img_csp;
        $154 = (_strcmp($153,25928)|0);
        $155 = ($154|0)==(0);
        if (!($155)) {
         $156 = HEAP32[_stderr>>2]|0;
         $157 = $img_csp;
         $158 = $img_csp;
         $159 = (_strlen(($158|0))|0);
         $160 = ($159>>>0)>(10);
         $161 = $160 ? 24888 : 24896;
         $162 = $7;
         $163 = (($162) + 1)|0;
         $164 = $imgrecord;
         $165 = HEAP32[$164>>2]|0;
         HEAP32[$vararg_buffer36>>2] = $157;
         $vararg_ptr39 = (($vararg_buffer36) + 4|0);
         HEAP32[$vararg_ptr39>>2] = $161;
         $vararg_ptr40 = (($vararg_buffer36) + 8|0);
         HEAP32[$vararg_ptr40>>2] = $163;
         $vararg_ptr41 = (($vararg_buffer36) + 12|0);
         HEAP32[$vararg_ptr41>>2] = $165;
         $vararg_ptr42 = (($vararg_buffer36) + 16|0);
         HEAP32[$vararg_ptr42>>2] = 12;
         (_fprintf(($156|0),(25936|0),($vararg_buffer36|0))|0);
         $0 = 0;
         $468 = $0;
         STACKTOP = sp;return ($468|0);
        }
       }
      }
     }
     $id1 = 24;
    }
   }
   $176 = $imgrecord;
   $177 = (_lookup_tagged_field_image_ppmm($ppmm,$176)|0);
   $ret = $177;
   $178 = $ret;
   $179 = ($178|0)<=(0);
   if ($179) {
    $180 = $ret;
    $0 = $180;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $181 = $imgrecord;
   $182 = (($181) + 12|0);
   $183 = HEAP32[$182>>2]|0;
   $184 = (($183) - 1)|0;
   $185 = $imgrecord;
   $186 = (($185) + 20|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = (($187) + ($184<<2)|0);
   $189 = HEAP32[$188>>2]|0;
   HEAP32[$field>>2] = $189;
   $190 = HEAP32[$field>>2]|0;
   $191 = (($190) + 24|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = HEAP32[$192>>2]|0;
   $194 = (($193) + 12|0);
   $195 = HEAP32[$194>>2]|0;
   $196 = HEAP32[$195>>2]|0;
   $197 = (($196) + 12|0);
   $198 = HEAP32[$197>>2]|0;
   HEAP32[$idata1>>2] = $198;
   $199 = HEAP32[$field>>2]|0;
   $200 = (($199) + 24|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = HEAP32[$201>>2]|0;
   $203 = (($202) + 12|0);
   $204 = HEAP32[$203>>2]|0;
   $205 = HEAP32[$204>>2]|0;
   $206 = HEAP32[$205>>2]|0;
   HEAP32[$ilen1>>2] = $206;
   $207 = $img_comp;
   $208 = (_strcmp($207,26072)|0);
   $209 = ($208|0)==(0);
   if ($209) {
    $210 = HEAP32[$ilen1>>2]|0;
    $211 = (_malloc($210)|0);
    HEAP32[$idata2>>2] = $211;
    $212 = HEAP32[$idata2>>2]|0;
    $213 = ($212|0)==(0|0);
    if ($213) {
     $214 = HEAP32[_stderr>>2]|0;
     $215 = HEAP32[$ilen1>>2]|0;
     HEAP32[$vararg_buffer43>>2] = $215;
     (_fprintf(($214|0),(26080|0),($vararg_buffer43|0))|0);
     $0 = -8;
     $468 = $0;
     STACKTOP = sp;return ($468|0);
    }
    $216 = HEAP32[$idata2>>2]|0;
    $217 = HEAP32[$idata1>>2]|0;
    $218 = HEAP32[$ilen1>>2]|0;
    _memcpy(($216|0),($217|0),($218|0))|0;
    $219 = $id1;
    $220 = ($219|0)==(24);
    do {
     if ($220) {
      $221 = $8;
      $222 = ($221|0)!=(0);
      if ($222) {
       $n_cmpnts = 3;
       $i = 0;
       while(1) {
        $223 = $i;
        $224 = $n_cmpnts;
        $225 = ($223|0)<($224|0);
        if (!($225)) {
         break;
        }
        $226 = $i;
        $227 = (($hor_sampfctr) + ($226<<2)|0);
        HEAP32[$227>>2] = 1;
        $228 = $i;
        $229 = (($vrt_sampfctr) + ($228<<2)|0);
        HEAP32[$229>>2] = 1;
        $230 = $i;
        $231 = (($230) + 1)|0;
        $i = $231;
       }
       $232 = HEAP32[$idata2>>2]|0;
       $233 = $iw1;
       $234 = $ih1;
       $235 = $id1;
       $236 = $n_cmpnts;
       $237 = (_not2intrlv_mem($idata1,$ilen1,$232,$233,$234,$235,$hor_sampfctr,$vrt_sampfctr,$236)|0);
       $ret = $237;
       $238 = ($237|0)!=(0);
       if (!($238)) {
        $241 = HEAP32[$idata2>>2]|0;
        _free($241);
        $242 = HEAP32[$idata1>>2]|0;
        HEAP32[$idata2>>2] = $242;
        $243 = HEAP32[$ilen1>>2]|0;
        HEAP32[$ilen2>>2] = $243;
        break;
       }
       $239 = HEAP32[$idata2>>2]|0;
       _free($239);
       $240 = $ret;
       $0 = $240;
       $468 = $0;
       STACKTOP = sp;return ($468|0);
      }
     }
    } while(0);
    $244 = HEAP32[$idata2>>2]|0;
    $245 = $1;
    HEAP32[$245>>2] = $244;
    $246 = $iw1;
    $247 = $2;
    HEAP32[$247>>2] = $246;
    $248 = $ih1;
    $249 = $3;
    HEAP32[$249>>2] = $248;
    $250 = $id1;
    $251 = $4;
    HEAP32[$251>>2] = $250;
    $252 = +HEAPF64[$ppmm>>3];
    $253 = $5;
    HEAPF64[$253>>3] = $252;
    $0 = 1;
    $468 = $0;
    STACKTOP = sp;return ($468|0);
   }
   $254 = $img_comp;
   $255 = (_strcmp($254,26144)|0);
   $256 = ($255|0)==(0);
   do {
    if ($256) {
     $257 = HEAP32[$idata1>>2]|0;
     $258 = HEAP32[$ilen1>>2]|0;
     $259 = (_wsq_decode_mem($idata2,$iw2,$ih2,$id2,$ppi,$lossyflag,$257,$258)|0);
     $ret = $259;
     $260 = ($259|0)!=(0);
     if (!($260)) {
      break;
     }
     $261 = $ret;
     $0 = $261;
     $468 = $0;
     STACKTOP = sp;return ($468|0);
    } else {
     $262 = $img_comp;
     $263 = (_strcmp($262,26152)|0);
     $264 = ($263|0)==(0);
     if ($264) {
      $265 = HEAP32[$idata1>>2]|0;
      $266 = HEAP32[$ilen1>>2]|0;
      $267 = (_jpegb_decode_mem($idata2,$iw2,$ih2,$id2,$ppi,$lossyflag,$265,$266)|0);
      $ret = $267;
      $268 = ($267|0)!=(0);
      if ($268) {
       $269 = $ret;
       $0 = $269;
       $468 = $0;
       STACKTOP = sp;return ($468|0);
      }
      $270 = HEAP32[$id2>>2]|0;
      $271 = ($270|0)==(24);
      do {
       if ($271) {
        $272 = $8;
        $273 = ($272|0)==(0);
        if ($273) {
         $n_cmpnts = 3;
         $i = 0;
         while(1) {
          $274 = $i;
          $275 = $n_cmpnts;
          $276 = ($274|0)<($275|0);
          if (!($276)) {
           break;
          }
          $277 = $i;
          $278 = (($hor_sampfctr) + ($277<<2)|0);
          HEAP32[$278>>2] = 1;
          $279 = $i;
          $280 = (($vrt_sampfctr) + ($279<<2)|0);
          HEAP32[$280>>2] = 1;
          $281 = $i;
          $282 = (($281) + 1)|0;
          $i = $282;
         }
         $283 = HEAP32[$idata2>>2]|0;
         $284 = HEAP32[$iw2>>2]|0;
         $285 = HEAP32[$ih2>>2]|0;
         $286 = HEAP32[$id2>>2]|0;
         $287 = $n_cmpnts;
         $288 = (_intrlv2not_mem($idata1,$ilen1,$283,$284,$285,$286,$hor_sampfctr,$vrt_sampfctr,$287)|0);
         $ret = $288;
         $289 = ($288|0)!=(0);
         if (!($289)) {
          $292 = HEAP32[$idata2>>2]|0;
          _free($292);
          $293 = HEAP32[$idata1>>2]|0;
          HEAP32[$idata2>>2] = $293;
          $294 = HEAP32[$ilen1>>2]|0;
          HEAP32[$ilen2>>2] = $294;
          break;
         }
         $290 = HEAP32[$idata2>>2]|0;
         _free($290);
         $291 = $ret;
         $0 = $291;
         $468 = $0;
         STACKTOP = sp;return ($468|0);
        }
       }
      } while(0);
     } else {
      $295 = $img_comp;
      $296 = (_strcmp($295,26160)|0);
      $297 = ($296|0)==(0);
      if ($297) {
       $298 = HEAP32[$idata1>>2]|0;
       $299 = HEAP32[$ilen1>>2]|0;
       $300 = (_jpegl_decode_mem($img_dat,$lossyflag,$298,$299)|0);
       $ret = $300;
       $301 = ($300|0)!=(0);
       if ($301) {
        $302 = $ret;
        $0 = $302;
        $468 = $0;
        STACKTOP = sp;return ($468|0);
       }
       $303 = HEAP32[$img_dat>>2]|0;
       $304 = (_get_IMG_DAT_image($idata2,$ilen2,$iw2,$ih2,$id2,$ppi,$303)|0);
       $ret = $304;
       $305 = ($304|0)!=(0);
       if ($305) {
        $306 = HEAP32[$img_dat>>2]|0;
        _free_IMG_DAT($306,1);
        $307 = $ret;
        $0 = $307;
        $468 = $0;
        STACKTOP = sp;return ($468|0);
       }
       $308 = HEAP32[$id2>>2]|0;
       $309 = ($308|0)==(24);
       do {
        if ($309) {
         $310 = $8;
         $311 = ($310|0)!=(0);
         if (!($311)) {
          break;
         }
         $312 = HEAP32[$idata2>>2]|0;
         $313 = HEAP32[$img_dat>>2]|0;
         $314 = HEAP32[$313>>2]|0;
         $315 = HEAP32[$img_dat>>2]|0;
         $316 = (($315) + 4|0);
         $317 = HEAP32[$316>>2]|0;
         $318 = HEAP32[$img_dat>>2]|0;
         $319 = (($318) + 8|0);
         $320 = HEAP32[$319>>2]|0;
         $321 = HEAP32[$img_dat>>2]|0;
         $322 = (($321) + 28|0);
         $323 = HEAP32[$img_dat>>2]|0;
         $324 = (($323) + 44|0);
         $325 = HEAP32[$img_dat>>2]|0;
         $326 = (($325) + 20|0);
         $327 = HEAP32[$326>>2]|0;
         $328 = (_not2intrlv_mem($idata1,$ilen1,$312,$314,$317,$320,$322,$324,$327)|0);
         $ret = $328;
         $329 = ($328|0)!=(0);
         if (!($329)) {
          $333 = HEAP32[$idata2>>2]|0;
          _free($333);
          $334 = HEAP32[$idata1>>2]|0;
          HEAP32[$idata2>>2] = $334;
          $335 = HEAP32[$ilen1>>2]|0;
          HEAP32[$ilen2>>2] = $335;
          break;
         }
         $330 = HEAP32[$img_dat>>2]|0;
         _free_IMG_DAT($330,1);
         $331 = HEAP32[$idata2>>2]|0;
         _free($331);
         $332 = $ret;
         $0 = $332;
         $468 = $0;
         STACKTOP = sp;return ($468|0);
        }
       } while(0);
       $336 = HEAP32[$img_dat>>2]|0;
       _free_IMG_DAT($336,1);
      } else {
       $337 = $img_comp;
       $338 = (_strcmp($337,26168)|0);
       $339 = ($338|0)==(0);
       if ($339) {
        label = 80;
       } else {
        $340 = $img_comp;
        $341 = (_strcmp($340,26176)|0);
        $342 = ($341|0)==(0);
        if ($342) {
         label = 80;
        } else {
         $382 = $img_comp;
         $383 = (_strcmp($382,26184)|0);
         $384 = ($383|0)==(0);
         if (!($384)) {
          $424 = HEAP32[_stderr>>2]|0;
          $425 = $img_comp;
          $426 = $img_comp;
          $427 = (_strlen(($426|0))|0);
          $428 = ($427>>>0)>(10);
          $429 = $428 ? 24888 : 24896;
          $430 = $7;
          $431 = (($430) + 1)|0;
          $432 = $imgrecord;
          $433 = HEAP32[$432>>2]|0;
          HEAP32[$vararg_buffer46>>2] = $425;
          $vararg_ptr49 = (($vararg_buffer46) + 4|0);
          HEAP32[$vararg_ptr49>>2] = $429;
          $vararg_ptr50 = (($vararg_buffer46) + 8|0);
          HEAP32[$vararg_ptr50>>2] = $431;
          $vararg_ptr51 = (($vararg_buffer46) + 12|0);
          HEAP32[$vararg_ptr51>>2] = $433;
          (_fprintf(($424|0),(26192|0),($vararg_buffer46|0))|0);
          $0 = 0;
          $468 = $0;
          STACKTOP = sp;return ($468|0);
         }
         $385 = HEAP32[$idata1>>2]|0;
         $386 = HEAP32[$ilen1>>2]|0;
         $387 = (_png_decode_mem($img_dat,$lossyflag,$385,$386)|0);
         $ret = $387;
         $388 = ($387|0)!=(0);
         if ($388) {
          $389 = $ret;
          $0 = $389;
          $468 = $0;
          STACKTOP = sp;return ($468|0);
         }
         $390 = HEAP32[$img_dat>>2]|0;
         $391 = (_get_IMG_DAT_image($idata2,$ilen2,$iw2,$ih2,$id2,$ppi,$390)|0);
         $ret = $391;
         $392 = ($391|0)!=(0);
         if ($392) {
          $393 = HEAP32[$img_dat>>2]|0;
          _free_IMG_DAT($393,1);
          $394 = $ret;
          $0 = $394;
          $468 = $0;
          STACKTOP = sp;return ($468|0);
         }
         $395 = HEAP32[$id2>>2]|0;
         $396 = ($395|0)==(24);
         do {
          if ($396) {
           $397 = $8;
           $398 = ($397|0)!=(0);
           if (!($398)) {
            break;
           }
           $399 = HEAP32[$idata2>>2]|0;
           $400 = HEAP32[$img_dat>>2]|0;
           $401 = HEAP32[$400>>2]|0;
           $402 = HEAP32[$img_dat>>2]|0;
           $403 = (($402) + 4|0);
           $404 = HEAP32[$403>>2]|0;
           $405 = HEAP32[$img_dat>>2]|0;
           $406 = (($405) + 8|0);
           $407 = HEAP32[$406>>2]|0;
           $408 = HEAP32[$img_dat>>2]|0;
           $409 = (($408) + 28|0);
           $410 = HEAP32[$img_dat>>2]|0;
           $411 = (($410) + 44|0);
           $412 = HEAP32[$img_dat>>2]|0;
           $413 = (($412) + 20|0);
           $414 = HEAP32[$413>>2]|0;
           $415 = (_not2intrlv_mem($idata1,$ilen1,$399,$401,$404,$407,$409,$411,$414)|0);
           $ret = $415;
           $416 = ($415|0)!=(0);
           if (!($416)) {
            $420 = HEAP32[$img_dat>>2]|0;
            _free_IMG_DAT($420,1);
            $421 = HEAP32[$idata2>>2]|0;
            _free($421);
            $422 = HEAP32[$idata1>>2]|0;
            HEAP32[$idata2>>2] = $422;
            $423 = HEAP32[$ilen1>>2]|0;
            HEAP32[$ilen2>>2] = $423;
            break;
           }
           $417 = HEAP32[$img_dat>>2]|0;
           _free_IMG_DAT($417,1);
           $418 = HEAP32[$idata2>>2]|0;
           _free($418);
           $419 = $ret;
           $0 = $419;
           $468 = $0;
           STACKTOP = sp;return ($468|0);
          }
         } while(0);
        }
       }
       if ((label|0) == 80) {
        $343 = HEAP32[$idata1>>2]|0;
        $344 = HEAP32[$ilen1>>2]|0;
        $345 = (_openjpeg2k_decode_mem($img_dat,$lossyflag,$343,$344)|0);
        $ret = $345;
        $346 = ($345|0)!=(0);
        if ($346) {
         $347 = $ret;
         $0 = $347;
         $468 = $0;
         STACKTOP = sp;return ($468|0);
        }
        $348 = HEAP32[$img_dat>>2]|0;
        $349 = (_get_IMG_DAT_image($idata2,$ilen2,$iw2,$ih2,$id2,$ppi,$348)|0);
        $ret = $349;
        $350 = ($349|0)!=(0);
        if ($350) {
         $351 = HEAP32[$img_dat>>2]|0;
         _free_IMG_DAT($351,1);
         $352 = $ret;
         $0 = $352;
         $468 = $0;
         STACKTOP = sp;return ($468|0);
        }
        $353 = HEAP32[$id2>>2]|0;
        $354 = ($353|0)==(24);
        do {
         if ($354) {
          $355 = $8;
          $356 = ($355|0)!=(0);
          if (!($356)) {
           break;
          }
          $357 = HEAP32[$idata2>>2]|0;
          $358 = HEAP32[$img_dat>>2]|0;
          $359 = HEAP32[$358>>2]|0;
          $360 = HEAP32[$img_dat>>2]|0;
          $361 = (($360) + 4|0);
          $362 = HEAP32[$361>>2]|0;
          $363 = HEAP32[$img_dat>>2]|0;
          $364 = (($363) + 8|0);
          $365 = HEAP32[$364>>2]|0;
          $366 = HEAP32[$img_dat>>2]|0;
          $367 = (($366) + 28|0);
          $368 = HEAP32[$img_dat>>2]|0;
          $369 = (($368) + 44|0);
          $370 = HEAP32[$img_dat>>2]|0;
          $371 = (($370) + 20|0);
          $372 = HEAP32[$371>>2]|0;
          $373 = (_not2intrlv_mem($idata1,$ilen1,$357,$359,$362,$365,$367,$369,$372)|0);
          $ret = $373;
          $374 = ($373|0)!=(0);
          if (!($374)) {
           $378 = HEAP32[$img_dat>>2]|0;
           _free_IMG_DAT($378,1);
           $379 = HEAP32[$idata2>>2]|0;
           _free($379);
           $380 = HEAP32[$idata1>>2]|0;
           HEAP32[$idata2>>2] = $380;
           $381 = HEAP32[$ilen1>>2]|0;
           HEAP32[$ilen2>>2] = $381;
           break;
          }
          $375 = HEAP32[$img_dat>>2]|0;
          _free_IMG_DAT($375,1);
          $376 = HEAP32[$idata2>>2]|0;
          _free($376);
          $377 = $ret;
          $0 = $377;
          $468 = $0;
          STACKTOP = sp;return ($468|0);
         }
        } while(0);
       }
      }
     }
    }
   } while(0);
   $434 = $iw1;
   $435 = HEAP32[$iw2>>2]|0;
   $436 = ($434|0)!=($435|0);
   if ($436) {
    $437 = HEAP32[_stderr>>2]|0;
    $438 = $iw1;
    $439 = HEAP32[$iw2>>2]|0;
    $440 = HEAP32[$iw2>>2]|0;
    HEAP32[$vararg_buffer52>>2] = $438;
    $vararg_ptr55 = (($vararg_buffer52) + 4|0);
    HEAP32[$vararg_ptr55>>2] = $439;
    $vararg_ptr56 = (($vararg_buffer52) + 8|0);
    HEAP32[$vararg_ptr56>>2] = $440;
    (_fprintf(($437|0),(26336|0),($vararg_buffer52|0))|0);
    $441 = HEAP32[$iw2>>2]|0;
    $iw1 = $441;
   }
   $442 = $ih1;
   $443 = HEAP32[$ih2>>2]|0;
   $444 = ($442|0)!=($443|0);
   if ($444) {
    $445 = HEAP32[_stderr>>2]|0;
    $446 = $ih1;
    $447 = HEAP32[$ih2>>2]|0;
    $448 = HEAP32[$ih2>>2]|0;
    HEAP32[$vararg_buffer57>>2] = $446;
    $vararg_ptr60 = (($vararg_buffer57) + 4|0);
    HEAP32[$vararg_ptr60>>2] = $447;
    $vararg_ptr61 = (($vararg_buffer57) + 8|0);
    HEAP32[$vararg_ptr61>>2] = $448;
    (_fprintf(($445|0),(26504|0),($vararg_buffer57|0))|0);
    $449 = HEAP32[$ih2>>2]|0;
    $ih1 = $449;
   }
   $450 = $id1;
   $451 = HEAP32[$id2>>2]|0;
   $452 = ($450|0)!=($451|0);
   if ($452) {
    $453 = HEAP32[_stderr>>2]|0;
    $454 = $id1;
    $455 = HEAP32[$id2>>2]|0;
    $456 = HEAP32[$id2>>2]|0;
    HEAP32[$vararg_buffer62>>2] = $454;
    $vararg_ptr65 = (($vararg_buffer62) + 4|0);
    HEAP32[$vararg_ptr65>>2] = $455;
    $vararg_ptr66 = (($vararg_buffer62) + 8|0);
    HEAP32[$vararg_ptr66>>2] = $456;
    (_fprintf(($453|0),(26680|0),($vararg_buffer62|0))|0);
    $457 = HEAP32[$id2>>2]|0;
    $id1 = $457;
   }
   $458 = HEAP32[$idata2>>2]|0;
   $459 = $1;
   HEAP32[$459>>2] = $458;
   $460 = $iw1;
   $461 = $2;
   HEAP32[$461>>2] = $460;
   $462 = $ih1;
   $463 = $3;
   HEAP32[$463>>2] = $462;
   $464 = $id1;
   $465 = $4;
   HEAP32[$465>>2] = $464;
   $466 = +HEAPF64[$ppmm>>3];
   $467 = $5;
   HEAPF64[$467>>3] = $466;
   $0 = 1;
   $468 = $0;
   STACKTOP = sp;return ($468|0);
  }
 }
 $16 = HEAP32[_stderr>>2]|0;
 $17 = $7;
 $18 = (($17) + 1)|0;
 $19 = $6;
 $20 = (($19) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$vararg_buffer>>2] = $18;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $22;
 (_fprintf(($16|0),(25232|0),($vararg_buffer|0))|0);
 $0 = -2;
 $468 = $0;
 STACKTOP = sp;return ($468|0);
}
function _openjpeg2k_decode_mem($oimg_dat,$lossyflag,$idata,$ilen) {
 $oimg_dat = $oimg_dat|0;
 $lossyflag = $lossyflag|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $cio = 0, $dinfo = 0, $event_mgr = 0, $image = 0, $img_dat = 0, $parameters = 0;
 var $sgnd_buf = 0, $size = 0, $unsgnd_buf = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 8320|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $img_dat = sp + 8288|0;
 $event_mgr = sp + 8272|0;
 $parameters = sp + 40|0;
 $1 = $oimg_dat;
 $2 = $lossyflag;
 $3 = $idata;
 $4 = $ilen;
 $image = 0;
 $dinfo = 0;
 $cio = 0;
 $sgnd_buf = 0;
 $unsgnd_buf = 0;
 ;HEAP32[$event_mgr+0>>2]=0|0;HEAP32[$event_mgr+4>>2]=0|0;HEAP32[$event_mgr+8>>2]=0|0;
 _opj_set_default_decoder_parameters($parameters);
 $5 = (_get_file_format(26992)|0);
 $6 = (($parameters) + 8200|0);
 HEAP32[$6>>2] = $5;
 $7 = (_get_file_format(27000)|0);
 $8 = (($parameters) + 8204|0);
 HEAP32[$8>>2] = $7;
 $9 = (_opj_create_decompress(2)|0);
 $dinfo = $9;
 $10 = $dinfo;
 $11 = HEAP32[_stderr>>2]|0;
 (_opj_set_event_mgr($10,$event_mgr,$11)|0);
 $12 = $dinfo;
 _opj_setup_decoder($12,$parameters);
 $13 = $dinfo;
 $14 = $3;
 $15 = $4;
 $16 = (_opj_cio_open($13,$14,$15)|0);
 $cio = $16;
 $17 = $dinfo;
 $18 = $cio;
 $19 = (_opj_decode($17,$18)|0);
 $image = $19;
 $20 = $image;
 $21 = ($20|0)!=(0|0);
 if (!($21)) {
  $22 = HEAP32[_stderr>>2]|0;
  (_fprintf(($22|0),(27008|0),($vararg_buffer|0))|0);
  $23 = $dinfo;
  _opj_destroy_decompress($23);
  $24 = $cio;
  _opj_cio_close($24);
  $0 = -1;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 }
 $25 = $cio;
 _opj_cio_close($25);
 $26 = $dinfo;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = $dinfo;
  _opj_destroy_decompress($28);
 }
 $29 = $image;
 $30 = (($29) + 16|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = $image;
 $33 = (($32) + 24|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = Math_imul($31, $36)|0;
 $38 = $image;
 $39 = (($38) + 24|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + 12|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = Math_imul($37, $42)|0;
 $size = $43;
 $44 = $image;
 $45 = (($44) + 24|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 32|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = ($48|0)==(1);
 if ($49) {
  $50 = HEAP32[_stderr>>2]|0;
  (_fprintf(($50|0),(27048|0),($vararg_buffer1|0))|0);
  $51 = $image;
  _opj_image_destroy($51);
  $0 = -2;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 }
 $52 = $size;
 $53 = $52;
 $54 = (_malloc($53)|0);
 $unsgnd_buf = $54;
 $55 = $image;
 $56 = $unsgnd_buf;
 $57 = (_image_to_raw($55,$56)|0);
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = HEAP32[_stderr>>2]|0;
  (_fprintf(($59|0),(27088|0),($vararg_buffer3|0))|0);
  $60 = $image;
  _opj_image_destroy($60);
  $0 = -3;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 }
 $61 = $image;
 $62 = $unsgnd_buf;
 $63 = (_img_dat_generate_openjpeg($img_dat,$61,$62)|0);
 $64 = ($63|0)!=(0);
 if ($64) {
  $65 = $unsgnd_buf;
  _free($65);
  $66 = $image;
  _opj_image_destroy($66);
  $67 = HEAP32[_stderr>>2]|0;
  (_fprintf(($67|0),(27112|0),($vararg_buffer5|0))|0);
  $0 = -4;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 } else {
  $68 = $unsgnd_buf;
  _free($68);
  $69 = $image;
  _opj_image_destroy($69);
  $70 = HEAP32[$img_dat>>2]|0;
  $71 = $1;
  HEAP32[$71>>2] = $70;
  $72 = $2;
  HEAP32[$72>>2] = 0;
  $0 = 0;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 }
 return 0|0;
}
function _get_file_format($filename) {
 $filename = $filename|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $ext = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $filename;
 $2 = $1;
 $3 = (_strrchr($2,46)|0);
 $ext = $3;
 $4 = $ext;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $0 = -1;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = $ext;
 $7 = (($6) + 1|0);
 $ext = $7;
 $i = 0;
 while(1) {
  $8 = $i;
  $9 = ($8>>>0)<(11);
  if (!($9)) {
   label = 9;
   break;
  }
  $10 = $ext;
  $11 = $i;
  $12 = (27664 + ($11<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_strncasecmp($10,$13,3)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   label = 6;
   break;
  }
  $19 = $i;
  $20 = (($19) + 1)|0;
  $i = $20;
 }
 if ((label|0) == 6) {
  $16 = $i;
  $17 = (27800 + ($16<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $0 = $18;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 else if ((label|0) == 9) {
  $0 = -1;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _image_to_raw($image,$unsgnd_buf) {
 $image = $image|0;
 $unsgnd_buf = $unsgnd_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $compno = 0, $curr = 0, $h = 0, $index = 0, $line = 0;
 var $mask = 0, $ptr = 0, $row = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer3 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $image;
 $2 = $unsgnd_buf;
 $3 = $1;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $1;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = Math_imul($5, $8)|0;
 $10 = $1;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = Math_imul($9, $12)|0;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(27376|0),($vararg_buffer|0))|0);
  $0 = 1;
  $154 = $0;
  STACKTOP = sp;return ($154|0);
 }
 $16 = HEAP32[_stdout>>2]|0;
 $17 = $1;
 $18 = (($17) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer1>>2] = $19;
 (_fprintf(($16|0),(27416|0),($vararg_buffer1|0))|0);
 $index = 0;
 $compno = 0;
 while(1) {
  $20 = $compno;
  $21 = $1;
  $22 = (($21) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   label = 24;
   break;
  }
  $25 = HEAP32[_stdout>>2]|0;
  $26 = $compno;
  $27 = $compno;
  $28 = $1;
  $29 = (($28) + 24|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + (($27*48)|0)|0);
  $32 = (($31) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $compno;
  $35 = $1;
  $36 = (($35) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + (($34*48)|0)|0);
  $39 = (($38) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $compno;
  $42 = $1;
  $43 = (($42) + 24|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($41*48)|0)|0);
  $46 = (($45) + 24|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $compno;
  $49 = $1;
  $50 = (($49) + 24|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + (($48*48)|0)|0);
  $53 = (($52) + 32|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)==(1);
  $56 = $55 ? 27512 : 27520;
  HEAP32[$vararg_buffer3>>2] = $26;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $33;
  $vararg_ptr7 = (($vararg_buffer3) + 8|0);
  HEAP32[$vararg_ptr7>>2] = $40;
  $vararg_ptr8 = (($vararg_buffer3) + 12|0);
  HEAP32[$vararg_ptr8>>2] = $47;
  $vararg_ptr9 = (($vararg_buffer3) + 16|0);
  HEAP32[$vararg_ptr9>>2] = $56;
  (_fprintf(($25|0),(27464|0),($vararg_buffer3|0))|0);
  $57 = $compno;
  $58 = $1;
  $59 = (($58) + 24|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + (($57*48)|0)|0);
  $62 = (($61) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  $w = $63;
  $64 = $compno;
  $65 = $1;
  $66 = (($65) + 24|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + (($64*48)|0)|0);
  $69 = (($68) + 12|0);
  $70 = HEAP32[$69>>2]|0;
  $h = $70;
  $71 = $compno;
  $72 = $1;
  $73 = (($72) + 24|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + (($71*48)|0)|0);
  $76 = (($75) + 24|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($77|0)<=(8);
  if (!($78)) {
   break;
  }
  $79 = $compno;
  $80 = $1;
  $81 = (($80) + 24|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + (($79*48)|0)|0);
  $84 = (($83) + 32|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($85|0)==(0);
  if ($86) {
   $87 = $compno;
   $88 = $1;
   $89 = (($88) + 24|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (($90) + (($87*48)|0)|0);
   $92 = (($91) + 24|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = 1 << $93;
   $95 = (($94) - 1)|0;
   $mask = $95;
   $96 = $compno;
   $97 = $1;
   $98 = (($97) + 24|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = (($99) + (($96*48)|0)|0);
   $101 = (($100) + 44|0);
   $102 = HEAP32[$101>>2]|0;
   $ptr = $102;
   $line = 0;
   while(1) {
    $103 = $line;
    $104 = $h;
    $105 = ($103|0)<($104|0);
    if (!($105)) {
     break;
    }
    $row = 0;
    while(1) {
     $106 = $row;
     $107 = $w;
     $108 = ($106|0)<($107|0);
     if (!($108)) {
      break;
     }
     $109 = $ptr;
     $110 = HEAP32[$109>>2]|0;
     $111 = $mask;
     $112 = $110 & $111;
     $113 = $112&255;
     $curr = $113;
     $114 = $curr;
     $115 = $index;
     $116 = $2;
     $117 = (($116) + ($115)|0);
     HEAP8[$117>>0] = $114;
     $118 = $ptr;
     $119 = (($118) + 4|0);
     $ptr = $119;
     $120 = $index;
     $121 = (($120) + 1)|0;
     $index = $121;
     $122 = $row;
     $123 = (($122) + 1)|0;
     $row = $123;
    }
    $124 = $line;
    $125 = (($124) + 1)|0;
    $line = $125;
   }
  }
  $152 = $compno;
  $153 = (($152) + 1)|0;
  $compno = $153;
 }
 if ((label|0) == 24) {
  $0 = 0;
  $154 = $0;
  STACKTOP = sp;return ($154|0);
 }
 $126 = $compno;
 $127 = $1;
 $128 = (($127) + 24|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = (($129) + (($126*48)|0)|0);
 $131 = (($130) + 24|0);
 $132 = HEAP32[$131>>2]|0;
 $133 = ($132|0)<=(16);
 if ($133) {
  $134 = HEAP32[_stderr>>2]|0;
  (_fprintf(($134|0),(27536|0),($vararg_buffer10|0))|0);
  $0 = 1;
  $154 = $0;
  STACKTOP = sp;return ($154|0);
 }
 $135 = $compno;
 $136 = $1;
 $137 = (($136) + 24|0);
 $138 = HEAP32[$137>>2]|0;
 $139 = (($138) + (($135*48)|0)|0);
 $140 = (($139) + 24|0);
 $141 = HEAP32[$140>>2]|0;
 $142 = ($141|0)<=(32);
 if ($142) {
  $143 = HEAP32[_stderr>>2]|0;
  (_fprintf(($143|0),(27584|0),($vararg_buffer12|0))|0);
  $0 = 1;
  $154 = $0;
  STACKTOP = sp;return ($154|0);
 } else {
  $144 = HEAP32[_stderr>>2]|0;
  $145 = $compno;
  $146 = $1;
  $147 = (($146) + 24|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = (($148) + (($145*48)|0)|0);
  $150 = (($149) + 24|0);
  $151 = HEAP32[$150>>2]|0;
  HEAP32[$vararg_buffer14>>2] = $151;
  (_fprintf(($144|0),(27632|0),($vararg_buffer14|0))|0);
  $0 = 1;
  $154 = $0;
  STACKTOP = sp;return ($154|0);
 }
 return 0|0;
}
function _img_dat_generate_openjpeg($oimg_dat,$image,$unsgnd_buf) {
 $oimg_dat = $oimg_dat|0;
 $image = $image|0;
 $unsgnd_buf = $unsgnd_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0.0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0.0, $132 = 0, $133 = 0.0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cdepth = 0, $cmptscnt = 0, $i = 0, $img_dat = 0, $max_hor = 0, $max_vrt = 0, $nptr = 0, $rwcnt = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer6 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $oimg_dat;
 $2 = $image;
 $3 = $unsgnd_buf;
 $4 = (_calloc(1,132)|0);
 $img_dat = $4;
 $5 = $img_dat;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 132;
  (_fprintf(($7|0),(27176|0),($vararg_buffer|0))|0);
  $0 = -2;
  $190 = $0;
  STACKTOP = sp;return ($190|0);
 }
 $8 = $2;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $cmptscnt = $10;
 $11 = $2;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $img_dat;
 HEAP32[$14>>2] = $13;
 $15 = $2;
 $16 = (($15) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $img_dat;
 $19 = (($18) + 4|0);
 HEAP32[$19>>2] = $17;
 $20 = $img_dat;
 $21 = (($20) + 12|0);
 HEAP32[$21>>2] = -1;
 $22 = $img_dat;
 $23 = (($22) + 16|0);
 HEAP32[$23>>2] = 0;
 $24 = $cmptscnt;
 $25 = $img_dat;
 $26 = (($25) + 20|0);
 HEAP32[$26>>2] = $24;
 $27 = $2;
 $28 = (($27) + 24|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = $img_dat;
 $33 = (($32) + 24|0);
 HEAP32[$33>>2] = $31;
 $max_hor = -1;
 $max_vrt = -1;
 $34 = $img_dat;
 $35 = (($34) + 8|0);
 HEAP32[$35>>2] = 0;
 $i = 0;
 while(1) {
  $36 = $i;
  $37 = $cmptscnt;
  $38 = ($36|0)<($37|0);
  if (!($38)) {
   break;
  }
  $39 = $i;
  $40 = $img_dat;
  $41 = (($40) + 28|0);
  $42 = (($41) + ($39<<2)|0);
  HEAP32[$42>>2] = 1;
  $43 = $i;
  $44 = $img_dat;
  $45 = (($44) + 44|0);
  $46 = (($45) + ($43<<2)|0);
  HEAP32[$46>>2] = 1;
  $47 = $2;
  $48 = (($47) + 24|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 24|0);
  $51 = HEAP32[$50>>2]|0;
  $cdepth = $51;
  $52 = $cdepth;
  $53 = $img_dat;
  $54 = (($53) + 8|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + ($52))|0;
  HEAP32[$54>>2] = $56;
  $57 = $cdepth;
  $58 = $img_dat;
  $59 = (($58) + 24|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = ($57|0)!=($60|0);
  if ($61) {
   $62 = HEAP32[_stderr>>2]|0;
   $63 = $i;
   $64 = $cdepth;
   $65 = $i;
   $66 = $2;
   $67 = (($66) + 24|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($68) + (($65*48)|0)|0);
   $70 = (($69) + 24|0);
   $71 = HEAP32[$70>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $63;
   $vararg_ptr4 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $64;
   $vararg_ptr5 = (($vararg_buffer1) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $71;
   (_fprintf(($62|0),(27240|0),($vararg_buffer1|0))|0);
  }
  $72 = $max_hor;
  $73 = $i;
  $74 = $img_dat;
  $75 = (($74) + 28|0);
  $76 = (($75) + ($73<<2)|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($72|0)<($77|0);
  if ($78) {
   $79 = $i;
   $80 = $img_dat;
   $81 = (($80) + 28|0);
   $82 = (($81) + ($79<<2)|0);
   $83 = HEAP32[$82>>2]|0;
   $max_hor = $83;
  }
  $84 = $max_vrt;
  $85 = $i;
  $86 = $img_dat;
  $87 = (($86) + 44|0);
  $88 = (($87) + ($85<<2)|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = ($84|0)<($89|0);
  if ($90) {
   $91 = $i;
   $92 = $img_dat;
   $93 = (($92) + 44|0);
   $94 = (($93) + ($91<<2)|0);
   $95 = HEAP32[$94>>2]|0;
   $max_vrt = $95;
  }
  $96 = $i;
  $97 = (($96) + 1)|0;
  $i = $97;
 }
 $i = 0;
 while(1) {
  $98 = $i;
  $99 = $img_dat;
  $100 = (($99) + 20|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = ($98|0)<($101|0);
  if (!($102)) {
   break;
  }
  $103 = $img_dat;
  $104 = HEAP32[$103>>2]|0;
  $105 = (+($104|0));
  $106 = $i;
  $107 = $img_dat;
  $108 = (($107) + 28|0);
  $109 = (($108) + ($106<<2)|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = (+($110|0));
  $112 = $max_hor;
  $113 = (+($112|0));
  $114 = $111 / $113;
  $115 = $105 * $114;
  $116 = (+Math_ceil((+$115)));
  $117 = (~~(($116)));
  $118 = $i;
  $119 = $img_dat;
  $120 = (($119) + 60|0);
  $121 = (($120) + ($118<<2)|0);
  HEAP32[$121>>2] = $117;
  $122 = $img_dat;
  $123 = (($122) + 4|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = (+($124|0));
  $126 = $i;
  $127 = $img_dat;
  $128 = (($127) + 44|0);
  $129 = (($128) + ($126<<2)|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = (+($130|0));
  $132 = $max_vrt;
  $133 = (+($132|0));
  $134 = $131 / $133;
  $135 = $125 * $134;
  $136 = (+Math_ceil((+$135)));
  $137 = (~~(($136)));
  $138 = $i;
  $139 = $img_dat;
  $140 = (($139) + 76|0);
  $141 = (($140) + ($138<<2)|0);
  HEAP32[$141>>2] = $137;
  $142 = $i;
  $143 = (($142) + 1)|0;
  $i = $143;
 }
 $144 = $2;
 $145 = (($144) + 16|0);
 $146 = HEAP32[$145>>2]|0;
 $cmptscnt = $146;
 $147 = $2;
 $148 = (($147) + 8|0);
 $149 = HEAP32[$148>>2]|0;
 $150 = $2;
 $151 = (($150) + 12|0);
 $152 = HEAP32[$151>>2]|0;
 $153 = Math_imul($149, $152)|0;
 $rwcnt = $153;
 $154 = $3;
 $nptr = $154;
 $i = 0;
 while(1) {
  $155 = $i;
  $156 = $cmptscnt;
  $157 = ($155|0)<($156|0);
  if (!($157)) {
   label = 23;
   break;
  }
  $158 = $rwcnt;
  $159 = $158;
  $160 = (_malloc($159)|0);
  $161 = $i;
  $162 = $img_dat;
  $163 = (($162) + 100|0);
  $164 = (($163) + ($161<<2)|0);
  HEAP32[$164>>2] = $160;
  $165 = $i;
  $166 = $img_dat;
  $167 = (($166) + 100|0);
  $168 = (($167) + ($165<<2)|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = ($169|0)==(0|0);
  if ($170) {
   label = 20;
   break;
  }
  $175 = $i;
  $176 = $img_dat;
  $177 = (($176) + 100|0);
  $178 = (($177) + ($175<<2)|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = $nptr;
  $181 = $rwcnt;
  $182 = $181;
  _memcpy(($179|0),($180|0),($182|0))|0;
  $183 = $rwcnt;
  $184 = $nptr;
  $185 = (($184) + ($183)|0);
  $nptr = $185;
  $186 = $i;
  $187 = (($186) + 1)|0;
  $i = $187;
 }
 if ((label|0) == 20) {
  $171 = HEAP32[_stderr>>2]|0;
  $172 = $i;
  $173 = $rwcnt;
  $174 = $173;
  HEAP32[$vararg_buffer6>>2] = $172;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $174;
  (_fprintf(($171|0),(27312|0),($vararg_buffer6|0))|0);
  $0 = -3;
  $190 = $0;
  STACKTOP = sp;return ($190|0);
 }
 else if ((label|0) == 23) {
  $188 = $img_dat;
  $189 = $1;
  HEAP32[$189>>2] = $188;
  $0 = 0;
  $190 = $0;
  STACKTOP = sp;return ($190|0);
 }
 return 0|0;
}
function _png_mem_read_data($png_ptr,$data,$length) {
 $png_ptr = $png_ptr|0;
 $data = $data|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $have = 0, $iop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $data;
 $2 = $length;
 $3 = $0;
 $4 = (_png_get_io_ptr($3)|0);
 $iop = $4;
 $5 = $iop;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $iop;
 $9 = HEAP32[$8>>2]|0;
 $10 = $7;
 $11 = $9;
 $12 = (($10) - ($11))|0;
 $have = $12;
 $13 = $have;
 $14 = $2;
 $15 = ($13|0)<($14|0);
 if ($15) {
  $16 = $0;
  _png_error($16,27848);
 }
 $17 = $1;
 $18 = $iop;
 $19 = HEAP32[$18>>2]|0;
 $20 = $2;
 _memcpy(($17|0),($19|0),($20|0))|0;
 $21 = $2;
 $22 = $iop;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($21)|0);
 HEAP32[$22>>2] = $24;
 STACKTOP = sp;return;
}
function _png_decode_mem($oimg_dat,$lossyflag,$idata,$ilen) {
 $oimg_dat = $oimg_dat|0;
 $lossyflag = $lossyflag|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $$0 = 0, $$reg2mem27$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $info_ptr = 0, $ios = 0, $png_ptr = 0, _setjmpTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 _setjmpTable = STACKTOP; STACKTOP=(STACKTOP+168)|0;HEAP32[_setjmpTable>>2]=0;
 $png_ptr = sp + 12|0;
 $info_ptr = sp + 8|0;
 $ios = sp;
 HEAP32[$ios>>2] = $idata;
 $0 = (($idata) + ($ilen)|0);
 $1 = (($ios) + 4|0);
 HEAP32[$1>>2] = $0;
 __THREW__ = 0;
 $2 = (invoke_iiiii(21,(27864|0),(0|0),(0|0),(0|0))|0);
 $3 = __THREW__; __THREW__ = 0;
 if ((($3|0) != 0) & ((threwValue|0) != 0)) { $4 = _testSetjmp(HEAP32[$3>>2]|0, _setjmpTable)|0; if (($4|0) == 0) { _longjmp($3|0, threwValue|0); } tempRet0 = threwValue; } else { $4 = -1; };
 $5 = tempRet0;
 do {
  if ((($4|0) == 1)) {
   $$reg2mem27$0 = $5;
  } else {
   HEAP32[$png_ptr>>2] = $2;
   $6 = HEAP32[$png_ptr>>2]|0;
   $7 = (0|0)==($6|0);
   if ($7) {
    $$0 = -1;
    STACKTOP = sp;return ($$0|0);
   }
   $8 = HEAP32[$png_ptr>>2]|0;
   __THREW__ = 0;
   $9 = (invoke_ii(22,($8|0))|0);
   $10 = __THREW__; __THREW__ = 0;
   if ((($10|0) != 0) & ((threwValue|0) != 0)) { $11 = _testSetjmp(HEAP32[$10>>2]|0, _setjmpTable)|0; if (($11|0) == 0) { _longjmp($10|0, threwValue|0); } tempRet0 = threwValue; } else { $11 = -1; };
   $12 = tempRet0;
   if ((($11|0) == 1)) {
    $$reg2mem27$0 = $12;
   } else {
    HEAP32[$info_ptr>>2] = $9;
    $13 = HEAP32[$info_ptr>>2]|0;
    $14 = (0|0)==($13|0);
    if (!($14)) {
     $18 = HEAP32[$png_ptr>>2]|0;
     _saveSetjmp($18,1,_setjmpTable|0)|0;
     __THREW__ = 0;
     $19 = __THREW__; __THREW__ = 0;
     if ((($19|0) != 0) & ((threwValue|0) != 0)) { $20 = _testSetjmp(HEAP32[$19>>2]|0, _setjmpTable)|0; if (($20|0) == 0) { _longjmp($19|0, threwValue|0); } tempRet0 = threwValue; } else { $20 = -1; };
     $21 = tempRet0;
     if ((($20|0) == 1)) {
      $$reg2mem27$0 = $21;
      break;
     }
     $$reg2mem27$0 = 0;
     break;
    }
    __THREW__ = 0;
    invoke_viii(23,($png_ptr|0),(0|0),(0|0));
    $15 = __THREW__; __THREW__ = 0;
    if ((($15|0) != 0) & ((threwValue|0) != 0)) { $16 = _testSetjmp(HEAP32[$15>>2]|0, _setjmpTable)|0; if (($16|0) == 0) { _longjmp($15|0, threwValue|0); } tempRet0 = threwValue; } else { $16 = -1; };
    $17 = tempRet0;
    if ((($16|0) == 1)) {
     $$reg2mem27$0 = $17;
    } else {
     $$0 = -2;
     STACKTOP = sp;return ($$0|0);
    }
   }
  }
 } while(0);
 while(1) {
  $22 = ($$reg2mem27$0|0)!=(0);
  if ($22) {
   __THREW__ = 0;
   invoke_viii(23,($png_ptr|0),($info_ptr|0),(0|0));
   $23 = __THREW__; __THREW__ = 0;
   if ((($23|0) != 0) & ((threwValue|0) != 0)) { $24 = _testSetjmp(HEAP32[$23>>2]|0, _setjmpTable)|0; if (($24|0) == 0) { _longjmp($23|0, threwValue|0); } tempRet0 = threwValue; } else { $24 = -1; };
   $25 = tempRet0;
   if ((($24|0) == 1)) {
    $$reg2mem27$0 = $25;
    continue;
   } else {
    label = 12;
    break;
   }
  }
  $26 = HEAP32[$png_ptr>>2]|0;
  __THREW__ = 0;
  invoke_viii(24,($26|0),($ios|0),(25|0));
  $27 = __THREW__; __THREW__ = 0;
  if ((($27|0) != 0) & ((threwValue|0) != 0)) { $28 = _testSetjmp(HEAP32[$27>>2]|0, _setjmpTable)|0; if (($28|0) == 0) { _longjmp($27|0, threwValue|0); } tempRet0 = threwValue; } else { $28 = -1; };
  $29 = tempRet0;
  if ((($28|0) == 1)) {
   $$reg2mem27$0 = $29;
   continue;
  }
  $30 = HEAP32[$png_ptr>>2]|0;
  $31 = HEAP32[$info_ptr>>2]|0;
  __THREW__ = 0;
  $32 = (invoke_iiii(26,($30|0),($31|0),($oimg_dat|0))|0);
  $33 = __THREW__; __THREW__ = 0;
  if ((($33|0) != 0) & ((threwValue|0) != 0)) { $34 = _testSetjmp(HEAP32[$33>>2]|0, _setjmpTable)|0; if (($34|0) == 0) { _longjmp($33|0, threwValue|0); } tempRet0 = threwValue; } else { $34 = -1; };
  $35 = tempRet0;
  if ((($34|0) == 1)) {
   $$reg2mem27$0 = $35;
   continue;
  }
  __THREW__ = 0;
  invoke_viii(23,($png_ptr|0),($info_ptr|0),(0|0));
  $36 = __THREW__; __THREW__ = 0;
  if ((($36|0) != 0) & ((threwValue|0) != 0)) { $37 = _testSetjmp(HEAP32[$36>>2]|0, _setjmpTable)|0; if (($37|0) == 0) { _longjmp($36|0, threwValue|0); } tempRet0 = threwValue; } else { $37 = -1; };
  $38 = tempRet0;
  if ((($37|0) == 1)) {
   $$reg2mem27$0 = $38;
  } else {
   label = 16;
   break;
  }
 }
 if ((label|0) == 12) {
  $$0 = -3;
  STACKTOP = sp;return ($$0|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$lossyflag>>2] = 0;
  $$0 = $32;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _read_png_stream($png_ptr,$info_ptr,$oimg_dat) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $oimg_dat = $oimg_dat|0;
 var $$0 = 0, $$reg2mem42$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $img_dat = 0, $ret$0 = 0;
 var $vararg_buffer = 0, $vararg_buffer44 = 0, $vararg_buffer47 = 0, $vararg_ptr46 = 0, $vararg_ptr50 = 0, $y$0 = 0, $y$1 = 0, _setjmpTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer47 = sp + 16|0;
 $vararg_buffer44 = sp + 8|0;
 $vararg_buffer = sp;
 _setjmpTable = STACKTOP; STACKTOP=(STACKTOP+168)|0;HEAP32[_setjmpTable>>2]=0;
 $img_dat = sp + 24|0;
 __THREW__ = 0;
 invoke_vii(27,($png_ptr|0),($info_ptr|0));
 $0 = __THREW__; __THREW__ = 0;
 if ((($0|0) != 0) & ((threwValue|0) != 0)) { $1 = _testSetjmp(HEAP32[$0>>2]|0, _setjmpTable)|0; if (($1|0) == 0) { _longjmp($0|0, threwValue|0); } tempRet0 = threwValue; } else { $1 = -1; };
 $2 = tempRet0;
 if ((($1|0) == 1)) {
  $$reg2mem42$0 = $2;
 } else {
  __THREW__ = 0;
  (invoke_ii(28,($png_ptr|0))|0);
  $3 = __THREW__; __THREW__ = 0;
  if ((($3|0) != 0) & ((threwValue|0) != 0)) { $4 = _testSetjmp(HEAP32[$3>>2]|0, _setjmpTable)|0; if (($4|0) == 0) { _longjmp($3|0, threwValue|0); } tempRet0 = threwValue; } else { $4 = -1; };
  $5 = tempRet0;
  if ((($4|0) == 1)) {
   $$reg2mem42$0 = $5;
  } else {
   __THREW__ = 0;
   invoke_vii(29,($png_ptr|0),($info_ptr|0));
   $6 = __THREW__; __THREW__ = 0;
   if ((($6|0) != 0) & ((threwValue|0) != 0)) { $7 = _testSetjmp(HEAP32[$6>>2]|0, _setjmpTable)|0; if (($7|0) == 0) { _longjmp($6|0, threwValue|0); } tempRet0 = threwValue; } else { $7 = -1; };
   $8 = tempRet0;
   if ((($7|0) == 1)) {
    $$reg2mem42$0 = $8;
   } else {
    _saveSetjmp($png_ptr,1,_setjmpTable|0)|0;
    __THREW__ = 0;
    $9 = __THREW__; __THREW__ = 0;
    if ((($9|0) != 0) & ((threwValue|0) != 0)) { $10 = _testSetjmp(HEAP32[$9>>2]|0, _setjmpTable)|0; if (($10|0) == 0) { _longjmp($9|0, threwValue|0); } tempRet0 = threwValue; } else { $10 = -1; };
    $11 = tempRet0;
    if ((($10|0) == 1)) {
     $$reg2mem42$0 = $11;
    } else {
     $$reg2mem42$0 = 0;
    }
   }
  }
 }
 L6: while(1) {
  $12 = ($$reg2mem42$0|0)!=(0);
  if ($12) {
   $13 = HEAP32[_stderr>>2]|0;
   __THREW__ = 0;
   (invoke_iiii(30,($13|0),(28000|0),($vararg_buffer|0))|0);
   $14 = __THREW__; __THREW__ = 0;
   if ((($14|0) != 0) & ((threwValue|0) != 0)) { $15 = _testSetjmp(HEAP32[$14>>2]|0, _setjmpTable)|0; if (($15|0) == 0) { _longjmp($14|0, threwValue|0); } tempRet0 = threwValue; } else { $15 = -1; };
   $16 = tempRet0;
   if ((($15|0) == 1)) {
    $$reg2mem42$0 = $16;
    continue;
   } else {
    label = 8;
    break;
   }
  }
  $17 = (($info_ptr) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $18<<2;
  __THREW__ = 0;
  $20 = (invoke_ii(31,($19|0))|0);
  $21 = __THREW__; __THREW__ = 0;
  if ((($21|0) != 0) & ((threwValue|0) != 0)) { $22 = _testSetjmp(HEAP32[$21>>2]|0, _setjmpTable)|0; if (($22|0) == 0) { _longjmp($21|0, threwValue|0); } tempRet0 = threwValue; } else { $22 = -1; };
  $23 = tempRet0;
  if ((($22|0) == 1)) {
   $$reg2mem42$0 = $23;
   continue;
  }
  $24 = ($20|0)==(0|0);
  if ($24) {
   $25 = HEAP32[_stderr>>2]|0;
   $26 = (($info_ptr) + 4|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($info_ptr) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $29<<2;
   __THREW__ = 0;
   HEAP32[$vararg_buffer44>>2] = $27;
   $vararg_ptr46 = (($vararg_buffer44) + 4|0);
   HEAP32[$vararg_ptr46>>2] = $30;
   (invoke_iiii(30,($25|0),(28080|0),($vararg_buffer44|0))|0);
   $31 = __THREW__; __THREW__ = 0;
   if ((($31|0) != 0) & ((threwValue|0) != 0)) { $32 = _testSetjmp(HEAP32[$31>>2]|0, _setjmpTable)|0; if (($32|0) == 0) { _longjmp($31|0, threwValue|0); } tempRet0 = threwValue; } else { $32 = -1; };
   $33 = tempRet0;
   if ((($32|0) == 1)) {
    $$reg2mem42$0 = $33;
    continue;
   } else {
    label = 12;
    break;
   }
  }
  $y$0 = 0;
  while(1) {
   $34 = (($info_ptr) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($y$0|0)<($35|0);
   if (!($36)) {
    break;
   }
   $37 = (($info_ptr) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   __THREW__ = 0;
   $39 = (invoke_ii(31,($38|0))|0);
   $40 = __THREW__; __THREW__ = 0;
   if ((($40|0) != 0) & ((threwValue|0) != 0)) { $41 = _testSetjmp(HEAP32[$40>>2]|0, _setjmpTable)|0; if (($41|0) == 0) { _longjmp($40|0, threwValue|0); } tempRet0 = threwValue; } else { $41 = -1; };
   $42 = tempRet0;
   if ((($41|0) == 1)) {
    $$reg2mem42$0 = $42;
    continue L6;
   }
   $43 = (($20) + ($y$0<<2)|0);
   HEAP32[$43>>2] = $39;
   $44 = (($20) + ($y$0<<2)|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)==(0|0);
   if ($46) {
    label = 17;
    break;
   }
   $53 = (($y$0) + 1)|0;
   $y$0 = $53;
  }
  if ((label|0) == 17) {
   label = 0;
   $47 = HEAP32[_stderr>>2]|0;
   $48 = (($info_ptr) + 12|0);
   $49 = HEAP32[$48>>2]|0;
   __THREW__ = 0;
   HEAP32[$vararg_buffer47>>2] = $y$0;
   $vararg_ptr50 = (($vararg_buffer47) + 4|0);
   HEAP32[$vararg_ptr50>>2] = $49;
   (invoke_iiii(30,($47|0),(28144|0),($vararg_buffer47|0))|0);
   $50 = __THREW__; __THREW__ = 0;
   if ((($50|0) != 0) & ((threwValue|0) != 0)) { $51 = _testSetjmp(HEAP32[$50>>2]|0, _setjmpTable)|0; if (($51|0) == 0) { _longjmp($50|0, threwValue|0); } tempRet0 = threwValue; } else { $51 = -1; };
   $52 = tempRet0;
   if ((($51|0) == 1)) {
    $$reg2mem42$0 = $52;
    continue;
   } else {
    label = 18;
    break;
   }
  }
  __THREW__ = 0;
  invoke_vii(32,($png_ptr|0),($20|0));
  $54 = __THREW__; __THREW__ = 0;
  if ((($54|0) != 0) & ((threwValue|0) != 0)) { $55 = _testSetjmp(HEAP32[$54>>2]|0, _setjmpTable)|0; if (($55|0) == 0) { _longjmp($54|0, threwValue|0); } tempRet0 = threwValue; } else { $55 = -1; };
  $56 = tempRet0;
  if ((($55|0) == 1)) {
   $$reg2mem42$0 = $56;
   continue;
  }
  __THREW__ = 0;
  $57 = (invoke_iiii(33,($20|0),($info_ptr|0),($img_dat|0))|0);
  $58 = __THREW__; __THREW__ = 0;
  if ((($58|0) != 0) & ((threwValue|0) != 0)) { $59 = _testSetjmp(HEAP32[$58>>2]|0, _setjmpTable)|0; if (($59|0) == 0) { _longjmp($58|0, threwValue|0); } tempRet0 = threwValue; } else { $59 = -1; };
  $60 = tempRet0;
  if ((($59|0) == 1)) {
   $$reg2mem42$0 = $60;
   continue;
  }
  $61 = ($57|0)!=(0);
  if ($61) {
   $ret$0 = -7;
  } else {
   $ret$0 = 0;
  }
  $y$1 = 0;
  while(1) {
   $62 = (($info_ptr) + 4|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($y$1|0)<($63|0);
   if (!($64)) {
    break;
   }
   $65 = (($20) + ($y$1<<2)|0);
   $66 = HEAP32[$65>>2]|0;
   __THREW__ = 0;
   invoke_vi(34,($66|0));
   $67 = __THREW__; __THREW__ = 0;
   if ((($67|0) != 0) & ((threwValue|0) != 0)) { $68 = _testSetjmp(HEAP32[$67>>2]|0, _setjmpTable)|0; if (($68|0) == 0) { _longjmp($67|0, threwValue|0); } tempRet0 = threwValue; } else { $68 = -1; };
   $69 = tempRet0;
   if ((($68|0) == 1)) {
    $$reg2mem42$0 = $69;
    continue L6;
   }
   $70 = (($y$1) + 1)|0;
   $y$1 = $70;
  }
  __THREW__ = 0;
  invoke_vi(34,($20|0));
  $71 = __THREW__; __THREW__ = 0;
  if ((($71|0) != 0) & ((threwValue|0) != 0)) { $72 = _testSetjmp(HEAP32[$71>>2]|0, _setjmpTable)|0; if (($72|0) == 0) { _longjmp($71|0, threwValue|0); } tempRet0 = threwValue; } else { $72 = -1; };
  $73 = tempRet0;
  if ((($72|0) == 1)) {
   $$reg2mem42$0 = $73;
  } else {
   label = 32;
   break;
  }
 }
 if ((label|0) == 8) {
  $$0 = -6;
  STACKTOP = sp;return ($$0|0);
 }
 else if ((label|0) == 12) {
  $$0 = -8;
  STACKTOP = sp;return ($$0|0);
 }
 else if ((label|0) == 18) {
  $$0 = -9;
  STACKTOP = sp;return ($$0|0);
 }
 else if ((label|0) == 32) {
  $74 = ($ret$0|0)==(0);
  if ($74) {
   $75 = HEAP32[$img_dat>>2]|0;
   HEAP32[$oimg_dat>>2] = $75;
  }
  $$0 = $ret$0;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _get_raw_image($row_pointers,$info_ptr,$oimg_dat) {
 $row_pointers = $row_pointers|0;
 $info_ptr = $info_ptr|0;
 $oimg_dat = $oimg_dat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0.0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $img_dat = 0, $index = 0, $j = 0, $max_hor = 0, $max_vrt = 0, $pixel_d = 0, $ptr = 0, $row = 0, $rwcnt = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $row_pointers;
 $2 = $info_ptr;
 $3 = $oimg_dat;
 $4 = (_calloc(1,132)|0);
 $img_dat = $4;
 $5 = $img_dat;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  HEAP32[$vararg_buffer>>2] = 132;
  (_fprintf(($7|0),(27872|0),($vararg_buffer|0))|0);
  $0 = -2;
  $229 = $0;
  STACKTOP = sp;return ($229|0);
 }
 $8 = $2;
 $9 = HEAP32[$8>>2]|0;
 $10 = $img_dat;
 HEAP32[$10>>2] = $9;
 $11 = $2;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $img_dat;
 $15 = (($14) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = $2;
 $17 = (($16) + 29|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(4);
 if ($20) {
  $21 = $2;
  $22 = (($21) + 29|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  $25 = (($24) - 1)|0;
  $26 = $25<<3;
  $27 = $img_dat;
  $28 = (($27) + 8|0);
  HEAP32[$28>>2] = $26;
  $29 = $2;
  $30 = (($29) + 29|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = (($32) - 1)|0;
  $34 = $img_dat;
  $35 = (($34) + 20|0);
  HEAP32[$35>>2] = $33;
 } else {
  $36 = $2;
  $37 = (($36) + 29|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = $38&255;
  $40 = $39<<3;
  $41 = $img_dat;
  $42 = (($41) + 8|0);
  HEAP32[$42>>2] = $40;
  $43 = $2;
  $44 = (($43) + 29|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $img_dat;
  $48 = (($47) + 20|0);
  HEAP32[$48>>2] = $46;
 }
 $49 = $img_dat;
 $50 = (($49) + 12|0);
 HEAP32[$50>>2] = -1;
 $51 = $img_dat;
 $52 = (($51) + 16|0);
 HEAP32[$52>>2] = 0;
 $53 = $img_dat;
 $54 = (($53) + 24|0);
 HEAP32[$54>>2] = 8;
 $max_hor = -1;
 $max_vrt = -1;
 $i = 0;
 while(1) {
  $55 = $i;
  $56 = $img_dat;
  $57 = (($56) + 20|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ($55|0)<($58|0);
  if (!($59)) {
   break;
  }
  $60 = $i;
  $61 = $img_dat;
  $62 = (($61) + 28|0);
  $63 = (($62) + ($60<<2)|0);
  HEAP32[$63>>2] = 1;
  $64 = $i;
  $65 = $img_dat;
  $66 = (($65) + 44|0);
  $67 = (($66) + ($64<<2)|0);
  HEAP32[$67>>2] = 1;
  $68 = $max_hor;
  $69 = $i;
  $70 = $img_dat;
  $71 = (($70) + 28|0);
  $72 = (($71) + ($69<<2)|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = ($68|0)<($73|0);
  if ($74) {
   $75 = $i;
   $76 = $img_dat;
   $77 = (($76) + 28|0);
   $78 = (($77) + ($75<<2)|0);
   $79 = HEAP32[$78>>2]|0;
   $max_hor = $79;
  }
  $80 = $max_vrt;
  $81 = $i;
  $82 = $img_dat;
  $83 = (($82) + 44|0);
  $84 = (($83) + ($81<<2)|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($80|0)<($85|0);
  if ($86) {
   $87 = $i;
   $88 = $img_dat;
   $89 = (($88) + 44|0);
   $90 = (($89) + ($87<<2)|0);
   $91 = HEAP32[$90>>2]|0;
   $max_vrt = $91;
  }
  $92 = $i;
  $93 = (($92) + 1)|0;
  $i = $93;
 }
 $i = 0;
 while(1) {
  $94 = $i;
  $95 = $img_dat;
  $96 = (($95) + 20|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($94|0)<($97|0);
  if (!($98)) {
   break;
  }
  $99 = $img_dat;
  $100 = HEAP32[$99>>2]|0;
  $101 = (+($100|0));
  $102 = $i;
  $103 = $img_dat;
  $104 = (($103) + 28|0);
  $105 = (($104) + ($102<<2)|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (+($106|0));
  $108 = $max_hor;
  $109 = (+($108|0));
  $110 = $107 / $109;
  $111 = $101 * $110;
  $112 = (+Math_ceil((+$111)));
  $113 = (~~(($112)));
  $114 = $i;
  $115 = $img_dat;
  $116 = (($115) + 60|0);
  $117 = (($116) + ($114<<2)|0);
  HEAP32[$117>>2] = $113;
  $118 = $img_dat;
  $119 = (($118) + 4|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = (+($120|0));
  $122 = $i;
  $123 = $img_dat;
  $124 = (($123) + 44|0);
  $125 = (($124) + ($122<<2)|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = (+($126|0));
  $128 = $max_vrt;
  $129 = (+($128|0));
  $130 = $127 / $129;
  $131 = $121 * $130;
  $132 = (+Math_ceil((+$131)));
  $133 = (~~(($132)));
  $134 = $i;
  $135 = $img_dat;
  $136 = (($135) + 76|0);
  $137 = (($136) + ($134<<2)|0);
  HEAP32[$137>>2] = $133;
  $138 = $i;
  $139 = (($138) + 1)|0;
  $i = $139;
 }
 $140 = $img_dat;
 $141 = HEAP32[$140>>2]|0;
 $142 = $img_dat;
 $143 = (($142) + 4|0);
 $144 = HEAP32[$143>>2]|0;
 $145 = Math_imul($141, $144)|0;
 $rwcnt = $145;
 $i = 0;
 while(1) {
  $146 = $i;
  $147 = $img_dat;
  $148 = (($147) + 20|0);
  $149 = HEAP32[$148>>2]|0;
  $150 = ($146|0)<($149|0);
  if (!($150)) {
   break;
  }
  $151 = $rwcnt;
  $152 = $151;
  $153 = (_malloc($152)|0);
  $154 = $i;
  $155 = $img_dat;
  $156 = (($155) + 100|0);
  $157 = (($156) + ($154<<2)|0);
  HEAP32[$157>>2] = $153;
  $158 = $i;
  $159 = $img_dat;
  $160 = (($159) + 100|0);
  $161 = (($160) + ($158<<2)|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = ($162|0)==(0|0);
  if ($163) {
   label = 21;
   break;
  }
  $179 = $i;
  $180 = (($179) + 1)|0;
  $i = $180;
 }
 if ((label|0) == 21) {
  $164 = HEAP32[_stderr>>2]|0;
  $165 = $i;
  $166 = $rwcnt;
  $167 = $166;
  HEAP32[$vararg_buffer1>>2] = $165;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $167;
  (_fprintf(($164|0),(27928|0),($vararg_buffer1|0))|0);
  $j = 0;
  while(1) {
   $168 = $j;
   $169 = $i;
   $170 = ($168|0)<($169|0);
   if (!($170)) {
    break;
   }
   $171 = $j;
   $172 = $img_dat;
   $173 = (($172) + 100|0);
   $174 = (($173) + ($171<<2)|0);
   $175 = HEAP32[$174>>2]|0;
   _free($175);
   $176 = $j;
   $177 = (($176) + 1)|0;
   $j = $177;
  }
  $178 = $img_dat;
  _free($178);
  $0 = -3;
  $229 = $0;
  STACKTOP = sp;return ($229|0);
 }
 $181 = $2;
 $182 = (($181) + 29|0);
 $183 = HEAP8[$182>>0]|0;
 $184 = $183&255;
 $pixel_d = $184;
 $index = 0;
 $y = 0;
 while(1) {
  $185 = $y;
  $186 = $2;
  $187 = (($186) + 4|0);
  $188 = HEAP32[$187>>2]|0;
  $189 = ($185>>>0)<($188>>>0);
  if (!($189)) {
   break;
  }
  $190 = $y;
  $191 = $1;
  $192 = (($191) + ($190<<2)|0);
  $193 = HEAP32[$192>>2]|0;
  $row = $193;
  $x = 0;
  while(1) {
   $194 = $x;
   $195 = $2;
   $196 = HEAP32[$195>>2]|0;
   $197 = ($194>>>0)<($196>>>0);
   if (!($197)) {
    break;
   }
   $198 = $x;
   $199 = $pixel_d;
   $200 = Math_imul($198, $199)|0;
   $201 = $row;
   $202 = (($201) + ($200)|0);
   $ptr = $202;
   $i = 0;
   while(1) {
    $203 = $i;
    $204 = $img_dat;
    $205 = (($204) + 20|0);
    $206 = HEAP32[$205>>2]|0;
    $207 = ($203|0)<($206|0);
    if (!($207)) {
     break;
    }
    $208 = $i;
    $209 = $ptr;
    $210 = (($209) + ($208)|0);
    $211 = HEAP8[$210>>0]|0;
    $212 = $i;
    $213 = $img_dat;
    $214 = (($213) + 100|0);
    $215 = (($214) + ($212<<2)|0);
    $216 = HEAP32[$215>>2]|0;
    $217 = $index;
    $218 = (($216) + ($217)|0);
    HEAP8[$218>>0] = $211;
    $219 = $i;
    $220 = (($219) + 1)|0;
    $i = $220;
   }
   $221 = $index;
   $222 = (($221) + 1)|0;
   $index = $222;
   $223 = $x;
   $224 = (($223) + 1)|0;
   $x = $224;
  }
  $225 = $y;
  $226 = (($225) + 1)|0;
  $y = $226;
 }
 $227 = $img_dat;
 $228 = $3;
 HEAP32[$228>>2] = $227;
 $0 = 0;
 $229 = $0;
 STACKTOP = sp;return ($229|0);
}
function _read_raw_from_filesize($ifile,$odata,$ofsize) {
 $ifile = $ifile|0;
 $odata = $odata|0;
 $ofsize = $ofsize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fsize = 0, $idata = 0;
 var $infp = 0, $n = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ifile;
 $2 = $odata;
 $3 = $ofsize;
 $4 = $1;
 $5 = (_filesize($4)|0);
 $ret = $5;
 $6 = ($5|0)<(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $37 = $0;
  STACKTOP = sp;return ($37|0);
 }
 $8 = $ret;
 $fsize = $8;
 $9 = $1;
 $10 = (_fopen(($9|0),(28200|0))|0);
 $infp = $10;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  $13 = $1;
  HEAP32[$vararg_buffer>>2] = $13;
  (_fprintf(($12|0),(28208|0),($vararg_buffer|0))|0);
  $0 = -2;
  $37 = $0;
  STACKTOP = sp;return ($37|0);
 }
 $14 = $fsize;
 $15 = $14;
 $16 = (_malloc($15)|0);
 $idata = $16;
 $17 = $idata;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(28256|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $37 = $0;
  STACKTOP = sp;return ($37|0);
 }
 $20 = $idata;
 $21 = $fsize;
 $22 = $infp;
 $23 = (_fread(($20|0),1,($21|0),($22|0))|0);
 $n = $23;
 $24 = $n;
 $25 = $fsize;
 $26 = ($24|0)!=($25|0);
 if ($26) {
  $27 = HEAP32[_stderr>>2]|0;
  (_fprintf(($27|0),(28312|0),($vararg_buffer3|0))|0);
  $28 = HEAP32[_stderr>>2]|0;
  $29 = $n;
  $30 = $fsize;
  $31 = $1;
  HEAP32[$vararg_buffer5>>2] = $29;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $30;
  $vararg_ptr9 = (($vararg_buffer5) + 8|0);
  HEAP32[$vararg_ptr9>>2] = $31;
  (_fprintf(($28|0),(28360|0),($vararg_buffer5|0))|0);
  $0 = -4;
  $37 = $0;
  STACKTOP = sp;return ($37|0);
 } else {
  $32 = $infp;
  (_fclose(($32|0))|0);
  $33 = $idata;
  $34 = $2;
  HEAP32[$34>>2] = $33;
  $35 = $fsize;
  $36 = $3;
  HEAP32[$36>>2] = $35;
  $0 = 0;
  $37 = $0;
  STACKTOP = sp;return ($37|0);
 }
 return 0|0;
}
function _write_raw_from_memsize($ofile,$odata,$olen) {
 $ofile = $ofile|0;
 $odata = $odata|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $n = 0, $outfp = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofile;
 $2 = $odata;
 $3 = $olen;
 $4 = $1;
 $5 = (_fopen(($4|0),(28392|0))|0);
 $outfp = $5;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $1;
  HEAP32[$vararg_buffer>>2] = $8;
  (_fprintf(($7|0),(28400|0),($vararg_buffer|0))|0);
  $0 = -2;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 $9 = $2;
 $10 = $3;
 $11 = $outfp;
 $12 = (_fwrite(($9|0),1,($10|0),($11|0))|0);
 $n = $12;
 $13 = $3;
 $14 = ($12|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(28448|0),($vararg_buffer1|0))|0);
  $16 = HEAP32[_stderr>>2]|0;
  $17 = $n;
  $18 = $3;
  $19 = $1;
  HEAP32[$vararg_buffer3>>2] = $17;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $18;
  $vararg_ptr7 = (($vararg_buffer3) + 8|0);
  HEAP32[$vararg_ptr7>>2] = $19;
  (_fprintf(($16|0),(28496|0),($vararg_buffer3|0))|0);
  $0 = -3;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 } else {
  $20 = $outfp;
  (_fclose(($20|0))|0);
  $0 = 0;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _trim_histtails_contrast_boost($idata,$iw,$ih) {
 $idata = $idata|0;
 $iw = $iw|0;
 $ih = $ih|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0.0, $107 = 0.0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0.0, $94 = 0.0, $95 = 0.0, $96 = 0.0, $97 = 0, $98 = 0, $99 = 0, $bins = 0, $cnt = 0, $factr = 0.0, $i = 0, $img_len = 0;
 var $img_mn = 0, $img_mnv = 0, $img_mxv = 0, $inrun = 0, $iptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1088|0;
 $bins = sp + 40|0;
 $0 = $idata;
 $1 = $iw;
 $2 = $ih;
 _memset(($bins|0),0,1024)|0;
 $3 = $1;
 $4 = $2;
 $5 = Math_imul($3, $4)|0;
 $img_len = $5;
 $img_mn = 0;
 $6 = $0;
 $iptr = $6;
 $i = 0;
 while(1) {
  $7 = $i;
  $8 = $img_len;
  $9 = ($7|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $iptr;
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = $img_mn;
  $14 = (($13) + ($12))|0;
  $img_mn = $14;
  $15 = $iptr;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($bins) + ($17<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$18>>2] = $20;
  $21 = $iptr;
  $22 = (($21) + 1|0);
  $iptr = $22;
  $23 = $i;
  $24 = (($23) + 1)|0;
  $i = $24;
 }
 $25 = $img_len;
 $26 = $img_mn;
 $27 = (($26|0) / ($25|0))&-1;
 $img_mn = $27;
 $inrun = 0;
 $img_mxv = 255;
 $cnt = 0;
 $28 = $img_mn;
 $i = $28;
 while(1) {
  $29 = $i;
  $30 = ($29|0)<(256);
  if (!($30)) {
   break;
  }
  $31 = $i;
  $32 = (($bins) + ($31<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)<(10);
  if ($34) {
   $35 = $inrun;
   $36 = ($35|0)!=(0);
   if (!($36)) {
    $inrun = 1;
    $cnt = 0;
   }
   $37 = $cnt;
   $38 = (($37) + 1)|0;
   $cnt = $38;
  } else {
   $inrun = 0;
  }
  $39 = $cnt;
  $40 = ($39|0)==(5);
  if ($40) {
   label = 13;
   break;
  }
  $43 = $i;
  $44 = (($43) + 1)|0;
  $i = $44;
 }
 if ((label|0) == 13) {
  $41 = $i;
  $42 = (($41) - 5)|0;
  $img_mxv = $42;
 }
 $inrun = 0;
 $img_mnv = 0;
 $cnt = 0;
 $45 = $img_mn;
 $i = $45;
 while(1) {
  $46 = $i;
  $47 = ($46|0)>=(0);
  if (!($47)) {
   break;
  }
  $48 = $i;
  $49 = (($bins) + ($48<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = ($50|0)<(10);
  if ($51) {
   $52 = $inrun;
   $53 = ($52|0)!=(0);
   if (!($53)) {
    $inrun = 1;
    $cnt = 0;
   }
   $54 = $cnt;
   $55 = (($54) + 1)|0;
   $cnt = $55;
  } else {
   $inrun = 0;
  }
  $56 = $cnt;
  $57 = ($56|0)==(5);
  if ($57) {
   label = 24;
   break;
  }
  $60 = $i;
  $61 = (($60) + -1)|0;
  $i = $61;
 }
 if ((label|0) == 24) {
  $58 = $i;
  $59 = (($58) + 5)|0;
  $img_mnv = $59;
 }
 $62 = $img_mnv;
 $63 = ($62|0)!=(0);
 if (!($63)) {
  $64 = $img_mxv;
  $65 = ($64|0)!=(255);
  if (!($65)) {
   STACKTOP = sp;return;
  }
 }
 $66 = $0;
 $iptr = $66;
 $i = 0;
 while(1) {
  $67 = $i;
  $68 = $img_len;
  $69 = ($67|0)<($68|0);
  if (!($69)) {
   break;
  }
  $70 = $iptr;
  $71 = HEAP8[$70>>0]|0;
  $72 = $71&255;
  $73 = $img_mnv;
  $74 = ($72|0)<($73|0);
  if ($74) {
   $75 = $img_mnv;
   $76 = $75&255;
   $77 = $iptr;
   HEAP8[$77>>0] = $76;
  } else {
   $78 = $iptr;
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = $img_mxv;
   $82 = ($80|0)>($81|0);
   if ($82) {
    $83 = $img_mxv;
    $84 = $83&255;
    $85 = $iptr;
    HEAP8[$85>>0] = $84;
   }
  }
  $86 = $iptr;
  $87 = (($86) + 1|0);
  $iptr = $87;
  $88 = $i;
  $89 = (($88) + 1)|0;
  $i = $89;
 }
 $90 = $img_mxv;
 $91 = $img_mnv;
 $92 = (($90) - ($91))|0;
 $93 = (+($92|0));
 $94 = $93;
 $95 = 255.0 / $94;
 $96 = $95;
 $factr = $96;
 $97 = $0;
 $iptr = $97;
 $i = 0;
 while(1) {
  $98 = $i;
  $99 = $img_len;
  $100 = ($98|0)<($99|0);
  if (!($100)) {
   break;
  }
  $101 = $iptr;
  $102 = HEAP8[$101>>0]|0;
  $103 = $102&255;
  $104 = $img_mnv;
  $105 = (($103) - ($104))|0;
  $106 = (+($105|0));
  $107 = $factr;
  $108 = $106 * $107;
  $109 = (~~(($108))&255);
  $110 = $iptr;
  HEAP8[$110>>0] = $109;
  $111 = $iptr;
  $112 = (($111) + 1|0);
  $iptr = $112;
  $113 = $i;
  $114 = (($113) + 1)|0;
  $i = $114;
 }
 STACKTOP = sp;return;
}
function _read_and_decode_grayscale_image($ifile,$oimg_type,$odata,$olen,$ow,$oh,$od,$oppi) {
 $ifile = $ifile|0;
 $oimg_type = $oimg_type|0;
 $odata = $odata|0;
 $olen = $olen|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppi = $oppi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0.0, $43 = 0.0, $44 = 0.0;
 var $45 = 0.0, $46 = 0.0, $47 = 0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $ansi_nist = 0, $d = 0, $h = 0, $hor_sampfctr = 0, $idata = 0, $ilen = 0, $img_idc = 0, $img_imp = 0, $img_type = 0, $imgrecord = 0, $imgrecord_i = 0, $intrlvflag = 0, $n_cmpnts = 0, $ppi = 0, $ppmm = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0;
 var $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr13 = 0, $vrt_sampfctr = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer10 = sp + 48|0;
 $vararg_buffer8 = sp + 40|0;
 $vararg_buffer5 = sp + 32|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $idata = sp + 140|0;
 $img_type = sp + 136|0;
 $ilen = sp + 132|0;
 $w = sp + 128|0;
 $h = sp + 124|0;
 $d = sp + 120|0;
 $ppi = sp + 116|0;
 $intrlvflag = sp + 112|0;
 $hor_sampfctr = sp + 96|0;
 $vrt_sampfctr = sp + 80|0;
 $n_cmpnts = sp + 76|0;
 $ansi_nist = sp + 72|0;
 $imgrecord = sp + 68|0;
 $img_idc = sp + 64|0;
 $img_imp = sp + 60|0;
 $imgrecord_i = sp + 56|0;
 $ppmm = sp;
 $1 = $ifile;
 $2 = $oimg_type;
 $3 = $odata;
 $4 = $olen;
 $5 = $ow;
 $6 = $oh;
 $7 = $od;
 $8 = $oppi;
 $9 = $3;
 HEAP32[$9>>2] = 0;
 $10 = $4;
 HEAP32[$10>>2] = 0;
 $11 = $1;
 $12 = (_is_ANSI_NIST_file($11)|0);
 $ret = $12;
 $13 = $ret;
 $14 = ($13|0)<(0);
 if ($14) {
  $15 = $ret;
  $0 = $15;
  $80 = $0;
  STACKTOP = sp;return ($80|0);
 }
 $16 = $ret;
 $17 = ($16|0)==(1);
 do {
  if ($17) {
   $18 = $1;
   $19 = (_read_ANSI_NIST_file($18,$ansi_nist)|0);
   $ret = $19;
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = $ret;
    $0 = $21;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   $22 = HEAP32[$ansi_nist>>2]|0;
   $23 = (_get_first_grayprint($idata,$w,$h,$d,$ppmm,$img_idc,$img_imp,$imgrecord,$imgrecord_i,$22)|0);
   $ret = $23;
   $24 = $ret;
   $25 = ($24|0)<(0);
   if ($25) {
    $26 = HEAP32[$ansi_nist>>2]|0;
    _free_ANSI_NIST($26);
    $27 = $ret;
    $0 = $27;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   $28 = $ret;
   $29 = ($28|0)!=(0);
   if (!($29)) {
    $30 = HEAP32[_stderr>>2]|0;
    (_fprintf(($30|0),(28544|0),($vararg_buffer|0))|0);
    $31 = HEAP32[_stderr>>2]|0;
    $32 = $1;
    HEAP32[$vararg_buffer1>>2] = $32;
    (_fprintf(($31|0),(28592|0),($vararg_buffer1|0))|0);
    $33 = HEAP32[$ansi_nist>>2]|0;
    _free_ANSI_NIST($33);
    $0 = -2;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   $34 = HEAP32[$ansi_nist>>2]|0;
   _free_ANSI_NIST($34);
   HEAP32[$img_type>>2] = 5;
   $35 = HEAP32[$w>>2]|0;
   $36 = HEAP32[$h>>2]|0;
   $37 = Math_imul($35, $36)|0;
   HEAP32[$ilen>>2] = $37;
   $38 = +HEAPF64[$ppmm>>3];
   $39 = $38 * 25.3999999999999985789;
   $40 = $39 < 0.0;
   if ($40) {
    $41 = +HEAPF64[$ppmm>>3];
    $42 = $41 * 25.3999999999999985789;
    $43 = $42 - 0.5;
    $48 = $43;
   } else {
    $44 = +HEAPF64[$ppmm>>3];
    $45 = $44 * 25.3999999999999985789;
    $46 = $45 + 0.5;
    $48 = $46;
   }
   $47 = (~~(($48)));
   HEAP32[$ppi>>2] = $47;
  } else {
   $49 = $1;
   $50 = (_read_and_decode_image($49,$img_type,$idata,$ilen,$w,$h,$d,$ppi,$intrlvflag,$hor_sampfctr,$vrt_sampfctr,$n_cmpnts)|0);
   $ret = $50;
   $51 = ($50|0)!=(0);
   if ($51) {
    $52 = $ret;
    $0 = $52;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   $53 = HEAP32[$img_type>>2]|0;
   $54 = ($53|0)==(-1);
   if ($54) {
    $55 = HEAP32[$idata>>2]|0;
    _free($55);
    $56 = HEAP32[_stderr>>2]|0;
    (_fprintf(($56|0),(28544|0),($vararg_buffer3|0))|0);
    $57 = HEAP32[_stderr>>2]|0;
    $58 = $1;
    HEAP32[$vararg_buffer5>>2] = $58;
    (_fprintf(($57|0),(28632|0),($vararg_buffer5|0))|0);
    $0 = -3;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   $59 = HEAP32[$d>>2]|0;
   $60 = ($59|0)!=(8);
   if (!($60)) {
    break;
   }
   $61 = HEAP32[$idata>>2]|0;
   _free($61);
   $62 = HEAP32[_stderr>>2]|0;
   (_fprintf(($62|0),(28544|0),($vararg_buffer8|0))|0);
   $63 = HEAP32[_stderr>>2]|0;
   $64 = $1;
   $65 = HEAP32[$d>>2]|0;
   HEAP32[$vararg_buffer10>>2] = $64;
   $vararg_ptr13 = (($vararg_buffer10) + 4|0);
   HEAP32[$vararg_ptr13>>2] = $65;
   (_fprintf(($63|0),(28680|0),($vararg_buffer10|0))|0);
   $0 = -4;
   $80 = $0;
   STACKTOP = sp;return ($80|0);
  }
 } while(0);
 $66 = HEAP32[$img_type>>2]|0;
 $67 = $2;
 HEAP32[$67>>2] = $66;
 $68 = HEAP32[$idata>>2]|0;
 $69 = $3;
 HEAP32[$69>>2] = $68;
 $70 = HEAP32[$ilen>>2]|0;
 $71 = $4;
 HEAP32[$71>>2] = $70;
 $72 = HEAP32[$w>>2]|0;
 $73 = $5;
 HEAP32[$73>>2] = $72;
 $74 = HEAP32[$h>>2]|0;
 $75 = $6;
 HEAP32[$75>>2] = $74;
 $76 = HEAP32[$d>>2]|0;
 $77 = $7;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$ppi>>2]|0;
 $79 = $8;
 HEAP32[$79>>2] = $78;
 $0 = 0;
 $80 = $0;
 STACKTOP = sp;return ($80|0);
}
function _read_and_decode_image($ifile,$oimg_type,$odata,$olen,$ow,$oh,$od,$oppi,$ointrlvflag,$hor_sampfctr,$vrt_sampfctr,$on_cmpnts) {
 $ifile = $ifile|0;
 $oimg_type = $oimg_type|0;
 $odata = $odata|0;
 $olen = $olen|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppi = $oppi|0;
 $ointrlvflag = $ointrlvflag|0;
 $hor_sampfctr = $hor_sampfctr|0;
 $vrt_sampfctr = $vrt_sampfctr|0;
 $on_cmpnts = $on_cmpnts|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d = 0, $h = 0, $i = 0, $idata = 0, $ilen = 0, $img_dat = 0, $img_type = 0, $intrlvflag = 0, $lossyflag = 0;
 var $n_cmpnts = 0, $ndata = 0, $nlen = 0, $ppi = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $idata = sp + 108|0;
 $ndata = sp + 104|0;
 $img_type = sp + 100|0;
 $ilen = sp + 96|0;
 $nlen = sp + 92|0;
 $w = sp + 88|0;
 $h = sp + 84|0;
 $d = sp + 80|0;
 $ppi = sp + 76|0;
 $lossyflag = sp + 72|0;
 $img_dat = sp + 60|0;
 $1 = $ifile;
 $2 = $oimg_type;
 $3 = $odata;
 $4 = $olen;
 $5 = $ow;
 $6 = $oh;
 $7 = $od;
 $8 = $oppi;
 $9 = $ointrlvflag;
 $10 = $hor_sampfctr;
 $11 = $vrt_sampfctr;
 $12 = $on_cmpnts;
 $intrlvflag = 0;
 $13 = $1;
 $14 = (_read_raw_from_filesize($13,$idata,$ilen)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $ret;
  $0 = $16;
  $250 = $0;
  STACKTOP = sp;return ($250|0);
 }
 $17 = HEAP32[$idata>>2]|0;
 $18 = HEAP32[$ilen>>2]|0;
 $19 = (_image_type($img_type,$17,$18)|0);
 $ret = $19;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$idata>>2]|0;
  _free($21);
  $22 = $ret;
  $0 = $22;
  $250 = $0;
  STACKTOP = sp;return ($250|0);
 }
 $23 = HEAP32[$img_type>>2]|0;
 L9: do {
  switch ($23|0) {
  case 7:  {
   $189 = HEAP32[$idata>>2]|0;
   $190 = HEAP32[$ilen>>2]|0;
   $191 = (_png_decode_mem($img_dat,$lossyflag,$189,$190)|0);
   $ret = $191;
   $192 = ($191|0)!=(0);
   if ($192) {
    $193 = HEAP32[$idata>>2]|0;
    _free($193);
    $194 = $ret;
    $0 = $194;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $195 = HEAP32[$img_dat>>2]|0;
   $196 = (_get_IMG_DAT_image($ndata,$nlen,$w,$h,$d,$ppi,$195)|0);
   $ret = $196;
   $197 = ($196|0)!=(0);
   if ($197) {
    $198 = HEAP32[$idata>>2]|0;
    _free($198);
    $199 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($199,1);
    $200 = $ret;
    $0 = $200;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $intrlvflag = 0;
   $201 = HEAP32[$img_dat>>2]|0;
   $202 = (($201) + 20|0);
   $203 = HEAP32[$202>>2]|0;
   $n_cmpnts = $203;
   $204 = HEAP32[$d>>2]|0;
   $205 = ($204|0)==(24);
   if ($205) {
    $i = 0;
    while(1) {
     $206 = $i;
     $207 = $n_cmpnts;
     $208 = ($206|0)<($207|0);
     if (!($208)) {
      break;
     }
     $209 = $i;
     $210 = HEAP32[$img_dat>>2]|0;
     $211 = (($210) + 28|0);
     $212 = (($211) + ($209<<2)|0);
     $213 = HEAP32[$212>>2]|0;
     $214 = $i;
     $215 = $10;
     $216 = (($215) + ($214<<2)|0);
     HEAP32[$216>>2] = $213;
     $217 = $i;
     $218 = HEAP32[$img_dat>>2]|0;
     $219 = (($218) + 44|0);
     $220 = (($219) + ($217<<2)|0);
     $221 = HEAP32[$220>>2]|0;
     $222 = $i;
     $223 = $11;
     $224 = (($223) + ($222<<2)|0);
     HEAP32[$224>>2] = $221;
     $225 = $i;
     $226 = (($225) + 1)|0;
     $i = $226;
    }
   }
   $227 = HEAP32[$img_dat>>2]|0;
   _free_IMG_DAT($227,1);
   break;
  }
  case -1:  {
   $24 = HEAP32[$img_type>>2]|0;
   $25 = $2;
   HEAP32[$25>>2] = $24;
   $26 = HEAP32[$idata>>2]|0;
   $27 = $3;
   HEAP32[$27>>2] = $26;
   $28 = HEAP32[$ilen>>2]|0;
   $29 = $4;
   HEAP32[$29>>2] = $28;
   $30 = $5;
   HEAP32[$30>>2] = -1;
   $31 = $6;
   HEAP32[$31>>2] = -1;
   $32 = $7;
   HEAP32[$32>>2] = -1;
   $33 = $8;
   HEAP32[$33>>2] = -1;
   $34 = $9;
   HEAP32[$34>>2] = -1;
   $35 = $12;
   HEAP32[$35>>2] = -1;
   $0 = 0;
   $250 = $0;
   STACKTOP = sp;return ($250|0);
   break;
  }
  case 2:  {
   $47 = HEAP32[$idata>>2]|0;
   $48 = HEAP32[$ilen>>2]|0;
   $49 = (_jpegl_decode_mem($img_dat,$lossyflag,$47,$48)|0);
   $ret = $49;
   $50 = ($49|0)!=(0);
   if ($50) {
    $51 = HEAP32[$idata>>2]|0;
    _free($51);
    $52 = $ret;
    $0 = $52;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $53 = HEAP32[$img_dat>>2]|0;
   $54 = (_get_IMG_DAT_image($ndata,$nlen,$w,$h,$d,$ppi,$53)|0);
   $ret = $54;
   $55 = ($54|0)!=(0);
   if ($55) {
    $56 = HEAP32[$idata>>2]|0;
    _free($56);
    $57 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($57,1);
    $58 = $ret;
    $0 = $58;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $intrlvflag = 0;
   $59 = HEAP32[$img_dat>>2]|0;
   $60 = (($59) + 20|0);
   $61 = HEAP32[$60>>2]|0;
   $n_cmpnts = $61;
   $62 = HEAP32[$d>>2]|0;
   $63 = ($62|0)==(24);
   if ($63) {
    $i = 0;
    while(1) {
     $64 = $i;
     $65 = $n_cmpnts;
     $66 = ($64|0)<($65|0);
     if (!($66)) {
      break;
     }
     $67 = $i;
     $68 = HEAP32[$img_dat>>2]|0;
     $69 = (($68) + 28|0);
     $70 = (($69) + ($67<<2)|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = $i;
     $73 = $10;
     $74 = (($73) + ($72<<2)|0);
     HEAP32[$74>>2] = $71;
     $75 = $i;
     $76 = HEAP32[$img_dat>>2]|0;
     $77 = (($76) + 44|0);
     $78 = (($77) + ($75<<2)|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = $i;
     $81 = $11;
     $82 = (($81) + ($80<<2)|0);
     HEAP32[$82>>2] = $79;
     $83 = $i;
     $84 = (($83) + 1)|0;
     $i = $84;
    }
   }
   $85 = HEAP32[$img_dat>>2]|0;
   _free_IMG_DAT($85,1);
   break;
  }
  case 6:  {
   $150 = HEAP32[$idata>>2]|0;
   $151 = HEAP32[$ilen>>2]|0;
   $152 = (_openjpeg2k_decode_mem($img_dat,$lossyflag,$150,$151)|0);
   $ret = $152;
   $153 = ($152|0)!=(0);
   if ($153) {
    $154 = HEAP32[$idata>>2]|0;
    _free($154);
    $155 = $ret;
    $0 = $155;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $156 = HEAP32[$img_dat>>2]|0;
   $157 = (_get_IMG_DAT_image($ndata,$nlen,$w,$h,$d,$ppi,$156)|0);
   $ret = $157;
   $158 = ($157|0)!=(0);
   if ($158) {
    $159 = HEAP32[$idata>>2]|0;
    _free($159);
    $160 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($160,1);
    $161 = $ret;
    $0 = $161;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $intrlvflag = 0;
   $162 = HEAP32[$img_dat>>2]|0;
   $163 = (($162) + 20|0);
   $164 = HEAP32[$163>>2]|0;
   $n_cmpnts = $164;
   $165 = HEAP32[$d>>2]|0;
   $166 = ($165|0)==(24);
   if ($166) {
    $i = 0;
    while(1) {
     $167 = $i;
     $168 = $n_cmpnts;
     $169 = ($167|0)<($168|0);
     if (!($169)) {
      break;
     }
     $170 = $i;
     $171 = HEAP32[$img_dat>>2]|0;
     $172 = (($171) + 28|0);
     $173 = (($172) + ($170<<2)|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = $i;
     $176 = $10;
     $177 = (($176) + ($175<<2)|0);
     HEAP32[$177>>2] = $174;
     $178 = $i;
     $179 = HEAP32[$img_dat>>2]|0;
     $180 = (($179) + 44|0);
     $181 = (($180) + ($178<<2)|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = $i;
     $184 = $11;
     $185 = (($184) + ($183<<2)|0);
     HEAP32[$185>>2] = $182;
     $186 = $i;
     $187 = (($186) + 1)|0;
     $i = $187;
    }
   }
   $188 = HEAP32[$img_dat>>2]|0;
   _free_IMG_DAT($188,1);
   break;
  }
  case 1:  {
   $36 = HEAP32[$idata>>2]|0;
   $37 = HEAP32[$ilen>>2]|0;
   $38 = (_wsq_decode_mem($ndata,$w,$h,$d,$ppi,$lossyflag,$36,$37)|0);
   $ret = $38;
   $39 = ($38|0)!=(0);
   if (!($39)) {
    $42 = HEAP32[$w>>2]|0;
    $43 = HEAP32[$h>>2]|0;
    $44 = Math_imul($42, $43)|0;
    HEAP32[$nlen>>2] = $44;
    $n_cmpnts = 1;
    $45 = $10;
    HEAP32[$45>>2] = 1;
    $46 = $11;
    HEAP32[$46>>2] = 1;
    break L9;
   }
   $40 = HEAP32[$idata>>2]|0;
   _free($40);
   $41 = $ret;
   $0 = $41;
   $250 = $0;
   STACKTOP = sp;return ($250|0);
   break;
  }
  case 4:  {
   $118 = HEAP32[$idata>>2]|0;
   $119 = HEAP32[$ilen>>2]|0;
   $120 = (_ihead_decode_mem($ndata,$w,$h,$d,$ppi,$lossyflag,$118,$119)|0);
   $ret = $120;
   $121 = ($120|0)!=(0);
   if ($121) {
    $122 = HEAP32[$idata>>2]|0;
    _free($122);
    $123 = $ret;
    $0 = $123;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $124 = HEAP32[$w>>2]|0;
   $125 = HEAP32[$h>>2]|0;
   $126 = HEAP32[$d>>2]|0;
   $127 = (_SizeFromDepth($124,$125,$126)|0);
   HEAP32[$nlen>>2] = $127;
   $128 = HEAP32[$d>>2]|0;
   $129 = ($128|0)==(1);
   do {
    if ($129) {
     label = 38;
    } else {
     $130 = HEAP32[$d>>2]|0;
     $131 = ($130|0)==(8);
     if ($131) {
      label = 38;
     } else {
      $132 = HEAP32[$d>>2]|0;
      $133 = ($132|0)==(24);
      if ($133) {
       $n_cmpnts = 3;
       $intrlvflag = 1;
       break;
      } else {
       $134 = HEAP32[_stderr>>2]|0;
       (_fprintf(($134|0),(28712|0),($vararg_buffer5|0))|0);
       $135 = HEAP32[_stderr>>2]|0;
       $136 = HEAP32[$d>>2]|0;
       HEAP32[$vararg_buffer7>>2] = $136;
       (_fprintf(($135|0),(28808|0),($vararg_buffer7|0))|0);
       $137 = HEAP32[_stderr>>2]|0;
       (_fprintf(($137|0),(28840|0),($vararg_buffer10|0))|0);
       $138 = HEAP32[$idata>>2]|0;
       _free($138);
       $0 = -2;
       $250 = $0;
       STACKTOP = sp;return ($250|0);
      }
     }
    }
   } while(0);
   if ((label|0) == 38) {
    $n_cmpnts = 1;
    $intrlvflag = 0;
   }
   $i = 0;
   while(1) {
    $139 = $i;
    $140 = $n_cmpnts;
    $141 = ($139|0)<($140|0);
    if (!($141)) {
     break;
    }
    $142 = $i;
    $143 = $10;
    $144 = (($143) + ($142<<2)|0);
    HEAP32[$144>>2] = 1;
    $145 = $i;
    $146 = $11;
    $147 = (($146) + ($145<<2)|0);
    HEAP32[$147>>2] = 1;
    $148 = $i;
    $149 = (($148) + 1)|0;
    $i = $149;
   }
   break;
  }
  case 3:  {
   $86 = HEAP32[$idata>>2]|0;
   $87 = HEAP32[$ilen>>2]|0;
   $88 = (_jpegb_decode_mem($ndata,$w,$h,$d,$ppi,$lossyflag,$86,$87)|0);
   $ret = $88;
   $89 = ($88|0)!=(0);
   if ($89) {
    $90 = HEAP32[$idata>>2]|0;
    _free($90);
    $91 = $ret;
    $0 = $91;
    $250 = $0;
    STACKTOP = sp;return ($250|0);
   }
   $92 = HEAP32[$d>>2]|0;
   $93 = ($92|0)==(8);
   do {
    if ($93) {
     $n_cmpnts = 1;
     $intrlvflag = 0;
    } else {
     $94 = HEAP32[$d>>2]|0;
     $95 = ($94|0)==(24);
     if ($95) {
      $n_cmpnts = 3;
      $intrlvflag = 1;
      break;
     } else {
      $96 = HEAP32[_stderr>>2]|0;
      (_fprintf(($96|0),(28712|0),($vararg_buffer|0))|0);
      $97 = HEAP32[_stderr>>2]|0;
      $98 = HEAP32[$d>>2]|0;
      HEAP32[$vararg_buffer1>>2] = $98;
      (_fprintf(($97|0),(28752|0),($vararg_buffer1|0))|0);
      $99 = HEAP32[_stderr>>2]|0;
      (_fprintf(($99|0),(28784|0),($vararg_buffer3|0))|0);
      $100 = HEAP32[$idata>>2]|0;
      _free($100);
      $0 = -2;
      $250 = $0;
      STACKTOP = sp;return ($250|0);
     }
    }
   } while(0);
   $101 = HEAP32[$w>>2]|0;
   $102 = HEAP32[$h>>2]|0;
   $103 = Math_imul($101, $102)|0;
   $104 = HEAP32[$d>>2]|0;
   $105 = $104 >> 3;
   $106 = Math_imul($103, $105)|0;
   HEAP32[$nlen>>2] = $106;
   $i = 0;
   while(1) {
    $107 = $i;
    $108 = $n_cmpnts;
    $109 = ($107|0)<($108|0);
    if (!($109)) {
     break;
    }
    $110 = $i;
    $111 = $10;
    $112 = (($111) + ($110<<2)|0);
    HEAP32[$112>>2] = 1;
    $113 = $i;
    $114 = $11;
    $115 = (($114) + ($113<<2)|0);
    HEAP32[$115>>2] = 1;
    $116 = $i;
    $117 = (($116) + 1)|0;
    $i = $117;
   }
   break;
  }
  default: {
   $228 = HEAP32[_stderr>>2]|0;
   (_fprintf(($228|0),(28712|0),($vararg_buffer12|0))|0);
   $229 = HEAP32[_stderr>>2]|0;
   $230 = HEAP32[$img_type>>2]|0;
   HEAP32[$vararg_buffer14>>2] = $230;
   (_fprintf(($229|0),(28864|0),($vararg_buffer14|0))|0);
   $0 = -3;
   $250 = $0;
   STACKTOP = sp;return ($250|0);
  }
  }
 } while(0);
 $231 = HEAP32[$idata>>2]|0;
 _free($231);
 $232 = HEAP32[$img_type>>2]|0;
 $233 = $2;
 HEAP32[$233>>2] = $232;
 $234 = HEAP32[$ndata>>2]|0;
 $235 = $3;
 HEAP32[$235>>2] = $234;
 $236 = HEAP32[$nlen>>2]|0;
 $237 = $4;
 HEAP32[$237>>2] = $236;
 $238 = HEAP32[$w>>2]|0;
 $239 = $5;
 HEAP32[$239>>2] = $238;
 $240 = HEAP32[$h>>2]|0;
 $241 = $6;
 HEAP32[$241>>2] = $240;
 $242 = HEAP32[$d>>2]|0;
 $243 = $7;
 HEAP32[$243>>2] = $242;
 $244 = HEAP32[$ppi>>2]|0;
 $245 = $8;
 HEAP32[$245>>2] = $244;
 $246 = $intrlvflag;
 $247 = $9;
 HEAP32[$247>>2] = $246;
 $248 = $n_cmpnts;
 $249 = $12;
 HEAP32[$249>>2] = $248;
 $0 = 0;
 $250 = $0;
 STACKTOP = sp;return ($250|0);
}
function _ihead_decode_mem($oodata,$ow,$oh,$od,$oppi,$lossyflag,$idata,$ilen) {
 $oodata = $oodata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppi = $oppi|0;
 $lossyflag = $lossyflag|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $compcode = 0, $complen = 0, $d = 0, $h = 0, $ihead = 0, $iptr = 0, $obytes = 0, $odata = 0, $olen = 0, $ppi = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $obytes = sp + 44|0;
 $1 = $oodata;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppi;
 $6 = $lossyflag;
 $7 = $idata;
 $8 = $ilen;
 $complen = 0;
 $9 = $7;
 $10 = (($9) + 8|0);
 $iptr = $10;
 $11 = $iptr;
 $ihead = $11;
 $12 = $iptr;
 $13 = (($12) + 288|0);
 $iptr = $13;
 $14 = $ihead;
 $15 = (_get_width($14)|0);
 $w = $15;
 $16 = $ihead;
 $17 = (_get_height($16)|0);
 $h = $17;
 $18 = $ihead;
 $19 = (_get_depth($18)|0);
 $d = $19;
 $20 = $ihead;
 $21 = (_get_density($20)|0);
 $ppi = $21;
 $22 = $ihead;
 $23 = (_get_compression($22)|0);
 $compcode = $23;
 $24 = $w;
 $25 = $h;
 $26 = $d;
 $27 = (_SizeFromDepth($24,$25,$26)|0);
 $olen = $27;
 $28 = $olen;
 $29 = (_malloc($28)|0);
 $odata = $29;
 $30 = $odata;
 $31 = ($30|0)==(0|0);
 if ($31) {
  $32 = HEAP32[_stderr>>2]|0;
  (_fprintf(($32|0),(28896|0),($vararg_buffer|0))|0);
  $0 = -2;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 $33 = $compcode;
 $34 = ($33|0)!=(0);
 if ($34) {
  $35 = $ihead;
  $36 = (_get_complen($35)|0);
  $complen = $36;
 }
 $37 = $compcode;
 if ((($37|0) == 5)) {
  $38 = $iptr;
  $39 = $complen;
  $40 = $odata;
  $41 = $olen;
  _rldecomp($38,$39,$40,$obytes,$41);
  $42 = $ihead;
  (_set_compression($42,0)|0);
  $43 = $ihead;
  (_set_complen($43,0)|0);
 } else if ((($37|0) == 0)) {
  $62 = $odata;
  $63 = $iptr;
  $64 = $olen;
  _memcpy(($62|0),($63|0),($64|0))|0;
 } else if ((($37|0) == 2)) {
  $44 = $ihead;
  $45 = (($44) + 170|0);
  $46 = HEAP8[$45>>0]|0;
  $47 = $46 << 24 >> 24;
  $48 = ($47|0)==(49);
  if ($48) {
   $49 = $iptr;
   $50 = $complen;
   _inv_bytes($49,$50);
   $51 = $ihead;
   $52 = (($51) + 170|0);
   HEAP8[$52>>0] = 48;
   $53 = $ihead;
   $54 = (($53) + 171|0);
   HEAP8[$54>>0] = 48;
  }
  $55 = $iptr;
  $56 = $complen;
  $57 = $w;
  $58 = $h;
  $59 = $odata;
  _grp4decomp($55,$56,$57,$58,$59,$obytes);
  $60 = $ihead;
  (_set_compression($60,0)|0);
  $61 = $ihead;
  (_set_complen($61,0)|0);
 } else {
  $65 = HEAP32[_stderr>>2]|0;
  (_fprintf(($65|0),(28944|0),($vararg_buffer1|0))|0);
  $66 = HEAP32[_stderr>>2]|0;
  $67 = $compcode;
  HEAP32[$vararg_buffer3>>2] = $67;
  (_fprintf(($66|0),(28976|0),($vararg_buffer3|0))|0);
  $0 = -3;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 $68 = $odata;
 $69 = $1;
 HEAP32[$69>>2] = $68;
 $70 = $w;
 $71 = $2;
 HEAP32[$71>>2] = $70;
 $72 = $h;
 $73 = $3;
 HEAP32[$73>>2] = $72;
 $74 = $d;
 $75 = $4;
 HEAP32[$75>>2] = $74;
 $76 = $ppi;
 $77 = $5;
 HEAP32[$77>>2] = $76;
 $78 = $6;
 HEAP32[$78>>2] = 0;
 $0 = 0;
 $79 = $0;
 STACKTOP = sp;return ($79|0);
}
function _image_type($img_type,$idata,$ilen) {
 $img_type = $img_type|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cbufptr = 0, $ebufptr = 0, $header = 0, $ihdr_size = 0, $marker = 0, $ret = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $marker = sp + 32|0;
 $cbufptr = sp + 8|0;
 $ihdr_size = sp + 48|0;
 $header = sp + 40|0;
 $1 = $img_type;
 $2 = $idata;
 $3 = $ilen;
 $4 = $2;
 HEAP32[$cbufptr>>2] = $4;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6)|0);
 $ebufptr = $7;
 $8 = $ebufptr;
 $9 = (_getc_ushort($marker,$cbufptr,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 do {
  if ($10) {
   $11 = $ret;
   $0 = $11;
  } else {
   $12 = HEAP16[$marker>>1]|0;
   $13 = $12&65535;
   $14 = ($13|0)==(65440);
   if ($14) {
    $15 = $1;
    HEAP32[$15>>2] = 1;
    $0 = 0;
    break;
   }
   $16 = HEAP16[$marker>>1]|0;
   $17 = $16&65535;
   $18 = ($17|0)==(65496);
   if ($18) {
    $19 = $1;
    $20 = $2;
    $21 = $3;
    $22 = (_jpeg_type($19,$20,$21)|0);
    $ret = $22;
    $23 = ($22|0)!=(0);
    if ($23) {
     $24 = $ret;
     $0 = $24;
     break;
    } else {
     $0 = 0;
     break;
    }
   }
   HEAP32[$vararg_buffer>>2] = 288;
   (_sprintf($ihdr_size,29008,$vararg_buffer)|0);
   $25 = $2;
   $26 = (_strlen(($ihdr_size|0))|0);
   $27 = (_strncmp($25,$ihdr_size,$26)|0);
   $28 = ($27|0)==(0);
   if ($28) {
    $29 = $1;
    HEAP32[$29>>2] = 4;
    $0 = 0;
    break;
   }
   $30 = $2;
   ;HEAP8[$header+0>>0]=HEAP8[$30+0>>0]|0;HEAP8[$header+1>>0]=HEAP8[$30+1>>0]|0;HEAP8[$header+2>>0]=HEAP8[$30+2>>0]|0;HEAP8[$header+3>>0]=HEAP8[$30+3>>0]|0;HEAP8[$header+4>>0]=HEAP8[$30+4>>0]|0;HEAP8[$header+5>>0]=HEAP8[$30+5>>0]|0;HEAP8[$header+6>>0]=HEAP8[$30+6>>0]|0;HEAP8[$header+7>>0]=HEAP8[$30+7>>0]|0;
   $31 = (_png_sig_cmp($header,0,8)|0);
   $32 = ($31|0)!=(0);
   if (!($32)) {
    $33 = $1;
    HEAP32[$33>>2] = 7;
    $0 = 0;
    break;
   }
   $34 = $2;
   $35 = $3;
   $36 = (_is_jp2($34,$35)|0);
   $37 = ($36|0)>(0);
   if ($37) {
    $38 = $1;
    HEAP32[$38>>2] = 6;
    $0 = 0;
    break;
   }
   $39 = $2;
   $40 = $3;
   $41 = (_is_ANSI_NIST($39,$40)|0);
   $ret = $41;
   $42 = $ret;
   $43 = ($42|0)<(0);
   if ($43) {
    $44 = $ret;
    $0 = $44;
    break;
   }
   $45 = $ret;
   $46 = ($45|0)==(1);
   if ($46) {
    $47 = $1;
    HEAP32[$47>>2] = 5;
    $0 = 0;
    break;
   } else {
    $48 = $1;
    HEAP32[$48>>2] = -1;
    $0 = 0;
    break;
   }
  }
 } while(0);
 $49 = $0;
 STACKTOP = sp;return ($49|0);
}
function _jpeg_type($img_type,$idata,$ilen) {
 $img_type = $img_type|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $cbufptr = 0, $ebufptr = 0, $marker = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $marker = sp + 40|0;
 $cbufptr = sp + 16|0;
 $1 = $img_type;
 $2 = $idata;
 $3 = $ilen;
 $4 = $2;
 HEAP32[$cbufptr>>2] = $4;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6)|0);
 $ebufptr = $7;
 $8 = $ebufptr;
 $9 = (_getc_marker_jpegl($marker,65496,$cbufptr,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 $12 = $ebufptr;
 $13 = (_getc_marker_jpegl($marker,65535,$cbufptr,$12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $ret;
  $0 = $15;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 while(1) {
  $16 = HEAP16[$marker>>1]|0;
  $17 = $16&65535;
  $18 = ($17|0)!=(65498);
  if (!($18)) {
   label = 17;
   break;
  }
  $19 = HEAP16[$marker>>1]|0;
  $20 = $19&65535;
  $21 = ($20|0)==(65475);
  if ($21) {
   label = 8;
   break;
  }
  $23 = HEAP16[$marker>>1]|0;
  $24 = $23&65535;
  $25 = ($24|0)==(65472);
  if ($25) {
   label = 10;
   break;
  }
  $27 = HEAP16[$marker>>1]|0;
  $28 = $ebufptr;
  $29 = (_getc_skip_marker_segment($27,$cbufptr,$28)|0);
  $ret = $29;
  $30 = ($29|0)!=(0);
  if ($30) {
   label = 13;
   break;
  }
  $32 = $ebufptr;
  $33 = (_getc_marker_jpegl($marker,65535,$cbufptr,$32)|0);
  $ret = $33;
  $34 = ($33|0)!=(0);
  if ($34) {
   label = 15;
   break;
  }
 }
 if ((label|0) == 8) {
  $22 = $1;
  HEAP32[$22>>2] = 2;
  $0 = 0;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 else if ((label|0) == 10) {
  $26 = $1;
  HEAP32[$26>>2] = 3;
  $0 = 0;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 else if ((label|0) == 13) {
  $31 = $ret;
  $0 = $31;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 else if ((label|0) == 15) {
  $35 = $ret;
  $0 = $35;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 else if ((label|0) == 17) {
  $36 = HEAP32[_stderr>>2]|0;
  (_fprintf(($36|0),(29016|0),($vararg_buffer|0))|0);
  $37 = HEAP32[_stderr>>2]|0;
  (_fprintf(($37|0),(29072|0),($vararg_buffer1|0))|0);
  $38 = $1;
  HEAP32[$38>>2] = -1;
  $0 = -2;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 return 0|0;
}
function _is_jp2($idata,$ilen) {
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $buf = 0, $nptr = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $buf = sp + 16|0;
 $0 = $idata;
 $1 = $ilen;
 $ret = 0;
 $2 = $0;
 $nptr = $2;
 $3 = $nptr;
 $4 = (($3) + 4|0);
 $nptr = $4;
 $5 = $nptr;
 ;HEAP8[$buf+0>>0]=HEAP8[$5+0>>0]|0;HEAP8[$buf+1>>0]=HEAP8[$5+1>>0]|0;HEAP8[$buf+2>>0]=HEAP8[$5+2>>0]|0;HEAP8[$buf+3>>0]=HEAP8[$5+3>>0]|0;
 $6 = (_memcmp($buf,29104,4)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  $8 = $ret;
  STACKTOP = sp;return ($8|0);
 }
 $ret = 1;
 $8 = $ret;
 STACKTOP = sp;return ($8|0);
}
function _SizeFromDepth($pixwidth,$pixheight,$depth) {
 $pixwidth = $pixwidth|0;
 $pixheight = $pixheight|0;
 $depth = $depth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $filesize = 0, $pixperbyte = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $pixwidth;
 $2 = $pixheight;
 $3 = $depth;
 $4 = $3;
 $5 = ($4|0)==(24);
 if ($5) {
  $6 = $1;
  $7 = $2;
  $8 = Math_imul($6, $7)|0;
  $9 = ($8*3)|0;
  $filesize = $9;
  $10 = $filesize;
  $0 = $10;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 } else {
  $11 = $3;
  $12 = (+_PixPerByte($11));
  $pixperbyte = $12;
  $13 = $1;
  $14 = (+($13|0));
  $15 = $pixperbyte;
  $16 = $14 / $15;
  $17 = $16;
  $18 = (+Math_ceil((+$17)));
  $19 = (~~(($18)));
  $20 = $2;
  $21 = Math_imul($19, $20)|0;
  $filesize = $21;
  $22 = $filesize;
  $0 = $22;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _PixPerByte($depth) {
 $depth = $depth|0;
 var $0 = 0, $1 = 0, $2 = 0.0, $pixperbyte = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $depth;
 $pixperbyte = 0.0;
 $1 = $0;
 switch ($1|0) {
 case 32:  {
  $pixperbyte = 0.25;
  break;
 }
 case 64:  {
  $pixperbyte = 0.125;
  break;
 }
 case 1:  {
  $pixperbyte = 8.0;
  break;
 }
 case 4:  {
  $pixperbyte = 2.0;
  break;
 }
 case 8:  {
  $pixperbyte = 1.0;
  break;
 }
 case 2:  {
  $pixperbyte = 4.0;
  break;
 }
 case 16:  {
  $pixperbyte = 0.5;
  break;
 }
 default: {
  _fatalerr(29112,29128,0);
 }
 }
 $2 = $pixperbyte;
 STACKTOP = sp;return (+$2);
}
function _intrlv2not_mem($oodata,$oolen,$idata,$width,$height,$depth,$hor_sampfctr,$vrt_sampfctr,$n_cmpnts) {
 $oodata = $oodata|0;
 $oolen = $oolen|0;
 $idata = $idata|0;
 $width = $width|0;
 $height = $height|0;
 $depth = $depth|0;
 $hor_sampfctr = $hor_sampfctr|0;
 $vrt_sampfctr = $vrt_sampfctr|0;
 $n_cmpnts = $n_cmpnts|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0.0, $64 = 0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $c = 0, $iptr = 0, $last_tile_height = 0, $last_tile_width = 0, $max_hor = 0, $max_vrt = 0, $n_hor_tiles = 0, $n_vrt_tiles = 0, $odata = 0, $offset = 0, $olen = 0;
 var $optrs = 0, $ox = 0, $oy = 0, $pad_height = 0, $pad_width = 0, $samp_height = 0, $samp_width = 0, $skip_vrt_tile_pad = 0, $tx = 0, $ty = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $optrs = sp + 208|0;
 $samp_width = sp + 168|0;
 $samp_height = sp + 152|0;
 $pad_width = sp + 136|0;
 $pad_height = sp + 120|0;
 $last_tile_width = sp + 104|0;
 $last_tile_height = sp + 88|0;
 $skip_vrt_tile_pad = sp + 72|0;
 $ox = sp + 32|0;
 $oy = sp + 16|0;
 $1 = $oodata;
 $2 = $oolen;
 $3 = $idata;
 $4 = $width;
 $5 = $height;
 $6 = $depth;
 $7 = $hor_sampfctr;
 $8 = $vrt_sampfctr;
 $9 = $n_cmpnts;
 $10 = $9;
 $11 = ($10|0)>(4);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  $13 = $9;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 4;
  (_fprintf(($12|0),(29184|0),($vararg_buffer|0))|0);
  $0 = -2;
  $426 = $0;
  STACKTOP = sp;return ($426|0);
 }
 $max_hor = -1;
 $max_vrt = -1;
 $c = 0;
 while(1) {
  $14 = $c;
  $15 = $9;
  $16 = ($14|0)<($15|0);
  if (!($16)) {
   break;
  }
  $17 = $c;
  $18 = $7;
  $19 = (($18) + ($17<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $max_hor;
  $22 = ($20|0)>($21|0);
  if ($22) {
   $23 = $c;
   $24 = $7;
   $25 = (($24) + ($23<<2)|0);
   $26 = HEAP32[$25>>2]|0;
   $max_hor = $26;
  }
  $27 = $c;
  $28 = $8;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $max_vrt;
  $32 = ($30|0)>($31|0);
  if ($32) {
   $33 = $c;
   $34 = $8;
   $35 = (($34) + ($33<<2)|0);
   $36 = HEAP32[$35>>2]|0;
   $max_vrt = $36;
  }
  $37 = $c;
  $38 = (($37) + 1)|0;
  $c = $38;
 }
 $olen = 0;
 $c = 0;
 while(1) {
  $39 = $c;
  $40 = $9;
  $41 = ($39|0)<($40|0);
  if (!($41)) {
   break;
  }
  $42 = $4;
  $43 = (+($42|0));
  $44 = $c;
  $45 = $7;
  $46 = (($45) + ($44<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (+($47|0));
  $49 = $max_hor;
  $50 = (+($49|0));
  $51 = $48 / $50;
  $52 = $43 * $51;
  $53 = (+Math_ceil((+$52)));
  $54 = (~~(($53)));
  $55 = $c;
  $56 = (($samp_width) + ($55<<2)|0);
  HEAP32[$56>>2] = $54;
  $57 = $5;
  $58 = (+($57|0));
  $59 = $c;
  $60 = $8;
  $61 = (($60) + ($59<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (+($62|0));
  $64 = $max_vrt;
  $65 = (+($64|0));
  $66 = $63 / $65;
  $67 = $58 * $66;
  $68 = (+Math_ceil((+$67)));
  $69 = (~~(($68)));
  $70 = $c;
  $71 = (($samp_height) + ($70<<2)|0);
  HEAP32[$71>>2] = $69;
  $72 = $c;
  $73 = (($samp_width) + ($72<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $c;
  $76 = (($samp_height) + ($75<<2)|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = Math_imul($74, $77)|0;
  $79 = $olen;
  $80 = (($79) + ($78))|0;
  $olen = $80;
  $81 = $c;
  $82 = (($81) + 1)|0;
  $c = $82;
 }
 $83 = $4;
 $84 = $5;
 $85 = $7;
 $86 = $8;
 $87 = $9;
 _compute_component_padding($pad_width,$pad_height,$83,$84,$samp_width,$samp_height,$85,$86,$87);
 $88 = $olen;
 $89 = $88;
 $90 = (_malloc($89)|0);
 $odata = $90;
 $91 = $odata;
 $92 = ($91|0)==(0|0);
 if ($92) {
  $93 = HEAP32[_stderr>>2]|0;
  (_fprintf(($93|0),(29248|0),($vararg_buffer2|0))|0);
  $0 = -3;
  $426 = $0;
  STACKTOP = sp;return ($426|0);
 }
 $offset = 0;
 $c = 0;
 while(1) {
  $94 = $c;
  $95 = $9;
  $96 = ($94|0)<($95|0);
  if (!($96)) {
   break;
  }
  $97 = $odata;
  $98 = $offset;
  $99 = (($97) + ($98)|0);
  $100 = $c;
  $101 = (($optrs) + ($100<<2)|0);
  HEAP32[$101>>2] = $99;
  $102 = $c;
  $103 = (($samp_width) + ($102<<2)|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = $c;
  $106 = (($samp_height) + ($105<<2)|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = Math_imul($104, $107)|0;
  $109 = $offset;
  $110 = (($109) + ($108))|0;
  $offset = $110;
  $111 = $c;
  $112 = (($oy) + ($111<<2)|0);
  HEAP32[$112>>2] = 0;
  $113 = $c;
  $114 = (($113) + 1)|0;
  $c = $114;
 }
 $115 = $3;
 $iptr = $115;
 $c = 0;
 while(1) {
  $116 = $c;
  $117 = $9;
  $118 = ($116|0)<($117|0);
  if (!($118)) {
   break;
  }
  $119 = $c;
  $120 = $7;
  $121 = (($120) + ($119<<2)|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = $c;
  $124 = (($pad_width) + ($123<<2)|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = (($122) - ($125))|0;
  $127 = $c;
  $128 = (($last_tile_width) + ($127<<2)|0);
  HEAP32[$128>>2] = $126;
  $129 = $c;
  $130 = $8;
  $131 = (($130) + ($129<<2)|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = $c;
  $134 = (($pad_height) + ($133<<2)|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (($132) - ($135))|0;
  $137 = $c;
  $138 = (($last_tile_height) + ($137<<2)|0);
  HEAP32[$138>>2] = $136;
  $139 = $c;
  $140 = (($pad_height) + ($139<<2)|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = $c;
  $143 = $7;
  $144 = (($143) + ($142<<2)|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = Math_imul($141, $145)|0;
  $147 = $c;
  $148 = (($skip_vrt_tile_pad) + ($147<<2)|0);
  HEAP32[$148>>2] = $146;
  $149 = $c;
  $150 = (($149) + 1)|0;
  $c = $150;
 }
 $151 = HEAP32[$samp_width>>2]|0;
 $152 = HEAP32[$pad_width>>2]|0;
 $153 = (($151) + ($152))|0;
 $154 = $7;
 $155 = HEAP32[$154>>2]|0;
 $156 = (($153|0) / ($155|0))&-1;
 $n_hor_tiles = $156;
 $157 = HEAP32[$samp_height>>2]|0;
 $158 = HEAP32[$pad_height>>2]|0;
 $159 = (($157) + ($158))|0;
 $160 = $8;
 $161 = HEAP32[$160>>2]|0;
 $162 = (($159|0) / ($161|0))&-1;
 $n_vrt_tiles = $162;
 $ty = 0;
 while(1) {
  $163 = $ty;
  $164 = $n_vrt_tiles;
  $165 = (($164) - 1)|0;
  $166 = ($163|0)<($165|0);
  if (!($166)) {
   break;
  }
  $c = 0;
  while(1) {
   $167 = $c;
   $168 = $9;
   $169 = ($167|0)<($168|0);
   if (!($169)) {
    break;
   }
   $170 = $c;
   $171 = (($ox) + ($170<<2)|0);
   HEAP32[$171>>2] = 0;
   $172 = $c;
   $173 = (($172) + 1)|0;
   $c = $173;
  }
  $tx = 0;
  while(1) {
   $174 = $tx;
   $175 = $n_hor_tiles;
   $176 = (($175) - 1)|0;
   $177 = ($174|0)<($176|0);
   if (!($177)) {
    break;
   }
   $c = 0;
   while(1) {
    $178 = $c;
    $179 = $9;
    $180 = ($178|0)<($179|0);
    if (!($180)) {
     break;
    }
    $b = 0;
    while(1) {
     $181 = $b;
     $182 = $c;
     $183 = $8;
     $184 = (($183) + ($182<<2)|0);
     $185 = HEAP32[$184>>2]|0;
     $186 = ($181|0)<($185|0);
     if (!($186)) {
      break;
     }
     $a = 0;
     while(1) {
      $187 = $a;
      $188 = $c;
      $189 = $7;
      $190 = (($189) + ($188<<2)|0);
      $191 = HEAP32[$190>>2]|0;
      $192 = ($187|0)<($191|0);
      if (!($192)) {
       break;
      }
      $193 = $iptr;
      $194 = (($193) + 1|0);
      $iptr = $194;
      $195 = HEAP8[$193>>0]|0;
      $196 = $c;
      $197 = (($optrs) + ($196<<2)|0);
      $198 = HEAP32[$197>>2]|0;
      $199 = $c;
      $200 = (($oy) + ($199<<2)|0);
      $201 = HEAP32[$200>>2]|0;
      $202 = $b;
      $203 = (($201) + ($202))|0;
      $204 = $c;
      $205 = (($samp_width) + ($204<<2)|0);
      $206 = HEAP32[$205>>2]|0;
      $207 = Math_imul($203, $206)|0;
      $208 = (($198) + ($207)|0);
      $209 = $c;
      $210 = (($ox) + ($209<<2)|0);
      $211 = HEAP32[$210>>2]|0;
      $212 = $a;
      $213 = (($211) + ($212))|0;
      $214 = (($208) + ($213)|0);
      HEAP8[$214>>0] = $195;
      $215 = $a;
      $216 = (($215) + 1)|0;
      $a = $216;
     }
     $217 = $b;
     $218 = (($217) + 1)|0;
     $b = $218;
    }
    $219 = $c;
    $220 = (($219) + 1)|0;
    $c = $220;
   }
   $c = 0;
   while(1) {
    $221 = $c;
    $222 = $9;
    $223 = ($221|0)<($222|0);
    if (!($223)) {
     break;
    }
    $224 = $c;
    $225 = $7;
    $226 = (($225) + ($224<<2)|0);
    $227 = HEAP32[$226>>2]|0;
    $228 = $c;
    $229 = (($ox) + ($228<<2)|0);
    $230 = HEAP32[$229>>2]|0;
    $231 = (($230) + ($227))|0;
    HEAP32[$229>>2] = $231;
    $232 = $c;
    $233 = (($232) + 1)|0;
    $c = $233;
   }
   $234 = $tx;
   $235 = (($234) + 1)|0;
   $tx = $235;
  }
  $c = 0;
  while(1) {
   $236 = $c;
   $237 = $9;
   $238 = ($236|0)<($237|0);
   if (!($238)) {
    break;
   }
   $b = 0;
   while(1) {
    $239 = $b;
    $240 = $c;
    $241 = $8;
    $242 = (($241) + ($240<<2)|0);
    $243 = HEAP32[$242>>2]|0;
    $244 = ($239|0)<($243|0);
    if (!($244)) {
     break;
    }
    $a = 0;
    while(1) {
     $245 = $a;
     $246 = $c;
     $247 = (($last_tile_width) + ($246<<2)|0);
     $248 = HEAP32[$247>>2]|0;
     $249 = ($245|0)<($248|0);
     if (!($249)) {
      break;
     }
     $250 = $iptr;
     $251 = (($250) + 1|0);
     $iptr = $251;
     $252 = HEAP8[$250>>0]|0;
     $253 = $c;
     $254 = (($optrs) + ($253<<2)|0);
     $255 = HEAP32[$254>>2]|0;
     $256 = $c;
     $257 = (($oy) + ($256<<2)|0);
     $258 = HEAP32[$257>>2]|0;
     $259 = $b;
     $260 = (($258) + ($259))|0;
     $261 = $c;
     $262 = (($samp_width) + ($261<<2)|0);
     $263 = HEAP32[$262>>2]|0;
     $264 = Math_imul($260, $263)|0;
     $265 = (($255) + ($264)|0);
     $266 = $c;
     $267 = (($ox) + ($266<<2)|0);
     $268 = HEAP32[$267>>2]|0;
     $269 = $a;
     $270 = (($268) + ($269))|0;
     $271 = (($265) + ($270)|0);
     HEAP8[$271>>0] = $252;
     $272 = $a;
     $273 = (($272) + 1)|0;
     $a = $273;
    }
    $274 = $c;
    $275 = (($pad_width) + ($274<<2)|0);
    $276 = HEAP32[$275>>2]|0;
    $277 = $iptr;
    $278 = (($277) + ($276)|0);
    $iptr = $278;
    $279 = $b;
    $280 = (($279) + 1)|0;
    $b = $280;
   }
   $281 = $c;
   $282 = (($281) + 1)|0;
   $c = $282;
  }
  $c = 0;
  while(1) {
   $283 = $c;
   $284 = $9;
   $285 = ($283|0)<($284|0);
   if (!($285)) {
    break;
   }
   $286 = $c;
   $287 = $8;
   $288 = (($287) + ($286<<2)|0);
   $289 = HEAP32[$288>>2]|0;
   $290 = $c;
   $291 = (($oy) + ($290<<2)|0);
   $292 = HEAP32[$291>>2]|0;
   $293 = (($292) + ($289))|0;
   HEAP32[$291>>2] = $293;
   $294 = $c;
   $295 = (($294) + 1)|0;
   $c = $295;
  }
  $296 = $ty;
  $297 = (($296) + 1)|0;
  $ty = $297;
 }
 $c = 0;
 while(1) {
  $298 = $c;
  $299 = $9;
  $300 = ($298|0)<($299|0);
  if (!($300)) {
   break;
  }
  $301 = $c;
  $302 = (($ox) + ($301<<2)|0);
  HEAP32[$302>>2] = 0;
  $303 = $c;
  $304 = (($303) + 1)|0;
  $c = $304;
 }
 $tx = 0;
 while(1) {
  $305 = $tx;
  $306 = $n_hor_tiles;
  $307 = (($306) - 1)|0;
  $308 = ($305|0)<($307|0);
  if (!($308)) {
   break;
  }
  $c = 0;
  while(1) {
   $309 = $c;
   $310 = $9;
   $311 = ($309|0)<($310|0);
   if (!($311)) {
    break;
   }
   $b = 0;
   while(1) {
    $312 = $b;
    $313 = $c;
    $314 = (($last_tile_height) + ($313<<2)|0);
    $315 = HEAP32[$314>>2]|0;
    $316 = ($312|0)<($315|0);
    if (!($316)) {
     break;
    }
    $a = 0;
    while(1) {
     $317 = $a;
     $318 = $c;
     $319 = $7;
     $320 = (($319) + ($318<<2)|0);
     $321 = HEAP32[$320>>2]|0;
     $322 = ($317|0)<($321|0);
     if (!($322)) {
      break;
     }
     $323 = $iptr;
     $324 = (($323) + 1|0);
     $iptr = $324;
     $325 = HEAP8[$323>>0]|0;
     $326 = $c;
     $327 = (($optrs) + ($326<<2)|0);
     $328 = HEAP32[$327>>2]|0;
     $329 = $c;
     $330 = (($oy) + ($329<<2)|0);
     $331 = HEAP32[$330>>2]|0;
     $332 = $b;
     $333 = (($331) + ($332))|0;
     $334 = $c;
     $335 = (($samp_width) + ($334<<2)|0);
     $336 = HEAP32[$335>>2]|0;
     $337 = Math_imul($333, $336)|0;
     $338 = (($328) + ($337)|0);
     $339 = $c;
     $340 = (($ox) + ($339<<2)|0);
     $341 = HEAP32[$340>>2]|0;
     $342 = $a;
     $343 = (($341) + ($342))|0;
     $344 = (($338) + ($343)|0);
     HEAP8[$344>>0] = $325;
     $345 = $a;
     $346 = (($345) + 1)|0;
     $a = $346;
    }
    $347 = $b;
    $348 = (($347) + 1)|0;
    $b = $348;
   }
   $349 = $c;
   $350 = (($skip_vrt_tile_pad) + ($349<<2)|0);
   $351 = HEAP32[$350>>2]|0;
   $352 = $iptr;
   $353 = (($352) + ($351)|0);
   $iptr = $353;
   $354 = $c;
   $355 = (($354) + 1)|0;
   $c = $355;
  }
  $c = 0;
  while(1) {
   $356 = $c;
   $357 = $9;
   $358 = ($356|0)<($357|0);
   if (!($358)) {
    break;
   }
   $359 = $c;
   $360 = $7;
   $361 = (($360) + ($359<<2)|0);
   $362 = HEAP32[$361>>2]|0;
   $363 = $c;
   $364 = (($ox) + ($363<<2)|0);
   $365 = HEAP32[$364>>2]|0;
   $366 = (($365) + ($362))|0;
   HEAP32[$364>>2] = $366;
   $367 = $c;
   $368 = (($367) + 1)|0;
   $c = $368;
  }
  $369 = $tx;
  $370 = (($369) + 1)|0;
  $tx = $370;
 }
 $c = 0;
 while(1) {
  $371 = $c;
  $372 = $9;
  $373 = ($371|0)<($372|0);
  if (!($373)) {
   break;
  }
  $b = 0;
  while(1) {
   $374 = $b;
   $375 = $c;
   $376 = (($last_tile_height) + ($375<<2)|0);
   $377 = HEAP32[$376>>2]|0;
   $378 = ($374|0)<($377|0);
   if (!($378)) {
    break;
   }
   $a = 0;
   while(1) {
    $379 = $a;
    $380 = $c;
    $381 = (($last_tile_width) + ($380<<2)|0);
    $382 = HEAP32[$381>>2]|0;
    $383 = ($379|0)<($382|0);
    if (!($383)) {
     break;
    }
    $384 = $iptr;
    $385 = (($384) + 1|0);
    $iptr = $385;
    $386 = HEAP8[$384>>0]|0;
    $387 = $c;
    $388 = (($optrs) + ($387<<2)|0);
    $389 = HEAP32[$388>>2]|0;
    $390 = $c;
    $391 = (($oy) + ($390<<2)|0);
    $392 = HEAP32[$391>>2]|0;
    $393 = $b;
    $394 = (($392) + ($393))|0;
    $395 = $c;
    $396 = (($samp_width) + ($395<<2)|0);
    $397 = HEAP32[$396>>2]|0;
    $398 = Math_imul($394, $397)|0;
    $399 = (($389) + ($398)|0);
    $400 = $c;
    $401 = (($ox) + ($400<<2)|0);
    $402 = HEAP32[$401>>2]|0;
    $403 = $a;
    $404 = (($402) + ($403))|0;
    $405 = (($399) + ($404)|0);
    HEAP8[$405>>0] = $386;
    $406 = $a;
    $407 = (($406) + 1)|0;
    $a = $407;
   }
   $408 = $c;
   $409 = (($pad_width) + ($408<<2)|0);
   $410 = HEAP32[$409>>2]|0;
   $411 = $iptr;
   $412 = (($411) + ($410)|0);
   $iptr = $412;
   $413 = $b;
   $414 = (($413) + 1)|0;
   $b = $414;
  }
  $415 = $c;
  $416 = (($skip_vrt_tile_pad) + ($415<<2)|0);
  $417 = HEAP32[$416>>2]|0;
  $418 = $iptr;
  $419 = (($418) + ($417)|0);
  $iptr = $419;
  $420 = $c;
  $421 = (($420) + 1)|0;
  $c = $421;
 }
 $422 = $odata;
 $423 = $1;
 HEAP32[$423>>2] = $422;
 $424 = $olen;
 $425 = $2;
 HEAP32[$425>>2] = $424;
 $0 = 0;
 $426 = $0;
 STACKTOP = sp;return ($426|0);
}
function _compute_component_padding($pad_width,$pad_height,$width,$height,$samp_width,$samp_height,$hor_sampfctr,$vrt_sampfctr,$n_cmpnts) {
 $pad_width = $pad_width|0;
 $pad_height = $pad_height|0;
 $width = $width|0;
 $height = $height|0;
 $samp_width = $samp_width|0;
 $samp_height = $samp_height|0;
 $hor_sampfctr = $hor_sampfctr|0;
 $vrt_sampfctr = $vrt_sampfctr|0;
 $n_cmpnts = $n_cmpnts|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0.0, $105 = 0.0, $106 = 0.0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0.0, $44 = 0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0.0, $63 = 0.0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0.0, $86 = 0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0.0, $c = 0, $max_hor = 0, $max_vrt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $pad_width;
 $1 = $pad_height;
 $2 = $width;
 $3 = $height;
 $4 = $samp_width;
 $5 = $samp_height;
 $6 = $hor_sampfctr;
 $7 = $vrt_sampfctr;
 $8 = $n_cmpnts;
 $max_hor = -1;
 $max_vrt = -1;
 $c = 0;
 while(1) {
  $9 = $c;
  $10 = $8;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = $c;
  $13 = $6;
  $14 = (($13) + ($12<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $max_hor;
  $17 = ($15|0)>($16|0);
  if ($17) {
   $18 = $c;
   $19 = $6;
   $20 = (($19) + ($18<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $max_hor = $21;
  }
  $22 = $c;
  $23 = $7;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $max_vrt;
  $27 = ($25|0)>($26|0);
  if ($27) {
   $28 = $c;
   $29 = $7;
   $30 = (($29) + ($28<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $max_vrt = $31;
  }
  $32 = $c;
  $33 = (($32) + 1)|0;
  $c = $33;
 }
 $c = 0;
 while(1) {
  $34 = $c;
  $35 = $8;
  $36 = ($34|0)<($35|0);
  if (!($36)) {
   break;
  }
  $37 = $2;
  $38 = (+($37|0));
  $39 = $c;
  $40 = $6;
  $41 = (($40) + ($39<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (+($42|0));
  $44 = $max_hor;
  $45 = (+($44|0));
  $46 = $43 / $45;
  $47 = $38 * $46;
  $48 = (+Math_ceil((+$47)));
  $49 = (~~(($48)));
  $50 = $c;
  $51 = $4;
  $52 = (($51) + ($50<<2)|0);
  HEAP32[$52>>2] = $49;
  $53 = $c;
  $54 = $4;
  $55 = (($54) + ($53<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (+($56|0));
  $58 = $c;
  $59 = $6;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (+($61|0));
  $63 = $57 / $62;
  $64 = (+Math_ceil((+$63)));
  $65 = (~~(($64)));
  $66 = $c;
  $67 = $6;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = Math_imul($65, $69)|0;
  $71 = $c;
  $72 = $4;
  $73 = (($72) + ($71<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($70) - ($74))|0;
  $76 = $c;
  $77 = $0;
  $78 = (($77) + ($76<<2)|0);
  HEAP32[$78>>2] = $75;
  $79 = $3;
  $80 = (+($79|0));
  $81 = $c;
  $82 = $7;
  $83 = (($82) + ($81<<2)|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = (+($84|0));
  $86 = $max_vrt;
  $87 = (+($86|0));
  $88 = $85 / $87;
  $89 = $80 * $88;
  $90 = (+Math_ceil((+$89)));
  $91 = (~~(($90)));
  $92 = $c;
  $93 = $5;
  $94 = (($93) + ($92<<2)|0);
  HEAP32[$94>>2] = $91;
  $95 = $c;
  $96 = $5;
  $97 = (($96) + ($95<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = (+($98|0));
  $100 = $c;
  $101 = $7;
  $102 = (($101) + ($100<<2)|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = (+($103|0));
  $105 = $99 / $104;
  $106 = (+Math_ceil((+$105)));
  $107 = (~~(($106)));
  $108 = $c;
  $109 = $7;
  $110 = (($109) + ($108<<2)|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = Math_imul($107, $111)|0;
  $113 = $c;
  $114 = $5;
  $115 = (($114) + ($113<<2)|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (($112) - ($116))|0;
  $118 = $c;
  $119 = $1;
  $120 = (($119) + ($118<<2)|0);
  HEAP32[$120>>2] = $117;
  $121 = $c;
  $122 = (($121) + 1)|0;
  $c = $122;
 }
 STACKTOP = sp;return;
}
function _not2intrlv_mem($oodata,$oolen,$idata,$width,$height,$depth,$hor_sampfctr,$vrt_sampfctr,$n_cmpnts) {
 $oodata = $oodata|0;
 $oolen = $oolen|0;
 $idata = $idata|0;
 $width = $width|0;
 $height = $height|0;
 $depth = $depth|0;
 $hor_sampfctr = $hor_sampfctr|0;
 $vrt_sampfctr = $vrt_sampfctr|0;
 $n_cmpnts = $n_cmpnts|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $c = 0, $iptrs = 0, $ix = 0, $iy = 0, $n_hor_tiles = 0, $n_vrt_tiles = 0, $new_samp_height = 0;
 var $new_samp_width = 0, $odata = 0, $offset = 0, $olen = 0, $optr = 0, $pad_height = 0, $pad_width = 0, $ret = 0, $samp_height = 0, $samp_width = 0, $tx = 0, $ty = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $iptrs = sp + 184|0;
 $olen = sp + 160|0;
 $samp_width = sp + 144|0;
 $samp_height = sp + 128|0;
 $pad_width = sp + 112|0;
 $pad_height = sp + 96|0;
 $new_samp_width = sp + 80|0;
 $new_samp_height = sp + 64|0;
 $ix = sp + 32|0;
 $iy = sp + 16|0;
 $1 = $oodata;
 $2 = $oolen;
 $3 = $idata;
 $4 = $width;
 $5 = $height;
 $6 = $depth;
 $7 = $hor_sampfctr;
 $8 = $vrt_sampfctr;
 $9 = $n_cmpnts;
 $10 = $9;
 $11 = ($10|0)>(4);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  $13 = $9;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 4;
  (_fprintf(($12|0),(29296|0),($vararg_buffer|0))|0);
  $0 = -2;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $14 = $4;
 $15 = $5;
 $16 = $7;
 $17 = $8;
 $18 = $9;
 _compute_component_padding($pad_width,$pad_height,$14,$15,$samp_width,$samp_height,$16,$17,$18);
 $19 = $3;
 $20 = $9;
 $21 = (_pad_component_planes($19,$olen,$new_samp_width,$new_samp_height,$samp_width,$samp_height,$pad_width,$pad_height,$20)|0);
 $ret = $21;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $ret;
  $0 = $23;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $24 = HEAP32[$new_samp_width>>2]|0;
 $25 = $7;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24|0) / ($26|0))&-1;
 $n_hor_tiles = $27;
 $28 = HEAP32[$new_samp_height>>2]|0;
 $29 = $8;
 $30 = HEAP32[$29>>2]|0;
 $31 = (($28|0) / ($30|0))&-1;
 $n_vrt_tiles = $31;
 $32 = HEAP32[$olen>>2]|0;
 $33 = $32;
 $34 = (_malloc($33)|0);
 $odata = $34;
 $35 = $odata;
 $36 = ($35|0)==(0|0);
 if ($36) {
  $37 = HEAP32[_stderr>>2]|0;
  (_fprintf(($37|0),(29360|0),($vararg_buffer2|0))|0);
  $0 = -3;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $offset = 0;
 $c = 0;
 while(1) {
  $38 = $c;
  $39 = $9;
  $40 = ($38|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = $3;
  $42 = $offset;
  $43 = (($41) + ($42)|0);
  $44 = $c;
  $45 = (($iptrs) + ($44<<2)|0);
  HEAP32[$45>>2] = $43;
  $46 = $c;
  $47 = (($new_samp_width) + ($46<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $c;
  $50 = (($new_samp_height) + ($49<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = Math_imul($48, $51)|0;
  $53 = $offset;
  $54 = (($53) + ($52))|0;
  $offset = $54;
  $55 = $c;
  $56 = (($iy) + ($55<<2)|0);
  HEAP32[$56>>2] = 0;
  $57 = $c;
  $58 = (($57) + 1)|0;
  $c = $58;
 }
 $59 = $odata;
 $optr = $59;
 $ty = 0;
 while(1) {
  $60 = $ty;
  $61 = $n_vrt_tiles;
  $62 = ($60|0)<($61|0);
  if (!($62)) {
   break;
  }
  $c = 0;
  while(1) {
   $63 = $c;
   $64 = $9;
   $65 = ($63|0)<($64|0);
   if (!($65)) {
    break;
   }
   $66 = $c;
   $67 = (($ix) + ($66<<2)|0);
   HEAP32[$67>>2] = 0;
   $68 = $c;
   $69 = (($68) + 1)|0;
   $c = $69;
  }
  $tx = 0;
  while(1) {
   $70 = $tx;
   $71 = $n_hor_tiles;
   $72 = ($70|0)<($71|0);
   if (!($72)) {
    break;
   }
   $c = 0;
   while(1) {
    $73 = $c;
    $74 = $9;
    $75 = ($73|0)<($74|0);
    if (!($75)) {
     break;
    }
    $b = 0;
    while(1) {
     $76 = $b;
     $77 = $c;
     $78 = $8;
     $79 = (($78) + ($77<<2)|0);
     $80 = HEAP32[$79>>2]|0;
     $81 = ($76|0)<($80|0);
     if (!($81)) {
      break;
     }
     $a = 0;
     while(1) {
      $82 = $a;
      $83 = $c;
      $84 = $7;
      $85 = (($84) + ($83<<2)|0);
      $86 = HEAP32[$85>>2]|0;
      $87 = ($82|0)<($86|0);
      if (!($87)) {
       break;
      }
      $88 = $c;
      $89 = (($iptrs) + ($88<<2)|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = $c;
      $92 = (($iy) + ($91<<2)|0);
      $93 = HEAP32[$92>>2]|0;
      $94 = $b;
      $95 = (($93) + ($94))|0;
      $96 = $c;
      $97 = (($new_samp_width) + ($96<<2)|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = Math_imul($95, $98)|0;
      $100 = (($90) + ($99)|0);
      $101 = $c;
      $102 = (($ix) + ($101<<2)|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = $a;
      $105 = (($103) + ($104))|0;
      $106 = (($100) + ($105)|0);
      $107 = HEAP8[$106>>0]|0;
      $108 = $optr;
      $109 = (($108) + 1|0);
      $optr = $109;
      HEAP8[$108>>0] = $107;
      $110 = $a;
      $111 = (($110) + 1)|0;
      $a = $111;
     }
     $112 = $b;
     $113 = (($112) + 1)|0;
     $b = $113;
    }
    $114 = $c;
    $115 = (($114) + 1)|0;
    $c = $115;
   }
   $c = 0;
   while(1) {
    $116 = $c;
    $117 = $9;
    $118 = ($116|0)<($117|0);
    if (!($118)) {
     break;
    }
    $119 = $c;
    $120 = $7;
    $121 = (($120) + ($119<<2)|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = $c;
    $124 = (($ix) + ($123<<2)|0);
    $125 = HEAP32[$124>>2]|0;
    $126 = (($125) + ($122))|0;
    HEAP32[$124>>2] = $126;
    $127 = $c;
    $128 = (($127) + 1)|0;
    $c = $128;
   }
   $129 = $tx;
   $130 = (($129) + 1)|0;
   $tx = $130;
  }
  $c = 0;
  while(1) {
   $131 = $c;
   $132 = $9;
   $133 = ($131|0)<($132|0);
   if (!($133)) {
    break;
   }
   $134 = $c;
   $135 = $8;
   $136 = (($135) + ($134<<2)|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = $c;
   $139 = (($iy) + ($138<<2)|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = (($140) + ($137))|0;
   HEAP32[$139>>2] = $141;
   $142 = $c;
   $143 = (($142) + 1)|0;
   $c = $143;
  }
  $144 = $ty;
  $145 = (($144) + 1)|0;
  $ty = $145;
 }
 $146 = $odata;
 $147 = $1;
 HEAP32[$147>>2] = $146;
 $148 = HEAP32[$olen>>2]|0;
 $149 = $2;
 HEAP32[$149>>2] = $148;
 $0 = 0;
 $150 = $0;
 STACKTOP = sp;return ($150|0);
}
function _pad_component_planes($idata,$onlen,$new_samp_width,$new_samp_height,$samp_width,$samp_height,$pad_width,$pad_height,$n_cmpnts) {
 $idata = $idata|0;
 $onlen = $onlen|0;
 $new_samp_width = $new_samp_width|0;
 $new_samp_height = $new_samp_height|0;
 $samp_width = $samp_width|0;
 $samp_height = $samp_height|0;
 $pad_width = $pad_width|0;
 $pad_height = $pad_height|0;
 $n_cmpnts = $n_cmpnts|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $d2ptr = 0;
 var $doptr = 0, $dpptr = 0, $dptr = 0, $i = 0, $j = 0, $nlen = 0, $padflag = 0, $padval = 0, $soptr = 0, $spptr = 0, $sptr = 0, $vararg_buffer = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp;
 $1 = $idata;
 $2 = $onlen;
 $3 = $new_samp_width;
 $4 = $new_samp_height;
 $5 = $samp_width;
 $6 = $samp_height;
 $7 = $pad_width;
 $8 = $pad_height;
 $9 = $n_cmpnts;
 $nlen = 0;
 $padflag = 0;
 $c = 0;
 while(1) {
  $10 = $c;
  $11 = $9;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = $c;
  $14 = $7;
  $15 = (($14) + ($13<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   label = 5;
  } else {
   $18 = $c;
   $19 = $8;
   $20 = (($19) + ($18<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0);
   if ($22) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $padflag = 1;
  }
  $23 = $c;
  $24 = $5;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $c;
  $28 = $7;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($26) + ($30))|0;
  $32 = $c;
  $33 = $3;
  $34 = (($33) + ($32<<2)|0);
  HEAP32[$34>>2] = $31;
  $35 = $c;
  $36 = $6;
  $37 = (($36) + ($35<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = $c;
  $40 = $8;
  $41 = (($40) + ($39<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($38) + ($42))|0;
  $44 = $c;
  $45 = $4;
  $46 = (($45) + ($44<<2)|0);
  HEAP32[$46>>2] = $43;
  $47 = $c;
  $48 = $3;
  $49 = (($48) + ($47<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = $c;
  $52 = $4;
  $53 = (($52) + ($51<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = Math_imul($50, $54)|0;
  $56 = $nlen;
  $57 = (($56) + ($55))|0;
  $nlen = $57;
  $58 = $c;
  $59 = (($58) + 1)|0;
  $c = $59;
 }
 $60 = $padflag;
 $61 = ($60|0)!=(0);
 if (!($61)) {
  $62 = $nlen;
  $63 = $2;
  HEAP32[$63>>2] = $62;
  $0 = 0;
  $337 = $0;
  STACKTOP = sp;return ($337|0);
 }
 $64 = $1;
 $65 = $nlen;
 $66 = $65;
 $67 = (_realloc($64,$66)|0);
 $68 = ($67|0)==(0|0);
 if ($68) {
  $69 = HEAP32[_stderr>>2]|0;
  (_fprintf(($69|0),(29408|0),($vararg_buffer|0))|0);
  $0 = -2;
  $337 = $0;
  STACKTOP = sp;return ($337|0);
 }
 $70 = $1;
 $soptr = $70;
 $71 = $1;
 $doptr = $71;
 $c = 0;
 while(1) {
  $72 = $c;
  $73 = $9;
  $74 = (($73) - 1)|0;
  $75 = ($72|0)<($74|0);
  if (!($75)) {
   break;
  }
  $76 = $c;
  $77 = $5;
  $78 = (($77) + ($76<<2)|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $c;
  $81 = $6;
  $82 = (($81) + ($80<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = Math_imul($79, $83)|0;
  $85 = $soptr;
  $86 = (($85) + ($84)|0);
  $soptr = $86;
  $87 = $c;
  $88 = $3;
  $89 = (($88) + ($87<<2)|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = $c;
  $92 = $4;
  $93 = (($92) + ($91<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = Math_imul($90, $94)|0;
  $96 = $doptr;
  $97 = (($96) + ($95)|0);
  $doptr = $97;
  $98 = $c;
  $99 = (($98) + 1)|0;
  $c = $99;
 }
 $100 = $9;
 $101 = (($100) - 1)|0;
 $c = $101;
 while(1) {
  $102 = $c;
  $103 = ($102|0)>=(0);
  if (!($103)) {
   break;
  }
  $104 = $soptr;
  $105 = $c;
  $106 = $6;
  $107 = (($106) + ($105<<2)|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = (($108) - 1)|0;
  $110 = $c;
  $111 = $5;
  $112 = (($111) + ($110<<2)|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = Math_imul($109, $113)|0;
  $115 = (($104) + ($114)|0);
  $sptr = $115;
  $116 = $doptr;
  $117 = $c;
  $118 = $6;
  $119 = (($118) + ($117<<2)|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = (($120) - 1)|0;
  $122 = $c;
  $123 = $3;
  $124 = (($123) + ($122<<2)|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = Math_imul($121, $125)|0;
  $127 = (($116) + ($126)|0);
  $dptr = $127;
  $128 = $sptr;
  $129 = $c;
  $130 = $5;
  $131 = (($130) + ($129<<2)|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = (($128) + ($132)|0);
  $134 = (($133) + -1|0);
  $spptr = $134;
  $135 = $dptr;
  $136 = $c;
  $137 = $5;
  $138 = (($137) + ($136<<2)|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($135) + ($139)|0);
  $141 = (($140) + -1|0);
  $dpptr = $141;
  $x = 0;
  while(1) {
   $142 = $x;
   $143 = $c;
   $144 = $5;
   $145 = (($144) + ($143<<2)|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = ($142|0)<($146|0);
   if (!($147)) {
    break;
   }
   $148 = $spptr;
   $149 = (($148) + -1|0);
   $spptr = $149;
   $150 = HEAP8[$148>>0]|0;
   $151 = $dpptr;
   $152 = (($151) + -1|0);
   $dpptr = $152;
   HEAP8[$151>>0] = $150;
   $153 = $x;
   $154 = (($153) + 1)|0;
   $x = $154;
  }
  $155 = $dptr;
  $156 = $c;
  $157 = $5;
  $158 = (($157) + ($156<<2)|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = (($155) + ($159)|0);
  $161 = (($160) + -1|0);
  $162 = HEAP8[$161>>0]|0;
  $163 = $162&255;
  $padval = $163;
  $i = 0;
  while(1) {
   $164 = $i;
   $165 = $c;
   $166 = $7;
   $167 = (($166) + ($165<<2)|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = ($164|0)<($168|0);
   if (!($169)) {
    break;
   }
   $170 = $padval;
   $171 = $170&255;
   $172 = $dptr;
   $173 = $c;
   $174 = $5;
   $175 = (($174) + ($173<<2)|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = (($172) + ($176)|0);
   $178 = $i;
   $179 = (($177) + ($178)|0);
   HEAP8[$179>>0] = $171;
   $180 = $i;
   $181 = (($180) + 1)|0;
   $i = $181;
  }
  $182 = $c;
  $183 = $8;
  $184 = (($183) + ($182<<2)|0);
  $185 = HEAP32[$184>>2]|0;
  $186 = ($185|0)>(0);
  if ($186) {
   $187 = $dptr;
   $188 = $c;
   $189 = $3;
   $190 = (($189) + ($188<<2)|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = (($187) + ($191)|0);
   $d2ptr = $192;
   $j = 0;
   while(1) {
    $193 = $j;
    $194 = $c;
    $195 = $8;
    $196 = (($195) + ($194<<2)|0);
    $197 = HEAP32[$196>>2]|0;
    $198 = ($193|0)<($197|0);
    if (!($198)) {
     break;
    }
    $199 = $d2ptr;
    $200 = $dptr;
    $201 = $c;
    $202 = $3;
    $203 = (($202) + ($201<<2)|0);
    $204 = HEAP32[$203>>2]|0;
    _memcpy(($199|0),($200|0),($204|0))|0;
    $205 = $c;
    $206 = $3;
    $207 = (($206) + ($205<<2)|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = $d2ptr;
    $210 = (($209) + ($208)|0);
    $d2ptr = $210;
    $211 = $j;
    $212 = (($211) + 1)|0;
    $j = $212;
   }
  }
  $213 = $c;
  $214 = $5;
  $215 = (($214) + ($213<<2)|0);
  $216 = HEAP32[$215>>2]|0;
  $217 = $sptr;
  $218 = (0 - ($216))|0;
  $219 = (($217) + ($218)|0);
  $sptr = $219;
  $220 = $c;
  $221 = $3;
  $222 = (($221) + ($220<<2)|0);
  $223 = HEAP32[$222>>2]|0;
  $224 = $dptr;
  $225 = (0 - ($223))|0;
  $226 = (($224) + ($225)|0);
  $dptr = $226;
  $y = 1;
  while(1) {
   $227 = $y;
   $228 = $c;
   $229 = $6;
   $230 = (($229) + ($228<<2)|0);
   $231 = HEAP32[$230>>2]|0;
   $232 = ($227|0)<($231|0);
   if (!($232)) {
    break;
   }
   $233 = $sptr;
   $234 = $c;
   $235 = $5;
   $236 = (($235) + ($234<<2)|0);
   $237 = HEAP32[$236>>2]|0;
   $238 = (($233) + ($237)|0);
   $239 = (($238) + -1|0);
   $spptr = $239;
   $240 = $dptr;
   $241 = $c;
   $242 = $5;
   $243 = (($242) + ($241<<2)|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (($240) + ($244)|0);
   $246 = (($245) + -1|0);
   $dpptr = $246;
   $x = 0;
   while(1) {
    $247 = $x;
    $248 = $c;
    $249 = $5;
    $250 = (($249) + ($248<<2)|0);
    $251 = HEAP32[$250>>2]|0;
    $252 = ($247|0)<($251|0);
    if (!($252)) {
     break;
    }
    $253 = $spptr;
    $254 = (($253) + -1|0);
    $spptr = $254;
    $255 = HEAP8[$253>>0]|0;
    $256 = $dpptr;
    $257 = (($256) + -1|0);
    $dpptr = $257;
    HEAP8[$256>>0] = $255;
    $258 = $x;
    $259 = (($258) + 1)|0;
    $x = $259;
   }
   $260 = $dptr;
   $261 = $c;
   $262 = $5;
   $263 = (($262) + ($261<<2)|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = (($260) + ($264)|0);
   $266 = (($265) + -1|0);
   $267 = HEAP8[$266>>0]|0;
   $268 = $267&255;
   $padval = $268;
   $i = 0;
   while(1) {
    $269 = $i;
    $270 = $c;
    $271 = $7;
    $272 = (($271) + ($270<<2)|0);
    $273 = HEAP32[$272>>2]|0;
    $274 = ($269|0)<($273|0);
    if (!($274)) {
     break;
    }
    $275 = $padval;
    $276 = $275&255;
    $277 = $dptr;
    $278 = $c;
    $279 = $5;
    $280 = (($279) + ($278<<2)|0);
    $281 = HEAP32[$280>>2]|0;
    $282 = (($277) + ($281)|0);
    $283 = $i;
    $284 = (($282) + ($283)|0);
    HEAP8[$284>>0] = $276;
    $285 = $i;
    $286 = (($285) + 1)|0;
    $i = $286;
   }
   $287 = $c;
   $288 = $5;
   $289 = (($288) + ($287<<2)|0);
   $290 = HEAP32[$289>>2]|0;
   $291 = $sptr;
   $292 = (0 - ($290))|0;
   $293 = (($291) + ($292)|0);
   $sptr = $293;
   $294 = $c;
   $295 = $3;
   $296 = (($295) + ($294<<2)|0);
   $297 = HEAP32[$296>>2]|0;
   $298 = $dptr;
   $299 = (0 - ($297))|0;
   $300 = (($298) + ($299)|0);
   $dptr = $300;
   $301 = $y;
   $302 = (($301) + 1)|0;
   $y = $302;
  }
  $303 = $c;
  $304 = ($303|0)>(0);
  if ($304) {
   $305 = $c;
   $306 = (($305) - 1)|0;
   $307 = $5;
   $308 = (($307) + ($306<<2)|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = $c;
   $311 = (($310) - 1)|0;
   $312 = $6;
   $313 = (($312) + ($311<<2)|0);
   $314 = HEAP32[$313>>2]|0;
   $315 = Math_imul($309, $314)|0;
   $316 = $soptr;
   $317 = (0 - ($315))|0;
   $318 = (($316) + ($317)|0);
   $soptr = $318;
   $319 = $c;
   $320 = (($319) - 1)|0;
   $321 = $3;
   $322 = (($321) + ($320<<2)|0);
   $323 = HEAP32[$322>>2]|0;
   $324 = $c;
   $325 = (($324) - 1)|0;
   $326 = $4;
   $327 = (($326) + ($325<<2)|0);
   $328 = HEAP32[$327>>2]|0;
   $329 = Math_imul($323, $328)|0;
   $330 = $doptr;
   $331 = (0 - ($329))|0;
   $332 = (($330) + ($331)|0);
   $doptr = $332;
  }
  $333 = $c;
  $334 = (($333) + -1)|0;
  $c = $334;
 }
 $335 = $nlen;
 $336 = $2;
 HEAP32[$336>>2] = $335;
 $0 = 0;
 $337 = $0;
 STACKTOP = sp;return ($337|0);
}
function _rldecomp($indata,$inbytes,$outdata,$outbytes,$outsize) {
 $indata = $indata|0;
 $inbytes = $inbytes|0;
 $outdata = $outdata|0;
 $outbytes = $outbytes|0;
 $outsize = $outsize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0, $incount = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $outptr = sp + 4|0;
 $0 = $indata;
 $1 = $inbytes;
 $2 = $outdata;
 $3 = $outbytes;
 $4 = $outsize;
 $5 = $2;
 HEAP32[$outptr>>2] = $5;
 $6 = $3;
 HEAP32[$6>>2] = 0;
 $7 = $0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $ch = $9;
 $incount = 1;
 $10 = $ch;
 $11 = $10&255;
 $12 = $4;
 $13 = $3;
 _RLL_putc($outptr,$11,$12,$13);
 while(1) {
  $14 = $incount;
  $15 = $1;
  $16 = ($14|0)<($15|0);
  if (!($16)) {
   break;
  }
  $17 = $0;
  $18 = (($17) + 1|0);
  $0 = $18;
  $19 = $0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $ch = $21;
  $22 = $incount;
  $23 = (($22) + 1)|0;
  $incount = $23;
  $24 = $ch;
  $25 = $24&255;
  $26 = $4;
  $27 = $3;
  _RLL_putc($outptr,$25,$26,$27);
 }
 STACKTOP = sp;return;
}
function _RLL_putc($outptr,$code,$outsize,$out_count) {
 $outptr = $outptr|0;
 $code = $code|0;
 $outsize = $outsize|0;
 $out_count = $out_count|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $tmpbytes = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $outptr;
 $1 = $code;
 $2 = $outsize;
 $3 = $out_count;
 $4 = HEAP32[29456>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $41 = $1;
  $42 = $41&255;
  $43 = ($42|0)==(144);
  if ($43) {
   HEAP32[29456>>2] = 1;
   STACKTOP = sp;return;
  }
  $44 = $3;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1)|0;
  $tmpbytes = $46;
  $47 = $2;
  $48 = ($46|0)>($47|0);
  if ($48) {
   $49 = HEAP32[_stderr>>2]|0;
   (_fprintf(($49|0),(29464|0),($vararg_buffer3|0))|0);
   _exit(-1);
   // unreachable;
  }
  $50 = $1;
  $51 = $0;
  $52 = HEAP32[$51>>2]|0;
  HEAP8[$52>>0] = $50;
  $53 = $0;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 1|0);
  HEAP32[$53>>2] = $55;
  $56 = $3;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$56>>2] = $58;
  $59 = $1;
  $60 = $59&255;
  HEAP32[29504>>2] = $60;
  STACKTOP = sp;return;
 }
 HEAP32[29456>>2] = 0;
 $6 = $1;
 $7 = ($6<<24>>24)!=(0);
 do {
  if ($7) {
   while(1) {
    $8 = $1;
    $9 = (($8) + -1)<<24>>24;
    $1 = $9;
    $10 = ($9<<24>>24)!=(0);
    if (!($10)) {
     label = 8;
     break;
    }
    $11 = $3;
    $12 = HEAP32[$11>>2]|0;
    $13 = (($12) + 1)|0;
    $tmpbytes = $13;
    $14 = $2;
    $15 = ($13|0)>($14|0);
    if ($15) {
     label = 6;
     break;
    }
    $17 = HEAP32[29504>>2]|0;
    $18 = $17&255;
    $19 = $0;
    $20 = HEAP32[$19>>2]|0;
    HEAP8[$20>>0] = $18;
    $21 = $0;
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + 1|0);
    HEAP32[$21>>2] = $23;
    $24 = $3;
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[$24>>2] = $26;
   }
   if ((label|0) == 6) {
    $16 = HEAP32[_stderr>>2]|0;
    (_fprintf(($16|0),(29464|0),($vararg_buffer|0))|0);
    _exit(-1);
    // unreachable;
   }
   else if ((label|0) == 8) {
    break;
   }
  } else {
   $27 = $3;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 1)|0;
   $tmpbytes = $29;
   $30 = $2;
   $31 = ($29|0)>($30|0);
   if ($31) {
    $32 = HEAP32[_stderr>>2]|0;
    (_fprintf(($32|0),(29464|0),($vararg_buffer1|0))|0);
    _exit(-1);
    // unreachable;
   } else {
    $33 = $0;
    $34 = HEAP32[$33>>2]|0;
    HEAP8[$34>>0] = -112;
    $35 = $0;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($36) + 1|0);
    HEAP32[$35>>2] = $37;
    $38 = $3;
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + 1)|0;
    HEAP32[$38>>2] = $40;
    break;
   }
  }
 } while(0);
 STACKTOP = sp;return;
}
function _grp4decomp($indata,$inbytes,$width,$height,$outdata,$outbytes) {
 $indata = $indata|0;
 $inbytes = $inbytes|0;
 $width = $width|0;
 $height = $height|0;
 $outdata = $outdata|0;
 $outbytes = $outbytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $compressed = 0, $decompressed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $compressed = sp + 16|0;
 $decompressed = sp;
 $0 = $indata;
 $1 = $inbytes;
 $2 = $width;
 $3 = $height;
 $4 = $outdata;
 $5 = $outbytes;
 $6 = $2;
 $7 = (($compressed) + 4|0);
 HEAP32[$7>>2] = $6;
 $8 = $3;
 $9 = (($compressed) + 8|0);
 HEAP32[$9>>2] = $8;
 $10 = $1;
 $11 = (($compressed) + 12|0);
 HEAP32[$11>>2] = $10;
 $12 = $0;
 HEAP32[$compressed>>2] = $12;
 HEAP32[29512>>2] = 0;
 HEAP32[29520>>2] = 1;
 HEAP32[29528>>2] = 1;
 _read_compressed_file_into_memory($compressed);
 $13 = $4;
 HEAP32[$decompressed>>2] = $13;
 _control_decompression($compressed,$decompressed);
 $14 = (($decompressed) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 >> 3;
 $17 = (($decompressed) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = Math_imul($16, $18)|0;
 $20 = $5;
 HEAP32[$20>>2] = $19;
 STACKTOP = sp;return;
}
function _read_compressed_file_into_memory($compressed) {
 $compressed = $compressed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $compressed;
 $1 = HEAP32[29512>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  $3 = $0;
  $4 = (($3) + 12|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = (_calloc($5,1)|0);
  $7 = $0;
  HEAP32[$7>>2] = $6;
  $8 = ($6|0)==(0|0);
  if ($8) {
   (_printf((29552|0),($vararg_buffer|0))|0);
   _exit(1);
   // unreachable;
  }
  STACKTOP = sp;return;
 } else {
  $9 = $0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   (_printf((29608|0),($vararg_buffer1|0))|0);
   _exit(1);
   // unreachable;
  }
  STACKTOP = sp;return;
 }
}
function _control_decompression($compressed,$decompressed) {
 $compressed = $compressed|0;
 $decompressed = $decompressed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $params = 0, $sole_parameters = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $sole_parameters = sp + 8|0;
 $0 = $compressed;
 $1 = $decompressed;
 $params = $sole_parameters;
 $2 = $0;
 $3 = $1;
 $4 = $params;
 _prepare_to_decompress($2,$3,$4);
 while(1) {
  $5 = $params;
  $6 = (_decompress_line($5)|0);
  $7 = ($6|0)!=(6);
  if (!($7)) {
   break;
  }
  $8 = $params;
  _prepare_to_decompress_next_line($8);
 }
 $9 = $params;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 _free($11);
 $12 = $params;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 _free($14);
 $15 = HEAP32[29536>>2]|0;
 _free($15);
 $16 = HEAP32[29544>>2]|0;
 _free($16);
 STACKTOP = sp;return;
}
function _prepare_to_decompress($compressed,$decompressed,$params) {
 $compressed = $compressed|0;
 $decompressed = $decompressed|0;
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = $compressed;
 $1 = $decompressed;
 $2 = $params;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $2;
 $7 = (($6) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = $0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $1;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = $0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $1;
 $17 = (($16) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = $2;
 _set_up_first_line_d($18);
 $19 = $0;
 $20 = HEAP32[$19>>2]|0;
 _prepare_to_read_bits($20);
 $21 = HEAP32[29512>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $0;
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $0;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = Math_imul($25, $28)|0;
  $30 = (($29|0) / 8)&-1;
  $31 = (_calloc($30,1)|0);
  $32 = $1;
  HEAP32[$32>>2] = $31;
  $36 = $1;
  $37 = HEAP32[$36>>2]|0;
  $38 = $0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40|0) / 8)&-1;
  _prepare_to_write_bits_d($37,$41);
  STACKTOP = sp;return;
 }
 $33 = $1;
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  (_printf((29648|0),($vararg_buffer|0))|0);
  _exit(1);
  // unreachable;
 }
 $36 = $1;
 $37 = HEAP32[$36>>2]|0;
 $38 = $0;
 $39 = (($38) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40|0) / 8)&-1;
 _prepare_to_write_bits_d($37,$41);
 STACKTOP = sp;return;
}
function _decompress_line($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $params;
 HEAP32[29752>>2] = 1;
 HEAP32[((29760 + 4|0))>>2] = 0;
 HEAP32[29760>>2] = 0;
 L1: while(1) {
  $2 = (_get_mode()|0);
  $mode = $2;
  $3 = $mode;
  switch ($3|0) {
  case -3: case 3: case -2: case 2: case -1: case 1: case 0:  {
   $4 = $1;
   $5 = $mode;
   _vertical_mode_d($4,$5);
   label = 8;
   break;
  }
  case 6:  {
   label = 6;
   break L1;
   break;
  }
  case 5:  {
   $6 = $1;
   _horizontal_mode_d($6);
   label = 8;
   break;
  }
  case 4:  {
   $7 = $1;
   _pass_mode_d($7);
   break;
  }
  default: {
   _crash_d();
   label = 8;
  }
  }
  if ((label|0) == 8) {
   label = 0;
  }
  $8 = HEAP32[((29760 + 4|0))>>2]|0;
  $9 = $1;
  $10 = (($9) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 6) {
  $0 = 6;
  $13 = $0;
  STACKTOP = sp;return ($13|0);
 }
 else if ((label|0) == 10) {
  $0 = 0;
  $13 = $0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _prepare_to_decompress_next_line($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 _set_up_first_and_last_changing_elements_d($1);
 $2 = $0;
 _swap_the_reference_and_coding_lines($2);
 $3 = $0;
 HEAP32[$3>>2] = 0;
 STACKTOP = sp;return;
}
function _set_up_first_line_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 25)|0;
 $5 = $4<<2;
 $6 = (_malloc($5)|0);
 $7 = $0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 25)|0;
 $13 = $12<<2;
 $14 = (_malloc($13)|0);
 $15 = $0;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = $0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$19>>2] = -1;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 4|0);
 HEAP32[$26>>2] = $22;
 $27 = $0;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 8|0);
 HEAP32[$33>>2] = $29;
 $34 = $0;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = $0;
 $38 = (($37) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + 12|0);
 HEAP32[$40>>2] = $36;
 $41 = $0;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$43>>2] = -1;
 $44 = $0;
 HEAP32[$44>>2] = 0;
 STACKTOP = sp;return;
}
function _prepare_to_read_bits($input_pointer) {
 $input_pointer = $input_pointer|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $input_pointer;
 $1 = $0;
 HEAP32[29824>>2] = $1;
 STACKTOP = sp;return;
}
function _prepare_to_write_bits_d($output_pointer,$bytes_per_line) {
 $output_pointer = $output_pointer|0;
 $bytes_per_line = $bytes_per_line|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $output_pointer;
 $1 = $bytes_per_line;
 $2 = $0;
 HEAP32[29784>>2] = $2;
 $3 = $1;
 $4 = (_calloc($3,1)|0);
 HEAP32[29536>>2] = $4;
 $5 = $1;
 $6 = (_calloc($5,1)|0);
 HEAP32[29544>>2] = $6;
 $7 = HEAP32[29544>>2]|0;
 $8 = $1;
 _memset(($7|0),-1,($8|0))|0;
 STACKTOP = sp;return;
}
function _set_up_first_and_last_changing_elements_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = -1;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$10>>2] = $12;
 $13 = (($9) + ($12<<2)|0);
 HEAP32[$13>>2] = $6;
 $14 = $0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $0;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$20>>2] = $22;
 $23 = (($19) + ($22<<2)|0);
 HEAP32[$23>>2] = $16;
 $24 = $0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $0;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 1)|0;
 HEAP32[$30>>2] = $32;
 $33 = (($29) + ($32<<2)|0);
 HEAP32[$33>>2] = $26;
 STACKTOP = sp;return;
}
function _swap_the_reference_and_coding_lines($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $temp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $temp = $3;
 $4 = $0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = $temp;
 $10 = $0;
 $11 = (($10) + 12|0);
 HEAP32[$11>>2] = $9;
 STACKTOP = sp;return;
}
function _crash_d() {
 var $0 = 0, $crash_program = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $crash_program = 0;
 $0 = $crash_program;
 (_fprintf(($0|0),(29696|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return;
}
function _get_mode() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = (_read_bit()|0);
 $2 = ($1|0)==(1);
 do {
  if ($2) {
   $0 = 0;
  } else {
   $3 = (_read_bit()|0);
   $4 = ($3|0)==(1);
   if ($4) {
    $5 = (_read_bit()|0);
    $6 = ($5|0)==(1);
    if ($6) {
     $0 = 1;
     break;
    } else {
     $0 = -1;
     break;
    }
   }
   $7 = (_read_bit()|0);
   $8 = ($7|0)==(1);
   if ($8) {
    $0 = 5;
    break;
   }
   $9 = (_read_bit()|0);
   $10 = ($9|0)==(1);
   if ($10) {
    $0 = 4;
    break;
   }
   $11 = (_read_bit()|0);
   $12 = ($11|0)==(1);
   if ($12) {
    $13 = (_read_bit()|0);
    $14 = ($13|0)==(1);
    if ($14) {
     $0 = 2;
     break;
    } else {
     $0 = -2;
     break;
    }
   }
   $15 = (_read_bit()|0);
   $16 = ($15|0)==(1);
   if ($16) {
    $17 = (_read_bit()|0);
    $18 = ($17|0)==(1);
    if ($18) {
     $0 = 3;
     break;
    } else {
     $0 = -3;
     break;
    }
   }
   $i = 0;
   while(1) {
    $19 = $i;
    $20 = ($19|0)<(5);
    if (!($20)) {
     break;
    }
    $21 = (_read_bit()|0);
    $22 = ($21|0)!=(0);
    if ($22) {
     _crash_d();
    }
    $23 = $i;
    $24 = (($23) + 1)|0;
    $i = $24;
   }
   $25 = (_read_bit()|0);
   $26 = ($25|0)!=(1);
   if ($26) {
    _crash_d();
   }
   $i = 0;
   while(1) {
    $27 = $i;
    $28 = ($27|0)<(11);
    if (!($28)) {
     break;
    }
    $29 = (_read_bit()|0);
    $30 = ($29|0)!=(0);
    if ($30) {
     _crash_d();
    }
    $31 = $i;
    $32 = (($31) + 1)|0;
    $i = $32;
   }
   $33 = (_read_bit()|0);
   $34 = ($33|0)!=(1);
   if ($34) {
    _crash_d();
   }
   $0 = 6;
  }
 } while(0);
 $35 = $0;
 STACKTOP = sp;return ($35|0);
}
function _vertical_mode_d($params,$offset) {
 $params = $params|0;
 $offset = $offset|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $run_length = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $offset;
 $2 = $0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[29752>>2]|0;
 $6 = (($4) + ($5<<2)|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $1;
 $9 = (($7) + ($8))|0;
 HEAP32[((29776 + 4|0))>>2] = $9;
 $10 = HEAP32[((29776 + 4|0))>>2]|0;
 $11 = HEAP32[((29760 + 4|0))>>2]|0;
 $12 = (($10) - ($11))|0;
 $run_length = $12;
 $13 = $run_length;
 $14 = HEAP32[29760>>2]|0;
 _write_bits_d($13,$14);
 $15 = HEAP32[((29776 + 4|0))>>2]|0;
 HEAP32[((29760 + 4|0))>>2] = $15;
 $16 = HEAP32[29760>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = $17 ^ 1;
 $19 = $18&1;
 HEAP32[29760>>2] = $19;
 $20 = HEAP32[((29760 + 4|0))>>2]|0;
 $21 = $0;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$24>>2] = $26;
 $27 = (($23) + ($26<<2)|0);
 HEAP32[$27>>2] = $20;
 $28 = $1;
 $29 = ($28|0)==(-1);
 if (!($29)) {
  $30 = $1;
  $31 = ($30|0)==(0);
  if (!($31)) {
   $44 = $1;
   $45 = ($44|0)==(1);
   if (!($45)) {
    $46 = $1;
    $47 = ($46|0)==(2);
    if (!($47)) {
     $70 = $1;
     $71 = ($70|0)==(-2);
     if ($71) {
      label = 14;
     } else {
      $72 = $1;
      $73 = ($72|0)==(-3);
      if ($73) {
       label = 14;
      }
     }
     do {
      if ((label|0) == 14) {
       $74 = $0;
       $75 = (($74) + 8|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = HEAP32[29752>>2]|0;
       $78 = (($76) + ($77<<2)|0);
       $79 = (($78) + -4|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = HEAP32[((29760 + 4|0))>>2]|0;
       $82 = ($80|0)>($81|0);
       if ($82) {
        $83 = HEAP32[29752>>2]|0;
        $84 = (($83) + -1)|0;
        HEAP32[29752>>2] = $84;
        break;
       }
       $85 = $0;
       $86 = (($85) + 8|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = HEAP32[29752>>2]|0;
       $89 = (($87) + ($88<<2)|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = $0;
       $92 = (($91) + 4|0);
       $93 = HEAP32[$92>>2]|0;
       $94 = ($90|0)!=($93|0);
       if ($94) {
        $95 = HEAP32[29752>>2]|0;
        $96 = (($95) + 1)|0;
        HEAP32[29752>>2] = $96;
       }
       STACKTOP = sp;return;
      }
     } while(0);
     $97 = $1;
     $98 = ($97|0)==(3);
     if (!($98)) {
      STACKTOP = sp;return;
     }
     $99 = HEAP32[29752>>2]|0;
     $100 = (($99) + 1)|0;
     HEAP32[29752>>2] = $100;
     while(1) {
      $101 = $0;
      $102 = (($101) + 8|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = HEAP32[29752>>2]|0;
      $105 = (($103) + ($104<<2)|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = HEAP32[((29760 + 4|0))>>2]|0;
      $108 = ($106|0)<=($107|0);
      if ($108) {
       $109 = $0;
       $110 = (($109) + 8|0);
       $111 = HEAP32[$110>>2]|0;
       $112 = HEAP32[29752>>2]|0;
       $113 = (($111) + ($112<<2)|0);
       $114 = HEAP32[$113>>2]|0;
       $115 = $0;
       $116 = (($115) + 4|0);
       $117 = HEAP32[$116>>2]|0;
       $118 = ($114|0)!=($117|0);
       $121 = $118;
      } else {
       $121 = 0;
      }
      if (!($121)) {
       break;
      }
      $119 = HEAP32[29752>>2]|0;
      $120 = (($119) + 2)|0;
      HEAP32[29752>>2] = $120;
     }
     STACKTOP = sp;return;
    }
   }
   $48 = HEAP32[29752>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[29752>>2] = $49;
   $50 = $0;
   $51 = (($50) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP32[29752>>2]|0;
   $54 = (($52) + ($53<<2)|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[((29760 + 4|0))>>2]|0;
   $57 = ($55|0)<=($56|0);
   if ($57) {
    $58 = $0;
    $59 = (($58) + 8|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[29752>>2]|0;
    $62 = (($60) + ($61<<2)|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = $0;
    $65 = (($64) + 4|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = ($63|0)!=($66|0);
    if ($67) {
     $68 = HEAP32[29752>>2]|0;
     $69 = (($68) + 2)|0;
     HEAP32[29752>>2] = $69;
    }
   }
   STACKTOP = sp;return;
  }
 }
 $32 = $0;
 $33 = (($32) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[29752>>2]|0;
 $36 = (($34) + ($35<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $0;
 $39 = (($38) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ($37|0)!=($40|0);
 if ($41) {
  $42 = HEAP32[29752>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[29752>>2] = $43;
 }
 STACKTOP = sp;return;
}
function _horizontal_mode_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $length = 0, $total_length = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $total_length = 0;
 while(1) {
  $1 = HEAP32[29760>>2]|0;
  $2 = (_find_run_length_code($1)|0);
  $length = $2;
  $3 = $length;
  $4 = $total_length;
  $5 = (($4) + ($3))|0;
  $total_length = $5;
  $6 = $length;
  $7 = ($6|0)>(63);
  if (!($7)) {
   break;
  }
 }
 $8 = $total_length;
 $9 = HEAP32[29760>>2]|0;
 _write_bits_d($8,$9);
 $10 = $total_length;
 $11 = HEAP32[((29760 + 4|0))>>2]|0;
 $12 = (($11) + ($10))|0;
 HEAP32[((29760 + 4|0))>>2] = $12;
 $13 = HEAP32[29760>>2]|0;
 $14 = ($13|0)!=(0);
 $15 = $14 ^ 1;
 $16 = $15&1;
 HEAP32[29760>>2] = $16;
 $17 = HEAP32[((29760 + 4|0))>>2]|0;
 $18 = $0;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $0;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 1)|0;
 HEAP32[$21>>2] = $23;
 $24 = (($20) + ($23<<2)|0);
 HEAP32[$24>>2] = $17;
 $total_length = 0;
 while(1) {
  $25 = HEAP32[29760>>2]|0;
  $26 = (_find_run_length_code($25)|0);
  $length = $26;
  $27 = $length;
  $28 = $total_length;
  $29 = (($28) + ($27))|0;
  $total_length = $29;
  $30 = $length;
  $31 = ($30|0)>(63);
  if (!($31)) {
   break;
  }
 }
 $32 = $total_length;
 $33 = HEAP32[29760>>2]|0;
 _write_bits_d($32,$33);
 $34 = $total_length;
 $35 = HEAP32[((29760 + 4|0))>>2]|0;
 $36 = (($35) + ($34))|0;
 HEAP32[((29760 + 4|0))>>2] = $36;
 $37 = HEAP32[29760>>2]|0;
 $38 = ($37|0)!=(0);
 $39 = $38 ^ 1;
 $40 = $39&1;
 HEAP32[29760>>2] = $40;
 $41 = HEAP32[((29760 + 4|0))>>2]|0;
 $42 = $0;
 $43 = (($42) + 12|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = $0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 1)|0;
 HEAP32[$45>>2] = $47;
 $48 = (($44) + ($47<<2)|0);
 HEAP32[$48>>2] = $41;
 while(1) {
  $49 = $0;
  $50 = (($49) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[29752>>2]|0;
  $53 = (($51) + ($52<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[((29760 + 4|0))>>2]|0;
  $56 = ($54|0)<=($55|0);
  if ($56) {
   $57 = $0;
   $58 = (($57) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[29752>>2]|0;
   $61 = (($59) + ($60<<2)|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = $0;
   $64 = (($63) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($62|0)<($65|0);
   $69 = $66;
  } else {
   $69 = 0;
  }
  if (!($69)) {
   break;
  }
  $67 = HEAP32[29752>>2]|0;
  $68 = (($67) + 2)|0;
  HEAP32[29752>>2] = $68;
 }
 STACKTOP = sp;return;
}
function _pass_mode_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $run_length = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = HEAP32[29752>>2]|0;
 $2 = (($1) + 1)|0;
 HEAP32[29768>>2] = $2;
 $3 = $0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[29768>>2]|0;
 $7 = (($5) + ($6<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[((29760 + 4|0))>>2]|0;
 $10 = (($8) - ($9))|0;
 $run_length = $10;
 $11 = $run_length;
 $12 = HEAP32[29760>>2]|0;
 _write_bits_d($11,$12);
 $13 = $run_length;
 $14 = HEAP32[((29760 + 4|0))>>2]|0;
 $15 = (($14) + ($13))|0;
 HEAP32[((29760 + 4|0))>>2] = $15;
 $16 = HEAP32[29752>>2]|0;
 $17 = (($16) + 2)|0;
 HEAP32[29752>>2] = $17;
 STACKTOP = sp;return;
}
function _read_bit() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = HEAP32[29528>>2]|0;
 $1 = ($0|0)!=(0);
 if ($1) {
  HEAP32[29832>>2] = 0;
  HEAP32[29840>>2] = 0;
  HEAP32[29528>>2] = 0;
 }
 $2 = HEAP32[29824>>2]|0;
 $3 = HEAP32[29840>>2]|0;
 $4 = (($2) + ($3)|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = HEAP32[29832>>2]|0;
 $8 = (29848 + ($7)|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = $6 & $10;
 $bit = $11;
 $12 = HEAP32[29832>>2]|0;
 $13 = ($12|0)==(7);
 if ($13) {
  HEAP32[29832>>2] = 0;
  $14 = HEAP32[29840>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[29840>>2] = $15;
  $18 = $bit;
  $19 = ($18|0)==(0);
  $20 = $19 ^ 1;
  $21 = $20&1;
  STACKTOP = sp;return ($21|0);
 } else {
  $16 = HEAP32[29832>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[29832>>2] = $17;
  $18 = $bit;
  $19 = ($18|0)==(0);
  $20 = $19 ^ 1;
  $21 = $20&1;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _write_bits_d($length,$color) {
 $length = $length|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytes = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $length;
 $1 = $color;
 $2 = HEAP32[29520>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[29792>>2] = 0;
  HEAP32[29800>>2] = 0;
  HEAP32[29520>>2] = 0;
 }
 $4 = $1;
 $5 = ($4|0)==(1);
 if ($5) {
  while(1) {
   $6 = $0;
   $7 = ($6>>>0)>(0);
   if ($7) {
    $8 = HEAP32[29792>>2]|0;
    $9 = ($8|0)!=(0);
    $116 = $9;
   } else {
    $116 = 0;
   }
   if (!($116)) {
    break;
   }
   $10 = HEAP32[29792>>2]|0;
   $11 = (29808 + ($10)|0);
   $12 = HEAP8[$11>>0]|0;
   $13 = $12 << 24 >> 24;
   $14 = HEAP32[29784>>2]|0;
   $15 = HEAP32[29800>>2]|0;
   $16 = (($14) + ($15)|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17 << 24 >> 24;
   $19 = $18 | $13;
   $20 = $19&255;
   HEAP8[$16>>0] = $20;
   $21 = $0;
   $22 = (($21) + -1)|0;
   $0 = $22;
   $23 = HEAP32[29792>>2]|0;
   $24 = ($23|0)==(7);
   if ($24) {
    HEAP32[29792>>2] = 0;
    $25 = HEAP32[29800>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[29800>>2] = $26;
   } else {
    $27 = HEAP32[29792>>2]|0;
    $28 = (($27) + 1)|0;
    HEAP32[29792>>2] = $28;
   }
  }
  $29 = HEAP32[29784>>2]|0;
  $30 = HEAP32[29800>>2]|0;
  $31 = (($29) + ($30)|0);
  $32 = HEAP32[29544>>2]|0;
  $33 = $0;
  $34 = (($33>>>0) / 8)&-1;
  $bytes = $34;
  _memcpy(($31|0),($32|0),($34|0))|0;
  $35 = $bytes;
  $36 = HEAP32[29800>>2]|0;
  $37 = (($36) + ($35))|0;
  HEAP32[29800>>2] = $37;
  $38 = $0;
  $39 = (($38>>>0) % 8)&-1;
  $0 = $39;
  while(1) {
   $40 = $0;
   $41 = ($40>>>0)>(0);
   if (!($41)) {
    break;
   }
   $42 = HEAP32[29792>>2]|0;
   $43 = (29808 + ($42)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44 << 24 >> 24;
   $46 = HEAP32[29784>>2]|0;
   $47 = HEAP32[29800>>2]|0;
   $48 = (($46) + ($47)|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49 << 24 >> 24;
   $51 = $50 | $45;
   $52 = $51&255;
   HEAP8[$48>>0] = $52;
   $53 = $0;
   $54 = (($53) + -1)|0;
   $0 = $54;
   $55 = HEAP32[29792>>2]|0;
   $56 = ($55|0)==(7);
   if ($56) {
    HEAP32[29792>>2] = 0;
    $57 = HEAP32[29800>>2]|0;
    $58 = (($57) + 1)|0;
    HEAP32[29800>>2] = $58;
   } else {
    $59 = HEAP32[29792>>2]|0;
    $60 = (($59) + 1)|0;
    HEAP32[29792>>2] = $60;
   }
  }
  STACKTOP = sp;return;
 } else {
  while(1) {
   $61 = $0;
   $62 = ($61>>>0)>(0);
   if ($62) {
    $63 = HEAP32[29792>>2]|0;
    $64 = ($63|0)!=(0);
    $117 = $64;
   } else {
    $117 = 0;
   }
   if (!($117)) {
    break;
   }
   $65 = HEAP32[29792>>2]|0;
   $66 = (29816 + ($65)|0);
   $67 = HEAP8[$66>>0]|0;
   $68 = $67 << 24 >> 24;
   $69 = HEAP32[29784>>2]|0;
   $70 = HEAP32[29800>>2]|0;
   $71 = (($69) + ($70)|0);
   $72 = HEAP8[$71>>0]|0;
   $73 = $72 << 24 >> 24;
   $74 = $73 & $68;
   $75 = $74&255;
   HEAP8[$71>>0] = $75;
   $76 = $0;
   $77 = (($76) + -1)|0;
   $0 = $77;
   $78 = HEAP32[29792>>2]|0;
   $79 = ($78|0)==(7);
   if ($79) {
    HEAP32[29792>>2] = 0;
    $80 = HEAP32[29800>>2]|0;
    $81 = (($80) + 1)|0;
    HEAP32[29800>>2] = $81;
   } else {
    $82 = HEAP32[29792>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[29792>>2] = $83;
   }
  }
  $84 = HEAP32[29784>>2]|0;
  $85 = HEAP32[29800>>2]|0;
  $86 = (($84) + ($85)|0);
  $87 = HEAP32[29536>>2]|0;
  $88 = $0;
  $89 = (($88>>>0) / 8)&-1;
  $bytes = $89;
  _memcpy(($86|0),($87|0),($89|0))|0;
  $90 = $bytes;
  $91 = HEAP32[29800>>2]|0;
  $92 = (($91) + ($90))|0;
  HEAP32[29800>>2] = $92;
  $93 = $0;
  $94 = (($93>>>0) % 8)&-1;
  $0 = $94;
  while(1) {
   $95 = $0;
   $96 = ($95>>>0)>(0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[29792>>2]|0;
   $98 = (29816 + ($97)|0);
   $99 = HEAP8[$98>>0]|0;
   $100 = $99 << 24 >> 24;
   $101 = HEAP32[29784>>2]|0;
   $102 = HEAP32[29800>>2]|0;
   $103 = (($101) + ($102)|0);
   $104 = HEAP8[$103>>0]|0;
   $105 = $104 << 24 >> 24;
   $106 = $105 & $100;
   $107 = $106&255;
   HEAP8[$103>>0] = $107;
   $108 = $0;
   $109 = (($108) + -1)|0;
   $0 = $109;
   $110 = HEAP32[29792>>2]|0;
   $111 = ($110|0)==(7);
   if ($111) {
    HEAP32[29792>>2] = 0;
    $112 = HEAP32[29800>>2]|0;
    $113 = (($112) + 1)|0;
    HEAP32[29800>>2] = $113;
   } else {
    $114 = HEAP32[29792>>2]|0;
    $115 = (($114) + 1)|0;
    HEAP32[29792>>2] = $115;
   }
  }
  STACKTOP = sp;return;
 }
}
function _find_run_length_code($color) {
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $color;
 $1 = $0;
 $2 = ($1|0)==(0);
 if ($2) {
  HEAP32[34848>>2] = 32352;
 } else {
  HEAP32[34848>>2] = 29856;
 }
 while(1) {
  $3 = HEAP32[34848>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)==(-1);
  if (!($5)) {
   break;
  }
  $6 = (_read_bit()|0);
  $7 = ($6|0)==(0);
  if ($7) {
   $8 = HEAP32[34848>>2]|0;
   $9 = (($8) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[34848>>2] = $10;
  } else {
   $11 = HEAP32[34848>>2]|0;
   $12 = (($11) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[34848>>2] = $13;
  }
 }
 $14 = HEAP32[34848>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _get_width($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $vararg_buffer = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $width = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(34880,34856,0);
 }
 $3 = $0;
 $4 = (($3) + 106|0);
 HEAP32[$vararg_buffer>>2] = $width;
 $5 = (_sscanf($4,34896,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$width>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 _fatalerr(34880,34904,0);
 $7 = HEAP32[$width>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _get_height($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $height = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $height = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(34936,34856,0);
 }
 $3 = $0;
 $4 = (($3) + 114|0);
 HEAP32[$vararg_buffer>>2] = $height;
 $5 = (_sscanf($4,34896,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$height>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 _fatalerr(34936,34952,0);
 $7 = HEAP32[$height>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _get_depth($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $depth = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $depth = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(34984,34856,0);
 }
 $3 = $0;
 $4 = (($3) + 122|0);
 HEAP32[$vararg_buffer>>2] = $depth;
 $5 = (_sscanf($4,34896,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$depth>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 _fatalerr(34984,35000,0);
 $7 = HEAP32[$depth>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _get_density($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $density = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $density = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(35056,34856,0);
 }
 $3 = $0;
 $4 = (($3) + 130|0);
 HEAP32[$vararg_buffer>>2] = $density;
 $5 = (_sscanf($4,34896,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$density>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 HEAP32[$density>>2] = -1;
 $7 = HEAP32[$density>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _get_compression($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $compression_code = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $compression_code = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(35072,34856,0);
 }
 $3 = $0;
 $4 = (($3) + 138|0);
 HEAP32[$vararg_buffer>>2] = $compression_code;
 $5 = (_sscanf($4,34896,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$compression_code>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 _fatalerr(35072,35088,0);
 $7 = HEAP32[$compression_code>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _set_compression($head,$compression_code) {
 $head = $head|0;
 $compression_code = $compression_code|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $1 = $head;
 $2 = $compression_code;
 $3 = $1;
 $4 = ($3|0)==(0|0);
 if ($4) {
  _fatalerr(35128,34856,0);
 }
 $5 = $2;
 $6 = (_valid_compression($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $0 = 0;
  $13 = $0;
  STACKTOP = sp;return ($13|0);
 }
 $8 = $1;
 $9 = (($8) + 138|0);
 ;HEAP8[$9+0>>0]=0|0;HEAP8[$9+1>>0]=0|0;HEAP8[$9+2>>0]=0|0;HEAP8[$9+3>>0]=0|0;HEAP8[$9+4>>0]=0|0;HEAP8[$9+5>>0]=0|0;HEAP8[$9+6>>0]=0|0;HEAP8[$9+7>>0]=0|0;
 $10 = $1;
 $11 = (($10) + 138|0);
 $12 = $2;
 HEAP32[$vararg_buffer>>2] = $12;
 (_sprintf($11,34896,$vararg_buffer)|0);
 $0 = 1;
 $13 = $0;
 STACKTOP = sp;return ($13|0);
}
function _get_complen($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $complen = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $complen = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(35144,34856,0);
 }
 $3 = $0;
 $4 = (($3) + 146|0);
 HEAP32[$vararg_buffer>>2] = $complen;
 $5 = (_sscanf($4,34896,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$complen>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 _fatalerr(35144,35160,0);
 $7 = HEAP32[$complen>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _set_complen($head,$complen) {
 $head = $head|0;
 $complen = $complen|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = $head;
 $1 = $complen;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  _fatalerr(35208,34856,0);
 }
 $4 = $0;
 $5 = (($4) + 146|0);
 ;HEAP8[$5+0>>0]=0|0;HEAP8[$5+1>>0]=0|0;HEAP8[$5+2>>0]=0|0;HEAP8[$5+3>>0]=0|0;HEAP8[$5+4>>0]=0|0;HEAP8[$5+5>>0]=0|0;HEAP8[$5+6>>0]=0|0;HEAP8[$5+7>>0]=0|0;
 $6 = $0;
 $7 = (($6) + 146|0);
 $8 = $1;
 HEAP32[$vararg_buffer>>2] = $8;
 (_sprintf($7,34896,$vararg_buffer)|0);
 STACKTOP = sp;return 1;
}
function _valid_compression($code) {
 $code = $code|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $code;
 $2 = $1;
 if ((($2|0) == 5)) {
  $0 = 1;
 } else if ((($2|0) == 0)) {
  $0 = 1;
 } else if ((($2|0) == 2)) {
  $0 = 1;
 } else if ((($2|0) == 1)) {
  $0 = 1;
 } else {
  $0 = 0;
 }
 $3 = $0;
 STACKTOP = sp;return ($3|0);
}
function _wsq_decode_mem($odata,$ow,$oh,$od,$oppi,$lossyflag,$idata,$ilen) {
 $odata = $odata|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppi = $oppi|0;
 $lossyflag = $lossyflag|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cbufptr = 0, $cdata = 0, $ebufptr = 0, $fdata = 0, $height = 0, $i = 0, $marker = 0, $num_pix = 0, $ppi = 0, $qdata = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer13 = sp + 56|0;
 $vararg_buffer11 = sp + 48|0;
 $vararg_buffer9 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $marker = sp + 140|0;
 $ppi = sp + 80|0;
 $fdata = sp + 72|0;
 $cbufptr = sp + 64|0;
 $1 = $odata;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppi;
 $6 = $lossyflag;
 $7 = $idata;
 $8 = $ilen;
 _init_wsq_decoder_resources();
 $9 = $7;
 HEAP32[$cbufptr>>2] = $9;
 $10 = $7;
 $11 = $8;
 $12 = (($10) + ($11)|0);
 $ebufptr = $12;
 $i = 0;
 while(1) {
  $13 = $i;
  $14 = ($13|0)<(8);
  if (!($14)) {
   break;
  }
  $15 = $i;
  $16 = (37624 + (($15*274)|0)|0);
  HEAP8[$16>>0] = 0;
  $17 = $i;
  $18 = (($17) + 1)|0;
  $i = $18;
 }
 $19 = $ebufptr;
 $20 = (_getc_marker_wsq($marker,65440,$cbufptr,$19)|0);
 $ret = $20;
 $21 = ($20|0)!=(0);
 if ($21) {
  _free_wsq_decoder_resources();
  $22 = $ret;
  $0 = $22;
  $126 = $0;
  STACKTOP = sp;return ($126|0);
 }
 $23 = $ebufptr;
 $24 = (_getc_marker_wsq($marker,2,$cbufptr,$23)|0);
 $ret = $24;
 $25 = ($24|0)!=(0);
 if ($25) {
  _free_wsq_decoder_resources();
  $26 = $ret;
  $0 = $26;
  $126 = $0;
  STACKTOP = sp;return ($126|0);
 }
 while(1) {
  $27 = HEAP16[$marker>>1]|0;
  $28 = $27&65535;
  $29 = ($28|0)!=(65442);
  if (!($29)) {
   label = 16;
   break;
  }
  $30 = HEAP16[$marker>>1]|0;
  $31 = $ebufptr;
  $32 = (_getc_table_wsq($30,37088,37104,37624,$cbufptr,$31)|0);
  $ret = $32;
  $33 = ($32|0)!=(0);
  if ($33) {
   label = 12;
   break;
  }
  $35 = $ebufptr;
  $36 = (_getc_marker_wsq($marker,2,$cbufptr,$35)|0);
  $ret = $36;
  $37 = ($36|0)!=(0);
  if ($37) {
   label = 14;
   break;
  }
 }
 if ((label|0) == 12) {
  _free_wsq_decoder_resources();
  $34 = $ret;
  $0 = $34;
  $126 = $0;
  STACKTOP = sp;return ($126|0);
 }
 else if ((label|0) == 14) {
  _free_wsq_decoder_resources();
  $38 = $ret;
  $0 = $38;
  $126 = $0;
  STACKTOP = sp;return ($126|0);
 }
 else if ((label|0) == 16) {
  $39 = $ebufptr;
  $40 = (_getc_frame_header_wsq(39816,$cbufptr,$39)|0);
  $ret = $40;
  $41 = ($40|0)!=(0);
  if ($41) {
   _free_wsq_decoder_resources();
   $42 = $ret;
   $0 = $42;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $43 = HEAP16[((39816 + 2|0))>>1]|0;
  $44 = $43&65535;
  $width = $44;
  $45 = HEAP16[((39816 + 4|0))>>1]|0;
  $46 = $45&65535;
  $height = $46;
  $47 = $width;
  $48 = $height;
  $49 = Math_imul($47, $48)|0;
  $num_pix = $49;
  $50 = $7;
  $51 = $8;
  $52 = (_getc_ppi_wsq($ppi,$50,$51)|0);
  $ret = $52;
  $53 = ($52|0)!=(0);
  if ($53) {
   _free_wsq_decoder_resources();
   $54 = $ret;
   $0 = $54;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $55 = HEAP32[8>>2]|0;
  $56 = ($55|0)>(0);
  if ($56) {
   $57 = HEAP32[_stderr>>2]|0;
   (_fprintf(($57|0),(35224|0),($vararg_buffer|0))|0);
  }
  $58 = $width;
  $59 = $height;
  _build_wsq_trees(36096,20,36576,64,$58,$59);
  $60 = HEAP32[8>>2]|0;
  $61 = ($60|0)>(0);
  if ($61) {
   $62 = HEAP32[_stderr>>2]|0;
   (_fprintf(($62|0),(35264|0),($vararg_buffer1|0))|0);
  }
  $63 = $num_pix;
  $64 = $63<<1;
  $65 = (_malloc($64)|0);
  $qdata = $65;
  $66 = $qdata;
  $67 = ($66|0)==(0|0);
  if ($67) {
   $68 = HEAP32[_stderr>>2]|0;
   (_fprintf(($68|0),(35312|0),($vararg_buffer3|0))|0);
   _free_wsq_decoder_resources();
   $0 = -20;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $69 = $qdata;
  $70 = $ebufptr;
  $71 = (_huffman_decode_data_mem($69,37088,37104,37624,$cbufptr,$70)|0);
  $ret = $71;
  $72 = ($71|0)!=(0);
  if ($72) {
   $73 = $qdata;
   _free($73);
   _free_wsq_decoder_resources();
   $74 = $ret;
   $0 = $74;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $75 = HEAP32[8>>2]|0;
  $76 = ($75|0)>(0);
  if ($76) {
   $77 = HEAP32[_stderr>>2]|0;
   (_fprintf(($77|0),(35360|0),($vararg_buffer5|0))|0);
  }
  $78 = $qdata;
  $79 = $width;
  $80 = $height;
  $81 = (_unquantize($fdata,37104,36576,64,$78,$79,$80)|0);
  $ret = $81;
  $82 = ($81|0)!=(0);
  if ($82) {
   $83 = $qdata;
   _free($83);
   _free_wsq_decoder_resources();
   $84 = $ret;
   $0 = $84;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $85 = HEAP32[8>>2]|0;
  $86 = ($85|0)>(0);
  if ($86) {
   $87 = HEAP32[_stderr>>2]|0;
   (_fprintf(($87|0),(35424|0),($vararg_buffer7|0))|0);
  }
  $88 = $qdata;
  _free($88);
  $89 = HEAP32[$fdata>>2]|0;
  $90 = $width;
  $91 = $height;
  $92 = (_wsq_reconstruct($89,$90,$91,36096,20,37088)|0);
  $ret = $92;
  $93 = ($92|0)!=(0);
  if ($93) {
   $94 = HEAP32[$fdata>>2]|0;
   _free($94);
   _free_wsq_decoder_resources();
   $95 = $ret;
   $0 = $95;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $96 = HEAP32[8>>2]|0;
  $97 = ($96|0)>(0);
  if ($97) {
   $98 = HEAP32[_stderr>>2]|0;
   (_fprintf(($98|0),(35464|0),($vararg_buffer9|0))|0);
  }
  $99 = $num_pix;
  $100 = $99;
  $101 = (_malloc($100)|0);
  $cdata = $101;
  $102 = $cdata;
  $103 = ($102|0)==(0|0);
  if ($103) {
   $104 = HEAP32[$fdata>>2]|0;
   _free($104);
   _free_wsq_decoder_resources();
   $105 = HEAP32[_stderr>>2]|0;
   (_fprintf(($105|0),(35504|0),($vararg_buffer11|0))|0);
   $0 = -21;
   $126 = $0;
   STACKTOP = sp;return ($126|0);
  }
  $106 = $cdata;
  $107 = HEAP32[$fdata>>2]|0;
  $108 = $width;
  $109 = $height;
  $110 = +HEAPF32[((39816 + 8|0))>>2];
  $111 = +HEAPF32[((39816 + 12|0))>>2];
  _conv_img_2_uchar($106,$107,$108,$109,$110,$111);
  $112 = HEAP32[$fdata>>2]|0;
  _free($112);
  _free_wsq_decoder_resources();
  $113 = HEAP32[8>>2]|0;
  $114 = ($113|0)>(0);
  if ($114) {
   $115 = HEAP32[_stderr>>2]|0;
   (_fprintf(($115|0),(35544|0),($vararg_buffer13|0))|0);
  }
  $116 = $cdata;
  $117 = $1;
  HEAP32[$117>>2] = $116;
  $118 = $width;
  $119 = $2;
  HEAP32[$119>>2] = $118;
  $120 = $height;
  $121 = $3;
  HEAP32[$121>>2] = $120;
  $122 = $4;
  HEAP32[$122>>2] = 8;
  $123 = HEAP32[$ppi>>2]|0;
  $124 = $5;
  HEAP32[$124>>2] = $123;
  $125 = $6;
  HEAP32[$125>>2] = 1;
  $0 = 0;
  $126 = $0;
  STACKTOP = sp;return ($126|0);
 }
 return 0|0;
}
function _huffman_decode_data_mem($ip,$dtt_table,$dqt_table,$dht_table,$cbufptr,$ebufptr) {
 $ip = $ip|0;
 $dtt_table = $dtt_table|0;
 $dqt_table = $dqt_table|0;
 $dht_table = $dht_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bit_count = 0, $blk = 0, $hufftable = 0, $hufftable_id = 0, $ipc = 0, $ipc_mx = 0, $ipc_q = 0, $last_size = 0, $marker = 0;
 var $maxcode = 0, $mincode = 0, $n = 0, $nodeptr = 0, $ret = 0, $tbits = 0, $valptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer24 = 0, $vararg_buffer26 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0;
 var $vararg_buffer30 = 0, $vararg_buffer6 = 0, $vararg_buffer8 = 0, $vararg_ptr33 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 416|0;
 $vararg_buffer30 = sp + 120|0;
 $vararg_buffer28 = sp + 112|0;
 $vararg_buffer26 = sp + 104|0;
 $vararg_buffer24 = sp + 96|0;
 $vararg_buffer22 = sp + 88|0;
 $vararg_buffer20 = sp + 80|0;
 $vararg_buffer18 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $marker = sp + 402|0;
 $bit_count = sp + 360|0;
 $nodeptr = sp + 352|0;
 $last_size = sp + 348|0;
 $hufftable_id = sp + 404|0;
 $hufftable = sp + 344|0;
 $maxcode = sp + 276|0;
 $mincode = sp + 208|0;
 $valptr = sp + 140|0;
 $tbits = sp + 400|0;
 $1 = $ip;
 $2 = $dtt_table;
 $3 = $dqt_table;
 $4 = $dht_table;
 $5 = $cbufptr;
 $6 = $ebufptr;
 $blk = 0;
 $7 = $5;
 $8 = $6;
 $9 = (_getc_marker_wsq($marker,4,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $289 = $0;
  STACKTOP = sp;return ($289|0);
 }
 HEAP32[$bit_count>>2] = 0;
 $ipc = 0;
 $ipc_q = 0;
 $12 = HEAP16[((39816 + 2|0))>>1]|0;
 $13 = $12&65535;
 $14 = HEAP16[((39816 + 4|0))>>1]|0;
 $15 = $14&65535;
 $16 = Math_imul($13, $15)|0;
 $ipc_mx = $16;
 L5: while(1) {
  $17 = HEAP16[$marker>>1]|0;
  $18 = $17&65535;
  $19 = ($18|0)!=(65441);
  if (!($19)) {
   label = 101;
   break;
  }
  $20 = HEAP16[$marker>>1]|0;
  $21 = $20&65535;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = $blk;
   $24 = (($23) + 1)|0;
   $blk = $24;
   while(1) {
    $25 = HEAP16[$marker>>1]|0;
    $26 = $25&65535;
    $27 = ($26|0)!=(65443);
    if (!($27)) {
     break;
    }
    $28 = HEAP16[$marker>>1]|0;
    $29 = $2;
    $30 = $3;
    $31 = $4;
    $32 = $5;
    $33 = $6;
    $34 = (_getc_table_wsq($28,$29,$30,$31,$32,$33)|0);
    $ret = $34;
    $35 = ($34|0)!=(0);
    if ($35) {
     label = 9;
     break L5;
    }
    $37 = $5;
    $38 = $6;
    $39 = (_getc_marker_wsq($marker,4,$37,$38)|0);
    $ret = $39;
    $40 = ($39|0)!=(0);
    if ($40) {
     label = 11;
     break L5;
    }
   }
   $42 = $3;
   $43 = (($42) + 516|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44 << 24 >> 24;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = $ipc_q;
    $48 = ($47|0)!=(0);
    if (!($48)) {
     $n = 0;
     while(1) {
      $49 = $n;
      $50 = ($49|0)<(64);
      if (!($50)) {
       break;
      }
      $51 = $n;
      $52 = $3;
      $53 = (($52) + 4|0);
      $54 = (($53) + ($51<<2)|0);
      $55 = +HEAPF32[$54>>2];
      $56 = $55;
      $57 = $56 == 0.0;
      if ($57) {
       $58 = $n;
       $59 = (36576 + ($58<<3)|0);
       $60 = (($59) + 4|0);
       $61 = HEAPU8[$60>>0]|(HEAPU8[$60+1>>0]<<8);
       $62 = $61 << 16 >> 16;
       $63 = $n;
       $64 = (36576 + ($63<<3)|0);
       $65 = (($64) + 6|0);
       $66 = HEAPU8[$65>>0]|(HEAPU8[$65+1>>0]<<8);
       $67 = $66 << 16 >> 16;
       $68 = Math_imul($62, $67)|0;
       $69 = $ipc_mx;
       $70 = (($69) - ($68))|0;
       $ipc_mx = $70;
      }
      $71 = $n;
      $72 = (($71) + 1)|0;
      $n = $72;
     }
     $ipc_q = 1;
    }
   }
   $73 = $5;
   $74 = $6;
   $75 = (_getc_block_header($hufftable_id,$73,$74)|0);
   $ret = $75;
   $76 = ($75|0)!=(0);
   if ($76) {
    label = 23;
    break;
   }
   $78 = $4;
   $79 = HEAP8[$hufftable_id>>0]|0;
   $80 = $79&255;
   $81 = (($78) + (($80*274)|0)|0);
   $82 = HEAP8[$81>>0]|0;
   $83 = $82&255;
   $84 = ($83|0)!=(1);
   if ($84) {
    label = 25;
    break;
   }
   $89 = $4;
   $90 = HEAP8[$hufftable_id>>0]|0;
   $91 = $90&255;
   $92 = (($89) + (($91*274)|0)|0);
   $93 = (($92) + 1|0);
   $94 = (_build_huffsizes($hufftable,$last_size,$93,256)|0);
   $ret = $94;
   $95 = ($94|0)!=(0);
   if ($95) {
    label = 27;
    break;
   }
   $97 = HEAP32[$hufftable>>2]|0;
   _build_huffcodes($97);
   $98 = HEAP32[$hufftable>>2]|0;
   $99 = HEAP32[$last_size>>2]|0;
   $100 = (_check_huffcodes_wsq($98,$99)|0);
   $ret = $100;
   $101 = ($100|0)!=(0);
   if ($101) {
    $102 = HEAP32[_stderr>>2]|0;
    $103 = HEAP8[$hufftable_id>>0]|0;
    $104 = $103&255;
    HEAP32[$vararg_buffer3>>2] = $104;
    (_fprintf(($102|0),(35672|0),($vararg_buffer3|0))|0);
   }
   $105 = HEAP32[$hufftable>>2]|0;
   $106 = $4;
   $107 = HEAP8[$hufftable_id>>0]|0;
   $108 = $107&255;
   $109 = (($106) + (($108*274)|0)|0);
   $110 = (($109) + 1|0);
   _gen_decode_table($105,$maxcode,$mincode,$valptr,$110);
   $111 = HEAP32[$hufftable>>2]|0;
   _free($111);
   HEAP32[$bit_count>>2] = 0;
   HEAP16[$marker>>1] = 0;
  }
  $112 = $4;
  $113 = HEAP8[$hufftable_id>>0]|0;
  $114 = $113&255;
  $115 = (($112) + (($114*274)|0)|0);
  $116 = (($115) + 17|0);
  $117 = $5;
  $118 = $6;
  $119 = (_decode_data_mem($nodeptr,$mincode,$maxcode,$valptr,$116,$117,$118,$bit_count,$marker)|0);
  $ret = $119;
  $120 = ($119|0)!=(0);
  if ($120) {
   label = 32;
   break;
  }
  $122 = HEAP32[$nodeptr>>2]|0;
  $123 = ($122|0)==(-1);
  if ($123) {
   while(1) {
    $124 = HEAP16[$marker>>1]|0;
    $125 = $124&65535;
    $126 = ($125|0)==(65448);
    if ($126) {
     $127 = $blk;
     $128 = ($127|0)==(3);
     $290 = $128;
    } else {
     $290 = 0;
    }
    if (!($290)) {
     break;
    }
    $129 = HEAP16[$marker>>1]|0;
    $130 = $2;
    $131 = $3;
    $132 = $4;
    $133 = $5;
    $134 = $6;
    $135 = (_getc_table_wsq($129,$130,$131,$132,$133,$134)|0);
    $ret = $135;
    $136 = ($135|0)!=(0);
    if ($136) {
     label = 39;
     break L5;
    }
    $138 = $5;
    $139 = $6;
    $140 = (_getc_marker_wsq($marker,65535,$138,$139)|0);
    $ret = $140;
    $141 = ($140|0)!=(0);
    if ($141) {
     label = 41;
     break L5;
    }
   }
   continue;
  }
  $143 = $ipc;
  $144 = $ipc_mx;
  $145 = ($143|0)>($144|0);
  if ($145) {
   label = 45;
   break;
  }
  $150 = HEAP32[$nodeptr>>2]|0;
  $151 = ($150|0)>(0);
  if ($151) {
   $152 = HEAP32[$nodeptr>>2]|0;
   $153 = ($152|0)<=(100);
   if ($153) {
    $154 = HEAP32[$nodeptr>>2]|0;
    $155 = $ipc;
    $156 = (($155) + ($154))|0;
    $ipc = $156;
    $157 = $ipc;
    $158 = $ipc_mx;
    $159 = ($157|0)>($158|0);
    if ($159) {
     label = 49;
     break;
    }
    $n = 0;
    while(1) {
     $164 = $n;
     $165 = HEAP32[$nodeptr>>2]|0;
     $166 = ($164|0)<($165|0);
     if (!($166)) {
      break;
     }
     $167 = $1;
     $168 = (($167) + 2|0);
     $1 = $168;
     HEAP16[$167>>1] = 0;
     $169 = $n;
     $170 = (($169) + 1)|0;
     $n = $170;
    }
   } else {
    label = 55;
   }
  } else {
   label = 55;
  }
  if ((label|0) == 55) {
   label = 0;
   $171 = HEAP32[$nodeptr>>2]|0;
   $172 = ($171|0)>(106);
   if ($172) {
    $173 = HEAP32[$nodeptr>>2]|0;
    $174 = ($173|0)<(255);
    if ($174) {
     $175 = HEAP32[$nodeptr>>2]|0;
     $176 = (($175) - 180)|0;
     $177 = $176&65535;
     $178 = $1;
     $179 = (($178) + 2|0);
     $1 = $179;
     HEAP16[$178>>1] = $177;
     $180 = $ipc;
     $181 = (($180) + 1)|0;
     $ipc = $181;
    } else {
     label = 58;
    }
   } else {
    label = 58;
   }
   if ((label|0) == 58) {
    label = 0;
    $182 = HEAP32[$nodeptr>>2]|0;
    $183 = ($182|0)==(101);
    if ($183) {
     $184 = $5;
     $185 = $6;
     $186 = (_getc_nextbits_wsq($tbits,$marker,$184,$185,$bit_count,8)|0);
     $ret = $186;
     $187 = ($186|0)!=(0);
     if ($187) {
      label = 60;
      break;
     }
     $189 = HEAP16[$tbits>>1]|0;
     $190 = $1;
     $191 = (($190) + 2|0);
     $1 = $191;
     HEAP16[$190>>1] = $189;
     $192 = $ipc;
     $193 = (($192) + 1)|0;
     $ipc = $193;
    } else {
     $194 = HEAP32[$nodeptr>>2]|0;
     $195 = ($194|0)==(102);
     if ($195) {
      $196 = $5;
      $197 = $6;
      $198 = (_getc_nextbits_wsq($tbits,$marker,$196,$197,$bit_count,8)|0);
      $ret = $198;
      $199 = ($198|0)!=(0);
      if ($199) {
       label = 64;
       break;
      }
      $201 = HEAP16[$tbits>>1]|0;
      $202 = $201&65535;
      $203 = (0 - ($202))|0;
      $204 = $203&65535;
      $205 = $1;
      $206 = (($205) + 2|0);
      $1 = $206;
      HEAP16[$205>>1] = $204;
      $207 = $ipc;
      $208 = (($207) + 1)|0;
      $ipc = $208;
     } else {
      $209 = HEAP32[$nodeptr>>2]|0;
      $210 = ($209|0)==(103);
      if ($210) {
       $211 = $5;
       $212 = $6;
       $213 = (_getc_nextbits_wsq($tbits,$marker,$211,$212,$bit_count,16)|0);
       $ret = $213;
       $214 = ($213|0)!=(0);
       if ($214) {
        label = 68;
        break;
       }
       $216 = HEAP16[$tbits>>1]|0;
       $217 = $1;
       $218 = (($217) + 2|0);
       $1 = $218;
       HEAP16[$217>>1] = $216;
       $219 = $ipc;
       $220 = (($219) + 1)|0;
       $ipc = $220;
      } else {
       $221 = HEAP32[$nodeptr>>2]|0;
       $222 = ($221|0)==(104);
       if ($222) {
        $223 = $5;
        $224 = $6;
        $225 = (_getc_nextbits_wsq($tbits,$marker,$223,$224,$bit_count,16)|0);
        $ret = $225;
        $226 = ($225|0)!=(0);
        if ($226) {
         label = 72;
         break;
        }
        $228 = HEAP16[$tbits>>1]|0;
        $229 = $228&65535;
        $230 = (0 - ($229))|0;
        $231 = $230&65535;
        $232 = $1;
        $233 = (($232) + 2|0);
        $1 = $233;
        HEAP16[$232>>1] = $231;
        $234 = $ipc;
        $235 = (($234) + 1)|0;
        $ipc = $235;
       } else {
        $236 = HEAP32[$nodeptr>>2]|0;
        $237 = ($236|0)==(105);
        if ($237) {
         $238 = $5;
         $239 = $6;
         $240 = (_getc_nextbits_wsq($tbits,$marker,$238,$239,$bit_count,8)|0);
         $ret = $240;
         $241 = ($240|0)!=(0);
         if ($241) {
          label = 76;
          break;
         }
         $243 = HEAP16[$tbits>>1]|0;
         $244 = $243&65535;
         $245 = $ipc;
         $246 = (($245) + ($244))|0;
         $ipc = $246;
         $247 = $ipc;
         $248 = $ipc_mx;
         $249 = ($247|0)>($248|0);
         if ($249) {
          label = 78;
          break;
         }
         $254 = HEAP16[$tbits>>1]|0;
         $255 = $254&65535;
         $n = $255;
         while(1) {
          $256 = $n;
          $257 = (($256) + -1)|0;
          $n = $257;
          $258 = ($256|0)!=(0);
          if (!($258)) {
           break;
          }
          $259 = $1;
          $260 = (($259) + 2|0);
          $1 = $260;
          HEAP16[$259>>1] = 0;
         }
        } else {
         $261 = HEAP32[$nodeptr>>2]|0;
         $262 = ($261|0)==(106);
         if (!($262)) {
          label = 92;
          break;
         }
         $263 = $5;
         $264 = $6;
         $265 = (_getc_nextbits_wsq($tbits,$marker,$263,$264,$bit_count,16)|0);
         $ret = $265;
         $266 = ($265|0)!=(0);
         if ($266) {
          label = 85;
          break;
         }
         $268 = HEAP16[$tbits>>1]|0;
         $269 = $268&65535;
         $270 = $ipc;
         $271 = (($270) + ($269))|0;
         $ipc = $271;
         $272 = $ipc;
         $273 = $ipc_mx;
         $274 = ($272|0)>($273|0);
         if ($274) {
          label = 87;
          break;
         }
         $279 = HEAP16[$tbits>>1]|0;
         $280 = $279&65535;
         $n = $280;
         while(1) {
          $281 = $n;
          $282 = (($281) + -1)|0;
          $n = $282;
          $283 = ($281|0)!=(0);
          if (!($283)) {
           break;
          }
          $284 = $1;
          $285 = (($284) + 2|0);
          $1 = $285;
          HEAP16[$284>>1] = 0;
         }
        }
       }
      }
     }
    }
   }
  }
 }
 switch (label|0) {
  case 9: {
   $36 = $ret;
   $0 = $36;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 11: {
   $41 = $ret;
   $0 = $41;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 23: {
   $77 = $ret;
   $0 = $77;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 25: {
   $85 = HEAP32[_stderr>>2]|0;
   (_fprintf(($85|0),(35600|0),($vararg_buffer|0))|0);
   $86 = HEAP32[_stderr>>2]|0;
   $87 = HEAP8[$hufftable_id>>0]|0;
   $88 = $87&255;
   HEAP32[$vararg_buffer1>>2] = $88;
   (_fprintf(($86|0),(35640|0),($vararg_buffer1|0))|0);
   $0 = -51;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 27: {
   $96 = $ret;
   $0 = $96;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 32: {
   $121 = $ret;
   $0 = $121;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 39: {
   $137 = $ret;
   $0 = $137;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 41: {
   $142 = $ret;
   $0 = $142;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 45: {
   $146 = HEAP32[_stderr>>2]|0;
   (_fprintf(($146|0),(35704|0),($vararg_buffer6|0))|0);
   $147 = HEAP32[_stderr>>2]|0;
   (_fprintf(($147|0),(35744|0),($vararg_buffer8|0))|0);
   $148 = HEAP32[_stderr>>2]|0;
   (_fprintf(($148|0),(35792|0),($vararg_buffer10|0))|0);
   $149 = HEAP32[_stderr>>2]|0;
   (_fflush(($149|0))|0);
   $0 = -51;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 49: {
   $160 = HEAP32[_stderr>>2]|0;
   (_fprintf(($160|0),(35840|0),($vararg_buffer12|0))|0);
   $161 = HEAP32[_stderr>>2]|0;
   (_fprintf(($161|0),(35744|0),($vararg_buffer14|0))|0);
   $162 = HEAP32[_stderr>>2]|0;
   (_fprintf(($162|0),(35792|0),($vararg_buffer16|0))|0);
   $163 = HEAP32[_stderr>>2]|0;
   (_fflush(($163|0))|0);
   $0 = -51;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 60: {
   $188 = $ret;
   $0 = $188;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 64: {
   $200 = $ret;
   $0 = $200;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 68: {
   $215 = $ret;
   $0 = $215;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 72: {
   $227 = $ret;
   $0 = $227;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 76: {
   $242 = $ret;
   $0 = $242;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 78: {
   $250 = HEAP32[_stderr>>2]|0;
   (_fprintf(($250|0),(35880|0),($vararg_buffer18|0))|0);
   $251 = HEAP32[_stderr>>2]|0;
   (_fprintf(($251|0),(35744|0),($vararg_buffer20|0))|0);
   $252 = HEAP32[_stderr>>2]|0;
   (_fprintf(($252|0),(35792|0),($vararg_buffer22|0))|0);
   $253 = HEAP32[_stderr>>2]|0;
   (_fflush(($253|0))|0);
   $0 = -51;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 85: {
   $267 = $ret;
   $0 = $267;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 87: {
   $275 = HEAP32[_stderr>>2]|0;
   (_fprintf(($275|0),(35920|0),($vararg_buffer24|0))|0);
   $276 = HEAP32[_stderr>>2]|0;
   (_fprintf(($276|0),(35744|0),($vararg_buffer26|0))|0);
   $277 = HEAP32[_stderr>>2]|0;
   (_fprintf(($277|0),(35792|0),($vararg_buffer28|0))|0);
   $278 = HEAP32[_stderr>>2]|0;
   (_fflush(($278|0))|0);
   $0 = -51;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 92: {
   $286 = HEAP32[_stderr>>2]|0;
   $287 = HEAP32[$nodeptr>>2]|0;
   $288 = HEAP32[$nodeptr>>2]|0;
   HEAP32[$vararg_buffer30>>2] = $287;
   $vararg_ptr33 = (($vararg_buffer30) + 4|0);
   HEAP32[$vararg_ptr33>>2] = $288;
   (_fprintf(($286|0),(35960|0),($vararg_buffer30|0))|0);
   $0 = -52;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
  case 101: {
   $0 = 0;
   $289 = $0;
   STACKTOP = sp;return ($289|0);
   break;
  }
 }
 return 0|0;
}
function _decode_data_mem($onodeptr,$mincode,$maxcode,$valptr,$huffvalues,$cbufptr,$ebufptr,$bit_count,$marker) {
 $onodeptr = $onodeptr|0;
 $mincode = $mincode|0;
 $maxcode = $maxcode|0;
 $valptr = $valptr|0;
 $huffvalues = $huffvalues|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 $bit_count = $bit_count|0;
 $marker = $marker|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $code = 0, $inx = 0, $inx2 = 0, $ret = 0, $tbits = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $code = sp + 54|0;
 $tbits = sp + 52|0;
 $1 = $onodeptr;
 $2 = $mincode;
 $3 = $maxcode;
 $4 = $valptr;
 $5 = $huffvalues;
 $6 = $cbufptr;
 $7 = $ebufptr;
 $8 = $bit_count;
 $9 = $marker;
 $10 = $9;
 $11 = $6;
 $12 = $7;
 $13 = $8;
 $14 = (_getc_nextbits_wsq($code,$10,$11,$12,$13,1)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $ret;
  $0 = $16;
  $69 = $0;
  STACKTOP = sp;return ($69|0);
 }
 $17 = $9;
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $1;
  HEAP32[$21>>2] = -1;
  $0 = 0;
  $69 = $0;
  STACKTOP = sp;return ($69|0);
 }
 $inx = 1;
 while(1) {
  $22 = HEAP16[$code>>1]|0;
  $23 = $22&65535;
  $24 = $inx;
  $25 = $3;
  $26 = (($25) + ($24<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($23|0)>($27|0);
  if (!($28)) {
   label = 13;
   break;
  }
  $29 = $9;
  $30 = $6;
  $31 = $7;
  $32 = $8;
  $33 = (_getc_nextbits_wsq($tbits,$29,$30,$31,$32,1)|0);
  $ret = $33;
  $34 = ($33|0)!=(0);
  if ($34) {
   label = 8;
   break;
  }
  $36 = HEAP16[$code>>1]|0;
  $37 = $36&65535;
  $38 = $37 << 1;
  $39 = HEAP16[$tbits>>1]|0;
  $40 = $39&65535;
  $41 = (($38) + ($40))|0;
  $42 = $41&65535;
  HEAP16[$code>>1] = $42;
  $43 = $9;
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 10;
   break;
  }
  $48 = $inx;
  $49 = (($48) + 1)|0;
  $inx = $49;
 }
 if ((label|0) == 8) {
  $35 = $ret;
  $0 = $35;
  $69 = $0;
  STACKTOP = sp;return ($69|0);
 }
 else if ((label|0) == 10) {
  $47 = $1;
  HEAP32[$47>>2] = -1;
  $0 = 0;
  $69 = $0;
  STACKTOP = sp;return ($69|0);
 }
 else if ((label|0) == 13) {
  $50 = $inx;
  $51 = $4;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $inx2 = $53;
  $54 = $inx2;
  $55 = HEAP16[$code>>1]|0;
  $56 = $55&65535;
  $57 = (($54) + ($56))|0;
  $58 = $inx;
  $59 = $2;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($57) - ($61))|0;
  $inx2 = $62;
  $63 = $inx2;
  $64 = $5;
  $65 = (($64) + ($63)|0);
  $66 = HEAP8[$65>>0]|0;
  $67 = $66&255;
  $68 = $1;
  HEAP32[$68>>2] = $67;
  $0 = 0;
  $69 = $0;
  STACKTOP = sp;return ($69|0);
 }
 return 0|0;
}
function _getc_nextbits_wsq($obits,$marker,$cbufptr,$ebufptr,$bit_count,$bits_req) {
 $obits = $obits|0;
 $marker = $marker|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 $bit_count = $bit_count|0;
 $bits_req = $bits_req|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $bits = 0, $bits_needed = 0;
 var $ret = 0, $tbits = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $tbits = sp + 40|0;
 $1 = $obits;
 $2 = $marker;
 $3 = $cbufptr;
 $4 = $ebufptr;
 $5 = $bit_count;
 $6 = $bits_req;
 $7 = $5;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = $3;
  $11 = $4;
  $12 = (_getc_byte(36016,$10,$11)|0);
  $ret = $12;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $ret;
   $0 = $14;
   $97 = $0;
   STACKTOP = sp;return ($97|0);
  }
  $15 = $5;
  HEAP32[$15>>2] = 8;
  $16 = HEAP8[36016>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(255);
  do {
   if ($18) {
    $19 = $3;
    $20 = $4;
    $21 = (_getc_byte(36024,$19,$20)|0);
    $ret = $21;
    $22 = ($21|0)!=(0);
    if ($22) {
     $23 = $ret;
     $0 = $23;
     $97 = $0;
     STACKTOP = sp;return ($97|0);
    }
    $24 = HEAP8[36024>>0]|0;
    $25 = $24&255;
    $26 = ($25|0)!=(0);
    if ($26) {
     $27 = $6;
     $28 = ($27|0)==(1);
     if ($28) {
      $29 = HEAP8[36016>>0]|0;
      $30 = $29&255;
      $31 = $30 << 8;
      $32 = HEAP8[36024>>0]|0;
      $33 = $32&255;
      $34 = $31 | $33;
      $35 = $34&65535;
      $36 = $2;
      HEAP16[$36>>1] = $35;
      $37 = $1;
      HEAP16[$37>>1] = 1;
      $0 = 0;
      $97 = $0;
      STACKTOP = sp;return ($97|0);
     }
    }
    $38 = HEAP8[36024>>0]|0;
    $39 = $38&255;
    $40 = ($39|0)!=(0);
    if (!($40)) {
     break;
    }
    $41 = HEAP32[_stderr>>2]|0;
    (_fprintf(($41|0),(36048|0),($vararg_buffer|0))|0);
    $0 = -41;
    $97 = $0;
    STACKTOP = sp;return ($97|0);
   }
  } while(0);
 }
 $42 = $6;
 $43 = $5;
 $44 = HEAP32[$43>>2]|0;
 $45 = ($42|0)<=($44|0);
 do {
  if ($45) {
   $46 = HEAP8[36016>>0]|0;
   $47 = $46&255;
   $48 = $5;
   $49 = HEAP32[$48>>2]|0;
   $50 = $6;
   $51 = (($49) - ($50))|0;
   $52 = $47 >> $51;
   $53 = $6;
   $54 = (36032 + ($53)|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55&255;
   $57 = $52 & $56;
   $58 = $57&65535;
   $bits = $58;
   $59 = $6;
   $60 = $5;
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) - ($59))|0;
   HEAP32[$60>>2] = $62;
   $63 = $5;
   $64 = HEAP32[$63>>2]|0;
   $65 = (36032 + ($64)|0);
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   $68 = HEAP8[36016>>0]|0;
   $69 = $68&255;
   $70 = $69 & $67;
   $71 = $70&255;
   HEAP8[36016>>0] = $71;
  } else {
   $72 = $6;
   $73 = $5;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) - ($74))|0;
   $bits_needed = $75;
   $76 = HEAP8[36016>>0]|0;
   $77 = $76&255;
   $78 = $bits_needed;
   $79 = $77 << $78;
   $80 = $79&65535;
   $bits = $80;
   $81 = $5;
   HEAP32[$81>>2] = 0;
   $82 = $3;
   $83 = $4;
   $84 = $5;
   $85 = $bits_needed;
   $86 = (_getc_nextbits_wsq($tbits,0,$82,$83,$84,$85)|0);
   $ret = $86;
   $87 = ($86|0)!=(0);
   if (!($87)) {
    $89 = HEAP16[$tbits>>1]|0;
    $90 = $89&65535;
    $91 = $bits;
    $92 = $91&65535;
    $93 = $92 | $90;
    $94 = $93&65535;
    $bits = $94;
    break;
   }
   $88 = $ret;
   $0 = $88;
   $97 = $0;
   STACKTOP = sp;return ($97|0);
  }
 } while(0);
 $95 = $bits;
 $96 = $1;
 HEAP16[$96>>1] = $95;
 $0 = 0;
 $97 = $0;
 STACKTOP = sp;return ($97|0);
}
function _check_huffcodes_wsq($hufftable,$last_size) {
 $hufftable = $hufftable|0;
 $last_size = $last_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $all_ones = 0, $i = 0, $k = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $hufftable;
 $2 = $last_size;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $2;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   label = 15;
   break;
  }
  $all_ones = 1;
  $k = 0;
  while(1) {
   $6 = $k;
   $7 = $1;
   $8 = $i;
   $9 = (($7) + ($8<<3)|0);
   $10 = HEAP16[$9>>1]|0;
   $11 = $10 << 16 >> 16;
   $12 = ($6|0)<($11|0);
   if ($12) {
    $13 = $all_ones;
    $14 = ($13|0)!=(0);
    $39 = $14;
   } else {
    $39 = 0;
   }
   if (!($39)) {
    break;
   }
   $15 = $all_ones;
   $16 = ($15|0)!=(0);
   if ($16) {
    $17 = $1;
    $18 = $i;
    $19 = (($17) + ($18<<3)|0);
    $20 = (($19) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = $k;
    $23 = $21 >>> $22;
    $24 = $23 & 1;
    $25 = ($24|0)!=(0);
    $27 = $25;
   } else {
    $27 = 0;
   }
   $26 = $27&1;
   $all_ones = $26;
   $28 = $k;
   $29 = (($28) + 1)|0;
   $k = $29;
  }
  $30 = $all_ones;
  $31 = ($30|0)!=(0);
  if ($31) {
   label = 12;
   break;
  }
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 if ((label|0) == 12) {
  $32 = HEAP32[_stderr>>2]|0;
  (_fprintf(($32|0),(39840|0),($vararg_buffer|0))|0);
  $33 = HEAP32[_stderr>>2]|0;
  (_fprintf(($33|0),(39888|0),($vararg_buffer1|0))|0);
  $34 = HEAP32[_stderr>>2]|0;
  (_fprintf(($34|0),(39936|0),($vararg_buffer3|0))|0);
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(39984|0),($vararg_buffer5|0))|0);
  $0 = -1;
  $38 = $0;
  STACKTOP = sp;return ($38|0);
 }
 else if ((label|0) == 15) {
  $0 = 0;
  $38 = $0;
  STACKTOP = sp;return ($38|0);
 }
 return 0|0;
}
function _getc_ppi_wsq($oppi,$idata,$ilen) {
 $oppi = $oppi|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nistcom = 0, $ppi = 0, $ret = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $value = sp + 4|0;
 $nistcom = sp;
 $1 = $oppi;
 $2 = $idata;
 $3 = $ilen;
 $4 = $2;
 $5 = $3;
 $6 = (_getc_nistcom_wsq($nistcom,$4,$5)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $24 = $0;
  STACKTOP = sp;return ($24|0);
 }
 $9 = HEAP32[$nistcom>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$nistcom>>2]|0;
  $12 = (_extractfet_ret($value,40008,$11)|0);
  $ret = $12;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$nistcom>>2]|0;
   _freefet($14);
   $15 = $ret;
   $0 = $15;
   $24 = $0;
   STACKTOP = sp;return ($24|0);
  }
  $16 = HEAP32[$value>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$value>>2]|0;
   $19 = (_atoi($18)|0);
   $ppi = $19;
   $20 = HEAP32[$value>>2]|0;
   _free($20);
  } else {
   $ppi = -1;
  }
  $21 = HEAP32[$nistcom>>2]|0;
  _freefet($21);
 } else {
  $ppi = -1;
 }
 $22 = $ppi;
 $23 = $1;
 HEAP32[$23>>2] = $22;
 $0 = 0;
 $24 = $0;
 STACKTOP = sp;return ($24|0);
}
function _getc_marker_wsq($omarker,$type,$cbufptr,$ebufptr) {
 $omarker = $omarker|0;
 $type = $type|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $marker = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0;
 var $vararg_buffer9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $marker = sp + 68|0;
 $1 = $omarker;
 $2 = $type;
 $3 = $cbufptr;
 $4 = $ebufptr;
 $5 = $3;
 $6 = $4;
 $7 = (_getc_ushort($marker,$5,$6)|0);
 $ret = $7;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $ret;
  $0 = $9;
  $72 = $0;
  STACKTOP = sp;return ($72|0);
 }
 $10 = $2;
 do {
  if ((($10|0) == 2)) {
   $17 = HEAP16[$marker>>1]|0;
   $18 = $17&65535;
   $19 = ($18|0)!=(65444);
   if ($19) {
    $20 = HEAP16[$marker>>1]|0;
    $21 = $20&65535;
    $22 = ($21|0)!=(65445);
    if ($22) {
     $23 = HEAP16[$marker>>1]|0;
     $24 = $23&65535;
     $25 = ($24|0)!=(65446);
     if ($25) {
      $26 = HEAP16[$marker>>1]|0;
      $27 = $26&65535;
      $28 = ($27|0)!=(65442);
      if ($28) {
       $29 = HEAP16[$marker>>1]|0;
       $30 = $29&65535;
       $31 = ($30|0)!=(65448);
       if ($31) {
        $32 = HEAP32[_stderr>>2]|0;
        (_fprintf(($32|0),(40072|0),($vararg_buffer1|0))|0);
        $0 = -89;
        $72 = $0;
        STACKTOP = sp;return ($72|0);
       }
      }
     }
    }
   }
  } else if ((($10|0) == 4)) {
   $33 = HEAP16[$marker>>1]|0;
   $34 = $33&65535;
   $35 = ($34|0)!=(65444);
   if ($35) {
    $36 = HEAP16[$marker>>1]|0;
    $37 = $36&65535;
    $38 = ($37|0)!=(65445);
    if ($38) {
     $39 = HEAP16[$marker>>1]|0;
     $40 = $39&65535;
     $41 = ($40|0)!=(65446);
     if ($41) {
      $42 = HEAP16[$marker>>1]|0;
      $43 = $42&65535;
      $44 = ($43|0)!=(65443);
      if ($44) {
       $45 = HEAP16[$marker>>1]|0;
       $46 = $45&65535;
       $47 = ($46|0)!=(65448);
       if ($47) {
        $48 = HEAP32[_stderr>>2]|0;
        $49 = HEAP16[$marker>>1]|0;
        $50 = $49&65535;
        HEAP32[$vararg_buffer3>>2] = $50;
        (_fprintf(($48|0),(40136|0),($vararg_buffer3|0))|0);
        $0 = -90;
        $72 = $0;
        STACKTOP = sp;return ($72|0);
       }
      }
     }
    }
   }
  } else if ((($10|0) == 65535)) {
   $51 = HEAP16[$marker>>1]|0;
   $52 = $51&65535;
   $53 = $52 & 65280;
   $54 = ($53|0)!=(65280);
   if ($54) {
    $55 = HEAP32[_stderr>>2]|0;
    $56 = HEAP16[$marker>>1]|0;
    $57 = $56&65535;
    HEAP32[$vararg_buffer6>>2] = $57;
    (_fprintf(($55|0),(40208|0),($vararg_buffer6|0))|0);
    $0 = -91;
    $72 = $0;
    STACKTOP = sp;return ($72|0);
   }
   $58 = HEAP16[$marker>>1]|0;
   $59 = $58&65535;
   $60 = ($59|0)<(65440);
   if (!($60)) {
    $61 = HEAP16[$marker>>1]|0;
    $62 = $61&65535;
    $63 = ($62|0)>(65448);
    if (!($63)) {
     break;
    }
   }
   $64 = HEAP32[_stderr>>2]|0;
   $65 = HEAP16[$marker>>1]|0;
   $66 = $65&65535;
   HEAP32[$vararg_buffer9>>2] = $66;
   (_fprintf(($64|0),(40264|0),($vararg_buffer9|0))|0);
   $0 = -92;
   $72 = $0;
   STACKTOP = sp;return ($72|0);
  } else if ((($10|0) == 65440)) {
   $11 = HEAP16[$marker>>1]|0;
   $12 = $11&65535;
   $13 = ($12|0)!=(65440);
   if (!($13)) {
    break;
   }
   $14 = HEAP32[_stderr>>2]|0;
   $15 = HEAP16[$marker>>1]|0;
   $16 = $15&65535;
   HEAP32[$vararg_buffer>>2] = $16;
   (_fprintf(($14|0),(40016|0),($vararg_buffer|0))|0);
   $0 = -88;
   $72 = $0;
   STACKTOP = sp;return ($72|0);
  } else {
   $67 = HEAP32[_stderr>>2]|0;
   $68 = HEAP16[$marker>>1]|0;
   $69 = $68&65535;
   HEAP32[$vararg_buffer12>>2] = $69;
   (_fprintf(($67|0),(40320|0),($vararg_buffer12|0))|0);
   $0 = -93;
   $72 = $0;
   STACKTOP = sp;return ($72|0);
  }
 } while(0);
 $70 = HEAP16[$marker>>1]|0;
 $71 = $1;
 HEAP16[$71>>1] = $70;
 $0 = 0;
 $72 = $0;
 STACKTOP = sp;return ($72|0);
}
function _getc_table_wsq($marker,$dtt_table,$dqt_table,$dht_table,$cbufptr,$ebufptr) {
 $marker = $marker|0;
 $dtt_table = $dtt_table|0;
 $dqt_table = $dqt_table|0;
 $dht_table = $dht_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $comment = 0, $ret = 0, $vararg_buffer = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $comment = sp + 4|0;
 $1 = $marker;
 $2 = $dtt_table;
 $3 = $dqt_table;
 $4 = $dht_table;
 $5 = $cbufptr;
 $6 = $ebufptr;
 $7 = $1;
 $8 = $7&65535;
 do {
  if ((($8|0) == 65445)) {
   $15 = $3;
   $16 = $5;
   $17 = $6;
   $18 = (_getc_quantization_table($15,$16,$17)|0);
   $ret = $18;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = $ret;
    $0 = $20;
    break;
   } else {
    label = 15;
    break;
   }
  } else if ((($8|0) == 65448)) {
   $27 = $5;
   $28 = $6;
   $29 = (_getc_comment($comment,$27,$28)|0);
   $ret = $29;
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = $ret;
    $0 = $31;
    break;
   } else {
    $32 = HEAP32[$comment>>2]|0;
    _free($32);
    label = 15;
    break;
   }
  } else if ((($8|0) == 65446)) {
   $21 = $4;
   $22 = $5;
   $23 = $6;
   $24 = (_getc_huffman_table_wsq($21,$22,$23)|0);
   $ret = $24;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $ret;
    $0 = $26;
    break;
   } else {
    label = 15;
    break;
   }
  } else if ((($8|0) == 65444)) {
   $9 = $2;
   $10 = $5;
   $11 = $6;
   $12 = (_getc_transform_table($9,$10,$11)|0);
   $ret = $12;
   $13 = ($12|0)!=(0);
   if ($13) {
    $14 = $ret;
    $0 = $14;
    break;
   } else {
    label = 15;
    break;
   }
  } else {
   $33 = HEAP32[_stderr>>2]|0;
   $34 = $1;
   $35 = $34&65535;
   HEAP32[$vararg_buffer>>2] = $35;
   (_fprintf(($33|0),(40376|0),($vararg_buffer|0))|0);
   $0 = -93;
  }
 } while(0);
 if ((label|0) == 15) {
  $0 = 0;
 }
 $36 = $0;
 STACKTOP = sp;return ($36|0);
}
function _getc_transform_table($dtt_table,$cbufptr,$ebufptr) {
 $dtt_table = $dtt_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0.0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0.0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0.0, $171 = 0.0, $172 = 0.0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0, $192 = 0, $193 = 0, $194 = 0.0, $195 = 0.0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0.0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0.0, $226 = 0, $227 = 0, $228 = 0, $229 = 0.0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0.0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0.0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0.0, $345 = 0.0, $346 = 0.0, $347 = 0.0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0.0, $357 = 0.0, $358 = 0.0, $359 = 0.0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0.0, $368 = 0.0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0.0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0.0, $382 = 0.0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0.0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0.0, $411 = 0, $412 = 0, $413 = 0, $414 = 0.0, $415 = 0.0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_hifilt = 0, $a_lofilt = 0, $a_size = 0, $cnt = 0, $hdr_size = 0, $ret = 0, $scale = 0, $shrt_dat = 0, $sign = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer18 = 0, $vararg_buffer22 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_buffer8 = 0, $vararg_ptr15 = 0, $vararg_ptr21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer22 = sp + 88|0;
 $vararg_buffer18 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $hdr_size = sp + 128|0;
 $shrt_dat = sp + 92|0;
 $scale = sp + 131|0;
 $sign = sp + 130|0;
 $1 = $dtt_table;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(40432|0),($vararg_buffer|0))|0);
 }
 $7 = $2;
 $8 = $3;
 $9 = (_getc_ushort($hdr_size,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 $12 = $1;
 $13 = (($12) + 9|0);
 $14 = $2;
 $15 = $3;
 $16 = (_getc_byte($13,$14,$15)|0);
 $ret = $16;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = $ret;
  $0 = $18;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 $19 = $1;
 $20 = (($19) + 8|0);
 $21 = $2;
 $22 = $3;
 $23 = (_getc_byte($20,$21,$22)|0);
 $ret = $23;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = $ret;
  $0 = $25;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 $26 = HEAP32[8>>2]|0;
 $27 = ($26|0)>(2);
 if ($27) {
  $28 = HEAP32[_stderr>>2]|0;
  $29 = $1;
  $30 = (($29) + 8|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  HEAP32[$vararg_buffer1>>2] = $32;
  (_fprintf(($28|0),(40464|0),($vararg_buffer1|0))|0);
  $33 = HEAP32[_stderr>>2]|0;
  $34 = $1;
  $35 = (($34) + 9|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  HEAP32[$vararg_buffer3>>2] = $37;
  (_fprintf(($33|0),(40480|0),($vararg_buffer3|0))|0);
 }
 $38 = $1;
 $39 = HEAP32[$38>>2]|0;
 $40 = ($39|0)!=(0|0);
 if ($40) {
  $41 = $1;
  $42 = HEAP32[$41>>2]|0;
  _free($42);
  $43 = $1;
  HEAP32[$43>>2] = 0;
 }
 $44 = $1;
 $45 = (($44) + 8|0);
 $46 = HEAP8[$45>>0]|0;
 $47 = $46&255;
 $48 = (_calloc($47,4)|0);
 $49 = $1;
 HEAP32[$49>>2] = $48;
 $50 = $1;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)==(0|0);
 if ($52) {
  $53 = HEAP32[_stderr>>2]|0;
  (_fprintf(($53|0),(40584|0),($vararg_buffer6|0))|0);
  $0 = -94;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 $54 = $1;
 $55 = (($54) + 4|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = ($56|0)!=(0|0);
 if ($57) {
  $58 = $1;
  $59 = (($58) + 4|0);
  $60 = HEAP32[$59>>2]|0;
  _free($60);
  $61 = $1;
  $62 = (($61) + 4|0);
  HEAP32[$62>>2] = 0;
 }
 $63 = $1;
 $64 = (($63) + 9|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = (_calloc($66,4)|0);
 $68 = $1;
 $69 = (($68) + 4|0);
 HEAP32[$69>>2] = $67;
 $70 = $1;
 $71 = (($70) + 4|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)==(0|0);
 if ($73) {
  $74 = $1;
  $75 = HEAP32[$74>>2]|0;
  _free($75);
  $76 = HEAP32[_stderr>>2]|0;
  (_fprintf(($76|0),(40632|0),($vararg_buffer8|0))|0);
  $0 = -95;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 $77 = $1;
 $78 = (($77) + 9|0);
 $79 = HEAP8[$78>>0]|0;
 $80 = $79&255;
 $81 = (($80|0) % 2)&-1;
 $82 = ($81|0)!=(0);
 if ($82) {
  $83 = $1;
  $84 = (($83) + 9|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = $85&255;
  $87 = (($86) + 1)|0;
  $88 = (($87|0) / 2)&-1;
  $89 = $88&255;
  $a_size = $89;
 } else {
  $90 = $1;
  $91 = (($90) + 9|0);
  $92 = HEAP8[$91>>0]|0;
  $93 = $92&255;
  $94 = (($93|0) / 2)&-1;
  $95 = $94&255;
  $a_size = $95;
 }
 $96 = $a_size;
 $97 = $96&255;
 $98 = (_calloc($97,4)|0);
 $a_lofilt = $98;
 $99 = $a_lofilt;
 $100 = ($99|0)==(0|0);
 if ($100) {
  $101 = $1;
  $102 = HEAP32[$101>>2]|0;
  _free($102);
  $103 = $1;
  $104 = (($103) + 4|0);
  $105 = HEAP32[$104>>2]|0;
  _free($105);
  $106 = HEAP32[_stderr>>2]|0;
  (_fprintf(($106|0),(40680|0),($vararg_buffer10|0))|0);
  $0 = -96;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 $107 = $a_size;
 $108 = (($107) + -1)<<24>>24;
 $a_size = $108;
 $cnt = 0;
 while(1) {
  $109 = $cnt;
  $110 = $a_size;
  $111 = $110&255;
  $112 = ($109>>>0)<=($111>>>0);
  if (!($112)) {
   label = 46;
   break;
  }
  $113 = $2;
  $114 = $3;
  $115 = (_getc_byte($sign,$113,$114)|0);
  $ret = $115;
  $116 = ($115|0)!=(0);
  if ($116) {
   label = 27;
   break;
  }
  $124 = $2;
  $125 = $3;
  $126 = (_getc_byte($scale,$124,$125)|0);
  $ret = $126;
  $127 = ($126|0)!=(0);
  if ($127) {
   label = 29;
   break;
  }
  $135 = $2;
  $136 = $3;
  $137 = (_getc_uint($shrt_dat,$135,$136)|0);
  $ret = $137;
  $138 = ($137|0)!=(0);
  if ($138) {
   label = 31;
   break;
  }
  $146 = HEAP32[$shrt_dat>>2]|0;
  $147 = (+($146>>>0));
  $148 = $cnt;
  $149 = $a_lofilt;
  $150 = (($149) + ($148<<2)|0);
  HEAPF32[$150>>2] = $147;
  while(1) {
   $151 = HEAP8[$scale>>0]|0;
   $152 = $151&255;
   $153 = ($152|0)>(0);
   if (!($153)) {
    break;
   }
   $154 = $cnt;
   $155 = $a_lofilt;
   $156 = (($155) + ($154<<2)|0);
   $157 = +HEAPF32[$156>>2];
   $158 = $157;
   $159 = $158 / 10.0;
   $160 = $159;
   HEAPF32[$156>>2] = $160;
   $161 = HEAP8[$scale>>0]|0;
   $162 = (($161) + -1)<<24>>24;
   HEAP8[$scale>>0] = $162;
  }
  $163 = HEAP8[$sign>>0]|0;
  $164 = $163&255;
  $165 = ($164|0)!=(0);
  if ($165) {
   $166 = $cnt;
   $167 = $a_lofilt;
   $168 = (($167) + ($166<<2)|0);
   $169 = +HEAPF32[$168>>2];
   $170 = $169;
   $171 = $170 * -1.0;
   $172 = $171;
   HEAPF32[$168>>2] = $172;
  }
  $173 = HEAP32[8>>2]|0;
  $174 = ($173|0)>(3);
  if ($174) {
   $175 = HEAP32[_stderr>>2]|0;
   $176 = $cnt;
   $177 = $cnt;
   $178 = $a_lofilt;
   $179 = (($178) + ($177<<2)|0);
   $180 = +HEAPF32[$179>>2];
   $181 = $180;
   HEAP32[$vararg_buffer12>>2] = $176;
   $vararg_ptr15 = (($vararg_buffer12) + 4|0);
   HEAPF64[tempDoublePtr>>3]=$181;HEAP32[$vararg_ptr15>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr15+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_fprintf(($175|0),(40496|0),($vararg_buffer12|0))|0);
  }
  $182 = $1;
  $183 = (($182) + 9|0);
  $184 = HEAP8[$183>>0]|0;
  $185 = $184&255;
  $186 = (($185|0) % 2)&-1;
  $187 = ($186|0)!=(0);
  if ($187) {
   $188 = $cnt;
   $189 = (_int_sign($188)|0);
   $190 = (+($189|0));
   $191 = $cnt;
   $192 = $a_lofilt;
   $193 = (($192) + ($191<<2)|0);
   $194 = +HEAPF32[$193>>2];
   $195 = $190 * $194;
   $196 = $cnt;
   $197 = $a_size;
   $198 = $197&255;
   $199 = (($196) + ($198))|0;
   $200 = $1;
   $201 = (($200) + 4|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (($202) + ($199<<2)|0);
   HEAPF32[$203>>2] = $195;
   $204 = $cnt;
   $205 = ($204>>>0)>(0);
   if ($205) {
    $206 = $cnt;
    $207 = $a_size;
    $208 = $207&255;
    $209 = (($206) + ($208))|0;
    $210 = $1;
    $211 = (($210) + 4|0);
    $212 = HEAP32[$211>>2]|0;
    $213 = (($212) + ($209<<2)|0);
    $214 = +HEAPF32[$213>>2];
    $215 = $a_size;
    $216 = $215&255;
    $217 = $cnt;
    $218 = (($216) - ($217))|0;
    $219 = $1;
    $220 = (($219) + 4|0);
    $221 = HEAP32[$220>>2]|0;
    $222 = (($221) + ($218<<2)|0);
    HEAPF32[$222>>2] = $214;
   }
  } else {
   $223 = $cnt;
   $224 = (_int_sign($223)|0);
   $225 = (+($224|0));
   $226 = $cnt;
   $227 = $a_lofilt;
   $228 = (($227) + ($226<<2)|0);
   $229 = +HEAPF32[$228>>2];
   $230 = $225 * $229;
   $231 = $cnt;
   $232 = $a_size;
   $233 = $232&255;
   $234 = (($231) + ($233))|0;
   $235 = (($234) + 1)|0;
   $236 = $1;
   $237 = (($236) + 4|0);
   $238 = HEAP32[$237>>2]|0;
   $239 = (($238) + ($235<<2)|0);
   HEAPF32[$239>>2] = $230;
   $240 = $cnt;
   $241 = $a_size;
   $242 = $241&255;
   $243 = (($240) + ($242))|0;
   $244 = (($243) + 1)|0;
   $245 = $1;
   $246 = (($245) + 4|0);
   $247 = HEAP32[$246>>2]|0;
   $248 = (($247) + ($244<<2)|0);
   $249 = +HEAPF32[$248>>2];
   $250 = $249;
   $251 = -1.0 * $250;
   $252 = $251;
   $253 = $a_size;
   $254 = $253&255;
   $255 = $cnt;
   $256 = (($254) - ($255))|0;
   $257 = $1;
   $258 = (($257) + 4|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = (($259) + ($256<<2)|0);
   HEAPF32[$260>>2] = $252;
  }
  $261 = $cnt;
  $262 = (($261) + 1)|0;
  $cnt = $262;
 }
 if ((label|0) == 27) {
  $117 = $1;
  $118 = HEAP32[$117>>2]|0;
  _free($118);
  $119 = $1;
  $120 = (($119) + 4|0);
  $121 = HEAP32[$120>>2]|0;
  _free($121);
  $122 = $a_lofilt;
  _free($122);
  $123 = $ret;
  $0 = $123;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 else if ((label|0) == 29) {
  $128 = $1;
  $129 = HEAP32[$128>>2]|0;
  _free($129);
  $130 = $1;
  $131 = (($130) + 4|0);
  $132 = HEAP32[$131>>2]|0;
  _free($132);
  $133 = $a_lofilt;
  _free($133);
  $134 = $ret;
  $0 = $134;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 else if ((label|0) == 31) {
  $139 = $1;
  $140 = HEAP32[$139>>2]|0;
  _free($140);
  $141 = $1;
  $142 = (($141) + 4|0);
  $143 = HEAP32[$142>>2]|0;
  _free($143);
  $144 = $a_lofilt;
  _free($144);
  $145 = $ret;
  $0 = $145;
  $450 = $0;
  STACKTOP = sp;return ($450|0);
 }
 else if ((label|0) == 46) {
  $263 = $a_lofilt;
  _free($263);
  $264 = $1;
  $265 = (($264) + 8|0);
  $266 = HEAP8[$265>>0]|0;
  $267 = $266&255;
  $268 = (($267|0) % 2)&-1;
  $269 = ($268|0)!=(0);
  if ($269) {
   $270 = $1;
   $271 = (($270) + 8|0);
   $272 = HEAP8[$271>>0]|0;
   $273 = $272&255;
   $274 = (($273) + 1)|0;
   $275 = (($274|0) / 2)&-1;
   $276 = $275&255;
   $a_size = $276;
  } else {
   $277 = $1;
   $278 = (($277) + 8|0);
   $279 = HEAP8[$278>>0]|0;
   $280 = $279&255;
   $281 = (($280|0) / 2)&-1;
   $282 = $281&255;
   $a_size = $282;
  }
  $283 = $a_size;
  $284 = $283&255;
  $285 = (_calloc($284,4)|0);
  $a_hifilt = $285;
  $286 = $a_hifilt;
  $287 = ($286|0)==(0|0);
  if ($287) {
   $288 = $1;
   $289 = HEAP32[$288>>2]|0;
   _free($289);
   $290 = $1;
   $291 = (($290) + 4|0);
   $292 = HEAP32[$291>>2]|0;
   _free($292);
   $293 = HEAP32[_stderr>>2]|0;
   (_fprintf(($293|0),(40736|0),($vararg_buffer16|0))|0);
   $0 = -97;
   $450 = $0;
   STACKTOP = sp;return ($450|0);
  }
  $294 = $a_size;
  $295 = (($294) + -1)<<24>>24;
  $a_size = $295;
  $cnt = 0;
  while(1) {
   $296 = $cnt;
   $297 = $a_size;
   $298 = $297&255;
   $299 = ($296>>>0)<=($298>>>0);
   if (!($299)) {
    label = 73;
    break;
   }
   $300 = $2;
   $301 = $3;
   $302 = (_getc_byte($sign,$300,$301)|0);
   $ret = $302;
   $303 = ($302|0)!=(0);
   if ($303) {
    label = 54;
    break;
   }
   $311 = $2;
   $312 = $3;
   $313 = (_getc_byte($scale,$311,$312)|0);
   $ret = $313;
   $314 = ($313|0)!=(0);
   if ($314) {
    label = 56;
    break;
   }
   $322 = $2;
   $323 = $3;
   $324 = (_getc_uint($shrt_dat,$322,$323)|0);
   $ret = $324;
   $325 = ($324|0)!=(0);
   if ($325) {
    label = 58;
    break;
   }
   $333 = HEAP32[$shrt_dat>>2]|0;
   $334 = (+($333>>>0));
   $335 = $cnt;
   $336 = $a_hifilt;
   $337 = (($336) + ($335<<2)|0);
   HEAPF32[$337>>2] = $334;
   while(1) {
    $338 = HEAP8[$scale>>0]|0;
    $339 = $338&255;
    $340 = ($339|0)>(0);
    if (!($340)) {
     break;
    }
    $341 = $cnt;
    $342 = $a_hifilt;
    $343 = (($342) + ($341<<2)|0);
    $344 = +HEAPF32[$343>>2];
    $345 = $344;
    $346 = $345 / 10.0;
    $347 = $346;
    HEAPF32[$343>>2] = $347;
    $348 = HEAP8[$scale>>0]|0;
    $349 = (($348) + -1)<<24>>24;
    HEAP8[$scale>>0] = $349;
   }
   $350 = HEAP8[$sign>>0]|0;
   $351 = $350&255;
   $352 = ($351|0)!=(0);
   if ($352) {
    $353 = $cnt;
    $354 = $a_hifilt;
    $355 = (($354) + ($353<<2)|0);
    $356 = +HEAPF32[$355>>2];
    $357 = $356;
    $358 = $357 * -1.0;
    $359 = $358;
    HEAPF32[$355>>2] = $359;
   }
   $360 = HEAP32[8>>2]|0;
   $361 = ($360|0)>(2);
   if ($361) {
    $362 = HEAP32[_stderr>>2]|0;
    $363 = $cnt;
    $364 = $cnt;
    $365 = $a_hifilt;
    $366 = (($365) + ($364<<2)|0);
    $367 = +HEAPF32[$366>>2];
    $368 = $367;
    HEAP32[$vararg_buffer18>>2] = $363;
    $vararg_ptr21 = (($vararg_buffer18) + 4|0);
    HEAPF64[tempDoublePtr>>3]=$368;HEAP32[$vararg_ptr21>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr21+4>>2]=HEAP32[tempDoublePtr+4>>2];
    (_fprintf(($362|0),(40520|0),($vararg_buffer18|0))|0);
   }
   $369 = $1;
   $370 = (($369) + 8|0);
   $371 = HEAP8[$370>>0]|0;
   $372 = $371&255;
   $373 = (($372|0) % 2)&-1;
   $374 = ($373|0)!=(0);
   if ($374) {
    $375 = $cnt;
    $376 = (_int_sign($375)|0);
    $377 = (+($376|0));
    $378 = $cnt;
    $379 = $a_hifilt;
    $380 = (($379) + ($378<<2)|0);
    $381 = +HEAPF32[$380>>2];
    $382 = $377 * $381;
    $383 = $cnt;
    $384 = $a_size;
    $385 = $384&255;
    $386 = (($383) + ($385))|0;
    $387 = $1;
    $388 = HEAP32[$387>>2]|0;
    $389 = (($388) + ($386<<2)|0);
    HEAPF32[$389>>2] = $382;
    $390 = $cnt;
    $391 = ($390>>>0)>(0);
    if ($391) {
     $392 = $cnt;
     $393 = $a_size;
     $394 = $393&255;
     $395 = (($392) + ($394))|0;
     $396 = $1;
     $397 = HEAP32[$396>>2]|0;
     $398 = (($397) + ($395<<2)|0);
     $399 = +HEAPF32[$398>>2];
     $400 = $a_size;
     $401 = $400&255;
     $402 = $cnt;
     $403 = (($401) - ($402))|0;
     $404 = $1;
     $405 = HEAP32[$404>>2]|0;
     $406 = (($405) + ($403<<2)|0);
     HEAPF32[$406>>2] = $399;
    }
   } else {
    $407 = $cnt;
    $408 = (($407) + 1)|0;
    $409 = (_int_sign($408)|0);
    $410 = (+($409|0));
    $411 = $cnt;
    $412 = $a_hifilt;
    $413 = (($412) + ($411<<2)|0);
    $414 = +HEAPF32[$413>>2];
    $415 = $410 * $414;
    $416 = $cnt;
    $417 = $a_size;
    $418 = $417&255;
    $419 = (($416) + ($418))|0;
    $420 = (($419) + 1)|0;
    $421 = $1;
    $422 = HEAP32[$421>>2]|0;
    $423 = (($422) + ($420<<2)|0);
    HEAPF32[$423>>2] = $415;
    $424 = $cnt;
    $425 = $a_size;
    $426 = $425&255;
    $427 = (($424) + ($426))|0;
    $428 = (($427) + 1)|0;
    $429 = $1;
    $430 = HEAP32[$429>>2]|0;
    $431 = (($430) + ($428<<2)|0);
    $432 = +HEAPF32[$431>>2];
    $433 = $a_size;
    $434 = $433&255;
    $435 = $cnt;
    $436 = (($434) - ($435))|0;
    $437 = $1;
    $438 = HEAP32[$437>>2]|0;
    $439 = (($438) + ($436<<2)|0);
    HEAPF32[$439>>2] = $432;
   }
   $440 = $cnt;
   $441 = (($440) + 1)|0;
   $cnt = $441;
  }
  if ((label|0) == 54) {
   $304 = $1;
   $305 = HEAP32[$304>>2]|0;
   _free($305);
   $306 = $1;
   $307 = (($306) + 4|0);
   $308 = HEAP32[$307>>2]|0;
   _free($308);
   $309 = $a_hifilt;
   _free($309);
   $310 = $ret;
   $0 = $310;
   $450 = $0;
   STACKTOP = sp;return ($450|0);
  }
  else if ((label|0) == 56) {
   $315 = $1;
   $316 = HEAP32[$315>>2]|0;
   _free($316);
   $317 = $1;
   $318 = (($317) + 4|0);
   $319 = HEAP32[$318>>2]|0;
   _free($319);
   $320 = $a_hifilt;
   _free($320);
   $321 = $ret;
   $0 = $321;
   $450 = $0;
   STACKTOP = sp;return ($450|0);
  }
  else if ((label|0) == 58) {
   $326 = $1;
   $327 = HEAP32[$326>>2]|0;
   _free($327);
   $328 = $1;
   $329 = (($328) + 4|0);
   $330 = HEAP32[$329>>2]|0;
   _free($330);
   $331 = $a_hifilt;
   _free($331);
   $332 = $ret;
   $0 = $332;
   $450 = $0;
   STACKTOP = sp;return ($450|0);
  }
  else if ((label|0) == 73) {
   $442 = $a_hifilt;
   _free($442);
   $443 = $1;
   $444 = (($443) + 10|0);
   HEAP8[$444>>0] = 1;
   $445 = $1;
   $446 = (($445) + 11|0);
   HEAP8[$446>>0] = 1;
   $447 = HEAP32[8>>2]|0;
   $448 = ($447|0)>(0);
   if ($448) {
    $449 = HEAP32[_stderr>>2]|0;
    (_fprintf(($449|0),(40544|0),($vararg_buffer22|0))|0);
   }
   $0 = 0;
   $450 = $0;
   STACKTOP = sp;return ($450|0);
  }
 }
 return 0|0;
}
function _getc_quantization_table($dqt_table,$cbufptr,$ebufptr) {
 $dqt_table = $dqt_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0.0, $95 = 0.0, $96 = 0.0, $97 = 0.0, $98 = 0, $99 = 0, $cnt = 0, $hdr_size = 0, $ret = 0, $scale = 0, $shrt_dat = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer6 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $hdr_size = sp + 60|0;
 $shrt_dat = sp + 56|0;
 $scale = sp + 62|0;
 $1 = $dqt_table;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(40792|0),($vararg_buffer|0))|0);
 }
 $7 = $2;
 $8 = $3;
 $9 = (_getc_ushort($hdr_size,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $12 = $2;
 $13 = $3;
 $14 = (_getc_byte($scale,$12,$13)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $ret;
  $0 = $16;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $17 = $2;
 $18 = $3;
 $19 = (_getc_ushort($shrt_dat,$17,$18)|0);
 $ret = $19;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $ret;
  $0 = $21;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $22 = HEAP16[$shrt_dat>>1]|0;
 $23 = (+($22&65535));
 $24 = $1;
 HEAPF32[$24>>2] = $23;
 while(1) {
  $25 = HEAP8[$scale>>0]|0;
  $26 = $25&255;
  $27 = ($26|0)>(0);
  if (!($27)) {
   break;
  }
  $28 = $1;
  $29 = +HEAPF32[$28>>2];
  $30 = $29;
  $31 = $30 / 10.0;
  $32 = $31;
  HEAPF32[$28>>2] = $32;
  $33 = HEAP8[$scale>>0]|0;
  $34 = (($33) + -1)<<24>>24;
  HEAP8[$scale>>0] = $34;
 }
 $cnt = 0;
 while(1) {
  $35 = $cnt;
  $36 = $35&65535;
  $37 = ($36|0)<(64);
  if (!($37)) {
   label = 32;
   break;
  }
  $38 = $2;
  $39 = $3;
  $40 = (_getc_byte($scale,$38,$39)|0);
  $ret = $40;
  $41 = ($40|0)!=(0);
  if ($41) {
   label = 15;
   break;
  }
  $43 = $2;
  $44 = $3;
  $45 = (_getc_ushort($shrt_dat,$43,$44)|0);
  $ret = $45;
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 17;
   break;
  }
  $48 = HEAP16[$shrt_dat>>1]|0;
  $49 = (+($48&65535));
  $50 = $cnt;
  $51 = $50&65535;
  $52 = $1;
  $53 = (($52) + 4|0);
  $54 = (($53) + ($51<<2)|0);
  HEAPF32[$54>>2] = $49;
  while(1) {
   $55 = HEAP8[$scale>>0]|0;
   $56 = $55&255;
   $57 = ($56|0)>(0);
   if (!($57)) {
    break;
   }
   $58 = $cnt;
   $59 = $58&65535;
   $60 = $1;
   $61 = (($60) + 4|0);
   $62 = (($61) + ($59<<2)|0);
   $63 = +HEAPF32[$62>>2];
   $64 = $63;
   $65 = $64 / 10.0;
   $66 = $65;
   HEAPF32[$62>>2] = $66;
   $67 = HEAP8[$scale>>0]|0;
   $68 = (($67) + -1)<<24>>24;
   HEAP8[$scale>>0] = $68;
  }
  $69 = $2;
  $70 = $3;
  $71 = (_getc_byte($scale,$69,$70)|0);
  $ret = $71;
  $72 = ($71|0)!=(0);
  if ($72) {
   label = 22;
   break;
  }
  $74 = $2;
  $75 = $3;
  $76 = (_getc_ushort($shrt_dat,$74,$75)|0);
  $ret = $76;
  $77 = ($76|0)!=(0);
  if ($77) {
   label = 24;
   break;
  }
  $79 = HEAP16[$shrt_dat>>1]|0;
  $80 = (+($79&65535));
  $81 = $cnt;
  $82 = $81&65535;
  $83 = $1;
  $84 = (($83) + 260|0);
  $85 = (($84) + ($82<<2)|0);
  HEAPF32[$85>>2] = $80;
  while(1) {
   $86 = HEAP8[$scale>>0]|0;
   $87 = $86&255;
   $88 = ($87|0)>(0);
   if (!($88)) {
    break;
   }
   $89 = $cnt;
   $90 = $89&65535;
   $91 = $1;
   $92 = (($91) + 260|0);
   $93 = (($92) + ($90<<2)|0);
   $94 = +HEAPF32[$93>>2];
   $95 = $94;
   $96 = $95 / 10.0;
   $97 = $96;
   HEAPF32[$93>>2] = $97;
   $98 = HEAP8[$scale>>0]|0;
   $99 = (($98) + -1)<<24>>24;
   HEAP8[$scale>>0] = $99;
  }
  $100 = HEAP32[8>>2]|0;
  $101 = ($100|0)>(2);
  if ($101) {
   $102 = HEAP32[_stderr>>2]|0;
   $103 = $cnt;
   $104 = $103&65535;
   $105 = $cnt;
   $106 = $105&65535;
   $107 = $1;
   $108 = (($107) + 4|0);
   $109 = (($108) + ($106<<2)|0);
   $110 = +HEAPF32[$109>>2];
   $111 = $110;
   $112 = $cnt;
   $113 = $112&65535;
   $114 = $cnt;
   $115 = $114&65535;
   $116 = $1;
   $117 = (($116) + 260|0);
   $118 = (($117) + ($115<<2)|0);
   $119 = +HEAPF32[$118>>2];
   $120 = $119;
   HEAP32[$vararg_buffer1>>2] = $104;
   $vararg_ptr3 = (($vararg_buffer1) + 4|0);
   HEAPF64[tempDoublePtr>>3]=$111;HEAP32[$vararg_ptr3>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr3+4>>2]=HEAP32[tempDoublePtr+4>>2];
   $vararg_ptr4 = (($vararg_buffer1) + 12|0);
   HEAP32[$vararg_ptr4>>2] = $113;
   $vararg_ptr5 = (($vararg_buffer1) + 16|0);
   HEAPF64[tempDoublePtr>>3]=$120;HEAP32[$vararg_ptr5>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr5+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_fprintf(($102|0),(40824|0),($vararg_buffer1|0))|0);
  }
  $121 = $cnt;
  $122 = (($121) + 1)<<16>>16;
  $cnt = $122;
 }
 if ((label|0) == 15) {
  $42 = $ret;
  $0 = $42;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 else if ((label|0) == 17) {
  $47 = $ret;
  $0 = $47;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 else if ((label|0) == 22) {
  $73 = $ret;
  $0 = $73;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 else if ((label|0) == 24) {
  $78 = $ret;
  $0 = $78;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 else if ((label|0) == 32) {
  $123 = $1;
  $124 = (($123) + 516|0);
  HEAP8[$124>>0] = 1;
  $125 = HEAP32[8>>2]|0;
  $126 = ($125|0)>(0);
  if ($126) {
   $127 = HEAP32[_stderr>>2]|0;
   (_fprintf(($127|0),(40856|0),($vararg_buffer6|0))|0);
  }
  $0 = 0;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 return 0|0;
}
function _getc_huffman_table_wsq($dht_table,$cbufptr,$ebufptr) {
 $dht_table = $dht_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $bytes_left = 0, $huffbits = 0, $huffvalues = 0, $ret = 0, $table_id = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $table_id = sp + 44|0;
 $huffbits = sp + 20|0;
 $huffvalues = sp + 16|0;
 $bytes_left = sp + 12|0;
 $1 = $dht_table;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = $2;
 $5 = $3;
 $6 = (_getc_huffman_table($table_id,$huffbits,$huffvalues,256,$4,$5,1,$bytes_left)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 $9 = $1;
 $10 = HEAP8[$table_id>>0]|0;
 $11 = $10&255;
 $12 = (($9) + (($11*274)|0)|0);
 $13 = (($12) + 1|0);
 $14 = HEAP32[$huffbits>>2]|0;
 dest=$13+0|0; src=$14+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $15 = $1;
 $16 = HEAP8[$table_id>>0]|0;
 $17 = $16&255;
 $18 = (($15) + (($17*274)|0)|0);
 $19 = (($18) + 17|0);
 $20 = HEAP32[$huffvalues>>2]|0;
 _memcpy(($19|0),($20|0),257)|0;
 $21 = $1;
 $22 = HEAP8[$table_id>>0]|0;
 $23 = $22&255;
 $24 = (($21) + (($23*274)|0)|0);
 HEAP8[$24>>0] = 1;
 $25 = HEAP32[$huffbits>>2]|0;
 _free($25);
 $26 = HEAP32[$huffvalues>>2]|0;
 _free($26);
 while(1) {
  $27 = HEAP32[$bytes_left>>2]|0;
  $28 = ($27|0)!=(0);
  if (!($28)) {
   label = 10;
   break;
  }
  $29 = $2;
  $30 = $3;
  $31 = (_getc_huffman_table($table_id,$huffbits,$huffvalues,256,$29,$30,0,$bytes_left)|0);
  $ret = $31;
  $32 = ($31|0)!=(0);
  if ($32) {
   label = 6;
   break;
  }
  $34 = $1;
  $35 = HEAP8[$table_id>>0]|0;
  $36 = $35&255;
  $37 = (($34) + (($36*274)|0)|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = ($38<<24>>24)!=(0);
  if ($39) {
   label = 8;
   break;
  }
  $46 = $1;
  $47 = HEAP8[$table_id>>0]|0;
  $48 = $47&255;
  $49 = (($46) + (($48*274)|0)|0);
  $50 = (($49) + 1|0);
  $51 = HEAP32[$huffbits>>2]|0;
  dest=$50+0|0; src=$51+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $52 = $1;
  $53 = HEAP8[$table_id>>0]|0;
  $54 = $53&255;
  $55 = (($52) + (($54*274)|0)|0);
  $56 = (($55) + 17|0);
  $57 = HEAP32[$huffvalues>>2]|0;
  _memcpy(($56|0),($57|0),257)|0;
  $58 = $1;
  $59 = HEAP8[$table_id>>0]|0;
  $60 = $59&255;
  $61 = (($58) + (($60*274)|0)|0);
  HEAP8[$61>>0] = 1;
  $62 = HEAP32[$huffbits>>2]|0;
  _free($62);
  $63 = HEAP32[$huffvalues>>2]|0;
  _free($63);
 }
 if ((label|0) == 6) {
  $33 = $ret;
  $0 = $33;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 else if ((label|0) == 8) {
  $40 = HEAP32[$huffbits>>2]|0;
  _free($40);
  $41 = HEAP32[$huffvalues>>2]|0;
  _free($41);
  $42 = HEAP32[_stderr>>2]|0;
  (_fprintf(($42|0),(40936|0),($vararg_buffer|0))|0);
  $43 = HEAP32[_stderr>>2]|0;
  $44 = HEAP8[$table_id>>0]|0;
  $45 = $44&255;
  HEAP32[$vararg_buffer1>>2] = $45;
  (_fprintf(($43|0),(40896|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 else if ((label|0) == 10) {
  $0 = 0;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 return 0|0;
}
function _getc_frame_header_wsq($frm_header,$cbufptr,$ebufptr) {
 $frm_header = $frm_header|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0.0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0;
 var $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $hdr_size = 0, $ret = 0, $scale = 0, $shrt_dat = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer15 = 0, $vararg_buffer18 = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer18 = sp + 56|0;
 $vararg_buffer15 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $hdr_size = sp + 82|0;
 $shrt_dat = sp + 80|0;
 $scale = sp + 84|0;
 $1 = $frm_header;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(40976|0),($vararg_buffer|0))|0);
 }
 $7 = $2;
 $8 = $3;
 $9 = (_getc_ushort($hdr_size,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $12 = $1;
 $13 = $2;
 $14 = $3;
 $15 = (_getc_byte($12,$13,$14)|0);
 $ret = $15;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = $ret;
  $0 = $17;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $18 = $1;
 $19 = (($18) + 1|0);
 $20 = $2;
 $21 = $3;
 $22 = (_getc_byte($19,$20,$21)|0);
 $ret = $22;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = $ret;
  $0 = $24;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $25 = $1;
 $26 = (($25) + 4|0);
 $27 = $2;
 $28 = $3;
 $29 = (_getc_ushort($26,$27,$28)|0);
 $ret = $29;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = $ret;
  $0 = $31;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $32 = $1;
 $33 = (($32) + 2|0);
 $34 = $2;
 $35 = $3;
 $36 = (_getc_ushort($33,$34,$35)|0);
 $ret = $36;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = $ret;
  $0 = $38;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $39 = $2;
 $40 = $3;
 $41 = (_getc_byte($scale,$39,$40)|0);
 $ret = $41;
 $42 = ($41|0)!=(0);
 if ($42) {
  $43 = $ret;
  $0 = $43;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $44 = $2;
 $45 = $3;
 $46 = (_getc_ushort($shrt_dat,$44,$45)|0);
 $ret = $46;
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = $ret;
  $0 = $48;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $49 = HEAP16[$shrt_dat>>1]|0;
 $50 = (+($49&65535));
 $51 = $1;
 $52 = (($51) + 8|0);
 HEAPF32[$52>>2] = $50;
 while(1) {
  $53 = HEAP8[$scale>>0]|0;
  $54 = $53&255;
  $55 = ($54|0)>(0);
  if (!($55)) {
   break;
  }
  $56 = $1;
  $57 = (($56) + 8|0);
  $58 = +HEAPF32[$57>>2];
  $59 = $58;
  $60 = $59 / 10.0;
  $61 = $60;
  HEAPF32[$57>>2] = $61;
  $62 = HEAP8[$scale>>0]|0;
  $63 = (($62) + -1)<<24>>24;
  HEAP8[$scale>>0] = $63;
 }
 $64 = $2;
 $65 = $3;
 $66 = (_getc_byte($scale,$64,$65)|0);
 $ret = $66;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = $ret;
  $0 = $68;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $69 = $2;
 $70 = $3;
 $71 = (_getc_ushort($shrt_dat,$69,$70)|0);
 $ret = $71;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = $ret;
  $0 = $73;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $74 = HEAP16[$shrt_dat>>1]|0;
 $75 = (+($74&65535));
 $76 = $1;
 $77 = (($76) + 12|0);
 HEAPF32[$77>>2] = $75;
 while(1) {
  $78 = HEAP8[$scale>>0]|0;
  $79 = $78&255;
  $80 = ($79|0)>(0);
  if (!($80)) {
   break;
  }
  $81 = $1;
  $82 = (($81) + 12|0);
  $83 = +HEAPF32[$82>>2];
  $84 = $83;
  $85 = $84 / 10.0;
  $86 = $85;
  HEAPF32[$82>>2] = $86;
  $87 = HEAP8[$scale>>0]|0;
  $88 = (($87) + -1)<<24>>24;
  HEAP8[$scale>>0] = $88;
 }
 $89 = $1;
 $90 = (($89) + 16|0);
 $91 = $2;
 $92 = $3;
 $93 = (_getc_byte($90,$91,$92)|0);
 $ret = $93;
 $94 = ($93|0)!=(0);
 if ($94) {
  $95 = $ret;
  $0 = $95;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $96 = $1;
 $97 = (($96) + 18|0);
 $98 = $2;
 $99 = $3;
 $100 = (_getc_ushort($97,$98,$99)|0);
 $ret = $100;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = $ret;
  $0 = $102;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $103 = HEAP32[8>>2]|0;
 $104 = ($103|0)>(2);
 if ($104) {
  $105 = HEAP32[_stderr>>2]|0;
  $106 = $1;
  $107 = HEAP8[$106>>0]|0;
  $108 = $107&255;
  $109 = $1;
  $110 = (($109) + 1|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = $111&255;
  HEAP32[$vararg_buffer1>>2] = $108;
  $vararg_ptr3 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr3>>2] = $112;
  (_fprintf(($105|0),(41000|0),($vararg_buffer1|0))|0);
  $113 = HEAP32[_stderr>>2]|0;
  $114 = $1;
  $115 = (($114) + 2|0);
  $116 = HEAP16[$115>>1]|0;
  $117 = $116&65535;
  $118 = $1;
  $119 = (($118) + 4|0);
  $120 = HEAP16[$119>>1]|0;
  $121 = $120&65535;
  HEAP32[$vararg_buffer4>>2] = $117;
  $vararg_ptr7 = (($vararg_buffer4) + 4|0);
  HEAP32[$vararg_ptr7>>2] = $121;
  (_fprintf(($113|0),(41032|0),($vararg_buffer4|0))|0);
  $122 = HEAP32[_stderr>>2]|0;
  $123 = $1;
  $124 = (($123) + 8|0);
  $125 = +HEAPF32[$124>>2];
  $126 = $125;
  $127 = $1;
  $128 = (($127) + 12|0);
  $129 = +HEAPF32[$128>>2];
  $130 = $129;
  HEAPF64[tempDoublePtr>>3]=$126;HEAP32[$vararg_buffer8>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer8+4>>2]=HEAP32[tempDoublePtr+4>>2];
  $vararg_ptr11 = (($vararg_buffer8) + 8|0);
  HEAPF64[tempDoublePtr>>3]=$130;HEAP32[$vararg_ptr11>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr11+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($122|0),(41056|0),($vararg_buffer8|0))|0);
  $131 = HEAP32[_stdout>>2]|0;
  $132 = $1;
  $133 = (($132) + 16|0);
  $134 = HEAP8[$133>>0]|0;
  $135 = $134&255;
  HEAP32[$vararg_buffer12>>2] = $135;
  (_fprintf(($131|0),(41088|0),($vararg_buffer12|0))|0);
  $136 = HEAP32[_stdout>>2]|0;
  $137 = $1;
  $138 = (($137) + 18|0);
  $139 = HEAP16[$138>>1]|0;
  $140 = $139&65535;
  HEAP32[$vararg_buffer15>>2] = $140;
  (_fprintf(($136|0),(41112|0),($vararg_buffer15|0))|0);
 }
 $141 = HEAP32[8>>2]|0;
 $142 = ($141|0)>(0);
 if ($142) {
  $143 = HEAP32[_stderr>>2]|0;
  (_fprintf(($143|0),(41128|0),($vararg_buffer18|0))|0);
 }
 $0 = 0;
 $144 = $0;
 STACKTOP = sp;return ($144|0);
}
function _getc_block_header($huff_table,$cbufptr,$ebufptr) {
 $huff_table = $huff_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $hdr_size = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $hdr_size = sp + 40|0;
 $1 = $huff_table;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(41168|0),($vararg_buffer|0))|0);
 }
 $7 = $2;
 $8 = $3;
 $9 = (_getc_ushort($hdr_size,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 $12 = $1;
 $13 = $2;
 $14 = $3;
 $15 = (_getc_byte($12,$13,$14)|0);
 $ret = $15;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = $ret;
  $0 = $17;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 $18 = HEAP32[8>>2]|0;
 $19 = ($18|0)>(2);
 if ($19) {
  $20 = HEAP32[_stderr>>2]|0;
  $21 = $1;
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  HEAP32[$vararg_buffer1>>2] = $23;
  (_fprintf(($20|0),(41192|0),($vararg_buffer1|0))|0);
 }
 $24 = HEAP32[8>>2]|0;
 $25 = ($24|0)>(0);
 if ($25) {
  $26 = HEAP32[_stderr>>2]|0;
  (_fprintf(($26|0),(41216|0),($vararg_buffer3|0))|0);
 }
 $0 = 0;
 $27 = $0;
 STACKTOP = sp;return ($27|0);
}
function _getc_nistcom_wsq($onistcom,$idata,$ilen) {
 $onistcom = $onistcom|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cbufptr = 0, $comment_text = 0, $ebufptr = 0, $marker = 0, $nistcom = 0, $ret = 0, $ucharptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $marker = sp + 40|0;
 $cbufptr = sp + 16|0;
 $nistcom = sp + 8|0;
 $ucharptr = sp;
 $1 = $onistcom;
 $2 = $idata;
 $3 = $ilen;
 $4 = $2;
 HEAP32[$cbufptr>>2] = $4;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6)|0);
 $ebufptr = $7;
 $8 = $ebufptr;
 $9 = (_getc_marker_wsq($marker,65440,$cbufptr,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 $12 = $ebufptr;
 $13 = (_getc_marker_wsq($marker,65535,$cbufptr,$12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $ret;
  $0 = $15;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 while(1) {
  $16 = HEAP16[$marker>>1]|0;
  $17 = $16&65535;
  $18 = ($17|0)!=(65443);
  if (!($18)) {
   label = 20;
   break;
  }
  $19 = HEAP16[$marker>>1]|0;
  $20 = $19&65535;
  $21 = ($20|0)==(65448);
  if ($21) {
   $22 = HEAP32[$cbufptr>>2]|0;
   $23 = (($22) + 2|0);
   $24 = (_strncmp($23,41256,8)|0);
   $25 = ($24|0)==(0);
   if ($25) {
    label = 9;
    break;
   }
  }
  $39 = HEAP16[$marker>>1]|0;
  $40 = $ebufptr;
  $41 = (_getc_skip_marker_segment($39,$cbufptr,$40)|0);
  $ret = $41;
  $42 = ($41|0)!=(0);
  if ($42) {
   label = 16;
   break;
  }
  $44 = $ebufptr;
  $45 = (_getc_marker_wsq($marker,65535,$cbufptr,$44)|0);
  $ret = $45;
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 18;
   break;
  }
 }
 if ((label|0) == 9) {
  $26 = $ebufptr;
  $27 = (_getc_comment($ucharptr,$cbufptr,$26)|0);
  $ret = $27;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = $ret;
   $0 = $29;
   $49 = $0;
   STACKTOP = sp;return ($49|0);
  }
  $30 = HEAP32[$ucharptr>>2]|0;
  $comment_text = $30;
  $31 = $comment_text;
  $32 = (_string2fet($nistcom,$31)|0);
  $ret = $32;
  $33 = ($32|0)!=(0);
  if ($33) {
   $34 = $comment_text;
   _free($34);
   $35 = $ret;
   $0 = $35;
   $49 = $0;
   STACKTOP = sp;return ($49|0);
  } else {
   $36 = HEAP32[$nistcom>>2]|0;
   $37 = $1;
   HEAP32[$37>>2] = $36;
   $38 = $comment_text;
   _free($38);
   $0 = 0;
   $49 = $0;
   STACKTOP = sp;return ($49|0);
  }
 }
 else if ((label|0) == 16) {
  $43 = $ret;
  $0 = $43;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 else if ((label|0) == 18) {
  $47 = $ret;
  $0 = $47;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 else if ((label|0) == 20) {
  $48 = $1;
  HEAP32[$48>>2] = 0;
  $0 = 0;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 return 0|0;
}
function _build_wsq_trees($w_tree,$w_treelen,$q_tree,$q_treelen,$width,$height) {
 $w_tree = $w_tree|0;
 $w_treelen = $w_treelen|0;
 $q_tree = $q_tree|0;
 $q_treelen = $q_treelen|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $w_tree;
 $1 = $w_treelen;
 $2 = $q_tree;
 $3 = $q_treelen;
 $4 = $width;
 $5 = $height;
 $6 = $0;
 $7 = $4;
 $8 = $5;
 _build_w_tree($6,$7,$8);
 $9 = $0;
 $10 = $2;
 _build_q_tree($9,$10);
 STACKTOP = sp;return;
}
function _build_w_tree($w_tree,$width,$height) {
 $w_tree = $w_tree|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $lenx = 0, $lenx2 = 0, $leny = 0, $leny2 = 0, $node = 0, $vararg_buffer = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer = sp;
 $0 = $w_tree;
 $1 = $width;
 $2 = $height;
 $node = 0;
 while(1) {
  $3 = $node;
  $4 = ($3|0)<(20);
  if (!($4)) {
   break;
  }
  $5 = $node;
  $6 = $0;
  $7 = (($6) + (($5*24)|0)|0);
  $8 = (($7) + 16|0);
  HEAP32[$8>>2] = 0;
  $9 = $node;
  $10 = $0;
  $11 = (($10) + (($9*24)|0)|0);
  $12 = (($11) + 20|0);
  HEAP32[$12>>2] = 0;
  $13 = $node;
  $14 = (($13) + 1)|0;
  $node = $14;
 }
 $15 = $0;
 $16 = (($15) + 48|0);
 $17 = (($16) + 16|0);
 HEAP32[$17>>2] = 1;
 $18 = $0;
 $19 = (($18) + 96|0);
 $20 = (($19) + 16|0);
 HEAP32[$20>>2] = 1;
 $21 = $0;
 $22 = (($21) + 168|0);
 $23 = (($22) + 16|0);
 HEAP32[$23>>2] = 1;
 $24 = $0;
 $25 = (($24) + 216|0);
 $26 = (($25) + 16|0);
 HEAP32[$26>>2] = 1;
 $27 = $0;
 $28 = (($27) + 264|0);
 $29 = (($28) + 16|0);
 HEAP32[$29>>2] = 1;
 $30 = $0;
 $31 = (($30) + 312|0);
 $32 = (($31) + 16|0);
 HEAP32[$32>>2] = 1;
 $33 = $0;
 $34 = (($33) + 384|0);
 $35 = (($34) + 16|0);
 HEAP32[$35>>2] = 1;
 $36 = $0;
 $37 = (($36) + 432|0);
 $38 = (($37) + 16|0);
 HEAP32[$38>>2] = 1;
 $39 = $0;
 $40 = (($39) + 72|0);
 $41 = (($40) + 20|0);
 HEAP32[$41>>2] = 1;
 $42 = $0;
 $43 = (($42) + 120|0);
 $44 = (($43) + 20|0);
 HEAP32[$44>>2] = 1;
 $45 = $0;
 $46 = (($45) + 192|0);
 $47 = (($46) + 20|0);
 HEAP32[$47>>2] = 1;
 $48 = $0;
 $49 = (($48) + 216|0);
 $50 = (($49) + 20|0);
 HEAP32[$50>>2] = 1;
 $51 = $0;
 $52 = (($51) + 288|0);
 $53 = (($52) + 20|0);
 HEAP32[$53>>2] = 1;
 $54 = $0;
 $55 = (($54) + 312|0);
 $56 = (($55) + 20|0);
 HEAP32[$56>>2] = 1;
 $57 = $0;
 $58 = (($57) + 408|0);
 $59 = (($58) + 20|0);
 HEAP32[$59>>2] = 1;
 $60 = $0;
 $61 = (($60) + 432|0);
 $62 = (($61) + 20|0);
 HEAP32[$62>>2] = 1;
 $63 = $0;
 $64 = $1;
 $65 = $2;
 _w_tree4($63,0,1,$64,$65,0,0,1);
 $66 = $0;
 $67 = (($66) + 24|0);
 $68 = (($67) + 8|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69|0) % 2)&-1;
 $71 = ($70|0)==(0);
 if ($71) {
  $72 = $0;
  $73 = (($72) + 24|0);
  $74 = (($73) + 8|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75|0) / 2)&-1;
  $lenx = $76;
  $77 = $lenx;
  $lenx2 = $77;
 } else {
  $78 = $0;
  $79 = (($78) + 24|0);
  $80 = (($79) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  $83 = (($82|0) / 2)&-1;
  $lenx = $83;
  $84 = $lenx;
  $85 = (($84) - 1)|0;
  $lenx2 = $85;
 }
 $86 = $0;
 $87 = (($86) + 24|0);
 $88 = (($87) + 12|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89|0) % 2)&-1;
 $91 = ($90|0)==(0);
 if ($91) {
  $92 = $0;
  $93 = (($92) + 24|0);
  $94 = (($93) + 12|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95|0) / 2)&-1;
  $leny = $96;
  $97 = $leny;
  $leny2 = $97;
 } else {
  $98 = $0;
  $99 = (($98) + 24|0);
  $100 = (($99) + 12|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = (($101) + 1)|0;
  $103 = (($102|0) / 2)&-1;
  $leny = $103;
  $104 = $leny;
  $105 = (($104) - 1)|0;
  $leny2 = $105;
 }
 $106 = $0;
 $107 = $lenx2;
 $108 = $leny;
 $109 = $lenx;
 _w_tree4($106,4,6,$107,$108,$109,0,0);
 $110 = $0;
 $111 = $lenx;
 $112 = $leny2;
 $113 = $leny;
 _w_tree4($110,5,10,$111,$112,0,$113,0);
 $114 = $0;
 $115 = $lenx;
 $116 = $leny;
 _w_tree4($114,14,15,$115,$116,0,0,0);
 $117 = $0;
 $118 = (($117) + 456|0);
 HEAP32[$118>>2] = 0;
 $119 = $0;
 $120 = (($119) + 456|0);
 $121 = (($120) + 4|0);
 HEAP32[$121>>2] = 0;
 $122 = $0;
 $123 = (($122) + 360|0);
 $124 = (($123) + 8|0);
 $125 = HEAP32[$124>>2]|0;
 $126 = (($125|0) % 2)&-1;
 $127 = ($126|0)==(0);
 if ($127) {
  $128 = $0;
  $129 = (($128) + 360|0);
  $130 = (($129) + 8|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = (($131|0) / 2)&-1;
  $133 = $0;
  $134 = (($133) + 456|0);
  $135 = (($134) + 8|0);
  HEAP32[$135>>2] = $132;
 } else {
  $136 = $0;
  $137 = (($136) + 360|0);
  $138 = (($137) + 8|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($139) + 1)|0;
  $141 = (($140|0) / 2)&-1;
  $142 = $0;
  $143 = (($142) + 456|0);
  $144 = (($143) + 8|0);
  HEAP32[$144>>2] = $141;
 }
 $145 = $0;
 $146 = (($145) + 360|0);
 $147 = (($146) + 12|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = (($148|0) % 2)&-1;
 $150 = ($149|0)==(0);
 if ($150) {
  $151 = $0;
  $152 = (($151) + 360|0);
  $153 = (($152) + 12|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = (($154|0) / 2)&-1;
  $156 = $0;
  $157 = (($156) + 456|0);
  $158 = (($157) + 12|0);
  HEAP32[$158>>2] = $155;
 } else {
  $159 = $0;
  $160 = (($159) + 360|0);
  $161 = (($160) + 12|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = (($162) + 1)|0;
  $164 = (($163|0) / 2)&-1;
  $165 = $0;
  $166 = (($165) + 456|0);
  $167 = (($166) + 12|0);
  HEAP32[$167>>2] = $164;
 }
 $168 = HEAP32[8>>2]|0;
 $169 = ($168|0)>(1);
 if (!($169)) {
  STACKTOP = sp;return;
 }
 $node = 0;
 while(1) {
  $170 = $node;
  $171 = ($170|0)<(20);
  if (!($171)) {
   break;
  }
  $172 = HEAP32[_stderr>>2]|0;
  $173 = $node;
  $174 = $node;
  $175 = $0;
  $176 = (($175) + (($174*24)|0)|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = $node;
  $179 = $0;
  $180 = (($179) + (($178*24)|0)|0);
  $181 = (($180) + 4|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = $node;
  $184 = $0;
  $185 = (($184) + (($183*24)|0)|0);
  $186 = (($185) + 8|0);
  $187 = HEAP32[$186>>2]|0;
  $188 = $node;
  $189 = $0;
  $190 = (($189) + (($188*24)|0)|0);
  $191 = (($190) + 12|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = $node;
  $194 = $0;
  $195 = (($194) + (($193*24)|0)|0);
  $196 = (($195) + 16|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = $node;
  $199 = $0;
  $200 = (($199) + (($198*24)|0)|0);
  $201 = (($200) + 20|0);
  $202 = HEAP32[$201>>2]|0;
  HEAP32[$vararg_buffer>>2] = $173;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $177;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $182;
  $vararg_ptr3 = (($vararg_buffer) + 12|0);
  HEAP32[$vararg_ptr3>>2] = $187;
  $vararg_ptr4 = (($vararg_buffer) + 16|0);
  HEAP32[$vararg_ptr4>>2] = $192;
  $vararg_ptr5 = (($vararg_buffer) + 20|0);
  HEAP32[$vararg_ptr5>>2] = $197;
  $vararg_ptr6 = (($vararg_buffer) + 24|0);
  HEAP32[$vararg_ptr6>>2] = $202;
  (_fprintf(($172|0),(41272|0),($vararg_buffer|0))|0);
  $203 = $node;
  $204 = (($203) + 1)|0;
  $node = $204;
 }
 $205 = HEAP32[_stderr>>2]|0;
 (_fprintf(($205|0),(41336|0),($vararg_buffer7|0))|0);
 STACKTOP = sp;return;
}
function _build_q_tree($w_tree,$q_tree) {
 $w_tree = $w_tree|0;
 $q_tree = $q_tree|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $node = 0, $vararg_buffer = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer = sp;
 $0 = $w_tree;
 $1 = $q_tree;
 $2 = $1;
 $3 = $0;
 $4 = (($3) + 336|0);
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 336|0);
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $0;
 $12 = (($11) + 336|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $0;
 $15 = (($14) + 336|0);
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 _q_tree16($2,3,$6,$10,$13,$17,0,0);
 $18 = $1;
 $19 = $0;
 $20 = (($19) + 96|0);
 $21 = (($20) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = (($23) + 96|0);
 $25 = (($24) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $0;
 $28 = (($27) + 96|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = (($30) + 96|0);
 $32 = (($31) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 _q_tree16($18,19,$22,$26,$29,$33,0,1);
 $34 = $1;
 $35 = $0;
 $36 = (($35) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $0;
 $39 = (($38) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $0;
 $42 = HEAP32[$41>>2]|0;
 $43 = $0;
 $44 = (($43) + 4|0);
 $45 = HEAP32[$44>>2]|0;
 _q_tree16($34,48,$37,$40,$42,$45,0,0);
 $46 = $1;
 $47 = $0;
 $48 = (($47) + 120|0);
 $49 = (($48) + 8|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = $0;
 $52 = (($51) + 120|0);
 $53 = (($52) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $0;
 $56 = (($55) + 120|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $0;
 $59 = (($58) + 120|0);
 $60 = (($59) + 4|0);
 $61 = HEAP32[$60>>2]|0;
 _q_tree16($46,35,$50,$54,$57,$61,1,0);
 $62 = $1;
 $63 = $0;
 $64 = (($63) + 456|0);
 $65 = (($64) + 8|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $0;
 $68 = (($67) + 456|0);
 $69 = (($68) + 12|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $0;
 $72 = (($71) + 456|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $0;
 $75 = (($74) + 456|0);
 $76 = (($75) + 4|0);
 $77 = HEAP32[$76>>2]|0;
 _q_tree4($62,0,$66,$70,$73,$77);
 $78 = HEAP32[8>>2]|0;
 $79 = ($78|0)>(1);
 if (!($79)) {
  STACKTOP = sp;return;
 }
 $node = 0;
 while(1) {
  $80 = $node;
  $81 = ($80|0)<(60);
  if (!($81)) {
   break;
  }
  $82 = HEAP32[_stderr>>2]|0;
  $83 = $node;
  $84 = $node;
  $85 = $1;
  $86 = (($85) + ($84<<3)|0);
  $87 = HEAP16[$86>>1]|0;
  $88 = $87 << 16 >> 16;
  $89 = $node;
  $90 = $1;
  $91 = (($90) + ($89<<3)|0);
  $92 = (($91) + 2|0);
  $93 = HEAP16[$92>>1]|0;
  $94 = $93 << 16 >> 16;
  $95 = $node;
  $96 = $1;
  $97 = (($96) + ($95<<3)|0);
  $98 = (($97) + 4|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $99 << 16 >> 16;
  $101 = $node;
  $102 = $1;
  $103 = (($102) + ($101<<3)|0);
  $104 = (($103) + 6|0);
  $105 = HEAP16[$104>>1]|0;
  $106 = $105 << 16 >> 16;
  HEAP32[$vararg_buffer>>2] = $83;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $88;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $94;
  $vararg_ptr3 = (($vararg_buffer) + 12|0);
  HEAP32[$vararg_ptr3>>2] = $100;
  $vararg_ptr4 = (($vararg_buffer) + 16|0);
  HEAP32[$vararg_ptr4>>2] = $106;
  (_fprintf(($82|0),(41344|0),($vararg_buffer|0))|0);
  $107 = $node;
  $108 = (($107) + 1)|0;
  $node = $108;
 }
 $109 = HEAP32[_stderr>>2]|0;
 (_fprintf(($109|0),(41336|0),($vararg_buffer5|0))|0);
 STACKTOP = sp;return;
}
function _w_tree4($w_tree,$start1,$start2,$lenx,$leny,$x,$y,$stop1) {
 $w_tree = $w_tree|0;
 $start1 = $start1|0;
 $start2 = $start2|0;
 $lenx = $lenx|0;
 $leny = $leny|0;
 $x = $x|0;
 $y = $y|0;
 $stop1 = $stop1|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $evenx = 0, $eveny = 0, $p1 = 0, $p2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $w_tree;
 $1 = $start1;
 $2 = $start2;
 $3 = $lenx;
 $4 = $leny;
 $5 = $x;
 $6 = $y;
 $7 = $stop1;
 $8 = $1;
 $p1 = $8;
 $9 = $2;
 $p2 = $9;
 $10 = $3;
 $11 = (($10|0) % 2)&-1;
 $evenx = $11;
 $12 = $4;
 $13 = (($12|0) % 2)&-1;
 $eveny = $13;
 $14 = $5;
 $15 = $p1;
 $16 = $0;
 $17 = (($16) + (($15*24)|0)|0);
 HEAP32[$17>>2] = $14;
 $18 = $6;
 $19 = $p1;
 $20 = $0;
 $21 = (($20) + (($19*24)|0)|0);
 $22 = (($21) + 4|0);
 HEAP32[$22>>2] = $18;
 $23 = $3;
 $24 = $p1;
 $25 = $0;
 $26 = (($25) + (($24*24)|0)|0);
 $27 = (($26) + 8|0);
 HEAP32[$27>>2] = $23;
 $28 = $4;
 $29 = $p1;
 $30 = $0;
 $31 = (($30) + (($29*24)|0)|0);
 $32 = (($31) + 12|0);
 HEAP32[$32>>2] = $28;
 $33 = $5;
 $34 = $p2;
 $35 = $0;
 $36 = (($35) + (($34*24)|0)|0);
 HEAP32[$36>>2] = $33;
 $37 = $5;
 $38 = $p2;
 $39 = (($38) + 2)|0;
 $40 = $0;
 $41 = (($40) + (($39*24)|0)|0);
 HEAP32[$41>>2] = $37;
 $42 = $6;
 $43 = $p2;
 $44 = $0;
 $45 = (($44) + (($43*24)|0)|0);
 $46 = (($45) + 4|0);
 HEAP32[$46>>2] = $42;
 $47 = $6;
 $48 = $p2;
 $49 = (($48) + 1)|0;
 $50 = $0;
 $51 = (($50) + (($49*24)|0)|0);
 $52 = (($51) + 4|0);
 HEAP32[$52>>2] = $47;
 $53 = $evenx;
 $54 = ($53|0)==(0);
 if ($54) {
  $55 = $3;
  $56 = (($55|0) / 2)&-1;
  $57 = $p2;
  $58 = $0;
  $59 = (($58) + (($57*24)|0)|0);
  $60 = (($59) + 8|0);
  HEAP32[$60>>2] = $56;
  $61 = $p2;
  $62 = $0;
  $63 = (($62) + (($61*24)|0)|0);
  $64 = (($63) + 8|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = $p2;
  $67 = (($66) + 1)|0;
  $68 = $0;
  $69 = (($68) + (($67*24)|0)|0);
  $70 = (($69) + 8|0);
  HEAP32[$70>>2] = $65;
 } else {
  $71 = $p1;
  $72 = ($71|0)==(4);
  if ($72) {
   $73 = $3;
   $74 = (($73) - 1)|0;
   $75 = (($74|0) / 2)&-1;
   $76 = $p2;
   $77 = $0;
   $78 = (($77) + (($76*24)|0)|0);
   $79 = (($78) + 8|0);
   HEAP32[$79>>2] = $75;
   $80 = $p2;
   $81 = $0;
   $82 = (($81) + (($80*24)|0)|0);
   $83 = (($82) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + 1)|0;
   $86 = $p2;
   $87 = (($86) + 1)|0;
   $88 = $0;
   $89 = (($88) + (($87*24)|0)|0);
   $90 = (($89) + 8|0);
   HEAP32[$90>>2] = $85;
  } else {
   $91 = $3;
   $92 = (($91) + 1)|0;
   $93 = (($92|0) / 2)&-1;
   $94 = $p2;
   $95 = $0;
   $96 = (($95) + (($94*24)|0)|0);
   $97 = (($96) + 8|0);
   HEAP32[$97>>2] = $93;
   $98 = $p2;
   $99 = $0;
   $100 = (($99) + (($98*24)|0)|0);
   $101 = (($100) + 8|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) - 1)|0;
   $104 = $p2;
   $105 = (($104) + 1)|0;
   $106 = $0;
   $107 = (($106) + (($105*24)|0)|0);
   $108 = (($107) + 8|0);
   HEAP32[$108>>2] = $103;
  }
 }
 $109 = $p2;
 $110 = $0;
 $111 = (($110) + (($109*24)|0)|0);
 $112 = (($111) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = $5;
 $115 = (($113) + ($114))|0;
 $116 = $p2;
 $117 = (($116) + 1)|0;
 $118 = $0;
 $119 = (($118) + (($117*24)|0)|0);
 HEAP32[$119>>2] = $115;
 $120 = $7;
 $121 = ($120|0)==(0);
 if ($121) {
  $122 = $p2;
  $123 = (($122) + 1)|0;
  $124 = $0;
  $125 = (($124) + (($123*24)|0)|0);
  $126 = (($125) + 8|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = $p2;
  $129 = (($128) + 3)|0;
  $130 = $0;
  $131 = (($130) + (($129*24)|0)|0);
  $132 = (($131) + 8|0);
  HEAP32[$132>>2] = $127;
  $133 = $p2;
  $134 = (($133) + 1)|0;
  $135 = $0;
  $136 = (($135) + (($134*24)|0)|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = $p2;
  $139 = (($138) + 3)|0;
  $140 = $0;
  $141 = (($140) + (($139*24)|0)|0);
  HEAP32[$141>>2] = $137;
 }
 $142 = $p2;
 $143 = $0;
 $144 = (($143) + (($142*24)|0)|0);
 $145 = (($144) + 8|0);
 $146 = HEAP32[$145>>2]|0;
 $147 = $p2;
 $148 = (($147) + 2)|0;
 $149 = $0;
 $150 = (($149) + (($148*24)|0)|0);
 $151 = (($150) + 8|0);
 HEAP32[$151>>2] = $146;
 $152 = $eveny;
 $153 = ($152|0)==(0);
 if ($153) {
  $154 = $4;
  $155 = (($154|0) / 2)&-1;
  $156 = $p2;
  $157 = $0;
  $158 = (($157) + (($156*24)|0)|0);
  $159 = (($158) + 12|0);
  HEAP32[$159>>2] = $155;
  $160 = $p2;
  $161 = $0;
  $162 = (($161) + (($160*24)|0)|0);
  $163 = (($162) + 12|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = $p2;
  $166 = (($165) + 2)|0;
  $167 = $0;
  $168 = (($167) + (($166*24)|0)|0);
  $169 = (($168) + 12|0);
  HEAP32[$169>>2] = $164;
 } else {
  $170 = $p1;
  $171 = ($170|0)==(5);
  if ($171) {
   $172 = $4;
   $173 = (($172) - 1)|0;
   $174 = (($173|0) / 2)&-1;
   $175 = $p2;
   $176 = $0;
   $177 = (($176) + (($175*24)|0)|0);
   $178 = (($177) + 12|0);
   HEAP32[$178>>2] = $174;
   $179 = $p2;
   $180 = $0;
   $181 = (($180) + (($179*24)|0)|0);
   $182 = (($181) + 12|0);
   $183 = HEAP32[$182>>2]|0;
   $184 = (($183) + 1)|0;
   $185 = $p2;
   $186 = (($185) + 2)|0;
   $187 = $0;
   $188 = (($187) + (($186*24)|0)|0);
   $189 = (($188) + 12|0);
   HEAP32[$189>>2] = $184;
  } else {
   $190 = $4;
   $191 = (($190) + 1)|0;
   $192 = (($191|0) / 2)&-1;
   $193 = $p2;
   $194 = $0;
   $195 = (($194) + (($193*24)|0)|0);
   $196 = (($195) + 12|0);
   HEAP32[$196>>2] = $192;
   $197 = $p2;
   $198 = $0;
   $199 = (($198) + (($197*24)|0)|0);
   $200 = (($199) + 12|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = (($201) - 1)|0;
   $203 = $p2;
   $204 = (($203) + 2)|0;
   $205 = $0;
   $206 = (($205) + (($204*24)|0)|0);
   $207 = (($206) + 12|0);
   HEAP32[$207>>2] = $202;
  }
 }
 $208 = $p2;
 $209 = $0;
 $210 = (($209) + (($208*24)|0)|0);
 $211 = (($210) + 12|0);
 $212 = HEAP32[$211>>2]|0;
 $213 = $6;
 $214 = (($212) + ($213))|0;
 $215 = $p2;
 $216 = (($215) + 2)|0;
 $217 = $0;
 $218 = (($217) + (($216*24)|0)|0);
 $219 = (($218) + 4|0);
 HEAP32[$219>>2] = $214;
 $220 = $7;
 $221 = ($220|0)==(0);
 if (!($221)) {
  $244 = $p2;
  $245 = $0;
  $246 = (($245) + (($244*24)|0)|0);
  $247 = (($246) + 12|0);
  $248 = HEAP32[$247>>2]|0;
  $249 = $p2;
  $250 = (($249) + 1)|0;
  $251 = $0;
  $252 = (($251) + (($250*24)|0)|0);
  $253 = (($252) + 12|0);
  HEAP32[$253>>2] = $248;
  STACKTOP = sp;return;
 }
 $222 = $p2;
 $223 = (($222) + 2)|0;
 $224 = $0;
 $225 = (($224) + (($223*24)|0)|0);
 $226 = (($225) + 12|0);
 $227 = HEAP32[$226>>2]|0;
 $228 = $p2;
 $229 = (($228) + 3)|0;
 $230 = $0;
 $231 = (($230) + (($229*24)|0)|0);
 $232 = (($231) + 12|0);
 HEAP32[$232>>2] = $227;
 $233 = $p2;
 $234 = (($233) + 2)|0;
 $235 = $0;
 $236 = (($235) + (($234*24)|0)|0);
 $237 = (($236) + 4|0);
 $238 = HEAP32[$237>>2]|0;
 $239 = $p2;
 $240 = (($239) + 3)|0;
 $241 = $0;
 $242 = (($241) + (($240*24)|0)|0);
 $243 = (($242) + 4|0);
 HEAP32[$243>>2] = $238;
 $244 = $p2;
 $245 = $0;
 $246 = (($245) + (($244*24)|0)|0);
 $247 = (($246) + 12|0);
 $248 = HEAP32[$247>>2]|0;
 $249 = $p2;
 $250 = (($249) + 1)|0;
 $251 = $0;
 $252 = (($251) + (($250*24)|0)|0);
 $253 = (($252) + 12|0);
 HEAP32[$253>>2] = $248;
 STACKTOP = sp;return;
}
function _q_tree16($q_tree,$start,$lenx,$leny,$x,$y,$rw,$cl) {
 $q_tree = $q_tree|0;
 $start = $start|0;
 $lenx = $lenx|0;
 $leny = $leny|0;
 $x = $x|0;
 $y = $y|0;
 $rw = $rw|0;
 $cl = $cl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $evenx = 0;
 var $eveny = 0, $p = 0, $temp2x = 0, $temp2y = 0, $tempx = 0, $tempy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $q_tree;
 $1 = $start;
 $2 = $lenx;
 $3 = $leny;
 $4 = $x;
 $5 = $y;
 $6 = $rw;
 $7 = $cl;
 $8 = $1;
 $p = $8;
 $9 = $2;
 $10 = (($9|0) % 2)&-1;
 $evenx = $10;
 $11 = $3;
 $12 = (($11|0) % 2)&-1;
 $eveny = $12;
 $13 = $evenx;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = $2;
  $16 = (($15|0) / 2)&-1;
  $tempx = $16;
  $17 = $tempx;
  $temp2x = $17;
 } else {
  $18 = $7;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = $2;
   $21 = (($20) + 1)|0;
   $22 = (($21|0) / 2)&-1;
   $temp2x = $22;
   $23 = $temp2x;
   $24 = (($23) - 1)|0;
   $tempx = $24;
  } else {
   $25 = $2;
   $26 = (($25) + 1)|0;
   $27 = (($26|0) / 2)&-1;
   $tempx = $27;
   $28 = $tempx;
   $29 = (($28) - 1)|0;
   $temp2x = $29;
  }
 }
 $30 = $eveny;
 $31 = ($30|0)==(0);
 if ($31) {
  $32 = $3;
  $33 = (($32|0) / 2)&-1;
  $tempy = $33;
  $34 = $tempy;
  $temp2y = $34;
 } else {
  $35 = $6;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = $3;
   $38 = (($37) + 1)|0;
   $39 = (($38|0) / 2)&-1;
   $temp2y = $39;
   $40 = $temp2y;
   $41 = (($40) - 1)|0;
   $tempy = $41;
  } else {
   $42 = $3;
   $43 = (($42) + 1)|0;
   $44 = (($43|0) / 2)&-1;
   $tempy = $44;
   $45 = $tempy;
   $46 = (($45) - 1)|0;
   $temp2y = $46;
  }
 }
 $47 = $tempx;
 $48 = (($47|0) % 2)&-1;
 $evenx = $48;
 $49 = $tempy;
 $50 = (($49|0) % 2)&-1;
 $eveny = $50;
 $51 = $4;
 $52 = $51&65535;
 $53 = $p;
 $54 = $0;
 $55 = (($54) + ($53<<3)|0);
 HEAP16[$55>>1] = $52;
 $56 = $4;
 $57 = $56&65535;
 $58 = $p;
 $59 = (($58) + 2)|0;
 $60 = $0;
 $61 = (($60) + ($59<<3)|0);
 HEAP16[$61>>1] = $57;
 $62 = $5;
 $63 = $62&65535;
 $64 = $p;
 $65 = $0;
 $66 = (($65) + ($64<<3)|0);
 $67 = (($66) + 2|0);
 HEAP16[$67>>1] = $63;
 $68 = $5;
 $69 = $68&65535;
 $70 = $p;
 $71 = (($70) + 1)|0;
 $72 = $0;
 $73 = (($72) + ($71<<3)|0);
 $74 = (($73) + 2|0);
 HEAP16[$74>>1] = $69;
 $75 = $evenx;
 $76 = ($75|0)==(0);
 if ($76) {
  $77 = $tempx;
  $78 = (($77|0) / 2)&-1;
  $79 = $78&65535;
  $80 = $p;
  $81 = $0;
  $82 = (($81) + ($80<<3)|0);
  $83 = (($82) + 4|0);
  HEAP16[$83>>1] = $79;
  $84 = $p;
  $85 = $0;
  $86 = (($85) + ($84<<3)|0);
  $87 = (($86) + 4|0);
  $88 = HEAP16[$87>>1]|0;
  $89 = $p;
  $90 = (($89) + 1)|0;
  $91 = $0;
  $92 = (($91) + ($90<<3)|0);
  $93 = (($92) + 4|0);
  HEAP16[$93>>1] = $88;
  $94 = $p;
  $95 = $0;
  $96 = (($95) + ($94<<3)|0);
  $97 = (($96) + 4|0);
  $98 = HEAP16[$97>>1]|0;
  $99 = $p;
  $100 = (($99) + 2)|0;
  $101 = $0;
  $102 = (($101) + ($100<<3)|0);
  $103 = (($102) + 4|0);
  HEAP16[$103>>1] = $98;
  $104 = $p;
  $105 = $0;
  $106 = (($105) + ($104<<3)|0);
  $107 = (($106) + 4|0);
  $108 = HEAP16[$107>>1]|0;
  $109 = $p;
  $110 = (($109) + 3)|0;
  $111 = $0;
  $112 = (($111) + ($110<<3)|0);
  $113 = (($112) + 4|0);
  HEAP16[$113>>1] = $108;
 } else {
  $114 = $tempx;
  $115 = (($114) + 1)|0;
  $116 = (($115|0) / 2)&-1;
  $117 = $116&65535;
  $118 = $p;
  $119 = $0;
  $120 = (($119) + ($118<<3)|0);
  $121 = (($120) + 4|0);
  HEAP16[$121>>1] = $117;
  $122 = $p;
  $123 = $0;
  $124 = (($123) + ($122<<3)|0);
  $125 = (($124) + 4|0);
  $126 = HEAP16[$125>>1]|0;
  $127 = $126 << 16 >> 16;
  $128 = (($127) - 1)|0;
  $129 = $128&65535;
  $130 = $p;
  $131 = (($130) + 1)|0;
  $132 = $0;
  $133 = (($132) + ($131<<3)|0);
  $134 = (($133) + 4|0);
  HEAP16[$134>>1] = $129;
  $135 = $p;
  $136 = $0;
  $137 = (($136) + ($135<<3)|0);
  $138 = (($137) + 4|0);
  $139 = HEAP16[$138>>1]|0;
  $140 = $p;
  $141 = (($140) + 2)|0;
  $142 = $0;
  $143 = (($142) + ($141<<3)|0);
  $144 = (($143) + 4|0);
  HEAP16[$144>>1] = $139;
  $145 = $p;
  $146 = (($145) + 1)|0;
  $147 = $0;
  $148 = (($147) + ($146<<3)|0);
  $149 = (($148) + 4|0);
  $150 = HEAP16[$149>>1]|0;
  $151 = $p;
  $152 = (($151) + 3)|0;
  $153 = $0;
  $154 = (($153) + ($152<<3)|0);
  $155 = (($154) + 4|0);
  HEAP16[$155>>1] = $150;
 }
 $156 = $4;
 $157 = $p;
 $158 = $0;
 $159 = (($158) + ($157<<3)|0);
 $160 = (($159) + 4|0);
 $161 = HEAP16[$160>>1]|0;
 $162 = $161 << 16 >> 16;
 $163 = (($156) + ($162))|0;
 $164 = $163&65535;
 $165 = $p;
 $166 = (($165) + 1)|0;
 $167 = $0;
 $168 = (($167) + ($166<<3)|0);
 HEAP16[$168>>1] = $164;
 $169 = $p;
 $170 = (($169) + 1)|0;
 $171 = $0;
 $172 = (($171) + ($170<<3)|0);
 $173 = HEAP16[$172>>1]|0;
 $174 = $p;
 $175 = (($174) + 3)|0;
 $176 = $0;
 $177 = (($176) + ($175<<3)|0);
 HEAP16[$177>>1] = $173;
 $178 = $eveny;
 $179 = ($178|0)==(0);
 if ($179) {
  $180 = $tempy;
  $181 = (($180|0) / 2)&-1;
  $182 = $181&65535;
  $183 = $p;
  $184 = $0;
  $185 = (($184) + ($183<<3)|0);
  $186 = (($185) + 6|0);
  HEAP16[$186>>1] = $182;
  $187 = $p;
  $188 = $0;
  $189 = (($188) + ($187<<3)|0);
  $190 = (($189) + 6|0);
  $191 = HEAP16[$190>>1]|0;
  $192 = $p;
  $193 = (($192) + 1)|0;
  $194 = $0;
  $195 = (($194) + ($193<<3)|0);
  $196 = (($195) + 6|0);
  HEAP16[$196>>1] = $191;
  $197 = $p;
  $198 = $0;
  $199 = (($198) + ($197<<3)|0);
  $200 = (($199) + 6|0);
  $201 = HEAP16[$200>>1]|0;
  $202 = $p;
  $203 = (($202) + 2)|0;
  $204 = $0;
  $205 = (($204) + ($203<<3)|0);
  $206 = (($205) + 6|0);
  HEAP16[$206>>1] = $201;
  $207 = $p;
  $208 = $0;
  $209 = (($208) + ($207<<3)|0);
  $210 = (($209) + 6|0);
  $211 = HEAP16[$210>>1]|0;
  $212 = $p;
  $213 = (($212) + 3)|0;
  $214 = $0;
  $215 = (($214) + ($213<<3)|0);
  $216 = (($215) + 6|0);
  HEAP16[$216>>1] = $211;
 } else {
  $217 = $tempy;
  $218 = (($217) + 1)|0;
  $219 = (($218|0) / 2)&-1;
  $220 = $219&65535;
  $221 = $p;
  $222 = $0;
  $223 = (($222) + ($221<<3)|0);
  $224 = (($223) + 6|0);
  HEAP16[$224>>1] = $220;
  $225 = $p;
  $226 = $0;
  $227 = (($226) + ($225<<3)|0);
  $228 = (($227) + 6|0);
  $229 = HEAP16[$228>>1]|0;
  $230 = $p;
  $231 = (($230) + 1)|0;
  $232 = $0;
  $233 = (($232) + ($231<<3)|0);
  $234 = (($233) + 6|0);
  HEAP16[$234>>1] = $229;
  $235 = $p;
  $236 = $0;
  $237 = (($236) + ($235<<3)|0);
  $238 = (($237) + 6|0);
  $239 = HEAP16[$238>>1]|0;
  $240 = $239 << 16 >> 16;
  $241 = (($240) - 1)|0;
  $242 = $241&65535;
  $243 = $p;
  $244 = (($243) + 2)|0;
  $245 = $0;
  $246 = (($245) + ($244<<3)|0);
  $247 = (($246) + 6|0);
  HEAP16[$247>>1] = $242;
  $248 = $p;
  $249 = (($248) + 2)|0;
  $250 = $0;
  $251 = (($250) + ($249<<3)|0);
  $252 = (($251) + 6|0);
  $253 = HEAP16[$252>>1]|0;
  $254 = $p;
  $255 = (($254) + 3)|0;
  $256 = $0;
  $257 = (($256) + ($255<<3)|0);
  $258 = (($257) + 6|0);
  HEAP16[$258>>1] = $253;
 }
 $259 = $5;
 $260 = $p;
 $261 = $0;
 $262 = (($261) + ($260<<3)|0);
 $263 = (($262) + 6|0);
 $264 = HEAP16[$263>>1]|0;
 $265 = $264 << 16 >> 16;
 $266 = (($259) + ($265))|0;
 $267 = $266&65535;
 $268 = $p;
 $269 = (($268) + 2)|0;
 $270 = $0;
 $271 = (($270) + ($269<<3)|0);
 $272 = (($271) + 2|0);
 HEAP16[$272>>1] = $267;
 $273 = $p;
 $274 = (($273) + 2)|0;
 $275 = $0;
 $276 = (($275) + ($274<<3)|0);
 $277 = (($276) + 2|0);
 $278 = HEAP16[$277>>1]|0;
 $279 = $p;
 $280 = (($279) + 3)|0;
 $281 = $0;
 $282 = (($281) + ($280<<3)|0);
 $283 = (($282) + 2|0);
 HEAP16[$283>>1] = $278;
 $284 = $temp2x;
 $285 = (($284|0) % 2)&-1;
 $evenx = $285;
 $286 = $4;
 $287 = $tempx;
 $288 = (($286) + ($287))|0;
 $289 = $288&65535;
 $290 = $p;
 $291 = (($290) + 4)|0;
 $292 = $0;
 $293 = (($292) + ($291<<3)|0);
 HEAP16[$293>>1] = $289;
 $294 = $p;
 $295 = (($294) + 4)|0;
 $296 = $0;
 $297 = (($296) + ($295<<3)|0);
 $298 = HEAP16[$297>>1]|0;
 $299 = $p;
 $300 = (($299) + 6)|0;
 $301 = $0;
 $302 = (($301) + ($300<<3)|0);
 HEAP16[$302>>1] = $298;
 $303 = $5;
 $304 = $303&65535;
 $305 = $p;
 $306 = (($305) + 4)|0;
 $307 = $0;
 $308 = (($307) + ($306<<3)|0);
 $309 = (($308) + 2|0);
 HEAP16[$309>>1] = $304;
 $310 = $5;
 $311 = $310&65535;
 $312 = $p;
 $313 = (($312) + 5)|0;
 $314 = $0;
 $315 = (($314) + ($313<<3)|0);
 $316 = (($315) + 2|0);
 HEAP16[$316>>1] = $311;
 $317 = $p;
 $318 = (($317) + 2)|0;
 $319 = $0;
 $320 = (($319) + ($318<<3)|0);
 $321 = (($320) + 2|0);
 $322 = HEAP16[$321>>1]|0;
 $323 = $p;
 $324 = (($323) + 6)|0;
 $325 = $0;
 $326 = (($325) + ($324<<3)|0);
 $327 = (($326) + 2|0);
 HEAP16[$327>>1] = $322;
 $328 = $p;
 $329 = (($328) + 2)|0;
 $330 = $0;
 $331 = (($330) + ($329<<3)|0);
 $332 = (($331) + 2|0);
 $333 = HEAP16[$332>>1]|0;
 $334 = $p;
 $335 = (($334) + 7)|0;
 $336 = $0;
 $337 = (($336) + ($335<<3)|0);
 $338 = (($337) + 2|0);
 HEAP16[$338>>1] = $333;
 $339 = $p;
 $340 = $0;
 $341 = (($340) + ($339<<3)|0);
 $342 = (($341) + 6|0);
 $343 = HEAP16[$342>>1]|0;
 $344 = $p;
 $345 = (($344) + 4)|0;
 $346 = $0;
 $347 = (($346) + ($345<<3)|0);
 $348 = (($347) + 6|0);
 HEAP16[$348>>1] = $343;
 $349 = $p;
 $350 = $0;
 $351 = (($350) + ($349<<3)|0);
 $352 = (($351) + 6|0);
 $353 = HEAP16[$352>>1]|0;
 $354 = $p;
 $355 = (($354) + 5)|0;
 $356 = $0;
 $357 = (($356) + ($355<<3)|0);
 $358 = (($357) + 6|0);
 HEAP16[$358>>1] = $353;
 $359 = $p;
 $360 = (($359) + 2)|0;
 $361 = $0;
 $362 = (($361) + ($360<<3)|0);
 $363 = (($362) + 6|0);
 $364 = HEAP16[$363>>1]|0;
 $365 = $p;
 $366 = (($365) + 6)|0;
 $367 = $0;
 $368 = (($367) + ($366<<3)|0);
 $369 = (($368) + 6|0);
 HEAP16[$369>>1] = $364;
 $370 = $p;
 $371 = (($370) + 2)|0;
 $372 = $0;
 $373 = (($372) + ($371<<3)|0);
 $374 = (($373) + 6|0);
 $375 = HEAP16[$374>>1]|0;
 $376 = $p;
 $377 = (($376) + 7)|0;
 $378 = $0;
 $379 = (($378) + ($377<<3)|0);
 $380 = (($379) + 6|0);
 HEAP16[$380>>1] = $375;
 $381 = $evenx;
 $382 = ($381|0)==(0);
 if ($382) {
  $383 = $temp2x;
  $384 = (($383|0) / 2)&-1;
  $385 = $384&65535;
  $386 = $p;
  $387 = (($386) + 4)|0;
  $388 = $0;
  $389 = (($388) + ($387<<3)|0);
  $390 = (($389) + 4|0);
  HEAP16[$390>>1] = $385;
  $391 = $p;
  $392 = (($391) + 4)|0;
  $393 = $0;
  $394 = (($393) + ($392<<3)|0);
  $395 = (($394) + 4|0);
  $396 = HEAP16[$395>>1]|0;
  $397 = $p;
  $398 = (($397) + 5)|0;
  $399 = $0;
  $400 = (($399) + ($398<<3)|0);
  $401 = (($400) + 4|0);
  HEAP16[$401>>1] = $396;
  $402 = $p;
  $403 = (($402) + 4)|0;
  $404 = $0;
  $405 = (($404) + ($403<<3)|0);
  $406 = (($405) + 4|0);
  $407 = HEAP16[$406>>1]|0;
  $408 = $p;
  $409 = (($408) + 6)|0;
  $410 = $0;
  $411 = (($410) + ($409<<3)|0);
  $412 = (($411) + 4|0);
  HEAP16[$412>>1] = $407;
  $413 = $p;
  $414 = (($413) + 4)|0;
  $415 = $0;
  $416 = (($415) + ($414<<3)|0);
  $417 = (($416) + 4|0);
  $418 = HEAP16[$417>>1]|0;
  $419 = $p;
  $420 = (($419) + 7)|0;
  $421 = $0;
  $422 = (($421) + ($420<<3)|0);
  $423 = (($422) + 4|0);
  HEAP16[$423>>1] = $418;
 } else {
  $424 = $temp2x;
  $425 = (($424) + 1)|0;
  $426 = (($425|0) / 2)&-1;
  $427 = $426&65535;
  $428 = $p;
  $429 = (($428) + 5)|0;
  $430 = $0;
  $431 = (($430) + ($429<<3)|0);
  $432 = (($431) + 4|0);
  HEAP16[$432>>1] = $427;
  $433 = $p;
  $434 = (($433) + 5)|0;
  $435 = $0;
  $436 = (($435) + ($434<<3)|0);
  $437 = (($436) + 4|0);
  $438 = HEAP16[$437>>1]|0;
  $439 = $438 << 16 >> 16;
  $440 = (($439) - 1)|0;
  $441 = $440&65535;
  $442 = $p;
  $443 = (($442) + 4)|0;
  $444 = $0;
  $445 = (($444) + ($443<<3)|0);
  $446 = (($445) + 4|0);
  HEAP16[$446>>1] = $441;
  $447 = $p;
  $448 = (($447) + 4)|0;
  $449 = $0;
  $450 = (($449) + ($448<<3)|0);
  $451 = (($450) + 4|0);
  $452 = HEAP16[$451>>1]|0;
  $453 = $p;
  $454 = (($453) + 6)|0;
  $455 = $0;
  $456 = (($455) + ($454<<3)|0);
  $457 = (($456) + 4|0);
  HEAP16[$457>>1] = $452;
  $458 = $p;
  $459 = (($458) + 5)|0;
  $460 = $0;
  $461 = (($460) + ($459<<3)|0);
  $462 = (($461) + 4|0);
  $463 = HEAP16[$462>>1]|0;
  $464 = $p;
  $465 = (($464) + 7)|0;
  $466 = $0;
  $467 = (($466) + ($465<<3)|0);
  $468 = (($467) + 4|0);
  HEAP16[$468>>1] = $463;
 }
 $469 = $p;
 $470 = (($469) + 4)|0;
 $471 = $0;
 $472 = (($471) + ($470<<3)|0);
 $473 = HEAP16[$472>>1]|0;
 $474 = $473 << 16 >> 16;
 $475 = $p;
 $476 = (($475) + 4)|0;
 $477 = $0;
 $478 = (($477) + ($476<<3)|0);
 $479 = (($478) + 4|0);
 $480 = HEAP16[$479>>1]|0;
 $481 = $480 << 16 >> 16;
 $482 = (($474) + ($481))|0;
 $483 = $482&65535;
 $484 = $p;
 $485 = (($484) + 5)|0;
 $486 = $0;
 $487 = (($486) + ($485<<3)|0);
 HEAP16[$487>>1] = $483;
 $488 = $p;
 $489 = (($488) + 5)|0;
 $490 = $0;
 $491 = (($490) + ($489<<3)|0);
 $492 = HEAP16[$491>>1]|0;
 $493 = $p;
 $494 = (($493) + 7)|0;
 $495 = $0;
 $496 = (($495) + ($494<<3)|0);
 HEAP16[$496>>1] = $492;
 $497 = $temp2y;
 $498 = (($497|0) % 2)&-1;
 $eveny = $498;
 $499 = $4;
 $500 = $499&65535;
 $501 = $p;
 $502 = (($501) + 8)|0;
 $503 = $0;
 $504 = (($503) + ($502<<3)|0);
 HEAP16[$504>>1] = $500;
 $505 = $p;
 $506 = (($505) + 1)|0;
 $507 = $0;
 $508 = (($507) + ($506<<3)|0);
 $509 = HEAP16[$508>>1]|0;
 $510 = $p;
 $511 = (($510) + 9)|0;
 $512 = $0;
 $513 = (($512) + ($511<<3)|0);
 HEAP16[$513>>1] = $509;
 $514 = $4;
 $515 = $514&65535;
 $516 = $p;
 $517 = (($516) + 10)|0;
 $518 = $0;
 $519 = (($518) + ($517<<3)|0);
 HEAP16[$519>>1] = $515;
 $520 = $p;
 $521 = (($520) + 1)|0;
 $522 = $0;
 $523 = (($522) + ($521<<3)|0);
 $524 = HEAP16[$523>>1]|0;
 $525 = $p;
 $526 = (($525) + 11)|0;
 $527 = $0;
 $528 = (($527) + ($526<<3)|0);
 HEAP16[$528>>1] = $524;
 $529 = $5;
 $530 = $tempy;
 $531 = (($529) + ($530))|0;
 $532 = $531&65535;
 $533 = $p;
 $534 = (($533) + 8)|0;
 $535 = $0;
 $536 = (($535) + ($534<<3)|0);
 $537 = (($536) + 2|0);
 HEAP16[$537>>1] = $532;
 $538 = $p;
 $539 = (($538) + 8)|0;
 $540 = $0;
 $541 = (($540) + ($539<<3)|0);
 $542 = (($541) + 2|0);
 $543 = HEAP16[$542>>1]|0;
 $544 = $p;
 $545 = (($544) + 9)|0;
 $546 = $0;
 $547 = (($546) + ($545<<3)|0);
 $548 = (($547) + 2|0);
 HEAP16[$548>>1] = $543;
 $549 = $p;
 $550 = $0;
 $551 = (($550) + ($549<<3)|0);
 $552 = (($551) + 4|0);
 $553 = HEAP16[$552>>1]|0;
 $554 = $p;
 $555 = (($554) + 8)|0;
 $556 = $0;
 $557 = (($556) + ($555<<3)|0);
 $558 = (($557) + 4|0);
 HEAP16[$558>>1] = $553;
 $559 = $p;
 $560 = (($559) + 1)|0;
 $561 = $0;
 $562 = (($561) + ($560<<3)|0);
 $563 = (($562) + 4|0);
 $564 = HEAP16[$563>>1]|0;
 $565 = $p;
 $566 = (($565) + 9)|0;
 $567 = $0;
 $568 = (($567) + ($566<<3)|0);
 $569 = (($568) + 4|0);
 HEAP16[$569>>1] = $564;
 $570 = $p;
 $571 = $0;
 $572 = (($571) + ($570<<3)|0);
 $573 = (($572) + 4|0);
 $574 = HEAP16[$573>>1]|0;
 $575 = $p;
 $576 = (($575) + 10)|0;
 $577 = $0;
 $578 = (($577) + ($576<<3)|0);
 $579 = (($578) + 4|0);
 HEAP16[$579>>1] = $574;
 $580 = $p;
 $581 = (($580) + 1)|0;
 $582 = $0;
 $583 = (($582) + ($581<<3)|0);
 $584 = (($583) + 4|0);
 $585 = HEAP16[$584>>1]|0;
 $586 = $p;
 $587 = (($586) + 11)|0;
 $588 = $0;
 $589 = (($588) + ($587<<3)|0);
 $590 = (($589) + 4|0);
 HEAP16[$590>>1] = $585;
 $591 = $eveny;
 $592 = ($591|0)==(0);
 if ($592) {
  $593 = $temp2y;
  $594 = (($593|0) / 2)&-1;
  $595 = $594&65535;
  $596 = $p;
  $597 = (($596) + 8)|0;
  $598 = $0;
  $599 = (($598) + ($597<<3)|0);
  $600 = (($599) + 6|0);
  HEAP16[$600>>1] = $595;
  $601 = $p;
  $602 = (($601) + 8)|0;
  $603 = $0;
  $604 = (($603) + ($602<<3)|0);
  $605 = (($604) + 6|0);
  $606 = HEAP16[$605>>1]|0;
  $607 = $p;
  $608 = (($607) + 9)|0;
  $609 = $0;
  $610 = (($609) + ($608<<3)|0);
  $611 = (($610) + 6|0);
  HEAP16[$611>>1] = $606;
  $612 = $p;
  $613 = (($612) + 8)|0;
  $614 = $0;
  $615 = (($614) + ($613<<3)|0);
  $616 = (($615) + 6|0);
  $617 = HEAP16[$616>>1]|0;
  $618 = $p;
  $619 = (($618) + 10)|0;
  $620 = $0;
  $621 = (($620) + ($619<<3)|0);
  $622 = (($621) + 6|0);
  HEAP16[$622>>1] = $617;
  $623 = $p;
  $624 = (($623) + 8)|0;
  $625 = $0;
  $626 = (($625) + ($624<<3)|0);
  $627 = (($626) + 6|0);
  $628 = HEAP16[$627>>1]|0;
  $629 = $p;
  $630 = (($629) + 11)|0;
  $631 = $0;
  $632 = (($631) + ($630<<3)|0);
  $633 = (($632) + 6|0);
  HEAP16[$633>>1] = $628;
 } else {
  $634 = $temp2y;
  $635 = (($634) + 1)|0;
  $636 = (($635|0) / 2)&-1;
  $637 = $636&65535;
  $638 = $p;
  $639 = (($638) + 10)|0;
  $640 = $0;
  $641 = (($640) + ($639<<3)|0);
  $642 = (($641) + 6|0);
  HEAP16[$642>>1] = $637;
  $643 = $p;
  $644 = (($643) + 10)|0;
  $645 = $0;
  $646 = (($645) + ($644<<3)|0);
  $647 = (($646) + 6|0);
  $648 = HEAP16[$647>>1]|0;
  $649 = $p;
  $650 = (($649) + 11)|0;
  $651 = $0;
  $652 = (($651) + ($650<<3)|0);
  $653 = (($652) + 6|0);
  HEAP16[$653>>1] = $648;
  $654 = $p;
  $655 = (($654) + 10)|0;
  $656 = $0;
  $657 = (($656) + ($655<<3)|0);
  $658 = (($657) + 6|0);
  $659 = HEAP16[$658>>1]|0;
  $660 = $659 << 16 >> 16;
  $661 = (($660) - 1)|0;
  $662 = $661&65535;
  $663 = $p;
  $664 = (($663) + 8)|0;
  $665 = $0;
  $666 = (($665) + ($664<<3)|0);
  $667 = (($666) + 6|0);
  HEAP16[$667>>1] = $662;
  $668 = $p;
  $669 = (($668) + 8)|0;
  $670 = $0;
  $671 = (($670) + ($669<<3)|0);
  $672 = (($671) + 6|0);
  $673 = HEAP16[$672>>1]|0;
  $674 = $p;
  $675 = (($674) + 9)|0;
  $676 = $0;
  $677 = (($676) + ($675<<3)|0);
  $678 = (($677) + 6|0);
  HEAP16[$678>>1] = $673;
 }
 $679 = $p;
 $680 = (($679) + 8)|0;
 $681 = $0;
 $682 = (($681) + ($680<<3)|0);
 $683 = (($682) + 2|0);
 $684 = HEAP16[$683>>1]|0;
 $685 = $684 << 16 >> 16;
 $686 = $p;
 $687 = (($686) + 8)|0;
 $688 = $0;
 $689 = (($688) + ($687<<3)|0);
 $690 = (($689) + 6|0);
 $691 = HEAP16[$690>>1]|0;
 $692 = $691 << 16 >> 16;
 $693 = (($685) + ($692))|0;
 $694 = $693&65535;
 $695 = $p;
 $696 = (($695) + 10)|0;
 $697 = $0;
 $698 = (($697) + ($696<<3)|0);
 $699 = (($698) + 2|0);
 HEAP16[$699>>1] = $694;
 $700 = $p;
 $701 = (($700) + 10)|0;
 $702 = $0;
 $703 = (($702) + ($701<<3)|0);
 $704 = (($703) + 2|0);
 $705 = HEAP16[$704>>1]|0;
 $706 = $p;
 $707 = (($706) + 11)|0;
 $708 = $0;
 $709 = (($708) + ($707<<3)|0);
 $710 = (($709) + 2|0);
 HEAP16[$710>>1] = $705;
 $711 = $p;
 $712 = (($711) + 4)|0;
 $713 = $0;
 $714 = (($713) + ($712<<3)|0);
 $715 = HEAP16[$714>>1]|0;
 $716 = $p;
 $717 = (($716) + 12)|0;
 $718 = $0;
 $719 = (($718) + ($717<<3)|0);
 HEAP16[$719>>1] = $715;
 $720 = $p;
 $721 = (($720) + 5)|0;
 $722 = $0;
 $723 = (($722) + ($721<<3)|0);
 $724 = HEAP16[$723>>1]|0;
 $725 = $p;
 $726 = (($725) + 13)|0;
 $727 = $0;
 $728 = (($727) + ($726<<3)|0);
 HEAP16[$728>>1] = $724;
 $729 = $p;
 $730 = (($729) + 4)|0;
 $731 = $0;
 $732 = (($731) + ($730<<3)|0);
 $733 = HEAP16[$732>>1]|0;
 $734 = $p;
 $735 = (($734) + 14)|0;
 $736 = $0;
 $737 = (($736) + ($735<<3)|0);
 HEAP16[$737>>1] = $733;
 $738 = $p;
 $739 = (($738) + 5)|0;
 $740 = $0;
 $741 = (($740) + ($739<<3)|0);
 $742 = HEAP16[$741>>1]|0;
 $743 = $p;
 $744 = (($743) + 15)|0;
 $745 = $0;
 $746 = (($745) + ($744<<3)|0);
 HEAP16[$746>>1] = $742;
 $747 = $p;
 $748 = (($747) + 8)|0;
 $749 = $0;
 $750 = (($749) + ($748<<3)|0);
 $751 = (($750) + 2|0);
 $752 = HEAP16[$751>>1]|0;
 $753 = $p;
 $754 = (($753) + 12)|0;
 $755 = $0;
 $756 = (($755) + ($754<<3)|0);
 $757 = (($756) + 2|0);
 HEAP16[$757>>1] = $752;
 $758 = $p;
 $759 = (($758) + 8)|0;
 $760 = $0;
 $761 = (($760) + ($759<<3)|0);
 $762 = (($761) + 2|0);
 $763 = HEAP16[$762>>1]|0;
 $764 = $p;
 $765 = (($764) + 13)|0;
 $766 = $0;
 $767 = (($766) + ($765<<3)|0);
 $768 = (($767) + 2|0);
 HEAP16[$768>>1] = $763;
 $769 = $p;
 $770 = (($769) + 10)|0;
 $771 = $0;
 $772 = (($771) + ($770<<3)|0);
 $773 = (($772) + 2|0);
 $774 = HEAP16[$773>>1]|0;
 $775 = $p;
 $776 = (($775) + 14)|0;
 $777 = $0;
 $778 = (($777) + ($776<<3)|0);
 $779 = (($778) + 2|0);
 HEAP16[$779>>1] = $774;
 $780 = $p;
 $781 = (($780) + 10)|0;
 $782 = $0;
 $783 = (($782) + ($781<<3)|0);
 $784 = (($783) + 2|0);
 $785 = HEAP16[$784>>1]|0;
 $786 = $p;
 $787 = (($786) + 15)|0;
 $788 = $0;
 $789 = (($788) + ($787<<3)|0);
 $790 = (($789) + 2|0);
 HEAP16[$790>>1] = $785;
 $791 = $p;
 $792 = (($791) + 4)|0;
 $793 = $0;
 $794 = (($793) + ($792<<3)|0);
 $795 = (($794) + 4|0);
 $796 = HEAP16[$795>>1]|0;
 $797 = $p;
 $798 = (($797) + 12)|0;
 $799 = $0;
 $800 = (($799) + ($798<<3)|0);
 $801 = (($800) + 4|0);
 HEAP16[$801>>1] = $796;
 $802 = $p;
 $803 = (($802) + 5)|0;
 $804 = $0;
 $805 = (($804) + ($803<<3)|0);
 $806 = (($805) + 4|0);
 $807 = HEAP16[$806>>1]|0;
 $808 = $p;
 $809 = (($808) + 13)|0;
 $810 = $0;
 $811 = (($810) + ($809<<3)|0);
 $812 = (($811) + 4|0);
 HEAP16[$812>>1] = $807;
 $813 = $p;
 $814 = (($813) + 4)|0;
 $815 = $0;
 $816 = (($815) + ($814<<3)|0);
 $817 = (($816) + 4|0);
 $818 = HEAP16[$817>>1]|0;
 $819 = $p;
 $820 = (($819) + 14)|0;
 $821 = $0;
 $822 = (($821) + ($820<<3)|0);
 $823 = (($822) + 4|0);
 HEAP16[$823>>1] = $818;
 $824 = $p;
 $825 = (($824) + 5)|0;
 $826 = $0;
 $827 = (($826) + ($825<<3)|0);
 $828 = (($827) + 4|0);
 $829 = HEAP16[$828>>1]|0;
 $830 = $p;
 $831 = (($830) + 15)|0;
 $832 = $0;
 $833 = (($832) + ($831<<3)|0);
 $834 = (($833) + 4|0);
 HEAP16[$834>>1] = $829;
 $835 = $p;
 $836 = (($835) + 8)|0;
 $837 = $0;
 $838 = (($837) + ($836<<3)|0);
 $839 = (($838) + 6|0);
 $840 = HEAP16[$839>>1]|0;
 $841 = $p;
 $842 = (($841) + 12)|0;
 $843 = $0;
 $844 = (($843) + ($842<<3)|0);
 $845 = (($844) + 6|0);
 HEAP16[$845>>1] = $840;
 $846 = $p;
 $847 = (($846) + 8)|0;
 $848 = $0;
 $849 = (($848) + ($847<<3)|0);
 $850 = (($849) + 6|0);
 $851 = HEAP16[$850>>1]|0;
 $852 = $p;
 $853 = (($852) + 13)|0;
 $854 = $0;
 $855 = (($854) + ($853<<3)|0);
 $856 = (($855) + 6|0);
 HEAP16[$856>>1] = $851;
 $857 = $p;
 $858 = (($857) + 10)|0;
 $859 = $0;
 $860 = (($859) + ($858<<3)|0);
 $861 = (($860) + 6|0);
 $862 = HEAP16[$861>>1]|0;
 $863 = $p;
 $864 = (($863) + 14)|0;
 $865 = $0;
 $866 = (($865) + ($864<<3)|0);
 $867 = (($866) + 6|0);
 HEAP16[$867>>1] = $862;
 $868 = $p;
 $869 = (($868) + 10)|0;
 $870 = $0;
 $871 = (($870) + ($869<<3)|0);
 $872 = (($871) + 6|0);
 $873 = HEAP16[$872>>1]|0;
 $874 = $p;
 $875 = (($874) + 15)|0;
 $876 = $0;
 $877 = (($876) + ($875<<3)|0);
 $878 = (($877) + 6|0);
 HEAP16[$878>>1] = $873;
 STACKTOP = sp;return;
}
function _q_tree4($q_tree,$start,$lenx,$leny,$x,$y) {
 $q_tree = $q_tree|0;
 $start = $start|0;
 $lenx = $lenx|0;
 $leny = $leny|0;
 $x = $x|0;
 $y = $y|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $evenx = 0, $eveny = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $q_tree;
 $1 = $start;
 $2 = $lenx;
 $3 = $leny;
 $4 = $x;
 $5 = $y;
 $6 = $1;
 $p = $6;
 $7 = $2;
 $8 = (($7|0) % 2)&-1;
 $evenx = $8;
 $9 = $3;
 $10 = (($9|0) % 2)&-1;
 $eveny = $10;
 $11 = $4;
 $12 = $11&65535;
 $13 = $p;
 $14 = $0;
 $15 = (($14) + ($13<<3)|0);
 HEAP16[$15>>1] = $12;
 $16 = $4;
 $17 = $16&65535;
 $18 = $p;
 $19 = (($18) + 2)|0;
 $20 = $0;
 $21 = (($20) + ($19<<3)|0);
 HEAP16[$21>>1] = $17;
 $22 = $5;
 $23 = $22&65535;
 $24 = $p;
 $25 = $0;
 $26 = (($25) + ($24<<3)|0);
 $27 = (($26) + 2|0);
 HEAP16[$27>>1] = $23;
 $28 = $5;
 $29 = $28&65535;
 $30 = $p;
 $31 = (($30) + 1)|0;
 $32 = $0;
 $33 = (($32) + ($31<<3)|0);
 $34 = (($33) + 2|0);
 HEAP16[$34>>1] = $29;
 $35 = $evenx;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = $2;
  $38 = (($37|0) / 2)&-1;
  $39 = $38&65535;
  $40 = $p;
  $41 = $0;
  $42 = (($41) + ($40<<3)|0);
  $43 = (($42) + 4|0);
  HEAP16[$43>>1] = $39;
  $44 = $p;
  $45 = $0;
  $46 = (($45) + ($44<<3)|0);
  $47 = (($46) + 4|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $p;
  $50 = (($49) + 1)|0;
  $51 = $0;
  $52 = (($51) + ($50<<3)|0);
  $53 = (($52) + 4|0);
  HEAP16[$53>>1] = $48;
  $54 = $p;
  $55 = $0;
  $56 = (($55) + ($54<<3)|0);
  $57 = (($56) + 4|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $p;
  $60 = (($59) + 2)|0;
  $61 = $0;
  $62 = (($61) + ($60<<3)|0);
  $63 = (($62) + 4|0);
  HEAP16[$63>>1] = $58;
  $64 = $p;
  $65 = $0;
  $66 = (($65) + ($64<<3)|0);
  $67 = (($66) + 4|0);
  $68 = HEAP16[$67>>1]|0;
  $69 = $p;
  $70 = (($69) + 3)|0;
  $71 = $0;
  $72 = (($71) + ($70<<3)|0);
  $73 = (($72) + 4|0);
  HEAP16[$73>>1] = $68;
 } else {
  $74 = $2;
  $75 = (($74) + 1)|0;
  $76 = (($75|0) / 2)&-1;
  $77 = $76&65535;
  $78 = $p;
  $79 = $0;
  $80 = (($79) + ($78<<3)|0);
  $81 = (($80) + 4|0);
  HEAP16[$81>>1] = $77;
  $82 = $p;
  $83 = $0;
  $84 = (($83) + ($82<<3)|0);
  $85 = (($84) + 4|0);
  $86 = HEAP16[$85>>1]|0;
  $87 = $86 << 16 >> 16;
  $88 = (($87) - 1)|0;
  $89 = $88&65535;
  $90 = $p;
  $91 = (($90) + 1)|0;
  $92 = $0;
  $93 = (($92) + ($91<<3)|0);
  $94 = (($93) + 4|0);
  HEAP16[$94>>1] = $89;
  $95 = $p;
  $96 = $0;
  $97 = (($96) + ($95<<3)|0);
  $98 = (($97) + 4|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $p;
  $101 = (($100) + 2)|0;
  $102 = $0;
  $103 = (($102) + ($101<<3)|0);
  $104 = (($103) + 4|0);
  HEAP16[$104>>1] = $99;
  $105 = $p;
  $106 = (($105) + 1)|0;
  $107 = $0;
  $108 = (($107) + ($106<<3)|0);
  $109 = (($108) + 4|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $p;
  $112 = (($111) + 3)|0;
  $113 = $0;
  $114 = (($113) + ($112<<3)|0);
  $115 = (($114) + 4|0);
  HEAP16[$115>>1] = $110;
 }
 $116 = $4;
 $117 = $p;
 $118 = $0;
 $119 = (($118) + ($117<<3)|0);
 $120 = (($119) + 4|0);
 $121 = HEAP16[$120>>1]|0;
 $122 = $121 << 16 >> 16;
 $123 = (($116) + ($122))|0;
 $124 = $123&65535;
 $125 = $p;
 $126 = (($125) + 1)|0;
 $127 = $0;
 $128 = (($127) + ($126<<3)|0);
 HEAP16[$128>>1] = $124;
 $129 = $p;
 $130 = (($129) + 1)|0;
 $131 = $0;
 $132 = (($131) + ($130<<3)|0);
 $133 = HEAP16[$132>>1]|0;
 $134 = $p;
 $135 = (($134) + 3)|0;
 $136 = $0;
 $137 = (($136) + ($135<<3)|0);
 HEAP16[$137>>1] = $133;
 $138 = $eveny;
 $139 = ($138|0)==(0);
 if ($139) {
  $140 = $3;
  $141 = (($140|0) / 2)&-1;
  $142 = $141&65535;
  $143 = $p;
  $144 = $0;
  $145 = (($144) + ($143<<3)|0);
  $146 = (($145) + 6|0);
  HEAP16[$146>>1] = $142;
  $147 = $p;
  $148 = $0;
  $149 = (($148) + ($147<<3)|0);
  $150 = (($149) + 6|0);
  $151 = HEAP16[$150>>1]|0;
  $152 = $p;
  $153 = (($152) + 1)|0;
  $154 = $0;
  $155 = (($154) + ($153<<3)|0);
  $156 = (($155) + 6|0);
  HEAP16[$156>>1] = $151;
  $157 = $p;
  $158 = $0;
  $159 = (($158) + ($157<<3)|0);
  $160 = (($159) + 6|0);
  $161 = HEAP16[$160>>1]|0;
  $162 = $p;
  $163 = (($162) + 2)|0;
  $164 = $0;
  $165 = (($164) + ($163<<3)|0);
  $166 = (($165) + 6|0);
  HEAP16[$166>>1] = $161;
  $167 = $p;
  $168 = $0;
  $169 = (($168) + ($167<<3)|0);
  $170 = (($169) + 6|0);
  $171 = HEAP16[$170>>1]|0;
  $172 = $p;
  $173 = (($172) + 3)|0;
  $174 = $0;
  $175 = (($174) + ($173<<3)|0);
  $176 = (($175) + 6|0);
  HEAP16[$176>>1] = $171;
  $219 = $5;
  $220 = $p;
  $221 = $0;
  $222 = (($221) + ($220<<3)|0);
  $223 = (($222) + 6|0);
  $224 = HEAP16[$223>>1]|0;
  $225 = $224 << 16 >> 16;
  $226 = (($219) + ($225))|0;
  $227 = $226&65535;
  $228 = $p;
  $229 = (($228) + 2)|0;
  $230 = $0;
  $231 = (($230) + ($229<<3)|0);
  $232 = (($231) + 2|0);
  HEAP16[$232>>1] = $227;
  $233 = $p;
  $234 = (($233) + 2)|0;
  $235 = $0;
  $236 = (($235) + ($234<<3)|0);
  $237 = (($236) + 2|0);
  $238 = HEAP16[$237>>1]|0;
  $239 = $p;
  $240 = (($239) + 3)|0;
  $241 = $0;
  $242 = (($241) + ($240<<3)|0);
  $243 = (($242) + 2|0);
  HEAP16[$243>>1] = $238;
  STACKTOP = sp;return;
 } else {
  $177 = $3;
  $178 = (($177) + 1)|0;
  $179 = (($178|0) / 2)&-1;
  $180 = $179&65535;
  $181 = $p;
  $182 = $0;
  $183 = (($182) + ($181<<3)|0);
  $184 = (($183) + 6|0);
  HEAP16[$184>>1] = $180;
  $185 = $p;
  $186 = $0;
  $187 = (($186) + ($185<<3)|0);
  $188 = (($187) + 6|0);
  $189 = HEAP16[$188>>1]|0;
  $190 = $p;
  $191 = (($190) + 1)|0;
  $192 = $0;
  $193 = (($192) + ($191<<3)|0);
  $194 = (($193) + 6|0);
  HEAP16[$194>>1] = $189;
  $195 = $p;
  $196 = $0;
  $197 = (($196) + ($195<<3)|0);
  $198 = (($197) + 6|0);
  $199 = HEAP16[$198>>1]|0;
  $200 = $199 << 16 >> 16;
  $201 = (($200) - 1)|0;
  $202 = $201&65535;
  $203 = $p;
  $204 = (($203) + 2)|0;
  $205 = $0;
  $206 = (($205) + ($204<<3)|0);
  $207 = (($206) + 6|0);
  HEAP16[$207>>1] = $202;
  $208 = $p;
  $209 = (($208) + 2)|0;
  $210 = $0;
  $211 = (($210) + ($209<<3)|0);
  $212 = (($211) + 6|0);
  $213 = HEAP16[$212>>1]|0;
  $214 = $p;
  $215 = (($214) + 3)|0;
  $216 = $0;
  $217 = (($216) + ($215<<3)|0);
  $218 = (($217) + 6|0);
  HEAP16[$218>>1] = $213;
  $219 = $5;
  $220 = $p;
  $221 = $0;
  $222 = (($221) + ($220<<3)|0);
  $223 = (($222) + 6|0);
  $224 = HEAP16[$223>>1]|0;
  $225 = $224 << 16 >> 16;
  $226 = (($219) + ($225))|0;
  $227 = $226&65535;
  $228 = $p;
  $229 = (($228) + 2)|0;
  $230 = $0;
  $231 = (($230) + ($229<<3)|0);
  $232 = (($231) + 2|0);
  HEAP16[$232>>1] = $227;
  $233 = $p;
  $234 = (($233) + 2)|0;
  $235 = $0;
  $236 = (($235) + ($234<<3)|0);
  $237 = (($236) + 2|0);
  $238 = HEAP16[$237>>1]|0;
  $239 = $p;
  $240 = (($239) + 3)|0;
  $241 = $0;
  $242 = (($241) + ($240<<3)|0);
  $243 = (($242) + 2|0);
  HEAP16[$243>>1] = $238;
  STACKTOP = sp;return;
 }
}
function _conv_img_2_uchar($data,$img,$width,$height,$m_shift,$r_scale) {
 $data = $data|0;
 $img = $img|0;
 $width = $width|0;
 $height = $height|0;
 $m_shift = +$m_shift;
 $r_scale = +$r_scale;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0.0, $40 = 0, $5 = 0.0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $c = 0, $img_tmp = 0.0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $data;
 $1 = $img;
 $2 = $width;
 $3 = $height;
 $4 = $m_shift;
 $5 = $r_scale;
 $r = 0;
 while(1) {
  $6 = $r;
  $7 = $3;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $c = 0;
  while(1) {
   $9 = $c;
   $10 = $2;
   $11 = ($9|0)<($10|0);
   if (!($11)) {
    break;
   }
   $12 = $1;
   $13 = +HEAPF32[$12>>2];
   $14 = $5;
   $15 = $13 * $14;
   $16 = $4;
   $17 = $15 + $16;
   $img_tmp = $17;
   $18 = $img_tmp;
   $19 = $18;
   $20 = $19 + 0.5;
   $21 = $20;
   $img_tmp = $21;
   $22 = $img_tmp;
   $23 = $22;
   $24 = $23 < 0.0;
   if ($24) {
    $25 = $0;
    HEAP8[$25>>0] = 0;
   } else {
    $26 = $img_tmp;
    $27 = $26;
    $28 = $27 > 255.0;
    if ($28) {
     $29 = $0;
     HEAP8[$29>>0] = -1;
    } else {
     $30 = $img_tmp;
     $31 = (~~(($30))&255);
     $32 = $0;
     HEAP8[$32>>0] = $31;
    }
   }
   $33 = $1;
   $34 = (($33) + 4|0);
   $1 = $34;
   $35 = $0;
   $36 = (($35) + 1|0);
   $0 = $36;
   $37 = $c;
   $38 = (($37) + 1)|0;
   $c = $38;
  }
  $39 = $r;
  $40 = (($39) + 1)|0;
  $r = $40;
 }
 STACKTOP = sp;return;
}
function _unquantize($ofip,$dqt_table,$q_tree,$q_treelen,$sip,$width,$height) {
 $ofip = $ofip|0;
 $dqt_table = $dqt_table|0;
 $q_tree = $q_tree|0;
 $q_treelen = $q_treelen|0;
 $sip = $sip|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0.0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0.0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $C = 0.0, $cnt = 0, $col = 0, $fip = 0, $fptr = 0, $row = 0, $sptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofip;
 $2 = $dqt_table;
 $3 = $q_tree;
 $4 = $q_treelen;
 $5 = $sip;
 $6 = $width;
 $7 = $height;
 $8 = $6;
 $9 = $7;
 $10 = Math_imul($8, $9)|0;
 $11 = (_calloc($10,4)|0);
 $fip = $11;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  (_fprintf(($13|0),(41392|0),($vararg_buffer|0))|0);
  $0 = -91;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $14 = $2;
 $15 = (($14) + 516|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16 << 24 >> 24;
 $18 = ($17|0)!=(1);
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(41432|0),($vararg_buffer1|0))|0);
  $0 = -92;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $20 = $5;
 $sptr = $20;
 $21 = $2;
 $22 = +HEAPF32[$21>>2];
 $C = $22;
 $cnt = 0;
 L9: while(1) {
  $23 = $cnt;
  $24 = ($23|0)<(60);
  if (!($24)) {
   label = 28;
   break;
  }
  $25 = $cnt;
  $26 = $2;
  $27 = (($26) + 4|0);
  $28 = (($27) + ($25<<2)|0);
  $29 = +HEAPF32[$28>>2];
  $30 = $29;
  $31 = $30 == 0.0;
  if ($31) {
  } else {
   $32 = $fip;
   $33 = $cnt;
   $34 = $3;
   $35 = (($34) + ($33<<3)|0);
   $36 = (($35) + 2|0);
   $37 = HEAP16[$36>>1]|0;
   $38 = $37 << 16 >> 16;
   $39 = $6;
   $40 = Math_imul($38, $39)|0;
   $41 = (($32) + ($40<<2)|0);
   $42 = $cnt;
   $43 = $3;
   $44 = (($43) + ($42<<3)|0);
   $45 = HEAP16[$44>>1]|0;
   $46 = $45 << 16 >> 16;
   $47 = (($41) + ($46<<2)|0);
   $fptr = $47;
   $row = 0;
   while(1) {
    $48 = $row;
    $49 = $cnt;
    $50 = $3;
    $51 = (($50) + ($49<<3)|0);
    $52 = (($51) + 6|0);
    $53 = HEAP16[$52>>1]|0;
    $54 = $53 << 16 >> 16;
    $55 = ($48|0)<($54|0);
    if (!($55)) {
     break;
    }
    $col = 0;
    while(1) {
     $56 = $col;
     $57 = $cnt;
     $58 = $3;
     $59 = (($58) + ($57<<3)|0);
     $60 = (($59) + 4|0);
     $61 = HEAP16[$60>>1]|0;
     $62 = $61 << 16 >> 16;
     $63 = ($56|0)<($62|0);
     if (!($63)) {
      break;
     }
     $64 = $sptr;
     $65 = HEAP16[$64>>1]|0;
     $66 = $65 << 16 >> 16;
     $67 = ($66|0)==(0);
     if ($67) {
      $68 = $fptr;
      HEAPF32[$68>>2] = 0.0;
     } else {
      $69 = $sptr;
      $70 = HEAP16[$69>>1]|0;
      $71 = $70 << 16 >> 16;
      $72 = ($71|0)>(0);
      if ($72) {
       $73 = $cnt;
       $74 = $2;
       $75 = (($74) + 4|0);
       $76 = (($75) + ($73<<2)|0);
       $77 = +HEAPF32[$76>>2];
       $78 = $sptr;
       $79 = HEAP16[$78>>1]|0;
       $80 = (+($79<<16>>16));
       $81 = $C;
       $82 = $80 - $81;
       $83 = $77 * $82;
       $84 = $83;
       $85 = $cnt;
       $86 = $2;
       $87 = (($86) + 260|0);
       $88 = (($87) + ($85<<2)|0);
       $89 = +HEAPF32[$88>>2];
       $90 = $89;
       $91 = $90 / 2.0;
       $92 = $84 + $91;
       $93 = $92;
       $94 = $fptr;
       HEAPF32[$94>>2] = $93;
      } else {
       $95 = $sptr;
       $96 = HEAP16[$95>>1]|0;
       $97 = $96 << 16 >> 16;
       $98 = ($97|0)<(0);
       if (!($98)) {
        label = 19;
        break L9;
       }
       $99 = $cnt;
       $100 = $2;
       $101 = (($100) + 4|0);
       $102 = (($101) + ($99<<2)|0);
       $103 = +HEAPF32[$102>>2];
       $104 = $sptr;
       $105 = HEAP16[$104>>1]|0;
       $106 = (+($105<<16>>16));
       $107 = $C;
       $108 = $106 + $107;
       $109 = $103 * $108;
       $110 = $109;
       $111 = $cnt;
       $112 = $2;
       $113 = (($112) + 260|0);
       $114 = (($113) + ($111<<2)|0);
       $115 = +HEAPF32[$114>>2];
       $116 = $115;
       $117 = $116 / 2.0;
       $118 = $110 - $117;
       $119 = $118;
       $120 = $fptr;
       HEAPF32[$120>>2] = $119;
      }
     }
     $122 = $fptr;
     $123 = (($122) + 4|0);
     $fptr = $123;
     $124 = $sptr;
     $125 = (($124) + 2|0);
     $sptr = $125;
     $126 = $col;
     $127 = (($126) + 1)|0;
     $col = $127;
    }
    $128 = $row;
    $129 = (($128) + 1)|0;
    $row = $129;
    $130 = $6;
    $131 = $cnt;
    $132 = $3;
    $133 = (($132) + ($131<<3)|0);
    $134 = (($133) + 4|0);
    $135 = HEAP16[$134>>1]|0;
    $136 = $135 << 16 >> 16;
    $137 = (($130) - ($136))|0;
    $138 = $fptr;
    $139 = (($138) + ($137<<2)|0);
    $fptr = $139;
   }
  }
  $140 = $cnt;
  $141 = (($140) + 1)|0;
  $cnt = $141;
 }
 if ((label|0) == 19) {
  $121 = HEAP32[_stderr>>2]|0;
  (_fprintf(($121|0),(41496|0),($vararg_buffer3|0))|0);
  $0 = -93;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 else if ((label|0) == 28) {
  $142 = $fip;
  $143 = $1;
  HEAP32[$143>>2] = $142;
  $0 = 0;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 return 0|0;
}
function _wsq_reconstruct($fdata,$width,$height,$w_tree,$w_treelen,$dtt_table) {
 $fdata = $fdata|0;
 $width = $width|0;
 $height = $height|0;
 $w_tree = $w_tree|0;
 $w_treelen = $w_treelen|0;
 $dtt_table = $dtt_table|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $fdata1 = 0, $fdata_bse = 0, $node = 0, $num_pix = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $fdata;
 $2 = $width;
 $3 = $height;
 $4 = $w_tree;
 $5 = $w_treelen;
 $6 = $dtt_table;
 $7 = $6;
 $8 = (($7) + 10|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($10|0)!=(1);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  (_fprintf(($12|0),(41552|0),($vararg_buffer|0))|0);
  $0 = -95;
  $110 = $0;
  STACKTOP = sp;return ($110|0);
 }
 $13 = $6;
 $14 = (($13) + 11|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)!=(1);
 if ($17) {
  $18 = HEAP32[_stderr>>2]|0;
  (_fprintf(($18|0),(41624|0),($vararg_buffer1|0))|0);
  $0 = -96;
  $110 = $0;
  STACKTOP = sp;return ($110|0);
 }
 $19 = $2;
 $20 = $3;
 $21 = Math_imul($19, $20)|0;
 $num_pix = $21;
 $22 = $num_pix;
 $23 = $22<<2;
 $24 = (_malloc($23)|0);
 $fdata1 = $24;
 $25 = ($24|0)==(0|0);
 if ($25) {
  $26 = HEAP32[_stderr>>2]|0;
  (_fprintf(($26|0),(41696|0),($vararg_buffer3|0))|0);
  $0 = -97;
  $110 = $0;
  STACKTOP = sp;return ($110|0);
 }
 $27 = $5;
 $28 = (($27) - 1)|0;
 $node = $28;
 while(1) {
  $29 = $node;
  $30 = ($29|0)>=(0);
  if (!($30)) {
   break;
  }
  $31 = $1;
  $32 = $node;
  $33 = $4;
  $34 = (($33) + (($32*24)|0)|0);
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $2;
  $38 = Math_imul($36, $37)|0;
  $39 = (($31) + ($38<<2)|0);
  $40 = $node;
  $41 = $4;
  $42 = (($41) + (($40*24)|0)|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($39) + ($43<<2)|0);
  $fdata_bse = $44;
  $45 = $fdata1;
  $46 = $fdata_bse;
  $47 = $node;
  $48 = $4;
  $49 = (($48) + (($47*24)|0)|0);
  $50 = (($49) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = $node;
  $53 = $4;
  $54 = (($53) + (($52*24)|0)|0);
  $55 = (($54) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $2;
  $58 = $6;
  $59 = (($58) + 4|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = $6;
  $62 = (($61) + 9|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = $63&255;
  $65 = $6;
  $66 = HEAP32[$65>>2]|0;
  $67 = $6;
  $68 = (($67) + 8|0);
  $69 = HEAP8[$68>>0]|0;
  $70 = $69&255;
  $71 = $node;
  $72 = $4;
  $73 = (($72) + (($71*24)|0)|0);
  $74 = (($73) + 20|0);
  $75 = HEAP32[$74>>2]|0;
  _join_lets($45,$46,$51,$56,1,$57,$60,$64,$66,$70,$75);
  $76 = $fdata_bse;
  $77 = $fdata1;
  $78 = $node;
  $79 = $4;
  $80 = (($79) + (($78*24)|0)|0);
  $81 = (($80) + 12|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = $node;
  $84 = $4;
  $85 = (($84) + (($83*24)|0)|0);
  $86 = (($85) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = $2;
  $89 = $6;
  $90 = (($89) + 4|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = $6;
  $93 = (($92) + 9|0);
  $94 = HEAP8[$93>>0]|0;
  $95 = $94&255;
  $96 = $6;
  $97 = HEAP32[$96>>2]|0;
  $98 = $6;
  $99 = (($98) + 8|0);
  $100 = HEAP8[$99>>0]|0;
  $101 = $100&255;
  $102 = $node;
  $103 = $4;
  $104 = (($103) + (($102*24)|0)|0);
  $105 = (($104) + 16|0);
  $106 = HEAP32[$105>>2]|0;
  _join_lets($76,$77,$82,$87,$88,1,$91,$95,$97,$101,$106);
  $107 = $node;
  $108 = (($107) + -1)|0;
  $node = $108;
 }
 $109 = $fdata1;
 _free($109);
 $0 = 0;
 $110 = $0;
 STACKTOP = sp;return ($110|0);
}
function _join_lets($new,$old,$len1,$len2,$pitch,$stride,$hi,$hsz,$lo,$lsz,$inv) {
 $new = $new|0;
 $old = $old|0;
 $len1 = $len1|0;
 $len2 = $len2|0;
 $pitch = $pitch|0;
 $stride = $stride|0;
 $hi = $hi|0;
 $hsz = $hsz|0;
 $lo = $lo|0;
 $lsz = $lsz|0;
 $inv = $inv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0.0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0.0, $185 = 0, $186 = 0, $187 = 0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0.0, $192 = 0.0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0.0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0.0, $241 = 0.0;
 var $242 = 0.0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0.0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0.0, $253 = 0.0, $254 = 0.0, $255 = 0.0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0.0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0, $324 = 0, $325 = 0, $326 = 0.0, $327 = 0.0, $328 = 0, $329 = 0.0, $33 = 0, $330 = 0.0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0.0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0.0, $384 = 0.0, $385 = 0.0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0.0, $392 = 0, $393 = 0, $394 = 0, $395 = 0.0, $396 = 0.0, $397 = 0.0, $398 = 0.0, $399 = 0, $4 = 0, $40 = 0, $400 = 0.0, $401 = 0.0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0.0, $423 = 0.0, $424 = 0.0, $425 = 0.0, $426 = 0, $427 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0.0;
 var $74 = 0.0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $asym = 0, $cl_rw = 0, $da_ev = 0, $fhre = 0, $fi_ev = 0, $himg = 0, $hipass = 0, $hle = 0, $hle2 = 0, $hlen = 0, $hoc = 0, $hotap = 0;
 var $hp0 = 0, $hp1 = 0, $hpx = 0, $hpxstr = 0, $hre = 0, $hre2 = 0, $hspx = 0, $hspxstr = 0, $hstap = 0, $i = 0, $limg = 0, $lle = 0, $lle2 = 0, $llen = 0, $loc = 0, $lopass = 0, $lotap = 0, $lp0 = 0, $lp1 = 0, $lpx = 0;
 var $lpxstr = 0, $lre = 0, $lre2 = 0, $lspx = 0, $lspxstr = 0, $lstap = 0, $nstr = 0, $ofhre = 0, $ohle = 0, $ohre = 0, $olle = 0, $olre = 0, $osfac = 0.0, $pix = 0, $pstr = 0, $sfac = 0.0, $ssfac = 0.0, $tap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $0 = $new;
 $1 = $old;
 $2 = $len1;
 $3 = $len2;
 $4 = $pitch;
 $5 = $stride;
 $6 = $hi;
 $7 = $hsz;
 $8 = $lo;
 $9 = $lsz;
 $10 = $inv;
 $fhre = 0;
 $11 = $3;
 $12 = (($11|0) % 2)&-1;
 $da_ev = $12;
 $13 = $9;
 $14 = (($13|0) % 2)&-1;
 $fi_ev = $14;
 $15 = $5;
 $pstr = $15;
 $16 = $pstr;
 $17 = (0 - ($16))|0;
 $nstr = $17;
 $18 = $da_ev;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = $3;
  $21 = (($20) + 1)|0;
  $22 = (($21|0) / 2)&-1;
  $llen = $22;
  $23 = $llen;
  $24 = (($23) - 1)|0;
  $hlen = $24;
 } else {
  $25 = $3;
  $26 = (($25|0) / 2)&-1;
  $llen = $26;
  $27 = $llen;
  $hlen = $27;
 }
 $28 = $fi_ev;
 $29 = ($28|0)!=(0);
 if ($29) {
  $asym = 0;
  $ssfac = 1.0;
  $ofhre = 0;
  $30 = $9;
  $31 = (($30) - 1)|0;
  $32 = (($31|0) / 4)&-1;
  $loc = $32;
  $33 = $7;
  $34 = (($33) + 1)|0;
  $35 = (($34|0) / 4)&-1;
  $36 = (($35) - 1)|0;
  $hoc = $36;
  $37 = $9;
  $38 = (($37) - 1)|0;
  $39 = (($38|0) / 2)&-1;
  $40 = (($39|0) % 2)&-1;
  $lotap = $40;
  $41 = $7;
  $42 = (($41) + 1)|0;
  $43 = (($42|0) / 2)&-1;
  $44 = (($43|0) % 2)&-1;
  $hotap = $44;
  $45 = $da_ev;
  $46 = ($45|0)!=(0);
  if ($46) {
   $olle = 0;
   $olre = 0;
   $ohle = 1;
   $ohre = 1;
  } else {
   $olle = 0;
   $olre = 1;
   $ohle = 1;
   $ohre = 0;
  }
 } else {
  $asym = 1;
  $ssfac = -1.0;
  $ofhre = 2;
  $47 = $9;
  $48 = (($47|0) / 4)&-1;
  $49 = (($48) - 1)|0;
  $loc = $49;
  $50 = $7;
  $51 = (($50|0) / 4)&-1;
  $52 = (($51) - 1)|0;
  $hoc = $52;
  $53 = $9;
  $54 = (($53|0) / 2)&-1;
  $55 = (($54|0) % 2)&-1;
  $lotap = $55;
  $56 = $7;
  $57 = (($56|0) / 2)&-1;
  $58 = (($57|0) % 2)&-1;
  $hotap = $58;
  $59 = $da_ev;
  $60 = ($59|0)!=(0);
  if ($60) {
   $olle = 1;
   $olre = 0;
   $ohle = 1;
   $ohre = 1;
  } else {
   $olle = 1;
   $olre = 1;
   $ohle = 1;
   $ohre = 1;
  }
  $61 = $loc;
  $62 = ($61|0)==(-1);
  if ($62) {
   $loc = 0;
   $olle = 0;
  }
  $63 = $hoc;
  $64 = ($63|0)==(-1);
  if ($64) {
   $hoc = 0;
   $ohle = 0;
  }
  $i = 0;
  while(1) {
   $65 = $i;
   $66 = $7;
   $67 = ($65|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = $i;
   $69 = $6;
   $70 = (($69) + ($68<<2)|0);
   $71 = +HEAPF32[$70>>2];
   $72 = $71;
   $73 = $72 * -1.0;
   $74 = $73;
   HEAPF32[$70>>2] = $74;
   $75 = $i;
   $76 = (($75) + 1)|0;
   $i = $76;
  }
 }
 $cl_rw = 0;
 while(1) {
  $77 = $cl_rw;
  $78 = $2;
  $79 = ($77|0)<($78|0);
  if (!($79)) {
   break;
  }
  $80 = $0;
  $81 = $cl_rw;
  $82 = $4;
  $83 = Math_imul($81, $82)|0;
  $84 = (($80) + ($83<<2)|0);
  $limg = $84;
  $85 = $limg;
  $himg = $85;
  $86 = $himg;
  HEAPF32[$86>>2] = 0.0;
  $87 = $himg;
  $88 = $5;
  $89 = (($87) + ($88<<2)|0);
  HEAPF32[$89>>2] = 0.0;
  $90 = $10;
  $91 = ($90|0)!=(0);
  if ($91) {
   $92 = $1;
   $93 = $cl_rw;
   $94 = $4;
   $95 = Math_imul($93, $94)|0;
   $96 = (($92) + ($95<<2)|0);
   $hipass = $96;
   $97 = $hipass;
   $98 = $5;
   $99 = $hlen;
   $100 = Math_imul($98, $99)|0;
   $101 = (($97) + ($100<<2)|0);
   $lopass = $101;
  } else {
   $102 = $1;
   $103 = $cl_rw;
   $104 = $4;
   $105 = Math_imul($103, $104)|0;
   $106 = (($102) + ($105<<2)|0);
   $lopass = $106;
   $107 = $lopass;
   $108 = $5;
   $109 = $llen;
   $110 = Math_imul($108, $109)|0;
   $111 = (($107) + ($110<<2)|0);
   $hipass = $111;
  }
  $112 = $lopass;
  $lp0 = $112;
  $113 = $lp0;
  $114 = $llen;
  $115 = (($114) - 1)|0;
  $116 = $5;
  $117 = Math_imul($115, $116)|0;
  $118 = (($113) + ($117<<2)|0);
  $lp1 = $118;
  $119 = $lp0;
  $120 = $loc;
  $121 = $5;
  $122 = Math_imul($120, $121)|0;
  $123 = (($119) + ($122<<2)|0);
  $lspx = $123;
  $124 = $nstr;
  $lspxstr = $124;
  $125 = $lotap;
  $lstap = $125;
  $126 = $olle;
  $lle2 = $126;
  $127 = $olre;
  $lre2 = $127;
  $128 = $hipass;
  $hp0 = $128;
  $129 = $hp0;
  $130 = $hlen;
  $131 = (($130) - 1)|0;
  $132 = $5;
  $133 = Math_imul($131, $132)|0;
  $134 = (($129) + ($133<<2)|0);
  $hp1 = $134;
  $135 = $hp0;
  $136 = $hoc;
  $137 = $5;
  $138 = Math_imul($136, $137)|0;
  $139 = (($135) + ($138<<2)|0);
  $hspx = $139;
  $140 = $nstr;
  $hspxstr = $140;
  $141 = $hotap;
  $hstap = $141;
  $142 = $ohle;
  $hle2 = $142;
  $143 = $ohre;
  $hre2 = $143;
  $144 = $ssfac;
  $osfac = $144;
  $pix = 0;
  while(1) {
   $145 = $pix;
   $146 = $hlen;
   $147 = ($145|0)<($146|0);
   if (!($147)) {
    break;
   }
   $148 = $lstap;
   $tap = $148;
   while(1) {
    $149 = $tap;
    $150 = ($149|0)>=(0);
    if (!($150)) {
     break;
    }
    $151 = $lle2;
    $lle = $151;
    $152 = $lre2;
    $lre = $152;
    $153 = $lspx;
    $lpx = $153;
    $154 = $lspxstr;
    $lpxstr = $154;
    $155 = $lpx;
    $156 = +HEAPF32[$155>>2];
    $157 = $tap;
    $158 = $8;
    $159 = (($158) + ($157<<2)|0);
    $160 = +HEAPF32[$159>>2];
    $161 = $156 * $160;
    $162 = $limg;
    HEAPF32[$162>>2] = $161;
    $163 = $tap;
    $164 = (($163) + 2)|0;
    $i = $164;
    while(1) {
     $165 = $i;
     $166 = $9;
     $167 = ($165|0)<($166|0);
     if (!($167)) {
      break;
     }
     $168 = $lpx;
     $169 = $lp0;
     $170 = ($168|0)==($169|0);
     if ($170) {
      $171 = $lle;
      $172 = ($171|0)!=(0);
      if ($172) {
       $lpxstr = 0;
       $lle = 0;
      } else {
       $173 = $pstr;
       $lpxstr = $173;
      }
     }
     $174 = $lpx;
     $175 = $lp1;
     $176 = ($174|0)==($175|0);
     if ($176) {
      $177 = $lre;
      $178 = ($177|0)!=(0);
      if ($178) {
       $lpxstr = 0;
       $lre = 0;
      } else {
       $179 = $nstr;
       $lpxstr = $179;
      }
     }
     $180 = $lpxstr;
     $181 = $lpx;
     $182 = (($181) + ($180<<2)|0);
     $lpx = $182;
     $183 = $lpx;
     $184 = +HEAPF32[$183>>2];
     $185 = $i;
     $186 = $8;
     $187 = (($186) + ($185<<2)|0);
     $188 = +HEAPF32[$187>>2];
     $189 = $184 * $188;
     $190 = $limg;
     $191 = +HEAPF32[$190>>2];
     $192 = $191 + $189;
     HEAPF32[$190>>2] = $192;
     $193 = $i;
     $194 = (($193) + 2)|0;
     $i = $194;
    }
    $195 = $5;
    $196 = $limg;
    $197 = (($196) + ($195<<2)|0);
    $limg = $197;
    $198 = $tap;
    $199 = (($198) + -1)|0;
    $tap = $199;
   }
   $200 = $lspx;
   $201 = $lp0;
   $202 = ($200|0)==($201|0);
   if ($202) {
    $203 = $lle2;
    $204 = ($203|0)!=(0);
    if ($204) {
     $lspxstr = 0;
     $lle2 = 0;
    } else {
     $205 = $pstr;
     $lspxstr = $205;
    }
   }
   $206 = $lspxstr;
   $207 = $lspx;
   $208 = (($207) + ($206<<2)|0);
   $lspx = $208;
   $lstap = 1;
   $209 = $hstap;
   $tap = $209;
   while(1) {
    $210 = $tap;
    $211 = ($210|0)>=(0);
    if (!($211)) {
     break;
    }
    $212 = $hle2;
    $hle = $212;
    $213 = $hre2;
    $hre = $213;
    $214 = $hspx;
    $hpx = $214;
    $215 = $hspxstr;
    $hpxstr = $215;
    $216 = $ofhre;
    $fhre = $216;
    $217 = $osfac;
    $sfac = $217;
    $218 = $tap;
    $i = $218;
    while(1) {
     $219 = $i;
     $220 = $7;
     $221 = ($219|0)<($220|0);
     if (!($221)) {
      break;
     }
     $222 = $hpx;
     $223 = $hp0;
     $224 = ($222|0)==($223|0);
     if ($224) {
      $225 = $hle;
      $226 = ($225|0)!=(0);
      if ($226) {
       $hpxstr = 0;
       $hle = 0;
      } else {
       $227 = $pstr;
       $hpxstr = $227;
       $sfac = 1.0;
      }
     }
     $228 = $hpx;
     $229 = $hp1;
     $230 = ($228|0)==($229|0);
     if ($230) {
      $231 = $hre;
      $232 = ($231|0)!=(0);
      if ($232) {
       $hpxstr = 0;
       $hre = 0;
       $233 = $asym;
       $234 = ($233|0)!=(0);
       if ($234) {
        $235 = $da_ev;
        $236 = ($235|0)!=(0);
        if ($236) {
         $hre = 1;
         $237 = $fhre;
         $238 = (($237) + -1)|0;
         $fhre = $238;
         $239 = $fhre;
         $240 = (+($239|0));
         $sfac = $240;
         $241 = $sfac;
         $242 = $241;
         $243 = $242 == 0.0;
         if ($243) {
          $hre = 0;
         }
        }
       }
      } else {
       $244 = $nstr;
       $hpxstr = $244;
       $245 = $asym;
       $246 = ($245|0)!=(0);
       if ($246) {
        $sfac = -1.0;
       }
      }
     }
     $247 = $hpx;
     $248 = +HEAPF32[$247>>2];
     $249 = $i;
     $250 = $6;
     $251 = (($250) + ($249<<2)|0);
     $252 = +HEAPF32[$251>>2];
     $253 = $248 * $252;
     $254 = $sfac;
     $255 = $253 * $254;
     $256 = $himg;
     $257 = +HEAPF32[$256>>2];
     $258 = $257 + $255;
     HEAPF32[$256>>2] = $258;
     $259 = $hpxstr;
     $260 = $hpx;
     $261 = (($260) + ($259<<2)|0);
     $hpx = $261;
     $262 = $i;
     $263 = (($262) + 2)|0;
     $i = $263;
    }
    $264 = $5;
    $265 = $himg;
    $266 = (($265) + ($264<<2)|0);
    $himg = $266;
    $267 = $tap;
    $268 = (($267) + -1)|0;
    $tap = $268;
   }
   $269 = $hspx;
   $270 = $hp0;
   $271 = ($269|0)==($270|0);
   if ($271) {
    $272 = $hle2;
    $273 = ($272|0)!=(0);
    if ($273) {
     $hspxstr = 0;
     $hle2 = 0;
    } else {
     $274 = $pstr;
     $hspxstr = $274;
     $osfac = 1.0;
    }
   }
   $275 = $hspxstr;
   $276 = $hspx;
   $277 = (($276) + ($275<<2)|0);
   $hspx = $277;
   $hstap = 1;
   $278 = $pix;
   $279 = (($278) + 1)|0;
   $pix = $279;
  }
  $280 = $da_ev;
  $281 = ($280|0)!=(0);
  if ($281) {
   $282 = $lotap;
   $283 = ($282|0)!=(0);
   if ($283) {
    $lstap = 1;
   } else {
    $lstap = 0;
   }
  } else {
   $284 = $lotap;
   $285 = ($284|0)!=(0);
   if ($285) {
    $lstap = 2;
   } else {
    $lstap = 1;
   }
  }
  $tap = 1;
  while(1) {
   $286 = $tap;
   $287 = $lstap;
   $288 = ($286|0)>=($287|0);
   if (!($288)) {
    break;
   }
   $289 = $lle2;
   $lle = $289;
   $290 = $lre2;
   $lre = $290;
   $291 = $lspx;
   $lpx = $291;
   $292 = $lspxstr;
   $lpxstr = $292;
   $293 = $lpx;
   $294 = +HEAPF32[$293>>2];
   $295 = $tap;
   $296 = $8;
   $297 = (($296) + ($295<<2)|0);
   $298 = +HEAPF32[$297>>2];
   $299 = $294 * $298;
   $300 = $limg;
   HEAPF32[$300>>2] = $299;
   $301 = $tap;
   $302 = (($301) + 2)|0;
   $i = $302;
   while(1) {
    $303 = $i;
    $304 = $9;
    $305 = ($303|0)<($304|0);
    if (!($305)) {
     break;
    }
    $306 = $lpx;
    $307 = $lp0;
    $308 = ($306|0)==($307|0);
    if ($308) {
     $309 = $lle;
     $310 = ($309|0)!=(0);
     if ($310) {
      $lpxstr = 0;
      $lle = 0;
     } else {
      $311 = $pstr;
      $lpxstr = $311;
     }
    }
    $312 = $lpx;
    $313 = $lp1;
    $314 = ($312|0)==($313|0);
    if ($314) {
     $315 = $lre;
     $316 = ($315|0)!=(0);
     if ($316) {
      $lpxstr = 0;
      $lre = 0;
     } else {
      $317 = $nstr;
      $lpxstr = $317;
     }
    }
    $318 = $lpxstr;
    $319 = $lpx;
    $320 = (($319) + ($318<<2)|0);
    $lpx = $320;
    $321 = $lpx;
    $322 = +HEAPF32[$321>>2];
    $323 = $i;
    $324 = $8;
    $325 = (($324) + ($323<<2)|0);
    $326 = +HEAPF32[$325>>2];
    $327 = $322 * $326;
    $328 = $limg;
    $329 = +HEAPF32[$328>>2];
    $330 = $329 + $327;
    HEAPF32[$328>>2] = $330;
    $331 = $i;
    $332 = (($331) + 2)|0;
    $i = $332;
   }
   $333 = $5;
   $334 = $limg;
   $335 = (($334) + ($333<<2)|0);
   $limg = $335;
   $336 = $tap;
   $337 = (($336) + -1)|0;
   $tap = $337;
  }
  $338 = $da_ev;
  $339 = ($338|0)!=(0);
  if ($339) {
   $340 = $hotap;
   $341 = ($340|0)!=(0);
   if ($341) {
    $hstap = 1;
   } else {
    $hstap = 0;
   }
   $342 = $7;
   $343 = ($342|0)==(2);
   if ($343) {
    $344 = $hspxstr;
    $345 = $hspx;
    $346 = (0 - ($344))|0;
    $347 = (($345) + ($346<<2)|0);
    $hspx = $347;
    $fhre = 1;
   }
  } else {
   $348 = $hotap;
   $349 = ($348|0)!=(0);
   if ($349) {
    $hstap = 2;
   } else {
    $hstap = 1;
   }
  }
  $tap = 1;
  while(1) {
   $350 = $tap;
   $351 = $hstap;
   $352 = ($350|0)>=($351|0);
   if (!($352)) {
    break;
   }
   $353 = $hle2;
   $hle = $353;
   $354 = $hre2;
   $hre = $354;
   $355 = $hspx;
   $hpx = $355;
   $356 = $hspxstr;
   $hpxstr = $356;
   $357 = $osfac;
   $sfac = $357;
   $358 = $7;
   $359 = ($358|0)!=(2);
   if ($359) {
    $360 = $ofhre;
    $fhre = $360;
   }
   $361 = $tap;
   $i = $361;
   while(1) {
    $362 = $i;
    $363 = $7;
    $364 = ($362|0)<($363|0);
    if (!($364)) {
     break;
    }
    $365 = $hpx;
    $366 = $hp0;
    $367 = ($365|0)==($366|0);
    if ($367) {
     $368 = $hle;
     $369 = ($368|0)!=(0);
     if ($369) {
      $hpxstr = 0;
      $hle = 0;
     } else {
      $370 = $pstr;
      $hpxstr = $370;
      $sfac = 1.0;
     }
    }
    $371 = $hpx;
    $372 = $hp1;
    $373 = ($371|0)==($372|0);
    if ($373) {
     $374 = $hre;
     $375 = ($374|0)!=(0);
     if ($375) {
      $hpxstr = 0;
      $hre = 0;
      $376 = $asym;
      $377 = ($376|0)!=(0);
      if ($377) {
       $378 = $da_ev;
       $379 = ($378|0)!=(0);
       if ($379) {
        $hre = 1;
        $380 = $fhre;
        $381 = (($380) + -1)|0;
        $fhre = $381;
        $382 = $fhre;
        $383 = (+($382|0));
        $sfac = $383;
        $384 = $sfac;
        $385 = $384;
        $386 = $385 == 0.0;
        if ($386) {
         $hre = 0;
        }
       }
      }
     } else {
      $387 = $nstr;
      $hpxstr = $387;
      $388 = $asym;
      $389 = ($388|0)!=(0);
      if ($389) {
       $sfac = -1.0;
      }
     }
    }
    $390 = $hpx;
    $391 = +HEAPF32[$390>>2];
    $392 = $i;
    $393 = $6;
    $394 = (($393) + ($392<<2)|0);
    $395 = +HEAPF32[$394>>2];
    $396 = $391 * $395;
    $397 = $sfac;
    $398 = $396 * $397;
    $399 = $himg;
    $400 = +HEAPF32[$399>>2];
    $401 = $400 + $398;
    HEAPF32[$399>>2] = $401;
    $402 = $hpxstr;
    $403 = $hpx;
    $404 = (($403) + ($402<<2)|0);
    $hpx = $404;
    $405 = $i;
    $406 = (($405) + 2)|0;
    $i = $406;
   }
   $407 = $5;
   $408 = $himg;
   $409 = (($408) + ($407<<2)|0);
   $himg = $409;
   $410 = $tap;
   $411 = (($410) + -1)|0;
   $tap = $411;
  }
  $412 = $cl_rw;
  $413 = (($412) + 1)|0;
  $cl_rw = $413;
 }
 $414 = $fi_ev;
 $415 = ($414|0)!=(0);
 if ($415) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $416 = $i;
  $417 = $7;
  $418 = ($416|0)<($417|0);
  if (!($418)) {
   break;
  }
  $419 = $i;
  $420 = $6;
  $421 = (($420) + ($419<<2)|0);
  $422 = +HEAPF32[$421>>2];
  $423 = $422;
  $424 = $423 * -1.0;
  $425 = $424;
  HEAPF32[$421>>2] = $425;
  $426 = $i;
  $427 = (($426) + 1)|0;
  $i = $427;
 }
 STACKTOP = sp;return;
}
function _int_sign($power) {
 $power = $power|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $num = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $power;
 $num = -1;
 $2 = $1;
 $3 = ($2|0)==(0);
 if ($3) {
  $0 = 1;
  $12 = $0;
  STACKTOP = sp;return ($12|0);
 }
 $cnt = 1;
 while(1) {
  $4 = $cnt;
  $5 = $1;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $num;
  $8 = Math_imul($7, -1)|0;
  $num = $8;
  $9 = $cnt;
  $10 = (($9) + 1)|0;
  $cnt = $10;
 }
 $11 = $num;
 $0 = $11;
 $12 = $0;
 STACKTOP = sp;return ($12|0);
}
function _init_wsq_decoder_resources() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 HEAP32[37088>>2] = 0;
 HEAP32[((37088 + 4|0))>>2] = 0;
 STACKTOP = sp;return;
}
function _free_wsq_decoder_resources() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[37088>>2]|0;
 $1 = ($0|0)!=(0|0);
 if ($1) {
  $2 = HEAP32[37088>>2]|0;
  _free($2);
  HEAP32[37088>>2] = 0;
 }
 $3 = HEAP32[((37088 + 4|0))>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[((37088 + 4|0))>>2]|0;
 _free($5);
 HEAP32[((37088 + 4|0))>>2] = 0;
 STACKTOP = sp;return;
}
function _jpegl_decode_mem($oimg_dat,$lossyflag,$idata,$ilen) {
 $oimg_dat = $oimg_dat|0;
 $lossyflag = $lossyflag|0;
 $idata = $idata|0;
 $ilen = $ilen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bit_count = 0, $cbufptr = 0, $cmpnt_i = 0, $data_pred = 0;
 var $diff_cat = 0, $diff_code = 0, $ebufptr = 0, $frm_header = 0, $full_diff_code = 0, $huf_table = 0, $huff_decoder = 0, $i = 0, $img_dat = 0, $jfif_header = 0, $marker = 0, $num_pixels = 0, $optr = 0, $pixel = 0, $ppi = 0, $ret = 0, $scn_header = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 20624|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $diff_code = sp + 20608|0;
 $diff_cat = sp + 20568|0;
 $bit_count = sp + 20552|0;
 $data_pred = sp + 20606|0;
 $huff_decoder = sp + 72|0;
 $ppi = sp + 64|0;
 $huf_table = sp + 48|0;
 $frm_header = sp + 44|0;
 $scn_header = sp + 40|0;
 $jfif_header = sp + 36|0;
 $img_dat = sp + 32|0;
 $marker = sp + 20604|0;
 $cbufptr = sp + 28|0;
 $1 = $oimg_dat;
 $2 = $lossyflag;
 $3 = $idata;
 $4 = $ilen;
 HEAP32[$bit_count>>2] = 0;
 _build_huff_decode_table($huff_decoder);
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = ($5|0)<(4);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = (($huf_table) + ($7<<2)|0);
  HEAP32[$8>>2] = 0;
  $9 = $i;
  $10 = (($9) + 1)|0;
  $i = $10;
 }
 $11 = $3;
 HEAP32[$cbufptr>>2] = $11;
 $12 = $3;
 $13 = $4;
 $14 = (($12) + ($13)|0);
 $ebufptr = $14;
 $15 = $ebufptr;
 $16 = (_getc_marker_jpegl($marker,65496,$cbufptr,$15)|0);
 $ret = $16;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = $ret;
  $0 = $18;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 $19 = $ebufptr;
 $20 = (_getc_marker_jpegl($marker,65504,$cbufptr,$19)|0);
 $ret = $20;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = $ret;
  $0 = $22;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 $23 = $ebufptr;
 $24 = (_getc_jfif_header($jfif_header,$cbufptr,$23)|0);
 $ret = $24;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = $ret;
  $0 = $26;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 $27 = HEAP32[$jfif_header>>2]|0;
 $28 = (_get_ppi_jpegl($ppi,$27)|0);
 $ret = $28;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$jfif_header>>2]|0;
  _free($30);
  $31 = $ret;
  $0 = $31;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 $32 = HEAP32[$jfif_header>>2]|0;
 _free($32);
 $33 = $ebufptr;
 $34 = (_getc_marker_jpegl($marker,2,$cbufptr,$33)|0);
 $ret = $34;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = $ret;
  $0 = $36;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 while(1) {
  $37 = HEAP16[$marker>>1]|0;
  $38 = $37&65535;
  $39 = ($38|0)!=(65475);
  if (!($39)) {
   label = 22;
   break;
  }
  $40 = HEAP16[$marker>>1]|0;
  $41 = $ebufptr;
  $42 = (_getc_table_jpegl($40,$huf_table,$cbufptr,$41)|0);
  $ret = $42;
  $43 = ($42|0)!=(0);
  if ($43) {
   label = 18;
   break;
  }
  $45 = $ebufptr;
  $46 = (_getc_marker_jpegl($marker,2,$cbufptr,$45)|0);
  $ret = $46;
  $47 = ($46|0)!=(0);
  if ($47) {
   label = 20;
   break;
  }
 }
 if ((label|0) == 18) {
  _free_HUFF_TABLES($huf_table,4);
  $44 = $ret;
  $0 = $44;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 else if ((label|0) == 20) {
  _free_HUFF_TABLES($huf_table,4);
  $48 = $ret;
  $0 = $48;
  $255 = $0;
  STACKTOP = sp;return ($255|0);
 }
 else if ((label|0) == 22) {
  $49 = $ebufptr;
  $50 = (_getc_frame_header_jpegl($frm_header,$cbufptr,$49)|0);
  $ret = $50;
  $51 = ($50|0)!=(0);
  if ($51) {
   _free_HUFF_TABLES($huf_table,4);
   $52 = $ret;
   $0 = $52;
   $255 = $0;
   STACKTOP = sp;return ($255|0);
  }
  $53 = HEAP32[$ppi>>2]|0;
  $54 = HEAP32[$frm_header>>2]|0;
  $55 = (_setup_IMG_DAT_decode($img_dat,$53,$54)|0);
  $ret = $55;
  $56 = ($55|0)!=(0);
  if ($56) {
   _free_HUFF_TABLES($huf_table,4);
   $57 = HEAP32[$frm_header>>2]|0;
   _free($57);
   $58 = $ret;
   $0 = $58;
   $255 = $0;
   STACKTOP = sp;return ($255|0);
  }
  $59 = HEAP32[$frm_header>>2]|0;
  _free($59);
  $60 = $ebufptr;
  $61 = (_getc_marker_jpegl($marker,3,$cbufptr,$60)|0);
  $ret = $61;
  $62 = ($61|0)!=(0);
  if ($62) {
   _free_HUFF_TABLES($huf_table,4);
   $63 = HEAP32[$img_dat>>2]|0;
   _free_IMG_DAT($63,0);
   $64 = $ret;
   $0 = $64;
   $255 = $0;
   STACKTOP = sp;return ($255|0);
  }
  L49: while(1) {
   $65 = HEAP16[$marker>>1]|0;
   $66 = $65&65535;
   $67 = ($66|0)!=(65497);
   if (!($67)) {
    label = 67;
    break;
   }
   while(1) {
    $68 = HEAP16[$marker>>1]|0;
    $69 = $68&65535;
    $70 = ($69|0)!=(65498);
    if (!($70)) {
     break;
    }
    $71 = HEAP16[$marker>>1]|0;
    $72 = $ebufptr;
    $73 = (_getc_table_jpegl($71,$huf_table,$cbufptr,$72)|0);
    $ret = $73;
    $74 = ($73|0)!=(0);
    if ($74) {
     label = 33;
     break L49;
    }
    $77 = $ebufptr;
    $78 = (_getc_marker_jpegl($marker,3,$cbufptr,$77)|0);
    $ret = $78;
    $79 = ($78|0)!=(0);
    if ($79) {
     label = 35;
     break L49;
    }
   }
   $82 = $ebufptr;
   $83 = (_getc_scan_header($scn_header,$cbufptr,$82)|0);
   $ret = $83;
   $84 = ($83|0)!=(0);
   if ($84) {
    label = 38;
    break;
   }
   $87 = HEAP32[$img_dat>>2]|0;
   $88 = HEAP32[$scn_header>>2]|0;
   $89 = (_update_IMG_DAT_decode($87,$88,$huf_table)|0);
   $ret = $89;
   $90 = ($89|0)!=(0);
   if ($90) {
    label = 40;
    break;
   }
   $94 = HEAP32[$img_dat>>2]|0;
   $95 = (($94) + 16|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = ($96|0)!=(0);
   if ($97) {
    label = 53;
    break;
   }
   $98 = HEAP32[$scn_header>>2]|0;
   $99 = (($98) + 1|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $cmpnt_i = $101;
   $102 = $cmpnt_i;
   $103 = HEAP32[$img_dat>>2]|0;
   $104 = (($103) + 100|0);
   $105 = (($104) + ($102<<2)|0);
   $106 = HEAP32[$105>>2]|0;
   $optr = $106;
   $107 = $cmpnt_i;
   $108 = HEAP32[$img_dat>>2]|0;
   $109 = (($108) + 60|0);
   $110 = (($109) + ($107<<2)|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $cmpnt_i;
   $113 = HEAP32[$img_dat>>2]|0;
   $114 = (($113) + 76|0);
   $115 = (($114) + ($112<<2)|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = Math_imul($111, $116)|0;
   $num_pixels = $117;
   HEAP32[$bit_count>>2] = 0;
   $pixel = 0;
   while(1) {
    $118 = $pixel;
    $119 = $num_pixels;
    $120 = ($118|0)<($119|0);
    if (!($120)) {
     break;
    }
    $121 = $cmpnt_i;
    $122 = (($huf_table) + ($121<<2)|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = (($123) + 28|0);
    $125 = HEAP32[$124>>2]|0;
    $126 = $cmpnt_i;
    $127 = (($huf_table) + ($126<<2)|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = (($128) + 24|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = $cmpnt_i;
    $132 = (($huf_table) + ($131<<2)|0);
    $133 = HEAP32[$132>>2]|0;
    $134 = (($133) + 32|0);
    $135 = HEAP32[$134>>2]|0;
    $136 = $cmpnt_i;
    $137 = (($huf_table) + ($136<<2)|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = (($138) + 8|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = $ebufptr;
    $142 = (_decode_data($diff_cat,$125,$130,$135,$140,$cbufptr,$141,$bit_count)|0);
    $ret = $142;
    $143 = ($142|0)!=(0);
    if ($143) {
     label = 45;
     break L49;
    }
    $147 = $ebufptr;
    $148 = HEAP32[$diff_cat>>2]|0;
    $149 = (_getc_nextbits_jpegl($diff_code,$cbufptr,$147,$bit_count,$148)|0);
    $ret = $149;
    $150 = ($149|0)!=(0);
    if ($150) {
     label = 47;
     break L49;
    }
    $154 = HEAP16[$diff_code>>1]|0;
    $155 = $154&65535;
    $156 = HEAP32[$diff_cat>>2]|0;
    $157 = (($huff_decoder) + ($156<<11)|0);
    $158 = (($157) + ($155<<2)|0);
    $159 = HEAP32[$158>>2]|0;
    $full_diff_code = $159;
    $160 = $optr;
    $161 = $cmpnt_i;
    $162 = HEAP32[$img_dat>>2]|0;
    $163 = (($162) + 60|0);
    $164 = (($163) + ($161<<2)|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = $pixel;
    $167 = HEAP32[$img_dat>>2]|0;
    $168 = (($167) + 24|0);
    $169 = HEAP32[$168>>2]|0;
    $170 = $cmpnt_i;
    $171 = HEAP32[$img_dat>>2]|0;
    $172 = (($171) + 96|0);
    $173 = (($172) + ($170)|0);
    $174 = HEAP8[$173>>0]|0;
    $175 = $174&255;
    $176 = $cmpnt_i;
    $177 = HEAP32[$img_dat>>2]|0;
    $178 = (($177) + 92|0);
    $179 = (($178) + ($176)|0);
    $180 = HEAP8[$179>>0]|0;
    $181 = $180&255;
    $182 = (_predict($data_pred,$160,$165,$166,$169,$175,$181)|0);
    $ret = $182;
    $183 = ($182|0)!=(0);
    if ($183) {
     label = 49;
     break L49;
    }
    $187 = $full_diff_code;
    $188 = HEAP16[$data_pred>>1]|0;
    $189 = $188 << 16 >> 16;
    $190 = (($187) + ($189))|0;
    $191 = $190&255;
    $192 = $optr;
    $193 = (($192) + 1|0);
    $optr = $193;
    HEAP8[$192>>0] = $191;
    $194 = $pixel;
    $195 = (($194) + 1)|0;
    $pixel = $195;
   }
   $201 = HEAP32[$scn_header>>2]|0;
   _free($201);
   $i = 0;
   while(1) {
    $202 = $i;
    $203 = HEAP32[$img_dat>>2]|0;
    $204 = (($203) + 20|0);
    $205 = HEAP32[$204>>2]|0;
    $206 = ($202|0)<($205|0);
    if (!($206)) {
     break;
    }
    $207 = $i;
    $208 = HEAP32[$img_dat>>2]|0;
    $209 = (($208) + 92|0);
    $210 = (($209) + ($207)|0);
    $211 = HEAP8[$210>>0]|0;
    $212 = ($211<<24>>24)!=(0);
    if ($212) {
     $pixel = 0;
     while(1) {
      $213 = $pixel;
      $214 = $i;
      $215 = HEAP32[$img_dat>>2]|0;
      $216 = (($215) + 60|0);
      $217 = (($216) + ($214<<2)|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $i;
      $220 = HEAP32[$img_dat>>2]|0;
      $221 = (($220) + 76|0);
      $222 = (($221) + ($219<<2)|0);
      $223 = HEAP32[$222>>2]|0;
      $224 = Math_imul($218, $223)|0;
      $225 = ($213|0)<($224|0);
      if (!($225)) {
       break;
      }
      $226 = $i;
      $227 = HEAP32[$img_dat>>2]|0;
      $228 = (($227) + 92|0);
      $229 = (($228) + ($226)|0);
      $230 = HEAP8[$229>>0]|0;
      $231 = $230&255;
      $232 = $i;
      $233 = HEAP32[$img_dat>>2]|0;
      $234 = (($233) + 100|0);
      $235 = (($234) + ($232<<2)|0);
      $236 = HEAP32[$235>>2]|0;
      $237 = $pixel;
      $238 = (($236) + ($237)|0);
      $239 = HEAP8[$238>>0]|0;
      $240 = $239&255;
      $241 = $240 << $231;
      $242 = $241&255;
      HEAP8[$238>>0] = $242;
      $243 = $pixel;
      $244 = (($243) + 1)|0;
      $pixel = $244;
     }
    }
    $245 = $i;
    $246 = (($245) + 1)|0;
    $i = $246;
   }
   $247 = $ebufptr;
   $248 = (_getc_ushort($marker,$cbufptr,$247)|0);
   $ret = $248;
   $249 = ($248|0)!=(0);
   if ($249) {
    label = 65;
    break;
   }
  }
  switch (label|0) {
   case 33: {
    _free_HUFF_TABLES($huf_table,4);
    $75 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($75,1);
    $76 = $ret;
    $0 = $76;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 35: {
    _free_HUFF_TABLES($huf_table,4);
    $80 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($80,1);
    $81 = $ret;
    $0 = $81;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 38: {
    _free_HUFF_TABLES($huf_table,4);
    $85 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($85,1);
    $86 = $ret;
    $0 = $86;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 40: {
    _free_HUFF_TABLES($huf_table,4);
    $91 = HEAP32[$scn_header>>2]|0;
    _free($91);
    $92 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($92,1);
    $93 = $ret;
    $0 = $93;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 45: {
    _free_HUFF_TABLES($huf_table,4);
    $144 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($144,1);
    $145 = HEAP32[$scn_header>>2]|0;
    _free($145);
    $146 = $ret;
    $0 = $146;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 47: {
    _free_HUFF_TABLES($huf_table,4);
    $151 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($151,1);
    $152 = HEAP32[$scn_header>>2]|0;
    _free($152);
    $153 = $ret;
    $0 = $153;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 49: {
    _free_HUFF_TABLES($huf_table,4);
    $184 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($184,1);
    $185 = HEAP32[$scn_header>>2]|0;
    _free($185);
    $186 = $ret;
    $0 = $186;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 53: {
    $196 = HEAP32[_stderr>>2]|0;
    (_fprintf(($196|0),(41744|0),($vararg_buffer|0))|0);
    $197 = HEAP32[_stderr>>2]|0;
    (_fprintf(($197|0),(41776|0),($vararg_buffer1|0))|0);
    $198 = HEAP32[_stderr>>2]|0;
    (_fprintf(($198|0),(41816|0),($vararg_buffer3|0))|0);
    _free_HUFF_TABLES($huf_table,4);
    $199 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($199,1);
    $200 = HEAP32[$scn_header>>2]|0;
    _free($200);
    $0 = -2;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 65: {
    _free_HUFF_TABLES($huf_table,4);
    $250 = HEAP32[$img_dat>>2]|0;
    _free_IMG_DAT($250,1);
    $251 = $ret;
    $0 = $251;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
   case 67: {
    _free_HUFF_TABLES($huf_table,4);
    $252 = HEAP32[$img_dat>>2]|0;
    $253 = $1;
    HEAP32[$253>>2] = $252;
    $254 = $2;
    HEAP32[$254>>2] = 0;
    $0 = 0;
    $255 = $0;
    STACKTOP = sp;return ($255|0);
    break;
   }
  }
 }
 return 0|0;
}
function _build_huff_decode_table($huff_decoder) {
 $huff_decoder = $huff_decoder|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bit = 0;
 var $cat = 0, $count = 0, $diff = 0, $difftemp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $huff_decoder;
 $count = -511;
 while(1) {
  $1 = $count;
  $2 = ($1|0)<=(511);
  if (!($2)) {
   break;
  }
  $3 = $count;
  $4 = $3&65535;
  $diff = $4;
  $5 = $diff;
  $6 = (_categorize($5)|0);
  $7 = $6 << 16 >> 16;
  $cat = $7;
  $8 = $diff;
  $9 = $8 << 16 >> 16;
  $10 = ($9|0)<(0);
  if ($10) {
   $11 = $diff;
   $12 = (($11) + -1)<<16>>16;
   $diff = $12;
   $difftemp = 0;
   $bit = 0;
   while(1) {
    $13 = $bit;
    $14 = $cat;
    $15 = ($13|0)<($14|0);
    if (!($15)) {
     break;
    }
    $16 = $diff;
    $17 = $16 << 16 >> 16;
    $18 = $bit;
    $19 = $17 >> $18;
    $20 = $19 & 1;
    $21 = ($20|0)!=(0);
    if ($21) {
     $22 = $bit;
     $23 = 1 << $22;
     $24 = $difftemp;
     $25 = $24 | $23;
     $difftemp = $25;
    }
    $26 = $bit;
    $27 = (($26) + 1)|0;
    $bit = $27;
   }
   $28 = $difftemp;
   $29 = $28&65535;
   $diff = $29;
  }
  $30 = $count;
  $31 = $diff;
  $32 = $31 << 16 >> 16;
  $33 = $cat;
  $34 = $0;
  $35 = (($34) + ($33<<11)|0);
  $36 = (($35) + ($32<<2)|0);
  HEAP32[$36>>2] = $30;
  $37 = $count;
  $38 = (($37) + 1)|0;
  $count = $38;
 }
 STACKTOP = sp;return;
}
function _decode_data($odiff_cat,$mincode,$maxcode,$valptr,$huffvalues,$cbufptr,$ebufptr,$bit_count) {
 $odiff_cat = $odiff_cat|0;
 $mincode = $mincode|0;
 $maxcode = $maxcode|0;
 $valptr = $valptr|0;
 $huffvalues = $huffvalues|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 $bit_count = $bit_count|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $code = 0, $diff_cat = 0, $inx = 0, $inx2 = 0;
 var $ret = 0, $tcode = 0, $tcode2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $tcode = sp + 58|0;
 $tcode2 = sp + 56|0;
 $1 = $odiff_cat;
 $2 = $mincode;
 $3 = $maxcode;
 $4 = $valptr;
 $5 = $huffvalues;
 $6 = $cbufptr;
 $7 = $ebufptr;
 $8 = $bit_count;
 $9 = $6;
 $10 = $7;
 $11 = $8;
 $12 = (_getc_nextbits_jpegl($tcode,$9,$10,$11,1)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $ret;
  $0 = $14;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $15 = HEAP16[$tcode>>1]|0;
 $16 = $15&65535;
 $code = $16;
 $inx = 1;
 while(1) {
  $17 = $code;
  $18 = $inx;
  $19 = $3;
  $20 = (($19) + ($18<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($17|0)>($21|0);
  if (!($22)) {
   label = 9;
   break;
  }
  $23 = $6;
  $24 = $7;
  $25 = $8;
  $26 = (_getc_nextbits_jpegl($tcode2,$23,$24,$25,1)|0);
  $ret = $26;
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 6;
   break;
  }
  $29 = $code;
  $30 = $29 << 1;
  $31 = HEAP16[$tcode2>>1]|0;
  $32 = $31&65535;
  $33 = (($30) + ($32))|0;
  $code = $33;
  $34 = $inx;
  $35 = (($34) + 1)|0;
  $inx = $35;
 }
 if ((label|0) == 6) {
  $28 = $ret;
  $0 = $28;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 else if ((label|0) == 9) {
  $36 = $inx;
  $37 = $4;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inx2 = $39;
  $40 = $inx2;
  $41 = $code;
  $42 = (($40) + ($41))|0;
  $43 = $inx;
  $44 = $2;
  $45 = (($44) + ($43<<2)|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($42) - ($46))|0;
  $inx2 = $47;
  $48 = $inx2;
  $49 = $5;
  $50 = (($49) + ($48)|0);
  $51 = HEAP8[$50>>0]|0;
  $52 = $51&255;
  $diff_cat = $52;
  $53 = $diff_cat;
  $54 = $1;
  HEAP32[$54>>2] = $53;
  $0 = 0;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 return 0|0;
}
function _getc_nextbits_jpegl($obits,$cbufptr,$ebufptr,$bit_count,$bits_req) {
 $obits = $obits|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 $bit_count = $bit_count|0;
 $bits_req = $bits_req|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $bits = 0, $bits_needed = 0, $code2 = 0, $ret = 0, $tbits = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $code2 = sp + 40|0;
 $tbits = sp + 36|0;
 $1 = $obits;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = $bit_count;
 $5 = $bits_req;
 $6 = $5;
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = $1;
  HEAP16[$8>>1] = 0;
  $0 = 0;
  $85 = $0;
  STACKTOP = sp;return ($85|0);
 }
 $9 = $4;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = $2;
  $13 = $3;
  $14 = (_getc_byte(41856,$12,$13)|0);
  $ret = $14;
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = $ret;
   $0 = $16;
   $85 = $0;
   STACKTOP = sp;return ($85|0);
  }
  $17 = $4;
  HEAP32[$17>>2] = 8;
  $18 = HEAP8[41856>>0]|0;
  $19 = $18&255;
  $20 = ($19|0)==(255);
  do {
   if ($20) {
    $21 = $2;
    $22 = $3;
    $23 = (_getc_byte($code2,$21,$22)|0);
    $ret = $23;
    $24 = ($23|0)!=(0);
    if ($24) {
     $25 = $ret;
     $0 = $25;
     $85 = $0;
     STACKTOP = sp;return ($85|0);
    }
    $26 = HEAP8[$code2>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)!=(0);
    if (!($28)) {
     break;
    }
    $29 = HEAP32[_stderr>>2]|0;
    (_fprintf(($29|0),(41880|0),($vararg_buffer|0))|0);
    $0 = -2;
    $85 = $0;
    STACKTOP = sp;return ($85|0);
   }
  } while(0);
 }
 $30 = $5;
 $31 = $4;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($30|0)<=($32|0);
 do {
  if ($33) {
   $34 = HEAP8[41856>>0]|0;
   $35 = $34&255;
   $36 = $4;
   $37 = HEAP32[$36>>2]|0;
   $38 = $5;
   $39 = (($37) - ($38))|0;
   $40 = $35 >> $39;
   $41 = $5;
   $42 = (41864 + ($41)|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $40 & $44;
   $46 = $45&65535;
   $bits = $46;
   $47 = $5;
   $48 = $4;
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) - ($47))|0;
   HEAP32[$48>>2] = $50;
   $51 = $4;
   $52 = HEAP32[$51>>2]|0;
   $53 = (41864 + ($52)|0);
   $54 = HEAP8[$53>>0]|0;
   $55 = $54&255;
   $56 = HEAP8[41856>>0]|0;
   $57 = $56&255;
   $58 = $57 & $55;
   $59 = $58&255;
   HEAP8[41856>>0] = $59;
  } else {
   $60 = $5;
   $61 = $4;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($60) - ($62))|0;
   $bits_needed = $63;
   $64 = HEAP8[41856>>0]|0;
   $65 = $64&255;
   $66 = $bits_needed;
   $67 = $65 << $66;
   $68 = $67&65535;
   $bits = $68;
   $69 = $4;
   HEAP32[$69>>2] = 0;
   $70 = $2;
   $71 = $3;
   $72 = $4;
   $73 = $bits_needed;
   $74 = (_getc_nextbits_jpegl($tbits,$70,$71,$72,$73)|0);
   $ret = $74;
   $75 = ($74|0)!=(0);
   if (!($75)) {
    $77 = HEAP16[$tbits>>1]|0;
    $78 = $77&65535;
    $79 = $bits;
    $80 = $79&65535;
    $81 = $80 | $78;
    $82 = $81&65535;
    $bits = $82;
    break;
   }
   $76 = $ret;
   $0 = $76;
   $85 = $0;
   STACKTOP = sp;return ($85|0);
  }
 } while(0);
 $83 = $bits;
 $84 = $1;
 HEAP16[$84>>1] = $83;
 $0 = 0;
 $85 = $0;
 STACKTOP = sp;return ($85|0);
}
function _getc_huffman_table($otable_id,$ohuffbits,$ohuffvalues,$max_huffcounts,$cbufptr,$ebufptr,$read_table_len,$bytes_left) {
 $otable_id = $otable_id|0;
 $ohuffbits = $ohuffbits|0;
 $ohuffvalues = $ohuffvalues|0;
 $max_huffcounts = $max_huffcounts|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 $read_table_len = $read_table_len|0;
 $bytes_left = $bytes_left|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $huffbits = 0, $huffvalues = 0, $i = 0, $num_hufvals = 0, $ret = 0, $table_id = 0, $table_len = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0;
 var $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer26 = 0, $vararg_buffer3 = 0, $vararg_buffer30 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer30 = sp + 96|0;
 $vararg_buffer26 = sp + 88|0;
 $vararg_buffer22 = sp + 80|0;
 $vararg_buffer19 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer11 = sp + 48|0;
 $vararg_buffer9 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $table_len = sp + 154|0;
 $table_id = sp + 156|0;
 $1 = $otable_id;
 $2 = $ohuffbits;
 $3 = $ohuffvalues;
 $4 = $max_huffcounts;
 $5 = $cbufptr;
 $6 = $ebufptr;
 $7 = $read_table_len;
 $8 = $bytes_left;
 $9 = HEAP32[8>>2]|0;
 $10 = ($9|0)>(0);
 if ($10) {
  $11 = HEAP32[_stdout>>2]|0;
  (_fprintf(($11|0),(41928|0),($vararg_buffer|0))|0);
 }
 $12 = $7;
 $13 = ($12|0)!=(0);
 do {
  if ($13) {
   $14 = $5;
   $15 = $6;
   $16 = (_getc_ushort($table_len,$14,$15)|0);
   $ret = $16;
   $17 = ($16|0)!=(0);
   if (!($17)) {
    $19 = HEAP16[$table_len>>1]|0;
    $20 = $19&65535;
    $21 = (($20) - 2)|0;
    $22 = $8;
    HEAP32[$22>>2] = $21;
    break;
   }
   $18 = $ret;
   $0 = $18;
   $147 = $0;
   STACKTOP = sp;return ($147|0);
  }
 } while(0);
 $23 = $8;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)<=(0);
 if ($25) {
  $26 = HEAP32[_stderr>>2]|0;
  (_fprintf(($26|0),(42176|0),($vararg_buffer1|0))|0);
  $27 = HEAP32[_stderr>>2]|0;
  (_fprintf(($27|0),(41960|0),($vararg_buffer3|0))|0);
  $0 = -2;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $28 = $5;
 $29 = $6;
 $30 = (_getc_byte($table_id,$28,$29)|0);
 $ret = $30;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = $ret;
  $0 = $32;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $33 = $8;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + -1)|0;
 HEAP32[$33>>2] = $35;
 $36 = (_calloc(16,1)|0);
 $huffbits = $36;
 $37 = $huffbits;
 $38 = ($37|0)==(0|0);
 if ($38) {
  $39 = HEAP32[_stderr>>2]|0;
  (_fprintf(($39|0),(42208|0),($vararg_buffer5|0))|0);
  $0 = -3;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $num_hufvals = 0;
 $i = 0;
 while(1) {
  $40 = $i;
  $41 = ($40|0)<(16);
  if (!($41)) {
   break;
  }
  $42 = $i;
  $43 = $huffbits;
  $44 = (($43) + ($42)|0);
  $45 = $5;
  $46 = $6;
  $47 = (_getc_byte($44,$45,$46)|0);
  $ret = $47;
  $48 = ($47|0)!=(0);
  if ($48) {
   label = 16;
   break;
  }
  $51 = $i;
  $52 = $huffbits;
  $53 = (($52) + ($51)|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = $num_hufvals;
  $57 = $56&65535;
  $58 = (($57) + ($55))|0;
  $59 = $58&65535;
  $num_hufvals = $59;
  $60 = $i;
  $61 = (($60) + 1)|0;
  $i = $61;
 }
 if ((label|0) == 16) {
  $49 = $huffbits;
  _free($49);
  $50 = $ret;
  $0 = $50;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $62 = $8;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($63) - 16)|0;
 HEAP32[$62>>2] = $64;
 $65 = $num_hufvals;
 $66 = $65&65535;
 $67 = $4;
 $68 = (($67) + 1)|0;
 $69 = ($66|0)>($68|0);
 if ($69) {
  $70 = HEAP32[_stderr>>2]|0;
  (_fprintf(($70|0),(42176|0),($vararg_buffer7|0))|0);
  $71 = HEAP32[_stderr>>2]|0;
  $72 = $num_hufvals;
  $73 = $72&65535;
  HEAP32[$vararg_buffer9>>2] = $73;
  (_fprintf(($71|0),(42000|0),($vararg_buffer9|0))|0);
  $74 = HEAP32[_stderr>>2]|0;
  $75 = $4;
  $76 = (($75) + 1)|0;
  HEAP32[$vararg_buffer11>>2] = $76;
  (_fprintf(($74|0),(42032|0),($vararg_buffer11|0))|0);
  $77 = $huffbits;
  _free($77);
  $0 = -4;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $78 = $4;
 $79 = (($78) + 1)|0;
 $80 = (_calloc($79,1)|0);
 $huffvalues = $80;
 $81 = $huffvalues;
 $82 = ($81|0)==(0|0);
 if ($82) {
  $83 = HEAP32[_stderr>>2]|0;
  (_fprintf(($83|0),(42256|0),($vararg_buffer14|0))|0);
  $84 = $huffbits;
  _free($84);
  $0 = -5;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $i = 0;
 while(1) {
  $85 = $i;
  $86 = $num_hufvals;
  $87 = $86&65535;
  $88 = ($85|0)<($87|0);
  if (!($88)) {
   break;
  }
  $89 = $i;
  $90 = $huffvalues;
  $91 = (($90) + ($89)|0);
  $92 = $5;
  $93 = $6;
  $94 = (_getc_byte($91,$92,$93)|0);
  $ret = $94;
  $95 = ($94|0)!=(0);
  if ($95) {
   label = 26;
   break;
  }
  $99 = $i;
  $100 = (($99) + 1)|0;
  $i = $100;
 }
 if ((label|0) == 26) {
  $96 = $huffbits;
  _free($96);
  $97 = $huffvalues;
  _free($97);
  $98 = $ret;
  $0 = $98;
  $147 = $0;
  STACKTOP = sp;return ($147|0);
 }
 $101 = $num_hufvals;
 $102 = $101&65535;
 $103 = $8;
 $104 = HEAP32[$103>>2]|0;
 $105 = (($104) - ($102))|0;
 HEAP32[$103>>2] = $105;
 $106 = HEAP32[8>>2]|0;
 $107 = ($106|0)>(1);
 if ($107) {
  $108 = HEAP32[_stdout>>2]|0;
  $109 = HEAP16[$table_len>>1]|0;
  $110 = $109&65535;
  HEAP32[$vararg_buffer16>>2] = $110;
  (_fprintf(($108|0),(42064|0),($vararg_buffer16|0))|0);
  $111 = HEAP32[_stdout>>2]|0;
  $112 = HEAP8[$table_id>>0]|0;
  $113 = $112&255;
  HEAP32[$vararg_buffer19>>2] = $113;
  (_fprintf(($111|0),(42080|0),($vararg_buffer19|0))|0);
  $i = 0;
  while(1) {
   $114 = $i;
   $115 = ($114|0)<(16);
   if (!($115)) {
    break;
   }
   $116 = HEAP32[_stdout>>2]|0;
   $117 = $i;
   $118 = $i;
   $119 = $huffbits;
   $120 = (($119) + ($118)|0);
   $121 = HEAP8[$120>>0]|0;
   $122 = $121&255;
   HEAP32[$vararg_buffer22>>2] = $117;
   $vararg_ptr25 = (($vararg_buffer22) + 4|0);
   HEAP32[$vararg_ptr25>>2] = $122;
   (_fprintf(($116|0),(42096|0),($vararg_buffer22|0))|0);
   $123 = $i;
   $124 = (($123) + 1)|0;
   $i = $124;
  }
  $i = 0;
  while(1) {
   $125 = $i;
   $126 = $num_hufvals;
   $127 = $126&65535;
   $128 = ($125|0)<($127|0);
   if (!($128)) {
    break;
   }
   $129 = HEAP32[_stdout>>2]|0;
   $130 = $i;
   $131 = $i;
   $132 = $huffvalues;
   $133 = (($132) + ($131)|0);
   $134 = HEAP8[$133>>0]|0;
   $135 = $134&255;
   HEAP32[$vararg_buffer26>>2] = $130;
   $vararg_ptr29 = (($vararg_buffer26) + 4|0);
   HEAP32[$vararg_ptr29>>2] = $135;
   (_fprintf(($129|0),(42112|0),($vararg_buffer26|0))|0);
   $136 = $i;
   $137 = (($136) + 1)|0;
   $i = $137;
  }
 }
 $138 = HEAP32[8>>2]|0;
 $139 = ($138|0)>(0);
 if ($139) {
  $140 = HEAP32[_stdout>>2]|0;
  (_fprintf(($140|0),(42136|0),($vararg_buffer30|0))|0);
 }
 $141 = HEAP8[$table_id>>0]|0;
 $142 = $1;
 HEAP8[$142>>0] = $141;
 $143 = $huffbits;
 $144 = $2;
 HEAP32[$144>>2] = $143;
 $145 = $huffvalues;
 $146 = $3;
 HEAP32[$146>>2] = $145;
 $0 = 0;
 $147 = $0;
 STACKTOP = sp;return ($147|0);
}
function _build_huffsizes($ohuffcode_table,$temp_size,$huffbits,$max_huffcounts) {
 $ohuffcode_table = $ohuffcode_table|0;
 $temp_size = $temp_size|0;
 $huffbits = $huffbits|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $code_size = 0;
 var $huffcode_table = 0, $ii = 0, $number_of_codes = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ohuffcode_table;
 $2 = $temp_size;
 $3 = $huffbits;
 $4 = $max_huffcounts;
 $number_of_codes = 1;
 $5 = $4;
 $6 = (($5) + 1)|0;
 $7 = (_calloc($6,8)|0);
 $huffcode_table = $7;
 $8 = $huffcode_table;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(42312|0),($vararg_buffer|0))|0);
  $0 = -2;
  $58 = $0;
  STACKTOP = sp;return ($58|0);
 }
 $11 = $2;
 HEAP32[$11>>2] = 0;
 $code_size = 1;
 while(1) {
  $12 = $code_size;
  $13 = ($12|0)<=(16);
  if (!($13)) {
   break;
  }
  while(1) {
   $14 = $number_of_codes;
   $15 = $code_size;
   $16 = (($15) - 1)|0;
   $17 = $3;
   $18 = (($17) + ($16)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = ($14|0)<=($20|0);
   if (!($21)) {
    break;
   }
   $22 = $code_size;
   $23 = $22&65535;
   $24 = $huffcode_table;
   $25 = $2;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + ($26<<3)|0);
   HEAP16[$27>>1] = $23;
   $28 = $2;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 1)|0;
   HEAP32[$28>>2] = $30;
   $31 = $number_of_codes;
   $32 = (($31) + 1)|0;
   $number_of_codes = $32;
  }
  $number_of_codes = 1;
  $33 = $code_size;
  $34 = (($33) + 1)|0;
  $code_size = $34;
 }
 $35 = $huffcode_table;
 $36 = $2;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + ($37<<3)|0);
 HEAP16[$38>>1] = 0;
 $39 = HEAP32[8>>2]|0;
 $40 = ($39|0)>(2);
 if ($40) {
  $41 = HEAP32[_stderr>>2]|0;
  (_fprintf(($41|0),(42368|0),($vararg_buffer1|0))|0);
  $ii = 0;
  while(1) {
   $42 = $ii;
   $43 = $4;
   $44 = (($43) + 1)|0;
   $45 = ($42|0)<($44|0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[_stderr>>2]|0;
   $47 = $ii;
   $48 = $ii;
   $49 = $huffcode_table;
   $50 = (($49) + ($48<<3)|0);
   $51 = HEAP16[$50>>1]|0;
   $52 = $51 << 16 >> 16;
   HEAP32[$vararg_buffer3>>2] = $47;
   $vararg_ptr5 = (($vararg_buffer3) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $52;
   (_fprintf(($46|0),(42392|0),($vararg_buffer3|0))|0);
   $53 = $ii;
   $54 = (($53) + 1)|0;
   $ii = $54;
  }
  $55 = HEAP32[_stderr>>2]|0;
  (_fflush(($55|0))|0);
 }
 $56 = $huffcode_table;
 $57 = $1;
 HEAP32[$57>>2] = $56;
 $0 = 0;
 $58 = $0;
 STACKTOP = sp;return ($58|0);
}
function _build_huffcodes($huffcode_table) {
 $huffcode_table = $huffcode_table|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pointer = 0, $temp_code = 0, $temp_size = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $huffcode_table;
 $pointer = 0;
 $temp_code = 0;
 $1 = $0;
 $2 = HEAP16[$1>>1]|0;
 $temp_size = $2;
 $3 = $0;
 $4 = $pointer;
 $5 = (($3) + ($4<<3)|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6 << 16 >> 16;
 $8 = ($7|0)==(0);
 if ($8) {
  STACKTOP = sp;return;
 }
 while(1) {
  while(1) {
   $9 = $temp_code;
   $10 = $9&65535;
   $11 = $0;
   $12 = $pointer;
   $13 = (($11) + ($12<<3)|0);
   $14 = (($13) + 4|0);
   HEAP32[$14>>2] = $10;
   $15 = $temp_code;
   $16 = (($15) + 1)<<16>>16;
   $temp_code = $16;
   $17 = $pointer;
   $18 = (($17) + 1)|0;
   $pointer = $18;
   $19 = $0;
   $20 = $pointer;
   $21 = (($19) + ($20<<3)|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = $temp_size;
   $25 = $24 << 16 >> 16;
   $26 = ($23|0)==($25|0);
   if (!($26)) {
    break;
   }
  }
  $27 = $0;
  $28 = $pointer;
  $29 = (($27) + ($28<<3)|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30 << 16 >> 16;
  $32 = ($31|0)==(0);
  if ($32) {
   break;
  }
  while(1) {
   $33 = $temp_code;
   $34 = $33&65535;
   $35 = $34 << 1;
   $36 = $35&65535;
   $temp_code = $36;
   $37 = $temp_size;
   $38 = (($37) + 1)<<16>>16;
   $temp_size = $38;
   $39 = $0;
   $40 = $pointer;
   $41 = (($39) + ($40<<3)|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = $42 << 16 >> 16;
   $44 = $temp_size;
   $45 = $44 << 16 >> 16;
   $46 = ($43|0)!=($45|0);
   if (!($46)) {
    break;
   }
  }
  $47 = $0;
  $48 = $pointer;
  $49 = (($47) + ($48<<3)|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = $50 << 16 >> 16;
  $52 = $temp_size;
  $53 = $52 << 16 >> 16;
  $54 = ($51|0)==($53|0);
  if (!($54)) {
   label = 14;
   break;
  }
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _gen_decode_table($huffcode_table,$maxcode,$mincode,$valptr,$huffbits) {
 $huffcode_table = $huffcode_table|0;
 $maxcode = $maxcode|0;
 $mincode = $mincode|0;
 $valptr = $valptr|0;
 $huffbits = $huffbits|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $i2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $huffcode_table;
 $1 = $maxcode;
 $2 = $mincode;
 $3 = $valptr;
 $4 = $huffbits;
 $i2 = 0;
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = ($5|0)<=(16);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $1;
  $9 = (($8) + ($7<<2)|0);
  HEAP32[$9>>2] = 0;
  $10 = $i;
  $11 = $2;
  $12 = (($11) + ($10<<2)|0);
  HEAP32[$12>>2] = 0;
  $13 = $i;
  $14 = $3;
  $15 = (($14) + ($13<<2)|0);
  HEAP32[$15>>2] = 0;
  $16 = $i;
  $17 = (($16) + 1)|0;
  $i = $17;
 }
 $i = 1;
 while(1) {
  $18 = $i;
  $19 = ($18|0)<=(16);
  if (!($19)) {
   break;
  }
  $20 = $i;
  $21 = (($20) - 1)|0;
  $22 = $4;
  $23 = (($22) + ($21)|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)==(0);
  if ($26) {
   $27 = $i;
   $28 = $1;
   $29 = (($28) + ($27<<2)|0);
   HEAP32[$29>>2] = -1;
  } else {
   $30 = $i2;
   $31 = $i;
   $32 = $3;
   $33 = (($32) + ($31<<2)|0);
   HEAP32[$33>>2] = $30;
   $34 = $0;
   $35 = $i2;
   $36 = (($34) + ($35<<3)|0);
   $37 = (($36) + 4|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $i;
   $40 = $2;
   $41 = (($40) + ($39<<2)|0);
   HEAP32[$41>>2] = $38;
   $42 = $i2;
   $43 = $i;
   $44 = (($43) - 1)|0;
   $45 = $4;
   $46 = (($45) + ($44)|0);
   $47 = HEAP8[$46>>0]|0;
   $48 = $47&255;
   $49 = (($42) + ($48))|0;
   $50 = (($49) - 1)|0;
   $i2 = $50;
   $51 = $0;
   $52 = $i2;
   $53 = (($51) + ($52<<3)|0);
   $54 = (($53) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = $i;
   $57 = $1;
   $58 = (($57) + ($56<<2)|0);
   HEAP32[$58>>2] = $55;
   $59 = $i2;
   $60 = (($59) + 1)|0;
   $i2 = $60;
  }
  $61 = $i;
  $62 = (($61) + 1)|0;
  $i = $62;
 }
 STACKTOP = sp;return;
}
function _free_HUFF_TABLE($huf_table) {
 $huf_table = $huf_table|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $huf_table;
 $1 = $0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  _free($7);
 }
 $8 = $0;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = $0;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  _free($14);
 }
 $15 = $0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = $0;
  $20 = (($19) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  _free($21);
 }
 $22 = $0;
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $26 = $0;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  _free($28);
 }
 $29 = $0;
 $30 = (($29) + 36|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = $0;
  $34 = (($33) + 36|0);
  $35 = HEAP32[$34>>2]|0;
  _free($35);
 }
 $36 = $0;
 $37 = (($36) + 24|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = $0;
  $41 = (($40) + 24|0);
  $42 = HEAP32[$41>>2]|0;
  _free($42);
 }
 $43 = $0;
 $44 = (($43) + 28|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)!=(0|0);
 if ($46) {
  $47 = $0;
  $48 = (($47) + 28|0);
  $49 = HEAP32[$48>>2]|0;
  _free($49);
 }
 $50 = $0;
 $51 = (($50) + 32|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)!=(0|0);
 if (!($53)) {
  $57 = $0;
  _free($57);
  STACKTOP = sp;return;
 }
 $54 = $0;
 $55 = (($54) + 32|0);
 $56 = HEAP32[$55>>2]|0;
 _free($56);
 $57 = $0;
 _free($57);
 STACKTOP = sp;return;
}
function _getc_huffman_table_jpegl($huf_table,$cbufptr,$ebufptr) {
 $huf_table = $huf_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytes_left = 0, $i = 0, $ret = 0, $table_id = 0, $thuf_table = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0;
 var $vararg_buffer30 = 0, $vararg_buffer32 = 0, $vararg_buffer34 = 0, $vararg_buffer36 = 0, $vararg_buffer38 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr23 = 0, $vararg_ptr24 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer38 = sp + 144|0;
 $vararg_buffer36 = sp + 136|0;
 $vararg_buffer34 = sp + 128|0;
 $vararg_buffer32 = sp + 120|0;
 $vararg_buffer30 = sp + 112|0;
 $vararg_buffer28 = sp + 104|0;
 $vararg_buffer25 = sp + 96|0;
 $vararg_buffer20 = sp + 80|0;
 $vararg_buffer18 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer9 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $bytes_left = sp + 152|0;
 $table_id = sp + 180|0;
 $1 = $huf_table;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = (_calloc(1,40)|0);
 $thuf_table = $4;
 $5 = $thuf_table;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  (_fprintf(($7|0),(42768|0),($vararg_buffer|0))|0);
  $8 = HEAP32[_stderr>>2]|0;
  (_fprintf(($8|0),(42408|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 $9 = $thuf_table;
 $10 = (($9) + 4|0);
 $11 = $thuf_table;
 $12 = (($11) + 8|0);
 $13 = $2;
 $14 = $3;
 $15 = (_getc_huffman_table($table_id,$10,$12,16,$13,$14,1,$bytes_left)|0);
 $ret = $15;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = $thuf_table;
  _free_HUFF_TABLE($17);
  $18 = $ret;
  $0 = $18;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 $19 = HEAP32[$bytes_left>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[_stderr>>2]|0;
  (_fprintf(($21|0),(42768|0),($vararg_buffer3|0))|0);
  $22 = HEAP32[_stderr>>2]|0;
  $23 = HEAP8[$table_id>>0]|0;
  $24 = $23&255;
  HEAP32[$vararg_buffer5>>2] = $24;
  (_fprintf(($22|0),(42432|0),($vararg_buffer5|0))|0);
  $25 = $thuf_table;
  _free_HUFF_TABLE($25);
  $0 = -3;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 $26 = HEAP8[$table_id>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)<(16);
 if ($28) {
  label = 9;
 } else {
  $29 = HEAP8[$table_id>>0]|0;
  $30 = $29&255;
  $31 = ($30|0)>=(20);
  if ($31) {
   label = 9;
  }
 }
 do {
  if ((label|0) == 9) {
   $32 = HEAP8[$table_id>>0]|0;
   $33 = $32&255;
   $34 = ($33|0)<=(3);
   if ($34) {
    $35 = HEAP32[_stderr>>2]|0;
    (_fprintf(($35|0),(42808|0),($vararg_buffer7|0))|0);
    $36 = HEAP32[_stderr>>2]|0;
    $37 = HEAP8[$table_id>>0]|0;
    $38 = $37&255;
    HEAP32[$vararg_buffer9>>2] = $38;
    $vararg_ptr12 = (($vararg_buffer9) + 4|0);
    HEAP32[$vararg_ptr12>>2] = 16;
    $vararg_ptr13 = (($vararg_buffer9) + 8|0);
    HEAP32[$vararg_ptr13>>2] = 19;
    (_fprintf(($36|0),(42480|0),($vararg_buffer9|0))|0);
    $39 = HEAP32[_stderr>>2]|0;
    (_fprintf(($39|0),(42528|0),($vararg_buffer14|0))|0);
    $40 = HEAP32[_stderr>>2]|0;
    (_fprintf(($40|0),(42576|0),($vararg_buffer16|0))|0);
    $41 = HEAP8[$table_id>>0]|0;
    $42 = $41&255;
    $43 = (($42) + 16)|0;
    $44 = $43&255;
    HEAP8[$table_id>>0] = $44;
    break;
   } else {
    $45 = HEAP32[_stderr>>2]|0;
    (_fprintf(($45|0),(42768|0),($vararg_buffer18|0))|0);
    $46 = HEAP32[_stderr>>2]|0;
    $47 = HEAP8[$table_id>>0]|0;
    $48 = $47&255;
    HEAP32[$vararg_buffer20>>2] = $48;
    $vararg_ptr23 = (($vararg_buffer20) + 4|0);
    HEAP32[$vararg_ptr23>>2] = 16;
    $vararg_ptr24 = (($vararg_buffer20) + 8|0);
    HEAP32[$vararg_ptr24>>2] = 19;
    (_fprintf(($46|0),(42480|0),($vararg_buffer20|0))|0);
    $49 = $thuf_table;
    _free_HUFF_TABLE($49);
    $0 = -4;
    $145 = $0;
    STACKTOP = sp;return ($145|0);
   }
  }
 } while(0);
 $50 = HEAP8[$table_id>>0]|0;
 $51 = $thuf_table;
 $52 = (($51) + 1|0);
 HEAP8[$52>>0] = $50;
 $53 = $thuf_table;
 HEAP8[$53>>0] = 1;
 $54 = $thuf_table;
 $55 = (($54) + 1|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = $56&255;
 $58 = (($57) - 16)|0;
 $i = $58;
 $59 = $i;
 $60 = $1;
 $61 = (($60) + ($59<<2)|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = ($62|0)!=(0|0);
 if ($63) {
  $64 = $i;
  $65 = $1;
  $66 = (($65) + ($64<<2)|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = ($69|0)==(1);
  if ($70) {
   $71 = HEAP32[_stderr>>2]|0;
   $72 = $thuf_table;
   $73 = (($72) + 1|0);
   $74 = HEAP8[$73>>0]|0;
   $75 = $74&255;
   HEAP32[$vararg_buffer25>>2] = $75;
   (_fprintf(($71|0),(42624|0),($vararg_buffer25|0))|0);
   $76 = $thuf_table;
   _free_HUFF_TABLE($76);
   $0 = -5;
   $145 = $0;
   STACKTOP = sp;return ($145|0);
  }
 }
 $77 = $thuf_table;
 $78 = $i;
 $79 = $1;
 $80 = (($79) + ($78<<2)|0);
 HEAP32[$80>>2] = $77;
 $81 = (_calloc(17,4)|0);
 $82 = $thuf_table;
 $83 = (($82) + 24|0);
 HEAP32[$83>>2] = $81;
 $84 = $thuf_table;
 $85 = (($84) + 24|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = ($86|0)==(0|0);
 if ($87) {
  $88 = HEAP32[_stderr>>2]|0;
  (_fprintf(($88|0),(42768|0),($vararg_buffer28|0))|0);
  $89 = HEAP32[_stderr>>2]|0;
  (_fprintf(($89|0),(42696|0),($vararg_buffer30|0))|0);
  $90 = $thuf_table;
  _free_HUFF_TABLE($90);
  $0 = -6;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 $91 = (_calloc(17,4)|0);
 $92 = $thuf_table;
 $93 = (($92) + 28|0);
 HEAP32[$93>>2] = $91;
 $94 = $thuf_table;
 $95 = (($94) + 28|0);
 $96 = HEAP32[$95>>2]|0;
 $97 = ($96|0)==(0|0);
 if ($97) {
  $98 = HEAP32[_stderr>>2]|0;
  (_fprintf(($98|0),(42768|0),($vararg_buffer32|0))|0);
  $99 = HEAP32[_stderr>>2]|0;
  (_fprintf(($99|0),(42720|0),($vararg_buffer34|0))|0);
  $100 = $thuf_table;
  _free_HUFF_TABLE($100);
  $0 = -7;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 $101 = (_calloc(17,4)|0);
 $102 = $thuf_table;
 $103 = (($102) + 32|0);
 HEAP32[$103>>2] = $101;
 $104 = $thuf_table;
 $105 = (($104) + 32|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = ($106|0)==(0|0);
 if ($107) {
  $108 = HEAP32[_stderr>>2]|0;
  (_fprintf(($108|0),(42768|0),($vararg_buffer36|0))|0);
  $109 = HEAP32[_stderr>>2]|0;
  (_fprintf(($109|0),(42744|0),($vararg_buffer38|0))|0);
  $110 = $thuf_table;
  _free_HUFF_TABLE($110);
  $0 = -8;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 $111 = $thuf_table;
 $112 = (($111) + 36|0);
 $113 = $thuf_table;
 $114 = (($113) + 12|0);
 $115 = $thuf_table;
 $116 = (($115) + 4|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = (_build_huffsizes($112,$114,$117,16)|0);
 $ret = $118;
 $119 = ($118|0)!=(0);
 if ($119) {
  $120 = $thuf_table;
  _free_HUFF_TABLE($120);
  $121 = $ret;
  $0 = $121;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 } else {
  $122 = $thuf_table;
  $123 = (($122) + 36|0);
  $124 = HEAP32[$123>>2]|0;
  _build_huffcodes($124);
  $125 = $thuf_table;
  $126 = (($125) + 36|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = $thuf_table;
  $129 = (($128) + 24|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = $thuf_table;
  $132 = (($131) + 28|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = $thuf_table;
  $135 = (($134) + 32|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = $thuf_table;
  $138 = (($137) + 4|0);
  $139 = HEAP32[$138>>2]|0;
  _gen_decode_table($127,$130,$133,$136,$139);
  $140 = $thuf_table;
  $141 = (($140) + 36|0);
  $142 = HEAP32[$141>>2]|0;
  _free($142);
  $143 = $thuf_table;
  $144 = (($143) + 36|0);
  HEAP32[$144>>2] = 0;
  $0 = 0;
  $145 = $0;
  STACKTOP = sp;return ($145|0);
 }
 return 0|0;
}
function _free_HUFF_TABLES($huf_table,$N) {
 $huf_table = $huf_table|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $huf_table;
 $1 = $N;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = $1;
  $4 = ($2|0)<($3|0);
  if (!($4)) {
   break;
  }
  $5 = $i;
  $6 = $0;
  $7 = (($6) + ($5<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  if ($9) {
   $10 = $i;
   $11 = $0;
   $12 = (($11) + ($10<<2)|0);
   $13 = HEAP32[$12>>2]|0;
   _free_HUFF_TABLE($13);
  }
  $14 = $i;
  $15 = (($14) + 1)|0;
  $i = $15;
 }
 STACKTOP = sp;return;
}
function _get_IMG_DAT_image($odata,$olen,$width,$height,$depth,$ppi,$img_dat) {
 $odata = $odata|0;
 $olen = $olen|0;
 $width = $width|0;
 $height = $height|0;
 $depth = $depth|0;
 $ppi = $ppi|0;
 $img_dat = $img_dat|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $i = 0, $ndata = 0, $nlen = 0, $nptr = 0, $nsizes = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $nsizes = sp + 16|0;
 $1 = $odata;
 $2 = $olen;
 $3 = $width;
 $4 = $height;
 $5 = $depth;
 $6 = $ppi;
 $7 = $img_dat;
 $nlen = 0;
 $i = 0;
 while(1) {
  $8 = $i;
  $9 = $7;
  $10 = (($9) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = $i;
  $14 = $7;
  $15 = (($14) + 60|0);
  $16 = (($15) + ($13<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $i;
  $19 = $7;
  $20 = (($19) + 76|0);
  $21 = (($20) + ($18<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = Math_imul($17, $22)|0;
  $24 = $i;
  $25 = (($nsizes) + ($24<<2)|0);
  HEAP32[$25>>2] = $23;
  $26 = $i;
  $27 = (($nsizes) + ($26<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $nlen;
  $30 = (($29) + ($28))|0;
  $nlen = $30;
  $31 = $i;
  $32 = (($31) + 1)|0;
  $i = $32;
 }
 $33 = $nlen;
 $34 = $33;
 $35 = (_malloc($34)|0);
 $ndata = $35;
 $36 = $ndata;
 $37 = ($36|0)==(0|0);
 if ($37) {
  $38 = HEAP32[_stderr>>2]|0;
  (_fprintf(($38|0),(42848|0),($vararg_buffer|0))|0);
  $0 = -2;
  $81 = $0;
  STACKTOP = sp;return ($81|0);
 }
 $39 = $ndata;
 $nptr = $39;
 $i = 0;
 while(1) {
  $40 = $i;
  $41 = $7;
  $42 = (($41) + 20|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   break;
  }
  $45 = $nptr;
  $46 = $i;
  $47 = $7;
  $48 = (($47) + 100|0);
  $49 = (($48) + ($46<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = $i;
  $52 = (($nsizes) + ($51<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $53;
  _memcpy(($45|0),($50|0),($54|0))|0;
  $55 = $i;
  $56 = (($nsizes) + ($55<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $nptr;
  $59 = (($58) + ($57)|0);
  $nptr = $59;
  $60 = $i;
  $61 = (($60) + 1)|0;
  $i = $61;
 }
 $62 = $ndata;
 $63 = $1;
 HEAP32[$63>>2] = $62;
 $64 = $nlen;
 $65 = $2;
 HEAP32[$65>>2] = $64;
 $66 = $7;
 $67 = HEAP32[$66>>2]|0;
 $68 = $3;
 HEAP32[$68>>2] = $67;
 $69 = $7;
 $70 = (($69) + 4|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = $4;
 HEAP32[$72>>2] = $71;
 $73 = $7;
 $74 = (($73) + 8|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = $5;
 HEAP32[$76>>2] = $75;
 $77 = $7;
 $78 = (($77) + 12|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = $6;
 HEAP32[$80>>2] = $79;
 $0 = 0;
 $81 = $0;
 STACKTOP = sp;return ($81|0);
}
function _setup_IMG_DAT_decode($oimg_dat,$ppi,$frm_header) {
 $oimg_dat = $oimg_dat|0;
 $ppi = $ppi|0;
 $frm_header = $frm_header|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0.0, $113 = 0.0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0.0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $img_dat = 0, $max_hor = 0, $max_vrt = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $1 = $oimg_dat;
 $2 = $ppi;
 $3 = $frm_header;
 $4 = (_calloc(1,132)|0);
 $img_dat = $4;
 $5 = $img_dat;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  (_fprintf(($7|0),(42928|0),($vararg_buffer|0))|0);
  $0 = -2;
  $143 = $0;
  STACKTOP = sp;return ($143|0);
 }
 $8 = $3;
 $9 = (($8) + 2|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $img_dat;
 HEAP32[$12>>2] = $11;
 $13 = $3;
 $14 = (($13) + 4|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $img_dat;
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = $16;
 $19 = $3;
 $20 = (($19) + 6|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $22<<3;
 $24 = $img_dat;
 $25 = (($24) + 8|0);
 HEAP32[$25>>2] = $23;
 $26 = $2;
 $27 = $img_dat;
 $28 = (($27) + 12|0);
 HEAP32[$28>>2] = $26;
 $29 = $img_dat;
 $30 = (($29) + 16|0);
 HEAP32[$30>>2] = -1;
 $31 = $3;
 $32 = (($31) + 6|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = $img_dat;
 $36 = (($35) + 20|0);
 HEAP32[$36>>2] = $34;
 $37 = $3;
 $38 = HEAP8[$37>>0]|0;
 $39 = $38&255;
 $40 = $img_dat;
 $41 = (($40) + 24|0);
 HEAP32[$41>>2] = $39;
 $max_hor = -1;
 $max_vrt = -1;
 $i = 0;
 while(1) {
  $42 = $i;
  $43 = $img_dat;
  $44 = (($43) + 20|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($42|0)<($45|0);
  if (!($46)) {
   break;
  }
  $47 = $i;
  $48 = $3;
  $49 = (($48) + 11|0);
  $50 = (($49) + ($47)|0);
  $51 = HEAP8[$50>>0]|0;
  $52 = $51&255;
  $53 = $52 >> 4;
  $54 = $i;
  $55 = $img_dat;
  $56 = (($55) + 28|0);
  $57 = (($56) + ($54<<2)|0);
  HEAP32[$57>>2] = $53;
  $58 = $i;
  $59 = $3;
  $60 = (($59) + 11|0);
  $61 = (($60) + ($58)|0);
  $62 = HEAP8[$61>>0]|0;
  $63 = $62&255;
  $64 = $63 & 15;
  $65 = $i;
  $66 = $img_dat;
  $67 = (($66) + 44|0);
  $68 = (($67) + ($65<<2)|0);
  HEAP32[$68>>2] = $64;
  $69 = $max_hor;
  $70 = $i;
  $71 = $img_dat;
  $72 = (($71) + 28|0);
  $73 = (($72) + ($70<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = ($69|0)<($74|0);
  if ($75) {
   $76 = $i;
   $77 = $img_dat;
   $78 = (($77) + 28|0);
   $79 = (($78) + ($76<<2)|0);
   $80 = HEAP32[$79>>2]|0;
   $max_hor = $80;
  }
  $81 = $max_vrt;
  $82 = $i;
  $83 = $img_dat;
  $84 = (($83) + 44|0);
  $85 = (($84) + ($82<<2)|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = ($81|0)<($86|0);
  if ($87) {
   $88 = $i;
   $89 = $img_dat;
   $90 = (($89) + 44|0);
   $91 = (($90) + ($88<<2)|0);
   $92 = HEAP32[$91>>2]|0;
   $max_vrt = $92;
  }
  $93 = $i;
  $94 = (($93) + 1)|0;
  $i = $94;
 }
 $i = 0;
 while(1) {
  $95 = $i;
  $96 = $img_dat;
  $97 = (($96) + 20|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = ($95|0)<($98|0);
  if (!($99)) {
   break;
  }
  $100 = $img_dat;
  $101 = HEAP32[$100>>2]|0;
  $102 = (+($101|0));
  $103 = $i;
  $104 = $img_dat;
  $105 = (($104) + 28|0);
  $106 = (($105) + ($103<<2)|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (+($107|0));
  $109 = $max_hor;
  $110 = (+($109|0));
  $111 = $108 / $110;
  $112 = $102 * $111;
  $113 = (+Math_ceil((+$112)));
  $114 = (~~(($113)));
  $115 = $i;
  $116 = $img_dat;
  $117 = (($116) + 60|0);
  $118 = (($117) + ($115<<2)|0);
  HEAP32[$118>>2] = $114;
  $119 = $img_dat;
  $120 = (($119) + 4|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = (+($121|0));
  $123 = $i;
  $124 = $img_dat;
  $125 = (($124) + 44|0);
  $126 = (($125) + ($123<<2)|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = (+($127|0));
  $129 = $max_vrt;
  $130 = (+($129|0));
  $131 = $128 / $130;
  $132 = $122 * $131;
  $133 = (+Math_ceil((+$132)));
  $134 = (~~(($133)));
  $135 = $i;
  $136 = $img_dat;
  $137 = (($136) + 76|0);
  $138 = (($137) + ($135<<2)|0);
  HEAP32[$138>>2] = $134;
  $139 = $i;
  $140 = (($139) + 1)|0;
  $i = $140;
 }
 $141 = $img_dat;
 $142 = $1;
 HEAP32[$142>>2] = $141;
 $0 = 0;
 $143 = $0;
 STACKTOP = sp;return ($143|0);
}
function _update_IMG_DAT_decode($img_dat,$scn_header,$huf_table) {
 $img_dat = $img_dat|0;
 $scn_header = $scn_header|0;
 $huf_table = $huf_table|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $cmpnt_i = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $img_dat;
 $2 = $scn_header;
 $3 = $huf_table;
 $4 = $2;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)>(1);
 if ($7) {
  $8 = $1;
  $9 = (($8) + 16|0);
  HEAP32[$9>>2] = 1;
 } else {
  $10 = $1;
  $11 = (($10) + 16|0);
  HEAP32[$11>>2] = 0;
 }
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $2;
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 13;
   break;
  }
  $17 = $i;
  $18 = $2;
  $19 = (($18) + 1|0);
  $20 = (($19) + ($17)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $cmpnt_i = $22;
  $23 = $cmpnt_i;
  $24 = $3;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)==(0|0);
  if ($27) {
   label = 8;
   break;
  }
  $28 = $cmpnt_i;
  $29 = $3;
  $30 = (($29) + ($28<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)!=(1);
  if ($34) {
   label = 8;
   break;
  }
  $38 = $2;
  $39 = (($38) + 11|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $cmpnt_i;
  $42 = $1;
  $43 = (($42) + 92|0);
  $44 = (($43) + ($41)|0);
  HEAP8[$44>>0] = $40;
  $45 = $2;
  $46 = (($45) + 9|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $cmpnt_i;
  $49 = $1;
  $50 = (($49) + 96|0);
  $51 = (($50) + ($48)|0);
  HEAP8[$51>>0] = $47;
  $52 = $cmpnt_i;
  $53 = $1;
  $54 = (($53) + 60|0);
  $55 = (($54) + ($52<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $cmpnt_i;
  $58 = $1;
  $59 = (($58) + 76|0);
  $60 = (($59) + ($57<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = Math_imul($56, $61)|0;
  $63 = (_malloc($62)|0);
  $64 = $cmpnt_i;
  $65 = $1;
  $66 = (($65) + 100|0);
  $67 = (($66) + ($64<<2)|0);
  HEAP32[$67>>2] = $63;
  $68 = $cmpnt_i;
  $69 = $1;
  $70 = (($69) + 100|0);
  $71 = (($70) + ($68<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)==(0|0);
  if ($73) {
   label = 10;
   break;
  }
  $77 = $i;
  $78 = (($77) + 1)|0;
  $i = $78;
 }
 if ((label|0) == 8) {
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(42984|0),($vararg_buffer|0))|0);
  $36 = HEAP32[_stderr>>2]|0;
  $37 = $cmpnt_i;
  HEAP32[$vararg_buffer1>>2] = $37;
  (_fprintf(($36|0),(43024|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 10) {
  $74 = HEAP32[_stderr>>2]|0;
  (_fprintf(($74|0),(42984|0),($vararg_buffer3|0))|0);
  $75 = HEAP32[_stderr>>2]|0;
  $76 = $cmpnt_i;
  HEAP32[$vararg_buffer5>>2] = $76;
  (_fprintf(($75|0),(42896|0),($vararg_buffer5|0))|0);
  $0 = -3;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 13) {
  $0 = 0;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 return 0|0;
}
function _free_IMG_DAT($img_dat,$img_flag) {
 $img_dat = $img_dat|0;
 $img_flag = $img_flag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $img_dat;
 $1 = $img_flag;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = $0;
  $4 = (($3) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $0;
  $9 = (($8) + 116|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = $i;
   $14 = $0;
   $15 = (($14) + 116|0);
   $16 = (($15) + ($13<<2)|0);
   $17 = HEAP32[$16>>2]|0;
   _free($17);
  }
  $18 = $i;
  $19 = (($18) + 1)|0;
  $i = $19;
 }
 $20 = $1;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  $40 = $0;
  _free($40);
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $22 = $i;
  $23 = $0;
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($22|0)<($25|0);
  if (!($26)) {
   break;
  }
  $27 = $i;
  $28 = $0;
  $29 = (($28) + 100|0);
  $30 = (($29) + ($27<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = $i;
   $34 = $0;
   $35 = (($34) + 100|0);
   $36 = (($35) + ($33<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   _free($37);
  }
  $38 = $i;
  $39 = (($38) + 1)|0;
  $i = $39;
 }
 $40 = $0;
 _free($40);
 STACKTOP = sp;return;
}
function _get_ppi_jpegl($oppi,$jfif_header) {
 $oppi = $oppi|0;
 $jfif_header = $jfif_header|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ppi = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $oppi;
 $2 = $jfif_header;
 $3 = $2;
 $4 = (($3) + 7|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 if ((($6|0) == 1)) {
  $7 = $2;
  $8 = (($7) + 8|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9&65535;
  $ppi = $10;
 } else if ((($6|0) == 0)) {
  $ppi = -1;
 } else if ((($6|0) == 2)) {
  $11 = $2;
  $12 = (($11) + 8|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  $15 = (+($14|0));
  $16 = $15 * 2.54000000000000003553;
  $17 = $16 + 0.5;
  $18 = (~~(($17)));
  $ppi = $18;
 } else {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(43056|0),($vararg_buffer|0))|0);
  $20 = HEAP32[_stderr>>2]|0;
  $21 = $2;
  $22 = (($21) + 7|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  HEAP32[$vararg_buffer1>>2] = $24;
  (_fprintf(($20|0),(43088|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 $25 = $ppi;
 $26 = $1;
 HEAP32[$26>>2] = $25;
 $0 = 0;
 $27 = $0;
 STACKTOP = sp;return ($27|0);
}
function _getc_marker_jpegl($omarker,$type,$cbufptr,$ebufptr) {
 $omarker = $omarker|0;
 $type = $type|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $marker = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer14 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $marker = sp + 92|0;
 $1 = $omarker;
 $2 = $type;
 $3 = $cbufptr;
 $4 = $ebufptr;
 $5 = $3;
 $6 = $4;
 $7 = (_getc_ushort($marker,$5,$6)|0);
 $ret = $7;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $ret;
  $0 = $9;
  $63 = $0;
  STACKTOP = sp;return ($63|0);
 }
 $10 = HEAP32[8>>2]|0;
 $11 = ($10|0)>(0);
 if ($11) {
  $12 = HEAP32[_stdout>>2]|0;
  $13 = HEAP16[$marker>>1]|0;
  $14 = $13&65535;
  $15 = $2;
  HEAP32[$vararg_buffer>>2] = $14;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $15;
  (_fprintf(($12|0),(43120|0),($vararg_buffer|0))|0);
 }
 $16 = $2;
 do {
  if ((($16|0) == 65504)) {
   $23 = HEAP16[$marker>>1]|0;
   $24 = $23&65535;
   $25 = ($24|0)!=(65504);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[_stderr>>2]|0;
   $27 = HEAP16[$marker>>1]|0;
   $28 = $27&65535;
   HEAP32[$vararg_buffer5>>2] = $28;
   (_fprintf(($26|0),(43272|0),($vararg_buffer5|0))|0);
   $0 = -3;
   $63 = $0;
   STACKTOP = sp;return ($63|0);
  } else if ((($16|0) == 2)) {
   $29 = HEAP16[$marker>>1]|0;
   $30 = $29&65535;
   $31 = ($30|0)!=(65476);
   if ($31) {
    $32 = HEAP16[$marker>>1]|0;
    $33 = $32&65535;
    $34 = ($33|0)!=(65534);
    if ($34) {
     $35 = HEAP16[$marker>>1]|0;
     $36 = $35&65535;
     $37 = ($36|0)!=(65475);
     if ($37) {
      $38 = HEAP32[_stderr>>2]|0;
      (_fprintf(($38|0),(43336|0),($vararg_buffer8|0))|0);
      $39 = HEAP32[_stderr>>2]|0;
      (_fprintf(($39|0),(43152|0),($vararg_buffer10|0))|0);
      $0 = -4;
      $63 = $0;
      STACKTOP = sp;return ($63|0);
     }
    }
   }
  } else if ((($16|0) == 65535)) {
   $51 = HEAP16[$marker>>1]|0;
   $52 = $51&65535;
   $53 = $52 & 65280;
   $54 = ($53|0)!=(65280);
   if (!($54)) {
    break;
   }
   $55 = HEAP32[_stderr>>2]|0;
   $56 = HEAP16[$marker>>1]|0;
   $57 = $56&65535;
   HEAP32[$vararg_buffer16>>2] = $57;
   (_fprintf(($55|0),(43368|0),($vararg_buffer16|0))|0);
   $0 = -6;
   $63 = $0;
   STACKTOP = sp;return ($63|0);
  } else if ((($16|0) == 3)) {
   $40 = HEAP16[$marker>>1]|0;
   $41 = $40&65535;
   $42 = ($41|0)!=(65476);
   if ($42) {
    $43 = HEAP16[$marker>>1]|0;
    $44 = $43&65535;
    $45 = ($44|0)!=(65534);
    if ($45) {
     $46 = HEAP16[$marker>>1]|0;
     $47 = $46&65535;
     $48 = ($47|0)!=(65498);
     if ($48) {
      $49 = HEAP32[_stderr>>2]|0;
      (_fprintf(($49|0),(43336|0),($vararg_buffer12|0))|0);
      $50 = HEAP32[_stderr>>2]|0;
      (_fprintf(($50|0),(43184|0),($vararg_buffer14|0))|0);
      $0 = -5;
      $63 = $0;
      STACKTOP = sp;return ($63|0);
     }
    }
   }
  } else if ((($16|0) == 65496)) {
   $17 = HEAP16[$marker>>1]|0;
   $18 = $17&65535;
   $19 = ($18|0)!=(65496);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[_stderr>>2]|0;
   $21 = HEAP16[$marker>>1]|0;
   $22 = $21&65535;
   HEAP32[$vararg_buffer2>>2] = $22;
   (_fprintf(($20|0),(43216|0),($vararg_buffer2|0))|0);
   $0 = -2;
   $63 = $0;
   STACKTOP = sp;return ($63|0);
  } else {
   $58 = HEAP32[_stderr>>2]|0;
   $59 = HEAP16[$marker>>1]|0;
   $60 = $59&65535;
   HEAP32[$vararg_buffer19>>2] = $60;
   (_fprintf(($58|0),(43424|0),($vararg_buffer19|0))|0);
   $0 = -6;
   $63 = $0;
   STACKTOP = sp;return ($63|0);
  }
 } while(0);
 $61 = HEAP16[$marker>>1]|0;
 $62 = $1;
 HEAP16[$62>>1] = $61;
 $0 = 0;
 $63 = $0;
 STACKTOP = sp;return ($63|0);
}
function _getc_jfif_header($ojfif_header,$cbufptr,$ebufptr) {
 $ojfif_header = $ojfif_header|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $jfif_header = 0;
 var $ret = 0, $table_len = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer15 = 0, $vararg_buffer18 = 0, $vararg_buffer21 = 0, $vararg_buffer24 = 0, $vararg_buffer27 = 0, $vararg_buffer3 = 0, $vararg_buffer30 = 0, $vararg_buffer33 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer33 = sp + 104|0;
 $vararg_buffer30 = sp + 96|0;
 $vararg_buffer27 = sp + 88|0;
 $vararg_buffer24 = sp + 80|0;
 $vararg_buffer21 = sp + 72|0;
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer15 = sp + 56|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer9 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $table_len = sp + 136|0;
 $1 = $ojfif_header;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stdout>>2]|0;
  (_fprintf(($6|0),(43488|0),($vararg_buffer|0))|0);
 }
 $7 = (_malloc(14)|0);
 $jfif_header = $7;
 $8 = $jfif_header;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(43688|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $11 = $2;
 $12 = $3;
 $13 = (_getc_ushort($table_len,$11,$12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $jfif_header;
  _free($15);
  $0 = -3;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $i = 0;
 while(1) {
  $16 = $i;
  $17 = ($16|0)<(5);
  if (!($17)) {
   break;
  }
  $18 = $i;
  $19 = $jfif_header;
  $20 = (($19) + 2|0);
  $21 = (($20) + ($18)|0);
  $22 = $2;
  $23 = $3;
  $24 = (_getc_byte($21,$22,$23)|0);
  $ret = $24;
  $25 = ($24|0)!=(0);
  if ($25) {
   label = 10;
   break;
  }
  $27 = $i;
  $28 = (($27) + 1)|0;
  $i = $28;
 }
 if ((label|0) == 10) {
  $26 = $jfif_header;
  _free($26);
  $0 = -4;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $29 = $jfif_header;
 $30 = (($29) + 2|0);
 $31 = (_strcmp($30,43480)|0);
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[_stderr>>2]|0;
  (_fprintf(($33|0),(43744|0),($vararg_buffer3|0))|0);
  $34 = $jfif_header;
  _free($34);
  $0 = -5;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $35 = $jfif_header;
 $36 = $2;
 $37 = $3;
 $38 = (_getc_ushort($35,$36,$37)|0);
 $ret = $38;
 $39 = ($38|0)!=(0);
 if ($39) {
  $40 = $jfif_header;
  _free($40);
  $0 = -6;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $41 = $jfif_header;
 $42 = (($41) + 7|0);
 $43 = $2;
 $44 = $3;
 $45 = (_getc_byte($42,$43,$44)|0);
 $ret = $45;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = $jfif_header;
  _free($47);
  $0 = -7;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $48 = $jfif_header;
 $49 = (($48) + 8|0);
 $50 = $2;
 $51 = $3;
 $52 = (_getc_ushort($49,$50,$51)|0);
 $ret = $52;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = $jfif_header;
  _free($54);
  $0 = -8;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $55 = $jfif_header;
 $56 = (($55) + 10|0);
 $57 = $2;
 $58 = $3;
 $59 = (_getc_ushort($56,$57,$58)|0);
 $ret = $59;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = $jfif_header;
  _free($61);
  $0 = -9;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $62 = $jfif_header;
 $63 = (($62) + 12|0);
 $64 = $2;
 $65 = $3;
 $66 = (_getc_byte($63,$64,$65)|0);
 $ret = $66;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = $jfif_header;
  _free($68);
  $0 = -10;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $69 = $jfif_header;
 $70 = (($69) + 13|0);
 $71 = $2;
 $72 = $3;
 $73 = (_getc_byte($70,$71,$72)|0);
 $ret = $73;
 $74 = ($73|0)!=(0);
 if ($74) {
  $75 = $jfif_header;
  _free($75);
  $0 = -11;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 $76 = $jfif_header;
 $77 = (($76) + 13|0);
 $78 = HEAP8[$77>>0]|0;
 $79 = $78&255;
 $80 = ($79|0)!=(0);
 if (!($80)) {
  $81 = $jfif_header;
  $82 = (($81) + 12|0);
  $83 = HEAP8[$82>>0]|0;
  $84 = $83&255;
  $85 = ($84|0)!=(0);
  if (!($85)) {
   $88 = HEAP32[8>>2]|0;
   $89 = ($88|0)>(1);
   if ($89) {
    $90 = HEAP32[_stdout>>2]|0;
    $91 = HEAP16[$table_len>>1]|0;
    $92 = $91&65535;
    HEAP32[$vararg_buffer7>>2] = $92;
    (_fprintf(($90|0),(43520|0),($vararg_buffer7|0))|0);
    $93 = HEAP32[_stdout>>2]|0;
    $94 = $jfif_header;
    $95 = (($94) + 2|0);
    HEAP32[$vararg_buffer9>>2] = $95;
    (_fprintf(($93|0),(43536|0),($vararg_buffer9|0))|0);
    $96 = HEAP32[_stdout>>2]|0;
    $97 = $jfif_header;
    $98 = HEAP16[$97>>1]|0;
    $99 = $98&65535;
    $100 = $99 & 65280;
    $101 = $100 >> 8;
    HEAP32[$vararg_buffer12>>2] = $101;
    (_fprintf(($96|0),(43552|0),($vararg_buffer12|0))|0);
    $102 = HEAP32[_stdout>>2]|0;
    $103 = $jfif_header;
    $104 = HEAP16[$103>>1]|0;
    $105 = $104&65535;
    $106 = $105 & 255;
    HEAP32[$vararg_buffer15>>2] = $106;
    (_fprintf(($102|0),(43568|0),($vararg_buffer15|0))|0);
    $107 = HEAP32[_stdout>>2]|0;
    $108 = $jfif_header;
    $109 = (($108) + 7|0);
    $110 = HEAP8[$109>>0]|0;
    $111 = $110&255;
    HEAP32[$vararg_buffer18>>2] = $111;
    (_fprintf(($107|0),(43576|0),($vararg_buffer18|0))|0);
    $112 = HEAP32[_stdout>>2]|0;
    $113 = $jfif_header;
    $114 = (($113) + 8|0);
    $115 = HEAP16[$114>>1]|0;
    $116 = $115&65535;
    HEAP32[$vararg_buffer21>>2] = $116;
    (_fprintf(($112|0),(43592|0),($vararg_buffer21|0))|0);
    $117 = HEAP32[_stdout>>2]|0;
    $118 = $jfif_header;
    $119 = (($118) + 10|0);
    $120 = HEAP16[$119>>1]|0;
    $121 = $120&65535;
    HEAP32[$vararg_buffer24>>2] = $121;
    (_fprintf(($117|0),(43608|0),($vararg_buffer24|0))|0);
    $122 = HEAP32[_stdout>>2]|0;
    $123 = $jfif_header;
    $124 = (($123) + 12|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = $125&255;
    HEAP32[$vararg_buffer27>>2] = $126;
    (_fprintf(($122|0),(43624|0),($vararg_buffer27|0))|0);
    $127 = HEAP32[_stdout>>2]|0;
    $128 = $jfif_header;
    $129 = (($128) + 13|0);
    $130 = HEAP8[$129>>0]|0;
    $131 = $130&255;
    HEAP32[$vararg_buffer30>>2] = $131;
    (_fprintf(($127|0),(43640|0),($vararg_buffer30|0))|0);
   }
   $132 = HEAP32[8>>2]|0;
   $133 = ($132|0)>(0);
   if ($133) {
    $134 = HEAP32[_stdout>>2]|0;
    (_fprintf(($134|0),(43656|0),($vararg_buffer33|0))|0);
   }
   $135 = $jfif_header;
   $136 = $1;
   HEAP32[$136>>2] = $135;
   $0 = 0;
   $137 = $0;
   STACKTOP = sp;return ($137|0);
  }
 }
 $86 = HEAP32[_stderr>>2]|0;
 (_fprintf(($86|0),(43792|0),($vararg_buffer5|0))|0);
 $87 = $jfif_header;
 _free($87);
 $0 = -12;
 $137 = $0;
 STACKTOP = sp;return ($137|0);
}
function _getc_table_jpegl($marker,$huf_table,$cbufptr,$ebufptr) {
 $marker = $marker|0;
 $huf_table = $huf_table|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $comment = 0, $ret = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $comment = sp + 4|0;
 $1 = $marker;
 $2 = $huf_table;
 $3 = $cbufptr;
 $4 = $ebufptr;
 $5 = $1;
 $6 = $5&65535;
 do {
  if ((($6|0) == 65534)) {
   $13 = $3;
   $14 = $4;
   $15 = (_getc_comment($comment,$13,$14)|0);
   $ret = $15;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $18 = HEAP32[$comment>>2]|0;
    _free($18);
    break;
   }
   $17 = $ret;
   $0 = $17;
   $22 = $0;
   STACKTOP = sp;return ($22|0);
  } else if ((($6|0) == 65476)) {
   $7 = $2;
   $8 = $3;
   $9 = $4;
   $10 = (_getc_huffman_table_jpegl($7,$8,$9)|0);
   $ret = $10;
   $11 = ($10|0)!=(0);
   if (!($11)) {
    break;
   }
   $12 = $ret;
   $0 = $12;
   $22 = $0;
   STACKTOP = sp;return ($22|0);
  } else {
   $19 = HEAP32[_stderr>>2]|0;
   $20 = $1;
   $21 = $20&65535;
   HEAP32[$vararg_buffer>>2] = $21;
   (_fprintf(($19|0),(43848|0),($vararg_buffer|0))|0);
   $0 = -2;
   $22 = $0;
   STACKTOP = sp;return ($22|0);
  }
 } while(0);
 $0 = 0;
 $22 = $0;
 STACKTOP = sp;return ($22|0);
}
function _getc_comment($ocomment,$cbufptr,$ebufptr) {
 $ocomment = $ocomment|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $comment = 0, $cs = 0, $hdr_size = 0, $ret = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $hdr_size = sp + 48|0;
 $comment = sp + 20|0;
 $1 = $ocomment;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(44368|0),($vararg_buffer|0))|0);
 }
 $7 = $2;
 $8 = $3;
 $9 = (_getc_ushort($hdr_size,$7,$8)|0);
 $ret = $9;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $ret;
  $0 = $11;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 $12 = HEAP16[$hdr_size>>1]|0;
 $13 = $12&65535;
 $14 = (($13) - 2)|0;
 $cs = $14;
 $15 = $cs;
 $16 = (($15) + 1)|0;
 $17 = (_calloc($16,1)|0);
 HEAP32[$comment>>2] = $17;
 $18 = HEAP32[$comment>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  $20 = HEAP32[_stderr>>2]|0;
  (_fprintf(($20|0),(44408|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 $21 = $cs;
 $22 = $2;
 $23 = $3;
 $24 = (_getc_bytes($comment,$21,$22,$23)|0);
 $ret = $24;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP32[$comment>>2]|0;
  _free($26);
  $27 = $ret;
  $0 = $27;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 $28 = HEAP32[8>>2]|0;
 $29 = ($28|0)>(0);
 if ($29) {
  $30 = HEAP32[_stderr>>2]|0;
  $31 = HEAP32[$comment>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $31;
  (_fprintf(($30|0),(44392|0),($vararg_buffer3|0))|0);
 }
 $32 = HEAP32[$comment>>2]|0;
 $33 = $1;
 HEAP32[$33>>2] = $32;
 $0 = 0;
 $34 = $0;
 STACKTOP = sp;return ($34|0);
}
function _getc_frame_header_jpegl($ofrm_header,$cbufptr,$ebufptr) {
 $ofrm_header = $ofrm_header|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Lf = 0, $frm_header = 0, $i = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer23 = 0;
 var $vararg_buffer27 = 0, $vararg_buffer3 = 0, $vararg_buffer31 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr30 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer31 = sp + 88|0;
 $vararg_buffer27 = sp + 80|0;
 $vararg_buffer23 = sp + 72|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $Lf = sp + 120|0;
 $1 = $ofrm_header;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stdout>>2]|0;
  (_fprintf(($6|0),(43936|0),($vararg_buffer|0))|0);
 }
 $7 = (_malloc(20)|0);
 $frm_header = $7;
 $8 = $frm_header;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(44104|0),($vararg_buffer1|0))|0);
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(43912|0),($vararg_buffer3|0))|0);
  $0 = -2;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 $12 = $2;
 $13 = $3;
 $14 = (_getc_ushort($Lf,$12,$13)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $frm_header;
  _free($16);
  $17 = $ret;
  $0 = $17;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 $18 = $frm_header;
 $19 = $2;
 $20 = $3;
 $21 = (_getc_byte($18,$19,$20)|0);
 $ret = $21;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $frm_header;
  _free($23);
  $24 = $ret;
  $0 = $24;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 $25 = $frm_header;
 $26 = (($25) + 4|0);
 $27 = $2;
 $28 = $3;
 $29 = (_getc_ushort($26,$27,$28)|0);
 $ret = $29;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = $frm_header;
  _free($31);
  $32 = $ret;
  $0 = $32;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 $33 = $frm_header;
 $34 = (($33) + 2|0);
 $35 = $2;
 $36 = $3;
 $37 = (_getc_ushort($34,$35,$36)|0);
 $ret = $37;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = $frm_header;
  _free($39);
  $40 = $ret;
  $0 = $40;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 $41 = $frm_header;
 $42 = (($41) + 6|0);
 $43 = $2;
 $44 = $3;
 $45 = (_getc_byte($42,$43,$44)|0);
 $ret = $45;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = $frm_header;
  _free($47);
  $48 = $ret;
  $0 = $48;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 $i = 0;
 while(1) {
  $49 = $i;
  $50 = $frm_header;
  $51 = (($50) + 6|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = ($49|0)<($53|0);
  if (!($54)) {
   label = 25;
   break;
  }
  $55 = $i;
  $56 = $frm_header;
  $57 = (($56) + 7|0);
  $58 = (($57) + ($55)|0);
  $59 = $2;
  $60 = $3;
  $61 = (_getc_byte($58,$59,$60)|0);
  $ret = $61;
  $62 = ($61|0)!=(0);
  if ($62) {
   label = 18;
   break;
  }
  $65 = $i;
  $66 = $frm_header;
  $67 = (($66) + 11|0);
  $68 = (($67) + ($65)|0);
  $69 = $2;
  $70 = $3;
  $71 = (_getc_byte($68,$69,$70)|0);
  $ret = $71;
  $72 = ($71|0)!=(0);
  if ($72) {
   label = 20;
   break;
  }
  $75 = $i;
  $76 = $frm_header;
  $77 = (($76) + 15|0);
  $78 = (($77) + ($75)|0);
  $79 = $2;
  $80 = $3;
  $81 = (_getc_byte($78,$79,$80)|0);
  $ret = $81;
  $82 = ($81|0)!=(0);
  if ($82) {
   label = 22;
   break;
  }
  $85 = $i;
  $86 = (($85) + 1)|0;
  $i = $86;
 }
 if ((label|0) == 18) {
  $63 = $frm_header;
  _free($63);
  $64 = $ret;
  $0 = $64;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 else if ((label|0) == 20) {
  $73 = $frm_header;
  _free($73);
  $74 = $ret;
  $0 = $74;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 else if ((label|0) == 22) {
  $83 = $frm_header;
  _free($83);
  $84 = $ret;
  $0 = $84;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 else if ((label|0) == 25) {
  $87 = HEAP32[8>>2]|0;
  $88 = ($87|0)>(1);
  if ($88) {
   $89 = HEAP32[_stdout>>2]|0;
   $90 = HEAP16[$Lf>>1]|0;
   $91 = $90&65535;
   HEAP32[$vararg_buffer5>>2] = $91;
   (_fprintf(($89|0),(43968|0),($vararg_buffer5|0))|0);
   $92 = HEAP32[_stdout>>2]|0;
   $93 = $frm_header;
   $94 = HEAP8[$93>>0]|0;
   $95 = $94&255;
   HEAP32[$vararg_buffer7>>2] = $95;
   (_fprintf(($92|0),(43984|0),($vararg_buffer7|0))|0);
   $96 = HEAP32[_stdout>>2]|0;
   $97 = $frm_header;
   $98 = (($97) + 4|0);
   $99 = HEAP16[$98>>1]|0;
   $100 = $99&65535;
   HEAP32[$vararg_buffer10>>2] = $100;
   (_fprintf(($96|0),(43992|0),($vararg_buffer10|0))|0);
   $101 = HEAP32[_stdout>>2]|0;
   $102 = $frm_header;
   $103 = (($102) + 2|0);
   $104 = HEAP16[$103>>1]|0;
   $105 = $104&65535;
   HEAP32[$vararg_buffer13>>2] = $105;
   (_fprintf(($101|0),(44000|0),($vararg_buffer13|0))|0);
   $106 = HEAP32[_stdout>>2]|0;
   $107 = $frm_header;
   $108 = (($107) + 6|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = $109&255;
   HEAP32[$vararg_buffer16>>2] = $110;
   (_fprintf(($106|0),(44008|0),($vararg_buffer16|0))|0);
   $i = 0;
   while(1) {
    $111 = $i;
    $112 = $frm_header;
    $113 = (($112) + 6|0);
    $114 = HEAP8[$113>>0]|0;
    $115 = $114&255;
    $116 = ($111|0)<($115|0);
    if (!($116)) {
     break;
    }
    $117 = HEAP32[_stdout>>2]|0;
    $118 = $i;
    $119 = $i;
    $120 = $frm_header;
    $121 = (($120) + 7|0);
    $122 = (($121) + ($119)|0);
    $123 = HEAP8[$122>>0]|0;
    $124 = $123&255;
    HEAP32[$vararg_buffer19>>2] = $118;
    $vararg_ptr22 = (($vararg_buffer19) + 4|0);
    HEAP32[$vararg_ptr22>>2] = $124;
    (_fprintf(($117|0),(44024|0),($vararg_buffer19|0))|0);
    $125 = HEAP32[_stdout>>2]|0;
    $126 = $i;
    $127 = $i;
    $128 = $frm_header;
    $129 = (($128) + 11|0);
    $130 = (($129) + ($127)|0);
    $131 = HEAP8[$130>>0]|0;
    $132 = $131&255;
    HEAP32[$vararg_buffer23>>2] = $126;
    $vararg_ptr26 = (($vararg_buffer23) + 4|0);
    HEAP32[$vararg_ptr26>>2] = $132;
    (_fprintf(($125|0),(44040|0),($vararg_buffer23|0))|0);
    $133 = HEAP32[_stdout>>2]|0;
    $134 = $i;
    $135 = $i;
    $136 = $frm_header;
    $137 = (($136) + 15|0);
    $138 = (($137) + ($135)|0);
    $139 = HEAP8[$138>>0]|0;
    $140 = $139&255;
    HEAP32[$vararg_buffer27>>2] = $134;
    $vararg_ptr30 = (($vararg_buffer27) + 4|0);
    HEAP32[$vararg_ptr30>>2] = $140;
    (_fprintf(($133|0),(44056|0),($vararg_buffer27|0))|0);
    $141 = $i;
    $142 = (($141) + 1)|0;
    $i = $142;
   }
  }
  $143 = HEAP32[8>>2]|0;
  $144 = ($143|0)>(0);
  if ($144) {
   $145 = HEAP32[_stdout>>2]|0;
   (_fprintf(($145|0),(44072|0),($vararg_buffer31|0))|0);
  }
  $146 = $frm_header;
  $147 = $1;
  HEAP32[$147>>2] = $146;
  $0 = 0;
  $148 = $0;
  STACKTOP = sp;return ($148|0);
 }
 return 0|0;
}
function _getc_scan_header($oscn_header,$cbufptr,$ebufptr) {
 $oscn_header = $oscn_header|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $Ls = 0, $i = 0, $ret = 0, $scn_header = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer25 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr15 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer25 = sp + 72|0;
 $vararg_buffer22 = sp + 64|0;
 $vararg_buffer19 = sp + 56|0;
 $vararg_buffer16 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $Ls = sp + 104|0;
 $1 = $oscn_header;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = HEAP32[8>>2]|0;
 $5 = ($4|0)>(0);
 if ($5) {
  $6 = HEAP32[_stdout>>2]|0;
  (_fprintf(($6|0),(44144|0),($vararg_buffer|0))|0);
 }
 $7 = (_malloc(12)|0);
 $scn_header = $7;
 $8 = $scn_header;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(44320|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $139 = $0;
  STACKTOP = sp;return ($139|0);
 }
 $11 = $2;
 $12 = $3;
 $13 = (_getc_ushort($Ls,$11,$12)|0);
 $ret = $13;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $scn_header;
  _free($15);
  $16 = $ret;
  $0 = $16;
  $139 = $0;
  STACKTOP = sp;return ($139|0);
 }
 $17 = $scn_header;
 $18 = $2;
 $19 = $3;
 $20 = (_getc_byte($17,$18,$19)|0);
 $ret = $20;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = $scn_header;
  _free($22);
  $23 = $ret;
  $0 = $23;
  $139 = $0;
  STACKTOP = sp;return ($139|0);
 }
 $i = 0;
 while(1) {
  $24 = $i;
  $25 = $scn_header;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = ($24|0)<($27|0);
  if (!($28)) {
   label = 17;
   break;
  }
  $29 = $i;
  $30 = $scn_header;
  $31 = (($30) + 1|0);
  $32 = (($31) + ($29)|0);
  $33 = $2;
  $34 = $3;
  $35 = (_getc_byte($32,$33,$34)|0);
  $ret = $35;
  $36 = ($35|0)!=(0);
  if ($36) {
   label = 12;
   break;
  }
  $39 = $i;
  $40 = $scn_header;
  $41 = (($40) + 5|0);
  $42 = (($41) + ($39)|0);
  $43 = $2;
  $44 = $3;
  $45 = (_getc_byte($42,$43,$44)|0);
  $ret = $45;
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 14;
   break;
  }
  $49 = $i;
  $50 = $scn_header;
  $51 = (($50) + 5|0);
  $52 = (($51) + ($49)|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $54 >> 4;
  $56 = $55&255;
  $57 = $i;
  $58 = $scn_header;
  $59 = (($58) + 5|0);
  $60 = (($59) + ($57)|0);
  HEAP8[$60>>0] = $56;
  $61 = $i;
  $62 = (($61) + 1)|0;
  $i = $62;
 }
 if ((label|0) == 12) {
  $37 = $scn_header;
  _free($37);
  $38 = $ret;
  $0 = $38;
  $139 = $0;
  STACKTOP = sp;return ($139|0);
 }
 else if ((label|0) == 14) {
  $47 = $scn_header;
  _free($47);
  $48 = $ret;
  $0 = $48;
  $139 = $0;
  STACKTOP = sp;return ($139|0);
 }
 else if ((label|0) == 17) {
  $63 = $scn_header;
  $64 = (($63) + 9|0);
  $65 = $2;
  $66 = $3;
  $67 = (_getc_byte($64,$65,$66)|0);
  $ret = $67;
  $68 = ($67|0)!=(0);
  if ($68) {
   $69 = $scn_header;
   _free($69);
   $70 = $ret;
   $0 = $70;
   $139 = $0;
   STACKTOP = sp;return ($139|0);
  }
  $71 = $scn_header;
  $72 = (($71) + 10|0);
  $73 = $2;
  $74 = $3;
  $75 = (_getc_byte($72,$73,$74)|0);
  $ret = $75;
  $76 = ($75|0)!=(0);
  if ($76) {
   $77 = $scn_header;
   _free($77);
   $78 = $ret;
   $0 = $78;
   $139 = $0;
   STACKTOP = sp;return ($139|0);
  }
  $79 = $scn_header;
  $80 = (($79) + 11|0);
  $81 = $2;
  $82 = $3;
  $83 = (_getc_byte($80,$81,$82)|0);
  $ret = $83;
  $84 = ($83|0)!=(0);
  if ($84) {
   $85 = $scn_header;
   _free($85);
   $86 = $ret;
   $0 = $86;
   $139 = $0;
   STACKTOP = sp;return ($139|0);
  }
  $87 = HEAP32[8>>2]|0;
  $88 = ($87|0)>(1);
  if ($88) {
   $89 = HEAP32[_stdout>>2]|0;
   $90 = HEAP16[$Ls>>1]|0;
   $91 = $90&65535;
   HEAP32[$vararg_buffer3>>2] = $91;
   (_fprintf(($89|0),(44176|0),($vararg_buffer3|0))|0);
   $92 = HEAP32[_stdout>>2]|0;
   $93 = $scn_header;
   $94 = HEAP8[$93>>0]|0;
   $95 = $94&255;
   HEAP32[$vararg_buffer5>>2] = $95;
   (_fprintf(($92|0),(44192|0),($vararg_buffer5|0))|0);
   $i = 0;
   while(1) {
    $96 = $i;
    $97 = $scn_header;
    $98 = HEAP8[$97>>0]|0;
    $99 = $98&255;
    $100 = ($96|0)<($99|0);
    if (!($100)) {
     break;
    }
    $101 = HEAP32[_stdout>>2]|0;
    $102 = $i;
    $103 = $i;
    $104 = $scn_header;
    $105 = (($104) + 1|0);
    $106 = (($105) + ($103)|0);
    $107 = HEAP8[$106>>0]|0;
    $108 = $107&255;
    HEAP32[$vararg_buffer8>>2] = $102;
    $vararg_ptr11 = (($vararg_buffer8) + 4|0);
    HEAP32[$vararg_ptr11>>2] = $108;
    (_fprintf(($101|0),(44208|0),($vararg_buffer8|0))|0);
    $109 = HEAP32[_stdout>>2]|0;
    $110 = $i;
    $111 = $i;
    $112 = $scn_header;
    $113 = (($112) + 5|0);
    $114 = (($113) + ($111)|0);
    $115 = HEAP8[$114>>0]|0;
    $116 = $115&255;
    HEAP32[$vararg_buffer12>>2] = $110;
    $vararg_ptr15 = (($vararg_buffer12) + 4|0);
    HEAP32[$vararg_ptr15>>2] = $116;
    (_fprintf(($109|0),(44224|0),($vararg_buffer12|0))|0);
    $117 = $i;
    $118 = (($117) + 1)|0;
    $i = $118;
   }
   $119 = HEAP32[_stdout>>2]|0;
   $120 = $scn_header;
   $121 = (($120) + 9|0);
   $122 = HEAP8[$121>>0]|0;
   $123 = $122&255;
   HEAP32[$vararg_buffer16>>2] = $123;
   (_fprintf(($119|0),(44240|0),($vararg_buffer16|0))|0);
   $124 = HEAP32[_stdout>>2]|0;
   $125 = $scn_header;
   $126 = (($125) + 10|0);
   $127 = HEAP8[$126>>0]|0;
   $128 = $127&255;
   HEAP32[$vararg_buffer19>>2] = $128;
   (_fprintf(($124|0),(44256|0),($vararg_buffer19|0))|0);
   $129 = HEAP32[_stdout>>2]|0;
   $130 = $scn_header;
   $131 = (($130) + 11|0);
   $132 = HEAP8[$131>>0]|0;
   $133 = $132&255;
   HEAP32[$vararg_buffer22>>2] = $133;
   (_fprintf(($129|0),(44272|0),($vararg_buffer22|0))|0);
  }
  $134 = HEAP32[8>>2]|0;
  $135 = ($134|0)>(0);
  if ($135) {
   $136 = HEAP32[_stdout>>2]|0;
   (_fprintf(($136|0),(44288|0),($vararg_buffer25|0))|0);
  }
  $137 = $scn_header;
  $138 = $1;
  HEAP32[$138>>2] = $137;
  $0 = 0;
  $139 = $0;
  STACKTOP = sp;return ($139|0);
 }
 return 0|0;
}
function _predict($odata_pred,$indata,$width,$pixel_num,$cmpnt_depth,$pred_type,$Pt) {
 $odata_pred = $odata_pred|0;
 $indata = $indata|0;
 $width = $width|0;
 $pixel_num = $pixel_num|0;
 $cmpnt_depth = $cmpnt_depth|0;
 $pred_type = $pred_type|0;
 $Pt = $Pt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data_pred = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $odata_pred;
 $2 = $indata;
 $3 = $width;
 $4 = $pixel_num;
 $5 = $cmpnt_depth;
 $6 = $pred_type;
 $7 = $Pt;
 $data_pred = 0;
 $8 = $4;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = $5;
  $11 = $7;
  $12 = (($10) - ($11))|0;
  $13 = (($12) - 1)|0;
  $14 = 1 << $13;
  $15 = $14&65535;
  $data_pred = $15;
  $16 = $data_pred;
  $17 = $1;
  HEAP16[$17>>1] = $16;
  $0 = 0;
  $143 = $0;
  STACKTOP = sp;return ($143|0);
 }
 $18 = $4;
 $19 = ($18|0)>(0);
 if ($19) {
  $20 = $4;
  $21 = $3;
  $22 = ($20|0)<($21|0);
  if ($22) {
   $23 = $2;
   $24 = (($23) + -1|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $data_pred = $26;
   $27 = $data_pred;
   $28 = $1;
   HEAP16[$28>>1] = $27;
   $0 = 0;
   $143 = $0;
   STACKTOP = sp;return ($143|0);
  }
 }
 $29 = $4;
 $30 = $3;
 $31 = (($30) - 1)|0;
 $32 = ($29|0)>($31|0);
 if ($32) {
  $33 = $4;
  $34 = $3;
  $35 = (($33|0) % ($34|0))&-1;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = $2;
   $38 = $3;
   $39 = (0 - ($38))|0;
   $40 = (($37) + ($39)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $data_pred = $42;
  } else {
   $43 = $6;
   switch ($43|0) {
   case 7:  {
    $125 = $2;
    $126 = (($125) + -1|0);
    $127 = HEAP8[$126>>0]|0;
    $128 = $127&255;
    $129 = $2;
    $130 = $3;
    $131 = (0 - ($130))|0;
    $132 = (($129) + ($131)|0);
    $133 = HEAP8[$132>>0]|0;
    $134 = $133&255;
    $135 = (($128) + ($134))|0;
    $136 = (($135|0) / 2)&-1;
    $137 = $136&65535;
    $data_pred = $137;
    break;
   }
   case 6:  {
    $103 = $2;
    $104 = $3;
    $105 = (0 - ($104))|0;
    $106 = (($103) + ($105)|0);
    $107 = HEAP8[$106>>0]|0;
    $108 = $107&255;
    $109 = $2;
    $110 = (($109) + -1|0);
    $111 = HEAP8[$110>>0]|0;
    $112 = $111&255;
    $113 = $112 >> 1;
    $114 = $2;
    $115 = $3;
    $116 = (($115) + 1)|0;
    $117 = (0 - ($116))|0;
    $118 = (($114) + ($117)|0);
    $119 = HEAP8[$118>>0]|0;
    $120 = $119&255;
    $121 = $120 >> 1;
    $122 = (($113) - ($121))|0;
    $123 = (($108) + ($122))|0;
    $124 = $123&65535;
    $data_pred = $124;
    break;
   }
   case 4:  {
    $61 = $2;
    $62 = (($61) + -1|0);
    $63 = HEAP8[$62>>0]|0;
    $64 = $63&255;
    $65 = $2;
    $66 = $3;
    $67 = (0 - ($66))|0;
    $68 = (($65) + ($67)|0);
    $69 = HEAP8[$68>>0]|0;
    $70 = $69&255;
    $71 = (($64) + ($70))|0;
    $72 = $2;
    $73 = $3;
    $74 = (($73) + 1)|0;
    $75 = (0 - ($74))|0;
    $76 = (($72) + ($75)|0);
    $77 = HEAP8[$76>>0]|0;
    $78 = $77&255;
    $79 = (($71) - ($78))|0;
    $80 = $79&65535;
    $data_pred = $80;
    break;
   }
   case 3:  {
    $54 = $2;
    $55 = $3;
    $56 = (($55) + 1)|0;
    $57 = (0 - ($56))|0;
    $58 = (($54) + ($57)|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $data_pred = $60;
    break;
   }
   case 5:  {
    $81 = $2;
    $82 = (($81) + -1|0);
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = $2;
    $86 = $3;
    $87 = (0 - ($86))|0;
    $88 = (($85) + ($87)|0);
    $89 = HEAP8[$88>>0]|0;
    $90 = $89&255;
    $91 = $90 >> 1;
    $92 = $2;
    $93 = $3;
    $94 = (($93) + 1)|0;
    $95 = (0 - ($94))|0;
    $96 = (($92) + ($95)|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $97&255;
    $99 = $98 >> 1;
    $100 = (($91) - ($99))|0;
    $101 = (($84) + ($100))|0;
    $102 = $101&65535;
    $data_pred = $102;
    break;
   }
   case 2:  {
    $48 = $2;
    $49 = $3;
    $50 = (0 - ($49))|0;
    $51 = (($48) + ($50)|0);
    $52 = HEAP8[$51>>0]|0;
    $53 = $52&255;
    $data_pred = $53;
    break;
   }
   case 1:  {
    $44 = $2;
    $45 = (($44) + -1|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $data_pred = $47;
    break;
   }
   default: {
    $138 = HEAP32[_stderr>>2]|0;
    (_fprintf(($138|0),(44456|0),($vararg_buffer|0))|0);
    $139 = HEAP32[_stderr>>2]|0;
    $140 = $6;
    HEAP32[$vararg_buffer1>>2] = $140;
    $vararg_ptr3 = (($vararg_buffer1) + 4|0);
    HEAP32[$vararg_ptr3>>2] = 1;
    $vararg_ptr4 = (($vararg_buffer1) + 8|0);
    HEAP32[$vararg_ptr4>>2] = 7;
    (_fprintf(($139|0),(44504|0),($vararg_buffer1|0))|0);
    $0 = -2;
    $143 = $0;
    STACKTOP = sp;return ($143|0);
   }
   }
  }
 }
 $141 = $data_pred;
 $142 = $1;
 HEAP16[$142>>1] = $141;
 $0 = 0;
 $143 = $0;
 STACKTOP = sp;return ($143|0);
}
function _categorize($idiff) {
 $idiff = $idiff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bit = 0, $diff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $idiff;
 $2 = $1;
 $diff = $2;
 $3 = $diff;
 $4 = $3 << 16 >> 16;
 $5 = ($4|0)==(0);
 if ($5) {
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 $6 = $diff;
 $7 = $6 << 16 >> 16;
 $8 = ($7|0)<(0);
 if ($8) {
  $9 = $diff;
  $10 = $9 << 16 >> 16;
  $11 = Math_imul($10, -1)|0;
  $12 = $11&65535;
  $diff = $12;
 }
 $bit = 0;
 while(1) {
  $13 = $bit;
  $14 = ($13|0)<(16);
  if (!($14)) {
   label = 11;
   break;
  }
  $15 = $diff;
  $16 = $15 << 16 >> 16;
  $17 = $16 & 32768;
  $18 = ($17|0)!=(0);
  if ($18) {
   label = 8;
   break;
  }
  $22 = $diff;
  $23 = $22 << 16 >> 16;
  $24 = $23 << 1;
  $25 = $24&65535;
  $diff = $25;
  $26 = $bit;
  $27 = (($26) + 1)|0;
  $bit = $27;
 }
 if ((label|0) == 8) {
  $19 = $bit;
  $20 = (16 - ($19))|0;
  $21 = $20&65535;
  $0 = $21;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 11) {
  $0 = -1;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _jpegb_decode_mem($oout_buffer,$ow,$oh,$od,$oppi,$lossy_flag,$in_buffer,$in_buffer_size) {
 $oout_buffer = $oout_buffer|0;
 $ow = $ow|0;
 $oh = $oh|0;
 $od = $od|0;
 $oppi = $oppi|0;
 $lossy_flag = $lossy_flag|0;
 $in_buffer = $in_buffer|0;
 $in_buffer_size = $in_buffer_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $bptr = 0, $cinfo = 0, $jerr = 0, $out_buffer = 0, $ret = 0, $row_stride = 0, $tbuffer = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 656|0;
 $vararg_buffer = sp + 464|0;
 $cinfo = sp;
 $jerr = sp + 488|0;
 $1 = $oout_buffer;
 $2 = $ow;
 $3 = $oh;
 $4 = $od;
 $5 = $oppi;
 $6 = $lossy_flag;
 $7 = $in_buffer;
 $8 = $in_buffer_size;
 $9 = (_jpeg_std_error($jerr)|0);
 HEAP32[$cinfo>>2] = $9;
 _jpeg_CreateDecompress($cinfo,62,464);
 $10 = $7;
 $11 = $8;
 _jpeg_membuf_src($cinfo,$10,$11);
 (_jpeg_read_header($cinfo,1)|0);
 (_jpeg_start_decompress($cinfo)|0);
 $12 = (($cinfo) + 112|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($cinfo) + 116|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = Math_imul($13, $15)|0;
 $17 = (($cinfo) + 124|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = Math_imul($16, $18)|0;
 $20 = $19;
 $21 = (_malloc($20)|0);
 $out_buffer = $21;
 $22 = $out_buffer;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[_stderr>>2]|0;
  (_fprintf(($24|0),(44536|0),($vararg_buffer|0))|0);
  $0 = -2;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $25 = (($cinfo) + 112|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($cinfo) + 124|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = Math_imul($26, $28)|0;
 $row_stride = $29;
 $30 = (($cinfo) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $row_stride;
 $35 = (FUNCTION_TABLE_iiiii[$33 & 255]($cinfo,1,$34,1)|0);
 $tbuffer = $35;
 $36 = $out_buffer;
 $bptr = $36;
 while(1) {
  $37 = (($cinfo) + 140|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($cinfo) + 116|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($38>>>0)<($40>>>0);
  if (!($41)) {
   break;
  }
  $42 = $tbuffer;
  (_jpeg_read_scanlines($cinfo,$42,1)|0);
  $43 = $bptr;
  $44 = $tbuffer;
  $45 = HEAP32[$44>>2]|0;
  $46 = $row_stride;
  _memcpy(($43|0),($45|0),($46|0))|0;
  $47 = $row_stride;
  $48 = $bptr;
  $49 = (($48) + ($47)|0);
  $bptr = $49;
 }
 $50 = $out_buffer;
 $51 = $1;
 HEAP32[$51>>2] = $50;
 $52 = (($cinfo) + 112|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = $2;
 HEAP32[$54>>2] = $53;
 $55 = (($cinfo) + 116|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = $3;
 HEAP32[$57>>2] = $56;
 $58 = (($cinfo) + 124|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 << 3;
 $61 = $4;
 HEAP32[$61>>2] = $60;
 $62 = $5;
 $63 = (_get_ppi_jpegb($62,$cinfo)|0);
 $ret = $63;
 $64 = ($63|0)!=(0);
 if ($64) {
  $65 = $out_buffer;
  _free($65);
  $66 = $ret;
  $0 = $66;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 } else {
  (_jpeg_finish_decompress($cinfo)|0);
  _jpeg_destroy_decompress($cinfo);
  $67 = $6;
  HEAP32[$67>>2] = 1;
  $0 = 0;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 return 0|0;
}
function _jpeg_CreateDecompress($cinfo,$version,$structsize) {
 $cinfo = $cinfo|0;
 $version = $version|0;
 $structsize = $structsize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $client_data = 0, $err = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $version;
 $2 = $structsize;
 $3 = $0;
 $4 = (($3) + 4|0);
 HEAP32[$4>>2] = 0;
 $5 = $1;
 $6 = ($5|0)!=(62);
 if ($6) {
  $7 = $0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 20|0);
  HEAP32[$9>>2] = 12;
  $10 = $0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 24|0);
  HEAP32[$12>>2] = 62;
  $13 = $1;
  $14 = $0;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 24|0);
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = $13;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 255]($21);
 }
 $22 = $2;
 $23 = ($22|0)!=(464);
 if ($23) {
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 20|0);
  HEAP32[$26>>2] = 21;
  $27 = $0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 24|0);
  HEAP32[$29>>2] = 464;
  $30 = $2;
  $31 = $0;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 24|0);
  $34 = (($33) + 4|0);
  HEAP32[$34>>2] = $30;
  $35 = $0;
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = $0;
  FUNCTION_TABLE_vi[$37 & 255]($38);
 }
 $39 = $0;
 $40 = HEAP32[$39>>2]|0;
 $err = $40;
 $41 = $0;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $client_data = $43;
 $44 = $0;
 _memset(($44|0),0,464)|0;
 $45 = $err;
 $46 = $0;
 HEAP32[$46>>2] = $45;
 $47 = $client_data;
 $48 = $0;
 $49 = (($48) + 12|0);
 HEAP32[$49>>2] = $47;
 $50 = $0;
 $51 = (($50) + 16|0);
 HEAP32[$51>>2] = 1;
 $52 = $0;
 _jinit_memory_mgr($52);
 $53 = $0;
 $54 = (($53) + 8|0);
 HEAP32[$54>>2] = 0;
 $55 = $0;
 $56 = (($55) + 24|0);
 HEAP32[$56>>2] = 0;
 $i = 0;
 while(1) {
  $57 = $i;
  $58 = ($57|0)<(4);
  if (!($58)) {
   break;
  }
  $59 = $i;
  $60 = $0;
  $61 = (($60) + 164|0);
  $62 = (($61) + ($59<<2)|0);
  HEAP32[$62>>2] = 0;
  $63 = $i;
  $64 = (($63) + 1)|0;
  $i = $64;
 }
 $i = 0;
 while(1) {
  $65 = $i;
  $66 = ($65|0)<(4);
  if (!($66)) {
   break;
  }
  $67 = $i;
  $68 = $0;
  $69 = (($68) + 180|0);
  $70 = (($69) + ($67<<2)|0);
  HEAP32[$70>>2] = 0;
  $71 = $i;
  $72 = $0;
  $73 = (($72) + 196|0);
  $74 = (($73) + ($71<<2)|0);
  HEAP32[$74>>2] = 0;
  $75 = $i;
  $76 = (($75) + 1)|0;
  $i = $76;
 }
 $77 = $0;
 $78 = (($77) + 304|0);
 HEAP32[$78>>2] = 0;
 $79 = $0;
 _jinit_marker_reader($79);
 $80 = $0;
 _jinit_input_controller($80);
 $81 = $0;
 $82 = (($81) + 20|0);
 HEAP32[$82>>2] = 200;
 STACKTOP = sp;return;
}
function _jpeg_destroy_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 _jpeg_destroy($1);
 STACKTOP = sp;return;
}
function _jpeg_read_header($cinfo,$require_image) {
 $cinfo = $cinfo|0;
 $require_image = $require_image|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $retcode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $require_image;
 $2 = $0;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(200);
 if ($5) {
  $6 = $0;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(201);
  if ($9) {
   $10 = $0;
   $11 = HEAP32[$10>>2]|0;
   $12 = (($11) + 20|0);
   HEAP32[$12>>2] = 20;
   $13 = $0;
   $14 = (($13) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + 24|0);
   HEAP32[$18>>2] = $15;
   $19 = $0;
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = $0;
   FUNCTION_TABLE_vi[$21 & 255]($22);
  }
 }
 $23 = $0;
 $24 = (_jpeg_consume_input($23)|0);
 $retcode = $24;
 $25 = $retcode;
 if ((($25|0) == 2)) {
  $26 = $1;
  $27 = ($26|0)!=(0);
  if ($27) {
   $28 = $0;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 20|0);
   HEAP32[$30>>2] = 51;
   $31 = $0;
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = $0;
   FUNCTION_TABLE_vi[$33 & 255]($34);
  }
  $35 = $0;
  _jpeg_abort($35);
  $retcode = 2;
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 } else if ((($25|0) == 0)) {
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 } else if ((($25|0) == 1)) {
  $retcode = 1;
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 } else {
  $36 = $retcode;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _jpeg_consume_input($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $retcode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $retcode = 0;
 $1 = $0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 switch ($3|0) {
 case 210: case 208: case 207: case 206: case 205: case 204: case 203:  {
  $29 = $0;
  $30 = (($29) + 436|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$31>>2]|0;
  $33 = $0;
  $34 = (FUNCTION_TABLE_ii[$32 & 127]($33)|0);
  $retcode = $34;
  $48 = $retcode;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 201:  {
  break;
 }
 case 202:  {
  $retcode = 1;
  $48 = $retcode;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 200:  {
  $4 = $0;
  $5 = (($4) + 436|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $0;
  FUNCTION_TABLE_vi[$8 & 255]($9);
  $10 = $0;
  $11 = (($10) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $0;
  FUNCTION_TABLE_vi[$14 & 255]($15);
  $16 = $0;
  $17 = (($16) + 20|0);
  HEAP32[$17>>2] = 201;
  break;
 }
 default: {
  $35 = $0;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + 20|0);
  HEAP32[$37>>2] = 20;
  $38 = $0;
  $39 = (($38) + 20|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 24|0);
  HEAP32[$43>>2] = $40;
  $44 = $0;
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $0;
  FUNCTION_TABLE_vi[$46 & 255]($47);
  $48 = $retcode;
  STACKTOP = sp;return ($48|0);
 }
 }
 $18 = $0;
 $19 = (($18) + 436|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = $0;
 $23 = (FUNCTION_TABLE_ii[$21 & 127]($22)|0);
 $retcode = $23;
 $24 = $retcode;
 $25 = ($24|0)==(1);
 if ($25) {
  $26 = $0;
  _default_decompress_parms($26);
  $27 = $0;
  $28 = (($27) + 20|0);
  HEAP32[$28>>2] = 202;
 }
 $48 = $retcode;
 STACKTOP = sp;return ($48|0);
}
function _default_decompress_parms($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $cid0 = 0, $cid1 = 0, $cid2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 36|0);
 $3 = HEAP32[$2>>2]|0;
 if ((($3|0) == 4)) {
  $96 = $0;
  $97 = (($96) + 292|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = ($98|0)!=(0);
  if ($99) {
   $100 = $0;
   $101 = (($100) + 296|0);
   $102 = HEAP8[$101>>0]|0;
   $103 = $102&255;
   if ((($103|0) == 2)) {
    $106 = $0;
    $107 = (($106) + 40|0);
    HEAP32[$107>>2] = 5;
   } else if ((($103|0) == 0)) {
    $104 = $0;
    $105 = (($104) + 40|0);
    HEAP32[$105>>2] = 4;
   } else {
    $108 = $0;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 20|0);
    HEAP32[$110>>2] = 114;
    $111 = $0;
    $112 = (($111) + 296|0);
    $113 = HEAP8[$112>>0]|0;
    $114 = $113&255;
    $115 = $0;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($116) + 24|0);
    HEAP32[$117>>2] = $114;
    $118 = $0;
    $119 = HEAP32[$118>>2]|0;
    $120 = (($119) + 4|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = $0;
    FUNCTION_TABLE_vii[$121 & 255]($122,-1);
    $123 = $0;
    $124 = (($123) + 40|0);
    HEAP32[$124>>2] = 5;
   }
  } else {
   $125 = $0;
   $126 = (($125) + 40|0);
   HEAP32[$126>>2] = 4;
  }
  $127 = $0;
  $128 = (($127) + 44|0);
  HEAP32[$128>>2] = 4;
 } else if ((($3|0) == 1)) {
  $4 = $0;
  $5 = (($4) + 40|0);
  HEAP32[$5>>2] = 1;
  $6 = $0;
  $7 = (($6) + 44|0);
  HEAP32[$7>>2] = 1;
 } else if ((($3|0) == 3)) {
  $8 = $0;
  $9 = (($8) + 280|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0);
  if ($11) {
   $12 = $0;
   $13 = (($12) + 40|0);
   HEAP32[$13>>2] = 3;
  } else {
   $14 = $0;
   $15 = (($14) + 292|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $0;
    $19 = (($18) + 296|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    if ((($21|0) == 1)) {
     $24 = $0;
     $25 = (($24) + 40|0);
     HEAP32[$25>>2] = 3;
    } else if ((($21|0) == 0)) {
     $22 = $0;
     $23 = (($22) + 40|0);
     HEAP32[$23>>2] = 2;
    } else {
     $26 = $0;
     $27 = HEAP32[$26>>2]|0;
     $28 = (($27) + 20|0);
     HEAP32[$28>>2] = 114;
     $29 = $0;
     $30 = (($29) + 296|0);
     $31 = HEAP8[$30>>0]|0;
     $32 = $31&255;
     $33 = $0;
     $34 = HEAP32[$33>>2]|0;
     $35 = (($34) + 24|0);
     HEAP32[$35>>2] = $32;
     $36 = $0;
     $37 = HEAP32[$36>>2]|0;
     $38 = (($37) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = $0;
     FUNCTION_TABLE_vii[$39 & 255]($40,-1);
     $41 = $0;
     $42 = (($41) + 40|0);
     HEAP32[$42>>2] = 3;
    }
   } else {
    $43 = $0;
    $44 = (($43) + 216|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$45>>2]|0;
    $cid0 = $46;
    $47 = $0;
    $48 = (($47) + 216|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 84|0);
    $51 = HEAP32[$50>>2]|0;
    $cid1 = $51;
    $52 = $0;
    $53 = (($52) + 216|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + 168|0);
    $56 = HEAP32[$55>>2]|0;
    $cid2 = $56;
    $57 = $cid0;
    $58 = ($57|0)==(1);
    if ($58) {
     $59 = $cid1;
     $60 = ($59|0)==(2);
     if ($60) {
      $61 = $cid2;
      $62 = ($61|0)==(3);
      if ($62) {
       $63 = $0;
       $64 = (($63) + 40|0);
       HEAP32[$64>>2] = 3;
      } else {
       label = 15;
      }
     } else {
      label = 15;
     }
    } else {
     label = 15;
    }
    if ((label|0) == 15) {
     $65 = $cid0;
     $66 = ($65|0)==(82);
     if ($66) {
      $67 = $cid1;
      $68 = ($67|0)==(71);
      if ($68) {
       $69 = $cid2;
       $70 = ($69|0)==(66);
       if ($70) {
        $71 = $0;
        $72 = (($71) + 40|0);
        HEAP32[$72>>2] = 2;
       } else {
        label = 19;
       }
      } else {
       label = 19;
      }
     } else {
      label = 19;
     }
     if ((label|0) == 19) {
      $73 = $0;
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + 24|0);
      $_mp = $75;
      $76 = $cid0;
      $77 = $_mp;
      HEAP32[$77>>2] = $76;
      $78 = $cid1;
      $79 = $_mp;
      $80 = (($79) + 4|0);
      HEAP32[$80>>2] = $78;
      $81 = $cid2;
      $82 = $_mp;
      $83 = (($82) + 8|0);
      HEAP32[$83>>2] = $81;
      $84 = $0;
      $85 = HEAP32[$84>>2]|0;
      $86 = (($85) + 20|0);
      HEAP32[$86>>2] = 111;
      $87 = $0;
      $88 = HEAP32[$87>>2]|0;
      $89 = (($88) + 4|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = $0;
      FUNCTION_TABLE_vii[$90 & 255]($91,1);
      $92 = $0;
      $93 = (($92) + 40|0);
      HEAP32[$93>>2] = 3;
     }
    }
   }
  }
  $94 = $0;
  $95 = (($94) + 44|0);
  HEAP32[$95>>2] = 2;
 } else {
  $129 = $0;
  $130 = (($129) + 40|0);
  HEAP32[$130>>2] = 0;
  $131 = $0;
  $132 = (($131) + 44|0);
  HEAP32[$132>>2] = 0;
 }
 $133 = $0;
 $134 = (($133) + 48|0);
 HEAP32[$134>>2] = 1;
 $135 = $0;
 $136 = (($135) + 52|0);
 HEAP32[$136>>2] = 1;
 $137 = $0;
 $138 = (($137) + 56|0);
 HEAPF64[$138>>3] = 1.0;
 $139 = $0;
 $140 = (($139) + 64|0);
 HEAP32[$140>>2] = 0;
 $141 = $0;
 $142 = (($141) + 68|0);
 HEAP32[$142>>2] = 0;
 $143 = $0;
 $144 = (($143) + 72|0);
 HEAP32[$144>>2] = 0;
 $145 = $0;
 $146 = (($145) + 76|0);
 HEAP32[$146>>2] = 1;
 $147 = $0;
 $148 = (($147) + 80|0);
 HEAP32[$148>>2] = 1;
 $149 = $0;
 $150 = (($149) + 84|0);
 HEAP32[$150>>2] = 0;
 $151 = $0;
 $152 = (($151) + 88|0);
 HEAP32[$152>>2] = 2;
 $153 = $0;
 $154 = (($153) + 92|0);
 HEAP32[$154>>2] = 1;
 $155 = $0;
 $156 = (($155) + 96|0);
 HEAP32[$156>>2] = 256;
 $157 = $0;
 $158 = (($157) + 136|0);
 HEAP32[$158>>2] = 0;
 $159 = $0;
 $160 = (($159) + 100|0);
 HEAP32[$160>>2] = 0;
 $161 = $0;
 $162 = (($161) + 104|0);
 HEAP32[$162>>2] = 0;
 $163 = $0;
 $164 = (($163) + 108|0);
 HEAP32[$164>>2] = 0;
 STACKTOP = sp;return;
}
function _jpeg_finish_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(205);
 if ($5) {
  label = 3;
 } else {
  $6 = $1;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(206);
  if ($9) {
   label = 3;
  } else {
   label = 7;
  }
 }
 if ((label|0) == 3) {
  $10 = $1;
  $11 = (($10) + 64|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   label = 7;
  } else {
   $14 = $1;
   $15 = (($14) + 140|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $1;
   $18 = (($17) + 116|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16>>>0)<($19>>>0);
   if ($20) {
    $21 = $1;
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + 20|0);
    HEAP32[$23>>2] = 67;
    $24 = $1;
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$25>>2]|0;
    $27 = $1;
    FUNCTION_TABLE_vi[$26 & 255]($27);
   }
   $28 = $1;
   $29 = (($28) + 420|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 4|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = $1;
   FUNCTION_TABLE_vi[$32 & 255]($33);
   $34 = $1;
   $35 = (($34) + 20|0);
   HEAP32[$35>>2] = 210;
  }
 }
 if ((label|0) == 7) {
  $36 = $1;
  $37 = (($36) + 20|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)==(207);
  if ($39) {
   $40 = $1;
   $41 = (($40) + 20|0);
   HEAP32[$41>>2] = 210;
  } else {
   $42 = $1;
   $43 = (($42) + 20|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(210);
   if ($45) {
    $46 = $1;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + 20|0);
    HEAP32[$48>>2] = 20;
    $49 = $1;
    $50 = (($49) + 20|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = $1;
    $53 = HEAP32[$52>>2]|0;
    $54 = (($53) + 24|0);
    HEAP32[$54>>2] = $51;
    $55 = $1;
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$56>>2]|0;
    $58 = $1;
    FUNCTION_TABLE_vi[$57 & 255]($58);
   }
  }
 }
 while(1) {
  $59 = $1;
  $60 = (($59) + 436|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)!=(0);
  $65 = $64 ^ 1;
  if (!($65)) {
   label = 18;
   break;
  }
  $66 = $1;
  $67 = (($66) + 436|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = $1;
  $71 = (FUNCTION_TABLE_ii[$69 & 127]($70)|0);
  $72 = ($71|0)==(0);
  if ($72) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 16) {
  $0 = 0;
  $80 = $0;
  STACKTOP = sp;return ($80|0);
 }
 else if ((label|0) == 18) {
  $73 = $1;
  $74 = (($73) + 24|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 24|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = $1;
  FUNCTION_TABLE_vi[$77 & 255]($78);
  $79 = $1;
  _jpeg_abort($79);
  $0 = 1;
  $80 = $0;
  STACKTOP = sp;return ($80|0);
 }
 return 0|0;
}
function _jpeg_start_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $retcode = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(202);
 do {
  if ($5) {
   $6 = $1;
   _jinit_master_decompress($6);
   $7 = $1;
   $8 = (($7) + 64|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0);
   if (!($10)) {
    $13 = $1;
    $14 = (($13) + 20|0);
    HEAP32[$14>>2] = 203;
    break;
   }
   $11 = $1;
   $12 = (($11) + 20|0);
   HEAP32[$12>>2] = 207;
   $0 = 1;
   $97 = $0;
   STACKTOP = sp;return ($97|0);
  }
 } while(0);
 $15 = $1;
 $16 = (($15) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(203);
 if ($18) {
  $19 = $1;
  $20 = (($19) + 436|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0);
  do {
   if ($24) {
    while(1) {
     $25 = $1;
     $26 = (($25) + 8|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = ($27|0)!=(0|0);
     if ($28) {
      $29 = $1;
      $30 = (($29) + 8|0);
      $31 = HEAP32[$30>>2]|0;
      $32 = HEAP32[$31>>2]|0;
      $33 = $1;
      FUNCTION_TABLE_vi[$32 & 255]($33);
     }
     $34 = $1;
     $35 = (($34) + 436|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = HEAP32[$36>>2]|0;
     $38 = $1;
     $39 = (FUNCTION_TABLE_ii[$37 & 127]($38)|0);
     $retcode = $39;
     $40 = $retcode;
     $41 = ($40|0)==(0);
     if ($41) {
      label = 11;
      break;
     }
     $42 = $retcode;
     $43 = ($42|0)==(2);
     if ($43) {
      label = 13;
      break;
     }
     $44 = $1;
     $45 = (($44) + 8|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)!=(0|0);
     do {
      if ($47) {
       $48 = $retcode;
       $49 = ($48|0)==(3);
       if (!($49)) {
        $50 = $retcode;
        $51 = ($50|0)==(1);
        if (!($51)) {
         break;
        }
       }
       $52 = $1;
       $53 = (($52) + 8|0);
       $54 = HEAP32[$53>>2]|0;
       $55 = (($54) + 4|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = (($56) + 1)|0;
       HEAP32[$55>>2] = $57;
       $58 = $1;
       $59 = (($58) + 8|0);
       $60 = HEAP32[$59>>2]|0;
       $61 = (($60) + 8|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = ($57|0)>=($62|0);
       if ($63) {
        $64 = $1;
        $65 = (($64) + 320|0);
        $66 = HEAP32[$65>>2]|0;
        $67 = $1;
        $68 = (($67) + 8|0);
        $69 = HEAP32[$68>>2]|0;
        $70 = (($69) + 8|0);
        $71 = HEAP32[$70>>2]|0;
        $72 = (($71) + ($66))|0;
        HEAP32[$70>>2] = $72;
       }
      }
     } while(0);
    }
    if ((label|0) == 11) {
     $0 = 0;
     $97 = $0;
     STACKTOP = sp;return ($97|0);
    }
    else if ((label|0) == 13) {
     break;
    }
   }
  } while(0);
  $73 = $1;
  $74 = (($73) + 144|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = $1;
  $77 = (($76) + 152|0);
  HEAP32[$77>>2] = $75;
 } else {
  $78 = $1;
  $79 = (($78) + 20|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = ($80|0)!=(204);
  if ($81) {
   $82 = $1;
   $83 = HEAP32[$82>>2]|0;
   $84 = (($83) + 20|0);
   HEAP32[$84>>2] = 20;
   $85 = $1;
   $86 = (($85) + 20|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = $1;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 24|0);
   HEAP32[$90>>2] = $87;
   $91 = $1;
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$92>>2]|0;
   $94 = $1;
   FUNCTION_TABLE_vi[$93 & 255]($94);
  }
 }
 $95 = $1;
 $96 = (_output_pass_setup($95)|0);
 $0 = $96;
 $97 = $0;
 STACKTOP = sp;return ($97|0);
}
function _output_pass_setup($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $last_scanline = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(204);
 if ($5) {
  $6 = $1;
  $7 = (($6) + 420|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = $1;
  FUNCTION_TABLE_vi[$9 & 255]($10);
  $11 = $1;
  $12 = (($11) + 140|0);
  HEAP32[$12>>2] = 0;
  $13 = $1;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = 204;
 }
 L4: while(1) {
  $15 = $1;
  $16 = (($15) + 420|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0);
  if (!($20)) {
   label = 13;
   break;
  }
  while(1) {
   $21 = $1;
   $22 = (($21) + 140|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = $1;
   $25 = (($24) + 116|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($23>>>0)<($26>>>0);
   if (!($27)) {
    break;
   }
   $28 = $1;
   $29 = (($28) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = $1;
    $33 = (($32) + 140|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = $1;
    $36 = (($35) + 8|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + 4|0);
    HEAP32[$38>>2] = $34;
    $39 = $1;
    $40 = (($39) + 116|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $1;
    $43 = (($42) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + 8|0);
    HEAP32[$45>>2] = $41;
    $46 = $1;
    $47 = (($46) + 8|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    $50 = $1;
    FUNCTION_TABLE_vi[$49 & 255]($50);
   }
   $51 = $1;
   $52 = (($51) + 140|0);
   $53 = HEAP32[$52>>2]|0;
   $last_scanline = $53;
   $54 = $1;
   $55 = (($54) + 424|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 4|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = $1;
   $60 = $1;
   $61 = (($60) + 140|0);
   FUNCTION_TABLE_viiii[$58 & 255]($59,0,$61,0);
   $62 = $1;
   $63 = (($62) + 140|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $last_scanline;
   $66 = ($64|0)==($65|0);
   if ($66) {
    label = 10;
    break L4;
   }
  }
  $67 = $1;
  $68 = (($67) + 420|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + 4|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = $1;
  FUNCTION_TABLE_vi[$71 & 255]($72);
  $73 = $1;
  $74 = (($73) + 420|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP32[$75>>2]|0;
  $77 = $1;
  FUNCTION_TABLE_vi[$76 & 255]($77);
  $78 = $1;
  $79 = (($78) + 140|0);
  HEAP32[$79>>2] = 0;
 }
 if ((label|0) == 10) {
  $0 = 0;
  $87 = $0;
  STACKTOP = sp;return ($87|0);
 }
 else if ((label|0) == 13) {
  $80 = $1;
  $81 = (($80) + 68|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($82|0)!=(0);
  $84 = $83 ? 206 : 205;
  $85 = $1;
  $86 = (($85) + 20|0);
  HEAP32[$86>>2] = $84;
  $0 = 1;
  $87 = $0;
  STACKTOP = sp;return ($87|0);
 }
 return 0|0;
}
function _jpeg_read_scanlines($cinfo,$scanlines,$max_lines) {
 $cinfo = $cinfo|0;
 $scanlines = $scanlines|0;
 $max_lines = $max_lines|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $row_ctr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $row_ctr = sp;
 $1 = $cinfo;
 $2 = $scanlines;
 $3 = $max_lines;
 $4 = $1;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(205);
 if ($7) {
  $8 = $1;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 20|0);
  HEAP32[$10>>2] = 20;
  $11 = $1;
  $12 = (($11) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $1;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 24|0);
  HEAP32[$16>>2] = $13;
  $17 = $1;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  FUNCTION_TABLE_vi[$19 & 255]($20);
 }
 $21 = $1;
 $22 = (($21) + 140|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = (($24) + 116|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($23>>>0)>=($26>>>0);
 if ($27) {
  $28 = $1;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 20|0);
  HEAP32[$30>>2] = 123;
  $31 = $1;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $1;
  FUNCTION_TABLE_vii[$34 & 255]($35,-1);
  $0 = 0;
  $73 = $0;
  STACKTOP = sp;return ($73|0);
 }
 $36 = $1;
 $37 = (($36) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = $1;
  $41 = (($40) + 140|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $1;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 4|0);
  HEAP32[$46>>2] = $42;
  $47 = $1;
  $48 = (($47) + 116|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = $1;
  $51 = (($50) + 8|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 8|0);
  HEAP32[$53>>2] = $49;
  $54 = $1;
  $55 = (($54) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = $1;
  FUNCTION_TABLE_vi[$57 & 255]($58);
 }
 HEAP32[$row_ctr>>2] = 0;
 $59 = $1;
 $60 = (($59) + 424|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (($61) + 4|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = $1;
 $65 = $2;
 $66 = $3;
 FUNCTION_TABLE_viiii[$63 & 255]($64,$65,$row_ctr,$66);
 $67 = HEAP32[$row_ctr>>2]|0;
 $68 = $1;
 $69 = (($68) + 140|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + ($67))|0;
 HEAP32[$69>>2] = $71;
 $72 = HEAP32[$row_ctr>>2]|0;
 $0 = $72;
 $73 = $0;
 STACKTOP = sp;return ($73|0);
}
function _jinit_input_controller($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $inputctl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,0,28)|0);
 $inputctl = $6;
 $7 = $inputctl;
 $8 = $0;
 $9 = (($8) + 436|0);
 HEAP32[$9>>2] = $7;
 $10 = $inputctl;
 HEAP32[$10>>2] = 35;
 $11 = $inputctl;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 36;
 $13 = $inputctl;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 37;
 $15 = $inputctl;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = 38;
 $17 = $inputctl;
 $18 = (($17) + 16|0);
 HEAP32[$18>>2] = 0;
 $19 = $inputctl;
 $20 = (($19) + 20|0);
 HEAP32[$20>>2] = 0;
 $21 = $inputctl;
 $22 = (($21) + 24|0);
 HEAP32[$22>>2] = 1;
 STACKTOP = sp;return;
}
function _consume_markers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $inputctl = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 436|0);
 $4 = HEAP32[$3>>2]|0;
 $inputctl = $4;
 $5 = $inputctl;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $0 = 2;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $9 = $1;
 $10 = (($9) + 440|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $1;
 $15 = (FUNCTION_TABLE_ii[$13 & 127]($14)|0);
 $val = $15;
 $16 = $val;
 if ((($16|0) == 0)) {
 } else if ((($16|0) == 2)) {
  $36 = $inputctl;
  $37 = (($36) + 20|0);
  HEAP32[$37>>2] = 1;
  $38 = $inputctl;
  $39 = (($38) + 24|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = $1;
   $43 = (($42) + 440|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + 16|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = $1;
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 20|0);
    HEAP32[$50>>2] = 59;
    $51 = $1;
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    $54 = $1;
    FUNCTION_TABLE_vi[$53 & 255]($54);
   }
  } else {
   $55 = $1;
   $56 = (($55) + 152|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = $1;
   $59 = (($58) + 144|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ($57|0)>($60|0);
   if ($61) {
    $62 = $1;
    $63 = (($62) + 144|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = $1;
    $66 = (($65) + 152|0);
    HEAP32[$66>>2] = $64;
   }
  }
 } else if ((($16|0) == 1)) {
  $17 = $inputctl;
  $18 = (($17) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = $1;
   _initial_setup($21);
   $22 = $inputctl;
   $23 = (($22) + 24|0);
   HEAP32[$23>>2] = 0;
  } else {
   $24 = $inputctl;
   $25 = (($24) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $28 = $1;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($29) + 20|0);
    HEAP32[$30>>2] = 35;
    $31 = $1;
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$32>>2]|0;
    $34 = $1;
    FUNCTION_TABLE_vi[$33 & 255]($34);
   }
   $35 = $1;
   _start_input_pass($35);
  }
 }
 $67 = $val;
 $0 = $67;
 $68 = $0;
 STACKTOP = sp;return ($68|0);
}
function _reset_input_controller($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $inputctl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 436|0);
 $3 = HEAP32[$2>>2]|0;
 $inputctl = $3;
 $4 = $inputctl;
 HEAP32[$4>>2] = 35;
 $5 = $inputctl;
 $6 = (($5) + 16|0);
 HEAP32[$6>>2] = 0;
 $7 = $inputctl;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = 0;
 $9 = $inputctl;
 $10 = (($9) + 24|0);
 HEAP32[$10>>2] = 1;
 $11 = $0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $0;
 FUNCTION_TABLE_vi[$14 & 255]($15);
 $16 = $0;
 $17 = (($16) + 440|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = $0;
 FUNCTION_TABLE_vi[$19 & 255]($20);
 $21 = $0;
 $22 = (($21) + 160|0);
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _start_input_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 _per_scan_setup($1);
 $2 = $0;
 _latch_quant_tables($2);
 $3 = $0;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 FUNCTION_TABLE_vi[$6 & 255]($7);
 $8 = $0;
 $9 = (($8) + 428|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = $0;
 FUNCTION_TABLE_vi[$11 & 255]($12);
 $13 = $0;
 $14 = (($13) + 428|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $0;
 $19 = (($18) + 436|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$20>>2] = $17;
 STACKTOP = sp;return;
}
function _finish_input_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 436|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = 35;
 STACKTOP = sp;return;
}
function _per_scan_setup($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0, $mcublks = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 328|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(1);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 332|0);
  $7 = HEAP32[$6>>2]|0;
  $compptr = $7;
  $8 = $compptr;
  $9 = (($8) + 28|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = (($11) + 348|0);
  HEAP32[$12>>2] = $10;
  $13 = $compptr;
  $14 = (($13) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $0;
  $17 = (($16) + 352|0);
  HEAP32[$17>>2] = $15;
  $18 = $compptr;
  $19 = (($18) + 52|0);
  HEAP32[$19>>2] = 1;
  $20 = $compptr;
  $21 = (($20) + 56|0);
  HEAP32[$21>>2] = 1;
  $22 = $compptr;
  $23 = (($22) + 60|0);
  HEAP32[$23>>2] = 1;
  $24 = $compptr;
  $25 = (($24) + 36|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $compptr;
  $28 = (($27) + 64|0);
  HEAP32[$28>>2] = $26;
  $29 = $compptr;
  $30 = (($29) + 68|0);
  HEAP32[$30>>2] = 1;
  $31 = $compptr;
  $32 = (($31) + 32|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $compptr;
  $35 = (($34) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($33>>>0) % ($36>>>0))&-1;
  $tmp = $37;
  $38 = $tmp;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = $compptr;
   $41 = (($40) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $tmp = $42;
  }
  $43 = $tmp;
  $44 = $compptr;
  $45 = (($44) + 72|0);
  HEAP32[$45>>2] = $43;
  $46 = $0;
  $47 = (($46) + 356|0);
  HEAP32[$47>>2] = 1;
  $48 = $0;
  $49 = (($48) + 360|0);
  HEAP32[$49>>2] = 0;
  STACKTOP = sp;return;
 }
 $50 = $0;
 $51 = (($50) + 328|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)<=(0);
 if ($53) {
  label = 7;
 } else {
  $54 = $0;
  $55 = (($54) + 328|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)>(4);
  if ($57) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $58 = $0;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($59) + 20|0);
  HEAP32[$60>>2] = 26;
  $61 = $0;
  $62 = (($61) + 328|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $0;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 24|0);
  HEAP32[$66>>2] = $63;
  $67 = $0;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + 24|0);
  $70 = (($69) + 4|0);
  HEAP32[$70>>2] = 4;
  $71 = $0;
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  FUNCTION_TABLE_vi[$73 & 255]($74);
 }
 $75 = $0;
 $76 = (($75) + 28|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = $0;
 $79 = (($78) + 308|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $80<<3;
 $82 = (_jdiv_round_up($77,$81)|0);
 $83 = $0;
 $84 = (($83) + 348|0);
 HEAP32[$84>>2] = $82;
 $85 = $0;
 $86 = (($85) + 32|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $0;
 $89 = (($88) + 312|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = $90<<3;
 $92 = (_jdiv_round_up($87,$91)|0);
 $93 = $0;
 $94 = (($93) + 352|0);
 HEAP32[$94>>2] = $92;
 $95 = $0;
 $96 = (($95) + 356|0);
 HEAP32[$96>>2] = 0;
 $ci = 0;
 while(1) {
  $97 = $ci;
  $98 = $0;
  $99 = (($98) + 328|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($97|0)<($100|0);
  if (!($101)) {
   break;
  }
  $102 = $ci;
  $103 = $0;
  $104 = (($103) + 332|0);
  $105 = (($104) + ($102<<2)|0);
  $106 = HEAP32[$105>>2]|0;
  $compptr = $106;
  $107 = $compptr;
  $108 = (($107) + 8|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = $compptr;
  $111 = (($110) + 52|0);
  HEAP32[$111>>2] = $109;
  $112 = $compptr;
  $113 = (($112) + 12|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = $compptr;
  $116 = (($115) + 56|0);
  HEAP32[$116>>2] = $114;
  $117 = $compptr;
  $118 = (($117) + 52|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = $compptr;
  $121 = (($120) + 56|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = Math_imul($119, $122)|0;
  $124 = $compptr;
  $125 = (($124) + 60|0);
  HEAP32[$125>>2] = $123;
  $126 = $compptr;
  $127 = (($126) + 52|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = $compptr;
  $130 = (($129) + 36|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = Math_imul($128, $131)|0;
  $133 = $compptr;
  $134 = (($133) + 64|0);
  HEAP32[$134>>2] = $132;
  $135 = $compptr;
  $136 = (($135) + 28|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = $compptr;
  $139 = (($138) + 52|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = (($137>>>0) % ($140>>>0))&-1;
  $tmp = $141;
  $142 = $tmp;
  $143 = ($142|0)==(0);
  if ($143) {
   $144 = $compptr;
   $145 = (($144) + 52|0);
   $146 = HEAP32[$145>>2]|0;
   $tmp = $146;
  }
  $147 = $tmp;
  $148 = $compptr;
  $149 = (($148) + 68|0);
  HEAP32[$149>>2] = $147;
  $150 = $compptr;
  $151 = (($150) + 32|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = $compptr;
  $154 = (($153) + 56|0);
  $155 = HEAP32[$154>>2]|0;
  $156 = (($152>>>0) % ($155>>>0))&-1;
  $tmp = $156;
  $157 = $tmp;
  $158 = ($157|0)==(0);
  if ($158) {
   $159 = $compptr;
   $160 = (($159) + 56|0);
   $161 = HEAP32[$160>>2]|0;
   $tmp = $161;
  }
  $162 = $tmp;
  $163 = $compptr;
  $164 = (($163) + 72|0);
  HEAP32[$164>>2] = $162;
  $165 = $compptr;
  $166 = (($165) + 60|0);
  $167 = HEAP32[$166>>2]|0;
  $mcublks = $167;
  $168 = $0;
  $169 = (($168) + 356|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = $mcublks;
  $172 = (($170) + ($171))|0;
  $173 = ($172|0)>(10);
  if ($173) {
   $174 = $0;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($175) + 20|0);
   HEAP32[$176>>2] = 13;
   $177 = $0;
   $178 = HEAP32[$177>>2]|0;
   $179 = HEAP32[$178>>2]|0;
   $180 = $0;
   FUNCTION_TABLE_vi[$179 & 255]($180);
  }
  while(1) {
   $181 = $mcublks;
   $182 = (($181) + -1)|0;
   $mcublks = $182;
   $183 = ($181|0)>(0);
   if (!($183)) {
    break;
   }
   $184 = $ci;
   $185 = $0;
   $186 = (($185) + 356|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = (($187) + 1)|0;
   HEAP32[$186>>2] = $188;
   $189 = $0;
   $190 = (($189) + 360|0);
   $191 = (($190) + ($187<<2)|0);
   HEAP32[$191>>2] = $184;
  }
  $192 = $ci;
  $193 = (($192) + 1)|0;
  $ci = $193;
 }
 STACKTOP = sp;return;
}
function _latch_quant_tables($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $compptr = 0, $qtbl = 0, $qtblno = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $ci = 0;
 while(1) {
  $1 = $ci;
  $2 = $0;
  $3 = (($2) + 328|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $ci;
  $7 = $0;
  $8 = (($7) + 332|0);
  $9 = (($8) + ($6<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $compptr = $10;
  $11 = $compptr;
  $12 = (($11) + 76|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
  } else {
   $15 = $compptr;
   $16 = (($15) + 16|0);
   $17 = HEAP32[$16>>2]|0;
   $qtblno = $17;
   $18 = $qtblno;
   $19 = ($18|0)<(0);
   if ($19) {
    label = 8;
   } else {
    $20 = $qtblno;
    $21 = ($20|0)>=(4);
    if ($21) {
     label = 8;
    } else {
     $22 = $qtblno;
     $23 = $0;
     $24 = (($23) + 164|0);
     $25 = (($24) + ($22<<2)|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==(0|0);
     if ($27) {
      label = 8;
     }
    }
   }
   if ((label|0) == 8) {
    label = 0;
    $28 = $0;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($29) + 20|0);
    HEAP32[$30>>2] = 52;
    $31 = $qtblno;
    $32 = $0;
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 24|0);
    HEAP32[$34>>2] = $31;
    $35 = $0;
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = $0;
    FUNCTION_TABLE_vi[$37 & 255]($38);
   }
   $39 = $0;
   $40 = (($39) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = $0;
   $44 = (FUNCTION_TABLE_iiii[$42 & 255]($43,1,132)|0);
   $qtbl = $44;
   $45 = $qtbl;
   $46 = $qtblno;
   $47 = $0;
   $48 = (($47) + 164|0);
   $49 = (($48) + ($46<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   _memcpy(($45|0),($50|0),132)|0;
   $51 = $qtbl;
   $52 = $compptr;
   $53 = (($52) + 76|0);
   HEAP32[$53>>2] = $51;
  }
  $54 = $ci;
  $55 = (($54) + 1)|0;
  $ci = $55;
 }
 STACKTOP = sp;return;
}
function _initial_setup($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(65500);
 if ($4) {
  label = 3;
 } else {
  $5 = $0;
  $6 = (($5) + 28|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)>(65500);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = $0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 20|0);
  HEAP32[$11>>2] = 41;
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 24|0);
  HEAP32[$14>>2] = 65500;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 255]($18);
 }
 $19 = $0;
 $20 = (($19) + 212|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(8);
 if ($22) {
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 15;
  $26 = $0;
  $27 = (($26) + 212|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $0;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 24|0);
  HEAP32[$31>>2] = $28;
  $32 = $0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  FUNCTION_TABLE_vi[$34 & 255]($35);
 }
 $36 = $0;
 $37 = (($36) + 36|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)>(10);
 if ($39) {
  $40 = $0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 20|0);
  HEAP32[$42>>2] = 26;
  $43 = $0;
  $44 = (($43) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 24|0);
  HEAP32[$48>>2] = $45;
  $49 = $0;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 24|0);
  $52 = (($51) + 4|0);
  HEAP32[$52>>2] = 10;
  $53 = $0;
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = $0;
  FUNCTION_TABLE_vi[$55 & 255]($56);
 }
 $57 = $0;
 $58 = (($57) + 308|0);
 HEAP32[$58>>2] = 1;
 $59 = $0;
 $60 = (($59) + 312|0);
 HEAP32[$60>>2] = 1;
 $ci = 0;
 $61 = $0;
 $62 = (($61) + 216|0);
 $63 = HEAP32[$62>>2]|0;
 $compptr = $63;
 while(1) {
  $64 = $ci;
  $65 = $0;
  $66 = (($65) + 36|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($64|0)<($67|0);
  if (!($68)) {
   break;
  }
  $69 = $compptr;
  $70 = (($69) + 8|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = ($71|0)<=(0);
  if ($72) {
   label = 14;
  } else {
   $73 = $compptr;
   $74 = (($73) + 8|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75|0)>(4);
   if ($76) {
    label = 14;
   } else {
    $77 = $compptr;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)<=(0);
    if ($80) {
     label = 14;
    } else {
     $81 = $compptr;
     $82 = (($81) + 12|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)>(4);
     if ($84) {
      label = 14;
     }
    }
   }
  }
  if ((label|0) == 14) {
   label = 0;
   $85 = $0;
   $86 = HEAP32[$85>>2]|0;
   $87 = (($86) + 20|0);
   HEAP32[$87>>2] = 18;
   $88 = $0;
   $89 = HEAP32[$88>>2]|0;
   $90 = HEAP32[$89>>2]|0;
   $91 = $0;
   FUNCTION_TABLE_vi[$90 & 255]($91);
  }
  $92 = $0;
  $93 = (($92) + 308|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $compptr;
  $96 = (($95) + 8|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($94|0)>($97|0);
  if ($98) {
   $99 = $0;
   $100 = (($99) + 308|0);
   $101 = HEAP32[$100>>2]|0;
   $107 = $101;
  } else {
   $102 = $compptr;
   $103 = (($102) + 8|0);
   $104 = HEAP32[$103>>2]|0;
   $107 = $104;
  }
  $105 = $0;
  $106 = (($105) + 308|0);
  HEAP32[$106>>2] = $107;
  $108 = $0;
  $109 = (($108) + 312|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = $compptr;
  $112 = (($111) + 12|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($110|0)>($113|0);
  if ($114) {
   $115 = $0;
   $116 = (($115) + 312|0);
   $117 = HEAP32[$116>>2]|0;
   $123 = $117;
  } else {
   $118 = $compptr;
   $119 = (($118) + 12|0);
   $120 = HEAP32[$119>>2]|0;
   $123 = $120;
  }
  $121 = $0;
  $122 = (($121) + 312|0);
  HEAP32[$122>>2] = $123;
  $124 = $ci;
  $125 = (($124) + 1)|0;
  $ci = $125;
  $126 = $compptr;
  $127 = (($126) + 84|0);
  $compptr = $127;
 }
 $128 = $0;
 $129 = (($128) + 316|0);
 HEAP32[$129>>2] = 8;
 $ci = 0;
 $130 = $0;
 $131 = (($130) + 216|0);
 $132 = HEAP32[$131>>2]|0;
 $compptr = $132;
 while(1) {
  $133 = $ci;
  $134 = $0;
  $135 = (($134) + 36|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = ($133|0)<($136|0);
  if (!($137)) {
   break;
  }
  $138 = $compptr;
  $139 = (($138) + 36|0);
  HEAP32[$139>>2] = 8;
  $140 = $0;
  $141 = (($140) + 28|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = $compptr;
  $144 = (($143) + 8|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = Math_imul($142, $145)|0;
  $147 = $0;
  $148 = (($147) + 308|0);
  $149 = HEAP32[$148>>2]|0;
  $150 = $149<<3;
  $151 = (_jdiv_round_up($146,$150)|0);
  $152 = $compptr;
  $153 = (($152) + 28|0);
  HEAP32[$153>>2] = $151;
  $154 = $0;
  $155 = (($154) + 32|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = $compptr;
  $158 = (($157) + 12|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = Math_imul($156, $159)|0;
  $161 = $0;
  $162 = (($161) + 312|0);
  $163 = HEAP32[$162>>2]|0;
  $164 = $163<<3;
  $165 = (_jdiv_round_up($160,$164)|0);
  $166 = $compptr;
  $167 = (($166) + 32|0);
  HEAP32[$167>>2] = $165;
  $168 = $0;
  $169 = (($168) + 28|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = $compptr;
  $172 = (($171) + 8|0);
  $173 = HEAP32[$172>>2]|0;
  $174 = Math_imul($170, $173)|0;
  $175 = $0;
  $176 = (($175) + 308|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = (_jdiv_round_up($174,$177)|0);
  $179 = $compptr;
  $180 = (($179) + 40|0);
  HEAP32[$180>>2] = $178;
  $181 = $0;
  $182 = (($181) + 32|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = $compptr;
  $185 = (($184) + 12|0);
  $186 = HEAP32[$185>>2]|0;
  $187 = Math_imul($183, $186)|0;
  $188 = $0;
  $189 = (($188) + 312|0);
  $190 = HEAP32[$189>>2]|0;
  $191 = (_jdiv_round_up($187,$190)|0);
  $192 = $compptr;
  $193 = (($192) + 44|0);
  HEAP32[$193>>2] = $191;
  $194 = $compptr;
  $195 = (($194) + 48|0);
  HEAP32[$195>>2] = 1;
  $196 = $compptr;
  $197 = (($196) + 76|0);
  HEAP32[$197>>2] = 0;
  $198 = $ci;
  $199 = (($198) + 1)|0;
  $ci = $199;
  $200 = $compptr;
  $201 = (($200) + 84|0);
  $compptr = $201;
 }
 $202 = $0;
 $203 = (($202) + 32|0);
 $204 = HEAP32[$203>>2]|0;
 $205 = $0;
 $206 = (($205) + 312|0);
 $207 = HEAP32[$206>>2]|0;
 $208 = $207<<3;
 $209 = (_jdiv_round_up($204,$208)|0);
 $210 = $0;
 $211 = (($210) + 320|0);
 HEAP32[$211>>2] = $209;
 $212 = $0;
 $213 = (($212) + 328|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = $0;
 $216 = (($215) + 36|0);
 $217 = HEAP32[$216>>2]|0;
 $218 = ($214|0)<($217|0);
 if (!($218)) {
  $219 = $0;
  $220 = (($219) + 220|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = ($221|0)!=(0);
  if (!($222)) {
   $227 = $0;
   $228 = (($227) + 436|0);
   $229 = HEAP32[$228>>2]|0;
   $230 = (($229) + 16|0);
   HEAP32[$230>>2] = 0;
   STACKTOP = sp;return;
  }
 }
 $223 = $0;
 $224 = (($223) + 436|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + 16|0);
 HEAP32[$226>>2] = 1;
 STACKTOP = sp;return;
}
function _jpeg_resync_to_restart($cinfo,$desired) {
 $cinfo = $cinfo|0;
 $desired = $desired|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $action = 0, $marker = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $desired;
 $3 = $1;
 $4 = (($3) + 416|0);
 $5 = HEAP32[$4>>2]|0;
 $marker = $5;
 $action = 1;
 $6 = $1;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = 121;
 $9 = $marker;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 24|0);
 HEAP32[$12>>2] = $9;
 $13 = $2;
 $14 = $1;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 24|0);
 $17 = (($16) + 4|0);
 HEAP32[$17>>2] = $13;
 $18 = $1;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $1;
 FUNCTION_TABLE_vii[$21 & 255]($22,-1);
 while(1) {
  $23 = $marker;
  $24 = ($23|0)<(192);
  if ($24) {
   $action = 2;
  } else {
   $25 = $marker;
   $26 = ($25|0)<(208);
   if ($26) {
    label = 6;
   } else {
    $27 = $marker;
    $28 = ($27|0)>(215);
    if ($28) {
     label = 6;
    } else {
     $29 = $marker;
     $30 = $2;
     $31 = (($30) + 1)|0;
     $32 = $31 & 7;
     $33 = (208 + ($32))|0;
     $34 = ($29|0)==($33|0);
     if ($34) {
      label = 9;
     } else {
      $35 = $marker;
      $36 = $2;
      $37 = (($36) + 2)|0;
      $38 = $37 & 7;
      $39 = (208 + ($38))|0;
      $40 = ($35|0)==($39|0);
      if ($40) {
       label = 9;
      } else {
       $41 = $marker;
       $42 = $2;
       $43 = (($42) - 1)|0;
       $44 = $43 & 7;
       $45 = (208 + ($44))|0;
       $46 = ($41|0)==($45|0);
       if ($46) {
        label = 12;
       } else {
        $47 = $marker;
        $48 = $2;
        $49 = (($48) - 2)|0;
        $50 = $49 & 7;
        $51 = (208 + ($50))|0;
        $52 = ($47|0)==($51|0);
        if ($52) {
         label = 12;
        } else {
         $action = 1;
        }
       }
       if ((label|0) == 12) {
        label = 0;
        $action = 2;
       }
      }
     }
     if ((label|0) == 9) {
      label = 0;
      $action = 3;
     }
    }
   }
   if ((label|0) == 6) {
    label = 0;
    $action = 3;
   }
  }
  $53 = $1;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 20|0);
  HEAP32[$55>>2] = 97;
  $56 = $marker;
  $57 = $1;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 24|0);
  HEAP32[$59>>2] = $56;
  $60 = $action;
  $61 = $1;
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) + 24|0);
  $64 = (($63) + 4|0);
  HEAP32[$64>>2] = $60;
  $65 = $1;
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + 4|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $1;
  FUNCTION_TABLE_vii[$68 & 255]($69,4);
  $70 = $action;
  if ((($70|0) == 3)) {
   label = 22;
   break;
  } else if ((($70|0) == 2)) {
   $73 = $1;
   $74 = (_next_marker($73)|0);
   $75 = ($74|0)!=(0);
   if (!($75)) {
    label = 20;
    break;
   }
   $76 = $1;
   $77 = (($76) + 416|0);
   $78 = HEAP32[$77>>2]|0;
   $marker = $78;
  } else if ((($70|0) == 1)) {
   label = 18;
   break;
  }
 }
 if ((label|0) == 18) {
  $71 = $1;
  $72 = (($71) + 416|0);
  HEAP32[$72>>2] = 0;
  $0 = 1;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 20) {
  $0 = 0;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 else if ((label|0) == 22) {
  $0 = 1;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 return 0|0;
}
function _next_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytes_in_buffer = 0, $c = 0, $datasrc = 0, $next_input_byte = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 L1: while(1) {
  $10 = $bytes_in_buffer;
  $11 = ($10|0)==(0);
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if (!($17)) {
    label = 5;
    break;
   }
   $18 = $datasrc;
   $19 = HEAP32[$18>>2]|0;
   $next_input_byte = $19;
   $20 = $datasrc;
   $21 = (($20) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $bytes_in_buffer = $22;
  }
  $23 = $bytes_in_buffer;
  $24 = (($23) + -1)|0;
  $bytes_in_buffer = $24;
  $25 = $next_input_byte;
  $26 = (($25) + 1|0);
  $next_input_byte = $26;
  $27 = HEAP8[$25>>0]|0;
  $28 = $27&255;
  $c = $28;
  while(1) {
   $29 = $c;
   $30 = ($29|0)!=(255);
   if (!($30)) {
    break;
   }
   $31 = $1;
   $32 = (($31) + 440|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 24|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 1)|0;
   HEAP32[$34>>2] = $36;
   $37 = $next_input_byte;
   $38 = $datasrc;
   HEAP32[$38>>2] = $37;
   $39 = $bytes_in_buffer;
   $40 = $datasrc;
   $41 = (($40) + 4|0);
   HEAP32[$41>>2] = $39;
   $42 = $bytes_in_buffer;
   $43 = ($42|0)==(0);
   if ($43) {
    $44 = $datasrc;
    $45 = (($44) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = $1;
    $48 = (FUNCTION_TABLE_ii[$46 & 127]($47)|0);
    $49 = ($48|0)!=(0);
    if (!($49)) {
     label = 13;
     break L1;
    }
    $50 = $datasrc;
    $51 = HEAP32[$50>>2]|0;
    $next_input_byte = $51;
    $52 = $datasrc;
    $53 = (($52) + 4|0);
    $54 = HEAP32[$53>>2]|0;
    $bytes_in_buffer = $54;
   }
   $55 = $bytes_in_buffer;
   $56 = (($55) + -1)|0;
   $bytes_in_buffer = $56;
   $57 = $next_input_byte;
   $58 = (($57) + 1|0);
   $next_input_byte = $58;
   $59 = HEAP8[$57>>0]|0;
   $60 = $59&255;
   $c = $60;
  }
  while(1) {
   $61 = $bytes_in_buffer;
   $62 = ($61|0)==(0);
   if ($62) {
    $63 = $datasrc;
    $64 = (($63) + 12|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = $1;
    $67 = (FUNCTION_TABLE_ii[$65 & 127]($66)|0);
    $68 = ($67|0)!=(0);
    if (!($68)) {
     label = 21;
     break L1;
    }
    $69 = $datasrc;
    $70 = HEAP32[$69>>2]|0;
    $next_input_byte = $70;
    $71 = $datasrc;
    $72 = (($71) + 4|0);
    $73 = HEAP32[$72>>2]|0;
    $bytes_in_buffer = $73;
   }
   $74 = $bytes_in_buffer;
   $75 = (($74) + -1)|0;
   $bytes_in_buffer = $75;
   $76 = $next_input_byte;
   $77 = (($76) + 1|0);
   $next_input_byte = $77;
   $78 = HEAP8[$76>>0]|0;
   $79 = $78&255;
   $c = $79;
   $80 = $c;
   $81 = ($80|0)==(255);
   if (!($81)) {
    break;
   }
  }
  $82 = $c;
  $83 = ($82|0)!=(0);
  if ($83) {
   label = 27;
   break;
  }
  $84 = $1;
  $85 = (($84) + 440|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + 24|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($88) + 2)|0;
  HEAP32[$87>>2] = $89;
  $90 = $next_input_byte;
  $91 = $datasrc;
  HEAP32[$91>>2] = $90;
  $92 = $bytes_in_buffer;
  $93 = $datasrc;
  $94 = (($93) + 4|0);
  HEAP32[$94>>2] = $92;
 }
 if ((label|0) == 5) {
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 13) {
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 21) {
  $0 = 0;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 else if ((label|0) == 27) {
  $95 = $1;
  $96 = (($95) + 440|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = (($97) + 24|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = $1;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) + 20|0);
   HEAP32[$103>>2] = 116;
   $104 = $1;
   $105 = (($104) + 440|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($106) + 24|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = $1;
   $110 = HEAP32[$109>>2]|0;
   $111 = (($110) + 24|0);
   HEAP32[$111>>2] = $108;
   $112 = $c;
   $113 = $1;
   $114 = HEAP32[$113>>2]|0;
   $115 = (($114) + 24|0);
   $116 = (($115) + 4|0);
   HEAP32[$116>>2] = $112;
   $117 = $1;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($118) + 4|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = $1;
   FUNCTION_TABLE_vii[$120 & 255]($121,-1);
   $122 = $1;
   $123 = (($122) + 440|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = (($124) + 24|0);
   HEAP32[$125>>2] = 0;
  }
  $126 = $c;
  $127 = $1;
  $128 = (($127) + 416|0);
  HEAP32[$128>>2] = $126;
  $129 = $next_input_byte;
  $130 = $datasrc;
  HEAP32[$130>>2] = $129;
  $131 = $bytes_in_buffer;
  $132 = $datasrc;
  $133 = (($132) + 4|0);
  HEAP32[$133>>2] = $131;
  $0 = 1;
  $134 = $0;
  STACKTOP = sp;return ($134|0);
 }
 return 0|0;
}
function _jinit_marker_reader($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $marker = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,0,172)|0);
 $marker = $6;
 $7 = $marker;
 $8 = $0;
 $9 = (($8) + 440|0);
 HEAP32[$9>>2] = $7;
 $10 = $marker;
 HEAP32[$10>>2] = 39;
 $11 = $marker;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 40;
 $13 = $marker;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 41;
 $15 = $marker;
 $16 = (($15) + 28|0);
 HEAP32[$16>>2] = 42;
 $17 = $marker;
 $18 = (($17) + 96|0);
 HEAP32[$18>>2] = 0;
 $i = 0;
 while(1) {
  $19 = $i;
  $20 = ($19|0)<(16);
  if (!($20)) {
   break;
  }
  $21 = $i;
  $22 = $marker;
  $23 = (($22) + 32|0);
  $24 = (($23) + ($21<<2)|0);
  HEAP32[$24>>2] = 42;
  $25 = $i;
  $26 = $marker;
  $27 = (($26) + 100|0);
  $28 = (($27) + ($25<<2)|0);
  HEAP32[$28>>2] = 0;
  $29 = $i;
  $30 = (($29) + 1)|0;
  $i = $30;
 }
 $31 = $marker;
 $32 = (($31) + 32|0);
 HEAP32[$32>>2] = 43;
 $33 = $marker;
 $34 = (($33) + 32|0);
 $35 = (($34) + 56|0);
 HEAP32[$35>>2] = 43;
 $36 = $0;
 _reset_marker_reader($36);
 STACKTOP = sp;return;
}
function _reset_marker_reader($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $marker = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 440|0);
 $3 = HEAP32[$2>>2]|0;
 $marker = $3;
 $4 = $0;
 $5 = (($4) + 216|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 $7 = (($6) + 144|0);
 HEAP32[$7>>2] = 0;
 $8 = $0;
 $9 = (($8) + 416|0);
 HEAP32[$9>>2] = 0;
 $10 = $marker;
 $11 = (($10) + 12|0);
 HEAP32[$11>>2] = 0;
 $12 = $marker;
 $13 = (($12) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = $marker;
 $15 = (($14) + 24|0);
 HEAP32[$15>>2] = 0;
 $16 = $marker;
 $17 = (($16) + 164|0);
 HEAP32[$17>>2] = 0;
 STACKTOP = sp;return;
}
function _read_markers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 L1: while(1) {
  $2 = $1;
  $3 = (($2) + 416|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)==(0);
  if ($5) {
   $6 = $1;
   $7 = (($6) + 440|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (($8) + 12|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0);
   if ($11) {
    $15 = $1;
    $16 = (_next_marker($15)|0);
    $17 = ($16|0)!=(0);
    if (!($17)) {
     label = 8;
     break;
    }
   } else {
    $12 = $1;
    $13 = (_first_marker($12)|0);
    $14 = ($13|0)!=(0);
    if (!($14)) {
     label = 5;
     break;
    }
   }
  }
  $18 = $1;
  $19 = (($18) + 416|0);
  $20 = HEAP32[$19>>2]|0;
  do {
   switch ($20|0) {
   case 221:  {
    $73 = $1;
    $74 = (_get_dri($73)|0);
    $75 = ($74|0)!=(0);
    if (!($75)) {
     label = 42;
     break L1;
    }
    break;
   }
   case 194:  {
    $27 = $1;
    $28 = (_get_sof($27,1,0)|0);
    $29 = ($28|0)!=(0);
    if (!($29)) {
     label = 19;
     break L1;
    }
    break;
   }
   case 217:  {
    label = 31;
    break L1;
    break;
   }
   case 254:  {
    $89 = $1;
    $90 = (($89) + 440|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = (($91) + 28|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = $1;
    $95 = (FUNCTION_TABLE_ii[$93 & 127]($94)|0);
    $96 = ($95|0)!=(0);
    if (!($96)) {
     label = 48;
     break L1;
    }
    break;
   }
   case 204:  {
    $64 = $1;
    $65 = (_skip_variable($64)|0);
    $66 = ($65|0)!=(0);
    if (!($66)) {
     label = 33;
     break L1;
    }
    break;
   }
   case 202:  {
    $33 = $1;
    $34 = (_get_sof($33,1,1)|0);
    $35 = ($34|0)!=(0);
    if (!($35)) {
     label = 25;
     break L1;
    }
    break;
   }
   case 207: case 206: case 205: case 203: case 200: case 199: case 198: case 197: case 195:  {
    $36 = $1;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + 20|0);
    HEAP32[$38>>2] = 60;
    $39 = $1;
    $40 = (($39) + 416|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $1;
    $43 = HEAP32[$42>>2]|0;
    $44 = (($43) + 24|0);
    HEAP32[$44>>2] = $41;
    $45 = $1;
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$46>>2]|0;
    $48 = $1;
    FUNCTION_TABLE_vi[$47 & 255]($48);
    break;
   }
   case 1: case 215: case 214: case 213: case 212: case 211: case 210: case 209: case 208:  {
    $97 = $1;
    $98 = HEAP32[$97>>2]|0;
    $99 = (($98) + 20|0);
    HEAP32[$99>>2] = 92;
    $100 = $1;
    $101 = (($100) + 416|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = $1;
    $104 = HEAP32[$103>>2]|0;
    $105 = (($104) + 24|0);
    HEAP32[$105>>2] = $102;
    $106 = $1;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($107) + 4|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = $1;
    FUNCTION_TABLE_vii[$109 & 255]($110,1);
    break;
   }
   case 201:  {
    $30 = $1;
    $31 = (_get_sof($30,0,1)|0);
    $32 = ($31|0)!=(0);
    if (!($32)) {
     label = 22;
     break L1;
    }
    break;
   }
   case 219:  {
    $70 = $1;
    $71 = (_get_dqt($70)|0);
    $72 = ($71|0)!=(0);
    if (!($72)) {
     label = 39;
     break L1;
    }
    break;
   }
   case 239: case 238: case 237: case 236: case 235: case 234: case 233: case 232: case 231: case 230: case 229: case 228: case 227: case 226: case 225: case 224:  {
    $76 = $1;
    $77 = (($76) + 416|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) - 224)|0;
    $80 = $1;
    $81 = (($80) + 440|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (($82) + 32|0);
    $84 = (($83) + ($79<<2)|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = $1;
    $87 = (FUNCTION_TABLE_ii[$85 & 127]($86)|0);
    $88 = ($87|0)!=(0);
    if (!($88)) {
     label = 45;
     break L1;
    }
    break;
   }
   case 196:  {
    $67 = $1;
    $68 = (_get_dht($67)|0);
    $69 = ($68|0)!=(0);
    if (!($69)) {
     label = 36;
     break L1;
    }
    break;
   }
   case 220:  {
    $111 = $1;
    $112 = (_skip_variable($111)|0);
    $113 = ($112|0)!=(0);
    if (!($113)) {
     label = 52;
     break L1;
    }
    break;
   }
   case 216:  {
    $21 = $1;
    $22 = (_get_soi($21)|0);
    $23 = ($22|0)!=(0);
    if (!($23)) {
     label = 13;
     break L1;
    }
    break;
   }
   case 218:  {
    label = 28;
    break L1;
    break;
   }
   case 193: case 192:  {
    $24 = $1;
    $25 = (_get_sof($24,0,0)|0);
    $26 = ($25|0)!=(0);
    if (!($26)) {
     label = 16;
     break L1;
    }
    break;
   }
   default: {
    $114 = $1;
    $115 = HEAP32[$114>>2]|0;
    $116 = (($115) + 20|0);
    HEAP32[$116>>2] = 68;
    $117 = $1;
    $118 = (($117) + 416|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = $1;
    $121 = HEAP32[$120>>2]|0;
    $122 = (($121) + 24|0);
    HEAP32[$122>>2] = $119;
    $123 = $1;
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP32[$124>>2]|0;
    $126 = $1;
    FUNCTION_TABLE_vi[$125 & 255]($126);
   }
   }
  } while(0);
  $127 = $1;
  $128 = (($127) + 416|0);
  HEAP32[$128>>2] = 0;
 }
 switch (label|0) {
  case 5: {
   $0 = 0;
   break;
  }
  case 8: {
   $0 = 0;
   break;
  }
  case 13: {
   $0 = 0;
   break;
  }
  case 16: {
   $0 = 0;
   break;
  }
  case 19: {
   $0 = 0;
   break;
  }
  case 22: {
   $0 = 0;
   break;
  }
  case 25: {
   $0 = 0;
   break;
  }
  case 28: {
   $49 = $1;
   $50 = (_get_sos($49)|0);
   $51 = ($50|0)!=(0);
   if ($51) {
    $52 = $1;
    $53 = (($52) + 416|0);
    HEAP32[$53>>2] = 0;
    $0 = 1;
    break;
   } else {
    $0 = 0;
    break;
   }
   break;
  }
  case 31: {
   $54 = $1;
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 20|0);
   HEAP32[$56>>2] = 85;
   $57 = $1;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($58) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $1;
   FUNCTION_TABLE_vii[$60 & 255]($61,1);
   $62 = $1;
   $63 = (($62) + 416|0);
   HEAP32[$63>>2] = 0;
   $0 = 2;
   break;
  }
  case 33: {
   $0 = 0;
   break;
  }
  case 36: {
   $0 = 0;
   break;
  }
  case 39: {
   $0 = 0;
   break;
  }
  case 42: {
   $0 = 0;
   break;
  }
  case 45: {
   $0 = 0;
   break;
  }
  case 48: {
   $0 = 0;
   break;
  }
  case 52: {
   $0 = 0;
   break;
  }
 }
 $129 = $0;
 STACKTOP = sp;return ($129|0);
}
function _read_restart_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 416|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = $1;
   $7 = (_next_marker($6)|0);
   $8 = ($7|0)!=(0);
   if ($8) {
    break;
   }
   $0 = 0;
   $61 = $0;
   STACKTOP = sp;return ($61|0);
  }
 } while(0);
 $9 = $1;
 $10 = (($9) + 416|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $1;
 $13 = (($12) + 440|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 20|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (208 + ($16))|0;
 $18 = ($11|0)==($17|0);
 do {
  if ($18) {
   $19 = $1;
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 20|0);
   HEAP32[$21>>2] = 98;
   $22 = $1;
   $23 = (($22) + 440|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + 20|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $1;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 24|0);
   HEAP32[$29>>2] = $26;
   $30 = $1;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $1;
   FUNCTION_TABLE_vii[$33 & 255]($34,3);
   $35 = $1;
   $36 = (($35) + 416|0);
   HEAP32[$36>>2] = 0;
  } else {
   $37 = $1;
   $38 = (($37) + 24|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + 20|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = $1;
   $43 = $1;
   $44 = (($43) + 440|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (($45) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (FUNCTION_TABLE_iii[$41 & 255]($42,$47)|0);
   $49 = ($48|0)!=(0);
   if ($49) {
    break;
   }
   $0 = 0;
   $61 = $0;
   STACKTOP = sp;return ($61|0);
  }
 } while(0);
 $50 = $1;
 $51 = (($50) + 440|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + 20|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 1)|0;
 $56 = $55 & 7;
 $57 = $1;
 $58 = (($57) + 440|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + 20|0);
 HEAP32[$60>>2] = $56;
 $0 = 1;
 $61 = $0;
 STACKTOP = sp;return ($61|0);
}
function _skip_variable($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $bytes_in_buffer = 0, $datasrc = 0, $length = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $86 = $0;
   STACKTOP = sp;return ($86|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $86 = $0;
   STACKTOP = sp;return ($86|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 $53 = $1;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 20|0);
 HEAP32[$55>>2] = 91;
 $56 = $1;
 $57 = (($56) + 416|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $1;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + 24|0);
 HEAP32[$61>>2] = $58;
 $62 = $length;
 $63 = $1;
 $64 = HEAP32[$63>>2]|0;
 $65 = (($64) + 24|0);
 $66 = (($65) + 4|0);
 HEAP32[$66>>2] = $62;
 $67 = $1;
 $68 = HEAP32[$67>>2]|0;
 $69 = (($68) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $1;
 FUNCTION_TABLE_vii[$70 & 255]($71,1);
 $72 = $next_input_byte;
 $73 = $datasrc;
 HEAP32[$73>>2] = $72;
 $74 = $bytes_in_buffer;
 $75 = $datasrc;
 $76 = (($75) + 4|0);
 HEAP32[$76>>2] = $74;
 $77 = $length;
 $78 = ($77|0)>(0);
 if ($78) {
  $79 = $1;
  $80 = (($79) + 24|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 16|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $1;
  $85 = $length;
  FUNCTION_TABLE_vii[$83 & 255]($84,$85);
 }
 $0 = 1;
 $86 = $0;
 STACKTOP = sp;return ($86|0);
}
function _get_interesting_appn($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $b = 0, $bytes_in_buffer = 0, $datasrc = 0, $i = 0, $length = 0, $next_input_byte = 0, $numtoread = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $b = sp + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $122 = $0;
   STACKTOP = sp;return ($122|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $122 = $0;
   STACKTOP = sp;return ($122|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 $53 = $length;
 $54 = ($53|0)>=(14);
 if ($54) {
  $numtoread = 14;
 } else {
  $55 = $length;
  $56 = ($55|0)>(0);
  if ($56) {
   $57 = $length;
   $numtoread = $57;
  } else {
   $numtoread = 0;
  }
 }
 $i = 0;
 while(1) {
  $58 = $i;
  $59 = $numtoread;
  $60 = ($58>>>0)<($59>>>0);
  if (!($60)) {
   break;
  }
  $61 = $bytes_in_buffer;
  $62 = ($61|0)==(0);
  if ($62) {
   $63 = $datasrc;
   $64 = (($63) + 12|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $1;
   $67 = (FUNCTION_TABLE_ii[$65 & 127]($66)|0);
   $68 = ($67|0)!=(0);
   if (!($68)) {
    label = 22;
    break;
   }
   $69 = $datasrc;
   $70 = HEAP32[$69>>2]|0;
   $next_input_byte = $70;
   $71 = $datasrc;
   $72 = (($71) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $bytes_in_buffer = $73;
  }
  $74 = $bytes_in_buffer;
  $75 = (($74) + -1)|0;
  $bytes_in_buffer = $75;
  $76 = $next_input_byte;
  $77 = (($76) + 1|0);
  $next_input_byte = $77;
  $78 = HEAP8[$76>>0]|0;
  $79 = $i;
  $80 = (($b) + ($79)|0);
  HEAP8[$80>>0] = $78;
  $81 = $i;
  $82 = (($81) + 1)|0;
  $i = $82;
 }
 if ((label|0) == 22) {
  $0 = 0;
  $122 = $0;
  STACKTOP = sp;return ($122|0);
 }
 $83 = $numtoread;
 $84 = $length;
 $85 = (($84) - ($83))|0;
 $length = $85;
 $86 = $1;
 $87 = (($86) + 416|0);
 $88 = HEAP32[$87>>2]|0;
 if ((($88|0) == 238)) {
  $92 = $1;
  $93 = $numtoread;
  $94 = $length;
  _examine_app14($92,$b,$93,$94);
 } else if ((($88|0) == 224)) {
  $89 = $1;
  $90 = $numtoread;
  $91 = $length;
  _examine_app0($89,$b,$90,$91);
 } else {
  $95 = $1;
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 20|0);
  HEAP32[$97>>2] = 68;
  $98 = $1;
  $99 = (($98) + 416|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = $1;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($102) + 24|0);
  HEAP32[$103>>2] = $100;
  $104 = $1;
  $105 = HEAP32[$104>>2]|0;
  $106 = HEAP32[$105>>2]|0;
  $107 = $1;
  FUNCTION_TABLE_vi[$106 & 255]($107);
 }
 $108 = $next_input_byte;
 $109 = $datasrc;
 HEAP32[$109>>2] = $108;
 $110 = $bytes_in_buffer;
 $111 = $datasrc;
 $112 = (($111) + 4|0);
 HEAP32[$112>>2] = $110;
 $113 = $length;
 $114 = ($113|0)>(0);
 if ($114) {
  $115 = $1;
  $116 = (($115) + 24|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = (($117) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = $1;
  $121 = $length;
  FUNCTION_TABLE_vii[$119 & 255]($120,$121);
 }
 $0 = 1;
 $122 = $0;
 STACKTOP = sp;return ($122|0);
}
function _examine_app0($cinfo,$data,$datalen,$remaining) {
 $cinfo = $cinfo|0;
 $data = $data|0;
 $datalen = $datalen|0;
 $remaining = $remaining|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $_mp = 0, $totallen = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $data;
 $2 = $datalen;
 $3 = $remaining;
 $4 = $2;
 $5 = $3;
 $6 = (($4) + ($5))|0;
 $totallen = $6;
 $7 = $2;
 $8 = ($7>>>0)>=(14);
 if ($8) {
  $9 = $1;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)==(74);
  if ($12) {
   $13 = $1;
   $14 = (($13) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)==(70);
   if ($17) {
    $18 = $1;
    $19 = (($18) + 2|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    $22 = ($21|0)==(73);
    if ($22) {
     $23 = $1;
     $24 = (($23) + 3|0);
     $25 = HEAP8[$24>>0]|0;
     $26 = $25&255;
     $27 = ($26|0)==(70);
     if ($27) {
      $28 = $1;
      $29 = (($28) + 4|0);
      $30 = HEAP8[$29>>0]|0;
      $31 = $30&255;
      $32 = ($31|0)==(0);
      if ($32) {
       $33 = $0;
       $34 = (($33) + 280|0);
       HEAP32[$34>>2] = 1;
       $35 = $1;
       $36 = (($35) + 5|0);
       $37 = HEAP8[$36>>0]|0;
       $38 = $0;
       $39 = (($38) + 284|0);
       HEAP8[$39>>0] = $37;
       $40 = $1;
       $41 = (($40) + 6|0);
       $42 = HEAP8[$41>>0]|0;
       $43 = $0;
       $44 = (($43) + 285|0);
       HEAP8[$44>>0] = $42;
       $45 = $1;
       $46 = (($45) + 7|0);
       $47 = HEAP8[$46>>0]|0;
       $48 = $0;
       $49 = (($48) + 286|0);
       HEAP8[$49>>0] = $47;
       $50 = $1;
       $51 = (($50) + 8|0);
       $52 = HEAP8[$51>>0]|0;
       $53 = $52&255;
       $54 = $53 << 8;
       $55 = $1;
       $56 = (($55) + 9|0);
       $57 = HEAP8[$56>>0]|0;
       $58 = $57&255;
       $59 = (($54) + ($58))|0;
       $60 = $59&65535;
       $61 = $0;
       $62 = (($61) + 288|0);
       HEAP16[$62>>1] = $60;
       $63 = $1;
       $64 = (($63) + 10|0);
       $65 = HEAP8[$64>>0]|0;
       $66 = $65&255;
       $67 = $66 << 8;
       $68 = $1;
       $69 = (($68) + 11|0);
       $70 = HEAP8[$69>>0]|0;
       $71 = $70&255;
       $72 = (($67) + ($71))|0;
       $73 = $72&65535;
       $74 = $0;
       $75 = (($74) + 290|0);
       HEAP16[$75>>1] = $73;
       $76 = $0;
       $77 = (($76) + 284|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78&255;
       $80 = ($79|0)!=(1);
       if ($80) {
        $81 = $0;
        $82 = HEAP32[$81>>2]|0;
        $83 = (($82) + 20|0);
        HEAP32[$83>>2] = 119;
        $84 = $0;
        $85 = (($84) + 284|0);
        $86 = HEAP8[$85>>0]|0;
        $87 = $86&255;
        $88 = $0;
        $89 = HEAP32[$88>>2]|0;
        $90 = (($89) + 24|0);
        HEAP32[$90>>2] = $87;
        $91 = $0;
        $92 = (($91) + 285|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = $93&255;
        $95 = $0;
        $96 = HEAP32[$95>>2]|0;
        $97 = (($96) + 24|0);
        $98 = (($97) + 4|0);
        HEAP32[$98>>2] = $94;
        $99 = $0;
        $100 = HEAP32[$99>>2]|0;
        $101 = (($100) + 4|0);
        $102 = HEAP32[$101>>2]|0;
        $103 = $0;
        FUNCTION_TABLE_vii[$102 & 255]($103,-1);
       }
       $104 = $0;
       $105 = HEAP32[$104>>2]|0;
       $106 = (($105) + 24|0);
       $_mp = $106;
       $107 = $0;
       $108 = (($107) + 284|0);
       $109 = HEAP8[$108>>0]|0;
       $110 = $109&255;
       $111 = $_mp;
       HEAP32[$111>>2] = $110;
       $112 = $0;
       $113 = (($112) + 285|0);
       $114 = HEAP8[$113>>0]|0;
       $115 = $114&255;
       $116 = $_mp;
       $117 = (($116) + 4|0);
       HEAP32[$117>>2] = $115;
       $118 = $0;
       $119 = (($118) + 288|0);
       $120 = HEAP16[$119>>1]|0;
       $121 = $120&65535;
       $122 = $_mp;
       $123 = (($122) + 8|0);
       HEAP32[$123>>2] = $121;
       $124 = $0;
       $125 = (($124) + 290|0);
       $126 = HEAP16[$125>>1]|0;
       $127 = $126&65535;
       $128 = $_mp;
       $129 = (($128) + 12|0);
       HEAP32[$129>>2] = $127;
       $130 = $0;
       $131 = (($130) + 286|0);
       $132 = HEAP8[$131>>0]|0;
       $133 = $132&255;
       $134 = $_mp;
       $135 = (($134) + 16|0);
       HEAP32[$135>>2] = $133;
       $136 = $0;
       $137 = HEAP32[$136>>2]|0;
       $138 = (($137) + 20|0);
       HEAP32[$138>>2] = 87;
       $139 = $0;
       $140 = HEAP32[$139>>2]|0;
       $141 = (($140) + 4|0);
       $142 = HEAP32[$141>>2]|0;
       $143 = $0;
       FUNCTION_TABLE_vii[$142 & 255]($143,1);
       $144 = $1;
       $145 = (($144) + 12|0);
       $146 = HEAP8[$145>>0]|0;
       $147 = $146&255;
       $148 = $1;
       $149 = (($148) + 13|0);
       $150 = HEAP8[$149>>0]|0;
       $151 = $150&255;
       $152 = $147 | $151;
       $153 = ($152|0)!=(0);
       if ($153) {
        $154 = $0;
        $155 = HEAP32[$154>>2]|0;
        $156 = (($155) + 20|0);
        HEAP32[$156>>2] = 90;
        $157 = $1;
        $158 = (($157) + 12|0);
        $159 = HEAP8[$158>>0]|0;
        $160 = $159&255;
        $161 = $0;
        $162 = HEAP32[$161>>2]|0;
        $163 = (($162) + 24|0);
        HEAP32[$163>>2] = $160;
        $164 = $1;
        $165 = (($164) + 13|0);
        $166 = HEAP8[$165>>0]|0;
        $167 = $166&255;
        $168 = $0;
        $169 = HEAP32[$168>>2]|0;
        $170 = (($169) + 24|0);
        $171 = (($170) + 4|0);
        HEAP32[$171>>2] = $167;
        $172 = $0;
        $173 = HEAP32[$172>>2]|0;
        $174 = (($173) + 4|0);
        $175 = HEAP32[$174>>2]|0;
        $176 = $0;
        FUNCTION_TABLE_vii[$175 & 255]($176,1);
       }
       $177 = $totallen;
       $178 = (($177) - 14)|0;
       $totallen = $178;
       $179 = $totallen;
       $180 = $1;
       $181 = (($180) + 12|0);
       $182 = HEAP8[$181>>0]|0;
       $183 = $182&255;
       $184 = $1;
       $185 = (($184) + 13|0);
       $186 = HEAP8[$185>>0]|0;
       $187 = $186&255;
       $188 = Math_imul($183, $187)|0;
       $189 = ($188*3)|0;
       $190 = ($179|0)!=($189|0);
       if ($190) {
        $191 = $0;
        $192 = HEAP32[$191>>2]|0;
        $193 = (($192) + 20|0);
        HEAP32[$193>>2] = 88;
        $194 = $totallen;
        $195 = $0;
        $196 = HEAP32[$195>>2]|0;
        $197 = (($196) + 24|0);
        HEAP32[$197>>2] = $194;
        $198 = $0;
        $199 = HEAP32[$198>>2]|0;
        $200 = (($199) + 4|0);
        $201 = HEAP32[$200>>2]|0;
        $202 = $0;
        FUNCTION_TABLE_vii[$201 & 255]($202,1);
       }
       STACKTOP = sp;return;
      }
     }
    }
   }
  }
 }
 $203 = $2;
 $204 = ($203>>>0)>=(6);
 if ($204) {
  $205 = $1;
  $206 = HEAP8[$205>>0]|0;
  $207 = $206&255;
  $208 = ($207|0)==(74);
  if ($208) {
   $209 = $1;
   $210 = (($209) + 1|0);
   $211 = HEAP8[$210>>0]|0;
   $212 = $211&255;
   $213 = ($212|0)==(70);
   if ($213) {
    $214 = $1;
    $215 = (($214) + 2|0);
    $216 = HEAP8[$215>>0]|0;
    $217 = $216&255;
    $218 = ($217|0)==(88);
    if ($218) {
     $219 = $1;
     $220 = (($219) + 3|0);
     $221 = HEAP8[$220>>0]|0;
     $222 = $221&255;
     $223 = ($222|0)==(88);
     if ($223) {
      $224 = $1;
      $225 = (($224) + 4|0);
      $226 = HEAP8[$225>>0]|0;
      $227 = $226&255;
      $228 = ($227|0)==(0);
      if ($228) {
       $229 = $1;
       $230 = (($229) + 5|0);
       $231 = HEAP8[$230>>0]|0;
       $232 = $231&255;
       if ((($232|0) == 16)) {
        $233 = $0;
        $234 = HEAP32[$233>>2]|0;
        $235 = (($234) + 20|0);
        HEAP32[$235>>2] = 108;
        $236 = $totallen;
        $237 = $0;
        $238 = HEAP32[$237>>2]|0;
        $239 = (($238) + 24|0);
        HEAP32[$239>>2] = $236;
        $240 = $0;
        $241 = HEAP32[$240>>2]|0;
        $242 = (($241) + 4|0);
        $243 = HEAP32[$242>>2]|0;
        $244 = $0;
        FUNCTION_TABLE_vii[$243 & 255]($244,1);
       } else if ((($232|0) == 17)) {
        $245 = $0;
        $246 = HEAP32[$245>>2]|0;
        $247 = (($246) + 20|0);
        HEAP32[$247>>2] = 109;
        $248 = $totallen;
        $249 = $0;
        $250 = HEAP32[$249>>2]|0;
        $251 = (($250) + 24|0);
        HEAP32[$251>>2] = $248;
        $252 = $0;
        $253 = HEAP32[$252>>2]|0;
        $254 = (($253) + 4|0);
        $255 = HEAP32[$254>>2]|0;
        $256 = $0;
        FUNCTION_TABLE_vii[$255 & 255]($256,1);
       } else if ((($232|0) == 19)) {
        $257 = $0;
        $258 = HEAP32[$257>>2]|0;
        $259 = (($258) + 20|0);
        HEAP32[$259>>2] = 110;
        $260 = $totallen;
        $261 = $0;
        $262 = HEAP32[$261>>2]|0;
        $263 = (($262) + 24|0);
        HEAP32[$263>>2] = $260;
        $264 = $0;
        $265 = HEAP32[$264>>2]|0;
        $266 = (($265) + 4|0);
        $267 = HEAP32[$266>>2]|0;
        $268 = $0;
        FUNCTION_TABLE_vii[$267 & 255]($268,1);
       } else {
        $269 = $0;
        $270 = HEAP32[$269>>2]|0;
        $271 = (($270) + 20|0);
        HEAP32[$271>>2] = 89;
        $272 = $1;
        $273 = (($272) + 5|0);
        $274 = HEAP8[$273>>0]|0;
        $275 = $274&255;
        $276 = $0;
        $277 = HEAP32[$276>>2]|0;
        $278 = (($277) + 24|0);
        HEAP32[$278>>2] = $275;
        $279 = $totallen;
        $280 = $0;
        $281 = HEAP32[$280>>2]|0;
        $282 = (($281) + 24|0);
        $283 = (($282) + 4|0);
        HEAP32[$283>>2] = $279;
        $284 = $0;
        $285 = HEAP32[$284>>2]|0;
        $286 = (($285) + 4|0);
        $287 = HEAP32[$286>>2]|0;
        $288 = $0;
        FUNCTION_TABLE_vii[$287 & 255]($288,1);
       }
      } else {
       label = 28;
      }
     } else {
      label = 28;
     }
    } else {
     label = 28;
    }
   } else {
    label = 28;
   }
  } else {
   label = 28;
  }
 } else {
  label = 28;
 }
 if ((label|0) == 28) {
  $289 = $0;
  $290 = HEAP32[$289>>2]|0;
  $291 = (($290) + 20|0);
  HEAP32[$291>>2] = 77;
  $292 = $totallen;
  $293 = $0;
  $294 = HEAP32[$293>>2]|0;
  $295 = (($294) + 24|0);
  HEAP32[$295>>2] = $292;
  $296 = $0;
  $297 = HEAP32[$296>>2]|0;
  $298 = (($297) + 4|0);
  $299 = HEAP32[$298>>2]|0;
  $300 = $0;
  FUNCTION_TABLE_vii[$299 & 255]($300,1);
 }
 STACKTOP = sp;return;
}
function _examine_app14($cinfo,$data,$datalen,$remaining) {
 $cinfo = $cinfo|0;
 $data = $data|0;
 $datalen = $datalen|0;
 $remaining = $remaining|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $flags0 = 0, $flags1 = 0, $transform = 0, $version = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $data;
 $2 = $datalen;
 $3 = $remaining;
 $4 = $2;
 $5 = ($4>>>0)>=(12);
 if ($5) {
  $6 = $1;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(65);
  if ($9) {
   $10 = $1;
   $11 = (($10) + 1|0);
   $12 = HEAP8[$11>>0]|0;
   $13 = $12&255;
   $14 = ($13|0)==(100);
   if ($14) {
    $15 = $1;
    $16 = (($15) + 2|0);
    $17 = HEAP8[$16>>0]|0;
    $18 = $17&255;
    $19 = ($18|0)==(111);
    if ($19) {
     $20 = $1;
     $21 = (($20) + 3|0);
     $22 = HEAP8[$21>>0]|0;
     $23 = $22&255;
     $24 = ($23|0)==(98);
     if ($24) {
      $25 = $1;
      $26 = (($25) + 4|0);
      $27 = HEAP8[$26>>0]|0;
      $28 = $27&255;
      $29 = ($28|0)==(101);
      if ($29) {
       $30 = $1;
       $31 = (($30) + 5|0);
       $32 = HEAP8[$31>>0]|0;
       $33 = $32&255;
       $34 = $33 << 8;
       $35 = $1;
       $36 = (($35) + 6|0);
       $37 = HEAP8[$36>>0]|0;
       $38 = $37&255;
       $39 = (($34) + ($38))|0;
       $version = $39;
       $40 = $1;
       $41 = (($40) + 7|0);
       $42 = HEAP8[$41>>0]|0;
       $43 = $42&255;
       $44 = $43 << 8;
       $45 = $1;
       $46 = (($45) + 8|0);
       $47 = HEAP8[$46>>0]|0;
       $48 = $47&255;
       $49 = (($44) + ($48))|0;
       $flags0 = $49;
       $50 = $1;
       $51 = (($50) + 9|0);
       $52 = HEAP8[$51>>0]|0;
       $53 = $52&255;
       $54 = $53 << 8;
       $55 = $1;
       $56 = (($55) + 10|0);
       $57 = HEAP8[$56>>0]|0;
       $58 = $57&255;
       $59 = (($54) + ($58))|0;
       $flags1 = $59;
       $60 = $1;
       $61 = (($60) + 11|0);
       $62 = HEAP8[$61>>0]|0;
       $63 = $62&255;
       $transform = $63;
       $64 = $0;
       $65 = HEAP32[$64>>2]|0;
       $66 = (($65) + 24|0);
       $_mp = $66;
       $67 = $version;
       $68 = $_mp;
       HEAP32[$68>>2] = $67;
       $69 = $flags0;
       $70 = $_mp;
       $71 = (($70) + 4|0);
       HEAP32[$71>>2] = $69;
       $72 = $flags1;
       $73 = $_mp;
       $74 = (($73) + 8|0);
       HEAP32[$74>>2] = $72;
       $75 = $transform;
       $76 = $_mp;
       $77 = (($76) + 12|0);
       HEAP32[$77>>2] = $75;
       $78 = $0;
       $79 = HEAP32[$78>>2]|0;
       $80 = (($79) + 20|0);
       HEAP32[$80>>2] = 76;
       $81 = $0;
       $82 = HEAP32[$81>>2]|0;
       $83 = (($82) + 4|0);
       $84 = HEAP32[$83>>2]|0;
       $85 = $0;
       FUNCTION_TABLE_vii[$84 & 255]($85,1);
       $86 = $0;
       $87 = (($86) + 292|0);
       HEAP32[$87>>2] = 1;
       $88 = $transform;
       $89 = $88&255;
       $90 = $0;
       $91 = (($90) + 296|0);
       HEAP8[$91>>0] = $89;
       STACKTOP = sp;return;
      }
     }
    }
   }
  }
 }
 $92 = $0;
 $93 = HEAP32[$92>>2]|0;
 $94 = (($93) + 20|0);
 HEAP32[$94>>2] = 78;
 $95 = $2;
 $96 = $3;
 $97 = (($95) + ($96))|0;
 $98 = $0;
 $99 = HEAP32[$98>>2]|0;
 $100 = (($99) + 24|0);
 HEAP32[$100>>2] = $97;
 $101 = $0;
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) + 4|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = $0;
 FUNCTION_TABLE_vii[$104 & 255]($105,1);
 STACKTOP = sp;return;
}
function _first_marker($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $bytes_in_buffer = 0, $c = 0, $c2 = 0;
 var $datasrc = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $76 = $0;
   STACKTOP = sp;return ($76|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $c = $28;
 $29 = $bytes_in_buffer;
 $30 = ($29|0)==(0);
 do {
  if ($30) {
   $31 = $datasrc;
   $32 = (($31) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $1;
   $35 = (FUNCTION_TABLE_ii[$33 & 127]($34)|0);
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = $datasrc;
    $38 = HEAP32[$37>>2]|0;
    $next_input_byte = $38;
    $39 = $datasrc;
    $40 = (($39) + 4|0);
    $41 = HEAP32[$40>>2]|0;
    $bytes_in_buffer = $41;
    break;
   }
   $0 = 0;
   $76 = $0;
   STACKTOP = sp;return ($76|0);
  }
 } while(0);
 $42 = $bytes_in_buffer;
 $43 = (($42) + -1)|0;
 $bytes_in_buffer = $43;
 $44 = $next_input_byte;
 $45 = (($44) + 1|0);
 $next_input_byte = $45;
 $46 = HEAP8[$44>>0]|0;
 $47 = $46&255;
 $c2 = $47;
 $48 = $c;
 $49 = ($48|0)!=(255);
 if ($49) {
  label = 15;
 } else {
  $50 = $c2;
  $51 = ($50|0)!=(216);
  if ($51) {
   label = 15;
  }
 }
 if ((label|0) == 15) {
  $52 = $1;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + 20|0);
  HEAP32[$54>>2] = 53;
  $55 = $c;
  $56 = $1;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 24|0);
  HEAP32[$58>>2] = $55;
  $59 = $c2;
  $60 = $1;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 24|0);
  $63 = (($62) + 4|0);
  HEAP32[$63>>2] = $59;
  $64 = $1;
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$65>>2]|0;
  $67 = $1;
  FUNCTION_TABLE_vi[$66 & 255]($67);
 }
 $68 = $c2;
 $69 = $1;
 $70 = (($69) + 416|0);
 HEAP32[$70>>2] = $68;
 $71 = $next_input_byte;
 $72 = $datasrc;
 HEAP32[$72>>2] = $71;
 $73 = $bytes_in_buffer;
 $74 = $datasrc;
 $75 = (($74) + 4|0);
 HEAP32[$75>>2] = $73;
 $0 = 1;
 $76 = $0;
 STACKTOP = sp;return ($76|0);
}
function _get_soi($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 HEAP32[$3>>2] = 102;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 FUNCTION_TABLE_vii[$7 & 255]($8,1);
 $9 = $0;
 $10 = (($9) + 440|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 20|0);
  HEAP32[$17>>2] = 61;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 255]($21);
 }
 $i = 0;
 while(1) {
  $22 = $i;
  $23 = ($22|0)<(16);
  if (!($23)) {
   break;
  }
  $24 = $i;
  $25 = $0;
  $26 = (($25) + 228|0);
  $27 = (($26) + ($24)|0);
  HEAP8[$27>>0] = 0;
  $28 = $i;
  $29 = $0;
  $30 = (($29) + 244|0);
  $31 = (($30) + ($28)|0);
  HEAP8[$31>>0] = 1;
  $32 = $i;
  $33 = $0;
  $34 = (($33) + 260|0);
  $35 = (($34) + ($32)|0);
  HEAP8[$35>>0] = 5;
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 $38 = $0;
 $39 = (($38) + 276|0);
 HEAP32[$39>>2] = 0;
 $40 = $0;
 $41 = (($40) + 40|0);
 HEAP32[$41>>2] = 0;
 $42 = $0;
 $43 = (($42) + 300|0);
 HEAP32[$43>>2] = 0;
 $44 = $0;
 $45 = (($44) + 280|0);
 HEAP32[$45>>2] = 0;
 $46 = $0;
 $47 = (($46) + 284|0);
 HEAP8[$47>>0] = 1;
 $48 = $0;
 $49 = (($48) + 285|0);
 HEAP8[$49>>0] = 1;
 $50 = $0;
 $51 = (($50) + 286|0);
 HEAP8[$51>>0] = 0;
 $52 = $0;
 $53 = (($52) + 288|0);
 HEAP16[$53>>1] = 1;
 $54 = $0;
 $55 = (($54) + 290|0);
 HEAP16[$55>>1] = 1;
 $56 = $0;
 $57 = (($56) + 292|0);
 HEAP32[$57>>2] = 0;
 $58 = $0;
 $59 = (($58) + 296|0);
 HEAP8[$59>>0] = 0;
 $60 = $0;
 $61 = (($60) + 440|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (($62) + 12|0);
 HEAP32[$63>>2] = 1;
 STACKTOP = sp;return 1;
}
function _get_sof($cinfo,$is_prog,$is_arith) {
 $cinfo = $cinfo|0;
 $is_prog = $is_prog|0;
 $is_arith = $is_arith|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $_mp1 = 0, $bytes_in_buffer = 0, $c = 0, $ci = 0, $compptr = 0, $datasrc = 0, $length = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $cinfo;
 $2 = $is_prog;
 $3 = $is_arith;
 $4 = $1;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $datasrc = $6;
 $7 = $datasrc;
 $8 = HEAP32[$7>>2]|0;
 $next_input_byte = $8;
 $9 = $datasrc;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $bytes_in_buffer = $11;
 $12 = $2;
 $13 = $1;
 $14 = (($13) + 220|0);
 HEAP32[$14>>2] = $12;
 $15 = $3;
 $16 = $1;
 $17 = (($16) + 224|0);
 HEAP32[$17>>2] = $15;
 $18 = $bytes_in_buffer;
 $19 = ($18|0)==(0);
 do {
  if ($19) {
   $20 = $datasrc;
   $21 = (($20) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = $1;
   $24 = (FUNCTION_TABLE_ii[$22 & 127]($23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $datasrc;
    $27 = HEAP32[$26>>2]|0;
    $next_input_byte = $27;
    $28 = $datasrc;
    $29 = (($28) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $bytes_in_buffer = $30;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $31 = $bytes_in_buffer;
 $32 = (($31) + -1)|0;
 $bytes_in_buffer = $32;
 $33 = $next_input_byte;
 $34 = (($33) + 1|0);
 $next_input_byte = $34;
 $35 = HEAP8[$33>>0]|0;
 $36 = $35&255;
 $37 = $36 << 8;
 $length = $37;
 $38 = $bytes_in_buffer;
 $39 = ($38|0)==(0);
 do {
  if ($39) {
   $40 = $datasrc;
   $41 = (($40) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = $1;
   $44 = (FUNCTION_TABLE_ii[$42 & 127]($43)|0);
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = $datasrc;
    $47 = HEAP32[$46>>2]|0;
    $next_input_byte = $47;
    $48 = $datasrc;
    $49 = (($48) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $bytes_in_buffer = $50;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $51 = $bytes_in_buffer;
 $52 = (($51) + -1)|0;
 $bytes_in_buffer = $52;
 $53 = $next_input_byte;
 $54 = (($53) + 1|0);
 $next_input_byte = $54;
 $55 = HEAP8[$53>>0]|0;
 $56 = $55&255;
 $57 = $length;
 $58 = (($57) + ($56))|0;
 $length = $58;
 $59 = $bytes_in_buffer;
 $60 = ($59|0)==(0);
 do {
  if ($60) {
   $61 = $datasrc;
   $62 = (($61) + 12|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $1;
   $65 = (FUNCTION_TABLE_ii[$63 & 127]($64)|0);
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = $datasrc;
    $68 = HEAP32[$67>>2]|0;
    $next_input_byte = $68;
    $69 = $datasrc;
    $70 = (($69) + 4|0);
    $71 = HEAP32[$70>>2]|0;
    $bytes_in_buffer = $71;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $72 = $bytes_in_buffer;
 $73 = (($72) + -1)|0;
 $bytes_in_buffer = $73;
 $74 = $next_input_byte;
 $75 = (($74) + 1|0);
 $next_input_byte = $75;
 $76 = HEAP8[$74>>0]|0;
 $77 = $76&255;
 $78 = $1;
 $79 = (($78) + 212|0);
 HEAP32[$79>>2] = $77;
 $80 = $bytes_in_buffer;
 $81 = ($80|0)==(0);
 do {
  if ($81) {
   $82 = $datasrc;
   $83 = (($82) + 12|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = $1;
   $86 = (FUNCTION_TABLE_ii[$84 & 127]($85)|0);
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = $datasrc;
    $89 = HEAP32[$88>>2]|0;
    $next_input_byte = $89;
    $90 = $datasrc;
    $91 = (($90) + 4|0);
    $92 = HEAP32[$91>>2]|0;
    $bytes_in_buffer = $92;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $93 = $bytes_in_buffer;
 $94 = (($93) + -1)|0;
 $bytes_in_buffer = $94;
 $95 = $next_input_byte;
 $96 = (($95) + 1|0);
 $next_input_byte = $96;
 $97 = HEAP8[$95>>0]|0;
 $98 = $97&255;
 $99 = $98 << 8;
 $100 = $1;
 $101 = (($100) + 32|0);
 HEAP32[$101>>2] = $99;
 $102 = $bytes_in_buffer;
 $103 = ($102|0)==(0);
 do {
  if ($103) {
   $104 = $datasrc;
   $105 = (($104) + 12|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = $1;
   $108 = (FUNCTION_TABLE_ii[$106 & 127]($107)|0);
   $109 = ($108|0)!=(0);
   if ($109) {
    $110 = $datasrc;
    $111 = HEAP32[$110>>2]|0;
    $next_input_byte = $111;
    $112 = $datasrc;
    $113 = (($112) + 4|0);
    $114 = HEAP32[$113>>2]|0;
    $bytes_in_buffer = $114;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $115 = $bytes_in_buffer;
 $116 = (($115) + -1)|0;
 $bytes_in_buffer = $116;
 $117 = $next_input_byte;
 $118 = (($117) + 1|0);
 $next_input_byte = $118;
 $119 = HEAP8[$117>>0]|0;
 $120 = $119&255;
 $121 = $1;
 $122 = (($121) + 32|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = (($123) + ($120))|0;
 HEAP32[$122>>2] = $124;
 $125 = $bytes_in_buffer;
 $126 = ($125|0)==(0);
 do {
  if ($126) {
   $127 = $datasrc;
   $128 = (($127) + 12|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = $1;
   $131 = (FUNCTION_TABLE_ii[$129 & 127]($130)|0);
   $132 = ($131|0)!=(0);
   if ($132) {
    $133 = $datasrc;
    $134 = HEAP32[$133>>2]|0;
    $next_input_byte = $134;
    $135 = $datasrc;
    $136 = (($135) + 4|0);
    $137 = HEAP32[$136>>2]|0;
    $bytes_in_buffer = $137;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $138 = $bytes_in_buffer;
 $139 = (($138) + -1)|0;
 $bytes_in_buffer = $139;
 $140 = $next_input_byte;
 $141 = (($140) + 1|0);
 $next_input_byte = $141;
 $142 = HEAP8[$140>>0]|0;
 $143 = $142&255;
 $144 = $143 << 8;
 $145 = $1;
 $146 = (($145) + 28|0);
 HEAP32[$146>>2] = $144;
 $147 = $bytes_in_buffer;
 $148 = ($147|0)==(0);
 do {
  if ($148) {
   $149 = $datasrc;
   $150 = (($149) + 12|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = $1;
   $153 = (FUNCTION_TABLE_ii[$151 & 127]($152)|0);
   $154 = ($153|0)!=(0);
   if ($154) {
    $155 = $datasrc;
    $156 = HEAP32[$155>>2]|0;
    $next_input_byte = $156;
    $157 = $datasrc;
    $158 = (($157) + 4|0);
    $159 = HEAP32[$158>>2]|0;
    $bytes_in_buffer = $159;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $160 = $bytes_in_buffer;
 $161 = (($160) + -1)|0;
 $bytes_in_buffer = $161;
 $162 = $next_input_byte;
 $163 = (($162) + 1|0);
 $next_input_byte = $163;
 $164 = HEAP8[$162>>0]|0;
 $165 = $164&255;
 $166 = $1;
 $167 = (($166) + 28|0);
 $168 = HEAP32[$167>>2]|0;
 $169 = (($168) + ($165))|0;
 HEAP32[$167>>2] = $169;
 $170 = $bytes_in_buffer;
 $171 = ($170|0)==(0);
 do {
  if ($171) {
   $172 = $datasrc;
   $173 = (($172) + 12|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = $1;
   $176 = (FUNCTION_TABLE_ii[$174 & 127]($175)|0);
   $177 = ($176|0)!=(0);
   if ($177) {
    $178 = $datasrc;
    $179 = HEAP32[$178>>2]|0;
    $next_input_byte = $179;
    $180 = $datasrc;
    $181 = (($180) + 4|0);
    $182 = HEAP32[$181>>2]|0;
    $bytes_in_buffer = $182;
    break;
   }
   $0 = 0;
   $406 = $0;
   STACKTOP = sp;return ($406|0);
  }
 } while(0);
 $183 = $bytes_in_buffer;
 $184 = (($183) + -1)|0;
 $bytes_in_buffer = $184;
 $185 = $next_input_byte;
 $186 = (($185) + 1|0);
 $next_input_byte = $186;
 $187 = HEAP8[$185>>0]|0;
 $188 = $187&255;
 $189 = $1;
 $190 = (($189) + 36|0);
 HEAP32[$190>>2] = $188;
 $191 = $length;
 $192 = (($191) - 8)|0;
 $length = $192;
 $193 = $1;
 $194 = HEAP32[$193>>2]|0;
 $195 = (($194) + 24|0);
 $_mp = $195;
 $196 = $1;
 $197 = (($196) + 416|0);
 $198 = HEAP32[$197>>2]|0;
 $199 = $_mp;
 HEAP32[$199>>2] = $198;
 $200 = $1;
 $201 = (($200) + 28|0);
 $202 = HEAP32[$201>>2]|0;
 $203 = $_mp;
 $204 = (($203) + 4|0);
 HEAP32[$204>>2] = $202;
 $205 = $1;
 $206 = (($205) + 32|0);
 $207 = HEAP32[$206>>2]|0;
 $208 = $_mp;
 $209 = (($208) + 8|0);
 HEAP32[$209>>2] = $207;
 $210 = $1;
 $211 = (($210) + 36|0);
 $212 = HEAP32[$211>>2]|0;
 $213 = $_mp;
 $214 = (($213) + 12|0);
 HEAP32[$214>>2] = $212;
 $215 = $1;
 $216 = HEAP32[$215>>2]|0;
 $217 = (($216) + 20|0);
 HEAP32[$217>>2] = 100;
 $218 = $1;
 $219 = HEAP32[$218>>2]|0;
 $220 = (($219) + 4|0);
 $221 = HEAP32[$220>>2]|0;
 $222 = $1;
 FUNCTION_TABLE_vii[$221 & 255]($222,1);
 $223 = $1;
 $224 = (($223) + 440|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + 16|0);
 $227 = HEAP32[$226>>2]|0;
 $228 = ($227|0)!=(0);
 if ($228) {
  $229 = $1;
  $230 = HEAP32[$229>>2]|0;
  $231 = (($230) + 20|0);
  HEAP32[$231>>2] = 58;
  $232 = $1;
  $233 = HEAP32[$232>>2]|0;
  $234 = HEAP32[$233>>2]|0;
  $235 = $1;
  FUNCTION_TABLE_vi[$234 & 255]($235);
 }
 $236 = $1;
 $237 = (($236) + 32|0);
 $238 = HEAP32[$237>>2]|0;
 $239 = ($238>>>0)<=(0);
 if ($239) {
  label = 50;
 } else {
  $240 = $1;
  $241 = (($240) + 28|0);
  $242 = HEAP32[$241>>2]|0;
  $243 = ($242>>>0)<=(0);
  if ($243) {
   label = 50;
  } else {
   $244 = $1;
   $245 = (($244) + 36|0);
   $246 = HEAP32[$245>>2]|0;
   $247 = ($246|0)<=(0);
   if ($247) {
    label = 50;
   }
  }
 }
 if ((label|0) == 50) {
  $248 = $1;
  $249 = HEAP32[$248>>2]|0;
  $250 = (($249) + 20|0);
  HEAP32[$250>>2] = 32;
  $251 = $1;
  $252 = HEAP32[$251>>2]|0;
  $253 = HEAP32[$252>>2]|0;
  $254 = $1;
  FUNCTION_TABLE_vi[$253 & 255]($254);
 }
 $255 = $length;
 $256 = $1;
 $257 = (($256) + 36|0);
 $258 = HEAP32[$257>>2]|0;
 $259 = ($258*3)|0;
 $260 = ($255|0)!=($259|0);
 if ($260) {
  $261 = $1;
  $262 = HEAP32[$261>>2]|0;
  $263 = (($262) + 20|0);
  HEAP32[$263>>2] = 11;
  $264 = $1;
  $265 = HEAP32[$264>>2]|0;
  $266 = HEAP32[$265>>2]|0;
  $267 = $1;
  FUNCTION_TABLE_vi[$266 & 255]($267);
 }
 $268 = $1;
 $269 = (($268) + 216|0);
 $270 = HEAP32[$269>>2]|0;
 $271 = ($270|0)==(0|0);
 if ($271) {
  $272 = $1;
  $273 = (($272) + 4|0);
  $274 = HEAP32[$273>>2]|0;
  $275 = HEAP32[$274>>2]|0;
  $276 = $1;
  $277 = $1;
  $278 = (($277) + 36|0);
  $279 = HEAP32[$278>>2]|0;
  $280 = ($279*84)|0;
  $281 = (FUNCTION_TABLE_iiii[$275 & 255]($276,1,$280)|0);
  $282 = $1;
  $283 = (($282) + 216|0);
  HEAP32[$283>>2] = $281;
 }
 $ci = 0;
 $284 = $1;
 $285 = (($284) + 216|0);
 $286 = HEAP32[$285>>2]|0;
 $compptr = $286;
 while(1) {
  $287 = $ci;
  $288 = $1;
  $289 = (($288) + 36|0);
  $290 = HEAP32[$289>>2]|0;
  $291 = ($287|0)<($290|0);
  if (!($291)) {
   label = 79;
   break;
  }
  $292 = $ci;
  $293 = $compptr;
  $294 = (($293) + 4|0);
  HEAP32[$294>>2] = $292;
  $295 = $bytes_in_buffer;
  $296 = ($295|0)==(0);
  if ($296) {
   $297 = $datasrc;
   $298 = (($297) + 12|0);
   $299 = HEAP32[$298>>2]|0;
   $300 = $1;
   $301 = (FUNCTION_TABLE_ii[$299 & 127]($300)|0);
   $302 = ($301|0)!=(0);
   if (!($302)) {
    label = 60;
    break;
   }
   $303 = $datasrc;
   $304 = HEAP32[$303>>2]|0;
   $next_input_byte = $304;
   $305 = $datasrc;
   $306 = (($305) + 4|0);
   $307 = HEAP32[$306>>2]|0;
   $bytes_in_buffer = $307;
  }
  $308 = $bytes_in_buffer;
  $309 = (($308) + -1)|0;
  $bytes_in_buffer = $309;
  $310 = $next_input_byte;
  $311 = (($310) + 1|0);
  $next_input_byte = $311;
  $312 = HEAP8[$310>>0]|0;
  $313 = $312&255;
  $314 = $compptr;
  HEAP32[$314>>2] = $313;
  $315 = $bytes_in_buffer;
  $316 = ($315|0)==(0);
  if ($316) {
   $317 = $datasrc;
   $318 = (($317) + 12|0);
   $319 = HEAP32[$318>>2]|0;
   $320 = $1;
   $321 = (FUNCTION_TABLE_ii[$319 & 127]($320)|0);
   $322 = ($321|0)!=(0);
   if (!($322)) {
    label = 66;
    break;
   }
   $323 = $datasrc;
   $324 = HEAP32[$323>>2]|0;
   $next_input_byte = $324;
   $325 = $datasrc;
   $326 = (($325) + 4|0);
   $327 = HEAP32[$326>>2]|0;
   $bytes_in_buffer = $327;
  }
  $328 = $bytes_in_buffer;
  $329 = (($328) + -1)|0;
  $bytes_in_buffer = $329;
  $330 = $next_input_byte;
  $331 = (($330) + 1|0);
  $next_input_byte = $331;
  $332 = HEAP8[$330>>0]|0;
  $333 = $332&255;
  $c = $333;
  $334 = $c;
  $335 = $334 >> 4;
  $336 = $335 & 15;
  $337 = $compptr;
  $338 = (($337) + 8|0);
  HEAP32[$338>>2] = $336;
  $339 = $c;
  $340 = $339 & 15;
  $341 = $compptr;
  $342 = (($341) + 12|0);
  HEAP32[$342>>2] = $340;
  $343 = $bytes_in_buffer;
  $344 = ($343|0)==(0);
  if ($344) {
   $345 = $datasrc;
   $346 = (($345) + 12|0);
   $347 = HEAP32[$346>>2]|0;
   $348 = $1;
   $349 = (FUNCTION_TABLE_ii[$347 & 127]($348)|0);
   $350 = ($349|0)!=(0);
   if (!($350)) {
    label = 72;
    break;
   }
   $351 = $datasrc;
   $352 = HEAP32[$351>>2]|0;
   $next_input_byte = $352;
   $353 = $datasrc;
   $354 = (($353) + 4|0);
   $355 = HEAP32[$354>>2]|0;
   $bytes_in_buffer = $355;
  }
  $356 = $bytes_in_buffer;
  $357 = (($356) + -1)|0;
  $bytes_in_buffer = $357;
  $358 = $next_input_byte;
  $359 = (($358) + 1|0);
  $next_input_byte = $359;
  $360 = HEAP8[$358>>0]|0;
  $361 = $360&255;
  $362 = $compptr;
  $363 = (($362) + 16|0);
  HEAP32[$363>>2] = $361;
  $364 = $1;
  $365 = HEAP32[$364>>2]|0;
  $366 = (($365) + 24|0);
  $_mp1 = $366;
  $367 = $compptr;
  $368 = HEAP32[$367>>2]|0;
  $369 = $_mp1;
  HEAP32[$369>>2] = $368;
  $370 = $compptr;
  $371 = (($370) + 8|0);
  $372 = HEAP32[$371>>2]|0;
  $373 = $_mp1;
  $374 = (($373) + 4|0);
  HEAP32[$374>>2] = $372;
  $375 = $compptr;
  $376 = (($375) + 12|0);
  $377 = HEAP32[$376>>2]|0;
  $378 = $_mp1;
  $379 = (($378) + 8|0);
  HEAP32[$379>>2] = $377;
  $380 = $compptr;
  $381 = (($380) + 16|0);
  $382 = HEAP32[$381>>2]|0;
  $383 = $_mp1;
  $384 = (($383) + 12|0);
  HEAP32[$384>>2] = $382;
  $385 = $1;
  $386 = HEAP32[$385>>2]|0;
  $387 = (($386) + 20|0);
  HEAP32[$387>>2] = 101;
  $388 = $1;
  $389 = HEAP32[$388>>2]|0;
  $390 = (($389) + 4|0);
  $391 = HEAP32[$390>>2]|0;
  $392 = $1;
  FUNCTION_TABLE_vii[$391 & 255]($392,1);
  $393 = $ci;
  $394 = (($393) + 1)|0;
  $ci = $394;
  $395 = $compptr;
  $396 = (($395) + 84|0);
  $compptr = $396;
 }
 if ((label|0) == 60) {
  $0 = 0;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 else if ((label|0) == 66) {
  $0 = 0;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 else if ((label|0) == 72) {
  $0 = 0;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 else if ((label|0) == 79) {
  $397 = $1;
  $398 = (($397) + 440|0);
  $399 = HEAP32[$398>>2]|0;
  $400 = (($399) + 16|0);
  HEAP32[$400>>2] = 1;
  $401 = $next_input_byte;
  $402 = $datasrc;
  HEAP32[$402>>2] = $401;
  $403 = $bytes_in_buffer;
  $404 = $datasrc;
  $405 = (($404) + 4|0);
  HEAP32[$405>>2] = $403;
  $0 = 1;
  $406 = $0;
  STACKTOP = sp;return ($406|0);
 }
 return 0|0;
}
function _get_sos($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $_mp1 = 0, $bytes_in_buffer = 0;
 var $c = 0, $cc = 0, $ci = 0, $compptr = 0, $datasrc = 0, $i = 0, $length = 0, $n = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $1;
 $11 = (($10) + 440|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0);
 if (!($15)) {
  $16 = $1;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 20|0);
  HEAP32[$18>>2] = 62;
  $19 = $1;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = $1;
  FUNCTION_TABLE_vi[$21 & 255]($22);
 }
 $23 = $bytes_in_buffer;
 $24 = ($23|0)==(0);
 do {
  if ($24) {
   $25 = $datasrc;
   $26 = (($25) + 12|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $1;
   $29 = (FUNCTION_TABLE_ii[$27 & 127]($28)|0);
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = $datasrc;
    $32 = HEAP32[$31>>2]|0;
    $next_input_byte = $32;
    $33 = $datasrc;
    $34 = (($33) + 4|0);
    $35 = HEAP32[$34>>2]|0;
    $bytes_in_buffer = $35;
    break;
   }
   $0 = 0;
   $336 = $0;
   STACKTOP = sp;return ($336|0);
  }
 } while(0);
 $36 = $bytes_in_buffer;
 $37 = (($36) + -1)|0;
 $bytes_in_buffer = $37;
 $38 = $next_input_byte;
 $39 = (($38) + 1|0);
 $next_input_byte = $39;
 $40 = HEAP8[$38>>0]|0;
 $41 = $40&255;
 $42 = $41 << 8;
 $length = $42;
 $43 = $bytes_in_buffer;
 $44 = ($43|0)==(0);
 do {
  if ($44) {
   $45 = $datasrc;
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $1;
   $49 = (FUNCTION_TABLE_ii[$47 & 127]($48)|0);
   $50 = ($49|0)!=(0);
   if ($50) {
    $51 = $datasrc;
    $52 = HEAP32[$51>>2]|0;
    $next_input_byte = $52;
    $53 = $datasrc;
    $54 = (($53) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $bytes_in_buffer = $55;
    break;
   }
   $0 = 0;
   $336 = $0;
   STACKTOP = sp;return ($336|0);
  }
 } while(0);
 $56 = $bytes_in_buffer;
 $57 = (($56) + -1)|0;
 $bytes_in_buffer = $57;
 $58 = $next_input_byte;
 $59 = (($58) + 1|0);
 $next_input_byte = $59;
 $60 = HEAP8[$58>>0]|0;
 $61 = $60&255;
 $62 = $length;
 $63 = (($62) + ($61))|0;
 $length = $63;
 $64 = $bytes_in_buffer;
 $65 = ($64|0)==(0);
 do {
  if ($65) {
   $66 = $datasrc;
   $67 = (($66) + 12|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = $1;
   $70 = (FUNCTION_TABLE_ii[$68 & 127]($69)|0);
   $71 = ($70|0)!=(0);
   if ($71) {
    $72 = $datasrc;
    $73 = HEAP32[$72>>2]|0;
    $next_input_byte = $73;
    $74 = $datasrc;
    $75 = (($74) + 4|0);
    $76 = HEAP32[$75>>2]|0;
    $bytes_in_buffer = $76;
    break;
   }
   $0 = 0;
   $336 = $0;
   STACKTOP = sp;return ($336|0);
  }
 } while(0);
 $77 = $bytes_in_buffer;
 $78 = (($77) + -1)|0;
 $bytes_in_buffer = $78;
 $79 = $next_input_byte;
 $80 = (($79) + 1|0);
 $next_input_byte = $80;
 $81 = HEAP8[$79>>0]|0;
 $82 = $81&255;
 $n = $82;
 $83 = $1;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($84) + 20|0);
 HEAP32[$85>>2] = 103;
 $86 = $n;
 $87 = $1;
 $88 = HEAP32[$87>>2]|0;
 $89 = (($88) + 24|0);
 HEAP32[$89>>2] = $86;
 $90 = $1;
 $91 = HEAP32[$90>>2]|0;
 $92 = (($91) + 4|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = $1;
 FUNCTION_TABLE_vii[$93 & 255]($94,1);
 $95 = $length;
 $96 = $n;
 $97 = $96<<1;
 $98 = (($97) + 6)|0;
 $99 = ($95|0)!=($98|0);
 if ($99) {
  label = 22;
 } else {
  $100 = $n;
  $101 = ($100|0)<(1);
  if ($101) {
   label = 22;
  } else {
   $102 = $n;
   $103 = ($102|0)>(4);
   if ($103) {
    label = 22;
   }
  }
 }
 if ((label|0) == 22) {
  $104 = $1;
  $105 = HEAP32[$104>>2]|0;
  $106 = (($105) + 20|0);
  HEAP32[$106>>2] = 11;
  $107 = $1;
  $108 = HEAP32[$107>>2]|0;
  $109 = HEAP32[$108>>2]|0;
  $110 = $1;
  FUNCTION_TABLE_vi[$109 & 255]($110);
 }
 $111 = $n;
 $112 = $1;
 $113 = (($112) + 328|0);
 HEAP32[$113>>2] = $111;
 $i = 0;
 while(1) {
  $114 = $i;
  $115 = $n;
  $116 = ($114|0)<($115|0);
  if (!($116)) {
   label = 48;
   break;
  }
  $117 = $bytes_in_buffer;
  $118 = ($117|0)==(0);
  if ($118) {
   $119 = $datasrc;
   $120 = (($119) + 12|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = $1;
   $123 = (FUNCTION_TABLE_ii[$121 & 127]($122)|0);
   $124 = ($123|0)!=(0);
   if (!($124)) {
    label = 28;
    break;
   }
   $125 = $datasrc;
   $126 = HEAP32[$125>>2]|0;
   $next_input_byte = $126;
   $127 = $datasrc;
   $128 = (($127) + 4|0);
   $129 = HEAP32[$128>>2]|0;
   $bytes_in_buffer = $129;
  }
  $130 = $bytes_in_buffer;
  $131 = (($130) + -1)|0;
  $bytes_in_buffer = $131;
  $132 = $next_input_byte;
  $133 = (($132) + 1|0);
  $next_input_byte = $133;
  $134 = HEAP8[$132>>0]|0;
  $135 = $134&255;
  $cc = $135;
  $136 = $bytes_in_buffer;
  $137 = ($136|0)==(0);
  if ($137) {
   $138 = $datasrc;
   $139 = (($138) + 12|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = $1;
   $142 = (FUNCTION_TABLE_ii[$140 & 127]($141)|0);
   $143 = ($142|0)!=(0);
   if (!($143)) {
    label = 34;
    break;
   }
   $144 = $datasrc;
   $145 = HEAP32[$144>>2]|0;
   $next_input_byte = $145;
   $146 = $datasrc;
   $147 = (($146) + 4|0);
   $148 = HEAP32[$147>>2]|0;
   $bytes_in_buffer = $148;
  }
  $149 = $bytes_in_buffer;
  $150 = (($149) + -1)|0;
  $bytes_in_buffer = $150;
  $151 = $next_input_byte;
  $152 = (($151) + 1|0);
  $next_input_byte = $152;
  $153 = HEAP8[$151>>0]|0;
  $154 = $153&255;
  $c = $154;
  $ci = 0;
  $155 = $1;
  $156 = (($155) + 216|0);
  $157 = HEAP32[$156>>2]|0;
  $compptr = $157;
  while(1) {
   $158 = $ci;
   $159 = $1;
   $160 = (($159) + 36|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = ($158|0)<($161|0);
   if (!($162)) {
    label = 43;
    break;
   }
   $163 = $cc;
   $164 = $compptr;
   $165 = HEAP32[$164>>2]|0;
   $166 = ($163|0)==($165|0);
   if ($166) {
    label = 40;
    break;
   }
   $167 = $ci;
   $168 = (($167) + 1)|0;
   $ci = $168;
   $169 = $compptr;
   $170 = (($169) + 84|0);
   $compptr = $170;
  }
  if ((label|0) == 40) {
   label = 0;
  }
  else if ((label|0) == 43) {
   label = 0;
   $171 = $1;
   $172 = HEAP32[$171>>2]|0;
   $173 = (($172) + 20|0);
   HEAP32[$173>>2] = 5;
   $174 = $cc;
   $175 = $1;
   $176 = HEAP32[$175>>2]|0;
   $177 = (($176) + 24|0);
   HEAP32[$177>>2] = $174;
   $178 = $1;
   $179 = HEAP32[$178>>2]|0;
   $180 = HEAP32[$179>>2]|0;
   $181 = $1;
   FUNCTION_TABLE_vi[$180 & 255]($181);
  }
  $182 = $compptr;
  $183 = $i;
  $184 = $1;
  $185 = (($184) + 332|0);
  $186 = (($185) + ($183<<2)|0);
  HEAP32[$186>>2] = $182;
  $187 = $c;
  $188 = $187 >> 4;
  $189 = $188 & 15;
  $190 = $compptr;
  $191 = (($190) + 20|0);
  HEAP32[$191>>2] = $189;
  $192 = $c;
  $193 = $192 & 15;
  $194 = $compptr;
  $195 = (($194) + 24|0);
  HEAP32[$195>>2] = $193;
  $196 = $1;
  $197 = HEAP32[$196>>2]|0;
  $198 = (($197) + 24|0);
  $_mp = $198;
  $199 = $cc;
  $200 = $_mp;
  HEAP32[$200>>2] = $199;
  $201 = $compptr;
  $202 = (($201) + 20|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = $_mp;
  $205 = (($204) + 4|0);
  HEAP32[$205>>2] = $203;
  $206 = $compptr;
  $207 = (($206) + 24|0);
  $208 = HEAP32[$207>>2]|0;
  $209 = $_mp;
  $210 = (($209) + 8|0);
  HEAP32[$210>>2] = $208;
  $211 = $1;
  $212 = HEAP32[$211>>2]|0;
  $213 = (($212) + 20|0);
  HEAP32[$213>>2] = 104;
  $214 = $1;
  $215 = HEAP32[$214>>2]|0;
  $216 = (($215) + 4|0);
  $217 = HEAP32[$216>>2]|0;
  $218 = $1;
  FUNCTION_TABLE_vii[$217 & 255]($218,1);
  $219 = $i;
  $220 = (($219) + 1)|0;
  $i = $220;
 }
 if ((label|0) == 28) {
  $0 = 0;
  $336 = $0;
  STACKTOP = sp;return ($336|0);
 }
 else if ((label|0) == 34) {
  $0 = 0;
  $336 = $0;
  STACKTOP = sp;return ($336|0);
 }
 else if ((label|0) == 48) {
  $221 = $bytes_in_buffer;
  $222 = ($221|0)==(0);
  do {
   if ($222) {
    $223 = $datasrc;
    $224 = (($223) + 12|0);
    $225 = HEAP32[$224>>2]|0;
    $226 = $1;
    $227 = (FUNCTION_TABLE_ii[$225 & 127]($226)|0);
    $228 = ($227|0)!=(0);
    if ($228) {
     $229 = $datasrc;
     $230 = HEAP32[$229>>2]|0;
     $next_input_byte = $230;
     $231 = $datasrc;
     $232 = (($231) + 4|0);
     $233 = HEAP32[$232>>2]|0;
     $bytes_in_buffer = $233;
     break;
    }
    $0 = 0;
    $336 = $0;
    STACKTOP = sp;return ($336|0);
   }
  } while(0);
  $234 = $bytes_in_buffer;
  $235 = (($234) + -1)|0;
  $bytes_in_buffer = $235;
  $236 = $next_input_byte;
  $237 = (($236) + 1|0);
  $next_input_byte = $237;
  $238 = HEAP8[$236>>0]|0;
  $239 = $238&255;
  $c = $239;
  $240 = $c;
  $241 = $1;
  $242 = (($241) + 400|0);
  HEAP32[$242>>2] = $240;
  $243 = $bytes_in_buffer;
  $244 = ($243|0)==(0);
  do {
   if ($244) {
    $245 = $datasrc;
    $246 = (($245) + 12|0);
    $247 = HEAP32[$246>>2]|0;
    $248 = $1;
    $249 = (FUNCTION_TABLE_ii[$247 & 127]($248)|0);
    $250 = ($249|0)!=(0);
    if ($250) {
     $251 = $datasrc;
     $252 = HEAP32[$251>>2]|0;
     $next_input_byte = $252;
     $253 = $datasrc;
     $254 = (($253) + 4|0);
     $255 = HEAP32[$254>>2]|0;
     $bytes_in_buffer = $255;
     break;
    }
    $0 = 0;
    $336 = $0;
    STACKTOP = sp;return ($336|0);
   }
  } while(0);
  $256 = $bytes_in_buffer;
  $257 = (($256) + -1)|0;
  $bytes_in_buffer = $257;
  $258 = $next_input_byte;
  $259 = (($258) + 1|0);
  $next_input_byte = $259;
  $260 = HEAP8[$258>>0]|0;
  $261 = $260&255;
  $c = $261;
  $262 = $c;
  $263 = $1;
  $264 = (($263) + 404|0);
  HEAP32[$264>>2] = $262;
  $265 = $bytes_in_buffer;
  $266 = ($265|0)==(0);
  do {
   if ($266) {
    $267 = $datasrc;
    $268 = (($267) + 12|0);
    $269 = HEAP32[$268>>2]|0;
    $270 = $1;
    $271 = (FUNCTION_TABLE_ii[$269 & 127]($270)|0);
    $272 = ($271|0)!=(0);
    if ($272) {
     $273 = $datasrc;
     $274 = HEAP32[$273>>2]|0;
     $next_input_byte = $274;
     $275 = $datasrc;
     $276 = (($275) + 4|0);
     $277 = HEAP32[$276>>2]|0;
     $bytes_in_buffer = $277;
     break;
    }
    $0 = 0;
    $336 = $0;
    STACKTOP = sp;return ($336|0);
   }
  } while(0);
  $278 = $bytes_in_buffer;
  $279 = (($278) + -1)|0;
  $bytes_in_buffer = $279;
  $280 = $next_input_byte;
  $281 = (($280) + 1|0);
  $next_input_byte = $281;
  $282 = HEAP8[$280>>0]|0;
  $283 = $282&255;
  $c = $283;
  $284 = $c;
  $285 = $284 >> 4;
  $286 = $285 & 15;
  $287 = $1;
  $288 = (($287) + 408|0);
  HEAP32[$288>>2] = $286;
  $289 = $c;
  $290 = $289 & 15;
  $291 = $1;
  $292 = (($291) + 412|0);
  HEAP32[$292>>2] = $290;
  $293 = $1;
  $294 = HEAP32[$293>>2]|0;
  $295 = (($294) + 24|0);
  $_mp1 = $295;
  $296 = $1;
  $297 = (($296) + 400|0);
  $298 = HEAP32[$297>>2]|0;
  $299 = $_mp1;
  HEAP32[$299>>2] = $298;
  $300 = $1;
  $301 = (($300) + 404|0);
  $302 = HEAP32[$301>>2]|0;
  $303 = $_mp1;
  $304 = (($303) + 4|0);
  HEAP32[$304>>2] = $302;
  $305 = $1;
  $306 = (($305) + 408|0);
  $307 = HEAP32[$306>>2]|0;
  $308 = $_mp1;
  $309 = (($308) + 8|0);
  HEAP32[$309>>2] = $307;
  $310 = $1;
  $311 = (($310) + 412|0);
  $312 = HEAP32[$311>>2]|0;
  $313 = $_mp1;
  $314 = (($313) + 12|0);
  HEAP32[$314>>2] = $312;
  $315 = $1;
  $316 = HEAP32[$315>>2]|0;
  $317 = (($316) + 20|0);
  HEAP32[$317>>2] = 105;
  $318 = $1;
  $319 = HEAP32[$318>>2]|0;
  $320 = (($319) + 4|0);
  $321 = HEAP32[$320>>2]|0;
  $322 = $1;
  FUNCTION_TABLE_vii[$321 & 255]($322,1);
  $323 = $1;
  $324 = (($323) + 440|0);
  $325 = HEAP32[$324>>2]|0;
  $326 = (($325) + 20|0);
  HEAP32[$326>>2] = 0;
  $327 = $1;
  $328 = (($327) + 144|0);
  $329 = HEAP32[$328>>2]|0;
  $330 = (($329) + 1)|0;
  HEAP32[$328>>2] = $330;
  $331 = $next_input_byte;
  $332 = $datasrc;
  HEAP32[$332>>2] = $331;
  $333 = $bytes_in_buffer;
  $334 = $datasrc;
  $335 = (($334) + 4|0);
  HEAP32[$335>>2] = $333;
  $0 = 1;
  $336 = $0;
  STACKTOP = sp;return ($336|0);
 }
 return 0|0;
}
function _get_dht($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $_mp1 = 0, $bits = 0, $bytes_in_buffer = 0, $count = 0, $datasrc = 0, $htblptr = 0, $huffval = 0;
 var $i = 0, $index = 0, $length = 0, $next_input_byte = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0;
 $bits = sp + 304|0;
 $huffval = sp + 48|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $311 = $0;
   STACKTOP = sp;return ($311|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $311 = $0;
   STACKTOP = sp;return ($311|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 L17: while(1) {
  $53 = $length;
  $54 = ($53|0)>(16);
  if (!($54)) {
   label = 55;
   break;
  }
  $55 = $bytes_in_buffer;
  $56 = ($55|0)==(0);
  if ($56) {
   $57 = $datasrc;
   $58 = (($57) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $1;
   $61 = (FUNCTION_TABLE_ii[$59 & 127]($60)|0);
   $62 = ($61|0)!=(0);
   if (!($62)) {
    label = 16;
    break;
   }
   $63 = $datasrc;
   $64 = HEAP32[$63>>2]|0;
   $next_input_byte = $64;
   $65 = $datasrc;
   $66 = (($65) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   $bytes_in_buffer = $67;
  }
  $68 = $bytes_in_buffer;
  $69 = (($68) + -1)|0;
  $bytes_in_buffer = $69;
  $70 = $next_input_byte;
  $71 = (($70) + 1|0);
  $next_input_byte = $71;
  $72 = HEAP8[$70>>0]|0;
  $73 = $72&255;
  $index = $73;
  $74 = $1;
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 20|0);
  HEAP32[$76>>2] = 80;
  $77 = $index;
  $78 = $1;
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + 24|0);
  HEAP32[$80>>2] = $77;
  $81 = $1;
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = $1;
  FUNCTION_TABLE_vii[$84 & 255]($85,1);
  HEAP8[$bits>>0] = 0;
  $count = 0;
  $i = 1;
  while(1) {
   $86 = $i;
   $87 = ($86|0)<=(16);
   if (!($87)) {
    break;
   }
   $88 = $bytes_in_buffer;
   $89 = ($88|0)==(0);
   if ($89) {
    $90 = $datasrc;
    $91 = (($90) + 12|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = $1;
    $94 = (FUNCTION_TABLE_ii[$92 & 127]($93)|0);
    $95 = ($94|0)!=(0);
    if (!($95)) {
     label = 24;
     break L17;
    }
    $96 = $datasrc;
    $97 = HEAP32[$96>>2]|0;
    $next_input_byte = $97;
    $98 = $datasrc;
    $99 = (($98) + 4|0);
    $100 = HEAP32[$99>>2]|0;
    $bytes_in_buffer = $100;
   }
   $101 = $bytes_in_buffer;
   $102 = (($101) + -1)|0;
   $bytes_in_buffer = $102;
   $103 = $next_input_byte;
   $104 = (($103) + 1|0);
   $next_input_byte = $104;
   $105 = HEAP8[$103>>0]|0;
   $106 = $i;
   $107 = (($bits) + ($106)|0);
   HEAP8[$107>>0] = $105;
   $108 = $i;
   $109 = (($bits) + ($108)|0);
   $110 = HEAP8[$109>>0]|0;
   $111 = $110&255;
   $112 = $count;
   $113 = (($112) + ($111))|0;
   $count = $113;
   $114 = $i;
   $115 = (($114) + 1)|0;
   $i = $115;
  }
  $116 = $length;
  $117 = (($116) - 17)|0;
  $length = $117;
  $118 = $1;
  $119 = HEAP32[$118>>2]|0;
  $120 = (($119) + 24|0);
  $_mp = $120;
  $121 = (($bits) + 1|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $_mp;
  HEAP32[$124>>2] = $123;
  $125 = (($bits) + 2|0);
  $126 = HEAP8[$125>>0]|0;
  $127 = $126&255;
  $128 = $_mp;
  $129 = (($128) + 4|0);
  HEAP32[$129>>2] = $127;
  $130 = (($bits) + 3|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = $131&255;
  $133 = $_mp;
  $134 = (($133) + 8|0);
  HEAP32[$134>>2] = $132;
  $135 = (($bits) + 4|0);
  $136 = HEAP8[$135>>0]|0;
  $137 = $136&255;
  $138 = $_mp;
  $139 = (($138) + 12|0);
  HEAP32[$139>>2] = $137;
  $140 = (($bits) + 5|0);
  $141 = HEAP8[$140>>0]|0;
  $142 = $141&255;
  $143 = $_mp;
  $144 = (($143) + 16|0);
  HEAP32[$144>>2] = $142;
  $145 = (($bits) + 6|0);
  $146 = HEAP8[$145>>0]|0;
  $147 = $146&255;
  $148 = $_mp;
  $149 = (($148) + 20|0);
  HEAP32[$149>>2] = $147;
  $150 = (($bits) + 7|0);
  $151 = HEAP8[$150>>0]|0;
  $152 = $151&255;
  $153 = $_mp;
  $154 = (($153) + 24|0);
  HEAP32[$154>>2] = $152;
  $155 = (($bits) + 8|0);
  $156 = HEAP8[$155>>0]|0;
  $157 = $156&255;
  $158 = $_mp;
  $159 = (($158) + 28|0);
  HEAP32[$159>>2] = $157;
  $160 = $1;
  $161 = HEAP32[$160>>2]|0;
  $162 = (($161) + 20|0);
  HEAP32[$162>>2] = 86;
  $163 = $1;
  $164 = HEAP32[$163>>2]|0;
  $165 = (($164) + 4|0);
  $166 = HEAP32[$165>>2]|0;
  $167 = $1;
  FUNCTION_TABLE_vii[$166 & 255]($167,2);
  $168 = $1;
  $169 = HEAP32[$168>>2]|0;
  $170 = (($169) + 24|0);
  $_mp1 = $170;
  $171 = (($bits) + 9|0);
  $172 = HEAP8[$171>>0]|0;
  $173 = $172&255;
  $174 = $_mp1;
  HEAP32[$174>>2] = $173;
  $175 = (($bits) + 10|0);
  $176 = HEAP8[$175>>0]|0;
  $177 = $176&255;
  $178 = $_mp1;
  $179 = (($178) + 4|0);
  HEAP32[$179>>2] = $177;
  $180 = (($bits) + 11|0);
  $181 = HEAP8[$180>>0]|0;
  $182 = $181&255;
  $183 = $_mp1;
  $184 = (($183) + 8|0);
  HEAP32[$184>>2] = $182;
  $185 = (($bits) + 12|0);
  $186 = HEAP8[$185>>0]|0;
  $187 = $186&255;
  $188 = $_mp1;
  $189 = (($188) + 12|0);
  HEAP32[$189>>2] = $187;
  $190 = (($bits) + 13|0);
  $191 = HEAP8[$190>>0]|0;
  $192 = $191&255;
  $193 = $_mp1;
  $194 = (($193) + 16|0);
  HEAP32[$194>>2] = $192;
  $195 = (($bits) + 14|0);
  $196 = HEAP8[$195>>0]|0;
  $197 = $196&255;
  $198 = $_mp1;
  $199 = (($198) + 20|0);
  HEAP32[$199>>2] = $197;
  $200 = (($bits) + 15|0);
  $201 = HEAP8[$200>>0]|0;
  $202 = $201&255;
  $203 = $_mp1;
  $204 = (($203) + 24|0);
  HEAP32[$204>>2] = $202;
  $205 = (($bits) + 16|0);
  $206 = HEAP8[$205>>0]|0;
  $207 = $206&255;
  $208 = $_mp1;
  $209 = (($208) + 28|0);
  HEAP32[$209>>2] = $207;
  $210 = $1;
  $211 = HEAP32[$210>>2]|0;
  $212 = (($211) + 20|0);
  HEAP32[$212>>2] = 86;
  $213 = $1;
  $214 = HEAP32[$213>>2]|0;
  $215 = (($214) + 4|0);
  $216 = HEAP32[$215>>2]|0;
  $217 = $1;
  FUNCTION_TABLE_vii[$216 & 255]($217,2);
  $218 = $count;
  $219 = ($218|0)>(256);
  if ($219) {
   label = 35;
  } else {
   $220 = $count;
   $221 = $length;
   $222 = ($220|0)>($221|0);
   if ($222) {
    label = 35;
   }
  }
  if ((label|0) == 35) {
   label = 0;
   $223 = $1;
   $224 = HEAP32[$223>>2]|0;
   $225 = (($224) + 20|0);
   HEAP32[$225>>2] = 8;
   $226 = $1;
   $227 = HEAP32[$226>>2]|0;
   $228 = HEAP32[$227>>2]|0;
   $229 = $1;
   FUNCTION_TABLE_vi[$228 & 255]($229);
  }
  $i = 0;
  while(1) {
   $230 = $i;
   $231 = $count;
   $232 = ($230|0)<($231|0);
   if (!($232)) {
    break;
   }
   $233 = $bytes_in_buffer;
   $234 = ($233|0)==(0);
   if ($234) {
    $235 = $datasrc;
    $236 = (($235) + 12|0);
    $237 = HEAP32[$236>>2]|0;
    $238 = $1;
    $239 = (FUNCTION_TABLE_ii[$237 & 127]($238)|0);
    $240 = ($239|0)!=(0);
    if (!($240)) {
     label = 41;
     break L17;
    }
    $241 = $datasrc;
    $242 = HEAP32[$241>>2]|0;
    $next_input_byte = $242;
    $243 = $datasrc;
    $244 = (($243) + 4|0);
    $245 = HEAP32[$244>>2]|0;
    $bytes_in_buffer = $245;
   }
   $246 = $bytes_in_buffer;
   $247 = (($246) + -1)|0;
   $bytes_in_buffer = $247;
   $248 = $next_input_byte;
   $249 = (($248) + 1|0);
   $next_input_byte = $249;
   $250 = HEAP8[$248>>0]|0;
   $251 = $i;
   $252 = (($huffval) + ($251)|0);
   HEAP8[$252>>0] = $250;
   $253 = $i;
   $254 = (($253) + 1)|0;
   $i = $254;
  }
  $255 = $count;
  $256 = $length;
  $257 = (($256) - ($255))|0;
  $length = $257;
  $258 = $index;
  $259 = $258 & 16;
  $260 = ($259|0)!=(0);
  if ($260) {
   $261 = $index;
   $262 = (($261) - 16)|0;
   $index = $262;
   $263 = $index;
   $264 = $1;
   $265 = (($264) + 196|0);
   $266 = (($265) + ($263<<2)|0);
   $htblptr = $266;
  } else {
   $267 = $index;
   $268 = $1;
   $269 = (($268) + 180|0);
   $270 = (($269) + ($267<<2)|0);
   $htblptr = $270;
  }
  $271 = $index;
  $272 = ($271|0)<(0);
  if ($272) {
   label = 51;
  } else {
   $273 = $index;
   $274 = ($273|0)>=(4);
   if ($274) {
    label = 51;
   }
  }
  if ((label|0) == 51) {
   label = 0;
   $275 = $1;
   $276 = HEAP32[$275>>2]|0;
   $277 = (($276) + 20|0);
   HEAP32[$277>>2] = 30;
   $278 = $index;
   $279 = $1;
   $280 = HEAP32[$279>>2]|0;
   $281 = (($280) + 24|0);
   HEAP32[$281>>2] = $278;
   $282 = $1;
   $283 = HEAP32[$282>>2]|0;
   $284 = HEAP32[$283>>2]|0;
   $285 = $1;
   FUNCTION_TABLE_vi[$284 & 255]($285);
  }
  $286 = $htblptr;
  $287 = HEAP32[$286>>2]|0;
  $288 = ($287|0)==(0|0);
  if ($288) {
   $289 = $1;
   $290 = (_jpeg_alloc_huff_table($289)|0);
   $291 = $htblptr;
   HEAP32[$291>>2] = $290;
  }
  $292 = $htblptr;
  $293 = HEAP32[$292>>2]|0;
  dest=$293+0|0; src=$bits+0|0; stop=dest+17|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $294 = $htblptr;
  $295 = HEAP32[$294>>2]|0;
  $296 = (($295) + 17|0);
  _memcpy(($296|0),($huffval|0),256)|0;
 }
 if ((label|0) == 16) {
  $0 = 0;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 else if ((label|0) == 24) {
  $0 = 0;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 else if ((label|0) == 41) {
  $0 = 0;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 else if ((label|0) == 55) {
  $297 = $length;
  $298 = ($297|0)!=(0);
  if ($298) {
   $299 = $1;
   $300 = HEAP32[$299>>2]|0;
   $301 = (($300) + 20|0);
   HEAP32[$301>>2] = 11;
   $302 = $1;
   $303 = HEAP32[$302>>2]|0;
   $304 = HEAP32[$303>>2]|0;
   $305 = $1;
   FUNCTION_TABLE_vi[$304 & 255]($305);
  }
  $306 = $next_input_byte;
  $307 = $datasrc;
  HEAP32[$307>>2] = $306;
  $308 = $bytes_in_buffer;
  $309 = $datasrc;
  $310 = (($309) + 4|0);
  HEAP32[$310>>2] = $308;
  $0 = 1;
  $311 = $0;
  STACKTOP = sp;return ($311|0);
 }
 return 0|0;
}
function _get_dqt($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $_mp = 0, $bytes_in_buffer = 0, $datasrc = 0, $i = 0, $length = 0, $n = 0, $next_input_byte = 0, $prec = 0, $quant_ptr = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $300 = $0;
   STACKTOP = sp;return ($300|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $300 = $0;
   STACKTOP = sp;return ($300|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = (($51) - 2)|0;
 $length = $52;
 L17: while(1) {
  $53 = $length;
  $54 = ($53|0)>(0);
  if (!($54)) {
   label = 57;
   break;
  }
  $55 = $bytes_in_buffer;
  $56 = ($55|0)==(0);
  if ($56) {
   $57 = $datasrc;
   $58 = (($57) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $1;
   $61 = (FUNCTION_TABLE_ii[$59 & 127]($60)|0);
   $62 = ($61|0)!=(0);
   if (!($62)) {
    label = 16;
    break;
   }
   $63 = $datasrc;
   $64 = HEAP32[$63>>2]|0;
   $next_input_byte = $64;
   $65 = $datasrc;
   $66 = (($65) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   $bytes_in_buffer = $67;
  }
  $68 = $bytes_in_buffer;
  $69 = (($68) + -1)|0;
  $bytes_in_buffer = $69;
  $70 = $next_input_byte;
  $71 = (($70) + 1|0);
  $next_input_byte = $71;
  $72 = HEAP8[$70>>0]|0;
  $73 = $72&255;
  $n = $73;
  $74 = $n;
  $75 = $74 >> 4;
  $prec = $75;
  $76 = $n;
  $77 = $76 & 15;
  $n = $77;
  $78 = $1;
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + 20|0);
  HEAP32[$80>>2] = 81;
  $81 = $n;
  $82 = $1;
  $83 = HEAP32[$82>>2]|0;
  $84 = (($83) + 24|0);
  HEAP32[$84>>2] = $81;
  $85 = $prec;
  $86 = $1;
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 24|0);
  $89 = (($88) + 4|0);
  HEAP32[$89>>2] = $85;
  $90 = $1;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = $1;
  FUNCTION_TABLE_vii[$93 & 255]($94,1);
  $95 = $n;
  $96 = ($95|0)>=(4);
  if ($96) {
   $97 = $1;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) + 20|0);
   HEAP32[$99>>2] = 31;
   $100 = $n;
   $101 = $1;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) + 24|0);
   HEAP32[$103>>2] = $100;
   $104 = $1;
   $105 = HEAP32[$104>>2]|0;
   $106 = HEAP32[$105>>2]|0;
   $107 = $1;
   FUNCTION_TABLE_vi[$106 & 255]($107);
  }
  $108 = $n;
  $109 = $1;
  $110 = (($109) + 164|0);
  $111 = (($110) + ($108<<2)|0);
  $112 = HEAP32[$111>>2]|0;
  $113 = ($112|0)==(0|0);
  if ($113) {
   $114 = $1;
   $115 = (_jpeg_alloc_quant_table($114)|0);
   $116 = $n;
   $117 = $1;
   $118 = (($117) + 164|0);
   $119 = (($118) + ($116<<2)|0);
   HEAP32[$119>>2] = $115;
  }
  $120 = $n;
  $121 = $1;
  $122 = (($121) + 164|0);
  $123 = (($122) + ($120<<2)|0);
  $124 = HEAP32[$123>>2]|0;
  $quant_ptr = $124;
  $i = 0;
  while(1) {
   $125 = $i;
   $126 = ($125|0)<(64);
   if (!($126)) {
    break;
   }
   $127 = $prec;
   $128 = ($127|0)!=(0);
   if ($128) {
    $129 = $bytes_in_buffer;
    $130 = ($129|0)==(0);
    if ($130) {
     $131 = $datasrc;
     $132 = (($131) + 12|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = $1;
     $135 = (FUNCTION_TABLE_ii[$133 & 127]($134)|0);
     $136 = ($135|0)!=(0);
     if (!($136)) {
      label = 29;
      break L17;
     }
     $137 = $datasrc;
     $138 = HEAP32[$137>>2]|0;
     $next_input_byte = $138;
     $139 = $datasrc;
     $140 = (($139) + 4|0);
     $141 = HEAP32[$140>>2]|0;
     $bytes_in_buffer = $141;
    }
    $142 = $bytes_in_buffer;
    $143 = (($142) + -1)|0;
    $bytes_in_buffer = $143;
    $144 = $next_input_byte;
    $145 = (($144) + 1|0);
    $next_input_byte = $145;
    $146 = HEAP8[$144>>0]|0;
    $147 = $146&255;
    $148 = $147 << 8;
    $tmp = $148;
    $149 = $bytes_in_buffer;
    $150 = ($149|0)==(0);
    if ($150) {
     $151 = $datasrc;
     $152 = (($151) + 12|0);
     $153 = HEAP32[$152>>2]|0;
     $154 = $1;
     $155 = (FUNCTION_TABLE_ii[$153 & 127]($154)|0);
     $156 = ($155|0)!=(0);
     if (!($156)) {
      label = 33;
      break L17;
     }
     $157 = $datasrc;
     $158 = HEAP32[$157>>2]|0;
     $next_input_byte = $158;
     $159 = $datasrc;
     $160 = (($159) + 4|0);
     $161 = HEAP32[$160>>2]|0;
     $bytes_in_buffer = $161;
    }
    $162 = $bytes_in_buffer;
    $163 = (($162) + -1)|0;
    $bytes_in_buffer = $163;
    $164 = $next_input_byte;
    $165 = (($164) + 1|0);
    $next_input_byte = $165;
    $166 = HEAP8[$164>>0]|0;
    $167 = $166&255;
    $168 = $tmp;
    $169 = (($168) + ($167))|0;
    $tmp = $169;
   } else {
    $170 = $bytes_in_buffer;
    $171 = ($170|0)==(0);
    if ($171) {
     $172 = $datasrc;
     $173 = (($172) + 12|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = $1;
     $176 = (FUNCTION_TABLE_ii[$174 & 127]($175)|0);
     $177 = ($176|0)!=(0);
     if (!($177)) {
      label = 40;
      break L17;
     }
     $178 = $datasrc;
     $179 = HEAP32[$178>>2]|0;
     $next_input_byte = $179;
     $180 = $datasrc;
     $181 = (($180) + 4|0);
     $182 = HEAP32[$181>>2]|0;
     $bytes_in_buffer = $182;
    }
    $183 = $bytes_in_buffer;
    $184 = (($183) + -1)|0;
    $bytes_in_buffer = $184;
    $185 = $next_input_byte;
    $186 = (($185) + 1|0);
    $next_input_byte = $186;
    $187 = HEAP8[$185>>0]|0;
    $188 = $187&255;
    $tmp = $188;
   }
   $189 = $tmp;
   $190 = $189&65535;
   $191 = $i;
   $192 = (50504 + ($191<<2)|0);
   $193 = HEAPU8[$192>>0]|(HEAPU8[$192+1>>0]<<8)|(HEAPU8[$192+2>>0]<<16)|(HEAPU8[$192+3>>0]<<24);
   $194 = $quant_ptr;
   $195 = (($194) + ($193<<1)|0);
   HEAP16[$195>>1] = $190;
   $196 = $i;
   $197 = (($196) + 1)|0;
   $i = $197;
  }
  $198 = $1;
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + 104|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = ($201|0)>=(2);
  if ($202) {
   $i = 0;
   while(1) {
    $203 = $i;
    $204 = ($203|0)<(64);
    if (!($204)) {
     break;
    }
    $205 = $1;
    $206 = HEAP32[$205>>2]|0;
    $207 = (($206) + 24|0);
    $_mp = $207;
    $208 = $i;
    $209 = $quant_ptr;
    $210 = (($209) + ($208<<1)|0);
    $211 = HEAP16[$210>>1]|0;
    $212 = $211&65535;
    $213 = $_mp;
    HEAP32[$213>>2] = $212;
    $214 = $i;
    $215 = (($214) + 1)|0;
    $216 = $quant_ptr;
    $217 = (($216) + ($215<<1)|0);
    $218 = HEAP16[$217>>1]|0;
    $219 = $218&65535;
    $220 = $_mp;
    $221 = (($220) + 4|0);
    HEAP32[$221>>2] = $219;
    $222 = $i;
    $223 = (($222) + 2)|0;
    $224 = $quant_ptr;
    $225 = (($224) + ($223<<1)|0);
    $226 = HEAP16[$225>>1]|0;
    $227 = $226&65535;
    $228 = $_mp;
    $229 = (($228) + 8|0);
    HEAP32[$229>>2] = $227;
    $230 = $i;
    $231 = (($230) + 3)|0;
    $232 = $quant_ptr;
    $233 = (($232) + ($231<<1)|0);
    $234 = HEAP16[$233>>1]|0;
    $235 = $234&65535;
    $236 = $_mp;
    $237 = (($236) + 12|0);
    HEAP32[$237>>2] = $235;
    $238 = $i;
    $239 = (($238) + 4)|0;
    $240 = $quant_ptr;
    $241 = (($240) + ($239<<1)|0);
    $242 = HEAP16[$241>>1]|0;
    $243 = $242&65535;
    $244 = $_mp;
    $245 = (($244) + 16|0);
    HEAP32[$245>>2] = $243;
    $246 = $i;
    $247 = (($246) + 5)|0;
    $248 = $quant_ptr;
    $249 = (($248) + ($247<<1)|0);
    $250 = HEAP16[$249>>1]|0;
    $251 = $250&65535;
    $252 = $_mp;
    $253 = (($252) + 20|0);
    HEAP32[$253>>2] = $251;
    $254 = $i;
    $255 = (($254) + 6)|0;
    $256 = $quant_ptr;
    $257 = (($256) + ($255<<1)|0);
    $258 = HEAP16[$257>>1]|0;
    $259 = $258&65535;
    $260 = $_mp;
    $261 = (($260) + 24|0);
    HEAP32[$261>>2] = $259;
    $262 = $i;
    $263 = (($262) + 7)|0;
    $264 = $quant_ptr;
    $265 = (($264) + ($263<<1)|0);
    $266 = HEAP16[$265>>1]|0;
    $267 = $266&65535;
    $268 = $_mp;
    $269 = (($268) + 28|0);
    HEAP32[$269>>2] = $267;
    $270 = $1;
    $271 = HEAP32[$270>>2]|0;
    $272 = (($271) + 20|0);
    HEAP32[$272>>2] = 93;
    $273 = $1;
    $274 = HEAP32[$273>>2]|0;
    $275 = (($274) + 4|0);
    $276 = HEAP32[$275>>2]|0;
    $277 = $1;
    FUNCTION_TABLE_vii[$276 & 255]($277,2);
    $278 = $i;
    $279 = (($278) + 8)|0;
    $i = $279;
   }
  }
  $280 = $length;
  $281 = (($280) - 65)|0;
  $length = $281;
  $282 = $prec;
  $283 = ($282|0)!=(0);
  if ($283) {
   $284 = $length;
   $285 = (($284) - 64)|0;
   $length = $285;
  }
 }
 if ((label|0) == 16) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 29) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 33) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 40) {
  $0 = 0;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 else if ((label|0) == 57) {
  $286 = $length;
  $287 = ($286|0)!=(0);
  if ($287) {
   $288 = $1;
   $289 = HEAP32[$288>>2]|0;
   $290 = (($289) + 20|0);
   HEAP32[$290>>2] = 11;
   $291 = $1;
   $292 = HEAP32[$291>>2]|0;
   $293 = HEAP32[$292>>2]|0;
   $294 = $1;
   FUNCTION_TABLE_vi[$293 & 255]($294);
  }
  $295 = $next_input_byte;
  $296 = $datasrc;
  HEAP32[$296>>2] = $295;
  $297 = $bytes_in_buffer;
  $298 = $datasrc;
  $299 = (($298) + 4|0);
  HEAP32[$299>>2] = $297;
  $0 = 1;
  $300 = $0;
  STACKTOP = sp;return ($300|0);
 }
 return 0|0;
}
function _get_dri($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $bytes_in_buffer = 0, $datasrc = 0, $length = 0, $next_input_byte = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $datasrc = $4;
 $5 = $datasrc;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $datasrc;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $bytes_in_buffer;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = $datasrc;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (FUNCTION_TABLE_ii[$14 & 127]($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = $datasrc;
    $19 = HEAP32[$18>>2]|0;
    $next_input_byte = $19;
    $20 = $datasrc;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $bytes_in_buffer = $22;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $23 = $bytes_in_buffer;
 $24 = (($23) + -1)|0;
 $bytes_in_buffer = $24;
 $25 = $next_input_byte;
 $26 = (($25) + 1|0);
 $next_input_byte = $26;
 $27 = HEAP8[$25>>0]|0;
 $28 = $27&255;
 $29 = $28 << 8;
 $length = $29;
 $30 = $bytes_in_buffer;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $datasrc;
   $33 = (($32) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $1;
   $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = $datasrc;
    $39 = HEAP32[$38>>2]|0;
    $next_input_byte = $39;
    $40 = $datasrc;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    $bytes_in_buffer = $42;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $43 = $bytes_in_buffer;
 $44 = (($43) + -1)|0;
 $bytes_in_buffer = $44;
 $45 = $next_input_byte;
 $46 = (($45) + 1|0);
 $next_input_byte = $46;
 $47 = HEAP8[$45>>0]|0;
 $48 = $47&255;
 $49 = $length;
 $50 = (($49) + ($48))|0;
 $length = $50;
 $51 = $length;
 $52 = ($51|0)!=(4);
 if ($52) {
  $53 = $1;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 20|0);
  HEAP32[$55>>2] = 11;
  $56 = $1;
  $57 = HEAP32[$56>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = $1;
  FUNCTION_TABLE_vi[$58 & 255]($59);
 }
 $60 = $bytes_in_buffer;
 $61 = ($60|0)==(0);
 do {
  if ($61) {
   $62 = $datasrc;
   $63 = (($62) + 12|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $1;
   $66 = (FUNCTION_TABLE_ii[$64 & 127]($65)|0);
   $67 = ($66|0)!=(0);
   if ($67) {
    $68 = $datasrc;
    $69 = HEAP32[$68>>2]|0;
    $next_input_byte = $69;
    $70 = $datasrc;
    $71 = (($70) + 4|0);
    $72 = HEAP32[$71>>2]|0;
    $bytes_in_buffer = $72;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $73 = $bytes_in_buffer;
 $74 = (($73) + -1)|0;
 $bytes_in_buffer = $74;
 $75 = $next_input_byte;
 $76 = (($75) + 1|0);
 $next_input_byte = $76;
 $77 = HEAP8[$75>>0]|0;
 $78 = $77&255;
 $79 = $78 << 8;
 $tmp = $79;
 $80 = $bytes_in_buffer;
 $81 = ($80|0)==(0);
 do {
  if ($81) {
   $82 = $datasrc;
   $83 = (($82) + 12|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = $1;
   $86 = (FUNCTION_TABLE_ii[$84 & 127]($85)|0);
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = $datasrc;
    $89 = HEAP32[$88>>2]|0;
    $next_input_byte = $89;
    $90 = $datasrc;
    $91 = (($90) + 4|0);
    $92 = HEAP32[$91>>2]|0;
    $bytes_in_buffer = $92;
    break;
   }
   $0 = 0;
   $121 = $0;
   STACKTOP = sp;return ($121|0);
  }
 } while(0);
 $93 = $bytes_in_buffer;
 $94 = (($93) + -1)|0;
 $bytes_in_buffer = $94;
 $95 = $next_input_byte;
 $96 = (($95) + 1|0);
 $next_input_byte = $96;
 $97 = HEAP8[$95>>0]|0;
 $98 = $97&255;
 $99 = $tmp;
 $100 = (($99) + ($98))|0;
 $tmp = $100;
 $101 = $1;
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) + 20|0);
 HEAP32[$103>>2] = 82;
 $104 = $tmp;
 $105 = $1;
 $106 = HEAP32[$105>>2]|0;
 $107 = (($106) + 24|0);
 HEAP32[$107>>2] = $104;
 $108 = $1;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($109) + 4|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = $1;
 FUNCTION_TABLE_vii[$111 & 255]($112,1);
 $113 = $tmp;
 $114 = $1;
 $115 = (($114) + 276|0);
 HEAP32[$115>>2] = $113;
 $116 = $next_input_byte;
 $117 = $datasrc;
 HEAP32[$117>>2] = $116;
 $118 = $bytes_in_buffer;
 $119 = $datasrc;
 $120 = (($119) + 4|0);
 HEAP32[$120>>2] = $118;
 $0 = 1;
 $121 = $0;
 STACKTOP = sp;return ($121|0);
}
function _jpeg_calc_output_dimensions($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0, $ssize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(202);
 if ($4) {
  $5 = $0;
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) + 20|0);
  HEAP32[$7>>2] = 20;
  $8 = $0;
  $9 = (($8) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 24|0);
  HEAP32[$13>>2] = $10;
  $14 = $0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = $0;
  FUNCTION_TABLE_vi[$16 & 255]($17);
 }
 $18 = $0;
 $19 = (($18) + 48|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20<<3;
 $22 = $0;
 $23 = (($22) + 52|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($21>>>0)<=($24>>>0);
 if ($25) {
  $26 = $0;
  $27 = (($26) + 28|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (_jdiv_round_up($28,8)|0);
  $30 = $0;
  $31 = (($30) + 112|0);
  HEAP32[$31>>2] = $29;
  $32 = $0;
  $33 = (($32) + 32|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (_jdiv_round_up($34,8)|0);
  $36 = $0;
  $37 = (($36) + 116|0);
  HEAP32[$37>>2] = $35;
  $38 = $0;
  $39 = (($38) + 316|0);
  HEAP32[$39>>2] = 1;
 } else {
  $40 = $0;
  $41 = (($40) + 48|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $42<<2;
  $44 = $0;
  $45 = (($44) + 52|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($43>>>0)<=($46>>>0);
  if ($47) {
   $48 = $0;
   $49 = (($48) + 28|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_jdiv_round_up($50,4)|0);
   $52 = $0;
   $53 = (($52) + 112|0);
   HEAP32[$53>>2] = $51;
   $54 = $0;
   $55 = (($54) + 32|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (_jdiv_round_up($56,4)|0);
   $58 = $0;
   $59 = (($58) + 116|0);
   HEAP32[$59>>2] = $57;
   $60 = $0;
   $61 = (($60) + 316|0);
   HEAP32[$61>>2] = 2;
  } else {
   $62 = $0;
   $63 = (($62) + 48|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $64<<1;
   $66 = $0;
   $67 = (($66) + 52|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = ($65>>>0)<=($68>>>0);
   if ($69) {
    $70 = $0;
    $71 = (($70) + 28|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (_jdiv_round_up($72,2)|0);
    $74 = $0;
    $75 = (($74) + 112|0);
    HEAP32[$75>>2] = $73;
    $76 = $0;
    $77 = (($76) + 32|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (_jdiv_round_up($78,2)|0);
    $80 = $0;
    $81 = (($80) + 116|0);
    HEAP32[$81>>2] = $79;
    $82 = $0;
    $83 = (($82) + 316|0);
    HEAP32[$83>>2] = 4;
   } else {
    $84 = $0;
    $85 = (($84) + 28|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = $0;
    $88 = (($87) + 112|0);
    HEAP32[$88>>2] = $86;
    $89 = $0;
    $90 = (($89) + 32|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = $0;
    $93 = (($92) + 116|0);
    HEAP32[$93>>2] = $91;
    $94 = $0;
    $95 = (($94) + 316|0);
    HEAP32[$95>>2] = 8;
   }
  }
 }
 $ci = 0;
 $96 = $0;
 $97 = (($96) + 216|0);
 $98 = HEAP32[$97>>2]|0;
 $compptr = $98;
 while(1) {
  $99 = $ci;
  $100 = $0;
  $101 = (($100) + 36|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = ($99|0)<($102|0);
  if (!($103)) {
   break;
  }
  $104 = $0;
  $105 = (($104) + 316|0);
  $106 = HEAP32[$105>>2]|0;
  $ssize = $106;
  while(1) {
   $107 = $ssize;
   $108 = ($107|0)<(8);
   if ($108) {
    $109 = $compptr;
    $110 = (($109) + 8|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = $ssize;
    $113 = Math_imul($111, $112)|0;
    $114 = $113<<1;
    $115 = $0;
    $116 = (($115) + 308|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = $0;
    $119 = (($118) + 316|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = Math_imul($117, $120)|0;
    $122 = ($114|0)<=($121|0);
    if ($122) {
     $123 = $compptr;
     $124 = (($123) + 12|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = $ssize;
     $127 = Math_imul($125, $126)|0;
     $128 = $127<<1;
     $129 = $0;
     $130 = (($129) + 312|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = $0;
     $133 = (($132) + 316|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = Math_imul($131, $134)|0;
     $136 = ($128|0)<=($135|0);
     $228 = $136;
    } else {
     $228 = 0;
    }
   } else {
    $228 = 0;
   }
   if (!($228)) {
    break;
   }
   $137 = $ssize;
   $138 = $137<<1;
   $ssize = $138;
  }
  $139 = $ssize;
  $140 = $compptr;
  $141 = (($140) + 36|0);
  HEAP32[$141>>2] = $139;
  $142 = $ci;
  $143 = (($142) + 1)|0;
  $ci = $143;
  $144 = $compptr;
  $145 = (($144) + 84|0);
  $compptr = $145;
 }
 $ci = 0;
 $146 = $0;
 $147 = (($146) + 216|0);
 $148 = HEAP32[$147>>2]|0;
 $compptr = $148;
 while(1) {
  $149 = $ci;
  $150 = $0;
  $151 = (($150) + 36|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = ($149|0)<($152|0);
  if (!($153)) {
   break;
  }
  $154 = $0;
  $155 = (($154) + 28|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = $compptr;
  $158 = (($157) + 8|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = $compptr;
  $161 = (($160) + 36|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = Math_imul($159, $162)|0;
  $164 = Math_imul($156, $163)|0;
  $165 = $0;
  $166 = (($165) + 308|0);
  $167 = HEAP32[$166>>2]|0;
  $168 = $167<<3;
  $169 = (_jdiv_round_up($164,$168)|0);
  $170 = $compptr;
  $171 = (($170) + 40|0);
  HEAP32[$171>>2] = $169;
  $172 = $0;
  $173 = (($172) + 32|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = $compptr;
  $176 = (($175) + 12|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = $compptr;
  $179 = (($178) + 36|0);
  $180 = HEAP32[$179>>2]|0;
  $181 = Math_imul($177, $180)|0;
  $182 = Math_imul($174, $181)|0;
  $183 = $0;
  $184 = (($183) + 312|0);
  $185 = HEAP32[$184>>2]|0;
  $186 = $185<<3;
  $187 = (_jdiv_round_up($182,$186)|0);
  $188 = $compptr;
  $189 = (($188) + 44|0);
  HEAP32[$189>>2] = $187;
  $190 = $ci;
  $191 = (($190) + 1)|0;
  $ci = $191;
  $192 = $compptr;
  $193 = (($192) + 84|0);
  $compptr = $193;
 }
 $194 = $0;
 $195 = (($194) + 44|0);
 $196 = HEAP32[$195>>2]|0;
 switch ($196|0) {
 case 3: case 2:  {
  $199 = $0;
  $200 = (($199) + 120|0);
  HEAP32[$200>>2] = 3;
  break;
 }
 case 1:  {
  $197 = $0;
  $198 = (($197) + 120|0);
  HEAP32[$198>>2] = 1;
  break;
 }
 case 5: case 4:  {
  $201 = $0;
  $202 = (($201) + 120|0);
  HEAP32[$202>>2] = 4;
  break;
 }
 default: {
  $203 = $0;
  $204 = (($203) + 36|0);
  $205 = HEAP32[$204>>2]|0;
  $206 = $0;
  $207 = (($206) + 120|0);
  HEAP32[$207>>2] = $205;
 }
 }
 $208 = $0;
 $209 = (($208) + 84|0);
 $210 = HEAP32[$209>>2]|0;
 $211 = ($210|0)!=(0);
 if ($211) {
  $217 = 1;
 } else {
  $212 = $0;
  $213 = (($212) + 120|0);
  $214 = HEAP32[$213>>2]|0;
  $217 = $214;
 }
 $215 = $0;
 $216 = (($215) + 124|0);
 HEAP32[$216>>2] = $217;
 $218 = $0;
 $219 = (_use_merged_upsample($218)|0);
 $220 = ($219|0)!=(0);
 if ($220) {
  $221 = $0;
  $222 = (($221) + 312|0);
  $223 = HEAP32[$222>>2]|0;
  $224 = $0;
  $225 = (($224) + 128|0);
  HEAP32[$225>>2] = $223;
  STACKTOP = sp;return;
 } else {
  $226 = $0;
  $227 = (($226) + 128|0);
  HEAP32[$227>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _use_merged_upsample($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 76|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $6 = $1;
  $7 = (($6) + 300|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0);
  if (!($9)) {
   $10 = $1;
   $11 = (($10) + 40|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)!=(3);
   if (!($13)) {
    $14 = $1;
    $15 = (($14) + 36|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($16|0)!=(3);
    if (!($17)) {
     $18 = $1;
     $19 = (($18) + 44|0);
     $20 = HEAP32[$19>>2]|0;
     $21 = ($20|0)!=(2);
     if (!($21)) {
      $22 = $1;
      $23 = (($22) + 120|0);
      $24 = HEAP32[$23>>2]|0;
      $25 = ($24|0)!=(3);
      if (!($25)) {
       $26 = $1;
       $27 = (($26) + 216|0);
       $28 = HEAP32[$27>>2]|0;
       $29 = (($28) + 8|0);
       $30 = HEAP32[$29>>2]|0;
       $31 = ($30|0)!=(2);
       if (!($31)) {
        $32 = $1;
        $33 = (($32) + 216|0);
        $34 = HEAP32[$33>>2]|0;
        $35 = (($34) + 84|0);
        $36 = (($35) + 8|0);
        $37 = HEAP32[$36>>2]|0;
        $38 = ($37|0)!=(1);
        if (!($38)) {
         $39 = $1;
         $40 = (($39) + 216|0);
         $41 = HEAP32[$40>>2]|0;
         $42 = (($41) + 168|0);
         $43 = (($42) + 8|0);
         $44 = HEAP32[$43>>2]|0;
         $45 = ($44|0)!=(1);
         if (!($45)) {
          $46 = $1;
          $47 = (($46) + 216|0);
          $48 = HEAP32[$47>>2]|0;
          $49 = (($48) + 12|0);
          $50 = HEAP32[$49>>2]|0;
          $51 = ($50|0)>(2);
          if (!($51)) {
           $52 = $1;
           $53 = (($52) + 216|0);
           $54 = HEAP32[$53>>2]|0;
           $55 = (($54) + 84|0);
           $56 = (($55) + 12|0);
           $57 = HEAP32[$56>>2]|0;
           $58 = ($57|0)!=(1);
           if (!($58)) {
            $59 = $1;
            $60 = (($59) + 216|0);
            $61 = HEAP32[$60>>2]|0;
            $62 = (($61) + 168|0);
            $63 = (($62) + 12|0);
            $64 = HEAP32[$63>>2]|0;
            $65 = ($64|0)!=(1);
            if (!($65)) {
             $66 = $1;
             $67 = (($66) + 216|0);
             $68 = HEAP32[$67>>2]|0;
             $69 = (($68) + 36|0);
             $70 = HEAP32[$69>>2]|0;
             $71 = $1;
             $72 = (($71) + 316|0);
             $73 = HEAP32[$72>>2]|0;
             $74 = ($70|0)!=($73|0);
             if (!($74)) {
              $75 = $1;
              $76 = (($75) + 216|0);
              $77 = HEAP32[$76>>2]|0;
              $78 = (($77) + 84|0);
              $79 = (($78) + 36|0);
              $80 = HEAP32[$79>>2]|0;
              $81 = $1;
              $82 = (($81) + 316|0);
              $83 = HEAP32[$82>>2]|0;
              $84 = ($80|0)!=($83|0);
              if (!($84)) {
               $85 = $1;
               $86 = (($85) + 216|0);
               $87 = HEAP32[$86>>2]|0;
               $88 = (($87) + 168|0);
               $89 = (($88) + 36|0);
               $90 = HEAP32[$89>>2]|0;
               $91 = $1;
               $92 = (($91) + 316|0);
               $93 = HEAP32[$92>>2]|0;
               $94 = ($90|0)!=($93|0);
               if (!($94)) {
                $0 = 1;
                $95 = $0;
                STACKTOP = sp;return ($95|0);
               }
              }
             }
             $0 = 0;
             $95 = $0;
             STACKTOP = sp;return ($95|0);
            }
           }
          }
         }
        }
       }
       $0 = 0;
       $95 = $0;
       STACKTOP = sp;return ($95|0);
      }
     }
    }
   }
   $0 = 0;
   $95 = $0;
   STACKTOP = sp;return ($95|0);
  }
 }
 $0 = 0;
 $95 = $0;
 STACKTOP = sp;return ($95|0);
}
function _jinit_master_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $master = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,28)|0);
 $master = $6;
 $7 = $master;
 $8 = $0;
 $9 = (($8) + 420|0);
 HEAP32[$9>>2] = $7;
 $10 = $master;
 HEAP32[$10>>2] = 44;
 $11 = $master;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 45;
 $13 = $master;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 $15 = $0;
 _master_selection($15);
 STACKTOP = sp;return;
}
function _prepare_for_output_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $master = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 420|0);
 $3 = HEAP32[$2>>2]|0;
 $master = $3;
 $4 = $master;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $master;
  $9 = (($8) + 8|0);
  HEAP32[$9>>2] = 0;
  $10 = $0;
  $11 = (($10) + 460|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = $0;
  FUNCTION_TABLE_vii[$13 & 255]($14,0);
  $15 = $0;
  $16 = (($15) + 432|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  FUNCTION_TABLE_vii[$18 & 255]($19,2);
  $20 = $0;
  $21 = (($20) + 424|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = $0;
  FUNCTION_TABLE_vii[$23 & 255]($24,2);
 } else {
  $25 = $0;
  $26 = (($25) + 84|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = $0;
   $30 = (($29) + 136|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)==(0|0);
   if ($32) {
    $33 = $0;
    $34 = (($33) + 92|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)!=(0);
    if ($36) {
     $37 = $0;
     $38 = (($37) + 108|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = ($39|0)!=(0);
     if ($40) {
      $41 = $master;
      $42 = (($41) + 24|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = $0;
      $45 = (($44) + 460|0);
      HEAP32[$45>>2] = $43;
      $46 = $master;
      $47 = (($46) + 8|0);
      HEAP32[$47>>2] = 1;
     } else {
      label = 8;
     }
    } else {
     label = 8;
    }
    if ((label|0) == 8) {
     $48 = $0;
     $49 = (($48) + 100|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = ($50|0)!=(0);
     if ($51) {
      $52 = $master;
      $53 = (($52) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = $0;
      $56 = (($55) + 460|0);
      HEAP32[$56>>2] = $54;
     } else {
      $57 = $0;
      $58 = HEAP32[$57>>2]|0;
      $59 = (($58) + 20|0);
      HEAP32[$59>>2] = 46;
      $60 = $0;
      $61 = HEAP32[$60>>2]|0;
      $62 = HEAP32[$61>>2]|0;
      $63 = $0;
      FUNCTION_TABLE_vi[$62 & 255]($63);
     }
    }
   }
  }
  $64 = $0;
  $65 = (($64) + 448|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = HEAP32[$66>>2]|0;
  $68 = $0;
  FUNCTION_TABLE_vi[$67 & 255]($68);
  $69 = $0;
  $70 = (($69) + 428|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  FUNCTION_TABLE_vi[$73 & 255]($74);
  $75 = $0;
  $76 = (($75) + 68|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($77|0)!=(0);
  if (!($78)) {
   $79 = $master;
   $80 = (($79) + 16|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = ($81|0)!=(0);
   if (!($82)) {
    $83 = $0;
    $84 = (($83) + 456|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = HEAP32[$85>>2]|0;
    $87 = $0;
    FUNCTION_TABLE_vi[$86 & 255]($87);
   }
   $88 = $0;
   $89 = (($88) + 452|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$90>>2]|0;
   $92 = $0;
   FUNCTION_TABLE_vi[$91 & 255]($92);
   $93 = $0;
   $94 = (($93) + 84|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = ($95|0)!=(0);
   if ($96) {
    $97 = $0;
    $98 = (($97) + 460|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = HEAP32[$99>>2]|0;
    $101 = $0;
    $102 = $master;
    $103 = (($102) + 8|0);
    $104 = HEAP32[$103>>2]|0;
    FUNCTION_TABLE_vii[$100 & 255]($101,$104);
   }
   $105 = $0;
   $106 = (($105) + 432|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = HEAP32[$107>>2]|0;
   $109 = $0;
   $110 = $master;
   $111 = (($110) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = ($112|0)!=(0);
   $114 = $113 ? 3 : 0;
   FUNCTION_TABLE_vii[$108 & 255]($109,$114);
   $115 = $0;
   $116 = (($115) + 424|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = HEAP32[$117>>2]|0;
   $119 = $0;
   FUNCTION_TABLE_vii[$118 & 255]($119,0);
  }
 }
 $120 = $0;
 $121 = (($120) + 8|0);
 $122 = HEAP32[$121>>2]|0;
 $123 = ($122|0)!=(0|0);
 if (!($123)) {
  STACKTOP = sp;return;
 }
 $124 = $master;
 $125 = (($124) + 12|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = $0;
 $128 = (($127) + 8|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = (($129) + 12|0);
 HEAP32[$130>>2] = $126;
 $131 = $master;
 $132 = (($131) + 12|0);
 $133 = HEAP32[$132>>2]|0;
 $134 = $master;
 $135 = (($134) + 8|0);
 $136 = HEAP32[$135>>2]|0;
 $137 = ($136|0)!=(0);
 $138 = $137 ? 2 : 1;
 $139 = (($133) + ($138))|0;
 $140 = $0;
 $141 = (($140) + 8|0);
 $142 = HEAP32[$141>>2]|0;
 $143 = (($142) + 16|0);
 HEAP32[$143>>2] = $139;
 $144 = $0;
 $145 = (($144) + 64|0);
 $146 = HEAP32[$145>>2]|0;
 $147 = ($146|0)!=(0);
 if ($147) {
  $148 = $0;
  $149 = (($148) + 436|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = (($150) + 20|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = ($152|0)!=(0);
  if (!($153)) {
   $154 = $0;
   $155 = (($154) + 108|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = ($156|0)!=(0);
   $158 = $157 ? 2 : 1;
   $159 = $0;
   $160 = (($159) + 8|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = (($161) + 16|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = (($163) + ($158))|0;
   HEAP32[$162>>2] = $164;
  }
 }
 STACKTOP = sp;return;
}
function _finish_output_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $master = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 420|0);
 $3 = HEAP32[$2>>2]|0;
 $master = $3;
 $4 = $0;
 $5 = (($4) + 84|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $0;
  $9 = (($8) + 460|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $0;
  FUNCTION_TABLE_vi[$12 & 255]($13);
 }
 $14 = $master;
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$15>>2] = $17;
 STACKTOP = sp;return;
}
function _master_selection($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $jd_samplesperrow = 0, $master = 0, $nscans = 0, $samplesperrow = 0, $use_c_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 420|0);
 $3 = HEAP32[$2>>2]|0;
 $master = $3;
 $4 = $0;
 _jpeg_calc_output_dimensions($4);
 $5 = $0;
 _prepare_range_limit_table($5);
 $6 = $0;
 $7 = (($6) + 112|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = (($9) + 120|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = Math_imul($8, $11)|0;
 $samplesperrow = $12;
 $13 = $samplesperrow;
 $jd_samplesperrow = $13;
 $14 = $jd_samplesperrow;
 $15 = $samplesperrow;
 $16 = ($14|0)!=($15|0);
 if ($16) {
  $17 = $0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 20|0);
  HEAP32[$19>>2] = 70;
  $20 = $0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = $0;
  FUNCTION_TABLE_vi[$22 & 255]($23);
 }
 $24 = $master;
 $25 = (($24) + 12|0);
 HEAP32[$25>>2] = 0;
 $26 = $0;
 $27 = (_use_merged_upsample($26)|0);
 $28 = $master;
 $29 = (($28) + 16|0);
 HEAP32[$29>>2] = $27;
 $30 = $master;
 $31 = (($30) + 20|0);
 HEAP32[$31>>2] = 0;
 $32 = $master;
 $33 = (($32) + 24|0);
 HEAP32[$33>>2] = 0;
 $34 = $0;
 $35 = (($34) + 84|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = $0;
  $39 = (($38) + 64|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $42 = $0;
  $43 = (($42) + 100|0);
  HEAP32[$43>>2] = 0;
  $44 = $0;
  $45 = (($44) + 104|0);
  HEAP32[$45>>2] = 0;
  $46 = $0;
  $47 = (($46) + 108|0);
  HEAP32[$47>>2] = 0;
 }
 $48 = $0;
 $49 = (($48) + 84|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = $0;
  $53 = (($52) + 68|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0);
  if ($55) {
   $56 = $0;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 20|0);
   HEAP32[$58>>2] = 47;
   $59 = $0;
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$60>>2]|0;
   $62 = $0;
   FUNCTION_TABLE_vi[$61 & 255]($62);
  }
  $63 = $0;
  $64 = (($63) + 120|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)!=(3);
  if ($66) {
   $67 = $0;
   $68 = (($67) + 100|0);
   HEAP32[$68>>2] = 1;
   $69 = $0;
   $70 = (($69) + 104|0);
   HEAP32[$70>>2] = 0;
   $71 = $0;
   $72 = (($71) + 108|0);
   HEAP32[$72>>2] = 0;
   $73 = $0;
   $74 = (($73) + 136|0);
   HEAP32[$74>>2] = 0;
  } else {
   $75 = $0;
   $76 = (($75) + 136|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = ($77|0)!=(0|0);
   if ($78) {
    $79 = $0;
    $80 = (($79) + 104|0);
    HEAP32[$80>>2] = 1;
   } else {
    $81 = $0;
    $82 = (($81) + 92|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ($83|0)!=(0);
    if ($84) {
     $85 = $0;
     $86 = (($85) + 108|0);
     HEAP32[$86>>2] = 1;
    } else {
     $87 = $0;
     $88 = (($87) + 100|0);
     HEAP32[$88>>2] = 1;
    }
   }
  }
  $89 = $0;
  $90 = (($89) + 100|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = ($91|0)!=(0);
  if ($92) {
   $93 = $0;
   _jinit_1pass_quantizer($93);
   $94 = $0;
   $95 = (($94) + 460|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = $master;
   $98 = (($97) + 20|0);
   HEAP32[$98>>2] = $96;
  }
  $99 = $0;
  $100 = (($99) + 108|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = ($101|0)!=(0);
  if ($102) {
   label = 22;
  } else {
   $103 = $0;
   $104 = (($103) + 104|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = ($105|0)!=(0);
   if ($106) {
    label = 22;
   }
  }
  if ((label|0) == 22) {
   $107 = $0;
   _jinit_2pass_quantizer($107);
   $108 = $0;
   $109 = (($108) + 460|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = $master;
   $112 = (($111) + 24|0);
   HEAP32[$112>>2] = $110;
  }
 }
 $113 = $0;
 $114 = (($113) + 68|0);
 $115 = HEAP32[$114>>2]|0;
 $116 = ($115|0)!=(0);
 if (!($116)) {
  $117 = $master;
  $118 = (($117) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = ($119|0)!=(0);
  if ($120) {
   $121 = $0;
   _jinit_merged_upsampler($121);
  } else {
   $122 = $0;
   _jinit_color_deconverter($122);
   $123 = $0;
   _jinit_upsampler($123);
  }
  $124 = $0;
  $125 = $0;
  $126 = (($125) + 108|0);
  $127 = HEAP32[$126>>2]|0;
  _jinit_d_post_controller($124,$127);
 }
 $128 = $0;
 _jinit_inverse_dct($128);
 $129 = $0;
 $130 = (($129) + 224|0);
 $131 = HEAP32[$130>>2]|0;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = $0;
  $134 = HEAP32[$133>>2]|0;
  $135 = (($134) + 20|0);
  HEAP32[$135>>2] = 1;
  $136 = $0;
  $137 = HEAP32[$136>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = $0;
  FUNCTION_TABLE_vi[$138 & 255]($139);
 } else {
  $140 = $0;
  $141 = (($140) + 220|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = ($142|0)!=(0);
  if ($143) {
   $144 = $0;
   _jinit_phuff_decoder($144);
  } else {
   $145 = $0;
   _jinit_huff_decoder($145);
  }
 }
 $146 = $0;
 $147 = (($146) + 436|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = (($148) + 16|0);
 $150 = HEAP32[$149>>2]|0;
 $151 = ($150|0)!=(0);
 if ($151) {
  $157 = 1;
 } else {
  $152 = $0;
  $153 = (($152) + 64|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = ($154|0)!=(0);
  $157 = $155;
 }
 $156 = $157&1;
 $use_c_buffer = $156;
 $158 = $0;
 $159 = $use_c_buffer;
 _jinit_d_coef_controller($158,$159);
 $160 = $0;
 $161 = (($160) + 68|0);
 $162 = HEAP32[$161>>2]|0;
 $163 = ($162|0)!=(0);
 if (!($163)) {
  $164 = $0;
  _jinit_d_main_controller($164,0);
 }
 $165 = $0;
 $166 = (($165) + 4|0);
 $167 = HEAP32[$166>>2]|0;
 $168 = (($167) + 24|0);
 $169 = HEAP32[$168>>2]|0;
 $170 = $0;
 FUNCTION_TABLE_vi[$169 & 255]($170);
 $171 = $0;
 $172 = (($171) + 436|0);
 $173 = HEAP32[$172>>2]|0;
 $174 = (($173) + 8|0);
 $175 = HEAP32[$174>>2]|0;
 $176 = $0;
 FUNCTION_TABLE_vi[$175 & 255]($176);
 $177 = $0;
 $178 = (($177) + 8|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = ($179|0)!=(0|0);
 if (!($180)) {
  STACKTOP = sp;return;
 }
 $181 = $0;
 $182 = (($181) + 64|0);
 $183 = HEAP32[$182>>2]|0;
 $184 = ($183|0)!=(0);
 if ($184) {
  STACKTOP = sp;return;
 }
 $185 = $0;
 $186 = (($185) + 436|0);
 $187 = HEAP32[$186>>2]|0;
 $188 = (($187) + 16|0);
 $189 = HEAP32[$188>>2]|0;
 $190 = ($189|0)!=(0);
 if (!($190)) {
  STACKTOP = sp;return;
 }
 $191 = $0;
 $192 = (($191) + 220|0);
 $193 = HEAP32[$192>>2]|0;
 $194 = ($193|0)!=(0);
 if ($194) {
  $195 = $0;
  $196 = (($195) + 36|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = ($197*3)|0;
  $199 = (2 + ($198))|0;
  $nscans = $199;
 } else {
  $200 = $0;
  $201 = (($200) + 36|0);
  $202 = HEAP32[$201>>2]|0;
  $nscans = $202;
 }
 $203 = $0;
 $204 = (($203) + 8|0);
 $205 = HEAP32[$204>>2]|0;
 $206 = (($205) + 4|0);
 HEAP32[$206>>2] = 0;
 $207 = $0;
 $208 = (($207) + 320|0);
 $209 = HEAP32[$208>>2]|0;
 $210 = $nscans;
 $211 = Math_imul($209, $210)|0;
 $212 = $0;
 $213 = (($212) + 8|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = (($214) + 8|0);
 HEAP32[$215>>2] = $211;
 $216 = $0;
 $217 = (($216) + 8|0);
 $218 = HEAP32[$217>>2]|0;
 $219 = (($218) + 12|0);
 HEAP32[$219>>2] = 0;
 $220 = $0;
 $221 = (($220) + 108|0);
 $222 = HEAP32[$221>>2]|0;
 $223 = ($222|0)!=(0);
 $224 = $223 ? 3 : 2;
 $225 = $0;
 $226 = (($225) + 8|0);
 $227 = HEAP32[$226>>2]|0;
 $228 = (($227) + 16|0);
 HEAP32[$228>>2] = $224;
 $229 = $master;
 $230 = (($229) + 12|0);
 $231 = HEAP32[$230>>2]|0;
 $232 = (($231) + 1)|0;
 HEAP32[$230>>2] = $232;
 STACKTOP = sp;return;
}
function _prepare_range_limit_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $table = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,1408)|0);
 $table = $6;
 $7 = $table;
 $8 = (($7) + 256|0);
 $table = $8;
 $9 = $table;
 $10 = $0;
 $11 = (($10) + 324|0);
 HEAP32[$11>>2] = $9;
 $12 = $table;
 $13 = (($12) + -256|0);
 _memset(($13|0),0,256)|0;
 $i = 0;
 while(1) {
  $14 = $i;
  $15 = ($14|0)<=(255);
  if (!($15)) {
   break;
  }
  $16 = $i;
  $17 = $16&255;
  $18 = $i;
  $19 = $table;
  $20 = (($19) + ($18)|0);
  HEAP8[$20>>0] = $17;
  $21 = $i;
  $22 = (($21) + 1)|0;
  $i = $22;
 }
 $23 = $table;
 $24 = (($23) + 128|0);
 $table = $24;
 $i = 128;
 while(1) {
  $25 = $i;
  $26 = ($25|0)<(512);
  if (!($26)) {
   break;
  }
  $27 = $i;
  $28 = $table;
  $29 = (($28) + ($27)|0);
  HEAP8[$29>>0] = -1;
  $30 = $i;
  $31 = (($30) + 1)|0;
  $i = $31;
 }
 $32 = $table;
 $33 = (($32) + 512|0);
 _memset(($33|0),0,384)|0;
 $34 = $table;
 $35 = (($34) + 896|0);
 $36 = $0;
 $37 = (($36) + 324|0);
 $38 = HEAP32[$37>>2]|0;
 dest=$35+0|0; src=$38+0|0; stop=dest+128|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _jinit_merged_upsampler($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,48)|0);
 $upsample = $6;
 $7 = $upsample;
 $8 = $0;
 $9 = (($8) + 452|0);
 HEAP32[$9>>2] = $7;
 $10 = $upsample;
 HEAP32[$10>>2] = 46;
 $11 = $upsample;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = $0;
 $14 = (($13) + 112|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 $17 = (($16) + 120|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = Math_imul($15, $18)|0;
 $20 = $upsample;
 $21 = (($20) + 40|0);
 HEAP32[$21>>2] = $19;
 $22 = $0;
 $23 = (($22) + 312|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)==(2);
 if ($25) {
  $26 = $upsample;
  $27 = (($26) + 4|0);
  HEAP32[$27>>2] = 47;
  $28 = $upsample;
  $29 = (($28) + 12|0);
  HEAP32[$29>>2] = 48;
  $30 = $0;
  $31 = (($30) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  $36 = $upsample;
  $37 = (($36) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = $38;
  $40 = (FUNCTION_TABLE_iiii[$34 & 255]($35,1,$39)|0);
  $41 = $upsample;
  $42 = (($41) + 32|0);
  HEAP32[$42>>2] = $40;
  $49 = $0;
  _build_ycc_rgb_table($49);
  STACKTOP = sp;return;
 } else {
  $43 = $upsample;
  $44 = (($43) + 4|0);
  HEAP32[$44>>2] = 49;
  $45 = $upsample;
  $46 = (($45) + 12|0);
  HEAP32[$46>>2] = 50;
  $47 = $upsample;
  $48 = (($47) + 32|0);
  HEAP32[$48>>2] = 0;
  $49 = $0;
  _build_ycc_rgb_table($49);
  STACKTOP = sp;return;
 }
}
function _start_pass_merged_upsample($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $upsample = $3;
 $4 = $upsample;
 $5 = (($4) + 36|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 $7 = (($6) + 116|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $upsample;
 $10 = (($9) + 44|0);
 HEAP32[$10>>2] = $8;
 STACKTOP = sp;return;
}
function _merged_2v_upsample($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $num_rows = 0, $upsample = 0, $work_ptrs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $work_ptrs = sp + 8|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 452|0);
 $9 = HEAP32[$8>>2]|0;
 $upsample = $9;
 $10 = $upsample;
 $11 = (($10) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $upsample;
  $15 = (($14) + 32|0);
  $16 = $4;
  $17 = $5;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + ($18<<2)|0);
  $20 = $upsample;
  $21 = (($20) + 40|0);
  $22 = HEAP32[$21>>2]|0;
  _jcopy_sample_rows($15,0,$19,0,1,$22);
  $num_rows = 1;
  $23 = $upsample;
  $24 = (($23) + 36|0);
  HEAP32[$24>>2] = 0;
 } else {
  $num_rows = 2;
  $25 = $num_rows;
  $26 = $upsample;
  $27 = (($26) + 44|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25>>>0)>($28>>>0);
  if ($29) {
   $30 = $upsample;
   $31 = (($30) + 44|0);
   $32 = HEAP32[$31>>2]|0;
   $num_rows = $32;
  }
  $33 = $5;
  $34 = HEAP32[$33>>2]|0;
  $35 = $6;
  $36 = (($35) - ($34))|0;
  $6 = $36;
  $37 = $num_rows;
  $38 = $6;
  $39 = ($37>>>0)>($38>>>0);
  if ($39) {
   $40 = $6;
   $num_rows = $40;
  }
  $41 = $5;
  $42 = HEAP32[$41>>2]|0;
  $43 = $4;
  $44 = (($43) + ($42<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$work_ptrs>>2] = $45;
  $46 = $num_rows;
  $47 = ($46>>>0)>(1);
  if ($47) {
   $48 = $5;
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + 1)|0;
   $51 = $4;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = (($work_ptrs) + 4|0);
   HEAP32[$54>>2] = $53;
  } else {
   $55 = $upsample;
   $56 = (($55) + 32|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($work_ptrs) + 4|0);
   HEAP32[$58>>2] = $57;
   $59 = $upsample;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = 1;
  }
  $61 = $upsample;
  $62 = (($61) + 12|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $0;
  $65 = $1;
  $66 = $2;
  $67 = HEAP32[$66>>2]|0;
  FUNCTION_TABLE_viiii[$63 & 255]($64,$65,$67,$work_ptrs);
 }
 $68 = $num_rows;
 $69 = $5;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + ($68))|0;
 HEAP32[$69>>2] = $71;
 $72 = $num_rows;
 $73 = $upsample;
 $74 = (($73) + 44|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($75) - ($72))|0;
 HEAP32[$74>>2] = $76;
 $77 = $upsample;
 $78 = (($77) + 36|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = ($79|0)!=(0);
 if ($80) {
  STACKTOP = sp;return;
 }
 $81 = $2;
 $82 = HEAP32[$81>>2]|0;
 $83 = (($82) + 1)|0;
 HEAP32[$81>>2] = $83;
 STACKTOP = sp;return;
}
function _h2v2_merged_upsample($cinfo,$input_buf,$in_row_group_ctr,$output_buf) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cblue = 0, $cgreen = 0, $col = 0, $cr = 0, $cred = 0, $inptr00 = 0, $inptr01 = 0, $inptr1 = 0, $inptr2 = 0, $outptr0 = 0, $outptr1 = 0, $range_limit = 0;
 var $upsample = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $output_buf;
 $4 = $0;
 $5 = (($4) + 452|0);
 $6 = HEAP32[$5>>2]|0;
 $upsample = $6;
 $7 = $0;
 $8 = (($7) + 324|0);
 $9 = HEAP32[$8>>2]|0;
 $range_limit = $9;
 $10 = $upsample;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $Crrtab = $12;
 $13 = $upsample;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $Cbbtab = $15;
 $16 = $upsample;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $Crgtab = $18;
 $19 = $upsample;
 $20 = (($19) + 28|0);
 $21 = HEAP32[$20>>2]|0;
 $Cbgtab = $21;
 $22 = $2;
 $23 = $22<<1;
 $24 = $1;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($23<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 $inptr00 = $27;
 $28 = $2;
 $29 = $28<<1;
 $30 = (($29) + 1)|0;
 $31 = $1;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($30<<2)|0);
 $34 = HEAP32[$33>>2]|0;
 $inptr01 = $34;
 $35 = $2;
 $36 = $1;
 $37 = (($36) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35<<2)|0);
 $40 = HEAP32[$39>>2]|0;
 $inptr1 = $40;
 $41 = $2;
 $42 = $1;
 $43 = (($42) + 8|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<2)|0);
 $46 = HEAP32[$45>>2]|0;
 $inptr2 = $46;
 $47 = $3;
 $48 = HEAP32[$47>>2]|0;
 $outptr0 = $48;
 $49 = $3;
 $50 = (($49) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $outptr1 = $51;
 $52 = $0;
 $53 = (($52) + 112|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $54 >>> 1;
 $col = $55;
 while(1) {
  $56 = $col;
  $57 = ($56>>>0)>(0);
  if (!($57)) {
   break;
  }
  $58 = $inptr1;
  $59 = (($58) + 1|0);
  $inptr1 = $59;
  $60 = HEAP8[$58>>0]|0;
  $61 = $60&255;
  $cb = $61;
  $62 = $inptr2;
  $63 = (($62) + 1|0);
  $inptr2 = $63;
  $64 = HEAP8[$62>>0]|0;
  $65 = $64&255;
  $cr = $65;
  $66 = $cr;
  $67 = $Crrtab;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  $cred = $69;
  $70 = $cb;
  $71 = $Cbgtab;
  $72 = (($71) + ($70<<2)|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $cr;
  $75 = $Crgtab;
  $76 = (($75) + ($74<<2)|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (($73) + ($77))|0;
  $79 = $78 >> 16;
  $cgreen = $79;
  $80 = $cb;
  $81 = $Cbbtab;
  $82 = (($81) + ($80<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $cblue = $83;
  $84 = $inptr00;
  $85 = (($84) + 1|0);
  $inptr00 = $85;
  $86 = HEAP8[$84>>0]|0;
  $87 = $86&255;
  $y = $87;
  $88 = $y;
  $89 = $cred;
  $90 = (($88) + ($89))|0;
  $91 = $range_limit;
  $92 = (($91) + ($90)|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $outptr0;
  HEAP8[$94>>0] = $93;
  $95 = $y;
  $96 = $cgreen;
  $97 = (($95) + ($96))|0;
  $98 = $range_limit;
  $99 = (($98) + ($97)|0);
  $100 = HEAP8[$99>>0]|0;
  $101 = $outptr0;
  $102 = (($101) + 1|0);
  HEAP8[$102>>0] = $100;
  $103 = $y;
  $104 = $cblue;
  $105 = (($103) + ($104))|0;
  $106 = $range_limit;
  $107 = (($106) + ($105)|0);
  $108 = HEAP8[$107>>0]|0;
  $109 = $outptr0;
  $110 = (($109) + 2|0);
  HEAP8[$110>>0] = $108;
  $111 = $outptr0;
  $112 = (($111) + 3|0);
  $outptr0 = $112;
  $113 = $inptr00;
  $114 = (($113) + 1|0);
  $inptr00 = $114;
  $115 = HEAP8[$113>>0]|0;
  $116 = $115&255;
  $y = $116;
  $117 = $y;
  $118 = $cred;
  $119 = (($117) + ($118))|0;
  $120 = $range_limit;
  $121 = (($120) + ($119)|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $outptr0;
  HEAP8[$123>>0] = $122;
  $124 = $y;
  $125 = $cgreen;
  $126 = (($124) + ($125))|0;
  $127 = $range_limit;
  $128 = (($127) + ($126)|0);
  $129 = HEAP8[$128>>0]|0;
  $130 = $outptr0;
  $131 = (($130) + 1|0);
  HEAP8[$131>>0] = $129;
  $132 = $y;
  $133 = $cblue;
  $134 = (($132) + ($133))|0;
  $135 = $range_limit;
  $136 = (($135) + ($134)|0);
  $137 = HEAP8[$136>>0]|0;
  $138 = $outptr0;
  $139 = (($138) + 2|0);
  HEAP8[$139>>0] = $137;
  $140 = $outptr0;
  $141 = (($140) + 3|0);
  $outptr0 = $141;
  $142 = $inptr01;
  $143 = (($142) + 1|0);
  $inptr01 = $143;
  $144 = HEAP8[$142>>0]|0;
  $145 = $144&255;
  $y = $145;
  $146 = $y;
  $147 = $cred;
  $148 = (($146) + ($147))|0;
  $149 = $range_limit;
  $150 = (($149) + ($148)|0);
  $151 = HEAP8[$150>>0]|0;
  $152 = $outptr1;
  HEAP8[$152>>0] = $151;
  $153 = $y;
  $154 = $cgreen;
  $155 = (($153) + ($154))|0;
  $156 = $range_limit;
  $157 = (($156) + ($155)|0);
  $158 = HEAP8[$157>>0]|0;
  $159 = $outptr1;
  $160 = (($159) + 1|0);
  HEAP8[$160>>0] = $158;
  $161 = $y;
  $162 = $cblue;
  $163 = (($161) + ($162))|0;
  $164 = $range_limit;
  $165 = (($164) + ($163)|0);
  $166 = HEAP8[$165>>0]|0;
  $167 = $outptr1;
  $168 = (($167) + 2|0);
  HEAP8[$168>>0] = $166;
  $169 = $outptr1;
  $170 = (($169) + 3|0);
  $outptr1 = $170;
  $171 = $inptr01;
  $172 = (($171) + 1|0);
  $inptr01 = $172;
  $173 = HEAP8[$171>>0]|0;
  $174 = $173&255;
  $y = $174;
  $175 = $y;
  $176 = $cred;
  $177 = (($175) + ($176))|0;
  $178 = $range_limit;
  $179 = (($178) + ($177)|0);
  $180 = HEAP8[$179>>0]|0;
  $181 = $outptr1;
  HEAP8[$181>>0] = $180;
  $182 = $y;
  $183 = $cgreen;
  $184 = (($182) + ($183))|0;
  $185 = $range_limit;
  $186 = (($185) + ($184)|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $outptr1;
  $189 = (($188) + 1|0);
  HEAP8[$189>>0] = $187;
  $190 = $y;
  $191 = $cblue;
  $192 = (($190) + ($191))|0;
  $193 = $range_limit;
  $194 = (($193) + ($192)|0);
  $195 = HEAP8[$194>>0]|0;
  $196 = $outptr1;
  $197 = (($196) + 2|0);
  HEAP8[$197>>0] = $195;
  $198 = $outptr1;
  $199 = (($198) + 3|0);
  $outptr1 = $199;
  $200 = $col;
  $201 = (($200) + -1)|0;
  $col = $201;
 }
 $202 = $0;
 $203 = (($202) + 112|0);
 $204 = HEAP32[$203>>2]|0;
 $205 = $204 & 1;
 $206 = ($205|0)!=(0);
 if (!($206)) {
  STACKTOP = sp;return;
 }
 $207 = $inptr1;
 $208 = HEAP8[$207>>0]|0;
 $209 = $208&255;
 $cb = $209;
 $210 = $inptr2;
 $211 = HEAP8[$210>>0]|0;
 $212 = $211&255;
 $cr = $212;
 $213 = $cr;
 $214 = $Crrtab;
 $215 = (($214) + ($213<<2)|0);
 $216 = HEAP32[$215>>2]|0;
 $cred = $216;
 $217 = $cb;
 $218 = $Cbgtab;
 $219 = (($218) + ($217<<2)|0);
 $220 = HEAP32[$219>>2]|0;
 $221 = $cr;
 $222 = $Crgtab;
 $223 = (($222) + ($221<<2)|0);
 $224 = HEAP32[$223>>2]|0;
 $225 = (($220) + ($224))|0;
 $226 = $225 >> 16;
 $cgreen = $226;
 $227 = $cb;
 $228 = $Cbbtab;
 $229 = (($228) + ($227<<2)|0);
 $230 = HEAP32[$229>>2]|0;
 $cblue = $230;
 $231 = $inptr00;
 $232 = HEAP8[$231>>0]|0;
 $233 = $232&255;
 $y = $233;
 $234 = $y;
 $235 = $cred;
 $236 = (($234) + ($235))|0;
 $237 = $range_limit;
 $238 = (($237) + ($236)|0);
 $239 = HEAP8[$238>>0]|0;
 $240 = $outptr0;
 HEAP8[$240>>0] = $239;
 $241 = $y;
 $242 = $cgreen;
 $243 = (($241) + ($242))|0;
 $244 = $range_limit;
 $245 = (($244) + ($243)|0);
 $246 = HEAP8[$245>>0]|0;
 $247 = $outptr0;
 $248 = (($247) + 1|0);
 HEAP8[$248>>0] = $246;
 $249 = $y;
 $250 = $cblue;
 $251 = (($249) + ($250))|0;
 $252 = $range_limit;
 $253 = (($252) + ($251)|0);
 $254 = HEAP8[$253>>0]|0;
 $255 = $outptr0;
 $256 = (($255) + 2|0);
 HEAP8[$256>>0] = $254;
 $257 = $inptr01;
 $258 = HEAP8[$257>>0]|0;
 $259 = $258&255;
 $y = $259;
 $260 = $y;
 $261 = $cred;
 $262 = (($260) + ($261))|0;
 $263 = $range_limit;
 $264 = (($263) + ($262)|0);
 $265 = HEAP8[$264>>0]|0;
 $266 = $outptr1;
 HEAP8[$266>>0] = $265;
 $267 = $y;
 $268 = $cgreen;
 $269 = (($267) + ($268))|0;
 $270 = $range_limit;
 $271 = (($270) + ($269)|0);
 $272 = HEAP8[$271>>0]|0;
 $273 = $outptr1;
 $274 = (($273) + 1|0);
 HEAP8[$274>>0] = $272;
 $275 = $y;
 $276 = $cblue;
 $277 = (($275) + ($276))|0;
 $278 = $range_limit;
 $279 = (($278) + ($277)|0);
 $280 = HEAP8[$279>>0]|0;
 $281 = $outptr1;
 $282 = (($281) + 2|0);
 HEAP8[$282>>0] = $280;
 STACKTOP = sp;return;
}
function _merged_1v_upsample($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 452|0);
 $9 = HEAP32[$8>>2]|0;
 $upsample = $9;
 $10 = $upsample;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = $1;
 $15 = $2;
 $16 = HEAP32[$15>>2]|0;
 $17 = $4;
 $18 = $5;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + ($19<<2)|0);
 FUNCTION_TABLE_viiii[$12 & 255]($13,$14,$16,$20);
 $21 = $5;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 1)|0;
 HEAP32[$21>>2] = $23;
 $24 = $2;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$24>>2] = $26;
 STACKTOP = sp;return;
}
function _h2v1_merged_upsample($cinfo,$input_buf,$in_row_group_ctr,$output_buf) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cblue = 0, $cgreen = 0, $col = 0, $cr = 0, $cred = 0, $inptr0 = 0, $inptr1 = 0;
 var $inptr2 = 0, $outptr = 0, $range_limit = 0, $upsample = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $output_buf;
 $4 = $0;
 $5 = (($4) + 452|0);
 $6 = HEAP32[$5>>2]|0;
 $upsample = $6;
 $7 = $0;
 $8 = (($7) + 324|0);
 $9 = HEAP32[$8>>2]|0;
 $range_limit = $9;
 $10 = $upsample;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $Crrtab = $12;
 $13 = $upsample;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $Cbbtab = $15;
 $16 = $upsample;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $Crgtab = $18;
 $19 = $upsample;
 $20 = (($19) + 28|0);
 $21 = HEAP32[$20>>2]|0;
 $Cbgtab = $21;
 $22 = $2;
 $23 = $1;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + ($22<<2)|0);
 $26 = HEAP32[$25>>2]|0;
 $inptr0 = $26;
 $27 = $2;
 $28 = $1;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + ($27<<2)|0);
 $32 = HEAP32[$31>>2]|0;
 $inptr1 = $32;
 $33 = $2;
 $34 = $1;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + ($33<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $inptr2 = $38;
 $39 = $3;
 $40 = HEAP32[$39>>2]|0;
 $outptr = $40;
 $41 = $0;
 $42 = (($41) + 112|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $43 >>> 1;
 $col = $44;
 while(1) {
  $45 = $col;
  $46 = ($45>>>0)>(0);
  if (!($46)) {
   break;
  }
  $47 = $inptr1;
  $48 = (($47) + 1|0);
  $inptr1 = $48;
  $49 = HEAP8[$47>>0]|0;
  $50 = $49&255;
  $cb = $50;
  $51 = $inptr2;
  $52 = (($51) + 1|0);
  $inptr2 = $52;
  $53 = HEAP8[$51>>0]|0;
  $54 = $53&255;
  $cr = $54;
  $55 = $cr;
  $56 = $Crrtab;
  $57 = (($56) + ($55<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $cred = $58;
  $59 = $cb;
  $60 = $Cbgtab;
  $61 = (($60) + ($59<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = $cr;
  $64 = $Crgtab;
  $65 = (($64) + ($63<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($62) + ($66))|0;
  $68 = $67 >> 16;
  $cgreen = $68;
  $69 = $cb;
  $70 = $Cbbtab;
  $71 = (($70) + ($69<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $cblue = $72;
  $73 = $inptr0;
  $74 = (($73) + 1|0);
  $inptr0 = $74;
  $75 = HEAP8[$73>>0]|0;
  $76 = $75&255;
  $y = $76;
  $77 = $y;
  $78 = $cred;
  $79 = (($77) + ($78))|0;
  $80 = $range_limit;
  $81 = (($80) + ($79)|0);
  $82 = HEAP8[$81>>0]|0;
  $83 = $outptr;
  HEAP8[$83>>0] = $82;
  $84 = $y;
  $85 = $cgreen;
  $86 = (($84) + ($85))|0;
  $87 = $range_limit;
  $88 = (($87) + ($86)|0);
  $89 = HEAP8[$88>>0]|0;
  $90 = $outptr;
  $91 = (($90) + 1|0);
  HEAP8[$91>>0] = $89;
  $92 = $y;
  $93 = $cblue;
  $94 = (($92) + ($93))|0;
  $95 = $range_limit;
  $96 = (($95) + ($94)|0);
  $97 = HEAP8[$96>>0]|0;
  $98 = $outptr;
  $99 = (($98) + 2|0);
  HEAP8[$99>>0] = $97;
  $100 = $outptr;
  $101 = (($100) + 3|0);
  $outptr = $101;
  $102 = $inptr0;
  $103 = (($102) + 1|0);
  $inptr0 = $103;
  $104 = HEAP8[$102>>0]|0;
  $105 = $104&255;
  $y = $105;
  $106 = $y;
  $107 = $cred;
  $108 = (($106) + ($107))|0;
  $109 = $range_limit;
  $110 = (($109) + ($108)|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = $outptr;
  HEAP8[$112>>0] = $111;
  $113 = $y;
  $114 = $cgreen;
  $115 = (($113) + ($114))|0;
  $116 = $range_limit;
  $117 = (($116) + ($115)|0);
  $118 = HEAP8[$117>>0]|0;
  $119 = $outptr;
  $120 = (($119) + 1|0);
  HEAP8[$120>>0] = $118;
  $121 = $y;
  $122 = $cblue;
  $123 = (($121) + ($122))|0;
  $124 = $range_limit;
  $125 = (($124) + ($123)|0);
  $126 = HEAP8[$125>>0]|0;
  $127 = $outptr;
  $128 = (($127) + 2|0);
  HEAP8[$128>>0] = $126;
  $129 = $outptr;
  $130 = (($129) + 3|0);
  $outptr = $130;
  $131 = $col;
  $132 = (($131) + -1)|0;
  $col = $132;
 }
 $133 = $0;
 $134 = (($133) + 112|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = $135 & 1;
 $137 = ($136|0)!=(0);
 if (!($137)) {
  STACKTOP = sp;return;
 }
 $138 = $inptr1;
 $139 = HEAP8[$138>>0]|0;
 $140 = $139&255;
 $cb = $140;
 $141 = $inptr2;
 $142 = HEAP8[$141>>0]|0;
 $143 = $142&255;
 $cr = $143;
 $144 = $cr;
 $145 = $Crrtab;
 $146 = (($145) + ($144<<2)|0);
 $147 = HEAP32[$146>>2]|0;
 $cred = $147;
 $148 = $cb;
 $149 = $Cbgtab;
 $150 = (($149) + ($148<<2)|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = $cr;
 $153 = $Crgtab;
 $154 = (($153) + ($152<<2)|0);
 $155 = HEAP32[$154>>2]|0;
 $156 = (($151) + ($155))|0;
 $157 = $156 >> 16;
 $cgreen = $157;
 $158 = $cb;
 $159 = $Cbbtab;
 $160 = (($159) + ($158<<2)|0);
 $161 = HEAP32[$160>>2]|0;
 $cblue = $161;
 $162 = $inptr0;
 $163 = HEAP8[$162>>0]|0;
 $164 = $163&255;
 $y = $164;
 $165 = $y;
 $166 = $cred;
 $167 = (($165) + ($166))|0;
 $168 = $range_limit;
 $169 = (($168) + ($167)|0);
 $170 = HEAP8[$169>>0]|0;
 $171 = $outptr;
 HEAP8[$171>>0] = $170;
 $172 = $y;
 $173 = $cgreen;
 $174 = (($172) + ($173))|0;
 $175 = $range_limit;
 $176 = (($175) + ($174)|0);
 $177 = HEAP8[$176>>0]|0;
 $178 = $outptr;
 $179 = (($178) + 1|0);
 HEAP8[$179>>0] = $177;
 $180 = $y;
 $181 = $cblue;
 $182 = (($180) + ($181))|0;
 $183 = $range_limit;
 $184 = (($183) + ($182)|0);
 $185 = HEAP8[$184>>0]|0;
 $186 = $outptr;
 $187 = (($186) + 2|0);
 HEAP8[$187>>0] = $185;
 STACKTOP = sp;return;
}
function _build_ycc_rgb_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $i = 0, $upsample = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $upsample = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 255]($8,1,1024)|0);
 $10 = $upsample;
 $11 = (($10) + 16|0);
 HEAP32[$11>>2] = $9;
 $12 = $0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 $17 = (FUNCTION_TABLE_iiii[$15 & 255]($16,1,1024)|0);
 $18 = $upsample;
 $19 = (($18) + 20|0);
 HEAP32[$19>>2] = $17;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = (FUNCTION_TABLE_iiii[$23 & 255]($24,1,1024)|0);
 $26 = $upsample;
 $27 = (($26) + 24|0);
 HEAP32[$27>>2] = $25;
 $28 = $0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = $0;
 $33 = (FUNCTION_TABLE_iiii[$31 & 255]($32,1,1024)|0);
 $34 = $upsample;
 $35 = (($34) + 28|0);
 HEAP32[$35>>2] = $33;
 $i = 0;
 $x = -128;
 while(1) {
  $36 = $i;
  $37 = ($36|0)<=(255);
  if (!($37)) {
   break;
  }
  $38 = $x;
  $39 = ($38*91881)|0;
  $40 = (($39) + 32768)|0;
  $41 = $40 >> 16;
  $42 = $i;
  $43 = $upsample;
  $44 = (($43) + 16|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  HEAP32[$46>>2] = $41;
  $47 = $x;
  $48 = ($47*116130)|0;
  $49 = (($48) + 32768)|0;
  $50 = $49 >> 16;
  $51 = $i;
  $52 = $upsample;
  $53 = (($52) + 20|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  HEAP32[$55>>2] = $50;
  $56 = $x;
  $57 = Math_imul(-46802, $56)|0;
  $58 = $i;
  $59 = $upsample;
  $60 = (($59) + 24|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + ($58<<2)|0);
  HEAP32[$62>>2] = $57;
  $63 = $x;
  $64 = Math_imul(-22554, $63)|0;
  $65 = (($64) + 32768)|0;
  $66 = $i;
  $67 = $upsample;
  $68 = (($67) + 28|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + ($66<<2)|0);
  HEAP32[$70>>2] = $65;
  $71 = $i;
  $72 = (($71) + 1)|0;
  $i = $72;
  $73 = $x;
  $74 = (($73) + 1)|0;
  $x = $74;
 }
 STACKTOP = sp;return;
}
function _jinit_phuff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $coef_bit_ptr = 0, $entropy = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,64)|0);
 $entropy = $6;
 $7 = $entropy;
 $8 = $0;
 $9 = (($8) + 444|0);
 HEAP32[$9>>2] = $7;
 $10 = $entropy;
 HEAP32[$10>>2] = 51;
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = ($11|0)<(4);
  if (!($12)) {
   break;
  }
  $13 = $i;
  $14 = $entropy;
  $15 = (($14) + 44|0);
  $16 = (($15) + ($13<<2)|0);
  HEAP32[$16>>2] = 0;
  $17 = $i;
  $18 = (($17) + 1)|0;
  $i = $18;
 }
 $19 = $0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = $0;
 $25 = (($24) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $26<<6;
 $28 = $27<<2;
 $29 = (FUNCTION_TABLE_iiii[$22 & 255]($23,1,$28)|0);
 $30 = $0;
 $31 = (($30) + 160|0);
 HEAP32[$31>>2] = $29;
 $32 = $0;
 $33 = (($32) + 160|0);
 $34 = HEAP32[$33>>2]|0;
 $coef_bit_ptr = $34;
 $ci = 0;
 while(1) {
  $35 = $ci;
  $36 = $0;
  $37 = (($36) + 36|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($35|0)<($38|0);
  if (!($39)) {
   break;
  }
  $i = 0;
  while(1) {
   $40 = $i;
   $41 = ($40|0)<(64);
   if (!($41)) {
    break;
   }
   $42 = $coef_bit_ptr;
   $43 = (($42) + 4|0);
   $coef_bit_ptr = $43;
   HEAP32[$42>>2] = -1;
   $44 = $i;
   $45 = (($44) + 1)|0;
   $i = $45;
  }
  $46 = $ci;
  $47 = (($46) + 1)|0;
  $ci = $47;
 }
 STACKTOP = sp;return;
}
function _start_pass_phuff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bad = 0, $ci = 0, $cindex = 0, $coef_bit_ptr = 0, $coefi = 0, $compptr = 0, $entropy = 0, $expected = 0, $is_DC_band = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 444|0);
 $3 = HEAP32[$2>>2]|0;
 $entropy = $3;
 $4 = $0;
 $5 = (($4) + 400|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0);
 $8 = $7&1;
 $is_DC_band = $8;
 $bad = 0;
 $9 = $is_DC_band;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $0;
  $12 = (($11) + 404|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0);
  if ($14) {
   $bad = 1;
  }
 } else {
  $15 = $0;
  $16 = (($15) + 400|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  $19 = (($18) + 404|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)>($20|0);
  if ($21) {
   label = 7;
  } else {
   $22 = $0;
   $23 = (($22) + 404|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)>=(64);
   if ($25) {
    label = 7;
   }
  }
  if ((label|0) == 7) {
   $bad = 1;
  }
  $26 = $0;
  $27 = (($26) + 328|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(1);
  if ($29) {
   $bad = 1;
  }
 }
 $30 = $0;
 $31 = (($30) + 408|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = $0;
  $35 = (($34) + 412|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $0;
  $38 = (($37) + 408|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) - 1)|0;
  $41 = ($36|0)!=($40|0);
  if ($41) {
   $bad = 1;
  }
 }
 $42 = $0;
 $43 = (($42) + 412|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)>(13);
 if ($45) {
  $bad = 1;
 }
 $46 = $bad;
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = $0;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 20|0);
  HEAP32[$50>>2] = 16;
  $51 = $0;
  $52 = (($51) + 400|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $0;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + 24|0);
  HEAP32[$56>>2] = $53;
  $57 = $0;
  $58 = (($57) + 404|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $0;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 24|0);
  $63 = (($62) + 4|0);
  HEAP32[$63>>2] = $59;
  $64 = $0;
  $65 = (($64) + 408|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $0;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + 24|0);
  $70 = (($69) + 8|0);
  HEAP32[$70>>2] = $66;
  $71 = $0;
  $72 = (($71) + 412|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 24|0);
  $77 = (($76) + 12|0);
  HEAP32[$77>>2] = $73;
  $78 = $0;
  $79 = HEAP32[$78>>2]|0;
  $80 = HEAP32[$79>>2]|0;
  $81 = $0;
  FUNCTION_TABLE_vi[$80 & 255]($81);
 }
 $ci = 0;
 while(1) {
  $82 = $ci;
  $83 = $0;
  $84 = (($83) + 328|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($82|0)<($85|0);
  if (!($86)) {
   break;
  }
  $87 = $ci;
  $88 = $0;
  $89 = (($88) + 332|0);
  $90 = (($89) + ($87<<2)|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $cindex = $93;
  $94 = $cindex;
  $95 = $0;
  $96 = (($95) + 160|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = (($97) + ($94<<8)|0);
  $coef_bit_ptr = $98;
  $99 = $is_DC_band;
  $100 = ($99|0)!=(0);
  if (!($100)) {
   $101 = $coef_bit_ptr;
   $102 = HEAP32[$101>>2]|0;
   $103 = ($102|0)<(0);
   if ($103) {
    $104 = $0;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($105) + 20|0);
    HEAP32[$106>>2] = 115;
    $107 = $cindex;
    $108 = $0;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 24|0);
    HEAP32[$110>>2] = $107;
    $111 = $0;
    $112 = HEAP32[$111>>2]|0;
    $113 = (($112) + 24|0);
    $114 = (($113) + 4|0);
    HEAP32[$114>>2] = 0;
    $115 = $0;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($116) + 4|0);
    $118 = HEAP32[$117>>2]|0;
    $119 = $0;
    FUNCTION_TABLE_vii[$118 & 255]($119,-1);
   }
  }
  $120 = $0;
  $121 = (($120) + 400|0);
  $122 = HEAP32[$121>>2]|0;
  $coefi = $122;
  while(1) {
   $123 = $coefi;
   $124 = $0;
   $125 = (($124) + 404|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = ($123|0)<=($126|0);
   if (!($127)) {
    break;
   }
   $128 = $coefi;
   $129 = $coef_bit_ptr;
   $130 = (($129) + ($128<<2)|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = ($131|0)<(0);
   if ($132) {
    $137 = 0;
   } else {
    $133 = $coefi;
    $134 = $coef_bit_ptr;
    $135 = (($134) + ($133<<2)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = $136;
   }
   $expected = $137;
   $138 = $0;
   $139 = (($138) + 408|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = $expected;
   $142 = ($140|0)!=($141|0);
   if ($142) {
    $143 = $0;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($144) + 20|0);
    HEAP32[$145>>2] = 115;
    $146 = $cindex;
    $147 = $0;
    $148 = HEAP32[$147>>2]|0;
    $149 = (($148) + 24|0);
    HEAP32[$149>>2] = $146;
    $150 = $coefi;
    $151 = $0;
    $152 = HEAP32[$151>>2]|0;
    $153 = (($152) + 24|0);
    $154 = (($153) + 4|0);
    HEAP32[$154>>2] = $150;
    $155 = $0;
    $156 = HEAP32[$155>>2]|0;
    $157 = (($156) + 4|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = $0;
    FUNCTION_TABLE_vii[$158 & 255]($159,-1);
   }
   $160 = $0;
   $161 = (($160) + 412|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = $coefi;
   $164 = $coef_bit_ptr;
   $165 = (($164) + ($163<<2)|0);
   HEAP32[$165>>2] = $162;
   $166 = $coefi;
   $167 = (($166) + 1)|0;
   $coefi = $167;
  }
  $168 = $ci;
  $169 = (($168) + 1)|0;
  $ci = $169;
 }
 $170 = $0;
 $171 = (($170) + 408|0);
 $172 = HEAP32[$171>>2]|0;
 $173 = ($172|0)==(0);
 if ($173) {
  $174 = $is_DC_band;
  $175 = ($174|0)!=(0);
  if ($175) {
   $176 = $entropy;
   $177 = (($176) + 4|0);
   HEAP32[$177>>2] = 52;
  } else {
   $178 = $entropy;
   $179 = (($178) + 4|0);
   HEAP32[$179>>2] = 53;
  }
 } else {
  $180 = $is_DC_band;
  $181 = ($180|0)!=(0);
  if ($181) {
   $182 = $entropy;
   $183 = (($182) + 4|0);
   HEAP32[$183>>2] = 54;
  } else {
   $184 = $entropy;
   $185 = (($184) + 4|0);
   HEAP32[$185>>2] = 55;
  }
 }
 $ci = 0;
 while(1) {
  $186 = $ci;
  $187 = $0;
  $188 = (($187) + 328|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = ($186|0)<($189|0);
  if (!($190)) {
   break;
  }
  $191 = $ci;
  $192 = $0;
  $193 = (($192) + 332|0);
  $194 = (($193) + ($191<<2)|0);
  $195 = HEAP32[$194>>2]|0;
  $compptr = $195;
  $196 = $is_DC_band;
  $197 = ($196|0)!=(0);
  if ($197) {
   $198 = $0;
   $199 = (($198) + 408|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = ($200|0)==(0);
   if ($201) {
    $202 = $compptr;
    $203 = (($202) + 20|0);
    $204 = HEAP32[$203>>2]|0;
    $tbl = $204;
    $205 = $0;
    $206 = $tbl;
    $207 = $tbl;
    $208 = $entropy;
    $209 = (($208) + 44|0);
    $210 = (($209) + ($207<<2)|0);
    _jpeg_make_d_derived_tbl($205,1,$206,$210);
   }
  } else {
   $211 = $compptr;
   $212 = (($211) + 24|0);
   $213 = HEAP32[$212>>2]|0;
   $tbl = $213;
   $214 = $0;
   $215 = $tbl;
   $216 = $tbl;
   $217 = $entropy;
   $218 = (($217) + 44|0);
   $219 = (($218) + ($216<<2)|0);
   _jpeg_make_d_derived_tbl($214,0,$215,$219);
   $220 = $tbl;
   $221 = $entropy;
   $222 = (($221) + 44|0);
   $223 = (($222) + ($220<<2)|0);
   $224 = HEAP32[$223>>2]|0;
   $225 = $entropy;
   $226 = (($225) + 60|0);
   HEAP32[$226>>2] = $224;
  }
  $227 = $ci;
  $228 = $entropy;
  $229 = (($228) + 20|0);
  $230 = (($229) + 4|0);
  $231 = (($230) + ($227<<2)|0);
  HEAP32[$231>>2] = 0;
  $232 = $ci;
  $233 = (($232) + 1)|0;
  $ci = $233;
 }
 $234 = $entropy;
 $235 = (($234) + 12|0);
 $236 = (($235) + 4|0);
 HEAP32[$236>>2] = 0;
 $237 = $entropy;
 $238 = (($237) + 12|0);
 HEAP32[$238>>2] = 0;
 $239 = $entropy;
 $240 = (($239) + 8|0);
 HEAP32[$240>>2] = 0;
 $241 = $entropy;
 $242 = (($241) + 20|0);
 HEAP32[$242>>2] = 0;
 $243 = $0;
 $244 = (($243) + 276|0);
 $245 = HEAP32[$244>>2]|0;
 $246 = $entropy;
 $247 = (($246) + 40|0);
 HEAP32[$247>>2] = $245;
 STACKTOP = sp;return;
}
function _decode_mcu_DC_first($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Al = 0, $bits_left = 0, $blkn = 0, $block = 0, $br_state = 0, $ci = 0, $compptr = 0, $entropy = 0, $get_buffer = 0, $look = 0, $nb = 0;
 var $r = 0, $s = 0, $state = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $br_state = sp + 36|0;
 $state = sp + 16|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 412|0);
 $8 = HEAP32[$7>>2]|0;
 $Al = $8;
 $9 = $1;
 $10 = (($9) + 276|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $entropy;
  $14 = (($13) + 40|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)==(0);
  do {
   if ($16) {
    $17 = $1;
    $18 = (_process_restart($17)|0);
    $19 = ($18|0)!=(0);
    if ($19) {
     break;
    }
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
  } while(0);
 }
 $20 = $entropy;
 $21 = (($20) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0);
 do {
  if (!($23)) {
   $24 = $1;
   $25 = (($br_state) + 16|0);
   HEAP32[$25>>2] = $24;
   $26 = $1;
   $27 = (($26) + 24|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$br_state>>2] = $29;
   $30 = $1;
   $31 = (($30) + 24|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($br_state) + 4|0);
   HEAP32[$35>>2] = $34;
   $36 = $entropy;
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $get_buffer = $38;
   $39 = $entropy;
   $40 = (($39) + 12|0);
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $bits_left = $42;
   $43 = $entropy;
   $44 = (($43) + 20|0);
   ;HEAP32[$state+0>>2]=HEAP32[$44+0>>2]|0;HEAP32[$state+4>>2]=HEAP32[$44+4>>2]|0;HEAP32[$state+8>>2]=HEAP32[$44+8>>2]|0;HEAP32[$state+12>>2]=HEAP32[$44+12>>2]|0;HEAP32[$state+16>>2]=HEAP32[$44+16>>2]|0;
   $blkn = 0;
   L13: while(1) {
    $45 = $blkn;
    $46 = $1;
    $47 = (($46) + 356|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = ($45|0)<($48|0);
    if (!($49)) {
     label = 33;
     break;
    }
    $50 = $blkn;
    $51 = $2;
    $52 = (($51) + ($50<<2)|0);
    $53 = HEAP32[$52>>2]|0;
    $block = $53;
    $54 = $blkn;
    $55 = $1;
    $56 = (($55) + 360|0);
    $57 = (($56) + ($54<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $ci = $58;
    $59 = $ci;
    $60 = $1;
    $61 = (($60) + 332|0);
    $62 = (($61) + ($59<<2)|0);
    $63 = HEAP32[$62>>2]|0;
    $compptr = $63;
    $64 = $compptr;
    $65 = (($64) + 20|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = $entropy;
    $68 = (($67) + 44|0);
    $69 = (($68) + ($66<<2)|0);
    $70 = HEAP32[$69>>2]|0;
    $tbl = $70;
    $71 = $bits_left;
    $72 = ($71|0)<(8);
    do {
     if ($72) {
      $73 = $get_buffer;
      $74 = $bits_left;
      $75 = (_jpeg_fill_bit_buffer($br_state,$73,$74,0)|0);
      $76 = ($75|0)!=(0);
      if (!($76)) {
       label = 12;
       break L13;
      }
      $77 = (($br_state) + 8|0);
      $78 = HEAP32[$77>>2]|0;
      $get_buffer = $78;
      $79 = (($br_state) + 12|0);
      $80 = HEAP32[$79>>2]|0;
      $bits_left = $80;
      $81 = $bits_left;
      $82 = ($81|0)<(8);
      if ($82) {
       $nb = 1;
       label = 19;
       break;
      } else {
       label = 16;
       break;
      }
     } else {
      label = 16;
     }
    } while(0);
    do {
     if ((label|0) == 16) {
      label = 0;
      $83 = $get_buffer;
      $84 = $bits_left;
      $85 = (($84) - 8)|0;
      $86 = $83 >> $85;
      $87 = $86 & 255;
      $look = $87;
      $88 = $look;
      $89 = $tbl;
      $90 = (($89) + 144|0);
      $91 = (($90) + ($88<<2)|0);
      $92 = HEAP32[$91>>2]|0;
      $nb = $92;
      $93 = ($92|0)!=(0);
      if ($93) {
       $94 = $nb;
       $95 = $bits_left;
       $96 = (($95) - ($94))|0;
       $bits_left = $96;
       $97 = $look;
       $98 = $tbl;
       $99 = (($98) + 1168|0);
       $100 = (($99) + ($97)|0);
       $101 = HEAP8[$100>>0]|0;
       $102 = $101&255;
       $s = $102;
       break;
      } else {
       $nb = 9;
       label = 19;
       break;
      }
     }
    } while(0);
    if ((label|0) == 19) {
     label = 0;
     $103 = $get_buffer;
     $104 = $bits_left;
     $105 = $tbl;
     $106 = $nb;
     $107 = (_jpeg_huff_decode($br_state,$103,$104,$105,$106)|0);
     $s = $107;
     $108 = ($107|0)<(0);
     if ($108) {
      label = 20;
      break;
     }
     $109 = (($br_state) + 8|0);
     $110 = HEAP32[$109>>2]|0;
     $get_buffer = $110;
     $111 = (($br_state) + 12|0);
     $112 = HEAP32[$111>>2]|0;
     $bits_left = $112;
    }
    $113 = $s;
    $114 = ($113|0)!=(0);
    if ($114) {
     $115 = $bits_left;
     $116 = $s;
     $117 = ($115|0)<($116|0);
     if ($117) {
      $118 = $get_buffer;
      $119 = $bits_left;
      $120 = $s;
      $121 = (_jpeg_fill_bit_buffer($br_state,$118,$119,$120)|0);
      $122 = ($121|0)!=(0);
      if (!($122)) {
       label = 25;
       break;
      }
      $123 = (($br_state) + 8|0);
      $124 = HEAP32[$123>>2]|0;
      $get_buffer = $124;
      $125 = (($br_state) + 12|0);
      $126 = HEAP32[$125>>2]|0;
      $bits_left = $126;
     }
     $127 = $get_buffer;
     $128 = $s;
     $129 = $bits_left;
     $130 = (($129) - ($128))|0;
     $bits_left = $130;
     $131 = $127 >> $130;
     $132 = $s;
     $133 = 1 << $132;
     $134 = (($133) - 1)|0;
     $135 = $131 & $134;
     $r = $135;
     $136 = $r;
     $137 = $s;
     $138 = (44584 + ($137<<2)|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($136|0)<($139|0);
     if ($140) {
      $141 = $r;
      $142 = $s;
      $143 = (44648 + ($142<<2)|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = (($141) + ($144))|0;
      $147 = $145;
     } else {
      $146 = $r;
      $147 = $146;
     }
     $s = $147;
    }
    $148 = $ci;
    $149 = (($state) + 4|0);
    $150 = (($149) + ($148<<2)|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = $s;
    $153 = (($152) + ($151))|0;
    $s = $153;
    $154 = $s;
    $155 = $ci;
    $156 = (($state) + 4|0);
    $157 = (($156) + ($155<<2)|0);
    HEAP32[$157>>2] = $154;
    $158 = $s;
    $159 = $Al;
    $160 = $158 << $159;
    $161 = $160&65535;
    $162 = $block;
    HEAP16[$162>>1] = $161;
    $163 = $blkn;
    $164 = (($163) + 1)|0;
    $blkn = $164;
   }
   if ((label|0) == 12) {
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
   else if ((label|0) == 20) {
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
   else if ((label|0) == 25) {
    $0 = 0;
    $188 = $0;
    STACKTOP = sp;return ($188|0);
   }
   else if ((label|0) == 33) {
    $165 = HEAP32[$br_state>>2]|0;
    $166 = $1;
    $167 = (($166) + 24|0);
    $168 = HEAP32[$167>>2]|0;
    HEAP32[$168>>2] = $165;
    $169 = (($br_state) + 4|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = $1;
    $172 = (($171) + 24|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = (($173) + 4|0);
    HEAP32[$174>>2] = $170;
    $175 = $get_buffer;
    $176 = $entropy;
    $177 = (($176) + 12|0);
    HEAP32[$177>>2] = $175;
    $178 = $bits_left;
    $179 = $entropy;
    $180 = (($179) + 12|0);
    $181 = (($180) + 4|0);
    HEAP32[$181>>2] = $178;
    $182 = $entropy;
    $183 = (($182) + 20|0);
    ;HEAP32[$183+0>>2]=HEAP32[$state+0>>2]|0;HEAP32[$183+4>>2]=HEAP32[$state+4>>2]|0;HEAP32[$183+8>>2]=HEAP32[$state+8>>2]|0;HEAP32[$183+12>>2]=HEAP32[$state+12>>2]|0;HEAP32[$183+16>>2]=HEAP32[$state+16>>2]|0;
    break;
   }
  }
 } while(0);
 $184 = $entropy;
 $185 = (($184) + 40|0);
 $186 = HEAP32[$185>>2]|0;
 $187 = (($186) + -1)|0;
 HEAP32[$185>>2] = $187;
 $0 = 1;
 $188 = $0;
 STACKTOP = sp;return ($188|0);
}
function _decode_mcu_AC_first($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Al = 0, $EOBRUN = 0;
 var $Se = 0, $bits_left = 0, $block = 0, $br_state = 0, $entropy = 0, $get_buffer = 0, $k = 0, $look = 0, $nb = 0, $r = 0, $s = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $br_state = sp + 12|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 404|0);
 $8 = HEAP32[$7>>2]|0;
 $Se = $8;
 $9 = $1;
 $10 = (($9) + 412|0);
 $11 = HEAP32[$10>>2]|0;
 $Al = $11;
 $12 = $1;
 $13 = (($12) + 276|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $entropy;
  $17 = (($16) + 40|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)==(0);
  do {
   if ($19) {
    $20 = $1;
    $21 = (_process_restart($20)|0);
    $22 = ($21|0)!=(0);
    if ($22) {
     break;
    }
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
  } while(0);
 }
 $23 = $entropy;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $27 = $entropy;
  $28 = (($27) + 20|0);
  $29 = HEAP32[$28>>2]|0;
  $EOBRUN = $29;
  $30 = $EOBRUN;
  $31 = ($30>>>0)>(0);
  if ($31) {
   $32 = $EOBRUN;
   $33 = (($32) + -1)|0;
   $EOBRUN = $33;
  } else {
   $34 = $1;
   $35 = (($br_state) + 16|0);
   HEAP32[$35>>2] = $34;
   $36 = $1;
   $37 = (($36) + 24|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   HEAP32[$br_state>>2] = $39;
   $40 = $1;
   $41 = (($40) + 24|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($br_state) + 4|0);
   HEAP32[$45>>2] = $44;
   $46 = $entropy;
   $47 = (($46) + 12|0);
   $48 = HEAP32[$47>>2]|0;
   $get_buffer = $48;
   $49 = $entropy;
   $50 = (($49) + 12|0);
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $bits_left = $52;
   $53 = $2;
   $54 = HEAP32[$53>>2]|0;
   $block = $54;
   $55 = $entropy;
   $56 = (($55) + 60|0);
   $57 = HEAP32[$56>>2]|0;
   $tbl = $57;
   $58 = $1;
   $59 = (($58) + 400|0);
   $60 = HEAP32[$59>>2]|0;
   $k = $60;
   L16: while(1) {
    $61 = $k;
    $62 = $Se;
    $63 = ($61|0)<=($62|0);
    if (!($63)) {
     break;
    }
    $64 = $bits_left;
    $65 = ($64|0)<(8);
    do {
     if ($65) {
      $66 = $get_buffer;
      $67 = $bits_left;
      $68 = (_jpeg_fill_bit_buffer($br_state,$66,$67,0)|0);
      $69 = ($68|0)!=(0);
      if (!($69)) {
       label = 14;
       break L16;
      }
      $70 = (($br_state) + 8|0);
      $71 = HEAP32[$70>>2]|0;
      $get_buffer = $71;
      $72 = (($br_state) + 12|0);
      $73 = HEAP32[$72>>2]|0;
      $bits_left = $73;
      $74 = $bits_left;
      $75 = ($74|0)<(8);
      if ($75) {
       $nb = 1;
       label = 21;
       break;
      } else {
       label = 18;
       break;
      }
     } else {
      label = 18;
     }
    } while(0);
    do {
     if ((label|0) == 18) {
      label = 0;
      $76 = $get_buffer;
      $77 = $bits_left;
      $78 = (($77) - 8)|0;
      $79 = $76 >> $78;
      $80 = $79 & 255;
      $look = $80;
      $81 = $look;
      $82 = $tbl;
      $83 = (($82) + 144|0);
      $84 = (($83) + ($81<<2)|0);
      $85 = HEAP32[$84>>2]|0;
      $nb = $85;
      $86 = ($85|0)!=(0);
      if ($86) {
       $87 = $nb;
       $88 = $bits_left;
       $89 = (($88) - ($87))|0;
       $bits_left = $89;
       $90 = $look;
       $91 = $tbl;
       $92 = (($91) + 1168|0);
       $93 = (($92) + ($90)|0);
       $94 = HEAP8[$93>>0]|0;
       $95 = $94&255;
       $s = $95;
       break;
      } else {
       $nb = 9;
       label = 21;
       break;
      }
     }
    } while(0);
    if ((label|0) == 21) {
     label = 0;
     $96 = $get_buffer;
     $97 = $bits_left;
     $98 = $tbl;
     $99 = $nb;
     $100 = (_jpeg_huff_decode($br_state,$96,$97,$98,$99)|0);
     $s = $100;
     $101 = ($100|0)<(0);
     if ($101) {
      label = 22;
      break;
     }
     $102 = (($br_state) + 8|0);
     $103 = HEAP32[$102>>2]|0;
     $get_buffer = $103;
     $104 = (($br_state) + 12|0);
     $105 = HEAP32[$104>>2]|0;
     $bits_left = $105;
    }
    $106 = $s;
    $107 = $106 >> 4;
    $r = $107;
    $108 = $s;
    $109 = $108 & 15;
    $s = $109;
    $110 = $s;
    $111 = ($110|0)!=(0);
    if ($111) {
     $112 = $r;
     $113 = $k;
     $114 = (($113) + ($112))|0;
     $k = $114;
     $115 = $bits_left;
     $116 = $s;
     $117 = ($115|0)<($116|0);
     if ($117) {
      $118 = $get_buffer;
      $119 = $bits_left;
      $120 = $s;
      $121 = (_jpeg_fill_bit_buffer($br_state,$118,$119,$120)|0);
      $122 = ($121|0)!=(0);
      if (!($122)) {
       label = 27;
       break;
      }
      $123 = (($br_state) + 8|0);
      $124 = HEAP32[$123>>2]|0;
      $get_buffer = $124;
      $125 = (($br_state) + 12|0);
      $126 = HEAP32[$125>>2]|0;
      $bits_left = $126;
     }
     $127 = $get_buffer;
     $128 = $s;
     $129 = $bits_left;
     $130 = (($129) - ($128))|0;
     $bits_left = $130;
     $131 = $127 >> $130;
     $132 = $s;
     $133 = 1 << $132;
     $134 = (($133) - 1)|0;
     $135 = $131 & $134;
     $r = $135;
     $136 = $r;
     $137 = $s;
     $138 = (44584 + ($137<<2)|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($136|0)<($139|0);
     if ($140) {
      $141 = $r;
      $142 = $s;
      $143 = (44648 + ($142<<2)|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = (($141) + ($144))|0;
      $147 = $145;
     } else {
      $146 = $r;
      $147 = $146;
     }
     $s = $147;
     $148 = $s;
     $149 = $Al;
     $150 = $148 << $149;
     $151 = $150&65535;
     $152 = $k;
     $153 = (50504 + ($152<<2)|0);
     $154 = HEAPU8[$153>>0]|(HEAPU8[$153+1>>0]<<8)|(HEAPU8[$153+2>>0]<<16)|(HEAPU8[$153+3>>0]<<24);
     $155 = $block;
     $156 = (($155) + ($154<<1)|0);
     HEAP16[$156>>1] = $151;
    } else {
     $157 = $r;
     $158 = ($157|0)==(15);
     if (!($158)) {
      label = 35;
      break;
     }
     $159 = $k;
     $160 = (($159) + 15)|0;
     $k = $160;
    }
    $191 = $k;
    $192 = (($191) + 1)|0;
    $k = $192;
   }
   if ((label|0) == 14) {
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
   else if ((label|0) == 22) {
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
   else if ((label|0) == 27) {
    $0 = 0;
    $217 = $0;
    STACKTOP = sp;return ($217|0);
   }
   else if ((label|0) == 35) {
    $161 = $r;
    $162 = 1 << $161;
    $EOBRUN = $162;
    $163 = $r;
    $164 = ($163|0)!=(0);
    if ($164) {
     $165 = $bits_left;
     $166 = $r;
     $167 = ($165|0)<($166|0);
     do {
      if ($167) {
       $168 = $get_buffer;
       $169 = $bits_left;
       $170 = $r;
       $171 = (_jpeg_fill_bit_buffer($br_state,$168,$169,$170)|0);
       $172 = ($171|0)!=(0);
       if ($172) {
        $173 = (($br_state) + 8|0);
        $174 = HEAP32[$173>>2]|0;
        $get_buffer = $174;
        $175 = (($br_state) + 12|0);
        $176 = HEAP32[$175>>2]|0;
        $bits_left = $176;
        break;
       }
       $0 = 0;
       $217 = $0;
       STACKTOP = sp;return ($217|0);
      }
     } while(0);
     $177 = $get_buffer;
     $178 = $r;
     $179 = $bits_left;
     $180 = (($179) - ($178))|0;
     $bits_left = $180;
     $181 = $177 >> $180;
     $182 = $r;
     $183 = 1 << $182;
     $184 = (($183) - 1)|0;
     $185 = $181 & $184;
     $r = $185;
     $186 = $r;
     $187 = $EOBRUN;
     $188 = (($187) + ($186))|0;
     $EOBRUN = $188;
    }
    $189 = $EOBRUN;
    $190 = (($189) + -1)|0;
    $EOBRUN = $190;
   }
   $193 = HEAP32[$br_state>>2]|0;
   $194 = $1;
   $195 = (($194) + 24|0);
   $196 = HEAP32[$195>>2]|0;
   HEAP32[$196>>2] = $193;
   $197 = (($br_state) + 4|0);
   $198 = HEAP32[$197>>2]|0;
   $199 = $1;
   $200 = (($199) + 24|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = (($201) + 4|0);
   HEAP32[$202>>2] = $198;
   $203 = $get_buffer;
   $204 = $entropy;
   $205 = (($204) + 12|0);
   HEAP32[$205>>2] = $203;
   $206 = $bits_left;
   $207 = $entropy;
   $208 = (($207) + 12|0);
   $209 = (($208) + 4|0);
   HEAP32[$209>>2] = $206;
  }
  $210 = $EOBRUN;
  $211 = $entropy;
  $212 = (($211) + 20|0);
  HEAP32[$212>>2] = $210;
 }
 $213 = $entropy;
 $214 = (($213) + 40|0);
 $215 = HEAP32[$214>>2]|0;
 $216 = (($215) + -1)|0;
 HEAP32[$214>>2] = $216;
 $0 = 1;
 $217 = $0;
 STACKTOP = sp;return ($217|0);
}
function _decode_mcu_DC_refine($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $bits_left = 0, $blkn = 0, $block = 0, $br_state = 0, $entropy = 0;
 var $get_buffer = 0, $p1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $br_state = sp;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 412|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = 1 << $8;
 $p1 = $9;
 $10 = $1;
 $11 = (($10) + 276|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $entropy;
  $15 = (($14) + 40|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(0);
  do {
   if ($17) {
    $18 = $1;
    $19 = (_process_restart($18)|0);
    $20 = ($19|0)!=(0);
    if ($20) {
     break;
    }
    $0 = 0;
    $94 = $0;
    STACKTOP = sp;return ($94|0);
   }
  } while(0);
 }
 $21 = $1;
 $22 = (($br_state) + 16|0);
 HEAP32[$22>>2] = $21;
 $23 = $1;
 $24 = (($23) + 24|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$br_state>>2] = $26;
 $27 = $1;
 $28 = (($27) + 24|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($br_state) + 4|0);
 HEAP32[$32>>2] = $31;
 $33 = $entropy;
 $34 = (($33) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $get_buffer = $35;
 $36 = $entropy;
 $37 = (($36) + 12|0);
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $bits_left = $39;
 $blkn = 0;
 while(1) {
  $40 = $blkn;
  $41 = $1;
  $42 = (($41) + 356|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   label = 17;
   break;
  }
  $45 = $blkn;
  $46 = $2;
  $47 = (($46) + ($45<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $block = $48;
  $49 = $bits_left;
  $50 = ($49|0)<(1);
  if ($50) {
   $51 = $get_buffer;
   $52 = $bits_left;
   $53 = (_jpeg_fill_bit_buffer($br_state,$51,$52,1)|0);
   $54 = ($53|0)!=(0);
   if (!($54)) {
    label = 11;
    break;
   }
   $55 = (($br_state) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $get_buffer = $56;
   $57 = (($br_state) + 12|0);
   $58 = HEAP32[$57>>2]|0;
   $bits_left = $58;
  }
  $59 = $get_buffer;
  $60 = $bits_left;
  $61 = (($60) - 1)|0;
  $bits_left = $61;
  $62 = $59 >> $61;
  $63 = $62 & 1;
  $64 = ($63|0)!=(0);
  if ($64) {
   $65 = $p1;
   $66 = $block;
   $67 = HEAP16[$66>>1]|0;
   $68 = $67 << 16 >> 16;
   $69 = $68 | $65;
   $70 = $69&65535;
   HEAP16[$66>>1] = $70;
  }
  $71 = $blkn;
  $72 = (($71) + 1)|0;
  $blkn = $72;
 }
 if ((label|0) == 11) {
  $0 = 0;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 else if ((label|0) == 17) {
  $73 = HEAP32[$br_state>>2]|0;
  $74 = $1;
  $75 = (($74) + 24|0);
  $76 = HEAP32[$75>>2]|0;
  HEAP32[$76>>2] = $73;
  $77 = (($br_state) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = $1;
  $80 = (($79) + 24|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 4|0);
  HEAP32[$82>>2] = $78;
  $83 = $get_buffer;
  $84 = $entropy;
  $85 = (($84) + 12|0);
  HEAP32[$85>>2] = $83;
  $86 = $bits_left;
  $87 = $entropy;
  $88 = (($87) + 12|0);
  $89 = (($88) + 4|0);
  HEAP32[$89>>2] = $86;
  $90 = $entropy;
  $91 = (($90) + 40|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + -1)|0;
  HEAP32[$91>>2] = $93;
  $0 = 1;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 return 0|0;
}
function _decode_mcu_AC_refine($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $EOBRUN = 0, $Se = 0, $bits_left = 0, $block = 0, $br_state = 0, $entropy = 0, $get_buffer = 0;
 var $k = 0, $look = 0, $m1 = 0, $nb = 0, $newnz_pos = 0, $num_newnz = 0, $p1 = 0, $pos = 0, $r = 0, $s = 0, $tbl = 0, $thiscoef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $br_state = sp + 280|0;
 $newnz_pos = sp + 16|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 404|0);
 $8 = HEAP32[$7>>2]|0;
 $Se = $8;
 $9 = $1;
 $10 = (($9) + 412|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = 1 << $11;
 $p1 = $12;
 $13 = $1;
 $14 = (($13) + 412|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = -1 << $15;
 $m1 = $16;
 $17 = $1;
 $18 = (($17) + 276|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $entropy;
  $22 = (($21) + 40|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)==(0);
  do {
   if ($24) {
    $25 = $1;
    $26 = (_process_restart($25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     break;
    }
    $0 = 0;
    $332 = $0;
    STACKTOP = sp;return ($332|0);
   }
  } while(0);
 }
 $28 = $entropy;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0);
 L11: do {
  if (!($31)) {
   $32 = $1;
   $33 = (($br_state) + 16|0);
   HEAP32[$33>>2] = $32;
   $34 = $1;
   $35 = (($34) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$br_state>>2] = $37;
   $38 = $1;
   $39 = (($38) + 24|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($br_state) + 4|0);
   HEAP32[$43>>2] = $42;
   $44 = $entropy;
   $45 = (($44) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $get_buffer = $46;
   $47 = $entropy;
   $48 = (($47) + 12|0);
   $49 = (($48) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   $bits_left = $50;
   $51 = $entropy;
   $52 = (($51) + 20|0);
   $53 = HEAP32[$52>>2]|0;
   $EOBRUN = $53;
   $54 = $2;
   $55 = HEAP32[$54>>2]|0;
   $block = $55;
   $56 = $entropy;
   $57 = (($56) + 60|0);
   $58 = HEAP32[$57>>2]|0;
   $tbl = $58;
   $num_newnz = 0;
   $59 = $1;
   $60 = (($59) + 400|0);
   $61 = HEAP32[$60>>2]|0;
   $k = $61;
   $62 = $EOBRUN;
   $63 = ($62|0)==(0);
   L13: do {
    if ($63) {
     L15: while(1) {
      $64 = $k;
      $65 = $Se;
      $66 = ($64|0)<=($65|0);
      if (!($66)) {
       break;
      }
      $67 = $bits_left;
      $68 = ($67|0)<(8);
      do {
       if ($68) {
        $69 = $get_buffer;
        $70 = $bits_left;
        $71 = (_jpeg_fill_bit_buffer($br_state,$69,$70,0)|0);
        $72 = ($71|0)!=(0);
        if (!($72)) {
         label = 13;
         break L15;
        }
        $73 = (($br_state) + 8|0);
        $74 = HEAP32[$73>>2]|0;
        $get_buffer = $74;
        $75 = (($br_state) + 12|0);
        $76 = HEAP32[$75>>2]|0;
        $bits_left = $76;
        $77 = $bits_left;
        $78 = ($77|0)<(8);
        if ($78) {
         $nb = 1;
         label = 20;
         break;
        } else {
         label = 17;
         break;
        }
       } else {
        label = 17;
       }
      } while(0);
      do {
       if ((label|0) == 17) {
        label = 0;
        $79 = $get_buffer;
        $80 = $bits_left;
        $81 = (($80) - 8)|0;
        $82 = $79 >> $81;
        $83 = $82 & 255;
        $look = $83;
        $84 = $look;
        $85 = $tbl;
        $86 = (($85) + 144|0);
        $87 = (($86) + ($84<<2)|0);
        $88 = HEAP32[$87>>2]|0;
        $nb = $88;
        $89 = ($88|0)!=(0);
        if ($89) {
         $90 = $nb;
         $91 = $bits_left;
         $92 = (($91) - ($90))|0;
         $bits_left = $92;
         $93 = $look;
         $94 = $tbl;
         $95 = (($94) + 1168|0);
         $96 = (($95) + ($93)|0);
         $97 = HEAP8[$96>>0]|0;
         $98 = $97&255;
         $s = $98;
         break;
        } else {
         $nb = 9;
         label = 20;
         break;
        }
       }
      } while(0);
      if ((label|0) == 20) {
       label = 0;
       $99 = $get_buffer;
       $100 = $bits_left;
       $101 = $tbl;
       $102 = $nb;
       $103 = (_jpeg_huff_decode($br_state,$99,$100,$101,$102)|0);
       $s = $103;
       $104 = ($103|0)<(0);
       if ($104) {
        label = 21;
        break;
       }
       $105 = (($br_state) + 8|0);
       $106 = HEAP32[$105>>2]|0;
       $get_buffer = $106;
       $107 = (($br_state) + 12|0);
       $108 = HEAP32[$107>>2]|0;
       $bits_left = $108;
      }
      $109 = $s;
      $110 = $109 >> 4;
      $r = $110;
      $111 = $s;
      $112 = $111 & 15;
      $s = $112;
      $113 = $s;
      $114 = ($113|0)!=(0);
      if ($114) {
       $115 = $s;
       $116 = ($115|0)!=(1);
       if ($116) {
        $117 = $1;
        $118 = HEAP32[$117>>2]|0;
        $119 = (($118) + 20|0);
        HEAP32[$119>>2] = 118;
        $120 = $1;
        $121 = HEAP32[$120>>2]|0;
        $122 = (($121) + 4|0);
        $123 = HEAP32[$122>>2]|0;
        $124 = $1;
        FUNCTION_TABLE_vii[$123 & 255]($124,-1);
       }
       $125 = $bits_left;
       $126 = ($125|0)<(1);
       if ($126) {
        $127 = $get_buffer;
        $128 = $bits_left;
        $129 = (_jpeg_fill_bit_buffer($br_state,$127,$128,1)|0);
        $130 = ($129|0)!=(0);
        if (!($130)) {
         label = 28;
         break;
        }
        $131 = (($br_state) + 8|0);
        $132 = HEAP32[$131>>2]|0;
        $get_buffer = $132;
        $133 = (($br_state) + 12|0);
        $134 = HEAP32[$133>>2]|0;
        $bits_left = $134;
       }
       $135 = $get_buffer;
       $136 = $bits_left;
       $137 = (($136) - 1)|0;
       $bits_left = $137;
       $138 = $135 >> $137;
       $139 = $138 & 1;
       $140 = ($139|0)!=(0);
       if ($140) {
        $141 = $p1;
        $s = $141;
       } else {
        $142 = $m1;
        $s = $142;
       }
      } else {
       $143 = $r;
       $144 = ($143|0)!=(15);
       if ($144) {
        label = 35;
        break;
       }
      }
      while(1) {
       $173 = $block;
       $174 = $k;
       $175 = (50504 + ($174<<2)|0);
       $176 = HEAPU8[$175>>0]|(HEAPU8[$175+1>>0]<<8)|(HEAPU8[$175+2>>0]<<16)|(HEAPU8[$175+3>>0]<<24);
       $177 = (($173) + ($176<<1)|0);
       $thiscoef = $177;
       $178 = $thiscoef;
       $179 = HEAP16[$178>>1]|0;
       $180 = $179 << 16 >> 16;
       $181 = ($180|0)!=(0);
       if ($181) {
        $182 = $bits_left;
        $183 = ($182|0)<(1);
        if ($183) {
         $184 = $get_buffer;
         $185 = $bits_left;
         $186 = (_jpeg_fill_bit_buffer($br_state,$184,$185,1)|0);
         $187 = ($186|0)!=(0);
         if (!($187)) {
          label = 47;
          break L15;
         }
         $188 = (($br_state) + 8|0);
         $189 = HEAP32[$188>>2]|0;
         $get_buffer = $189;
         $190 = (($br_state) + 12|0);
         $191 = HEAP32[$190>>2]|0;
         $bits_left = $191;
        }
        $192 = $get_buffer;
        $193 = $bits_left;
        $194 = (($193) - 1)|0;
        $bits_left = $194;
        $195 = $192 >> $194;
        $196 = $195 & 1;
        $197 = ($196|0)!=(0);
        if ($197) {
         $198 = $thiscoef;
         $199 = HEAP16[$198>>1]|0;
         $200 = $199 << 16 >> 16;
         $201 = $p1;
         $202 = $200 & $201;
         $203 = ($202|0)==(0);
         if ($203) {
          $204 = $thiscoef;
          $205 = HEAP16[$204>>1]|0;
          $206 = $205 << 16 >> 16;
          $207 = ($206|0)>=(0);
          if ($207) {
           $208 = $p1;
           $209 = $thiscoef;
           $210 = HEAP16[$209>>1]|0;
           $211 = $210 << 16 >> 16;
           $212 = (($211) + ($208))|0;
           $213 = $212&65535;
           HEAP16[$209>>1] = $213;
          } else {
           $214 = $m1;
           $215 = $thiscoef;
           $216 = HEAP16[$215>>1]|0;
           $217 = $216 << 16 >> 16;
           $218 = (($217) + ($214))|0;
           $219 = $218&65535;
           HEAP16[$215>>1] = $219;
          }
         }
        }
       } else {
        $220 = $r;
        $221 = (($220) + -1)|0;
        $r = $221;
        $222 = ($221|0)<(0);
        if ($222) {
         label = 58;
         break;
        }
       }
       $223 = $k;
       $224 = (($223) + 1)|0;
       $k = $224;
       $225 = $k;
       $226 = $Se;
       $227 = ($225|0)<=($226|0);
       if (!($227)) {
        break;
       }
      }
      if ((label|0) == 58) {
       label = 0;
      }
      $228 = $s;
      $229 = ($228|0)!=(0);
      if ($229) {
       $230 = $k;
       $231 = (50504 + ($230<<2)|0);
       $232 = HEAPU8[$231>>0]|(HEAPU8[$231+1>>0]<<8)|(HEAPU8[$231+2>>0]<<16)|(HEAPU8[$231+3>>0]<<24);
       $pos = $232;
       $233 = $s;
       $234 = $233&65535;
       $235 = $pos;
       $236 = $block;
       $237 = (($236) + ($235<<1)|0);
       HEAP16[$237>>1] = $234;
       $238 = $pos;
       $239 = $num_newnz;
       $240 = (($239) + 1)|0;
       $num_newnz = $240;
       $241 = (($newnz_pos) + ($239<<2)|0);
       HEAP32[$241>>2] = $238;
      }
      $242 = $k;
      $243 = (($242) + 1)|0;
      $k = $243;
     }
     if ((label|0) == 13) {
      break;
     }
     else if ((label|0) == 21) {
      break;
     }
     else if ((label|0) == 28) {
      break;
     }
     else if ((label|0) == 35) {
      $145 = $r;
      $146 = 1 << $145;
      $EOBRUN = $146;
      $147 = $r;
      $148 = ($147|0)!=(0);
      if ($148) {
       $149 = $bits_left;
       $150 = $r;
       $151 = ($149|0)<($150|0);
       do {
        if ($151) {
         $152 = $get_buffer;
         $153 = $bits_left;
         $154 = $r;
         $155 = (_jpeg_fill_bit_buffer($br_state,$152,$153,$154)|0);
         $156 = ($155|0)!=(0);
         if ($156) {
          $157 = (($br_state) + 8|0);
          $158 = HEAP32[$157>>2]|0;
          $get_buffer = $158;
          $159 = (($br_state) + 12|0);
          $160 = HEAP32[$159>>2]|0;
          $bits_left = $160;
          break;
         } else {
          break L13;
         }
        }
       } while(0);
       $161 = $get_buffer;
       $162 = $r;
       $163 = $bits_left;
       $164 = (($163) - ($162))|0;
       $bits_left = $164;
       $165 = $161 >> $164;
       $166 = $r;
       $167 = 1 << $166;
       $168 = (($167) - 1)|0;
       $169 = $165 & $168;
       $r = $169;
       $170 = $r;
       $171 = $EOBRUN;
       $172 = (($171) + ($170))|0;
       $EOBRUN = $172;
      }
     }
     else if ((label|0) == 47) {
      break;
     }
     label = 67;
    } else {
     label = 67;
    }
   } while(0);
   L94: do {
    if ((label|0) == 67) {
     $244 = $EOBRUN;
     $245 = ($244>>>0)>(0);
     do {
      if ($245) {
       while(1) {
        $246 = $k;
        $247 = $Se;
        $248 = ($246|0)<=($247|0);
        if (!($248)) {
         label = 85;
         break;
        }
        $249 = $block;
        $250 = $k;
        $251 = (50504 + ($250<<2)|0);
        $252 = HEAPU8[$251>>0]|(HEAPU8[$251+1>>0]<<8)|(HEAPU8[$251+2>>0]<<16)|(HEAPU8[$251+3>>0]<<24);
        $253 = (($249) + ($252<<1)|0);
        $thiscoef = $253;
        $254 = $thiscoef;
        $255 = HEAP16[$254>>1]|0;
        $256 = $255 << 16 >> 16;
        $257 = ($256|0)!=(0);
        if ($257) {
         $258 = $bits_left;
         $259 = ($258|0)<(1);
         if ($259) {
          $260 = $get_buffer;
          $261 = $bits_left;
          $262 = (_jpeg_fill_bit_buffer($br_state,$260,$261,1)|0);
          $263 = ($262|0)!=(0);
          if (!($263)) {
           label = 73;
           break;
          }
          $264 = (($br_state) + 8|0);
          $265 = HEAP32[$264>>2]|0;
          $get_buffer = $265;
          $266 = (($br_state) + 12|0);
          $267 = HEAP32[$266>>2]|0;
          $bits_left = $267;
         }
         $268 = $get_buffer;
         $269 = $bits_left;
         $270 = (($269) - 1)|0;
         $bits_left = $270;
         $271 = $268 >> $270;
         $272 = $271 & 1;
         $273 = ($272|0)!=(0);
         if ($273) {
          $274 = $thiscoef;
          $275 = HEAP16[$274>>1]|0;
          $276 = $275 << 16 >> 16;
          $277 = $p1;
          $278 = $276 & $277;
          $279 = ($278|0)==(0);
          if ($279) {
           $280 = $thiscoef;
           $281 = HEAP16[$280>>1]|0;
           $282 = $281 << 16 >> 16;
           $283 = ($282|0)>=(0);
           if ($283) {
            $284 = $p1;
            $285 = $thiscoef;
            $286 = HEAP16[$285>>1]|0;
            $287 = $286 << 16 >> 16;
            $288 = (($287) + ($284))|0;
            $289 = $288&65535;
            HEAP16[$285>>1] = $289;
           } else {
            $290 = $m1;
            $291 = $thiscoef;
            $292 = HEAP16[$291>>1]|0;
            $293 = $292 << 16 >> 16;
            $294 = (($293) + ($290))|0;
            $295 = $294&65535;
            HEAP16[$291>>1] = $295;
           }
          }
         }
        }
        $296 = $k;
        $297 = (($296) + 1)|0;
        $k = $297;
       }
       if ((label|0) == 73) {
        break L94;
       }
       else if ((label|0) == 85) {
        $298 = $EOBRUN;
        $299 = (($298) + -1)|0;
        $EOBRUN = $299;
        break;
       }
      }
     } while(0);
     $300 = HEAP32[$br_state>>2]|0;
     $301 = $1;
     $302 = (($301) + 24|0);
     $303 = HEAP32[$302>>2]|0;
     HEAP32[$303>>2] = $300;
     $304 = (($br_state) + 4|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = $1;
     $307 = (($306) + 24|0);
     $308 = HEAP32[$307>>2]|0;
     $309 = (($308) + 4|0);
     HEAP32[$309>>2] = $305;
     $310 = $get_buffer;
     $311 = $entropy;
     $312 = (($311) + 12|0);
     HEAP32[$312>>2] = $310;
     $313 = $bits_left;
     $314 = $entropy;
     $315 = (($314) + 12|0);
     $316 = (($315) + 4|0);
     HEAP32[$316>>2] = $313;
     $317 = $EOBRUN;
     $318 = $entropy;
     $319 = (($318) + 20|0);
     HEAP32[$319>>2] = $317;
     break L11;
    }
   } while(0);
   while(1) {
    $324 = $num_newnz;
    $325 = ($324|0)>(0);
    if (!($325)) {
     break;
    }
    $326 = $num_newnz;
    $327 = (($326) + -1)|0;
    $num_newnz = $327;
    $328 = (($newnz_pos) + ($327<<2)|0);
    $329 = HEAP32[$328>>2]|0;
    $330 = $block;
    $331 = (($330) + ($329<<1)|0);
    HEAP16[$331>>1] = 0;
   }
   $0 = 0;
   $332 = $0;
   STACKTOP = sp;return ($332|0);
  }
 } while(0);
 $320 = $entropy;
 $321 = (($320) + 40|0);
 $322 = HEAP32[$321>>2]|0;
 $323 = (($322) + -1)|0;
 HEAP32[$321>>2] = $323;
 $0 = 1;
 $332 = $0;
 STACKTOP = sp;return ($332|0);
}
function _process_restart($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $entropy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 444|0);
 $4 = HEAP32[$3>>2]|0;
 $entropy = $4;
 $5 = $entropy;
 $6 = (($5) + 12|0);
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8|0) / 8)&-1;
 $10 = $1;
 $11 = (($10) + 440|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($9))|0;
 HEAP32[$13>>2] = $15;
 $16 = $entropy;
 $17 = (($16) + 12|0);
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = 0;
 $19 = $1;
 $20 = (($19) + 440|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = (FUNCTION_TABLE_ii[$23 & 127]($24)|0);
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $0 = 0;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 $ci = 0;
 while(1) {
  $27 = $ci;
  $28 = $1;
  $29 = (($28) + 328|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = $ci;
  $33 = $entropy;
  $34 = (($33) + 20|0);
  $35 = (($34) + 4|0);
  $36 = (($35) + ($32<<2)|0);
  HEAP32[$36>>2] = 0;
  $37 = $ci;
  $38 = (($37) + 1)|0;
  $ci = $38;
 }
 $39 = $entropy;
 $40 = (($39) + 20|0);
 HEAP32[$40>>2] = 0;
 $41 = $1;
 $42 = (($41) + 276|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $entropy;
 $45 = (($44) + 40|0);
 HEAP32[$45>>2] = $43;
 $46 = $1;
 $47 = (($46) + 416|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = ($48|0)==(0);
 if ($49) {
  $50 = $entropy;
  $51 = (($50) + 8|0);
  HEAP32[$51>>2] = 0;
 }
 $0 = 1;
 $52 = $0;
 STACKTOP = sp;return ($52|0);
}
function _jinit_d_post_controller($cinfo,$need_full_buffer) {
 $cinfo = $cinfo|0;
 $need_full_buffer = $need_full_buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $need_full_buffer;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 255]($6,1,28)|0);
 $post = $7;
 $8 = $post;
 $9 = $0;
 $10 = (($9) + 432|0);
 HEAP32[$10>>2] = $8;
 $11 = $post;
 HEAP32[$11>>2] = 56;
 $12 = $post;
 $13 = (($12) + 8|0);
 HEAP32[$13>>2] = 0;
 $14 = $post;
 $15 = (($14) + 12|0);
 HEAP32[$15>>2] = 0;
 $16 = $0;
 $17 = (($16) + 84|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 $20 = $0;
 $21 = (($20) + 312|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $post;
 $24 = (($23) + 16|0);
 HEAP32[$24>>2] = $22;
 $25 = $1;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $0;
  $28 = (($27) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $0;
  $33 = $0;
  $34 = (($33) + 112|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = $0;
  $37 = (($36) + 120|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = Math_imul($35, $38)|0;
  $40 = $0;
  $41 = (($40) + 116|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $post;
  $44 = (($43) + 16|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_jround_up($42,$45)|0);
  $47 = $post;
  $48 = (($47) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (FUNCTION_TABLE_iiiiiii[$31 & 127]($32,1,0,$39,$46,$49)|0);
  $51 = $post;
  $52 = (($51) + 8|0);
  HEAP32[$52>>2] = $50;
 } else {
  $53 = $0;
  $54 = (($53) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $0;
  $59 = $0;
  $60 = (($59) + 112|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = $0;
  $63 = (($62) + 120|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = Math_imul($61, $64)|0;
  $66 = $post;
  $67 = (($66) + 16|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (FUNCTION_TABLE_iiiii[$57 & 255]($58,1,$65,$68)|0);
  $70 = $post;
  $71 = (($70) + 12|0);
  HEAP32[$71>>2] = $69;
 }
 STACKTOP = sp;return;
}
function _start_pass_dpost($cinfo,$pass_mode) {
 $cinfo = $cinfo|0;
 $pass_mode = $pass_mode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $pass_mode;
 $2 = $0;
 $3 = (($2) + 432|0);
 $4 = HEAP32[$3>>2]|0;
 $post = $4;
 $5 = $1;
 if ((($5|0) == 2)) {
  $51 = $post;
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($53|0)==(0|0);
  if ($54) {
   $55 = $0;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 20|0);
   HEAP32[$57>>2] = 4;
   $58 = $0;
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$59>>2]|0;
   $61 = $0;
   FUNCTION_TABLE_vi[$60 & 255]($61);
  }
  $62 = $post;
  $63 = (($62) + 4|0);
  HEAP32[$63>>2] = 59;
 } else if ((($5|0) == 3)) {
  $38 = $post;
  $39 = (($38) + 8|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)==(0|0);
  if ($41) {
   $42 = $0;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($43) + 20|0);
   HEAP32[$44>>2] = 4;
   $45 = $0;
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$46>>2]|0;
   $48 = $0;
   FUNCTION_TABLE_vi[$47 & 255]($48);
  }
  $49 = $post;
  $50 = (($49) + 4|0);
  HEAP32[$50>>2] = 58;
 } else if ((($5|0) == 0)) {
  $6 = $0;
  $7 = (($6) + 84|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0);
  if ($9) {
   $10 = $post;
   $11 = (($10) + 4|0);
   HEAP32[$11>>2] = 57;
   $12 = $post;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(0|0);
   if ($15) {
    $16 = $0;
    $17 = (($16) + 4|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + 28|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $0;
    $22 = $post;
    $23 = (($22) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $post;
    $26 = (($25) + 16|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (FUNCTION_TABLE_iiiiii[$20 & 127]($21,$24,0,$27,1)|0);
    $29 = $post;
    $30 = (($29) + 12|0);
    HEAP32[$30>>2] = $28;
   }
  } else {
   $31 = $0;
   $32 = (($31) + 452|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $post;
   $37 = (($36) + 4|0);
   HEAP32[$37>>2] = $35;
  }
 } else {
  $64 = $0;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 20|0);
  HEAP32[$66>>2] = 4;
  $67 = $0;
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = $0;
  FUNCTION_TABLE_vi[$69 & 255]($70);
 }
 $71 = $post;
 $72 = (($71) + 24|0);
 HEAP32[$72>>2] = 0;
 $73 = $post;
 $74 = (($73) + 20|0);
 HEAP32[$74>>2] = 0;
 STACKTOP = sp;return;
}
function _post_process_1pass($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $max_rows = 0, $num_rows = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $num_rows = sp + 4|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 432|0);
 $9 = HEAP32[$8>>2]|0;
 $post = $9;
 $10 = $6;
 $11 = $5;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) - ($12))|0;
 $max_rows = $13;
 $14 = $max_rows;
 $15 = $post;
 $16 = (($15) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14>>>0)>($17>>>0);
 if ($18) {
  $19 = $post;
  $20 = (($19) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  $max_rows = $21;
 }
 HEAP32[$num_rows>>2] = 0;
 $22 = $0;
 $23 = (($22) + 452|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $0;
 $28 = $1;
 $29 = $2;
 $30 = $3;
 $31 = $post;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $max_rows;
 FUNCTION_TABLE_viiiiiii[$26 & 63]($27,$28,$29,$30,$33,$num_rows,$34);
 $35 = $0;
 $36 = (($35) + 460|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = $0;
 $41 = $post;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $4;
 $45 = $5;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + ($46<<2)|0);
 $48 = HEAP32[$num_rows>>2]|0;
 FUNCTION_TABLE_viiii[$39 & 255]($40,$43,$47,$48);
 $49 = HEAP32[$num_rows>>2]|0;
 $50 = $5;
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($49))|0;
 HEAP32[$50>>2] = $52;
 STACKTOP = sp;return;
}
function _post_process_prepass($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $num_rows = 0, $old_next_row = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 432|0);
 $9 = HEAP32[$8>>2]|0;
 $post = $9;
 $10 = $post;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  $20 = $post;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $post;
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $post;
  $27 = (($26) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (FUNCTION_TABLE_iiiiii[$18 & 127]($19,$22,$25,$28,1)|0);
  $30 = $post;
  $31 = (($30) + 12|0);
  HEAP32[$31>>2] = $29;
 }
 $32 = $post;
 $33 = (($32) + 24|0);
 $34 = HEAP32[$33>>2]|0;
 $old_next_row = $34;
 $35 = $0;
 $36 = (($35) + 452|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = $0;
 $41 = $1;
 $42 = $2;
 $43 = $3;
 $44 = $post;
 $45 = (($44) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $post;
 $48 = (($47) + 24|0);
 $49 = $post;
 $50 = (($49) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 FUNCTION_TABLE_viiiiiii[$39 & 63]($40,$41,$42,$43,$46,$48,$51);
 $52 = $post;
 $53 = (($52) + 24|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $old_next_row;
 $56 = ($54>>>0)>($55>>>0);
 if ($56) {
  $57 = $post;
  $58 = (($57) + 24|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $old_next_row;
  $61 = (($59) - ($60))|0;
  $num_rows = $61;
  $62 = $0;
  $63 = (($62) + 460|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($64) + 4|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $0;
  $68 = $post;
  $69 = (($68) + 12|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $old_next_row;
  $72 = (($70) + ($71<<2)|0);
  $73 = $num_rows;
  FUNCTION_TABLE_viiii[$66 & 255]($67,$72,0,$73);
  $74 = $num_rows;
  $75 = $5;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76) + ($74))|0;
  HEAP32[$75>>2] = $77;
 }
 $78 = $post;
 $79 = (($78) + 24|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $post;
 $82 = (($81) + 16|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = ($80>>>0)>=($83>>>0);
 if (!($84)) {
  STACKTOP = sp;return;
 }
 $85 = $post;
 $86 = (($85) + 16|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $post;
 $89 = (($88) + 20|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = (($90) + ($87))|0;
 HEAP32[$89>>2] = $91;
 $92 = $post;
 $93 = (($92) + 24|0);
 HEAP32[$93>>2] = 0;
 STACKTOP = sp;return;
}
function _post_process_2pass($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $max_rows = 0, $num_rows = 0, $post = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 432|0);
 $9 = HEAP32[$8>>2]|0;
 $post = $9;
 $10 = $post;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  $20 = $post;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $post;
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $post;
  $27 = (($26) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (FUNCTION_TABLE_iiiiii[$18 & 127]($19,$22,$25,$28,0)|0);
  $30 = $post;
  $31 = (($30) + 12|0);
  HEAP32[$31>>2] = $29;
 }
 $32 = $post;
 $33 = (($32) + 16|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $post;
 $36 = (($35) + 24|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($34) - ($37))|0;
 $num_rows = $38;
 $39 = $6;
 $40 = $5;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($39) - ($41))|0;
 $max_rows = $42;
 $43 = $num_rows;
 $44 = $max_rows;
 $45 = ($43>>>0)>($44>>>0);
 if ($45) {
  $46 = $max_rows;
  $num_rows = $46;
 }
 $47 = $0;
 $48 = (($47) + 116|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $post;
 $51 = (($50) + 20|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($49) - ($52))|0;
 $max_rows = $53;
 $54 = $num_rows;
 $55 = $max_rows;
 $56 = ($54>>>0)>($55>>>0);
 if ($56) {
  $57 = $max_rows;
  $num_rows = $57;
 }
 $58 = $0;
 $59 = (($58) + 460|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + 4|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $0;
 $64 = $post;
 $65 = (($64) + 12|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $post;
 $68 = (($67) + 24|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (($66) + ($69<<2)|0);
 $71 = $4;
 $72 = $5;
 $73 = HEAP32[$72>>2]|0;
 $74 = (($71) + ($73<<2)|0);
 $75 = $num_rows;
 FUNCTION_TABLE_viiii[$62 & 255]($63,$70,$74,$75);
 $76 = $num_rows;
 $77 = $5;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($78) + ($76))|0;
 HEAP32[$77>>2] = $79;
 $80 = $num_rows;
 $81 = $post;
 $82 = (($81) + 24|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (($83) + ($80))|0;
 HEAP32[$82>>2] = $84;
 $85 = $post;
 $86 = (($85) + 24|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $post;
 $89 = (($88) + 16|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = ($87>>>0)>=($90>>>0);
 if (!($91)) {
  STACKTOP = sp;return;
 }
 $92 = $post;
 $93 = (($92) + 16|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $post;
 $96 = (($95) + 20|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = (($97) + ($94))|0;
 HEAP32[$96>>2] = $98;
 $99 = $post;
 $100 = (($99) + 24|0);
 HEAP32[$100>>2] = 0;
 STACKTOP = sp;return;
}
function _jinit_upsampler($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $ci = 0, $compptr = 0, $do_fancy = 0, $h_in_group = 0, $h_out_group = 0, $need_buffer = 0, $upsample = 0, $v_in_group = 0, $v_out_group = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,160)|0);
 $upsample = $6;
 $7 = $upsample;
 $8 = $0;
 $9 = (($8) + 452|0);
 HEAP32[$9>>2] = $7;
 $10 = $upsample;
 HEAP32[$10>>2] = 60;
 $11 = $upsample;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 61;
 $13 = $upsample;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 $15 = $0;
 $16 = (($15) + 300|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = $0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 25;
  $22 = $0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = $0;
  FUNCTION_TABLE_vi[$24 & 255]($25);
 }
 $26 = $0;
 $27 = (($26) + 76|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = $0;
  $31 = (($30) + 316|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)>(1);
  $35 = $33;
 } else {
  $35 = 0;
 }
 $34 = $35&1;
 $do_fancy = $34;
 $ci = 0;
 $36 = $0;
 $37 = (($36) + 216|0);
 $38 = HEAP32[$37>>2]|0;
 $compptr = $38;
 while(1) {
  $39 = $ci;
  $40 = $0;
  $41 = (($40) + 36|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($39|0)<($42|0);
  if (!($43)) {
   break;
  }
  $44 = $compptr;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $compptr;
  $48 = (($47) + 36|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = Math_imul($46, $49)|0;
  $51 = $0;
  $52 = (($51) + 316|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($50|0) / ($53|0))&-1;
  $h_in_group = $54;
  $55 = $compptr;
  $56 = (($55) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $compptr;
  $59 = (($58) + 36|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = Math_imul($57, $60)|0;
  $62 = $0;
  $63 = (($62) + 316|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($61|0) / ($64|0))&-1;
  $v_in_group = $65;
  $66 = $0;
  $67 = (($66) + 308|0);
  $68 = HEAP32[$67>>2]|0;
  $h_out_group = $68;
  $69 = $0;
  $70 = (($69) + 312|0);
  $71 = HEAP32[$70>>2]|0;
  $v_out_group = $71;
  $72 = $v_in_group;
  $73 = $ci;
  $74 = $upsample;
  $75 = (($74) + 100|0);
  $76 = (($75) + ($73<<2)|0);
  HEAP32[$76>>2] = $72;
  $need_buffer = 1;
  $77 = $compptr;
  $78 = (($77) + 48|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0);
  if ($80) {
   $85 = $h_in_group;
   $86 = $h_out_group;
   $87 = ($85|0)==($86|0);
   if ($87) {
    $88 = $v_in_group;
    $89 = $v_out_group;
    $90 = ($88|0)==($89|0);
    if ($90) {
     $91 = $ci;
     $92 = $upsample;
     $93 = (($92) + 52|0);
     $94 = (($93) + ($91<<2)|0);
     HEAP32[$94>>2] = 63;
     $need_buffer = 0;
    } else {
     label = 12;
    }
   } else {
    label = 12;
   }
   if ((label|0) == 12) {
    label = 0;
    $95 = $h_in_group;
    $96 = $95<<1;
    $97 = $h_out_group;
    $98 = ($96|0)==($97|0);
    if ($98) {
     $99 = $v_in_group;
     $100 = $v_out_group;
     $101 = ($99|0)==($100|0);
     if ($101) {
      $102 = $do_fancy;
      $103 = ($102|0)!=(0);
      if ($103) {
       $104 = $compptr;
       $105 = (($104) + 40|0);
       $106 = HEAP32[$105>>2]|0;
       $107 = ($106>>>0)>(2);
       if ($107) {
        $108 = $ci;
        $109 = $upsample;
        $110 = (($109) + 52|0);
        $111 = (($110) + ($108<<2)|0);
        HEAP32[$111>>2] = 64;
       } else {
        label = 17;
       }
      } else {
       label = 17;
      }
      if ((label|0) == 17) {
       label = 0;
       $112 = $ci;
       $113 = $upsample;
       $114 = (($113) + 52|0);
       $115 = (($114) + ($112<<2)|0);
       HEAP32[$115>>2] = 65;
      }
     } else {
      label = 19;
     }
    } else {
     label = 19;
    }
    if ((label|0) == 19) {
     label = 0;
     $116 = $h_in_group;
     $117 = $116<<1;
     $118 = $h_out_group;
     $119 = ($117|0)==($118|0);
     if ($119) {
      $120 = $v_in_group;
      $121 = $120<<1;
      $122 = $v_out_group;
      $123 = ($121|0)==($122|0);
      if ($123) {
       $124 = $do_fancy;
       $125 = ($124|0)!=(0);
       if ($125) {
        $126 = $compptr;
        $127 = (($126) + 40|0);
        $128 = HEAP32[$127>>2]|0;
        $129 = ($128>>>0)>(2);
        if ($129) {
         $130 = $ci;
         $131 = $upsample;
         $132 = (($131) + 52|0);
         $133 = (($132) + ($130<<2)|0);
         HEAP32[$133>>2] = 66;
         $134 = $upsample;
         $135 = (($134) + 8|0);
         HEAP32[$135>>2] = 1;
        } else {
         label = 24;
        }
       } else {
        label = 24;
       }
       if ((label|0) == 24) {
        label = 0;
        $136 = $ci;
        $137 = $upsample;
        $138 = (($137) + 52|0);
        $139 = (($138) + ($136<<2)|0);
        HEAP32[$139>>2] = 67;
       }
      } else {
       label = 26;
      }
     } else {
      label = 26;
     }
     if ((label|0) == 26) {
      label = 0;
      $140 = $h_out_group;
      $141 = $h_in_group;
      $142 = (($140|0) % ($141|0))&-1;
      $143 = ($142|0)==(0);
      if ($143) {
       $144 = $v_out_group;
       $145 = $v_in_group;
       $146 = (($144|0) % ($145|0))&-1;
       $147 = ($146|0)==(0);
       if ($147) {
        $148 = $ci;
        $149 = $upsample;
        $150 = (($149) + 52|0);
        $151 = (($150) + ($148<<2)|0);
        HEAP32[$151>>2] = 68;
        $152 = $h_out_group;
        $153 = $h_in_group;
        $154 = (($152|0) / ($153|0))&-1;
        $155 = $154&255;
        $156 = $ci;
        $157 = $upsample;
        $158 = (($157) + 140|0);
        $159 = (($158) + ($156)|0);
        HEAP8[$159>>0] = $155;
        $160 = $v_out_group;
        $161 = $v_in_group;
        $162 = (($160|0) / ($161|0))&-1;
        $163 = $162&255;
        $164 = $ci;
        $165 = $upsample;
        $166 = (($165) + 150|0);
        $167 = (($166) + ($164)|0);
        HEAP8[$167>>0] = $163;
       } else {
        label = 29;
       }
      } else {
       label = 29;
      }
      if ((label|0) == 29) {
       label = 0;
       $168 = $0;
       $169 = HEAP32[$168>>2]|0;
       $170 = (($169) + 20|0);
       HEAP32[$170>>2] = 38;
       $171 = $0;
       $172 = HEAP32[$171>>2]|0;
       $173 = HEAP32[$172>>2]|0;
       $174 = $0;
       FUNCTION_TABLE_vi[$173 & 255]($174);
      }
     }
    }
   }
  } else {
   $81 = $ci;
   $82 = $upsample;
   $83 = (($82) + 52|0);
   $84 = (($83) + ($81<<2)|0);
   HEAP32[$84>>2] = 62;
   $need_buffer = 0;
  }
  $175 = $need_buffer;
  $176 = ($175|0)!=(0);
  if ($176) {
   $177 = $0;
   $178 = (($177) + 4|0);
   $179 = HEAP32[$178>>2]|0;
   $180 = (($179) + 8|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = $0;
   $183 = $0;
   $184 = (($183) + 112|0);
   $185 = HEAP32[$184>>2]|0;
   $186 = $0;
   $187 = (($186) + 308|0);
   $188 = HEAP32[$187>>2]|0;
   $189 = (_jround_up($185,$188)|0);
   $190 = $0;
   $191 = (($190) + 312|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = (FUNCTION_TABLE_iiiii[$181 & 255]($182,1,$189,$192)|0);
   $194 = $ci;
   $195 = $upsample;
   $196 = (($195) + 12|0);
   $197 = (($196) + ($194<<2)|0);
   HEAP32[$197>>2] = $193;
  }
  $198 = $ci;
  $199 = (($198) + 1)|0;
  $ci = $199;
  $200 = $compptr;
  $201 = (($200) + 84|0);
  $compptr = $201;
 }
 STACKTOP = sp;return;
}
function _start_pass_upsample($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $upsample = $3;
 $4 = $0;
 $5 = (($4) + 312|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $upsample;
 $8 = (($7) + 92|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = (($9) + 116|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $upsample;
 $13 = (($12) + 96|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _sep_upsample($cinfo,$input_buf,$in_row_group_ctr,$in_row_groups_avail,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $in_row_group_ctr = $in_row_group_ctr|0;
 $in_row_groups_avail = $in_row_groups_avail|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $compptr = 0;
 var $num_rows = 0, $upsample = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $in_row_group_ctr;
 $3 = $in_row_groups_avail;
 $4 = $output_buf;
 $5 = $out_row_ctr;
 $6 = $out_rows_avail;
 $7 = $0;
 $8 = (($7) + 452|0);
 $9 = HEAP32[$8>>2]|0;
 $upsample = $9;
 $10 = $upsample;
 $11 = (($10) + 92|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 312|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($12|0)>=($15|0);
 if ($16) {
  $ci = 0;
  $17 = $0;
  $18 = (($17) + 216|0);
  $19 = HEAP32[$18>>2]|0;
  $compptr = $19;
  while(1) {
   $20 = $ci;
   $21 = $0;
   $22 = (($21) + 36|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($20|0)<($23|0);
   if (!($24)) {
    break;
   }
   $25 = $ci;
   $26 = $upsample;
   $27 = (($26) + 52|0);
   $28 = (($27) + ($25<<2)|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $0;
   $31 = $compptr;
   $32 = $ci;
   $33 = $1;
   $34 = (($33) + ($32<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $2;
   $37 = HEAP32[$36>>2]|0;
   $38 = $ci;
   $39 = $upsample;
   $40 = (($39) + 100|0);
   $41 = (($40) + ($38<<2)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = Math_imul($37, $42)|0;
   $44 = (($35) + ($43<<2)|0);
   $45 = $upsample;
   $46 = (($45) + 12|0);
   $47 = $ci;
   $48 = (($46) + ($47<<2)|0);
   FUNCTION_TABLE_viiii[$29 & 255]($30,$31,$44,$48);
   $49 = $ci;
   $50 = (($49) + 1)|0;
   $ci = $50;
   $51 = $compptr;
   $52 = (($51) + 84|0);
   $compptr = $52;
  }
  $53 = $upsample;
  $54 = (($53) + 92|0);
  HEAP32[$54>>2] = 0;
 }
 $55 = $0;
 $56 = (($55) + 312|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $upsample;
 $59 = (($58) + 92|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($57) - ($60))|0;
 $num_rows = $61;
 $62 = $num_rows;
 $63 = $upsample;
 $64 = (($63) + 96|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = ($62>>>0)>($65>>>0);
 if ($66) {
  $67 = $upsample;
  $68 = (($67) + 96|0);
  $69 = HEAP32[$68>>2]|0;
  $num_rows = $69;
 }
 $70 = $5;
 $71 = HEAP32[$70>>2]|0;
 $72 = $6;
 $73 = (($72) - ($71))|0;
 $6 = $73;
 $74 = $num_rows;
 $75 = $6;
 $76 = ($74>>>0)>($75>>>0);
 if ($76) {
  $77 = $6;
  $num_rows = $77;
 }
 $78 = $0;
 $79 = (($78) + 456|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = (($80) + 4|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = $0;
 $84 = $upsample;
 $85 = (($84) + 12|0);
 $86 = $upsample;
 $87 = (($86) + 92|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $4;
 $90 = $5;
 $91 = HEAP32[$90>>2]|0;
 $92 = (($89) + ($91<<2)|0);
 $93 = $num_rows;
 FUNCTION_TABLE_viiiii[$82 & 127]($83,$85,$88,$92,$93);
 $94 = $num_rows;
 $95 = $5;
 $96 = HEAP32[$95>>2]|0;
 $97 = (($96) + ($94))|0;
 HEAP32[$95>>2] = $97;
 $98 = $num_rows;
 $99 = $upsample;
 $100 = (($99) + 96|0);
 $101 = HEAP32[$100>>2]|0;
 $102 = (($101) - ($98))|0;
 HEAP32[$100>>2] = $102;
 $103 = $num_rows;
 $104 = $upsample;
 $105 = (($104) + 92|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = (($106) + ($103))|0;
 HEAP32[$105>>2] = $107;
 $108 = $upsample;
 $109 = (($108) + 92|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = $0;
 $112 = (($111) + 312|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = ($110|0)>=($113|0);
 if (!($114)) {
  STACKTOP = sp;return;
 }
 $115 = $2;
 $116 = HEAP32[$115>>2]|0;
 $117 = (($116) + 1)|0;
 HEAP32[$115>>2] = $117;
 STACKTOP = sp;return;
}
function _noop_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 STACKTOP = sp;return;
}
function _fullsize_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $2;
 $5 = $3;
 HEAP32[$5>>2] = $4;
 STACKTOP = sp;return;
}
function _h2v1_fancy_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $colctr = 0, $inptr = 0, $inrow = 0, $invalue = 0, $outptr = 0, $output_data = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $inrow = 0;
 while(1) {
  $6 = $inrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $inrow;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $inptr = $14;
  $15 = $inrow;
  $16 = $output_data;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $outptr = $18;
  $19 = $inptr;
  $20 = (($19) + 1|0);
  $inptr = $20;
  $21 = HEAP8[$19>>0]|0;
  $22 = $21&255;
  $invalue = $22;
  $23 = $invalue;
  $24 = $23&255;
  $25 = $outptr;
  $26 = (($25) + 1|0);
  $outptr = $26;
  HEAP8[$25>>0] = $24;
  $27 = $invalue;
  $28 = ($27*3)|0;
  $29 = $inptr;
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = (($28) + ($31))|0;
  $33 = (($32) + 2)|0;
  $34 = $33 >> 2;
  $35 = $34&255;
  $36 = $outptr;
  $37 = (($36) + 1|0);
  $outptr = $37;
  HEAP8[$36>>0] = $35;
  $38 = $1;
  $39 = (($38) + 40|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) - 2)|0;
  $colctr = $41;
  while(1) {
   $42 = $colctr;
   $43 = ($42>>>0)>(0);
   if (!($43)) {
    break;
   }
   $44 = $inptr;
   $45 = (($44) + 1|0);
   $inptr = $45;
   $46 = HEAP8[$44>>0]|0;
   $47 = $46&255;
   $48 = ($47*3)|0;
   $invalue = $48;
   $49 = $invalue;
   $50 = $inptr;
   $51 = (($50) + -2|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = (($49) + ($53))|0;
   $55 = (($54) + 1)|0;
   $56 = $55 >> 2;
   $57 = $56&255;
   $58 = $outptr;
   $59 = (($58) + 1|0);
   $outptr = $59;
   HEAP8[$58>>0] = $57;
   $60 = $invalue;
   $61 = $inptr;
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = (($60) + ($63))|0;
   $65 = (($64) + 2)|0;
   $66 = $65 >> 2;
   $67 = $66&255;
   $68 = $outptr;
   $69 = (($68) + 1|0);
   $outptr = $69;
   HEAP8[$68>>0] = $67;
   $70 = $colctr;
   $71 = (($70) + -1)|0;
   $colctr = $71;
  }
  $72 = $inptr;
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $invalue = $74;
  $75 = $invalue;
  $76 = ($75*3)|0;
  $77 = $inptr;
  $78 = (($77) + -1|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = (($76) + ($80))|0;
  $82 = (($81) + 1)|0;
  $83 = $82 >> 2;
  $84 = $83&255;
  $85 = $outptr;
  $86 = (($85) + 1|0);
  $outptr = $86;
  HEAP8[$85>>0] = $84;
  $87 = $invalue;
  $88 = $87&255;
  $89 = $outptr;
  $90 = (($89) + 1|0);
  $outptr = $90;
  HEAP8[$89>>0] = $88;
  $91 = $inrow;
  $92 = (($91) + 1)|0;
  $inrow = $92;
 }
 STACKTOP = sp;return;
}
function _h2v1_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inptr = 0, $inrow = 0;
 var $invalue = 0, $outend = 0, $outptr = 0, $output_data = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $inrow = 0;
 while(1) {
  $6 = $inrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $inrow;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $inptr = $14;
  $15 = $inrow;
  $16 = $output_data;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $outptr = $18;
  $19 = $outptr;
  $20 = $0;
  $21 = (($20) + 112|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) + ($22)|0);
  $outend = $23;
  while(1) {
   $24 = $outptr;
   $25 = $outend;
   $26 = ($24>>>0)<($25>>>0);
   if (!($26)) {
    break;
   }
   $27 = $inptr;
   $28 = (($27) + 1|0);
   $inptr = $28;
   $29 = HEAP8[$27>>0]|0;
   $invalue = $29;
   $30 = $invalue;
   $31 = $outptr;
   $32 = (($31) + 1|0);
   $outptr = $32;
   HEAP8[$31>>0] = $30;
   $33 = $invalue;
   $34 = $outptr;
   $35 = (($34) + 1|0);
   $outptr = $35;
   HEAP8[$34>>0] = $33;
  }
  $36 = $inrow;
  $37 = (($36) + 1)|0;
  $inrow = $37;
 }
 STACKTOP = sp;return;
}
function _h2v2_fancy_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $colctr = 0, $inptr0 = 0, $inptr1 = 0, $inrow = 0, $lastcolsum = 0, $nextcolsum = 0, $outptr = 0, $output_data = 0, $outrow = 0, $thiscolsum = 0;
 var $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $outrow = 0;
 $inrow = 0;
 while(1) {
  $6 = $outrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $v = 0;
  while(1) {
   $11 = $v;
   $12 = ($11|0)<(2);
   if (!($12)) {
    break;
   }
   $13 = $inrow;
   $14 = $2;
   $15 = (($14) + ($13<<2)|0);
   $16 = HEAP32[$15>>2]|0;
   $inptr0 = $16;
   $17 = $v;
   $18 = ($17|0)==(0);
   if ($18) {
    $19 = $inrow;
    $20 = (($19) - 1)|0;
    $21 = $2;
    $22 = (($21) + ($20<<2)|0);
    $23 = HEAP32[$22>>2]|0;
    $inptr1 = $23;
   } else {
    $24 = $inrow;
    $25 = (($24) + 1)|0;
    $26 = $2;
    $27 = (($26) + ($25<<2)|0);
    $28 = HEAP32[$27>>2]|0;
    $inptr1 = $28;
   }
   $29 = $outrow;
   $30 = (($29) + 1)|0;
   $outrow = $30;
   $31 = $output_data;
   $32 = (($31) + ($29<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   $outptr = $33;
   $34 = $inptr0;
   $35 = (($34) + 1|0);
   $inptr0 = $35;
   $36 = HEAP8[$34>>0]|0;
   $37 = $36&255;
   $38 = ($37*3)|0;
   $39 = $inptr1;
   $40 = (($39) + 1|0);
   $inptr1 = $40;
   $41 = HEAP8[$39>>0]|0;
   $42 = $41&255;
   $43 = (($38) + ($42))|0;
   $thiscolsum = $43;
   $44 = $inptr0;
   $45 = (($44) + 1|0);
   $inptr0 = $45;
   $46 = HEAP8[$44>>0]|0;
   $47 = $46&255;
   $48 = ($47*3)|0;
   $49 = $inptr1;
   $50 = (($49) + 1|0);
   $inptr1 = $50;
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   $53 = (($48) + ($52))|0;
   $nextcolsum = $53;
   $54 = $thiscolsum;
   $55 = $54<<2;
   $56 = (($55) + 8)|0;
   $57 = $56 >> 4;
   $58 = $57&255;
   $59 = $outptr;
   $60 = (($59) + 1|0);
   $outptr = $60;
   HEAP8[$59>>0] = $58;
   $61 = $thiscolsum;
   $62 = ($61*3)|0;
   $63 = $nextcolsum;
   $64 = (($62) + ($63))|0;
   $65 = (($64) + 7)|0;
   $66 = $65 >> 4;
   $67 = $66&255;
   $68 = $outptr;
   $69 = (($68) + 1|0);
   $outptr = $69;
   HEAP8[$68>>0] = $67;
   $70 = $thiscolsum;
   $lastcolsum = $70;
   $71 = $nextcolsum;
   $thiscolsum = $71;
   $72 = $1;
   $73 = (($72) + 40|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) - 2)|0;
   $colctr = $75;
   while(1) {
    $76 = $colctr;
    $77 = ($76>>>0)>(0);
    if (!($77)) {
     break;
    }
    $78 = $inptr0;
    $79 = (($78) + 1|0);
    $inptr0 = $79;
    $80 = HEAP8[$78>>0]|0;
    $81 = $80&255;
    $82 = ($81*3)|0;
    $83 = $inptr1;
    $84 = (($83) + 1|0);
    $inptr1 = $84;
    $85 = HEAP8[$83>>0]|0;
    $86 = $85&255;
    $87 = (($82) + ($86))|0;
    $nextcolsum = $87;
    $88 = $thiscolsum;
    $89 = ($88*3)|0;
    $90 = $lastcolsum;
    $91 = (($89) + ($90))|0;
    $92 = (($91) + 8)|0;
    $93 = $92 >> 4;
    $94 = $93&255;
    $95 = $outptr;
    $96 = (($95) + 1|0);
    $outptr = $96;
    HEAP8[$95>>0] = $94;
    $97 = $thiscolsum;
    $98 = ($97*3)|0;
    $99 = $nextcolsum;
    $100 = (($98) + ($99))|0;
    $101 = (($100) + 7)|0;
    $102 = $101 >> 4;
    $103 = $102&255;
    $104 = $outptr;
    $105 = (($104) + 1|0);
    $outptr = $105;
    HEAP8[$104>>0] = $103;
    $106 = $thiscolsum;
    $lastcolsum = $106;
    $107 = $nextcolsum;
    $thiscolsum = $107;
    $108 = $colctr;
    $109 = (($108) + -1)|0;
    $colctr = $109;
   }
   $110 = $thiscolsum;
   $111 = ($110*3)|0;
   $112 = $lastcolsum;
   $113 = (($111) + ($112))|0;
   $114 = (($113) + 8)|0;
   $115 = $114 >> 4;
   $116 = $115&255;
   $117 = $outptr;
   $118 = (($117) + 1|0);
   $outptr = $118;
   HEAP8[$117>>0] = $116;
   $119 = $thiscolsum;
   $120 = $119<<2;
   $121 = (($120) + 7)|0;
   $122 = $121 >> 4;
   $123 = $122&255;
   $124 = $outptr;
   $125 = (($124) + 1|0);
   $outptr = $125;
   HEAP8[$124>>0] = $123;
   $126 = $v;
   $127 = (($126) + 1)|0;
   $v = $127;
  }
  $128 = $inrow;
  $129 = (($128) + 1)|0;
  $inrow = $129;
 }
 STACKTOP = sp;return;
}
function _h2v2_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inptr = 0, $inrow = 0, $invalue = 0, $outend = 0, $outptr = 0, $output_data = 0, $outrow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $output_data = $5;
 $outrow = 0;
 $inrow = 0;
 while(1) {
  $6 = $outrow;
  $7 = $0;
  $8 = (($7) + 312|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $inrow;
  $12 = $2;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $inptr = $14;
  $15 = $outrow;
  $16 = $output_data;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $outptr = $18;
  $19 = $outptr;
  $20 = $0;
  $21 = (($20) + 112|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) + ($22)|0);
  $outend = $23;
  while(1) {
   $24 = $outptr;
   $25 = $outend;
   $26 = ($24>>>0)<($25>>>0);
   if (!($26)) {
    break;
   }
   $27 = $inptr;
   $28 = (($27) + 1|0);
   $inptr = $28;
   $29 = HEAP8[$27>>0]|0;
   $invalue = $29;
   $30 = $invalue;
   $31 = $outptr;
   $32 = (($31) + 1|0);
   $outptr = $32;
   HEAP8[$31>>0] = $30;
   $33 = $invalue;
   $34 = $outptr;
   $35 = (($34) + 1|0);
   $outptr = $35;
   HEAP8[$34>>0] = $33;
  }
  $36 = $output_data;
  $37 = $outrow;
  $38 = $output_data;
  $39 = $outrow;
  $40 = (($39) + 1)|0;
  $41 = $0;
  $42 = (($41) + 112|0);
  $43 = HEAP32[$42>>2]|0;
  _jcopy_sample_rows($36,$37,$38,$40,1,$43);
  $44 = $inrow;
  $45 = (($44) + 1)|0;
  $inrow = $45;
  $46 = $outrow;
  $47 = (($46) + 2)|0;
  $outrow = $47;
 }
 STACKTOP = sp;return;
}
function _int_upsample($cinfo,$compptr,$input_data,$output_data_ptr) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $input_data = $input_data|0;
 $output_data_ptr = $output_data_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $h = 0, $h_expand = 0, $inptr = 0, $inrow = 0, $invalue = 0, $outend = 0;
 var $outptr = 0, $output_data = 0, $outrow = 0, $upsample = 0, $v_expand = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $input_data;
 $3 = $output_data_ptr;
 $4 = $0;
 $5 = (($4) + 452|0);
 $6 = HEAP32[$5>>2]|0;
 $upsample = $6;
 $7 = $3;
 $8 = HEAP32[$7>>2]|0;
 $output_data = $8;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $upsample;
 $13 = (($12) + 140|0);
 $14 = (($13) + ($11)|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $h_expand = $16;
 $17 = $1;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $upsample;
 $21 = (($20) + 150|0);
 $22 = (($21) + ($19)|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $v_expand = $24;
 $outrow = 0;
 $inrow = 0;
 while(1) {
  $25 = $outrow;
  $26 = $0;
  $27 = (($26) + 312|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = $inrow;
  $31 = $2;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $inptr = $33;
  $34 = $outrow;
  $35 = $output_data;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $outptr = $37;
  $38 = $outptr;
  $39 = $0;
  $40 = (($39) + 112|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($38) + ($41)|0);
  $outend = $42;
  while(1) {
   $43 = $outptr;
   $44 = $outend;
   $45 = ($43>>>0)<($44>>>0);
   if (!($45)) {
    break;
   }
   $46 = $inptr;
   $47 = (($46) + 1|0);
   $inptr = $47;
   $48 = HEAP8[$46>>0]|0;
   $invalue = $48;
   $49 = $h_expand;
   $h = $49;
   while(1) {
    $50 = $h;
    $51 = ($50|0)>(0);
    if (!($51)) {
     break;
    }
    $52 = $invalue;
    $53 = $outptr;
    $54 = (($53) + 1|0);
    $outptr = $54;
    HEAP8[$53>>0] = $52;
    $55 = $h;
    $56 = (($55) + -1)|0;
    $h = $56;
   }
  }
  $57 = $v_expand;
  $58 = ($57|0)>(1);
  if ($58) {
   $59 = $output_data;
   $60 = $outrow;
   $61 = $output_data;
   $62 = $outrow;
   $63 = (($62) + 1)|0;
   $64 = $v_expand;
   $65 = (($64) - 1)|0;
   $66 = $0;
   $67 = (($66) + 112|0);
   $68 = HEAP32[$67>>2]|0;
   _jcopy_sample_rows($59,$60,$61,$63,$65,$68);
  }
  $69 = $inrow;
  $70 = (($69) + 1)|0;
  $inrow = $70;
  $71 = $v_expand;
  $72 = $outrow;
  $73 = (($72) + ($71))|0;
  $outrow = $73;
 }
 STACKTOP = sp;return;
}
function _jpeg_std_error($err) {
 $err = $err|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $err;
 $1 = $0;
 HEAP32[$1>>2] = 69;
 $2 = $0;
 $3 = (($2) + 4|0);
 HEAP32[$3>>2] = 70;
 $4 = $0;
 $5 = (($4) + 8|0);
 HEAP32[$5>>2] = 71;
 $6 = $0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 72;
 $8 = $0;
 $9 = (($8) + 16|0);
 HEAP32[$9>>2] = 73;
 $10 = $0;
 $11 = (($10) + 104|0);
 HEAP32[$11>>2] = 0;
 $12 = $0;
 $13 = (($12) + 108|0);
 HEAP32[$13>>2] = 0;
 $14 = $0;
 $15 = (($14) + 20|0);
 HEAP32[$15>>2] = 0;
 $16 = $0;
 $17 = (($16) + 112|0);
 HEAP32[$17>>2] = 49688;
 $18 = $0;
 $19 = (($18) + 116|0);
 HEAP32[$19>>2] = 123;
 $20 = $0;
 $21 = (($20) + 120|0);
 HEAP32[$21>>2] = 0;
 $22 = $0;
 $23 = (($22) + 124|0);
 HEAP32[$23>>2] = 0;
 $24 = $0;
 $25 = (($24) + 128|0);
 HEAP32[$25>>2] = 0;
 $26 = $0;
 STACKTOP = sp;return ($26|0);
}
function _error_exit($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 FUNCTION_TABLE_vi[$4 & 255]($5);
 $6 = $0;
 _jpeg_destroy($6);
 _exit(1);
 // unreachable;
}
function _emit_message($cinfo,$msg_level) {
 $cinfo = $cinfo|0;
 $msg_level = $msg_level|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $err = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $msg_level;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $err = $3;
 $4 = $1;
 $5 = ($4|0)<(0);
 if (!($5)) {
  $22 = $err;
  $23 = (($22) + 104|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $1;
  $26 = ($24|0)>=($25|0);
  if ($26) {
   $27 = $err;
   $28 = (($27) + 8|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $0;
   FUNCTION_TABLE_vi[$29 & 255]($30);
  }
  STACKTOP = sp;return;
 }
 $6 = $err;
 $7 = (($6) + 108|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  label = 4;
 } else {
  $10 = $err;
  $11 = (($10) + 104|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>=(3);
  if ($13) {
   label = 4;
  }
 }
 if ((label|0) == 4) {
  $14 = $err;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $0;
  FUNCTION_TABLE_vi[$16 & 255]($17);
 }
 $18 = $err;
 $19 = (($18) + 108|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _output_message($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buffer = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer = sp;
 $buffer = sp + 8|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 FUNCTION_TABLE_vii[$4 & 255]($5,$buffer);
 $6 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer>>2] = $buffer;
 (_fprintf(($6|0),(50192|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return;
}
function _format_message($cinfo,$buffer) {
 $cinfo = $cinfo|0;
 $buffer = $buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ch = 0, $err = 0, $isstring = 0, $msg_code = 0, $msgptr = 0, $msgtext = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr10 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $cinfo;
 $1 = $buffer;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $err = $3;
 $4 = $err;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $msg_code = $6;
 $msgtext = 0;
 $7 = $msg_code;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = $msg_code;
  $10 = $err;
  $11 = (($10) + 116|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<=($12|0);
  if ($13) {
   $14 = $msg_code;
   $15 = $err;
   $16 = (($15) + 112|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + ($14<<2)|0);
   $19 = HEAP32[$18>>2]|0;
   $msgtext = $19;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $20 = $err;
  $21 = (($20) + 120|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = $msg_code;
   $25 = $err;
   $26 = (($25) + 124|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($24|0)>=($27|0);
   if ($28) {
    $29 = $msg_code;
    $30 = $err;
    $31 = (($30) + 128|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($29|0)<=($32|0);
    if ($33) {
     $34 = $msg_code;
     $35 = $err;
     $36 = (($35) + 124|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = (($34) - ($37))|0;
     $39 = $err;
     $40 = (($39) + 120|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($38<<2)|0);
     $43 = HEAP32[$42>>2]|0;
     $msgtext = $43;
    }
   }
  }
 }
 $44 = $msgtext;
 $45 = ($44|0)==(0|0);
 if ($45) {
  $46 = $msg_code;
  $47 = $err;
  $48 = (($47) + 24|0);
  HEAP32[$48>>2] = $46;
  $49 = $err;
  $50 = (($49) + 112|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $msgtext = $52;
 }
 $isstring = 0;
 $53 = $msgtext;
 $msgptr = $53;
 while(1) {
  $54 = $msgptr;
  $55 = (($54) + 1|0);
  $msgptr = $55;
  $56 = HEAP8[$54>>0]|0;
  $ch = $56;
  $57 = $56 << 24 >> 24;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   break;
  }
  $59 = $ch;
  $60 = $59 << 24 >> 24;
  $61 = ($60|0)==(37);
  if ($61) {
   label = 14;
   break;
  }
 }
 if ((label|0) == 14) {
  $62 = $msgptr;
  $63 = HEAP8[$62>>0]|0;
  $64 = $63 << 24 >> 24;
  $65 = ($64|0)==(115);
  if ($65) {
   $isstring = 1;
  }
 }
 $66 = $isstring;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = $1;
  $69 = $msgtext;
  $70 = $err;
  $71 = (($70) + 24|0);
  HEAP32[$vararg_buffer>>2] = $71;
  (_sprintf($68,$69,$vararg_buffer)|0);
  STACKTOP = sp;return;
 } else {
  $72 = $1;
  $73 = $msgtext;
  $74 = $err;
  $75 = (($74) + 24|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $err;
  $78 = (($77) + 24|0);
  $79 = (($78) + 4|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = $err;
  $82 = (($81) + 24|0);
  $83 = (($82) + 8|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = $err;
  $86 = (($85) + 24|0);
  $87 = (($86) + 12|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $err;
  $90 = (($89) + 24|0);
  $91 = (($90) + 16|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = $err;
  $94 = (($93) + 24|0);
  $95 = (($94) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = $err;
  $98 = (($97) + 24|0);
  $99 = (($98) + 24|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = $err;
  $102 = (($101) + 24|0);
  $103 = (($102) + 28|0);
  $104 = HEAP32[$103>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $76;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $80;
  $vararg_ptr5 = (($vararg_buffer1) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $84;
  $vararg_ptr6 = (($vararg_buffer1) + 12|0);
  HEAP32[$vararg_ptr6>>2] = $88;
  $vararg_ptr7 = (($vararg_buffer1) + 16|0);
  HEAP32[$vararg_ptr7>>2] = $92;
  $vararg_ptr8 = (($vararg_buffer1) + 20|0);
  HEAP32[$vararg_ptr8>>2] = $96;
  $vararg_ptr9 = (($vararg_buffer1) + 24|0);
  HEAP32[$vararg_ptr9>>2] = $100;
  $vararg_ptr10 = (($vararg_buffer1) + 28|0);
  HEAP32[$vararg_ptr10>>2] = $104;
  (_sprintf($72,$73,$vararg_buffer1)|0);
  STACKTOP = sp;return;
 }
}
function _reset_error_mgr($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 108|0);
 HEAP32[$3>>2] = 0;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 20|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _jinit_memory_mgr($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $ch = 0, $max_to_use = 0, $mem = 0, $memenv = 0, $pool = 0, $test_mac = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $max_to_use = sp + 20|0;
 $ch = sp + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 HEAP32[$2>>2] = 0;
 $test_mac = 1000000000;
 $3 = $test_mac;
 $4 = ($3|0)!=(1000000000);
 if ($4) {
  $5 = $0;
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) + 20|0);
  HEAP32[$7>>2] = 3;
  $8 = $0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  FUNCTION_TABLE_vi[$10 & 255]($11);
 }
 $12 = $0;
 $13 = (_jpeg_mem_init($12)|0);
 HEAP32[$max_to_use>>2] = $13;
 $14 = $0;
 $15 = (_jpeg_get_small($14,84)|0);
 $mem = $15;
 $16 = $mem;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = $0;
  _jpeg_mem_term($18);
  $19 = $0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 54;
  $22 = $0;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 24|0);
  HEAP32[$24>>2] = 0;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  $28 = $0;
  FUNCTION_TABLE_vi[$27 & 255]($28);
 }
 $29 = $mem;
 HEAP32[$29>>2] = 74;
 $30 = $mem;
 $31 = (($30) + 4|0);
 HEAP32[$31>>2] = 75;
 $32 = $mem;
 $33 = (($32) + 8|0);
 HEAP32[$33>>2] = 76;
 $34 = $mem;
 $35 = (($34) + 12|0);
 HEAP32[$35>>2] = 77;
 $36 = $mem;
 $37 = (($36) + 16|0);
 HEAP32[$37>>2] = 78;
 $38 = $mem;
 $39 = (($38) + 20|0);
 HEAP32[$39>>2] = 79;
 $40 = $mem;
 $41 = (($40) + 24|0);
 HEAP32[$41>>2] = 80;
 $42 = $mem;
 $43 = (($42) + 28|0);
 HEAP32[$43>>2] = 81;
 $44 = $mem;
 $45 = (($44) + 32|0);
 HEAP32[$45>>2] = 82;
 $46 = $mem;
 $47 = (($46) + 36|0);
 HEAP32[$47>>2] = 83;
 $48 = $mem;
 $49 = (($48) + 40|0);
 HEAP32[$49>>2] = 84;
 $50 = $mem;
 $51 = (($50) + 48|0);
 HEAP32[$51>>2] = 1000000000;
 $52 = HEAP32[$max_to_use>>2]|0;
 $53 = $mem;
 $54 = (($53) + 44|0);
 HEAP32[$54>>2] = $52;
 $pool = 1;
 while(1) {
  $55 = $pool;
  $56 = ($55|0)>=(0);
  if (!($56)) {
   break;
  }
  $57 = $pool;
  $58 = $mem;
  $59 = (($58) + 52|0);
  $60 = (($59) + ($57<<2)|0);
  HEAP32[$60>>2] = 0;
  $61 = $pool;
  $62 = $mem;
  $63 = (($62) + 60|0);
  $64 = (($63) + ($61<<2)|0);
  HEAP32[$64>>2] = 0;
  $65 = $pool;
  $66 = (($65) + -1)|0;
  $pool = $66;
 }
 $67 = $mem;
 $68 = (($67) + 68|0);
 HEAP32[$68>>2] = 0;
 $69 = $mem;
 $70 = (($69) + 72|0);
 HEAP32[$70>>2] = 0;
 $71 = $mem;
 $72 = (($71) + 76|0);
 HEAP32[$72>>2] = 84;
 $73 = $mem;
 $74 = $0;
 $75 = (($74) + 4|0);
 HEAP32[$75>>2] = $73;
 $76 = (_getenv((50200|0))|0);
 $memenv = $76;
 $77 = ($76|0)!=(0|0);
 if (!($77)) {
  STACKTOP = sp;return;
 }
 HEAP8[$ch>>0] = 120;
 $78 = $memenv;
 HEAP32[$vararg_buffer>>2] = $max_to_use;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $ch;
 $79 = (_sscanf($78,50208,$vararg_buffer)|0);
 $80 = ($79|0)>(0);
 if ($80) {
  $81 = HEAP8[$ch>>0]|0;
  $82 = $81 << 24 >> 24;
  $83 = ($82|0)==(109);
  if ($83) {
   label = 13;
  } else {
   $84 = HEAP8[$ch>>0]|0;
   $85 = $84 << 24 >> 24;
   $86 = ($85|0)==(77);
   if ($86) {
    label = 13;
   }
  }
  if ((label|0) == 13) {
   $87 = HEAP32[$max_to_use>>2]|0;
   $88 = ($87*1000)|0;
   HEAP32[$max_to_use>>2] = $88;
  }
  $89 = HEAP32[$max_to_use>>2]|0;
  $90 = ($89*1000)|0;
  $91 = $mem;
  $92 = (($91) + 44|0);
  HEAP32[$92>>2] = $90;
 }
 STACKTOP = sp;return;
}
function _alloc_small($cinfo,$pool_id,$sizeofobject) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data_ptr = 0;
 var $hdr_ptr = 0, $mem = 0, $min_request = 0, $odd_bytes = 0, $prev_hdr_ptr = 0, $slop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $sizeofobject;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $mem = $5;
 $6 = $2;
 $7 = ($6>>>0)>(999999984);
 if ($7) {
  $8 = $0;
  _out_of_memory($8,1);
 }
 $9 = $2;
 $10 = (($9>>>0) % 8)&-1;
 $odd_bytes = $10;
 $11 = $odd_bytes;
 $12 = ($11>>>0)>(0);
 if ($12) {
  $13 = $odd_bytes;
  $14 = (8 - ($13))|0;
  $15 = $2;
  $16 = (($15) + ($14))|0;
  $2 = $16;
 }
 $17 = $1;
 $18 = ($17|0)<(0);
 if ($18) {
  label = 7;
 } else {
  $19 = $1;
  $20 = ($19|0)>=(2);
  if ($20) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 14;
  $24 = $1;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 24|0);
  HEAP32[$27>>2] = $24;
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vi[$30 & 255]($31);
 }
 $prev_hdr_ptr = 0;
 $32 = $1;
 $33 = $mem;
 $34 = (($33) + 52|0);
 $35 = (($34) + ($32<<2)|0);
 $36 = HEAP32[$35>>2]|0;
 $hdr_ptr = $36;
 while(1) {
  $37 = $hdr_ptr;
  $38 = ($37|0)!=(0|0);
  if (!($38)) {
   break;
  }
  $39 = $hdr_ptr;
  $40 = (($39) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $2;
  $43 = ($41>>>0)>=($42>>>0);
  if ($43) {
   label = 11;
   break;
  }
  $44 = $hdr_ptr;
  $prev_hdr_ptr = $44;
  $45 = $hdr_ptr;
  $46 = HEAP32[$45>>2]|0;
  $hdr_ptr = $46;
 }
 if ((label|0) == 11) {
 }
 $47 = $hdr_ptr;
 $48 = ($47|0)==(0|0);
 if (!($48)) {
  $101 = $hdr_ptr;
  $102 = (($101) + 16|0);
  $data_ptr = $102;
  $103 = $hdr_ptr;
  $104 = (($103) + 4|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = $data_ptr;
  $107 = (($106) + ($105)|0);
  $data_ptr = $107;
  $108 = $2;
  $109 = $hdr_ptr;
  $110 = (($109) + 4|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (($111) + ($108))|0;
  HEAP32[$110>>2] = $112;
  $113 = $2;
  $114 = $hdr_ptr;
  $115 = (($114) + 8|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (($116) - ($113))|0;
  HEAP32[$115>>2] = $117;
  $118 = $data_ptr;
  STACKTOP = sp;return ($118|0);
 }
 $49 = $2;
 $50 = (($49) + 16)|0;
 $min_request = $50;
 $51 = $prev_hdr_ptr;
 $52 = ($51|0)==(0|0);
 if ($52) {
  $53 = $1;
  $54 = (50216 + ($53<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $slop = $55;
 } else {
  $56 = $1;
  $57 = (50224 + ($56<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $slop = $58;
 }
 $59 = $slop;
 $60 = $min_request;
 $61 = (1000000000 - ($60))|0;
 $62 = ($59>>>0)>($61>>>0);
 if ($62) {
  $63 = $min_request;
  $64 = (1000000000 - ($63))|0;
  $slop = $64;
 }
 while(1) {
  $65 = $0;
  $66 = $min_request;
  $67 = $slop;
  $68 = (($66) + ($67))|0;
  $69 = (_jpeg_get_small($65,$68)|0);
  $hdr_ptr = $69;
  $70 = $hdr_ptr;
  $71 = ($70|0)!=(0|0);
  if ($71) {
   break;
  }
  $72 = $slop;
  $73 = (($72>>>0) / 2)&-1;
  $slop = $73;
  $74 = $slop;
  $75 = ($74>>>0)<(50);
  if ($75) {
   $76 = $0;
   _out_of_memory($76,2);
  }
 }
 $77 = $min_request;
 $78 = $slop;
 $79 = (($77) + ($78))|0;
 $80 = $mem;
 $81 = (($80) + 76|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = (($82) + ($79))|0;
 HEAP32[$81>>2] = $83;
 $84 = $hdr_ptr;
 HEAP32[$84>>2] = 0;
 $85 = $hdr_ptr;
 $86 = (($85) + 4|0);
 HEAP32[$86>>2] = 0;
 $87 = $2;
 $88 = $slop;
 $89 = (($87) + ($88))|0;
 $90 = $hdr_ptr;
 $91 = (($90) + 8|0);
 HEAP32[$91>>2] = $89;
 $92 = $prev_hdr_ptr;
 $93 = ($92|0)==(0|0);
 if ($93) {
  $94 = $hdr_ptr;
  $95 = $1;
  $96 = $mem;
  $97 = (($96) + 52|0);
  $98 = (($97) + ($95<<2)|0);
  HEAP32[$98>>2] = $94;
 } else {
  $99 = $hdr_ptr;
  $100 = $prev_hdr_ptr;
  HEAP32[$100>>2] = $99;
 }
 $101 = $hdr_ptr;
 $102 = (($101) + 16|0);
 $data_ptr = $102;
 $103 = $hdr_ptr;
 $104 = (($103) + 4|0);
 $105 = HEAP32[$104>>2]|0;
 $106 = $data_ptr;
 $107 = (($106) + ($105)|0);
 $data_ptr = $107;
 $108 = $2;
 $109 = $hdr_ptr;
 $110 = (($109) + 4|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = (($111) + ($108))|0;
 HEAP32[$110>>2] = $112;
 $113 = $2;
 $114 = $hdr_ptr;
 $115 = (($114) + 8|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = (($116) - ($113))|0;
 HEAP32[$115>>2] = $117;
 $118 = $data_ptr;
 STACKTOP = sp;return ($118|0);
}
function _alloc_large($cinfo,$pool_id,$sizeofobject) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $hdr_ptr = 0, $mem = 0, $odd_bytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $sizeofobject;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $mem = $5;
 $6 = $2;
 $7 = ($6>>>0)>(999999984);
 if ($7) {
  $8 = $0;
  _out_of_memory($8,3);
 }
 $9 = $2;
 $10 = (($9>>>0) % 8)&-1;
 $odd_bytes = $10;
 $11 = $odd_bytes;
 $12 = ($11>>>0)>(0);
 if ($12) {
  $13 = $odd_bytes;
  $14 = (8 - ($13))|0;
  $15 = $2;
  $16 = (($15) + ($14))|0;
  $2 = $16;
 }
 $17 = $1;
 $18 = ($17|0)<(0);
 if ($18) {
  label = 7;
 } else {
  $19 = $1;
  $20 = ($19|0)>=(2);
  if ($20) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 14;
  $24 = $1;
  $25 = $0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 24|0);
  HEAP32[$27>>2] = $24;
  $28 = $0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $0;
  FUNCTION_TABLE_vi[$30 & 255]($31);
 }
 $32 = $0;
 $33 = $2;
 $34 = (($33) + 16)|0;
 $35 = (_jpeg_get_large($32,$34)|0);
 $hdr_ptr = $35;
 $36 = $hdr_ptr;
 $37 = ($36|0)==(0|0);
 if ($37) {
  $38 = $0;
  _out_of_memory($38,4);
 }
 $39 = $2;
 $40 = (($39) + 16)|0;
 $41 = $mem;
 $42 = (($41) + 76|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + ($40))|0;
 HEAP32[$42>>2] = $44;
 $45 = $1;
 $46 = $mem;
 $47 = (($46) + 60|0);
 $48 = (($47) + ($45<<2)|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $hdr_ptr;
 HEAP32[$50>>2] = $49;
 $51 = $2;
 $52 = $hdr_ptr;
 $53 = (($52) + 4|0);
 HEAP32[$53>>2] = $51;
 $54 = $hdr_ptr;
 $55 = (($54) + 8|0);
 HEAP32[$55>>2] = 0;
 $56 = $hdr_ptr;
 $57 = $1;
 $58 = $mem;
 $59 = (($58) + 60|0);
 $60 = (($59) + ($57<<2)|0);
 HEAP32[$60>>2] = $56;
 $61 = $hdr_ptr;
 $62 = (($61) + 16|0);
 STACKTOP = sp;return ($62|0);
}
function _alloc_sarray($cinfo,$pool_id,$samplesperrow,$numrows) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $samplesperrow = $samplesperrow|0;
 $numrows = $numrows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $currow = 0, $i = 0, $ltemp = 0, $mem = 0, $result = 0, $rowsperchunk = 0, $workspace = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $samplesperrow;
 $3 = $numrows;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $mem = $6;
 $7 = $2;
 $8 = $7;
 $9 = (999999984 / ($8>>>0))&-1;
 $ltemp = $9;
 $10 = $ltemp;
 $11 = ($10|0)<=(0);
 if ($11) {
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = 70;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 255]($18);
 }
 $19 = $ltemp;
 $20 = $3;
 $21 = ($19|0)<($20|0);
 if ($21) {
  $22 = $ltemp;
  $rowsperchunk = $22;
 } else {
  $23 = $3;
  $rowsperchunk = $23;
 }
 $24 = $rowsperchunk;
 $25 = $mem;
 $26 = (($25) + 80|0);
 HEAP32[$26>>2] = $24;
 $27 = $0;
 $28 = $1;
 $29 = $3;
 $30 = $29<<2;
 $31 = (_alloc_small($27,$28,$30)|0);
 $result = $31;
 $currow = 0;
 while(1) {
  $32 = $currow;
  $33 = $3;
  $34 = ($32>>>0)<($33>>>0);
  if (!($34)) {
   break;
  }
  $35 = $rowsperchunk;
  $36 = $3;
  $37 = $currow;
  $38 = (($36) - ($37))|0;
  $39 = ($35>>>0)<($38>>>0);
  if ($39) {
   $40 = $rowsperchunk;
   $44 = $40;
  } else {
   $41 = $3;
   $42 = $currow;
   $43 = (($41) - ($42))|0;
   $44 = $43;
  }
  $rowsperchunk = $44;
  $45 = $0;
  $46 = $1;
  $47 = $rowsperchunk;
  $48 = $2;
  $49 = Math_imul($47, $48)|0;
  $50 = $49;
  $51 = (_alloc_large($45,$46,$50)|0);
  $workspace = $51;
  $52 = $rowsperchunk;
  $i = $52;
  while(1) {
   $53 = $i;
   $54 = ($53>>>0)>(0);
   if (!($54)) {
    break;
   }
   $55 = $workspace;
   $56 = $currow;
   $57 = (($56) + 1)|0;
   $currow = $57;
   $58 = $result;
   $59 = (($58) + ($56<<2)|0);
   HEAP32[$59>>2] = $55;
   $60 = $2;
   $61 = $workspace;
   $62 = (($61) + ($60)|0);
   $workspace = $62;
   $63 = $i;
   $64 = (($63) + -1)|0;
   $i = $64;
  }
 }
 $65 = $result;
 STACKTOP = sp;return ($65|0);
}
function _alloc_barray($cinfo,$pool_id,$blocksperrow,$numrows) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $blocksperrow = $blocksperrow|0;
 $numrows = $numrows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $currow = 0, $i = 0, $ltemp = 0, $mem = 0, $result = 0, $rowsperchunk = 0, $workspace = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $blocksperrow;
 $3 = $numrows;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $mem = $6;
 $7 = $2;
 $8 = $7<<7;
 $9 = (999999984 / ($8>>>0))&-1;
 $ltemp = $9;
 $10 = $ltemp;
 $11 = ($10|0)<=(0);
 if ($11) {
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = 70;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 255]($18);
 }
 $19 = $ltemp;
 $20 = $3;
 $21 = ($19|0)<($20|0);
 if ($21) {
  $22 = $ltemp;
  $rowsperchunk = $22;
 } else {
  $23 = $3;
  $rowsperchunk = $23;
 }
 $24 = $rowsperchunk;
 $25 = $mem;
 $26 = (($25) + 80|0);
 HEAP32[$26>>2] = $24;
 $27 = $0;
 $28 = $1;
 $29 = $3;
 $30 = $29<<2;
 $31 = (_alloc_small($27,$28,$30)|0);
 $result = $31;
 $currow = 0;
 while(1) {
  $32 = $currow;
  $33 = $3;
  $34 = ($32>>>0)<($33>>>0);
  if (!($34)) {
   break;
  }
  $35 = $rowsperchunk;
  $36 = $3;
  $37 = $currow;
  $38 = (($36) - ($37))|0;
  $39 = ($35>>>0)<($38>>>0);
  if ($39) {
   $40 = $rowsperchunk;
   $44 = $40;
  } else {
   $41 = $3;
   $42 = $currow;
   $43 = (($41) - ($42))|0;
   $44 = $43;
  }
  $rowsperchunk = $44;
  $45 = $0;
  $46 = $1;
  $47 = $rowsperchunk;
  $48 = $2;
  $49 = Math_imul($47, $48)|0;
  $50 = $49<<7;
  $51 = (_alloc_large($45,$46,$50)|0);
  $workspace = $51;
  $52 = $rowsperchunk;
  $i = $52;
  while(1) {
   $53 = $i;
   $54 = ($53>>>0)>(0);
   if (!($54)) {
    break;
   }
   $55 = $workspace;
   $56 = $currow;
   $57 = (($56) + 1)|0;
   $currow = $57;
   $58 = $result;
   $59 = (($58) + ($56<<2)|0);
   HEAP32[$59>>2] = $55;
   $60 = $2;
   $61 = $workspace;
   $62 = (($61) + ($60<<7)|0);
   $workspace = $62;
   $63 = $i;
   $64 = (($63) + -1)|0;
   $i = $64;
  }
 }
 $65 = $result;
 STACKTOP = sp;return ($65|0);
}
function _request_virt_sarray($cinfo,$pool_id,$pre_zero,$samplesperrow,$numrows,$maxaccess) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $pre_zero = $pre_zero|0;
 $samplesperrow = $samplesperrow|0;
 $numrows = $numrows|0;
 $maxaccess = $maxaccess|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mem = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $pre_zero;
 $3 = $samplesperrow;
 $4 = $numrows;
 $5 = $maxaccess;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mem = $8;
 $9 = $1;
 $10 = ($9|0)!=(1);
 if ($10) {
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 20|0);
  HEAP32[$13>>2] = 14;
  $14 = $1;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 24|0);
  HEAP32[$17>>2] = $14;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 255]($21);
 }
 $22 = $0;
 $23 = $1;
 $24 = (_alloc_small($22,$23,128)|0);
 $result = $24;
 $25 = $result;
 HEAP32[$25>>2] = 0;
 $26 = $4;
 $27 = $result;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = $3;
 $30 = $result;
 $31 = (($30) + 8|0);
 HEAP32[$31>>2] = $29;
 $32 = $5;
 $33 = $result;
 $34 = (($33) + 12|0);
 HEAP32[$34>>2] = $32;
 $35 = $2;
 $36 = $result;
 $37 = (($36) + 32|0);
 HEAP32[$37>>2] = $35;
 $38 = $result;
 $39 = (($38) + 40|0);
 HEAP32[$39>>2] = 0;
 $40 = $mem;
 $41 = (($40) + 68|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $result;
 $44 = (($43) + 44|0);
 HEAP32[$44>>2] = $42;
 $45 = $result;
 $46 = $mem;
 $47 = (($46) + 68|0);
 HEAP32[$47>>2] = $45;
 $48 = $result;
 STACKTOP = sp;return ($48|0);
}
function _request_virt_barray($cinfo,$pool_id,$pre_zero,$blocksperrow,$numrows,$maxaccess) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 $pre_zero = $pre_zero|0;
 $blocksperrow = $blocksperrow|0;
 $numrows = $numrows|0;
 $maxaccess = $maxaccess|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mem = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $pre_zero;
 $3 = $blocksperrow;
 $4 = $numrows;
 $5 = $maxaccess;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mem = $8;
 $9 = $1;
 $10 = ($9|0)!=(1);
 if ($10) {
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 20|0);
  HEAP32[$13>>2] = 14;
  $14 = $1;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 24|0);
  HEAP32[$17>>2] = $14;
  $18 = $0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  FUNCTION_TABLE_vi[$20 & 255]($21);
 }
 $22 = $0;
 $23 = $1;
 $24 = (_alloc_small($22,$23,128)|0);
 $result = $24;
 $25 = $result;
 HEAP32[$25>>2] = 0;
 $26 = $4;
 $27 = $result;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = $3;
 $30 = $result;
 $31 = (($30) + 8|0);
 HEAP32[$31>>2] = $29;
 $32 = $5;
 $33 = $result;
 $34 = (($33) + 12|0);
 HEAP32[$34>>2] = $32;
 $35 = $2;
 $36 = $result;
 $37 = (($36) + 32|0);
 HEAP32[$37>>2] = $35;
 $38 = $result;
 $39 = (($38) + 40|0);
 HEAP32[$39>>2] = 0;
 $40 = $mem;
 $41 = (($40) + 72|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $result;
 $44 = (($43) + 44|0);
 HEAP32[$44>>2] = $42;
 $45 = $result;
 $46 = $mem;
 $47 = (($46) + 72|0);
 HEAP32[$47>>2] = $45;
 $48 = $result;
 STACKTOP = sp;return ($48|0);
}
function _realize_virt_arrays($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $avail_mem = 0;
 var $bptr = 0, $max_minheights = 0, $maximum_space = 0, $mem = 0, $minheights = 0, $space_per_minheight = 0, $sptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $mem = $3;
 $space_per_minheight = 0;
 $maximum_space = 0;
 $4 = $mem;
 $5 = (($4) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $sptr = $6;
 while(1) {
  $7 = $sptr;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = $sptr;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   $12 = $sptr;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $sptr;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = Math_imul($14, $17)|0;
   $19 = $18;
   $20 = $space_per_minheight;
   $21 = (($20) + ($19))|0;
   $space_per_minheight = $21;
   $22 = $sptr;
   $23 = (($22) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = $sptr;
   $26 = (($25) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = Math_imul($24, $27)|0;
   $29 = $28;
   $30 = $maximum_space;
   $31 = (($30) + ($29))|0;
   $maximum_space = $31;
  }
  $32 = $sptr;
  $33 = (($32) + 44|0);
  $34 = HEAP32[$33>>2]|0;
  $sptr = $34;
 }
 $35 = $mem;
 $36 = (($35) + 72|0);
 $37 = HEAP32[$36>>2]|0;
 $bptr = $37;
 while(1) {
  $38 = $bptr;
  $39 = ($38|0)!=(0|0);
  if (!($39)) {
   break;
  }
  $40 = $bptr;
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)==(0|0);
  if ($42) {
   $43 = $bptr;
   $44 = (($43) + 12|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = $bptr;
   $47 = (($46) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = Math_imul($45, $48)|0;
   $50 = $49<<7;
   $51 = $space_per_minheight;
   $52 = (($51) + ($50))|0;
   $space_per_minheight = $52;
   $53 = $bptr;
   $54 = (($53) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = $bptr;
   $57 = (($56) + 8|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = Math_imul($55, $58)|0;
   $60 = $59<<7;
   $61 = $maximum_space;
   $62 = (($61) + ($60))|0;
   $maximum_space = $62;
  }
  $63 = $bptr;
  $64 = (($63) + 44|0);
  $65 = HEAP32[$64>>2]|0;
  $bptr = $65;
 }
 $66 = $space_per_minheight;
 $67 = ($66|0)<=(0);
 if ($67) {
  STACKTOP = sp;return;
 }
 $68 = $0;
 $69 = $space_per_minheight;
 $70 = $maximum_space;
 $71 = $mem;
 $72 = (($71) + 76|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = (_jpeg_mem_available($68,$69,$70,$73)|0);
 $avail_mem = $74;
 $75 = $avail_mem;
 $76 = $maximum_space;
 $77 = ($75|0)>=($76|0);
 if ($77) {
  $max_minheights = 1000000000;
 } else {
  $78 = $avail_mem;
  $79 = $space_per_minheight;
  $80 = (($78|0) / ($79|0))&-1;
  $max_minheights = $80;
  $81 = $max_minheights;
  $82 = ($81|0)<=(0);
  if ($82) {
   $max_minheights = 1;
  }
 }
 $83 = $mem;
 $84 = (($83) + 68|0);
 $85 = HEAP32[$84>>2]|0;
 $sptr = $85;
 while(1) {
  $86 = $sptr;
  $87 = ($86|0)!=(0|0);
  if (!($87)) {
   break;
  }
  $88 = $sptr;
  $89 = HEAP32[$88>>2]|0;
  $90 = ($89|0)==(0|0);
  if ($90) {
   $91 = $sptr;
   $92 = (($91) + 4|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (($93) - 1)|0;
   $95 = $sptr;
   $96 = (($95) + 12|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = (($94>>>0) / ($97>>>0))&-1;
   $99 = (($98) + 1)|0;
   $minheights = $99;
   $100 = $minheights;
   $101 = $max_minheights;
   $102 = ($100|0)<=($101|0);
   if ($102) {
    $103 = $sptr;
    $104 = (($103) + 4|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $sptr;
    $107 = (($106) + 16|0);
    HEAP32[$107>>2] = $105;
   } else {
    $108 = $max_minheights;
    $109 = $sptr;
    $110 = (($109) + 12|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = Math_imul($108, $111)|0;
    $113 = $sptr;
    $114 = (($113) + 16|0);
    HEAP32[$114>>2] = $112;
    $115 = $0;
    $116 = $sptr;
    $117 = (($116) + 48|0);
    $118 = $sptr;
    $119 = (($118) + 4|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = $sptr;
    $122 = (($121) + 8|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = Math_imul($120, $123)|0;
    $125 = $124;
    _jpeg_open_backing_store($115,$117,$125);
    $126 = $sptr;
    $127 = (($126) + 40|0);
    HEAP32[$127>>2] = 1;
   }
   $128 = $0;
   $129 = $sptr;
   $130 = (($129) + 8|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = $sptr;
   $133 = (($132) + 16|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = (_alloc_sarray($128,1,$131,$134)|0);
   $136 = $sptr;
   HEAP32[$136>>2] = $135;
   $137 = $mem;
   $138 = (($137) + 80|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $sptr;
   $141 = (($140) + 20|0);
   HEAP32[$141>>2] = $139;
   $142 = $sptr;
   $143 = (($142) + 24|0);
   HEAP32[$143>>2] = 0;
   $144 = $sptr;
   $145 = (($144) + 28|0);
   HEAP32[$145>>2] = 0;
   $146 = $sptr;
   $147 = (($146) + 36|0);
   HEAP32[$147>>2] = 0;
  }
  $148 = $sptr;
  $149 = (($148) + 44|0);
  $150 = HEAP32[$149>>2]|0;
  $sptr = $150;
 }
 $151 = $mem;
 $152 = (($151) + 72|0);
 $153 = HEAP32[$152>>2]|0;
 $bptr = $153;
 while(1) {
  $154 = $bptr;
  $155 = ($154|0)!=(0|0);
  if (!($155)) {
   break;
  }
  $156 = $bptr;
  $157 = HEAP32[$156>>2]|0;
  $158 = ($157|0)==(0|0);
  if ($158) {
   $159 = $bptr;
   $160 = (($159) + 4|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = (($161) - 1)|0;
   $163 = $bptr;
   $164 = (($163) + 12|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = (($162>>>0) / ($165>>>0))&-1;
   $167 = (($166) + 1)|0;
   $minheights = $167;
   $168 = $minheights;
   $169 = $max_minheights;
   $170 = ($168|0)<=($169|0);
   if ($170) {
    $171 = $bptr;
    $172 = (($171) + 4|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = $bptr;
    $175 = (($174) + 16|0);
    HEAP32[$175>>2] = $173;
   } else {
    $176 = $max_minheights;
    $177 = $bptr;
    $178 = (($177) + 12|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = Math_imul($176, $179)|0;
    $181 = $bptr;
    $182 = (($181) + 16|0);
    HEAP32[$182>>2] = $180;
    $183 = $0;
    $184 = $bptr;
    $185 = (($184) + 48|0);
    $186 = $bptr;
    $187 = (($186) + 4|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = $bptr;
    $190 = (($189) + 8|0);
    $191 = HEAP32[$190>>2]|0;
    $192 = Math_imul($188, $191)|0;
    $193 = $192<<7;
    _jpeg_open_backing_store($183,$185,$193);
    $194 = $bptr;
    $195 = (($194) + 40|0);
    HEAP32[$195>>2] = 1;
   }
   $196 = $0;
   $197 = $bptr;
   $198 = (($197) + 8|0);
   $199 = HEAP32[$198>>2]|0;
   $200 = $bptr;
   $201 = (($200) + 16|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (_alloc_barray($196,1,$199,$202)|0);
   $204 = $bptr;
   HEAP32[$204>>2] = $203;
   $205 = $mem;
   $206 = (($205) + 80|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = $bptr;
   $209 = (($208) + 20|0);
   HEAP32[$209>>2] = $207;
   $210 = $bptr;
   $211 = (($210) + 24|0);
   HEAP32[$211>>2] = 0;
   $212 = $bptr;
   $213 = (($212) + 28|0);
   HEAP32[$213>>2] = 0;
   $214 = $bptr;
   $215 = (($214) + 36|0);
   HEAP32[$215>>2] = 0;
  }
  $216 = $bptr;
  $217 = (($216) + 44|0);
  $218 = HEAP32[$217>>2]|0;
  $bptr = $218;
 }
 STACKTOP = sp;return;
}
function _access_virt_sarray($cinfo,$ptr,$start_row,$num_rows,$writable) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $start_row = $start_row|0;
 $num_rows = $num_rows|0;
 $writable = $writable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytesperrow = 0;
 var $end_row = 0, $ltemp = 0, $undef_row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $start_row;
 $3 = $num_rows;
 $4 = $writable;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6))|0;
 $end_row = $7;
 $8 = $end_row;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 if ($12) {
  label = 4;
 } else {
  $13 = $3;
  $14 = $1;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13>>>0)>($16>>>0);
  if ($17) {
   label = 4;
  } else {
   $18 = $1;
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    label = 4;
   }
  }
 }
 if ((label|0) == 4) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 22;
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vi[$26 & 255]($27);
 }
 $28 = $2;
 $29 = $1;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($28>>>0)<($31>>>0);
 if ($32) {
  label = 7;
 } else {
  $33 = $end_row;
  $34 = $1;
  $35 = (($34) + 24|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $1;
  $38 = (($37) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($36) + ($39))|0;
  $41 = ($33>>>0)>($40>>>0);
  if ($41) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $42 = $1;
  $43 = (($42) + 40|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(0);
  if (!($45)) {
   $46 = $0;
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + 20|0);
   HEAP32[$48>>2] = 69;
   $49 = $0;
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$50>>2]|0;
   $52 = $0;
   FUNCTION_TABLE_vi[$51 & 255]($52);
  }
  $53 = $1;
  $54 = (($53) + 36|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0);
  if ($56) {
   $57 = $0;
   $58 = $1;
   _do_sarray_io($57,$58,1);
   $59 = $1;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = 0;
  }
  $61 = $2;
  $62 = $1;
  $63 = (($62) + 24|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($61>>>0)>($64>>>0);
  if ($65) {
   $66 = $2;
   $67 = $1;
   $68 = (($67) + 24|0);
   HEAP32[$68>>2] = $66;
  } else {
   $69 = $end_row;
   $70 = $1;
   $71 = (($70) + 16|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($69) - ($72))|0;
   $ltemp = $73;
   $74 = $ltemp;
   $75 = ($74|0)<(0);
   if ($75) {
    $ltemp = 0;
   }
   $76 = $ltemp;
   $77 = $1;
   $78 = (($77) + 24|0);
   HEAP32[$78>>2] = $76;
  }
  $79 = $0;
  $80 = $1;
  _do_sarray_io($79,$80,0);
 }
 $81 = $1;
 $82 = (($81) + 28|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = $end_row;
 $85 = ($83>>>0)<($84>>>0);
 if ($85) {
  $86 = $1;
  $87 = (($86) + 28|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $2;
  $90 = ($88>>>0)<($89>>>0);
  if ($90) {
   $91 = $4;
   $92 = ($91|0)!=(0);
   if ($92) {
    $93 = $0;
    $94 = HEAP32[$93>>2]|0;
    $95 = (($94) + 20|0);
    HEAP32[$95>>2] = 22;
    $96 = $0;
    $97 = HEAP32[$96>>2]|0;
    $98 = HEAP32[$97>>2]|0;
    $99 = $0;
    FUNCTION_TABLE_vi[$98 & 255]($99);
   }
   $100 = $2;
   $undef_row = $100;
  } else {
   $101 = $1;
   $102 = (($101) + 28|0);
   $103 = HEAP32[$102>>2]|0;
   $undef_row = $103;
  }
  $104 = $4;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = $end_row;
   $107 = $1;
   $108 = (($107) + 28|0);
   HEAP32[$108>>2] = $106;
  }
  $109 = $1;
  $110 = (($109) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ($111|0)!=(0);
  if ($112) {
   $113 = $1;
   $114 = (($113) + 8|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = $115;
   $bytesperrow = $116;
   $117 = $1;
   $118 = (($117) + 24|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = $undef_row;
   $121 = (($120) - ($119))|0;
   $undef_row = $121;
   $122 = $1;
   $123 = (($122) + 24|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = $end_row;
   $126 = (($125) - ($124))|0;
   $end_row = $126;
   while(1) {
    $127 = $undef_row;
    $128 = $end_row;
    $129 = ($127>>>0)<($128>>>0);
    if (!($129)) {
     break;
    }
    $130 = $undef_row;
    $131 = $1;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($132) + ($130<<2)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $bytesperrow;
    _jzero_far($134,$135);
    $136 = $undef_row;
    $137 = (($136) + 1)|0;
    $undef_row = $137;
   }
  } else {
   $138 = $4;
   $139 = ($138|0)!=(0);
   if (!($139)) {
    $140 = $0;
    $141 = HEAP32[$140>>2]|0;
    $142 = (($141) + 20|0);
    HEAP32[$142>>2] = 22;
    $143 = $0;
    $144 = HEAP32[$143>>2]|0;
    $145 = HEAP32[$144>>2]|0;
    $146 = $0;
    FUNCTION_TABLE_vi[$145 & 255]($146);
   }
  }
 }
 $147 = $4;
 $148 = ($147|0)!=(0);
 if (!($148)) {
  $151 = $1;
  $152 = HEAP32[$151>>2]|0;
  $153 = $2;
  $154 = $1;
  $155 = (($154) + 24|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = (($153) - ($156))|0;
  $158 = (($152) + ($157<<2)|0);
  STACKTOP = sp;return ($158|0);
 }
 $149 = $1;
 $150 = (($149) + 36|0);
 HEAP32[$150>>2] = 1;
 $151 = $1;
 $152 = HEAP32[$151>>2]|0;
 $153 = $2;
 $154 = $1;
 $155 = (($154) + 24|0);
 $156 = HEAP32[$155>>2]|0;
 $157 = (($153) - ($156))|0;
 $158 = (($152) + ($157<<2)|0);
 STACKTOP = sp;return ($158|0);
}
function _access_virt_barray($cinfo,$ptr,$start_row,$num_rows,$writable) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $start_row = $start_row|0;
 $num_rows = $num_rows|0;
 $writable = $writable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytesperrow = 0;
 var $end_row = 0, $ltemp = 0, $undef_row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $start_row;
 $3 = $num_rows;
 $4 = $writable;
 $5 = $2;
 $6 = $3;
 $7 = (($5) + ($6))|0;
 $end_row = $7;
 $8 = $end_row;
 $9 = $1;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 if ($12) {
  label = 4;
 } else {
  $13 = $3;
  $14 = $1;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13>>>0)>($16>>>0);
  if ($17) {
   label = 4;
  } else {
   $18 = $1;
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    label = 4;
   }
  }
 }
 if ((label|0) == 4) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 22;
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vi[$26 & 255]($27);
 }
 $28 = $2;
 $29 = $1;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($28>>>0)<($31>>>0);
 if ($32) {
  label = 7;
 } else {
  $33 = $end_row;
  $34 = $1;
  $35 = (($34) + 24|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $1;
  $38 = (($37) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($36) + ($39))|0;
  $41 = ($33>>>0)>($40>>>0);
  if ($41) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $42 = $1;
  $43 = (($42) + 40|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)!=(0);
  if (!($45)) {
   $46 = $0;
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + 20|0);
   HEAP32[$48>>2] = 69;
   $49 = $0;
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$50>>2]|0;
   $52 = $0;
   FUNCTION_TABLE_vi[$51 & 255]($52);
  }
  $53 = $1;
  $54 = (($53) + 36|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0);
  if ($56) {
   $57 = $0;
   $58 = $1;
   _do_barray_io($57,$58,1);
   $59 = $1;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = 0;
  }
  $61 = $2;
  $62 = $1;
  $63 = (($62) + 24|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($61>>>0)>($64>>>0);
  if ($65) {
   $66 = $2;
   $67 = $1;
   $68 = (($67) + 24|0);
   HEAP32[$68>>2] = $66;
  } else {
   $69 = $end_row;
   $70 = $1;
   $71 = (($70) + 16|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($69) - ($72))|0;
   $ltemp = $73;
   $74 = $ltemp;
   $75 = ($74|0)<(0);
   if ($75) {
    $ltemp = 0;
   }
   $76 = $ltemp;
   $77 = $1;
   $78 = (($77) + 24|0);
   HEAP32[$78>>2] = $76;
  }
  $79 = $0;
  $80 = $1;
  _do_barray_io($79,$80,0);
 }
 $81 = $1;
 $82 = (($81) + 28|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = $end_row;
 $85 = ($83>>>0)<($84>>>0);
 if ($85) {
  $86 = $1;
  $87 = (($86) + 28|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $2;
  $90 = ($88>>>0)<($89>>>0);
  if ($90) {
   $91 = $4;
   $92 = ($91|0)!=(0);
   if ($92) {
    $93 = $0;
    $94 = HEAP32[$93>>2]|0;
    $95 = (($94) + 20|0);
    HEAP32[$95>>2] = 22;
    $96 = $0;
    $97 = HEAP32[$96>>2]|0;
    $98 = HEAP32[$97>>2]|0;
    $99 = $0;
    FUNCTION_TABLE_vi[$98 & 255]($99);
   }
   $100 = $2;
   $undef_row = $100;
  } else {
   $101 = $1;
   $102 = (($101) + 28|0);
   $103 = HEAP32[$102>>2]|0;
   $undef_row = $103;
  }
  $104 = $4;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = $end_row;
   $107 = $1;
   $108 = (($107) + 28|0);
   HEAP32[$108>>2] = $106;
  }
  $109 = $1;
  $110 = (($109) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ($111|0)!=(0);
  if ($112) {
   $113 = $1;
   $114 = (($113) + 8|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = $115<<7;
   $bytesperrow = $116;
   $117 = $1;
   $118 = (($117) + 24|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = $undef_row;
   $121 = (($120) - ($119))|0;
   $undef_row = $121;
   $122 = $1;
   $123 = (($122) + 24|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = $end_row;
   $126 = (($125) - ($124))|0;
   $end_row = $126;
   while(1) {
    $127 = $undef_row;
    $128 = $end_row;
    $129 = ($127>>>0)<($128>>>0);
    if (!($129)) {
     break;
    }
    $130 = $undef_row;
    $131 = $1;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($132) + ($130<<2)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $bytesperrow;
    _jzero_far($134,$135);
    $136 = $undef_row;
    $137 = (($136) + 1)|0;
    $undef_row = $137;
   }
  } else {
   $138 = $4;
   $139 = ($138|0)!=(0);
   if (!($139)) {
    $140 = $0;
    $141 = HEAP32[$140>>2]|0;
    $142 = (($141) + 20|0);
    HEAP32[$142>>2] = 22;
    $143 = $0;
    $144 = HEAP32[$143>>2]|0;
    $145 = HEAP32[$144>>2]|0;
    $146 = $0;
    FUNCTION_TABLE_vi[$145 & 255]($146);
   }
  }
 }
 $147 = $4;
 $148 = ($147|0)!=(0);
 if (!($148)) {
  $151 = $1;
  $152 = HEAP32[$151>>2]|0;
  $153 = $2;
  $154 = $1;
  $155 = (($154) + 24|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = (($153) - ($156))|0;
  $158 = (($152) + ($157<<2)|0);
  STACKTOP = sp;return ($158|0);
 }
 $149 = $1;
 $150 = (($149) + 36|0);
 HEAP32[$150>>2] = 1;
 $151 = $1;
 $152 = HEAP32[$151>>2]|0;
 $153 = $2;
 $154 = $1;
 $155 = (($154) + 24|0);
 $156 = HEAP32[$155>>2]|0;
 $157 = (($153) - ($156))|0;
 $158 = (($152) + ($157<<2)|0);
 STACKTOP = sp;return ($158|0);
}
function _free_pool($cinfo,$pool_id) {
 $cinfo = $cinfo|0;
 $pool_id = $pool_id|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bptr = 0, $lhdr_ptr = 0, $mem = 0, $next_lhdr_ptr = 0, $next_shdr_ptr = 0, $shdr_ptr = 0, $space_freed = 0, $sptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $pool_id;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $mem = $4;
 $5 = $1;
 $6 = ($5|0)<(0);
 if ($6) {
  label = 3;
 } else {
  $7 = $1;
  $8 = ($7|0)>=(2);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = $0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 20|0);
  HEAP32[$11>>2] = 14;
  $12 = $1;
  $13 = $0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 24|0);
  HEAP32[$15>>2] = $12;
  $16 = $0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  FUNCTION_TABLE_vi[$18 & 255]($19);
 }
 $20 = $1;
 $21 = ($20|0)==(1);
 if ($21) {
  $22 = $mem;
  $23 = (($22) + 68|0);
  $24 = HEAP32[$23>>2]|0;
  $sptr = $24;
  while(1) {
   $25 = $sptr;
   $26 = ($25|0)!=(0|0);
   if (!($26)) {
    break;
   }
   $27 = $sptr;
   $28 = (($27) + 40|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = $sptr;
    $32 = (($31) + 40|0);
    HEAP32[$32>>2] = 0;
    $33 = $sptr;
    $34 = (($33) + 48|0);
    $35 = (($34) + 8|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = $0;
    $38 = $sptr;
    $39 = (($38) + 48|0);
    FUNCTION_TABLE_vii[$36 & 255]($37,$39);
   }
   $40 = $sptr;
   $41 = (($40) + 44|0);
   $42 = HEAP32[$41>>2]|0;
   $sptr = $42;
  }
  $43 = $mem;
  $44 = (($43) + 68|0);
  HEAP32[$44>>2] = 0;
  $45 = $mem;
  $46 = (($45) + 72|0);
  $47 = HEAP32[$46>>2]|0;
  $bptr = $47;
  while(1) {
   $48 = $bptr;
   $49 = ($48|0)!=(0|0);
   if (!($49)) {
    break;
   }
   $50 = $bptr;
   $51 = (($50) + 40|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)!=(0);
   if ($53) {
    $54 = $bptr;
    $55 = (($54) + 40|0);
    HEAP32[$55>>2] = 0;
    $56 = $bptr;
    $57 = (($56) + 48|0);
    $58 = (($57) + 8|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = $0;
    $61 = $bptr;
    $62 = (($61) + 48|0);
    FUNCTION_TABLE_vii[$59 & 255]($60,$62);
   }
   $63 = $bptr;
   $64 = (($63) + 44|0);
   $65 = HEAP32[$64>>2]|0;
   $bptr = $65;
  }
  $66 = $mem;
  $67 = (($66) + 72|0);
  HEAP32[$67>>2] = 0;
 }
 $68 = $1;
 $69 = $mem;
 $70 = (($69) + 60|0);
 $71 = (($70) + ($68<<2)|0);
 $72 = HEAP32[$71>>2]|0;
 $lhdr_ptr = $72;
 $73 = $1;
 $74 = $mem;
 $75 = (($74) + 60|0);
 $76 = (($75) + ($73<<2)|0);
 HEAP32[$76>>2] = 0;
 while(1) {
  $77 = $lhdr_ptr;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = $lhdr_ptr;
  $80 = HEAP32[$79>>2]|0;
  $next_lhdr_ptr = $80;
  $81 = $lhdr_ptr;
  $82 = (($81) + 4|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $lhdr_ptr;
  $85 = (($84) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($83) + ($86))|0;
  $88 = (($87) + 16)|0;
  $space_freed = $88;
  $89 = $0;
  $90 = $lhdr_ptr;
  $91 = $space_freed;
  _jpeg_free_large($89,$90,$91);
  $92 = $space_freed;
  $93 = $mem;
  $94 = (($93) + 76|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95) - ($92))|0;
  HEAP32[$94>>2] = $96;
  $97 = $next_lhdr_ptr;
  $lhdr_ptr = $97;
 }
 $98 = $1;
 $99 = $mem;
 $100 = (($99) + 52|0);
 $101 = (($100) + ($98<<2)|0);
 $102 = HEAP32[$101>>2]|0;
 $shdr_ptr = $102;
 $103 = $1;
 $104 = $mem;
 $105 = (($104) + 52|0);
 $106 = (($105) + ($103<<2)|0);
 HEAP32[$106>>2] = 0;
 while(1) {
  $107 = $shdr_ptr;
  $108 = ($107|0)!=(0|0);
  if (!($108)) {
   break;
  }
  $109 = $shdr_ptr;
  $110 = HEAP32[$109>>2]|0;
  $next_shdr_ptr = $110;
  $111 = $shdr_ptr;
  $112 = (($111) + 4|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = $shdr_ptr;
  $115 = (($114) + 8|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (($113) + ($116))|0;
  $118 = (($117) + 16)|0;
  $space_freed = $118;
  $119 = $0;
  $120 = $shdr_ptr;
  $121 = $space_freed;
  _jpeg_free_small($119,$120,$121);
  $122 = $space_freed;
  $123 = $mem;
  $124 = (($123) + 76|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = (($125) - ($122))|0;
  HEAP32[$124>>2] = $126;
  $127 = $next_shdr_ptr;
  $shdr_ptr = $127;
 }
 STACKTOP = sp;return;
}
function _self_destruct($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pool = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $pool = 1;
 while(1) {
  $1 = $pool;
  $2 = ($1|0)>=(0);
  if (!($2)) {
   break;
  }
  $3 = $0;
  $4 = $pool;
  _free_pool($3,$4);
  $5 = $pool;
  $6 = (($5) + -1)|0;
  $pool = $6;
 }
 $7 = $0;
 $8 = $0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 _jpeg_free_small($7,$10,84);
 $11 = $0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = $0;
 _jpeg_mem_term($13);
 STACKTOP = sp;return;
}
function _do_barray_io($cinfo,$ptr,$writing) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $writing = $writing|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $byte_count = 0, $bytesperrow = 0, $file_offset = 0, $i = 0, $rows = 0, $thisrow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $writing;
 $3 = $1;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<7;
 $bytesperrow = $6;
 $7 = $1;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $bytesperrow;
 $11 = Math_imul($9, $10)|0;
 $file_offset = $11;
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $1;
  $14 = (($13) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 19;
   break;
  }
  $17 = $1;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $i;
  $24 = (($22) - ($23))|0;
  $25 = ($19|0)<($24|0);
  if ($25) {
   $26 = $1;
   $27 = (($26) + 20|0);
   $28 = HEAP32[$27>>2]|0;
   $34 = $28;
  } else {
   $29 = $1;
   $30 = (($29) + 16|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $i;
   $33 = (($31) - ($32))|0;
   $34 = $33;
  }
  $rows = $34;
  $35 = $1;
  $36 = (($35) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $i;
  $39 = (($37) + ($38))|0;
  $thisrow = $39;
  $40 = $rows;
  $41 = $1;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $thisrow;
  $45 = (($43) - ($44))|0;
  $46 = ($40|0)<($45|0);
  if ($46) {
   $47 = $rows;
   $53 = $47;
  } else {
   $48 = $1;
   $49 = (($48) + 28|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $thisrow;
   $52 = (($50) - ($51))|0;
   $53 = $52;
  }
  $rows = $53;
  $54 = $rows;
  $55 = $1;
  $56 = (($55) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $thisrow;
  $59 = (($57) - ($58))|0;
  $60 = ($54|0)<($59|0);
  if ($60) {
   $61 = $rows;
   $67 = $61;
  } else {
   $62 = $1;
   $63 = (($62) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $thisrow;
   $66 = (($64) - ($65))|0;
   $67 = $66;
  }
  $rows = $67;
  $68 = $rows;
  $69 = ($68|0)<=(0);
  if ($69) {
   break;
  }
  $70 = $rows;
  $71 = $bytesperrow;
  $72 = Math_imul($70, $71)|0;
  $byte_count = $72;
  $73 = $2;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $1;
   $76 = (($75) + 48|0);
   $77 = (($76) + 4|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $0;
   $80 = $1;
   $81 = (($80) + 48|0);
   $82 = $i;
   $83 = $1;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + ($82<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $file_offset;
   $88 = $byte_count;
   FUNCTION_TABLE_viiiii[$78 & 127]($79,$81,$86,$87,$88);
  } else {
   $89 = $1;
   $90 = (($89) + 48|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = $0;
   $93 = $1;
   $94 = (($93) + 48|0);
   $95 = $i;
   $96 = $1;
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + ($95<<2)|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = $file_offset;
   $101 = $byte_count;
   FUNCTION_TABLE_viiiii[$91 & 127]($92,$94,$99,$100,$101);
  }
  $102 = $byte_count;
  $103 = $file_offset;
  $104 = (($103) + ($102))|0;
  $file_offset = $104;
  $105 = $1;
  $106 = (($105) + 20|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $i;
  $109 = (($108) + ($107))|0;
  $i = $109;
 }
 if ((label|0) == 19) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _do_sarray_io($cinfo,$ptr,$writing) {
 $cinfo = $cinfo|0;
 $ptr = $ptr|0;
 $writing = $writing|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $byte_count = 0, $bytesperrow = 0, $file_offset = 0, $i = 0, $rows = 0, $thisrow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $ptr;
 $2 = $writing;
 $3 = $1;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5;
 $bytesperrow = $6;
 $7 = $1;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $bytesperrow;
 $11 = Math_imul($9, $10)|0;
 $file_offset = $11;
 $i = 0;
 while(1) {
  $12 = $i;
  $13 = $1;
  $14 = (($13) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 19;
   break;
  }
  $17 = $1;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $i;
  $24 = (($22) - ($23))|0;
  $25 = ($19|0)<($24|0);
  if ($25) {
   $26 = $1;
   $27 = (($26) + 20|0);
   $28 = HEAP32[$27>>2]|0;
   $34 = $28;
  } else {
   $29 = $1;
   $30 = (($29) + 16|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $i;
   $33 = (($31) - ($32))|0;
   $34 = $33;
  }
  $rows = $34;
  $35 = $1;
  $36 = (($35) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $i;
  $39 = (($37) + ($38))|0;
  $thisrow = $39;
  $40 = $rows;
  $41 = $1;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $thisrow;
  $45 = (($43) - ($44))|0;
  $46 = ($40|0)<($45|0);
  if ($46) {
   $47 = $rows;
   $53 = $47;
  } else {
   $48 = $1;
   $49 = (($48) + 28|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $thisrow;
   $52 = (($50) - ($51))|0;
   $53 = $52;
  }
  $rows = $53;
  $54 = $rows;
  $55 = $1;
  $56 = (($55) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $thisrow;
  $59 = (($57) - ($58))|0;
  $60 = ($54|0)<($59|0);
  if ($60) {
   $61 = $rows;
   $67 = $61;
  } else {
   $62 = $1;
   $63 = (($62) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $thisrow;
   $66 = (($64) - ($65))|0;
   $67 = $66;
  }
  $rows = $67;
  $68 = $rows;
  $69 = ($68|0)<=(0);
  if ($69) {
   break;
  }
  $70 = $rows;
  $71 = $bytesperrow;
  $72 = Math_imul($70, $71)|0;
  $byte_count = $72;
  $73 = $2;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $1;
   $76 = (($75) + 48|0);
   $77 = (($76) + 4|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $0;
   $80 = $1;
   $81 = (($80) + 48|0);
   $82 = $i;
   $83 = $1;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + ($82<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $file_offset;
   $88 = $byte_count;
   FUNCTION_TABLE_viiiii[$78 & 127]($79,$81,$86,$87,$88);
  } else {
   $89 = $1;
   $90 = (($89) + 48|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = $0;
   $93 = $1;
   $94 = (($93) + 48|0);
   $95 = $i;
   $96 = $1;
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + ($95<<2)|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = $file_offset;
   $101 = $byte_count;
   FUNCTION_TABLE_viiiii[$91 & 127]($92,$94,$99,$100,$101);
  }
  $102 = $byte_count;
  $103 = $file_offset;
  $104 = (($103) + ($102))|0;
  $file_offset = $104;
  $105 = $1;
  $106 = (($105) + 20|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $i;
  $109 = (($108) + ($107))|0;
  $i = $109;
 }
 if ((label|0) == 19) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _out_of_memory($cinfo,$which) {
 $cinfo = $cinfo|0;
 $which = $which|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $which;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 20|0);
 HEAP32[$4>>2] = 54;
 $5 = $1;
 $6 = $0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 24|0);
 HEAP32[$8>>2] = $5;
 $9 = $0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = $0;
 FUNCTION_TABLE_vi[$11 & 255]($12);
 STACKTOP = sp;return;
}
function _jpeg_get_small($cinfo,$sizeofobject) {
 $cinfo = $cinfo|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $sizeofobject;
 $2 = $1;
 $3 = (_malloc($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _jpeg_free_small($cinfo,$object,$sizeofobject) {
 $cinfo = $cinfo|0;
 $object = $object|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $object;
 $2 = $sizeofobject;
 $3 = $1;
 _free($3);
 STACKTOP = sp;return;
}
function _jpeg_get_large($cinfo,$sizeofobject) {
 $cinfo = $cinfo|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $sizeofobject;
 $2 = $1;
 $3 = (_malloc($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _jpeg_free_large($cinfo,$object,$sizeofobject) {
 $cinfo = $cinfo|0;
 $object = $object|0;
 $sizeofobject = $sizeofobject|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $object;
 $2 = $sizeofobject;
 $3 = $1;
 _free($3);
 STACKTOP = sp;return;
}
function _jpeg_mem_available($cinfo,$min_bytes_needed,$max_bytes_needed,$already_allocated) {
 $cinfo = $cinfo|0;
 $min_bytes_needed = $min_bytes_needed|0;
 $max_bytes_needed = $max_bytes_needed|0;
 $already_allocated = $already_allocated|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $min_bytes_needed;
 $2 = $max_bytes_needed;
 $3 = $already_allocated;
 $4 = $2;
 STACKTOP = sp;return ($4|0);
}
function _jpeg_open_backing_store($cinfo,$info,$total_bytes_needed) {
 $cinfo = $cinfo|0;
 $info = $info|0;
 $total_bytes_needed = $total_bytes_needed|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $info;
 $2 = $total_bytes_needed;
 $3 = $0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 20|0);
 HEAP32[$5>>2] = 49;
 $6 = $0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 FUNCTION_TABLE_vi[$8 & 255]($9);
 STACKTOP = sp;return;
}
function _jpeg_mem_init($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return 0;
}
function _jpeg_mem_term($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _jinit_1pass_quantizer($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,88)|0);
 $cquantize = $6;
 $7 = $cquantize;
 $8 = $0;
 $9 = (($8) + 460|0);
 HEAP32[$9>>2] = $7;
 $10 = $cquantize;
 HEAP32[$10>>2] = 85;
 $11 = $cquantize;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = 86;
 $13 = $cquantize;
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = 87;
 $15 = $cquantize;
 $16 = (($15) + 68|0);
 HEAP32[$16>>2] = 0;
 $17 = $cquantize;
 $18 = (($17) + 52|0);
 HEAP32[$18>>2] = 0;
 $19 = $0;
 $20 = (($19) + 120|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)>(4);
 if ($22) {
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 55;
  $26 = $0;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 24|0);
  HEAP32[$28>>2] = 4;
  $29 = $0;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = $0;
  FUNCTION_TABLE_vi[$31 & 255]($32);
 }
 $33 = $0;
 $34 = (($33) + 96|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)>(256);
 if ($36) {
  $37 = $0;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 20|0);
  HEAP32[$39>>2] = 57;
  $40 = $0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 24|0);
  HEAP32[$42>>2] = 256;
  $43 = $0;
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$44>>2]|0;
  $46 = $0;
  FUNCTION_TABLE_vi[$45 & 255]($46);
 }
 $47 = $0;
 _create_colormap($47);
 $48 = $0;
 _create_colorindex($48);
 $49 = $0;
 $50 = (($49) + 88|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)==(2);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = $0;
 _alloc_fs_workspace($53);
 STACKTOP = sp;return;
}
function _start_pass_1_quant($cinfo,$is_pre_scan) {
 $cinfo = $cinfo|0;
 $is_pre_scan = $is_pre_scan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $arraysize = 0, $cquantize = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $is_pre_scan;
 $2 = $0;
 $3 = (($2) + 460|0);
 $4 = HEAP32[$3>>2]|0;
 $cquantize = $4;
 $5 = $cquantize;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (($8) + 136|0);
 HEAP32[$9>>2] = $7;
 $10 = $cquantize;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 132|0);
 HEAP32[$14>>2] = $12;
 $15 = $0;
 $16 = (($15) + 88|0);
 $17 = HEAP32[$16>>2]|0;
 if ((($17|0) == 0)) {
  $18 = $0;
  $19 = (($18) + 120|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(3);
  if ($21) {
   $22 = $cquantize;
   $23 = (($22) + 4|0);
   HEAP32[$23>>2] = 88;
  } else {
   $24 = $cquantize;
   $25 = (($24) + 4|0);
   HEAP32[$25>>2] = 89;
  }
  STACKTOP = sp;return;
 } else if ((($17|0) == 2)) {
  $46 = $cquantize;
  $47 = (($46) + 4|0);
  HEAP32[$47>>2] = 92;
  $48 = $cquantize;
  $49 = (($48) + 84|0);
  HEAP32[$49>>2] = 0;
  $50 = $cquantize;
  $51 = (($50) + 68|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($52|0)==(0|0);
  if ($53) {
   $54 = $0;
   _alloc_fs_workspace($54);
  }
  $55 = $0;
  $56 = (($55) + 112|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 2)|0;
  $59 = $58<<1;
  $arraysize = $59;
  $i = 0;
  while(1) {
   $60 = $i;
   $61 = $0;
   $62 = (($61) + 120|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($60|0)<($63|0);
   if (!($64)) {
    break;
   }
   $65 = $i;
   $66 = $cquantize;
   $67 = (($66) + 68|0);
   $68 = (($67) + ($65<<2)|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = $arraysize;
   _jzero_far($69,$70);
   $71 = $i;
   $72 = (($71) + 1)|0;
   $i = $72;
  }
  STACKTOP = sp;return;
 } else if ((($17|0) == 1)) {
  $26 = $0;
  $27 = (($26) + 120|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(3);
  if ($29) {
   $30 = $cquantize;
   $31 = (($30) + 4|0);
   HEAP32[$31>>2] = 90;
  } else {
   $32 = $cquantize;
   $33 = (($32) + 4|0);
   HEAP32[$33>>2] = 91;
  }
  $34 = $cquantize;
  $35 = (($34) + 48|0);
  HEAP32[$35>>2] = 0;
  $36 = $cquantize;
  $37 = (($36) + 28|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(0);
  if (!($39)) {
   $40 = $0;
   _create_colorindex($40);
  }
  $41 = $cquantize;
  $42 = (($41) + 52|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($43|0)==(0|0);
  if ($44) {
   $45 = $0;
   _create_odither_tables($45);
  }
  STACKTOP = sp;return;
 } else {
  $73 = $0;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + 20|0);
  HEAP32[$75>>2] = 48;
  $76 = $0;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$77>>2]|0;
  $79 = $0;
  FUNCTION_TABLE_vi[$78 & 255]($79);
  STACKTOP = sp;return;
 }
}
function _finish_pass_1_quant($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _new_color_map_1_quant($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 HEAP32[$3>>2] = 46;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 FUNCTION_TABLE_vi[$6 & 255]($7);
 STACKTOP = sp;return;
}
function _create_colormap($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $_mp = 0, $blkdist = 0, $blksize = 0, $colormap = 0, $cquantize = 0, $i = 0, $j = 0, $k = 0, $nci = 0, $ptr = 0, $total_colors = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = $cquantize;
 $6 = (($5) + 32|0);
 $7 = (_select_ncolors($4,$6)|0);
 $total_colors = $7;
 $8 = $0;
 $9 = (($8) + 120|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(3);
 if ($11) {
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 24|0);
  $_mp = $14;
  $15 = $total_colors;
  $16 = $_mp;
  HEAP32[$16>>2] = $15;
  $17 = $cquantize;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $_mp;
  $21 = (($20) + 4|0);
  HEAP32[$21>>2] = $19;
  $22 = $cquantize;
  $23 = (($22) + 32|0);
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $_mp;
  $27 = (($26) + 8|0);
  HEAP32[$27>>2] = $25;
  $28 = $cquantize;
  $29 = (($28) + 32|0);
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $_mp;
  $33 = (($32) + 12|0);
  HEAP32[$33>>2] = $31;
  $34 = $0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 20|0);
  HEAP32[$36>>2] = 94;
  $37 = $0;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $0;
  FUNCTION_TABLE_vii[$40 & 255]($41,1);
 } else {
  $42 = $0;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 20|0);
  HEAP32[$44>>2] = 95;
  $45 = $total_colors;
  $46 = $0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 24|0);
  HEAP32[$48>>2] = $45;
  $49 = $0;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $0;
  FUNCTION_TABLE_vii[$52 & 255]($53,1);
 }
 $54 = $0;
 $55 = (($54) + 4|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = (($56) + 8|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $0;
 $60 = $total_colors;
 $61 = $0;
 $62 = (($61) + 120|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = (FUNCTION_TABLE_iiiii[$58 & 255]($59,1,$60,$63)|0);
 $colormap = $64;
 $65 = $total_colors;
 $blkdist = $65;
 $i = 0;
 while(1) {
  $66 = $i;
  $67 = $0;
  $68 = (($67) + 120|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = ($66|0)<($69|0);
  if (!($70)) {
   break;
  }
  $71 = $i;
  $72 = $cquantize;
  $73 = (($72) + 32|0);
  $74 = (($73) + ($71<<2)|0);
  $75 = HEAP32[$74>>2]|0;
  $nci = $75;
  $76 = $blkdist;
  $77 = $nci;
  $78 = (($76|0) / ($77|0))&-1;
  $blksize = $78;
  $j = 0;
  while(1) {
   $79 = $j;
   $80 = $nci;
   $81 = ($79|0)<($80|0);
   if (!($81)) {
    break;
   }
   $82 = $0;
   $83 = $i;
   $84 = $j;
   $85 = $nci;
   $86 = (($85) - 1)|0;
   $87 = (_output_value($82,$83,$84,$86)|0);
   $val = $87;
   $88 = $j;
   $89 = $blksize;
   $90 = Math_imul($88, $89)|0;
   $ptr = $90;
   while(1) {
    $91 = $ptr;
    $92 = $total_colors;
    $93 = ($91|0)<($92|0);
    if (!($93)) {
     break;
    }
    $k = 0;
    while(1) {
     $94 = $k;
     $95 = $blksize;
     $96 = ($94|0)<($95|0);
     if (!($96)) {
      break;
     }
     $97 = $val;
     $98 = $97&255;
     $99 = $ptr;
     $100 = $k;
     $101 = (($99) + ($100))|0;
     $102 = $i;
     $103 = $colormap;
     $104 = (($103) + ($102<<2)|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (($105) + ($101)|0);
     HEAP8[$106>>0] = $98;
     $107 = $k;
     $108 = (($107) + 1)|0;
     $k = $108;
    }
    $109 = $blkdist;
    $110 = $ptr;
    $111 = (($110) + ($109))|0;
    $ptr = $111;
   }
   $112 = $j;
   $113 = (($112) + 1)|0;
   $j = $113;
  }
  $114 = $blksize;
  $blkdist = $114;
  $115 = $i;
  $116 = (($115) + 1)|0;
  $i = $116;
 }
 $117 = $colormap;
 $118 = $cquantize;
 $119 = (($118) + 16|0);
 HEAP32[$119>>2] = $117;
 $120 = $total_colors;
 $121 = $cquantize;
 $122 = (($121) + 20|0);
 HEAP32[$122>>2] = $120;
 STACKTOP = sp;return;
}
function _create_colorindex($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $blksize = 0, $cquantize = 0, $i = 0, $indexptr = 0, $j = 0, $k = 0, $nci = 0, $pad = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = (($4) + 88|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(1);
 if ($7) {
  $pad = 510;
  $8 = $cquantize;
  $9 = (($8) + 28|0);
  HEAP32[$9>>2] = 1;
 } else {
  $pad = 0;
  $10 = $cquantize;
  $11 = (($10) + 28|0);
  HEAP32[$11>>2] = 0;
 }
 $12 = $0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = $pad;
 $19 = (256 + ($18))|0;
 $20 = $0;
 $21 = (($20) + 120|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiii[$16 & 255]($17,1,$19,$22)|0);
 $24 = $cquantize;
 $25 = (($24) + 24|0);
 HEAP32[$25>>2] = $23;
 $26 = $cquantize;
 $27 = (($26) + 20|0);
 $28 = HEAP32[$27>>2]|0;
 $blksize = $28;
 $i = 0;
 while(1) {
  $29 = $i;
  $30 = $0;
  $31 = (($30) + 120|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($29|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = $i;
  $35 = $cquantize;
  $36 = (($35) + 32|0);
  $37 = (($36) + ($34<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $nci = $38;
  $39 = $blksize;
  $40 = $nci;
  $41 = (($39|0) / ($40|0))&-1;
  $blksize = $41;
  $42 = $pad;
  $43 = ($42|0)!=(0);
  if ($43) {
   $44 = $i;
   $45 = $cquantize;
   $46 = (($45) + 24|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($44<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + 255|0);
   HEAP32[$48>>2] = $50;
  }
  $51 = $i;
  $52 = $cquantize;
  $53 = (($52) + 24|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $indexptr = $56;
  $val = 0;
  $57 = $0;
  $58 = $i;
  $59 = $nci;
  $60 = (($59) - 1)|0;
  $61 = (_largest_input_value($57,$58,0,$60)|0);
  $k = $61;
  $j = 0;
  while(1) {
   $62 = $j;
   $63 = ($62|0)<=(255);
   if (!($63)) {
    break;
   }
   while(1) {
    $64 = $j;
    $65 = $k;
    $66 = ($64|0)>($65|0);
    if (!($66)) {
     break;
    }
    $67 = $0;
    $68 = $i;
    $69 = $val;
    $70 = (($69) + 1)|0;
    $val = $70;
    $71 = $nci;
    $72 = (($71) - 1)|0;
    $73 = (_largest_input_value($67,$68,$70,$72)|0);
    $k = $73;
   }
   $74 = $val;
   $75 = $blksize;
   $76 = Math_imul($74, $75)|0;
   $77 = $76&255;
   $78 = $j;
   $79 = $indexptr;
   $80 = (($79) + ($78)|0);
   HEAP8[$80>>0] = $77;
   $81 = $j;
   $82 = (($81) + 1)|0;
   $j = $82;
  }
  $83 = $pad;
  $84 = ($83|0)!=(0);
  if ($84) {
   $j = 1;
   while(1) {
    $85 = $j;
    $86 = ($85|0)<=(255);
    if (!($86)) {
     break;
    }
    $87 = $indexptr;
    $88 = HEAP8[$87>>0]|0;
    $89 = $j;
    $90 = (0 - ($89))|0;
    $91 = $indexptr;
    $92 = (($91) + ($90)|0);
    HEAP8[$92>>0] = $88;
    $93 = $indexptr;
    $94 = (($93) + 255|0);
    $95 = HEAP8[$94>>0]|0;
    $96 = $j;
    $97 = (255 + ($96))|0;
    $98 = $indexptr;
    $99 = (($98) + ($97)|0);
    HEAP8[$99>>0] = $95;
    $100 = $j;
    $101 = (($100) + 1)|0;
    $j = $101;
   }
  }
  $102 = $i;
  $103 = (($102) + 1)|0;
  $i = $103;
 }
 STACKTOP = sp;return;
}
function _alloc_fs_workspace($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $arraysize = 0, $cquantize = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = (($4) + 112|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 2)|0;
 $8 = $7<<1;
 $arraysize = $8;
 $i = 0;
 while(1) {
  $9 = $i;
  $10 = $0;
  $11 = (($10) + 120|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $0;
  $20 = $arraysize;
  $21 = (FUNCTION_TABLE_iiii[$18 & 255]($19,1,$20)|0);
  $22 = $i;
  $23 = $cquantize;
  $24 = (($23) + 68|0);
  $25 = (($24) + ($22<<2)|0);
  HEAP32[$25>>2] = $21;
  $26 = $i;
  $27 = (($26) + 1)|0;
  $i = $27;
 }
 STACKTOP = sp;return;
}
function _largest_input_value($cinfo,$ci,$j,$maxj) {
 $cinfo = $cinfo|0;
 $ci = $ci|0;
 $j = $j|0;
 $maxj = $maxj|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $ci;
 $2 = $j;
 $3 = $maxj;
 $4 = $2;
 $5 = $4<<1;
 $6 = (($5) + 1)|0;
 $7 = ($6*255)|0;
 $8 = $3;
 $9 = (($7) + ($8))|0;
 $10 = $3;
 $11 = $10<<1;
 $12 = (($9|0) / ($11|0))&-1;
 STACKTOP = sp;return ($12|0);
}
function _select_ncolors($cinfo,$Ncolors) {
 $cinfo = $cinfo|0;
 $Ncolors = $Ncolors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $changed = 0, $i = 0, $iroot = 0, $j = 0, $max_colors = 0, $nc = 0, $temp = 0, $total_colors = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $Ncolors;
 $2 = $0;
 $3 = (($2) + 120|0);
 $4 = HEAP32[$3>>2]|0;
 $nc = $4;
 $5 = $0;
 $6 = (($5) + 96|0);
 $7 = HEAP32[$6>>2]|0;
 $max_colors = $7;
 $iroot = 1;
 while(1) {
  $8 = $iroot;
  $9 = (($8) + 1)|0;
  $iroot = $9;
  $10 = $iroot;
  $temp = $10;
  $i = 1;
  while(1) {
   $11 = $i;
   $12 = $nc;
   $13 = ($11|0)<($12|0);
   if (!($13)) {
    break;
   }
   $14 = $iroot;
   $15 = $temp;
   $16 = Math_imul($15, $14)|0;
   $temp = $16;
   $17 = $i;
   $18 = (($17) + 1)|0;
   $i = $18;
  }
  $19 = $temp;
  $20 = $max_colors;
  $21 = ($19|0)<=($20|0);
  if (!($21)) {
   break;
  }
 }
 $22 = $iroot;
 $23 = (($22) + -1)|0;
 $iroot = $23;
 $24 = $iroot;
 $25 = ($24|0)<(2);
 if ($25) {
  $26 = $0;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 20|0);
  HEAP32[$28>>2] = 56;
  $29 = $temp;
  $30 = $0;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 24|0);
  HEAP32[$32>>2] = $29;
  $33 = $0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = $0;
  FUNCTION_TABLE_vi[$35 & 255]($36);
 }
 $total_colors = 1;
 $i = 0;
 while(1) {
  $37 = $i;
  $38 = $nc;
  $39 = ($37|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = $iroot;
  $41 = $i;
  $42 = $1;
  $43 = (($42) + ($41<<2)|0);
  HEAP32[$43>>2] = $40;
  $44 = $iroot;
  $45 = $total_colors;
  $46 = Math_imul($45, $44)|0;
  $total_colors = $46;
  $47 = $i;
  $48 = (($47) + 1)|0;
  $i = $48;
 }
 while(1) {
  $changed = 0;
  $i = 0;
  while(1) {
   $49 = $i;
   $50 = $nc;
   $51 = ($49|0)<($50|0);
   if (!($51)) {
    break;
   }
   $52 = $0;
   $53 = (($52) + 44|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = ($54|0)==(2);
   if ($55) {
    $56 = $i;
    $57 = (50232 + ($56<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $60 = $58;
   } else {
    $59 = $i;
    $60 = $59;
   }
   $j = $60;
   $61 = $total_colors;
   $62 = $j;
   $63 = $1;
   $64 = (($63) + ($62<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($61|0) / ($65|0))&-1;
   $temp = $66;
   $67 = $j;
   $68 = $1;
   $69 = (($68) + ($67<<2)|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + 1)|0;
   $72 = $temp;
   $73 = Math_imul($72, $71)|0;
   $temp = $73;
   $74 = $temp;
   $75 = $max_colors;
   $76 = ($74|0)>($75|0);
   if ($76) {
    label = 21;
    break;
   }
   $77 = $j;
   $78 = $1;
   $79 = (($78) + ($77<<2)|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = (($80) + 1)|0;
   HEAP32[$79>>2] = $81;
   $82 = $temp;
   $total_colors = $82;
   $changed = 1;
   $83 = $i;
   $84 = (($83) + 1)|0;
   $i = $84;
  }
  if ((label|0) == 21) {
   label = 0;
  }
  $85 = $changed;
  $86 = ($85|0)!=(0);
  if (!($86)) {
   break;
  }
 }
 $87 = $total_colors;
 STACKTOP = sp;return ($87|0);
}
function _output_value($cinfo,$ci,$j,$maxj) {
 $cinfo = $cinfo|0;
 $ci = $ci|0;
 $j = $j|0;
 $maxj = $maxj|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $ci;
 $2 = $j;
 $3 = $maxj;
 $4 = $2;
 $5 = ($4*255)|0;
 $6 = $3;
 $7 = (($6|0) / 2)&-1;
 $8 = (($5) + ($7))|0;
 $9 = $3;
 $10 = (($8|0) / ($9|0))&-1;
 STACKTOP = sp;return ($10|0);
}
function _color_quantize3($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $col = 0, $colorindex0 = 0, $colorindex1 = 0, $colorindex2 = 0, $cquantize = 0, $pixcode = 0;
 var $ptrin = 0, $ptrout = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $colorindex0 = $10;
 $11 = $cquantize;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $colorindex1 = $15;
 $16 = $cquantize;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $colorindex2 = $20;
 $21 = $0;
 $22 = (($21) + 112|0);
 $23 = HEAP32[$22>>2]|0;
 $width = $23;
 $row = 0;
 while(1) {
  $24 = $row;
  $25 = $3;
  $26 = ($24|0)<($25|0);
  if (!($26)) {
   break;
  }
  $27 = $row;
  $28 = $1;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $ptrin = $30;
  $31 = $row;
  $32 = $2;
  $33 = (($32) + ($31<<2)|0);
  $34 = HEAP32[$33>>2]|0;
  $ptrout = $34;
  $35 = $width;
  $col = $35;
  while(1) {
   $36 = $col;
   $37 = ($36>>>0)>(0);
   if (!($37)) {
    break;
   }
   $38 = $ptrin;
   $39 = (($38) + 1|0);
   $ptrin = $39;
   $40 = HEAP8[$38>>0]|0;
   $41 = $40&255;
   $42 = $colorindex0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $pixcode = $45;
   $46 = $ptrin;
   $47 = (($46) + 1|0);
   $ptrin = $47;
   $48 = HEAP8[$46>>0]|0;
   $49 = $48&255;
   $50 = $colorindex1;
   $51 = (($50) + ($49)|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = $pixcode;
   $55 = (($54) + ($53))|0;
   $pixcode = $55;
   $56 = $ptrin;
   $57 = (($56) + 1|0);
   $ptrin = $57;
   $58 = HEAP8[$56>>0]|0;
   $59 = $58&255;
   $60 = $colorindex2;
   $61 = (($60) + ($59)|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = $pixcode;
   $65 = (($64) + ($63))|0;
   $pixcode = $65;
   $66 = $pixcode;
   $67 = $66&255;
   $68 = $ptrout;
   $69 = (($68) + 1|0);
   $ptrout = $69;
   HEAP8[$68>>0] = $67;
   $70 = $col;
   $71 = (($70) + -1)|0;
   $col = $71;
  }
  $72 = $row;
  $73 = (($72) + 1)|0;
  $row = $73;
 }
 STACKTOP = sp;return;
}
function _color_quantize($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $col = 0, $colorindex = 0, $cquantize = 0;
 var $nc = 0, $pixcode = 0, $ptrin = 0, $ptrout = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $colorindex = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $13 = $0;
 $14 = (($13) + 120|0);
 $15 = HEAP32[$14>>2]|0;
 $nc = $15;
 $row = 0;
 while(1) {
  $16 = $row;
  $17 = $3;
  $18 = ($16|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $row;
  $20 = $1;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $ptrin = $22;
  $23 = $row;
  $24 = $2;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $ptrout = $26;
  $27 = $width;
  $col = $27;
  while(1) {
   $28 = $col;
   $29 = ($28>>>0)>(0);
   if (!($29)) {
    break;
   }
   $pixcode = 0;
   $ci = 0;
   while(1) {
    $30 = $ci;
    $31 = $nc;
    $32 = ($30|0)<($31|0);
    if (!($32)) {
     break;
    }
    $33 = $ptrin;
    $34 = (($33) + 1|0);
    $ptrin = $34;
    $35 = HEAP8[$33>>0]|0;
    $36 = $35&255;
    $37 = $ci;
    $38 = $colorindex;
    $39 = (($38) + ($37<<2)|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + ($36)|0);
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $pixcode;
    $45 = (($44) + ($43))|0;
    $pixcode = $45;
    $46 = $ci;
    $47 = (($46) + 1)|0;
    $ci = $47;
   }
   $48 = $pixcode;
   $49 = $48&255;
   $50 = $ptrout;
   $51 = (($50) + 1|0);
   $ptrout = $51;
   HEAP8[$50>>0] = $49;
   $52 = $col;
   $53 = (($52) + -1)|0;
   $col = $53;
  }
  $54 = $row;
  $55 = (($54) + 1)|0;
  $row = $55;
 }
 STACKTOP = sp;return;
}
function _quantize3_ord_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $col = 0, $col_index = 0;
 var $colorindex0 = 0, $colorindex1 = 0, $colorindex2 = 0, $cquantize = 0, $dither0 = 0, $dither1 = 0, $dither2 = 0, $input_ptr = 0, $output_ptr = 0, $pixcode = 0, $row = 0, $row_index = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $colorindex0 = $10;
 $11 = $cquantize;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $colorindex1 = $15;
 $16 = $cquantize;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $colorindex2 = $20;
 $21 = $0;
 $22 = (($21) + 112|0);
 $23 = HEAP32[$22>>2]|0;
 $width = $23;
 $row = 0;
 while(1) {
  $24 = $row;
  $25 = $3;
  $26 = ($24|0)<($25|0);
  if (!($26)) {
   break;
  }
  $27 = $cquantize;
  $28 = (($27) + 48|0);
  $29 = HEAP32[$28>>2]|0;
  $row_index = $29;
  $30 = $row;
  $31 = $1;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $input_ptr = $33;
  $34 = $row;
  $35 = $2;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $output_ptr = $37;
  $38 = $row_index;
  $39 = $cquantize;
  $40 = (($39) + 52|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($38<<6)|0);
  $dither0 = $42;
  $43 = $row_index;
  $44 = $cquantize;
  $45 = (($44) + 52|0);
  $46 = (($45) + 4|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + ($43<<6)|0);
  $dither1 = $48;
  $49 = $row_index;
  $50 = $cquantize;
  $51 = (($50) + 52|0);
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($49<<6)|0);
  $dither2 = $54;
  $col_index = 0;
  $55 = $width;
  $col = $55;
  while(1) {
   $56 = $col;
   $57 = ($56>>>0)>(0);
   if (!($57)) {
    break;
   }
   $58 = $input_ptr;
   $59 = (($58) + 1|0);
   $input_ptr = $59;
   $60 = HEAP8[$58>>0]|0;
   $61 = $60&255;
   $62 = $col_index;
   $63 = $dither0;
   $64 = (($63) + ($62<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($61) + ($65))|0;
   $67 = $colorindex0;
   $68 = (($67) + ($66)|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = $69&255;
   $pixcode = $70;
   $71 = $input_ptr;
   $72 = (($71) + 1|0);
   $input_ptr = $72;
   $73 = HEAP8[$71>>0]|0;
   $74 = $73&255;
   $75 = $col_index;
   $76 = $dither1;
   $77 = (($76) + ($75<<2)|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = (($74) + ($78))|0;
   $80 = $colorindex1;
   $81 = (($80) + ($79)|0);
   $82 = HEAP8[$81>>0]|0;
   $83 = $82&255;
   $84 = $pixcode;
   $85 = (($84) + ($83))|0;
   $pixcode = $85;
   $86 = $input_ptr;
   $87 = (($86) + 1|0);
   $input_ptr = $87;
   $88 = HEAP8[$86>>0]|0;
   $89 = $88&255;
   $90 = $col_index;
   $91 = $dither2;
   $92 = (($91) + ($90<<2)|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (($89) + ($93))|0;
   $95 = $colorindex2;
   $96 = (($95) + ($94)|0);
   $97 = HEAP8[$96>>0]|0;
   $98 = $97&255;
   $99 = $pixcode;
   $100 = (($99) + ($98))|0;
   $pixcode = $100;
   $101 = $pixcode;
   $102 = $101&255;
   $103 = $output_ptr;
   $104 = (($103) + 1|0);
   $output_ptr = $104;
   HEAP8[$103>>0] = $102;
   $105 = $col_index;
   $106 = (($105) + 1)|0;
   $107 = $106 & 15;
   $col_index = $107;
   $108 = $col;
   $109 = (($108) + -1)|0;
   $col = $109;
  }
  $110 = $row_index;
  $111 = (($110) + 1)|0;
  $112 = $111 & 15;
  $row_index = $112;
  $113 = $row_index;
  $114 = $cquantize;
  $115 = (($114) + 48|0);
  HEAP32[$115>>2] = $113;
  $116 = $row;
  $117 = (($116) + 1)|0;
  $row = $117;
 }
 STACKTOP = sp;return;
}
function _quantize_ord_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $ci = 0, $col = 0, $col_index = 0, $colorindex_ci = 0, $cquantize = 0, $dither = 0, $input_ptr = 0, $nc = 0, $output_ptr = 0;
 var $row = 0, $row_index = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $0;
 $8 = (($7) + 120|0);
 $9 = HEAP32[$8>>2]|0;
 $nc = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $row = 0;
 while(1) {
  $13 = $row;
  $14 = $3;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $row;
  $17 = $2;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $width;
  $21 = $20;
  _jzero_far($19,$21);
  $22 = $cquantize;
  $23 = (($22) + 48|0);
  $24 = HEAP32[$23>>2]|0;
  $row_index = $24;
  $ci = 0;
  while(1) {
   $25 = $ci;
   $26 = $nc;
   $27 = ($25|0)<($26|0);
   if (!($27)) {
    break;
   }
   $28 = $row;
   $29 = $1;
   $30 = (($29) + ($28<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $ci;
   $33 = (($31) + ($32)|0);
   $input_ptr = $33;
   $34 = $row;
   $35 = $2;
   $36 = (($35) + ($34<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   $output_ptr = $37;
   $38 = $ci;
   $39 = $cquantize;
   $40 = (($39) + 24|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + ($38<<2)|0);
   $43 = HEAP32[$42>>2]|0;
   $colorindex_ci = $43;
   $44 = $row_index;
   $45 = $ci;
   $46 = $cquantize;
   $47 = (($46) + 52|0);
   $48 = (($47) + ($45<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + ($44<<6)|0);
   $dither = $50;
   $col_index = 0;
   $51 = $width;
   $col = $51;
   while(1) {
    $52 = $col;
    $53 = ($52>>>0)>(0);
    if (!($53)) {
     break;
    }
    $54 = $input_ptr;
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = $col_index;
    $58 = $dither;
    $59 = (($58) + ($57<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = (($56) + ($60))|0;
    $62 = $colorindex_ci;
    $63 = (($62) + ($61)|0);
    $64 = HEAP8[$63>>0]|0;
    $65 = $64&255;
    $66 = $output_ptr;
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = (($68) + ($65))|0;
    $70 = $69&255;
    HEAP8[$66>>0] = $70;
    $71 = $nc;
    $72 = $input_ptr;
    $73 = (($72) + ($71)|0);
    $input_ptr = $73;
    $74 = $output_ptr;
    $75 = (($74) + 1|0);
    $output_ptr = $75;
    $76 = $col_index;
    $77 = (($76) + 1)|0;
    $78 = $77 & 15;
    $col_index = $78;
    $79 = $col;
    $80 = (($79) + -1)|0;
    $col = $80;
   }
   $81 = $ci;
   $82 = (($81) + 1)|0;
   $ci = $82;
  }
  $83 = $row_index;
  $84 = (($83) + 1)|0;
  $85 = $84 & 15;
  $row_index = $85;
  $86 = $row_index;
  $87 = $cquantize;
  $88 = (($87) + 48|0);
  HEAP32[$88>>2] = $86;
  $89 = $row;
  $90 = (($89) + 1)|0;
  $row = $90;
 }
 STACKTOP = sp;return;
}
function _create_odither_tables($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, $i = 0, $j = 0, $nci = 0, $odither = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $0;
  $6 = (($5) + 120|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $i;
  $10 = $cquantize;
  $11 = (($10) + 32|0);
  $12 = (($11) + ($9<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $nci = $13;
  $odither = 0;
  $j = 0;
  while(1) {
   $14 = $j;
   $15 = $i;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $nci;
   $18 = $j;
   $19 = $cquantize;
   $20 = (($19) + 32|0);
   $21 = (($20) + ($18<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($17|0)==($22|0);
   if ($23) {
    label = 6;
    break;
   }
   $29 = $j;
   $30 = (($29) + 1)|0;
   $j = $30;
  }
  if ((label|0) == 6) {
   label = 0;
   $24 = $j;
   $25 = $cquantize;
   $26 = (($25) + 52|0);
   $27 = (($26) + ($24<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $odither = $28;
  }
  $31 = $odither;
  $32 = ($31|0)==(0|0);
  if ($32) {
   $33 = $0;
   $34 = $nci;
   $35 = (_make_odither_array($33,$34)|0);
   $odither = $35;
  }
  $36 = $odither;
  $37 = $i;
  $38 = $cquantize;
  $39 = (($38) + 52|0);
  $40 = (($39) + ($37<<2)|0);
  HEAP32[$40>>2] = $36;
  $41 = $i;
  $42 = (($41) + 1)|0;
  $i = $42;
 }
 STACKTOP = sp;return;
}
function _quantize_fs_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $belowerr = 0, $bnexterr = 0, $bpreverr = 0, $ci = 0, $col = 0, $colorindex_ci = 0, $colormap_ci = 0, $cquantize = 0, $cur = 0, $delta = 0, $dir = 0, $dirnc = 0;
 var $errorptr = 0, $input_ptr = 0, $nc = 0, $output_ptr = 0, $pixcode = 0, $range_limit = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $0;
 $8 = (($7) + 120|0);
 $9 = HEAP32[$8>>2]|0;
 $nc = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $13 = $0;
 $14 = (($13) + 324|0);
 $15 = HEAP32[$14>>2]|0;
 $range_limit = $15;
 $row = 0;
 while(1) {
  $16 = $row;
  $17 = $3;
  $18 = ($16|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $row;
  $20 = $2;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $width;
  $24 = $23;
  _jzero_far($22,$24);
  $ci = 0;
  while(1) {
   $25 = $ci;
   $26 = $nc;
   $27 = ($25|0)<($26|0);
   if (!($27)) {
    break;
   }
   $28 = $row;
   $29 = $1;
   $30 = (($29) + ($28<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $ci;
   $33 = (($31) + ($32)|0);
   $input_ptr = $33;
   $34 = $row;
   $35 = $2;
   $36 = (($35) + ($34<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   $output_ptr = $37;
   $38 = $cquantize;
   $39 = (($38) + 84|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = $width;
    $43 = (($42) - 1)|0;
    $44 = $nc;
    $45 = Math_imul($43, $44)|0;
    $46 = $input_ptr;
    $47 = (($46) + ($45)|0);
    $input_ptr = $47;
    $48 = $width;
    $49 = (($48) - 1)|0;
    $50 = $output_ptr;
    $51 = (($50) + ($49)|0);
    $output_ptr = $51;
    $dir = -1;
    $52 = $nc;
    $53 = (0 - ($52))|0;
    $dirnc = $53;
    $54 = $ci;
    $55 = $cquantize;
    $56 = (($55) + 68|0);
    $57 = (($56) + ($54<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $width;
    $60 = (($59) + 1)|0;
    $61 = (($58) + ($60<<1)|0);
    $errorptr = $61;
   } else {
    $dir = 1;
    $62 = $nc;
    $dirnc = $62;
    $63 = $ci;
    $64 = $cquantize;
    $65 = (($64) + 68|0);
    $66 = (($65) + ($63<<2)|0);
    $67 = HEAP32[$66>>2]|0;
    $errorptr = $67;
   }
   $68 = $ci;
   $69 = $cquantize;
   $70 = (($69) + 24|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + ($68<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $colorindex_ci = $73;
   $74 = $ci;
   $75 = $cquantize;
   $76 = (($75) + 16|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (($77) + ($74<<2)|0);
   $79 = HEAP32[$78>>2]|0;
   $colormap_ci = $79;
   $cur = 0;
   $bpreverr = 0;
   $belowerr = 0;
   $80 = $width;
   $col = $80;
   while(1) {
    $81 = $col;
    $82 = ($81>>>0)>(0);
    if (!($82)) {
     break;
    }
    $83 = $cur;
    $84 = $dir;
    $85 = $errorptr;
    $86 = (($85) + ($84<<1)|0);
    $87 = HEAP16[$86>>1]|0;
    $88 = $87 << 16 >> 16;
    $89 = (($83) + ($88))|0;
    $90 = (($89) + 8)|0;
    $91 = $90 >> 4;
    $cur = $91;
    $92 = $input_ptr;
    $93 = HEAP8[$92>>0]|0;
    $94 = $93&255;
    $95 = $cur;
    $96 = (($95) + ($94))|0;
    $cur = $96;
    $97 = $cur;
    $98 = $range_limit;
    $99 = (($98) + ($97)|0);
    $100 = HEAP8[$99>>0]|0;
    $101 = $100&255;
    $cur = $101;
    $102 = $cur;
    $103 = $colorindex_ci;
    $104 = (($103) + ($102)|0);
    $105 = HEAP8[$104>>0]|0;
    $106 = $105&255;
    $pixcode = $106;
    $107 = $pixcode;
    $108 = $107&255;
    $109 = $108&255;
    $110 = $output_ptr;
    $111 = HEAP8[$110>>0]|0;
    $112 = $111&255;
    $113 = (($112) + ($109))|0;
    $114 = $113&255;
    HEAP8[$110>>0] = $114;
    $115 = $pixcode;
    $116 = $colormap_ci;
    $117 = (($116) + ($115)|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = $118&255;
    $120 = $cur;
    $121 = (($120) - ($119))|0;
    $cur = $121;
    $122 = $cur;
    $bnexterr = $122;
    $123 = $cur;
    $124 = $123<<1;
    $delta = $124;
    $125 = $delta;
    $126 = $cur;
    $127 = (($126) + ($125))|0;
    $cur = $127;
    $128 = $bpreverr;
    $129 = $cur;
    $130 = (($128) + ($129))|0;
    $131 = $130&65535;
    $132 = $errorptr;
    HEAP16[$132>>1] = $131;
    $133 = $delta;
    $134 = $cur;
    $135 = (($134) + ($133))|0;
    $cur = $135;
    $136 = $belowerr;
    $137 = $cur;
    $138 = (($136) + ($137))|0;
    $bpreverr = $138;
    $139 = $bnexterr;
    $belowerr = $139;
    $140 = $delta;
    $141 = $cur;
    $142 = (($141) + ($140))|0;
    $cur = $142;
    $143 = $dirnc;
    $144 = $input_ptr;
    $145 = (($144) + ($143)|0);
    $input_ptr = $145;
    $146 = $dir;
    $147 = $output_ptr;
    $148 = (($147) + ($146)|0);
    $output_ptr = $148;
    $149 = $dir;
    $150 = $errorptr;
    $151 = (($150) + ($149<<1)|0);
    $errorptr = $151;
    $152 = $col;
    $153 = (($152) + -1)|0;
    $col = $153;
   }
   $154 = $bpreverr;
   $155 = $154&65535;
   $156 = $errorptr;
   HEAP16[$156>>1] = $155;
   $157 = $ci;
   $158 = (($157) + 1)|0;
   $ci = $158;
  }
  $159 = $cquantize;
  $160 = (($159) + 84|0);
  $161 = HEAP32[$160>>2]|0;
  $162 = ($161|0)!=(0);
  $163 = $162 ? 0 : 1;
  $164 = $cquantize;
  $165 = (($164) + 84|0);
  HEAP32[$165>>2] = $163;
  $166 = $row;
  $167 = (($166) + 1)|0;
  $row = $167;
 }
 STACKTOP = sp;return;
}
function _make_odither_array($cinfo,$ncolors) {
 $cinfo = $cinfo|0;
 $ncolors = $ncolors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $den = 0, $j = 0, $k = 0, $num = 0, $odither = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $ncolors;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 255]($6,1,1024)|0);
 $odither = $7;
 $8 = $1;
 $9 = (($8) - 1)|0;
 $10 = $9<<9;
 $den = $10;
 $j = 0;
 while(1) {
  $11 = $j;
  $12 = ($11|0)<(16);
  if (!($12)) {
   break;
  }
  $k = 0;
  while(1) {
   $13 = $k;
   $14 = ($13|0)<(16);
   if (!($14)) {
    break;
   }
   $15 = $k;
   $16 = $j;
   $17 = (50248 + ($16<<4)|0);
   $18 = (($17) + ($15)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = $20<<1;
   $22 = (255 - ($21))|0;
   $23 = ($22*255)|0;
   $num = $23;
   $24 = $num;
   $25 = ($24|0)<(0);
   if ($25) {
    $26 = $num;
    $27 = (0 - ($26))|0;
    $28 = $den;
    $29 = (($27|0) / ($28|0))&-1;
    $30 = (0 - ($29))|0;
    $39 = $30;
   } else {
    $31 = $num;
    $32 = $den;
    $33 = (($31|0) / ($32|0))&-1;
    $39 = $33;
   }
   $34 = $k;
   $35 = $j;
   $36 = $odither;
   $37 = (($36) + ($35<<6)|0);
   $38 = (($37) + ($34<<2)|0);
   HEAP32[$38>>2] = $39;
   $40 = $k;
   $41 = (($40) + 1)|0;
   $k = $41;
  }
  $42 = $j;
  $43 = (($42) + 1)|0;
  $j = $43;
 }
 $44 = $odither;
 STACKTOP = sp;return ($44|0);
}
function _jinit_2pass_quantizer($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cquantize = 0, $desired = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,44)|0);
 $cquantize = $6;
 $7 = $cquantize;
 $8 = $0;
 $9 = (($8) + 460|0);
 HEAP32[$9>>2] = $7;
 $10 = $cquantize;
 HEAP32[$10>>2] = 93;
 $11 = $cquantize;
 $12 = (($11) + 12|0);
 HEAP32[$12>>2] = 94;
 $13 = $cquantize;
 $14 = (($13) + 32|0);
 HEAP32[$14>>2] = 0;
 $15 = $cquantize;
 $16 = (($15) + 40|0);
 HEAP32[$16>>2] = 0;
 $17 = $0;
 $18 = (($17) + 120|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(3);
 if ($20) {
  $21 = $0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$23>>2] = 47;
  $24 = $0;
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vi[$26 & 255]($27);
 }
 $28 = $0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = $0;
 $33 = (FUNCTION_TABLE_iiii[$31 & 255]($32,1,128)|0);
 $34 = $cquantize;
 $35 = (($34) + 24|0);
 HEAP32[$35>>2] = $33;
 $i = 0;
 while(1) {
  $36 = $i;
  $37 = ($36|0)<(32);
  if (!($37)) {
   break;
  }
  $38 = $0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $0;
  $44 = (FUNCTION_TABLE_iiii[$42 & 255]($43,1,4096)|0);
  $45 = $i;
  $46 = $cquantize;
  $47 = (($46) + 24|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($45<<2)|0);
  HEAP32[$49>>2] = $44;
  $50 = $i;
  $51 = (($50) + 1)|0;
  $i = $51;
 }
 $52 = $cquantize;
 $53 = (($52) + 28|0);
 HEAP32[$53>>2] = 1;
 $54 = $0;
 $55 = (($54) + 108|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = ($56|0)!=(0);
 if ($57) {
  $58 = $0;
  $59 = (($58) + 96|0);
  $60 = HEAP32[$59>>2]|0;
  $desired = $60;
  $61 = $desired;
  $62 = ($61|0)<(8);
  if ($62) {
   $63 = $0;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($64) + 20|0);
   HEAP32[$65>>2] = 56;
   $66 = $0;
   $67 = HEAP32[$66>>2]|0;
   $68 = (($67) + 24|0);
   HEAP32[$68>>2] = 8;
   $69 = $0;
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$70>>2]|0;
   $72 = $0;
   FUNCTION_TABLE_vi[$71 & 255]($72);
  }
  $73 = $desired;
  $74 = ($73|0)>(256);
  if ($74) {
   $75 = $0;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($76) + 20|0);
   HEAP32[$77>>2] = 57;
   $78 = $0;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + 24|0);
   HEAP32[$80>>2] = 256;
   $81 = $0;
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$82>>2]|0;
   $84 = $0;
   FUNCTION_TABLE_vi[$83 & 255]($84);
  }
  $85 = $0;
  $86 = (($85) + 4|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 8|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $0;
  $91 = $desired;
  $92 = (FUNCTION_TABLE_iiiii[$89 & 255]($90,1,$91,3)|0);
  $93 = $cquantize;
  $94 = (($93) + 16|0);
  HEAP32[$94>>2] = $92;
  $95 = $desired;
  $96 = $cquantize;
  $97 = (($96) + 20|0);
  HEAP32[$97>>2] = $95;
 } else {
  $98 = $cquantize;
  $99 = (($98) + 16|0);
  HEAP32[$99>>2] = 0;
 }
 $100 = $0;
 $101 = (($100) + 88|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = ($102|0)!=(0);
 if ($103) {
  $104 = $0;
  $105 = (($104) + 88|0);
  HEAP32[$105>>2] = 2;
 }
 $106 = $0;
 $107 = (($106) + 88|0);
 $108 = HEAP32[$107>>2]|0;
 $109 = ($108|0)==(2);
 if (!($109)) {
  STACKTOP = sp;return;
 }
 $110 = $0;
 $111 = (($110) + 4|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = (($112) + 4|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $0;
 $116 = $0;
 $117 = (($116) + 112|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = (($118) + 2)|0;
 $120 = ($119*6)|0;
 $121 = (FUNCTION_TABLE_iiii[$114 & 255]($115,1,$120)|0);
 $122 = $cquantize;
 $123 = (($122) + 32|0);
 HEAP32[$123>>2] = $121;
 $124 = $0;
 _init_error_limit($124);
 STACKTOP = sp;return;
}
function _start_pass_2_quant($cinfo,$is_pre_scan) {
 $cinfo = $cinfo|0;
 $is_pre_scan = $is_pre_scan|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $arraysize = 0, $cquantize = 0, $histogram = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $is_pre_scan;
 $2 = $0;
 $3 = (($2) + 460|0);
 $4 = HEAP32[$3>>2]|0;
 $cquantize = $4;
 $5 = $cquantize;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $histogram = $7;
 $8 = $0;
 $9 = (($8) + 88|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $0;
  $13 = (($12) + 88|0);
  HEAP32[$13>>2] = 2;
 }
 $14 = $1;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $cquantize;
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = 95;
  $18 = $cquantize;
  $19 = (($18) + 8|0);
  HEAP32[$19>>2] = 96;
  $20 = $cquantize;
  $21 = (($20) + 28|0);
  HEAP32[$21>>2] = 1;
 } else {
  $22 = $0;
  $23 = (($22) + 88|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)==(2);
  if ($25) {
   $26 = $cquantize;
   $27 = (($26) + 4|0);
   HEAP32[$27>>2] = 97;
  } else {
   $28 = $cquantize;
   $29 = (($28) + 4|0);
   HEAP32[$29>>2] = 98;
  }
  $30 = $cquantize;
  $31 = (($30) + 8|0);
  HEAP32[$31>>2] = 99;
  $32 = $0;
  $33 = (($32) + 132|0);
  $34 = HEAP32[$33>>2]|0;
  $i = $34;
  $35 = $i;
  $36 = ($35|0)<(1);
  if ($36) {
   $37 = $0;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + 20|0);
   HEAP32[$39>>2] = 56;
   $40 = $0;
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + 24|0);
   HEAP32[$42>>2] = 1;
   $43 = $0;
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $0;
   FUNCTION_TABLE_vi[$45 & 255]($46);
  }
  $47 = $i;
  $48 = ($47|0)>(256);
  if ($48) {
   $49 = $0;
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + 20|0);
   HEAP32[$51>>2] = 57;
   $52 = $0;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + 24|0);
   HEAP32[$54>>2] = 256;
   $55 = $0;
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$56>>2]|0;
   $58 = $0;
   FUNCTION_TABLE_vi[$57 & 255]($58);
  }
  $59 = $0;
  $60 = (($59) + 88|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($61|0)==(2);
  if ($62) {
   $63 = $0;
   $64 = (($63) + 112|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($65) + 2)|0;
   $67 = ($66*6)|0;
   $arraysize = $67;
   $68 = $cquantize;
   $69 = (($68) + 32|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ($70|0)==(0|0);
   if ($71) {
    $72 = $0;
    $73 = (($72) + 4|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + 4|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = $0;
    $78 = $arraysize;
    $79 = (FUNCTION_TABLE_iiii[$76 & 255]($77,1,$78)|0);
    $80 = $cquantize;
    $81 = (($80) + 32|0);
    HEAP32[$81>>2] = $79;
   }
   $82 = $cquantize;
   $83 = (($82) + 32|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = $arraysize;
   _jzero_far($84,$85);
   $86 = $cquantize;
   $87 = (($86) + 40|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)==(0|0);
   if ($89) {
    $90 = $0;
    _init_error_limit($90);
   }
   $91 = $cquantize;
   $92 = (($91) + 36|0);
   HEAP32[$92>>2] = 0;
  }
 }
 $93 = $cquantize;
 $94 = (($93) + 28|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = ($95|0)!=(0);
 if (!($96)) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $97 = $i;
  $98 = ($97|0)<(32);
  if (!($98)) {
   break;
  }
  $99 = $i;
  $100 = $histogram;
  $101 = (($100) + ($99<<2)|0);
  $102 = HEAP32[$101>>2]|0;
  _jzero_far($102,4096);
  $103 = $i;
  $104 = (($103) + 1)|0;
  $i = $104;
 }
 $105 = $cquantize;
 $106 = (($105) + 28|0);
 HEAP32[$106>>2] = 0;
 STACKTOP = sp;return;
}
function _new_color_map_2_quant($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $cquantize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $cquantize;
 $5 = (($4) + 28|0);
 HEAP32[$5>>2] = 1;
 STACKTOP = sp;return;
}
function _init_error_limit($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, $in = 0, $out = 0, $table = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 255]($8,1,2044)|0);
 $table = $9;
 $10 = $table;
 $11 = (($10) + 1020|0);
 $table = $11;
 $12 = $table;
 $13 = $cquantize;
 $14 = (($13) + 40|0);
 HEAP32[$14>>2] = $12;
 $out = 0;
 $in = 0;
 while(1) {
  $15 = $in;
  $16 = ($15|0)<(16);
  if (!($16)) {
   break;
  }
  $17 = $out;
  $18 = $in;
  $19 = $table;
  $20 = (($19) + ($18<<2)|0);
  HEAP32[$20>>2] = $17;
  $21 = $out;
  $22 = (0 - ($21))|0;
  $23 = $in;
  $24 = (0 - ($23))|0;
  $25 = $table;
  $26 = (($25) + ($24<<2)|0);
  HEAP32[$26>>2] = $22;
  $27 = $in;
  $28 = (($27) + 1)|0;
  $in = $28;
  $29 = $out;
  $30 = (($29) + 1)|0;
  $out = $30;
 }
 while(1) {
  $31 = $in;
  $32 = ($31|0)<(48);
  if (!($32)) {
   break;
  }
  $33 = $out;
  $34 = $in;
  $35 = $table;
  $36 = (($35) + ($34<<2)|0);
  HEAP32[$36>>2] = $33;
  $37 = $out;
  $38 = (0 - ($37))|0;
  $39 = $in;
  $40 = (0 - ($39))|0;
  $41 = $table;
  $42 = (($41) + ($40<<2)|0);
  HEAP32[$42>>2] = $38;
  $43 = $in;
  $44 = (($43) + 1)|0;
  $in = $44;
  $45 = $in;
  $46 = $45 & 1;
  $47 = ($46|0)!=(0);
  $48 = $47 ? 0 : 1;
  $49 = $out;
  $50 = (($49) + ($48))|0;
  $out = $50;
 }
 while(1) {
  $51 = $in;
  $52 = ($51|0)<=(255);
  if (!($52)) {
   break;
  }
  $53 = $out;
  $54 = $in;
  $55 = $table;
  $56 = (($55) + ($54<<2)|0);
  HEAP32[$56>>2] = $53;
  $57 = $out;
  $58 = (0 - ($57))|0;
  $59 = $in;
  $60 = (0 - ($59))|0;
  $61 = $table;
  $62 = (($61) + ($60<<2)|0);
  HEAP32[$62>>2] = $58;
  $63 = $in;
  $64 = (($63) + 1)|0;
  $in = $64;
 }
 STACKTOP = sp;return;
}
function _prescan_quantize($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $cquantize = 0, $histogram = 0, $histp = 0;
 var $ptr = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $row = 0;
 while(1) {
  $13 = $row;
  $14 = $3;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $row;
  $17 = $1;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $ptr = $19;
  $20 = $width;
  $col = $20;
  while(1) {
   $21 = $col;
   $22 = ($21>>>0)>(0);
   if (!($22)) {
    break;
   }
   $23 = $ptr;
   $24 = (($23) + 2|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = $26 >> 3;
   $28 = $ptr;
   $29 = (($28) + 1|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = $31 >> 2;
   $33 = $ptr;
   $34 = HEAP8[$33>>0]|0;
   $35 = $34&255;
   $36 = $35 >> 3;
   $37 = $histogram;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($32<<6)|0);
   $41 = (($40) + ($27<<1)|0);
   $histp = $41;
   $42 = $histp;
   $43 = HEAP16[$42>>1]|0;
   $44 = (($43) + 1)<<16>>16;
   HEAP16[$42>>1] = $44;
   $45 = $44&65535;
   $46 = ($45|0)<=(0);
   if ($46) {
    $47 = $histp;
    $48 = HEAP16[$47>>1]|0;
    $49 = (($48) + -1)<<16>>16;
    HEAP16[$47>>1] = $49;
   }
   $50 = $ptr;
   $51 = (($50) + 3|0);
   $ptr = $51;
   $52 = $col;
   $53 = (($52) + -1)|0;
   $col = $53;
  }
  $54 = $row;
  $55 = (($54) + 1)|0;
  $row = $55;
 }
 STACKTOP = sp;return;
}
function _finish_pass1($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cquantize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 460|0);
 $3 = HEAP32[$2>>2]|0;
 $cquantize = $3;
 $4 = $cquantize;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 136|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = $cquantize;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 _select_colors($9,$12);
 $13 = $cquantize;
 $14 = (($13) + 28|0);
 HEAP32[$14>>2] = 1;
 STACKTOP = sp;return;
}
function _pass2_fs_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $belowerr0 = 0, $belowerr1 = 0, $belowerr2 = 0, $bnexterr = 0, $bpreverr0 = 0, $bpreverr1 = 0, $bpreverr2 = 0, $cachep = 0, $col = 0, $colormap0 = 0, $colormap1 = 0, $colormap2 = 0, $cquantize = 0;
 var $cur0 = 0, $cur1 = 0, $cur2 = 0, $delta = 0, $dir = 0, $dir3 = 0, $error_limit = 0, $errorptr = 0, $histogram = 0, $inptr = 0, $outptr = 0, $pixcode = 0, $range_limit = 0, $row = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $13 = $0;
 $14 = (($13) + 324|0);
 $15 = HEAP32[$14>>2]|0;
 $range_limit = $15;
 $16 = $cquantize;
 $17 = (($16) + 40|0);
 $18 = HEAP32[$17>>2]|0;
 $error_limit = $18;
 $19 = $0;
 $20 = (($19) + 136|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $colormap0 = $22;
 $23 = $0;
 $24 = (($23) + 136|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $colormap1 = $27;
 $28 = $0;
 $29 = (($28) + 136|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $colormap2 = $32;
 $row = 0;
 while(1) {
  $33 = $row;
  $34 = $3;
  $35 = ($33|0)<($34|0);
  if (!($35)) {
   break;
  }
  $36 = $row;
  $37 = $1;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inptr = $39;
  $40 = $row;
  $41 = $2;
  $42 = (($41) + ($40<<2)|0);
  $43 = HEAP32[$42>>2]|0;
  $outptr = $43;
  $44 = $cquantize;
  $45 = (($44) + 36|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = $width;
   $49 = (($48) - 1)|0;
   $50 = ($49*3)|0;
   $51 = $inptr;
   $52 = (($51) + ($50)|0);
   $inptr = $52;
   $53 = $width;
   $54 = (($53) - 1)|0;
   $55 = $outptr;
   $56 = (($55) + ($54)|0);
   $outptr = $56;
   $dir = -1;
   $dir3 = -3;
   $57 = $cquantize;
   $58 = (($57) + 32|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $width;
   $61 = (($60) + 1)|0;
   $62 = ($61*3)|0;
   $63 = (($59) + ($62<<1)|0);
   $errorptr = $63;
   $64 = $cquantize;
   $65 = (($64) + 36|0);
   HEAP32[$65>>2] = 0;
  } else {
   $dir = 1;
   $dir3 = 3;
   $66 = $cquantize;
   $67 = (($66) + 32|0);
   $68 = HEAP32[$67>>2]|0;
   $errorptr = $68;
   $69 = $cquantize;
   $70 = (($69) + 36|0);
   HEAP32[$70>>2] = 1;
  }
  $cur2 = 0;
  $cur1 = 0;
  $cur0 = 0;
  $belowerr2 = 0;
  $belowerr1 = 0;
  $belowerr0 = 0;
  $bpreverr2 = 0;
  $bpreverr1 = 0;
  $bpreverr0 = 0;
  $71 = $width;
  $col = $71;
  while(1) {
   $72 = $col;
   $73 = ($72>>>0)>(0);
   if (!($73)) {
    break;
   }
   $74 = $cur0;
   $75 = $dir3;
   $76 = (($75) + 0)|0;
   $77 = $errorptr;
   $78 = (($77) + ($76<<1)|0);
   $79 = HEAP16[$78>>1]|0;
   $80 = $79 << 16 >> 16;
   $81 = (($74) + ($80))|0;
   $82 = (($81) + 8)|0;
   $83 = $82 >> 4;
   $cur0 = $83;
   $84 = $cur1;
   $85 = $dir3;
   $86 = (($85) + 1)|0;
   $87 = $errorptr;
   $88 = (($87) + ($86<<1)|0);
   $89 = HEAP16[$88>>1]|0;
   $90 = $89 << 16 >> 16;
   $91 = (($84) + ($90))|0;
   $92 = (($91) + 8)|0;
   $93 = $92 >> 4;
   $cur1 = $93;
   $94 = $cur2;
   $95 = $dir3;
   $96 = (($95) + 2)|0;
   $97 = $errorptr;
   $98 = (($97) + ($96<<1)|0);
   $99 = HEAP16[$98>>1]|0;
   $100 = $99 << 16 >> 16;
   $101 = (($94) + ($100))|0;
   $102 = (($101) + 8)|0;
   $103 = $102 >> 4;
   $cur2 = $103;
   $104 = $cur0;
   $105 = $error_limit;
   $106 = (($105) + ($104<<2)|0);
   $107 = HEAP32[$106>>2]|0;
   $cur0 = $107;
   $108 = $cur1;
   $109 = $error_limit;
   $110 = (($109) + ($108<<2)|0);
   $111 = HEAP32[$110>>2]|0;
   $cur1 = $111;
   $112 = $cur2;
   $113 = $error_limit;
   $114 = (($113) + ($112<<2)|0);
   $115 = HEAP32[$114>>2]|0;
   $cur2 = $115;
   $116 = $inptr;
   $117 = HEAP8[$116>>0]|0;
   $118 = $117&255;
   $119 = $cur0;
   $120 = (($119) + ($118))|0;
   $cur0 = $120;
   $121 = $inptr;
   $122 = (($121) + 1|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = $123&255;
   $125 = $cur1;
   $126 = (($125) + ($124))|0;
   $cur1 = $126;
   $127 = $inptr;
   $128 = (($127) + 2|0);
   $129 = HEAP8[$128>>0]|0;
   $130 = $129&255;
   $131 = $cur2;
   $132 = (($131) + ($130))|0;
   $cur2 = $132;
   $133 = $cur0;
   $134 = $range_limit;
   $135 = (($134) + ($133)|0);
   $136 = HEAP8[$135>>0]|0;
   $137 = $136&255;
   $cur0 = $137;
   $138 = $cur1;
   $139 = $range_limit;
   $140 = (($139) + ($138)|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141&255;
   $cur1 = $142;
   $143 = $cur2;
   $144 = $range_limit;
   $145 = (($144) + ($143)|0);
   $146 = HEAP8[$145>>0]|0;
   $147 = $146&255;
   $cur2 = $147;
   $148 = $cur2;
   $149 = $148 >> 3;
   $150 = $cur1;
   $151 = $150 >> 2;
   $152 = $cur0;
   $153 = $152 >> 3;
   $154 = $histogram;
   $155 = (($154) + ($153<<2)|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = (($156) + ($151<<6)|0);
   $158 = (($157) + ($149<<1)|0);
   $cachep = $158;
   $159 = $cachep;
   $160 = HEAP16[$159>>1]|0;
   $161 = $160&65535;
   $162 = ($161|0)==(0);
   if ($162) {
    $163 = $0;
    $164 = $cur0;
    $165 = $164 >> 3;
    $166 = $cur1;
    $167 = $166 >> 2;
    $168 = $cur2;
    $169 = $168 >> 3;
    _fill_inverse_cmap($163,$165,$167,$169);
   }
   $170 = $cachep;
   $171 = HEAP16[$170>>1]|0;
   $172 = $171&65535;
   $173 = (($172) - 1)|0;
   $pixcode = $173;
   $174 = $pixcode;
   $175 = $174&255;
   $176 = $outptr;
   HEAP8[$176>>0] = $175;
   $177 = $pixcode;
   $178 = $colormap0;
   $179 = (($178) + ($177)|0);
   $180 = HEAP8[$179>>0]|0;
   $181 = $180&255;
   $182 = $cur0;
   $183 = (($182) - ($181))|0;
   $cur0 = $183;
   $184 = $pixcode;
   $185 = $colormap1;
   $186 = (($185) + ($184)|0);
   $187 = HEAP8[$186>>0]|0;
   $188 = $187&255;
   $189 = $cur1;
   $190 = (($189) - ($188))|0;
   $cur1 = $190;
   $191 = $pixcode;
   $192 = $colormap2;
   $193 = (($192) + ($191)|0);
   $194 = HEAP8[$193>>0]|0;
   $195 = $194&255;
   $196 = $cur2;
   $197 = (($196) - ($195))|0;
   $cur2 = $197;
   $198 = $cur0;
   $bnexterr = $198;
   $199 = $cur0;
   $200 = $199<<1;
   $delta = $200;
   $201 = $delta;
   $202 = $cur0;
   $203 = (($202) + ($201))|0;
   $cur0 = $203;
   $204 = $bpreverr0;
   $205 = $cur0;
   $206 = (($204) + ($205))|0;
   $207 = $206&65535;
   $208 = $errorptr;
   HEAP16[$208>>1] = $207;
   $209 = $delta;
   $210 = $cur0;
   $211 = (($210) + ($209))|0;
   $cur0 = $211;
   $212 = $belowerr0;
   $213 = $cur0;
   $214 = (($212) + ($213))|0;
   $bpreverr0 = $214;
   $215 = $bnexterr;
   $belowerr0 = $215;
   $216 = $delta;
   $217 = $cur0;
   $218 = (($217) + ($216))|0;
   $cur0 = $218;
   $219 = $cur1;
   $bnexterr = $219;
   $220 = $cur1;
   $221 = $220<<1;
   $delta = $221;
   $222 = $delta;
   $223 = $cur1;
   $224 = (($223) + ($222))|0;
   $cur1 = $224;
   $225 = $bpreverr1;
   $226 = $cur1;
   $227 = (($225) + ($226))|0;
   $228 = $227&65535;
   $229 = $errorptr;
   $230 = (($229) + 2|0);
   HEAP16[$230>>1] = $228;
   $231 = $delta;
   $232 = $cur1;
   $233 = (($232) + ($231))|0;
   $cur1 = $233;
   $234 = $belowerr1;
   $235 = $cur1;
   $236 = (($234) + ($235))|0;
   $bpreverr1 = $236;
   $237 = $bnexterr;
   $belowerr1 = $237;
   $238 = $delta;
   $239 = $cur1;
   $240 = (($239) + ($238))|0;
   $cur1 = $240;
   $241 = $cur2;
   $bnexterr = $241;
   $242 = $cur2;
   $243 = $242<<1;
   $delta = $243;
   $244 = $delta;
   $245 = $cur2;
   $246 = (($245) + ($244))|0;
   $cur2 = $246;
   $247 = $bpreverr2;
   $248 = $cur2;
   $249 = (($247) + ($248))|0;
   $250 = $249&65535;
   $251 = $errorptr;
   $252 = (($251) + 4|0);
   HEAP16[$252>>1] = $250;
   $253 = $delta;
   $254 = $cur2;
   $255 = (($254) + ($253))|0;
   $cur2 = $255;
   $256 = $belowerr2;
   $257 = $cur2;
   $258 = (($256) + ($257))|0;
   $bpreverr2 = $258;
   $259 = $bnexterr;
   $belowerr2 = $259;
   $260 = $delta;
   $261 = $cur2;
   $262 = (($261) + ($260))|0;
   $cur2 = $262;
   $263 = $dir3;
   $264 = $inptr;
   $265 = (($264) + ($263)|0);
   $inptr = $265;
   $266 = $dir;
   $267 = $outptr;
   $268 = (($267) + ($266)|0);
   $outptr = $268;
   $269 = $dir3;
   $270 = $errorptr;
   $271 = (($270) + ($269<<1)|0);
   $errorptr = $271;
   $272 = $col;
   $273 = (($272) + -1)|0;
   $col = $273;
  }
  $274 = $bpreverr0;
  $275 = $274&65535;
  $276 = $errorptr;
  HEAP16[$276>>1] = $275;
  $277 = $bpreverr1;
  $278 = $277&65535;
  $279 = $errorptr;
  $280 = (($279) + 2|0);
  HEAP16[$280>>1] = $278;
  $281 = $bpreverr2;
  $282 = $281&65535;
  $283 = $errorptr;
  $284 = (($283) + 4|0);
  HEAP16[$284>>1] = $282;
  $285 = $row;
  $286 = (($285) + 1)|0;
  $row = $286;
 }
 STACKTOP = sp;return;
}
function _pass2_no_dither($cinfo,$input_buf,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $c0 = 0, $c1 = 0, $c2 = 0, $cachep = 0, $col = 0, $cquantize = 0, $histogram = 0, $inptr = 0, $outptr = 0, $row = 0, $width = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $output_buf;
 $3 = $num_rows;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 $width = $12;
 $row = 0;
 while(1) {
  $13 = $row;
  $14 = $3;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $row;
  $17 = $1;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $inptr = $19;
  $20 = $row;
  $21 = $2;
  $22 = (($21) + ($20<<2)|0);
  $23 = HEAP32[$22>>2]|0;
  $outptr = $23;
  $24 = $width;
  $col = $24;
  while(1) {
   $25 = $col;
   $26 = ($25>>>0)>(0);
   if (!($26)) {
    break;
   }
   $27 = $inptr;
   $28 = (($27) + 1|0);
   $inptr = $28;
   $29 = HEAP8[$27>>0]|0;
   $30 = $29&255;
   $31 = $30 >> 3;
   $c0 = $31;
   $32 = $inptr;
   $33 = (($32) + 1|0);
   $inptr = $33;
   $34 = HEAP8[$32>>0]|0;
   $35 = $34&255;
   $36 = $35 >> 2;
   $c1 = $36;
   $37 = $inptr;
   $38 = (($37) + 1|0);
   $inptr = $38;
   $39 = HEAP8[$37>>0]|0;
   $40 = $39&255;
   $41 = $40 >> 3;
   $c2 = $41;
   $42 = $c2;
   $43 = $c1;
   $44 = $c0;
   $45 = $histogram;
   $46 = (($45) + ($44<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($43<<6)|0);
   $49 = (($48) + ($42<<1)|0);
   $cachep = $49;
   $50 = $cachep;
   $51 = HEAP16[$50>>1]|0;
   $52 = $51&65535;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = $0;
    $55 = $c0;
    $56 = $c1;
    $57 = $c2;
    _fill_inverse_cmap($54,$55,$56,$57);
   }
   $58 = $cachep;
   $59 = HEAP16[$58>>1]|0;
   $60 = $59&65535;
   $61 = (($60) - 1)|0;
   $62 = $61&255;
   $63 = $outptr;
   $64 = (($63) + 1|0);
   $outptr = $64;
   HEAP8[$63>>0] = $62;
   $65 = $col;
   $66 = (($65) + -1)|0;
   $col = $66;
  }
  $67 = $row;
  $68 = (($67) + 1)|0;
  $row = $68;
 }
 STACKTOP = sp;return;
}
function _finish_pass2($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _fill_inverse_cmap($cinfo,$c0,$c1,$c2) {
 $cinfo = $cinfo|0;
 $c0 = $c0|0;
 $c1 = $c1|0;
 $c2 = $c2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $bestcolor = 0, $cachep = 0, $colorlist = 0, $cptr = 0, $cquantize = 0, $histogram = 0, $ic0 = 0;
 var $ic1 = 0, $ic2 = 0, $minc0 = 0, $minc1 = 0, $minc2 = 0, $numcolors = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 448|0;
 $colorlist = sp + 192|0;
 $bestcolor = sp + 64|0;
 $0 = $cinfo;
 $1 = $c0;
 $2 = $c1;
 $3 = $c2;
 $4 = $0;
 $5 = (($4) + 460|0);
 $6 = HEAP32[$5>>2]|0;
 $cquantize = $6;
 $7 = $cquantize;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $histogram = $9;
 $10 = $1;
 $11 = $10 >> 2;
 $1 = $11;
 $12 = $2;
 $13 = $12 >> 3;
 $2 = $13;
 $14 = $3;
 $15 = $14 >> 2;
 $3 = $15;
 $16 = $1;
 $17 = $16 << 5;
 $18 = (($17) + 4)|0;
 $minc0 = $18;
 $19 = $2;
 $20 = $19 << 5;
 $21 = (($20) + 2)|0;
 $minc1 = $21;
 $22 = $3;
 $23 = $22 << 5;
 $24 = (($23) + 4)|0;
 $minc2 = $24;
 $25 = $0;
 $26 = $minc0;
 $27 = $minc1;
 $28 = $minc2;
 $29 = (_find_nearby_colors($25,$26,$27,$28,$colorlist)|0);
 $numcolors = $29;
 $30 = $0;
 $31 = $minc0;
 $32 = $minc1;
 $33 = $minc2;
 $34 = $numcolors;
 _find_best_colors($30,$31,$32,$33,$34,$colorlist,$bestcolor);
 $35 = $1;
 $36 = $35 << 2;
 $1 = $36;
 $37 = $2;
 $38 = $37 << 3;
 $2 = $38;
 $39 = $3;
 $40 = $39 << 2;
 $3 = $40;
 $cptr = $bestcolor;
 $ic0 = 0;
 while(1) {
  $41 = $ic0;
  $42 = ($41|0)<(4);
  if (!($42)) {
   break;
  }
  $ic1 = 0;
  while(1) {
   $43 = $ic1;
   $44 = ($43|0)<(8);
   if (!($44)) {
    break;
   }
   $45 = $3;
   $46 = $2;
   $47 = $ic1;
   $48 = (($46) + ($47))|0;
   $49 = $1;
   $50 = $ic0;
   $51 = (($49) + ($50))|0;
   $52 = $histogram;
   $53 = (($52) + ($51<<2)|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = (($54) + ($48<<6)|0);
   $56 = (($55) + ($45<<1)|0);
   $cachep = $56;
   $ic2 = 0;
   while(1) {
    $57 = $ic2;
    $58 = ($57|0)<(4);
    if (!($58)) {
     break;
    }
    $59 = $cptr;
    $60 = (($59) + 1|0);
    $cptr = $60;
    $61 = HEAP8[$59>>0]|0;
    $62 = $61&255;
    $63 = (($62) + 1)|0;
    $64 = $63&65535;
    $65 = $cachep;
    $66 = (($65) + 2|0);
    $cachep = $66;
    HEAP16[$65>>1] = $64;
    $67 = $ic2;
    $68 = (($67) + 1)|0;
    $ic2 = $68;
   }
   $69 = $ic1;
   $70 = (($69) + 1)|0;
   $ic1 = $70;
  }
  $71 = $ic0;
  $72 = (($71) + 1)|0;
  $ic0 = $72;
 }
 STACKTOP = sp;return;
}
function _find_nearby_colors($cinfo,$minc0,$minc1,$minc2,$colorlist) {
 $cinfo = $cinfo|0;
 $minc0 = $minc0|0;
 $minc1 = $minc1|0;
 $minc2 = $minc2|0;
 $colorlist = $colorlist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $centerc0 = 0, $centerc1 = 0;
 var $centerc2 = 0, $i = 0, $max_dist = 0, $maxc0 = 0, $maxc1 = 0, $maxc2 = 0, $min_dist = 0, $mindist = 0, $minmaxdist = 0, $ncolors = 0, $numcolors = 0, $tdist = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1104|0;
 $mindist = sp;
 $0 = $cinfo;
 $1 = $minc0;
 $2 = $minc1;
 $3 = $minc2;
 $4 = $colorlist;
 $5 = $0;
 $6 = (($5) + 132|0);
 $7 = HEAP32[$6>>2]|0;
 $numcolors = $7;
 $8 = $1;
 $9 = (($8) + 24)|0;
 $maxc0 = $9;
 $10 = $1;
 $11 = $maxc0;
 $12 = (($10) + ($11))|0;
 $13 = $12 >> 1;
 $centerc0 = $13;
 $14 = $2;
 $15 = (($14) + 28)|0;
 $maxc1 = $15;
 $16 = $2;
 $17 = $maxc1;
 $18 = (($16) + ($17))|0;
 $19 = $18 >> 1;
 $centerc1 = $19;
 $20 = $3;
 $21 = (($20) + 24)|0;
 $maxc2 = $21;
 $22 = $3;
 $23 = $maxc2;
 $24 = (($22) + ($23))|0;
 $25 = $24 >> 1;
 $centerc2 = $25;
 $minmaxdist = 2147483647;
 $i = 0;
 while(1) {
  $26 = $i;
  $27 = $numcolors;
  $28 = ($26|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = $i;
  $30 = $0;
  $31 = (($30) + 136|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($29)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  $x = $36;
  $37 = $x;
  $38 = $1;
  $39 = ($37|0)<($38|0);
  if ($39) {
   $40 = $x;
   $41 = $1;
   $42 = (($40) - ($41))|0;
   $43 = $42<<1;
   $tdist = $43;
   $44 = $tdist;
   $45 = $tdist;
   $46 = Math_imul($44, $45)|0;
   $min_dist = $46;
   $47 = $x;
   $48 = $maxc0;
   $49 = (($47) - ($48))|0;
   $50 = $49<<1;
   $tdist = $50;
   $51 = $tdist;
   $52 = $tdist;
   $53 = Math_imul($51, $52)|0;
   $max_dist = $53;
  } else {
   $54 = $x;
   $55 = $maxc0;
   $56 = ($54|0)>($55|0);
   if ($56) {
    $57 = $x;
    $58 = $maxc0;
    $59 = (($57) - ($58))|0;
    $60 = $59<<1;
    $tdist = $60;
    $61 = $tdist;
    $62 = $tdist;
    $63 = Math_imul($61, $62)|0;
    $min_dist = $63;
    $64 = $x;
    $65 = $1;
    $66 = (($64) - ($65))|0;
    $67 = $66<<1;
    $tdist = $67;
    $68 = $tdist;
    $69 = $tdist;
    $70 = Math_imul($68, $69)|0;
    $max_dist = $70;
   } else {
    $min_dist = 0;
    $71 = $x;
    $72 = $centerc0;
    $73 = ($71|0)<=($72|0);
    if ($73) {
     $74 = $x;
     $75 = $maxc0;
     $76 = (($74) - ($75))|0;
     $77 = $76<<1;
     $tdist = $77;
     $78 = $tdist;
     $79 = $tdist;
     $80 = Math_imul($78, $79)|0;
     $max_dist = $80;
    } else {
     $81 = $x;
     $82 = $1;
     $83 = (($81) - ($82))|0;
     $84 = $83<<1;
     $tdist = $84;
     $85 = $tdist;
     $86 = $tdist;
     $87 = Math_imul($85, $86)|0;
     $max_dist = $87;
    }
   }
  }
  $88 = $i;
  $89 = $0;
  $90 = (($89) + 136|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + 4|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + ($88)|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $x = $96;
  $97 = $x;
  $98 = $2;
  $99 = ($97|0)<($98|0);
  if ($99) {
   $100 = $x;
   $101 = $2;
   $102 = (($100) - ($101))|0;
   $103 = ($102*3)|0;
   $tdist = $103;
   $104 = $tdist;
   $105 = $tdist;
   $106 = Math_imul($104, $105)|0;
   $107 = $min_dist;
   $108 = (($107) + ($106))|0;
   $min_dist = $108;
   $109 = $x;
   $110 = $maxc1;
   $111 = (($109) - ($110))|0;
   $112 = ($111*3)|0;
   $tdist = $112;
   $113 = $tdist;
   $114 = $tdist;
   $115 = Math_imul($113, $114)|0;
   $116 = $max_dist;
   $117 = (($116) + ($115))|0;
   $max_dist = $117;
  } else {
   $118 = $x;
   $119 = $maxc1;
   $120 = ($118|0)>($119|0);
   if ($120) {
    $121 = $x;
    $122 = $maxc1;
    $123 = (($121) - ($122))|0;
    $124 = ($123*3)|0;
    $tdist = $124;
    $125 = $tdist;
    $126 = $tdist;
    $127 = Math_imul($125, $126)|0;
    $128 = $min_dist;
    $129 = (($128) + ($127))|0;
    $min_dist = $129;
    $130 = $x;
    $131 = $2;
    $132 = (($130) - ($131))|0;
    $133 = ($132*3)|0;
    $tdist = $133;
    $134 = $tdist;
    $135 = $tdist;
    $136 = Math_imul($134, $135)|0;
    $137 = $max_dist;
    $138 = (($137) + ($136))|0;
    $max_dist = $138;
   } else {
    $139 = $x;
    $140 = $centerc1;
    $141 = ($139|0)<=($140|0);
    if ($141) {
     $142 = $x;
     $143 = $maxc1;
     $144 = (($142) - ($143))|0;
     $145 = ($144*3)|0;
     $tdist = $145;
     $146 = $tdist;
     $147 = $tdist;
     $148 = Math_imul($146, $147)|0;
     $149 = $max_dist;
     $150 = (($149) + ($148))|0;
     $max_dist = $150;
    } else {
     $151 = $x;
     $152 = $2;
     $153 = (($151) - ($152))|0;
     $154 = ($153*3)|0;
     $tdist = $154;
     $155 = $tdist;
     $156 = $tdist;
     $157 = Math_imul($155, $156)|0;
     $158 = $max_dist;
     $159 = (($158) + ($157))|0;
     $max_dist = $159;
    }
   }
  }
  $160 = $i;
  $161 = $0;
  $162 = (($161) + 136|0);
  $163 = HEAP32[$162>>2]|0;
  $164 = (($163) + 8|0);
  $165 = HEAP32[$164>>2]|0;
  $166 = (($165) + ($160)|0);
  $167 = HEAP8[$166>>0]|0;
  $168 = $167&255;
  $x = $168;
  $169 = $x;
  $170 = $3;
  $171 = ($169|0)<($170|0);
  if ($171) {
   $172 = $x;
   $173 = $3;
   $174 = (($172) - ($173))|0;
   $175 = $174;
   $tdist = $175;
   $176 = $tdist;
   $177 = $tdist;
   $178 = Math_imul($176, $177)|0;
   $179 = $min_dist;
   $180 = (($179) + ($178))|0;
   $min_dist = $180;
   $181 = $x;
   $182 = $maxc2;
   $183 = (($181) - ($182))|0;
   $184 = $183;
   $tdist = $184;
   $185 = $tdist;
   $186 = $tdist;
   $187 = Math_imul($185, $186)|0;
   $188 = $max_dist;
   $189 = (($188) + ($187))|0;
   $max_dist = $189;
  } else {
   $190 = $x;
   $191 = $maxc2;
   $192 = ($190|0)>($191|0);
   if ($192) {
    $193 = $x;
    $194 = $maxc2;
    $195 = (($193) - ($194))|0;
    $196 = $195;
    $tdist = $196;
    $197 = $tdist;
    $198 = $tdist;
    $199 = Math_imul($197, $198)|0;
    $200 = $min_dist;
    $201 = (($200) + ($199))|0;
    $min_dist = $201;
    $202 = $x;
    $203 = $3;
    $204 = (($202) - ($203))|0;
    $205 = $204;
    $tdist = $205;
    $206 = $tdist;
    $207 = $tdist;
    $208 = Math_imul($206, $207)|0;
    $209 = $max_dist;
    $210 = (($209) + ($208))|0;
    $max_dist = $210;
   } else {
    $211 = $x;
    $212 = $centerc2;
    $213 = ($211|0)<=($212|0);
    if ($213) {
     $214 = $x;
     $215 = $maxc2;
     $216 = (($214) - ($215))|0;
     $217 = $216;
     $tdist = $217;
     $218 = $tdist;
     $219 = $tdist;
     $220 = Math_imul($218, $219)|0;
     $221 = $max_dist;
     $222 = (($221) + ($220))|0;
     $max_dist = $222;
    } else {
     $223 = $x;
     $224 = $3;
     $225 = (($223) - ($224))|0;
     $226 = $225;
     $tdist = $226;
     $227 = $tdist;
     $228 = $tdist;
     $229 = Math_imul($227, $228)|0;
     $230 = $max_dist;
     $231 = (($230) + ($229))|0;
     $max_dist = $231;
    }
   }
  }
  $232 = $min_dist;
  $233 = $i;
  $234 = (($mindist) + ($233<<2)|0);
  HEAP32[$234>>2] = $232;
  $235 = $max_dist;
  $236 = $minmaxdist;
  $237 = ($235|0)<($236|0);
  if ($237) {
   $238 = $max_dist;
   $minmaxdist = $238;
  }
  $239 = $i;
  $240 = (($239) + 1)|0;
  $i = $240;
 }
 $ncolors = 0;
 $i = 0;
 while(1) {
  $241 = $i;
  $242 = $numcolors;
  $243 = ($241|0)<($242|0);
  if (!($243)) {
   break;
  }
  $244 = $i;
  $245 = (($mindist) + ($244<<2)|0);
  $246 = HEAP32[$245>>2]|0;
  $247 = $minmaxdist;
  $248 = ($246|0)<=($247|0);
  if ($248) {
   $249 = $i;
   $250 = $249&255;
   $251 = $ncolors;
   $252 = (($251) + 1)|0;
   $ncolors = $252;
   $253 = $4;
   $254 = (($253) + ($251)|0);
   HEAP8[$254>>0] = $250;
  }
  $255 = $i;
  $256 = (($255) + 1)|0;
  $i = $256;
 }
 $257 = $ncolors;
 STACKTOP = sp;return ($257|0);
}
function _find_best_colors($cinfo,$minc0,$minc1,$minc2,$numcolors,$colorlist,$bestcolor) {
 $cinfo = $cinfo|0;
 $minc0 = $minc0|0;
 $minc1 = $minc1|0;
 $minc2 = $minc2|0;
 $numcolors = $numcolors|0;
 $colorlist = $colorlist|0;
 $bestcolor = $bestcolor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bestdist = 0, $bptr = 0, $cptr = 0, $dist0 = 0, $dist1 = 0, $dist2 = 0, $i = 0, $ic0 = 0, $ic1 = 0, $ic2 = 0, $icolor = 0, $inc0 = 0, $inc1 = 0, $inc2 = 0;
 var $xx0 = 0, $xx1 = 0, $xx2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 608|0;
 $bestdist = sp;
 $0 = $cinfo;
 $1 = $minc0;
 $2 = $minc1;
 $3 = $minc2;
 $4 = $numcolors;
 $5 = $colorlist;
 $6 = $bestcolor;
 $bptr = $bestdist;
 $i = 127;
 while(1) {
  $7 = $i;
  $8 = ($7|0)>=(0);
  if (!($8)) {
   break;
  }
  $9 = $bptr;
  $10 = (($9) + 4|0);
  $bptr = $10;
  HEAP32[$9>>2] = 2147483647;
  $11 = $i;
  $12 = (($11) + -1)|0;
  $i = $12;
 }
 $i = 0;
 while(1) {
  $13 = $i;
  $14 = $4;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $i;
  $17 = $5;
  $18 = (($17) + ($16)|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $icolor = $20;
  $21 = $1;
  $22 = $icolor;
  $23 = $0;
  $24 = (($23) + 136|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($22)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = (($21) - ($29))|0;
  $31 = $30<<1;
  $inc0 = $31;
  $32 = $inc0;
  $33 = $inc0;
  $34 = Math_imul($32, $33)|0;
  $dist0 = $34;
  $35 = $2;
  $36 = $icolor;
  $37 = $0;
  $38 = (($37) + 136|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($36)|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = (($35) - ($44))|0;
  $46 = ($45*3)|0;
  $inc1 = $46;
  $47 = $inc1;
  $48 = $inc1;
  $49 = Math_imul($47, $48)|0;
  $50 = $dist0;
  $51 = (($50) + ($49))|0;
  $dist0 = $51;
  $52 = $3;
  $53 = $icolor;
  $54 = $0;
  $55 = (($54) + 136|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($53)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = (($52) - ($61))|0;
  $63 = $62;
  $inc2 = $63;
  $64 = $inc2;
  $65 = $inc2;
  $66 = Math_imul($64, $65)|0;
  $67 = $dist0;
  $68 = (($67) + ($66))|0;
  $dist0 = $68;
  $69 = $inc0;
  $70 = $69<<5;
  $71 = (($70) + 256)|0;
  $inc0 = $71;
  $72 = $inc1;
  $73 = ($72*24)|0;
  $74 = (($73) + 144)|0;
  $inc1 = $74;
  $75 = $inc2;
  $76 = $75<<4;
  $77 = (($76) + 64)|0;
  $inc2 = $77;
  $bptr = $bestdist;
  $78 = $6;
  $cptr = $78;
  $79 = $inc0;
  $xx0 = $79;
  $ic0 = 3;
  while(1) {
   $80 = $ic0;
   $81 = ($80|0)>=(0);
   if (!($81)) {
    break;
   }
   $82 = $dist0;
   $dist1 = $82;
   $83 = $inc1;
   $xx1 = $83;
   $ic1 = 7;
   while(1) {
    $84 = $ic1;
    $85 = ($84|0)>=(0);
    if (!($85)) {
     break;
    }
    $86 = $dist1;
    $dist2 = $86;
    $87 = $inc2;
    $xx2 = $87;
    $ic2 = 3;
    while(1) {
     $88 = $ic2;
     $89 = ($88|0)>=(0);
     if (!($89)) {
      break;
     }
     $90 = $dist2;
     $91 = $bptr;
     $92 = HEAP32[$91>>2]|0;
     $93 = ($90|0)<($92|0);
     if ($93) {
      $94 = $dist2;
      $95 = $bptr;
      HEAP32[$95>>2] = $94;
      $96 = $icolor;
      $97 = $96&255;
      $98 = $cptr;
      HEAP8[$98>>0] = $97;
     }
     $99 = $xx2;
     $100 = $dist2;
     $101 = (($100) + ($99))|0;
     $dist2 = $101;
     $102 = $xx2;
     $103 = (($102) + 128)|0;
     $xx2 = $103;
     $104 = $bptr;
     $105 = (($104) + 4|0);
     $bptr = $105;
     $106 = $cptr;
     $107 = (($106) + 1|0);
     $cptr = $107;
     $108 = $ic2;
     $109 = (($108) + -1)|0;
     $ic2 = $109;
    }
    $110 = $xx1;
    $111 = $dist1;
    $112 = (($111) + ($110))|0;
    $dist1 = $112;
    $113 = $xx1;
    $114 = (($113) + 288)|0;
    $xx1 = $114;
    $115 = $ic1;
    $116 = (($115) + -1)|0;
    $ic1 = $116;
   }
   $117 = $xx0;
   $118 = $dist0;
   $119 = (($118) + ($117))|0;
   $dist0 = $119;
   $120 = $xx0;
   $121 = (($120) + 512)|0;
   $xx0 = $121;
   $122 = $ic0;
   $123 = (($122) + -1)|0;
   $ic0 = $123;
  }
  $124 = $i;
  $125 = (($124) + 1)|0;
  $i = $125;
 }
 STACKTOP = sp;return;
}
function _select_colors($cinfo,$desired_colors) {
 $cinfo = $cinfo|0;
 $desired_colors = $desired_colors|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $boxlist = 0, $i = 0, $numboxes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $desired_colors;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = $1;
 $8 = $7<<5;
 $9 = (FUNCTION_TABLE_iiii[$5 & 255]($6,1,$8)|0);
 $boxlist = $9;
 $numboxes = 1;
 $10 = $boxlist;
 HEAP32[$10>>2] = 0;
 $11 = $boxlist;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 31;
 $13 = $boxlist;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 $15 = $boxlist;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = 63;
 $17 = $boxlist;
 $18 = (($17) + 16|0);
 HEAP32[$18>>2] = 0;
 $19 = $boxlist;
 $20 = (($19) + 20|0);
 HEAP32[$20>>2] = 31;
 $21 = $0;
 $22 = $boxlist;
 _update_box($21,$22);
 $23 = $0;
 $24 = $boxlist;
 $25 = $numboxes;
 $26 = $1;
 $27 = (_median_cut($23,$24,$25,$26)|0);
 $numboxes = $27;
 $i = 0;
 while(1) {
  $28 = $i;
  $29 = $numboxes;
  $30 = ($28|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = $0;
  $32 = $i;
  $33 = $boxlist;
  $34 = (($33) + ($32<<5)|0);
  $35 = $i;
  _compute_color($31,$34,$35);
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 $38 = $numboxes;
 $39 = $0;
 $40 = (($39) + 132|0);
 HEAP32[$40>>2] = $38;
 $41 = $0;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + 20|0);
 HEAP32[$43>>2] = 96;
 $44 = $numboxes;
 $45 = $0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 24|0);
 HEAP32[$47>>2] = $44;
 $48 = $0;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = $0;
 FUNCTION_TABLE_vii[$51 & 255]($52,1);
 STACKTOP = sp;return;
}
function _update_box($cinfo,$boxp) {
 $cinfo = $cinfo|0;
 $boxp = $boxp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c0 = 0, $c0max = 0, $c0min = 0, $c1 = 0, $c1max = 0, $c1min = 0, $c2 = 0;
 var $c2max = 0, $c2min = 0, $ccount = 0, $cquantize = 0, $dist0 = 0, $dist1 = 0, $dist2 = 0, $histogram = 0, $histp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $boxp;
 $2 = $0;
 $3 = (($2) + 460|0);
 $4 = HEAP32[$3>>2]|0;
 $cquantize = $4;
 $5 = $cquantize;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $histogram = $7;
 $8 = $1;
 $9 = HEAP32[$8>>2]|0;
 $c0min = $9;
 $10 = $1;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $c0max = $12;
 $13 = $1;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $c1min = $15;
 $16 = $1;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $c1max = $18;
 $19 = $1;
 $20 = (($19) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $c2min = $21;
 $22 = $1;
 $23 = (($22) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $c2max = $24;
 $25 = $c0max;
 $26 = $c0min;
 $27 = ($25|0)>($26|0);
 do {
  if ($27) {
   $28 = $c0min;
   $c0 = $28;
   L3: while(1) {
    $29 = $c0;
    $30 = $c0max;
    $31 = ($29|0)<=($30|0);
    if (!($31)) {
     label = 16;
     break;
    }
    $32 = $c1min;
    $c1 = $32;
    while(1) {
     $33 = $c1;
     $34 = $c1max;
     $35 = ($33|0)<=($34|0);
     if (!($35)) {
      break;
     }
     $36 = $c2min;
     $37 = $c1;
     $38 = $c0;
     $39 = $histogram;
     $40 = (($39) + ($38<<2)|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($37<<6)|0);
     $43 = (($42) + ($36<<1)|0);
     $histp = $43;
     $44 = $c2min;
     $c2 = $44;
     while(1) {
      $45 = $c2;
      $46 = $c2max;
      $47 = ($45|0)<=($46|0);
      if (!($47)) {
       break;
      }
      $48 = $histp;
      $49 = (($48) + 2|0);
      $histp = $49;
      $50 = HEAP16[$48>>1]|0;
      $51 = $50&65535;
      $52 = ($51|0)!=(0);
      if ($52) {
       label = 9;
       break L3;
      }
      $55 = $c2;
      $56 = (($55) + 1)|0;
      $c2 = $56;
     }
     $57 = $c1;
     $58 = (($57) + 1)|0;
     $c1 = $58;
    }
    $59 = $c0;
    $60 = (($59) + 1)|0;
    $c0 = $60;
   }
   if ((label|0) == 9) {
    $53 = $c0;
    $c0min = $53;
    $54 = $1;
    HEAP32[$54>>2] = $53;
    break;
   }
   else if ((label|0) == 16) {
    label = 17;
    break;
   }
  } else {
   label = 17;
  }
 } while(0);
 if ((label|0) == 17) {
 }
 $61 = $c0max;
 $62 = $c0min;
 $63 = ($61|0)>($62|0);
 do {
  if ($63) {
   $64 = $c0max;
   $c0 = $64;
   L26: while(1) {
    $65 = $c0;
    $66 = $c0min;
    $67 = ($65|0)>=($66|0);
    if (!($67)) {
     label = 33;
     break;
    }
    $68 = $c1min;
    $c1 = $68;
    while(1) {
     $69 = $c1;
     $70 = $c1max;
     $71 = ($69|0)<=($70|0);
     if (!($71)) {
      break;
     }
     $72 = $c2min;
     $73 = $c1;
     $74 = $c0;
     $75 = $histogram;
     $76 = (($75) + ($74<<2)|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = (($77) + ($73<<6)|0);
     $79 = (($78) + ($72<<1)|0);
     $histp = $79;
     $80 = $c2min;
     $c2 = $80;
     while(1) {
      $81 = $c2;
      $82 = $c2max;
      $83 = ($81|0)<=($82|0);
      if (!($83)) {
       break;
      }
      $84 = $histp;
      $85 = (($84) + 2|0);
      $histp = $85;
      $86 = HEAP16[$84>>1]|0;
      $87 = $86&65535;
      $88 = ($87|0)!=(0);
      if ($88) {
       label = 26;
       break L26;
      }
      $92 = $c2;
      $93 = (($92) + 1)|0;
      $c2 = $93;
     }
     $94 = $c1;
     $95 = (($94) + 1)|0;
     $c1 = $95;
    }
    $96 = $c0;
    $97 = (($96) + -1)|0;
    $c0 = $97;
   }
   if ((label|0) == 26) {
    $89 = $c0;
    $c0max = $89;
    $90 = $1;
    $91 = (($90) + 4|0);
    HEAP32[$91>>2] = $89;
    break;
   }
   else if ((label|0) == 33) {
    label = 34;
    break;
   }
  } else {
   label = 34;
  }
 } while(0);
 if ((label|0) == 34) {
 }
 $98 = $c1max;
 $99 = $c1min;
 $100 = ($98|0)>($99|0);
 do {
  if ($100) {
   $101 = $c1min;
   $c1 = $101;
   L49: while(1) {
    $102 = $c1;
    $103 = $c1max;
    $104 = ($102|0)<=($103|0);
    if (!($104)) {
     label = 50;
     break;
    }
    $105 = $c0min;
    $c0 = $105;
    while(1) {
     $106 = $c0;
     $107 = $c0max;
     $108 = ($106|0)<=($107|0);
     if (!($108)) {
      break;
     }
     $109 = $c2min;
     $110 = $c1;
     $111 = $c0;
     $112 = $histogram;
     $113 = (($112) + ($111<<2)|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + ($110<<6)|0);
     $116 = (($115) + ($109<<1)|0);
     $histp = $116;
     $117 = $c2min;
     $c2 = $117;
     while(1) {
      $118 = $c2;
      $119 = $c2max;
      $120 = ($118|0)<=($119|0);
      if (!($120)) {
       break;
      }
      $121 = $histp;
      $122 = (($121) + 2|0);
      $histp = $122;
      $123 = HEAP16[$121>>1]|0;
      $124 = $123&65535;
      $125 = ($124|0)!=(0);
      if ($125) {
       label = 43;
       break L49;
      }
      $129 = $c2;
      $130 = (($129) + 1)|0;
      $c2 = $130;
     }
     $131 = $c0;
     $132 = (($131) + 1)|0;
     $c0 = $132;
    }
    $133 = $c1;
    $134 = (($133) + 1)|0;
    $c1 = $134;
   }
   if ((label|0) == 43) {
    $126 = $c1;
    $c1min = $126;
    $127 = $1;
    $128 = (($127) + 8|0);
    HEAP32[$128>>2] = $126;
    break;
   }
   else if ((label|0) == 50) {
    label = 51;
    break;
   }
  } else {
   label = 51;
  }
 } while(0);
 if ((label|0) == 51) {
 }
 $135 = $c1max;
 $136 = $c1min;
 $137 = ($135|0)>($136|0);
 do {
  if ($137) {
   $138 = $c1max;
   $c1 = $138;
   L72: while(1) {
    $139 = $c1;
    $140 = $c1min;
    $141 = ($139|0)>=($140|0);
    if (!($141)) {
     label = 67;
     break;
    }
    $142 = $c0min;
    $c0 = $142;
    while(1) {
     $143 = $c0;
     $144 = $c0max;
     $145 = ($143|0)<=($144|0);
     if (!($145)) {
      break;
     }
     $146 = $c2min;
     $147 = $c1;
     $148 = $c0;
     $149 = $histogram;
     $150 = (($149) + ($148<<2)|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = (($151) + ($147<<6)|0);
     $153 = (($152) + ($146<<1)|0);
     $histp = $153;
     $154 = $c2min;
     $c2 = $154;
     while(1) {
      $155 = $c2;
      $156 = $c2max;
      $157 = ($155|0)<=($156|0);
      if (!($157)) {
       break;
      }
      $158 = $histp;
      $159 = (($158) + 2|0);
      $histp = $159;
      $160 = HEAP16[$158>>1]|0;
      $161 = $160&65535;
      $162 = ($161|0)!=(0);
      if ($162) {
       label = 60;
       break L72;
      }
      $166 = $c2;
      $167 = (($166) + 1)|0;
      $c2 = $167;
     }
     $168 = $c0;
     $169 = (($168) + 1)|0;
     $c0 = $169;
    }
    $170 = $c1;
    $171 = (($170) + -1)|0;
    $c1 = $171;
   }
   if ((label|0) == 60) {
    $163 = $c1;
    $c1max = $163;
    $164 = $1;
    $165 = (($164) + 12|0);
    HEAP32[$165>>2] = $163;
    break;
   }
   else if ((label|0) == 67) {
    label = 68;
    break;
   }
  } else {
   label = 68;
  }
 } while(0);
 if ((label|0) == 68) {
 }
 $172 = $c2max;
 $173 = $c2min;
 $174 = ($172|0)>($173|0);
 do {
  if ($174) {
   $175 = $c2min;
   $c2 = $175;
   L95: while(1) {
    $176 = $c2;
    $177 = $c2max;
    $178 = ($176|0)<=($177|0);
    if (!($178)) {
     label = 84;
     break;
    }
    $179 = $c0min;
    $c0 = $179;
    while(1) {
     $180 = $c0;
     $181 = $c0max;
     $182 = ($180|0)<=($181|0);
     if (!($182)) {
      break;
     }
     $183 = $c2;
     $184 = $c1min;
     $185 = $c0;
     $186 = $histogram;
     $187 = (($186) + ($185<<2)|0);
     $188 = HEAP32[$187>>2]|0;
     $189 = (($188) + ($184<<6)|0);
     $190 = (($189) + ($183<<1)|0);
     $histp = $190;
     $191 = $c1min;
     $c1 = $191;
     while(1) {
      $192 = $c1;
      $193 = $c1max;
      $194 = ($192|0)<=($193|0);
      if (!($194)) {
       break;
      }
      $195 = $histp;
      $196 = HEAP16[$195>>1]|0;
      $197 = $196&65535;
      $198 = ($197|0)!=(0);
      if ($198) {
       label = 77;
       break L95;
      }
      $202 = $c1;
      $203 = (($202) + 1)|0;
      $c1 = $203;
      $204 = $histp;
      $205 = (($204) + 64|0);
      $histp = $205;
     }
     $206 = $c0;
     $207 = (($206) + 1)|0;
     $c0 = $207;
    }
    $208 = $c2;
    $209 = (($208) + 1)|0;
    $c2 = $209;
   }
   if ((label|0) == 77) {
    $199 = $c2;
    $c2min = $199;
    $200 = $1;
    $201 = (($200) + 16|0);
    HEAP32[$201>>2] = $199;
    break;
   }
   else if ((label|0) == 84) {
    label = 85;
    break;
   }
  } else {
   label = 85;
  }
 } while(0);
 if ((label|0) == 85) {
 }
 $210 = $c2max;
 $211 = $c2min;
 $212 = ($210|0)>($211|0);
 do {
  if ($212) {
   $213 = $c2max;
   $c2 = $213;
   L118: while(1) {
    $214 = $c2;
    $215 = $c2min;
    $216 = ($214|0)>=($215|0);
    if (!($216)) {
     label = 101;
     break;
    }
    $217 = $c0min;
    $c0 = $217;
    while(1) {
     $218 = $c0;
     $219 = $c0max;
     $220 = ($218|0)<=($219|0);
     if (!($220)) {
      break;
     }
     $221 = $c2;
     $222 = $c1min;
     $223 = $c0;
     $224 = $histogram;
     $225 = (($224) + ($223<<2)|0);
     $226 = HEAP32[$225>>2]|0;
     $227 = (($226) + ($222<<6)|0);
     $228 = (($227) + ($221<<1)|0);
     $histp = $228;
     $229 = $c1min;
     $c1 = $229;
     while(1) {
      $230 = $c1;
      $231 = $c1max;
      $232 = ($230|0)<=($231|0);
      if (!($232)) {
       break;
      }
      $233 = $histp;
      $234 = HEAP16[$233>>1]|0;
      $235 = $234&65535;
      $236 = ($235|0)!=(0);
      if ($236) {
       label = 94;
       break L118;
      }
      $240 = $c1;
      $241 = (($240) + 1)|0;
      $c1 = $241;
      $242 = $histp;
      $243 = (($242) + 64|0);
      $histp = $243;
     }
     $244 = $c0;
     $245 = (($244) + 1)|0;
     $c0 = $245;
    }
    $246 = $c2;
    $247 = (($246) + -1)|0;
    $c2 = $247;
   }
   if ((label|0) == 94) {
    $237 = $c2;
    $c2max = $237;
    $238 = $1;
    $239 = (($238) + 20|0);
    HEAP32[$239>>2] = $237;
    break;
   }
   else if ((label|0) == 101) {
    label = 102;
    break;
   }
  } else {
   label = 102;
  }
 } while(0);
 if ((label|0) == 102) {
 }
 $248 = $c0max;
 $249 = $c0min;
 $250 = (($248) - ($249))|0;
 $251 = $250 << 3;
 $252 = $251<<1;
 $dist0 = $252;
 $253 = $c1max;
 $254 = $c1min;
 $255 = (($253) - ($254))|0;
 $256 = $255 << 2;
 $257 = ($256*3)|0;
 $dist1 = $257;
 $258 = $c2max;
 $259 = $c2min;
 $260 = (($258) - ($259))|0;
 $261 = $260 << 3;
 $262 = $261;
 $dist2 = $262;
 $263 = $dist0;
 $264 = $dist0;
 $265 = Math_imul($263, $264)|0;
 $266 = $dist1;
 $267 = $dist1;
 $268 = Math_imul($266, $267)|0;
 $269 = (($265) + ($268))|0;
 $270 = $dist2;
 $271 = $dist2;
 $272 = Math_imul($270, $271)|0;
 $273 = (($269) + ($272))|0;
 $274 = $1;
 $275 = (($274) + 24|0);
 HEAP32[$275>>2] = $273;
 $ccount = 0;
 $276 = $c0min;
 $c0 = $276;
 while(1) {
  $277 = $c0;
  $278 = $c0max;
  $279 = ($277|0)<=($278|0);
  if (!($279)) {
   break;
  }
  $280 = $c1min;
  $c1 = $280;
  while(1) {
   $281 = $c1;
   $282 = $c1max;
   $283 = ($281|0)<=($282|0);
   if (!($283)) {
    break;
   }
   $284 = $c2min;
   $285 = $c1;
   $286 = $c0;
   $287 = $histogram;
   $288 = (($287) + ($286<<2)|0);
   $289 = HEAP32[$288>>2]|0;
   $290 = (($289) + ($285<<6)|0);
   $291 = (($290) + ($284<<1)|0);
   $histp = $291;
   $292 = $c2min;
   $c2 = $292;
   while(1) {
    $293 = $c2;
    $294 = $c2max;
    $295 = ($293|0)<=($294|0);
    if (!($295)) {
     break;
    }
    $296 = $histp;
    $297 = HEAP16[$296>>1]|0;
    $298 = $297&65535;
    $299 = ($298|0)!=(0);
    if ($299) {
     $300 = $ccount;
     $301 = (($300) + 1)|0;
     $ccount = $301;
    }
    $302 = $c2;
    $303 = (($302) + 1)|0;
    $c2 = $303;
    $304 = $histp;
    $305 = (($304) + 2|0);
    $histp = $305;
   }
   $306 = $c1;
   $307 = (($306) + 1)|0;
   $c1 = $307;
  }
  $308 = $c0;
  $309 = (($308) + 1)|0;
  $c0 = $309;
 }
 $310 = $ccount;
 $311 = $1;
 $312 = (($311) + 28|0);
 HEAP32[$312>>2] = $310;
 STACKTOP = sp;return;
}
function _median_cut($cinfo,$boxlist,$numboxes,$desired_colors) {
 $cinfo = $cinfo|0;
 $boxlist = $boxlist|0;
 $numboxes = $numboxes|0;
 $desired_colors = $desired_colors|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $b1 = 0, $b2 = 0, $c0 = 0, $c1 = 0, $c2 = 0;
 var $cmax = 0, $lb = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $boxlist;
 $2 = $numboxes;
 $3 = $desired_colors;
 while(1) {
  $4 = $2;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   label = 17;
   break;
  }
  $7 = $2;
  $8 = $7<<1;
  $9 = $3;
  $10 = ($8|0)<=($9|0);
  if ($10) {
   $11 = $1;
   $12 = $2;
   $13 = (_find_biggest_color_pop($11,$12)|0);
   $b1 = $13;
  } else {
   $14 = $1;
   $15 = $2;
   $16 = (_find_biggest_volume($14,$15)|0);
   $b1 = $16;
  }
  $17 = $b1;
  $18 = ($17|0)==(0|0);
  if ($18) {
   break;
  }
  $19 = $2;
  $20 = $1;
  $21 = (($20) + ($19<<5)|0);
  $b2 = $21;
  $22 = $b1;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $b2;
  $26 = (($25) + 4|0);
  HEAP32[$26>>2] = $24;
  $27 = $b1;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $b2;
  $31 = (($30) + 12|0);
  HEAP32[$31>>2] = $29;
  $32 = $b1;
  $33 = (($32) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $b2;
  $36 = (($35) + 20|0);
  HEAP32[$36>>2] = $34;
  $37 = $b1;
  $38 = HEAP32[$37>>2]|0;
  $39 = $b2;
  HEAP32[$39>>2] = $38;
  $40 = $b1;
  $41 = (($40) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $b2;
  $44 = (($43) + 8|0);
  HEAP32[$44>>2] = $42;
  $45 = $b1;
  $46 = (($45) + 16|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $b2;
  $49 = (($48) + 16|0);
  HEAP32[$49>>2] = $47;
  $50 = $b1;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $b1;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) - ($54))|0;
  $56 = $55 << 3;
  $57 = $56<<1;
  $c0 = $57;
  $58 = $b1;
  $59 = (($58) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = $b1;
  $62 = (($61) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($60) - ($63))|0;
  $65 = $64 << 2;
  $66 = ($65*3)|0;
  $c1 = $66;
  $67 = $b1;
  $68 = (($67) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = $b1;
  $71 = (($70) + 16|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($69) - ($72))|0;
  $74 = $73 << 3;
  $75 = $74;
  $c2 = $75;
  $76 = $c1;
  $cmax = $76;
  $n = 1;
  $77 = $c0;
  $78 = $cmax;
  $79 = ($77|0)>($78|0);
  if ($79) {
   $80 = $c0;
   $cmax = $80;
   $n = 0;
  }
  $81 = $c2;
  $82 = $cmax;
  $83 = ($81|0)>($82|0);
  if ($83) {
   $n = 2;
  }
  $84 = $n;
  if ((($84|0) == 1)) {
   $98 = $b1;
   $99 = (($98) + 12|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = $b1;
   $102 = (($101) + 8|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (($100) + ($103))|0;
   $105 = (($104|0) / 2)&-1;
   $lb = $105;
   $106 = $lb;
   $107 = $b1;
   $108 = (($107) + 12|0);
   HEAP32[$108>>2] = $106;
   $109 = $lb;
   $110 = (($109) + 1)|0;
   $111 = $b2;
   $112 = (($111) + 8|0);
   HEAP32[$112>>2] = $110;
  } else if ((($84|0) == 0)) {
   $85 = $b1;
   $86 = (($85) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = $b1;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($87) + ($89))|0;
   $91 = (($90|0) / 2)&-1;
   $lb = $91;
   $92 = $lb;
   $93 = $b1;
   $94 = (($93) + 4|0);
   HEAP32[$94>>2] = $92;
   $95 = $lb;
   $96 = (($95) + 1)|0;
   $97 = $b2;
   HEAP32[$97>>2] = $96;
  } else if ((($84|0) == 2)) {
   $113 = $b1;
   $114 = (($113) + 20|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = $b1;
   $117 = (($116) + 16|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (($115) + ($118))|0;
   $120 = (($119|0) / 2)&-1;
   $lb = $120;
   $121 = $lb;
   $122 = $b1;
   $123 = (($122) + 20|0);
   HEAP32[$123>>2] = $121;
   $124 = $lb;
   $125 = (($124) + 1)|0;
   $126 = $b2;
   $127 = (($126) + 16|0);
   HEAP32[$127>>2] = $125;
  }
  $128 = $0;
  $129 = $b1;
  _update_box($128,$129);
  $130 = $0;
  $131 = $b2;
  _update_box($130,$131);
  $132 = $2;
  $133 = (($132) + 1)|0;
  $2 = $133;
 }
 if ((label|0) == 17) {
  $134 = $2;
  STACKTOP = sp;return ($134|0);
 }
 $134 = $2;
 STACKTOP = sp;return ($134|0);
}
function _compute_color($cinfo,$boxp,$icolor) {
 $cinfo = $cinfo|0;
 $boxp = $boxp|0;
 $icolor = $icolor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $c0 = 0, $c0max = 0, $c0min = 0, $c0total = 0, $c1 = 0, $c1max = 0, $c1min = 0, $c1total = 0, $c2 = 0, $c2max = 0, $c2min = 0, $c2total = 0, $count = 0, $cquantize = 0, $histogram = 0, $histp = 0, $total = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $boxp;
 $2 = $icolor;
 $3 = $0;
 $4 = (($3) + 460|0);
 $5 = HEAP32[$4>>2]|0;
 $cquantize = $5;
 $6 = $cquantize;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $histogram = $8;
 $total = 0;
 $c0total = 0;
 $c1total = 0;
 $c2total = 0;
 $9 = $1;
 $10 = HEAP32[$9>>2]|0;
 $c0min = $10;
 $11 = $1;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $c0max = $13;
 $14 = $1;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $c1min = $16;
 $17 = $1;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $c1max = $19;
 $20 = $1;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $c2min = $22;
 $23 = $1;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $c2max = $25;
 $26 = $c0min;
 $c0 = $26;
 while(1) {
  $27 = $c0;
  $28 = $c0max;
  $29 = ($27|0)<=($28|0);
  if (!($29)) {
   break;
  }
  $30 = $c1min;
  $c1 = $30;
  while(1) {
   $31 = $c1;
   $32 = $c1max;
   $33 = ($31|0)<=($32|0);
   if (!($33)) {
    break;
   }
   $34 = $c2min;
   $35 = $c1;
   $36 = $c0;
   $37 = $histogram;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($35<<6)|0);
   $41 = (($40) + ($34<<1)|0);
   $histp = $41;
   $42 = $c2min;
   $c2 = $42;
   while(1) {
    $43 = $c2;
    $44 = $c2max;
    $45 = ($43|0)<=($44|0);
    if (!($45)) {
     break;
    }
    $46 = $histp;
    $47 = (($46) + 2|0);
    $histp = $47;
    $48 = HEAP16[$46>>1]|0;
    $49 = $48&65535;
    $count = $49;
    $50 = ($49|0)!=(0);
    if ($50) {
     $51 = $count;
     $52 = $total;
     $53 = (($52) + ($51))|0;
     $total = $53;
     $54 = $c0;
     $55 = $54 << 3;
     $56 = (($55) + 4)|0;
     $57 = $count;
     $58 = Math_imul($56, $57)|0;
     $59 = $c0total;
     $60 = (($59) + ($58))|0;
     $c0total = $60;
     $61 = $c1;
     $62 = $61 << 2;
     $63 = (($62) + 2)|0;
     $64 = $count;
     $65 = Math_imul($63, $64)|0;
     $66 = $c1total;
     $67 = (($66) + ($65))|0;
     $c1total = $67;
     $68 = $c2;
     $69 = $68 << 3;
     $70 = (($69) + 4)|0;
     $71 = $count;
     $72 = Math_imul($70, $71)|0;
     $73 = $c2total;
     $74 = (($73) + ($72))|0;
     $c2total = $74;
    }
    $75 = $c2;
    $76 = (($75) + 1)|0;
    $c2 = $76;
   }
   $77 = $c1;
   $78 = (($77) + 1)|0;
   $c1 = $78;
  }
  $79 = $c0;
  $80 = (($79) + 1)|0;
  $c0 = $80;
 }
 $81 = $c0total;
 $82 = $total;
 $83 = $82 >> 1;
 $84 = (($81) + ($83))|0;
 $85 = $total;
 $86 = (($84|0) / ($85|0))&-1;
 $87 = $86&255;
 $88 = $2;
 $89 = $0;
 $90 = (($89) + 136|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = HEAP32[$91>>2]|0;
 $93 = (($92) + ($88)|0);
 HEAP8[$93>>0] = $87;
 $94 = $c1total;
 $95 = $total;
 $96 = $95 >> 1;
 $97 = (($94) + ($96))|0;
 $98 = $total;
 $99 = (($97|0) / ($98|0))&-1;
 $100 = $99&255;
 $101 = $2;
 $102 = $0;
 $103 = (($102) + 136|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = (($104) + 4|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = (($106) + ($101)|0);
 HEAP8[$107>>0] = $100;
 $108 = $c2total;
 $109 = $total;
 $110 = $109 >> 1;
 $111 = (($108) + ($110))|0;
 $112 = $total;
 $113 = (($111|0) / ($112|0))&-1;
 $114 = $113&255;
 $115 = $2;
 $116 = $0;
 $117 = (($116) + 136|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = (($118) + 8|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = (($120) + ($115)|0);
 HEAP8[$121>>0] = $114;
 STACKTOP = sp;return;
}
function _find_biggest_color_pop($boxlist,$numboxes) {
 $boxlist = $boxlist|0;
 $numboxes = $numboxes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $boxp = 0, $i = 0, $maxc = 0, $which = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $boxlist;
 $1 = $numboxes;
 $maxc = 0;
 $which = 0;
 $i = 0;
 $2 = $0;
 $boxp = $2;
 while(1) {
  $3 = $i;
  $4 = $1;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $boxp;
  $7 = (($6) + 28|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $maxc;
  $10 = ($8|0)>($9|0);
  if ($10) {
   $11 = $boxp;
   $12 = (($11) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)>(0);
   if ($14) {
    $15 = $boxp;
    $which = $15;
    $16 = $boxp;
    $17 = (($16) + 28|0);
    $18 = HEAP32[$17>>2]|0;
    $maxc = $18;
   }
  }
  $19 = $i;
  $20 = (($19) + 1)|0;
  $i = $20;
  $21 = $boxp;
  $22 = (($21) + 32|0);
  $boxp = $22;
 }
 $23 = $which;
 STACKTOP = sp;return ($23|0);
}
function _find_biggest_volume($boxlist,$numboxes) {
 $boxlist = $boxlist|0;
 $numboxes = $numboxes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $boxp = 0, $i = 0, $maxv = 0, $which = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $boxlist;
 $1 = $numboxes;
 $maxv = 0;
 $which = 0;
 $i = 0;
 $2 = $0;
 $boxp = $2;
 while(1) {
  $3 = $i;
  $4 = $1;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $boxp;
  $7 = (($6) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $maxv;
  $10 = ($8|0)>($9|0);
  if ($10) {
   $11 = $boxp;
   $which = $11;
   $12 = $boxp;
   $13 = (($12) + 24|0);
   $14 = HEAP32[$13>>2]|0;
   $maxv = $14;
  }
  $15 = $i;
  $16 = (($15) + 1)|0;
  $i = $16;
  $17 = $boxp;
  $18 = (($17) + 32|0);
  $boxp = $18;
 }
 $19 = $which;
 STACKTOP = sp;return ($19|0);
}
function _jdiv_round_up($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = (($2) + ($3))|0;
 $5 = (($4) - 1)|0;
 $6 = $1;
 $7 = (($5|0) / ($6|0))&-1;
 STACKTOP = sp;return ($7|0);
}
function _jround_up($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $1;
 $3 = (($2) - 1)|0;
 $4 = $0;
 $5 = (($4) + ($3))|0;
 $0 = $5;
 $6 = $0;
 $7 = $0;
 $8 = $1;
 $9 = (($7|0) % ($8|0))&-1;
 $10 = (($6) - ($9))|0;
 STACKTOP = sp;return ($10|0);
}
function _jcopy_sample_rows($input_array,$source_row,$output_array,$dest_row,$num_rows,$num_cols) {
 $input_array = $input_array|0;
 $source_row = $source_row|0;
 $output_array = $output_array|0;
 $dest_row = $dest_row|0;
 $num_rows = $num_rows|0;
 $num_cols = $num_cols|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $count = 0, $inptr = 0, $outptr = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $input_array;
 $1 = $source_row;
 $2 = $output_array;
 $3 = $dest_row;
 $4 = $num_rows;
 $5 = $num_cols;
 $6 = $5;
 $7 = $6;
 $count = $7;
 $8 = $1;
 $9 = $0;
 $10 = (($9) + ($8<<2)|0);
 $0 = $10;
 $11 = $3;
 $12 = $2;
 $13 = (($12) + ($11<<2)|0);
 $2 = $13;
 $14 = $4;
 $row = $14;
 while(1) {
  $15 = $row;
  $16 = ($15|0)>(0);
  if (!($16)) {
   break;
  }
  $17 = $0;
  $18 = (($17) + 4|0);
  $0 = $18;
  $19 = HEAP32[$17>>2]|0;
  $inptr = $19;
  $20 = $2;
  $21 = (($20) + 4|0);
  $2 = $21;
  $22 = HEAP32[$20>>2]|0;
  $outptr = $22;
  $23 = $outptr;
  $24 = $inptr;
  $25 = $count;
  _memcpy(($23|0),($24|0),($25|0))|0;
  $26 = $row;
  $27 = (($26) + -1)|0;
  $row = $27;
 }
 STACKTOP = sp;return;
}
function _jcopy_block_row($input_row,$output_row,$num_blocks) {
 $input_row = $input_row|0;
 $output_row = $output_row|0;
 $num_blocks = $num_blocks|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $input_row;
 $1 = $output_row;
 $2 = $num_blocks;
 $3 = $1;
 $4 = $0;
 $5 = $2;
 $6 = $5<<7;
 _memcpy(($3|0),($4|0),($6|0))|0;
 STACKTOP = sp;return;
}
function _jzero_far($target,$bytestozero) {
 $target = $target|0;
 $bytestozero = $bytestozero|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $target;
 $1 = $bytestozero;
 $2 = $0;
 $3 = $1;
 _memset(($2|0),0,($3|0))|0;
 STACKTOP = sp;return;
}
function _jpeg_membuf_src($cinfo,$in_buffer,$in_buffer_size) {
 $cinfo = $cinfo|0;
 $in_buffer = $in_buffer|0;
 $in_buffer_size = $in_buffer_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $in_buffer;
 $2 = $in_buffer_size;
 $3 = $0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = $0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = (FUNCTION_TABLE_iiii[$10 & 255]($11,0,36)|0);
  $13 = $0;
  $14 = (($13) + 24|0);
  HEAP32[$14>>2] = $12;
 }
 $15 = $0;
 $16 = (($15) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $src = $17;
 $18 = $src;
 $19 = (($18) + 8|0);
 HEAP32[$19>>2] = 100;
 $20 = $src;
 $21 = (($20) + 12|0);
 HEAP32[$21>>2] = 101;
 $22 = $src;
 $23 = (($22) + 16|0);
 HEAP32[$23>>2] = 102;
 $24 = $src;
 $25 = (($24) + 20|0);
 HEAP32[$25>>2] = 103;
 $26 = $src;
 $27 = (($26) + 24|0);
 HEAP32[$27>>2] = 104;
 $28 = $1;
 $29 = $src;
 $30 = (($29) + 28|0);
 HEAP32[$30>>2] = $28;
 $31 = $2;
 $32 = $src;
 $33 = (($32) + 32|0);
 HEAP32[$33>>2] = $31;
 STACKTOP = sp;return;
}
function _init_source1874($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $src = $3;
 $4 = $src;
 $5 = (($4) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $src;
 HEAP32[$7>>2] = $6;
 $8 = $src;
 $9 = (($8) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $src;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 STACKTOP = sp;return;
}
function _fill_input_buffer1875($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 HEAP32[$3>>2] = 67;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 FUNCTION_TABLE_vi[$6 & 255]($7);
 STACKTOP = sp;return 1;
}
function _skip_input_data1876($cinfo,$num_bytes) {
 $cinfo = $cinfo|0;
 $num_bytes = $num_bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $src = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $num_bytes;
 $2 = $0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $src = $4;
 $5 = $1;
 $6 = $src;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5>>>0)>($8>>>0);
 if ($9) {
  $10 = $0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 20|0);
  HEAP32[$12>>2] = 23;
  $13 = $0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = $0;
  FUNCTION_TABLE_vi[$15 & 255]($16);
 }
 $17 = $1;
 $18 = $src;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + ($17)|0);
 HEAP32[$18>>2] = $20;
 $21 = $1;
 $22 = $src;
 $23 = (($22) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) - ($21))|0;
 HEAP32[$23>>2] = $25;
 STACKTOP = sp;return;
}
function _term_source1877($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _get_ppi_jpegb($oppi,$cinfo) {
 $oppi = $oppi|0;
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ppi = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $oppi;
 $2 = $cinfo;
 $3 = $2;
 $4 = (($3) + 286|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 if ((($6|0) == 1)) {
  $7 = $2;
  $8 = (($7) + 288|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9&65535;
  $ppi = $10;
 } else if ((($6|0) == 0)) {
  $ppi = -1;
 } else if ((($6|0) == 2)) {
  $11 = $2;
  $12 = (($11) + 288|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  $15 = (+($14|0));
  $16 = $15 * 2.54000000000000003553;
  $17 = $16 + 0.5;
  $18 = (~~(($17)));
  $ppi = $18;
 } else {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(50824|0),($vararg_buffer|0))|0);
  $20 = HEAP32[_stderr>>2]|0;
  $21 = $2;
  $22 = (($21) + 286|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  HEAP32[$vararg_buffer1>>2] = $24;
  (_fprintf(($20|0),(50856|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $27 = $0;
  STACKTOP = sp;return ($27|0);
 }
 $25 = $ppi;
 $26 = $1;
 HEAP32[$26>>2] = $25;
 $0 = 0;
 $27 = $0;
 STACKTOP = sp;return ($27|0);
}
function _jpeg_abort($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pool = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $pool = 1;
 while(1) {
  $5 = $pool;
  $6 = ($5|0)>(0);
  if (!($6)) {
   break;
  }
  $7 = $0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 36|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $0;
  $13 = $pool;
  FUNCTION_TABLE_vii[$11 & 255]($12,$13);
  $14 = $pool;
  $15 = (($14) + -1)|0;
  $pool = $15;
 }
 $16 = $0;
 $17 = (($16) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = $0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = 200;
  $22 = $0;
  $23 = (($22) + 304|0);
  HEAP32[$23>>2] = 0;
  STACKTOP = sp;return;
 } else {
  $24 = $0;
  $25 = (($24) + 20|0);
  HEAP32[$25>>2] = 100;
  STACKTOP = sp;return;
 }
}
function _jpeg_destroy($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) + 40|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $0;
  FUNCTION_TABLE_vi[$9 & 255]($10);
 }
 $11 = $0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = $0;
 $14 = (($13) + 20|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _jpeg_alloc_quant_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,0,132)|0);
 $tbl = $6;
 $7 = $tbl;
 $8 = (($7) + 128|0);
 HEAP32[$8>>2] = 0;
 $9 = $tbl;
 STACKTOP = sp;return ($9|0);
}
function _jpeg_alloc_huff_table($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tbl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,0,280)|0);
 $tbl = $6;
 $7 = $tbl;
 $8 = (($7) + 276|0);
 HEAP32[$8>>2] = 0;
 $9 = $tbl;
 STACKTOP = sp;return ($9|0);
}
function _jinit_d_coef_controller($cinfo,$need_full_buffer) {
 $cinfo = $cinfo|0;
 $need_full_buffer = $need_full_buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $access_rows = 0, $buffer = 0, $ci = 0;
 var $coef = 0, $compptr = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $need_full_buffer;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 255]($6,1,116)|0);
 $coef = $7;
 $8 = $coef;
 $9 = $0;
 $10 = (($9) + 428|0);
 HEAP32[$10>>2] = $8;
 $11 = $coef;
 HEAP32[$11>>2] = 105;
 $12 = $coef;
 $13 = (($12) + 8|0);
 HEAP32[$13>>2] = 106;
 $14 = $coef;
 $15 = (($14) + 112|0);
 HEAP32[$15>>2] = 0;
 $16 = $1;
 $17 = ($16|0)!=(0);
 if (!($17)) {
  $73 = $0;
  $74 = (($73) + 4|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 4|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = $0;
  $79 = (FUNCTION_TABLE_iiii[$77 & 255]($78,1,1280)|0);
  $buffer = $79;
  $i = 0;
  while(1) {
   $80 = $i;
   $81 = ($80|0)<(10);
   if (!($81)) {
    break;
   }
   $82 = $buffer;
   $83 = $i;
   $84 = (($82) + ($83<<7)|0);
   $85 = $i;
   $86 = $coef;
   $87 = (($86) + 32|0);
   $88 = (($87) + ($85<<2)|0);
   HEAP32[$88>>2] = $84;
   $89 = $i;
   $90 = (($89) + 1)|0;
   $i = $90;
  }
  $91 = $coef;
  $92 = (($91) + 4|0);
  HEAP32[$92>>2] = 109;
  $93 = $coef;
  $94 = (($93) + 12|0);
  HEAP32[$94>>2] = 110;
  $95 = $coef;
  $96 = (($95) + 16|0);
  HEAP32[$96>>2] = 0;
  STACKTOP = sp;return;
 }
 $ci = 0;
 $18 = $0;
 $19 = (($18) + 216|0);
 $20 = HEAP32[$19>>2]|0;
 $compptr = $20;
 while(1) {
  $21 = $ci;
  $22 = $0;
  $23 = (($22) + 36|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($21|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = $compptr;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $access_rows = $28;
  $29 = $0;
  $30 = (($29) + 220|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = $access_rows;
   $34 = ($33*3)|0;
   $access_rows = $34;
  }
  $35 = $0;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + 20|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = $0;
  $41 = $compptr;
  $42 = (($41) + 28|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $compptr;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (_jround_up($43,$46)|0);
  $48 = $compptr;
  $49 = (($48) + 32|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = $compptr;
  $52 = (($51) + 12|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_jround_up($50,$53)|0);
  $55 = $access_rows;
  $56 = (FUNCTION_TABLE_iiiiiii[$39 & 127]($40,1,1,$47,$54,$55)|0);
  $57 = $ci;
  $58 = $coef;
  $59 = (($58) + 72|0);
  $60 = (($59) + ($57<<2)|0);
  HEAP32[$60>>2] = $56;
  $61 = $ci;
  $62 = (($61) + 1)|0;
  $ci = $62;
  $63 = $compptr;
  $64 = (($63) + 84|0);
  $compptr = $64;
 }
 $65 = $coef;
 $66 = (($65) + 4|0);
 HEAP32[$66>>2] = 107;
 $67 = $coef;
 $68 = (($67) + 12|0);
 HEAP32[$68>>2] = 108;
 $69 = $coef;
 $70 = (($69) + 72|0);
 $71 = $coef;
 $72 = (($71) + 16|0);
 HEAP32[$72>>2] = $70;
 STACKTOP = sp;return;
}
function _start_input_pass1892($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 148|0);
 HEAP32[$2>>2] = 0;
 $3 = $0;
 _start_iMCU_row($3);
 STACKTOP = sp;return;
}
function _start_output_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $coef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 428|0);
 $3 = HEAP32[$2>>2]|0;
 $coef = $3;
 $4 = $coef;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $19 = $0;
  $20 = (($19) + 156|0);
  HEAP32[$20>>2] = 0;
  STACKTOP = sp;return;
 }
 $8 = $0;
 $9 = (($8) + 80|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $0;
  $13 = (_smoothing_ok($12)|0);
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = $coef;
   $16 = (($15) + 12|0);
   HEAP32[$16>>2] = 111;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $17 = $coef;
  $18 = (($17) + 12|0);
  HEAP32[$18>>2] = 108;
 }
 $19 = $0;
 $20 = (($19) + 156|0);
 HEAP32[$20>>2] = 0;
 STACKTOP = sp;return;
}
function _consume_data($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $MCU_col_num = 0, $blkn = 0, $buffer = 0, $buffer_ptr = 0, $ci = 0, $coef = 0, $compptr = 0, $start_col = 0, $xindex = 0, $yindex = 0, $yoffset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $buffer = sp + 8|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 428|0);
 $4 = HEAP32[$3>>2]|0;
 $coef = $4;
 $ci = 0;
 while(1) {
  $5 = $ci;
  $6 = $1;
  $7 = (($6) + 328|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $ci;
  $11 = $1;
  $12 = (($11) + 332|0);
  $13 = (($12) + ($10<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $compptr = $14;
  $15 = $1;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $1;
  $21 = $compptr;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = $coef;
  $25 = (($24) + 72|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $1;
  $29 = (($28) + 148|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $compptr;
  $32 = (($31) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = Math_imul($30, $33)|0;
  $35 = $compptr;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (FUNCTION_TABLE_iiiiii[$19 & 127]($20,$27,$34,$37,1)|0);
  $39 = $ci;
  $40 = (($buffer) + ($39<<2)|0);
  HEAP32[$40>>2] = $38;
  $41 = $ci;
  $42 = (($41) + 1)|0;
  $ci = $42;
 }
 $43 = $coef;
 $44 = (($43) + 24|0);
 $45 = HEAP32[$44>>2]|0;
 $yoffset = $45;
 L6: while(1) {
  $46 = $yoffset;
  $47 = $coef;
  $48 = (($47) + 28|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($46|0)<($49|0);
  if (!($50)) {
   break;
  }
  $51 = $coef;
  $52 = (($51) + 20|0);
  $53 = HEAP32[$52>>2]|0;
  $MCU_col_num = $53;
  while(1) {
   $54 = $MCU_col_num;
   $55 = $1;
   $56 = (($55) + 348|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = ($54>>>0)<($57>>>0);
   if (!($58)) {
    break;
   }
   $blkn = 0;
   $ci = 0;
   while(1) {
    $59 = $ci;
    $60 = $1;
    $61 = (($60) + 328|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($59|0)<($62|0);
    if (!($63)) {
     break;
    }
    $64 = $ci;
    $65 = $1;
    $66 = (($65) + 332|0);
    $67 = (($66) + ($64<<2)|0);
    $68 = HEAP32[$67>>2]|0;
    $compptr = $68;
    $69 = $MCU_col_num;
    $70 = $compptr;
    $71 = (($70) + 52|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = Math_imul($69, $72)|0;
    $start_col = $73;
    $yindex = 0;
    while(1) {
     $74 = $yindex;
     $75 = $compptr;
     $76 = (($75) + 56|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = ($74|0)<($77|0);
     if (!($78)) {
      break;
     }
     $79 = $yindex;
     $80 = $yoffset;
     $81 = (($79) + ($80))|0;
     $82 = $ci;
     $83 = (($buffer) + ($82<<2)|0);
     $84 = HEAP32[$83>>2]|0;
     $85 = (($84) + ($81<<2)|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = $start_col;
     $88 = (($86) + ($87<<7)|0);
     $buffer_ptr = $88;
     $xindex = 0;
     while(1) {
      $89 = $xindex;
      $90 = $compptr;
      $91 = (($90) + 52|0);
      $92 = HEAP32[$91>>2]|0;
      $93 = ($89|0)<($92|0);
      if (!($93)) {
       break;
      }
      $94 = $buffer_ptr;
      $95 = (($94) + 128|0);
      $buffer_ptr = $95;
      $96 = $blkn;
      $97 = (($96) + 1)|0;
      $blkn = $97;
      $98 = $coef;
      $99 = (($98) + 32|0);
      $100 = (($99) + ($96<<2)|0);
      HEAP32[$100>>2] = $94;
      $101 = $xindex;
      $102 = (($101) + 1)|0;
      $xindex = $102;
     }
     $103 = $yindex;
     $104 = (($103) + 1)|0;
     $yindex = $104;
    }
    $105 = $ci;
    $106 = (($105) + 1)|0;
    $ci = $106;
   }
   $107 = $1;
   $108 = (($107) + 444|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + 4|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $1;
   $113 = $coef;
   $114 = (($113) + 32|0);
   $115 = (FUNCTION_TABLE_iii[$111 & 255]($112,$114)|0);
   $116 = ($115|0)!=(0);
   if (!($116)) {
    label = 22;
    break L6;
   }
   $123 = $MCU_col_num;
   $124 = (($123) + 1)|0;
   $MCU_col_num = $124;
  }
  $125 = $coef;
  $126 = (($125) + 20|0);
  HEAP32[$126>>2] = 0;
  $127 = $yoffset;
  $128 = (($127) + 1)|0;
  $yoffset = $128;
 }
 if ((label|0) == 22) {
  $117 = $yoffset;
  $118 = $coef;
  $119 = (($118) + 24|0);
  HEAP32[$119>>2] = $117;
  $120 = $MCU_col_num;
  $121 = $coef;
  $122 = (($121) + 20|0);
  HEAP32[$122>>2] = $120;
  $0 = 0;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 $129 = $1;
 $130 = (($129) + 148|0);
 $131 = HEAP32[$130>>2]|0;
 $132 = (($131) + 1)|0;
 HEAP32[$130>>2] = $132;
 $133 = $1;
 $134 = (($133) + 320|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = ($132>>>0)<($135>>>0);
 if ($136) {
  $137 = $1;
  _start_iMCU_row($137);
  $0 = 3;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 } else {
  $138 = $1;
  $139 = (($138) + 436|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = (($140) + 12|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = $1;
  FUNCTION_TABLE_vi[$142 & 255]($143);
  $0 = 4;
  $144 = $0;
  STACKTOP = sp;return ($144|0);
 }
 return 0|0;
}
function _decompress_data($cinfo,$output_buf) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $block_num = 0, $block_row = 0, $block_rows = 0, $buffer = 0, $buffer_ptr = 0, $ci = 0, $coef = 0, $compptr = 0;
 var $inverse_DCT = 0, $last_iMCU_row = 0, $output_col = 0, $output_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $cinfo;
 $2 = $output_buf;
 $3 = $1;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$4>>2]|0;
 $coef = $5;
 $6 = $1;
 $7 = (($6) + 320|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $last_iMCU_row = $9;
 while(1) {
  $10 = $1;
  $11 = (($10) + 144|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $1;
  $14 = (($13) + 152|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if ($16) {
   $150 = 1;
  } else {
   $17 = $1;
   $18 = (($17) + 144|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = $1;
   $21 = (($20) + 152|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)==($22|0);
   if ($23) {
    $24 = $1;
    $25 = (($24) + 148|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = $1;
    $28 = (($27) + 156|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($26>>>0)<=($29>>>0);
    $151 = $30;
   } else {
    $151 = 0;
   }
   $150 = $151;
  }
  if (!($150)) {
   break;
  }
  $31 = $1;
  $32 = (($31) + 436|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $1;
  $36 = (FUNCTION_TABLE_ii[$34 & 127]($35)|0);
  $37 = ($36|0)==(0);
  if ($37) {
   label = 8;
   break;
  }
 }
 if ((label|0) == 8) {
  $0 = 0;
  $149 = $0;
  STACKTOP = sp;return ($149|0);
 }
 $ci = 0;
 $38 = $1;
 $39 = (($38) + 216|0);
 $40 = HEAP32[$39>>2]|0;
 $compptr = $40;
 while(1) {
  $41 = $ci;
  $42 = $1;
  $43 = (($42) + 36|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ($41|0)<($44|0);
  if (!($45)) {
   break;
  }
  $46 = $compptr;
  $47 = (($46) + 48|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)!=(0);
  if ($49) {
   $50 = $1;
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 32|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = $1;
   $56 = $ci;
   $57 = $coef;
   $58 = (($57) + 72|0);
   $59 = (($58) + ($56<<2)|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $1;
   $62 = (($61) + 156|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $compptr;
   $65 = (($64) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = Math_imul($63, $66)|0;
   $68 = $compptr;
   $69 = (($68) + 12|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (FUNCTION_TABLE_iiiiii[$54 & 127]($55,$60,$67,$70,0)|0);
   $buffer = $71;
   $72 = $1;
   $73 = (($72) + 156|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = $last_iMCU_row;
   $76 = ($74>>>0)<($75>>>0);
   if ($76) {
    $77 = $compptr;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $block_rows = $79;
   } else {
    $80 = $compptr;
    $81 = (($80) + 32|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = $compptr;
    $84 = (($83) + 12|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = (($82>>>0) % ($85>>>0))&-1;
    $block_rows = $86;
    $87 = $block_rows;
    $88 = ($87|0)==(0);
    if ($88) {
     $89 = $compptr;
     $90 = (($89) + 12|0);
     $91 = HEAP32[$90>>2]|0;
     $block_rows = $91;
    }
   }
   $92 = $ci;
   $93 = $1;
   $94 = (($93) + 448|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (($95) + 4|0);
   $97 = (($96) + ($92<<2)|0);
   $98 = HEAP32[$97>>2]|0;
   $inverse_DCT = $98;
   $99 = $ci;
   $100 = $2;
   $101 = (($100) + ($99<<2)|0);
   $102 = HEAP32[$101>>2]|0;
   $output_ptr = $102;
   $block_row = 0;
   while(1) {
    $103 = $block_row;
    $104 = $block_rows;
    $105 = ($103|0)<($104|0);
    if (!($105)) {
     break;
    }
    $106 = $block_row;
    $107 = $buffer;
    $108 = (($107) + ($106<<2)|0);
    $109 = HEAP32[$108>>2]|0;
    $buffer_ptr = $109;
    $output_col = 0;
    $block_num = 0;
    while(1) {
     $110 = $block_num;
     $111 = $compptr;
     $112 = (($111) + 28|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = ($110>>>0)<($113>>>0);
     if (!($114)) {
      break;
     }
     $115 = $inverse_DCT;
     $116 = $1;
     $117 = $compptr;
     $118 = $buffer_ptr;
     $119 = $output_ptr;
     $120 = $output_col;
     FUNCTION_TABLE_viiiii[$115 & 127]($116,$117,$118,$119,$120);
     $121 = $buffer_ptr;
     $122 = (($121) + 128|0);
     $buffer_ptr = $122;
     $123 = $compptr;
     $124 = (($123) + 36|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = $output_col;
     $127 = (($126) + ($125))|0;
     $output_col = $127;
     $128 = $block_num;
     $129 = (($128) + 1)|0;
     $block_num = $129;
    }
    $130 = $compptr;
    $131 = (($130) + 36|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = $output_ptr;
    $134 = (($133) + ($132<<2)|0);
    $output_ptr = $134;
    $135 = $block_row;
    $136 = (($135) + 1)|0;
    $block_row = $136;
   }
  } else {
  }
  $137 = $ci;
  $138 = (($137) + 1)|0;
  $ci = $138;
  $139 = $compptr;
  $140 = (($139) + 84|0);
  $compptr = $140;
 }
 $141 = $1;
 $142 = (($141) + 156|0);
 $143 = HEAP32[$142>>2]|0;
 $144 = (($143) + 1)|0;
 HEAP32[$142>>2] = $144;
 $145 = $1;
 $146 = (($145) + 320|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = ($144>>>0)<($147>>>0);
 if ($148) {
  $0 = 3;
  $149 = $0;
  STACKTOP = sp;return ($149|0);
 } else {
  $0 = 4;
  $149 = $0;
  STACKTOP = sp;return ($149|0);
 }
 return 0|0;
}
function _dummy_consume_data($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return 0;
}
function _decompress_onepass($cinfo,$output_buf) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $MCU_col_num = 0, $blkn = 0, $ci = 0, $coef = 0, $compptr = 0, $inverse_DCT = 0, $last_MCU_col = 0, $last_iMCU_row = 0, $output_col = 0, $output_ptr = 0, $start_col = 0, $useful_width = 0, $xindex = 0, $yindex = 0;
 var $yoffset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $cinfo;
 $2 = $output_buf;
 $3 = $1;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$4>>2]|0;
 $coef = $5;
 $6 = $1;
 $7 = (($6) + 348|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $last_MCU_col = $9;
 $10 = $1;
 $11 = (($10) + 320|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) - 1)|0;
 $last_iMCU_row = $13;
 $14 = $coef;
 $15 = (($14) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $yoffset = $16;
 L1: while(1) {
  $17 = $yoffset;
  $18 = $coef;
  $19 = (($18) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = $coef;
  $23 = (($22) + 20|0);
  $24 = HEAP32[$23>>2]|0;
  $MCU_col_num = $24;
  while(1) {
   $25 = $MCU_col_num;
   $26 = $last_MCU_col;
   $27 = ($25>>>0)<=($26>>>0);
   if (!($27)) {
    break;
   }
   $28 = $coef;
   $29 = (($28) + 32|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $1;
   $32 = (($31) + 356|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $33<<7;
   _jzero_far($30,$34);
   $35 = $1;
   $36 = (($35) + 444|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $1;
   $41 = $coef;
   $42 = (($41) + 32|0);
   $43 = (FUNCTION_TABLE_iii[$39 & 255]($40,$42)|0);
   $44 = ($43|0)!=(0);
   if (!($44)) {
    label = 6;
    break L1;
   }
   $blkn = 0;
   $ci = 0;
   while(1) {
    $51 = $ci;
    $52 = $1;
    $53 = (($52) + 328|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = ($51|0)<($54|0);
    if (!($55)) {
     break;
    }
    $56 = $ci;
    $57 = $1;
    $58 = (($57) + 332|0);
    $59 = (($58) + ($56<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $compptr = $60;
    $61 = $compptr;
    $62 = (($61) + 48|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = ($63|0)!=(0);
    if ($64) {
     $70 = $compptr;
     $71 = (($70) + 4|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = $1;
     $74 = (($73) + 448|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + 4|0);
     $77 = (($76) + ($72<<2)|0);
     $78 = HEAP32[$77>>2]|0;
     $inverse_DCT = $78;
     $79 = $MCU_col_num;
     $80 = $last_MCU_col;
     $81 = ($79>>>0)<($80>>>0);
     if ($81) {
      $82 = $compptr;
      $83 = (($82) + 52|0);
      $84 = HEAP32[$83>>2]|0;
      $88 = $84;
     } else {
      $85 = $compptr;
      $86 = (($85) + 68|0);
      $87 = HEAP32[$86>>2]|0;
      $88 = $87;
     }
     $useful_width = $88;
     $89 = $compptr;
     $90 = (($89) + 4|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = $2;
     $93 = (($92) + ($91<<2)|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = $yoffset;
     $96 = $compptr;
     $97 = (($96) + 36|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = Math_imul($95, $98)|0;
     $100 = (($94) + ($99<<2)|0);
     $output_ptr = $100;
     $101 = $MCU_col_num;
     $102 = $compptr;
     $103 = (($102) + 64|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = Math_imul($101, $104)|0;
     $start_col = $105;
     $yindex = 0;
     while(1) {
      $106 = $yindex;
      $107 = $compptr;
      $108 = (($107) + 56|0);
      $109 = HEAP32[$108>>2]|0;
      $110 = ($106|0)<($109|0);
      if (!($110)) {
       break;
      }
      $111 = $1;
      $112 = (($111) + 148|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = $last_iMCU_row;
      $115 = ($113>>>0)<($114>>>0);
      if ($115) {
       label = 18;
      } else {
       $116 = $yoffset;
       $117 = $yindex;
       $118 = (($116) + ($117))|0;
       $119 = $compptr;
       $120 = (($119) + 72|0);
       $121 = HEAP32[$120>>2]|0;
       $122 = ($118|0)<($121|0);
       if ($122) {
        label = 18;
       }
      }
      if ((label|0) == 18) {
       label = 0;
       $123 = $start_col;
       $output_col = $123;
       $xindex = 0;
       while(1) {
        $124 = $xindex;
        $125 = $useful_width;
        $126 = ($124|0)<($125|0);
        if (!($126)) {
         break;
        }
        $127 = $inverse_DCT;
        $128 = $1;
        $129 = $compptr;
        $130 = $blkn;
        $131 = $xindex;
        $132 = (($130) + ($131))|0;
        $133 = $coef;
        $134 = (($133) + 32|0);
        $135 = (($134) + ($132<<2)|0);
        $136 = HEAP32[$135>>2]|0;
        $137 = $output_ptr;
        $138 = $output_col;
        FUNCTION_TABLE_viiiii[$127 & 127]($128,$129,$136,$137,$138);
        $139 = $compptr;
        $140 = (($139) + 36|0);
        $141 = HEAP32[$140>>2]|0;
        $142 = $output_col;
        $143 = (($142) + ($141))|0;
        $output_col = $143;
        $144 = $xindex;
        $145 = (($144) + 1)|0;
        $xindex = $145;
       }
      }
      $146 = $compptr;
      $147 = (($146) + 52|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = $blkn;
      $150 = (($149) + ($148))|0;
      $blkn = $150;
      $151 = $compptr;
      $152 = (($151) + 36|0);
      $153 = HEAP32[$152>>2]|0;
      $154 = $output_ptr;
      $155 = (($154) + ($153<<2)|0);
      $output_ptr = $155;
      $156 = $yindex;
      $157 = (($156) + 1)|0;
      $yindex = $157;
     }
    } else {
     $65 = $compptr;
     $66 = (($65) + 60|0);
     $67 = HEAP32[$66>>2]|0;
     $68 = $blkn;
     $69 = (($68) + ($67))|0;
     $blkn = $69;
    }
    $158 = $ci;
    $159 = (($158) + 1)|0;
    $ci = $159;
   }
   $160 = $MCU_col_num;
   $161 = (($160) + 1)|0;
   $MCU_col_num = $161;
  }
  $162 = $coef;
  $163 = (($162) + 20|0);
  HEAP32[$163>>2] = 0;
  $164 = $yoffset;
  $165 = (($164) + 1)|0;
  $yoffset = $165;
 }
 if ((label|0) == 6) {
  $45 = $yoffset;
  $46 = $coef;
  $47 = (($46) + 24|0);
  HEAP32[$47>>2] = $45;
  $48 = $MCU_col_num;
  $49 = $coef;
  $50 = (($49) + 20|0);
  HEAP32[$50>>2] = $48;
  $0 = 0;
  $185 = $0;
  STACKTOP = sp;return ($185|0);
 }
 $166 = $1;
 $167 = (($166) + 156|0);
 $168 = HEAP32[$167>>2]|0;
 $169 = (($168) + 1)|0;
 HEAP32[$167>>2] = $169;
 $170 = $1;
 $171 = (($170) + 148|0);
 $172 = HEAP32[$171>>2]|0;
 $173 = (($172) + 1)|0;
 HEAP32[$171>>2] = $173;
 $174 = $1;
 $175 = (($174) + 320|0);
 $176 = HEAP32[$175>>2]|0;
 $177 = ($173>>>0)<($176>>>0);
 if ($177) {
  $178 = $1;
  _start_iMCU_row($178);
  $0 = 3;
  $185 = $0;
  STACKTOP = sp;return ($185|0);
 } else {
  $179 = $1;
  $180 = (($179) + 436|0);
  $181 = HEAP32[$180>>2]|0;
  $182 = (($181) + 12|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = $1;
  FUNCTION_TABLE_vi[$183 & 255]($184);
  $0 = 4;
  $185 = $0;
  STACKTOP = sp;return ($185|0);
 }
 return 0|0;
}
function _start_iMCU_row($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $coef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 428|0);
 $3 = HEAP32[$2>>2]|0;
 $coef = $3;
 $4 = $0;
 $5 = (($4) + 328|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>(1);
 if ($7) {
  $8 = $coef;
  $9 = (($8) + 28|0);
  HEAP32[$9>>2] = 1;
  $32 = $coef;
  $33 = (($32) + 20|0);
  HEAP32[$33>>2] = 0;
  $34 = $coef;
  $35 = (($34) + 24|0);
  HEAP32[$35>>2] = 0;
  STACKTOP = sp;return;
 }
 $10 = $0;
 $11 = (($10) + 148|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 320|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = ($12>>>0)<($16>>>0);
 if ($17) {
  $18 = $0;
  $19 = (($18) + 332|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $coef;
  $24 = (($23) + 28|0);
  HEAP32[$24>>2] = $22;
 } else {
  $25 = $0;
  $26 = (($25) + 332|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 72|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $coef;
  $31 = (($30) + 28|0);
  HEAP32[$31>>2] = $29;
 }
 $32 = $coef;
 $33 = (($32) + 20|0);
 HEAP32[$33>>2] = 0;
 $34 = $coef;
 $35 = (($34) + 24|0);
 HEAP32[$35>>2] = 0;
 STACKTOP = sp;return;
}
function _smoothing_ok($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0, $coef = 0, $coef_bits = 0, $coef_bits_latch = 0, $coefi = 0, $compptr = 0, $qtable = 0, $smoothing_useful = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 428|0);
 $4 = HEAP32[$3>>2]|0;
 $coef = $4;
 $smoothing_useful = 0;
 $5 = $1;
 $6 = (($5) + 220|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $1;
  $10 = (($9) + 160|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)==(0|0);
  if (!($12)) {
   $13 = $coef;
   $14 = (($13) + 112|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)==(0|0);
   if ($16) {
    $17 = $1;
    $18 = (($17) + 4|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = HEAP32[$19>>2]|0;
    $21 = $1;
    $22 = $1;
    $23 = (($22) + 36|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($24*24)|0;
    $26 = (FUNCTION_TABLE_iiii[$20 & 255]($21,1,$25)|0);
    $27 = $coef;
    $28 = (($27) + 112|0);
    HEAP32[$28>>2] = $26;
   }
   $29 = $coef;
   $30 = (($29) + 112|0);
   $31 = HEAP32[$30>>2]|0;
   $coef_bits_latch = $31;
   $ci = 0;
   $32 = $1;
   $33 = (($32) + 216|0);
   $34 = HEAP32[$33>>2]|0;
   $compptr = $34;
   while(1) {
    $35 = $ci;
    $36 = $1;
    $37 = (($36) + 36|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ($35|0)<($38|0);
    if (!($39)) {
     label = 27;
     break;
    }
    $40 = $compptr;
    $41 = (($40) + 76|0);
    $42 = HEAP32[$41>>2]|0;
    $qtable = $42;
    $43 = ($42|0)==(0|0);
    if ($43) {
     label = 9;
     break;
    }
    $44 = $qtable;
    $45 = HEAP16[$44>>1]|0;
    $46 = $45&65535;
    $47 = ($46|0)==(0);
    if ($47) {
     label = 16;
     break;
    }
    $48 = $qtable;
    $49 = (($48) + 2|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = ($51|0)==(0);
    if ($52) {
     label = 16;
     break;
    }
    $53 = $qtable;
    $54 = (($53) + 16|0);
    $55 = HEAP16[$54>>1]|0;
    $56 = $55&65535;
    $57 = ($56|0)==(0);
    if ($57) {
     label = 16;
     break;
    }
    $58 = $qtable;
    $59 = (($58) + 32|0);
    $60 = HEAP16[$59>>1]|0;
    $61 = $60&65535;
    $62 = ($61|0)==(0);
    if ($62) {
     label = 16;
     break;
    }
    $63 = $qtable;
    $64 = (($63) + 18|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = $65&65535;
    $67 = ($66|0)==(0);
    if ($67) {
     label = 16;
     break;
    }
    $68 = $qtable;
    $69 = (($68) + 4|0);
    $70 = HEAP16[$69>>1]|0;
    $71 = $70&65535;
    $72 = ($71|0)==(0);
    if ($72) {
     label = 16;
     break;
    }
    $73 = $ci;
    $74 = $1;
    $75 = (($74) + 160|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = (($76) + ($73<<8)|0);
    $coef_bits = $77;
    $78 = $coef_bits;
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)<(0);
    if ($80) {
     label = 18;
     break;
    }
    $coefi = 1;
    while(1) {
     $81 = $coefi;
     $82 = ($81|0)<=(5);
     if (!($82)) {
      break;
     }
     $83 = $coefi;
     $84 = $coef_bits;
     $85 = (($84) + ($83<<2)|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = $coefi;
     $88 = $coef_bits_latch;
     $89 = (($88) + ($87<<2)|0);
     HEAP32[$89>>2] = $86;
     $90 = $coefi;
     $91 = $coef_bits;
     $92 = (($91) + ($90<<2)|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = ($93|0)!=(0);
     if ($94) {
      $smoothing_useful = 1;
     }
     $95 = $coefi;
     $96 = (($95) + 1)|0;
     $coefi = $96;
    }
    $97 = $coef_bits_latch;
    $98 = (($97) + 24|0);
    $coef_bits_latch = $98;
    $99 = $ci;
    $100 = (($99) + 1)|0;
    $ci = $100;
    $101 = $compptr;
    $102 = (($101) + 84|0);
    $compptr = $102;
   }
   if ((label|0) == 9) {
    $0 = 0;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   else if ((label|0) == 16) {
    $0 = 0;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   else if ((label|0) == 18) {
    $0 = 0;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   else if ((label|0) == 27) {
    $103 = $smoothing_useful;
    $0 = $103;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
  }
 }
 $0 = 0;
 $104 = $0;
 STACKTOP = sp;return ($104|0);
}
function _decompress_smooth_data($cinfo,$output_buf) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Al = 0;
 var $DC1 = 0, $DC2 = 0, $DC3 = 0, $DC4 = 0, $DC5 = 0, $DC6 = 0, $DC7 = 0, $DC8 = 0, $DC9 = 0, $Q00 = 0, $Q01 = 0, $Q02 = 0, $Q10 = 0, $Q11 = 0, $Q20 = 0, $access_rows = 0, $block_num = 0, $block_row = 0, $block_rows = 0, $buffer = 0;
 var $buffer_ptr = 0, $ci = 0, $coef = 0, $coef_bits = 0, $compptr = 0, $delta = 0, $first_row = 0, $inverse_DCT = 0, $last_block_column = 0, $last_iMCU_row = 0, $last_row = 0, $next_block_row = 0, $num = 0, $output_col = 0, $output_ptr = 0, $pred = 0, $prev_block_row = 0, $quanttbl = 0, $workspace = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $workspace = sp + 168|0;
 $1 = $cinfo;
 $2 = $output_buf;
 $3 = $1;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$4>>2]|0;
 $coef = $5;
 $6 = $1;
 $7 = (($6) + 320|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $last_iMCU_row = $9;
 while(1) {
  $10 = $1;
  $11 = (($10) + 144|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $1;
  $14 = (($13) + 152|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<=($15|0);
  if ($16) {
   $17 = $1;
   $18 = (($17) + 436|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0);
   $23 = $22 ^ 1;
   $558 = $23;
  } else {
   $558 = 0;
  }
  if (!($558)) {
   break;
  }
  $24 = $1;
  $25 = (($24) + 144|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $1;
  $28 = (($27) + 152|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)==($29|0);
  if ($30) {
   $31 = $1;
   $32 = (($31) + 400|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)==(0);
   $35 = $34 ? 1 : 0;
   $delta = $35;
   $36 = $1;
   $37 = (($36) + 148|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $1;
   $40 = (($39) + 156|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = $delta;
   $43 = (($41) + ($42))|0;
   $44 = ($38>>>0)>($43>>>0);
   if ($44) {
    label = 7;
    break;
   }
  }
  $45 = $1;
  $46 = (($45) + 436|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = $1;
  $50 = (FUNCTION_TABLE_ii[$48 & 127]($49)|0);
  $51 = ($50|0)==(0);
  if ($51) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 7) {
 }
 else if ((label|0) == 10) {
  $0 = 0;
  $557 = $0;
  STACKTOP = sp;return ($557|0);
 }
 $ci = 0;
 $52 = $1;
 $53 = (($52) + 216|0);
 $54 = HEAP32[$53>>2]|0;
 $compptr = $54;
 while(1) {
  $55 = $ci;
  $56 = $1;
  $57 = (($56) + 36|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ($55|0)<($58|0);
  if (!($59)) {
   break;
  }
  $60 = $compptr;
  $61 = (($60) + 48|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($62|0)!=(0);
  if ($63) {
   $64 = $1;
   $65 = (($64) + 156|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = $last_iMCU_row;
   $68 = ($66>>>0)<($67>>>0);
   if ($68) {
    $69 = $compptr;
    $70 = (($69) + 12|0);
    $71 = HEAP32[$70>>2]|0;
    $block_rows = $71;
    $72 = $block_rows;
    $73 = $72<<1;
    $access_rows = $73;
    $last_row = 0;
   } else {
    $74 = $compptr;
    $75 = (($74) + 32|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = $compptr;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = (($76>>>0) % ($79>>>0))&-1;
    $block_rows = $80;
    $81 = $block_rows;
    $82 = ($81|0)==(0);
    if ($82) {
     $83 = $compptr;
     $84 = (($83) + 12|0);
     $85 = HEAP32[$84>>2]|0;
     $block_rows = $85;
    }
    $86 = $block_rows;
    $access_rows = $86;
    $last_row = 1;
   }
   $87 = $1;
   $88 = (($87) + 156|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = ($89>>>0)>(0);
   if ($90) {
    $91 = $compptr;
    $92 = (($91) + 12|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = $access_rows;
    $95 = (($94) + ($93))|0;
    $access_rows = $95;
    $96 = $1;
    $97 = (($96) + 4|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = (($98) + 32|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $1;
    $102 = $ci;
    $103 = $coef;
    $104 = (($103) + 72|0);
    $105 = (($104) + ($102<<2)|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = $1;
    $108 = (($107) + 156|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) - 1)|0;
    $111 = $compptr;
    $112 = (($111) + 12|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = Math_imul($110, $113)|0;
    $115 = $access_rows;
    $116 = (FUNCTION_TABLE_iiiiii[$100 & 127]($101,$106,$114,$115,0)|0);
    $buffer = $116;
    $117 = $compptr;
    $118 = (($117) + 12|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = $buffer;
    $121 = (($120) + ($119<<2)|0);
    $buffer = $121;
    $first_row = 0;
   } else {
    $122 = $1;
    $123 = (($122) + 4|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = (($124) + 32|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = $1;
    $128 = $ci;
    $129 = $coef;
    $130 = (($129) + 72|0);
    $131 = (($130) + ($128<<2)|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = $access_rows;
    $134 = (FUNCTION_TABLE_iiiiii[$126 & 127]($127,$132,0,$133,0)|0);
    $buffer = $134;
    $first_row = 1;
   }
   $135 = $coef;
   $136 = (($135) + 112|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = $ci;
   $139 = ($138*6)|0;
   $140 = (($137) + ($139<<2)|0);
   $coef_bits = $140;
   $141 = $compptr;
   $142 = (($141) + 76|0);
   $143 = HEAP32[$142>>2]|0;
   $quanttbl = $143;
   $144 = $quanttbl;
   $145 = HEAP16[$144>>1]|0;
   $146 = $145&65535;
   $Q00 = $146;
   $147 = $quanttbl;
   $148 = (($147) + 2|0);
   $149 = HEAP16[$148>>1]|0;
   $150 = $149&65535;
   $Q01 = $150;
   $151 = $quanttbl;
   $152 = (($151) + 16|0);
   $153 = HEAP16[$152>>1]|0;
   $154 = $153&65535;
   $Q10 = $154;
   $155 = $quanttbl;
   $156 = (($155) + 32|0);
   $157 = HEAP16[$156>>1]|0;
   $158 = $157&65535;
   $Q20 = $158;
   $159 = $quanttbl;
   $160 = (($159) + 18|0);
   $161 = HEAP16[$160>>1]|0;
   $162 = $161&65535;
   $Q11 = $162;
   $163 = $quanttbl;
   $164 = (($163) + 4|0);
   $165 = HEAP16[$164>>1]|0;
   $166 = $165&65535;
   $Q02 = $166;
   $167 = $ci;
   $168 = $1;
   $169 = (($168) + 448|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = (($170) + 4|0);
   $172 = (($171) + ($167<<2)|0);
   $173 = HEAP32[$172>>2]|0;
   $inverse_DCT = $173;
   $174 = $ci;
   $175 = $2;
   $176 = (($175) + ($174<<2)|0);
   $177 = HEAP32[$176>>2]|0;
   $output_ptr = $177;
   $block_row = 0;
   while(1) {
    $178 = $block_row;
    $179 = $block_rows;
    $180 = ($178|0)<($179|0);
    if (!($180)) {
     break;
    }
    $181 = $block_row;
    $182 = $buffer;
    $183 = (($182) + ($181<<2)|0);
    $184 = HEAP32[$183>>2]|0;
    $buffer_ptr = $184;
    $185 = $first_row;
    $186 = ($185|0)!=(0);
    if ($186) {
     $187 = $block_row;
     $188 = ($187|0)==(0);
     if ($188) {
      $189 = $buffer_ptr;
      $prev_block_row = $189;
     } else {
      label = 29;
     }
    } else {
     label = 29;
    }
    if ((label|0) == 29) {
     label = 0;
     $190 = $block_row;
     $191 = (($190) - 1)|0;
     $192 = $buffer;
     $193 = (($192) + ($191<<2)|0);
     $194 = HEAP32[$193>>2]|0;
     $prev_block_row = $194;
    }
    $195 = $last_row;
    $196 = ($195|0)!=(0);
    if ($196) {
     $197 = $block_row;
     $198 = $block_rows;
     $199 = (($198) - 1)|0;
     $200 = ($197|0)==($199|0);
     if ($200) {
      $201 = $buffer_ptr;
      $next_block_row = $201;
     } else {
      label = 33;
     }
    } else {
     label = 33;
    }
    if ((label|0) == 33) {
     label = 0;
     $202 = $block_row;
     $203 = (($202) + 1)|0;
     $204 = $buffer;
     $205 = (($204) + ($203<<2)|0);
     $206 = HEAP32[$205>>2]|0;
     $next_block_row = $206;
    }
    $207 = $prev_block_row;
    $208 = HEAP16[$207>>1]|0;
    $209 = $208 << 16 >> 16;
    $DC3 = $209;
    $DC2 = $209;
    $DC1 = $209;
    $210 = $buffer_ptr;
    $211 = HEAP16[$210>>1]|0;
    $212 = $211 << 16 >> 16;
    $DC6 = $212;
    $DC5 = $212;
    $DC4 = $212;
    $213 = $next_block_row;
    $214 = HEAP16[$213>>1]|0;
    $215 = $214 << 16 >> 16;
    $DC9 = $215;
    $DC8 = $215;
    $DC7 = $215;
    $output_col = 0;
    $216 = $compptr;
    $217 = (($216) + 28|0);
    $218 = HEAP32[$217>>2]|0;
    $219 = (($218) - 1)|0;
    $last_block_column = $219;
    $block_num = 0;
    while(1) {
     $220 = $block_num;
     $221 = $last_block_column;
     $222 = ($220>>>0)<=($221>>>0);
     if (!($222)) {
      break;
     }
     $223 = $buffer_ptr;
     _jcopy_block_row($223,$workspace,1);
     $224 = $block_num;
     $225 = $last_block_column;
     $226 = ($224>>>0)<($225>>>0);
     if ($226) {
      $227 = $prev_block_row;
      $228 = (($227) + 128|0);
      $229 = HEAP16[$228>>1]|0;
      $230 = $229 << 16 >> 16;
      $DC3 = $230;
      $231 = $buffer_ptr;
      $232 = (($231) + 128|0);
      $233 = HEAP16[$232>>1]|0;
      $234 = $233 << 16 >> 16;
      $DC6 = $234;
      $235 = $next_block_row;
      $236 = (($235) + 128|0);
      $237 = HEAP16[$236>>1]|0;
      $238 = $237 << 16 >> 16;
      $DC9 = $238;
     }
     $239 = $coef_bits;
     $240 = (($239) + 4|0);
     $241 = HEAP32[$240>>2]|0;
     $Al = $241;
     $242 = ($241|0)!=(0);
     if ($242) {
      $243 = (($workspace) + 2|0);
      $244 = HEAP16[$243>>1]|0;
      $245 = $244 << 16 >> 16;
      $246 = ($245|0)==(0);
      if ($246) {
       $247 = $Q00;
       $248 = ($247*36)|0;
       $249 = $DC4;
       $250 = $DC6;
       $251 = (($249) - ($250))|0;
       $252 = Math_imul($248, $251)|0;
       $num = $252;
       $253 = $num;
       $254 = ($253|0)>=(0);
       if ($254) {
        $255 = $Q01;
        $256 = $255 << 7;
        $257 = $num;
        $258 = (($256) + ($257))|0;
        $259 = $Q01;
        $260 = $259 << 8;
        $261 = (($258|0) / ($260|0))&-1;
        $pred = $261;
        $262 = $Al;
        $263 = ($262|0)>(0);
        do {
         if ($263) {
          $264 = $pred;
          $265 = $Al;
          $266 = 1 << $265;
          $267 = ($264|0)>=($266|0);
          if (!($267)) {
           break;
          }
          $268 = $Al;
          $269 = 1 << $268;
          $270 = (($269) - 1)|0;
          $pred = $270;
         }
        } while(0);
       } else {
        $271 = $Q01;
        $272 = $271 << 7;
        $273 = $num;
        $274 = (($272) - ($273))|0;
        $275 = $Q01;
        $276 = $275 << 8;
        $277 = (($274|0) / ($276|0))&-1;
        $pred = $277;
        $278 = $Al;
        $279 = ($278|0)>(0);
        do {
         if ($279) {
          $280 = $pred;
          $281 = $Al;
          $282 = 1 << $281;
          $283 = ($280|0)>=($282|0);
          if (!($283)) {
           break;
          }
          $284 = $Al;
          $285 = 1 << $284;
          $286 = (($285) - 1)|0;
          $pred = $286;
         }
        } while(0);
        $287 = $pred;
        $288 = (0 - ($287))|0;
        $pred = $288;
       }
       $289 = $pred;
       $290 = $289&65535;
       $291 = (($workspace) + 2|0);
       HEAP16[$291>>1] = $290;
      }
     }
     $292 = $coef_bits;
     $293 = (($292) + 8|0);
     $294 = HEAP32[$293>>2]|0;
     $Al = $294;
     $295 = ($294|0)!=(0);
     if ($295) {
      $296 = (($workspace) + 16|0);
      $297 = HEAP16[$296>>1]|0;
      $298 = $297 << 16 >> 16;
      $299 = ($298|0)==(0);
      if ($299) {
       $300 = $Q00;
       $301 = ($300*36)|0;
       $302 = $DC2;
       $303 = $DC8;
       $304 = (($302) - ($303))|0;
       $305 = Math_imul($301, $304)|0;
       $num = $305;
       $306 = $num;
       $307 = ($306|0)>=(0);
       if ($307) {
        $308 = $Q10;
        $309 = $308 << 7;
        $310 = $num;
        $311 = (($309) + ($310))|0;
        $312 = $Q10;
        $313 = $312 << 8;
        $314 = (($311|0) / ($313|0))&-1;
        $pred = $314;
        $315 = $Al;
        $316 = ($315|0)>(0);
        do {
         if ($316) {
          $317 = $pred;
          $318 = $Al;
          $319 = 1 << $318;
          $320 = ($317|0)>=($319|0);
          if (!($320)) {
           break;
          }
          $321 = $Al;
          $322 = 1 << $321;
          $323 = (($322) - 1)|0;
          $pred = $323;
         }
        } while(0);
       } else {
        $324 = $Q10;
        $325 = $324 << 7;
        $326 = $num;
        $327 = (($325) - ($326))|0;
        $328 = $Q10;
        $329 = $328 << 8;
        $330 = (($327|0) / ($329|0))&-1;
        $pred = $330;
        $331 = $Al;
        $332 = ($331|0)>(0);
        do {
         if ($332) {
          $333 = $pred;
          $334 = $Al;
          $335 = 1 << $334;
          $336 = ($333|0)>=($335|0);
          if (!($336)) {
           break;
          }
          $337 = $Al;
          $338 = 1 << $337;
          $339 = (($338) - 1)|0;
          $pred = $339;
         }
        } while(0);
        $340 = $pred;
        $341 = (0 - ($340))|0;
        $pred = $341;
       }
       $342 = $pred;
       $343 = $342&65535;
       $344 = (($workspace) + 16|0);
       HEAP16[$344>>1] = $343;
      }
     }
     $345 = $coef_bits;
     $346 = (($345) + 12|0);
     $347 = HEAP32[$346>>2]|0;
     $Al = $347;
     $348 = ($347|0)!=(0);
     if ($348) {
      $349 = (($workspace) + 32|0);
      $350 = HEAP16[$349>>1]|0;
      $351 = $350 << 16 >> 16;
      $352 = ($351|0)==(0);
      if ($352) {
       $353 = $Q00;
       $354 = ($353*9)|0;
       $355 = $DC2;
       $356 = $DC8;
       $357 = (($355) + ($356))|0;
       $358 = $DC5;
       $359 = $358<<1;
       $360 = (($357) - ($359))|0;
       $361 = Math_imul($354, $360)|0;
       $num = $361;
       $362 = $num;
       $363 = ($362|0)>=(0);
       if ($363) {
        $364 = $Q20;
        $365 = $364 << 7;
        $366 = $num;
        $367 = (($365) + ($366))|0;
        $368 = $Q20;
        $369 = $368 << 8;
        $370 = (($367|0) / ($369|0))&-1;
        $pred = $370;
        $371 = $Al;
        $372 = ($371|0)>(0);
        do {
         if ($372) {
          $373 = $pred;
          $374 = $Al;
          $375 = 1 << $374;
          $376 = ($373|0)>=($375|0);
          if (!($376)) {
           break;
          }
          $377 = $Al;
          $378 = 1 << $377;
          $379 = (($378) - 1)|0;
          $pred = $379;
         }
        } while(0);
       } else {
        $380 = $Q20;
        $381 = $380 << 7;
        $382 = $num;
        $383 = (($381) - ($382))|0;
        $384 = $Q20;
        $385 = $384 << 8;
        $386 = (($383|0) / ($385|0))&-1;
        $pred = $386;
        $387 = $Al;
        $388 = ($387|0)>(0);
        do {
         if ($388) {
          $389 = $pred;
          $390 = $Al;
          $391 = 1 << $390;
          $392 = ($389|0)>=($391|0);
          if (!($392)) {
           break;
          }
          $393 = $Al;
          $394 = 1 << $393;
          $395 = (($394) - 1)|0;
          $pred = $395;
         }
        } while(0);
        $396 = $pred;
        $397 = (0 - ($396))|0;
        $pred = $397;
       }
       $398 = $pred;
       $399 = $398&65535;
       $400 = (($workspace) + 32|0);
       HEAP16[$400>>1] = $399;
      }
     }
     $401 = $coef_bits;
     $402 = (($401) + 16|0);
     $403 = HEAP32[$402>>2]|0;
     $Al = $403;
     $404 = ($403|0)!=(0);
     if ($404) {
      $405 = (($workspace) + 18|0);
      $406 = HEAP16[$405>>1]|0;
      $407 = $406 << 16 >> 16;
      $408 = ($407|0)==(0);
      if ($408) {
       $409 = $Q00;
       $410 = ($409*5)|0;
       $411 = $DC1;
       $412 = $DC3;
       $413 = (($411) - ($412))|0;
       $414 = $DC7;
       $415 = (($413) - ($414))|0;
       $416 = $DC9;
       $417 = (($415) + ($416))|0;
       $418 = Math_imul($410, $417)|0;
       $num = $418;
       $419 = $num;
       $420 = ($419|0)>=(0);
       if ($420) {
        $421 = $Q11;
        $422 = $421 << 7;
        $423 = $num;
        $424 = (($422) + ($423))|0;
        $425 = $Q11;
        $426 = $425 << 8;
        $427 = (($424|0) / ($426|0))&-1;
        $pred = $427;
        $428 = $Al;
        $429 = ($428|0)>(0);
        do {
         if ($429) {
          $430 = $pred;
          $431 = $Al;
          $432 = 1 << $431;
          $433 = ($430|0)>=($432|0);
          if (!($433)) {
           break;
          }
          $434 = $Al;
          $435 = 1 << $434;
          $436 = (($435) - 1)|0;
          $pred = $436;
         }
        } while(0);
       } else {
        $437 = $Q11;
        $438 = $437 << 7;
        $439 = $num;
        $440 = (($438) - ($439))|0;
        $441 = $Q11;
        $442 = $441 << 8;
        $443 = (($440|0) / ($442|0))&-1;
        $pred = $443;
        $444 = $Al;
        $445 = ($444|0)>(0);
        do {
         if ($445) {
          $446 = $pred;
          $447 = $Al;
          $448 = 1 << $447;
          $449 = ($446|0)>=($448|0);
          if (!($449)) {
           break;
          }
          $450 = $Al;
          $451 = 1 << $450;
          $452 = (($451) - 1)|0;
          $pred = $452;
         }
        } while(0);
        $453 = $pred;
        $454 = (0 - ($453))|0;
        $pred = $454;
       }
       $455 = $pred;
       $456 = $455&65535;
       $457 = (($workspace) + 18|0);
       HEAP16[$457>>1] = $456;
      }
     }
     $458 = $coef_bits;
     $459 = (($458) + 20|0);
     $460 = HEAP32[$459>>2]|0;
     $Al = $460;
     $461 = ($460|0)!=(0);
     if ($461) {
      $462 = (($workspace) + 4|0);
      $463 = HEAP16[$462>>1]|0;
      $464 = $463 << 16 >> 16;
      $465 = ($464|0)==(0);
      if ($465) {
       $466 = $Q00;
       $467 = ($466*9)|0;
       $468 = $DC4;
       $469 = $DC6;
       $470 = (($468) + ($469))|0;
       $471 = $DC5;
       $472 = $471<<1;
       $473 = (($470) - ($472))|0;
       $474 = Math_imul($467, $473)|0;
       $num = $474;
       $475 = $num;
       $476 = ($475|0)>=(0);
       if ($476) {
        $477 = $Q02;
        $478 = $477 << 7;
        $479 = $num;
        $480 = (($478) + ($479))|0;
        $481 = $Q02;
        $482 = $481 << 8;
        $483 = (($480|0) / ($482|0))&-1;
        $pred = $483;
        $484 = $Al;
        $485 = ($484|0)>(0);
        do {
         if ($485) {
          $486 = $pred;
          $487 = $Al;
          $488 = 1 << $487;
          $489 = ($486|0)>=($488|0);
          if (!($489)) {
           break;
          }
          $490 = $Al;
          $491 = 1 << $490;
          $492 = (($491) - 1)|0;
          $pred = $492;
         }
        } while(0);
       } else {
        $493 = $Q02;
        $494 = $493 << 7;
        $495 = $num;
        $496 = (($494) - ($495))|0;
        $497 = $Q02;
        $498 = $497 << 8;
        $499 = (($496|0) / ($498|0))&-1;
        $pred = $499;
        $500 = $Al;
        $501 = ($500|0)>(0);
        do {
         if ($501) {
          $502 = $pred;
          $503 = $Al;
          $504 = 1 << $503;
          $505 = ($502|0)>=($504|0);
          if (!($505)) {
           break;
          }
          $506 = $Al;
          $507 = 1 << $506;
          $508 = (($507) - 1)|0;
          $pred = $508;
         }
        } while(0);
        $509 = $pred;
        $510 = (0 - ($509))|0;
        $pred = $510;
       }
       $511 = $pred;
       $512 = $511&65535;
       $513 = (($workspace) + 4|0);
       HEAP16[$513>>1] = $512;
      }
     }
     $514 = $inverse_DCT;
     $515 = $1;
     $516 = $compptr;
     $517 = $output_ptr;
     $518 = $output_col;
     FUNCTION_TABLE_viiiii[$514 & 127]($515,$516,$workspace,$517,$518);
     $519 = $DC2;
     $DC1 = $519;
     $520 = $DC3;
     $DC2 = $520;
     $521 = $DC5;
     $DC4 = $521;
     $522 = $DC6;
     $DC5 = $522;
     $523 = $DC8;
     $DC7 = $523;
     $524 = $DC9;
     $DC8 = $524;
     $525 = $buffer_ptr;
     $526 = (($525) + 128|0);
     $buffer_ptr = $526;
     $527 = $prev_block_row;
     $528 = (($527) + 128|0);
     $prev_block_row = $528;
     $529 = $next_block_row;
     $530 = (($529) + 128|0);
     $next_block_row = $530;
     $531 = $compptr;
     $532 = (($531) + 36|0);
     $533 = HEAP32[$532>>2]|0;
     $534 = $output_col;
     $535 = (($534) + ($533))|0;
     $output_col = $535;
     $536 = $block_num;
     $537 = (($536) + 1)|0;
     $block_num = $537;
    }
    $538 = $compptr;
    $539 = (($538) + 36|0);
    $540 = HEAP32[$539>>2]|0;
    $541 = $output_ptr;
    $542 = (($541) + ($540<<2)|0);
    $output_ptr = $542;
    $543 = $block_row;
    $544 = (($543) + 1)|0;
    $block_row = $544;
   }
  } else {
  }
  $545 = $ci;
  $546 = (($545) + 1)|0;
  $ci = $546;
  $547 = $compptr;
  $548 = (($547) + 84|0);
  $compptr = $548;
 }
 $549 = $1;
 $550 = (($549) + 156|0);
 $551 = HEAP32[$550>>2]|0;
 $552 = (($551) + 1)|0;
 HEAP32[$550>>2] = $552;
 $553 = $1;
 $554 = (($553) + 320|0);
 $555 = HEAP32[$554>>2]|0;
 $556 = ($552>>>0)<($555>>>0);
 if ($556) {
  $0 = 3;
  $557 = $0;
  STACKTOP = sp;return ($557|0);
 } else {
  $0 = 4;
  $557 = $0;
  STACKTOP = sp;return ($557|0);
 }
 return 0|0;
}
function _jinit_color_deconverter($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cconvert = 0, $ci = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,24)|0);
 $cconvert = $6;
 $7 = $cconvert;
 $8 = $0;
 $9 = (($8) + 456|0);
 HEAP32[$9>>2] = $7;
 $10 = $cconvert;
 HEAP32[$10>>2] = 112;
 $11 = $0;
 $12 = (($11) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 switch ($13|0) {
 case 5: case 4:  {
  $36 = $0;
  $37 = (($36) + 36|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(4);
  if ($39) {
   $40 = $0;
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + 20|0);
   HEAP32[$42>>2] = 10;
   $43 = $0;
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $0;
   FUNCTION_TABLE_vi[$45 & 255]($46);
  }
  break;
 }
 case 1:  {
  $14 = $0;
  $15 = (($14) + 36|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(1);
  if ($17) {
   $18 = $0;
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 20|0);
   HEAP32[$20>>2] = 10;
   $21 = $0;
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = $0;
   FUNCTION_TABLE_vi[$23 & 255]($24);
  }
  break;
 }
 case 3: case 2:  {
  $25 = $0;
  $26 = (($25) + 36|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(3);
  if ($28) {
   $29 = $0;
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 20|0);
   HEAP32[$31>>2] = 10;
   $32 = $0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   $35 = $0;
   FUNCTION_TABLE_vi[$34 & 255]($35);
  }
  break;
 }
 default: {
  $47 = $0;
  $48 = (($47) + 36|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($49|0)<(1);
  if ($50) {
   $51 = $0;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 20|0);
   HEAP32[$53>>2] = 10;
   $54 = $0;
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$55>>2]|0;
   $57 = $0;
   FUNCTION_TABLE_vi[$56 & 255]($57);
  }
 }
 }
 $58 = $0;
 $59 = (($58) + 44|0);
 $60 = HEAP32[$59>>2]|0;
 if ((($60|0) == 2)) {
  $93 = $0;
  $94 = (($93) + 120|0);
  HEAP32[$94>>2] = 3;
  $95 = $0;
  $96 = (($95) + 40|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($97|0)==(3);
  if ($98) {
   $99 = $cconvert;
   $100 = (($99) + 4|0);
   HEAP32[$100>>2] = 114;
   $101 = $0;
   _build_ycc_rgb_table1895($101);
  } else {
   $102 = $0;
   $103 = (($102) + 40|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)==(1);
   if ($105) {
    $106 = $cconvert;
    $107 = (($106) + 4|0);
    HEAP32[$107>>2] = 115;
   } else {
    $108 = $0;
    $109 = (($108) + 40|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = ($110|0)==(2);
    if ($111) {
     $112 = $cconvert;
     $113 = (($112) + 4|0);
     HEAP32[$113>>2] = 116;
    } else {
     $114 = $0;
     $115 = HEAP32[$114>>2]|0;
     $116 = (($115) + 20|0);
     HEAP32[$116>>2] = 27;
     $117 = $0;
     $118 = HEAP32[$117>>2]|0;
     $119 = HEAP32[$118>>2]|0;
     $120 = $0;
     FUNCTION_TABLE_vi[$119 & 255]($120);
    }
   }
  }
 } else if ((($60|0) == 1)) {
  $61 = $0;
  $62 = (($61) + 120|0);
  HEAP32[$62>>2] = 1;
  $63 = $0;
  $64 = (($63) + 40|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)==(1);
  if ($66) {
   label = 17;
  } else {
   $67 = $0;
   $68 = (($67) + 40|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)==(3);
   if ($70) {
    label = 17;
   } else {
    $86 = $0;
    $87 = HEAP32[$86>>2]|0;
    $88 = (($87) + 20|0);
    HEAP32[$88>>2] = 27;
    $89 = $0;
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = $0;
    FUNCTION_TABLE_vi[$91 & 255]($92);
   }
  }
  if ((label|0) == 17) {
   $71 = $cconvert;
   $72 = (($71) + 4|0);
   HEAP32[$72>>2] = 113;
   $ci = 1;
   while(1) {
    $73 = $ci;
    $74 = $0;
    $75 = (($74) + 36|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = ($73|0)<($76|0);
    if (!($77)) {
     break;
    }
    $78 = $ci;
    $79 = $0;
    $80 = (($79) + 216|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = (($81) + (($78*84)|0)|0);
    $83 = (($82) + 48|0);
    HEAP32[$83>>2] = 0;
    $84 = $ci;
    $85 = (($84) + 1)|0;
    $ci = $85;
   }
  }
 } else if ((($60|0) == 4)) {
  $121 = $0;
  $122 = (($121) + 120|0);
  HEAP32[$122>>2] = 4;
  $123 = $0;
  $124 = (($123) + 40|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = ($125|0)==(5);
  if ($126) {
   $127 = $cconvert;
   $128 = (($127) + 4|0);
   HEAP32[$128>>2] = 117;
   $129 = $0;
   _build_ycc_rgb_table1895($129);
  } else {
   $130 = $0;
   $131 = (($130) + 40|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = ($132|0)==(4);
   if ($133) {
    $134 = $cconvert;
    $135 = (($134) + 4|0);
    HEAP32[$135>>2] = 116;
   } else {
    $136 = $0;
    $137 = HEAP32[$136>>2]|0;
    $138 = (($137) + 20|0);
    HEAP32[$138>>2] = 27;
    $139 = $0;
    $140 = HEAP32[$139>>2]|0;
    $141 = HEAP32[$140>>2]|0;
    $142 = $0;
    FUNCTION_TABLE_vi[$141 & 255]($142);
   }
  }
 } else {
  $143 = $0;
  $144 = (($143) + 44|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = $0;
  $147 = (($146) + 40|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = ($145|0)==($148|0);
  if ($149) {
   $150 = $0;
   $151 = (($150) + 36|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = $0;
   $154 = (($153) + 120|0);
   HEAP32[$154>>2] = $152;
   $155 = $cconvert;
   $156 = (($155) + 4|0);
   HEAP32[$156>>2] = 116;
  } else {
   $157 = $0;
   $158 = HEAP32[$157>>2]|0;
   $159 = (($158) + 20|0);
   HEAP32[$159>>2] = 27;
   $160 = $0;
   $161 = HEAP32[$160>>2]|0;
   $162 = HEAP32[$161>>2]|0;
   $163 = $0;
   FUNCTION_TABLE_vi[$162 & 255]($163);
  }
 }
 $164 = $0;
 $165 = (($164) + 84|0);
 $166 = HEAP32[$165>>2]|0;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = $0;
  $169 = (($168) + 124|0);
  HEAP32[$169>>2] = 1;
  STACKTOP = sp;return;
 } else {
  $170 = $0;
  $171 = (($170) + 120|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = $0;
  $174 = (($173) + 124|0);
  HEAP32[$174>>2] = $172;
  STACKTOP = sp;return;
 }
}
function _start_pass_dcolor($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 STACKTOP = sp;return;
}
function _grayscale_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $1;
 $6 = HEAP32[$5>>2]|0;
 $7 = $2;
 $8 = $3;
 $9 = $4;
 $10 = $0;
 $11 = (($10) + 112|0);
 $12 = HEAP32[$11>>2]|0;
 _jcopy_sample_rows($6,$7,$8,0,$9,$12);
 STACKTOP = sp;return;
}
function _ycc_rgb_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cconvert = 0, $col = 0, $cr = 0, $inptr0 = 0;
 var $inptr1 = 0, $inptr2 = 0, $num_cols = 0, $outptr = 0, $range_limit = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 456|0);
 $7 = HEAP32[$6>>2]|0;
 $cconvert = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $num_cols = $10;
 $11 = $0;
 $12 = (($11) + 324|0);
 $13 = HEAP32[$12>>2]|0;
 $range_limit = $13;
 $14 = $cconvert;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $Crrtab = $16;
 $17 = $cconvert;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $Cbbtab = $19;
 $20 = $cconvert;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $Crgtab = $22;
 $23 = $cconvert;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $Cbgtab = $25;
 while(1) {
  $26 = $4;
  $27 = (($26) + -1)|0;
  $4 = $27;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   break;
  }
  $29 = $2;
  $30 = $1;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($29<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $inptr0 = $33;
  $34 = $2;
  $35 = $1;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inptr1 = $39;
  $40 = $2;
  $41 = $1;
  $42 = (($41) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $inptr2 = $45;
  $46 = $2;
  $47 = (($46) + 1)|0;
  $2 = $47;
  $48 = $3;
  $49 = (($48) + 4|0);
  $3 = $49;
  $50 = HEAP32[$48>>2]|0;
  $outptr = $50;
  $col = 0;
  while(1) {
   $51 = $col;
   $52 = $num_cols;
   $53 = ($51>>>0)<($52>>>0);
   if (!($53)) {
    break;
   }
   $54 = $col;
   $55 = $inptr0;
   $56 = (($55) + ($54)|0);
   $57 = HEAP8[$56>>0]|0;
   $58 = $57&255;
   $y = $58;
   $59 = $col;
   $60 = $inptr1;
   $61 = (($60) + ($59)|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $cb = $63;
   $64 = $col;
   $65 = $inptr2;
   $66 = (($65) + ($64)|0);
   $67 = HEAP8[$66>>0]|0;
   $68 = $67&255;
   $cr = $68;
   $69 = $y;
   $70 = $cr;
   $71 = $Crrtab;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($69) + ($73))|0;
   $75 = $range_limit;
   $76 = (($75) + ($74)|0);
   $77 = HEAP8[$76>>0]|0;
   $78 = $outptr;
   HEAP8[$78>>0] = $77;
   $79 = $y;
   $80 = $cb;
   $81 = $Cbgtab;
   $82 = (($81) + ($80<<2)|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = $cr;
   $85 = $Crgtab;
   $86 = (($85) + ($84<<2)|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (($83) + ($87))|0;
   $89 = $88 >> 16;
   $90 = (($79) + ($89))|0;
   $91 = $range_limit;
   $92 = (($91) + ($90)|0);
   $93 = HEAP8[$92>>0]|0;
   $94 = $outptr;
   $95 = (($94) + 1|0);
   HEAP8[$95>>0] = $93;
   $96 = $y;
   $97 = $cb;
   $98 = $Cbbtab;
   $99 = (($98) + ($97<<2)|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (($96) + ($100))|0;
   $102 = $range_limit;
   $103 = (($102) + ($101)|0);
   $104 = HEAP8[$103>>0]|0;
   $105 = $outptr;
   $106 = (($105) + 2|0);
   HEAP8[$106>>0] = $104;
   $107 = $outptr;
   $108 = (($107) + 3|0);
   $outptr = $108;
   $109 = $col;
   $110 = (($109) + 1)|0;
   $col = $110;
  }
 }
 STACKTOP = sp;return;
}
function _build_ycc_rgb_table1895($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $cconvert = 0, $i = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 456|0);
 $3 = HEAP32[$2>>2]|0;
 $cconvert = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 255]($8,1,1024)|0);
 $10 = $cconvert;
 $11 = (($10) + 8|0);
 HEAP32[$11>>2] = $9;
 $12 = $0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = $0;
 $17 = (FUNCTION_TABLE_iiii[$15 & 255]($16,1,1024)|0);
 $18 = $cconvert;
 $19 = (($18) + 12|0);
 HEAP32[$19>>2] = $17;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = (FUNCTION_TABLE_iiii[$23 & 255]($24,1,1024)|0);
 $26 = $cconvert;
 $27 = (($26) + 16|0);
 HEAP32[$27>>2] = $25;
 $28 = $0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = $0;
 $33 = (FUNCTION_TABLE_iiii[$31 & 255]($32,1,1024)|0);
 $34 = $cconvert;
 $35 = (($34) + 20|0);
 HEAP32[$35>>2] = $33;
 $i = 0;
 $x = -128;
 while(1) {
  $36 = $i;
  $37 = ($36|0)<=(255);
  if (!($37)) {
   break;
  }
  $38 = $x;
  $39 = ($38*91881)|0;
  $40 = (($39) + 32768)|0;
  $41 = $40 >> 16;
  $42 = $i;
  $43 = $cconvert;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  HEAP32[$46>>2] = $41;
  $47 = $x;
  $48 = ($47*116130)|0;
  $49 = (($48) + 32768)|0;
  $50 = $49 >> 16;
  $51 = $i;
  $52 = $cconvert;
  $53 = (($52) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  HEAP32[$55>>2] = $50;
  $56 = $x;
  $57 = Math_imul(-46802, $56)|0;
  $58 = $i;
  $59 = $cconvert;
  $60 = (($59) + 16|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + ($58<<2)|0);
  HEAP32[$62>>2] = $57;
  $63 = $x;
  $64 = Math_imul(-22554, $63)|0;
  $65 = (($64) + 32768)|0;
  $66 = $i;
  $67 = $cconvert;
  $68 = (($67) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + ($66<<2)|0);
  HEAP32[$70>>2] = $65;
  $71 = $i;
  $72 = (($71) + 1)|0;
  $i = $72;
  $73 = $x;
  $74 = (($73) + 1)|0;
  $x = $74;
 }
 STACKTOP = sp;return;
}
function _gray_rgb_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $col = 0, $inptr = 0, $num_cols = 0, $outptr = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 112|0);
 $7 = HEAP32[$6>>2]|0;
 $num_cols = $7;
 while(1) {
  $8 = $4;
  $9 = (($8) + -1)|0;
  $4 = $9;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = $2;
  $12 = (($11) + 1)|0;
  $2 = $12;
  $13 = $1;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $inptr = $16;
  $17 = $3;
  $18 = (($17) + 4|0);
  $3 = $18;
  $19 = HEAP32[$17>>2]|0;
  $outptr = $19;
  $col = 0;
  while(1) {
   $20 = $col;
   $21 = $num_cols;
   $22 = ($20>>>0)<($21>>>0);
   if (!($22)) {
    break;
   }
   $23 = $col;
   $24 = $inptr;
   $25 = (($24) + ($23)|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $outptr;
   $28 = (($27) + 2|0);
   HEAP8[$28>>0] = $26;
   $29 = $outptr;
   $30 = (($29) + 1|0);
   HEAP8[$30>>0] = $26;
   $31 = $outptr;
   HEAP8[$31>>0] = $26;
   $32 = $outptr;
   $33 = (($32) + 3|0);
   $outptr = $33;
   $34 = $col;
   $35 = (($34) + 1)|0;
   $col = $35;
  }
 }
 STACKTOP = sp;return;
}
function _null_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $count = 0, $inptr = 0, $num_cols = 0, $num_components = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $num_components = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $num_cols = $10;
 while(1) {
  $11 = $4;
  $12 = (($11) + -1)|0;
  $4 = $12;
  $13 = ($12|0)>=(0);
  if (!($13)) {
   break;
  }
  $ci = 0;
  while(1) {
   $14 = $ci;
   $15 = $num_components;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $2;
   $18 = $ci;
   $19 = $1;
   $20 = (($19) + ($18<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($17<<2)|0);
   $23 = HEAP32[$22>>2]|0;
   $inptr = $23;
   $24 = $3;
   $25 = HEAP32[$24>>2]|0;
   $26 = $ci;
   $27 = (($25) + ($26)|0);
   $outptr = $27;
   $28 = $num_cols;
   $count = $28;
   while(1) {
    $29 = $count;
    $30 = ($29>>>0)>(0);
    if (!($30)) {
     break;
    }
    $31 = $inptr;
    $32 = (($31) + 1|0);
    $inptr = $32;
    $33 = HEAP8[$31>>0]|0;
    $34 = $outptr;
    HEAP8[$34>>0] = $33;
    $35 = $num_components;
    $36 = $outptr;
    $37 = (($36) + ($35)|0);
    $outptr = $37;
    $38 = $count;
    $39 = (($38) + -1)|0;
    $count = $39;
   }
   $40 = $ci;
   $41 = (($40) + 1)|0;
   $ci = $41;
  }
  $42 = $2;
  $43 = (($42) + 1)|0;
  $2 = $43;
  $44 = $3;
  $45 = (($44) + 4|0);
  $3 = $45;
 }
 STACKTOP = sp;return;
}
function _ycck_cmyk_convert($cinfo,$input_buf,$input_row,$output_buf,$num_rows) {
 $cinfo = $cinfo|0;
 $input_buf = $input_buf|0;
 $input_row = $input_row|0;
 $output_buf = $output_buf|0;
 $num_rows = $num_rows|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Cbbtab = 0, $Cbgtab = 0, $Crgtab = 0, $Crrtab = 0, $cb = 0, $cconvert = 0, $col = 0, $cr = 0, $inptr0 = 0, $inptr1 = 0, $inptr2 = 0, $inptr3 = 0, $num_cols = 0, $outptr = 0;
 var $range_limit = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $cinfo;
 $1 = $input_buf;
 $2 = $input_row;
 $3 = $output_buf;
 $4 = $num_rows;
 $5 = $0;
 $6 = (($5) + 456|0);
 $7 = HEAP32[$6>>2]|0;
 $cconvert = $7;
 $8 = $0;
 $9 = (($8) + 112|0);
 $10 = HEAP32[$9>>2]|0;
 $num_cols = $10;
 $11 = $0;
 $12 = (($11) + 324|0);
 $13 = HEAP32[$12>>2]|0;
 $range_limit = $13;
 $14 = $cconvert;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $Crrtab = $16;
 $17 = $cconvert;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $Cbbtab = $19;
 $20 = $cconvert;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $Crgtab = $22;
 $23 = $cconvert;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $Cbgtab = $25;
 while(1) {
  $26 = $4;
  $27 = (($26) + -1)|0;
  $4 = $27;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   break;
  }
  $29 = $2;
  $30 = $1;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($29<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $inptr0 = $33;
  $34 = $2;
  $35 = $1;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $inptr1 = $39;
  $40 = $2;
  $41 = $1;
  $42 = (($41) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $inptr2 = $45;
  $46 = $2;
  $47 = $1;
  $48 = (($47) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($46<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $inptr3 = $51;
  $52 = $2;
  $53 = (($52) + 1)|0;
  $2 = $53;
  $54 = $3;
  $55 = (($54) + 4|0);
  $3 = $55;
  $56 = HEAP32[$54>>2]|0;
  $outptr = $56;
  $col = 0;
  while(1) {
   $57 = $col;
   $58 = $num_cols;
   $59 = ($57>>>0)<($58>>>0);
   if (!($59)) {
    break;
   }
   $60 = $col;
   $61 = $inptr0;
   $62 = (($61) + ($60)|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = $63&255;
   $y = $64;
   $65 = $col;
   $66 = $inptr1;
   $67 = (($66) + ($65)|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = $68&255;
   $cb = $69;
   $70 = $col;
   $71 = $inptr2;
   $72 = (($71) + ($70)|0);
   $73 = HEAP8[$72>>0]|0;
   $74 = $73&255;
   $cr = $74;
   $75 = $y;
   $76 = $cr;
   $77 = $Crrtab;
   $78 = (($77) + ($76<<2)|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($75) + ($79))|0;
   $81 = (255 - ($80))|0;
   $82 = $range_limit;
   $83 = (($82) + ($81)|0);
   $84 = HEAP8[$83>>0]|0;
   $85 = $outptr;
   HEAP8[$85>>0] = $84;
   $86 = $y;
   $87 = $cb;
   $88 = $Cbgtab;
   $89 = (($88) + ($87<<2)|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = $cr;
   $92 = $Crgtab;
   $93 = (($92) + ($91<<2)|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($90) + ($94))|0;
   $96 = $95 >> 16;
   $97 = (($86) + ($96))|0;
   $98 = (255 - ($97))|0;
   $99 = $range_limit;
   $100 = (($99) + ($98)|0);
   $101 = HEAP8[$100>>0]|0;
   $102 = $outptr;
   $103 = (($102) + 1|0);
   HEAP8[$103>>0] = $101;
   $104 = $y;
   $105 = $cb;
   $106 = $Cbbtab;
   $107 = (($106) + ($105<<2)|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($104) + ($108))|0;
   $110 = (255 - ($109))|0;
   $111 = $range_limit;
   $112 = (($111) + ($110)|0);
   $113 = HEAP8[$112>>0]|0;
   $114 = $outptr;
   $115 = (($114) + 2|0);
   HEAP8[$115>>0] = $113;
   $116 = $col;
   $117 = $inptr3;
   $118 = (($117) + ($116)|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = $outptr;
   $121 = (($120) + 3|0);
   HEAP8[$121>>0] = $119;
   $122 = $outptr;
   $123 = (($122) + 4|0);
   $outptr = $123;
   $124 = $col;
   $125 = (($124) + 1)|0;
   $col = $125;
  }
 }
 STACKTOP = sp;return;
}
function _jinit_inverse_dct($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $compptr = 0;
 var $idct = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,84)|0);
 $idct = $6;
 $7 = $idct;
 $8 = $0;
 $9 = (($8) + 448|0);
 HEAP32[$9>>2] = $7;
 $10 = $idct;
 HEAP32[$10>>2] = 118;
 $ci = 0;
 $11 = $0;
 $12 = (($11) + 216|0);
 $13 = HEAP32[$12>>2]|0;
 $compptr = $13;
 while(1) {
  $14 = $ci;
  $15 = $0;
  $16 = (($15) + 36|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($14|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $0;
  $20 = (($19) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = $0;
  $24 = (FUNCTION_TABLE_iiii[$22 & 255]($23,1,256)|0);
  $25 = $compptr;
  $26 = (($25) + 80|0);
  HEAP32[$26>>2] = $24;
  $27 = $compptr;
  $28 = (($27) + 80|0);
  $29 = HEAP32[$28>>2]|0;
  _memset(($29|0),0,256)|0;
  $30 = $ci;
  $31 = $idct;
  $32 = (($31) + 44|0);
  $33 = (($32) + ($30<<2)|0);
  HEAP32[$33>>2] = -1;
  $34 = $ci;
  $35 = (($34) + 1)|0;
  $ci = $35;
  $36 = $compptr;
  $37 = (($36) + 84|0);
  $compptr = $37;
 }
 STACKTOP = sp;return;
}
function _start_pass($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0.0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $ci = 0, $col = 0, $compptr = 0, $fmtbl = 0, $i = 0, $idct = 0, $ifmtbl = 0, $ismtbl = 0, $method = 0, $method_ptr = 0, $qtbl = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 448|0);
 $3 = HEAP32[$2>>2]|0;
 $idct = $3;
 $method = 0;
 $method_ptr = 0;
 $ci = 0;
 $4 = $0;
 $5 = (($4) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $compptr = $6;
 while(1) {
  $7 = $ci;
  $8 = $0;
  $9 = (($8) + 36|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = $compptr;
  $13 = (($12) + 36|0);
  $14 = HEAP32[$13>>2]|0;
  if ((($14|0) == 8)) {
   $15 = $0;
   $16 = (($15) + 72|0);
   $17 = HEAP32[$16>>2]|0;
   if ((($17|0) == 1)) {
    $method_ptr = 123;
    $method = 1;
   } else if ((($17|0) == 2)) {
    $method_ptr = 124;
    $method = 2;
   } else if ((($17|0) == 0)) {
    $method_ptr = 122;
    $method = 0;
   } else {
    $18 = $0;
    $19 = HEAP32[$18>>2]|0;
    $20 = (($19) + 20|0);
    HEAP32[$20>>2] = 48;
    $21 = $0;
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$22>>2]|0;
    $24 = $0;
    FUNCTION_TABLE_vi[$23 & 255]($24);
   }
  } else if ((($14|0) == 4)) {
   $method_ptr = 121;
   $method = 0;
  } else if ((($14|0) == 1)) {
   $method_ptr = 119;
   $method = 0;
  } else if ((($14|0) == 2)) {
   $method_ptr = 120;
   $method = 0;
  } else {
   $25 = $0;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + 20|0);
   HEAP32[$27>>2] = 7;
   $28 = $compptr;
   $29 = (($28) + 36|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $0;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 24|0);
   HEAP32[$33>>2] = $30;
   $34 = $0;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $0;
   FUNCTION_TABLE_vi[$36 & 255]($37);
  }
  $38 = $method_ptr;
  $39 = $ci;
  $40 = $idct;
  $41 = (($40) + 4|0);
  $42 = (($41) + ($39<<2)|0);
  HEAP32[$42>>2] = $38;
  $43 = $compptr;
  $44 = (($43) + 48|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)!=(0);
  do {
   if ($46) {
    $47 = $ci;
    $48 = $idct;
    $49 = (($48) + 44|0);
    $50 = (($49) + ($47<<2)|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = $method;
    $53 = ($51|0)==($52|0);
    if ($53) {
     label = 16;
    } else {
     $54 = $compptr;
     $55 = (($54) + 76|0);
     $56 = HEAP32[$55>>2]|0;
     $qtbl = $56;
     $57 = $qtbl;
     $58 = ($57|0)==(0|0);
     if ($58) {
      break;
     }
     $59 = $method;
     $60 = $ci;
     $61 = $idct;
     $62 = (($61) + 44|0);
     $63 = (($62) + ($60<<2)|0);
     HEAP32[$63>>2] = $59;
     $64 = $method;
     if ((($64|0) == 1)) {
      $80 = $compptr;
      $81 = (($80) + 80|0);
      $82 = HEAP32[$81>>2]|0;
      $ifmtbl = $82;
      $i = 0;
      while(1) {
       $83 = $i;
       $84 = ($83|0)<(64);
       if (!($84)) {
        break;
       }
       $85 = $i;
       $86 = $qtbl;
       $87 = (($86) + ($85<<1)|0);
       $88 = HEAP16[$87>>1]|0;
       $89 = $88&65535;
       $90 = $i;
       $91 = (50888 + ($90<<1)|0);
       $92 = HEAP16[$91>>1]|0;
       $93 = $92 << 16 >> 16;
       $94 = Math_imul($89, $93)|0;
       $95 = (($94) + 2048)|0;
       $96 = $95 >> 12;
       $97 = $i;
       $98 = $ifmtbl;
       $99 = (($98) + ($97<<2)|0);
       HEAP32[$99>>2] = $96;
       $100 = $i;
       $101 = (($100) + 1)|0;
       $i = $101;
      }
     } else if ((($64|0) == 0)) {
      $65 = $compptr;
      $66 = (($65) + 80|0);
      $67 = HEAP32[$66>>2]|0;
      $ismtbl = $67;
      $i = 0;
      while(1) {
       $68 = $i;
       $69 = ($68|0)<(64);
       if (!($69)) {
        break;
       }
       $70 = $i;
       $71 = $qtbl;
       $72 = (($71) + ($70<<1)|0);
       $73 = HEAP16[$72>>1]|0;
       $74 = $73&65535;
       $75 = $i;
       $76 = $ismtbl;
       $77 = (($76) + ($75<<2)|0);
       HEAP32[$77>>2] = $74;
       $78 = $i;
       $79 = (($78) + 1)|0;
       $i = $79;
      }
     } else if ((($64|0) == 2)) {
      $102 = $compptr;
      $103 = (($102) + 80|0);
      $104 = HEAP32[$103>>2]|0;
      $fmtbl = $104;
      $i = 0;
      $row = 0;
      while(1) {
       $105 = $row;
       $106 = ($105|0)<(8);
       if (!($106)) {
        break;
       }
       $col = 0;
       while(1) {
        $107 = $col;
        $108 = ($107|0)<(8);
        if (!($108)) {
         break;
        }
        $109 = $i;
        $110 = $qtbl;
        $111 = (($110) + ($109<<1)|0);
        $112 = HEAP16[$111>>1]|0;
        $113 = (+($112&65535));
        $114 = $row;
        $115 = (51016 + ($114<<3)|0);
        $116 = +HEAPF64[$115>>3];
        $117 = $113 * $116;
        $118 = $col;
        $119 = (51016 + ($118<<3)|0);
        $120 = +HEAPF64[$119>>3];
        $121 = $117 * $120;
        $122 = $121;
        $123 = $i;
        $124 = $fmtbl;
        $125 = (($124) + ($123<<2)|0);
        HEAPF32[$125>>2] = $122;
        $126 = $i;
        $127 = (($126) + 1)|0;
        $i = $127;
        $128 = $col;
        $129 = (($128) + 1)|0;
        $col = $129;
       }
       $130 = $row;
       $131 = (($130) + 1)|0;
       $row = $131;
      }
     } else {
      $132 = $0;
      $133 = HEAP32[$132>>2]|0;
      $134 = (($133) + 20|0);
      HEAP32[$134>>2] = 48;
      $135 = $0;
      $136 = HEAP32[$135>>2]|0;
      $137 = HEAP32[$136>>2]|0;
      $138 = $0;
      FUNCTION_TABLE_vi[$137 & 255]($138);
     }
    }
   } else {
    label = 16;
   }
  } while(0);
  if ((label|0) == 16) {
   label = 0;
  }
  $139 = $ci;
  $140 = (($139) + 1)|0;
  $ci = $140;
  $141 = $compptr;
  $142 = (($141) + 84|0);
  $compptr = $142;
 }
 STACKTOP = sp;return;
}
function _jpeg_make_d_derived_tbl($cinfo,$isDC,$tblno,$pdtbl) {
 $cinfo = $cinfo|0;
 $isDC = $isDC|0;
 $tblno = $tblno|0;
 $pdtbl = $pdtbl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $ctr = 0, $dtbl = 0, $htbl = 0, $huffcode = 0, $huffsize = 0, $i = 0;
 var $l = 0, $lookbits = 0, $numsymbols = 0, $p = 0, $si = 0, $sym = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1360|0;
 $huffsize = sp + 1088|0;
 $huffcode = sp + 8|0;
 $0 = $cinfo;
 $1 = $isDC;
 $2 = $tblno;
 $3 = $pdtbl;
 $4 = $2;
 $5 = ($4|0)<(0);
 if ($5) {
  label = 3;
 } else {
  $6 = $2;
  $7 = ($6|0)>=(4);
  if ($7) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $8 = $0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 20|0);
  HEAP32[$10>>2] = 50;
  $11 = $2;
  $12 = $0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 24|0);
  HEAP32[$14>>2] = $11;
  $15 = $0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = $0;
  FUNCTION_TABLE_vi[$17 & 255]($18);
 }
 $19 = $1;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = $2;
  $22 = $0;
  $23 = (($22) + 180|0);
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $31 = $25;
 } else {
  $26 = $2;
  $27 = $0;
  $28 = (($27) + 196|0);
  $29 = (($28) + ($26<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30;
 }
 $htbl = $31;
 $32 = $htbl;
 $33 = ($32|0)==(0|0);
 if ($33) {
  $34 = $0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 20|0);
  HEAP32[$36>>2] = 50;
  $37 = $2;
  $38 = $0;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 24|0);
  HEAP32[$40>>2] = $37;
  $41 = $0;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = $0;
  FUNCTION_TABLE_vi[$43 & 255]($44);
 }
 $45 = $3;
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0|0);
 if ($47) {
  $48 = $0;
  $49 = (($48) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$50>>2]|0;
  $52 = $0;
  $53 = (FUNCTION_TABLE_iiii[$51 & 255]($52,1,1424)|0);
  $54 = $3;
  HEAP32[$54>>2] = $53;
 }
 $55 = $3;
 $56 = HEAP32[$55>>2]|0;
 $dtbl = $56;
 $57 = $htbl;
 $58 = $dtbl;
 $59 = (($58) + 140|0);
 HEAP32[$59>>2] = $57;
 $p = 0;
 $l = 1;
 while(1) {
  $60 = $l;
  $61 = ($60|0)<=(16);
  if (!($61)) {
   break;
  }
  $62 = $l;
  $63 = $htbl;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $i = $66;
  $67 = $i;
  $68 = ($67|0)<(0);
  if ($68) {
   label = 15;
  } else {
   $69 = $p;
   $70 = $i;
   $71 = (($69) + ($70))|0;
   $72 = ($71|0)>(256);
   if ($72) {
    label = 15;
   }
  }
  if ((label|0) == 15) {
   label = 0;
   $73 = $0;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + 20|0);
   HEAP32[$75>>2] = 8;
   $76 = $0;
   $77 = HEAP32[$76>>2]|0;
   $78 = HEAP32[$77>>2]|0;
   $79 = $0;
   FUNCTION_TABLE_vi[$78 & 255]($79);
  }
  while(1) {
   $80 = $i;
   $81 = (($80) + -1)|0;
   $i = $81;
   $82 = ($80|0)!=(0);
   if (!($82)) {
    break;
   }
   $83 = $l;
   $84 = $83&255;
   $85 = $p;
   $86 = (($85) + 1)|0;
   $p = $86;
   $87 = (($huffsize) + ($85)|0);
   HEAP8[$87>>0] = $84;
  }
  $88 = $l;
  $89 = (($88) + 1)|0;
  $l = $89;
 }
 $90 = $p;
 $91 = (($huffsize) + ($90)|0);
 HEAP8[$91>>0] = 0;
 $92 = $p;
 $numsymbols = $92;
 $code = 0;
 $93 = HEAP8[$huffsize>>0]|0;
 $94 = $93 << 24 >> 24;
 $si = $94;
 $p = 0;
 while(1) {
  $95 = $p;
  $96 = (($huffsize) + ($95)|0);
  $97 = HEAP8[$96>>0]|0;
  $98 = ($97<<24>>24)!=(0);
  if (!($98)) {
   break;
  }
  while(1) {
   $99 = $p;
   $100 = (($huffsize) + ($99)|0);
   $101 = HEAP8[$100>>0]|0;
   $102 = $101 << 24 >> 24;
   $103 = $si;
   $104 = ($102|0)==($103|0);
   if (!($104)) {
    break;
   }
   $105 = $code;
   $106 = $p;
   $107 = (($106) + 1)|0;
   $p = $107;
   $108 = (($huffcode) + ($106<<2)|0);
   HEAP32[$108>>2] = $105;
   $109 = $code;
   $110 = (($109) + 1)|0;
   $code = $110;
  }
  $111 = $code;
  $112 = $si;
  $113 = 1 << $112;
  $114 = ($111|0)>=($113|0);
  if ($114) {
   $115 = $0;
   $116 = HEAP32[$115>>2]|0;
   $117 = (($116) + 20|0);
   HEAP32[$117>>2] = 8;
   $118 = $0;
   $119 = HEAP32[$118>>2]|0;
   $120 = HEAP32[$119>>2]|0;
   $121 = $0;
   FUNCTION_TABLE_vi[$120 & 255]($121);
  }
  $122 = $code;
  $123 = $122 << 1;
  $code = $123;
  $124 = $si;
  $125 = (($124) + 1)|0;
  $si = $125;
 }
 $p = 0;
 $l = 1;
 while(1) {
  $126 = $l;
  $127 = ($126|0)<=(16);
  if (!($127)) {
   break;
  }
  $128 = $l;
  $129 = $htbl;
  $130 = (($129) + ($128)|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = ($131<<24>>24)!=(0);
  if ($132) {
   $133 = $p;
   $134 = $p;
   $135 = (($huffcode) + ($134<<2)|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($133) - ($136))|0;
   $138 = $l;
   $139 = $dtbl;
   $140 = (($139) + 72|0);
   $141 = (($140) + ($138<<2)|0);
   HEAP32[$141>>2] = $137;
   $142 = $l;
   $143 = $htbl;
   $144 = (($143) + ($142)|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = $145&255;
   $147 = $p;
   $148 = (($147) + ($146))|0;
   $p = $148;
   $149 = $p;
   $150 = (($149) - 1)|0;
   $151 = (($huffcode) + ($150<<2)|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = $l;
   $154 = $dtbl;
   $155 = (($154) + ($153<<2)|0);
   HEAP32[$155>>2] = $152;
  } else {
   $156 = $l;
   $157 = $dtbl;
   $158 = (($157) + ($156<<2)|0);
   HEAP32[$158>>2] = -1;
  }
  $159 = $l;
  $160 = (($159) + 1)|0;
  $l = $160;
 }
 $161 = $dtbl;
 $162 = (($161) + 68|0);
 HEAP32[$162>>2] = 1048575;
 $163 = $dtbl;
 $164 = (($163) + 144|0);
 _memset(($164|0),0,1024)|0;
 $p = 0;
 $l = 1;
 while(1) {
  $165 = $l;
  $166 = ($165|0)<=(8);
  if (!($166)) {
   break;
  }
  $i = 1;
  while(1) {
   $167 = $i;
   $168 = $l;
   $169 = $htbl;
   $170 = (($169) + ($168)|0);
   $171 = HEAP8[$170>>0]|0;
   $172 = $171&255;
   $173 = ($167|0)<=($172|0);
   if (!($173)) {
    break;
   }
   $174 = $p;
   $175 = (($huffcode) + ($174<<2)|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = $l;
   $178 = (8 - ($177))|0;
   $179 = $176 << $178;
   $lookbits = $179;
   $180 = $l;
   $181 = (8 - ($180))|0;
   $182 = 1 << $181;
   $ctr = $182;
   while(1) {
    $183 = $ctr;
    $184 = ($183|0)>(0);
    if (!($184)) {
     break;
    }
    $185 = $l;
    $186 = $lookbits;
    $187 = $dtbl;
    $188 = (($187) + 144|0);
    $189 = (($188) + ($186<<2)|0);
    HEAP32[$189>>2] = $185;
    $190 = $p;
    $191 = $htbl;
    $192 = (($191) + 17|0);
    $193 = (($192) + ($190)|0);
    $194 = HEAP8[$193>>0]|0;
    $195 = $lookbits;
    $196 = $dtbl;
    $197 = (($196) + 1168|0);
    $198 = (($197) + ($195)|0);
    HEAP8[$198>>0] = $194;
    $199 = $lookbits;
    $200 = (($199) + 1)|0;
    $lookbits = $200;
    $201 = $ctr;
    $202 = (($201) + -1)|0;
    $ctr = $202;
   }
   $203 = $i;
   $204 = (($203) + 1)|0;
   $i = $204;
   $205 = $p;
   $206 = (($205) + 1)|0;
   $p = $206;
  }
  $207 = $l;
  $208 = (($207) + 1)|0;
  $l = $208;
 }
 $209 = $1;
 $210 = ($209|0)!=(0);
 if (!($210)) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $211 = $i;
  $212 = $numsymbols;
  $213 = ($211|0)<($212|0);
  if (!($213)) {
   break;
  }
  $214 = $i;
  $215 = $htbl;
  $216 = (($215) + 17|0);
  $217 = (($216) + ($214)|0);
  $218 = HEAP8[$217>>0]|0;
  $219 = $218&255;
  $sym = $219;
  $220 = $sym;
  $221 = ($220|0)<(0);
  if ($221) {
   label = 53;
  } else {
   $222 = $sym;
   $223 = ($222|0)>(15);
   if ($223) {
    label = 53;
   }
  }
  if ((label|0) == 53) {
   label = 0;
   $224 = $0;
   $225 = HEAP32[$224>>2]|0;
   $226 = (($225) + 20|0);
   HEAP32[$226>>2] = 8;
   $227 = $0;
   $228 = HEAP32[$227>>2]|0;
   $229 = HEAP32[$228>>2]|0;
   $230 = $0;
   FUNCTION_TABLE_vi[$229 & 255]($230);
  }
  $231 = $i;
  $232 = (($231) + 1)|0;
  $i = $232;
 }
 STACKTOP = sp;return;
}
function _jpeg_fill_bit_buffer($state,$get_buffer,$bits_left,$nbits) {
 $state = $state|0;
 $get_buffer = $get_buffer|0;
 $bits_left = $bits_left|0;
 $nbits = $nbits|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $bytes_in_buffer = 0, $c = 0, $cinfo = 0, $next_input_byte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $state;
 $2 = $get_buffer;
 $3 = $bits_left;
 $4 = $nbits;
 $5 = $1;
 $6 = HEAP32[$5>>2]|0;
 $next_input_byte = $6;
 $7 = $1;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $bytes_in_buffer = $9;
 $10 = $1;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $cinfo = $12;
 $13 = $cinfo;
 $14 = (($13) + 416|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0);
 do {
  if ($16) {
   L3: while(1) {
    $17 = $3;
    $18 = ($17|0)<(25);
    if (!($18)) {
     label = 21;
     break;
    }
    $19 = $bytes_in_buffer;
    $20 = ($19|0)==(0);
    if ($20) {
     $21 = $cinfo;
     $22 = (($21) + 24|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = (($23) + 12|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = $cinfo;
     $27 = (FUNCTION_TABLE_ii[$25 & 127]($26)|0);
     $28 = ($27|0)!=(0);
     if (!($28)) {
      label = 6;
      break;
     }
     $29 = $cinfo;
     $30 = (($29) + 24|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = HEAP32[$31>>2]|0;
     $next_input_byte = $32;
     $33 = $cinfo;
     $34 = (($33) + 24|0);
     $35 = HEAP32[$34>>2]|0;
     $36 = (($35) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $bytes_in_buffer = $37;
    }
    $38 = $bytes_in_buffer;
    $39 = (($38) + -1)|0;
    $bytes_in_buffer = $39;
    $40 = $next_input_byte;
    $41 = (($40) + 1|0);
    $next_input_byte = $41;
    $42 = HEAP8[$40>>0]|0;
    $43 = $42&255;
    $c = $43;
    $44 = $c;
    $45 = ($44|0)==(255);
    if ($45) {
     while(1) {
      $46 = $bytes_in_buffer;
      $47 = ($46|0)==(0);
      if ($47) {
       $48 = $cinfo;
       $49 = (($48) + 24|0);
       $50 = HEAP32[$49>>2]|0;
       $51 = (($50) + 12|0);
       $52 = HEAP32[$51>>2]|0;
       $53 = $cinfo;
       $54 = (FUNCTION_TABLE_ii[$52 & 127]($53)|0);
       $55 = ($54|0)!=(0);
       if (!($55)) {
        label = 12;
        break L3;
       }
       $56 = $cinfo;
       $57 = (($56) + 24|0);
       $58 = HEAP32[$57>>2]|0;
       $59 = HEAP32[$58>>2]|0;
       $next_input_byte = $59;
       $60 = $cinfo;
       $61 = (($60) + 24|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = (($62) + 4|0);
       $64 = HEAP32[$63>>2]|0;
       $bytes_in_buffer = $64;
      }
      $65 = $bytes_in_buffer;
      $66 = (($65) + -1)|0;
      $bytes_in_buffer = $66;
      $67 = $next_input_byte;
      $68 = (($67) + 1|0);
      $next_input_byte = $68;
      $69 = HEAP8[$67>>0]|0;
      $70 = $69&255;
      $c = $70;
      $71 = $c;
      $72 = ($71|0)==(255);
      if (!($72)) {
       break;
      }
     }
     $73 = $c;
     $74 = ($73|0)==(0);
     if (!($74)) {
      label = 18;
      break;
     }
     $c = 255;
    }
    $78 = $2;
    $79 = $78 << 8;
    $80 = $c;
    $81 = $79 | $80;
    $2 = $81;
    $82 = $3;
    $83 = (($82) + 8)|0;
    $3 = $83;
   }
   if ((label|0) == 6) {
    $0 = 0;
    $120 = $0;
    STACKTOP = sp;return ($120|0);
   }
   else if ((label|0) == 12) {
    $0 = 0;
    $120 = $0;
    STACKTOP = sp;return ($120|0);
   }
   else if ((label|0) == 18) {
    $75 = $c;
    $76 = $cinfo;
    $77 = (($76) + 416|0);
    HEAP32[$77>>2] = $75;
    label = 23;
    break;
   }
   else if ((label|0) == 21) {
    break;
   }
  } else {
   label = 23;
  }
 } while(0);
 if ((label|0) == 23) {
  $84 = $4;
  $85 = $3;
  $86 = ($84|0)>($85|0);
  if ($86) {
   $87 = $cinfo;
   $88 = (($87) + 444|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 8|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = ($91|0)!=(0);
   if (!($92)) {
    $93 = $cinfo;
    $94 = HEAP32[$93>>2]|0;
    $95 = (($94) + 20|0);
    HEAP32[$95>>2] = 117;
    $96 = $cinfo;
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97) + 4|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = $cinfo;
    FUNCTION_TABLE_vii[$99 & 255]($100,-1);
    $101 = $cinfo;
    $102 = (($101) + 444|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) + 8|0);
    HEAP32[$104>>2] = 1;
   }
   $105 = $3;
   $106 = (25 - ($105))|0;
   $107 = $2;
   $108 = $107 << $106;
   $2 = $108;
   $3 = 25;
  }
 }
 $109 = $next_input_byte;
 $110 = $1;
 HEAP32[$110>>2] = $109;
 $111 = $bytes_in_buffer;
 $112 = $1;
 $113 = (($112) + 4|0);
 HEAP32[$113>>2] = $111;
 $114 = $2;
 $115 = $1;
 $116 = (($115) + 8|0);
 HEAP32[$116>>2] = $114;
 $117 = $3;
 $118 = $1;
 $119 = (($118) + 12|0);
 HEAP32[$119>>2] = $117;
 $0 = 1;
 $120 = $0;
 STACKTOP = sp;return ($120|0);
}
function _jpeg_huff_decode($state,$get_buffer,$bits_left,$htbl,$min_bits) {
 $state = $state|0;
 $get_buffer = $get_buffer|0;
 $bits_left = $bits_left|0;
 $htbl = $htbl|0;
 $min_bits = $min_bits|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $code = 0, $l = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $state;
 $2 = $get_buffer;
 $3 = $bits_left;
 $4 = $htbl;
 $5 = $min_bits;
 $6 = $5;
 $l = $6;
 $7 = $3;
 $8 = $l;
 $9 = ($7|0)<($8|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = $2;
   $12 = $3;
   $13 = $l;
   $14 = (_jpeg_fill_bit_buffer($10,$11,$12,$13)|0);
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = $1;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $2 = $18;
    $19 = $1;
    $20 = (($19) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    $3 = $21;
    break;
   }
   $0 = -1;
   $97 = $0;
   STACKTOP = sp;return ($97|0);
  }
 } while(0);
 $22 = $2;
 $23 = $l;
 $24 = $3;
 $25 = (($24) - ($23))|0;
 $3 = $25;
 $26 = $22 >> $25;
 $27 = $l;
 $28 = 1 << $27;
 $29 = (($28) - 1)|0;
 $30 = $26 & $29;
 $code = $30;
 while(1) {
  $31 = $code;
  $32 = $l;
  $33 = $4;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($31|0)>($35|0);
  if (!($36)) {
   break;
  }
  $37 = $code;
  $38 = $37 << 1;
  $code = $38;
  $39 = $3;
  $40 = ($39|0)<(1);
  if ($40) {
   $41 = $1;
   $42 = $2;
   $43 = $3;
   $44 = (_jpeg_fill_bit_buffer($41,$42,$43,1)|0);
   $45 = ($44|0)!=(0);
   if (!($45)) {
    label = 9;
    break;
   }
   $46 = $1;
   $47 = (($46) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $2 = $48;
   $49 = $1;
   $50 = (($49) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   $3 = $51;
  }
  $52 = $2;
  $53 = $3;
  $54 = (($53) - 1)|0;
  $3 = $54;
  $55 = $52 >> $54;
  $56 = $55 & 1;
  $57 = $code;
  $58 = $57 | $56;
  $code = $58;
  $59 = $l;
  $60 = (($59) + 1)|0;
  $l = $60;
 }
 if ((label|0) == 9) {
  $0 = -1;
  $97 = $0;
  STACKTOP = sp;return ($97|0);
 }
 $61 = $2;
 $62 = $1;
 $63 = (($62) + 8|0);
 HEAP32[$63>>2] = $61;
 $64 = $3;
 $65 = $1;
 $66 = (($65) + 12|0);
 HEAP32[$66>>2] = $64;
 $67 = $l;
 $68 = ($67|0)>(16);
 if ($68) {
  $69 = $1;
  $70 = (($69) + 16|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + 20|0);
  HEAP32[$73>>2] = 118;
  $74 = $1;
  $75 = (($74) + 16|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($77) + 4|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $1;
  $81 = (($80) + 16|0);
  $82 = HEAP32[$81>>2]|0;
  FUNCTION_TABLE_vii[$79 & 255]($82,-1);
  $0 = 0;
  $97 = $0;
  STACKTOP = sp;return ($97|0);
 } else {
  $83 = $code;
  $84 = $l;
  $85 = $4;
  $86 = (($85) + 72|0);
  $87 = (($86) + ($84<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($83) + ($88))|0;
  $90 = $4;
  $91 = (($90) + 140|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + 17|0);
  $94 = (($93) + ($89)|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $0 = $96;
  $97 = $0;
  STACKTOP = sp;return ($97|0);
 }
 return 0|0;
}
function _jinit_huff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $entropy = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (FUNCTION_TABLE_iiii[$4 & 255]($5,1,232)|0);
 $entropy = $6;
 $7 = $entropy;
 $8 = $0;
 $9 = (($8) + 444|0);
 HEAP32[$9>>2] = $7;
 $10 = $entropy;
 HEAP32[$10>>2] = 125;
 $11 = $entropy;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 126;
 $i = 0;
 while(1) {
  $13 = $i;
  $14 = ($13|0)<(4);
  if (!($14)) {
   break;
  }
  $15 = $i;
  $16 = $entropy;
  $17 = (($16) + 56|0);
  $18 = (($17) + ($15<<2)|0);
  HEAP32[$18>>2] = 0;
  $19 = $i;
  $20 = $entropy;
  $21 = (($20) + 40|0);
  $22 = (($21) + ($19<<2)|0);
  HEAP32[$22>>2] = 0;
  $23 = $i;
  $24 = (($23) + 1)|0;
  $i = $24;
 }
 STACKTOP = sp;return;
}
function _start_pass_huff_decoder($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $actbl = 0, $blkn = 0;
 var $ci = 0, $compptr = 0, $dctbl = 0, $entropy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 444|0);
 $3 = HEAP32[$2>>2]|0;
 $entropy = $3;
 $4 = $0;
 $5 = (($4) + 400|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  label = 5;
 } else {
  $8 = $0;
  $9 = (($8) + 404|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(63);
  if ($11) {
   label = 5;
  } else {
   $12 = $0;
   $13 = (($12) + 408|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)!=(0);
   if ($15) {
    label = 5;
   } else {
    $16 = $0;
    $17 = (($16) + 412|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($18|0)!=(0);
    if ($19) {
     label = 5;
    }
   }
  }
 }
 if ((label|0) == 5) {
  $20 = $0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 20|0);
  HEAP32[$22>>2] = 122;
  $23 = $0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $0;
  FUNCTION_TABLE_vii[$26 & 255]($27,-1);
 }
 $ci = 0;
 while(1) {
  $28 = $ci;
  $29 = $0;
  $30 = (($29) + 328|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = $ci;
  $34 = $0;
  $35 = (($34) + 332|0);
  $36 = (($35) + ($33<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $compptr = $37;
  $38 = $compptr;
  $39 = (($38) + 20|0);
  $40 = HEAP32[$39>>2]|0;
  $dctbl = $40;
  $41 = $compptr;
  $42 = (($41) + 24|0);
  $43 = HEAP32[$42>>2]|0;
  $actbl = $43;
  $44 = $0;
  $45 = $dctbl;
  $46 = $dctbl;
  $47 = $entropy;
  $48 = (($47) + 40|0);
  $49 = (($48) + ($46<<2)|0);
  _jpeg_make_d_derived_tbl($44,1,$45,$49);
  $50 = $0;
  $51 = $actbl;
  $52 = $actbl;
  $53 = $entropy;
  $54 = (($53) + 56|0);
  $55 = (($54) + ($52<<2)|0);
  _jpeg_make_d_derived_tbl($50,0,$51,$55);
  $56 = $ci;
  $57 = $entropy;
  $58 = (($57) + 20|0);
  $59 = (($58) + ($56<<2)|0);
  HEAP32[$59>>2] = 0;
  $60 = $ci;
  $61 = (($60) + 1)|0;
  $ci = $61;
 }
 $blkn = 0;
 while(1) {
  $62 = $blkn;
  $63 = $0;
  $64 = (($63) + 356|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($62|0)<($65|0);
  if (!($66)) {
   break;
  }
  $67 = $blkn;
  $68 = $0;
  $69 = (($68) + 360|0);
  $70 = (($69) + ($67<<2)|0);
  $71 = HEAP32[$70>>2]|0;
  $ci = $71;
  $72 = $ci;
  $73 = $0;
  $74 = (($73) + 332|0);
  $75 = (($74) + ($72<<2)|0);
  $76 = HEAP32[$75>>2]|0;
  $compptr = $76;
  $77 = $compptr;
  $78 = (($77) + 20|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $entropy;
  $81 = (($80) + 40|0);
  $82 = (($81) + ($79<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $blkn;
  $85 = $entropy;
  $86 = (($85) + 72|0);
  $87 = (($86) + ($84<<2)|0);
  HEAP32[$87>>2] = $83;
  $88 = $compptr;
  $89 = (($88) + 24|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = $entropy;
  $92 = (($91) + 56|0);
  $93 = (($92) + ($90<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $blkn;
  $96 = $entropy;
  $97 = (($96) + 112|0);
  $98 = (($97) + ($95<<2)|0);
  HEAP32[$98>>2] = $94;
  $99 = $compptr;
  $100 = (($99) + 48|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = ($101|0)!=(0);
  if ($102) {
   $103 = $blkn;
   $104 = $entropy;
   $105 = (($104) + 152|0);
   $106 = (($105) + ($103<<2)|0);
   HEAP32[$106>>2] = 1;
   $107 = $compptr;
   $108 = (($107) + 36|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = ($109|0)>(1);
   $111 = $110&1;
   $112 = $blkn;
   $113 = $entropy;
   $114 = (($113) + 192|0);
   $115 = (($114) + ($112<<2)|0);
   HEAP32[$115>>2] = $111;
  } else {
   $116 = $blkn;
   $117 = $entropy;
   $118 = (($117) + 192|0);
   $119 = (($118) + ($116<<2)|0);
   HEAP32[$119>>2] = 0;
   $120 = $blkn;
   $121 = $entropy;
   $122 = (($121) + 152|0);
   $123 = (($122) + ($120<<2)|0);
   HEAP32[$123>>2] = 0;
  }
  $124 = $blkn;
  $125 = (($124) + 1)|0;
  $blkn = $125;
 }
 $126 = $entropy;
 $127 = (($126) + 12|0);
 $128 = (($127) + 4|0);
 HEAP32[$128>>2] = 0;
 $129 = $entropy;
 $130 = (($129) + 12|0);
 HEAP32[$130>>2] = 0;
 $131 = $entropy;
 $132 = (($131) + 8|0);
 HEAP32[$132>>2] = 0;
 $133 = $0;
 $134 = (($133) + 276|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = $entropy;
 $137 = (($136) + 36|0);
 HEAP32[$137>>2] = $135;
 STACKTOP = sp;return;
}
function _decode_mcu($cinfo,$MCU_data) {
 $cinfo = $cinfo|0;
 $MCU_data = $MCU_data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $actbl = 0, $bits_left = 0, $blkn = 0, $block = 0, $br_state = 0, $ci = 0, $dctbl = 0, $entropy = 0, $get_buffer = 0, $k = 0, $look = 0, $look2 = 0, $look4 = 0, $nb = 0, $nb1 = 0;
 var $nb3 = 0, $r = 0, $s = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $br_state = sp + 72|0;
 $state = sp + 56|0;
 $1 = $cinfo;
 $2 = $MCU_data;
 $3 = $1;
 $4 = (($3) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 $entropy = $5;
 $6 = $1;
 $7 = (($6) + 276|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = $entropy;
  $11 = (($10) + 36|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)==(0);
  do {
   if ($13) {
    $14 = $1;
    $15 = (_process_restart1908($14)|0);
    $16 = ($15|0)!=(0);
    if ($16) {
     break;
    }
    $0 = 0;
    $364 = $0;
    STACKTOP = sp;return ($364|0);
   }
  } while(0);
 }
 $17 = $entropy;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0);
 L11: do {
  if (!($20)) {
   $21 = $1;
   $22 = (($br_state) + 16|0);
   HEAP32[$22>>2] = $21;
   $23 = $1;
   $24 = (($23) + 24|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$br_state>>2] = $26;
   $27 = $1;
   $28 = (($27) + 24|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($br_state) + 4|0);
   HEAP32[$32>>2] = $31;
   $33 = $entropy;
   $34 = (($33) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $get_buffer = $35;
   $36 = $entropy;
   $37 = (($36) + 12|0);
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $bits_left = $39;
   $40 = $entropy;
   $41 = (($40) + 20|0);
   ;HEAP32[$state+0>>2]=HEAP32[$41+0>>2]|0;HEAP32[$state+4>>2]=HEAP32[$41+4>>2]|0;HEAP32[$state+8>>2]=HEAP32[$41+8>>2]|0;HEAP32[$state+12>>2]=HEAP32[$41+12>>2]|0;
   $blkn = 0;
   L13: while(1) {
    $42 = $blkn;
    $43 = $1;
    $44 = (($43) + 356|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($42|0)<($45|0);
    if (!($46)) {
     label = 91;
     break;
    }
    $47 = $blkn;
    $48 = $2;
    $49 = (($48) + ($47<<2)|0);
    $50 = HEAP32[$49>>2]|0;
    $block = $50;
    $51 = $blkn;
    $52 = $entropy;
    $53 = (($52) + 72|0);
    $54 = (($53) + ($51<<2)|0);
    $55 = HEAP32[$54>>2]|0;
    $dctbl = $55;
    $56 = $blkn;
    $57 = $entropy;
    $58 = (($57) + 112|0);
    $59 = (($58) + ($56<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $actbl = $60;
    $61 = $bits_left;
    $62 = ($61|0)<(8);
    do {
     if ($62) {
      $63 = $get_buffer;
      $64 = $bits_left;
      $65 = (_jpeg_fill_bit_buffer($br_state,$63,$64,0)|0);
      $66 = ($65|0)!=(0);
      if (!($66)) {
       label = 12;
       break L13;
      }
      $67 = (($br_state) + 8|0);
      $68 = HEAP32[$67>>2]|0;
      $get_buffer = $68;
      $69 = (($br_state) + 12|0);
      $70 = HEAP32[$69>>2]|0;
      $bits_left = $70;
      $71 = $bits_left;
      $72 = ($71|0)<(8);
      if ($72) {
       $nb = 1;
       label = 19;
       break;
      } else {
       label = 16;
       break;
      }
     } else {
      label = 16;
     }
    } while(0);
    do {
     if ((label|0) == 16) {
      label = 0;
      $73 = $get_buffer;
      $74 = $bits_left;
      $75 = (($74) - 8)|0;
      $76 = $73 >> $75;
      $77 = $76 & 255;
      $look = $77;
      $78 = $look;
      $79 = $dctbl;
      $80 = (($79) + 144|0);
      $81 = (($80) + ($78<<2)|0);
      $82 = HEAP32[$81>>2]|0;
      $nb = $82;
      $83 = ($82|0)!=(0);
      if ($83) {
       $84 = $nb;
       $85 = $bits_left;
       $86 = (($85) - ($84))|0;
       $bits_left = $86;
       $87 = $look;
       $88 = $dctbl;
       $89 = (($88) + 1168|0);
       $90 = (($89) + ($87)|0);
       $91 = HEAP8[$90>>0]|0;
       $92 = $91&255;
       $s = $92;
       break;
      } else {
       $nb = 9;
       label = 19;
       break;
      }
     }
    } while(0);
    if ((label|0) == 19) {
     label = 0;
     $93 = $get_buffer;
     $94 = $bits_left;
     $95 = $dctbl;
     $96 = $nb;
     $97 = (_jpeg_huff_decode($br_state,$93,$94,$95,$96)|0);
     $s = $97;
     $98 = ($97|0)<(0);
     if ($98) {
      label = 20;
      break;
     }
     $99 = (($br_state) + 8|0);
     $100 = HEAP32[$99>>2]|0;
     $get_buffer = $100;
     $101 = (($br_state) + 12|0);
     $102 = HEAP32[$101>>2]|0;
     $bits_left = $102;
    }
    $103 = $s;
    $104 = ($103|0)!=(0);
    if ($104) {
     $105 = $bits_left;
     $106 = $s;
     $107 = ($105|0)<($106|0);
     if ($107) {
      $108 = $get_buffer;
      $109 = $bits_left;
      $110 = $s;
      $111 = (_jpeg_fill_bit_buffer($br_state,$108,$109,$110)|0);
      $112 = ($111|0)!=(0);
      if (!($112)) {
       label = 25;
       break;
      }
      $113 = (($br_state) + 8|0);
      $114 = HEAP32[$113>>2]|0;
      $get_buffer = $114;
      $115 = (($br_state) + 12|0);
      $116 = HEAP32[$115>>2]|0;
      $bits_left = $116;
     }
     $117 = $get_buffer;
     $118 = $s;
     $119 = $bits_left;
     $120 = (($119) - ($118))|0;
     $bits_left = $120;
     $121 = $117 >> $120;
     $122 = $s;
     $123 = 1 << $122;
     $124 = (($123) - 1)|0;
     $125 = $121 & $124;
     $r = $125;
     $126 = $r;
     $127 = $s;
     $128 = (51080 + ($127<<2)|0);
     $129 = HEAP32[$128>>2]|0;
     $130 = ($126|0)<($129|0);
     if ($130) {
      $131 = $r;
      $132 = $s;
      $133 = (51144 + ($132<<2)|0);
      $134 = HEAP32[$133>>2]|0;
      $135 = (($131) + ($134))|0;
      $137 = $135;
     } else {
      $136 = $r;
      $137 = $136;
     }
     $s = $137;
    }
    $138 = $blkn;
    $139 = $entropy;
    $140 = (($139) + 152|0);
    $141 = (($140) + ($138<<2)|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ($142|0)!=(0);
    if ($143) {
     $144 = $blkn;
     $145 = $1;
     $146 = (($145) + 360|0);
     $147 = (($146) + ($144<<2)|0);
     $148 = HEAP32[$147>>2]|0;
     $ci = $148;
     $149 = $ci;
     $150 = (($state) + ($149<<2)|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = $s;
     $153 = (($152) + ($151))|0;
     $s = $153;
     $154 = $s;
     $155 = $ci;
     $156 = (($state) + ($155<<2)|0);
     HEAP32[$156>>2] = $154;
     $157 = $s;
     $158 = $157&65535;
     $159 = $block;
     HEAP16[$159>>1] = $158;
    }
    $160 = $blkn;
    $161 = $entropy;
    $162 = (($161) + 192|0);
    $163 = (($162) + ($160<<2)|0);
    $164 = HEAP32[$163>>2]|0;
    $165 = ($164|0)!=(0);
    if ($165) {
     $k = 1;
     while(1) {
      $166 = $k;
      $167 = ($166|0)<(64);
      if (!($167)) {
       break;
      }
      $168 = $bits_left;
      $169 = ($168|0)<(8);
      do {
       if ($169) {
        $170 = $get_buffer;
        $171 = $bits_left;
        $172 = (_jpeg_fill_bit_buffer($br_state,$170,$171,0)|0);
        $173 = ($172|0)!=(0);
        if (!($173)) {
         label = 38;
         break L13;
        }
        $174 = (($br_state) + 8|0);
        $175 = HEAP32[$174>>2]|0;
        $get_buffer = $175;
        $176 = (($br_state) + 12|0);
        $177 = HEAP32[$176>>2]|0;
        $bits_left = $177;
        $178 = $bits_left;
        $179 = ($178|0)<(8);
        if ($179) {
         $nb1 = 1;
         label = 45;
         break;
        } else {
         label = 42;
         break;
        }
       } else {
        label = 42;
       }
      } while(0);
      do {
       if ((label|0) == 42) {
        label = 0;
        $180 = $get_buffer;
        $181 = $bits_left;
        $182 = (($181) - 8)|0;
        $183 = $180 >> $182;
        $184 = $183 & 255;
        $look2 = $184;
        $185 = $look2;
        $186 = $actbl;
        $187 = (($186) + 144|0);
        $188 = (($187) + ($185<<2)|0);
        $189 = HEAP32[$188>>2]|0;
        $nb1 = $189;
        $190 = ($189|0)!=(0);
        if ($190) {
         $191 = $nb1;
         $192 = $bits_left;
         $193 = (($192) - ($191))|0;
         $bits_left = $193;
         $194 = $look2;
         $195 = $actbl;
         $196 = (($195) + 1168|0);
         $197 = (($196) + ($194)|0);
         $198 = HEAP8[$197>>0]|0;
         $199 = $198&255;
         $s = $199;
         break;
        } else {
         $nb1 = 9;
         label = 45;
         break;
        }
       }
      } while(0);
      if ((label|0) == 45) {
       label = 0;
       $200 = $get_buffer;
       $201 = $bits_left;
       $202 = $actbl;
       $203 = $nb1;
       $204 = (_jpeg_huff_decode($br_state,$200,$201,$202,$203)|0);
       $s = $204;
       $205 = ($204|0)<(0);
       if ($205) {
        label = 46;
        break L13;
       }
       $206 = (($br_state) + 8|0);
       $207 = HEAP32[$206>>2]|0;
       $get_buffer = $207;
       $208 = (($br_state) + 12|0);
       $209 = HEAP32[$208>>2]|0;
       $bits_left = $209;
      }
      $210 = $s;
      $211 = $210 >> 4;
      $r = $211;
      $212 = $s;
      $213 = $212 & 15;
      $s = $213;
      $214 = $s;
      $215 = ($214|0)!=(0);
      if ($215) {
       $216 = $r;
       $217 = $k;
       $218 = (($217) + ($216))|0;
       $k = $218;
       $219 = $bits_left;
       $220 = $s;
       $221 = ($219|0)<($220|0);
       if ($221) {
        $222 = $get_buffer;
        $223 = $bits_left;
        $224 = $s;
        $225 = (_jpeg_fill_bit_buffer($br_state,$222,$223,$224)|0);
        $226 = ($225|0)!=(0);
        if (!($226)) {
         label = 51;
         break L13;
        }
        $227 = (($br_state) + 8|0);
        $228 = HEAP32[$227>>2]|0;
        $get_buffer = $228;
        $229 = (($br_state) + 12|0);
        $230 = HEAP32[$229>>2]|0;
        $bits_left = $230;
       }
       $231 = $get_buffer;
       $232 = $s;
       $233 = $bits_left;
       $234 = (($233) - ($232))|0;
       $bits_left = $234;
       $235 = $231 >> $234;
       $236 = $s;
       $237 = 1 << $236;
       $238 = (($237) - 1)|0;
       $239 = $235 & $238;
       $r = $239;
       $240 = $r;
       $241 = $s;
       $242 = (51080 + ($241<<2)|0);
       $243 = HEAP32[$242>>2]|0;
       $244 = ($240|0)<($243|0);
       if ($244) {
        $245 = $r;
        $246 = $s;
        $247 = (51144 + ($246<<2)|0);
        $248 = HEAP32[$247>>2]|0;
        $249 = (($245) + ($248))|0;
        $251 = $249;
       } else {
        $250 = $r;
        $251 = $250;
       }
       $s = $251;
       $252 = $s;
       $253 = $252&65535;
       $254 = $k;
       $255 = (50504 + ($254<<2)|0);
       $256 = HEAPU8[$255>>0]|(HEAPU8[$255+1>>0]<<8)|(HEAPU8[$255+2>>0]<<16)|(HEAPU8[$255+3>>0]<<24);
       $257 = $block;
       $258 = (($257) + ($256<<1)|0);
       HEAP16[$258>>1] = $253;
      } else {
       $259 = $r;
       $260 = ($259|0)!=(15);
       if ($260) {
        label = 58;
        break;
       }
       $261 = $k;
       $262 = (($261) + 15)|0;
       $k = $262;
      }
      $263 = $k;
      $264 = (($263) + 1)|0;
      $k = $264;
     }
     if ((label|0) == 58) {
      label = 0;
     }
    } else {
     $k = 1;
     while(1) {
      $265 = $k;
      $266 = ($265|0)<(64);
      if (!($266)) {
       break;
      }
      $267 = $bits_left;
      $268 = ($267|0)<(8);
      do {
       if ($268) {
        $269 = $get_buffer;
        $270 = $bits_left;
        $271 = (_jpeg_fill_bit_buffer($br_state,$269,$270,0)|0);
        $272 = ($271|0)!=(0);
        if (!($272)) {
         label = 67;
         break L13;
        }
        $273 = (($br_state) + 8|0);
        $274 = HEAP32[$273>>2]|0;
        $get_buffer = $274;
        $275 = (($br_state) + 12|0);
        $276 = HEAP32[$275>>2]|0;
        $bits_left = $276;
        $277 = $bits_left;
        $278 = ($277|0)<(8);
        if ($278) {
         $nb3 = 1;
         label = 74;
         break;
        } else {
         label = 71;
         break;
        }
       } else {
        label = 71;
       }
      } while(0);
      do {
       if ((label|0) == 71) {
        label = 0;
        $279 = $get_buffer;
        $280 = $bits_left;
        $281 = (($280) - 8)|0;
        $282 = $279 >> $281;
        $283 = $282 & 255;
        $look4 = $283;
        $284 = $look4;
        $285 = $actbl;
        $286 = (($285) + 144|0);
        $287 = (($286) + ($284<<2)|0);
        $288 = HEAP32[$287>>2]|0;
        $nb3 = $288;
        $289 = ($288|0)!=(0);
        if ($289) {
         $290 = $nb3;
         $291 = $bits_left;
         $292 = (($291) - ($290))|0;
         $bits_left = $292;
         $293 = $look4;
         $294 = $actbl;
         $295 = (($294) + 1168|0);
         $296 = (($295) + ($293)|0);
         $297 = HEAP8[$296>>0]|0;
         $298 = $297&255;
         $s = $298;
         break;
        } else {
         $nb3 = 9;
         label = 74;
         break;
        }
       }
      } while(0);
      if ((label|0) == 74) {
       label = 0;
       $299 = $get_buffer;
       $300 = $bits_left;
       $301 = $actbl;
       $302 = $nb3;
       $303 = (_jpeg_huff_decode($br_state,$299,$300,$301,$302)|0);
       $s = $303;
       $304 = ($303|0)<(0);
       if ($304) {
        label = 75;
        break L13;
       }
       $305 = (($br_state) + 8|0);
       $306 = HEAP32[$305>>2]|0;
       $get_buffer = $306;
       $307 = (($br_state) + 12|0);
       $308 = HEAP32[$307>>2]|0;
       $bits_left = $308;
      }
      $309 = $s;
      $310 = $309 >> 4;
      $r = $310;
      $311 = $s;
      $312 = $311 & 15;
      $s = $312;
      $313 = $s;
      $314 = ($313|0)!=(0);
      if ($314) {
       $315 = $r;
       $316 = $k;
       $317 = (($316) + ($315))|0;
       $k = $317;
       $318 = $bits_left;
       $319 = $s;
       $320 = ($318|0)<($319|0);
       if ($320) {
        $321 = $get_buffer;
        $322 = $bits_left;
        $323 = $s;
        $324 = (_jpeg_fill_bit_buffer($br_state,$321,$322,$323)|0);
        $325 = ($324|0)!=(0);
        if (!($325)) {
         label = 80;
         break L13;
        }
        $326 = (($br_state) + 8|0);
        $327 = HEAP32[$326>>2]|0;
        $get_buffer = $327;
        $328 = (($br_state) + 12|0);
        $329 = HEAP32[$328>>2]|0;
        $bits_left = $329;
       }
       $330 = $s;
       $331 = $bits_left;
       $332 = (($331) - ($330))|0;
       $bits_left = $332;
      } else {
       $333 = $r;
       $334 = ($333|0)!=(15);
       if ($334) {
        label = 84;
        break;
       }
       $335 = $k;
       $336 = (($335) + 15)|0;
       $k = $336;
      }
      $337 = $k;
      $338 = (($337) + 1)|0;
      $k = $338;
     }
     if ((label|0) == 84) {
      label = 0;
     }
    }
    $339 = $blkn;
    $340 = (($339) + 1)|0;
    $blkn = $340;
   }
   switch (label|0) {
    case 12: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 20: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 25: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 38: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 46: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 51: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 67: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 75: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 80: {
     $0 = 0;
     $364 = $0;
     STACKTOP = sp;return ($364|0);
     break;
    }
    case 91: {
     $341 = HEAP32[$br_state>>2]|0;
     $342 = $1;
     $343 = (($342) + 24|0);
     $344 = HEAP32[$343>>2]|0;
     HEAP32[$344>>2] = $341;
     $345 = (($br_state) + 4|0);
     $346 = HEAP32[$345>>2]|0;
     $347 = $1;
     $348 = (($347) + 24|0);
     $349 = HEAP32[$348>>2]|0;
     $350 = (($349) + 4|0);
     HEAP32[$350>>2] = $346;
     $351 = $get_buffer;
     $352 = $entropy;
     $353 = (($352) + 12|0);
     HEAP32[$353>>2] = $351;
     $354 = $bits_left;
     $355 = $entropy;
     $356 = (($355) + 12|0);
     $357 = (($356) + 4|0);
     HEAP32[$357>>2] = $354;
     $358 = $entropy;
     $359 = (($358) + 20|0);
     ;HEAP32[$359+0>>2]=HEAP32[$state+0>>2]|0;HEAP32[$359+4>>2]=HEAP32[$state+4>>2]|0;HEAP32[$359+8>>2]=HEAP32[$state+8>>2]|0;HEAP32[$359+12>>2]=HEAP32[$state+12>>2]|0;
     break L11;
     break;
    }
   }
  }
 } while(0);
 $360 = $entropy;
 $361 = (($360) + 36|0);
 $362 = HEAP32[$361>>2]|0;
 $363 = (($362) + -1)|0;
 HEAP32[$361>>2] = $363;
 $0 = 1;
 $364 = $0;
 STACKTOP = sp;return ($364|0);
}
function _process_restart1908($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ci = 0, $entropy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = $1;
 $3 = (($2) + 444|0);
 $4 = HEAP32[$3>>2]|0;
 $entropy = $4;
 $5 = $entropy;
 $6 = (($5) + 12|0);
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8|0) / 8)&-1;
 $10 = $1;
 $11 = (($10) + 440|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($9))|0;
 HEAP32[$13>>2] = $15;
 $16 = $entropy;
 $17 = (($16) + 12|0);
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = 0;
 $19 = $1;
 $20 = (($19) + 440|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = (FUNCTION_TABLE_ii[$23 & 127]($24)|0);
 $26 = ($25|0)!=(0);
 if (!($26)) {
  $0 = 0;
  $49 = $0;
  STACKTOP = sp;return ($49|0);
 }
 $ci = 0;
 while(1) {
  $27 = $ci;
  $28 = $1;
  $29 = (($28) + 328|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = $ci;
  $33 = $entropy;
  $34 = (($33) + 20|0);
  $35 = (($34) + ($32<<2)|0);
  HEAP32[$35>>2] = 0;
  $36 = $ci;
  $37 = (($36) + 1)|0;
  $ci = $37;
 }
 $38 = $1;
 $39 = (($38) + 276|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $entropy;
 $42 = (($41) + 36|0);
 HEAP32[$42>>2] = $40;
 $43 = $1;
 $44 = (($43) + 416|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  $47 = $entropy;
  $48 = (($47) + 8|0);
  HEAP32[$48>>2] = 0;
 }
 $0 = 1;
 $49 = $0;
 STACKTOP = sp;return ($49|0);
}
function _jinit_d_main_controller($cinfo,$need_full_buffer) {
 $cinfo = $cinfo|0;
 $need_full_buffer = $need_full_buffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $ci = 0, $compptr = 0, $main = 0, $ngroups = 0, $rgroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $need_full_buffer;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (FUNCTION_TABLE_iiii[$5 & 255]($6,1,80)|0);
 $main = $7;
 $8 = $main;
 $9 = $0;
 $10 = (($9) + 424|0);
 HEAP32[$10>>2] = $8;
 $11 = $main;
 HEAP32[$11>>2] = 127;
 $12 = $1;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $0;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 20|0);
  HEAP32[$16>>2] = 4;
  $17 = $0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = $0;
  FUNCTION_TABLE_vi[$19 & 255]($20);
 }
 $21 = $0;
 $22 = (($21) + 452|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $0;
  $28 = (($27) + 316|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)<(2);
  if ($30) {
   $31 = $0;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 20|0);
   HEAP32[$33>>2] = 47;
   $34 = $0;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $0;
   FUNCTION_TABLE_vi[$36 & 255]($37);
  }
  $38 = $0;
  _alloc_funny_pointers($38);
  $39 = $0;
  $40 = (($39) + 316|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 2)|0;
  $ngroups = $42;
 } else {
  $43 = $0;
  $44 = (($43) + 316|0);
  $45 = HEAP32[$44>>2]|0;
  $ngroups = $45;
 }
 $ci = 0;
 $46 = $0;
 $47 = (($46) + 216|0);
 $48 = HEAP32[$47>>2]|0;
 $compptr = $48;
 while(1) {
  $49 = $ci;
  $50 = $0;
  $51 = (($50) + 36|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($49|0)<($52|0);
  if (!($53)) {
   break;
  }
  $54 = $compptr;
  $55 = (($54) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $compptr;
  $58 = (($57) + 36|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = Math_imul($56, $59)|0;
  $61 = $0;
  $62 = (($61) + 316|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($60|0) / ($63|0))&-1;
  $rgroup = $64;
  $65 = $0;
  $66 = (($65) + 4|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = $0;
  $71 = $compptr;
  $72 = (($71) + 28|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $compptr;
  $75 = (($74) + 36|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = Math_imul($73, $76)|0;
  $78 = $rgroup;
  $79 = $ngroups;
  $80 = Math_imul($78, $79)|0;
  $81 = (FUNCTION_TABLE_iiiii[$69 & 255]($70,1,$77,$80)|0);
  $82 = $ci;
  $83 = $main;
  $84 = (($83) + 8|0);
  $85 = (($84) + ($82<<2)|0);
  HEAP32[$85>>2] = $81;
  $86 = $ci;
  $87 = (($86) + 1)|0;
  $ci = $87;
  $88 = $compptr;
  $89 = (($88) + 84|0);
  $compptr = $89;
 }
 STACKTOP = sp;return;
}
function _start_pass_main($cinfo,$pass_mode) {
 $cinfo = $cinfo|0;
 $pass_mode = $pass_mode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $main = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $pass_mode;
 $2 = $0;
 $3 = (($2) + 424|0);
 $4 = HEAP32[$3>>2]|0;
 $main = $4;
 $5 = $1;
 if ((($5|0) == 0)) {
  $6 = $0;
  $7 = (($6) + 452|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0);
  if ($11) {
   $12 = $main;
   $13 = (($12) + 4|0);
   HEAP32[$13>>2] = 128;
   $14 = $0;
   _make_funny_pointers($14);
   $15 = $main;
   $16 = (($15) + 64|0);
   HEAP32[$16>>2] = 0;
   $17 = $main;
   $18 = (($17) + 68|0);
   HEAP32[$18>>2] = 0;
   $19 = $main;
   $20 = (($19) + 76|0);
   HEAP32[$20>>2] = 0;
  } else {
   $21 = $main;
   $22 = (($21) + 4|0);
   HEAP32[$22>>2] = 129;
  }
  $23 = $main;
  $24 = (($23) + 48|0);
  HEAP32[$24>>2] = 0;
  $25 = $main;
  $26 = (($25) + 52|0);
  HEAP32[$26>>2] = 0;
  STACKTOP = sp;return;
 } else if ((($5|0) == 2)) {
  $27 = $main;
  $28 = (($27) + 4|0);
  HEAP32[$28>>2] = 130;
  STACKTOP = sp;return;
 } else {
  $29 = $0;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 20|0);
  HEAP32[$31>>2] = 4;
  $32 = $0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $0;
  FUNCTION_TABLE_vi[$34 & 255]($35);
  STACKTOP = sp;return;
 }
}
function _alloc_funny_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $M = 0, $ci = 0, $compptr = 0, $main = 0, $rgroup = 0, $xbuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $4 = $0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 $M = $6;
 $7 = $0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = $0;
 $12 = $0;
 $13 = (($12) + 36|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14<<1;
 $16 = $15<<2;
 $17 = (FUNCTION_TABLE_iiii[$10 & 255]($11,1,$16)|0);
 $18 = $main;
 $19 = (($18) + 56|0);
 HEAP32[$19>>2] = $17;
 $20 = $main;
 $21 = (($20) + 56|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = (($23) + 36|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($22) + ($25<<2)|0);
 $27 = $main;
 $28 = (($27) + 56|0);
 $29 = (($28) + 4|0);
 HEAP32[$29>>2] = $26;
 $ci = 0;
 $30 = $0;
 $31 = (($30) + 216|0);
 $32 = HEAP32[$31>>2]|0;
 $compptr = $32;
 while(1) {
  $33 = $ci;
  $34 = $0;
  $35 = (($34) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($33|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = $compptr;
  $39 = (($38) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $compptr;
  $42 = (($41) + 36|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = Math_imul($40, $43)|0;
  $45 = $0;
  $46 = (($45) + 316|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($44|0) / ($47|0))&-1;
  $rgroup = $48;
  $49 = $0;
  $50 = (($49) + 4|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $53 = $0;
  $54 = $rgroup;
  $55 = $M;
  $56 = (($55) + 4)|0;
  $57 = Math_imul($54, $56)|0;
  $58 = $57<<1;
  $59 = $58<<2;
  $60 = (FUNCTION_TABLE_iiii[$52 & 255]($53,1,$59)|0);
  $xbuf = $60;
  $61 = $rgroup;
  $62 = $xbuf;
  $63 = (($62) + ($61<<2)|0);
  $xbuf = $63;
  $64 = $xbuf;
  $65 = $ci;
  $66 = $main;
  $67 = (($66) + 56|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + ($65<<2)|0);
  HEAP32[$69>>2] = $64;
  $70 = $rgroup;
  $71 = $M;
  $72 = (($71) + 4)|0;
  $73 = Math_imul($70, $72)|0;
  $74 = $xbuf;
  $75 = (($74) + ($73<<2)|0);
  $xbuf = $75;
  $76 = $xbuf;
  $77 = $ci;
  $78 = $main;
  $79 = (($78) + 56|0);
  $80 = (($79) + 4|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + ($77<<2)|0);
  HEAP32[$82>>2] = $76;
  $83 = $ci;
  $84 = (($83) + 1)|0;
  $ci = $84;
  $85 = $compptr;
  $86 = (($85) + 84|0);
  $compptr = $86;
 }
 STACKTOP = sp;return;
}
function _process_data_context_main($cinfo,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $main = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $output_buf;
 $2 = $out_row_ctr;
 $3 = $out_rows_avail;
 $4 = $0;
 $5 = (($4) + 424|0);
 $6 = HEAP32[$5>>2]|0;
 $main = $6;
 $7 = $main;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 do {
  if (!($10)) {
   $11 = $0;
   $12 = (($11) + 428|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (($13) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $main;
   $18 = (($17) + 64|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = $main;
   $21 = (($20) + 56|0);
   $22 = (($21) + ($19<<2)|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (FUNCTION_TABLE_iii[$15 & 255]($16,$23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $main;
    $27 = (($26) + 48|0);
    HEAP32[$27>>2] = 1;
    $28 = $main;
    $29 = (($28) + 76|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$29>>2] = $31;
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 $32 = $main;
 $33 = (($32) + 68|0);
 $34 = HEAP32[$33>>2]|0;
 if ((($34|0) == 2)) {
  label = 6;
 } else if ((($34|0) == 0)) {
  label = 11;
 } else if (!((($34|0) == 1))) {
  STACKTOP = sp;return;
 }
 do {
  if ((label|0) == 6) {
   $35 = $0;
   $36 = (($35) + 432|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $0;
   $41 = $main;
   $42 = (($41) + 64|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = $main;
   $45 = (($44) + 56|0);
   $46 = (($45) + ($43<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $main;
   $49 = (($48) + 52|0);
   $50 = $main;
   $51 = (($50) + 72|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = $1;
   $54 = $2;
   $55 = $3;
   FUNCTION_TABLE_viiiiiii[$39 & 63]($40,$47,$49,$52,$53,$54,$55);
   $56 = $main;
   $57 = (($56) + 52|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = $main;
   $60 = (($59) + 72|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ($58>>>0)<($61>>>0);
   if ($62) {
    STACKTOP = sp;return;
   }
   $63 = $main;
   $64 = (($63) + 68|0);
   HEAP32[$64>>2] = 0;
   $65 = $2;
   $66 = HEAP32[$65>>2]|0;
   $67 = $3;
   $68 = ($66>>>0)>=($67>>>0);
   if (!($68)) {
    label = 11;
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 if ((label|0) == 11) {
  $69 = $main;
  $70 = (($69) + 52|0);
  HEAP32[$70>>2] = 0;
  $71 = $0;
  $72 = (($71) + 316|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) - 1)|0;
  $75 = $main;
  $76 = (($75) + 72|0);
  HEAP32[$76>>2] = $74;
  $77 = $main;
  $78 = (($77) + 76|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $0;
  $81 = (($80) + 320|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($79|0)==($82|0);
  if ($83) {
   $84 = $0;
   _set_bottom_pointers($84);
  }
  $85 = $main;
  $86 = (($85) + 68|0);
  HEAP32[$86>>2] = 1;
 }
 $87 = $0;
 $88 = (($87) + 432|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + 4|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $0;
 $93 = $main;
 $94 = (($93) + 64|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = $main;
 $97 = (($96) + 56|0);
 $98 = (($97) + ($95<<2)|0);
 $99 = HEAP32[$98>>2]|0;
 $100 = $main;
 $101 = (($100) + 52|0);
 $102 = $main;
 $103 = (($102) + 72|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = $1;
 $106 = $2;
 $107 = $3;
 FUNCTION_TABLE_viiiiiii[$91 & 63]($92,$99,$101,$104,$105,$106,$107);
 $108 = $main;
 $109 = (($108) + 52|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = $main;
 $112 = (($111) + 72|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = ($110>>>0)<($113>>>0);
 if ($114) {
  STACKTOP = sp;return;
 }
 $115 = $main;
 $116 = (($115) + 76|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = ($117|0)==(1);
 if ($118) {
  $119 = $0;
  _set_wraparound_pointers($119);
 }
 $120 = $main;
 $121 = (($120) + 64|0);
 $122 = HEAP32[$121>>2]|0;
 $123 = $122 ^ 1;
 HEAP32[$121>>2] = $123;
 $124 = $main;
 $125 = (($124) + 48|0);
 HEAP32[$125>>2] = 0;
 $126 = $0;
 $127 = (($126) + 316|0);
 $128 = HEAP32[$127>>2]|0;
 $129 = (($128) + 1)|0;
 $130 = $main;
 $131 = (($130) + 52|0);
 HEAP32[$131>>2] = $129;
 $132 = $0;
 $133 = (($132) + 316|0);
 $134 = HEAP32[$133>>2]|0;
 $135 = (($134) + 2)|0;
 $136 = $main;
 $137 = (($136) + 72|0);
 HEAP32[$137>>2] = $135;
 $138 = $main;
 $139 = (($138) + 68|0);
 HEAP32[$139>>2] = 2;
 STACKTOP = sp;return;
}
function _make_funny_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $M = 0, $buf = 0, $ci = 0, $compptr = 0;
 var $i = 0, $main = 0, $rgroup = 0, $xbuf0 = 0, $xbuf1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $4 = $0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 $M = $6;
 $ci = 0;
 $7 = $0;
 $8 = (($7) + 216|0);
 $9 = HEAP32[$8>>2]|0;
 $compptr = $9;
 while(1) {
  $10 = $ci;
  $11 = $0;
  $12 = (($11) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $compptr;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $compptr;
  $19 = (($18) + 36|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = Math_imul($17, $20)|0;
  $22 = $0;
  $23 = (($22) + 316|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($21|0) / ($24|0))&-1;
  $rgroup = $25;
  $26 = $ci;
  $27 = $main;
  $28 = (($27) + 56|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $xbuf0 = $31;
  $32 = $ci;
  $33 = $main;
  $34 = (($33) + 56|0);
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + ($32<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $xbuf1 = $38;
  $39 = $ci;
  $40 = $main;
  $41 = (($40) + 8|0);
  $42 = (($41) + ($39<<2)|0);
  $43 = HEAP32[$42>>2]|0;
  $buf = $43;
  $i = 0;
  while(1) {
   $44 = $i;
   $45 = $rgroup;
   $46 = $M;
   $47 = (($46) + 2)|0;
   $48 = Math_imul($45, $47)|0;
   $49 = ($44|0)<($48|0);
   if (!($49)) {
    break;
   }
   $50 = $i;
   $51 = $buf;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = $i;
   $55 = $xbuf1;
   $56 = (($55) + ($54<<2)|0);
   HEAP32[$56>>2] = $53;
   $57 = $i;
   $58 = $xbuf0;
   $59 = (($58) + ($57<<2)|0);
   HEAP32[$59>>2] = $53;
   $60 = $i;
   $61 = (($60) + 1)|0;
   $i = $61;
  }
  $i = 0;
  while(1) {
   $62 = $i;
   $63 = $rgroup;
   $64 = $63<<1;
   $65 = ($62|0)<($64|0);
   if (!($65)) {
    break;
   }
   $66 = $rgroup;
   $67 = $M;
   $68 = Math_imul($66, $67)|0;
   $69 = $i;
   $70 = (($68) + ($69))|0;
   $71 = $buf;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = $rgroup;
   $75 = $M;
   $76 = (($75) - 2)|0;
   $77 = Math_imul($74, $76)|0;
   $78 = $i;
   $79 = (($77) + ($78))|0;
   $80 = $xbuf1;
   $81 = (($80) + ($79<<2)|0);
   HEAP32[$81>>2] = $73;
   $82 = $rgroup;
   $83 = $M;
   $84 = (($83) - 2)|0;
   $85 = Math_imul($82, $84)|0;
   $86 = $i;
   $87 = (($85) + ($86))|0;
   $88 = $buf;
   $89 = (($88) + ($87<<2)|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = $rgroup;
   $92 = $M;
   $93 = Math_imul($91, $92)|0;
   $94 = $i;
   $95 = (($93) + ($94))|0;
   $96 = $xbuf1;
   $97 = (($96) + ($95<<2)|0);
   HEAP32[$97>>2] = $90;
   $98 = $i;
   $99 = (($98) + 1)|0;
   $i = $99;
  }
  $i = 0;
  while(1) {
   $100 = $i;
   $101 = $rgroup;
   $102 = ($100|0)<($101|0);
   if (!($102)) {
    break;
   }
   $103 = $xbuf0;
   $104 = HEAP32[$103>>2]|0;
   $105 = $i;
   $106 = $rgroup;
   $107 = (($105) - ($106))|0;
   $108 = $xbuf0;
   $109 = (($108) + ($107<<2)|0);
   HEAP32[$109>>2] = $104;
   $110 = $i;
   $111 = (($110) + 1)|0;
   $i = $111;
  }
  $112 = $ci;
  $113 = (($112) + 1)|0;
  $ci = $113;
  $114 = $compptr;
  $115 = (($114) + 84|0);
  $compptr = $115;
 }
 STACKTOP = sp;return;
}
function _process_data_simple_main($cinfo,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $main = 0, $rowgroups_avail = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $output_buf;
 $2 = $out_row_ctr;
 $3 = $out_rows_avail;
 $4 = $0;
 $5 = (($4) + 424|0);
 $6 = HEAP32[$5>>2]|0;
 $main = $6;
 $7 = $main;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 do {
  if (!($10)) {
   $11 = $0;
   $12 = (($11) + 428|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (($13) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $main;
   $18 = (($17) + 8|0);
   $19 = (FUNCTION_TABLE_iii[$15 & 255]($16,$18)|0);
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = $main;
    $22 = (($21) + 48|0);
    HEAP32[$22>>2] = 1;
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 $23 = $0;
 $24 = (($23) + 316|0);
 $25 = HEAP32[$24>>2]|0;
 $rowgroups_avail = $25;
 $26 = $0;
 $27 = (($26) + 432|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $0;
 $32 = $main;
 $33 = (($32) + 8|0);
 $34 = $main;
 $35 = (($34) + 52|0);
 $36 = $rowgroups_avail;
 $37 = $1;
 $38 = $2;
 $39 = $3;
 FUNCTION_TABLE_viiiiiii[$30 & 63]($31,$33,$35,$36,$37,$38,$39);
 $40 = $main;
 $41 = (($40) + 52|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $rowgroups_avail;
 $44 = ($42>>>0)>=($43>>>0);
 if (!($44)) {
  STACKTOP = sp;return;
 }
 $45 = $main;
 $46 = (($45) + 48|0);
 HEAP32[$46>>2] = 0;
 $47 = $main;
 $48 = (($47) + 52|0);
 HEAP32[$48>>2] = 0;
 STACKTOP = sp;return;
}
function _process_data_crank_post($cinfo,$output_buf,$out_row_ctr,$out_rows_avail) {
 $cinfo = $cinfo|0;
 $output_buf = $output_buf|0;
 $out_row_ctr = $out_row_ctr|0;
 $out_rows_avail = $out_rows_avail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cinfo;
 $1 = $output_buf;
 $2 = $out_row_ctr;
 $3 = $out_rows_avail;
 $4 = $0;
 $5 = (($4) + 432|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = $1;
 $11 = $2;
 $12 = $3;
 FUNCTION_TABLE_viiiiiii[$8 & 63]($9,0,0,0,$10,$11,$12);
 STACKTOP = sp;return;
}
function _set_bottom_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $ci = 0, $compptr = 0, $i = 0, $iMCUheight = 0, $main = 0, $rgroup = 0, $rows_left = 0, $xbuf = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $ci = 0;
 $4 = $0;
 $5 = (($4) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $compptr = $6;
 while(1) {
  $7 = $ci;
  $8 = $0;
  $9 = (($8) + 36|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = $compptr;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $compptr;
  $16 = (($15) + 36|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = Math_imul($14, $17)|0;
  $iMCUheight = $18;
  $19 = $iMCUheight;
  $20 = $0;
  $21 = (($20) + 316|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19|0) / ($22|0))&-1;
  $rgroup = $23;
  $24 = $compptr;
  $25 = (($24) + 44|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $iMCUheight;
  $28 = (($26>>>0) % ($27>>>0))&-1;
  $rows_left = $28;
  $29 = $rows_left;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = $iMCUheight;
   $rows_left = $31;
  }
  $32 = $ci;
  $33 = ($32|0)==(0);
  if ($33) {
   $34 = $rows_left;
   $35 = (($34) - 1)|0;
   $36 = $rgroup;
   $37 = (($35|0) / ($36|0))&-1;
   $38 = (($37) + 1)|0;
   $39 = $main;
   $40 = (($39) + 72|0);
   HEAP32[$40>>2] = $38;
  }
  $41 = $ci;
  $42 = $main;
  $43 = (($42) + 64|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = $main;
  $46 = (($45) + 56|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($41<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $xbuf = $50;
  $i = 0;
  while(1) {
   $51 = $i;
   $52 = $rgroup;
   $53 = $52<<1;
   $54 = ($51|0)<($53|0);
   if (!($54)) {
    break;
   }
   $55 = $rows_left;
   $56 = (($55) - 1)|0;
   $57 = $xbuf;
   $58 = (($57) + ($56<<2)|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $rows_left;
   $61 = $i;
   $62 = (($60) + ($61))|0;
   $63 = $xbuf;
   $64 = (($63) + ($62<<2)|0);
   HEAP32[$64>>2] = $59;
   $65 = $i;
   $66 = (($65) + 1)|0;
   $i = $66;
  }
  $67 = $ci;
  $68 = (($67) + 1)|0;
  $ci = $68;
  $69 = $compptr;
  $70 = (($69) + 84|0);
  $compptr = $70;
 }
 STACKTOP = sp;return;
}
function _set_wraparound_pointers($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $M = 0, $ci = 0, $compptr = 0, $i = 0, $main = 0, $rgroup = 0, $xbuf0 = 0, $xbuf1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $cinfo;
 $1 = $0;
 $2 = (($1) + 424|0);
 $3 = HEAP32[$2>>2]|0;
 $main = $3;
 $4 = $0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 $M = $6;
 $ci = 0;
 $7 = $0;
 $8 = (($7) + 216|0);
 $9 = HEAP32[$8>>2]|0;
 $compptr = $9;
 while(1) {
  $10 = $ci;
  $11 = $0;
  $12 = (($11) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $compptr;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $compptr;
  $19 = (($18) + 36|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = Math_imul($17, $20)|0;
  $22 = $0;
  $23 = (($22) + 316|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($21|0) / ($24|0))&-1;
  $rgroup = $25;
  $26 = $ci;
  $27 = $main;
  $28 = (($27) + 56|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $xbuf0 = $31;
  $32 = $ci;
  $33 = $main;
  $34 = (($33) + 56|0);
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + ($32<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $xbuf1 = $38;
  $i = 0;
  while(1) {
   $39 = $i;
   $40 = $rgroup;
   $41 = ($39|0)<($40|0);
   if (!($41)) {
    break;
   }
   $42 = $rgroup;
   $43 = $M;
   $44 = (($43) + 1)|0;
   $45 = Math_imul($42, $44)|0;
   $46 = $i;
   $47 = (($45) + ($46))|0;
   $48 = $xbuf0;
   $49 = (($48) + ($47<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $i;
   $52 = $rgroup;
   $53 = (($51) - ($52))|0;
   $54 = $xbuf0;
   $55 = (($54) + ($53<<2)|0);
   HEAP32[$55>>2] = $50;
   $56 = $rgroup;
   $57 = $M;
   $58 = (($57) + 1)|0;
   $59 = Math_imul($56, $58)|0;
   $60 = $i;
   $61 = (($59) + ($60))|0;
   $62 = $xbuf1;
   $63 = (($62) + ($61<<2)|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $i;
   $66 = $rgroup;
   $67 = (($65) - ($66))|0;
   $68 = $xbuf1;
   $69 = (($68) + ($67<<2)|0);
   HEAP32[$69>>2] = $64;
   $70 = $i;
   $71 = $xbuf0;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = $rgroup;
   $75 = $M;
   $76 = (($75) + 2)|0;
   $77 = Math_imul($74, $76)|0;
   $78 = $i;
   $79 = (($77) + ($78))|0;
   $80 = $xbuf0;
   $81 = (($80) + ($79<<2)|0);
   HEAP32[$81>>2] = $73;
   $82 = $i;
   $83 = $xbuf1;
   $84 = (($83) + ($82<<2)|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $rgroup;
   $87 = $M;
   $88 = (($87) + 2)|0;
   $89 = Math_imul($86, $88)|0;
   $90 = $i;
   $91 = (($89) + ($90))|0;
   $92 = $xbuf1;
   $93 = (($92) + ($91<<2)|0);
   HEAP32[$93>>2] = $85;
   $94 = $i;
   $95 = (($94) + 1)|0;
   $i = $95;
  }
  $96 = $ci;
  $97 = (($96) + 1)|0;
  $ci = $97;
  $98 = $compptr;
  $99 = (($98) + 84|0);
  $compptr = $99;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_float($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0.0, $106 = 0.0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0.0, $124 = 0.0, $125 = 0.0, $126 = 0.0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0.0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0.0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0, $145 = 0.0, $146 = 0, $147 = 0, $148 = 0.0, $149 = 0.0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0.0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0.0, $162 = 0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0.0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0.0, $196 = 0.0, $197 = 0.0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0.0, $203 = 0.0, $204 = 0.0, $205 = 0.0;
 var $206 = 0.0, $207 = 0.0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0.0, $212 = 0.0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0.0, $225 = 0.0, $226 = 0, $227 = 0, $228 = 0.0, $229 = 0.0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0, $233 = 0.0, $234 = 0.0, $235 = 0.0, $236 = 0, $237 = 0, $238 = 0.0, $239 = 0.0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0.0, $245 = 0.0, $246 = 0, $247 = 0, $248 = 0.0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0.0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0.0, $275 = 0, $276 = 0.0, $277 = 0, $278 = 0;
 var $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0, $282 = 0, $283 = 0.0, $284 = 0, $285 = 0, $286 = 0.0, $287 = 0.0, $288 = 0, $289 = 0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0, $293 = 0.0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0.0, $302 = 0.0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0.0, $307 = 0.0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0;
 var $314 = 0, $315 = 0.0, $316 = 0.0, $317 = 0, $318 = 0, $319 = 0.0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0.0, $324 = 0, $325 = 0, $326 = 0.0, $327 = 0, $328 = 0, $329 = 0.0, $33 = 0, $330 = 0.0, $331 = 0;
 var $332 = 0, $333 = 0.0, $334 = 0, $335 = 0, $336 = 0.0, $337 = 0.0, $338 = 0.0, $339 = 0.0, $34 = 0, $340 = 0.0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0.0, $345 = 0.0, $346 = 0.0, $347 = 0.0, $348 = 0.0, $349 = 0.0, $35 = 0;
 var $350 = 0.0, $351 = 0.0, $352 = 0.0, $353 = 0.0, $354 = 0.0, $355 = 0.0, $356 = 0.0, $357 = 0.0, $358 = 0.0, $359 = 0.0, $36 = 0, $360 = 0.0, $361 = 0.0, $362 = 0.0, $363 = 0.0, $364 = 0.0, $365 = 0.0, $366 = 0.0, $367 = 0.0, $368 = 0.0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0.0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0.0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0.0, $402 = 0.0, $403 = 0.0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0.0, $414 = 0.0, $415 = 0.0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0.0, $426 = 0.0, $427 = 0.0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0.0, $438 = 0.0, $439 = 0.0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0.0, $45 = 0, $450 = 0.0, $451 = 0.0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0.0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0, $75 = 0, $76 = 0.0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0.0;
 var $95 = 0, $96 = 0, $97 = 0.0, $98 = 0.0, $99 = 0, $ctr = 0, $dcval = 0.0, $inptr = 0, $outptr = 0, $quantptr = 0, $range_limit = 0, $tmp0 = 0.0, $tmp1 = 0.0, $tmp10 = 0.0, $tmp11 = 0.0, $tmp12 = 0.0, $tmp13 = 0.0, $tmp2 = 0.0, $tmp3 = 0.0, $tmp4 = 0.0;
 var $tmp5 = 0.0, $tmp6 = 0.0, $tmp7 = 0.0, $workspace = 0, $wsptr = 0, $z10 = 0.0, $z11 = 0.0, $z12 = 0.0, $z13 = 0.0, $z5 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 384|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $inptr;
  $16 = (($15) + 16|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = $inptr;
   $21 = (($20) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $inptr;
    $26 = (($25) + 48|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27 << 16 >> 16;
    $29 = ($28|0)==(0);
    if ($29) {
     $30 = $inptr;
     $31 = (($30) + 64|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32 << 16 >> 16;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = $inptr;
      $36 = (($35) + 80|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37 << 16 >> 16;
      $39 = ($38|0)==(0);
      if ($39) {
       $40 = $inptr;
       $41 = (($40) + 96|0);
       $42 = HEAP16[$41>>1]|0;
       $43 = $42 << 16 >> 16;
       $44 = ($43|0)==(0);
       if ($44) {
        $45 = $inptr;
        $46 = (($45) + 112|0);
        $47 = HEAP16[$46>>1]|0;
        $48 = $47 << 16 >> 16;
        $49 = ($48|0)==(0);
        if ($49) {
         $50 = $inptr;
         $51 = HEAP16[$50>>1]|0;
         $52 = (+($51<<16>>16));
         $53 = $quantptr;
         $54 = +HEAPF32[$53>>2];
         $55 = $52 * $54;
         $dcval = $55;
         $56 = $dcval;
         $57 = $wsptr;
         HEAPF32[$57>>2] = $56;
         $58 = $dcval;
         $59 = $wsptr;
         $60 = (($59) + 32|0);
         HEAPF32[$60>>2] = $58;
         $61 = $dcval;
         $62 = $wsptr;
         $63 = (($62) + 64|0);
         HEAPF32[$63>>2] = $61;
         $64 = $dcval;
         $65 = $wsptr;
         $66 = (($65) + 96|0);
         HEAPF32[$66>>2] = $64;
         $67 = $dcval;
         $68 = $wsptr;
         $69 = (($68) + 128|0);
         HEAPF32[$69>>2] = $67;
         $70 = $dcval;
         $71 = $wsptr;
         $72 = (($71) + 160|0);
         HEAPF32[$72>>2] = $70;
         $73 = $dcval;
         $74 = $wsptr;
         $75 = (($74) + 192|0);
         HEAPF32[$75>>2] = $73;
         $76 = $dcval;
         $77 = $wsptr;
         $78 = (($77) + 224|0);
         HEAPF32[$78>>2] = $76;
         $79 = $inptr;
         $80 = (($79) + 2|0);
         $inptr = $80;
         $81 = $quantptr;
         $82 = (($81) + 4|0);
         $quantptr = $82;
         $83 = $wsptr;
         $84 = (($83) + 4|0);
         $wsptr = $84;
        } else {
         label = 11;
        }
       } else {
        label = 11;
       }
      } else {
       label = 11;
      }
     } else {
      label = 11;
     }
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   label = 0;
   $85 = $inptr;
   $86 = HEAP16[$85>>1]|0;
   $87 = (+($86<<16>>16));
   $88 = $quantptr;
   $89 = +HEAPF32[$88>>2];
   $90 = $87 * $89;
   $tmp0 = $90;
   $91 = $inptr;
   $92 = (($91) + 32|0);
   $93 = HEAP16[$92>>1]|0;
   $94 = (+($93<<16>>16));
   $95 = $quantptr;
   $96 = (($95) + 64|0);
   $97 = +HEAPF32[$96>>2];
   $98 = $94 * $97;
   $tmp1 = $98;
   $99 = $inptr;
   $100 = (($99) + 64|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = (+($101<<16>>16));
   $103 = $quantptr;
   $104 = (($103) + 128|0);
   $105 = +HEAPF32[$104>>2];
   $106 = $102 * $105;
   $tmp2 = $106;
   $107 = $inptr;
   $108 = (($107) + 96|0);
   $109 = HEAP16[$108>>1]|0;
   $110 = (+($109<<16>>16));
   $111 = $quantptr;
   $112 = (($111) + 192|0);
   $113 = +HEAPF32[$112>>2];
   $114 = $110 * $113;
   $tmp3 = $114;
   $115 = $tmp0;
   $116 = $tmp2;
   $117 = $115 + $116;
   $tmp10 = $117;
   $118 = $tmp0;
   $119 = $tmp2;
   $120 = $118 - $119;
   $tmp11 = $120;
   $121 = $tmp1;
   $122 = $tmp3;
   $123 = $121 + $122;
   $tmp13 = $123;
   $124 = $tmp1;
   $125 = $tmp3;
   $126 = $124 - $125;
   $127 = $126 * 1.41421353816986083984;
   $128 = $tmp13;
   $129 = $127 - $128;
   $tmp12 = $129;
   $130 = $tmp10;
   $131 = $tmp13;
   $132 = $130 + $131;
   $tmp0 = $132;
   $133 = $tmp10;
   $134 = $tmp13;
   $135 = $133 - $134;
   $tmp3 = $135;
   $136 = $tmp11;
   $137 = $tmp12;
   $138 = $136 + $137;
   $tmp1 = $138;
   $139 = $tmp11;
   $140 = $tmp12;
   $141 = $139 - $140;
   $tmp2 = $141;
   $142 = $inptr;
   $143 = (($142) + 16|0);
   $144 = HEAP16[$143>>1]|0;
   $145 = (+($144<<16>>16));
   $146 = $quantptr;
   $147 = (($146) + 32|0);
   $148 = +HEAPF32[$147>>2];
   $149 = $145 * $148;
   $tmp4 = $149;
   $150 = $inptr;
   $151 = (($150) + 48|0);
   $152 = HEAP16[$151>>1]|0;
   $153 = (+($152<<16>>16));
   $154 = $quantptr;
   $155 = (($154) + 96|0);
   $156 = +HEAPF32[$155>>2];
   $157 = $153 * $156;
   $tmp5 = $157;
   $158 = $inptr;
   $159 = (($158) + 80|0);
   $160 = HEAP16[$159>>1]|0;
   $161 = (+($160<<16>>16));
   $162 = $quantptr;
   $163 = (($162) + 160|0);
   $164 = +HEAPF32[$163>>2];
   $165 = $161 * $164;
   $tmp6 = $165;
   $166 = $inptr;
   $167 = (($166) + 112|0);
   $168 = HEAP16[$167>>1]|0;
   $169 = (+($168<<16>>16));
   $170 = $quantptr;
   $171 = (($170) + 224|0);
   $172 = +HEAPF32[$171>>2];
   $173 = $169 * $172;
   $tmp7 = $173;
   $174 = $tmp6;
   $175 = $tmp5;
   $176 = $174 + $175;
   $z13 = $176;
   $177 = $tmp6;
   $178 = $tmp5;
   $179 = $177 - $178;
   $z10 = $179;
   $180 = $tmp4;
   $181 = $tmp7;
   $182 = $180 + $181;
   $z11 = $182;
   $183 = $tmp4;
   $184 = $tmp7;
   $185 = $183 - $184;
   $z12 = $185;
   $186 = $z11;
   $187 = $z13;
   $188 = $186 + $187;
   $tmp7 = $188;
   $189 = $z11;
   $190 = $z13;
   $191 = $189 - $190;
   $192 = $191 * 1.41421353816986083984;
   $tmp11 = $192;
   $193 = $z10;
   $194 = $z12;
   $195 = $193 + $194;
   $196 = $195 * 1.84775900840759277344;
   $z5 = $196;
   $197 = $z12;
   $198 = 1.08239221572875976563 * $197;
   $199 = $z5;
   $200 = $198 - $199;
   $tmp10 = $200;
   $201 = $z10;
   $202 = -2.61312603950500488281 * $201;
   $203 = $z5;
   $204 = $202 + $203;
   $tmp12 = $204;
   $205 = $tmp12;
   $206 = $tmp7;
   $207 = $205 - $206;
   $tmp6 = $207;
   $208 = $tmp11;
   $209 = $tmp6;
   $210 = $208 - $209;
   $tmp5 = $210;
   $211 = $tmp10;
   $212 = $tmp5;
   $213 = $211 + $212;
   $tmp4 = $213;
   $214 = $tmp0;
   $215 = $tmp7;
   $216 = $214 + $215;
   $217 = $wsptr;
   HEAPF32[$217>>2] = $216;
   $218 = $tmp0;
   $219 = $tmp7;
   $220 = $218 - $219;
   $221 = $wsptr;
   $222 = (($221) + 224|0);
   HEAPF32[$222>>2] = $220;
   $223 = $tmp1;
   $224 = $tmp6;
   $225 = $223 + $224;
   $226 = $wsptr;
   $227 = (($226) + 32|0);
   HEAPF32[$227>>2] = $225;
   $228 = $tmp1;
   $229 = $tmp6;
   $230 = $228 - $229;
   $231 = $wsptr;
   $232 = (($231) + 192|0);
   HEAPF32[$232>>2] = $230;
   $233 = $tmp2;
   $234 = $tmp5;
   $235 = $233 + $234;
   $236 = $wsptr;
   $237 = (($236) + 64|0);
   HEAPF32[$237>>2] = $235;
   $238 = $tmp2;
   $239 = $tmp5;
   $240 = $238 - $239;
   $241 = $wsptr;
   $242 = (($241) + 160|0);
   HEAPF32[$242>>2] = $240;
   $243 = $tmp3;
   $244 = $tmp4;
   $245 = $243 + $244;
   $246 = $wsptr;
   $247 = (($246) + 128|0);
   HEAPF32[$247>>2] = $245;
   $248 = $tmp3;
   $249 = $tmp4;
   $250 = $248 - $249;
   $251 = $wsptr;
   $252 = (($251) + 96|0);
   HEAPF32[$252>>2] = $250;
   $253 = $inptr;
   $254 = (($253) + 2|0);
   $inptr = $254;
   $255 = $quantptr;
   $256 = (($255) + 4|0);
   $quantptr = $256;
   $257 = $wsptr;
   $258 = (($257) + 4|0);
   $wsptr = $258;
  }
  $259 = $ctr;
  $260 = (($259) + -1)|0;
  $ctr = $260;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $261 = $ctr;
  $262 = ($261|0)<(8);
  if (!($262)) {
   break;
  }
  $263 = $ctr;
  $264 = $3;
  $265 = (($264) + ($263<<2)|0);
  $266 = HEAP32[$265>>2]|0;
  $267 = $4;
  $268 = (($266) + ($267)|0);
  $outptr = $268;
  $269 = $wsptr;
  $270 = +HEAPF32[$269>>2];
  $271 = $wsptr;
  $272 = (($271) + 16|0);
  $273 = +HEAPF32[$272>>2];
  $274 = $270 + $273;
  $tmp10 = $274;
  $275 = $wsptr;
  $276 = +HEAPF32[$275>>2];
  $277 = $wsptr;
  $278 = (($277) + 16|0);
  $279 = +HEAPF32[$278>>2];
  $280 = $276 - $279;
  $tmp11 = $280;
  $281 = $wsptr;
  $282 = (($281) + 8|0);
  $283 = +HEAPF32[$282>>2];
  $284 = $wsptr;
  $285 = (($284) + 24|0);
  $286 = +HEAPF32[$285>>2];
  $287 = $283 + $286;
  $tmp13 = $287;
  $288 = $wsptr;
  $289 = (($288) + 8|0);
  $290 = +HEAPF32[$289>>2];
  $291 = $wsptr;
  $292 = (($291) + 24|0);
  $293 = +HEAPF32[$292>>2];
  $294 = $290 - $293;
  $295 = $294 * 1.41421353816986083984;
  $296 = $tmp13;
  $297 = $295 - $296;
  $tmp12 = $297;
  $298 = $tmp10;
  $299 = $tmp13;
  $300 = $298 + $299;
  $tmp0 = $300;
  $301 = $tmp10;
  $302 = $tmp13;
  $303 = $301 - $302;
  $tmp3 = $303;
  $304 = $tmp11;
  $305 = $tmp12;
  $306 = $304 + $305;
  $tmp1 = $306;
  $307 = $tmp11;
  $308 = $tmp12;
  $309 = $307 - $308;
  $tmp2 = $309;
  $310 = $wsptr;
  $311 = (($310) + 20|0);
  $312 = +HEAPF32[$311>>2];
  $313 = $wsptr;
  $314 = (($313) + 12|0);
  $315 = +HEAPF32[$314>>2];
  $316 = $312 + $315;
  $z13 = $316;
  $317 = $wsptr;
  $318 = (($317) + 20|0);
  $319 = +HEAPF32[$318>>2];
  $320 = $wsptr;
  $321 = (($320) + 12|0);
  $322 = +HEAPF32[$321>>2];
  $323 = $319 - $322;
  $z10 = $323;
  $324 = $wsptr;
  $325 = (($324) + 4|0);
  $326 = +HEAPF32[$325>>2];
  $327 = $wsptr;
  $328 = (($327) + 28|0);
  $329 = +HEAPF32[$328>>2];
  $330 = $326 + $329;
  $z11 = $330;
  $331 = $wsptr;
  $332 = (($331) + 4|0);
  $333 = +HEAPF32[$332>>2];
  $334 = $wsptr;
  $335 = (($334) + 28|0);
  $336 = +HEAPF32[$335>>2];
  $337 = $333 - $336;
  $z12 = $337;
  $338 = $z11;
  $339 = $z13;
  $340 = $338 + $339;
  $tmp7 = $340;
  $341 = $z11;
  $342 = $z13;
  $343 = $341 - $342;
  $344 = $343 * 1.41421353816986083984;
  $tmp11 = $344;
  $345 = $z10;
  $346 = $z12;
  $347 = $345 + $346;
  $348 = $347 * 1.84775900840759277344;
  $z5 = $348;
  $349 = $z12;
  $350 = 1.08239221572875976563 * $349;
  $351 = $z5;
  $352 = $350 - $351;
  $tmp10 = $352;
  $353 = $z10;
  $354 = -2.61312603950500488281 * $353;
  $355 = $z5;
  $356 = $354 + $355;
  $tmp12 = $356;
  $357 = $tmp12;
  $358 = $tmp7;
  $359 = $357 - $358;
  $tmp6 = $359;
  $360 = $tmp11;
  $361 = $tmp6;
  $362 = $360 - $361;
  $tmp5 = $362;
  $363 = $tmp10;
  $364 = $tmp5;
  $365 = $363 + $364;
  $tmp4 = $365;
  $366 = $tmp0;
  $367 = $tmp7;
  $368 = $366 + $367;
  $369 = (~~(($368)));
  $370 = (($369) + 4)|0;
  $371 = $370 >> 3;
  $372 = $371 & 1023;
  $373 = $range_limit;
  $374 = (($373) + ($372)|0);
  $375 = HEAP8[$374>>0]|0;
  $376 = $outptr;
  HEAP8[$376>>0] = $375;
  $377 = $tmp0;
  $378 = $tmp7;
  $379 = $377 - $378;
  $380 = (~~(($379)));
  $381 = (($380) + 4)|0;
  $382 = $381 >> 3;
  $383 = $382 & 1023;
  $384 = $range_limit;
  $385 = (($384) + ($383)|0);
  $386 = HEAP8[$385>>0]|0;
  $387 = $outptr;
  $388 = (($387) + 7|0);
  HEAP8[$388>>0] = $386;
  $389 = $tmp1;
  $390 = $tmp6;
  $391 = $389 + $390;
  $392 = (~~(($391)));
  $393 = (($392) + 4)|0;
  $394 = $393 >> 3;
  $395 = $394 & 1023;
  $396 = $range_limit;
  $397 = (($396) + ($395)|0);
  $398 = HEAP8[$397>>0]|0;
  $399 = $outptr;
  $400 = (($399) + 1|0);
  HEAP8[$400>>0] = $398;
  $401 = $tmp1;
  $402 = $tmp6;
  $403 = $401 - $402;
  $404 = (~~(($403)));
  $405 = (($404) + 4)|0;
  $406 = $405 >> 3;
  $407 = $406 & 1023;
  $408 = $range_limit;
  $409 = (($408) + ($407)|0);
  $410 = HEAP8[$409>>0]|0;
  $411 = $outptr;
  $412 = (($411) + 6|0);
  HEAP8[$412>>0] = $410;
  $413 = $tmp2;
  $414 = $tmp5;
  $415 = $413 + $414;
  $416 = (~~(($415)));
  $417 = (($416) + 4)|0;
  $418 = $417 >> 3;
  $419 = $418 & 1023;
  $420 = $range_limit;
  $421 = (($420) + ($419)|0);
  $422 = HEAP8[$421>>0]|0;
  $423 = $outptr;
  $424 = (($423) + 2|0);
  HEAP8[$424>>0] = $422;
  $425 = $tmp2;
  $426 = $tmp5;
  $427 = $425 - $426;
  $428 = (~~(($427)));
  $429 = (($428) + 4)|0;
  $430 = $429 >> 3;
  $431 = $430 & 1023;
  $432 = $range_limit;
  $433 = (($432) + ($431)|0);
  $434 = HEAP8[$433>>0]|0;
  $435 = $outptr;
  $436 = (($435) + 5|0);
  HEAP8[$436>>0] = $434;
  $437 = $tmp3;
  $438 = $tmp4;
  $439 = $437 + $438;
  $440 = (~~(($439)));
  $441 = (($440) + 4)|0;
  $442 = $441 >> 3;
  $443 = $442 & 1023;
  $444 = $range_limit;
  $445 = (($444) + ($443)|0);
  $446 = HEAP8[$445>>0]|0;
  $447 = $outptr;
  $448 = (($447) + 4|0);
  HEAP8[$448>>0] = $446;
  $449 = $tmp3;
  $450 = $tmp4;
  $451 = $449 - $450;
  $452 = (~~(($451)));
  $453 = (($452) + 4)|0;
  $454 = $453 >> 3;
  $455 = $454 & 1023;
  $456 = $range_limit;
  $457 = (($456) + ($455)|0);
  $458 = HEAP8[$457>>0]|0;
  $459 = $outptr;
  $460 = (($459) + 3|0);
  HEAP8[$460>>0] = $458;
  $461 = $wsptr;
  $462 = (($461) + 32|0);
  $wsptr = $462;
  $463 = $ctr;
  $464 = (($463) + 1)|0;
  $ctr = $464;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_ifast($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctr = 0;
 var $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0, $quantptr = 0, $range_limit = 0, $tmp0 = 0, $tmp1 = 0, $tmp10 = 0, $tmp11 = 0, $tmp12 = 0, $tmp13 = 0, $tmp2 = 0, $tmp3 = 0, $tmp4 = 0, $tmp5 = 0, $tmp6 = 0, $tmp7 = 0, $workspace = 0, $wsptr = 0;
 var $z10 = 0, $z11 = 0, $z12 = 0, $z13 = 0, $z5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 384|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $inptr;
  $16 = (($15) + 16|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = $inptr;
   $21 = (($20) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $inptr;
    $26 = (($25) + 48|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27 << 16 >> 16;
    $29 = ($28|0)==(0);
    if ($29) {
     $30 = $inptr;
     $31 = (($30) + 64|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32 << 16 >> 16;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = $inptr;
      $36 = (($35) + 80|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37 << 16 >> 16;
      $39 = ($38|0)==(0);
      if ($39) {
       $40 = $inptr;
       $41 = (($40) + 96|0);
       $42 = HEAP16[$41>>1]|0;
       $43 = $42 << 16 >> 16;
       $44 = ($43|0)==(0);
       if ($44) {
        $45 = $inptr;
        $46 = (($45) + 112|0);
        $47 = HEAP16[$46>>1]|0;
        $48 = $47 << 16 >> 16;
        $49 = ($48|0)==(0);
        if ($49) {
         $50 = $inptr;
         $51 = HEAP16[$50>>1]|0;
         $52 = $51 << 16 >> 16;
         $53 = $quantptr;
         $54 = HEAP32[$53>>2]|0;
         $55 = Math_imul($52, $54)|0;
         $dcval = $55;
         $56 = $dcval;
         $57 = $wsptr;
         HEAP32[$57>>2] = $56;
         $58 = $dcval;
         $59 = $wsptr;
         $60 = (($59) + 32|0);
         HEAP32[$60>>2] = $58;
         $61 = $dcval;
         $62 = $wsptr;
         $63 = (($62) + 64|0);
         HEAP32[$63>>2] = $61;
         $64 = $dcval;
         $65 = $wsptr;
         $66 = (($65) + 96|0);
         HEAP32[$66>>2] = $64;
         $67 = $dcval;
         $68 = $wsptr;
         $69 = (($68) + 128|0);
         HEAP32[$69>>2] = $67;
         $70 = $dcval;
         $71 = $wsptr;
         $72 = (($71) + 160|0);
         HEAP32[$72>>2] = $70;
         $73 = $dcval;
         $74 = $wsptr;
         $75 = (($74) + 192|0);
         HEAP32[$75>>2] = $73;
         $76 = $dcval;
         $77 = $wsptr;
         $78 = (($77) + 224|0);
         HEAP32[$78>>2] = $76;
         $79 = $inptr;
         $80 = (($79) + 2|0);
         $inptr = $80;
         $81 = $quantptr;
         $82 = (($81) + 4|0);
         $quantptr = $82;
         $83 = $wsptr;
         $84 = (($83) + 4|0);
         $wsptr = $84;
        } else {
         label = 11;
        }
       } else {
        label = 11;
       }
      } else {
       label = 11;
      }
     } else {
      label = 11;
     }
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   label = 0;
   $85 = $inptr;
   $86 = HEAP16[$85>>1]|0;
   $87 = $86 << 16 >> 16;
   $88 = $quantptr;
   $89 = HEAP32[$88>>2]|0;
   $90 = Math_imul($87, $89)|0;
   $tmp0 = $90;
   $91 = $inptr;
   $92 = (($91) + 32|0);
   $93 = HEAP16[$92>>1]|0;
   $94 = $93 << 16 >> 16;
   $95 = $quantptr;
   $96 = (($95) + 64|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = Math_imul($94, $97)|0;
   $tmp1 = $98;
   $99 = $inptr;
   $100 = (($99) + 64|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = $101 << 16 >> 16;
   $103 = $quantptr;
   $104 = (($103) + 128|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = Math_imul($102, $105)|0;
   $tmp2 = $106;
   $107 = $inptr;
   $108 = (($107) + 96|0);
   $109 = HEAP16[$108>>1]|0;
   $110 = $109 << 16 >> 16;
   $111 = $quantptr;
   $112 = (($111) + 192|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = Math_imul($110, $113)|0;
   $tmp3 = $114;
   $115 = $tmp0;
   $116 = $tmp2;
   $117 = (($115) + ($116))|0;
   $tmp10 = $117;
   $118 = $tmp0;
   $119 = $tmp2;
   $120 = (($118) - ($119))|0;
   $tmp11 = $120;
   $121 = $tmp1;
   $122 = $tmp3;
   $123 = (($121) + ($122))|0;
   $tmp13 = $123;
   $124 = $tmp1;
   $125 = $tmp3;
   $126 = (($124) - ($125))|0;
   $127 = ($126*362)|0;
   $128 = $127 >> 8;
   $129 = $tmp13;
   $130 = (($128) - ($129))|0;
   $tmp12 = $130;
   $131 = $tmp10;
   $132 = $tmp13;
   $133 = (($131) + ($132))|0;
   $tmp0 = $133;
   $134 = $tmp10;
   $135 = $tmp13;
   $136 = (($134) - ($135))|0;
   $tmp3 = $136;
   $137 = $tmp11;
   $138 = $tmp12;
   $139 = (($137) + ($138))|0;
   $tmp1 = $139;
   $140 = $tmp11;
   $141 = $tmp12;
   $142 = (($140) - ($141))|0;
   $tmp2 = $142;
   $143 = $inptr;
   $144 = (($143) + 16|0);
   $145 = HEAP16[$144>>1]|0;
   $146 = $145 << 16 >> 16;
   $147 = $quantptr;
   $148 = (($147) + 32|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = Math_imul($146, $149)|0;
   $tmp4 = $150;
   $151 = $inptr;
   $152 = (($151) + 48|0);
   $153 = HEAP16[$152>>1]|0;
   $154 = $153 << 16 >> 16;
   $155 = $quantptr;
   $156 = (($155) + 96|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = Math_imul($154, $157)|0;
   $tmp5 = $158;
   $159 = $inptr;
   $160 = (($159) + 80|0);
   $161 = HEAP16[$160>>1]|0;
   $162 = $161 << 16 >> 16;
   $163 = $quantptr;
   $164 = (($163) + 160|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = Math_imul($162, $165)|0;
   $tmp6 = $166;
   $167 = $inptr;
   $168 = (($167) + 112|0);
   $169 = HEAP16[$168>>1]|0;
   $170 = $169 << 16 >> 16;
   $171 = $quantptr;
   $172 = (($171) + 224|0);
   $173 = HEAP32[$172>>2]|0;
   $174 = Math_imul($170, $173)|0;
   $tmp7 = $174;
   $175 = $tmp6;
   $176 = $tmp5;
   $177 = (($175) + ($176))|0;
   $z13 = $177;
   $178 = $tmp6;
   $179 = $tmp5;
   $180 = (($178) - ($179))|0;
   $z10 = $180;
   $181 = $tmp4;
   $182 = $tmp7;
   $183 = (($181) + ($182))|0;
   $z11 = $183;
   $184 = $tmp4;
   $185 = $tmp7;
   $186 = (($184) - ($185))|0;
   $z12 = $186;
   $187 = $z11;
   $188 = $z13;
   $189 = (($187) + ($188))|0;
   $tmp7 = $189;
   $190 = $z11;
   $191 = $z13;
   $192 = (($190) - ($191))|0;
   $193 = ($192*362)|0;
   $194 = $193 >> 8;
   $tmp11 = $194;
   $195 = $z10;
   $196 = $z12;
   $197 = (($195) + ($196))|0;
   $198 = ($197*473)|0;
   $199 = $198 >> 8;
   $z5 = $199;
   $200 = $z12;
   $201 = ($200*277)|0;
   $202 = $201 >> 8;
   $203 = $z5;
   $204 = (($202) - ($203))|0;
   $tmp10 = $204;
   $205 = $z10;
   $206 = Math_imul($205, -669)|0;
   $207 = $206 >> 8;
   $208 = $z5;
   $209 = (($207) + ($208))|0;
   $tmp12 = $209;
   $210 = $tmp12;
   $211 = $tmp7;
   $212 = (($210) - ($211))|0;
   $tmp6 = $212;
   $213 = $tmp11;
   $214 = $tmp6;
   $215 = (($213) - ($214))|0;
   $tmp5 = $215;
   $216 = $tmp10;
   $217 = $tmp5;
   $218 = (($216) + ($217))|0;
   $tmp4 = $218;
   $219 = $tmp0;
   $220 = $tmp7;
   $221 = (($219) + ($220))|0;
   $222 = $wsptr;
   HEAP32[$222>>2] = $221;
   $223 = $tmp0;
   $224 = $tmp7;
   $225 = (($223) - ($224))|0;
   $226 = $wsptr;
   $227 = (($226) + 224|0);
   HEAP32[$227>>2] = $225;
   $228 = $tmp1;
   $229 = $tmp6;
   $230 = (($228) + ($229))|0;
   $231 = $wsptr;
   $232 = (($231) + 32|0);
   HEAP32[$232>>2] = $230;
   $233 = $tmp1;
   $234 = $tmp6;
   $235 = (($233) - ($234))|0;
   $236 = $wsptr;
   $237 = (($236) + 192|0);
   HEAP32[$237>>2] = $235;
   $238 = $tmp2;
   $239 = $tmp5;
   $240 = (($238) + ($239))|0;
   $241 = $wsptr;
   $242 = (($241) + 64|0);
   HEAP32[$242>>2] = $240;
   $243 = $tmp2;
   $244 = $tmp5;
   $245 = (($243) - ($244))|0;
   $246 = $wsptr;
   $247 = (($246) + 160|0);
   HEAP32[$247>>2] = $245;
   $248 = $tmp3;
   $249 = $tmp4;
   $250 = (($248) + ($249))|0;
   $251 = $wsptr;
   $252 = (($251) + 128|0);
   HEAP32[$252>>2] = $250;
   $253 = $tmp3;
   $254 = $tmp4;
   $255 = (($253) - ($254))|0;
   $256 = $wsptr;
   $257 = (($256) + 96|0);
   HEAP32[$257>>2] = $255;
   $258 = $inptr;
   $259 = (($258) + 2|0);
   $inptr = $259;
   $260 = $quantptr;
   $261 = (($260) + 4|0);
   $quantptr = $261;
   $262 = $wsptr;
   $263 = (($262) + 4|0);
   $wsptr = $263;
  }
  $264 = $ctr;
  $265 = (($264) + -1)|0;
  $ctr = $265;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $266 = $ctr;
  $267 = ($266|0)<(8);
  if (!($267)) {
   break;
  }
  $268 = $ctr;
  $269 = $3;
  $270 = (($269) + ($268<<2)|0);
  $271 = HEAP32[$270>>2]|0;
  $272 = $4;
  $273 = (($271) + ($272)|0);
  $outptr = $273;
  $274 = $wsptr;
  $275 = (($274) + 4|0);
  $276 = HEAP32[$275>>2]|0;
  $277 = ($276|0)==(0);
  if ($277) {
   $278 = $wsptr;
   $279 = (($278) + 8|0);
   $280 = HEAP32[$279>>2]|0;
   $281 = ($280|0)==(0);
   if ($281) {
    $282 = $wsptr;
    $283 = (($282) + 12|0);
    $284 = HEAP32[$283>>2]|0;
    $285 = ($284|0)==(0);
    if ($285) {
     $286 = $wsptr;
     $287 = (($286) + 16|0);
     $288 = HEAP32[$287>>2]|0;
     $289 = ($288|0)==(0);
     if ($289) {
      $290 = $wsptr;
      $291 = (($290) + 20|0);
      $292 = HEAP32[$291>>2]|0;
      $293 = ($292|0)==(0);
      if ($293) {
       $294 = $wsptr;
       $295 = (($294) + 24|0);
       $296 = HEAP32[$295>>2]|0;
       $297 = ($296|0)==(0);
       if ($297) {
        $298 = $wsptr;
        $299 = (($298) + 28|0);
        $300 = HEAP32[$299>>2]|0;
        $301 = ($300|0)==(0);
        if ($301) {
         $302 = $wsptr;
         $303 = HEAP32[$302>>2]|0;
         $304 = $303 >> 5;
         $305 = $304 & 1023;
         $306 = $range_limit;
         $307 = (($306) + ($305)|0);
         $308 = HEAP8[$307>>0]|0;
         $dcval1 = $308;
         $309 = $dcval1;
         $310 = $outptr;
         HEAP8[$310>>0] = $309;
         $311 = $dcval1;
         $312 = $outptr;
         $313 = (($312) + 1|0);
         HEAP8[$313>>0] = $311;
         $314 = $dcval1;
         $315 = $outptr;
         $316 = (($315) + 2|0);
         HEAP8[$316>>0] = $314;
         $317 = $dcval1;
         $318 = $outptr;
         $319 = (($318) + 3|0);
         HEAP8[$319>>0] = $317;
         $320 = $dcval1;
         $321 = $outptr;
         $322 = (($321) + 4|0);
         HEAP8[$322>>0] = $320;
         $323 = $dcval1;
         $324 = $outptr;
         $325 = (($324) + 5|0);
         HEAP8[$325>>0] = $323;
         $326 = $dcval1;
         $327 = $outptr;
         $328 = (($327) + 6|0);
         HEAP8[$328>>0] = $326;
         $329 = $dcval1;
         $330 = $outptr;
         $331 = (($330) + 7|0);
         HEAP8[$331>>0] = $329;
         $332 = $wsptr;
         $333 = (($332) + 32|0);
         $wsptr = $333;
        } else {
         label = 23;
        }
       } else {
        label = 23;
       }
      } else {
       label = 23;
      }
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $334 = $wsptr;
   $335 = HEAP32[$334>>2]|0;
   $336 = $wsptr;
   $337 = (($336) + 16|0);
   $338 = HEAP32[$337>>2]|0;
   $339 = (($335) + ($338))|0;
   $tmp10 = $339;
   $340 = $wsptr;
   $341 = HEAP32[$340>>2]|0;
   $342 = $wsptr;
   $343 = (($342) + 16|0);
   $344 = HEAP32[$343>>2]|0;
   $345 = (($341) - ($344))|0;
   $tmp11 = $345;
   $346 = $wsptr;
   $347 = (($346) + 8|0);
   $348 = HEAP32[$347>>2]|0;
   $349 = $wsptr;
   $350 = (($349) + 24|0);
   $351 = HEAP32[$350>>2]|0;
   $352 = (($348) + ($351))|0;
   $tmp13 = $352;
   $353 = $wsptr;
   $354 = (($353) + 8|0);
   $355 = HEAP32[$354>>2]|0;
   $356 = $wsptr;
   $357 = (($356) + 24|0);
   $358 = HEAP32[$357>>2]|0;
   $359 = (($355) - ($358))|0;
   $360 = ($359*362)|0;
   $361 = $360 >> 8;
   $362 = $tmp13;
   $363 = (($361) - ($362))|0;
   $tmp12 = $363;
   $364 = $tmp10;
   $365 = $tmp13;
   $366 = (($364) + ($365))|0;
   $tmp0 = $366;
   $367 = $tmp10;
   $368 = $tmp13;
   $369 = (($367) - ($368))|0;
   $tmp3 = $369;
   $370 = $tmp11;
   $371 = $tmp12;
   $372 = (($370) + ($371))|0;
   $tmp1 = $372;
   $373 = $tmp11;
   $374 = $tmp12;
   $375 = (($373) - ($374))|0;
   $tmp2 = $375;
   $376 = $wsptr;
   $377 = (($376) + 20|0);
   $378 = HEAP32[$377>>2]|0;
   $379 = $wsptr;
   $380 = (($379) + 12|0);
   $381 = HEAP32[$380>>2]|0;
   $382 = (($378) + ($381))|0;
   $z13 = $382;
   $383 = $wsptr;
   $384 = (($383) + 20|0);
   $385 = HEAP32[$384>>2]|0;
   $386 = $wsptr;
   $387 = (($386) + 12|0);
   $388 = HEAP32[$387>>2]|0;
   $389 = (($385) - ($388))|0;
   $z10 = $389;
   $390 = $wsptr;
   $391 = (($390) + 4|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = $wsptr;
   $394 = (($393) + 28|0);
   $395 = HEAP32[$394>>2]|0;
   $396 = (($392) + ($395))|0;
   $z11 = $396;
   $397 = $wsptr;
   $398 = (($397) + 4|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = $wsptr;
   $401 = (($400) + 28|0);
   $402 = HEAP32[$401>>2]|0;
   $403 = (($399) - ($402))|0;
   $z12 = $403;
   $404 = $z11;
   $405 = $z13;
   $406 = (($404) + ($405))|0;
   $tmp7 = $406;
   $407 = $z11;
   $408 = $z13;
   $409 = (($407) - ($408))|0;
   $410 = ($409*362)|0;
   $411 = $410 >> 8;
   $tmp11 = $411;
   $412 = $z10;
   $413 = $z12;
   $414 = (($412) + ($413))|0;
   $415 = ($414*473)|0;
   $416 = $415 >> 8;
   $z5 = $416;
   $417 = $z12;
   $418 = ($417*277)|0;
   $419 = $418 >> 8;
   $420 = $z5;
   $421 = (($419) - ($420))|0;
   $tmp10 = $421;
   $422 = $z10;
   $423 = Math_imul($422, -669)|0;
   $424 = $423 >> 8;
   $425 = $z5;
   $426 = (($424) + ($425))|0;
   $tmp12 = $426;
   $427 = $tmp12;
   $428 = $tmp7;
   $429 = (($427) - ($428))|0;
   $tmp6 = $429;
   $430 = $tmp11;
   $431 = $tmp6;
   $432 = (($430) - ($431))|0;
   $tmp5 = $432;
   $433 = $tmp10;
   $434 = $tmp5;
   $435 = (($433) + ($434))|0;
   $tmp4 = $435;
   $436 = $tmp0;
   $437 = $tmp7;
   $438 = (($436) + ($437))|0;
   $439 = $438 >> 5;
   $440 = $439 & 1023;
   $441 = $range_limit;
   $442 = (($441) + ($440)|0);
   $443 = HEAP8[$442>>0]|0;
   $444 = $outptr;
   HEAP8[$444>>0] = $443;
   $445 = $tmp0;
   $446 = $tmp7;
   $447 = (($445) - ($446))|0;
   $448 = $447 >> 5;
   $449 = $448 & 1023;
   $450 = $range_limit;
   $451 = (($450) + ($449)|0);
   $452 = HEAP8[$451>>0]|0;
   $453 = $outptr;
   $454 = (($453) + 7|0);
   HEAP8[$454>>0] = $452;
   $455 = $tmp1;
   $456 = $tmp6;
   $457 = (($455) + ($456))|0;
   $458 = $457 >> 5;
   $459 = $458 & 1023;
   $460 = $range_limit;
   $461 = (($460) + ($459)|0);
   $462 = HEAP8[$461>>0]|0;
   $463 = $outptr;
   $464 = (($463) + 1|0);
   HEAP8[$464>>0] = $462;
   $465 = $tmp1;
   $466 = $tmp6;
   $467 = (($465) - ($466))|0;
   $468 = $467 >> 5;
   $469 = $468 & 1023;
   $470 = $range_limit;
   $471 = (($470) + ($469)|0);
   $472 = HEAP8[$471>>0]|0;
   $473 = $outptr;
   $474 = (($473) + 6|0);
   HEAP8[$474>>0] = $472;
   $475 = $tmp2;
   $476 = $tmp5;
   $477 = (($475) + ($476))|0;
   $478 = $477 >> 5;
   $479 = $478 & 1023;
   $480 = $range_limit;
   $481 = (($480) + ($479)|0);
   $482 = HEAP8[$481>>0]|0;
   $483 = $outptr;
   $484 = (($483) + 2|0);
   HEAP8[$484>>0] = $482;
   $485 = $tmp2;
   $486 = $tmp5;
   $487 = (($485) - ($486))|0;
   $488 = $487 >> 5;
   $489 = $488 & 1023;
   $490 = $range_limit;
   $491 = (($490) + ($489)|0);
   $492 = HEAP8[$491>>0]|0;
   $493 = $outptr;
   $494 = (($493) + 5|0);
   HEAP8[$494>>0] = $492;
   $495 = $tmp3;
   $496 = $tmp4;
   $497 = (($495) + ($496))|0;
   $498 = $497 >> 5;
   $499 = $498 & 1023;
   $500 = $range_limit;
   $501 = (($500) + ($499)|0);
   $502 = HEAP8[$501>>0]|0;
   $503 = $outptr;
   $504 = (($503) + 4|0);
   HEAP8[$504>>0] = $502;
   $505 = $tmp3;
   $506 = $tmp4;
   $507 = (($505) - ($506))|0;
   $508 = $507 >> 5;
   $509 = $508 & 1023;
   $510 = $range_limit;
   $511 = (($510) + ($509)|0);
   $512 = HEAP8[$511>>0]|0;
   $513 = $outptr;
   $514 = (($513) + 3|0);
   HEAP8[$514>>0] = $512;
   $515 = $wsptr;
   $516 = (($515) + 32|0);
   $wsptr = $516;
  }
  $517 = $ctr;
  $518 = (($517) + 1)|0;
  $ctr = $518;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_islow($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctr = 0, $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0;
 var $quantptr = 0, $range_limit = 0, $tmp0 = 0, $tmp1 = 0, $tmp10 = 0, $tmp11 = 0, $tmp12 = 0, $tmp13 = 0, $tmp2 = 0, $tmp3 = 0, $workspace = 0, $wsptr = 0, $z1 = 0, $z2 = 0, $z3 = 0, $z4 = 0, $z5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $inptr;
  $16 = (($15) + 16|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = $inptr;
   $21 = (($20) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $inptr;
    $26 = (($25) + 48|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27 << 16 >> 16;
    $29 = ($28|0)==(0);
    if ($29) {
     $30 = $inptr;
     $31 = (($30) + 64|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32 << 16 >> 16;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = $inptr;
      $36 = (($35) + 80|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37 << 16 >> 16;
      $39 = ($38|0)==(0);
      if ($39) {
       $40 = $inptr;
       $41 = (($40) + 96|0);
       $42 = HEAP16[$41>>1]|0;
       $43 = $42 << 16 >> 16;
       $44 = ($43|0)==(0);
       if ($44) {
        $45 = $inptr;
        $46 = (($45) + 112|0);
        $47 = HEAP16[$46>>1]|0;
        $48 = $47 << 16 >> 16;
        $49 = ($48|0)==(0);
        if ($49) {
         $50 = $inptr;
         $51 = HEAP16[$50>>1]|0;
         $52 = $51 << 16 >> 16;
         $53 = $quantptr;
         $54 = HEAP32[$53>>2]|0;
         $55 = Math_imul($52, $54)|0;
         $56 = $55 << 2;
         $dcval = $56;
         $57 = $dcval;
         $58 = $wsptr;
         HEAP32[$58>>2] = $57;
         $59 = $dcval;
         $60 = $wsptr;
         $61 = (($60) + 32|0);
         HEAP32[$61>>2] = $59;
         $62 = $dcval;
         $63 = $wsptr;
         $64 = (($63) + 64|0);
         HEAP32[$64>>2] = $62;
         $65 = $dcval;
         $66 = $wsptr;
         $67 = (($66) + 96|0);
         HEAP32[$67>>2] = $65;
         $68 = $dcval;
         $69 = $wsptr;
         $70 = (($69) + 128|0);
         HEAP32[$70>>2] = $68;
         $71 = $dcval;
         $72 = $wsptr;
         $73 = (($72) + 160|0);
         HEAP32[$73>>2] = $71;
         $74 = $dcval;
         $75 = $wsptr;
         $76 = (($75) + 192|0);
         HEAP32[$76>>2] = $74;
         $77 = $dcval;
         $78 = $wsptr;
         $79 = (($78) + 224|0);
         HEAP32[$79>>2] = $77;
         $80 = $inptr;
         $81 = (($80) + 2|0);
         $inptr = $81;
         $82 = $quantptr;
         $83 = (($82) + 4|0);
         $quantptr = $83;
         $84 = $wsptr;
         $85 = (($84) + 4|0);
         $wsptr = $85;
        } else {
         label = 11;
        }
       } else {
        label = 11;
       }
      } else {
       label = 11;
      }
     } else {
      label = 11;
     }
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   label = 0;
   $86 = $inptr;
   $87 = (($86) + 32|0);
   $88 = HEAP16[$87>>1]|0;
   $89 = $88 << 16 >> 16;
   $90 = $quantptr;
   $91 = (($90) + 64|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = Math_imul($89, $92)|0;
   $z2 = $93;
   $94 = $inptr;
   $95 = (($94) + 96|0);
   $96 = HEAP16[$95>>1]|0;
   $97 = $96 << 16 >> 16;
   $98 = $quantptr;
   $99 = (($98) + 192|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = Math_imul($97, $100)|0;
   $z3 = $101;
   $102 = $z2;
   $103 = $z3;
   $104 = (($102) + ($103))|0;
   $105 = ($104*4433)|0;
   $z1 = $105;
   $106 = $z1;
   $107 = $z3;
   $108 = Math_imul($107, -15137)|0;
   $109 = (($106) + ($108))|0;
   $tmp2 = $109;
   $110 = $z1;
   $111 = $z2;
   $112 = ($111*6270)|0;
   $113 = (($110) + ($112))|0;
   $tmp3 = $113;
   $114 = $inptr;
   $115 = HEAP16[$114>>1]|0;
   $116 = $115 << 16 >> 16;
   $117 = $quantptr;
   $118 = HEAP32[$117>>2]|0;
   $119 = Math_imul($116, $118)|0;
   $z2 = $119;
   $120 = $inptr;
   $121 = (($120) + 64|0);
   $122 = HEAP16[$121>>1]|0;
   $123 = $122 << 16 >> 16;
   $124 = $quantptr;
   $125 = (($124) + 128|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = Math_imul($123, $126)|0;
   $z3 = $127;
   $128 = $z2;
   $129 = $z3;
   $130 = (($128) + ($129))|0;
   $131 = $130 << 13;
   $tmp0 = $131;
   $132 = $z2;
   $133 = $z3;
   $134 = (($132) - ($133))|0;
   $135 = $134 << 13;
   $tmp1 = $135;
   $136 = $tmp0;
   $137 = $tmp3;
   $138 = (($136) + ($137))|0;
   $tmp10 = $138;
   $139 = $tmp0;
   $140 = $tmp3;
   $141 = (($139) - ($140))|0;
   $tmp13 = $141;
   $142 = $tmp1;
   $143 = $tmp2;
   $144 = (($142) + ($143))|0;
   $tmp11 = $144;
   $145 = $tmp1;
   $146 = $tmp2;
   $147 = (($145) - ($146))|0;
   $tmp12 = $147;
   $148 = $inptr;
   $149 = (($148) + 112|0);
   $150 = HEAP16[$149>>1]|0;
   $151 = $150 << 16 >> 16;
   $152 = $quantptr;
   $153 = (($152) + 224|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = Math_imul($151, $154)|0;
   $tmp0 = $155;
   $156 = $inptr;
   $157 = (($156) + 80|0);
   $158 = HEAP16[$157>>1]|0;
   $159 = $158 << 16 >> 16;
   $160 = $quantptr;
   $161 = (($160) + 160|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = Math_imul($159, $162)|0;
   $tmp1 = $163;
   $164 = $inptr;
   $165 = (($164) + 48|0);
   $166 = HEAP16[$165>>1]|0;
   $167 = $166 << 16 >> 16;
   $168 = $quantptr;
   $169 = (($168) + 96|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = Math_imul($167, $170)|0;
   $tmp2 = $171;
   $172 = $inptr;
   $173 = (($172) + 16|0);
   $174 = HEAP16[$173>>1]|0;
   $175 = $174 << 16 >> 16;
   $176 = $quantptr;
   $177 = (($176) + 32|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = Math_imul($175, $178)|0;
   $tmp3 = $179;
   $180 = $tmp0;
   $181 = $tmp3;
   $182 = (($180) + ($181))|0;
   $z1 = $182;
   $183 = $tmp1;
   $184 = $tmp2;
   $185 = (($183) + ($184))|0;
   $z2 = $185;
   $186 = $tmp0;
   $187 = $tmp2;
   $188 = (($186) + ($187))|0;
   $z3 = $188;
   $189 = $tmp1;
   $190 = $tmp3;
   $191 = (($189) + ($190))|0;
   $z4 = $191;
   $192 = $z3;
   $193 = $z4;
   $194 = (($192) + ($193))|0;
   $195 = ($194*9633)|0;
   $z5 = $195;
   $196 = $tmp0;
   $197 = ($196*2446)|0;
   $tmp0 = $197;
   $198 = $tmp1;
   $199 = ($198*16819)|0;
   $tmp1 = $199;
   $200 = $tmp2;
   $201 = ($200*25172)|0;
   $tmp2 = $201;
   $202 = $tmp3;
   $203 = ($202*12299)|0;
   $tmp3 = $203;
   $204 = $z1;
   $205 = Math_imul($204, -7373)|0;
   $z1 = $205;
   $206 = $z2;
   $207 = Math_imul($206, -20995)|0;
   $z2 = $207;
   $208 = $z3;
   $209 = Math_imul($208, -16069)|0;
   $z3 = $209;
   $210 = $z4;
   $211 = Math_imul($210, -3196)|0;
   $z4 = $211;
   $212 = $z5;
   $213 = $z3;
   $214 = (($213) + ($212))|0;
   $z3 = $214;
   $215 = $z5;
   $216 = $z4;
   $217 = (($216) + ($215))|0;
   $z4 = $217;
   $218 = $z1;
   $219 = $z3;
   $220 = (($218) + ($219))|0;
   $221 = $tmp0;
   $222 = (($221) + ($220))|0;
   $tmp0 = $222;
   $223 = $z2;
   $224 = $z4;
   $225 = (($223) + ($224))|0;
   $226 = $tmp1;
   $227 = (($226) + ($225))|0;
   $tmp1 = $227;
   $228 = $z2;
   $229 = $z3;
   $230 = (($228) + ($229))|0;
   $231 = $tmp2;
   $232 = (($231) + ($230))|0;
   $tmp2 = $232;
   $233 = $z1;
   $234 = $z4;
   $235 = (($233) + ($234))|0;
   $236 = $tmp3;
   $237 = (($236) + ($235))|0;
   $tmp3 = $237;
   $238 = $tmp10;
   $239 = $tmp3;
   $240 = (($238) + ($239))|0;
   $241 = (($240) + 1024)|0;
   $242 = $241 >> 11;
   $243 = $wsptr;
   HEAP32[$243>>2] = $242;
   $244 = $tmp10;
   $245 = $tmp3;
   $246 = (($244) - ($245))|0;
   $247 = (($246) + 1024)|0;
   $248 = $247 >> 11;
   $249 = $wsptr;
   $250 = (($249) + 224|0);
   HEAP32[$250>>2] = $248;
   $251 = $tmp11;
   $252 = $tmp2;
   $253 = (($251) + ($252))|0;
   $254 = (($253) + 1024)|0;
   $255 = $254 >> 11;
   $256 = $wsptr;
   $257 = (($256) + 32|0);
   HEAP32[$257>>2] = $255;
   $258 = $tmp11;
   $259 = $tmp2;
   $260 = (($258) - ($259))|0;
   $261 = (($260) + 1024)|0;
   $262 = $261 >> 11;
   $263 = $wsptr;
   $264 = (($263) + 192|0);
   HEAP32[$264>>2] = $262;
   $265 = $tmp12;
   $266 = $tmp1;
   $267 = (($265) + ($266))|0;
   $268 = (($267) + 1024)|0;
   $269 = $268 >> 11;
   $270 = $wsptr;
   $271 = (($270) + 64|0);
   HEAP32[$271>>2] = $269;
   $272 = $tmp12;
   $273 = $tmp1;
   $274 = (($272) - ($273))|0;
   $275 = (($274) + 1024)|0;
   $276 = $275 >> 11;
   $277 = $wsptr;
   $278 = (($277) + 160|0);
   HEAP32[$278>>2] = $276;
   $279 = $tmp13;
   $280 = $tmp0;
   $281 = (($279) + ($280))|0;
   $282 = (($281) + 1024)|0;
   $283 = $282 >> 11;
   $284 = $wsptr;
   $285 = (($284) + 96|0);
   HEAP32[$285>>2] = $283;
   $286 = $tmp13;
   $287 = $tmp0;
   $288 = (($286) - ($287))|0;
   $289 = (($288) + 1024)|0;
   $290 = $289 >> 11;
   $291 = $wsptr;
   $292 = (($291) + 128|0);
   HEAP32[$292>>2] = $290;
   $293 = $inptr;
   $294 = (($293) + 2|0);
   $inptr = $294;
   $295 = $quantptr;
   $296 = (($295) + 4|0);
   $quantptr = $296;
   $297 = $wsptr;
   $298 = (($297) + 4|0);
   $wsptr = $298;
  }
  $299 = $ctr;
  $300 = (($299) + -1)|0;
  $ctr = $300;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $301 = $ctr;
  $302 = ($301|0)<(8);
  if (!($302)) {
   break;
  }
  $303 = $ctr;
  $304 = $3;
  $305 = (($304) + ($303<<2)|0);
  $306 = HEAP32[$305>>2]|0;
  $307 = $4;
  $308 = (($306) + ($307)|0);
  $outptr = $308;
  $309 = $wsptr;
  $310 = (($309) + 4|0);
  $311 = HEAP32[$310>>2]|0;
  $312 = ($311|0)==(0);
  if ($312) {
   $313 = $wsptr;
   $314 = (($313) + 8|0);
   $315 = HEAP32[$314>>2]|0;
   $316 = ($315|0)==(0);
   if ($316) {
    $317 = $wsptr;
    $318 = (($317) + 12|0);
    $319 = HEAP32[$318>>2]|0;
    $320 = ($319|0)==(0);
    if ($320) {
     $321 = $wsptr;
     $322 = (($321) + 16|0);
     $323 = HEAP32[$322>>2]|0;
     $324 = ($323|0)==(0);
     if ($324) {
      $325 = $wsptr;
      $326 = (($325) + 20|0);
      $327 = HEAP32[$326>>2]|0;
      $328 = ($327|0)==(0);
      if ($328) {
       $329 = $wsptr;
       $330 = (($329) + 24|0);
       $331 = HEAP32[$330>>2]|0;
       $332 = ($331|0)==(0);
       if ($332) {
        $333 = $wsptr;
        $334 = (($333) + 28|0);
        $335 = HEAP32[$334>>2]|0;
        $336 = ($335|0)==(0);
        if ($336) {
         $337 = $wsptr;
         $338 = HEAP32[$337>>2]|0;
         $339 = (($338) + 16)|0;
         $340 = $339 >> 5;
         $341 = $340 & 1023;
         $342 = $range_limit;
         $343 = (($342) + ($341)|0);
         $344 = HEAP8[$343>>0]|0;
         $dcval1 = $344;
         $345 = $dcval1;
         $346 = $outptr;
         HEAP8[$346>>0] = $345;
         $347 = $dcval1;
         $348 = $outptr;
         $349 = (($348) + 1|0);
         HEAP8[$349>>0] = $347;
         $350 = $dcval1;
         $351 = $outptr;
         $352 = (($351) + 2|0);
         HEAP8[$352>>0] = $350;
         $353 = $dcval1;
         $354 = $outptr;
         $355 = (($354) + 3|0);
         HEAP8[$355>>0] = $353;
         $356 = $dcval1;
         $357 = $outptr;
         $358 = (($357) + 4|0);
         HEAP8[$358>>0] = $356;
         $359 = $dcval1;
         $360 = $outptr;
         $361 = (($360) + 5|0);
         HEAP8[$361>>0] = $359;
         $362 = $dcval1;
         $363 = $outptr;
         $364 = (($363) + 6|0);
         HEAP8[$364>>0] = $362;
         $365 = $dcval1;
         $366 = $outptr;
         $367 = (($366) + 7|0);
         HEAP8[$367>>0] = $365;
         $368 = $wsptr;
         $369 = (($368) + 32|0);
         $wsptr = $369;
        } else {
         label = 23;
        }
       } else {
        label = 23;
       }
      } else {
       label = 23;
      }
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $370 = $wsptr;
   $371 = (($370) + 8|0);
   $372 = HEAP32[$371>>2]|0;
   $z2 = $372;
   $373 = $wsptr;
   $374 = (($373) + 24|0);
   $375 = HEAP32[$374>>2]|0;
   $z3 = $375;
   $376 = $z2;
   $377 = $z3;
   $378 = (($376) + ($377))|0;
   $379 = ($378*4433)|0;
   $z1 = $379;
   $380 = $z1;
   $381 = $z3;
   $382 = Math_imul($381, -15137)|0;
   $383 = (($380) + ($382))|0;
   $tmp2 = $383;
   $384 = $z1;
   $385 = $z2;
   $386 = ($385*6270)|0;
   $387 = (($384) + ($386))|0;
   $tmp3 = $387;
   $388 = $wsptr;
   $389 = HEAP32[$388>>2]|0;
   $390 = $wsptr;
   $391 = (($390) + 16|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = (($389) + ($392))|0;
   $394 = $393 << 13;
   $tmp0 = $394;
   $395 = $wsptr;
   $396 = HEAP32[$395>>2]|0;
   $397 = $wsptr;
   $398 = (($397) + 16|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = (($396) - ($399))|0;
   $401 = $400 << 13;
   $tmp1 = $401;
   $402 = $tmp0;
   $403 = $tmp3;
   $404 = (($402) + ($403))|0;
   $tmp10 = $404;
   $405 = $tmp0;
   $406 = $tmp3;
   $407 = (($405) - ($406))|0;
   $tmp13 = $407;
   $408 = $tmp1;
   $409 = $tmp2;
   $410 = (($408) + ($409))|0;
   $tmp11 = $410;
   $411 = $tmp1;
   $412 = $tmp2;
   $413 = (($411) - ($412))|0;
   $tmp12 = $413;
   $414 = $wsptr;
   $415 = (($414) + 28|0);
   $416 = HEAP32[$415>>2]|0;
   $tmp0 = $416;
   $417 = $wsptr;
   $418 = (($417) + 20|0);
   $419 = HEAP32[$418>>2]|0;
   $tmp1 = $419;
   $420 = $wsptr;
   $421 = (($420) + 12|0);
   $422 = HEAP32[$421>>2]|0;
   $tmp2 = $422;
   $423 = $wsptr;
   $424 = (($423) + 4|0);
   $425 = HEAP32[$424>>2]|0;
   $tmp3 = $425;
   $426 = $tmp0;
   $427 = $tmp3;
   $428 = (($426) + ($427))|0;
   $z1 = $428;
   $429 = $tmp1;
   $430 = $tmp2;
   $431 = (($429) + ($430))|0;
   $z2 = $431;
   $432 = $tmp0;
   $433 = $tmp2;
   $434 = (($432) + ($433))|0;
   $z3 = $434;
   $435 = $tmp1;
   $436 = $tmp3;
   $437 = (($435) + ($436))|0;
   $z4 = $437;
   $438 = $z3;
   $439 = $z4;
   $440 = (($438) + ($439))|0;
   $441 = ($440*9633)|0;
   $z5 = $441;
   $442 = $tmp0;
   $443 = ($442*2446)|0;
   $tmp0 = $443;
   $444 = $tmp1;
   $445 = ($444*16819)|0;
   $tmp1 = $445;
   $446 = $tmp2;
   $447 = ($446*25172)|0;
   $tmp2 = $447;
   $448 = $tmp3;
   $449 = ($448*12299)|0;
   $tmp3 = $449;
   $450 = $z1;
   $451 = Math_imul($450, -7373)|0;
   $z1 = $451;
   $452 = $z2;
   $453 = Math_imul($452, -20995)|0;
   $z2 = $453;
   $454 = $z3;
   $455 = Math_imul($454, -16069)|0;
   $z3 = $455;
   $456 = $z4;
   $457 = Math_imul($456, -3196)|0;
   $z4 = $457;
   $458 = $z5;
   $459 = $z3;
   $460 = (($459) + ($458))|0;
   $z3 = $460;
   $461 = $z5;
   $462 = $z4;
   $463 = (($462) + ($461))|0;
   $z4 = $463;
   $464 = $z1;
   $465 = $z3;
   $466 = (($464) + ($465))|0;
   $467 = $tmp0;
   $468 = (($467) + ($466))|0;
   $tmp0 = $468;
   $469 = $z2;
   $470 = $z4;
   $471 = (($469) + ($470))|0;
   $472 = $tmp1;
   $473 = (($472) + ($471))|0;
   $tmp1 = $473;
   $474 = $z2;
   $475 = $z3;
   $476 = (($474) + ($475))|0;
   $477 = $tmp2;
   $478 = (($477) + ($476))|0;
   $tmp2 = $478;
   $479 = $z1;
   $480 = $z4;
   $481 = (($479) + ($480))|0;
   $482 = $tmp3;
   $483 = (($482) + ($481))|0;
   $tmp3 = $483;
   $484 = $tmp10;
   $485 = $tmp3;
   $486 = (($484) + ($485))|0;
   $487 = (($486) + 131072)|0;
   $488 = $487 >> 18;
   $489 = $488 & 1023;
   $490 = $range_limit;
   $491 = (($490) + ($489)|0);
   $492 = HEAP8[$491>>0]|0;
   $493 = $outptr;
   HEAP8[$493>>0] = $492;
   $494 = $tmp10;
   $495 = $tmp3;
   $496 = (($494) - ($495))|0;
   $497 = (($496) + 131072)|0;
   $498 = $497 >> 18;
   $499 = $498 & 1023;
   $500 = $range_limit;
   $501 = (($500) + ($499)|0);
   $502 = HEAP8[$501>>0]|0;
   $503 = $outptr;
   $504 = (($503) + 7|0);
   HEAP8[$504>>0] = $502;
   $505 = $tmp11;
   $506 = $tmp2;
   $507 = (($505) + ($506))|0;
   $508 = (($507) + 131072)|0;
   $509 = $508 >> 18;
   $510 = $509 & 1023;
   $511 = $range_limit;
   $512 = (($511) + ($510)|0);
   $513 = HEAP8[$512>>0]|0;
   $514 = $outptr;
   $515 = (($514) + 1|0);
   HEAP8[$515>>0] = $513;
   $516 = $tmp11;
   $517 = $tmp2;
   $518 = (($516) - ($517))|0;
   $519 = (($518) + 131072)|0;
   $520 = $519 >> 18;
   $521 = $520 & 1023;
   $522 = $range_limit;
   $523 = (($522) + ($521)|0);
   $524 = HEAP8[$523>>0]|0;
   $525 = $outptr;
   $526 = (($525) + 6|0);
   HEAP8[$526>>0] = $524;
   $527 = $tmp12;
   $528 = $tmp1;
   $529 = (($527) + ($528))|0;
   $530 = (($529) + 131072)|0;
   $531 = $530 >> 18;
   $532 = $531 & 1023;
   $533 = $range_limit;
   $534 = (($533) + ($532)|0);
   $535 = HEAP8[$534>>0]|0;
   $536 = $outptr;
   $537 = (($536) + 2|0);
   HEAP8[$537>>0] = $535;
   $538 = $tmp12;
   $539 = $tmp1;
   $540 = (($538) - ($539))|0;
   $541 = (($540) + 131072)|0;
   $542 = $541 >> 18;
   $543 = $542 & 1023;
   $544 = $range_limit;
   $545 = (($544) + ($543)|0);
   $546 = HEAP8[$545>>0]|0;
   $547 = $outptr;
   $548 = (($547) + 5|0);
   HEAP8[$548>>0] = $546;
   $549 = $tmp13;
   $550 = $tmp0;
   $551 = (($549) + ($550))|0;
   $552 = (($551) + 131072)|0;
   $553 = $552 >> 18;
   $554 = $553 & 1023;
   $555 = $range_limit;
   $556 = (($555) + ($554)|0);
   $557 = HEAP8[$556>>0]|0;
   $558 = $outptr;
   $559 = (($558) + 3|0);
   HEAP8[$559>>0] = $557;
   $560 = $tmp13;
   $561 = $tmp0;
   $562 = (($560) - ($561))|0;
   $563 = (($562) + 131072)|0;
   $564 = $563 >> 18;
   $565 = $564 & 1023;
   $566 = $range_limit;
   $567 = (($566) + ($565)|0);
   $568 = HEAP8[$567>>0]|0;
   $569 = $outptr;
   $570 = (($569) + 4|0);
   HEAP8[$570>>0] = $568;
   $571 = $wsptr;
   $572 = (($571) + 32|0);
   $wsptr = $572;
  }
  $573 = $ctr;
  $574 = (($573) + 1)|0;
  $ctr = $574;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_4x4($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $ctr = 0, $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0, $quantptr = 0, $range_limit = 0, $tmp0 = 0, $tmp10 = 0, $tmp12 = 0, $tmp2 = 0, $workspace = 0, $wsptr = 0, $z1 = 0, $z2 = 0, $z3 = 0, $z4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $ctr;
  $16 = ($15|0)==(4);
  do {
   if ($16) {
   } else {
    $17 = $inptr;
    $18 = (($17) + 16|0);
    $19 = HEAP16[$18>>1]|0;
    $20 = $19 << 16 >> 16;
    $21 = ($20|0)==(0);
    if ($21) {
     $22 = $inptr;
     $23 = (($22) + 32|0);
     $24 = HEAP16[$23>>1]|0;
     $25 = $24 << 16 >> 16;
     $26 = ($25|0)==(0);
     if ($26) {
      $27 = $inptr;
      $28 = (($27) + 48|0);
      $29 = HEAP16[$28>>1]|0;
      $30 = $29 << 16 >> 16;
      $31 = ($30|0)==(0);
      if ($31) {
       $32 = $inptr;
       $33 = (($32) + 80|0);
       $34 = HEAP16[$33>>1]|0;
       $35 = $34 << 16 >> 16;
       $36 = ($35|0)==(0);
       if ($36) {
        $37 = $inptr;
        $38 = (($37) + 96|0);
        $39 = HEAP16[$38>>1]|0;
        $40 = $39 << 16 >> 16;
        $41 = ($40|0)==(0);
        if ($41) {
         $42 = $inptr;
         $43 = (($42) + 112|0);
         $44 = HEAP16[$43>>1]|0;
         $45 = $44 << 16 >> 16;
         $46 = ($45|0)==(0);
         if ($46) {
          $47 = $inptr;
          $48 = HEAP16[$47>>1]|0;
          $49 = $48 << 16 >> 16;
          $50 = $quantptr;
          $51 = HEAP32[$50>>2]|0;
          $52 = Math_imul($49, $51)|0;
          $53 = $52 << 2;
          $dcval = $53;
          $54 = $dcval;
          $55 = $wsptr;
          HEAP32[$55>>2] = $54;
          $56 = $dcval;
          $57 = $wsptr;
          $58 = (($57) + 32|0);
          HEAP32[$58>>2] = $56;
          $59 = $dcval;
          $60 = $wsptr;
          $61 = (($60) + 64|0);
          HEAP32[$61>>2] = $59;
          $62 = $dcval;
          $63 = $wsptr;
          $64 = (($63) + 96|0);
          HEAP32[$64>>2] = $62;
          break;
         }
        }
       }
      }
     }
    }
    $65 = $inptr;
    $66 = HEAP16[$65>>1]|0;
    $67 = $66 << 16 >> 16;
    $68 = $quantptr;
    $69 = HEAP32[$68>>2]|0;
    $70 = Math_imul($67, $69)|0;
    $tmp0 = $70;
    $71 = $tmp0;
    $72 = $71 << 14;
    $tmp0 = $72;
    $73 = $inptr;
    $74 = (($73) + 32|0);
    $75 = HEAP16[$74>>1]|0;
    $76 = $75 << 16 >> 16;
    $77 = $quantptr;
    $78 = (($77) + 64|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = Math_imul($76, $79)|0;
    $z2 = $80;
    $81 = $inptr;
    $82 = (($81) + 96|0);
    $83 = HEAP16[$82>>1]|0;
    $84 = $83 << 16 >> 16;
    $85 = $quantptr;
    $86 = (($85) + 192|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = Math_imul($84, $87)|0;
    $z3 = $88;
    $89 = $z2;
    $90 = ($89*15137)|0;
    $91 = $z3;
    $92 = Math_imul($91, -6270)|0;
    $93 = (($90) + ($92))|0;
    $tmp2 = $93;
    $94 = $tmp0;
    $95 = $tmp2;
    $96 = (($94) + ($95))|0;
    $tmp10 = $96;
    $97 = $tmp0;
    $98 = $tmp2;
    $99 = (($97) - ($98))|0;
    $tmp12 = $99;
    $100 = $inptr;
    $101 = (($100) + 112|0);
    $102 = HEAP16[$101>>1]|0;
    $103 = $102 << 16 >> 16;
    $104 = $quantptr;
    $105 = (($104) + 224|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = Math_imul($103, $106)|0;
    $z1 = $107;
    $108 = $inptr;
    $109 = (($108) + 80|0);
    $110 = HEAP16[$109>>1]|0;
    $111 = $110 << 16 >> 16;
    $112 = $quantptr;
    $113 = (($112) + 160|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = Math_imul($111, $114)|0;
    $z2 = $115;
    $116 = $inptr;
    $117 = (($116) + 48|0);
    $118 = HEAP16[$117>>1]|0;
    $119 = $118 << 16 >> 16;
    $120 = $quantptr;
    $121 = (($120) + 96|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = Math_imul($119, $122)|0;
    $z3 = $123;
    $124 = $inptr;
    $125 = (($124) + 16|0);
    $126 = HEAP16[$125>>1]|0;
    $127 = $126 << 16 >> 16;
    $128 = $quantptr;
    $129 = (($128) + 32|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = Math_imul($127, $130)|0;
    $z4 = $131;
    $132 = $z1;
    $133 = Math_imul($132, -1730)|0;
    $134 = $z2;
    $135 = ($134*11893)|0;
    $136 = (($133) + ($135))|0;
    $137 = $z3;
    $138 = Math_imul($137, -17799)|0;
    $139 = (($136) + ($138))|0;
    $140 = $z4;
    $141 = ($140*8697)|0;
    $142 = (($139) + ($141))|0;
    $tmp0 = $142;
    $143 = $z1;
    $144 = Math_imul($143, -4176)|0;
    $145 = $z2;
    $146 = Math_imul($145, -4926)|0;
    $147 = (($144) + ($146))|0;
    $148 = $z3;
    $149 = ($148*7373)|0;
    $150 = (($147) + ($149))|0;
    $151 = $z4;
    $152 = ($151*20995)|0;
    $153 = (($150) + ($152))|0;
    $tmp2 = $153;
    $154 = $tmp10;
    $155 = $tmp2;
    $156 = (($154) + ($155))|0;
    $157 = (($156) + 2048)|0;
    $158 = $157 >> 12;
    $159 = $wsptr;
    HEAP32[$159>>2] = $158;
    $160 = $tmp10;
    $161 = $tmp2;
    $162 = (($160) - ($161))|0;
    $163 = (($162) + 2048)|0;
    $164 = $163 >> 12;
    $165 = $wsptr;
    $166 = (($165) + 96|0);
    HEAP32[$166>>2] = $164;
    $167 = $tmp12;
    $168 = $tmp0;
    $169 = (($167) + ($168))|0;
    $170 = (($169) + 2048)|0;
    $171 = $170 >> 12;
    $172 = $wsptr;
    $173 = (($172) + 32|0);
    HEAP32[$173>>2] = $171;
    $174 = $tmp12;
    $175 = $tmp0;
    $176 = (($174) - ($175))|0;
    $177 = (($176) + 2048)|0;
    $178 = $177 >> 12;
    $179 = $wsptr;
    $180 = (($179) + 64|0);
    HEAP32[$180>>2] = $178;
   }
  } while(0);
  $181 = $inptr;
  $182 = (($181) + 2|0);
  $inptr = $182;
  $183 = $quantptr;
  $184 = (($183) + 4|0);
  $quantptr = $184;
  $185 = $wsptr;
  $186 = (($185) + 4|0);
  $wsptr = $186;
  $187 = $ctr;
  $188 = (($187) + -1)|0;
  $ctr = $188;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $189 = $ctr;
  $190 = ($189|0)<(4);
  if (!($190)) {
   break;
  }
  $191 = $ctr;
  $192 = $3;
  $193 = (($192) + ($191<<2)|0);
  $194 = HEAP32[$193>>2]|0;
  $195 = $4;
  $196 = (($194) + ($195)|0);
  $outptr = $196;
  $197 = $wsptr;
  $198 = (($197) + 4|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = ($199|0)==(0);
  if ($200) {
   $201 = $wsptr;
   $202 = (($201) + 8|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = ($203|0)==(0);
   if ($204) {
    $205 = $wsptr;
    $206 = (($205) + 12|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = ($207|0)==(0);
    if ($208) {
     $209 = $wsptr;
     $210 = (($209) + 20|0);
     $211 = HEAP32[$210>>2]|0;
     $212 = ($211|0)==(0);
     if ($212) {
      $213 = $wsptr;
      $214 = (($213) + 24|0);
      $215 = HEAP32[$214>>2]|0;
      $216 = ($215|0)==(0);
      if ($216) {
       $217 = $wsptr;
       $218 = (($217) + 28|0);
       $219 = HEAP32[$218>>2]|0;
       $220 = ($219|0)==(0);
       if ($220) {
        $221 = $wsptr;
        $222 = HEAP32[$221>>2]|0;
        $223 = (($222) + 16)|0;
        $224 = $223 >> 5;
        $225 = $224 & 1023;
        $226 = $range_limit;
        $227 = (($226) + ($225)|0);
        $228 = HEAP8[$227>>0]|0;
        $dcval1 = $228;
        $229 = $dcval1;
        $230 = $outptr;
        HEAP8[$230>>0] = $229;
        $231 = $dcval1;
        $232 = $outptr;
        $233 = (($232) + 1|0);
        HEAP8[$233>>0] = $231;
        $234 = $dcval1;
        $235 = $outptr;
        $236 = (($235) + 2|0);
        HEAP8[$236>>0] = $234;
        $237 = $dcval1;
        $238 = $outptr;
        $239 = (($238) + 3|0);
        HEAP8[$239>>0] = $237;
        $240 = $wsptr;
        $241 = (($240) + 32|0);
        $wsptr = $241;
       } else {
        label = 23;
       }
      } else {
       label = 23;
      }
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $242 = $wsptr;
   $243 = HEAP32[$242>>2]|0;
   $244 = $243 << 14;
   $tmp0 = $244;
   $245 = $wsptr;
   $246 = (($245) + 8|0);
   $247 = HEAP32[$246>>2]|0;
   $248 = ($247*15137)|0;
   $249 = $wsptr;
   $250 = (($249) + 24|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = Math_imul($251, -6270)|0;
   $253 = (($248) + ($252))|0;
   $tmp2 = $253;
   $254 = $tmp0;
   $255 = $tmp2;
   $256 = (($254) + ($255))|0;
   $tmp10 = $256;
   $257 = $tmp0;
   $258 = $tmp2;
   $259 = (($257) - ($258))|0;
   $tmp12 = $259;
   $260 = $wsptr;
   $261 = (($260) + 28|0);
   $262 = HEAP32[$261>>2]|0;
   $z1 = $262;
   $263 = $wsptr;
   $264 = (($263) + 20|0);
   $265 = HEAP32[$264>>2]|0;
   $z2 = $265;
   $266 = $wsptr;
   $267 = (($266) + 12|0);
   $268 = HEAP32[$267>>2]|0;
   $z3 = $268;
   $269 = $wsptr;
   $270 = (($269) + 4|0);
   $271 = HEAP32[$270>>2]|0;
   $z4 = $271;
   $272 = $z1;
   $273 = Math_imul($272, -1730)|0;
   $274 = $z2;
   $275 = ($274*11893)|0;
   $276 = (($273) + ($275))|0;
   $277 = $z3;
   $278 = Math_imul($277, -17799)|0;
   $279 = (($276) + ($278))|0;
   $280 = $z4;
   $281 = ($280*8697)|0;
   $282 = (($279) + ($281))|0;
   $tmp0 = $282;
   $283 = $z1;
   $284 = Math_imul($283, -4176)|0;
   $285 = $z2;
   $286 = Math_imul($285, -4926)|0;
   $287 = (($284) + ($286))|0;
   $288 = $z3;
   $289 = ($288*7373)|0;
   $290 = (($287) + ($289))|0;
   $291 = $z4;
   $292 = ($291*20995)|0;
   $293 = (($290) + ($292))|0;
   $tmp2 = $293;
   $294 = $tmp10;
   $295 = $tmp2;
   $296 = (($294) + ($295))|0;
   $297 = (($296) + 262144)|0;
   $298 = $297 >> 19;
   $299 = $298 & 1023;
   $300 = $range_limit;
   $301 = (($300) + ($299)|0);
   $302 = HEAP8[$301>>0]|0;
   $303 = $outptr;
   HEAP8[$303>>0] = $302;
   $304 = $tmp10;
   $305 = $tmp2;
   $306 = (($304) - ($305))|0;
   $307 = (($306) + 262144)|0;
   $308 = $307 >> 19;
   $309 = $308 & 1023;
   $310 = $range_limit;
   $311 = (($310) + ($309)|0);
   $312 = HEAP8[$311>>0]|0;
   $313 = $outptr;
   $314 = (($313) + 3|0);
   HEAP8[$314>>0] = $312;
   $315 = $tmp12;
   $316 = $tmp0;
   $317 = (($315) + ($316))|0;
   $318 = (($317) + 262144)|0;
   $319 = $318 >> 19;
   $320 = $319 & 1023;
   $321 = $range_limit;
   $322 = (($321) + ($320)|0);
   $323 = HEAP8[$322>>0]|0;
   $324 = $outptr;
   $325 = (($324) + 1|0);
   HEAP8[$325>>0] = $323;
   $326 = $tmp12;
   $327 = $tmp0;
   $328 = (($326) - ($327))|0;
   $329 = (($328) + 262144)|0;
   $330 = $329 >> 19;
   $331 = $330 & 1023;
   $332 = $range_limit;
   $333 = (($332) + ($331)|0);
   $334 = HEAP8[$333>>0]|0;
   $335 = $outptr;
   $336 = (($335) + 2|0);
   HEAP8[$336>>0] = $334;
   $337 = $wsptr;
   $338 = (($337) + 32|0);
   $wsptr = $338;
  }
  $339 = $ctr;
  $340 = (($339) + 1)|0;
  $ctr = $340;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_2x2($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctr = 0, $dcval = 0, $dcval1 = 0, $inptr = 0, $outptr = 0, $quantptr = 0;
 var $range_limit = 0, $tmp0 = 0, $tmp10 = 0, $workspace = 0, $wsptr = 0, $z1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $workspace = sp + 8|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $2;
 $inptr = $9;
 $10 = $1;
 $11 = (($10) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 $quantptr = $12;
 $wsptr = $workspace;
 $ctr = 8;
 while(1) {
  $13 = $ctr;
  $14 = ($13|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = $ctr;
  $16 = ($15|0)==(6);
  do {
   if ($16) {
    label = 6;
   } else {
    $17 = $ctr;
    $18 = ($17|0)==(4);
    if ($18) {
     label = 6;
    } else {
     $19 = $ctr;
     $20 = ($19|0)==(2);
     if ($20) {
      label = 6;
     } else {
      $21 = $inptr;
      $22 = (($21) + 16|0);
      $23 = HEAP16[$22>>1]|0;
      $24 = $23 << 16 >> 16;
      $25 = ($24|0)==(0);
      if ($25) {
       $26 = $inptr;
       $27 = (($26) + 48|0);
       $28 = HEAP16[$27>>1]|0;
       $29 = $28 << 16 >> 16;
       $30 = ($29|0)==(0);
       if ($30) {
        $31 = $inptr;
        $32 = (($31) + 80|0);
        $33 = HEAP16[$32>>1]|0;
        $34 = $33 << 16 >> 16;
        $35 = ($34|0)==(0);
        if ($35) {
         $36 = $inptr;
         $37 = (($36) + 112|0);
         $38 = HEAP16[$37>>1]|0;
         $39 = $38 << 16 >> 16;
         $40 = ($39|0)==(0);
         if ($40) {
          $41 = $inptr;
          $42 = HEAP16[$41>>1]|0;
          $43 = $42 << 16 >> 16;
          $44 = $quantptr;
          $45 = HEAP32[$44>>2]|0;
          $46 = Math_imul($43, $45)|0;
          $47 = $46 << 2;
          $dcval = $47;
          $48 = $dcval;
          $49 = $wsptr;
          HEAP32[$49>>2] = $48;
          $50 = $dcval;
          $51 = $wsptr;
          $52 = (($51) + 32|0);
          HEAP32[$52>>2] = $50;
          break;
         }
        }
       }
      }
      $53 = $inptr;
      $54 = HEAP16[$53>>1]|0;
      $55 = $54 << 16 >> 16;
      $56 = $quantptr;
      $57 = HEAP32[$56>>2]|0;
      $58 = Math_imul($55, $57)|0;
      $z1 = $58;
      $59 = $z1;
      $60 = $59 << 15;
      $tmp10 = $60;
      $61 = $inptr;
      $62 = (($61) + 112|0);
      $63 = HEAP16[$62>>1]|0;
      $64 = $63 << 16 >> 16;
      $65 = $quantptr;
      $66 = (($65) + 224|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = Math_imul($64, $67)|0;
      $z1 = $68;
      $69 = $z1;
      $70 = Math_imul($69, -5906)|0;
      $tmp0 = $70;
      $71 = $inptr;
      $72 = (($71) + 80|0);
      $73 = HEAP16[$72>>1]|0;
      $74 = $73 << 16 >> 16;
      $75 = $quantptr;
      $76 = (($75) + 160|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = Math_imul($74, $77)|0;
      $z1 = $78;
      $79 = $z1;
      $80 = ($79*6967)|0;
      $81 = $tmp0;
      $82 = (($81) + ($80))|0;
      $tmp0 = $82;
      $83 = $inptr;
      $84 = (($83) + 48|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85 << 16 >> 16;
      $87 = $quantptr;
      $88 = (($87) + 96|0);
      $89 = HEAP32[$88>>2]|0;
      $90 = Math_imul($86, $89)|0;
      $z1 = $90;
      $91 = $z1;
      $92 = Math_imul($91, -10426)|0;
      $93 = $tmp0;
      $94 = (($93) + ($92))|0;
      $tmp0 = $94;
      $95 = $inptr;
      $96 = (($95) + 16|0);
      $97 = HEAP16[$96>>1]|0;
      $98 = $97 << 16 >> 16;
      $99 = $quantptr;
      $100 = (($99) + 32|0);
      $101 = HEAP32[$100>>2]|0;
      $102 = Math_imul($98, $101)|0;
      $z1 = $102;
      $103 = $z1;
      $104 = ($103*29692)|0;
      $105 = $tmp0;
      $106 = (($105) + ($104))|0;
      $tmp0 = $106;
      $107 = $tmp10;
      $108 = $tmp0;
      $109 = (($107) + ($108))|0;
      $110 = (($109) + 4096)|0;
      $111 = $110 >> 13;
      $112 = $wsptr;
      HEAP32[$112>>2] = $111;
      $113 = $tmp10;
      $114 = $tmp0;
      $115 = (($113) - ($114))|0;
      $116 = (($115) + 4096)|0;
      $117 = $116 >> 13;
      $118 = $wsptr;
      $119 = (($118) + 32|0);
      HEAP32[$119>>2] = $117;
     }
    }
   }
  } while(0);
  if ((label|0) == 6) {
   label = 0;
  }
  $120 = $inptr;
  $121 = (($120) + 2|0);
  $inptr = $121;
  $122 = $quantptr;
  $123 = (($122) + 4|0);
  $quantptr = $123;
  $124 = $wsptr;
  $125 = (($124) + 4|0);
  $wsptr = $125;
  $126 = $ctr;
  $127 = (($126) + -1)|0;
  $ctr = $127;
 }
 $wsptr = $workspace;
 $ctr = 0;
 while(1) {
  $128 = $ctr;
  $129 = ($128|0)<(2);
  if (!($129)) {
   break;
  }
  $130 = $ctr;
  $131 = $3;
  $132 = (($131) + ($130<<2)|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = $4;
  $135 = (($133) + ($134)|0);
  $outptr = $135;
  $136 = $wsptr;
  $137 = (($136) + 4|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = ($138|0)==(0);
  if ($139) {
   $140 = $wsptr;
   $141 = (($140) + 12|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = ($142|0)==(0);
   if ($143) {
    $144 = $wsptr;
    $145 = (($144) + 20|0);
    $146 = HEAP32[$145>>2]|0;
    $147 = ($146|0)==(0);
    if ($147) {
     $148 = $wsptr;
     $149 = (($148) + 28|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = ($150|0)==(0);
     if ($151) {
      $152 = $wsptr;
      $153 = HEAP32[$152>>2]|0;
      $154 = (($153) + 16)|0;
      $155 = $154 >> 5;
      $156 = $155 & 1023;
      $157 = $range_limit;
      $158 = (($157) + ($156)|0);
      $159 = HEAP8[$158>>0]|0;
      $dcval1 = $159;
      $160 = $dcval1;
      $161 = $outptr;
      HEAP8[$161>>0] = $160;
      $162 = $dcval1;
      $163 = $outptr;
      $164 = (($163) + 1|0);
      HEAP8[$164>>0] = $162;
      $165 = $wsptr;
      $166 = (($165) + 32|0);
      $wsptr = $166;
     } else {
      label = 21;
     }
    } else {
     label = 21;
    }
   } else {
    label = 21;
   }
  } else {
   label = 21;
  }
  if ((label|0) == 21) {
   label = 0;
   $167 = $wsptr;
   $168 = HEAP32[$167>>2]|0;
   $169 = $168 << 15;
   $tmp10 = $169;
   $170 = $wsptr;
   $171 = (($170) + 28|0);
   $172 = HEAP32[$171>>2]|0;
   $173 = Math_imul($172, -5906)|0;
   $174 = $wsptr;
   $175 = (($174) + 20|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = ($176*6967)|0;
   $178 = (($173) + ($177))|0;
   $179 = $wsptr;
   $180 = (($179) + 12|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = Math_imul($181, -10426)|0;
   $183 = (($178) + ($182))|0;
   $184 = $wsptr;
   $185 = (($184) + 4|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = ($186*29692)|0;
   $188 = (($183) + ($187))|0;
   $tmp0 = $188;
   $189 = $tmp10;
   $190 = $tmp0;
   $191 = (($189) + ($190))|0;
   $192 = (($191) + 524288)|0;
   $193 = $192 >> 20;
   $194 = $193 & 1023;
   $195 = $range_limit;
   $196 = (($195) + ($194)|0);
   $197 = HEAP8[$196>>0]|0;
   $198 = $outptr;
   HEAP8[$198>>0] = $197;
   $199 = $tmp10;
   $200 = $tmp0;
   $201 = (($199) - ($200))|0;
   $202 = (($201) + 524288)|0;
   $203 = $202 >> 20;
   $204 = $203 & 1023;
   $205 = $range_limit;
   $206 = (($205) + ($204)|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $outptr;
   $209 = (($208) + 1|0);
   HEAP8[$209>>0] = $207;
   $210 = $wsptr;
   $211 = (($210) + 32|0);
   $wsptr = $211;
  }
  $212 = $ctr;
  $213 = (($212) + 1)|0;
  $ctr = $213;
 }
 STACKTOP = sp;return;
}
function _jpeg_idct_1x1($cinfo,$compptr,$coef_block,$output_buf,$output_col) {
 $cinfo = $cinfo|0;
 $compptr = $compptr|0;
 $coef_block = $coef_block|0;
 $output_buf = $output_buf|0;
 $output_col = $output_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dcval = 0, $quantptr = 0, $range_limit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cinfo;
 $1 = $compptr;
 $2 = $coef_block;
 $3 = $output_buf;
 $4 = $output_col;
 $5 = $0;
 $6 = (($5) + 324|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 128|0);
 $range_limit = $8;
 $9 = $1;
 $10 = (($9) + 80|0);
 $11 = HEAP32[$10>>2]|0;
 $quantptr = $11;
 $12 = $2;
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = $quantptr;
 $16 = HEAP32[$15>>2]|0;
 $17 = Math_imul($14, $16)|0;
 $dcval = $17;
 $18 = $dcval;
 $19 = (($18) + 4)|0;
 $20 = $19 >> 3;
 $dcval = $20;
 $21 = $dcval;
 $22 = $21 & 1023;
 $23 = $range_limit;
 $24 = (($23) + ($22)|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $4;
 $27 = $3;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($26)|0);
 HEAP8[$29>>0] = $25;
 STACKTOP = sp;return;
}
function _extractfet_ret($ovalue,$feature,$fet) {
 $ovalue = $ovalue|0;
 $feature = $feature|0;
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, $value = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ovalue;
 $2 = $feature;
 $3 = $fet;
 $item = 0;
 while(1) {
  $4 = $item;
  $5 = $3;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if ($8) {
   $9 = $item;
   $10 = $3;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + ($9<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $2;
   $16 = (_strcmp($14,$15)|0);
   $17 = ($16|0)!=(0);
   $47 = $17;
  } else {
   $47 = 0;
  }
  if (!($47)) {
   break;
  }
  $18 = $item;
  $19 = (($18) + 1)|0;
  $item = $19;
 }
 $20 = $item;
 $21 = $3;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)>=($23|0);
 if ($24) {
  $25 = HEAP32[_stderr>>2]|0;
  $26 = $2;
  HEAP32[$vararg_buffer>>2] = $26;
  (_fprintf(($25|0),(51208|0),($vararg_buffer|0))|0);
  $0 = -2;
  $46 = $0;
  STACKTOP = sp;return ($46|0);
 }
 $27 = $item;
 $28 = $3;
 $29 = (($28) + 12|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + ($27<<2)|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0|0);
 do {
  if ($33) {
   $34 = $item;
   $35 = $3;
   $36 = (($35) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + ($34<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (___strdup($39)|0);
   $value = $40;
   $41 = $value;
   $42 = ($41|0)==(0|0);
   if (!($42)) {
    break;
   }
   $43 = HEAP32[_stderr>>2]|0;
   (_fprintf(($43|0),(51256|0),($vararg_buffer1|0))|0);
   $0 = -3;
   $46 = $0;
   STACKTOP = sp;return ($46|0);
  } else {
   $value = 0;
  }
 } while(0);
 $44 = $value;
 $45 = $1;
 HEAP32[$45>>2] = $44;
 $0 = 0;
 $46 = $0;
 STACKTOP = sp;return ($46|0);
}
function _freefet($fet) {
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $fet;
 $item = 0;
 while(1) {
  $1 = $item;
  $2 = $0;
  $3 = (($2) + 4|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $item;
  $7 = $0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free($11);
  $12 = $item;
  $13 = $0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  _free($17);
  $18 = $item;
  $19 = (($18) + 1)|0;
  $item = $19;
 }
 $20 = $0;
 $21 = (($20) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 _free($22);
 $23 = $0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 _free($25);
 $26 = $0;
 _free($26);
 STACKTOP = sp;return;
}
function _string2fet($ofet,$istr) {
 $ofet = $ofet|0;
 $istr = $istr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $fet = 0, $iptr = 0, $name = 0, $optr = 0, $ret = 0;
 var $value = 0, $vararg_buffer = 0, $vptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1072|0;
 $vararg_buffer = sp;
 $name = sp + 552|0;
 $value = sp + 40|0;
 $fet = sp + 4|0;
 $1 = $ofet;
 $2 = $istr;
 $3 = (_allocfet_ret($fet,100)|0);
 $ret = $3;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = $ret;
  $0 = $5;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 $6 = $2;
 $iptr = $6;
 while(1) {
  $7 = $iptr;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   label = 39;
   break;
  }
  $optr = $name;
  while(1) {
   $11 = $iptr;
   $12 = HEAP8[$11>>0]|0;
   $13 = $12 << 24 >> 24;
   $14 = ($13|0)!=(0);
   if ($14) {
    $15 = $iptr;
    $16 = HEAP8[$15>>0]|0;
    $17 = $16 << 24 >> 24;
    $18 = ($17|0)!=(32);
    if ($18) {
     $19 = $iptr;
     $20 = HEAP8[$19>>0]|0;
     $21 = $20 << 24 >> 24;
     $22 = ($21|0)!=(9);
     $89 = $22;
    } else {
     $89 = 0;
    }
   } else {
    $89 = 0;
   }
   if (!($89)) {
    break;
   }
   $23 = $iptr;
   $24 = (($23) + 1|0);
   $iptr = $24;
   $25 = HEAP8[$23>>0]|0;
   $26 = $optr;
   $27 = (($26) + 1|0);
   $optr = $27;
   HEAP8[$26>>0] = $25;
  }
  $28 = $optr;
  HEAP8[$28>>0] = 0;
  while(1) {
   $29 = $iptr;
   $30 = HEAP8[$29>>0]|0;
   $31 = $30 << 24 >> 24;
   $32 = ($31|0)!=(0);
   if ($32) {
    $33 = $iptr;
    $34 = HEAP8[$33>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(32);
    if ($36) {
     $91 = 1;
    } else {
     $37 = $iptr;
     $38 = HEAP8[$37>>0]|0;
     $39 = $38 << 24 >> 24;
     $40 = ($39|0)==(9);
     $91 = $40;
    }
    $90 = $91;
   } else {
    $90 = 0;
   }
   if (!($90)) {
    break;
   }
   $41 = $iptr;
   $42 = (($41) + 1|0);
   $iptr = $42;
  }
  $optr = $value;
  while(1) {
   $43 = $iptr;
   $44 = HEAP8[$43>>0]|0;
   $45 = $44 << 24 >> 24;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = $iptr;
    $48 = HEAP8[$47>>0]|0;
    $49 = $48 << 24 >> 24;
    $50 = ($49|0)!=(10);
    $92 = $50;
   } else {
    $92 = 0;
   }
   if (!($92)) {
    break;
   }
   $51 = $iptr;
   $52 = (($51) + 1|0);
   $iptr = $52;
   $53 = HEAP8[$51>>0]|0;
   $54 = $optr;
   $55 = (($54) + 1|0);
   $optr = $55;
   HEAP8[$54>>0] = $53;
  }
  $56 = $optr;
  HEAP8[$56>>0] = 0;
  while(1) {
   $57 = $iptr;
   $58 = HEAP8[$57>>0]|0;
   $59 = $58 << 24 >> 24;
   $60 = ($59|0)!=(0);
   if ($60) {
    $61 = $iptr;
    $62 = HEAP8[$61>>0]|0;
    $63 = $62 << 24 >> 24;
    $64 = ($63|0)==(32);
    if ($64) {
     $94 = 1;
    } else {
     $65 = $iptr;
     $66 = HEAP8[$65>>0]|0;
     $67 = $66 << 24 >> 24;
     $68 = ($67|0)==(9);
     if ($68) {
      $94 = 1;
     } else {
      $69 = $iptr;
      $70 = HEAP8[$69>>0]|0;
      $71 = $70 << 24 >> 24;
      $72 = ($71|0)==(10);
      $94 = $72;
     }
    }
    $93 = $94;
   } else {
    $93 = 0;
   }
   if (!($93)) {
    break;
   }
   $73 = $iptr;
   $74 = (($73) + 1|0);
   $iptr = $74;
  }
  $75 = (_strlen(($name|0))|0);
  $76 = ($75|0)==(0);
  if ($76) {
   label = 32;
   break;
  }
  $78 = (_strlen(($value|0))|0);
  $79 = ($78|0)==(0);
  if ($79) {
   $vptr = 0;
  } else {
   $vptr = $value;
  }
  $80 = $vptr;
  $81 = HEAP32[$fet>>2]|0;
  $82 = (_updatefet_ret($name,$80,$81)|0);
  $ret = $82;
  $83 = ($82|0)!=(0);
  if ($83) {
   label = 37;
   break;
  }
 }
 if ((label|0) == 32) {
  $77 = HEAP32[_stderr>>2]|0;
  (_fprintf(($77|0),(51304|0),($vararg_buffer|0))|0);
  $0 = -2;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 else if ((label|0) == 37) {
  $84 = HEAP32[$fet>>2]|0;
  _freefet($84);
  $85 = $ret;
  $0 = $85;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 else if ((label|0) == 39) {
  $86 = HEAP32[$fet>>2]|0;
  $87 = $1;
  HEAP32[$87>>2] = $86;
  $0 = 0;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 return 0|0;
}
function _updatefet_ret($feature,$value,$fet) {
 $feature = $feature|0;
 $value = $value|0;
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $incr = 0, $increased = 0, $item = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $3 = sp + 36|0;
 $1 = $feature;
 $2 = $value;
 HEAP32[$3>>2] = $fet;
 $item = 0;
 while(1) {
  $4 = $item;
  $5 = HEAP32[$3>>2]|0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if ($8) {
   $9 = $item;
   $10 = HEAP32[$3>>2]|0;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + ($9<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (_strcmp($14,$15)|0);
   $17 = ($16|0)!=(0);
   $125 = $17;
  } else {
   $125 = 0;
  }
  if (!($125)) {
   break;
  }
  $18 = $item;
  $19 = (($18) + 1)|0;
  $item = $19;
 }
 $20 = $item;
 $21 = HEAP32[$3>>2]|0;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)<($23|0);
 if ($24) {
  $25 = $item;
  $26 = HEAP32[$3>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = $item;
   $33 = HEAP32[$3>>2]|0;
   $34 = (($33) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   _free($37);
   $38 = $item;
   $39 = HEAP32[$3>>2]|0;
   $40 = (($39) + 12|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + ($38<<2)|0);
   HEAP32[$42>>2] = 0;
  }
  $43 = $2;
  $44 = ($43|0)!=(0|0);
  do {
   if ($44) {
    $45 = $2;
    $46 = (___strdup($45)|0);
    $47 = $item;
    $48 = HEAP32[$3>>2]|0;
    $49 = (($48) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + ($47<<2)|0);
    HEAP32[$51>>2] = $46;
    $52 = $item;
    $53 = HEAP32[$3>>2]|0;
    $54 = (($53) + 12|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($52<<2)|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)==(0|0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[_stderr>>2]|0;
    (_fprintf(($59|0),(51352|0),($vararg_buffer|0))|0);
    $0 = -2;
    $124 = $0;
    STACKTOP = sp;return ($124|0);
   }
  } while(0);
 } else {
  $60 = HEAP32[$3>>2]|0;
  $61 = (($60) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  $64 = HEAP32[$63>>2]|0;
  $65 = ($62|0)>=($64|0);
  do {
   if ($65) {
    $66 = HEAP32[$3>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67|0) / 10)&-1;
    $incr = $68;
    $69 = HEAP32[$3>>2]|0;
    $70 = HEAP32[$69>>2]|0;
    $71 = $incr;
    $72 = (10)>($71|0);
    if ($72) {
     $75 = 10;
    } else {
     $73 = $incr;
     $75 = $73;
    }
    $74 = (($70) + ($75))|0;
    $increased = $74;
    $76 = $increased;
    $77 = (_reallocfet_ret($3,$76)|0);
    $ret = $77;
    $78 = ($77|0)!=(0);
    if (!($78)) {
     break;
    }
    $79 = $ret;
    $0 = $79;
    $124 = $0;
    STACKTOP = sp;return ($124|0);
   }
  } while(0);
  $80 = $1;
  $81 = (___strdup($80)|0);
  $82 = HEAP32[$3>>2]|0;
  $83 = (($82) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = HEAP32[$3>>2]|0;
  $86 = (($85) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + ($84<<2)|0);
  HEAP32[$88>>2] = $81;
  $89 = HEAP32[$3>>2]|0;
  $90 = (($89) + 4|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = HEAP32[$3>>2]|0;
  $93 = (($92) + 8|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)==(0|0);
  if ($97) {
   $98 = HEAP32[_stderr>>2]|0;
   (_fprintf(($98|0),(51400|0),($vararg_buffer1|0))|0);
   $0 = -3;
   $124 = $0;
   STACKTOP = sp;return ($124|0);
  }
  $99 = $2;
  $100 = ($99|0)!=(0|0);
  do {
   if ($100) {
    $101 = $2;
    $102 = (___strdup($101)|0);
    $103 = HEAP32[$3>>2]|0;
    $104 = (($103) + 4|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = HEAP32[$3>>2]|0;
    $107 = (($106) + 12|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = (($108) + ($105<<2)|0);
    HEAP32[$109>>2] = $102;
    $110 = HEAP32[$3>>2]|0;
    $111 = (($110) + 4|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$3>>2]|0;
    $114 = (($113) + 12|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = (($115) + ($112<<2)|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = ($117|0)==(0|0);
    if (!($118)) {
     break;
    }
    $119 = HEAP32[_stderr>>2]|0;
    (_fprintf(($119|0),(51352|0),($vararg_buffer3|0))|0);
    $0 = -4;
    $124 = $0;
    STACKTOP = sp;return ($124|0);
   }
  } while(0);
  $120 = HEAP32[$3>>2]|0;
  $121 = (($120) + 4|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = (($122) + 1)|0;
  HEAP32[$121>>2] = $123;
 }
 $0 = 0;
 $124 = $0;
 STACKTOP = sp;return ($124|0);
}
function _allocfet_ret($ofet,$numfeatures) {
 $ofet = $ofet|0;
 $numfeatures = $numfeatures|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fet = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofet;
 $2 = $numfeatures;
 $3 = (_malloc(16)|0);
 $fet = $3;
 $4 = $fet;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(51448|0),($vararg_buffer|0))|0);
  $0 = -2;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $7 = $2;
 $8 = (_calloc($7,4)|0);
 $9 = $fet;
 $10 = (($9) + 8|0);
 HEAP32[$10>>2] = $8;
 $11 = $fet;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(51488|0),($vararg_buffer1|0))|0);
  $16 = $fet;
  _free($16);
  $0 = -3;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $17 = $2;
 $18 = (_calloc($17,4)|0);
 $19 = $fet;
 $20 = (($19) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = $fet;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)==(0|0);
 if ($24) {
  $25 = HEAP32[_stderr>>2]|0;
  (_fprintf(($25|0),(51536|0),($vararg_buffer3|0))|0);
  $26 = $fet;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  _free($28);
  $29 = $fet;
  _free($29);
  $0 = -4;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 } else {
  $30 = $2;
  $31 = $fet;
  HEAP32[$31>>2] = $30;
  $32 = $fet;
  $33 = (($32) + 4|0);
  HEAP32[$33>>2] = 0;
  $34 = $fet;
  $35 = $1;
  HEAP32[$35>>2] = $34;
  $0 = 0;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _reallocfet_ret($ofet,$newlen) {
 $ofet = $ofet|0;
 $newlen = $newlen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fet = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofet;
 $2 = $newlen;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $fet = $4;
 $5 = $fet;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $fet;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0);
  if (!($9)) {
   $15 = $fet;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $2;
   $19 = $18<<2;
   $20 = (_realloc($17,$19)|0);
   $21 = $fet;
   $22 = (($21) + 8|0);
   HEAP32[$22>>2] = $20;
   $23 = $fet;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)==(0|0);
   if ($26) {
    $27 = HEAP32[_stderr>>2]|0;
    (_fprintf(($27|0),(51584|0),($vararg_buffer|0))|0);
    $0 = -2;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   }
   $28 = $fet;
   $29 = (($28) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $2;
   $32 = $31<<2;
   $33 = (_realloc($30,$32)|0);
   $34 = $fet;
   $35 = (($34) + 12|0);
   HEAP32[$35>>2] = $33;
   $36 = $fet;
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($38|0)==(0|0);
   if ($39) {
    $40 = HEAP32[_stderr>>2]|0;
    (_fprintf(($40|0),(51632|0),($vararg_buffer1|0))|0);
    $0 = -3;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   } else {
    $41 = $2;
    $42 = $fet;
    HEAP32[$42>>2] = $41;
    $0 = 0;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   }
  }
 }
 $10 = $1;
 $11 = $2;
 $12 = (_allocfet_ret($10,$11)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $ret;
  $0 = $14;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 } else {
  $0 = 0;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 }
 return 0|0;
}
function _getc_byte($ochar_dat,$cbufptr,$ebufptr) {
 $ochar_dat = $ochar_dat|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $ochar_dat;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = $2;
 $5 = HEAP32[$4>>2]|0;
 $6 = $3;
 $7 = ($5>>>0)>=($6>>>0);
 if ($7) {
  $8 = HEAP32[_stderr>>2]|0;
  (_fprintf(($8|0),(51680|0),($vararg_buffer|0))|0);
  $0 = -39;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 } else {
  $9 = $2;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP8[$10>>0]|0;
  $12 = $1;
  HEAP8[$12>>0] = $11;
  $13 = $2;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 1|0);
  HEAP32[$13>>2] = $15;
  $0 = 0;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _getc_bytes($ochar_dat,$ilen,$cbufptr,$ebufptr) {
 $ochar_dat = $ochar_dat|0;
 $ilen = $ilen|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $ochar_dat;
 $2 = $ilen;
 $3 = $cbufptr;
 $4 = $ebufptr;
 $5 = $3;
 $6 = HEAP32[$5>>2]|0;
 $7 = $4;
 $8 = $2;
 $9 = (($8) - 1)|0;
 $10 = (0 - ($9))|0;
 $11 = (($7) + ($10)|0);
 $12 = ($6>>>0)>=($11>>>0);
 if ($12) {
  $13 = HEAP32[_stderr>>2]|0;
  (_fprintf(($13|0),(51728|0),($vararg_buffer|0))|0);
  $0 = -40;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 } else {
  $14 = $1;
  $15 = HEAP32[$14>>2]|0;
  $16 = $3;
  $17 = HEAP32[$16>>2]|0;
  $18 = $2;
  _memcpy(($15|0),($17|0),($18|0))|0;
  $19 = $2;
  $20 = $3;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($19)|0);
  HEAP32[$20>>2] = $22;
  $0 = 0;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _getc_ushort($oshrt_dat,$cbufptr,$ebufptr) {
 $oshrt_dat = $oshrt_dat|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $_b_ = 0, $_f_ = 0, $_t_ = 0, $cptr = 0, $ret = 0, $shrt_dat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $shrt_dat = sp + 34|0;
 $cptr = sp + 8|0;
 $_b_ = sp + 32|0;
 $1 = $oshrt_dat;
 $2 = $cbufptr;
 $3 = $ebufptr;
 HEAP32[$cptr>>2] = $shrt_dat;
 $4 = $2;
 $5 = $3;
 $6 = (_getc_bytes($cptr,2,$4,$5)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 } else {
  $9 = HEAP16[$shrt_dat>>1]|0;
  HEAP16[$_b_>>1] = $9;
  $_f_ = $_b_;
  $_t_ = $shrt_dat;
  $10 = $_f_;
  $11 = HEAP8[$10>>0]|0;
  $12 = $_t_;
  $13 = (($12) + 1|0);
  HEAP8[$13>>0] = $11;
  $14 = $_f_;
  $15 = (($14) + 1|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $_t_;
  HEAP8[$17>>0] = $16;
  $18 = HEAP16[$shrt_dat>>1]|0;
  $19 = $1;
  HEAP16[$19>>1] = $18;
  $0 = 0;
  $20 = $0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _getc_uint($oint_dat,$cbufptr,$ebufptr) {
 $oint_dat = $oint_dat|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $_b_ = 0, $_f_ = 0, $_t_ = 0, $cptr = 0, $int_dat = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $int_dat = sp + 16|0;
 $cptr = sp + 12|0;
 $_b_ = sp + 8|0;
 $1 = $oint_dat;
 $2 = $cbufptr;
 $3 = $ebufptr;
 HEAP32[$cptr>>2] = $int_dat;
 $4 = $2;
 $5 = $3;
 $6 = (_getc_bytes($cptr,4,$4,$5)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $30 = $0;
  STACKTOP = sp;return ($30|0);
 } else {
  $9 = HEAP32[$int_dat>>2]|0;
  HEAP32[$_b_>>2] = $9;
  $_f_ = $_b_;
  $_t_ = $int_dat;
  $10 = $_f_;
  $11 = HEAP8[$10>>0]|0;
  $12 = $_t_;
  $13 = (($12) + 3|0);
  HEAP8[$13>>0] = $11;
  $14 = $_f_;
  $15 = (($14) + 1|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $_t_;
  $18 = (($17) + 2|0);
  HEAP8[$18>>0] = $16;
  $19 = $_f_;
  $20 = (($19) + 2|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $_t_;
  $23 = (($22) + 1|0);
  HEAP8[$23>>0] = $21;
  $24 = $_f_;
  $25 = (($24) + 3|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $_t_;
  HEAP8[$27>>0] = $26;
  $28 = HEAP32[$int_dat>>2]|0;
  $29 = $1;
  HEAP32[$29>>2] = $28;
  $0 = 0;
  $30 = $0;
  STACKTOP = sp;return ($30|0);
 }
 return 0|0;
}
function _filesize($ifile) {
 $ifile = $ifile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $fp = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ifile;
 $2 = $1;
 $3 = (_fopen(($2|0),(51776|0))|0);
 $fp = $3;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  $6 = $1;
  HEAP32[$vararg_buffer>>2] = $6;
  (_fprintf(($5|0),(51784|0),($vararg_buffer|0))|0);
  $0 = -2;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 $7 = $fp;
 $8 = (_fseek(($7|0),0,2)|0);
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $1;
  HEAP32[$vararg_buffer1>>2] = $11;
  (_fprintf(($10|0),(51832|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 $12 = $fp;
 $13 = (_ftell(($12|0))|0);
 $ret = $13;
 $14 = ($13|0)<(0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = $1;
  HEAP32[$vararg_buffer4>>2] = $16;
  (_fprintf(($15|0),(51888|0),($vararg_buffer4|0))|0);
  $0 = -4;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 } else {
  $17 = $fp;
  (_fclose(($17|0))|0);
  $18 = $ret;
  $0 = $18;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _getc_skip_marker_segment($marker,$cbufptr,$ebufptr) {
 $marker = $marker|0;
 $cbufptr = $cbufptr|0;
 $ebufptr = $ebufptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $length = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr5 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $length = sp + 40|0;
 $1 = $marker;
 $2 = $cbufptr;
 $3 = $ebufptr;
 $4 = $2;
 $5 = $3;
 $6 = (_getc_ushort($length,$4,$5)|0);
 $ret = $6;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $ret;
  $0 = $8;
  $32 = $0;
  STACKTOP = sp;return ($32|0);
 }
 $9 = HEAP16[$length>>1]|0;
 $10 = $9&65535;
 $11 = (($10) - 2)|0;
 $12 = $11&65535;
 HEAP16[$length>>1] = $12;
 $13 = $2;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP16[$length>>1]|0;
 $16 = $15&65535;
 $17 = (($14) + ($16)|0);
 $18 = $3;
 $19 = ($17>>>0)>=($18>>>0);
 if ($19) {
  $20 = HEAP32[_stderr>>2]|0;
  (_fprintf(($20|0),(51976|0),($vararg_buffer|0))|0);
  $21 = HEAP32[_stderr>>2]|0;
  (_fprintf(($21|0),(52016|0),($vararg_buffer1|0))|0);
  $22 = HEAP32[_stderr>>2]|0;
  $23 = $1;
  $24 = $23&65535;
  $25 = HEAP16[$length>>1]|0;
  $26 = $25&65535;
  HEAP32[$vararg_buffer3>>2] = $24;
  $vararg_ptr5 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $26;
  (_fprintf(($22|0),(51944|0),($vararg_buffer3|0))|0);
  $0 = -2;
  $32 = $0;
  STACKTOP = sp;return ($32|0);
 } else {
  $27 = HEAP16[$length>>1]|0;
  $28 = $27&65535;
  $29 = $2;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($28)|0);
  HEAP32[$29>>2] = $31;
  $0 = 0;
  $32 = $0;
  STACKTOP = sp;return ($32|0);
 }
 return 0|0;
}
function _fatalerr($s1,$s2,$s3) {
 $s1 = $s1|0;
 $s2 = $s2|0;
 $s3 = $s3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer5 = 0, $vararg_ptr4 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $s1;
 $1 = $s2;
 $2 = $s3;
 $3 = HEAP32[_stdout>>2]|0;
 (_fflush(($3|0))|0);
 $4 = $1;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  $7 = $0;
  HEAP32[$vararg_buffer>>2] = $7;
  (_fprintf(($6|0),(52056|0),($vararg_buffer|0))|0);
  $17 = HEAP32[_stderr>>2]|0;
  (_fflush(($17|0))|0);
  _exit(1);
  // unreachable;
 }
 $8 = $2;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $0;
  $12 = $1;
  HEAP32[$vararg_buffer1>>2] = $11;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $12;
  (_fprintf(($10|0),(52072|0),($vararg_buffer1|0))|0);
 } else {
  $13 = HEAP32[_stderr>>2]|0;
  $14 = $0;
  $15 = $1;
  $16 = $2;
  HEAP32[$vararg_buffer5>>2] = $14;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $15;
  $vararg_ptr9 = (($vararg_buffer5) + 8|0);
  HEAP32[$vararg_ptr9>>2] = $16;
  (_fprintf(($13|0),(52088|0),($vararg_buffer5|0))|0);
 }
 $17 = HEAP32[_stderr>>2]|0;
 (_fflush(($17|0))|0);
 _exit(1);
 // unreachable;
}
function _inv_bytes($data,$nbytes) {
 $data = $data|0;
 $nbytes = $nbytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $data;
 $1 = $nbytes;
 while(1) {
  $2 = $1;
  $3 = ($2|0)!=(0);
  if (!($3)) {
   break;
  }
  $4 = $0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = (52112 + ($6)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $0;
  HEAP8[$9>>0] = $8;
  $10 = $1;
  $11 = (($10) + -1)|0;
  $1 = $11;
  $12 = $0;
  $13 = (($12) + 1|0);
  $0 = $13;
 }
 STACKTOP = sp;return;
}
function _bio_create() {
 var $0 = 0, $1 = 0, $bio = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = (_malloc(20)|0);
 $bio = $0;
 $1 = $bio;
 STACKTOP = sp;return ($1|0);
}
function _bio_destroy($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $bio;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = $0;
  _free($3);
 }
 STACKTOP = sp;return;
}
function _bio_numbytes($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $bio;
 $1 = $0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $3;
 $7 = $5;
 $8 = (($6) - ($7))|0;
 STACKTOP = sp;return ($8|0);
}
function _bio_init_dec($bio,$bp,$len) {
 $bio = $bio|0;
 $bp = $bp|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $bio;
 $1 = $bp;
 $2 = $len;
 $3 = $1;
 $4 = $0;
 HEAP32[$4>>2] = $3;
 $5 = $1;
 $6 = $2;
 $7 = (($5) + ($6)|0);
 $8 = $0;
 $9 = (($8) + 4|0);
 HEAP32[$9>>2] = $7;
 $10 = $1;
 $11 = $0;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = $10;
 $13 = $0;
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = 0;
 $15 = $0;
 $16 = (($15) + 16|0);
 HEAP32[$16>>2] = 0;
 STACKTOP = sp;return;
}
function _bio_read($bio,$n) {
 $bio = $bio|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $bio;
 $1 = $n;
 $v = 0;
 $2 = $1;
 $3 = (($2) - 1)|0;
 $i = $3;
 while(1) {
  $4 = $i;
  $5 = ($4|0)>=(0);
  if (!($5)) {
   break;
  }
  $6 = $0;
  $7 = (_bio_getbit($6)|0);
  $8 = $i;
  $9 = $7 << $8;
  $10 = $v;
  $11 = (($10) + ($9))|0;
  $v = $11;
  $12 = $i;
  $13 = (($12) + -1)|0;
  $i = $13;
 }
 $14 = $v;
 STACKTOP = sp;return ($14|0);
}
function _bio_getbit($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $bio;
 $1 = $0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = $0;
  (_bio_bytein($5)|0);
 }
 $6 = $0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = $0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $0;
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $12 >>> $15;
 $17 = $16 & 1;
 STACKTOP = sp;return ($17|0);
}
function _bio_inalign($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $bio;
 $2 = $1;
 $3 = (($2) + 16|0);
 HEAP32[$3>>2] = 0;
 $4 = $1;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 255;
 $8 = ($7|0)==(255);
 do {
  if ($8) {
   $9 = $1;
   $10 = (_bio_bytein($9)|0);
   $11 = ($10|0)!=(0);
   if (!($11)) {
    $12 = $1;
    $13 = (($12) + 16|0);
    HEAP32[$13>>2] = 0;
    break;
   }
   $0 = 1;
   $14 = $0;
   STACKTOP = sp;return ($14|0);
  }
 } while(0);
 $0 = 0;
 $14 = $0;
 STACKTOP = sp;return ($14|0);
}
function _bio_bytein($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $bio;
 $2 = $1;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 << 8;
 $6 = $5 & 65535;
 $7 = $1;
 $8 = (($7) + 12|0);
 HEAP32[$8>>2] = $6;
 $9 = $1;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(65280);
 $13 = $12 ? 7 : 8;
 $14 = $1;
 $15 = (($14) + 16|0);
 HEAP32[$15>>2] = $13;
 $16 = $1;
 $17 = (($16) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $1;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($18>>>0)>=($21>>>0);
 if ($22) {
  $0 = 1;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 } else {
  $23 = $1;
  $24 = (($23) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 1|0);
  HEAP32[$24>>2] = $26;
  $27 = HEAP8[$25>>0]|0;
  $28 = $27&255;
  $29 = $1;
  $30 = (($29) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $31 | $28;
  HEAP32[$30>>2] = $32;
  $0 = 0;
  $33 = $0;
  STACKTOP = sp;return ($33|0);
 }
 return 0|0;
}
function _opj_cio_open($cinfo,$buffer,$length) {
 $cinfo = $cinfo|0;
 $buffer = $buffer|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $cio = 0, $cp = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $cinfo;
 $2 = $buffer;
 $3 = $length;
 $cp = 0;
 $4 = (_malloc(28)|0);
 $cio = $4;
 $5 = $cio;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $0 = 0;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 $7 = $1;
 $8 = $cio;
 HEAP32[$8>>2] = $7;
 $9 = $2;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = $3;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = $cio;
   $14 = (($13) + 4|0);
   HEAP32[$14>>2] = 1;
   $15 = $2;
   $16 = $cio;
   $17 = (($16) + 8|0);
   HEAP32[$17>>2] = $15;
   $18 = $3;
   $19 = $cio;
   $20 = (($19) + 12|0);
   HEAP32[$20>>2] = $18;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 do {
  if ((label|0) == 6) {
   $21 = $2;
   $22 = ($21|0)!=(0|0);
   if (!($22)) {
    $23 = $3;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     $25 = $1;
     $26 = ($25|0)!=(0|0);
     if ($26) {
      $27 = $cio;
      $28 = (($27) + 4|0);
      HEAP32[$28>>2] = 2;
      $29 = $1;
      $30 = (($29) + 12|0);
      $31 = HEAP32[$30>>2]|0;
      if ((($31|0) == 0)) {
       $32 = $1;
       $33 = (($32) + 16|0);
       $34 = HEAP32[$33>>2]|0;
       $35 = (($34) + 64|0);
       $36 = HEAP32[$35>>2]|0;
       $cp = $36;
      } else if ((($31|0) == 2)) {
       $37 = $1;
       $38 = (($37) + 20|0);
       $39 = HEAP32[$38>>2]|0;
       $40 = (($39) + 4|0);
       $41 = HEAP32[$40>>2]|0;
       $42 = (($41) + 64|0);
       $43 = HEAP32[$42>>2]|0;
       $cp = $43;
      } else {
       $44 = $cio;
       _free($44);
       $0 = 0;
       $88 = $0;
       STACKTOP = sp;return ($88|0);
      }
      $45 = $cp;
      $46 = (($45) + 8|0);
      $47 = HEAP32[$46>>2]|0;
      $48 = (+($47|0));
      $49 = 0.162500000000000005551 * $48;
      $50 = $49 + 2000.0;
      $51 = (~~(($50))>>>0);
      $52 = $cio;
      $53 = (($52) + 12|0);
      HEAP32[$53>>2] = $51;
      $54 = $cio;
      $55 = (($54) + 12|0);
      $56 = HEAP32[$55>>2]|0;
      $57 = (_malloc($56)|0);
      $58 = $cio;
      $59 = (($58) + 8|0);
      HEAP32[$59>>2] = $57;
      $60 = $cio;
      $61 = (($60) + 8|0);
      $62 = HEAP32[$61>>2]|0;
      $63 = ($62|0)!=(0|0);
      if ($63) {
       break;
      } else {
       $64 = $cio;
       $65 = HEAP32[$64>>2]|0;
       (_opj_event_msg($65,1,52368,$vararg_buffer)|0);
       $66 = $cio;
       _free($66);
       $0 = 0;
       $88 = $0;
       STACKTOP = sp;return ($88|0);
      }
     }
    }
   }
   $67 = $cio;
   _free($67);
   $0 = 0;
   $88 = $0;
   STACKTOP = sp;return ($88|0);
  }
 } while(0);
 $68 = $cio;
 $69 = (($68) + 8|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $cio;
 $72 = (($71) + 16|0);
 HEAP32[$72>>2] = $70;
 $73 = $cio;
 $74 = (($73) + 8|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = $cio;
 $77 = (($76) + 12|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = (($75) + ($78)|0);
 $80 = $cio;
 $81 = (($80) + 20|0);
 HEAP32[$81>>2] = $79;
 $82 = $cio;
 $83 = (($82) + 8|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = $cio;
 $86 = (($85) + 24|0);
 HEAP32[$86>>2] = $84;
 $87 = $cio;
 $0 = $87;
 $88 = $0;
 STACKTOP = sp;return ($88|0);
}
function _opj_cio_close($cio) {
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(2);
 if ($6) {
  $7 = $0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  _free($9);
 }
 $10 = $0;
 _free($10);
 STACKTOP = sp;return;
}
function _cio_tell($cio) {
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $3;
 $8 = $6;
 $9 = (($7) - ($8))|0;
 STACKTOP = sp;return ($9|0);
}
function _cio_seek($cio,$pos) {
 $cio = $cio|0;
 $pos = $pos|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $pos;
 $2 = $0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $1;
 $6 = (($4) + ($5)|0);
 $7 = $0;
 $8 = (($7) + 24|0);
 HEAP32[$8>>2] = $6;
 STACKTOP = sp;return;
}
function _cio_numbytesleft($cio) {
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $3;
 $8 = $6;
 $9 = (($7) - ($8))|0;
 STACKTOP = sp;return ($9|0);
}
function _cio_getbp($cio) {
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _cio_bytein($cio) {
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $cio;
 $2 = $1;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $1;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($4>>>0)>=($7>>>0);
 if ($8) {
  $9 = $1;
  $10 = HEAP32[$9>>2]|0;
  $11 = $1;
  $12 = (($11) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $1;
  $15 = (($14) + 24|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $1;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $16;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $19;
  (_opj_event_msg($10,1,52424,$vararg_buffer)|0);
  $0 = 0;
  $25 = $0;
  STACKTOP = sp;return ($25|0);
 } else {
  $20 = $1;
  $21 = (($20) + 24|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 1|0);
  HEAP32[$21>>2] = $23;
  $24 = HEAP8[$22>>0]|0;
  $0 = $24;
  $25 = $0;
  STACKTOP = sp;return ($25|0);
 }
 return 0|0;
}
function _cio_read($cio,$n) {
 $cio = $cio|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $n;
 $v = 0;
 $2 = $1;
 $3 = (($2) - 1)|0;
 $i = $3;
 while(1) {
  $4 = $i;
  $5 = ($4|0)>=(0);
  if (!($5)) {
   break;
  }
  $6 = $0;
  $7 = (_cio_bytein($6)|0);
  $8 = $7&255;
  $9 = $i;
  $10 = $9 << 3;
  $11 = $8 << $10;
  $12 = $v;
  $13 = (($12) + ($11))|0;
  $v = $13;
  $14 = $i;
  $15 = (($14) + -1)|0;
  $i = $15;
 }
 $16 = $v;
 STACKTOP = sp;return ($16|0);
}
function _cio_skip($cio,$n) {
 $cio = $cio|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $n;
 $2 = $1;
 $3 = $0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2)|0);
 HEAP32[$4>>2] = $6;
 STACKTOP = sp;return;
}
function _dwt_decode($tilec,$numres) {
 $tilec = $tilec|0;
 $numres = $numres|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $tilec;
 $1 = $numres;
 $2 = $0;
 $3 = $1;
 _dwt_decode_tile($2,$3,131);
 STACKTOP = sp;return;
}
function _dwt_decode_tile($tilec,$numres,$dwt_1D) {
 $tilec = $tilec|0;
 $numres = $numres|0;
 $dwt_1D = $dwt_1D|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $h = 0, $j = 0, $k = 0, $rh = 0, $rw = 0, $tiledp = 0, $tr = 0, $v = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $h = sp + 48|0;
 $v = sp + 32|0;
 $0 = $tilec;
 $1 = $numres;
 $2 = $dwt_1D;
 $3 = $0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $tr = $5;
 $6 = $tr;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $tr;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) - ($10))|0;
 $rw = $11;
 $12 = $tr;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $tr;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($14) - ($17))|0;
 $rh = $18;
 $19 = $0;
 $20 = (($19) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) - ($23))|0;
 $w = $24;
 $25 = $tr;
 $26 = $1;
 $27 = (_dwt_decode_max_resolution($25,$26)|0);
 $28 = $27<<2;
 $29 = (_memalign(16,$28)|0);
 HEAP32[$h>>2] = $29;
 $30 = HEAP32[$h>>2]|0;
 HEAP32[$v>>2] = $30;
 while(1) {
  $31 = $1;
  $32 = (($31) + -1)|0;
  $1 = $32;
  $33 = ($32|0)!=(0);
  if (!($33)) {
   break;
  }
  $34 = $0;
  $35 = (($34) + 24|0);
  $36 = HEAP32[$35>>2]|0;
  $tiledp = $36;
  $37 = $tr;
  $38 = (($37) + 124|0);
  $tr = $38;
  $39 = $rw;
  $40 = (($h) + 8|0);
  HEAP32[$40>>2] = $39;
  $41 = $rh;
  $42 = (($v) + 8|0);
  HEAP32[$42>>2] = $41;
  $43 = $tr;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $tr;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($45) - ($47))|0;
  $rw = $48;
  $49 = $tr;
  $50 = (($49) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = $tr;
  $53 = (($52) + 4|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($51) - ($54))|0;
  $rh = $55;
  $56 = $rw;
  $57 = (($h) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($56) - ($58))|0;
  $60 = (($h) + 4|0);
  HEAP32[$60>>2] = $59;
  $61 = $tr;
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62|0) % 2)&-1;
  $64 = (($h) + 12|0);
  HEAP32[$64>>2] = $63;
  $j = 0;
  while(1) {
   $65 = $j;
   $66 = $rh;
   $67 = ($65|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = $j;
   $69 = $w;
   $70 = Math_imul($68, $69)|0;
   $71 = $tiledp;
   $72 = (($71) + ($70<<2)|0);
   _dwt_interleave_h($h,$72);
   $73 = $2;
   FUNCTION_TABLE_vi[$73 & 255]($h);
   $74 = $j;
   $75 = $w;
   $76 = Math_imul($74, $75)|0;
   $77 = $tiledp;
   $78 = (($77) + ($76<<2)|0);
   $79 = HEAP32[$h>>2]|0;
   $80 = $rw;
   $81 = $80<<2;
   _memcpy(($78|0),($79|0),($81|0))|0;
   $82 = $j;
   $83 = (($82) + 1)|0;
   $j = $83;
  }
  $84 = $rh;
  $85 = (($v) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($84) - ($86))|0;
  $88 = (($v) + 4|0);
  HEAP32[$88>>2] = $87;
  $89 = $tr;
  $90 = (($89) + 4|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91|0) % 2)&-1;
  $93 = (($v) + 12|0);
  HEAP32[$93>>2] = $92;
  $j = 0;
  while(1) {
   $94 = $j;
   $95 = $rw;
   $96 = ($94|0)<($95|0);
   if (!($96)) {
    break;
   }
   $97 = $j;
   $98 = $tiledp;
   $99 = (($98) + ($97<<2)|0);
   $100 = $w;
   _dwt_interleave_v($v,$99,$100);
   $101 = $2;
   FUNCTION_TABLE_vi[$101 & 255]($v);
   $k = 0;
   while(1) {
    $102 = $k;
    $103 = $rh;
    $104 = ($102|0)<($103|0);
    if (!($104)) {
     break;
    }
    $105 = $k;
    $106 = HEAP32[$v>>2]|0;
    $107 = (($106) + ($105<<2)|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = $k;
    $110 = $w;
    $111 = Math_imul($109, $110)|0;
    $112 = $j;
    $113 = (($111) + ($112))|0;
    $114 = $tiledp;
    $115 = (($114) + ($113<<2)|0);
    HEAP32[$115>>2] = $108;
    $116 = $k;
    $117 = (($116) + 1)|0;
    $k = $117;
   }
   $118 = $j;
   $119 = (($118) + 1)|0;
   $j = $119;
  }
 }
 $120 = HEAP32[$h>>2]|0;
 _free($120);
 STACKTOP = sp;return;
}
function _dwt_decode_1($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $v;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 _dwt_decode_1_($2,$5,$8,$11);
 STACKTOP = sp;return;
}
function _dwt_getgain($orient) {
 $orient = $orient|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $orient;
 $2 = $1;
 $3 = ($2|0)==(0);
 do {
  if ($3) {
   $0 = 0;
  } else {
   $4 = $1;
   $5 = ($4|0)==(1);
   if (!($5)) {
    $6 = $1;
    $7 = ($6|0)==(2);
    if (!($7)) {
     $0 = 2;
     break;
    }
   }
   $0 = 1;
  }
 } while(0);
 $8 = $0;
 STACKTOP = sp;return ($8|0);
}
function _dwt_getgain_real($orient) {
 $orient = $orient|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $orient;
 STACKTOP = sp;return 0;
}
function _dwt_decode_real($tilec,$numres) {
 $tilec = $tilec|0;
 $numres = $numres|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0.0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aj = 0, $bufsize = 0, $h = 0;
 var $j = 0, $k = 0, $k1 = 0, $k2 = 0, $k3 = 0, $res = 0, $rh = 0, $rw = 0, $v = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $h = sp + 64|0;
 $v = sp + 48|0;
 $0 = $tilec;
 $1 = $numres;
 $2 = $0;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $res = $4;
 $5 = $res;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $res;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) - ($9))|0;
 $rw = $10;
 $11 = $res;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $res;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($13) - ($16))|0;
 $rh = $17;
 $18 = $0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $0;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) - ($22))|0;
 $w = $23;
 $24 = $res;
 $25 = $1;
 $26 = (_dwt_decode_max_resolution($24,$25)|0);
 $27 = (($26) + 5)|0;
 $28 = $27<<4;
 $29 = (_memalign(16,$28)|0);
 HEAP32[$h>>2] = $29;
 $30 = HEAP32[$h>>2]|0;
 HEAP32[$v>>2] = $30;
 while(1) {
  $31 = $1;
  $32 = (($31) + -1)|0;
  $1 = $32;
  $33 = ($32|0)!=(0);
  if (!($33)) {
   break;
  }
  $34 = $0;
  $35 = (($34) + 24|0);
  $36 = HEAP32[$35>>2]|0;
  $aj = $36;
  $37 = $0;
  $38 = (($37) + 8|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = $0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($39) - ($41))|0;
  $43 = $0;
  $44 = (($43) + 12|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $0;
  $47 = (($46) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($45) - ($48))|0;
  $50 = Math_imul($42, $49)|0;
  $bufsize = $50;
  $51 = $rw;
  $52 = (($h) + 8|0);
  HEAP32[$52>>2] = $51;
  $53 = $rh;
  $54 = (($v) + 8|0);
  HEAP32[$54>>2] = $53;
  $55 = $res;
  $56 = (($55) + 124|0);
  $res = $56;
  $57 = $res;
  $58 = (($57) + 8|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $res;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) - ($61))|0;
  $rw = $62;
  $63 = $res;
  $64 = (($63) + 12|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = $res;
  $67 = (($66) + 4|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($65) - ($68))|0;
  $rh = $69;
  $70 = $rw;
  $71 = (($h) + 8|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) - ($72))|0;
  $74 = (($h) + 4|0);
  HEAP32[$74>>2] = $73;
  $75 = $res;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76|0) % 2)&-1;
  $78 = (($h) + 12|0);
  HEAP32[$78>>2] = $77;
  $79 = $rh;
  $j = $79;
  while(1) {
   $80 = $j;
   $81 = ($80|0)>(3);
   if (!($81)) {
    break;
   }
   $82 = $aj;
   $83 = $w;
   $84 = $bufsize;
   _v4dwt_interleave_h($h,$82,$83,$84);
   _v4dwt_decode($h);
   $85 = $rw;
   $k = $85;
   while(1) {
    $86 = $k;
    $87 = (($86) + -1)|0;
    $k = $87;
    $88 = ($87|0)>=(0);
    if (!($88)) {
     break;
    }
    $89 = $k;
    $90 = HEAP32[$h>>2]|0;
    $91 = (($90) + ($89<<4)|0);
    $92 = +HEAPF32[$91>>2];
    $93 = $k;
    $94 = $aj;
    $95 = (($94) + ($93<<2)|0);
    HEAPF32[$95>>2] = $92;
    $96 = $k;
    $97 = HEAP32[$h>>2]|0;
    $98 = (($97) + ($96<<4)|0);
    $99 = (($98) + 4|0);
    $100 = +HEAPF32[$99>>2];
    $101 = $k;
    $102 = $w;
    $103 = (($101) + ($102))|0;
    $104 = $aj;
    $105 = (($104) + ($103<<2)|0);
    HEAPF32[$105>>2] = $100;
    $106 = $k;
    $107 = HEAP32[$h>>2]|0;
    $108 = (($107) + ($106<<4)|0);
    $109 = (($108) + 8|0);
    $110 = +HEAPF32[$109>>2];
    $111 = $k;
    $112 = $w;
    $113 = $112<<1;
    $114 = (($111) + ($113))|0;
    $115 = $aj;
    $116 = (($115) + ($114<<2)|0);
    HEAPF32[$116>>2] = $110;
    $117 = $k;
    $118 = HEAP32[$h>>2]|0;
    $119 = (($118) + ($117<<4)|0);
    $120 = (($119) + 12|0);
    $121 = +HEAPF32[$120>>2];
    $122 = $k;
    $123 = $w;
    $124 = ($123*3)|0;
    $125 = (($122) + ($124))|0;
    $126 = $aj;
    $127 = (($126) + ($125<<2)|0);
    HEAPF32[$127>>2] = $121;
   }
   $128 = $w;
   $129 = $128<<2;
   $130 = $aj;
   $131 = (($130) + ($129<<2)|0);
   $aj = $131;
   $132 = $w;
   $133 = $132<<2;
   $134 = $bufsize;
   $135 = (($134) - ($133))|0;
   $bufsize = $135;
   $136 = $j;
   $137 = (($136) - 4)|0;
   $j = $137;
  }
  $138 = $rh;
  $139 = $138 & 3;
  $140 = ($139|0)!=(0);
  if ($140) {
   $141 = $rh;
   $142 = $141 & 3;
   $j = $142;
   $143 = $aj;
   $144 = $w;
   $145 = $bufsize;
   _v4dwt_interleave_h($h,$143,$144,$145);
   _v4dwt_decode($h);
   $146 = $rw;
   $k1 = $146;
   while(1) {
    $147 = $k1;
    $148 = (($147) + -1)|0;
    $k1 = $148;
    $149 = ($148|0)>=(0);
    if (!($149)) {
     break;
    }
    $150 = $j;
    if ((($150|0) == 3)) {
     $151 = $k1;
     $152 = HEAP32[$h>>2]|0;
     $153 = (($152) + ($151<<4)|0);
     $154 = (($153) + 8|0);
     $155 = +HEAPF32[$154>>2];
     $156 = $k1;
     $157 = $w;
     $158 = $157<<1;
     $159 = (($156) + ($158))|0;
     $160 = $aj;
     $161 = (($160) + ($159<<2)|0);
     HEAPF32[$161>>2] = $155;
     label = 15;
    } else if ((($150|0) == 2)) {
     label = 15;
    } else if ((($150|0) == 1)) {
     label = 16;
    }
    if ((label|0) == 15) {
     label = 0;
     $162 = $k1;
     $163 = HEAP32[$h>>2]|0;
     $164 = (($163) + ($162<<4)|0);
     $165 = (($164) + 4|0);
     $166 = +HEAPF32[$165>>2];
     $167 = $k1;
     $168 = $w;
     $169 = (($167) + ($168))|0;
     $170 = $aj;
     $171 = (($170) + ($169<<2)|0);
     HEAPF32[$171>>2] = $166;
     label = 16;
    }
    if ((label|0) == 16) {
     label = 0;
     $172 = $k1;
     $173 = HEAP32[$h>>2]|0;
     $174 = (($173) + ($172<<4)|0);
     $175 = +HEAPF32[$174>>2];
     $176 = $k1;
     $177 = $aj;
     $178 = (($177) + ($176<<2)|0);
     HEAPF32[$178>>2] = $175;
    }
   }
  }
  $179 = $rh;
  $180 = (($v) + 8|0);
  $181 = HEAP32[$180>>2]|0;
  $182 = (($179) - ($181))|0;
  $183 = (($v) + 4|0);
  HEAP32[$183>>2] = $182;
  $184 = $res;
  $185 = (($184) + 4|0);
  $186 = HEAP32[$185>>2]|0;
  $187 = (($186|0) % 2)&-1;
  $188 = (($v) + 12|0);
  HEAP32[$188>>2] = $187;
  $189 = $0;
  $190 = (($189) + 24|0);
  $191 = HEAP32[$190>>2]|0;
  $aj = $191;
  $192 = $rw;
  $j = $192;
  while(1) {
   $193 = $j;
   $194 = ($193|0)>(3);
   if (!($194)) {
    break;
   }
   $195 = $aj;
   $196 = $w;
   _v4dwt_interleave_v($v,$195,$196);
   _v4dwt_decode($v);
   $k2 = 0;
   while(1) {
    $197 = $k2;
    $198 = $rh;
    $199 = ($197|0)<($198|0);
    if (!($199)) {
     break;
    }
    $200 = $k2;
    $201 = $w;
    $202 = Math_imul($200, $201)|0;
    $203 = $aj;
    $204 = (($203) + ($202<<2)|0);
    $205 = $k2;
    $206 = HEAP32[$v>>2]|0;
    $207 = (($206) + ($205<<4)|0);
    ;HEAP32[$204+0>>2]=HEAP32[$207+0>>2]|0;HEAP32[$204+4>>2]=HEAP32[$207+4>>2]|0;HEAP32[$204+8>>2]=HEAP32[$207+8>>2]|0;HEAP32[$204+12>>2]=HEAP32[$207+12>>2]|0;
    $208 = $k2;
    $209 = (($208) + 1)|0;
    $k2 = $209;
   }
   $210 = $aj;
   $211 = (($210) + 16|0);
   $aj = $211;
   $212 = $j;
   $213 = (($212) - 4)|0;
   $j = $213;
  }
  $214 = $rw;
  $215 = $214 & 3;
  $216 = ($215|0)!=(0);
  if ($216) {
   $217 = $rw;
   $218 = $217 & 3;
   $j = $218;
   $219 = $aj;
   $220 = $w;
   _v4dwt_interleave_v($v,$219,$220);
   _v4dwt_decode($v);
   $k3 = 0;
   while(1) {
    $221 = $k3;
    $222 = $rh;
    $223 = ($221|0)<($222|0);
    if (!($223)) {
     break;
    }
    $224 = $k3;
    $225 = $w;
    $226 = Math_imul($224, $225)|0;
    $227 = $aj;
    $228 = (($227) + ($226<<2)|0);
    $229 = $k3;
    $230 = HEAP32[$v>>2]|0;
    $231 = (($230) + ($229<<4)|0);
    $232 = $j;
    $233 = $232<<2;
    _memcpy(($228|0),($231|0),($233|0))|0;
    $234 = $k3;
    $235 = (($234) + 1)|0;
    $k3 = $235;
   }
  }
 }
 $236 = HEAP32[$h>>2]|0;
 _free($236);
 STACKTOP = sp;return;
}
function _dwt_decode_max_resolution($r,$i) {
 $r = $r|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mr = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $r;
 $1 = $i;
 $mr = 1;
 while(1) {
  $2 = $1;
  $3 = (($2) + -1)|0;
  $1 = $3;
  $4 = ($3|0)!=(0);
  if (!($4)) {
   break;
  }
  $5 = $0;
  $6 = (($5) + 124|0);
  $0 = $6;
  $7 = $mr;
  $8 = $0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) - ($12))|0;
  $w = $13;
  $14 = ($7|0)<($13|0);
  if ($14) {
   $15 = $w;
   $mr = $15;
  }
  $16 = $mr;
  $17 = $0;
  $18 = (($17) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $0;
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) - ($22))|0;
  $w = $23;
  $24 = ($16|0)<($23|0);
  if ($24) {
   $25 = $w;
   $mr = $25;
  }
 }
 $26 = $mr;
 STACKTOP = sp;return ($26|0);
}
function _v4dwt_interleave_h($w,$a,$x,$size) {
 $w = $w|0;
 $a = $a|0;
 $x = $x|0;
 $size = $size|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0.0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bi = 0, $count = 0, $i = 0, $j = 0, $j1 = 0, $k = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $w;
 $1 = $a;
 $2 = $x;
 $3 = $size;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5) + ($8<<4)|0);
 $bi = $9;
 $10 = $0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $count = $12;
 $k = 0;
 while(1) {
  $13 = $k;
  $14 = ($13|0)<(2);
  if (!($14)) {
   break;
  }
  $15 = $count;
  $16 = $2;
  $17 = ($16*3)|0;
  $18 = (($15) + ($17))|0;
  $19 = $3;
  $20 = ($18|0)<($19|0);
  if ($20) {
   $21 = $1;
   $22 = $21;
   $23 = $22 & 15;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = $bi;
    $26 = $25;
    $27 = $26 & 15;
    $28 = ($27|0)==(0);
    if ($28) {
     $29 = $2;
     $30 = $29 & 15;
     $31 = ($30|0)==(0);
     if ($31) {
      $i = 0;
      while(1) {
       $32 = $i;
       $33 = $count;
       $34 = ($32|0)<($33|0);
       if (!($34)) {
        break;
       }
       $35 = $i;
       $j = $35;
       $36 = $j;
       $37 = $1;
       $38 = (($37) + ($36<<2)|0);
       $39 = +HEAPF32[$38>>2];
       $40 = $i;
       $41 = $40<<3;
       $42 = $bi;
       $43 = (($42) + ($41<<2)|0);
       HEAPF32[$43>>2] = $39;
       $44 = $2;
       $45 = $j;
       $46 = (($45) + ($44))|0;
       $j = $46;
       $47 = $j;
       $48 = $1;
       $49 = (($48) + ($47<<2)|0);
       $50 = +HEAPF32[$49>>2];
       $51 = $i;
       $52 = $51<<3;
       $53 = (($52) + 1)|0;
       $54 = $bi;
       $55 = (($54) + ($53<<2)|0);
       HEAPF32[$55>>2] = $50;
       $56 = $2;
       $57 = $j;
       $58 = (($57) + ($56))|0;
       $j = $58;
       $59 = $j;
       $60 = $1;
       $61 = (($60) + ($59<<2)|0);
       $62 = +HEAPF32[$61>>2];
       $63 = $i;
       $64 = $63<<3;
       $65 = (($64) + 2)|0;
       $66 = $bi;
       $67 = (($66) + ($65<<2)|0);
       HEAPF32[$67>>2] = $62;
       $68 = $2;
       $69 = $j;
       $70 = (($69) + ($68))|0;
       $j = $70;
       $71 = $j;
       $72 = $1;
       $73 = (($72) + ($71<<2)|0);
       $74 = +HEAPF32[$73>>2];
       $75 = $i;
       $76 = $75<<3;
       $77 = (($76) + 3)|0;
       $78 = $bi;
       $79 = (($78) + ($77<<2)|0);
       HEAPF32[$79>>2] = $74;
       $80 = $i;
       $81 = (($80) + 1)|0;
       $i = $81;
      }
     } else {
      label = 12;
     }
    } else {
     label = 12;
    }
   } else {
    label = 12;
   }
  } else {
   label = 12;
  }
  if ((label|0) == 12) {
   label = 0;
   $i = 0;
   while(1) {
    $82 = $i;
    $83 = $count;
    $84 = ($82|0)<($83|0);
    if (!($84)) {
     break;
    }
    $85 = $i;
    $j1 = $85;
    $86 = $j1;
    $87 = $1;
    $88 = (($87) + ($86<<2)|0);
    $89 = +HEAPF32[$88>>2];
    $90 = $i;
    $91 = $90<<3;
    $92 = $bi;
    $93 = (($92) + ($91<<2)|0);
    HEAPF32[$93>>2] = $89;
    $94 = $2;
    $95 = $j1;
    $96 = (($95) + ($94))|0;
    $j1 = $96;
    $97 = $j1;
    $98 = $3;
    $99 = ($97|0)>($98|0);
    do {
     if ($99) {
     } else {
      $100 = $j1;
      $101 = $1;
      $102 = (($101) + ($100<<2)|0);
      $103 = +HEAPF32[$102>>2];
      $104 = $i;
      $105 = $104<<3;
      $106 = (($105) + 1)|0;
      $107 = $bi;
      $108 = (($107) + ($106<<2)|0);
      HEAPF32[$108>>2] = $103;
      $109 = $2;
      $110 = $j1;
      $111 = (($110) + ($109))|0;
      $j1 = $111;
      $112 = $j1;
      $113 = $3;
      $114 = ($112|0)>($113|0);
      if ($114) {
       break;
      }
      $115 = $j1;
      $116 = $1;
      $117 = (($116) + ($115<<2)|0);
      $118 = +HEAPF32[$117>>2];
      $119 = $i;
      $120 = $119<<3;
      $121 = (($120) + 2)|0;
      $122 = $bi;
      $123 = (($122) + ($121<<2)|0);
      HEAPF32[$123>>2] = $118;
      $124 = $2;
      $125 = $j1;
      $126 = (($125) + ($124))|0;
      $j1 = $126;
      $127 = $j1;
      $128 = $3;
      $129 = ($127|0)>($128|0);
      if ($129) {
       break;
      } else {
       $130 = $j1;
       $131 = $1;
       $132 = (($131) + ($130<<2)|0);
       $133 = +HEAPF32[$132>>2];
       $134 = $i;
       $135 = $134<<3;
       $136 = (($135) + 3)|0;
       $137 = $bi;
       $138 = (($137) + ($136<<2)|0);
       HEAPF32[$138>>2] = $133;
       break;
      }
     }
    } while(0);
    $139 = $i;
    $140 = (($139) + 1)|0;
    $i = $140;
   }
  }
  $141 = $0;
  $142 = HEAP32[$141>>2]|0;
  $143 = (($142) + 16|0);
  $144 = $0;
  $145 = (($144) + 12|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = (0 - ($146))|0;
  $148 = (($143) + ($147<<4)|0);
  $bi = $148;
  $149 = $0;
  $150 = (($149) + 8|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = $1;
  $153 = (($152) + ($151<<2)|0);
  $1 = $153;
  $154 = $0;
  $155 = (($154) + 8|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = $3;
  $158 = (($157) - ($156))|0;
  $3 = $158;
  $159 = $0;
  $160 = (($159) + 4|0);
  $161 = HEAP32[$160>>2]|0;
  $count = $161;
  $162 = $k;
  $163 = (($162) + 1)|0;
  $k = $163;
 }
 STACKTOP = sp;return;
}
function _v4dwt_decode($dwt) {
 $dwt = $dwt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0;
 var $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dwt;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)>(0);
  if (!($8)) {
   $9 = $0;
   $10 = (($9) + 8|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = ($11|0)>(1);
   if (!($12)) {
    STACKTOP = sp;return;
   }
  }
  $a = 0;
  $b = 1;
 } else {
  $13 = $0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)>(0);
  if (!($16)) {
   $17 = $0;
   $18 = (($17) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)>(1);
   if (!($20)) {
    STACKTOP = sp;return;
   }
  }
  $a = 1;
  $b = 0;
 }
 $21 = $0;
 $22 = HEAP32[$21>>2]|0;
 $23 = $a;
 $24 = (($22) + ($23<<4)|0);
 $25 = $0;
 $26 = (($25) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 _v4dwt_decode_step1($24,$27,1.23017406463623046875);
 $28 = $0;
 $29 = HEAP32[$28>>2]|0;
 $30 = $b;
 $31 = (($29) + ($30<<4)|0);
 $32 = $0;
 $33 = (($32) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 _v4dwt_decode_step1($31,$34,1.625732421875);
 $35 = $0;
 $36 = HEAP32[$35>>2]|0;
 $37 = $b;
 $38 = (($36) + ($37<<4)|0);
 $39 = $0;
 $40 = HEAP32[$39>>2]|0;
 $41 = $a;
 $42 = (($40) + ($41<<4)|0);
 $43 = (($42) + 16|0);
 $44 = $0;
 $45 = (($44) + 8|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $0;
 $48 = (($47) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $0;
 $51 = (($50) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = $a;
 $54 = (($52) - ($53))|0;
 $55 = (_int_min($49,$54)|0);
 _v4dwt_decode_step2($38,$43,$46,$55,-0.443506866693496704102);
 $56 = $0;
 $57 = HEAP32[$56>>2]|0;
 $58 = $a;
 $59 = (($57) + ($58<<4)|0);
 $60 = $0;
 $61 = HEAP32[$60>>2]|0;
 $62 = $b;
 $63 = (($61) + ($62<<4)|0);
 $64 = (($63) + 16|0);
 $65 = $0;
 $66 = (($65) + 4|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = $0;
 $69 = (($68) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $0;
 $72 = (($71) + 8|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $b;
 $75 = (($73) - ($74))|0;
 $76 = (_int_min($70,$75)|0);
 _v4dwt_decode_step2($59,$64,$67,$76,-0.882911086082458496093);
 $77 = $0;
 $78 = HEAP32[$77>>2]|0;
 $79 = $b;
 $80 = (($78) + ($79<<4)|0);
 $81 = $0;
 $82 = HEAP32[$81>>2]|0;
 $83 = $a;
 $84 = (($82) + ($83<<4)|0);
 $85 = (($84) + 16|0);
 $86 = $0;
 $87 = (($86) + 8|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $0;
 $90 = (($89) + 8|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $0;
 $93 = (($92) + 4|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $a;
 $96 = (($94) - ($95))|0;
 $97 = (_int_min($91,$96)|0);
 _v4dwt_decode_step2($80,$85,$88,$97,0.052980117499828338623);
 $98 = $0;
 $99 = HEAP32[$98>>2]|0;
 $100 = $a;
 $101 = (($99) + ($100<<4)|0);
 $102 = $0;
 $103 = HEAP32[$102>>2]|0;
 $104 = $b;
 $105 = (($103) + ($104<<4)|0);
 $106 = (($105) + 16|0);
 $107 = $0;
 $108 = (($107) + 4|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = $0;
 $111 = (($110) + 4|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $0;
 $114 = (($113) + 8|0);
 $115 = HEAP32[$114>>2]|0;
 $116 = $b;
 $117 = (($115) - ($116))|0;
 $118 = (_int_min($112,$117)|0);
 _v4dwt_decode_step2($101,$106,$109,$118,1.58613431453704833984);
 STACKTOP = sp;return;
}
function _v4dwt_interleave_v($v,$a,$x) {
 $v = $v|0;
 $a = $a|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bi = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $v;
 $1 = $a;
 $2 = $x;
 $3 = $0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($4) + ($7<<4)|0);
 $bi = $8;
 $i = 0;
 while(1) {
  $9 = $i;
  $10 = $0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $i;
  $15 = $14<<1;
  $16 = $bi;
  $17 = (($16) + ($15<<4)|0);
  $18 = $i;
  $19 = $2;
  $20 = Math_imul($18, $19)|0;
  $21 = $1;
  $22 = (($21) + ($20<<2)|0);
  ;HEAP32[$17+0>>2]=HEAP32[$22+0>>2]|0;HEAP32[$17+4>>2]=HEAP32[$22+4>>2]|0;HEAP32[$17+8>>2]=HEAP32[$22+8>>2]|0;HEAP32[$17+12>>2]=HEAP32[$22+12>>2]|0;
  $23 = $i;
  $24 = (($23) + 1)|0;
  $i = $24;
 }
 $25 = $0;
 $26 = (($25) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $2;
 $29 = Math_imul($27, $28)|0;
 $30 = $1;
 $31 = (($30) + ($29<<2)|0);
 $1 = $31;
 $32 = $0;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + 16|0);
 $35 = $0;
 $36 = (($35) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (0 - ($37))|0;
 $39 = (($34) + ($38<<4)|0);
 $bi = $39;
 $i = 0;
 while(1) {
  $40 = $i;
  $41 = $0;
  $42 = (($41) + 4|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   break;
  }
  $45 = $i;
  $46 = $45<<1;
  $47 = $bi;
  $48 = (($47) + ($46<<4)|0);
  $49 = $i;
  $50 = $2;
  $51 = Math_imul($49, $50)|0;
  $52 = $1;
  $53 = (($52) + ($51<<2)|0);
  ;HEAP32[$48+0>>2]=HEAP32[$53+0>>2]|0;HEAP32[$48+4>>2]=HEAP32[$53+4>>2]|0;HEAP32[$48+8>>2]=HEAP32[$53+8>>2]|0;HEAP32[$48+12>>2]=HEAP32[$53+12>>2]|0;
  $54 = $i;
  $55 = (($54) + 1)|0;
  $i = $55;
 }
 STACKTOP = sp;return;
}
function _v4dwt_decode_step1($w,$count,$c) {
 $w = $w|0;
 $count = $count|0;
 $c = +$c;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $fw = 0, $i = 0, $tmp1 = 0.0, $tmp2 = 0.0, $tmp3 = 0.0, $tmp4 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $w;
 $1 = $count;
 $2 = $c;
 $3 = $0;
 $fw = $3;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $1;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $7<<3;
  $9 = $fw;
  $10 = (($9) + ($8<<2)|0);
  $11 = +HEAPF32[$10>>2];
  $tmp1 = $11;
  $12 = $i;
  $13 = $12<<3;
  $14 = (($13) + 1)|0;
  $15 = $fw;
  $16 = (($15) + ($14<<2)|0);
  $17 = +HEAPF32[$16>>2];
  $tmp2 = $17;
  $18 = $i;
  $19 = $18<<3;
  $20 = (($19) + 2)|0;
  $21 = $fw;
  $22 = (($21) + ($20<<2)|0);
  $23 = +HEAPF32[$22>>2];
  $tmp3 = $23;
  $24 = $i;
  $25 = $24<<3;
  $26 = (($25) + 3)|0;
  $27 = $fw;
  $28 = (($27) + ($26<<2)|0);
  $29 = +HEAPF32[$28>>2];
  $tmp4 = $29;
  $30 = $tmp1;
  $31 = $2;
  $32 = $30 * $31;
  $33 = $i;
  $34 = $33<<3;
  $35 = $fw;
  $36 = (($35) + ($34<<2)|0);
  HEAPF32[$36>>2] = $32;
  $37 = $tmp2;
  $38 = $2;
  $39 = $37 * $38;
  $40 = $i;
  $41 = $40<<3;
  $42 = (($41) + 1)|0;
  $43 = $fw;
  $44 = (($43) + ($42<<2)|0);
  HEAPF32[$44>>2] = $39;
  $45 = $tmp3;
  $46 = $2;
  $47 = $45 * $46;
  $48 = $i;
  $49 = $48<<3;
  $50 = (($49) + 2)|0;
  $51 = $fw;
  $52 = (($51) + ($50<<2)|0);
  HEAPF32[$52>>2] = $47;
  $53 = $tmp4;
  $54 = $2;
  $55 = $53 * $54;
  $56 = $i;
  $57 = $56<<3;
  $58 = (($57) + 3)|0;
  $59 = $fw;
  $60 = (($59) + ($58<<2)|0);
  HEAPF32[$60>>2] = $55;
  $61 = $i;
  $62 = (($61) + 1)|0;
  $i = $62;
 }
 STACKTOP = sp;return;
}
function _v4dwt_decode_step2($l,$w,$k,$m,$c) {
 $l = $l|0;
 $w = $w|0;
 $k = $k|0;
 $m = $m|0;
 $c = +$c;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0.0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0.0, $125 = 0.0, $126 = 0.0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0.0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0.0, $140 = 0.0, $141 = 0.0, $142 = 0.0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0.0, $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0.0, $33 = 0, $34 = 0.0, $35 = 0, $36 = 0;
 var $37 = 0.0, $38 = 0, $39 = 0, $4 = 0.0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0;
 var $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0, $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0;
 var $73 = 0.0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0.0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0;
 var $91 = 0, $92 = 0.0, $93 = 0.0, $94 = 0.0, $95 = 0, $96 = 0, $97 = 0.0, $98 = 0.0, $99 = 0.0, $c1 = 0.0, $c2 = 0.0, $c3 = 0.0, $c4 = 0.0, $fl = 0, $fw = 0, $i = 0, $tmp1 = 0.0, $tmp1_1 = 0.0, $tmp1_2 = 0.0, $tmp1_3 = 0.0;
 var $tmp1_4 = 0.0, $tmp2 = 0.0, $tmp2_1 = 0.0, $tmp2_2 = 0.0, $tmp2_3 = 0.0, $tmp2_4 = 0.0, $tmp3 = 0.0, $tmp3_1 = 0.0, $tmp3_2 = 0.0, $tmp3_3 = 0.0, $tmp3_4 = 0.0, $tmp4 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = $l;
 $1 = $w;
 $2 = $k;
 $3 = $m;
 $4 = $c;
 $5 = $0;
 $fl = $5;
 $6 = $1;
 $fw = $6;
 $i = 0;
 while(1) {
  $7 = $i;
  $8 = $3;
  $9 = ($7|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $fl;
  $11 = +HEAPF32[$10>>2];
  $tmp1_1 = $11;
  $12 = $fl;
  $13 = (($12) + 4|0);
  $14 = +HEAPF32[$13>>2];
  $tmp1_2 = $14;
  $15 = $fl;
  $16 = (($15) + 8|0);
  $17 = +HEAPF32[$16>>2];
  $tmp1_3 = $17;
  $18 = $fl;
  $19 = (($18) + 12|0);
  $20 = +HEAPF32[$19>>2];
  $tmp1_4 = $20;
  $21 = $fw;
  $22 = (($21) + -16|0);
  $23 = +HEAPF32[$22>>2];
  $tmp2_1 = $23;
  $24 = $fw;
  $25 = (($24) + -12|0);
  $26 = +HEAPF32[$25>>2];
  $tmp2_2 = $26;
  $27 = $fw;
  $28 = (($27) + -8|0);
  $29 = +HEAPF32[$28>>2];
  $tmp2_3 = $29;
  $30 = $fw;
  $31 = (($30) + -4|0);
  $32 = +HEAPF32[$31>>2];
  $tmp2_4 = $32;
  $33 = $fw;
  $34 = +HEAPF32[$33>>2];
  $tmp3_1 = $34;
  $35 = $fw;
  $36 = (($35) + 4|0);
  $37 = +HEAPF32[$36>>2];
  $tmp3_2 = $37;
  $38 = $fw;
  $39 = (($38) + 8|0);
  $40 = +HEAPF32[$39>>2];
  $tmp3_3 = $40;
  $41 = $fw;
  $42 = (($41) + 12|0);
  $43 = +HEAPF32[$42>>2];
  $tmp3_4 = $43;
  $44 = $tmp2_1;
  $45 = $tmp1_1;
  $46 = $tmp3_1;
  $47 = $45 + $46;
  $48 = $4;
  $49 = $47 * $48;
  $50 = $44 + $49;
  $51 = $fw;
  $52 = (($51) + -16|0);
  HEAPF32[$52>>2] = $50;
  $53 = $tmp2_2;
  $54 = $tmp1_2;
  $55 = $tmp3_2;
  $56 = $54 + $55;
  $57 = $4;
  $58 = $56 * $57;
  $59 = $53 + $58;
  $60 = $fw;
  $61 = (($60) + -12|0);
  HEAPF32[$61>>2] = $59;
  $62 = $tmp2_3;
  $63 = $tmp1_3;
  $64 = $tmp3_3;
  $65 = $63 + $64;
  $66 = $4;
  $67 = $65 * $66;
  $68 = $62 + $67;
  $69 = $fw;
  $70 = (($69) + -8|0);
  HEAPF32[$70>>2] = $68;
  $71 = $tmp2_4;
  $72 = $tmp1_4;
  $73 = $tmp3_4;
  $74 = $72 + $73;
  $75 = $4;
  $76 = $74 * $75;
  $77 = $71 + $76;
  $78 = $fw;
  $79 = (($78) + -4|0);
  HEAPF32[$79>>2] = $77;
  $80 = $fw;
  $fl = $80;
  $81 = $fw;
  $82 = (($81) + 32|0);
  $fw = $82;
  $83 = $i;
  $84 = (($83) + 1)|0;
  $i = $84;
 }
 $85 = $3;
 $86 = $2;
 $87 = ($85|0)<($86|0);
 if (!($87)) {
  STACKTOP = sp;return;
 }
 $88 = $4;
 $89 = $4;
 $90 = $89 + $88;
 $4 = $90;
 $91 = $fl;
 $92 = +HEAPF32[$91>>2];
 $93 = $4;
 $94 = $92 * $93;
 $c1 = $94;
 $95 = $fl;
 $96 = (($95) + 4|0);
 $97 = +HEAPF32[$96>>2];
 $98 = $4;
 $99 = $97 * $98;
 $c2 = $99;
 $100 = $fl;
 $101 = (($100) + 8|0);
 $102 = +HEAPF32[$101>>2];
 $103 = $4;
 $104 = $102 * $103;
 $c3 = $104;
 $105 = $fl;
 $106 = (($105) + 12|0);
 $107 = +HEAPF32[$106>>2];
 $108 = $4;
 $109 = $107 * $108;
 $c4 = $109;
 while(1) {
  $110 = $3;
  $111 = $2;
  $112 = ($110|0)<($111|0);
  if (!($112)) {
   break;
  }
  $113 = $fw;
  $114 = (($113) + -16|0);
  $115 = +HEAPF32[$114>>2];
  $tmp1 = $115;
  $116 = $fw;
  $117 = (($116) + -12|0);
  $118 = +HEAPF32[$117>>2];
  $tmp2 = $118;
  $119 = $fw;
  $120 = (($119) + -8|0);
  $121 = +HEAPF32[$120>>2];
  $tmp3 = $121;
  $122 = $fw;
  $123 = (($122) + -4|0);
  $124 = +HEAPF32[$123>>2];
  $tmp4 = $124;
  $125 = $tmp1;
  $126 = $c1;
  $127 = $125 + $126;
  $128 = $fw;
  $129 = (($128) + -16|0);
  HEAPF32[$129>>2] = $127;
  $130 = $tmp2;
  $131 = $c2;
  $132 = $130 + $131;
  $133 = $fw;
  $134 = (($133) + -12|0);
  HEAPF32[$134>>2] = $132;
  $135 = $tmp3;
  $136 = $c3;
  $137 = $135 + $136;
  $138 = $fw;
  $139 = (($138) + -8|0);
  HEAPF32[$139>>2] = $137;
  $140 = $tmp4;
  $141 = $c4;
  $142 = $140 + $141;
  $143 = $fw;
  $144 = (($143) + -4|0);
  HEAPF32[$144>>2] = $142;
  $145 = $fw;
  $146 = (($145) + 32|0);
  $fw = $146;
  $147 = $3;
  $148 = (($147) + 1)|0;
  $3 = $148;
 }
 STACKTOP = sp;return;
}
function _int_min($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)<($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _dwt_interleave_h($h,$a) {
 $h = $h|0;
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ai = 0, $bi = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $h;
 $1 = $a;
 $2 = $1;
 $ai = $2;
 $3 = $0;
 $4 = HEAP32[$3>>2]|0;
 $5 = $0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($4) + ($7<<2)|0);
 $bi = $8;
 $9 = $0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $i = $11;
 while(1) {
  $12 = $i;
  $13 = (($12) + -1)|0;
  $i = $13;
  $14 = ($12|0)!=(0);
  if (!($14)) {
   break;
  }
  $15 = $ai;
  $16 = (($15) + 4|0);
  $ai = $16;
  $17 = HEAP32[$15>>2]|0;
  $18 = $bi;
  HEAP32[$18>>2] = $17;
  $19 = $bi;
  $20 = (($19) + 8|0);
  $bi = $20;
 }
 $21 = $1;
 $22 = $0;
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($21) + ($24<<2)|0);
 $ai = $25;
 $26 = $0;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 4|0);
 $29 = $0;
 $30 = (($29) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (0 - ($31))|0;
 $33 = (($28) + ($32<<2)|0);
 $bi = $33;
 $34 = $0;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $i = $36;
 while(1) {
  $37 = $i;
  $38 = (($37) + -1)|0;
  $i = $38;
  $39 = ($37|0)!=(0);
  if (!($39)) {
   break;
  }
  $40 = $ai;
  $41 = (($40) + 4|0);
  $ai = $41;
  $42 = HEAP32[$40>>2]|0;
  $43 = $bi;
  HEAP32[$43>>2] = $42;
  $44 = $bi;
  $45 = (($44) + 8|0);
  $bi = $45;
 }
 STACKTOP = sp;return;
}
function _dwt_interleave_v($v,$a,$x) {
 $v = $v|0;
 $a = $a|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ai = 0, $bi = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $v;
 $1 = $a;
 $2 = $x;
 $3 = $1;
 $ai = $3;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5) + ($8<<2)|0);
 $bi = $9;
 $10 = $0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $i = $12;
 while(1) {
  $13 = $i;
  $14 = (($13) + -1)|0;
  $i = $14;
  $15 = ($13|0)!=(0);
  if (!($15)) {
   break;
  }
  $16 = $ai;
  $17 = HEAP32[$16>>2]|0;
  $18 = $bi;
  HEAP32[$18>>2] = $17;
  $19 = $bi;
  $20 = (($19) + 8|0);
  $bi = $20;
  $21 = $2;
  $22 = $ai;
  $23 = (($22) + ($21<<2)|0);
  $ai = $23;
 }
 $24 = $1;
 $25 = $0;
 $26 = (($25) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $2;
 $29 = Math_imul($27, $28)|0;
 $30 = (($24) + ($29<<2)|0);
 $ai = $30;
 $31 = $0;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 4|0);
 $34 = $0;
 $35 = (($34) + 12|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (0 - ($36))|0;
 $38 = (($33) + ($37<<2)|0);
 $bi = $38;
 $39 = $0;
 $40 = (($39) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 $i = $41;
 while(1) {
  $42 = $i;
  $43 = (($42) + -1)|0;
  $i = $43;
  $44 = ($42|0)!=(0);
  if (!($44)) {
   break;
  }
  $45 = $ai;
  $46 = HEAP32[$45>>2]|0;
  $47 = $bi;
  HEAP32[$47>>2] = $46;
  $48 = $bi;
  $49 = (($48) + 8|0);
  $bi = $49;
  $50 = $2;
  $51 = $ai;
  $52 = (($51) + ($50<<2)|0);
  $ai = $52;
 }
 STACKTOP = sp;return;
}
function _dwt_decode_1_($a,$dn,$sn,$cas) {
 $a = $a|0;
 $dn = $dn|0;
 $sn = $sn|0;
 $cas = $cas|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $a;
 $1 = $dn;
 $2 = $sn;
 $3 = $cas;
 $4 = $3;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $6 = $1;
  $7 = ($6|0)>(0);
  if ($7) {
   label = 4;
  } else {
   $8 = $2;
   $9 = ($8|0)>(1);
   if ($9) {
    label = 4;
   }
  }
  if ((label|0) == 4) {
   $i = 0;
   while(1) {
    $10 = $i;
    $11 = $2;
    $12 = ($10|0)<($11|0);
    if (!($12)) {
     break;
    }
    $13 = $i;
    $14 = (($13) - 1)|0;
    $15 = ($14|0)<(0);
    if ($15) {
     $16 = $0;
     $17 = (($16) + 4|0);
     $18 = HEAP32[$17>>2]|0;
     $59 = $18;
    } else {
     $19 = $i;
     $20 = (($19) - 1)|0;
     $21 = $1;
     $22 = ($20|0)>=($21|0);
     if ($22) {
      $23 = $1;
      $24 = (($23) - 1)|0;
      $25 = $24<<1;
      $26 = (1 + ($25))|0;
      $27 = $0;
      $28 = (($27) + ($26<<2)|0);
      $29 = HEAP32[$28>>2]|0;
      $249 = $29;
     } else {
      $30 = $i;
      $31 = (($30) - 1)|0;
      $32 = $31<<1;
      $33 = (1 + ($32))|0;
      $34 = $0;
      $35 = (($34) + ($33<<2)|0);
      $36 = HEAP32[$35>>2]|0;
      $249 = $36;
     }
     $59 = $249;
    }
    $37 = $i;
    $38 = ($37|0)<(0);
    if ($38) {
     $39 = $0;
     $40 = (($39) + 4|0);
     $41 = HEAP32[$40>>2]|0;
     $60 = $41;
    } else {
     $42 = $i;
     $43 = $1;
     $44 = ($42|0)>=($43|0);
     if ($44) {
      $45 = $1;
      $46 = (($45) - 1)|0;
      $47 = $46<<1;
      $48 = (1 + ($47))|0;
      $49 = $0;
      $50 = (($49) + ($48<<2)|0);
      $51 = HEAP32[$50>>2]|0;
      $250 = $51;
     } else {
      $52 = $i;
      $53 = $52<<1;
      $54 = (1 + ($53))|0;
      $55 = $0;
      $56 = (($55) + ($54<<2)|0);
      $57 = HEAP32[$56>>2]|0;
      $250 = $57;
     }
     $60 = $250;
    }
    $58 = (($59) + ($60))|0;
    $61 = (($58) + 2)|0;
    $62 = $61 >> 2;
    $63 = $i;
    $64 = $63<<1;
    $65 = $0;
    $66 = (($65) + ($64<<2)|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67) - ($62))|0;
    HEAP32[$66>>2] = $68;
    $69 = $i;
    $70 = (($69) + 1)|0;
    $i = $70;
   }
   $i = 0;
   while(1) {
    $71 = $i;
    $72 = $1;
    $73 = ($71|0)<($72|0);
    if (!($73)) {
     break;
    }
    $74 = $i;
    $75 = ($74|0)<(0);
    if ($75) {
     $76 = $0;
     $77 = HEAP32[$76>>2]|0;
     $114 = $77;
    } else {
     $78 = $i;
     $79 = $2;
     $80 = ($78|0)>=($79|0);
     if ($80) {
      $81 = $2;
      $82 = (($81) - 1)|0;
      $83 = $82<<1;
      $84 = $0;
      $85 = (($84) + ($83<<2)|0);
      $86 = HEAP32[$85>>2]|0;
      $251 = $86;
     } else {
      $87 = $i;
      $88 = $87<<1;
      $89 = $0;
      $90 = (($89) + ($88<<2)|0);
      $91 = HEAP32[$90>>2]|0;
      $251 = $91;
     }
     $114 = $251;
    }
    $92 = $i;
    $93 = (($92) + 1)|0;
    $94 = ($93|0)<(0);
    if ($94) {
     $95 = $0;
     $96 = HEAP32[$95>>2]|0;
     $115 = $96;
    } else {
     $97 = $i;
     $98 = (($97) + 1)|0;
     $99 = $2;
     $100 = ($98|0)>=($99|0);
     if ($100) {
      $101 = $2;
      $102 = (($101) - 1)|0;
      $103 = $102<<1;
      $104 = $0;
      $105 = (($104) + ($103<<2)|0);
      $106 = HEAP32[$105>>2]|0;
      $252 = $106;
     } else {
      $107 = $i;
      $108 = (($107) + 1)|0;
      $109 = $108<<1;
      $110 = $0;
      $111 = (($110) + ($109<<2)|0);
      $112 = HEAP32[$111>>2]|0;
      $252 = $112;
     }
     $115 = $252;
    }
    $113 = (($114) + ($115))|0;
    $116 = $113 >> 1;
    $117 = $i;
    $118 = $117<<1;
    $119 = (1 + ($118))|0;
    $120 = $0;
    $121 = (($120) + ($119<<2)|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = (($122) + ($116))|0;
    HEAP32[$121>>2] = $123;
    $124 = $i;
    $125 = (($124) + 1)|0;
    $i = $125;
   }
  }
  STACKTOP = sp;return;
 }
 $126 = $2;
 $127 = ($126|0)!=(0);
 if ($127) {
  label = 41;
 } else {
  $128 = $1;
  $129 = ($128|0)==(1);
  if ($129) {
   $130 = $0;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($131|0) / 2)&-1;
   HEAP32[$130>>2] = $132;
  } else {
   label = 41;
  }
 }
 if ((label|0) == 41) {
  $i = 0;
  while(1) {
   $133 = $i;
   $134 = $2;
   $135 = ($133|0)<($134|0);
   if (!($135)) {
    break;
   }
   $136 = $i;
   $137 = ($136|0)<(0);
   if ($137) {
    $138 = $0;
    $139 = HEAP32[$138>>2]|0;
    $176 = $139;
   } else {
    $140 = $i;
    $141 = $1;
    $142 = ($140|0)>=($141|0);
    if ($142) {
     $143 = $1;
     $144 = (($143) - 1)|0;
     $145 = $144<<1;
     $146 = $0;
     $147 = (($146) + ($145<<2)|0);
     $148 = HEAP32[$147>>2]|0;
     $253 = $148;
    } else {
     $149 = $i;
     $150 = $149<<1;
     $151 = $0;
     $152 = (($151) + ($150<<2)|0);
     $153 = HEAP32[$152>>2]|0;
     $253 = $153;
    }
    $176 = $253;
   }
   $154 = $i;
   $155 = (($154) + 1)|0;
   $156 = ($155|0)<(0);
   if ($156) {
    $157 = $0;
    $158 = HEAP32[$157>>2]|0;
    $177 = $158;
   } else {
    $159 = $i;
    $160 = (($159) + 1)|0;
    $161 = $1;
    $162 = ($160|0)>=($161|0);
    if ($162) {
     $163 = $1;
     $164 = (($163) - 1)|0;
     $165 = $164<<1;
     $166 = $0;
     $167 = (($166) + ($165<<2)|0);
     $168 = HEAP32[$167>>2]|0;
     $254 = $168;
    } else {
     $169 = $i;
     $170 = (($169) + 1)|0;
     $171 = $170<<1;
     $172 = $0;
     $173 = (($172) + ($171<<2)|0);
     $174 = HEAP32[$173>>2]|0;
     $254 = $174;
    }
    $177 = $254;
   }
   $175 = (($176) + ($177))|0;
   $178 = (($175) + 2)|0;
   $179 = $178 >> 2;
   $180 = $i;
   $181 = $180<<1;
   $182 = (1 + ($181))|0;
   $183 = $0;
   $184 = (($183) + ($182<<2)|0);
   $185 = HEAP32[$184>>2]|0;
   $186 = (($185) - ($179))|0;
   HEAP32[$184>>2] = $186;
   $187 = $i;
   $188 = (($187) + 1)|0;
   $i = $188;
  }
  $i = 0;
  while(1) {
   $189 = $i;
   $190 = $1;
   $191 = ($189|0)<($190|0);
   if (!($191)) {
    break;
   }
   $192 = $i;
   $193 = ($192|0)<(0);
   if ($193) {
    $194 = $0;
    $195 = (($194) + 4|0);
    $196 = HEAP32[$195>>2]|0;
    $238 = $196;
   } else {
    $197 = $i;
    $198 = $2;
    $199 = ($197|0)>=($198|0);
    if ($199) {
     $200 = $2;
     $201 = (($200) - 1)|0;
     $202 = $201<<1;
     $203 = (1 + ($202))|0;
     $204 = $0;
     $205 = (($204) + ($203<<2)|0);
     $206 = HEAP32[$205>>2]|0;
     $255 = $206;
    } else {
     $207 = $i;
     $208 = $207<<1;
     $209 = (1 + ($208))|0;
     $210 = $0;
     $211 = (($210) + ($209<<2)|0);
     $212 = HEAP32[$211>>2]|0;
     $255 = $212;
    }
    $238 = $255;
   }
   $213 = $i;
   $214 = (($213) - 1)|0;
   $215 = ($214|0)<(0);
   if ($215) {
    $216 = $0;
    $217 = (($216) + 4|0);
    $218 = HEAP32[$217>>2]|0;
    $239 = $218;
   } else {
    $219 = $i;
    $220 = (($219) - 1)|0;
    $221 = $2;
    $222 = ($220|0)>=($221|0);
    if ($222) {
     $223 = $2;
     $224 = (($223) - 1)|0;
     $225 = $224<<1;
     $226 = (1 + ($225))|0;
     $227 = $0;
     $228 = (($227) + ($226<<2)|0);
     $229 = HEAP32[$228>>2]|0;
     $256 = $229;
    } else {
     $230 = $i;
     $231 = (($230) - 1)|0;
     $232 = $231<<1;
     $233 = (1 + ($232))|0;
     $234 = $0;
     $235 = (($234) + ($233<<2)|0);
     $236 = HEAP32[$235>>2]|0;
     $256 = $236;
    }
    $239 = $256;
   }
   $237 = (($238) + ($239))|0;
   $240 = $237 >> 1;
   $241 = $i;
   $242 = $241<<1;
   $243 = $0;
   $244 = (($243) + ($242<<2)|0);
   $245 = HEAP32[$244>>2]|0;
   $246 = (($245) + ($240))|0;
   HEAP32[$244>>2] = $246;
   $247 = $i;
   $248 = (($247) + 1)|0;
   $i = $248;
  }
 }
 STACKTOP = sp;return;
}
function _opj_set_event_mgr($cinfo,$event_mgr,$context) {
 $cinfo = $cinfo|0;
 $event_mgr = $event_mgr|0;
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $previous = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $event_mgr;
 $3 = $context;
 $4 = $1;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = $1;
  $7 = HEAP32[$6>>2]|0;
  $previous = $7;
  $8 = $2;
  $9 = $1;
  HEAP32[$9>>2] = $8;
  $10 = $3;
  $11 = $1;
  $12 = (($11) + 4|0);
  HEAP32[$12>>2] = $10;
  $13 = $previous;
  $0 = $13;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 } else {
  $0 = 0;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 }
 return 0|0;
}
function _opj_event_msg($cinfo,$event_type,$fmt,$varargs) {
 $cinfo = $cinfo|0;
 $event_type = $event_type|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $arg = 0, $event_mgr = 0, $message = 0, $msg_handler = 0, $str_length = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 560|0;
 $arg = sp + 8|0;
 $message = sp + 48|0;
 $1 = $cinfo;
 $2 = $event_type;
 $3 = $fmt;
 $msg_handler = 0;
 $4 = $1;
 $5 = HEAP32[$4>>2]|0;
 $event_mgr = $5;
 $6 = $event_mgr;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $0 = 0;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = $2;
 if ((($8|0) == 1)) {
  $9 = $event_mgr;
  $10 = HEAP32[$9>>2]|0;
  $msg_handler = $10;
 } else if ((($8|0) == 2)) {
  $11 = $event_mgr;
  $12 = (($11) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $msg_handler = $13;
 } else if ((($8|0) == 4)) {
  $14 = $event_mgr;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $msg_handler = $16;
 } else {
 }
 $17 = $msg_handler;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $0 = 0;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 $19 = $3;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = $event_mgr;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   _memset(($message|0),0,512)|0;
   HEAP32[$arg>>2] = $varargs;
   $23 = $3;
   $24 = (_strlen(($23|0))|0);
   $25 = ($24>>>0)>(512);
   if ($25) {
    $28 = 512;
   } else {
    $26 = $3;
    $27 = (_strlen(($26|0))|0);
    $28 = $27;
   }
   $str_length = $28;
   $29 = $3;
   (_vsprintf($message,$29,$arg)|0);
   $30 = $msg_handler;
   $31 = $1;
   $32 = (($31) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   FUNCTION_TABLE_vii[$30 & 255]($message,$33);
  }
 }
 $0 = 1;
 $34 = $0;
 STACKTOP = sp;return ($34|0);
}
function _opj_image_create0() {
 var $0 = 0, $1 = 0, $image = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = (_calloc(1,36)|0);
 $image = $0;
 $1 = $image;
 STACKTOP = sp;return ($1|0);
}
function _opj_image_destroy($image) {
 $image = $image|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $image_comp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $image;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $i = 0;
  while(1) {
   $7 = $i;
   $8 = $0;
   $9 = (($8) + 16|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($7|0)<($10|0);
   if (!($11)) {
    break;
   }
   $12 = $i;
   $13 = $0;
   $14 = (($13) + 24|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + (($12*48)|0)|0);
   $image_comp = $16;
   $17 = $image_comp;
   $18 = (($17) + 44|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)!=(0|0);
   if ($20) {
    $21 = $image_comp;
    $22 = (($21) + 44|0);
    $23 = HEAP32[$22>>2]|0;
    _free($23);
   }
   $24 = $i;
   $25 = (($24) + 1)|0;
   $i = $25;
  }
  $26 = $0;
  $27 = (($26) + 24|0);
  $28 = HEAP32[$27>>2]|0;
  _free($28);
 }
 $29 = $0;
 _free($29);
 STACKTOP = sp;return;
}
function _j2k_read_soc($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 4|0);
 HEAP32[$2>>2] = 2;
 $3 = $0;
 $4 = (($3) + 68|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $0;
 $8 = (($7) + 72|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_cio_tell($9)|0);
 $11 = (($10) - 2)|0;
 $12 = $0;
 $13 = (($12) + 68|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 76|0);
 HEAP32[$15>>2] = $11;
 $16 = $0;
 $17 = (($16) + 72|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (_cio_numbytesleft($18)|0);
 $20 = (($19) + 2)|0;
 $21 = $0;
 $22 = (($21) + 68|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 76|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($20) - ($25))|0;
 $27 = $0;
 $28 = (($27) + 68|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + 84|0);
 HEAP32[$30>>2] = $26;
 STACKTOP = sp;return;
}
function _j2k_read_sot($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $cio = 0, $cp = 0, $i = 0, $len = 0, $numparts = 0, $partno = 0, $status = 0, $tcp = 0, $tileno = 0, $tmp = 0, $totlen = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $j2k;
 $tcp = 0;
 $status = 0;
 $1 = $0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $cp = $3;
 $4 = $0;
 $5 = (($4) + 72|0);
 $6 = HEAP32[$5>>2]|0;
 $cio = $6;
 $7 = $cio;
 $8 = (_cio_read($7,2)|0);
 $len = $8;
 $9 = $cio;
 $10 = (_cio_read($9,2)|0);
 $tileno = $10;
 $11 = $cp;
 $12 = (($11) + 80|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = $tileno;
  $16 = $cp;
  $17 = (($16) + 80|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $cp;
  $20 = (($19) + 76|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<2)|0);
  HEAP32[$22>>2] = $15;
  $23 = $cp;
  $24 = (($23) + 80|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$24>>2] = $26;
 } else {
  $i = 0;
  while(1) {
   $27 = $i;
   $28 = $cp;
   $29 = (($28) + 80|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($27|0)<($30|0);
   if ($31) {
    $32 = $status;
    $33 = $32 << 24 >> 24;
    $34 = ($33|0)==(0);
    $281 = $34;
   } else {
    $281 = 0;
   }
   if (!($281)) {
    break;
   }
   $35 = $i;
   $36 = $cp;
   $37 = (($36) + 76|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + ($35<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = $tileno;
   $42 = ($40|0)==($41|0);
   $43 = $42 ? 1 : 0;
   $44 = $43&255;
   $status = $44;
   $45 = $i;
   $46 = (($45) + 1)|0;
   $i = $46;
  }
  $47 = $status;
  $48 = $47 << 24 >> 24;
  $49 = ($48|0)==(0);
  if ($49) {
   $50 = $tileno;
   $51 = $cp;
   $52 = (($51) + 80|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = $cp;
   $55 = (($54) + 76|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + ($53<<2)|0);
   HEAP32[$57>>2] = $50;
   $58 = $cp;
   $59 = (($58) + 80|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = (($60) + 1)|0;
   HEAP32[$59>>2] = $61;
  }
 }
 $62 = $cio;
 $63 = (_cio_read($62,4)|0);
 $totlen = $63;
 $64 = $totlen;
 $65 = ($64|0)!=(0);
 if (!($65)) {
  $66 = $cio;
  $67 = (_cio_numbytesleft($66)|0);
  $68 = (($67) + 8)|0;
  $totlen = $68;
 }
 $69 = $cio;
 $70 = (_cio_read($69,1)|0);
 $partno = $70;
 $71 = $cio;
 $72 = (_cio_read($71,1)|0);
 $numparts = $72;
 $73 = $tileno;
 $74 = $0;
 $75 = (($74) + 8|0);
 HEAP32[$75>>2] = $73;
 $76 = $partno;
 $77 = $0;
 $78 = (($77) + 16|0);
 HEAP32[$78>>2] = $76;
 $79 = $cio;
 $80 = (_cio_getbp($79)|0);
 $81 = (($80) + -12|0);
 $82 = $totlen;
 $83 = (($81) + ($82)|0);
 $84 = $0;
 $85 = (($84) + 32|0);
 HEAP32[$85>>2] = $83;
 $86 = $0;
 $87 = (($86) + 4|0);
 HEAP32[$87>>2] = 16;
 $88 = $0;
 $89 = (($88) + 8|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = $cp;
 $92 = (($91) + 108|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = (($93) + (($90*5588)|0)|0);
 $tcp = $94;
 $95 = $0;
 $96 = (($95) + 68|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = ($97|0)!=(0|0);
 if ($98) {
  $99 = $tcp;
  $100 = HEAP32[$99>>2]|0;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = $tileno;
   $103 = ($102|0)==(0);
   if ($103) {
    $104 = $cio;
    $105 = (_cio_tell($104)|0);
    $106 = (($105) - 13)|0;
    $107 = $0;
    $108 = (($107) + 68|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 80|0);
    HEAP32[$110>>2] = $106;
   }
   $111 = $tileno;
   $112 = $tileno;
   $113 = $0;
   $114 = (($113) + 68|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = (($115) + 88|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = (($117) + (($112*576)|0)|0);
   $119 = (($118) + 4|0);
   HEAP32[$119>>2] = $111;
   $120 = $cio;
   $121 = (_cio_tell($120)|0);
   $122 = (($121) - 12)|0;
   $123 = $tileno;
   $124 = $0;
   $125 = (($124) + 68|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = (($126) + 88|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = (($128) + (($123*576)|0)|0);
   $130 = (($129) + 8|0);
   HEAP32[$130>>2] = $122;
   $131 = $tileno;
   $132 = $0;
   $133 = (($132) + 68|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = (($134) + 88|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($136) + (($131*576)|0)|0);
   $138 = (($137) + 8|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $totlen;
   $141 = (($139) + ($140))|0;
   $142 = (($141) - 1)|0;
   $143 = $tileno;
   $144 = $0;
   $145 = (($144) + 68|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = (($146) + 88|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = (($148) + (($143*576)|0)|0);
   $150 = (($149) + 16|0);
   HEAP32[$150>>2] = $142;
   $151 = $numparts;
   $152 = $tileno;
   $153 = $0;
   $154 = (($153) + 68|0);
   $155 = HEAP32[$154>>2]|0;
   $156 = (($155) + 88|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (($157) + (($152*576)|0)|0);
   $159 = (($158) + 568|0);
   HEAP32[$159>>2] = $151;
   $160 = $numparts;
   $161 = ($160|0)!=(0);
   if ($161) {
    $162 = $numparts;
    $163 = ($162*20)|0;
    $164 = (_malloc($163)|0);
    $165 = $tileno;
    $166 = $0;
    $167 = (($166) + 68|0);
    $168 = HEAP32[$167>>2]|0;
    $169 = (($168) + 88|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = (($170) + (($165*576)|0)|0);
    $172 = (($171) + 572|0);
    HEAP32[$172>>2] = $164;
   } else {
    $173 = (_malloc(200)|0);
    $174 = $tileno;
    $175 = $0;
    $176 = (($175) + 68|0);
    $177 = HEAP32[$176>>2]|0;
    $178 = (($177) + 88|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = (($179) + (($174*576)|0)|0);
    $181 = (($180) + 572|0);
    HEAP32[$181>>2] = $173;
   }
  } else {
   $182 = $totlen;
   $183 = $tileno;
   $184 = $0;
   $185 = (($184) + 68|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = (($186) + 88|0);
   $188 = HEAP32[$187>>2]|0;
   $189 = (($188) + (($183*576)|0)|0);
   $190 = (($189) + 16|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = (($191) + ($182))|0;
   HEAP32[$190>>2] = $192;
  }
  $193 = $cio;
  $194 = (_cio_tell($193)|0);
  $195 = (($194) - 12)|0;
  $196 = $partno;
  $197 = $tileno;
  $198 = $0;
  $199 = (($198) + 68|0);
  $200 = HEAP32[$199>>2]|0;
  $201 = (($200) + 88|0);
  $202 = HEAP32[$201>>2]|0;
  $203 = (($202) + (($197*576)|0)|0);
  $204 = (($203) + 572|0);
  $205 = HEAP32[$204>>2]|0;
  $206 = (($205) + (($196*20)|0)|0);
  HEAP32[$206>>2] = $195;
  $207 = $partno;
  $208 = $tileno;
  $209 = $0;
  $210 = (($209) + 68|0);
  $211 = HEAP32[$210>>2]|0;
  $212 = (($211) + 88|0);
  $213 = HEAP32[$212>>2]|0;
  $214 = (($213) + (($208*576)|0)|0);
  $215 = (($214) + 572|0);
  $216 = HEAP32[$215>>2]|0;
  $217 = (($216) + (($207*20)|0)|0);
  $218 = HEAP32[$217>>2]|0;
  $219 = $totlen;
  $220 = (($218) + ($219))|0;
  $221 = (($220) - 1)|0;
  $222 = $partno;
  $223 = $tileno;
  $224 = $0;
  $225 = (($224) + 68|0);
  $226 = HEAP32[$225>>2]|0;
  $227 = (($226) + 88|0);
  $228 = HEAP32[$227>>2]|0;
  $229 = (($228) + (($223*576)|0)|0);
  $230 = (($229) + 572|0);
  $231 = HEAP32[$230>>2]|0;
  $232 = (($231) + (($222*20)|0)|0);
  $233 = (($232) + 8|0);
  HEAP32[$233>>2] = $221;
 }
 $234 = $tcp;
 $235 = HEAP32[$234>>2]|0;
 $236 = ($235|0)==(1);
 if (!($236)) {
  STACKTOP = sp;return;
 }
 $237 = $tcp;
 $238 = (($237) + 5584|0);
 $239 = HEAP32[$238>>2]|0;
 $tmp = $239;
 $240 = $tcp;
 $241 = $0;
 $242 = (($241) + 56|0);
 $243 = HEAP32[$242>>2]|0;
 _memcpy(($240|0),($243|0),5588)|0;
 $244 = $tcp;
 $245 = (($244) + 5172|0);
 HEAP32[$245>>2] = 0;
 $246 = $tcp;
 $247 = (($246) + 5164|0);
 HEAP32[$247>>2] = 0;
 $248 = $tcp;
 $249 = (($248) + 5168|0);
 HEAP32[$249>>2] = 0;
 $250 = $tmp;
 $251 = $tcp;
 $252 = (($251) + 5584|0);
 HEAP32[$252>>2] = $250;
 $i = 0;
 while(1) {
  $253 = $i;
  $254 = $0;
  $255 = (($254) + 60|0);
  $256 = HEAP32[$255>>2]|0;
  $257 = (($256) + 16|0);
  $258 = HEAP32[$257>>2]|0;
  $259 = ($253|0)<($258|0);
  if (!($259)) {
   break;
  }
  $260 = $i;
  $261 = $tcp;
  $262 = (($261) + 5584|0);
  $263 = HEAP32[$262>>2]|0;
  $264 = (($263) + (($260*1076)|0)|0);
  $265 = $i;
  $266 = $0;
  $267 = (($266) + 56|0);
  $268 = HEAP32[$267>>2]|0;
  $269 = (($268) + 5584|0);
  $270 = HEAP32[$269>>2]|0;
  $271 = (($270) + (($265*1076)|0)|0);
  _memcpy(($264|0),($271|0),1076)|0;
  $272 = $i;
  $273 = (($272) + 1)|0;
  $i = $273;
 }
 $274 = $0;
 $275 = (($274) + 8|0);
 $276 = HEAP32[$275>>2]|0;
 $277 = $cp;
 $278 = (($277) + 108|0);
 $279 = HEAP32[$278>>2]|0;
 $280 = (($279) + (($276*5588)|0)|0);
 HEAP32[$280>>2] = 0;
 STACKTOP = sp;return;
}
function _j2k_read_sod($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cio = 0, $curtileno = 0, $data = 0, $data_ptr = 0;
 var $i = 0, $len = 0, $truncate = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $truncate = 0;
 $data = 0;
 $data_ptr = 0;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $curtileno = $6;
 $7 = $0;
 $8 = (($7) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = $cio;
  $12 = (_cio_tell($11)|0);
  $13 = $0;
  $14 = (($13) + 44|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($12) + ($15))|0;
  $17 = (($16) - 1)|0;
  $18 = $0;
  $19 = (($18) + 16|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $0;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = $0;
  $25 = (($24) + 68|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 88|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($23*576)|0)|0);
  $30 = (($29) + 572|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + (($20*20)|0)|0);
  $33 = (($32) + 4|0);
  HEAP32[$33>>2] = $17;
  $34 = $0;
  $35 = (($34) + 16|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)==(0);
  if ($37) {
   $38 = $cio;
   $39 = (_cio_tell($38)|0);
   $40 = $0;
   $41 = (($40) + 44|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($39) + ($42))|0;
   $44 = (($43) - 1)|0;
   $45 = $0;
   $46 = (($45) + 8|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $0;
   $49 = (($48) + 68|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + 88|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + (($47*576)|0)|0);
   $54 = (($53) + 12|0);
   HEAP32[$54>>2] = $44;
  }
  $55 = $0;
  $56 = (($55) + 68|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 8|0);
  HEAP32[$58>>2] = 0;
 }
 $59 = $0;
 $60 = (($59) + 32|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = $cio;
 $63 = (_cio_getbp($62)|0);
 $64 = $61;
 $65 = $63;
 $66 = (($64) - ($65))|0;
 $67 = $cio;
 $68 = (_cio_numbytesleft($67)|0);
 $69 = (($68) + 1)|0;
 $70 = (_int_min10($66,$69)|0);
 $len = $70;
 $71 = $len;
 $72 = $cio;
 $73 = (_cio_numbytesleft($72)|0);
 $74 = (($73) + 1)|0;
 $75 = ($71|0)==($74|0);
 if ($75) {
  $truncate = 1;
 }
 $76 = $curtileno;
 $77 = $0;
 $78 = (($77) + 48|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = (($79) + ($76<<2)|0);
 $81 = HEAP32[$80>>2]|0;
 $data = $81;
 $82 = $data;
 $83 = $curtileno;
 $84 = $0;
 $85 = (($84) + 52|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = (($86) + ($83<<2)|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $len;
 $90 = (($88) + ($89))|0;
 $91 = $90;
 $92 = (_realloc($82,$91)|0);
 $data = $92;
 $93 = $data;
 $94 = $curtileno;
 $95 = $0;
 $96 = (($95) + 52|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = (($97) + ($94<<2)|0);
 $99 = HEAP32[$98>>2]|0;
 $100 = (($93) + ($99)|0);
 $data_ptr = $100;
 $i = 0;
 while(1) {
  $101 = $i;
  $102 = $len;
  $103 = ($101|0)<($102|0);
  if (!($103)) {
   break;
  }
  $104 = $cio;
  $105 = (_cio_read($104,1)|0);
  $106 = $105&255;
  $107 = $i;
  $108 = $data_ptr;
  $109 = (($108) + ($107)|0);
  HEAP8[$109>>0] = $106;
  $110 = $i;
  $111 = (($110) + 1)|0;
  $i = $111;
 }
 $112 = $len;
 $113 = $curtileno;
 $114 = $0;
 $115 = (($114) + 52|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = (($116) + ($113<<2)|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = (($118) + ($112))|0;
 HEAP32[$117>>2] = $119;
 $120 = $data;
 $121 = $curtileno;
 $122 = $0;
 $123 = (($122) + 48|0);
 $124 = HEAP32[$123>>2]|0;
 $125 = (($124) + ($121<<2)|0);
 HEAP32[$125>>2] = $120;
 $126 = $truncate;
 $127 = ($126|0)!=(0);
 if ($127) {
  $130 = $0;
  $131 = (($130) + 4|0);
  HEAP32[$131>>2] = 64;
  $132 = $0;
  $133 = (($132) + 16|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = (($134) + 1)|0;
  HEAP32[$133>>2] = $135;
  STACKTOP = sp;return;
 } else {
  $128 = $0;
  $129 = (($128) + 4|0);
  HEAP32[$129>>2] = 8;
  $132 = $0;
  $133 = (($132) + 16|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = (($134) + 1)|0;
  HEAP32[$133>>2] = $135;
  STACKTOP = sp;return;
 }
}
function _j2k_read_eoc($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $success = 0, $tcd = 0, $tileno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(2);
 if ($6) {
  $7 = $0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_tcd_create($8)|0);
  $tcd = $9;
  $10 = $tcd;
  $11 = $0;
  $12 = (($11) + 60|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $0;
  $15 = (($14) + 64|0);
  $16 = HEAP32[$15>>2]|0;
  _tcd_malloc_decode($10,$13,$16);
  $i = 0;
  while(1) {
   $17 = $i;
   $18 = $0;
   $19 = (($18) + 64|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 80|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($17|0)<($22|0);
   if (!($23)) {
    break;
   }
   $24 = $tcd;
   $25 = $0;
   $26 = (($25) + 60|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $0;
   $29 = (($28) + 64|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $i;
   $32 = $0;
   $33 = (($32) + 68|0);
   $34 = HEAP32[$33>>2]|0;
   _tcd_malloc_decode_tile($24,$27,$30,$31,$34);
   $35 = $i;
   $36 = $0;
   $37 = (($36) + 64|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + 76|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + ($35<<2)|0);
   $42 = HEAP32[$41>>2]|0;
   $tileno = $42;
   $43 = $tcd;
   $44 = $tileno;
   $45 = $0;
   $46 = (($45) + 48|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($44<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = $tileno;
   $51 = $0;
   $52 = (($51) + 52|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + ($50<<2)|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = $tileno;
   $57 = $0;
   $58 = (($57) + 68|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_tcd_decode_tile($43,$49,$55,$56,$59)|0);
   $success = $60;
   $61 = $tileno;
   $62 = $0;
   $63 = (($62) + 48|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = (($64) + ($61<<2)|0);
   $66 = HEAP32[$65>>2]|0;
   _free($66);
   $67 = $tileno;
   $68 = $0;
   $69 = (($68) + 48|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + ($67<<2)|0);
   HEAP32[$71>>2] = 0;
   $72 = $tcd;
   $73 = $i;
   _tcd_free_decode_tile($72,$73);
   $74 = $success;
   $75 = ($74|0)==(0);
   if ($75) {
    label = 5;
    break;
   }
   $80 = $i;
   $81 = (($80) + 1)|0;
   $i = $81;
  }
  if ((label|0) == 5) {
   $76 = $0;
   $77 = (($76) + 4|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $78 | 128;
   HEAP32[$77>>2] = $79;
  }
  $82 = $tcd;
  _tcd_free_decode($82);
  $83 = $tcd;
  _tcd_destroy($83);
 } else {
  $i = 0;
  while(1) {
   $84 = $i;
   $85 = $0;
   $86 = (($85) + 64|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (($87) + 80|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = ($84|0)<($89|0);
   if (!($90)) {
    break;
   }
   $91 = $i;
   $92 = $0;
   $93 = (($92) + 64|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + 76|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = (($96) + ($91<<2)|0);
   $98 = HEAP32[$97>>2]|0;
   $tileno = $98;
   $99 = $tileno;
   $100 = $0;
   $101 = (($100) + 48|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) + ($99<<2)|0);
   $104 = HEAP32[$103>>2]|0;
   _free($104);
   $105 = $tileno;
   $106 = $0;
   $107 = (($106) + 48|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($108) + ($105<<2)|0);
   HEAP32[$109>>2] = 0;
   $110 = $i;
   $111 = (($110) + 1)|0;
   $i = $111;
  }
 }
 $112 = $0;
 $113 = (($112) + 4|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $114 & 128;
 $116 = ($115|0)!=(0);
 if ($116) {
  $117 = $0;
  $118 = (($117) + 4|0);
  HEAP32[$118>>2] = 160;
  STACKTOP = sp;return;
 } else {
  $119 = $0;
  $120 = (($119) + 4|0);
  HEAP32[$120>>2] = 32;
  STACKTOP = sp;return;
 }
}
function _j2k_read_siz($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cio = 0, $cp = 0, $cstr_info = 0, $h = 0, $i = 0, $image = 0;
 var $len = 0, $tmp = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $0;
 $5 = (($4) + 60|0);
 $6 = HEAP32[$5>>2]|0;
 $image = $6;
 $7 = $0;
 $8 = (($7) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $cp = $9;
 $10 = $cio;
 $11 = (_cio_read($10,2)|0);
 $len = $11;
 $12 = $cio;
 (_cio_read($12,2)|0);
 $13 = $cio;
 $14 = (_cio_read($13,4)|0);
 $15 = $image;
 $16 = (($15) + 8|0);
 HEAP32[$16>>2] = $14;
 $17 = $cio;
 $18 = (_cio_read($17,4)|0);
 $19 = $image;
 $20 = (($19) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = $cio;
 $22 = (_cio_read($21,4)|0);
 $23 = $image;
 HEAP32[$23>>2] = $22;
 $24 = $cio;
 $25 = (_cio_read($24,4)|0);
 $26 = $image;
 $27 = (($26) + 4|0);
 HEAP32[$27>>2] = $25;
 $28 = $cio;
 $29 = (_cio_read($28,4)|0);
 $30 = $cp;
 $31 = (($30) + 56|0);
 HEAP32[$31>>2] = $29;
 $32 = $cio;
 $33 = (_cio_read($32,4)|0);
 $34 = $cp;
 $35 = (($34) + 60|0);
 HEAP32[$35>>2] = $33;
 $36 = $cio;
 $37 = (_cio_read($36,4)|0);
 $38 = $cp;
 $39 = (($38) + 48|0);
 HEAP32[$39>>2] = $37;
 $40 = $cio;
 $41 = (_cio_read($40,4)|0);
 $42 = $cp;
 $43 = (($42) + 52|0);
 HEAP32[$43>>2] = $41;
 $44 = $image;
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)<(0);
 if (!($46)) {
  $47 = $image;
  $48 = (($47) + 8|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($49|0)<(0);
  if (!($50)) {
   $51 = $image;
   $52 = (($51) + 4|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ($53|0)<(0);
   if (!($54)) {
    $55 = $image;
    $56 = (($55) + 12|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)<(0);
    if (!($58)) {
     $72 = $cio;
     $73 = (_cio_read($72,2)|0);
     $74 = $image;
     $75 = (($74) + 16|0);
     HEAP32[$75>>2] = $73;
     $76 = $image;
     $77 = (($76) + 16|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = (_calloc($78,48)|0);
     $80 = $image;
     $81 = (($80) + 24|0);
     HEAP32[$81>>2] = $79;
     $i = 0;
     while(1) {
      $82 = $i;
      $83 = $image;
      $84 = (($83) + 16|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = ($82|0)<($85|0);
      if (!($86)) {
       break;
      }
      $87 = $cio;
      $88 = (_cio_read($87,1)|0);
      $tmp = $88;
      $89 = $tmp;
      $90 = $89 & 127;
      $91 = (($90) + 1)|0;
      $92 = $i;
      $93 = $image;
      $94 = (($93) + 24|0);
      $95 = HEAP32[$94>>2]|0;
      $96 = (($95) + (($92*48)|0)|0);
      $97 = (($96) + 24|0);
      HEAP32[$97>>2] = $91;
      $98 = $tmp;
      $99 = $98 >> 7;
      $100 = $i;
      $101 = $image;
      $102 = (($101) + 24|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = (($103) + (($100*48)|0)|0);
      $105 = (($104) + 32|0);
      HEAP32[$105>>2] = $99;
      $106 = $cio;
      $107 = (_cio_read($106,1)|0);
      $108 = $i;
      $109 = $image;
      $110 = (($109) + 24|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = (($111) + (($108*48)|0)|0);
      HEAP32[$112>>2] = $107;
      $113 = $cio;
      $114 = (_cio_read($113,1)|0);
      $115 = $i;
      $116 = $image;
      $117 = (($116) + 24|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = (($118) + (($115*48)|0)|0);
      $120 = (($119) + 4|0);
      HEAP32[$120>>2] = $114;
      $121 = $image;
      $122 = (($121) + 8|0);
      $123 = HEAP32[$122>>2]|0;
      $124 = $image;
      $125 = HEAP32[$124>>2]|0;
      $126 = (($123) - ($125))|0;
      $127 = $i;
      $128 = $image;
      $129 = (($128) + 24|0);
      $130 = HEAP32[$129>>2]|0;
      $131 = (($130) + (($127*48)|0)|0);
      $132 = HEAP32[$131>>2]|0;
      $133 = (_int_ceildiv($126,$132)|0);
      $w = $133;
      $134 = $image;
      $135 = (($134) + 12|0);
      $136 = HEAP32[$135>>2]|0;
      $137 = $image;
      $138 = (($137) + 4|0);
      $139 = HEAP32[$138>>2]|0;
      $140 = (($136) - ($139))|0;
      $141 = $i;
      $142 = $image;
      $143 = (($142) + 24|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = (($144) + (($141*48)|0)|0);
      $146 = (($145) + 4|0);
      $147 = HEAP32[$146>>2]|0;
      $148 = (_int_ceildiv($140,$147)|0);
      $h = $148;
      $149 = $i;
      $150 = $image;
      $151 = (($150) + 24|0);
      $152 = HEAP32[$151>>2]|0;
      $153 = (($152) + (($149*48)|0)|0);
      $154 = (($153) + 36|0);
      HEAP32[$154>>2] = 0;
      $155 = $cp;
      $156 = (($155) + 36|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = $i;
      $159 = $image;
      $160 = (($159) + 24|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = (($161) + (($158*48)|0)|0);
      $163 = (($162) + 40|0);
      HEAP32[$163>>2] = $157;
      $164 = $i;
      $165 = (($164) + 1)|0;
      $i = $165;
     }
     $166 = $image;
     $167 = (($166) + 8|0);
     $168 = HEAP32[$167>>2]|0;
     $169 = $cp;
     $170 = (($169) + 48|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = (($168) - ($171))|0;
     $173 = $cp;
     $174 = (($173) + 56|0);
     $175 = HEAP32[$174>>2]|0;
     $176 = (_int_ceildiv($172,$175)|0);
     $177 = $cp;
     $178 = (($177) + 68|0);
     HEAP32[$178>>2] = $176;
     $179 = $image;
     $180 = (($179) + 12|0);
     $181 = HEAP32[$180>>2]|0;
     $182 = $cp;
     $183 = (($182) + 52|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = (($181) - ($184))|0;
     $186 = $cp;
     $187 = (($186) + 60|0);
     $188 = HEAP32[$187>>2]|0;
     $189 = (_int_ceildiv($185,$188)|0);
     $190 = $cp;
     $191 = (($190) + 72|0);
     HEAP32[$191>>2] = $189;
     $192 = $cp;
     $193 = (($192) + 68|0);
     $194 = HEAP32[$193>>2]|0;
     $195 = $cp;
     $196 = (($195) + 72|0);
     $197 = HEAP32[$196>>2]|0;
     $198 = Math_imul($194, $197)|0;
     $199 = (_calloc($198,5588)|0);
     $200 = $cp;
     $201 = (($200) + 108|0);
     HEAP32[$201>>2] = $199;
     $202 = $cp;
     $203 = (($202) + 68|0);
     $204 = HEAP32[$203>>2]|0;
     $205 = $cp;
     $206 = (($205) + 72|0);
     $207 = HEAP32[$206>>2]|0;
     $208 = Math_imul($204, $207)|0;
     $209 = $208<<2;
     $210 = (_malloc($209)|0);
     $211 = $cp;
     $212 = (($211) + 76|0);
     HEAP32[$212>>2] = $210;
     $213 = $cp;
     $214 = (($213) + 80|0);
     HEAP32[$214>>2] = 0;
     $i = 0;
     while(1) {
      $215 = $i;
      $216 = $cp;
      $217 = (($216) + 68|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $cp;
      $220 = (($219) + 72|0);
      $221 = HEAP32[$220>>2]|0;
      $222 = Math_imul($218, $221)|0;
      $223 = ($215|0)<($222|0);
      if (!($223)) {
       break;
      }
      $224 = $i;
      $225 = $cp;
      $226 = (($225) + 108|0);
      $227 = HEAP32[$226>>2]|0;
      $228 = (($227) + (($224*5588)|0)|0);
      $229 = (($228) + 424|0);
      HEAP32[$229>>2] = 0;
      $230 = $i;
      $231 = $cp;
      $232 = (($231) + 108|0);
      $233 = HEAP32[$232>>2]|0;
      $234 = (($233) + (($230*5588)|0)|0);
      $235 = (($234) + 420|0);
      HEAP32[$235>>2] = 0;
      $236 = $i;
      $237 = $cp;
      $238 = (($237) + 108|0);
      $239 = HEAP32[$238>>2]|0;
      $240 = (($239) + (($236*5588)|0)|0);
      HEAP32[$240>>2] = 1;
      $241 = $i;
      $242 = (($241) + 1)|0;
      $i = $242;
     }
     $243 = $cp;
     $244 = (($243) + 92|0);
     HEAP32[$244>>2] = 0;
     $245 = $cp;
     $246 = (($245) + 84|0);
     HEAP32[$246>>2] = 0;
     $247 = $cp;
     $248 = (($247) + 88|0);
     HEAP32[$248>>2] = 0;
     $249 = $cp;
     $250 = (($249) + 100|0);
     HEAP32[$250>>2] = 0;
     $251 = $cp;
     $252 = (($251) + 96|0);
     HEAP32[$252>>2] = 0;
     $253 = $image;
     $254 = (($253) + 16|0);
     $255 = HEAP32[$254>>2]|0;
     $256 = (_calloc($255,1076)|0);
     $257 = $0;
     $258 = (($257) + 56|0);
     $259 = HEAP32[$258>>2]|0;
     $260 = (($259) + 5584|0);
     HEAP32[$260>>2] = $256;
     $i = 0;
     while(1) {
      $261 = $i;
      $262 = $cp;
      $263 = (($262) + 68|0);
      $264 = HEAP32[$263>>2]|0;
      $265 = $cp;
      $266 = (($265) + 72|0);
      $267 = HEAP32[$266>>2]|0;
      $268 = Math_imul($264, $267)|0;
      $269 = ($261|0)<($268|0);
      if (!($269)) {
       break;
      }
      $270 = $image;
      $271 = (($270) + 16|0);
      $272 = HEAP32[$271>>2]|0;
      $273 = ($272*1076)|0;
      $274 = (_malloc($273)|0);
      $275 = $i;
      $276 = $cp;
      $277 = (($276) + 108|0);
      $278 = HEAP32[$277>>2]|0;
      $279 = (($278) + (($275*5588)|0)|0);
      $280 = (($279) + 5584|0);
      HEAP32[$280>>2] = $274;
      $281 = $i;
      $282 = (($281) + 1)|0;
      $i = $282;
     }
     $283 = $cp;
     $284 = (($283) + 68|0);
     $285 = HEAP32[$284>>2]|0;
     $286 = $cp;
     $287 = (($286) + 72|0);
     $288 = HEAP32[$287>>2]|0;
     $289 = Math_imul($285, $288)|0;
     $290 = (_calloc($289,4)|0);
     $291 = $0;
     $292 = (($291) + 48|0);
     HEAP32[$292>>2] = $290;
     $293 = $cp;
     $294 = (($293) + 68|0);
     $295 = HEAP32[$294>>2]|0;
     $296 = $cp;
     $297 = (($296) + 72|0);
     $298 = HEAP32[$297>>2]|0;
     $299 = Math_imul($295, $298)|0;
     $300 = (_calloc($299,4)|0);
     $301 = $0;
     $302 = (($301) + 52|0);
     HEAP32[$302>>2] = $300;
     $303 = $0;
     $304 = (($303) + 4|0);
     HEAP32[$304>>2] = 4;
     $305 = $0;
     $306 = (($305) + 68|0);
     $307 = HEAP32[$306>>2]|0;
     $308 = ($307|0)!=(0|0);
     if (!($308)) {
      STACKTOP = sp;return;
     }
     $309 = $0;
     $310 = (($309) + 68|0);
     $311 = HEAP32[$310>>2]|0;
     $cstr_info = $311;
     $312 = $image;
     $313 = (($312) + 8|0);
     $314 = HEAP32[$313>>2]|0;
     $315 = $image;
     $316 = HEAP32[$315>>2]|0;
     $317 = (($314) - ($316))|0;
     $318 = $cstr_info;
     $319 = (($318) + 16|0);
     HEAP32[$319>>2] = $317;
     $320 = $image;
     $321 = (($320) + 12|0);
     $322 = HEAP32[$321>>2]|0;
     $323 = $image;
     $324 = (($323) + 4|0);
     $325 = HEAP32[$324>>2]|0;
     $326 = (($322) - ($325))|0;
     $327 = $cstr_info;
     $328 = (($327) + 20|0);
     HEAP32[$328>>2] = $326;
     $329 = $image;
     $330 = (($329) + 16|0);
     $331 = HEAP32[$330>>2]|0;
     $332 = $cstr_info;
     $333 = (($332) + 52|0);
     HEAP32[$333>>2] = $331;
     $334 = $cp;
     $335 = (($334) + 68|0);
     $336 = HEAP32[$335>>2]|0;
     $337 = $cstr_info;
     $338 = (($337) + 44|0);
     HEAP32[$338>>2] = $336;
     $339 = $cp;
     $340 = (($339) + 72|0);
     $341 = HEAP32[$340>>2]|0;
     $342 = $cstr_info;
     $343 = (($342) + 48|0);
     HEAP32[$343>>2] = $341;
     $344 = $cp;
     $345 = (($344) + 56|0);
     $346 = HEAP32[$345>>2]|0;
     $347 = $cstr_info;
     $348 = (($347) + 28|0);
     HEAP32[$348>>2] = $346;
     $349 = $cp;
     $350 = (($349) + 60|0);
     $351 = HEAP32[$350>>2]|0;
     $352 = $cstr_info;
     $353 = (($352) + 32|0);
     HEAP32[$353>>2] = $351;
     $354 = $cp;
     $355 = (($354) + 48|0);
     $356 = HEAP32[$355>>2]|0;
     $357 = $cstr_info;
     $358 = (($357) + 36|0);
     HEAP32[$358>>2] = $356;
     $359 = $cp;
     $360 = (($359) + 52|0);
     $361 = HEAP32[$360>>2]|0;
     $362 = $cstr_info;
     $363 = (($362) + 40|0);
     HEAP32[$363>>2] = $361;
     $364 = $cp;
     $365 = (($364) + 68|0);
     $366 = HEAP32[$365>>2]|0;
     $367 = $cp;
     $368 = (($367) + 72|0);
     $369 = HEAP32[$368>>2]|0;
     $370 = Math_imul($366, $369)|0;
     $371 = (_calloc($370,576)|0);
     $372 = $cstr_info;
     $373 = (($372) + 88|0);
     HEAP32[$373>>2] = $371;
     STACKTOP = sp;return;
    }
   }
  }
 }
 $59 = $0;
 $60 = HEAP32[$59>>2]|0;
 $61 = $image;
 $62 = HEAP32[$61>>2]|0;
 $63 = $image;
 $64 = (($63) + 8|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = $image;
 $67 = (($66) + 4|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = $image;
 $70 = (($69) + 12|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$vararg_buffer>>2] = $62;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $65;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $68;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $71;
 (_opj_event_msg($60,1,53152,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _j2k_read_cod($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cio = 0, $cp = 0, $cstr_info = 0, $i = 0, $image = 0, $len = 0, $pos = 0, $tcp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $0;
 $5 = (($4) + 64|0);
 $6 = HEAP32[$5>>2]|0;
 $cp = $6;
 $7 = $0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(16);
 if ($10) {
  $11 = $0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $cp;
  $15 = (($14) + 108|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*5588)|0)|0);
  $21 = $17;
 } else {
  $18 = $0;
  $19 = (($18) + 56|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $20;
 }
 $tcp = $21;
 $22 = $0;
 $23 = (($22) + 60|0);
 $24 = HEAP32[$23>>2]|0;
 $image = $24;
 $25 = $cio;
 $26 = (_cio_read($25,2)|0);
 $len = $26;
 $27 = $cio;
 $28 = (_cio_read($27,1)|0);
 $29 = $tcp;
 $30 = (($29) + 4|0);
 HEAP32[$30>>2] = $28;
 $31 = $cio;
 $32 = (_cio_read($31,1)|0);
 $33 = $tcp;
 $34 = (($33) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = $cio;
 $36 = (_cio_read($35,2)|0);
 $37 = $tcp;
 $38 = (($37) + 12|0);
 HEAP32[$38>>2] = $36;
 $39 = $cio;
 $40 = (_cio_read($39,1)|0);
 $41 = $tcp;
 $42 = (($41) + 16|0);
 HEAP32[$42>>2] = $40;
 $43 = $cio;
 $44 = (_cio_tell($43)|0);
 $pos = $44;
 $i = 0;
 while(1) {
  $45 = $i;
  $46 = $image;
  $47 = (($46) + 16|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($45|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = $tcp;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $52 & 1;
  $54 = $i;
  $55 = $tcp;
  $56 = (($55) + 5584|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + (($54*1076)|0)|0);
  HEAP32[$58>>2] = $53;
  $59 = $cio;
  $60 = $pos;
  _cio_seek($59,$60);
  $61 = $0;
  $62 = $i;
  _j2k_read_cox($61,$62);
  $63 = $i;
  $64 = (($63) + 1)|0;
  $i = $64;
 }
 $65 = $0;
 $66 = (($65) + 68|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = ($67|0)!=(0|0);
 if (!($68)) {
  STACKTOP = sp;return;
 }
 $69 = $0;
 $70 = (($69) + 68|0);
 $71 = HEAP32[$70>>2]|0;
 $cstr_info = $71;
 $72 = $tcp;
 $73 = (($72) + 8|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = $cstr_info;
 $76 = (($75) + 24|0);
 HEAP32[$76>>2] = $74;
 $77 = $tcp;
 $78 = (($77) + 12|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = $cstr_info;
 $81 = (($80) + 56|0);
 HEAP32[$81>>2] = $79;
 $82 = $image;
 $83 = (($82) + 16|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = $84<<2;
 $86 = (_malloc($85)|0);
 $87 = $cstr_info;
 $88 = (($87) + 60|0);
 HEAP32[$88>>2] = $86;
 $i = 0;
 while(1) {
  $89 = $i;
  $90 = $image;
  $91 = (($90) + 16|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = ($89|0)<($92|0);
  if (!($93)) {
   break;
  }
  $94 = $i;
  $95 = $tcp;
  $96 = (($95) + 5584|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = (($97) + (($94*1076)|0)|0);
  $99 = (($98) + 4|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = (($100) - 1)|0;
  $102 = $i;
  $103 = $cstr_info;
  $104 = (($103) + 60|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = (($105) + ($102<<2)|0);
  HEAP32[$106>>2] = $101;
  $107 = $i;
  $108 = (($107) + 1)|0;
  $i = $108;
 }
 STACKTOP = sp;return;
}
function _j2k_read_coc($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $cio = 0, $compno = 0, $cp = 0, $image = 0, $len = 0, $tcp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $cp = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(16);
 if ($7) {
  $8 = $0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $cp;
  $12 = (($11) + 108|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + (($10*5588)|0)|0);
  $18 = $14;
 } else {
  $15 = $0;
  $16 = (($15) + 56|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $17;
 }
 $tcp = $18;
 $19 = $0;
 $20 = (($19) + 60|0);
 $21 = HEAP32[$20>>2]|0;
 $image = $21;
 $22 = $0;
 $23 = (($22) + 72|0);
 $24 = HEAP32[$23>>2]|0;
 $cio = $24;
 $25 = $cio;
 $26 = (_cio_read($25,2)|0);
 $len = $26;
 $27 = $cio;
 $28 = $image;
 $29 = (($28) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)<=(256);
 $32 = $31 ? 1 : 2;
 $33 = (_cio_read($27,$32)|0);
 $compno = $33;
 $34 = $cio;
 $35 = (_cio_read($34,1)|0);
 $36 = $compno;
 $37 = $tcp;
 $38 = (($37) + 5584|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + (($36*1076)|0)|0);
 HEAP32[$40>>2] = $35;
 $41 = $0;
 $42 = $compno;
 _j2k_read_cox($41,$42);
 STACKTOP = sp;return;
}
function _j2k_read_rgn($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cio = 0, $compno = 0, $cp = 0, $len = 0, $numcomps = 0, $roisty = 0, $tcp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $cp = $3;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(16);
 if ($7) {
  $8 = $0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $cp;
  $12 = (($11) + 108|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + (($10*5588)|0)|0);
  $18 = $14;
 } else {
  $15 = $0;
  $16 = (($15) + 56|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $17;
 }
 $tcp = $18;
 $19 = $0;
 $20 = (($19) + 72|0);
 $21 = HEAP32[$20>>2]|0;
 $cio = $21;
 $22 = $0;
 $23 = (($22) + 60|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $numcomps = $26;
 $27 = $cio;
 $28 = (_cio_read($27,2)|0);
 $len = $28;
 $29 = $cio;
 $30 = $numcomps;
 $31 = ($30|0)<=(256);
 $32 = $31 ? 1 : 2;
 $33 = (_cio_read($29,$32)|0);
 $compno = $33;
 $34 = $cio;
 $35 = (_cio_read($34,1)|0);
 $roisty = $35;
 $36 = $cio;
 $37 = (_cio_read($36,1)|0);
 $38 = $compno;
 $39 = $tcp;
 $40 = (($39) + 5584|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + (($38*1076)|0)|0);
 $43 = (($42) + 808|0);
 HEAP32[$43>>2] = $37;
 STACKTOP = sp;return;
}
function _j2k_read_qcd($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cio = 0, $i = 0, $image = 0, $len = 0, $pos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $0;
 $5 = (($4) + 60|0);
 $6 = HEAP32[$5>>2]|0;
 $image = $6;
 $7 = $cio;
 $8 = (_cio_read($7,2)|0);
 $len = $8;
 $9 = $cio;
 $10 = (_cio_tell($9)|0);
 $pos = $10;
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $image;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $cio;
  $17 = $pos;
  _cio_seek($16,$17);
  $18 = $0;
  $19 = $i;
  $20 = $len;
  $21 = (($20) - 2)|0;
  _j2k_read_qcx($18,$19,$21);
  $22 = $i;
  $23 = (($22) + 1)|0;
  $i = $23;
 }
 STACKTOP = sp;return;
}
function _j2k_read_qcc($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cio = 0, $compno = 0, $len = 0, $numcomp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $numcomp = $5;
 $6 = $0;
 $7 = (($6) + 72|0);
 $8 = HEAP32[$7>>2]|0;
 $cio = $8;
 $9 = $cio;
 $10 = (_cio_read($9,2)|0);
 $len = $10;
 $11 = $cio;
 $12 = $numcomp;
 $13 = ($12|0)<=(256);
 $14 = $13 ? 1 : 2;
 $15 = (_cio_read($11,$14)|0);
 $compno = $15;
 $16 = $0;
 $17 = $compno;
 $18 = $len;
 $19 = (($18) - 2)|0;
 $20 = $numcomp;
 $21 = ($20|0)<=(256);
 $22 = $21 ? 1 : 2;
 $23 = (($19) - ($22))|0;
 _j2k_read_qcx($16,$17,$23);
 STACKTOP = sp;return;
}
function _j2k_read_poc($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $cio = 0, $cp = 0, $i = 0;
 var $len = 0, $numcomps = 0, $numpchgs = 0, $old_poc = 0, $poc = 0, $tcp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $numcomps = $5;
 $6 = $0;
 $7 = (($6) + 64|0);
 $8 = HEAP32[$7>>2]|0;
 $cp = $8;
 $9 = $0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(16);
 if ($12) {
  $13 = $0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $cp;
  $17 = (($16) + 108|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + (($15*5588)|0)|0);
  $23 = $19;
 } else {
  $20 = $0;
  $21 = (($20) + 56|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $22;
 }
 $tcp = $23;
 $24 = $0;
 $25 = (($24) + 72|0);
 $26 = HEAP32[$25>>2]|0;
 $cio = $26;
 $27 = $tcp;
 $28 = (($27) + 424|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = $tcp;
  $32 = (($31) + 420|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = $34;
 } else {
  $35 = 0;
 }
 $old_poc = $35;
 $36 = $tcp;
 $37 = (($36) + 424|0);
 HEAP32[$37>>2] = 1;
 $38 = $cio;
 $39 = (_cio_read($38,2)|0);
 $len = $39;
 $40 = $len;
 $41 = (($40) - 2)|0;
 $42 = $numcomps;
 $43 = ($42|0)<=(256);
 $44 = $43 ? 1 : 2;
 $45 = $44<<1;
 $46 = (5 + ($45))|0;
 $47 = (($41|0) / ($46|0))&-1;
 $numpchgs = $47;
 $48 = $old_poc;
 $i = $48;
 while(1) {
  $49 = $i;
  $50 = $numpchgs;
  $51 = $old_poc;
  $52 = (($50) + ($51))|0;
  $53 = ($49|0)<($52|0);
  if (!($53)) {
   break;
  }
  $54 = $i;
  $55 = $tcp;
  $56 = (($55) + 428|0);
  $57 = (($56) + (($54*148)|0)|0);
  $poc = $57;
  $58 = $cio;
  $59 = (_cio_read($58,1)|0);
  $60 = $poc;
  HEAP32[$60>>2] = $59;
  $61 = $cio;
  $62 = $numcomps;
  $63 = ($62|0)<=(256);
  $64 = $63 ? 1 : 2;
  $65 = (_cio_read($61,$64)|0);
  $66 = $poc;
  $67 = (($66) + 4|0);
  HEAP32[$67>>2] = $65;
  $68 = $cio;
  $69 = (_cio_read($68,2)|0);
  $70 = $poc;
  $71 = (($70) + 8|0);
  HEAP32[$71>>2] = $69;
  $72 = $cio;
  $73 = (_cio_read($72,1)|0);
  $74 = $poc;
  $75 = (($74) + 12|0);
  HEAP32[$75>>2] = $73;
  $76 = $cio;
  $77 = $numcomps;
  $78 = ($77|0)<=(256);
  $79 = $78 ? 1 : 2;
  $80 = (_cio_read($76,$79)|0);
  $81 = $numcomps;
  $82 = (_int_min10($80,$81)|0);
  $83 = $poc;
  $84 = (($83) + 16|0);
  HEAP32[$84>>2] = $82;
  $85 = $cio;
  $86 = (_cio_read($85,1)|0);
  $87 = $poc;
  $88 = (($87) + 36|0);
  HEAP32[$88>>2] = $86;
  $89 = $i;
  $90 = (($89) + 1)|0;
  $i = $90;
 }
 $91 = $numpchgs;
 $92 = $old_poc;
 $93 = (($91) + ($92))|0;
 $94 = (($93) - 1)|0;
 $95 = $tcp;
 $96 = (($95) + 420|0);
 HEAP32[$96>>2] = $94;
 STACKTOP = sp;return;
}
function _j2k_read_tlm($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $Ptlm_i = 0, $SP = 0, $ST = 0, $Stlm = 0, $Ttlm_i = 0, $Ztlm = 0, $cio = 0, $i = 0, $len = 0, $tile_tlm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $cio;
 $5 = (_cio_read($4,2)|0);
 $len = $5;
 $6 = $cio;
 $7 = (_cio_read($6,1)|0);
 $Ztlm = $7;
 $8 = $cio;
 $9 = (_cio_read($8,1)|0);
 $Stlm = $9;
 $10 = $Stlm;
 $11 = $10 >> 4;
 $12 = $11 & 1;
 $13 = $Stlm;
 $14 = $13 >> 4;
 $15 = $14 & 2;
 $16 = (($12) + ($15))|0;
 $ST = $16;
 $17 = $Stlm;
 $18 = $17 >> 6;
 $19 = $18 & 1;
 $SP = $19;
 $20 = $len;
 $21 = (($20) - 4)|0;
 $22 = $SP;
 $23 = (($22) + 1)|0;
 $24 = $23<<1;
 $25 = $ST;
 $26 = (($24) + ($25))|0;
 $27 = (($21|0) / ($26|0))&-1;
 $tile_tlm = $27;
 $i = 0;
 while(1) {
  $28 = $i;
  $29 = $tile_tlm;
  $30 = ($28|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = $cio;
  $32 = $ST;
  $33 = (_cio_read($31,$32)|0);
  $Ttlm_i = $33;
  $34 = $cio;
  $35 = $SP;
  $36 = ($35|0)!=(0);
  $37 = $36 ? 4 : 2;
  $38 = (_cio_read($34,$37)|0);
  $Ptlm_i = $38;
  $39 = $i;
  $40 = (($39) + 1)|0;
  $i = $40;
 }
 STACKTOP = sp;return;
}
function _j2k_read_plm($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $Nplm = 0, $Zplm = 0, $add = 0, $cio = 0, $i = 0, $len = 0;
 var $packet_len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $packet_len = 0;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $cio;
 $5 = (_cio_read($4,2)|0);
 $len = $5;
 $6 = $cio;
 $7 = (_cio_read($6,1)|0);
 $Zplm = $7;
 $8 = $len;
 $9 = (($8) - 3)|0;
 $len = $9;
 while(1) {
  $10 = $len;
  $11 = ($10|0)>(0);
  if (!($11)) {
   break;
  }
  $12 = $cio;
  $13 = (_cio_read($12,4)|0);
  $Nplm = $13;
  $14 = $len;
  $15 = (($14) - 4)|0;
  $len = $15;
  $16 = $Nplm;
  $i = $16;
  while(1) {
   $17 = $i;
   $18 = ($17|0)>(0);
   if (!($18)) {
    break;
   }
   $19 = $cio;
   $20 = (_cio_read($19,1)|0);
   $add = $20;
   $21 = $len;
   $22 = (($21) + -1)|0;
   $len = $22;
   $23 = $packet_len;
   $24 = $23 << 7;
   $25 = $add;
   $26 = (($24) + ($25))|0;
   $packet_len = $26;
   $27 = $add;
   $28 = $27 & 128;
   $29 = ($28|0)==(0);
   if ($29) {
    $packet_len = 0;
   }
   $30 = $len;
   $31 = ($30|0)<=(0);
   if ($31) {
    label = 8;
    break;
   }
   $32 = $i;
   $33 = (($32) + -1)|0;
   $i = $33;
  }
  if ((label|0) == 8) {
   label = 0;
  }
 }
 STACKTOP = sp;return;
}
function _j2k_read_plt($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $Zplt = 0, $add = 0, $cio = 0, $i = 0, $len = 0, $packet_len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $packet_len = 0;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $cio;
 $5 = (_cio_read($4,2)|0);
 $len = $5;
 $6 = $cio;
 $7 = (_cio_read($6,1)|0);
 $Zplt = $7;
 $8 = $len;
 $9 = (($8) - 3)|0;
 $i = $9;
 while(1) {
  $10 = $i;
  $11 = ($10|0)>(0);
  if (!($11)) {
   break;
  }
  $12 = $cio;
  $13 = (_cio_read($12,1)|0);
  $add = $13;
  $14 = $packet_len;
  $15 = $14 << 7;
  $16 = $add;
  $17 = (($15) + ($16))|0;
  $packet_len = $17;
  $18 = $add;
  $19 = $18 & 128;
  $20 = ($19|0)==(0);
  if ($20) {
   $packet_len = 0;
  }
  $21 = $i;
  $22 = (($21) + -1)|0;
  $i = $22;
 }
 STACKTOP = sp;return;
}
function _j2k_read_ppm($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $N_ppm = 0, $Z_ppm = 0, $cio = 0, $cp = 0;
 var $i = 0, $j = 0, $len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $cp = $3;
 $4 = $0;
 $5 = (($4) + 72|0);
 $6 = HEAP32[$5>>2]|0;
 $cio = $6;
 $7 = $cio;
 $8 = (_cio_read($7,2)|0);
 $len = $8;
 $9 = $cp;
 $10 = (($9) + 92|0);
 HEAP32[$10>>2] = 1;
 $11 = $cio;
 $12 = (_cio_read($11,1)|0);
 $Z_ppm = $12;
 $13 = $len;
 $14 = (($13) - 3)|0;
 $len = $14;
 while(1) {
  $15 = $len;
  $16 = ($15|0)>(0);
  if (!($16)) {
   break;
  }
  $17 = $cp;
  $18 = (($17) + 100|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)==(0);
  if ($20) {
   $21 = $cio;
   $22 = (_cio_read($21,4)|0);
   $N_ppm = $22;
   $23 = $len;
   $24 = (($23) - 4)|0;
   $len = $24;
  } else {
   $25 = $cp;
   $26 = (($25) + 100|0);
   $27 = HEAP32[$26>>2]|0;
   $N_ppm = $27;
  }
  $28 = $cp;
  $29 = (($28) + 96|0);
  $30 = HEAP32[$29>>2]|0;
  $j = $30;
  $31 = $Z_ppm;
  $32 = ($31|0)==(0);
  if ($32) {
   $33 = $N_ppm;
   $34 = $33;
   $35 = (_malloc($34)|0);
   $36 = $cp;
   $37 = (($36) + 84|0);
   HEAP32[$37>>2] = $35;
   $38 = $cp;
   $39 = (($38) + 84|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = $cp;
   $42 = (($41) + 88|0);
   HEAP32[$42>>2] = $40;
   $43 = $N_ppm;
   $44 = $cp;
   $45 = (($44) + 104|0);
   HEAP32[$45>>2] = $43;
  } else {
   $46 = $cp;
   $47 = (($46) + 84|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = $N_ppm;
   $50 = $cp;
   $51 = (($50) + 96|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($49) + ($52))|0;
   $54 = $53;
   $55 = (_realloc($48,$54)|0);
   $56 = $cp;
   $57 = (($56) + 84|0);
   HEAP32[$57>>2] = $55;
   $58 = $cp;
   $59 = (($58) + 84|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $cp;
   $62 = (($61) + 88|0);
   HEAP32[$62>>2] = $60;
   $63 = $N_ppm;
   $64 = $cp;
   $65 = (($64) + 96|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($63) + ($66))|0;
   $68 = $cp;
   $69 = (($68) + 104|0);
   HEAP32[$69>>2] = $67;
  }
  $70 = $N_ppm;
  $i = $70;
  while(1) {
   $71 = $i;
   $72 = ($71|0)>(0);
   if (!($72)) {
    break;
   }
   $73 = $cio;
   $74 = (_cio_read($73,1)|0);
   $75 = $74&255;
   $76 = $j;
   $77 = $cp;
   $78 = (($77) + 84|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76)|0);
   HEAP8[$80>>0] = $75;
   $81 = $j;
   $82 = (($81) + 1)|0;
   $j = $82;
   $83 = $len;
   $84 = (($83) + -1)|0;
   $len = $84;
   $85 = $len;
   $86 = ($85|0)==(0);
   if ($86) {
    label = 12;
    break;
   }
   $87 = $i;
   $88 = (($87) + -1)|0;
   $i = $88;
  }
  if ((label|0) == 12) {
   label = 0;
  }
  $89 = $i;
  $90 = (($89) - 1)|0;
  $91 = $cp;
  $92 = (($91) + 100|0);
  HEAP32[$92>>2] = $90;
  $93 = $j;
  $94 = $cp;
  $95 = (($94) + 96|0);
  HEAP32[$95>>2] = $93;
 }
 STACKTOP = sp;return;
}
function _j2k_read_ppt($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $Z_ppt = 0, $cio = 0, $cp = 0, $i = 0, $j = 0, $len = 0, $tcp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $j = 0;
 $1 = $0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $cp = $3;
 $4 = $cp;
 $5 = (($4) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6) + (($9*5588)|0)|0);
 $tcp = $10;
 $11 = $0;
 $12 = (($11) + 72|0);
 $13 = HEAP32[$12>>2]|0;
 $cio = $13;
 $14 = $cio;
 $15 = (_cio_read($14,2)|0);
 $len = $15;
 $16 = $cio;
 $17 = (_cio_read($16,1)|0);
 $Z_ppt = $17;
 $18 = $tcp;
 $19 = (($18) + 5172|0);
 HEAP32[$19>>2] = 1;
 $20 = $Z_ppt;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = $len;
  $23 = (($22) - 3)|0;
  $24 = $23;
  $25 = (_malloc($24)|0);
  $26 = $tcp;
  $27 = (($26) + 5164|0);
  HEAP32[$27>>2] = $25;
  $28 = $tcp;
  $29 = (($28) + 5164|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $tcp;
  $32 = (($31) + 5168|0);
  HEAP32[$32>>2] = $30;
  $33 = $tcp;
  $34 = (($33) + 5176|0);
  HEAP32[$34>>2] = 0;
  $35 = $len;
  $36 = (($35) - 3)|0;
  $37 = $tcp;
  $38 = (($37) + 5180|0);
  HEAP32[$38>>2] = $36;
 } else {
  $39 = $tcp;
  $40 = (($39) + 5164|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $len;
  $43 = (($42) - 3)|0;
  $44 = $tcp;
  $45 = (($44) + 5176|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($43) + ($46))|0;
  $48 = $47;
  $49 = (_realloc($41,$48)|0);
  $50 = $tcp;
  $51 = (($50) + 5164|0);
  HEAP32[$51>>2] = $49;
  $52 = $tcp;
  $53 = (($52) + 5164|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = $tcp;
  $56 = (($55) + 5168|0);
  HEAP32[$56>>2] = $54;
  $57 = $len;
  $58 = (($57) - 3)|0;
  $59 = $tcp;
  $60 = (($59) + 5176|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($58) + ($61))|0;
  $63 = $tcp;
  $64 = (($63) + 5180|0);
  HEAP32[$64>>2] = $62;
 }
 $65 = $tcp;
 $66 = (($65) + 5176|0);
 $67 = HEAP32[$66>>2]|0;
 $j = $67;
 $68 = $len;
 $69 = (($68) - 3)|0;
 $i = $69;
 while(1) {
  $70 = $i;
  $71 = ($70|0)>(0);
  if (!($71)) {
   break;
  }
  $72 = $cio;
  $73 = (_cio_read($72,1)|0);
  $74 = $73&255;
  $75 = $j;
  $76 = $tcp;
  $77 = (($76) + 5164|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (($78) + ($75)|0);
  HEAP8[$79>>0] = $74;
  $80 = $j;
  $81 = (($80) + 1)|0;
  $j = $81;
  $82 = $i;
  $83 = (($82) + -1)|0;
  $i = $83;
 }
 $84 = $j;
 $85 = $tcp;
 $86 = (($85) + 5176|0);
 HEAP32[$86>>2] = $84;
 STACKTOP = sp;return;
}
function _j2k_read_crg($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $Xcrg_i = 0, $Ycrg_i = 0, $cio = 0, $i = 0, $len = 0, $numcomps = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $0;
 $5 = (($4) + 60|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $numcomps = $8;
 $9 = $cio;
 $10 = (_cio_read($9,2)|0);
 $len = $10;
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $numcomps;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $cio;
  $15 = (_cio_read($14,2)|0);
  $Xcrg_i = $15;
  $16 = $cio;
  $17 = (_cio_read($16,2)|0);
  $Ycrg_i = $17;
  $18 = $i;
  $19 = (($18) + 1)|0;
  $i = $19;
 }
 STACKTOP = sp;return;
}
function _j2k_read_com($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $cio = 0, $len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $j2k;
 $1 = $0;
 $2 = (($1) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $cio = $3;
 $4 = $cio;
 $5 = (_cio_read($4,2)|0);
 $len = $5;
 $6 = $cio;
 $7 = $len;
 $8 = (($7) - 2)|0;
 _cio_skip($6,$8);
 STACKTOP = sp;return;
}
function _j2k_read_unk($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $2 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = $j2k;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 (_opj_event_msg($2,2,52976,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _j2k_create_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j2k = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = (_calloc(1,76)|0);
 $j2k = $2;
 $3 = $j2k;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $0 = 0;
  $18 = $0;
  STACKTOP = sp;return ($18|0);
 }
 $5 = (_calloc(1,5588)|0);
 $6 = $j2k;
 $7 = (($6) + 56|0);
 HEAP32[$7>>2] = $5;
 $8 = $j2k;
 $9 = (($8) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $13 = $1;
  $14 = $j2k;
  HEAP32[$14>>2] = $13;
  $15 = $j2k;
  $16 = (($15) + 48|0);
  HEAP32[$16>>2] = 0;
  $17 = $j2k;
  $0 = $17;
  $18 = $0;
  STACKTOP = sp;return ($18|0);
 } else {
  $12 = $j2k;
  _free($12);
  $0 = 0;
  $18 = $0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _j2k_destroy_decompress($j2k) {
 $j2k = $j2k|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $cp = 0, $default_tcp = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $j2k;
 $i = 0;
 $1 = $0;
 $2 = (($1) + 52|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 52|0);
  $7 = HEAP32[$6>>2]|0;
  _free($7);
 }
 $8 = $0;
 $9 = (($8) + 48|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = $0;
  $13 = (($12) + 48|0);
  $14 = HEAP32[$13>>2]|0;
  _free($14);
 }
 $15 = $0;
 $16 = (($15) + 56|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = $0;
  $20 = (($19) + 56|0);
  $21 = HEAP32[$20>>2]|0;
  $default_tcp = $21;
  $22 = $default_tcp;
  $23 = (($22) + 5168|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = $default_tcp;
   $27 = (($26) + 5168|0);
   $28 = HEAP32[$27>>2]|0;
   _free($28);
  }
  $29 = $0;
  $30 = (($29) + 56|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 5584|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)!=(0|0);
  if ($34) {
   $35 = $0;
   $36 = (($35) + 56|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 5584|0);
   $39 = HEAP32[$38>>2]|0;
   _free($39);
  }
  $40 = $0;
  $41 = (($40) + 56|0);
  $42 = HEAP32[$41>>2]|0;
  _free($42);
 }
 $43 = $0;
 $44 = (($43) + 64|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)!=(0|0);
 if (!($46)) {
  $120 = $0;
  _free($120);
  STACKTOP = sp;return;
 }
 $47 = $0;
 $48 = (($47) + 64|0);
 $49 = HEAP32[$48>>2]|0;
 $cp = $49;
 $50 = $cp;
 $51 = (($50) + 108|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)!=(0|0);
 if ($53) {
  $i = 0;
  while(1) {
   $54 = $i;
   $55 = $cp;
   $56 = (($55) + 68|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = $cp;
   $59 = (($58) + 72|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = Math_imul($57, $60)|0;
   $62 = ($54|0)<($61|0);
   if (!($62)) {
    break;
   }
   $63 = $i;
   $64 = $cp;
   $65 = (($64) + 108|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($66) + (($63*5588)|0)|0);
   $68 = (($67) + 5168|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    $71 = $i;
    $72 = $cp;
    $73 = (($72) + 108|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + (($71*5588)|0)|0);
    $76 = (($75) + 5168|0);
    $77 = HEAP32[$76>>2]|0;
    _free($77);
   }
   $78 = $i;
   $79 = $cp;
   $80 = (($79) + 108|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = (($81) + (($78*5588)|0)|0);
   $83 = (($82) + 5584|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)!=(0|0);
   if ($85) {
    $86 = $i;
    $87 = $cp;
    $88 = (($87) + 108|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = (($89) + (($86*5588)|0)|0);
    $91 = (($90) + 5584|0);
    $92 = HEAP32[$91>>2]|0;
    _free($92);
   }
   $93 = $i;
   $94 = (($93) + 1)|0;
   $i = $94;
  }
  $95 = $cp;
  $96 = (($95) + 108|0);
  $97 = HEAP32[$96>>2]|0;
  _free($97);
 }
 $98 = $cp;
 $99 = (($98) + 88|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100|0)!=(0|0);
 if ($101) {
  $102 = $cp;
  $103 = (($102) + 88|0);
  $104 = HEAP32[$103>>2]|0;
  _free($104);
 }
 $105 = $cp;
 $106 = (($105) + 76|0);
 $107 = HEAP32[$106>>2]|0;
 $108 = ($107|0)!=(0|0);
 if ($108) {
  $109 = $cp;
  $110 = (($109) + 76|0);
  $111 = HEAP32[$110>>2]|0;
  _free($111);
 }
 $112 = $cp;
 $113 = (($112) + 64|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = ($114|0)!=(0|0);
 if ($115) {
  $116 = $cp;
  $117 = (($116) + 64|0);
  $118 = HEAP32[$117>>2]|0;
  _free($118);
 }
 $119 = $cp;
 _free($119);
 $120 = $0;
 _free($120);
 STACKTOP = sp;return;
}
function _j2k_setup_decoder($j2k,$parameters) {
 $j2k = $j2k|0;
 $parameters = $parameters|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $j2k;
 $1 = $parameters;
 $2 = $0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = $1;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = (_calloc(1,116)|0);
 $cp = $6;
 $7 = $1;
 $8 = HEAP32[$7>>2]|0;
 $9 = $cp;
 $10 = (($9) + 36|0);
 HEAP32[$10>>2] = $8;
 $11 = $1;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $cp;
 $15 = (($14) + 40|0);
 HEAP32[$15>>2] = $13;
 $16 = $1;
 $17 = (($16) + 8220|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $cp;
 $20 = (($19) + 44|0);
 HEAP32[$20>>2] = $18;
 $21 = $cp;
 $22 = $0;
 $23 = (($22) + 64|0);
 HEAP32[$23>>2] = $21;
 STACKTOP = sp;return;
}
function _j2k_decode($j2k,$cio,$cstr_info) {
 $j2k = $j2k|0;
 $cio = $cio|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $cinfo = 0, $e = 0, $id = 0, $image = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_buffer8 = 0;
 var $vararg_ptr1 = 0, $vararg_ptr5 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $j2k;
 $2 = $cio;
 $3 = $cstr_info;
 $image = 0;
 $4 = $1;
 $5 = HEAP32[$4>>2]|0;
 $cinfo = $5;
 $6 = $2;
 $7 = $1;
 $8 = (($7) + 72|0);
 HEAP32[$8>>2] = $6;
 $9 = $3;
 $10 = $1;
 $11 = (($10) + 68|0);
 HEAP32[$11>>2] = $9;
 $12 = $3;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = $3;
  dest=$14+0|0; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 }
 $15 = (_opj_image_create0()|0);
 $image = $15;
 $16 = $image;
 $17 = $1;
 $18 = (($17) + 60|0);
 HEAP32[$18>>2] = $16;
 $19 = $1;
 $20 = (($19) + 4|0);
 HEAP32[$20>>2] = 1;
 while(1) {
  $21 = $2;
  $22 = (_cio_read($21,2)|0);
  $id = $22;
  $23 = $id;
  $24 = $23 >> 8;
  $25 = ($24|0)!=(255);
  if ($25) {
   label = 5;
   break;
  }
  $32 = $id;
  $33 = (_j2k_dec_mstab_lookup($32)|0);
  $e = $33;
  $34 = $1;
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $e;
  $38 = (($37) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = $36 & $39;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   label = 7;
   break;
  }
  $48 = $e;
  $49 = HEAP32[$48>>2]|0;
  $50 = ($49|0)==(65424);
  if ($50) {
   $51 = $1;
   $52 = (($51) + 64|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + 44|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)==(1);
   if ($56) {
    label = 10;
    break;
   }
  }
  $59 = $e;
  $60 = (($59) + 8|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($61|0)!=(0|0);
  if ($62) {
   $63 = $e;
   $64 = (($63) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $1;
   FUNCTION_TABLE_vi[$65 & 255]($66);
  }
  $67 = $1;
  $68 = (($67) + 4|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = $69 & 128;
  $71 = ($70|0)!=(0);
  if ($71) {
   label = 14;
   break;
  }
  $72 = $1;
  $73 = (($72) + 4|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = ($74|0)==(32);
  if ($75) {
   label = 16;
   break;
  }
  $76 = $1;
  $77 = (($76) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)==(64);
  if ($79) {
   label = 18;
   break;
  }
 }
 if ((label|0) == 5) {
  $26 = $image;
  _opj_image_destroy($26);
  $27 = $cinfo;
  $28 = $2;
  $29 = (_cio_tell($28)|0);
  $30 = (($29) - 2)|0;
  $31 = $id;
  HEAP32[$vararg_buffer>>2] = $30;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $31;
  (_opj_event_msg($27,1,52752,$vararg_buffer)|0);
  $0 = 0;
  $91 = $0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 7) {
  $42 = $image;
  _opj_image_destroy($42);
  $43 = $cinfo;
  $44 = $2;
  $45 = (_cio_tell($44)|0);
  $46 = (($45) - 2)|0;
  $47 = $id;
  HEAP32[$vararg_buffer2>>2] = $46;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $47;
  (_opj_event_msg($43,1,52792,$vararg_buffer2)|0);
  $0 = 0;
  $91 = $0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 10) {
  $57 = $cinfo;
  (_opj_event_msg($57,4,52824,$vararg_buffer6)|0);
  $58 = $image;
  $0 = $58;
  $91 = $0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 14) {
  $0 = 0;
  $91 = $0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 16) {
 }
 else if ((label|0) == 18) {
 }
 $80 = $1;
 $81 = (($80) + 4|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = ($82|0)==(64);
 if ($83) {
  $84 = $1;
  _j2k_read_eoc($84);
 }
 $85 = $1;
 $86 = (($85) + 4|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = ($87|0)!=(32);
 if ($88) {
  $89 = $cinfo;
  (_opj_event_msg($89,2,52848,$vararg_buffer8)|0);
 }
 $90 = $image;
 $0 = $90;
 $91 = $0;
 STACKTOP = sp;return ($91|0);
}
function _j2k_dec_mstab_lookup($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $e = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $id;
 $e = 52512;
 while(1) {
  $1 = $e;
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)!=(0);
  if (!($3)) {
   label = 7;
   break;
  }
  $4 = $e;
  $5 = HEAP32[$4>>2]|0;
  $6 = $0;
  $7 = ($5|0)==($6|0);
  if ($7) {
   break;
  }
  $8 = $e;
  $9 = (($8) + 12|0);
  $e = $9;
 }
 if ((label|0) == 7) {
  $10 = $e;
  STACKTOP = sp;return ($10|0);
 }
 $10 = $e;
 STACKTOP = sp;return ($10|0);
}
function _j2k_decode_jpt_stream($j2k,$cio,$cstr_info) {
 $j2k = $j2k|0;
 $cio = $cio|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $cinfo = 0, $e = 0, $header = 0, $id = 0, $image = 0, $position = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $header = sp + 52|0;
 $1 = $j2k;
 $2 = $cio;
 $3 = $cstr_info;
 $image = 0;
 $4 = $1;
 $5 = HEAP32[$4>>2]|0;
 $cinfo = $5;
 $6 = $2;
 $7 = $1;
 $8 = (($7) + 72|0);
 HEAP32[$8>>2] = $6;
 $9 = (_opj_image_create0()|0);
 $image = $9;
 $10 = $image;
 $11 = $1;
 $12 = (($11) + 60|0);
 HEAP32[$12>>2] = $10;
 $13 = $1;
 $14 = (($13) + 4|0);
 HEAP32[$14>>2] = 1;
 _jpt_init_msg_header($header);
 $15 = $cinfo;
 $16 = $2;
 _jpt_read_msg_header($15,$16,$header);
 $17 = $2;
 $18 = (_cio_tell($17)|0);
 $position = $18;
 $19 = (($header) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(6);
 if ($21) {
  $22 = $image;
  _opj_image_destroy($22);
  $23 = $cinfo;
  $24 = (($header) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$vararg_buffer>>2] = $25;
  (_opj_event_msg($23,1,52872,$vararg_buffer)|0);
  $0 = 0;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 while(1) {
  $e = 0;
  $26 = $2;
  $27 = (_cio_numbytesleft($26)|0);
  $28 = ($27|0)!=(0);
  if (!($28)) {
   label = 5;
   break;
  }
  $31 = $2;
  $32 = (_cio_tell($31)|0);
  $33 = $position;
  $34 = (($32) - ($33))|0;
  $35 = (($header) + 20|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($34|0)==($36|0);
  if ($37) {
   $38 = $cinfo;
   $39 = $2;
   _jpt_read_msg_header($38,$39,$header);
   $40 = $2;
   $41 = (_cio_tell($40)|0);
   $position = $41;
   $42 = (($header) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)!=(4);
   if ($44) {
    label = 8;
    break;
   }
  }
  $47 = $2;
  $48 = (_cio_read($47,2)|0);
  $id = $48;
  $49 = $id;
  $50 = $49 >> 8;
  $51 = ($50|0)!=(255);
  if ($51) {
   label = 11;
   break;
  }
  $58 = $id;
  $59 = (_j2k_dec_mstab_lookup($58)|0);
  $e = $59;
  $60 = $1;
  $61 = (($60) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = $e;
  $64 = (($63) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = $62 & $65;
  $67 = ($66|0)!=(0);
  if (!($67)) {
   label = 13;
   break;
  }
  $74 = $e;
  $75 = (($74) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = ($76|0)!=(0|0);
  if ($77) {
   $78 = $e;
   $79 = (($78) + 8|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = $1;
   FUNCTION_TABLE_vi[$80 & 255]($81);
  }
  $82 = $1;
  $83 = (($82) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = ($84|0)==(32);
  if ($85) {
   label = 17;
   break;
  }
  $86 = $1;
  $87 = (($86) + 4|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = ($88|0)==(64);
  if ($89) {
   label = 19;
   break;
  }
 }
 if ((label|0) == 5) {
  $29 = $1;
  _j2k_read_eoc($29);
  $30 = $image;
  $0 = $30;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 else if ((label|0) == 8) {
  $45 = $image;
  _opj_image_destroy($45);
  $46 = $cinfo;
  (_opj_event_msg($46,1,52936,$vararg_buffer1)|0);
  $0 = 0;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 else if ((label|0) == 11) {
  $52 = $image;
  _opj_image_destroy($52);
  $53 = $cinfo;
  $54 = $2;
  $55 = (_cio_tell($54)|0);
  $56 = (($55) - 2)|0;
  $57 = $id;
  HEAP32[$vararg_buffer3>>2] = $56;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $57;
  (_opj_event_msg($53,1,52752,$vararg_buffer3)|0);
  $0 = 0;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 else if ((label|0) == 13) {
  $68 = $image;
  _opj_image_destroy($68);
  $69 = $cinfo;
  $70 = $2;
  $71 = (_cio_tell($70)|0);
  $72 = (($71) - 2)|0;
  $73 = $id;
  HEAP32[$vararg_buffer7>>2] = $72;
  $vararg_ptr10 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr10>>2] = $73;
  (_opj_event_msg($69,1,52792,$vararg_buffer7)|0);
  $0 = 0;
  $101 = $0;
  STACKTOP = sp;return ($101|0);
 }
 else if ((label|0) == 17) {
 }
 else if ((label|0) == 19) {
 }
 $90 = $1;
 $91 = (($90) + 4|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = ($92|0)==(64);
 if ($93) {
  $94 = $1;
  _j2k_read_eoc($94);
 }
 $95 = $1;
 $96 = (($95) + 4|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = ($97|0)!=(32);
 if ($98) {
  $99 = $cinfo;
  (_opj_event_msg($99,2,52848,$vararg_buffer11)|0);
 }
 $100 = $image;
 $0 = $100;
 $101 = $0;
 STACKTOP = sp;return ($101|0);
}
function _int_ceildiv($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = (($2) + ($3))|0;
 $5 = (($4) - 1)|0;
 $6 = $1;
 $7 = (($5|0) / ($6|0))&-1;
 STACKTOP = sp;return ($7|0);
}
function _int_min10($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)<($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _j2k_read_qcx($j2k,$compno,$len) {
 $j2k = $j2k|0;
 $compno = $compno|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $bandno = 0, $cio = 0, $cp = 0, $expn = 0, $mant = 0, $numbands = 0, $tccp = 0, $tcp = 0, $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $j2k;
 $1 = $compno;
 $2 = $len;
 $3 = $0;
 $4 = (($3) + 64|0);
 $5 = HEAP32[$4>>2]|0;
 $cp = $5;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(16);
 if ($9) {
  $10 = $0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $cp;
  $14 = (($13) + 108|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*5588)|0)|0);
  $20 = $16;
 } else {
  $17 = $0;
  $18 = (($17) + 56|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19;
 }
 $tcp = $20;
 $21 = $1;
 $22 = $tcp;
 $23 = (($22) + 5584|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + (($21*1076)|0)|0);
 $tccp = $25;
 $26 = $0;
 $27 = (($26) + 72|0);
 $28 = HEAP32[$27>>2]|0;
 $cio = $28;
 $29 = $cio;
 $30 = (_cio_read($29,1)|0);
 $tmp = $30;
 $31 = $tmp;
 $32 = $31 & 31;
 $33 = $tccp;
 $34 = (($33) + 24|0);
 HEAP32[$34>>2] = $32;
 $35 = $tmp;
 $36 = $35 >> 5;
 $37 = $tccp;
 $38 = (($37) + 804|0);
 HEAP32[$38>>2] = $36;
 $39 = $tccp;
 $40 = (($39) + 24|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)==(1);
 if ($42) {
  $52 = 1;
 } else {
  $43 = $tccp;
  $44 = (($43) + 24|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)==(0);
  if ($46) {
   $47 = $2;
   $48 = (($47) - 1)|0;
   $119 = $48;
  } else {
   $49 = $2;
   $50 = (($49) - 1)|0;
   $51 = (($50|0) / 2)&-1;
   $119 = $51;
  }
  $52 = $119;
 }
 $numbands = $52;
 $bandno = 0;
 while(1) {
  $53 = $bandno;
  $54 = $numbands;
  $55 = ($53|0)<($54|0);
  if (!($55)) {
   break;
  }
  $56 = $tccp;
  $57 = (($56) + 24|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ($58|0)==(0);
  if ($59) {
   $60 = $cio;
   $61 = (_cio_read($60,1)|0);
   $62 = $61 >>> 3;
   $expn = $62;
   $mant = 0;
  } else {
   $63 = $cio;
   $64 = (_cio_read($63,2)|0);
   $tmp = $64;
   $65 = $tmp;
   $66 = $65 >> 11;
   $expn = $66;
   $67 = $tmp;
   $68 = $67 & 2047;
   $mant = $68;
  }
  $69 = $expn;
  $70 = $bandno;
  $71 = $tccp;
  $72 = (($71) + 28|0);
  $73 = (($72) + ($70<<3)|0);
  HEAP32[$73>>2] = $69;
  $74 = $mant;
  $75 = $bandno;
  $76 = $tccp;
  $77 = (($76) + 28|0);
  $78 = (($77) + ($75<<3)|0);
  $79 = (($78) + 4|0);
  HEAP32[$79>>2] = $74;
  $80 = $bandno;
  $81 = (($80) + 1)|0;
  $bandno = $81;
 }
 $82 = $tccp;
 $83 = (($82) + 24|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = ($84|0)==(1);
 if (!($85)) {
  STACKTOP = sp;return;
 }
 $bandno = 1;
 while(1) {
  $86 = $bandno;
  $87 = ($86|0)<(97);
  if (!($87)) {
   break;
  }
  $88 = $tccp;
  $89 = (($88) + 28|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = $bandno;
  $92 = (($91) - 1)|0;
  $93 = (($92|0) / 3)&-1;
  $94 = (($90) - ($93))|0;
  $95 = ($94|0)>(0);
  if ($95) {
   $96 = $tccp;
   $97 = (($96) + 28|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = $bandno;
   $100 = (($99) - 1)|0;
   $101 = (($100|0) / 3)&-1;
   $102 = (($98) - ($101))|0;
   $107 = $102;
  } else {
   $107 = 0;
  }
  $103 = $bandno;
  $104 = $tccp;
  $105 = (($104) + 28|0);
  $106 = (($105) + ($103<<3)|0);
  HEAP32[$106>>2] = $107;
  $108 = $tccp;
  $109 = (($108) + 28|0);
  $110 = (($109) + 4|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = $bandno;
  $113 = $tccp;
  $114 = (($113) + 28|0);
  $115 = (($114) + ($112<<3)|0);
  $116 = (($115) + 4|0);
  HEAP32[$116>>2] = $111;
  $117 = $bandno;
  $118 = (($117) + 1)|0;
  $bandno = $118;
 }
 STACKTOP = sp;return;
}
function _j2k_read_cox($j2k,$compno) {
 $j2k = $j2k|0;
 $compno = $compno|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cio = 0, $cp = 0, $i = 0, $tccp = 0, $tcp = 0, $tmp = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = $j2k;
 $1 = $compno;
 $2 = $0;
 $3 = (($2) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $cp = $4;
 $5 = $0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(16);
 if ($8) {
  $9 = $0;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $cp;
  $13 = (($12) + 108|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + (($11*5588)|0)|0);
  $19 = $15;
 } else {
  $16 = $0;
  $17 = (($16) + 56|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $18;
 }
 $tcp = $19;
 $20 = $1;
 $21 = $tcp;
 $22 = (($21) + 5584|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + (($20*1076)|0)|0);
 $tccp = $24;
 $25 = $0;
 $26 = (($25) + 72|0);
 $27 = HEAP32[$26>>2]|0;
 $cio = $27;
 $28 = $cio;
 $29 = (_cio_read($28,1)|0);
 $30 = (($29) + 1)|0;
 $31 = $tccp;
 $32 = (($31) + 4|0);
 HEAP32[$32>>2] = $30;
 $33 = $cp;
 $34 = (($33) + 36|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $tccp;
 $37 = (($36) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($35|0)>=($38|0);
 if ($39) {
  $40 = $0;
  $41 = HEAP32[$40>>2]|0;
  $42 = $1;
  HEAP32[$vararg_buffer>>2] = $42;
  (_opj_event_msg($41,1,52992,$vararg_buffer)|0);
  $43 = $0;
  $44 = (($43) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = $45 | 128;
  HEAP32[$44>>2] = $46;
 }
 $47 = $cio;
 $48 = (_cio_read($47,1)|0);
 $49 = (($48) + 2)|0;
 $50 = $tccp;
 $51 = (($50) + 8|0);
 HEAP32[$51>>2] = $49;
 $52 = $cio;
 $53 = (_cio_read($52,1)|0);
 $54 = (($53) + 2)|0;
 $55 = $tccp;
 $56 = (($55) + 12|0);
 HEAP32[$56>>2] = $54;
 $57 = $cio;
 $58 = (_cio_read($57,1)|0);
 $59 = $tccp;
 $60 = (($59) + 16|0);
 HEAP32[$60>>2] = $58;
 $61 = $cio;
 $62 = (_cio_read($61,1)|0);
 $63 = $tccp;
 $64 = (($63) + 20|0);
 HEAP32[$64>>2] = $62;
 $65 = $tccp;
 $66 = HEAP32[$65>>2]|0;
 $67 = $66 & 1;
 $68 = ($67|0)!=(0);
 if ($68) {
  $i = 0;
  while(1) {
   $69 = $i;
   $70 = $tccp;
   $71 = (($70) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = ($69|0)<($72|0);
   if (!($73)) {
    break;
   }
   $74 = $cio;
   $75 = (_cio_read($74,1)|0);
   $tmp = $75;
   $76 = $tmp;
   $77 = $76 & 15;
   $78 = $i;
   $79 = $tccp;
   $80 = (($79) + 812|0);
   $81 = (($80) + ($78<<2)|0);
   HEAP32[$81>>2] = $77;
   $82 = $tmp;
   $83 = $82 >> 4;
   $84 = $i;
   $85 = $tccp;
   $86 = (($85) + 944|0);
   $87 = (($86) + ($84<<2)|0);
   HEAP32[$87>>2] = $83;
   $88 = $i;
   $89 = (($88) + 1)|0;
   $i = $89;
  }
 }
 $90 = $0;
 $91 = (($90) + 68|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = ($92|0)!=(0|0);
 if (!($93)) {
  STACKTOP = sp;return;
 }
 $94 = $1;
 $95 = ($94|0)==(0);
 if (!($95)) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $96 = $i;
  $97 = $tccp;
  $98 = (($97) + 4|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($96|0)<($99|0);
  if (!($100)) {
   break;
  }
  $101 = $tccp;
  $102 = HEAP32[$101>>2]|0;
  $103 = $102 & 1;
  $104 = ($103|0)!=(0);
  if ($104) {
   $105 = $i;
   $106 = $tccp;
   $107 = (($106) + 812|0);
   $108 = (($107) + ($105<<2)|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = $i;
   $111 = $0;
   $112 = (($111) + 8|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = $0;
   $115 = (($114) + 68|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = (($116) + 88|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (($118) + (($113*576)|0)|0);
   $120 = (($119) + 284|0);
   $121 = (($120) + ($110<<2)|0);
   HEAP32[$121>>2] = $109;
   $122 = $i;
   $123 = $tccp;
   $124 = (($123) + 944|0);
   $125 = (($124) + ($122<<2)|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = $i;
   $128 = $0;
   $129 = (($128) + 8|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = $0;
   $132 = (($131) + 68|0);
   $133 = HEAP32[$132>>2]|0;
   $134 = (($133) + 88|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = (($135) + (($130*576)|0)|0);
   $137 = (($136) + 416|0);
   $138 = (($137) + ($127<<2)|0);
   HEAP32[$138>>2] = $126;
  } else {
   $139 = $i;
   $140 = $0;
   $141 = (($140) + 8|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = $0;
   $144 = (($143) + 68|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = (($145) + 88|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = (($147) + (($142*576)|0)|0);
   $149 = (($148) + 284|0);
   $150 = (($149) + ($139<<2)|0);
   HEAP32[$150>>2] = 15;
   $151 = $i;
   $152 = $0;
   $153 = (($152) + 8|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = $0;
   $156 = (($155) + 68|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (($157) + 88|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = (($159) + (($154*576)|0)|0);
   $161 = (($160) + 284|0);
   $162 = (($161) + ($151<<2)|0);
   HEAP32[$162>>2] = 15;
  }
  $163 = $i;
  $164 = (($163) + 1)|0;
  $i = $164;
 }
 STACKTOP = sp;return;
}
function _opj_clock() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $2 = 0, $3 = 0, $4 = 0.0, $5 = 0.0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $procTime = 0.0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $t = sp + 8|0;
 (_getrusage(0,($t|0))|0);
 $0 = HEAP32[$t>>2]|0;
 $1 = (($t) + 8|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = (($0) + ($2))|0;
 $4 = (+($3|0));
 $procTime = $4;
 $5 = $procTime;
 $6 = (($t) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($t) + 8|0);
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7) + ($10))|0;
 $12 = (+($11|0));
 $13 = $12 * 9.99999999999999954748E-7;
 $14 = $5 + $13;
 STACKTOP = sp;return (+$14);
}
function _jp2_read_jp2h($jp2,$cio,$color) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $box = 0, $cinfo = 0;
 var $jp2h_end = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $box = sp + 12|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $color;
 $4 = $1;
 $5 = HEAP32[$4>>2]|0;
 $cinfo = $5;
 $6 = $cinfo;
 $7 = $2;
 (_jp2_read_boxhdr($6,$7,$box)|0);
 while(1) {
  $8 = (($box) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (1785737832)!=($9|0);
  if ($10) {
   $11 = (($box) + 4|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)==(1785737827);
   if ($13) {
    label = 4;
    break;
   }
   $15 = $2;
   $16 = HEAP32[$box>>2]|0;
   $17 = (($16) - 8)|0;
   _cio_skip($15,$17);
   $18 = $2;
   $19 = (($18) + 24|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $2;
   $22 = (($21) + 20|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($20>>>0)>=($23>>>0);
   if ($24) {
    label = 6;
    break;
   }
   $25 = $cinfo;
   $26 = $2;
   (_jp2_read_boxhdr($25,$26,$box)|0);
  }
  $27 = (($box) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (1785737832)!=($28|0);
  if (!($29)) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 4) {
  $14 = $cinfo;
  (_opj_event_msg($14,1,53208,$vararg_buffer)|0);
  $0 = 0;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 else if ((label|0) == 6) {
  $0 = 0;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 else if ((label|0) == 10) {
  $30 = $1;
  $31 = $2;
  $32 = (_jp2_read_ihdr($30,$31)|0);
  $33 = ($32|0)!=(0);
  if (!($33)) {
   $0 = 0;
   $137 = $0;
   STACKTOP = sp;return ($137|0);
  }
  $34 = (($box) + 8|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$box>>2]|0;
  $37 = (($35) + ($36))|0;
  $jp2h_end = $37;
  $38 = $1;
  $39 = (($38) + 20|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)==(255);
  do {
   if ($41) {
    $42 = $1;
    $43 = $2;
    $44 = (_jp2_read_bpcc($42,$43)|0);
    $45 = ($44|0)!=(0);
    if ($45) {
     break;
    }
    $0 = 0;
    $137 = $0;
    STACKTOP = sp;return ($137|0);
   }
  } while(0);
  $46 = $cinfo;
  $47 = $2;
  (_jp2_read_boxhdr($46,$47,$box)|0);
  while(1) {
   $48 = $2;
   $49 = (_cio_tell($48)|0);
   $50 = $jp2h_end;
   $51 = ($49>>>0)<($50>>>0);
   if (!($51)) {
    break;
   }
   $52 = (($box) + 4|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ($53|0)==(1668246642);
   if ($54) {
    $55 = $1;
    $56 = $2;
    $57 = $3;
    $58 = (_jp2_read_colr($55,$56,$box,$57)|0);
    $59 = ($58|0)!=(0);
    if (!($59)) {
     $60 = $2;
     $61 = (($box) + 8|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = (($62) + 8)|0;
     _cio_seek($60,$63);
     $64 = $2;
     $65 = HEAP32[$box>>2]|0;
     $66 = (($65) - 8)|0;
     _cio_skip($64,$66);
    }
    $67 = $cinfo;
    $68 = $2;
    (_jp2_read_boxhdr($67,$68,$box)|0);
    continue;
   }
   $69 = (($box) + 4|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ($70|0)==(1667523942);
   if ($71) {
    $72 = $1;
    $73 = $2;
    $74 = $3;
    $75 = (_jp2_read_cdef($72,$73,$box,$74)|0);
    $76 = ($75|0)!=(0);
    if (!($76)) {
     $77 = $2;
     $78 = (($box) + 8|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = (($79) + 8)|0;
     _cio_seek($77,$80);
     $81 = $2;
     $82 = HEAP32[$box>>2]|0;
     $83 = (($82) - 8)|0;
     _cio_skip($81,$83);
    }
    $84 = $cinfo;
    $85 = $2;
    (_jp2_read_boxhdr($84,$85,$box)|0);
    continue;
   }
   $86 = (($box) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = ($87|0)==(1885564018);
   if ($88) {
    $89 = $1;
    $90 = $2;
    $91 = $3;
    $92 = (_jp2_read_pclr($89,$90,$box,$91)|0);
    $93 = ($92|0)!=(0);
    if (!($93)) {
     $94 = $2;
     $95 = (($box) + 8|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = (($96) + 8)|0;
     _cio_seek($94,$97);
     $98 = $2;
     $99 = HEAP32[$box>>2]|0;
     $100 = (($99) - 8)|0;
     _cio_skip($98,$100);
    }
    $101 = $cinfo;
    $102 = $2;
    (_jp2_read_boxhdr($101,$102,$box)|0);
    continue;
   }
   $103 = (($box) + 4|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)==(1668112752);
   if (!($105)) {
    $120 = $2;
    $121 = (($box) + 8|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = (($122) + 8)|0;
    _cio_seek($120,$123);
    $124 = $2;
    $125 = HEAP32[$box>>2]|0;
    $126 = (($125) - 8)|0;
    _cio_skip($124,$126);
    $127 = $cinfo;
    $128 = $2;
    (_jp2_read_boxhdr($127,$128,$box)|0);
    continue;
   }
   $106 = $1;
   $107 = $2;
   $108 = $3;
   $109 = (_jp2_read_cmap($106,$107,$box,$108)|0);
   $110 = ($109|0)!=(0);
   if (!($110)) {
    $111 = $2;
    $112 = (($box) + 8|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = (($113) + 8)|0;
    _cio_seek($111,$114);
    $115 = $2;
    $116 = HEAP32[$box>>2]|0;
    $117 = (($116) - 8)|0;
    _cio_skip($115,$117);
   }
   $118 = $cinfo;
   $119 = $2;
   (_jp2_read_boxhdr($118,$119,$box)|0);
  }
  $129 = $2;
  $130 = $jp2h_end;
  _cio_seek($129,$130);
  $131 = $3;
  $132 = (($131) + 16|0);
  $133 = HEAP8[$132>>0]|0;
  $134 = $133&255;
  $135 = ($134|0)==(1);
  $136 = $135&1;
  $0 = $136;
  $137 = $0;
  STACKTOP = sp;return ($137|0);
 }
 return 0|0;
}
function _jp2_read_boxhdr($cinfo,$cio,$box) {
 $cinfo = $cinfo|0;
 $cio = $cio|0;
 $box = $box|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $cinfo;
 $2 = $cio;
 $3 = $box;
 $4 = $2;
 $5 = (_cio_tell($4)|0);
 $6 = $3;
 $7 = (($6) + 8|0);
 HEAP32[$7>>2] = $5;
 $8 = $2;
 $9 = (_cio_read($8,4)|0);
 $10 = $3;
 HEAP32[$10>>2] = $9;
 $11 = $2;
 $12 = (_cio_read($11,4)|0);
 $13 = $3;
 $14 = (($13) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = $3;
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = $2;
  $19 = (_cio_read($18,4)|0);
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = $1;
   (_opj_event_msg($21,1,53568,$vararg_buffer)|0);
   $0 = 0;
   $39 = $0;
   STACKTOP = sp;return ($39|0);
  }
  $22 = $2;
  $23 = (_cio_read($22,4)|0);
  $24 = $3;
  HEAP32[$24>>2] = $23;
  $25 = $3;
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)==(0);
  if ($27) {
   $28 = $2;
   $29 = (_cio_numbytesleft($28)|0);
   $30 = (($29) + 12)|0;
   $31 = $3;
   HEAP32[$31>>2] = $30;
  }
 } else {
  $32 = $3;
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)==(0);
  if ($34) {
   $35 = $2;
   $36 = (_cio_numbytesleft($35)|0);
   $37 = (($36) + 8)|0;
   $38 = $3;
   HEAP32[$38>>2] = $37;
  }
 }
 $0 = 1;
 $39 = $0;
 STACKTOP = sp;return ($39|0);
}
function _jp2_read_ihdr($jp2,$cio) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $box = 0, $cinfo = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $box = sp + 16|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $cinfo = $4;
 $5 = $cinfo;
 $6 = $2;
 (_jp2_read_boxhdr($5,$6,$box)|0);
 $7 = (($box) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (1768449138)!=($8|0);
 if ($9) {
  $10 = $cinfo;
  (_opj_event_msg($10,1,53520,$vararg_buffer)|0);
  $0 = 0;
  $54 = $0;
  STACKTOP = sp;return ($54|0);
 }
 $11 = $2;
 $12 = (_cio_read($11,4)|0);
 $13 = $1;
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = $12;
 $15 = $2;
 $16 = (_cio_read($15,4)|0);
 $17 = $1;
 $18 = (($17) + 8|0);
 HEAP32[$18>>2] = $16;
 $19 = $2;
 $20 = (_cio_read($19,2)|0);
 $21 = $1;
 $22 = (($21) + 16|0);
 HEAP32[$22>>2] = $20;
 $23 = $1;
 $24 = (($23) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25*12)|0;
 $27 = (_malloc($26)|0);
 $28 = $1;
 $29 = (($28) + 68|0);
 HEAP32[$29>>2] = $27;
 $30 = $2;
 $31 = (_cio_read($30,1)|0);
 $32 = $1;
 $33 = (($32) + 20|0);
 HEAP32[$33>>2] = $31;
 $34 = $2;
 $35 = (_cio_read($34,1)|0);
 $36 = $1;
 $37 = (($36) + 24|0);
 HEAP32[$37>>2] = $35;
 $38 = $2;
 $39 = (_cio_read($38,1)|0);
 $40 = $1;
 $41 = (($40) + 28|0);
 HEAP32[$41>>2] = $39;
 $42 = $2;
 $43 = (_cio_read($42,1)|0);
 $44 = $1;
 $45 = (($44) + 32|0);
 HEAP32[$45>>2] = $43;
 $46 = $2;
 $47 = (_cio_tell($46)|0);
 $48 = (($box) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) - ($49))|0;
 $51 = HEAP32[$box>>2]|0;
 $52 = ($50|0)!=($51|0);
 if ($52) {
  $53 = $cinfo;
  (_opj_event_msg($53,1,53544,$vararg_buffer1)|0);
  $0 = 0;
  $54 = $0;
  STACKTOP = sp;return ($54|0);
 } else {
  $0 = 1;
  $54 = $0;
  STACKTOP = sp;return ($54|0);
 }
 return 0|0;
}
function _jp2_read_bpcc($jp2,$cio) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $box = 0, $cinfo = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $box = sp + 16|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $cinfo = $4;
 $5 = $cinfo;
 $6 = $2;
 (_jp2_read_boxhdr($5,$6,$box)|0);
 $7 = (($box) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (1651532643)!=($8|0);
 if ($9) {
  $10 = $cinfo;
  (_opj_event_msg($10,1,53472,$vararg_buffer)|0);
  $0 = 0;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 $i = 0;
 while(1) {
  $11 = $i;
  $12 = $1;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11>>>0)<($14>>>0);
  if (!($15)) {
   break;
  }
  $16 = $2;
  $17 = (_cio_read($16,1)|0);
  $18 = $i;
  $19 = $1;
  $20 = (($19) + 68|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + (($18*12)|0)|0);
  $23 = (($22) + 8|0);
  HEAP32[$23>>2] = $17;
  $24 = $i;
  $25 = (($24) + 1)|0;
  $i = $25;
 }
 $26 = $2;
 $27 = (_cio_tell($26)|0);
 $28 = (($box) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) - ($29))|0;
 $31 = HEAP32[$box>>2]|0;
 $32 = ($30|0)!=($31|0);
 if ($32) {
  $33 = $cinfo;
  (_opj_event_msg($33,1,53496,$vararg_buffer1)|0);
  $0 = 0;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 } else {
  $0 = 1;
  $34 = $0;
  STACKTOP = sp;return ($34|0);
 }
 return 0|0;
}
function _jp2_read_colr($jp2,$cio,$box,$color) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $box = $box|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $cinfo = 0, $skip_len = 0, $start = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $jp2;
 $2 = $cio;
 $3 = $box;
 $4 = $color;
 $5 = $4;
 $6 = (($5) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $0 = 0;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 $9 = $1;
 $10 = HEAP32[$9>>2]|0;
 $cinfo = $10;
 $11 = $2;
 $12 = (_cio_read($11,1)|0);
 $13 = $1;
 $14 = (($13) + 36|0);
 HEAP32[$14>>2] = $12;
 $15 = $2;
 $16 = (_cio_read($15,1)|0);
 $17 = $1;
 $18 = (($17) + 48|0);
 HEAP32[$18>>2] = $16;
 $19 = $2;
 $20 = (_cio_read($19,1)|0);
 $21 = $1;
 $22 = (($21) + 40|0);
 HEAP32[$22>>2] = $20;
 $23 = $1;
 $24 = (($23) + 36|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)==(1);
 if ($26) {
  $27 = $2;
  $28 = (_cio_read($27,4)|0);
  $29 = $1;
  $30 = (($29) + 44|0);
  HEAP32[$30>>2] = $28;
 } else {
  $31 = $3;
  $32 = (($31) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $3;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($33) + ($35))|0;
  $37 = $2;
  $38 = (_cio_tell($37)|0);
  $39 = (($36) - ($38))|0;
  $skip_len = $39;
  $40 = $skip_len;
  $41 = ($40|0)<(0);
  if ($41) {
   $42 = $cinfo;
   (_opj_event_msg($42,1,53416,$vararg_buffer)|0);
   $0 = 0;
   $79 = $0;
   STACKTOP = sp;return ($79|0);
  }
  $43 = $skip_len;
  $44 = ($43|0)>(0);
  if ($44) {
   $45 = $2;
   $46 = (_cio_getbp($45)|0);
   $start = $46;
   $47 = $skip_len;
   $48 = (_malloc($47)|0);
   $49 = $4;
   HEAP32[$49>>2] = $48;
   $50 = $skip_len;
   $51 = $4;
   $52 = (($51) + 4|0);
   HEAP32[$52>>2] = $50;
   $53 = $2;
   $54 = $3;
   $55 = (($54) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = $3;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($56) + ($58))|0;
   $60 = $2;
   $61 = (_cio_tell($60)|0);
   $62 = (($59) - ($61))|0;
   _cio_skip($53,$62);
   $63 = $4;
   $64 = HEAP32[$63>>2]|0;
   $65 = $start;
   $66 = $skip_len;
   _memcpy(($64|0),($65|0),($66|0))|0;
  }
 }
 $67 = $2;
 $68 = (_cio_tell($67)|0);
 $69 = $3;
 $70 = (($69) + 8|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($68) - ($71))|0;
 $73 = $3;
 $74 = HEAP32[$73>>2]|0;
 $75 = ($72|0)!=($74|0);
 if ($75) {
  $76 = $cinfo;
  (_opj_event_msg($76,1,53448,$vararg_buffer1)|0);
  $0 = 0;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 } else {
  $77 = $4;
  $78 = (($77) + 16|0);
  HEAP8[$78>>0] = 1;
  $0 = 1;
  $79 = $0;
  STACKTOP = sp;return ($79|0);
 }
 return 0|0;
}
function _jp2_read_cdef($jp2,$cio,$box,$color) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $box = $box|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $info = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $box;
 $4 = $color;
 $5 = $4;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $0 = 0;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 $9 = $2;
 $10 = (_cio_read($9,2)|0);
 $11 = $10&65535;
 $n = $11;
 $12 = $11&65535;
 $13 = ($12|0)==(0);
 if ($13) {
  $0 = 0;
  $60 = $0;
  STACKTOP = sp;return ($60|0);
 }
 $14 = $n;
 $15 = $14&65535;
 $16 = ($15*6)|0;
 $17 = (_malloc($16)|0);
 $info = $17;
 $18 = (_malloc(8)|0);
 $19 = $4;
 $20 = (($19) + 8|0);
 HEAP32[$20>>2] = $18;
 $21 = $info;
 $22 = $4;
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$24>>2] = $21;
 $25 = $n;
 $26 = $4;
 $27 = (($26) + 8|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 4|0);
 HEAP16[$29>>1] = $25;
 $i = 0;
 while(1) {
  $30 = $i;
  $31 = $30&65535;
  $32 = $n;
  $33 = $32&65535;
  $34 = ($31|0)<($33|0);
  if (!($34)) {
   break;
  }
  $35 = $2;
  $36 = (_cio_read($35,2)|0);
  $37 = $36&65535;
  $38 = $i;
  $39 = $38&65535;
  $40 = $info;
  $41 = (($40) + (($39*6)|0)|0);
  HEAP16[$41>>1] = $37;
  $42 = $2;
  $43 = (_cio_read($42,2)|0);
  $44 = $43&65535;
  $45 = $i;
  $46 = $45&65535;
  $47 = $info;
  $48 = (($47) + (($46*6)|0)|0);
  $49 = (($48) + 2|0);
  HEAP16[$49>>1] = $44;
  $50 = $2;
  $51 = (_cio_read($50,2)|0);
  $52 = $51&65535;
  $53 = $i;
  $54 = $53&65535;
  $55 = $info;
  $56 = (($55) + (($54*6)|0)|0);
  $57 = (($56) + 4|0);
  HEAP16[$57>>1] = $52;
  $58 = $i;
  $59 = (($58) + 1)<<16>>16;
  $i = $59;
 }
 $0 = 1;
 $60 = $0;
 STACKTOP = sp;return ($60|0);
}
function _jp2_read_pclr($jp2,$cio,$box,$color) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $box = $box|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $channel_sign = 0, $channel_size = 0, $entries = 0, $i = 0, $j = 0, $jp2_pclr = 0, $nr_channels = 0, $nr_entries = 0, $uc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $box;
 $4 = $color;
 $5 = $4;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $0 = 0;
  $102 = $0;
  STACKTOP = sp;return ($102|0);
 }
 $9 = $2;
 $10 = (_cio_read($9,2)|0);
 $11 = $10&65535;
 $nr_entries = $11;
 $12 = $2;
 $13 = (_cio_read($12,1)|0);
 $14 = $13&65535;
 $nr_channels = $14;
 $15 = $nr_channels;
 $16 = $15&65535;
 $17 = $nr_entries;
 $18 = $17&65535;
 $19 = Math_imul($16, $18)|0;
 $20 = $19<<2;
 $21 = (_malloc($20)|0);
 $entries = $21;
 $22 = $nr_channels;
 $23 = $22&65535;
 $24 = (_malloc($23)|0);
 $channel_size = $24;
 $25 = $nr_channels;
 $26 = $25&65535;
 $27 = (_malloc($26)|0);
 $channel_sign = $27;
 $28 = (_malloc(20)|0);
 $jp2_pclr = $28;
 $29 = $channel_sign;
 $30 = $jp2_pclr;
 $31 = (($30) + 4|0);
 HEAP32[$31>>2] = $29;
 $32 = $channel_size;
 $33 = $jp2_pclr;
 $34 = (($33) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = $entries;
 $36 = $jp2_pclr;
 HEAP32[$36>>2] = $35;
 $37 = $nr_entries;
 $38 = $jp2_pclr;
 $39 = (($38) + 16|0);
 HEAP16[$39>>1] = $37;
 $40 = $nr_channels;
 $41 = $jp2_pclr;
 $42 = (($41) + 18|0);
 HEAP16[$42>>1] = $40;
 $43 = $jp2_pclr;
 $44 = (($43) + 12|0);
 HEAP32[$44>>2] = 0;
 $45 = $jp2_pclr;
 $46 = $4;
 $47 = (($46) + 12|0);
 HEAP32[$47>>2] = $45;
 $i = 0;
 while(1) {
  $48 = $i;
  $49 = $48&65535;
  $50 = $nr_channels;
  $51 = $50&65535;
  $52 = ($49|0)<($51|0);
  if (!($52)) {
   break;
  }
  $53 = $2;
  $54 = (_cio_read($53,1)|0);
  $55 = $54&255;
  $uc = $55;
  $56 = $uc;
  $57 = $56&255;
  $58 = $57 & 127;
  $59 = (($58) + 1)|0;
  $60 = $59&255;
  $61 = $i;
  $62 = $61&65535;
  $63 = $channel_size;
  $64 = (($63) + ($62)|0);
  HEAP8[$64>>0] = $60;
  $65 = $uc;
  $66 = $65&255;
  $67 = $66 & 128;
  $68 = ($67|0)!=(0);
  $69 = $68 ? 1 : 0;
  $70 = $69&255;
  $71 = $i;
  $72 = $71&65535;
  $73 = $channel_sign;
  $74 = (($73) + ($72)|0);
  HEAP8[$74>>0] = $70;
  $75 = $i;
  $76 = (($75) + 1)<<16>>16;
  $i = $76;
 }
 $j = 0;
 while(1) {
  $77 = $j;
  $78 = $77&65535;
  $79 = $nr_entries;
  $80 = $79&65535;
  $81 = ($78|0)<($80|0);
  if (!($81)) {
   break;
  }
  $i = 0;
  while(1) {
   $82 = $i;
   $83 = $82&65535;
   $84 = $nr_channels;
   $85 = $84&65535;
   $86 = ($83|0)<($85|0);
   if (!($86)) {
    break;
   }
   $87 = $2;
   $88 = $i;
   $89 = $88&65535;
   $90 = $channel_size;
   $91 = (($90) + ($89)|0);
   $92 = HEAP8[$91>>0]|0;
   $93 = $92&255;
   $94 = $93 >> 3;
   $95 = (_cio_read($87,$94)|0);
   $96 = $entries;
   $97 = (($96) + 4|0);
   $entries = $97;
   HEAP32[$96>>2] = $95;
   $98 = $i;
   $99 = (($98) + 1)<<16>>16;
   $i = $99;
  }
  $100 = $j;
  $101 = (($100) + 1)<<16>>16;
  $j = $101;
 }
 $0 = 1;
 $102 = $0;
 STACKTOP = sp;return ($102|0);
}
function _jp2_read_cmap($jp2,$cio,$box,$color) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $box = $box|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $cmap = 0, $i = 0, $nr_channels = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $box;
 $4 = $color;
 $5 = $4;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $0 = 0;
  $59 = $0;
  STACKTOP = sp;return ($59|0);
 }
 $9 = $4;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $0 = 0;
  $59 = $0;
  STACKTOP = sp;return ($59|0);
 }
 $15 = $4;
 $16 = (($15) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 18|0);
 $19 = HEAP16[$18>>1]|0;
 $nr_channels = $19;
 $20 = $nr_channels;
 $21 = $20&65535;
 $22 = $21<<2;
 $23 = (_malloc($22)|0);
 $cmap = $23;
 $i = 0;
 while(1) {
  $24 = $i;
  $25 = $24&65535;
  $26 = $nr_channels;
  $27 = $26&65535;
  $28 = ($25|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = $2;
  $30 = (_cio_read($29,2)|0);
  $31 = $30&65535;
  $32 = $i;
  $33 = $32&65535;
  $34 = $cmap;
  $35 = (($34) + ($33<<2)|0);
  HEAP16[$35>>1] = $31;
  $36 = $2;
  $37 = (_cio_read($36,1)|0);
  $38 = $37&255;
  $39 = $i;
  $40 = $39&65535;
  $41 = $cmap;
  $42 = (($41) + ($40<<2)|0);
  $43 = (($42) + 2|0);
  HEAP8[$43>>0] = $38;
  $44 = $2;
  $45 = (_cio_read($44,1)|0);
  $46 = $45&255;
  $47 = $i;
  $48 = $47&65535;
  $49 = $cmap;
  $50 = (($49) + ($48<<2)|0);
  $51 = (($50) + 3|0);
  HEAP8[$51>>0] = $46;
  $52 = $i;
  $53 = (($52) + 1)<<16>>16;
  $i = $53;
 }
 $54 = $cmap;
 $55 = $4;
 $56 = (($55) + 12|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + 12|0);
 HEAP32[$58>>2] = $54;
 $0 = 1;
 $59 = $0;
 STACKTOP = sp;return ($59|0);
}
function _jp2_decode($jp2,$cio,$cstr_info) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $cinfo = 0, $color = 0, $image = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $color = sp + 12|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $cstr_info;
 $image = 0;
 $4 = $1;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = $2;
  $7 = ($6|0)!=(0|0);
  if ($7) {
   ;HEAP32[$color+0>>2]=0|0;HEAP32[$color+4>>2]=0|0;HEAP32[$color+8>>2]=0|0;HEAP32[$color+12>>2]=0|0;HEAP32[$color+16>>2]=0|0;
   $8 = $1;
   $9 = HEAP32[$8>>2]|0;
   $cinfo = $9;
   $10 = $1;
   $11 = $2;
   $12 = (_jp2_read_struct($10,$11,$color)|0);
   $13 = ($12|0)!=(0);
   if (!($13)) {
    _free_color_data($color);
    $14 = $cinfo;
    (_opj_event_msg($14,1,53232,$vararg_buffer)|0);
    $0 = 0;
    $67 = $0;
    STACKTOP = sp;return ($67|0);
   }
   $15 = $1;
   $16 = (($15) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $2;
   $19 = $3;
   $20 = (_j2k_decode($17,$18,$19)|0);
   $image = $20;
   $21 = $image;
   $22 = ($21|0)!=(0|0);
   if (!($22)) {
    _free_color_data($color);
    $23 = $cinfo;
    (_opj_event_msg($23,1,53264,$vararg_buffer1)|0);
    $0 = 0;
    $67 = $0;
    STACKTOP = sp;return ($67|0);
   }
   $24 = $1;
   $25 = (($24) + 44|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)==(16);
   if ($27) {
    $28 = $image;
    $29 = (($28) + 20|0);
    HEAP32[$29>>2] = 1;
   } else {
    $30 = $1;
    $31 = (($30) + 44|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($32|0)==(17);
    if ($33) {
     $34 = $image;
     $35 = (($34) + 20|0);
     HEAP32[$35>>2] = 2;
    } else {
     $36 = $1;
     $37 = (($36) + 44|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = ($38|0)==(18);
     if ($39) {
      $40 = $image;
      $41 = (($40) + 20|0);
      HEAP32[$41>>2] = 3;
     } else {
      $42 = $image;
      $43 = (($42) + 20|0);
      HEAP32[$43>>2] = -1;
     }
    }
   }
   $44 = (($color) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)!=(0|0);
   if ($46) {
    $47 = $image;
    _jp2_apply_cdef($47,$color);
   }
   $48 = (($color) + 12|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ($49|0)!=(0|0);
   if ($50) {
    $51 = (($color) + 12|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (($52) + 12|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = ($54|0)!=(0|0);
    if ($55) {
     $56 = $image;
     _jp2_apply_pclr($color,$56);
    } else {
     _jp2_free_pclr($color);
    }
   }
   $57 = HEAP32[$color>>2]|0;
   $58 = ($57|0)!=(0|0);
   if ($58) {
    $59 = HEAP32[$color>>2]|0;
    $60 = $image;
    $61 = (($60) + 28|0);
    HEAP32[$61>>2] = $59;
    HEAP32[$color>>2] = 0;
    $62 = (($color) + 4|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = $image;
    $65 = (($64) + 32|0);
    HEAP32[$65>>2] = $63;
   }
   $66 = $image;
   $0 = $66;
   $67 = $0;
   STACKTOP = sp;return ($67|0);
  }
 }
 $0 = 0;
 $67 = $0;
 STACKTOP = sp;return ($67|0);
}
function _jp2_read_struct($jp2,$cio,$color) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $color;
 $4 = $1;
 $5 = $2;
 $6 = (_jp2_read_jp($4,$5)|0);
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = $1;
   $9 = $2;
   $10 = (_jp2_read_ftyp($8,$9)|0);
   $11 = ($10|0)!=(0);
   if (!($11)) {
    $0 = 0;
    break;
   }
   $12 = $1;
   $13 = $2;
   $14 = $3;
   $15 = (_jp2_read_jp2h($12,$13,$14)|0);
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $0 = 0;
    break;
   }
   $17 = $1;
   $18 = $2;
   $19 = $1;
   $20 = (($19) + 76|0);
   $21 = $1;
   $22 = (($21) + 72|0);
   $23 = (_jp2_read_jp2c($17,$18,$20,$22)|0);
   $24 = ($23|0)!=(0);
   if ($24) {
    $0 = 1;
    break;
   } else {
    $0 = 0;
    break;
   }
  } else {
   $0 = 0;
  }
 } while(0);
 $25 = $0;
 STACKTOP = sp;return ($25|0);
}
function _free_color_data($color) {
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $color;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  _jp2_free_pclr($5);
 }
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = $0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = $0;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   _free($18);
  }
  $19 = $0;
  $20 = (($19) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  _free($21);
 }
 $22 = $0;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0|0);
 if (!($24)) {
  STACKTOP = sp;return;
 }
 $25 = $0;
 $26 = HEAP32[$25>>2]|0;
 _free($26);
 STACKTOP = sp;return;
}
function _jp2_apply_cdef($image,$color) {
 $image = $image|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $acn = 0, $asoc = 0, $cn = 0, $color_space = 0, $i = 0, $info = 0, $n = 0, $saved = 0, $typ = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $saved = sp;
 $0 = $image;
 $1 = $color;
 $2 = $0;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $color_space = $4;
 $5 = $1;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $info = $8;
 $9 = $1;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP16[$12>>1]|0;
 $n = $13;
 $i = 0;
 while(1) {
  $14 = $i;
  $15 = $14&65535;
  $16 = $n;
  $17 = $16&65535;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = $i;
  $20 = $19&65535;
  $21 = $info;
  $22 = (($21) + (($20*6)|0)|0);
  $23 = (($22) + 4|0);
  $24 = HEAP16[$23>>1]|0;
  $asoc = $24;
  $25 = $24&65535;
  $26 = ($25|0)==(0);
  if ($26) {
  } else {
   $27 = $i;
   $28 = $27&65535;
   $29 = $info;
   $30 = (($29) + (($28*6)|0)|0);
   $31 = HEAP16[$30>>1]|0;
   $cn = $31;
   $32 = $i;
   $33 = $32&65535;
   $34 = $info;
   $35 = (($34) + (($33*6)|0)|0);
   $36 = (($35) + 2|0);
   $37 = HEAP16[$36>>1]|0;
   $typ = $37;
   $38 = $asoc;
   $39 = $38&65535;
   $40 = (($39) - 1)|0;
   $41 = $40&65535;
   $acn = $41;
   $42 = $cn;
   $43 = $42&65535;
   $44 = $acn;
   $45 = $44&65535;
   $46 = ($43|0)!=($45|0);
   if ($46) {
    $47 = $cn;
    $48 = $47&65535;
    $49 = $0;
    $50 = (($49) + 24|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + (($48*48)|0)|0);
    dest=$saved+0|0; src=$52+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $53 = $cn;
    $54 = $53&65535;
    $55 = $0;
    $56 = (($55) + 24|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + (($54*48)|0)|0);
    $59 = $acn;
    $60 = $59&65535;
    $61 = $0;
    $62 = (($61) + 24|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = (($63) + (($60*48)|0)|0);
    dest=$58+0|0; src=$64+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $65 = $acn;
    $66 = $65&65535;
    $67 = $0;
    $68 = (($67) + 24|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + (($66*48)|0)|0);
    dest=$70+0|0; src=$saved+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $71 = $cn;
    $72 = $71&65535;
    $73 = (($72) + 1)|0;
    $74 = $73&65535;
    $75 = $i;
    $76 = $75&65535;
    $77 = $info;
    $78 = (($77) + (($76*6)|0)|0);
    $79 = (($78) + 4|0);
    HEAP16[$79>>1] = $74;
    $80 = $acn;
    $81 = $80&65535;
    $82 = $info;
    $83 = (($82) + (($81*6)|0)|0);
    $84 = HEAP16[$83>>1]|0;
    $85 = $84&65535;
    $86 = (($85) + 1)|0;
    $87 = $86&65535;
    $88 = $acn;
    $89 = $88&65535;
    $90 = $info;
    $91 = (($90) + (($89*6)|0)|0);
    $92 = (($91) + 4|0);
    HEAP16[$92>>1] = $87;
   }
  }
  $93 = $i;
  $94 = (($93) + 1)<<16>>16;
  $i = $94;
 }
 $95 = $1;
 $96 = (($95) + 8|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = HEAP32[$97>>2]|0;
 $99 = ($98|0)!=(0|0);
 if (!($99)) {
  $104 = $1;
  $105 = (($104) + 8|0);
  $106 = HEAP32[$105>>2]|0;
  _free($106);
  $107 = $1;
  $108 = (($107) + 8|0);
  HEAP32[$108>>2] = 0;
  STACKTOP = sp;return;
 }
 $100 = $1;
 $101 = (($100) + 8|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = HEAP32[$102>>2]|0;
 _free($103);
 $104 = $1;
 $105 = (($104) + 8|0);
 $106 = HEAP32[$105>>2]|0;
 _free($106);
 $107 = $1;
 $108 = (($107) + 8|0);
 HEAP32[$108>>2] = 0;
 STACKTOP = sp;return;
}
function _jp2_free_pclr($color) {
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $color;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _free($5);
 $6 = $0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 _free($10);
 $11 = $0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 _free($14);
 $15 = $0;
 $16 = (($15) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = $0;
  $22 = (($21) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 12|0);
  $25 = HEAP32[$24>>2]|0;
  _free($25);
 }
 $26 = $0;
 $27 = (($26) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 _free($28);
 $29 = $0;
 $30 = (($29) + 12|0);
 HEAP32[$30>>2] = 0;
 STACKTOP = sp;return;
}
function _jp2_apply_pclr($color,$image) {
 $color = $color|0;
 $image = $image|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $channel_sign = 0, $channel_size = 0, $cmap = 0, $cmp = 0, $dst = 0, $entries = 0, $i = 0, $j = 0;
 var $k = 0, $max = 0, $new_comps = 0, $nr_channels = 0, $old_comps = 0, $pcol = 0, $src = 0, $top_k = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $color;
 $1 = $image;
 $2 = $0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $channel_size = $6;
 $7 = $0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $channel_sign = $11;
 $12 = $0;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $entries = $15;
 $16 = $0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $cmap = $20;
 $21 = $0;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 18|0);
 $25 = HEAP16[$24>>1]|0;
 $nr_channels = $25;
 $26 = $1;
 $27 = (($26) + 24|0);
 $28 = HEAP32[$27>>2]|0;
 $old_comps = $28;
 $29 = $nr_channels;
 $30 = $29&65535;
 $31 = ($30*48)|0;
 $32 = (_malloc($31)|0);
 $new_comps = $32;
 $i = 0;
 while(1) {
  $33 = $i;
  $34 = $33&65535;
  $35 = $nr_channels;
  $36 = $35&65535;
  $37 = ($34|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = $i;
  $39 = $38&65535;
  $40 = $cmap;
  $41 = (($40) + ($39<<2)|0);
  $42 = (($41) + 3|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $pcol = $44;
  $45 = $i;
  $46 = $45&65535;
  $47 = $cmap;
  $48 = (($47) + ($46<<2)|0);
  $49 = HEAP16[$48>>1]|0;
  $cmp = $49;
  $50 = $pcol;
  $51 = $50&65535;
  $52 = $new_comps;
  $53 = (($52) + (($51*48)|0)|0);
  $54 = $cmp;
  $55 = $54&65535;
  $56 = $old_comps;
  $57 = (($56) + (($55*48)|0)|0);
  dest=$53+0|0; src=$57+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $58 = $i;
  $59 = $58&65535;
  $60 = $cmap;
  $61 = (($60) + ($59<<2)|0);
  $62 = (($61) + 2|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = $63&255;
  $65 = ($64|0)==(0);
  if ($65) {
   $66 = $cmp;
   $67 = $66&65535;
   $68 = $old_comps;
   $69 = (($68) + (($67*48)|0)|0);
   $70 = (($69) + 44|0);
   HEAP32[$70>>2] = 0;
  } else {
   $71 = $cmp;
   $72 = $71&65535;
   $73 = $old_comps;
   $74 = (($73) + (($72*48)|0)|0);
   $75 = (($74) + 8|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = $cmp;
   $78 = $77&65535;
   $79 = $old_comps;
   $80 = (($79) + (($78*48)|0)|0);
   $81 = (($80) + 12|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = Math_imul($76, $82)|0;
   $84 = $83<<2;
   $85 = (_malloc($84)|0);
   $86 = $pcol;
   $87 = $86&65535;
   $88 = $new_comps;
   $89 = (($88) + (($87*48)|0)|0);
   $90 = (($89) + 44|0);
   HEAP32[$90>>2] = $85;
   $91 = $i;
   $92 = $91&65535;
   $93 = $channel_size;
   $94 = (($93) + ($92)|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = $95&255;
   $97 = $pcol;
   $98 = $97&65535;
   $99 = $new_comps;
   $100 = (($99) + (($98*48)|0)|0);
   $101 = (($100) + 24|0);
   HEAP32[$101>>2] = $96;
   $102 = $i;
   $103 = $102&65535;
   $104 = $channel_sign;
   $105 = (($104) + ($103)|0);
   $106 = HEAP8[$105>>0]|0;
   $107 = $106&255;
   $108 = $pcol;
   $109 = $108&65535;
   $110 = $new_comps;
   $111 = (($110) + (($109*48)|0)|0);
   $112 = (($111) + 32|0);
   HEAP32[$112>>2] = $107;
  }
  $113 = $i;
  $114 = (($113) + 1)<<16>>16;
  $i = $114;
 }
 $115 = $0;
 $116 = (($115) + 12|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = (($117) + 16|0);
 $119 = HEAP16[$118>>1]|0;
 $120 = $119&65535;
 $121 = (($120) - 1)|0;
 $top_k = $121;
 $i = 0;
 while(1) {
  $122 = $i;
  $123 = $122&65535;
  $124 = $nr_channels;
  $125 = $124&65535;
  $126 = ($123|0)<($125|0);
  if (!($126)) {
   break;
  }
  $127 = $i;
  $128 = $127&65535;
  $129 = $cmap;
  $130 = (($129) + ($128<<2)|0);
  $131 = (($130) + 2|0);
  $132 = HEAP8[$131>>0]|0;
  $133 = $132&255;
  $134 = ($133|0)==(0);
  if ($134) {
  } else {
   $135 = $i;
   $136 = $135&65535;
   $137 = $cmap;
   $138 = (($137) + ($136<<2)|0);
   $139 = HEAP16[$138>>1]|0;
   $cmp = $139;
   $140 = $i;
   $141 = $140&65535;
   $142 = $cmap;
   $143 = (($142) + ($141<<2)|0);
   $144 = (($143) + 3|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = $145&255;
   $pcol = $146;
   $147 = $cmp;
   $148 = $147&65535;
   $149 = $old_comps;
   $150 = (($149) + (($148*48)|0)|0);
   $151 = (($150) + 44|0);
   $152 = HEAP32[$151>>2]|0;
   $src = $152;
   $153 = $pcol;
   $154 = $153&65535;
   $155 = $new_comps;
   $156 = (($155) + (($154*48)|0)|0);
   $157 = (($156) + 44|0);
   $158 = HEAP32[$157>>2]|0;
   $dst = $158;
   $159 = $pcol;
   $160 = $159&65535;
   $161 = $new_comps;
   $162 = (($161) + (($160*48)|0)|0);
   $163 = (($162) + 8|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = $pcol;
   $166 = $165&65535;
   $167 = $new_comps;
   $168 = (($167) + (($166*48)|0)|0);
   $169 = (($168) + 12|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = Math_imul($164, $170)|0;
   $max = $171;
   $j = 0;
   while(1) {
    $172 = $j;
    $173 = $max;
    $174 = ($172>>>0)<($173>>>0);
    if (!($174)) {
     break;
    }
    $175 = $j;
    $176 = $src;
    $177 = (($176) + ($175<<2)|0);
    $178 = HEAP32[$177>>2]|0;
    $k = $178;
    $179 = ($178|0)<(0);
    if ($179) {
     $k = 0;
    } else {
     $180 = $k;
     $181 = $top_k;
     $182 = ($180|0)>($181|0);
     if ($182) {
      $183 = $top_k;
      $k = $183;
     }
    }
    $184 = $k;
    $185 = $nr_channels;
    $186 = $185&65535;
    $187 = Math_imul($184, $186)|0;
    $188 = $pcol;
    $189 = $188&65535;
    $190 = (($187) + ($189))|0;
    $191 = $entries;
    $192 = (($191) + ($190<<2)|0);
    $193 = HEAP32[$192>>2]|0;
    $194 = $j;
    $195 = $dst;
    $196 = (($195) + ($194<<2)|0);
    HEAP32[$196>>2] = $193;
    $197 = $j;
    $198 = (($197) + 1)|0;
    $j = $198;
   }
  }
  $199 = $i;
  $200 = (($199) + 1)<<16>>16;
  $i = $200;
 }
 $201 = $1;
 $202 = (($201) + 16|0);
 $203 = HEAP32[$202>>2]|0;
 $max = $203;
 $i = 0;
 while(1) {
  $204 = $i;
  $205 = $204&65535;
  $206 = $max;
  $207 = ($205>>>0)<($206>>>0);
  if (!($207)) {
   break;
  }
  $208 = $i;
  $209 = $208&65535;
  $210 = $old_comps;
  $211 = (($210) + (($209*48)|0)|0);
  $212 = (($211) + 44|0);
  $213 = HEAP32[$212>>2]|0;
  $214 = ($213|0)!=(0|0);
  if ($214) {
   $215 = $i;
   $216 = $215&65535;
   $217 = $old_comps;
   $218 = (($217) + (($216*48)|0)|0);
   $219 = (($218) + 44|0);
   $220 = HEAP32[$219>>2]|0;
   _free($220);
  }
  $221 = $i;
  $222 = (($221) + 1)<<16>>16;
  $i = $222;
 }
 $223 = $old_comps;
 _free($223);
 $224 = $new_comps;
 $225 = $1;
 $226 = (($225) + 24|0);
 HEAP32[$226>>2] = $224;
 $227 = $nr_channels;
 $228 = $227&65535;
 $229 = $1;
 $230 = (($229) + 16|0);
 HEAP32[$230>>2] = $228;
 $231 = $0;
 _jp2_free_pclr($231);
 STACKTOP = sp;return;
}
function _jp2_create_decompress($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $jp2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = (_calloc(1,80)|0);
 $jp2 = $2;
 $3 = $jp2;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = $1;
   $6 = $jp2;
   HEAP32[$6>>2] = $5;
   $7 = $1;
   $8 = (_j2k_create_decompress($7)|0);
   $9 = $jp2;
   $10 = (($9) + 4|0);
   HEAP32[$10>>2] = $8;
   $11 = $jp2;
   $12 = (($11) + 4|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if (!($14)) {
    break;
   }
   $15 = $jp2;
   _jp2_destroy_decompress($15);
   $0 = 0;
   $17 = $0;
   STACKTOP = sp;return ($17|0);
  }
 } while(0);
 $16 = $jp2;
 $0 = $16;
 $17 = $0;
 STACKTOP = sp;return ($17|0);
}
function _jp2_destroy_decompress($jp2) {
 $jp2 = $jp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $jp2;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _j2k_destroy_decompress($5);
 $6 = $0;
 $7 = (($6) + 68|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = $0;
  $11 = (($10) + 68|0);
  $12 = HEAP32[$11>>2]|0;
  _free($12);
 }
 $13 = $0;
 $14 = (($13) + 64|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = $0;
  $18 = (($17) + 64|0);
  $19 = HEAP32[$18>>2]|0;
  _free($19);
 }
 $20 = $0;
 _free($20);
 STACKTOP = sp;return;
}
function _jp2_setup_decoder($jp2,$parameters) {
 $jp2 = $jp2|0;
 $parameters = $parameters|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $jp2;
 $1 = $parameters;
 $2 = $0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $1;
 _j2k_setup_decoder($4,$5);
 STACKTOP = sp;return;
}
function _jp2_read_jp($jp2,$cio) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $box = 0, $cinfo = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $box = sp + 24|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $cinfo = $4;
 $5 = $cinfo;
 $6 = $2;
 (_jp2_read_boxhdr($5,$6,$box)|0);
 $7 = (($box) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (1783636000)!=($8|0);
 if ($9) {
  $10 = $cinfo;
  (_opj_event_msg($10,1,53344,$vararg_buffer)|0);
  $0 = 0;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 $11 = $2;
 $12 = (_cio_read($11,4)|0);
 $13 = (218793738)!=($12|0);
 if ($13) {
  $14 = $cinfo;
  (_opj_event_msg($14,1,53368,$vararg_buffer1)|0);
  $0 = 0;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 $15 = $2;
 $16 = (_cio_tell($15)|0);
 $17 = (($box) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) - ($18))|0;
 $20 = HEAP32[$box>>2]|0;
 $21 = ($19|0)!=($20|0);
 if ($21) {
  $22 = $cinfo;
  (_opj_event_msg($22,1,53392,$vararg_buffer3)|0);
  $0 = 0;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 } else {
  $0 = 1;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _jp2_read_ftyp($jp2,$cio) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $box = 0, $cinfo = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $box = sp + 16|0;
 $1 = $jp2;
 $2 = $cio;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $cinfo = $4;
 $5 = $cinfo;
 $6 = $2;
 (_jp2_read_boxhdr($5,$6,$box)|0);
 $7 = (($box) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (1718909296)!=($8|0);
 if ($9) {
  $10 = $cinfo;
  (_opj_event_msg($10,1,53296,$vararg_buffer)|0);
  $0 = 0;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 $11 = $2;
 $12 = (_cio_read($11,4)|0);
 $13 = $1;
 $14 = (($13) + 52|0);
 HEAP32[$14>>2] = $12;
 $15 = $2;
 $16 = (_cio_read($15,4)|0);
 $17 = $1;
 $18 = (($17) + 56|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$box>>2]|0;
 $20 = (($19) - 16)|0;
 $21 = (($20|0) / 4)&-1;
 $22 = $1;
 $23 = (($22) + 60|0);
 HEAP32[$23>>2] = $21;
 $24 = $1;
 $25 = (($24) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $26<<2;
 $28 = (_malloc($27)|0);
 $29 = $1;
 $30 = (($29) + 64|0);
 HEAP32[$30>>2] = $28;
 $i = 0;
 while(1) {
  $31 = $i;
  $32 = $1;
  $33 = (($32) + 60|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($31|0)<($34|0);
  if (!($35)) {
   break;
  }
  $36 = $2;
  $37 = (_cio_read($36,4)|0);
  $38 = $i;
  $39 = $1;
  $40 = (($39) + 64|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($38<<2)|0);
  HEAP32[$42>>2] = $37;
  $43 = $i;
  $44 = (($43) + 1)|0;
  $i = $44;
 }
 $45 = $2;
 $46 = (_cio_tell($45)|0);
 $47 = (($box) + 8|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) - ($48))|0;
 $50 = HEAP32[$box>>2]|0;
 $51 = ($49|0)!=($50|0);
 if ($51) {
  $52 = $cinfo;
  (_opj_event_msg($52,1,53320,$vararg_buffer1)|0);
  $0 = 0;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 } else {
  $0 = 1;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 return 0|0;
}
function _jp2_read_jp2c($jp2,$cio,$j2k_codestream_length,$j2k_codestream_offset) {
 $jp2 = $jp2|0;
 $cio = $cio|0;
 $j2k_codestream_length = $j2k_codestream_length|0;
 $j2k_codestream_offset = $j2k_codestream_offset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $box = 0, $cinfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $box = sp + 4|0;
 $0 = $jp2;
 $1 = $cio;
 $2 = $j2k_codestream_length;
 $3 = $j2k_codestream_offset;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $cinfo = $5;
 $6 = $cinfo;
 $7 = $1;
 (_jp2_read_boxhdr($6,$7,$box)|0);
 while(1) {
  $8 = (($box) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (1785737827)!=($9|0);
  if ($10) {
   $11 = $1;
   $12 = HEAP32[$box>>2]|0;
   $13 = (($12) - 8)|0;
   _cio_skip($11,$13);
   $14 = $cinfo;
   $15 = $1;
   (_jp2_read_boxhdr($14,$15,$box)|0);
  }
  $16 = (($box) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (1785737827)!=($17|0);
  if (!($18)) {
   break;
  }
 }
 $19 = $1;
 $20 = (_cio_tell($19)|0);
 $21 = $3;
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[$box>>2]|0;
 $23 = (($22) - 8)|0;
 $24 = $2;
 HEAP32[$24>>2] = $23;
 STACKTOP = sp;return 1;
}
function _jpt_read_VBAS_info($cio,$value) {
 $cio = $cio|0;
 $value = $value|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $cio;
 $1 = $value;
 $2 = $0;
 $3 = (_cio_read($2,1)|0);
 $4 = $3&255;
 $elmt = $4;
 while(1) {
  $5 = $elmt;
  $6 = $5&255;
  $7 = $6 >> 7;
  $8 = ($7|0)==(1);
  if (!($8)) {
   break;
  }
  $9 = $1;
  $10 = $9 << 7;
  $1 = $10;
  $11 = $elmt;
  $12 = $11&255;
  $13 = $12 & 127;
  $14 = $1;
  $15 = $14 | $13;
  $1 = $15;
  $16 = $0;
  $17 = (_cio_read($16,1)|0);
  $18 = $17&255;
  $elmt = $18;
 }
 $19 = $1;
 $20 = $19 << 7;
 $1 = $20;
 $21 = $elmt;
 $22 = $21&255;
 $23 = $22 & 127;
 $24 = $1;
 $25 = $24 | $23;
 $1 = $25;
 $26 = $1;
 STACKTOP = sp;return ($26|0);
}
function _jpt_init_msg_header($header) {
 $header = $header|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $header;
 $1 = $0;
 HEAP32[$1>>2] = 0;
 $2 = $0;
 $3 = (($2) + 4|0);
 HEAP32[$3>>2] = 0;
 $4 = $0;
 $5 = (($4) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = $0;
 $9 = (($8) + 16|0);
 HEAP32[$9>>2] = 0;
 $10 = $0;
 $11 = (($10) + 20|0);
 HEAP32[$11>>2] = 0;
 $12 = $0;
 $13 = (($12) + 24|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _jpt_reinit_msg_header($header) {
 $header = $header|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $header;
 $1 = $0;
 HEAP32[$1>>2] = 0;
 $2 = $0;
 $3 = (($2) + 4|0);
 HEAP32[$3>>2] = 0;
 $4 = $0;
 $5 = (($4) + 16|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 $7 = (($6) + 20|0);
 HEAP32[$7>>2] = 0;
 STACKTOP = sp;return;
}
function _jpt_read_msg_header($cinfo,$cio,$header) {
 $cinfo = $cinfo|0;
 $cio = $cio|0;
 $header = $header|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $CSn = 0, $Class = 0, $elmt = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = $cinfo;
 $1 = $cio;
 $2 = $header;
 $Class = 0;
 $CSn = 0;
 $3 = $2;
 _jpt_reinit_msg_header($3);
 $4 = $1;
 $5 = (_cio_read($4,1)|0);
 $6 = $5&255;
 $elmt = $6;
 $7 = $elmt;
 $8 = $7&255;
 $9 = $8 >> 5;
 $10 = $9 & 3;
 if ((($10|0) == 2)) {
  $Class = 1;
  $CSn = 0;
 } else if ((($10|0) == 3)) {
  $Class = 1;
  $CSn = 1;
 } else if ((($10|0) == 0)) {
  $11 = $0;
  (_opj_event_msg($11,1,53616,$vararg_buffer)|0);
 } else if ((($10|0) == 1)) {
  $Class = 0;
  $CSn = 0;
 } else {
 }
 $12 = $elmt;
 $13 = $12&255;
 $14 = $13 >> 4;
 $15 = $14 & 1;
 $16 = ($15|0)==(1);
 if ($16) {
  $17 = $2;
  $18 = (($17) + 4|0);
  HEAP32[$18>>2] = 1;
 }
 $19 = $elmt;
 $20 = $19&255;
 $21 = $20 & 15;
 $22 = $2;
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 | $21;
 HEAP32[$22>>2] = $24;
 $25 = $elmt;
 $26 = $25&255;
 $27 = $26 >> 7;
 $28 = ($27|0)==(1);
 if ($28) {
  $29 = $1;
  $30 = $2;
  $31 = HEAP32[$30>>2]|0;
  $32 = (_jpt_read_VBAS_info($29,$31)|0);
  $33 = $2;
  HEAP32[$33>>2] = $32;
 }
 $34 = $Class;
 $35 = $34&255;
 $36 = ($35|0)==(1);
 if ($36) {
  $37 = $2;
  $38 = (($37) + 8|0);
  HEAP32[$38>>2] = 0;
  $39 = $1;
  $40 = $2;
  $41 = (($40) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_jpt_read_VBAS_info($39,$42)|0);
  $44 = $2;
  $45 = (($44) + 8|0);
  HEAP32[$45>>2] = $43;
 }
 $46 = $CSn;
 $47 = $46&255;
 $48 = ($47|0)==(1);
 if ($48) {
  $49 = $2;
  $50 = (($49) + 12|0);
  HEAP32[$50>>2] = 0;
  $51 = $1;
  $52 = $2;
  $53 = (($52) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_jpt_read_VBAS_info($51,$54)|0);
  $56 = $2;
  $57 = (($56) + 12|0);
  HEAP32[$57>>2] = $55;
 }
 $58 = $1;
 $59 = $2;
 $60 = (($59) + 16|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (_jpt_read_VBAS_info($58,$61)|0);
 $63 = $2;
 $64 = (($63) + 16|0);
 HEAP32[$64>>2] = $62;
 $65 = $1;
 $66 = $2;
 $67 = (($66) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = (_jpt_read_VBAS_info($65,$68)|0);
 $70 = $2;
 $71 = (($70) + 20|0);
 HEAP32[$71>>2] = $69;
 $72 = $2;
 $73 = (($72) + 8|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = $74 & 1;
 $76 = ($75|0)==(1);
 if (!($76)) {
  STACKTOP = sp;return;
 }
 $77 = $2;
 $78 = (($77) + 24|0);
 HEAP32[$78>>2] = 0;
 $79 = $1;
 $80 = $2;
 $81 = (($80) + 24|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = (_jpt_read_VBAS_info($79,$82)|0);
 $84 = $2;
 $85 = (($84) + 24|0);
 HEAP32[$85>>2] = $83;
 STACKTOP = sp;return;
}
function _mct_decode($c0,$c1,$c2,$n) {
 $c0 = $c0|0;
 $c1 = $c1|0;
 $c2 = $c2|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $g = 0, $i = 0, $r = 0, $u = 0, $v = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $c0;
 $1 = $c1;
 $2 = $c2;
 $3 = $n;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $0;
  $9 = (($8) + ($7<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  $y = $10;
  $11 = $i;
  $12 = $1;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $u = $14;
  $15 = $i;
  $16 = $2;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $v = $18;
  $19 = $y;
  $20 = $u;
  $21 = $v;
  $22 = (($20) + ($21))|0;
  $23 = $22 >> 2;
  $24 = (($19) - ($23))|0;
  $g = $24;
  $25 = $v;
  $26 = $g;
  $27 = (($25) + ($26))|0;
  $r = $27;
  $28 = $u;
  $29 = $g;
  $30 = (($28) + ($29))|0;
  $b = $30;
  $31 = $r;
  $32 = $i;
  $33 = $0;
  $34 = (($33) + ($32<<2)|0);
  HEAP32[$34>>2] = $31;
  $35 = $g;
  $36 = $i;
  $37 = $1;
  $38 = (($37) + ($36<<2)|0);
  HEAP32[$38>>2] = $35;
  $39 = $b;
  $40 = $i;
  $41 = $2;
  $42 = (($41) + ($40<<2)|0);
  HEAP32[$42>>2] = $39;
  $43 = $i;
  $44 = (($43) + 1)|0;
  $i = $44;
 }
 STACKTOP = sp;return;
}
function _mct_decode_real($c0,$c1,$c2,$n) {
 $c0 = $c0|0;
 $c1 = $c1|0;
 $c2 = $c2|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0.0, $g = 0.0, $i = 0, $r = 0.0, $u = 0.0, $v = 0.0, $y = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $c0;
 $1 = $c1;
 $2 = $c2;
 $3 = $n;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $3;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = $i;
  $8 = $0;
  $9 = (($8) + ($7<<2)|0);
  $10 = +HEAPF32[$9>>2];
  $y = $10;
  $11 = $i;
  $12 = $1;
  $13 = (($12) + ($11<<2)|0);
  $14 = +HEAPF32[$13>>2];
  $u = $14;
  $15 = $i;
  $16 = $2;
  $17 = (($16) + ($15<<2)|0);
  $18 = +HEAPF32[$17>>2];
  $v = $18;
  $19 = $y;
  $20 = $v;
  $21 = $20 * 1.40199995040893554688;
  $22 = $19 + $21;
  $r = $22;
  $23 = $y;
  $24 = $u;
  $25 = $24 * 0.34413000941276550293;
  $26 = $23 - $25;
  $27 = $v;
  $28 = $27 * 0.71413999795913696289;
  $29 = $26 - $28;
  $g = $29;
  $30 = $y;
  $31 = $u;
  $32 = $31 * 1.77199995517730712891;
  $33 = $30 + $32;
  $b = $33;
  $34 = $r;
  $35 = $i;
  $36 = $0;
  $37 = (($36) + ($35<<2)|0);
  HEAPF32[$37>>2] = $34;
  $38 = $g;
  $39 = $i;
  $40 = $1;
  $41 = (($40) + ($39<<2)|0);
  HEAPF32[$41>>2] = $38;
  $42 = $b;
  $43 = $i;
  $44 = $2;
  $45 = (($44) + ($43<<2)|0);
  HEAPF32[$45>>2] = $42;
  $46 = $i;
  $47 = (($46) + 1)|0;
  $i = $47;
 }
 STACKTOP = sp;return;
}
function _mqc_create() {
 var $0 = 0, $1 = 0, $mqc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = (_malloc(104)|0);
 $mqc = $0;
 $1 = $mqc;
 STACKTOP = sp;return ($1|0);
}
function _mqc_destroy($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = $0;
  _free($3);
 }
 STACKTOP = sp;return;
}
function _mqc_resetstates($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $i = 0;
 while(1) {
  $1 = $i;
  $2 = ($1|0)<(19);
  if (!($2)) {
   break;
  }
  $3 = $i;
  $4 = $0;
  $5 = (($4) + 24|0);
  $6 = (($5) + ($3<<2)|0);
  HEAP32[$6>>2] = 53664;
  $7 = $i;
  $8 = (($7) + 1)|0;
  $i = $8;
 }
 STACKTOP = sp;return;
}
function _mqc_setstate($mqc,$ctxno,$msb,$prob) {
 $mqc = $mqc|0;
 $ctxno = $ctxno|0;
 $msb = $msb|0;
 $prob = $prob|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $ctxno;
 $2 = $msb;
 $3 = $prob;
 $4 = $2;
 $5 = $3;
 $6 = $5 << 1;
 $7 = (($4) + ($6))|0;
 $8 = (53664 + ($7<<4)|0);
 $9 = $1;
 $10 = $0;
 $11 = (($10) + 24|0);
 $12 = (($11) + ($9<<2)|0);
 HEAP32[$12>>2] = $8;
 STACKTOP = sp;return;
}
function _mqc_init_dec($mqc,$bp,$len) {
 $mqc = $mqc|0;
 $bp = $bp|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $bp;
 $2 = $len;
 $3 = $0;
 $4 = (($3) + 24|0);
 $5 = $0;
 $6 = (($5) + 100|0);
 HEAP32[$6>>2] = $4;
 $7 = $1;
 $8 = $0;
 $9 = (($8) + 16|0);
 HEAP32[$9>>2] = $7;
 $10 = $1;
 $11 = $2;
 $12 = (($10) + ($11)|0);
 $13 = $0;
 $14 = (($13) + 20|0);
 HEAP32[$14>>2] = $12;
 $15 = $1;
 $16 = $0;
 $17 = (($16) + 12|0);
 HEAP32[$17>>2] = $15;
 $18 = $2;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = $0;
  HEAP32[$20>>2] = 16711680;
 } else {
  $21 = $0;
  $22 = (($21) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = $25 << 16;
  $27 = $0;
  HEAP32[$27>>2] = $26;
 }
 $28 = $0;
 _mqc_bytein($28);
 $29 = $0;
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 << 7;
 HEAP32[$29>>2] = $31;
 $32 = $0;
 $33 = (($32) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) - 7)|0;
 HEAP32[$33>>2] = $35;
 $36 = $0;
 $37 = (($36) + 4|0);
 HEAP32[$37>>2] = 32768;
 STACKTOP = sp;return;
}
function _mqc_bytein($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)!=($6|0);
 if (!($7)) {
  $57 = $0;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 65280)|0;
  HEAP32[$57>>2] = $59;
  $60 = $0;
  $61 = (($60) + 8|0);
  HEAP32[$61>>2] = 8;
  STACKTOP = sp;return;
 }
 $8 = $0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 1|0);
 $12 = $0;
 $13 = (($12) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)!=($14|0);
 if ($15) {
  $16 = $0;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 1|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $c = $21;
 } else {
  $c = 255;
 }
 $22 = $0;
 $23 = (($22) + 12|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)==(255);
 if ($27) {
  $28 = $c;
  $29 = ($28>>>0)>(143);
  if ($29) {
   $30 = $0;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + 65280)|0;
   HEAP32[$30>>2] = $32;
   $33 = $0;
   $34 = (($33) + 8|0);
   HEAP32[$34>>2] = 8;
  } else {
   $35 = $0;
   $36 = (($35) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 1|0);
   HEAP32[$36>>2] = $38;
   $39 = $c;
   $40 = $39 << 9;
   $41 = $0;
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + ($40))|0;
   HEAP32[$41>>2] = $43;
   $44 = $0;
   $45 = (($44) + 8|0);
   HEAP32[$45>>2] = 7;
  }
 } else {
  $46 = $0;
  $47 = (($46) + 12|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + 1|0);
  HEAP32[$47>>2] = $49;
  $50 = $c;
  $51 = $50 << 8;
  $52 = $0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($51))|0;
  HEAP32[$52>>2] = $54;
  $55 = $0;
  $56 = (($55) + 8|0);
  HEAP32[$56>>2] = 8;
 }
 STACKTOP = sp;return;
}
function _mqc_decode($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $0;
 $2 = (($1) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - ($5))|0;
 HEAP32[$7>>2] = $9;
 $10 = $0;
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 >>> 16;
 $13 = $0;
 $14 = (($13) + 100|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($12>>>0)<($17>>>0);
 if ($18) {
  $19 = $0;
  $20 = (_mqc_lpsexchange($19)|0);
  $d = $20;
  $21 = $0;
  _mqc_renormd($21);
  $45 = $d;
  STACKTOP = sp;return ($45|0);
 }
 $22 = $0;
 $23 = (($22) + 100|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = $26 << 16;
 $28 = $0;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) - ($27))|0;
 HEAP32[$28>>2] = $30;
 $31 = $0;
 $32 = (($31) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $33 & 32768;
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = $0;
  $37 = (_mqc_mpsexchange($36)|0);
  $d = $37;
  $38 = $0;
  _mqc_renormd($38);
 } else {
  $39 = $0;
  $40 = (($39) + 100|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $d = $44;
 }
 $45 = $d;
 STACKTOP = sp;return ($45|0);
}
function _mqc_lpsexchange($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = (($4) + 100|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($3>>>0)<($8>>>0);
 if ($9) {
  $10 = $0;
  $11 = (($10) + 100|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = $0;
  $16 = (($15) + 4|0);
  HEAP32[$16>>2] = $14;
  $17 = $0;
  $18 = (($17) + 100|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $d = $22;
  $23 = $0;
  $24 = (($23) + 100|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $0;
  $30 = (($29) + 100|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$31>>2] = $28;
  $55 = $d;
  STACKTOP = sp;return ($55|0);
 } else {
  $32 = $0;
  $33 = (($32) + 100|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$35>>2]|0;
  $37 = $0;
  $38 = (($37) + 4|0);
  HEAP32[$38>>2] = $36;
  $39 = $0;
  $40 = (($39) + 100|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (1 - ($44))|0;
  $d = $45;
  $46 = $0;
  $47 = (($46) + 100|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = $0;
  $53 = (($52) + 100|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$54>>2] = $51;
  $55 = $d;
  STACKTOP = sp;return ($55|0);
 }
 return 0|0;
}
function _mqc_renormd($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 while(1) {
  $1 = $0;
  $2 = (($1) + 8|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)==(0);
  if ($4) {
   $5 = $0;
   _mqc_bytein($5);
  }
  $6 = $0;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8 << 1;
  HEAP32[$7>>2] = $9;
  $10 = $0;
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 << 1;
  HEAP32[$10>>2] = $12;
  $13 = $0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + -1)|0;
  HEAP32[$14>>2] = $16;
  $17 = $0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19>>>0)<(32768);
  if (!($20)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _mqc_mpsexchange($mqc) {
 $mqc = $mqc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $d = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $mqc;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $0;
 $5 = (($4) + 100|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($3>>>0)<($8>>>0);
 if ($9) {
  $10 = $0;
  $11 = (($10) + 100|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (1 - ($15))|0;
  $d = $16;
  $17 = $0;
  $18 = (($17) + 100|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $0;
  $24 = (($23) + 100|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$25>>2] = $22;
  $41 = $d;
  STACKTOP = sp;return ($41|0);
 } else {
  $26 = $0;
  $27 = (($26) + 100|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $d = $31;
  $32 = $0;
  $33 = (($32) + 100|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $0;
  $39 = (($38) + 100|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$40>>2] = $37;
  $41 = $d;
  STACKTOP = sp;return ($41|0);
 }
 return 0|0;
}
function _opj_create_decompress($format) {
 $format = $format|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dinfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $format;
 $2 = (_calloc(1,28)|0);
 $dinfo = $2;
 $3 = $dinfo;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   $5 = $dinfo;
   $6 = (($5) + 8|0);
   HEAP32[$6>>2] = 1;
   $7 = $1;
   do {
    if ((($7|0) == 1) | (($7|0) == 0)) {
     $8 = $dinfo;
     $9 = (_j2k_create_decompress($8)|0);
     $10 = $dinfo;
     $11 = (($10) + 16|0);
     HEAP32[$11>>2] = $9;
     $12 = $dinfo;
     $13 = (($12) + 16|0);
     $14 = HEAP32[$13>>2]|0;
     $15 = ($14|0)!=(0|0);
     if ($15) {
      label = 12;
      break;
     } else {
      $16 = $dinfo;
      _free($16);
      $0 = 0;
      break L1;
     }
    } else if ((($7|0) == 2)) {
     $17 = $dinfo;
     $18 = (_jp2_create_decompress($17)|0);
     $19 = $dinfo;
     $20 = (($19) + 20|0);
     HEAP32[$20>>2] = $18;
     $21 = $dinfo;
     $22 = (($21) + 20|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = ($23|0)!=(0|0);
     if ($24) {
      label = 12;
      break;
     } else {
      $25 = $dinfo;
      _free($25);
      $0 = 0;
      break L1;
     }
    } else if ((($7|0) == -1)) {
     label = 11;
    } else {
     label = 11;
    }
   } while(0);
   if ((label|0) == 11) {
    $26 = $dinfo;
    _free($26);
    $0 = 0;
    break;
   }
   else if ((label|0) == 12) {
    $27 = $1;
    $28 = $dinfo;
    $29 = (($28) + 12|0);
    HEAP32[$29>>2] = $27;
    $30 = $dinfo;
    $0 = $30;
    break;
   }
  } else {
   $0 = 0;
  }
 } while(0);
 $31 = $0;
 STACKTOP = sp;return ($31|0);
}
function _opj_destroy_decompress($dinfo) {
 $dinfo = $dinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 if ((($5|0) == 2)) {
  $9 = $0;
  $10 = (($9) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  _jp2_destroy_decompress($11);
 } else if ((($5|0) == -1)) {
  label = 6;
 } else if ((($5|0) == 1) | (($5|0) == 0)) {
  $6 = $0;
  $7 = (($6) + 16|0);
  $8 = HEAP32[$7>>2]|0;
  _j2k_destroy_decompress($8);
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
 }
 $12 = $0;
 _free($12);
 STACKTOP = sp;return;
}
function _opj_set_default_decoder_parameters($parameters) {
 $parameters = $parameters|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $parameters;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 _memset(($3|0),0,8224)|0;
 $4 = $0;
 $5 = (($4) + 4|0);
 HEAP32[$5>>2] = 0;
 $6 = $0;
 HEAP32[$6>>2] = 0;
 $7 = $0;
 $8 = (($7) + 8220|0);
 HEAP32[$8>>2] = 0;
 $9 = $0;
 $10 = (($9) + 8200|0);
 HEAP32[$10>>2] = -1;
 $11 = $0;
 $12 = (($11) + 8204|0);
 HEAP32[$12>>2] = -1;
 STACKTOP = sp;return;
}
function _opj_setup_decoder($dinfo,$parameters) {
 $dinfo = $dinfo|0;
 $parameters = $parameters|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $parameters;
 $2 = $0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = $1;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = $0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 if ((($8|0) == 2)) {
  $13 = $0;
  $14 = (($13) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $1;
  _jp2_setup_decoder($15,$16);
 } else if ((($8|0) == -1)) {
  label = 7;
 } else if ((($8|0) == 1) | (($8|0) == 0)) {
  $9 = $0;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $1;
  _j2k_setup_decoder($11,$12);
 } else {
  label = 7;
 }
 if ((label|0) == 7) {
 }
 STACKTOP = sp;return;
}
function _opj_decode($dinfo,$cio) {
 $dinfo = $dinfo|0;
 $cio = $cio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $dinfo;
 $1 = $cio;
 $2 = $0;
 $3 = $1;
 $4 = (_opj_decode_with_info($2,$3,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _opj_decode_with_info($dinfo,$cio,$cstr_info) {
 $dinfo = $dinfo|0;
 $cio = $cio|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $dinfo;
 $2 = $cio;
 $3 = $cstr_info;
 $4 = $1;
 $5 = ($4|0)!=(0|0);
 do {
  if ($5) {
   $6 = $2;
   $7 = ($6|0)!=(0|0);
   if ($7) {
    $8 = $1;
    $9 = (($8) + 12|0);
    $10 = HEAP32[$9>>2]|0;
    if ((($10|0) == 0)) {
     $11 = $1;
     $12 = (($11) + 16|0);
     $13 = HEAP32[$12>>2]|0;
     $14 = $2;
     $15 = $3;
     $16 = (_j2k_decode($13,$14,$15)|0);
     $0 = $16;
     break;
    } else if ((($10|0) == 1)) {
     $17 = $1;
     $18 = (($17) + 16|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = $2;
     $21 = $3;
     $22 = (_j2k_decode_jpt_stream($19,$20,$21)|0);
     $0 = $22;
     break;
    } else if ((($10|0) == -1)) {
    } else if ((($10|0) == 2)) {
     $23 = $1;
     $24 = (($23) + 20|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = $2;
     $27 = $3;
     $28 = (_jp2_decode($25,$26,$27)|0);
     $0 = $28;
     break;
    }
    label = 10;
   } else {
    label = 10;
   }
  } else {
   label = 10;
  }
 } while(0);
 if ((label|0) == 10) {
  $0 = 0;
 }
 $29 = $0;
 STACKTOP = sp;return ($29|0);
}
function _pi_create_decode($image,$cp,$tileno) {
 $image = $image|0;
 $cp = $cp|0;
 $tileno = $tileno|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp = 0, $compno = 0, $levelno = 0, $maxprec = 0, $maxres = 0, $p = 0, $pi = 0, $pino = 0, $px0 = 0, $px1 = 0, $py0 = 0;
 var $py1 = 0, $q = 0, $res = 0, $resno = 0, $rx0 = 0, $rx1 = 0, $ry0 = 0, $ry1 = 0, $tccp = 0, $tcp = 0, $tcx0 = 0, $tcx1 = 0, $tcy0 = 0, $tcy1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $1 = $image;
 $2 = $cp;
 $3 = $tileno;
 $pi = 0;
 $tcp = 0;
 $tccp = 0;
 $4 = $3;
 $5 = $2;
 $6 = (($5) + 108|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + (($4*5588)|0)|0);
 $tcp = $8;
 $9 = $tcp;
 $10 = (($9) + 420|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 1)|0;
 $13 = (_calloc($12,232)|0);
 $pi = $13;
 $14 = $pi;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $0 = 0;
  $548 = $0;
  STACKTOP = sp;return ($548|0);
 }
 $pino = 0;
 L5: while(1) {
  $16 = $pino;
  $17 = $tcp;
  $18 = (($17) + 420|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 1)|0;
  $21 = ($16|0)<($20|0);
  if (!($21)) {
   label = 40;
   break;
  }
  $maxres = 0;
  $maxprec = 0;
  $22 = $3;
  $23 = $2;
  $24 = (($23) + 68|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($22|0) % ($25|0))&-1;
  $p = $26;
  $27 = $3;
  $28 = $2;
  $29 = (($28) + 68|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($27|0) / ($30|0))&-1;
  $q = $31;
  $32 = $2;
  $33 = (($32) + 48|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $p;
  $36 = $2;
  $37 = (($36) + 56|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = Math_imul($35, $38)|0;
  $40 = (($34) + ($39))|0;
  $41 = $1;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_int_max($40,$42)|0);
  $44 = $pino;
  $45 = $pi;
  $46 = (($45) + (($44*232)|0)|0);
  $47 = (($46) + 200|0);
  HEAP32[$47>>2] = $43;
  $48 = $2;
  $49 = (($48) + 52|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = $q;
  $52 = $2;
  $53 = (($52) + 60|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = Math_imul($51, $54)|0;
  $56 = (($50) + ($55))|0;
  $57 = $1;
  $58 = (($57) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (_int_max($56,$59)|0);
  $61 = $pino;
  $62 = $pi;
  $63 = (($62) + (($61*232)|0)|0);
  $64 = (($63) + 204|0);
  HEAP32[$64>>2] = $60;
  $65 = $2;
  $66 = (($65) + 48|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = $p;
  $69 = (($68) + 1)|0;
  $70 = $2;
  $71 = (($70) + 56|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = Math_imul($69, $72)|0;
  $74 = (($67) + ($73))|0;
  $75 = $1;
  $76 = (($75) + 8|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (_int_min36($74,$77)|0);
  $79 = $pino;
  $80 = $pi;
  $81 = (($80) + (($79*232)|0)|0);
  $82 = (($81) + 208|0);
  HEAP32[$82>>2] = $78;
  $83 = $2;
  $84 = (($83) + 52|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = $q;
  $87 = (($86) + 1)|0;
  $88 = $2;
  $89 = (($88) + 60|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = Math_imul($87, $90)|0;
  $92 = (($85) + ($91))|0;
  $93 = $1;
  $94 = (($93) + 12|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (_int_min36($92,$95)|0);
  $97 = $pino;
  $98 = $pi;
  $99 = (($98) + (($97*232)|0)|0);
  $100 = (($99) + 212|0);
  HEAP32[$100>>2] = $96;
  $101 = $1;
  $102 = (($101) + 16|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = $pino;
  $105 = $pi;
  $106 = (($105) + (($104*232)|0)|0);
  $107 = (($106) + 192|0);
  HEAP32[$107>>2] = $103;
  $108 = $1;
  $109 = (($108) + 16|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = (_calloc($110,16)|0);
  $112 = $pino;
  $113 = $pi;
  $114 = (($113) + (($112*232)|0)|0);
  $115 = (($114) + 196|0);
  HEAP32[$115>>2] = $111;
  $116 = $pino;
  $117 = $pi;
  $118 = (($117) + (($116*232)|0)|0);
  $119 = (($118) + 196|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = ($120|0)!=(0|0);
  if (!($121)) {
   label = 6;
   break;
  }
  $compno = 0;
  while(1) {
   $125 = $compno;
   $126 = $pi;
   $127 = (($126) + 192|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = ($125|0)<($128|0);
   if (!($129)) {
    break;
   }
   $130 = $compno;
   $131 = $pino;
   $132 = $pi;
   $133 = (($132) + (($131*232)|0)|0);
   $134 = (($133) + 196|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = (($135) + ($130<<4)|0);
   $comp = $136;
   $137 = $compno;
   $138 = $tcp;
   $139 = (($138) + 5584|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = (($140) + (($137*1076)|0)|0);
   $tccp = $141;
   $142 = $compno;
   $143 = $1;
   $144 = (($143) + 24|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = (($145) + (($142*48)|0)|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = $comp;
   HEAP32[$148>>2] = $147;
   $149 = $compno;
   $150 = $1;
   $151 = (($150) + 24|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = (($152) + (($149*48)|0)|0);
   $154 = (($153) + 4|0);
   $155 = HEAP32[$154>>2]|0;
   $156 = $comp;
   $157 = (($156) + 4|0);
   HEAP32[$157>>2] = $155;
   $158 = $tccp;
   $159 = (($158) + 4|0);
   $160 = HEAP32[$159>>2]|0;
   $161 = $comp;
   $162 = (($161) + 8|0);
   HEAP32[$162>>2] = $160;
   $163 = $comp;
   $164 = (($163) + 8|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = (_calloc($165,16)|0);
   $167 = $comp;
   $168 = (($167) + 12|0);
   HEAP32[$168>>2] = $166;
   $169 = $comp;
   $170 = (($169) + 12|0);
   $171 = HEAP32[$170>>2]|0;
   $172 = ($171|0)!=(0|0);
   if (!($172)) {
    label = 10;
    break L5;
   }
   $176 = $pi;
   $177 = (($176) + 200|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = $comp;
   $180 = HEAP32[$179>>2]|0;
   $181 = (_int_ceildiv39($178,$180)|0);
   $tcx0 = $181;
   $182 = $pi;
   $183 = (($182) + 204|0);
   $184 = HEAP32[$183>>2]|0;
   $185 = $comp;
   $186 = (($185) + 4|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = (_int_ceildiv39($184,$187)|0);
   $tcy0 = $188;
   $189 = $pi;
   $190 = (($189) + 208|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = $comp;
   $193 = HEAP32[$192>>2]|0;
   $194 = (_int_ceildiv39($191,$193)|0);
   $tcx1 = $194;
   $195 = $pi;
   $196 = (($195) + 212|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = $comp;
   $199 = (($198) + 4|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = (_int_ceildiv39($197,$200)|0);
   $tcy1 = $201;
   $202 = $comp;
   $203 = (($202) + 8|0);
   $204 = HEAP32[$203>>2]|0;
   $205 = $maxres;
   $206 = ($204|0)>($205|0);
   if ($206) {
    $207 = $comp;
    $208 = (($207) + 8|0);
    $209 = HEAP32[$208>>2]|0;
    $maxres = $209;
   }
   $resno = 0;
   while(1) {
    $210 = $resno;
    $211 = $comp;
    $212 = (($211) + 8|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = ($210|0)<($213|0);
    if (!($214)) {
     break;
    }
    $215 = $resno;
    $216 = $comp;
    $217 = (($216) + 12|0);
    $218 = HEAP32[$217>>2]|0;
    $219 = (($218) + ($215<<4)|0);
    $res = $219;
    $220 = $tccp;
    $221 = HEAP32[$220>>2]|0;
    $222 = $221 & 1;
    $223 = ($222|0)!=(0);
    if ($223) {
     $224 = $resno;
     $225 = $tccp;
     $226 = (($225) + 812|0);
     $227 = (($226) + ($224<<2)|0);
     $228 = HEAP32[$227>>2]|0;
     $229 = $res;
     HEAP32[$229>>2] = $228;
     $230 = $resno;
     $231 = $tccp;
     $232 = (($231) + 944|0);
     $233 = (($232) + ($230<<2)|0);
     $234 = HEAP32[$233>>2]|0;
     $235 = $res;
     $236 = (($235) + 4|0);
     HEAP32[$236>>2] = $234;
    } else {
     $237 = $res;
     HEAP32[$237>>2] = 15;
     $238 = $res;
     $239 = (($238) + 4|0);
     HEAP32[$239>>2] = 15;
    }
    $240 = $comp;
    $241 = (($240) + 8|0);
    $242 = HEAP32[$241>>2]|0;
    $243 = (($242) - 1)|0;
    $244 = $resno;
    $245 = (($243) - ($244))|0;
    $levelno = $245;
    $246 = $tcx0;
    $247 = $levelno;
    $248 = (_int_ceildivpow2($246,$247)|0);
    $rx0 = $248;
    $249 = $tcy0;
    $250 = $levelno;
    $251 = (_int_ceildivpow2($249,$250)|0);
    $ry0 = $251;
    $252 = $tcx1;
    $253 = $levelno;
    $254 = (_int_ceildivpow2($252,$253)|0);
    $rx1 = $254;
    $255 = $tcy1;
    $256 = $levelno;
    $257 = (_int_ceildivpow2($255,$256)|0);
    $ry1 = $257;
    $258 = $rx0;
    $259 = $res;
    $260 = HEAP32[$259>>2]|0;
    $261 = (_int_floordivpow2($258,$260)|0);
    $262 = $res;
    $263 = HEAP32[$262>>2]|0;
    $264 = $261 << $263;
    $px0 = $264;
    $265 = $ry0;
    $266 = $res;
    $267 = (($266) + 4|0);
    $268 = HEAP32[$267>>2]|0;
    $269 = (_int_floordivpow2($265,$268)|0);
    $270 = $res;
    $271 = (($270) + 4|0);
    $272 = HEAP32[$271>>2]|0;
    $273 = $269 << $272;
    $py0 = $273;
    $274 = $rx1;
    $275 = $res;
    $276 = HEAP32[$275>>2]|0;
    $277 = (_int_ceildivpow2($274,$276)|0);
    $278 = $res;
    $279 = HEAP32[$278>>2]|0;
    $280 = $277 << $279;
    $px1 = $280;
    $281 = $ry1;
    $282 = $res;
    $283 = (($282) + 4|0);
    $284 = HEAP32[$283>>2]|0;
    $285 = (_int_ceildivpow2($281,$284)|0);
    $286 = $res;
    $287 = (($286) + 4|0);
    $288 = HEAP32[$287>>2]|0;
    $289 = $285 << $288;
    $py1 = $289;
    $290 = $rx0;
    $291 = $rx1;
    $292 = ($290|0)==($291|0);
    if ($292) {
     $301 = 0;
    } else {
     $293 = $px1;
     $294 = $px0;
     $295 = (($293) - ($294))|0;
     $296 = $res;
     $297 = HEAP32[$296>>2]|0;
     $298 = $295 >> $297;
     $301 = $298;
    }
    $299 = $res;
    $300 = (($299) + 8|0);
    HEAP32[$300>>2] = $301;
    $302 = $ry0;
    $303 = $ry1;
    $304 = ($302|0)==($303|0);
    if ($304) {
     $314 = 0;
    } else {
     $305 = $py1;
     $306 = $py0;
     $307 = (($305) - ($306))|0;
     $308 = $res;
     $309 = (($308) + 4|0);
     $310 = HEAP32[$309>>2]|0;
     $311 = $307 >> $310;
     $314 = $311;
    }
    $312 = $res;
    $313 = (($312) + 12|0);
    HEAP32[$313>>2] = $314;
    $315 = $res;
    $316 = (($315) + 8|0);
    $317 = HEAP32[$316>>2]|0;
    $318 = $res;
    $319 = (($318) + 12|0);
    $320 = HEAP32[$319>>2]|0;
    $321 = Math_imul($317, $320)|0;
    $322 = $maxprec;
    $323 = ($321|0)>($322|0);
    if ($323) {
     $324 = $res;
     $325 = (($324) + 8|0);
     $326 = HEAP32[$325>>2]|0;
     $327 = $res;
     $328 = (($327) + 12|0);
     $329 = HEAP32[$328>>2]|0;
     $330 = Math_imul($326, $329)|0;
     $maxprec = $330;
    }
    $331 = $resno;
    $332 = (($331) + 1)|0;
    $resno = $332;
   }
   $333 = $compno;
   $334 = (($333) + 1)|0;
   $compno = $334;
  }
  $335 = $tcp;
  $336 = (($335) + 5584|0);
  $337 = HEAP32[$336>>2]|0;
  $tccp = $337;
  $338 = $pino;
  $339 = $pi;
  $340 = (($339) + (($338*232)|0)|0);
  $341 = (($340) + 20|0);
  HEAP32[$341>>2] = 1;
  $342 = $maxprec;
  $343 = $pino;
  $344 = $pi;
  $345 = (($344) + (($343*232)|0)|0);
  $346 = (($345) + 20|0);
  $347 = HEAP32[$346>>2]|0;
  $348 = Math_imul($342, $347)|0;
  $349 = $pino;
  $350 = $pi;
  $351 = (($350) + (($349*232)|0)|0);
  $352 = (($351) + 16|0);
  HEAP32[$352>>2] = $348;
  $353 = $1;
  $354 = (($353) + 16|0);
  $355 = HEAP32[$354>>2]|0;
  $356 = $pino;
  $357 = $pi;
  $358 = (($357) + (($356*232)|0)|0);
  $359 = (($358) + 16|0);
  $360 = HEAP32[$359>>2]|0;
  $361 = Math_imul($355, $360)|0;
  $362 = $pino;
  $363 = $pi;
  $364 = (($363) + (($362*232)|0)|0);
  $365 = (($364) + 12|0);
  HEAP32[$365>>2] = $361;
  $366 = $maxres;
  $367 = $pino;
  $368 = $pi;
  $369 = (($368) + (($367*232)|0)|0);
  $370 = (($369) + 12|0);
  $371 = HEAP32[$370>>2]|0;
  $372 = Math_imul($366, $371)|0;
  $373 = $pino;
  $374 = $pi;
  $375 = (($374) + (($373*232)|0)|0);
  $376 = (($375) + 8|0);
  HEAP32[$376>>2] = $372;
  $377 = $pino;
  $378 = ($377|0)==(0);
  if ($378) {
   $379 = $1;
   $380 = (($379) + 16|0);
   $381 = HEAP32[$380>>2]|0;
   $382 = $maxres;
   $383 = Math_imul($381, $382)|0;
   $384 = $tcp;
   $385 = (($384) + 12|0);
   $386 = HEAP32[$385>>2]|0;
   $387 = Math_imul($383, $386)|0;
   $388 = $maxprec;
   $389 = Math_imul($387, $388)|0;
   $390 = (_calloc($389,2)|0);
   $391 = $pino;
   $392 = $pi;
   $393 = (($392) + (($391*232)|0)|0);
   $394 = (($393) + 4|0);
   HEAP32[$394>>2] = $390;
   $395 = $pino;
   $396 = $pi;
   $397 = (($396) + (($395*232)|0)|0);
   $398 = (($397) + 4|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = ($399|0)!=(0|0);
   if (!($400)) {
    label = 32;
    break;
   }
  } else {
   $404 = $pino;
   $405 = (($404) - 1)|0;
   $406 = $pi;
   $407 = (($406) + (($405*232)|0)|0);
   $408 = (($407) + 4|0);
   $409 = HEAP32[$408>>2]|0;
   $410 = $pino;
   $411 = $pi;
   $412 = (($411) + (($410*232)|0)|0);
   $413 = (($412) + 4|0);
   HEAP32[$413>>2] = $409;
  }
  $414 = $tcp;
  $415 = (($414) + 424|0);
  $416 = HEAP32[$415>>2]|0;
  $417 = ($416|0)==(0);
  if ($417) {
   $418 = $pino;
   $419 = $pi;
   $420 = (($419) + (($418*232)|0)|0);
   $421 = (($420) + 40|0);
   HEAP32[$421>>2] = 1;
   $422 = $pino;
   $423 = $pi;
   $424 = (($423) + (($422*232)|0)|0);
   $425 = (($424) + 44|0);
   HEAP32[$425>>2] = 0;
   $426 = $pino;
   $427 = $pi;
   $428 = (($427) + (($426*232)|0)|0);
   $429 = (($428) + 44|0);
   $430 = (($429) + 4|0);
   HEAP32[$430>>2] = 0;
   $431 = $tcp;
   $432 = (($431) + 12|0);
   $433 = HEAP32[$432>>2]|0;
   $434 = $pino;
   $435 = $pi;
   $436 = (($435) + (($434*232)|0)|0);
   $437 = (($436) + 44|0);
   $438 = (($437) + 8|0);
   HEAP32[$438>>2] = $433;
   $439 = $maxres;
   $440 = $pino;
   $441 = $pi;
   $442 = (($441) + (($440*232)|0)|0);
   $443 = (($442) + 44|0);
   $444 = (($443) + 12|0);
   HEAP32[$444>>2] = $439;
   $445 = $1;
   $446 = (($445) + 16|0);
   $447 = HEAP32[$446>>2]|0;
   $448 = $pino;
   $449 = $pi;
   $450 = (($449) + (($448*232)|0)|0);
   $451 = (($450) + 44|0);
   $452 = (($451) + 16|0);
   HEAP32[$452>>2] = $447;
   $453 = $tcp;
   $454 = (($453) + 8|0);
   $455 = HEAP32[$454>>2]|0;
   $456 = $pino;
   $457 = $pi;
   $458 = (($457) + (($456*232)|0)|0);
   $459 = (($458) + 44|0);
   $460 = (($459) + 36|0);
   HEAP32[$460>>2] = $455;
  } else {
   $461 = $pino;
   $462 = $pi;
   $463 = (($462) + (($461*232)|0)|0);
   $464 = (($463) + 40|0);
   HEAP32[$464>>2] = 1;
   $465 = $pino;
   $466 = $tcp;
   $467 = (($466) + 428|0);
   $468 = (($467) + (($465*148)|0)|0);
   $469 = HEAP32[$468>>2]|0;
   $470 = $pino;
   $471 = $pi;
   $472 = (($471) + (($470*232)|0)|0);
   $473 = (($472) + 44|0);
   HEAP32[$473>>2] = $469;
   $474 = $pino;
   $475 = $tcp;
   $476 = (($475) + 428|0);
   $477 = (($476) + (($474*148)|0)|0);
   $478 = (($477) + 4|0);
   $479 = HEAP32[$478>>2]|0;
   $480 = $pino;
   $481 = $pi;
   $482 = (($481) + (($480*232)|0)|0);
   $483 = (($482) + 44|0);
   $484 = (($483) + 4|0);
   HEAP32[$484>>2] = $479;
   $485 = $pino;
   $486 = $tcp;
   $487 = (($486) + 428|0);
   $488 = (($487) + (($485*148)|0)|0);
   $489 = (($488) + 8|0);
   $490 = HEAP32[$489>>2]|0;
   $491 = $pino;
   $492 = $pi;
   $493 = (($492) + (($491*232)|0)|0);
   $494 = (($493) + 44|0);
   $495 = (($494) + 8|0);
   HEAP32[$495>>2] = $490;
   $496 = $pino;
   $497 = $tcp;
   $498 = (($497) + 428|0);
   $499 = (($498) + (($496*148)|0)|0);
   $500 = (($499) + 12|0);
   $501 = HEAP32[$500>>2]|0;
   $502 = $pino;
   $503 = $pi;
   $504 = (($503) + (($502*232)|0)|0);
   $505 = (($504) + 44|0);
   $506 = (($505) + 12|0);
   HEAP32[$506>>2] = $501;
   $507 = $pino;
   $508 = $tcp;
   $509 = (($508) + 428|0);
   $510 = (($509) + (($507*148)|0)|0);
   $511 = (($510) + 16|0);
   $512 = HEAP32[$511>>2]|0;
   $513 = $pino;
   $514 = $pi;
   $515 = (($514) + (($513*232)|0)|0);
   $516 = (($515) + 44|0);
   $517 = (($516) + 16|0);
   HEAP32[$517>>2] = $512;
   $518 = $pino;
   $519 = $tcp;
   $520 = (($519) + 428|0);
   $521 = (($520) + (($518*148)|0)|0);
   $522 = (($521) + 36|0);
   $523 = HEAP32[$522>>2]|0;
   $524 = $pino;
   $525 = $pi;
   $526 = (($525) + (($524*232)|0)|0);
   $527 = (($526) + 44|0);
   $528 = (($527) + 36|0);
   HEAP32[$528>>2] = $523;
  }
  $529 = $pino;
  $530 = $pi;
  $531 = (($530) + (($529*232)|0)|0);
  $532 = (($531) + 44|0);
  $533 = (($532) + 20|0);
  HEAP32[$533>>2] = 0;
  $534 = $pino;
  $535 = $pi;
  $536 = (($535) + (($534*232)|0)|0);
  $537 = (($536) + 44|0);
  $538 = (($537) + 24|0);
  HEAP32[$538>>2] = 0;
  $539 = $maxprec;
  $540 = $pino;
  $541 = $pi;
  $542 = (($541) + (($540*232)|0)|0);
  $543 = (($542) + 44|0);
  $544 = (($543) + 28|0);
  HEAP32[$544>>2] = $539;
  $545 = $pino;
  $546 = (($545) + 1)|0;
  $pino = $546;
 }
 if ((label|0) == 6) {
  $122 = $pi;
  $123 = $2;
  $124 = $3;
  _pi_destroy($122,$123,$124);
  $0 = 0;
  $548 = $0;
  STACKTOP = sp;return ($548|0);
 }
 else if ((label|0) == 10) {
  $173 = $pi;
  $174 = $2;
  $175 = $3;
  _pi_destroy($173,$174,$175);
  $0 = 0;
  $548 = $0;
  STACKTOP = sp;return ($548|0);
 }
 else if ((label|0) == 32) {
  $401 = $pi;
  $402 = $2;
  $403 = $3;
  _pi_destroy($401,$402,$403);
  $0 = 0;
  $548 = $0;
  STACKTOP = sp;return ($548|0);
 }
 else if ((label|0) == 40) {
  $547 = $pi;
  $0 = $547;
  $548 = $0;
  STACKTOP = sp;return ($548|0);
 }
 return 0|0;
}
function _int_max($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)>($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _int_min36($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)<($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _pi_destroy($pi,$cp,$tileno) {
 $pi = $pi|0;
 $cp = $cp|0;
 $tileno = $tileno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $comp = 0, $compno = 0;
 var $pino = 0, $tcp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $pi;
 $1 = $cp;
 $2 = $tileno;
 $3 = $2;
 $4 = $1;
 $5 = (($4) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + (($3*5588)|0)|0);
 $tcp = $7;
 $8 = $0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $pino = 0;
 while(1) {
  $10 = $pino;
  $11 = $tcp;
  $12 = (($11) + 420|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 1)|0;
  $15 = ($10|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $pino;
  $17 = $0;
  $18 = (($17) + (($16*232)|0)|0);
  $19 = (($18) + 196|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $compno = 0;
   while(1) {
    $22 = $compno;
    $23 = $0;
    $24 = (($23) + 192|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)<($25|0);
    if (!($26)) {
     break;
    }
    $27 = $compno;
    $28 = $pino;
    $29 = $0;
    $30 = (($29) + (($28*232)|0)|0);
    $31 = (($30) + 196|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) + ($27<<4)|0);
    $comp = $33;
    $34 = $comp;
    $35 = (($34) + 12|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = ($36|0)!=(0|0);
    if ($37) {
     $38 = $comp;
     $39 = (($38) + 12|0);
     $40 = HEAP32[$39>>2]|0;
     _free($40);
    }
    $41 = $compno;
    $42 = (($41) + 1)|0;
    $compno = $42;
   }
   $43 = $pino;
   $44 = $0;
   $45 = (($44) + (($43*232)|0)|0);
   $46 = (($45) + 196|0);
   $47 = HEAP32[$46>>2]|0;
   _free($47);
  }
  $48 = $pino;
  $49 = (($48) + 1)|0;
  $pino = $49;
 }
 $50 = $0;
 $51 = (($50) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)!=(0|0);
 if ($53) {
  $54 = $0;
  $55 = (($54) + 4|0);
  $56 = HEAP32[$55>>2]|0;
  _free($56);
 }
 $57 = $0;
 _free($57);
 STACKTOP = sp;return;
}
function _int_ceildiv39($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = (($2) + ($3))|0;
 $5 = (($4) - 1)|0;
 $6 = $1;
 $7 = (($5|0) / ($6|0))&-1;
 STACKTOP = sp;return ($7|0);
}
function _int_ceildivpow2($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = 1 << $3;
 $5 = (($2) + ($4))|0;
 $6 = (($5) - 1)|0;
 $7 = $1;
 $8 = $6 >> $7;
 STACKTOP = sp;return ($8|0);
}
function _int_floordivpow2($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = $2 >> $3;
 STACKTOP = sp;return ($4|0);
}
function _pi_next($pi) {
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $pi;
 $2 = $1;
 $3 = (($2) + 44|0);
 $4 = (($3) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 switch ($5|0) {
 case 3:  {
  $12 = $1;
  $13 = (_pi_next_pcrl($12)|0);
  $0 = $13;
  break;
 }
 case 4:  {
  $14 = $1;
  $15 = (_pi_next_cprl($14)|0);
  $0 = $15;
  break;
 }
 case 2:  {
  $10 = $1;
  $11 = (_pi_next_rpcl($10)|0);
  $0 = $11;
  break;
 }
 case 1:  {
  $8 = $1;
  $9 = (_pi_next_rlcp($8)|0);
  $0 = $9;
  break;
 }
 case -1:  {
  $0 = 0;
  break;
 }
 case 0:  {
  $6 = $1;
  $7 = (_pi_next_lrcp($6)|0);
  $0 = $7;
  break;
 }
 default: {
  $0 = 0;
 }
 }
 $16 = $0;
 STACKTOP = sp;return ($16|0);
}
function _pi_next_lrcp($pi) {
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp = 0, $index = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $pi;
 $comp = 0;
 $res = 0;
 $index = 0;
 $2 = $1;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $20 = $1;
  $21 = (($20) + 40|0);
  HEAP32[$21>>2] = 0;
  $22 = $1;
  $23 = (($22) + 44|0);
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $1;
  $27 = (($26) + 36|0);
  HEAP32[$27>>2] = $25;
  label = 5;
 } else {
  $6 = $1;
  $7 = (($6) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $1;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $comp = $12;
  $13 = $1;
  $14 = (($13) + 28|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $comp;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  $res = $19;
  label = 19;
 }
 while(1) {
  if ((label|0) == 5) {
   label = 0;
   $28 = $1;
   $29 = (($28) + 36|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $1;
   $32 = (($31) + 44|0);
   $33 = (($32) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = ($30|0)<($34|0);
   if (!($35)) {
    label = 27;
    break;
   }
   $36 = $1;
   $37 = (($36) + 44|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $1;
   $40 = (($39) + 28|0);
   HEAP32[$40>>2] = $38;
   label = 7;
  }
  else if ((label|0) == 19) {
   label = 0;
   $154 = $1;
   $155 = (($154) + 32|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = (($156) + 1)|0;
   HEAP32[$155>>2] = $157;
   label = 15;
  }
  while(1) {
   if ((label|0) == 7) {
    label = 0;
    $41 = $1;
    $42 = (($41) + 28|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = $1;
    $45 = (($44) + 44|0);
    $46 = (($45) + 12|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = ($43|0)<($47|0);
    if (!($48)) {
     label = 25;
     break;
    }
    $49 = $1;
    $50 = (($49) + 44|0);
    $51 = (($50) + 4|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = $1;
    $54 = (($53) + 24|0);
    HEAP32[$54>>2] = $52;
   }
   else if ((label|0) == 15) {
    label = 0;
    $103 = $1;
    $104 = (($103) + 32|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $1;
    $107 = (($106) + 44|0);
    $108 = (($107) + 28|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = ($105|0)<($109|0);
    if ($110) {
     label = 16;
     break;
    }
    label = 22;
   }
   while(1) {
    if ((label|0) == 22) {
     label = 0;
     $158 = $1;
     $159 = (($158) + 24|0);
     $160 = HEAP32[$159>>2]|0;
     $161 = (($160) + 1)|0;
     HEAP32[$159>>2] = $161;
    }
    $55 = $1;
    $56 = (($55) + 24|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = $1;
    $59 = (($58) + 44|0);
    $60 = (($59) + 16|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($57|0)<($61|0);
    if (!($62)) {
     label = 23;
     break;
    }
    $63 = $1;
    $64 = (($63) + 24|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = $1;
    $67 = (($66) + 196|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + ($65<<4)|0);
    $comp = $69;
    $70 = $1;
    $71 = (($70) + 28|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $comp;
    $74 = (($73) + 8|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ($72|0)>=($75|0);
    if (!($76)) {
     break;
    }
    label = 22;
   }
   if ((label|0) == 23) {
    label = 0;
    $162 = $1;
    $163 = (($162) + 28|0);
    $164 = HEAP32[$163>>2]|0;
    $165 = (($164) + 1)|0;
    HEAP32[$163>>2] = $165;
    label = 7;
    continue;
   }
   $77 = $1;
   $78 = (($77) + 28|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = $comp;
   $81 = (($80) + 12|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (($82) + ($79<<4)|0);
   $res = $83;
   $84 = $1;
   $85 = HEAP8[$84>>0]|0;
   $86 = ($85<<24>>24)!=(0);
   if (!($86)) {
    $87 = $res;
    $88 = (($87) + 8|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = $res;
    $91 = (($90) + 12|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = Math_imul($89, $92)|0;
    $94 = $1;
    $95 = (($94) + 44|0);
    $96 = (($95) + 28|0);
    HEAP32[$96>>2] = $93;
   }
   $97 = $1;
   $98 = (($97) + 44|0);
   $99 = (($98) + 24|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = $1;
   $102 = (($101) + 32|0);
   HEAP32[$102>>2] = $100;
   label = 15;
  }
  if ((label|0) == 16) {
   label = 0;
   $111 = $1;
   $112 = (($111) + 36|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = $1;
   $115 = (($114) + 8|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = Math_imul($113, $116)|0;
   $118 = $1;
   $119 = (($118) + 28|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = $1;
   $122 = (($121) + 12|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = Math_imul($120, $123)|0;
   $125 = (($117) + ($124))|0;
   $126 = $1;
   $127 = (($126) + 24|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = $1;
   $130 = (($129) + 16|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = Math_imul($128, $131)|0;
   $133 = (($125) + ($132))|0;
   $134 = $1;
   $135 = (($134) + 32|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = $1;
   $138 = (($137) + 20|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = Math_imul($136, $139)|0;
   $141 = (($133) + ($140))|0;
   $index = $141;
   $142 = $index;
   $143 = $1;
   $144 = (($143) + 4|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = (($145) + ($142<<1)|0);
   $147 = HEAP16[$146>>1]|0;
   $148 = ($147<<16>>16)!=(0);
   if (!($148)) {
    label = 17;
    break;
   }
   label = 19;
   continue;
  }
  else if ((label|0) == 25) {
   label = 0;
   $166 = $1;
   $167 = (($166) + 36|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (($168) + 1)|0;
   HEAP32[$167>>2] = $169;
   label = 5;
   continue;
  }
 }
 if ((label|0) == 17) {
  $149 = $index;
  $150 = $1;
  $151 = (($150) + 4|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = (($152) + ($149<<1)|0);
  HEAP16[$153>>1] = 1;
  $0 = 1;
  $170 = $0;
  STACKTOP = sp;return ($170|0);
 }
 else if ((label|0) == 27) {
  $0 = 0;
  $170 = $0;
  STACKTOP = sp;return ($170|0);
 }
 return 0|0;
}
function _pi_next_rlcp($pi) {
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp = 0, $index = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $pi;
 $comp = 0;
 $res = 0;
 $index = 0;
 $2 = $1;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $20 = $1;
  $21 = (($20) + 40|0);
  HEAP32[$21>>2] = 0;
  $22 = $1;
  $23 = (($22) + 44|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $1;
  $26 = (($25) + 28|0);
  HEAP32[$26>>2] = $24;
  label = 5;
 } else {
  $6 = $1;
  $7 = (($6) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $1;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $comp = $12;
  $13 = $1;
  $14 = (($13) + 28|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $comp;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  $res = $19;
  label = 19;
 }
 while(1) {
  if ((label|0) == 5) {
   label = 0;
   $27 = $1;
   $28 = (($27) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $1;
   $31 = (($30) + 44|0);
   $32 = (($31) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($29|0)<($33|0);
   if (!($34)) {
    label = 27;
    break;
   }
   $35 = $1;
   $36 = (($35) + 44|0);
   $37 = (($36) + 20|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $1;
   $40 = (($39) + 36|0);
   HEAP32[$40>>2] = $38;
   label = 7;
  }
  else if ((label|0) == 19) {
   label = 0;
   $154 = $1;
   $155 = (($154) + 32|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = (($156) + 1)|0;
   HEAP32[$155>>2] = $157;
   label = 15;
  }
  while(1) {
   if ((label|0) == 7) {
    label = 0;
    $41 = $1;
    $42 = (($41) + 36|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = $1;
    $45 = (($44) + 44|0);
    $46 = (($45) + 8|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = ($43|0)<($47|0);
    if (!($48)) {
     label = 25;
     break;
    }
    $49 = $1;
    $50 = (($49) + 44|0);
    $51 = (($50) + 4|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = $1;
    $54 = (($53) + 24|0);
    HEAP32[$54>>2] = $52;
   }
   else if ((label|0) == 15) {
    label = 0;
    $103 = $1;
    $104 = (($103) + 32|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $1;
    $107 = (($106) + 44|0);
    $108 = (($107) + 28|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = ($105|0)<($109|0);
    if ($110) {
     label = 16;
     break;
    }
    label = 22;
   }
   while(1) {
    if ((label|0) == 22) {
     label = 0;
     $158 = $1;
     $159 = (($158) + 24|0);
     $160 = HEAP32[$159>>2]|0;
     $161 = (($160) + 1)|0;
     HEAP32[$159>>2] = $161;
    }
    $55 = $1;
    $56 = (($55) + 24|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = $1;
    $59 = (($58) + 44|0);
    $60 = (($59) + 16|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($57|0)<($61|0);
    if (!($62)) {
     label = 23;
     break;
    }
    $63 = $1;
    $64 = (($63) + 24|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = $1;
    $67 = (($66) + 196|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + ($65<<4)|0);
    $comp = $69;
    $70 = $1;
    $71 = (($70) + 28|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $comp;
    $74 = (($73) + 8|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ($72|0)>=($75|0);
    if (!($76)) {
     break;
    }
    label = 22;
   }
   if ((label|0) == 23) {
    label = 0;
    $162 = $1;
    $163 = (($162) + 36|0);
    $164 = HEAP32[$163>>2]|0;
    $165 = (($164) + 1)|0;
    HEAP32[$163>>2] = $165;
    label = 7;
    continue;
   }
   $77 = $1;
   $78 = (($77) + 28|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = $comp;
   $81 = (($80) + 12|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (($82) + ($79<<4)|0);
   $res = $83;
   $84 = $1;
   $85 = HEAP8[$84>>0]|0;
   $86 = ($85<<24>>24)!=(0);
   if (!($86)) {
    $87 = $res;
    $88 = (($87) + 8|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = $res;
    $91 = (($90) + 12|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = Math_imul($89, $92)|0;
    $94 = $1;
    $95 = (($94) + 44|0);
    $96 = (($95) + 28|0);
    HEAP32[$96>>2] = $93;
   }
   $97 = $1;
   $98 = (($97) + 44|0);
   $99 = (($98) + 24|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = $1;
   $102 = (($101) + 32|0);
   HEAP32[$102>>2] = $100;
   label = 15;
  }
  if ((label|0) == 16) {
   label = 0;
   $111 = $1;
   $112 = (($111) + 36|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = $1;
   $115 = (($114) + 8|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = Math_imul($113, $116)|0;
   $118 = $1;
   $119 = (($118) + 28|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = $1;
   $122 = (($121) + 12|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = Math_imul($120, $123)|0;
   $125 = (($117) + ($124))|0;
   $126 = $1;
   $127 = (($126) + 24|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = $1;
   $130 = (($129) + 16|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = Math_imul($128, $131)|0;
   $133 = (($125) + ($132))|0;
   $134 = $1;
   $135 = (($134) + 32|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = $1;
   $138 = (($137) + 20|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = Math_imul($136, $139)|0;
   $141 = (($133) + ($140))|0;
   $index = $141;
   $142 = $index;
   $143 = $1;
   $144 = (($143) + 4|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = (($145) + ($142<<1)|0);
   $147 = HEAP16[$146>>1]|0;
   $148 = ($147<<16>>16)!=(0);
   if (!($148)) {
    label = 17;
    break;
   }
   label = 19;
   continue;
  }
  else if ((label|0) == 25) {
   label = 0;
   $166 = $1;
   $167 = (($166) + 28|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (($168) + 1)|0;
   HEAP32[$167>>2] = $169;
   label = 5;
   continue;
  }
 }
 if ((label|0) == 17) {
  $149 = $index;
  $150 = $1;
  $151 = (($150) + 4|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = (($152) + ($149<<1)|0);
  HEAP16[$153>>1] = 1;
  $0 = 1;
  $170 = $0;
  STACKTOP = sp;return ($170|0);
 }
 else if ((label|0) == 27) {
  $0 = 0;
  $170 = $0;
  STACKTOP = sp;return ($170|0);
 }
 return 0|0;
}
function _pi_next_rpcl($pi) {
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp = 0, $compno = 0, $dx = 0, $dy = 0, $index = 0, $levelno = 0, $prci = 0, $prcj = 0, $res = 0, $resno = 0, $rpx = 0;
 var $rpy = 0, $trx0 = 0, $trx1 = 0, $try0 = 0, $try1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $pi;
 $comp = 0;
 $res = 0;
 $index = 0;
 $2 = $1;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = $1;
  $7 = (($6) + 40|0);
  HEAP32[$7>>2] = 0;
  $8 = $1;
  $9 = (($8) + 224|0);
  HEAP32[$9>>2] = 0;
  $10 = $1;
  $11 = (($10) + 228|0);
  HEAP32[$11>>2] = 0;
  $compno = 0;
  while(1) {
   $12 = $compno;
   $13 = $1;
   $14 = (($13) + 192|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($12|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $compno;
   $18 = $1;
   $19 = (($18) + 196|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + ($17<<4)|0);
   $comp = $21;
   $resno = 0;
   while(1) {
    $22 = $resno;
    $23 = $comp;
    $24 = (($23) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)<($25|0);
    if (!($26)) {
     break;
    }
    $27 = $resno;
    $28 = $comp;
    $29 = (($28) + 12|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    $res = $31;
    $32 = $comp;
    $33 = HEAP32[$32>>2]|0;
    $34 = $res;
    $35 = HEAP32[$34>>2]|0;
    $36 = $comp;
    $37 = (($36) + 8|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (($35) + ($38))|0;
    $40 = (($39) - 1)|0;
    $41 = $resno;
    $42 = (($40) - ($41))|0;
    $43 = 1 << $42;
    $44 = Math_imul($33, $43)|0;
    $dx = $44;
    $45 = $comp;
    $46 = (($45) + 4|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = $res;
    $49 = (($48) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = $comp;
    $52 = (($51) + 8|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (($50) + ($53))|0;
    $55 = (($54) - 1)|0;
    $56 = $resno;
    $57 = (($55) - ($56))|0;
    $58 = 1 << $57;
    $59 = Math_imul($47, $58)|0;
    $dy = $59;
    $60 = $1;
    $61 = (($60) + 224|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($62|0)!=(0);
    if ($63) {
     $65 = $1;
     $66 = (($65) + 224|0);
     $67 = HEAP32[$66>>2]|0;
     $68 = $dx;
     $69 = (_int_min36($67,$68)|0);
     $72 = $69;
    } else {
     $64 = $dx;
     $72 = $64;
    }
    $70 = $1;
    $71 = (($70) + 224|0);
    HEAP32[$71>>2] = $72;
    $73 = $1;
    $74 = (($73) + 228|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ($75|0)!=(0);
    if ($76) {
     $78 = $1;
     $79 = (($78) + 228|0);
     $80 = HEAP32[$79>>2]|0;
     $81 = $dy;
     $82 = (_int_min36($80,$81)|0);
     $85 = $82;
    } else {
     $77 = $dy;
     $85 = $77;
    }
    $83 = $1;
    $84 = (($83) + 228|0);
    HEAP32[$84>>2] = $85;
    $86 = $resno;
    $87 = (($86) + 1)|0;
    $resno = $87;
   }
   $88 = $compno;
   $89 = (($88) + 1)|0;
   $compno = $89;
  }
  $90 = $1;
  $91 = HEAP8[$90>>0]|0;
  $92 = ($91<<24>>24)!=(0);
  if (!($92)) {
   $93 = $1;
   $94 = (($93) + 204|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = $1;
   $97 = (($96) + 44|0);
   $98 = (($97) + 60|0);
   HEAP32[$98>>2] = $95;
   $99 = $1;
   $100 = (($99) + 200|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = $1;
   $103 = (($102) + 44|0);
   $104 = (($103) + 52|0);
   HEAP32[$104>>2] = $101;
   $105 = $1;
   $106 = (($105) + 212|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = $1;
   $109 = (($108) + 44|0);
   $110 = (($109) + 64|0);
   HEAP32[$110>>2] = $107;
   $111 = $1;
   $112 = (($111) + 208|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = $1;
   $115 = (($114) + 44|0);
   $116 = (($115) + 56|0);
   HEAP32[$116>>2] = $113;
  }
  $117 = $1;
  $118 = (($117) + 44|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = $1;
  $121 = (($120) + 28|0);
  HEAP32[$121>>2] = $119;
  label = 21;
 } else {
  label = 49;
 }
 while(1) {
  if ((label|0) == 21) {
   label = 0;
   $122 = $1;
   $123 = (($122) + 28|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = $1;
   $126 = (($125) + 44|0);
   $127 = (($126) + 12|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = ($124|0)<($128|0);
   if (!($129)) {
    label = 59;
    break;
   }
   $130 = $1;
   $131 = (($130) + 44|0);
   $132 = (($131) + 60|0);
   $133 = HEAP32[$132>>2]|0;
   $134 = $1;
   $135 = (($134) + 220|0);
   HEAP32[$135>>2] = $133;
   label = 23;
  }
  else if ((label|0) == 49) {
   label = 0;
   $406 = $1;
   $407 = (($406) + 36|0);
   $408 = HEAP32[$407>>2]|0;
   $409 = (($408) + 1)|0;
   HEAP32[$407>>2] = $409;
   label = 45;
  }
  while(1) {
   if ((label|0) == 23) {
    label = 0;
    $136 = $1;
    $137 = (($136) + 220|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = $1;
    $140 = (($139) + 44|0);
    $141 = (($140) + 64|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ($138|0)<($142|0);
    if (!($143)) {
     label = 57;
     break;
    }
    $144 = $1;
    $145 = (($144) + 44|0);
    $146 = (($145) + 52|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = $1;
    $149 = (($148) + 216|0);
    HEAP32[$149>>2] = $147;
    label = 25;
   }
   else if ((label|0) == 45) {
    label = 0;
    $355 = $1;
    $356 = (($355) + 36|0);
    $357 = HEAP32[$356>>2]|0;
    $358 = $1;
    $359 = (($358) + 44|0);
    $360 = (($359) + 8|0);
    $361 = HEAP32[$360>>2]|0;
    $362 = ($357|0)<($361|0);
    if ($362) {
     label = 46;
     break;
    }
    label = 52;
   }
   L38: while(1) {
    if ((label|0) == 25) {
     label = 0;
     $150 = $1;
     $151 = (($150) + 216|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = $1;
     $154 = (($153) + 44|0);
     $155 = (($154) + 56|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($152|0)<($156|0);
     if (!($157)) {
      break;
     }
     $158 = $1;
     $159 = (($158) + 44|0);
     $160 = (($159) + 4|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = $1;
     $163 = (($162) + 24|0);
     HEAP32[$163>>2] = $161;
    }
    else if ((label|0) == 52) {
     label = 0;
     $410 = $1;
     $411 = (($410) + 24|0);
     $412 = HEAP32[$411>>2]|0;
     $413 = (($412) + 1)|0;
     HEAP32[$411>>2] = $413;
    }
    $164 = $1;
    $165 = (($164) + 24|0);
    $166 = HEAP32[$165>>2]|0;
    $167 = $1;
    $168 = (($167) + 44|0);
    $169 = (($168) + 16|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = ($166|0)<($170|0);
    if (!($171)) {
     $414 = $1;
     $415 = (($414) + 224|0);
     $416 = HEAP32[$415>>2]|0;
     $417 = $1;
     $418 = (($417) + 216|0);
     $419 = HEAP32[$418>>2]|0;
     $420 = $1;
     $421 = (($420) + 224|0);
     $422 = HEAP32[$421>>2]|0;
     $423 = (($419|0) % ($422|0))&-1;
     $424 = (($416) - ($423))|0;
     $425 = $1;
     $426 = (($425) + 216|0);
     $427 = HEAP32[$426>>2]|0;
     $428 = (($427) + ($424))|0;
     HEAP32[$426>>2] = $428;
     label = 25;
     continue;
    }
    $172 = $1;
    $173 = (($172) + 24|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = $1;
    $176 = (($175) + 196|0);
    $177 = HEAP32[$176>>2]|0;
    $178 = (($177) + ($174<<4)|0);
    $comp = $178;
    $179 = $1;
    $180 = (($179) + 28|0);
    $181 = HEAP32[$180>>2]|0;
    $182 = $comp;
    $183 = (($182) + 8|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = ($181|0)>=($184|0);
    if ($185) {
     label = 52;
     continue;
    }
    $186 = $1;
    $187 = (($186) + 28|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = $comp;
    $190 = (($189) + 12|0);
    $191 = HEAP32[$190>>2]|0;
    $192 = (($191) + ($188<<4)|0);
    $res = $192;
    $193 = $comp;
    $194 = (($193) + 8|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = (($195) - 1)|0;
    $197 = $1;
    $198 = (($197) + 28|0);
    $199 = HEAP32[$198>>2]|0;
    $200 = (($196) - ($199))|0;
    $levelno = $200;
    $201 = $1;
    $202 = (($201) + 200|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = $comp;
    $205 = HEAP32[$204>>2]|0;
    $206 = $levelno;
    $207 = $205 << $206;
    $208 = (_int_ceildiv39($203,$207)|0);
    $trx0 = $208;
    $209 = $1;
    $210 = (($209) + 204|0);
    $211 = HEAP32[$210>>2]|0;
    $212 = $comp;
    $213 = (($212) + 4|0);
    $214 = HEAP32[$213>>2]|0;
    $215 = $levelno;
    $216 = $214 << $215;
    $217 = (_int_ceildiv39($211,$216)|0);
    $try0 = $217;
    $218 = $1;
    $219 = (($218) + 208|0);
    $220 = HEAP32[$219>>2]|0;
    $221 = $comp;
    $222 = HEAP32[$221>>2]|0;
    $223 = $levelno;
    $224 = $222 << $223;
    $225 = (_int_ceildiv39($220,$224)|0);
    $trx1 = $225;
    $226 = $1;
    $227 = (($226) + 212|0);
    $228 = HEAP32[$227>>2]|0;
    $229 = $comp;
    $230 = (($229) + 4|0);
    $231 = HEAP32[$230>>2]|0;
    $232 = $levelno;
    $233 = $231 << $232;
    $234 = (_int_ceildiv39($228,$233)|0);
    $try1 = $234;
    $235 = $res;
    $236 = HEAP32[$235>>2]|0;
    $237 = $levelno;
    $238 = (($236) + ($237))|0;
    $rpx = $238;
    $239 = $res;
    $240 = (($239) + 4|0);
    $241 = HEAP32[$240>>2]|0;
    $242 = $levelno;
    $243 = (($241) + ($242))|0;
    $rpy = $243;
    $244 = $1;
    $245 = (($244) + 220|0);
    $246 = HEAP32[$245>>2]|0;
    $247 = $comp;
    $248 = (($247) + 4|0);
    $249 = HEAP32[$248>>2]|0;
    $250 = $rpy;
    $251 = $249 << $250;
    $252 = (($246|0) % ($251|0))&-1;
    $253 = ($252|0)==(0);
    do {
     if (!($253)) {
      $254 = $1;
      $255 = (($254) + 220|0);
      $256 = HEAP32[$255>>2]|0;
      $257 = $1;
      $258 = (($257) + 204|0);
      $259 = HEAP32[$258>>2]|0;
      $260 = ($256|0)==($259|0);
      if ($260) {
       $261 = $try0;
       $262 = $levelno;
       $263 = $261 << $262;
       $264 = $rpy;
       $265 = 1 << $264;
       $266 = (($263|0) % ($265|0))&-1;
       $267 = ($266|0)!=(0);
       if ($267) {
        break;
       }
      }
      label = 52;
      continue L38;
     }
    } while(0);
    $268 = $1;
    $269 = (($268) + 216|0);
    $270 = HEAP32[$269>>2]|0;
    $271 = $comp;
    $272 = HEAP32[$271>>2]|0;
    $273 = $rpx;
    $274 = $272 << $273;
    $275 = (($270|0) % ($274|0))&-1;
    $276 = ($275|0)==(0);
    do {
     if (!($276)) {
      $277 = $1;
      $278 = (($277) + 216|0);
      $279 = HEAP32[$278>>2]|0;
      $280 = $1;
      $281 = (($280) + 200|0);
      $282 = HEAP32[$281>>2]|0;
      $283 = ($279|0)==($282|0);
      if ($283) {
       $284 = $trx0;
       $285 = $levelno;
       $286 = $284 << $285;
       $287 = $rpx;
       $288 = 1 << $287;
       $289 = (($286|0) % ($288|0))&-1;
       $290 = ($289|0)!=(0);
       if ($290) {
        break;
       }
      }
      label = 52;
      continue L38;
     }
    } while(0);
    $291 = $res;
    $292 = (($291) + 8|0);
    $293 = HEAP32[$292>>2]|0;
    $294 = ($293|0)==(0);
    if (!($294)) {
     $295 = $res;
     $296 = (($295) + 12|0);
     $297 = HEAP32[$296>>2]|0;
     $298 = ($297|0)==(0);
     if (!($298)) {
      $299 = $trx0;
      $300 = $trx1;
      $301 = ($299|0)==($300|0);
      if (!($301)) {
       $302 = $try0;
       $303 = $try1;
       $304 = ($302|0)==($303|0);
       if (!($304)) {
        label = 44;
        break;
       }
      }
      label = 52;
      continue;
     }
    }
    label = 52;
   }
   if ((label|0) == 44) {
    label = 0;
    $305 = $1;
    $306 = (($305) + 216|0);
    $307 = HEAP32[$306>>2]|0;
    $308 = $comp;
    $309 = HEAP32[$308>>2]|0;
    $310 = $levelno;
    $311 = $309 << $310;
    $312 = (_int_ceildiv39($307,$311)|0);
    $313 = $res;
    $314 = HEAP32[$313>>2]|0;
    $315 = (_int_floordivpow2($312,$314)|0);
    $316 = $trx0;
    $317 = $res;
    $318 = HEAP32[$317>>2]|0;
    $319 = (_int_floordivpow2($316,$318)|0);
    $320 = (($315) - ($319))|0;
    $prci = $320;
    $321 = $1;
    $322 = (($321) + 220|0);
    $323 = HEAP32[$322>>2]|0;
    $324 = $comp;
    $325 = (($324) + 4|0);
    $326 = HEAP32[$325>>2]|0;
    $327 = $levelno;
    $328 = $326 << $327;
    $329 = (_int_ceildiv39($323,$328)|0);
    $330 = $res;
    $331 = (($330) + 4|0);
    $332 = HEAP32[$331>>2]|0;
    $333 = (_int_floordivpow2($329,$332)|0);
    $334 = $try0;
    $335 = $res;
    $336 = (($335) + 4|0);
    $337 = HEAP32[$336>>2]|0;
    $338 = (_int_floordivpow2($334,$337)|0);
    $339 = (($333) - ($338))|0;
    $prcj = $339;
    $340 = $prci;
    $341 = $prcj;
    $342 = $res;
    $343 = (($342) + 8|0);
    $344 = HEAP32[$343>>2]|0;
    $345 = Math_imul($341, $344)|0;
    $346 = (($340) + ($345))|0;
    $347 = $1;
    $348 = (($347) + 32|0);
    HEAP32[$348>>2] = $346;
    $349 = $1;
    $350 = (($349) + 44|0);
    $351 = (($350) + 20|0);
    $352 = HEAP32[$351>>2]|0;
    $353 = $1;
    $354 = (($353) + 36|0);
    HEAP32[$354>>2] = $352;
    label = 45;
    continue;
   }
   $429 = $1;
   $430 = (($429) + 228|0);
   $431 = HEAP32[$430>>2]|0;
   $432 = $1;
   $433 = (($432) + 220|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = $1;
   $436 = (($435) + 228|0);
   $437 = HEAP32[$436>>2]|0;
   $438 = (($434|0) % ($437|0))&-1;
   $439 = (($431) - ($438))|0;
   $440 = $1;
   $441 = (($440) + 220|0);
   $442 = HEAP32[$441>>2]|0;
   $443 = (($442) + ($439))|0;
   HEAP32[$441>>2] = $443;
   label = 23;
  }
  if ((label|0) == 46) {
   label = 0;
   $363 = $1;
   $364 = (($363) + 36|0);
   $365 = HEAP32[$364>>2]|0;
   $366 = $1;
   $367 = (($366) + 8|0);
   $368 = HEAP32[$367>>2]|0;
   $369 = Math_imul($365, $368)|0;
   $370 = $1;
   $371 = (($370) + 28|0);
   $372 = HEAP32[$371>>2]|0;
   $373 = $1;
   $374 = (($373) + 12|0);
   $375 = HEAP32[$374>>2]|0;
   $376 = Math_imul($372, $375)|0;
   $377 = (($369) + ($376))|0;
   $378 = $1;
   $379 = (($378) + 24|0);
   $380 = HEAP32[$379>>2]|0;
   $381 = $1;
   $382 = (($381) + 16|0);
   $383 = HEAP32[$382>>2]|0;
   $384 = Math_imul($380, $383)|0;
   $385 = (($377) + ($384))|0;
   $386 = $1;
   $387 = (($386) + 32|0);
   $388 = HEAP32[$387>>2]|0;
   $389 = $1;
   $390 = (($389) + 20|0);
   $391 = HEAP32[$390>>2]|0;
   $392 = Math_imul($388, $391)|0;
   $393 = (($385) + ($392))|0;
   $index = $393;
   $394 = $index;
   $395 = $1;
   $396 = (($395) + 4|0);
   $397 = HEAP32[$396>>2]|0;
   $398 = (($397) + ($394<<1)|0);
   $399 = HEAP16[$398>>1]|0;
   $400 = ($399<<16>>16)!=(0);
   if (!($400)) {
    label = 47;
    break;
   }
   label = 49;
   continue;
  }
  else if ((label|0) == 57) {
   label = 0;
   $444 = $1;
   $445 = (($444) + 28|0);
   $446 = HEAP32[$445>>2]|0;
   $447 = (($446) + 1)|0;
   HEAP32[$445>>2] = $447;
   label = 21;
   continue;
  }
 }
 if ((label|0) == 47) {
  $401 = $index;
  $402 = $1;
  $403 = (($402) + 4|0);
  $404 = HEAP32[$403>>2]|0;
  $405 = (($404) + ($401<<1)|0);
  HEAP16[$405>>1] = 1;
  $0 = 1;
  $448 = $0;
  STACKTOP = sp;return ($448|0);
 }
 else if ((label|0) == 59) {
  $0 = 0;
  $448 = $0;
  STACKTOP = sp;return ($448|0);
 }
 return 0|0;
}
function _pi_next_pcrl($pi) {
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp = 0, $compno = 0, $dx = 0, $dy = 0, $index = 0, $levelno = 0, $prci = 0;
 var $prcj = 0, $res = 0, $resno = 0, $rpx = 0, $rpy = 0, $trx0 = 0, $trx1 = 0, $try0 = 0, $try1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $pi;
 $comp = 0;
 $res = 0;
 $index = 0;
 $2 = $1;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $13 = $1;
  $14 = (($13) + 40|0);
  HEAP32[$14>>2] = 0;
  $15 = $1;
  $16 = (($15) + 224|0);
  HEAP32[$16>>2] = 0;
  $17 = $1;
  $18 = (($17) + 228|0);
  HEAP32[$18>>2] = 0;
  $compno = 0;
  while(1) {
   $19 = $compno;
   $20 = $1;
   $21 = (($20) + 192|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)<($22|0);
   if (!($23)) {
    break;
   }
   $24 = $compno;
   $25 = $1;
   $26 = (($25) + 196|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<4)|0);
   $comp = $28;
   $resno = 0;
   while(1) {
    $29 = $resno;
    $30 = $comp;
    $31 = (($30) + 8|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($29|0)<($32|0);
    if (!($33)) {
     break;
    }
    $34 = $resno;
    $35 = $comp;
    $36 = (($35) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + ($34<<4)|0);
    $res = $38;
    $39 = $comp;
    $40 = HEAP32[$39>>2]|0;
    $41 = $res;
    $42 = HEAP32[$41>>2]|0;
    $43 = $comp;
    $44 = (($43) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (($42) + ($45))|0;
    $47 = (($46) - 1)|0;
    $48 = $resno;
    $49 = (($47) - ($48))|0;
    $50 = 1 << $49;
    $51 = Math_imul($40, $50)|0;
    $dx = $51;
    $52 = $comp;
    $53 = (($52) + 4|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = $res;
    $56 = (($55) + 4|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = $comp;
    $59 = (($58) + 8|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = (($57) + ($60))|0;
    $62 = (($61) - 1)|0;
    $63 = $resno;
    $64 = (($62) - ($63))|0;
    $65 = 1 << $64;
    $66 = Math_imul($54, $65)|0;
    $dy = $66;
    $67 = $1;
    $68 = (($67) + 224|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = ($69|0)!=(0);
    if ($70) {
     $72 = $1;
     $73 = (($72) + 224|0);
     $74 = HEAP32[$73>>2]|0;
     $75 = $dx;
     $76 = (_int_min36($74,$75)|0);
     $79 = $76;
    } else {
     $71 = $dx;
     $79 = $71;
    }
    $77 = $1;
    $78 = (($77) + 224|0);
    HEAP32[$78>>2] = $79;
    $80 = $1;
    $81 = (($80) + 228|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = ($82|0)!=(0);
    if ($83) {
     $85 = $1;
     $86 = (($85) + 228|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = $dy;
     $89 = (_int_min36($87,$88)|0);
     $92 = $89;
    } else {
     $84 = $dy;
     $92 = $84;
    }
    $90 = $1;
    $91 = (($90) + 228|0);
    HEAP32[$91>>2] = $92;
    $93 = $resno;
    $94 = (($93) + 1)|0;
    $resno = $94;
   }
   $95 = $compno;
   $96 = (($95) + 1)|0;
   $compno = $96;
  }
  $97 = $1;
  $98 = HEAP8[$97>>0]|0;
  $99 = ($98<<24>>24)!=(0);
  if (!($99)) {
   $100 = $1;
   $101 = (($100) + 204|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = $1;
   $104 = (($103) + 44|0);
   $105 = (($104) + 60|0);
   HEAP32[$105>>2] = $102;
   $106 = $1;
   $107 = (($106) + 200|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = $1;
   $110 = (($109) + 44|0);
   $111 = (($110) + 52|0);
   HEAP32[$111>>2] = $108;
   $112 = $1;
   $113 = (($112) + 212|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = $1;
   $116 = (($115) + 44|0);
   $117 = (($116) + 64|0);
   HEAP32[$117>>2] = $114;
   $118 = $1;
   $119 = (($118) + 208|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = $1;
   $122 = (($121) + 44|0);
   $123 = (($122) + 56|0);
   HEAP32[$123>>2] = $120;
  }
  $124 = $1;
  $125 = (($124) + 44|0);
  $126 = (($125) + 60|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = $1;
  $129 = (($128) + 220|0);
  HEAP32[$129>>2] = $127;
  label = 21;
 } else {
  $6 = $1;
  $7 = (($6) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $1;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $comp = $12;
  label = 47;
 }
 while(1) {
  if ((label|0) == 21) {
   label = 0;
   $130 = $1;
   $131 = (($130) + 220|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = $1;
   $134 = (($133) + 44|0);
   $135 = (($134) + 64|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = ($132|0)<($136|0);
   if (!($137)) {
    label = 57;
    break;
   }
   $138 = $1;
   $139 = (($138) + 44|0);
   $140 = (($139) + 52|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = $1;
   $143 = (($142) + 216|0);
   HEAP32[$143>>2] = $141;
   label = 23;
  }
  else if ((label|0) == 47) {
   label = 0;
   $410 = $1;
   $411 = (($410) + 36|0);
   $412 = HEAP32[$411>>2]|0;
   $413 = (($412) + 1)|0;
   HEAP32[$411>>2] = $413;
   label = 43;
  }
  while(1) {
   if ((label|0) == 23) {
    label = 0;
    $144 = $1;
    $145 = (($144) + 216|0);
    $146 = HEAP32[$145>>2]|0;
    $147 = $1;
    $148 = (($147) + 44|0);
    $149 = (($148) + 56|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = ($146|0)<($150|0);
    if (!($151)) {
     label = 55;
     break;
    }
    $152 = $1;
    $153 = (($152) + 44|0);
    $154 = (($153) + 4|0);
    $155 = HEAP32[$154>>2]|0;
    $156 = $1;
    $157 = (($156) + 24|0);
    HEAP32[$157>>2] = $155;
    label = 25;
   }
   else if ((label|0) == 43) {
    label = 0;
    $359 = $1;
    $360 = (($359) + 36|0);
    $361 = HEAP32[$360>>2]|0;
    $362 = $1;
    $363 = (($362) + 44|0);
    $364 = (($363) + 8|0);
    $365 = HEAP32[$364>>2]|0;
    $366 = ($361|0)<($365|0);
    if ($366) {
     label = 44;
     break;
    }
    label = 50;
   }
   L38: while(1) {
    if ((label|0) == 25) {
     label = 0;
     $158 = $1;
     $159 = (($158) + 24|0);
     $160 = HEAP32[$159>>2]|0;
     $161 = $1;
     $162 = (($161) + 44|0);
     $163 = (($162) + 16|0);
     $164 = HEAP32[$163>>2]|0;
     $165 = ($160|0)<($164|0);
     if (!($165)) {
      break;
     }
     $166 = $1;
     $167 = (($166) + 24|0);
     $168 = HEAP32[$167>>2]|0;
     $169 = $1;
     $170 = (($169) + 196|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = (($171) + ($168<<4)|0);
     $comp = $172;
     $173 = $1;
     $174 = (($173) + 44|0);
     $175 = HEAP32[$174>>2]|0;
     $176 = $1;
     $177 = (($176) + 28|0);
     HEAP32[$177>>2] = $175;
    }
    else if ((label|0) == 50) {
     label = 0;
     $414 = $1;
     $415 = (($414) + 28|0);
     $416 = HEAP32[$415>>2]|0;
     $417 = (($416) + 1)|0;
     HEAP32[$415>>2] = $417;
    }
    $178 = $1;
    $179 = (($178) + 28|0);
    $180 = HEAP32[$179>>2]|0;
    $181 = $1;
    $182 = (($181) + 44|0);
    $183 = (($182) + 12|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = $comp;
    $186 = (($185) + 8|0);
    $187 = HEAP32[$186>>2]|0;
    $188 = (_int_min36($184,$187)|0);
    $189 = ($180|0)<($188|0);
    if (!($189)) {
     $418 = $1;
     $419 = (($418) + 24|0);
     $420 = HEAP32[$419>>2]|0;
     $421 = (($420) + 1)|0;
     HEAP32[$419>>2] = $421;
     label = 25;
     continue;
    }
    $190 = $1;
    $191 = (($190) + 28|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = $comp;
    $194 = (($193) + 12|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = (($195) + ($192<<4)|0);
    $res = $196;
    $197 = $comp;
    $198 = (($197) + 8|0);
    $199 = HEAP32[$198>>2]|0;
    $200 = (($199) - 1)|0;
    $201 = $1;
    $202 = (($201) + 28|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = (($200) - ($203))|0;
    $levelno = $204;
    $205 = $1;
    $206 = (($205) + 200|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = $comp;
    $209 = HEAP32[$208>>2]|0;
    $210 = $levelno;
    $211 = $209 << $210;
    $212 = (_int_ceildiv39($207,$211)|0);
    $trx0 = $212;
    $213 = $1;
    $214 = (($213) + 204|0);
    $215 = HEAP32[$214>>2]|0;
    $216 = $comp;
    $217 = (($216) + 4|0);
    $218 = HEAP32[$217>>2]|0;
    $219 = $levelno;
    $220 = $218 << $219;
    $221 = (_int_ceildiv39($215,$220)|0);
    $try0 = $221;
    $222 = $1;
    $223 = (($222) + 208|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = $comp;
    $226 = HEAP32[$225>>2]|0;
    $227 = $levelno;
    $228 = $226 << $227;
    $229 = (_int_ceildiv39($224,$228)|0);
    $trx1 = $229;
    $230 = $1;
    $231 = (($230) + 212|0);
    $232 = HEAP32[$231>>2]|0;
    $233 = $comp;
    $234 = (($233) + 4|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = $levelno;
    $237 = $235 << $236;
    $238 = (_int_ceildiv39($232,$237)|0);
    $try1 = $238;
    $239 = $res;
    $240 = HEAP32[$239>>2]|0;
    $241 = $levelno;
    $242 = (($240) + ($241))|0;
    $rpx = $242;
    $243 = $res;
    $244 = (($243) + 4|0);
    $245 = HEAP32[$244>>2]|0;
    $246 = $levelno;
    $247 = (($245) + ($246))|0;
    $rpy = $247;
    $248 = $1;
    $249 = (($248) + 220|0);
    $250 = HEAP32[$249>>2]|0;
    $251 = $comp;
    $252 = (($251) + 4|0);
    $253 = HEAP32[$252>>2]|0;
    $254 = $rpy;
    $255 = $253 << $254;
    $256 = (($250|0) % ($255|0))&-1;
    $257 = ($256|0)==(0);
    do {
     if (!($257)) {
      $258 = $1;
      $259 = (($258) + 220|0);
      $260 = HEAP32[$259>>2]|0;
      $261 = $1;
      $262 = (($261) + 204|0);
      $263 = HEAP32[$262>>2]|0;
      $264 = ($260|0)==($263|0);
      if ($264) {
       $265 = $try0;
       $266 = $levelno;
       $267 = $265 << $266;
       $268 = $rpy;
       $269 = 1 << $268;
       $270 = (($267|0) % ($269|0))&-1;
       $271 = ($270|0)!=(0);
       if ($271) {
        break;
       }
      }
      label = 50;
      continue L38;
     }
    } while(0);
    $272 = $1;
    $273 = (($272) + 216|0);
    $274 = HEAP32[$273>>2]|0;
    $275 = $comp;
    $276 = HEAP32[$275>>2]|0;
    $277 = $rpx;
    $278 = $276 << $277;
    $279 = (($274|0) % ($278|0))&-1;
    $280 = ($279|0)==(0);
    do {
     if (!($280)) {
      $281 = $1;
      $282 = (($281) + 216|0);
      $283 = HEAP32[$282>>2]|0;
      $284 = $1;
      $285 = (($284) + 200|0);
      $286 = HEAP32[$285>>2]|0;
      $287 = ($283|0)==($286|0);
      if ($287) {
       $288 = $trx0;
       $289 = $levelno;
       $290 = $288 << $289;
       $291 = $rpx;
       $292 = 1 << $291;
       $293 = (($290|0) % ($292|0))&-1;
       $294 = ($293|0)!=(0);
       if ($294) {
        break;
       }
      }
      label = 50;
      continue L38;
     }
    } while(0);
    $295 = $res;
    $296 = (($295) + 8|0);
    $297 = HEAP32[$296>>2]|0;
    $298 = ($297|0)==(0);
    if (!($298)) {
     $299 = $res;
     $300 = (($299) + 12|0);
     $301 = HEAP32[$300>>2]|0;
     $302 = ($301|0)==(0);
     if (!($302)) {
      $303 = $trx0;
      $304 = $trx1;
      $305 = ($303|0)==($304|0);
      if (!($305)) {
       $306 = $try0;
       $307 = $try1;
       $308 = ($306|0)==($307|0);
       if (!($308)) {
        label = 42;
        break;
       }
      }
      label = 50;
      continue;
     }
    }
    label = 50;
   }
   if ((label|0) == 42) {
    label = 0;
    $309 = $1;
    $310 = (($309) + 216|0);
    $311 = HEAP32[$310>>2]|0;
    $312 = $comp;
    $313 = HEAP32[$312>>2]|0;
    $314 = $levelno;
    $315 = $313 << $314;
    $316 = (_int_ceildiv39($311,$315)|0);
    $317 = $res;
    $318 = HEAP32[$317>>2]|0;
    $319 = (_int_floordivpow2($316,$318)|0);
    $320 = $trx0;
    $321 = $res;
    $322 = HEAP32[$321>>2]|0;
    $323 = (_int_floordivpow2($320,$322)|0);
    $324 = (($319) - ($323))|0;
    $prci = $324;
    $325 = $1;
    $326 = (($325) + 220|0);
    $327 = HEAP32[$326>>2]|0;
    $328 = $comp;
    $329 = (($328) + 4|0);
    $330 = HEAP32[$329>>2]|0;
    $331 = $levelno;
    $332 = $330 << $331;
    $333 = (_int_ceildiv39($327,$332)|0);
    $334 = $res;
    $335 = (($334) + 4|0);
    $336 = HEAP32[$335>>2]|0;
    $337 = (_int_floordivpow2($333,$336)|0);
    $338 = $try0;
    $339 = $res;
    $340 = (($339) + 4|0);
    $341 = HEAP32[$340>>2]|0;
    $342 = (_int_floordivpow2($338,$341)|0);
    $343 = (($337) - ($342))|0;
    $prcj = $343;
    $344 = $prci;
    $345 = $prcj;
    $346 = $res;
    $347 = (($346) + 8|0);
    $348 = HEAP32[$347>>2]|0;
    $349 = Math_imul($345, $348)|0;
    $350 = (($344) + ($349))|0;
    $351 = $1;
    $352 = (($351) + 32|0);
    HEAP32[$352>>2] = $350;
    $353 = $1;
    $354 = (($353) + 44|0);
    $355 = (($354) + 20|0);
    $356 = HEAP32[$355>>2]|0;
    $357 = $1;
    $358 = (($357) + 36|0);
    HEAP32[$358>>2] = $356;
    label = 43;
    continue;
   }
   $422 = $1;
   $423 = (($422) + 224|0);
   $424 = HEAP32[$423>>2]|0;
   $425 = $1;
   $426 = (($425) + 216|0);
   $427 = HEAP32[$426>>2]|0;
   $428 = $1;
   $429 = (($428) + 224|0);
   $430 = HEAP32[$429>>2]|0;
   $431 = (($427|0) % ($430|0))&-1;
   $432 = (($424) - ($431))|0;
   $433 = $1;
   $434 = (($433) + 216|0);
   $435 = HEAP32[$434>>2]|0;
   $436 = (($435) + ($432))|0;
   HEAP32[$434>>2] = $436;
   label = 23;
  }
  if ((label|0) == 44) {
   label = 0;
   $367 = $1;
   $368 = (($367) + 36|0);
   $369 = HEAP32[$368>>2]|0;
   $370 = $1;
   $371 = (($370) + 8|0);
   $372 = HEAP32[$371>>2]|0;
   $373 = Math_imul($369, $372)|0;
   $374 = $1;
   $375 = (($374) + 28|0);
   $376 = HEAP32[$375>>2]|0;
   $377 = $1;
   $378 = (($377) + 12|0);
   $379 = HEAP32[$378>>2]|0;
   $380 = Math_imul($376, $379)|0;
   $381 = (($373) + ($380))|0;
   $382 = $1;
   $383 = (($382) + 24|0);
   $384 = HEAP32[$383>>2]|0;
   $385 = $1;
   $386 = (($385) + 16|0);
   $387 = HEAP32[$386>>2]|0;
   $388 = Math_imul($384, $387)|0;
   $389 = (($381) + ($388))|0;
   $390 = $1;
   $391 = (($390) + 32|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = $1;
   $394 = (($393) + 20|0);
   $395 = HEAP32[$394>>2]|0;
   $396 = Math_imul($392, $395)|0;
   $397 = (($389) + ($396))|0;
   $index = $397;
   $398 = $index;
   $399 = $1;
   $400 = (($399) + 4|0);
   $401 = HEAP32[$400>>2]|0;
   $402 = (($401) + ($398<<1)|0);
   $403 = HEAP16[$402>>1]|0;
   $404 = ($403<<16>>16)!=(0);
   if (!($404)) {
    label = 45;
    break;
   }
   label = 47;
   continue;
  }
  else if ((label|0) == 55) {
   label = 0;
   $437 = $1;
   $438 = (($437) + 228|0);
   $439 = HEAP32[$438>>2]|0;
   $440 = $1;
   $441 = (($440) + 220|0);
   $442 = HEAP32[$441>>2]|0;
   $443 = $1;
   $444 = (($443) + 228|0);
   $445 = HEAP32[$444>>2]|0;
   $446 = (($442|0) % ($445|0))&-1;
   $447 = (($439) - ($446))|0;
   $448 = $1;
   $449 = (($448) + 220|0);
   $450 = HEAP32[$449>>2]|0;
   $451 = (($450) + ($447))|0;
   HEAP32[$449>>2] = $451;
   label = 21;
   continue;
  }
 }
 if ((label|0) == 45) {
  $405 = $index;
  $406 = $1;
  $407 = (($406) + 4|0);
  $408 = HEAP32[$407>>2]|0;
  $409 = (($408) + ($405<<1)|0);
  HEAP16[$409>>1] = 1;
  $0 = 1;
  $452 = $0;
  STACKTOP = sp;return ($452|0);
 }
 else if ((label|0) == 57) {
  $0 = 0;
  $452 = $0;
  STACKTOP = sp;return ($452|0);
 }
 return 0|0;
}
function _pi_next_cprl($pi) {
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $comp = 0, $dx = 0, $dy = 0, $index = 0, $levelno = 0, $prci = 0, $prcj = 0, $res = 0, $resno = 0, $rpx = 0, $rpy = 0, $trx0 = 0, $trx1 = 0, $try0 = 0, $try1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $pi;
 $comp = 0;
 $res = 0;
 $index = 0;
 $2 = $1;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $13 = $1;
  $14 = (($13) + 40|0);
  HEAP32[$14>>2] = 0;
  $15 = $1;
  $16 = (($15) + 44|0);
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $1;
  $20 = (($19) + 24|0);
  HEAP32[$20>>2] = $18;
  label = 5;
 } else {
  $6 = $1;
  $7 = (($6) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $1;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $comp = $12;
  label = 43;
 }
 while(1) {
  if ((label|0) == 5) {
   label = 0;
   $21 = $1;
   $22 = (($21) + 24|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = $1;
   $25 = (($24) + 44|0);
   $26 = (($25) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($23|0)<($27|0);
   if (!($28)) {
    label = 53;
    break;
   }
   $29 = $1;
   $30 = (($29) + 24|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $1;
   $33 = (($32) + 196|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<4)|0);
   $comp = $35;
   $36 = $1;
   $37 = (($36) + 224|0);
   HEAP32[$37>>2] = 0;
   $38 = $1;
   $39 = (($38) + 228|0);
   HEAP32[$39>>2] = 0;
   $resno = 0;
   while(1) {
    $40 = $resno;
    $41 = $comp;
    $42 = (($41) + 8|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($40|0)<($43|0);
    if (!($44)) {
     break;
    }
    $45 = $resno;
    $46 = $comp;
    $47 = (($46) + 12|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + ($45<<4)|0);
    $res = $49;
    $50 = $comp;
    $51 = HEAP32[$50>>2]|0;
    $52 = $res;
    $53 = HEAP32[$52>>2]|0;
    $54 = $comp;
    $55 = (($54) + 8|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = (($53) + ($56))|0;
    $58 = (($57) - 1)|0;
    $59 = $resno;
    $60 = (($58) - ($59))|0;
    $61 = 1 << $60;
    $62 = Math_imul($51, $61)|0;
    $dx = $62;
    $63 = $comp;
    $64 = (($63) + 4|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = $res;
    $67 = (($66) + 4|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = $comp;
    $70 = (($69) + 8|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = (($68) + ($71))|0;
    $73 = (($72) - 1)|0;
    $74 = $resno;
    $75 = (($73) - ($74))|0;
    $76 = 1 << $75;
    $77 = Math_imul($65, $76)|0;
    $dy = $77;
    $78 = $1;
    $79 = (($78) + 224|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     $83 = $1;
     $84 = (($83) + 224|0);
     $85 = HEAP32[$84>>2]|0;
     $86 = $dx;
     $87 = (_int_min36($85,$86)|0);
     $90 = $87;
    } else {
     $82 = $dx;
     $90 = $82;
    }
    $88 = $1;
    $89 = (($88) + 224|0);
    HEAP32[$89>>2] = $90;
    $91 = $1;
    $92 = (($91) + 228|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = ($93|0)!=(0);
    if ($94) {
     $96 = $1;
     $97 = (($96) + 228|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = $dy;
     $100 = (_int_min36($98,$99)|0);
     $103 = $100;
    } else {
     $95 = $dy;
     $103 = $95;
    }
    $101 = $1;
    $102 = (($101) + 228|0);
    HEAP32[$102>>2] = $103;
    $104 = $resno;
    $105 = (($104) + 1)|0;
    $resno = $105;
   }
   $106 = $1;
   $107 = HEAP8[$106>>0]|0;
   $108 = ($107<<24>>24)!=(0);
   if (!($108)) {
    $109 = $1;
    $110 = (($109) + 204|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = $1;
    $113 = (($112) + 44|0);
    $114 = (($113) + 60|0);
    HEAP32[$114>>2] = $111;
    $115 = $1;
    $116 = (($115) + 200|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = $1;
    $119 = (($118) + 44|0);
    $120 = (($119) + 52|0);
    HEAP32[$120>>2] = $117;
    $121 = $1;
    $122 = (($121) + 212|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = $1;
    $125 = (($124) + 44|0);
    $126 = (($125) + 64|0);
    HEAP32[$126>>2] = $123;
    $127 = $1;
    $128 = (($127) + 208|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = $1;
    $131 = (($130) + 44|0);
    $132 = (($131) + 56|0);
    HEAP32[$132>>2] = $129;
   }
   $133 = $1;
   $134 = (($133) + 44|0);
   $135 = (($134) + 60|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = $1;
   $138 = (($137) + 220|0);
   HEAP32[$138>>2] = $136;
   label = 19;
  }
  else if ((label|0) == 43) {
   label = 0;
   $398 = $1;
   $399 = (($398) + 36|0);
   $400 = HEAP32[$399>>2]|0;
   $401 = (($400) + 1)|0;
   HEAP32[$399>>2] = $401;
   label = 39;
  }
  while(1) {
   if ((label|0) == 19) {
    label = 0;
    $139 = $1;
    $140 = (($139) + 220|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = $1;
    $143 = (($142) + 44|0);
    $144 = (($143) + 64|0);
    $145 = HEAP32[$144>>2]|0;
    $146 = ($141|0)<($145|0);
    if (!($146)) {
     label = 51;
     break;
    }
    $147 = $1;
    $148 = (($147) + 44|0);
    $149 = (($148) + 52|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = $1;
    $152 = (($151) + 216|0);
    HEAP32[$152>>2] = $150;
    label = 21;
   }
   else if ((label|0) == 39) {
    label = 0;
    $347 = $1;
    $348 = (($347) + 36|0);
    $349 = HEAP32[$348>>2]|0;
    $350 = $1;
    $351 = (($350) + 44|0);
    $352 = (($351) + 8|0);
    $353 = HEAP32[$352>>2]|0;
    $354 = ($349|0)<($353|0);
    if ($354) {
     label = 40;
     break;
    }
    label = 46;
   }
   L33: while(1) {
    if ((label|0) == 21) {
     label = 0;
     $153 = $1;
     $154 = (($153) + 216|0);
     $155 = HEAP32[$154>>2]|0;
     $156 = $1;
     $157 = (($156) + 44|0);
     $158 = (($157) + 56|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ($155|0)<($159|0);
     if (!($160)) {
      break;
     }
     $161 = $1;
     $162 = (($161) + 44|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = $1;
     $165 = (($164) + 28|0);
     HEAP32[$165>>2] = $163;
    }
    else if ((label|0) == 46) {
     label = 0;
     $402 = $1;
     $403 = (($402) + 28|0);
     $404 = HEAP32[$403>>2]|0;
     $405 = (($404) + 1)|0;
     HEAP32[$403>>2] = $405;
    }
    $166 = $1;
    $167 = (($166) + 28|0);
    $168 = HEAP32[$167>>2]|0;
    $169 = $1;
    $170 = (($169) + 44|0);
    $171 = (($170) + 12|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = $comp;
    $174 = (($173) + 8|0);
    $175 = HEAP32[$174>>2]|0;
    $176 = (_int_min36($172,$175)|0);
    $177 = ($168|0)<($176|0);
    if (!($177)) {
     $406 = $1;
     $407 = (($406) + 224|0);
     $408 = HEAP32[$407>>2]|0;
     $409 = $1;
     $410 = (($409) + 216|0);
     $411 = HEAP32[$410>>2]|0;
     $412 = $1;
     $413 = (($412) + 224|0);
     $414 = HEAP32[$413>>2]|0;
     $415 = (($411|0) % ($414|0))&-1;
     $416 = (($408) - ($415))|0;
     $417 = $1;
     $418 = (($417) + 216|0);
     $419 = HEAP32[$418>>2]|0;
     $420 = (($419) + ($416))|0;
     HEAP32[$418>>2] = $420;
     label = 21;
     continue;
    }
    $178 = $1;
    $179 = (($178) + 28|0);
    $180 = HEAP32[$179>>2]|0;
    $181 = $comp;
    $182 = (($181) + 12|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = (($183) + ($180<<4)|0);
    $res = $184;
    $185 = $comp;
    $186 = (($185) + 8|0);
    $187 = HEAP32[$186>>2]|0;
    $188 = (($187) - 1)|0;
    $189 = $1;
    $190 = (($189) + 28|0);
    $191 = HEAP32[$190>>2]|0;
    $192 = (($188) - ($191))|0;
    $levelno = $192;
    $193 = $1;
    $194 = (($193) + 200|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = $comp;
    $197 = HEAP32[$196>>2]|0;
    $198 = $levelno;
    $199 = $197 << $198;
    $200 = (_int_ceildiv39($195,$199)|0);
    $trx0 = $200;
    $201 = $1;
    $202 = (($201) + 204|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = $comp;
    $205 = (($204) + 4|0);
    $206 = HEAP32[$205>>2]|0;
    $207 = $levelno;
    $208 = $206 << $207;
    $209 = (_int_ceildiv39($203,$208)|0);
    $try0 = $209;
    $210 = $1;
    $211 = (($210) + 208|0);
    $212 = HEAP32[$211>>2]|0;
    $213 = $comp;
    $214 = HEAP32[$213>>2]|0;
    $215 = $levelno;
    $216 = $214 << $215;
    $217 = (_int_ceildiv39($212,$216)|0);
    $trx1 = $217;
    $218 = $1;
    $219 = (($218) + 212|0);
    $220 = HEAP32[$219>>2]|0;
    $221 = $comp;
    $222 = (($221) + 4|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = $levelno;
    $225 = $223 << $224;
    $226 = (_int_ceildiv39($220,$225)|0);
    $try1 = $226;
    $227 = $res;
    $228 = HEAP32[$227>>2]|0;
    $229 = $levelno;
    $230 = (($228) + ($229))|0;
    $rpx = $230;
    $231 = $res;
    $232 = (($231) + 4|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = $levelno;
    $235 = (($233) + ($234))|0;
    $rpy = $235;
    $236 = $1;
    $237 = (($236) + 220|0);
    $238 = HEAP32[$237>>2]|0;
    $239 = $comp;
    $240 = (($239) + 4|0);
    $241 = HEAP32[$240>>2]|0;
    $242 = $rpy;
    $243 = $241 << $242;
    $244 = (($238|0) % ($243|0))&-1;
    $245 = ($244|0)==(0);
    do {
     if (!($245)) {
      $246 = $1;
      $247 = (($246) + 220|0);
      $248 = HEAP32[$247>>2]|0;
      $249 = $1;
      $250 = (($249) + 204|0);
      $251 = HEAP32[$250>>2]|0;
      $252 = ($248|0)==($251|0);
      if ($252) {
       $253 = $try0;
       $254 = $levelno;
       $255 = $253 << $254;
       $256 = $rpy;
       $257 = 1 << $256;
       $258 = (($255|0) % ($257|0))&-1;
       $259 = ($258|0)!=(0);
       if ($259) {
        break;
       }
      }
      label = 46;
      continue L33;
     }
    } while(0);
    $260 = $1;
    $261 = (($260) + 216|0);
    $262 = HEAP32[$261>>2]|0;
    $263 = $comp;
    $264 = HEAP32[$263>>2]|0;
    $265 = $rpx;
    $266 = $264 << $265;
    $267 = (($262|0) % ($266|0))&-1;
    $268 = ($267|0)==(0);
    do {
     if (!($268)) {
      $269 = $1;
      $270 = (($269) + 216|0);
      $271 = HEAP32[$270>>2]|0;
      $272 = $1;
      $273 = (($272) + 200|0);
      $274 = HEAP32[$273>>2]|0;
      $275 = ($271|0)==($274|0);
      if ($275) {
       $276 = $trx0;
       $277 = $levelno;
       $278 = $276 << $277;
       $279 = $rpx;
       $280 = 1 << $279;
       $281 = (($278|0) % ($280|0))&-1;
       $282 = ($281|0)!=(0);
       if ($282) {
        break;
       }
      }
      label = 46;
      continue L33;
     }
    } while(0);
    $283 = $res;
    $284 = (($283) + 8|0);
    $285 = HEAP32[$284>>2]|0;
    $286 = ($285|0)==(0);
    if (!($286)) {
     $287 = $res;
     $288 = (($287) + 12|0);
     $289 = HEAP32[$288>>2]|0;
     $290 = ($289|0)==(0);
     if (!($290)) {
      $291 = $trx0;
      $292 = $trx1;
      $293 = ($291|0)==($292|0);
      if (!($293)) {
       $294 = $try0;
       $295 = $try1;
       $296 = ($294|0)==($295|0);
       if (!($296)) {
        label = 38;
        break;
       }
      }
      label = 46;
      continue;
     }
    }
    label = 46;
   }
   if ((label|0) == 38) {
    label = 0;
    $297 = $1;
    $298 = (($297) + 216|0);
    $299 = HEAP32[$298>>2]|0;
    $300 = $comp;
    $301 = HEAP32[$300>>2]|0;
    $302 = $levelno;
    $303 = $301 << $302;
    $304 = (_int_ceildiv39($299,$303)|0);
    $305 = $res;
    $306 = HEAP32[$305>>2]|0;
    $307 = (_int_floordivpow2($304,$306)|0);
    $308 = $trx0;
    $309 = $res;
    $310 = HEAP32[$309>>2]|0;
    $311 = (_int_floordivpow2($308,$310)|0);
    $312 = (($307) - ($311))|0;
    $prci = $312;
    $313 = $1;
    $314 = (($313) + 220|0);
    $315 = HEAP32[$314>>2]|0;
    $316 = $comp;
    $317 = (($316) + 4|0);
    $318 = HEAP32[$317>>2]|0;
    $319 = $levelno;
    $320 = $318 << $319;
    $321 = (_int_ceildiv39($315,$320)|0);
    $322 = $res;
    $323 = (($322) + 4|0);
    $324 = HEAP32[$323>>2]|0;
    $325 = (_int_floordivpow2($321,$324)|0);
    $326 = $try0;
    $327 = $res;
    $328 = (($327) + 4|0);
    $329 = HEAP32[$328>>2]|0;
    $330 = (_int_floordivpow2($326,$329)|0);
    $331 = (($325) - ($330))|0;
    $prcj = $331;
    $332 = $prci;
    $333 = $prcj;
    $334 = $res;
    $335 = (($334) + 8|0);
    $336 = HEAP32[$335>>2]|0;
    $337 = Math_imul($333, $336)|0;
    $338 = (($332) + ($337))|0;
    $339 = $1;
    $340 = (($339) + 32|0);
    HEAP32[$340>>2] = $338;
    $341 = $1;
    $342 = (($341) + 44|0);
    $343 = (($342) + 20|0);
    $344 = HEAP32[$343>>2]|0;
    $345 = $1;
    $346 = (($345) + 36|0);
    HEAP32[$346>>2] = $344;
    label = 39;
    continue;
   }
   $421 = $1;
   $422 = (($421) + 228|0);
   $423 = HEAP32[$422>>2]|0;
   $424 = $1;
   $425 = (($424) + 220|0);
   $426 = HEAP32[$425>>2]|0;
   $427 = $1;
   $428 = (($427) + 228|0);
   $429 = HEAP32[$428>>2]|0;
   $430 = (($426|0) % ($429|0))&-1;
   $431 = (($423) - ($430))|0;
   $432 = $1;
   $433 = (($432) + 220|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = (($434) + ($431))|0;
   HEAP32[$433>>2] = $435;
   label = 19;
  }
  if ((label|0) == 40) {
   label = 0;
   $355 = $1;
   $356 = (($355) + 36|0);
   $357 = HEAP32[$356>>2]|0;
   $358 = $1;
   $359 = (($358) + 8|0);
   $360 = HEAP32[$359>>2]|0;
   $361 = Math_imul($357, $360)|0;
   $362 = $1;
   $363 = (($362) + 28|0);
   $364 = HEAP32[$363>>2]|0;
   $365 = $1;
   $366 = (($365) + 12|0);
   $367 = HEAP32[$366>>2]|0;
   $368 = Math_imul($364, $367)|0;
   $369 = (($361) + ($368))|0;
   $370 = $1;
   $371 = (($370) + 24|0);
   $372 = HEAP32[$371>>2]|0;
   $373 = $1;
   $374 = (($373) + 16|0);
   $375 = HEAP32[$374>>2]|0;
   $376 = Math_imul($372, $375)|0;
   $377 = (($369) + ($376))|0;
   $378 = $1;
   $379 = (($378) + 32|0);
   $380 = HEAP32[$379>>2]|0;
   $381 = $1;
   $382 = (($381) + 20|0);
   $383 = HEAP32[$382>>2]|0;
   $384 = Math_imul($380, $383)|0;
   $385 = (($377) + ($384))|0;
   $index = $385;
   $386 = $index;
   $387 = $1;
   $388 = (($387) + 4|0);
   $389 = HEAP32[$388>>2]|0;
   $390 = (($389) + ($386<<1)|0);
   $391 = HEAP16[$390>>1]|0;
   $392 = ($391<<16>>16)!=(0);
   if (!($392)) {
    label = 41;
    break;
   }
   label = 43;
   continue;
  }
  else if ((label|0) == 51) {
   label = 0;
   $436 = $1;
   $437 = (($436) + 24|0);
   $438 = HEAP32[$437>>2]|0;
   $439 = (($438) + 1)|0;
   HEAP32[$437>>2] = $439;
   label = 5;
   continue;
  }
 }
 if ((label|0) == 41) {
  $393 = $index;
  $394 = $1;
  $395 = (($394) + 4|0);
  $396 = HEAP32[$395>>2]|0;
  $397 = (($396) + ($393<<1)|0);
  HEAP16[$397>>1] = 1;
  $0 = 1;
  $440 = $0;
  STACKTOP = sp;return ($440|0);
 }
 else if ((label|0) == 53) {
  $0 = 0;
  $440 = $0;
  STACKTOP = sp;return ($440|0);
 }
 return 0|0;
}
function _raw_create() {
 var $0 = 0, $1 = 0, $raw = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = (_malloc(28)|0);
 $raw = $0;
 $1 = $raw;
 STACKTOP = sp;return ($1|0);
}
function _raw_destroy($raw) {
 $raw = $raw|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $raw;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = $0;
  _free($3);
 }
 STACKTOP = sp;return;
}
function _raw_init_dec($raw,$bp,$len) {
 $raw = $raw|0;
 $bp = $bp|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $raw;
 $1 = $bp;
 $2 = $len;
 $3 = $1;
 $4 = $0;
 $5 = (($4) + 20|0);
 HEAP32[$5>>2] = $3;
 $6 = $2;
 $7 = $0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = (($9) + 12|0);
 HEAP32[$10>>2] = 0;
 $11 = $0;
 HEAP8[$11>>0] = 0;
 $12 = $0;
 $13 = (($12) + 4|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _raw_decode($raw) {
 $raw = $raw|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $raw;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 4|0);
  HEAP32[$6>>2] = 8;
  $7 = $0;
  $8 = (($7) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)==($12|0);
  if ($13) {
   $14 = $0;
   HEAP8[$14>>0] = -1;
  } else {
   $15 = $0;
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   $18 = ($17|0)==(255);
   if ($18) {
    $19 = $0;
    $20 = (($19) + 4|0);
    HEAP32[$20>>2] = 7;
   }
   $21 = $0;
   $22 = (($21) + 20|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = $0;
   $25 = (($24) + 12|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($23) + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $0;
   HEAP8[$29>>0] = $28;
   $30 = $0;
   $31 = (($30) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 1)|0;
   HEAP32[$31>>2] = $33;
  }
 }
 $34 = $0;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + -1)|0;
 HEAP32[$35>>2] = $37;
 $38 = $0;
 $39 = HEAP8[$38>>0]|0;
 $40 = $39&255;
 $41 = $0;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $40 >> $43;
 $45 = $44 & 1;
 $d = $45;
 $46 = $d;
 STACKTOP = sp;return ($46|0);
}
function _t1_create($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $t1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = (_malloc(40)|0);
 $t1 = $2;
 $3 = $t1;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $1;
  $6 = $t1;
  HEAP32[$6>>2] = $5;
  $7 = (_mqc_create()|0);
  $8 = $t1;
  $9 = (($8) + 4|0);
  HEAP32[$9>>2] = $7;
  $10 = (_raw_create()|0);
  $11 = $t1;
  $12 = (($11) + 8|0);
  HEAP32[$12>>2] = $10;
  $13 = $t1;
  $14 = (($13) + 12|0);
  HEAP32[$14>>2] = 0;
  $15 = $t1;
  $16 = (($15) + 16|0);
  HEAP32[$16>>2] = 0;
  $17 = $t1;
  $18 = (($17) + 28|0);
  HEAP32[$18>>2] = 0;
  $19 = $t1;
  $20 = (($19) + 32|0);
  HEAP32[$20>>2] = 0;
  $21 = $t1;
  $0 = $21;
  $22 = $0;
  STACKTOP = sp;return ($22|0);
 } else {
  $0 = 0;
  $22 = $0;
  STACKTOP = sp;return ($22|0);
 }
 return 0|0;
}
function _t1_destroy($t1) {
 $t1 = $t1|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $t1;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _mqc_destroy($5);
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 _raw_destroy($8);
 $9 = $0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 _free($11);
 $12 = $0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 _free($14);
 $15 = $0;
 _free($15);
 STACKTOP = sp;return;
}
function _allocate_buffers($t1,$w,$h) {
 $t1 = $t1|0;
 $w = $w|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $datasize = 0, $flagssize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $t1;
 $2 = $w;
 $3 = $h;
 $4 = $2;
 $5 = $3;
 $6 = Math_imul($4, $5)|0;
 $datasize = $6;
 $7 = $datasize;
 $8 = $1;
 $9 = (($8) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7|0)>($10|0);
 do {
  if ($11) {
   $12 = $1;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   _free($14);
   $15 = $datasize;
   $16 = $15<<2;
   $17 = (_memalign(16,$16)|0);
   $18 = $1;
   $19 = (($18) + 12|0);
   HEAP32[$19>>2] = $17;
   $20 = $1;
   $21 = (($20) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)!=(0|0);
   if ($23) {
    $24 = $datasize;
    $25 = $1;
    $26 = (($25) + 28|0);
    HEAP32[$26>>2] = $24;
    break;
   }
   $0 = 0;
   $73 = $0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $27 = $1;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $datasize;
 $31 = $30<<2;
 _memset(($29|0),0,($31|0))|0;
 $32 = $2;
 $33 = (($32) + 2)|0;
 $34 = $1;
 $35 = (($34) + 36|0);
 HEAP32[$35>>2] = $33;
 $36 = $1;
 $37 = (($36) + 36|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = $3;
 $40 = (($39) + 2)|0;
 $41 = Math_imul($38, $40)|0;
 $flagssize = $41;
 $42 = $flagssize;
 $43 = $1;
 $44 = (($43) + 32|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($42|0)>($45|0);
 do {
  if ($46) {
   $47 = $1;
   $48 = (($47) + 16|0);
   $49 = HEAP32[$48>>2]|0;
   _free($49);
   $50 = $flagssize;
   $51 = $50<<1;
   $52 = (_memalign(16,$51)|0);
   $53 = $1;
   $54 = (($53) + 16|0);
   HEAP32[$54>>2] = $52;
   $55 = $1;
   $56 = (($55) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = ($57|0)!=(0|0);
   if ($58) {
    $59 = $flagssize;
    $60 = $1;
    $61 = (($60) + 32|0);
    HEAP32[$61>>2] = $59;
    break;
   }
   $0 = 0;
   $73 = $0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $62 = $1;
 $63 = (($62) + 16|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = $flagssize;
 $66 = $65<<1;
 _memset(($64|0),0,($66|0))|0;
 $67 = $2;
 $68 = $1;
 $69 = (($68) + 20|0);
 HEAP32[$69>>2] = $67;
 $70 = $3;
 $71 = $1;
 $72 = (($71) + 24|0);
 HEAP32[$72>>2] = $70;
 $0 = 1;
 $73 = $0;
 STACKTOP = sp;return ($73|0);
}
function _t1_decode_cblks($t1,$tilec,$tccp) {
 $t1 = $t1|0;
 $tilec = $tilec|0;
 $tccp = $tccp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0.0, $236 = 0, $237 = 0, $238 = 0.0, $239 = 0.0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $band = 0, $bandno = 0, $cblk = 0, $cblk_h = 0, $cblk_w = 0, $cblkno = 0, $datap = 0, $i = 0, $j = 0, $mag = 0, $precinct = 0;
 var $precno = 0, $pres = 0, $pres1 = 0, $res = 0, $resno = 0, $thresh = 0, $tile_w = 0, $tiledp = 0, $tiledp2 = 0, $tiledp23 = 0, $tmp = 0, $tmp4 = 0.0, $val = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = $t1;
 $1 = $tilec;
 $2 = $tccp;
 $3 = $1;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $1;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) - ($7))|0;
 $tile_w = $8;
 $resno = 0;
 while(1) {
  $9 = $resno;
  $10 = $1;
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $resno;
  $15 = $1;
  $16 = (($15) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($14*124)|0)|0);
  $res = $18;
  $bandno = 0;
  while(1) {
   $19 = $bandno;
   $20 = $res;
   $21 = (($20) + 24|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)<($22|0);
   if (!($23)) {
    break;
   }
   $24 = $bandno;
   $25 = $res;
   $26 = (($25) + 28|0);
   $27 = (($26) + ($24<<5)|0);
   $band = $27;
   $precno = 0;
   while(1) {
    $28 = $precno;
    $29 = $res;
    $30 = (($29) + 16|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = $res;
    $33 = (($32) + 20|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = Math_imul($31, $34)|0;
    $36 = ($28|0)<($35|0);
    if (!($36)) {
     break;
    }
    $37 = $precno;
    $38 = $band;
    $39 = (($38) + 20|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + (($37*36)|0)|0);
    $precinct = $41;
    $cblkno = 0;
    while(1) {
     $42 = $cblkno;
     $43 = $precinct;
     $44 = (($43) + 16|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = $precinct;
     $47 = (($46) + 20|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = Math_imul($45, $48)|0;
     $50 = ($42|0)<($49|0);
     if (!($50)) {
      break;
     }
     $51 = $cblkno;
     $52 = $precinct;
     $53 = (($52) + 24|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = (($54) + (($51*44)|0)|0);
     $cblk = $55;
     $56 = $0;
     $57 = $cblk;
     $58 = $band;
     $59 = (($58) + 16|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = $2;
     $62 = (($61) + 808|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = $2;
     $65 = (($64) + 16|0);
     $66 = HEAP32[$65>>2]|0;
     _t1_decode_cblk($56,$57,$60,$63,$66);
     $67 = $cblk;
     $68 = (($67) + 8|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = $band;
     $71 = HEAP32[$70>>2]|0;
     $72 = (($69) - ($71))|0;
     $x = $72;
     $73 = $cblk;
     $74 = (($73) + 12|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = $band;
     $77 = (($76) + 4|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = (($75) - ($78))|0;
     $y = $79;
     $80 = $band;
     $81 = (($80) + 16|0);
     $82 = HEAP32[$81>>2]|0;
     $83 = $82 & 1;
     $84 = ($83|0)!=(0);
     if ($84) {
      $85 = $resno;
      $86 = (($85) - 1)|0;
      $87 = $1;
      $88 = (($87) + 20|0);
      $89 = HEAP32[$88>>2]|0;
      $90 = (($89) + (($86*124)|0)|0);
      $pres = $90;
      $91 = $pres;
      $92 = (($91) + 8|0);
      $93 = HEAP32[$92>>2]|0;
      $94 = $pres;
      $95 = HEAP32[$94>>2]|0;
      $96 = (($93) - ($95))|0;
      $97 = $x;
      $98 = (($97) + ($96))|0;
      $x = $98;
     }
     $99 = $band;
     $100 = (($99) + 16|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = $101 & 2;
     $103 = ($102|0)!=(0);
     if ($103) {
      $104 = $resno;
      $105 = (($104) - 1)|0;
      $106 = $1;
      $107 = (($106) + 20|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = (($108) + (($105*124)|0)|0);
      $pres1 = $109;
      $110 = $pres1;
      $111 = (($110) + 12|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = $pres1;
      $114 = (($113) + 4|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = (($112) - ($115))|0;
      $117 = $y;
      $118 = (($117) + ($116))|0;
      $y = $118;
     }
     $119 = $0;
     $120 = (($119) + 12|0);
     $121 = HEAP32[$120>>2]|0;
     $datap = $121;
     $122 = $0;
     $123 = (($122) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $cblk_w = $124;
     $125 = $0;
     $126 = (($125) + 24|0);
     $127 = HEAP32[$126>>2]|0;
     $cblk_h = $127;
     $128 = $2;
     $129 = (($128) + 808|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = ($130|0)!=(0);
     if ($131) {
      $132 = $2;
      $133 = (($132) + 808|0);
      $134 = HEAP32[$133>>2]|0;
      $135 = 1 << $134;
      $thresh = $135;
      $j = 0;
      while(1) {
       $136 = $j;
       $137 = $cblk_h;
       $138 = ($136|0)<($137|0);
       if (!($138)) {
        break;
       }
       $i = 0;
       while(1) {
        $139 = $i;
        $140 = $cblk_w;
        $141 = ($139|0)<($140|0);
        if (!($141)) {
         break;
        }
        $142 = $j;
        $143 = $cblk_w;
        $144 = Math_imul($142, $143)|0;
        $145 = $i;
        $146 = (($144) + ($145))|0;
        $147 = $datap;
        $148 = (($147) + ($146<<2)|0);
        $149 = HEAP32[$148>>2]|0;
        $val = $149;
        $150 = $val;
        $151 = (Math_abs(($150|0))|0);
        $mag = $151;
        $152 = $mag;
        $153 = $thresh;
        $154 = ($152|0)>=($153|0);
        if ($154) {
         $155 = $2;
         $156 = (($155) + 808|0);
         $157 = HEAP32[$156>>2]|0;
         $158 = $mag;
         $159 = $158 >> $157;
         $mag = $159;
         $160 = $val;
         $161 = ($160|0)<(0);
         if ($161) {
          $162 = $mag;
          $163 = (0 - ($162))|0;
          $172 = $163;
         } else {
          $164 = $mag;
          $172 = $164;
         }
         $165 = $j;
         $166 = $cblk_w;
         $167 = Math_imul($165, $166)|0;
         $168 = $i;
         $169 = (($167) + ($168))|0;
         $170 = $datap;
         $171 = (($170) + ($169<<2)|0);
         HEAP32[$171>>2] = $172;
        }
        $173 = $i;
        $174 = (($173) + 1)|0;
        $i = $174;
       }
       $175 = $j;
       $176 = (($175) + 1)|0;
       $j = $176;
      }
     }
     $177 = $2;
     $178 = (($177) + 20|0);
     $179 = HEAP32[$178>>2]|0;
     $180 = ($179|0)==(1);
     if ($180) {
      $181 = $y;
      $182 = $tile_w;
      $183 = Math_imul($181, $182)|0;
      $184 = $x;
      $185 = (($183) + ($184))|0;
      $186 = $1;
      $187 = (($186) + 24|0);
      $188 = HEAP32[$187>>2]|0;
      $189 = (($188) + ($185<<2)|0);
      $tiledp = $189;
      $j = 0;
      while(1) {
       $190 = $j;
       $191 = $cblk_h;
       $192 = ($190|0)<($191|0);
       if (!($192)) {
        break;
       }
       $i = 0;
       while(1) {
        $193 = $i;
        $194 = $cblk_w;
        $195 = ($193|0)<($194|0);
        if (!($195)) {
         break;
        }
        $196 = $j;
        $197 = $cblk_w;
        $198 = Math_imul($196, $197)|0;
        $199 = $i;
        $200 = (($198) + ($199))|0;
        $201 = $datap;
        $202 = (($201) + ($200<<2)|0);
        $203 = HEAP32[$202>>2]|0;
        $tmp = $203;
        $204 = $tmp;
        $205 = (($204|0) / 2)&-1;
        $206 = $j;
        $207 = $tile_w;
        $208 = Math_imul($206, $207)|0;
        $209 = $i;
        $210 = (($208) + ($209))|0;
        $211 = $tiledp;
        $212 = (($211) + ($210<<2)|0);
        HEAP32[$212>>2] = $205;
        $213 = $i;
        $214 = (($213) + 1)|0;
        $i = $214;
       }
       $215 = $j;
       $216 = (($215) + 1)|0;
       $j = $216;
      }
     } else {
      $217 = $y;
      $218 = $tile_w;
      $219 = Math_imul($217, $218)|0;
      $220 = $x;
      $221 = (($219) + ($220))|0;
      $222 = $1;
      $223 = (($222) + 24|0);
      $224 = HEAP32[$223>>2]|0;
      $225 = (($224) + ($221<<2)|0);
      $tiledp2 = $225;
      $j = 0;
      while(1) {
       $226 = $j;
       $227 = $cblk_h;
       $228 = ($226|0)<($227|0);
       if (!($228)) {
        break;
       }
       $229 = $tiledp2;
       $tiledp23 = $229;
       $i = 0;
       while(1) {
        $230 = $i;
        $231 = $cblk_w;
        $232 = ($230|0)<($231|0);
        if (!($232)) {
         break;
        }
        $233 = $datap;
        $234 = HEAP32[$233>>2]|0;
        $235 = (+($234|0));
        $236 = $band;
        $237 = (($236) + 28|0);
        $238 = +HEAPF32[$237>>2];
        $239 = $235 * $238;
        $tmp4 = $239;
        $240 = $tmp4;
        $241 = $tiledp23;
        HEAPF32[$241>>2] = $240;
        $242 = $datap;
        $243 = (($242) + 4|0);
        $datap = $243;
        $244 = $tiledp23;
        $245 = (($244) + 4|0);
        $tiledp23 = $245;
        $246 = $i;
        $247 = (($246) + 1)|0;
        $i = $247;
       }
       $248 = $tile_w;
       $249 = $tiledp2;
       $250 = (($249) + ($248<<2)|0);
       $tiledp2 = $250;
       $251 = $j;
       $252 = (($251) + 1)|0;
       $j = $252;
      }
     }
     $253 = $cblk;
     $254 = HEAP32[$253>>2]|0;
     _free($254);
     $255 = $cblk;
     $256 = (($255) + 4|0);
     $257 = HEAP32[$256>>2]|0;
     _free($257);
     $258 = $cblkno;
     $259 = (($258) + 1)|0;
     $cblkno = $259;
    }
    $260 = $precinct;
    $261 = (($260) + 24|0);
    $262 = HEAP32[$261>>2]|0;
    _free($262);
    $263 = $precno;
    $264 = (($263) + 1)|0;
    $precno = $264;
   }
   $265 = $bandno;
   $266 = (($265) + 1)|0;
   $bandno = $266;
  }
  $267 = $resno;
  $268 = (($267) + 1)|0;
  $resno = $268;
 }
 STACKTOP = sp;return;
}
function _t1_decode_cblk($t1,$cblk,$orient,$roishift,$cblksty) {
 $t1 = $t1|0;
 $cblk = $cblk|0;
 $orient = $orient|0;
 $roishift = $roishift|0;
 $cblksty = $cblksty|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bpno = 0, $mqc = 0, $passno = 0, $passtype = 0;
 var $raw = 0, $seg = 0, $segno = 0, $type = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $t1;
 $1 = $cblk;
 $2 = $orient;
 $3 = $roishift;
 $4 = $cblksty;
 $5 = $0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $raw = $7;
 $8 = $0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $mqc = $10;
 $type = 0;
 $11 = $0;
 $12 = $1;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $1;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($14) - ($17))|0;
 $19 = $1;
 $20 = (($19) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $1;
 $23 = (($22) + 12|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($21) - ($24))|0;
 $26 = (_allocate_buffers($11,$18,$25)|0);
 $27 = ($26|0)!=(0);
 if (!($27)) {
  STACKTOP = sp;return;
 }
 $28 = $3;
 $29 = $1;
 $30 = (($29) + 24|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($28) + ($31))|0;
 $33 = (($32) - 1)|0;
 $bpno = $33;
 $passtype = 2;
 $34 = $mqc;
 _mqc_resetstates($34);
 $35 = $mqc;
 _mqc_setstate($35,18,0,46);
 $36 = $mqc;
 _mqc_setstate($36,17,0,3);
 $37 = $mqc;
 _mqc_setstate($37,0,0,4);
 $segno = 0;
 while(1) {
  $38 = $segno;
  $39 = $1;
  $40 = (($39) + 40|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ($38|0)<($41|0);
  if (!($42)) {
   break;
  }
  $43 = $segno;
  $44 = $1;
  $45 = (($44) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + (($43*28)|0)|0);
  $seg = $47;
  $48 = $bpno;
  $49 = $1;
  $50 = (($49) + 24|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) - 1)|0;
  $53 = (($52) - 4)|0;
  $54 = ($48|0)<=($53|0);
  if ($54) {
   $55 = $passtype;
   $56 = ($55|0)<(2);
   if ($56) {
    $57 = $4;
    $58 = $57 & 1;
    $59 = ($58|0)!=(0);
    $61 = $59;
   } else {
    $61 = 0;
   }
  } else {
   $61 = 0;
  }
  $60 = $61 ? 1 : 0;
  $62 = $60&255;
  $type = $62;
  $63 = $seg;
  $64 = HEAP32[$63>>2]|0;
  $65 = ($64|0)==(0|0);
  if ($65) {
  } else {
   $66 = $type;
   $67 = $66 << 24 >> 24;
   $68 = ($67|0)==(1);
   if ($68) {
    $69 = $raw;
    $70 = $seg;
    $71 = HEAP32[$70>>2]|0;
    $72 = HEAP32[$71>>2]|0;
    $73 = $seg;
    $74 = (($73) + 4|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($72) + ($75)|0);
    $77 = $seg;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    _raw_init_dec($69,$76,$79);
   } else {
    $80 = $mqc;
    $81 = $seg;
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$82>>2]|0;
    $84 = $seg;
    $85 = (($84) + 4|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = (($83) + ($86)|0);
    $88 = $seg;
    $89 = (($88) + 12|0);
    $90 = HEAP32[$89>>2]|0;
    _mqc_init_dec($80,$87,$90);
   }
   $passno = 0;
   while(1) {
    $91 = $passno;
    $92 = $seg;
    $93 = (($92) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($91|0)<($94|0);
    if (!($95)) {
     break;
    }
    $96 = $passtype;
    if ((($96|0) == 0)) {
     $97 = $type;
     $98 = $97 << 24 >> 24;
     $99 = ($98|0)==(1);
     if ($99) {
      $100 = $0;
      $101 = $bpno;
      $102 = (($101) + 1)|0;
      $103 = $2;
      $104 = $4;
      _t1_dec_sigpass_raw($100,$102,$103,$104);
     } else {
      $105 = $4;
      $106 = $105 & 8;
      $107 = ($106|0)!=(0);
      if ($107) {
       $108 = $0;
       $109 = $bpno;
       $110 = (($109) + 1)|0;
       $111 = $2;
       _t1_dec_sigpass_mqc_vsc($108,$110,$111);
      } else {
       $112 = $0;
       $113 = $bpno;
       $114 = (($113) + 1)|0;
       $115 = $2;
       _t1_dec_sigpass_mqc($112,$114,$115);
      }
     }
    } else if ((($96|0) == 1)) {
     $116 = $type;
     $117 = $116 << 24 >> 24;
     $118 = ($117|0)==(1);
     if ($118) {
      $119 = $0;
      $120 = $bpno;
      $121 = (($120) + 1)|0;
      $122 = $4;
      _t1_dec_refpass_raw($119,$121,$122);
     } else {
      $123 = $4;
      $124 = $123 & 8;
      $125 = ($124|0)!=(0);
      if ($125) {
       $126 = $0;
       $127 = $bpno;
       $128 = (($127) + 1)|0;
       _t1_dec_refpass_mqc_vsc($126,$128);
      } else {
       $129 = $0;
       $130 = $bpno;
       $131 = (($130) + 1)|0;
       _t1_dec_refpass_mqc($129,$131);
      }
     }
    } else if ((($96|0) == 2)) {
     $132 = $0;
     $133 = $bpno;
     $134 = (($133) + 1)|0;
     $135 = $2;
     $136 = $4;
     _t1_dec_clnpass($132,$134,$135,$136);
    }
    $137 = $4;
    $138 = $137 & 2;
    $139 = ($138|0)!=(0);
    if ($139) {
     $140 = $type;
     $141 = $140 << 24 >> 24;
     $142 = ($141|0)==(0);
     if ($142) {
      $143 = $mqc;
      _mqc_resetstates($143);
      $144 = $mqc;
      _mqc_setstate($144,18,0,46);
      $145 = $mqc;
      _mqc_setstate($145,17,0,3);
      $146 = $mqc;
      _mqc_setstate($146,0,0,4);
     }
    }
    $147 = $passtype;
    $148 = (($147) + 1)|0;
    $passtype = $148;
    $149 = ($148|0)==(3);
    if ($149) {
     $passtype = 0;
     $150 = $bpno;
     $151 = (($150) + -1)|0;
     $bpno = $151;
    }
    $152 = $passno;
    $153 = (($152) + 1)|0;
    $passno = $153;
   }
  }
  $154 = $segno;
  $155 = (($154) + 1)|0;
  $segno = $155;
 }
 STACKTOP = sp;return;
}
function _t1_dec_sigpass_raw($t1,$bpno,$orient,$cblksty) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 $orient = $orient|0;
 $cblksty = $cblksty|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $half = 0, $i = 0, $j = 0, $k = 0, $one = 0, $oneplushalf = 0, $vsc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $orient;
 $3 = $cblksty;
 $4 = $1;
 $5 = 1 << $4;
 $one = $5;
 $6 = $one;
 $7 = $6 >> 1;
 $half = $7;
 $8 = $one;
 $9 = $half;
 $10 = $8 | $9;
 $oneplushalf = $10;
 $k = 0;
 while(1) {
  $11 = $k;
  $12 = $0;
  $13 = (($12) + 24|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   break;
  }
  $i = 0;
  while(1) {
   $16 = $i;
   $17 = $0;
   $18 = (($17) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = $k;
   $j = $21;
   while(1) {
    $22 = $j;
    $23 = $k;
    $24 = (($23) + 4)|0;
    $25 = ($22|0)<($24|0);
    if ($25) {
     $26 = $j;
     $27 = $0;
     $28 = (($27) + 24|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = ($26|0)<($29|0);
     $80 = $30;
    } else {
     $80 = 0;
    }
    if (!($80)) {
     break;
    }
    $31 = $3;
    $32 = $31 & 8;
    $33 = ($32|0)!=(0);
    if ($33) {
     $34 = $j;
     $35 = $k;
     $36 = (($35) + 3)|0;
     $37 = ($34|0)==($36|0);
     if ($37) {
      $81 = 1;
     } else {
      $38 = $j;
      $39 = $0;
      $40 = (($39) + 24|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = (($41) - 1)|0;
      $43 = ($38|0)==($42|0);
      $81 = $43;
     }
     $45 = $81;
    } else {
     $45 = 0;
    }
    $44 = $45 ? 1 : 0;
    $vsc = $44;
    $46 = $0;
    $47 = $j;
    $48 = (($47) + 1)|0;
    $49 = $0;
    $50 = (($49) + 36|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = Math_imul($48, $51)|0;
    $53 = $i;
    $54 = (($52) + ($53))|0;
    $55 = (($54) + 1)|0;
    $56 = $0;
    $57 = (($56) + 16|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = (($58) + ($55<<1)|0);
    $60 = $j;
    $61 = $0;
    $62 = (($61) + 20|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = Math_imul($60, $63)|0;
    $65 = $i;
    $66 = (($64) + ($65))|0;
    $67 = $0;
    $68 = (($67) + 12|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + ($66<<2)|0);
    $71 = $2;
    $72 = $oneplushalf;
    $73 = $vsc;
    _t1_dec_sigpass_step_raw($46,$59,$70,$71,$72,$73);
    $74 = $j;
    $75 = (($74) + 1)|0;
    $j = $75;
   }
   $76 = $i;
   $77 = (($76) + 1)|0;
   $i = $77;
  }
  $78 = $k;
  $79 = (($78) + 4)|0;
  $k = $79;
 }
 STACKTOP = sp;return;
}
function _t1_dec_sigpass_mqc_vsc($t1,$bpno,$orient) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 $orient = $orient|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $half = 0, $i = 0, $j = 0;
 var $k = 0, $one = 0, $oneplushalf = 0, $vsc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $orient;
 $3 = $1;
 $4 = 1 << $3;
 $one = $4;
 $5 = $one;
 $6 = $5 >> 1;
 $half = $6;
 $7 = $one;
 $8 = $half;
 $9 = $7 | $8;
 $oneplushalf = $9;
 $k = 0;
 while(1) {
  $10 = $k;
  $11 = $0;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $i = 0;
  while(1) {
   $15 = $i;
   $16 = $0;
   $17 = (($16) + 20|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($15|0)<($18|0);
   if (!($19)) {
    break;
   }
   $20 = $k;
   $j = $20;
   while(1) {
    $21 = $j;
    $22 = $k;
    $23 = (($22) + 4)|0;
    $24 = ($21|0)<($23|0);
    if ($24) {
     $25 = $j;
     $26 = $0;
     $27 = (($26) + 24|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = ($25|0)<($28|0);
     $76 = $29;
    } else {
     $76 = 0;
    }
    if (!($76)) {
     break;
    }
    $30 = $j;
    $31 = $k;
    $32 = (($31) + 3)|0;
    $33 = ($30|0)==($32|0);
    if ($33) {
     $41 = 1;
    } else {
     $34 = $j;
     $35 = $0;
     $36 = (($35) + 24|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = (($37) - 1)|0;
     $39 = ($34|0)==($38|0);
     $41 = $39;
    }
    $40 = $41 ? 1 : 0;
    $vsc = $40;
    $42 = $0;
    $43 = $j;
    $44 = (($43) + 1)|0;
    $45 = $0;
    $46 = (($45) + 36|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = Math_imul($44, $47)|0;
    $49 = $i;
    $50 = (($48) + ($49))|0;
    $51 = (($50) + 1)|0;
    $52 = $0;
    $53 = (($52) + 16|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + ($51<<1)|0);
    $56 = $j;
    $57 = $0;
    $58 = (($57) + 20|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = Math_imul($56, $59)|0;
    $61 = $i;
    $62 = (($60) + ($61))|0;
    $63 = $0;
    $64 = (($63) + 12|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = (($65) + ($62<<2)|0);
    $67 = $2;
    $68 = $oneplushalf;
    $69 = $vsc;
    _t1_dec_sigpass_step_mqc_vsc($42,$55,$66,$67,$68,$69);
    $70 = $j;
    $71 = (($70) + 1)|0;
    $j = $71;
   }
   $72 = $i;
   $73 = (($72) + 1)|0;
   $i = $73;
  }
  $74 = $k;
  $75 = (($74) + 4)|0;
  $k = $75;
 }
 STACKTOP = sp;return;
}
function _t1_dec_sigpass_mqc($t1,$bpno,$orient) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 $orient = $orient|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data1 = 0, $data2 = 0, $data21 = 0, $flags1 = 0, $flags2 = 0, $flags22 = 0, $half = 0, $i = 0, $j = 0, $k = 0, $one = 0, $oneplushalf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $orient;
 $3 = $0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $data1 = $5;
 $6 = $0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 2|0);
 $flags1 = $9;
 $10 = $1;
 $11 = 1 << $10;
 $one = $11;
 $12 = $one;
 $13 = $12 >> 1;
 $half = $13;
 $14 = $one;
 $15 = $half;
 $16 = $14 | $15;
 $oneplushalf = $16;
 $k = 0;
 while(1) {
  $17 = $k;
  $18 = $0;
  $19 = (($18) + 24|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $20 & -4;
  $22 = ($17|0)<($21|0);
  if (!($22)) {
   break;
  }
  $i = 0;
  while(1) {
   $23 = $i;
   $24 = $0;
   $25 = (($24) + 20|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($23|0)<($26|0);
   if (!($27)) {
    break;
   }
   $28 = $data1;
   $29 = $i;
   $30 = (($28) + ($29<<2)|0);
   $data2 = $30;
   $31 = $flags1;
   $32 = $i;
   $33 = (($31) + ($32<<1)|0);
   $flags2 = $33;
   $34 = $0;
   $35 = (($34) + 36|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = $flags2;
   $38 = (($37) + ($36<<1)|0);
   $flags2 = $38;
   $39 = $0;
   $40 = $flags2;
   $41 = $data2;
   $42 = $2;
   $43 = $oneplushalf;
   _t1_dec_sigpass_step_mqc($39,$40,$41,$42,$43);
   $44 = $0;
   $45 = (($44) + 20|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = $data2;
   $48 = (($47) + ($46<<2)|0);
   $data2 = $48;
   $49 = $0;
   $50 = (($49) + 36|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = $flags2;
   $53 = (($52) + ($51<<1)|0);
   $flags2 = $53;
   $54 = $0;
   $55 = $flags2;
   $56 = $data2;
   $57 = $2;
   $58 = $oneplushalf;
   _t1_dec_sigpass_step_mqc($54,$55,$56,$57,$58);
   $59 = $0;
   $60 = (($59) + 20|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = $data2;
   $63 = (($62) + ($61<<2)|0);
   $data2 = $63;
   $64 = $0;
   $65 = (($64) + 36|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = $flags2;
   $68 = (($67) + ($66<<1)|0);
   $flags2 = $68;
   $69 = $0;
   $70 = $flags2;
   $71 = $data2;
   $72 = $2;
   $73 = $oneplushalf;
   _t1_dec_sigpass_step_mqc($69,$70,$71,$72,$73);
   $74 = $0;
   $75 = (($74) + 20|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = $data2;
   $78 = (($77) + ($76<<2)|0);
   $data2 = $78;
   $79 = $0;
   $80 = (($79) + 36|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = $flags2;
   $83 = (($82) + ($81<<1)|0);
   $flags2 = $83;
   $84 = $0;
   $85 = $flags2;
   $86 = $data2;
   $87 = $2;
   $88 = $oneplushalf;
   _t1_dec_sigpass_step_mqc($84,$85,$86,$87,$88);
   $89 = $0;
   $90 = (($89) + 20|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = $data2;
   $93 = (($92) + ($91<<2)|0);
   $data2 = $93;
   $94 = $i;
   $95 = (($94) + 1)|0;
   $i = $95;
  }
  $96 = $0;
  $97 = (($96) + 20|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = $98 << 2;
  $100 = $data1;
  $101 = (($100) + ($99<<2)|0);
  $data1 = $101;
  $102 = $0;
  $103 = (($102) + 36|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = $104 << 2;
  $106 = $flags1;
  $107 = (($106) + ($105<<1)|0);
  $flags1 = $107;
  $108 = $k;
  $109 = (($108) + 4)|0;
  $k = $109;
 }
 $i = 0;
 while(1) {
  $110 = $i;
  $111 = $0;
  $112 = (($111) + 20|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($110|0)<($113|0);
  if (!($114)) {
   break;
  }
  $115 = $data1;
  $116 = $i;
  $117 = (($115) + ($116<<2)|0);
  $data21 = $117;
  $118 = $flags1;
  $119 = $i;
  $120 = (($118) + ($119<<1)|0);
  $flags22 = $120;
  $121 = $k;
  $j = $121;
  while(1) {
   $122 = $j;
   $123 = $0;
   $124 = (($123) + 24|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = ($122|0)<($125|0);
   if (!($126)) {
    break;
   }
   $127 = $0;
   $128 = (($127) + 36|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = $flags22;
   $131 = (($130) + ($129<<1)|0);
   $flags22 = $131;
   $132 = $0;
   $133 = $flags22;
   $134 = $data21;
   $135 = $2;
   $136 = $oneplushalf;
   _t1_dec_sigpass_step_mqc($132,$133,$134,$135,$136);
   $137 = $0;
   $138 = (($137) + 20|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $data21;
   $141 = (($140) + ($139<<2)|0);
   $data21 = $141;
   $142 = $j;
   $143 = (($142) + 1)|0;
   $j = $143;
  }
  $144 = $i;
  $145 = (($144) + 1)|0;
  $i = $145;
 }
 STACKTOP = sp;return;
}
function _t1_dec_refpass_raw($t1,$bpno,$cblksty) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 $cblksty = $cblksty|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $i = 0, $j = 0, $k = 0, $neghalf = 0, $one = 0, $poshalf = 0, $vsc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $cblksty;
 $3 = $1;
 $4 = 1 << $3;
 $one = $4;
 $5 = $one;
 $6 = $5 >> 1;
 $poshalf = $6;
 $7 = $1;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = $poshalf;
  $10 = (0 - ($9))|0;
  $11 = $10;
 } else {
  $11 = -1;
 }
 $neghalf = $11;
 $k = 0;
 while(1) {
  $12 = $k;
  $13 = $0;
  $14 = (($13) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   break;
  }
  $i = 0;
  while(1) {
   $17 = $i;
   $18 = $0;
   $19 = (($18) + 20|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($17|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = $k;
   $j = $22;
   while(1) {
    $23 = $j;
    $24 = $k;
    $25 = (($24) + 4)|0;
    $26 = ($23|0)<($25|0);
    if ($26) {
     $27 = $j;
     $28 = $0;
     $29 = (($28) + 24|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($27|0)<($30|0);
     $81 = $31;
    } else {
     $81 = 0;
    }
    if (!($81)) {
     break;
    }
    $32 = $2;
    $33 = $32 & 8;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = $j;
     $36 = $k;
     $37 = (($36) + 3)|0;
     $38 = ($35|0)==($37|0);
     if ($38) {
      $82 = 1;
     } else {
      $39 = $j;
      $40 = $0;
      $41 = (($40) + 24|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = (($42) - 1)|0;
      $44 = ($39|0)==($43|0);
      $82 = $44;
     }
     $46 = $82;
    } else {
     $46 = 0;
    }
    $45 = $46 ? 1 : 0;
    $vsc = $45;
    $47 = $0;
    $48 = $j;
    $49 = (($48) + 1)|0;
    $50 = $0;
    $51 = (($50) + 36|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = Math_imul($49, $52)|0;
    $54 = $i;
    $55 = (($53) + ($54))|0;
    $56 = (($55) + 1)|0;
    $57 = $0;
    $58 = (($57) + 16|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = (($59) + ($56<<1)|0);
    $61 = $j;
    $62 = $0;
    $63 = (($62) + 20|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = Math_imul($61, $64)|0;
    $66 = $i;
    $67 = (($65) + ($66))|0;
    $68 = $0;
    $69 = (($68) + 12|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (($70) + ($67<<2)|0);
    $72 = $poshalf;
    $73 = $neghalf;
    $74 = $vsc;
    _t1_dec_refpass_step_raw($47,$60,$71,$72,$73,$74);
    $75 = $j;
    $76 = (($75) + 1)|0;
    $j = $76;
   }
   $77 = $i;
   $78 = (($77) + 1)|0;
   $i = $78;
  }
  $79 = $k;
  $80 = (($79) + 4)|0;
  $k = $80;
 }
 STACKTOP = sp;return;
}
function _t1_dec_refpass_mqc_vsc($t1,$bpno) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0;
 var $k = 0, $neghalf = 0, $one = 0, $poshalf = 0, $vsc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $1;
 $3 = 1 << $2;
 $one = $3;
 $4 = $one;
 $5 = $4 >> 1;
 $poshalf = $5;
 $6 = $1;
 $7 = ($6|0)>(0);
 if ($7) {
  $8 = $poshalf;
  $9 = (0 - ($8))|0;
  $10 = $9;
 } else {
  $10 = -1;
 }
 $neghalf = $10;
 $k = 0;
 while(1) {
  $11 = $k;
  $12 = $0;
  $13 = (($12) + 24|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   break;
  }
  $i = 0;
  while(1) {
   $16 = $i;
   $17 = $0;
   $18 = (($17) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = $k;
   $j = $21;
   while(1) {
    $22 = $j;
    $23 = $k;
    $24 = (($23) + 4)|0;
    $25 = ($22|0)<($24|0);
    if ($25) {
     $26 = $j;
     $27 = $0;
     $28 = (($27) + 24|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = ($26|0)<($29|0);
     $77 = $30;
    } else {
     $77 = 0;
    }
    if (!($77)) {
     break;
    }
    $31 = $j;
    $32 = $k;
    $33 = (($32) + 3)|0;
    $34 = ($31|0)==($33|0);
    if ($34) {
     $42 = 1;
    } else {
     $35 = $j;
     $36 = $0;
     $37 = (($36) + 24|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = (($38) - 1)|0;
     $40 = ($35|0)==($39|0);
     $42 = $40;
    }
    $41 = $42 ? 1 : 0;
    $vsc = $41;
    $43 = $0;
    $44 = $j;
    $45 = (($44) + 1)|0;
    $46 = $0;
    $47 = (($46) + 36|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = Math_imul($45, $48)|0;
    $50 = $i;
    $51 = (($49) + ($50))|0;
    $52 = (($51) + 1)|0;
    $53 = $0;
    $54 = (($53) + 16|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($52<<1)|0);
    $57 = $j;
    $58 = $0;
    $59 = (($58) + 20|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = Math_imul($57, $60)|0;
    $62 = $i;
    $63 = (($61) + ($62))|0;
    $64 = $0;
    $65 = (($64) + 12|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = (($66) + ($63<<2)|0);
    $68 = $poshalf;
    $69 = $neghalf;
    $70 = $vsc;
    _t1_dec_refpass_step_mqc_vsc($43,$56,$67,$68,$69,$70);
    $71 = $j;
    $72 = (($71) + 1)|0;
    $j = $72;
   }
   $73 = $i;
   $74 = (($73) + 1)|0;
   $i = $74;
  }
  $75 = $k;
  $76 = (($75) + 4)|0;
  $k = $76;
 }
 STACKTOP = sp;return;
}
function _t1_dec_refpass_mqc($t1,$bpno) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data1 = 0, $data2 = 0, $data21 = 0, $flags1 = 0, $flags2 = 0, $flags22 = 0, $i = 0, $j = 0, $k = 0, $neghalf = 0, $one = 0, $poshalf = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $data1 = $4;
 $5 = $0;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 2|0);
 $flags1 = $8;
 $9 = $1;
 $10 = 1 << $9;
 $one = $10;
 $11 = $one;
 $12 = $11 >> 1;
 $poshalf = $12;
 $13 = $1;
 $14 = ($13|0)>(0);
 if ($14) {
  $15 = $poshalf;
  $16 = (0 - ($15))|0;
  $17 = $16;
 } else {
  $17 = -1;
 }
 $neghalf = $17;
 $k = 0;
 while(1) {
  $18 = $k;
  $19 = $0;
  $20 = (($19) + 24|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 & -4;
  $23 = ($18|0)<($22|0);
  if (!($23)) {
   break;
  }
  $i = 0;
  while(1) {
   $24 = $i;
   $25 = $0;
   $26 = (($25) + 20|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($24|0)<($27|0);
   if (!($28)) {
    break;
   }
   $29 = $data1;
   $30 = $i;
   $31 = (($29) + ($30<<2)|0);
   $data2 = $31;
   $32 = $flags1;
   $33 = $i;
   $34 = (($32) + ($33<<1)|0);
   $flags2 = $34;
   $35 = $0;
   $36 = (($35) + 36|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = $flags2;
   $39 = (($38) + ($37<<1)|0);
   $flags2 = $39;
   $40 = $0;
   $41 = $flags2;
   $42 = $data2;
   $43 = $poshalf;
   $44 = $neghalf;
   _t1_dec_refpass_step_mqc($40,$41,$42,$43,$44);
   $45 = $0;
   $46 = (($45) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $data2;
   $49 = (($48) + ($47<<2)|0);
   $data2 = $49;
   $50 = $0;
   $51 = (($50) + 36|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = $flags2;
   $54 = (($53) + ($52<<1)|0);
   $flags2 = $54;
   $55 = $0;
   $56 = $flags2;
   $57 = $data2;
   $58 = $poshalf;
   $59 = $neghalf;
   _t1_dec_refpass_step_mqc($55,$56,$57,$58,$59);
   $60 = $0;
   $61 = (($60) + 20|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = $data2;
   $64 = (($63) + ($62<<2)|0);
   $data2 = $64;
   $65 = $0;
   $66 = (($65) + 36|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = $flags2;
   $69 = (($68) + ($67<<1)|0);
   $flags2 = $69;
   $70 = $0;
   $71 = $flags2;
   $72 = $data2;
   $73 = $poshalf;
   $74 = $neghalf;
   _t1_dec_refpass_step_mqc($70,$71,$72,$73,$74);
   $75 = $0;
   $76 = (($75) + 20|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = $data2;
   $79 = (($78) + ($77<<2)|0);
   $data2 = $79;
   $80 = $0;
   $81 = (($80) + 36|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = $flags2;
   $84 = (($83) + ($82<<1)|0);
   $flags2 = $84;
   $85 = $0;
   $86 = $flags2;
   $87 = $data2;
   $88 = $poshalf;
   $89 = $neghalf;
   _t1_dec_refpass_step_mqc($85,$86,$87,$88,$89);
   $90 = $0;
   $91 = (($90) + 20|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = $data2;
   $94 = (($93) + ($92<<2)|0);
   $data2 = $94;
   $95 = $i;
   $96 = (($95) + 1)|0;
   $i = $96;
  }
  $97 = $0;
  $98 = (($97) + 20|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = $99 << 2;
  $101 = $data1;
  $102 = (($101) + ($100<<2)|0);
  $data1 = $102;
  $103 = $0;
  $104 = (($103) + 36|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = $105 << 2;
  $107 = $flags1;
  $108 = (($107) + ($106<<1)|0);
  $flags1 = $108;
  $109 = $k;
  $110 = (($109) + 4)|0;
  $k = $110;
 }
 $i = 0;
 while(1) {
  $111 = $i;
  $112 = $0;
  $113 = (($112) + 20|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = ($111|0)<($114|0);
  if (!($115)) {
   break;
  }
  $116 = $data1;
  $117 = $i;
  $118 = (($116) + ($117<<2)|0);
  $data21 = $118;
  $119 = $flags1;
  $120 = $i;
  $121 = (($119) + ($120<<1)|0);
  $flags22 = $121;
  $122 = $k;
  $j = $122;
  while(1) {
   $123 = $j;
   $124 = $0;
   $125 = (($124) + 24|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = ($123|0)<($126|0);
   if (!($127)) {
    break;
   }
   $128 = $0;
   $129 = (($128) + 36|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = $flags22;
   $132 = (($131) + ($130<<1)|0);
   $flags22 = $132;
   $133 = $0;
   $134 = $flags22;
   $135 = $data21;
   $136 = $poshalf;
   $137 = $neghalf;
   _t1_dec_refpass_step_mqc($133,$134,$135,$136,$137);
   $138 = $0;
   $139 = (($138) + 20|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = $data21;
   $142 = (($141) + ($140<<2)|0);
   $data21 = $142;
   $143 = $j;
   $144 = (($143) + 1)|0;
   $j = $144;
  }
  $145 = $i;
  $146 = (($145) + 1)|0;
  $i = $146;
 }
 STACKTOP = sp;return;
}
function _t1_dec_clnpass($t1,$bpno,$orient,$cblksty) {
 $t1 = $t1|0;
 $bpno = $bpno|0;
 $orient = $orient|0;
 $cblksty = $cblksty|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $agg = 0, $data1 = 0, $data2 = 0, $data21 = 0, $flags1 = 0, $flags2 = 0, $flags22 = 0, $half = 0;
 var $i = 0, $j = 0, $k = 0, $mqc = 0, $one = 0, $oneplushalf = 0, $runlen = 0, $segsym = 0, $v = 0, $vsc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $t1;
 $1 = $bpno;
 $2 = $orient;
 $3 = $cblksty;
 $4 = $3;
 $5 = $4 & 32;
 $segsym = $5;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mqc = $8;
 $9 = $1;
 $10 = 1 << $9;
 $one = $10;
 $11 = $one;
 $12 = $11 >> 1;
 $half = $12;
 $13 = $one;
 $14 = $half;
 $15 = $13 | $14;
 $oneplushalf = $15;
 $16 = $3;
 $17 = $16 & 8;
 $18 = ($17|0)!=(0);
 if ($18) {
  $k = 0;
  while(1) {
   $19 = $k;
   $20 = $0;
   $21 = (($20) + 24|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)<($22|0);
   if (!($23)) {
    break;
   }
   $i = 0;
   while(1) {
    $24 = $i;
    $25 = $0;
    $26 = (($25) + 20|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($24|0)<($27|0);
    if (!($28)) {
     break;
    }
    $29 = $k;
    $30 = (($29) + 3)|0;
    $31 = $0;
    $32 = (($31) + 24|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30|0)<($33|0);
    if ($34) {
     $35 = $k;
     $36 = (1 + ($35))|0;
     $37 = $0;
     $38 = (($37) + 36|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = Math_imul($36, $39)|0;
     $41 = $i;
     $42 = (1 + ($41))|0;
     $43 = (($40) + ($42))|0;
     $44 = $0;
     $45 = (($44) + 16|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = (($46) + ($43<<1)|0);
     $48 = HEAP16[$47>>1]|0;
     $49 = $48 << 16 >> 16;
     $50 = $49 & 20735;
     $51 = ($50|0)!=(0);
     if ($51) {
      $108 = 1;
     } else {
      $52 = $k;
      $53 = (1 + ($52))|0;
      $54 = (($53) + 1)|0;
      $55 = $0;
      $56 = (($55) + 36|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = Math_imul($54, $57)|0;
      $59 = $i;
      $60 = (1 + ($59))|0;
      $61 = (($58) + ($60))|0;
      $62 = $0;
      $63 = (($62) + 16|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (($64) + ($61<<1)|0);
      $66 = HEAP16[$65>>1]|0;
      $67 = $66 << 16 >> 16;
      $68 = $67 & 20735;
      $69 = ($68|0)!=(0);
      if ($69) {
       $108 = 1;
      } else {
       $70 = $k;
       $71 = (1 + ($70))|0;
       $72 = (($71) + 2)|0;
       $73 = $0;
       $74 = (($73) + 36|0);
       $75 = HEAP32[$74>>2]|0;
       $76 = Math_imul($72, $75)|0;
       $77 = $i;
       $78 = (1 + ($77))|0;
       $79 = (($76) + ($78))|0;
       $80 = $0;
       $81 = (($80) + 16|0);
       $82 = HEAP32[$81>>2]|0;
       $83 = (($82) + ($79<<1)|0);
       $84 = HEAP16[$83>>1]|0;
       $85 = $84 << 16 >> 16;
       $86 = $85 & 20735;
       $87 = ($86|0)!=(0);
       if ($87) {
        $108 = 1;
       } else {
        $88 = $k;
        $89 = (1 + ($88))|0;
        $90 = (($89) + 3)|0;
        $91 = $0;
        $92 = (($91) + 36|0);
        $93 = HEAP32[$92>>2]|0;
        $94 = Math_imul($90, $93)|0;
        $95 = $i;
        $96 = (1 + ($95))|0;
        $97 = (($94) + ($96))|0;
        $98 = $0;
        $99 = (($98) + 16|0);
        $100 = HEAP32[$99>>2]|0;
        $101 = (($100) + ($97<<1)|0);
        $102 = HEAP16[$101>>1]|0;
        $103 = $102 << 16 >> 16;
        $104 = $103 & -1095;
        $105 = $104 & 20735;
        $106 = ($105|0)!=(0);
        $108 = $106;
       }
      }
     }
     $107 = $108 ^ 1;
     $109 = $107&1;
     $agg = $109;
    } else {
     $agg = 0;
    }
    $110 = $agg;
    $111 = ($110|0)!=(0);
    do {
     if ($111) {
      $112 = $mqc;
      $113 = (($112) + 24|0);
      $114 = (($113) + 68|0);
      $115 = $mqc;
      $116 = (($115) + 100|0);
      HEAP32[$116>>2] = $114;
      $117 = $mqc;
      $118 = (_mqc_decode($117)|0);
      $119 = ($118|0)!=(0);
      if ($119) {
       $120 = $mqc;
       $121 = (($120) + 24|0);
       $122 = (($121) + 72|0);
       $123 = $mqc;
       $124 = (($123) + 100|0);
       HEAP32[$124>>2] = $122;
       $125 = $mqc;
       $126 = (_mqc_decode($125)|0);
       $runlen = $126;
       $127 = $runlen;
       $128 = $127 << 1;
       $129 = $mqc;
       $130 = (_mqc_decode($129)|0);
       $131 = $128 | $130;
       $runlen = $131;
       label = 18;
       break;
      } else {
       break;
      }
     } else {
      $runlen = 0;
      label = 18;
     }
    } while(0);
    if ((label|0) == 18) {
     label = 0;
     $132 = $k;
     $133 = $runlen;
     $134 = (($132) + ($133))|0;
     $j = $134;
     while(1) {
      $135 = $j;
      $136 = $k;
      $137 = (($136) + 4)|0;
      $138 = ($135|0)<($137|0);
      if ($138) {
       $139 = $j;
       $140 = $0;
       $141 = (($140) + 24|0);
       $142 = HEAP32[$141>>2]|0;
       $143 = ($139|0)<($142|0);
       $510 = $143;
      } else {
       $510 = 0;
      }
      if (!($510)) {
       break;
      }
      $144 = $j;
      $145 = $k;
      $146 = (($145) + 3)|0;
      $147 = ($144|0)==($146|0);
      if ($147) {
       $155 = 1;
      } else {
       $148 = $j;
       $149 = $0;
       $150 = (($149) + 24|0);
       $151 = HEAP32[$150>>2]|0;
       $152 = (($151) - 1)|0;
       $153 = ($148|0)==($152|0);
       $155 = $153;
      }
      $154 = $155 ? 1 : 0;
      $vsc = $154;
      $156 = $0;
      $157 = $j;
      $158 = (($157) + 1)|0;
      $159 = $0;
      $160 = (($159) + 36|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = Math_imul($158, $161)|0;
      $163 = $i;
      $164 = (($162) + ($163))|0;
      $165 = (($164) + 1)|0;
      $166 = $0;
      $167 = (($166) + 16|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = (($168) + ($165<<1)|0);
      $170 = $j;
      $171 = $0;
      $172 = (($171) + 20|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = Math_imul($170, $173)|0;
      $175 = $i;
      $176 = (($174) + ($175))|0;
      $177 = $0;
      $178 = (($177) + 12|0);
      $179 = HEAP32[$178>>2]|0;
      $180 = (($179) + ($176<<2)|0);
      $181 = $2;
      $182 = $oneplushalf;
      $183 = $agg;
      $184 = ($183|0)!=(0);
      if ($184) {
       $185 = $j;
       $186 = $k;
       $187 = $runlen;
       $188 = (($186) + ($187))|0;
       $189 = ($185|0)==($188|0);
       $191 = $189;
      } else {
       $191 = 0;
      }
      $190 = $191&1;
      $192 = $vsc;
      _t1_dec_clnpass_step_vsc($156,$169,$180,$181,$182,$190,$192);
      $193 = $j;
      $194 = (($193) + 1)|0;
      $j = $194;
     }
    }
    $195 = $i;
    $196 = (($195) + 1)|0;
    $i = $196;
   }
   $197 = $k;
   $198 = (($197) + 4)|0;
   $k = $198;
  }
 } else {
  $199 = $0;
  $200 = (($199) + 12|0);
  $201 = HEAP32[$200>>2]|0;
  $data1 = $201;
  $202 = $0;
  $203 = (($202) + 16|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = (($204) + 2|0);
  $flags1 = $205;
  $k = 0;
  while(1) {
   $206 = $k;
   $207 = $0;
   $208 = (($207) + 24|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = $209 & -4;
   $211 = ($206|0)<($210|0);
   if (!($211)) {
    break;
   }
   $i = 0;
   while(1) {
    $212 = $i;
    $213 = $0;
    $214 = (($213) + 20|0);
    $215 = HEAP32[$214>>2]|0;
    $216 = ($212|0)<($215|0);
    if (!($216)) {
     break;
    }
    $217 = $data1;
    $218 = $i;
    $219 = (($217) + ($218<<2)|0);
    $data2 = $219;
    $220 = $flags1;
    $221 = $i;
    $222 = (($220) + ($221<<1)|0);
    $flags2 = $222;
    $223 = $k;
    $224 = (1 + ($223))|0;
    $225 = $0;
    $226 = (($225) + 36|0);
    $227 = HEAP32[$226>>2]|0;
    $228 = Math_imul($224, $227)|0;
    $229 = $i;
    $230 = (1 + ($229))|0;
    $231 = (($228) + ($230))|0;
    $232 = $0;
    $233 = (($232) + 16|0);
    $234 = HEAP32[$233>>2]|0;
    $235 = (($234) + ($231<<1)|0);
    $236 = HEAP16[$235>>1]|0;
    $237 = $236 << 16 >> 16;
    $238 = $237 & 20735;
    $239 = ($238|0)!=(0);
    if ($239) {
     $295 = 1;
    } else {
     $240 = $k;
     $241 = (1 + ($240))|0;
     $242 = (($241) + 1)|0;
     $243 = $0;
     $244 = (($243) + 36|0);
     $245 = HEAP32[$244>>2]|0;
     $246 = Math_imul($242, $245)|0;
     $247 = $i;
     $248 = (1 + ($247))|0;
     $249 = (($246) + ($248))|0;
     $250 = $0;
     $251 = (($250) + 16|0);
     $252 = HEAP32[$251>>2]|0;
     $253 = (($252) + ($249<<1)|0);
     $254 = HEAP16[$253>>1]|0;
     $255 = $254 << 16 >> 16;
     $256 = $255 & 20735;
     $257 = ($256|0)!=(0);
     if ($257) {
      $295 = 1;
     } else {
      $258 = $k;
      $259 = (1 + ($258))|0;
      $260 = (($259) + 2)|0;
      $261 = $0;
      $262 = (($261) + 36|0);
      $263 = HEAP32[$262>>2]|0;
      $264 = Math_imul($260, $263)|0;
      $265 = $i;
      $266 = (1 + ($265))|0;
      $267 = (($264) + ($266))|0;
      $268 = $0;
      $269 = (($268) + 16|0);
      $270 = HEAP32[$269>>2]|0;
      $271 = (($270) + ($267<<1)|0);
      $272 = HEAP16[$271>>1]|0;
      $273 = $272 << 16 >> 16;
      $274 = $273 & 20735;
      $275 = ($274|0)!=(0);
      if ($275) {
       $295 = 1;
      } else {
       $276 = $k;
       $277 = (1 + ($276))|0;
       $278 = (($277) + 3)|0;
       $279 = $0;
       $280 = (($279) + 36|0);
       $281 = HEAP32[$280>>2]|0;
       $282 = Math_imul($278, $281)|0;
       $283 = $i;
       $284 = (1 + ($283))|0;
       $285 = (($282) + ($284))|0;
       $286 = $0;
       $287 = (($286) + 16|0);
       $288 = HEAP32[$287>>2]|0;
       $289 = (($288) + ($285<<1)|0);
       $290 = HEAP16[$289>>1]|0;
       $291 = $290 << 16 >> 16;
       $292 = $291 & 20735;
       $293 = ($292|0)!=(0);
       $295 = $293;
      }
     }
    }
    $294 = $295 ^ 1;
    $296 = $294&1;
    $agg = $296;
    $297 = $agg;
    $298 = ($297|0)!=(0);
    do {
     if ($298) {
      $299 = $mqc;
      $300 = (($299) + 24|0);
      $301 = (($300) + 68|0);
      $302 = $mqc;
      $303 = (($302) + 100|0);
      HEAP32[$303>>2] = $301;
      $304 = $mqc;
      $305 = (_mqc_decode($304)|0);
      $306 = ($305|0)!=(0);
      if (!($306)) {
       break;
      }
      $307 = $mqc;
      $308 = (($307) + 24|0);
      $309 = (($308) + 72|0);
      $310 = $mqc;
      $311 = (($310) + 100|0);
      HEAP32[$311>>2] = $309;
      $312 = $mqc;
      $313 = (_mqc_decode($312)|0);
      $runlen = $313;
      $314 = $runlen;
      $315 = $314 << 1;
      $316 = $mqc;
      $317 = (_mqc_decode($316)|0);
      $318 = $315 | $317;
      $runlen = $318;
      $319 = $runlen;
      $320 = $0;
      $321 = (($320) + 36|0);
      $322 = HEAP32[$321>>2]|0;
      $323 = Math_imul($319, $322)|0;
      $324 = $flags2;
      $325 = (($324) + ($323<<1)|0);
      $flags2 = $325;
      $326 = $runlen;
      $327 = $0;
      $328 = (($327) + 20|0);
      $329 = HEAP32[$328>>2]|0;
      $330 = Math_imul($326, $329)|0;
      $331 = $data2;
      $332 = (($331) + ($330<<2)|0);
      $data2 = $332;
      $333 = $k;
      $334 = $runlen;
      $335 = (($333) + ($334))|0;
      $j = $335;
      while(1) {
       $336 = $j;
       $337 = $k;
       $338 = (($337) + 4)|0;
       $339 = ($336|0)<($338|0);
       if ($339) {
        $340 = $j;
        $341 = $0;
        $342 = (($341) + 24|0);
        $343 = HEAP32[$342>>2]|0;
        $344 = ($340|0)<($343|0);
        $511 = $344;
       } else {
        $511 = 0;
       }
       if (!($511)) {
        break;
       }
       $345 = $0;
       $346 = (($345) + 36|0);
       $347 = HEAP32[$346>>2]|0;
       $348 = $flags2;
       $349 = (($348) + ($347<<1)|0);
       $flags2 = $349;
       $350 = $agg;
       $351 = ($350|0)!=(0);
       if ($351) {
        $352 = $j;
        $353 = $k;
        $354 = $runlen;
        $355 = (($353) + ($354))|0;
        $356 = ($352|0)==($355|0);
        if ($356) {
         $357 = $0;
         $358 = $flags2;
         $359 = $data2;
         $360 = $2;
         $361 = $oneplushalf;
         _t1_dec_clnpass_step_partial($357,$358,$359,$360,$361);
        } else {
         label = 51;
        }
       } else {
        label = 51;
       }
       if ((label|0) == 51) {
        label = 0;
        $362 = $0;
        $363 = $flags2;
        $364 = $data2;
        $365 = $2;
        $366 = $oneplushalf;
        _t1_dec_clnpass_step($362,$363,$364,$365,$366);
       }
       $367 = $0;
       $368 = (($367) + 20|0);
       $369 = HEAP32[$368>>2]|0;
       $370 = $data2;
       $371 = (($370) + ($369<<2)|0);
       $data2 = $371;
       $372 = $j;
       $373 = (($372) + 1)|0;
       $j = $373;
      }
      label = 56;
     } else {
      $374 = $0;
      $375 = (($374) + 36|0);
      $376 = HEAP32[$375>>2]|0;
      $377 = $flags2;
      $378 = (($377) + ($376<<1)|0);
      $flags2 = $378;
      $379 = $0;
      $380 = $flags2;
      $381 = $data2;
      $382 = $2;
      $383 = $oneplushalf;
      _t1_dec_clnpass_step($379,$380,$381,$382,$383);
      $384 = $0;
      $385 = (($384) + 20|0);
      $386 = HEAP32[$385>>2]|0;
      $387 = $data2;
      $388 = (($387) + ($386<<2)|0);
      $data2 = $388;
      $389 = $0;
      $390 = (($389) + 36|0);
      $391 = HEAP32[$390>>2]|0;
      $392 = $flags2;
      $393 = (($392) + ($391<<1)|0);
      $flags2 = $393;
      $394 = $0;
      $395 = $flags2;
      $396 = $data2;
      $397 = $2;
      $398 = $oneplushalf;
      _t1_dec_clnpass_step($394,$395,$396,$397,$398);
      $399 = $0;
      $400 = (($399) + 20|0);
      $401 = HEAP32[$400>>2]|0;
      $402 = $data2;
      $403 = (($402) + ($401<<2)|0);
      $data2 = $403;
      $404 = $0;
      $405 = (($404) + 36|0);
      $406 = HEAP32[$405>>2]|0;
      $407 = $flags2;
      $408 = (($407) + ($406<<1)|0);
      $flags2 = $408;
      $409 = $0;
      $410 = $flags2;
      $411 = $data2;
      $412 = $2;
      $413 = $oneplushalf;
      _t1_dec_clnpass_step($409,$410,$411,$412,$413);
      $414 = $0;
      $415 = (($414) + 20|0);
      $416 = HEAP32[$415>>2]|0;
      $417 = $data2;
      $418 = (($417) + ($416<<2)|0);
      $data2 = $418;
      $419 = $0;
      $420 = (($419) + 36|0);
      $421 = HEAP32[$420>>2]|0;
      $422 = $flags2;
      $423 = (($422) + ($421<<1)|0);
      $flags2 = $423;
      $424 = $0;
      $425 = $flags2;
      $426 = $data2;
      $427 = $2;
      $428 = $oneplushalf;
      _t1_dec_clnpass_step($424,$425,$426,$427,$428);
      $429 = $0;
      $430 = (($429) + 20|0);
      $431 = HEAP32[$430>>2]|0;
      $432 = $data2;
      $433 = (($432) + ($431<<2)|0);
      $data2 = $433;
      label = 56;
     }
    } while(0);
    if ((label|0) == 56) {
     label = 0;
    }
    $434 = $i;
    $435 = (($434) + 1)|0;
    $i = $435;
   }
   $436 = $0;
   $437 = (($436) + 20|0);
   $438 = HEAP32[$437>>2]|0;
   $439 = $438 << 2;
   $440 = $data1;
   $441 = (($440) + ($439<<2)|0);
   $data1 = $441;
   $442 = $0;
   $443 = (($442) + 36|0);
   $444 = HEAP32[$443>>2]|0;
   $445 = $444 << 2;
   $446 = $flags1;
   $447 = (($446) + ($445<<1)|0);
   $flags1 = $447;
   $448 = $k;
   $449 = (($448) + 4)|0;
   $k = $449;
  }
  $i = 0;
  while(1) {
   $450 = $i;
   $451 = $0;
   $452 = (($451) + 20|0);
   $453 = HEAP32[$452>>2]|0;
   $454 = ($450|0)<($453|0);
   if (!($454)) {
    break;
   }
   $455 = $data1;
   $456 = $i;
   $457 = (($455) + ($456<<2)|0);
   $data21 = $457;
   $458 = $flags1;
   $459 = $i;
   $460 = (($458) + ($459<<1)|0);
   $flags22 = $460;
   $461 = $k;
   $j = $461;
   while(1) {
    $462 = $j;
    $463 = $0;
    $464 = (($463) + 24|0);
    $465 = HEAP32[$464>>2]|0;
    $466 = ($462|0)<($465|0);
    if (!($466)) {
     break;
    }
    $467 = $0;
    $468 = (($467) + 36|0);
    $469 = HEAP32[$468>>2]|0;
    $470 = $flags22;
    $471 = (($470) + ($469<<1)|0);
    $flags22 = $471;
    $472 = $0;
    $473 = $flags22;
    $474 = $data21;
    $475 = $2;
    $476 = $oneplushalf;
    _t1_dec_clnpass_step($472,$473,$474,$475,$476);
    $477 = $0;
    $478 = (($477) + 20|0);
    $479 = HEAP32[$478>>2]|0;
    $480 = $data21;
    $481 = (($480) + ($479<<2)|0);
    $data21 = $481;
    $482 = $j;
    $483 = (($482) + 1)|0;
    $j = $483;
   }
   $484 = $i;
   $485 = (($484) + 1)|0;
   $i = $485;
  }
 }
 $486 = $segsym;
 $487 = ($486|0)!=(0);
 if (!($487)) {
  STACKTOP = sp;return;
 }
 $v = 0;
 $488 = $mqc;
 $489 = (($488) + 24|0);
 $490 = (($489) + 72|0);
 $491 = $mqc;
 $492 = (($491) + 100|0);
 HEAP32[$492>>2] = $490;
 $493 = $mqc;
 $494 = (_mqc_decode($493)|0);
 $v = $494;
 $495 = $v;
 $496 = $495 << 1;
 $497 = $mqc;
 $498 = (_mqc_decode($497)|0);
 $499 = $496 | $498;
 $v = $499;
 $500 = $v;
 $501 = $500 << 1;
 $502 = $mqc;
 $503 = (_mqc_decode($502)|0);
 $504 = $501 | $503;
 $v = $504;
 $505 = $v;
 $506 = $505 << 1;
 $507 = $mqc;
 $508 = (_mqc_decode($507)|0);
 $509 = $506 | $508;
 $v = $509;
 STACKTOP = sp;return;
}
function _t1_dec_clnpass_step_vsc($t1,$flagsp,$datap,$orient,$oneplushalf,$partial,$vsc) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $orient = $orient|0;
 $oneplushalf = $oneplushalf|0;
 $partial = $partial|0;
 $vsc = $vsc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $flag = 0, $mqc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $orient;
 $4 = $oneplushalf;
 $5 = $partial;
 $6 = $vsc;
 $7 = $0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $mqc = $9;
 $10 = $6;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $1;
  $13 = HEAP16[$12>>1]|0;
  $14 = $13 << 16 >> 16;
  $15 = $14 & -1095;
  $19 = $15;
 } else {
  $16 = $1;
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = $18;
 }
 $flag = $19;
 $20 = $5;
 $21 = ($20|0)!=(0);
 if ($21) {
  label = 9;
 } else {
  $22 = $flag;
  $23 = $22 & 20480;
  $24 = ($23|0)!=(0);
  if ($24) {
   $63 = $1;
   $64 = HEAP16[$63>>1]|0;
   $65 = $64 << 16 >> 16;
   $66 = $65 & -16385;
   $67 = $66&65535;
   HEAP16[$63>>1] = $67;
   STACKTOP = sp;return;
  }
  $25 = $flag;
  $26 = $3;
  $27 = (_t1_getctxno_zc($25,$26)|0);
  $28 = $27 << 24 >> 24;
  $29 = $mqc;
  $30 = (($29) + 24|0);
  $31 = (($30) + ($28<<2)|0);
  $32 = $mqc;
  $33 = (($32) + 100|0);
  HEAP32[$33>>2] = $31;
  $34 = $mqc;
  $35 = (_mqc_decode($34)|0);
  $36 = ($35|0)!=(0);
  if ($36) {
   label = 9;
  }
 }
 if ((label|0) == 9) {
  $37 = $flag;
  $38 = (_t1_getctxno_sc($37)|0);
  $39 = $38 << 24 >> 24;
  $40 = $mqc;
  $41 = (($40) + 24|0);
  $42 = (($41) + ($39<<2)|0);
  $43 = $mqc;
  $44 = (($43) + 100|0);
  HEAP32[$44>>2] = $42;
  $45 = $mqc;
  $46 = (_mqc_decode($45)|0);
  $47 = $flag;
  $48 = (_t1_getspb($47)|0);
  $49 = $48 << 24 >> 24;
  $50 = $46 ^ $49;
  $v = $50;
  $51 = $v;
  $52 = ($51|0)!=(0);
  if ($52) {
   $53 = $4;
   $54 = (0 - ($53))|0;
   $57 = $54;
  } else {
   $55 = $4;
   $57 = $55;
  }
  $56 = $2;
  HEAP32[$56>>2] = $57;
  $58 = $1;
  $59 = $v;
  $60 = $0;
  $61 = (($60) + 36|0);
  $62 = HEAP32[$61>>2]|0;
  _t1_updateflags($58,$59,$62);
 }
 $63 = $1;
 $64 = HEAP16[$63>>1]|0;
 $65 = $64 << 16 >> 16;
 $66 = $65 & -16385;
 $67 = $66&65535;
 HEAP16[$63>>1] = $67;
 STACKTOP = sp;return;
}
function _t1_dec_clnpass_step_partial($t1,$flagsp,$datap,$orient,$oneplushalf) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $orient = $orient|0;
 $oneplushalf = $oneplushalf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $flag = 0, $mqc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $orient;
 $4 = $oneplushalf;
 $5 = $0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $mqc = $7;
 $8 = $1;
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $flag = $10;
 $11 = $flag;
 $12 = (_t1_getctxno_sc($11)|0);
 $13 = $12 << 24 >> 24;
 $14 = $mqc;
 $15 = (($14) + 24|0);
 $16 = (($15) + ($13<<2)|0);
 $17 = $mqc;
 $18 = (($17) + 100|0);
 HEAP32[$18>>2] = $16;
 $19 = $mqc;
 $20 = (_mqc_decode($19)|0);
 $21 = $flag;
 $22 = (_t1_getspb($21)|0);
 $23 = $22 << 24 >> 24;
 $24 = $20 ^ $23;
 $v = $24;
 $25 = $v;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $4;
  $28 = (0 - ($27))|0;
  $31 = $28;
 } else {
  $29 = $4;
  $31 = $29;
 }
 $30 = $2;
 HEAP32[$30>>2] = $31;
 $32 = $1;
 $33 = $v;
 $34 = $0;
 $35 = (($34) + 36|0);
 $36 = HEAP32[$35>>2]|0;
 _t1_updateflags($32,$33,$36);
 $37 = $1;
 $38 = HEAP16[$37>>1]|0;
 $39 = $38 << 16 >> 16;
 $40 = $39 & -16385;
 $41 = $40&65535;
 HEAP16[$37>>1] = $41;
 STACKTOP = sp;return;
}
function _t1_dec_clnpass_step($t1,$flagsp,$datap,$orient,$oneplushalf) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $orient = $orient|0;
 $oneplushalf = $oneplushalf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flag = 0, $mqc = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $orient;
 $4 = $oneplushalf;
 $5 = $0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $mqc = $7;
 $8 = $1;
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $flag = $10;
 $11 = $flag;
 $12 = $11 & 20480;
 $13 = ($12|0)!=(0);
 if ($13) {
  $52 = $1;
  $53 = HEAP16[$52>>1]|0;
  $54 = $53 << 16 >> 16;
  $55 = $54 & -16385;
  $56 = $55&65535;
  HEAP16[$52>>1] = $56;
  STACKTOP = sp;return;
 }
 $14 = $flag;
 $15 = $3;
 $16 = (_t1_getctxno_zc($14,$15)|0);
 $17 = $16 << 24 >> 24;
 $18 = $mqc;
 $19 = (($18) + 24|0);
 $20 = (($19) + ($17<<2)|0);
 $21 = $mqc;
 $22 = (($21) + 100|0);
 HEAP32[$22>>2] = $20;
 $23 = $mqc;
 $24 = (_mqc_decode($23)|0);
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = $flag;
  $27 = (_t1_getctxno_sc($26)|0);
  $28 = $27 << 24 >> 24;
  $29 = $mqc;
  $30 = (($29) + 24|0);
  $31 = (($30) + ($28<<2)|0);
  $32 = $mqc;
  $33 = (($32) + 100|0);
  HEAP32[$33>>2] = $31;
  $34 = $mqc;
  $35 = (_mqc_decode($34)|0);
  $36 = $flag;
  $37 = (_t1_getspb($36)|0);
  $38 = $37 << 24 >> 24;
  $39 = $35 ^ $38;
  $v = $39;
  $40 = $v;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = $4;
   $43 = (0 - ($42))|0;
   $46 = $43;
  } else {
   $44 = $4;
   $46 = $44;
  }
  $45 = $2;
  HEAP32[$45>>2] = $46;
  $47 = $1;
  $48 = $v;
  $49 = $0;
  $50 = (($49) + 36|0);
  $51 = HEAP32[$50>>2]|0;
  _t1_updateflags($47,$48,$51);
 }
 $52 = $1;
 $53 = HEAP16[$52>>1]|0;
 $54 = $53 << 16 >> 16;
 $55 = $54 & -16385;
 $56 = $55&65535;
 HEAP16[$52>>1] = $56;
 STACKTOP = sp;return;
}
function _t1_getctxno_zc($f,$orient) {
 $f = $f|0;
 $orient = $orient|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $f;
 $1 = $orient;
 $2 = $1;
 $3 = $2 << 8;
 $4 = $0;
 $5 = $4 & 255;
 $6 = $3 | $5;
 $7 = (55696 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 STACKTOP = sp;return ($8|0);
}
function _t1_getctxno_sc($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $f;
 $1 = $0;
 $2 = $1 & 4080;
 $3 = $2 >> 4;
 $4 = (55440 + ($3)|0);
 $5 = HEAP8[$4>>0]|0;
 STACKTOP = sp;return ($5|0);
}
function _t1_getspb($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $f;
 $1 = $0;
 $2 = $1 & 4080;
 $3 = $2 >> 4;
 $4 = (55184 + ($3)|0);
 $5 = HEAP8[$4>>0]|0;
 STACKTOP = sp;return ($5|0);
}
function _t1_updateflags($flagsp,$s,$stride) {
 $flagsp = $flagsp|0;
 $s = $s|0;
 $stride = $stride|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $np = 0, $sp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $flagsp;
 $1 = $s;
 $2 = $stride;
 $3 = $0;
 $4 = $2;
 $5 = (0 - ($4))|0;
 $6 = (($3) + ($5<<1)|0);
 $np = $6;
 $7 = $0;
 $8 = $2;
 $9 = (($7) + ($8<<1)|0);
 $sp = $9;
 $10 = $np;
 $11 = (($10) + -2|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 << 16 >> 16;
 $14 = $13 | 2;
 $15 = $14&65535;
 HEAP16[$11>>1] = $15;
 $16 = $1;
 $17 = (55168 + ($16<<1)|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = $np;
 $21 = HEAP16[$20>>1]|0;
 $22 = $21 << 16 >> 16;
 $23 = $22 | $19;
 $24 = $23&65535;
 HEAP16[$20>>1] = $24;
 $25 = $np;
 $26 = (($25) + 2|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27 << 16 >> 16;
 $29 = $28 | 4;
 $30 = $29&65535;
 HEAP16[$26>>1] = $30;
 $31 = $1;
 $32 = (($31) + 2)|0;
 $33 = (55168 + ($32<<1)|0);
 $34 = HEAP16[$33>>1]|0;
 $35 = $34 << 16 >> 16;
 $36 = $0;
 $37 = (($36) + -2|0);
 $38 = HEAP16[$37>>1]|0;
 $39 = $38 << 16 >> 16;
 $40 = $39 | $35;
 $41 = $40&65535;
 HEAP16[$37>>1] = $41;
 $42 = $0;
 $43 = HEAP16[$42>>1]|0;
 $44 = $43 << 16 >> 16;
 $45 = $44 | 4096;
 $46 = $45&65535;
 HEAP16[$42>>1] = $46;
 $47 = $1;
 $48 = (($47) + 4)|0;
 $49 = (55168 + ($48<<1)|0);
 $50 = HEAP16[$49>>1]|0;
 $51 = $50 << 16 >> 16;
 $52 = $0;
 $53 = (($52) + 2|0);
 $54 = HEAP16[$53>>1]|0;
 $55 = $54 << 16 >> 16;
 $56 = $55 | $51;
 $57 = $56&65535;
 HEAP16[$53>>1] = $57;
 $58 = $sp;
 $59 = (($58) + -2|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60 << 16 >> 16;
 $62 = $61 | 1;
 $63 = $62&65535;
 HEAP16[$59>>1] = $63;
 $64 = $1;
 $65 = (($64) + 6)|0;
 $66 = (55168 + ($65<<1)|0);
 $67 = HEAP16[$66>>1]|0;
 $68 = $67 << 16 >> 16;
 $69 = $sp;
 $70 = HEAP16[$69>>1]|0;
 $71 = $70 << 16 >> 16;
 $72 = $71 | $68;
 $73 = $72&65535;
 HEAP16[$69>>1] = $73;
 $74 = $sp;
 $75 = (($74) + 2|0);
 $76 = HEAP16[$75>>1]|0;
 $77 = $76 << 16 >> 16;
 $78 = $77 | 8;
 $79 = $78&65535;
 HEAP16[$75>>1] = $79;
 STACKTOP = sp;return;
}
function _t1_dec_refpass_step_mqc($t1,$flagsp,$datap,$poshalf,$neghalf) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $poshalf = $poshalf|0;
 $neghalf = $neghalf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $flag = 0, $mqc = 0, $t = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $poshalf;
 $4 = $neghalf;
 $5 = $0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $mqc = $7;
 $8 = $1;
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $flag = $10;
 $11 = $flag;
 $12 = $11 & 20480;
 $13 = ($12|0)==(4096);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = $flag;
 $15 = (_t1_getctxno_mag($14)|0);
 $16 = $mqc;
 $17 = (($16) + 24|0);
 $18 = (($17) + ($15<<2)|0);
 $19 = $mqc;
 $20 = (($19) + 100|0);
 HEAP32[$20>>2] = $18;
 $21 = $mqc;
 $22 = (_mqc_decode($21)|0);
 $v = $22;
 $23 = $v;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = $3;
  $27 = $25;
 } else {
  $26 = $4;
  $27 = $26;
 }
 $t = $27;
 $28 = $2;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)<(0);
 if ($30) {
  $31 = $t;
  $32 = (0 - ($31))|0;
  $37 = $32;
 } else {
  $33 = $t;
  $37 = $33;
 }
 $34 = $2;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + ($37))|0;
 HEAP32[$34>>2] = $36;
 $38 = $1;
 $39 = HEAP16[$38>>1]|0;
 $40 = $39 << 16 >> 16;
 $41 = $40 | 8192;
 $42 = $41&65535;
 HEAP16[$38>>1] = $42;
 STACKTOP = sp;return;
}
function _t1_getctxno_mag($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tmp1 = 0, $tmp2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $f;
 $1 = $0;
 $2 = $1 & 255;
 $3 = ($2|0)!=(0);
 $4 = $3 ? 15 : 14;
 $tmp1 = $4;
 $5 = $0;
 $6 = $5 & 8192;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = 16;
 } else {
  $8 = $tmp1;
  $9 = $8;
 }
 $tmp2 = $9;
 $10 = $tmp2;
 STACKTOP = sp;return ($10|0);
}
function _t1_dec_refpass_step_mqc_vsc($t1,$flagsp,$datap,$poshalf,$neghalf,$vsc) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $poshalf = $poshalf|0;
 $neghalf = $neghalf|0;
 $vsc = $vsc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flag = 0, $mqc = 0, $t = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $poshalf;
 $4 = $neghalf;
 $5 = $vsc;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mqc = $8;
 $9 = $5;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $1;
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = $13 & -1095;
  $18 = $14;
 } else {
  $15 = $1;
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = $17;
 }
 $flag = $18;
 $19 = $flag;
 $20 = $19 & 20480;
 $21 = ($20|0)==(4096);
 if (!($21)) {
  STACKTOP = sp;return;
 }
 $22 = $flag;
 $23 = (_t1_getctxno_mag($22)|0);
 $24 = $mqc;
 $25 = (($24) + 24|0);
 $26 = (($25) + ($23<<2)|0);
 $27 = $mqc;
 $28 = (($27) + 100|0);
 HEAP32[$28>>2] = $26;
 $29 = $mqc;
 $30 = (_mqc_decode($29)|0);
 $v = $30;
 $31 = $v;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = $3;
  $35 = $33;
 } else {
  $34 = $4;
  $35 = $34;
 }
 $t = $35;
 $36 = $2;
 $37 = HEAP32[$36>>2]|0;
 $38 = ($37|0)<(0);
 if ($38) {
  $39 = $t;
  $40 = (0 - ($39))|0;
  $45 = $40;
 } else {
  $41 = $t;
  $45 = $41;
 }
 $42 = $2;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + ($45))|0;
 HEAP32[$42>>2] = $44;
 $46 = $1;
 $47 = HEAP16[$46>>1]|0;
 $48 = $47 << 16 >> 16;
 $49 = $48 | 8192;
 $50 = $49&65535;
 HEAP16[$46>>1] = $50;
 STACKTOP = sp;return;
}
function _t1_dec_refpass_step_raw($t1,$flagsp,$datap,$poshalf,$neghalf,$vsc) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $poshalf = $poshalf|0;
 $neghalf = $neghalf|0;
 $vsc = $vsc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flag = 0, $raw = 0, $t = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $poshalf;
 $4 = $neghalf;
 $5 = $vsc;
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $raw = $8;
 $9 = $5;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $1;
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = $13 & -1095;
  $18 = $14;
 } else {
  $15 = $1;
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = $17;
 }
 $flag = $18;
 $19 = $flag;
 $20 = $19 & 20480;
 $21 = ($20|0)==(4096);
 if (!($21)) {
  STACKTOP = sp;return;
 }
 $22 = $raw;
 $23 = (_raw_decode($22)|0);
 $v = $23;
 $24 = $v;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = $3;
  $28 = $26;
 } else {
  $27 = $4;
  $28 = $27;
 }
 $t = $28;
 $29 = $2;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)<(0);
 if ($31) {
  $32 = $t;
  $33 = (0 - ($32))|0;
  $38 = $33;
 } else {
  $34 = $t;
  $38 = $34;
 }
 $35 = $2;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + ($38))|0;
 HEAP32[$35>>2] = $37;
 $39 = $1;
 $40 = HEAP16[$39>>1]|0;
 $41 = $40 << 16 >> 16;
 $42 = $41 | 8192;
 $43 = $42&65535;
 HEAP16[$39>>1] = $43;
 STACKTOP = sp;return;
}
function _t1_dec_sigpass_step_mqc($t1,$flagsp,$datap,$orient,$oneplushalf) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $orient = $orient|0;
 $oneplushalf = $oneplushalf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $flag = 0, $mqc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $orient;
 $4 = $oneplushalf;
 $5 = $0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $mqc = $7;
 $8 = $1;
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $flag = $10;
 $11 = $flag;
 $12 = $11 & 255;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = $flag;
 $15 = $14 & 20480;
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = $flag;
 $18 = $3;
 $19 = (_t1_getctxno_zc($17,$18)|0);
 $20 = $19 << 24 >> 24;
 $21 = $mqc;
 $22 = (($21) + 24|0);
 $23 = (($22) + ($20<<2)|0);
 $24 = $mqc;
 $25 = (($24) + 100|0);
 HEAP32[$25>>2] = $23;
 $26 = $mqc;
 $27 = (_mqc_decode($26)|0);
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = $flag;
  $30 = (_t1_getctxno_sc($29)|0);
  $31 = $30 << 24 >> 24;
  $32 = $mqc;
  $33 = (($32) + 24|0);
  $34 = (($33) + ($31<<2)|0);
  $35 = $mqc;
  $36 = (($35) + 100|0);
  HEAP32[$36>>2] = $34;
  $37 = $mqc;
  $38 = (_mqc_decode($37)|0);
  $39 = $flag;
  $40 = (_t1_getspb($39)|0);
  $41 = $40 << 24 >> 24;
  $42 = $38 ^ $41;
  $v = $42;
  $43 = $v;
  $44 = ($43|0)!=(0);
  if ($44) {
   $45 = $4;
   $46 = (0 - ($45))|0;
   $49 = $46;
  } else {
   $47 = $4;
   $49 = $47;
  }
  $48 = $2;
  HEAP32[$48>>2] = $49;
  $50 = $1;
  $51 = $v;
  $52 = $0;
  $53 = (($52) + 36|0);
  $54 = HEAP32[$53>>2]|0;
  _t1_updateflags($50,$51,$54);
 }
 $55 = $1;
 $56 = HEAP16[$55>>1]|0;
 $57 = $56 << 16 >> 16;
 $58 = $57 | 16384;
 $59 = $58&65535;
 HEAP16[$55>>1] = $59;
 STACKTOP = sp;return;
}
function _t1_dec_sigpass_step_mqc_vsc($t1,$flagsp,$datap,$orient,$oneplushalf,$vsc) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $orient = $orient|0;
 $oneplushalf = $oneplushalf|0;
 $vsc = $vsc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $flag = 0, $mqc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $orient;
 $4 = $oneplushalf;
 $5 = $vsc;
 $6 = $0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $mqc = $8;
 $9 = $5;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $1;
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = $13 & -1095;
  $18 = $14;
 } else {
  $15 = $1;
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = $17;
 }
 $flag = $18;
 $19 = $flag;
 $20 = $19 & 255;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  STACKTOP = sp;return;
 }
 $22 = $flag;
 $23 = $22 & 20480;
 $24 = ($23|0)!=(0);
 if ($24) {
  STACKTOP = sp;return;
 }
 $25 = $flag;
 $26 = $3;
 $27 = (_t1_getctxno_zc($25,$26)|0);
 $28 = $27 << 24 >> 24;
 $29 = $mqc;
 $30 = (($29) + 24|0);
 $31 = (($30) + ($28<<2)|0);
 $32 = $mqc;
 $33 = (($32) + 100|0);
 HEAP32[$33>>2] = $31;
 $34 = $mqc;
 $35 = (_mqc_decode($34)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = $flag;
  $38 = (_t1_getctxno_sc($37)|0);
  $39 = $38 << 24 >> 24;
  $40 = $mqc;
  $41 = (($40) + 24|0);
  $42 = (($41) + ($39<<2)|0);
  $43 = $mqc;
  $44 = (($43) + 100|0);
  HEAP32[$44>>2] = $42;
  $45 = $mqc;
  $46 = (_mqc_decode($45)|0);
  $47 = $flag;
  $48 = (_t1_getspb($47)|0);
  $49 = $48 << 24 >> 24;
  $50 = $46 ^ $49;
  $v = $50;
  $51 = $v;
  $52 = ($51|0)!=(0);
  if ($52) {
   $53 = $4;
   $54 = (0 - ($53))|0;
   $57 = $54;
  } else {
   $55 = $4;
   $57 = $55;
  }
  $56 = $2;
  HEAP32[$56>>2] = $57;
  $58 = $1;
  $59 = $v;
  $60 = $0;
  $61 = (($60) + 36|0);
  $62 = HEAP32[$61>>2]|0;
  _t1_updateflags($58,$59,$62);
 }
 $63 = $1;
 $64 = HEAP16[$63>>1]|0;
 $65 = $64 << 16 >> 16;
 $66 = $65 | 16384;
 $67 = $66&65535;
 HEAP16[$63>>1] = $67;
 STACKTOP = sp;return;
}
function _t1_dec_sigpass_step_raw($t1,$flagsp,$datap,$orient,$oneplushalf,$vsc) {
 $t1 = $t1|0;
 $flagsp = $flagsp|0;
 $datap = $datap|0;
 $orient = $orient|0;
 $oneplushalf = $oneplushalf|0;
 $vsc = $vsc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flag = 0, $raw = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $t1;
 $1 = $flagsp;
 $2 = $datap;
 $3 = $orient;
 $4 = $oneplushalf;
 $5 = $vsc;
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $raw = $8;
 $9 = $5;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $1;
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = $13 & -1095;
  $18 = $14;
 } else {
  $15 = $1;
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = $17;
 }
 $flag = $18;
 $19 = $flag;
 $20 = $19 & 255;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  STACKTOP = sp;return;
 }
 $22 = $flag;
 $23 = $22 & 20480;
 $24 = ($23|0)!=(0);
 if ($24) {
  STACKTOP = sp;return;
 }
 $25 = $raw;
 $26 = (_raw_decode($25)|0);
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = $raw;
  $29 = (_raw_decode($28)|0);
  $v = $29;
  $30 = $v;
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = $4;
   $33 = (0 - ($32))|0;
   $36 = $33;
  } else {
   $34 = $4;
   $36 = $34;
  }
  $35 = $2;
  HEAP32[$35>>2] = $36;
  $37 = $1;
  $38 = $v;
  $39 = $0;
  $40 = (($39) + 36|0);
  $41 = HEAP32[$40>>2]|0;
  _t1_updateflags($37,$38,$41);
 }
 $42 = $1;
 $43 = HEAP16[$42>>1]|0;
 $44 = $43 << 16 >> 16;
 $45 = $44 | 16384;
 $46 = $45&65535;
 HEAP16[$42>>1] = $46;
 STACKTOP = sp;return;
}
function _t2_decode_packets($t2,$src,$len,$tileno,$tile,$cstr_info) {
 $t2 = $t2|0;
 $src = $src|0;
 $len = $len|0;
 $tileno = $tileno|0;
 $tile = $tile|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $cp = 0, $curtp = 0;
 var $e = 0, $image = 0, $info_PK = 0, $info_TL = 0, $n = 0, $pack_info = 0, $pi = 0, $pino = 0, $tp_start_packno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $1 = $t2;
 $2 = $src;
 $3 = $len;
 $4 = $tileno;
 $5 = $tile;
 $6 = $cstr_info;
 $7 = $2;
 $c = $7;
 $e = 0;
 $n = 0;
 $curtp = 0;
 $8 = $1;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $image = $10;
 $11 = $1;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $cp = $13;
 $14 = $image;
 $15 = $cp;
 $16 = $4;
 $17 = (_pi_create_decode($14,$15,$16)|0);
 $pi = $17;
 $18 = $pi;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $0 = -999;
  $276 = $0;
  STACKTOP = sp;return ($276|0);
 }
 $tp_start_packno = 0;
 $pino = 0;
 L5: while(1) {
  $20 = $pino;
  $21 = $4;
  $22 = $cp;
  $23 = (($22) + 108|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + (($21*5588)|0)|0);
  $26 = (($25) + 420|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($20|0)<=($27|0);
  if (!($28)) {
   break;
  }
  while(1) {
   $29 = $pino;
   $30 = $pi;
   $31 = (($30) + (($29*232)|0)|0);
   $32 = (_pi_next($31)|0);
   $33 = ($32|0)!=(0);
   if (!($33)) {
    break;
   }
   $34 = $cp;
   $35 = (($34) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)==(0);
   if ($37) {
    label = 9;
   } else {
    $38 = $cp;
    $39 = (($38) + 40|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = $pino;
    $42 = $pi;
    $43 = (($42) + (($41*232)|0)|0);
    $44 = (($43) + 36|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (($45) + 1)|0;
    $47 = ($40|0)>=($46|0);
    if ($47) {
     label = 9;
    } else {
     $e = 0;
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $48 = $6;
    $49 = ($48|0)!=(0|0);
    if ($49) {
     $50 = $6;
     $51 = (($50) + 8|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = $4;
     $54 = $6;
     $55 = (($54) + 88|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (($56) + (($53*576)|0)|0);
     $58 = (($57) + 548|0);
     $59 = HEAP32[$58>>2]|0;
     $60 = (($59) + (($52*24)|0)|0);
     $pack_info = $60;
    } else {
     $pack_info = 0;
    }
    $61 = $1;
    $62 = $c;
    $63 = $2;
    $64 = $3;
    $65 = (($63) + ($64)|0);
    $66 = $c;
    $67 = $65;
    $68 = $66;
    $69 = (($67) - ($68))|0;
    $70 = $5;
    $71 = $4;
    $72 = $cp;
    $73 = (($72) + 108|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + (($71*5588)|0)|0);
    $76 = $pino;
    $77 = $pi;
    $78 = (($77) + (($76*232)|0)|0);
    $79 = $pack_info;
    $80 = (_t2_decode_packet($61,$62,$69,$70,$75,$78,$79)|0);
    $e = $80;
   }
   $81 = $e;
   $82 = ($81|0)==(-999);
   if ($82) {
    label = 15;
    break L5;
   }
   $83 = $e;
   $84 = ($83|0)>(0);
   if ($84) {
    $85 = $pino;
    $86 = $pi;
    $87 = (($86) + (($85*232)|0)|0);
    $88 = (($87) + 28|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = $pino;
    $91 = $pi;
    $92 = (($91) + (($90*232)|0)|0);
    $93 = (($92) + 24|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = $image;
    $96 = (($95) + 24|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97) + (($94*48)|0)|0);
    $99 = (($98) + 36|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = (_int_max49($89,$100)|0);
    $123 = $101;
   } else {
    $102 = $pino;
    $103 = $pi;
    $104 = (($103) + (($102*232)|0)|0);
    $105 = (($104) + 24|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = $image;
    $108 = (($107) + 24|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + (($106*48)|0)|0);
    $111 = (($110) + 36|0);
    $112 = HEAP32[$111>>2]|0;
    $123 = $112;
   }
   $113 = $pino;
   $114 = $pi;
   $115 = (($114) + (($113*232)|0)|0);
   $116 = (($115) + 24|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = $image;
   $119 = (($118) + 24|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (($120) + (($117*48)|0)|0);
   $122 = (($121) + 36|0);
   HEAP32[$122>>2] = $123;
   $124 = $n;
   $125 = (($124) + 1)|0;
   $n = $125;
   $126 = $6;
   $127 = ($126|0)!=(0|0);
   if ($127) {
    $128 = $4;
    $129 = $6;
    $130 = (($129) + 88|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = (($131) + (($128*576)|0)|0);
    $info_TL = $132;
    $133 = $6;
    $134 = (($133) + 8|0);
    $135 = HEAP32[$134>>2]|0;
    $136 = $info_TL;
    $137 = (($136) + 548|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = (($138) + (($135*24)|0)|0);
    $info_PK = $139;
    $140 = $6;
    $141 = (($140) + 8|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ($142|0)!=(0);
    if ($143) {
     $149 = $6;
     $150 = (($149) + 8|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = (($151) - 1)|0;
     $153 = $info_TL;
     $154 = (($153) + 548|0);
     $155 = HEAP32[$154>>2]|0;
     $156 = (($155) + (($152*24)|0)|0);
     $157 = (($156) + 8|0);
     $158 = HEAP32[$157>>2]|0;
     $159 = $curtp;
     $160 = $4;
     $161 = $6;
     $162 = (($161) + 88|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = (($163) + (($160*576)|0)|0);
     $165 = (($164) + 572|0);
     $166 = HEAP32[$165>>2]|0;
     $167 = (($166) + (($159*20)|0)|0);
     $168 = (($167) + 8|0);
     $169 = HEAP32[$168>>2]|0;
     $170 = ($158|0)>=($169|0);
     if ($170) {
      $171 = $6;
      $172 = (($171) + 8|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = $tp_start_packno;
      $175 = (($173) - ($174))|0;
      $176 = $curtp;
      $177 = $info_TL;
      $178 = (($177) + 572|0);
      $179 = HEAP32[$178>>2]|0;
      $180 = (($179) + (($176*20)|0)|0);
      $181 = (($180) + 16|0);
      HEAP32[$181>>2] = $175;
      $182 = $6;
      $183 = (($182) + 8|0);
      $184 = HEAP32[$183>>2]|0;
      $tp_start_packno = $184;
      $185 = $curtp;
      $186 = (($185) + 1)|0;
      $curtp = $186;
      $187 = $curtp;
      $188 = $4;
      $189 = $6;
      $190 = (($189) + 88|0);
      $191 = HEAP32[$190>>2]|0;
      $192 = (($191) + (($188*576)|0)|0);
      $193 = (($192) + 572|0);
      $194 = HEAP32[$193>>2]|0;
      $195 = (($194) + (($187*20)|0)|0);
      $196 = (($195) + 4|0);
      $197 = HEAP32[$196>>2]|0;
      $198 = (($197) + 1)|0;
      $199 = $info_PK;
      HEAP32[$199>>2] = $198;
     } else {
      $200 = $cp;
      $201 = (($200) + 16|0);
      $202 = HEAP8[$201>>0]|0;
      $203 = $202 << 24 >> 24;
      $204 = ($203|0)!=(0);
      if ($204) {
       $205 = $info_PK;
       $206 = HEAP32[$205>>2]|0;
       $207 = ($206|0)!=(0);
       if ($207) {
        $208 = $info_PK;
        $209 = HEAP32[$208>>2]|0;
        $222 = $209;
       } else {
        label = 27;
       }
      } else {
       label = 27;
      }
      if ((label|0) == 27) {
       label = 0;
       $210 = $6;
       $211 = (($210) + 8|0);
       $212 = HEAP32[$211>>2]|0;
       $213 = (($212) - 1)|0;
       $214 = $info_TL;
       $215 = (($214) + 548|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = (($216) + (($213*24)|0)|0);
       $218 = (($217) + 8|0);
       $219 = HEAP32[$218>>2]|0;
       $220 = (($219) + 1)|0;
       $222 = $220;
      }
      $221 = $info_PK;
      HEAP32[$221>>2] = $222;
     }
    } else {
     $144 = $info_TL;
     $145 = (($144) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = (($146) + 1)|0;
     $148 = $info_PK;
     HEAP32[$148>>2] = $147;
    }
    $223 = $info_PK;
    $224 = HEAP32[$223>>2]|0;
    $225 = $e;
    $226 = (($224) + ($225))|0;
    $227 = (($226) - 1)|0;
    $228 = $info_PK;
    $229 = (($228) + 8|0);
    HEAP32[$229>>2] = $227;
    $230 = $info_PK;
    $231 = HEAP32[$230>>2]|0;
    $232 = (($231) - 1)|0;
    $233 = $info_PK;
    $234 = (($233) + 4|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = (($235) + ($232))|0;
    HEAP32[$234>>2] = $236;
    $237 = $6;
    $238 = (($237) + 8|0);
    $239 = HEAP32[$238>>2]|0;
    $240 = (($239) + 1)|0;
    HEAP32[$238>>2] = $240;
   }
   $241 = $e;
   $242 = ($241|0)==(-999);
   if ($242) {
    label = 32;
    break;
   }
   $243 = $e;
   $244 = $c;
   $245 = (($244) + ($243)|0);
   $c = $245;
  }
  if ((label|0) == 32) {
   label = 0;
  }
  $246 = $pino;
  $247 = (($246) + 1)|0;
  $pino = $247;
 }
 if ((label|0) == 15) {
  $0 = -999;
  $276 = $0;
  STACKTOP = sp;return ($276|0);
 }
 $248 = $6;
 $249 = ($248|0)!=(0|0);
 if ($249) {
  $250 = $6;
  $251 = (($250) + 8|0);
  $252 = HEAP32[$251>>2]|0;
  $253 = $tp_start_packno;
  $254 = (($252) - ($253))|0;
  $255 = $curtp;
  $256 = $4;
  $257 = $6;
  $258 = (($257) + 88|0);
  $259 = HEAP32[$258>>2]|0;
  $260 = (($259) + (($256*576)|0)|0);
  $261 = (($260) + 572|0);
  $262 = HEAP32[$261>>2]|0;
  $263 = (($262) + (($255*20)|0)|0);
  $264 = (($263) + 16|0);
  HEAP32[$264>>2] = $254;
 }
 $265 = $pi;
 $266 = $cp;
 $267 = $4;
 _pi_destroy($265,$266,$267);
 $268 = $e;
 $269 = ($268|0)==(-999);
 if ($269) {
  $270 = $e;
  $0 = $270;
  $276 = $0;
  STACKTOP = sp;return ($276|0);
 } else {
  $271 = $c;
  $272 = $2;
  $273 = $271;
  $274 = $272;
  $275 = (($273) - ($274))|0;
  $0 = $275;
  $276 = $0;
  STACKTOP = sp;return ($276|0);
 }
 return 0|0;
}
function _t2_decode_packet($t2,$src,$len,$tile,$tcp,$pi,$pack_info) {
 $t2 = $t2|0;
 $src = $src|0;
 $len = $len|0;
 $tile = $tile|0;
 $tcp = $tcp|0;
 $pi = $pi|0;
 $pack_info = $pack_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $band = 0, $band1 = 0, $band4 = 0;
 var $bandno = 0, $bio = 0, $c = 0, $cblk = 0, $cblk3 = 0, $cblk6 = 0, $cblkno = 0, $compno = 0, $cp = 0, $hd = 0, $i = 0, $included = 0, $increment = 0, $layno = 0, $n = 0, $numimsbs = 0, $prc = 0, $prc2 = 0, $prc5 = 0, $precno = 0;
 var $present = 0, $res = 0, $resno = 0, $seg = 0, $segno = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $t2;
 $2 = $src;
 $3 = $len;
 $4 = $tile;
 $5 = $tcp;
 $6 = $pi;
 $7 = $pack_info;
 $8 = $2;
 $c = $8;
 $9 = $1;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $cp = $11;
 $12 = $6;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $compno = $14;
 $15 = $6;
 $16 = (($15) + 28|0);
 $17 = HEAP32[$16>>2]|0;
 $resno = $17;
 $18 = $6;
 $19 = (($18) + 32|0);
 $20 = HEAP32[$19>>2]|0;
 $precno = $20;
 $21 = $6;
 $22 = (($21) + 36|0);
 $23 = HEAP32[$22>>2]|0;
 $layno = $23;
 $24 = $resno;
 $25 = $compno;
 $26 = $4;
 $27 = (($26) + 20|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25<<5)|0);
 $30 = (($29) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + (($24*124)|0)|0);
 $res = $32;
 $hd = 0;
 $bio = 0;
 $33 = $layno;
 $34 = ($33|0)==(0);
 if ($34) {
  $bandno = 0;
  while(1) {
   $35 = $bandno;
   $36 = $res;
   $37 = (($36) + 24|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($35|0)<($38|0);
   if (!($39)) {
    break;
   }
   $40 = $bandno;
   $41 = $res;
   $42 = (($41) + 28|0);
   $43 = (($42) + ($40<<5)|0);
   $band = $43;
   $44 = $precno;
   $45 = $band;
   $46 = (($45) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + (($44*36)|0)|0);
   $prc = $48;
   $49 = $band;
   $50 = (($49) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = $band;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($51) - ($53))|0;
   $55 = ($54|0)==(0);
   if ($55) {
    label = 6;
   } else {
    $56 = $band;
    $57 = (($56) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $band;
    $60 = (($59) + 4|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = (($58) - ($61))|0;
    $63 = ($62|0)==(0);
    if ($63) {
     label = 6;
    } else {
     $64 = $prc;
     $65 = (($64) + 28|0);
     $66 = HEAP32[$65>>2]|0;
     _tgt_reset($66);
     $67 = $prc;
     $68 = (($67) + 32|0);
     $69 = HEAP32[$68>>2]|0;
     _tgt_reset($69);
     $cblkno = 0;
     while(1) {
      $70 = $cblkno;
      $71 = $prc;
      $72 = (($71) + 16|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = $prc;
      $75 = (($74) + 20|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = Math_imul($73, $76)|0;
      $78 = ($70|0)<($77|0);
      if (!($78)) {
       break;
      }
      $79 = $cblkno;
      $80 = $prc;
      $81 = (($80) + 24|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = (($82) + (($79*44)|0)|0);
      $cblk = $83;
      $84 = $cblk;
      $85 = (($84) + 40|0);
      HEAP32[$85>>2] = 0;
      $86 = $cblkno;
      $87 = (($86) + 1)|0;
      $cblkno = $87;
     }
    }
   }
   if ((label|0) == 6) {
    label = 0;
   }
   $88 = $bandno;
   $89 = (($88) + 1)|0;
   $bandno = $89;
  }
 }
 $90 = $5;
 $91 = (($90) + 4|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = $92 & 2;
 $94 = ($93|0)!=(0);
 if ($94) {
  $95 = $c;
  $96 = HEAP8[$95>>0]|0;
  $97 = $96&255;
  $98 = ($97|0)!=(255);
  if ($98) {
   label = 17;
  } else {
   $99 = $c;
   $100 = (($99) + 1|0);
   $101 = HEAP8[$100>>0]|0;
   $102 = $101&255;
   $103 = ($102|0)!=(145);
   if ($103) {
    label = 17;
   } else {
    $106 = $c;
    $107 = (($106) + 6|0);
    $c = $107;
   }
  }
  if ((label|0) == 17) {
   $104 = $1;
   $105 = HEAP32[$104>>2]|0;
   (_opj_event_msg($105,2,56720,$vararg_buffer)|0);
  }
 }
 $108 = (_bio_create()|0);
 $bio = $108;
 $109 = $cp;
 $110 = (($109) + 92|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = ($111|0)==(1);
 if ($112) {
  $113 = $cp;
  $114 = (($113) + 84|0);
  $115 = HEAP32[$114>>2]|0;
  $hd = $115;
  $116 = $bio;
  $117 = $hd;
  $118 = $cp;
  $119 = (($118) + 104|0);
  $120 = HEAP32[$119>>2]|0;
  _bio_init_dec($116,$117,$120);
 } else {
  $121 = $5;
  $122 = (($121) + 5172|0);
  $123 = HEAP32[$122>>2]|0;
  $124 = ($123|0)==(1);
  if ($124) {
   $125 = $5;
   $126 = (($125) + 5164|0);
   $127 = HEAP32[$126>>2]|0;
   $hd = $127;
   $128 = $bio;
   $129 = $hd;
   $130 = $5;
   $131 = (($130) + 5180|0);
   $132 = HEAP32[$131>>2]|0;
   _bio_init_dec($128,$129,$132);
  } else {
   $133 = $c;
   $hd = $133;
   $134 = $bio;
   $135 = $hd;
   $136 = $2;
   $137 = $3;
   $138 = (($136) + ($137)|0);
   $139 = $hd;
   $140 = $138;
   $141 = $139;
   $142 = (($140) - ($141))|0;
   _bio_init_dec($134,$135,$142);
  }
 }
 $143 = $bio;
 $144 = (_bio_read($143,1)|0);
 $present = $144;
 $145 = $present;
 $146 = ($145|0)!=(0);
 if (!($146)) {
  $147 = $bio;
  (_bio_inalign($147)|0);
  $148 = $bio;
  $149 = (_bio_numbytes($148)|0);
  $150 = $hd;
  $151 = (($150) + ($149)|0);
  $hd = $151;
  $152 = $bio;
  _bio_destroy($152);
  $153 = $5;
  $154 = (($153) + 4|0);
  $155 = HEAP32[$154>>2]|0;
  $156 = $155 & 4;
  $157 = ($156|0)!=(0);
  if ($157) {
   $158 = $hd;
   $159 = HEAP8[$158>>0]|0;
   $160 = $159&255;
   $161 = ($160|0)!=(255);
   if ($161) {
    label = 30;
   } else {
    $162 = $hd;
    $163 = (($162) + 1|0);
    $164 = HEAP8[$163>>0]|0;
    $165 = $164&255;
    $166 = ($165|0)!=(146);
    if ($166) {
     label = 30;
    } else {
     $167 = $hd;
     $168 = (($167) + 2|0);
     $hd = $168;
    }
   }
   if ((label|0) == 30) {
    (_printf((56744|0),($vararg_buffer1|0))|0);
   }
  }
  $169 = $7;
  $170 = ($169|0)!=(0|0);
  if ($170) {
   $171 = $c;
   $172 = $2;
   $173 = $171;
   $174 = $172;
   $175 = (($173) - ($174))|0;
   $176 = $7;
   $177 = (($176) + 4|0);
   HEAP32[$177>>2] = $175;
  }
  $178 = $cp;
  $179 = (($178) + 92|0);
  $180 = HEAP32[$179>>2]|0;
  $181 = ($180|0)==(1);
  if ($181) {
   $182 = $cp;
   $183 = (($182) + 84|0);
   $184 = HEAP32[$183>>2]|0;
   $185 = $hd;
   $186 = $184;
   $187 = $185;
   $188 = (($186) - ($187))|0;
   $189 = $cp;
   $190 = (($189) + 104|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = (($191) + ($188))|0;
   HEAP32[$190>>2] = $192;
   $193 = $hd;
   $194 = $cp;
   $195 = (($194) + 84|0);
   HEAP32[$195>>2] = $193;
   $196 = $c;
   $197 = $2;
   $198 = $196;
   $199 = $197;
   $200 = (($198) - ($199))|0;
   $0 = $200;
   $696 = $0;
   STACKTOP = sp;return ($696|0);
  }
  $201 = $5;
  $202 = (($201) + 5172|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = ($203|0)==(1);
  if ($204) {
   $205 = $5;
   $206 = (($205) + 5164|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = $hd;
   $209 = $207;
   $210 = $208;
   $211 = (($209) - ($210))|0;
   $212 = $5;
   $213 = (($212) + 5180|0);
   $214 = HEAP32[$213>>2]|0;
   $215 = (($214) + ($211))|0;
   HEAP32[$213>>2] = $215;
   $216 = $hd;
   $217 = $5;
   $218 = (($217) + 5164|0);
   HEAP32[$218>>2] = $216;
   $219 = $c;
   $220 = $2;
   $221 = $219;
   $222 = $220;
   $223 = (($221) - ($222))|0;
   $0 = $223;
   $696 = $0;
   STACKTOP = sp;return ($696|0);
  } else {
   $224 = $hd;
   $225 = $2;
   $226 = $224;
   $227 = $225;
   $228 = (($226) - ($227))|0;
   $0 = $228;
   $696 = $0;
   STACKTOP = sp;return ($696|0);
  }
 }
 $bandno = 0;
 while(1) {
  $229 = $bandno;
  $230 = $res;
  $231 = (($230) + 24|0);
  $232 = HEAP32[$231>>2]|0;
  $233 = ($229|0)<($232|0);
  if (!($233)) {
   break;
  }
  $234 = $bandno;
  $235 = $res;
  $236 = (($235) + 28|0);
  $237 = (($236) + ($234<<5)|0);
  $band1 = $237;
  $238 = $precno;
  $239 = $band1;
  $240 = (($239) + 20|0);
  $241 = HEAP32[$240>>2]|0;
  $242 = (($241) + (($238*36)|0)|0);
  $prc2 = $242;
  $243 = $band1;
  $244 = (($243) + 8|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = $band1;
  $247 = HEAP32[$246>>2]|0;
  $248 = (($245) - ($247))|0;
  $249 = ($248|0)==(0);
  if ($249) {
   label = 44;
  } else {
   $250 = $band1;
   $251 = (($250) + 12|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = $band1;
   $254 = (($253) + 4|0);
   $255 = HEAP32[$254>>2]|0;
   $256 = (($252) - ($255))|0;
   $257 = ($256|0)==(0);
   if ($257) {
    label = 44;
   } else {
    $cblkno = 0;
    while(1) {
     $258 = $cblkno;
     $259 = $prc2;
     $260 = (($259) + 16|0);
     $261 = HEAP32[$260>>2]|0;
     $262 = $prc2;
     $263 = (($262) + 20|0);
     $264 = HEAP32[$263>>2]|0;
     $265 = Math_imul($261, $264)|0;
     $266 = ($258|0)<($265|0);
     if (!($266)) {
      break;
     }
     $267 = $cblkno;
     $268 = $prc2;
     $269 = (($268) + 24|0);
     $270 = HEAP32[$269>>2]|0;
     $271 = (($270) + (($267*44)|0)|0);
     $cblk3 = $271;
     $272 = $cblk3;
     $273 = (($272) + 40|0);
     $274 = HEAP32[$273>>2]|0;
     $275 = ($274|0)!=(0);
     if ($275) {
      $284 = $bio;
      $285 = (_bio_read($284,1)|0);
      $included = $285;
     } else {
      $276 = $bio;
      $277 = $prc2;
      $278 = (($277) + 28|0);
      $279 = HEAP32[$278>>2]|0;
      $280 = $cblkno;
      $281 = $layno;
      $282 = (($281) + 1)|0;
      $283 = (_tgt_decode($276,$279,$280,$282)|0);
      $included = $283;
     }
     $286 = $included;
     $287 = ($286|0)!=(0);
     if ($287) {
      $290 = $cblk3;
      $291 = (($290) + 40|0);
      $292 = HEAP32[$291>>2]|0;
      $293 = ($292|0)!=(0);
      if (!($293)) {
       $i = 0;
       while(1) {
        $294 = $bio;
        $295 = $prc2;
        $296 = (($295) + 32|0);
        $297 = HEAP32[$296>>2]|0;
        $298 = $cblkno;
        $299 = $i;
        $300 = (_tgt_decode($294,$297,$298,$299)|0);
        $301 = ($300|0)!=(0);
        $302 = $301 ^ 1;
        if (!($302)) {
         break;
        }
        $303 = $i;
        $304 = (($303) + 1)|0;
        $i = $304;
       }
       $305 = $i;
       $306 = (($305) - 1)|0;
       $numimsbs = $306;
       $307 = $band1;
       $308 = (($307) + 24|0);
       $309 = HEAP32[$308>>2]|0;
       $310 = $numimsbs;
       $311 = (($309) - ($310))|0;
       $312 = $cblk3;
       $313 = (($312) + 24|0);
       HEAP32[$313>>2] = $311;
       $314 = $cblk3;
       $315 = (($314) + 28|0);
       HEAP32[$315>>2] = 3;
      }
      $316 = $bio;
      $317 = (_t2_getnumpasses($316)|0);
      $318 = $cblk3;
      $319 = (($318) + 36|0);
      HEAP32[$319>>2] = $317;
      $320 = $bio;
      $321 = (_t2_getcommacode($320)|0);
      $increment = $321;
      $322 = $increment;
      $323 = $cblk3;
      $324 = (($323) + 28|0);
      $325 = HEAP32[$324>>2]|0;
      $326 = (($325) + ($322))|0;
      HEAP32[$324>>2] = $326;
      $segno = 0;
      $327 = $cblk3;
      $328 = (($327) + 40|0);
      $329 = HEAP32[$328>>2]|0;
      $330 = ($329|0)!=(0);
      if ($330) {
       $340 = $cblk3;
       $341 = (($340) + 40|0);
       $342 = HEAP32[$341>>2]|0;
       $343 = (($342) - 1)|0;
       $segno = $343;
       $344 = $segno;
       $345 = $cblk3;
       $346 = (($345) + 4|0);
       $347 = HEAP32[$346>>2]|0;
       $348 = (($347) + (($344*28)|0)|0);
       $349 = (($348) + 8|0);
       $350 = HEAP32[$349>>2]|0;
       $351 = $segno;
       $352 = $cblk3;
       $353 = (($352) + 4|0);
       $354 = HEAP32[$353>>2]|0;
       $355 = (($354) + (($351*28)|0)|0);
       $356 = (($355) + 16|0);
       $357 = HEAP32[$356>>2]|0;
       $358 = ($350|0)==($357|0);
       if ($358) {
        $359 = $segno;
        $360 = (($359) + 1)|0;
        $segno = $360;
        $361 = $cblk3;
        $362 = $segno;
        $363 = $compno;
        $364 = $5;
        $365 = (($364) + 5584|0);
        $366 = HEAP32[$365>>2]|0;
        $367 = (($366) + (($363*1076)|0)|0);
        $368 = (($367) + 16|0);
        $369 = HEAP32[$368>>2]|0;
        _t2_init_seg($361,$362,$369,0);
       }
      } else {
       $331 = $cblk3;
       $332 = $segno;
       $333 = $compno;
       $334 = $5;
       $335 = (($334) + 5584|0);
       $336 = HEAP32[$335>>2]|0;
       $337 = (($336) + (($333*1076)|0)|0);
       $338 = (($337) + 16|0);
       $339 = HEAP32[$338>>2]|0;
       _t2_init_seg($331,$332,$339,1);
      }
      $370 = $cblk3;
      $371 = (($370) + 36|0);
      $372 = HEAP32[$371>>2]|0;
      $n = $372;
      while(1) {
       $373 = $segno;
       $374 = $cblk3;
       $375 = (($374) + 4|0);
       $376 = HEAP32[$375>>2]|0;
       $377 = (($376) + (($373*28)|0)|0);
       $378 = (($377) + 16|0);
       $379 = HEAP32[$378>>2]|0;
       $380 = $segno;
       $381 = $cblk3;
       $382 = (($381) + 4|0);
       $383 = HEAP32[$382>>2]|0;
       $384 = (($383) + (($380*28)|0)|0);
       $385 = (($384) + 8|0);
       $386 = HEAP32[$385>>2]|0;
       $387 = (($379) - ($386))|0;
       $388 = $n;
       $389 = (_int_min50($387,$388)|0);
       $390 = $segno;
       $391 = $cblk3;
       $392 = (($391) + 4|0);
       $393 = HEAP32[$392>>2]|0;
       $394 = (($393) + (($390*28)|0)|0);
       $395 = (($394) + 20|0);
       HEAP32[$395>>2] = $389;
       $396 = $bio;
       $397 = $cblk3;
       $398 = (($397) + 28|0);
       $399 = HEAP32[$398>>2]|0;
       $400 = $segno;
       $401 = $cblk3;
       $402 = (($401) + 4|0);
       $403 = HEAP32[$402>>2]|0;
       $404 = (($403) + (($400*28)|0)|0);
       $405 = (($404) + 20|0);
       $406 = HEAP32[$405>>2]|0;
       $407 = (_int_floorlog251($406)|0);
       $408 = (($399) + ($407))|0;
       $409 = (_bio_read($396,$408)|0);
       $410 = $segno;
       $411 = $cblk3;
       $412 = (($411) + 4|0);
       $413 = HEAP32[$412>>2]|0;
       $414 = (($413) + (($410*28)|0)|0);
       $415 = (($414) + 24|0);
       HEAP32[$415>>2] = $409;
       $416 = $segno;
       $417 = $cblk3;
       $418 = (($417) + 4|0);
       $419 = HEAP32[$418>>2]|0;
       $420 = (($419) + (($416*28)|0)|0);
       $421 = (($420) + 20|0);
       $422 = HEAP32[$421>>2]|0;
       $423 = $n;
       $424 = (($423) - ($422))|0;
       $n = $424;
       $425 = $n;
       $426 = ($425|0)>(0);
       if ($426) {
        $427 = $segno;
        $428 = (($427) + 1)|0;
        $segno = $428;
        $429 = $cblk3;
        $430 = $segno;
        $431 = $compno;
        $432 = $5;
        $433 = (($432) + 5584|0);
        $434 = HEAP32[$433>>2]|0;
        $435 = (($434) + (($431*1076)|0)|0);
        $436 = (($435) + 16|0);
        $437 = HEAP32[$436>>2]|0;
        _t2_init_seg($429,$430,$437,0);
       }
       $438 = $n;
       $439 = ($438|0)>(0);
       if (!($439)) {
        break;
       }
      }
     } else {
      $288 = $cblk3;
      $289 = (($288) + 36|0);
      HEAP32[$289>>2] = 0;
     }
     $440 = $cblkno;
     $441 = (($440) + 1)|0;
     $cblkno = $441;
    }
   }
  }
  if ((label|0) == 44) {
   label = 0;
  }
  $442 = $bandno;
  $443 = (($442) + 1)|0;
  $bandno = $443;
 }
 $444 = $bio;
 $445 = (_bio_inalign($444)|0);
 $446 = ($445|0)!=(0);
 if ($446) {
  $447 = $bio;
  _bio_destroy($447);
  $0 = -999;
  $696 = $0;
  STACKTOP = sp;return ($696|0);
 }
 $448 = $bio;
 $449 = (_bio_numbytes($448)|0);
 $450 = $hd;
 $451 = (($450) + ($449)|0);
 $hd = $451;
 $452 = $bio;
 _bio_destroy($452);
 $453 = $5;
 $454 = (($453) + 4|0);
 $455 = HEAP32[$454>>2]|0;
 $456 = $455 & 4;
 $457 = ($456|0)!=(0);
 do {
  if ($457) {
   $458 = $hd;
   $459 = HEAP8[$458>>0]|0;
   $460 = $459&255;
   $461 = ($460|0)!=(255);
   if (!($461)) {
    $462 = $hd;
    $463 = (($462) + 1|0);
    $464 = HEAP8[$463>>0]|0;
    $465 = $464&255;
    $466 = ($465|0)!=(146);
    if (!($466)) {
     $469 = $hd;
     $470 = (($469) + 2|0);
     $hd = $470;
     break;
    }
   }
   $467 = $1;
   $468 = HEAP32[$467>>2]|0;
   (_opj_event_msg($468,1,56776,$vararg_buffer3)|0);
   $0 = -999;
   $696 = $0;
   STACKTOP = sp;return ($696|0);
  }
 } while(0);
 $471 = $7;
 $472 = ($471|0)!=(0|0);
 if ($472) {
  $473 = $hd;
  $474 = $2;
  $475 = $473;
  $476 = $474;
  $477 = (($475) - ($476))|0;
  $478 = $7;
  $479 = (($478) + 4|0);
  HEAP32[$479>>2] = $477;
 }
 $480 = $cp;
 $481 = (($480) + 92|0);
 $482 = HEAP32[$481>>2]|0;
 $483 = ($482|0)==(1);
 if ($483) {
  $484 = $cp;
  $485 = (($484) + 84|0);
  $486 = HEAP32[$485>>2]|0;
  $487 = $hd;
  $488 = $486;
  $489 = $487;
  $490 = (($488) - ($489))|0;
  $491 = $cp;
  $492 = (($491) + 104|0);
  $493 = HEAP32[$492>>2]|0;
  $494 = (($493) + ($490))|0;
  HEAP32[$492>>2] = $494;
  $495 = $hd;
  $496 = $cp;
  $497 = (($496) + 84|0);
  HEAP32[$497>>2] = $495;
 } else {
  $498 = $5;
  $499 = (($498) + 5172|0);
  $500 = HEAP32[$499>>2]|0;
  $501 = ($500|0)==(1);
  if ($501) {
   $502 = $5;
   $503 = (($502) + 5164|0);
   $504 = HEAP32[$503>>2]|0;
   $505 = $hd;
   $506 = $504;
   $507 = $505;
   $508 = (($506) - ($507))|0;
   $509 = $5;
   $510 = (($509) + 5180|0);
   $511 = HEAP32[$510>>2]|0;
   $512 = (($511) + ($508))|0;
   HEAP32[$510>>2] = $512;
   $513 = $hd;
   $514 = $5;
   $515 = (($514) + 5164|0);
   HEAP32[$515>>2] = $513;
  } else {
   $516 = $hd;
   $c = $516;
  }
 }
 $bandno = 0;
 L128: while(1) {
  $517 = $bandno;
  $518 = $res;
  $519 = (($518) + 24|0);
  $520 = HEAP32[$519>>2]|0;
  $521 = ($517|0)<($520|0);
  if (!($521)) {
   label = 115;
   break;
  }
  $522 = $bandno;
  $523 = $res;
  $524 = (($523) + 28|0);
  $525 = (($524) + ($522<<5)|0);
  $band4 = $525;
  $526 = $precno;
  $527 = $band4;
  $528 = (($527) + 20|0);
  $529 = HEAP32[$528>>2]|0;
  $530 = (($529) + (($526*36)|0)|0);
  $prc5 = $530;
  $531 = $band4;
  $532 = (($531) + 8|0);
  $533 = HEAP32[$532>>2]|0;
  $534 = $band4;
  $535 = HEAP32[$534>>2]|0;
  $536 = (($533) - ($535))|0;
  $537 = ($536|0)==(0);
  if ($537) {
   label = 92;
  } else {
   $538 = $band4;
   $539 = (($538) + 12|0);
   $540 = HEAP32[$539>>2]|0;
   $541 = $band4;
   $542 = (($541) + 4|0);
   $543 = HEAP32[$542>>2]|0;
   $544 = (($540) - ($543))|0;
   $545 = ($544|0)==(0);
   if ($545) {
    label = 92;
   } else {
    $cblkno = 0;
    while(1) {
     $546 = $cblkno;
     $547 = $prc5;
     $548 = (($547) + 16|0);
     $549 = HEAP32[$548>>2]|0;
     $550 = $prc5;
     $551 = (($550) + 20|0);
     $552 = HEAP32[$551>>2]|0;
     $553 = Math_imul($549, $552)|0;
     $554 = ($546|0)<($553|0);
     if (!($554)) {
      break;
     }
     $555 = $cblkno;
     $556 = $prc5;
     $557 = (($556) + 24|0);
     $558 = HEAP32[$557>>2]|0;
     $559 = (($558) + (($555*44)|0)|0);
     $cblk6 = $559;
     $seg = 0;
     $560 = $cblk6;
     $561 = (($560) + 36|0);
     $562 = HEAP32[$561>>2]|0;
     $563 = ($562|0)!=(0);
     if ($563) {
      $564 = $cblk6;
      $565 = (($564) + 40|0);
      $566 = HEAP32[$565>>2]|0;
      $567 = ($566|0)!=(0);
      if ($567) {
       $577 = $cblk6;
       $578 = (($577) + 40|0);
       $579 = HEAP32[$578>>2]|0;
       $580 = (($579) - 1)|0;
       $581 = $cblk6;
       $582 = (($581) + 4|0);
       $583 = HEAP32[$582>>2]|0;
       $584 = (($583) + (($580*28)|0)|0);
       $seg = $584;
       $585 = $seg;
       $586 = (($585) + 8|0);
       $587 = HEAP32[$586>>2]|0;
       $588 = $seg;
       $589 = (($588) + 16|0);
       $590 = HEAP32[$589>>2]|0;
       $591 = ($587|0)==($590|0);
       if ($591) {
        $592 = $seg;
        $593 = (($592) + 28|0);
        $seg = $593;
        $594 = $cblk6;
        $595 = (($594) + 40|0);
        $596 = HEAP32[$595>>2]|0;
        $597 = (($596) + 1)|0;
        HEAP32[$595>>2] = $597;
       }
      } else {
       $568 = $cblk6;
       $569 = (($568) + 4|0);
       $570 = HEAP32[$569>>2]|0;
       $seg = $570;
       $571 = $cblk6;
       $572 = (($571) + 40|0);
       $573 = HEAP32[$572>>2]|0;
       $574 = (($573) + 1)|0;
       HEAP32[$572>>2] = $574;
       $575 = $cblk6;
       $576 = (($575) + 32|0);
       HEAP32[$576>>2] = 0;
      }
      while(1) {
       $598 = $c;
       $599 = $seg;
       $600 = (($599) + 24|0);
       $601 = HEAP32[$600>>2]|0;
       $602 = (($598) + ($601)|0);
       $603 = $2;
       $604 = $3;
       $605 = (($603) + ($604)|0);
       $606 = ($602>>>0)>($605>>>0);
       if ($606) {
        label = 104;
        break L128;
       }
       $607 = $cblk6;
       $608 = HEAP32[$607>>2]|0;
       $609 = $cblk6;
       $610 = (($609) + 32|0);
       $611 = HEAP32[$610>>2]|0;
       $612 = $seg;
       $613 = (($612) + 24|0);
       $614 = HEAP32[$613>>2]|0;
       $615 = (($611) + ($614))|0;
       $616 = $615<<2;
       $617 = (_realloc($608,$616)|0);
       $618 = $cblk6;
       HEAP32[$618>>2] = $617;
       $619 = $cblk6;
       $620 = HEAP32[$619>>2]|0;
       $621 = $cblk6;
       $622 = (($621) + 32|0);
       $623 = HEAP32[$622>>2]|0;
       $624 = (($620) + ($623)|0);
       $625 = $c;
       $626 = $seg;
       $627 = (($626) + 24|0);
       $628 = HEAP32[$627>>2]|0;
       _memcpy(($624|0),($625|0),($628|0))|0;
       $629 = $seg;
       $630 = (($629) + 8|0);
       $631 = HEAP32[$630>>2]|0;
       $632 = ($631|0)==(0);
       if ($632) {
        $633 = $cblk6;
        $634 = $seg;
        HEAP32[$634>>2] = $633;
        $635 = $cblk6;
        $636 = (($635) + 32|0);
        $637 = HEAP32[$636>>2]|0;
        $638 = $seg;
        $639 = (($638) + 4|0);
        HEAP32[$639>>2] = $637;
       }
       $640 = $seg;
       $641 = (($640) + 24|0);
       $642 = HEAP32[$641>>2]|0;
       $643 = $c;
       $644 = (($643) + ($642)|0);
       $c = $644;
       $645 = $seg;
       $646 = (($645) + 24|0);
       $647 = HEAP32[$646>>2]|0;
       $648 = $cblk6;
       $649 = (($648) + 32|0);
       $650 = HEAP32[$649>>2]|0;
       $651 = (($650) + ($647))|0;
       HEAP32[$649>>2] = $651;
       $652 = $seg;
       $653 = (($652) + 24|0);
       $654 = HEAP32[$653>>2]|0;
       $655 = $seg;
       $656 = (($655) + 12|0);
       $657 = HEAP32[$656>>2]|0;
       $658 = (($657) + ($654))|0;
       HEAP32[$656>>2] = $658;
       $659 = $seg;
       $660 = (($659) + 20|0);
       $661 = HEAP32[$660>>2]|0;
       $662 = $seg;
       $663 = (($662) + 8|0);
       $664 = HEAP32[$663>>2]|0;
       $665 = (($664) + ($661))|0;
       HEAP32[$663>>2] = $665;
       $666 = $seg;
       $667 = (($666) + 20|0);
       $668 = HEAP32[$667>>2]|0;
       $669 = $cblk6;
       $670 = (($669) + 36|0);
       $671 = HEAP32[$670>>2]|0;
       $672 = (($671) - ($668))|0;
       HEAP32[$670>>2] = $672;
       $673 = $cblk6;
       $674 = (($673) + 36|0);
       $675 = HEAP32[$674>>2]|0;
       $676 = ($675|0)>(0);
       if ($676) {
        $677 = $seg;
        $678 = (($677) + 28|0);
        $seg = $678;
        $679 = $cblk6;
        $680 = (($679) + 40|0);
        $681 = HEAP32[$680>>2]|0;
        $682 = (($681) + 1)|0;
        HEAP32[$680>>2] = $682;
       }
       $683 = $cblk6;
       $684 = (($683) + 36|0);
       $685 = HEAP32[$684>>2]|0;
       $686 = ($685|0)>(0);
       if (!($686)) {
        break;
       }
      }
     } else {
     }
     $687 = $cblkno;
     $688 = (($687) + 1)|0;
     $cblkno = $688;
    }
   }
  }
  if ((label|0) == 92) {
   label = 0;
  }
  $689 = $bandno;
  $690 = (($689) + 1)|0;
  $bandno = $690;
 }
 if ((label|0) == 104) {
  $0 = -999;
  $696 = $0;
  STACKTOP = sp;return ($696|0);
 }
 else if ((label|0) == 115) {
  $691 = $c;
  $692 = $2;
  $693 = $691;
  $694 = $692;
  $695 = (($693) - ($694))|0;
  $0 = $695;
  $696 = $0;
  STACKTOP = sp;return ($696|0);
 }
 return 0|0;
}
function _int_max49($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)>($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _t2_create($cinfo,$image,$cp) {
 $cinfo = $cinfo|0;
 $image = $image|0;
 $cp = $cp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $t2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $cinfo;
 $2 = $image;
 $3 = $cp;
 $4 = (_malloc(12)|0);
 $t2 = $4;
 $5 = $t2;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = $1;
  $8 = $t2;
  HEAP32[$8>>2] = $7;
  $9 = $2;
  $10 = $t2;
  $11 = (($10) + 4|0);
  HEAP32[$11>>2] = $9;
  $12 = $3;
  $13 = $t2;
  $14 = (($13) + 8|0);
  HEAP32[$14>>2] = $12;
  $15 = $t2;
  $0 = $15;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 } else {
  $0 = 0;
  $16 = $0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _t2_destroy($t2) {
 $t2 = $t2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $t2;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = $0;
  _free($3);
 }
 STACKTOP = sp;return;
}
function _t2_getnumpasses($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $bio;
 $2 = $1;
 $3 = (_bio_read($2,1)|0);
 $4 = ($3|0)!=(0);
 do {
  if ($4) {
   $5 = $1;
   $6 = (_bio_read($5,1)|0);
   $7 = ($6|0)!=(0);
   if (!($7)) {
    $0 = 2;
    break;
   }
   $8 = $1;
   $9 = (_bio_read($8,2)|0);
   $n = $9;
   $10 = ($9|0)!=(3);
   if ($10) {
    $11 = $n;
    $12 = (3 + ($11))|0;
    $0 = $12;
    break;
   }
   $13 = $1;
   $14 = (_bio_read($13,5)|0);
   $n = $14;
   $15 = ($14|0)!=(31);
   if ($15) {
    $16 = $n;
    $17 = (6 + ($16))|0;
    $0 = $17;
    break;
   } else {
    $18 = $1;
    $19 = (_bio_read($18,7)|0);
    $20 = (37 + ($19))|0;
    $0 = $20;
    break;
   }
  } else {
   $0 = 1;
  }
 } while(0);
 $21 = $0;
 STACKTOP = sp;return ($21|0);
}
function _t2_getcommacode($bio) {
 $bio = $bio|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $bio;
 $n = 0;
 while(1) {
  $1 = $0;
  $2 = (_bio_read($1,1)|0);
  $3 = ($2|0)!=(0);
  if (!($3)) {
   break;
  }
  $4 = $n;
  $5 = (($4) + 1)|0;
  $n = $5;
 }
 $6 = $n;
 STACKTOP = sp;return ($6|0);
}
function _t2_init_seg($cblk,$index,$cblksty,$first) {
 $cblk = $cblk|0;
 $index = $index|0;
 $cblksty = $cblksty|0;
 $first = $first|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $seg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $cblk;
 $1 = $index;
 $2 = $cblksty;
 $3 = $first;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $1;
 $8 = (($7) + 1)|0;
 $9 = ($8*28)|0;
 $10 = (_realloc($6,$9)|0);
 $11 = $0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = $1;
 $14 = $0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + (($13*28)|0)|0);
 $seg = $17;
 $18 = $seg;
 HEAP32[$18>>2] = 0;
 $19 = $seg;
 $20 = (($19) + 4|0);
 HEAP32[$20>>2] = 0;
 $21 = $seg;
 $22 = (($21) + 8|0);
 HEAP32[$22>>2] = 0;
 $23 = $seg;
 $24 = (($23) + 12|0);
 HEAP32[$24>>2] = 0;
 $25 = $2;
 $26 = $25 & 4;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = $seg;
  $29 = (($28) + 16|0);
  HEAP32[$29>>2] = 1;
  STACKTOP = sp;return;
 }
 $30 = $2;
 $31 = $30 & 1;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = $3;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = $seg;
   $36 = (($35) + 16|0);
   HEAP32[$36>>2] = 10;
  } else {
   $37 = $seg;
   $38 = (($37) + -28|0);
   $39 = (($38) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)==(1);
   if ($41) {
    $48 = 1;
   } else {
    $42 = $seg;
    $43 = (($42) + -28|0);
    $44 = (($43) + 16|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)==(10);
    $48 = $46;
   }
   $47 = $48 ? 2 : 1;
   $49 = $seg;
   $50 = (($49) + 16|0);
   HEAP32[$50>>2] = $47;
  }
 } else {
  $51 = $seg;
  $52 = (($51) + 16|0);
  HEAP32[$52>>2] = 109;
 }
 STACKTOP = sp;return;
}
function _int_min50($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)<($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _int_floorlog251($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $l = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $l = 0;
 while(1) {
  $1 = $0;
  $2 = ($1|0)>(1);
  if (!($2)) {
   break;
  }
  $3 = $0;
  $4 = $3 >> 1;
  $0 = $4;
  $5 = $l;
  $6 = (($5) + 1)|0;
  $l = $6;
 }
 $7 = $l;
 STACKTOP = sp;return ($7|0);
}
function _tcd_create($cinfo) {
 $cinfo = $cinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tcd = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $cinfo;
 $2 = (_malloc(56)|0);
 $tcd = $2;
 $3 = $tcd;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = $1;
   $6 = $tcd;
   $7 = (($6) + 20|0);
   HEAP32[$7>>2] = $5;
   $8 = (_malloc(12)|0);
   $9 = $tcd;
   $10 = (($9) + 24|0);
   HEAP32[$10>>2] = $8;
   $11 = $tcd;
   $12 = (($11) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $16 = $tcd;
    $0 = $16;
    break;
   } else {
    $15 = $tcd;
    _free($15);
    $0 = 0;
    break;
   }
  } else {
   $0 = 0;
  }
 } while(0);
 $17 = $0;
 STACKTOP = sp;return ($17|0);
}
function _tcd_destroy($tcd) {
 $tcd = $tcd|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $tcd;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = $0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 _free($5);
 $6 = $0;
 _free($6);
 STACKTOP = sp;return;
}
function _int_max76($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)>($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _int_min77($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = ($2|0)<($3|0);
 if ($4) {
  $5 = $0;
  $7 = $5;
 } else {
  $6 = $1;
  $7 = $6;
 }
 STACKTOP = sp;return ($7|0);
}
function _int_ceildiv78($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = (($2) + ($3))|0;
 $5 = (($4) - 1)|0;
 $6 = $1;
 $7 = (($5|0) / ($6|0))&-1;
 STACKTOP = sp;return ($7|0);
}
function _int_ceildivpow279($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = 1 << $3;
 $5 = (($2) + ($4))|0;
 $6 = (($5) - 1)|0;
 $7 = $1;
 $8 = $6 >> $7;
 STACKTOP = sp;return ($8|0);
}
function _int_floordivpow280($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = $1;
 $4 = $2 >> $3;
 STACKTOP = sp;return ($4|0);
}
function _tcd_malloc_decode($tcd,$image,$cp) {
 $tcd = $tcd|0;
 $image = $image|0;
 $cp = $cp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $h = 0, $i = 0, $j = 0, $p = 0, $q = 0, $tile = 0;
 var $tile1 = 0, $tilec = 0, $tileno = 0, $w = 0, $x0 = 0, $x1 = 0, $y0 = 0, $y1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $tcd;
 $1 = $image;
 $2 = $cp;
 $x0 = 0;
 $y0 = 0;
 $x1 = 0;
 $y1 = 0;
 $3 = $1;
 $4 = $0;
 $5 = (($4) + 28|0);
 HEAP32[$5>>2] = $3;
 $6 = $2;
 $7 = (($6) + 68|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $0;
 $10 = (($9) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$11>>2] = $8;
 $12 = $2;
 $13 = (($12) + 72|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $0;
 $16 = (($15) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = $14;
 $19 = $2;
 $20 = (($19) + 68|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $2;
 $23 = (($22) + 72|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = Math_imul($21, $24)|0;
 $26 = ($25*848)|0;
 $27 = (_malloc($26)|0);
 $28 = $0;
 $29 = (($28) + 24|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 8|0);
 HEAP32[$31>>2] = $27;
 $j = 0;
 while(1) {
  $32 = $j;
  $33 = $2;
  $34 = (($33) + 80|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($32|0)<($35|0);
  if (!($36)) {
   break;
  }
  $37 = $j;
  $38 = $2;
  $39 = (($38) + 76|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + ($37<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $tileno = $42;
  $43 = $tileno;
  $44 = $2;
  $45 = (($44) + 76|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + ($43<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $0;
  $50 = (($49) + 24|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + (($48*848)|0)|0);
  $tile = $54;
  $55 = $1;
  $56 = (($55) + 16|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $tile;
  $59 = (($58) + 16|0);
  HEAP32[$59>>2] = $57;
  $60 = $1;
  $61 = (($60) + 16|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (_calloc($62,32)|0);
  $64 = $tile;
  $65 = (($64) + 20|0);
  HEAP32[$65>>2] = $63;
  $66 = $j;
  $67 = (($66) + 1)|0;
  $j = $67;
 }
 $i = 0;
 while(1) {
  $68 = $i;
  $69 = $1;
  $70 = (($69) + 16|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = ($68|0)<($71|0);
  if (!($72)) {
   break;
  }
  $j = 0;
  while(1) {
   $73 = $j;
   $74 = $2;
   $75 = (($74) + 80|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = ($73|0)<($76|0);
   if (!($77)) {
    break;
   }
   $78 = $j;
   $79 = $2;
   $80 = (($79) + 76|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = (($81) + ($78<<2)|0);
   $83 = HEAP32[$82>>2]|0;
   $tileno = $83;
   $84 = $tileno;
   $85 = $2;
   $86 = (($85) + 76|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (($87) + ($84<<2)|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = $0;
   $91 = (($90) + 24|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = (($92) + 8|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + (($89*848)|0)|0);
   $tile1 = $95;
   $96 = $i;
   $97 = $tile1;
   $98 = (($97) + 20|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = (($99) + ($96<<5)|0);
   $tilec = $100;
   $101 = $tileno;
   $102 = $2;
   $103 = (($102) + 68|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = (($101|0) % ($104|0))&-1;
   $p = $105;
   $106 = $tileno;
   $107 = $2;
   $108 = (($107) + 68|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($106|0) / ($109|0))&-1;
   $q = $110;
   $111 = $2;
   $112 = (($111) + 48|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = $p;
   $115 = $2;
   $116 = (($115) + 56|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = Math_imul($114, $117)|0;
   $119 = (($113) + ($118))|0;
   $120 = $1;
   $121 = HEAP32[$120>>2]|0;
   $122 = (_int_max76($119,$121)|0);
   $123 = $tile1;
   HEAP32[$123>>2] = $122;
   $124 = $2;
   $125 = (($124) + 52|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = $q;
   $128 = $2;
   $129 = (($128) + 60|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = Math_imul($127, $130)|0;
   $132 = (($126) + ($131))|0;
   $133 = $1;
   $134 = (($133) + 4|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = (_int_max76($132,$135)|0);
   $137 = $tile1;
   $138 = (($137) + 4|0);
   HEAP32[$138>>2] = $136;
   $139 = $2;
   $140 = (($139) + 48|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = $p;
   $143 = (($142) + 1)|0;
   $144 = $2;
   $145 = (($144) + 56|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = Math_imul($143, $146)|0;
   $148 = (($141) + ($147))|0;
   $149 = $1;
   $150 = (($149) + 8|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = (_int_min77($148,$151)|0);
   $153 = $tile1;
   $154 = (($153) + 8|0);
   HEAP32[$154>>2] = $152;
   $155 = $2;
   $156 = (($155) + 52|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = $q;
   $159 = (($158) + 1)|0;
   $160 = $2;
   $161 = (($160) + 60|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = Math_imul($159, $162)|0;
   $164 = (($157) + ($163))|0;
   $165 = $1;
   $166 = (($165) + 12|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = (_int_min77($164,$167)|0);
   $169 = $tile1;
   $170 = (($169) + 12|0);
   HEAP32[$170>>2] = $168;
   $171 = $tile1;
   $172 = HEAP32[$171>>2]|0;
   $173 = $i;
   $174 = $1;
   $175 = (($174) + 24|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = (($176) + (($173*48)|0)|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = (_int_ceildiv78($172,$178)|0);
   $180 = $tilec;
   HEAP32[$180>>2] = $179;
   $181 = $tile1;
   $182 = (($181) + 4|0);
   $183 = HEAP32[$182>>2]|0;
   $184 = $i;
   $185 = $1;
   $186 = (($185) + 24|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = (($187) + (($184*48)|0)|0);
   $189 = (($188) + 4|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = (_int_ceildiv78($183,$190)|0);
   $192 = $tilec;
   $193 = (($192) + 4|0);
   HEAP32[$193>>2] = $191;
   $194 = $tile1;
   $195 = (($194) + 8|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = $i;
   $198 = $1;
   $199 = (($198) + 24|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = (($200) + (($197*48)|0)|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (_int_ceildiv78($196,$202)|0);
   $204 = $tilec;
   $205 = (($204) + 8|0);
   HEAP32[$205>>2] = $203;
   $206 = $tile1;
   $207 = (($206) + 12|0);
   $208 = HEAP32[$207>>2]|0;
   $209 = $i;
   $210 = $1;
   $211 = (($210) + 24|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = (($212) + (($209*48)|0)|0);
   $214 = (($213) + 4|0);
   $215 = HEAP32[$214>>2]|0;
   $216 = (_int_ceildiv78($208,$215)|0);
   $217 = $tilec;
   $218 = (($217) + 12|0);
   HEAP32[$218>>2] = $216;
   $219 = $j;
   $220 = ($219|0)==(0);
   if ($220) {
    $221 = $tilec;
    $222 = HEAP32[$221>>2]|0;
    $227 = $222;
   } else {
    $223 = $x0;
    $224 = $tilec;
    $225 = HEAP32[$224>>2]|0;
    $226 = (_int_min77($223,$225)|0);
    $227 = $226;
   }
   $x0 = $227;
   $228 = $j;
   $229 = ($228|0)==(0);
   if ($229) {
    $230 = $tilec;
    $231 = (($230) + 4|0);
    $232 = HEAP32[$231>>2]|0;
    $237 = $232;
   } else {
    $233 = $y0;
    $234 = $tilec;
    $235 = HEAP32[$234>>2]|0;
    $236 = (_int_min77($233,$235)|0);
    $237 = $236;
   }
   $y0 = $237;
   $238 = $j;
   $239 = ($238|0)==(0);
   if ($239) {
    $240 = $tilec;
    $241 = (($240) + 8|0);
    $242 = HEAP32[$241>>2]|0;
    $248 = $242;
   } else {
    $243 = $x1;
    $244 = $tilec;
    $245 = (($244) + 8|0);
    $246 = HEAP32[$245>>2]|0;
    $247 = (_int_max76($243,$246)|0);
    $248 = $247;
   }
   $x1 = $248;
   $249 = $j;
   $250 = ($249|0)==(0);
   if ($250) {
    $251 = $tilec;
    $252 = (($251) + 12|0);
    $253 = HEAP32[$252>>2]|0;
    $259 = $253;
   } else {
    $254 = $y1;
    $255 = $tilec;
    $256 = (($255) + 12|0);
    $257 = HEAP32[$256>>2]|0;
    $258 = (_int_max76($254,$257)|0);
    $259 = $258;
   }
   $y1 = $259;
   $260 = $j;
   $261 = (($260) + 1)|0;
   $j = $261;
  }
  $262 = $x1;
  $263 = $x0;
  $264 = (($262) - ($263))|0;
  $265 = $i;
  $266 = $1;
  $267 = (($266) + 24|0);
  $268 = HEAP32[$267>>2]|0;
  $269 = (($268) + (($265*48)|0)|0);
  $270 = (($269) + 40|0);
  $271 = HEAP32[$270>>2]|0;
  $272 = (_int_ceildivpow279($264,$271)|0);
  $w = $272;
  $273 = $y1;
  $274 = $y0;
  $275 = (($273) - ($274))|0;
  $276 = $i;
  $277 = $1;
  $278 = (($277) + 24|0);
  $279 = HEAP32[$278>>2]|0;
  $280 = (($279) + (($276*48)|0)|0);
  $281 = (($280) + 40|0);
  $282 = HEAP32[$281>>2]|0;
  $283 = (_int_ceildivpow279($275,$282)|0);
  $h = $283;
  $284 = $w;
  $285 = $i;
  $286 = $1;
  $287 = (($286) + 24|0);
  $288 = HEAP32[$287>>2]|0;
  $289 = (($288) + (($285*48)|0)|0);
  $290 = (($289) + 8|0);
  HEAP32[$290>>2] = $284;
  $291 = $h;
  $292 = $i;
  $293 = $1;
  $294 = (($293) + 24|0);
  $295 = HEAP32[$294>>2]|0;
  $296 = (($295) + (($292*48)|0)|0);
  $297 = (($296) + 12|0);
  HEAP32[$297>>2] = $291;
  $298 = $x0;
  $299 = $i;
  $300 = $1;
  $301 = (($300) + 24|0);
  $302 = HEAP32[$301>>2]|0;
  $303 = (($302) + (($299*48)|0)|0);
  $304 = (($303) + 16|0);
  HEAP32[$304>>2] = $298;
  $305 = $y0;
  $306 = $i;
  $307 = $1;
  $308 = (($307) + 24|0);
  $309 = HEAP32[$308>>2]|0;
  $310 = (($309) + (($306*48)|0)|0);
  $311 = (($310) + 20|0);
  HEAP32[$311>>2] = $305;
  $312 = $i;
  $313 = (($312) + 1)|0;
  $i = $313;
 }
 STACKTOP = sp;return;
}
function _tcd_malloc_decode_tile($tcd,$image,$cp,$tileno,$cstr_info) {
 $tcd = $tcd|0;
 $image = $image|0;
 $cp = $cp|0;
 $tileno = $tileno|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0.0, $412 = 0.0, $413 = 0.0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0.0, $419 = 0.0, $42 = 0, $420 = 0.0, $421 = 0.0;
 var $422 = 0.0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $band = 0, $bandno = 0, $brcbgxend = 0, $brcbgyend = 0, $brcblkxend = 0, $brcblkyend = 0, $brprcxend = 0, $brprcyend = 0, $cbgheightexpn = 0, $cbgwidthexpn = 0, $cbgxend = 0, $cbgxstart = 0, $cbgyend = 0;
 var $cbgystart = 0, $cblk = 0, $cblkheightexpn = 0, $cblkno = 0, $cblkwidthexpn = 0, $cblkxend = 0, $cblkxstart = 0, $cblkyend = 0, $cblkystart = 0, $compno = 0, $gain = 0, $levelno = 0, $numbps = 0, $pdx = 0, $pdy = 0, $prc = 0, $precno = 0, $res = 0, $resno = 0, $ss = 0;
 var $tccp = 0, $tcp = 0, $tile = 0, $tilec = 0, $tlcbgxstart = 0, $tlcbgystart = 0, $tlcblkxstart = 0, $tlcblkystart = 0, $tlprcxstart = 0, $tlprcystart = 0, $x0b = 0, $y0b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $0 = $tcd;
 $1 = $image;
 $2 = $cp;
 $3 = $tileno;
 $4 = $cstr_info;
 $5 = $2;
 $6 = $0;
 $7 = (($6) + 32|0);
 HEAP32[$7>>2] = $5;
 $8 = $3;
 $9 = $2;
 $10 = (($9) + 76|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($8<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $2;
 $15 = (($14) + 108|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + (($13*5588)|0)|0);
 $tcp = $17;
 $18 = $3;
 $19 = $2;
 $20 = (($19) + 76|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + ($18<<2)|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = (($24) + 24|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + 8|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + (($23*848)|0)|0);
 $tile = $29;
 $30 = $3;
 $31 = $2;
 $32 = (($31) + 76|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + ($30<<2)|0);
 $35 = HEAP32[$34>>2]|0;
 $3 = $35;
 $compno = 0;
 while(1) {
  $36 = $compno;
  $37 = $tile;
  $38 = (($37) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($36|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = $compno;
  $42 = $tcp;
  $43 = (($42) + 5584|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($41*1076)|0)|0);
  $tccp = $45;
  $46 = $compno;
  $47 = $tile;
  $48 = (($47) + 20|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($46<<5)|0);
  $tilec = $50;
  $51 = $tile;
  $52 = HEAP32[$51>>2]|0;
  $53 = $compno;
  $54 = $1;
  $55 = (($54) + 24|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + (($53*48)|0)|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (_int_ceildiv78($52,$58)|0);
  $60 = $tilec;
  HEAP32[$60>>2] = $59;
  $61 = $tile;
  $62 = (($61) + 4|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $compno;
  $65 = $1;
  $66 = (($65) + 24|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + (($64*48)|0)|0);
  $69 = (($68) + 4|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (_int_ceildiv78($63,$70)|0);
  $72 = $tilec;
  $73 = (($72) + 4|0);
  HEAP32[$73>>2] = $71;
  $74 = $tile;
  $75 = (($74) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $compno;
  $78 = $1;
  $79 = (($78) + 24|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = (($80) + (($77*48)|0)|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (_int_ceildiv78($76,$82)|0);
  $84 = $tilec;
  $85 = (($84) + 8|0);
  HEAP32[$85>>2] = $83;
  $86 = $tile;
  $87 = (($86) + 12|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = $compno;
  $90 = $1;
  $91 = (($90) + 24|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + (($89*48)|0)|0);
  $94 = (($93) + 4|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (_int_ceildiv78($88,$95)|0);
  $97 = $tilec;
  $98 = (($97) + 12|0);
  HEAP32[$98>>2] = $96;
  $99 = $tccp;
  $100 = (($99) + 4|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = $tilec;
  $103 = (($102) + 16|0);
  HEAP32[$103>>2] = $101;
  $104 = $tilec;
  $105 = (($104) + 16|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = ($106*124)|0;
  $108 = (_malloc($107)|0);
  $109 = $tilec;
  $110 = (($109) + 20|0);
  HEAP32[$110>>2] = $108;
  $resno = 0;
  while(1) {
   $111 = $resno;
   $112 = $tilec;
   $113 = (($112) + 16|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = ($111|0)<($114|0);
   if (!($115)) {
    break;
   }
   $116 = $tilec;
   $117 = (($116) + 16|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (($118) - 1)|0;
   $120 = $resno;
   $121 = (($119) - ($120))|0;
   $levelno = $121;
   $122 = $resno;
   $123 = $tilec;
   $124 = (($123) + 20|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = (($125) + (($122*124)|0)|0);
   $res = $126;
   $127 = $tilec;
   $128 = HEAP32[$127>>2]|0;
   $129 = $levelno;
   $130 = (_int_ceildivpow279($128,$129)|0);
   $131 = $res;
   HEAP32[$131>>2] = $130;
   $132 = $tilec;
   $133 = (($132) + 4|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = $levelno;
   $136 = (_int_ceildivpow279($134,$135)|0);
   $137 = $res;
   $138 = (($137) + 4|0);
   HEAP32[$138>>2] = $136;
   $139 = $tilec;
   $140 = (($139) + 8|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = $levelno;
   $143 = (_int_ceildivpow279($141,$142)|0);
   $144 = $res;
   $145 = (($144) + 8|0);
   HEAP32[$145>>2] = $143;
   $146 = $tilec;
   $147 = (($146) + 12|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = $levelno;
   $150 = (_int_ceildivpow279($148,$149)|0);
   $151 = $res;
   $152 = (($151) + 12|0);
   HEAP32[$152>>2] = $150;
   $153 = $resno;
   $154 = ($153|0)==(0);
   $155 = $154 ? 1 : 3;
   $156 = $res;
   $157 = (($156) + 24|0);
   HEAP32[$157>>2] = $155;
   $158 = $tccp;
   $159 = HEAP32[$158>>2]|0;
   $160 = $159 & 1;
   $161 = ($160|0)!=(0);
   if ($161) {
    $162 = $resno;
    $163 = $tccp;
    $164 = (($163) + 812|0);
    $165 = (($164) + ($162<<2)|0);
    $166 = HEAP32[$165>>2]|0;
    $pdx = $166;
    $167 = $resno;
    $168 = $tccp;
    $169 = (($168) + 944|0);
    $170 = (($169) + ($167<<2)|0);
    $171 = HEAP32[$170>>2]|0;
    $pdy = $171;
   } else {
    $pdx = 15;
    $pdy = 15;
   }
   $172 = $res;
   $173 = HEAP32[$172>>2]|0;
   $174 = $pdx;
   $175 = (_int_floordivpow280($173,$174)|0);
   $176 = $pdx;
   $177 = $175 << $176;
   $tlprcxstart = $177;
   $178 = $res;
   $179 = (($178) + 4|0);
   $180 = HEAP32[$179>>2]|0;
   $181 = $pdy;
   $182 = (_int_floordivpow280($180,$181)|0);
   $183 = $pdy;
   $184 = $182 << $183;
   $tlprcystart = $184;
   $185 = $res;
   $186 = (($185) + 8|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = $pdx;
   $189 = (_int_ceildivpow279($187,$188)|0);
   $190 = $pdx;
   $191 = $189 << $190;
   $brprcxend = $191;
   $192 = $res;
   $193 = (($192) + 12|0);
   $194 = HEAP32[$193>>2]|0;
   $195 = $pdy;
   $196 = (_int_ceildivpow279($194,$195)|0);
   $197 = $pdy;
   $198 = $196 << $197;
   $brprcyend = $198;
   $199 = $res;
   $200 = HEAP32[$199>>2]|0;
   $201 = $res;
   $202 = (($201) + 8|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = ($200|0)==($203|0);
   if ($204) {
    $212 = 0;
   } else {
    $205 = $brprcxend;
    $206 = $tlprcxstart;
    $207 = (($205) - ($206))|0;
    $208 = $pdx;
    $209 = $207 >> $208;
    $212 = $209;
   }
   $210 = $res;
   $211 = (($210) + 16|0);
   HEAP32[$211>>2] = $212;
   $213 = $res;
   $214 = (($213) + 4|0);
   $215 = HEAP32[$214>>2]|0;
   $216 = $res;
   $217 = (($216) + 12|0);
   $218 = HEAP32[$217>>2]|0;
   $219 = ($215|0)==($218|0);
   if ($219) {
    $227 = 0;
   } else {
    $220 = $brprcyend;
    $221 = $tlprcystart;
    $222 = (($220) - ($221))|0;
    $223 = $pdy;
    $224 = $222 >> $223;
    $227 = $224;
   }
   $225 = $res;
   $226 = (($225) + 20|0);
   HEAP32[$226>>2] = $227;
   $228 = $resno;
   $229 = ($228|0)==(0);
   if ($229) {
    $230 = $tlprcxstart;
    $tlcbgxstart = $230;
    $231 = $tlprcystart;
    $tlcbgystart = $231;
    $232 = $brprcxend;
    $brcbgxend = $232;
    $233 = $brprcyend;
    $brcbgyend = $233;
    $234 = $pdx;
    $cbgwidthexpn = $234;
    $235 = $pdy;
    $cbgheightexpn = $235;
   } else {
    $236 = $tlprcxstart;
    $237 = (_int_ceildivpow279($236,1)|0);
    $tlcbgxstart = $237;
    $238 = $tlprcystart;
    $239 = (_int_ceildivpow279($238,1)|0);
    $tlcbgystart = $239;
    $240 = $brprcxend;
    $241 = (_int_ceildivpow279($240,1)|0);
    $brcbgxend = $241;
    $242 = $brprcyend;
    $243 = (_int_ceildivpow279($242,1)|0);
    $brcbgyend = $243;
    $244 = $pdx;
    $245 = (($244) - 1)|0;
    $cbgwidthexpn = $245;
    $246 = $pdy;
    $247 = (($246) - 1)|0;
    $cbgheightexpn = $247;
   }
   $248 = $tccp;
   $249 = (($248) + 8|0);
   $250 = HEAP32[$249>>2]|0;
   $251 = $cbgwidthexpn;
   $252 = (_int_min77($250,$251)|0);
   $cblkwidthexpn = $252;
   $253 = $tccp;
   $254 = (($253) + 12|0);
   $255 = HEAP32[$254>>2]|0;
   $256 = $cbgheightexpn;
   $257 = (_int_min77($255,$256)|0);
   $cblkheightexpn = $257;
   $bandno = 0;
   while(1) {
    $258 = $bandno;
    $259 = $res;
    $260 = (($259) + 24|0);
    $261 = HEAP32[$260>>2]|0;
    $262 = ($258|0)<($261|0);
    if (!($262)) {
     break;
    }
    $ss = 0;
    $263 = $bandno;
    $264 = $res;
    $265 = (($264) + 28|0);
    $266 = (($265) + ($263<<5)|0);
    $band = $266;
    $267 = $resno;
    $268 = ($267|0)==(0);
    if ($268) {
     $273 = 0;
    } else {
     $269 = $bandno;
     $270 = (($269) + 1)|0;
     $273 = $270;
    }
    $271 = $band;
    $272 = (($271) + 16|0);
    HEAP32[$272>>2] = $273;
    $274 = $band;
    $275 = (($274) + 16|0);
    $276 = HEAP32[$275>>2]|0;
    $277 = ($276|0)==(1);
    if ($277) {
     $283 = 1;
    } else {
     $278 = $band;
     $279 = (($278) + 16|0);
     $280 = HEAP32[$279>>2]|0;
     $281 = ($280|0)==(3);
     $283 = $281;
    }
    $282 = $283 ? 1 : 0;
    $x0b = $282;
    $284 = $band;
    $285 = (($284) + 16|0);
    $286 = HEAP32[$285>>2]|0;
    $287 = ($286|0)==(2);
    if ($287) {
     $293 = 1;
    } else {
     $288 = $band;
     $289 = (($288) + 16|0);
     $290 = HEAP32[$289>>2]|0;
     $291 = ($290|0)==(3);
     $293 = $291;
    }
    $292 = $293 ? 1 : 0;
    $y0b = $292;
    $294 = $band;
    $295 = (($294) + 16|0);
    $296 = HEAP32[$295>>2]|0;
    $297 = ($296|0)==(0);
    if ($297) {
     $298 = $tilec;
     $299 = HEAP32[$298>>2]|0;
     $300 = $levelno;
     $301 = (_int_ceildivpow279($299,$300)|0);
     $302 = $band;
     HEAP32[$302>>2] = $301;
     $303 = $tilec;
     $304 = (($303) + 4|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = $levelno;
     $307 = (_int_ceildivpow279($305,$306)|0);
     $308 = $band;
     $309 = (($308) + 4|0);
     HEAP32[$309>>2] = $307;
     $310 = $tilec;
     $311 = (($310) + 8|0);
     $312 = HEAP32[$311>>2]|0;
     $313 = $levelno;
     $314 = (_int_ceildivpow279($312,$313)|0);
     $315 = $band;
     $316 = (($315) + 8|0);
     HEAP32[$316>>2] = $314;
     $317 = $tilec;
     $318 = (($317) + 12|0);
     $319 = HEAP32[$318>>2]|0;
     $320 = $levelno;
     $321 = (_int_ceildivpow279($319,$320)|0);
     $322 = $band;
     $323 = (($322) + 12|0);
     HEAP32[$323>>2] = $321;
    } else {
     $324 = $tilec;
     $325 = HEAP32[$324>>2]|0;
     $326 = $levelno;
     $327 = 1 << $326;
     $328 = $x0b;
     $329 = Math_imul($327, $328)|0;
     $330 = (($325) - ($329))|0;
     $331 = $levelno;
     $332 = (($331) + 1)|0;
     $333 = (_int_ceildivpow279($330,$332)|0);
     $334 = $band;
     HEAP32[$334>>2] = $333;
     $335 = $tilec;
     $336 = (($335) + 4|0);
     $337 = HEAP32[$336>>2]|0;
     $338 = $levelno;
     $339 = 1 << $338;
     $340 = $y0b;
     $341 = Math_imul($339, $340)|0;
     $342 = (($337) - ($341))|0;
     $343 = $levelno;
     $344 = (($343) + 1)|0;
     $345 = (_int_ceildivpow279($342,$344)|0);
     $346 = $band;
     $347 = (($346) + 4|0);
     HEAP32[$347>>2] = $345;
     $348 = $tilec;
     $349 = (($348) + 8|0);
     $350 = HEAP32[$349>>2]|0;
     $351 = $levelno;
     $352 = 1 << $351;
     $353 = $x0b;
     $354 = Math_imul($352, $353)|0;
     $355 = (($350) - ($354))|0;
     $356 = $levelno;
     $357 = (($356) + 1)|0;
     $358 = (_int_ceildivpow279($355,$357)|0);
     $359 = $band;
     $360 = (($359) + 8|0);
     HEAP32[$360>>2] = $358;
     $361 = $tilec;
     $362 = (($361) + 12|0);
     $363 = HEAP32[$362>>2]|0;
     $364 = $levelno;
     $365 = 1 << $364;
     $366 = $y0b;
     $367 = Math_imul($365, $366)|0;
     $368 = (($363) - ($367))|0;
     $369 = $levelno;
     $370 = (($369) + 1)|0;
     $371 = (_int_ceildivpow279($368,$370)|0);
     $372 = $band;
     $373 = (($372) + 12|0);
     HEAP32[$373>>2] = $371;
    }
    $374 = $resno;
    $375 = ($374|0)==(0);
    if ($375) {
     $385 = 0;
    } else {
     $376 = $resno;
     $377 = (($376) - 1)|0;
     $378 = ($377*3)|0;
     $379 = $bandno;
     $380 = (($378) + ($379))|0;
     $381 = (($380) + 1)|0;
     $385 = $381;
    }
    $382 = $tccp;
    $383 = (($382) + 28|0);
    $384 = (($383) + ($385<<3)|0);
    $ss = $384;
    $386 = $tccp;
    $387 = (($386) + 20|0);
    $388 = HEAP32[$387>>2]|0;
    $389 = ($388|0)==(0);
    if ($389) {
     $390 = $band;
     $391 = (($390) + 16|0);
     $392 = HEAP32[$391>>2]|0;
     $393 = (_dwt_getgain_real($392)|0);
     $398 = $393;
    } else {
     $394 = $band;
     $395 = (($394) + 16|0);
     $396 = HEAP32[$395>>2]|0;
     $397 = (_dwt_getgain($396)|0);
     $398 = $397;
    }
    $gain = $398;
    $399 = $compno;
    $400 = $1;
    $401 = (($400) + 24|0);
    $402 = HEAP32[$401>>2]|0;
    $403 = (($402) + (($399*48)|0)|0);
    $404 = (($403) + 24|0);
    $405 = HEAP32[$404>>2]|0;
    $406 = $gain;
    $407 = (($405) + ($406))|0;
    $numbps = $407;
    $408 = $ss;
    $409 = (($408) + 4|0);
    $410 = HEAP32[$409>>2]|0;
    $411 = (+($410|0));
    $412 = $411 / 2048.0;
    $413 = 1.0 + $412;
    $414 = $numbps;
    $415 = $ss;
    $416 = HEAP32[$415>>2]|0;
    $417 = (($414) - ($416))|0;
    $418 = (+($417|0));
    $419 = (+Math_pow(2.0,(+$418)));
    $420 = $413 * $419;
    $421 = $420 * 0.5;
    $422 = $421;
    $423 = $band;
    $424 = (($423) + 28|0);
    HEAPF32[$424>>2] = $422;
    $425 = $ss;
    $426 = HEAP32[$425>>2]|0;
    $427 = $tccp;
    $428 = (($427) + 804|0);
    $429 = HEAP32[$428>>2]|0;
    $430 = (($426) + ($429))|0;
    $431 = (($430) - 1)|0;
    $432 = $band;
    $433 = (($432) + 24|0);
    HEAP32[$433>>2] = $431;
    $434 = $res;
    $435 = (($434) + 16|0);
    $436 = HEAP32[$435>>2]|0;
    $437 = $res;
    $438 = (($437) + 20|0);
    $439 = HEAP32[$438>>2]|0;
    $440 = Math_imul($436, $439)|0;
    $441 = ($440*36)|0;
    $442 = (_malloc($441)|0);
    $443 = $band;
    $444 = (($443) + 20|0);
    HEAP32[$444>>2] = $442;
    $precno = 0;
    while(1) {
     $445 = $precno;
     $446 = $res;
     $447 = (($446) + 16|0);
     $448 = HEAP32[$447>>2]|0;
     $449 = $res;
     $450 = (($449) + 20|0);
     $451 = HEAP32[$450>>2]|0;
     $452 = Math_imul($448, $451)|0;
     $453 = ($445|0)<($452|0);
     if (!($453)) {
      break;
     }
     $454 = $tlcbgxstart;
     $455 = $precno;
     $456 = $res;
     $457 = (($456) + 16|0);
     $458 = HEAP32[$457>>2]|0;
     $459 = (($455|0) % ($458|0))&-1;
     $460 = $cbgwidthexpn;
     $461 = 1 << $460;
     $462 = Math_imul($459, $461)|0;
     $463 = (($454) + ($462))|0;
     $cbgxstart = $463;
     $464 = $tlcbgystart;
     $465 = $precno;
     $466 = $res;
     $467 = (($466) + 16|0);
     $468 = HEAP32[$467>>2]|0;
     $469 = (($465|0) / ($468|0))&-1;
     $470 = $cbgheightexpn;
     $471 = 1 << $470;
     $472 = Math_imul($469, $471)|0;
     $473 = (($464) + ($472))|0;
     $cbgystart = $473;
     $474 = $cbgxstart;
     $475 = $cbgwidthexpn;
     $476 = 1 << $475;
     $477 = (($474) + ($476))|0;
     $cbgxend = $477;
     $478 = $cbgystart;
     $479 = $cbgheightexpn;
     $480 = 1 << $479;
     $481 = (($478) + ($480))|0;
     $cbgyend = $481;
     $482 = $precno;
     $483 = $band;
     $484 = (($483) + 20|0);
     $485 = HEAP32[$484>>2]|0;
     $486 = (($485) + (($482*36)|0)|0);
     $prc = $486;
     $487 = $cbgxstart;
     $488 = $band;
     $489 = HEAP32[$488>>2]|0;
     $490 = (_int_max76($487,$489)|0);
     $491 = $prc;
     HEAP32[$491>>2] = $490;
     $492 = $cbgystart;
     $493 = $band;
     $494 = (($493) + 4|0);
     $495 = HEAP32[$494>>2]|0;
     $496 = (_int_max76($492,$495)|0);
     $497 = $prc;
     $498 = (($497) + 4|0);
     HEAP32[$498>>2] = $496;
     $499 = $cbgxend;
     $500 = $band;
     $501 = (($500) + 8|0);
     $502 = HEAP32[$501>>2]|0;
     $503 = (_int_min77($499,$502)|0);
     $504 = $prc;
     $505 = (($504) + 8|0);
     HEAP32[$505>>2] = $503;
     $506 = $cbgyend;
     $507 = $band;
     $508 = (($507) + 12|0);
     $509 = HEAP32[$508>>2]|0;
     $510 = (_int_min77($506,$509)|0);
     $511 = $prc;
     $512 = (($511) + 12|0);
     HEAP32[$512>>2] = $510;
     $513 = $prc;
     $514 = HEAP32[$513>>2]|0;
     $515 = $cblkwidthexpn;
     $516 = (_int_floordivpow280($514,$515)|0);
     $517 = $cblkwidthexpn;
     $518 = $516 << $517;
     $tlcblkxstart = $518;
     $519 = $prc;
     $520 = (($519) + 4|0);
     $521 = HEAP32[$520>>2]|0;
     $522 = $cblkheightexpn;
     $523 = (_int_floordivpow280($521,$522)|0);
     $524 = $cblkheightexpn;
     $525 = $523 << $524;
     $tlcblkystart = $525;
     $526 = $prc;
     $527 = (($526) + 8|0);
     $528 = HEAP32[$527>>2]|0;
     $529 = $cblkwidthexpn;
     $530 = (_int_ceildivpow279($528,$529)|0);
     $531 = $cblkwidthexpn;
     $532 = $530 << $531;
     $brcblkxend = $532;
     $533 = $prc;
     $534 = (($533) + 12|0);
     $535 = HEAP32[$534>>2]|0;
     $536 = $cblkheightexpn;
     $537 = (_int_ceildivpow279($535,$536)|0);
     $538 = $cblkheightexpn;
     $539 = $537 << $538;
     $brcblkyend = $539;
     $540 = $brcblkxend;
     $541 = $tlcblkxstart;
     $542 = (($540) - ($541))|0;
     $543 = $cblkwidthexpn;
     $544 = $542 >> $543;
     $545 = $prc;
     $546 = (($545) + 16|0);
     HEAP32[$546>>2] = $544;
     $547 = $brcblkyend;
     $548 = $tlcblkystart;
     $549 = (($547) - ($548))|0;
     $550 = $cblkheightexpn;
     $551 = $549 >> $550;
     $552 = $prc;
     $553 = (($552) + 20|0);
     HEAP32[$553>>2] = $551;
     $554 = $prc;
     $555 = (($554) + 16|0);
     $556 = HEAP32[$555>>2]|0;
     $557 = $prc;
     $558 = (($557) + 20|0);
     $559 = HEAP32[$558>>2]|0;
     $560 = Math_imul($556, $559)|0;
     $561 = ($560*44)|0;
     $562 = (_malloc($561)|0);
     $563 = $prc;
     $564 = (($563) + 24|0);
     HEAP32[$564>>2] = $562;
     $565 = $prc;
     $566 = (($565) + 16|0);
     $567 = HEAP32[$566>>2]|0;
     $568 = $prc;
     $569 = (($568) + 20|0);
     $570 = HEAP32[$569>>2]|0;
     $571 = (_tgt_create($567,$570)|0);
     $572 = $prc;
     $573 = (($572) + 28|0);
     HEAP32[$573>>2] = $571;
     $574 = $prc;
     $575 = (($574) + 16|0);
     $576 = HEAP32[$575>>2]|0;
     $577 = $prc;
     $578 = (($577) + 20|0);
     $579 = HEAP32[$578>>2]|0;
     $580 = (_tgt_create($576,$579)|0);
     $581 = $prc;
     $582 = (($581) + 32|0);
     HEAP32[$582>>2] = $580;
     $cblkno = 0;
     while(1) {
      $583 = $cblkno;
      $584 = $prc;
      $585 = (($584) + 16|0);
      $586 = HEAP32[$585>>2]|0;
      $587 = $prc;
      $588 = (($587) + 20|0);
      $589 = HEAP32[$588>>2]|0;
      $590 = Math_imul($586, $589)|0;
      $591 = ($583|0)<($590|0);
      if (!($591)) {
       break;
      }
      $592 = $tlcblkxstart;
      $593 = $cblkno;
      $594 = $prc;
      $595 = (($594) + 16|0);
      $596 = HEAP32[$595>>2]|0;
      $597 = (($593|0) % ($596|0))&-1;
      $598 = $cblkwidthexpn;
      $599 = 1 << $598;
      $600 = Math_imul($597, $599)|0;
      $601 = (($592) + ($600))|0;
      $cblkxstart = $601;
      $602 = $tlcblkystart;
      $603 = $cblkno;
      $604 = $prc;
      $605 = (($604) + 16|0);
      $606 = HEAP32[$605>>2]|0;
      $607 = (($603|0) / ($606|0))&-1;
      $608 = $cblkheightexpn;
      $609 = 1 << $608;
      $610 = Math_imul($607, $609)|0;
      $611 = (($602) + ($610))|0;
      $cblkystart = $611;
      $612 = $cblkxstart;
      $613 = $cblkwidthexpn;
      $614 = 1 << $613;
      $615 = (($612) + ($614))|0;
      $cblkxend = $615;
      $616 = $cblkystart;
      $617 = $cblkheightexpn;
      $618 = 1 << $617;
      $619 = (($616) + ($618))|0;
      $cblkyend = $619;
      $620 = $cblkno;
      $621 = $prc;
      $622 = (($621) + 24|0);
      $623 = HEAP32[$622>>2]|0;
      $624 = (($623) + (($620*44)|0)|0);
      $cblk = $624;
      $625 = $cblk;
      HEAP32[$625>>2] = 0;
      $626 = $cblk;
      $627 = (($626) + 4|0);
      HEAP32[$627>>2] = 0;
      $628 = $cblkxstart;
      $629 = $prc;
      $630 = HEAP32[$629>>2]|0;
      $631 = (_int_max76($628,$630)|0);
      $632 = $cblk;
      $633 = (($632) + 8|0);
      HEAP32[$633>>2] = $631;
      $634 = $cblkystart;
      $635 = $prc;
      $636 = (($635) + 4|0);
      $637 = HEAP32[$636>>2]|0;
      $638 = (_int_max76($634,$637)|0);
      $639 = $cblk;
      $640 = (($639) + 12|0);
      HEAP32[$640>>2] = $638;
      $641 = $cblkxend;
      $642 = $prc;
      $643 = (($642) + 8|0);
      $644 = HEAP32[$643>>2]|0;
      $645 = (_int_min77($641,$644)|0);
      $646 = $cblk;
      $647 = (($646) + 16|0);
      HEAP32[$647>>2] = $645;
      $648 = $cblkyend;
      $649 = $prc;
      $650 = (($649) + 12|0);
      $651 = HEAP32[$650>>2]|0;
      $652 = (_int_min77($648,$651)|0);
      $653 = $cblk;
      $654 = (($653) + 20|0);
      HEAP32[$654>>2] = $652;
      $655 = $cblk;
      $656 = (($655) + 40|0);
      HEAP32[$656>>2] = 0;
      $657 = $cblkno;
      $658 = (($657) + 1)|0;
      $cblkno = $658;
     }
     $659 = $precno;
     $660 = (($659) + 1)|0;
     $precno = $660;
    }
    $661 = $bandno;
    $662 = (($661) + 1)|0;
    $bandno = $662;
   }
   $663 = $resno;
   $664 = (($663) + 1)|0;
   $resno = $664;
  }
  $665 = $compno;
  $666 = (($665) + 1)|0;
  $compno = $666;
 }
 STACKTOP = sp;return;
}
function _tcd_decode_tile($tcd,$src,$len,$tileno,$cstr_info) {
 $tcd = $tcd|0;
 $src = $src|0;
 $len = $len|0;
 $tileno = $tileno|0;
 $cstr_info = $cstr_info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0.0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0.0, $245 = 0.0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0.0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0.0, $344 = 0.0, $345 = 0.0, $346 = 0, $347 = 0, $348 = 0, $349 = 0.0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0.0, $619 = 0.0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0.0, $651 = 0.0, $652 = 0.0, $653 = 0, $654 = 0, $655 = 0, $656 = 0.0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $adjust = 0, $compno = 0, $compno1 = 0, $dwt_time = 0.0, $eof = 0, $i = 0, $imagec = 0, $j = 0, $l = 0, $max = 0, $min = 0, $n = 0, $numprec = 0, $numres2decode = 0, $offset_x = 0, $offset_y = 0, $res = 0, $res_idx = 0, $resno = 0, $t1 = 0;
 var $t1_time = 0.0, $t2 = 0, $tccp = 0, $tcp = 0, $tile = 0, $tile_time = 0.0, $tilec = 0, $tilec2 = 0, $tilec3 = 0, $tilec_idx = 0, $tmp = 0.0, $tw = 0, $v = 0, $v4 = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0;
 var $vararg_ptr1 = 0, $vararg_ptr10 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer14 = sp + 64|0;
 $vararg_buffer11 = sp + 56|0;
 $vararg_buffer7 = sp + 48|0;
 $vararg_buffer4 = sp + 40|0;
 $vararg_buffer2 = sp + 32|0;
 $vararg_buffer = sp + 24|0;
 $1 = $tcd;
 $2 = $src;
 $3 = $len;
 $4 = $tileno;
 $5 = $cstr_info;
 $eof = 0;
 $tile = 0;
 $t1 = 0;
 $t2 = 0;
 $6 = $4;
 $7 = $1;
 $8 = (($7) + 44|0);
 HEAP32[$8>>2] = $6;
 $9 = $4;
 $10 = $1;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + (($9*848)|0)|0);
 $16 = $1;
 $17 = (($16) + 36|0);
 HEAP32[$17>>2] = $15;
 $18 = $4;
 $19 = $1;
 $20 = (($19) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 108|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + (($18*5588)|0)|0);
 $25 = $1;
 $26 = (($25) + 40|0);
 HEAP32[$26>>2] = $24;
 $27 = $1;
 $28 = (($27) + 36|0);
 $29 = HEAP32[$28>>2]|0;
 $tile = $29;
 $30 = (+_opj_clock());
 $tile_time = $30;
 $31 = $1;
 $32 = (($31) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $4;
 $35 = (($34) + 1)|0;
 $36 = $1;
 $37 = (($36) + 32|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 68|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $1;
 $42 = (($41) + 32|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 72|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = Math_imul($40, $45)|0;
 HEAP32[$vararg_buffer>>2] = $35;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $46;
 (_opj_event_msg($33,4,56800,$vararg_buffer)|0);
 $47 = $5;
 $48 = ($47|0)!=(0|0);
 if ($48) {
  $numprec = 0;
  $compno1 = 0;
  while(1) {
   $49 = $compno1;
   $50 = $5;
   $51 = (($50) + 52|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($49|0)<($52|0);
   if (!($53)) {
    break;
   }
   $54 = $1;
   $55 = (($54) + 32|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 108|0);
   $58 = HEAP32[$57>>2]|0;
   $tcp = $58;
   $59 = $compno1;
   $60 = $tcp;
   $61 = (($60) + 5584|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + (($59*1076)|0)|0);
   $tccp = $63;
   $64 = $compno1;
   $65 = $tile;
   $66 = (($65) + 20|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (($67) + ($64<<5)|0);
   $tilec_idx = $68;
   $resno = 0;
   while(1) {
    $69 = $resno;
    $70 = $tilec_idx;
    $71 = (($70) + 16|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = ($69|0)<($72|0);
    if (!($73)) {
     break;
    }
    $74 = $resno;
    $75 = $tilec_idx;
    $76 = (($75) + 20|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = (($77) + (($74*124)|0)|0);
    $res_idx = $78;
    $79 = $res_idx;
    $80 = (($79) + 16|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = $resno;
    $83 = $4;
    $84 = $5;
    $85 = (($84) + 88|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = (($86) + (($83*576)|0)|0);
    $88 = (($87) + 20|0);
    $89 = (($88) + ($82<<2)|0);
    HEAP32[$89>>2] = $81;
    $90 = $res_idx;
    $91 = (($90) + 20|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = $resno;
    $94 = $4;
    $95 = $5;
    $96 = (($95) + 88|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97) + (($94*576)|0)|0);
    $99 = (($98) + 152|0);
    $100 = (($99) + ($93<<2)|0);
    HEAP32[$100>>2] = $92;
    $101 = $res_idx;
    $102 = (($101) + 16|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = $res_idx;
    $105 = (($104) + 20|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = Math_imul($103, $106)|0;
    $108 = $numprec;
    $109 = (($108) + ($107))|0;
    $numprec = $109;
    $110 = $tccp;
    $111 = HEAP32[$110>>2]|0;
    $112 = $111 & 1;
    $113 = ($112|0)!=(0);
    if ($113) {
     $114 = $resno;
     $115 = $tccp;
     $116 = (($115) + 812|0);
     $117 = (($116) + ($114<<2)|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = $resno;
     $120 = $4;
     $121 = $5;
     $122 = (($121) + 88|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = (($123) + (($120*576)|0)|0);
     $125 = (($124) + 284|0);
     $126 = (($125) + ($119<<2)|0);
     HEAP32[$126>>2] = $118;
     $127 = $resno;
     $128 = $tccp;
     $129 = (($128) + 944|0);
     $130 = (($129) + ($127<<2)|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = $resno;
     $133 = $4;
     $134 = $5;
     $135 = (($134) + 88|0);
     $136 = HEAP32[$135>>2]|0;
     $137 = (($136) + (($133*576)|0)|0);
     $138 = (($137) + 416|0);
     $139 = (($138) + ($132<<2)|0);
     HEAP32[$139>>2] = $131;
    } else {
     $140 = $resno;
     $141 = $4;
     $142 = $5;
     $143 = (($142) + 88|0);
     $144 = HEAP32[$143>>2]|0;
     $145 = (($144) + (($141*576)|0)|0);
     $146 = (($145) + 284|0);
     $147 = (($146) + ($140<<2)|0);
     HEAP32[$147>>2] = 15;
     $148 = $resno;
     $149 = $4;
     $150 = $5;
     $151 = (($150) + 88|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = (($152) + (($149*576)|0)|0);
     $154 = (($153) + 284|0);
     $155 = (($154) + ($148<<2)|0);
     HEAP32[$155>>2] = 15;
    }
    $156 = $resno;
    $157 = (($156) + 1)|0;
    $resno = $157;
   }
   $158 = $compno1;
   $159 = (($158) + 1)|0;
   $compno1 = $159;
  }
  $160 = $5;
  $161 = (($160) + 56|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = $numprec;
  $164 = Math_imul($162, $163)|0;
  $165 = ($164*24)|0;
  $166 = (_malloc($165)|0);
  $167 = $4;
  $168 = $5;
  $169 = (($168) + 88|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = (($170) + (($167*576)|0)|0);
  $172 = (($171) + 548|0);
  HEAP32[$172>>2] = $166;
  $173 = $5;
  $174 = (($173) + 8|0);
  HEAP32[$174>>2] = 0;
 }
 $175 = $1;
 $176 = (($175) + 20|0);
 $177 = HEAP32[$176>>2]|0;
 $178 = $1;
 $179 = (($178) + 28|0);
 $180 = HEAP32[$179>>2]|0;
 $181 = $1;
 $182 = (($181) + 32|0);
 $183 = HEAP32[$182>>2]|0;
 $184 = (_t2_create($177,$180,$183)|0);
 $t2 = $184;
 $185 = $t2;
 $186 = $2;
 $187 = $3;
 $188 = $4;
 $189 = $tile;
 $190 = $5;
 $191 = (_t2_decode_packets($185,$186,$187,$188,$189,$190)|0);
 $l = $191;
 $192 = $t2;
 _t2_destroy($192);
 $193 = $l;
 $194 = ($193|0)==(-999);
 if ($194) {
  $eof = 1;
  $195 = $1;
  $196 = (($195) + 20|0);
  $197 = HEAP32[$196>>2]|0;
  (_opj_event_msg($197,1,56816,$vararg_buffer2)|0);
 }
 $198 = (+_opj_clock());
 $t1_time = $198;
 $199 = $1;
 $200 = (($199) + 20|0);
 $201 = HEAP32[$200>>2]|0;
 $202 = (_t1_create($201)|0);
 $t1 = $202;
 $compno = 0;
 while(1) {
  $203 = $compno;
  $204 = $tile;
  $205 = (($204) + 16|0);
  $206 = HEAP32[$205>>2]|0;
  $207 = ($203|0)<($206|0);
  if (!($207)) {
   break;
  }
  $208 = $compno;
  $209 = $tile;
  $210 = (($209) + 20|0);
  $211 = HEAP32[$210>>2]|0;
  $212 = (($211) + ($208<<5)|0);
  $tilec = $212;
  $213 = $tilec;
  $214 = (($213) + 8|0);
  $215 = HEAP32[$214>>2]|0;
  $216 = $tilec;
  $217 = HEAP32[$216>>2]|0;
  $218 = (($215) - ($217))|0;
  $219 = $tilec;
  $220 = (($219) + 12|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = $tilec;
  $223 = (($222) + 4|0);
  $224 = HEAP32[$223>>2]|0;
  $225 = (($221) - ($224))|0;
  $226 = Math_imul($218, $225)|0;
  $227 = (($226) + 3)|0;
  $228 = $227<<2;
  $229 = (_memalign(16,$228)|0);
  $230 = $tilec;
  $231 = (($230) + 24|0);
  HEAP32[$231>>2] = $229;
  $232 = $t1;
  $233 = $tilec;
  $234 = $compno;
  $235 = $1;
  $236 = (($235) + 40|0);
  $237 = HEAP32[$236>>2]|0;
  $238 = (($237) + 5584|0);
  $239 = HEAP32[$238>>2]|0;
  $240 = (($239) + (($234*1076)|0)|0);
  _t1_decode_cblks($232,$233,$240);
  $241 = $compno;
  $242 = (($241) + 1)|0;
  $compno = $242;
 }
 $243 = $t1;
 _t1_destroy($243);
 $244 = (+_opj_clock());
 $245 = $t1_time;
 $246 = $244 - $245;
 $t1_time = $246;
 $247 = $1;
 $248 = (($247) + 20|0);
 $249 = HEAP32[$248>>2]|0;
 $250 = $t1_time;
 HEAPF64[tempDoublePtr>>3]=$250;HEAP32[$vararg_buffer4>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer4+4>>2]=HEAP32[tempDoublePtr+4>>2];
 (_opj_event_msg($249,4,56856,$vararg_buffer4)|0);
 $251 = (+_opj_clock());
 $dwt_time = $251;
 $compno = 0;
 while(1) {
  $252 = $compno;
  $253 = $tile;
  $254 = (($253) + 16|0);
  $255 = HEAP32[$254>>2]|0;
  $256 = ($252|0)<($255|0);
  if (!($256)) {
   break;
  }
  $257 = $compno;
  $258 = $tile;
  $259 = (($258) + 20|0);
  $260 = HEAP32[$259>>2]|0;
  $261 = (($260) + ($257<<5)|0);
  $tilec2 = $261;
  $262 = $1;
  $263 = (($262) + 32|0);
  $264 = HEAP32[$263>>2]|0;
  $265 = (($264) + 36|0);
  $266 = HEAP32[$265>>2]|0;
  $267 = ($266|0)!=(0);
  if ($267) {
   $268 = $compno;
   $269 = $tile;
   $270 = (($269) + 20|0);
   $271 = HEAP32[$270>>2]|0;
   $272 = (($271) + ($268<<5)|0);
   $273 = (($272) + 16|0);
   $274 = HEAP32[$273>>2]|0;
   $275 = $1;
   $276 = (($275) + 32|0);
   $277 = HEAP32[$276>>2]|0;
   $278 = (($277) + 36|0);
   $279 = HEAP32[$278>>2]|0;
   $280 = (($274) - ($279))|0;
   $281 = (($280) - 1)|0;
   $282 = $compno;
   $283 = $1;
   $284 = (($283) + 28|0);
   $285 = HEAP32[$284>>2]|0;
   $286 = (($285) + 24|0);
   $287 = HEAP32[$286>>2]|0;
   $288 = (($287) + (($282*48)|0)|0);
   $289 = (($288) + 36|0);
   HEAP32[$289>>2] = $281;
   $290 = $compno;
   $291 = $1;
   $292 = (($291) + 28|0);
   $293 = HEAP32[$292>>2]|0;
   $294 = (($293) + 24|0);
   $295 = HEAP32[$294>>2]|0;
   $296 = (($295) + (($290*48)|0)|0);
   $297 = (($296) + 36|0);
   $298 = HEAP32[$297>>2]|0;
   $299 = ($298|0)<(0);
   if ($299) {
    label = 24;
    break;
   }
  }
  $315 = $compno;
  $316 = $1;
  $317 = (($316) + 28|0);
  $318 = HEAP32[$317>>2]|0;
  $319 = (($318) + 24|0);
  $320 = HEAP32[$319>>2]|0;
  $321 = (($320) + (($315*48)|0)|0);
  $322 = (($321) + 36|0);
  $323 = HEAP32[$322>>2]|0;
  $324 = (($323) + 1)|0;
  $numres2decode = $324;
  $325 = $numres2decode;
  $326 = ($325|0)>(0);
  if ($326) {
   $327 = $compno;
   $328 = $1;
   $329 = (($328) + 40|0);
   $330 = HEAP32[$329>>2]|0;
   $331 = (($330) + 5584|0);
   $332 = HEAP32[$331>>2]|0;
   $333 = (($332) + (($327*1076)|0)|0);
   $334 = (($333) + 20|0);
   $335 = HEAP32[$334>>2]|0;
   $336 = ($335|0)==(1);
   if ($336) {
    $337 = $tilec2;
    $338 = $numres2decode;
    _dwt_decode($337,$338);
   } else {
    $339 = $tilec2;
    $340 = $numres2decode;
    _dwt_decode_real($339,$340);
   }
  }
  $341 = $compno;
  $342 = (($341) + 1)|0;
  $compno = $342;
 }
 if ((label|0) == 24) {
  $300 = $1;
  $301 = (($300) + 20|0);
  $302 = HEAP32[$301>>2]|0;
  $303 = $1;
  $304 = (($303) + 32|0);
  $305 = HEAP32[$304>>2]|0;
  $306 = (($305) + 36|0);
  $307 = HEAP32[$306>>2]|0;
  $308 = $compno;
  $309 = $tile;
  $310 = (($309) + 20|0);
  $311 = HEAP32[$310>>2]|0;
  $312 = (($311) + ($308<<5)|0);
  $313 = (($312) + 16|0);
  $314 = HEAP32[$313>>2]|0;
  HEAP32[$vararg_buffer7>>2] = $307;
  $vararg_ptr10 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr10>>2] = $314;
  (_opj_event_msg($302,1,56880,$vararg_buffer7)|0);
  $0 = 0;
  $659 = $0;
  STACKTOP = sp;return ($659|0);
 }
 $343 = (+_opj_clock());
 $344 = $dwt_time;
 $345 = $343 - $344;
 $dwt_time = $345;
 $346 = $1;
 $347 = (($346) + 20|0);
 $348 = HEAP32[$347>>2]|0;
 $349 = $dwt_time;
 HEAPF64[tempDoublePtr>>3]=$349;HEAP32[$vararg_buffer11>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer11+4>>2]=HEAP32[tempDoublePtr+4>>2];
 (_opj_event_msg($348,4,57056,$vararg_buffer11)|0);
 $350 = $1;
 $351 = (($350) + 40|0);
 $352 = HEAP32[$351>>2]|0;
 $353 = (($352) + 16|0);
 $354 = HEAP32[$353>>2]|0;
 $355 = ($354|0)!=(0);
 if ($355) {
  $356 = $tile;
  $357 = (($356) + 20|0);
  $358 = HEAP32[$357>>2]|0;
  $359 = (($358) + 8|0);
  $360 = HEAP32[$359>>2]|0;
  $361 = $tile;
  $362 = (($361) + 20|0);
  $363 = HEAP32[$362>>2]|0;
  $364 = HEAP32[$363>>2]|0;
  $365 = (($360) - ($364))|0;
  $366 = $tile;
  $367 = (($366) + 20|0);
  $368 = HEAP32[$367>>2]|0;
  $369 = (($368) + 12|0);
  $370 = HEAP32[$369>>2]|0;
  $371 = $tile;
  $372 = (($371) + 20|0);
  $373 = HEAP32[$372>>2]|0;
  $374 = (($373) + 4|0);
  $375 = HEAP32[$374>>2]|0;
  $376 = (($370) - ($375))|0;
  $377 = Math_imul($365, $376)|0;
  $n = $377;
  $378 = $1;
  $379 = (($378) + 40|0);
  $380 = HEAP32[$379>>2]|0;
  $381 = (($380) + 5584|0);
  $382 = HEAP32[$381>>2]|0;
  $383 = (($382) + 20|0);
  $384 = HEAP32[$383>>2]|0;
  $385 = ($384|0)==(1);
  if ($385) {
   $386 = $tile;
   $387 = (($386) + 20|0);
   $388 = HEAP32[$387>>2]|0;
   $389 = (($388) + 24|0);
   $390 = HEAP32[$389>>2]|0;
   $391 = $tile;
   $392 = (($391) + 20|0);
   $393 = HEAP32[$392>>2]|0;
   $394 = (($393) + 32|0);
   $395 = (($394) + 24|0);
   $396 = HEAP32[$395>>2]|0;
   $397 = $tile;
   $398 = (($397) + 20|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = (($399) + 64|0);
   $401 = (($400) + 24|0);
   $402 = HEAP32[$401>>2]|0;
   $403 = $n;
   _mct_decode($390,$396,$402,$403);
  } else {
   $404 = $tile;
   $405 = (($404) + 20|0);
   $406 = HEAP32[$405>>2]|0;
   $407 = (($406) + 24|0);
   $408 = HEAP32[$407>>2]|0;
   $409 = $tile;
   $410 = (($409) + 20|0);
   $411 = HEAP32[$410>>2]|0;
   $412 = (($411) + 32|0);
   $413 = (($412) + 24|0);
   $414 = HEAP32[$413>>2]|0;
   $415 = $tile;
   $416 = (($415) + 20|0);
   $417 = HEAP32[$416>>2]|0;
   $418 = (($417) + 64|0);
   $419 = (($418) + 24|0);
   $420 = HEAP32[$419>>2]|0;
   $421 = $n;
   _mct_decode_real($408,$414,$420,$421);
  }
 }
 $compno = 0;
 while(1) {
  $422 = $compno;
  $423 = $tile;
  $424 = (($423) + 16|0);
  $425 = HEAP32[$424>>2]|0;
  $426 = ($422|0)<($425|0);
  if (!($426)) {
   break;
  }
  $427 = $compno;
  $428 = $tile;
  $429 = (($428) + 20|0);
  $430 = HEAP32[$429>>2]|0;
  $431 = (($430) + ($427<<5)|0);
  $tilec3 = $431;
  $432 = $compno;
  $433 = $1;
  $434 = (($433) + 28|0);
  $435 = HEAP32[$434>>2]|0;
  $436 = (($435) + 24|0);
  $437 = HEAP32[$436>>2]|0;
  $438 = (($437) + (($432*48)|0)|0);
  $imagec = $438;
  $439 = $imagec;
  $440 = (($439) + 36|0);
  $441 = HEAP32[$440>>2]|0;
  $442 = $tilec3;
  $443 = (($442) + 20|0);
  $444 = HEAP32[$443>>2]|0;
  $445 = (($444) + (($441*124)|0)|0);
  $res = $445;
  $446 = $imagec;
  $447 = (($446) + 32|0);
  $448 = HEAP32[$447>>2]|0;
  $449 = ($448|0)!=(0);
  if ($449) {
   $455 = 0;
  } else {
   $450 = $imagec;
   $451 = (($450) + 24|0);
   $452 = HEAP32[$451>>2]|0;
   $453 = (($452) - 1)|0;
   $454 = 1 << $453;
   $455 = $454;
  }
  $adjust = $455;
  $456 = $imagec;
  $457 = (($456) + 32|0);
  $458 = HEAP32[$457>>2]|0;
  $459 = ($458|0)!=(0);
  if ($459) {
   $460 = $imagec;
   $461 = (($460) + 24|0);
   $462 = HEAP32[$461>>2]|0;
   $463 = (($462) - 1)|0;
   $464 = 1 << $463;
   $465 = (0 - ($464))|0;
   $466 = $465;
  } else {
   $466 = 0;
  }
  $min = $466;
  $467 = $imagec;
  $468 = (($467) + 32|0);
  $469 = HEAP32[$468>>2]|0;
  $470 = ($469|0)!=(0);
  if ($470) {
   $471 = $imagec;
   $472 = (($471) + 24|0);
   $473 = HEAP32[$472>>2]|0;
   $474 = (($473) - 1)|0;
   $475 = 1 << $474;
   $476 = (($475) - 1)|0;
   $482 = $476;
  } else {
   $477 = $imagec;
   $478 = (($477) + 24|0);
   $479 = HEAP32[$478>>2]|0;
   $480 = 1 << $479;
   $481 = (($480) - 1)|0;
   $482 = $481;
  }
  $max = $482;
  $483 = $tilec3;
  $484 = (($483) + 8|0);
  $485 = HEAP32[$484>>2]|0;
  $486 = $tilec3;
  $487 = HEAP32[$486>>2]|0;
  $488 = (($485) - ($487))|0;
  $tw = $488;
  $489 = $imagec;
  $490 = (($489) + 8|0);
  $491 = HEAP32[$490>>2]|0;
  $w = $491;
  $492 = $imagec;
  $493 = (($492) + 16|0);
  $494 = HEAP32[$493>>2]|0;
  $495 = $imagec;
  $496 = (($495) + 40|0);
  $497 = HEAP32[$496>>2]|0;
  $498 = (_int_ceildivpow279($494,$497)|0);
  $offset_x = $498;
  $499 = $imagec;
  $500 = (($499) + 20|0);
  $501 = HEAP32[$500>>2]|0;
  $502 = $imagec;
  $503 = (($502) + 40|0);
  $504 = HEAP32[$503>>2]|0;
  $505 = (_int_ceildivpow279($501,$504)|0);
  $offset_y = $505;
  $506 = $imagec;
  $507 = (($506) + 44|0);
  $508 = HEAP32[$507>>2]|0;
  $509 = ($508|0)!=(0|0);
  if (!($509)) {
   $510 = $imagec;
   $511 = (($510) + 8|0);
   $512 = HEAP32[$511>>2]|0;
   $513 = $imagec;
   $514 = (($513) + 12|0);
   $515 = HEAP32[$514>>2]|0;
   $516 = Math_imul($512, $515)|0;
   $517 = $516<<2;
   $518 = (_malloc($517)|0);
   $519 = $imagec;
   $520 = (($519) + 44|0);
   HEAP32[$520>>2] = $518;
  }
  $521 = $compno;
  $522 = $1;
  $523 = (($522) + 40|0);
  $524 = HEAP32[$523>>2]|0;
  $525 = (($524) + 5584|0);
  $526 = HEAP32[$525>>2]|0;
  $527 = (($526) + (($521*1076)|0)|0);
  $528 = (($527) + 20|0);
  $529 = HEAP32[$528>>2]|0;
  $530 = ($529|0)==(1);
  if ($530) {
   $531 = $res;
   $532 = (($531) + 4|0);
   $533 = HEAP32[$532>>2]|0;
   $j = $533;
   while(1) {
    $534 = $j;
    $535 = $res;
    $536 = (($535) + 12|0);
    $537 = HEAP32[$536>>2]|0;
    $538 = ($534|0)<($537|0);
    if (!($538)) {
     break;
    }
    $539 = $res;
    $540 = HEAP32[$539>>2]|0;
    $i = $540;
    while(1) {
     $541 = $i;
     $542 = $res;
     $543 = (($542) + 8|0);
     $544 = HEAP32[$543>>2]|0;
     $545 = ($541|0)<($544|0);
     if (!($545)) {
      break;
     }
     $546 = $i;
     $547 = $res;
     $548 = HEAP32[$547>>2]|0;
     $549 = (($546) - ($548))|0;
     $550 = $j;
     $551 = $res;
     $552 = (($551) + 4|0);
     $553 = HEAP32[$552>>2]|0;
     $554 = (($550) - ($553))|0;
     $555 = $tw;
     $556 = Math_imul($554, $555)|0;
     $557 = (($549) + ($556))|0;
     $558 = $tilec3;
     $559 = (($558) + 24|0);
     $560 = HEAP32[$559>>2]|0;
     $561 = (($560) + ($557<<2)|0);
     $562 = HEAP32[$561>>2]|0;
     $v = $562;
     $563 = $adjust;
     $564 = $v;
     $565 = (($564) + ($563))|0;
     $v = $565;
     $566 = $v;
     $567 = $min;
     $568 = $max;
     $569 = (_int_clamp($566,$567,$568)|0);
     $570 = $i;
     $571 = $offset_x;
     $572 = (($570) - ($571))|0;
     $573 = $j;
     $574 = $offset_y;
     $575 = (($573) - ($574))|0;
     $576 = $w;
     $577 = Math_imul($575, $576)|0;
     $578 = (($572) + ($577))|0;
     $579 = $imagec;
     $580 = (($579) + 44|0);
     $581 = HEAP32[$580>>2]|0;
     $582 = (($581) + ($578<<2)|0);
     HEAP32[$582>>2] = $569;
     $583 = $i;
     $584 = (($583) + 1)|0;
     $i = $584;
    }
    $585 = $j;
    $586 = (($585) + 1)|0;
    $j = $586;
   }
  } else {
   $587 = $res;
   $588 = (($587) + 4|0);
   $589 = HEAP32[$588>>2]|0;
   $j = $589;
   while(1) {
    $590 = $j;
    $591 = $res;
    $592 = (($591) + 12|0);
    $593 = HEAP32[$592>>2]|0;
    $594 = ($590|0)<($593|0);
    if (!($594)) {
     break;
    }
    $595 = $res;
    $596 = HEAP32[$595>>2]|0;
    $i = $596;
    while(1) {
     $597 = $i;
     $598 = $res;
     $599 = (($598) + 8|0);
     $600 = HEAP32[$599>>2]|0;
     $601 = ($597|0)<($600|0);
     if (!($601)) {
      break;
     }
     $602 = $i;
     $603 = $res;
     $604 = HEAP32[$603>>2]|0;
     $605 = (($602) - ($604))|0;
     $606 = $j;
     $607 = $res;
     $608 = (($607) + 4|0);
     $609 = HEAP32[$608>>2]|0;
     $610 = (($606) - ($609))|0;
     $611 = $tw;
     $612 = Math_imul($610, $611)|0;
     $613 = (($605) + ($612))|0;
     $614 = $tilec3;
     $615 = (($614) + 24|0);
     $616 = HEAP32[$615>>2]|0;
     $617 = (($616) + ($613<<2)|0);
     $618 = +HEAPF32[$617>>2];
     $tmp = $618;
     $619 = $tmp;
     $620 = (_lrintf((+$619))|0);
     $v4 = $620;
     $621 = $adjust;
     $622 = $v4;
     $623 = (($622) + ($621))|0;
     $v4 = $623;
     $624 = $v4;
     $625 = $min;
     $626 = $max;
     $627 = (_int_clamp($624,$625,$626)|0);
     $628 = $i;
     $629 = $offset_x;
     $630 = (($628) - ($629))|0;
     $631 = $j;
     $632 = $offset_y;
     $633 = (($631) - ($632))|0;
     $634 = $w;
     $635 = Math_imul($633, $634)|0;
     $636 = (($630) + ($635))|0;
     $637 = $imagec;
     $638 = (($637) + 44|0);
     $639 = HEAP32[$638>>2]|0;
     $640 = (($639) + ($636<<2)|0);
     HEAP32[$640>>2] = $627;
     $641 = $i;
     $642 = (($641) + 1)|0;
     $i = $642;
    }
    $643 = $j;
    $644 = (($643) + 1)|0;
    $j = $644;
   }
  }
  $645 = $tilec3;
  $646 = (($645) + 24|0);
  $647 = HEAP32[$646>>2]|0;
  _free($647);
  $648 = $compno;
  $649 = (($648) + 1)|0;
  $compno = $649;
 }
 $650 = (+_opj_clock());
 $651 = $tile_time;
 $652 = $650 - $651;
 $tile_time = $652;
 $653 = $1;
 $654 = (($653) + 20|0);
 $655 = HEAP32[$654>>2]|0;
 $656 = $tile_time;
 HEAPF64[tempDoublePtr>>3]=$656;HEAP32[$vararg_buffer14>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer14+4>>2]=HEAP32[tempDoublePtr+4>>2];
 (_opj_event_msg($655,4,57080,$vararg_buffer14)|0);
 $657 = $eof;
 $658 = ($657|0)!=(0);
 if ($658) {
  $0 = 0;
  $659 = $0;
  STACKTOP = sp;return ($659|0);
 } else {
  $0 = 1;
  $659 = $0;
  STACKTOP = sp;return ($659|0);
 }
 return 0|0;
}
function _int_clamp($a,$min,$max) {
 $a = $a|0;
 $min = $min|0;
 $max = $max|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $a;
 $2 = $min;
 $3 = $max;
 $4 = $1;
 $5 = $2;
 $6 = ($4|0)<($5|0);
 do {
  if ($6) {
   $7 = $2;
   $0 = $7;
  } else {
   $8 = $1;
   $9 = $3;
   $10 = ($8|0)>($9|0);
   if ($10) {
    $11 = $3;
    $0 = $11;
    break;
   } else {
    $12 = $1;
    $0 = $12;
    break;
   }
  }
 } while(0);
 $13 = $0;
 STACKTOP = sp;return ($13|0);
}
function _tcd_free_decode($tcd) {
 $tcd = $tcd|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $tcd_image = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $tcd;
 $1 = $0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $tcd_image = $3;
 $4 = $tcd_image;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 _free($6);
 STACKTOP = sp;return;
}
function _tcd_free_decode_tile($tcd,$tileno) {
 $tcd = $tcd|0;
 $tileno = $tileno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $band = 0, $bandno = 0, $compno = 0, $prec = 0, $precno = 0, $res = 0, $resno = 0, $tcd_image = 0, $tile = 0, $tilec = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $tcd;
 $1 = $tileno;
 $2 = $0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $tcd_image = $4;
 $5 = $1;
 $6 = $tcd_image;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + (($5*848)|0)|0);
 $tile = $9;
 $compno = 0;
 while(1) {
  $10 = $compno;
  $11 = $tile;
  $12 = (($11) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $compno;
  $16 = $tile;
  $17 = (($16) + 20|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<5)|0);
  $tilec = $19;
  $resno = 0;
  while(1) {
   $20 = $resno;
   $21 = $tilec;
   $22 = (($21) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($20|0)<($23|0);
   if (!($24)) {
    break;
   }
   $25 = $resno;
   $26 = $tilec;
   $27 = (($26) + 20|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + (($25*124)|0)|0);
   $res = $29;
   $bandno = 0;
   while(1) {
    $30 = $bandno;
    $31 = $res;
    $32 = (($31) + 24|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30|0)<($33|0);
    if (!($34)) {
     break;
    }
    $35 = $bandno;
    $36 = $res;
    $37 = (($36) + 28|0);
    $38 = (($37) + ($35<<5)|0);
    $band = $38;
    $precno = 0;
    while(1) {
     $39 = $precno;
     $40 = $res;
     $41 = (($40) + 20|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = $res;
     $44 = (($43) + 16|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = Math_imul($42, $45)|0;
     $47 = ($39|0)<($46|0);
     if (!($47)) {
      break;
     }
     $48 = $precno;
     $49 = $band;
     $50 = (($49) + 20|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + (($48*36)|0)|0);
     $prec = $52;
     $53 = $prec;
     $54 = (($53) + 32|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)!=(0|0);
     if ($56) {
      $57 = $prec;
      $58 = (($57) + 32|0);
      $59 = HEAP32[$58>>2]|0;
      _tgt_destroy($59);
     }
     $60 = $prec;
     $61 = (($60) + 28|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = ($62|0)!=(0|0);
     if ($63) {
      $64 = $prec;
      $65 = (($64) + 28|0);
      $66 = HEAP32[$65>>2]|0;
      _tgt_destroy($66);
     }
     $67 = $precno;
     $68 = (($67) + 1)|0;
     $precno = $68;
    }
    $69 = $band;
    $70 = (($69) + 20|0);
    $71 = HEAP32[$70>>2]|0;
    _free($71);
    $72 = $bandno;
    $73 = (($72) + 1)|0;
    $bandno = $73;
   }
   $74 = $resno;
   $75 = (($74) + 1)|0;
   $resno = $75;
  }
  $76 = $tilec;
  $77 = (($76) + 20|0);
  $78 = HEAP32[$77>>2]|0;
  _free($78);
  $79 = $compno;
  $80 = (($79) + 1)|0;
  $compno = $80;
 }
 $81 = $tile;
 $82 = (($81) + 20|0);
 $83 = HEAP32[$82>>2]|0;
 _free($83);
 STACKTOP = sp;return;
}
function _tgt_create($numleafsh,$numleafsv) {
 $numleafsh = $numleafsh|0;
 $numleafsv = $numleafsv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0, $k = 0, $n = 0, $node = 0, $nplh = 0, $nplv = 0, $numlvls = 0, $parentnode = 0, $parentnode0 = 0, $tree = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 320|0;
 $nplh = sp + 168|0;
 $nplv = sp + 40|0;
 $1 = $numleafsh;
 $2 = $numleafsv;
 $node = 0;
 $parentnode = 0;
 $parentnode0 = 0;
 $tree = 0;
 $3 = (_malloc(16)|0);
 $tree = $3;
 $4 = $tree;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $0 = 0;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $6 = $1;
 $7 = $tree;
 HEAP32[$7>>2] = $6;
 $8 = $2;
 $9 = $tree;
 $10 = (($9) + 4|0);
 HEAP32[$10>>2] = $8;
 $numlvls = 0;
 $11 = $1;
 HEAP32[$nplh>>2] = $11;
 $12 = $2;
 HEAP32[$nplv>>2] = $12;
 $13 = $tree;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = 0;
 while(1) {
  $15 = $numlvls;
  $16 = (($nplh) + ($15<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $numlvls;
  $19 = (($nplv) + ($18<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = Math_imul($17, $20)|0;
  $n = $21;
  $22 = $numlvls;
  $23 = (($nplh) + ($22<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 1)|0;
  $26 = (($25|0) / 2)&-1;
  $27 = $numlvls;
  $28 = (($27) + 1)|0;
  $29 = (($nplh) + ($28<<2)|0);
  HEAP32[$29>>2] = $26;
  $30 = $numlvls;
  $31 = (($nplv) + ($30<<2)|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 1)|0;
  $34 = (($33|0) / 2)&-1;
  $35 = $numlvls;
  $36 = (($35) + 1)|0;
  $37 = (($nplv) + ($36<<2)|0);
  HEAP32[$37>>2] = $34;
  $38 = $n;
  $39 = $tree;
  $40 = (($39) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($38))|0;
  HEAP32[$40>>2] = $42;
  $43 = $numlvls;
  $44 = (($43) + 1)|0;
  $numlvls = $44;
  $45 = $n;
  $46 = ($45|0)>(1);
  if (!($46)) {
   break;
  }
 }
 $47 = $tree;
 $48 = (($47) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = $tree;
  _free($51);
  $0 = 0;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $52 = $tree;
 $53 = (($52) + 8|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (_calloc($54,16)|0);
 $56 = $tree;
 $57 = (($56) + 12|0);
 HEAP32[$57>>2] = $55;
 $58 = $tree;
 $59 = (($58) + 12|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)!=(0|0);
 if (!($61)) {
  $62 = $tree;
  _free($62);
  $0 = 0;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $63 = $tree;
 $64 = (($63) + 12|0);
 $65 = HEAP32[$64>>2]|0;
 $node = $65;
 $66 = $tree;
 $67 = HEAP32[$66>>2]|0;
 $68 = $tree;
 $69 = (($68) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = Math_imul($67, $70)|0;
 $72 = $tree;
 $73 = (($72) + 12|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = (($74) + ($71<<4)|0);
 $parentnode = $75;
 $76 = $parentnode;
 $parentnode0 = $76;
 $i = 0;
 while(1) {
  $77 = $i;
  $78 = $numlvls;
  $79 = (($78) - 1)|0;
  $80 = ($77|0)<($79|0);
  if (!($80)) {
   break;
  }
  $j = 0;
  while(1) {
   $81 = $j;
   $82 = $i;
   $83 = (($nplv) + ($82<<2)|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($81|0)<($84|0);
   if (!($85)) {
    break;
   }
   $86 = $i;
   $87 = (($nplh) + ($86<<2)|0);
   $88 = HEAP32[$87>>2]|0;
   $k = $88;
   while(1) {
    $89 = $k;
    $90 = (($89) + -1)|0;
    $k = $90;
    $91 = ($90|0)>=(0);
    if (!($91)) {
     break;
    }
    $92 = $parentnode;
    $93 = $node;
    HEAP32[$93>>2] = $92;
    $94 = $node;
    $95 = (($94) + 16|0);
    $node = $95;
    $96 = $k;
    $97 = (($96) + -1)|0;
    $k = $97;
    $98 = ($97|0)>=(0);
    if ($98) {
     $99 = $parentnode;
     $100 = $node;
     HEAP32[$100>>2] = $99;
     $101 = $node;
     $102 = (($101) + 16|0);
     $node = $102;
    }
    $103 = $parentnode;
    $104 = (($103) + 16|0);
    $parentnode = $104;
   }
   $105 = $j;
   $106 = $105 & 1;
   $107 = ($106|0)!=(0);
   if ($107) {
    label = 21;
   } else {
    $108 = $j;
    $109 = $i;
    $110 = (($nplv) + ($109<<2)|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = (($111) - 1)|0;
    $113 = ($108|0)==($112|0);
    if ($113) {
     label = 21;
    } else {
     $115 = $parentnode0;
     $parentnode = $115;
     $116 = $i;
     $117 = (($nplh) + ($116<<2)|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = $parentnode0;
     $120 = (($119) + ($118<<4)|0);
     $parentnode0 = $120;
    }
   }
   if ((label|0) == 21) {
    label = 0;
    $114 = $parentnode;
    $parentnode0 = $114;
   }
   $121 = $j;
   $122 = (($121) + 1)|0;
   $j = $122;
  }
  $123 = $i;
  $124 = (($123) + 1)|0;
  $i = $124;
 }
 $125 = $node;
 HEAP32[$125>>2] = 0;
 $126 = $tree;
 _tgt_reset($126);
 $127 = $tree;
 $0 = $127;
 $128 = $0;
 STACKTOP = sp;return ($128|0);
}
function _tgt_reset($tree) {
 $tree = $tree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $tree;
 $1 = $0;
 $2 = (0|0)==($1|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $0;
  $5 = (($4) + 8|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $i;
  $9 = $0;
  $10 = (($9) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = (($12) + 4|0);
  HEAP32[$13>>2] = 999;
  $14 = $i;
  $15 = $0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<4)|0);
  $19 = (($18) + 8|0);
  HEAP32[$19>>2] = 0;
  $20 = $i;
  $21 = $0;
  $22 = (($21) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<4)|0);
  $25 = (($24) + 12|0);
  HEAP32[$25>>2] = 0;
  $26 = $i;
  $27 = (($26) + 1)|0;
  $i = $27;
 }
 STACKTOP = sp;return;
}
function _tgt_destroy($tree) {
 $tree = $tree|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $tree;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 _free($3);
 $4 = $0;
 _free($4);
 STACKTOP = sp;return;
}
function _tgt_decode($bio,$tree,$leafno,$threshold) {
 $bio = $bio|0;
 $tree = $tree|0;
 $leafno = $leafno|0;
 $threshold = $threshold|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $low = 0;
 var $node = 0, $stk = 0, $stkptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $stk = sp + 12|0;
 $0 = $bio;
 $1 = $tree;
 $2 = $leafno;
 $3 = $threshold;
 $stkptr = $stk;
 $4 = $2;
 $5 = $1;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 $node = $8;
 while(1) {
  $9 = $node;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = $node;
  $13 = $stkptr;
  $14 = (($13) + 4|0);
  $stkptr = $14;
  HEAP32[$13>>2] = $12;
  $15 = $node;
  $16 = HEAP32[$15>>2]|0;
  $node = $16;
 }
 $low = 0;
 while(1) {
  $17 = $low;
  $18 = $node;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)>($20|0);
  if ($21) {
   $22 = $low;
   $23 = $node;
   $24 = (($23) + 8|0);
   HEAP32[$24>>2] = $22;
  } else {
   $25 = $node;
   $26 = (($25) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $low = $27;
  }
  while(1) {
   $28 = $low;
   $29 = $3;
   $30 = ($28|0)<($29|0);
   if ($30) {
    $31 = $low;
    $32 = $node;
    $33 = (($32) + 4|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = ($31|0)<($34|0);
    $58 = $35;
   } else {
    $58 = 0;
   }
   if (!($58)) {
    break;
   }
   $36 = $0;
   $37 = (_bio_read($36,1)|0);
   $38 = ($37|0)!=(0);
   if ($38) {
    $39 = $low;
    $40 = $node;
    $41 = (($40) + 4|0);
    HEAP32[$41>>2] = $39;
   } else {
    $42 = $low;
    $43 = (($42) + 1)|0;
    $low = $43;
   }
  }
  $44 = $low;
  $45 = $node;
  $46 = (($45) + 8|0);
  HEAP32[$46>>2] = $44;
  $47 = $stkptr;
  $48 = ($47|0)==($stk|0);
  if ($48) {
   break;
  }
  $49 = $stkptr;
  $50 = (($49) + -4|0);
  $stkptr = $50;
  $51 = HEAP32[$50>>2]|0;
  $node = $51;
 }
 $52 = $node;
 $53 = (($52) + 4|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $3;
 $56 = ($54|0)<($55|0);
 $57 = $56 ? 1 : 0;
 STACKTOP = sp;return ($57|0);
}
function _png_sig_cmp($sig,$start,$num_to_check) {
 $sig = $sig|0;
 $start = $start|0;
 $num_to_check = $num_to_check|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $png_signature = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $png_signature = sp + 16|0;
 $1 = $sig;
 $2 = $start;
 $3 = $num_to_check;
 ;HEAP8[$png_signature+0>>0]=HEAP8[57480+0>>0]|0;HEAP8[$png_signature+1>>0]=HEAP8[57480+1>>0]|0;HEAP8[$png_signature+2>>0]=HEAP8[57480+2>>0]|0;HEAP8[$png_signature+3>>0]=HEAP8[57480+3>>0]|0;HEAP8[$png_signature+4>>0]=HEAP8[57480+4>>0]|0;HEAP8[$png_signature+5>>0]=HEAP8[57480+5>>0]|0;HEAP8[$png_signature+6>>0]=HEAP8[57480+6>>0]|0;HEAP8[$png_signature+7>>0]=HEAP8[57480+7>>0]|0;
 $4 = $3;
 $5 = ($4>>>0)>(8);
 do {
  if ($5) {
   $3 = 8;
  } else {
   $6 = $3;
   $7 = ($6>>>0)<(1);
   if (!($7)) {
    break;
   }
   $0 = -1;
   $23 = $0;
   STACKTOP = sp;return ($23|0);
  }
 } while(0);
 $8 = $2;
 $9 = ($8>>>0)>(7);
 if ($9) {
  $0 = -1;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 $10 = $2;
 $11 = $3;
 $12 = (($10) + ($11))|0;
 $13 = ($12>>>0)>(8);
 if ($13) {
  $14 = $2;
  $15 = (8 - ($14))|0;
  $3 = $15;
 }
 $16 = $2;
 $17 = $1;
 $18 = (($17) + ($16)|0);
 $19 = $2;
 $20 = (($png_signature) + ($19)|0);
 $21 = $3;
 $22 = (_memcmp($18,$20,$21)|0);
 $0 = $22;
 $23 = $0;
 STACKTOP = sp;return ($23|0);
}
function _png_zalloc($png_ptr,$items,$size) {
 $png_ptr = $png_ptr|0;
 $items = $items|0;
 $size = $size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $num_bytes = 0, $p = 0, $ptr = 0, $save_flags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $png_ptr;
 $2 = $items;
 $3 = $size;
 $4 = $1;
 $p = $4;
 $5 = $p;
 $6 = (($5) + 200|0);
 $7 = HEAP32[$6>>2]|0;
 $save_flags = $7;
 $8 = $1;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $0 = 0;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 }
 $10 = $2;
 $11 = $3;
 $12 = (4294967295 / ($11>>>0))&-1;
 $13 = ($10>>>0)>($12>>>0);
 if ($13) {
  $14 = $p;
  _png_warning($14,57488);
  $0 = 0;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 } else {
  $15 = $2;
  $16 = $3;
  $17 = Math_imul($15, $16)|0;
  $num_bytes = $17;
  $18 = $p;
  $19 = (($18) + 200|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $20 | 1048576;
  HEAP32[$19>>2] = $21;
  $22 = $1;
  $23 = $num_bytes;
  $24 = (_png_malloc($22,$23)|0);
  $ptr = $24;
  $25 = $save_flags;
  $26 = $p;
  $27 = (($26) + 200|0);
  HEAP32[$27>>2] = $25;
  $28 = $ptr;
  $0 = $28;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _png_zfree($png_ptr,$ptr) {
 $png_ptr = $png_ptr|0;
 $ptr = $ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $ptr;
 $2 = $0;
 $3 = $1;
 _png_free($2,$3);
 STACKTOP = sp;return;
}
function _png_reset_crc($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = (_crc32(0,0,0)|0);
 $2 = $0;
 $3 = (($2) + 364|0);
 HEAP32[$3>>2] = $1;
 STACKTOP = sp;return;
}
function _png_calculate_crc($png_ptr,$ptr,$length) {
 $png_ptr = $png_ptr|0;
 $ptr = $ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $need_crc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $ptr;
 $2 = $length;
 $need_crc = 1;
 $3 = $0;
 $4 = (($3) + 376|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 32;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $0;
  $10 = (($9) + 200|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 768;
  $13 = ($12|0)==(768);
  if ($13) {
   $need_crc = 0;
  }
 } else {
  $14 = $0;
  $15 = (($14) + 200|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $16 & 2048;
  $18 = ($17|0)!=(0);
  if ($18) {
   $need_crc = 0;
  }
 }
 $19 = $need_crc;
 $20 = ($19|0)!=(0);
 if (!($20)) {
  STACKTOP = sp;return;
 }
 $21 = $0;
 $22 = (($21) + 364|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $1;
 $25 = $2;
 $26 = (_crc32($23,$24,$25)|0);
 $27 = $0;
 $28 = (($27) + 364|0);
 HEAP32[$28>>2] = $26;
 STACKTOP = sp;return;
}
function _png_create_info_struct($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $info_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $info_ptr = sp;
 $1 = $png_ptr;
 $2 = $1;
 $3 = ($2|0)==(0|0);
 if ($3) {
  $0 = 0;
  $14 = $0;
  STACKTOP = sp;return ($14|0);
 }
 $4 = $1;
 $5 = (($4) + 676|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $1;
 $8 = (($7) + 672|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_png_create_struct_2(2,$6,$9)|0);
 HEAP32[$info_ptr>>2] = $10;
 $11 = HEAP32[$info_ptr>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  _png_info_init_3($info_ptr,288);
 }
 $13 = HEAP32[$info_ptr>>2]|0;
 $0 = $13;
 $14 = $0;
 STACKTOP = sp;return ($14|0);
}
function _png_info_init_3($ptr_ptr,$png_info_struct_size) {
 $ptr_ptr = $ptr_ptr|0;
 $png_info_struct_size = $png_info_struct_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $info_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $ptr_ptr;
 $1 = $png_info_struct_size;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $info_ptr = $3;
 $4 = $info_ptr;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = $1;
 $7 = (288)>($6>>>0);
 if ($7) {
  $8 = $info_ptr;
  _png_destroy_struct($8);
  $9 = (_png_create_struct(2)|0);
  $info_ptr = $9;
  $10 = $info_ptr;
  $11 = $0;
  HEAP32[$11>>2] = $10;
 }
 $12 = $info_ptr;
 _memset(($12|0),0,288)|0;
 STACKTOP = sp;return;
}
function _png_info_destroy($png_ptr,$info_ptr) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = sp;
 $0 = $png_ptr;
 HEAP32[$1>>2] = $info_ptr;
 $2 = $0;
 $3 = HEAP32[$1>>2]|0;
 _png_free_data($2,$3,32767,-1);
 $4 = $0;
 $5 = (($4) + 636|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  _png_info_init_3($1,288);
  STACKTOP = sp;return;
 }
 $8 = $0;
 $9 = $0;
 $10 = (($9) + 640|0);
 $11 = HEAP32[$10>>2]|0;
 _png_free($8,$11);
 $12 = $0;
 $13 = (($12) + 640|0);
 HEAP32[$13>>2] = 0;
 $14 = $0;
 $15 = (($14) + 636|0);
 HEAP32[$15>>2] = 0;
 _png_info_init_3($1,288);
 STACKTOP = sp;return;
}
function _png_free_data($png_ptr,$info_ptr,$mask,$num) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $mask = $mask|0;
 $num = $num|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $i = 0, $i1 = 0, $i2 = 0, $i3 = 0, $row = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $mask;
 $3 = $num;
 $4 = $0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = $1;
  $7 = ($6|0)==(0|0);
  if (!($7)) {
   $8 = $2;
   $9 = $8 & 16384;
   $10 = $1;
   $11 = (($10) + 184|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = $9 & $12;
   $14 = ($13|0)!=(0);
   if ($14) {
    $15 = $3;
    $16 = ($15|0)!=(-1);
    if ($16) {
     $17 = $1;
     $18 = (($17) + 56|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = ($19|0)!=(0|0);
     if ($20) {
      $21 = $3;
      $22 = $1;
      $23 = (($22) + 56|0);
      $24 = HEAP32[$23>>2]|0;
      $25 = (($24) + ($21<<4)|0);
      $26 = (($25) + 4|0);
      $27 = HEAP32[$26>>2]|0;
      $28 = ($27|0)!=(0|0);
      if ($28) {
       $29 = $0;
       $30 = $3;
       $31 = $1;
       $32 = (($31) + 56|0);
       $33 = HEAP32[$32>>2]|0;
       $34 = (($33) + ($30<<4)|0);
       $35 = (($34) + 4|0);
       $36 = HEAP32[$35>>2]|0;
       _png_free($29,$36);
       $37 = $3;
       $38 = $1;
       $39 = (($38) + 56|0);
       $40 = HEAP32[$39>>2]|0;
       $41 = (($40) + ($37<<4)|0);
       $42 = (($41) + 4|0);
       HEAP32[$42>>2] = 0;
      }
     }
    } else {
     $i = 0;
     while(1) {
      $43 = $i;
      $44 = $1;
      $45 = (($44) + 48|0);
      $46 = HEAP32[$45>>2]|0;
      $47 = ($43|0)<($46|0);
      if (!($47)) {
       break;
      }
      $48 = $0;
      $49 = $1;
      $50 = $i;
      _png_free_data($48,$49,16384,$50);
      $51 = $i;
      $52 = (($51) + 1)|0;
      $i = $52;
     }
     $53 = $0;
     $54 = $1;
     $55 = (($54) + 56|0);
     $56 = HEAP32[$55>>2]|0;
     _png_free($53,$56);
     $57 = $1;
     $58 = (($57) + 56|0);
     HEAP32[$58>>2] = 0;
     $59 = $1;
     $60 = (($59) + 48|0);
     HEAP32[$60>>2] = 0;
    }
   }
   $61 = $2;
   $62 = $61 & 8192;
   $63 = $1;
   $64 = (($63) + 184|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $62 & $65;
   $67 = ($66|0)!=(0);
   if ($67) {
    $68 = $0;
    $69 = $1;
    $70 = (($69) + 76|0);
    $71 = HEAP32[$70>>2]|0;
    _png_free($68,$71);
    $72 = $1;
    $73 = (($72) + 8|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = $74 & -17;
    HEAP32[$73>>2] = $75;
    $76 = $1;
    $77 = (($76) + 76|0);
    HEAP32[$77>>2] = 0;
   }
   $78 = $2;
   $79 = $78 & 256;
   $80 = $1;
   $81 = (($80) + 184|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = $79 & $82;
   $84 = ($83|0)!=(0);
   if ($84) {
    $85 = $1;
    $86 = (($85) + 8|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = $87 & -16385;
    HEAP32[$86>>2] = $88;
   }
   $89 = $2;
   $90 = $89 & 128;
   $91 = $1;
   $92 = (($91) + 184|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = $90 & $93;
   $95 = ($94|0)!=(0);
   if ($95) {
    $96 = $0;
    $97 = $1;
    $98 = (($97) + 160|0);
    $99 = HEAP32[$98>>2]|0;
    _png_free($96,$99);
    $100 = $0;
    $101 = $1;
    $102 = (($101) + 172|0);
    $103 = HEAP32[$102>>2]|0;
    _png_free($100,$103);
    $104 = $1;
    $105 = (($104) + 160|0);
    HEAP32[$105>>2] = 0;
    $106 = $1;
    $107 = (($106) + 172|0);
    HEAP32[$107>>2] = 0;
    $108 = $1;
    $109 = (($108) + 176|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = ($110|0)!=(0|0);
    if ($111) {
     $i1 = 0;
     while(1) {
      $112 = $i1;
      $113 = $1;
      $114 = (($113) + 181|0);
      $115 = HEAP8[$114>>0]|0;
      $116 = $115&255;
      $117 = ($112|0)<($116|0);
      if (!($117)) {
       break;
      }
      $118 = $0;
      $119 = $i1;
      $120 = $1;
      $121 = (($120) + 176|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = (($122) + ($119<<2)|0);
      $124 = HEAP32[$123>>2]|0;
      _png_free($118,$124);
      $125 = $i1;
      $126 = $1;
      $127 = (($126) + 176|0);
      $128 = HEAP32[$127>>2]|0;
      $129 = (($128) + ($125<<2)|0);
      HEAP32[$129>>2] = 0;
      $130 = $i1;
      $131 = (($130) + 1)|0;
      $i1 = $131;
     }
     $132 = $0;
     $133 = $1;
     $134 = (($133) + 176|0);
     $135 = HEAP32[$134>>2]|0;
     _png_free($132,$135);
     $136 = $1;
     $137 = (($136) + 176|0);
     HEAP32[$137>>2] = 0;
    }
    $138 = $1;
    $139 = (($138) + 8|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = $140 & -1025;
    HEAP32[$139>>2] = $141;
   }
   $142 = $2;
   $143 = $142 & 16;
   $144 = $1;
   $145 = (($144) + 184|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = $143 & $146;
   $148 = ($147|0)!=(0);
   if ($148) {
    $149 = $0;
    $150 = $1;
    $151 = (($150) + 196|0);
    $152 = HEAP32[$151>>2]|0;
    _png_free($149,$152);
    $153 = $0;
    $154 = $1;
    $155 = (($154) + 200|0);
    $156 = HEAP32[$155>>2]|0;
    _png_free($153,$156);
    $157 = $1;
    $158 = (($157) + 196|0);
    HEAP32[$158>>2] = 0;
    $159 = $1;
    $160 = (($159) + 200|0);
    HEAP32[$160>>2] = 0;
    $161 = $1;
    $162 = (($161) + 8|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = $163 & -4097;
    HEAP32[$162>>2] = $164;
   }
   $165 = $2;
   $166 = $165 & 32;
   $167 = $1;
   $168 = (($167) + 184|0);
   $169 = HEAP32[$168>>2]|0;
   $170 = $166 & $169;
   $171 = ($170|0)!=(0);
   if ($171) {
    $172 = $3;
    $173 = ($172|0)!=(-1);
    if ($173) {
     $174 = $1;
     $175 = (($174) + 212|0);
     $176 = HEAP32[$175>>2]|0;
     $177 = ($176|0)!=(0|0);
     if ($177) {
      $178 = $0;
      $179 = $3;
      $180 = $1;
      $181 = (($180) + 212|0);
      $182 = HEAP32[$181>>2]|0;
      $183 = (($182) + ($179<<4)|0);
      $184 = HEAP32[$183>>2]|0;
      _png_free($178,$184);
      $185 = $0;
      $186 = $3;
      $187 = $1;
      $188 = (($187) + 212|0);
      $189 = HEAP32[$188>>2]|0;
      $190 = (($189) + ($186<<4)|0);
      $191 = (($190) + 8|0);
      $192 = HEAP32[$191>>2]|0;
      _png_free($185,$192);
      $193 = $3;
      $194 = $1;
      $195 = (($194) + 212|0);
      $196 = HEAP32[$195>>2]|0;
      $197 = (($196) + ($193<<4)|0);
      HEAP32[$197>>2] = 0;
      $198 = $3;
      $199 = $1;
      $200 = (($199) + 212|0);
      $201 = HEAP32[$200>>2]|0;
      $202 = (($201) + ($198<<4)|0);
      $203 = (($202) + 8|0);
      HEAP32[$203>>2] = 0;
     }
    } else {
     $204 = $1;
     $205 = (($204) + 216|0);
     $206 = HEAP32[$205>>2]|0;
     $207 = ($206|0)!=(0);
     if ($207) {
      $i2 = 0;
      while(1) {
       $208 = $i2;
       $209 = $1;
       $210 = (($209) + 216|0);
       $211 = HEAP32[$210>>2]|0;
       $212 = ($208|0)<($211|0);
       if (!($212)) {
        break;
       }
       $213 = $0;
       $214 = $1;
       $215 = $i2;
       _png_free_data($213,$214,32,$215);
       $216 = $i2;
       $217 = (($216) + 1)|0;
       $i2 = $217;
      }
      $218 = $0;
      $219 = $1;
      $220 = (($219) + 212|0);
      $221 = HEAP32[$220>>2]|0;
      _png_free($218,$221);
      $222 = $1;
      $223 = (($222) + 212|0);
      HEAP32[$223>>2] = 0;
      $224 = $1;
      $225 = (($224) + 216|0);
      HEAP32[$225>>2] = 0;
     }
     $226 = $1;
     $227 = (($226) + 8|0);
     $228 = HEAP32[$227>>2]|0;
     $229 = $228 & -8193;
     HEAP32[$227>>2] = $229;
    }
   }
   $230 = $0;
   $231 = (($230) + 712|0);
   $232 = (($231) + 8|0);
   $233 = HEAP32[$232>>2]|0;
   $234 = ($233|0)!=(0|0);
   if ($234) {
    $235 = $0;
    $236 = $0;
    $237 = (($236) + 712|0);
    $238 = (($237) + 8|0);
    $239 = HEAP32[$238>>2]|0;
    _png_free($235,$239);
    $240 = $0;
    $241 = (($240) + 712|0);
    $242 = (($241) + 8|0);
    HEAP32[$242>>2] = 0;
   }
   $243 = $2;
   $244 = $243 & 512;
   $245 = $1;
   $246 = (($245) + 184|0);
   $247 = HEAP32[$246>>2]|0;
   $248 = $244 & $247;
   $249 = ($248|0)!=(0);
   if ($249) {
    $250 = $3;
    $251 = ($250|0)!=(-1);
    if ($251) {
     $252 = $1;
     $253 = (($252) + 188|0);
     $254 = HEAP32[$253>>2]|0;
     $255 = ($254|0)!=(0|0);
     if ($255) {
      $256 = $0;
      $257 = $3;
      $258 = $1;
      $259 = (($258) + 188|0);
      $260 = HEAP32[$259>>2]|0;
      $261 = (($260) + (($257*20)|0)|0);
      $262 = (($261) + 8|0);
      $263 = HEAP32[$262>>2]|0;
      _png_free($256,$263);
      $264 = $3;
      $265 = $1;
      $266 = (($265) + 188|0);
      $267 = HEAP32[$266>>2]|0;
      $268 = (($267) + (($264*20)|0)|0);
      $269 = (($268) + 8|0);
      HEAP32[$269>>2] = 0;
     }
    } else {
     $270 = $1;
     $271 = (($270) + 192|0);
     $272 = HEAP32[$271>>2]|0;
     $273 = ($272|0)!=(0);
     if ($273) {
      $i3 = 0;
      while(1) {
       $274 = $i3;
       $275 = $1;
       $276 = (($275) + 192|0);
       $277 = HEAP32[$276>>2]|0;
       $278 = ($274|0)<($277|0);
       if (!($278)) {
        break;
       }
       $279 = $0;
       $280 = $1;
       $281 = $i3;
       _png_free_data($279,$280,512,$281);
       $282 = $i3;
       $283 = (($282) + 1)|0;
       $i3 = $283;
      }
      $284 = $0;
      $285 = $1;
      $286 = (($285) + 188|0);
      $287 = HEAP32[$286>>2]|0;
      _png_free($284,$287);
      $288 = $1;
      $289 = (($288) + 188|0);
      HEAP32[$289>>2] = 0;
      $290 = $1;
      $291 = (($290) + 192|0);
      HEAP32[$291>>2] = 0;
     }
    }
   }
   $292 = $2;
   $293 = $292 & 8;
   $294 = $1;
   $295 = (($294) + 184|0);
   $296 = HEAP32[$295>>2]|0;
   $297 = $293 & $296;
   $298 = ($297|0)!=(0);
   if ($298) {
    $299 = $0;
    $300 = $1;
    $301 = (($300) + 124|0);
    $302 = HEAP32[$301>>2]|0;
    _png_free($299,$302);
    $303 = $1;
    $304 = (($303) + 124|0);
    HEAP32[$304>>2] = 0;
    $305 = $1;
    $306 = (($305) + 8|0);
    $307 = HEAP32[$306>>2]|0;
    $308 = $307 & -65;
    HEAP32[$306>>2] = $308;
   }
   $309 = $2;
   $310 = $309 & 4096;
   $311 = $1;
   $312 = (($311) + 184|0);
   $313 = HEAP32[$312>>2]|0;
   $314 = $310 & $313;
   $315 = ($314|0)!=(0);
   if ($315) {
    $316 = $0;
    $317 = $1;
    $318 = (($317) + 16|0);
    $319 = HEAP32[$318>>2]|0;
    _png_zfree($316,$319);
    $320 = $1;
    $321 = (($320) + 16|0);
    HEAP32[$321>>2] = 0;
    $322 = $1;
    $323 = (($322) + 8|0);
    $324 = HEAP32[$323>>2]|0;
    $325 = $324 & -9;
    HEAP32[$323>>2] = $325;
    $326 = $1;
    $327 = (($326) + 20|0);
    HEAP16[$327>>1] = 0;
   }
   $328 = $2;
   $329 = $328 & 64;
   $330 = $1;
   $331 = (($330) + 184|0);
   $332 = HEAP32[$331>>2]|0;
   $333 = $329 & $332;
   $334 = ($333|0)!=(0);
   if ($334) {
    $335 = $1;
    $336 = (($335) + 248|0);
    $337 = HEAP32[$336>>2]|0;
    $338 = ($337|0)!=(0|0);
    if ($338) {
     $row = 0;
     while(1) {
      $339 = $row;
      $340 = $1;
      $341 = (($340) + 4|0);
      $342 = HEAP32[$341>>2]|0;
      $343 = ($339|0)<($342|0);
      if (!($343)) {
       break;
      }
      $344 = $0;
      $345 = $row;
      $346 = $1;
      $347 = (($346) + 248|0);
      $348 = HEAP32[$347>>2]|0;
      $349 = (($348) + ($345<<2)|0);
      $350 = HEAP32[$349>>2]|0;
      _png_free($344,$350);
      $351 = $row;
      $352 = $1;
      $353 = (($352) + 248|0);
      $354 = HEAP32[$353>>2]|0;
      $355 = (($354) + ($351<<2)|0);
      HEAP32[$355>>2] = 0;
      $356 = $row;
      $357 = (($356) + 1)|0;
      $row = $357;
     }
     $358 = $0;
     $359 = $1;
     $360 = (($359) + 248|0);
     $361 = HEAP32[$360>>2]|0;
     _png_free($358,$361);
     $362 = $1;
     $363 = (($362) + 248|0);
     HEAP32[$363>>2] = 0;
    }
    $364 = $1;
    $365 = (($364) + 8|0);
    $366 = HEAP32[$365>>2]|0;
    $367 = $366 & -32769;
    HEAP32[$365>>2] = $367;
   }
   $368 = $3;
   $369 = ($368|0)==(-1);
   if ($369) {
    $370 = $2;
    $371 = $370 ^ -1;
    $372 = $1;
    $373 = (($372) + 184|0);
    $374 = HEAP32[$373>>2]|0;
    $375 = $374 & $371;
    HEAP32[$373>>2] = $375;
    STACKTOP = sp;return;
   } else {
    $376 = $2;
    $377 = $376 & -16929;
    $378 = $377 ^ -1;
    $379 = $1;
    $380 = (($379) + 184|0);
    $381 = HEAP32[$380>>2]|0;
    $382 = $381 & $378;
    HEAP32[$380>>2] = $382;
    STACKTOP = sp;return;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_get_io_ptr($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $png_ptr;
 $2 = $1;
 $3 = ($2|0)==(0|0);
 if ($3) {
  $0 = 0;
 } else {
  $4 = $1;
  $5 = (($4) + 176|0);
  $6 = HEAP32[$5>>2]|0;
  $0 = $6;
 }
 $7 = $0;
 STACKTOP = sp;return ($7|0);
}
function _png_handle_as_unknown($png_ptr,$chunk_name) {
 $png_ptr = $png_ptr|0;
 $chunk_name = $chunk_name|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $png_ptr;
 $2 = $chunk_name;
 $3 = $1;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $2;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $1;
   $8 = (($7) + 636|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)<=(0);
   if (!($10)) {
    $11 = $1;
    $12 = (($11) + 640|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = $1;
    $15 = (($14) + 636|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($16*5)|0;
    $18 = (($13) + ($17)|0);
    $19 = (($18) + -5|0);
    $p = $19;
    $20 = $1;
    $21 = (($20) + 636|0);
    $22 = HEAP32[$21>>2]|0;
    $i = $22;
    while(1) {
     $23 = $i;
     $24 = ($23|0)!=(0);
     if (!($24)) {
      label = 11;
      break;
     }
     $25 = $2;
     $26 = $p;
     $27 = (_memcmp($25,$26,4)|0);
     $28 = ($27|0)!=(0);
     if (!($28)) {
      label = 8;
      break;
     }
     $33 = $i;
     $34 = (($33) + -1)|0;
     $i = $34;
     $35 = $p;
     $36 = (($35) + -5|0);
     $p = $36;
    }
    if ((label|0) == 8) {
     $29 = $p;
     $30 = (($29) + 4|0);
     $31 = HEAP8[$30>>0]|0;
     $32 = $31&255;
     $0 = $32;
     $37 = $0;
     STACKTOP = sp;return ($37|0);
    }
    else if ((label|0) == 11) {
     $0 = 0;
     $37 = $0;
     STACKTOP = sp;return ($37|0);
    }
   }
  }
 }
 $0 = 0;
 $37 = $0;
 STACKTOP = sp;return ($37|0);
}
function _png_error($png_ptr,$error_message) {
 $png_ptr = $png_ptr|0;
 $error_message = $error_message|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $msg = 0, $offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $msg = sp + 16|0;
 $0 = $png_ptr;
 $1 = $error_message;
 $2 = $0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = $0;
  $5 = (($4) + 200|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = $6 & 786432;
  $8 = ($7|0)!=(0);
  if ($8) {
   $9 = $1;
   $10 = HEAP8[$9>>0]|0;
   $11 = $10 << 24 >> 24;
   $12 = ($11|0)==(35);
   if ($12) {
    $offset = 1;
    while(1) {
     $13 = $offset;
     $14 = ($13|0)<(15);
     if (!($14)) {
      break;
     }
     $15 = $1;
     $16 = $offset;
     $17 = (($15) + ($16)|0);
     $18 = HEAP8[$17>>0]|0;
     $19 = $18 << 24 >> 24;
     $20 = ($19|0)==(32);
     if ($20) {
      label = 7;
      break;
     }
     $21 = $offset;
     $22 = (($21) + 1)|0;
     $offset = $22;
    }
    if ((label|0) == 7) {
    }
    $23 = $0;
    $24 = (($23) + 200|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = $25 & 524288;
    $27 = ($26|0)!=(0);
    if ($27) {
     $i = 0;
     while(1) {
      $28 = $i;
      $29 = $offset;
      $30 = (($29) - 1)|0;
      $31 = ($28|0)<($30|0);
      if (!($31)) {
       break;
      }
      $32 = $i;
      $33 = (($32) + 1)|0;
      $34 = $1;
      $35 = (($34) + ($33)|0);
      $36 = HEAP8[$35>>0]|0;
      $37 = $i;
      $38 = (($msg) + ($37)|0);
      HEAP8[$38>>0] = $36;
      $39 = $i;
      $40 = (($39) + 1)|0;
      $i = $40;
     }
     $41 = $i;
     $42 = (($msg) + ($41)|0);
     HEAP8[$42>>0] = 0;
     $1 = $msg;
    } else {
     $43 = $offset;
     $44 = $1;
     $45 = (($44) + ($43)|0);
     $1 = $45;
    }
   } else {
    $46 = $0;
    $47 = (($46) + 200|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = $48 & 524288;
    $50 = ($49|0)!=(0);
    if ($50) {
     HEAP8[$msg>>0] = 48;
     $51 = (($msg) + 1|0);
     HEAP8[$51>>0] = 0;
     $1 = $msg;
    }
   }
  }
 }
 $52 = $0;
 $53 = ($52|0)!=(0|0);
 if (!($53)) {
  $63 = $0;
  $64 = $1;
  _png_default_error($63,$64);
  STACKTOP = sp;return;
 }
 $54 = $0;
 $55 = (($54) + 156|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = ($56|0)!=(0|0);
 if (!($57)) {
  $63 = $0;
  $64 = $1;
  _png_default_error($63,$64);
  STACKTOP = sp;return;
 }
 $58 = $0;
 $59 = (($58) + 156|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = $0;
 $62 = $1;
 FUNCTION_TABLE_vii[$60 & 255]($61,$62);
 $63 = $0;
 $64 = $1;
 _png_default_error($63,$64);
 STACKTOP = sp;return;
}
function _png_default_error($png_ptr,$error_message) {
 $png_ptr = $png_ptr|0;
 $error_message = $error_message|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $error_number = 0, $offset = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $error_number = sp + 32|0;
 $0 = $png_ptr;
 $1 = $error_message;
 $2 = $1;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(35);
 if ($5) {
  $offset = 0;
  while(1) {
   $6 = $offset;
   $7 = ($6|0)<(15);
   if (!($7)) {
    break;
   }
   $8 = $1;
   $9 = $offset;
   $10 = (($8) + ($9)|0);
   $11 = (($10) + 1|0);
   $12 = HEAP8[$11>>0]|0;
   $13 = $offset;
   $14 = (($error_number) + ($13)|0);
   HEAP8[$14>>0] = $12;
   $15 = $1;
   $16 = $offset;
   $17 = (($15) + ($16)|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   $20 = ($19|0)==(32);
   if ($20) {
    label = 5;
    break;
   }
   $21 = $offset;
   $22 = (($21) + 1)|0;
   $offset = $22;
  }
  if ((label|0) == 5) {
  }
  $23 = $offset;
  $24 = ($23|0)>(1);
  if ($24) {
   $25 = $offset;
   $26 = ($25|0)<(15);
   if ($26) {
    $27 = $offset;
    $28 = (($27) - 1)|0;
    $29 = (($error_number) + ($28)|0);
    HEAP8[$29>>0] = 0;
    $30 = HEAP32[_stderr>>2]|0;
    $31 = $1;
    $32 = $offset;
    $33 = (($31) + ($32)|0);
    HEAP32[$vararg_buffer>>2] = $error_number;
    $vararg_ptr1 = (($vararg_buffer) + 4|0);
    HEAP32[$vararg_ptr1>>2] = $33;
    (_fprintf(($30|0),(57584|0),($vararg_buffer|0))|0);
   } else {
    label = 11;
   }
  } else {
   label = 11;
  }
  if ((label|0) == 11) {
   $34 = HEAP32[_stderr>>2]|0;
   $35 = $1;
   $36 = $offset;
   HEAP32[$vararg_buffer2>>2] = $35;
   $vararg_ptr5 = (($vararg_buffer2) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $36;
   (_fprintf(($34|0),(57616|0),($vararg_buffer2|0))|0);
  }
 } else {
  $37 = HEAP32[_stderr>>2]|0;
  $38 = $1;
  HEAP32[$vararg_buffer6>>2] = $38;
  (_fprintf(($37|0),(57648|0),($vararg_buffer6|0))|0);
 }
 $39 = $0;
 $40 = ($39|0)!=(0|0);
 if ($40) {
  $41 = $0;
  _longjmp(($41|0),1);
  // unreachable;
 } else {
  STACKTOP = sp;return;
 }
}
function _png_warning($png_ptr,$warning_message) {
 $png_ptr = $png_ptr|0;
 $warning_message = $warning_message|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $warning_message;
 $offset = 0;
 $2 = $0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $36 = $0;
  $37 = $1;
  $38 = $offset;
  $39 = (($37) + ($38)|0);
  _png_default_warning($36,$39);
  STACKTOP = sp;return;
 }
 $4 = $0;
 $5 = (($4) + 200|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 786432;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = $1;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(35);
  if ($12) {
   $offset = 1;
   while(1) {
    $13 = $offset;
    $14 = ($13|0)<(15);
    if (!($14)) {
     break;
    }
    $15 = $1;
    $16 = $offset;
    $17 = (($15) + ($16)|0);
    $18 = HEAP8[$17>>0]|0;
    $19 = $18 << 24 >> 24;
    $20 = ($19|0)==(32);
    if ($20) {
     label = 7;
     break;
    }
    $21 = $offset;
    $22 = (($21) + 1)|0;
    $offset = $22;
   }
   if ((label|0) == 7) {
   }
  }
 }
 $23 = $0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = $0;
  $26 = (($25) + 160|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0|0);
  if ($28) {
   $29 = $0;
   $30 = (($29) + 160|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $0;
   $33 = $1;
   $34 = $offset;
   $35 = (($33) + ($34)|0);
   FUNCTION_TABLE_vii[$31 & 255]($32,$35);
  }
 }
 STACKTOP = sp;return;
}
function _png_default_warning($png_ptr,$warning_message) {
 $png_ptr = $png_ptr|0;
 $warning_message = $warning_message|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $offset = 0;
 var $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $warning_number = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $warning_number = sp + 32|0;
 $0 = $png_ptr;
 $1 = $warning_message;
 $2 = $1;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(35);
 if (!($5)) {
  $36 = HEAP32[_stderr>>2]|0;
  $37 = $1;
  HEAP32[$vararg_buffer5>>2] = $37;
  (_fprintf(($36|0),(57560|0),($vararg_buffer5|0))|0);
  $38 = $0;
  $0 = $38;
  STACKTOP = sp;return;
 }
 $offset = 0;
 while(1) {
  $6 = $offset;
  $7 = ($6|0)<(15);
  if (!($7)) {
   break;
  }
  $8 = $1;
  $9 = $offset;
  $10 = (($8) + ($9)|0);
  $11 = (($10) + 1|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $offset;
  $14 = (($warning_number) + ($13)|0);
  HEAP8[$14>>0] = $12;
  $15 = $1;
  $16 = $offset;
  $17 = (($15) + ($16)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = ($19|0)==(32);
  if ($20) {
   label = 5;
   break;
  }
  $21 = $offset;
  $22 = (($21) + 1)|0;
  $offset = $22;
 }
 if ((label|0) == 5) {
 }
 $23 = $offset;
 $24 = ($23|0)>(1);
 if ($24) {
  $25 = $offset;
  $26 = ($25|0)<(15);
  if ($26) {
   $27 = $offset;
   $28 = (($27) - 1)|0;
   $29 = (($warning_number) + ($28)|0);
   HEAP8[$29>>0] = 0;
   $30 = HEAP32[_stderr>>2]|0;
   $31 = $1;
   $32 = $offset;
   $33 = (($31) + ($32)|0);
   HEAP32[$vararg_buffer>>2] = $warning_number;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $33;
   (_fprintf(($30|0),(57528|0),($vararg_buffer|0))|0);
  } else {
   label = 11;
  }
 } else {
  label = 11;
 }
 if ((label|0) == 11) {
  $34 = HEAP32[_stderr>>2]|0;
  $35 = $1;
  HEAP32[$vararg_buffer2>>2] = $35;
  (_fprintf(($34|0),(57560|0),($vararg_buffer2|0))|0);
 }
 $38 = $0;
 $0 = $38;
 STACKTOP = sp;return;
}
function _png_chunk_error($png_ptr,$error_message) {
 $png_ptr = $png_ptr|0;
 $error_message = $error_message|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $msg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $msg = sp + 8|0;
 $0 = $png_ptr;
 $1 = $error_message;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  $4 = $0;
  $5 = $1;
  _png_error($4,$5);
  STACKTOP = sp;return;
 } else {
  $6 = $0;
  $7 = $1;
  _png_format_buffer($6,$msg,$7);
  $8 = $0;
  _png_error($8,$msg);
  STACKTOP = sp;return;
 }
}
function _png_format_buffer($png_ptr,$buffer,$error_message) {
 $png_ptr = $png_ptr|0;
 $buffer = $buffer|0;
 $error_message = $error_message|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $c = 0, $iin = 0, $iout = 0, dest = 0, label = 0, sp = 0, src = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $buffer;
 $2 = $error_message;
 $iout = 0;
 $iin = 0;
 while(1) {
  $3 = $iin;
  $4 = ($3|0)<(4);
  if (!($4)) {
   break;
  }
  $5 = $iin;
  $6 = (($5) + 1)|0;
  $iin = $6;
  $7 = $0;
  $8 = (($7) + 376|0);
  $9 = (($8) + ($5)|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $c = $11;
  $12 = $c;
  $13 = ($12|0)<(65);
  do {
   if ($13) {
    label = 7;
   } else {
    $14 = $c;
    $15 = ($14|0)>(122);
    if ($15) {
     label = 7;
    } else {
     $16 = $c;
     $17 = ($16|0)>(90);
     if ($17) {
      $18 = $c;
      $19 = ($18|0)<(97);
      if ($19) {
       label = 7;
       break;
      }
     }
     $45 = $c;
     $46 = $45&255;
     $47 = $iout;
     $48 = (($47) + 1)|0;
     $iout = $48;
     $49 = $1;
     $50 = (($49) + ($47)|0);
     HEAP8[$50>>0] = $46;
    }
   }
  } while(0);
  if ((label|0) == 7) {
   label = 0;
   $20 = $iout;
   $21 = (($20) + 1)|0;
   $iout = $21;
   $22 = $1;
   $23 = (($22) + ($20)|0);
   HEAP8[$23>>0] = 91;
   $24 = $c;
   $25 = $24 & 240;
   $26 = $25 >> 4;
   $27 = (57672 + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $iout;
   $30 = (($29) + 1)|0;
   $iout = $30;
   $31 = $1;
   $32 = (($31) + ($29)|0);
   HEAP8[$32>>0] = $28;
   $33 = $c;
   $34 = $33 & 15;
   $35 = (57672 + ($34)|0);
   $36 = HEAP8[$35>>0]|0;
   $37 = $iout;
   $38 = (($37) + 1)|0;
   $iout = $38;
   $39 = $1;
   $40 = (($39) + ($37)|0);
   HEAP8[$40>>0] = $36;
   $41 = $iout;
   $42 = (($41) + 1)|0;
   $iout = $42;
   $43 = $1;
   $44 = (($43) + ($41)|0);
   HEAP8[$44>>0] = 93;
  }
 }
 $51 = $2;
 $52 = ($51|0)==(0|0);
 if ($52) {
  $53 = $iout;
  $54 = $1;
  $55 = (($54) + ($53)|0);
  HEAP8[$55>>0] = 0;
  STACKTOP = sp;return;
 } else {
  $56 = $iout;
  $57 = (($56) + 1)|0;
  $iout = $57;
  $58 = $1;
  $59 = (($58) + ($56)|0);
  HEAP8[$59>>0] = 58;
  $60 = $iout;
  $61 = (($60) + 1)|0;
  $iout = $61;
  $62 = $1;
  $63 = (($62) + ($60)|0);
  HEAP8[$63>>0] = 32;
  $64 = $1;
  $65 = $iout;
  $66 = (($64) + ($65)|0);
  $67 = $2;
  dest=$66+0|0; src=$67+0|0; stop=dest+64|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $68 = $iout;
  $69 = (($68) + 64)|0;
  $70 = (($69) - 1)|0;
  $71 = $1;
  $72 = (($71) + ($70)|0);
  HEAP8[$72>>0] = 0;
  STACKTOP = sp;return;
 }
}
function _png_chunk_warning($png_ptr,$warning_message) {
 $png_ptr = $png_ptr|0;
 $warning_message = $warning_message|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $msg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $msg = sp + 8|0;
 $0 = $png_ptr;
 $1 = $warning_message;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  $4 = $0;
  $5 = $1;
  _png_warning($4,$5);
  STACKTOP = sp;return;
 } else {
  $6 = $0;
  $7 = $1;
  _png_format_buffer($6,$msg,$7);
  $8 = $0;
  _png_warning($8,$msg);
  STACKTOP = sp;return;
 }
}
function _png_set_error_fn($png_ptr,$error_ptr,$error_fn,$warning_fn) {
 $png_ptr = $png_ptr|0;
 $error_ptr = $error_ptr|0;
 $error_fn = $error_fn|0;
 $warning_fn = $warning_fn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $error_ptr;
 $2 = $error_fn;
 $3 = $warning_fn;
 $4 = $0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 } else {
  $6 = $1;
  $7 = $0;
  $8 = (($7) + 164|0);
  HEAP32[$8>>2] = $6;
  $9 = $2;
  $10 = $0;
  $11 = (($10) + 156|0);
  HEAP32[$11>>2] = $9;
  $12 = $3;
  $13 = $0;
  $14 = (($13) + 160|0);
  HEAP32[$14>>2] = $12;
  STACKTOP = sp;return;
 }
}
function _png_create_struct($type) {
 $type = $type|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $type;
 $1 = $0;
 $2 = (_png_create_struct_2($1,0,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _png_create_struct_2($type,$malloc_fn,$mem_ptr) {
 $type = $type|0;
 $malloc_fn = $malloc_fn|0;
 $mem_ptr = $mem_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dummy_struct = 0, $png_ptr = 0, $size = 0, $struct_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 768|0;
 $dummy_struct = sp + 4|0;
 $1 = $type;
 $2 = $malloc_fn;
 $3 = $mem_ptr;
 $4 = $1;
 $5 = ($4|0)==(2);
 do {
  if ($5) {
   $size = 288;
  } else {
   $6 = $1;
   $7 = ($6|0)==(1);
   if ($7) {
    $size = 732;
    break;
   } else {
    $0 = 0;
    $29 = $0;
    STACKTOP = sp;return ($29|0);
   }
  }
 } while(0);
 $8 = $2;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $png_ptr = $dummy_struct;
  $10 = $3;
  $11 = $png_ptr;
  $12 = (($11) + 672|0);
  HEAP32[$12>>2] = $10;
  $13 = $2;
  $14 = $png_ptr;
  $15 = $size;
  $16 = (FUNCTION_TABLE_iii[$13 & 255]($14,$15)|0);
  $struct_ptr = $16;
  $17 = $struct_ptr;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = $struct_ptr;
   $20 = $size;
   _memset(($19|0),0,($20|0))|0;
  }
  $21 = $struct_ptr;
  $0 = $21;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 } else {
  $22 = $size;
  $23 = (_malloc($22)|0);
  $struct_ptr = $23;
  $24 = $struct_ptr;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = $struct_ptr;
   $27 = $size;
   _memset(($26|0),0,($27|0))|0;
  }
  $28 = $struct_ptr;
  $0 = $28;
  $29 = $0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _png_destroy_struct($struct_ptr) {
 $struct_ptr = $struct_ptr|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $struct_ptr;
 $1 = $0;
 _png_destroy_struct_2($1,0,0);
 STACKTOP = sp;return;
}
function _png_destroy_struct_2($struct_ptr,$free_fn,$mem_ptr) {
 $struct_ptr = $struct_ptr|0;
 $free_fn = $free_fn|0;
 $mem_ptr = $mem_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dummy_struct = 0, $png_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 752|0;
 $dummy_struct = sp + 4|0;
 $0 = $struct_ptr;
 $1 = $free_fn;
 $2 = $mem_ptr;
 $3 = $0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = $1;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $png_ptr = $dummy_struct;
  $7 = $2;
  $8 = $png_ptr;
  $9 = (($8) + 672|0);
  HEAP32[$9>>2] = $7;
  $10 = $1;
  $11 = $png_ptr;
  $12 = $0;
  FUNCTION_TABLE_vii[$10 & 255]($11,$12);
  STACKTOP = sp;return;
 } else {
  $13 = $0;
  _free($13);
  STACKTOP = sp;return;
 }
}
function _png_malloc($png_ptr,$size) {
 $png_ptr = $png_ptr|0;
 $size = $size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $png_ptr;
 $2 = $size;
 $3 = $1;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $2;
  $6 = ($5|0)==(0);
  if (!($6)) {
   $7 = $1;
   $8 = (($7) + 676|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    $11 = $1;
    $12 = (($11) + 676|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = $1;
    $15 = $2;
    $16 = (FUNCTION_TABLE_iii[$13 & 255]($14,$15)|0);
    $ret = $16;
   } else {
    $17 = $1;
    $18 = $2;
    $19 = (_png_malloc_default($17,$18)|0);
    $ret = $19;
   }
   $20 = $ret;
   $21 = ($20|0)==(0|0);
   if ($21) {
    $22 = $1;
    $23 = (($22) + 200|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $24 & 1048576;
    $26 = ($25|0)==(0);
    if ($26) {
     $27 = $1;
     _png_error($27,57688);
    }
   }
   $28 = $ret;
   $0 = $28;
   $29 = $0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $0 = 0;
 $29 = $0;
 STACKTOP = sp;return ($29|0);
}
function _png_malloc_default($png_ptr,$size) {
 $png_ptr = $png_ptr|0;
 $size = $size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $png_ptr;
 $2 = $size;
 $3 = $1;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $2;
  $6 = ($5|0)==(0);
  if (!($6)) {
   $7 = $2;
   $8 = $2;
   $9 = ($7|0)!=($8|0);
   if ($9) {
    $ret = 0;
   } else {
    $10 = $2;
    $11 = (_malloc($10)|0);
    $ret = $11;
   }
   $12 = $ret;
   $0 = $12;
   $13 = $0;
   STACKTOP = sp;return ($13|0);
  }
 }
 $0 = 0;
 $13 = $0;
 STACKTOP = sp;return ($13|0);
}
function _png_free($png_ptr,$ptr) {
 $png_ptr = $png_ptr|0;
 $ptr = $ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $ptr;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = $1;
  $5 = ($4|0)==(0|0);
  if (!($5)) {
   $6 = $0;
   $7 = (($6) + 680|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = ($8|0)!=(0|0);
   if ($9) {
    $10 = $0;
    $11 = (($10) + 680|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = $0;
    $14 = $1;
    FUNCTION_TABLE_vii[$12 & 255]($13,$14);
    STACKTOP = sp;return;
   } else {
    $15 = $0;
    $16 = $1;
    _png_free_default($15,$16);
    STACKTOP = sp;return;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_free_default($png_ptr,$ptr) {
 $png_ptr = $png_ptr|0;
 $ptr = $ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $ptr;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = $1;
  $5 = ($4|0)==(0|0);
  if (!($5)) {
   $6 = $1;
   _free($6);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_malloc_warn($png_ptr,$size) {
 $png_ptr = $png_ptr|0;
 $size = $size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ptr = 0, $save_flags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $png_ptr;
 $2 = $size;
 $3 = $1;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $0 = 0;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 } else {
  $5 = $1;
  $6 = (($5) + 200|0);
  $7 = HEAP32[$6>>2]|0;
  $save_flags = $7;
  $8 = $1;
  $9 = (($8) + 200|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 | 1048576;
  HEAP32[$9>>2] = $11;
  $12 = $1;
  $13 = $2;
  $14 = (_png_malloc($12,$13)|0);
  $ptr = $14;
  $15 = $save_flags;
  $16 = $1;
  $17 = (($16) + 200|0);
  HEAP32[$17>>2] = $15;
  $18 = $ptr;
  $0 = $18;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _png_memcpy_check($png_ptr,$s1,$s2,$length) {
 $png_ptr = $png_ptr|0;
 $s1 = $s1|0;
 $s2 = $s2|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $size = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $s1;
 $2 = $s2;
 $3 = $length;
 $4 = $3;
 $size = $4;
 $5 = $size;
 $6 = $3;
 $7 = ($5|0)!=($6|0);
 if ($7) {
  $8 = $0;
  _png_error($8,57704);
 }
 $9 = $1;
 $10 = $2;
 $11 = $size;
 _memcpy(($9|0),($10|0),($11|0))|0;
 STACKTOP = sp;return ($9|0);
}
function _png_memset_check($png_ptr,$s1,$value,$length) {
 $png_ptr = $png_ptr|0;
 $s1 = $s1|0;
 $value = $value|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $size = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $s1;
 $2 = $value;
 $3 = $length;
 $4 = $3;
 $size = $4;
 $5 = $size;
 $6 = $3;
 $7 = ($5|0)!=($6|0);
 if ($7) {
  $8 = $0;
  _png_error($8,57736);
 }
 $9 = $1;
 $10 = $2;
 $11 = $10&255;
 $12 = $size;
 _memset(($9|0),($11|0),($12|0))|0;
 STACKTOP = sp;return ($9|0);
}
function _png_set_mem_fn($png_ptr,$mem_ptr,$malloc_fn,$free_fn) {
 $png_ptr = $png_ptr|0;
 $mem_ptr = $mem_ptr|0;
 $malloc_fn = $malloc_fn|0;
 $free_fn = $free_fn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $mem_ptr;
 $2 = $malloc_fn;
 $3 = $free_fn;
 $4 = $0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = $1;
 $7 = $0;
 $8 = (($7) + 672|0);
 HEAP32[$8>>2] = $6;
 $9 = $2;
 $10 = $0;
 $11 = (($10) + 676|0);
 HEAP32[$11>>2] = $9;
 $12 = $3;
 $13 = $0;
 $14 = (($13) + 680|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _png_create_read_struct($user_png_ver,$error_ptr,$error_fn,$warn_fn) {
 $user_png_ver = $user_png_ver|0;
 $error_ptr = $error_ptr|0;
 $error_fn = $error_fn|0;
 $warn_fn = $warn_fn|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $user_png_ver;
 $1 = $error_ptr;
 $2 = $error_fn;
 $3 = $warn_fn;
 $4 = $0;
 $5 = $1;
 $6 = $2;
 $7 = $3;
 $8 = (_png_create_read_struct_2($4,$5,$6,$7,0,0,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _png_create_read_struct_2($user_png_ver,$error_ptr,$error_fn,$warn_fn,$mem_ptr,$malloc_fn,$free_fn) {
 $user_png_ver = $user_png_ver|0;
 $error_ptr = $error_ptr|0;
 $error_fn = $error_fn|0;
 $warn_fn = $warn_fn|0;
 $mem_ptr = $mem_ptr|0;
 $malloc_fn = $malloc_fn|0;
 $free_fn = $free_fn|0;
 var $$0 = 0, $$reg2mem75$0 = 0, $$reg2mem77$0 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0;
 var $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i$0 = 0, $msg = 0, $png_ptr$0 = 0, $png_ptr$0$phi = 0, $png_ptr$1 = 0, $png_ptr$1$phi = 0, $vararg_buffer = 0, $vararg_buffer79 = 0;
 var _setjmpTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer79 = sp + 8|0;
 $vararg_buffer = sp;
 _setjmpTable = STACKTOP; STACKTOP=(STACKTOP+168)|0;HEAP32[_setjmpTable>>2]=0;
 $msg = sp + 16|0;
 __THREW__ = 0;
 $0 = (invoke_iiii(132,1,($malloc_fn|0),($mem_ptr|0))|0);
 $1 = __THREW__; __THREW__ = 0;
 if ((($1|0) != 0) & ((threwValue|0) != 0)) { $2 = _testSetjmp(HEAP32[$1>>2]|0, _setjmpTable)|0; if (($2|0) == 0) { _longjmp($1|0, threwValue|0); } tempRet0 = threwValue; } else { $2 = -1; };
 $3 = tempRet0;
 do {
  if ((($2|0) == 2)) {
   $$reg2mem77$0 = $3;$png_ptr$0 = 0;
  } else if ((($2|0) == 1)) {
   $$reg2mem75$0 = $3;$png_ptr$1 = 0;
   label = 46;
  } else {
   $4 = ($0|0)==(0|0);
   if ($4) {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   } else {
    $5 = (($0) + 704|0);
    HEAP32[$5>>2] = 1000000;
    $6 = (($0) + 708|0);
    HEAP32[$6>>2] = 1000000;
    _saveSetjmp($0,2,_setjmpTable|0)|0;
    __THREW__ = 0;
    $7 = __THREW__; __THREW__ = 0;
    if ((($7|0) != 0) & ((threwValue|0) != 0)) { $8 = _testSetjmp(HEAP32[$7>>2]|0, _setjmpTable)|0; if (($8|0) == 0) { _longjmp($7|0, threwValue|0); } tempRet0 = threwValue; } else { $8 = -1; };
    $9 = tempRet0;
    if ((($8|0) == 2)) {
     $$reg2mem77$0 = $9;$png_ptr$0 = $0;
     break;
    } else if ((($8|0) == 1)) {
     $$reg2mem75$0 = $9;$png_ptr$1 = $0;
     label = 46;
     break;
    }
    $$reg2mem77$0 = 0;$png_ptr$0 = $0;
    break;
   }
  }
 } while(0);
 while(1) {
  if ((label|0) == 46) {
   label = 0;
   $125 = ($$reg2mem75$0|0)!=(0);
   if (!($125)) {
    label = 49;
    break;
   }
   __THREW__ = 0;
   invoke_v(144);
   $126 = __THREW__; __THREW__ = 0;
   if ((($126|0) != 0) & ((threwValue|0) != 0)) { $127 = _testSetjmp(HEAP32[$126>>2]|0, _setjmpTable)|0; if (($127|0) == 0) { _longjmp($126|0, threwValue|0); } tempRet0 = threwValue; } else { $127 = -1; };
   $128 = tempRet0;
   if ((($127|0) == 2)) {
    $$reg2mem77$0 = $128;$png_ptr$0 = $png_ptr$1;
   } else if ((($127|0) == 1)) {
    $png_ptr$1$phi = $png_ptr$1;$$reg2mem75$0 = $128;$png_ptr$1 = $png_ptr$1$phi;
    label = 46;
    continue;
   } else {
    label = 48;
    break;
   }
  }
  $10 = ($$reg2mem77$0|0)!=(0);
  if ($10) {
   $11 = (($png_ptr$0) + 264|0);
   $12 = HEAP32[$11>>2]|0;
   __THREW__ = 0;
   invoke_vii(133,($png_ptr$0|0),($12|0));
   $13 = __THREW__; __THREW__ = 0;
   if ((($13|0) != 0) & ((threwValue|0) != 0)) { $14 = _testSetjmp(HEAP32[$13>>2]|0, _setjmpTable)|0; if (($14|0) == 0) { _longjmp($13|0, threwValue|0); } tempRet0 = threwValue; } else { $14 = -1; };
   $15 = tempRet0;
   if ((($14|0) == 2)) {
    $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $15;$png_ptr$0 = $png_ptr$0$phi;
    continue;
   } else if ((($14|0) == 1)) {
    $$reg2mem75$0 = $15;$png_ptr$1 = $png_ptr$0;
    label = 46;
    continue;
   }
   $16 = (($png_ptr$0) + 264|0);
   HEAP32[$16>>2] = 0;
   __THREW__ = 0;
   invoke_viii(134,($png_ptr$0|0),($free_fn|0),($mem_ptr|0));
   $17 = __THREW__; __THREW__ = 0;
   if ((($17|0) != 0) & ((threwValue|0) != 0)) { $18 = _testSetjmp(HEAP32[$17>>2]|0, _setjmpTable)|0; if (($18|0) == 0) { _longjmp($17|0, threwValue|0); } tempRet0 = threwValue; } else { $18 = -1; };
   $19 = tempRet0;
   if ((($18|0) == 2)) {
    $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $19;$png_ptr$0 = $png_ptr$0$phi;
    continue;
   } else if ((($18|0) == 1)) {
    $$reg2mem75$0 = $19;$png_ptr$1 = $png_ptr$0;
    label = 46;
    continue;
   } else {
    label = 9;
    break;
   }
  }
  __THREW__ = 0;
  invoke_viiii(135,($png_ptr$0|0),($mem_ptr|0),($malloc_fn|0),($free_fn|0));
  $20 = __THREW__; __THREW__ = 0;
  if ((($20|0) != 0) & ((threwValue|0) != 0)) { $21 = _testSetjmp(HEAP32[$20>>2]|0, _setjmpTable)|0; if (($21|0) == 0) { _longjmp($20|0, threwValue|0); } tempRet0 = threwValue; } else { $21 = -1; };
  $22 = tempRet0;
  if ((($21|0) == 2)) {
   $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $22;$png_ptr$0 = $png_ptr$0$phi;
   continue;
  } else if ((($21|0) == 1)) {
   $$reg2mem75$0 = $22;$png_ptr$1 = $png_ptr$0;
   label = 46;
   continue;
  }
  __THREW__ = 0;
  invoke_viiii(136,($png_ptr$0|0),($error_ptr|0),($error_fn|0),($warn_fn|0));
  $23 = __THREW__; __THREW__ = 0;
  if ((($23|0) != 0) & ((threwValue|0) != 0)) { $24 = _testSetjmp(HEAP32[$23>>2]|0, _setjmpTable)|0; if (($24|0) == 0) { _longjmp($23|0, threwValue|0); } tempRet0 = threwValue; } else { $24 = -1; };
  $25 = tempRet0;
  if ((($24|0) == 2)) {
   $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $25;$png_ptr$0 = $png_ptr$0$phi;
   continue;
  } else if ((($24|0) == 1)) {
   $$reg2mem75$0 = $25;$png_ptr$1 = $png_ptr$0;
   label = 46;
   continue;
  }
  $i$0 = 0;
  while(1) {
   $26 = (($user_png_ver) + ($i$0)|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27 << 24 >> 24;
   $29 = (57104 + ($i$0)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30 << 24 >> 24;
   $32 = ($28|0)!=($31|0);
   if ($32) {
    $33 = (($png_ptr$0) + 200|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = $34 | 131072;
    HEAP32[$33>>2] = $35;
   }
   $36 = (($i$0) + 1)|0;
   $37 = (57104 + ($i$0)|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = ($38<<24>>24)!=(0);
   if ($39) {
    $i$0 = $36;
   } else {
    break;
   }
  }
  $40 = (($png_ptr$0) + 200|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $41 & 131072;
  $43 = ($42|0)!=(0);
  if ($43) {
   $44 = ($user_png_ver|0)==(0|0);
   do {
    if ($44) {
     label = 24;
    } else {
     $45 = HEAP8[$user_png_ver>>0]|0;
     $46 = $45 << 24 >> 24;
     $47 = HEAP8[57104>>0]|0;
     $48 = $47 << 24 >> 24;
     $49 = ($46|0)!=($48|0);
     if ($49) {
      label = 24;
     } else {
      $50 = HEAP8[$user_png_ver>>0]|0;
      $51 = $50 << 24 >> 24;
      $52 = ($51|0)==(49);
      if ($52) {
       $53 = (($user_png_ver) + 2|0);
       $54 = HEAP8[$53>>0]|0;
       $55 = $54 << 24 >> 24;
       $56 = HEAP8[((57104 + 2|0))>>0]|0;
       $57 = $56 << 24 >> 24;
       $58 = ($55|0)!=($57|0);
       if ($58) {
        label = 24;
        break;
       }
      }
      $59 = HEAP8[$user_png_ver>>0]|0;
      $60 = $59 << 24 >> 24;
      $61 = ($60|0)==(48);
      if ($61) {
       $62 = (($user_png_ver) + 2|0);
       $63 = HEAP8[$62>>0]|0;
       $64 = $63 << 24 >> 24;
       $65 = ($64|0)<(57);
       if ($65) {
        label = 24;
       }
      }
     }
    }
   } while(0);
   if ((label|0) == 24) {
    label = 0;
    $66 = ($user_png_ver|0)!=(0|0);
    if ($66) {
     __THREW__ = 0;
     HEAP32[$vararg_buffer>>2] = $user_png_ver;
     (invoke_iiiii(137,($msg|0),80,(57768|0),($vararg_buffer|0))|0);
     $67 = __THREW__; __THREW__ = 0;
     if ((($67|0) != 0) & ((threwValue|0) != 0)) { $68 = _testSetjmp(HEAP32[$67>>2]|0, _setjmpTable)|0; if (($68|0) == 0) { _longjmp($67|0, threwValue|0); } tempRet0 = threwValue; } else { $68 = -1; };
     $69 = tempRet0;
     if ((($68|0) == 2)) {
      $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $69;$png_ptr$0 = $png_ptr$0$phi;
      continue;
     } else if ((($68|0) == 1)) {
      $$reg2mem75$0 = $69;$png_ptr$1 = $png_ptr$0;
      label = 46;
      continue;
     }
     __THREW__ = 0;
     invoke_vii(138,($png_ptr$0|0),($msg|0));
     $70 = __THREW__; __THREW__ = 0;
     if ((($70|0) != 0) & ((threwValue|0) != 0)) { $71 = _testSetjmp(HEAP32[$70>>2]|0, _setjmpTable)|0; if (($71|0) == 0) { _longjmp($70|0, threwValue|0); } tempRet0 = threwValue; } else { $71 = -1; };
     $72 = tempRet0;
     if ((($71|0) == 2)) {
      $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $72;$png_ptr$0 = $png_ptr$0$phi;
      continue;
     } else if ((($71|0) == 1)) {
      $$reg2mem75$0 = $72;$png_ptr$1 = $png_ptr$0;
      label = 46;
      continue;
     }
    }
    __THREW__ = 0;
    HEAP32[$vararg_buffer79>>2] = 57104;
    (invoke_iiiii(137,($msg|0),80,(57824|0),($vararg_buffer79|0))|0);
    $73 = __THREW__; __THREW__ = 0;
    if ((($73|0) != 0) & ((threwValue|0) != 0)) { $74 = _testSetjmp(HEAP32[$73>>2]|0, _setjmpTable)|0; if (($74|0) == 0) { _longjmp($73|0, threwValue|0); } tempRet0 = threwValue; } else { $74 = -1; };
    $75 = tempRet0;
    if ((($74|0) == 2)) {
     $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $75;$png_ptr$0 = $png_ptr$0$phi;
     continue;
    } else if ((($74|0) == 1)) {
     $$reg2mem75$0 = $75;$png_ptr$1 = $png_ptr$0;
     label = 46;
     continue;
    }
    __THREW__ = 0;
    invoke_vii(138,($png_ptr$0|0),($msg|0));
    $76 = __THREW__; __THREW__ = 0;
    if ((($76|0) != 0) & ((threwValue|0) != 0)) { $77 = _testSetjmp(HEAP32[$76>>2]|0, _setjmpTable)|0; if (($77|0) == 0) { _longjmp($76|0, threwValue|0); } tempRet0 = threwValue; } else { $77 = -1; };
    $78 = tempRet0;
    if ((($77|0) == 2)) {
     $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $78;$png_ptr$0 = $png_ptr$0$phi;
     continue;
    } else if ((($77|0) == 1)) {
     $$reg2mem75$0 = $78;$png_ptr$1 = $png_ptr$0;
     label = 46;
     continue;
    }
    $79 = (($png_ptr$0) + 200|0);
    HEAP32[$79>>2] = 0;
    __THREW__ = 0;
    invoke_vii(139,($png_ptr$0|0),(57880|0));
    $80 = __THREW__; __THREW__ = 0;
    if ((($80|0) != 0) & ((threwValue|0) != 0)) { $81 = _testSetjmp(HEAP32[$80>>2]|0, _setjmpTable)|0; if (($81|0) == 0) { _longjmp($80|0, threwValue|0); } tempRet0 = threwValue; } else { $81 = -1; };
    $82 = tempRet0;
    if ((($81|0) == 2)) {
     $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $82;$png_ptr$0 = $png_ptr$0$phi;
     continue;
    } else if ((($81|0) == 1)) {
     $$reg2mem75$0 = $82;$png_ptr$1 = $png_ptr$0;
     label = 46;
     continue;
    }
   }
  }
  $83 = (($png_ptr$0) + 268|0);
  HEAP32[$83>>2] = 8192;
  $84 = (($png_ptr$0) + 268|0);
  $85 = HEAP32[$84>>2]|0;
  __THREW__ = 0;
  $86 = (invoke_iii(140,($png_ptr$0|0),($85|0))|0);
  $87 = __THREW__; __THREW__ = 0;
  if ((($87|0) != 0) & ((threwValue|0) != 0)) { $88 = _testSetjmp(HEAP32[$87>>2]|0, _setjmpTable)|0; if (($88|0) == 0) { _longjmp($87|0, threwValue|0); } tempRet0 = threwValue; } else { $88 = -1; };
  $89 = tempRet0;
  if ((($88|0) == 2)) {
   $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $89;$png_ptr$0 = $png_ptr$0$phi;
   continue;
  } else if ((($88|0) == 1)) {
   $$reg2mem75$0 = $89;$png_ptr$1 = $png_ptr$0;
   label = 46;
   continue;
  }
  $90 = (($png_ptr$0) + 264|0);
  HEAP32[$90>>2] = $86;
  $91 = (($png_ptr$0) + 208|0);
  $92 = (($91) + 32|0);
  HEAP32[$92>>2] = 141;
  $93 = (($png_ptr$0) + 208|0);
  $94 = (($93) + 36|0);
  HEAP32[$94>>2] = 142;
  $95 = (($png_ptr$0) + 208|0);
  $96 = (($95) + 40|0);
  HEAP32[$96>>2] = $png_ptr$0;
  $97 = (($png_ptr$0) + 208|0);
  __THREW__ = 0;
  $98 = (invoke_iiii(143,($97|0),(57936|0),56)|0);
  $99 = __THREW__; __THREW__ = 0;
  if ((($99|0) != 0) & ((threwValue|0) != 0)) { $100 = _testSetjmp(HEAP32[$99>>2]|0, _setjmpTable)|0; if (($100|0) == 0) { _longjmp($99|0, threwValue|0); } tempRet0 = threwValue; } else { $100 = -1; };
  $101 = tempRet0;
  if ((($100|0) == 2)) {
   $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $101;$png_ptr$0 = $png_ptr$0$phi;
   continue;
  } else if ((($100|0) == 1)) {
   $$reg2mem75$0 = $101;$png_ptr$1 = $png_ptr$0;
   label = 46;
   continue;
  }
  if ((($98|0) == -2) | (($98|0) == -4)) {
   __THREW__ = 0;
   invoke_vii(139,($png_ptr$0|0),(57944|0));
   $102 = __THREW__; __THREW__ = 0;
   if ((($102|0) != 0) & ((threwValue|0) != 0)) { $103 = _testSetjmp(HEAP32[$102>>2]|0, _setjmpTable)|0; if (($103|0) == 0) { _longjmp($102|0, threwValue|0); } tempRet0 = threwValue; } else { $103 = -1; };
   $104 = tempRet0;
   if ((($103|0) == 2)) {
    $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $104;$png_ptr$0 = $png_ptr$0$phi;
    continue;
   } else if ((($103|0) == 1)) {
    $$reg2mem75$0 = $104;$png_ptr$1 = $png_ptr$0;
    label = 46;
    continue;
   }
  } else if ((($98|0) == 0)) {
  } else if ((($98|0) == -6)) {
   __THREW__ = 0;
   invoke_vii(139,($png_ptr$0|0),(57968|0));
   $105 = __THREW__; __THREW__ = 0;
   if ((($105|0) != 0) & ((threwValue|0) != 0)) { $106 = _testSetjmp(HEAP32[$105>>2]|0, _setjmpTable)|0; if (($106|0) == 0) { _longjmp($105|0, threwValue|0); } tempRet0 = threwValue; } else { $106 = -1; };
   $107 = tempRet0;
   if ((($106|0) == 2)) {
    $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $107;$png_ptr$0 = $png_ptr$0$phi;
    continue;
   } else if ((($106|0) == 1)) {
    $$reg2mem75$0 = $107;$png_ptr$1 = $png_ptr$0;
    label = 46;
    continue;
   }
  } else {
   __THREW__ = 0;
   invoke_vii(139,($png_ptr$0|0),(57992|0));
   $108 = __THREW__; __THREW__ = 0;
   if ((($108|0) != 0) & ((threwValue|0) != 0)) { $109 = _testSetjmp(HEAP32[$108>>2]|0, _setjmpTable)|0; if (($109|0) == 0) { _longjmp($108|0, threwValue|0); } tempRet0 = threwValue; } else { $109 = -1; };
   $110 = tempRet0;
   if ((($109|0) == 2)) {
    $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $110;$png_ptr$0 = $png_ptr$0$phi;
    continue;
   } else if ((($109|0) == 1)) {
    $$reg2mem75$0 = $110;$png_ptr$1 = $png_ptr$0;
    label = 46;
    continue;
   }
  }
  $111 = (($png_ptr$0) + 264|0);
  $112 = HEAP32[$111>>2]|0;
  $113 = (($png_ptr$0) + 208|0);
  $114 = (($113) + 12|0);
  HEAP32[$114>>2] = $112;
  $115 = (($png_ptr$0) + 268|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (($png_ptr$0) + 208|0);
  $118 = (($117) + 16|0);
  HEAP32[$118>>2] = $116;
  __THREW__ = 0;
  invoke_viii(24,($png_ptr$0|0),(0|0),(0|0));
  $119 = __THREW__; __THREW__ = 0;
  if ((($119|0) != 0) & ((threwValue|0) != 0)) { $120 = _testSetjmp(HEAP32[$119>>2]|0, _setjmpTable)|0; if (($120|0) == 0) { _longjmp($119|0, threwValue|0); } tempRet0 = threwValue; } else { $120 = -1; };
  $121 = tempRet0;
  if ((($120|0) == 2)) {
   $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $121;$png_ptr$0 = $png_ptr$0$phi;
   continue;
  } else if ((($120|0) == 1)) {
   $$reg2mem75$0 = $121;$png_ptr$1 = $png_ptr$0;
   label = 46;
   continue;
  }
  _saveSetjmp($png_ptr$0,1,_setjmpTable|0)|0;
  __THREW__ = 0;
  $122 = __THREW__; __THREW__ = 0;
  if ((($122|0) != 0) & ((threwValue|0) != 0)) { $123 = _testSetjmp(HEAP32[$122>>2]|0, _setjmpTable)|0; if (($123|0) == 0) { _longjmp($122|0, threwValue|0); } tempRet0 = threwValue; } else { $123 = -1; };
  $124 = tempRet0;
  if ((($123|0) == 2)) {
   $png_ptr$0$phi = $png_ptr$0;$$reg2mem77$0 = $124;$png_ptr$0 = $png_ptr$0$phi;
   continue;
  } else if ((($123|0) == 1)) {
   $$reg2mem75$0 = $124;$png_ptr$1 = $png_ptr$0;
   label = 46;
   continue;
  }
  $$reg2mem75$0 = 0;$png_ptr$1 = $png_ptr$0;
  label = 46;
 }
 if ((label|0) == 9) {
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 else if ((label|0) == 48) {
  // unreachable;
 }
 else if ((label|0) == 49) {
  $$0 = $png_ptr$1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _png_read_info($png_ptr,$info_ptr) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $chunk_length = 0, $length = 0, $num_checked = 0, $num_to_check = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $chunk_length = sp + 20|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = $0;
 $5 = (($4) + 392|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(8);
 if ($8) {
  $9 = $0;
  $10 = (($9) + 392|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $num_checked = $12;
  $13 = $num_checked;
  $14 = (8 - ($13))|0;
  $num_to_check = $14;
  $15 = $0;
  $16 = $num_checked;
  $17 = $1;
  $18 = (($17) + 32|0);
  $19 = (($18) + ($16)|0);
  $20 = $num_to_check;
  _png_read_data($15,$19,$20);
  $21 = $0;
  $22 = (($21) + 392|0);
  HEAP8[$22>>0] = 8;
  $23 = $1;
  $24 = (($23) + 32|0);
  $25 = $num_checked;
  $26 = $num_to_check;
  $27 = (_png_sig_cmp($24,$25,$26)|0);
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = $num_checked;
   $30 = ($29>>>0)<(4);
   if ($30) {
    $31 = $1;
    $32 = (($31) + 32|0);
    $33 = $num_checked;
    $34 = $num_to_check;
    $35 = (($34) - 4)|0;
    $36 = (_png_sig_cmp($32,$33,$35)|0);
    $37 = ($36|0)!=(0);
    if ($37) {
     $38 = $0;
     _png_error($38,58016);
    } else {
     label = 8;
    }
   } else {
    label = 8;
   }
   if ((label|0) == 8) {
    $39 = $0;
    _png_error($39,58032);
   }
  }
  $40 = $num_checked;
  $41 = ($40>>>0)<(3);
  if ($41) {
   $42 = $0;
   $43 = (($42) + 196|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = $44 | 4096;
   HEAP32[$43>>2] = $45;
  }
 }
 while(1) {
  $46 = $0;
  _png_read_data($46,$chunk_length,4);
  $47 = $0;
  $48 = (_png_get_uint_31($47,$chunk_length)|0);
  $length = $48;
  $49 = $0;
  _png_reset_crc($49);
  $50 = $0;
  $51 = $0;
  $52 = (($51) + 376|0);
  _png_crc_read($50,$52,4);
  $53 = $0;
  $54 = (($53) + 376|0);
  $55 = (_memcmp($54,57136,4)|0);
  $56 = ($55|0)!=(0);
  if (!($56)) {
   $57 = $0;
   $58 = (($57) + 196|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $59 & 8;
   $61 = ($60|0)!=(0);
   if ($61) {
    $62 = $0;
    $63 = (($62) + 196|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = $64 | 8192;
    HEAP32[$63>>2] = $65;
   }
  }
  $66 = $0;
  $67 = (($66) + 376|0);
  $68 = (_memcmp($67,57128,4)|0);
  $69 = ($68|0)!=(0);
  if ($69) {
   $73 = $0;
   $74 = (($73) + 376|0);
   $75 = (_memcmp($74,57144,4)|0);
   $76 = ($75|0)!=(0);
   if ($76) {
    $80 = $0;
    $81 = $0;
    $82 = (($81) + 376|0);
    $83 = (_png_handle_as_unknown($80,$82)|0);
    $84 = ($83|0)!=(0);
    if ($84) {
     $85 = $0;
     $86 = (($85) + 376|0);
     $87 = (_memcmp($86,57136,4)|0);
     $88 = ($87|0)!=(0);
     if (!($88)) {
      $89 = $0;
      $90 = (($89) + 196|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = $91 | 4;
      HEAP32[$90>>2] = $92;
     }
     $93 = $0;
     $94 = $1;
     $95 = $length;
     _png_handle_unknown($93,$94,$95);
     $96 = $0;
     $97 = (($96) + 376|0);
     $98 = (_memcmp($97,57152,4)|0);
     $99 = ($98|0)!=(0);
     if ($99) {
      $104 = $0;
      $105 = (($104) + 376|0);
      $106 = (_memcmp($105,57136,4)|0);
      $107 = ($106|0)!=(0);
      if (!($107)) {
       label = 28;
       break;
      }
     } else {
      $100 = $0;
      $101 = (($100) + 196|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = $102 | 2;
      HEAP32[$101>>2] = $103;
     }
    } else {
     $125 = $0;
     $126 = (($125) + 376|0);
     $127 = (_memcmp($126,57152,4)|0);
     $128 = ($127|0)!=(0);
     if ($128) {
      $132 = $0;
      $133 = (($132) + 376|0);
      $134 = (_memcmp($133,57136,4)|0);
      $135 = ($134|0)!=(0);
      if (!($135)) {
       label = 40;
       break;
      }
      $160 = $0;
      $161 = (($160) + 376|0);
      $162 = (_memcmp($161,57160,4)|0);
      $163 = ($162|0)!=(0);
      if ($163) {
       $167 = $0;
       $168 = (($167) + 376|0);
       $169 = (_memcmp($168,57168,4)|0);
       $170 = ($169|0)!=(0);
       if ($170) {
        $174 = $0;
        $175 = (($174) + 376|0);
        $176 = (_memcmp($175,57176,4)|0);
        $177 = ($176|0)!=(0);
        if ($177) {
         $181 = $0;
         $182 = (($181) + 376|0);
         $183 = (_memcmp($182,57184,4)|0);
         $184 = ($183|0)!=(0);
         if ($184) {
          $188 = $0;
          $189 = (($188) + 376|0);
          $190 = (_memcmp($189,57200,4)|0);
          $191 = ($190|0)!=(0);
          if ($191) {
           $195 = $0;
           $196 = (($195) + 376|0);
           $197 = (_memcmp($196,57208,4)|0);
           $198 = ($197|0)!=(0);
           if ($198) {
            $202 = $0;
            $203 = (($202) + 376|0);
            $204 = (_memcmp($203,57216,4)|0);
            $205 = ($204|0)!=(0);
            if ($205) {
             $209 = $0;
             $210 = (($209) + 376|0);
             $211 = (_memcmp($210,57224,4)|0);
             $212 = ($211|0)!=(0);
             if ($212) {
              $216 = $0;
              $217 = (($216) + 376|0);
              $218 = (_memcmp($217,57232,4)|0);
              $219 = ($218|0)!=(0);
              if ($219) {
               $223 = $0;
               $224 = (($223) + 376|0);
               $225 = (_memcmp($224,57248,4)|0);
               $226 = ($225|0)!=(0);
               if ($226) {
                $230 = $0;
                $231 = (($230) + 376|0);
                $232 = (_memcmp($231,57192,4)|0);
                $233 = ($232|0)!=(0);
                if ($233) {
                 $237 = $0;
                 $238 = (($237) + 376|0);
                 $239 = (_memcmp($238,57240,4)|0);
                 $240 = ($239|0)!=(0);
                 if ($240) {
                  $244 = $0;
                  $245 = (($244) + 376|0);
                  $246 = (_memcmp($245,57256,4)|0);
                  $247 = ($246|0)!=(0);
                  if ($247) {
                   $251 = $0;
                   $252 = (($251) + 376|0);
                   $253 = (_memcmp($252,57264,4)|0);
                   $254 = ($253|0)!=(0);
                   if ($254) {
                    $258 = $0;
                    $259 = (($258) + 376|0);
                    $260 = (_memcmp($259,57272,4)|0);
                    $261 = ($260|0)!=(0);
                    if ($261) {
                     $265 = $0;
                     $266 = (($265) + 376|0);
                     $267 = (_memcmp($266,57280,4)|0);
                     $268 = ($267|0)!=(0);
                     if ($268) {
                      $272 = $0;
                      $273 = $1;
                      $274 = $length;
                      _png_handle_unknown($272,$273,$274);
                     } else {
                      $269 = $0;
                      $270 = $1;
                      $271 = $length;
                      _png_handle_zTXt($269,$270,$271);
                     }
                    } else {
                     $262 = $0;
                     $263 = $1;
                     $264 = $length;
                     _png_handle_tRNS($262,$263,$264);
                    }
                   } else {
                    $255 = $0;
                    $256 = $1;
                    $257 = $length;
                    _png_handle_tIME($255,$256,$257);
                   }
                  } else {
                   $248 = $0;
                   $249 = $1;
                   $250 = $length;
                   _png_handle_tEXt($248,$249,$250);
                  }
                 } else {
                  $241 = $0;
                  $242 = $1;
                  $243 = $length;
                  _png_handle_sPLT($241,$242,$243);
                 }
                } else {
                 $234 = $0;
                 $235 = $1;
                 $236 = $length;
                 _png_handle_iCCP($234,$235,$236);
                }
               } else {
                $227 = $0;
                $228 = $1;
                $229 = $length;
                _png_handle_sRGB($227,$228,$229);
               }
              } else {
               $220 = $0;
               $221 = $1;
               $222 = $length;
               _png_handle_sBIT($220,$221,$222);
              }
             } else {
              $213 = $0;
              $214 = $1;
              $215 = $length;
              _png_handle_pHYs($213,$214,$215);
             }
            } else {
             $206 = $0;
             $207 = $1;
             $208 = $length;
             _png_handle_sCAL($206,$207,$208);
            }
           } else {
            $199 = $0;
            $200 = $1;
            $201 = $length;
            _png_handle_pCAL($199,$200,$201);
           }
          } else {
           $192 = $0;
           $193 = $1;
           $194 = $length;
           _png_handle_oFFs($192,$193,$194);
          }
         } else {
          $185 = $0;
          $186 = $1;
          $187 = $length;
          _png_handle_hIST($185,$186,$187);
         }
        } else {
         $178 = $0;
         $179 = $1;
         $180 = $length;
         _png_handle_gAMA($178,$179,$180);
        }
       } else {
        $171 = $0;
        $172 = $1;
        $173 = $length;
        _png_handle_cHRM($171,$172,$173);
       }
      } else {
       $164 = $0;
       $165 = $1;
       $166 = $length;
       _png_handle_bKGD($164,$165,$166);
      }
     } else {
      $129 = $0;
      $130 = $1;
      $131 = $length;
      _png_handle_PLTE($129,$130,$131);
     }
    }
   } else {
    $77 = $0;
    $78 = $1;
    $79 = $length;
    _png_handle_IEND($77,$78,$79);
   }
  } else {
   $70 = $0;
   $71 = $1;
   $72 = $length;
   _png_handle_IHDR($70,$71,$72);
  }
 }
 if ((label|0) == 28) {
  $108 = $0;
  $109 = (($108) + 196|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = $110 & 1;
  $112 = ($111|0)!=(0);
  if ($112) {
   $114 = $0;
   $115 = (($114) + 386|0);
   $116 = HEAP8[$115>>0]|0;
   $117 = $116&255;
   $118 = ($117|0)==(3);
   if ($118) {
    $119 = $0;
    $120 = (($119) + 196|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = $121 & 2;
    $123 = ($122|0)!=(0);
    if (!($123)) {
     $124 = $0;
     _png_error($124,58104);
    }
   }
  } else {
   $113 = $0;
   _png_error($113,58072);
  }
  STACKTOP = sp;return;
 }
 else if ((label|0) == 40) {
  $136 = $0;
  $137 = (($136) + 196|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = $138 & 1;
  $140 = ($139|0)!=(0);
  if ($140) {
   $142 = $0;
   $143 = (($142) + 386|0);
   $144 = HEAP8[$143>>0]|0;
   $145 = $144&255;
   $146 = ($145|0)==(3);
   if ($146) {
    $147 = $0;
    $148 = (($147) + 196|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = $149 & 2;
    $151 = ($150|0)!=(0);
    if (!($151)) {
     $152 = $0;
     _png_error($152,58104);
    }
   }
  } else {
   $141 = $0;
   _png_error($141,58072);
  }
  $153 = $length;
  $154 = $0;
  $155 = (($154) + 360|0);
  HEAP32[$155>>2] = $153;
  $156 = $0;
  $157 = (($156) + 196|0);
  $158 = HEAP32[$157>>2]|0;
  $159 = $158 | 4;
  HEAP32[$157>>2] = $159;
  STACKTOP = sp;return;
 }
}
function _png_read_update_info($png_ptr,$info_ptr) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = $0;
 $5 = (($4) + 200|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 64;
 $8 = ($7|0)!=(0);
 if ($8) {
  $10 = $0;
  _png_warning($10,58136);
 } else {
  $9 = $0;
  _png_read_start_row($9);
 }
 $11 = $0;
 $12 = $1;
 _png_read_transform_info($11,$12);
 STACKTOP = sp;return;
}
function _png_read_row($png_ptr,$row,$dsp_row) {
 $png_ptr = $png_ptr|0;
 $row = $row|0;
 $dsp_row = $dsp_row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $chunk_length = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $chunk_length = sp + 16|0;
 $0 = $png_ptr;
 $1 = $row;
 $2 = $dsp_row;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = $0;
 $6 = (($5) + 200|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 64;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  $10 = $0;
  _png_read_start_row($10);
 }
 $11 = $0;
 $12 = (($11) + 320|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = $0;
  $16 = (($15) + 384|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = ($18|0)==(0);
  if ($19) {
  }
 }
 $20 = $0;
 $21 = (($20) + 383|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = $0;
  $26 = (($25) + 204|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $27 & 2;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = $0;
   $31 = (($30) + 384|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   L15: do {
    switch ($33|0) {
    case 1:  {
     $50 = $0;
     $51 = (($50) + 320|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = $52 & 7;
     $54 = ($53|0)!=(0);
     if (!($54)) {
      $55 = $0;
      $56 = (($55) + 292|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57>>>0)<(5);
      if (!($58)) {
       break L15;
      }
     }
     $59 = $2;
     $60 = ($59|0)!=(0|0);
     if ($60) {
      $61 = $0;
      $62 = $2;
      $63 = $0;
      $64 = (($63) + 384|0);
      $65 = HEAP8[$64>>0]|0;
      $66 = $65&255;
      $67 = (57448 + ($66<<2)|0);
      $68 = HEAP32[$67>>2]|0;
      _png_combine_row($61,$62,$68);
     }
     $69 = $0;
     _png_read_finish_row($69);
     STACKTOP = sp;return;
     break;
    }
    case 3:  {
     $91 = $0;
     $92 = (($91) + 320|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = $93 & 3;
     $95 = ($94|0)!=(0);
     if (!($95)) {
      $96 = $0;
      $97 = (($96) + 292|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = ($98>>>0)<(3);
      if (!($99)) {
       break L15;
      }
     }
     $100 = $2;
     $101 = ($100|0)!=(0|0);
     if ($101) {
      $102 = $0;
      $103 = $2;
      $104 = $0;
      $105 = (($104) + 384|0);
      $106 = HEAP8[$105>>0]|0;
      $107 = $106&255;
      $108 = (57448 + ($107<<2)|0);
      $109 = HEAP32[$108>>2]|0;
      _png_combine_row($102,$103,$109);
     }
     $110 = $0;
     _png_read_finish_row($110);
     STACKTOP = sp;return;
     break;
    }
    case 0:  {
     $34 = $0;
     $35 = (($34) + 320|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = $36 & 7;
     $38 = ($37|0)!=(0);
     if (!($38)) {
      break L15;
     }
     $39 = $2;
     $40 = ($39|0)!=(0|0);
     if ($40) {
      $41 = $0;
      $42 = $2;
      $43 = $0;
      $44 = (($43) + 384|0);
      $45 = HEAP8[$44>>0]|0;
      $46 = $45&255;
      $47 = (57448 + ($46<<2)|0);
      $48 = HEAP32[$47>>2]|0;
      _png_combine_row($41,$42,$48);
     }
     $49 = $0;
     _png_read_finish_row($49);
     STACKTOP = sp;return;
     break;
    }
    case 2:  {
     $70 = $0;
     $71 = (($70) + 320|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = $72 & 7;
     $74 = ($73|0)!=(4);
     if (!($74)) {
      break L15;
     }
     $75 = $2;
     $76 = ($75|0)!=(0|0);
     if ($76) {
      $77 = $0;
      $78 = (($77) + 320|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = $79 & 4;
      $81 = ($80|0)!=(0);
      if ($81) {
       $82 = $0;
       $83 = $2;
       $84 = $0;
       $85 = (($84) + 384|0);
       $86 = HEAP8[$85>>0]|0;
       $87 = $86&255;
       $88 = (57448 + ($87<<2)|0);
       $89 = HEAP32[$88>>2]|0;
       _png_combine_row($82,$83,$89);
      }
     }
     $90 = $0;
     _png_read_finish_row($90);
     STACKTOP = sp;return;
     break;
    }
    case 5:  {
     $132 = $0;
     $133 = (($132) + 320|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & 1;
     $136 = ($135|0)!=(0);
     if (!($136)) {
      $137 = $0;
      $138 = (($137) + 292|0);
      $139 = HEAP32[$138>>2]|0;
      $140 = ($139>>>0)<(2);
      if (!($140)) {
       break L15;
      }
     }
     $141 = $2;
     $142 = ($141|0)!=(0|0);
     if ($142) {
      $143 = $0;
      $144 = $2;
      $145 = $0;
      $146 = (($145) + 384|0);
      $147 = HEAP8[$146>>0]|0;
      $148 = $147&255;
      $149 = (57448 + ($148<<2)|0);
      $150 = HEAP32[$149>>2]|0;
      _png_combine_row($143,$144,$150);
     }
     $151 = $0;
     _png_read_finish_row($151);
     STACKTOP = sp;return;
     break;
    }
    case 4:  {
     $111 = $0;
     $112 = (($111) + 320|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = $113 & 3;
     $115 = ($114|0)!=(2);
     if (!($115)) {
      break L15;
     }
     $116 = $2;
     $117 = ($116|0)!=(0|0);
     if ($117) {
      $118 = $0;
      $119 = (($118) + 320|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = $120 & 2;
      $122 = ($121|0)!=(0);
      if ($122) {
       $123 = $0;
       $124 = $2;
       $125 = $0;
       $126 = (($125) + 384|0);
       $127 = HEAP8[$126>>0]|0;
       $128 = $127&255;
       $129 = (57448 + ($128<<2)|0);
       $130 = HEAP32[$129>>2]|0;
       _png_combine_row($123,$124,$130);
      }
     }
     $131 = $0;
     _png_read_finish_row($131);
     STACKTOP = sp;return;
     break;
    }
    case 6:  {
     $152 = $0;
     $153 = (($152) + 320|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = $154 & 1;
     $156 = ($155|0)!=(0);
     if ($156) {
      break L15;
     }
     $157 = $0;
     _png_read_finish_row($157);
     STACKTOP = sp;return;
     break;
    }
    default: {
    }
    }
   } while(0);
  }
 }
 $158 = $0;
 $159 = (($158) + 196|0);
 $160 = HEAP32[$159>>2]|0;
 $161 = $160 & 4;
 $162 = ($161|0)!=(0);
 if (!($162)) {
  $163 = $0;
  _png_error($163,58208);
 }
 $164 = $0;
 $165 = (($164) + 328|0);
 $166 = HEAP32[$165>>2]|0;
 $167 = $0;
 $168 = (($167) + 208|0);
 $169 = (($168) + 12|0);
 HEAP32[$169>>2] = $166;
 $170 = $0;
 $171 = (($170) + 312|0);
 $172 = HEAP32[$171>>2]|0;
 $173 = $0;
 $174 = (($173) + 208|0);
 $175 = (($174) + 16|0);
 HEAP32[$175>>2] = $172;
 while(1) {
  $176 = $0;
  $177 = (($176) + 208|0);
  $178 = (($177) + 4|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = ($179|0)!=(0);
  if (!($180)) {
   while(1) {
    $181 = $0;
    $182 = (($181) + 360|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = ($183|0)!=(0);
    $185 = $184 ^ 1;
    if (!($185)) {
     break;
    }
    $186 = $0;
    (_png_crc_finish($186,0)|0);
    $187 = $0;
    _png_read_data($187,$chunk_length,4);
    $188 = $0;
    $189 = (_png_get_uint_31($188,$chunk_length)|0);
    $190 = $0;
    $191 = (($190) + 360|0);
    HEAP32[$191>>2] = $189;
    $192 = $0;
    _png_reset_crc($192);
    $193 = $0;
    $194 = $0;
    $195 = (($194) + 376|0);
    _png_crc_read($193,$195,4);
    $196 = $0;
    $197 = (($196) + 376|0);
    $198 = (_memcmp($197,57136,4)|0);
    $199 = ($198|0)!=(0);
    if ($199) {
     $200 = $0;
     _png_error($200,58248);
    }
   }
   $201 = $0;
   $202 = (($201) + 268|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = $0;
   $205 = (($204) + 208|0);
   $206 = (($205) + 4|0);
   HEAP32[$206>>2] = $203;
   $207 = $0;
   $208 = (($207) + 264|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = $0;
   $211 = (($210) + 208|0);
   HEAP32[$211>>2] = $209;
   $212 = $0;
   $213 = (($212) + 268|0);
   $214 = HEAP32[$213>>2]|0;
   $215 = $0;
   $216 = (($215) + 360|0);
   $217 = HEAP32[$216>>2]|0;
   $218 = ($214>>>0)>($217>>>0);
   if ($218) {
    $219 = $0;
    $220 = (($219) + 360|0);
    $221 = HEAP32[$220>>2]|0;
    $222 = $0;
    $223 = (($222) + 208|0);
    $224 = (($223) + 4|0);
    HEAP32[$224>>2] = $221;
   }
   $225 = $0;
   $226 = $0;
   $227 = (($226) + 264|0);
   $228 = HEAP32[$227>>2]|0;
   $229 = $0;
   $230 = (($229) + 208|0);
   $231 = (($230) + 4|0);
   $232 = HEAP32[$231>>2]|0;
   _png_crc_read($225,$228,$232);
   $233 = $0;
   $234 = (($233) + 208|0);
   $235 = (($234) + 4|0);
   $236 = HEAP32[$235>>2]|0;
   $237 = $0;
   $238 = (($237) + 360|0);
   $239 = HEAP32[$238>>2]|0;
   $240 = (($239) - ($236))|0;
   HEAP32[$238>>2] = $240;
  }
  $241 = $0;
  $242 = (($241) + 208|0);
  $243 = (_inflate($242,1)|0);
  $ret = $243;
  $244 = $ret;
  $245 = ($244|0)==(1);
  if ($245) {
   label = 63;
   break;
  }
  $269 = $ret;
  $270 = ($269|0)!=(0);
  if ($270) {
   $271 = $0;
   $272 = $0;
   $273 = (($272) + 208|0);
   $274 = (($273) + 24|0);
   $275 = HEAP32[$274>>2]|0;
   $276 = ($275|0)!=(0|0);
   if ($276) {
    $277 = $0;
    $278 = (($277) + 208|0);
    $279 = (($278) + 24|0);
    $280 = HEAP32[$279>>2]|0;
    $281 = $280;
   } else {
    $281 = 58296;
   }
   _png_error($271,$281);
  }
  $282 = $0;
  $283 = (($282) + 208|0);
  $284 = (($283) + 16|0);
  $285 = HEAP32[$284>>2]|0;
  $286 = ($285|0)!=(0);
  if (!($286)) {
   break;
  }
 }
 if ((label|0) == 63) {
  $246 = $0;
  $247 = (($246) + 208|0);
  $248 = (($247) + 16|0);
  $249 = HEAP32[$248>>2]|0;
  $250 = ($249|0)!=(0);
  if ($250) {
   label = 66;
  } else {
   $251 = $0;
   $252 = (($251) + 208|0);
   $253 = (($252) + 4|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = ($254|0)!=(0);
   if ($255) {
    label = 66;
   } else {
    $256 = $0;
    $257 = (($256) + 360|0);
    $258 = HEAP32[$257>>2]|0;
    $259 = ($258|0)!=(0);
    if ($259) {
     label = 66;
    }
   }
  }
  if ((label|0) == 66) {
   $260 = $0;
   _png_error($260,58272);
  }
  $261 = $0;
  $262 = (($261) + 196|0);
  $263 = HEAP32[$262>>2]|0;
  $264 = $263 | 8;
  HEAP32[$262>>2] = $264;
  $265 = $0;
  $266 = (($265) + 200|0);
  $267 = HEAP32[$266>>2]|0;
  $268 = $267 | 32;
  HEAP32[$266>>2] = $268;
 }
 $287 = $0;
 $288 = (($287) + 386|0);
 $289 = HEAP8[$288>>0]|0;
 $290 = $0;
 $291 = (($290) + 348|0);
 $292 = (($291) + 8|0);
 HEAP8[$292>>0] = $289;
 $293 = $0;
 $294 = (($293) + 316|0);
 $295 = HEAP32[$294>>2]|0;
 $296 = $0;
 $297 = (($296) + 348|0);
 HEAP32[$297>>2] = $295;
 $298 = $0;
 $299 = (($298) + 390|0);
 $300 = HEAP8[$299>>0]|0;
 $301 = $0;
 $302 = (($301) + 348|0);
 $303 = (($302) + 10|0);
 HEAP8[$303>>0] = $300;
 $304 = $0;
 $305 = (($304) + 387|0);
 $306 = HEAP8[$305>>0]|0;
 $307 = $0;
 $308 = (($307) + 348|0);
 $309 = (($308) + 9|0);
 HEAP8[$309>>0] = $306;
 $310 = $0;
 $311 = (($310) + 389|0);
 $312 = HEAP8[$311>>0]|0;
 $313 = $0;
 $314 = (($313) + 348|0);
 $315 = (($314) + 11|0);
 HEAP8[$315>>0] = $312;
 $316 = $0;
 $317 = (($316) + 348|0);
 $318 = (($317) + 11|0);
 $319 = HEAP8[$318>>0]|0;
 $320 = $319&255;
 $321 = ($320|0)>=(8);
 if ($321) {
  $322 = $0;
  $323 = (($322) + 348|0);
  $324 = HEAP32[$323>>2]|0;
  $325 = $0;
  $326 = (($325) + 348|0);
  $327 = (($326) + 11|0);
  $328 = HEAP8[$327>>0]|0;
  $329 = $328&255;
  $330 = $329 >>> 3;
  $331 = Math_imul($324, $330)|0;
  $346 = $331;
 } else {
  $332 = $0;
  $333 = (($332) + 348|0);
  $334 = HEAP32[$333>>2]|0;
  $335 = $0;
  $336 = (($335) + 348|0);
  $337 = (($336) + 11|0);
  $338 = HEAP8[$337>>0]|0;
  $339 = $338&255;
  $340 = Math_imul($334, $339)|0;
  $341 = (($340) + 7)|0;
  $342 = $341 >>> 3;
  $346 = $342;
 }
 $343 = $0;
 $344 = (($343) + 348|0);
 $345 = (($344) + 4|0);
 HEAP32[$345>>2] = $346;
 $347 = $0;
 $348 = (($347) + 328|0);
 $349 = HEAP32[$348>>2]|0;
 $350 = HEAP8[$349>>0]|0;
 $351 = ($350<<24>>24)!=(0);
 if ($351) {
  $352 = $0;
  $353 = $0;
  $354 = (($353) + 348|0);
  $355 = $0;
  $356 = (($355) + 328|0);
  $357 = HEAP32[$356>>2]|0;
  $358 = (($357) + 1|0);
  $359 = $0;
  $360 = (($359) + 324|0);
  $361 = HEAP32[$360>>2]|0;
  $362 = (($361) + 1|0);
  $363 = $0;
  $364 = (($363) + 328|0);
  $365 = HEAP32[$364>>2]|0;
  $366 = HEAP8[$365>>0]|0;
  $367 = $366&255;
  _png_read_filter_row($352,$354,$358,$362,$367);
 }
 $368 = $0;
 $369 = $0;
 $370 = (($369) + 324|0);
 $371 = HEAP32[$370>>2]|0;
 $372 = $0;
 $373 = (($372) + 328|0);
 $374 = HEAP32[$373>>2]|0;
 $375 = $0;
 $376 = (($375) + 308|0);
 $377 = HEAP32[$376>>2]|0;
 $378 = (($377) + 1)|0;
 (_png_memcpy_check($368,$371,$374,$378)|0);
 $379 = $0;
 $380 = (($379) + 652|0);
 $381 = HEAP32[$380>>2]|0;
 $382 = $381 & 4;
 $383 = ($382|0)!=(0);
 if ($383) {
  $384 = $0;
  $385 = (($384) + 660|0);
  $386 = HEAP8[$385>>0]|0;
  $387 = $386&255;
  $388 = ($387|0)==(64);
  if ($388) {
   $389 = $0;
   $390 = (($389) + 348|0);
   $391 = $0;
   $392 = (($391) + 328|0);
   $393 = HEAP32[$392>>2]|0;
   $394 = (($393) + 1|0);
   _png_do_read_intrapixel($390,$394);
  }
 }
 $395 = $0;
 $396 = (($395) + 204|0);
 $397 = HEAP32[$396>>2]|0;
 $398 = ($397|0)!=(0);
 if ($398) {
  label = 85;
 } else {
  $399 = $0;
  $400 = (($399) + 200|0);
  $401 = HEAP32[$400>>2]|0;
  $402 = $401 & 4194304;
  $403 = ($402|0)!=(0);
  if ($403) {
   label = 85;
  }
 }
 if ((label|0) == 85) {
  $404 = $0;
  _png_do_read_transformations($404);
 }
 $405 = $0;
 $406 = (($405) + 383|0);
 $407 = HEAP8[$406>>0]|0;
 $408 = $407&255;
 $409 = ($408|0)!=(0);
 if ($409) {
  $410 = $0;
  $411 = (($410) + 204|0);
  $412 = HEAP32[$411>>2]|0;
  $413 = $412 & 2;
  $414 = ($413|0)!=(0);
  if ($414) {
   $415 = $0;
   $416 = (($415) + 384|0);
   $417 = HEAP8[$416>>0]|0;
   $418 = $417&255;
   $419 = ($418|0)<(6);
   if ($419) {
    $420 = $0;
    _png_do_read_interlace($420);
   }
   $421 = $2;
   $422 = ($421|0)!=(0|0);
   if ($422) {
    $423 = $0;
    $424 = $2;
    $425 = $0;
    $426 = (($425) + 384|0);
    $427 = HEAP8[$426>>0]|0;
    $428 = $427&255;
    $429 = (57448 + ($428<<2)|0);
    $430 = HEAP32[$429>>2]|0;
    _png_combine_row($423,$424,$430);
   }
   $431 = $1;
   $432 = ($431|0)!=(0|0);
   if ($432) {
    $433 = $0;
    $434 = $1;
    $435 = $0;
    $436 = (($435) + 384|0);
    $437 = HEAP8[$436>>0]|0;
    $438 = $437&255;
    $439 = (57416 + ($438<<2)|0);
    $440 = HEAP32[$439>>2]|0;
    _png_combine_row($433,$434,$440);
   }
  } else {
   label = 95;
  }
 } else {
  label = 95;
 }
 if ((label|0) == 95) {
  $441 = $1;
  $442 = ($441|0)!=(0|0);
  if ($442) {
   $443 = $0;
   $444 = $1;
   _png_combine_row($443,$444,255);
  }
  $445 = $2;
  $446 = ($445|0)!=(0|0);
  if ($446) {
   $447 = $0;
   $448 = $2;
   _png_combine_row($447,$448,255);
  }
 }
 $449 = $0;
 _png_read_finish_row($449);
 $450 = $0;
 $451 = (($450) + 500|0);
 $452 = HEAP32[$451>>2]|0;
 $453 = ($452|0)!=(0|0);
 if (!($453)) {
  STACKTOP = sp;return;
 }
 $454 = $0;
 $455 = (($454) + 500|0);
 $456 = HEAP32[$455>>2]|0;
 $457 = $0;
 $458 = $0;
 $459 = (($458) + 320|0);
 $460 = HEAP32[$459>>2]|0;
 $461 = $0;
 $462 = (($461) + 384|0);
 $463 = HEAP8[$462>>0]|0;
 $464 = $463&255;
 FUNCTION_TABLE_viii[$456 & 255]($457,$460,$464);
 STACKTOP = sp;return;
}
function _png_read_image($png_ptr,$image) {
 $png_ptr = $png_ptr|0;
 $image = $image|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $image_height = 0, $j = 0, $pass = 0, $rp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $image;
 $2 = $0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = $0;
 $5 = (_png_set_interlace_handling($4)|0);
 $pass = $5;
 $6 = $0;
 $7 = (($6) + 296|0);
 $8 = HEAP32[$7>>2]|0;
 $image_height = $8;
 $9 = $image_height;
 $10 = $0;
 $11 = (($10) + 300|0);
 HEAP32[$11>>2] = $9;
 $j = 0;
 while(1) {
  $12 = $j;
  $13 = $pass;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $1;
  $rp = $15;
  $i = 0;
  while(1) {
   $16 = $i;
   $17 = $image_height;
   $18 = ($16>>>0)<($17>>>0);
   if (!($18)) {
    break;
   }
   $19 = $0;
   $20 = $rp;
   $21 = HEAP32[$20>>2]|0;
   _png_read_row($19,$21,0);
   $22 = $rp;
   $23 = (($22) + 4|0);
   $rp = $23;
   $24 = $i;
   $25 = (($24) + 1)|0;
   $i = $25;
  }
  $26 = $j;
  $27 = (($26) + 1)|0;
  $j = $27;
 }
 STACKTOP = sp;return;
}
function _png_destroy_read_struct($png_ptr_ptr,$info_ptr_ptr,$end_info_ptr_ptr) {
 $png_ptr_ptr = $png_ptr_ptr|0;
 $info_ptr_ptr = $info_ptr_ptr|0;
 $end_info_ptr_ptr = $end_info_ptr_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $end_info_ptr = 0, $free_fn = 0, $info_ptr = 0, $mem_ptr = 0, $png_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr_ptr;
 $1 = $info_ptr_ptr;
 $2 = $end_info_ptr_ptr;
 $png_ptr = 0;
 $info_ptr = 0;
 $end_info_ptr = 0;
 $3 = $0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = HEAP32[$5>>2]|0;
  $png_ptr = $6;
 }
 $7 = $1;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = $1;
  $10 = HEAP32[$9>>2]|0;
  $info_ptr = $10;
 }
 $11 = $2;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = $2;
  $14 = HEAP32[$13>>2]|0;
  $end_info_ptr = $14;
 }
 $15 = $png_ptr;
 $16 = (($15) + 680|0);
 $17 = HEAP32[$16>>2]|0;
 $free_fn = $17;
 $18 = $png_ptr;
 $19 = (($18) + 672|0);
 $20 = HEAP32[$19>>2]|0;
 $mem_ptr = $20;
 $21 = $png_ptr;
 $22 = $info_ptr;
 $23 = $end_info_ptr;
 _png_read_destroy($21,$22,$23);
 $24 = $info_ptr;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $26 = $png_ptr;
  $27 = $info_ptr;
  _png_free_data($26,$27,16384,-1);
  $28 = $info_ptr;
  $29 = $free_fn;
  $30 = $mem_ptr;
  _png_destroy_struct_2($28,$29,$30);
  $31 = $1;
  HEAP32[$31>>2] = 0;
 }
 $32 = $end_info_ptr;
 $33 = ($32|0)!=(0|0);
 if ($33) {
  $34 = $png_ptr;
  $35 = $end_info_ptr;
  _png_free_data($34,$35,16384,-1);
  $36 = $end_info_ptr;
  $37 = $free_fn;
  $38 = $mem_ptr;
  _png_destroy_struct_2($36,$37,$38);
  $39 = $2;
  HEAP32[$39>>2] = 0;
 }
 $40 = $png_ptr;
 $41 = ($40|0)!=(0|0);
 if (!($41)) {
  STACKTOP = sp;return;
 }
 $42 = $png_ptr;
 $43 = $free_fn;
 $44 = $mem_ptr;
 _png_destroy_struct_2($42,$43,$44);
 $45 = $0;
 HEAP32[$45>>2] = 0;
 STACKTOP = sp;return;
}
function _png_read_destroy($png_ptr,$info_ptr,$end_info_ptr) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $end_info_ptr = $end_info_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $error_fn = 0, $error_ptr = 0;
 var $free_fn = 0, $i = 0, $i1 = 0, $i3 = 0, $istop = 0, $istop2 = 0, $istop4 = 0, $tmp_jmp = 0, $warning_fn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $tmp_jmp = sp + 40|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $end_info_ptr;
 $3 = $1;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $0;
  $6 = $1;
  _png_info_destroy($5,$6);
 }
 $7 = $2;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = $0;
  $10 = $2;
  _png_info_destroy($9,$10);
 }
 $11 = $0;
 $12 = $0;
 $13 = (($12) + 264|0);
 $14 = HEAP32[$13>>2]|0;
 _png_free($11,$14);
 $15 = $0;
 $16 = $0;
 $17 = (($16) + 684|0);
 $18 = HEAP32[$17>>2]|0;
 _png_free($15,$18);
 $19 = $0;
 $20 = $0;
 $21 = (($20) + 324|0);
 $22 = HEAP32[$21>>2]|0;
 _png_free($19,$22);
 $23 = $0;
 $24 = $0;
 $25 = (($24) + 584|0);
 $26 = HEAP32[$25>>2]|0;
 _png_free($23,$26);
 $27 = $0;
 $28 = $0;
 $29 = (($28) + 588|0);
 $30 = HEAP32[$29>>2]|0;
 _png_free($27,$30);
 $31 = $0;
 $32 = $0;
 $33 = (($32) + 448|0);
 $34 = HEAP32[$33>>2]|0;
 _png_free($31,$34);
 $35 = $0;
 $36 = $0;
 $37 = (($36) + 452|0);
 $38 = HEAP32[$37>>2]|0;
 _png_free($35,$38);
 $39 = $0;
 $40 = $0;
 $41 = (($40) + 456|0);
 $42 = HEAP32[$41>>2]|0;
 _png_free($39,$42);
 $43 = $0;
 $44 = (($43) + 624|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = $45 & 4096;
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = $0;
  $49 = $0;
  $50 = (($49) + 368|0);
  $51 = HEAP32[$50>>2]|0;
  _png_zfree($48,$51);
 }
 $52 = $0;
 $53 = (($52) + 624|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $54 & -4097;
 HEAP32[$53>>2] = $55;
 $56 = $0;
 $57 = (($56) + 624|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $58 & 8192;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = $0;
  $62 = $0;
  $63 = (($62) + 484|0);
  $64 = HEAP32[$63>>2]|0;
  _png_free($61,$64);
 }
 $65 = $0;
 $66 = (($65) + 624|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = $67 & -8193;
 HEAP32[$66>>2] = $68;
 $69 = $0;
 $70 = (($69) + 624|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = $71 & 8;
 $73 = ($72|0)!=(0);
 if ($73) {
  $74 = $0;
  $75 = $0;
  $76 = (($75) + 592|0);
  $77 = HEAP32[$76>>2]|0;
  _png_free($74,$77);
 }
 $78 = $0;
 $79 = (($78) + 624|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $80 & -9;
 HEAP32[$79>>2] = $81;
 $82 = $0;
 $83 = (($82) + 460|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = ($84|0)!=(0|0);
 if ($85) {
  $86 = $0;
  $87 = (($86) + 436|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (8 - ($88))|0;
  $90 = 1 << $89;
  $istop = $90;
  $i = 0;
  while(1) {
   $91 = $i;
   $92 = $istop;
   $93 = ($91|0)<($92|0);
   if (!($93)) {
    break;
   }
   $94 = $0;
   $95 = $i;
   $96 = $0;
   $97 = (($96) + 460|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) + ($95<<2)|0);
   $100 = HEAP32[$99>>2]|0;
   _png_free($94,$100);
   $101 = $i;
   $102 = (($101) + 1)|0;
   $i = $102;
  }
  $103 = $0;
  $104 = $0;
  $105 = (($104) + 460|0);
  $106 = HEAP32[$105>>2]|0;
  _png_free($103,$106);
 }
 $107 = $0;
 $108 = (($107) + 464|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = ($109|0)!=(0|0);
 if ($110) {
  $111 = $0;
  $112 = (($111) + 436|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = (8 - ($113))|0;
  $115 = 1 << $114;
  $istop2 = $115;
  $i1 = 0;
  while(1) {
   $116 = $i1;
   $117 = $istop2;
   $118 = ($116|0)<($117|0);
   if (!($118)) {
    break;
   }
   $119 = $0;
   $120 = $i1;
   $121 = $0;
   $122 = (($121) + 464|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = (($123) + ($120<<2)|0);
   $125 = HEAP32[$124>>2]|0;
   _png_free($119,$125);
   $126 = $i1;
   $127 = (($126) + 1)|0;
   $i1 = $127;
  }
  $128 = $0;
  $129 = $0;
  $130 = (($129) + 464|0);
  $131 = HEAP32[$130>>2]|0;
  _png_free($128,$131);
 }
 $132 = $0;
 $133 = (($132) + 468|0);
 $134 = HEAP32[$133>>2]|0;
 $135 = ($134|0)!=(0|0);
 if ($135) {
  $136 = $0;
  $137 = (($136) + 436|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = (8 - ($138))|0;
  $140 = 1 << $139;
  $istop4 = $140;
  $i3 = 0;
  while(1) {
   $141 = $i3;
   $142 = $istop4;
   $143 = ($141|0)<($142|0);
   if (!($143)) {
    break;
   }
   $144 = $0;
   $145 = $i3;
   $146 = $0;
   $147 = (($146) + 468|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = (($148) + ($145<<2)|0);
   $150 = HEAP32[$149>>2]|0;
   _png_free($144,$150);
   $151 = $i3;
   $152 = (($151) + 1)|0;
   $i3 = $152;
  }
  $153 = $0;
  $154 = $0;
  $155 = (($154) + 468|0);
  $156 = HEAP32[$155>>2]|0;
  _png_free($153,$156);
 }
 $157 = $0;
 $158 = $0;
 $159 = (($158) + 620|0);
 $160 = HEAP32[$159>>2]|0;
 _png_free($157,$160);
 $161 = $0;
 $162 = (($161) + 208|0);
 (_inflateEnd($162)|0);
 $163 = $0;
 $164 = $0;
 $165 = (($164) + 524|0);
 $166 = HEAP32[$165>>2]|0;
 _png_free($163,$166);
 $167 = $0;
 $168 = $0;
 $169 = (($168) + 576|0);
 $170 = HEAP32[$169>>2]|0;
 _png_free($167,$170);
 $171 = $0;
 _memcpy(($tmp_jmp|0),($171|0),156)|0;
 $172 = $0;
 $173 = (($172) + 156|0);
 $174 = HEAP32[$173>>2]|0;
 $error_fn = $174;
 $175 = $0;
 $176 = (($175) + 160|0);
 $177 = HEAP32[$176>>2]|0;
 $warning_fn = $177;
 $178 = $0;
 $179 = (($178) + 164|0);
 $180 = HEAP32[$179>>2]|0;
 $error_ptr = $180;
 $181 = $0;
 $182 = (($181) + 680|0);
 $183 = HEAP32[$182>>2]|0;
 $free_fn = $183;
 $184 = $0;
 _memset(($184|0),0,732)|0;
 $185 = $error_fn;
 $186 = $0;
 $187 = (($186) + 156|0);
 HEAP32[$187>>2] = $185;
 $188 = $warning_fn;
 $189 = $0;
 $190 = (($189) + 160|0);
 HEAP32[$190>>2] = $188;
 $191 = $error_ptr;
 $192 = $0;
 $193 = (($192) + 164|0);
 HEAP32[$193>>2] = $191;
 $194 = $free_fn;
 $195 = $0;
 $196 = (($195) + 680|0);
 HEAP32[$196>>2] = $194;
 $197 = $0;
 _memcpy(($197|0),($tmp_jmp|0),156)|0;
 STACKTOP = sp;return;
}
function _png_read_data($png_ptr,$data,$length) {
 $png_ptr = $png_ptr|0;
 $data = $data|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $data;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 172|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = $0;
  $8 = (($7) + 172|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $0;
  $11 = $1;
  $12 = $2;
  FUNCTION_TABLE_viii[$9 & 255]($10,$11,$12);
  STACKTOP = sp;return;
 } else {
  $13 = $0;
  _png_error($13,58320);
  STACKTOP = sp;return;
 }
}
function _png_default_read_data($png_ptr,$data,$length) {
 $png_ptr = $png_ptr|0;
 $data = $data|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $check = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $data;
 $2 = $length;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = $1;
 $6 = $2;
 $7 = $0;
 $8 = (($7) + 176|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_fread(($5|0),1,($6|0),($9|0))|0);
 $check = $10;
 $11 = $check;
 $12 = $2;
 $13 = ($11|0)!=($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = $0;
 _png_error($14,58352);
 STACKTOP = sp;return;
}
function _png_set_read_fn($png_ptr,$io_ptr,$read_data_fn) {
 $png_ptr = $png_ptr|0;
 $io_ptr = $io_ptr|0;
 $read_data_fn = $read_data_fn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $io_ptr;
 $2 = $read_data_fn;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = $1;
 $6 = $0;
 $7 = (($6) + 176|0);
 HEAP32[$7>>2] = $5;
 $8 = $2;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = $2;
  $11 = $0;
  $12 = (($11) + 172|0);
  HEAP32[$12>>2] = $10;
 } else {
  $13 = $0;
  $14 = (($13) + 172|0);
  HEAP32[$14>>2] = 145;
 }
 $15 = $0;
 $16 = (($15) + 168|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = $0;
  $20 = (($19) + 168|0);
  HEAP32[$20>>2] = 0;
  $21 = $0;
  _png_warning($21,58368);
  $22 = $0;
  _png_warning($22,58440);
 }
 $23 = $0;
 $24 = (($23) + 424|0);
 HEAP32[$24>>2] = 0;
 STACKTOP = sp;return;
}
function _png_init_read_transformations($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0.0, $1009 = 0.0, $101 = 0, $1010 = 0.0, $1011 = 0.0, $1012 = 0.0, $1013 = 0.0, $1014 = 0.0;
 var $1015 = 0.0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0.0, $1025 = 0.0, $1026 = 0.0, $1027 = 0.0, $1028 = 0.0, $1029 = 0.0, $103 = 0, $1030 = 0.0, $1031 = 0.0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $14 = 0;
 var $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0;
 var $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0;
 var $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0;
 var $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0;
 var $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0;
 var $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0;
 var $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0;
 var $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0;
 var $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0;
 var $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0.0, $32 = 0;
 var $320 = 0, $321 = 0, $322 = 0.0, $323 = 0.0, $324 = 0.0, $325 = 0.0, $326 = 0.0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0;
 var $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0;
 var $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0.0, $366 = 0.0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0;
 var $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0;
 var $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0;
 var $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0;
 var $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0;
 var $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0.0, $459 = 0.0, $46 = 0, $460 = 0, $461 = 0, $462 = 0.0, $463 = 0.0, $464 = 0.0;
 var $465 = 0, $466 = 0, $467 = 0.0, $468 = 0, $469 = 0, $47 = 0, $470 = 0.0, $471 = 0.0, $472 = 0.0, $473 = 0.0, $474 = 0, $475 = 0, $476 = 0.0, $477 = 0.0, $478 = 0.0, $479 = 0, $48 = 0, $480 = 0, $481 = 0.0, $482 = 0;
 var $483 = 0, $484 = 0.0, $485 = 0.0, $486 = 0.0, $487 = 0.0, $488 = 0.0, $489 = 0.0, $49 = 0, $490 = 0.0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0;
 var $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0.0, $514 = 0.0, $515 = 0.0, $516 = 0.0, $517 = 0.0, $518 = 0.0;
 var $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0.0, $525 = 0.0, $526 = 0.0, $527 = 0.0, $528 = 0.0, $529 = 0.0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0.0;
 var $537 = 0.0, $538 = 0.0, $539 = 0.0, $54 = 0, $540 = 0.0, $541 = 0.0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0.0, $549 = 0.0, $55 = 0, $550 = 0.0, $551 = 0.0, $552 = 0.0, $553 = 0.0, $554 = 0;
 var $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0.0, $561 = 0.0, $562 = 0.0, $563 = 0.0, $564 = 0.0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0.0, $572 = 0.0;
 var $573 = 0.0, $574 = 0.0, $575 = 0.0, $576 = 0.0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0;
 var $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0;
 var $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0;
 var $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0;
 var $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0;
 var $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0;
 var $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0;
 var $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0;
 var $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0;
 var $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0;
 var $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0;
 var $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0;
 var $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0;
 var $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0;
 var $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0.0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0;
 var $843 = 0, $844 = 0, $845 = 0.0, $846 = 0.0, $847 = 0, $848 = 0, $849 = 0.0, $85 = 0, $850 = 0.0, $851 = 0.0, $852 = 0, $853 = 0, $854 = 0.0, $855 = 0, $856 = 0, $857 = 0.0, $858 = 0.0, $859 = 0.0, $86 = 0, $860 = 0.0;
 var $861 = 0, $862 = 0, $863 = 0.0, $864 = 0.0, $865 = 0.0, $866 = 0, $867 = 0, $868 = 0.0, $869 = 0, $87 = 0, $870 = 0, $871 = 0.0, $872 = 0.0, $873 = 0.0, $874 = 0.0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0.0;
 var $88 = 0, $880 = 0.0, $881 = 0.0, $882 = 0.0, $883 = 0.0, $884 = 0.0, $885 = 0.0, $886 = 0.0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0.0, $896 = 0.0, $897 = 0.0;
 var $898 = 0.0, $899 = 0.0, $9 = 0, $90 = 0, $900 = 0.0, $901 = 0.0, $902 = 0.0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0;
 var $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0;
 var $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0.0, $945 = 0.0, $946 = 0.0, $947 = 0.0, $948 = 0.0, $949 = 0.0, $95 = 0, $950 = 0.0;
 var $951 = 0.0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0.0, $961 = 0.0, $962 = 0.0, $963 = 0.0, $964 = 0.0, $965 = 0.0, $966 = 0.0, $967 = 0.0, $968 = 0, $969 = 0;
 var $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0.0, $977 = 0.0, $978 = 0.0, $979 = 0.0, $98 = 0, $980 = 0.0, $981 = 0.0, $982 = 0.0, $983 = 0.0, $984 = 0, $985 = 0, $986 = 0, $987 = 0;
 var $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0.0, $993 = 0.0, $994 = 0.0, $995 = 0.0, $996 = 0.0, $997 = 0.0, $998 = 0.0, $999 = 0.0, $back = 0, $back12 = 0, $back_1 = 0, $color_type = 0, $g = 0.0, $g5 = 0.0, $gs = 0.0;
 var $gs6 = 0.0, $i = 0, $i1 = 0, $i10 = 0, $i17 = 0, $i2 = 0, $i9 = 0, $istop = 0, $istop11 = 0, $istop18 = 0, $k = 0, $m = 0.0, $num_palette = 0, $num_palette8 = 0, $palette = 0, $palette13 = 0, $palette7 = 0, $sb = 0, $sg = 0, $sr = 0;
 var $temp = 0, $temp14 = 0, $temp15 = 0, $temp16 = 0, $temp3 = 0, $temp4 = 0, $v = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $back = sp + 136|0;
 $back_1 = sp + 133|0;
 $back12 = sp + 128|0;
 $0 = $png_ptr;
 $1 = $0;
 $2 = (($1) + 386|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $color_type = $4;
 $5 = $0;
 $6 = (($5) + 204|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 256;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = $color_type;
  $11 = $10 & 2;
  $12 = ($11|0)!=(0);
  if ($12) {
   label = 4;
  } else {
   $13 = $0;
   $14 = (($13) + 196|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $15 | 2048;
   HEAP32[$14>>2] = $16;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $17 = $0;
  $18 = (($17) + 204|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 128;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = $0;
   $23 = (($22) + 204|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = $24 & 256;
   $26 = ($25|0)!=(0);
   if (!($26)) {
    $27 = $0;
    $28 = (($27) + 204|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = $29 & 16384;
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = $0;
     $33 = (($32) + 404|0);
     $34 = (($33) + 2|0);
     $35 = HEAP16[$34>>1]|0;
     $36 = $35&65535;
     $37 = $0;
     $38 = (($37) + 404|0);
     $39 = (($38) + 4|0);
     $40 = HEAP16[$39>>1]|0;
     $41 = $40&65535;
     $42 = ($36|0)==($41|0);
     if ($42) {
      $43 = $0;
      $44 = (($43) + 404|0);
      $45 = (($44) + 2|0);
      $46 = HEAP16[$45>>1]|0;
      $47 = $46&65535;
      $48 = $0;
      $49 = (($48) + 404|0);
      $50 = (($49) + 6|0);
      $51 = HEAP16[$50>>1]|0;
      $52 = $51&65535;
      $53 = ($47|0)==($52|0);
      if ($53) {
       $54 = $0;
       $55 = (($54) + 196|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = $56 | 2048;
       HEAP32[$55>>2] = $57;
       $58 = $0;
       $59 = (($58) + 404|0);
       $60 = (($59) + 2|0);
       $61 = HEAP16[$60>>1]|0;
       $62 = $0;
       $63 = (($62) + 404|0);
       $64 = (($63) + 8|0);
       HEAP16[$64>>1] = $61;
      }
     }
    }
   }
  }
 }
 $65 = $0;
 $66 = (($65) + 204|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = $67 & 256;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = $0;
  $71 = (($70) + 204|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = $72 & 4096;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $color_type;
   $76 = $75 & 2;
   $77 = ($76|0)!=(0);
   if ($77) {
    $230 = $color_type;
    $231 = ($230|0)==(3);
    if ($231) {
     $232 = $0;
     $233 = (($232) + 404|0);
     $234 = HEAP8[$233>>0]|0;
     $235 = $234&255;
     $236 = $0;
     $237 = (($236) + 368|0);
     $238 = HEAP32[$237>>2]|0;
     $239 = (($238) + (($235*3)|0)|0);
     $240 = HEAP8[$239>>0]|0;
     $241 = $240&255;
     $242 = $0;
     $243 = (($242) + 404|0);
     $244 = (($243) + 2|0);
     HEAP16[$244>>1] = $241;
     $245 = $0;
     $246 = (($245) + 404|0);
     $247 = HEAP8[$246>>0]|0;
     $248 = $247&255;
     $249 = $0;
     $250 = (($249) + 368|0);
     $251 = HEAP32[$250>>2]|0;
     $252 = (($251) + (($248*3)|0)|0);
     $253 = (($252) + 1|0);
     $254 = HEAP8[$253>>0]|0;
     $255 = $254&255;
     $256 = $0;
     $257 = (($256) + 404|0);
     $258 = (($257) + 4|0);
     HEAP16[$258>>1] = $255;
     $259 = $0;
     $260 = (($259) + 404|0);
     $261 = HEAP8[$260>>0]|0;
     $262 = $261&255;
     $263 = $0;
     $264 = (($263) + 368|0);
     $265 = HEAP32[$264>>2]|0;
     $266 = (($265) + (($262*3)|0)|0);
     $267 = (($266) + 2|0);
     $268 = HEAP8[$267>>0]|0;
     $269 = $268&255;
     $270 = $0;
     $271 = (($270) + 404|0);
     $272 = (($271) + 6|0);
     HEAP16[$272>>1] = $269;
     $273 = $0;
     $274 = (($273) + 204|0);
     $275 = HEAP32[$274>>2]|0;
     $276 = $275 & 524288;
     $277 = ($276|0)!=(0);
     if ($277) {
      $278 = $0;
      $279 = (($278) + 204|0);
      $280 = HEAP32[$279>>2]|0;
      $281 = $280 & 33554432;
      $282 = ($281|0)!=(0);
      if (!($282)) {
       $283 = $0;
       $284 = (($283) + 374|0);
       $285 = HEAP16[$284>>1]|0;
       $286 = $285&65535;
       $istop = $286;
       $i = 0;
       while(1) {
        $287 = $i;
        $288 = $istop;
        $289 = ($287|0)<($288|0);
        if (!($289)) {
         break;
        }
        $290 = $i;
        $291 = $0;
        $292 = (($291) + 484|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = (($293) + ($290)|0);
        $295 = HEAP8[$294>>0]|0;
        $296 = $295&255;
        $297 = (255 - ($296))|0;
        $298 = $297&255;
        $299 = $i;
        $300 = $0;
        $301 = (($300) + 484|0);
        $302 = HEAP32[$301>>2]|0;
        $303 = (($302) + ($299)|0);
        HEAP8[$303>>0] = $298;
        $304 = $i;
        $305 = (($304) + 1)|0;
        $i = $305;
       }
      }
     }
    }
   } else {
    $78 = $0;
    $79 = (($78) + 387|0);
    $80 = HEAP8[$79>>0]|0;
    $81 = $80&255;
    switch ($81|0) {
    case 1:  {
     $82 = $0;
     $83 = (($82) + 404|0);
     $84 = (($83) + 8|0);
     $85 = HEAP16[$84>>1]|0;
     $86 = $85&65535;
     $87 = ($86*255)|0;
     $88 = $87&65535;
     HEAP16[$84>>1] = $88;
     $89 = $0;
     $90 = (($89) + 404|0);
     $91 = (($90) + 8|0);
     $92 = HEAP16[$91>>1]|0;
     $93 = $0;
     $94 = (($93) + 404|0);
     $95 = (($94) + 6|0);
     HEAP16[$95>>1] = $92;
     $96 = $0;
     $97 = (($96) + 404|0);
     $98 = (($97) + 4|0);
     HEAP16[$98>>1] = $92;
     $99 = $0;
     $100 = (($99) + 404|0);
     $101 = (($100) + 2|0);
     HEAP16[$101>>1] = $92;
     $102 = $0;
     $103 = (($102) + 204|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = $104 & 33554432;
     $106 = ($105|0)!=(0);
     if (!($106)) {
      $107 = $0;
      $108 = (($107) + 488|0);
      $109 = (($108) + 8|0);
      $110 = HEAP16[$109>>1]|0;
      $111 = $110&65535;
      $112 = ($111*255)|0;
      $113 = $112&65535;
      HEAP16[$109>>1] = $113;
      $114 = $0;
      $115 = (($114) + 488|0);
      $116 = (($115) + 8|0);
      $117 = HEAP16[$116>>1]|0;
      $118 = $0;
      $119 = (($118) + 488|0);
      $120 = (($119) + 6|0);
      HEAP16[$120>>1] = $117;
      $121 = $0;
      $122 = (($121) + 488|0);
      $123 = (($122) + 4|0);
      HEAP16[$123>>1] = $117;
      $124 = $0;
      $125 = (($124) + 488|0);
      $126 = (($125) + 2|0);
      HEAP16[$126>>1] = $117;
     }
     break;
    }
    case 2:  {
     $127 = $0;
     $128 = (($127) + 404|0);
     $129 = (($128) + 8|0);
     $130 = HEAP16[$129>>1]|0;
     $131 = $130&65535;
     $132 = ($131*85)|0;
     $133 = $132&65535;
     HEAP16[$129>>1] = $133;
     $134 = $0;
     $135 = (($134) + 404|0);
     $136 = (($135) + 8|0);
     $137 = HEAP16[$136>>1]|0;
     $138 = $0;
     $139 = (($138) + 404|0);
     $140 = (($139) + 6|0);
     HEAP16[$140>>1] = $137;
     $141 = $0;
     $142 = (($141) + 404|0);
     $143 = (($142) + 4|0);
     HEAP16[$143>>1] = $137;
     $144 = $0;
     $145 = (($144) + 404|0);
     $146 = (($145) + 2|0);
     HEAP16[$146>>1] = $137;
     $147 = $0;
     $148 = (($147) + 204|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = $149 & 33554432;
     $151 = ($150|0)!=(0);
     if (!($151)) {
      $152 = $0;
      $153 = (($152) + 488|0);
      $154 = (($153) + 8|0);
      $155 = HEAP16[$154>>1]|0;
      $156 = $155&65535;
      $157 = ($156*85)|0;
      $158 = $157&65535;
      HEAP16[$154>>1] = $158;
      $159 = $0;
      $160 = (($159) + 488|0);
      $161 = (($160) + 8|0);
      $162 = HEAP16[$161>>1]|0;
      $163 = $0;
      $164 = (($163) + 488|0);
      $165 = (($164) + 6|0);
      HEAP16[$165>>1] = $162;
      $166 = $0;
      $167 = (($166) + 488|0);
      $168 = (($167) + 4|0);
      HEAP16[$168>>1] = $162;
      $169 = $0;
      $170 = (($169) + 488|0);
      $171 = (($170) + 2|0);
      HEAP16[$171>>1] = $162;
     }
     break;
    }
    case 4:  {
     $172 = $0;
     $173 = (($172) + 404|0);
     $174 = (($173) + 8|0);
     $175 = HEAP16[$174>>1]|0;
     $176 = $175&65535;
     $177 = ($176*17)|0;
     $178 = $177&65535;
     HEAP16[$174>>1] = $178;
     $179 = $0;
     $180 = (($179) + 404|0);
     $181 = (($180) + 8|0);
     $182 = HEAP16[$181>>1]|0;
     $183 = $0;
     $184 = (($183) + 404|0);
     $185 = (($184) + 6|0);
     HEAP16[$185>>1] = $182;
     $186 = $0;
     $187 = (($186) + 404|0);
     $188 = (($187) + 4|0);
     HEAP16[$188>>1] = $182;
     $189 = $0;
     $190 = (($189) + 404|0);
     $191 = (($190) + 2|0);
     HEAP16[$191>>1] = $182;
     $192 = $0;
     $193 = (($192) + 204|0);
     $194 = HEAP32[$193>>2]|0;
     $195 = $194 & 33554432;
     $196 = ($195|0)!=(0);
     if (!($196)) {
      $197 = $0;
      $198 = (($197) + 488|0);
      $199 = (($198) + 8|0);
      $200 = HEAP16[$199>>1]|0;
      $201 = $200&65535;
      $202 = ($201*17)|0;
      $203 = $202&65535;
      HEAP16[$199>>1] = $203;
      $204 = $0;
      $205 = (($204) + 488|0);
      $206 = (($205) + 8|0);
      $207 = HEAP16[$206>>1]|0;
      $208 = $0;
      $209 = (($208) + 488|0);
      $210 = (($209) + 6|0);
      HEAP16[$210>>1] = $207;
      $211 = $0;
      $212 = (($211) + 488|0);
      $213 = (($212) + 4|0);
      HEAP16[$213>>1] = $207;
      $214 = $0;
      $215 = (($214) + 488|0);
      $216 = (($215) + 2|0);
      HEAP16[$216>>1] = $207;
     }
     break;
    }
    case 16: case 8:  {
     $217 = $0;
     $218 = (($217) + 404|0);
     $219 = (($218) + 8|0);
     $220 = HEAP16[$219>>1]|0;
     $221 = $0;
     $222 = (($221) + 404|0);
     $223 = (($222) + 6|0);
     HEAP16[$223>>1] = $220;
     $224 = $0;
     $225 = (($224) + 404|0);
     $226 = (($225) + 4|0);
     HEAP16[$226>>1] = $220;
     $227 = $0;
     $228 = (($227) + 404|0);
     $229 = (($228) + 2|0);
     HEAP16[$229>>1] = $220;
     break;
    }
    default: {
    }
    }
   }
  }
 }
 $306 = $0;
 $307 = (($306) + 414|0);
 $308 = $0;
 $309 = (($308) + 404|0);
 ;HEAP16[$307+0>>1]=HEAP16[$309+0>>1]|0;HEAP16[$307+2>>1]=HEAP16[$309+2>>1]|0;HEAP16[$307+4>>1]=HEAP16[$309+4>>1]|0;HEAP16[$307+6>>1]=HEAP16[$309+6>>1]|0;HEAP16[$307+8>>1]=HEAP16[$309+8>>1]|0;
 $310 = $color_type;
 $311 = ($310|0)==(3);
 if ($311) {
  $312 = $0;
  $313 = (($312) + 374|0);
  $314 = HEAP16[$313>>1]|0;
  $315 = $314&65535;
  $316 = ($315|0)!=(0);
  if ($316) {
   $317 = $0;
   $318 = (($317) + 444|0);
   $319 = +HEAPF32[$318>>2];
   $320 = $0;
   $321 = (($320) + 440|0);
   $322 = +HEAPF32[$321>>2];
   $323 = $319 * $322;
   $324 = $323;
   $325 = $324 - 1.0;
   $326 = (+Math_abs((+$325)));
   $327 = $326 < 0.0500000000000000027756;
   if ($327) {
    $k = 0;
    $i1 = 0;
    while(1) {
     $328 = $i1;
     $329 = $0;
     $330 = (($329) + 374|0);
     $331 = HEAP16[$330>>1]|0;
     $332 = $331&65535;
     $333 = ($328|0)<($332|0);
     if (!($333)) {
      break;
     }
     $334 = $i1;
     $335 = $0;
     $336 = (($335) + 484|0);
     $337 = HEAP32[$336>>2]|0;
     $338 = (($337) + ($334)|0);
     $339 = HEAP8[$338>>0]|0;
     $340 = $339&255;
     $341 = ($340|0)!=(0);
     if ($341) {
      $342 = $i1;
      $343 = $0;
      $344 = (($343) + 484|0);
      $345 = HEAP32[$344>>2]|0;
      $346 = (($345) + ($342)|0);
      $347 = HEAP8[$346>>0]|0;
      $348 = $347&255;
      $349 = ($348|0)!=(255);
      if ($349) {
       $k = 1;
      }
     }
     $350 = $i1;
     $351 = (($350) + 1)|0;
     $i1 = $351;
    }
    $352 = $k;
    $353 = ($352|0)==(0);
    if ($353) {
     $354 = $0;
     $355 = (($354) + 204|0);
     $356 = HEAP32[$355>>2]|0;
     $357 = $356 & -8193;
     HEAP32[$355>>2] = $357;
    }
   }
  }
 }
 $358 = $0;
 $359 = (($358) + 204|0);
 $360 = HEAP32[$359>>2]|0;
 $361 = $360 & 6299648;
 $362 = ($361|0)!=(0);
 if ($362) {
  $363 = $0;
  $364 = (($363) + 440|0);
  $365 = +HEAPF32[$364>>2];
  $366 = $365;
  $367 = $366 != 0.0;
  if ($367) {
   $368 = $0;
   _png_build_gamma_table($368);
   $369 = $0;
   $370 = (($369) + 204|0);
   $371 = HEAP32[$370>>2]|0;
   $372 = $371 & 128;
   $373 = ($372|0)!=(0);
   if ($373) {
    $374 = $color_type;
    $375 = ($374|0)==(3);
    if ($375) {
     $376 = $0;
     $377 = (($376) + 368|0);
     $378 = HEAP32[$377>>2]|0;
     $palette = $378;
     $379 = $0;
     $380 = (($379) + 372|0);
     $381 = HEAP16[$380>>1]|0;
     $382 = $381&65535;
     $num_palette = $382;
     $383 = $0;
     $384 = (($383) + 396|0);
     $385 = HEAP8[$384>>0]|0;
     $386 = $385&255;
     $387 = ($386|0)==(2);
     if ($387) {
      $388 = $0;
      $389 = (($388) + 404|0);
      $390 = (($389) + 2|0);
      $391 = HEAP16[$390>>1]|0;
      $392 = $391&65535;
      $393 = $0;
      $394 = (($393) + 448|0);
      $395 = HEAP32[$394>>2]|0;
      $396 = (($395) + ($392)|0);
      $397 = HEAP8[$396>>0]|0;
      HEAP8[$back>>0] = $397;
      $398 = $0;
      $399 = (($398) + 404|0);
      $400 = (($399) + 4|0);
      $401 = HEAP16[$400>>1]|0;
      $402 = $401&65535;
      $403 = $0;
      $404 = (($403) + 448|0);
      $405 = HEAP32[$404>>2]|0;
      $406 = (($405) + ($402)|0);
      $407 = HEAP8[$406>>0]|0;
      $408 = (($back) + 1|0);
      HEAP8[$408>>0] = $407;
      $409 = $0;
      $410 = (($409) + 404|0);
      $411 = (($410) + 6|0);
      $412 = HEAP16[$411>>1]|0;
      $413 = $412&65535;
      $414 = $0;
      $415 = (($414) + 448|0);
      $416 = HEAP32[$415>>2]|0;
      $417 = (($416) + ($413)|0);
      $418 = HEAP8[$417>>0]|0;
      $419 = (($back) + 2|0);
      HEAP8[$419>>0] = $418;
      $420 = $0;
      $421 = (($420) + 404|0);
      $422 = (($421) + 2|0);
      $423 = HEAP16[$422>>1]|0;
      $424 = $423&65535;
      $425 = $0;
      $426 = (($425) + 456|0);
      $427 = HEAP32[$426>>2]|0;
      $428 = (($427) + ($424)|0);
      $429 = HEAP8[$428>>0]|0;
      HEAP8[$back_1>>0] = $429;
      $430 = $0;
      $431 = (($430) + 404|0);
      $432 = (($431) + 4|0);
      $433 = HEAP16[$432>>1]|0;
      $434 = $433&65535;
      $435 = $0;
      $436 = (($435) + 456|0);
      $437 = HEAP32[$436>>2]|0;
      $438 = (($437) + ($434)|0);
      $439 = HEAP8[$438>>0]|0;
      $440 = (($back_1) + 1|0);
      HEAP8[$440>>0] = $439;
      $441 = $0;
      $442 = (($441) + 404|0);
      $443 = (($442) + 6|0);
      $444 = HEAP16[$443>>1]|0;
      $445 = $444&65535;
      $446 = $0;
      $447 = (($446) + 456|0);
      $448 = HEAP32[$447>>2]|0;
      $449 = (($448) + ($445)|0);
      $450 = HEAP8[$449>>0]|0;
      $451 = (($back_1) + 2|0);
      HEAP8[$451>>0] = $450;
     } else {
      $452 = $0;
      $453 = (($452) + 396|0);
      $454 = HEAP8[$453>>0]|0;
      $455 = $454&255;
      if ((($455|0) == 2)) {
       $460 = $0;
       $461 = (($460) + 440|0);
       $462 = +HEAPF32[$461>>2];
       $463 = $462;
       $464 = 1.0 / $463;
       $g = $464;
       $465 = $0;
       $466 = (($465) + 440|0);
       $467 = +HEAPF32[$466>>2];
       $468 = $0;
       $469 = (($468) + 444|0);
       $470 = +HEAPF32[$469>>2];
       $471 = $467 * $470;
       $472 = $471;
       $473 = 1.0 / $472;
       $gs = $473;
      } else if ((($455|0) == 3)) {
       $474 = $0;
       $475 = (($474) + 400|0);
       $476 = +HEAPF32[$475>>2];
       $477 = $476;
       $478 = 1.0 / $477;
       $g = $478;
       $479 = $0;
       $480 = (($479) + 400|0);
       $481 = +HEAPF32[$480>>2];
       $482 = $0;
       $483 = (($482) + 444|0);
       $484 = +HEAPF32[$483>>2];
       $485 = $481 * $484;
       $486 = $485;
       $487 = 1.0 / $486;
       $gs = $487;
      } else if ((($455|0) == 1)) {
       $456 = $0;
       $457 = (($456) + 444|0);
       $458 = +HEAPF32[$457>>2];
       $459 = $458;
       $g = $459;
       $gs = 1.0;
      } else {
       $g = 1.0;
       $gs = 1.0;
      }
      $488 = $gs;
      $489 = $488 - 1.0;
      $490 = (+Math_abs((+$489)));
      $491 = $490 < 0.0500000000000000027756;
      if ($491) {
       $492 = $0;
       $493 = (($492) + 404|0);
       $494 = (($493) + 2|0);
       $495 = HEAP16[$494>>1]|0;
       $496 = $495&255;
       HEAP8[$back>>0] = $496;
       $497 = $0;
       $498 = (($497) + 404|0);
       $499 = (($498) + 4|0);
       $500 = HEAP16[$499>>1]|0;
       $501 = $500&255;
       $502 = (($back) + 1|0);
       HEAP8[$502>>0] = $501;
       $503 = $0;
       $504 = (($503) + 404|0);
       $505 = (($504) + 6|0);
       $506 = HEAP16[$505>>1]|0;
       $507 = $506&255;
       $508 = (($back) + 2|0);
       HEAP8[$508>>0] = $507;
      } else {
       $509 = $0;
       $510 = (($509) + 404|0);
       $511 = (($510) + 2|0);
       $512 = HEAP16[$511>>1]|0;
       $513 = (+($512&65535));
       $514 = $513 / 255.0;
       $515 = $gs;
       $516 = (+Math_pow((+$514),(+$515)));
       $517 = $516 * 255.0;
       $518 = $517 + 0.5;
       $519 = (~~(($518))&255);
       HEAP8[$back>>0] = $519;
       $520 = $0;
       $521 = (($520) + 404|0);
       $522 = (($521) + 4|0);
       $523 = HEAP16[$522>>1]|0;
       $524 = (+($523&65535));
       $525 = $524 / 255.0;
       $526 = $gs;
       $527 = (+Math_pow((+$525),(+$526)));
       $528 = $527 * 255.0;
       $529 = $528 + 0.5;
       $530 = (~~(($529))&255);
       $531 = (($back) + 1|0);
       HEAP8[$531>>0] = $530;
       $532 = $0;
       $533 = (($532) + 404|0);
       $534 = (($533) + 6|0);
       $535 = HEAP16[$534>>1]|0;
       $536 = (+($535&65535));
       $537 = $536 / 255.0;
       $538 = $gs;
       $539 = (+Math_pow((+$537),(+$538)));
       $540 = $539 * 255.0;
       $541 = $540 + 0.5;
       $542 = (~~(($541))&255);
       $543 = (($back) + 2|0);
       HEAP8[$543>>0] = $542;
      }
      $544 = $0;
      $545 = (($544) + 404|0);
      $546 = (($545) + 2|0);
      $547 = HEAP16[$546>>1]|0;
      $548 = (+($547&65535));
      $549 = $548 / 255.0;
      $550 = $g;
      $551 = (+Math_pow((+$549),(+$550)));
      $552 = $551 * 255.0;
      $553 = $552 + 0.5;
      $554 = (~~(($553))&255);
      HEAP8[$back_1>>0] = $554;
      $555 = $0;
      $556 = (($555) + 404|0);
      $557 = (($556) + 4|0);
      $558 = HEAP16[$557>>1]|0;
      $559 = (+($558&65535));
      $560 = $559 / 255.0;
      $561 = $g;
      $562 = (+Math_pow((+$560),(+$561)));
      $563 = $562 * 255.0;
      $564 = $563 + 0.5;
      $565 = (~~(($564))&255);
      $566 = (($back_1) + 1|0);
      HEAP8[$566>>0] = $565;
      $567 = $0;
      $568 = (($567) + 404|0);
      $569 = (($568) + 6|0);
      $570 = HEAP16[$569>>1]|0;
      $571 = (+($570&65535));
      $572 = $571 / 255.0;
      $573 = $g;
      $574 = (+Math_pow((+$572),(+$573)));
      $575 = $574 * 255.0;
      $576 = $575 + 0.5;
      $577 = (~~(($576))&255);
      $578 = (($back_1) + 2|0);
      HEAP8[$578>>0] = $577;
     }
     $i2 = 0;
     while(1) {
      $579 = $i2;
      $580 = $num_palette;
      $581 = ($579|0)<($580|0);
      if (!($581)) {
       break;
      }
      $582 = $i2;
      $583 = $0;
      $584 = (($583) + 374|0);
      $585 = HEAP16[$584>>1]|0;
      $586 = $585&65535;
      $587 = ($582|0)<($586|0);
      if ($587) {
       $588 = $i2;
       $589 = $0;
       $590 = (($589) + 484|0);
       $591 = HEAP32[$590>>2]|0;
       $592 = (($591) + ($588)|0);
       $593 = HEAP8[$592>>0]|0;
       $594 = $593&255;
       $595 = ($594|0)!=(255);
       if ($595) {
        $596 = $i2;
        $597 = $0;
        $598 = (($597) + 484|0);
        $599 = HEAP32[$598>>2]|0;
        $600 = (($599) + ($596)|0);
        $601 = HEAP8[$600>>0]|0;
        $602 = $601&255;
        $603 = ($602|0)==(0);
        if ($603) {
         $604 = $i2;
         $605 = $palette;
         $606 = (($605) + (($604*3)|0)|0);
         ;HEAP8[$606+0>>0]=HEAP8[$back+0>>0]|0;HEAP8[$606+1>>0]=HEAP8[$back+1>>0]|0;HEAP8[$606+2>>0]=HEAP8[$back+2>>0]|0;
        } else {
         $607 = $i2;
         $608 = $palette;
         $609 = (($608) + (($607*3)|0)|0);
         $610 = HEAP8[$609>>0]|0;
         $611 = $610&255;
         $612 = $0;
         $613 = (($612) + 456|0);
         $614 = HEAP32[$613>>2]|0;
         $615 = (($614) + ($611)|0);
         $616 = HEAP8[$615>>0]|0;
         $v = $616;
         $617 = $v;
         $618 = $617&255;
         $619 = $618&65535;
         $620 = $i2;
         $621 = $0;
         $622 = (($621) + 484|0);
         $623 = HEAP32[$622>>2]|0;
         $624 = (($623) + ($620)|0);
         $625 = HEAP8[$624>>0]|0;
         $626 = $625&255;
         $627 = $626&65535;
         $628 = Math_imul($619, $627)|0;
         $629 = HEAP8[$back_1>>0]|0;
         $630 = $629&255;
         $631 = $630&65535;
         $632 = $i2;
         $633 = $0;
         $634 = (($633) + 484|0);
         $635 = HEAP32[$634>>2]|0;
         $636 = (($635) + ($632)|0);
         $637 = HEAP8[$636>>0]|0;
         $638 = $637&255;
         $639 = $638&65535;
         $640 = (255 - ($639))|0;
         $641 = $640&65535;
         $642 = $641&65535;
         $643 = Math_imul($631, $642)|0;
         $644 = (($628) + ($643))|0;
         $645 = (($644) + 128)|0;
         $646 = $645&65535;
         $temp = $646;
         $647 = $temp;
         $648 = $647&65535;
         $649 = $temp;
         $650 = $649&65535;
         $651 = $650 >> 8;
         $652 = (($648) + ($651))|0;
         $653 = $652 >> 8;
         $654 = $653&255;
         $w = $654;
         $655 = $w;
         $656 = $655&255;
         $657 = $0;
         $658 = (($657) + 452|0);
         $659 = HEAP32[$658>>2]|0;
         $660 = (($659) + ($656)|0);
         $661 = HEAP8[$660>>0]|0;
         $662 = $i2;
         $663 = $palette;
         $664 = (($663) + (($662*3)|0)|0);
         HEAP8[$664>>0] = $661;
         $665 = $i2;
         $666 = $palette;
         $667 = (($666) + (($665*3)|0)|0);
         $668 = (($667) + 1|0);
         $669 = HEAP8[$668>>0]|0;
         $670 = $669&255;
         $671 = $0;
         $672 = (($671) + 456|0);
         $673 = HEAP32[$672>>2]|0;
         $674 = (($673) + ($670)|0);
         $675 = HEAP8[$674>>0]|0;
         $v = $675;
         $676 = $v;
         $677 = $676&255;
         $678 = $677&65535;
         $679 = $i2;
         $680 = $0;
         $681 = (($680) + 484|0);
         $682 = HEAP32[$681>>2]|0;
         $683 = (($682) + ($679)|0);
         $684 = HEAP8[$683>>0]|0;
         $685 = $684&255;
         $686 = $685&65535;
         $687 = Math_imul($678, $686)|0;
         $688 = (($back_1) + 1|0);
         $689 = HEAP8[$688>>0]|0;
         $690 = $689&255;
         $691 = $690&65535;
         $692 = $i2;
         $693 = $0;
         $694 = (($693) + 484|0);
         $695 = HEAP32[$694>>2]|0;
         $696 = (($695) + ($692)|0);
         $697 = HEAP8[$696>>0]|0;
         $698 = $697&255;
         $699 = $698&65535;
         $700 = (255 - ($699))|0;
         $701 = $700&65535;
         $702 = $701&65535;
         $703 = Math_imul($691, $702)|0;
         $704 = (($687) + ($703))|0;
         $705 = (($704) + 128)|0;
         $706 = $705&65535;
         $temp3 = $706;
         $707 = $temp3;
         $708 = $707&65535;
         $709 = $temp3;
         $710 = $709&65535;
         $711 = $710 >> 8;
         $712 = (($708) + ($711))|0;
         $713 = $712 >> 8;
         $714 = $713&255;
         $w = $714;
         $715 = $w;
         $716 = $715&255;
         $717 = $0;
         $718 = (($717) + 452|0);
         $719 = HEAP32[$718>>2]|0;
         $720 = (($719) + ($716)|0);
         $721 = HEAP8[$720>>0]|0;
         $722 = $i2;
         $723 = $palette;
         $724 = (($723) + (($722*3)|0)|0);
         $725 = (($724) + 1|0);
         HEAP8[$725>>0] = $721;
         $726 = $i2;
         $727 = $palette;
         $728 = (($727) + (($726*3)|0)|0);
         $729 = (($728) + 2|0);
         $730 = HEAP8[$729>>0]|0;
         $731 = $730&255;
         $732 = $0;
         $733 = (($732) + 456|0);
         $734 = HEAP32[$733>>2]|0;
         $735 = (($734) + ($731)|0);
         $736 = HEAP8[$735>>0]|0;
         $v = $736;
         $737 = $v;
         $738 = $737&255;
         $739 = $738&65535;
         $740 = $i2;
         $741 = $0;
         $742 = (($741) + 484|0);
         $743 = HEAP32[$742>>2]|0;
         $744 = (($743) + ($740)|0);
         $745 = HEAP8[$744>>0]|0;
         $746 = $745&255;
         $747 = $746&65535;
         $748 = Math_imul($739, $747)|0;
         $749 = (($back_1) + 2|0);
         $750 = HEAP8[$749>>0]|0;
         $751 = $750&255;
         $752 = $751&65535;
         $753 = $i2;
         $754 = $0;
         $755 = (($754) + 484|0);
         $756 = HEAP32[$755>>2]|0;
         $757 = (($756) + ($753)|0);
         $758 = HEAP8[$757>>0]|0;
         $759 = $758&255;
         $760 = $759&65535;
         $761 = (255 - ($760))|0;
         $762 = $761&65535;
         $763 = $762&65535;
         $764 = Math_imul($752, $763)|0;
         $765 = (($748) + ($764))|0;
         $766 = (($765) + 128)|0;
         $767 = $766&65535;
         $temp4 = $767;
         $768 = $temp4;
         $769 = $768&65535;
         $770 = $temp4;
         $771 = $770&65535;
         $772 = $771 >> 8;
         $773 = (($769) + ($772))|0;
         $774 = $773 >> 8;
         $775 = $774&255;
         $w = $775;
         $776 = $w;
         $777 = $776&255;
         $778 = $0;
         $779 = (($778) + 452|0);
         $780 = HEAP32[$779>>2]|0;
         $781 = (($780) + ($777)|0);
         $782 = HEAP8[$781>>0]|0;
         $783 = $i2;
         $784 = $palette;
         $785 = (($784) + (($783*3)|0)|0);
         $786 = (($785) + 2|0);
         HEAP8[$786>>0] = $782;
        }
       } else {
        label = 74;
       }
      } else {
       label = 74;
      }
      if ((label|0) == 74) {
       label = 0;
       $787 = $i2;
       $788 = $palette;
       $789 = (($788) + (($787*3)|0)|0);
       $790 = HEAP8[$789>>0]|0;
       $791 = $790&255;
       $792 = $0;
       $793 = (($792) + 448|0);
       $794 = HEAP32[$793>>2]|0;
       $795 = (($794) + ($791)|0);
       $796 = HEAP8[$795>>0]|0;
       $797 = $i2;
       $798 = $palette;
       $799 = (($798) + (($797*3)|0)|0);
       HEAP8[$799>>0] = $796;
       $800 = $i2;
       $801 = $palette;
       $802 = (($801) + (($800*3)|0)|0);
       $803 = (($802) + 1|0);
       $804 = HEAP8[$803>>0]|0;
       $805 = $804&255;
       $806 = $0;
       $807 = (($806) + 448|0);
       $808 = HEAP32[$807>>2]|0;
       $809 = (($808) + ($805)|0);
       $810 = HEAP8[$809>>0]|0;
       $811 = $i2;
       $812 = $palette;
       $813 = (($812) + (($811*3)|0)|0);
       $814 = (($813) + 1|0);
       HEAP8[$814>>0] = $810;
       $815 = $i2;
       $816 = $palette;
       $817 = (($816) + (($815*3)|0)|0);
       $818 = (($817) + 2|0);
       $819 = HEAP8[$818>>0]|0;
       $820 = $819&255;
       $821 = $0;
       $822 = (($821) + 448|0);
       $823 = HEAP32[$822>>2]|0;
       $824 = (($823) + ($820)|0);
       $825 = HEAP8[$824>>0]|0;
       $826 = $i2;
       $827 = $palette;
       $828 = (($827) + (($826*3)|0)|0);
       $829 = (($828) + 2|0);
       HEAP8[$829>>0] = $825;
      }
      $830 = $i2;
      $831 = (($830) + 1)|0;
      $i2 = $831;
     }
    } else {
     $832 = $0;
     $833 = (($832) + 387|0);
     $834 = HEAP8[$833>>0]|0;
     $835 = $834&255;
     $836 = 1 << $835;
     $837 = (($836) - 1)|0;
     $838 = (+($837>>>0));
     $m = $838;
     $g5 = 1.0;
     $gs6 = 1.0;
     $839 = $0;
     $840 = (($839) + 396|0);
     $841 = HEAP8[$840>>0]|0;
     $842 = $841&255;
     if ((($842|0) == 1)) {
      $843 = $0;
      $844 = (($843) + 444|0);
      $845 = +HEAPF32[$844>>2];
      $846 = $845;
      $g5 = $846;
      $gs6 = 1.0;
     } else if ((($842|0) == 2)) {
      $847 = $0;
      $848 = (($847) + 440|0);
      $849 = +HEAPF32[$848>>2];
      $850 = $849;
      $851 = 1.0 / $850;
      $g5 = $851;
      $852 = $0;
      $853 = (($852) + 440|0);
      $854 = +HEAPF32[$853>>2];
      $855 = $0;
      $856 = (($855) + 444|0);
      $857 = +HEAPF32[$856>>2];
      $858 = $854 * $857;
      $859 = $858;
      $860 = 1.0 / $859;
      $gs6 = $860;
     } else if ((($842|0) == 3)) {
      $861 = $0;
      $862 = (($861) + 400|0);
      $863 = +HEAPF32[$862>>2];
      $864 = $863;
      $865 = 1.0 / $864;
      $g5 = $865;
      $866 = $0;
      $867 = (($866) + 400|0);
      $868 = +HEAPF32[$867>>2];
      $869 = $0;
      $870 = (($869) + 444|0);
      $871 = +HEAPF32[$870>>2];
      $872 = $868 * $871;
      $873 = $872;
      $874 = 1.0 / $873;
      $gs6 = $874;
     }
     $875 = $0;
     $876 = (($875) + 404|0);
     $877 = (($876) + 8|0);
     $878 = HEAP16[$877>>1]|0;
     $879 = (+($878&65535));
     $880 = $m;
     $881 = $879 / $880;
     $882 = $g5;
     $883 = (+Math_pow((+$881),(+$882)));
     $884 = $m;
     $885 = $883 * $884;
     $886 = $885 + 0.5;
     $887 = (~~(($886))&65535);
     $888 = $0;
     $889 = (($888) + 414|0);
     $890 = (($889) + 8|0);
     HEAP16[$890>>1] = $887;
     $891 = $0;
     $892 = (($891) + 404|0);
     $893 = (($892) + 8|0);
     $894 = HEAP16[$893>>1]|0;
     $895 = (+($894&65535));
     $896 = $m;
     $897 = $895 / $896;
     $898 = $gs6;
     $899 = (+Math_pow((+$897),(+$898)));
     $900 = $m;
     $901 = $899 * $900;
     $902 = $901 + 0.5;
     $903 = (~~(($902))&65535);
     $904 = $0;
     $905 = (($904) + 404|0);
     $906 = (($905) + 8|0);
     HEAP16[$906>>1] = $903;
     $907 = $0;
     $908 = (($907) + 404|0);
     $909 = (($908) + 2|0);
     $910 = HEAP16[$909>>1]|0;
     $911 = $910&65535;
     $912 = $0;
     $913 = (($912) + 404|0);
     $914 = (($913) + 4|0);
     $915 = HEAP16[$914>>1]|0;
     $916 = $915&65535;
     $917 = ($911|0)!=($916|0);
     if ($917) {
      label = 85;
     } else {
      $918 = $0;
      $919 = (($918) + 404|0);
      $920 = (($919) + 2|0);
      $921 = HEAP16[$920>>1]|0;
      $922 = $921&65535;
      $923 = $0;
      $924 = (($923) + 404|0);
      $925 = (($924) + 6|0);
      $926 = HEAP16[$925>>1]|0;
      $927 = $926&65535;
      $928 = ($922|0)!=($927|0);
      if ($928) {
       label = 85;
      } else {
       $929 = $0;
       $930 = (($929) + 404|0);
       $931 = (($930) + 2|0);
       $932 = HEAP16[$931>>1]|0;
       $933 = $932&65535;
       $934 = $0;
       $935 = (($934) + 404|0);
       $936 = (($935) + 8|0);
       $937 = HEAP16[$936>>1]|0;
       $938 = $937&65535;
       $939 = ($933|0)!=($938|0);
       if ($939) {
        label = 85;
       } else {
        $1036 = $0;
        $1037 = (($1036) + 414|0);
        $1038 = (($1037) + 8|0);
        $1039 = HEAP16[$1038>>1]|0;
        $1040 = $0;
        $1041 = (($1040) + 414|0);
        $1042 = (($1041) + 6|0);
        HEAP16[$1042>>1] = $1039;
        $1043 = $0;
        $1044 = (($1043) + 414|0);
        $1045 = (($1044) + 4|0);
        HEAP16[$1045>>1] = $1039;
        $1046 = $0;
        $1047 = (($1046) + 414|0);
        $1048 = (($1047) + 2|0);
        HEAP16[$1048>>1] = $1039;
        $1049 = $0;
        $1050 = (($1049) + 404|0);
        $1051 = (($1050) + 8|0);
        $1052 = HEAP16[$1051>>1]|0;
        $1053 = $0;
        $1054 = (($1053) + 404|0);
        $1055 = (($1054) + 6|0);
        HEAP16[$1055>>1] = $1052;
        $1056 = $0;
        $1057 = (($1056) + 404|0);
        $1058 = (($1057) + 4|0);
        HEAP16[$1058>>1] = $1052;
        $1059 = $0;
        $1060 = (($1059) + 404|0);
        $1061 = (($1060) + 2|0);
        HEAP16[$1061>>1] = $1052;
       }
      }
     }
     if ((label|0) == 85) {
      $940 = $0;
      $941 = (($940) + 404|0);
      $942 = (($941) + 2|0);
      $943 = HEAP16[$942>>1]|0;
      $944 = (+($943&65535));
      $945 = $m;
      $946 = $944 / $945;
      $947 = $g5;
      $948 = (+Math_pow((+$946),(+$947)));
      $949 = $m;
      $950 = $948 * $949;
      $951 = $950 + 0.5;
      $952 = (~~(($951))&65535);
      $953 = $0;
      $954 = (($953) + 414|0);
      $955 = (($954) + 2|0);
      HEAP16[$955>>1] = $952;
      $956 = $0;
      $957 = (($956) + 404|0);
      $958 = (($957) + 4|0);
      $959 = HEAP16[$958>>1]|0;
      $960 = (+($959&65535));
      $961 = $m;
      $962 = $960 / $961;
      $963 = $g5;
      $964 = (+Math_pow((+$962),(+$963)));
      $965 = $m;
      $966 = $964 * $965;
      $967 = $966 + 0.5;
      $968 = (~~(($967))&65535);
      $969 = $0;
      $970 = (($969) + 414|0);
      $971 = (($970) + 4|0);
      HEAP16[$971>>1] = $968;
      $972 = $0;
      $973 = (($972) + 404|0);
      $974 = (($973) + 6|0);
      $975 = HEAP16[$974>>1]|0;
      $976 = (+($975&65535));
      $977 = $m;
      $978 = $976 / $977;
      $979 = $g5;
      $980 = (+Math_pow((+$978),(+$979)));
      $981 = $m;
      $982 = $980 * $981;
      $983 = $982 + 0.5;
      $984 = (~~(($983))&65535);
      $985 = $0;
      $986 = (($985) + 414|0);
      $987 = (($986) + 6|0);
      HEAP16[$987>>1] = $984;
      $988 = $0;
      $989 = (($988) + 404|0);
      $990 = (($989) + 2|0);
      $991 = HEAP16[$990>>1]|0;
      $992 = (+($991&65535));
      $993 = $m;
      $994 = $992 / $993;
      $995 = $gs6;
      $996 = (+Math_pow((+$994),(+$995)));
      $997 = $m;
      $998 = $996 * $997;
      $999 = $998 + 0.5;
      $1000 = (~~(($999))&65535);
      $1001 = $0;
      $1002 = (($1001) + 404|0);
      $1003 = (($1002) + 2|0);
      HEAP16[$1003>>1] = $1000;
      $1004 = $0;
      $1005 = (($1004) + 404|0);
      $1006 = (($1005) + 4|0);
      $1007 = HEAP16[$1006>>1]|0;
      $1008 = (+($1007&65535));
      $1009 = $m;
      $1010 = $1008 / $1009;
      $1011 = $gs6;
      $1012 = (+Math_pow((+$1010),(+$1011)));
      $1013 = $m;
      $1014 = $1012 * $1013;
      $1015 = $1014 + 0.5;
      $1016 = (~~(($1015))&65535);
      $1017 = $0;
      $1018 = (($1017) + 404|0);
      $1019 = (($1018) + 4|0);
      HEAP16[$1019>>1] = $1016;
      $1020 = $0;
      $1021 = (($1020) + 404|0);
      $1022 = (($1021) + 6|0);
      $1023 = HEAP16[$1022>>1]|0;
      $1024 = (+($1023&65535));
      $1025 = $m;
      $1026 = $1024 / $1025;
      $1027 = $gs6;
      $1028 = (+Math_pow((+$1026),(+$1027)));
      $1029 = $m;
      $1030 = $1028 * $1029;
      $1031 = $1030 + 0.5;
      $1032 = (~~(($1031))&65535);
      $1033 = $0;
      $1034 = (($1033) + 404|0);
      $1035 = (($1034) + 6|0);
      HEAP16[$1035>>1] = $1032;
     }
    }
   } else {
    $1062 = $color_type;
    $1063 = ($1062|0)==(3);
    if ($1063) {
     $1064 = $0;
     $1065 = (($1064) + 368|0);
     $1066 = HEAP32[$1065>>2]|0;
     $palette7 = $1066;
     $1067 = $0;
     $1068 = (($1067) + 372|0);
     $1069 = HEAP16[$1068>>1]|0;
     $1070 = $1069&65535;
     $num_palette8 = $1070;
     $i9 = 0;
     while(1) {
      $1071 = $i9;
      $1072 = $num_palette8;
      $1073 = ($1071|0)<($1072|0);
      if (!($1073)) {
       break;
      }
      $1074 = $i9;
      $1075 = $palette7;
      $1076 = (($1075) + (($1074*3)|0)|0);
      $1077 = HEAP8[$1076>>0]|0;
      $1078 = $1077&255;
      $1079 = $0;
      $1080 = (($1079) + 448|0);
      $1081 = HEAP32[$1080>>2]|0;
      $1082 = (($1081) + ($1078)|0);
      $1083 = HEAP8[$1082>>0]|0;
      $1084 = $i9;
      $1085 = $palette7;
      $1086 = (($1085) + (($1084*3)|0)|0);
      HEAP8[$1086>>0] = $1083;
      $1087 = $i9;
      $1088 = $palette7;
      $1089 = (($1088) + (($1087*3)|0)|0);
      $1090 = (($1089) + 1|0);
      $1091 = HEAP8[$1090>>0]|0;
      $1092 = $1091&255;
      $1093 = $0;
      $1094 = (($1093) + 448|0);
      $1095 = HEAP32[$1094>>2]|0;
      $1096 = (($1095) + ($1092)|0);
      $1097 = HEAP8[$1096>>0]|0;
      $1098 = $i9;
      $1099 = $palette7;
      $1100 = (($1099) + (($1098*3)|0)|0);
      $1101 = (($1100) + 1|0);
      HEAP8[$1101>>0] = $1097;
      $1102 = $i9;
      $1103 = $palette7;
      $1104 = (($1103) + (($1102*3)|0)|0);
      $1105 = (($1104) + 2|0);
      $1106 = HEAP8[$1105>>0]|0;
      $1107 = $1106&255;
      $1108 = $0;
      $1109 = (($1108) + 448|0);
      $1110 = HEAP32[$1109>>2]|0;
      $1111 = (($1110) + ($1107)|0);
      $1112 = HEAP8[$1111>>0]|0;
      $1113 = $i9;
      $1114 = $palette7;
      $1115 = (($1114) + (($1113*3)|0)|0);
      $1116 = (($1115) + 2|0);
      HEAP8[$1116>>0] = $1112;
      $1117 = $i9;
      $1118 = (($1117) + 1)|0;
      $i9 = $1118;
     }
    }
   }
  } else {
   label = 97;
  }
 } else {
  label = 97;
 }
 if ((label|0) == 97) {
  $1119 = $0;
  $1120 = (($1119) + 204|0);
  $1121 = HEAP32[$1120>>2]|0;
  $1122 = $1121 & 128;
  $1123 = ($1122|0)!=(0);
  if ($1123) {
   $1124 = $color_type;
   $1125 = ($1124|0)==(3);
   if ($1125) {
    $1126 = $0;
    $1127 = (($1126) + 374|0);
    $1128 = HEAP16[$1127>>1]|0;
    $1129 = $1128&65535;
    $istop11 = $1129;
    $1130 = $0;
    $1131 = (($1130) + 368|0);
    $1132 = HEAP32[$1131>>2]|0;
    $palette13 = $1132;
    $1133 = $0;
    $1134 = (($1133) + 404|0);
    $1135 = (($1134) + 2|0);
    $1136 = HEAP16[$1135>>1]|0;
    $1137 = $1136&255;
    HEAP8[$back12>>0] = $1137;
    $1138 = $0;
    $1139 = (($1138) + 404|0);
    $1140 = (($1139) + 4|0);
    $1141 = HEAP16[$1140>>1]|0;
    $1142 = $1141&255;
    $1143 = (($back12) + 1|0);
    HEAP8[$1143>>0] = $1142;
    $1144 = $0;
    $1145 = (($1144) + 404|0);
    $1146 = (($1145) + 6|0);
    $1147 = HEAP16[$1146>>1]|0;
    $1148 = $1147&255;
    $1149 = (($back12) + 2|0);
    HEAP8[$1149>>0] = $1148;
    $i10 = 0;
    while(1) {
     $1150 = $i10;
     $1151 = $istop11;
     $1152 = ($1150|0)<($1151|0);
     if (!($1152)) {
      break;
     }
     $1153 = $i10;
     $1154 = $0;
     $1155 = (($1154) + 484|0);
     $1156 = HEAP32[$1155>>2]|0;
     $1157 = (($1156) + ($1153)|0);
     $1158 = HEAP8[$1157>>0]|0;
     $1159 = $1158&255;
     $1160 = ($1159|0)==(0);
     if ($1160) {
      $1161 = $i10;
      $1162 = $palette13;
      $1163 = (($1162) + (($1161*3)|0)|0);
      ;HEAP8[$1163+0>>0]=HEAP8[$back12+0>>0]|0;HEAP8[$1163+1>>0]=HEAP8[$back12+1>>0]|0;HEAP8[$1163+2>>0]=HEAP8[$back12+2>>0]|0;
     } else {
      $1164 = $i10;
      $1165 = $0;
      $1166 = (($1165) + 484|0);
      $1167 = HEAP32[$1166>>2]|0;
      $1168 = (($1167) + ($1164)|0);
      $1169 = HEAP8[$1168>>0]|0;
      $1170 = $1169&255;
      $1171 = ($1170|0)!=(255);
      if ($1171) {
       $1172 = $i10;
       $1173 = $palette13;
       $1174 = (($1173) + (($1172*3)|0)|0);
       $1175 = HEAP8[$1174>>0]|0;
       $1176 = $1175&255;
       $1177 = $1176&65535;
       $1178 = $i10;
       $1179 = $0;
       $1180 = (($1179) + 484|0);
       $1181 = HEAP32[$1180>>2]|0;
       $1182 = (($1181) + ($1178)|0);
       $1183 = HEAP8[$1182>>0]|0;
       $1184 = $1183&255;
       $1185 = $1184&65535;
       $1186 = Math_imul($1177, $1185)|0;
       $1187 = HEAP8[$back12>>0]|0;
       $1188 = $1187&255;
       $1189 = $1188&65535;
       $1190 = $i10;
       $1191 = $0;
       $1192 = (($1191) + 484|0);
       $1193 = HEAP32[$1192>>2]|0;
       $1194 = (($1193) + ($1190)|0);
       $1195 = HEAP8[$1194>>0]|0;
       $1196 = $1195&255;
       $1197 = $1196&65535;
       $1198 = (255 - ($1197))|0;
       $1199 = $1198&65535;
       $1200 = $1199&65535;
       $1201 = Math_imul($1189, $1200)|0;
       $1202 = (($1186) + ($1201))|0;
       $1203 = (($1202) + 128)|0;
       $1204 = $1203&65535;
       $temp14 = $1204;
       $1205 = $temp14;
       $1206 = $1205&65535;
       $1207 = $temp14;
       $1208 = $1207&65535;
       $1209 = $1208 >> 8;
       $1210 = (($1206) + ($1209))|0;
       $1211 = $1210 >> 8;
       $1212 = $1211&255;
       $1213 = $i10;
       $1214 = $palette13;
       $1215 = (($1214) + (($1213*3)|0)|0);
       HEAP8[$1215>>0] = $1212;
       $1216 = $i10;
       $1217 = $palette13;
       $1218 = (($1217) + (($1216*3)|0)|0);
       $1219 = (($1218) + 1|0);
       $1220 = HEAP8[$1219>>0]|0;
       $1221 = $1220&255;
       $1222 = $1221&65535;
       $1223 = $i10;
       $1224 = $0;
       $1225 = (($1224) + 484|0);
       $1226 = HEAP32[$1225>>2]|0;
       $1227 = (($1226) + ($1223)|0);
       $1228 = HEAP8[$1227>>0]|0;
       $1229 = $1228&255;
       $1230 = $1229&65535;
       $1231 = Math_imul($1222, $1230)|0;
       $1232 = (($back12) + 1|0);
       $1233 = HEAP8[$1232>>0]|0;
       $1234 = $1233&255;
       $1235 = $1234&65535;
       $1236 = $i10;
       $1237 = $0;
       $1238 = (($1237) + 484|0);
       $1239 = HEAP32[$1238>>2]|0;
       $1240 = (($1239) + ($1236)|0);
       $1241 = HEAP8[$1240>>0]|0;
       $1242 = $1241&255;
       $1243 = $1242&65535;
       $1244 = (255 - ($1243))|0;
       $1245 = $1244&65535;
       $1246 = $1245&65535;
       $1247 = Math_imul($1235, $1246)|0;
       $1248 = (($1231) + ($1247))|0;
       $1249 = (($1248) + 128)|0;
       $1250 = $1249&65535;
       $temp15 = $1250;
       $1251 = $temp15;
       $1252 = $1251&65535;
       $1253 = $temp15;
       $1254 = $1253&65535;
       $1255 = $1254 >> 8;
       $1256 = (($1252) + ($1255))|0;
       $1257 = $1256 >> 8;
       $1258 = $1257&255;
       $1259 = $i10;
       $1260 = $palette13;
       $1261 = (($1260) + (($1259*3)|0)|0);
       $1262 = (($1261) + 1|0);
       HEAP8[$1262>>0] = $1258;
       $1263 = $i10;
       $1264 = $palette13;
       $1265 = (($1264) + (($1263*3)|0)|0);
       $1266 = (($1265) + 2|0);
       $1267 = HEAP8[$1266>>0]|0;
       $1268 = $1267&255;
       $1269 = $1268&65535;
       $1270 = $i10;
       $1271 = $0;
       $1272 = (($1271) + 484|0);
       $1273 = HEAP32[$1272>>2]|0;
       $1274 = (($1273) + ($1270)|0);
       $1275 = HEAP8[$1274>>0]|0;
       $1276 = $1275&255;
       $1277 = $1276&65535;
       $1278 = Math_imul($1269, $1277)|0;
       $1279 = (($back12) + 2|0);
       $1280 = HEAP8[$1279>>0]|0;
       $1281 = $1280&255;
       $1282 = $1281&65535;
       $1283 = $i10;
       $1284 = $0;
       $1285 = (($1284) + 484|0);
       $1286 = HEAP32[$1285>>2]|0;
       $1287 = (($1286) + ($1283)|0);
       $1288 = HEAP8[$1287>>0]|0;
       $1289 = $1288&255;
       $1290 = $1289&65535;
       $1291 = (255 - ($1290))|0;
       $1292 = $1291&65535;
       $1293 = $1292&65535;
       $1294 = Math_imul($1282, $1293)|0;
       $1295 = (($1278) + ($1294))|0;
       $1296 = (($1295) + 128)|0;
       $1297 = $1296&65535;
       $temp16 = $1297;
       $1298 = $temp16;
       $1299 = $1298&65535;
       $1300 = $temp16;
       $1301 = $1300&65535;
       $1302 = $1301 >> 8;
       $1303 = (($1299) + ($1302))|0;
       $1304 = $1303 >> 8;
       $1305 = $1304&255;
       $1306 = $i10;
       $1307 = $palette13;
       $1308 = (($1307) + (($1306*3)|0)|0);
       $1309 = (($1308) + 2|0);
       HEAP8[$1309>>0] = $1305;
      }
     }
     $1310 = $i10;
     $1311 = (($1310) + 1)|0;
     $i10 = $1311;
    }
   }
  }
 }
 $1312 = $0;
 $1313 = (($1312) + 204|0);
 $1314 = HEAP32[$1313>>2]|0;
 $1315 = $1314 & 8;
 $1316 = ($1315|0)!=(0);
 if (!($1316)) {
  STACKTOP = sp;return;
 }
 $1317 = $color_type;
 $1318 = ($1317|0)==(3);
 if (!($1318)) {
  STACKTOP = sp;return;
 }
 $1319 = $0;
 $1320 = (($1319) + 372|0);
 $1321 = HEAP16[$1320>>1]|0;
 $istop18 = $1321;
 $1322 = $0;
 $1323 = (($1322) + 472|0);
 $1324 = HEAP8[$1323>>0]|0;
 $1325 = $1324&255;
 $1326 = (8 - ($1325))|0;
 $sr = $1326;
 $1327 = $0;
 $1328 = (($1327) + 472|0);
 $1329 = (($1328) + 1|0);
 $1330 = HEAP8[$1329>>0]|0;
 $1331 = $1330&255;
 $1332 = (8 - ($1331))|0;
 $sg = $1332;
 $1333 = $0;
 $1334 = (($1333) + 472|0);
 $1335 = (($1334) + 2|0);
 $1336 = HEAP8[$1335>>0]|0;
 $1337 = $1336&255;
 $1338 = (8 - ($1337))|0;
 $sb = $1338;
 $1339 = $sr;
 $1340 = ($1339|0)<(0);
 if ($1340) {
  label = 114;
 } else {
  $1341 = $sr;
  $1342 = ($1341|0)>(8);
  if ($1342) {
   label = 114;
  }
 }
 if ((label|0) == 114) {
  $sr = 0;
 }
 $1343 = $sg;
 $1344 = ($1343|0)<(0);
 if ($1344) {
  label = 117;
 } else {
  $1345 = $sg;
  $1346 = ($1345|0)>(8);
  if ($1346) {
   label = 117;
  }
 }
 if ((label|0) == 117) {
  $sg = 0;
 }
 $1347 = $sb;
 $1348 = ($1347|0)<(0);
 if ($1348) {
  label = 120;
 } else {
  $1349 = $sb;
  $1350 = ($1349|0)>(8);
  if ($1350) {
   label = 120;
  }
 }
 if ((label|0) == 120) {
  $sb = 0;
 }
 $i17 = 0;
 while(1) {
  $1351 = $i17;
  $1352 = $1351&65535;
  $1353 = $istop18;
  $1354 = $1353&65535;
  $1355 = ($1352|0)<($1354|0);
  if (!($1355)) {
   break;
  }
  $1356 = $sr;
  $1357 = $i17;
  $1358 = $1357&65535;
  $1359 = $0;
  $1360 = (($1359) + 368|0);
  $1361 = HEAP32[$1360>>2]|0;
  $1362 = (($1361) + (($1358*3)|0)|0);
  $1363 = HEAP8[$1362>>0]|0;
  $1364 = $1363&255;
  $1365 = $1364 >> $1356;
  $1366 = $1365&255;
  HEAP8[$1362>>0] = $1366;
  $1367 = $sg;
  $1368 = $i17;
  $1369 = $1368&65535;
  $1370 = $0;
  $1371 = (($1370) + 368|0);
  $1372 = HEAP32[$1371>>2]|0;
  $1373 = (($1372) + (($1369*3)|0)|0);
  $1374 = (($1373) + 1|0);
  $1375 = HEAP8[$1374>>0]|0;
  $1376 = $1375&255;
  $1377 = $1376 >> $1367;
  $1378 = $1377&255;
  HEAP8[$1374>>0] = $1378;
  $1379 = $sb;
  $1380 = $i17;
  $1381 = $1380&65535;
  $1382 = $0;
  $1383 = (($1382) + 368|0);
  $1384 = HEAP32[$1383>>2]|0;
  $1385 = (($1384) + (($1381*3)|0)|0);
  $1386 = (($1385) + 2|0);
  $1387 = HEAP8[$1386>>0]|0;
  $1388 = $1387&255;
  $1389 = $1388 >> $1379;
  $1390 = $1389&255;
  HEAP8[$1386>>0] = $1390;
  $1391 = $i17;
  $1392 = (($1391) + 1)<<16>>16;
  $i17 = $1392;
 }
 STACKTOP = sp;return;
}
function _png_build_gamma_table($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0.0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0.0;
 var $170 = 0, $171 = 0.0, $172 = 0.0, $173 = 0, $174 = 0, $175 = 0, $176 = 0.0, $177 = 0, $178 = 0, $179 = 0.0, $18 = 0.0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0.0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0.0, $207 = 0.0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0.0, $212 = 0.0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0.0, $218 = 0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0.0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0.0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0.0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0.0, $300 = 0.0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0.0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0.0, $320 = 0, $321 = 0.0, $322 = 0.0, $323 = 0.0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0.0, $353 = 0.0, $354 = 0.0, $355 = 0.0, $356 = 0.0, $357 = 0.0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0.0, $374 = 0.0, $375 = 0, $376 = 0, $377 = 0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0.0, $381 = 0, $382 = 0, $383 = 0.0, $384 = 0.0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0.0, $414 = 0.0, $415 = 0.0, $416 = 0.0, $417 = 0.0, $418 = 0.0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0.0, $63 = 0.0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0.0, $79 = 0.0, $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0, $87 = 0, $88 = 0.0;
 var $89 = 0.0, $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0.0, $94 = 0.0, $95 = 0.0, $96 = 0.0, $97 = 0.0, $98 = 0.0, $99 = 0, $fin = 0.0, $fout = 0.0, $g = 0.0, $g1 = 0.0, $i = 0, $i2 = 0, $ig = 0, $j = 0;
 var $last = 0, $max = 0, $num = 0, $shift = 0, $sig_bit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $png_ptr;
 $1 = $0;
 $2 = (($1) + 387|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)<=(8);
 if ($5) {
  $6 = $0;
  $7 = (($6) + 444|0);
  $8 = +HEAPF32[$7>>2];
  $9 = $8;
  $10 = $9 > 9.99999999999999954748E-7;
  if ($10) {
   $11 = $0;
   $12 = (($11) + 440|0);
   $13 = +HEAPF32[$12>>2];
   $14 = $0;
   $15 = (($14) + 444|0);
   $16 = +HEAPF32[$15>>2];
   $17 = $13 * $16;
   $18 = $17;
   $19 = 1.0 / $18;
   $g = $19;
  } else {
   $g = 1.0;
  }
  $20 = $0;
  $21 = (_png_malloc($20,256)|0);
  $22 = $0;
  $23 = (($22) + 448|0);
  HEAP32[$23>>2] = $21;
  $i = 0;
  while(1) {
   $24 = $i;
   $25 = ($24|0)<(256);
   if (!($25)) {
    break;
   }
   $26 = $i;
   $27 = (+($26|0));
   $28 = $27 / 255.0;
   $29 = $g;
   $30 = (+Math_pow((+$28),(+$29)));
   $31 = $30 * 255.0;
   $32 = $31 + 0.5;
   $33 = (~~(($32))&255);
   $34 = $i;
   $35 = $0;
   $36 = (($35) + 448|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + ($34)|0);
   HEAP8[$38>>0] = $33;
   $39 = $i;
   $40 = (($39) + 1)|0;
   $i = $40;
  }
  $41 = $0;
  $42 = (($41) + 204|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = $43 & 6291584;
  $45 = ($44|0)!=(0);
  if ($45) {
   $46 = $0;
   $47 = (($46) + 440|0);
   $48 = +HEAPF32[$47>>2];
   $49 = $48;
   $50 = 1.0 / $49;
   $g = $50;
   $51 = $0;
   $52 = (_png_malloc($51,256)|0);
   $53 = $0;
   $54 = (($53) + 456|0);
   HEAP32[$54>>2] = $52;
   $i = 0;
   while(1) {
    $55 = $i;
    $56 = ($55|0)<(256);
    if (!($56)) {
     break;
    }
    $57 = $i;
    $58 = (+($57|0));
    $59 = $58 / 255.0;
    $60 = $g;
    $61 = (+Math_pow((+$59),(+$60)));
    $62 = $61 * 255.0;
    $63 = $62 + 0.5;
    $64 = (~~(($63))&255);
    $65 = $i;
    $66 = $0;
    $67 = (($66) + 456|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + ($65)|0);
    HEAP8[$69>>0] = $64;
    $70 = $i;
    $71 = (($70) + 1)|0;
    $i = $71;
   }
   $72 = $0;
   $73 = (_png_malloc($72,256)|0);
   $74 = $0;
   $75 = (($74) + 452|0);
   HEAP32[$75>>2] = $73;
   $76 = $0;
   $77 = (($76) + 444|0);
   $78 = +HEAPF32[$77>>2];
   $79 = $78;
   $80 = $79 > 9.99999999999999954748E-7;
   if ($80) {
    $81 = $0;
    $82 = (($81) + 444|0);
    $83 = +HEAPF32[$82>>2];
    $84 = $83;
    $85 = 1.0 / $84;
    $g = $85;
   } else {
    $86 = $0;
    $87 = (($86) + 440|0);
    $88 = +HEAPF32[$87>>2];
    $89 = $88;
    $g = $89;
   }
   $i = 0;
   while(1) {
    $90 = $i;
    $91 = ($90|0)<(256);
    if (!($91)) {
     break;
    }
    $92 = $i;
    $93 = (+($92|0));
    $94 = $93 / 255.0;
    $95 = $g;
    $96 = (+Math_pow((+$94),(+$95)));
    $97 = $96 * 255.0;
    $98 = $97 + 0.5;
    $99 = (~~(($98))&255);
    $100 = $i;
    $101 = $0;
    $102 = (($101) + 452|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) + ($100)|0);
    HEAP8[$104>>0] = $99;
    $105 = $i;
    $106 = (($105) + 1)|0;
    $i = $106;
   }
  }
  STACKTOP = sp;return;
 }
 $107 = $0;
 $108 = (($107) + 386|0);
 $109 = HEAP8[$108>>0]|0;
 $110 = $109&255;
 $111 = $110 & 2;
 $112 = ($111|0)!=(0);
 if ($112) {
  $113 = $0;
  $114 = (($113) + 472|0);
  $115 = HEAP8[$114>>0]|0;
  $116 = $115&255;
  $sig_bit = $116;
  $117 = $0;
  $118 = (($117) + 472|0);
  $119 = (($118) + 1|0);
  $120 = HEAP8[$119>>0]|0;
  $121 = $120&255;
  $122 = $sig_bit;
  $123 = ($121|0)>($122|0);
  if ($123) {
   $124 = $0;
   $125 = (($124) + 472|0);
   $126 = (($125) + 1|0);
   $127 = HEAP8[$126>>0]|0;
   $128 = $127&255;
   $sig_bit = $128;
  }
  $129 = $0;
  $130 = (($129) + 472|0);
  $131 = (($130) + 2|0);
  $132 = HEAP8[$131>>0]|0;
  $133 = $132&255;
  $134 = $sig_bit;
  $135 = ($133|0)>($134|0);
  if ($135) {
   $136 = $0;
   $137 = (($136) + 472|0);
   $138 = (($137) + 2|0);
   $139 = HEAP8[$138>>0]|0;
   $140 = $139&255;
   $sig_bit = $140;
  }
 } else {
  $141 = $0;
  $142 = (($141) + 472|0);
  $143 = (($142) + 3|0);
  $144 = HEAP8[$143>>0]|0;
  $145 = $144&255;
  $sig_bit = $145;
 }
 $146 = $sig_bit;
 $147 = ($146|0)>(0);
 if ($147) {
  $148 = $sig_bit;
  $149 = (16 - ($148))|0;
  $shift = $149;
 } else {
  $shift = 0;
 }
 $150 = $0;
 $151 = (($150) + 204|0);
 $152 = HEAP32[$151>>2]|0;
 $153 = $152 & 1024;
 $154 = ($153|0)!=(0);
 if ($154) {
  $155 = $shift;
  $156 = ($155|0)<(5);
  if ($156) {
   $shift = 5;
  }
 }
 $157 = $shift;
 $158 = ($157|0)>(8);
 if ($158) {
  $shift = 8;
 }
 $159 = $shift;
 $160 = ($159|0)<(0);
 if ($160) {
  $shift = 0;
 }
 $161 = $shift;
 $162 = $161&255;
 $163 = $162&255;
 $164 = $0;
 $165 = (($164) + 436|0);
 HEAP32[$165>>2] = $163;
 $166 = $shift;
 $167 = (8 - ($166))|0;
 $168 = 1 << $167;
 $num = $168;
 $169 = $0;
 $170 = (($169) + 444|0);
 $171 = +HEAPF32[$170>>2];
 $172 = $171;
 $173 = $172 > 9.99999999999999954748E-7;
 if ($173) {
  $174 = $0;
  $175 = (($174) + 440|0);
  $176 = +HEAPF32[$175>>2];
  $177 = $0;
  $178 = (($177) + 444|0);
  $179 = +HEAPF32[$178>>2];
  $180 = $176 * $179;
  $181 = $180;
  $182 = 1.0 / $181;
  $g1 = $182;
 } else {
  $g1 = 1.0;
 }
 $183 = $0;
 $184 = $num;
 $185 = $184<<2;
 $186 = (_png_malloc($183,$185)|0);
 $187 = $0;
 $188 = (($187) + 460|0);
 HEAP32[$188>>2] = $186;
 $189 = $0;
 $190 = (($189) + 204|0);
 $191 = HEAP32[$190>>2]|0;
 $192 = $191 & 1152;
 $193 = ($192|0)!=(0);
 if ($193) {
  $i2 = 0;
  while(1) {
   $194 = $i2;
   $195 = $num;
   $196 = ($194|0)<($195|0);
   if (!($196)) {
    break;
   }
   $197 = $0;
   $198 = (_png_malloc($197,512)|0);
   $199 = $i2;
   $200 = $0;
   $201 = (($200) + 460|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (($202) + ($199<<2)|0);
   HEAP32[$203>>2] = $198;
   $204 = $i2;
   $205 = (($204) + 1)|0;
   $i2 = $205;
  }
  $206 = $g1;
  $207 = 1.0 / $206;
  $g1 = $207;
  $last = 0;
  $i2 = 0;
  while(1) {
   $208 = $i2;
   $209 = ($208|0)<(256);
   if (!($209)) {
    break;
   }
   $210 = $i2;
   $211 = (+($210|0));
   $212 = $211 + 0.5;
   $213 = $212 / 256.0;
   $fout = $213;
   $214 = $fout;
   $215 = $g1;
   $216 = (+Math_pow((+$214),(+$215)));
   $fin = $216;
   $217 = $fin;
   $218 = $num;
   $219 = $218 << 8;
   $220 = (+($219>>>0));
   $221 = $217 * $220;
   $222 = (~~(($221))>>>0);
   $max = $222;
   while(1) {
    $223 = $last;
    $224 = $max;
    $225 = ($223>>>0)<=($224>>>0);
    if (!($225)) {
     break;
    }
    $226 = $i2;
    $227 = $226&65535;
    $228 = $227&65535;
    $229 = $i2;
    $230 = $229&65535;
    $231 = $230&65535;
    $232 = $231 << 8;
    $233 = $228 | $232;
    $234 = $233&65535;
    $235 = $last;
    $236 = $shift;
    $237 = (8 - ($236))|0;
    $238 = $235 >>> $237;
    $239 = $last;
    $240 = $shift;
    $241 = 255 >> $240;
    $242 = $239 & $241;
    $243 = $0;
    $244 = (($243) + 460|0);
    $245 = HEAP32[$244>>2]|0;
    $246 = (($245) + ($242<<2)|0);
    $247 = HEAP32[$246>>2]|0;
    $248 = (($247) + ($238<<1)|0);
    HEAP16[$248>>1] = $234;
    $249 = $last;
    $250 = (($249) + 1)|0;
    $last = $250;
   }
   $251 = $i2;
   $252 = (($251) + 1)|0;
   $i2 = $252;
  }
  while(1) {
   $253 = $last;
   $254 = $num;
   $255 = $254 << 8;
   $256 = ($253>>>0)<($255>>>0);
   if (!($256)) {
    break;
   }
   $257 = $last;
   $258 = $shift;
   $259 = (8 - ($258))|0;
   $260 = $257 >>> $259;
   $261 = $last;
   $262 = $shift;
   $263 = 255 >> $262;
   $264 = $261 & $263;
   $265 = $0;
   $266 = (($265) + 460|0);
   $267 = HEAP32[$266>>2]|0;
   $268 = (($267) + ($264<<2)|0);
   $269 = HEAP32[$268>>2]|0;
   $270 = (($269) + ($260<<1)|0);
   HEAP16[$270>>1] = -1;
   $271 = $last;
   $272 = (($271) + 1)|0;
   $last = $272;
  }
 } else {
  $i2 = 0;
  while(1) {
   $273 = $i2;
   $274 = $num;
   $275 = ($273|0)<($274|0);
   if (!($275)) {
    break;
   }
   $276 = $0;
   $277 = (_png_malloc($276,512)|0);
   $278 = $i2;
   $279 = $0;
   $280 = (($279) + 460|0);
   $281 = HEAP32[$280>>2]|0;
   $282 = (($281) + ($278<<2)|0);
   HEAP32[$282>>2] = $277;
   $283 = $i2;
   $284 = $shift;
   $285 = (58640 + ($284<<2)|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = Math_imul($283, $286)|0;
   $288 = $287 >>> 4;
   $ig = $288;
   $j = 0;
   while(1) {
    $289 = $j;
    $290 = ($289|0)<(256);
    if (!($290)) {
     break;
    }
    $291 = $ig;
    $292 = $j;
    $293 = $292 << 8;
    $294 = (($291) + ($293))|0;
    $295 = (+($294>>>0));
    $296 = $295 / 65535.0;
    $297 = $g1;
    $298 = (+Math_pow((+$296),(+$297)));
    $299 = $298 * 65535.0;
    $300 = $299 + 0.5;
    $301 = (~~(($300))&65535);
    $302 = $j;
    $303 = $i2;
    $304 = $0;
    $305 = (($304) + 460|0);
    $306 = HEAP32[$305>>2]|0;
    $307 = (($306) + ($303<<2)|0);
    $308 = HEAP32[$307>>2]|0;
    $309 = (($308) + ($302<<1)|0);
    HEAP16[$309>>1] = $301;
    $310 = $j;
    $311 = (($310) + 1)|0;
    $j = $311;
   }
   $312 = $i2;
   $313 = (($312) + 1)|0;
   $i2 = $313;
  }
 }
 $314 = $0;
 $315 = (($314) + 204|0);
 $316 = HEAP32[$315>>2]|0;
 $317 = $316 & 6291584;
 $318 = ($317|0)!=(0);
 if ($318) {
  $319 = $0;
  $320 = (($319) + 440|0);
  $321 = +HEAPF32[$320>>2];
  $322 = $321;
  $323 = 1.0 / $322;
  $g1 = $323;
  $324 = $0;
  $325 = $num;
  $326 = $325<<2;
  $327 = (_png_malloc($324,$326)|0);
  $328 = $0;
  $329 = (($328) + 468|0);
  HEAP32[$329>>2] = $327;
  $i2 = 0;
  while(1) {
   $330 = $i2;
   $331 = $num;
   $332 = ($330|0)<($331|0);
   if (!($332)) {
    break;
   }
   $333 = $0;
   $334 = (_png_malloc($333,512)|0);
   $335 = $i2;
   $336 = $0;
   $337 = (($336) + 468|0);
   $338 = HEAP32[$337>>2]|0;
   $339 = (($338) + ($335<<2)|0);
   HEAP32[$339>>2] = $334;
   $340 = $i2;
   $341 = $shift;
   $342 = (58640 + ($341<<2)|0);
   $343 = HEAP32[$342>>2]|0;
   $344 = Math_imul($340, $343)|0;
   $345 = $344 >>> 4;
   $ig = $345;
   $j = 0;
   while(1) {
    $346 = $j;
    $347 = ($346|0)<(256);
    if (!($347)) {
     break;
    }
    $348 = $ig;
    $349 = $j;
    $350 = $349 << 8;
    $351 = (($348) + ($350))|0;
    $352 = (+($351>>>0));
    $353 = $352 / 65535.0;
    $354 = $g1;
    $355 = (+Math_pow((+$353),(+$354)));
    $356 = $355 * 65535.0;
    $357 = $356 + 0.5;
    $358 = (~~(($357))&65535);
    $359 = $j;
    $360 = $i2;
    $361 = $0;
    $362 = (($361) + 468|0);
    $363 = HEAP32[$362>>2]|0;
    $364 = (($363) + ($360<<2)|0);
    $365 = HEAP32[$364>>2]|0;
    $366 = (($365) + ($359<<1)|0);
    HEAP16[$366>>1] = $358;
    $367 = $j;
    $368 = (($367) + 1)|0;
    $j = $368;
   }
   $369 = $i2;
   $370 = (($369) + 1)|0;
   $i2 = $370;
  }
  $371 = $0;
  $372 = (($371) + 444|0);
  $373 = +HEAPF32[$372>>2];
  $374 = $373;
  $375 = $374 > 9.99999999999999954748E-7;
  if ($375) {
   $376 = $0;
   $377 = (($376) + 444|0);
   $378 = +HEAPF32[$377>>2];
   $379 = $378;
   $380 = 1.0 / $379;
   $g1 = $380;
  } else {
   $381 = $0;
   $382 = (($381) + 440|0);
   $383 = +HEAPF32[$382>>2];
   $384 = $383;
   $g1 = $384;
  }
  $385 = $0;
  $386 = $num;
  $387 = $386<<2;
  $388 = (_png_malloc($385,$387)|0);
  $389 = $0;
  $390 = (($389) + 464|0);
  HEAP32[$390>>2] = $388;
  $i2 = 0;
  while(1) {
   $391 = $i2;
   $392 = $num;
   $393 = ($391|0)<($392|0);
   if (!($393)) {
    break;
   }
   $394 = $0;
   $395 = (_png_malloc($394,512)|0);
   $396 = $i2;
   $397 = $0;
   $398 = (($397) + 464|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = (($399) + ($396<<2)|0);
   HEAP32[$400>>2] = $395;
   $401 = $i2;
   $402 = $shift;
   $403 = (58640 + ($402<<2)|0);
   $404 = HEAP32[$403>>2]|0;
   $405 = Math_imul($401, $404)|0;
   $406 = $405 >>> 4;
   $ig = $406;
   $j = 0;
   while(1) {
    $407 = $j;
    $408 = ($407|0)<(256);
    if (!($408)) {
     break;
    }
    $409 = $ig;
    $410 = $j;
    $411 = $410 << 8;
    $412 = (($409) + ($411))|0;
    $413 = (+($412>>>0));
    $414 = $413 / 65535.0;
    $415 = $g1;
    $416 = (+Math_pow((+$414),(+$415)));
    $417 = $416 * 65535.0;
    $418 = $417 + 0.5;
    $419 = (~~(($418))&65535);
    $420 = $j;
    $421 = $i2;
    $422 = $0;
    $423 = (($422) + 464|0);
    $424 = HEAP32[$423>>2]|0;
    $425 = (($424) + ($421<<2)|0);
    $426 = HEAP32[$425>>2]|0;
    $427 = (($426) + ($420<<1)|0);
    HEAP16[$427>>1] = $419;
    $428 = $j;
    $429 = (($428) + 1)|0;
    $j = $429;
   }
   $430 = $i2;
   $431 = (($430) + 1)|0;
   $i2 = $431;
  }
 }
 STACKTOP = sp;return;
}
function _png_read_transform_info($png_ptr,$info_ptr) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $0;
 $3 = (($2) + 204|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 4096;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $1;
  $8 = (($7) + 25|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = ($10|0)==(3);
  if ($11) {
   $12 = $0;
   $13 = (($12) + 374|0);
   $14 = HEAP16[$13>>1]|0;
   $15 = $14&65535;
   $16 = ($15|0)!=(0);
   if ($16) {
    $17 = $0;
    $18 = (($17) + 204|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = $19 & 33554432;
    $21 = ($20|0)!=(0);
    if ($21) {
     $22 = $1;
     $23 = (($22) + 25|0);
     HEAP8[$23>>0] = 6;
    } else {
     label = 6;
    }
   } else {
    label = 6;
   }
   if ((label|0) == 6) {
    $24 = $1;
    $25 = (($24) + 25|0);
    HEAP8[$25>>0] = 2;
   }
   $26 = $1;
   $27 = (($26) + 24|0);
   HEAP8[$27>>0] = 8;
   $28 = $1;
   $29 = (($28) + 22|0);
   HEAP16[$29>>1] = 0;
  } else {
   $30 = $0;
   $31 = (($30) + 374|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = ($32<<16>>16)!=(0);
   if ($33) {
    $34 = $0;
    $35 = (($34) + 204|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = $36 & 33554432;
    $38 = ($37|0)!=(0);
    if ($38) {
     $39 = $1;
     $40 = (($39) + 25|0);
     $41 = HEAP8[$40>>0]|0;
     $42 = $41&255;
     $43 = $42 | 4;
     $44 = $43&255;
     HEAP8[$40>>0] = $44;
    } else {
     $45 = $1;
     $46 = (($45) + 25|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = $47&255;
     $49 = $48 | 2;
     $50 = $49&255;
     HEAP8[$46>>0] = $50;
    }
   }
   $51 = $1;
   $52 = (($51) + 24|0);
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = ($54|0)<(8);
   if ($55) {
    $56 = $1;
    $57 = (($56) + 24|0);
    HEAP8[$57>>0] = 8;
   }
   $58 = $1;
   $59 = (($58) + 22|0);
   HEAP16[$59>>1] = 0;
  }
 }
 $60 = $0;
 $61 = (($60) + 204|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $62 & 128;
 $64 = ($63|0)!=(0);
 if ($64) {
  $65 = $1;
  $66 = (($65) + 25|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $68 & -5;
  $70 = $69&255;
  HEAP8[$66>>0] = $70;
  $71 = $1;
  $72 = (($71) + 22|0);
  HEAP16[$72>>1] = 0;
  $73 = $1;
  $74 = (($73) + 90|0);
  $75 = $0;
  $76 = (($75) + 404|0);
  ;HEAP16[$74+0>>1]=HEAP16[$76+0>>1]|0;HEAP16[$74+2>>1]=HEAP16[$76+2>>1]|0;HEAP16[$74+4>>1]=HEAP16[$76+4>>1]|0;HEAP16[$74+6>>1]=HEAP16[$76+6>>1]|0;HEAP16[$74+8>>1]=HEAP16[$76+8>>1]|0;
 }
 $77 = $0;
 $78 = (($77) + 204|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = $79 & 8192;
 $81 = ($80|0)!=(0);
 if ($81) {
  $82 = $0;
  $83 = (($82) + 440|0);
  $84 = +HEAPF32[$83>>2];
  $85 = $1;
  $86 = (($85) + 40|0);
  HEAPF32[$86>>2] = $84;
  $87 = $0;
  $88 = (($87) + 656|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $1;
  $91 = (($90) + 252|0);
  HEAP32[$91>>2] = $89;
 }
 $92 = $0;
 $93 = (($92) + 204|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $94 & 1024;
 $96 = ($95|0)!=(0);
 if ($96) {
  $97 = $1;
  $98 = (($97) + 24|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = ($100|0)==(16);
  if ($101) {
   $102 = $1;
   $103 = (($102) + 24|0);
   HEAP8[$103>>0] = 8;
  }
 }
 $104 = $0;
 $105 = (($104) + 204|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = $106 & 16384;
 $108 = ($107|0)!=(0);
 if ($108) {
  $109 = $1;
  $110 = (($109) + 25|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = $111&255;
  $113 = $112 | 2;
  $114 = $113&255;
  HEAP8[$110>>0] = $114;
 }
 $115 = $0;
 $116 = (($115) + 204|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = $117 & 6291456;
 $119 = ($118|0)!=(0);
 if ($119) {
  $120 = $1;
  $121 = (($120) + 25|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $123 & -3;
  $125 = $124&255;
  HEAP8[$121>>0] = $125;
 }
 $126 = $0;
 $127 = (($126) + 204|0);
 $128 = HEAP32[$127>>2]|0;
 $129 = $128 & 64;
 $130 = ($129|0)!=(0);
 if ($130) {
  $131 = $1;
  $132 = (($131) + 25|0);
  $133 = HEAP8[$132>>0]|0;
  $134 = $133&255;
  $135 = ($134|0)==(2);
  if ($135) {
   label = 31;
  } else {
   $136 = $1;
   $137 = (($136) + 25|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = $138&255;
   $140 = ($139|0)==(6);
   if ($140) {
    label = 31;
   }
  }
  if ((label|0) == 31) {
   $141 = $0;
   $142 = (($141) + 584|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = ($143|0)!=(0|0);
   if ($144) {
    $145 = $1;
    $146 = (($145) + 24|0);
    $147 = HEAP8[$146>>0]|0;
    $148 = $147&255;
    $149 = ($148|0)==(8);
    if ($149) {
     $150 = $1;
     $151 = (($150) + 25|0);
     HEAP8[$151>>0] = 3;
    }
   }
  }
 }
 $152 = $0;
 $153 = (($152) + 204|0);
 $154 = HEAP32[$153>>2]|0;
 $155 = $154 & 4;
 $156 = ($155|0)!=(0);
 if ($156) {
  $157 = $1;
  $158 = (($157) + 24|0);
  $159 = HEAP8[$158>>0]|0;
  $160 = $159&255;
  $161 = ($160|0)<(8);
  if ($161) {
   $162 = $1;
   $163 = (($162) + 24|0);
   HEAP8[$163>>0] = 8;
  }
 }
 $164 = $1;
 $165 = (($164) + 25|0);
 $166 = HEAP8[$165>>0]|0;
 $167 = $166&255;
 $168 = ($167|0)==(3);
 if ($168) {
  $169 = $1;
  $170 = (($169) + 29|0);
  HEAP8[$170>>0] = 1;
 } else {
  $171 = $1;
  $172 = (($171) + 25|0);
  $173 = HEAP8[$172>>0]|0;
  $174 = $173&255;
  $175 = $174 & 2;
  $176 = ($175|0)!=(0);
  if ($176) {
   $177 = $1;
   $178 = (($177) + 29|0);
   HEAP8[$178>>0] = 3;
  } else {
   $179 = $1;
   $180 = (($179) + 29|0);
   HEAP8[$180>>0] = 1;
  }
 }
 $181 = $0;
 $182 = (($181) + 200|0);
 $183 = HEAP32[$182>>2]|0;
 $184 = $183 & 4194304;
 $185 = ($184|0)!=(0);
 if ($185) {
  $186 = $1;
  $187 = (($186) + 25|0);
  $188 = HEAP8[$187>>0]|0;
  $189 = $188&255;
  $190 = $189 & -5;
  $191 = $190&255;
  HEAP8[$187>>0] = $191;
 }
 $192 = $1;
 $193 = (($192) + 25|0);
 $194 = HEAP8[$193>>0]|0;
 $195 = $194&255;
 $196 = $195 & 4;
 $197 = ($196|0)!=(0);
 if ($197) {
  $198 = $1;
  $199 = (($198) + 29|0);
  $200 = HEAP8[$199>>0]|0;
  $201 = (($200) + 1)<<24>>24;
  HEAP8[$199>>0] = $201;
 }
 $202 = $0;
 $203 = (($202) + 204|0);
 $204 = HEAP32[$203>>2]|0;
 $205 = $204 & 32768;
 $206 = ($205|0)!=(0);
 do {
  if ($206) {
   $207 = $1;
   $208 = (($207) + 25|0);
   $209 = HEAP8[$208>>0]|0;
   $210 = $209&255;
   $211 = ($210|0)==(2);
   if (!($211)) {
    $212 = $1;
    $213 = (($212) + 25|0);
    $214 = HEAP8[$213>>0]|0;
    $215 = $214&255;
    $216 = ($215|0)==(0);
    if (!($216)) {
     break;
    }
   }
   $217 = $1;
   $218 = (($217) + 29|0);
   $219 = HEAP8[$218>>0]|0;
   $220 = (($219) + 1)<<24>>24;
   HEAP8[$218>>0] = $220;
   $221 = $0;
   $222 = (($221) + 204|0);
   $223 = HEAP32[$222>>2]|0;
   $224 = $223 & 16777216;
   $225 = ($224|0)!=(0);
   if ($225) {
    $226 = $1;
    $227 = (($226) + 25|0);
    $228 = HEAP8[$227>>0]|0;
    $229 = $228&255;
    $230 = $229 | 4;
    $231 = $230&255;
    HEAP8[$227>>0] = $231;
   }
  }
 } while(0);
 $232 = $0;
 $233 = (($232) + 204|0);
 $234 = HEAP32[$233>>2]|0;
 $235 = $234 & 1048576;
 $236 = ($235|0)!=(0);
 if ($236) {
  $237 = $1;
  $238 = (($237) + 24|0);
  $239 = HEAP8[$238>>0]|0;
  $240 = $239&255;
  $241 = $0;
  $242 = (($241) + 192|0);
  $243 = HEAP8[$242>>0]|0;
  $244 = $243&255;
  $245 = ($240|0)<($244|0);
  if ($245) {
   $246 = $0;
   $247 = (($246) + 192|0);
   $248 = HEAP8[$247>>0]|0;
   $249 = $1;
   $250 = (($249) + 24|0);
   HEAP8[$250>>0] = $248;
  }
  $251 = $1;
  $252 = (($251) + 29|0);
  $253 = HEAP8[$252>>0]|0;
  $254 = $253&255;
  $255 = $0;
  $256 = (($255) + 193|0);
  $257 = HEAP8[$256>>0]|0;
  $258 = $257&255;
  $259 = ($254|0)<($258|0);
  if ($259) {
   $260 = $0;
   $261 = (($260) + 193|0);
   $262 = HEAP8[$261>>0]|0;
   $263 = $1;
   $264 = (($263) + 29|0);
   HEAP8[$264>>0] = $262;
  }
 }
 $265 = $1;
 $266 = (($265) + 29|0);
 $267 = HEAP8[$266>>0]|0;
 $268 = $267&255;
 $269 = $1;
 $270 = (($269) + 24|0);
 $271 = HEAP8[$270>>0]|0;
 $272 = $271&255;
 $273 = Math_imul($268, $272)|0;
 $274 = $273&255;
 $275 = $1;
 $276 = (($275) + 30|0);
 HEAP8[$276>>0] = $274;
 $277 = $1;
 $278 = (($277) + 30|0);
 $279 = HEAP8[$278>>0]|0;
 $280 = $279&255;
 $281 = ($280|0)>=(8);
 if ($281) {
  $282 = $1;
  $283 = HEAP32[$282>>2]|0;
  $284 = $1;
  $285 = (($284) + 30|0);
  $286 = HEAP8[$285>>0]|0;
  $287 = $286&255;
  $288 = $287 >>> 3;
  $289 = Math_imul($283, $288)|0;
  $301 = $289;
  $299 = $1;
  $300 = (($299) + 12|0);
  HEAP32[$300>>2] = $301;
  STACKTOP = sp;return;
 } else {
  $290 = $1;
  $291 = HEAP32[$290>>2]|0;
  $292 = $1;
  $293 = (($292) + 30|0);
  $294 = HEAP8[$293>>0]|0;
  $295 = $294&255;
  $296 = Math_imul($291, $295)|0;
  $297 = (($296) + 7)|0;
  $298 = $297 >>> 3;
  $301 = $298;
  $299 = $1;
  $300 = (($299) + 12|0);
  HEAP32[$300>>2] = $301;
  STACKTOP = sp;return;
 }
}
function _png_do_read_transformations($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $msg = 0, $rgb_error = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $msg = sp + 16|0;
 $0 = $png_ptr;
 $1 = $0;
 $2 = (($1) + 328|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = $0;
  $6 = (($5) + 320|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $0;
  $9 = (($8) + 384|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  HEAP32[$vararg_buffer>>2] = $7;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $11;
  (_snprintf($msg,50,58496,$vararg_buffer)|0);
  $12 = $0;
  _png_error($12,$msg);
 }
 $13 = $0;
 $14 = (($13) + 200|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 & 64;
 $17 = ($16|0)!=(0);
 if (!($17)) {
  $18 = $0;
  _png_error($18,58536);
 }
 $19 = $0;
 $20 = (($19) + 204|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $21 & 4096;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = $0;
  $25 = (($24) + 348|0);
  $26 = (($25) + 8|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  $29 = ($28|0)==(3);
  if ($29) {
   $30 = $0;
   $31 = (($30) + 348|0);
   $32 = $0;
   $33 = (($32) + 328|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 1|0);
   $36 = $0;
   $37 = (($36) + 368|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = $0;
   $40 = (($39) + 484|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = $0;
   $43 = (($42) + 374|0);
   $44 = HEAP16[$43>>1]|0;
   $45 = $44&65535;
   _png_do_expand_palette($31,$35,$38,$41,$45);
  } else {
   $46 = $0;
   $47 = (($46) + 374|0);
   $48 = HEAP16[$47>>1]|0;
   $49 = $48&65535;
   $50 = ($49|0)!=(0);
   if ($50) {
    $51 = $0;
    $52 = (($51) + 204|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = $53 & 33554432;
    $55 = ($54|0)!=(0);
    if ($55) {
     $56 = $0;
     $57 = (($56) + 348|0);
     $58 = $0;
     $59 = (($58) + 328|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = (($60) + 1|0);
     $62 = $0;
     $63 = (($62) + 488|0);
     _png_do_expand($57,$61,$63);
    } else {
     label = 11;
    }
   } else {
    label = 11;
   }
   if ((label|0) == 11) {
    $64 = $0;
    $65 = (($64) + 348|0);
    $66 = $0;
    $67 = (($66) + 328|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + 1|0);
    _png_do_expand($65,$69,0);
   }
  }
 }
 $70 = $0;
 $71 = (($70) + 200|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = $72 & 4194304;
 $74 = ($73|0)!=(0);
 if ($74) {
  $75 = $0;
  $76 = (($75) + 348|0);
  $77 = $0;
  $78 = (($77) + 328|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + 1|0);
  $81 = $0;
  $82 = (($81) + 200|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $83 & 4194304;
  $85 = 128 | $84;
  _png_do_strip_filler($76,$80,$85);
 }
 $86 = $0;
 $87 = (($86) + 204|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $88 & 6291456;
 $90 = ($89|0)!=(0);
 if ($90) {
  $91 = $0;
  $92 = $0;
  $93 = (($92) + 348|0);
  $94 = $0;
  $95 = (($94) + 328|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 1|0);
  $98 = (_png_do_rgb_to_gray($91,$93,$97)|0);
  $rgb_error = $98;
  $99 = $rgb_error;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = $0;
   $102 = (($101) + 644|0);
   HEAP8[$102>>0] = 1;
   $103 = $0;
   $104 = (($103) + 204|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = $105 & 6291456;
   $107 = ($106|0)==(4194304);
   if ($107) {
    $108 = $0;
    _png_warning($108,58560);
   }
   $109 = $0;
   $110 = (($109) + 204|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $111 & 6291456;
   $113 = ($112|0)==(2097152);
   if ($113) {
    $114 = $0;
    _png_error($114,58560);
   }
  }
 }
 $115 = $0;
 $116 = (($115) + 204|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = $117 & 16384;
 $119 = ($118|0)!=(0);
 if ($119) {
  $120 = $0;
  $121 = (($120) + 196|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = $122 & 2048;
  $124 = ($123|0)!=(0);
  if (!($124)) {
   $125 = $0;
   $126 = (($125) + 348|0);
   $127 = $0;
   $128 = (($127) + 328|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = (($129) + 1|0);
   _png_do_gray_to_rgb($126,$130);
  }
 }
 $131 = $0;
 $132 = (($131) + 204|0);
 $133 = HEAP32[$132>>2]|0;
 $134 = $133 & 128;
 $135 = ($134|0)!=(0);
 do {
  if ($135) {
   $136 = $0;
   $137 = (($136) + 374|0);
   $138 = HEAP16[$137>>1]|0;
   $139 = $138&65535;
   $140 = ($139|0)!=(0);
   if (!($140)) {
    $141 = $0;
    $142 = (($141) + 386|0);
    $143 = HEAP8[$142>>0]|0;
    $144 = $143&255;
    $145 = $144 & 4;
    $146 = ($145|0)!=(0);
    if (!($146)) {
     break;
    }
   }
   $147 = $0;
   $148 = (($147) + 348|0);
   $149 = $0;
   $150 = (($149) + 328|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = (($151) + 1|0);
   $153 = $0;
   $154 = (($153) + 488|0);
   $155 = $0;
   $156 = (($155) + 404|0);
   $157 = $0;
   $158 = (($157) + 414|0);
   $159 = $0;
   $160 = (($159) + 448|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = $0;
   $163 = (($162) + 452|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = $0;
   $166 = (($165) + 456|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = $0;
   $169 = (($168) + 460|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = $0;
   $172 = (($171) + 464|0);
   $173 = HEAP32[$172>>2]|0;
   $174 = $0;
   $175 = (($174) + 468|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = $0;
   $178 = (($177) + 436|0);
   $179 = HEAP32[$178>>2]|0;
   _png_do_background($148,$152,$154,$156,$158,$161,$164,$167,$170,$173,$176,$179);
  }
 } while(0);
 $180 = $0;
 $181 = (($180) + 204|0);
 $182 = HEAP32[$181>>2]|0;
 $183 = $182 & 8192;
 $184 = ($183|0)!=(0);
 do {
  if ($184) {
   $185 = $0;
   $186 = (($185) + 204|0);
   $187 = HEAP32[$186>>2]|0;
   $188 = $187 & 128;
   $189 = ($188|0)!=(0);
   if ($189) {
    $190 = $0;
    $191 = (($190) + 374|0);
    $192 = HEAP16[$191>>1]|0;
    $193 = $192&65535;
    $194 = ($193|0)!=(0);
    if ($194) {
     break;
    }
    $195 = $0;
    $196 = (($195) + 386|0);
    $197 = HEAP8[$196>>0]|0;
    $198 = $197&255;
    $199 = $198 & 4;
    $200 = ($199|0)!=(0);
    if ($200) {
     break;
    }
   }
   $201 = $0;
   $202 = (($201) + 386|0);
   $203 = HEAP8[$202>>0]|0;
   $204 = $203&255;
   $205 = ($204|0)!=(3);
   if ($205) {
    $206 = $0;
    $207 = (($206) + 348|0);
    $208 = $0;
    $209 = (($208) + 328|0);
    $210 = HEAP32[$209>>2]|0;
    $211 = (($210) + 1|0);
    $212 = $0;
    $213 = (($212) + 448|0);
    $214 = HEAP32[$213>>2]|0;
    $215 = $0;
    $216 = (($215) + 460|0);
    $217 = HEAP32[$216>>2]|0;
    $218 = $0;
    $219 = (($218) + 436|0);
    $220 = HEAP32[$219>>2]|0;
    _png_do_gamma($207,$211,$214,$217,$220);
   }
  }
 } while(0);
 $221 = $0;
 $222 = (($221) + 204|0);
 $223 = HEAP32[$222>>2]|0;
 $224 = $223 & 1024;
 $225 = ($224|0)!=(0);
 if ($225) {
  $226 = $0;
  $227 = (($226) + 348|0);
  $228 = $0;
  $229 = (($228) + 328|0);
  $230 = HEAP32[$229>>2]|0;
  $231 = (($230) + 1|0);
  _png_do_chop($227,$231);
 }
 $232 = $0;
 $233 = (($232) + 204|0);
 $234 = HEAP32[$233>>2]|0;
 $235 = $234 & 64;
 $236 = ($235|0)!=(0);
 if ($236) {
  $237 = $0;
  $238 = (($237) + 348|0);
  $239 = $0;
  $240 = (($239) + 328|0);
  $241 = HEAP32[$240>>2]|0;
  $242 = (($241) + 1|0);
  $243 = $0;
  $244 = (($243) + 584|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = $0;
  $247 = (($246) + 588|0);
  $248 = HEAP32[$247>>2]|0;
  _png_do_dither($238,$242,$245,$248);
  $249 = $0;
  $250 = (($249) + 348|0);
  $251 = (($250) + 4|0);
  $252 = HEAP32[$251>>2]|0;
  $253 = ($252|0)==(0);
  if ($253) {
   $254 = $0;
   _png_error($254,58600);
  }
 }
 $255 = $0;
 $256 = (($255) + 204|0);
 $257 = HEAP32[$256>>2]|0;
 $258 = $257 & 32;
 $259 = ($258|0)!=(0);
 if ($259) {
  $260 = $0;
  $261 = (($260) + 348|0);
  $262 = $0;
  $263 = (($262) + 328|0);
  $264 = HEAP32[$263>>2]|0;
  $265 = (($264) + 1|0);
  _png_do_invert($261,$265);
 }
 $266 = $0;
 $267 = (($266) + 204|0);
 $268 = HEAP32[$267>>2]|0;
 $269 = $268 & 8;
 $270 = ($269|0)!=(0);
 if ($270) {
  $271 = $0;
  $272 = (($271) + 348|0);
  $273 = $0;
  $274 = (($273) + 328|0);
  $275 = HEAP32[$274>>2]|0;
  $276 = (($275) + 1|0);
  $277 = $0;
  $278 = (($277) + 477|0);
  _png_do_unshift($272,$276,$278);
 }
 $279 = $0;
 $280 = (($279) + 204|0);
 $281 = HEAP32[$280>>2]|0;
 $282 = $281 & 4;
 $283 = ($282|0)!=(0);
 if ($283) {
  $284 = $0;
  $285 = (($284) + 348|0);
  $286 = $0;
  $287 = (($286) + 328|0);
  $288 = HEAP32[$287>>2]|0;
  $289 = (($288) + 1|0);
  _png_do_unpack($285,$289);
 }
 $290 = $0;
 $291 = (($290) + 204|0);
 $292 = HEAP32[$291>>2]|0;
 $293 = $292 & 1;
 $294 = ($293|0)!=(0);
 if ($294) {
  $295 = $0;
  $296 = (($295) + 348|0);
  $297 = $0;
  $298 = (($297) + 328|0);
  $299 = HEAP32[$298>>2]|0;
  $300 = (($299) + 1|0);
  _png_do_bgr($296,$300);
 }
 $301 = $0;
 $302 = (($301) + 204|0);
 $303 = HEAP32[$302>>2]|0;
 $304 = $303 & 65536;
 $305 = ($304|0)!=(0);
 if ($305) {
  $306 = $0;
  $307 = (($306) + 348|0);
  $308 = $0;
  $309 = (($308) + 328|0);
  $310 = HEAP32[$309>>2]|0;
  $311 = (($310) + 1|0);
  _png_do_packswap($307,$311);
 }
 $312 = $0;
 $313 = (($312) + 204|0);
 $314 = HEAP32[$313>>2]|0;
 $315 = $314 & 16384;
 $316 = ($315|0)!=(0);
 if ($316) {
  $317 = $0;
  $318 = (($317) + 196|0);
  $319 = HEAP32[$318>>2]|0;
  $320 = $319 & 2048;
  $321 = ($320|0)!=(0);
  if ($321) {
   $322 = $0;
   $323 = (($322) + 348|0);
   $324 = $0;
   $325 = (($324) + 328|0);
   $326 = HEAP32[$325>>2]|0;
   $327 = (($326) + 1|0);
   _png_do_gray_to_rgb($323,$327);
  }
 }
 $328 = $0;
 $329 = (($328) + 204|0);
 $330 = HEAP32[$329>>2]|0;
 $331 = $330 & 32768;
 $332 = ($331|0)!=(0);
 if ($332) {
  $333 = $0;
  $334 = (($333) + 348|0);
  $335 = $0;
  $336 = (($335) + 328|0);
  $337 = HEAP32[$336>>2]|0;
  $338 = (($337) + 1|0);
  $339 = $0;
  $340 = (($339) + 394|0);
  $341 = HEAP16[$340>>1]|0;
  $342 = $341&65535;
  $343 = $0;
  $344 = (($343) + 200|0);
  $345 = HEAP32[$344>>2]|0;
  _png_do_read_filler($334,$338,$342,$345);
 }
 $346 = $0;
 $347 = (($346) + 204|0);
 $348 = HEAP32[$347>>2]|0;
 $349 = $348 & 524288;
 $350 = ($349|0)!=(0);
 if ($350) {
  $351 = $0;
  $352 = (($351) + 348|0);
  $353 = $0;
  $354 = (($353) + 328|0);
  $355 = HEAP32[$354>>2]|0;
  $356 = (($355) + 1|0);
  _png_do_read_invert_alpha($352,$356);
 }
 $357 = $0;
 $358 = (($357) + 204|0);
 $359 = HEAP32[$358>>2]|0;
 $360 = $359 & 131072;
 $361 = ($360|0)!=(0);
 if ($361) {
  $362 = $0;
  $363 = (($362) + 348|0);
  $364 = $0;
  $365 = (($364) + 328|0);
  $366 = HEAP32[$365>>2]|0;
  $367 = (($366) + 1|0);
  _png_do_read_swap_alpha($363,$367);
 }
 $368 = $0;
 $369 = (($368) + 204|0);
 $370 = HEAP32[$369>>2]|0;
 $371 = $370 & 16;
 $372 = ($371|0)!=(0);
 if ($372) {
  $373 = $0;
  $374 = (($373) + 348|0);
  $375 = $0;
  $376 = (($375) + 328|0);
  $377 = HEAP32[$376>>2]|0;
  $378 = (($377) + 1|0);
  _png_do_swap($374,$378);
 }
 $379 = $0;
 $380 = (($379) + 204|0);
 $381 = HEAP32[$380>>2]|0;
 $382 = $381 & 1048576;
 $383 = ($382|0)!=(0);
 if (!($383)) {
  STACKTOP = sp;return;
 }
 $384 = $0;
 $385 = (($384) + 180|0);
 $386 = HEAP32[$385>>2]|0;
 $387 = ($386|0)!=(0|0);
 if ($387) {
  $388 = $0;
  $389 = (($388) + 180|0);
  $390 = HEAP32[$389>>2]|0;
  $391 = $0;
  $392 = $0;
  $393 = (($392) + 348|0);
  $394 = $0;
  $395 = (($394) + 328|0);
  $396 = HEAP32[$395>>2]|0;
  $397 = (($396) + 1|0);
  FUNCTION_TABLE_viii[$390 & 255]($391,$393,$397);
 }
 $398 = $0;
 $399 = (($398) + 192|0);
 $400 = HEAP8[$399>>0]|0;
 $401 = ($400<<24>>24)!=(0);
 if ($401) {
  $402 = $0;
  $403 = (($402) + 192|0);
  $404 = HEAP8[$403>>0]|0;
  $405 = $0;
  $406 = (($405) + 348|0);
  $407 = (($406) + 9|0);
  HEAP8[$407>>0] = $404;
 }
 $408 = $0;
 $409 = (($408) + 193|0);
 $410 = HEAP8[$409>>0]|0;
 $411 = ($410<<24>>24)!=(0);
 if ($411) {
  $412 = $0;
  $413 = (($412) + 193|0);
  $414 = HEAP8[$413>>0]|0;
  $415 = $0;
  $416 = (($415) + 348|0);
  $417 = (($416) + 10|0);
  HEAP8[$417>>0] = $414;
 }
 $418 = $0;
 $419 = (($418) + 348|0);
 $420 = (($419) + 9|0);
 $421 = HEAP8[$420>>0]|0;
 $422 = $421&255;
 $423 = $0;
 $424 = (($423) + 348|0);
 $425 = (($424) + 10|0);
 $426 = HEAP8[$425>>0]|0;
 $427 = $426&255;
 $428 = Math_imul($422, $427)|0;
 $429 = $428&255;
 $430 = $0;
 $431 = (($430) + 348|0);
 $432 = (($431) + 11|0);
 HEAP8[$432>>0] = $429;
 $433 = $0;
 $434 = (($433) + 348|0);
 $435 = (($434) + 11|0);
 $436 = HEAP8[$435>>0]|0;
 $437 = $436&255;
 $438 = ($437|0)>=(8);
 if ($438) {
  $439 = $0;
  $440 = (($439) + 348|0);
  $441 = HEAP32[$440>>2]|0;
  $442 = $0;
  $443 = (($442) + 348|0);
  $444 = (($443) + 11|0);
  $445 = HEAP8[$444>>0]|0;
  $446 = $445&255;
  $447 = $446 >>> 3;
  $448 = Math_imul($441, $447)|0;
  $463 = $448;
 } else {
  $449 = $0;
  $450 = (($449) + 348|0);
  $451 = HEAP32[$450>>2]|0;
  $452 = $0;
  $453 = (($452) + 348|0);
  $454 = (($453) + 11|0);
  $455 = HEAP8[$454>>0]|0;
  $456 = $455&255;
  $457 = Math_imul($451, $456)|0;
  $458 = (($457) + 7)|0;
  $459 = $458 >>> 3;
  $463 = $459;
 }
 $460 = $0;
 $461 = (($460) + 348|0);
 $462 = (($461) + 4|0);
 HEAP32[$462>>2] = $463;
 STACKTOP = sp;return;
}
function _png_do_expand_palette($row_info,$row,$palette,$trans,$num_trans) {
 $row_info = $row_info|0;
 $row = $row|0;
 $palette = $palette|0;
 $trans = $trans|0;
 $num_trans = $num_trans|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $dp = 0, $i = 0, $row_width = 0, $shift = 0, $sp = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $palette;
 $3 = $trans;
 $4 = $num_trans;
 $5 = $0;
 $6 = HEAP32[$5>>2]|0;
 $row_width = $6;
 $7 = $0;
 $8 = (($7) + 8|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)==(3);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = $0;
 $13 = (($12) + 9|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)<(8);
 if ($16) {
  $17 = $0;
  $18 = (($17) + 9|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  if ((($20|0) == 1)) {
   $21 = $1;
   $22 = $row_width;
   $23 = (($22) - 1)|0;
   $24 = $23 >>> 3;
   $25 = (($21) + ($24)|0);
   $sp = $25;
   $26 = $1;
   $27 = $row_width;
   $28 = (($26) + ($27)|0);
   $29 = (($28) + -1|0);
   $dp = $29;
   $30 = $row_width;
   $31 = (($30) + 7)|0;
   $32 = $31 & 7;
   $33 = (7 - ($32))|0;
   $shift = $33;
   $i = 0;
   while(1) {
    $34 = $i;
    $35 = $row_width;
    $36 = ($34>>>0)<($35>>>0);
    if (!($36)) {
     break;
    }
    $37 = $sp;
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = $shift;
    $41 = $39 >> $40;
    $42 = $41 & 1;
    $43 = ($42|0)!=(0);
    if ($43) {
     $44 = $dp;
     HEAP8[$44>>0] = 1;
    } else {
     $45 = $dp;
     HEAP8[$45>>0] = 0;
    }
    $46 = $shift;
    $47 = ($46|0)==(7);
    if ($47) {
     $shift = 0;
     $48 = $sp;
     $49 = (($48) + -1|0);
     $sp = $49;
    } else {
     $50 = $shift;
     $51 = (($50) + 1)|0;
     $shift = $51;
    }
    $52 = $dp;
    $53 = (($52) + -1|0);
    $dp = $53;
    $54 = $i;
    $55 = (($54) + 1)|0;
    $i = $55;
   }
  } else if ((($20|0) == 4)) {
   $92 = $1;
   $93 = $row_width;
   $94 = (($93) - 1)|0;
   $95 = $94 >>> 1;
   $96 = (($92) + ($95)|0);
   $sp = $96;
   $97 = $1;
   $98 = $row_width;
   $99 = (($97) + ($98)|0);
   $100 = (($99) + -1|0);
   $dp = $100;
   $101 = $row_width;
   $102 = $101 & 1;
   $103 = $102 << 2;
   $shift = $103;
   $i = 0;
   while(1) {
    $104 = $i;
    $105 = $row_width;
    $106 = ($104>>>0)<($105>>>0);
    if (!($106)) {
     break;
    }
    $107 = $sp;
    $108 = HEAP8[$107>>0]|0;
    $109 = $108&255;
    $110 = $shift;
    $111 = $109 >> $110;
    $112 = $111 & 15;
    $value = $112;
    $113 = $value;
    $114 = $113&255;
    $115 = $dp;
    HEAP8[$115>>0] = $114;
    $116 = $shift;
    $117 = ($116|0)==(4);
    if ($117) {
     $shift = 0;
     $118 = $sp;
     $119 = (($118) + -1|0);
     $sp = $119;
    } else {
     $120 = $shift;
     $121 = (($120) + 4)|0;
     $shift = $121;
    }
    $122 = $dp;
    $123 = (($122) + -1|0);
    $dp = $123;
    $124 = $i;
    $125 = (($124) + 1)|0;
    $i = $125;
   }
  } else if ((($20|0) == 2)) {
   $56 = $1;
   $57 = $row_width;
   $58 = (($57) - 1)|0;
   $59 = $58 >>> 2;
   $60 = (($56) + ($59)|0);
   $sp = $60;
   $61 = $1;
   $62 = $row_width;
   $63 = (($61) + ($62)|0);
   $64 = (($63) + -1|0);
   $dp = $64;
   $65 = $row_width;
   $66 = (($65) + 3)|0;
   $67 = $66 & 3;
   $68 = (3 - ($67))|0;
   $69 = $68 << 1;
   $shift = $69;
   $i = 0;
   while(1) {
    $70 = $i;
    $71 = $row_width;
    $72 = ($70>>>0)<($71>>>0);
    if (!($72)) {
     break;
    }
    $73 = $sp;
    $74 = HEAP8[$73>>0]|0;
    $75 = $74&255;
    $76 = $shift;
    $77 = $75 >> $76;
    $78 = $77 & 3;
    $value = $78;
    $79 = $value;
    $80 = $79&255;
    $81 = $dp;
    HEAP8[$81>>0] = $80;
    $82 = $shift;
    $83 = ($82|0)==(6);
    if ($83) {
     $shift = 0;
     $84 = $sp;
     $85 = (($84) + -1|0);
     $sp = $85;
    } else {
     $86 = $shift;
     $87 = (($86) + 2)|0;
     $shift = $87;
    }
    $88 = $dp;
    $89 = (($88) + -1|0);
    $dp = $89;
    $90 = $i;
    $91 = (($90) + 1)|0;
    $i = $91;
   }
  }
  $126 = $0;
  $127 = (($126) + 9|0);
  HEAP8[$127>>0] = 8;
  $128 = $0;
  $129 = (($128) + 11|0);
  HEAP8[$129>>0] = 8;
  $130 = $row_width;
  $131 = $0;
  $132 = (($131) + 4|0);
  HEAP32[$132>>2] = $130;
 }
 $133 = $0;
 $134 = (($133) + 9|0);
 $135 = HEAP8[$134>>0]|0;
 $136 = $135&255;
 if ((($136|0) == 8)) {
  $137 = $3;
  $138 = ($137|0)!=(0|0);
  if ($138) {
   $139 = $1;
   $140 = $row_width;
   $141 = (($139) + ($140)|0);
   $142 = (($141) + -1|0);
   $sp = $142;
   $143 = $1;
   $144 = $row_width;
   $145 = $144 << 2;
   $146 = (($143) + ($145)|0);
   $147 = (($146) + -1|0);
   $dp = $147;
   $i = 0;
   while(1) {
    $148 = $i;
    $149 = $row_width;
    $150 = ($148>>>0)<($149>>>0);
    if (!($150)) {
     break;
    }
    $151 = $sp;
    $152 = HEAP8[$151>>0]|0;
    $153 = $152&255;
    $154 = $4;
    $155 = ($153|0)>=($154|0);
    if ($155) {
     $156 = $dp;
     $157 = (($156) + -1|0);
     $dp = $157;
     HEAP8[$156>>0] = -1;
    } else {
     $158 = $sp;
     $159 = HEAP8[$158>>0]|0;
     $160 = $159&255;
     $161 = $3;
     $162 = (($161) + ($160)|0);
     $163 = HEAP8[$162>>0]|0;
     $164 = $dp;
     $165 = (($164) + -1|0);
     $dp = $165;
     HEAP8[$164>>0] = $163;
    }
    $166 = $sp;
    $167 = HEAP8[$166>>0]|0;
    $168 = $167&255;
    $169 = $2;
    $170 = (($169) + (($168*3)|0)|0);
    $171 = (($170) + 2|0);
    $172 = HEAP8[$171>>0]|0;
    $173 = $dp;
    $174 = (($173) + -1|0);
    $dp = $174;
    HEAP8[$173>>0] = $172;
    $175 = $sp;
    $176 = HEAP8[$175>>0]|0;
    $177 = $176&255;
    $178 = $2;
    $179 = (($178) + (($177*3)|0)|0);
    $180 = (($179) + 1|0);
    $181 = HEAP8[$180>>0]|0;
    $182 = $dp;
    $183 = (($182) + -1|0);
    $dp = $183;
    HEAP8[$182>>0] = $181;
    $184 = $sp;
    $185 = HEAP8[$184>>0]|0;
    $186 = $185&255;
    $187 = $2;
    $188 = (($187) + (($186*3)|0)|0);
    $189 = HEAP8[$188>>0]|0;
    $190 = $dp;
    $191 = (($190) + -1|0);
    $dp = $191;
    HEAP8[$190>>0] = $189;
    $192 = $sp;
    $193 = (($192) + -1|0);
    $sp = $193;
    $194 = $i;
    $195 = (($194) + 1)|0;
    $i = $195;
   }
   $196 = $0;
   $197 = (($196) + 9|0);
   HEAP8[$197>>0] = 8;
   $198 = $0;
   $199 = (($198) + 11|0);
   HEAP8[$199>>0] = 32;
   $200 = $row_width;
   $201 = $200<<2;
   $202 = $0;
   $203 = (($202) + 4|0);
   HEAP32[$203>>2] = $201;
   $204 = $0;
   $205 = (($204) + 8|0);
   HEAP8[$205>>0] = 6;
   $206 = $0;
   $207 = (($206) + 10|0);
   HEAP8[$207>>0] = 4;
  } else {
   $208 = $1;
   $209 = $row_width;
   $210 = (($208) + ($209)|0);
   $211 = (($210) + -1|0);
   $sp = $211;
   $212 = $1;
   $213 = $row_width;
   $214 = ($213*3)|0;
   $215 = (($212) + ($214)|0);
   $216 = (($215) + -1|0);
   $dp = $216;
   $i = 0;
   while(1) {
    $217 = $i;
    $218 = $row_width;
    $219 = ($217>>>0)<($218>>>0);
    if (!($219)) {
     break;
    }
    $220 = $sp;
    $221 = HEAP8[$220>>0]|0;
    $222 = $221&255;
    $223 = $2;
    $224 = (($223) + (($222*3)|0)|0);
    $225 = (($224) + 2|0);
    $226 = HEAP8[$225>>0]|0;
    $227 = $dp;
    $228 = (($227) + -1|0);
    $dp = $228;
    HEAP8[$227>>0] = $226;
    $229 = $sp;
    $230 = HEAP8[$229>>0]|0;
    $231 = $230&255;
    $232 = $2;
    $233 = (($232) + (($231*3)|0)|0);
    $234 = (($233) + 1|0);
    $235 = HEAP8[$234>>0]|0;
    $236 = $dp;
    $237 = (($236) + -1|0);
    $dp = $237;
    HEAP8[$236>>0] = $235;
    $238 = $sp;
    $239 = HEAP8[$238>>0]|0;
    $240 = $239&255;
    $241 = $2;
    $242 = (($241) + (($240*3)|0)|0);
    $243 = HEAP8[$242>>0]|0;
    $244 = $dp;
    $245 = (($244) + -1|0);
    $dp = $245;
    HEAP8[$244>>0] = $243;
    $246 = $sp;
    $247 = (($246) + -1|0);
    $sp = $247;
    $248 = $i;
    $249 = (($248) + 1)|0;
    $i = $249;
   }
   $250 = $0;
   $251 = (($250) + 9|0);
   HEAP8[$251>>0] = 8;
   $252 = $0;
   $253 = (($252) + 11|0);
   HEAP8[$253>>0] = 24;
   $254 = $row_width;
   $255 = ($254*3)|0;
   $256 = $0;
   $257 = (($256) + 4|0);
   HEAP32[$257>>2] = $255;
   $258 = $0;
   $259 = (($258) + 8|0);
   HEAP8[$259>>0] = 2;
   $260 = $0;
   $261 = (($260) + 10|0);
   HEAP8[$261>>0] = 3;
  }
 }
 STACKTOP = sp;return;
}
function _png_do_expand($row_info,$row,$trans_value) {
 $row_info = $row_info|0;
 $row = $row|0;
 $trans_value = $trans_value|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $blue = 0, $blue_high = 0, $blue_low = 0, $dp = 0, $gray = 0, $gray_high = 0, $gray_low = 0, $green = 0, $green_high = 0, $green_low = 0, $i = 0, $red = 0, $red_high = 0, $red_low = 0, $row_width = 0, $shift = 0, $sp = 0;
 var $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $trans_value;
 $3 = $0;
 $4 = HEAP32[$3>>2]|0;
 $row_width = $4;
 $5 = $0;
 $6 = (($5) + 8|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $304 = $0;
  $305 = (($304) + 8|0);
  $306 = HEAP8[$305>>0]|0;
  $307 = $306&255;
  $308 = ($307|0)==(2);
  if ($308) {
   $309 = $2;
   $310 = ($309|0)!=(0|0);
   if ($310) {
    $311 = $0;
    $312 = (($311) + 9|0);
    $313 = HEAP8[$312>>0]|0;
    $314 = $313&255;
    $315 = ($314|0)==(8);
    if ($315) {
     $316 = $2;
     $317 = (($316) + 2|0);
     $318 = HEAP16[$317>>1]|0;
     $319 = $318&65535;
     $320 = $319 & 255;
     $321 = $320&255;
     $red = $321;
     $322 = $2;
     $323 = (($322) + 4|0);
     $324 = HEAP16[$323>>1]|0;
     $325 = $324&65535;
     $326 = $325 & 255;
     $327 = $326&255;
     $green = $327;
     $328 = $2;
     $329 = (($328) + 6|0);
     $330 = HEAP16[$329>>1]|0;
     $331 = $330&65535;
     $332 = $331 & 255;
     $333 = $332&255;
     $blue = $333;
     $334 = $1;
     $335 = $0;
     $336 = (($335) + 4|0);
     $337 = HEAP32[$336>>2]|0;
     $338 = (($334) + ($337)|0);
     $339 = (($338) + -1|0);
     $sp = $339;
     $340 = $1;
     $341 = $row_width;
     $342 = $341 << 2;
     $343 = (($340) + ($342)|0);
     $344 = (($343) + -1|0);
     $dp = $344;
     $i = 0;
     while(1) {
      $345 = $i;
      $346 = $row_width;
      $347 = ($345>>>0)<($346>>>0);
      if (!($347)) {
       break;
      }
      $348 = $sp;
      $349 = (($348) + -2|0);
      $350 = HEAP8[$349>>0]|0;
      $351 = $350&255;
      $352 = $red;
      $353 = $352&255;
      $354 = ($351|0)==($353|0);
      if ($354) {
       $355 = $sp;
       $356 = (($355) + -1|0);
       $357 = HEAP8[$356>>0]|0;
       $358 = $357&255;
       $359 = $green;
       $360 = $359&255;
       $361 = ($358|0)==($360|0);
       if ($361) {
        $362 = $sp;
        $363 = HEAP8[$362>>0]|0;
        $364 = $363&255;
        $365 = $blue;
        $366 = $365&255;
        $367 = ($364|0)==($366|0);
        if ($367) {
         $368 = $dp;
         $369 = (($368) + -1|0);
         $dp = $369;
         HEAP8[$368>>0] = 0;
        } else {
         label = 70;
        }
       } else {
        label = 70;
       }
      } else {
       label = 70;
      }
      if ((label|0) == 70) {
       label = 0;
       $370 = $dp;
       $371 = (($370) + -1|0);
       $dp = $371;
       HEAP8[$370>>0] = -1;
      }
      $372 = $sp;
      $373 = (($372) + -1|0);
      $sp = $373;
      $374 = HEAP8[$372>>0]|0;
      $375 = $dp;
      $376 = (($375) + -1|0);
      $dp = $376;
      HEAP8[$375>>0] = $374;
      $377 = $sp;
      $378 = (($377) + -1|0);
      $sp = $378;
      $379 = HEAP8[$377>>0]|0;
      $380 = $dp;
      $381 = (($380) + -1|0);
      $dp = $381;
      HEAP8[$380>>0] = $379;
      $382 = $sp;
      $383 = (($382) + -1|0);
      $sp = $383;
      $384 = HEAP8[$382>>0]|0;
      $385 = $dp;
      $386 = (($385) + -1|0);
      $dp = $386;
      HEAP8[$385>>0] = $384;
      $387 = $i;
      $388 = (($387) + 1)|0;
      $i = $388;
     }
    } else {
     $389 = $0;
     $390 = (($389) + 9|0);
     $391 = HEAP8[$390>>0]|0;
     $392 = $391&255;
     $393 = ($392|0)==(16);
     if ($393) {
      $394 = $2;
      $395 = (($394) + 2|0);
      $396 = HEAP16[$395>>1]|0;
      $397 = $396&65535;
      $398 = $397 >> 8;
      $399 = $398 & 255;
      $400 = $399&255;
      $red_high = $400;
      $401 = $2;
      $402 = (($401) + 4|0);
      $403 = HEAP16[$402>>1]|0;
      $404 = $403&65535;
      $405 = $404 >> 8;
      $406 = $405 & 255;
      $407 = $406&255;
      $green_high = $407;
      $408 = $2;
      $409 = (($408) + 6|0);
      $410 = HEAP16[$409>>1]|0;
      $411 = $410&65535;
      $412 = $411 >> 8;
      $413 = $412 & 255;
      $414 = $413&255;
      $blue_high = $414;
      $415 = $2;
      $416 = (($415) + 2|0);
      $417 = HEAP16[$416>>1]|0;
      $418 = $417&65535;
      $419 = $418 & 255;
      $420 = $419&255;
      $red_low = $420;
      $421 = $2;
      $422 = (($421) + 4|0);
      $423 = HEAP16[$422>>1]|0;
      $424 = $423&65535;
      $425 = $424 & 255;
      $426 = $425&255;
      $green_low = $426;
      $427 = $2;
      $428 = (($427) + 6|0);
      $429 = HEAP16[$428>>1]|0;
      $430 = $429&65535;
      $431 = $430 & 255;
      $432 = $431&255;
      $blue_low = $432;
      $433 = $1;
      $434 = $0;
      $435 = (($434) + 4|0);
      $436 = HEAP32[$435>>2]|0;
      $437 = (($433) + ($436)|0);
      $438 = (($437) + -1|0);
      $sp = $438;
      $439 = $1;
      $440 = $row_width;
      $441 = $440 << 3;
      $442 = (($439) + ($441)|0);
      $443 = (($442) + -1|0);
      $dp = $443;
      $i = 0;
      while(1) {
       $444 = $i;
       $445 = $row_width;
       $446 = ($444>>>0)<($445>>>0);
       if (!($446)) {
        break;
       }
       $447 = $sp;
       $448 = (($447) + -5|0);
       $449 = HEAP8[$448>>0]|0;
       $450 = $449&255;
       $451 = $red_high;
       $452 = $451&255;
       $453 = ($450|0)==($452|0);
       if ($453) {
        $454 = $sp;
        $455 = (($454) + -4|0);
        $456 = HEAP8[$455>>0]|0;
        $457 = $456&255;
        $458 = $red_low;
        $459 = $458&255;
        $460 = ($457|0)==($459|0);
        if ($460) {
         $461 = $sp;
         $462 = (($461) + -3|0);
         $463 = HEAP8[$462>>0]|0;
         $464 = $463&255;
         $465 = $green_high;
         $466 = $465&255;
         $467 = ($464|0)==($466|0);
         if ($467) {
          $468 = $sp;
          $469 = (($468) + -2|0);
          $470 = HEAP8[$469>>0]|0;
          $471 = $470&255;
          $472 = $green_low;
          $473 = $472&255;
          $474 = ($471|0)==($473|0);
          if ($474) {
           $475 = $sp;
           $476 = (($475) + -1|0);
           $477 = HEAP8[$476>>0]|0;
           $478 = $477&255;
           $479 = $blue_high;
           $480 = $479&255;
           $481 = ($478|0)==($480|0);
           if ($481) {
            $482 = $sp;
            $483 = HEAP8[$482>>0]|0;
            $484 = $483&255;
            $485 = $blue_low;
            $486 = $485&255;
            $487 = ($484|0)==($486|0);
            if ($487) {
             $488 = $dp;
             $489 = (($488) + -1|0);
             $dp = $489;
             HEAP8[$488>>0] = 0;
             $490 = $dp;
             $491 = (($490) + -1|0);
             $dp = $491;
             HEAP8[$490>>0] = 0;
            } else {
             label = 84;
            }
           } else {
            label = 84;
           }
          } else {
           label = 84;
          }
         } else {
          label = 84;
         }
        } else {
         label = 84;
        }
       } else {
        label = 84;
       }
       if ((label|0) == 84) {
        label = 0;
        $492 = $dp;
        $493 = (($492) + -1|0);
        $dp = $493;
        HEAP8[$492>>0] = -1;
        $494 = $dp;
        $495 = (($494) + -1|0);
        $dp = $495;
        HEAP8[$494>>0] = -1;
       }
       $496 = $sp;
       $497 = (($496) + -1|0);
       $sp = $497;
       $498 = HEAP8[$496>>0]|0;
       $499 = $dp;
       $500 = (($499) + -1|0);
       $dp = $500;
       HEAP8[$499>>0] = $498;
       $501 = $sp;
       $502 = (($501) + -1|0);
       $sp = $502;
       $503 = HEAP8[$501>>0]|0;
       $504 = $dp;
       $505 = (($504) + -1|0);
       $dp = $505;
       HEAP8[$504>>0] = $503;
       $506 = $sp;
       $507 = (($506) + -1|0);
       $sp = $507;
       $508 = HEAP8[$506>>0]|0;
       $509 = $dp;
       $510 = (($509) + -1|0);
       $dp = $510;
       HEAP8[$509>>0] = $508;
       $511 = $sp;
       $512 = (($511) + -1|0);
       $sp = $512;
       $513 = HEAP8[$511>>0]|0;
       $514 = $dp;
       $515 = (($514) + -1|0);
       $dp = $515;
       HEAP8[$514>>0] = $513;
       $516 = $sp;
       $517 = (($516) + -1|0);
       $sp = $517;
       $518 = HEAP8[$516>>0]|0;
       $519 = $dp;
       $520 = (($519) + -1|0);
       $dp = $520;
       HEAP8[$519>>0] = $518;
       $521 = $sp;
       $522 = (($521) + -1|0);
       $sp = $522;
       $523 = HEAP8[$521>>0]|0;
       $524 = $dp;
       $525 = (($524) + -1|0);
       $dp = $525;
       HEAP8[$524>>0] = $523;
       $526 = $i;
       $527 = (($526) + 1)|0;
       $i = $527;
      }
     }
    }
    $528 = $0;
    $529 = (($528) + 8|0);
    HEAP8[$529>>0] = 6;
    $530 = $0;
    $531 = (($530) + 10|0);
    HEAP8[$531>>0] = 4;
    $532 = $0;
    $533 = (($532) + 9|0);
    $534 = HEAP8[$533>>0]|0;
    $535 = $534&255;
    $536 = $535 << 2;
    $537 = $536&255;
    $538 = $0;
    $539 = (($538) + 11|0);
    HEAP8[$539>>0] = $537;
    $540 = $0;
    $541 = (($540) + 11|0);
    $542 = HEAP8[$541>>0]|0;
    $543 = $542&255;
    $544 = ($543|0)>=(8);
    if ($544) {
     $545 = $row_width;
     $546 = $0;
     $547 = (($546) + 11|0);
     $548 = HEAP8[$547>>0]|0;
     $549 = $548&255;
     $550 = $549 >>> 3;
     $551 = Math_imul($545, $550)|0;
     $562 = $551;
    } else {
     $552 = $row_width;
     $553 = $0;
     $554 = (($553) + 11|0);
     $555 = HEAP8[$554>>0]|0;
     $556 = $555&255;
     $557 = Math_imul($552, $556)|0;
     $558 = (($557) + 7)|0;
     $559 = $558 >>> 3;
     $562 = $559;
    }
    $560 = $0;
    $561 = (($560) + 4|0);
    HEAP32[$561>>2] = $562;
   }
  }
  STACKTOP = sp;return;
 }
 $10 = $2;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = $2;
  $13 = (($12) + 8|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $17 = $15;
 } else {
  $17 = 0;
 }
 $16 = $17&65535;
 $gray = $16;
 $18 = $0;
 $19 = (($18) + 9|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)<(8);
 if ($22) {
  $23 = $0;
  $24 = (($23) + 9|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  if ((($26|0) == 2)) {
   $67 = $gray;
   $68 = $67&65535;
   $69 = $68 & 3;
   $70 = ($69*85)|0;
   $71 = $70&65535;
   $gray = $71;
   $72 = $1;
   $73 = $row_width;
   $74 = (($73) - 1)|0;
   $75 = $74 >>> 2;
   $76 = (($72) + ($75)|0);
   $sp = $76;
   $77 = $1;
   $78 = $row_width;
   $79 = (($77) + ($78)|0);
   $80 = (($79) + -1|0);
   $dp = $80;
   $81 = $row_width;
   $82 = (($81) + 3)|0;
   $83 = $82 & 3;
   $84 = (3 - ($83))|0;
   $85 = $84 << 1;
   $shift = $85;
   $i = 0;
   while(1) {
    $86 = $i;
    $87 = $row_width;
    $88 = ($86>>>0)<($87>>>0);
    if (!($88)) {
     break;
    }
    $89 = $sp;
    $90 = HEAP8[$89>>0]|0;
    $91 = $90&255;
    $92 = $shift;
    $93 = $91 >> $92;
    $94 = $93 & 3;
    $value = $94;
    $95 = $value;
    $96 = $value;
    $97 = $96 << 2;
    $98 = $95 | $97;
    $99 = $value;
    $100 = $99 << 4;
    $101 = $98 | $100;
    $102 = $value;
    $103 = $102 << 6;
    $104 = $101 | $103;
    $105 = $104&255;
    $106 = $dp;
    HEAP8[$106>>0] = $105;
    $107 = $shift;
    $108 = ($107|0)==(6);
    if ($108) {
     $shift = 0;
     $109 = $sp;
     $110 = (($109) + -1|0);
     $sp = $110;
    } else {
     $111 = $shift;
     $112 = (($111) + 2)|0;
     $shift = $112;
    }
    $113 = $dp;
    $114 = (($113) + -1|0);
    $dp = $114;
    $115 = $i;
    $116 = (($115) + 1)|0;
    $i = $116;
   }
  } else if ((($26|0) == 1)) {
   $27 = $gray;
   $28 = $27&65535;
   $29 = $28 & 1;
   $30 = ($29*255)|0;
   $31 = $30&65535;
   $gray = $31;
   $32 = $1;
   $33 = $row_width;
   $34 = (($33) - 1)|0;
   $35 = $34 >>> 3;
   $36 = (($32) + ($35)|0);
   $sp = $36;
   $37 = $1;
   $38 = $row_width;
   $39 = (($37) + ($38)|0);
   $40 = (($39) + -1|0);
   $dp = $40;
   $41 = $row_width;
   $42 = (($41) + 7)|0;
   $43 = $42 & 7;
   $44 = (7 - ($43))|0;
   $shift = $44;
   $i = 0;
   while(1) {
    $45 = $i;
    $46 = $row_width;
    $47 = ($45>>>0)<($46>>>0);
    if (!($47)) {
     break;
    }
    $48 = $sp;
    $49 = HEAP8[$48>>0]|0;
    $50 = $49&255;
    $51 = $shift;
    $52 = $50 >> $51;
    $53 = $52 & 1;
    $54 = ($53|0)!=(0);
    if ($54) {
     $55 = $dp;
     HEAP8[$55>>0] = -1;
    } else {
     $56 = $dp;
     HEAP8[$56>>0] = 0;
    }
    $57 = $shift;
    $58 = ($57|0)==(7);
    if ($58) {
     $shift = 0;
     $59 = $sp;
     $60 = (($59) + -1|0);
     $sp = $60;
    } else {
     $61 = $shift;
     $62 = (($61) + 1)|0;
     $shift = $62;
    }
    $63 = $dp;
    $64 = (($63) + -1|0);
    $dp = $64;
    $65 = $i;
    $66 = (($65) + 1)|0;
    $i = $66;
   }
  } else if ((($26|0) == 4)) {
   $117 = $gray;
   $118 = $117&65535;
   $119 = $118 & 15;
   $120 = ($119*17)|0;
   $121 = $120&65535;
   $gray = $121;
   $122 = $1;
   $123 = $row_width;
   $124 = (($123) - 1)|0;
   $125 = $124 >>> 1;
   $126 = (($122) + ($125)|0);
   $sp = $126;
   $127 = $1;
   $128 = $row_width;
   $129 = (($127) + ($128)|0);
   $130 = (($129) + -1|0);
   $dp = $130;
   $131 = $row_width;
   $132 = (($131) + 1)|0;
   $133 = $132 & 1;
   $134 = (1 - ($133))|0;
   $135 = $134 << 2;
   $shift = $135;
   $i = 0;
   while(1) {
    $136 = $i;
    $137 = $row_width;
    $138 = ($136>>>0)<($137>>>0);
    if (!($138)) {
     break;
    }
    $139 = $sp;
    $140 = HEAP8[$139>>0]|0;
    $141 = $140&255;
    $142 = $shift;
    $143 = $141 >> $142;
    $144 = $143 & 15;
    $value = $144;
    $145 = $value;
    $146 = $value;
    $147 = $146 << 4;
    $148 = $145 | $147;
    $149 = $148&255;
    $150 = $dp;
    HEAP8[$150>>0] = $149;
    $151 = $shift;
    $152 = ($151|0)==(4);
    if ($152) {
     $shift = 0;
     $153 = $sp;
     $154 = (($153) + -1|0);
     $sp = $154;
    } else {
     $shift = 4;
    }
    $155 = $dp;
    $156 = (($155) + -1|0);
    $dp = $156;
    $157 = $i;
    $158 = (($157) + 1)|0;
    $i = $158;
   }
  }
  $159 = $0;
  $160 = (($159) + 9|0);
  HEAP8[$160>>0] = 8;
  $161 = $0;
  $162 = (($161) + 11|0);
  HEAP8[$162>>0] = 8;
  $163 = $row_width;
  $164 = $0;
  $165 = (($164) + 4|0);
  HEAP32[$165>>2] = $163;
 }
 $166 = $2;
 $167 = ($166|0)!=(0|0);
 if ($167) {
  $168 = $0;
  $169 = (($168) + 9|0);
  $170 = HEAP8[$169>>0]|0;
  $171 = $170&255;
  $172 = ($171|0)==(8);
  if ($172) {
   $173 = $gray;
   $174 = $173&65535;
   $175 = $174 & 255;
   $176 = $175&65535;
   $gray = $176;
   $177 = $1;
   $178 = $row_width;
   $179 = (($177) + ($178)|0);
   $180 = (($179) + -1|0);
   $sp = $180;
   $181 = $1;
   $182 = $row_width;
   $183 = $182 << 1;
   $184 = (($181) + ($183)|0);
   $185 = (($184) + -1|0);
   $dp = $185;
   $i = 0;
   while(1) {
    $186 = $i;
    $187 = $row_width;
    $188 = ($186>>>0)<($187>>>0);
    if (!($188)) {
     break;
    }
    $189 = $sp;
    $190 = HEAP8[$189>>0]|0;
    $191 = $190&255;
    $192 = $gray;
    $193 = $192&65535;
    $194 = ($191|0)==($193|0);
    if ($194) {
     $195 = $dp;
     $196 = (($195) + -1|0);
     $dp = $196;
     HEAP8[$195>>0] = 0;
    } else {
     $197 = $dp;
     $198 = (($197) + -1|0);
     $dp = $198;
     HEAP8[$197>>0] = -1;
    }
    $199 = $sp;
    $200 = (($199) + -1|0);
    $sp = $200;
    $201 = HEAP8[$199>>0]|0;
    $202 = $dp;
    $203 = (($202) + -1|0);
    $dp = $203;
    HEAP8[$202>>0] = $201;
    $204 = $i;
    $205 = (($204) + 1)|0;
    $i = $205;
   }
  } else {
   $206 = $0;
   $207 = (($206) + 9|0);
   $208 = HEAP8[$207>>0]|0;
   $209 = $208&255;
   $210 = ($209|0)==(16);
   if ($210) {
    $211 = $gray;
    $212 = $211&65535;
    $213 = $212 >> 8;
    $214 = $213 & 255;
    $215 = $214&255;
    $gray_high = $215;
    $216 = $gray;
    $217 = $216&65535;
    $218 = $217 & 255;
    $219 = $218&255;
    $gray_low = $219;
    $220 = $1;
    $221 = $0;
    $222 = (($221) + 4|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = (($220) + ($223)|0);
    $225 = (($224) + -1|0);
    $sp = $225;
    $226 = $1;
    $227 = $0;
    $228 = (($227) + 4|0);
    $229 = HEAP32[$228>>2]|0;
    $230 = $229 << 1;
    $231 = (($226) + ($230)|0);
    $232 = (($231) + -1|0);
    $dp = $232;
    $i = 0;
    while(1) {
     $233 = $i;
     $234 = $row_width;
     $235 = ($233>>>0)<($234>>>0);
     if (!($235)) {
      break;
     }
     $236 = $sp;
     $237 = (($236) + -1|0);
     $238 = HEAP8[$237>>0]|0;
     $239 = $238&255;
     $240 = $gray_high;
     $241 = $240&255;
     $242 = ($239|0)==($241|0);
     if ($242) {
      $243 = $sp;
      $244 = HEAP8[$243>>0]|0;
      $245 = $244&255;
      $246 = $gray_low;
      $247 = $246&255;
      $248 = ($245|0)==($247|0);
      if ($248) {
       $249 = $dp;
       $250 = (($249) + -1|0);
       $dp = $250;
       HEAP8[$249>>0] = 0;
       $251 = $dp;
       $252 = (($251) + -1|0);
       $dp = $252;
       HEAP8[$251>>0] = 0;
      } else {
       label = 51;
      }
     } else {
      label = 51;
     }
     if ((label|0) == 51) {
      label = 0;
      $253 = $dp;
      $254 = (($253) + -1|0);
      $dp = $254;
      HEAP8[$253>>0] = -1;
      $255 = $dp;
      $256 = (($255) + -1|0);
      $dp = $256;
      HEAP8[$255>>0] = -1;
     }
     $257 = $sp;
     $258 = (($257) + -1|0);
     $sp = $258;
     $259 = HEAP8[$257>>0]|0;
     $260 = $dp;
     $261 = (($260) + -1|0);
     $dp = $261;
     HEAP8[$260>>0] = $259;
     $262 = $sp;
     $263 = (($262) + -1|0);
     $sp = $263;
     $264 = HEAP8[$262>>0]|0;
     $265 = $dp;
     $266 = (($265) + -1|0);
     $dp = $266;
     HEAP8[$265>>0] = $264;
     $267 = $i;
     $268 = (($267) + 1)|0;
     $i = $268;
    }
   }
  }
  $269 = $0;
  $270 = (($269) + 8|0);
  HEAP8[$270>>0] = 4;
  $271 = $0;
  $272 = (($271) + 10|0);
  HEAP8[$272>>0] = 2;
  $273 = $0;
  $274 = (($273) + 9|0);
  $275 = HEAP8[$274>>0]|0;
  $276 = $275&255;
  $277 = $276 << 1;
  $278 = $277&255;
  $279 = $0;
  $280 = (($279) + 11|0);
  HEAP8[$280>>0] = $278;
  $281 = $0;
  $282 = (($281) + 11|0);
  $283 = HEAP8[$282>>0]|0;
  $284 = $283&255;
  $285 = ($284|0)>=(8);
  if ($285) {
   $286 = $row_width;
   $287 = $0;
   $288 = (($287) + 11|0);
   $289 = HEAP8[$288>>0]|0;
   $290 = $289&255;
   $291 = $290 >>> 3;
   $292 = Math_imul($286, $291)|0;
   $303 = $292;
  } else {
   $293 = $row_width;
   $294 = $0;
   $295 = (($294) + 11|0);
   $296 = HEAP8[$295>>0]|0;
   $297 = $296&255;
   $298 = Math_imul($293, $297)|0;
   $299 = (($298) + 7)|0;
   $300 = $299 >>> 3;
   $303 = $300;
  }
  $301 = $0;
  $302 = (($301) + 4|0);
  HEAP32[$302>>2] = $303;
 }
 STACKTOP = sp;return;
}
function _png_do_rgb_to_gray($png_ptr,$row_info,$row) {
 $png_ptr = $png_ptr|0;
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bc = 0, $blue = 0, $blue10 = 0, $blue15 = 0, $blue21 = 0, $blue26 = 0, $blue31 = 0, $blue41 = 0, $blue5 = 0, $blue_1 = 0, $blue_135 = 0, $dp = 0, $dp12 = 0, $dp18 = 0, $dp2 = 0;
 var $dp23 = 0, $dp28 = 0, $dp38 = 0, $dp7 = 0, $gc = 0, $gray16 = 0, $gray1616 = 0, $gray1636 = 0, $gray1642 = 0, $green = 0, $green14 = 0, $green20 = 0, $green25 = 0, $green30 = 0, $green4 = 0, $green40 = 0, $green9 = 0, $green_1 = 0, $green_134 = 0, $i = 0;
 var $rc = 0, $red = 0, $red13 = 0, $red19 = 0, $red24 = 0, $red29 = 0, $red3 = 0, $red39 = 0, $red8 = 0, $red_1 = 0, $red_133 = 0, $rgb_error = 0, $row_width = 0, $sp = 0, $sp1 = 0, $sp11 = 0, $sp17 = 0, $sp22 = 0, $sp27 = 0, $sp37 = 0;
 var $sp6 = 0, $w = 0, $w32 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = $png_ptr;
 $1 = $row_info;
 $2 = $row;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $row_width = $4;
 $rgb_error = 0;
 $5 = $1;
 $6 = (($5) + 8|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 & 2;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $864 = $rgb_error;
  STACKTOP = sp;return ($864|0);
 }
 $11 = $0;
 $12 = (($11) + 646|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $rc = $14;
 $15 = $0;
 $16 = (($15) + 648|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17&65535;
 $gc = $18;
 $19 = $0;
 $20 = (($19) + 650|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $bc = $22;
 $23 = $1;
 $24 = (($23) + 8|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)==(2);
 if ($27) {
  $28 = $1;
  $29 = (($28) + 9|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = ($31|0)==(8);
  if ($32) {
   $33 = $0;
   $34 = (($33) + 452|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = $0;
    $38 = (($37) + 456|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)!=(0|0);
    if ($40) {
     $41 = $2;
     $sp = $41;
     $42 = $2;
     $dp = $42;
     $i = 0;
     while(1) {
      $43 = $i;
      $44 = $row_width;
      $45 = ($43>>>0)<($44>>>0);
      if (!($45)) {
       break;
      }
      $46 = $sp;
      $47 = (($46) + 1|0);
      $sp = $47;
      $48 = HEAP8[$46>>0]|0;
      $49 = $48&255;
      $50 = $0;
      $51 = (($50) + 456|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = (($52) + ($49)|0);
      $54 = HEAP8[$53>>0]|0;
      $red = $54;
      $55 = $sp;
      $56 = (($55) + 1|0);
      $sp = $56;
      $57 = HEAP8[$55>>0]|0;
      $58 = $57&255;
      $59 = $0;
      $60 = (($59) + 456|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = (($61) + ($58)|0);
      $63 = HEAP8[$62>>0]|0;
      $green = $63;
      $64 = $sp;
      $65 = (($64) + 1|0);
      $sp = $65;
      $66 = HEAP8[$64>>0]|0;
      $67 = $66&255;
      $68 = $0;
      $69 = (($68) + 456|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = (($70) + ($67)|0);
      $72 = HEAP8[$71>>0]|0;
      $blue = $72;
      $73 = $red;
      $74 = $73&255;
      $75 = $green;
      $76 = $75&255;
      $77 = ($74|0)!=($76|0);
      if ($77) {
       label = 10;
      } else {
       $78 = $red;
       $79 = $78&255;
       $80 = $blue;
       $81 = $80&255;
       $82 = ($79|0)!=($81|0);
       if ($82) {
        label = 10;
       } else {
        $107 = $sp;
        $108 = (($107) + -1|0);
        $109 = HEAP8[$108>>0]|0;
        $110 = $dp;
        $111 = (($110) + 1|0);
        $dp = $111;
        HEAP8[$110>>0] = $109;
       }
      }
      if ((label|0) == 10) {
       label = 0;
       $83 = $rgb_error;
       $84 = $83 | 1;
       $rgb_error = $84;
       $85 = $rc;
       $86 = $red;
       $87 = $86&255;
       $88 = Math_imul($85, $87)|0;
       $89 = $gc;
       $90 = $green;
       $91 = $90&255;
       $92 = Math_imul($89, $91)|0;
       $93 = (($88) + ($92))|0;
       $94 = $bc;
       $95 = $blue;
       $96 = $95&255;
       $97 = Math_imul($94, $96)|0;
       $98 = (($93) + ($97))|0;
       $99 = $98 >>> 15;
       $100 = $0;
       $101 = (($100) + 452|0);
       $102 = HEAP32[$101>>2]|0;
       $103 = (($102) + ($99)|0);
       $104 = HEAP8[$103>>0]|0;
       $105 = $dp;
       $106 = (($105) + 1|0);
       $dp = $106;
       HEAP8[$105>>0] = $104;
      }
      $112 = $i;
      $113 = (($112) + 1)|0;
      $i = $113;
     }
    } else {
     label = 15;
    }
   } else {
    label = 15;
   }
   if ((label|0) == 15) {
    $114 = $2;
    $sp1 = $114;
    $115 = $2;
    $dp2 = $115;
    $i = 0;
    while(1) {
     $116 = $i;
     $117 = $row_width;
     $118 = ($116>>>0)<($117>>>0);
     if (!($118)) {
      break;
     }
     $119 = $sp1;
     $120 = (($119) + 1|0);
     $sp1 = $120;
     $121 = HEAP8[$119>>0]|0;
     $red3 = $121;
     $122 = $sp1;
     $123 = (($122) + 1|0);
     $sp1 = $123;
     $124 = HEAP8[$122>>0]|0;
     $green4 = $124;
     $125 = $sp1;
     $126 = (($125) + 1|0);
     $sp1 = $126;
     $127 = HEAP8[$125>>0]|0;
     $blue5 = $127;
     $128 = $red3;
     $129 = $128&255;
     $130 = $green4;
     $131 = $130&255;
     $132 = ($129|0)!=($131|0);
     if ($132) {
      label = 19;
     } else {
      $133 = $red3;
      $134 = $133&255;
      $135 = $blue5;
      $136 = $135&255;
      $137 = ($134|0)!=($136|0);
      if ($137) {
       label = 19;
      } else {
       $158 = $sp1;
       $159 = (($158) + -1|0);
       $160 = HEAP8[$159>>0]|0;
       $161 = $dp2;
       $162 = (($161) + 1|0);
       $dp2 = $162;
       HEAP8[$161>>0] = $160;
      }
     }
     if ((label|0) == 19) {
      label = 0;
      $138 = $rgb_error;
      $139 = $138 | 1;
      $rgb_error = $139;
      $140 = $rc;
      $141 = $red3;
      $142 = $141&255;
      $143 = Math_imul($140, $142)|0;
      $144 = $gc;
      $145 = $green4;
      $146 = $145&255;
      $147 = Math_imul($144, $146)|0;
      $148 = (($143) + ($147))|0;
      $149 = $bc;
      $150 = $blue5;
      $151 = $150&255;
      $152 = Math_imul($149, $151)|0;
      $153 = (($148) + ($152))|0;
      $154 = $153 >>> 15;
      $155 = $154&255;
      $156 = $dp2;
      $157 = (($156) + 1|0);
      $dp2 = $157;
      HEAP8[$156>>0] = $155;
     }
     $163 = $i;
     $164 = (($163) + 1)|0;
     $i = $164;
    }
   }
  } else {
   $165 = $0;
   $166 = (($165) + 468|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = ($167|0)!=(0|0);
   if ($168) {
    $169 = $0;
    $170 = (($169) + 464|0);
    $171 = HEAP32[$170>>2]|0;
    $172 = ($171|0)!=(0|0);
    if ($172) {
     $173 = $2;
     $sp6 = $173;
     $174 = $2;
     $dp7 = $174;
     $i = 0;
     while(1) {
      $175 = $i;
      $176 = $row_width;
      $177 = ($175>>>0)<($176>>>0);
      if (!($177)) {
       break;
      }
      $178 = $sp6;
      $179 = HEAP8[$178>>0]|0;
      $180 = $179&255;
      $181 = $180 << 8;
      $182 = $sp6;
      $183 = (($182) + 1|0);
      $184 = HEAP8[$183>>0]|0;
      $185 = $184&255;
      $186 = $181 | $185;
      $187 = $186&65535;
      $red8 = $187;
      $188 = $sp6;
      $189 = (($188) + 2|0);
      $sp6 = $189;
      $190 = $sp6;
      $191 = HEAP8[$190>>0]|0;
      $192 = $191&255;
      $193 = $192 << 8;
      $194 = $sp6;
      $195 = (($194) + 1|0);
      $196 = HEAP8[$195>>0]|0;
      $197 = $196&255;
      $198 = $193 | $197;
      $199 = $198&65535;
      $green9 = $199;
      $200 = $sp6;
      $201 = (($200) + 2|0);
      $sp6 = $201;
      $202 = $sp6;
      $203 = HEAP8[$202>>0]|0;
      $204 = $203&255;
      $205 = $204 << 8;
      $206 = $sp6;
      $207 = (($206) + 1|0);
      $208 = HEAP8[$207>>0]|0;
      $209 = $208&255;
      $210 = $205 | $209;
      $211 = $210&65535;
      $blue10 = $211;
      $212 = $sp6;
      $213 = (($212) + 2|0);
      $sp6 = $213;
      $214 = $red8;
      $215 = $214&65535;
      $216 = $green9;
      $217 = $216&65535;
      $218 = ($215|0)==($217|0);
      if ($218) {
       $219 = $red8;
       $220 = $219&65535;
       $221 = $blue10;
       $222 = $221&65535;
       $223 = ($220|0)==($222|0);
       if ($223) {
        $224 = $red8;
        $w = $224;
       } else {
        label = 32;
       }
      } else {
       label = 32;
      }
      if ((label|0) == 32) {
       label = 0;
       $225 = $red8;
       $226 = $225&65535;
       $227 = $226 >> 8;
       $228 = $red8;
       $229 = $228&65535;
       $230 = $229 & 255;
       $231 = $0;
       $232 = (($231) + 436|0);
       $233 = HEAP32[$232>>2]|0;
       $234 = $230 >> $233;
       $235 = $0;
       $236 = (($235) + 468|0);
       $237 = HEAP32[$236>>2]|0;
       $238 = (($237) + ($234<<2)|0);
       $239 = HEAP32[$238>>2]|0;
       $240 = (($239) + ($227<<1)|0);
       $241 = HEAP16[$240>>1]|0;
       $red_1 = $241;
       $242 = $green9;
       $243 = $242&65535;
       $244 = $243 >> 8;
       $245 = $green9;
       $246 = $245&65535;
       $247 = $246 & 255;
       $248 = $0;
       $249 = (($248) + 436|0);
       $250 = HEAP32[$249>>2]|0;
       $251 = $247 >> $250;
       $252 = $0;
       $253 = (($252) + 468|0);
       $254 = HEAP32[$253>>2]|0;
       $255 = (($254) + ($251<<2)|0);
       $256 = HEAP32[$255>>2]|0;
       $257 = (($256) + ($244<<1)|0);
       $258 = HEAP16[$257>>1]|0;
       $green_1 = $258;
       $259 = $blue10;
       $260 = $259&65535;
       $261 = $260 >> 8;
       $262 = $blue10;
       $263 = $262&65535;
       $264 = $263 & 255;
       $265 = $0;
       $266 = (($265) + 436|0);
       $267 = HEAP32[$266>>2]|0;
       $268 = $264 >> $267;
       $269 = $0;
       $270 = (($269) + 468|0);
       $271 = HEAP32[$270>>2]|0;
       $272 = (($271) + ($268<<2)|0);
       $273 = HEAP32[$272>>2]|0;
       $274 = (($273) + ($261<<1)|0);
       $275 = HEAP16[$274>>1]|0;
       $blue_1 = $275;
       $276 = $rc;
       $277 = $red_1;
       $278 = $277&65535;
       $279 = Math_imul($276, $278)|0;
       $280 = $gc;
       $281 = $green_1;
       $282 = $281&65535;
       $283 = Math_imul($280, $282)|0;
       $284 = (($279) + ($283))|0;
       $285 = $bc;
       $286 = $blue_1;
       $287 = $286&65535;
       $288 = Math_imul($285, $287)|0;
       $289 = (($284) + ($288))|0;
       $290 = $289 >>> 15;
       $291 = $290&65535;
       $gray16 = $291;
       $292 = $gray16;
       $293 = $292&65535;
       $294 = $293 >> 8;
       $295 = $gray16;
       $296 = $295&65535;
       $297 = $296 & 255;
       $298 = $0;
       $299 = (($298) + 436|0);
       $300 = HEAP32[$299>>2]|0;
       $301 = $297 >> $300;
       $302 = $0;
       $303 = (($302) + 464|0);
       $304 = HEAP32[$303>>2]|0;
       $305 = (($304) + ($301<<2)|0);
       $306 = HEAP32[$305>>2]|0;
       $307 = (($306) + ($294<<1)|0);
       $308 = HEAP16[$307>>1]|0;
       $w = $308;
       $309 = $rgb_error;
       $310 = $309 | 1;
       $rgb_error = $310;
      }
      $311 = $w;
      $312 = $311&65535;
      $313 = $312 >> 8;
      $314 = $313 & 255;
      $315 = $314&255;
      $316 = $dp7;
      $317 = (($316) + 1|0);
      $dp7 = $317;
      HEAP8[$316>>0] = $315;
      $318 = $w;
      $319 = $318&65535;
      $320 = $319 & 255;
      $321 = $320&255;
      $322 = $dp7;
      $323 = (($322) + 1|0);
      $dp7 = $323;
      HEAP8[$322>>0] = $321;
      $324 = $i;
      $325 = (($324) + 1)|0;
      $i = $325;
     }
    } else {
     label = 36;
    }
   } else {
    label = 36;
   }
   if ((label|0) == 36) {
    $326 = $2;
    $sp11 = $326;
    $327 = $2;
    $dp12 = $327;
    $i = 0;
    while(1) {
     $328 = $i;
     $329 = $row_width;
     $330 = ($328>>>0)<($329>>>0);
     if (!($330)) {
      break;
     }
     $331 = $sp11;
     $332 = HEAP8[$331>>0]|0;
     $333 = $332&255;
     $334 = $333 << 8;
     $335 = $sp11;
     $336 = (($335) + 1|0);
     $337 = HEAP8[$336>>0]|0;
     $338 = $337&255;
     $339 = $334 | $338;
     $340 = $339&65535;
     $red13 = $340;
     $341 = $sp11;
     $342 = (($341) + 2|0);
     $sp11 = $342;
     $343 = $sp11;
     $344 = HEAP8[$343>>0]|0;
     $345 = $344&255;
     $346 = $345 << 8;
     $347 = $sp11;
     $348 = (($347) + 1|0);
     $349 = HEAP8[$348>>0]|0;
     $350 = $349&255;
     $351 = $346 | $350;
     $352 = $351&65535;
     $green14 = $352;
     $353 = $sp11;
     $354 = (($353) + 2|0);
     $sp11 = $354;
     $355 = $sp11;
     $356 = HEAP8[$355>>0]|0;
     $357 = $356&255;
     $358 = $357 << 8;
     $359 = $sp11;
     $360 = (($359) + 1|0);
     $361 = HEAP8[$360>>0]|0;
     $362 = $361&255;
     $363 = $358 | $362;
     $364 = $363&65535;
     $blue15 = $364;
     $365 = $sp11;
     $366 = (($365) + 2|0);
     $sp11 = $366;
     $367 = $red13;
     $368 = $367&65535;
     $369 = $green14;
     $370 = $369&65535;
     $371 = ($368|0)!=($370|0);
     if ($371) {
      label = 40;
     } else {
      $372 = $red13;
      $373 = $372&65535;
      $374 = $blue15;
      $375 = $374&65535;
      $376 = ($373|0)!=($375|0);
      if ($376) {
       label = 40;
      }
     }
     if ((label|0) == 40) {
      label = 0;
      $377 = $rgb_error;
      $378 = $377 | 1;
      $rgb_error = $378;
     }
     $379 = $rc;
     $380 = $red13;
     $381 = $380&65535;
     $382 = Math_imul($379, $381)|0;
     $383 = $gc;
     $384 = $green14;
     $385 = $384&65535;
     $386 = Math_imul($383, $385)|0;
     $387 = (($382) + ($386))|0;
     $388 = $bc;
     $389 = $blue15;
     $390 = $389&65535;
     $391 = Math_imul($388, $390)|0;
     $392 = (($387) + ($391))|0;
     $393 = $392 >>> 15;
     $394 = $393&65535;
     $gray1616 = $394;
     $395 = $gray1616;
     $396 = $395&65535;
     $397 = $396 >> 8;
     $398 = $397 & 255;
     $399 = $398&255;
     $400 = $dp12;
     $401 = (($400) + 1|0);
     $dp12 = $401;
     HEAP8[$400>>0] = $399;
     $402 = $gray1616;
     $403 = $402&65535;
     $404 = $403 & 255;
     $405 = $404&255;
     $406 = $dp12;
     $407 = (($406) + 1|0);
     $dp12 = $407;
     HEAP8[$406>>0] = $405;
     $408 = $i;
     $409 = (($408) + 1)|0;
     $i = $409;
    }
   }
  }
 }
 $410 = $1;
 $411 = (($410) + 8|0);
 $412 = HEAP8[$411>>0]|0;
 $413 = $412&255;
 $414 = ($413|0)==(6);
 if ($414) {
  $415 = $1;
  $416 = (($415) + 9|0);
  $417 = HEAP8[$416>>0]|0;
  $418 = $417&255;
  $419 = ($418|0)==(8);
  if ($419) {
   $420 = $0;
   $421 = (($420) + 452|0);
   $422 = HEAP32[$421>>2]|0;
   $423 = ($422|0)!=(0|0);
   if ($423) {
    $424 = $0;
    $425 = (($424) + 456|0);
    $426 = HEAP32[$425>>2]|0;
    $427 = ($426|0)!=(0|0);
    if ($427) {
     $428 = $2;
     $sp17 = $428;
     $429 = $2;
     $dp18 = $429;
     $i = 0;
     while(1) {
      $430 = $i;
      $431 = $row_width;
      $432 = ($430>>>0)<($431>>>0);
      if (!($432)) {
       break;
      }
      $433 = $sp17;
      $434 = (($433) + 1|0);
      $sp17 = $434;
      $435 = HEAP8[$433>>0]|0;
      $436 = $435&255;
      $437 = $0;
      $438 = (($437) + 456|0);
      $439 = HEAP32[$438>>2]|0;
      $440 = (($439) + ($436)|0);
      $441 = HEAP8[$440>>0]|0;
      $red19 = $441;
      $442 = $sp17;
      $443 = (($442) + 1|0);
      $sp17 = $443;
      $444 = HEAP8[$442>>0]|0;
      $445 = $444&255;
      $446 = $0;
      $447 = (($446) + 456|0);
      $448 = HEAP32[$447>>2]|0;
      $449 = (($448) + ($445)|0);
      $450 = HEAP8[$449>>0]|0;
      $green20 = $450;
      $451 = $sp17;
      $452 = (($451) + 1|0);
      $sp17 = $452;
      $453 = HEAP8[$451>>0]|0;
      $454 = $453&255;
      $455 = $0;
      $456 = (($455) + 456|0);
      $457 = HEAP32[$456>>2]|0;
      $458 = (($457) + ($454)|0);
      $459 = HEAP8[$458>>0]|0;
      $blue21 = $459;
      $460 = $red19;
      $461 = $460&255;
      $462 = $green20;
      $463 = $462&255;
      $464 = ($461|0)!=($463|0);
      if ($464) {
       label = 54;
      } else {
       $465 = $red19;
       $466 = $465&255;
       $467 = $blue21;
       $468 = $467&255;
       $469 = ($466|0)!=($468|0);
       if ($469) {
        label = 54;
       }
      }
      if ((label|0) == 54) {
       label = 0;
       $470 = $rgb_error;
       $471 = $470 | 1;
       $rgb_error = $471;
      }
      $472 = $rc;
      $473 = $red19;
      $474 = $473&255;
      $475 = Math_imul($472, $474)|0;
      $476 = $gc;
      $477 = $green20;
      $478 = $477&255;
      $479 = Math_imul($476, $478)|0;
      $480 = (($475) + ($479))|0;
      $481 = $bc;
      $482 = $blue21;
      $483 = $482&255;
      $484 = Math_imul($481, $483)|0;
      $485 = (($480) + ($484))|0;
      $486 = $485 >>> 15;
      $487 = $0;
      $488 = (($487) + 452|0);
      $489 = HEAP32[$488>>2]|0;
      $490 = (($489) + ($486)|0);
      $491 = HEAP8[$490>>0]|0;
      $492 = $dp18;
      $493 = (($492) + 1|0);
      $dp18 = $493;
      HEAP8[$492>>0] = $491;
      $494 = $sp17;
      $495 = (($494) + 1|0);
      $sp17 = $495;
      $496 = HEAP8[$494>>0]|0;
      $497 = $dp18;
      $498 = (($497) + 1|0);
      $dp18 = $498;
      HEAP8[$497>>0] = $496;
      $499 = $i;
      $500 = (($499) + 1)|0;
      $i = $500;
     }
    } else {
     label = 58;
    }
   } else {
    label = 58;
   }
   if ((label|0) == 58) {
    $501 = $2;
    $sp22 = $501;
    $502 = $2;
    $dp23 = $502;
    $i = 0;
    while(1) {
     $503 = $i;
     $504 = $row_width;
     $505 = ($503>>>0)<($504>>>0);
     if (!($505)) {
      break;
     }
     $506 = $sp22;
     $507 = (($506) + 1|0);
     $sp22 = $507;
     $508 = HEAP8[$506>>0]|0;
     $red24 = $508;
     $509 = $sp22;
     $510 = (($509) + 1|0);
     $sp22 = $510;
     $511 = HEAP8[$509>>0]|0;
     $green25 = $511;
     $512 = $sp22;
     $513 = (($512) + 1|0);
     $sp22 = $513;
     $514 = HEAP8[$512>>0]|0;
     $blue26 = $514;
     $515 = $red24;
     $516 = $515&255;
     $517 = $green25;
     $518 = $517&255;
     $519 = ($516|0)!=($518|0);
     if ($519) {
      label = 62;
     } else {
      $520 = $red24;
      $521 = $520&255;
      $522 = $blue26;
      $523 = $522&255;
      $524 = ($521|0)!=($523|0);
      if ($524) {
       label = 62;
      }
     }
     if ((label|0) == 62) {
      label = 0;
      $525 = $rgb_error;
      $526 = $525 | 1;
      $rgb_error = $526;
     }
     $527 = $rc;
     $528 = $red24;
     $529 = $528&255;
     $530 = Math_imul($527, $529)|0;
     $531 = $gc;
     $532 = $green25;
     $533 = $532&255;
     $534 = Math_imul($531, $533)|0;
     $535 = (($530) + ($534))|0;
     $536 = $bc;
     $537 = $blue26;
     $538 = $537&255;
     $539 = Math_imul($536, $538)|0;
     $540 = (($535) + ($539))|0;
     $541 = $540 >>> 15;
     $542 = $541&255;
     $543 = $dp23;
     $544 = (($543) + 1|0);
     $dp23 = $544;
     HEAP8[$543>>0] = $542;
     $545 = $sp22;
     $546 = (($545) + 1|0);
     $sp22 = $546;
     $547 = HEAP8[$545>>0]|0;
     $548 = $dp23;
     $549 = (($548) + 1|0);
     $dp23 = $549;
     HEAP8[$548>>0] = $547;
     $550 = $i;
     $551 = (($550) + 1)|0;
     $i = $551;
    }
   }
  } else {
   $552 = $0;
   $553 = (($552) + 468|0);
   $554 = HEAP32[$553>>2]|0;
   $555 = ($554|0)!=(0|0);
   if ($555) {
    $556 = $0;
    $557 = (($556) + 464|0);
    $558 = HEAP32[$557>>2]|0;
    $559 = ($558|0)!=(0|0);
    if ($559) {
     $560 = $2;
     $sp27 = $560;
     $561 = $2;
     $dp28 = $561;
     $i = 0;
     while(1) {
      $562 = $i;
      $563 = $row_width;
      $564 = ($562>>>0)<($563>>>0);
      if (!($564)) {
       break;
      }
      $565 = $sp27;
      $566 = HEAP8[$565>>0]|0;
      $567 = $566&255;
      $568 = $567 << 8;
      $569 = $sp27;
      $570 = (($569) + 1|0);
      $571 = HEAP8[$570>>0]|0;
      $572 = $571&255;
      $573 = $568 | $572;
      $574 = $573&65535;
      $red29 = $574;
      $575 = $sp27;
      $576 = (($575) + 2|0);
      $sp27 = $576;
      $577 = $sp27;
      $578 = HEAP8[$577>>0]|0;
      $579 = $578&255;
      $580 = $579 << 8;
      $581 = $sp27;
      $582 = (($581) + 1|0);
      $583 = HEAP8[$582>>0]|0;
      $584 = $583&255;
      $585 = $580 | $584;
      $586 = $585&65535;
      $green30 = $586;
      $587 = $sp27;
      $588 = (($587) + 2|0);
      $sp27 = $588;
      $589 = $sp27;
      $590 = HEAP8[$589>>0]|0;
      $591 = $590&255;
      $592 = $591 << 8;
      $593 = $sp27;
      $594 = (($593) + 1|0);
      $595 = HEAP8[$594>>0]|0;
      $596 = $595&255;
      $597 = $592 | $596;
      $598 = $597&65535;
      $blue31 = $598;
      $599 = $sp27;
      $600 = (($599) + 2|0);
      $sp27 = $600;
      $601 = $red29;
      $602 = $601&65535;
      $603 = $green30;
      $604 = $603&65535;
      $605 = ($602|0)==($604|0);
      if ($605) {
       $606 = $red29;
       $607 = $606&65535;
       $608 = $blue31;
       $609 = $608&65535;
       $610 = ($607|0)==($609|0);
       if ($610) {
        $611 = $red29;
        $w32 = $611;
       } else {
        label = 74;
       }
      } else {
       label = 74;
      }
      if ((label|0) == 74) {
       label = 0;
       $612 = $red29;
       $613 = $612&65535;
       $614 = $613 >> 8;
       $615 = $red29;
       $616 = $615&65535;
       $617 = $616 & 255;
       $618 = $0;
       $619 = (($618) + 436|0);
       $620 = HEAP32[$619>>2]|0;
       $621 = $617 >> $620;
       $622 = $0;
       $623 = (($622) + 468|0);
       $624 = HEAP32[$623>>2]|0;
       $625 = (($624) + ($621<<2)|0);
       $626 = HEAP32[$625>>2]|0;
       $627 = (($626) + ($614<<1)|0);
       $628 = HEAP16[$627>>1]|0;
       $red_133 = $628;
       $629 = $green30;
       $630 = $629&65535;
       $631 = $630 >> 8;
       $632 = $green30;
       $633 = $632&65535;
       $634 = $633 & 255;
       $635 = $0;
       $636 = (($635) + 436|0);
       $637 = HEAP32[$636>>2]|0;
       $638 = $634 >> $637;
       $639 = $0;
       $640 = (($639) + 468|0);
       $641 = HEAP32[$640>>2]|0;
       $642 = (($641) + ($638<<2)|0);
       $643 = HEAP32[$642>>2]|0;
       $644 = (($643) + ($631<<1)|0);
       $645 = HEAP16[$644>>1]|0;
       $green_134 = $645;
       $646 = $blue31;
       $647 = $646&65535;
       $648 = $647 >> 8;
       $649 = $blue31;
       $650 = $649&65535;
       $651 = $650 & 255;
       $652 = $0;
       $653 = (($652) + 436|0);
       $654 = HEAP32[$653>>2]|0;
       $655 = $651 >> $654;
       $656 = $0;
       $657 = (($656) + 468|0);
       $658 = HEAP32[$657>>2]|0;
       $659 = (($658) + ($655<<2)|0);
       $660 = HEAP32[$659>>2]|0;
       $661 = (($660) + ($648<<1)|0);
       $662 = HEAP16[$661>>1]|0;
       $blue_135 = $662;
       $663 = $rc;
       $664 = $red_133;
       $665 = $664&65535;
       $666 = Math_imul($663, $665)|0;
       $667 = $gc;
       $668 = $green_134;
       $669 = $668&65535;
       $670 = Math_imul($667, $669)|0;
       $671 = (($666) + ($670))|0;
       $672 = $bc;
       $673 = $blue_135;
       $674 = $673&65535;
       $675 = Math_imul($672, $674)|0;
       $676 = (($671) + ($675))|0;
       $677 = $676 >>> 15;
       $678 = $677&65535;
       $gray1636 = $678;
       $679 = $gray1636;
       $680 = $679&65535;
       $681 = $680 >> 8;
       $682 = $gray1636;
       $683 = $682&65535;
       $684 = $683 & 255;
       $685 = $0;
       $686 = (($685) + 436|0);
       $687 = HEAP32[$686>>2]|0;
       $688 = $684 >> $687;
       $689 = $0;
       $690 = (($689) + 464|0);
       $691 = HEAP32[$690>>2]|0;
       $692 = (($691) + ($688<<2)|0);
       $693 = HEAP32[$692>>2]|0;
       $694 = (($693) + ($681<<1)|0);
       $695 = HEAP16[$694>>1]|0;
       $w32 = $695;
       $696 = $rgb_error;
       $697 = $696 | 1;
       $rgb_error = $697;
      }
      $698 = $w32;
      $699 = $698&65535;
      $700 = $699 >> 8;
      $701 = $700 & 255;
      $702 = $701&255;
      $703 = $dp28;
      $704 = (($703) + 1|0);
      $dp28 = $704;
      HEAP8[$703>>0] = $702;
      $705 = $w32;
      $706 = $705&65535;
      $707 = $706 & 255;
      $708 = $707&255;
      $709 = $dp28;
      $710 = (($709) + 1|0);
      $dp28 = $710;
      HEAP8[$709>>0] = $708;
      $711 = $sp27;
      $712 = (($711) + 1|0);
      $sp27 = $712;
      $713 = HEAP8[$711>>0]|0;
      $714 = $dp28;
      $715 = (($714) + 1|0);
      $dp28 = $715;
      HEAP8[$714>>0] = $713;
      $716 = $sp27;
      $717 = (($716) + 1|0);
      $sp27 = $717;
      $718 = HEAP8[$716>>0]|0;
      $719 = $dp28;
      $720 = (($719) + 1|0);
      $dp28 = $720;
      HEAP8[$719>>0] = $718;
      $721 = $i;
      $722 = (($721) + 1)|0;
      $i = $722;
     }
    } else {
     label = 78;
    }
   } else {
    label = 78;
   }
   if ((label|0) == 78) {
    $723 = $2;
    $sp37 = $723;
    $724 = $2;
    $dp38 = $724;
    $i = 0;
    while(1) {
     $725 = $i;
     $726 = $row_width;
     $727 = ($725>>>0)<($726>>>0);
     if (!($727)) {
      break;
     }
     $728 = $sp37;
     $729 = HEAP8[$728>>0]|0;
     $730 = $729&255;
     $731 = $730 << 8;
     $732 = $sp37;
     $733 = (($732) + 1|0);
     $734 = HEAP8[$733>>0]|0;
     $735 = $734&255;
     $736 = $731 | $735;
     $737 = $736&65535;
     $red39 = $737;
     $738 = $sp37;
     $739 = (($738) + 2|0);
     $sp37 = $739;
     $740 = $sp37;
     $741 = HEAP8[$740>>0]|0;
     $742 = $741&255;
     $743 = $742 << 8;
     $744 = $sp37;
     $745 = (($744) + 1|0);
     $746 = HEAP8[$745>>0]|0;
     $747 = $746&255;
     $748 = $743 | $747;
     $749 = $748&65535;
     $green40 = $749;
     $750 = $sp37;
     $751 = (($750) + 2|0);
     $sp37 = $751;
     $752 = $sp37;
     $753 = HEAP8[$752>>0]|0;
     $754 = $753&255;
     $755 = $754 << 8;
     $756 = $sp37;
     $757 = (($756) + 1|0);
     $758 = HEAP8[$757>>0]|0;
     $759 = $758&255;
     $760 = $755 | $759;
     $761 = $760&65535;
     $blue41 = $761;
     $762 = $sp37;
     $763 = (($762) + 2|0);
     $sp37 = $763;
     $764 = $red39;
     $765 = $764&65535;
     $766 = $green40;
     $767 = $766&65535;
     $768 = ($765|0)!=($767|0);
     if ($768) {
      label = 82;
     } else {
      $769 = $red39;
      $770 = $769&65535;
      $771 = $blue41;
      $772 = $771&65535;
      $773 = ($770|0)!=($772|0);
      if ($773) {
       label = 82;
      }
     }
     if ((label|0) == 82) {
      label = 0;
      $774 = $rgb_error;
      $775 = $774 | 1;
      $rgb_error = $775;
     }
     $776 = $rc;
     $777 = $red39;
     $778 = $777&65535;
     $779 = Math_imul($776, $778)|0;
     $780 = $gc;
     $781 = $green40;
     $782 = $781&65535;
     $783 = Math_imul($780, $782)|0;
     $784 = (($779) + ($783))|0;
     $785 = $bc;
     $786 = $blue41;
     $787 = $786&65535;
     $788 = Math_imul($785, $787)|0;
     $789 = (($784) + ($788))|0;
     $790 = $789 >>> 15;
     $791 = $790&65535;
     $gray1642 = $791;
     $792 = $gray1642;
     $793 = $792&65535;
     $794 = $793 >> 8;
     $795 = $794 & 255;
     $796 = $795&255;
     $797 = $dp38;
     $798 = (($797) + 1|0);
     $dp38 = $798;
     HEAP8[$797>>0] = $796;
     $799 = $gray1642;
     $800 = $799&65535;
     $801 = $800 & 255;
     $802 = $801&255;
     $803 = $dp38;
     $804 = (($803) + 1|0);
     $dp38 = $804;
     HEAP8[$803>>0] = $802;
     $805 = $sp37;
     $806 = (($805) + 1|0);
     $sp37 = $806;
     $807 = HEAP8[$805>>0]|0;
     $808 = $dp38;
     $809 = (($808) + 1|0);
     $dp38 = $809;
     HEAP8[$808>>0] = $807;
     $810 = $sp37;
     $811 = (($810) + 1|0);
     $sp37 = $811;
     $812 = HEAP8[$810>>0]|0;
     $813 = $dp38;
     $814 = (($813) + 1|0);
     $dp38 = $814;
     HEAP8[$813>>0] = $812;
     $815 = $i;
     $816 = (($815) + 1)|0;
     $i = $816;
    }
   }
  }
 }
 $817 = $1;
 $818 = (($817) + 10|0);
 $819 = HEAP8[$818>>0]|0;
 $820 = $819&255;
 $821 = (($820) - 2)|0;
 $822 = $821&255;
 HEAP8[$818>>0] = $822;
 $823 = $1;
 $824 = (($823) + 8|0);
 $825 = HEAP8[$824>>0]|0;
 $826 = $825&255;
 $827 = $826 & -3;
 $828 = $827&255;
 HEAP8[$824>>0] = $828;
 $829 = $1;
 $830 = (($829) + 10|0);
 $831 = HEAP8[$830>>0]|0;
 $832 = $831&255;
 $833 = $1;
 $834 = (($833) + 9|0);
 $835 = HEAP8[$834>>0]|0;
 $836 = $835&255;
 $837 = Math_imul($832, $836)|0;
 $838 = $837&255;
 $839 = $1;
 $840 = (($839) + 11|0);
 HEAP8[$840>>0] = $838;
 $841 = $1;
 $842 = (($841) + 11|0);
 $843 = HEAP8[$842>>0]|0;
 $844 = $843&255;
 $845 = ($844|0)>=(8);
 if ($845) {
  $846 = $row_width;
  $847 = $1;
  $848 = (($847) + 11|0);
  $849 = HEAP8[$848>>0]|0;
  $850 = $849&255;
  $851 = $850 >>> 3;
  $852 = Math_imul($846, $851)|0;
  $863 = $852;
 } else {
  $853 = $row_width;
  $854 = $1;
  $855 = (($854) + 11|0);
  $856 = HEAP8[$855>>0]|0;
  $857 = $856&255;
  $858 = Math_imul($853, $857)|0;
  $859 = (($858) + 7)|0;
  $860 = $859 >>> 3;
  $863 = $860;
 }
 $861 = $1;
 $862 = (($861) + 4|0);
 HEAP32[$862>>2] = $863;
 $864 = $rgb_error;
 STACKTOP = sp;return ($864|0);
}
function _png_do_gray_to_rgb($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp2 = 0, $dp4 = 0, $dp6 = 0, $i = 0, $row_width = 0;
 var $sp = 0, $sp1 = 0, $sp3 = 0, $sp5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $row_width = $3;
 $4 = $0;
 $5 = (($4) + 9|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)>=(8);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = $0;
 $10 = (($9) + 8|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 2;
 $14 = ($13|0)!=(0);
 if ($14) {
  STACKTOP = sp;return;
 }
 $15 = $0;
 $16 = (($15) + 8|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = $0;
  $21 = (($20) + 9|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = ($23|0)==(8);
  if ($24) {
   $25 = $1;
   $26 = $row_width;
   $27 = (($25) + ($26)|0);
   $28 = (($27) + -1|0);
   $sp = $28;
   $29 = $sp;
   $30 = $row_width;
   $31 = $30<<1;
   $32 = (($29) + ($31)|0);
   $dp = $32;
   $i = 0;
   while(1) {
    $33 = $i;
    $34 = $row_width;
    $35 = ($33>>>0)<($34>>>0);
    if (!($35)) {
     break;
    }
    $36 = $sp;
    $37 = HEAP8[$36>>0]|0;
    $38 = $dp;
    $39 = (($38) + -1|0);
    $dp = $39;
    HEAP8[$38>>0] = $37;
    $40 = $sp;
    $41 = HEAP8[$40>>0]|0;
    $42 = $dp;
    $43 = (($42) + -1|0);
    $dp = $43;
    HEAP8[$42>>0] = $41;
    $44 = $sp;
    $45 = (($44) + -1|0);
    $sp = $45;
    $46 = HEAP8[$44>>0]|0;
    $47 = $dp;
    $48 = (($47) + -1|0);
    $dp = $48;
    HEAP8[$47>>0] = $46;
    $49 = $i;
    $50 = (($49) + 1)|0;
    $i = $50;
   }
  } else {
   $51 = $1;
   $52 = $row_width;
   $53 = $52<<1;
   $54 = (($51) + ($53)|0);
   $55 = (($54) + -1|0);
   $sp1 = $55;
   $56 = $sp1;
   $57 = $row_width;
   $58 = $57<<2;
   $59 = (($56) + ($58)|0);
   $dp2 = $59;
   $i = 0;
   while(1) {
    $60 = $i;
    $61 = $row_width;
    $62 = ($60>>>0)<($61>>>0);
    if (!($62)) {
     break;
    }
    $63 = $sp1;
    $64 = HEAP8[$63>>0]|0;
    $65 = $dp2;
    $66 = (($65) + -1|0);
    $dp2 = $66;
    HEAP8[$65>>0] = $64;
    $67 = $sp1;
    $68 = (($67) + -1|0);
    $69 = HEAP8[$68>>0]|0;
    $70 = $dp2;
    $71 = (($70) + -1|0);
    $dp2 = $71;
    HEAP8[$70>>0] = $69;
    $72 = $sp1;
    $73 = HEAP8[$72>>0]|0;
    $74 = $dp2;
    $75 = (($74) + -1|0);
    $dp2 = $75;
    HEAP8[$74>>0] = $73;
    $76 = $sp1;
    $77 = (($76) + -1|0);
    $78 = HEAP8[$77>>0]|0;
    $79 = $dp2;
    $80 = (($79) + -1|0);
    $dp2 = $80;
    HEAP8[$79>>0] = $78;
    $81 = $sp1;
    $82 = (($81) + -1|0);
    $sp1 = $82;
    $83 = HEAP8[$81>>0]|0;
    $84 = $dp2;
    $85 = (($84) + -1|0);
    $dp2 = $85;
    HEAP8[$84>>0] = $83;
    $86 = $sp1;
    $87 = (($86) + -1|0);
    $sp1 = $87;
    $88 = HEAP8[$86>>0]|0;
    $89 = $dp2;
    $90 = (($89) + -1|0);
    $dp2 = $90;
    HEAP8[$89>>0] = $88;
    $91 = $i;
    $92 = (($91) + 1)|0;
    $i = $92;
   }
  }
 } else {
  $93 = $0;
  $94 = (($93) + 8|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $97 = ($96|0)==(4);
  if ($97) {
   $98 = $0;
   $99 = (($98) + 9|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = ($101|0)==(8);
   if ($102) {
    $103 = $1;
    $104 = $row_width;
    $105 = $104<<1;
    $106 = (($103) + ($105)|0);
    $107 = (($106) + -1|0);
    $sp3 = $107;
    $108 = $sp3;
    $109 = $row_width;
    $110 = $109<<1;
    $111 = (($108) + ($110)|0);
    $dp4 = $111;
    $i = 0;
    while(1) {
     $112 = $i;
     $113 = $row_width;
     $114 = ($112>>>0)<($113>>>0);
     if (!($114)) {
      break;
     }
     $115 = $sp3;
     $116 = (($115) + -1|0);
     $sp3 = $116;
     $117 = HEAP8[$115>>0]|0;
     $118 = $dp4;
     $119 = (($118) + -1|0);
     $dp4 = $119;
     HEAP8[$118>>0] = $117;
     $120 = $sp3;
     $121 = HEAP8[$120>>0]|0;
     $122 = $dp4;
     $123 = (($122) + -1|0);
     $dp4 = $123;
     HEAP8[$122>>0] = $121;
     $124 = $sp3;
     $125 = HEAP8[$124>>0]|0;
     $126 = $dp4;
     $127 = (($126) + -1|0);
     $dp4 = $127;
     HEAP8[$126>>0] = $125;
     $128 = $sp3;
     $129 = (($128) + -1|0);
     $sp3 = $129;
     $130 = HEAP8[$128>>0]|0;
     $131 = $dp4;
     $132 = (($131) + -1|0);
     $dp4 = $132;
     HEAP8[$131>>0] = $130;
     $133 = $i;
     $134 = (($133) + 1)|0;
     $i = $134;
    }
   } else {
    $135 = $1;
    $136 = $row_width;
    $137 = $136<<2;
    $138 = (($135) + ($137)|0);
    $139 = (($138) + -1|0);
    $sp5 = $139;
    $140 = $sp5;
    $141 = $row_width;
    $142 = $141<<2;
    $143 = (($140) + ($142)|0);
    $dp6 = $143;
    $i = 0;
    while(1) {
     $144 = $i;
     $145 = $row_width;
     $146 = ($144>>>0)<($145>>>0);
     if (!($146)) {
      break;
     }
     $147 = $sp5;
     $148 = (($147) + -1|0);
     $sp5 = $148;
     $149 = HEAP8[$147>>0]|0;
     $150 = $dp6;
     $151 = (($150) + -1|0);
     $dp6 = $151;
     HEAP8[$150>>0] = $149;
     $152 = $sp5;
     $153 = (($152) + -1|0);
     $sp5 = $153;
     $154 = HEAP8[$152>>0]|0;
     $155 = $dp6;
     $156 = (($155) + -1|0);
     $dp6 = $156;
     HEAP8[$155>>0] = $154;
     $157 = $sp5;
     $158 = HEAP8[$157>>0]|0;
     $159 = $dp6;
     $160 = (($159) + -1|0);
     $dp6 = $160;
     HEAP8[$159>>0] = $158;
     $161 = $sp5;
     $162 = (($161) + -1|0);
     $163 = HEAP8[$162>>0]|0;
     $164 = $dp6;
     $165 = (($164) + -1|0);
     $dp6 = $165;
     HEAP8[$164>>0] = $163;
     $166 = $sp5;
     $167 = HEAP8[$166>>0]|0;
     $168 = $dp6;
     $169 = (($168) + -1|0);
     $dp6 = $169;
     HEAP8[$168>>0] = $167;
     $170 = $sp5;
     $171 = (($170) + -1|0);
     $172 = HEAP8[$171>>0]|0;
     $173 = $dp6;
     $174 = (($173) + -1|0);
     $dp6 = $174;
     HEAP8[$173>>0] = $172;
     $175 = $sp5;
     $176 = (($175) + -1|0);
     $sp5 = $176;
     $177 = HEAP8[$175>>0]|0;
     $178 = $dp6;
     $179 = (($178) + -1|0);
     $dp6 = $179;
     HEAP8[$178>>0] = $177;
     $180 = $sp5;
     $181 = (($180) + -1|0);
     $sp5 = $181;
     $182 = HEAP8[$180>>0]|0;
     $183 = $dp6;
     $184 = (($183) + -1|0);
     $dp6 = $184;
     HEAP8[$183>>0] = $182;
     $185 = $i;
     $186 = (($185) + 1)|0;
     $i = $186;
    }
   }
  }
 }
 $187 = $0;
 $188 = (($187) + 10|0);
 $189 = HEAP8[$188>>0]|0;
 $190 = $189&255;
 $191 = (($190) + 2)|0;
 $192 = $191&255;
 HEAP8[$188>>0] = $192;
 $193 = $0;
 $194 = (($193) + 8|0);
 $195 = HEAP8[$194>>0]|0;
 $196 = $195&255;
 $197 = $196 | 2;
 $198 = $197&255;
 HEAP8[$194>>0] = $198;
 $199 = $0;
 $200 = (($199) + 10|0);
 $201 = HEAP8[$200>>0]|0;
 $202 = $201&255;
 $203 = $0;
 $204 = (($203) + 9|0);
 $205 = HEAP8[$204>>0]|0;
 $206 = $205&255;
 $207 = Math_imul($202, $206)|0;
 $208 = $207&255;
 $209 = $0;
 $210 = (($209) + 11|0);
 HEAP8[$210>>0] = $208;
 $211 = $0;
 $212 = (($211) + 11|0);
 $213 = HEAP8[$212>>0]|0;
 $214 = $213&255;
 $215 = ($214|0)>=(8);
 if ($215) {
  $216 = $row_width;
  $217 = $0;
  $218 = (($217) + 11|0);
  $219 = HEAP8[$218>>0]|0;
  $220 = $219&255;
  $221 = $220 >>> 3;
  $222 = Math_imul($216, $221)|0;
  $233 = $222;
 } else {
  $223 = $row_width;
  $224 = $0;
  $225 = (($224) + 11|0);
  $226 = HEAP8[$225>>0]|0;
  $227 = $226&255;
  $228 = Math_imul($223, $227)|0;
  $229 = (($228) + 7)|0;
  $230 = $229 >>> 3;
  $233 = $230;
 }
 $231 = $0;
 $232 = (($231) + 4|0);
 HEAP32[$232>>2] = $233;
 STACKTOP = sp;return;
}
function _png_do_background($row_info,$row,$trans_values,$background,$background_1,$gamma_table,$gamma_from_1,$gamma_to_1,$gamma_16,$gamma_16_from_1,$gamma_16_to_1,$gamma_shift) {
 $row_info = $row_info|0;
 $row = $row|0;
 $trans_values = $trans_values|0;
 $background = $background|0;
 $background_1 = $background_1|0;
 $gamma_table = $gamma_table|0;
 $gamma_from_1 = $gamma_from_1|0;
 $gamma_to_1 = $gamma_to_1|0;
 $gamma_16 = $gamma_16|0;
 $gamma_16_from_1 = $gamma_16_from_1|0;
 $gamma_16_to_1 = $gamma_16_to_1|0;
 $gamma_shift = $gamma_shift|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0, $1988 = 0;
 var $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0, $2004 = 0;
 var $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0, $2022 = 0;
 var $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0, $2040 = 0;
 var $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0, $2058 = 0, $2059 = 0;
 var $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0, $2076 = 0, $2077 = 0;
 var $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0, $2094 = 0, $2095 = 0;
 var $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0, $2111 = 0, $2112 = 0;
 var $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0, $213 = 0, $2130 = 0;
 var $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0, $2148 = 0, $2149 = 0;
 var $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0, $2166 = 0, $2167 = 0;
 var $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0, $2184 = 0, $2185 = 0;
 var $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0, $2201 = 0, $2202 = 0;
 var $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0, $222 = 0, $2220 = 0;
 var $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0, $2238 = 0, $2239 = 0;
 var $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0, $2253 = 0, $2254 = 0, $2255 = 0, $2256 = 0, $2257 = 0;
 var $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0, $2274 = 0, $2275 = 0;
 var $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0, $2292 = 0, $2293 = 0;
 var $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0, $231 = 0, $2310 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0;
 var $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0;
 var $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0;
 var $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0;
 var $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0;
 var $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0;
 var $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0;
 var $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0;
 var $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0;
 var $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0;
 var $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0;
 var $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0;
 var $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0;
 var $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0;
 var $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0;
 var $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0;
 var $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0;
 var $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0;
 var $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0;
 var $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0;
 var $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0;
 var $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0;
 var $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0;
 var $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0;
 var $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0;
 var $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0;
 var $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0;
 var $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0;
 var $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0;
 var $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0;
 var $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0;
 var $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0;
 var $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0;
 var $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0;
 var $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0;
 var $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0;
 var $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0;
 var $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $a = 0, $a10 = 0, $a12 = 0, $a18 = 0, $a22 = 0, $a28 = 0, $a32 = 0, $a39 = 0, $b = 0;
 var $b43 = 0, $b8 = 0, $dp = 0, $g = 0, $g14 = 0, $g19 = 0, $g2 = 0, $g4 = 0, $g42 = 0, $g7 = 0, $i = 0, $p = 0, $p1 = 0, $r = 0, $r41 = 0, $r6 = 0, $row_width = 0, $shift = 0, $sp = 0, $temp = 0;
 var $temp11 = 0, $temp17 = 0, $temp21 = 0, $temp25 = 0, $temp26 = 0, $temp27 = 0, $temp29 = 0, $temp30 = 0, $temp31 = 0, $temp36 = 0, $temp37 = 0, $temp38 = 0, $temp44 = 0, $temp45 = 0, $temp46 = 0, $v = 0, $v13 = 0, $v15 = 0, $v20 = 0, $v23 = 0;
 var $v3 = 0, $v33 = 0, $v34 = 0, $v40 = 0, $v5 = 0, $v9 = 0, $w = 0, $w16 = 0, $w24 = 0, $w35 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $trans_values;
 $3 = $background;
 $4 = $background_1;
 $5 = $gamma_table;
 $6 = $gamma_from_1;
 $7 = $gamma_to_1;
 $8 = $gamma_16;
 $9 = $gamma_16_from_1;
 $10 = $gamma_16_to_1;
 $11 = $gamma_shift;
 $12 = $0;
 $13 = HEAP32[$12>>2]|0;
 $row_width = $13;
 $14 = $3;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  STACKTOP = sp;return;
 }
 $16 = $0;
 $17 = (($16) + 8|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = $19 & 4;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = $0;
  $23 = (($22) + 8|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)!=(3);
  if (!($26)) {
   STACKTOP = sp;return;
  }
  $27 = $2;
  $28 = ($27|0)!=(0|0);
  if (!($28)) {
   STACKTOP = sp;return;
  }
 }
 $29 = $0;
 $30 = (($29) + 8|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 if ((($32|0) == 0)) {
  $33 = $0;
  $34 = (($33) + 9|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  switch ($36|0) {
  case 1:  {
   $37 = $1;
   $sp = $37;
   $shift = 7;
   $i = 0;
   while(1) {
    $38 = $i;
    $39 = $row_width;
    $40 = ($38>>>0)<($39>>>0);
    if (!($40)) {
     break;
    }
    $41 = $sp;
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $shift;
    $45 = $43 >> $44;
    $46 = $45 & 1;
    $47 = $46&65535;
    $48 = $47&65535;
    $49 = $2;
    $50 = (($49) + 8|0);
    $51 = HEAP16[$50>>1]|0;
    $52 = $51&65535;
    $53 = ($48|0)==($52|0);
    if ($53) {
     $54 = $shift;
     $55 = (7 - ($54))|0;
     $56 = 32639 >> $55;
     $57 = $56 & 255;
     $58 = $57&255;
     $59 = $58&255;
     $60 = $sp;
     $61 = HEAP8[$60>>0]|0;
     $62 = $61&255;
     $63 = $62 & $59;
     $64 = $63&255;
     HEAP8[$60>>0] = $64;
     $65 = $3;
     $66 = (($65) + 8|0);
     $67 = HEAP16[$66>>1]|0;
     $68 = $67&65535;
     $69 = $shift;
     $70 = $68 << $69;
     $71 = $70&255;
     $72 = $71&255;
     $73 = $sp;
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     $76 = $75 | $72;
     $77 = $76&255;
     HEAP8[$73>>0] = $77;
    }
    $78 = $shift;
    $79 = ($78|0)!=(0);
    if ($79) {
     $82 = $shift;
     $83 = (($82) + -1)|0;
     $shift = $83;
    } else {
     $shift = 7;
     $80 = $sp;
     $81 = (($80) + 1|0);
     $sp = $81;
    }
    $84 = $i;
    $85 = (($84) + 1)|0;
    $i = $85;
   }
   break;
  }
  case 4:  {
   $236 = $5;
   $237 = ($236|0)!=(0|0);
   if ($237) {
    $238 = $1;
    $sp = $238;
    $shift = 4;
    $i = 0;
    while(1) {
     $239 = $i;
     $240 = $row_width;
     $241 = ($239>>>0)<($240>>>0);
     if (!($241)) {
      break;
     }
     $242 = $sp;
     $243 = HEAP8[$242>>0]|0;
     $244 = $243&255;
     $245 = $shift;
     $246 = $244 >> $245;
     $247 = $246 & 15;
     $248 = $247&65535;
     $249 = $248&65535;
     $250 = $2;
     $251 = (($250) + 8|0);
     $252 = HEAP16[$251>>1]|0;
     $253 = $252&65535;
     $254 = ($249|0)==($253|0);
     if ($254) {
      $255 = $shift;
      $256 = (4 - ($255))|0;
      $257 = 3855 >> $256;
      $258 = $257 & 255;
      $259 = $258&255;
      $260 = $259&255;
      $261 = $sp;
      $262 = HEAP8[$261>>0]|0;
      $263 = $262&255;
      $264 = $263 & $260;
      $265 = $264&255;
      HEAP8[$261>>0] = $265;
      $266 = $3;
      $267 = (($266) + 8|0);
      $268 = HEAP16[$267>>1]|0;
      $269 = $268&65535;
      $270 = $shift;
      $271 = $269 << $270;
      $272 = $271&255;
      $273 = $272&255;
      $274 = $sp;
      $275 = HEAP8[$274>>0]|0;
      $276 = $275&255;
      $277 = $276 | $273;
      $278 = $277&255;
      HEAP8[$274>>0] = $278;
     } else {
      $279 = $sp;
      $280 = HEAP8[$279>>0]|0;
      $281 = $280&255;
      $282 = $shift;
      $283 = $281 >> $282;
      $284 = $283 & 15;
      $285 = $284&255;
      $p1 = $285;
      $286 = $p1;
      $287 = $286&255;
      $288 = $p1;
      $289 = $288&255;
      $290 = $289 << 4;
      $291 = $287 | $290;
      $292 = $5;
      $293 = (($292) + ($291)|0);
      $294 = HEAP8[$293>>0]|0;
      $295 = $294&255;
      $296 = $295 >> 4;
      $297 = $296 & 15;
      $298 = $297&255;
      $g2 = $298;
      $299 = $shift;
      $300 = (4 - ($299))|0;
      $301 = 3855 >> $300;
      $302 = $301 & 255;
      $303 = $302&255;
      $304 = $303&255;
      $305 = $sp;
      $306 = HEAP8[$305>>0]|0;
      $307 = $306&255;
      $308 = $307 & $304;
      $309 = $308&255;
      HEAP8[$305>>0] = $309;
      $310 = $g2;
      $311 = $310&255;
      $312 = $shift;
      $313 = $311 << $312;
      $314 = $313&255;
      $315 = $314&255;
      $316 = $sp;
      $317 = HEAP8[$316>>0]|0;
      $318 = $317&255;
      $319 = $318 | $315;
      $320 = $319&255;
      HEAP8[$316>>0] = $320;
     }
     $321 = $shift;
     $322 = ($321|0)!=(0);
     if ($322) {
      $325 = $shift;
      $326 = (($325) - 4)|0;
      $shift = $326;
     } else {
      $shift = 4;
      $323 = $sp;
      $324 = (($323) + 1|0);
      $sp = $324;
     }
     $327 = $i;
     $328 = (($327) + 1)|0;
     $i = $328;
    }
   } else {
    $329 = $1;
    $sp = $329;
    $shift = 4;
    $i = 0;
    while(1) {
     $330 = $i;
     $331 = $row_width;
     $332 = ($330>>>0)<($331>>>0);
     if (!($332)) {
      break;
     }
     $333 = $sp;
     $334 = HEAP8[$333>>0]|0;
     $335 = $334&255;
     $336 = $shift;
     $337 = $335 >> $336;
     $338 = $337 & 15;
     $339 = $338&65535;
     $340 = $339&65535;
     $341 = $2;
     $342 = (($341) + 8|0);
     $343 = HEAP16[$342>>1]|0;
     $344 = $343&65535;
     $345 = ($340|0)==($344|0);
     if ($345) {
      $346 = $shift;
      $347 = (4 - ($346))|0;
      $348 = 3855 >> $347;
      $349 = $348 & 255;
      $350 = $349&255;
      $351 = $350&255;
      $352 = $sp;
      $353 = HEAP8[$352>>0]|0;
      $354 = $353&255;
      $355 = $354 & $351;
      $356 = $355&255;
      HEAP8[$352>>0] = $356;
      $357 = $3;
      $358 = (($357) + 8|0);
      $359 = HEAP16[$358>>1]|0;
      $360 = $359&65535;
      $361 = $shift;
      $362 = $360 << $361;
      $363 = $362&255;
      $364 = $363&255;
      $365 = $sp;
      $366 = HEAP8[$365>>0]|0;
      $367 = $366&255;
      $368 = $367 | $364;
      $369 = $368&255;
      HEAP8[$365>>0] = $369;
     }
     $370 = $shift;
     $371 = ($370|0)!=(0);
     if ($371) {
      $374 = $shift;
      $375 = (($374) - 4)|0;
      $shift = $375;
     } else {
      $shift = 4;
      $372 = $sp;
      $373 = (($372) + 1|0);
      $sp = $373;
     }
     $376 = $i;
     $377 = (($376) + 1)|0;
     $i = $377;
    }
   }
   break;
  }
  case 8:  {
   $378 = $5;
   $379 = ($378|0)!=(0|0);
   if ($379) {
    $380 = $1;
    $sp = $380;
    $i = 0;
    while(1) {
     $381 = $i;
     $382 = $row_width;
     $383 = ($381>>>0)<($382>>>0);
     if (!($383)) {
      break;
     }
     $384 = $sp;
     $385 = HEAP8[$384>>0]|0;
     $386 = $385&255;
     $387 = $2;
     $388 = (($387) + 8|0);
     $389 = HEAP16[$388>>1]|0;
     $390 = $389&65535;
     $391 = ($386|0)==($390|0);
     if ($391) {
      $392 = $3;
      $393 = (($392) + 8|0);
      $394 = HEAP16[$393>>1]|0;
      $395 = $394&255;
      $396 = $sp;
      HEAP8[$396>>0] = $395;
     } else {
      $397 = $sp;
      $398 = HEAP8[$397>>0]|0;
      $399 = $398&255;
      $400 = $5;
      $401 = (($400) + ($399)|0);
      $402 = HEAP8[$401>>0]|0;
      $403 = $sp;
      HEAP8[$403>>0] = $402;
     }
     $404 = $i;
     $405 = (($404) + 1)|0;
     $i = $405;
     $406 = $sp;
     $407 = (($406) + 1|0);
     $sp = $407;
    }
   } else {
    $408 = $1;
    $sp = $408;
    $i = 0;
    while(1) {
     $409 = $i;
     $410 = $row_width;
     $411 = ($409>>>0)<($410>>>0);
     if (!($411)) {
      break;
     }
     $412 = $sp;
     $413 = HEAP8[$412>>0]|0;
     $414 = $413&255;
     $415 = $2;
     $416 = (($415) + 8|0);
     $417 = HEAP16[$416>>1]|0;
     $418 = $417&65535;
     $419 = ($414|0)==($418|0);
     if ($419) {
      $420 = $3;
      $421 = (($420) + 8|0);
      $422 = HEAP16[$421>>1]|0;
      $423 = $422&255;
      $424 = $sp;
      HEAP8[$424>>0] = $423;
     }
     $425 = $i;
     $426 = (($425) + 1)|0;
     $i = $426;
     $427 = $sp;
     $428 = (($427) + 1|0);
     $sp = $428;
    }
   }
   break;
  }
  case 16:  {
   $429 = $8;
   $430 = ($429|0)!=(0|0);
   if ($430) {
    $431 = $1;
    $sp = $431;
    $i = 0;
    while(1) {
     $432 = $i;
     $433 = $row_width;
     $434 = ($432>>>0)<($433>>>0);
     if (!($434)) {
      break;
     }
     $435 = $sp;
     $436 = HEAP8[$435>>0]|0;
     $437 = $436&255;
     $438 = $437 << 8;
     $439 = $sp;
     $440 = (($439) + 1|0);
     $441 = HEAP8[$440>>0]|0;
     $442 = $441&255;
     $443 = (($438) + ($442))|0;
     $444 = $443&65535;
     $v = $444;
     $445 = $v;
     $446 = $445&65535;
     $447 = $2;
     $448 = (($447) + 8|0);
     $449 = HEAP16[$448>>1]|0;
     $450 = $449&65535;
     $451 = ($446|0)==($450|0);
     if ($451) {
      $452 = $3;
      $453 = (($452) + 8|0);
      $454 = HEAP16[$453>>1]|0;
      $455 = $454&65535;
      $456 = $455 >> 8;
      $457 = $456 & 255;
      $458 = $457&255;
      $459 = $sp;
      HEAP8[$459>>0] = $458;
      $460 = $3;
      $461 = (($460) + 8|0);
      $462 = HEAP16[$461>>1]|0;
      $463 = $462&65535;
      $464 = $463 & 255;
      $465 = $464&255;
      $466 = $sp;
      $467 = (($466) + 1|0);
      HEAP8[$467>>0] = $465;
     } else {
      $468 = $sp;
      $469 = HEAP8[$468>>0]|0;
      $470 = $469&255;
      $471 = $sp;
      $472 = (($471) + 1|0);
      $473 = HEAP8[$472>>0]|0;
      $474 = $473&255;
      $475 = $11;
      $476 = $474 >> $475;
      $477 = $8;
      $478 = (($477) + ($476<<2)|0);
      $479 = HEAP32[$478>>2]|0;
      $480 = (($479) + ($470<<1)|0);
      $481 = HEAP16[$480>>1]|0;
      $v = $481;
      $482 = $v;
      $483 = $482&65535;
      $484 = $483 >> 8;
      $485 = $484 & 255;
      $486 = $485&255;
      $487 = $sp;
      HEAP8[$487>>0] = $486;
      $488 = $v;
      $489 = $488&65535;
      $490 = $489 & 255;
      $491 = $490&255;
      $492 = $sp;
      $493 = (($492) + 1|0);
      HEAP8[$493>>0] = $491;
     }
     $494 = $i;
     $495 = (($494) + 1)|0;
     $i = $495;
     $496 = $sp;
     $497 = (($496) + 2|0);
     $sp = $497;
    }
   } else {
    $498 = $1;
    $sp = $498;
    $i = 0;
    while(1) {
     $499 = $i;
     $500 = $row_width;
     $501 = ($499>>>0)<($500>>>0);
     if (!($501)) {
      break;
     }
     $502 = $sp;
     $503 = HEAP8[$502>>0]|0;
     $504 = $503&255;
     $505 = $504 << 8;
     $506 = $sp;
     $507 = (($506) + 1|0);
     $508 = HEAP8[$507>>0]|0;
     $509 = $508&255;
     $510 = (($505) + ($509))|0;
     $511 = $510&65535;
     $v3 = $511;
     $512 = $v3;
     $513 = $512&65535;
     $514 = $2;
     $515 = (($514) + 8|0);
     $516 = HEAP16[$515>>1]|0;
     $517 = $516&65535;
     $518 = ($513|0)==($517|0);
     if ($518) {
      $519 = $3;
      $520 = (($519) + 8|0);
      $521 = HEAP16[$520>>1]|0;
      $522 = $521&65535;
      $523 = $522 >> 8;
      $524 = $523 & 255;
      $525 = $524&255;
      $526 = $sp;
      HEAP8[$526>>0] = $525;
      $527 = $3;
      $528 = (($527) + 8|0);
      $529 = HEAP16[$528>>1]|0;
      $530 = $529&65535;
      $531 = $530 & 255;
      $532 = $531&255;
      $533 = $sp;
      $534 = (($533) + 1|0);
      HEAP8[$534>>0] = $532;
     }
     $535 = $i;
     $536 = (($535) + 1)|0;
     $i = $536;
     $537 = $sp;
     $538 = (($537) + 2|0);
     $sp = $538;
    }
   }
   break;
  }
  case 2:  {
   $86 = $5;
   $87 = ($86|0)!=(0|0);
   if ($87) {
    $88 = $1;
    $sp = $88;
    $shift = 6;
    $i = 0;
    while(1) {
     $89 = $i;
     $90 = $row_width;
     $91 = ($89>>>0)<($90>>>0);
     if (!($91)) {
      break;
     }
     $92 = $sp;
     $93 = HEAP8[$92>>0]|0;
     $94 = $93&255;
     $95 = $shift;
     $96 = $94 >> $95;
     $97 = $96 & 3;
     $98 = $97&65535;
     $99 = $98&65535;
     $100 = $2;
     $101 = (($100) + 8|0);
     $102 = HEAP16[$101>>1]|0;
     $103 = $102&65535;
     $104 = ($99|0)==($103|0);
     if ($104) {
      $105 = $shift;
      $106 = (6 - ($105))|0;
      $107 = 16191 >> $106;
      $108 = $107 & 255;
      $109 = $108&255;
      $110 = $109&255;
      $111 = $sp;
      $112 = HEAP8[$111>>0]|0;
      $113 = $112&255;
      $114 = $113 & $110;
      $115 = $114&255;
      HEAP8[$111>>0] = $115;
      $116 = $3;
      $117 = (($116) + 8|0);
      $118 = HEAP16[$117>>1]|0;
      $119 = $118&65535;
      $120 = $shift;
      $121 = $119 << $120;
      $122 = $121&255;
      $123 = $122&255;
      $124 = $sp;
      $125 = HEAP8[$124>>0]|0;
      $126 = $125&255;
      $127 = $126 | $123;
      $128 = $127&255;
      HEAP8[$124>>0] = $128;
     } else {
      $129 = $sp;
      $130 = HEAP8[$129>>0]|0;
      $131 = $130&255;
      $132 = $shift;
      $133 = $131 >> $132;
      $134 = $133 & 3;
      $135 = $134&255;
      $p = $135;
      $136 = $p;
      $137 = $136&255;
      $138 = $p;
      $139 = $138&255;
      $140 = $139 << 2;
      $141 = $137 | $140;
      $142 = $p;
      $143 = $142&255;
      $144 = $143 << 4;
      $145 = $141 | $144;
      $146 = $p;
      $147 = $146&255;
      $148 = $147 << 6;
      $149 = $145 | $148;
      $150 = $5;
      $151 = (($150) + ($149)|0);
      $152 = HEAP8[$151>>0]|0;
      $153 = $152&255;
      $154 = $153 >> 6;
      $155 = $154 & 3;
      $156 = $155&255;
      $g = $156;
      $157 = $shift;
      $158 = (6 - ($157))|0;
      $159 = 16191 >> $158;
      $160 = $159 & 255;
      $161 = $160&255;
      $162 = $161&255;
      $163 = $sp;
      $164 = HEAP8[$163>>0]|0;
      $165 = $164&255;
      $166 = $165 & $162;
      $167 = $166&255;
      HEAP8[$163>>0] = $167;
      $168 = $g;
      $169 = $168&255;
      $170 = $shift;
      $171 = $169 << $170;
      $172 = $171&255;
      $173 = $172&255;
      $174 = $sp;
      $175 = HEAP8[$174>>0]|0;
      $176 = $175&255;
      $177 = $176 | $173;
      $178 = $177&255;
      HEAP8[$174>>0] = $178;
     }
     $179 = $shift;
     $180 = ($179|0)!=(0);
     if ($180) {
      $183 = $shift;
      $184 = (($183) - 2)|0;
      $shift = $184;
     } else {
      $shift = 6;
      $181 = $sp;
      $182 = (($181) + 1|0);
      $sp = $182;
     }
     $185 = $i;
     $186 = (($185) + 1)|0;
     $i = $186;
    }
   } else {
    $187 = $1;
    $sp = $187;
    $shift = 6;
    $i = 0;
    while(1) {
     $188 = $i;
     $189 = $row_width;
     $190 = ($188>>>0)<($189>>>0);
     if (!($190)) {
      break;
     }
     $191 = $sp;
     $192 = HEAP8[$191>>0]|0;
     $193 = $192&255;
     $194 = $shift;
     $195 = $193 >> $194;
     $196 = $195 & 3;
     $197 = $196&65535;
     $198 = $197&65535;
     $199 = $2;
     $200 = (($199) + 8|0);
     $201 = HEAP16[$200>>1]|0;
     $202 = $201&65535;
     $203 = ($198|0)==($202|0);
     if ($203) {
      $204 = $shift;
      $205 = (6 - ($204))|0;
      $206 = 16191 >> $205;
      $207 = $206 & 255;
      $208 = $207&255;
      $209 = $208&255;
      $210 = $sp;
      $211 = HEAP8[$210>>0]|0;
      $212 = $211&255;
      $213 = $212 & $209;
      $214 = $213&255;
      HEAP8[$210>>0] = $214;
      $215 = $3;
      $216 = (($215) + 8|0);
      $217 = HEAP16[$216>>1]|0;
      $218 = $217&65535;
      $219 = $shift;
      $220 = $218 << $219;
      $221 = $220&255;
      $222 = $221&255;
      $223 = $sp;
      $224 = HEAP8[$223>>0]|0;
      $225 = $224&255;
      $226 = $225 | $222;
      $227 = $226&255;
      HEAP8[$223>>0] = $227;
     }
     $228 = $shift;
     $229 = ($228|0)!=(0);
     if ($229) {
      $232 = $shift;
      $233 = (($232) - 2)|0;
      $shift = $233;
     } else {
      $shift = 6;
      $230 = $sp;
      $231 = (($230) + 1|0);
      $sp = $231;
     }
     $234 = $i;
     $235 = (($234) + 1)|0;
     $i = $235;
    }
   }
   break;
  }
  default: {
  }
  }
 } else if ((($32|0) == 2)) {
  $539 = $0;
  $540 = (($539) + 9|0);
  $541 = HEAP8[$540>>0]|0;
  $542 = $541&255;
  $543 = ($542|0)==(8);
  if ($543) {
   $544 = $5;
   $545 = ($544|0)!=(0|0);
   if ($545) {
    $546 = $1;
    $sp = $546;
    $i = 0;
    while(1) {
     $547 = $i;
     $548 = $row_width;
     $549 = ($547>>>0)<($548>>>0);
     if (!($549)) {
      break;
     }
     $550 = $sp;
     $551 = HEAP8[$550>>0]|0;
     $552 = $551&255;
     $553 = $2;
     $554 = (($553) + 2|0);
     $555 = HEAP16[$554>>1]|0;
     $556 = $555&65535;
     $557 = ($552|0)==($556|0);
     if ($557) {
      $558 = $sp;
      $559 = (($558) + 1|0);
      $560 = HEAP8[$559>>0]|0;
      $561 = $560&255;
      $562 = $2;
      $563 = (($562) + 4|0);
      $564 = HEAP16[$563>>1]|0;
      $565 = $564&65535;
      $566 = ($561|0)==($565|0);
      if ($566) {
       $567 = $sp;
       $568 = (($567) + 2|0);
       $569 = HEAP8[$568>>0]|0;
       $570 = $569&255;
       $571 = $2;
       $572 = (($571) + 6|0);
       $573 = HEAP16[$572>>1]|0;
       $574 = $573&65535;
       $575 = ($570|0)==($574|0);
       if ($575) {
        $576 = $3;
        $577 = (($576) + 2|0);
        $578 = HEAP16[$577>>1]|0;
        $579 = $578&255;
        $580 = $sp;
        HEAP8[$580>>0] = $579;
        $581 = $3;
        $582 = (($581) + 4|0);
        $583 = HEAP16[$582>>1]|0;
        $584 = $583&255;
        $585 = $sp;
        $586 = (($585) + 1|0);
        HEAP8[$586>>0] = $584;
        $587 = $3;
        $588 = (($587) + 6|0);
        $589 = HEAP16[$588>>1]|0;
        $590 = $589&255;
        $591 = $sp;
        $592 = (($591) + 2|0);
        HEAP8[$592>>0] = $590;
       } else {
        label = 106;
       }
      } else {
       label = 106;
      }
     } else {
      label = 106;
     }
     if ((label|0) == 106) {
      label = 0;
      $593 = $sp;
      $594 = HEAP8[$593>>0]|0;
      $595 = $594&255;
      $596 = $5;
      $597 = (($596) + ($595)|0);
      $598 = HEAP8[$597>>0]|0;
      $599 = $sp;
      HEAP8[$599>>0] = $598;
      $600 = $sp;
      $601 = (($600) + 1|0);
      $602 = HEAP8[$601>>0]|0;
      $603 = $602&255;
      $604 = $5;
      $605 = (($604) + ($603)|0);
      $606 = HEAP8[$605>>0]|0;
      $607 = $sp;
      $608 = (($607) + 1|0);
      HEAP8[$608>>0] = $606;
      $609 = $sp;
      $610 = (($609) + 2|0);
      $611 = HEAP8[$610>>0]|0;
      $612 = $611&255;
      $613 = $5;
      $614 = (($613) + ($612)|0);
      $615 = HEAP8[$614>>0]|0;
      $616 = $sp;
      $617 = (($616) + 2|0);
      HEAP8[$617>>0] = $615;
     }
     $618 = $i;
     $619 = (($618) + 1)|0;
     $i = $619;
     $620 = $sp;
     $621 = (($620) + 3|0);
     $sp = $621;
    }
   } else {
    $622 = $1;
    $sp = $622;
    $i = 0;
    while(1) {
     $623 = $i;
     $624 = $row_width;
     $625 = ($623>>>0)<($624>>>0);
     if (!($625)) {
      break;
     }
     $626 = $sp;
     $627 = HEAP8[$626>>0]|0;
     $628 = $627&255;
     $629 = $2;
     $630 = (($629) + 2|0);
     $631 = HEAP16[$630>>1]|0;
     $632 = $631&65535;
     $633 = ($628|0)==($632|0);
     if ($633) {
      $634 = $sp;
      $635 = (($634) + 1|0);
      $636 = HEAP8[$635>>0]|0;
      $637 = $636&255;
      $638 = $2;
      $639 = (($638) + 4|0);
      $640 = HEAP16[$639>>1]|0;
      $641 = $640&65535;
      $642 = ($637|0)==($641|0);
      if ($642) {
       $643 = $sp;
       $644 = (($643) + 2|0);
       $645 = HEAP8[$644>>0]|0;
       $646 = $645&255;
       $647 = $2;
       $648 = (($647) + 6|0);
       $649 = HEAP16[$648>>1]|0;
       $650 = $649&65535;
       $651 = ($646|0)==($650|0);
       if ($651) {
        $652 = $3;
        $653 = (($652) + 2|0);
        $654 = HEAP16[$653>>1]|0;
        $655 = $654&255;
        $656 = $sp;
        HEAP8[$656>>0] = $655;
        $657 = $3;
        $658 = (($657) + 4|0);
        $659 = HEAP16[$658>>1]|0;
        $660 = $659&255;
        $661 = $sp;
        $662 = (($661) + 1|0);
        HEAP8[$662>>0] = $660;
        $663 = $3;
        $664 = (($663) + 6|0);
        $665 = HEAP16[$664>>1]|0;
        $666 = $665&255;
        $667 = $sp;
        $668 = (($667) + 2|0);
        HEAP8[$668>>0] = $666;
       }
      }
     }
     $669 = $i;
     $670 = (($669) + 1)|0;
     $i = $670;
     $671 = $sp;
     $672 = (($671) + 3|0);
     $sp = $672;
    }
   }
  } else {
   $673 = $8;
   $674 = ($673|0)!=(0|0);
   if ($674) {
    $675 = $1;
    $sp = $675;
    $i = 0;
    while(1) {
     $676 = $i;
     $677 = $row_width;
     $678 = ($676>>>0)<($677>>>0);
     if (!($678)) {
      break;
     }
     $679 = $sp;
     $680 = HEAP8[$679>>0]|0;
     $681 = $680&255;
     $682 = $681 << 8;
     $683 = $sp;
     $684 = (($683) + 1|0);
     $685 = HEAP8[$684>>0]|0;
     $686 = $685&255;
     $687 = (($682) + ($686))|0;
     $688 = $687&65535;
     $r = $688;
     $689 = $sp;
     $690 = (($689) + 2|0);
     $691 = HEAP8[$690>>0]|0;
     $692 = $691&255;
     $693 = $692 << 8;
     $694 = $sp;
     $695 = (($694) + 3|0);
     $696 = HEAP8[$695>>0]|0;
     $697 = $696&255;
     $698 = (($693) + ($697))|0;
     $699 = $698&65535;
     $g4 = $699;
     $700 = $sp;
     $701 = (($700) + 4|0);
     $702 = HEAP8[$701>>0]|0;
     $703 = $702&255;
     $704 = $703 << 8;
     $705 = $sp;
     $706 = (($705) + 5|0);
     $707 = HEAP8[$706>>0]|0;
     $708 = $707&255;
     $709 = (($704) + ($708))|0;
     $710 = $709&65535;
     $b = $710;
     $711 = $r;
     $712 = $711&65535;
     $713 = $2;
     $714 = (($713) + 2|0);
     $715 = HEAP16[$714>>1]|0;
     $716 = $715&65535;
     $717 = ($712|0)==($716|0);
     if ($717) {
      $718 = $g4;
      $719 = $718&65535;
      $720 = $2;
      $721 = (($720) + 4|0);
      $722 = HEAP16[$721>>1]|0;
      $723 = $722&65535;
      $724 = ($719|0)==($723|0);
      if ($724) {
       $725 = $b;
       $726 = $725&65535;
       $727 = $2;
       $728 = (($727) + 6|0);
       $729 = HEAP16[$728>>1]|0;
       $730 = $729&65535;
       $731 = ($726|0)==($730|0);
       if ($731) {
        $732 = $3;
        $733 = (($732) + 2|0);
        $734 = HEAP16[$733>>1]|0;
        $735 = $734&65535;
        $736 = $735 >> 8;
        $737 = $736 & 255;
        $738 = $737&255;
        $739 = $sp;
        HEAP8[$739>>0] = $738;
        $740 = $3;
        $741 = (($740) + 2|0);
        $742 = HEAP16[$741>>1]|0;
        $743 = $742&65535;
        $744 = $743 & 255;
        $745 = $744&255;
        $746 = $sp;
        $747 = (($746) + 1|0);
        HEAP8[$747>>0] = $745;
        $748 = $3;
        $749 = (($748) + 4|0);
        $750 = HEAP16[$749>>1]|0;
        $751 = $750&65535;
        $752 = $751 >> 8;
        $753 = $752 & 255;
        $754 = $753&255;
        $755 = $sp;
        $756 = (($755) + 2|0);
        HEAP8[$756>>0] = $754;
        $757 = $3;
        $758 = (($757) + 4|0);
        $759 = HEAP16[$758>>1]|0;
        $760 = $759&65535;
        $761 = $760 & 255;
        $762 = $761&255;
        $763 = $sp;
        $764 = (($763) + 3|0);
        HEAP8[$764>>0] = $762;
        $765 = $3;
        $766 = (($765) + 6|0);
        $767 = HEAP16[$766>>1]|0;
        $768 = $767&65535;
        $769 = $768 >> 8;
        $770 = $769 & 255;
        $771 = $770&255;
        $772 = $sp;
        $773 = (($772) + 4|0);
        HEAP8[$773>>0] = $771;
        $774 = $3;
        $775 = (($774) + 6|0);
        $776 = HEAP16[$775>>1]|0;
        $777 = $776&65535;
        $778 = $777 & 255;
        $779 = $778&255;
        $780 = $sp;
        $781 = (($780) + 5|0);
        HEAP8[$781>>0] = $779;
       } else {
        label = 127;
       }
      } else {
       label = 127;
      }
     } else {
      label = 127;
     }
     if ((label|0) == 127) {
      label = 0;
      $782 = $sp;
      $783 = HEAP8[$782>>0]|0;
      $784 = $783&255;
      $785 = $sp;
      $786 = (($785) + 1|0);
      $787 = HEAP8[$786>>0]|0;
      $788 = $787&255;
      $789 = $11;
      $790 = $788 >> $789;
      $791 = $8;
      $792 = (($791) + ($790<<2)|0);
      $793 = HEAP32[$792>>2]|0;
      $794 = (($793) + ($784<<1)|0);
      $795 = HEAP16[$794>>1]|0;
      $v5 = $795;
      $796 = $v5;
      $797 = $796&65535;
      $798 = $797 >> 8;
      $799 = $798 & 255;
      $800 = $799&255;
      $801 = $sp;
      HEAP8[$801>>0] = $800;
      $802 = $v5;
      $803 = $802&65535;
      $804 = $803 & 255;
      $805 = $804&255;
      $806 = $sp;
      $807 = (($806) + 1|0);
      HEAP8[$807>>0] = $805;
      $808 = $sp;
      $809 = (($808) + 2|0);
      $810 = HEAP8[$809>>0]|0;
      $811 = $810&255;
      $812 = $sp;
      $813 = (($812) + 3|0);
      $814 = HEAP8[$813>>0]|0;
      $815 = $814&255;
      $816 = $11;
      $817 = $815 >> $816;
      $818 = $8;
      $819 = (($818) + ($817<<2)|0);
      $820 = HEAP32[$819>>2]|0;
      $821 = (($820) + ($811<<1)|0);
      $822 = HEAP16[$821>>1]|0;
      $v5 = $822;
      $823 = $v5;
      $824 = $823&65535;
      $825 = $824 >> 8;
      $826 = $825 & 255;
      $827 = $826&255;
      $828 = $sp;
      $829 = (($828) + 2|0);
      HEAP8[$829>>0] = $827;
      $830 = $v5;
      $831 = $830&65535;
      $832 = $831 & 255;
      $833 = $832&255;
      $834 = $sp;
      $835 = (($834) + 3|0);
      HEAP8[$835>>0] = $833;
      $836 = $sp;
      $837 = (($836) + 4|0);
      $838 = HEAP8[$837>>0]|0;
      $839 = $838&255;
      $840 = $sp;
      $841 = (($840) + 5|0);
      $842 = HEAP8[$841>>0]|0;
      $843 = $842&255;
      $844 = $11;
      $845 = $843 >> $844;
      $846 = $8;
      $847 = (($846) + ($845<<2)|0);
      $848 = HEAP32[$847>>2]|0;
      $849 = (($848) + ($839<<1)|0);
      $850 = HEAP16[$849>>1]|0;
      $v5 = $850;
      $851 = $v5;
      $852 = $851&65535;
      $853 = $852 >> 8;
      $854 = $853 & 255;
      $855 = $854&255;
      $856 = $sp;
      $857 = (($856) + 4|0);
      HEAP8[$857>>0] = $855;
      $858 = $v5;
      $859 = $858&65535;
      $860 = $859 & 255;
      $861 = $860&255;
      $862 = $sp;
      $863 = (($862) + 5|0);
      HEAP8[$863>>0] = $861;
     }
     $864 = $i;
     $865 = (($864) + 1)|0;
     $i = $865;
     $866 = $sp;
     $867 = (($866) + 6|0);
     $sp = $867;
    }
   } else {
    $868 = $1;
    $sp = $868;
    $i = 0;
    while(1) {
     $869 = $i;
     $870 = $row_width;
     $871 = ($869>>>0)<($870>>>0);
     if (!($871)) {
      break;
     }
     $872 = $sp;
     $873 = HEAP8[$872>>0]|0;
     $874 = $873&255;
     $875 = $874 << 8;
     $876 = $sp;
     $877 = (($876) + 1|0);
     $878 = HEAP8[$877>>0]|0;
     $879 = $878&255;
     $880 = (($875) + ($879))|0;
     $881 = $880&65535;
     $r6 = $881;
     $882 = $sp;
     $883 = (($882) + 2|0);
     $884 = HEAP8[$883>>0]|0;
     $885 = $884&255;
     $886 = $885 << 8;
     $887 = $sp;
     $888 = (($887) + 3|0);
     $889 = HEAP8[$888>>0]|0;
     $890 = $889&255;
     $891 = (($886) + ($890))|0;
     $892 = $891&65535;
     $g7 = $892;
     $893 = $sp;
     $894 = (($893) + 4|0);
     $895 = HEAP8[$894>>0]|0;
     $896 = $895&255;
     $897 = $896 << 8;
     $898 = $sp;
     $899 = (($898) + 5|0);
     $900 = HEAP8[$899>>0]|0;
     $901 = $900&255;
     $902 = (($897) + ($901))|0;
     $903 = $902&65535;
     $b8 = $903;
     $904 = $r6;
     $905 = $904&65535;
     $906 = $2;
     $907 = (($906) + 2|0);
     $908 = HEAP16[$907>>1]|0;
     $909 = $908&65535;
     $910 = ($905|0)==($909|0);
     if ($910) {
      $911 = $g7;
      $912 = $911&65535;
      $913 = $2;
      $914 = (($913) + 4|0);
      $915 = HEAP16[$914>>1]|0;
      $916 = $915&65535;
      $917 = ($912|0)==($916|0);
      if ($917) {
       $918 = $b8;
       $919 = $918&65535;
       $920 = $2;
       $921 = (($920) + 6|0);
       $922 = HEAP16[$921>>1]|0;
       $923 = $922&65535;
       $924 = ($919|0)==($923|0);
       if ($924) {
        $925 = $3;
        $926 = (($925) + 2|0);
        $927 = HEAP16[$926>>1]|0;
        $928 = $927&65535;
        $929 = $928 >> 8;
        $930 = $929 & 255;
        $931 = $930&255;
        $932 = $sp;
        HEAP8[$932>>0] = $931;
        $933 = $3;
        $934 = (($933) + 2|0);
        $935 = HEAP16[$934>>1]|0;
        $936 = $935&65535;
        $937 = $936 & 255;
        $938 = $937&255;
        $939 = $sp;
        $940 = (($939) + 1|0);
        HEAP8[$940>>0] = $938;
        $941 = $3;
        $942 = (($941) + 4|0);
        $943 = HEAP16[$942>>1]|0;
        $944 = $943&65535;
        $945 = $944 >> 8;
        $946 = $945 & 255;
        $947 = $946&255;
        $948 = $sp;
        $949 = (($948) + 2|0);
        HEAP8[$949>>0] = $947;
        $950 = $3;
        $951 = (($950) + 4|0);
        $952 = HEAP16[$951>>1]|0;
        $953 = $952&65535;
        $954 = $953 & 255;
        $955 = $954&255;
        $956 = $sp;
        $957 = (($956) + 3|0);
        HEAP8[$957>>0] = $955;
        $958 = $3;
        $959 = (($958) + 6|0);
        $960 = HEAP16[$959>>1]|0;
        $961 = $960&65535;
        $962 = $961 >> 8;
        $963 = $962 & 255;
        $964 = $963&255;
        $965 = $sp;
        $966 = (($965) + 4|0);
        HEAP8[$966>>0] = $964;
        $967 = $3;
        $968 = (($967) + 6|0);
        $969 = HEAP16[$968>>1]|0;
        $970 = $969&65535;
        $971 = $970 & 255;
        $972 = $971&255;
        $973 = $sp;
        $974 = (($973) + 5|0);
        HEAP8[$974>>0] = $972;
       }
      }
     }
     $975 = $i;
     $976 = (($975) + 1)|0;
     $i = $976;
     $977 = $sp;
     $978 = (($977) + 6|0);
     $sp = $978;
    }
   }
  }
 } else if ((($32|0) == 4)) {
  $979 = $0;
  $980 = (($979) + 9|0);
  $981 = HEAP8[$980>>0]|0;
  $982 = $981&255;
  $983 = ($982|0)==(8);
  if ($983) {
   $984 = $7;
   $985 = ($984|0)!=(0|0);
   if ($985) {
    $986 = $6;
    $987 = ($986|0)!=(0|0);
    if ($987) {
     $988 = $5;
     $989 = ($988|0)!=(0|0);
     if ($989) {
      $990 = $1;
      $sp = $990;
      $991 = $1;
      $dp = $991;
      $i = 0;
      while(1) {
       $992 = $i;
       $993 = $row_width;
       $994 = ($992>>>0)<($993>>>0);
       if (!($994)) {
        break;
       }
       $995 = $sp;
       $996 = (($995) + 1|0);
       $997 = HEAP8[$996>>0]|0;
       $998 = $997&255;
       $a = $998;
       $999 = $a;
       $1000 = $999&65535;
       $1001 = ($1000|0)==(255);
       if ($1001) {
        $1002 = $sp;
        $1003 = HEAP8[$1002>>0]|0;
        $1004 = $1003&255;
        $1005 = $5;
        $1006 = (($1005) + ($1004)|0);
        $1007 = HEAP8[$1006>>0]|0;
        $1008 = $dp;
        HEAP8[$1008>>0] = $1007;
       } else {
        $1009 = $a;
        $1010 = $1009&65535;
        $1011 = ($1010|0)==(0);
        if ($1011) {
         $1012 = $3;
         $1013 = (($1012) + 8|0);
         $1014 = HEAP16[$1013>>1]|0;
         $1015 = $1014&255;
         $1016 = $dp;
         HEAP8[$1016>>0] = $1015;
        } else {
         $1017 = $sp;
         $1018 = HEAP8[$1017>>0]|0;
         $1019 = $1018&255;
         $1020 = $7;
         $1021 = (($1020) + ($1019)|0);
         $1022 = HEAP8[$1021>>0]|0;
         $v9 = $1022;
         $1023 = $v9;
         $1024 = $1023&255;
         $1025 = $1024&65535;
         $1026 = $a;
         $1027 = $1026&65535;
         $1028 = Math_imul($1025, $1027)|0;
         $1029 = $4;
         $1030 = (($1029) + 8|0);
         $1031 = HEAP16[$1030>>1]|0;
         $1032 = $1031&65535;
         $1033 = $a;
         $1034 = $1033&65535;
         $1035 = (255 - ($1034))|0;
         $1036 = $1035&65535;
         $1037 = $1036&65535;
         $1038 = Math_imul($1032, $1037)|0;
         $1039 = (($1028) + ($1038))|0;
         $1040 = (($1039) + 128)|0;
         $1041 = $1040&65535;
         $temp = $1041;
         $1042 = $temp;
         $1043 = $1042&65535;
         $1044 = $temp;
         $1045 = $1044&65535;
         $1046 = $1045 >> 8;
         $1047 = (($1043) + ($1046))|0;
         $1048 = $1047 >> 8;
         $1049 = $1048&255;
         $w = $1049;
         $1050 = $w;
         $1051 = $1050&255;
         $1052 = $6;
         $1053 = (($1052) + ($1051)|0);
         $1054 = HEAP8[$1053>>0]|0;
         $1055 = $dp;
         HEAP8[$1055>>0] = $1054;
        }
       }
       $1056 = $i;
       $1057 = (($1056) + 1)|0;
       $i = $1057;
       $1058 = $sp;
       $1059 = (($1058) + 2|0);
       $sp = $1059;
       $1060 = $dp;
       $1061 = (($1060) + 1|0);
       $dp = $1061;
      }
     } else {
      label = 157;
     }
    } else {
     label = 157;
    }
   } else {
    label = 157;
   }
   if ((label|0) == 157) {
    $1062 = $1;
    $sp = $1062;
    $1063 = $1;
    $dp = $1063;
    $i = 0;
    while(1) {
     $1064 = $i;
     $1065 = $row_width;
     $1066 = ($1064>>>0)<($1065>>>0);
     if (!($1066)) {
      break;
     }
     $1067 = $sp;
     $1068 = (($1067) + 1|0);
     $1069 = HEAP8[$1068>>0]|0;
     $a10 = $1069;
     $1070 = $a10;
     $1071 = $1070&255;
     $1072 = ($1071|0)==(255);
     if ($1072) {
      $1073 = $sp;
      $1074 = HEAP8[$1073>>0]|0;
      $1075 = $dp;
      HEAP8[$1075>>0] = $1074;
     } else {
      $1076 = $a10;
      $1077 = $1076&255;
      $1078 = ($1077|0)==(0);
      if ($1078) {
       $1079 = $3;
       $1080 = (($1079) + 8|0);
       $1081 = HEAP16[$1080>>1]|0;
       $1082 = $1081&255;
       $1083 = $dp;
       HEAP8[$1083>>0] = $1082;
      } else {
       $1084 = $sp;
       $1085 = HEAP8[$1084>>0]|0;
       $1086 = $1085&255;
       $1087 = $1086&65535;
       $1088 = $a10;
       $1089 = $1088&255;
       $1090 = $1089&65535;
       $1091 = Math_imul($1087, $1090)|0;
       $1092 = $4;
       $1093 = (($1092) + 8|0);
       $1094 = HEAP16[$1093>>1]|0;
       $1095 = $1094&65535;
       $1096 = $a10;
       $1097 = $1096&255;
       $1098 = $1097&65535;
       $1099 = (255 - ($1098))|0;
       $1100 = $1099&65535;
       $1101 = $1100&65535;
       $1102 = Math_imul($1095, $1101)|0;
       $1103 = (($1091) + ($1102))|0;
       $1104 = (($1103) + 128)|0;
       $1105 = $1104&65535;
       $temp11 = $1105;
       $1106 = $temp11;
       $1107 = $1106&65535;
       $1108 = $temp11;
       $1109 = $1108&65535;
       $1110 = $1109 >> 8;
       $1111 = (($1107) + ($1110))|0;
       $1112 = $1111 >> 8;
       $1113 = $1112&255;
       $1114 = $dp;
       HEAP8[$1114>>0] = $1113;
      }
     }
     $1115 = $i;
     $1116 = (($1115) + 1)|0;
     $i = $1116;
     $1117 = $sp;
     $1118 = (($1117) + 2|0);
     $sp = $1118;
     $1119 = $dp;
     $1120 = (($1119) + 1|0);
     $dp = $1120;
    }
   }
  } else {
   $1121 = $8;
   $1122 = ($1121|0)!=(0|0);
   if ($1122) {
    $1123 = $9;
    $1124 = ($1123|0)!=(0|0);
    if ($1124) {
     $1125 = $10;
     $1126 = ($1125|0)!=(0|0);
     if ($1126) {
      $1127 = $1;
      $sp = $1127;
      $1128 = $1;
      $dp = $1128;
      $i = 0;
      while(1) {
       $1129 = $i;
       $1130 = $row_width;
       $1131 = ($1129>>>0)<($1130>>>0);
       if (!($1131)) {
        break;
       }
       $1132 = $sp;
       $1133 = (($1132) + 2|0);
       $1134 = HEAP8[$1133>>0]|0;
       $1135 = $1134&255;
       $1136 = $1135 << 8;
       $1137 = $sp;
       $1138 = (($1137) + 3|0);
       $1139 = HEAP8[$1138>>0]|0;
       $1140 = $1139&255;
       $1141 = (($1136) + ($1140))|0;
       $1142 = $1141&65535;
       $a12 = $1142;
       $1143 = $a12;
       $1144 = $1143&65535;
       $1145 = ($1144|0)==(65535);
       if ($1145) {
        $1146 = $sp;
        $1147 = HEAP8[$1146>>0]|0;
        $1148 = $1147&255;
        $1149 = $sp;
        $1150 = (($1149) + 1|0);
        $1151 = HEAP8[$1150>>0]|0;
        $1152 = $1151&255;
        $1153 = $11;
        $1154 = $1152 >> $1153;
        $1155 = $8;
        $1156 = (($1155) + ($1154<<2)|0);
        $1157 = HEAP32[$1156>>2]|0;
        $1158 = (($1157) + ($1148<<1)|0);
        $1159 = HEAP16[$1158>>1]|0;
        $v13 = $1159;
        $1160 = $v13;
        $1161 = $1160&65535;
        $1162 = $1161 >> 8;
        $1163 = $1162 & 255;
        $1164 = $1163&255;
        $1165 = $dp;
        HEAP8[$1165>>0] = $1164;
        $1166 = $v13;
        $1167 = $1166&65535;
        $1168 = $1167 & 255;
        $1169 = $1168&255;
        $1170 = $dp;
        $1171 = (($1170) + 1|0);
        HEAP8[$1171>>0] = $1169;
       } else {
        $1172 = $a12;
        $1173 = $1172&65535;
        $1174 = ($1173|0)==(0);
        if ($1174) {
         $1175 = $3;
         $1176 = (($1175) + 8|0);
         $1177 = HEAP16[$1176>>1]|0;
         $1178 = $1177&65535;
         $1179 = $1178 >> 8;
         $1180 = $1179 & 255;
         $1181 = $1180&255;
         $1182 = $dp;
         HEAP8[$1182>>0] = $1181;
         $1183 = $3;
         $1184 = (($1183) + 8|0);
         $1185 = HEAP16[$1184>>1]|0;
         $1186 = $1185&65535;
         $1187 = $1186 & 255;
         $1188 = $1187&255;
         $1189 = $dp;
         $1190 = (($1189) + 1|0);
         HEAP8[$1190>>0] = $1188;
        } else {
         $1191 = $sp;
         $1192 = HEAP8[$1191>>0]|0;
         $1193 = $1192&255;
         $1194 = $sp;
         $1195 = (($1194) + 1|0);
         $1196 = HEAP8[$1195>>0]|0;
         $1197 = $1196&255;
         $1198 = $11;
         $1199 = $1197 >> $1198;
         $1200 = $10;
         $1201 = (($1200) + ($1199<<2)|0);
         $1202 = HEAP32[$1201>>2]|0;
         $1203 = (($1202) + ($1193<<1)|0);
         $1204 = HEAP16[$1203>>1]|0;
         $g14 = $1204;
         $1205 = $g14;
         $1206 = $1205&65535;
         $1207 = $a12;
         $1208 = $1207&65535;
         $1209 = Math_imul($1206, $1208)|0;
         $1210 = $4;
         $1211 = (($1210) + 8|0);
         $1212 = HEAP16[$1211>>1]|0;
         $1213 = $1212&65535;
         $1214 = $a12;
         $1215 = $1214&65535;
         $1216 = (65535 - ($1215))|0;
         $1217 = Math_imul($1213, $1216)|0;
         $1218 = (($1209) + ($1217))|0;
         $1219 = (($1218) + 32768)|0;
         $temp17 = $1219;
         $1220 = $temp17;
         $1221 = $temp17;
         $1222 = $1221 >>> 16;
         $1223 = (($1220) + ($1222))|0;
         $1224 = $1223 >>> 16;
         $1225 = $1224&65535;
         $v15 = $1225;
         $1226 = $v15;
         $1227 = $1226&65535;
         $1228 = $1227 >> 8;
         $1229 = $v15;
         $1230 = $1229&65535;
         $1231 = $1230 & 255;
         $1232 = $11;
         $1233 = $1231 >> $1232;
         $1234 = $9;
         $1235 = (($1234) + ($1233<<2)|0);
         $1236 = HEAP32[$1235>>2]|0;
         $1237 = (($1236) + ($1228<<1)|0);
         $1238 = HEAP16[$1237>>1]|0;
         $w16 = $1238;
         $1239 = $w16;
         $1240 = $1239&65535;
         $1241 = $1240 >> 8;
         $1242 = $1241 & 255;
         $1243 = $1242&255;
         $1244 = $dp;
         HEAP8[$1244>>0] = $1243;
         $1245 = $w16;
         $1246 = $1245&65535;
         $1247 = $1246 & 255;
         $1248 = $1247&255;
         $1249 = $dp;
         $1250 = (($1249) + 1|0);
         HEAP8[$1250>>0] = $1248;
        }
       }
       $1251 = $i;
       $1252 = (($1251) + 1)|0;
       $i = $1252;
       $1253 = $sp;
       $1254 = (($1253) + 4|0);
       $sp = $1254;
       $1255 = $dp;
       $1256 = (($1255) + 2|0);
       $dp = $1256;
      }
     } else {
      label = 183;
     }
    } else {
     label = 183;
    }
   } else {
    label = 183;
   }
   if ((label|0) == 183) {
    $1257 = $1;
    $sp = $1257;
    $1258 = $1;
    $dp = $1258;
    $i = 0;
    while(1) {
     $1259 = $i;
     $1260 = $row_width;
     $1261 = ($1259>>>0)<($1260>>>0);
     if (!($1261)) {
      break;
     }
     $1262 = $sp;
     $1263 = (($1262) + 2|0);
     $1264 = HEAP8[$1263>>0]|0;
     $1265 = $1264&255;
     $1266 = $1265 << 8;
     $1267 = $sp;
     $1268 = (($1267) + 3|0);
     $1269 = HEAP8[$1268>>0]|0;
     $1270 = $1269&255;
     $1271 = (($1266) + ($1270))|0;
     $1272 = $1271&65535;
     $a18 = $1272;
     $1273 = $a18;
     $1274 = $1273&65535;
     $1275 = ($1274|0)==(65535);
     if ($1275) {
      $1276 = $dp;
      $1277 = $sp;
      ;HEAP8[$1276+0>>0]=HEAP8[$1277+0>>0]|0;HEAP8[$1276+1>>0]=HEAP8[$1277+1>>0]|0;
     } else {
      $1278 = $a18;
      $1279 = $1278&65535;
      $1280 = ($1279|0)==(0);
      if ($1280) {
       $1281 = $3;
       $1282 = (($1281) + 8|0);
       $1283 = HEAP16[$1282>>1]|0;
       $1284 = $1283&65535;
       $1285 = $1284 >> 8;
       $1286 = $1285 & 255;
       $1287 = $1286&255;
       $1288 = $dp;
       HEAP8[$1288>>0] = $1287;
       $1289 = $3;
       $1290 = (($1289) + 8|0);
       $1291 = HEAP16[$1290>>1]|0;
       $1292 = $1291&65535;
       $1293 = $1292 & 255;
       $1294 = $1293&255;
       $1295 = $dp;
       $1296 = (($1295) + 1|0);
       HEAP8[$1296>>0] = $1294;
      } else {
       $1297 = $sp;
       $1298 = HEAP8[$1297>>0]|0;
       $1299 = $1298&255;
       $1300 = $1299 << 8;
       $1301 = $sp;
       $1302 = (($1301) + 1|0);
       $1303 = HEAP8[$1302>>0]|0;
       $1304 = $1303&255;
       $1305 = (($1300) + ($1304))|0;
       $1306 = $1305&65535;
       $g19 = $1306;
       $1307 = $g19;
       $1308 = $1307&65535;
       $1309 = $a18;
       $1310 = $1309&65535;
       $1311 = Math_imul($1308, $1310)|0;
       $1312 = $4;
       $1313 = (($1312) + 8|0);
       $1314 = HEAP16[$1313>>1]|0;
       $1315 = $1314&65535;
       $1316 = $a18;
       $1317 = $1316&65535;
       $1318 = (65535 - ($1317))|0;
       $1319 = Math_imul($1315, $1318)|0;
       $1320 = (($1311) + ($1319))|0;
       $1321 = (($1320) + 32768)|0;
       $temp21 = $1321;
       $1322 = $temp21;
       $1323 = $temp21;
       $1324 = $1323 >>> 16;
       $1325 = (($1322) + ($1324))|0;
       $1326 = $1325 >>> 16;
       $1327 = $1326&65535;
       $v20 = $1327;
       $1328 = $v20;
       $1329 = $1328&65535;
       $1330 = $1329 >> 8;
       $1331 = $1330 & 255;
       $1332 = $1331&255;
       $1333 = $dp;
       HEAP8[$1333>>0] = $1332;
       $1334 = $v20;
       $1335 = $1334&65535;
       $1336 = $1335 & 255;
       $1337 = $1336&255;
       $1338 = $dp;
       $1339 = (($1338) + 1|0);
       HEAP8[$1339>>0] = $1337;
      }
     }
     $1340 = $i;
     $1341 = (($1340) + 1)|0;
     $i = $1341;
     $1342 = $sp;
     $1343 = (($1342) + 4|0);
     $sp = $1343;
     $1344 = $dp;
     $1345 = (($1344) + 2|0);
     $dp = $1345;
    }
   }
  }
 } else if ((($32|0) == 6)) {
  $1346 = $0;
  $1347 = (($1346) + 9|0);
  $1348 = HEAP8[$1347>>0]|0;
  $1349 = $1348&255;
  $1350 = ($1349|0)==(8);
  if ($1350) {
   $1351 = $7;
   $1352 = ($1351|0)!=(0|0);
   if ($1352) {
    $1353 = $6;
    $1354 = ($1353|0)!=(0|0);
    if ($1354) {
     $1355 = $5;
     $1356 = ($1355|0)!=(0|0);
     if ($1356) {
      $1357 = $1;
      $sp = $1357;
      $1358 = $1;
      $dp = $1358;
      $i = 0;
      while(1) {
       $1359 = $i;
       $1360 = $row_width;
       $1361 = ($1359>>>0)<($1360>>>0);
       if (!($1361)) {
        break;
       }
       $1362 = $sp;
       $1363 = (($1362) + 3|0);
       $1364 = HEAP8[$1363>>0]|0;
       $a22 = $1364;
       $1365 = $a22;
       $1366 = $1365&255;
       $1367 = ($1366|0)==(255);
       if ($1367) {
        $1368 = $sp;
        $1369 = HEAP8[$1368>>0]|0;
        $1370 = $1369&255;
        $1371 = $5;
        $1372 = (($1371) + ($1370)|0);
        $1373 = HEAP8[$1372>>0]|0;
        $1374 = $dp;
        HEAP8[$1374>>0] = $1373;
        $1375 = $sp;
        $1376 = (($1375) + 1|0);
        $1377 = HEAP8[$1376>>0]|0;
        $1378 = $1377&255;
        $1379 = $5;
        $1380 = (($1379) + ($1378)|0);
        $1381 = HEAP8[$1380>>0]|0;
        $1382 = $dp;
        $1383 = (($1382) + 1|0);
        HEAP8[$1383>>0] = $1381;
        $1384 = $sp;
        $1385 = (($1384) + 2|0);
        $1386 = HEAP8[$1385>>0]|0;
        $1387 = $1386&255;
        $1388 = $5;
        $1389 = (($1388) + ($1387)|0);
        $1390 = HEAP8[$1389>>0]|0;
        $1391 = $dp;
        $1392 = (($1391) + 2|0);
        HEAP8[$1392>>0] = $1390;
       } else {
        $1393 = $a22;
        $1394 = $1393&255;
        $1395 = ($1394|0)==(0);
        if ($1395) {
         $1396 = $3;
         $1397 = (($1396) + 2|0);
         $1398 = HEAP16[$1397>>1]|0;
         $1399 = $1398&255;
         $1400 = $dp;
         HEAP8[$1400>>0] = $1399;
         $1401 = $3;
         $1402 = (($1401) + 4|0);
         $1403 = HEAP16[$1402>>1]|0;
         $1404 = $1403&255;
         $1405 = $dp;
         $1406 = (($1405) + 1|0);
         HEAP8[$1406>>0] = $1404;
         $1407 = $3;
         $1408 = (($1407) + 6|0);
         $1409 = HEAP16[$1408>>1]|0;
         $1410 = $1409&255;
         $1411 = $dp;
         $1412 = (($1411) + 2|0);
         HEAP8[$1412>>0] = $1410;
        } else {
         $1413 = $sp;
         $1414 = HEAP8[$1413>>0]|0;
         $1415 = $1414&255;
         $1416 = $7;
         $1417 = (($1416) + ($1415)|0);
         $1418 = HEAP8[$1417>>0]|0;
         $v23 = $1418;
         $1419 = $v23;
         $1420 = $1419&255;
         $1421 = $1420&65535;
         $1422 = $a22;
         $1423 = $1422&255;
         $1424 = $1423&65535;
         $1425 = Math_imul($1421, $1424)|0;
         $1426 = $4;
         $1427 = (($1426) + 2|0);
         $1428 = HEAP16[$1427>>1]|0;
         $1429 = $1428&65535;
         $1430 = $a22;
         $1431 = $1430&255;
         $1432 = $1431&65535;
         $1433 = (255 - ($1432))|0;
         $1434 = $1433&65535;
         $1435 = $1434&65535;
         $1436 = Math_imul($1429, $1435)|0;
         $1437 = (($1425) + ($1436))|0;
         $1438 = (($1437) + 128)|0;
         $1439 = $1438&65535;
         $temp25 = $1439;
         $1440 = $temp25;
         $1441 = $1440&65535;
         $1442 = $temp25;
         $1443 = $1442&65535;
         $1444 = $1443 >> 8;
         $1445 = (($1441) + ($1444))|0;
         $1446 = $1445 >> 8;
         $1447 = $1446&255;
         $w24 = $1447;
         $1448 = $w24;
         $1449 = $1448&255;
         $1450 = $6;
         $1451 = (($1450) + ($1449)|0);
         $1452 = HEAP8[$1451>>0]|0;
         $1453 = $dp;
         HEAP8[$1453>>0] = $1452;
         $1454 = $sp;
         $1455 = (($1454) + 1|0);
         $1456 = HEAP8[$1455>>0]|0;
         $1457 = $1456&255;
         $1458 = $7;
         $1459 = (($1458) + ($1457)|0);
         $1460 = HEAP8[$1459>>0]|0;
         $v23 = $1460;
         $1461 = $v23;
         $1462 = $1461&255;
         $1463 = $1462&65535;
         $1464 = $a22;
         $1465 = $1464&255;
         $1466 = $1465&65535;
         $1467 = Math_imul($1463, $1466)|0;
         $1468 = $4;
         $1469 = (($1468) + 4|0);
         $1470 = HEAP16[$1469>>1]|0;
         $1471 = $1470&65535;
         $1472 = $a22;
         $1473 = $1472&255;
         $1474 = $1473&65535;
         $1475 = (255 - ($1474))|0;
         $1476 = $1475&65535;
         $1477 = $1476&65535;
         $1478 = Math_imul($1471, $1477)|0;
         $1479 = (($1467) + ($1478))|0;
         $1480 = (($1479) + 128)|0;
         $1481 = $1480&65535;
         $temp26 = $1481;
         $1482 = $temp26;
         $1483 = $1482&65535;
         $1484 = $temp26;
         $1485 = $1484&65535;
         $1486 = $1485 >> 8;
         $1487 = (($1483) + ($1486))|0;
         $1488 = $1487 >> 8;
         $1489 = $1488&255;
         $w24 = $1489;
         $1490 = $w24;
         $1491 = $1490&255;
         $1492 = $6;
         $1493 = (($1492) + ($1491)|0);
         $1494 = HEAP8[$1493>>0]|0;
         $1495 = $dp;
         $1496 = (($1495) + 1|0);
         HEAP8[$1496>>0] = $1494;
         $1497 = $sp;
         $1498 = (($1497) + 2|0);
         $1499 = HEAP8[$1498>>0]|0;
         $1500 = $1499&255;
         $1501 = $7;
         $1502 = (($1501) + ($1500)|0);
         $1503 = HEAP8[$1502>>0]|0;
         $v23 = $1503;
         $1504 = $v23;
         $1505 = $1504&255;
         $1506 = $1505&65535;
         $1507 = $a22;
         $1508 = $1507&255;
         $1509 = $1508&65535;
         $1510 = Math_imul($1506, $1509)|0;
         $1511 = $4;
         $1512 = (($1511) + 6|0);
         $1513 = HEAP16[$1512>>1]|0;
         $1514 = $1513&65535;
         $1515 = $a22;
         $1516 = $1515&255;
         $1517 = $1516&65535;
         $1518 = (255 - ($1517))|0;
         $1519 = $1518&65535;
         $1520 = $1519&65535;
         $1521 = Math_imul($1514, $1520)|0;
         $1522 = (($1510) + ($1521))|0;
         $1523 = (($1522) + 128)|0;
         $1524 = $1523&65535;
         $temp27 = $1524;
         $1525 = $temp27;
         $1526 = $1525&65535;
         $1527 = $temp27;
         $1528 = $1527&65535;
         $1529 = $1528 >> 8;
         $1530 = (($1526) + ($1529))|0;
         $1531 = $1530 >> 8;
         $1532 = $1531&255;
         $w24 = $1532;
         $1533 = $w24;
         $1534 = $1533&255;
         $1535 = $6;
         $1536 = (($1535) + ($1534)|0);
         $1537 = HEAP8[$1536>>0]|0;
         $1538 = $dp;
         $1539 = (($1538) + 2|0);
         HEAP8[$1539>>0] = $1537;
        }
       }
       $1540 = $i;
       $1541 = (($1540) + 1)|0;
       $i = $1541;
       $1542 = $sp;
       $1543 = (($1542) + 4|0);
       $sp = $1543;
       $1544 = $dp;
       $1545 = (($1544) + 3|0);
       $dp = $1545;
      }
     } else {
      label = 211;
     }
    } else {
     label = 211;
    }
   } else {
    label = 211;
   }
   if ((label|0) == 211) {
    $1546 = $1;
    $sp = $1546;
    $1547 = $1;
    $dp = $1547;
    $i = 0;
    while(1) {
     $1548 = $i;
     $1549 = $row_width;
     $1550 = ($1548>>>0)<($1549>>>0);
     if (!($1550)) {
      break;
     }
     $1551 = $sp;
     $1552 = (($1551) + 3|0);
     $1553 = HEAP8[$1552>>0]|0;
     $a28 = $1553;
     $1554 = $a28;
     $1555 = $1554&255;
     $1556 = ($1555|0)==(255);
     if ($1556) {
      $1557 = $sp;
      $1558 = HEAP8[$1557>>0]|0;
      $1559 = $dp;
      HEAP8[$1559>>0] = $1558;
      $1560 = $sp;
      $1561 = (($1560) + 1|0);
      $1562 = HEAP8[$1561>>0]|0;
      $1563 = $dp;
      $1564 = (($1563) + 1|0);
      HEAP8[$1564>>0] = $1562;
      $1565 = $sp;
      $1566 = (($1565) + 2|0);
      $1567 = HEAP8[$1566>>0]|0;
      $1568 = $dp;
      $1569 = (($1568) + 2|0);
      HEAP8[$1569>>0] = $1567;
     } else {
      $1570 = $a28;
      $1571 = $1570&255;
      $1572 = ($1571|0)==(0);
      if ($1572) {
       $1573 = $3;
       $1574 = (($1573) + 2|0);
       $1575 = HEAP16[$1574>>1]|0;
       $1576 = $1575&255;
       $1577 = $dp;
       HEAP8[$1577>>0] = $1576;
       $1578 = $3;
       $1579 = (($1578) + 4|0);
       $1580 = HEAP16[$1579>>1]|0;
       $1581 = $1580&255;
       $1582 = $dp;
       $1583 = (($1582) + 1|0);
       HEAP8[$1583>>0] = $1581;
       $1584 = $3;
       $1585 = (($1584) + 6|0);
       $1586 = HEAP16[$1585>>1]|0;
       $1587 = $1586&255;
       $1588 = $dp;
       $1589 = (($1588) + 2|0);
       HEAP8[$1589>>0] = $1587;
      } else {
       $1590 = $sp;
       $1591 = HEAP8[$1590>>0]|0;
       $1592 = $1591&255;
       $1593 = $1592&65535;
       $1594 = $a28;
       $1595 = $1594&255;
       $1596 = $1595&65535;
       $1597 = Math_imul($1593, $1596)|0;
       $1598 = $3;
       $1599 = (($1598) + 2|0);
       $1600 = HEAP16[$1599>>1]|0;
       $1601 = $1600&65535;
       $1602 = $a28;
       $1603 = $1602&255;
       $1604 = $1603&65535;
       $1605 = (255 - ($1604))|0;
       $1606 = $1605&65535;
       $1607 = $1606&65535;
       $1608 = Math_imul($1601, $1607)|0;
       $1609 = (($1597) + ($1608))|0;
       $1610 = (($1609) + 128)|0;
       $1611 = $1610&65535;
       $temp29 = $1611;
       $1612 = $temp29;
       $1613 = $1612&65535;
       $1614 = $temp29;
       $1615 = $1614&65535;
       $1616 = $1615 >> 8;
       $1617 = (($1613) + ($1616))|0;
       $1618 = $1617 >> 8;
       $1619 = $1618&255;
       $1620 = $dp;
       HEAP8[$1620>>0] = $1619;
       $1621 = $sp;
       $1622 = (($1621) + 1|0);
       $1623 = HEAP8[$1622>>0]|0;
       $1624 = $1623&255;
       $1625 = $1624&65535;
       $1626 = $a28;
       $1627 = $1626&255;
       $1628 = $1627&65535;
       $1629 = Math_imul($1625, $1628)|0;
       $1630 = $3;
       $1631 = (($1630) + 4|0);
       $1632 = HEAP16[$1631>>1]|0;
       $1633 = $1632&65535;
       $1634 = $a28;
       $1635 = $1634&255;
       $1636 = $1635&65535;
       $1637 = (255 - ($1636))|0;
       $1638 = $1637&65535;
       $1639 = $1638&65535;
       $1640 = Math_imul($1633, $1639)|0;
       $1641 = (($1629) + ($1640))|0;
       $1642 = (($1641) + 128)|0;
       $1643 = $1642&65535;
       $temp30 = $1643;
       $1644 = $temp30;
       $1645 = $1644&65535;
       $1646 = $temp30;
       $1647 = $1646&65535;
       $1648 = $1647 >> 8;
       $1649 = (($1645) + ($1648))|0;
       $1650 = $1649 >> 8;
       $1651 = $1650&255;
       $1652 = $dp;
       $1653 = (($1652) + 1|0);
       HEAP8[$1653>>0] = $1651;
       $1654 = $sp;
       $1655 = (($1654) + 2|0);
       $1656 = HEAP8[$1655>>0]|0;
       $1657 = $1656&255;
       $1658 = $1657&65535;
       $1659 = $a28;
       $1660 = $1659&255;
       $1661 = $1660&65535;
       $1662 = Math_imul($1658, $1661)|0;
       $1663 = $3;
       $1664 = (($1663) + 6|0);
       $1665 = HEAP16[$1664>>1]|0;
       $1666 = $1665&65535;
       $1667 = $a28;
       $1668 = $1667&255;
       $1669 = $1668&65535;
       $1670 = (255 - ($1669))|0;
       $1671 = $1670&65535;
       $1672 = $1671&65535;
       $1673 = Math_imul($1666, $1672)|0;
       $1674 = (($1662) + ($1673))|0;
       $1675 = (($1674) + 128)|0;
       $1676 = $1675&65535;
       $temp31 = $1676;
       $1677 = $temp31;
       $1678 = $1677&65535;
       $1679 = $temp31;
       $1680 = $1679&65535;
       $1681 = $1680 >> 8;
       $1682 = (($1678) + ($1681))|0;
       $1683 = $1682 >> 8;
       $1684 = $1683&255;
       $1685 = $dp;
       $1686 = (($1685) + 2|0);
       HEAP8[$1686>>0] = $1684;
      }
     }
     $1687 = $i;
     $1688 = (($1687) + 1)|0;
     $i = $1688;
     $1689 = $sp;
     $1690 = (($1689) + 4|0);
     $sp = $1690;
     $1691 = $dp;
     $1692 = (($1691) + 3|0);
     $dp = $1692;
    }
   }
  } else {
   $1693 = $8;
   $1694 = ($1693|0)!=(0|0);
   if ($1694) {
    $1695 = $9;
    $1696 = ($1695|0)!=(0|0);
    if ($1696) {
     $1697 = $10;
     $1698 = ($1697|0)!=(0|0);
     if ($1698) {
      $1699 = $1;
      $sp = $1699;
      $1700 = $1;
      $dp = $1700;
      $i = 0;
      while(1) {
       $1701 = $i;
       $1702 = $row_width;
       $1703 = ($1701>>>0)<($1702>>>0);
       if (!($1703)) {
        break;
       }
       $1704 = $sp;
       $1705 = (($1704) + 6|0);
       $1706 = HEAP8[$1705>>0]|0;
       $1707 = $1706&255;
       $1708 = $1707&65535;
       $1709 = $1708 << 8;
       $1710 = $sp;
       $1711 = (($1710) + 7|0);
       $1712 = HEAP8[$1711>>0]|0;
       $1713 = $1712&255;
       $1714 = $1713&65535;
       $1715 = (($1709) + ($1714))|0;
       $1716 = $1715&65535;
       $a32 = $1716;
       $1717 = $a32;
       $1718 = $1717&65535;
       $1719 = ($1718|0)==(65535);
       if ($1719) {
        $1720 = $sp;
        $1721 = HEAP8[$1720>>0]|0;
        $1722 = $1721&255;
        $1723 = $sp;
        $1724 = (($1723) + 1|0);
        $1725 = HEAP8[$1724>>0]|0;
        $1726 = $1725&255;
        $1727 = $11;
        $1728 = $1726 >> $1727;
        $1729 = $8;
        $1730 = (($1729) + ($1728<<2)|0);
        $1731 = HEAP32[$1730>>2]|0;
        $1732 = (($1731) + ($1722<<1)|0);
        $1733 = HEAP16[$1732>>1]|0;
        $v33 = $1733;
        $1734 = $v33;
        $1735 = $1734&65535;
        $1736 = $1735 >> 8;
        $1737 = $1736 & 255;
        $1738 = $1737&255;
        $1739 = $dp;
        HEAP8[$1739>>0] = $1738;
        $1740 = $v33;
        $1741 = $1740&65535;
        $1742 = $1741 & 255;
        $1743 = $1742&255;
        $1744 = $dp;
        $1745 = (($1744) + 1|0);
        HEAP8[$1745>>0] = $1743;
        $1746 = $sp;
        $1747 = (($1746) + 2|0);
        $1748 = HEAP8[$1747>>0]|0;
        $1749 = $1748&255;
        $1750 = $sp;
        $1751 = (($1750) + 3|0);
        $1752 = HEAP8[$1751>>0]|0;
        $1753 = $1752&255;
        $1754 = $11;
        $1755 = $1753 >> $1754;
        $1756 = $8;
        $1757 = (($1756) + ($1755<<2)|0);
        $1758 = HEAP32[$1757>>2]|0;
        $1759 = (($1758) + ($1749<<1)|0);
        $1760 = HEAP16[$1759>>1]|0;
        $v33 = $1760;
        $1761 = $v33;
        $1762 = $1761&65535;
        $1763 = $1762 >> 8;
        $1764 = $1763 & 255;
        $1765 = $1764&255;
        $1766 = $dp;
        $1767 = (($1766) + 2|0);
        HEAP8[$1767>>0] = $1765;
        $1768 = $v33;
        $1769 = $1768&65535;
        $1770 = $1769 & 255;
        $1771 = $1770&255;
        $1772 = $dp;
        $1773 = (($1772) + 3|0);
        HEAP8[$1773>>0] = $1771;
        $1774 = $sp;
        $1775 = (($1774) + 4|0);
        $1776 = HEAP8[$1775>>0]|0;
        $1777 = $1776&255;
        $1778 = $sp;
        $1779 = (($1778) + 5|0);
        $1780 = HEAP8[$1779>>0]|0;
        $1781 = $1780&255;
        $1782 = $11;
        $1783 = $1781 >> $1782;
        $1784 = $8;
        $1785 = (($1784) + ($1783<<2)|0);
        $1786 = HEAP32[$1785>>2]|0;
        $1787 = (($1786) + ($1777<<1)|0);
        $1788 = HEAP16[$1787>>1]|0;
        $v33 = $1788;
        $1789 = $v33;
        $1790 = $1789&65535;
        $1791 = $1790 >> 8;
        $1792 = $1791 & 255;
        $1793 = $1792&255;
        $1794 = $dp;
        $1795 = (($1794) + 4|0);
        HEAP8[$1795>>0] = $1793;
        $1796 = $v33;
        $1797 = $1796&65535;
        $1798 = $1797 & 255;
        $1799 = $1798&255;
        $1800 = $dp;
        $1801 = (($1800) + 5|0);
        HEAP8[$1801>>0] = $1799;
       } else {
        $1802 = $a32;
        $1803 = $1802&65535;
        $1804 = ($1803|0)==(0);
        if ($1804) {
         $1805 = $3;
         $1806 = (($1805) + 2|0);
         $1807 = HEAP16[$1806>>1]|0;
         $1808 = $1807&65535;
         $1809 = $1808 >> 8;
         $1810 = $1809 & 255;
         $1811 = $1810&255;
         $1812 = $dp;
         HEAP8[$1812>>0] = $1811;
         $1813 = $3;
         $1814 = (($1813) + 2|0);
         $1815 = HEAP16[$1814>>1]|0;
         $1816 = $1815&65535;
         $1817 = $1816 & 255;
         $1818 = $1817&255;
         $1819 = $dp;
         $1820 = (($1819) + 1|0);
         HEAP8[$1820>>0] = $1818;
         $1821 = $3;
         $1822 = (($1821) + 4|0);
         $1823 = HEAP16[$1822>>1]|0;
         $1824 = $1823&65535;
         $1825 = $1824 >> 8;
         $1826 = $1825 & 255;
         $1827 = $1826&255;
         $1828 = $dp;
         $1829 = (($1828) + 2|0);
         HEAP8[$1829>>0] = $1827;
         $1830 = $3;
         $1831 = (($1830) + 4|0);
         $1832 = HEAP16[$1831>>1]|0;
         $1833 = $1832&65535;
         $1834 = $1833 & 255;
         $1835 = $1834&255;
         $1836 = $dp;
         $1837 = (($1836) + 3|0);
         HEAP8[$1837>>0] = $1835;
         $1838 = $3;
         $1839 = (($1838) + 6|0);
         $1840 = HEAP16[$1839>>1]|0;
         $1841 = $1840&65535;
         $1842 = $1841 >> 8;
         $1843 = $1842 & 255;
         $1844 = $1843&255;
         $1845 = $dp;
         $1846 = (($1845) + 4|0);
         HEAP8[$1846>>0] = $1844;
         $1847 = $3;
         $1848 = (($1847) + 6|0);
         $1849 = HEAP16[$1848>>1]|0;
         $1850 = $1849&65535;
         $1851 = $1850 & 255;
         $1852 = $1851&255;
         $1853 = $dp;
         $1854 = (($1853) + 5|0);
         HEAP8[$1854>>0] = $1852;
        } else {
         $1855 = $sp;
         $1856 = HEAP8[$1855>>0]|0;
         $1857 = $1856&255;
         $1858 = $sp;
         $1859 = (($1858) + 1|0);
         $1860 = HEAP8[$1859>>0]|0;
         $1861 = $1860&255;
         $1862 = $11;
         $1863 = $1861 >> $1862;
         $1864 = $10;
         $1865 = (($1864) + ($1863<<2)|0);
         $1866 = HEAP32[$1865>>2]|0;
         $1867 = (($1866) + ($1857<<1)|0);
         $1868 = HEAP16[$1867>>1]|0;
         $v34 = $1868;
         $1869 = $v34;
         $1870 = $1869&65535;
         $1871 = $a32;
         $1872 = $1871&65535;
         $1873 = Math_imul($1870, $1872)|0;
         $1874 = $4;
         $1875 = (($1874) + 2|0);
         $1876 = HEAP16[$1875>>1]|0;
         $1877 = $1876&65535;
         $1878 = $a32;
         $1879 = $1878&65535;
         $1880 = (65535 - ($1879))|0;
         $1881 = Math_imul($1877, $1880)|0;
         $1882 = (($1873) + ($1881))|0;
         $1883 = (($1882) + 32768)|0;
         $temp36 = $1883;
         $1884 = $temp36;
         $1885 = $temp36;
         $1886 = $1885 >>> 16;
         $1887 = (($1884) + ($1886))|0;
         $1888 = $1887 >>> 16;
         $1889 = $1888&65535;
         $w35 = $1889;
         $1890 = $w35;
         $1891 = $1890&65535;
         $1892 = $1891 >> 8;
         $1893 = $w35;
         $1894 = $1893&65535;
         $1895 = $1894 & 255;
         $1896 = $11;
         $1897 = $1895 >> $1896;
         $1898 = $9;
         $1899 = (($1898) + ($1897<<2)|0);
         $1900 = HEAP32[$1899>>2]|0;
         $1901 = (($1900) + ($1892<<1)|0);
         $1902 = HEAP16[$1901>>1]|0;
         $x = $1902;
         $1903 = $x;
         $1904 = $1903&65535;
         $1905 = $1904 >> 8;
         $1906 = $1905 & 255;
         $1907 = $1906&255;
         $1908 = $dp;
         HEAP8[$1908>>0] = $1907;
         $1909 = $x;
         $1910 = $1909&65535;
         $1911 = $1910 & 255;
         $1912 = $1911&255;
         $1913 = $dp;
         $1914 = (($1913) + 1|0);
         HEAP8[$1914>>0] = $1912;
         $1915 = $sp;
         $1916 = (($1915) + 2|0);
         $1917 = HEAP8[$1916>>0]|0;
         $1918 = $1917&255;
         $1919 = $sp;
         $1920 = (($1919) + 3|0);
         $1921 = HEAP8[$1920>>0]|0;
         $1922 = $1921&255;
         $1923 = $11;
         $1924 = $1922 >> $1923;
         $1925 = $10;
         $1926 = (($1925) + ($1924<<2)|0);
         $1927 = HEAP32[$1926>>2]|0;
         $1928 = (($1927) + ($1918<<1)|0);
         $1929 = HEAP16[$1928>>1]|0;
         $v34 = $1929;
         $1930 = $v34;
         $1931 = $1930&65535;
         $1932 = $a32;
         $1933 = $1932&65535;
         $1934 = Math_imul($1931, $1933)|0;
         $1935 = $4;
         $1936 = (($1935) + 4|0);
         $1937 = HEAP16[$1936>>1]|0;
         $1938 = $1937&65535;
         $1939 = $a32;
         $1940 = $1939&65535;
         $1941 = (65535 - ($1940))|0;
         $1942 = Math_imul($1938, $1941)|0;
         $1943 = (($1934) + ($1942))|0;
         $1944 = (($1943) + 32768)|0;
         $temp37 = $1944;
         $1945 = $temp37;
         $1946 = $temp37;
         $1947 = $1946 >>> 16;
         $1948 = (($1945) + ($1947))|0;
         $1949 = $1948 >>> 16;
         $1950 = $1949&65535;
         $w35 = $1950;
         $1951 = $w35;
         $1952 = $1951&65535;
         $1953 = $1952 >> 8;
         $1954 = $w35;
         $1955 = $1954&65535;
         $1956 = $1955 & 255;
         $1957 = $11;
         $1958 = $1956 >> $1957;
         $1959 = $9;
         $1960 = (($1959) + ($1958<<2)|0);
         $1961 = HEAP32[$1960>>2]|0;
         $1962 = (($1961) + ($1953<<1)|0);
         $1963 = HEAP16[$1962>>1]|0;
         $x = $1963;
         $1964 = $x;
         $1965 = $1964&65535;
         $1966 = $1965 >> 8;
         $1967 = $1966 & 255;
         $1968 = $1967&255;
         $1969 = $dp;
         $1970 = (($1969) + 2|0);
         HEAP8[$1970>>0] = $1968;
         $1971 = $x;
         $1972 = $1971&65535;
         $1973 = $1972 & 255;
         $1974 = $1973&255;
         $1975 = $dp;
         $1976 = (($1975) + 3|0);
         HEAP8[$1976>>0] = $1974;
         $1977 = $sp;
         $1978 = (($1977) + 4|0);
         $1979 = HEAP8[$1978>>0]|0;
         $1980 = $1979&255;
         $1981 = $sp;
         $1982 = (($1981) + 5|0);
         $1983 = HEAP8[$1982>>0]|0;
         $1984 = $1983&255;
         $1985 = $11;
         $1986 = $1984 >> $1985;
         $1987 = $10;
         $1988 = (($1987) + ($1986<<2)|0);
         $1989 = HEAP32[$1988>>2]|0;
         $1990 = (($1989) + ($1980<<1)|0);
         $1991 = HEAP16[$1990>>1]|0;
         $v34 = $1991;
         $1992 = $v34;
         $1993 = $1992&65535;
         $1994 = $a32;
         $1995 = $1994&65535;
         $1996 = Math_imul($1993, $1995)|0;
         $1997 = $4;
         $1998 = (($1997) + 6|0);
         $1999 = HEAP16[$1998>>1]|0;
         $2000 = $1999&65535;
         $2001 = $a32;
         $2002 = $2001&65535;
         $2003 = (65535 - ($2002))|0;
         $2004 = Math_imul($2000, $2003)|0;
         $2005 = (($1996) + ($2004))|0;
         $2006 = (($2005) + 32768)|0;
         $temp38 = $2006;
         $2007 = $temp38;
         $2008 = $temp38;
         $2009 = $2008 >>> 16;
         $2010 = (($2007) + ($2009))|0;
         $2011 = $2010 >>> 16;
         $2012 = $2011&65535;
         $w35 = $2012;
         $2013 = $w35;
         $2014 = $2013&65535;
         $2015 = $2014 >> 8;
         $2016 = $w35;
         $2017 = $2016&65535;
         $2018 = $2017 & 255;
         $2019 = $11;
         $2020 = $2018 >> $2019;
         $2021 = $9;
         $2022 = (($2021) + ($2020<<2)|0);
         $2023 = HEAP32[$2022>>2]|0;
         $2024 = (($2023) + ($2015<<1)|0);
         $2025 = HEAP16[$2024>>1]|0;
         $x = $2025;
         $2026 = $x;
         $2027 = $2026&65535;
         $2028 = $2027 >> 8;
         $2029 = $2028 & 255;
         $2030 = $2029&255;
         $2031 = $dp;
         $2032 = (($2031) + 4|0);
         HEAP8[$2032>>0] = $2030;
         $2033 = $x;
         $2034 = $2033&65535;
         $2035 = $2034 & 255;
         $2036 = $2035&255;
         $2037 = $dp;
         $2038 = (($2037) + 5|0);
         HEAP8[$2038>>0] = $2036;
        }
       }
       $2039 = $i;
       $2040 = (($2039) + 1)|0;
       $i = $2040;
       $2041 = $sp;
       $2042 = (($2041) + 8|0);
       $sp = $2042;
       $2043 = $dp;
       $2044 = (($2043) + 6|0);
       $dp = $2044;
      }
     } else {
      label = 237;
     }
    } else {
     label = 237;
    }
   } else {
    label = 237;
   }
   if ((label|0) == 237) {
    $2045 = $1;
    $sp = $2045;
    $2046 = $1;
    $dp = $2046;
    $i = 0;
    while(1) {
     $2047 = $i;
     $2048 = $row_width;
     $2049 = ($2047>>>0)<($2048>>>0);
     if (!($2049)) {
      break;
     }
     $2050 = $sp;
     $2051 = (($2050) + 6|0);
     $2052 = HEAP8[$2051>>0]|0;
     $2053 = $2052&255;
     $2054 = $2053&65535;
     $2055 = $2054 << 8;
     $2056 = $sp;
     $2057 = (($2056) + 7|0);
     $2058 = HEAP8[$2057>>0]|0;
     $2059 = $2058&255;
     $2060 = $2059&65535;
     $2061 = (($2055) + ($2060))|0;
     $2062 = $2061&65535;
     $a39 = $2062;
     $2063 = $a39;
     $2064 = $2063&65535;
     $2065 = ($2064|0)==(65535);
     if ($2065) {
      $2066 = $dp;
      $2067 = $sp;
      ;HEAP8[$2066+0>>0]=HEAP8[$2067+0>>0]|0;HEAP8[$2066+1>>0]=HEAP8[$2067+1>>0]|0;HEAP8[$2066+2>>0]=HEAP8[$2067+2>>0]|0;HEAP8[$2066+3>>0]=HEAP8[$2067+3>>0]|0;HEAP8[$2066+4>>0]=HEAP8[$2067+4>>0]|0;HEAP8[$2066+5>>0]=HEAP8[$2067+5>>0]|0;
     } else {
      $2068 = $a39;
      $2069 = $2068&65535;
      $2070 = ($2069|0)==(0);
      if ($2070) {
       $2071 = $3;
       $2072 = (($2071) + 2|0);
       $2073 = HEAP16[$2072>>1]|0;
       $2074 = $2073&65535;
       $2075 = $2074 >> 8;
       $2076 = $2075 & 255;
       $2077 = $2076&255;
       $2078 = $dp;
       HEAP8[$2078>>0] = $2077;
       $2079 = $3;
       $2080 = (($2079) + 2|0);
       $2081 = HEAP16[$2080>>1]|0;
       $2082 = $2081&65535;
       $2083 = $2082 & 255;
       $2084 = $2083&255;
       $2085 = $dp;
       $2086 = (($2085) + 1|0);
       HEAP8[$2086>>0] = $2084;
       $2087 = $3;
       $2088 = (($2087) + 4|0);
       $2089 = HEAP16[$2088>>1]|0;
       $2090 = $2089&65535;
       $2091 = $2090 >> 8;
       $2092 = $2091 & 255;
       $2093 = $2092&255;
       $2094 = $dp;
       $2095 = (($2094) + 2|0);
       HEAP8[$2095>>0] = $2093;
       $2096 = $3;
       $2097 = (($2096) + 4|0);
       $2098 = HEAP16[$2097>>1]|0;
       $2099 = $2098&65535;
       $2100 = $2099 & 255;
       $2101 = $2100&255;
       $2102 = $dp;
       $2103 = (($2102) + 3|0);
       HEAP8[$2103>>0] = $2101;
       $2104 = $3;
       $2105 = (($2104) + 6|0);
       $2106 = HEAP16[$2105>>1]|0;
       $2107 = $2106&65535;
       $2108 = $2107 >> 8;
       $2109 = $2108 & 255;
       $2110 = $2109&255;
       $2111 = $dp;
       $2112 = (($2111) + 4|0);
       HEAP8[$2112>>0] = $2110;
       $2113 = $3;
       $2114 = (($2113) + 6|0);
       $2115 = HEAP16[$2114>>1]|0;
       $2116 = $2115&65535;
       $2117 = $2116 & 255;
       $2118 = $2117&255;
       $2119 = $dp;
       $2120 = (($2119) + 5|0);
       HEAP8[$2120>>0] = $2118;
      } else {
       $2121 = $sp;
       $2122 = HEAP8[$2121>>0]|0;
       $2123 = $2122&255;
       $2124 = $2123 << 8;
       $2125 = $sp;
       $2126 = (($2125) + 1|0);
       $2127 = HEAP8[$2126>>0]|0;
       $2128 = $2127&255;
       $2129 = (($2124) + ($2128))|0;
       $2130 = $2129&65535;
       $r41 = $2130;
       $2131 = $sp;
       $2132 = (($2131) + 2|0);
       $2133 = HEAP8[$2132>>0]|0;
       $2134 = $2133&255;
       $2135 = $2134 << 8;
       $2136 = $sp;
       $2137 = (($2136) + 3|0);
       $2138 = HEAP8[$2137>>0]|0;
       $2139 = $2138&255;
       $2140 = (($2135) + ($2139))|0;
       $2141 = $2140&65535;
       $g42 = $2141;
       $2142 = $sp;
       $2143 = (($2142) + 4|0);
       $2144 = HEAP8[$2143>>0]|0;
       $2145 = $2144&255;
       $2146 = $2145 << 8;
       $2147 = $sp;
       $2148 = (($2147) + 5|0);
       $2149 = HEAP8[$2148>>0]|0;
       $2150 = $2149&255;
       $2151 = (($2146) + ($2150))|0;
       $2152 = $2151&65535;
       $b43 = $2152;
       $2153 = $r41;
       $2154 = $2153&65535;
       $2155 = $a39;
       $2156 = $2155&65535;
       $2157 = Math_imul($2154, $2156)|0;
       $2158 = $3;
       $2159 = (($2158) + 2|0);
       $2160 = HEAP16[$2159>>1]|0;
       $2161 = $2160&65535;
       $2162 = $a39;
       $2163 = $2162&65535;
       $2164 = (65535 - ($2163))|0;
       $2165 = Math_imul($2161, $2164)|0;
       $2166 = (($2157) + ($2165))|0;
       $2167 = (($2166) + 32768)|0;
       $temp44 = $2167;
       $2168 = $temp44;
       $2169 = $temp44;
       $2170 = $2169 >>> 16;
       $2171 = (($2168) + ($2170))|0;
       $2172 = $2171 >>> 16;
       $2173 = $2172&65535;
       $v40 = $2173;
       $2174 = $v40;
       $2175 = $2174&65535;
       $2176 = $2175 >> 8;
       $2177 = $2176 & 255;
       $2178 = $2177&255;
       $2179 = $dp;
       HEAP8[$2179>>0] = $2178;
       $2180 = $v40;
       $2181 = $2180&65535;
       $2182 = $2181 & 255;
       $2183 = $2182&255;
       $2184 = $dp;
       $2185 = (($2184) + 1|0);
       HEAP8[$2185>>0] = $2183;
       $2186 = $g42;
       $2187 = $2186&65535;
       $2188 = $a39;
       $2189 = $2188&65535;
       $2190 = Math_imul($2187, $2189)|0;
       $2191 = $3;
       $2192 = (($2191) + 4|0);
       $2193 = HEAP16[$2192>>1]|0;
       $2194 = $2193&65535;
       $2195 = $a39;
       $2196 = $2195&65535;
       $2197 = (65535 - ($2196))|0;
       $2198 = Math_imul($2194, $2197)|0;
       $2199 = (($2190) + ($2198))|0;
       $2200 = (($2199) + 32768)|0;
       $temp45 = $2200;
       $2201 = $temp45;
       $2202 = $temp45;
       $2203 = $2202 >>> 16;
       $2204 = (($2201) + ($2203))|0;
       $2205 = $2204 >>> 16;
       $2206 = $2205&65535;
       $v40 = $2206;
       $2207 = $v40;
       $2208 = $2207&65535;
       $2209 = $2208 >> 8;
       $2210 = $2209 & 255;
       $2211 = $2210&255;
       $2212 = $dp;
       $2213 = (($2212) + 2|0);
       HEAP8[$2213>>0] = $2211;
       $2214 = $v40;
       $2215 = $2214&65535;
       $2216 = $2215 & 255;
       $2217 = $2216&255;
       $2218 = $dp;
       $2219 = (($2218) + 3|0);
       HEAP8[$2219>>0] = $2217;
       $2220 = $b43;
       $2221 = $2220&65535;
       $2222 = $a39;
       $2223 = $2222&65535;
       $2224 = Math_imul($2221, $2223)|0;
       $2225 = $3;
       $2226 = (($2225) + 6|0);
       $2227 = HEAP16[$2226>>1]|0;
       $2228 = $2227&65535;
       $2229 = $a39;
       $2230 = $2229&65535;
       $2231 = (65535 - ($2230))|0;
       $2232 = Math_imul($2228, $2231)|0;
       $2233 = (($2224) + ($2232))|0;
       $2234 = (($2233) + 32768)|0;
       $temp46 = $2234;
       $2235 = $temp46;
       $2236 = $temp46;
       $2237 = $2236 >>> 16;
       $2238 = (($2235) + ($2237))|0;
       $2239 = $2238 >>> 16;
       $2240 = $2239&65535;
       $v40 = $2240;
       $2241 = $v40;
       $2242 = $2241&65535;
       $2243 = $2242 >> 8;
       $2244 = $2243 & 255;
       $2245 = $2244&255;
       $2246 = $dp;
       $2247 = (($2246) + 4|0);
       HEAP8[$2247>>0] = $2245;
       $2248 = $v40;
       $2249 = $2248&65535;
       $2250 = $2249 & 255;
       $2251 = $2250&255;
       $2252 = $dp;
       $2253 = (($2252) + 5|0);
       HEAP8[$2253>>0] = $2251;
      }
     }
     $2254 = $i;
     $2255 = (($2254) + 1)|0;
     $i = $2255;
     $2256 = $sp;
     $2257 = (($2256) + 8|0);
     $sp = $2257;
     $2258 = $dp;
     $2259 = (($2258) + 6|0);
     $dp = $2259;
    }
   }
  }
 }
 $2260 = $0;
 $2261 = (($2260) + 8|0);
 $2262 = HEAP8[$2261>>0]|0;
 $2263 = $2262&255;
 $2264 = $2263 & 4;
 $2265 = ($2264|0)!=(0);
 if ($2265) {
  $2266 = $0;
  $2267 = (($2266) + 8|0);
  $2268 = HEAP8[$2267>>0]|0;
  $2269 = $2268&255;
  $2270 = $2269 & -5;
  $2271 = $2270&255;
  HEAP8[$2267>>0] = $2271;
  $2272 = $0;
  $2273 = (($2272) + 10|0);
  $2274 = HEAP8[$2273>>0]|0;
  $2275 = (($2274) + -1)<<24>>24;
  HEAP8[$2273>>0] = $2275;
  $2276 = $0;
  $2277 = (($2276) + 10|0);
  $2278 = HEAP8[$2277>>0]|0;
  $2279 = $2278&255;
  $2280 = $0;
  $2281 = (($2280) + 9|0);
  $2282 = HEAP8[$2281>>0]|0;
  $2283 = $2282&255;
  $2284 = Math_imul($2279, $2283)|0;
  $2285 = $2284&255;
  $2286 = $0;
  $2287 = (($2286) + 11|0);
  HEAP8[$2287>>0] = $2285;
  $2288 = $0;
  $2289 = (($2288) + 11|0);
  $2290 = HEAP8[$2289>>0]|0;
  $2291 = $2290&255;
  $2292 = ($2291|0)>=(8);
  if ($2292) {
   $2293 = $row_width;
   $2294 = $0;
   $2295 = (($2294) + 11|0);
   $2296 = HEAP8[$2295>>0]|0;
   $2297 = $2296&255;
   $2298 = $2297 >>> 3;
   $2299 = Math_imul($2293, $2298)|0;
   $2310 = $2299;
  } else {
   $2300 = $row_width;
   $2301 = $0;
   $2302 = (($2301) + 11|0);
   $2303 = HEAP8[$2302>>0]|0;
   $2304 = $2303&255;
   $2305 = Math_imul($2300, $2304)|0;
   $2306 = (($2305) + 7)|0;
   $2307 = $2306 >>> 3;
   $2310 = $2307;
  }
  $2308 = $0;
  $2309 = (($2308) + 4|0);
  HEAP32[$2309>>2] = $2310;
 }
 STACKTOP = sp;return;
}
function _png_do_gamma($row_info,$row,$gamma_table,$gamma_16_table,$gamma_shift) {
 $row_info = $row_info|0;
 $row = $row|0;
 $gamma_table = $gamma_table|0;
 $gamma_16_table = $gamma_16_table|0;
 $gamma_shift = $gamma_shift|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $c = 0, $d = 0, $i = 0, $lsb = 0;
 var $msb = 0, $row_width = 0, $sp = 0, $v = 0, $v1 = 0, $v2 = 0, $v3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $gamma_table;
 $3 = $gamma_16_table;
 $4 = $gamma_shift;
 $5 = $0;
 $6 = HEAP32[$5>>2]|0;
 $row_width = $6;
 $7 = $0;
 $8 = (($7) + 9|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)<=(8);
 if ($11) {
  $12 = $2;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $14 = $0;
  $15 = (($14) + 9|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(16);
  if (!($18)) {
   STACKTOP = sp;return;
  }
  $19 = $3;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   STACKTOP = sp;return;
  }
 }
 $21 = $0;
 $22 = (($21) + 8|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 if ((($24|0) == 6)) {
  $153 = $0;
  $154 = (($153) + 9|0);
  $155 = HEAP8[$154>>0]|0;
  $156 = $155&255;
  $157 = ($156|0)==(8);
  if ($157) {
   $158 = $1;
   $sp = $158;
   $i = 0;
   while(1) {
    $159 = $i;
    $160 = $row_width;
    $161 = ($159>>>0)<($160>>>0);
    if (!($161)) {
     break;
    }
    $162 = $sp;
    $163 = HEAP8[$162>>0]|0;
    $164 = $163&255;
    $165 = $2;
    $166 = (($165) + ($164)|0);
    $167 = HEAP8[$166>>0]|0;
    $168 = $sp;
    HEAP8[$168>>0] = $167;
    $169 = $sp;
    $170 = (($169) + 1|0);
    $sp = $170;
    $171 = $sp;
    $172 = HEAP8[$171>>0]|0;
    $173 = $172&255;
    $174 = $2;
    $175 = (($174) + ($173)|0);
    $176 = HEAP8[$175>>0]|0;
    $177 = $sp;
    HEAP8[$177>>0] = $176;
    $178 = $sp;
    $179 = (($178) + 1|0);
    $sp = $179;
    $180 = $sp;
    $181 = HEAP8[$180>>0]|0;
    $182 = $181&255;
    $183 = $2;
    $184 = (($183) + ($182)|0);
    $185 = HEAP8[$184>>0]|0;
    $186 = $sp;
    HEAP8[$186>>0] = $185;
    $187 = $sp;
    $188 = (($187) + 1|0);
    $sp = $188;
    $189 = $sp;
    $190 = (($189) + 1|0);
    $sp = $190;
    $191 = $i;
    $192 = (($191) + 1)|0;
    $i = $192;
   }
  } else {
   $193 = $1;
   $sp = $193;
   $i = 0;
   while(1) {
    $194 = $i;
    $195 = $row_width;
    $196 = ($194>>>0)<($195>>>0);
    if (!($196)) {
     break;
    }
    $197 = $sp;
    $198 = HEAP8[$197>>0]|0;
    $199 = $198&255;
    $200 = $sp;
    $201 = (($200) + 1|0);
    $202 = HEAP8[$201>>0]|0;
    $203 = $202&255;
    $204 = $4;
    $205 = $203 >> $204;
    $206 = $3;
    $207 = (($206) + ($205<<2)|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = (($208) + ($199<<1)|0);
    $210 = HEAP16[$209>>1]|0;
    $v1 = $210;
    $211 = $v1;
    $212 = $211&65535;
    $213 = $212 >> 8;
    $214 = $213 & 255;
    $215 = $214&255;
    $216 = $sp;
    HEAP8[$216>>0] = $215;
    $217 = $v1;
    $218 = $217&65535;
    $219 = $218 & 255;
    $220 = $219&255;
    $221 = $sp;
    $222 = (($221) + 1|0);
    HEAP8[$222>>0] = $220;
    $223 = $sp;
    $224 = (($223) + 2|0);
    $sp = $224;
    $225 = $sp;
    $226 = HEAP8[$225>>0]|0;
    $227 = $226&255;
    $228 = $sp;
    $229 = (($228) + 1|0);
    $230 = HEAP8[$229>>0]|0;
    $231 = $230&255;
    $232 = $4;
    $233 = $231 >> $232;
    $234 = $3;
    $235 = (($234) + ($233<<2)|0);
    $236 = HEAP32[$235>>2]|0;
    $237 = (($236) + ($227<<1)|0);
    $238 = HEAP16[$237>>1]|0;
    $v1 = $238;
    $239 = $v1;
    $240 = $239&65535;
    $241 = $240 >> 8;
    $242 = $241 & 255;
    $243 = $242&255;
    $244 = $sp;
    HEAP8[$244>>0] = $243;
    $245 = $v1;
    $246 = $245&65535;
    $247 = $246 & 255;
    $248 = $247&255;
    $249 = $sp;
    $250 = (($249) + 1|0);
    HEAP8[$250>>0] = $248;
    $251 = $sp;
    $252 = (($251) + 2|0);
    $sp = $252;
    $253 = $sp;
    $254 = HEAP8[$253>>0]|0;
    $255 = $254&255;
    $256 = $sp;
    $257 = (($256) + 1|0);
    $258 = HEAP8[$257>>0]|0;
    $259 = $258&255;
    $260 = $4;
    $261 = $259 >> $260;
    $262 = $3;
    $263 = (($262) + ($261<<2)|0);
    $264 = HEAP32[$263>>2]|0;
    $265 = (($264) + ($255<<1)|0);
    $266 = HEAP16[$265>>1]|0;
    $v1 = $266;
    $267 = $v1;
    $268 = $267&65535;
    $269 = $268 >> 8;
    $270 = $269 & 255;
    $271 = $270&255;
    $272 = $sp;
    HEAP8[$272>>0] = $271;
    $273 = $v1;
    $274 = $273&65535;
    $275 = $274 & 255;
    $276 = $275&255;
    $277 = $sp;
    $278 = (($277) + 1|0);
    HEAP8[$278>>0] = $276;
    $279 = $sp;
    $280 = (($279) + 4|0);
    $sp = $280;
    $281 = $i;
    $282 = (($281) + 1)|0;
    $i = $282;
   }
  }
 } else if ((($24|0) == 2)) {
  $25 = $0;
  $26 = (($25) + 9|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  $29 = ($28|0)==(8);
  if ($29) {
   $30 = $1;
   $sp = $30;
   $i = 0;
   while(1) {
    $31 = $i;
    $32 = $row_width;
    $33 = ($31>>>0)<($32>>>0);
    if (!($33)) {
     break;
    }
    $34 = $sp;
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = $2;
    $38 = (($37) + ($36)|0);
    $39 = HEAP8[$38>>0]|0;
    $40 = $sp;
    HEAP8[$40>>0] = $39;
    $41 = $sp;
    $42 = (($41) + 1|0);
    $sp = $42;
    $43 = $sp;
    $44 = HEAP8[$43>>0]|0;
    $45 = $44&255;
    $46 = $2;
    $47 = (($46) + ($45)|0);
    $48 = HEAP8[$47>>0]|0;
    $49 = $sp;
    HEAP8[$49>>0] = $48;
    $50 = $sp;
    $51 = (($50) + 1|0);
    $sp = $51;
    $52 = $sp;
    $53 = HEAP8[$52>>0]|0;
    $54 = $53&255;
    $55 = $2;
    $56 = (($55) + ($54)|0);
    $57 = HEAP8[$56>>0]|0;
    $58 = $sp;
    HEAP8[$58>>0] = $57;
    $59 = $sp;
    $60 = (($59) + 1|0);
    $sp = $60;
    $61 = $i;
    $62 = (($61) + 1)|0;
    $i = $62;
   }
  } else {
   $63 = $1;
   $sp = $63;
   $i = 0;
   while(1) {
    $64 = $i;
    $65 = $row_width;
    $66 = ($64>>>0)<($65>>>0);
    if (!($66)) {
     break;
    }
    $67 = $sp;
    $68 = HEAP8[$67>>0]|0;
    $69 = $68&255;
    $70 = $sp;
    $71 = (($70) + 1|0);
    $72 = HEAP8[$71>>0]|0;
    $73 = $72&255;
    $74 = $4;
    $75 = $73 >> $74;
    $76 = $3;
    $77 = (($76) + ($75<<2)|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) + ($69<<1)|0);
    $80 = HEAP16[$79>>1]|0;
    $v = $80;
    $81 = $v;
    $82 = $81&65535;
    $83 = $82 >> 8;
    $84 = $83 & 255;
    $85 = $84&255;
    $86 = $sp;
    HEAP8[$86>>0] = $85;
    $87 = $v;
    $88 = $87&65535;
    $89 = $88 & 255;
    $90 = $89&255;
    $91 = $sp;
    $92 = (($91) + 1|0);
    HEAP8[$92>>0] = $90;
    $93 = $sp;
    $94 = (($93) + 2|0);
    $sp = $94;
    $95 = $sp;
    $96 = HEAP8[$95>>0]|0;
    $97 = $96&255;
    $98 = $sp;
    $99 = (($98) + 1|0);
    $100 = HEAP8[$99>>0]|0;
    $101 = $100&255;
    $102 = $4;
    $103 = $101 >> $102;
    $104 = $3;
    $105 = (($104) + ($103<<2)|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = (($106) + ($97<<1)|0);
    $108 = HEAP16[$107>>1]|0;
    $v = $108;
    $109 = $v;
    $110 = $109&65535;
    $111 = $110 >> 8;
    $112 = $111 & 255;
    $113 = $112&255;
    $114 = $sp;
    HEAP8[$114>>0] = $113;
    $115 = $v;
    $116 = $115&65535;
    $117 = $116 & 255;
    $118 = $117&255;
    $119 = $sp;
    $120 = (($119) + 1|0);
    HEAP8[$120>>0] = $118;
    $121 = $sp;
    $122 = (($121) + 2|0);
    $sp = $122;
    $123 = $sp;
    $124 = HEAP8[$123>>0]|0;
    $125 = $124&255;
    $126 = $sp;
    $127 = (($126) + 1|0);
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = $4;
    $131 = $129 >> $130;
    $132 = $3;
    $133 = (($132) + ($131<<2)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = (($134) + ($125<<1)|0);
    $136 = HEAP16[$135>>1]|0;
    $v = $136;
    $137 = $v;
    $138 = $137&65535;
    $139 = $138 >> 8;
    $140 = $139 & 255;
    $141 = $140&255;
    $142 = $sp;
    HEAP8[$142>>0] = $141;
    $143 = $v;
    $144 = $143&65535;
    $145 = $144 & 255;
    $146 = $145&255;
    $147 = $sp;
    $148 = (($147) + 1|0);
    HEAP8[$148>>0] = $146;
    $149 = $sp;
    $150 = (($149) + 2|0);
    $sp = $150;
    $151 = $i;
    $152 = (($151) + 1)|0;
    $i = $152;
   }
  }
 } else if ((($24|0) == 4)) {
  $283 = $0;
  $284 = (($283) + 9|0);
  $285 = HEAP8[$284>>0]|0;
  $286 = $285&255;
  $287 = ($286|0)==(8);
  if ($287) {
   $288 = $1;
   $sp = $288;
   $i = 0;
   while(1) {
    $289 = $i;
    $290 = $row_width;
    $291 = ($289>>>0)<($290>>>0);
    if (!($291)) {
     break;
    }
    $292 = $sp;
    $293 = HEAP8[$292>>0]|0;
    $294 = $293&255;
    $295 = $2;
    $296 = (($295) + ($294)|0);
    $297 = HEAP8[$296>>0]|0;
    $298 = $sp;
    HEAP8[$298>>0] = $297;
    $299 = $sp;
    $300 = (($299) + 2|0);
    $sp = $300;
    $301 = $i;
    $302 = (($301) + 1)|0;
    $i = $302;
   }
  } else {
   $303 = $1;
   $sp = $303;
   $i = 0;
   while(1) {
    $304 = $i;
    $305 = $row_width;
    $306 = ($304>>>0)<($305>>>0);
    if (!($306)) {
     break;
    }
    $307 = $sp;
    $308 = HEAP8[$307>>0]|0;
    $309 = $308&255;
    $310 = $sp;
    $311 = (($310) + 1|0);
    $312 = HEAP8[$311>>0]|0;
    $313 = $312&255;
    $314 = $4;
    $315 = $313 >> $314;
    $316 = $3;
    $317 = (($316) + ($315<<2)|0);
    $318 = HEAP32[$317>>2]|0;
    $319 = (($318) + ($309<<1)|0);
    $320 = HEAP16[$319>>1]|0;
    $v2 = $320;
    $321 = $v2;
    $322 = $321&65535;
    $323 = $322 >> 8;
    $324 = $323 & 255;
    $325 = $324&255;
    $326 = $sp;
    HEAP8[$326>>0] = $325;
    $327 = $v2;
    $328 = $327&65535;
    $329 = $328 & 255;
    $330 = $329&255;
    $331 = $sp;
    $332 = (($331) + 1|0);
    HEAP8[$332>>0] = $330;
    $333 = $sp;
    $334 = (($333) + 4|0);
    $sp = $334;
    $335 = $i;
    $336 = (($335) + 1)|0;
    $i = $336;
   }
  }
 } else if ((($24|0) == 0)) {
  $337 = $0;
  $338 = (($337) + 9|0);
  $339 = HEAP8[$338>>0]|0;
  $340 = $339&255;
  $341 = ($340|0)==(2);
  if ($341) {
   $342 = $1;
   $sp = $342;
   $i = 0;
   while(1) {
    $343 = $i;
    $344 = $row_width;
    $345 = ($343>>>0)<($344>>>0);
    if (!($345)) {
     break;
    }
    $346 = $sp;
    $347 = HEAP8[$346>>0]|0;
    $348 = $347&255;
    $349 = $348 & 192;
    $a = $349;
    $350 = $sp;
    $351 = HEAP8[$350>>0]|0;
    $352 = $351&255;
    $353 = $352 & 48;
    $b = $353;
    $354 = $sp;
    $355 = HEAP8[$354>>0]|0;
    $356 = $355&255;
    $357 = $356 & 12;
    $c = $357;
    $358 = $sp;
    $359 = HEAP8[$358>>0]|0;
    $360 = $359&255;
    $361 = $360 & 3;
    $d = $361;
    $362 = $a;
    $363 = $a;
    $364 = $363 >> 2;
    $365 = $362 | $364;
    $366 = $a;
    $367 = $366 >> 4;
    $368 = $365 | $367;
    $369 = $a;
    $370 = $369 >> 6;
    $371 = $368 | $370;
    $372 = $2;
    $373 = (($372) + ($371)|0);
    $374 = HEAP8[$373>>0]|0;
    $375 = $374&255;
    $376 = $375 & 192;
    $377 = $b;
    $378 = $377 << 2;
    $379 = $b;
    $380 = $378 | $379;
    $381 = $b;
    $382 = $381 >> 2;
    $383 = $380 | $382;
    $384 = $b;
    $385 = $384 >> 4;
    $386 = $383 | $385;
    $387 = $2;
    $388 = (($387) + ($386)|0);
    $389 = HEAP8[$388>>0]|0;
    $390 = $389&255;
    $391 = $390 >> 2;
    $392 = $391 & 48;
    $393 = $376 | $392;
    $394 = $c;
    $395 = $394 << 4;
    $396 = $c;
    $397 = $396 << 2;
    $398 = $395 | $397;
    $399 = $c;
    $400 = $398 | $399;
    $401 = $c;
    $402 = $401 >> 2;
    $403 = $400 | $402;
    $404 = $2;
    $405 = (($404) + ($403)|0);
    $406 = HEAP8[$405>>0]|0;
    $407 = $406&255;
    $408 = $407 >> 4;
    $409 = $408 & 12;
    $410 = $393 | $409;
    $411 = $d;
    $412 = $411 << 6;
    $413 = $d;
    $414 = $413 << 4;
    $415 = $412 | $414;
    $416 = $d;
    $417 = $416 << 2;
    $418 = $415 | $417;
    $419 = $d;
    $420 = $418 | $419;
    $421 = $2;
    $422 = (($421) + ($420)|0);
    $423 = HEAP8[$422>>0]|0;
    $424 = $423&255;
    $425 = $424 >> 6;
    $426 = $410 | $425;
    $427 = $426&255;
    $428 = $sp;
    HEAP8[$428>>0] = $427;
    $429 = $sp;
    $430 = (($429) + 1|0);
    $sp = $430;
    $431 = $i;
    $432 = (($431) + 4)|0;
    $i = $432;
   }
  }
  $433 = $0;
  $434 = (($433) + 9|0);
  $435 = HEAP8[$434>>0]|0;
  $436 = $435&255;
  $437 = ($436|0)==(4);
  if ($437) {
   $438 = $1;
   $sp = $438;
   $i = 0;
   while(1) {
    $439 = $i;
    $440 = $row_width;
    $441 = ($439>>>0)<($440>>>0);
    if (!($441)) {
     break;
    }
    $442 = $sp;
    $443 = HEAP8[$442>>0]|0;
    $444 = $443&255;
    $445 = $444 & 240;
    $msb = $445;
    $446 = $sp;
    $447 = HEAP8[$446>>0]|0;
    $448 = $447&255;
    $449 = $448 & 15;
    $lsb = $449;
    $450 = $msb;
    $451 = $msb;
    $452 = $451 >> 4;
    $453 = $450 | $452;
    $454 = $2;
    $455 = (($454) + ($453)|0);
    $456 = HEAP8[$455>>0]|0;
    $457 = $456&255;
    $458 = $457 & 240;
    $459 = $lsb;
    $460 = $459 << 4;
    $461 = $lsb;
    $462 = $460 | $461;
    $463 = $2;
    $464 = (($463) + ($462)|0);
    $465 = HEAP8[$464>>0]|0;
    $466 = $465&255;
    $467 = $466 >> 4;
    $468 = $458 | $467;
    $469 = $468&255;
    $470 = $sp;
    HEAP8[$470>>0] = $469;
    $471 = $sp;
    $472 = (($471) + 1|0);
    $sp = $472;
    $473 = $i;
    $474 = (($473) + 2)|0;
    $i = $474;
   }
  } else {
   $475 = $0;
   $476 = (($475) + 9|0);
   $477 = HEAP8[$476>>0]|0;
   $478 = $477&255;
   $479 = ($478|0)==(8);
   if ($479) {
    $480 = $1;
    $sp = $480;
    $i = 0;
    while(1) {
     $481 = $i;
     $482 = $row_width;
     $483 = ($481>>>0)<($482>>>0);
     if (!($483)) {
      break;
     }
     $484 = $sp;
     $485 = HEAP8[$484>>0]|0;
     $486 = $485&255;
     $487 = $2;
     $488 = (($487) + ($486)|0);
     $489 = HEAP8[$488>>0]|0;
     $490 = $sp;
     HEAP8[$490>>0] = $489;
     $491 = $sp;
     $492 = (($491) + 1|0);
     $sp = $492;
     $493 = $i;
     $494 = (($493) + 1)|0;
     $i = $494;
    }
   } else {
    $495 = $0;
    $496 = (($495) + 9|0);
    $497 = HEAP8[$496>>0]|0;
    $498 = $497&255;
    $499 = ($498|0)==(16);
    if ($499) {
     $500 = $1;
     $sp = $500;
     $i = 0;
     while(1) {
      $501 = $i;
      $502 = $row_width;
      $503 = ($501>>>0)<($502>>>0);
      if (!($503)) {
       break;
      }
      $504 = $sp;
      $505 = HEAP8[$504>>0]|0;
      $506 = $505&255;
      $507 = $sp;
      $508 = (($507) + 1|0);
      $509 = HEAP8[$508>>0]|0;
      $510 = $509&255;
      $511 = $4;
      $512 = $510 >> $511;
      $513 = $3;
      $514 = (($513) + ($512<<2)|0);
      $515 = HEAP32[$514>>2]|0;
      $516 = (($515) + ($506<<1)|0);
      $517 = HEAP16[$516>>1]|0;
      $v3 = $517;
      $518 = $v3;
      $519 = $518&65535;
      $520 = $519 >> 8;
      $521 = $520 & 255;
      $522 = $521&255;
      $523 = $sp;
      HEAP8[$523>>0] = $522;
      $524 = $v3;
      $525 = $524&65535;
      $526 = $525 & 255;
      $527 = $526&255;
      $528 = $sp;
      $529 = (($528) + 1|0);
      HEAP8[$529>>0] = $527;
      $530 = $sp;
      $531 = (($530) + 2|0);
      $sp = $531;
      $532 = $i;
      $533 = (($532) + 1)|0;
      $i = $533;
     }
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_do_chop($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dp = 0, $i = 0, $istop = 0, $sp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 9|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(16);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $1;
 $sp = $7;
 $8 = $1;
 $dp = $8;
 $9 = $0;
 $10 = HEAP32[$9>>2]|0;
 $11 = $0;
 $12 = (($11) + 10|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = Math_imul($10, $14)|0;
 $istop = $15;
 $i = 0;
 while(1) {
  $16 = $i;
  $17 = $istop;
  $18 = ($16>>>0)<($17>>>0);
  if (!($18)) {
   break;
  }
  $19 = $sp;
  $20 = HEAP8[$19>>0]|0;
  $21 = $dp;
  HEAP8[$21>>0] = $20;
  $22 = $i;
  $23 = (($22) + 1)|0;
  $i = $23;
  $24 = $sp;
  $25 = (($24) + 2|0);
  $sp = $25;
  $26 = $dp;
  $27 = (($26) + 1|0);
  $dp = $27;
 }
 $28 = $0;
 $29 = (($28) + 9|0);
 HEAP8[$29>>0] = 8;
 $30 = $0;
 $31 = (($30) + 10|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = $33<<3;
 $35 = $34&255;
 $36 = $0;
 $37 = (($36) + 11|0);
 HEAP8[$37>>0] = $35;
 $38 = $0;
 $39 = HEAP32[$38>>2]|0;
 $40 = $0;
 $41 = (($40) + 10|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = Math_imul($39, $43)|0;
 $45 = $0;
 $46 = (($45) + 4|0);
 HEAP32[$46>>2] = $44;
 STACKTOP = sp;return;
}
function _png_do_dither($row_info,$row,$palette_lookup,$dither_lookup) {
 $row_info = $row_info|0;
 $row = $row|0;
 $palette_lookup = $palette_lookup|0;
 $dither_lookup = $dither_lookup|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $b = 0;
 var $b3 = 0, $dp = 0, $g = 0, $g2 = 0, $i = 0, $p = 0, $p4 = 0, $r = 0, $r1 = 0, $row_width = 0, $sp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $palette_lookup;
 $3 = $dither_lookup;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $row_width = $5;
 $6 = $0;
 $7 = (($6) + 8|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(2);
 if ($10) {
  $11 = $2;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = $0;
   $14 = (($13) + 9|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)==(8);
   if ($17) {
    $18 = $1;
    $sp = $18;
    $19 = $1;
    $dp = $19;
    $i = 0;
    while(1) {
     $20 = $i;
     $21 = $row_width;
     $22 = ($20>>>0)<($21>>>0);
     if (!($22)) {
      break;
     }
     $23 = $sp;
     $24 = (($23) + 1|0);
     $sp = $24;
     $25 = HEAP8[$23>>0]|0;
     $26 = $25&255;
     $r = $26;
     $27 = $sp;
     $28 = (($27) + 1|0);
     $sp = $28;
     $29 = HEAP8[$27>>0]|0;
     $30 = $29&255;
     $g = $30;
     $31 = $sp;
     $32 = (($31) + 1|0);
     $sp = $32;
     $33 = HEAP8[$31>>0]|0;
     $34 = $33&255;
     $b = $34;
     $35 = $r;
     $36 = $35 >> 3;
     $37 = $36 & 31;
     $38 = $37 << 10;
     $39 = $g;
     $40 = $39 >> 3;
     $41 = $40 & 31;
     $42 = $41 << 5;
     $43 = $38 | $42;
     $44 = $b;
     $45 = $44 >> 3;
     $46 = $45 & 31;
     $47 = $43 | $46;
     $p = $47;
     $48 = $p;
     $49 = $2;
     $50 = (($49) + ($48)|0);
     $51 = HEAP8[$50>>0]|0;
     $52 = $dp;
     $53 = (($52) + 1|0);
     $dp = $53;
     HEAP8[$52>>0] = $51;
     $54 = $i;
     $55 = (($54) + 1)|0;
     $i = $55;
    }
    $56 = $0;
    $57 = (($56) + 8|0);
    HEAP8[$57>>0] = 3;
    $58 = $0;
    $59 = (($58) + 10|0);
    HEAP8[$59>>0] = 1;
    $60 = $0;
    $61 = (($60) + 9|0);
    $62 = HEAP8[$61>>0]|0;
    $63 = $0;
    $64 = (($63) + 11|0);
    HEAP8[$64>>0] = $62;
    $65 = $0;
    $66 = (($65) + 11|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = ($68|0)>=(8);
    if ($69) {
     $70 = $row_width;
     $71 = $0;
     $72 = (($71) + 11|0);
     $73 = HEAP8[$72>>0]|0;
     $74 = $73&255;
     $75 = $74 >>> 3;
     $76 = Math_imul($70, $75)|0;
     $87 = $76;
    } else {
     $77 = $row_width;
     $78 = $0;
     $79 = (($78) + 11|0);
     $80 = HEAP8[$79>>0]|0;
     $81 = $80&255;
     $82 = Math_imul($77, $81)|0;
     $83 = (($82) + 7)|0;
     $84 = $83 >>> 3;
     $87 = $84;
    }
    $85 = $0;
    $86 = (($85) + 4|0);
    HEAP32[$86>>2] = $87;
    STACKTOP = sp;return;
   }
  }
 }
 $88 = $0;
 $89 = (($88) + 8|0);
 $90 = HEAP8[$89>>0]|0;
 $91 = $90&255;
 $92 = ($91|0)==(6);
 if ($92) {
  $93 = $2;
  $94 = ($93|0)!=(0|0);
  if ($94) {
   $95 = $0;
   $96 = (($95) + 9|0);
   $97 = HEAP8[$96>>0]|0;
   $98 = $97&255;
   $99 = ($98|0)==(8);
   if ($99) {
    $100 = $1;
    $sp = $100;
    $101 = $1;
    $dp = $101;
    $i = 0;
    while(1) {
     $102 = $i;
     $103 = $row_width;
     $104 = ($102>>>0)<($103>>>0);
     if (!($104)) {
      break;
     }
     $105 = $sp;
     $106 = (($105) + 1|0);
     $sp = $106;
     $107 = HEAP8[$105>>0]|0;
     $108 = $107&255;
     $r1 = $108;
     $109 = $sp;
     $110 = (($109) + 1|0);
     $sp = $110;
     $111 = HEAP8[$109>>0]|0;
     $112 = $111&255;
     $g2 = $112;
     $113 = $sp;
     $114 = (($113) + 1|0);
     $sp = $114;
     $115 = HEAP8[$113>>0]|0;
     $116 = $115&255;
     $b3 = $116;
     $117 = $sp;
     $118 = (($117) + 1|0);
     $sp = $118;
     $119 = $r1;
     $120 = $119 >> 3;
     $121 = $120 & 31;
     $122 = $121 << 10;
     $123 = $g2;
     $124 = $123 >> 3;
     $125 = $124 & 31;
     $126 = $125 << 5;
     $127 = $122 | $126;
     $128 = $b3;
     $129 = $128 >> 3;
     $130 = $129 & 31;
     $131 = $127 | $130;
     $p4 = $131;
     $132 = $p4;
     $133 = $2;
     $134 = (($133) + ($132)|0);
     $135 = HEAP8[$134>>0]|0;
     $136 = $dp;
     $137 = (($136) + 1|0);
     $dp = $137;
     HEAP8[$136>>0] = $135;
     $138 = $i;
     $139 = (($138) + 1)|0;
     $i = $139;
    }
    $140 = $0;
    $141 = (($140) + 8|0);
    HEAP8[$141>>0] = 3;
    $142 = $0;
    $143 = (($142) + 10|0);
    HEAP8[$143>>0] = 1;
    $144 = $0;
    $145 = (($144) + 9|0);
    $146 = HEAP8[$145>>0]|0;
    $147 = $0;
    $148 = (($147) + 11|0);
    HEAP8[$148>>0] = $146;
    $149 = $0;
    $150 = (($149) + 11|0);
    $151 = HEAP8[$150>>0]|0;
    $152 = $151&255;
    $153 = ($152|0)>=(8);
    if ($153) {
     $154 = $row_width;
     $155 = $0;
     $156 = (($155) + 11|0);
     $157 = HEAP8[$156>>0]|0;
     $158 = $157&255;
     $159 = $158 >>> 3;
     $160 = Math_imul($154, $159)|0;
     $171 = $160;
    } else {
     $161 = $row_width;
     $162 = $0;
     $163 = (($162) + 11|0);
     $164 = HEAP8[$163>>0]|0;
     $165 = $164&255;
     $166 = Math_imul($161, $165)|0;
     $167 = (($166) + 7)|0;
     $168 = $167 >>> 3;
     $171 = $168;
    }
    $169 = $0;
    $170 = (($169) + 4|0);
    HEAP32[$170>>2] = $171;
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
 } else {
  label = 23;
 }
 if ((label|0) == 23) {
  $172 = $0;
  $173 = (($172) + 8|0);
  $174 = HEAP8[$173>>0]|0;
  $175 = $174&255;
  $176 = ($175|0)==(3);
  if ($176) {
   $177 = $3;
   $178 = ($177|0)!=(0|0);
   if ($178) {
    $179 = $0;
    $180 = (($179) + 9|0);
    $181 = HEAP8[$180>>0]|0;
    $182 = $181&255;
    $183 = ($182|0)==(8);
    if ($183) {
     $184 = $1;
     $sp = $184;
     $i = 0;
     while(1) {
      $185 = $i;
      $186 = $row_width;
      $187 = ($185>>>0)<($186>>>0);
      if (!($187)) {
       break;
      }
      $188 = $sp;
      $189 = HEAP8[$188>>0]|0;
      $190 = $189&255;
      $191 = $3;
      $192 = (($191) + ($190)|0);
      $193 = HEAP8[$192>>0]|0;
      $194 = $sp;
      HEAP8[$194>>0] = $193;
      $195 = $i;
      $196 = (($195) + 1)|0;
      $i = $196;
      $197 = $sp;
      $198 = (($197) + 1|0);
      $sp = $198;
     }
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_do_unshift($row_info,$row,$sig_bits) {
 $row_info = $row_info|0;
 $row = $row|0;
 $sig_bits = $sig_bits|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bp = 0, $bp1 = 0, $bp4 = 0, $bp7 = 0, $c = 0, $channels = 0, $i = 0, $i2 = 0, $i5 = 0, $i8 = 0;
 var $istop = 0, $istop3 = 0, $istop6 = 0, $istop9 = 0, $mask = 0, $row_width = 0, $shift = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $shift = sp + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $sig_bits;
 $3 = $0;
 $4 = (($3) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(3);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $channels = 0;
 $value = 0;
 $8 = $0;
 $9 = HEAP32[$8>>2]|0;
 $row_width = $9;
 $10 = $0;
 $11 = (($10) + 8|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 2;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $0;
  $17 = (($16) + 9|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = $2;
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (($19) - ($22))|0;
  $24 = $channels;
  $25 = (($24) + 1)|0;
  $channels = $25;
  $26 = (($shift) + ($24<<2)|0);
  HEAP32[$26>>2] = $23;
  $27 = $0;
  $28 = (($27) + 9|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29&255;
  $31 = $2;
  $32 = (($31) + 1|0);
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = (($30) - ($34))|0;
  $36 = $channels;
  $37 = (($36) + 1)|0;
  $channels = $37;
  $38 = (($shift) + ($36<<2)|0);
  HEAP32[$38>>2] = $35;
  $39 = $0;
  $40 = (($39) + 9|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41&255;
  $43 = $2;
  $44 = (($43) + 2|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = (($42) - ($46))|0;
  $48 = $channels;
  $49 = (($48) + 1)|0;
  $channels = $49;
  $50 = (($shift) + ($48<<2)|0);
  HEAP32[$50>>2] = $47;
 } else {
  $51 = $0;
  $52 = (($51) + 9|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $2;
  $56 = (($55) + 3|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = $57&255;
  $59 = (($54) - ($58))|0;
  $60 = $channels;
  $61 = (($60) + 1)|0;
  $channels = $61;
  $62 = (($shift) + ($60<<2)|0);
  HEAP32[$62>>2] = $59;
 }
 $63 = $0;
 $64 = (($63) + 8|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = $66 & 4;
 $68 = ($67|0)!=(0);
 if ($68) {
  $69 = $0;
  $70 = (($69) + 9|0);
  $71 = HEAP8[$70>>0]|0;
  $72 = $71&255;
  $73 = $2;
  $74 = (($73) + 4|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  $77 = (($72) - ($76))|0;
  $78 = $channels;
  $79 = (($78) + 1)|0;
  $channels = $79;
  $80 = (($shift) + ($78<<2)|0);
  HEAP32[$80>>2] = $77;
 }
 $c = 0;
 while(1) {
  $81 = $c;
  $82 = $channels;
  $83 = ($81|0)<($82|0);
  if (!($83)) {
   break;
  }
  $84 = $c;
  $85 = (($shift) + ($84<<2)|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = ($86|0)<=(0);
  if ($87) {
   $88 = $c;
   $89 = (($shift) + ($88<<2)|0);
   HEAP32[$89>>2] = 0;
  } else {
   $value = 1;
  }
  $90 = $c;
  $91 = (($90) + 1)|0;
  $c = $91;
 }
 $92 = $value;
 $93 = ($92<<16>>16)!=(0);
 if (!($93)) {
  STACKTOP = sp;return;
 }
 $94 = $0;
 $95 = (($94) + 9|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = $96&255;
 if ((($97|0) == 2)) {
  $98 = $0;
  $99 = (($98) + 4|0);
  $100 = HEAP32[$99>>2]|0;
  $istop = $100;
  $101 = $1;
  $bp = $101;
  $i = 0;
  while(1) {
   $102 = $i;
   $103 = $istop;
   $104 = ($102>>>0)<($103>>>0);
   if (!($104)) {
    break;
   }
   $105 = $bp;
   $106 = HEAP8[$105>>0]|0;
   $107 = $106&255;
   $108 = $107 >> 1;
   $109 = $108&255;
   HEAP8[$105>>0] = $109;
   $110 = $bp;
   $111 = (($110) + 1|0);
   $bp = $111;
   $112 = HEAP8[$110>>0]|0;
   $113 = $112&255;
   $114 = $113 & 85;
   $115 = $114&255;
   HEAP8[$110>>0] = $115;
   $116 = $i;
   $117 = (($116) + 1)|0;
   $i = $117;
  }
 } else if ((($97|0) == 16)) {
  $170 = $1;
  $bp7 = $170;
  $171 = $channels;
  $172 = $row_width;
  $173 = Math_imul($171, $172)|0;
  $istop9 = $173;
  $i8 = 0;
  while(1) {
   $174 = $i8;
   $175 = $istop9;
   $176 = ($174>>>0)<($175>>>0);
   if (!($176)) {
    break;
   }
   $177 = $bp7;
   $178 = HEAP8[$177>>0]|0;
   $179 = $178&255;
   $180 = $179 << 8;
   $181 = $bp7;
   $182 = (($181) + 1|0);
   $183 = HEAP8[$182>>0]|0;
   $184 = $183&255;
   $185 = (($180) + ($184))|0;
   $186 = $185&65535;
   $value = $186;
   $187 = $i8;
   $188 = $channels;
   $189 = (($187>>>0) % ($188>>>0))&-1;
   $190 = (($shift) + ($189<<2)|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = $value;
   $193 = $192&65535;
   $194 = $193 >> $191;
   $195 = $194&65535;
   $value = $195;
   $196 = $value;
   $197 = $196&65535;
   $198 = $197 >> 8;
   $199 = $198&255;
   $200 = $bp7;
   $201 = (($200) + 1|0);
   $bp7 = $201;
   HEAP8[$200>>0] = $199;
   $202 = $value;
   $203 = $202&65535;
   $204 = $203 & 255;
   $205 = $204&255;
   $206 = $bp7;
   $207 = (($206) + 1|0);
   $bp7 = $207;
   HEAP8[$206>>0] = $205;
   $208 = $i8;
   $209 = (($208) + 1)|0;
   $i8 = $209;
  }
 } else if ((($97|0) == 4)) {
  $118 = $1;
  $bp1 = $118;
  $119 = $0;
  $120 = (($119) + 4|0);
  $121 = HEAP32[$120>>2]|0;
  $istop3 = $121;
  $122 = HEAP32[$shift>>2]|0;
  $123 = 240 >> $122;
  $124 = $123 & 240;
  $125 = HEAP32[$shift>>2]|0;
  $126 = 15 >> $125;
  $127 = $126&255;
  $128 = $127&255;
  $129 = $124 | $128;
  $130 = $129&255;
  $mask = $130;
  $i2 = 0;
  while(1) {
   $131 = $i2;
   $132 = $istop3;
   $133 = ($131>>>0)<($132>>>0);
   if (!($133)) {
    break;
   }
   $134 = HEAP32[$shift>>2]|0;
   $135 = $bp1;
   $136 = HEAP8[$135>>0]|0;
   $137 = $136&255;
   $138 = $137 >> $134;
   $139 = $138&255;
   HEAP8[$135>>0] = $139;
   $140 = $mask;
   $141 = $140&255;
   $142 = $bp1;
   $143 = (($142) + 1|0);
   $bp1 = $143;
   $144 = HEAP8[$142>>0]|0;
   $145 = $144&255;
   $146 = $145 & $141;
   $147 = $146&255;
   HEAP8[$142>>0] = $147;
   $148 = $i2;
   $149 = (($148) + 1)|0;
   $i2 = $149;
  }
 } else if ((($97|0) == 8)) {
  $150 = $1;
  $bp4 = $150;
  $151 = $row_width;
  $152 = $channels;
  $153 = Math_imul($151, $152)|0;
  $istop6 = $153;
  $i5 = 0;
  while(1) {
   $154 = $i5;
   $155 = $istop6;
   $156 = ($154>>>0)<($155>>>0);
   if (!($156)) {
    break;
   }
   $157 = $i5;
   $158 = $channels;
   $159 = (($157>>>0) % ($158>>>0))&-1;
   $160 = (($shift) + ($159<<2)|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = $bp4;
   $163 = (($162) + 1|0);
   $bp4 = $163;
   $164 = HEAP8[$162>>0]|0;
   $165 = $164&255;
   $166 = $165 >> $161;
   $167 = $166&255;
   HEAP8[$162>>0] = $167;
   $168 = $i5;
   $169 = (($168) + 1)|0;
   $i5 = $169;
  }
 }
 STACKTOP = sp;return;
}
function _png_do_unpack($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp2 = 0, $dp5 = 0, $i = 0, $row_width = 0, $shift = 0, $shift3 = 0;
 var $shift6 = 0, $sp = 0, $sp1 = 0, $sp4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 9|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)<(8);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $0;
 $8 = HEAP32[$7>>2]|0;
 $row_width = $8;
 $9 = $0;
 $10 = (($9) + 9|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 if ((($12|0) == 1)) {
  $13 = $1;
  $14 = $row_width;
  $15 = (($14) - 1)|0;
  $16 = $15 >>> 3;
  $17 = (($13) + ($16)|0);
  $sp = $17;
  $18 = $1;
  $19 = $row_width;
  $20 = (($18) + ($19)|0);
  $21 = (($20) + -1|0);
  $dp = $21;
  $22 = $row_width;
  $23 = (($22) + 7)|0;
  $24 = $23 & 7;
  $25 = (7 - ($24))|0;
  $shift = $25;
  $i = 0;
  while(1) {
   $26 = $i;
   $27 = $row_width;
   $28 = ($26>>>0)<($27>>>0);
   if (!($28)) {
    break;
   }
   $29 = $sp;
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = $shift;
   $33 = $31 >> $32;
   $34 = $33 & 1;
   $35 = $34&255;
   $36 = $dp;
   HEAP8[$36>>0] = $35;
   $37 = $shift;
   $38 = ($37|0)==(7);
   if ($38) {
    $shift = 0;
    $39 = $sp;
    $40 = (($39) + -1|0);
    $sp = $40;
   } else {
    $41 = $shift;
    $42 = (($41) + 1)|0;
    $shift = $42;
   }
   $43 = $dp;
   $44 = (($43) + -1|0);
   $dp = $44;
   $45 = $i;
   $46 = (($45) + 1)|0;
   $i = $46;
  }
 } else if ((($12|0) == 2)) {
  $47 = $1;
  $48 = $row_width;
  $49 = (($48) - 1)|0;
  $50 = $49 >>> 2;
  $51 = (($47) + ($50)|0);
  $sp1 = $51;
  $52 = $1;
  $53 = $row_width;
  $54 = (($52) + ($53)|0);
  $55 = (($54) + -1|0);
  $dp2 = $55;
  $56 = $row_width;
  $57 = (($56) + 3)|0;
  $58 = $57 & 3;
  $59 = (3 - ($58))|0;
  $60 = $59 << 1;
  $shift3 = $60;
  $i = 0;
  while(1) {
   $61 = $i;
   $62 = $row_width;
   $63 = ($61>>>0)<($62>>>0);
   if (!($63)) {
    break;
   }
   $64 = $sp1;
   $65 = HEAP8[$64>>0]|0;
   $66 = $65&255;
   $67 = $shift3;
   $68 = $66 >> $67;
   $69 = $68 & 3;
   $70 = $69&255;
   $71 = $dp2;
   HEAP8[$71>>0] = $70;
   $72 = $shift3;
   $73 = ($72|0)==(6);
   if ($73) {
    $shift3 = 0;
    $74 = $sp1;
    $75 = (($74) + -1|0);
    $sp1 = $75;
   } else {
    $76 = $shift3;
    $77 = (($76) + 2)|0;
    $shift3 = $77;
   }
   $78 = $dp2;
   $79 = (($78) + -1|0);
   $dp2 = $79;
   $80 = $i;
   $81 = (($80) + 1)|0;
   $i = $81;
  }
 } else if ((($12|0) == 4)) {
  $82 = $1;
  $83 = $row_width;
  $84 = (($83) - 1)|0;
  $85 = $84 >>> 1;
  $86 = (($82) + ($85)|0);
  $sp4 = $86;
  $87 = $1;
  $88 = $row_width;
  $89 = (($87) + ($88)|0);
  $90 = (($89) + -1|0);
  $dp5 = $90;
  $91 = $row_width;
  $92 = (($91) + 1)|0;
  $93 = $92 & 1;
  $94 = (1 - ($93))|0;
  $95 = $94 << 2;
  $shift6 = $95;
  $i = 0;
  while(1) {
   $96 = $i;
   $97 = $row_width;
   $98 = ($96>>>0)<($97>>>0);
   if (!($98)) {
    break;
   }
   $99 = $sp4;
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = $shift6;
   $103 = $101 >> $102;
   $104 = $103 & 15;
   $105 = $104&255;
   $106 = $dp5;
   HEAP8[$106>>0] = $105;
   $107 = $shift6;
   $108 = ($107|0)==(4);
   if ($108) {
    $shift6 = 0;
    $109 = $sp4;
    $110 = (($109) + -1|0);
    $sp4 = $110;
   } else {
    $shift6 = 4;
   }
   $111 = $dp5;
   $112 = (($111) + -1|0);
   $dp5 = $112;
   $113 = $i;
   $114 = (($113) + 1)|0;
   $i = $114;
  }
 }
 $115 = $0;
 $116 = (($115) + 9|0);
 HEAP8[$116>>0] = 8;
 $117 = $0;
 $118 = (($117) + 10|0);
 $119 = HEAP8[$118>>0]|0;
 $120 = $119&255;
 $121 = $120<<3;
 $122 = $121&255;
 $123 = $0;
 $124 = (($123) + 11|0);
 HEAP8[$124>>0] = $122;
 $125 = $row_width;
 $126 = $0;
 $127 = (($126) + 10|0);
 $128 = HEAP8[$127>>0]|0;
 $129 = $128&255;
 $130 = Math_imul($125, $129)|0;
 $131 = $0;
 $132 = (($131) + 4|0);
 HEAP32[$132>>2] = $130;
 STACKTOP = sp;return;
}
function _png_do_read_filler($row_info,$row,$filler,$flags) {
 $row_info = $row_info|0;
 $row = $row|0;
 $filler = $filler|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp10 = 0, $dp12 = 0, $dp14 = 0, $dp2 = 0, $dp4 = 0, $dp6 = 0, $dp8 = 0, $hi_filler = 0;
 var $i = 0, $lo_filler = 0, $row_width = 0, $sp = 0, $sp1 = 0, $sp11 = 0, $sp13 = 0, $sp3 = 0, $sp5 = 0, $sp7 = 0, $sp9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $filler;
 $3 = $flags;
 $4 = $0;
 $5 = HEAP32[$4>>2]|0;
 $row_width = $5;
 $6 = $2;
 $7 = $6 >>> 8;
 $8 = $7 & 255;
 $9 = $8&255;
 $hi_filler = $9;
 $10 = $2;
 $11 = $10 & 255;
 $12 = $11&255;
 $lo_filler = $12;
 $13 = $0;
 $14 = (($13) + 8|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = $0;
  $19 = (($18) + 9|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = ($21|0)==(8);
  if ($22) {
   $23 = $3;
   $24 = $23 & 128;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $1;
    $27 = $row_width;
    $28 = (($26) + ($27)|0);
    $sp = $28;
    $29 = $sp;
    $30 = $row_width;
    $31 = (($29) + ($30)|0);
    $dp = $31;
    $i = 1;
    while(1) {
     $32 = $i;
     $33 = $row_width;
     $34 = ($32>>>0)<($33>>>0);
     if (!($34)) {
      break;
     }
     $35 = $lo_filler;
     $36 = $dp;
     $37 = (($36) + -1|0);
     $dp = $37;
     HEAP8[$37>>0] = $35;
     $38 = $sp;
     $39 = (($38) + -1|0);
     $sp = $39;
     $40 = HEAP8[$39>>0]|0;
     $41 = $dp;
     $42 = (($41) + -1|0);
     $dp = $42;
     HEAP8[$42>>0] = $40;
     $43 = $i;
     $44 = (($43) + 1)|0;
     $i = $44;
    }
    $45 = $lo_filler;
    $46 = $dp;
    $47 = (($46) + -1|0);
    $dp = $47;
    HEAP8[$47>>0] = $45;
    $48 = $0;
    $49 = (($48) + 10|0);
    HEAP8[$49>>0] = 2;
    $50 = $0;
    $51 = (($50) + 11|0);
    HEAP8[$51>>0] = 16;
    $52 = $row_width;
    $53 = $52<<1;
    $54 = $0;
    $55 = (($54) + 4|0);
    HEAP32[$55>>2] = $53;
   } else {
    $56 = $1;
    $57 = $row_width;
    $58 = (($56) + ($57)|0);
    $sp1 = $58;
    $59 = $sp1;
    $60 = $row_width;
    $61 = (($59) + ($60)|0);
    $dp2 = $61;
    $i = 0;
    while(1) {
     $62 = $i;
     $63 = $row_width;
     $64 = ($62>>>0)<($63>>>0);
     if (!($64)) {
      break;
     }
     $65 = $sp1;
     $66 = (($65) + -1|0);
     $sp1 = $66;
     $67 = HEAP8[$66>>0]|0;
     $68 = $dp2;
     $69 = (($68) + -1|0);
     $dp2 = $69;
     HEAP8[$69>>0] = $67;
     $70 = $lo_filler;
     $71 = $dp2;
     $72 = (($71) + -1|0);
     $dp2 = $72;
     HEAP8[$72>>0] = $70;
     $73 = $i;
     $74 = (($73) + 1)|0;
     $i = $74;
    }
    $75 = $0;
    $76 = (($75) + 10|0);
    HEAP8[$76>>0] = 2;
    $77 = $0;
    $78 = (($77) + 11|0);
    HEAP8[$78>>0] = 16;
    $79 = $row_width;
    $80 = $79<<1;
    $81 = $0;
    $82 = (($81) + 4|0);
    HEAP32[$82>>2] = $80;
   }
  } else {
   $83 = $0;
   $84 = (($83) + 9|0);
   $85 = HEAP8[$84>>0]|0;
   $86 = $85&255;
   $87 = ($86|0)==(16);
   if ($87) {
    $88 = $3;
    $89 = $88 & 128;
    $90 = ($89|0)!=(0);
    if ($90) {
     $91 = $1;
     $92 = $row_width;
     $93 = $92<<1;
     $94 = (($91) + ($93)|0);
     $sp3 = $94;
     $95 = $sp3;
     $96 = $row_width;
     $97 = $96<<1;
     $98 = (($95) + ($97)|0);
     $dp4 = $98;
     $i = 1;
     while(1) {
      $99 = $i;
      $100 = $row_width;
      $101 = ($99>>>0)<($100>>>0);
      if (!($101)) {
       break;
      }
      $102 = $hi_filler;
      $103 = $dp4;
      $104 = (($103) + -1|0);
      $dp4 = $104;
      HEAP8[$104>>0] = $102;
      $105 = $lo_filler;
      $106 = $dp4;
      $107 = (($106) + -1|0);
      $dp4 = $107;
      HEAP8[$107>>0] = $105;
      $108 = $sp3;
      $109 = (($108) + -1|0);
      $sp3 = $109;
      $110 = HEAP8[$109>>0]|0;
      $111 = $dp4;
      $112 = (($111) + -1|0);
      $dp4 = $112;
      HEAP8[$112>>0] = $110;
      $113 = $sp3;
      $114 = (($113) + -1|0);
      $sp3 = $114;
      $115 = HEAP8[$114>>0]|0;
      $116 = $dp4;
      $117 = (($116) + -1|0);
      $dp4 = $117;
      HEAP8[$117>>0] = $115;
      $118 = $i;
      $119 = (($118) + 1)|0;
      $i = $119;
     }
     $120 = $hi_filler;
     $121 = $dp4;
     $122 = (($121) + -1|0);
     $dp4 = $122;
     HEAP8[$122>>0] = $120;
     $123 = $lo_filler;
     $124 = $dp4;
     $125 = (($124) + -1|0);
     $dp4 = $125;
     HEAP8[$125>>0] = $123;
     $126 = $0;
     $127 = (($126) + 10|0);
     HEAP8[$127>>0] = 2;
     $128 = $0;
     $129 = (($128) + 11|0);
     HEAP8[$129>>0] = 32;
     $130 = $row_width;
     $131 = $130<<2;
     $132 = $0;
     $133 = (($132) + 4|0);
     HEAP32[$133>>2] = $131;
    } else {
     $134 = $1;
     $135 = $row_width;
     $136 = $135<<1;
     $137 = (($134) + ($136)|0);
     $sp5 = $137;
     $138 = $sp5;
     $139 = $row_width;
     $140 = $139<<1;
     $141 = (($138) + ($140)|0);
     $dp6 = $141;
     $i = 0;
     while(1) {
      $142 = $i;
      $143 = $row_width;
      $144 = ($142>>>0)<($143>>>0);
      if (!($144)) {
       break;
      }
      $145 = $sp5;
      $146 = (($145) + -1|0);
      $sp5 = $146;
      $147 = HEAP8[$146>>0]|0;
      $148 = $dp6;
      $149 = (($148) + -1|0);
      $dp6 = $149;
      HEAP8[$149>>0] = $147;
      $150 = $sp5;
      $151 = (($150) + -1|0);
      $sp5 = $151;
      $152 = HEAP8[$151>>0]|0;
      $153 = $dp6;
      $154 = (($153) + -1|0);
      $dp6 = $154;
      HEAP8[$154>>0] = $152;
      $155 = $hi_filler;
      $156 = $dp6;
      $157 = (($156) + -1|0);
      $dp6 = $157;
      HEAP8[$157>>0] = $155;
      $158 = $lo_filler;
      $159 = $dp6;
      $160 = (($159) + -1|0);
      $dp6 = $160;
      HEAP8[$160>>0] = $158;
      $161 = $i;
      $162 = (($161) + 1)|0;
      $i = $162;
     }
     $163 = $0;
     $164 = (($163) + 10|0);
     HEAP8[$164>>0] = 2;
     $165 = $0;
     $166 = (($165) + 11|0);
     HEAP8[$166>>0] = 32;
     $167 = $row_width;
     $168 = $167<<2;
     $169 = $0;
     $170 = (($169) + 4|0);
     HEAP32[$170>>2] = $168;
    }
   }
  }
  STACKTOP = sp;return;
 }
 $171 = $0;
 $172 = (($171) + 8|0);
 $173 = HEAP8[$172>>0]|0;
 $174 = $173&255;
 $175 = ($174|0)==(2);
 if ($175) {
  $176 = $0;
  $177 = (($176) + 9|0);
  $178 = HEAP8[$177>>0]|0;
  $179 = $178&255;
  $180 = ($179|0)==(8);
  if ($180) {
   $181 = $3;
   $182 = $181 & 128;
   $183 = ($182|0)!=(0);
   if ($183) {
    $184 = $1;
    $185 = $row_width;
    $186 = ($185*3)|0;
    $187 = (($184) + ($186)|0);
    $sp7 = $187;
    $188 = $sp7;
    $189 = $row_width;
    $190 = (($188) + ($189)|0);
    $dp8 = $190;
    $i = 1;
    while(1) {
     $191 = $i;
     $192 = $row_width;
     $193 = ($191>>>0)<($192>>>0);
     if (!($193)) {
      break;
     }
     $194 = $lo_filler;
     $195 = $dp8;
     $196 = (($195) + -1|0);
     $dp8 = $196;
     HEAP8[$196>>0] = $194;
     $197 = $sp7;
     $198 = (($197) + -1|0);
     $sp7 = $198;
     $199 = HEAP8[$198>>0]|0;
     $200 = $dp8;
     $201 = (($200) + -1|0);
     $dp8 = $201;
     HEAP8[$201>>0] = $199;
     $202 = $sp7;
     $203 = (($202) + -1|0);
     $sp7 = $203;
     $204 = HEAP8[$203>>0]|0;
     $205 = $dp8;
     $206 = (($205) + -1|0);
     $dp8 = $206;
     HEAP8[$206>>0] = $204;
     $207 = $sp7;
     $208 = (($207) + -1|0);
     $sp7 = $208;
     $209 = HEAP8[$208>>0]|0;
     $210 = $dp8;
     $211 = (($210) + -1|0);
     $dp8 = $211;
     HEAP8[$211>>0] = $209;
     $212 = $i;
     $213 = (($212) + 1)|0;
     $i = $213;
    }
    $214 = $lo_filler;
    $215 = $dp8;
    $216 = (($215) + -1|0);
    $dp8 = $216;
    HEAP8[$216>>0] = $214;
    $217 = $0;
    $218 = (($217) + 10|0);
    HEAP8[$218>>0] = 4;
    $219 = $0;
    $220 = (($219) + 11|0);
    HEAP8[$220>>0] = 32;
    $221 = $row_width;
    $222 = $221<<2;
    $223 = $0;
    $224 = (($223) + 4|0);
    HEAP32[$224>>2] = $222;
   } else {
    $225 = $1;
    $226 = $row_width;
    $227 = ($226*3)|0;
    $228 = (($225) + ($227)|0);
    $sp9 = $228;
    $229 = $sp9;
    $230 = $row_width;
    $231 = (($229) + ($230)|0);
    $dp10 = $231;
    $i = 0;
    while(1) {
     $232 = $i;
     $233 = $row_width;
     $234 = ($232>>>0)<($233>>>0);
     if (!($234)) {
      break;
     }
     $235 = $sp9;
     $236 = (($235) + -1|0);
     $sp9 = $236;
     $237 = HEAP8[$236>>0]|0;
     $238 = $dp10;
     $239 = (($238) + -1|0);
     $dp10 = $239;
     HEAP8[$239>>0] = $237;
     $240 = $sp9;
     $241 = (($240) + -1|0);
     $sp9 = $241;
     $242 = HEAP8[$241>>0]|0;
     $243 = $dp10;
     $244 = (($243) + -1|0);
     $dp10 = $244;
     HEAP8[$244>>0] = $242;
     $245 = $sp9;
     $246 = (($245) + -1|0);
     $sp9 = $246;
     $247 = HEAP8[$246>>0]|0;
     $248 = $dp10;
     $249 = (($248) + -1|0);
     $dp10 = $249;
     HEAP8[$249>>0] = $247;
     $250 = $lo_filler;
     $251 = $dp10;
     $252 = (($251) + -1|0);
     $dp10 = $252;
     HEAP8[$252>>0] = $250;
     $253 = $i;
     $254 = (($253) + 1)|0;
     $i = $254;
    }
    $255 = $0;
    $256 = (($255) + 10|0);
    HEAP8[$256>>0] = 4;
    $257 = $0;
    $258 = (($257) + 11|0);
    HEAP8[$258>>0] = 32;
    $259 = $row_width;
    $260 = $259<<2;
    $261 = $0;
    $262 = (($261) + 4|0);
    HEAP32[$262>>2] = $260;
   }
  } else {
   $263 = $0;
   $264 = (($263) + 9|0);
   $265 = HEAP8[$264>>0]|0;
   $266 = $265&255;
   $267 = ($266|0)==(16);
   if ($267) {
    $268 = $3;
    $269 = $268 & 128;
    $270 = ($269|0)!=(0);
    if ($270) {
     $271 = $1;
     $272 = $row_width;
     $273 = ($272*6)|0;
     $274 = (($271) + ($273)|0);
     $sp11 = $274;
     $275 = $sp11;
     $276 = $row_width;
     $277 = $276<<1;
     $278 = (($275) + ($277)|0);
     $dp12 = $278;
     $i = 1;
     while(1) {
      $279 = $i;
      $280 = $row_width;
      $281 = ($279>>>0)<($280>>>0);
      if (!($281)) {
       break;
      }
      $282 = $hi_filler;
      $283 = $dp12;
      $284 = (($283) + -1|0);
      $dp12 = $284;
      HEAP8[$284>>0] = $282;
      $285 = $lo_filler;
      $286 = $dp12;
      $287 = (($286) + -1|0);
      $dp12 = $287;
      HEAP8[$287>>0] = $285;
      $288 = $sp11;
      $289 = (($288) + -1|0);
      $sp11 = $289;
      $290 = HEAP8[$289>>0]|0;
      $291 = $dp12;
      $292 = (($291) + -1|0);
      $dp12 = $292;
      HEAP8[$292>>0] = $290;
      $293 = $sp11;
      $294 = (($293) + -1|0);
      $sp11 = $294;
      $295 = HEAP8[$294>>0]|0;
      $296 = $dp12;
      $297 = (($296) + -1|0);
      $dp12 = $297;
      HEAP8[$297>>0] = $295;
      $298 = $sp11;
      $299 = (($298) + -1|0);
      $sp11 = $299;
      $300 = HEAP8[$299>>0]|0;
      $301 = $dp12;
      $302 = (($301) + -1|0);
      $dp12 = $302;
      HEAP8[$302>>0] = $300;
      $303 = $sp11;
      $304 = (($303) + -1|0);
      $sp11 = $304;
      $305 = HEAP8[$304>>0]|0;
      $306 = $dp12;
      $307 = (($306) + -1|0);
      $dp12 = $307;
      HEAP8[$307>>0] = $305;
      $308 = $sp11;
      $309 = (($308) + -1|0);
      $sp11 = $309;
      $310 = HEAP8[$309>>0]|0;
      $311 = $dp12;
      $312 = (($311) + -1|0);
      $dp12 = $312;
      HEAP8[$312>>0] = $310;
      $313 = $sp11;
      $314 = (($313) + -1|0);
      $sp11 = $314;
      $315 = HEAP8[$314>>0]|0;
      $316 = $dp12;
      $317 = (($316) + -1|0);
      $dp12 = $317;
      HEAP8[$317>>0] = $315;
      $318 = $i;
      $319 = (($318) + 1)|0;
      $i = $319;
     }
     $320 = $hi_filler;
     $321 = $dp12;
     $322 = (($321) + -1|0);
     $dp12 = $322;
     HEAP8[$322>>0] = $320;
     $323 = $lo_filler;
     $324 = $dp12;
     $325 = (($324) + -1|0);
     $dp12 = $325;
     HEAP8[$325>>0] = $323;
     $326 = $0;
     $327 = (($326) + 10|0);
     HEAP8[$327>>0] = 4;
     $328 = $0;
     $329 = (($328) + 11|0);
     HEAP8[$329>>0] = 64;
     $330 = $row_width;
     $331 = $330<<3;
     $332 = $0;
     $333 = (($332) + 4|0);
     HEAP32[$333>>2] = $331;
    } else {
     $334 = $1;
     $335 = $row_width;
     $336 = ($335*6)|0;
     $337 = (($334) + ($336)|0);
     $sp13 = $337;
     $338 = $sp13;
     $339 = $row_width;
     $340 = $339<<1;
     $341 = (($338) + ($340)|0);
     $dp14 = $341;
     $i = 0;
     while(1) {
      $342 = $i;
      $343 = $row_width;
      $344 = ($342>>>0)<($343>>>0);
      if (!($344)) {
       break;
      }
      $345 = $sp13;
      $346 = (($345) + -1|0);
      $sp13 = $346;
      $347 = HEAP8[$346>>0]|0;
      $348 = $dp14;
      $349 = (($348) + -1|0);
      $dp14 = $349;
      HEAP8[$349>>0] = $347;
      $350 = $sp13;
      $351 = (($350) + -1|0);
      $sp13 = $351;
      $352 = HEAP8[$351>>0]|0;
      $353 = $dp14;
      $354 = (($353) + -1|0);
      $dp14 = $354;
      HEAP8[$354>>0] = $352;
      $355 = $sp13;
      $356 = (($355) + -1|0);
      $sp13 = $356;
      $357 = HEAP8[$356>>0]|0;
      $358 = $dp14;
      $359 = (($358) + -1|0);
      $dp14 = $359;
      HEAP8[$359>>0] = $357;
      $360 = $sp13;
      $361 = (($360) + -1|0);
      $sp13 = $361;
      $362 = HEAP8[$361>>0]|0;
      $363 = $dp14;
      $364 = (($363) + -1|0);
      $dp14 = $364;
      HEAP8[$364>>0] = $362;
      $365 = $sp13;
      $366 = (($365) + -1|0);
      $sp13 = $366;
      $367 = HEAP8[$366>>0]|0;
      $368 = $dp14;
      $369 = (($368) + -1|0);
      $dp14 = $369;
      HEAP8[$369>>0] = $367;
      $370 = $sp13;
      $371 = (($370) + -1|0);
      $sp13 = $371;
      $372 = HEAP8[$371>>0]|0;
      $373 = $dp14;
      $374 = (($373) + -1|0);
      $dp14 = $374;
      HEAP8[$374>>0] = $372;
      $375 = $hi_filler;
      $376 = $dp14;
      $377 = (($376) + -1|0);
      $dp14 = $377;
      HEAP8[$377>>0] = $375;
      $378 = $lo_filler;
      $379 = $dp14;
      $380 = (($379) + -1|0);
      $dp14 = $380;
      HEAP8[$380>>0] = $378;
      $381 = $i;
      $382 = (($381) + 1)|0;
      $i = $382;
     }
     $383 = $0;
     $384 = (($383) + 10|0);
     HEAP8[$384>>0] = 4;
     $385 = $0;
     $386 = (($385) + 11|0);
     HEAP8[$386>>0] = 64;
     $387 = $row_width;
     $388 = $387<<3;
     $389 = $0;
     $390 = (($389) + 4|0);
     HEAP32[$390>>2] = $388;
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_do_read_invert_alpha($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp2 = 0, $dp5 = 0, $dp8 = 0, $i = 0, $i3 = 0, $i6 = 0, $i9 = 0, $row_width = 0, $sp = 0;
 var $sp1 = 0, $sp4 = 0, $sp7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $row_width = $3;
 $4 = $0;
 $5 = (($4) + 8|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(6);
 if ($8) {
  $9 = $0;
  $10 = (($9) + 9|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(8);
  if ($13) {
   $14 = $1;
   $15 = $0;
   $16 = (($15) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($14) + ($17)|0);
   $sp = $18;
   $19 = $sp;
   $dp = $19;
   $i = 0;
   while(1) {
    $20 = $i;
    $21 = $row_width;
    $22 = ($20>>>0)<($21>>>0);
    if (!($22)) {
     break;
    }
    $23 = $sp;
    $24 = (($23) + -1|0);
    $sp = $24;
    $25 = HEAP8[$24>>0]|0;
    $26 = $25&255;
    $27 = (255 - ($26))|0;
    $28 = $27&255;
    $29 = $dp;
    $30 = (($29) + -1|0);
    $dp = $30;
    HEAP8[$30>>0] = $28;
    $31 = $sp;
    $32 = (($31) + -3|0);
    $sp = $32;
    $33 = $sp;
    $dp = $33;
    $34 = $i;
    $35 = (($34) + 1)|0;
    $i = $35;
   }
  } else {
   $36 = $1;
   $37 = $0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($36) + ($39)|0);
   $sp1 = $40;
   $41 = $sp1;
   $dp2 = $41;
   $i3 = 0;
   while(1) {
    $42 = $i3;
    $43 = $row_width;
    $44 = ($42>>>0)<($43>>>0);
    if (!($44)) {
     break;
    }
    $45 = $sp1;
    $46 = (($45) + -1|0);
    $sp1 = $46;
    $47 = HEAP8[$46>>0]|0;
    $48 = $47&255;
    $49 = (255 - ($48))|0;
    $50 = $49&255;
    $51 = $dp2;
    $52 = (($51) + -1|0);
    $dp2 = $52;
    HEAP8[$52>>0] = $50;
    $53 = $sp1;
    $54 = (($53) + -1|0);
    $sp1 = $54;
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = (255 - ($56))|0;
    $58 = $57&255;
    $59 = $dp2;
    $60 = (($59) + -1|0);
    $dp2 = $60;
    HEAP8[$60>>0] = $58;
    $61 = $sp1;
    $62 = (($61) + -6|0);
    $sp1 = $62;
    $63 = $sp1;
    $dp2 = $63;
    $64 = $i3;
    $65 = (($64) + 1)|0;
    $i3 = $65;
   }
  }
  STACKTOP = sp;return;
 }
 $66 = $0;
 $67 = (($66) + 8|0);
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = ($69|0)==(4);
 if ($70) {
  $71 = $0;
  $72 = (($71) + 9|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = ($74|0)==(8);
  if ($75) {
   $76 = $1;
   $77 = $0;
   $78 = (($77) + 4|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($76) + ($79)|0);
   $sp4 = $80;
   $81 = $sp4;
   $dp5 = $81;
   $i6 = 0;
   while(1) {
    $82 = $i6;
    $83 = $row_width;
    $84 = ($82>>>0)<($83>>>0);
    if (!($84)) {
     break;
    }
    $85 = $sp4;
    $86 = (($85) + -1|0);
    $sp4 = $86;
    $87 = HEAP8[$86>>0]|0;
    $88 = $87&255;
    $89 = (255 - ($88))|0;
    $90 = $89&255;
    $91 = $dp5;
    $92 = (($91) + -1|0);
    $dp5 = $92;
    HEAP8[$92>>0] = $90;
    $93 = $sp4;
    $94 = (($93) + -1|0);
    $sp4 = $94;
    $95 = HEAP8[$94>>0]|0;
    $96 = $dp5;
    $97 = (($96) + -1|0);
    $dp5 = $97;
    HEAP8[$97>>0] = $95;
    $98 = $i6;
    $99 = (($98) + 1)|0;
    $i6 = $99;
   }
  } else {
   $100 = $1;
   $101 = $0;
   $102 = (($101) + 4|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (($100) + ($103)|0);
   $sp7 = $104;
   $105 = $sp7;
   $dp8 = $105;
   $i9 = 0;
   while(1) {
    $106 = $i9;
    $107 = $row_width;
    $108 = ($106>>>0)<($107>>>0);
    if (!($108)) {
     break;
    }
    $109 = $sp7;
    $110 = (($109) + -1|0);
    $sp7 = $110;
    $111 = HEAP8[$110>>0]|0;
    $112 = $111&255;
    $113 = (255 - ($112))|0;
    $114 = $113&255;
    $115 = $dp8;
    $116 = (($115) + -1|0);
    $dp8 = $116;
    HEAP8[$116>>0] = $114;
    $117 = $sp7;
    $118 = (($117) + -1|0);
    $sp7 = $118;
    $119 = HEAP8[$118>>0]|0;
    $120 = $119&255;
    $121 = (255 - ($120))|0;
    $122 = $121&255;
    $123 = $dp8;
    $124 = (($123) + -1|0);
    $dp8 = $124;
    HEAP8[$124>>0] = $122;
    $125 = $sp7;
    $126 = (($125) + -2|0);
    $sp7 = $126;
    $127 = $sp7;
    $dp8 = $127;
    $128 = $i9;
    $129 = (($128) + 1)|0;
    $i9 = $129;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_do_read_swap_alpha($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp10 = 0, $dp2 = 0, $dp6 = 0, $i = 0, $i12 = 0, $i4 = 0, $i8 = 0, $row_width = 0, $save = 0, $save11 = 0, $save3 = 0;
 var $save7 = 0, $sp = 0, $sp1 = 0, $sp5 = 0, $sp9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $save3 = sp + 64|0;
 $save11 = sp + 60|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = HEAP32[$2>>2]|0;
 $row_width = $3;
 $4 = $0;
 $5 = (($4) + 8|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(6);
 if ($8) {
  $9 = $0;
  $10 = (($9) + 9|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(8);
  if ($13) {
   $14 = $1;
   $15 = $0;
   $16 = (($15) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($14) + ($17)|0);
   $sp = $18;
   $19 = $sp;
   $dp = $19;
   $i = 0;
   while(1) {
    $20 = $i;
    $21 = $row_width;
    $22 = ($20>>>0)<($21>>>0);
    if (!($22)) {
     break;
    }
    $23 = $sp;
    $24 = (($23) + -1|0);
    $sp = $24;
    $25 = HEAP8[$24>>0]|0;
    $save = $25;
    $26 = $sp;
    $27 = (($26) + -1|0);
    $sp = $27;
    $28 = HEAP8[$27>>0]|0;
    $29 = $dp;
    $30 = (($29) + -1|0);
    $dp = $30;
    HEAP8[$30>>0] = $28;
    $31 = $sp;
    $32 = (($31) + -1|0);
    $sp = $32;
    $33 = HEAP8[$32>>0]|0;
    $34 = $dp;
    $35 = (($34) + -1|0);
    $dp = $35;
    HEAP8[$35>>0] = $33;
    $36 = $sp;
    $37 = (($36) + -1|0);
    $sp = $37;
    $38 = HEAP8[$37>>0]|0;
    $39 = $dp;
    $40 = (($39) + -1|0);
    $dp = $40;
    HEAP8[$40>>0] = $38;
    $41 = $save;
    $42 = $dp;
    $43 = (($42) + -1|0);
    $dp = $43;
    HEAP8[$43>>0] = $41;
    $44 = $i;
    $45 = (($44) + 1)|0;
    $i = $45;
   }
  } else {
   $46 = $1;
   $47 = $0;
   $48 = (($47) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($46) + ($49)|0);
   $sp1 = $50;
   $51 = $sp1;
   $dp2 = $51;
   $i4 = 0;
   while(1) {
    $52 = $i4;
    $53 = $row_width;
    $54 = ($52>>>0)<($53>>>0);
    if (!($54)) {
     break;
    }
    $55 = $sp1;
    $56 = (($55) + -1|0);
    $sp1 = $56;
    $57 = HEAP8[$56>>0]|0;
    HEAP8[$save3>>0] = $57;
    $58 = $sp1;
    $59 = (($58) + -1|0);
    $sp1 = $59;
    $60 = HEAP8[$59>>0]|0;
    $61 = (($save3) + 1|0);
    HEAP8[$61>>0] = $60;
    $62 = $sp1;
    $63 = (($62) + -1|0);
    $sp1 = $63;
    $64 = HEAP8[$63>>0]|0;
    $65 = $dp2;
    $66 = (($65) + -1|0);
    $dp2 = $66;
    HEAP8[$66>>0] = $64;
    $67 = $sp1;
    $68 = (($67) + -1|0);
    $sp1 = $68;
    $69 = HEAP8[$68>>0]|0;
    $70 = $dp2;
    $71 = (($70) + -1|0);
    $dp2 = $71;
    HEAP8[$71>>0] = $69;
    $72 = $sp1;
    $73 = (($72) + -1|0);
    $sp1 = $73;
    $74 = HEAP8[$73>>0]|0;
    $75 = $dp2;
    $76 = (($75) + -1|0);
    $dp2 = $76;
    HEAP8[$76>>0] = $74;
    $77 = $sp1;
    $78 = (($77) + -1|0);
    $sp1 = $78;
    $79 = HEAP8[$78>>0]|0;
    $80 = $dp2;
    $81 = (($80) + -1|0);
    $dp2 = $81;
    HEAP8[$81>>0] = $79;
    $82 = $sp1;
    $83 = (($82) + -1|0);
    $sp1 = $83;
    $84 = HEAP8[$83>>0]|0;
    $85 = $dp2;
    $86 = (($85) + -1|0);
    $dp2 = $86;
    HEAP8[$86>>0] = $84;
    $87 = $sp1;
    $88 = (($87) + -1|0);
    $sp1 = $88;
    $89 = HEAP8[$88>>0]|0;
    $90 = $dp2;
    $91 = (($90) + -1|0);
    $dp2 = $91;
    HEAP8[$91>>0] = $89;
    $92 = HEAP8[$save3>>0]|0;
    $93 = $dp2;
    $94 = (($93) + -1|0);
    $dp2 = $94;
    HEAP8[$94>>0] = $92;
    $95 = (($save3) + 1|0);
    $96 = HEAP8[$95>>0]|0;
    $97 = $dp2;
    $98 = (($97) + -1|0);
    $dp2 = $98;
    HEAP8[$98>>0] = $96;
    $99 = $i4;
    $100 = (($99) + 1)|0;
    $i4 = $100;
   }
  }
  STACKTOP = sp;return;
 }
 $101 = $0;
 $102 = (($101) + 8|0);
 $103 = HEAP8[$102>>0]|0;
 $104 = $103&255;
 $105 = ($104|0)==(4);
 if ($105) {
  $106 = $0;
  $107 = (($106) + 9|0);
  $108 = HEAP8[$107>>0]|0;
  $109 = $108&255;
  $110 = ($109|0)==(8);
  if ($110) {
   $111 = $1;
   $112 = $0;
   $113 = (($112) + 4|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = (($111) + ($114)|0);
   $sp5 = $115;
   $116 = $sp5;
   $dp6 = $116;
   $i8 = 0;
   while(1) {
    $117 = $i8;
    $118 = $row_width;
    $119 = ($117>>>0)<($118>>>0);
    if (!($119)) {
     break;
    }
    $120 = $sp5;
    $121 = (($120) + -1|0);
    $sp5 = $121;
    $122 = HEAP8[$121>>0]|0;
    $save7 = $122;
    $123 = $sp5;
    $124 = (($123) + -1|0);
    $sp5 = $124;
    $125 = HEAP8[$124>>0]|0;
    $126 = $dp6;
    $127 = (($126) + -1|0);
    $dp6 = $127;
    HEAP8[$127>>0] = $125;
    $128 = $save7;
    $129 = $dp6;
    $130 = (($129) + -1|0);
    $dp6 = $130;
    HEAP8[$130>>0] = $128;
    $131 = $i8;
    $132 = (($131) + 1)|0;
    $i8 = $132;
   }
  } else {
   $133 = $1;
   $134 = $0;
   $135 = (($134) + 4|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($133) + ($136)|0);
   $sp9 = $137;
   $138 = $sp9;
   $dp10 = $138;
   $i12 = 0;
   while(1) {
    $139 = $i12;
    $140 = $row_width;
    $141 = ($139>>>0)<($140>>>0);
    if (!($141)) {
     break;
    }
    $142 = $sp9;
    $143 = (($142) + -1|0);
    $sp9 = $143;
    $144 = HEAP8[$143>>0]|0;
    HEAP8[$save11>>0] = $144;
    $145 = $sp9;
    $146 = (($145) + -1|0);
    $sp9 = $146;
    $147 = HEAP8[$146>>0]|0;
    $148 = (($save11) + 1|0);
    HEAP8[$148>>0] = $147;
    $149 = $sp9;
    $150 = (($149) + -1|0);
    $sp9 = $150;
    $151 = HEAP8[$150>>0]|0;
    $152 = $dp10;
    $153 = (($152) + -1|0);
    $dp10 = $153;
    HEAP8[$153>>0] = $151;
    $154 = $sp9;
    $155 = (($154) + -1|0);
    $sp9 = $155;
    $156 = HEAP8[$155>>0]|0;
    $157 = $dp10;
    $158 = (($157) + -1|0);
    $dp10 = $158;
    HEAP8[$158>>0] = $156;
    $159 = HEAP8[$save11>>0]|0;
    $160 = $dp10;
    $161 = (($160) + -1|0);
    $dp10 = $161;
    HEAP8[$161>>0] = $159;
    $162 = (($save11) + 1|0);
    $163 = HEAP8[$162>>0]|0;
    $164 = $dp10;
    $165 = (($164) + -1|0);
    $dp10 = $165;
    HEAP8[$165>>0] = $163;
    $166 = $i12;
    $167 = (($166) + 1)|0;
    $i12 = $167;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_do_read_intrapixel($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $blue = 0, $bytes_per_pixel = 0, $i = 0, $i2 = 0, $red = 0, $row_width = 0, $rp = 0, $rp1 = 0, $s0 = 0, $s1 = 0, $s2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 8|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 2;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = $0;
 $9 = HEAP32[$8>>2]|0;
 $row_width = $9;
 $10 = $0;
 $11 = (($10) + 9|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(8);
 if ($14) {
  $15 = $0;
  $16 = (($15) + 8|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = ($18|0)==(2);
  do {
   if ($19) {
    $bytes_per_pixel = 3;
   } else {
    $20 = $0;
    $21 = (($20) + 8|0);
    $22 = HEAP8[$21>>0]|0;
    $23 = $22&255;
    $24 = ($23|0)==(6);
    if ($24) {
     $bytes_per_pixel = 4;
     break;
    } else {
     STACKTOP = sp;return;
    }
   }
  } while(0);
  $i = 0;
  $25 = $1;
  $rp = $25;
  while(1) {
   $26 = $i;
   $27 = $row_width;
   $28 = ($26>>>0)<($27>>>0);
   if (!($28)) {
    break;
   }
   $29 = $rp;
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = (256 + ($31))|0;
   $33 = $rp;
   $34 = (($33) + 1|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = $35&255;
   $37 = (($32) + ($36))|0;
   $38 = $37 & 255;
   $39 = $38&255;
   $40 = $rp;
   HEAP8[$40>>0] = $39;
   $41 = $rp;
   $42 = (($41) + 2|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = (256 + ($44))|0;
   $46 = $rp;
   $47 = (($46) + 1|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   $50 = (($45) + ($49))|0;
   $51 = $50 & 255;
   $52 = $51&255;
   $53 = $rp;
   $54 = (($53) + 2|0);
   HEAP8[$54>>0] = $52;
   $55 = $i;
   $56 = (($55) + 1)|0;
   $i = $56;
   $57 = $bytes_per_pixel;
   $58 = $rp;
   $59 = (($58) + ($57)|0);
   $rp = $59;
  }
 } else {
  $60 = $0;
  $61 = (($60) + 9|0);
  $62 = HEAP8[$61>>0]|0;
  $63 = $62&255;
  $64 = ($63|0)==(16);
  if ($64) {
   $65 = $0;
   $66 = (($65) + 8|0);
   $67 = HEAP8[$66>>0]|0;
   $68 = $67&255;
   $69 = ($68|0)==(2);
   do {
    if ($69) {
     $bytes_per_pixel = 6;
    } else {
     $70 = $0;
     $71 = (($70) + 8|0);
     $72 = HEAP8[$71>>0]|0;
     $73 = $72&255;
     $74 = ($73|0)==(6);
     if ($74) {
      $bytes_per_pixel = 8;
      break;
     } else {
      STACKTOP = sp;return;
     }
    }
   } while(0);
   $i2 = 0;
   $75 = $1;
   $rp1 = $75;
   while(1) {
    $76 = $i2;
    $77 = $row_width;
    $78 = ($76>>>0)<($77>>>0);
    if (!($78)) {
     break;
    }
    $79 = $rp1;
    $80 = HEAP8[$79>>0]|0;
    $81 = $80&255;
    $82 = $81 << 8;
    $83 = $rp1;
    $84 = (($83) + 1|0);
    $85 = HEAP8[$84>>0]|0;
    $86 = $85&255;
    $87 = $82 | $86;
    $s0 = $87;
    $88 = $rp1;
    $89 = (($88) + 2|0);
    $90 = HEAP8[$89>>0]|0;
    $91 = $90&255;
    $92 = $91 << 8;
    $93 = $rp1;
    $94 = (($93) + 3|0);
    $95 = HEAP8[$94>>0]|0;
    $96 = $95&255;
    $97 = $92 | $96;
    $s1 = $97;
    $98 = $rp1;
    $99 = (($98) + 4|0);
    $100 = HEAP8[$99>>0]|0;
    $101 = $100&255;
    $102 = $101 << 8;
    $103 = $rp1;
    $104 = (($103) + 5|0);
    $105 = HEAP8[$104>>0]|0;
    $106 = $105&255;
    $107 = $102 | $106;
    $s2 = $107;
    $108 = $s0;
    $109 = $s1;
    $110 = (($108) + ($109))|0;
    $111 = (($110) + 65536)|0;
    $112 = $111 & 65535;
    $red = $112;
    $113 = $s2;
    $114 = $s1;
    $115 = (($113) + ($114))|0;
    $116 = (($115) + 65536)|0;
    $117 = $116 & 65535;
    $blue = $117;
    $118 = $red;
    $119 = $118 >>> 8;
    $120 = $119 & 255;
    $121 = $120&255;
    $122 = $rp1;
    HEAP8[$122>>0] = $121;
    $123 = $red;
    $124 = $123 & 255;
    $125 = $124&255;
    $126 = $rp1;
    $127 = (($126) + 1|0);
    HEAP8[$127>>0] = $125;
    $128 = $blue;
    $129 = $128 >>> 8;
    $130 = $129 & 255;
    $131 = $130&255;
    $132 = $rp1;
    $133 = (($132) + 4|0);
    HEAP8[$133>>0] = $131;
    $134 = $blue;
    $135 = $134 & 255;
    $136 = $135&255;
    $137 = $rp1;
    $138 = (($137) + 5|0);
    HEAP8[$138>>0] = $136;
    $139 = $i2;
    $140 = (($139) + 1)|0;
    $i2 = $140;
    $141 = $bytes_per_pixel;
    $142 = $rp1;
    $143 = (($142) + ($141)|0);
    $rp1 = $143;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_get_uint_31($png_ptr,$buf) {
 $png_ptr = $png_ptr|0;
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $buf;
 $2 = $1;
 $3 = (_png_get_uint_32($2)|0);
 $i = $3;
 $4 = $i;
 $5 = ($4>>>0)>(2147483647);
 if ($5) {
  $6 = $0;
  _png_error($6,58680);
 }
 $7 = $i;
 STACKTOP = sp;return ($7|0);
}
function _png_get_uint_32($buf) {
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $buf;
 $1 = $0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = $3 << 24;
 $5 = $0;
 $6 = (($5) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 << 16;
 $10 = (($4) + ($9))|0;
 $11 = $0;
 $12 = (($11) + 2|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 << 8;
 $16 = (($10) + ($15))|0;
 $17 = $0;
 $18 = (($17) + 3|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = (($16) + ($20))|0;
 $i = $21;
 $22 = $i;
 STACKTOP = sp;return ($22|0);
}
function _png_get_int_32($buf) {
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $buf;
 $1 = $0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = $3 << 24;
 $5 = $0;
 $6 = (($5) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 << 16;
 $10 = (($4) + ($9))|0;
 $11 = $0;
 $12 = (($11) + 2|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 << 8;
 $16 = (($10) + ($15))|0;
 $17 = $0;
 $18 = (($17) + 3|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = (($16) + ($20))|0;
 $i = $21;
 $22 = $i;
 STACKTOP = sp;return ($22|0);
}
function _png_get_uint_16($buf) {
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $buf;
 $1 = $0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = $3&65535;
 $5 = $4 << 8;
 $6 = $0;
 $7 = (($6) + 1|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9&65535;
 $11 = (($5) + ($10))|0;
 $12 = $11&65535;
 $i = $12;
 $13 = $i;
 STACKTOP = sp;return ($13|0);
}
function _png_crc_read($png_ptr,$buf,$length) {
 $png_ptr = $png_ptr|0;
 $buf = $buf|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $buf;
 $2 = $length;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 } else {
  $5 = $0;
  $6 = $1;
  $7 = $2;
  _png_read_data($5,$6,$7);
  $8 = $0;
  $9 = $1;
  $10 = $2;
  _png_calculate_crc($8,$9,$10);
  STACKTOP = sp;return;
 }
}
function _png_crc_finish($png_ptr,$skip) {
 $png_ptr = $png_ptr|0;
 $skip = $skip|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $istop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $png_ptr;
 $2 = $skip;
 $3 = $1;
 $4 = (($3) + 268|0);
 $5 = HEAP32[$4>>2]|0;
 $istop = $5;
 $6 = $2;
 $i = $6;
 while(1) {
  $7 = $i;
  $8 = $istop;
  $9 = ($7>>>0)>($8>>>0);
  if (!($9)) {
   break;
  }
  $10 = $1;
  $11 = $1;
  $12 = (($11) + 264|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $1;
  $15 = (($14) + 268|0);
  $16 = HEAP32[$15>>2]|0;
  _png_crc_read($10,$13,$16);
  $17 = $istop;
  $18 = $i;
  $19 = (($18) - ($17))|0;
  $i = $19;
 }
 $20 = $i;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = $1;
  $23 = $1;
  $24 = (($23) + 264|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $i;
  _png_crc_read($22,$25,$26);
 }
 $27 = $1;
 $28 = (_png_crc_error($27)|0);
 $29 = ($28|0)!=(0);
 if (!($29)) {
  $0 = 0;
  $54 = $0;
  STACKTOP = sp;return ($54|0);
 }
 $30 = $1;
 $31 = (($30) + 376|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = $33 & 32;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = $1;
  $37 = (($36) + 200|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = $38 & 512;
  $40 = ($39|0)!=(0);
  if ($40) {
   label = 10;
  } else {
   label = 12;
  }
 } else {
  label = 10;
 }
 do {
  if ((label|0) == 10) {
   $41 = $1;
   $42 = (($41) + 376|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $44 & 32;
   $46 = ($45|0)!=(0);
   if (!($46)) {
    $47 = $1;
    $48 = (($47) + 200|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = $49 & 1024;
    $51 = ($50|0)!=(0);
    if ($51) {
     label = 12;
     break;
    }
   }
   $53 = $1;
   _png_chunk_error($53,58720);
  }
 } while(0);
 if ((label|0) == 12) {
  $52 = $1;
  _png_chunk_warning($52,58720);
 }
 $0 = 1;
 $54 = $0;
 STACKTOP = sp;return ($54|0);
}
function _png_crc_error($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $crc = 0, $crc_bytes = 0, $need_crc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $crc_bytes = sp + 16|0;
 $1 = $png_ptr;
 $need_crc = 1;
 $2 = $1;
 $3 = (($2) + 376|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 32;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $1;
  $9 = (($8) + 200|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 768;
  $12 = ($11|0)==(768);
  if ($12) {
   $need_crc = 0;
  }
 } else {
  $13 = $1;
  $14 = (($13) + 200|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 2048;
  $17 = ($16|0)!=(0);
  if ($17) {
   $need_crc = 0;
  }
 }
 $18 = $1;
 _png_read_data($18,$crc_bytes,4);
 $19 = $need_crc;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = (_png_get_uint_32($crc_bytes)|0);
  $crc = $21;
  $22 = $crc;
  $23 = $1;
  $24 = (($23) + 364|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($22|0)!=($25|0);
  $27 = $26&1;
  $0 = $27;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 } else {
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _png_decompress_chunk($png_ptr,$comp_type,$chunkdata,$chunklength,$prefix_size,$newlength) {
 $png_ptr = $png_ptr|0;
 $comp_type = $comp_type|0;
 $chunkdata = $chunkdata|0;
 $chunklength = $chunklength|0;
 $prefix_size = $prefix_size|0;
 $newlength = $newlength|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ret = 0, $text = 0, $text_size = 0, $tmp = 0, $umsg = 0;
 var $umsg1 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $umsg = sp + 120|0;
 $umsg1 = sp + 68|0;
 $0 = $png_ptr;
 $1 = $comp_type;
 $2 = $chunkdata;
 $3 = $chunklength;
 $4 = $prefix_size;
 $5 = $newlength;
 $6 = $1;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $247 = $1;
  HEAP32[$vararg_buffer7>>2] = $247;
  (_snprintf($umsg1,50,59072,$vararg_buffer7)|0);
  $248 = $0;
  _png_warning($248,$umsg1);
  $249 = $2;
  $250 = $4;
  $251 = (($249) + ($250)|0);
  HEAP8[$251>>0] = 0;
  $252 = $4;
  $253 = $5;
  HEAP32[$253>>2] = $252;
  $254 = $2;
  STACKTOP = sp;return ($254|0);
 }
 $ret = 0;
 $8 = $2;
 $9 = $4;
 $10 = (($8) + ($9)|0);
 $11 = $0;
 $12 = (($11) + 208|0);
 HEAP32[$12>>2] = $10;
 $13 = $3;
 $14 = $4;
 $15 = (($13) - ($14))|0;
 $16 = $0;
 $17 = (($16) + 208|0);
 $18 = (($17) + 4|0);
 HEAP32[$18>>2] = $15;
 $19 = $0;
 $20 = (($19) + 264|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $0;
 $23 = (($22) + 208|0);
 $24 = (($23) + 12|0);
 HEAP32[$24>>2] = $21;
 $25 = $0;
 $26 = (($25) + 268|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $0;
 $29 = (($28) + 208|0);
 $30 = (($29) + 16|0);
 HEAP32[$30>>2] = $27;
 $text_size = 0;
 $text = 0;
 while(1) {
  $31 = $0;
  $32 = (($31) + 208|0);
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)!=(0);
  if (!($35)) {
   break;
  }
  $36 = $0;
  $37 = (($36) + 208|0);
  $38 = (_inflate($37,1)|0);
  $ret = $38;
  $39 = $ret;
  $40 = ($39|0)!=(0);
  if ($40) {
   $41 = $ret;
   $42 = ($41|0)!=(1);
   if ($42) {
    label = 6;
    break;
   }
  }
  $96 = $0;
  $97 = (($96) + 208|0);
  $98 = (($97) + 16|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = $ret;
   $102 = ($101|0)==(1);
   if ($102) {
    label = 19;
   }
  } else {
   label = 19;
  }
  if ((label|0) == 19) {
   label = 0;
   $103 = $text;
   $104 = ($103|0)==(0|0);
   if ($104) {
    $105 = $4;
    $106 = $0;
    $107 = (($106) + 268|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = (($105) + ($108))|0;
    $110 = $0;
    $111 = (($110) + 208|0);
    $112 = (($111) + 16|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = (($109) - ($113))|0;
    $text_size = $114;
    $115 = $0;
    $116 = $text_size;
    $117 = (($116) + 1)|0;
    $118 = (_png_malloc_warn($115,$117)|0);
    $text = $118;
    $119 = $text;
    $120 = ($119|0)==(0|0);
    if ($120) {
     $121 = $0;
     $122 = $2;
     _png_free($121,$122);
     $123 = $0;
     _png_error($123,58808);
    }
    $124 = $text;
    $125 = $4;
    $126 = (($124) + ($125)|0);
    $127 = $0;
    $128 = (($127) + 264|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = $text_size;
    $131 = $4;
    $132 = (($130) - ($131))|0;
    _memcpy(($126|0),($129|0),($132|0))|0;
    $133 = $text;
    $134 = $2;
    $135 = $4;
    _memcpy(($133|0),($134|0),($135|0))|0;
    $136 = $text;
    $137 = $text_size;
    $138 = (($136) + ($137)|0);
    HEAP8[$138>>0] = 0;
   } else {
    $139 = $text;
    $tmp = $139;
    $140 = $0;
    $141 = $text_size;
    $142 = $0;
    $143 = (($142) + 268|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = (($141) + ($144))|0;
    $146 = $0;
    $147 = (($146) + 208|0);
    $148 = (($147) + 16|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = (($145) - ($149))|0;
    $151 = (($150) + 1)|0;
    $152 = (_png_malloc_warn($140,$151)|0);
    $text = $152;
    $153 = $text;
    $154 = ($153|0)==(0|0);
    if ($154) {
     $155 = $0;
     $156 = $tmp;
     _png_free($155,$156);
     $157 = $0;
     $158 = $2;
     _png_free($157,$158);
     $159 = $0;
     _png_error($159,58848);
    }
    $160 = $text;
    $161 = $tmp;
    $162 = $text_size;
    _memcpy(($160|0),($161|0),($162|0))|0;
    $163 = $0;
    $164 = $tmp;
    _png_free($163,$164);
    $165 = $text;
    $166 = $text_size;
    $167 = (($165) + ($166)|0);
    $168 = $0;
    $169 = (($168) + 264|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = $0;
    $172 = (($171) + 268|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = $0;
    $175 = (($174) + 208|0);
    $176 = (($175) + 16|0);
    $177 = HEAP32[$176>>2]|0;
    $178 = (($173) - ($177))|0;
    _memcpy(($167|0),($170|0),($178|0))|0;
    $179 = $0;
    $180 = (($179) + 268|0);
    $181 = HEAP32[$180>>2]|0;
    $182 = $0;
    $183 = (($182) + 208|0);
    $184 = (($183) + 16|0);
    $185 = HEAP32[$184>>2]|0;
    $186 = (($181) - ($185))|0;
    $187 = $text_size;
    $188 = (($187) + ($186))|0;
    $text_size = $188;
    $189 = $text;
    $190 = $text_size;
    $191 = (($189) + ($190)|0);
    HEAP8[$191>>0] = 0;
   }
   $192 = $ret;
   $193 = ($192|0)==(1);
   if ($193) {
    label = 27;
    break;
   }
   $194 = $0;
   $195 = (($194) + 264|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = $0;
   $198 = (($197) + 208|0);
   $199 = (($198) + 12|0);
   HEAP32[$199>>2] = $196;
   $200 = $0;
   $201 = (($200) + 268|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = $0;
   $204 = (($203) + 208|0);
   $205 = (($204) + 16|0);
   HEAP32[$205>>2] = $202;
  }
 }
 if ((label|0) == 6) {
  $43 = $0;
  $44 = (($43) + 208|0);
  $45 = (($44) + 24|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0|0);
  if ($47) {
   $48 = $0;
   $49 = $0;
   $50 = (($49) + 208|0);
   $51 = (($50) + 24|0);
   $52 = HEAP32[$51>>2]|0;
   _png_warning($48,$52);
  } else {
   $53 = $0;
   _png_warning($53,58736);
  }
  $54 = $0;
  $55 = (($54) + 208|0);
  (_inflateReset($55)|0);
  $56 = $0;
  $57 = (($56) + 208|0);
  $58 = (($57) + 4|0);
  HEAP32[$58>>2] = 0;
  $59 = $text;
  $60 = ($59|0)==(0|0);
  if ($60) {
   $61 = $4;
   $62 = (($61) + 31)|0;
   $63 = (($62) + 1)|0;
   $text_size = $63;
   $64 = $0;
   $65 = $text_size;
   $66 = (_png_malloc_warn($64,$65)|0);
   $text = $66;
   $67 = $text;
   $68 = ($67|0)==(0|0);
   if ($68) {
    $69 = $0;
    $70 = $2;
    _png_free($69,$70);
    $71 = $0;
    _png_error($71,58768);
   }
   $72 = $text;
   $73 = $2;
   $74 = $4;
   _memcpy(($72|0),($73|0),($74|0))|0;
  }
  $75 = $text_size;
  $76 = (($75) - 1)|0;
  $77 = $text;
  $78 = (($77) + ($76)|0);
  HEAP8[$78>>0] = 0;
  $79 = $3;
  $80 = $text;
  $81 = $2;
  $82 = $80;
  $83 = $81;
  $84 = (($82) - ($83))|0;
  $85 = (($79) - ($84))|0;
  $86 = (($85) - 1)|0;
  $text_size = $86;
  $87 = $text_size;
  $88 = (31)>($87>>>0);
  if ($88) {
   $89 = $text_size;
   $90 = $89;
  } else {
   $90 = 31;
  }
  $text_size = $90;
  $91 = $text;
  $92 = $4;
  $93 = (($91) + ($92)|0);
  $94 = $text_size;
  $95 = (($94) + 1)|0;
  _memcpy(($93|0),(58736|0),($95|0))|0;
 }
 else if ((label|0) == 27) {
 }
 $206 = $ret;
 $207 = ($206|0)!=(1);
 if ($207) {
  $208 = $ret;
  $209 = ($208|0)==(-5);
  if ($209) {
   $210 = $0;
   $211 = (($210) + 376|0);
   HEAP32[$vararg_buffer>>2] = $211;
   (_snprintf($umsg,52,58888,$vararg_buffer)|0);
  } else {
   $212 = $ret;
   $213 = ($212|0)==(-3);
   if ($213) {
    $214 = $0;
    $215 = (($214) + 376|0);
    HEAP32[$vararg_buffer1>>2] = $215;
    (_snprintf($umsg,52,58944,$vararg_buffer1)|0);
   } else {
    $216 = $0;
    $217 = (($216) + 376|0);
    HEAP32[$vararg_buffer4>>2] = $217;
    (_snprintf($umsg,52,58992,$vararg_buffer4)|0);
   }
  }
  $218 = $0;
  _png_warning($218,$umsg);
  $219 = $4;
  $text_size = $219;
  $220 = $text;
  $221 = ($220|0)==(0|0);
  if ($221) {
   $222 = $0;
   $223 = $text_size;
   $224 = (($223) + 1)|0;
   $225 = (_png_malloc_warn($222,$224)|0);
   $text = $225;
   $226 = $text;
   $227 = ($226|0)==(0|0);
   if ($227) {
    $228 = $0;
    $229 = $2;
    _png_free($228,$229);
    $230 = $0;
    _png_error($230,59040);
   }
   $231 = $text;
   $232 = $2;
   $233 = $4;
   _memcpy(($231|0),($232|0),($233|0))|0;
  }
  $234 = $text;
  $235 = $text_size;
  $236 = (($234) + ($235)|0);
  HEAP8[$236>>0] = 0;
 }
 $237 = $0;
 $238 = (($237) + 208|0);
 (_inflateReset($238)|0);
 $239 = $0;
 $240 = (($239) + 208|0);
 $241 = (($240) + 4|0);
 HEAP32[$241>>2] = 0;
 $242 = $0;
 $243 = $2;
 _png_free($242,$243);
 $244 = $text;
 $2 = $244;
 $245 = $text_size;
 $246 = $5;
 HEAP32[$246>>2] = $245;
 $254 = $2;
 STACKTOP = sp;return ($254|0);
}
function _png_handle_IHDR($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bit_depth = 0, $buf = 0, $color_type = 0, $compression_type = 0, $filter_type = 0, $height = 0, $interlace_type = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $buf = sp + 40|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $0;
  _png_error($8,59112);
 }
 $9 = $2;
 $10 = ($9|0)!=(13);
 if ($10) {
  $11 = $0;
  _png_error($11,59136);
 }
 $12 = $0;
 $13 = (($12) + 196|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 | 1;
 HEAP32[$13>>2] = $15;
 $16 = $0;
 _png_crc_read($16,$buf,13);
 $17 = $0;
 (_png_crc_finish($17,0)|0);
 $18 = $0;
 $19 = (_png_get_uint_31($18,$buf)|0);
 $width = $19;
 $20 = $0;
 $21 = (($buf) + 4|0);
 $22 = (_png_get_uint_31($20,$21)|0);
 $height = $22;
 $23 = (($buf) + 8|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $bit_depth = $25;
 $26 = (($buf) + 9|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $color_type = $28;
 $29 = (($buf) + 10|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $compression_type = $31;
 $32 = (($buf) + 11|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $filter_type = $34;
 $35 = (($buf) + 12|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $interlace_type = $37;
 $38 = $width;
 $39 = $0;
 $40 = (($39) + 292|0);
 HEAP32[$40>>2] = $38;
 $41 = $height;
 $42 = $0;
 $43 = (($42) + 296|0);
 HEAP32[$43>>2] = $41;
 $44 = $bit_depth;
 $45 = $44&255;
 $46 = $0;
 $47 = (($46) + 387|0);
 HEAP8[$47>>0] = $45;
 $48 = $interlace_type;
 $49 = $48&255;
 $50 = $0;
 $51 = (($50) + 383|0);
 HEAP8[$51>>0] = $49;
 $52 = $color_type;
 $53 = $52&255;
 $54 = $0;
 $55 = (($54) + 386|0);
 HEAP8[$55>>0] = $53;
 $56 = $filter_type;
 $57 = $56&255;
 $58 = $0;
 $59 = (($58) + 660|0);
 HEAP8[$59>>0] = $57;
 $60 = $compression_type;
 $61 = $60&255;
 $62 = $0;
 $63 = (($62) + 700|0);
 HEAP8[$63>>0] = $61;
 $64 = $0;
 $65 = (($64) + 386|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = $66&255;
 switch ($67|0) {
 case 2:  {
  $70 = $0;
  $71 = (($70) + 390|0);
  HEAP8[$71>>0] = 3;
  break;
 }
 case 6:  {
  $74 = $0;
  $75 = (($74) + 390|0);
  HEAP8[$75>>0] = 4;
  break;
 }
 case 3: case 0:  {
  $68 = $0;
  $69 = (($68) + 390|0);
  HEAP8[$69>>0] = 1;
  break;
 }
 case 4:  {
  $72 = $0;
  $73 = (($72) + 390|0);
  HEAP8[$73>>0] = 2;
  break;
 }
 default: {
 }
 }
 $76 = $0;
 $77 = (($76) + 387|0);
 $78 = HEAP8[$77>>0]|0;
 $79 = $78&255;
 $80 = $0;
 $81 = (($80) + 390|0);
 $82 = HEAP8[$81>>0]|0;
 $83 = $82&255;
 $84 = Math_imul($79, $83)|0;
 $85 = $84&255;
 $86 = $0;
 $87 = (($86) + 389|0);
 HEAP8[$87>>0] = $85;
 $88 = $0;
 $89 = (($88) + 389|0);
 $90 = HEAP8[$89>>0]|0;
 $91 = $90&255;
 $92 = ($91|0)>=(8);
 if ($92) {
  $93 = $0;
  $94 = (($93) + 292|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = $0;
  $97 = (($96) + 389|0);
  $98 = HEAP8[$97>>0]|0;
  $99 = $98&255;
  $100 = $99 >>> 3;
  $101 = Math_imul($95, $100)|0;
  $114 = $101;
  $112 = $0;
  $113 = (($112) + 308|0);
  HEAP32[$113>>2] = $114;
  $115 = $0;
  $116 = $1;
  $117 = $width;
  $118 = $height;
  $119 = $bit_depth;
  $120 = $color_type;
  $121 = $interlace_type;
  $122 = $compression_type;
  $123 = $filter_type;
  _png_set_IHDR($115,$116,$117,$118,$119,$120,$121,$122,$123);
  STACKTOP = sp;return;
 } else {
  $102 = $0;
  $103 = (($102) + 292|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = $0;
  $106 = (($105) + 389|0);
  $107 = HEAP8[$106>>0]|0;
  $108 = $107&255;
  $109 = Math_imul($104, $108)|0;
  $110 = (($109) + 7)|0;
  $111 = $110 >>> 3;
  $114 = $111;
  $112 = $0;
  $113 = (($112) + 308|0);
  HEAP32[$113>>2] = $114;
  $115 = $0;
  $116 = $1;
  $117 = $width;
  $118 = $height;
  $119 = $bit_depth;
  $120 = $color_type;
  $121 = $interlace_type;
  $122 = $compression_type;
  $123 = $filter_type;
  _png_set_IHDR($115,$116,$117,$118,$119,$120,$121,$122,$123);
  STACKTOP = sp;return;
 }
}
function _png_handle_PLTE($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0, $i = 0, $num = 0, $pal_ptr = 0, $palette = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 800|0;
 $palette = sp + 32|0;
 $buf = sp + 24|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,59192);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = $0;
   _png_error($22,59216);
  }
 } else {
  $8 = $0;
  _png_error($8,59160);
 }
 $23 = $0;
 $24 = (($23) + 196|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 | 2;
 HEAP32[$24>>2] = $26;
 $27 = $0;
 $28 = (($27) + 386|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = $29&255;
 $31 = $30 & 2;
 $32 = ($31|0)!=(0);
 if (!($32)) {
  $33 = $0;
  _png_warning($33,59240);
  $34 = $0;
  $35 = $2;
  (_png_crc_finish($34,$35)|0);
  STACKTOP = sp;return;
 }
 $36 = $2;
 $37 = ($36>>>0)>(768);
 if ($37) {
  label = 13;
 } else {
  $38 = $2;
  $39 = (($38>>>0) % 3)&-1;
  $40 = ($39|0)!=(0);
  if ($40) {
   label = 13;
  }
 }
 do {
  if ((label|0) == 13) {
   $41 = $0;
   $42 = (($41) + 386|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = ($44|0)!=(3);
   if ($45) {
    $46 = $0;
    _png_warning($46,59280);
    $47 = $0;
    $48 = $2;
    (_png_crc_finish($47,$48)|0);
    STACKTOP = sp;return;
   } else {
    $49 = $0;
    _png_error($49,59280);
    break;
   }
  }
 } while(0);
 $50 = $2;
 $51 = (($50|0) / 3)&-1;
 $num = $51;
 $i = 0;
 $pal_ptr = $palette;
 while(1) {
  $52 = $i;
  $53 = $num;
  $54 = ($52|0)<($53|0);
  if (!($54)) {
   break;
  }
  $55 = $0;
  _png_crc_read($55,$buf,3);
  $56 = HEAP8[$buf>>0]|0;
  $57 = $pal_ptr;
  HEAP8[$57>>0] = $56;
  $58 = (($buf) + 1|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = $pal_ptr;
  $61 = (($60) + 1|0);
  HEAP8[$61>>0] = $59;
  $62 = (($buf) + 2|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = $pal_ptr;
  $65 = (($64) + 2|0);
  HEAP8[$65>>0] = $63;
  $66 = $i;
  $67 = (($66) + 1)|0;
  $i = $67;
  $68 = $pal_ptr;
  $69 = (($68) + 3|0);
  $pal_ptr = $69;
 }
 $70 = $0;
 (_png_crc_finish($70,0)|0);
 $71 = $0;
 $72 = $1;
 $73 = $num;
 _png_set_PLTE($71,$72,$palette,$73);
 $74 = $0;
 $75 = (($74) + 386|0);
 $76 = HEAP8[$75>>0]|0;
 $77 = $76&255;
 $78 = ($77|0)==(3);
 if (!($78)) {
  STACKTOP = sp;return;
 }
 $79 = $1;
 $80 = ($79|0)!=(0|0);
 if ($80) {
  $81 = $1;
  $82 = (($81) + 8|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $83 & 16;
  $85 = ($84|0)!=(0);
  if ($85) {
   $86 = $0;
   $87 = (($86) + 374|0);
   $88 = HEAP16[$87>>1]|0;
   $89 = $88&65535;
   $90 = $num;
   $91 = $90&65535;
   $92 = $91&65535;
   $93 = ($89|0)>($92|0);
   if ($93) {
    $94 = $0;
    _png_warning($94,59304);
    $95 = $num;
    $96 = $95&65535;
    $97 = $0;
    $98 = (($97) + 374|0);
    HEAP16[$98>>1] = $96;
   }
   $99 = $1;
   $100 = (($99) + 22|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = $101&65535;
   $103 = $num;
   $104 = $103&65535;
   $105 = $104&65535;
   $106 = ($102|0)>($105|0);
   if ($106) {
    $107 = $0;
    _png_warning($107,59344);
    $108 = $num;
    $109 = $108&65535;
    $110 = $1;
    $111 = (($110) + 22|0);
    HEAP16[$111>>1] = $109;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_handle_IEND($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $0;
  $9 = (($8) + 196|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 4;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $13 = $0;
  _png_error($13,59392);
 }
 $14 = $0;
 $15 = (($14) + 196|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16 | 24;
 HEAP32[$15>>2] = $17;
 $18 = $2;
 $19 = ($18|0)!=(0);
 if (!($19)) {
  $21 = $0;
  $22 = $2;
  (_png_crc_finish($21,$22)|0);
  $23 = $1;
  $1 = $23;
  STACKTOP = sp;return;
 }
 $20 = $0;
 _png_warning($20,59416);
 $21 = $0;
 $22 = $2;
 (_png_crc_finish($21,$22)|0);
 $23 = $1;
 $1 = $23;
 STACKTOP = sp;return;
}
function _png_handle_gAMA($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $buf = 0, $file_gamma = 0.0;
 var $igamma = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,59480);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = $0;
   _png_warning($22,59504);
  }
 } else {
  $8 = $0;
  _png_error($8,59448);
 }
 $23 = $1;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = $1;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $27 & 1;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = $1;
   $31 = (($30) + 8|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = $32 & 2048;
   $34 = ($33|0)!=(0);
   if (!($34)) {
    $35 = $0;
    _png_warning($35,59528);
    $36 = $0;
    $37 = $2;
    (_png_crc_finish($36,$37)|0);
    STACKTOP = sp;return;
   }
  }
 }
 $38 = $2;
 $39 = ($38|0)!=(4);
 if ($39) {
  $40 = $0;
  _png_warning($40,59552);
  $41 = $0;
  $42 = $2;
  (_png_crc_finish($41,$42)|0);
  STACKTOP = sp;return;
 }
 $43 = $0;
 _png_crc_read($43,$buf,4);
 $44 = $0;
 $45 = (_png_crc_finish($44,0)|0);
 $46 = ($45|0)!=(0);
 if ($46) {
  STACKTOP = sp;return;
 }
 $47 = (_png_get_uint_32($buf)|0);
 $igamma = $47;
 $48 = $igamma;
 $49 = ($48|0)==(0);
 if ($49) {
  $50 = $0;
  _png_warning($50,59584);
  STACKTOP = sp;return;
 }
 $51 = $1;
 $52 = ($51|0)!=(0|0);
 do {
  if ($52) {
   $53 = $1;
   $54 = (($53) + 8|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = $55 & 2048;
   $57 = ($56|0)!=(0);
   if ($57) {
    $58 = $igamma;
    $59 = ($58|0)<(45000);
    if (!($59)) {
     $60 = $igamma;
     $61 = ($60|0)>(46000);
     if (!($61)) {
      break;
     }
    }
    $62 = $0;
    _png_warning($62,59624);
    $63 = HEAP32[_stderr>>2]|0;
    $64 = $igamma;
    HEAP32[$vararg_buffer>>2] = $64;
    (_fprintf(($63|0),(59680|0),($vararg_buffer|0))|0);
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $65 = $igamma;
 $66 = (+($65|0));
 $67 = $66 / 1.0E+5;
 $file_gamma = $67;
 $68 = $file_gamma;
 $69 = $0;
 $70 = (($69) + 440|0);
 HEAPF32[$70>>2] = $68;
 $71 = $0;
 $72 = $1;
 $73 = $file_gamma;
 $74 = $73;
 _png_set_gAMA($71,$72,$74);
 $75 = $0;
 $76 = $1;
 $77 = $igamma;
 _png_set_gAMA_fixed($75,$76,$77);
 STACKTOP = sp;return;
}
function _png_handle_sBIT($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0, $truelen = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $buf = sp + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = (($buf) + 3|0);
 HEAP8[$3>>0] = 0;
 $4 = (($buf) + 2|0);
 HEAP8[$4>>0] = 0;
 $5 = (($buf) + 1|0);
 HEAP8[$5>>0] = 0;
 HEAP8[$buf>>0] = 0;
 $6 = $0;
 $7 = (($6) + 196|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)!=(0);
 if ($10) {
  $12 = $0;
  $13 = (($12) + 196|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $14 & 4;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = $0;
   _png_warning($17,59736);
   $18 = $0;
   $19 = $2;
   (_png_crc_finish($18,$19)|0);
   STACKTOP = sp;return;
  }
  $20 = $0;
  $21 = (($20) + 196|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $22 & 2;
  $24 = ($23|0)!=(0);
  if ($24) {
   $25 = $0;
   _png_warning($25,59760);
  }
 } else {
  $11 = $0;
  _png_error($11,59704);
 }
 $26 = $1;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = $1;
  $29 = (($28) + 8|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30 & 2;
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = $0;
   _png_warning($33,59784);
   $34 = $0;
   $35 = $2;
   (_png_crc_finish($34,$35)|0);
   STACKTOP = sp;return;
  }
 }
 $36 = $0;
 $37 = (($36) + 386|0);
 $38 = HEAP8[$37>>0]|0;
 $39 = $38&255;
 $40 = ($39|0)==(3);
 if ($40) {
  $truelen = 3;
 } else {
  $41 = $0;
  $42 = (($41) + 390|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $truelen = $44;
 }
 $45 = $2;
 $46 = $truelen;
 $47 = ($45|0)!=($46|0);
 if (!($47)) {
  $48 = $2;
  $49 = ($48>>>0)>(4);
  if (!($49)) {
   $53 = $0;
   $54 = $truelen;
   _png_crc_read($53,$buf,$54);
   $55 = $0;
   $56 = (_png_crc_finish($55,0)|0);
   $57 = ($56|0)!=(0);
   if ($57) {
    STACKTOP = sp;return;
   }
   $58 = $0;
   $59 = (($58) + 386|0);
   $60 = HEAP8[$59>>0]|0;
   $61 = $60&255;
   $62 = $61 & 2;
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = HEAP8[$buf>>0]|0;
    $65 = $0;
    $66 = (($65) + 472|0);
    HEAP8[$66>>0] = $64;
    $67 = (($buf) + 1|0);
    $68 = HEAP8[$67>>0]|0;
    $69 = $0;
    $70 = (($69) + 472|0);
    $71 = (($70) + 1|0);
    HEAP8[$71>>0] = $68;
    $72 = (($buf) + 2|0);
    $73 = HEAP8[$72>>0]|0;
    $74 = $0;
    $75 = (($74) + 472|0);
    $76 = (($75) + 2|0);
    HEAP8[$76>>0] = $73;
    $77 = (($buf) + 3|0);
    $78 = HEAP8[$77>>0]|0;
    $79 = $0;
    $80 = (($79) + 472|0);
    $81 = (($80) + 4|0);
    HEAP8[$81>>0] = $78;
   } else {
    $82 = HEAP8[$buf>>0]|0;
    $83 = $0;
    $84 = (($83) + 472|0);
    $85 = (($84) + 3|0);
    HEAP8[$85>>0] = $82;
    $86 = HEAP8[$buf>>0]|0;
    $87 = $0;
    $88 = (($87) + 472|0);
    HEAP8[$88>>0] = $86;
    $89 = HEAP8[$buf>>0]|0;
    $90 = $0;
    $91 = (($90) + 472|0);
    $92 = (($91) + 1|0);
    HEAP8[$92>>0] = $89;
    $93 = HEAP8[$buf>>0]|0;
    $94 = $0;
    $95 = (($94) + 472|0);
    $96 = (($95) + 2|0);
    HEAP8[$96>>0] = $93;
    $97 = (($buf) + 1|0);
    $98 = HEAP8[$97>>0]|0;
    $99 = $0;
    $100 = (($99) + 472|0);
    $101 = (($100) + 4|0);
    HEAP8[$101>>0] = $98;
   }
   $102 = $0;
   $103 = $1;
   $104 = $0;
   $105 = (($104) + 472|0);
   _png_set_sBIT($102,$103,$105);
   STACKTOP = sp;return;
  }
 }
 $50 = $0;
 _png_warning($50,59808);
 $51 = $0;
 $52 = $2;
 (_png_crc_finish($51,$52)|0);
 STACKTOP = sp;return;
}
function _png_handle_cHRM($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0.0, $106 = 0.0, $107 = 0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0.0, $113 = 0, $114 = 0.0, $115 = 0.0;
 var $116 = 0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0.0, $166 = 0.0, $167 = 0.0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0.0, $171 = 0.0, $172 = 0.0, $173 = 0.0, $174 = 0.0, $175 = 0.0, $176 = 0.0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0.0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0, $99 = 0.0, $blue_x = 0.0, $blue_y = 0.0, $buf = 0, $green_x = 0.0, $green_y = 0.0, $int_x_blue = 0, $int_x_green = 0, $int_x_red = 0, $int_x_white = 0, $int_y_blue = 0, $int_y_green = 0;
 var $int_y_red = 0, $int_y_white = 0, $red_x = 0.0, $red_y = 0.0, $uint_x = 0, $uint_y = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $white_x = 0.0, $white_y = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer4 = sp + 32|0;
 $vararg_buffer = sp;
 $buf = sp + 148|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,59872);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = $0;
   _png_warning($22,59896);
  }
 } else {
  $8 = $0;
  _png_error($8,59840);
 }
 $23 = $1;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = $1;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $27 & 4;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = $1;
   $31 = (($30) + 8|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = $32 & 2048;
   $34 = ($33|0)!=(0);
   if (!($34)) {
    $35 = $0;
    _png_warning($35,59928);
    $36 = $0;
    $37 = $2;
    (_png_crc_finish($36,$37)|0);
    STACKTOP = sp;return;
   }
  }
 }
 $38 = $2;
 $39 = ($38|0)!=(32);
 if ($39) {
  $40 = $0;
  _png_warning($40,59952);
  $41 = $0;
  $42 = $2;
  (_png_crc_finish($41,$42)|0);
  STACKTOP = sp;return;
 }
 $43 = $0;
 _png_crc_read($43,$buf,4);
 $44 = (_png_get_uint_32($buf)|0);
 $uint_x = $44;
 $45 = $0;
 _png_crc_read($45,$buf,4);
 $46 = (_png_get_uint_32($buf)|0);
 $uint_y = $46;
 $47 = $uint_x;
 $48 = ($47>>>0)>(80000);
 if (!($48)) {
  $49 = $uint_y;
  $50 = ($49>>>0)>(80000);
  if (!($50)) {
   $51 = $uint_x;
   $52 = $uint_y;
   $53 = (($51) + ($52))|0;
   $54 = ($53>>>0)>(100000);
   if (!($54)) {
    $57 = $uint_x;
    $int_x_white = $57;
    $58 = $uint_y;
    $int_y_white = $58;
    $59 = $0;
    _png_crc_read($59,$buf,4);
    $60 = (_png_get_uint_32($buf)|0);
    $uint_x = $60;
    $61 = $0;
    _png_crc_read($61,$buf,4);
    $62 = (_png_get_uint_32($buf)|0);
    $uint_y = $62;
    $63 = $uint_x;
    $64 = $uint_y;
    $65 = (($63) + ($64))|0;
    $66 = ($65>>>0)>(100000);
    if ($66) {
     $67 = $0;
     _png_warning($67,60016);
     $68 = $0;
     (_png_crc_finish($68,16)|0);
     STACKTOP = sp;return;
    }
    $69 = $uint_x;
    $int_x_red = $69;
    $70 = $uint_y;
    $int_y_red = $70;
    $71 = $0;
    _png_crc_read($71,$buf,4);
    $72 = (_png_get_uint_32($buf)|0);
    $uint_x = $72;
    $73 = $0;
    _png_crc_read($73,$buf,4);
    $74 = (_png_get_uint_32($buf)|0);
    $uint_y = $74;
    $75 = $uint_x;
    $76 = $uint_y;
    $77 = (($75) + ($76))|0;
    $78 = ($77>>>0)>(100000);
    if ($78) {
     $79 = $0;
     _png_warning($79,60040);
     $80 = $0;
     (_png_crc_finish($80,8)|0);
     STACKTOP = sp;return;
    }
    $81 = $uint_x;
    $int_x_green = $81;
    $82 = $uint_y;
    $int_y_green = $82;
    $83 = $0;
    _png_crc_read($83,$buf,4);
    $84 = (_png_get_uint_32($buf)|0);
    $uint_x = $84;
    $85 = $0;
    _png_crc_read($85,$buf,4);
    $86 = (_png_get_uint_32($buf)|0);
    $uint_y = $86;
    $87 = $uint_x;
    $88 = $uint_y;
    $89 = (($87) + ($88))|0;
    $90 = ($89>>>0)>(100000);
    if ($90) {
     $91 = $0;
     _png_warning($91,60072);
     $92 = $0;
     (_png_crc_finish($92,0)|0);
     STACKTOP = sp;return;
    }
    $93 = $uint_x;
    $int_x_blue = $93;
    $94 = $uint_y;
    $int_y_blue = $94;
    $95 = $int_x_white;
    $96 = (+($95|0));
    $97 = $96 / 1.0E+5;
    $white_x = $97;
    $98 = $int_y_white;
    $99 = (+($98|0));
    $100 = $99 / 1.0E+5;
    $white_y = $100;
    $101 = $int_x_red;
    $102 = (+($101|0));
    $103 = $102 / 1.0E+5;
    $red_x = $103;
    $104 = $int_y_red;
    $105 = (+($104|0));
    $106 = $105 / 1.0E+5;
    $red_y = $106;
    $107 = $int_x_green;
    $108 = (+($107|0));
    $109 = $108 / 1.0E+5;
    $green_x = $109;
    $110 = $int_y_green;
    $111 = (+($110|0));
    $112 = $111 / 1.0E+5;
    $green_y = $112;
    $113 = $int_x_blue;
    $114 = (+($113|0));
    $115 = $114 / 1.0E+5;
    $blue_x = $115;
    $116 = $int_y_blue;
    $117 = (+($116|0));
    $118 = $117 / 1.0E+5;
    $blue_y = $118;
    $119 = $1;
    $120 = ($119|0)!=(0|0);
    if ($120) {
     $121 = $1;
     $122 = (($121) + 8|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = $123 & 2048;
     $125 = ($124|0)!=(0);
     if ($125) {
      $126 = $int_x_white;
      $127 = ($126|0)<(30270);
      do {
       if ($127) {
        label = 43;
       } else {
        $128 = $int_x_white;
        $129 = ($128|0)>(32270);
        if ($129) {
         label = 43;
        } else {
         $130 = $int_y_white;
         $131 = ($130|0)<(31900);
         if ($131) {
          label = 43;
         } else {
          $132 = $int_y_white;
          $133 = ($132|0)>(33900);
          if ($133) {
           label = 43;
          } else {
           $134 = $int_x_red;
           $135 = ($134|0)<(63000);
           if ($135) {
            label = 43;
           } else {
            $136 = $int_x_red;
            $137 = ($136|0)>(65000);
            if ($137) {
             label = 43;
            } else {
             $138 = $int_y_red;
             $139 = ($138|0)<(32000);
             if ($139) {
              label = 43;
              break;
             }
             $140 = $int_y_red;
             $141 = ($140|0)>(34000);
             if ($141) {
              label = 43;
              break;
             }
             $142 = $int_x_green;
             $143 = ($142|0)<(29000);
             if ($143) {
              label = 43;
              break;
             }
             $144 = $int_x_green;
             $145 = ($144|0)>(31000);
             if ($145) {
              label = 43;
              break;
             }
             $146 = $int_y_green;
             $147 = ($146|0)<(59000);
             if ($147) {
              label = 43;
              break;
             }
             $148 = $int_y_green;
             $149 = ($148|0)>(61000);
             if ($149) {
              label = 43;
              break;
             }
             $150 = $int_x_blue;
             $151 = ($150|0)<(14000);
             if ($151) {
              label = 43;
              break;
             }
             $152 = $int_x_blue;
             $153 = ($152|0)>(16000);
             if ($153) {
              label = 43;
              break;
             }
             $154 = $int_y_blue;
             $155 = ($154|0)<(5000);
             if ($155) {
              label = 43;
              break;
             }
             $156 = $int_y_blue;
             $157 = ($156|0)>(7000);
             if ($157) {
              label = 43;
             }
            }
           }
          }
         }
        }
       }
      } while(0);
      if ((label|0) == 43) {
       $158 = $0;
       _png_warning($158,60096);
       $159 = HEAP32[_stderr>>2]|0;
       $160 = $white_x;
       $161 = $160;
       $162 = $white_y;
       $163 = $162;
       $164 = $red_x;
       $165 = $164;
       $166 = $red_y;
       $167 = $166;
       HEAPF64[tempDoublePtr>>3]=$161;HEAP32[$vararg_buffer>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer+4>>2]=HEAP32[tempDoublePtr+4>>2];
       $vararg_ptr1 = (($vararg_buffer) + 8|0);
       HEAPF64[tempDoublePtr>>3]=$163;HEAP32[$vararg_ptr1>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr1+4>>2]=HEAP32[tempDoublePtr+4>>2];
       $vararg_ptr2 = (($vararg_buffer) + 16|0);
       HEAPF64[tempDoublePtr>>3]=$165;HEAP32[$vararg_ptr2>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr2+4>>2]=HEAP32[tempDoublePtr+4>>2];
       $vararg_ptr3 = (($vararg_buffer) + 24|0);
       HEAPF64[tempDoublePtr>>3]=$167;HEAP32[$vararg_ptr3>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr3+4>>2]=HEAP32[tempDoublePtr+4>>2];
       (_fprintf(($159|0),(60152|0),($vararg_buffer|0))|0);
       $168 = HEAP32[_stderr>>2]|0;
       $169 = $green_x;
       $170 = $169;
       $171 = $green_y;
       $172 = $171;
       $173 = $blue_x;
       $174 = $173;
       $175 = $blue_y;
       $176 = $175;
       HEAPF64[tempDoublePtr>>3]=$170;HEAP32[$vararg_buffer4>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer4+4>>2]=HEAP32[tempDoublePtr+4>>2];
       $vararg_ptr7 = (($vararg_buffer4) + 8|0);
       HEAPF64[tempDoublePtr>>3]=$172;HEAP32[$vararg_ptr7>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr7+4>>2]=HEAP32[tempDoublePtr+4>>2];
       $vararg_ptr8 = (($vararg_buffer4) + 16|0);
       HEAPF64[tempDoublePtr>>3]=$174;HEAP32[$vararg_ptr8>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr8+4>>2]=HEAP32[tempDoublePtr+4>>2];
       $vararg_ptr9 = (($vararg_buffer4) + 24|0);
       HEAPF64[tempDoublePtr>>3]=$176;HEAP32[$vararg_ptr9>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr9+4>>2]=HEAP32[tempDoublePtr+4>>2];
       (_fprintf(($168|0),(60184|0),($vararg_buffer4|0))|0);
      }
      $177 = $0;
      (_png_crc_finish($177,0)|0);
      STACKTOP = sp;return;
     }
    }
    $178 = $0;
    $179 = $1;
    $180 = $white_x;
    $181 = $180;
    $182 = $white_y;
    $183 = $182;
    $184 = $red_x;
    $185 = $184;
    $186 = $red_y;
    $187 = $186;
    $188 = $green_x;
    $189 = $188;
    $190 = $green_y;
    $191 = $190;
    $192 = $blue_x;
    $193 = $192;
    $194 = $blue_y;
    $195 = $194;
    _png_set_cHRM($178,$179,$181,$183,$185,$187,$189,$191,$193,$195);
    $196 = $0;
    $197 = $1;
    $198 = $int_x_white;
    $199 = $int_y_white;
    $200 = $int_x_red;
    $201 = $int_y_red;
    $202 = $int_x_green;
    $203 = $int_y_green;
    $204 = $int_x_blue;
    $205 = $int_y_blue;
    _png_set_cHRM_fixed($196,$197,$198,$199,$200,$201,$202,$203,$204,$205);
    $206 = $0;
    $207 = (_png_crc_finish($206,0)|0);
    $208 = ($207|0)!=(0);
    if (!($208)) {
     STACKTOP = sp;return;
    }
    STACKTOP = sp;return;
   }
  }
 }
 $55 = $0;
 _png_warning($55,59984);
 $56 = $0;
 (_png_crc_finish($56,24)|0);
 STACKTOP = sp;return;
}
function _png_handle_sRGB($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $buf = 0, $igamma = 0, $intent = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,60248);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = $0;
   _png_warning($22,60272);
  }
 } else {
  $8 = $0;
  _png_error($8,60216);
 }
 $23 = $1;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = $1;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $27 & 2048;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = $0;
   _png_warning($30,60296);
   $31 = $0;
   $32 = $2;
   (_png_crc_finish($31,$32)|0);
   STACKTOP = sp;return;
  }
 }
 $33 = $2;
 $34 = ($33|0)!=(1);
 if ($34) {
  $35 = $0;
  _png_warning($35,60320);
  $36 = $0;
  $37 = $2;
  (_png_crc_finish($36,$37)|0);
  STACKTOP = sp;return;
 }
 $38 = $0;
 _png_crc_read($38,$buf,1);
 $39 = $0;
 $40 = (_png_crc_finish($39,0)|0);
 $41 = ($40|0)!=(0);
 if ($41) {
  STACKTOP = sp;return;
 }
 $42 = HEAP8[$buf>>0]|0;
 $43 = $42&255;
 $intent = $43;
 $44 = $intent;
 $45 = ($44|0)>=(4);
 if ($45) {
  $46 = $0;
  _png_warning($46,60352);
  STACKTOP = sp;return;
 }
 $47 = $1;
 $48 = ($47|0)!=(0|0);
 if ($48) {
  $49 = $1;
  $50 = (($49) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = $51 & 1;
  $53 = ($52|0)!=(0);
  if ($53) {
   $54 = $1;
   $55 = (($54) + 252|0);
   $56 = HEAP32[$55>>2]|0;
   $igamma = $56;
   $57 = $igamma;
   $58 = ($57|0)<(45000);
   if ($58) {
    label = 22;
   } else {
    $59 = $igamma;
    $60 = ($59|0)>(46000);
    if ($60) {
     label = 22;
    }
   }
   if ((label|0) == 22) {
    $61 = $0;
    _png_warning($61,59624);
    $62 = HEAP32[_stderr>>2]|0;
    $63 = $0;
    $64 = (($63) + 656|0);
    $65 = HEAP32[$64>>2]|0;
    HEAP32[$vararg_buffer>>2] = $65;
    (_fprintf(($62|0),(60376|0),($vararg_buffer|0))|0);
   }
  }
 }
 $66 = $1;
 $67 = ($66|0)!=(0|0);
 if ($67) {
  $68 = $1;
  $69 = (($68) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $70 & 4;
  $72 = ($71|0)!=(0);
  if ($72) {
   $73 = $1;
   $74 = (($73) + 256|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75|0)<(30270);
   do {
    if ($76) {
     label = 42;
    } else {
     $77 = $1;
     $78 = (($77) + 256|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)>(32270);
     if ($80) {
      label = 42;
     } else {
      $81 = $1;
      $82 = (($81) + 260|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = ($83|0)<(31900);
      if ($84) {
       label = 42;
      } else {
       $85 = $1;
       $86 = (($85) + 260|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = ($87|0)>(33900);
       if ($88) {
        label = 42;
       } else {
        $89 = $1;
        $90 = (($89) + 264|0);
        $91 = HEAP32[$90>>2]|0;
        $92 = ($91|0)<(63000);
        if ($92) {
         label = 42;
        } else {
         $93 = $1;
         $94 = (($93) + 264|0);
         $95 = HEAP32[$94>>2]|0;
         $96 = ($95|0)>(65000);
         if ($96) {
          label = 42;
         } else {
          $97 = $1;
          $98 = (($97) + 268|0);
          $99 = HEAP32[$98>>2]|0;
          $100 = ($99|0)<(32000);
          if ($100) {
           label = 42;
          } else {
           $101 = $1;
           $102 = (($101) + 268|0);
           $103 = HEAP32[$102>>2]|0;
           $104 = ($103|0)>(34000);
           if ($104) {
            label = 42;
           } else {
            $105 = $1;
            $106 = (($105) + 272|0);
            $107 = HEAP32[$106>>2]|0;
            $108 = ($107|0)<(29000);
            if ($108) {
             label = 42;
            } else {
             $109 = $1;
             $110 = (($109) + 272|0);
             $111 = HEAP32[$110>>2]|0;
             $112 = ($111|0)>(31000);
             if ($112) {
              label = 42;
             } else {
              $113 = $1;
              $114 = (($113) + 276|0);
              $115 = HEAP32[$114>>2]|0;
              $116 = ($115|0)<(59000);
              if ($116) {
               label = 42;
               break;
              }
              $117 = $1;
              $118 = (($117) + 276|0);
              $119 = HEAP32[$118>>2]|0;
              $120 = ($119|0)>(61000);
              if ($120) {
               label = 42;
               break;
              }
              $121 = $1;
              $122 = (($121) + 280|0);
              $123 = HEAP32[$122>>2]|0;
              $124 = ($123|0)<(14000);
              if ($124) {
               label = 42;
               break;
              }
              $125 = $1;
              $126 = (($125) + 280|0);
              $127 = HEAP32[$126>>2]|0;
              $128 = ($127|0)>(16000);
              if ($128) {
               label = 42;
               break;
              }
              $129 = $1;
              $130 = (($129) + 284|0);
              $131 = HEAP32[$130>>2]|0;
              $132 = ($131|0)<(5000);
              if ($132) {
               label = 42;
               break;
              }
              $133 = $1;
              $134 = (($133) + 284|0);
              $135 = HEAP32[$134>>2]|0;
              $136 = ($135|0)>(7000);
              if ($136) {
               label = 42;
              }
             }
            }
           }
          }
         }
        }
       }
      }
     }
    }
   } while(0);
   if ((label|0) == 42) {
    $137 = $0;
    _png_warning($137,60096);
   }
  }
 }
 $138 = $0;
 $139 = $1;
 $140 = $intent;
 _png_set_sRGB_gAMA_and_cHRM($138,$139,$140);
 STACKTOP = sp;return;
}
function _png_handle_iCCP($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $chunkdata = 0, $compression_type = 0, $data_length = 0, $pC = 0, $prefix_length = 0, $profile = 0, $profile_length = 0, $profile_size = 0, $skip = 0, $slength = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $data_length = sp;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $skip = 0;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,60440);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = $0;
   _png_warning($22,60464);
  }
 } else {
  $8 = $0;
  _png_error($8,60408);
 }
 $23 = $1;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = $1;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $27 & 4096;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = $0;
   _png_warning($30,60488);
   $31 = $0;
   $32 = $2;
   (_png_crc_finish($31,$32)|0);
   STACKTOP = sp;return;
  }
 }
 $33 = $0;
 $34 = $2;
 $35 = (($34) + 1)|0;
 $36 = (_png_malloc($33,$35)|0);
 $chunkdata = $36;
 $37 = $2;
 $slength = $37;
 $38 = $0;
 $39 = $chunkdata;
 $40 = $slength;
 _png_crc_read($38,$39,$40);
 $41 = $0;
 $42 = $skip;
 $43 = (_png_crc_finish($41,$42)|0);
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = $0;
  $46 = $chunkdata;
  _png_free($45,$46);
  STACKTOP = sp;return;
 }
 $47 = $slength;
 $48 = $chunkdata;
 $49 = (($48) + ($47)|0);
 HEAP8[$49>>0] = 0;
 $50 = $chunkdata;
 $profile = $50;
 while(1) {
  $51 = $profile;
  $52 = HEAP8[$51>>0]|0;
  $53 = ($52<<24>>24)!=(0);
  if (!($53)) {
   break;
  }
  $54 = $profile;
  $55 = (($54) + 1|0);
  $profile = $55;
 }
 $56 = $profile;
 $57 = (($56) + 1|0);
 $profile = $57;
 $58 = $profile;
 $59 = $chunkdata;
 $60 = $slength;
 $61 = (($59) + ($60)|0);
 $62 = (($61) + -1|0);
 $63 = ($58>>>0)>=($62>>>0);
 if ($63) {
  $64 = $0;
  $65 = $chunkdata;
  _png_free($64,$65);
  $66 = $0;
  _png_warning($66,60512);
  STACKTOP = sp;return;
 }
 $67 = $profile;
 $68 = (($67) + 1|0);
 $profile = $68;
 $69 = HEAP8[$67>>0]|0;
 $compression_type = $69;
 $70 = $compression_type;
 $71 = ($70<<24>>24)!=(0);
 if ($71) {
  $72 = $0;
  _png_warning($72,60536);
  $compression_type = 0;
 }
 $73 = $profile;
 $74 = $chunkdata;
 $75 = $73;
 $76 = $74;
 $77 = (($75) - ($76))|0;
 $prefix_length = $77;
 $78 = $0;
 $79 = $compression_type;
 $80 = $79&255;
 $81 = $chunkdata;
 $82 = $slength;
 $83 = $prefix_length;
 $84 = (_png_decompress_chunk($78,$80,$81,$82,$83,$data_length)|0);
 $chunkdata = $84;
 $85 = HEAP32[$data_length>>2]|0;
 $86 = $prefix_length;
 $87 = (($85) - ($86))|0;
 $profile_length = $87;
 $88 = $prefix_length;
 $89 = HEAP32[$data_length>>2]|0;
 $90 = ($88>>>0)>($89>>>0);
 if (!($90)) {
  $91 = $profile_length;
  $92 = ($91>>>0)<(4);
  if (!($92)) {
   $96 = $chunkdata;
   $97 = $prefix_length;
   $98 = (($96) + ($97)|0);
   $pC = $98;
   $99 = $pC;
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = $101 << 24;
   $103 = $pC;
   $104 = (($103) + 1|0);
   $105 = HEAP8[$104>>0]|0;
   $106 = $105&255;
   $107 = $106 << 16;
   $108 = $102 | $107;
   $109 = $pC;
   $110 = (($109) + 2|0);
   $111 = HEAP8[$110>>0]|0;
   $112 = $111&255;
   $113 = $112 << 8;
   $114 = $108 | $113;
   $115 = $pC;
   $116 = (($115) + 3|0);
   $117 = HEAP8[$116>>0]|0;
   $118 = $117&255;
   $119 = $114 | $118;
   $profile_size = $119;
   $120 = $profile_size;
   $121 = $profile_length;
   $122 = ($120>>>0)<($121>>>0);
   if ($122) {
    $123 = $profile_size;
    $profile_length = $123;
   }
   $124 = $profile_size;
   $125 = $profile_length;
   $126 = ($124>>>0)>($125>>>0);
   if ($126) {
    $127 = $0;
    $128 = $chunkdata;
    _png_free($127,$128);
    $129 = $0;
    _png_warning($129,60632);
    STACKTOP = sp;return;
   } else {
    $130 = $0;
    $131 = $1;
    $132 = $chunkdata;
    $133 = $compression_type;
    $134 = $133&255;
    $135 = $chunkdata;
    $136 = $prefix_length;
    $137 = (($135) + ($136)|0);
    $138 = $profile_length;
    _png_set_iCCP($130,$131,$132,$134,$137,$138);
    $139 = $0;
    $140 = $chunkdata;
    _png_free($139,$140);
    STACKTOP = sp;return;
   }
  }
 }
 $93 = $0;
 $94 = $chunkdata;
 _png_free($93,$94);
 $95 = $0;
 _png_warning($95,60584);
 STACKTOP = sp;return;
}
function _png_handle_sPLT($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $chunkdata = 0, $data_length = 0, $entry_size = 0, $entry_start = 0, $i = 0, $new_palette = 0, $pp = 0, $skip = 0, $slength = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $new_palette = sp + 24|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $skip = 0;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $9 = $0;
   $10 = (($9) + 196|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = $11 & 4;
   $13 = ($12|0)!=(0);
   if (!($13)) {
    break;
   }
   $14 = $0;
   _png_warning($14,60704);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  } else {
   $8 = $0;
   _png_error($8,60672);
  }
 } while(0);
 $17 = $0;
 $18 = $2;
 $19 = (($18) + 1)|0;
 $20 = (_png_malloc($17,$19)|0);
 $chunkdata = $20;
 $21 = $2;
 $slength = $21;
 $22 = $0;
 $23 = $chunkdata;
 $24 = $slength;
 _png_crc_read($22,$23,$24);
 $25 = $0;
 $26 = $skip;
 $27 = (_png_crc_finish($25,$26)|0);
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = $0;
  $30 = $chunkdata;
  _png_free($29,$30);
  STACKTOP = sp;return;
 }
 $31 = $slength;
 $32 = $chunkdata;
 $33 = (($32) + ($31)|0);
 HEAP8[$33>>0] = 0;
 $34 = $chunkdata;
 $entry_start = $34;
 while(1) {
  $35 = $entry_start;
  $36 = HEAP8[$35>>0]|0;
  $37 = ($36<<24>>24)!=(0);
  if (!($37)) {
   break;
  }
  $38 = $entry_start;
  $39 = (($38) + 1|0);
  $entry_start = $39;
 }
 $40 = $entry_start;
 $41 = (($40) + 1|0);
 $entry_start = $41;
 $42 = $entry_start;
 $43 = $chunkdata;
 $44 = $slength;
 $45 = (($43) + ($44)|0);
 $46 = (($45) + -2|0);
 $47 = ($42>>>0)>($46>>>0);
 if ($47) {
  $48 = $0;
  $49 = $chunkdata;
  _png_free($48,$49);
  $50 = $0;
  _png_warning($50,60728);
  STACKTOP = sp;return;
 }
 $51 = $entry_start;
 $52 = (($51) + 1|0);
 $entry_start = $52;
 $53 = HEAP8[$51>>0]|0;
 $54 = (($new_palette) + 4|0);
 HEAP8[$54>>0] = $53;
 $55 = (($new_palette) + 4|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = $56&255;
 $58 = ($57|0)==(8);
 $59 = $58 ? 6 : 10;
 $entry_size = $59;
 $60 = $slength;
 $61 = $entry_start;
 $62 = $chunkdata;
 $63 = $61;
 $64 = $62;
 $65 = (($63) - ($64))|0;
 $66 = (($60) - ($65))|0;
 $data_length = $66;
 $67 = $data_length;
 $68 = $entry_size;
 $69 = (($67|0) % ($68|0))&-1;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = $0;
  $72 = $chunkdata;
  _png_free($71,$72);
  $73 = $0;
  _png_warning($73,60752);
  STACKTOP = sp;return;
 }
 $74 = $data_length;
 $75 = $entry_size;
 $76 = (($74|0) / ($75|0))&-1;
 $77 = (($new_palette) + 12|0);
 HEAP32[$77>>2] = $76;
 $78 = (($new_palette) + 12|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = ($79>>>0)>(429496729);
 if ($80) {
  $81 = $0;
  _png_warning($81,60784);
  STACKTOP = sp;return;
 }
 $82 = $0;
 $83 = (($new_palette) + 12|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = ($84*10)|0;
 $86 = (_png_malloc_warn($82,$85)|0);
 $87 = (($new_palette) + 8|0);
 HEAP32[$87>>2] = $86;
 $88 = (($new_palette) + 8|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = ($89|0)==(0|0);
 if ($90) {
  $91 = $0;
  _png_warning($91,60808);
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $92 = $i;
  $93 = (($new_palette) + 12|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = ($92|0)<($94|0);
  if (!($95)) {
   break;
  }
  $96 = (($new_palette) + 8|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = $i;
  $99 = (($97) + (($98*10)|0)|0);
  $pp = $99;
  $100 = (($new_palette) + 4|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = $101&255;
  $103 = ($102|0)==(8);
  if ($103) {
   $104 = $entry_start;
   $105 = (($104) + 1|0);
   $entry_start = $105;
   $106 = HEAP8[$104>>0]|0;
   $107 = $106&255;
   $108 = $pp;
   HEAP16[$108>>1] = $107;
   $109 = $entry_start;
   $110 = (($109) + 1|0);
   $entry_start = $110;
   $111 = HEAP8[$109>>0]|0;
   $112 = $111&255;
   $113 = $pp;
   $114 = (($113) + 2|0);
   HEAP16[$114>>1] = $112;
   $115 = $entry_start;
   $116 = (($115) + 1|0);
   $entry_start = $116;
   $117 = HEAP8[$115>>0]|0;
   $118 = $117&255;
   $119 = $pp;
   $120 = (($119) + 4|0);
   HEAP16[$120>>1] = $118;
   $121 = $entry_start;
   $122 = (($121) + 1|0);
   $entry_start = $122;
   $123 = HEAP8[$121>>0]|0;
   $124 = $123&255;
   $125 = $pp;
   $126 = (($125) + 6|0);
   HEAP16[$126>>1] = $124;
  } else {
   $127 = $entry_start;
   $128 = (_png_get_uint_16($127)|0);
   $129 = $pp;
   HEAP16[$129>>1] = $128;
   $130 = $entry_start;
   $131 = (($130) + 2|0);
   $entry_start = $131;
   $132 = $entry_start;
   $133 = (_png_get_uint_16($132)|0);
   $134 = $pp;
   $135 = (($134) + 2|0);
   HEAP16[$135>>1] = $133;
   $136 = $entry_start;
   $137 = (($136) + 2|0);
   $entry_start = $137;
   $138 = $entry_start;
   $139 = (_png_get_uint_16($138)|0);
   $140 = $pp;
   $141 = (($140) + 4|0);
   HEAP16[$141>>1] = $139;
   $142 = $entry_start;
   $143 = (($142) + 2|0);
   $entry_start = $143;
   $144 = $entry_start;
   $145 = (_png_get_uint_16($144)|0);
   $146 = $pp;
   $147 = (($146) + 6|0);
   HEAP16[$147>>1] = $145;
   $148 = $entry_start;
   $149 = (($148) + 2|0);
   $entry_start = $149;
  }
  $150 = $entry_start;
  $151 = (_png_get_uint_16($150)|0);
  $152 = $pp;
  $153 = (($152) + 8|0);
  HEAP16[$153>>1] = $151;
  $154 = $entry_start;
  $155 = (($154) + 2|0);
  $entry_start = $155;
  $156 = $i;
  $157 = (($156) + 1)|0;
  $i = $157;
 }
 $158 = $chunkdata;
 HEAP32[$new_palette>>2] = $158;
 $159 = $0;
 $160 = $1;
 _png_set_sPLT($159,$160,$new_palette,1);
 $161 = $0;
 $162 = $chunkdata;
 _png_free($161,$162);
 $163 = $0;
 $164 = (($new_palette) + 8|0);
 $165 = HEAP32[$164>>2]|0;
 _png_free($163,$165);
 STACKTOP = sp;return;
}
function _png_handle_tRNS($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $bit_mask = 0, $buf = 0, $buf1 = 0, $readbuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $readbuf = sp + 24|0;
 $buf = sp + 22|0;
 $buf1 = sp + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 387|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = 1 << $6;
 $8 = (($7) - 1)|0;
 $bit_mask = $8;
 $9 = $0;
 $10 = (($9) + 196|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 & 1;
 $13 = ($12|0)!=(0);
 if ($13) {
  $15 = $0;
  $16 = (($15) + 196|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $17 & 4;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = $0;
   _png_warning($20,60880);
   $21 = $0;
   $22 = $2;
   (_png_crc_finish($21,$22)|0);
   STACKTOP = sp;return;
  }
  $23 = $1;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = $1;
   $26 = (($25) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $27 & 16;
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = $0;
    _png_warning($30,60904);
    $31 = $0;
    $32 = $2;
    (_png_crc_finish($31,$32)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $14 = $0;
  _png_error($14,60848);
 }
 $33 = $0;
 $34 = (($33) + 386|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35&255;
 $37 = ($36|0)==(0);
 do {
  if ($37) {
   $38 = $2;
   $39 = ($38|0)!=(2);
   if (!($39)) {
    $43 = $0;
    _png_crc_read($43,$buf,2);
    $44 = $0;
    $45 = (($44) + 374|0);
    HEAP16[$45>>1] = 1;
    $46 = (_png_get_uint_16($buf)|0);
    $47 = $46&65535;
    $48 = $bit_mask;
    $49 = $47 & $48;
    $50 = $49&65535;
    $51 = $0;
    $52 = (($51) + 488|0);
    $53 = (($52) + 8|0);
    HEAP16[$53>>1] = $50;
    break;
   }
   $40 = $0;
   _png_warning($40,60928);
   $41 = $0;
   $42 = $2;
   (_png_crc_finish($41,$42)|0);
   STACKTOP = sp;return;
  } else {
   $54 = $0;
   $55 = (($54) + 386|0);
   $56 = HEAP8[$55>>0]|0;
   $57 = $56&255;
   $58 = ($57|0)==(2);
   do {
    if ($58) {
     $59 = $2;
     $60 = ($59|0)!=(6);
     if (!($60)) {
      $64 = $0;
      $65 = $2;
      _png_crc_read($64,$buf1,$65);
      $66 = $0;
      $67 = (($66) + 374|0);
      HEAP16[$67>>1] = 1;
      $68 = (_png_get_uint_16($buf1)|0);
      $69 = $68&65535;
      $70 = $bit_mask;
      $71 = $69 & $70;
      $72 = $71&65535;
      $73 = $0;
      $74 = (($73) + 488|0);
      $75 = (($74) + 2|0);
      HEAP16[$75>>1] = $72;
      $76 = (($buf1) + 2|0);
      $77 = (_png_get_uint_16($76)|0);
      $78 = $77&65535;
      $79 = $bit_mask;
      $80 = $78 & $79;
      $81 = $80&65535;
      $82 = $0;
      $83 = (($82) + 488|0);
      $84 = (($83) + 4|0);
      HEAP16[$84>>1] = $81;
      $85 = (($buf1) + 4|0);
      $86 = (_png_get_uint_16($85)|0);
      $87 = $86&65535;
      $88 = $bit_mask;
      $89 = $87 & $88;
      $90 = $89&65535;
      $91 = $0;
      $92 = (($91) + 488|0);
      $93 = (($92) + 6|0);
      HEAP16[$93>>1] = $90;
      break;
     }
     $61 = $0;
     _png_warning($61,60928);
     $62 = $0;
     $63 = $2;
     (_png_crc_finish($62,$63)|0);
     STACKTOP = sp;return;
    } else {
     $94 = $0;
     $95 = (($94) + 386|0);
     $96 = HEAP8[$95>>0]|0;
     $97 = $96&255;
     $98 = ($97|0)==(3);
     if (!($98)) {
      $127 = $0;
      _png_warning($127,61016);
      $128 = $0;
      $129 = $2;
      (_png_crc_finish($128,$129)|0);
      STACKTOP = sp;return;
     }
     $99 = $0;
     $100 = (($99) + 196|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = $101 & 2;
     $103 = ($102|0)!=(0);
     if (!($103)) {
      $104 = $0;
      _png_warning($104,60960);
     }
     $105 = $2;
     $106 = $0;
     $107 = (($106) + 372|0);
     $108 = HEAP16[$107>>1]|0;
     $109 = $108&65535;
     $110 = ($105>>>0)>($109>>>0);
     if (!($110)) {
      $111 = $2;
      $112 = ($111>>>0)>(256);
      if (!($112)) {
       $116 = $2;
       $117 = ($116|0)==(0);
       if ($117) {
        $118 = $0;
        _png_warning($118,60992);
        $119 = $0;
        $120 = $2;
        (_png_crc_finish($119,$120)|0);
        STACKTOP = sp;return;
       } else {
        $121 = $0;
        $122 = $2;
        _png_crc_read($121,$readbuf,$122);
        $123 = $2;
        $124 = $123&65535;
        $125 = $0;
        $126 = (($125) + 374|0);
        HEAP16[$126>>1] = $124;
        break;
       }
      }
     }
     $113 = $0;
     _png_warning($113,60928);
     $114 = $0;
     $115 = $2;
     (_png_crc_finish($114,$115)|0);
     STACKTOP = sp;return;
    }
   } while(0);
  }
 } while(0);
 $130 = $0;
 $131 = (_png_crc_finish($130,0)|0);
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = $0;
  $134 = (($133) + 374|0);
  HEAP16[$134>>1] = 0;
  STACKTOP = sp;return;
 } else {
  $135 = $0;
  $136 = $1;
  $137 = $0;
  $138 = (($137) + 374|0);
  $139 = HEAP16[$138>>1]|0;
  $140 = $139&65535;
  $141 = $0;
  $142 = (($141) + 488|0);
  _png_set_tRNS($135,$136,$readbuf,$140,$142);
  STACKTOP = sp;return;
 }
}
function _png_handle_bKGD($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0, $truelen = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $buf = sp + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,61096);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 386|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)==(3);
  if ($21) {
   $22 = $0;
   $23 = (($22) + 196|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = $24 & 2;
   $26 = ($25|0)!=(0);
   if (!($26)) {
    $27 = $0;
    _png_warning($27,61120);
    $28 = $0;
    $29 = $2;
    (_png_crc_finish($28,$29)|0);
    STACKTOP = sp;return;
   }
  }
  $30 = $1;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = $1;
   $33 = (($32) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $34 & 32;
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = $0;
    _png_warning($37,61152);
    $38 = $0;
    $39 = $2;
    (_png_crc_finish($38,$39)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,61064);
 }
 $40 = $0;
 $41 = (($40) + 386|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = ($43|0)==(3);
 if ($44) {
  $truelen = 1;
 } else {
  $45 = $0;
  $46 = (($45) + 386|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = $48 & 2;
  $50 = ($49|0)!=(0);
  if ($50) {
   $truelen = 6;
  } else {
   $truelen = 2;
  }
 }
 $51 = $2;
 $52 = $truelen;
 $53 = ($51|0)!=($52|0);
 if ($53) {
  $54 = $0;
  _png_warning($54,61176);
  $55 = $0;
  $56 = $2;
  (_png_crc_finish($55,$56)|0);
  STACKTOP = sp;return;
 }
 $57 = $0;
 $58 = $truelen;
 _png_crc_read($57,$buf,$58);
 $59 = $0;
 $60 = (_png_crc_finish($59,0)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  STACKTOP = sp;return;
 }
 $62 = $0;
 $63 = (($62) + 386|0);
 $64 = HEAP8[$63>>0]|0;
 $65 = $64&255;
 $66 = ($65|0)==(3);
 if ($66) {
  $67 = HEAP8[$buf>>0]|0;
  $68 = $0;
  $69 = (($68) + 404|0);
  HEAP8[$69>>0] = $67;
  $70 = $1;
  $71 = (($70) + 20|0);
  $72 = HEAP16[$71>>1]|0;
  $73 = ($72<<16>>16)!=(0);
  do {
   if ($73) {
    $74 = HEAP8[$buf>>0]|0;
    $75 = $74&255;
    $76 = $1;
    $77 = (($76) + 20|0);
    $78 = HEAP16[$77>>1]|0;
    $79 = $78&65535;
    $80 = ($75|0)>($79|0);
    if (!($80)) {
     $82 = HEAP8[$buf>>0]|0;
     $83 = $82&255;
     $84 = $0;
     $85 = (($84) + 368|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = (($86) + (($83*3)|0)|0);
     $88 = HEAP8[$87>>0]|0;
     $89 = $88&255;
     $90 = $0;
     $91 = (($90) + 404|0);
     $92 = (($91) + 2|0);
     HEAP16[$92>>1] = $89;
     $93 = HEAP8[$buf>>0]|0;
     $94 = $93&255;
     $95 = $0;
     $96 = (($95) + 368|0);
     $97 = HEAP32[$96>>2]|0;
     $98 = (($97) + (($94*3)|0)|0);
     $99 = (($98) + 1|0);
     $100 = HEAP8[$99>>0]|0;
     $101 = $100&255;
     $102 = $0;
     $103 = (($102) + 404|0);
     $104 = (($103) + 4|0);
     HEAP16[$104>>1] = $101;
     $105 = HEAP8[$buf>>0]|0;
     $106 = $105&255;
     $107 = $0;
     $108 = (($107) + 368|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = (($109) + (($106*3)|0)|0);
     $111 = (($110) + 2|0);
     $112 = HEAP8[$111>>0]|0;
     $113 = $112&255;
     $114 = $0;
     $115 = (($114) + 404|0);
     $116 = (($115) + 6|0);
     HEAP16[$116>>1] = $113;
     break;
    }
    $81 = $0;
    _png_warning($81,61208);
    STACKTOP = sp;return;
   }
  } while(0);
 } else {
  $117 = $0;
  $118 = (($117) + 386|0);
  $119 = HEAP8[$118>>0]|0;
  $120 = $119&255;
  $121 = $120 & 2;
  $122 = ($121|0)!=(0);
  if ($122) {
   $136 = (_png_get_uint_16($buf)|0);
   $137 = $0;
   $138 = (($137) + 404|0);
   $139 = (($138) + 2|0);
   HEAP16[$139>>1] = $136;
   $140 = (($buf) + 2|0);
   $141 = (_png_get_uint_16($140)|0);
   $142 = $0;
   $143 = (($142) + 404|0);
   $144 = (($143) + 4|0);
   HEAP16[$144>>1] = $141;
   $145 = (($buf) + 4|0);
   $146 = (_png_get_uint_16($145)|0);
   $147 = $0;
   $148 = (($147) + 404|0);
   $149 = (($148) + 6|0);
   HEAP16[$149>>1] = $146;
  } else {
   $123 = (_png_get_uint_16($buf)|0);
   $124 = $0;
   $125 = (($124) + 404|0);
   $126 = (($125) + 8|0);
   HEAP16[$126>>1] = $123;
   $127 = $0;
   $128 = (($127) + 404|0);
   $129 = (($128) + 6|0);
   HEAP16[$129>>1] = $123;
   $130 = $0;
   $131 = (($130) + 404|0);
   $132 = (($131) + 4|0);
   HEAP16[$132>>1] = $123;
   $133 = $0;
   $134 = (($133) + 404|0);
   $135 = (($134) + 2|0);
   HEAP16[$135>>1] = $123;
  }
 }
 $150 = $0;
 $151 = $1;
 $152 = $0;
 $153 = (($152) + 404|0);
 _png_set_bKGD($150,$151,$153);
 STACKTOP = sp;return;
}
function _png_handle_hIST($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $buf = 0, $i = 0, $num = 0, $readbuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 544|0;
 $readbuf = sp + 24|0;
 $buf = sp + 536|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,61280);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $0;
  $18 = (($17) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2;
  $21 = ($20|0)!=(0);
  if (!($21)) {
   $22 = $0;
   _png_warning($22,61304);
   $23 = $0;
   $24 = $2;
   (_png_crc_finish($23,$24)|0);
   STACKTOP = sp;return;
  }
  $25 = $1;
  $26 = ($25|0)!=(0|0);
  if ($26) {
   $27 = $1;
   $28 = (($27) + 8|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $29 & 64;
   $31 = ($30|0)!=(0);
   if ($31) {
    $32 = $0;
    _png_warning($32,61336);
    $33 = $0;
    $34 = $2;
    (_png_crc_finish($33,$34)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,61248);
 }
 $35 = $2;
 $36 = (($35>>>0) / 2)&-1;
 $num = $36;
 $37 = $num;
 $38 = $0;
 $39 = (($38) + 372|0);
 $40 = HEAP16[$39>>1]|0;
 $41 = $40&65535;
 $42 = ($37|0)!=($41|0);
 if (!($42)) {
  $43 = $num;
  $44 = ($43>>>0)>(256);
  if (!($44)) {
   $i = 0;
   while(1) {
    $48 = $i;
    $49 = $num;
    $50 = ($48>>>0)<($49>>>0);
    if (!($50)) {
     break;
    }
    $51 = $0;
    _png_crc_read($51,$buf,2);
    $52 = (_png_get_uint_16($buf)|0);
    $53 = $i;
    $54 = (($readbuf) + ($53<<1)|0);
    HEAP16[$54>>1] = $52;
    $55 = $i;
    $56 = (($55) + 1)|0;
    $i = $56;
   }
   $57 = $0;
   $58 = (_png_crc_finish($57,0)|0);
   $59 = ($58|0)!=(0);
   if ($59) {
    STACKTOP = sp;return;
   } else {
    $60 = $0;
    $61 = $1;
    _png_set_hIST($60,$61,$readbuf);
    STACKTOP = sp;return;
   }
  }
 }
 $45 = $0;
 _png_warning($45,61360);
 $46 = $0;
 $47 = $2;
 (_png_crc_finish($46,$47)|0);
 STACKTOP = sp;return;
}
function _png_handle_pHYs($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $res_x = 0, $res_y = 0, $unit_type = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $buf = sp + 24|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,61424);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $1;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 & 128;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = $0;
    _png_warning($24,61448);
    $25 = $0;
    $26 = $2;
    (_png_crc_finish($25,$26)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,61392);
 }
 $27 = $2;
 $28 = ($27|0)!=(9);
 if ($28) {
  $29 = $0;
  _png_warning($29,61472);
  $30 = $0;
  $31 = $2;
  (_png_crc_finish($30,$31)|0);
  STACKTOP = sp;return;
 }
 $32 = $0;
 _png_crc_read($32,$buf,9);
 $33 = $0;
 $34 = (_png_crc_finish($33,0)|0);
 $35 = ($34|0)!=(0);
 if ($35) {
  STACKTOP = sp;return;
 } else {
  $36 = (_png_get_uint_32($buf)|0);
  $res_x = $36;
  $37 = (($buf) + 4|0);
  $38 = (_png_get_uint_32($37)|0);
  $res_y = $38;
  $39 = (($buf) + 8|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $unit_type = $41;
  $42 = $0;
  $43 = $1;
  $44 = $res_x;
  $45 = $res_y;
  $46 = $unit_type;
  _png_set_pHYs($42,$43,$44,$45,$46);
  STACKTOP = sp;return;
 }
}
function _png_handle_oFFs($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $offset_x = 0, $offset_y = 0, $unit_type = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $buf = sp + 24|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,61536);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $1;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 & 256;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = $0;
    _png_warning($24,61560);
    $25 = $0;
    $26 = $2;
    (_png_crc_finish($25,$26)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,61504);
 }
 $27 = $2;
 $28 = ($27|0)!=(9);
 if ($28) {
  $29 = $0;
  _png_warning($29,61584);
  $30 = $0;
  $31 = $2;
  (_png_crc_finish($30,$31)|0);
  STACKTOP = sp;return;
 }
 $32 = $0;
 _png_crc_read($32,$buf,9);
 $33 = $0;
 $34 = (_png_crc_finish($33,0)|0);
 $35 = ($34|0)!=(0);
 if ($35) {
  STACKTOP = sp;return;
 } else {
  $36 = (_png_get_int_32($buf)|0);
  $offset_x = $36;
  $37 = (($buf) + 4|0);
  $38 = (_png_get_int_32($37)|0);
  $offset_y = $38;
  $39 = (($buf) + 8|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $unit_type = $41;
  $42 = $0;
  $43 = $1;
  $44 = $offset_x;
  $45 = $offset_y;
  $46 = $unit_type;
  _png_set_oFFs($42,$43,$44,$45,$46);
  STACKTOP = sp;return;
 }
}
function _png_handle_pCAL($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $X0 = 0, $X1 = 0, $buf = 0, $endptr = 0, $i = 0, $nparams = 0, $params = 0, $purpose = 0, $slength = 0, $type = 0, $units = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,61648);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $1;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 & 1024;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = $0;
    _png_warning($24,61672);
    $25 = $0;
    $26 = $2;
    (_png_crc_finish($25,$26)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,61616);
 }
 $27 = $0;
 $28 = $2;
 $29 = (($28) + 1)|0;
 $30 = (_png_malloc_warn($27,$29)|0);
 $purpose = $30;
 $31 = $purpose;
 $32 = ($31|0)==(0|0);
 if ($32) {
  $33 = $0;
  _png_warning($33,61696);
  STACKTOP = sp;return;
 }
 $34 = $2;
 $slength = $34;
 $35 = $0;
 $36 = $purpose;
 $37 = $slength;
 _png_crc_read($35,$36,$37);
 $38 = $0;
 $39 = (_png_crc_finish($38,0)|0);
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = $0;
  $42 = $purpose;
  _png_free($41,$42);
  STACKTOP = sp;return;
 }
 $43 = $slength;
 $44 = $purpose;
 $45 = (($44) + ($43)|0);
 HEAP8[$45>>0] = 0;
 $46 = $purpose;
 $buf = $46;
 while(1) {
  $47 = $buf;
  $48 = HEAP8[$47>>0]|0;
  $49 = ($48<<24>>24)!=(0);
  if (!($49)) {
   break;
  }
  $50 = $buf;
  $51 = (($50) + 1|0);
  $buf = $51;
 }
 $52 = $purpose;
 $53 = $slength;
 $54 = (($52) + ($53)|0);
 $endptr = $54;
 $55 = $endptr;
 $56 = $buf;
 $57 = (($56) + 12|0);
 $58 = ($55>>>0)<=($57>>>0);
 if ($58) {
  $59 = $0;
  _png_warning($59,61728);
  $60 = $0;
  $61 = $purpose;
  _png_free($60,$61);
  STACKTOP = sp;return;
 }
 $62 = $buf;
 $63 = (($62) + 1|0);
 $64 = (_png_get_int_32($63)|0);
 $X0 = $64;
 $65 = $buf;
 $66 = (($65) + 5|0);
 $67 = (_png_get_int_32($66)|0);
 $X1 = $67;
 $68 = $buf;
 $69 = (($68) + 9|0);
 $70 = HEAP8[$69>>0]|0;
 $type = $70;
 $71 = $buf;
 $72 = (($71) + 10|0);
 $73 = HEAP8[$72>>0]|0;
 $nparams = $73;
 $74 = $buf;
 $75 = (($74) + 11|0);
 $units = $75;
 $76 = $type;
 $77 = $76&255;
 $78 = ($77|0)==(0);
 if ($78) {
  $79 = $nparams;
  $80 = $79&255;
  $81 = ($80|0)!=(2);
  if (!($81)) {
   label = 22;
  }
 } else {
  label = 22;
 }
 do {
  if ((label|0) == 22) {
   $82 = $type;
   $83 = $82&255;
   $84 = ($83|0)==(1);
   if ($84) {
    $85 = $nparams;
    $86 = $85&255;
    $87 = ($86|0)!=(3);
    if ($87) {
     break;
    }
   }
   $88 = $type;
   $89 = $88&255;
   $90 = ($89|0)==(2);
   if ($90) {
    $91 = $nparams;
    $92 = $91&255;
    $93 = ($92|0)!=(3);
    if ($93) {
     break;
    }
   }
   $94 = $type;
   $95 = $94&255;
   $96 = ($95|0)==(3);
   if ($96) {
    $97 = $nparams;
    $98 = $97&255;
    $99 = ($98|0)!=(4);
    if ($99) {
     break;
    }
   }
   $103 = $type;
   $104 = $103&255;
   $105 = ($104|0)>=(4);
   if ($105) {
    $106 = $0;
    _png_warning($106,61800);
   }
   $107 = $units;
   $buf = $107;
   while(1) {
    $108 = $buf;
    $109 = HEAP8[$108>>0]|0;
    $110 = ($109<<24>>24)!=(0);
    if (!($110)) {
     break;
    }
    $111 = $buf;
    $112 = (($111) + 1|0);
    $buf = $112;
   }
   $113 = $0;
   $114 = $nparams;
   $115 = $114&255;
   $116 = $115<<2;
   $117 = (_png_malloc_warn($113,$116)|0);
   $params = $117;
   $118 = $params;
   $119 = ($118|0)==(0|0);
   if ($119) {
    $120 = $0;
    $121 = $purpose;
    _png_free($120,$121);
    $122 = $0;
    _png_warning($122,61848);
    STACKTOP = sp;return;
   }
   $i = 0;
   while(1) {
    $123 = $i;
    $124 = $nparams;
    $125 = $124&255;
    $126 = ($123|0)<($125|0);
    if (!($126)) {
     label = 50;
     break;
    }
    $127 = $buf;
    $128 = (($127) + 1|0);
    $buf = $128;
    $129 = $buf;
    $130 = $i;
    $131 = $params;
    $132 = (($131) + ($130<<2)|0);
    HEAP32[$132>>2] = $129;
    while(1) {
     $133 = $buf;
     $134 = $endptr;
     $135 = ($133>>>0)<=($134>>>0);
     if ($135) {
      $136 = $buf;
      $137 = HEAP8[$136>>0]|0;
      $138 = $137 << 24 >> 24;
      $139 = ($138|0)!=(0);
      $167 = $139;
     } else {
      $167 = 0;
     }
     if (!($167)) {
      break;
     }
     $140 = $buf;
     $141 = (($140) + 1|0);
     $buf = $141;
    }
    $142 = $buf;
    $143 = $endptr;
    $144 = ($142>>>0)>($143>>>0);
    if ($144) {
     label = 47;
     break;
    }
    $150 = $i;
    $151 = (($150) + 1)|0;
    $i = $151;
   }
   if ((label|0) == 47) {
    $145 = $0;
    _png_warning($145,61728);
    $146 = $0;
    $147 = $purpose;
    _png_free($146,$147);
    $148 = $0;
    $149 = $params;
    _png_free($148,$149);
    STACKTOP = sp;return;
   }
   else if ((label|0) == 50) {
    $152 = $0;
    $153 = $1;
    $154 = $purpose;
    $155 = $X0;
    $156 = $X1;
    $157 = $type;
    $158 = $157&255;
    $159 = $nparams;
    $160 = $159&255;
    $161 = $units;
    $162 = $params;
    _png_set_pCAL($152,$153,$154,$155,$156,$158,$160,$161,$162);
    $163 = $0;
    $164 = $purpose;
    _png_free($163,$164);
    $165 = $0;
    $166 = $params;
    _png_free($165,$166);
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $100 = $0;
 _png_warning($100,61752);
 $101 = $0;
 $102 = $purpose;
 _png_free($101,$102);
 STACKTOP = sp;return;
}
function _png_handle_sCAL($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0.0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0.0, $94 = 0.0, $95 = 0, $96 = 0, $buffer = 0, $ep = 0, $height = 0.0;
 var $slength = 0, $vp = 0, $width = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vp = sp + 20|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $0;
  $10 = (($9) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 4;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = $0;
   _png_warning($14,61912);
   $15 = $0;
   $16 = $2;
   (_png_crc_finish($15,$16)|0);
   STACKTOP = sp;return;
  }
  $17 = $1;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 & 16384;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = $0;
    _png_warning($24,61936);
    $25 = $0;
    $26 = $2;
    (_png_crc_finish($25,$26)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,61880);
 }
 $27 = $0;
 $28 = $2;
 $29 = (($28) + 1)|0;
 $30 = (_png_malloc_warn($27,$29)|0);
 $buffer = $30;
 $31 = $buffer;
 $32 = ($31|0)==(0|0);
 if ($32) {
  $33 = $0;
  _png_warning($33,61960);
  STACKTOP = sp;return;
 }
 $34 = $2;
 $slength = $34;
 $35 = $0;
 $36 = $buffer;
 $37 = $slength;
 _png_crc_read($35,$36,$37);
 $38 = $0;
 $39 = (_png_crc_finish($38,0)|0);
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = $0;
  $42 = $buffer;
  _png_free($41,$42);
  STACKTOP = sp;return;
 }
 $43 = $slength;
 $44 = $buffer;
 $45 = (($44) + ($43)|0);
 HEAP8[$45>>0] = 0;
 $46 = $buffer;
 $47 = (($46) + 1|0);
 $ep = $47;
 $48 = $ep;
 $49 = (+_strtod($48,$vp));
 $width = $49;
 $50 = HEAP32[$vp>>2]|0;
 $51 = HEAP8[$50>>0]|0;
 $52 = ($51<<24>>24)!=(0);
 if ($52) {
  $53 = $0;
  _png_warning($53,62008);
  STACKTOP = sp;return;
 }
 $54 = $buffer;
 $ep = $54;
 while(1) {
  $55 = $ep;
  $56 = HEAP8[$55>>0]|0;
  $57 = ($56<<24>>24)!=(0);
  if (!($57)) {
   break;
  }
  $58 = $ep;
  $59 = (($58) + 1|0);
  $ep = $59;
 }
 $60 = $ep;
 $61 = (($60) + 1|0);
 $ep = $61;
 $62 = $buffer;
 $63 = $slength;
 $64 = (($62) + ($63)|0);
 $65 = $ep;
 $66 = ($64>>>0)<($65>>>0);
 if ($66) {
  $67 = $0;
  _png_warning($67,62048);
  $68 = $0;
  $69 = $buffer;
  _png_free($68,$69);
  STACKTOP = sp;return;
 }
 $70 = $ep;
 $71 = (+_strtod($70,$vp));
 $height = $71;
 $72 = HEAP32[$vp>>2]|0;
 $73 = HEAP8[$72>>0]|0;
 $74 = ($73<<24>>24)!=(0);
 if ($74) {
  $75 = $0;
  _png_warning($75,62072);
  STACKTOP = sp;return;
 }
 $76 = $buffer;
 $77 = $slength;
 $78 = (($76) + ($77)|0);
 $79 = $ep;
 $80 = ($78>>>0)<($79>>>0);
 if (!($80)) {
  $81 = $width;
  $82 = $81 <= 0.0;
  if (!($82)) {
   $83 = $height;
   $84 = $83 <= 0.0;
   if (!($84)) {
    $88 = $0;
    $89 = $1;
    $90 = $buffer;
    $91 = HEAP8[$90>>0]|0;
    $92 = $91 << 24 >> 24;
    $93 = $width;
    $94 = $height;
    _png_set_sCAL($88,$89,$92,$93,$94);
    $95 = $0;
    $96 = $buffer;
    _png_free($95,$96);
    STACKTOP = sp;return;
   }
  }
 }
 $85 = $0;
 _png_warning($85,62112);
 $86 = $0;
 $87 = $buffer;
 _png_free($86,$87);
 STACKTOP = sp;return;
}
function _png_handle_tIME($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $mod_time = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $buf = sp + 24|0;
 $mod_time = sp + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if ($7) {
  $9 = $1;
  $10 = ($9|0)!=(0|0);
  if ($10) {
   $11 = $1;
   $12 = (($11) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = $13 & 512;
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = $0;
    _png_warning($16,62160);
    $17 = $0;
    $18 = $2;
    (_png_crc_finish($17,$18)|0);
    STACKTOP = sp;return;
   }
  }
 } else {
  $8 = $0;
  _png_error($8,62136);
 }
 $19 = $0;
 $20 = (($19) + 196|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $21 & 4;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = $0;
  $25 = (($24) + 196|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $26 | 8;
  HEAP32[$25>>2] = $27;
 }
 $28 = $2;
 $29 = ($28|0)!=(7);
 if ($29) {
  $30 = $0;
  _png_warning($30,62184);
  $31 = $0;
  $32 = $2;
  (_png_crc_finish($31,$32)|0);
  STACKTOP = sp;return;
 }
 $33 = $0;
 _png_crc_read($33,$buf,7);
 $34 = $0;
 $35 = (_png_crc_finish($34,0)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  STACKTOP = sp;return;
 } else {
  $37 = (($buf) + 6|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = (($mod_time) + 6|0);
  HEAP8[$39>>0] = $38;
  $40 = (($buf) + 5|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = (($mod_time) + 5|0);
  HEAP8[$42>>0] = $41;
  $43 = (($buf) + 4|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = (($mod_time) + 4|0);
  HEAP8[$45>>0] = $44;
  $46 = (($buf) + 3|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = (($mod_time) + 3|0);
  HEAP8[$48>>0] = $47;
  $49 = (($buf) + 2|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = (($mod_time) + 2|0);
  HEAP8[$51>>0] = $50;
  $52 = (_png_get_uint_16($buf)|0);
  HEAP16[$mod_time>>1] = $52;
  $53 = $0;
  $54 = $1;
  _png_set_tIME($53,$54,$mod_time);
  STACKTOP = sp;return;
 }
}
function _png_handle_tEXt($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $key = 0, $ret = 0, $skip = 0, $slength = 0, $text = 0, $text_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $skip = 0;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = $0;
  _png_error($8,62216);
 }
 $9 = $0;
 $10 = (($9) + 196|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 & 4;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $0;
  $15 = (($14) + 196|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $16 | 8;
  HEAP32[$15>>2] = $17;
 }
 $18 = $0;
 $19 = $2;
 $20 = (($19) + 1)|0;
 $21 = (_png_malloc_warn($18,$20)|0);
 $key = $21;
 $22 = $key;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = $0;
  _png_warning($24,62248);
  STACKTOP = sp;return;
 }
 $25 = $2;
 $slength = $25;
 $26 = $0;
 $27 = $key;
 $28 = $slength;
 _png_crc_read($26,$27,$28);
 $29 = $0;
 $30 = $skip;
 $31 = (_png_crc_finish($29,$30)|0);
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = $0;
  $34 = $key;
  _png_free($33,$34);
  STACKTOP = sp;return;
 }
 $35 = $slength;
 $36 = $key;
 $37 = (($36) + ($35)|0);
 HEAP8[$37>>0] = 0;
 $38 = $key;
 $text = $38;
 while(1) {
  $39 = $text;
  $40 = HEAP8[$39>>0]|0;
  $41 = ($40<<24>>24)!=(0);
  if (!($41)) {
   break;
  }
  $42 = $text;
  $43 = (($42) + 1|0);
  $text = $43;
 }
 $44 = $text;
 $45 = $key;
 $46 = $slength;
 $47 = (($45) + ($46)|0);
 $48 = ($44|0)!=($47|0);
 if ($48) {
  $49 = $text;
  $50 = (($49) + 1|0);
  $text = $50;
 }
 $51 = $0;
 $52 = (_png_malloc_warn($51,16)|0);
 $text_ptr = $52;
 $53 = $text_ptr;
 $54 = ($53|0)==(0|0);
 if ($54) {
  $55 = $0;
  _png_warning($55,62288);
  $56 = $0;
  $57 = $key;
  _png_free($56,$57);
  STACKTOP = sp;return;
 }
 $58 = $text_ptr;
 HEAP32[$58>>2] = -1;
 $59 = $key;
 $60 = $text_ptr;
 $61 = (($60) + 4|0);
 HEAP32[$61>>2] = $59;
 $62 = $text;
 $63 = $text_ptr;
 $64 = (($63) + 8|0);
 HEAP32[$64>>2] = $62;
 $65 = $text;
 $66 = (_strlen(($65|0))|0);
 $67 = $text_ptr;
 $68 = (($67) + 12|0);
 HEAP32[$68>>2] = $66;
 $69 = $0;
 $70 = $1;
 $71 = $text_ptr;
 $72 = (_png_set_text_2($69,$70,$71,1)|0);
 $ret = $72;
 $73 = $0;
 $74 = $key;
 _png_free($73,$74);
 $75 = $0;
 $76 = $text_ptr;
 _png_free($75,$76);
 $77 = $ret;
 $78 = ($77|0)!=(0);
 if (!($78)) {
  STACKTOP = sp;return;
 }
 $79 = $0;
 _png_warning($79,62336);
 STACKTOP = sp;return;
}
function _png_handle_zTXt($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $chunkdata = 0, $comp_type = 0, $data_len = 0, $prefix_len = 0, $ret = 0, $slength = 0, $text = 0, $text_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $data_len = sp;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = $0;
  _png_error($8,62384);
 }
 $9 = $0;
 $10 = (($9) + 196|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11 & 4;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $0;
  $15 = (($14) + 196|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $16 | 8;
  HEAP32[$15>>2] = $17;
 }
 $18 = $0;
 $19 = $2;
 $20 = (($19) + 1)|0;
 $21 = (_png_malloc_warn($18,$20)|0);
 $chunkdata = $21;
 $22 = $chunkdata;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = $0;
  _png_warning($24,62416);
  STACKTOP = sp;return;
 }
 $25 = $2;
 $slength = $25;
 $26 = $0;
 $27 = $chunkdata;
 $28 = $slength;
 _png_crc_read($26,$27,$28);
 $29 = $0;
 $30 = (_png_crc_finish($29,0)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = $0;
  $33 = $chunkdata;
  _png_free($32,$33);
  STACKTOP = sp;return;
 }
 $34 = $slength;
 $35 = $chunkdata;
 $36 = (($35) + ($34)|0);
 HEAP8[$36>>0] = 0;
 $37 = $chunkdata;
 $text = $37;
 while(1) {
  $38 = $text;
  $39 = HEAP8[$38>>0]|0;
  $40 = ($39<<24>>24)!=(0);
  if (!($40)) {
   break;
  }
  $41 = $text;
  $42 = (($41) + 1|0);
  $text = $42;
 }
 $43 = $text;
 $44 = $chunkdata;
 $45 = $slength;
 $46 = (($44) + ($45)|0);
 $47 = (($46) + -2|0);
 $48 = ($43>>>0)>=($47>>>0);
 if ($48) {
  $49 = $0;
  _png_warning($49,62456);
  $50 = $0;
  $51 = $chunkdata;
  _png_free($50,$51);
  STACKTOP = sp;return;
 }
 $52 = $text;
 $53 = (($52) + 1|0);
 $text = $53;
 $54 = HEAP8[$53>>0]|0;
 $55 = $54 << 24 >> 24;
 $comp_type = $55;
 $56 = $comp_type;
 $57 = ($56|0)!=(0);
 if ($57) {
  $58 = $0;
  _png_warning($58,62480);
  $comp_type = 0;
 }
 $59 = $text;
 $60 = (($59) + 1|0);
 $text = $60;
 $61 = $text;
 $62 = $chunkdata;
 $63 = $61;
 $64 = $62;
 $65 = (($63) - ($64))|0;
 $prefix_len = $65;
 $66 = $0;
 $67 = $comp_type;
 $68 = $chunkdata;
 $69 = $2;
 $70 = $prefix_len;
 $71 = (_png_decompress_chunk($66,$67,$68,$69,$70,$data_len)|0);
 $chunkdata = $71;
 $72 = $0;
 $73 = (_png_malloc_warn($72,16)|0);
 $text_ptr = $73;
 $74 = $text_ptr;
 $75 = ($74|0)==(0|0);
 if ($75) {
  $76 = $0;
  _png_warning($76,62520);
  $77 = $0;
  $78 = $chunkdata;
  _png_free($77,$78);
  STACKTOP = sp;return;
 }
 $79 = $comp_type;
 $80 = $text_ptr;
 HEAP32[$80>>2] = $79;
 $81 = $chunkdata;
 $82 = $text_ptr;
 $83 = (($82) + 4|0);
 HEAP32[$83>>2] = $81;
 $84 = $chunkdata;
 $85 = $prefix_len;
 $86 = (($84) + ($85)|0);
 $87 = $text_ptr;
 $88 = (($87) + 8|0);
 HEAP32[$88>>2] = $86;
 $89 = HEAP32[$data_len>>2]|0;
 $90 = $text_ptr;
 $91 = (($90) + 12|0);
 HEAP32[$91>>2] = $89;
 $92 = $0;
 $93 = $1;
 $94 = $text_ptr;
 $95 = (_png_set_text_2($92,$93,$94,1)|0);
 $ret = $95;
 $96 = $0;
 $97 = $text_ptr;
 _png_free($96,$97);
 $98 = $0;
 $99 = $chunkdata;
 _png_free($98,$99);
 $100 = $ret;
 $101 = ($100|0)!=(0);
 if (!($101)) {
  STACKTOP = sp;return;
 }
 $102 = $0;
 _png_error($102,62568);
 STACKTOP = sp;return;
}
function _png_handle_unknown($png_ptr,$info_ptr,$length) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ret = 0, $skip = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $length;
 $skip = 0;
 $3 = $0;
 $4 = (($3) + 196|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 4;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = $0;
  $9 = (($8) + 376|0);
  $10 = (_memcmp($9,57136,4)|0);
  $11 = ($10|0)!=(0);
  if ($11) {
   $12 = $0;
   $13 = (($12) + 196|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $14 | 8;
   HEAP32[$13>>2] = $15;
  }
 }
 $16 = $0;
 $17 = $0;
 $18 = (($17) + 376|0);
 _png_check_chunk_name($16,$18);
 $19 = $0;
 $20 = (($19) + 376|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $22 & 32;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $25 = $0;
  $26 = $0;
  $27 = (($26) + 376|0);
  $28 = (_png_handle_as_unknown($25,$27)|0);
  $29 = ($28|0)!=(3);
  if ($29) {
   $30 = $0;
   $31 = (($30) + 632|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ($32|0)==(0|0);
   if ($33) {
    $34 = $0;
    _png_chunk_error($34,62616);
   }
  }
 }
 $35 = $0;
 $36 = (($35) + 200|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 & 32768;
 $39 = ($38|0)!=(0);
 if (!($39)) {
  $40 = $0;
  $41 = (($40) + 632|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)!=(0|0);
  if (!($43)) {
   $107 = $2;
   $skip = $107;
   $108 = $0;
   $109 = $skip;
   (_png_crc_finish($108,$109)|0);
   STACKTOP = sp;return;
  }
 }
 $44 = $0;
 $45 = (($44) + 712|0);
 $46 = $0;
 $47 = (($46) + 376|0);
 ;HEAP8[$45+0>>0]=HEAP8[$47+0>>0]|0;HEAP8[$45+1>>0]=HEAP8[$47+1>>0]|0;HEAP8[$45+2>>0]=HEAP8[$47+2>>0]|0;HEAP8[$45+3>>0]=HEAP8[$47+3>>0]|0;HEAP8[$45+4>>0]=HEAP8[$47+4>>0]|0;
 $48 = $0;
 $49 = (($48) + 712|0);
 $50 = (($49) + 4|0);
 HEAP8[$50>>0] = 0;
 $51 = $0;
 $52 = $2;
 $53 = (_png_malloc($51,$52)|0);
 $54 = $0;
 $55 = (($54) + 712|0);
 $56 = (($55) + 8|0);
 HEAP32[$56>>2] = $53;
 $57 = $2;
 $58 = $0;
 $59 = (($58) + 712|0);
 $60 = (($59) + 12|0);
 HEAP32[$60>>2] = $57;
 $61 = $0;
 $62 = $0;
 $63 = (($62) + 712|0);
 $64 = (($63) + 8|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = $2;
 _png_crc_read($61,$65,$66);
 $67 = $0;
 $68 = (($67) + 632|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = ($69|0)!=(0|0);
 if ($70) {
  $71 = $0;
  $72 = (($71) + 632|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $0;
  $75 = $0;
  $76 = (($75) + 712|0);
  $77 = (FUNCTION_TABLE_iii[$73 & 255]($74,$76)|0);
  $ret = $77;
  $78 = $ret;
  $79 = ($78|0)<(0);
  if ($79) {
   $80 = $0;
   _png_chunk_error($80,62640);
  }
  $81 = $ret;
  $82 = ($81|0)==(0);
  if ($82) {
   $83 = $0;
   $84 = (($83) + 376|0);
   $85 = HEAP8[$84>>0]|0;
   $86 = $85&255;
   $87 = $86 & 32;
   $88 = ($87|0)!=(0);
   if (!($88)) {
    $89 = $0;
    $90 = $0;
    $91 = (($90) + 376|0);
    $92 = (_png_handle_as_unknown($89,$91)|0);
    $93 = ($92|0)!=(3);
    if ($93) {
     $94 = $0;
     _png_chunk_error($94,62616);
    }
   }
   $95 = $0;
   $96 = $1;
   $97 = $0;
   $98 = (($97) + 712|0);
   _png_set_unknown_chunks($95,$96,$98,1);
  }
 }
 $99 = $0;
 $100 = $0;
 $101 = (($100) + 712|0);
 $102 = (($101) + 8|0);
 $103 = HEAP32[$102>>2]|0;
 _png_free($99,$103);
 $104 = $0;
 $105 = (($104) + 712|0);
 $106 = (($105) + 8|0);
 HEAP32[$106>>2] = 0;
 $108 = $0;
 $109 = $skip;
 (_png_crc_finish($108,$109)|0);
 STACKTOP = sp;return;
}
function _png_check_chunk_name($png_ptr,$chunk_name) {
 $png_ptr = $png_ptr|0;
 $chunk_name = $chunk_name|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $chunk_name;
 $2 = $1;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)<(65);
 do {
  if (!($5)) {
   $6 = $1;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7&255;
   $9 = ($8|0)>(122);
   if (!($9)) {
    $10 = $1;
    $11 = HEAP8[$10>>0]|0;
    $12 = $11&255;
    $13 = ($12|0)>(90);
    if ($13) {
     $14 = $1;
     $15 = HEAP8[$14>>0]|0;
     $16 = $15&255;
     $17 = ($16|0)<(97);
     if ($17) {
      break;
     }
    }
    $18 = $1;
    $19 = (($18) + 1|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    $22 = ($21|0)<(65);
    if (!($22)) {
     $23 = $1;
     $24 = (($23) + 1|0);
     $25 = HEAP8[$24>>0]|0;
     $26 = $25&255;
     $27 = ($26|0)>(122);
     if (!($27)) {
      $28 = $1;
      $29 = (($28) + 1|0);
      $30 = HEAP8[$29>>0]|0;
      $31 = $30&255;
      $32 = ($31|0)>(90);
      if ($32) {
       $33 = $1;
       $34 = (($33) + 1|0);
       $35 = HEAP8[$34>>0]|0;
       $36 = $35&255;
       $37 = ($36|0)<(97);
       if ($37) {
        break;
       }
      }
      $38 = $1;
      $39 = (($38) + 2|0);
      $40 = HEAP8[$39>>0]|0;
      $41 = $40&255;
      $42 = ($41|0)<(65);
      if (!($42)) {
       $43 = $1;
       $44 = (($43) + 2|0);
       $45 = HEAP8[$44>>0]|0;
       $46 = $45&255;
       $47 = ($46|0)>(122);
       if (!($47)) {
        $48 = $1;
        $49 = (($48) + 2|0);
        $50 = HEAP8[$49>>0]|0;
        $51 = $50&255;
        $52 = ($51|0)>(90);
        if ($52) {
         $53 = $1;
         $54 = (($53) + 2|0);
         $55 = HEAP8[$54>>0]|0;
         $56 = $55&255;
         $57 = ($56|0)<(97);
         if ($57) {
          break;
         }
        }
        $58 = $1;
        $59 = (($58) + 3|0);
        $60 = HEAP8[$59>>0]|0;
        $61 = $60&255;
        $62 = ($61|0)<(65);
        if (!($62)) {
         $63 = $1;
         $64 = (($63) + 3|0);
         $65 = HEAP8[$64>>0]|0;
         $66 = $65&255;
         $67 = ($66|0)>(122);
         if (!($67)) {
          $68 = $1;
          $69 = (($68) + 3|0);
          $70 = HEAP8[$69>>0]|0;
          $71 = $70&255;
          $72 = ($71|0)>(90);
          if (!($72)) {
           STACKTOP = sp;return;
          }
          $73 = $1;
          $74 = (($73) + 3|0);
          $75 = HEAP8[$74>>0]|0;
          $76 = $75&255;
          $77 = ($76|0)<(97);
          if (!($77)) {
           STACKTOP = sp;return;
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $78 = $0;
 _png_chunk_error($78,62664);
 STACKTOP = sp;return;
}
function _png_combine_row($png_ptr,$row,$mask) {
 $png_ptr = $png_ptr|0;
 $row = $row|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp12 = 0;
 var $dp2 = 0, $dp22 = 0, $i = 0, $i18 = 0, $i23 = 0, $i8 = 0, $m = 0, $m16 = 0, $m25 = 0, $m6 = 0, $pixel_bytes = 0, $row_width = 0, $row_width19 = 0, $row_width24 = 0, $row_width9 = 0, $s_end = 0, $s_end14 = 0, $s_end4 = 0, $s_inc = 0, $s_inc15 = 0;
 var $s_inc5 = 0, $s_start = 0, $s_start13 = 0, $s_start3 = 0, $shift = 0, $shift17 = 0, $shift7 = 0, $sp = 0, $sp1 = 0, $sp11 = 0, $sp21 = 0, $value = 0, $value10 = 0, $value20 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = $png_ptr;
 $1 = $row;
 $2 = $mask;
 $3 = $2;
 $4 = ($3|0)==(255);
 if ($4) {
  $5 = $1;
  $6 = $0;
  $7 = (($6) + 328|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 1|0);
  $10 = $0;
  $11 = (($10) + 348|0);
  $12 = (($11) + 11|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)>=(8);
  if ($15) {
   $16 = $0;
   $17 = (($16) + 292|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = $0;
   $20 = (($19) + 348|0);
   $21 = (($20) + 11|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 >>> 3;
   $25 = Math_imul($18, $24)|0;
   $37 = $25;
  } else {
   $26 = $0;
   $27 = (($26) + 292|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = $0;
   $30 = (($29) + 348|0);
   $31 = (($30) + 11|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   $34 = Math_imul($28, $33)|0;
   $35 = (($34) + 7)|0;
   $36 = $35 >>> 3;
   $37 = $36;
  }
  _memcpy(($5|0),($9|0),($37|0))|0;
  STACKTOP = sp;return;
 }
 $38 = $0;
 $39 = (($38) + 348|0);
 $40 = (($39) + 11|0);
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 if ((($42|0) == 1)) {
  $43 = $0;
  $44 = (($43) + 328|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1|0);
  $sp = $46;
  $47 = $1;
  $dp = $47;
  $m = 128;
  $48 = $0;
  $49 = (($48) + 292|0);
  $50 = HEAP32[$49>>2]|0;
  $row_width = $50;
  $51 = $0;
  $52 = (($51) + 204|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $53 & 65536;
  $55 = ($54|0)!=(0);
  if ($55) {
   $s_start = 0;
   $s_end = 7;
   $s_inc = 1;
  } else {
   $s_start = 7;
   $s_end = 0;
   $s_inc = -1;
  }
  $56 = $s_start;
  $shift = $56;
  $i = 0;
  while(1) {
   $57 = $i;
   $58 = $row_width;
   $59 = ($57>>>0)<($58>>>0);
   if (!($59)) {
    break;
   }
   $60 = $m;
   $61 = $2;
   $62 = $60 & $61;
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = $sp;
    $65 = HEAP8[$64>>0]|0;
    $66 = $65&255;
    $67 = $shift;
    $68 = $66 >> $67;
    $69 = $68 & 1;
    $value = $69;
    $70 = $shift;
    $71 = (7 - ($70))|0;
    $72 = 32639 >> $71;
    $73 = $72 & 255;
    $74 = $73&255;
    $75 = $74&255;
    $76 = $dp;
    $77 = HEAP8[$76>>0]|0;
    $78 = $77&255;
    $79 = $78 & $75;
    $80 = $79&255;
    HEAP8[$76>>0] = $80;
    $81 = $value;
    $82 = $shift;
    $83 = $81 << $82;
    $84 = $83&255;
    $85 = $84&255;
    $86 = $dp;
    $87 = HEAP8[$86>>0]|0;
    $88 = $87&255;
    $89 = $88 | $85;
    $90 = $89&255;
    HEAP8[$86>>0] = $90;
   }
   $91 = $shift;
   $92 = $s_end;
   $93 = ($91|0)==($92|0);
   if ($93) {
    $94 = $s_start;
    $shift = $94;
    $95 = $sp;
    $96 = (($95) + 1|0);
    $sp = $96;
    $97 = $dp;
    $98 = (($97) + 1|0);
    $dp = $98;
   } else {
    $99 = $s_inc;
    $100 = $shift;
    $101 = (($100) + ($99))|0;
    $shift = $101;
   }
   $102 = $m;
   $103 = ($102|0)==(1);
   if ($103) {
    $m = 128;
   } else {
    $104 = $m;
    $105 = $104 >> 1;
    $m = $105;
   }
   $106 = $i;
   $107 = (($106) + 1)|0;
   $i = $107;
  }
 } else if ((($42|0) == 2)) {
  $108 = $0;
  $109 = (($108) + 328|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = (($110) + 1|0);
  $sp1 = $111;
  $112 = $1;
  $dp2 = $112;
  $m6 = 128;
  $113 = $0;
  $114 = (($113) + 292|0);
  $115 = HEAP32[$114>>2]|0;
  $row_width9 = $115;
  $116 = $0;
  $117 = (($116) + 204|0);
  $118 = HEAP32[$117>>2]|0;
  $119 = $118 & 65536;
  $120 = ($119|0)!=(0);
  if ($120) {
   $s_start3 = 0;
   $s_end4 = 6;
   $s_inc5 = 2;
  } else {
   $s_start3 = 6;
   $s_end4 = 0;
   $s_inc5 = -2;
  }
  $121 = $s_start3;
  $shift7 = $121;
  $i8 = 0;
  while(1) {
   $122 = $i8;
   $123 = $row_width9;
   $124 = ($122>>>0)<($123>>>0);
   if (!($124)) {
    break;
   }
   $125 = $m6;
   $126 = $2;
   $127 = $125 & $126;
   $128 = ($127|0)!=(0);
   if ($128) {
    $129 = $sp1;
    $130 = HEAP8[$129>>0]|0;
    $131 = $130&255;
    $132 = $shift7;
    $133 = $131 >> $132;
    $134 = $133 & 3;
    $value10 = $134;
    $135 = $shift7;
    $136 = (6 - ($135))|0;
    $137 = 16191 >> $136;
    $138 = $137 & 255;
    $139 = $138&255;
    $140 = $139&255;
    $141 = $dp2;
    $142 = HEAP8[$141>>0]|0;
    $143 = $142&255;
    $144 = $143 & $140;
    $145 = $144&255;
    HEAP8[$141>>0] = $145;
    $146 = $value10;
    $147 = $shift7;
    $148 = $146 << $147;
    $149 = $148&255;
    $150 = $149&255;
    $151 = $dp2;
    $152 = HEAP8[$151>>0]|0;
    $153 = $152&255;
    $154 = $153 | $150;
    $155 = $154&255;
    HEAP8[$151>>0] = $155;
   }
   $156 = $shift7;
   $157 = $s_end4;
   $158 = ($156|0)==($157|0);
   if ($158) {
    $159 = $s_start3;
    $shift7 = $159;
    $160 = $sp1;
    $161 = (($160) + 1|0);
    $sp1 = $161;
    $162 = $dp2;
    $163 = (($162) + 1|0);
    $dp2 = $163;
   } else {
    $164 = $s_inc5;
    $165 = $shift7;
    $166 = (($165) + ($164))|0;
    $shift7 = $166;
   }
   $167 = $m6;
   $168 = ($167|0)==(1);
   if ($168) {
    $m6 = 128;
   } else {
    $169 = $m6;
    $170 = $169 >> 1;
    $m6 = $170;
   }
   $171 = $i8;
   $172 = (($171) + 1)|0;
   $i8 = $172;
  }
 } else if ((($42|0) == 4)) {
  $173 = $0;
  $174 = (($173) + 328|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = (($175) + 1|0);
  $sp11 = $176;
  $177 = $1;
  $dp12 = $177;
  $m16 = 128;
  $178 = $0;
  $179 = (($178) + 292|0);
  $180 = HEAP32[$179>>2]|0;
  $row_width19 = $180;
  $181 = $0;
  $182 = (($181) + 204|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = $183 & 65536;
  $185 = ($184|0)!=(0);
  if ($185) {
   $s_start13 = 0;
   $s_end14 = 4;
   $s_inc15 = 4;
  } else {
   $s_start13 = 4;
   $s_end14 = 0;
   $s_inc15 = -4;
  }
  $186 = $s_start13;
  $shift17 = $186;
  $i18 = 0;
  while(1) {
   $187 = $i18;
   $188 = $row_width19;
   $189 = ($187>>>0)<($188>>>0);
   if (!($189)) {
    break;
   }
   $190 = $m16;
   $191 = $2;
   $192 = $190 & $191;
   $193 = ($192|0)!=(0);
   if ($193) {
    $194 = $sp11;
    $195 = HEAP8[$194>>0]|0;
    $196 = $195&255;
    $197 = $shift17;
    $198 = $196 >> $197;
    $199 = $198 & 15;
    $value20 = $199;
    $200 = $shift17;
    $201 = (4 - ($200))|0;
    $202 = 3855 >> $201;
    $203 = $202 & 255;
    $204 = $203&255;
    $205 = $204&255;
    $206 = $dp12;
    $207 = HEAP8[$206>>0]|0;
    $208 = $207&255;
    $209 = $208 & $205;
    $210 = $209&255;
    HEAP8[$206>>0] = $210;
    $211 = $value20;
    $212 = $shift17;
    $213 = $211 << $212;
    $214 = $213&255;
    $215 = $214&255;
    $216 = $dp12;
    $217 = HEAP8[$216>>0]|0;
    $218 = $217&255;
    $219 = $218 | $215;
    $220 = $219&255;
    HEAP8[$216>>0] = $220;
   }
   $221 = $shift17;
   $222 = $s_end14;
   $223 = ($221|0)==($222|0);
   if ($223) {
    $224 = $s_start13;
    $shift17 = $224;
    $225 = $sp11;
    $226 = (($225) + 1|0);
    $sp11 = $226;
    $227 = $dp12;
    $228 = (($227) + 1|0);
    $dp12 = $228;
   } else {
    $229 = $s_inc15;
    $230 = $shift17;
    $231 = (($230) + ($229))|0;
    $shift17 = $231;
   }
   $232 = $m16;
   $233 = ($232|0)==(1);
   if ($233) {
    $m16 = 128;
   } else {
    $234 = $m16;
    $235 = $234 >> 1;
    $m16 = $235;
   }
   $236 = $i18;
   $237 = (($236) + 1)|0;
   $i18 = $237;
  }
 } else {
  $238 = $0;
  $239 = (($238) + 328|0);
  $240 = HEAP32[$239>>2]|0;
  $241 = (($240) + 1|0);
  $sp21 = $241;
  $242 = $1;
  $dp22 = $242;
  $243 = $0;
  $244 = (($243) + 348|0);
  $245 = (($244) + 11|0);
  $246 = HEAP8[$245>>0]|0;
  $247 = $246&255;
  $248 = $247 >> 3;
  $pixel_bytes = $248;
  $249 = $0;
  $250 = (($249) + 292|0);
  $251 = HEAP32[$250>>2]|0;
  $row_width24 = $251;
  $m25 = -128;
  $i23 = 0;
  while(1) {
   $252 = $i23;
   $253 = $row_width24;
   $254 = ($252>>>0)<($253>>>0);
   if (!($254)) {
    break;
   }
   $255 = $m25;
   $256 = $255&255;
   $257 = $2;
   $258 = $256 & $257;
   $259 = ($258|0)!=(0);
   if ($259) {
    $260 = $dp22;
    $261 = $sp21;
    $262 = $pixel_bytes;
    _memcpy(($260|0),($261|0),($262|0))|0;
   }
   $263 = $pixel_bytes;
   $264 = $sp21;
   $265 = (($264) + ($263)|0);
   $sp21 = $265;
   $266 = $pixel_bytes;
   $267 = $dp22;
   $268 = (($267) + ($266)|0);
   $dp22 = $268;
   $269 = $m25;
   $270 = $269&255;
   $271 = ($270|0)==(1);
   if ($271) {
    $m25 = -128;
   } else {
    $272 = $m25;
    $273 = $272&255;
    $274 = $273 >> 1;
    $275 = $274&255;
    $m25 = $275;
   }
   $276 = $i23;
   $277 = (($276) + 1)|0;
   $i23 = $277;
  }
 }
 STACKTOP = sp;return;
}
function _png_do_read_interlace($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $dp13 = 0;
 var $dp2 = 0, $dp24 = 0, $dshift = 0, $dshift15 = 0, $dshift4 = 0, $final_width = 0, $i = 0, $i19 = 0, $i26 = 0, $i9 = 0, $j = 0, $j11 = 0, $j22 = 0, $j28 = 0, $jstop = 0, $jstop20 = 0, $jstop25 = 0, $jstop8 = 0, $pass = 0, $pixel_bytes = 0;
 var $row = 0, $row_info = 0, $s_end = 0, $s_end17 = 0, $s_end6 = 0, $s_inc = 0, $s_inc18 = 0, $s_inc7 = 0, $s_start = 0, $s_start16 = 0, $s_start5 = 0, $sp = 0, $sp1 = 0, $sp12 = 0, $sp23 = 0, $sshift = 0, $sshift14 = 0, $sshift3 = 0, $transformations = 0, $v = 0;
 var $v10 = 0, $v21 = 0, $v27 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $v27 = sp + 168|0;
 $0 = $png_ptr;
 $1 = $0;
 $2 = (($1) + 348|0);
 $row_info = $2;
 $3 = $0;
 $4 = (($3) + 328|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1|0);
 $row = $6;
 $7 = $0;
 $8 = (($7) + 384|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $pass = $10;
 $11 = $0;
 $12 = (($11) + 204|0);
 $13 = HEAP32[$12>>2]|0;
 $transformations = $13;
 $14 = $row;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  STACKTOP = sp;return;
 }
 $16 = $row_info;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = $row_info;
 $19 = HEAP32[$18>>2]|0;
 $20 = $pass;
 $21 = (57320 + ($20<<2)|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = Math_imul($19, $22)|0;
 $final_width = $23;
 $24 = $row_info;
 $25 = (($24) + 11|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 if ((($27|0) == 1)) {
  $28 = $row;
  $29 = $row_info;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) - 1)|0;
  $32 = $31 >>> 3;
  $33 = (($28) + ($32)|0);
  $sp = $33;
  $34 = $row;
  $35 = $final_width;
  $36 = (($35) - 1)|0;
  $37 = $36 >>> 3;
  $38 = (($34) + ($37)|0);
  $dp = $38;
  $39 = $pass;
  $40 = (57320 + ($39<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $jstop = $41;
  $42 = $transformations;
  $43 = $42 & 65536;
  $44 = ($43|0)!=(0);
  if ($44) {
   $45 = $row_info;
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + 7)|0;
   $48 = $47 & 7;
   $sshift = $48;
   $49 = $final_width;
   $50 = (($49) + 7)|0;
   $51 = $50 & 7;
   $dshift = $51;
   $s_start = 7;
   $s_end = 0;
   $s_inc = -1;
  } else {
   $52 = $row_info;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + 7)|0;
   $55 = $54 & 7;
   $56 = (7 - ($55))|0;
   $sshift = $56;
   $57 = $final_width;
   $58 = (($57) + 7)|0;
   $59 = $58 & 7;
   $60 = (7 - ($59))|0;
   $dshift = $60;
   $s_start = 0;
   $s_end = 7;
   $s_inc = 1;
  }
  $i = 0;
  while(1) {
   $61 = $i;
   $62 = $row_info;
   $63 = HEAP32[$62>>2]|0;
   $64 = ($61>>>0)<($63>>>0);
   if (!($64)) {
    break;
   }
   $65 = $sp;
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   $68 = $sshift;
   $69 = $67 >> $68;
   $70 = $69 & 1;
   $71 = $70&255;
   $v = $71;
   $j = 0;
   while(1) {
    $72 = $j;
    $73 = $jstop;
    $74 = ($72|0)<($73|0);
    if (!($74)) {
     break;
    }
    $75 = $dshift;
    $76 = (7 - ($75))|0;
    $77 = 32639 >> $76;
    $78 = $77 & 255;
    $79 = $78&255;
    $80 = $79&255;
    $81 = $dp;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $83 & $80;
    $85 = $84&255;
    HEAP8[$81>>0] = $85;
    $86 = $v;
    $87 = $86&255;
    $88 = $dshift;
    $89 = $87 << $88;
    $90 = $89&255;
    $91 = $90&255;
    $92 = $dp;
    $93 = HEAP8[$92>>0]|0;
    $94 = $93&255;
    $95 = $94 | $91;
    $96 = $95&255;
    HEAP8[$92>>0] = $96;
    $97 = $dshift;
    $98 = $s_end;
    $99 = ($97|0)==($98|0);
    if ($99) {
     $100 = $s_start;
     $dshift = $100;
     $101 = $dp;
     $102 = (($101) + -1|0);
     $dp = $102;
    } else {
     $103 = $s_inc;
     $104 = $dshift;
     $105 = (($104) + ($103))|0;
     $dshift = $105;
    }
    $106 = $j;
    $107 = (($106) + 1)|0;
    $j = $107;
   }
   $108 = $sshift;
   $109 = $s_end;
   $110 = ($108|0)==($109|0);
   if ($110) {
    $111 = $s_start;
    $sshift = $111;
    $112 = $sp;
    $113 = (($112) + -1|0);
    $sp = $113;
   } else {
    $114 = $s_inc;
    $115 = $sshift;
    $116 = (($115) + ($114))|0;
    $sshift = $116;
   }
   $117 = $i;
   $118 = (($117) + 1)|0;
   $i = $118;
  }
 } else if ((($27|0) == 4)) {
  $214 = $row;
  $215 = $row_info;
  $216 = HEAP32[$215>>2]|0;
  $217 = (($216) - 1)|0;
  $218 = $217 >>> 1;
  $219 = (($214) + ($218)|0);
  $sp12 = $219;
  $220 = $row;
  $221 = $final_width;
  $222 = (($221) - 1)|0;
  $223 = $222 >>> 1;
  $224 = (($220) + ($223)|0);
  $dp13 = $224;
  $225 = $pass;
  $226 = (57320 + ($225<<2)|0);
  $227 = HEAP32[$226>>2]|0;
  $jstop20 = $227;
  $228 = $transformations;
  $229 = $228 & 65536;
  $230 = ($229|0)!=(0);
  if ($230) {
   $231 = $row_info;
   $232 = HEAP32[$231>>2]|0;
   $233 = (($232) + 1)|0;
   $234 = $233 & 1;
   $235 = $234 << 2;
   $sshift14 = $235;
   $236 = $final_width;
   $237 = (($236) + 1)|0;
   $238 = $237 & 1;
   $239 = $238 << 2;
   $dshift15 = $239;
   $s_start16 = 4;
   $s_end17 = 0;
   $s_inc18 = -4;
  } else {
   $240 = $row_info;
   $241 = HEAP32[$240>>2]|0;
   $242 = (($241) + 1)|0;
   $243 = $242 & 1;
   $244 = (1 - ($243))|0;
   $245 = $244 << 2;
   $sshift14 = $245;
   $246 = $final_width;
   $247 = (($246) + 1)|0;
   $248 = $247 & 1;
   $249 = (1 - ($248))|0;
   $250 = $249 << 2;
   $dshift15 = $250;
   $s_start16 = 0;
   $s_end17 = 4;
   $s_inc18 = 4;
  }
  $i19 = 0;
  while(1) {
   $251 = $i19;
   $252 = $row_info;
   $253 = HEAP32[$252>>2]|0;
   $254 = ($251>>>0)<($253>>>0);
   if (!($254)) {
    break;
   }
   $255 = $sp12;
   $256 = HEAP8[$255>>0]|0;
   $257 = $256&255;
   $258 = $sshift14;
   $259 = $257 >> $258;
   $260 = $259 & 15;
   $261 = $260&255;
   $v21 = $261;
   $j22 = 0;
   while(1) {
    $262 = $j22;
    $263 = $jstop20;
    $264 = ($262|0)<($263|0);
    if (!($264)) {
     break;
    }
    $265 = $dshift15;
    $266 = (4 - ($265))|0;
    $267 = 3855 >> $266;
    $268 = $267 & 255;
    $269 = $268&255;
    $270 = $269&255;
    $271 = $dp13;
    $272 = HEAP8[$271>>0]|0;
    $273 = $272&255;
    $274 = $273 & $270;
    $275 = $274&255;
    HEAP8[$271>>0] = $275;
    $276 = $v21;
    $277 = $276&255;
    $278 = $dshift15;
    $279 = $277 << $278;
    $280 = $279&255;
    $281 = $280&255;
    $282 = $dp13;
    $283 = HEAP8[$282>>0]|0;
    $284 = $283&255;
    $285 = $284 | $281;
    $286 = $285&255;
    HEAP8[$282>>0] = $286;
    $287 = $dshift15;
    $288 = $s_end17;
    $289 = ($287|0)==($288|0);
    if ($289) {
     $290 = $s_start16;
     $dshift15 = $290;
     $291 = $dp13;
     $292 = (($291) + -1|0);
     $dp13 = $292;
    } else {
     $293 = $s_inc18;
     $294 = $dshift15;
     $295 = (($294) + ($293))|0;
     $dshift15 = $295;
    }
    $296 = $j22;
    $297 = (($296) + 1)|0;
    $j22 = $297;
   }
   $298 = $sshift14;
   $299 = $s_end17;
   $300 = ($298|0)==($299|0);
   if ($300) {
    $301 = $s_start16;
    $sshift14 = $301;
    $302 = $sp12;
    $303 = (($302) + -1|0);
    $sp12 = $303;
   } else {
    $304 = $s_inc18;
    $305 = $sshift14;
    $306 = (($305) + ($304))|0;
    $sshift14 = $306;
   }
   $307 = $i19;
   $308 = (($307) + 1)|0;
   $i19 = $308;
  }
 } else if ((($27|0) == 2)) {
  $119 = $row;
  $120 = $row_info;
  $121 = HEAP32[$120>>2]|0;
  $122 = (($121) - 1)|0;
  $123 = $122 >>> 2;
  $124 = (($119) + ($123)|0);
  $sp1 = $124;
  $125 = $row;
  $126 = $final_width;
  $127 = (($126) - 1)|0;
  $128 = $127 >>> 2;
  $129 = (($125) + ($128)|0);
  $dp2 = $129;
  $130 = $pass;
  $131 = (57320 + ($130<<2)|0);
  $132 = HEAP32[$131>>2]|0;
  $jstop8 = $132;
  $133 = $transformations;
  $134 = $133 & 65536;
  $135 = ($134|0)!=(0);
  if ($135) {
   $136 = $row_info;
   $137 = HEAP32[$136>>2]|0;
   $138 = (($137) + 3)|0;
   $139 = $138 & 3;
   $140 = $139 << 1;
   $sshift3 = $140;
   $141 = $final_width;
   $142 = (($141) + 3)|0;
   $143 = $142 & 3;
   $144 = $143 << 1;
   $dshift4 = $144;
   $s_start5 = 6;
   $s_end6 = 0;
   $s_inc7 = -2;
  } else {
   $145 = $row_info;
   $146 = HEAP32[$145>>2]|0;
   $147 = (($146) + 3)|0;
   $148 = $147 & 3;
   $149 = (3 - ($148))|0;
   $150 = $149 << 1;
   $sshift3 = $150;
   $151 = $final_width;
   $152 = (($151) + 3)|0;
   $153 = $152 & 3;
   $154 = (3 - ($153))|0;
   $155 = $154 << 1;
   $dshift4 = $155;
   $s_start5 = 0;
   $s_end6 = 6;
   $s_inc7 = 2;
  }
  $i9 = 0;
  while(1) {
   $156 = $i9;
   $157 = $row_info;
   $158 = HEAP32[$157>>2]|0;
   $159 = ($156>>>0)<($158>>>0);
   if (!($159)) {
    break;
   }
   $160 = $sp1;
   $161 = HEAP8[$160>>0]|0;
   $162 = $161&255;
   $163 = $sshift3;
   $164 = $162 >> $163;
   $165 = $164 & 3;
   $166 = $165&255;
   $v10 = $166;
   $j11 = 0;
   while(1) {
    $167 = $j11;
    $168 = $jstop8;
    $169 = ($167|0)<($168|0);
    if (!($169)) {
     break;
    }
    $170 = $dshift4;
    $171 = (6 - ($170))|0;
    $172 = 16191 >> $171;
    $173 = $172 & 255;
    $174 = $173&255;
    $175 = $174&255;
    $176 = $dp2;
    $177 = HEAP8[$176>>0]|0;
    $178 = $177&255;
    $179 = $178 & $175;
    $180 = $179&255;
    HEAP8[$176>>0] = $180;
    $181 = $v10;
    $182 = $181&255;
    $183 = $dshift4;
    $184 = $182 << $183;
    $185 = $184&255;
    $186 = $185&255;
    $187 = $dp2;
    $188 = HEAP8[$187>>0]|0;
    $189 = $188&255;
    $190 = $189 | $186;
    $191 = $190&255;
    HEAP8[$187>>0] = $191;
    $192 = $dshift4;
    $193 = $s_end6;
    $194 = ($192|0)==($193|0);
    if ($194) {
     $195 = $s_start5;
     $dshift4 = $195;
     $196 = $dp2;
     $197 = (($196) + -1|0);
     $dp2 = $197;
    } else {
     $198 = $s_inc7;
     $199 = $dshift4;
     $200 = (($199) + ($198))|0;
     $dshift4 = $200;
    }
    $201 = $j11;
    $202 = (($201) + 1)|0;
    $j11 = $202;
   }
   $203 = $sshift3;
   $204 = $s_end6;
   $205 = ($203|0)==($204|0);
   if ($205) {
    $206 = $s_start5;
    $sshift3 = $206;
    $207 = $sp1;
    $208 = (($207) + -1|0);
    $sp1 = $208;
   } else {
    $209 = $s_inc7;
    $210 = $sshift3;
    $211 = (($210) + ($209))|0;
    $sshift3 = $211;
   }
   $212 = $i9;
   $213 = (($212) + 1)|0;
   $i9 = $213;
  }
 } else {
  $309 = $row_info;
  $310 = (($309) + 11|0);
  $311 = HEAP8[$310>>0]|0;
  $312 = $311&255;
  $313 = $312 >> 3;
  $pixel_bytes = $313;
  $314 = $row;
  $315 = $row_info;
  $316 = HEAP32[$315>>2]|0;
  $317 = (($316) - 1)|0;
  $318 = $pixel_bytes;
  $319 = Math_imul($317, $318)|0;
  $320 = (($314) + ($319)|0);
  $sp23 = $320;
  $321 = $row;
  $322 = $final_width;
  $323 = (($322) - 1)|0;
  $324 = $pixel_bytes;
  $325 = Math_imul($323, $324)|0;
  $326 = (($321) + ($325)|0);
  $dp24 = $326;
  $327 = $pass;
  $328 = (57320 + ($327<<2)|0);
  $329 = HEAP32[$328>>2]|0;
  $jstop25 = $329;
  $i26 = 0;
  while(1) {
   $330 = $i26;
   $331 = $row_info;
   $332 = HEAP32[$331>>2]|0;
   $333 = ($330>>>0)<($332>>>0);
   if (!($333)) {
    break;
   }
   $334 = $sp23;
   $335 = $pixel_bytes;
   _memcpy(($v27|0),($334|0),($335|0))|0;
   $j28 = 0;
   while(1) {
    $336 = $j28;
    $337 = $jstop25;
    $338 = ($336|0)<($337|0);
    if (!($338)) {
     break;
    }
    $339 = $dp24;
    $340 = $pixel_bytes;
    _memcpy(($339|0),($v27|0),($340|0))|0;
    $341 = $pixel_bytes;
    $342 = $dp24;
    $343 = (0 - ($341))|0;
    $344 = (($342) + ($343)|0);
    $dp24 = $344;
    $345 = $j28;
    $346 = (($345) + 1)|0;
    $j28 = $346;
   }
   $347 = $pixel_bytes;
   $348 = $sp23;
   $349 = (0 - ($347))|0;
   $350 = (($348) + ($349)|0);
   $sp23 = $350;
   $351 = $i26;
   $352 = (($351) + 1)|0;
   $i26 = $352;
  }
 }
 $353 = $final_width;
 $354 = $row_info;
 HEAP32[$354>>2] = $353;
 $355 = $row_info;
 $356 = (($355) + 11|0);
 $357 = HEAP8[$356>>0]|0;
 $358 = $357&255;
 $359 = ($358|0)>=(8);
 if ($359) {
  $360 = $final_width;
  $361 = $row_info;
  $362 = (($361) + 11|0);
  $363 = HEAP8[$362>>0]|0;
  $364 = $363&255;
  $365 = $364 >>> 3;
  $366 = Math_imul($360, $365)|0;
  $377 = $366;
 } else {
  $367 = $final_width;
  $368 = $row_info;
  $369 = (($368) + 11|0);
  $370 = HEAP8[$369>>0]|0;
  $371 = $370&255;
  $372 = Math_imul($367, $371)|0;
  $373 = (($372) + 7)|0;
  $374 = $373 >>> 3;
  $377 = $374;
 }
 $375 = $row_info;
 $376 = (($375) + 4|0);
 HEAP32[$376>>2] = $377;
 STACKTOP = sp;return;
}
function _png_read_filter_row($png_ptr,$row_info,$row,$prev_row,$filter) {
 $png_ptr = $png_ptr|0;
 $row_info = $row_info|0;
 $row = $row|0;
 $prev_row = $prev_row|0;
 $filter = $filter|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $bpp = 0, $bpp14 = 0, $bpp8 = 0, $c = 0, $cp = 0, $i = 0, $i1 = 0, $i10 = 0, $i4 = 0, $istop = 0, $istop15 = 0, $istop2 = 0, $istop9 = 0, $lp = 0;
 var $lp13 = 0, $lp7 = 0, $p = 0, $pa = 0, $pb = 0, $pc = 0, $pp = 0, $pp12 = 0, $pp6 = 0, $rp = 0, $rp11 = 0, $rp3 = 0, $rp5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = $png_ptr;
 $1 = $row_info;
 $2 = $row;
 $3 = $prev_row;
 $4 = $filter;
 $5 = $4;
 switch ($5|0) {
 case 0:  {
  STACKTOP = sp;return;
  break;
 }
 case 3:  {
  $61 = $2;
  $rp5 = $61;
  $62 = $3;
  $pp6 = $62;
  $63 = $2;
  $lp7 = $63;
  $64 = $1;
  $65 = (($64) + 11|0);
  $66 = HEAP8[$65>>0]|0;
  $67 = $66&255;
  $68 = (($67) + 7)|0;
  $69 = $68 >> 3;
  $bpp8 = $69;
  $70 = $1;
  $71 = (($70) + 4|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = $bpp8;
  $74 = (($72) - ($73))|0;
  $istop9 = $74;
  $i4 = 0;
  while(1) {
   $75 = $i4;
   $76 = $bpp8;
   $77 = ($75>>>0)<($76>>>0);
   if (!($77)) {
    break;
   }
   $78 = $rp5;
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = $pp6;
   $82 = (($81) + 1|0);
   $pp6 = $82;
   $83 = HEAP8[$81>>0]|0;
   $84 = $83&255;
   $85 = (($84|0) / 2)&-1;
   $86 = (($80) + ($85))|0;
   $87 = $86 & 255;
   $88 = $87&255;
   $89 = $rp5;
   HEAP8[$89>>0] = $88;
   $90 = $rp5;
   $91 = (($90) + 1|0);
   $rp5 = $91;
   $92 = $i4;
   $93 = (($92) + 1)|0;
   $i4 = $93;
  }
  $i4 = 0;
  while(1) {
   $94 = $i4;
   $95 = $istop9;
   $96 = ($94>>>0)<($95>>>0);
   if (!($96)) {
    break;
   }
   $97 = $rp5;
   $98 = HEAP8[$97>>0]|0;
   $99 = $98&255;
   $100 = $pp6;
   $101 = (($100) + 1|0);
   $pp6 = $101;
   $102 = HEAP8[$100>>0]|0;
   $103 = $102&255;
   $104 = $lp7;
   $105 = (($104) + 1|0);
   $lp7 = $105;
   $106 = HEAP8[$104>>0]|0;
   $107 = $106&255;
   $108 = (($103) + ($107))|0;
   $109 = (($108|0) / 2)&-1;
   $110 = (($99) + ($109))|0;
   $111 = $110 & 255;
   $112 = $111&255;
   $113 = $rp5;
   HEAP8[$113>>0] = $112;
   $114 = $rp5;
   $115 = (($114) + 1|0);
   $rp5 = $115;
   $116 = $i4;
   $117 = (($116) + 1)|0;
   $i4 = $117;
  }
  STACKTOP = sp;return;
  break;
 }
 case 2:  {
  $38 = $1;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $istop2 = $40;
  $41 = $2;
  $rp3 = $41;
  $42 = $3;
  $pp = $42;
  $i1 = 0;
  while(1) {
   $43 = $i1;
   $44 = $istop2;
   $45 = ($43>>>0)<($44>>>0);
   if (!($45)) {
    break;
   }
   $46 = $rp3;
   $47 = HEAP8[$46>>0]|0;
   $48 = $47&255;
   $49 = $pp;
   $50 = (($49) + 1|0);
   $pp = $50;
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   $53 = (($48) + ($52))|0;
   $54 = $53 & 255;
   $55 = $54&255;
   $56 = $rp3;
   HEAP8[$56>>0] = $55;
   $57 = $rp3;
   $58 = (($57) + 1|0);
   $rp3 = $58;
   $59 = $i1;
   $60 = (($59) + 1)|0;
   $i1 = $60;
  }
  STACKTOP = sp;return;
  break;
 }
 case 4:  {
  $118 = $2;
  $rp11 = $118;
  $119 = $3;
  $pp12 = $119;
  $120 = $2;
  $lp13 = $120;
  $121 = $3;
  $cp = $121;
  $122 = $1;
  $123 = (($122) + 11|0);
  $124 = HEAP8[$123>>0]|0;
  $125 = $124&255;
  $126 = (($125) + 7)|0;
  $127 = $126 >> 3;
  $bpp14 = $127;
  $128 = $1;
  $129 = (($128) + 4|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = $bpp14;
  $132 = (($130) - ($131))|0;
  $istop15 = $132;
  $i10 = 0;
  while(1) {
   $133 = $i10;
   $134 = $bpp14;
   $135 = ($133>>>0)<($134>>>0);
   if (!($135)) {
    break;
   }
   $136 = $rp11;
   $137 = HEAP8[$136>>0]|0;
   $138 = $137&255;
   $139 = $pp12;
   $140 = (($139) + 1|0);
   $pp12 = $140;
   $141 = HEAP8[$139>>0]|0;
   $142 = $141&255;
   $143 = (($138) + ($142))|0;
   $144 = $143 & 255;
   $145 = $144&255;
   $146 = $rp11;
   HEAP8[$146>>0] = $145;
   $147 = $rp11;
   $148 = (($147) + 1|0);
   $rp11 = $148;
   $149 = $i10;
   $150 = (($149) + 1)|0;
   $i10 = $150;
  }
  $i10 = 0;
  while(1) {
   $151 = $i10;
   $152 = $istop15;
   $153 = ($151>>>0)<($152>>>0);
   if (!($153)) {
    break;
   }
   $154 = $lp13;
   $155 = (($154) + 1|0);
   $lp13 = $155;
   $156 = HEAP8[$154>>0]|0;
   $157 = $156&255;
   $a = $157;
   $158 = $pp12;
   $159 = (($158) + 1|0);
   $pp12 = $159;
   $160 = HEAP8[$158>>0]|0;
   $161 = $160&255;
   $b = $161;
   $162 = $cp;
   $163 = (($162) + 1|0);
   $cp = $163;
   $164 = HEAP8[$162>>0]|0;
   $165 = $164&255;
   $c = $165;
   $166 = $b;
   $167 = $c;
   $168 = (($166) - ($167))|0;
   $p = $168;
   $169 = $a;
   $170 = $c;
   $171 = (($169) - ($170))|0;
   $pc = $171;
   $172 = $p;
   $173 = ($172|0)<(0);
   if ($173) {
    $174 = $p;
    $175 = (0 - ($174))|0;
    $177 = $175;
   } else {
    $176 = $p;
    $177 = $176;
   }
   $pa = $177;
   $178 = $pc;
   $179 = ($178|0)<(0);
   if ($179) {
    $180 = $pc;
    $181 = (0 - ($180))|0;
    $183 = $181;
   } else {
    $182 = $pc;
    $183 = $182;
   }
   $pb = $183;
   $184 = $p;
   $185 = $pc;
   $186 = (($184) + ($185))|0;
   $187 = ($186|0)<(0);
   if ($187) {
    $188 = $p;
    $189 = $pc;
    $190 = (($188) + ($189))|0;
    $191 = (0 - ($190))|0;
    $195 = $191;
   } else {
    $192 = $p;
    $193 = $pc;
    $194 = (($192) + ($193))|0;
    $195 = $194;
   }
   $pc = $195;
   $196 = $pa;
   $197 = $pb;
   $198 = ($196|0)<=($197|0);
   if ($198) {
    $199 = $pa;
    $200 = $pc;
    $201 = ($199|0)<=($200|0);
    if ($201) {
     $202 = $a;
     $208 = $202;
    } else {
     label = 40;
    }
   } else {
    label = 40;
   }
   if ((label|0) == 40) {
    label = 0;
    $203 = $pb;
    $204 = $pc;
    $205 = ($203|0)<=($204|0);
    if ($205) {
     $206 = $b;
     $223 = $206;
    } else {
     $207 = $c;
     $223 = $207;
    }
    $208 = $223;
   }
   $p = $208;
   $209 = $rp11;
   $210 = HEAP8[$209>>0]|0;
   $211 = $210&255;
   $212 = $p;
   $213 = (($211) + ($212))|0;
   $214 = $213 & 255;
   $215 = $214&255;
   $216 = $rp11;
   HEAP8[$216>>0] = $215;
   $217 = $rp11;
   $218 = (($217) + 1|0);
   $rp11 = $218;
   $219 = $i10;
   $220 = (($219) + 1)|0;
   $i10 = $220;
  }
  STACKTOP = sp;return;
  break;
 }
 case 1:  {
  $6 = $1;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $istop = $8;
  $9 = $1;
  $10 = (($9) + 11|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = (($12) + 7)|0;
  $14 = $13 >> 3;
  $bpp = $14;
  $15 = $2;
  $16 = $bpp;
  $17 = (($15) + ($16)|0);
  $rp = $17;
  $18 = $2;
  $lp = $18;
  $19 = $bpp;
  $i = $19;
  while(1) {
   $20 = $i;
   $21 = $istop;
   $22 = ($20>>>0)<($21>>>0);
   if (!($22)) {
    break;
   }
   $23 = $rp;
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = $lp;
   $27 = (($26) + 1|0);
   $lp = $27;
   $28 = HEAP8[$26>>0]|0;
   $29 = $28&255;
   $30 = (($25) + ($29))|0;
   $31 = $30 & 255;
   $32 = $31&255;
   $33 = $rp;
   HEAP8[$33>>0] = $32;
   $34 = $rp;
   $35 = (($34) + 1|0);
   $rp = $35;
   $36 = $i;
   $37 = (($36) + 1)|0;
   $i = $37;
  }
  STACKTOP = sp;return;
  break;
 }
 default: {
  $221 = $0;
  _png_warning($221,62688);
  $222 = $2;
  HEAP8[$222>>0] = 0;
  STACKTOP = sp;return;
 }
 }
}
function _png_read_finish_row($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $chunk_length = 0, $extra = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $extra = sp + 12|0;
 $chunk_length = sp + 8|0;
 $0 = $png_ptr;
 $1 = $0;
 $2 = (($1) + 320|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 $5 = $0;
 $6 = (($5) + 320|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $0;
 $9 = (($8) + 300|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7>>>0)<($10>>>0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = $0;
 $13 = (($12) + 383|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = ($14<<24>>24)!=(0);
 do {
  if ($15) {
   $16 = $0;
   $17 = (($16) + 320|0);
   HEAP32[$17>>2] = 0;
   $18 = $0;
   $19 = $0;
   $20 = (($19) + 324|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $0;
   $23 = (($22) + 308|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + 1)|0;
   (_png_memset_check($18,$21,0,$25)|0);
   while(1) {
    $26 = $0;
    $27 = (($26) + 384|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = (($28) + 1)<<24>>24;
    HEAP8[$27>>0] = $29;
    $30 = $0;
    $31 = (($30) + 384|0);
    $32 = HEAP8[$31>>0]|0;
    $33 = $32&255;
    $34 = ($33|0)>=(7);
    if ($34) {
     label = 6;
     break;
    }
    $35 = $0;
    $36 = (($35) + 292|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = $0;
    $39 = (($38) + 384|0);
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    $42 = (57320 + ($41<<2)|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = (($37) + ($43))|0;
    $45 = (($44) - 1)|0;
    $46 = $0;
    $47 = (($46) + 384|0);
    $48 = HEAP8[$47>>0]|0;
    $49 = $48&255;
    $50 = (57288 + ($49<<2)|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($45) - ($51))|0;
    $53 = $0;
    $54 = (($53) + 384|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = (57320 + ($56<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = (($52>>>0) / ($58>>>0))&-1;
    $60 = $0;
    $61 = (($60) + 316|0);
    HEAP32[$61>>2] = $59;
    $62 = $0;
    $63 = (($62) + 389|0);
    $64 = HEAP8[$63>>0]|0;
    $65 = $64&255;
    $66 = ($65|0)>=(8);
    if ($66) {
     $67 = $0;
     $68 = (($67) + 316|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = $0;
     $71 = (($70) + 389|0);
     $72 = HEAP8[$71>>0]|0;
     $73 = $72&255;
     $74 = $73 >>> 3;
     $75 = Math_imul($69, $74)|0;
     $87 = $75;
    } else {
     $76 = $0;
     $77 = (($76) + 316|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = $0;
     $80 = (($79) + 389|0);
     $81 = HEAP8[$80>>0]|0;
     $82 = $81&255;
     $83 = Math_imul($78, $82)|0;
     $84 = (($83) + 7)|0;
     $85 = $84 >>> 3;
     $87 = $85;
    }
    $86 = (($87) + 1)|0;
    $88 = $0;
    $89 = (($88) + 312|0);
    HEAP32[$89>>2] = $86;
    $90 = $0;
    $91 = (($90) + 204|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = $92 & 2;
    $94 = ($93|0)!=(0);
    if ($94) {
     label = 14;
     break;
    }
    $95 = $0;
    $96 = (($95) + 296|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = $0;
    $99 = (($98) + 384|0);
    $100 = HEAP8[$99>>0]|0;
    $101 = $100&255;
    $102 = (57384 + ($101<<2)|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($97) + ($103))|0;
    $105 = (($104) - 1)|0;
    $106 = $0;
    $107 = (($106) + 384|0);
    $108 = HEAP8[$107>>0]|0;
    $109 = $108&255;
    $110 = (57352 + ($109<<2)|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = (($105) - ($111))|0;
    $113 = $0;
    $114 = (($113) + 384|0);
    $115 = HEAP8[$114>>0]|0;
    $116 = $115&255;
    $117 = (57384 + ($116<<2)|0);
    $118 = HEAP32[$117>>2]|0;
    $119 = (($112>>>0) / ($118>>>0))&-1;
    $120 = $0;
    $121 = (($120) + 300|0);
    HEAP32[$121>>2] = $119;
    $122 = $0;
    $123 = (($122) + 300|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = ($124|0)!=(0);
    if ($125) {
    } else {
    }
    $126 = $0;
    $127 = (($126) + 316|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = ($128|0)==(0);
    if (!($129)) {
     break;
    }
   }
   if ((label|0) == 6) {
   }
   else if ((label|0) == 14) {
   }
   $130 = $0;
   $131 = (($130) + 384|0);
   $132 = HEAP8[$131>>0]|0;
   $133 = $132&255;
   $134 = ($133|0)<(7);
   if (!($134)) {
    break;
   }
   STACKTOP = sp;return;
  }
 } while(0);
 $135 = $0;
 $136 = (($135) + 200|0);
 $137 = HEAP32[$136>>2]|0;
 $138 = $137 & 32;
 $139 = ($138|0)!=(0);
 if (!($139)) {
  $140 = $0;
  $141 = (($140) + 208|0);
  $142 = (($141) + 12|0);
  HEAP32[$142>>2] = $extra;
  $143 = $0;
  $144 = (($143) + 208|0);
  $145 = (($144) + 16|0);
  HEAP32[$145>>2] = 1;
  while(1) {
   $146 = $0;
   $147 = (($146) + 208|0);
   $148 = (($147) + 4|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = ($149|0)!=(0);
   if (!($150)) {
    while(1) {
     $151 = $0;
     $152 = (($151) + 360|0);
     $153 = HEAP32[$152>>2]|0;
     $154 = ($153|0)!=(0);
     $155 = $154 ^ 1;
     if (!($155)) {
      break;
     }
     $156 = $0;
     (_png_crc_finish($156,0)|0);
     $157 = $0;
     _png_read_data($157,$chunk_length,4);
     $158 = $0;
     $159 = (_png_get_uint_31($158,$chunk_length)|0);
     $160 = $0;
     $161 = (($160) + 360|0);
     HEAP32[$161>>2] = $159;
     $162 = $0;
     _png_reset_crc($162);
     $163 = $0;
     $164 = $0;
     $165 = (($164) + 376|0);
     _png_crc_read($163,$165,4);
     $166 = $0;
     $167 = (($166) + 376|0);
     $168 = (_memcmp($167,57136,4)|0);
     $169 = ($168|0)!=(0);
     if ($169) {
      $170 = $0;
      _png_error($170,62728);
     }
    }
    $171 = $0;
    $172 = (($171) + 268|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = $0;
    $175 = (($174) + 208|0);
    $176 = (($175) + 4|0);
    HEAP32[$176>>2] = $173;
    $177 = $0;
    $178 = (($177) + 264|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = $0;
    $181 = (($180) + 208|0);
    HEAP32[$181>>2] = $179;
    $182 = $0;
    $183 = (($182) + 268|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = $0;
    $186 = (($185) + 360|0);
    $187 = HEAP32[$186>>2]|0;
    $188 = ($184>>>0)>($187>>>0);
    if ($188) {
     $189 = $0;
     $190 = (($189) + 360|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = $0;
     $193 = (($192) + 208|0);
     $194 = (($193) + 4|0);
     HEAP32[$194>>2] = $191;
    }
    $195 = $0;
    $196 = $0;
    $197 = (($196) + 264|0);
    $198 = HEAP32[$197>>2]|0;
    $199 = $0;
    $200 = (($199) + 208|0);
    $201 = (($200) + 4|0);
    $202 = HEAP32[$201>>2]|0;
    _png_crc_read($195,$198,$202);
    $203 = $0;
    $204 = (($203) + 208|0);
    $205 = (($204) + 4|0);
    $206 = HEAP32[$205>>2]|0;
    $207 = $0;
    $208 = (($207) + 360|0);
    $209 = HEAP32[$208>>2]|0;
    $210 = (($209) - ($206))|0;
    HEAP32[$208>>2] = $210;
   }
   $211 = $0;
   $212 = (($211) + 208|0);
   $213 = (_inflate($212,1)|0);
   $ret = $213;
   $214 = $ret;
   $215 = ($214|0)==(1);
   if ($215) {
    label = 32;
    break;
   }
   $239 = $ret;
   $240 = ($239|0)!=(0);
   if ($240) {
    $241 = $0;
    $242 = $0;
    $243 = (($242) + 208|0);
    $244 = (($243) + 24|0);
    $245 = HEAP32[$244>>2]|0;
    $246 = ($245|0)!=(0|0);
    if ($246) {
     $247 = $0;
     $248 = (($247) + 208|0);
     $249 = (($248) + 24|0);
     $250 = HEAP32[$249>>2]|0;
     $251 = $250;
    } else {
     $251 = 62776;
    }
    _png_error($241,$251);
   }
   $252 = $0;
   $253 = (($252) + 208|0);
   $254 = (($253) + 16|0);
   $255 = HEAP32[$254>>2]|0;
   $256 = ($255|0)!=(0);
   if (!($256)) {
    label = 43;
    break;
   }
  }
  if ((label|0) == 32) {
   $216 = $0;
   $217 = (($216) + 208|0);
   $218 = (($217) + 16|0);
   $219 = HEAP32[$218>>2]|0;
   $220 = ($219|0)!=(0);
   if ($220) {
    $221 = $0;
    $222 = (($221) + 208|0);
    $223 = (($222) + 4|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = ($224|0)!=(0);
    if ($225) {
     label = 35;
    } else {
     $226 = $0;
     $227 = (($226) + 360|0);
     $228 = HEAP32[$227>>2]|0;
     $229 = ($228|0)!=(0);
     if ($229) {
      label = 35;
     }
    }
   } else {
    label = 35;
   }
   if ((label|0) == 35) {
    $230 = $0;
    _png_warning($230,62752);
   }
   $231 = $0;
   $232 = (($231) + 196|0);
   $233 = HEAP32[$232>>2]|0;
   $234 = $233 | 8;
   HEAP32[$232>>2] = $234;
   $235 = $0;
   $236 = (($235) + 200|0);
   $237 = HEAP32[$236>>2]|0;
   $238 = $237 | 32;
   HEAP32[$236>>2] = $238;
  }
  else if ((label|0) == 43) {
   $257 = $0;
   _png_warning($257,62800);
   $258 = $0;
   $259 = (($258) + 196|0);
   $260 = HEAP32[$259>>2]|0;
   $261 = $260 | 8;
   HEAP32[$259>>2] = $261;
   $262 = $0;
   $263 = (($262) + 200|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = $264 | 32;
   HEAP32[$263>>2] = $265;
  }
  $266 = $0;
  $267 = (($266) + 208|0);
  $268 = (($267) + 16|0);
  HEAP32[$268>>2] = 0;
 }
 $269 = $0;
 $270 = (($269) + 360|0);
 $271 = HEAP32[$270>>2]|0;
 $272 = ($271|0)!=(0);
 if ($272) {
  label = 48;
 } else {
  $273 = $0;
  $274 = (($273) + 208|0);
  $275 = (($274) + 4|0);
  $276 = HEAP32[$275>>2]|0;
  $277 = ($276|0)!=(0);
  if ($277) {
   label = 48;
  }
 }
 if ((label|0) == 48) {
  $278 = $0;
  _png_warning($278,62824);
 }
 $279 = $0;
 $280 = (($279) + 208|0);
 (_inflateReset($280)|0);
 $281 = $0;
 $282 = (($281) + 196|0);
 $283 = HEAP32[$282>>2]|0;
 $284 = $283 | 8;
 HEAP32[$282>>2] = $284;
 STACKTOP = sp;return;
}
function _png_read_start_row($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $max_pixel_depth = 0, $row_bytes = 0, $user_pixel_depth = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $0;
 $2 = (($1) + 208|0);
 $3 = (($2) + 4|0);
 HEAP32[$3>>2] = 0;
 $4 = $0;
 _png_init_read_transformations($4);
 $5 = $0;
 $6 = (($5) + 383|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = $0;
  $10 = (($9) + 204|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 2;
  $13 = ($12|0)!=(0);
  if ($13) {
   $26 = $0;
   $27 = (($26) + 296|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = $0;
   $30 = (($29) + 300|0);
   HEAP32[$30>>2] = $28;
  } else {
   $14 = $0;
   $15 = (($14) + 296|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[57384>>2]|0;
   $18 = (($16) + ($17))|0;
   $19 = (($18) - 1)|0;
   $20 = HEAP32[57352>>2]|0;
   $21 = (($19) - ($20))|0;
   $22 = HEAP32[57384>>2]|0;
   $23 = (($21>>>0) / ($22>>>0))&-1;
   $24 = $0;
   $25 = (($24) + 300|0);
   HEAP32[$25>>2] = $23;
  }
  $31 = $0;
  $32 = (($31) + 292|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $0;
  $35 = (($34) + 384|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = (57320 + ($37<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($33) + ($39))|0;
  $41 = (($40) - 1)|0;
  $42 = $0;
  $43 = (($42) + 384|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = (57288 + ($45<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($41) - ($47))|0;
  $49 = $0;
  $50 = (($49) + 384|0);
  $51 = HEAP8[$50>>0]|0;
  $52 = $51&255;
  $53 = (57320 + ($52<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($48>>>0) / ($54>>>0))&-1;
  $56 = $0;
  $57 = (($56) + 316|0);
  HEAP32[$57>>2] = $55;
  $58 = $0;
  $59 = (($58) + 389|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = ($61|0)>=(8);
  if ($62) {
   $63 = $0;
   $64 = (($63) + 316|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $0;
   $67 = (($66) + 389|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = $68&255;
   $70 = $69 >>> 3;
   $71 = Math_imul($65, $70)|0;
   $83 = $71;
  } else {
   $72 = $0;
   $73 = (($72) + 316|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = $0;
   $76 = (($75) + 389|0);
   $77 = HEAP8[$76>>0]|0;
   $78 = $77&255;
   $79 = Math_imul($74, $78)|0;
   $80 = (($79) + 7)|0;
   $81 = $80 >>> 3;
   $83 = $81;
  }
  $82 = (($83) + 1)|0;
  $row_bytes = $82;
  $84 = $row_bytes;
  $85 = $0;
  $86 = (($85) + 312|0);
  HEAP32[$86>>2] = $84;
  $87 = $0;
  $88 = (($87) + 312|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $row_bytes;
  $91 = ($89|0)!=($90|0);
  if ($91) {
   $92 = $0;
   _png_error($92,62848);
  }
 } else {
  $93 = $0;
  $94 = (($93) + 296|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = $0;
  $97 = (($96) + 300|0);
  HEAP32[$97>>2] = $95;
  $98 = $0;
  $99 = (($98) + 292|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = $0;
  $102 = (($101) + 316|0);
  HEAP32[$102>>2] = $100;
  $103 = $0;
  $104 = (($103) + 308|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = (($105) + 1)|0;
  $107 = $0;
  $108 = (($107) + 312|0);
  HEAP32[$108>>2] = $106;
 }
 $109 = $0;
 $110 = (($109) + 389|0);
 $111 = HEAP8[$110>>0]|0;
 $112 = $111&255;
 $max_pixel_depth = $112;
 $113 = $0;
 $114 = (($113) + 204|0);
 $115 = HEAP32[$114>>2]|0;
 $116 = $115 & 4;
 $117 = ($116|0)!=(0);
 if ($117) {
  $118 = $0;
  $119 = (($118) + 387|0);
  $120 = HEAP8[$119>>0]|0;
  $121 = $120&255;
  $122 = ($121|0)<(8);
  if ($122) {
   $max_pixel_depth = 8;
  }
 }
 $123 = $0;
 $124 = (($123) + 204|0);
 $125 = HEAP32[$124>>2]|0;
 $126 = $125 & 4096;
 $127 = ($126|0)!=(0);
 if ($127) {
  $128 = $0;
  $129 = (($128) + 386|0);
  $130 = HEAP8[$129>>0]|0;
  $131 = $130&255;
  $132 = ($131|0)==(3);
  if ($132) {
   $133 = $0;
   $134 = (($133) + 374|0);
   $135 = HEAP16[$134>>1]|0;
   $136 = ($135<<16>>16)!=(0);
   if ($136) {
    $max_pixel_depth = 32;
   } else {
    $max_pixel_depth = 24;
   }
  } else {
   $137 = $0;
   $138 = (($137) + 386|0);
   $139 = HEAP8[$138>>0]|0;
   $140 = $139&255;
   $141 = ($140|0)==(0);
   if ($141) {
    $142 = $max_pixel_depth;
    $143 = ($142|0)<(8);
    if ($143) {
     $max_pixel_depth = 8;
    }
    $144 = $0;
    $145 = (($144) + 374|0);
    $146 = HEAP16[$145>>1]|0;
    $147 = ($146<<16>>16)!=(0);
    if ($147) {
     $148 = $max_pixel_depth;
     $149 = $148<<1;
     $max_pixel_depth = $149;
    }
   } else {
    $150 = $0;
    $151 = (($150) + 386|0);
    $152 = HEAP8[$151>>0]|0;
    $153 = $152&255;
    $154 = ($153|0)==(2);
    if ($154) {
     $155 = $0;
     $156 = (($155) + 374|0);
     $157 = HEAP16[$156>>1]|0;
     $158 = ($157<<16>>16)!=(0);
     if ($158) {
      $159 = $max_pixel_depth;
      $160 = $159<<2;
      $max_pixel_depth = $160;
      $161 = $max_pixel_depth;
      $162 = (($161|0) / 3)&-1;
      $max_pixel_depth = $162;
     }
    }
   }
  }
 }
 $163 = $0;
 $164 = (($163) + 204|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = $165 & 32768;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = $0;
  $169 = (($168) + 386|0);
  $170 = HEAP8[$169>>0]|0;
  $171 = $170&255;
  $172 = ($171|0)==(3);
  if ($172) {
   $max_pixel_depth = 32;
  } else {
   $173 = $0;
   $174 = (($173) + 386|0);
   $175 = HEAP8[$174>>0]|0;
   $176 = $175&255;
   $177 = ($176|0)==(0);
   if ($177) {
    $178 = $max_pixel_depth;
    $179 = ($178|0)<=(8);
    if ($179) {
     $max_pixel_depth = 16;
    } else {
     $max_pixel_depth = 32;
    }
   } else {
    $180 = $0;
    $181 = (($180) + 386|0);
    $182 = HEAP8[$181>>0]|0;
    $183 = $182&255;
    $184 = ($183|0)==(2);
    if ($184) {
     $185 = $max_pixel_depth;
     $186 = ($185|0)<=(32);
     if ($186) {
      $max_pixel_depth = 32;
     } else {
      $max_pixel_depth = 64;
     }
    }
   }
  }
 }
 $187 = $0;
 $188 = (($187) + 204|0);
 $189 = HEAP32[$188>>2]|0;
 $190 = $189 & 16384;
 $191 = ($190|0)!=(0);
 if ($191) {
  $192 = $0;
  $193 = (($192) + 374|0);
  $194 = HEAP16[$193>>1]|0;
  $195 = $194&65535;
  $196 = ($195|0)!=(0);
  if ($196) {
   $197 = $0;
   $198 = (($197) + 204|0);
   $199 = HEAP32[$198>>2]|0;
   $200 = $199 & 4096;
   $201 = ($200|0)!=(0);
   if ($201) {
    label = 55;
   } else {
    label = 53;
   }
  } else {
   label = 53;
  }
  if ((label|0) == 53) {
   $202 = $0;
   $203 = (($202) + 204|0);
   $204 = HEAP32[$203>>2]|0;
   $205 = $204 & 32768;
   $206 = ($205|0)!=(0);
   if ($206) {
    label = 55;
   } else {
    $207 = $0;
    $208 = (($207) + 386|0);
    $209 = HEAP8[$208>>0]|0;
    $210 = $209&255;
    $211 = ($210|0)==(4);
    if ($211) {
     label = 55;
    } else {
     $214 = $max_pixel_depth;
     $215 = ($214|0)<=(8);
     if ($215) {
      $216 = $0;
      $217 = (($216) + 386|0);
      $218 = HEAP8[$217>>0]|0;
      $219 = $218&255;
      $220 = ($219|0)==(6);
      if ($220) {
       $max_pixel_depth = 32;
      } else {
       $max_pixel_depth = 24;
      }
     } else {
      $221 = $0;
      $222 = (($221) + 386|0);
      $223 = HEAP8[$222>>0]|0;
      $224 = $223&255;
      $225 = ($224|0)==(6);
      if ($225) {
       $max_pixel_depth = 64;
      } else {
       $max_pixel_depth = 48;
      }
     }
    }
   }
  }
  if ((label|0) == 55) {
   $212 = $max_pixel_depth;
   $213 = ($212|0)<=(16);
   if ($213) {
    $max_pixel_depth = 32;
   } else {
    $max_pixel_depth = 64;
   }
  }
 }
 $226 = $0;
 $227 = (($226) + 204|0);
 $228 = HEAP32[$227>>2]|0;
 $229 = $228 & 1048576;
 $230 = ($229|0)!=(0);
 if ($230) {
  $231 = $0;
  $232 = (($231) + 192|0);
  $233 = HEAP8[$232>>0]|0;
  $234 = $233&255;
  $235 = $0;
  $236 = (($235) + 193|0);
  $237 = HEAP8[$236>>0]|0;
  $238 = $237&255;
  $239 = Math_imul($234, $238)|0;
  $user_pixel_depth = $239;
  $240 = $user_pixel_depth;
  $241 = $max_pixel_depth;
  $242 = ($240|0)>($241|0);
  if ($242) {
   $243 = $user_pixel_depth;
   $max_pixel_depth = $243;
  }
 }
 $244 = $0;
 $245 = (($244) + 292|0);
 $246 = HEAP32[$245>>2]|0;
 $247 = (($246) + 7)|0;
 $248 = $247 & -8;
 $row_bytes = $248;
 $249 = $max_pixel_depth;
 $250 = ($249|0)>=(8);
 if ($250) {
  $251 = $row_bytes;
  $252 = $max_pixel_depth;
  $253 = $252 >>> 3;
  $254 = Math_imul($251, $253)|0;
  $261 = $254;
 } else {
  $255 = $row_bytes;
  $256 = $max_pixel_depth;
  $257 = Math_imul($255, $256)|0;
  $258 = (($257) + 7)|0;
  $259 = $258 >>> 3;
  $261 = $259;
 }
 $260 = (($261) + 1)|0;
 $262 = $max_pixel_depth;
 $263 = (($262) + 7)|0;
 $264 = $263 >> 3;
 $265 = (($260) + ($264))|0;
 $row_bytes = $265;
 $266 = $0;
 $267 = $row_bytes;
 $268 = (($267) + 64)|0;
 $269 = (_png_malloc($266,$268)|0);
 $270 = $0;
 $271 = (($270) + 684|0);
 HEAP32[$271>>2] = $269;
 $272 = $0;
 $273 = (($272) + 684|0);
 $274 = HEAP32[$273>>2]|0;
 $275 = (($274) + 32|0);
 $276 = $0;
 $277 = (($276) + 328|0);
 HEAP32[$277>>2] = $275;
 $278 = $0;
 $279 = (($278) + 308|0);
 $280 = HEAP32[$279>>2]|0;
 $281 = ($280>>>0)>(4294967294);
 if (!($281)) {
  $283 = $0;
  $284 = $0;
  $285 = (($284) + 308|0);
  $286 = HEAP32[$285>>2]|0;
  $287 = (($286) + 1)|0;
  $288 = (_png_malloc($283,$287)|0);
  $289 = $0;
  $290 = (($289) + 324|0);
  HEAP32[$290>>2] = $288;
  $291 = $0;
  $292 = $0;
  $293 = (($292) + 324|0);
  $294 = HEAP32[$293>>2]|0;
  $295 = $0;
  $296 = (($295) + 308|0);
  $297 = HEAP32[$296>>2]|0;
  $298 = (($297) + 1)|0;
  (_png_memset_check($291,$294,0,$298)|0);
  $299 = $0;
  $300 = (($299) + 200|0);
  $301 = HEAP32[$300>>2]|0;
  $302 = $301 | 64;
  HEAP32[$300>>2] = $302;
  STACKTOP = sp;return;
 }
 $282 = $0;
 _png_error($282,62888);
 $283 = $0;
 $284 = $0;
 $285 = (($284) + 308|0);
 $286 = HEAP32[$285>>2]|0;
 $287 = (($286) + 1)|0;
 $288 = (_png_malloc($283,$287)|0);
 $289 = $0;
 $290 = (($289) + 324|0);
 HEAP32[$290>>2] = $288;
 $291 = $0;
 $292 = $0;
 $293 = (($292) + 324|0);
 $294 = HEAP32[$293>>2]|0;
 $295 = $0;
 $296 = (($295) + 308|0);
 $297 = HEAP32[$296>>2]|0;
 $298 = (($297) + 1)|0;
 (_png_memset_check($291,$294,0,$298)|0);
 $299 = $0;
 $300 = (($299) + 200|0);
 $301 = HEAP32[$300>>2]|0;
 $302 = $301 | 64;
 HEAP32[$300>>2] = $302;
 STACKTOP = sp;return;
}
function _png_set_bKGD($png_ptr,$info_ptr,$background) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $background = $background|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $background;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $1;
   $8 = (($7) + 90|0);
   $9 = $2;
   ;HEAP16[$8+0>>1]=HEAP16[$9+0>>1]|0;HEAP16[$8+2>>1]=HEAP16[$9+2>>1]|0;HEAP16[$8+4>>1]=HEAP16[$9+4>>1]|0;HEAP16[$8+6>>1]=HEAP16[$9+6>>1]|0;HEAP16[$8+8>>1]=HEAP16[$9+8>>1]|0;
   $10 = $1;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = $12 | 32;
   HEAP32[$11>>2] = $13;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_cHRM($png_ptr,$info_ptr,$white_x,$white_y,$red_x,$red_y,$green_x,$green_y,$blue_x,$blue_y) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $white_x = +$white_x;
 $white_y = +$white_y;
 $red_x = +$red_x;
 $red_y = +$red_y;
 $green_x = +$green_x;
 $green_y = +$green_y;
 $blue_x = +$blue_x;
 $blue_y = +$blue_y;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0.0, $123 = 0.0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0.0, $135 = 0.0, $136 = 0, $137 = 0, $138 = 0, $139 = 0.0, $14 = 0.0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0.0, $17 = 0, $18 = 0.0;
 var $19 = 0, $2 = 0.0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0.0, $27 = 0, $28 = 0.0, $29 = 0, $3 = 0.0, $30 = 0, $31 = 0.0, $32 = 0, $33 = 0.0, $34 = 0, $35 = 0.0, $36 = 0;
 var $37 = 0.0, $38 = 0, $39 = 0.0, $4 = 0.0, $40 = 0, $41 = 0.0, $42 = 0, $43 = 0.0, $44 = 0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0, $5 = 0.0, $50 = 0.0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0.0;
 var $55 = 0, $56 = 0.0, $57 = 0, $58 = 0.0, $59 = 0, $6 = 0.0, $60 = 0.0, $61 = 0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0, $69 = 0.0, $7 = 0.0, $70 = 0.0, $71 = 0, $72 = 0;
 var $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0.0, $80 = 0, $81 = 0.0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0.0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0.0, $9 = 0.0, $90 = 0.0;
 var $91 = 0, $92 = 0, $93 = 0.0, $94 = 0.0, $95 = 0, $96 = 0, $97 = 0.0, $98 = 0.0, $99 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $white_x;
 $3 = $white_y;
 $4 = $red_x;
 $5 = $red_y;
 $6 = $green_x;
 $7 = $green_y;
 $8 = $blue_x;
 $9 = $blue_y;
 $10 = $0;
 $11 = ($10|0)==(0|0);
 if (!($11)) {
  $12 = $1;
  $13 = ($12|0)==(0|0);
  if (!($13)) {
   $14 = $2;
   $15 = $14 != 0.0;
   if (!($15)) {
    $16 = $3;
    $17 = $16 != 0.0;
    if (!($17)) {
     $18 = $4;
     $19 = $18 != 0.0;
     if (!($19)) {
      $20 = $5;
      $21 = $20 != 0.0;
      if (!($21)) {
       $22 = $6;
       $23 = $22 != 0.0;
       if (!($23)) {
        $24 = $7;
        $25 = $24 != 0.0;
        if (!($25)) {
         $26 = $8;
         $27 = $26 != 0.0;
         if (!($27)) {
          $28 = $9;
          $29 = $28 != 0.0;
          if (!($29)) {
           $30 = $0;
           _png_warning($30,62936);
           STACKTOP = sp;return;
          }
         }
        }
       }
      }
     }
    }
   }
   $31 = $2;
   $32 = $31 < 0.0;
   if (!($32)) {
    $33 = $3;
    $34 = $33 < 0.0;
    if (!($34)) {
     $35 = $4;
     $36 = $35 < 0.0;
     if (!($36)) {
      $37 = $5;
      $38 = $37 < 0.0;
      if (!($38)) {
       $39 = $6;
       $40 = $39 < 0.0;
       if (!($40)) {
        $41 = $7;
        $42 = $41 < 0.0;
        if (!($42)) {
         $43 = $8;
         $44 = $43 < 0.0;
         if (!($44)) {
          $45 = $9;
          $46 = $45 < 0.0;
          if (!($46)) {
           $48 = $2;
           $49 = $48 > 21474.8300000000017462;
           do {
            if (!($49)) {
             $50 = $3;
             $51 = $50 > 21474.8300000000017462;
             if (!($51)) {
              $52 = $4;
              $53 = $52 > 21474.8300000000017462;
              if (!($53)) {
               $54 = $5;
               $55 = $54 > 21474.8300000000017462;
               if (!($55)) {
                $56 = $6;
                $57 = $56 > 21474.8300000000017462;
                if (!($57)) {
                 $58 = $7;
                 $59 = $58 > 21474.8300000000017462;
                 if (!($59)) {
                  $60 = $8;
                  $61 = $60 > 21474.8300000000017462;
                  if ($61) {
                   break;
                  }
                  $62 = $9;
                  $63 = $62 > 21474.8300000000017462;
                  if ($63) {
                   break;
                  }
                  $65 = $2;
                  $66 = $65;
                  $67 = $1;
                  $68 = (($67) + 128|0);
                  HEAPF32[$68>>2] = $66;
                  $69 = $3;
                  $70 = $69;
                  $71 = $1;
                  $72 = (($71) + 132|0);
                  HEAPF32[$72>>2] = $70;
                  $73 = $4;
                  $74 = $73;
                  $75 = $1;
                  $76 = (($75) + 136|0);
                  HEAPF32[$76>>2] = $74;
                  $77 = $5;
                  $78 = $77;
                  $79 = $1;
                  $80 = (($79) + 140|0);
                  HEAPF32[$80>>2] = $78;
                  $81 = $6;
                  $82 = $81;
                  $83 = $1;
                  $84 = (($83) + 144|0);
                  HEAPF32[$84>>2] = $82;
                  $85 = $7;
                  $86 = $85;
                  $87 = $1;
                  $88 = (($87) + 148|0);
                  HEAPF32[$88>>2] = $86;
                  $89 = $8;
                  $90 = $89;
                  $91 = $1;
                  $92 = (($91) + 152|0);
                  HEAPF32[$92>>2] = $90;
                  $93 = $9;
                  $94 = $93;
                  $95 = $1;
                  $96 = (($95) + 156|0);
                  HEAPF32[$96>>2] = $94;
                  $97 = $2;
                  $98 = $97 * 1.0E+5;
                  $99 = $98 + 0.5;
                  $100 = (~~(($99)));
                  $101 = $1;
                  $102 = (($101) + 256|0);
                  HEAP32[$102>>2] = $100;
                  $103 = $3;
                  $104 = $103 * 1.0E+5;
                  $105 = $104 + 0.5;
                  $106 = (~~(($105)));
                  $107 = $1;
                  $108 = (($107) + 260|0);
                  HEAP32[$108>>2] = $106;
                  $109 = $4;
                  $110 = $109 * 1.0E+5;
                  $111 = $110 + 0.5;
                  $112 = (~~(($111)));
                  $113 = $1;
                  $114 = (($113) + 264|0);
                  HEAP32[$114>>2] = $112;
                  $115 = $5;
                  $116 = $115 * 1.0E+5;
                  $117 = $116 + 0.5;
                  $118 = (~~(($117)));
                  $119 = $1;
                  $120 = (($119) + 268|0);
                  HEAP32[$120>>2] = $118;
                  $121 = $6;
                  $122 = $121 * 1.0E+5;
                  $123 = $122 + 0.5;
                  $124 = (~~(($123)));
                  $125 = $1;
                  $126 = (($125) + 272|0);
                  HEAP32[$126>>2] = $124;
                  $127 = $7;
                  $128 = $127 * 1.0E+5;
                  $129 = $128 + 0.5;
                  $130 = (~~(($129)));
                  $131 = $1;
                  $132 = (($131) + 276|0);
                  HEAP32[$132>>2] = $130;
                  $133 = $8;
                  $134 = $133 * 1.0E+5;
                  $135 = $134 + 0.5;
                  $136 = (~~(($135)));
                  $137 = $1;
                  $138 = (($137) + 280|0);
                  HEAP32[$138>>2] = $136;
                  $139 = $9;
                  $140 = $139 * 1.0E+5;
                  $141 = $140 + 0.5;
                  $142 = (~~(($141)));
                  $143 = $1;
                  $144 = (($143) + 284|0);
                  HEAP32[$144>>2] = $142;
                  $145 = $1;
                  $146 = (($145) + 8|0);
                  $147 = HEAP32[$146>>2]|0;
                  $148 = $147 | 4;
                  HEAP32[$146>>2] = $148;
                  STACKTOP = sp;return;
                 }
                }
               }
              }
             }
            }
           } while(0);
           $64 = $0;
           _png_warning($64,63048);
           STACKTOP = sp;return;
          }
         }
        }
       }
      }
     }
    }
   }
   $47 = $0;
   _png_warning($47,62992);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_cHRM_fixed($png_ptr,$info_ptr,$white_x,$white_y,$red_x,$red_y,$green_x,$green_y,$blue_x,$blue_y) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $white_x = $white_x|0;
 $white_y = $white_y|0;
 $red_x = $red_x|0;
 $red_y = $red_y|0;
 $green_x = $green_x|0;
 $green_y = $green_y|0;
 $blue_x = $blue_x|0;
 $blue_y = $blue_y|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $104 = 0.0, $105 = 0.0, $106 = 0.0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0.0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0.0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0;
 var $55 = 0.0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $white_x;
 $3 = $white_y;
 $4 = $red_x;
 $5 = $red_y;
 $6 = $green_x;
 $7 = $green_y;
 $8 = $blue_x;
 $9 = $blue_y;
 $10 = $0;
 $11 = ($10|0)==(0|0);
 if (!($11)) {
  $12 = $1;
  $13 = ($12|0)==(0|0);
  if (!($13)) {
   $14 = $2;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    $16 = $3;
    $17 = ($16|0)!=(0);
    if (!($17)) {
     $18 = $4;
     $19 = ($18|0)!=(0);
     if (!($19)) {
      $20 = $5;
      $21 = ($20|0)!=(0);
      if (!($21)) {
       $22 = $6;
       $23 = ($22|0)!=(0);
       if (!($23)) {
        $24 = $7;
        $25 = ($24|0)!=(0);
        if (!($25)) {
         $26 = $8;
         $27 = ($26|0)!=(0);
         if (!($27)) {
          $28 = $9;
          $29 = ($28|0)!=(0);
          if (!($29)) {
           $30 = $0;
           _png_warning($30,62936);
           STACKTOP = sp;return;
          }
         }
        }
       }
      }
     }
    }
   }
   $31 = $2;
   $32 = ($31|0)<(0);
   if (!($32)) {
    $33 = $3;
    $34 = ($33|0)<(0);
    if (!($34)) {
     $35 = $4;
     $36 = ($35|0)<(0);
     if (!($36)) {
      $37 = $5;
      $38 = ($37|0)<(0);
      if (!($38)) {
       $39 = $6;
       $40 = ($39|0)<(0);
       if (!($40)) {
        $41 = $7;
        $42 = ($41|0)<(0);
        if (!($42)) {
         $43 = $8;
         $44 = ($43|0)<(0);
         if (!($44)) {
          $45 = $9;
          $46 = ($45|0)<(0);
          if (!($46)) {
           $48 = $2;
           $49 = (+($48|0));
           $50 = $49 > 2147483647.0;
           do {
            if (!($50)) {
             $51 = $3;
             $52 = (+($51|0));
             $53 = $52 > 2147483647.0;
             if (!($53)) {
              $54 = $4;
              $55 = (+($54|0));
              $56 = $55 > 2147483647.0;
              if (!($56)) {
               $57 = $5;
               $58 = (+($57|0));
               $59 = $58 > 2147483647.0;
               if (!($59)) {
                $60 = $6;
                $61 = (+($60|0));
                $62 = $61 > 2147483647.0;
                if (!($62)) {
                 $63 = $7;
                 $64 = (+($63|0));
                 $65 = $64 > 2147483647.0;
                 if (!($65)) {
                  $66 = $8;
                  $67 = (+($66|0));
                  $68 = $67 > 2147483647.0;
                  if ($68) {
                   break;
                  }
                  $69 = $9;
                  $70 = (+($69|0));
                  $71 = $70 > 2147483647.0;
                  if ($71) {
                   break;
                  }
                  $73 = $2;
                  $74 = $1;
                  $75 = (($74) + 256|0);
                  HEAP32[$75>>2] = $73;
                  $76 = $3;
                  $77 = $1;
                  $78 = (($77) + 260|0);
                  HEAP32[$78>>2] = $76;
                  $79 = $4;
                  $80 = $1;
                  $81 = (($80) + 264|0);
                  HEAP32[$81>>2] = $79;
                  $82 = $5;
                  $83 = $1;
                  $84 = (($83) + 268|0);
                  HEAP32[$84>>2] = $82;
                  $85 = $6;
                  $86 = $1;
                  $87 = (($86) + 272|0);
                  HEAP32[$87>>2] = $85;
                  $88 = $7;
                  $89 = $1;
                  $90 = (($89) + 276|0);
                  HEAP32[$90>>2] = $88;
                  $91 = $8;
                  $92 = $1;
                  $93 = (($92) + 280|0);
                  HEAP32[$93>>2] = $91;
                  $94 = $9;
                  $95 = $1;
                  $96 = (($95) + 284|0);
                  HEAP32[$96>>2] = $94;
                  $97 = $2;
                  $98 = (+($97|0));
                  $99 = $98 / 1.0E+5;
                  $100 = $99;
                  $101 = $1;
                  $102 = (($101) + 128|0);
                  HEAPF32[$102>>2] = $100;
                  $103 = $3;
                  $104 = (+($103|0));
                  $105 = $104 / 1.0E+5;
                  $106 = $105;
                  $107 = $1;
                  $108 = (($107) + 132|0);
                  HEAPF32[$108>>2] = $106;
                  $109 = $4;
                  $110 = (+($109|0));
                  $111 = $110 / 1.0E+5;
                  $112 = $111;
                  $113 = $1;
                  $114 = (($113) + 136|0);
                  HEAPF32[$114>>2] = $112;
                  $115 = $5;
                  $116 = (+($115|0));
                  $117 = $116 / 1.0E+5;
                  $118 = $117;
                  $119 = $1;
                  $120 = (($119) + 140|0);
                  HEAPF32[$120>>2] = $118;
                  $121 = $6;
                  $122 = (+($121|0));
                  $123 = $122 / 1.0E+5;
                  $124 = $123;
                  $125 = $1;
                  $126 = (($125) + 144|0);
                  HEAPF32[$126>>2] = $124;
                  $127 = $7;
                  $128 = (+($127|0));
                  $129 = $128 / 1.0E+5;
                  $130 = $129;
                  $131 = $1;
                  $132 = (($131) + 148|0);
                  HEAPF32[$132>>2] = $130;
                  $133 = $8;
                  $134 = (+($133|0));
                  $135 = $134 / 1.0E+5;
                  $136 = $135;
                  $137 = $1;
                  $138 = (($137) + 152|0);
                  HEAPF32[$138>>2] = $136;
                  $139 = $9;
                  $140 = (+($139|0));
                  $141 = $140 / 1.0E+5;
                  $142 = $141;
                  $143 = $1;
                  $144 = (($143) + 156|0);
                  HEAPF32[$144>>2] = $142;
                  $145 = $1;
                  $146 = (($145) + 8|0);
                  $147 = HEAP32[$146>>2]|0;
                  $148 = $147 | 4;
                  HEAP32[$146>>2] = $148;
                  STACKTOP = sp;return;
                 }
                }
               }
              }
             }
            }
           } while(0);
           $72 = $0;
           _png_warning($72,63048);
           STACKTOP = sp;return;
          }
         }
        }
       }
      }
     }
    }
   }
   $47 = $0;
   _png_warning($47,62992);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_gAMA($png_ptr,$info_ptr,$file_gamma) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $file_gamma = +$file_gamma;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0, $9 = 0, $gamma = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $file_gamma;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $2;
   $8 = $7 > 21474.8300000000017462;
   if ($8) {
    $9 = $0;
    _png_warning($9,63112);
    $gamma = 21474.8300000000017462;
   } else {
    $10 = $2;
    $gamma = $10;
   }
   $11 = $gamma;
   $12 = $11;
   $13 = $1;
   $14 = (($13) + 40|0);
   HEAPF32[$14>>2] = $12;
   $15 = $gamma;
   $16 = $15 * 1.0E+5;
   $17 = $16 + 0.5;
   $18 = (~~(($17)));
   $19 = $1;
   $20 = (($19) + 252|0);
   HEAP32[$20>>2] = $18;
   $21 = $1;
   $22 = (($21) + 8|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = $23 | 1;
   HEAP32[$22>>2] = $24;
   $25 = $gamma;
   $26 = $25 == 0.0;
   if (!($26)) {
    STACKTOP = sp;return;
   }
   $27 = $0;
   _png_warning($27,63144);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_gAMA_fixed($png_ptr,$info_ptr,$int_gamma) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $int_gamma = $int_gamma|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $gamma = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $int_gamma;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $2;
   $8 = ($7|0)>(2147483647);
   if ($8) {
    $9 = $0;
    _png_warning($9,63112);
    $gamma = 2147483647;
   } else {
    $10 = $2;
    $11 = ($10|0)<(0);
    if ($11) {
     $12 = $0;
     _png_warning($12,63160);
     $gamma = 0;
    } else {
     $13 = $2;
     $gamma = $13;
    }
   }
   $14 = $gamma;
   $15 = (+($14|0));
   $16 = $15 / 1.0E+5;
   $17 = $16;
   $18 = $1;
   $19 = (($18) + 40|0);
   HEAPF32[$19>>2] = $17;
   $20 = $gamma;
   $21 = $1;
   $22 = (($21) + 252|0);
   HEAP32[$22>>2] = $20;
   $23 = $1;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25 | 1;
   HEAP32[$24>>2] = $26;
   $27 = $gamma;
   $28 = ($27|0)==(0);
   if (!($28)) {
    STACKTOP = sp;return;
   }
   $29 = $0;
   _png_warning($29,63144);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_hIST($png_ptr,$info_ptr,$hist) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $hist = $hist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $hist;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $1;
   $8 = (($7) + 20|0);
   $9 = HEAP16[$8>>1]|0;
   $10 = $9&65535;
   $11 = ($10|0)==(0);
   if (!($11)) {
    $12 = $1;
    $13 = (($12) + 20|0);
    $14 = HEAP16[$13>>1]|0;
    $15 = $14&65535;
    $16 = ($15|0)>(256);
    if (!($16)) {
     $18 = $0;
     $19 = $1;
     _png_free_data($18,$19,8,0);
     $20 = $0;
     $21 = (_png_malloc_warn($20,512)|0);
     $22 = $0;
     $23 = (($22) + 592|0);
     HEAP32[$23>>2] = $21;
     $24 = $0;
     $25 = (($24) + 592|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==(0|0);
     if ($27) {
      $28 = $0;
      _png_warning($28,63240);
      STACKTOP = sp;return;
     }
     $i = 0;
     while(1) {
      $29 = $i;
      $30 = $1;
      $31 = (($30) + 20|0);
      $32 = HEAP16[$31>>1]|0;
      $33 = $32&65535;
      $34 = ($29|0)<($33|0);
      if (!($34)) {
       break;
      }
      $35 = $i;
      $36 = $2;
      $37 = (($36) + ($35<<1)|0);
      $38 = HEAP16[$37>>1]|0;
      $39 = $i;
      $40 = $0;
      $41 = (($40) + 592|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = (($42) + ($39<<1)|0);
      HEAP16[$43>>1] = $38;
      $44 = $i;
      $45 = (($44) + 1)|0;
      $i = $45;
     }
     $46 = $0;
     $47 = (($46) + 592|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = $1;
     $50 = (($49) + 124|0);
     HEAP32[$50>>2] = $48;
     $51 = $1;
     $52 = (($51) + 8|0);
     $53 = HEAP32[$52>>2]|0;
     $54 = $53 | 64;
     HEAP32[$52>>2] = $54;
     $55 = $1;
     $56 = (($55) + 184|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = $57 | 8;
     HEAP32[$56>>2] = $58;
     STACKTOP = sp;return;
    }
   }
   $17 = $0;
   _png_warning($17,63192);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_IHDR($png_ptr,$info_ptr,$width,$height,$bit_depth,$color_type,$interlace_type,$compression_type,$filter_type) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $width = $width|0;
 $height = $height|0;
 $bit_depth = $bit_depth|0;
 $color_type = $color_type|0;
 $interlace_type = $interlace_type|0;
 $compression_type = $compression_type|0;
 $filter_type = $filter_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $width;
 $3 = $height;
 $4 = $bit_depth;
 $5 = $color_type;
 $6 = $interlace_type;
 $7 = $compression_type;
 $8 = $filter_type;
 $9 = $0;
 $10 = ($9|0)==(0|0);
 if (!($10)) {
  $11 = $1;
  $12 = ($11|0)==(0|0);
  if (!($12)) {
   $13 = $2;
   $14 = ($13|0)==(0);
   if ($14) {
    label = 6;
   } else {
    $15 = $3;
    $16 = ($15|0)==(0);
    if ($16) {
     label = 6;
    }
   }
   if ((label|0) == 6) {
    $17 = $0;
    _png_error($17,63288);
   }
   $18 = $2;
   $19 = $0;
   $20 = (($19) + 704|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($18>>>0)>($21>>>0);
   if ($22) {
    label = 9;
   } else {
    $23 = $3;
    $24 = $0;
    $25 = (($24) + 708|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ($23>>>0)>($26>>>0);
    if ($27) {
     label = 9;
    }
   }
   if ((label|0) == 9) {
    $28 = $0;
    _png_error($28,63328);
   }
   $29 = $2;
   $30 = ($29>>>0)>(2147483647);
   if ($30) {
    label = 12;
   } else {
    $31 = $3;
    $32 = ($31>>>0)>(2147483647);
    if ($32) {
     label = 12;
    }
   }
   if ((label|0) == 12) {
    $33 = $0;
    _png_error($33,63368);
   }
   $34 = $2;
   $35 = ($34>>>0)>(536870782);
   if ($35) {
    $36 = $0;
    _png_warning($36,63400);
   }
   $37 = $4;
   $38 = ($37|0)!=(1);
   if ($38) {
    $39 = $4;
    $40 = ($39|0)!=(2);
    if ($40) {
     $41 = $4;
     $42 = ($41|0)!=(4);
     if ($42) {
      $43 = $4;
      $44 = ($43|0)!=(8);
      if ($44) {
       $45 = $4;
       $46 = ($45|0)!=(16);
       if ($46) {
        $47 = $0;
        _png_error($47,63448);
       }
      }
     }
    }
   }
   $48 = $5;
   $49 = ($48|0)<(0);
   if ($49) {
    label = 25;
   } else {
    $50 = $5;
    $51 = ($50|0)==(1);
    if ($51) {
     label = 25;
    } else {
     $52 = $5;
     $53 = ($52|0)==(5);
     if ($53) {
      label = 25;
     } else {
      $54 = $5;
      $55 = ($54|0)>(6);
      if ($55) {
       label = 25;
      }
     }
    }
   }
   if ((label|0) == 25) {
    $56 = $0;
    _png_error($56,63480);
   }
   $57 = $5;
   $58 = ($57|0)==(3);
   if ($58) {
    $59 = $4;
    $60 = ($59|0)>(8);
    if ($60) {
     label = 32;
    } else {
     label = 28;
    }
   } else {
    label = 28;
   }
   do {
    if ((label|0) == 28) {
     $61 = $5;
     $62 = ($61|0)==(2);
     if (!($62)) {
      $63 = $5;
      $64 = ($63|0)==(4);
      if (!($64)) {
       $65 = $5;
       $66 = ($65|0)==(6);
       if (!($66)) {
        break;
       }
      }
     }
     $67 = $4;
     $68 = ($67|0)<(8);
     if ($68) {
      label = 32;
     }
    }
   } while(0);
   if ((label|0) == 32) {
    $69 = $0;
    _png_error($69,63512);
   }
   $70 = $6;
   $71 = ($70|0)>=(2);
   if ($71) {
    $72 = $0;
    _png_error($72,63568);
   }
   $73 = $7;
   $74 = ($73|0)!=(0);
   if ($74) {
    $75 = $0;
    _png_error($75,63608);
   }
   $76 = $0;
   $77 = (($76) + 196|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $78 & 4096;
   $80 = ($79|0)!=(0);
   if ($80) {
    $81 = $0;
    $82 = (($81) + 652|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ($83|0)!=(0);
    if ($84) {
     $85 = $0;
     _png_warning($85,63648);
    }
   }
   $86 = $8;
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = $0;
    $89 = (($88) + 652|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = $90 & 4;
    $92 = ($91|0)!=(0);
    if ($92) {
     $93 = $8;
     $94 = ($93|0)==(64);
     if ($94) {
      $95 = $0;
      $96 = (($95) + 196|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = $97 & 4096;
      $99 = ($98|0)==(0);
      if ($99) {
       $100 = $5;
       $101 = ($100|0)==(2);
       if (!($101)) {
        $102 = $5;
        $103 = ($102|0)==(6);
        if (!($103)) {
         label = 46;
        }
       }
      } else {
       label = 46;
      }
     } else {
      label = 46;
     }
    } else {
     label = 46;
    }
    if ((label|0) == 46) {
     $104 = $0;
     _png_error($104,63704);
    }
    $105 = $0;
    $106 = (($105) + 196|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = $107 & 4096;
    $109 = ($108|0)!=(0);
    if ($109) {
     $110 = $0;
     _png_warning($110,63736);
    }
   }
   $111 = $2;
   $112 = $1;
   HEAP32[$112>>2] = $111;
   $113 = $3;
   $114 = $1;
   $115 = (($114) + 4|0);
   HEAP32[$115>>2] = $113;
   $116 = $4;
   $117 = $116&255;
   $118 = $1;
   $119 = (($118) + 24|0);
   HEAP8[$119>>0] = $117;
   $120 = $5;
   $121 = $120&255;
   $122 = $1;
   $123 = (($122) + 25|0);
   HEAP8[$123>>0] = $121;
   $124 = $7;
   $125 = $124&255;
   $126 = $1;
   $127 = (($126) + 26|0);
   HEAP8[$127>>0] = $125;
   $128 = $8;
   $129 = $128&255;
   $130 = $1;
   $131 = (($130) + 27|0);
   HEAP8[$131>>0] = $129;
   $132 = $6;
   $133 = $132&255;
   $134 = $1;
   $135 = (($134) + 28|0);
   HEAP8[$135>>0] = $133;
   $136 = $1;
   $137 = (($136) + 25|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = $138&255;
   $140 = ($139|0)==(3);
   if ($140) {
    $141 = $1;
    $142 = (($141) + 29|0);
    HEAP8[$142>>0] = 1;
   } else {
    $143 = $1;
    $144 = (($143) + 25|0);
    $145 = HEAP8[$144>>0]|0;
    $146 = $145&255;
    $147 = $146 & 2;
    $148 = ($147|0)!=(0);
    if ($148) {
     $149 = $1;
     $150 = (($149) + 29|0);
     HEAP8[$150>>0] = 3;
    } else {
     $151 = $1;
     $152 = (($151) + 29|0);
     HEAP8[$152>>0] = 1;
    }
   }
   $153 = $1;
   $154 = (($153) + 25|0);
   $155 = HEAP8[$154>>0]|0;
   $156 = $155&255;
   $157 = $156 & 4;
   $158 = ($157|0)!=(0);
   if ($158) {
    $159 = $1;
    $160 = (($159) + 29|0);
    $161 = HEAP8[$160>>0]|0;
    $162 = (($161) + 1)<<24>>24;
    HEAP8[$160>>0] = $162;
   }
   $163 = $1;
   $164 = (($163) + 29|0);
   $165 = HEAP8[$164>>0]|0;
   $166 = $165&255;
   $167 = $1;
   $168 = (($167) + 24|0);
   $169 = HEAP8[$168>>0]|0;
   $170 = $169&255;
   $171 = Math_imul($166, $170)|0;
   $172 = $171&255;
   $173 = $1;
   $174 = (($173) + 30|0);
   HEAP8[$174>>0] = $172;
   $175 = $2;
   $176 = ($175>>>0)>(536870782);
   if ($176) {
    $177 = $1;
    $178 = (($177) + 12|0);
    HEAP32[$178>>2] = 0;
    STACKTOP = sp;return;
   }
   $179 = $1;
   $180 = (($179) + 30|0);
   $181 = HEAP8[$180>>0]|0;
   $182 = $181&255;
   $183 = ($182|0)>=(8);
   if ($183) {
    $184 = $2;
    $185 = $1;
    $186 = (($185) + 30|0);
    $187 = HEAP8[$186>>0]|0;
    $188 = $187&255;
    $189 = $188 >>> 3;
    $190 = Math_imul($184, $189)|0;
    $201 = $190;
   } else {
    $191 = $2;
    $192 = $1;
    $193 = (($192) + 30|0);
    $194 = HEAP8[$193>>0]|0;
    $195 = $194&255;
    $196 = Math_imul($191, $195)|0;
    $197 = (($196) + 7)|0;
    $198 = $197 >>> 3;
    $201 = $198;
   }
   $199 = $1;
   $200 = (($199) + 12|0);
   HEAP32[$200>>2] = $201;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_oFFs($png_ptr,$info_ptr,$offset_x,$offset_y,$unit_type) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $offset_x = $offset_x|0;
 $offset_y = $offset_y|0;
 $unit_type = $unit_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $offset_x;
 $3 = $offset_y;
 $4 = $unit_type;
 $5 = $0;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $1;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = $2;
   $10 = $1;
   $11 = (($10) + 100|0);
   HEAP32[$11>>2] = $9;
   $12 = $3;
   $13 = $1;
   $14 = (($13) + 104|0);
   HEAP32[$14>>2] = $12;
   $15 = $4;
   $16 = $15&255;
   $17 = $1;
   $18 = (($17) + 108|0);
   HEAP8[$18>>0] = $16;
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 | 256;
   HEAP32[$20>>2] = $22;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_pCAL($png_ptr,$info_ptr,$purpose,$X0,$X1,$type,$nparams,$units,$params) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $purpose = $purpose|0;
 $X0 = $X0|0;
 $X1 = $X1|0;
 $type = $type|0;
 $nparams = $nparams|0;
 $units = $units|0;
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $length = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $purpose;
 $3 = $X0;
 $4 = $X1;
 $5 = $type;
 $6 = $nparams;
 $7 = $units;
 $8 = $params;
 $9 = $0;
 $10 = ($9|0)==(0|0);
 if (!($10)) {
  $11 = $1;
  $12 = ($11|0)==(0|0);
  if (!($12)) {
   $13 = $2;
   $14 = (_strlen(($13|0))|0);
   $15 = (($14) + 1)|0;
   $length = $15;
   $16 = $0;
   $17 = $length;
   $18 = (_png_malloc_warn($16,$17)|0);
   $19 = $1;
   $20 = (($19) + 160|0);
   HEAP32[$20>>2] = $18;
   $21 = $1;
   $22 = (($21) + 160|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)==(0|0);
   if ($24) {
    $25 = $0;
    _png_warning($25,63768);
    STACKTOP = sp;return;
   }
   $26 = $1;
   $27 = (($26) + 160|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = $2;
   $30 = $length;
   _memcpy(($28|0),($29|0),($30|0))|0;
   $31 = $3;
   $32 = $1;
   $33 = (($32) + 164|0);
   HEAP32[$33>>2] = $31;
   $34 = $4;
   $35 = $1;
   $36 = (($35) + 168|0);
   HEAP32[$36>>2] = $34;
   $37 = $5;
   $38 = $37&255;
   $39 = $1;
   $40 = (($39) + 180|0);
   HEAP8[$40>>0] = $38;
   $41 = $6;
   $42 = $41&255;
   $43 = $1;
   $44 = (($43) + 181|0);
   HEAP8[$44>>0] = $42;
   $45 = $7;
   $46 = (_strlen(($45|0))|0);
   $47 = (($46) + 1)|0;
   $length = $47;
   $48 = $0;
   $49 = $length;
   $50 = (_png_malloc_warn($48,$49)|0);
   $51 = $1;
   $52 = (($51) + 172|0);
   HEAP32[$52>>2] = $50;
   $53 = $1;
   $54 = (($53) + 172|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)==(0|0);
   if ($56) {
    $57 = $0;
    _png_warning($57,63808);
    STACKTOP = sp;return;
   }
   $58 = $1;
   $59 = (($58) + 172|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $7;
   $62 = $length;
   _memcpy(($60|0),($61|0),($62|0))|0;
   $63 = $0;
   $64 = $6;
   $65 = (($64) + 1)|0;
   $66 = $65<<2;
   $67 = (_png_malloc_warn($63,$66)|0);
   $68 = $1;
   $69 = (($68) + 176|0);
   HEAP32[$69>>2] = $67;
   $70 = $1;
   $71 = (($70) + 176|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = ($72|0)==(0|0);
   if ($73) {
    $74 = $0;
    _png_warning($74,63848);
    STACKTOP = sp;return;
   }
   $75 = $6;
   $76 = $1;
   $77 = (($76) + 176|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = (($78) + ($75<<2)|0);
   HEAP32[$79>>2] = 0;
   $i = 0;
   while(1) {
    $80 = $i;
    $81 = $6;
    $82 = ($80|0)<($81|0);
    if (!($82)) {
     label = 16;
     break;
    }
    $83 = $i;
    $84 = $8;
    $85 = (($84) + ($83<<2)|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = (_strlen(($86|0))|0);
    $88 = (($87) + 1)|0;
    $length = $88;
    $89 = $0;
    $90 = $length;
    $91 = (_png_malloc_warn($89,$90)|0);
    $92 = $i;
    $93 = $1;
    $94 = (($93) + 176|0);
    $95 = HEAP32[$94>>2]|0;
    $96 = (($95) + ($92<<2)|0);
    HEAP32[$96>>2] = $91;
    $97 = $i;
    $98 = $1;
    $99 = (($98) + 176|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = (($100) + ($97<<2)|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = ($102|0)==(0|0);
    if ($103) {
     label = 13;
     break;
    }
    $105 = $i;
    $106 = $1;
    $107 = (($106) + 176|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = (($108) + ($105<<2)|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = $i;
    $112 = $8;
    $113 = (($112) + ($111<<2)|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = $length;
    _memcpy(($110|0),($114|0),($115|0))|0;
    $116 = $i;
    $117 = (($116) + 1)|0;
    $i = $117;
   }
   if ((label|0) == 13) {
    $104 = $0;
    _png_warning($104,63888);
    STACKTOP = sp;return;
   }
   else if ((label|0) == 16) {
    $118 = $1;
    $119 = (($118) + 8|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = $120 | 1024;
    HEAP32[$119>>2] = $121;
    $122 = $1;
    $123 = (($122) + 184|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = $124 | 128;
    HEAP32[$123>>2] = $125;
    STACKTOP = sp;return;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_set_sCAL($png_ptr,$info_ptr,$unit,$width,$height) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $unit = $unit|0;
 $width = +$width;
 $height = +$height;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0.0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0.0, $4 = 0.0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $unit;
 $3 = $width;
 $4 = $height;
 $5 = $0;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $1;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = $2;
   $10 = $9&255;
   $11 = $1;
   $12 = (($11) + 220|0);
   HEAP8[$12>>0] = $10;
   $13 = $3;
   $14 = $1;
   $15 = (($14) + 224|0);
   HEAPF64[$15>>3] = $13;
   $16 = $4;
   $17 = $1;
   $18 = (($17) + 232|0);
   HEAPF64[$18>>3] = $16;
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 | 16384;
   HEAP32[$20>>2] = $22;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_pHYs($png_ptr,$info_ptr,$res_x,$res_y,$unit_type) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $res_x = $res_x|0;
 $res_y = $res_y|0;
 $unit_type = $unit_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $res_x;
 $3 = $res_y;
 $4 = $unit_type;
 $5 = $0;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $1;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = $2;
   $10 = $1;
   $11 = (($10) + 112|0);
   HEAP32[$11>>2] = $9;
   $12 = $3;
   $13 = $1;
   $14 = (($13) + 116|0);
   HEAP32[$14>>2] = $12;
   $15 = $4;
   $16 = $15&255;
   $17 = $1;
   $18 = (($17) + 120|0);
   HEAP8[$18>>0] = $16;
   $19 = $1;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 | 128;
   HEAP32[$20>>2] = $22;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_PLTE($png_ptr,$info_ptr,$palette,$num_palette) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $palette = $palette|0;
 $num_palette = $num_palette|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $palette;
 $3 = $num_palette;
 $4 = $0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = $1;
  $7 = ($6|0)==(0|0);
  if (!($7)) {
   $8 = $3;
   $9 = ($8|0)<(0);
   if ($9) {
    label = 6;
   } else {
    $10 = $3;
    $11 = ($10|0)>(256);
    if ($11) {
     label = 6;
    }
   }
   do {
    if ((label|0) == 6) {
     $12 = $1;
     $13 = (($12) + 25|0);
     $14 = HEAP8[$13>>0]|0;
     $15 = $14&255;
     $16 = ($15|0)==(3);
     if ($16) {
      $17 = $0;
      _png_error($17,63928);
      break;
     } else {
      $18 = $0;
      _png_warning($18,63928);
      STACKTOP = sp;return;
     }
    }
   } while(0);
   $19 = $0;
   $20 = $1;
   _png_free_data($19,$20,4096,0);
   $21 = $0;
   $22 = (_png_malloc($21,768)|0);
   $23 = $0;
   $24 = (($23) + 368|0);
   HEAP32[$24>>2] = $22;
   $25 = $0;
   $26 = (($25) + 368|0);
   $27 = HEAP32[$26>>2]|0;
   _memset(($27|0),0,768)|0;
   $28 = $0;
   $29 = (($28) + 368|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $2;
   $32 = $3;
   $33 = ($32*3)|0;
   _memcpy(($30|0),($31|0),($33|0))|0;
   $34 = $0;
   $35 = (($34) + 368|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = $1;
   $38 = (($37) + 16|0);
   HEAP32[$38>>2] = $36;
   $39 = $3;
   $40 = $39&65535;
   $41 = $0;
   $42 = (($41) + 372|0);
   HEAP16[$42>>1] = $40;
   $43 = $1;
   $44 = (($43) + 20|0);
   HEAP16[$44>>1] = $40;
   $45 = $1;
   $46 = (($45) + 184|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $47 | 4096;
   HEAP32[$46>>2] = $48;
   $49 = $1;
   $50 = (($49) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = $51 | 8;
   HEAP32[$50>>2] = $52;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_sBIT($png_ptr,$info_ptr,$sig_bit) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $sig_bit = $sig_bit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $sig_bit;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $1;
   $8 = (($7) + 68|0);
   $9 = $2;
   ;HEAP8[$8+0>>0]=HEAP8[$9+0>>0]|0;HEAP8[$8+1>>0]=HEAP8[$9+1>>0]|0;HEAP8[$8+2>>0]=HEAP8[$9+2>>0]|0;HEAP8[$8+3>>0]=HEAP8[$9+3>>0]|0;HEAP8[$8+4>>0]=HEAP8[$9+4>>0]|0;
   $10 = $1;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = $12 | 2;
   HEAP32[$11>>2] = $13;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_sRGB($png_ptr,$info_ptr,$intent) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $intent = $intent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $intent;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $2;
   $8 = $7&255;
   $9 = $1;
   $10 = (($9) + 44|0);
   HEAP8[$10>>0] = $8;
   $11 = $1;
   $12 = (($11) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = $13 | 2048;
   HEAP32[$12>>2] = $14;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_sRGB_gAMA_and_cHRM($png_ptr,$info_ptr,$intent) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $intent = $intent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0.0, $44 = 0.0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $blue_x = 0.0, $blue_y = 0.0, $file_gamma = 0.0, $green_x = 0.0, $green_y = 0.0, $int_blue_x = 0, $int_blue_y = 0, $int_file_gamma = 0, $int_green_x = 0, $int_green_y = 0, $int_red_x = 0, $int_red_y = 0, $int_white_x = 0, $int_white_y = 0, $red_x = 0.0;
 var $red_y = 0.0, $white_x = 0.0, $white_y = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $intent;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $0;
   $8 = $1;
   $9 = $2;
   _png_set_sRGB($7,$8,$9);
   $file_gamma = 0.454549998044967651367;
   $10 = $0;
   $11 = $1;
   $12 = $file_gamma;
   $13 = $12;
   _png_set_gAMA($10,$11,$13);
   $int_file_gamma = 45455;
   $14 = $0;
   $15 = $1;
   $16 = $int_file_gamma;
   _png_set_gAMA_fixed($14,$15,$16);
   $int_white_x = 31270;
   $int_white_y = 32900;
   $int_red_x = 64000;
   $int_red_y = 33000;
   $int_green_x = 30000;
   $int_green_y = 60000;
   $int_blue_x = 15000;
   $int_blue_y = 6000;
   $17 = $0;
   $18 = $1;
   $19 = $int_white_x;
   $20 = $int_white_y;
   $21 = $int_red_x;
   $22 = $int_red_y;
   $23 = $int_green_x;
   $24 = $int_green_y;
   $25 = $int_blue_x;
   $26 = $int_blue_y;
   _png_set_cHRM_fixed($17,$18,$19,$20,$21,$22,$23,$24,$25,$26);
   $white_x = 0.312700003385543823242;
   $white_y = 0.328999996185302734375;
   $red_x = 0.639999985694885253906;
   $red_y = 0.330000013113021850586;
   $green_x = 0.300000011920928955078;
   $green_y = 0.600000023841857910156;
   $blue_x = 0.150000005960464477539;
   $blue_y = 0.0599999986588954925537;
   $27 = $0;
   $28 = $1;
   $29 = $white_x;
   $30 = $29;
   $31 = $white_y;
   $32 = $31;
   $33 = $red_x;
   $34 = $33;
   $35 = $red_y;
   $36 = $35;
   $37 = $green_x;
   $38 = $37;
   $39 = $green_y;
   $40 = $39;
   $41 = $blue_x;
   $42 = $41;
   $43 = $blue_y;
   $44 = $43;
   _png_set_cHRM($27,$28,$30,$32,$34,$36,$38,$40,$42,$44);
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_iCCP($png_ptr,$info_ptr,$name,$compression_type,$profile,$proflen) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $name = $name|0;
 $compression_type = $compression_type|0;
 $profile = $profile|0;
 $proflen = $proflen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $length = 0, $new_iccp_name = 0, $new_iccp_profile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $name;
 $3 = $compression_type;
 $4 = $profile;
 $5 = $proflen;
 $6 = $0;
 $7 = ($6|0)==(0|0);
 if (!($7)) {
  $8 = $1;
  $9 = ($8|0)==(0|0);
  if (!($9)) {
   $10 = $2;
   $11 = ($10|0)==(0|0);
   if (!($11)) {
    $12 = $4;
    $13 = ($12|0)==(0|0);
    if (!($13)) {
     $14 = $2;
     $15 = (_strlen(($14|0))|0);
     $16 = (($15) + 1)|0;
     $length = $16;
     $17 = $0;
     $18 = $length;
     $19 = (_png_malloc_warn($17,$18)|0);
     $new_iccp_name = $19;
     $20 = $new_iccp_name;
     $21 = ($20|0)==(0|0);
     if ($21) {
      $22 = $0;
      _png_warning($22,63952);
      STACKTOP = sp;return;
     }
     $23 = $new_iccp_name;
     $24 = $2;
     $25 = $length;
     _memcpy(($23|0),($24|0),($25|0))|0;
     $26 = $0;
     $27 = $5;
     $28 = (_png_malloc_warn($26,$27)|0);
     $new_iccp_profile = $28;
     $29 = $new_iccp_profile;
     $30 = ($29|0)==(0|0);
     if ($30) {
      $31 = $0;
      $32 = $new_iccp_name;
      _png_free($31,$32);
      $33 = $0;
      _png_warning($33,64000);
      STACKTOP = sp;return;
     } else {
      $34 = $new_iccp_profile;
      $35 = $4;
      $36 = $5;
      _memcpy(($34|0),($35|0),($36|0))|0;
      $37 = $0;
      $38 = $1;
      _png_free_data($37,$38,16,0);
      $39 = $5;
      $40 = $1;
      $41 = (($40) + 204|0);
      HEAP32[$41>>2] = $39;
      $42 = $new_iccp_name;
      $43 = $1;
      $44 = (($43) + 196|0);
      HEAP32[$44>>2] = $42;
      $45 = $new_iccp_profile;
      $46 = $1;
      $47 = (($46) + 200|0);
      HEAP32[$47>>2] = $45;
      $48 = $3;
      $49 = $48&255;
      $50 = $1;
      $51 = (($50) + 208|0);
      HEAP8[$51>>0] = $49;
      $52 = $1;
      $53 = (($52) + 184|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = $54 | 16;
      HEAP32[$53>>2] = $55;
      $56 = $1;
      $57 = (($56) + 8|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = $58 | 4096;
      HEAP32[$57>>2] = $59;
      STACKTOP = sp;return;
     }
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_set_text_2($png_ptr,$info_ptr,$text_ptr,$num_text) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $text_ptr = $text_ptr|0;
 $num_text = $num_text|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $key_len = 0, $lang_key_len = 0, $lang_len = 0, $old_max = 0, $old_text = 0, $text_length = 0, $textp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $1 = $png_ptr;
 $2 = $info_ptr;
 $3 = $text_ptr;
 $4 = $num_text;
 $5 = $1;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $2;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = $4;
   $10 = ($9|0)==(0);
   if (!($10)) {
    $11 = $2;
    $12 = (($11) + 48|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = $4;
    $15 = (($13) + ($14))|0;
    $16 = $2;
    $17 = (($16) + 52|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($15|0)>($18|0);
    if ($19) {
     $20 = $2;
     $21 = (($20) + 56|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = ($22|0)!=(0|0);
     do {
      if ($23) {
       $24 = $2;
       $25 = (($24) + 52|0);
       $26 = HEAP32[$25>>2]|0;
       $old_max = $26;
       $27 = $2;
       $28 = (($27) + 48|0);
       $29 = HEAP32[$28>>2]|0;
       $30 = $4;
       $31 = (($29) + ($30))|0;
       $32 = (($31) + 8)|0;
       $33 = $2;
       $34 = (($33) + 52|0);
       HEAP32[$34>>2] = $32;
       $35 = $2;
       $36 = (($35) + 56|0);
       $37 = HEAP32[$36>>2]|0;
       $old_text = $37;
       $38 = $1;
       $39 = $2;
       $40 = (($39) + 52|0);
       $41 = HEAP32[$40>>2]|0;
       $42 = $41<<4;
       $43 = (_png_malloc_warn($38,$42)|0);
       $44 = $2;
       $45 = (($44) + 56|0);
       HEAP32[$45>>2] = $43;
       $46 = $2;
       $47 = (($46) + 56|0);
       $48 = HEAP32[$47>>2]|0;
       $49 = ($48|0)==(0|0);
       if (!($49)) {
        $52 = $2;
        $53 = (($52) + 56|0);
        $54 = HEAP32[$53>>2]|0;
        $55 = $old_text;
        $56 = $old_max;
        $57 = $56<<4;
        _memcpy(($54|0),($55|0),($57|0))|0;
        $58 = $1;
        $59 = $old_text;
        _png_free($58,$59);
        break;
       }
       $50 = $1;
       $51 = $old_text;
       _png_free($50,$51);
       $0 = 1;
       $199 = $0;
       STACKTOP = sp;return ($199|0);
      } else {
       $60 = $4;
       $61 = (($60) + 8)|0;
       $62 = $2;
       $63 = (($62) + 52|0);
       HEAP32[$63>>2] = $61;
       $64 = $2;
       $65 = (($64) + 48|0);
       HEAP32[$65>>2] = 0;
       $66 = $1;
       $67 = $2;
       $68 = (($67) + 52|0);
       $69 = HEAP32[$68>>2]|0;
       $70 = $69<<4;
       $71 = (_png_malloc_warn($66,$70)|0);
       $72 = $2;
       $73 = (($72) + 56|0);
       HEAP32[$73>>2] = $71;
       $74 = $2;
       $75 = (($74) + 56|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = ($76|0)==(0|0);
       if (!($77)) {
        $78 = $2;
        $79 = (($78) + 184|0);
        $80 = HEAP32[$79>>2]|0;
        $81 = $80 | 16384;
        HEAP32[$79>>2] = $81;
        break;
       }
       $0 = 1;
       $199 = $0;
       STACKTOP = sp;return ($199|0);
      }
     } while(0);
    }
    $i = 0;
    L20: while(1) {
     $82 = $i;
     $83 = $4;
     $84 = ($82|0)<($83|0);
     if (!($84)) {
      label = 31;
      break;
     }
     $85 = $2;
     $86 = (($85) + 48|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = $2;
     $89 = (($88) + 56|0);
     $90 = HEAP32[$89>>2]|0;
     $91 = (($90) + ($87<<4)|0);
     $textp = $91;
     $92 = $i;
     $93 = $3;
     $94 = (($93) + ($92<<4)|0);
     $95 = (($94) + 4|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = ($96|0)==(0|0);
     do {
      if ($97) {
      } else {
       $98 = $i;
       $99 = $3;
       $100 = (($99) + ($98<<4)|0);
       $101 = (($100) + 4|0);
       $102 = HEAP32[$101>>2]|0;
       $103 = (_strlen(($102|0))|0);
       $key_len = $103;
       $104 = $i;
       $105 = $3;
       $106 = (($105) + ($104<<4)|0);
       $107 = HEAP32[$106>>2]|0;
       $108 = ($107|0)<=(0);
       if (!($108)) {
        $109 = $1;
        _png_warning($109,64048);
        break;
       }
       $lang_len = 0;
       $lang_key_len = 0;
       $110 = $i;
       $111 = $3;
       $112 = (($111) + ($110<<4)|0);
       $113 = (($112) + 8|0);
       $114 = HEAP32[$113>>2]|0;
       $115 = ($114|0)==(0|0);
       if ($115) {
        label = 23;
       } else {
        $116 = $i;
        $117 = $3;
        $118 = (($117) + ($116<<4)|0);
        $119 = (($118) + 8|0);
        $120 = HEAP32[$119>>2]|0;
        $121 = HEAP8[$120>>0]|0;
        $122 = $121 << 24 >> 24;
        $123 = ($122|0)==(0);
        if ($123) {
         label = 23;
        } else {
         $125 = $i;
         $126 = $3;
         $127 = (($126) + ($125<<4)|0);
         $128 = (($127) + 8|0);
         $129 = HEAP32[$128>>2]|0;
         $130 = (_strlen(($129|0))|0);
         $text_length = $130;
         $131 = $i;
         $132 = $3;
         $133 = (($132) + ($131<<4)|0);
         $134 = HEAP32[$133>>2]|0;
         $135 = $textp;
         HEAP32[$135>>2] = $134;
        }
       }
       if ((label|0) == 23) {
        label = 0;
        $text_length = 0;
        $124 = $textp;
        HEAP32[$124>>2] = -1;
       }
       $136 = $1;
       $137 = $key_len;
       $138 = $text_length;
       $139 = (($137) + ($138))|0;
       $140 = $lang_len;
       $141 = (($139) + ($140))|0;
       $142 = $lang_key_len;
       $143 = (($141) + ($142))|0;
       $144 = (($143) + 4)|0;
       $145 = (_png_malloc_warn($136,$144)|0);
       $146 = $textp;
       $147 = (($146) + 4|0);
       HEAP32[$147>>2] = $145;
       $148 = $textp;
       $149 = (($148) + 4|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = ($150|0)==(0|0);
       if ($151) {
        label = 26;
        break L20;
       }
       $152 = $textp;
       $153 = (($152) + 4|0);
       $154 = HEAP32[$153>>2]|0;
       $155 = $i;
       $156 = $3;
       $157 = (($156) + ($155<<4)|0);
       $158 = (($157) + 4|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = $key_len;
       _memcpy(($154|0),($159|0),($160|0))|0;
       $161 = $textp;
       $162 = (($161) + 4|0);
       $163 = HEAP32[$162>>2]|0;
       $164 = $key_len;
       $165 = (($163) + ($164)|0);
       HEAP8[$165>>0] = 0;
       $166 = $textp;
       $167 = (($166) + 4|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = $key_len;
       $170 = (($168) + ($169)|0);
       $171 = (($170) + 1|0);
       $172 = $textp;
       $173 = (($172) + 8|0);
       HEAP32[$173>>2] = $171;
       $174 = $text_length;
       $175 = ($174|0)!=(0);
       if ($175) {
        $176 = $textp;
        $177 = (($176) + 8|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = $i;
        $180 = $3;
        $181 = (($180) + ($179<<4)|0);
        $182 = (($181) + 8|0);
        $183 = HEAP32[$182>>2]|0;
        $184 = $text_length;
        _memcpy(($178|0),($183|0),($184|0))|0;
       }
       $185 = $textp;
       $186 = (($185) + 8|0);
       $187 = HEAP32[$186>>2]|0;
       $188 = $text_length;
       $189 = (($187) + ($188)|0);
       HEAP8[$189>>0] = 0;
       $190 = $text_length;
       $191 = $textp;
       $192 = (($191) + 12|0);
       HEAP32[$192>>2] = $190;
       $193 = $2;
       $194 = (($193) + 48|0);
       $195 = HEAP32[$194>>2]|0;
       $196 = (($195) + 1)|0;
       HEAP32[$194>>2] = $196;
      }
     } while(0);
     $197 = $i;
     $198 = (($197) + 1)|0;
     $i = $198;
    }
    if ((label|0) == 26) {
     $0 = 1;
     $199 = $0;
     STACKTOP = sp;return ($199|0);
    }
    else if ((label|0) == 31) {
     $0 = 0;
     $199 = $0;
     STACKTOP = sp;return ($199|0);
    }
   }
  }
 }
 $0 = 0;
 $199 = $0;
 STACKTOP = sp;return ($199|0);
}
function _png_set_tIME($png_ptr,$info_ptr,$mod_time) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $mod_time = $mod_time|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $mod_time;
 $3 = $0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = $1;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = $0;
   $8 = (($7) + 196|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = $9 & 512;
   $11 = ($10|0)!=(0);
   if (!($11)) {
    $12 = $1;
    $13 = (($12) + 60|0);
    $14 = $2;
    ;HEAP16[$13+0>>1]=HEAP16[$14+0>>1]|0;HEAP16[$13+2>>1]=HEAP16[$14+2>>1]|0;HEAP16[$13+4>>1]=HEAP16[$14+4>>1]|0;HEAP16[$13+6>>1]=HEAP16[$14+6>>1]|0;
    $15 = $1;
    $16 = (($15) + 8|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = $17 | 512;
    HEAP32[$16>>2] = $18;
    STACKTOP = sp;return;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_set_tRNS($png_ptr,$info_ptr,$trans,$num_trans,$trans_values) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $trans = $trans|0;
 $num_trans = $num_trans|0;
 $trans_values = $trans_values|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $trans;
 $3 = $num_trans;
 $4 = $trans_values;
 $5 = $0;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $1;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = $2;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    $11 = $0;
    $12 = $1;
    _png_free_data($11,$12,8192,0);
    $13 = $0;
    $14 = (_png_malloc($13,256)|0);
    $15 = $1;
    $16 = (($15) + 76|0);
    HEAP32[$16>>2] = $14;
    $17 = $0;
    $18 = (($17) + 484|0);
    HEAP32[$18>>2] = $14;
    $19 = $3;
    $20 = ($19|0)<=(256);
    if ($20) {
     $21 = $1;
     $22 = (($21) + 76|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = $2;
     $25 = $3;
     _memcpy(($23|0),($24|0),($25|0))|0;
    }
    $26 = $1;
    $27 = (($26) + 184|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = $28 | 8192;
    HEAP32[$27>>2] = $29;
   }
   $30 = $4;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = $1;
    $33 = (($32) + 80|0);
    $34 = $4;
    ;HEAP16[$33+0>>1]=HEAP16[$34+0>>1]|0;HEAP16[$33+2>>1]=HEAP16[$34+2>>1]|0;HEAP16[$33+4>>1]=HEAP16[$34+4>>1]|0;HEAP16[$33+6>>1]=HEAP16[$34+6>>1]|0;HEAP16[$33+8>>1]=HEAP16[$34+8>>1]|0;
    $35 = $3;
    $36 = ($35|0)==(0);
    if ($36) {
     $3 = 1;
    }
   }
   $37 = $3;
   $38 = $37&65535;
   $39 = $1;
   $40 = (($39) + 22|0);
   HEAP16[$40>>1] = $38;
   $41 = $1;
   $42 = (($41) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = $43 | 16;
   HEAP32[$42>>2] = $44;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_sPLT($png_ptr,$info_ptr,$entries,$nentries) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $entries = $entries|0;
 $nentries = $nentries|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $from = 0, $i = 0;
 var $length = 0, $np = 0, $to = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $entries;
 $3 = $nentries;
 $4 = $0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = $1;
  $7 = ($6|0)==(0|0);
  if (!($7)) {
   $8 = $0;
   $9 = $1;
   $10 = (($9) + 216|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = $3;
   $13 = (($11) + ($12))|0;
   $14 = $13<<4;
   $15 = (_png_malloc_warn($8,$14)|0);
   $np = $15;
   $16 = $np;
   $17 = ($16|0)==(0|0);
   if ($17) {
    $18 = $0;
    _png_warning($18,64080);
    STACKTOP = sp;return;
   }
   $19 = $np;
   $20 = $1;
   $21 = (($20) + 212|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = $1;
   $24 = (($23) + 216|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25<<4;
   _memcpy(($19|0),($22|0),($26|0))|0;
   $27 = $0;
   $28 = $1;
   $29 = (($28) + 212|0);
   $30 = HEAP32[$29>>2]|0;
   _png_free($27,$30);
   $31 = $1;
   $32 = (($31) + 212|0);
   HEAP32[$32>>2] = 0;
   $i = 0;
   while(1) {
    $33 = $i;
    $34 = $3;
    $35 = ($33|0)<($34|0);
    if (!($35)) {
     break;
    }
    $36 = $np;
    $37 = $1;
    $38 = (($37) + 216|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($36) + ($39<<4)|0);
    $41 = $i;
    $42 = (($40) + ($41<<4)|0);
    $to = $42;
    $43 = $2;
    $44 = $i;
    $45 = (($43) + ($44<<4)|0);
    $from = $45;
    $46 = $from;
    $47 = HEAP32[$46>>2]|0;
    $48 = (_strlen(($47|0))|0);
    $49 = (($48) + 1)|0;
    $length = $49;
    $50 = $0;
    $51 = $length;
    $52 = (_png_malloc_warn($50,$51)|0);
    $53 = $to;
    HEAP32[$53>>2] = $52;
    $54 = $to;
    $55 = HEAP32[$54>>2]|0;
    $56 = ($55|0)==(0|0);
    if ($56) {
     $57 = $0;
     _png_warning($57,64112);
    }
    $58 = $to;
    $59 = HEAP32[$58>>2]|0;
    $60 = $from;
    $61 = HEAP32[$60>>2]|0;
    $62 = $length;
    _memcpy(($59|0),($61|0),($62|0))|0;
    $63 = $0;
    $64 = $from;
    $65 = (($64) + 12|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = ($66*10)|0;
    $68 = (_png_malloc_warn($63,$67)|0);
    $69 = $to;
    $70 = (($69) + 8|0);
    HEAP32[$70>>2] = $68;
    $71 = $to;
    $72 = (($71) + 8|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = $from;
    $75 = (($74) + 8|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = $from;
    $78 = (($77) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79*10)|0;
    _memcpy(($73|0),($76|0),($80|0))|0;
    $81 = $to;
    $82 = (($81) + 8|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ($83|0)==(0|0);
    if ($84) {
     $85 = $0;
     _png_warning($85,64112);
     $86 = $0;
     $87 = $to;
     $88 = HEAP32[$87>>2]|0;
     _png_free($86,$88);
     $89 = $to;
     HEAP32[$89>>2] = 0;
    }
    $90 = $from;
    $91 = (($90) + 12|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = $to;
    $94 = (($93) + 12|0);
    HEAP32[$94>>2] = $92;
    $95 = $from;
    $96 = (($95) + 4|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $to;
    $99 = (($98) + 4|0);
    HEAP8[$99>>0] = $97;
    $100 = $i;
    $101 = (($100) + 1)|0;
    $i = $101;
   }
   $102 = $np;
   $103 = $1;
   $104 = (($103) + 212|0);
   HEAP32[$104>>2] = $102;
   $105 = $3;
   $106 = $1;
   $107 = (($106) + 216|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($108) + ($105))|0;
   HEAP32[$107>>2] = $109;
   $110 = $1;
   $111 = (($110) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = $112 | 8192;
   HEAP32[$111>>2] = $113;
   $114 = $1;
   $115 = (($114) + 184|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = $116 | 32;
   HEAP32[$115>>2] = $117;
   STACKTOP = sp;return;
  }
 }
 STACKTOP = sp;return;
}
function _png_set_unknown_chunks($png_ptr,$info_ptr,$unknowns,$num_unknowns) {
 $png_ptr = $png_ptr|0;
 $info_ptr = $info_ptr|0;
 $unknowns = $unknowns|0;
 $num_unknowns = $num_unknowns|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $from = 0;
 var $i = 0, $np = 0, $to = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $png_ptr;
 $1 = $info_ptr;
 $2 = $unknowns;
 $3 = $num_unknowns;
 $4 = $0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = $1;
  $7 = ($6|0)==(0|0);
  if (!($7)) {
   $8 = $3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $0;
    $11 = $1;
    $12 = (($11) + 192|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = $3;
    $15 = (($13) + ($14))|0;
    $16 = ($15*20)|0;
    $17 = (_png_malloc_warn($10,$16)|0);
    $np = $17;
    $18 = $np;
    $19 = ($18|0)==(0|0);
    if ($19) {
     $20 = $0;
     _png_warning($20,64160);
     STACKTOP = sp;return;
    }
    $21 = $np;
    $22 = $1;
    $23 = (($22) + 188|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $1;
    $26 = (($25) + 192|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27*20)|0;
    _memcpy(($21|0),($24|0),($28|0))|0;
    $29 = $0;
    $30 = $1;
    $31 = (($30) + 188|0);
    $32 = HEAP32[$31>>2]|0;
    _png_free($29,$32);
    $33 = $1;
    $34 = (($33) + 188|0);
    HEAP32[$34>>2] = 0;
    $i = 0;
    while(1) {
     $35 = $i;
     $36 = $3;
     $37 = ($35|0)<($36|0);
     if (!($37)) {
      break;
     }
     $38 = $np;
     $39 = $1;
     $40 = (($39) + 192|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($38) + (($41*20)|0)|0);
     $43 = $i;
     $44 = (($42) + (($43*20)|0)|0);
     $to = $44;
     $45 = $2;
     $46 = $i;
     $47 = (($45) + (($46*20)|0)|0);
     $from = $47;
     $48 = $to;
     $49 = $from;
     ;HEAP8[$48+0>>0]=HEAP8[$49+0>>0]|0;HEAP8[$48+1>>0]=HEAP8[$49+1>>0]|0;HEAP8[$48+2>>0]=HEAP8[$49+2>>0]|0;HEAP8[$48+3>>0]=HEAP8[$49+3>>0]|0;HEAP8[$48+4>>0]=HEAP8[$49+4>>0]|0;
     $50 = $to;
     $51 = (($50) + 4|0);
     HEAP8[$51>>0] = 0;
     $52 = $0;
     $53 = $from;
     $54 = (($53) + 12|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (_png_malloc_warn($52,$55)|0);
     $57 = $to;
     $58 = (($57) + 8|0);
     HEAP32[$58>>2] = $56;
     $59 = $to;
     $60 = (($59) + 8|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = ($61|0)==(0|0);
     if ($62) {
      $63 = $0;
      _png_warning($63,64160);
     } else {
      $64 = $to;
      $65 = (($64) + 8|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = $from;
      $68 = (($67) + 8|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = $from;
      $71 = (($70) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      _memcpy(($66|0),($69|0),($72|0))|0;
      $73 = $from;
      $74 = (($73) + 12|0);
      $75 = HEAP32[$74>>2]|0;
      $76 = $to;
      $77 = (($76) + 12|0);
      HEAP32[$77>>2] = $75;
      $78 = $0;
      $79 = (($78) + 196|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = $80 & 255;
      $82 = $81&255;
      $83 = $to;
      $84 = (($83) + 16|0);
      HEAP8[$84>>0] = $82;
     }
     $85 = $i;
     $86 = (($85) + 1)|0;
     $i = $86;
    }
    $87 = $np;
    $88 = $1;
    $89 = (($88) + 188|0);
    HEAP32[$89>>2] = $87;
    $90 = $3;
    $91 = $1;
    $92 = (($91) + 192|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = (($93) + ($90))|0;
    HEAP32[$92>>2] = $94;
    $95 = $1;
    $96 = (($95) + 184|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = $97 | 512;
    HEAP32[$96>>2] = $98;
    STACKTOP = sp;return;
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_set_interlace_handling($png_ptr) {
 $png_ptr = $png_ptr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $png_ptr;
 $2 = $1;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = $1;
  $5 = (($4) + 383|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)!=(0);
  if ($8) {
   $9 = $1;
   $10 = (($9) + 204|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = $11 | 2;
   HEAP32[$10>>2] = $12;
   $0 = 7;
   $13 = $0;
   STACKTOP = sp;return ($13|0);
  }
 }
 $0 = 1;
 $13 = $0;
 STACKTOP = sp;return ($13|0);
}
function _png_do_invert($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $i = 0, $i2 = 0, $i5 = 0, $istop = 0, $istop3 = 0, $istop6 = 0, $rp = 0, $rp1 = 0, $rp4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 8|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = $1;
  $rp = $7;
  $8 = $0;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $istop = $10;
  $i = 0;
  while(1) {
   $11 = $i;
   $12 = $istop;
   $13 = ($11>>>0)<($12>>>0);
   if (!($13)) {
    break;
   }
   $14 = $rp;
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = $16 ^ -1;
   $18 = $17&255;
   $19 = $rp;
   HEAP8[$19>>0] = $18;
   $20 = $rp;
   $21 = (($20) + 1|0);
   $rp = $21;
   $22 = $i;
   $23 = (($22) + 1)|0;
   $i = $23;
  }
  STACKTOP = sp;return;
 }
 $24 = $0;
 $25 = (($24) + 8|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)==(4);
 if ($28) {
  $29 = $0;
  $30 = (($29) + 9|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = ($32|0)==(8);
  if ($33) {
   $34 = $1;
   $rp1 = $34;
   $35 = $0;
   $36 = (($35) + 4|0);
   $37 = HEAP32[$36>>2]|0;
   $istop3 = $37;
   $i2 = 0;
   while(1) {
    $38 = $i2;
    $39 = $istop3;
    $40 = ($38>>>0)<($39>>>0);
    if (!($40)) {
     break;
    }
    $41 = $rp1;
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $43 ^ -1;
    $45 = $44&255;
    $46 = $rp1;
    HEAP8[$46>>0] = $45;
    $47 = $rp1;
    $48 = (($47) + 2|0);
    $rp1 = $48;
    $49 = $i2;
    $50 = (($49) + 2)|0;
    $i2 = $50;
   }
  } else {
   label = 14;
  }
 } else {
  label = 14;
 }
 if ((label|0) == 14) {
  $51 = $0;
  $52 = (($51) + 8|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = ($54|0)==(4);
  if ($55) {
   $56 = $0;
   $57 = (($56) + 9|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = ($59|0)==(16);
   if ($60) {
    $61 = $1;
    $rp4 = $61;
    $62 = $0;
    $63 = (($62) + 4|0);
    $64 = HEAP32[$63>>2]|0;
    $istop6 = $64;
    $i5 = 0;
    while(1) {
     $65 = $i5;
     $66 = $istop6;
     $67 = ($65>>>0)<($66>>>0);
     if (!($67)) {
      break;
     }
     $68 = $rp4;
     $69 = HEAP8[$68>>0]|0;
     $70 = $69&255;
     $71 = $70 ^ -1;
     $72 = $71&255;
     $73 = $rp4;
     HEAP8[$73>>0] = $72;
     $74 = $rp4;
     $75 = (($74) + 1|0);
     $76 = HEAP8[$75>>0]|0;
     $77 = $76&255;
     $78 = $77 ^ -1;
     $79 = $78&255;
     $80 = $rp4;
     $81 = (($80) + 1|0);
     HEAP8[$81>>0] = $79;
     $82 = $rp4;
     $83 = (($82) + 4|0);
     $rp4 = $83;
     $84 = $i5;
     $85 = (($84) + 4)|0;
     $i5 = $85;
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _png_do_swap($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $istop = 0, $rp = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 9|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(16);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $1;
 $rp = $7;
 $8 = $0;
 $9 = HEAP32[$8>>2]|0;
 $10 = $0;
 $11 = (($10) + 10|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = Math_imul($9, $13)|0;
 $istop = $14;
 $i = 0;
 while(1) {
  $15 = $i;
  $16 = $istop;
  $17 = ($15>>>0)<($16>>>0);
  if (!($17)) {
   break;
  }
  $18 = $rp;
  $19 = HEAP8[$18>>0]|0;
  $t = $19;
  $20 = $rp;
  $21 = (($20) + 1|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $rp;
  HEAP8[$23>>0] = $22;
  $24 = $t;
  $25 = $rp;
  $26 = (($25) + 1|0);
  HEAP8[$26>>0] = $24;
  $27 = $i;
  $28 = (($27) + 1)|0;
  $i = $28;
  $29 = $rp;
  $30 = (($29) + 2|0);
  $rp = $30;
 }
 STACKTOP = sp;return;
}
function _png_do_packswap($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $end = 0, $rp = 0, $table = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 9|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)<(8);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $1;
 $8 = $0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7) + ($10)|0);
 $end = $11;
 $12 = $0;
 $13 = (($12) + 9|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(1);
 if ($16) {
  $table = 64208;
 } else {
  $17 = $0;
  $18 = (($17) + 9|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)==(2);
  do {
   if ($21) {
    $table = 64464;
   } else {
    $22 = $0;
    $23 = (($22) + 9|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24&255;
    $26 = ($25|0)==(4);
    if ($26) {
     $table = 64720;
     break;
    } else {
     STACKTOP = sp;return;
    }
   }
  } while(0);
 }
 $27 = $1;
 $rp = $27;
 while(1) {
  $28 = $rp;
  $29 = $end;
  $30 = ($28>>>0)<($29>>>0);
  if (!($30)) {
   break;
  }
  $31 = $rp;
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = $table;
  $35 = (($34) + ($33)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $rp;
  HEAP8[$37>>0] = $36;
  $38 = $rp;
  $39 = (($38) + 1|0);
  $rp = $39;
 }
 STACKTOP = sp;return;
}
function _png_do_strip_filler($row_info,$row,$flags) {
 $row_info = $row_info|0;
 $row = $row|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dp = 0, $i = 0, $row_width = 0, $sp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $flags;
 $3 = $1;
 $sp = $3;
 $4 = $1;
 $dp = $4;
 $5 = $0;
 $6 = HEAP32[$5>>2]|0;
 $row_width = $6;
 $7 = $0;
 $8 = (($7) + 8|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)==(2);
 if ($11) {
  label = 4;
 } else {
  $12 = $0;
  $13 = (($12) + 8|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(6);
  if ($16) {
   $17 = $2;
   $18 = $17 & 4194304;
   $19 = ($18|0)!=(0);
   if ($19) {
    label = 4;
   } else {
    label = 31;
   }
  } else {
   label = 31;
  }
 }
 if ((label|0) == 4) {
  $20 = $0;
  $21 = (($20) + 10|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = ($23|0)==(4);
  if ($24) {
   $25 = $0;
   $26 = (($25) + 9|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27&255;
   $29 = ($28|0)==(8);
   if ($29) {
    $30 = $2;
    $31 = $30 & 128;
    $32 = ($31|0)!=(0);
    if ($32) {
     $33 = $dp;
     $34 = (($33) + 3|0);
     $dp = $34;
     $35 = $sp;
     $36 = (($35) + 4|0);
     $sp = $36;
     $i = 1;
     while(1) {
      $37 = $i;
      $38 = $row_width;
      $39 = ($37>>>0)<($38>>>0);
      if (!($39)) {
       break;
      }
      $40 = $sp;
      $41 = (($40) + 1|0);
      $sp = $41;
      $42 = HEAP8[$40>>0]|0;
      $43 = $dp;
      $44 = (($43) + 1|0);
      $dp = $44;
      HEAP8[$43>>0] = $42;
      $45 = $sp;
      $46 = (($45) + 1|0);
      $sp = $46;
      $47 = HEAP8[$45>>0]|0;
      $48 = $dp;
      $49 = (($48) + 1|0);
      $dp = $49;
      HEAP8[$48>>0] = $47;
      $50 = $sp;
      $51 = (($50) + 1|0);
      $sp = $51;
      $52 = HEAP8[$50>>0]|0;
      $53 = $dp;
      $54 = (($53) + 1|0);
      $dp = $54;
      HEAP8[$53>>0] = $52;
      $55 = $sp;
      $56 = (($55) + 1|0);
      $sp = $56;
      $57 = $i;
      $58 = (($57) + 1)|0;
      $i = $58;
     }
    } else {
     $i = 0;
     while(1) {
      $59 = $i;
      $60 = $row_width;
      $61 = ($59>>>0)<($60>>>0);
      if (!($61)) {
       break;
      }
      $62 = $sp;
      $63 = (($62) + 1|0);
      $sp = $63;
      $64 = $sp;
      $65 = (($64) + 1|0);
      $sp = $65;
      $66 = HEAP8[$64>>0]|0;
      $67 = $dp;
      $68 = (($67) + 1|0);
      $dp = $68;
      HEAP8[$67>>0] = $66;
      $69 = $sp;
      $70 = (($69) + 1|0);
      $sp = $70;
      $71 = HEAP8[$69>>0]|0;
      $72 = $dp;
      $73 = (($72) + 1|0);
      $dp = $73;
      HEAP8[$72>>0] = $71;
      $74 = $sp;
      $75 = (($74) + 1|0);
      $sp = $75;
      $76 = HEAP8[$74>>0]|0;
      $77 = $dp;
      $78 = (($77) + 1|0);
      $dp = $78;
      HEAP8[$77>>0] = $76;
      $79 = $i;
      $80 = (($79) + 1)|0;
      $i = $80;
     }
    }
    $81 = $0;
    $82 = (($81) + 11|0);
    HEAP8[$82>>0] = 24;
    $83 = $row_width;
    $84 = ($83*3)|0;
    $85 = $0;
    $86 = (($85) + 4|0);
    HEAP32[$86>>2] = $84;
   } else {
    $87 = $2;
    $88 = $87 & 128;
    $89 = ($88|0)!=(0);
    if ($89) {
     $90 = $sp;
     $91 = (($90) + 8|0);
     $sp = $91;
     $92 = $dp;
     $93 = (($92) + 6|0);
     $dp = $93;
     $i = 1;
     while(1) {
      $94 = $i;
      $95 = $row_width;
      $96 = ($94>>>0)<($95>>>0);
      if (!($96)) {
       break;
      }
      $97 = $sp;
      $98 = (($97) + 1|0);
      $sp = $98;
      $99 = HEAP8[$97>>0]|0;
      $100 = $dp;
      $101 = (($100) + 1|0);
      $dp = $101;
      HEAP8[$100>>0] = $99;
      $102 = $sp;
      $103 = (($102) + 1|0);
      $sp = $103;
      $104 = HEAP8[$102>>0]|0;
      $105 = $dp;
      $106 = (($105) + 1|0);
      $dp = $106;
      HEAP8[$105>>0] = $104;
      $107 = $sp;
      $108 = (($107) + 1|0);
      $sp = $108;
      $109 = HEAP8[$107>>0]|0;
      $110 = $dp;
      $111 = (($110) + 1|0);
      $dp = $111;
      HEAP8[$110>>0] = $109;
      $112 = $sp;
      $113 = (($112) + 1|0);
      $sp = $113;
      $114 = HEAP8[$112>>0]|0;
      $115 = $dp;
      $116 = (($115) + 1|0);
      $dp = $116;
      HEAP8[$115>>0] = $114;
      $117 = $sp;
      $118 = (($117) + 1|0);
      $sp = $118;
      $119 = HEAP8[$117>>0]|0;
      $120 = $dp;
      $121 = (($120) + 1|0);
      $dp = $121;
      HEAP8[$120>>0] = $119;
      $122 = $sp;
      $123 = (($122) + 1|0);
      $sp = $123;
      $124 = HEAP8[$122>>0]|0;
      $125 = $dp;
      $126 = (($125) + 1|0);
      $dp = $126;
      HEAP8[$125>>0] = $124;
      $127 = $sp;
      $128 = (($127) + 2|0);
      $sp = $128;
      $129 = $i;
      $130 = (($129) + 1)|0;
      $i = $130;
     }
    } else {
     $i = 0;
     while(1) {
      $131 = $i;
      $132 = $row_width;
      $133 = ($131>>>0)<($132>>>0);
      if (!($133)) {
       break;
      }
      $134 = $sp;
      $135 = (($134) + 2|0);
      $sp = $135;
      $136 = $sp;
      $137 = (($136) + 1|0);
      $sp = $137;
      $138 = HEAP8[$136>>0]|0;
      $139 = $dp;
      $140 = (($139) + 1|0);
      $dp = $140;
      HEAP8[$139>>0] = $138;
      $141 = $sp;
      $142 = (($141) + 1|0);
      $sp = $142;
      $143 = HEAP8[$141>>0]|0;
      $144 = $dp;
      $145 = (($144) + 1|0);
      $dp = $145;
      HEAP8[$144>>0] = $143;
      $146 = $sp;
      $147 = (($146) + 1|0);
      $sp = $147;
      $148 = HEAP8[$146>>0]|0;
      $149 = $dp;
      $150 = (($149) + 1|0);
      $dp = $150;
      HEAP8[$149>>0] = $148;
      $151 = $sp;
      $152 = (($151) + 1|0);
      $sp = $152;
      $153 = HEAP8[$151>>0]|0;
      $154 = $dp;
      $155 = (($154) + 1|0);
      $dp = $155;
      HEAP8[$154>>0] = $153;
      $156 = $sp;
      $157 = (($156) + 1|0);
      $sp = $157;
      $158 = HEAP8[$156>>0]|0;
      $159 = $dp;
      $160 = (($159) + 1|0);
      $dp = $160;
      HEAP8[$159>>0] = $158;
      $161 = $sp;
      $162 = (($161) + 1|0);
      $sp = $162;
      $163 = HEAP8[$161>>0]|0;
      $164 = $dp;
      $165 = (($164) + 1|0);
      $dp = $165;
      HEAP8[$164>>0] = $163;
      $166 = $i;
      $167 = (($166) + 1)|0;
      $i = $167;
     }
    }
    $168 = $0;
    $169 = (($168) + 11|0);
    HEAP8[$169>>0] = 48;
    $170 = $row_width;
    $171 = ($170*6)|0;
    $172 = $0;
    $173 = (($172) + 4|0);
    HEAP32[$173>>2] = $171;
   }
   $174 = $0;
   $175 = (($174) + 10|0);
   HEAP8[$175>>0] = 3;
  } else {
   label = 31;
  }
 }
 if ((label|0) == 31) {
  $176 = $0;
  $177 = (($176) + 8|0);
  $178 = HEAP8[$177>>0]|0;
  $179 = $178&255;
  $180 = ($179|0)==(0);
  if ($180) {
   label = 34;
  } else {
   $181 = $0;
   $182 = (($181) + 8|0);
   $183 = HEAP8[$182>>0]|0;
   $184 = $183&255;
   $185 = ($184|0)==(4);
   if ($185) {
    $186 = $2;
    $187 = $186 & 4194304;
    $188 = ($187|0)!=(0);
    if ($188) {
     label = 34;
    }
   }
  }
  if ((label|0) == 34) {
   $189 = $0;
   $190 = (($189) + 10|0);
   $191 = HEAP8[$190>>0]|0;
   $192 = $191&255;
   $193 = ($192|0)==(2);
   if ($193) {
    $194 = $0;
    $195 = (($194) + 9|0);
    $196 = HEAP8[$195>>0]|0;
    $197 = $196&255;
    $198 = ($197|0)==(8);
    if ($198) {
     $199 = $2;
     $200 = $199 & 128;
     $201 = ($200|0)!=(0);
     if ($201) {
      $i = 0;
      while(1) {
       $202 = $i;
       $203 = $row_width;
       $204 = ($202>>>0)<($203>>>0);
       if (!($204)) {
        break;
       }
       $205 = $sp;
       $206 = (($205) + 1|0);
       $sp = $206;
       $207 = HEAP8[$205>>0]|0;
       $208 = $dp;
       $209 = (($208) + 1|0);
       $dp = $209;
       HEAP8[$208>>0] = $207;
       $210 = $sp;
       $211 = (($210) + 1|0);
       $sp = $211;
       $212 = $i;
       $213 = (($212) + 1)|0;
       $i = $213;
      }
     } else {
      $i = 0;
      while(1) {
       $214 = $i;
       $215 = $row_width;
       $216 = ($214>>>0)<($215>>>0);
       if (!($216)) {
        break;
       }
       $217 = $sp;
       $218 = (($217) + 1|0);
       $sp = $218;
       $219 = $sp;
       $220 = (($219) + 1|0);
       $sp = $220;
       $221 = HEAP8[$219>>0]|0;
       $222 = $dp;
       $223 = (($222) + 1|0);
       $dp = $223;
       HEAP8[$222>>0] = $221;
       $224 = $i;
       $225 = (($224) + 1)|0;
       $i = $225;
      }
     }
     $226 = $0;
     $227 = (($226) + 11|0);
     HEAP8[$227>>0] = 8;
     $228 = $row_width;
     $229 = $0;
     $230 = (($229) + 4|0);
     HEAP32[$230>>2] = $228;
    } else {
     $231 = $2;
     $232 = $231 & 128;
     $233 = ($232|0)!=(0);
     if ($233) {
      $234 = $sp;
      $235 = (($234) + 4|0);
      $sp = $235;
      $236 = $dp;
      $237 = (($236) + 2|0);
      $dp = $237;
      $i = 1;
      while(1) {
       $238 = $i;
       $239 = $row_width;
       $240 = ($238>>>0)<($239>>>0);
       if (!($240)) {
        break;
       }
       $241 = $sp;
       $242 = (($241) + 1|0);
       $sp = $242;
       $243 = HEAP8[$241>>0]|0;
       $244 = $dp;
       $245 = (($244) + 1|0);
       $dp = $245;
       HEAP8[$244>>0] = $243;
       $246 = $sp;
       $247 = (($246) + 1|0);
       $sp = $247;
       $248 = HEAP8[$246>>0]|0;
       $249 = $dp;
       $250 = (($249) + 1|0);
       $dp = $250;
       HEAP8[$249>>0] = $248;
       $251 = $sp;
       $252 = (($251) + 2|0);
       $sp = $252;
       $253 = $i;
       $254 = (($253) + 1)|0;
       $i = $254;
      }
     } else {
      $i = 0;
      while(1) {
       $255 = $i;
       $256 = $row_width;
       $257 = ($255>>>0)<($256>>>0);
       if (!($257)) {
        break;
       }
       $258 = $sp;
       $259 = (($258) + 2|0);
       $sp = $259;
       $260 = $sp;
       $261 = (($260) + 1|0);
       $sp = $261;
       $262 = HEAP8[$260>>0]|0;
       $263 = $dp;
       $264 = (($263) + 1|0);
       $dp = $264;
       HEAP8[$263>>0] = $262;
       $265 = $sp;
       $266 = (($265) + 1|0);
       $sp = $266;
       $267 = HEAP8[$265>>0]|0;
       $268 = $dp;
       $269 = (($268) + 1|0);
       $dp = $269;
       HEAP8[$268>>0] = $267;
       $270 = $i;
       $271 = (($270) + 1)|0;
       $i = $271;
      }
     }
     $272 = $0;
     $273 = (($272) + 11|0);
     HEAP8[$273>>0] = 16;
     $274 = $row_width;
     $275 = $274<<1;
     $276 = $0;
     $277 = (($276) + 4|0);
     HEAP32[$277>>2] = $275;
    }
    $278 = $0;
    $279 = (($278) + 10|0);
    HEAP8[$279>>0] = 1;
   }
  }
 }
 $280 = $2;
 $281 = $280 & 4194304;
 $282 = ($281|0)!=(0);
 if (!($282)) {
  STACKTOP = sp;return;
 }
 $283 = $0;
 $284 = (($283) + 8|0);
 $285 = HEAP8[$284>>0]|0;
 $286 = $285&255;
 $287 = $286 & -5;
 $288 = $287&255;
 HEAP8[$284>>0] = $288;
 STACKTOP = sp;return;
}
function _png_do_bgr($row_info,$row) {
 $row_info = $row_info|0;
 $row = $row|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $i2 = 0, $i5 = 0, $i8 = 0, $row_width = 0, $rp = 0, $rp1 = 0, $rp4 = 0, $rp7 = 0, $save = 0;
 var $save3 = 0, $save6 = 0, $save9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $row_info;
 $1 = $row;
 $2 = $0;
 $3 = (($2) + 8|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 2;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = $0;
 $9 = HEAP32[$8>>2]|0;
 $row_width = $9;
 $10 = $0;
 $11 = (($10) + 9|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(8);
 if ($14) {
  $15 = $0;
  $16 = (($15) + 8|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = ($18|0)==(2);
  if ($19) {
   $i = 0;
   $20 = $1;
   $rp = $20;
   while(1) {
    $21 = $i;
    $22 = $row_width;
    $23 = ($21>>>0)<($22>>>0);
    if (!($23)) {
     break;
    }
    $24 = $rp;
    $25 = HEAP8[$24>>0]|0;
    $save = $25;
    $26 = $rp;
    $27 = (($26) + 2|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = $rp;
    HEAP8[$29>>0] = $28;
    $30 = $save;
    $31 = $rp;
    $32 = (($31) + 2|0);
    HEAP8[$32>>0] = $30;
    $33 = $i;
    $34 = (($33) + 1)|0;
    $i = $34;
    $35 = $rp;
    $36 = (($35) + 3|0);
    $rp = $36;
   }
  } else {
   $37 = $0;
   $38 = (($37) + 8|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = ($40|0)==(6);
   if ($41) {
    $i2 = 0;
    $42 = $1;
    $rp1 = $42;
    while(1) {
     $43 = $i2;
     $44 = $row_width;
     $45 = ($43>>>0)<($44>>>0);
     if (!($45)) {
      break;
     }
     $46 = $rp1;
     $47 = HEAP8[$46>>0]|0;
     $save3 = $47;
     $48 = $rp1;
     $49 = (($48) + 2|0);
     $50 = HEAP8[$49>>0]|0;
     $51 = $rp1;
     HEAP8[$51>>0] = $50;
     $52 = $save3;
     $53 = $rp1;
     $54 = (($53) + 2|0);
     HEAP8[$54>>0] = $52;
     $55 = $i2;
     $56 = (($55) + 1)|0;
     $i2 = $56;
     $57 = $rp1;
     $58 = (($57) + 4|0);
     $rp1 = $58;
    }
   }
  }
 } else {
  $59 = $0;
  $60 = (($59) + 9|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = $61&255;
  $63 = ($62|0)==(16);
  if ($63) {
   $64 = $0;
   $65 = (($64) + 8|0);
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   $68 = ($67|0)==(2);
   if ($68) {
    $i5 = 0;
    $69 = $1;
    $rp4 = $69;
    while(1) {
     $70 = $i5;
     $71 = $row_width;
     $72 = ($70>>>0)<($71>>>0);
     if (!($72)) {
      break;
     }
     $73 = $rp4;
     $74 = HEAP8[$73>>0]|0;
     $save6 = $74;
     $75 = $rp4;
     $76 = (($75) + 4|0);
     $77 = HEAP8[$76>>0]|0;
     $78 = $rp4;
     HEAP8[$78>>0] = $77;
     $79 = $save6;
     $80 = $rp4;
     $81 = (($80) + 4|0);
     HEAP8[$81>>0] = $79;
     $82 = $rp4;
     $83 = (($82) + 1|0);
     $84 = HEAP8[$83>>0]|0;
     $save6 = $84;
     $85 = $rp4;
     $86 = (($85) + 5|0);
     $87 = HEAP8[$86>>0]|0;
     $88 = $rp4;
     $89 = (($88) + 1|0);
     HEAP8[$89>>0] = $87;
     $90 = $save6;
     $91 = $rp4;
     $92 = (($91) + 5|0);
     HEAP8[$92>>0] = $90;
     $93 = $i5;
     $94 = (($93) + 1)|0;
     $i5 = $94;
     $95 = $rp4;
     $96 = (($95) + 6|0);
     $rp4 = $96;
    }
   } else {
    $97 = $0;
    $98 = (($97) + 8|0);
    $99 = HEAP8[$98>>0]|0;
    $100 = $99&255;
    $101 = ($100|0)==(6);
    if ($101) {
     $i8 = 0;
     $102 = $1;
     $rp7 = $102;
     while(1) {
      $103 = $i8;
      $104 = $row_width;
      $105 = ($103>>>0)<($104>>>0);
      if (!($105)) {
       break;
      }
      $106 = $rp7;
      $107 = HEAP8[$106>>0]|0;
      $save9 = $107;
      $108 = $rp7;
      $109 = (($108) + 4|0);
      $110 = HEAP8[$109>>0]|0;
      $111 = $rp7;
      HEAP8[$111>>0] = $110;
      $112 = $save9;
      $113 = $rp7;
      $114 = (($113) + 4|0);
      HEAP8[$114>>0] = $112;
      $115 = $rp7;
      $116 = (($115) + 1|0);
      $117 = HEAP8[$116>>0]|0;
      $save9 = $117;
      $118 = $rp7;
      $119 = (($118) + 5|0);
      $120 = HEAP8[$119>>0]|0;
      $121 = $rp7;
      $122 = (($121) + 1|0);
      HEAP8[$122>>0] = $120;
      $123 = $save9;
      $124 = $rp7;
      $125 = (($124) + 5|0);
      HEAP8[$125>>0] = $123;
      $126 = $i8;
      $127 = (($126) + 1)|0;
      $i8 = $127;
      $128 = $rp7;
      $129 = (($128) + 8|0);
      $rp7 = $129;
     }
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _crc32($crc,$buf,$len) {
 $crc = $crc|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $endian = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $endian = sp;
 $1 = $crc;
 $2 = $buf;
 $3 = $len;
 $4 = $2;
 $5 = ($4|0)==(0|0);
 do {
  if ($5) {
   $0 = 0;
  } else {
   HEAP32[$endian>>2] = 1;
   $6 = HEAP8[$endian>>0]|0;
   $7 = ($6<<24>>24)!=(0);
   if ($7) {
    $8 = $1;
    $9 = $2;
    $10 = $3;
    $11 = (_crc32_little($8,$9,$10)|0);
    $0 = $11;
    break;
   } else {
    $12 = $1;
    $13 = $2;
    $14 = $3;
    $15 = (_crc32_big($12,$13,$14)|0);
    $0 = $15;
    break;
   }
  }
 } while(0);
 $16 = $0;
 STACKTOP = sp;return ($16|0);
}
function _crc32_little($crc,$buf,$len) {
 $crc = $crc|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf4 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $crc;
 $1 = $buf;
 $2 = $len;
 $3 = $0;
 $c = $3;
 $4 = $c;
 $5 = $4 ^ -1;
 $c = $5;
 while(1) {
  $6 = $2;
  $7 = ($6|0)!=(0);
  if ($7) {
   $8 = $1;
   $9 = $8;
   $10 = $9 & 3;
   $11 = ($10|0)!=(0);
   $290 = $11;
  } else {
   $290 = 0;
  }
  if (!($290)) {
   break;
  }
  $12 = $c;
  $13 = $1;
  $14 = (($13) + 1|0);
  $1 = $14;
  $15 = HEAP8[$13>>0]|0;
  $16 = $15&255;
  $17 = $12 ^ $16;
  $18 = $17 & 255;
  $19 = (64976 + ($18<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $c;
  $22 = $21 >>> 8;
  $23 = $20 ^ $22;
  $c = $23;
  $24 = $2;
  $25 = (($24) + -1)|0;
  $2 = $25;
 }
 $26 = $1;
 $buf4 = $26;
 while(1) {
  $27 = $2;
  $28 = ($27>>>0)>=(32);
  if (!($28)) {
   break;
  }
  $29 = $buf4;
  $30 = (($29) + 4|0);
  $buf4 = $30;
  $31 = HEAP32[$29>>2]|0;
  $32 = $c;
  $33 = $32 ^ $31;
  $c = $33;
  $34 = $c;
  $35 = $34 & 255;
  $36 = (((64976 + 3072|0)) + ($35<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $c;
  $39 = $38 >>> 8;
  $40 = $39 & 255;
  $41 = (((64976 + 2048|0)) + ($40<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $37 ^ $42;
  $44 = $c;
  $45 = $44 >>> 16;
  $46 = $45 & 255;
  $47 = (((64976 + 1024|0)) + ($46<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $43 ^ $48;
  $50 = $c;
  $51 = $50 >>> 24;
  $52 = (64976 + ($51<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $49 ^ $53;
  $c = $54;
  $55 = $buf4;
  $56 = (($55) + 4|0);
  $buf4 = $56;
  $57 = HEAP32[$55>>2]|0;
  $58 = $c;
  $59 = $58 ^ $57;
  $c = $59;
  $60 = $c;
  $61 = $60 & 255;
  $62 = (((64976 + 3072|0)) + ($61<<2)|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $c;
  $65 = $64 >>> 8;
  $66 = $65 & 255;
  $67 = (((64976 + 2048|0)) + ($66<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $63 ^ $68;
  $70 = $c;
  $71 = $70 >>> 16;
  $72 = $71 & 255;
  $73 = (((64976 + 1024|0)) + ($72<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $69 ^ $74;
  $76 = $c;
  $77 = $76 >>> 24;
  $78 = (64976 + ($77<<2)|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = $75 ^ $79;
  $c = $80;
  $81 = $buf4;
  $82 = (($81) + 4|0);
  $buf4 = $82;
  $83 = HEAP32[$81>>2]|0;
  $84 = $c;
  $85 = $84 ^ $83;
  $c = $85;
  $86 = $c;
  $87 = $86 & 255;
  $88 = (((64976 + 3072|0)) + ($87<<2)|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $c;
  $91 = $90 >>> 8;
  $92 = $91 & 255;
  $93 = (((64976 + 2048|0)) + ($92<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $89 ^ $94;
  $96 = $c;
  $97 = $96 >>> 16;
  $98 = $97 & 255;
  $99 = (((64976 + 1024|0)) + ($98<<2)|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = $95 ^ $100;
  $102 = $c;
  $103 = $102 >>> 24;
  $104 = (64976 + ($103<<2)|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = $101 ^ $105;
  $c = $106;
  $107 = $buf4;
  $108 = (($107) + 4|0);
  $buf4 = $108;
  $109 = HEAP32[$107>>2]|0;
  $110 = $c;
  $111 = $110 ^ $109;
  $c = $111;
  $112 = $c;
  $113 = $112 & 255;
  $114 = (((64976 + 3072|0)) + ($113<<2)|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = $c;
  $117 = $116 >>> 8;
  $118 = $117 & 255;
  $119 = (((64976 + 2048|0)) + ($118<<2)|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = $115 ^ $120;
  $122 = $c;
  $123 = $122 >>> 16;
  $124 = $123 & 255;
  $125 = (((64976 + 1024|0)) + ($124<<2)|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = $121 ^ $126;
  $128 = $c;
  $129 = $128 >>> 24;
  $130 = (64976 + ($129<<2)|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = $127 ^ $131;
  $c = $132;
  $133 = $buf4;
  $134 = (($133) + 4|0);
  $buf4 = $134;
  $135 = HEAP32[$133>>2]|0;
  $136 = $c;
  $137 = $136 ^ $135;
  $c = $137;
  $138 = $c;
  $139 = $138 & 255;
  $140 = (((64976 + 3072|0)) + ($139<<2)|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = $c;
  $143 = $142 >>> 8;
  $144 = $143 & 255;
  $145 = (((64976 + 2048|0)) + ($144<<2)|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = $141 ^ $146;
  $148 = $c;
  $149 = $148 >>> 16;
  $150 = $149 & 255;
  $151 = (((64976 + 1024|0)) + ($150<<2)|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = $147 ^ $152;
  $154 = $c;
  $155 = $154 >>> 24;
  $156 = (64976 + ($155<<2)|0);
  $157 = HEAP32[$156>>2]|0;
  $158 = $153 ^ $157;
  $c = $158;
  $159 = $buf4;
  $160 = (($159) + 4|0);
  $buf4 = $160;
  $161 = HEAP32[$159>>2]|0;
  $162 = $c;
  $163 = $162 ^ $161;
  $c = $163;
  $164 = $c;
  $165 = $164 & 255;
  $166 = (((64976 + 3072|0)) + ($165<<2)|0);
  $167 = HEAP32[$166>>2]|0;
  $168 = $c;
  $169 = $168 >>> 8;
  $170 = $169 & 255;
  $171 = (((64976 + 2048|0)) + ($170<<2)|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = $167 ^ $172;
  $174 = $c;
  $175 = $174 >>> 16;
  $176 = $175 & 255;
  $177 = (((64976 + 1024|0)) + ($176<<2)|0);
  $178 = HEAP32[$177>>2]|0;
  $179 = $173 ^ $178;
  $180 = $c;
  $181 = $180 >>> 24;
  $182 = (64976 + ($181<<2)|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = $179 ^ $183;
  $c = $184;
  $185 = $buf4;
  $186 = (($185) + 4|0);
  $buf4 = $186;
  $187 = HEAP32[$185>>2]|0;
  $188 = $c;
  $189 = $188 ^ $187;
  $c = $189;
  $190 = $c;
  $191 = $190 & 255;
  $192 = (((64976 + 3072|0)) + ($191<<2)|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = $c;
  $195 = $194 >>> 8;
  $196 = $195 & 255;
  $197 = (((64976 + 2048|0)) + ($196<<2)|0);
  $198 = HEAP32[$197>>2]|0;
  $199 = $193 ^ $198;
  $200 = $c;
  $201 = $200 >>> 16;
  $202 = $201 & 255;
  $203 = (((64976 + 1024|0)) + ($202<<2)|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = $199 ^ $204;
  $206 = $c;
  $207 = $206 >>> 24;
  $208 = (64976 + ($207<<2)|0);
  $209 = HEAP32[$208>>2]|0;
  $210 = $205 ^ $209;
  $c = $210;
  $211 = $buf4;
  $212 = (($211) + 4|0);
  $buf4 = $212;
  $213 = HEAP32[$211>>2]|0;
  $214 = $c;
  $215 = $214 ^ $213;
  $c = $215;
  $216 = $c;
  $217 = $216 & 255;
  $218 = (((64976 + 3072|0)) + ($217<<2)|0);
  $219 = HEAP32[$218>>2]|0;
  $220 = $c;
  $221 = $220 >>> 8;
  $222 = $221 & 255;
  $223 = (((64976 + 2048|0)) + ($222<<2)|0);
  $224 = HEAP32[$223>>2]|0;
  $225 = $219 ^ $224;
  $226 = $c;
  $227 = $226 >>> 16;
  $228 = $227 & 255;
  $229 = (((64976 + 1024|0)) + ($228<<2)|0);
  $230 = HEAP32[$229>>2]|0;
  $231 = $225 ^ $230;
  $232 = $c;
  $233 = $232 >>> 24;
  $234 = (64976 + ($233<<2)|0);
  $235 = HEAP32[$234>>2]|0;
  $236 = $231 ^ $235;
  $c = $236;
  $237 = $2;
  $238 = (($237) - 32)|0;
  $2 = $238;
 }
 while(1) {
  $239 = $2;
  $240 = ($239>>>0)>=(4);
  if (!($240)) {
   break;
  }
  $241 = $buf4;
  $242 = (($241) + 4|0);
  $buf4 = $242;
  $243 = HEAP32[$241>>2]|0;
  $244 = $c;
  $245 = $244 ^ $243;
  $c = $245;
  $246 = $c;
  $247 = $246 & 255;
  $248 = (((64976 + 3072|0)) + ($247<<2)|0);
  $249 = HEAP32[$248>>2]|0;
  $250 = $c;
  $251 = $250 >>> 8;
  $252 = $251 & 255;
  $253 = (((64976 + 2048|0)) + ($252<<2)|0);
  $254 = HEAP32[$253>>2]|0;
  $255 = $249 ^ $254;
  $256 = $c;
  $257 = $256 >>> 16;
  $258 = $257 & 255;
  $259 = (((64976 + 1024|0)) + ($258<<2)|0);
  $260 = HEAP32[$259>>2]|0;
  $261 = $255 ^ $260;
  $262 = $c;
  $263 = $262 >>> 24;
  $264 = (64976 + ($263<<2)|0);
  $265 = HEAP32[$264>>2]|0;
  $266 = $261 ^ $265;
  $c = $266;
  $267 = $2;
  $268 = (($267) - 4)|0;
  $2 = $268;
 }
 $269 = $buf4;
 $1 = $269;
 $270 = $2;
 $271 = ($270|0)!=(0);
 if (!($271)) {
  $287 = $c;
  $288 = $287 ^ -1;
  $c = $288;
  $289 = $c;
  STACKTOP = sp;return ($289|0);
 }
 while(1) {
  $272 = $c;
  $273 = $1;
  $274 = (($273) + 1|0);
  $1 = $274;
  $275 = HEAP8[$273>>0]|0;
  $276 = $275&255;
  $277 = $272 ^ $276;
  $278 = $277 & 255;
  $279 = (64976 + ($278<<2)|0);
  $280 = HEAP32[$279>>2]|0;
  $281 = $c;
  $282 = $281 >>> 8;
  $283 = $280 ^ $282;
  $c = $283;
  $284 = $2;
  $285 = (($284) + -1)|0;
  $2 = $285;
  $286 = ($285|0)!=(0);
  if (!($286)) {
   break;
  }
 }
 $287 = $c;
 $288 = $287 ^ -1;
 $c = $288;
 $289 = $c;
 STACKTOP = sp;return ($289|0);
}
function _crc32_big($crc,$buf,$len) {
 $crc = $crc|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $buf4 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $crc;
 $1 = $buf;
 $2 = $len;
 $3 = $0;
 $4 = $3 >>> 24;
 $5 = $0;
 $6 = $5 >>> 8;
 $7 = $6 & 65280;
 $8 = (($4) + ($7))|0;
 $9 = $0;
 $10 = $9 & 65280;
 $11 = $10 << 8;
 $12 = (($8) + ($11))|0;
 $13 = $0;
 $14 = $13 & 255;
 $15 = $14 << 24;
 $16 = (($12) + ($15))|0;
 $c = $16;
 $17 = $c;
 $18 = $17 ^ -1;
 $c = $18;
 while(1) {
  $19 = $2;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = $1;
   $22 = $21;
   $23 = $22 & 3;
   $24 = ($23|0)!=(0);
   $320 = $24;
  } else {
   $320 = 0;
  }
  if (!($320)) {
   break;
  }
  $25 = $c;
  $26 = $25 >>> 24;
  $27 = $1;
  $28 = (($27) + 1|0);
  $1 = $28;
  $29 = HEAP8[$27>>0]|0;
  $30 = $29&255;
  $31 = $26 ^ $30;
  $32 = (((64976 + 4096|0)) + ($31<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $c;
  $35 = $34 << 8;
  $36 = $33 ^ $35;
  $c = $36;
  $37 = $2;
  $38 = (($37) + -1)|0;
  $2 = $38;
 }
 $39 = $1;
 $buf4 = $39;
 $40 = $buf4;
 $41 = (($40) + -4|0);
 $buf4 = $41;
 while(1) {
  $42 = $2;
  $43 = ($42>>>0)>=(32);
  if (!($43)) {
   break;
  }
  $44 = $buf4;
  $45 = (($44) + 4|0);
  $buf4 = $45;
  $46 = HEAP32[$45>>2]|0;
  $47 = $c;
  $48 = $47 ^ $46;
  $c = $48;
  $49 = $c;
  $50 = $49 & 255;
  $51 = (((64976 + 4096|0)) + ($50<<2)|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $c;
  $54 = $53 >>> 8;
  $55 = $54 & 255;
  $56 = (((64976 + 5120|0)) + ($55<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $52 ^ $57;
  $59 = $c;
  $60 = $59 >>> 16;
  $61 = $60 & 255;
  $62 = (((64976 + 6144|0)) + ($61<<2)|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $58 ^ $63;
  $65 = $c;
  $66 = $65 >>> 24;
  $67 = (((64976 + 7168|0)) + ($66<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $64 ^ $68;
  $c = $69;
  $70 = $buf4;
  $71 = (($70) + 4|0);
  $buf4 = $71;
  $72 = HEAP32[$71>>2]|0;
  $73 = $c;
  $74 = $73 ^ $72;
  $c = $74;
  $75 = $c;
  $76 = $75 & 255;
  $77 = (((64976 + 4096|0)) + ($76<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = $c;
  $80 = $79 >>> 8;
  $81 = $80 & 255;
  $82 = (((64976 + 5120|0)) + ($81<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = $78 ^ $83;
  $85 = $c;
  $86 = $85 >>> 16;
  $87 = $86 & 255;
  $88 = (((64976 + 6144|0)) + ($87<<2)|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $84 ^ $89;
  $91 = $c;
  $92 = $91 >>> 24;
  $93 = (((64976 + 7168|0)) + ($92<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $90 ^ $94;
  $c = $95;
  $96 = $buf4;
  $97 = (($96) + 4|0);
  $buf4 = $97;
  $98 = HEAP32[$97>>2]|0;
  $99 = $c;
  $100 = $99 ^ $98;
  $c = $100;
  $101 = $c;
  $102 = $101 & 255;
  $103 = (((64976 + 4096|0)) + ($102<<2)|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = $c;
  $106 = $105 >>> 8;
  $107 = $106 & 255;
  $108 = (((64976 + 5120|0)) + ($107<<2)|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = $104 ^ $109;
  $111 = $c;
  $112 = $111 >>> 16;
  $113 = $112 & 255;
  $114 = (((64976 + 6144|0)) + ($113<<2)|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = $110 ^ $115;
  $117 = $c;
  $118 = $117 >>> 24;
  $119 = (((64976 + 7168|0)) + ($118<<2)|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = $116 ^ $120;
  $c = $121;
  $122 = $buf4;
  $123 = (($122) + 4|0);
  $buf4 = $123;
  $124 = HEAP32[$123>>2]|0;
  $125 = $c;
  $126 = $125 ^ $124;
  $c = $126;
  $127 = $c;
  $128 = $127 & 255;
  $129 = (((64976 + 4096|0)) + ($128<<2)|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = $c;
  $132 = $131 >>> 8;
  $133 = $132 & 255;
  $134 = (((64976 + 5120|0)) + ($133<<2)|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = $130 ^ $135;
  $137 = $c;
  $138 = $137 >>> 16;
  $139 = $138 & 255;
  $140 = (((64976 + 6144|0)) + ($139<<2)|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = $136 ^ $141;
  $143 = $c;
  $144 = $143 >>> 24;
  $145 = (((64976 + 7168|0)) + ($144<<2)|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = $142 ^ $146;
  $c = $147;
  $148 = $buf4;
  $149 = (($148) + 4|0);
  $buf4 = $149;
  $150 = HEAP32[$149>>2]|0;
  $151 = $c;
  $152 = $151 ^ $150;
  $c = $152;
  $153 = $c;
  $154 = $153 & 255;
  $155 = (((64976 + 4096|0)) + ($154<<2)|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = $c;
  $158 = $157 >>> 8;
  $159 = $158 & 255;
  $160 = (((64976 + 5120|0)) + ($159<<2)|0);
  $161 = HEAP32[$160>>2]|0;
  $162 = $156 ^ $161;
  $163 = $c;
  $164 = $163 >>> 16;
  $165 = $164 & 255;
  $166 = (((64976 + 6144|0)) + ($165<<2)|0);
  $167 = HEAP32[$166>>2]|0;
  $168 = $162 ^ $167;
  $169 = $c;
  $170 = $169 >>> 24;
  $171 = (((64976 + 7168|0)) + ($170<<2)|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = $168 ^ $172;
  $c = $173;
  $174 = $buf4;
  $175 = (($174) + 4|0);
  $buf4 = $175;
  $176 = HEAP32[$175>>2]|0;
  $177 = $c;
  $178 = $177 ^ $176;
  $c = $178;
  $179 = $c;
  $180 = $179 & 255;
  $181 = (((64976 + 4096|0)) + ($180<<2)|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = $c;
  $184 = $183 >>> 8;
  $185 = $184 & 255;
  $186 = (((64976 + 5120|0)) + ($185<<2)|0);
  $187 = HEAP32[$186>>2]|0;
  $188 = $182 ^ $187;
  $189 = $c;
  $190 = $189 >>> 16;
  $191 = $190 & 255;
  $192 = (((64976 + 6144|0)) + ($191<<2)|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = $188 ^ $193;
  $195 = $c;
  $196 = $195 >>> 24;
  $197 = (((64976 + 7168|0)) + ($196<<2)|0);
  $198 = HEAP32[$197>>2]|0;
  $199 = $194 ^ $198;
  $c = $199;
  $200 = $buf4;
  $201 = (($200) + 4|0);
  $buf4 = $201;
  $202 = HEAP32[$201>>2]|0;
  $203 = $c;
  $204 = $203 ^ $202;
  $c = $204;
  $205 = $c;
  $206 = $205 & 255;
  $207 = (((64976 + 4096|0)) + ($206<<2)|0);
  $208 = HEAP32[$207>>2]|0;
  $209 = $c;
  $210 = $209 >>> 8;
  $211 = $210 & 255;
  $212 = (((64976 + 5120|0)) + ($211<<2)|0);
  $213 = HEAP32[$212>>2]|0;
  $214 = $208 ^ $213;
  $215 = $c;
  $216 = $215 >>> 16;
  $217 = $216 & 255;
  $218 = (((64976 + 6144|0)) + ($217<<2)|0);
  $219 = HEAP32[$218>>2]|0;
  $220 = $214 ^ $219;
  $221 = $c;
  $222 = $221 >>> 24;
  $223 = (((64976 + 7168|0)) + ($222<<2)|0);
  $224 = HEAP32[$223>>2]|0;
  $225 = $220 ^ $224;
  $c = $225;
  $226 = $buf4;
  $227 = (($226) + 4|0);
  $buf4 = $227;
  $228 = HEAP32[$227>>2]|0;
  $229 = $c;
  $230 = $229 ^ $228;
  $c = $230;
  $231 = $c;
  $232 = $231 & 255;
  $233 = (((64976 + 4096|0)) + ($232<<2)|0);
  $234 = HEAP32[$233>>2]|0;
  $235 = $c;
  $236 = $235 >>> 8;
  $237 = $236 & 255;
  $238 = (((64976 + 5120|0)) + ($237<<2)|0);
  $239 = HEAP32[$238>>2]|0;
  $240 = $234 ^ $239;
  $241 = $c;
  $242 = $241 >>> 16;
  $243 = $242 & 255;
  $244 = (((64976 + 6144|0)) + ($243<<2)|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = $240 ^ $245;
  $247 = $c;
  $248 = $247 >>> 24;
  $249 = (((64976 + 7168|0)) + ($248<<2)|0);
  $250 = HEAP32[$249>>2]|0;
  $251 = $246 ^ $250;
  $c = $251;
  $252 = $2;
  $253 = (($252) - 32)|0;
  $2 = $253;
 }
 while(1) {
  $254 = $2;
  $255 = ($254>>>0)>=(4);
  if (!($255)) {
   break;
  }
  $256 = $buf4;
  $257 = (($256) + 4|0);
  $buf4 = $257;
  $258 = HEAP32[$257>>2]|0;
  $259 = $c;
  $260 = $259 ^ $258;
  $c = $260;
  $261 = $c;
  $262 = $261 & 255;
  $263 = (((64976 + 4096|0)) + ($262<<2)|0);
  $264 = HEAP32[$263>>2]|0;
  $265 = $c;
  $266 = $265 >>> 8;
  $267 = $266 & 255;
  $268 = (((64976 + 5120|0)) + ($267<<2)|0);
  $269 = HEAP32[$268>>2]|0;
  $270 = $264 ^ $269;
  $271 = $c;
  $272 = $271 >>> 16;
  $273 = $272 & 255;
  $274 = (((64976 + 6144|0)) + ($273<<2)|0);
  $275 = HEAP32[$274>>2]|0;
  $276 = $270 ^ $275;
  $277 = $c;
  $278 = $277 >>> 24;
  $279 = (((64976 + 7168|0)) + ($278<<2)|0);
  $280 = HEAP32[$279>>2]|0;
  $281 = $276 ^ $280;
  $c = $281;
  $282 = $2;
  $283 = (($282) - 4)|0;
  $2 = $283;
 }
 $284 = $buf4;
 $285 = (($284) + 4|0);
 $buf4 = $285;
 $286 = $buf4;
 $1 = $286;
 $287 = $2;
 $288 = ($287|0)!=(0);
 if (!($288)) {
  $304 = $c;
  $305 = $304 ^ -1;
  $c = $305;
  $306 = $c;
  $307 = $306 >>> 24;
  $308 = $c;
  $309 = $308 >>> 8;
  $310 = $309 & 65280;
  $311 = (($307) + ($310))|0;
  $312 = $c;
  $313 = $312 & 65280;
  $314 = $313 << 8;
  $315 = (($311) + ($314))|0;
  $316 = $c;
  $317 = $316 & 255;
  $318 = $317 << 24;
  $319 = (($315) + ($318))|0;
  STACKTOP = sp;return ($319|0);
 }
 while(1) {
  $289 = $c;
  $290 = $289 >>> 24;
  $291 = $1;
  $292 = (($291) + 1|0);
  $1 = $292;
  $293 = HEAP8[$291>>0]|0;
  $294 = $293&255;
  $295 = $290 ^ $294;
  $296 = (((64976 + 4096|0)) + ($295<<2)|0);
  $297 = HEAP32[$296>>2]|0;
  $298 = $c;
  $299 = $298 << 8;
  $300 = $297 ^ $299;
  $c = $300;
  $301 = $2;
  $302 = (($301) + -1)|0;
  $2 = $302;
  $303 = ($302|0)!=(0);
  if (!($303)) {
   break;
  }
 }
 $304 = $c;
 $305 = $304 ^ -1;
 $c = $305;
 $306 = $c;
 $307 = $306 >>> 24;
 $308 = $c;
 $309 = $308 >>> 8;
 $310 = $309 & 65280;
 $311 = (($307) + ($310))|0;
 $312 = $c;
 $313 = $312 & 65280;
 $314 = $313 << 8;
 $315 = (($311) + ($314))|0;
 $316 = $c;
 $317 = $316 & 255;
 $318 = $317 << 24;
 $319 = (($315) + ($318))|0;
 STACKTOP = sp;return ($319|0);
}
function _inflateReset($strm) {
 $strm = $strm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $strm;
 $2 = $1;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = $1;
  $5 = (($4) + 28|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)==(0|0);
  if (!($7)) {
   $8 = $1;
   $9 = (($8) + 28|0);
   $10 = HEAP32[$9>>2]|0;
   $state = $10;
   $11 = $state;
   $12 = (($11) + 28|0);
   HEAP32[$12>>2] = 0;
   $13 = $1;
   $14 = (($13) + 20|0);
   HEAP32[$14>>2] = 0;
   $15 = $1;
   $16 = (($15) + 8|0);
   HEAP32[$16>>2] = 0;
   $17 = $1;
   $18 = (($17) + 24|0);
   HEAP32[$18>>2] = 0;
   $19 = $1;
   $20 = (($19) + 48|0);
   HEAP32[$20>>2] = 1;
   $21 = $state;
   HEAP32[$21>>2] = 0;
   $22 = $state;
   $23 = (($22) + 4|0);
   HEAP32[$23>>2] = 0;
   $24 = $state;
   $25 = (($24) + 12|0);
   HEAP32[$25>>2] = 0;
   $26 = $state;
   $27 = (($26) + 20|0);
   HEAP32[$27>>2] = 32768;
   $28 = $state;
   $29 = (($28) + 32|0);
   HEAP32[$29>>2] = 0;
   $30 = $state;
   $31 = (($30) + 40|0);
   HEAP32[$31>>2] = 0;
   $32 = $state;
   $33 = (($32) + 44|0);
   HEAP32[$33>>2] = 0;
   $34 = $state;
   $35 = (($34) + 48|0);
   HEAP32[$35>>2] = 0;
   $36 = $state;
   $37 = (($36) + 56|0);
   HEAP32[$37>>2] = 0;
   $38 = $state;
   $39 = (($38) + 60|0);
   HEAP32[$39>>2] = 0;
   $40 = $state;
   $41 = (($40) + 1328|0);
   $42 = $state;
   $43 = (($42) + 108|0);
   HEAP32[$43>>2] = $41;
   $44 = $state;
   $45 = (($44) + 80|0);
   HEAP32[$45>>2] = $41;
   $46 = $state;
   $47 = (($46) + 76|0);
   HEAP32[$47>>2] = $41;
   $0 = 0;
   $48 = $0;
   STACKTOP = sp;return ($48|0);
  }
 }
 $0 = -2;
 $48 = $0;
 STACKTOP = sp;return ($48|0);
}
function _inflateInit2_($strm,$windowBits,$version,$stream_size) {
 $strm = $strm|0;
 $windowBits = $windowBits|0;
 $version = $version|0;
 $stream_size = $stream_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $strm;
 $2 = $windowBits;
 $3 = $version;
 $4 = $stream_size;
 $5 = $3;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $3;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = HEAP8[73168>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($9|0)!=($11|0);
  if (!($12)) {
   $13 = $4;
   $14 = ($13|0)!=(56);
   if (!($14)) {
    $15 = $1;
    $16 = ($15|0)==(0|0);
    if ($16) {
     $0 = -2;
     $80 = $0;
     STACKTOP = sp;return ($80|0);
    }
    $17 = $1;
    $18 = (($17) + 24|0);
    HEAP32[$18>>2] = 0;
    $19 = $1;
    $20 = (($19) + 32|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)==(0|0);
    if ($22) {
     $23 = $1;
     $24 = (($23) + 32|0);
     HEAP32[$24>>2] = 146;
     $25 = $1;
     $26 = (($25) + 40|0);
     HEAP32[$26>>2] = 0;
    }
    $27 = $1;
    $28 = (($27) + 36|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($29|0)==(0|0);
    if ($30) {
     $31 = $1;
     $32 = (($31) + 36|0);
     HEAP32[$32>>2] = 147;
    }
    $33 = $1;
    $34 = (($33) + 32|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = $1;
    $37 = (($36) + 40|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (FUNCTION_TABLE_iiii[$35 & 255]($38,1,9520)|0);
    $state = $39;
    $40 = $state;
    $41 = ($40|0)==(0|0);
    if ($41) {
     $0 = -4;
     $80 = $0;
     STACKTOP = sp;return ($80|0);
    }
    $42 = $state;
    $43 = $1;
    $44 = (($43) + 28|0);
    HEAP32[$44>>2] = $42;
    $45 = $2;
    $46 = ($45|0)<(0);
    if ($46) {
     $47 = $state;
     $48 = (($47) + 8|0);
     HEAP32[$48>>2] = 0;
     $49 = $2;
     $50 = (0 - ($49))|0;
     $2 = $50;
    } else {
     $51 = $2;
     $52 = $51 >> 4;
     $53 = (($52) + 1)|0;
     $54 = $state;
     $55 = (($54) + 8|0);
     HEAP32[$55>>2] = $53;
     $56 = $2;
     $57 = ($56|0)<(48);
     if ($57) {
      $58 = $2;
      $59 = $58 & 15;
      $2 = $59;
     }
    }
    $60 = $2;
    $61 = ($60|0)<(8);
    if (!($61)) {
     $62 = $2;
     $63 = ($62|0)>(15);
     if (!($63)) {
      $73 = $2;
      $74 = $state;
      $75 = (($74) + 36|0);
      HEAP32[$75>>2] = $73;
      $76 = $state;
      $77 = (($76) + 52|0);
      HEAP32[$77>>2] = 0;
      $78 = $1;
      $79 = (_inflateReset($78)|0);
      $0 = $79;
      $80 = $0;
      STACKTOP = sp;return ($80|0);
     }
    }
    $64 = $1;
    $65 = (($64) + 36|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = $1;
    $68 = (($67) + 40|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = $state;
    FUNCTION_TABLE_vii[$66 & 255]($69,$70);
    $71 = $1;
    $72 = (($71) + 28|0);
    HEAP32[$72>>2] = 0;
    $0 = -2;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
  }
 }
 $0 = -6;
 $80 = $0;
 STACKTOP = sp;return ($80|0);
}
function _inflateInit_($strm,$version,$stream_size) {
 $strm = $strm|0;
 $version = $version|0;
 $stream_size = $stream_size|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $strm;
 $1 = $version;
 $2 = $stream_size;
 $3 = $0;
 $4 = $1;
 $5 = $2;
 $6 = (_inflateInit2_($3,15,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _inflate($strm,$flush) {
 $strm = $strm|0;
 $flush = $flush|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0, $1988 = 0;
 var $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0, $2004 = 0;
 var $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0, $2022 = 0;
 var $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0, $2040 = 0;
 var $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0;
 var $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0;
 var $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0;
 var $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0;
 var $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0;
 var $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0;
 var $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0;
 var $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0;
 var $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0;
 var $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0;
 var $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0;
 var $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0;
 var $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0;
 var $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0;
 var $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0;
 var $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0;
 var $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0;
 var $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0;
 var $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0;
 var $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0;
 var $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0;
 var $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0;
 var $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0;
 var $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0;
 var $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0;
 var $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0;
 var $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0;
 var $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0;
 var $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0;
 var $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0;
 var $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0;
 var $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0;
 var $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0;
 var $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0;
 var $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0;
 var $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0;
 var $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0;
 var $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0;
 var $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0;
 var $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0;
 var $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0;
 var $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0;
 var $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0;
 var $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0;
 var $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $bits = 0, $copy = 0, $from = 0, $have = 0, $hbuf = 0, $hold = 0, $in = 0, $last = 0, $left = 0, $len = 0, $next = 0, $out = 0;
 var $put = 0, $ret = 0, $state = 0, $this = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $this = sp + 68|0;
 $last = sp + 64|0;
 $hbuf = sp + 72|0;
 $1 = $strm;
 $2 = $flush;
 $3 = $1;
 $4 = ($3|0)==(0|0);
 do {
  if (!($4)) {
   $5 = $1;
   $6 = (($5) + 28|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)==(0|0);
   if (!($8)) {
    $9 = $1;
    $10 = (($9) + 12|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($11|0)==(0|0);
    if (!($12)) {
     $13 = $1;
     $14 = HEAP32[$13>>2]|0;
     $15 = ($14|0)==(0|0);
     if ($15) {
      $16 = $1;
      $17 = (($16) + 4|0);
      $18 = HEAP32[$17>>2]|0;
      $19 = ($18|0)!=(0);
      if ($19) {
       break;
      }
     }
     $20 = $1;
     $21 = (($20) + 28|0);
     $22 = HEAP32[$21>>2]|0;
     $state = $22;
     $23 = $state;
     $24 = HEAP32[$23>>2]|0;
     $25 = ($24|0)==(11);
     if ($25) {
      $26 = $state;
      HEAP32[$26>>2] = 12;
     }
     $27 = $1;
     $28 = (($27) + 12|0);
     $29 = HEAP32[$28>>2]|0;
     $put = $29;
     $30 = $1;
     $31 = (($30) + 16|0);
     $32 = HEAP32[$31>>2]|0;
     $left = $32;
     $33 = $1;
     $34 = HEAP32[$33>>2]|0;
     $next = $34;
     $35 = $1;
     $36 = (($35) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $have = $37;
     $38 = $state;
     $39 = (($38) + 56|0);
     $40 = HEAP32[$39>>2]|0;
     $hold = $40;
     $41 = $state;
     $42 = (($41) + 60|0);
     $43 = HEAP32[$42>>2]|0;
     $bits = $43;
     $44 = $have;
     $in = $44;
     $45 = $left;
     $out = $45;
     $ret = 0;
     L13: while(1) {
      $46 = $state;
      $47 = HEAP32[$46>>2]|0;
      L15: do {
       switch ($47|0) {
       case 0:  {
        $48 = $state;
        $49 = (($48) + 8|0);
        $50 = HEAP32[$49>>2]|0;
        $51 = ($50|0)==(0);
        if ($51) {
         $52 = $state;
         HEAP32[$52>>2] = 12;
         break L15;
        }
        while(1) {
         $53 = $bits;
         $54 = ($53>>>0)<(16);
         if (!($54)) {
          break;
         }
         $55 = $have;
         $56 = ($55|0)==(0);
         if ($56) {
          label = 20;
          break L13;
         }
         $57 = $have;
         $58 = (($57) + -1)|0;
         $have = $58;
         $59 = $next;
         $60 = (($59) + 1|0);
         $next = $60;
         $61 = HEAP8[$59>>0]|0;
         $62 = $61&255;
         $63 = $bits;
         $64 = $62 << $63;
         $65 = $hold;
         $66 = (($65) + ($64))|0;
         $hold = $66;
         $67 = $bits;
         $68 = (($67) + 8)|0;
         $bits = $68;
        }
        $69 = $state;
        $70 = (($69) + 8|0);
        $71 = HEAP32[$70>>2]|0;
        $72 = $71 & 2;
        $73 = ($72|0)!=(0);
        if ($73) {
         $74 = $hold;
         $75 = ($74|0)==(35615);
         if ($75) {
          $76 = (_crc32(0,0,0)|0);
          $77 = $state;
          $78 = (($77) + 24|0);
          HEAP32[$78>>2] = $76;
          $79 = $hold;
          $80 = $79&255;
          HEAP8[$hbuf>>0] = $80;
          $81 = $hold;
          $82 = $81 >>> 8;
          $83 = $82&255;
          $84 = (($hbuf) + 1|0);
          HEAP8[$84>>0] = $83;
          $85 = $state;
          $86 = (($85) + 24|0);
          $87 = HEAP32[$86>>2]|0;
          $88 = (_crc32($87,$hbuf,2)|0);
          $89 = $state;
          $90 = (($89) + 24|0);
          HEAP32[$90>>2] = $88;
          $hold = 0;
          $bits = 0;
          $91 = $state;
          HEAP32[$91>>2] = 1;
          break L15;
         }
        }
        $92 = $state;
        $93 = (($92) + 16|0);
        HEAP32[$93>>2] = 0;
        $94 = $state;
        $95 = (($94) + 32|0);
        $96 = HEAP32[$95>>2]|0;
        $97 = ($96|0)!=(0|0);
        if ($97) {
         $98 = $state;
         $99 = (($98) + 32|0);
         $100 = HEAP32[$99>>2]|0;
         $101 = (($100) + 48|0);
         HEAP32[$101>>2] = -1;
        }
        $102 = $state;
        $103 = (($102) + 8|0);
        $104 = HEAP32[$103>>2]|0;
        $105 = $104 & 1;
        $106 = ($105|0)!=(0);
        if ($106) {
         $107 = $hold;
         $108 = $107 & 255;
         $109 = $108 << 8;
         $110 = $hold;
         $111 = $110 >>> 8;
         $112 = (($109) + ($111))|0;
         $113 = (($112>>>0) % 31)&-1;
         $114 = ($113|0)!=(0);
         if (!($114)) {
          $118 = $hold;
          $119 = $118 & 15;
          $120 = ($119|0)!=(8);
          if ($120) {
           $121 = $1;
           $122 = (($121) + 24|0);
           HEAP32[$122>>2] = 73240;
           $123 = $state;
           HEAP32[$123>>2] = 27;
           break L15;
          }
          $124 = $hold;
          $125 = $124 >>> 4;
          $hold = $125;
          $126 = $bits;
          $127 = (($126) - 4)|0;
          $bits = $127;
          $128 = $hold;
          $129 = $128 & 15;
          $130 = (($129) + 8)|0;
          $len = $130;
          $131 = $len;
          $132 = $state;
          $133 = (($132) + 36|0);
          $134 = HEAP32[$133>>2]|0;
          $135 = ($131>>>0)>($134>>>0);
          if ($135) {
           $136 = $1;
           $137 = (($136) + 24|0);
           HEAP32[$137>>2] = 73272;
           $138 = $state;
           HEAP32[$138>>2] = 27;
           break L15;
          }
          $139 = $len;
          $140 = 1 << $139;
          $141 = $state;
          $142 = (($141) + 20|0);
          HEAP32[$142>>2] = $140;
          $143 = (_adler32(0,0,0)|0);
          $144 = $state;
          $145 = (($144) + 24|0);
          HEAP32[$145>>2] = $143;
          $146 = $1;
          $147 = (($146) + 48|0);
          HEAP32[$147>>2] = $143;
          $148 = $hold;
          $149 = $148 & 512;
          $150 = ($149|0)!=(0);
          $151 = $150 ? 9 : 11;
          $152 = $state;
          HEAP32[$152>>2] = $151;
          $hold = 0;
          $bits = 0;
          break L15;
         }
        }
        $115 = $1;
        $116 = (($115) + 24|0);
        HEAP32[$116>>2] = 73216;
        $117 = $state;
        HEAP32[$117>>2] = 27;
        break;
       }
       case 1:  {
        while(1) {
         $153 = $bits;
         $154 = ($153>>>0)<(16);
         if (!($154)) {
          break;
         }
         $155 = $have;
         $156 = ($155|0)==(0);
         if ($156) {
          label = 50;
          break L13;
         }
         $157 = $have;
         $158 = (($157) + -1)|0;
         $have = $158;
         $159 = $next;
         $160 = (($159) + 1|0);
         $next = $160;
         $161 = HEAP8[$159>>0]|0;
         $162 = $161&255;
         $163 = $bits;
         $164 = $162 << $163;
         $165 = $hold;
         $166 = (($165) + ($164))|0;
         $hold = $166;
         $167 = $bits;
         $168 = (($167) + 8)|0;
         $bits = $168;
        }
        $169 = $hold;
        $170 = $state;
        $171 = (($170) + 16|0);
        HEAP32[$171>>2] = $169;
        $172 = $state;
        $173 = (($172) + 16|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = $174 & 255;
        $176 = ($175|0)!=(8);
        if ($176) {
         $177 = $1;
         $178 = (($177) + 24|0);
         HEAP32[$178>>2] = 73240;
         $179 = $state;
         HEAP32[$179>>2] = 27;
         break L15;
        }
        $180 = $state;
        $181 = (($180) + 16|0);
        $182 = HEAP32[$181>>2]|0;
        $183 = $182 & 57344;
        $184 = ($183|0)!=(0);
        if ($184) {
         $185 = $1;
         $186 = (($185) + 24|0);
         HEAP32[$186>>2] = 73296;
         $187 = $state;
         HEAP32[$187>>2] = 27;
         break L15;
        }
        $188 = $state;
        $189 = (($188) + 32|0);
        $190 = HEAP32[$189>>2]|0;
        $191 = ($190|0)!=(0|0);
        if ($191) {
         $192 = $hold;
         $193 = $192 >>> 8;
         $194 = $193 & 1;
         $195 = $state;
         $196 = (($195) + 32|0);
         $197 = HEAP32[$196>>2]|0;
         HEAP32[$197>>2] = $194;
        }
        $198 = $state;
        $199 = (($198) + 16|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = $200 & 512;
        $202 = ($201|0)!=(0);
        if ($202) {
         $203 = $hold;
         $204 = $203&255;
         HEAP8[$hbuf>>0] = $204;
         $205 = $hold;
         $206 = $205 >>> 8;
         $207 = $206&255;
         $208 = (($hbuf) + 1|0);
         HEAP8[$208>>0] = $207;
         $209 = $state;
         $210 = (($209) + 24|0);
         $211 = HEAP32[$210>>2]|0;
         $212 = (_crc32($211,$hbuf,2)|0);
         $213 = $state;
         $214 = (($213) + 24|0);
         HEAP32[$214>>2] = $212;
        }
        $hold = 0;
        $bits = 0;
        $215 = $state;
        HEAP32[$215>>2] = 2;
        label = 67;
        break;
       }
       case 2:  {
        label = 67;
        break;
       }
       case 6:  {
        label = 143;
        break;
       }
       case 5:  {
        label = 126;
        break;
       }
       case 3:  {
        label = 85;
        break;
       }
       case 4:  {
        label = 103;
        break;
       }
       case 9:  {
        while(1) {
         $679 = $bits;
         $680 = ($679>>>0)<(32);
         if (!($680)) {
          break;
         }
         $681 = $have;
         $682 = ($681|0)==(0);
         if ($682) {
          label = 208;
          break L13;
         }
         $683 = $have;
         $684 = (($683) + -1)|0;
         $have = $684;
         $685 = $next;
         $686 = (($685) + 1|0);
         $next = $686;
         $687 = HEAP8[$685>>0]|0;
         $688 = $687&255;
         $689 = $bits;
         $690 = $688 << $689;
         $691 = $hold;
         $692 = (($691) + ($690))|0;
         $hold = $692;
         $693 = $bits;
         $694 = (($693) + 8)|0;
         $bits = $694;
        }
        $695 = $hold;
        $696 = $695 >>> 24;
        $697 = $696 & 255;
        $698 = $hold;
        $699 = $698 >>> 8;
        $700 = $699 & 65280;
        $701 = (($697) + ($700))|0;
        $702 = $hold;
        $703 = $702 & 65280;
        $704 = $703 << 8;
        $705 = (($701) + ($704))|0;
        $706 = $hold;
        $707 = $706 & 255;
        $708 = $707 << 24;
        $709 = (($705) + ($708))|0;
        $710 = $state;
        $711 = (($710) + 24|0);
        HEAP32[$711>>2] = $709;
        $712 = $1;
        $713 = (($712) + 48|0);
        HEAP32[$713>>2] = $709;
        $hold = 0;
        $bits = 0;
        $714 = $state;
        HEAP32[$714>>2] = 10;
        label = 215;
        break;
       }
       case 10:  {
        label = 215;
        break;
       }
       case 12:  {
        label = 223;
        break;
       }
       case 11:  {
        label = 220;
        break;
       }
       case 13:  {
        $794 = $bits;
        $795 = $794 & 7;
        $796 = $hold;
        $797 = $796 >>> $795;
        $hold = $797;
        $798 = $bits;
        $799 = $798 & 7;
        $800 = $bits;
        $801 = (($800) - ($799))|0;
        $bits = $801;
        while(1) {
         $802 = $bits;
         $803 = ($802>>>0)<(32);
         if (!($803)) {
          break;
         }
         $804 = $have;
         $805 = ($804|0)==(0);
         if ($805) {
          label = 253;
          break L13;
         }
         $806 = $have;
         $807 = (($806) + -1)|0;
         $have = $807;
         $808 = $next;
         $809 = (($808) + 1|0);
         $next = $809;
         $810 = HEAP8[$808>>0]|0;
         $811 = $810&255;
         $812 = $bits;
         $813 = $811 << $812;
         $814 = $hold;
         $815 = (($814) + ($813))|0;
         $hold = $815;
         $816 = $bits;
         $817 = (($816) + 8)|0;
         $bits = $817;
        }
        $818 = $hold;
        $819 = $818 & 65535;
        $820 = $hold;
        $821 = $820 >>> 16;
        $822 = $821 ^ 65535;
        $823 = ($819|0)!=($822|0);
        if ($823) {
         $824 = $1;
         $825 = (($824) + 24|0);
         HEAP32[$825>>2] = 73376;
         $826 = $state;
         HEAP32[$826>>2] = 27;
         break L15;
        }
        $827 = $hold;
        $828 = $827 & 65535;
        $829 = $state;
        $830 = (($829) + 64|0);
        HEAP32[$830>>2] = $828;
        $hold = 0;
        $bits = 0;
        $831 = $state;
        HEAP32[$831>>2] = 14;
        label = 262;
        break;
       }
       case 14:  {
        label = 262;
        break;
       }
       case 15:  {
        while(1) {
         $868 = $bits;
         $869 = ($868>>>0)<(14);
         if (!($869)) {
          break;
         }
         $870 = $have;
         $871 = ($870|0)==(0);
         if ($871) {
          label = 276;
          break L13;
         }
         $872 = $have;
         $873 = (($872) + -1)|0;
         $have = $873;
         $874 = $next;
         $875 = (($874) + 1|0);
         $next = $875;
         $876 = HEAP8[$874>>0]|0;
         $877 = $876&255;
         $878 = $bits;
         $879 = $877 << $878;
         $880 = $hold;
         $881 = (($880) + ($879))|0;
         $hold = $881;
         $882 = $bits;
         $883 = (($882) + 8)|0;
         $bits = $883;
        }
        $884 = $hold;
        $885 = $884 & 31;
        $886 = (($885) + 257)|0;
        $887 = $state;
        $888 = (($887) + 96|0);
        HEAP32[$888>>2] = $886;
        $889 = $hold;
        $890 = $889 >>> 5;
        $hold = $890;
        $891 = $bits;
        $892 = (($891) - 5)|0;
        $bits = $892;
        $893 = $hold;
        $894 = $893 & 31;
        $895 = (($894) + 1)|0;
        $896 = $state;
        $897 = (($896) + 100|0);
        HEAP32[$897>>2] = $895;
        $898 = $hold;
        $899 = $898 >>> 5;
        $hold = $899;
        $900 = $bits;
        $901 = (($900) - 5)|0;
        $bits = $901;
        $902 = $hold;
        $903 = $902 & 15;
        $904 = (($903) + 4)|0;
        $905 = $state;
        $906 = (($905) + 92|0);
        HEAP32[$906>>2] = $904;
        $907 = $hold;
        $908 = $907 >>> 4;
        $hold = $908;
        $909 = $bits;
        $910 = (($909) - 4)|0;
        $bits = $910;
        $911 = $state;
        $912 = (($911) + 96|0);
        $913 = HEAP32[$912>>2]|0;
        $914 = ($913>>>0)>(286);
        if (!($914)) {
         $915 = $state;
         $916 = (($915) + 100|0);
         $917 = HEAP32[$916>>2]|0;
         $918 = ($917>>>0)>(30);
         if (!($918)) {
          $922 = $state;
          $923 = (($922) + 104|0);
          HEAP32[$923>>2] = 0;
          $924 = $state;
          HEAP32[$924>>2] = 16;
          label = 290;
          break L15;
         }
        }
        $919 = $1;
        $920 = (($919) + 24|0);
        HEAP32[$920>>2] = 73408;
        $921 = $state;
        HEAP32[$921>>2] = 27;
        break;
       }
       case 16:  {
        label = 290;
        break;
       }
       case 17:  {
        label = 310;
        break;
       }
       case 18:  {
        label = 394;
        break;
       }
       case 19:  {
        label = 431;
        break;
       }
       case 20:  {
        label = 445;
        break;
       }
       case 21:  {
        label = 470;
        break;
       }
       case 22:  {
        label = 486;
        break;
       }
       case 23:  {
        $1788 = $left;
        $1789 = ($1788|0)==(0);
        if ($1789) {
         label = 505;
         break L13;
        }
        $1790 = $state;
        $1791 = (($1790) + 64|0);
        $1792 = HEAP32[$1791>>2]|0;
        $1793 = $1792&255;
        $1794 = $put;
        $1795 = (($1794) + 1|0);
        $put = $1795;
        HEAP8[$1794>>0] = $1793;
        $1796 = $left;
        $1797 = (($1796) + -1)|0;
        $left = $1797;
        $1798 = $state;
        HEAP32[$1798>>2] = 18;
        break;
       }
       case 24:  {
        $1799 = $state;
        $1800 = (($1799) + 8|0);
        $1801 = HEAP32[$1800>>2]|0;
        $1802 = ($1801|0)!=(0);
        if ($1802) {
         while(1) {
          $1803 = $bits;
          $1804 = ($1803>>>0)<(32);
          if (!($1804)) {
           break;
          }
          $1805 = $have;
          $1806 = ($1805|0)==(0);
          if ($1806) {
           label = 513;
           break L13;
          }
          $1807 = $have;
          $1808 = (($1807) + -1)|0;
          $have = $1808;
          $1809 = $next;
          $1810 = (($1809) + 1|0);
          $next = $1810;
          $1811 = HEAP8[$1809>>0]|0;
          $1812 = $1811&255;
          $1813 = $bits;
          $1814 = $1812 << $1813;
          $1815 = $hold;
          $1816 = (($1815) + ($1814))|0;
          $hold = $1816;
          $1817 = $bits;
          $1818 = (($1817) + 8)|0;
          $bits = $1818;
         }
         $1819 = $left;
         $1820 = $out;
         $1821 = (($1820) - ($1819))|0;
         $out = $1821;
         $1822 = $out;
         $1823 = $1;
         $1824 = (($1823) + 20|0);
         $1825 = HEAP32[$1824>>2]|0;
         $1826 = (($1825) + ($1822))|0;
         HEAP32[$1824>>2] = $1826;
         $1827 = $out;
         $1828 = $state;
         $1829 = (($1828) + 28|0);
         $1830 = HEAP32[$1829>>2]|0;
         $1831 = (($1830) + ($1827))|0;
         HEAP32[$1829>>2] = $1831;
         $1832 = $out;
         $1833 = ($1832|0)!=(0);
         if ($1833) {
          $1834 = $state;
          $1835 = (($1834) + 16|0);
          $1836 = HEAP32[$1835>>2]|0;
          $1837 = ($1836|0)!=(0);
          if ($1837) {
           $1838 = $state;
           $1839 = (($1838) + 24|0);
           $1840 = HEAP32[$1839>>2]|0;
           $1841 = $put;
           $1842 = $out;
           $1843 = (0 - ($1842))|0;
           $1844 = (($1841) + ($1843)|0);
           $1845 = $out;
           $1846 = (_crc32($1840,$1844,$1845)|0);
           $1858 = $1846;
          } else {
           $1847 = $state;
           $1848 = (($1847) + 24|0);
           $1849 = HEAP32[$1848>>2]|0;
           $1850 = $put;
           $1851 = $out;
           $1852 = (0 - ($1851))|0;
           $1853 = (($1850) + ($1852)|0);
           $1854 = $out;
           $1855 = (_adler32($1849,$1853,$1854)|0);
           $1858 = $1855;
          }
          $1856 = $state;
          $1857 = (($1856) + 24|0);
          HEAP32[$1857>>2] = $1858;
          $1859 = $1;
          $1860 = (($1859) + 48|0);
          HEAP32[$1860>>2] = $1858;
         }
         $1861 = $left;
         $out = $1861;
         $1862 = $state;
         $1863 = (($1862) + 16|0);
         $1864 = HEAP32[$1863>>2]|0;
         $1865 = ($1864|0)!=(0);
         if ($1865) {
          $1866 = $hold;
          $1886 = $1866;
         } else {
          $1867 = $hold;
          $1868 = $1867 >>> 24;
          $1869 = $1868 & 255;
          $1870 = $hold;
          $1871 = $1870 >>> 8;
          $1872 = $1871 & 65280;
          $1873 = (($1869) + ($1872))|0;
          $1874 = $hold;
          $1875 = $1874 & 65280;
          $1876 = $1875 << 8;
          $1877 = (($1873) + ($1876))|0;
          $1878 = $hold;
          $1879 = $1878 & 255;
          $1880 = $1879 << 24;
          $1881 = (($1877) + ($1880))|0;
          $1886 = $1881;
         }
         $1882 = $state;
         $1883 = (($1882) + 24|0);
         $1884 = HEAP32[$1883>>2]|0;
         $1885 = ($1886|0)!=($1884|0);
         if ($1885) {
          $1887 = $1;
          $1888 = (($1887) + 24|0);
          HEAP32[$1888>>2] = 73656;
          $1889 = $state;
          HEAP32[$1889>>2] = 27;
          break L15;
         }
         $hold = 0;
         $bits = 0;
        }
        $1890 = $state;
        HEAP32[$1890>>2] = 25;
        label = 531;
        break;
       }
       case 25:  {
        label = 531;
        break;
       }
       case 26:  {
        label = 548;
        break L13;
        break;
       }
       case 27:  {
        label = 549;
        break L13;
        break;
       }
       case 28:  {
        label = 550;
        break L13;
        break;
       }
       case 29:  {
        label = 551;
        break L13;
        break;
       }
       case 7:  {
        label = 164;
        break;
       }
       case 8:  {
        label = 185;
        break;
       }
       default: {
        label = 552;
        break L13;
       }
       }
      } while(0);
      do {
       if ((label|0) == 67) {
        label = 0;
        while(1) {
         $216 = $bits;
         $217 = ($216>>>0)<(32);
         if (!($217)) {
          break;
         }
         $218 = $have;
         $219 = ($218|0)==(0);
         if ($219) {
          label = 72;
          break L13;
         }
         $220 = $have;
         $221 = (($220) + -1)|0;
         $have = $221;
         $222 = $next;
         $223 = (($222) + 1|0);
         $next = $223;
         $224 = HEAP8[$222>>0]|0;
         $225 = $224&255;
         $226 = $bits;
         $227 = $225 << $226;
         $228 = $hold;
         $229 = (($228) + ($227))|0;
         $hold = $229;
         $230 = $bits;
         $231 = (($230) + 8)|0;
         $bits = $231;
        }
        $232 = $state;
        $233 = (($232) + 32|0);
        $234 = HEAP32[$233>>2]|0;
        $235 = ($234|0)!=(0|0);
        if ($235) {
         $236 = $hold;
         $237 = $state;
         $238 = (($237) + 32|0);
         $239 = HEAP32[$238>>2]|0;
         $240 = (($239) + 4|0);
         HEAP32[$240>>2] = $236;
        }
        $241 = $state;
        $242 = (($241) + 16|0);
        $243 = HEAP32[$242>>2]|0;
        $244 = $243 & 512;
        $245 = ($244|0)!=(0);
        if ($245) {
         $246 = $hold;
         $247 = $246&255;
         HEAP8[$hbuf>>0] = $247;
         $248 = $hold;
         $249 = $248 >>> 8;
         $250 = $249&255;
         $251 = (($hbuf) + 1|0);
         HEAP8[$251>>0] = $250;
         $252 = $hold;
         $253 = $252 >>> 16;
         $254 = $253&255;
         $255 = (($hbuf) + 2|0);
         HEAP8[$255>>0] = $254;
         $256 = $hold;
         $257 = $256 >>> 24;
         $258 = $257&255;
         $259 = (($hbuf) + 3|0);
         HEAP8[$259>>0] = $258;
         $260 = $state;
         $261 = (($260) + 24|0);
         $262 = HEAP32[$261>>2]|0;
         $263 = (_crc32($262,$hbuf,4)|0);
         $264 = $state;
         $265 = (($264) + 24|0);
         HEAP32[$265>>2] = $263;
        }
        $hold = 0;
        $bits = 0;
        $266 = $state;
        HEAP32[$266>>2] = 3;
        label = 85;
       }
       else if ((label|0) == 215) {
        label = 0;
        $715 = $state;
        $716 = (($715) + 12|0);
        $717 = HEAP32[$716>>2]|0;
        $718 = ($717|0)==(0);
        if ($718) {
         label = 216;
         break L13;
        }
        $736 = (_adler32(0,0,0)|0);
        $737 = $state;
        $738 = (($737) + 24|0);
        HEAP32[$738>>2] = $736;
        $739 = $1;
        $740 = (($739) + 48|0);
        HEAP32[$740>>2] = $736;
        $741 = $state;
        HEAP32[$741>>2] = 11;
        label = 220;
       }
       else if ((label|0) == 262) {
        label = 0;
        $832 = $state;
        $833 = (($832) + 64|0);
        $834 = HEAP32[$833>>2]|0;
        $copy = $834;
        $835 = $copy;
        $836 = ($835|0)!=(0);
        if (!($836)) {
         $867 = $state;
         HEAP32[$867>>2] = 11;
         break;
        }
        $837 = $copy;
        $838 = $have;
        $839 = ($837>>>0)>($838>>>0);
        if ($839) {
         $840 = $have;
         $copy = $840;
        }
        $841 = $copy;
        $842 = $left;
        $843 = ($841>>>0)>($842>>>0);
        if ($843) {
         $844 = $left;
         $copy = $844;
        }
        $845 = $copy;
        $846 = ($845|0)==(0);
        if ($846) {
         label = 268;
         break L13;
        }
        $847 = $put;
        $848 = $next;
        $849 = $copy;
        _memcpy(($847|0),($848|0),($849|0))|0;
        $850 = $copy;
        $851 = $have;
        $852 = (($851) - ($850))|0;
        $have = $852;
        $853 = $copy;
        $854 = $next;
        $855 = (($854) + ($853)|0);
        $next = $855;
        $856 = $copy;
        $857 = $left;
        $858 = (($857) - ($856))|0;
        $left = $858;
        $859 = $copy;
        $860 = $put;
        $861 = (($860) + ($859)|0);
        $put = $861;
        $862 = $copy;
        $863 = $state;
        $864 = (($863) + 64|0);
        $865 = HEAP32[$864>>2]|0;
        $866 = (($865) - ($862))|0;
        HEAP32[$864>>2] = $866;
       }
       else if ((label|0) == 290) {
        label = 0;
        while(1) {
         $925 = $state;
         $926 = (($925) + 104|0);
         $927 = HEAP32[$926>>2]|0;
         $928 = $state;
         $929 = (($928) + 92|0);
         $930 = HEAP32[$929>>2]|0;
         $931 = ($927>>>0)<($930>>>0);
         if (!($931)) {
          break;
         }
         while(1) {
          $932 = $bits;
          $933 = ($932>>>0)<(3);
          if (!($933)) {
           break;
          }
          $934 = $have;
          $935 = ($934|0)==(0);
          if ($935) {
           label = 297;
           break L13;
          }
          $936 = $have;
          $937 = (($936) + -1)|0;
          $have = $937;
          $938 = $next;
          $939 = (($938) + 1|0);
          $next = $939;
          $940 = HEAP8[$938>>0]|0;
          $941 = $940&255;
          $942 = $bits;
          $943 = $941 << $942;
          $944 = $hold;
          $945 = (($944) + ($943))|0;
          $hold = $945;
          $946 = $bits;
          $947 = (($946) + 8)|0;
          $bits = $947;
         }
         $948 = $hold;
         $949 = $948 & 7;
         $950 = $949&65535;
         $951 = $state;
         $952 = (($951) + 104|0);
         $953 = HEAP32[$952>>2]|0;
         $954 = (($953) + 1)|0;
         HEAP32[$952>>2] = $954;
         $955 = (73176 + ($953<<1)|0);
         $956 = HEAP16[$955>>1]|0;
         $957 = $956&65535;
         $958 = $state;
         $959 = (($958) + 112|0);
         $960 = (($959) + ($957<<1)|0);
         HEAP16[$960>>1] = $950;
         $961 = $hold;
         $962 = $961 >>> 3;
         $hold = $962;
         $963 = $bits;
         $964 = (($963) - 3)|0;
         $bits = $964;
        }
        while(1) {
         $965 = $state;
         $966 = (($965) + 104|0);
         $967 = HEAP32[$966>>2]|0;
         $968 = ($967>>>0)<(19);
         if (!($968)) {
          break;
         }
         $969 = $state;
         $970 = (($969) + 104|0);
         $971 = HEAP32[$970>>2]|0;
         $972 = (($971) + 1)|0;
         HEAP32[$970>>2] = $972;
         $973 = (73176 + ($971<<1)|0);
         $974 = HEAP16[$973>>1]|0;
         $975 = $974&65535;
         $976 = $state;
         $977 = (($976) + 112|0);
         $978 = (($977) + ($975<<1)|0);
         HEAP16[$978>>1] = 0;
        }
        $979 = $state;
        $980 = (($979) + 1328|0);
        $981 = $state;
        $982 = (($981) + 108|0);
        HEAP32[$982>>2] = $980;
        $983 = $state;
        $984 = (($983) + 108|0);
        $985 = HEAP32[$984>>2]|0;
        $986 = $state;
        $987 = (($986) + 76|0);
        HEAP32[$987>>2] = $985;
        $988 = $state;
        $989 = (($988) + 84|0);
        HEAP32[$989>>2] = 7;
        $990 = $state;
        $991 = (($990) + 112|0);
        $992 = $state;
        $993 = (($992) + 108|0);
        $994 = $state;
        $995 = (($994) + 84|0);
        $996 = $state;
        $997 = (($996) + 752|0);
        $998 = (_inflate_table(0,$991,19,$993,$995,$997)|0);
        $ret = $998;
        $999 = $ret;
        $1000 = ($999|0)!=(0);
        if ($1000) {
         $1001 = $1;
         $1002 = (($1001) + 24|0);
         HEAP32[$1002>>2] = 73448;
         $1003 = $state;
         HEAP32[$1003>>2] = 27;
         break;
        } else {
         $1004 = $state;
         $1005 = (($1004) + 104|0);
         HEAP32[$1005>>2] = 0;
         $1006 = $state;
         HEAP32[$1006>>2] = 17;
         label = 310;
         break;
        }
       }
       else if ((label|0) == 531) {
        label = 0;
        $1891 = $state;
        $1892 = (($1891) + 8|0);
        $1893 = HEAP32[$1892>>2]|0;
        $1894 = ($1893|0)!=(0);
        if (!($1894)) {
         label = 547;
         break L13;
        }
        $1895 = $state;
        $1896 = (($1895) + 16|0);
        $1897 = HEAP32[$1896>>2]|0;
        $1898 = ($1897|0)!=(0);
        if (!($1898)) {
         label = 547;
         break L13;
        }
        while(1) {
         $1899 = $bits;
         $1900 = ($1899>>>0)<(32);
         if (!($1900)) {
          break;
         }
         $1901 = $have;
         $1902 = ($1901|0)==(0);
         if ($1902) {
          label = 538;
          break L13;
         }
         $1903 = $have;
         $1904 = (($1903) + -1)|0;
         $have = $1904;
         $1905 = $next;
         $1906 = (($1905) + 1|0);
         $next = $1906;
         $1907 = HEAP8[$1905>>0]|0;
         $1908 = $1907&255;
         $1909 = $bits;
         $1910 = $1908 << $1909;
         $1911 = $hold;
         $1912 = (($1911) + ($1910))|0;
         $hold = $1912;
         $1913 = $bits;
         $1914 = (($1913) + 8)|0;
         $bits = $1914;
        }
        $1915 = $hold;
        $1916 = $state;
        $1917 = (($1916) + 28|0);
        $1918 = HEAP32[$1917>>2]|0;
        $1919 = ($1915|0)!=($1918|0);
        if (!($1919)) {
         label = 544;
         break L13;
        }
        $1920 = $1;
        $1921 = (($1920) + 24|0);
        HEAP32[$1921>>2] = 73680;
        $1922 = $state;
        HEAP32[$1922>>2] = 27;
       }
      } while(0);
      do {
       if ((label|0) == 85) {
        label = 0;
        while(1) {
         $267 = $bits;
         $268 = ($267>>>0)<(16);
         if (!($268)) {
          break;
         }
         $269 = $have;
         $270 = ($269|0)==(0);
         if ($270) {
          label = 90;
          break L13;
         }
         $271 = $have;
         $272 = (($271) + -1)|0;
         $have = $272;
         $273 = $next;
         $274 = (($273) + 1|0);
         $next = $274;
         $275 = HEAP8[$273>>0]|0;
         $276 = $275&255;
         $277 = $bits;
         $278 = $276 << $277;
         $279 = $hold;
         $280 = (($279) + ($278))|0;
         $hold = $280;
         $281 = $bits;
         $282 = (($281) + 8)|0;
         $bits = $282;
        }
        $283 = $state;
        $284 = (($283) + 32|0);
        $285 = HEAP32[$284>>2]|0;
        $286 = ($285|0)!=(0|0);
        if ($286) {
         $287 = $hold;
         $288 = $287 & 255;
         $289 = $state;
         $290 = (($289) + 32|0);
         $291 = HEAP32[$290>>2]|0;
         $292 = (($291) + 8|0);
         HEAP32[$292>>2] = $288;
         $293 = $hold;
         $294 = $293 >>> 8;
         $295 = $state;
         $296 = (($295) + 32|0);
         $297 = HEAP32[$296>>2]|0;
         $298 = (($297) + 12|0);
         HEAP32[$298>>2] = $294;
        }
        $299 = $state;
        $300 = (($299) + 16|0);
        $301 = HEAP32[$300>>2]|0;
        $302 = $301 & 512;
        $303 = ($302|0)!=(0);
        if ($303) {
         $304 = $hold;
         $305 = $304&255;
         HEAP8[$hbuf>>0] = $305;
         $306 = $hold;
         $307 = $306 >>> 8;
         $308 = $307&255;
         $309 = (($hbuf) + 1|0);
         HEAP8[$309>>0] = $308;
         $310 = $state;
         $311 = (($310) + 24|0);
         $312 = HEAP32[$311>>2]|0;
         $313 = (_crc32($312,$hbuf,2)|0);
         $314 = $state;
         $315 = (($314) + 24|0);
         HEAP32[$315>>2] = $313;
        }
        $hold = 0;
        $bits = 0;
        $316 = $state;
        HEAP32[$316>>2] = 4;
        label = 103;
       }
       else if ((label|0) == 220) {
        label = 0;
        $742 = $2;
        $743 = ($742|0)==(5);
        if ($743) {
         label = 221;
         break L13;
        }
        label = 223;
       }
       else if ((label|0) == 310) {
        label = 0;
        while(1) {
         $1007 = $state;
         $1008 = (($1007) + 104|0);
         $1009 = HEAP32[$1008>>2]|0;
         $1010 = $state;
         $1011 = (($1010) + 96|0);
         $1012 = HEAP32[$1011>>2]|0;
         $1013 = $state;
         $1014 = (($1013) + 100|0);
         $1015 = HEAP32[$1014>>2]|0;
         $1016 = (($1012) + ($1015))|0;
         $1017 = ($1009>>>0)<($1016>>>0);
         if (!($1017)) {
          break;
         }
         while(1) {
          $1018 = $hold;
          $1019 = $state;
          $1020 = (($1019) + 84|0);
          $1021 = HEAP32[$1020>>2]|0;
          $1022 = 1 << $1021;
          $1023 = (($1022) - 1)|0;
          $1024 = $1018 & $1023;
          $1025 = $state;
          $1026 = (($1025) + 76|0);
          $1027 = HEAP32[$1026>>2]|0;
          $1028 = (($1027) + ($1024<<2)|0);
          ;HEAP16[$this+0>>1]=HEAP16[$1028+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$1028+2>>1]|0;
          $1029 = (($this) + 1|0);
          $1030 = HEAP8[$1029>>0]|0;
          $1031 = $1030&255;
          $1032 = $bits;
          $1033 = ($1031>>>0)<=($1032>>>0);
          if ($1033) {
           break;
          }
          $1034 = $have;
          $1035 = ($1034|0)==(0);
          if ($1035) {
           label = 317;
           break L13;
          }
          $1036 = $have;
          $1037 = (($1036) + -1)|0;
          $have = $1037;
          $1038 = $next;
          $1039 = (($1038) + 1|0);
          $next = $1039;
          $1040 = HEAP8[$1038>>0]|0;
          $1041 = $1040&255;
          $1042 = $bits;
          $1043 = $1041 << $1042;
          $1044 = $hold;
          $1045 = (($1044) + ($1043))|0;
          $hold = $1045;
          $1046 = $bits;
          $1047 = (($1046) + 8)|0;
          $bits = $1047;
         }
         $1048 = (($this) + 2|0);
         $1049 = HEAP16[$1048>>1]|0;
         $1050 = $1049&65535;
         $1051 = ($1050|0)<(16);
         if ($1051) {
          while(1) {
           $1052 = $bits;
           $1053 = (($this) + 1|0);
           $1054 = HEAP8[$1053>>0]|0;
           $1055 = $1054&255;
           $1056 = ($1052>>>0)<($1055>>>0);
           if (!($1056)) {
            break;
           }
           $1057 = $have;
           $1058 = ($1057|0)==(0);
           if ($1058) {
            label = 326;
            break L13;
           }
           $1059 = $have;
           $1060 = (($1059) + -1)|0;
           $have = $1060;
           $1061 = $next;
           $1062 = (($1061) + 1|0);
           $next = $1062;
           $1063 = HEAP8[$1061>>0]|0;
           $1064 = $1063&255;
           $1065 = $bits;
           $1066 = $1064 << $1065;
           $1067 = $hold;
           $1068 = (($1067) + ($1066))|0;
           $hold = $1068;
           $1069 = $bits;
           $1070 = (($1069) + 8)|0;
           $bits = $1070;
          }
          $1071 = (($this) + 1|0);
          $1072 = HEAP8[$1071>>0]|0;
          $1073 = $1072&255;
          $1074 = $hold;
          $1075 = $1074 >>> $1073;
          $hold = $1075;
          $1076 = (($this) + 1|0);
          $1077 = HEAP8[$1076>>0]|0;
          $1078 = $1077&255;
          $1079 = $bits;
          $1080 = (($1079) - ($1078))|0;
          $bits = $1080;
          $1081 = (($this) + 2|0);
          $1082 = HEAP16[$1081>>1]|0;
          $1083 = $state;
          $1084 = (($1083) + 104|0);
          $1085 = HEAP32[$1084>>2]|0;
          $1086 = (($1085) + 1)|0;
          HEAP32[$1084>>2] = $1086;
          $1087 = $state;
          $1088 = (($1087) + 112|0);
          $1089 = (($1088) + ($1085<<1)|0);
          HEAP16[$1089>>1] = $1082;
         } else {
          $1090 = (($this) + 2|0);
          $1091 = HEAP16[$1090>>1]|0;
          $1092 = $1091&65535;
          $1093 = ($1092|0)==(16);
          if ($1093) {
           while(1) {
            $1094 = $bits;
            $1095 = (($this) + 1|0);
            $1096 = HEAP8[$1095>>0]|0;
            $1097 = $1096&255;
            $1098 = (($1097) + 2)|0;
            $1099 = ($1094>>>0)<($1098>>>0);
            if (!($1099)) {
             break;
            }
            $1100 = $have;
            $1101 = ($1100|0)==(0);
            if ($1101) {
             label = 339;
             break L13;
            }
            $1102 = $have;
            $1103 = (($1102) + -1)|0;
            $have = $1103;
            $1104 = $next;
            $1105 = (($1104) + 1|0);
            $next = $1105;
            $1106 = HEAP8[$1104>>0]|0;
            $1107 = $1106&255;
            $1108 = $bits;
            $1109 = $1107 << $1108;
            $1110 = $hold;
            $1111 = (($1110) + ($1109))|0;
            $hold = $1111;
            $1112 = $bits;
            $1113 = (($1112) + 8)|0;
            $bits = $1113;
           }
           $1114 = (($this) + 1|0);
           $1115 = HEAP8[$1114>>0]|0;
           $1116 = $1115&255;
           $1117 = $hold;
           $1118 = $1117 >>> $1116;
           $hold = $1118;
           $1119 = (($this) + 1|0);
           $1120 = HEAP8[$1119>>0]|0;
           $1121 = $1120&255;
           $1122 = $bits;
           $1123 = (($1122) - ($1121))|0;
           $bits = $1123;
           $1124 = $state;
           $1125 = (($1124) + 104|0);
           $1126 = HEAP32[$1125>>2]|0;
           $1127 = ($1126|0)==(0);
           if ($1127) {
            label = 346;
            break;
           }
           $1131 = $state;
           $1132 = (($1131) + 104|0);
           $1133 = HEAP32[$1132>>2]|0;
           $1134 = (($1133) - 1)|0;
           $1135 = $state;
           $1136 = (($1135) + 112|0);
           $1137 = (($1136) + ($1134<<1)|0);
           $1138 = HEAP16[$1137>>1]|0;
           $1139 = $1138&65535;
           $len = $1139;
           $1140 = $hold;
           $1141 = $1140 & 3;
           $1142 = (3 + ($1141))|0;
           $copy = $1142;
           $1143 = $hold;
           $1144 = $1143 >>> 2;
           $hold = $1144;
           $1145 = $bits;
           $1146 = (($1145) - 2)|0;
           $bits = $1146;
          } else {
           $1147 = (($this) + 2|0);
           $1148 = HEAP16[$1147>>1]|0;
           $1149 = $1148&65535;
           $1150 = ($1149|0)==(17);
           if ($1150) {
            while(1) {
             $1151 = $bits;
             $1152 = (($this) + 1|0);
             $1153 = HEAP8[$1152>>0]|0;
             $1154 = $1153&255;
             $1155 = (($1154) + 3)|0;
             $1156 = ($1151>>>0)<($1155>>>0);
             if (!($1156)) {
              break;
             }
             $1157 = $have;
             $1158 = ($1157|0)==(0);
             if ($1158) {
              label = 356;
              break L13;
             }
             $1159 = $have;
             $1160 = (($1159) + -1)|0;
             $have = $1160;
             $1161 = $next;
             $1162 = (($1161) + 1|0);
             $next = $1162;
             $1163 = HEAP8[$1161>>0]|0;
             $1164 = $1163&255;
             $1165 = $bits;
             $1166 = $1164 << $1165;
             $1167 = $hold;
             $1168 = (($1167) + ($1166))|0;
             $hold = $1168;
             $1169 = $bits;
             $1170 = (($1169) + 8)|0;
             $bits = $1170;
            }
            $1171 = (($this) + 1|0);
            $1172 = HEAP8[$1171>>0]|0;
            $1173 = $1172&255;
            $1174 = $hold;
            $1175 = $1174 >>> $1173;
            $hold = $1175;
            $1176 = (($this) + 1|0);
            $1177 = HEAP8[$1176>>0]|0;
            $1178 = $1177&255;
            $1179 = $bits;
            $1180 = (($1179) - ($1178))|0;
            $bits = $1180;
            $len = 0;
            $1181 = $hold;
            $1182 = $1181 & 7;
            $1183 = (3 + ($1182))|0;
            $copy = $1183;
            $1184 = $hold;
            $1185 = $1184 >>> 3;
            $hold = $1185;
            $1186 = $bits;
            $1187 = (($1186) - 3)|0;
            $bits = $1187;
           } else {
            while(1) {
             $1188 = $bits;
             $1189 = (($this) + 1|0);
             $1190 = HEAP8[$1189>>0]|0;
             $1191 = $1190&255;
             $1192 = (($1191) + 7)|0;
             $1193 = ($1188>>>0)<($1192>>>0);
             if (!($1193)) {
              break;
             }
             $1194 = $have;
             $1195 = ($1194|0)==(0);
             if ($1195) {
              label = 370;
              break L13;
             }
             $1196 = $have;
             $1197 = (($1196) + -1)|0;
             $have = $1197;
             $1198 = $next;
             $1199 = (($1198) + 1|0);
             $next = $1199;
             $1200 = HEAP8[$1198>>0]|0;
             $1201 = $1200&255;
             $1202 = $bits;
             $1203 = $1201 << $1202;
             $1204 = $hold;
             $1205 = (($1204) + ($1203))|0;
             $hold = $1205;
             $1206 = $bits;
             $1207 = (($1206) + 8)|0;
             $bits = $1207;
            }
            $1208 = (($this) + 1|0);
            $1209 = HEAP8[$1208>>0]|0;
            $1210 = $1209&255;
            $1211 = $hold;
            $1212 = $1211 >>> $1210;
            $hold = $1212;
            $1213 = (($this) + 1|0);
            $1214 = HEAP8[$1213>>0]|0;
            $1215 = $1214&255;
            $1216 = $bits;
            $1217 = (($1216) - ($1215))|0;
            $bits = $1217;
            $len = 0;
            $1218 = $hold;
            $1219 = $1218 & 127;
            $1220 = (11 + ($1219))|0;
            $copy = $1220;
            $1221 = $hold;
            $1222 = $1221 >>> 7;
            $hold = $1222;
            $1223 = $bits;
            $1224 = (($1223) - 7)|0;
            $bits = $1224;
           }
          }
          $1225 = $state;
          $1226 = (($1225) + 104|0);
          $1227 = HEAP32[$1226>>2]|0;
          $1228 = $copy;
          $1229 = (($1227) + ($1228))|0;
          $1230 = $state;
          $1231 = (($1230) + 96|0);
          $1232 = HEAP32[$1231>>2]|0;
          $1233 = $state;
          $1234 = (($1233) + 100|0);
          $1235 = HEAP32[$1234>>2]|0;
          $1236 = (($1232) + ($1235))|0;
          $1237 = ($1229>>>0)>($1236>>>0);
          if ($1237) {
           label = 381;
           break;
          }
          while(1) {
           $1241 = $copy;
           $1242 = (($1241) + -1)|0;
           $copy = $1242;
           $1243 = ($1241|0)!=(0);
           if (!($1243)) {
            break;
           }
           $1244 = $len;
           $1245 = $1244&65535;
           $1246 = $state;
           $1247 = (($1246) + 104|0);
           $1248 = HEAP32[$1247>>2]|0;
           $1249 = (($1248) + 1)|0;
           HEAP32[$1247>>2] = $1249;
           $1250 = $state;
           $1251 = (($1250) + 112|0);
           $1252 = (($1251) + ($1248<<1)|0);
           HEAP16[$1252>>1] = $1245;
          }
         }
        }
        if ((label|0) == 346) {
         label = 0;
         $1128 = $1;
         $1129 = (($1128) + 24|0);
         HEAP32[$1129>>2] = 73480;
         $1130 = $state;
         HEAP32[$1130>>2] = 27;
        }
        else if ((label|0) == 381) {
         label = 0;
         $1238 = $1;
         $1239 = (($1238) + 24|0);
         HEAP32[$1239>>2] = 73480;
         $1240 = $state;
         HEAP32[$1240>>2] = 27;
        }
        $1253 = $state;
        $1254 = HEAP32[$1253>>2]|0;
        $1255 = ($1254|0)==(27);
        if ($1255) {
         break;
        }
        $1256 = $state;
        $1257 = (($1256) + 1328|0);
        $1258 = $state;
        $1259 = (($1258) + 108|0);
        HEAP32[$1259>>2] = $1257;
        $1260 = $state;
        $1261 = (($1260) + 108|0);
        $1262 = HEAP32[$1261>>2]|0;
        $1263 = $state;
        $1264 = (($1263) + 76|0);
        HEAP32[$1264>>2] = $1262;
        $1265 = $state;
        $1266 = (($1265) + 84|0);
        HEAP32[$1266>>2] = 9;
        $1267 = $state;
        $1268 = (($1267) + 112|0);
        $1269 = $state;
        $1270 = (($1269) + 96|0);
        $1271 = HEAP32[$1270>>2]|0;
        $1272 = $state;
        $1273 = (($1272) + 108|0);
        $1274 = $state;
        $1275 = (($1274) + 84|0);
        $1276 = $state;
        $1277 = (($1276) + 752|0);
        $1278 = (_inflate_table(1,$1268,$1271,$1273,$1275,$1277)|0);
        $ret = $1278;
        $1279 = $ret;
        $1280 = ($1279|0)!=(0);
        if ($1280) {
         $1281 = $1;
         $1282 = (($1281) + 24|0);
         HEAP32[$1282>>2] = 73512;
         $1283 = $state;
         HEAP32[$1283>>2] = 27;
         break;
        }
        $1284 = $state;
        $1285 = (($1284) + 108|0);
        $1286 = HEAP32[$1285>>2]|0;
        $1287 = $state;
        $1288 = (($1287) + 80|0);
        HEAP32[$1288>>2] = $1286;
        $1289 = $state;
        $1290 = (($1289) + 88|0);
        HEAP32[$1290>>2] = 6;
        $1291 = $state;
        $1292 = (($1291) + 112|0);
        $1293 = $state;
        $1294 = (($1293) + 96|0);
        $1295 = HEAP32[$1294>>2]|0;
        $1296 = (($1292) + ($1295<<1)|0);
        $1297 = $state;
        $1298 = (($1297) + 100|0);
        $1299 = HEAP32[$1298>>2]|0;
        $1300 = $state;
        $1301 = (($1300) + 108|0);
        $1302 = $state;
        $1303 = (($1302) + 88|0);
        $1304 = $state;
        $1305 = (($1304) + 752|0);
        $1306 = (_inflate_table(2,$1296,$1299,$1301,$1303,$1305)|0);
        $ret = $1306;
        $1307 = $ret;
        $1308 = ($1307|0)!=(0);
        if ($1308) {
         $1309 = $1;
         $1310 = (($1309) + 24|0);
         HEAP32[$1310>>2] = 73544;
         $1311 = $state;
         HEAP32[$1311>>2] = 27;
         break;
        } else {
         $1312 = $state;
         HEAP32[$1312>>2] = 18;
         label = 394;
         break;
        }
       }
      } while(0);
      do {
       if ((label|0) == 103) {
        label = 0;
        $317 = $state;
        $318 = (($317) + 16|0);
        $319 = HEAP32[$318>>2]|0;
        $320 = $319 & 1024;
        $321 = ($320|0)!=(0);
        if ($321) {
         while(1) {
          $322 = $bits;
          $323 = ($322>>>0)<(16);
          if (!($323)) {
           break;
          }
          $324 = $have;
          $325 = ($324|0)==(0);
          if ($325) {
           label = 109;
           break L13;
          }
          $326 = $have;
          $327 = (($326) + -1)|0;
          $have = $327;
          $328 = $next;
          $329 = (($328) + 1|0);
          $next = $329;
          $330 = HEAP8[$328>>0]|0;
          $331 = $330&255;
          $332 = $bits;
          $333 = $331 << $332;
          $334 = $hold;
          $335 = (($334) + ($333))|0;
          $hold = $335;
          $336 = $bits;
          $337 = (($336) + 8)|0;
          $bits = $337;
         }
         $338 = $hold;
         $339 = $state;
         $340 = (($339) + 64|0);
         HEAP32[$340>>2] = $338;
         $341 = $state;
         $342 = (($341) + 32|0);
         $343 = HEAP32[$342>>2]|0;
         $344 = ($343|0)!=(0|0);
         if ($344) {
          $345 = $hold;
          $346 = $state;
          $347 = (($346) + 32|0);
          $348 = HEAP32[$347>>2]|0;
          $349 = (($348) + 20|0);
          HEAP32[$349>>2] = $345;
         }
         $350 = $state;
         $351 = (($350) + 16|0);
         $352 = HEAP32[$351>>2]|0;
         $353 = $352 & 512;
         $354 = ($353|0)!=(0);
         if ($354) {
          $355 = $hold;
          $356 = $355&255;
          HEAP8[$hbuf>>0] = $356;
          $357 = $hold;
          $358 = $357 >>> 8;
          $359 = $358&255;
          $360 = (($hbuf) + 1|0);
          HEAP8[$360>>0] = $359;
          $361 = $state;
          $362 = (($361) + 24|0);
          $363 = HEAP32[$362>>2]|0;
          $364 = (_crc32($363,$hbuf,2)|0);
          $365 = $state;
          $366 = (($365) + 24|0);
          HEAP32[$366>>2] = $364;
         }
         $hold = 0;
         $bits = 0;
        } else {
         $367 = $state;
         $368 = (($367) + 32|0);
         $369 = HEAP32[$368>>2]|0;
         $370 = ($369|0)!=(0|0);
         if ($370) {
          $371 = $state;
          $372 = (($371) + 32|0);
          $373 = HEAP32[$372>>2]|0;
          $374 = (($373) + 16|0);
          HEAP32[$374>>2] = 0;
         }
        }
        $375 = $state;
        HEAP32[$375>>2] = 5;
        label = 126;
       }
       else if ((label|0) == 223) {
        label = 0;
        $744 = $state;
        $745 = (($744) + 4|0);
        $746 = HEAP32[$745>>2]|0;
        $747 = ($746|0)!=(0);
        if ($747) {
         $748 = $bits;
         $749 = $748 & 7;
         $750 = $hold;
         $751 = $750 >>> $749;
         $hold = $751;
         $752 = $bits;
         $753 = $752 & 7;
         $754 = $bits;
         $755 = (($754) - ($753))|0;
         $bits = $755;
         $756 = $state;
         HEAP32[$756>>2] = 24;
         break;
        }
        while(1) {
         $757 = $bits;
         $758 = ($757>>>0)<(3);
         if (!($758)) {
          break;
         }
         $759 = $have;
         $760 = ($759|0)==(0);
         if ($760) {
          label = 232;
          break L13;
         }
         $761 = $have;
         $762 = (($761) + -1)|0;
         $have = $762;
         $763 = $next;
         $764 = (($763) + 1|0);
         $next = $764;
         $765 = HEAP8[$763>>0]|0;
         $766 = $765&255;
         $767 = $bits;
         $768 = $766 << $767;
         $769 = $hold;
         $770 = (($769) + ($768))|0;
         $hold = $770;
         $771 = $bits;
         $772 = (($771) + 8)|0;
         $bits = $772;
        }
        $773 = $hold;
        $774 = $773 & 1;
        $775 = $state;
        $776 = (($775) + 4|0);
        HEAP32[$776>>2] = $774;
        $777 = $hold;
        $778 = $777 >>> 1;
        $hold = $778;
        $779 = $bits;
        $780 = (($779) - 1)|0;
        $bits = $780;
        $781 = $hold;
        $782 = $781 & 3;
        if ((($782|0) == 0)) {
         $783 = $state;
         HEAP32[$783>>2] = 13;
        } else if ((($782|0) == 1)) {
         $784 = $state;
         _fixedtables($784);
         $785 = $state;
         HEAP32[$785>>2] = 18;
        } else if ((($782|0) == 2)) {
         $786 = $state;
         HEAP32[$786>>2] = 15;
        } else if ((($782|0) == 3)) {
         $787 = $1;
         $788 = (($787) + 24|0);
         HEAP32[$788>>2] = 73352;
         $789 = $state;
         HEAP32[$789>>2] = 27;
        }
        $790 = $hold;
        $791 = $790 >>> 2;
        $hold = $791;
        $792 = $bits;
        $793 = (($792) - 2)|0;
        $bits = $793;
       }
       else if ((label|0) == 394) {
        label = 0;
        $1313 = $have;
        $1314 = ($1313>>>0)>=(6);
        if ($1314) {
         $1315 = $left;
         $1316 = ($1315>>>0)>=(258);
         if ($1316) {
          $1317 = $put;
          $1318 = $1;
          $1319 = (($1318) + 12|0);
          HEAP32[$1319>>2] = $1317;
          $1320 = $left;
          $1321 = $1;
          $1322 = (($1321) + 16|0);
          HEAP32[$1322>>2] = $1320;
          $1323 = $next;
          $1324 = $1;
          HEAP32[$1324>>2] = $1323;
          $1325 = $have;
          $1326 = $1;
          $1327 = (($1326) + 4|0);
          HEAP32[$1327>>2] = $1325;
          $1328 = $hold;
          $1329 = $state;
          $1330 = (($1329) + 56|0);
          HEAP32[$1330>>2] = $1328;
          $1331 = $bits;
          $1332 = $state;
          $1333 = (($1332) + 60|0);
          HEAP32[$1333>>2] = $1331;
          $1334 = $1;
          $1335 = $out;
          _inflate_fast($1334,$1335);
          $1336 = $1;
          $1337 = (($1336) + 12|0);
          $1338 = HEAP32[$1337>>2]|0;
          $put = $1338;
          $1339 = $1;
          $1340 = (($1339) + 16|0);
          $1341 = HEAP32[$1340>>2]|0;
          $left = $1341;
          $1342 = $1;
          $1343 = HEAP32[$1342>>2]|0;
          $next = $1343;
          $1344 = $1;
          $1345 = (($1344) + 4|0);
          $1346 = HEAP32[$1345>>2]|0;
          $have = $1346;
          $1347 = $state;
          $1348 = (($1347) + 56|0);
          $1349 = HEAP32[$1348>>2]|0;
          $hold = $1349;
          $1350 = $state;
          $1351 = (($1350) + 60|0);
          $1352 = HEAP32[$1351>>2]|0;
          $bits = $1352;
          break;
         }
        }
        while(1) {
         $1353 = $hold;
         $1354 = $state;
         $1355 = (($1354) + 84|0);
         $1356 = HEAP32[$1355>>2]|0;
         $1357 = 1 << $1356;
         $1358 = (($1357) - 1)|0;
         $1359 = $1353 & $1358;
         $1360 = $state;
         $1361 = (($1360) + 76|0);
         $1362 = HEAP32[$1361>>2]|0;
         $1363 = (($1362) + ($1359<<2)|0);
         ;HEAP16[$this+0>>1]=HEAP16[$1363+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$1363+2>>1]|0;
         $1364 = (($this) + 1|0);
         $1365 = HEAP8[$1364>>0]|0;
         $1366 = $1365&255;
         $1367 = $bits;
         $1368 = ($1366>>>0)<=($1367>>>0);
         if ($1368) {
          break;
         }
         $1369 = $have;
         $1370 = ($1369|0)==(0);
         if ($1370) {
          label = 406;
          break L13;
         }
         $1371 = $have;
         $1372 = (($1371) + -1)|0;
         $have = $1372;
         $1373 = $next;
         $1374 = (($1373) + 1|0);
         $next = $1374;
         $1375 = HEAP8[$1373>>0]|0;
         $1376 = $1375&255;
         $1377 = $bits;
         $1378 = $1376 << $1377;
         $1379 = $hold;
         $1380 = (($1379) + ($1378))|0;
         $hold = $1380;
         $1381 = $bits;
         $1382 = (($1381) + 8)|0;
         $bits = $1382;
        }
        $1383 = HEAP8[$this>>0]|0;
        $1384 = $1383&255;
        $1385 = ($1384|0)!=(0);
        if ($1385) {
         $1386 = HEAP8[$this>>0]|0;
         $1387 = $1386&255;
         $1388 = $1387 & 240;
         $1389 = ($1388|0)==(0);
         if ($1389) {
          ;HEAP16[$last+0>>1]=HEAP16[$this+0>>1]|0;HEAP16[$last+2>>1]=HEAP16[$this+2>>1]|0;
          while(1) {
           $1390 = (($last) + 2|0);
           $1391 = HEAP16[$1390>>1]|0;
           $1392 = $1391&65535;
           $1393 = $hold;
           $1394 = (($last) + 1|0);
           $1395 = HEAP8[$1394>>0]|0;
           $1396 = $1395&255;
           $1397 = HEAP8[$last>>0]|0;
           $1398 = $1397&255;
           $1399 = (($1396) + ($1398))|0;
           $1400 = 1 << $1399;
           $1401 = (($1400) - 1)|0;
           $1402 = $1393 & $1401;
           $1403 = (($last) + 1|0);
           $1404 = HEAP8[$1403>>0]|0;
           $1405 = $1404&255;
           $1406 = $1402 >>> $1405;
           $1407 = (($1392) + ($1406))|0;
           $1408 = $state;
           $1409 = (($1408) + 76|0);
           $1410 = HEAP32[$1409>>2]|0;
           $1411 = (($1410) + ($1407<<2)|0);
           ;HEAP16[$this+0>>1]=HEAP16[$1411+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$1411+2>>1]|0;
           $1412 = (($last) + 1|0);
           $1413 = HEAP8[$1412>>0]|0;
           $1414 = $1413&255;
           $1415 = (($this) + 1|0);
           $1416 = HEAP8[$1415>>0]|0;
           $1417 = $1416&255;
           $1418 = (($1414) + ($1417))|0;
           $1419 = $bits;
           $1420 = ($1418>>>0)<=($1419>>>0);
           if ($1420) {
            break;
           }
           $1421 = $have;
           $1422 = ($1421|0)==(0);
           if ($1422) {
            label = 416;
            break L13;
           }
           $1423 = $have;
           $1424 = (($1423) + -1)|0;
           $have = $1424;
           $1425 = $next;
           $1426 = (($1425) + 1|0);
           $next = $1426;
           $1427 = HEAP8[$1425>>0]|0;
           $1428 = $1427&255;
           $1429 = $bits;
           $1430 = $1428 << $1429;
           $1431 = $hold;
           $1432 = (($1431) + ($1430))|0;
           $hold = $1432;
           $1433 = $bits;
           $1434 = (($1433) + 8)|0;
           $bits = $1434;
          }
          $1435 = (($last) + 1|0);
          $1436 = HEAP8[$1435>>0]|0;
          $1437 = $1436&255;
          $1438 = $hold;
          $1439 = $1438 >>> $1437;
          $hold = $1439;
          $1440 = (($last) + 1|0);
          $1441 = HEAP8[$1440>>0]|0;
          $1442 = $1441&255;
          $1443 = $bits;
          $1444 = (($1443) - ($1442))|0;
          $bits = $1444;
         }
        }
        $1445 = (($this) + 1|0);
        $1446 = HEAP8[$1445>>0]|0;
        $1447 = $1446&255;
        $1448 = $hold;
        $1449 = $1448 >>> $1447;
        $hold = $1449;
        $1450 = (($this) + 1|0);
        $1451 = HEAP8[$1450>>0]|0;
        $1452 = $1451&255;
        $1453 = $bits;
        $1454 = (($1453) - ($1452))|0;
        $bits = $1454;
        $1455 = (($this) + 2|0);
        $1456 = HEAP16[$1455>>1]|0;
        $1457 = $1456&65535;
        $1458 = $state;
        $1459 = (($1458) + 64|0);
        HEAP32[$1459>>2] = $1457;
        $1460 = HEAP8[$this>>0]|0;
        $1461 = $1460&255;
        $1462 = ($1461|0)==(0);
        if ($1462) {
         $1463 = $state;
         HEAP32[$1463>>2] = 23;
         break;
        }
        $1464 = HEAP8[$this>>0]|0;
        $1465 = $1464&255;
        $1466 = $1465 & 32;
        $1467 = ($1466|0)!=(0);
        if ($1467) {
         $1468 = $state;
         HEAP32[$1468>>2] = 11;
         break;
        }
        $1469 = HEAP8[$this>>0]|0;
        $1470 = $1469&255;
        $1471 = $1470 & 64;
        $1472 = ($1471|0)!=(0);
        if ($1472) {
         $1473 = $1;
         $1474 = (($1473) + 24|0);
         HEAP32[$1474>>2] = 73568;
         $1475 = $state;
         HEAP32[$1475>>2] = 27;
         break;
        } else {
         $1476 = HEAP8[$this>>0]|0;
         $1477 = $1476&255;
         $1478 = $1477 & 15;
         $1479 = $state;
         $1480 = (($1479) + 72|0);
         HEAP32[$1480>>2] = $1478;
         $1481 = $state;
         HEAP32[$1481>>2] = 19;
         label = 431;
         break;
        }
       }
      } while(0);
      if ((label|0) == 126) {
       label = 0;
       $376 = $state;
       $377 = (($376) + 16|0);
       $378 = HEAP32[$377>>2]|0;
       $379 = $378 & 1024;
       $380 = ($379|0)!=(0);
       if ($380) {
        $381 = $state;
        $382 = (($381) + 64|0);
        $383 = HEAP32[$382>>2]|0;
        $copy = $383;
        $384 = $copy;
        $385 = $have;
        $386 = ($384>>>0)>($385>>>0);
        if ($386) {
         $387 = $have;
         $copy = $387;
        }
        $388 = $copy;
        $389 = ($388|0)!=(0);
        if ($389) {
         $390 = $state;
         $391 = (($390) + 32|0);
         $392 = HEAP32[$391>>2]|0;
         $393 = ($392|0)!=(0|0);
         if ($393) {
          $394 = $state;
          $395 = (($394) + 32|0);
          $396 = HEAP32[$395>>2]|0;
          $397 = (($396) + 16|0);
          $398 = HEAP32[$397>>2]|0;
          $399 = ($398|0)!=(0|0);
          if ($399) {
           $400 = $state;
           $401 = (($400) + 32|0);
           $402 = HEAP32[$401>>2]|0;
           $403 = (($402) + 20|0);
           $404 = HEAP32[$403>>2]|0;
           $405 = $state;
           $406 = (($405) + 64|0);
           $407 = HEAP32[$406>>2]|0;
           $408 = (($404) - ($407))|0;
           $len = $408;
           $409 = $state;
           $410 = (($409) + 32|0);
           $411 = HEAP32[$410>>2]|0;
           $412 = (($411) + 16|0);
           $413 = HEAP32[$412>>2]|0;
           $414 = $len;
           $415 = (($413) + ($414)|0);
           $416 = $next;
           $417 = $len;
           $418 = $copy;
           $419 = (($417) + ($418))|0;
           $420 = $state;
           $421 = (($420) + 32|0);
           $422 = HEAP32[$421>>2]|0;
           $423 = (($422) + 24|0);
           $424 = HEAP32[$423>>2]|0;
           $425 = ($419>>>0)>($424>>>0);
           if ($425) {
            $426 = $state;
            $427 = (($426) + 32|0);
            $428 = HEAP32[$427>>2]|0;
            $429 = (($428) + 24|0);
            $430 = HEAP32[$429>>2]|0;
            $431 = $len;
            $432 = (($430) - ($431))|0;
            $434 = $432;
           } else {
            $433 = $copy;
            $434 = $433;
           }
           _memcpy(($415|0),($416|0),($434|0))|0;
          }
         }
         $435 = $state;
         $436 = (($435) + 16|0);
         $437 = HEAP32[$436>>2]|0;
         $438 = $437 & 512;
         $439 = ($438|0)!=(0);
         if ($439) {
          $440 = $state;
          $441 = (($440) + 24|0);
          $442 = HEAP32[$441>>2]|0;
          $443 = $next;
          $444 = $copy;
          $445 = (_crc32($442,$443,$444)|0);
          $446 = $state;
          $447 = (($446) + 24|0);
          HEAP32[$447>>2] = $445;
         }
         $448 = $copy;
         $449 = $have;
         $450 = (($449) - ($448))|0;
         $have = $450;
         $451 = $copy;
         $452 = $next;
         $453 = (($452) + ($451)|0);
         $next = $453;
         $454 = $copy;
         $455 = $state;
         $456 = (($455) + 64|0);
         $457 = HEAP32[$456>>2]|0;
         $458 = (($457) - ($454))|0;
         HEAP32[$456>>2] = $458;
        }
        $459 = $state;
        $460 = (($459) + 64|0);
        $461 = HEAP32[$460>>2]|0;
        $462 = ($461|0)!=(0);
        if ($462) {
         label = 140;
         break;
        }
       }
       $463 = $state;
       $464 = (($463) + 64|0);
       HEAP32[$464>>2] = 0;
       $465 = $state;
       HEAP32[$465>>2] = 6;
       label = 143;
      }
      else if ((label|0) == 431) {
       label = 0;
       $1482 = $state;
       $1483 = (($1482) + 72|0);
       $1484 = HEAP32[$1483>>2]|0;
       $1485 = ($1484|0)!=(0);
       if ($1485) {
        while(1) {
         $1486 = $bits;
         $1487 = $state;
         $1488 = (($1487) + 72|0);
         $1489 = HEAP32[$1488>>2]|0;
         $1490 = ($1486>>>0)<($1489>>>0);
         if (!($1490)) {
          break;
         }
         $1491 = $have;
         $1492 = ($1491|0)==(0);
         if ($1492) {
          label = 437;
          break L13;
         }
         $1493 = $have;
         $1494 = (($1493) + -1)|0;
         $have = $1494;
         $1495 = $next;
         $1496 = (($1495) + 1|0);
         $next = $1496;
         $1497 = HEAP8[$1495>>0]|0;
         $1498 = $1497&255;
         $1499 = $bits;
         $1500 = $1498 << $1499;
         $1501 = $hold;
         $1502 = (($1501) + ($1500))|0;
         $hold = $1502;
         $1503 = $bits;
         $1504 = (($1503) + 8)|0;
         $bits = $1504;
        }
        $1505 = $hold;
        $1506 = $state;
        $1507 = (($1506) + 72|0);
        $1508 = HEAP32[$1507>>2]|0;
        $1509 = 1 << $1508;
        $1510 = (($1509) - 1)|0;
        $1511 = $1505 & $1510;
        $1512 = $state;
        $1513 = (($1512) + 64|0);
        $1514 = HEAP32[$1513>>2]|0;
        $1515 = (($1514) + ($1511))|0;
        HEAP32[$1513>>2] = $1515;
        $1516 = $state;
        $1517 = (($1516) + 72|0);
        $1518 = HEAP32[$1517>>2]|0;
        $1519 = $hold;
        $1520 = $1519 >>> $1518;
        $hold = $1520;
        $1521 = $state;
        $1522 = (($1521) + 72|0);
        $1523 = HEAP32[$1522>>2]|0;
        $1524 = $bits;
        $1525 = (($1524) - ($1523))|0;
        $bits = $1525;
       }
       $1526 = $state;
       HEAP32[$1526>>2] = 20;
       label = 445;
      }
      do {
       if ((label|0) == 143) {
        label = 0;
        $466 = $state;
        $467 = (($466) + 16|0);
        $468 = HEAP32[$467>>2]|0;
        $469 = $468 & 2048;
        $470 = ($469|0)!=(0);
        if ($470) {
         $471 = $have;
         $472 = ($471|0)==(0);
         if ($472) {
          label = 145;
          break L13;
         }
         $copy = 0;
         while(1) {
          $473 = $copy;
          $474 = (($473) + 1)|0;
          $copy = $474;
          $475 = $next;
          $476 = (($475) + ($473)|0);
          $477 = HEAP8[$476>>0]|0;
          $478 = $477&255;
          $len = $478;
          $479 = $state;
          $480 = (($479) + 32|0);
          $481 = HEAP32[$480>>2]|0;
          $482 = ($481|0)!=(0|0);
          if ($482) {
           $483 = $state;
           $484 = (($483) + 32|0);
           $485 = HEAP32[$484>>2]|0;
           $486 = (($485) + 28|0);
           $487 = HEAP32[$486>>2]|0;
           $488 = ($487|0)!=(0|0);
           if ($488) {
            $489 = $state;
            $490 = (($489) + 64|0);
            $491 = HEAP32[$490>>2]|0;
            $492 = $state;
            $493 = (($492) + 32|0);
            $494 = HEAP32[$493>>2]|0;
            $495 = (($494) + 32|0);
            $496 = HEAP32[$495>>2]|0;
            $497 = ($491>>>0)<($496>>>0);
            if ($497) {
             $498 = $len;
             $499 = $498&255;
             $500 = $state;
             $501 = (($500) + 64|0);
             $502 = HEAP32[$501>>2]|0;
             $503 = (($502) + 1)|0;
             HEAP32[$501>>2] = $503;
             $504 = $state;
             $505 = (($504) + 32|0);
             $506 = HEAP32[$505>>2]|0;
             $507 = (($506) + 28|0);
             $508 = HEAP32[$507>>2]|0;
             $509 = (($508) + ($502)|0);
             HEAP8[$509>>0] = $499;
            }
           }
          }
          $510 = $len;
          $511 = ($510|0)!=(0);
          if ($511) {
           $512 = $copy;
           $513 = $have;
           $514 = ($512>>>0)<($513>>>0);
           $2046 = $514;
          } else {
           $2046 = 0;
          }
          if (!($2046)) {
           break;
          }
         }
         $515 = $state;
         $516 = (($515) + 16|0);
         $517 = HEAP32[$516>>2]|0;
         $518 = $517 & 512;
         $519 = ($518|0)!=(0);
         if ($519) {
          $520 = $state;
          $521 = (($520) + 24|0);
          $522 = HEAP32[$521>>2]|0;
          $523 = $next;
          $524 = $copy;
          $525 = (_crc32($522,$523,$524)|0);
          $526 = $state;
          $527 = (($526) + 24|0);
          HEAP32[$527>>2] = $525;
         }
         $528 = $copy;
         $529 = $have;
         $530 = (($529) - ($528))|0;
         $have = $530;
         $531 = $copy;
         $532 = $next;
         $533 = (($532) + ($531)|0);
         $next = $533;
         $534 = $len;
         $535 = ($534|0)!=(0);
         if ($535) {
          label = 158;
          break L13;
         }
        } else {
         $536 = $state;
         $537 = (($536) + 32|0);
         $538 = HEAP32[$537>>2]|0;
         $539 = ($538|0)!=(0|0);
         if ($539) {
          $540 = $state;
          $541 = (($540) + 32|0);
          $542 = HEAP32[$541>>2]|0;
          $543 = (($542) + 28|0);
          HEAP32[$543>>2] = 0;
         }
        }
        $544 = $state;
        $545 = (($544) + 64|0);
        HEAP32[$545>>2] = 0;
        $546 = $state;
        HEAP32[$546>>2] = 7;
        label = 164;
       }
       else if ((label|0) == 445) {
        label = 0;
        while(1) {
         $1527 = $hold;
         $1528 = $state;
         $1529 = (($1528) + 88|0);
         $1530 = HEAP32[$1529>>2]|0;
         $1531 = 1 << $1530;
         $1532 = (($1531) - 1)|0;
         $1533 = $1527 & $1532;
         $1534 = $state;
         $1535 = (($1534) + 80|0);
         $1536 = HEAP32[$1535>>2]|0;
         $1537 = (($1536) + ($1533<<2)|0);
         ;HEAP16[$this+0>>1]=HEAP16[$1537+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$1537+2>>1]|0;
         $1538 = (($this) + 1|0);
         $1539 = HEAP8[$1538>>0]|0;
         $1540 = $1539&255;
         $1541 = $bits;
         $1542 = ($1540>>>0)<=($1541>>>0);
         if ($1542) {
          break;
         }
         $1543 = $have;
         $1544 = ($1543|0)==(0);
         if ($1544) {
          label = 450;
          break L13;
         }
         $1545 = $have;
         $1546 = (($1545) + -1)|0;
         $have = $1546;
         $1547 = $next;
         $1548 = (($1547) + 1|0);
         $next = $1548;
         $1549 = HEAP8[$1547>>0]|0;
         $1550 = $1549&255;
         $1551 = $bits;
         $1552 = $1550 << $1551;
         $1553 = $hold;
         $1554 = (($1553) + ($1552))|0;
         $hold = $1554;
         $1555 = $bits;
         $1556 = (($1555) + 8)|0;
         $bits = $1556;
        }
        $1557 = HEAP8[$this>>0]|0;
        $1558 = $1557&255;
        $1559 = $1558 & 240;
        $1560 = ($1559|0)==(0);
        if ($1560) {
         ;HEAP16[$last+0>>1]=HEAP16[$this+0>>1]|0;HEAP16[$last+2>>1]=HEAP16[$this+2>>1]|0;
         while(1) {
          $1561 = (($last) + 2|0);
          $1562 = HEAP16[$1561>>1]|0;
          $1563 = $1562&65535;
          $1564 = $hold;
          $1565 = (($last) + 1|0);
          $1566 = HEAP8[$1565>>0]|0;
          $1567 = $1566&255;
          $1568 = HEAP8[$last>>0]|0;
          $1569 = $1568&255;
          $1570 = (($1567) + ($1569))|0;
          $1571 = 1 << $1570;
          $1572 = (($1571) - 1)|0;
          $1573 = $1564 & $1572;
          $1574 = (($last) + 1|0);
          $1575 = HEAP8[$1574>>0]|0;
          $1576 = $1575&255;
          $1577 = $1573 >>> $1576;
          $1578 = (($1563) + ($1577))|0;
          $1579 = $state;
          $1580 = (($1579) + 80|0);
          $1581 = HEAP32[$1580>>2]|0;
          $1582 = (($1581) + ($1578<<2)|0);
          ;HEAP16[$this+0>>1]=HEAP16[$1582+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$1582+2>>1]|0;
          $1583 = (($last) + 1|0);
          $1584 = HEAP8[$1583>>0]|0;
          $1585 = $1584&255;
          $1586 = (($this) + 1|0);
          $1587 = HEAP8[$1586>>0]|0;
          $1588 = $1587&255;
          $1589 = (($1585) + ($1588))|0;
          $1590 = $bits;
          $1591 = ($1589>>>0)<=($1590>>>0);
          if ($1591) {
           break;
          }
          $1592 = $have;
          $1593 = ($1592|0)==(0);
          if ($1593) {
           label = 459;
           break L13;
          }
          $1594 = $have;
          $1595 = (($1594) + -1)|0;
          $have = $1595;
          $1596 = $next;
          $1597 = (($1596) + 1|0);
          $next = $1597;
          $1598 = HEAP8[$1596>>0]|0;
          $1599 = $1598&255;
          $1600 = $bits;
          $1601 = $1599 << $1600;
          $1602 = $hold;
          $1603 = (($1602) + ($1601))|0;
          $hold = $1603;
          $1604 = $bits;
          $1605 = (($1604) + 8)|0;
          $bits = $1605;
         }
         $1606 = (($last) + 1|0);
         $1607 = HEAP8[$1606>>0]|0;
         $1608 = $1607&255;
         $1609 = $hold;
         $1610 = $1609 >>> $1608;
         $hold = $1610;
         $1611 = (($last) + 1|0);
         $1612 = HEAP8[$1611>>0]|0;
         $1613 = $1612&255;
         $1614 = $bits;
         $1615 = (($1614) - ($1613))|0;
         $bits = $1615;
        }
        $1616 = (($this) + 1|0);
        $1617 = HEAP8[$1616>>0]|0;
        $1618 = $1617&255;
        $1619 = $hold;
        $1620 = $1619 >>> $1618;
        $hold = $1620;
        $1621 = (($this) + 1|0);
        $1622 = HEAP8[$1621>>0]|0;
        $1623 = $1622&255;
        $1624 = $bits;
        $1625 = (($1624) - ($1623))|0;
        $bits = $1625;
        $1626 = HEAP8[$this>>0]|0;
        $1627 = $1626&255;
        $1628 = $1627 & 64;
        $1629 = ($1628|0)!=(0);
        if ($1629) {
         $1630 = $1;
         $1631 = (($1630) + 24|0);
         HEAP32[$1631>>2] = 73600;
         $1632 = $state;
         HEAP32[$1632>>2] = 27;
         break;
        } else {
         $1633 = (($this) + 2|0);
         $1634 = HEAP16[$1633>>1]|0;
         $1635 = $1634&65535;
         $1636 = $state;
         $1637 = (($1636) + 68|0);
         HEAP32[$1637>>2] = $1635;
         $1638 = HEAP8[$this>>0]|0;
         $1639 = $1638&255;
         $1640 = $1639 & 15;
         $1641 = $state;
         $1642 = (($1641) + 72|0);
         HEAP32[$1642>>2] = $1640;
         $1643 = $state;
         HEAP32[$1643>>2] = 21;
         label = 470;
         break;
        }
       }
      } while(0);
      do {
       if ((label|0) == 164) {
        label = 0;
        $547 = $state;
        $548 = (($547) + 16|0);
        $549 = HEAP32[$548>>2]|0;
        $550 = $549 & 4096;
        $551 = ($550|0)!=(0);
        if ($551) {
         $552 = $have;
         $553 = ($552|0)==(0);
         if ($553) {
          label = 166;
          break L13;
         }
         $copy = 0;
         while(1) {
          $554 = $copy;
          $555 = (($554) + 1)|0;
          $copy = $555;
          $556 = $next;
          $557 = (($556) + ($554)|0);
          $558 = HEAP8[$557>>0]|0;
          $559 = $558&255;
          $len = $559;
          $560 = $state;
          $561 = (($560) + 32|0);
          $562 = HEAP32[$561>>2]|0;
          $563 = ($562|0)!=(0|0);
          if ($563) {
           $564 = $state;
           $565 = (($564) + 32|0);
           $566 = HEAP32[$565>>2]|0;
           $567 = (($566) + 36|0);
           $568 = HEAP32[$567>>2]|0;
           $569 = ($568|0)!=(0|0);
           if ($569) {
            $570 = $state;
            $571 = (($570) + 64|0);
            $572 = HEAP32[$571>>2]|0;
            $573 = $state;
            $574 = (($573) + 32|0);
            $575 = HEAP32[$574>>2]|0;
            $576 = (($575) + 40|0);
            $577 = HEAP32[$576>>2]|0;
            $578 = ($572>>>0)<($577>>>0);
            if ($578) {
             $579 = $len;
             $580 = $579&255;
             $581 = $state;
             $582 = (($581) + 64|0);
             $583 = HEAP32[$582>>2]|0;
             $584 = (($583) + 1)|0;
             HEAP32[$582>>2] = $584;
             $585 = $state;
             $586 = (($585) + 32|0);
             $587 = HEAP32[$586>>2]|0;
             $588 = (($587) + 36|0);
             $589 = HEAP32[$588>>2]|0;
             $590 = (($589) + ($583)|0);
             HEAP8[$590>>0] = $580;
            }
           }
          }
          $591 = $len;
          $592 = ($591|0)!=(0);
          if ($592) {
           $593 = $copy;
           $594 = $have;
           $595 = ($593>>>0)<($594>>>0);
           $2047 = $595;
          } else {
           $2047 = 0;
          }
          if (!($2047)) {
           break;
          }
         }
         $596 = $state;
         $597 = (($596) + 16|0);
         $598 = HEAP32[$597>>2]|0;
         $599 = $598 & 512;
         $600 = ($599|0)!=(0);
         if ($600) {
          $601 = $state;
          $602 = (($601) + 24|0);
          $603 = HEAP32[$602>>2]|0;
          $604 = $next;
          $605 = $copy;
          $606 = (_crc32($603,$604,$605)|0);
          $607 = $state;
          $608 = (($607) + 24|0);
          HEAP32[$608>>2] = $606;
         }
         $609 = $copy;
         $610 = $have;
         $611 = (($610) - ($609))|0;
         $have = $611;
         $612 = $copy;
         $613 = $next;
         $614 = (($613) + ($612)|0);
         $next = $614;
         $615 = $len;
         $616 = ($615|0)!=(0);
         if ($616) {
          label = 179;
          break L13;
         }
        } else {
         $617 = $state;
         $618 = (($617) + 32|0);
         $619 = HEAP32[$618>>2]|0;
         $620 = ($619|0)!=(0|0);
         if ($620) {
          $621 = $state;
          $622 = (($621) + 32|0);
          $623 = HEAP32[$622>>2]|0;
          $624 = (($623) + 36|0);
          HEAP32[$624>>2] = 0;
         }
        }
        $625 = $state;
        HEAP32[$625>>2] = 8;
        label = 185;
       }
       else if ((label|0) == 470) {
        label = 0;
        $1644 = $state;
        $1645 = (($1644) + 72|0);
        $1646 = HEAP32[$1645>>2]|0;
        $1647 = ($1646|0)!=(0);
        if ($1647) {
         while(1) {
          $1648 = $bits;
          $1649 = $state;
          $1650 = (($1649) + 72|0);
          $1651 = HEAP32[$1650>>2]|0;
          $1652 = ($1648>>>0)<($1651>>>0);
          if (!($1652)) {
           break;
          }
          $1653 = $have;
          $1654 = ($1653|0)==(0);
          if ($1654) {
           label = 476;
           break L13;
          }
          $1655 = $have;
          $1656 = (($1655) + -1)|0;
          $have = $1656;
          $1657 = $next;
          $1658 = (($1657) + 1|0);
          $next = $1658;
          $1659 = HEAP8[$1657>>0]|0;
          $1660 = $1659&255;
          $1661 = $bits;
          $1662 = $1660 << $1661;
          $1663 = $hold;
          $1664 = (($1663) + ($1662))|0;
          $hold = $1664;
          $1665 = $bits;
          $1666 = (($1665) + 8)|0;
          $bits = $1666;
         }
         $1667 = $hold;
         $1668 = $state;
         $1669 = (($1668) + 72|0);
         $1670 = HEAP32[$1669>>2]|0;
         $1671 = 1 << $1670;
         $1672 = (($1671) - 1)|0;
         $1673 = $1667 & $1672;
         $1674 = $state;
         $1675 = (($1674) + 68|0);
         $1676 = HEAP32[$1675>>2]|0;
         $1677 = (($1676) + ($1673))|0;
         HEAP32[$1675>>2] = $1677;
         $1678 = $state;
         $1679 = (($1678) + 72|0);
         $1680 = HEAP32[$1679>>2]|0;
         $1681 = $hold;
         $1682 = $1681 >>> $1680;
         $hold = $1682;
         $1683 = $state;
         $1684 = (($1683) + 72|0);
         $1685 = HEAP32[$1684>>2]|0;
         $1686 = $bits;
         $1687 = (($1686) - ($1685))|0;
         $bits = $1687;
        }
        $1688 = $state;
        $1689 = (($1688) + 68|0);
        $1690 = HEAP32[$1689>>2]|0;
        $1691 = $state;
        $1692 = (($1691) + 44|0);
        $1693 = HEAP32[$1692>>2]|0;
        $1694 = $out;
        $1695 = (($1693) + ($1694))|0;
        $1696 = $left;
        $1697 = (($1695) - ($1696))|0;
        $1698 = ($1690>>>0)>($1697>>>0);
        if ($1698) {
         $1699 = $1;
         $1700 = (($1699) + 24|0);
         HEAP32[$1700>>2] = 73624;
         $1701 = $state;
         HEAP32[$1701>>2] = 27;
         break;
        } else {
         $1702 = $state;
         HEAP32[$1702>>2] = 22;
         label = 486;
         break;
        }
       }
      } while(0);
      do {
       if ((label|0) == 185) {
        label = 0;
        $626 = $state;
        $627 = (($626) + 16|0);
        $628 = HEAP32[$627>>2]|0;
        $629 = $628 & 512;
        $630 = ($629|0)!=(0);
        if ($630) {
         while(1) {
          $631 = $bits;
          $632 = ($631>>>0)<(16);
          if (!($632)) {
           break;
          }
          $633 = $have;
          $634 = ($633|0)==(0);
          if ($634) {
           label = 191;
           break L13;
          }
          $635 = $have;
          $636 = (($635) + -1)|0;
          $have = $636;
          $637 = $next;
          $638 = (($637) + 1|0);
          $next = $638;
          $639 = HEAP8[$637>>0]|0;
          $640 = $639&255;
          $641 = $bits;
          $642 = $640 << $641;
          $643 = $hold;
          $644 = (($643) + ($642))|0;
          $hold = $644;
          $645 = $bits;
          $646 = (($645) + 8)|0;
          $bits = $646;
         }
         $647 = $hold;
         $648 = $state;
         $649 = (($648) + 24|0);
         $650 = HEAP32[$649>>2]|0;
         $651 = $650 & 65535;
         $652 = ($647|0)!=($651|0);
         if ($652) {
          $653 = $1;
          $654 = (($653) + 24|0);
          HEAP32[$654>>2] = 73328;
          $655 = $state;
          HEAP32[$655>>2] = 27;
          break;
         }
         $hold = 0;
         $bits = 0;
        }
        $656 = $state;
        $657 = (($656) + 32|0);
        $658 = HEAP32[$657>>2]|0;
        $659 = ($658|0)!=(0|0);
        if ($659) {
         $660 = $state;
         $661 = (($660) + 16|0);
         $662 = HEAP32[$661>>2]|0;
         $663 = $662 >> 9;
         $664 = $663 & 1;
         $665 = $state;
         $666 = (($665) + 32|0);
         $667 = HEAP32[$666>>2]|0;
         $668 = (($667) + 44|0);
         HEAP32[$668>>2] = $664;
         $669 = $state;
         $670 = (($669) + 32|0);
         $671 = HEAP32[$670>>2]|0;
         $672 = (($671) + 48|0);
         HEAP32[$672>>2] = 1;
        }
        $673 = (_crc32(0,0,0)|0);
        $674 = $state;
        $675 = (($674) + 24|0);
        HEAP32[$675>>2] = $673;
        $676 = $1;
        $677 = (($676) + 48|0);
        HEAP32[$677>>2] = $673;
        $678 = $state;
        HEAP32[$678>>2] = 11;
       }
       else if ((label|0) == 486) {
        label = 0;
        $1703 = $left;
        $1704 = ($1703|0)==(0);
        if ($1704) {
         label = 487;
         break L13;
        }
        $1705 = $out;
        $1706 = $left;
        $1707 = (($1705) - ($1706))|0;
        $copy = $1707;
        $1708 = $state;
        $1709 = (($1708) + 68|0);
        $1710 = HEAP32[$1709>>2]|0;
        $1711 = $copy;
        $1712 = ($1710>>>0)>($1711>>>0);
        if ($1712) {
         $1713 = $state;
         $1714 = (($1713) + 68|0);
         $1715 = HEAP32[$1714>>2]|0;
         $1716 = $copy;
         $1717 = (($1715) - ($1716))|0;
         $copy = $1717;
         $1718 = $copy;
         $1719 = $state;
         $1720 = (($1719) + 48|0);
         $1721 = HEAP32[$1720>>2]|0;
         $1722 = ($1718>>>0)>($1721>>>0);
         if ($1722) {
          $1723 = $state;
          $1724 = (($1723) + 48|0);
          $1725 = HEAP32[$1724>>2]|0;
          $1726 = $copy;
          $1727 = (($1726) - ($1725))|0;
          $copy = $1727;
          $1728 = $state;
          $1729 = (($1728) + 52|0);
          $1730 = HEAP32[$1729>>2]|0;
          $1731 = $state;
          $1732 = (($1731) + 40|0);
          $1733 = HEAP32[$1732>>2]|0;
          $1734 = $copy;
          $1735 = (($1733) - ($1734))|0;
          $1736 = (($1730) + ($1735)|0);
          $from = $1736;
         } else {
          $1737 = $state;
          $1738 = (($1737) + 52|0);
          $1739 = HEAP32[$1738>>2]|0;
          $1740 = $state;
          $1741 = (($1740) + 48|0);
          $1742 = HEAP32[$1741>>2]|0;
          $1743 = $copy;
          $1744 = (($1742) - ($1743))|0;
          $1745 = (($1739) + ($1744)|0);
          $from = $1745;
         }
         $1746 = $copy;
         $1747 = $state;
         $1748 = (($1747) + 64|0);
         $1749 = HEAP32[$1748>>2]|0;
         $1750 = ($1746>>>0)>($1749>>>0);
         if ($1750) {
          $1751 = $state;
          $1752 = (($1751) + 64|0);
          $1753 = HEAP32[$1752>>2]|0;
          $copy = $1753;
         }
        } else {
         $1754 = $put;
         $1755 = $state;
         $1756 = (($1755) + 68|0);
         $1757 = HEAP32[$1756>>2]|0;
         $1758 = (0 - ($1757))|0;
         $1759 = (($1754) + ($1758)|0);
         $from = $1759;
         $1760 = $state;
         $1761 = (($1760) + 64|0);
         $1762 = HEAP32[$1761>>2]|0;
         $copy = $1762;
        }
        $1763 = $copy;
        $1764 = $left;
        $1765 = ($1763>>>0)>($1764>>>0);
        if ($1765) {
         $1766 = $left;
         $copy = $1766;
        }
        $1767 = $copy;
        $1768 = $left;
        $1769 = (($1768) - ($1767))|0;
        $left = $1769;
        $1770 = $copy;
        $1771 = $state;
        $1772 = (($1771) + 64|0);
        $1773 = HEAP32[$1772>>2]|0;
        $1774 = (($1773) - ($1770))|0;
        HEAP32[$1772>>2] = $1774;
        while(1) {
         $1775 = $from;
         $1776 = (($1775) + 1|0);
         $from = $1776;
         $1777 = HEAP8[$1775>>0]|0;
         $1778 = $put;
         $1779 = (($1778) + 1|0);
         $put = $1779;
         HEAP8[$1778>>0] = $1777;
         $1780 = $copy;
         $1781 = (($1780) + -1)|0;
         $copy = $1781;
         $1782 = ($1781|0)!=(0);
         if (!($1782)) {
          break;
         }
        }
        $1783 = $state;
        $1784 = (($1783) + 64|0);
        $1785 = HEAP32[$1784>>2]|0;
        $1786 = ($1785|0)==(0);
        if ($1786) {
         $1787 = $state;
         HEAP32[$1787>>2] = 18;
        }
       }
      } while(0);
     }
     switch (label|0) {
      case 20: {
       break;
      }
      case 50: {
       break;
      }
      case 72: {
       break;
      }
      case 90: {
       break;
      }
      case 109: {
       break;
      }
      case 140: {
       break;
      }
      case 145: {
       break;
      }
      case 158: {
       break;
      }
      case 166: {
       break;
      }
      case 179: {
       break;
      }
      case 191: {
       break;
      }
      case 208: {
       break;
      }
      case 216: {
       $719 = $put;
       $720 = $1;
       $721 = (($720) + 12|0);
       HEAP32[$721>>2] = $719;
       $722 = $left;
       $723 = $1;
       $724 = (($723) + 16|0);
       HEAP32[$724>>2] = $722;
       $725 = $next;
       $726 = $1;
       HEAP32[$726>>2] = $725;
       $727 = $have;
       $728 = $1;
       $729 = (($728) + 4|0);
       HEAP32[$729>>2] = $727;
       $730 = $hold;
       $731 = $state;
       $732 = (($731) + 56|0);
       HEAP32[$732>>2] = $730;
       $733 = $bits;
       $734 = $state;
       $735 = (($734) + 60|0);
       HEAP32[$735>>2] = $733;
       $0 = 2;
       $2045 = $0;
       STACKTOP = sp;return ($2045|0);
       break;
      }
      case 221: {
       break;
      }
      case 232: {
       break;
      }
      case 253: {
       break;
      }
      case 268: {
       break;
      }
      case 276: {
       break;
      }
      case 297: {
       break;
      }
      case 317: {
       break;
      }
      case 326: {
       break;
      }
      case 339: {
       break;
      }
      case 356: {
       break;
      }
      case 370: {
       break;
      }
      case 406: {
       break;
      }
      case 416: {
       break;
      }
      case 437: {
       break;
      }
      case 450: {
       break;
      }
      case 459: {
       break;
      }
      case 476: {
       break;
      }
      case 487: {
       break;
      }
      case 505: {
       break;
      }
      case 513: {
       break;
      }
      case 538: {
       break;
      }
      case 544: {
       $hold = 0;
       $bits = 0;
       label = 547;
       break;
      }
      case 549: {
       $ret = -3;
       break;
      }
      case 550: {
       $0 = -4;
       $2045 = $0;
       STACKTOP = sp;return ($2045|0);
       break;
      }
      case 551: {
       label = 552;
       break;
      }
     }
     if ((label|0) == 547) {
      $1923 = $state;
      HEAP32[$1923>>2] = 26;
      label = 548;
     }
     else if ((label|0) == 552) {
      $0 = -2;
      $2045 = $0;
      STACKTOP = sp;return ($2045|0);
     }
     if ((label|0) == 548) {
      $ret = 1;
     }
     $1924 = $put;
     $1925 = $1;
     $1926 = (($1925) + 12|0);
     HEAP32[$1926>>2] = $1924;
     $1927 = $left;
     $1928 = $1;
     $1929 = (($1928) + 16|0);
     HEAP32[$1929>>2] = $1927;
     $1930 = $next;
     $1931 = $1;
     HEAP32[$1931>>2] = $1930;
     $1932 = $have;
     $1933 = $1;
     $1934 = (($1933) + 4|0);
     HEAP32[$1934>>2] = $1932;
     $1935 = $hold;
     $1936 = $state;
     $1937 = (($1936) + 56|0);
     HEAP32[$1937>>2] = $1935;
     $1938 = $bits;
     $1939 = $state;
     $1940 = (($1939) + 60|0);
     HEAP32[$1940>>2] = $1938;
     $1941 = $state;
     $1942 = (($1941) + 40|0);
     $1943 = HEAP32[$1942>>2]|0;
     $1944 = ($1943|0)!=(0);
     if ($1944) {
      label = 559;
     } else {
      $1945 = $state;
      $1946 = HEAP32[$1945>>2]|0;
      $1947 = ($1946>>>0)<(24);
      if ($1947) {
       $1948 = $out;
       $1949 = $1;
       $1950 = (($1949) + 16|0);
       $1951 = HEAP32[$1950>>2]|0;
       $1952 = ($1948|0)!=($1951|0);
       if ($1952) {
        label = 559;
       }
      }
     }
     do {
      if ((label|0) == 559) {
       $1953 = $1;
       $1954 = $out;
       $1955 = (_updatewindow($1953,$1954)|0);
       $1956 = ($1955|0)!=(0);
       if (!($1956)) {
        break;
       }
       $1957 = $state;
       HEAP32[$1957>>2] = 28;
       $0 = -4;
       $2045 = $0;
       STACKTOP = sp;return ($2045|0);
      }
     } while(0);
     $1958 = $1;
     $1959 = (($1958) + 4|0);
     $1960 = HEAP32[$1959>>2]|0;
     $1961 = $in;
     $1962 = (($1961) - ($1960))|0;
     $in = $1962;
     $1963 = $1;
     $1964 = (($1963) + 16|0);
     $1965 = HEAP32[$1964>>2]|0;
     $1966 = $out;
     $1967 = (($1966) - ($1965))|0;
     $out = $1967;
     $1968 = $in;
     $1969 = $1;
     $1970 = (($1969) + 8|0);
     $1971 = HEAP32[$1970>>2]|0;
     $1972 = (($1971) + ($1968))|0;
     HEAP32[$1970>>2] = $1972;
     $1973 = $out;
     $1974 = $1;
     $1975 = (($1974) + 20|0);
     $1976 = HEAP32[$1975>>2]|0;
     $1977 = (($1976) + ($1973))|0;
     HEAP32[$1975>>2] = $1977;
     $1978 = $out;
     $1979 = $state;
     $1980 = (($1979) + 28|0);
     $1981 = HEAP32[$1980>>2]|0;
     $1982 = (($1981) + ($1978))|0;
     HEAP32[$1980>>2] = $1982;
     $1983 = $state;
     $1984 = (($1983) + 8|0);
     $1985 = HEAP32[$1984>>2]|0;
     $1986 = ($1985|0)!=(0);
     if ($1986) {
      $1987 = $out;
      $1988 = ($1987|0)!=(0);
      if ($1988) {
       $1989 = $state;
       $1990 = (($1989) + 16|0);
       $1991 = HEAP32[$1990>>2]|0;
       $1992 = ($1991|0)!=(0);
       if ($1992) {
        $1993 = $state;
        $1994 = (($1993) + 24|0);
        $1995 = HEAP32[$1994>>2]|0;
        $1996 = $1;
        $1997 = (($1996) + 12|0);
        $1998 = HEAP32[$1997>>2]|0;
        $1999 = $out;
        $2000 = (0 - ($1999))|0;
        $2001 = (($1998) + ($2000)|0);
        $2002 = $out;
        $2003 = (_crc32($1995,$2001,$2002)|0);
        $2017 = $2003;
       } else {
        $2004 = $state;
        $2005 = (($2004) + 24|0);
        $2006 = HEAP32[$2005>>2]|0;
        $2007 = $1;
        $2008 = (($2007) + 12|0);
        $2009 = HEAP32[$2008>>2]|0;
        $2010 = $out;
        $2011 = (0 - ($2010))|0;
        $2012 = (($2009) + ($2011)|0);
        $2013 = $out;
        $2014 = (_adler32($2006,$2012,$2013)|0);
        $2017 = $2014;
       }
       $2015 = $state;
       $2016 = (($2015) + 24|0);
       HEAP32[$2016>>2] = $2017;
       $2018 = $1;
       $2019 = (($2018) + 48|0);
       HEAP32[$2019>>2] = $2017;
      }
     }
     $2020 = $state;
     $2021 = (($2020) + 60|0);
     $2022 = HEAP32[$2021>>2]|0;
     $2023 = $state;
     $2024 = (($2023) + 4|0);
     $2025 = HEAP32[$2024>>2]|0;
     $2026 = ($2025|0)!=(0);
     $2027 = $2026 ? 64 : 0;
     $2028 = (($2022) + ($2027))|0;
     $2029 = $state;
     $2030 = HEAP32[$2029>>2]|0;
     $2031 = ($2030|0)==(11);
     $2032 = $2031 ? 128 : 0;
     $2033 = (($2028) + ($2032))|0;
     $2034 = $1;
     $2035 = (($2034) + 44|0);
     HEAP32[$2035>>2] = $2033;
     $2036 = $in;
     $2037 = ($2036|0)==(0);
     if ($2037) {
      $2038 = $out;
      $2039 = ($2038|0)==(0);
      if ($2039) {
       label = 571;
      } else {
       label = 570;
      }
     } else {
      label = 570;
     }
     if ((label|0) == 570) {
      $2040 = $2;
      $2041 = ($2040|0)==(4);
      if ($2041) {
       label = 571;
      }
     }
     if ((label|0) == 571) {
      $2042 = $ret;
      $2043 = ($2042|0)==(0);
      if ($2043) {
       $ret = -5;
      }
     }
     $2044 = $ret;
     $0 = $2044;
     $2045 = $0;
     STACKTOP = sp;return ($2045|0);
    }
   }
  }
 } while(0);
 $0 = -2;
 $2045 = $0;
 STACKTOP = sp;return ($2045|0);
}
function _fixedtables($state) {
 $state = $state|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $state;
 $1 = $0;
 $2 = (($1) + 76|0);
 HEAP32[$2>>2] = 73704;
 $3 = $0;
 $4 = (($3) + 84|0);
 HEAP32[$4>>2] = 9;
 $5 = $0;
 $6 = (($5) + 80|0);
 HEAP32[$6>>2] = 75752;
 $7 = $0;
 $8 = (($7) + 88|0);
 HEAP32[$8>>2] = 5;
 STACKTOP = sp;return;
}
function _updatewindow($strm,$out) {
 $strm = $strm|0;
 $out = $out|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $copy = 0, $dist = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $strm;
 $2 = $out;
 $3 = $1;
 $4 = (($3) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $state = $5;
 $6 = $state;
 $7 = (($6) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 do {
  if ($9) {
   $10 = $1;
   $11 = (($10) + 32|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = $1;
   $14 = (($13) + 40|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $state;
   $17 = (($16) + 36|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = 1 << $18;
   $20 = (FUNCTION_TABLE_iiii[$12 & 255]($15,$19,1)|0);
   $21 = $state;
   $22 = (($21) + 52|0);
   HEAP32[$22>>2] = $20;
   $23 = $state;
   $24 = (($23) + 52|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)==(0|0);
   if (!($26)) {
    break;
   }
   $0 = 1;
   $146 = $0;
   STACKTOP = sp;return ($146|0);
  }
 } while(0);
 $27 = $state;
 $28 = (($27) + 40|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)==(0);
 if ($30) {
  $31 = $state;
  $32 = (($31) + 36|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = 1 << $33;
  $35 = $state;
  $36 = (($35) + 40|0);
  HEAP32[$36>>2] = $34;
  $37 = $state;
  $38 = (($37) + 48|0);
  HEAP32[$38>>2] = 0;
  $39 = $state;
  $40 = (($39) + 44|0);
  HEAP32[$40>>2] = 0;
 }
 $41 = $2;
 $42 = $1;
 $43 = (($42) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($41) - ($44))|0;
 $copy = $45;
 $46 = $copy;
 $47 = $state;
 $48 = (($47) + 40|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($46>>>0)>=($49>>>0);
 if ($50) {
  $51 = $state;
  $52 = (($51) + 52|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = $1;
  $55 = (($54) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $state;
  $58 = (($57) + 40|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (0 - ($59))|0;
  $61 = (($56) + ($60)|0);
  $62 = $state;
  $63 = (($62) + 40|0);
  $64 = HEAP32[$63>>2]|0;
  _memcpy(($53|0),($61|0),($64|0))|0;
  $65 = $state;
  $66 = (($65) + 48|0);
  HEAP32[$66>>2] = 0;
  $67 = $state;
  $68 = (($67) + 40|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = $state;
  $71 = (($70) + 44|0);
  HEAP32[$71>>2] = $69;
 } else {
  $72 = $state;
  $73 = (($72) + 40|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $state;
  $76 = (($75) + 48|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (($74) - ($77))|0;
  $dist = $78;
  $79 = $dist;
  $80 = $copy;
  $81 = ($79>>>0)>($80>>>0);
  if ($81) {
   $82 = $copy;
   $dist = $82;
  }
  $83 = $state;
  $84 = (($83) + 52|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = $state;
  $87 = (($86) + 48|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($85) + ($88)|0);
  $90 = $1;
  $91 = (($90) + 12|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = $copy;
  $94 = (0 - ($93))|0;
  $95 = (($92) + ($94)|0);
  $96 = $dist;
  _memcpy(($89|0),($95|0),($96|0))|0;
  $97 = $dist;
  $98 = $copy;
  $99 = (($98) - ($97))|0;
  $copy = $99;
  $100 = $copy;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = $state;
   $103 = (($102) + 52|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = $1;
   $106 = (($105) + 12|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = $copy;
   $109 = (0 - ($108))|0;
   $110 = (($107) + ($109)|0);
   $111 = $copy;
   _memcpy(($104|0),($110|0),($111|0))|0;
   $112 = $copy;
   $113 = $state;
   $114 = (($113) + 48|0);
   HEAP32[$114>>2] = $112;
   $115 = $state;
   $116 = (($115) + 40|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = $state;
   $119 = (($118) + 44|0);
   HEAP32[$119>>2] = $117;
  } else {
   $120 = $dist;
   $121 = $state;
   $122 = (($121) + 48|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = (($123) + ($120))|0;
   HEAP32[$122>>2] = $124;
   $125 = $state;
   $126 = (($125) + 48|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = $state;
   $129 = (($128) + 40|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = ($127|0)==($130|0);
   if ($131) {
    $132 = $state;
    $133 = (($132) + 48|0);
    HEAP32[$133>>2] = 0;
   }
   $134 = $state;
   $135 = (($134) + 44|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = $state;
   $138 = (($137) + 40|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = ($136>>>0)<($139>>>0);
   if ($140) {
    $141 = $dist;
    $142 = $state;
    $143 = (($142) + 44|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = (($144) + ($141))|0;
    HEAP32[$143>>2] = $145;
   }
  }
 }
 $0 = 0;
 $146 = $0;
 STACKTOP = sp;return ($146|0);
}
function _inflateEnd($strm) {
 $strm = $strm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $strm;
 $2 = $1;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = $1;
  $5 = (($4) + 28|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)==(0|0);
  if (!($7)) {
   $8 = $1;
   $9 = (($8) + 36|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)==(0|0);
   if (!($11)) {
    $12 = $1;
    $13 = (($12) + 28|0);
    $14 = HEAP32[$13>>2]|0;
    $state = $14;
    $15 = $state;
    $16 = (($15) + 52|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ($17|0)!=(0|0);
    if ($18) {
     $19 = $1;
     $20 = (($19) + 36|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = $1;
     $23 = (($22) + 40|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = $state;
     $26 = (($25) + 52|0);
     $27 = HEAP32[$26>>2]|0;
     FUNCTION_TABLE_vii[$21 & 255]($24,$27);
    }
    $28 = $1;
    $29 = (($28) + 36|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = $1;
    $32 = (($31) + 40|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = $1;
    $35 = (($34) + 28|0);
    $36 = HEAP32[$35>>2]|0;
    FUNCTION_TABLE_vii[$30 & 255]($33,$36);
    $37 = $1;
    $38 = (($37) + 28|0);
    HEAP32[$38>>2] = 0;
    $0 = 0;
    $39 = $0;
    STACKTOP = sp;return ($39|0);
   }
  }
 }
 $0 = -2;
 $39 = $0;
 STACKTOP = sp;return ($39|0);
}
function _inflate_table($type,$lens,$codes,$table,$bits,$work) {
 $type = $type|0;
 $lens = $lens|0;
 $codes = $codes|0;
 $table = $table|0;
 $bits = $bits|0;
 $work = $work|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $base = 0, $count = 0, $curr = 0, $drop = 0, $end = 0, $extra = 0, $fill = 0, $huff = 0, $incr = 0;
 var $left = 0, $len = 0, $low = 0, $mask = 0, $max = 0, $min = 0, $next = 0, $offs = 0, $root = 0, $sym = 0, $this = 0, $used = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $this = sp + 168|0;
 $count = sp + 136|0;
 $offs = sp + 104|0;
 $1 = $type;
 $2 = $lens;
 $3 = $codes;
 $4 = $table;
 $5 = $bits;
 $6 = $work;
 $len = 0;
 while(1) {
  $7 = $len;
  $8 = ($7>>>0)<=(15);
  if (!($8)) {
   break;
  }
  $9 = $len;
  $10 = (($count) + ($9<<1)|0);
  HEAP16[$10>>1] = 0;
  $11 = $len;
  $12 = (($11) + 1)|0;
  $len = $12;
 }
 $sym = 0;
 while(1) {
  $13 = $sym;
  $14 = $3;
  $15 = ($13>>>0)<($14>>>0);
  if (!($15)) {
   break;
  }
  $16 = $sym;
  $17 = $2;
  $18 = (($17) + ($16<<1)|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19&65535;
  $21 = (($count) + ($20<<1)|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = (($22) + 1)<<16>>16;
  HEAP16[$21>>1] = $23;
  $24 = $sym;
  $25 = (($24) + 1)|0;
  $sym = $25;
 }
 $26 = $5;
 $27 = HEAP32[$26>>2]|0;
 $root = $27;
 $max = 15;
 while(1) {
  $28 = $max;
  $29 = ($28>>>0)>=(1);
  if (!($29)) {
   break;
  }
  $30 = $max;
  $31 = (($count) + ($30<<1)|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32&65535;
  $34 = ($33|0)!=(0);
  if ($34) {
   label = 12;
   break;
  }
  $35 = $max;
  $36 = (($35) + -1)|0;
  $max = $36;
 }
 if ((label|0) == 12) {
 }
 $37 = $root;
 $38 = $max;
 $39 = ($37>>>0)>($38>>>0);
 if ($39) {
  $40 = $max;
  $root = $40;
 }
 $41 = $max;
 $42 = ($41|0)==(0);
 if ($42) {
  HEAP8[$this>>0] = 64;
  $43 = (($this) + 1|0);
  HEAP8[$43>>0] = 1;
  $44 = (($this) + 2|0);
  HEAP16[$44>>1] = 0;
  $45 = $4;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + 4|0);
  HEAP32[$45>>2] = $47;
  ;HEAP16[$46+0>>1]=HEAP16[$this+0>>1]|0;HEAP16[$46+2>>1]=HEAP16[$this+2>>1]|0;
  $48 = $4;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 4|0);
  HEAP32[$48>>2] = $50;
  ;HEAP16[$49+0>>1]=HEAP16[$this+0>>1]|0;HEAP16[$49+2>>1]=HEAP16[$this+2>>1]|0;
  $51 = $5;
  HEAP32[$51>>2] = 1;
  $0 = 0;
  $370 = $0;
  STACKTOP = sp;return ($370|0);
 }
 $min = 1;
 while(1) {
  $52 = $min;
  $53 = ($52>>>0)<=(15);
  if (!($53)) {
   break;
  }
  $54 = $min;
  $55 = (($count) + ($54<<1)|0);
  $56 = HEAP16[$55>>1]|0;
  $57 = $56&65535;
  $58 = ($57|0)!=(0);
  if ($58) {
   label = 22;
   break;
  }
  $59 = $min;
  $60 = (($59) + 1)|0;
  $min = $60;
 }
 if ((label|0) == 22) {
 }
 $61 = $root;
 $62 = $min;
 $63 = ($61>>>0)<($62>>>0);
 if ($63) {
  $64 = $min;
  $root = $64;
 }
 $left = 1;
 $len = 1;
 while(1) {
  $65 = $len;
  $66 = ($65>>>0)<=(15);
  if (!($66)) {
   break;
  }
  $67 = $left;
  $68 = $67 << 1;
  $left = $68;
  $69 = $len;
  $70 = (($count) + ($69<<1)|0);
  $71 = HEAP16[$70>>1]|0;
  $72 = $71&65535;
  $73 = $left;
  $74 = (($73) - ($72))|0;
  $left = $74;
  $75 = $left;
  $76 = ($75|0)<(0);
  if ($76) {
   label = 30;
   break;
  }
  $77 = $len;
  $78 = (($77) + 1)|0;
  $len = $78;
 }
 if ((label|0) == 30) {
  $0 = -1;
  $370 = $0;
  STACKTOP = sp;return ($370|0);
 }
 $79 = $left;
 $80 = ($79|0)>(0);
 do {
  if ($80) {
   $81 = $1;
   $82 = ($81|0)==(0);
   if (!($82)) {
    $83 = $max;
    $84 = ($83|0)!=(1);
    if (!($84)) {
     break;
    }
   }
   $0 = -1;
   $370 = $0;
   STACKTOP = sp;return ($370|0);
  }
 } while(0);
 $85 = (($offs) + 2|0);
 HEAP16[$85>>1] = 0;
 $len = 1;
 while(1) {
  $86 = $len;
  $87 = ($86>>>0)<(15);
  if (!($87)) {
   break;
  }
  $88 = $len;
  $89 = (($offs) + ($88<<1)|0);
  $90 = HEAP16[$89>>1]|0;
  $91 = $90&65535;
  $92 = $len;
  $93 = (($count) + ($92<<1)|0);
  $94 = HEAP16[$93>>1]|0;
  $95 = $94&65535;
  $96 = (($91) + ($95))|0;
  $97 = $96&65535;
  $98 = $len;
  $99 = (($98) + 1)|0;
  $100 = (($offs) + ($99<<1)|0);
  HEAP16[$100>>1] = $97;
  $101 = $len;
  $102 = (($101) + 1)|0;
  $len = $102;
 }
 $sym = 0;
 while(1) {
  $103 = $sym;
  $104 = $3;
  $105 = ($103>>>0)<($104>>>0);
  if (!($105)) {
   break;
  }
  $106 = $sym;
  $107 = $2;
  $108 = (($107) + ($106<<1)|0);
  $109 = HEAP16[$108>>1]|0;
  $110 = $109&65535;
  $111 = ($110|0)!=(0);
  if ($111) {
   $112 = $sym;
   $113 = $112&65535;
   $114 = $sym;
   $115 = $2;
   $116 = (($115) + ($114<<1)|0);
   $117 = HEAP16[$116>>1]|0;
   $118 = $117&65535;
   $119 = (($offs) + ($118<<1)|0);
   $120 = HEAP16[$119>>1]|0;
   $121 = (($120) + 1)<<16>>16;
   HEAP16[$119>>1] = $121;
   $122 = $120&65535;
   $123 = $6;
   $124 = (($123) + ($122<<1)|0);
   HEAP16[$124>>1] = $113;
  }
  $125 = $sym;
  $126 = (($125) + 1)|0;
  $sym = $126;
 }
 $127 = $1;
 if ((($127|0) == 1)) {
  $base = 75880;
  $129 = $base;
  $130 = (($129) + -514|0);
  $base = $130;
  $extra = 75944;
  $131 = $extra;
  $132 = (($131) + -514|0);
  $extra = $132;
  $end = 256;
 } else if ((($127|0) == 0)) {
  $128 = $6;
  $extra = $128;
  $base = $128;
  $end = 19;
 } else {
  $base = 76008;
  $extra = 76072;
  $end = -1;
 }
 $huff = 0;
 $sym = 0;
 $133 = $min;
 $len = $133;
 $134 = $4;
 $135 = HEAP32[$134>>2]|0;
 $next = $135;
 $136 = $root;
 $curr = $136;
 $drop = 0;
 $low = -1;
 $137 = $root;
 $138 = 1 << $137;
 $used = $138;
 $139 = $used;
 $140 = (($139) - 1)|0;
 $mask = $140;
 $141 = $1;
 $142 = ($141|0)==(1);
 if ($142) {
  $143 = $used;
  $144 = ($143>>>0)>=(1456);
  if ($144) {
   $0 = 1;
   $370 = $0;
   STACKTOP = sp;return ($370|0);
  }
 }
 while(1) {
  $145 = $len;
  $146 = $drop;
  $147 = (($145) - ($146))|0;
  $148 = $147&255;
  $149 = (($this) + 1|0);
  HEAP8[$149>>0] = $148;
  $150 = $sym;
  $151 = $6;
  $152 = (($151) + ($150<<1)|0);
  $153 = HEAP16[$152>>1]|0;
  $154 = $153&65535;
  $155 = $end;
  $156 = ($154|0)<($155|0);
  if ($156) {
   HEAP8[$this>>0] = 0;
   $157 = $sym;
   $158 = $6;
   $159 = (($158) + ($157<<1)|0);
   $160 = HEAP16[$159>>1]|0;
   $161 = (($this) + 2|0);
   HEAP16[$161>>1] = $160;
  } else {
   $162 = $sym;
   $163 = $6;
   $164 = (($163) + ($162<<1)|0);
   $165 = HEAP16[$164>>1]|0;
   $166 = $165&65535;
   $167 = $end;
   $168 = ($166|0)>($167|0);
   if ($168) {
    $169 = $sym;
    $170 = $6;
    $171 = (($170) + ($169<<1)|0);
    $172 = HEAP16[$171>>1]|0;
    $173 = $172&65535;
    $174 = $extra;
    $175 = (($174) + ($173<<1)|0);
    $176 = HEAP16[$175>>1]|0;
    $177 = $176&255;
    HEAP8[$this>>0] = $177;
    $178 = $sym;
    $179 = $6;
    $180 = (($179) + ($178<<1)|0);
    $181 = HEAP16[$180>>1]|0;
    $182 = $181&65535;
    $183 = $base;
    $184 = (($183) + ($182<<1)|0);
    $185 = HEAP16[$184>>1]|0;
    $186 = (($this) + 2|0);
    HEAP16[$186>>1] = $185;
   } else {
    HEAP8[$this>>0] = 96;
    $187 = (($this) + 2|0);
    HEAP16[$187>>1] = 0;
   }
  }
  $188 = $len;
  $189 = $drop;
  $190 = (($188) - ($189))|0;
  $191 = 1 << $190;
  $incr = $191;
  $192 = $curr;
  $193 = 1 << $192;
  $fill = $193;
  $194 = $fill;
  $min = $194;
  while(1) {
   $195 = $incr;
   $196 = $fill;
   $197 = (($196) - ($195))|0;
   $fill = $197;
   $198 = $huff;
   $199 = $drop;
   $200 = $198 >>> $199;
   $201 = $fill;
   $202 = (($200) + ($201))|0;
   $203 = $next;
   $204 = (($203) + ($202<<2)|0);
   ;HEAP16[$204+0>>1]=HEAP16[$this+0>>1]|0;HEAP16[$204+2>>1]=HEAP16[$this+2>>1]|0;
   $205 = $fill;
   $206 = ($205|0)!=(0);
   if (!($206)) {
    break;
   }
  }
  $207 = $len;
  $208 = (($207) - 1)|0;
  $209 = 1 << $208;
  $incr = $209;
  while(1) {
   $210 = $huff;
   $211 = $incr;
   $212 = $210 & $211;
   $213 = ($212|0)!=(0);
   if (!($213)) {
    break;
   }
   $214 = $incr;
   $215 = $214 >>> 1;
   $incr = $215;
  }
  $216 = $incr;
  $217 = ($216|0)!=(0);
  if ($217) {
   $218 = $incr;
   $219 = (($218) - 1)|0;
   $220 = $huff;
   $221 = $220 & $219;
   $huff = $221;
   $222 = $incr;
   $223 = $huff;
   $224 = (($223) + ($222))|0;
   $huff = $224;
  } else {
   $huff = 0;
  }
  $225 = $sym;
  $226 = (($225) + 1)|0;
  $sym = $226;
  $227 = $len;
  $228 = (($count) + ($227<<1)|0);
  $229 = HEAP16[$228>>1]|0;
  $230 = (($229) + -1)<<16>>16;
  HEAP16[$228>>1] = $230;
  $231 = $230&65535;
  $232 = ($231|0)==(0);
  if ($232) {
   $233 = $len;
   $234 = $max;
   $235 = ($233|0)==($234|0);
   if ($235) {
    break;
   }
   $236 = $sym;
   $237 = $6;
   $238 = (($237) + ($236<<1)|0);
   $239 = HEAP16[$238>>1]|0;
   $240 = $239&65535;
   $241 = $2;
   $242 = (($241) + ($240<<1)|0);
   $243 = HEAP16[$242>>1]|0;
   $244 = $243&65535;
   $len = $244;
  }
  $245 = $len;
  $246 = $root;
  $247 = ($245>>>0)>($246>>>0);
  if ($247) {
   $248 = $huff;
   $249 = $mask;
   $250 = $248 & $249;
   $251 = $low;
   $252 = ($250|0)!=($251|0);
   if ($252) {
    $253 = $drop;
    $254 = ($253|0)==(0);
    if ($254) {
     $255 = $root;
     $drop = $255;
    }
    $256 = $min;
    $257 = $next;
    $258 = (($257) + ($256<<2)|0);
    $next = $258;
    $259 = $len;
    $260 = $drop;
    $261 = (($259) - ($260))|0;
    $curr = $261;
    $262 = $curr;
    $263 = 1 << $262;
    $left = $263;
    while(1) {
     $264 = $curr;
     $265 = $drop;
     $266 = (($264) + ($265))|0;
     $267 = $max;
     $268 = ($266>>>0)<($267>>>0);
     if (!($268)) {
      break;
     }
     $269 = $curr;
     $270 = $drop;
     $271 = (($269) + ($270))|0;
     $272 = (($count) + ($271<<1)|0);
     $273 = HEAP16[$272>>1]|0;
     $274 = $273&65535;
     $275 = $left;
     $276 = (($275) - ($274))|0;
     $left = $276;
     $277 = $left;
     $278 = ($277|0)<=(0);
     if ($278) {
      label = 81;
      break;
     }
     $279 = $curr;
     $280 = (($279) + 1)|0;
     $curr = $280;
     $281 = $left;
     $282 = $281 << 1;
     $left = $282;
    }
    if ((label|0) == 81) {
     label = 0;
    }
    $283 = $curr;
    $284 = 1 << $283;
    $285 = $used;
    $286 = (($285) + ($284))|0;
    $used = $286;
    $287 = $1;
    $288 = ($287|0)==(1);
    if ($288) {
     $289 = $used;
     $290 = ($289>>>0)>=(1456);
     if ($290) {
      label = 85;
      break;
     }
    }
    $291 = $huff;
    $292 = $mask;
    $293 = $291 & $292;
    $low = $293;
    $294 = $curr;
    $295 = $294&255;
    $296 = $low;
    $297 = $4;
    $298 = HEAP32[$297>>2]|0;
    $299 = (($298) + ($296<<2)|0);
    HEAP8[$299>>0] = $295;
    $300 = $root;
    $301 = $300&255;
    $302 = $low;
    $303 = $4;
    $304 = HEAP32[$303>>2]|0;
    $305 = (($304) + ($302<<2)|0);
    $306 = (($305) + 1|0);
    HEAP8[$306>>0] = $301;
    $307 = $next;
    $308 = $4;
    $309 = HEAP32[$308>>2]|0;
    $310 = $307;
    $311 = $309;
    $312 = (($310) - ($311))|0;
    $313 = (($312|0) / 4)&-1;
    $314 = $313&65535;
    $315 = $low;
    $316 = $4;
    $317 = HEAP32[$316>>2]|0;
    $318 = (($317) + ($315<<2)|0);
    $319 = (($318) + 2|0);
    HEAP16[$319>>1] = $314;
   }
  }
 }
 if ((label|0) == 85) {
  $0 = 1;
  $370 = $0;
  STACKTOP = sp;return ($370|0);
 }
 HEAP8[$this>>0] = 64;
 $320 = $len;
 $321 = $drop;
 $322 = (($320) - ($321))|0;
 $323 = $322&255;
 $324 = (($this) + 1|0);
 HEAP8[$324>>0] = $323;
 $325 = (($this) + 2|0);
 HEAP16[$325>>1] = 0;
 while(1) {
  $326 = $huff;
  $327 = ($326|0)!=(0);
  if (!($327)) {
   break;
  }
  $328 = $drop;
  $329 = ($328|0)!=(0);
  if ($329) {
   $330 = $huff;
   $331 = $mask;
   $332 = $330 & $331;
   $333 = $low;
   $334 = ($332|0)!=($333|0);
   if ($334) {
    $drop = 0;
    $335 = $root;
    $len = $335;
    $336 = $4;
    $337 = HEAP32[$336>>2]|0;
    $next = $337;
    $338 = $len;
    $339 = $338&255;
    $340 = (($this) + 1|0);
    HEAP8[$340>>0] = $339;
   }
  }
  $341 = $huff;
  $342 = $drop;
  $343 = $341 >>> $342;
  $344 = $next;
  $345 = (($344) + ($343<<2)|0);
  ;HEAP16[$345+0>>1]=HEAP16[$this+0>>1]|0;HEAP16[$345+2>>1]=HEAP16[$this+2>>1]|0;
  $346 = $len;
  $347 = (($346) - 1)|0;
  $348 = 1 << $347;
  $incr = $348;
  while(1) {
   $349 = $huff;
   $350 = $incr;
   $351 = $349 & $350;
   $352 = ($351|0)!=(0);
   if (!($352)) {
    break;
   }
   $353 = $incr;
   $354 = $353 >>> 1;
   $incr = $354;
  }
  $355 = $incr;
  $356 = ($355|0)!=(0);
  if ($356) {
   $357 = $incr;
   $358 = (($357) - 1)|0;
   $359 = $huff;
   $360 = $359 & $358;
   $huff = $360;
   $361 = $incr;
   $362 = $huff;
   $363 = (($362) + ($361))|0;
   $huff = $363;
  } else {
   $huff = 0;
  }
 }
 $364 = $used;
 $365 = $4;
 $366 = HEAP32[$365>>2]|0;
 $367 = (($366) + ($364<<2)|0);
 HEAP32[$365>>2] = $367;
 $368 = $root;
 $369 = $5;
 HEAP32[$369>>2] = $368;
 $0 = 0;
 $370 = $0;
 STACKTOP = sp;return ($370|0);
}
function _zcalloc($opaque,$items,$size) {
 $opaque = $opaque|0;
 $items = $items|0;
 $size = $size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $opaque;
 $1 = $items;
 $2 = $size;
 $3 = $0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = $2;
  $6 = $2;
  $7 = (($5) - ($6))|0;
  $8 = $1;
  $9 = (($8) + ($7))|0;
  $1 = $9;
 }
 $10 = $1;
 $11 = $2;
 $12 = Math_imul($10, $11)|0;
 $13 = (_malloc($12)|0);
 STACKTOP = sp;return ($13|0);
}
function _zcfree($opaque,$ptr) {
 $opaque = $opaque|0;
 $ptr = $ptr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $opaque;
 $1 = $ptr;
 $2 = $1;
 _free($2);
 $3 = $0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
 }
 STACKTOP = sp;return;
}
function _adler32($adler,$buf,$len) {
 $adler = $adler|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $n = 0, $sum2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $adler;
 $2 = $buf;
 $3 = $len;
 $4 = $1;
 $5 = $4 >>> 16;
 $6 = $5 & 65535;
 $sum2 = $6;
 $7 = $1;
 $8 = $7 & 65535;
 $1 = $8;
 $9 = $3;
 $10 = ($9|0)==(1);
 if ($10) {
  $11 = $2;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = $1;
  $15 = (($14) + ($13))|0;
  $1 = $15;
  $16 = $1;
  $17 = ($16>>>0)>=(65521);
  if ($17) {
   $18 = $1;
   $19 = (($18) - 65521)|0;
   $1 = $19;
  }
  $20 = $1;
  $21 = $sum2;
  $22 = (($21) + ($20))|0;
  $sum2 = $22;
  $23 = $sum2;
  $24 = ($23>>>0)>=(65521);
  if ($24) {
   $25 = $sum2;
   $26 = (($25) - 65521)|0;
   $sum2 = $26;
  }
  $27 = $1;
  $28 = $sum2;
  $29 = $28 << 16;
  $30 = $27 | $29;
  $0 = $30;
  $384 = $0;
  STACKTOP = sp;return ($384|0);
 }
 $31 = $2;
 $32 = ($31|0)==(0|0);
 if ($32) {
  $0 = 1;
  $384 = $0;
  STACKTOP = sp;return ($384|0);
 }
 $33 = $3;
 $34 = ($33>>>0)<(16);
 if ($34) {
  while(1) {
   $35 = $3;
   $36 = (($35) + -1)|0;
   $3 = $36;
   $37 = ($35|0)!=(0);
   if (!($37)) {
    break;
   }
   $38 = $2;
   $39 = (($38) + 1|0);
   $2 = $39;
   $40 = HEAP8[$38>>0]|0;
   $41 = $40&255;
   $42 = $1;
   $43 = (($42) + ($41))|0;
   $1 = $43;
   $44 = $1;
   $45 = $sum2;
   $46 = (($45) + ($44))|0;
   $sum2 = $46;
  }
  $47 = $1;
  $48 = ($47>>>0)>=(65521);
  if ($48) {
   $49 = $1;
   $50 = (($49) - 65521)|0;
   $1 = $50;
  }
  $51 = $sum2;
  $52 = (($51>>>0) % 65521)&-1;
  $sum2 = $52;
  $53 = $1;
  $54 = $sum2;
  $55 = $54 << 16;
  $56 = $53 | $55;
  $0 = $56;
  $384 = $0;
  STACKTOP = sp;return ($384|0);
 }
 while(1) {
  $57 = $3;
  $58 = ($57>>>0)>=(5552);
  if (!($58)) {
   break;
  }
  $59 = $3;
  $60 = (($59) - 5552)|0;
  $3 = $60;
  $n = 347;
  while(1) {
   $61 = $2;
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = $1;
   $65 = (($64) + ($63))|0;
   $1 = $65;
   $66 = $1;
   $67 = $sum2;
   $68 = (($67) + ($66))|0;
   $sum2 = $68;
   $69 = $2;
   $70 = (($69) + 1|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = $1;
   $74 = (($73) + ($72))|0;
   $1 = $74;
   $75 = $1;
   $76 = $sum2;
   $77 = (($76) + ($75))|0;
   $sum2 = $77;
   $78 = $2;
   $79 = (($78) + 2|0);
   $80 = HEAP8[$79>>0]|0;
   $81 = $80&255;
   $82 = $1;
   $83 = (($82) + ($81))|0;
   $1 = $83;
   $84 = $1;
   $85 = $sum2;
   $86 = (($85) + ($84))|0;
   $sum2 = $86;
   $87 = $2;
   $88 = (($87) + 3|0);
   $89 = HEAP8[$88>>0]|0;
   $90 = $89&255;
   $91 = $1;
   $92 = (($91) + ($90))|0;
   $1 = $92;
   $93 = $1;
   $94 = $sum2;
   $95 = (($94) + ($93))|0;
   $sum2 = $95;
   $96 = $2;
   $97 = (($96) + 4|0);
   $98 = HEAP8[$97>>0]|0;
   $99 = $98&255;
   $100 = $1;
   $101 = (($100) + ($99))|0;
   $1 = $101;
   $102 = $1;
   $103 = $sum2;
   $104 = (($103) + ($102))|0;
   $sum2 = $104;
   $105 = $2;
   $106 = (($105) + 5|0);
   $107 = HEAP8[$106>>0]|0;
   $108 = $107&255;
   $109 = $1;
   $110 = (($109) + ($108))|0;
   $1 = $110;
   $111 = $1;
   $112 = $sum2;
   $113 = (($112) + ($111))|0;
   $sum2 = $113;
   $114 = $2;
   $115 = (($114) + 6|0);
   $116 = HEAP8[$115>>0]|0;
   $117 = $116&255;
   $118 = $1;
   $119 = (($118) + ($117))|0;
   $1 = $119;
   $120 = $1;
   $121 = $sum2;
   $122 = (($121) + ($120))|0;
   $sum2 = $122;
   $123 = $2;
   $124 = (($123) + 7|0);
   $125 = HEAP8[$124>>0]|0;
   $126 = $125&255;
   $127 = $1;
   $128 = (($127) + ($126))|0;
   $1 = $128;
   $129 = $1;
   $130 = $sum2;
   $131 = (($130) + ($129))|0;
   $sum2 = $131;
   $132 = $2;
   $133 = (($132) + 8|0);
   $134 = HEAP8[$133>>0]|0;
   $135 = $134&255;
   $136 = $1;
   $137 = (($136) + ($135))|0;
   $1 = $137;
   $138 = $1;
   $139 = $sum2;
   $140 = (($139) + ($138))|0;
   $sum2 = $140;
   $141 = $2;
   $142 = (($141) + 9|0);
   $143 = HEAP8[$142>>0]|0;
   $144 = $143&255;
   $145 = $1;
   $146 = (($145) + ($144))|0;
   $1 = $146;
   $147 = $1;
   $148 = $sum2;
   $149 = (($148) + ($147))|0;
   $sum2 = $149;
   $150 = $2;
   $151 = (($150) + 10|0);
   $152 = HEAP8[$151>>0]|0;
   $153 = $152&255;
   $154 = $1;
   $155 = (($154) + ($153))|0;
   $1 = $155;
   $156 = $1;
   $157 = $sum2;
   $158 = (($157) + ($156))|0;
   $sum2 = $158;
   $159 = $2;
   $160 = (($159) + 11|0);
   $161 = HEAP8[$160>>0]|0;
   $162 = $161&255;
   $163 = $1;
   $164 = (($163) + ($162))|0;
   $1 = $164;
   $165 = $1;
   $166 = $sum2;
   $167 = (($166) + ($165))|0;
   $sum2 = $167;
   $168 = $2;
   $169 = (($168) + 12|0);
   $170 = HEAP8[$169>>0]|0;
   $171 = $170&255;
   $172 = $1;
   $173 = (($172) + ($171))|0;
   $1 = $173;
   $174 = $1;
   $175 = $sum2;
   $176 = (($175) + ($174))|0;
   $sum2 = $176;
   $177 = $2;
   $178 = (($177) + 13|0);
   $179 = HEAP8[$178>>0]|0;
   $180 = $179&255;
   $181 = $1;
   $182 = (($181) + ($180))|0;
   $1 = $182;
   $183 = $1;
   $184 = $sum2;
   $185 = (($184) + ($183))|0;
   $sum2 = $185;
   $186 = $2;
   $187 = (($186) + 14|0);
   $188 = HEAP8[$187>>0]|0;
   $189 = $188&255;
   $190 = $1;
   $191 = (($190) + ($189))|0;
   $1 = $191;
   $192 = $1;
   $193 = $sum2;
   $194 = (($193) + ($192))|0;
   $sum2 = $194;
   $195 = $2;
   $196 = (($195) + 15|0);
   $197 = HEAP8[$196>>0]|0;
   $198 = $197&255;
   $199 = $1;
   $200 = (($199) + ($198))|0;
   $1 = $200;
   $201 = $1;
   $202 = $sum2;
   $203 = (($202) + ($201))|0;
   $sum2 = $203;
   $204 = $2;
   $205 = (($204) + 16|0);
   $2 = $205;
   $206 = $n;
   $207 = (($206) + -1)|0;
   $n = $207;
   $208 = ($207|0)!=(0);
   if (!($208)) {
    break;
   }
  }
  $209 = $1;
  $210 = (($209>>>0) % 65521)&-1;
  $1 = $210;
  $211 = $sum2;
  $212 = (($211>>>0) % 65521)&-1;
  $sum2 = $212;
 }
 $213 = $3;
 $214 = ($213|0)!=(0);
 if ($214) {
  while(1) {
   $215 = $3;
   $216 = ($215>>>0)>=(16);
   if (!($216)) {
    break;
   }
   $217 = $3;
   $218 = (($217) - 16)|0;
   $3 = $218;
   $219 = $2;
   $220 = HEAP8[$219>>0]|0;
   $221 = $220&255;
   $222 = $1;
   $223 = (($222) + ($221))|0;
   $1 = $223;
   $224 = $1;
   $225 = $sum2;
   $226 = (($225) + ($224))|0;
   $sum2 = $226;
   $227 = $2;
   $228 = (($227) + 1|0);
   $229 = HEAP8[$228>>0]|0;
   $230 = $229&255;
   $231 = $1;
   $232 = (($231) + ($230))|0;
   $1 = $232;
   $233 = $1;
   $234 = $sum2;
   $235 = (($234) + ($233))|0;
   $sum2 = $235;
   $236 = $2;
   $237 = (($236) + 2|0);
   $238 = HEAP8[$237>>0]|0;
   $239 = $238&255;
   $240 = $1;
   $241 = (($240) + ($239))|0;
   $1 = $241;
   $242 = $1;
   $243 = $sum2;
   $244 = (($243) + ($242))|0;
   $sum2 = $244;
   $245 = $2;
   $246 = (($245) + 3|0);
   $247 = HEAP8[$246>>0]|0;
   $248 = $247&255;
   $249 = $1;
   $250 = (($249) + ($248))|0;
   $1 = $250;
   $251 = $1;
   $252 = $sum2;
   $253 = (($252) + ($251))|0;
   $sum2 = $253;
   $254 = $2;
   $255 = (($254) + 4|0);
   $256 = HEAP8[$255>>0]|0;
   $257 = $256&255;
   $258 = $1;
   $259 = (($258) + ($257))|0;
   $1 = $259;
   $260 = $1;
   $261 = $sum2;
   $262 = (($261) + ($260))|0;
   $sum2 = $262;
   $263 = $2;
   $264 = (($263) + 5|0);
   $265 = HEAP8[$264>>0]|0;
   $266 = $265&255;
   $267 = $1;
   $268 = (($267) + ($266))|0;
   $1 = $268;
   $269 = $1;
   $270 = $sum2;
   $271 = (($270) + ($269))|0;
   $sum2 = $271;
   $272 = $2;
   $273 = (($272) + 6|0);
   $274 = HEAP8[$273>>0]|0;
   $275 = $274&255;
   $276 = $1;
   $277 = (($276) + ($275))|0;
   $1 = $277;
   $278 = $1;
   $279 = $sum2;
   $280 = (($279) + ($278))|0;
   $sum2 = $280;
   $281 = $2;
   $282 = (($281) + 7|0);
   $283 = HEAP8[$282>>0]|0;
   $284 = $283&255;
   $285 = $1;
   $286 = (($285) + ($284))|0;
   $1 = $286;
   $287 = $1;
   $288 = $sum2;
   $289 = (($288) + ($287))|0;
   $sum2 = $289;
   $290 = $2;
   $291 = (($290) + 8|0);
   $292 = HEAP8[$291>>0]|0;
   $293 = $292&255;
   $294 = $1;
   $295 = (($294) + ($293))|0;
   $1 = $295;
   $296 = $1;
   $297 = $sum2;
   $298 = (($297) + ($296))|0;
   $sum2 = $298;
   $299 = $2;
   $300 = (($299) + 9|0);
   $301 = HEAP8[$300>>0]|0;
   $302 = $301&255;
   $303 = $1;
   $304 = (($303) + ($302))|0;
   $1 = $304;
   $305 = $1;
   $306 = $sum2;
   $307 = (($306) + ($305))|0;
   $sum2 = $307;
   $308 = $2;
   $309 = (($308) + 10|0);
   $310 = HEAP8[$309>>0]|0;
   $311 = $310&255;
   $312 = $1;
   $313 = (($312) + ($311))|0;
   $1 = $313;
   $314 = $1;
   $315 = $sum2;
   $316 = (($315) + ($314))|0;
   $sum2 = $316;
   $317 = $2;
   $318 = (($317) + 11|0);
   $319 = HEAP8[$318>>0]|0;
   $320 = $319&255;
   $321 = $1;
   $322 = (($321) + ($320))|0;
   $1 = $322;
   $323 = $1;
   $324 = $sum2;
   $325 = (($324) + ($323))|0;
   $sum2 = $325;
   $326 = $2;
   $327 = (($326) + 12|0);
   $328 = HEAP8[$327>>0]|0;
   $329 = $328&255;
   $330 = $1;
   $331 = (($330) + ($329))|0;
   $1 = $331;
   $332 = $1;
   $333 = $sum2;
   $334 = (($333) + ($332))|0;
   $sum2 = $334;
   $335 = $2;
   $336 = (($335) + 13|0);
   $337 = HEAP8[$336>>0]|0;
   $338 = $337&255;
   $339 = $1;
   $340 = (($339) + ($338))|0;
   $1 = $340;
   $341 = $1;
   $342 = $sum2;
   $343 = (($342) + ($341))|0;
   $sum2 = $343;
   $344 = $2;
   $345 = (($344) + 14|0);
   $346 = HEAP8[$345>>0]|0;
   $347 = $346&255;
   $348 = $1;
   $349 = (($348) + ($347))|0;
   $1 = $349;
   $350 = $1;
   $351 = $sum2;
   $352 = (($351) + ($350))|0;
   $sum2 = $352;
   $353 = $2;
   $354 = (($353) + 15|0);
   $355 = HEAP8[$354>>0]|0;
   $356 = $355&255;
   $357 = $1;
   $358 = (($357) + ($356))|0;
   $1 = $358;
   $359 = $1;
   $360 = $sum2;
   $361 = (($360) + ($359))|0;
   $sum2 = $361;
   $362 = $2;
   $363 = (($362) + 16|0);
   $2 = $363;
  }
  while(1) {
   $364 = $3;
   $365 = (($364) + -1)|0;
   $3 = $365;
   $366 = ($364|0)!=(0);
   if (!($366)) {
    break;
   }
   $367 = $2;
   $368 = (($367) + 1|0);
   $2 = $368;
   $369 = HEAP8[$367>>0]|0;
   $370 = $369&255;
   $371 = $1;
   $372 = (($371) + ($370))|0;
   $1 = $372;
   $373 = $1;
   $374 = $sum2;
   $375 = (($374) + ($373))|0;
   $sum2 = $375;
  }
  $376 = $1;
  $377 = (($376>>>0) % 65521)&-1;
  $1 = $377;
  $378 = $sum2;
  $379 = (($378>>>0) % 65521)&-1;
  $sum2 = $379;
 }
 $380 = $1;
 $381 = $sum2;
 $382 = $381 << 16;
 $383 = $380 | $382;
 $0 = $383;
 $384 = $0;
 STACKTOP = sp;return ($384|0);
}
function _inflate_fast($strm,$start) {
 $strm = $strm|0;
 $start = $start|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $beg = 0, $bits = 0, $dcode = 0, $dist = 0, $dmask = 0, $end = 0, $from = 0, $hold = 0, $in = 0, $last = 0, $lcode = 0, $len = 0, $lmask = 0, $op = 0;
 var $out = 0, $state = 0, $this = 0, $whave = 0, $window = 0, $write = 0, $wsize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $this = sp + 88|0;
 $0 = $strm;
 $1 = $start;
 $2 = $0;
 $3 = (($2) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 $state = $4;
 $5 = $0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + -1|0);
 $in = $7;
 $8 = $in;
 $9 = $0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) - 5)|0;
 $13 = (($8) + ($12)|0);
 $last = $13;
 $14 = $0;
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + -1|0);
 $out = $17;
 $18 = $out;
 $19 = $1;
 $20 = $0;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($19) - ($22))|0;
 $24 = (0 - ($23))|0;
 $25 = (($18) + ($24)|0);
 $beg = $25;
 $26 = $out;
 $27 = $0;
 $28 = (($27) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) - 257)|0;
 $31 = (($26) + ($30)|0);
 $end = $31;
 $32 = $state;
 $33 = (($32) + 40|0);
 $34 = HEAP32[$33>>2]|0;
 $wsize = $34;
 $35 = $state;
 $36 = (($35) + 44|0);
 $37 = HEAP32[$36>>2]|0;
 $whave = $37;
 $38 = $state;
 $39 = (($38) + 48|0);
 $40 = HEAP32[$39>>2]|0;
 $write = $40;
 $41 = $state;
 $42 = (($41) + 52|0);
 $43 = HEAP32[$42>>2]|0;
 $window = $43;
 $44 = $state;
 $45 = (($44) + 56|0);
 $46 = HEAP32[$45>>2]|0;
 $hold = $46;
 $47 = $state;
 $48 = (($47) + 60|0);
 $49 = HEAP32[$48>>2]|0;
 $bits = $49;
 $50 = $state;
 $51 = (($50) + 76|0);
 $52 = HEAP32[$51>>2]|0;
 $lcode = $52;
 $53 = $state;
 $54 = (($53) + 80|0);
 $55 = HEAP32[$54>>2]|0;
 $dcode = $55;
 $56 = $state;
 $57 = (($56) + 84|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = 1 << $58;
 $60 = (($59) - 1)|0;
 $lmask = $60;
 $61 = $state;
 $62 = (($61) + 88|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = 1 << $63;
 $65 = (($64) - 1)|0;
 $dmask = $65;
 L1: while(1) {
  $66 = $bits;
  $67 = ($66>>>0)<(15);
  if ($67) {
   $68 = $in;
   $69 = (($68) + 1|0);
   $in = $69;
   $70 = HEAP8[$69>>0]|0;
   $71 = $70&255;
   $72 = $bits;
   $73 = $71 << $72;
   $74 = $hold;
   $75 = (($74) + ($73))|0;
   $hold = $75;
   $76 = $bits;
   $77 = (($76) + 8)|0;
   $bits = $77;
   $78 = $in;
   $79 = (($78) + 1|0);
   $in = $79;
   $80 = HEAP8[$79>>0]|0;
   $81 = $80&255;
   $82 = $bits;
   $83 = $81 << $82;
   $84 = $hold;
   $85 = (($84) + ($83))|0;
   $hold = $85;
   $86 = $bits;
   $87 = (($86) + 8)|0;
   $bits = $87;
  }
  $88 = $hold;
  $89 = $lmask;
  $90 = $88 & $89;
  $91 = $lcode;
  $92 = (($91) + ($90<<2)|0);
  ;HEAP16[$this+0>>1]=HEAP16[$92+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$92+2>>1]|0;
  while(1) {
   $93 = (($this) + 1|0);
   $94 = HEAP8[$93>>0]|0;
   $95 = $94&255;
   $op = $95;
   $96 = $op;
   $97 = $hold;
   $98 = $97 >>> $96;
   $hold = $98;
   $99 = $op;
   $100 = $bits;
   $101 = (($100) - ($99))|0;
   $bits = $101;
   $102 = HEAP8[$this>>0]|0;
   $103 = $102&255;
   $op = $103;
   $104 = $op;
   $105 = ($104|0)==(0);
   if ($105) {
    label = 6;
    break;
   }
   $111 = $op;
   $112 = $111 & 16;
   $113 = ($112|0)!=(0);
   if ($113) {
    label = 8;
    break;
   }
   $434 = $op;
   $435 = $434 & 64;
   $436 = ($435|0)==(0);
   if (!($436)) {
    label = 72;
    break L1;
   }
   $437 = (($this) + 2|0);
   $438 = HEAP16[$437>>1]|0;
   $439 = $438&65535;
   $440 = $hold;
   $441 = $op;
   $442 = 1 << $441;
   $443 = (($442) - 1)|0;
   $444 = $440 & $443;
   $445 = (($439) + ($444))|0;
   $446 = $lcode;
   $447 = (($446) + ($445<<2)|0);
   ;HEAP16[$this+0>>1]=HEAP16[$447+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$447+2>>1]|0;
  }
  if ((label|0) == 6) {
   label = 0;
   $106 = (($this) + 2|0);
   $107 = HEAP16[$106>>1]|0;
   $108 = $107&255;
   $109 = $out;
   $110 = (($109) + 1|0);
   $out = $110;
   HEAP8[$110>>0] = $108;
  }
  else if ((label|0) == 8) {
   label = 0;
   $114 = (($this) + 2|0);
   $115 = HEAP16[$114>>1]|0;
   $116 = $115&65535;
   $len = $116;
   $117 = $op;
   $118 = $117 & 15;
   $op = $118;
   $119 = $op;
   $120 = ($119|0)!=(0);
   if ($120) {
    $121 = $bits;
    $122 = $op;
    $123 = ($121>>>0)<($122>>>0);
    if ($123) {
     $124 = $in;
     $125 = (($124) + 1|0);
     $in = $125;
     $126 = HEAP8[$125>>0]|0;
     $127 = $126&255;
     $128 = $bits;
     $129 = $127 << $128;
     $130 = $hold;
     $131 = (($130) + ($129))|0;
     $hold = $131;
     $132 = $bits;
     $133 = (($132) + 8)|0;
     $bits = $133;
    }
    $134 = $hold;
    $135 = $op;
    $136 = 1 << $135;
    $137 = (($136) - 1)|0;
    $138 = $134 & $137;
    $139 = $len;
    $140 = (($139) + ($138))|0;
    $len = $140;
    $141 = $op;
    $142 = $hold;
    $143 = $142 >>> $141;
    $hold = $143;
    $144 = $op;
    $145 = $bits;
    $146 = (($145) - ($144))|0;
    $bits = $146;
   }
   $147 = $bits;
   $148 = ($147>>>0)<(15);
   if ($148) {
    $149 = $in;
    $150 = (($149) + 1|0);
    $in = $150;
    $151 = HEAP8[$150>>0]|0;
    $152 = $151&255;
    $153 = $bits;
    $154 = $152 << $153;
    $155 = $hold;
    $156 = (($155) + ($154))|0;
    $hold = $156;
    $157 = $bits;
    $158 = (($157) + 8)|0;
    $bits = $158;
    $159 = $in;
    $160 = (($159) + 1|0);
    $in = $160;
    $161 = HEAP8[$160>>0]|0;
    $162 = $161&255;
    $163 = $bits;
    $164 = $162 << $163;
    $165 = $hold;
    $166 = (($165) + ($164))|0;
    $hold = $166;
    $167 = $bits;
    $168 = (($167) + 8)|0;
    $bits = $168;
   }
   $169 = $hold;
   $170 = $dmask;
   $171 = $169 & $170;
   $172 = $dcode;
   $173 = (($172) + ($171<<2)|0);
   ;HEAP16[$this+0>>1]=HEAP16[$173+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$173+2>>1]|0;
   while(1) {
    $174 = (($this) + 1|0);
    $175 = HEAP8[$174>>0]|0;
    $176 = $175&255;
    $op = $176;
    $177 = $op;
    $178 = $hold;
    $179 = $178 >>> $177;
    $hold = $179;
    $180 = $op;
    $181 = $bits;
    $182 = (($181) - ($180))|0;
    $bits = $182;
    $183 = HEAP8[$this>>0]|0;
    $184 = $183&255;
    $op = $184;
    $185 = $op;
    $186 = $185 & 16;
    $187 = ($186|0)!=(0);
    if ($187) {
     break;
    }
    $417 = $op;
    $418 = $417 & 64;
    $419 = ($418|0)==(0);
    if (!($419)) {
     label = 68;
     break L1;
    }
    $420 = (($this) + 2|0);
    $421 = HEAP16[$420>>1]|0;
    $422 = $421&65535;
    $423 = $hold;
    $424 = $op;
    $425 = 1 << $424;
    $426 = (($425) - 1)|0;
    $427 = $423 & $426;
    $428 = (($422) + ($427))|0;
    $429 = $dcode;
    $430 = (($429) + ($428<<2)|0);
    ;HEAP16[$this+0>>1]=HEAP16[$430+0>>1]|0;HEAP16[$this+2>>1]=HEAP16[$430+2>>1]|0;
   }
   $188 = (($this) + 2|0);
   $189 = HEAP16[$188>>1]|0;
   $190 = $189&65535;
   $dist = $190;
   $191 = $op;
   $192 = $191 & 15;
   $op = $192;
   $193 = $bits;
   $194 = $op;
   $195 = ($193>>>0)<($194>>>0);
   if ($195) {
    $196 = $in;
    $197 = (($196) + 1|0);
    $in = $197;
    $198 = HEAP8[$197>>0]|0;
    $199 = $198&255;
    $200 = $bits;
    $201 = $199 << $200;
    $202 = $hold;
    $203 = (($202) + ($201))|0;
    $hold = $203;
    $204 = $bits;
    $205 = (($204) + 8)|0;
    $bits = $205;
    $206 = $bits;
    $207 = $op;
    $208 = ($206>>>0)<($207>>>0);
    if ($208) {
     $209 = $in;
     $210 = (($209) + 1|0);
     $in = $210;
     $211 = HEAP8[$210>>0]|0;
     $212 = $211&255;
     $213 = $bits;
     $214 = $212 << $213;
     $215 = $hold;
     $216 = (($215) + ($214))|0;
     $hold = $216;
     $217 = $bits;
     $218 = (($217) + 8)|0;
     $bits = $218;
    }
   }
   $219 = $hold;
   $220 = $op;
   $221 = 1 << $220;
   $222 = (($221) - 1)|0;
   $223 = $219 & $222;
   $224 = $dist;
   $225 = (($224) + ($223))|0;
   $dist = $225;
   $226 = $op;
   $227 = $hold;
   $228 = $227 >>> $226;
   $hold = $228;
   $229 = $op;
   $230 = $bits;
   $231 = (($230) - ($229))|0;
   $bits = $231;
   $232 = $out;
   $233 = $beg;
   $234 = $232;
   $235 = $233;
   $236 = (($234) - ($235))|0;
   $op = $236;
   $237 = $dist;
   $238 = $op;
   $239 = ($237>>>0)>($238>>>0);
   if ($239) {
    $240 = $dist;
    $241 = $op;
    $242 = (($240) - ($241))|0;
    $op = $242;
    $243 = $op;
    $244 = $whave;
    $245 = ($243>>>0)>($244>>>0);
    if ($245) {
     label = 22;
     break;
    }
    $249 = $window;
    $250 = (($249) + -1|0);
    $from = $250;
    $251 = $write;
    $252 = ($251|0)==(0);
    if ($252) {
     $253 = $wsize;
     $254 = $op;
     $255 = (($253) - ($254))|0;
     $256 = $from;
     $257 = (($256) + ($255)|0);
     $from = $257;
     $258 = $op;
     $259 = $len;
     $260 = ($258>>>0)<($259>>>0);
     if ($260) {
      $261 = $op;
      $262 = $len;
      $263 = (($262) - ($261))|0;
      $len = $263;
      while(1) {
       $264 = $from;
       $265 = (($264) + 1|0);
       $from = $265;
       $266 = HEAP8[$265>>0]|0;
       $267 = $out;
       $268 = (($267) + 1|0);
       $out = $268;
       HEAP8[$268>>0] = $266;
       $269 = $op;
       $270 = (($269) + -1)|0;
       $op = $270;
       $271 = ($270|0)!=(0);
       if (!($271)) {
        break;
       }
      }
      $272 = $out;
      $273 = $dist;
      $274 = (0 - ($273))|0;
      $275 = (($272) + ($274)|0);
      $from = $275;
     }
    } else {
     $276 = $write;
     $277 = $op;
     $278 = ($276>>>0)<($277>>>0);
     if ($278) {
      $279 = $wsize;
      $280 = $write;
      $281 = (($279) + ($280))|0;
      $282 = $op;
      $283 = (($281) - ($282))|0;
      $284 = $from;
      $285 = (($284) + ($283)|0);
      $from = $285;
      $286 = $write;
      $287 = $op;
      $288 = (($287) - ($286))|0;
      $op = $288;
      $289 = $op;
      $290 = $len;
      $291 = ($289>>>0)<($290>>>0);
      if ($291) {
       $292 = $op;
       $293 = $len;
       $294 = (($293) - ($292))|0;
       $len = $294;
       while(1) {
        $295 = $from;
        $296 = (($295) + 1|0);
        $from = $296;
        $297 = HEAP8[$296>>0]|0;
        $298 = $out;
        $299 = (($298) + 1|0);
        $out = $299;
        HEAP8[$299>>0] = $297;
        $300 = $op;
        $301 = (($300) + -1)|0;
        $op = $301;
        $302 = ($301|0)!=(0);
        if (!($302)) {
         break;
        }
       }
       $303 = $window;
       $304 = (($303) + -1|0);
       $from = $304;
       $305 = $write;
       $306 = $len;
       $307 = ($305>>>0)<($306>>>0);
       if ($307) {
        $308 = $write;
        $op = $308;
        $309 = $op;
        $310 = $len;
        $311 = (($310) - ($309))|0;
        $len = $311;
        while(1) {
         $312 = $from;
         $313 = (($312) + 1|0);
         $from = $313;
         $314 = HEAP8[$313>>0]|0;
         $315 = $out;
         $316 = (($315) + 1|0);
         $out = $316;
         HEAP8[$316>>0] = $314;
         $317 = $op;
         $318 = (($317) + -1)|0;
         $op = $318;
         $319 = ($318|0)!=(0);
         if (!($319)) {
          break;
         }
        }
        $320 = $out;
        $321 = $dist;
        $322 = (0 - ($321))|0;
        $323 = (($320) + ($322)|0);
        $from = $323;
       }
      }
     } else {
      $324 = $write;
      $325 = $op;
      $326 = (($324) - ($325))|0;
      $327 = $from;
      $328 = (($327) + ($326)|0);
      $from = $328;
      $329 = $op;
      $330 = $len;
      $331 = ($329>>>0)<($330>>>0);
      if ($331) {
       $332 = $op;
       $333 = $len;
       $334 = (($333) - ($332))|0;
       $len = $334;
       while(1) {
        $335 = $from;
        $336 = (($335) + 1|0);
        $from = $336;
        $337 = HEAP8[$336>>0]|0;
        $338 = $out;
        $339 = (($338) + 1|0);
        $out = $339;
        HEAP8[$339>>0] = $337;
        $340 = $op;
        $341 = (($340) + -1)|0;
        $op = $341;
        $342 = ($341|0)!=(0);
        if (!($342)) {
         break;
        }
       }
       $343 = $out;
       $344 = $dist;
       $345 = (0 - ($344))|0;
       $346 = (($343) + ($345)|0);
       $from = $346;
      }
     }
    }
    while(1) {
     $347 = $len;
     $348 = ($347>>>0)>(2);
     if (!($348)) {
      break;
     }
     $349 = $from;
     $350 = (($349) + 1|0);
     $from = $350;
     $351 = HEAP8[$350>>0]|0;
     $352 = $out;
     $353 = (($352) + 1|0);
     $out = $353;
     HEAP8[$353>>0] = $351;
     $354 = $from;
     $355 = (($354) + 1|0);
     $from = $355;
     $356 = HEAP8[$355>>0]|0;
     $357 = $out;
     $358 = (($357) + 1|0);
     $out = $358;
     HEAP8[$358>>0] = $356;
     $359 = $from;
     $360 = (($359) + 1|0);
     $from = $360;
     $361 = HEAP8[$360>>0]|0;
     $362 = $out;
     $363 = (($362) + 1|0);
     $out = $363;
     HEAP8[$363>>0] = $361;
     $364 = $len;
     $365 = (($364) - 3)|0;
     $len = $365;
    }
    $366 = $len;
    $367 = ($366|0)!=(0);
    if ($367) {
     $368 = $from;
     $369 = (($368) + 1|0);
     $from = $369;
     $370 = HEAP8[$369>>0]|0;
     $371 = $out;
     $372 = (($371) + 1|0);
     $out = $372;
     HEAP8[$372>>0] = $370;
     $373 = $len;
     $374 = ($373>>>0)>(1);
     if ($374) {
      $375 = $from;
      $376 = (($375) + 1|0);
      $from = $376;
      $377 = HEAP8[$376>>0]|0;
      $378 = $out;
      $379 = (($378) + 1|0);
      $out = $379;
      HEAP8[$379>>0] = $377;
     }
    }
   } else {
    $380 = $out;
    $381 = $dist;
    $382 = (0 - ($381))|0;
    $383 = (($380) + ($382)|0);
    $from = $383;
    while(1) {
     $384 = $from;
     $385 = (($384) + 1|0);
     $from = $385;
     $386 = HEAP8[$385>>0]|0;
     $387 = $out;
     $388 = (($387) + 1|0);
     $out = $388;
     HEAP8[$388>>0] = $386;
     $389 = $from;
     $390 = (($389) + 1|0);
     $from = $390;
     $391 = HEAP8[$390>>0]|0;
     $392 = $out;
     $393 = (($392) + 1|0);
     $out = $393;
     HEAP8[$393>>0] = $391;
     $394 = $from;
     $395 = (($394) + 1|0);
     $from = $395;
     $396 = HEAP8[$395>>0]|0;
     $397 = $out;
     $398 = (($397) + 1|0);
     $out = $398;
     HEAP8[$398>>0] = $396;
     $399 = $len;
     $400 = (($399) - 3)|0;
     $len = $400;
     $401 = $len;
     $402 = ($401>>>0)>(2);
     if (!($402)) {
      break;
     }
    }
    $403 = $len;
    $404 = ($403|0)!=(0);
    if ($404) {
     $405 = $from;
     $406 = (($405) + 1|0);
     $from = $406;
     $407 = HEAP8[$406>>0]|0;
     $408 = $out;
     $409 = (($408) + 1|0);
     $out = $409;
     HEAP8[$409>>0] = $407;
     $410 = $len;
     $411 = ($410>>>0)>(1);
     if ($411) {
      $412 = $from;
      $413 = (($412) + 1|0);
      $from = $413;
      $414 = HEAP8[$413>>0]|0;
      $415 = $out;
      $416 = (($415) + 1|0);
      $out = $416;
      HEAP8[$416>>0] = $414;
     }
    }
   }
  }
  $455 = $in;
  $456 = $last;
  $457 = ($455>>>0)<($456>>>0);
  if ($457) {
   $458 = $out;
   $459 = $end;
   $460 = ($458>>>0)<($459>>>0);
   $525 = $460;
  } else {
   $525 = 0;
  }
  if (!($525)) {
   break;
  }
 }
 do {
  if ((label|0) == 22) {
   $246 = $0;
   $247 = (($246) + 24|0);
   HEAP32[$247>>2] = 76136;
   $248 = $state;
   HEAP32[$248>>2] = 27;
  }
  else if ((label|0) == 68) {
   $431 = $0;
   $432 = (($431) + 24|0);
   HEAP32[$432>>2] = 76168;
   $433 = $state;
   HEAP32[$433>>2] = 27;
  }
  else if ((label|0) == 72) {
   $448 = $op;
   $449 = $448 & 32;
   $450 = ($449|0)!=(0);
   if ($450) {
    $451 = $state;
    HEAP32[$451>>2] = 11;
    break;
   } else {
    $452 = $0;
    $453 = (($452) + 24|0);
    HEAP32[$453>>2] = 76192;
    $454 = $state;
    HEAP32[$454>>2] = 27;
    break;
   }
  }
 } while(0);
 $461 = $bits;
 $462 = $461 >>> 3;
 $len = $462;
 $463 = $len;
 $464 = $in;
 $465 = (0 - ($463))|0;
 $466 = (($464) + ($465)|0);
 $in = $466;
 $467 = $len;
 $468 = $467 << 3;
 $469 = $bits;
 $470 = (($469) - ($468))|0;
 $bits = $470;
 $471 = $bits;
 $472 = 1 << $471;
 $473 = (($472) - 1)|0;
 $474 = $hold;
 $475 = $474 & $473;
 $hold = $475;
 $476 = $in;
 $477 = (($476) + 1|0);
 $478 = $0;
 HEAP32[$478>>2] = $477;
 $479 = $out;
 $480 = (($479) + 1|0);
 $481 = $0;
 $482 = (($481) + 12|0);
 HEAP32[$482>>2] = $480;
 $483 = $in;
 $484 = $last;
 $485 = ($483>>>0)<($484>>>0);
 if ($485) {
  $486 = $last;
  $487 = $in;
  $488 = $486;
  $489 = $487;
  $490 = (($488) - ($489))|0;
  $491 = (5 + ($490))|0;
  $500 = $491;
 } else {
  $492 = $in;
  $493 = $last;
  $494 = $492;
  $495 = $493;
  $496 = (($494) - ($495))|0;
  $497 = (5 - ($496))|0;
  $500 = $497;
 }
 $498 = $0;
 $499 = (($498) + 4|0);
 HEAP32[$499>>2] = $500;
 $501 = $out;
 $502 = $end;
 $503 = ($501>>>0)<($502>>>0);
 if ($503) {
  $504 = $end;
  $505 = $out;
  $506 = $504;
  $507 = $505;
  $508 = (($506) - ($507))|0;
  $509 = (257 + ($508))|0;
  $518 = $509;
  $516 = $0;
  $517 = (($516) + 16|0);
  HEAP32[$517>>2] = $518;
  $519 = $hold;
  $520 = $state;
  $521 = (($520) + 56|0);
  HEAP32[$521>>2] = $519;
  $522 = $bits;
  $523 = $state;
  $524 = (($523) + 60|0);
  HEAP32[$524>>2] = $522;
  STACKTOP = sp;return;
 } else {
  $510 = $out;
  $511 = $end;
  $512 = $510;
  $513 = $511;
  $514 = (($512) - ($513))|0;
  $515 = (257 - ($514))|0;
  $518 = $515;
  $516 = $0;
  $517 = (($516) + 16|0);
  HEAP32[$517>>2] = $518;
  $519 = $hold;
  $520 = $state;
  $521 = (($520) + 56|0);
  HEAP32[$521>>2] = $519;
  $522 = $bits;
  $523 = $state;
  $524 = (($523) + 60|0);
  HEAP32[$524>>2] = $522;
  STACKTOP = sp;return;
 }
}
function _mbrtowc($wc,$src,$n,$st) {
 $wc = $wc|0;
 $src = $src|0;
 $n = $n|0;
 $st = $st|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c$0 = 0, $c$1 = 0, $c$2 = 0, $s$0 = 0, $s$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $wc;
 $1 = ($st|0)==(0|0);
 $$01 = $1 ? 76432 : $st;
 $2 = HEAP32[$$01>>2]|0;
 $3 = ($src|0)==(0|0);
 L1: do {
  if ($3) {
   $4 = ($2|0)==(0);
   if ($4) {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  } else {
   $5 = ($wc|0)==(0|0);
   if ($5) {
    HEAP32[$0>>2] = $0;
    $11 = $0;
   } else {
    $11 = $wc;
   }
   $6 = ($n|0)==(0);
   if ($6) {
    $$0 = -2;
    STACKTOP = sp;return ($$0|0);
   }
   $7 = ($2|0)==(0);
   do {
    if ($7) {
     $8 = HEAP8[$src>>0]|0;
     $9 = $8&255;
     $10 = ($8<<24>>24)>(-1);
     if ($10) {
      HEAP32[$11>>2] = $9;
      $12 = ($8<<24>>24)!=(0);
      $13 = $12&1;
      $$0 = $13;
      STACKTOP = sp;return ($$0|0);
     } else {
      $14 = (($9) + -194)|0;
      $15 = ($14>>>0)>(50);
      if ($15) {
       break L1;
      }
      $16 = (($src) + 1|0);
      $17 = (76224 + ($14<<2)|0);
      $18 = HEAP32[$17>>2]|0;
      $19 = (($n) + -1)|0;
      $$02 = $19;$c$0 = $18;$s$0 = $16;
      break;
     }
    } else {
     $$02 = $n;$c$0 = $2;$s$0 = $src;
    }
   } while(0);
   $20 = ($$02|0)==(0);
   L17: do {
    if ($20) {
     $c$2 = $c$0;
    } else {
     $21 = HEAP8[$s$0>>0]|0;
     $22 = $21&255;
     $23 = $22 >>> 3;
     $24 = (($23) + -16)|0;
     $25 = $c$0 >> 26;
     $26 = (($23) + ($25))|0;
     $27 = $24 | $26;
     $28 = ($27>>>0)>(7);
     if ($28) {
      break L1;
     } else {
      $$1 = $$02;$32 = $21;$c$1 = $c$0;$s$1 = $s$0;
     }
     while(1) {
      $29 = $c$1 << 6;
      $30 = (($s$1) + 1|0);
      $31 = $32&255;
      $33 = (($31) + -128)|0;
      $34 = $33 | $29;
      $35 = (($$1) + -1)|0;
      $36 = ($34|0)<(0);
      if (!($36)) {
       break;
      }
      $38 = ($35|0)==(0);
      if ($38) {
       $c$2 = $34;
       break L17;
      }
      $39 = HEAP8[$30>>0]|0;
      $40 = $39&255;
      $41 = (($40) + -128)|0;
      $42 = ($41>>>0)>(63);
      if ($42) {
       break L1;
      } else {
       $$1 = $35;$32 = $39;$c$1 = $34;$s$1 = $30;
      }
     }
     HEAP32[$$01>>2] = 0;
     HEAP32[$11>>2] = $34;
     $37 = (($n) - ($35))|0;
     $$0 = $37;
     STACKTOP = sp;return ($$0|0);
    }
   } while(0);
   HEAP32[$$01>>2] = $c$2;
   $$0 = -2;
   STACKTOP = sp;return ($$0|0);
  }
 } while(0);
 HEAP32[$$01>>2] = 0;
 $43 = (___errno_location()|0);
 HEAP32[$43>>2] = 84;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function _mbsinit($st) {
 $st = $st|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($st|0)==(0|0);
 if ($0) {
  $4 = 1;
 } else {
  $1 = HEAP32[$st>>2]|0;
  $2 = ($1|0)==(0);
  $4 = $2;
 }
 $3 = $4&1;
 STACKTOP = sp;return ($3|0);
}
function ___string_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $k$0 = 0, $k$0$len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 84|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($len) + 256)|0;
 $3 = (_memchr($1,0,$2)|0);
 $4 = ($3|0)==(0|0);
 if ($4) {
  $k$0 = $2;
 } else {
  $5 = $3;
  $6 = $1;
  $7 = (($5) - ($6))|0;
  $k$0 = $7;
 }
 $8 = ($k$0>>>0)<($len>>>0);
 $k$0$len = $8 ? $k$0 : $len;
 _memcpy(($buf|0),($1|0),($k$0$len|0))|0;
 $9 = (($1) + ($k$0$len)|0);
 $10 = (($f) + 4|0);
 HEAP32[$10>>2] = $9;
 $11 = (($1) + ($k$0)|0);
 $12 = (($f) + 8|0);
 HEAP32[$12>>2] = $11;
 HEAP32[$0>>2] = $11;
 STACKTOP = sp;return ($k$0$len|0);
}
function _sscanf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsscanf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfscanf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$10 = 0, $$4 = 0, $$7 = 0, $$8 = 0, $$9 = 0, $$lcssa18 = 0, $$matches$0 = 0, $$not = 0, $$pre = 0, $$pre$phi213Z2D = 0, $$pre193 = 0, $$pre206 = 0, $$pre207 = 0, $$pre208 = 0, $$pre209 = 0, $$pre210 = 0, $$pre211 = 0, $$pre212 = 0, $$size$0 = 0;
 var $$width$0 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0.0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $alloc$0 = 0, $alloc$1 = 0, $alloc$2 = 0, $ap2$i = 0, $arglist_current = 0, $arglist_current2 = 0, $arglist_next = 0, $arglist_next3 = 0, $base$0 = 0, $c$027 = 0, $dest$0 = 0, $factor = 0;
 var $factor14 = 0, $i$0$i = 0, $i$0$ph = 0, $i$0$ph46 = 0, $i$0$ph46$lcssa = 0, $i$0$ph46$us53 = 0, $i$1$lcssa = 0, $i$2 = 0, $i$2$ph = 0, $i$2$ph$phi = 0, $i$3 = 0, $i$4 = 0, $invert$0 = 0, $isdigit = 0, $isdigit3 = 0, $isdigit320 = 0, $isdigittmp = 0, $isdigittmp2 = 0, $isdigittmp219 = 0, $k$0$ph = 0;
 var $k$1$ph = 0, $matches$0$ = 0, $matches$0116 = 0, $matches$1 = 0, $matches$2 = 0, $matches$3 = 0, $not$ = 0, $or$cond$us = 0, $or$cond5 = 0, $or$cond6 = 0, $p$0122 = 0, $p$1 = 0, $p$10 = 0, $p$11 = 0, $p$2 = 0, $p$3$lcssa = 0, $p$321 = 0, $p$4 = 0, $p$5 = 0, $p$6 = 0;
 var $p$7 = 0, $p$7$ph = 0, $p$8 = 0, $p$9 = 0, $pos$0121 = 0, $pos$1 = 0, $pos$2 = 0, $s$0114 = 0, $s$1$ph = 0, $s$2 = 0, $s$3 = 0, $s$4 = 0, $s$5 = 0, $s$6 = 0, $scanset = 0, $size$0 = 0, $st = 0, $vacopy_currentptr = 0, $wc = 0, $wcs$0115 = 0;
 var $wcs$1 = 0, $wcs$2$ph = 0, $wcs$2$ph146 = 0, $wcs$3 = 0, $wcs$4 = 0, $wcs$5 = 0, $wcs$6 = 0, $wcs$7 = 0, $width$0$lcssa = 0, $width$022 = 0, $width$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $ap2$i = sp + 16|0;
 $st = sp + 8|0;
 $scanset = sp + 33|0;
 $wc = sp;
 $0 = sp + 32|0;
 $1 = HEAP8[$fmt>>0]|0;
 $2 = ($1<<24>>24)==(0);
 if ($2) {
  $matches$3 = 0;
  STACKTOP = sp;return ($matches$3|0);
 }
 $3 = (($f) + 4|0);
 $4 = (($f) + 100|0);
 $5 = (($f) + 108|0);
 $6 = (($f) + 8|0);
 $7 = (($scanset) + 10|0);
 $8 = (($scanset) + 33|0);
 $9 = (($st) + 4|0);
 $10 = (($scanset) + 46|0);
 $11 = (($scanset) + 94|0);
 $13 = $1;$matches$0116 = 0;$p$0122 = $fmt;$pos$0121 = 0;$s$0114 = 0;$wcs$0115 = 0;
 L4: while(1) {
  $12 = $13&255;
  $14 = (_isspace($12)|0);
  $15 = ($14|0)==(0);
  L6: do {
   if ($15) {
    $42 = HEAP8[$p$0122>>0]|0;
    $43 = ($42<<24>>24)==(37);
    L21: do {
     if ($43) {
      $44 = (($p$0122) + 1|0);
      $45 = HEAP8[$44>>0]|0;
      do {
       if ((($45<<24>>24) == 42)) {
        $65 = (($p$0122) + 2|0);
        $dest$0 = 0;$p$2 = $65;
       } else if ((($45<<24>>24) == 37)) {
        break L21;
       } else {
        $66 = $45&255;
        $isdigittmp = (($66) + -48)|0;
        $isdigit = ($isdigittmp>>>0)<(10);
        if ($isdigit) {
         $67 = (($p$0122) + 2|0);
         $68 = HEAP8[$67>>0]|0;
         $69 = ($68<<24>>24)==(36);
         if ($69) {
          $vacopy_currentptr = HEAP32[$ap>>2]|0;
          HEAP32[$ap2$i>>2] = $vacopy_currentptr;
          $i$0$i = $isdigittmp;
          while(1) {
           $70 = ($i$0$i>>>0)>(1);
           $arglist_current = HEAP32[$ap2$i>>2]|0;
           $71 = HEAP32[$arglist_current>>2]|0;
           $arglist_next = (($arglist_current) + 4|0);
           HEAP32[$ap2$i>>2] = $arglist_next;
           $72 = (($i$0$i) + -1)|0;
           if ($70) {
            $i$0$i = $72;
           } else {
            break;
           }
          }
          $73 = (($p$0122) + 3|0);
          $dest$0 = $71;$p$2 = $73;
          break;
         }
        }
        $arglist_current2 = HEAP32[$ap>>2]|0;
        $74 = HEAP32[$arglist_current2>>2]|0;
        $arglist_next3 = (($arglist_current2) + 4|0);
        HEAP32[$ap>>2] = $arglist_next3;
        $dest$0 = $74;$p$2 = $44;
       }
      } while(0);
      $75 = HEAP8[$p$2>>0]|0;
      $76 = $75&255;
      $isdigittmp219 = (($76) + -48)|0;
      $isdigit320 = ($isdigittmp219>>>0)<(10);
      if ($isdigit320) {
       $80 = $76;$p$321 = $p$2;$width$022 = 0;
       while(1) {
        $77 = ($width$022*10)|0;
        $78 = (($77) + -48)|0;
        $79 = (($78) + ($80))|0;
        $81 = (($p$321) + 1|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $isdigittmp2 = (($83) + -48)|0;
        $isdigit3 = ($isdigittmp2>>>0)<(10);
        if ($isdigit3) {
         $80 = $83;$p$321 = $81;$width$022 = $79;
        } else {
         $$lcssa18 = $82;$p$3$lcssa = $81;$width$0$lcssa = $79;
         break;
        }
       }
      } else {
       $$lcssa18 = $75;$p$3$lcssa = $p$2;$width$0$lcssa = 0;
      }
      $84 = ($$lcssa18<<24>>24)==(109);
      if ($84) {
       $85 = ($dest$0|0)!=(0|0);
       $86 = $85&1;
       $87 = (($p$3$lcssa) + 1|0);
       $$pre = HEAP8[$87>>0]|0;
       $90 = $$pre;$alloc$0 = $86;$p$4 = $87;
      } else {
       $90 = $$lcssa18;$alloc$0 = 0;$p$4 = $p$3$lcssa;
      }
      $88 = (($p$4) + 1|0);
      $89 = $90&255;
      switch ($89|0) {
      case 104:  {
       $91 = HEAP8[$88>>0]|0;
       $92 = ($91<<24>>24)==(104);
       $93 = (($p$4) + 2|0);
       $$7 = $92 ? $93 : $88;
       $$8 = $92 ? -2 : -1;
       $p$5 = $$7;$size$0 = $$8;
       break;
      }
      case 116: case 122:  {
       $p$5 = $88;$size$0 = 1;
       break;
      }
      case 76:  {
       $p$5 = $88;$size$0 = 2;
       break;
      }
      case 106:  {
       $p$5 = $88;$size$0 = 3;
       break;
      }
      case 110: case 112: case 67: case 83: case 91: case 99: case 115: case 88: case 71: case 70: case 69: case 65: case 103: case 102: case 101: case 97: case 120: case 117: case 111: case 105: case 100:  {
       $p$5 = $p$4;$size$0 = 0;
       break;
      }
      case 108:  {
       $94 = HEAP8[$88>>0]|0;
       $95 = ($94<<24>>24)==(108);
       $96 = (($p$4) + 2|0);
       $$9 = $95 ? $96 : $88;
       $$10 = $95 ? 3 : 1;
       $p$5 = $$9;$size$0 = $$10;
       break;
      }
      default: {
       $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
       label = 162;
       break L4;
      }
      }
      $97 = HEAP8[$p$5>>0]|0;
      $98 = $97&255;
      $99 = $98 & 47;
      $100 = ($99|0)==(3);
      $101 = $98 | 32;
      $$ = $100 ? $101 : $98;
      $$size$0 = $100 ? 1 : $size$0;
      if ((($$|0) == 110)) {
       $103 = ($pos$0121|0)<(0);
       $104 = $103 << 31 >> 31;
       $105 = ($dest$0|0)==(0|0);
       if ($105) {
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
       }
       switch ($$size$0|0) {
       case -1:  {
        $107 = $pos$0121&65535;
        HEAP16[$dest$0>>1] = $107;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 0:  {
        HEAP32[$dest$0>>2] = $pos$0121;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 3:  {
        $108 = $dest$0;
        $109 = $108;
        HEAP32[$109>>2] = $pos$0121;
        $110 = (($108) + 4)|0;
        $111 = $110;
        HEAP32[$111>>2] = $104;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 1:  {
        HEAP32[$dest$0>>2] = $pos$0121;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case -2:  {
        $106 = $pos$0121&255;
        HEAP8[$dest$0>>0] = $106;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       default: {
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
       }
       }
      } else if ((($$|0) == 99)) {
       $102 = ($width$0$lcssa|0)<(1);
       $$width$0 = $102 ? 1 : $width$0$lcssa;
       $pos$1 = $pos$0121;$width$1 = $$width$0;
      } else if ((($$|0) == 91)) {
       $pos$1 = $pos$0121;$width$1 = $width$0$lcssa;
      } else {
       ___shlim($f,0);
       while(1) {
        $112 = HEAP32[$3>>2]|0;
        $113 = HEAP32[$4>>2]|0;
        $114 = ($112>>>0)<($113>>>0);
        if ($114) {
         $115 = (($112) + 1|0);
         HEAP32[$3>>2] = $115;
         $116 = HEAP8[$112>>0]|0;
         $117 = $116&255;
         $119 = $117;
        } else {
         $118 = (___shgetc($f)|0);
         $119 = $118;
        }
        $120 = (_isspace($119)|0);
        $121 = ($120|0)==(0);
        if ($121) {
         break;
        }
       }
       $122 = HEAP32[$4>>2]|0;
       $123 = ($122|0)==(0|0);
       $$pre206 = HEAP32[$3>>2]|0;
       if ($123) {
        $128 = $$pre206;
       } else {
        $124 = (($$pre206) + -1|0);
        HEAP32[$3>>2] = $124;
        $128 = $124;
       }
       $125 = HEAP32[$5>>2]|0;
       $126 = HEAP32[$6>>2]|0;
       $127 = $128;
       $129 = $126;
       $130 = (($125) + ($pos$0121))|0;
       $131 = (($130) + ($127))|0;
       $132 = (($131) - ($129))|0;
       $pos$1 = $132;$width$1 = $width$0$lcssa;
      }
      ___shlim($f,$width$1);
      $133 = HEAP32[$3>>2]|0;
      $134 = HEAP32[$4>>2]|0;
      $135 = ($133>>>0)<($134>>>0);
      if ($135) {
       $136 = (($133) + 1|0);
       HEAP32[$3>>2] = $136;
       $140 = $134;
      } else {
       $137 = (___shgetc($f)|0);
       $138 = ($137|0)<(0);
       if ($138) {
        $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
        label = 162;
        break L4;
       }
       $$pre207 = HEAP32[$4>>2]|0;
       $140 = $$pre207;
      }
      $139 = ($140|0)==(0|0);
      if (!($139)) {
       $141 = HEAP32[$3>>2]|0;
       $142 = (($141) + -1|0);
       HEAP32[$3>>2] = $142;
      }
      L78: do {
       switch ($$|0) {
       case 71: case 103: case 70: case 102: case 69: case 101: case 65: case 97:  {
        $322 = (+___floatscan($f,$$size$0,0));
        $323 = HEAP32[$5>>2]|0;
        $324 = HEAP32[$3>>2]|0;
        $325 = HEAP32[$6>>2]|0;
        $326 = $324;
        $327 = $325;
        $328 = (($327) - ($326))|0;
        $329 = ($323|0)==($328|0);
        if ($329) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$0114;$wcs$7 = $wcs$0115;
         break L4;
        }
        $330 = ($dest$0|0)==(0|0);
        if ($330) {
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
        } else {
         if ((($$size$0|0) == 0)) {
          $331 = $322;
          HEAPF32[$dest$0>>2] = $331;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else if ((($$size$0|0) == 1)) {
          HEAPF64[$dest$0>>3] = $322;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else if ((($$size$0|0) == 2)) {
          HEAPF64[$dest$0>>3] = $322;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else {
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         }
        }
        break;
       }
       case 111:  {
        $base$0 = 8;
        label = 144;
        break;
       }
       case 117: case 100:  {
        $base$0 = 10;
        label = 144;
        break;
       }
       case 105:  {
        $base$0 = 0;
        label = 144;
        break;
       }
       case 120: case 88: case 112:  {
        $base$0 = 16;
        label = 144;
        break;
       }
       case 91: case 99: case 115:  {
        $143 = ($$|0)==(99);
        $144 = $$ & 239;
        $145 = ($144|0)==(99);
        L90: do {
         if ($145) {
          $146 = ($$|0)==(115);
          _memset(($scanset|0),-1,257)|0;
          HEAP8[$scanset>>0] = 0;
          if ($146) {
           HEAP8[$8>>0] = 0;
           ;HEAP8[$7+0>>0]=0|0;HEAP8[$7+1>>0]=0|0;HEAP8[$7+2>>0]=0|0;HEAP8[$7+3>>0]=0|0;HEAP8[$7+4>>0]=0|0;
           $p$9 = $p$5;
          } else {
           $p$9 = $p$5;
          }
         } else {
          $147 = (($p$5) + 1|0);
          $148 = HEAP8[$147>>0]|0;
          $149 = ($148<<24>>24)==(94);
          $150 = (($p$5) + 2|0);
          $invert$0 = $149&1;
          $p$6 = $149 ? $150 : $147;
          $151 = $149&1;
          _memset(($scanset|0),($151|0),257)|0;
          HEAP8[$scanset>>0] = 0;
          $152 = HEAP8[$p$6>>0]|0;
          if ((($152<<24>>24) == 45)) {
           $153 = (($p$6) + 1|0);
           $154 = $invert$0 ^ 1;
           $155 = $154&255;
           HEAP8[$10>>0] = $155;
           $$pre$phi213Z2D = $155;$p$7$ph = $153;
          } else if ((($152<<24>>24) == 93)) {
           $156 = (($p$6) + 1|0);
           $157 = $invert$0 ^ 1;
           $158 = $157&255;
           HEAP8[$11>>0] = $158;
           $$pre$phi213Z2D = $158;$p$7$ph = $156;
          } else {
           $$pre211 = $invert$0 ^ 1;
           $$pre212 = $$pre211&255;
           $$pre$phi213Z2D = $$pre212;$p$7$ph = $p$6;
          }
          $p$7 = $p$7$ph;
          while(1) {
           $159 = HEAP8[$p$7>>0]|0;
           if ((($159<<24>>24) == 45)) {
            $160 = (($p$7) + 1|0);
            $161 = HEAP8[$160>>0]|0;
            if ((($161<<24>>24) == 93) | (($161<<24>>24) == 0)) {
             $172 = 45;$p$8 = $p$7;
            } else {
             $162 = (($p$7) + -1|0);
             $163 = HEAP8[$162>>0]|0;
             $164 = ($163&255)<($161&255);
             if ($164) {
              $165 = $163&255;
              $c$027 = $165;
              while(1) {
               $166 = (($c$027) + 1)|0;
               $167 = (($scanset) + ($166)|0);
               HEAP8[$167>>0] = $$pre$phi213Z2D;
               $168 = HEAP8[$160>>0]|0;
               $169 = $168&255;
               $170 = ($166|0)<($169|0);
               if ($170) {
                $c$027 = $166;
               } else {
                $172 = $168;$p$8 = $160;
                break;
               }
              }
             } else {
              $172 = $161;$p$8 = $160;
             }
            }
           } else if ((($159<<24>>24) == 0)) {
            $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
            label = 162;
            break L4;
           } else if ((($159<<24>>24) == 93)) {
            $p$9 = $p$7;
            break L90;
           } else {
            $172 = $159;$p$8 = $p$7;
           }
           $171 = $172&255;
           $173 = (($171) + 1)|0;
           $174 = (($scanset) + ($173)|0);
           HEAP8[$174>>0] = $$pre$phi213Z2D;
           $175 = (($p$8) + 1|0);
           $p$7 = $175;
          }
         }
        } while(0);
        $176 = (($width$1) + 1)|0;
        $$4 = $143 ? $176 : 31;
        $177 = ($$size$0|0)==(1);
        $178 = ($alloc$0|0)!=(0);
        L109: do {
         if ($177) {
          if ($178) {
           $179 = $$4 << 2;
           $180 = (_malloc($179)|0);
           $181 = ($180|0)==(0|0);
           if ($181) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $180;
            label = 162;
            break L4;
           } else {
            $wcs$1 = $180;
           }
          } else {
           $wcs$1 = $dest$0;
          }
          HEAP32[$st>>2] = 0;
          HEAP32[$9>>2] = 0;
          $i$0$ph = 0;$k$0$ph = $$4;$wcs$2$ph = $wcs$1;
          L144: while(1) {
           $182 = ($wcs$2$ph|0)==(0|0);
           if ($182) {
            while(1) {
             $185 = HEAP32[$3>>2]|0;
             $196 = HEAP32[$4>>2]|0;
             $197 = ($185>>>0)<($196>>>0);
             if ($197) {
              $184 = (($185) + 1|0);
              HEAP32[$3>>2] = $184;
              $186 = HEAP8[$185>>0]|0;
              $187 = $186&255;
              $189 = $187;
             } else {
              $183 = (___shgetc($f)|0);
              $189 = $183;
             }
             $188 = (($189) + 1)|0;
             $190 = (($scanset) + ($188)|0);
             $191 = HEAP8[$190>>0]|0;
             $192 = ($191<<24>>24)==(0);
             if ($192) {
              $i$0$ph46$lcssa = $i$0$ph;$wcs$2$ph146 = 0;
              break L144;
             }
             $193 = $189&255;
             HEAP8[$0>>0] = $193;
             $194 = (_mbrtowc($wc,$0,1,$st)|0);
             if ((($194|0) == -1)) {
              $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = 0;
              label = 162;
              break L4;
             } else if ((($194|0) == -2)) {
              continue;
             }
             $195 = ($i$0$ph|0)==($k$0$ph|0);
             $or$cond$us = $178 & $195;
             if ($or$cond$us) {
              $i$1$lcssa = $i$0$ph;
              break;
             }
            }
           } else {
            if ($178) {
             $i$0$ph46$us53 = $i$0$ph;
            } else {
             $i$0$ph46 = $i$0$ph;
             label = 98;
             break;
            }
            while(1) {
             while(1) {
              $200 = HEAP32[$3>>2]|0;
              $214 = HEAP32[$4>>2]|0;
              $215 = ($200>>>0)<($214>>>0);
              if ($215) {
               $199 = (($200) + 1|0);
               HEAP32[$3>>2] = $199;
               $201 = HEAP8[$200>>0]|0;
               $202 = $201&255;
               $204 = $202;
              } else {
               $198 = (___shgetc($f)|0);
               $204 = $198;
              }
              $203 = (($204) + 1)|0;
              $205 = (($scanset) + ($203)|0);
              $206 = HEAP8[$205>>0]|0;
              $207 = ($206<<24>>24)==(0);
              if ($207) {
               $i$0$ph46$lcssa = $i$0$ph46$us53;$wcs$2$ph146 = $wcs$2$ph;
               break L144;
              }
              $208 = $204&255;
              HEAP8[$0>>0] = $208;
              $209 = (_mbrtowc($wc,$0,1,$st)|0);
              if ((($209|0) == -1)) {
               $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
               label = 162;
               break L4;
              } else if (!((($209|0) == -2))) {
               break;
              }
             }
             $210 = HEAP32[$wc>>2]|0;
             $211 = (($i$0$ph46$us53) + 1)|0;
             $212 = (($wcs$2$ph) + ($i$0$ph46$us53<<2)|0);
             HEAP32[$212>>2] = $210;
             $213 = ($211|0)==($k$0$ph|0);
             if ($213) {
              $i$1$lcssa = $k$0$ph;
              break;
             } else {
              $i$0$ph46$us53 = $211;
             }
            }
           }
           $factor = $k$0$ph << 1;
           $233 = $factor | 1;
           $234 = $233 << 2;
           $235 = (_realloc($wcs$2$ph,$234)|0);
           $236 = ($235|0)==(0|0);
           if ($236) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
            label = 162;
            break L4;
           }
           $i$0$ph = $i$1$lcssa;$k$0$ph = $233;$wcs$2$ph = $235;
          }
          L168: do {
           if ((label|0) == 98) {
            while(1) {
             label = 0;
             while(1) {
              $216 = HEAP32[$3>>2]|0;
              $217 = HEAP32[$4>>2]|0;
              $218 = ($216>>>0)<($217>>>0);
              if ($218) {
               $219 = (($216) + 1|0);
               HEAP32[$3>>2] = $219;
               $220 = HEAP8[$216>>0]|0;
               $221 = $220&255;
               $224 = $221;
              } else {
               $222 = (___shgetc($f)|0);
               $224 = $222;
              }
              $223 = (($224) + 1)|0;
              $225 = (($scanset) + ($223)|0);
              $226 = HEAP8[$225>>0]|0;
              $227 = ($226<<24>>24)==(0);
              if ($227) {
               $i$0$ph46$lcssa = $i$0$ph46;$wcs$2$ph146 = $wcs$2$ph;
               break L168;
              }
              $228 = $224&255;
              HEAP8[$0>>0] = $228;
              $229 = (_mbrtowc($wc,$0,1,$st)|0);
              if ((($229|0) == -1)) {
               $alloc$1 = 0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
               label = 162;
               break L4;
              } else if (!((($229|0) == -2))) {
               break;
              }
             }
             $230 = HEAP32[$wc>>2]|0;
             $231 = (($i$0$ph46) + 1)|0;
             $232 = (($wcs$2$ph) + ($i$0$ph46<<2)|0);
             HEAP32[$232>>2] = $230;
             $i$0$ph46 = $231;
             label = 98;
            }
           }
          } while(0);
          $237 = (_mbsinit($st)|0);
          $238 = ($237|0)==(0);
          if ($238) {
           $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph146;
           label = 162;
           break L4;
          } else {
           $i$4 = $i$0$ph46$lcssa;$s$2 = 0;$wcs$3 = $wcs$2$ph146;
          }
         } else {
          if ($178) {
           $239 = (_malloc($$4)|0);
           $240 = ($239|0)==(0|0);
           if ($240) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = 0;
            label = 162;
            break L4;
           } else {
            $i$2$ph = 0;$k$1$ph = $$4;$s$1$ph = $239;
           }
           while(1) {
            $i$2 = $i$2$ph;
            while(1) {
             $241 = HEAP32[$3>>2]|0;
             $242 = HEAP32[$4>>2]|0;
             $243 = ($241>>>0)<($242>>>0);
             if ($243) {
              $244 = (($241) + 1|0);
              HEAP32[$3>>2] = $244;
              $245 = HEAP8[$241>>0]|0;
              $246 = $245&255;
              $249 = $246;
             } else {
              $247 = (___shgetc($f)|0);
              $249 = $247;
             }
             $248 = (($249) + 1)|0;
             $250 = (($scanset) + ($248)|0);
             $251 = HEAP8[$250>>0]|0;
             $252 = ($251<<24>>24)==(0);
             if ($252) {
              $i$4 = $i$2;$s$2 = $s$1$ph;$wcs$3 = 0;
              break L109;
             }
             $253 = $249&255;
             $254 = (($i$2) + 1)|0;
             $255 = (($s$1$ph) + ($i$2)|0);
             HEAP8[$255>>0] = $253;
             $256 = ($254|0)==($k$1$ph|0);
             if ($256) {
              break;
             } else {
              $i$2 = $254;
             }
            }
            $factor14 = $k$1$ph << 1;
            $257 = $factor14 | 1;
            $258 = (_realloc($s$1$ph,$257)|0);
            $259 = ($258|0)==(0|0);
            if ($259) {
             $alloc$1 = $alloc$0;$s$5 = $s$1$ph;$wcs$6 = 0;
             label = 162;
             break L4;
            } else {
             $i$2$ph$phi = $k$1$ph;$k$1$ph = $257;$s$1$ph = $258;$i$2$ph = $i$2$ph$phi;
            }
           }
          }
          $260 = ($dest$0|0)==(0|0);
          if ($260) {
           $278 = $140;
           while(1) {
            $276 = HEAP32[$3>>2]|0;
            $277 = ($276>>>0)<($278>>>0);
            if ($277) {
             $279 = (($276) + 1|0);
             HEAP32[$3>>2] = $279;
             $280 = HEAP8[$276>>0]|0;
             $281 = $280&255;
             $284 = $281;
            } else {
             $282 = (___shgetc($f)|0);
             $284 = $282;
            }
            $283 = (($284) + 1)|0;
            $285 = (($scanset) + ($283)|0);
            $286 = HEAP8[$285>>0]|0;
            $287 = ($286<<24>>24)==(0);
            if ($287) {
             $i$4 = 0;$s$2 = 0;$wcs$3 = 0;
             break L109;
            }
            $$pre209 = HEAP32[$4>>2]|0;
            $278 = $$pre209;
           }
          } else {
           $263 = $140;$i$3 = 0;
           while(1) {
            $261 = HEAP32[$3>>2]|0;
            $262 = ($261>>>0)<($263>>>0);
            if ($262) {
             $264 = (($261) + 1|0);
             HEAP32[$3>>2] = $264;
             $265 = HEAP8[$261>>0]|0;
             $266 = $265&255;
             $269 = $266;
            } else {
             $267 = (___shgetc($f)|0);
             $269 = $267;
            }
            $268 = (($269) + 1)|0;
            $270 = (($scanset) + ($268)|0);
            $271 = HEAP8[$270>>0]|0;
            $272 = ($271<<24>>24)==(0);
            if ($272) {
             $i$4 = $i$3;$s$2 = $dest$0;$wcs$3 = 0;
             break L109;
            }
            $273 = $269&255;
            $274 = (($i$3) + 1)|0;
            $275 = (($dest$0) + ($i$3)|0);
            HEAP8[$275>>0] = $273;
            $$pre193 = HEAP32[$4>>2]|0;
            $263 = $$pre193;$i$3 = $274;
           }
          }
         }
        } while(0);
        $288 = HEAP32[$4>>2]|0;
        $289 = ($288|0)==(0|0);
        $$pre208 = HEAP32[$3>>2]|0;
        if ($289) {
         $294 = $$pre208;
        } else {
         $290 = (($$pre208) + -1|0);
         HEAP32[$3>>2] = $290;
         $294 = $290;
        }
        $291 = HEAP32[$5>>2]|0;
        $292 = HEAP32[$6>>2]|0;
        $293 = $294;
        $295 = $292;
        $296 = (($293) - ($295))|0;
        $297 = (($296) + ($291))|0;
        $298 = ($297|0)==(0);
        if ($298) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$2;$wcs$7 = $wcs$3;
         break L4;
        }
        $$not = $143 ^ 1;
        $299 = ($297|0)==($width$1|0);
        $or$cond5 = $299 | $$not;
        if (!($or$cond5)) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$2;$wcs$7 = $wcs$3;
         break L4;
        }
        do {
         if ($178) {
          if ($177) {
           HEAP32[$dest$0>>2] = $wcs$3;
           break;
          } else {
           HEAP32[$dest$0>>2] = $s$2;
           break;
          }
         }
        } while(0);
        if ($143) {
         $p$10 = $p$9;$s$3 = $s$2;$wcs$4 = $wcs$3;
        } else {
         $300 = ($wcs$3|0)==(0|0);
         if (!($300)) {
          $301 = (($wcs$3) + ($i$4<<2)|0);
          HEAP32[$301>>2] = 0;
         }
         $302 = ($s$2|0)==(0|0);
         if ($302) {
          $p$10 = $p$9;$s$3 = 0;$wcs$4 = $wcs$3;
         } else {
          $303 = (($s$2) + ($i$4)|0);
          HEAP8[$303>>0] = 0;
          $p$10 = $p$9;$s$3 = $s$2;$wcs$4 = $wcs$3;
         }
        }
        break;
       }
       default: {
        $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
       }
       }
      } while(0);
      L197: do {
       if ((label|0) == 144) {
        label = 0;
        $304 = (___intscan($f,$base$0,0,-1,-1)|0);
        $305 = tempRet0;
        $306 = HEAP32[$5>>2]|0;
        $307 = HEAP32[$3>>2]|0;
        $308 = HEAP32[$6>>2]|0;
        $309 = $307;
        $310 = $308;
        $311 = (($310) - ($309))|0;
        $312 = ($306|0)==($311|0);
        if ($312) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$0114;$wcs$7 = $wcs$0115;
         break L4;
        }
        $313 = ($$|0)!=(112);
        $314 = ($dest$0|0)==(0|0);
        $or$cond6 = $313 | $314;
        if (!($or$cond6)) {
         $315 = $304;
         HEAP32[$dest$0>>2] = $315;
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
         break;
        }
        if ($314) {
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
        } else {
         switch ($$size$0|0) {
         case 3:  {
          $318 = $dest$0;
          $319 = $318;
          HEAP32[$319>>2] = $304;
          $320 = (($318) + 4)|0;
          $321 = $320;
          HEAP32[$321>>2] = $305;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case -2:  {
          $316 = $304&255;
          HEAP8[$dest$0>>0] = $316;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case -1:  {
          $317 = $304&65535;
          HEAP16[$dest$0>>1] = $317;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 0:  {
          HEAP32[$dest$0>>2] = $304;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 1:  {
          HEAP32[$dest$0>>2] = $304;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         default: {
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
         }
         }
        }
       }
      } while(0);
      $332 = HEAP32[$5>>2]|0;
      $333 = HEAP32[$3>>2]|0;
      $334 = HEAP32[$6>>2]|0;
      $335 = $333;
      $336 = $334;
      $337 = (($332) + ($pos$1))|0;
      $338 = (($337) + ($335))|0;
      $339 = (($338) - ($336))|0;
      $not$ = ($dest$0|0)!=(0|0);
      $340 = $not$&1;
      $matches$0$ = (($340) + ($matches$0116))|0;
      $matches$1 = $matches$0$;$p$11 = $p$10;$pos$2 = $339;$s$4 = $s$3;$wcs$5 = $wcs$4;
      break L6;
     }
    } while(0);
    $46 = $43&1;
    $47 = (($p$0122) + ($46)|0);
    ___shlim($f,0);
    $48 = HEAP32[$3>>2]|0;
    $49 = HEAP32[$4>>2]|0;
    $50 = ($48>>>0)<($49>>>0);
    if ($50) {
     $51 = (($48) + 1|0);
     HEAP32[$3>>2] = $51;
     $52 = HEAP8[$48>>0]|0;
     $53 = $52&255;
     $58 = $53;
    } else {
     $54 = (___shgetc($f)|0);
     $58 = $54;
    }
    $55 = HEAP8[$47>>0]|0;
    $56 = $55&255;
    $57 = ($58|0)==($56|0);
    if (!($57)) {
     label = 19;
     break L4;
    }
    $64 = (($pos$0121) + 1)|0;
    $matches$1 = $matches$0116;$p$11 = $47;$pos$2 = $64;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
   } else {
    $p$1 = $p$0122;
    while(1) {
     $16 = (($p$1) + 1|0);
     $17 = HEAP8[$16>>0]|0;
     $18 = $17&255;
     $19 = (_isspace($18)|0);
     $20 = ($19|0)==(0);
     if ($20) {
      break;
     } else {
      $p$1 = $16;
     }
    }
    ___shlim($f,0);
    while(1) {
     $21 = HEAP32[$3>>2]|0;
     $22 = HEAP32[$4>>2]|0;
     $23 = ($21>>>0)<($22>>>0);
     if ($23) {
      $24 = (($21) + 1|0);
      HEAP32[$3>>2] = $24;
      $25 = HEAP8[$21>>0]|0;
      $26 = $25&255;
      $28 = $26;
     } else {
      $27 = (___shgetc($f)|0);
      $28 = $27;
     }
     $29 = (_isspace($28)|0);
     $30 = ($29|0)==(0);
     if ($30) {
      break;
     }
    }
    $31 = HEAP32[$4>>2]|0;
    $32 = ($31|0)==(0|0);
    $$pre210 = HEAP32[$3>>2]|0;
    if ($32) {
     $37 = $$pre210;
    } else {
     $33 = (($$pre210) + -1|0);
     HEAP32[$3>>2] = $33;
     $37 = $33;
    }
    $34 = HEAP32[$5>>2]|0;
    $35 = HEAP32[$6>>2]|0;
    $36 = $37;
    $38 = $35;
    $39 = (($34) + ($pos$0121))|0;
    $40 = (($39) + ($36))|0;
    $41 = (($40) - ($38))|0;
    $matches$1 = $matches$0116;$p$11 = $p$1;$pos$2 = $41;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
   }
  } while(0);
  $341 = (($p$11) + 1|0);
  $342 = HEAP8[$341>>0]|0;
  $343 = ($342<<24>>24)==(0);
  if ($343) {
   $matches$3 = $matches$1;
   label = 165;
   break;
  } else {
   $13 = $342;$matches$0116 = $matches$1;$p$0122 = $341;$pos$0121 = $pos$2;$s$0114 = $s$4;$wcs$0115 = $wcs$5;
  }
 }
 if ((label|0) == 165) {
  STACKTOP = sp;return ($matches$3|0);
 }
 if ((label|0) == 19) {
  $59 = HEAP32[$4>>2]|0;
  $60 = ($59|0)==(0|0);
  if (!($60)) {
   $61 = HEAP32[$3>>2]|0;
   $62 = (($61) + -1|0);
   HEAP32[$3>>2] = $62;
  }
  $63 = ($58|0)<(0);
  if ($63) {
   $alloc$1 = 0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
   label = 162;
  } else {
   $matches$3 = $matches$0116;
   STACKTOP = sp;return ($matches$3|0);
  }
 }
 if ((label|0) == 162) {
  $344 = ($matches$0116|0)==(0);
  $345 = $344 << 31 >> 31;
  $$matches$0 = (($345) + ($matches$0116))|0;
  $alloc$2 = $alloc$1;$matches$2 = $$matches$0;$s$6 = $s$5;$wcs$7 = $wcs$6;
 }
 $346 = ($alloc$2|0)==(0);
 if ($346) {
  $matches$3 = $matches$2;
  STACKTOP = sp;return ($matches$3|0);
 }
 _free($s$6);
 _free($wcs$7);
 $matches$3 = $matches$2;
 STACKTOP = sp;return ($matches$3|0);
}
function _vsscanf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $f = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $f = sp;
 dest=$f+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $0 = (($f) + 32|0);
 HEAP32[$0>>2] = 148;
 $1 = (($f) + 44|0);
 HEAP32[$1>>2] = $s;
 $2 = (($f) + 76|0);
 HEAP32[$2>>2] = -1;
 $3 = (($f) + 84|0);
 HEAP32[$3>>2] = $s;
 $4 = (_MUSL_vfscanf($f,$fmt,$ap)|0);
 STACKTOP = sp;return ($4|0);
}
function _do_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___string_read($f,$buf,$len)|0);
 STACKTOP = sp;return ($0|0);
}
function ___memrchr($m,$c,$n) {
 $m = $m|0;
 $c = $c|0;
 $n = $n|0;
 var $$0 = 0, $$01 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c&255;
 $$01 = $n;
 while(1) {
  $1 = (($$01) + -1)|0;
  $2 = ($$01|0)==(0);
  if ($2) {
   $$0 = 0;
   label = 4;
   break;
  }
  $3 = (($m) + ($1)|0);
  $4 = HEAP8[$3>>0]|0;
  $5 = ($4<<24>>24)==($0<<24>>24);
  if ($5) {
   $$0 = $3;
   label = 4;
   break;
  } else {
   $$01 = $1;
  }
 }
 if ((label|0) == 4) {
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _strchr($s,$c) {
 $s = $s|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___strchrnul($s,$c)|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $c&255;
 $3 = ($1<<24>>24)==($2<<24>>24);
 $4 = $3 ? $0 : 0;
 STACKTOP = sp;return ($4|0);
}
function ___strchrnul($s,$c) {
 $s = $s|0;
 $c = $c|0;
 var $$0 = 0, $$02$lcssa = 0, $$026 = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $w$0$lcssa = 0, $w$03 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = ($0|0)==(0);
 if ($1) {
  $6 = (_strlen(($s|0))|0);
  $7 = (($s) + ($6)|0);
  $$0 = $7;
  STACKTOP = sp;return ($$0|0);
 }
 $2 = $s;
 $3 = $2 & 3;
 $4 = ($3|0)==(0);
 L5: do {
  if ($4) {
   $$02$lcssa = $s;
  } else {
   $5 = $c&255;
   $$026 = $s;
   while(1) {
    $12 = HEAP8[$$026>>0]|0;
    $13 = ($12<<24>>24)==(0);
    if ($13) {
     $$0 = $$026;
     label = 13;
     break;
    }
    $14 = ($12<<24>>24)==($5<<24>>24);
    $9 = (($$026) + 1|0);
    if ($14) {
     $$0 = $$026;
     label = 13;
     break;
    }
    $8 = $9;
    $10 = $8 & 3;
    $11 = ($10|0)==(0);
    if ($11) {
     $$02$lcssa = $9;
     break L5;
    } else {
     $$026 = $9;
    }
   }
   if ((label|0) == 13) {
    STACKTOP = sp;return ($$0|0);
   }
  }
 } while(0);
 $15 = Math_imul($0, 16843009)|0;
 $16 = HEAP32[$$02$lcssa>>2]|0;
 $17 = (($16) + -16843009)|0;
 $18 = $16 & -2139062144;
 $19 = $18 ^ -2139062144;
 $20 = $19 & $17;
 $21 = ($20|0)==(0);
 L15: do {
  if ($21) {
   $30 = $16;$w$03 = $$02$lcssa;
   while(1) {
    $29 = $30 ^ $15;
    $31 = (($29) + -16843009)|0;
    $32 = $29 & -2139062144;
    $33 = $32 ^ -2139062144;
    $34 = $33 & $31;
    $35 = ($34|0)==(0);
    $23 = (($w$03) + 4|0);
    if (!($35)) {
     $w$0$lcssa = $w$03;
     break L15;
    }
    $22 = HEAP32[$23>>2]|0;
    $24 = (($22) + -16843009)|0;
    $25 = $22 & -2139062144;
    $26 = $25 ^ -2139062144;
    $27 = $26 & $24;
    $28 = ($27|0)==(0);
    if ($28) {
     $30 = $22;$w$03 = $23;
    } else {
     $w$0$lcssa = $23;
     break;
    }
   }
  } else {
   $w$0$lcssa = $$02$lcssa;
  }
 } while(0);
 $36 = $c&255;
 $$1 = $w$0$lcssa;
 while(1) {
  $37 = HEAP8[$$1>>0]|0;
  $38 = ($37<<24>>24)==(0);
  $39 = ($37<<24>>24)==($36<<24>>24);
  $or$cond = $38 | $39;
  $40 = (($$1) + 1|0);
  if ($or$cond) {
   $$0 = $$1;
   break;
  } else {
   $$1 = $40;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___strdup($s) {
 $s = $s|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strlen(($s|0))|0);
 $1 = (($0) + 1)|0;
 $2 = (_malloc($1)|0);
 $3 = ($2|0)==(0|0);
 if ($3) {
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 _memcpy(($2|0),($s|0),($1|0))|0;
 $$0 = $2;
 STACKTOP = sp;return ($$0|0);
}
function _strrchr($s,$c) {
 $s = $s|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strlen(($s|0))|0);
 $1 = (($0) + 1)|0;
 $2 = (___memrchr($s,$c,$1)|0);
 STACKTOP = sp;return ($2|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$$i = 0, $$3$i = 0, $$4$i = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i25 = 0, $$pre$i25$i = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i26$iZ2D = 0, $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre57$i$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0;
 var $$sum$i14$i = 0, $$sum$i15$i = 0, $$sum$i18$i = 0, $$sum$i21$i = 0, $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i16$i = 0, $$sum1$i22$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum10$pre$i$i = 0, $$sum107$i = 0, $$sum108$i = 0, $$sum109$i = 0;
 var $$sum11$i = 0, $$sum11$i$i = 0, $$sum11$i24$i = 0, $$sum110$i = 0, $$sum111$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0, $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0;
 var $$sum14$pre$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0, $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i17$i = 0, $$sum2$i19$i = 0, $$sum2$i23$i = 0, $$sum2$pre$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0;
 var $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum26$pre$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i$i = 0, $$sum3$i27 = 0, $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0;
 var $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0, $$sum8$pre = 0, $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0;
 var $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0;
 var $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0;
 var $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0;
 var $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0;
 var $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0;
 var $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0;
 var $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0;
 var $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0;
 var $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0;
 var $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0;
 var $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0;
 var $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0;
 var $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0;
 var $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0;
 var $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0;
 var $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0;
 var $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0;
 var $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0;
 var $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0;
 var $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0;
 var $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0;
 var $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0;
 var $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0;
 var $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0;
 var $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0;
 var $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0;
 var $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0;
 var $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0;
 var $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0;
 var $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0;
 var $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0;
 var $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0;
 var $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0;
 var $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0;
 var $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0;
 var $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0;
 var $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0;
 var $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0;
 var $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0;
 var $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0;
 var $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0;
 var $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0;
 var $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0;
 var $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0;
 var $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0;
 var $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0;
 var $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0;
 var $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0;
 var $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0;
 var $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0;
 var $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0;
 var $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0;
 var $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0;
 var $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0, $F5$0$i = 0;
 var $I1$0$c$i$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$025$i = 0, $K2$014$i$i = 0, $K8$052$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i18 = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i17 = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i28$i = 0, $T$013$i$i = 0;
 var $T$024$i = 0, $T$051$i$i = 0, $br$0$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0, $notlhs$i = 0, $notrhs$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i29 = 0, $or$cond1$i = 0, $or$cond10$i = 0, $or$cond19$i = 0, $or$cond2$i = 0;
 var $or$cond49$i = 0, $or$cond5$i = 0, $or$cond6$i = 0, $or$cond8$not$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i15 = 0, $rsize$1$i = 0, $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$329$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$075$i = 0, $sp$168$i = 0, $ssize$0$$i = 0;
 var $ssize$0$i = 0, $ssize$1$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0, $t$2$v$3$i = 0, $t$228$i = 0, $tbase$0$i = 0, $tbase$247$i = 0, $tsize$0$i = 0, $tsize$0323841$i = 0, $tsize$1$i = 0, $tsize$246$i = 0, $v$0$i = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0;
 var $v$330$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   if ($1) {
    $5 = 16;
   } else {
    $2 = (($bytes) + 11)|0;
    $3 = $2 & -8;
    $5 = $3;
   }
   $4 = $5 >>> 3;
   $6 = HEAP32[76440>>2]|0;
   $7 = $6 >>> $4;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($4))|0;
    $13 = $12 << 1;
    $14 = ((76440 + ($13<<2)|0) + 40|0);
    $$sum10 = (($13) + 2)|0;
    $15 = ((76440 + ($$sum10<<2)|0) + 40|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[76440>>2] = $22;
     } else {
      $23 = HEAP32[((76440 + 16|0))>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = (($18) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = (($16) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    STACKTOP = sp;return ($mem$0|0);
   }
   $34 = HEAP32[((76440 + 8|0))>>2]|0;
   $35 = ($5>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $4;
     $38 = 2 << $4;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = ((76440 + ($65<<2)|0) + 40|0);
     $$sum4 = (($65) + 2)|0;
     $67 = ((76440 + ($$sum4<<2)|0) + 40|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[76440>>2] = $74;
      } else {
       $75 = HEAP32[((76440 + 16|0))>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = (($70) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($5))|0;
     $82 = $5 | 3;
     $83 = (($68) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($5)|0);
     $85 = $81 | 1;
     $$sum56 = $5 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $88 = HEAP32[((76440 + 8|0))>>2]|0;
     $89 = ($88|0)==(0);
     if (!($89)) {
      $90 = HEAP32[((76440 + 20|0))>>2]|0;
      $91 = $88 >>> 3;
      $92 = $91 << 1;
      $93 = ((76440 + ($92<<2)|0) + 40|0);
      $94 = HEAP32[76440>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[76440>>2] = $98;
       $$sum8$pre = (($92) + 2)|0;
       $$pre = ((76440 + ($$sum8$pre<<2)|0) + 40|0);
       $$pre$phiZ2D = $$pre;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = ((76440 + ($$sum9<<2)|0) + 40|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[((76440 + 16|0))>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = (($F4$0) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = (($90) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = (($90) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[((76440 + 8|0))>>2] = $81;
     HEAP32[((76440 + 20|0))>>2] = $84;
     $mem$0 = $69;
     STACKTOP = sp;return ($mem$0|0);
    }
    $106 = HEAP32[((76440 + 4|0))>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $5;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = ((76440 + ($130<<2)|0) + 304|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (($132) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($5))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = (($t$0$i) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = (($t$0$i) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = (($144) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($5))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[((76440 + 16|0))>>2]|0;
     $150 = ($v$0$i>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i) + ($5)|0);
     $152 = ($v$0$i>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = (($v$0$i) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = (($v$0$i) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i|0);
     do {
      if ($157) {
       $167 = (($v$0$i) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = (($v$0$i) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = (($R$0$i) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = (($R$0$i) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i>>2] = 0;
        $R$1$i = $R$0$i;
        break;
       }
      } else {
       $158 = (($v$0$i) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = (($159) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = (($156) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = (($v$0$i) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ((76440 + ($182<<2)|0) + 304|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[((76440 + 4|0))>>2]|0;
         $189 = $188 & $187;
         HEAP32[((76440 + 4|0))>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[((76440 + 16|0))>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = (($154) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = (($154) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[((76440 + 16|0))>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = (($R$1$i) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = (($v$0$i) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = HEAP32[((76440 + 16|0))>>2]|0;
         $204 = ($201>>>0)<($203>>>0);
         if ($204) {
          _abort();
          // unreachable;
         } else {
          $205 = (($R$1$i) + 16|0);
          HEAP32[$205>>2] = $201;
          $206 = (($201) + 24|0);
          HEAP32[$206>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $207 = (($v$0$i) + 20|0);
       $208 = HEAP32[$207>>2]|0;
       $209 = ($208|0)==(0|0);
       if (!($209)) {
        $210 = HEAP32[((76440 + 16|0))>>2]|0;
        $211 = ($208>>>0)<($210>>>0);
        if ($211) {
         _abort();
         // unreachable;
        } else {
         $212 = (($R$1$i) + 20|0);
         HEAP32[$212>>2] = $208;
         $213 = (($208) + 24|0);
         HEAP32[$213>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $214 = ($rsize$0$i>>>0)<(16);
     if ($214) {
      $215 = (($rsize$0$i) + ($5))|0;
      $216 = $215 | 3;
      $217 = (($v$0$i) + 4|0);
      HEAP32[$217>>2] = $216;
      $$sum4$i = (($215) + 4)|0;
      $218 = (($v$0$i) + ($$sum4$i)|0);
      $219 = HEAP32[$218>>2]|0;
      $220 = $219 | 1;
      HEAP32[$218>>2] = $220;
     } else {
      $221 = $5 | 3;
      $222 = (($v$0$i) + 4|0);
      HEAP32[$222>>2] = $221;
      $223 = $rsize$0$i | 1;
      $$sum$i35 = $5 | 4;
      $224 = (($v$0$i) + ($$sum$i35)|0);
      HEAP32[$224>>2] = $223;
      $$sum1$i = (($rsize$0$i) + ($5))|0;
      $225 = (($v$0$i) + ($$sum1$i)|0);
      HEAP32[$225>>2] = $rsize$0$i;
      $226 = HEAP32[((76440 + 8|0))>>2]|0;
      $227 = ($226|0)==(0);
      if (!($227)) {
       $228 = HEAP32[((76440 + 20|0))>>2]|0;
       $229 = $226 >>> 3;
       $230 = $229 << 1;
       $231 = ((76440 + ($230<<2)|0) + 40|0);
       $232 = HEAP32[76440>>2]|0;
       $233 = 1 << $229;
       $234 = $232 & $233;
       $235 = ($234|0)==(0);
       if ($235) {
        $236 = $232 | $233;
        HEAP32[76440>>2] = $236;
        $$sum2$pre$i = (($230) + 2)|0;
        $$pre$i = ((76440 + ($$sum2$pre$i<<2)|0) + 40|0);
        $$pre$phi$iZ2D = $$pre$i;$F1$0$i = $231;
       } else {
        $$sum3$i = (($230) + 2)|0;
        $237 = ((76440 + ($$sum3$i<<2)|0) + 40|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = HEAP32[((76440 + 16|0))>>2]|0;
        $240 = ($238>>>0)<($239>>>0);
        if ($240) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $237;$F1$0$i = $238;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $228;
       $241 = (($F1$0$i) + 12|0);
       HEAP32[$241>>2] = $228;
       $242 = (($228) + 8|0);
       HEAP32[$242>>2] = $F1$0$i;
       $243 = (($228) + 12|0);
       HEAP32[$243>>2] = $231;
      }
      HEAP32[((76440 + 8|0))>>2] = $rsize$0$i;
      HEAP32[((76440 + 20|0))>>2] = $151;
     }
     $244 = (($v$0$i) + 8|0);
     $mem$0 = $244;
     STACKTOP = sp;return ($mem$0|0);
    }
   } else {
    $nb$0 = $5;
   }
  } else {
   $245 = ($bytes>>>0)>(4294967231);
   if ($245) {
    $nb$0 = -1;
   } else {
    $246 = (($bytes) + 11)|0;
    $247 = $246 & -8;
    $248 = HEAP32[((76440 + 4|0))>>2]|0;
    $249 = ($248|0)==(0);
    if ($249) {
     $nb$0 = $247;
    } else {
     $250 = (0 - ($247))|0;
     $251 = $246 >>> 8;
     $252 = ($251|0)==(0);
     if ($252) {
      $idx$0$i = 0;
     } else {
      $253 = ($247>>>0)>(16777215);
      if ($253) {
       $idx$0$i = 31;
      } else {
       $254 = (($251) + 1048320)|0;
       $255 = $254 >>> 16;
       $256 = $255 & 8;
       $257 = $251 << $256;
       $258 = (($257) + 520192)|0;
       $259 = $258 >>> 16;
       $260 = $259 & 4;
       $261 = $260 | $256;
       $262 = $257 << $260;
       $263 = (($262) + 245760)|0;
       $264 = $263 >>> 16;
       $265 = $264 & 2;
       $266 = $261 | $265;
       $267 = (14 - ($266))|0;
       $268 = $262 << $265;
       $269 = $268 >>> 15;
       $270 = (($267) + ($269))|0;
       $271 = $270 << 1;
       $272 = (($270) + 7)|0;
       $273 = $247 >>> $272;
       $274 = $273 & 1;
       $275 = $274 | $271;
       $idx$0$i = $275;
      }
     }
     $276 = ((76440 + ($idx$0$i<<2)|0) + 304|0);
     $277 = HEAP32[$276>>2]|0;
     $278 = ($277|0)==(0|0);
     L126: do {
      if ($278) {
       $rsize$2$i = $250;$t$1$i = 0;$v$2$i = 0;
      } else {
       $279 = ($idx$0$i|0)==(31);
       if ($279) {
        $283 = 0;
       } else {
        $280 = $idx$0$i >>> 1;
        $281 = (25 - ($280))|0;
        $283 = $281;
       }
       $282 = $247 << $283;
       $rsize$0$i15 = $250;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $277;$v$0$i16 = 0;
       while(1) {
        $284 = (($t$0$i14) + 4|0);
        $285 = HEAP32[$284>>2]|0;
        $286 = $285 & -8;
        $287 = (($286) - ($247))|0;
        $288 = ($287>>>0)<($rsize$0$i15>>>0);
        if ($288) {
         $289 = ($286|0)==($247|0);
         if ($289) {
          $rsize$2$i = $287;$t$1$i = $t$0$i14;$v$2$i = $t$0$i14;
          break L126;
         } else {
          $rsize$1$i = $287;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $290 = (($t$0$i14) + 20|0);
        $291 = HEAP32[$290>>2]|0;
        $292 = $sizebits$0$i >>> 31;
        $293 = ((($t$0$i14) + ($292<<2)|0) + 16|0);
        $294 = HEAP32[$293>>2]|0;
        $295 = ($291|0)==(0|0);
        $296 = ($291|0)==($294|0);
        $or$cond$i = $295 | $296;
        $rst$1$i = $or$cond$i ? $rst$0$i : $291;
        $297 = ($294|0)==(0|0);
        $298 = $sizebits$0$i << 1;
        if ($297) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $298;$t$0$i14 = $294;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     $299 = ($t$1$i|0)==(0|0);
     $300 = ($v$2$i|0)==(0|0);
     $or$cond19$i = $299 & $300;
     if ($or$cond19$i) {
      $301 = 2 << $idx$0$i;
      $302 = (0 - ($301))|0;
      $303 = $301 | $302;
      $304 = $248 & $303;
      $305 = ($304|0)==(0);
      if ($305) {
       $nb$0 = $247;
       break;
      }
      $306 = (0 - ($304))|0;
      $307 = $304 & $306;
      $308 = (($307) + -1)|0;
      $309 = $308 >>> 12;
      $310 = $309 & 16;
      $311 = $308 >>> $310;
      $312 = $311 >>> 5;
      $313 = $312 & 8;
      $314 = $313 | $310;
      $315 = $311 >>> $313;
      $316 = $315 >>> 2;
      $317 = $316 & 4;
      $318 = $314 | $317;
      $319 = $315 >>> $317;
      $320 = $319 >>> 1;
      $321 = $320 & 2;
      $322 = $318 | $321;
      $323 = $319 >>> $321;
      $324 = $323 >>> 1;
      $325 = $324 & 1;
      $326 = $322 | $325;
      $327 = $323 >>> $325;
      $328 = (($326) + ($327))|0;
      $329 = ((76440 + ($328<<2)|0) + 304|0);
      $330 = HEAP32[$329>>2]|0;
      $t$2$ph$i = $330;
     } else {
      $t$2$ph$i = $t$1$i;
     }
     $331 = ($t$2$ph$i|0)==(0|0);
     if ($331) {
      $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$2$i;
     } else {
      $rsize$329$i = $rsize$2$i;$t$228$i = $t$2$ph$i;$v$330$i = $v$2$i;
      while(1) {
       $332 = (($t$228$i) + 4|0);
       $333 = HEAP32[$332>>2]|0;
       $334 = $333 & -8;
       $335 = (($334) - ($247))|0;
       $336 = ($335>>>0)<($rsize$329$i>>>0);
       $$rsize$3$i = $336 ? $335 : $rsize$329$i;
       $t$2$v$3$i = $336 ? $t$228$i : $v$330$i;
       $337 = (($t$228$i) + 16|0);
       $338 = HEAP32[$337>>2]|0;
       $339 = ($338|0)==(0|0);
       if (!($339)) {
        $rsize$329$i = $$rsize$3$i;$t$228$i = $338;$v$330$i = $t$2$v$3$i;
        continue;
       }
       $340 = (($t$228$i) + 20|0);
       $341 = HEAP32[$340>>2]|0;
       $342 = ($341|0)==(0|0);
       if ($342) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$329$i = $$rsize$3$i;$t$228$i = $341;$v$330$i = $t$2$v$3$i;
       }
      }
     }
     $343 = ($v$3$lcssa$i|0)==(0|0);
     if ($343) {
      $nb$0 = $247;
     } else {
      $344 = HEAP32[((76440 + 8|0))>>2]|0;
      $345 = (($344) - ($247))|0;
      $346 = ($rsize$3$lcssa$i>>>0)<($345>>>0);
      if ($346) {
       $347 = HEAP32[((76440 + 16|0))>>2]|0;
       $348 = ($v$3$lcssa$i>>>0)<($347>>>0);
       if ($348) {
        _abort();
        // unreachable;
       }
       $349 = (($v$3$lcssa$i) + ($247)|0);
       $350 = ($v$3$lcssa$i>>>0)<($349>>>0);
       if (!($350)) {
        _abort();
        // unreachable;
       }
       $351 = (($v$3$lcssa$i) + 24|0);
       $352 = HEAP32[$351>>2]|0;
       $353 = (($v$3$lcssa$i) + 12|0);
       $354 = HEAP32[$353>>2]|0;
       $355 = ($354|0)==($v$3$lcssa$i|0);
       do {
        if ($355) {
         $365 = (($v$3$lcssa$i) + 20|0);
         $366 = HEAP32[$365>>2]|0;
         $367 = ($366|0)==(0|0);
         if ($367) {
          $368 = (($v$3$lcssa$i) + 16|0);
          $369 = HEAP32[$368>>2]|0;
          $370 = ($369|0)==(0|0);
          if ($370) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $369;$RP$0$i17 = $368;
          }
         } else {
          $R$0$i18 = $366;$RP$0$i17 = $365;
         }
         while(1) {
          $371 = (($R$0$i18) + 20|0);
          $372 = HEAP32[$371>>2]|0;
          $373 = ($372|0)==(0|0);
          if (!($373)) {
           $R$0$i18 = $372;$RP$0$i17 = $371;
           continue;
          }
          $374 = (($R$0$i18) + 16|0);
          $375 = HEAP32[$374>>2]|0;
          $376 = ($375|0)==(0|0);
          if ($376) {
           break;
          } else {
           $R$0$i18 = $375;$RP$0$i17 = $374;
          }
         }
         $377 = ($RP$0$i17>>>0)<($347>>>0);
         if ($377) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17>>2] = 0;
          $R$1$i20 = $R$0$i18;
          break;
         }
        } else {
         $356 = (($v$3$lcssa$i) + 8|0);
         $357 = HEAP32[$356>>2]|0;
         $358 = ($357>>>0)<($347>>>0);
         if ($358) {
          _abort();
          // unreachable;
         }
         $359 = (($357) + 12|0);
         $360 = HEAP32[$359>>2]|0;
         $361 = ($360|0)==($v$3$lcssa$i|0);
         if (!($361)) {
          _abort();
          // unreachable;
         }
         $362 = (($354) + 8|0);
         $363 = HEAP32[$362>>2]|0;
         $364 = ($363|0)==($v$3$lcssa$i|0);
         if ($364) {
          HEAP32[$359>>2] = $354;
          HEAP32[$362>>2] = $357;
          $R$1$i20 = $354;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $378 = ($352|0)==(0|0);
       do {
        if (!($378)) {
         $379 = (($v$3$lcssa$i) + 28|0);
         $380 = HEAP32[$379>>2]|0;
         $381 = ((76440 + ($380<<2)|0) + 304|0);
         $382 = HEAP32[$381>>2]|0;
         $383 = ($v$3$lcssa$i|0)==($382|0);
         if ($383) {
          HEAP32[$381>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $384 = 1 << $380;
           $385 = $384 ^ -1;
           $386 = HEAP32[((76440 + 4|0))>>2]|0;
           $387 = $386 & $385;
           HEAP32[((76440 + 4|0))>>2] = $387;
           break;
          }
         } else {
          $388 = HEAP32[((76440 + 16|0))>>2]|0;
          $389 = ($352>>>0)<($388>>>0);
          if ($389) {
           _abort();
           // unreachable;
          }
          $390 = (($352) + 16|0);
          $391 = HEAP32[$390>>2]|0;
          $392 = ($391|0)==($v$3$lcssa$i|0);
          if ($392) {
           HEAP32[$390>>2] = $R$1$i20;
          } else {
           $393 = (($352) + 20|0);
           HEAP32[$393>>2] = $R$1$i20;
          }
          $394 = ($R$1$i20|0)==(0|0);
          if ($394) {
           break;
          }
         }
         $395 = HEAP32[((76440 + 16|0))>>2]|0;
         $396 = ($R$1$i20>>>0)<($395>>>0);
         if ($396) {
          _abort();
          // unreachable;
         }
         $397 = (($R$1$i20) + 24|0);
         HEAP32[$397>>2] = $352;
         $398 = (($v$3$lcssa$i) + 16|0);
         $399 = HEAP32[$398>>2]|0;
         $400 = ($399|0)==(0|0);
         do {
          if (!($400)) {
           $401 = HEAP32[((76440 + 16|0))>>2]|0;
           $402 = ($399>>>0)<($401>>>0);
           if ($402) {
            _abort();
            // unreachable;
           } else {
            $403 = (($R$1$i20) + 16|0);
            HEAP32[$403>>2] = $399;
            $404 = (($399) + 24|0);
            HEAP32[$404>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $405 = (($v$3$lcssa$i) + 20|0);
         $406 = HEAP32[$405>>2]|0;
         $407 = ($406|0)==(0|0);
         if (!($407)) {
          $408 = HEAP32[((76440 + 16|0))>>2]|0;
          $409 = ($406>>>0)<($408>>>0);
          if ($409) {
           _abort();
           // unreachable;
          } else {
           $410 = (($R$1$i20) + 20|0);
           HEAP32[$410>>2] = $406;
           $411 = (($406) + 24|0);
           HEAP32[$411>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $412 = ($rsize$3$lcssa$i>>>0)<(16);
       L204: do {
        if ($412) {
         $413 = (($rsize$3$lcssa$i) + ($247))|0;
         $414 = $413 | 3;
         $415 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$415>>2] = $414;
         $$sum18$i = (($413) + 4)|0;
         $416 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $417 = HEAP32[$416>>2]|0;
         $418 = $417 | 1;
         HEAP32[$416>>2] = $418;
        } else {
         $419 = $247 | 3;
         $420 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$420>>2] = $419;
         $421 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $247 | 4;
         $422 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$422>>2] = $421;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($247))|0;
         $423 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$423>>2] = $rsize$3$lcssa$i;
         $424 = $rsize$3$lcssa$i >>> 3;
         $425 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($425) {
          $426 = $424 << 1;
          $427 = ((76440 + ($426<<2)|0) + 40|0);
          $428 = HEAP32[76440>>2]|0;
          $429 = 1 << $424;
          $430 = $428 & $429;
          $431 = ($430|0)==(0);
          do {
           if ($431) {
            $432 = $428 | $429;
            HEAP32[76440>>2] = $432;
            $$sum14$pre$i = (($426) + 2)|0;
            $$pre$i25 = ((76440 + ($$sum14$pre$i<<2)|0) + 40|0);
            $$pre$phi$i26Z2D = $$pre$i25;$F5$0$i = $427;
           } else {
            $$sum17$i = (($426) + 2)|0;
            $433 = ((76440 + ($$sum17$i<<2)|0) + 40|0);
            $434 = HEAP32[$433>>2]|0;
            $435 = HEAP32[((76440 + 16|0))>>2]|0;
            $436 = ($434>>>0)<($435>>>0);
            if (!($436)) {
             $$pre$phi$i26Z2D = $433;$F5$0$i = $434;
             break;
            }
            _abort();
            // unreachable;
           }
          } while(0);
          HEAP32[$$pre$phi$i26Z2D>>2] = $349;
          $437 = (($F5$0$i) + 12|0);
          HEAP32[$437>>2] = $349;
          $$sum15$i = (($247) + 8)|0;
          $438 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$438>>2] = $F5$0$i;
          $$sum16$i = (($247) + 12)|0;
          $439 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$439>>2] = $427;
          break;
         }
         $440 = $rsize$3$lcssa$i >>> 8;
         $441 = ($440|0)==(0);
         if ($441) {
          $I7$0$i = 0;
         } else {
          $442 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($442) {
           $I7$0$i = 31;
          } else {
           $443 = (($440) + 1048320)|0;
           $444 = $443 >>> 16;
           $445 = $444 & 8;
           $446 = $440 << $445;
           $447 = (($446) + 520192)|0;
           $448 = $447 >>> 16;
           $449 = $448 & 4;
           $450 = $449 | $445;
           $451 = $446 << $449;
           $452 = (($451) + 245760)|0;
           $453 = $452 >>> 16;
           $454 = $453 & 2;
           $455 = $450 | $454;
           $456 = (14 - ($455))|0;
           $457 = $451 << $454;
           $458 = $457 >>> 15;
           $459 = (($456) + ($458))|0;
           $460 = $459 << 1;
           $461 = (($459) + 7)|0;
           $462 = $rsize$3$lcssa$i >>> $461;
           $463 = $462 & 1;
           $464 = $463 | $460;
           $I7$0$i = $464;
          }
         }
         $465 = ((76440 + ($I7$0$i<<2)|0) + 304|0);
         $$sum2$i = (($247) + 28)|0;
         $466 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$466>>2] = $I7$0$i;
         $$sum3$i27 = (($247) + 16)|0;
         $467 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($247) + 20)|0;
         $468 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$468>>2] = 0;
         HEAP32[$467>>2] = 0;
         $469 = HEAP32[((76440 + 4|0))>>2]|0;
         $470 = 1 << $I7$0$i;
         $471 = $469 & $470;
         $472 = ($471|0)==(0);
         if ($472) {
          $473 = $469 | $470;
          HEAP32[((76440 + 4|0))>>2] = $473;
          HEAP32[$465>>2] = $349;
          $$sum5$i = (($247) + 24)|0;
          $474 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$474>>2] = $465;
          $$sum6$i = (($247) + 12)|0;
          $475 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$475>>2] = $349;
          $$sum7$i = (($247) + 8)|0;
          $476 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$476>>2] = $349;
          break;
         }
         $477 = HEAP32[$465>>2]|0;
         $478 = ($I7$0$i|0)==(31);
         if ($478) {
          $486 = 0;
         } else {
          $479 = $I7$0$i >>> 1;
          $480 = (25 - ($479))|0;
          $486 = $480;
         }
         $481 = (($477) + 4|0);
         $482 = HEAP32[$481>>2]|0;
         $483 = $482 & -8;
         $484 = ($483|0)==($rsize$3$lcssa$i|0);
         L225: do {
          if ($484) {
           $T$0$lcssa$i = $477;
          } else {
           $485 = $rsize$3$lcssa$i << $486;
           $K12$025$i = $485;$T$024$i = $477;
           while(1) {
            $493 = $K12$025$i >>> 31;
            $494 = ((($T$024$i) + ($493<<2)|0) + 16|0);
            $489 = HEAP32[$494>>2]|0;
            $495 = ($489|0)==(0|0);
            if ($495) {
             break;
            }
            $487 = $K12$025$i << 1;
            $488 = (($489) + 4|0);
            $490 = HEAP32[$488>>2]|0;
            $491 = $490 & -8;
            $492 = ($491|0)==($rsize$3$lcssa$i|0);
            if ($492) {
             $T$0$lcssa$i = $489;
             break L225;
            } else {
             $K12$025$i = $487;$T$024$i = $489;
            }
           }
           $496 = HEAP32[((76440 + 16|0))>>2]|0;
           $497 = ($494>>>0)<($496>>>0);
           if ($497) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$494>>2] = $349;
            $$sum11$i = (($247) + 24)|0;
            $498 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$498>>2] = $T$024$i;
            $$sum12$i = (($247) + 12)|0;
            $499 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$499>>2] = $349;
            $$sum13$i = (($247) + 8)|0;
            $500 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$500>>2] = $349;
            break L204;
           }
          }
         } while(0);
         $501 = (($T$0$lcssa$i) + 8|0);
         $502 = HEAP32[$501>>2]|0;
         $503 = HEAP32[((76440 + 16|0))>>2]|0;
         $504 = ($T$0$lcssa$i>>>0)<($503>>>0);
         if ($504) {
          _abort();
          // unreachable;
         }
         $505 = ($502>>>0)<($503>>>0);
         if ($505) {
          _abort();
          // unreachable;
         } else {
          $506 = (($502) + 12|0);
          HEAP32[$506>>2] = $349;
          HEAP32[$501>>2] = $349;
          $$sum8$i = (($247) + 8)|0;
          $507 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$507>>2] = $502;
          $$sum9$i = (($247) + 12)|0;
          $508 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$508>>2] = $T$0$lcssa$i;
          $$sum10$i = (($247) + 24)|0;
          $509 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$509>>2] = 0;
          break;
         }
        }
       } while(0);
       $510 = (($v$3$lcssa$i) + 8|0);
       $mem$0 = $510;
       STACKTOP = sp;return ($mem$0|0);
      } else {
       $nb$0 = $247;
      }
     }
    }
   }
  }
 } while(0);
 $511 = HEAP32[((76440 + 8|0))>>2]|0;
 $512 = ($nb$0>>>0)>($511>>>0);
 if (!($512)) {
  $513 = (($511) - ($nb$0))|0;
  $514 = HEAP32[((76440 + 20|0))>>2]|0;
  $515 = ($513>>>0)>(15);
  if ($515) {
   $516 = (($514) + ($nb$0)|0);
   HEAP32[((76440 + 20|0))>>2] = $516;
   HEAP32[((76440 + 8|0))>>2] = $513;
   $517 = $513 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $518 = (($514) + ($$sum2)|0);
   HEAP32[$518>>2] = $517;
   $519 = (($514) + ($511)|0);
   HEAP32[$519>>2] = $513;
   $520 = $nb$0 | 3;
   $521 = (($514) + 4|0);
   HEAP32[$521>>2] = $520;
  } else {
   HEAP32[((76440 + 8|0))>>2] = 0;
   HEAP32[((76440 + 20|0))>>2] = 0;
   $522 = $511 | 3;
   $523 = (($514) + 4|0);
   HEAP32[$523>>2] = $522;
   $$sum1 = (($511) + 4)|0;
   $524 = (($514) + ($$sum1)|0);
   $525 = HEAP32[$524>>2]|0;
   $526 = $525 | 1;
   HEAP32[$524>>2] = $526;
  }
  $527 = (($514) + 8|0);
  $mem$0 = $527;
  STACKTOP = sp;return ($mem$0|0);
 }
 $528 = HEAP32[((76440 + 12|0))>>2]|0;
 $529 = ($nb$0>>>0)<($528>>>0);
 if ($529) {
  $530 = (($528) - ($nb$0))|0;
  HEAP32[((76440 + 12|0))>>2] = $530;
  $531 = HEAP32[((76440 + 24|0))>>2]|0;
  $532 = (($531) + ($nb$0)|0);
  HEAP32[((76440 + 24|0))>>2] = $532;
  $533 = $530 | 1;
  $$sum = (($nb$0) + 4)|0;
  $534 = (($531) + ($$sum)|0);
  HEAP32[$534>>2] = $533;
  $535 = $nb$0 | 3;
  $536 = (($531) + 4|0);
  HEAP32[$536>>2] = $535;
  $537 = (($531) + 8|0);
  $mem$0 = $537;
  STACKTOP = sp;return ($mem$0|0);
 }
 $538 = HEAP32[76912>>2]|0;
 $539 = ($538|0)==(0);
 do {
  if ($539) {
   $540 = (_sysconf(30)|0);
   $541 = (($540) + -1)|0;
   $542 = $541 & $540;
   $543 = ($542|0)==(0);
   if ($543) {
    HEAP32[((76912 + 8|0))>>2] = $540;
    HEAP32[((76912 + 4|0))>>2] = $540;
    HEAP32[((76912 + 12|0))>>2] = -1;
    HEAP32[((76912 + 16|0))>>2] = -1;
    HEAP32[((76912 + 20|0))>>2] = 0;
    HEAP32[((76440 + 444|0))>>2] = 0;
    $544 = (_time((0|0))|0);
    $545 = $544 & -16;
    $546 = $545 ^ 1431655768;
    HEAP32[76912>>2] = $546;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $547 = (($nb$0) + 48)|0;
 $548 = HEAP32[((76912 + 8|0))>>2]|0;
 $549 = (($nb$0) + 47)|0;
 $550 = (($548) + ($549))|0;
 $551 = (0 - ($548))|0;
 $552 = $550 & $551;
 $553 = ($552>>>0)>($nb$0>>>0);
 if (!($553)) {
  $mem$0 = 0;
  STACKTOP = sp;return ($mem$0|0);
 }
 $554 = HEAP32[((76440 + 440|0))>>2]|0;
 $555 = ($554|0)==(0);
 if (!($555)) {
  $556 = HEAP32[((76440 + 432|0))>>2]|0;
  $557 = (($556) + ($552))|0;
  $558 = ($557>>>0)<=($556>>>0);
  $559 = ($557>>>0)>($554>>>0);
  $or$cond1$i = $558 | $559;
  if ($or$cond1$i) {
   $mem$0 = 0;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $560 = HEAP32[((76440 + 444|0))>>2]|0;
 $561 = $560 & 4;
 $562 = ($561|0)==(0);
 L269: do {
  if ($562) {
   $563 = HEAP32[((76440 + 24|0))>>2]|0;
   $564 = ($563|0)==(0|0);
   L271: do {
    if ($564) {
     label = 182;
    } else {
     $sp$0$i$i = ((76440 + 448|0));
     while(1) {
      $565 = HEAP32[$sp$0$i$i>>2]|0;
      $566 = ($565>>>0)>($563>>>0);
      if (!($566)) {
       $567 = (($sp$0$i$i) + 4|0);
       $568 = HEAP32[$567>>2]|0;
       $569 = (($565) + ($568)|0);
       $570 = ($569>>>0)>($563>>>0);
       if ($570) {
        break;
       }
      }
      $571 = (($sp$0$i$i) + 8|0);
      $572 = HEAP32[$571>>2]|0;
      $573 = ($572|0)==(0|0);
      if ($573) {
       label = 182;
       break L271;
      } else {
       $sp$0$i$i = $572;
      }
     }
     $574 = ($sp$0$i$i|0)==(0|0);
     if ($574) {
      label = 182;
     } else {
      $597 = HEAP32[((76440 + 12|0))>>2]|0;
      $598 = (($550) - ($597))|0;
      $599 = $598 & $551;
      $600 = ($599>>>0)<(2147483647);
      if ($600) {
       $601 = (_sbrk(($599|0))|0);
       $602 = HEAP32[$sp$0$i$i>>2]|0;
       $603 = HEAP32[$567>>2]|0;
       $604 = (($602) + ($603)|0);
       $605 = ($601|0)==($604|0);
       $$3$i = $605 ? $599 : 0;
       $$4$i = $605 ? $601 : (-1);
       $br$0$i = $601;$ssize$1$i = $599;$tbase$0$i = $$4$i;$tsize$0$i = $$3$i;
       label = 191;
      } else {
       $tsize$0323841$i = 0;
      }
     }
    }
   } while(0);
   do {
    if ((label|0) == 182) {
     $575 = (_sbrk(0)|0);
     $576 = ($575|0)==((-1)|0);
     if ($576) {
      $tsize$0323841$i = 0;
     } else {
      $577 = $575;
      $578 = HEAP32[((76912 + 4|0))>>2]|0;
      $579 = (($578) + -1)|0;
      $580 = $579 & $577;
      $581 = ($580|0)==(0);
      if ($581) {
       $ssize$0$i = $552;
      } else {
       $582 = (($579) + ($577))|0;
       $583 = (0 - ($578))|0;
       $584 = $582 & $583;
       $585 = (($552) - ($577))|0;
       $586 = (($585) + ($584))|0;
       $ssize$0$i = $586;
      }
      $587 = HEAP32[((76440 + 432|0))>>2]|0;
      $588 = (($587) + ($ssize$0$i))|0;
      $589 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $590 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i29 = $589 & $590;
      if ($or$cond$i29) {
       $591 = HEAP32[((76440 + 440|0))>>2]|0;
       $592 = ($591|0)==(0);
       if (!($592)) {
        $593 = ($588>>>0)<=($587>>>0);
        $594 = ($588>>>0)>($591>>>0);
        $or$cond2$i = $593 | $594;
        if ($or$cond2$i) {
         $tsize$0323841$i = 0;
         break;
        }
       }
       $595 = (_sbrk(($ssize$0$i|0))|0);
       $596 = ($595|0)==($575|0);
       $ssize$0$$i = $596 ? $ssize$0$i : 0;
       $$$i = $596 ? $575 : (-1);
       $br$0$i = $595;$ssize$1$i = $ssize$0$i;$tbase$0$i = $$$i;$tsize$0$i = $ssize$0$$i;
       label = 191;
      } else {
       $tsize$0323841$i = 0;
      }
     }
    }
   } while(0);
   L291: do {
    if ((label|0) == 191) {
     $606 = (0 - ($ssize$1$i))|0;
     $607 = ($tbase$0$i|0)==((-1)|0);
     if (!($607)) {
      $tbase$247$i = $tbase$0$i;$tsize$246$i = $tsize$0$i;
      label = 202;
      break L269;
     }
     $608 = ($br$0$i|0)!=((-1)|0);
     $609 = ($ssize$1$i>>>0)<(2147483647);
     $or$cond5$i = $608 & $609;
     $610 = ($ssize$1$i>>>0)<($547>>>0);
     $or$cond6$i = $or$cond5$i & $610;
     do {
      if ($or$cond6$i) {
       $611 = HEAP32[((76912 + 8|0))>>2]|0;
       $612 = (($549) - ($ssize$1$i))|0;
       $613 = (($612) + ($611))|0;
       $614 = (0 - ($611))|0;
       $615 = $613 & $614;
       $616 = ($615>>>0)<(2147483647);
       if ($616) {
        $617 = (_sbrk(($615|0))|0);
        $618 = ($617|0)==((-1)|0);
        if ($618) {
         (_sbrk(($606|0))|0);
         $tsize$0323841$i = $tsize$0$i;
         break L291;
        } else {
         $619 = (($615) + ($ssize$1$i))|0;
         $ssize$2$i = $619;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$i;
       }
      } else {
       $ssize$2$i = $ssize$1$i;
      }
     } while(0);
     $620 = ($br$0$i|0)==((-1)|0);
     if ($620) {
      $tsize$0323841$i = $tsize$0$i;
     } else {
      $tbase$247$i = $br$0$i;$tsize$246$i = $ssize$2$i;
      label = 202;
      break L269;
     }
    }
   } while(0);
   $621 = HEAP32[((76440 + 444|0))>>2]|0;
   $622 = $621 | 4;
   HEAP32[((76440 + 444|0))>>2] = $622;
   $tsize$1$i = $tsize$0323841$i;
   label = 199;
  } else {
   $tsize$1$i = 0;
   label = 199;
  }
 } while(0);
 if ((label|0) == 199) {
  $623 = ($552>>>0)<(2147483647);
  if ($623) {
   $624 = (_sbrk(($552|0))|0);
   $625 = (_sbrk(0)|0);
   $notlhs$i = ($624|0)!=((-1)|0);
   $notrhs$i = ($625|0)!=((-1)|0);
   $or$cond8$not$i = $notrhs$i & $notlhs$i;
   $626 = ($624>>>0)<($625>>>0);
   $or$cond9$i = $or$cond8$not$i & $626;
   if ($or$cond9$i) {
    $627 = $625;
    $628 = $624;
    $629 = (($627) - ($628))|0;
    $630 = (($nb$0) + 40)|0;
    $631 = ($629>>>0)>($630>>>0);
    $$tsize$1$i = $631 ? $629 : $tsize$1$i;
    if ($631) {
     $tbase$247$i = $624;$tsize$246$i = $$tsize$1$i;
     label = 202;
    }
   }
  }
 }
 if ((label|0) == 202) {
  $632 = HEAP32[((76440 + 432|0))>>2]|0;
  $633 = (($632) + ($tsize$246$i))|0;
  HEAP32[((76440 + 432|0))>>2] = $633;
  $634 = HEAP32[((76440 + 436|0))>>2]|0;
  $635 = ($633>>>0)>($634>>>0);
  if ($635) {
   HEAP32[((76440 + 436|0))>>2] = $633;
  }
  $636 = HEAP32[((76440 + 24|0))>>2]|0;
  $637 = ($636|0)==(0|0);
  L311: do {
   if ($637) {
    $638 = HEAP32[((76440 + 16|0))>>2]|0;
    $639 = ($638|0)==(0|0);
    $640 = ($tbase$247$i>>>0)<($638>>>0);
    $or$cond10$i = $639 | $640;
    if ($or$cond10$i) {
     HEAP32[((76440 + 16|0))>>2] = $tbase$247$i;
    }
    HEAP32[((76440 + 448|0))>>2] = $tbase$247$i;
    HEAP32[((76440 + 452|0))>>2] = $tsize$246$i;
    HEAP32[((76440 + 460|0))>>2] = 0;
    $641 = HEAP32[76912>>2]|0;
    HEAP32[((76440 + 36|0))>>2] = $641;
    HEAP32[((76440 + 32|0))>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $642 = $i$02$i$i << 1;
     $643 = ((76440 + ($642<<2)|0) + 40|0);
     $$sum$i$i = (($642) + 3)|0;
     $644 = ((76440 + ($$sum$i$i<<2)|0) + 40|0);
     HEAP32[$644>>2] = $643;
     $$sum1$i$i = (($642) + 2)|0;
     $645 = ((76440 + ($$sum1$i$i<<2)|0) + 40|0);
     HEAP32[$645>>2] = $643;
     $646 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($646|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $646;
     }
    }
    $647 = (($tsize$246$i) + -40)|0;
    $648 = (($tbase$247$i) + 8|0);
    $649 = $648;
    $650 = $649 & 7;
    $651 = ($650|0)==(0);
    if ($651) {
     $655 = 0;
    } else {
     $652 = (0 - ($649))|0;
     $653 = $652 & 7;
     $655 = $653;
    }
    $654 = (($tbase$247$i) + ($655)|0);
    $656 = (($647) - ($655))|0;
    HEAP32[((76440 + 24|0))>>2] = $654;
    HEAP32[((76440 + 12|0))>>2] = $656;
    $657 = $656 | 1;
    $$sum$i14$i = (($655) + 4)|0;
    $658 = (($tbase$247$i) + ($$sum$i14$i)|0);
    HEAP32[$658>>2] = $657;
    $$sum2$i$i = (($tsize$246$i) + -36)|0;
    $659 = (($tbase$247$i) + ($$sum2$i$i)|0);
    HEAP32[$659>>2] = 40;
    $660 = HEAP32[((76912 + 16|0))>>2]|0;
    HEAP32[((76440 + 28|0))>>2] = $660;
   } else {
    $sp$075$i = ((76440 + 448|0));
    while(1) {
     $661 = HEAP32[$sp$075$i>>2]|0;
     $662 = (($sp$075$i) + 4|0);
     $663 = HEAP32[$662>>2]|0;
     $664 = (($661) + ($663)|0);
     $665 = ($tbase$247$i|0)==($664|0);
     if ($665) {
      label = 214;
      break;
     }
     $666 = (($sp$075$i) + 8|0);
     $667 = HEAP32[$666>>2]|0;
     $668 = ($667|0)==(0|0);
     if ($668) {
      break;
     } else {
      $sp$075$i = $667;
     }
    }
    if ((label|0) == 214) {
     $669 = (($sp$075$i) + 12|0);
     $670 = HEAP32[$669>>2]|0;
     $671 = $670 & 8;
     $672 = ($671|0)==(0);
     if ($672) {
      $673 = ($636>>>0)>=($661>>>0);
      $674 = ($636>>>0)<($tbase$247$i>>>0);
      $or$cond49$i = $673 & $674;
      if ($or$cond49$i) {
       $675 = (($663) + ($tsize$246$i))|0;
       HEAP32[$662>>2] = $675;
       $676 = HEAP32[((76440 + 12|0))>>2]|0;
       $677 = (($676) + ($tsize$246$i))|0;
       $678 = (($636) + 8|0);
       $679 = $678;
       $680 = $679 & 7;
       $681 = ($680|0)==(0);
       if ($681) {
        $685 = 0;
       } else {
        $682 = (0 - ($679))|0;
        $683 = $682 & 7;
        $685 = $683;
       }
       $684 = (($636) + ($685)|0);
       $686 = (($677) - ($685))|0;
       HEAP32[((76440 + 24|0))>>2] = $684;
       HEAP32[((76440 + 12|0))>>2] = $686;
       $687 = $686 | 1;
       $$sum$i18$i = (($685) + 4)|0;
       $688 = (($636) + ($$sum$i18$i)|0);
       HEAP32[$688>>2] = $687;
       $$sum2$i19$i = (($677) + 4)|0;
       $689 = (($636) + ($$sum2$i19$i)|0);
       HEAP32[$689>>2] = 40;
       $690 = HEAP32[((76912 + 16|0))>>2]|0;
       HEAP32[((76440 + 28|0))>>2] = $690;
       break;
      }
     }
    }
    $691 = HEAP32[((76440 + 16|0))>>2]|0;
    $692 = ($tbase$247$i>>>0)<($691>>>0);
    if ($692) {
     HEAP32[((76440 + 16|0))>>2] = $tbase$247$i;
    }
    $693 = (($tbase$247$i) + ($tsize$246$i)|0);
    $sp$168$i = ((76440 + 448|0));
    while(1) {
     $694 = HEAP32[$sp$168$i>>2]|0;
     $695 = ($694|0)==($693|0);
     if ($695) {
      label = 224;
      break;
     }
     $696 = (($sp$168$i) + 8|0);
     $697 = HEAP32[$696>>2]|0;
     $698 = ($697|0)==(0|0);
     if ($698) {
      break;
     } else {
      $sp$168$i = $697;
     }
    }
    if ((label|0) == 224) {
     $699 = (($sp$168$i) + 12|0);
     $700 = HEAP32[$699>>2]|0;
     $701 = $700 & 8;
     $702 = ($701|0)==(0);
     if ($702) {
      HEAP32[$sp$168$i>>2] = $tbase$247$i;
      $703 = (($sp$168$i) + 4|0);
      $704 = HEAP32[$703>>2]|0;
      $705 = (($704) + ($tsize$246$i))|0;
      HEAP32[$703>>2] = $705;
      $706 = (($tbase$247$i) + 8|0);
      $707 = $706;
      $708 = $707 & 7;
      $709 = ($708|0)==(0);
      if ($709) {
       $713 = 0;
      } else {
       $710 = (0 - ($707))|0;
       $711 = $710 & 7;
       $713 = $711;
      }
      $712 = (($tbase$247$i) + ($713)|0);
      $$sum107$i = (($tsize$246$i) + 8)|0;
      $714 = (($tbase$247$i) + ($$sum107$i)|0);
      $715 = $714;
      $716 = $715 & 7;
      $717 = ($716|0)==(0);
      if ($717) {
       $720 = 0;
      } else {
       $718 = (0 - ($715))|0;
       $719 = $718 & 7;
       $720 = $719;
      }
      $$sum108$i = (($720) + ($tsize$246$i))|0;
      $721 = (($tbase$247$i) + ($$sum108$i)|0);
      $722 = $721;
      $723 = $712;
      $724 = (($722) - ($723))|0;
      $$sum$i21$i = (($713) + ($nb$0))|0;
      $725 = (($tbase$247$i) + ($$sum$i21$i)|0);
      $726 = (($724) - ($nb$0))|0;
      $727 = $nb$0 | 3;
      $$sum1$i22$i = (($713) + 4)|0;
      $728 = (($tbase$247$i) + ($$sum1$i22$i)|0);
      HEAP32[$728>>2] = $727;
      $729 = HEAP32[((76440 + 24|0))>>2]|0;
      $730 = ($721|0)==($729|0);
      L348: do {
       if ($730) {
        $731 = HEAP32[((76440 + 12|0))>>2]|0;
        $732 = (($731) + ($726))|0;
        HEAP32[((76440 + 12|0))>>2] = $732;
        HEAP32[((76440 + 24|0))>>2] = $725;
        $733 = $732 | 1;
        $$sum42$i$i = (($$sum$i21$i) + 4)|0;
        $734 = (($tbase$247$i) + ($$sum42$i$i)|0);
        HEAP32[$734>>2] = $733;
       } else {
        $735 = HEAP32[((76440 + 20|0))>>2]|0;
        $736 = ($721|0)==($735|0);
        if ($736) {
         $737 = HEAP32[((76440 + 8|0))>>2]|0;
         $738 = (($737) + ($726))|0;
         HEAP32[((76440 + 8|0))>>2] = $738;
         HEAP32[((76440 + 20|0))>>2] = $725;
         $739 = $738 | 1;
         $$sum40$i$i = (($$sum$i21$i) + 4)|0;
         $740 = (($tbase$247$i) + ($$sum40$i$i)|0);
         HEAP32[$740>>2] = $739;
         $$sum41$i$i = (($738) + ($$sum$i21$i))|0;
         $741 = (($tbase$247$i) + ($$sum41$i$i)|0);
         HEAP32[$741>>2] = $738;
         break;
        }
        $$sum2$i23$i = (($tsize$246$i) + 4)|0;
        $$sum109$i = (($$sum2$i23$i) + ($720))|0;
        $742 = (($tbase$247$i) + ($$sum109$i)|0);
        $743 = HEAP32[$742>>2]|0;
        $744 = $743 & 3;
        $745 = ($744|0)==(1);
        if ($745) {
         $746 = $743 & -8;
         $747 = $743 >>> 3;
         $748 = ($743>>>0)<(256);
         L356: do {
          if ($748) {
           $$sum3738$i$i = $720 | 8;
           $$sum119$i = (($$sum3738$i$i) + ($tsize$246$i))|0;
           $749 = (($tbase$247$i) + ($$sum119$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $$sum39$i$i = (($tsize$246$i) + 12)|0;
           $$sum120$i = (($$sum39$i$i) + ($720))|0;
           $751 = (($tbase$247$i) + ($$sum120$i)|0);
           $752 = HEAP32[$751>>2]|0;
           $753 = $747 << 1;
           $754 = ((76440 + ($753<<2)|0) + 40|0);
           $755 = ($750|0)==($754|0);
           do {
            if (!($755)) {
             $756 = HEAP32[((76440 + 16|0))>>2]|0;
             $757 = ($750>>>0)<($756>>>0);
             if ($757) {
              _abort();
              // unreachable;
             }
             $758 = (($750) + 12|0);
             $759 = HEAP32[$758>>2]|0;
             $760 = ($759|0)==($721|0);
             if ($760) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $761 = ($752|0)==($750|0);
           if ($761) {
            $762 = 1 << $747;
            $763 = $762 ^ -1;
            $764 = HEAP32[76440>>2]|0;
            $765 = $764 & $763;
            HEAP32[76440>>2] = $765;
            break;
           }
           $766 = ($752|0)==($754|0);
           do {
            if ($766) {
             $$pre57$i$i = (($752) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $767 = HEAP32[((76440 + 16|0))>>2]|0;
             $768 = ($752>>>0)<($767>>>0);
             if ($768) {
              _abort();
              // unreachable;
             }
             $769 = (($752) + 8|0);
             $770 = HEAP32[$769>>2]|0;
             $771 = ($770|0)==($721|0);
             if ($771) {
              $$pre$phi58$i$iZ2D = $769;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $772 = (($750) + 12|0);
           HEAP32[$772>>2] = $752;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $750;
          } else {
           $$sum34$i$i = $720 | 24;
           $$sum110$i = (($$sum34$i$i) + ($tsize$246$i))|0;
           $773 = (($tbase$247$i) + ($$sum110$i)|0);
           $774 = HEAP32[$773>>2]|0;
           $$sum5$i$i = (($tsize$246$i) + 12)|0;
           $$sum111$i = (($$sum5$i$i) + ($720))|0;
           $775 = (($tbase$247$i) + ($$sum111$i)|0);
           $776 = HEAP32[$775>>2]|0;
           $777 = ($776|0)==($721|0);
           do {
            if ($777) {
             $$sum67$i$i = $720 | 16;
             $$sum117$i = (($$sum2$i23$i) + ($$sum67$i$i))|0;
             $788 = (($tbase$247$i) + ($$sum117$i)|0);
             $789 = HEAP32[$788>>2]|0;
             $790 = ($789|0)==(0|0);
             if ($790) {
              $$sum118$i = (($$sum67$i$i) + ($tsize$246$i))|0;
              $791 = (($tbase$247$i) + ($$sum118$i)|0);
              $792 = HEAP32[$791>>2]|0;
              $793 = ($792|0)==(0|0);
              if ($793) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $792;$RP$0$i$i = $791;
              }
             } else {
              $R$0$i$i = $789;$RP$0$i$i = $788;
             }
             while(1) {
              $794 = (($R$0$i$i) + 20|0);
              $795 = HEAP32[$794>>2]|0;
              $796 = ($795|0)==(0|0);
              if (!($796)) {
               $R$0$i$i = $795;$RP$0$i$i = $794;
               continue;
              }
              $797 = (($R$0$i$i) + 16|0);
              $798 = HEAP32[$797>>2]|0;
              $799 = ($798|0)==(0|0);
              if ($799) {
               break;
              } else {
               $R$0$i$i = $798;$RP$0$i$i = $797;
              }
             }
             $800 = HEAP32[((76440 + 16|0))>>2]|0;
             $801 = ($RP$0$i$i>>>0)<($800>>>0);
             if ($801) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i>>2] = 0;
              $R$1$i$i = $R$0$i$i;
              break;
             }
            } else {
             $$sum3536$i$i = $720 | 8;
             $$sum112$i = (($$sum3536$i$i) + ($tsize$246$i))|0;
             $778 = (($tbase$247$i) + ($$sum112$i)|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = HEAP32[((76440 + 16|0))>>2]|0;
             $781 = ($779>>>0)<($780>>>0);
             if ($781) {
              _abort();
              // unreachable;
             }
             $782 = (($779) + 12|0);
             $783 = HEAP32[$782>>2]|0;
             $784 = ($783|0)==($721|0);
             if (!($784)) {
              _abort();
              // unreachable;
             }
             $785 = (($776) + 8|0);
             $786 = HEAP32[$785>>2]|0;
             $787 = ($786|0)==($721|0);
             if ($787) {
              HEAP32[$782>>2] = $776;
              HEAP32[$785>>2] = $779;
              $R$1$i$i = $776;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $802 = ($774|0)==(0|0);
           if ($802) {
            break;
           }
           $$sum30$i$i = (($tsize$246$i) + 28)|0;
           $$sum113$i = (($$sum30$i$i) + ($720))|0;
           $803 = (($tbase$247$i) + ($$sum113$i)|0);
           $804 = HEAP32[$803>>2]|0;
           $805 = ((76440 + ($804<<2)|0) + 304|0);
           $806 = HEAP32[$805>>2]|0;
           $807 = ($721|0)==($806|0);
           do {
            if ($807) {
             HEAP32[$805>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $808 = 1 << $804;
             $809 = $808 ^ -1;
             $810 = HEAP32[((76440 + 4|0))>>2]|0;
             $811 = $810 & $809;
             HEAP32[((76440 + 4|0))>>2] = $811;
             break L356;
            } else {
             $812 = HEAP32[((76440 + 16|0))>>2]|0;
             $813 = ($774>>>0)<($812>>>0);
             if ($813) {
              _abort();
              // unreachable;
             }
             $814 = (($774) + 16|0);
             $815 = HEAP32[$814>>2]|0;
             $816 = ($815|0)==($721|0);
             if ($816) {
              HEAP32[$814>>2] = $R$1$i$i;
             } else {
              $817 = (($774) + 20|0);
              HEAP32[$817>>2] = $R$1$i$i;
             }
             $818 = ($R$1$i$i|0)==(0|0);
             if ($818) {
              break L356;
             }
            }
           } while(0);
           $819 = HEAP32[((76440 + 16|0))>>2]|0;
           $820 = ($R$1$i$i>>>0)<($819>>>0);
           if ($820) {
            _abort();
            // unreachable;
           }
           $821 = (($R$1$i$i) + 24|0);
           HEAP32[$821>>2] = $774;
           $$sum3132$i$i = $720 | 16;
           $$sum114$i = (($$sum3132$i$i) + ($tsize$246$i))|0;
           $822 = (($tbase$247$i) + ($$sum114$i)|0);
           $823 = HEAP32[$822>>2]|0;
           $824 = ($823|0)==(0|0);
           do {
            if (!($824)) {
             $825 = HEAP32[((76440 + 16|0))>>2]|0;
             $826 = ($823>>>0)<($825>>>0);
             if ($826) {
              _abort();
              // unreachable;
             } else {
              $827 = (($R$1$i$i) + 16|0);
              HEAP32[$827>>2] = $823;
              $828 = (($823) + 24|0);
              HEAP32[$828>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum115$i = (($$sum2$i23$i) + ($$sum3132$i$i))|0;
           $829 = (($tbase$247$i) + ($$sum115$i)|0);
           $830 = HEAP32[$829>>2]|0;
           $831 = ($830|0)==(0|0);
           if ($831) {
            break;
           }
           $832 = HEAP32[((76440 + 16|0))>>2]|0;
           $833 = ($830>>>0)<($832>>>0);
           if ($833) {
            _abort();
            // unreachable;
           } else {
            $834 = (($R$1$i$i) + 20|0);
            HEAP32[$834>>2] = $830;
            $835 = (($830) + 24|0);
            HEAP32[$835>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $746 | $720;
         $$sum116$i = (($$sum9$i$i) + ($tsize$246$i))|0;
         $836 = (($tbase$247$i) + ($$sum116$i)|0);
         $837 = (($746) + ($726))|0;
         $oldfirst$0$i$i = $836;$qsize$0$i$i = $837;
        } else {
         $oldfirst$0$i$i = $721;$qsize$0$i$i = $726;
        }
        $838 = (($oldfirst$0$i$i) + 4|0);
        $839 = HEAP32[$838>>2]|0;
        $840 = $839 & -2;
        HEAP32[$838>>2] = $840;
        $841 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i21$i) + 4)|0;
        $842 = (($tbase$247$i) + ($$sum10$i$i)|0);
        HEAP32[$842>>2] = $841;
        $$sum11$i24$i = (($qsize$0$i$i) + ($$sum$i21$i))|0;
        $843 = (($tbase$247$i) + ($$sum11$i24$i)|0);
        HEAP32[$843>>2] = $qsize$0$i$i;
        $844 = $qsize$0$i$i >>> 3;
        $845 = ($qsize$0$i$i>>>0)<(256);
        if ($845) {
         $846 = $844 << 1;
         $847 = ((76440 + ($846<<2)|0) + 40|0);
         $848 = HEAP32[76440>>2]|0;
         $849 = 1 << $844;
         $850 = $848 & $849;
         $851 = ($850|0)==(0);
         do {
          if ($851) {
           $852 = $848 | $849;
           HEAP32[76440>>2] = $852;
           $$sum26$pre$i$i = (($846) + 2)|0;
           $$pre$i25$i = ((76440 + ($$sum26$pre$i$i<<2)|0) + 40|0);
           $$pre$phi$i26$iZ2D = $$pre$i25$i;$F4$0$i$i = $847;
          } else {
           $$sum29$i$i = (($846) + 2)|0;
           $853 = ((76440 + ($$sum29$i$i<<2)|0) + 40|0);
           $854 = HEAP32[$853>>2]|0;
           $855 = HEAP32[((76440 + 16|0))>>2]|0;
           $856 = ($854>>>0)<($855>>>0);
           if (!($856)) {
            $$pre$phi$i26$iZ2D = $853;$F4$0$i$i = $854;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i26$iZ2D>>2] = $725;
         $857 = (($F4$0$i$i) + 12|0);
         HEAP32[$857>>2] = $725;
         $$sum27$i$i = (($$sum$i21$i) + 8)|0;
         $858 = (($tbase$247$i) + ($$sum27$i$i)|0);
         HEAP32[$858>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i21$i) + 12)|0;
         $859 = (($tbase$247$i) + ($$sum28$i$i)|0);
         HEAP32[$859>>2] = $847;
         break;
        }
        $860 = $qsize$0$i$i >>> 8;
        $861 = ($860|0)==(0);
        do {
         if ($861) {
          $I7$0$i$i = 0;
         } else {
          $862 = ($qsize$0$i$i>>>0)>(16777215);
          if ($862) {
           $I7$0$i$i = 31;
           break;
          }
          $863 = (($860) + 1048320)|0;
          $864 = $863 >>> 16;
          $865 = $864 & 8;
          $866 = $860 << $865;
          $867 = (($866) + 520192)|0;
          $868 = $867 >>> 16;
          $869 = $868 & 4;
          $870 = $869 | $865;
          $871 = $866 << $869;
          $872 = (($871) + 245760)|0;
          $873 = $872 >>> 16;
          $874 = $873 & 2;
          $875 = $870 | $874;
          $876 = (14 - ($875))|0;
          $877 = $871 << $874;
          $878 = $877 >>> 15;
          $879 = (($876) + ($878))|0;
          $880 = $879 << 1;
          $881 = (($879) + 7)|0;
          $882 = $qsize$0$i$i >>> $881;
          $883 = $882 & 1;
          $884 = $883 | $880;
          $I7$0$i$i = $884;
         }
        } while(0);
        $885 = ((76440 + ($I7$0$i$i<<2)|0) + 304|0);
        $$sum12$i$i = (($$sum$i21$i) + 28)|0;
        $886 = (($tbase$247$i) + ($$sum12$i$i)|0);
        HEAP32[$886>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i21$i) + 16)|0;
        $887 = (($tbase$247$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i21$i) + 20)|0;
        $888 = (($tbase$247$i) + ($$sum14$i$i)|0);
        HEAP32[$888>>2] = 0;
        HEAP32[$887>>2] = 0;
        $889 = HEAP32[((76440 + 4|0))>>2]|0;
        $890 = 1 << $I7$0$i$i;
        $891 = $889 & $890;
        $892 = ($891|0)==(0);
        if ($892) {
         $893 = $889 | $890;
         HEAP32[((76440 + 4|0))>>2] = $893;
         HEAP32[$885>>2] = $725;
         $$sum15$i$i = (($$sum$i21$i) + 24)|0;
         $894 = (($tbase$247$i) + ($$sum15$i$i)|0);
         HEAP32[$894>>2] = $885;
         $$sum16$i$i = (($$sum$i21$i) + 12)|0;
         $895 = (($tbase$247$i) + ($$sum16$i$i)|0);
         HEAP32[$895>>2] = $725;
         $$sum17$i$i = (($$sum$i21$i) + 8)|0;
         $896 = (($tbase$247$i) + ($$sum17$i$i)|0);
         HEAP32[$896>>2] = $725;
         break;
        }
        $897 = HEAP32[$885>>2]|0;
        $898 = ($I7$0$i$i|0)==(31);
        if ($898) {
         $906 = 0;
        } else {
         $899 = $I7$0$i$i >>> 1;
         $900 = (25 - ($899))|0;
         $906 = $900;
        }
        $901 = (($897) + 4|0);
        $902 = HEAP32[$901>>2]|0;
        $903 = $902 & -8;
        $904 = ($903|0)==($qsize$0$i$i|0);
        L445: do {
         if ($904) {
          $T$0$lcssa$i28$i = $897;
         } else {
          $905 = $qsize$0$i$i << $906;
          $K8$052$i$i = $905;$T$051$i$i = $897;
          while(1) {
           $913 = $K8$052$i$i >>> 31;
           $914 = ((($T$051$i$i) + ($913<<2)|0) + 16|0);
           $909 = HEAP32[$914>>2]|0;
           $915 = ($909|0)==(0|0);
           if ($915) {
            break;
           }
           $907 = $K8$052$i$i << 1;
           $908 = (($909) + 4|0);
           $910 = HEAP32[$908>>2]|0;
           $911 = $910 & -8;
           $912 = ($911|0)==($qsize$0$i$i|0);
           if ($912) {
            $T$0$lcssa$i28$i = $909;
            break L445;
           } else {
            $K8$052$i$i = $907;$T$051$i$i = $909;
           }
          }
          $916 = HEAP32[((76440 + 16|0))>>2]|0;
          $917 = ($914>>>0)<($916>>>0);
          if ($917) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$914>>2] = $725;
           $$sum23$i$i = (($$sum$i21$i) + 24)|0;
           $918 = (($tbase$247$i) + ($$sum23$i$i)|0);
           HEAP32[$918>>2] = $T$051$i$i;
           $$sum24$i$i = (($$sum$i21$i) + 12)|0;
           $919 = (($tbase$247$i) + ($$sum24$i$i)|0);
           HEAP32[$919>>2] = $725;
           $$sum25$i$i = (($$sum$i21$i) + 8)|0;
           $920 = (($tbase$247$i) + ($$sum25$i$i)|0);
           HEAP32[$920>>2] = $725;
           break L348;
          }
         }
        } while(0);
        $921 = (($T$0$lcssa$i28$i) + 8|0);
        $922 = HEAP32[$921>>2]|0;
        $923 = HEAP32[((76440 + 16|0))>>2]|0;
        $924 = ($T$0$lcssa$i28$i>>>0)<($923>>>0);
        if ($924) {
         _abort();
         // unreachable;
        }
        $925 = ($922>>>0)<($923>>>0);
        if ($925) {
         _abort();
         // unreachable;
        } else {
         $926 = (($922) + 12|0);
         HEAP32[$926>>2] = $725;
         HEAP32[$921>>2] = $725;
         $$sum20$i$i = (($$sum$i21$i) + 8)|0;
         $927 = (($tbase$247$i) + ($$sum20$i$i)|0);
         HEAP32[$927>>2] = $922;
         $$sum21$i$i = (($$sum$i21$i) + 12)|0;
         $928 = (($tbase$247$i) + ($$sum21$i$i)|0);
         HEAP32[$928>>2] = $T$0$lcssa$i28$i;
         $$sum22$i$i = (($$sum$i21$i) + 24)|0;
         $929 = (($tbase$247$i) + ($$sum22$i$i)|0);
         HEAP32[$929>>2] = 0;
         break;
        }
       }
      } while(0);
      $$sum1819$i$i = $713 | 8;
      $930 = (($tbase$247$i) + ($$sum1819$i$i)|0);
      $mem$0 = $930;
      STACKTOP = sp;return ($mem$0|0);
     }
    }
    $sp$0$i$i$i = ((76440 + 448|0));
    while(1) {
     $931 = HEAP32[$sp$0$i$i$i>>2]|0;
     $932 = ($931>>>0)>($636>>>0);
     if (!($932)) {
      $933 = (($sp$0$i$i$i) + 4|0);
      $934 = HEAP32[$933>>2]|0;
      $935 = (($931) + ($934)|0);
      $936 = ($935>>>0)>($636>>>0);
      if ($936) {
       break;
      }
     }
     $937 = (($sp$0$i$i$i) + 8|0);
     $938 = HEAP32[$937>>2]|0;
     $sp$0$i$i$i = $938;
    }
    $$sum$i15$i = (($934) + -47)|0;
    $$sum1$i16$i = (($934) + -39)|0;
    $939 = (($931) + ($$sum1$i16$i)|0);
    $940 = $939;
    $941 = $940 & 7;
    $942 = ($941|0)==(0);
    if ($942) {
     $945 = 0;
    } else {
     $943 = (0 - ($940))|0;
     $944 = $943 & 7;
     $945 = $944;
    }
    $$sum2$i17$i = (($$sum$i15$i) + ($945))|0;
    $946 = (($931) + ($$sum2$i17$i)|0);
    $947 = (($636) + 16|0);
    $948 = ($946>>>0)<($947>>>0);
    $949 = $948 ? $636 : $946;
    $950 = (($949) + 8|0);
    $951 = (($tsize$246$i) + -40)|0;
    $952 = (($tbase$247$i) + 8|0);
    $953 = $952;
    $954 = $953 & 7;
    $955 = ($954|0)==(0);
    if ($955) {
     $959 = 0;
    } else {
     $956 = (0 - ($953))|0;
     $957 = $956 & 7;
     $959 = $957;
    }
    $958 = (($tbase$247$i) + ($959)|0);
    $960 = (($951) - ($959))|0;
    HEAP32[((76440 + 24|0))>>2] = $958;
    HEAP32[((76440 + 12|0))>>2] = $960;
    $961 = $960 | 1;
    $$sum$i$i$i = (($959) + 4)|0;
    $962 = (($tbase$247$i) + ($$sum$i$i$i)|0);
    HEAP32[$962>>2] = $961;
    $$sum2$i$i$i = (($tsize$246$i) + -36)|0;
    $963 = (($tbase$247$i) + ($$sum2$i$i$i)|0);
    HEAP32[$963>>2] = 40;
    $964 = HEAP32[((76912 + 16|0))>>2]|0;
    HEAP32[((76440 + 28|0))>>2] = $964;
    $965 = (($949) + 4|0);
    HEAP32[$965>>2] = 27;
    ;HEAP32[$950+0>>2]=HEAP32[((76440 + 448|0))+0>>2]|0;HEAP32[$950+4>>2]=HEAP32[((76440 + 448|0))+4>>2]|0;HEAP32[$950+8>>2]=HEAP32[((76440 + 448|0))+8>>2]|0;HEAP32[$950+12>>2]=HEAP32[((76440 + 448|0))+12>>2]|0;
    HEAP32[((76440 + 448|0))>>2] = $tbase$247$i;
    HEAP32[((76440 + 452|0))>>2] = $tsize$246$i;
    HEAP32[((76440 + 460|0))>>2] = 0;
    HEAP32[((76440 + 456|0))>>2] = $950;
    $966 = (($949) + 28|0);
    HEAP32[$966>>2] = 7;
    $967 = (($949) + 32|0);
    $968 = ($967>>>0)<($935>>>0);
    if ($968) {
     $970 = $966;
     while(1) {
      $969 = (($970) + 4|0);
      HEAP32[$969>>2] = 7;
      $971 = (($970) + 8|0);
      $972 = ($971>>>0)<($935>>>0);
      if ($972) {
       $970 = $969;
      } else {
       break;
      }
     }
    }
    $973 = ($949|0)==($636|0);
    if (!($973)) {
     $974 = $949;
     $975 = $636;
     $976 = (($974) - ($975))|0;
     $977 = (($636) + ($976)|0);
     $$sum3$i$i = (($976) + 4)|0;
     $978 = (($636) + ($$sum3$i$i)|0);
     $979 = HEAP32[$978>>2]|0;
     $980 = $979 & -2;
     HEAP32[$978>>2] = $980;
     $981 = $976 | 1;
     $982 = (($636) + 4|0);
     HEAP32[$982>>2] = $981;
     HEAP32[$977>>2] = $976;
     $983 = $976 >>> 3;
     $984 = ($976>>>0)<(256);
     if ($984) {
      $985 = $983 << 1;
      $986 = ((76440 + ($985<<2)|0) + 40|0);
      $987 = HEAP32[76440>>2]|0;
      $988 = 1 << $983;
      $989 = $987 & $988;
      $990 = ($989|0)==(0);
      do {
       if ($990) {
        $991 = $987 | $988;
        HEAP32[76440>>2] = $991;
        $$sum10$pre$i$i = (($985) + 2)|0;
        $$pre$i$i = ((76440 + ($$sum10$pre$i$i<<2)|0) + 40|0);
        $$pre$phi$i$iZ2D = $$pre$i$i;$F$0$i$i = $986;
       } else {
        $$sum11$i$i = (($985) + 2)|0;
        $992 = ((76440 + ($$sum11$i$i<<2)|0) + 40|0);
        $993 = HEAP32[$992>>2]|0;
        $994 = HEAP32[((76440 + 16|0))>>2]|0;
        $995 = ($993>>>0)<($994>>>0);
        if (!($995)) {
         $$pre$phi$i$iZ2D = $992;$F$0$i$i = $993;
         break;
        }
        _abort();
        // unreachable;
       }
      } while(0);
      HEAP32[$$pre$phi$i$iZ2D>>2] = $636;
      $996 = (($F$0$i$i) + 12|0);
      HEAP32[$996>>2] = $636;
      $997 = (($636) + 8|0);
      HEAP32[$997>>2] = $F$0$i$i;
      $998 = (($636) + 12|0);
      HEAP32[$998>>2] = $986;
      break;
     }
     $999 = $976 >>> 8;
     $1000 = ($999|0)==(0);
     if ($1000) {
      $I1$0$i$i = 0;
     } else {
      $1001 = ($976>>>0)>(16777215);
      if ($1001) {
       $I1$0$i$i = 31;
      } else {
       $1002 = (($999) + 1048320)|0;
       $1003 = $1002 >>> 16;
       $1004 = $1003 & 8;
       $1005 = $999 << $1004;
       $1006 = (($1005) + 520192)|0;
       $1007 = $1006 >>> 16;
       $1008 = $1007 & 4;
       $1009 = $1008 | $1004;
       $1010 = $1005 << $1008;
       $1011 = (($1010) + 245760)|0;
       $1012 = $1011 >>> 16;
       $1013 = $1012 & 2;
       $1014 = $1009 | $1013;
       $1015 = (14 - ($1014))|0;
       $1016 = $1010 << $1013;
       $1017 = $1016 >>> 15;
       $1018 = (($1015) + ($1017))|0;
       $1019 = $1018 << 1;
       $1020 = (($1018) + 7)|0;
       $1021 = $976 >>> $1020;
       $1022 = $1021 & 1;
       $1023 = $1022 | $1019;
       $I1$0$i$i = $1023;
      }
     }
     $1024 = ((76440 + ($I1$0$i$i<<2)|0) + 304|0);
     $1025 = (($636) + 28|0);
     $I1$0$c$i$i = $I1$0$i$i;
     HEAP32[$1025>>2] = $I1$0$c$i$i;
     $1026 = (($636) + 20|0);
     HEAP32[$1026>>2] = 0;
     $1027 = (($636) + 16|0);
     HEAP32[$1027>>2] = 0;
     $1028 = HEAP32[((76440 + 4|0))>>2]|0;
     $1029 = 1 << $I1$0$i$i;
     $1030 = $1028 & $1029;
     $1031 = ($1030|0)==(0);
     if ($1031) {
      $1032 = $1028 | $1029;
      HEAP32[((76440 + 4|0))>>2] = $1032;
      HEAP32[$1024>>2] = $636;
      $1033 = (($636) + 24|0);
      HEAP32[$1033>>2] = $1024;
      $1034 = (($636) + 12|0);
      HEAP32[$1034>>2] = $636;
      $1035 = (($636) + 8|0);
      HEAP32[$1035>>2] = $636;
      break;
     }
     $1036 = HEAP32[$1024>>2]|0;
     $1037 = ($I1$0$i$i|0)==(31);
     if ($1037) {
      $1045 = 0;
     } else {
      $1038 = $I1$0$i$i >>> 1;
      $1039 = (25 - ($1038))|0;
      $1045 = $1039;
     }
     $1040 = (($1036) + 4|0);
     $1041 = HEAP32[$1040>>2]|0;
     $1042 = $1041 & -8;
     $1043 = ($1042|0)==($976|0);
     L499: do {
      if ($1043) {
       $T$0$lcssa$i$i = $1036;
      } else {
       $1044 = $976 << $1045;
       $K2$014$i$i = $1044;$T$013$i$i = $1036;
       while(1) {
        $1052 = $K2$014$i$i >>> 31;
        $1053 = ((($T$013$i$i) + ($1052<<2)|0) + 16|0);
        $1048 = HEAP32[$1053>>2]|0;
        $1054 = ($1048|0)==(0|0);
        if ($1054) {
         break;
        }
        $1046 = $K2$014$i$i << 1;
        $1047 = (($1048) + 4|0);
        $1049 = HEAP32[$1047>>2]|0;
        $1050 = $1049 & -8;
        $1051 = ($1050|0)==($976|0);
        if ($1051) {
         $T$0$lcssa$i$i = $1048;
         break L499;
        } else {
         $K2$014$i$i = $1046;$T$013$i$i = $1048;
        }
       }
       $1055 = HEAP32[((76440 + 16|0))>>2]|0;
       $1056 = ($1053>>>0)<($1055>>>0);
       if ($1056) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$1053>>2] = $636;
        $1057 = (($636) + 24|0);
        HEAP32[$1057>>2] = $T$013$i$i;
        $1058 = (($636) + 12|0);
        HEAP32[$1058>>2] = $636;
        $1059 = (($636) + 8|0);
        HEAP32[$1059>>2] = $636;
        break L311;
       }
      }
     } while(0);
     $1060 = (($T$0$lcssa$i$i) + 8|0);
     $1061 = HEAP32[$1060>>2]|0;
     $1062 = HEAP32[((76440 + 16|0))>>2]|0;
     $1063 = ($T$0$lcssa$i$i>>>0)<($1062>>>0);
     if ($1063) {
      _abort();
      // unreachable;
     }
     $1064 = ($1061>>>0)<($1062>>>0);
     if ($1064) {
      _abort();
      // unreachable;
     } else {
      $1065 = (($1061) + 12|0);
      HEAP32[$1065>>2] = $636;
      HEAP32[$1060>>2] = $636;
      $1066 = (($636) + 8|0);
      HEAP32[$1066>>2] = $1061;
      $1067 = (($636) + 12|0);
      HEAP32[$1067>>2] = $T$0$lcssa$i$i;
      $1068 = (($636) + 24|0);
      HEAP32[$1068>>2] = 0;
      break;
     }
    }
   }
  } while(0);
  $1069 = HEAP32[((76440 + 12|0))>>2]|0;
  $1070 = ($1069>>>0)>($nb$0>>>0);
  if ($1070) {
   $1071 = (($1069) - ($nb$0))|0;
   HEAP32[((76440 + 12|0))>>2] = $1071;
   $1072 = HEAP32[((76440 + 24|0))>>2]|0;
   $1073 = (($1072) + ($nb$0)|0);
   HEAP32[((76440 + 24|0))>>2] = $1073;
   $1074 = $1071 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1075 = (($1072) + ($$sum$i32)|0);
   HEAP32[$1075>>2] = $1074;
   $1076 = $nb$0 | 3;
   $1077 = (($1072) + 4|0);
   HEAP32[$1077>>2] = $1076;
   $1078 = (($1072) + 8|0);
   $mem$0 = $1078;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $1079 = (___errno_location()|0);
 HEAP32[$1079>>2] = 12;
 $mem$0 = 0;
 STACKTOP = sp;return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$pre = 0, $$pre$phi68Z2D = 0, $$pre$phi70Z2D = 0, $$pre$phiZ2D = 0, $$pre67 = 0, $$pre69 = 0, $$sum = 0, $$sum16$pre = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum2324 = 0, $$sum25 = 0, $$sum26 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0;
 var $$sum31 = 0, $$sum32 = 0, $$sum33 = 0, $$sum34 = 0, $$sum35 = 0, $$sum36 = 0, $$sum37 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0;
 var $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $I18$0$c = 0, $K19$057 = 0;
 var $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$056 = 0, $cond = 0, $cond54 = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  STACKTOP = sp;return;
 }
 $1 = (($mem) + -8|0);
 $2 = HEAP32[((76440 + 16|0))>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = (($mem) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    STACKTOP = sp;return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[((76440 + 20|0))>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $104 = (($mem) + ($$sum3)|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $105 & 3;
    $107 = ($106|0)==(3);
    if (!($107)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[((76440 + 8|0))>>2] = $15;
    $108 = HEAP32[$104>>2]|0;
    $109 = $108 & -2;
    HEAP32[$104>>2] = $109;
    $110 = $15 | 1;
    $$sum26 = (($$sum2) + 4)|0;
    $111 = (($mem) + ($$sum26)|0);
    HEAP32[$111>>2] = $110;
    HEAP32[$9>>2] = $15;
    STACKTOP = sp;return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum36 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum36)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum37 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum37)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = ((76440 + ($25<<2)|0) + 40|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = (($22) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[76440>>2]|0;
     $36 = $35 & $34;
     HEAP32[76440>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre69 = (($24) + 8|0);
     $$pre$phi70Z2D = $$pre69;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = (($24) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi70Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = (($22) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi70Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum28 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum28)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum29 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum29)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum31 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum31)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum30 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum30)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = (($R$0) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = (($R$0) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum35 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum35)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = (($49) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = (($46) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum32 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum32)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = ((76440 + ($72<<2)|0) + 304|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[((76440 + 4|0))>>2]|0;
      $79 = $78 & $77;
      HEAP32[((76440 + 4|0))>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[((76440 + 16|0))>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = (($44) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = (($44) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[((76440 + 16|0))>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = (($R$1) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum33 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum33)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = HEAP32[((76440 + 16|0))>>2]|0;
      $94 = ($91>>>0)<($93>>>0);
      if ($94) {
       _abort();
       // unreachable;
      } else {
       $95 = (($R$1) + 16|0);
       HEAP32[$95>>2] = $91;
       $96 = (($91) + 24|0);
       HEAP32[$96>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum34 = (($$sum2) + 20)|0;
    $97 = (($mem) + ($$sum34)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0|0);
    if ($99) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $100 = HEAP32[((76440 + 16|0))>>2]|0;
     $101 = ($98>>>0)<($100>>>0);
     if ($101) {
      _abort();
      // unreachable;
     } else {
      $102 = (($R$1) + 20|0);
      HEAP32[$102>>2] = $98;
      $103 = (($98) + 24|0);
      HEAP32[$103>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $112 = ($p$0>>>0)<($9>>>0);
 if (!($112)) {
  _abort();
  // unreachable;
 }
 $$sum25 = (($8) + -4)|0;
 $113 = (($mem) + ($$sum25)|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $114 & 1;
 $116 = ($115|0)==(0);
 if ($116) {
  _abort();
  // unreachable;
 }
 $117 = $114 & 2;
 $118 = ($117|0)==(0);
 if ($118) {
  $119 = HEAP32[((76440 + 24|0))>>2]|0;
  $120 = ($9|0)==($119|0);
  if ($120) {
   $121 = HEAP32[((76440 + 12|0))>>2]|0;
   $122 = (($121) + ($psize$0))|0;
   HEAP32[((76440 + 12|0))>>2] = $122;
   HEAP32[((76440 + 24|0))>>2] = $p$0;
   $123 = $122 | 1;
   $124 = (($p$0) + 4|0);
   HEAP32[$124>>2] = $123;
   $125 = HEAP32[((76440 + 20|0))>>2]|0;
   $126 = ($p$0|0)==($125|0);
   if (!($126)) {
    STACKTOP = sp;return;
   }
   HEAP32[((76440 + 20|0))>>2] = 0;
   HEAP32[((76440 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $127 = HEAP32[((76440 + 20|0))>>2]|0;
  $128 = ($9|0)==($127|0);
  if ($128) {
   $129 = HEAP32[((76440 + 8|0))>>2]|0;
   $130 = (($129) + ($psize$0))|0;
   HEAP32[((76440 + 8|0))>>2] = $130;
   HEAP32[((76440 + 20|0))>>2] = $p$0;
   $131 = $130 | 1;
   $132 = (($p$0) + 4|0);
   HEAP32[$132>>2] = $131;
   $133 = (($p$0) + ($130)|0);
   HEAP32[$133>>2] = $130;
   STACKTOP = sp;return;
  }
  $134 = $114 & -8;
  $135 = (($134) + ($psize$0))|0;
  $136 = $114 >>> 3;
  $137 = ($114>>>0)<(256);
  do {
   if ($137) {
    $138 = (($mem) + ($8)|0);
    $139 = HEAP32[$138>>2]|0;
    $$sum2324 = $8 | 4;
    $140 = (($mem) + ($$sum2324)|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = $136 << 1;
    $143 = ((76440 + ($142<<2)|0) + 40|0);
    $144 = ($139|0)==($143|0);
    if (!($144)) {
     $145 = HEAP32[((76440 + 16|0))>>2]|0;
     $146 = ($139>>>0)<($145>>>0);
     if ($146) {
      _abort();
      // unreachable;
     }
     $147 = (($139) + 12|0);
     $148 = HEAP32[$147>>2]|0;
     $149 = ($148|0)==($9|0);
     if (!($149)) {
      _abort();
      // unreachable;
     }
    }
    $150 = ($141|0)==($139|0);
    if ($150) {
     $151 = 1 << $136;
     $152 = $151 ^ -1;
     $153 = HEAP32[76440>>2]|0;
     $154 = $153 & $152;
     HEAP32[76440>>2] = $154;
     break;
    }
    $155 = ($141|0)==($143|0);
    if ($155) {
     $$pre67 = (($141) + 8|0);
     $$pre$phi68Z2D = $$pre67;
    } else {
     $156 = HEAP32[((76440 + 16|0))>>2]|0;
     $157 = ($141>>>0)<($156>>>0);
     if ($157) {
      _abort();
      // unreachable;
     }
     $158 = (($141) + 8|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ($159|0)==($9|0);
     if ($160) {
      $$pre$phi68Z2D = $158;
     } else {
      _abort();
      // unreachable;
     }
    }
    $161 = (($139) + 12|0);
    HEAP32[$161>>2] = $141;
    HEAP32[$$pre$phi68Z2D>>2] = $139;
   } else {
    $$sum5 = (($8) + 16)|0;
    $162 = (($mem) + ($$sum5)|0);
    $163 = HEAP32[$162>>2]|0;
    $$sum67 = $8 | 4;
    $164 = (($mem) + ($$sum67)|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = ($165|0)==($9|0);
    do {
     if ($166) {
      $$sum9 = (($8) + 12)|0;
      $177 = (($mem) + ($$sum9)|0);
      $178 = HEAP32[$177>>2]|0;
      $179 = ($178|0)==(0|0);
      if ($179) {
       $$sum8 = (($8) + 8)|0;
       $180 = (($mem) + ($$sum8)|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = ($181|0)==(0|0);
       if ($182) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $181;$RP9$0 = $180;
       }
      } else {
       $R7$0 = $178;$RP9$0 = $177;
      }
      while(1) {
       $183 = (($R7$0) + 20|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($184|0)==(0|0);
       if (!($185)) {
        $R7$0 = $184;$RP9$0 = $183;
        continue;
       }
       $186 = (($R7$0) + 16|0);
       $187 = HEAP32[$186>>2]|0;
       $188 = ($187|0)==(0|0);
       if ($188) {
        break;
       } else {
        $R7$0 = $187;$RP9$0 = $186;
       }
      }
      $189 = HEAP32[((76440 + 16|0))>>2]|0;
      $190 = ($RP9$0>>>0)<($189>>>0);
      if ($190) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $167 = (($mem) + ($8)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = HEAP32[((76440 + 16|0))>>2]|0;
      $170 = ($168>>>0)<($169>>>0);
      if ($170) {
       _abort();
       // unreachable;
      }
      $171 = (($168) + 12|0);
      $172 = HEAP32[$171>>2]|0;
      $173 = ($172|0)==($9|0);
      if (!($173)) {
       _abort();
       // unreachable;
      }
      $174 = (($165) + 8|0);
      $175 = HEAP32[$174>>2]|0;
      $176 = ($175|0)==($9|0);
      if ($176) {
       HEAP32[$171>>2] = $165;
       HEAP32[$174>>2] = $168;
       $R7$1 = $165;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $191 = ($163|0)==(0|0);
    if (!($191)) {
     $$sum18 = (($8) + 20)|0;
     $192 = (($mem) + ($$sum18)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ((76440 + ($193<<2)|0) + 304|0);
     $195 = HEAP32[$194>>2]|0;
     $196 = ($9|0)==($195|0);
     if ($196) {
      HEAP32[$194>>2] = $R7$1;
      $cond54 = ($R7$1|0)==(0|0);
      if ($cond54) {
       $197 = 1 << $193;
       $198 = $197 ^ -1;
       $199 = HEAP32[((76440 + 4|0))>>2]|0;
       $200 = $199 & $198;
       HEAP32[((76440 + 4|0))>>2] = $200;
       break;
      }
     } else {
      $201 = HEAP32[((76440 + 16|0))>>2]|0;
      $202 = ($163>>>0)<($201>>>0);
      if ($202) {
       _abort();
       // unreachable;
      }
      $203 = (($163) + 16|0);
      $204 = HEAP32[$203>>2]|0;
      $205 = ($204|0)==($9|0);
      if ($205) {
       HEAP32[$203>>2] = $R7$1;
      } else {
       $206 = (($163) + 20|0);
       HEAP32[$206>>2] = $R7$1;
      }
      $207 = ($R7$1|0)==(0|0);
      if ($207) {
       break;
      }
     }
     $208 = HEAP32[((76440 + 16|0))>>2]|0;
     $209 = ($R7$1>>>0)<($208>>>0);
     if ($209) {
      _abort();
      // unreachable;
     }
     $210 = (($R7$1) + 24|0);
     HEAP32[$210>>2] = $163;
     $$sum19 = (($8) + 8)|0;
     $211 = (($mem) + ($$sum19)|0);
     $212 = HEAP32[$211>>2]|0;
     $213 = ($212|0)==(0|0);
     do {
      if (!($213)) {
       $214 = HEAP32[((76440 + 16|0))>>2]|0;
       $215 = ($212>>>0)<($214>>>0);
       if ($215) {
        _abort();
        // unreachable;
       } else {
        $216 = (($R7$1) + 16|0);
        HEAP32[$216>>2] = $212;
        $217 = (($212) + 24|0);
        HEAP32[$217>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum20 = (($8) + 12)|0;
     $218 = (($mem) + ($$sum20)|0);
     $219 = HEAP32[$218>>2]|0;
     $220 = ($219|0)==(0|0);
     if (!($220)) {
      $221 = HEAP32[((76440 + 16|0))>>2]|0;
      $222 = ($219>>>0)<($221>>>0);
      if ($222) {
       _abort();
       // unreachable;
      } else {
       $223 = (($R7$1) + 20|0);
       HEAP32[$223>>2] = $219;
       $224 = (($219) + 24|0);
       HEAP32[$224>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $225 = $135 | 1;
  $226 = (($p$0) + 4|0);
  HEAP32[$226>>2] = $225;
  $227 = (($p$0) + ($135)|0);
  HEAP32[$227>>2] = $135;
  $228 = HEAP32[((76440 + 20|0))>>2]|0;
  $229 = ($p$0|0)==($228|0);
  if ($229) {
   HEAP32[((76440 + 8|0))>>2] = $135;
   STACKTOP = sp;return;
  } else {
   $psize$1 = $135;
  }
 } else {
  $230 = $114 & -2;
  HEAP32[$113>>2] = $230;
  $231 = $psize$0 | 1;
  $232 = (($p$0) + 4|0);
  HEAP32[$232>>2] = $231;
  $233 = (($p$0) + ($psize$0)|0);
  HEAP32[$233>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $234 = $psize$1 >>> 3;
 $235 = ($psize$1>>>0)<(256);
 if ($235) {
  $236 = $234 << 1;
  $237 = ((76440 + ($236<<2)|0) + 40|0);
  $238 = HEAP32[76440>>2]|0;
  $239 = 1 << $234;
  $240 = $238 & $239;
  $241 = ($240|0)==(0);
  if ($241) {
   $242 = $238 | $239;
   HEAP32[76440>>2] = $242;
   $$sum16$pre = (($236) + 2)|0;
   $$pre = ((76440 + ($$sum16$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $237;
  } else {
   $$sum17 = (($236) + 2)|0;
   $243 = ((76440 + ($$sum17<<2)|0) + 40|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = HEAP32[((76440 + 16|0))>>2]|0;
   $246 = ($244>>>0)<($245>>>0);
   if ($246) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $243;$F16$0 = $244;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $247 = (($F16$0) + 12|0);
  HEAP32[$247>>2] = $p$0;
  $248 = (($p$0) + 8|0);
  HEAP32[$248>>2] = $F16$0;
  $249 = (($p$0) + 12|0);
  HEAP32[$249>>2] = $237;
  STACKTOP = sp;return;
 }
 $250 = $psize$1 >>> 8;
 $251 = ($250|0)==(0);
 if ($251) {
  $I18$0 = 0;
 } else {
  $252 = ($psize$1>>>0)>(16777215);
  if ($252) {
   $I18$0 = 31;
  } else {
   $253 = (($250) + 1048320)|0;
   $254 = $253 >>> 16;
   $255 = $254 & 8;
   $256 = $250 << $255;
   $257 = (($256) + 520192)|0;
   $258 = $257 >>> 16;
   $259 = $258 & 4;
   $260 = $259 | $255;
   $261 = $256 << $259;
   $262 = (($261) + 245760)|0;
   $263 = $262 >>> 16;
   $264 = $263 & 2;
   $265 = $260 | $264;
   $266 = (14 - ($265))|0;
   $267 = $261 << $264;
   $268 = $267 >>> 15;
   $269 = (($266) + ($268))|0;
   $270 = $269 << 1;
   $271 = (($269) + 7)|0;
   $272 = $psize$1 >>> $271;
   $273 = $272 & 1;
   $274 = $273 | $270;
   $I18$0 = $274;
  }
 }
 $275 = ((76440 + ($I18$0<<2)|0) + 304|0);
 $276 = (($p$0) + 28|0);
 $I18$0$c = $I18$0;
 HEAP32[$276>>2] = $I18$0$c;
 $277 = (($p$0) + 20|0);
 HEAP32[$277>>2] = 0;
 $278 = (($p$0) + 16|0);
 HEAP32[$278>>2] = 0;
 $279 = HEAP32[((76440 + 4|0))>>2]|0;
 $280 = 1 << $I18$0;
 $281 = $279 & $280;
 $282 = ($281|0)==(0);
 L199: do {
  if ($282) {
   $283 = $279 | $280;
   HEAP32[((76440 + 4|0))>>2] = $283;
   HEAP32[$275>>2] = $p$0;
   $284 = (($p$0) + 24|0);
   HEAP32[$284>>2] = $275;
   $285 = (($p$0) + 12|0);
   HEAP32[$285>>2] = $p$0;
   $286 = (($p$0) + 8|0);
   HEAP32[$286>>2] = $p$0;
  } else {
   $287 = HEAP32[$275>>2]|0;
   $288 = ($I18$0|0)==(31);
   if ($288) {
    $296 = 0;
   } else {
    $289 = $I18$0 >>> 1;
    $290 = (25 - ($289))|0;
    $296 = $290;
   }
   $291 = (($287) + 4|0);
   $292 = HEAP32[$291>>2]|0;
   $293 = $292 & -8;
   $294 = ($293|0)==($psize$1|0);
   L205: do {
    if ($294) {
     $T$0$lcssa = $287;
    } else {
     $295 = $psize$1 << $296;
     $K19$057 = $295;$T$056 = $287;
     while(1) {
      $303 = $K19$057 >>> 31;
      $304 = ((($T$056) + ($303<<2)|0) + 16|0);
      $299 = HEAP32[$304>>2]|0;
      $305 = ($299|0)==(0|0);
      if ($305) {
       break;
      }
      $297 = $K19$057 << 1;
      $298 = (($299) + 4|0);
      $300 = HEAP32[$298>>2]|0;
      $301 = $300 & -8;
      $302 = ($301|0)==($psize$1|0);
      if ($302) {
       $T$0$lcssa = $299;
       break L205;
      } else {
       $K19$057 = $297;$T$056 = $299;
      }
     }
     $306 = HEAP32[((76440 + 16|0))>>2]|0;
     $307 = ($304>>>0)<($306>>>0);
     if ($307) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$304>>2] = $p$0;
      $308 = (($p$0) + 24|0);
      HEAP32[$308>>2] = $T$056;
      $309 = (($p$0) + 12|0);
      HEAP32[$309>>2] = $p$0;
      $310 = (($p$0) + 8|0);
      HEAP32[$310>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $311 = (($T$0$lcssa) + 8|0);
   $312 = HEAP32[$311>>2]|0;
   $313 = HEAP32[((76440 + 16|0))>>2]|0;
   $314 = ($T$0$lcssa>>>0)<($313>>>0);
   if ($314) {
    _abort();
    // unreachable;
   }
   $315 = ($312>>>0)<($313>>>0);
   if ($315) {
    _abort();
    // unreachable;
   } else {
    $316 = (($312) + 12|0);
    HEAP32[$316>>2] = $p$0;
    HEAP32[$311>>2] = $p$0;
    $317 = (($p$0) + 8|0);
    HEAP32[$317>>2] = $312;
    $318 = (($p$0) + 12|0);
    HEAP32[$318>>2] = $T$0$lcssa;
    $319 = (($p$0) + 24|0);
    HEAP32[$319>>2] = 0;
    break;
   }
  }
 } while(0);
 $320 = HEAP32[((76440 + 32|0))>>2]|0;
 $321 = (($320) + -1)|0;
 HEAP32[((76440 + 32|0))>>2] = $321;
 $322 = ($321|0)==(0);
 if ($322) {
  $sp$0$in$i = ((76440 + 456|0));
 } else {
  STACKTOP = sp;return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $323 = ($sp$0$i|0)==(0|0);
  $324 = (($sp$0$i) + 8|0);
  if ($323) {
   break;
  } else {
   $sp$0$in$i = $324;
  }
 }
 HEAP32[((76440 + 32|0))>>2] = -1;
 STACKTOP = sp;return;
}
function _calloc($n_elements,$elem_size) {
 $n_elements = $n_elements|0;
 $elem_size = $elem_size|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $req$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n_elements|0)==(0);
 if ($0) {
  $req$0 = 0;
 } else {
  $1 = Math_imul($elem_size, $n_elements)|0;
  $2 = $elem_size | $n_elements;
  $3 = ($2>>>0)>(65535);
  if ($3) {
   $4 = (($1>>>0) / ($n_elements>>>0))&-1;
   $5 = ($4|0)==($elem_size|0);
   $$ = $5 ? $1 : -1;
   $req$0 = $$;
  } else {
   $req$0 = $1;
  }
 }
 $6 = (_malloc($req$0)|0);
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return ($6|0);
 }
 $8 = (($6) + -4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 3;
 $11 = ($10|0)==(0);
 if ($11) {
  STACKTOP = sp;return ($6|0);
 }
 _memset(($6|0),0,($req$0|0))|0;
 STACKTOP = sp;return ($6|0);
}
function _realloc($oldmem,$bytes) {
 $oldmem = $oldmem|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $mem$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($oldmem|0)==(0|0);
 do {
  if ($0) {
   $1 = (_malloc($bytes)|0);
   $mem$0 = $1;
  } else {
   $2 = ($bytes>>>0)>(4294967231);
   if ($2) {
    $3 = (___errno_location()|0);
    HEAP32[$3>>2] = 12;
    $mem$0 = 0;
    break;
   }
   $4 = ($bytes>>>0)<(11);
   if ($4) {
    $8 = 16;
   } else {
    $5 = (($bytes) + 11)|0;
    $6 = $5 & -8;
    $8 = $6;
   }
   $7 = (($oldmem) + -8|0);
   $9 = (_try_realloc_chunk($7,$8)|0);
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = (($9) + 8|0);
    $mem$0 = $11;
    break;
   }
   $12 = (_malloc($bytes)|0);
   $13 = ($12|0)==(0|0);
   if ($13) {
    $mem$0 = 0;
   } else {
    $14 = (($oldmem) + -4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = $15 & -8;
    $17 = $15 & 3;
    $18 = ($17|0)==(0);
    $19 = $18 ? 8 : 4;
    $20 = (($16) - ($19))|0;
    $21 = ($20>>>0)<($bytes>>>0);
    $22 = $21 ? $20 : $bytes;
    _memcpy(($12|0),($oldmem|0),($22|0))|0;
    _free($oldmem);
    $mem$0 = $12;
   }
  }
 } while(0);
 STACKTOP = sp;return ($mem$0|0);
}
function _try_realloc_chunk($p,$nb) {
 $p = $p|0;
 $nb = $nb|0;
 var $$pre = 0, $$pre$phiZ2D = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum2728 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum78 = 0;
 var $$sum910 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $R$0 = 0, $R$1 = 0, $RP$0 = 0;
 var $cond = 0, $newp$0 = 0, $or$cond = 0, $storemerge = 0, $storemerge21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 & -8;
 $3 = (($p) + ($2)|0);
 $4 = HEAP32[((76440 + 16|0))>>2]|0;
 $5 = ($p>>>0)<($4>>>0);
 if ($5) {
  _abort();
  // unreachable;
 }
 $6 = $1 & 3;
 $7 = ($6|0)!=(1);
 $8 = ($p>>>0)<($3>>>0);
 $or$cond = $7 & $8;
 if (!($or$cond)) {
  _abort();
  // unreachable;
 }
 $$sum2728 = $2 | 4;
 $9 = (($p) + ($$sum2728)|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 1;
 $12 = ($11|0)==(0);
 if ($12) {
  _abort();
  // unreachable;
 }
 $13 = ($6|0)==(0);
 if ($13) {
  $14 = ($nb>>>0)<(256);
  if ($14) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $15 = (($nb) + 4)|0;
  $16 = ($2>>>0)<($15>>>0);
  if (!($16)) {
   $17 = (($2) - ($nb))|0;
   $18 = HEAP32[((76912 + 8|0))>>2]|0;
   $19 = $18 << 1;
   $20 = ($17>>>0)>($19>>>0);
   if (!($20)) {
    $newp$0 = $p;
    STACKTOP = sp;return ($newp$0|0);
   }
  }
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $21 = ($2>>>0)<($nb>>>0);
 if (!($21)) {
  $22 = (($2) - ($nb))|0;
  $23 = ($22>>>0)>(15);
  if (!($23)) {
   $newp$0 = $p;
   STACKTOP = sp;return ($newp$0|0);
  }
  $24 = (($p) + ($nb)|0);
  $25 = $1 & 1;
  $26 = $25 | $nb;
  $27 = $26 | 2;
  HEAP32[$0>>2] = $27;
  $$sum23 = (($nb) + 4)|0;
  $28 = (($p) + ($$sum23)|0);
  $29 = $22 | 3;
  HEAP32[$28>>2] = $29;
  $30 = HEAP32[$9>>2]|0;
  $31 = $30 | 1;
  HEAP32[$9>>2] = $31;
  _dispose_chunk($24,$22);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $32 = HEAP32[((76440 + 24|0))>>2]|0;
 $33 = ($3|0)==($32|0);
 if ($33) {
  $34 = HEAP32[((76440 + 12|0))>>2]|0;
  $35 = (($34) + ($2))|0;
  $36 = ($35>>>0)>($nb>>>0);
  if (!($36)) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $37 = (($35) - ($nb))|0;
  $38 = (($p) + ($nb)|0);
  $39 = $1 & 1;
  $40 = $39 | $nb;
  $41 = $40 | 2;
  HEAP32[$0>>2] = $41;
  $$sum22 = (($nb) + 4)|0;
  $42 = (($p) + ($$sum22)|0);
  $43 = $37 | 1;
  HEAP32[$42>>2] = $43;
  HEAP32[((76440 + 24|0))>>2] = $38;
  HEAP32[((76440 + 12|0))>>2] = $37;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $44 = HEAP32[((76440 + 20|0))>>2]|0;
 $45 = ($3|0)==($44|0);
 if ($45) {
  $46 = HEAP32[((76440 + 8|0))>>2]|0;
  $47 = (($46) + ($2))|0;
  $48 = ($47>>>0)<($nb>>>0);
  if ($48) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $49 = (($47) - ($nb))|0;
  $50 = ($49>>>0)>(15);
  if ($50) {
   $51 = (($p) + ($nb)|0);
   $52 = (($p) + ($47)|0);
   $53 = $1 & 1;
   $54 = $53 | $nb;
   $55 = $54 | 2;
   HEAP32[$0>>2] = $55;
   $$sum19 = (($nb) + 4)|0;
   $56 = (($p) + ($$sum19)|0);
   $57 = $49 | 1;
   HEAP32[$56>>2] = $57;
   HEAP32[$52>>2] = $49;
   $$sum20 = (($47) + 4)|0;
   $58 = (($p) + ($$sum20)|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $59 & -2;
   HEAP32[$58>>2] = $60;
   $storemerge = $51;$storemerge21 = $49;
  } else {
   $61 = $1 & 1;
   $62 = $61 | $47;
   $63 = $62 | 2;
   HEAP32[$0>>2] = $63;
   $$sum17 = (($47) + 4)|0;
   $64 = (($p) + ($$sum17)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 | 1;
   HEAP32[$64>>2] = $66;
   $storemerge = 0;$storemerge21 = 0;
  }
  HEAP32[((76440 + 8|0))>>2] = $storemerge21;
  HEAP32[((76440 + 20|0))>>2] = $storemerge;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $67 = $10 & 2;
 $68 = ($67|0)==(0);
 if (!($68)) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $69 = $10 & -8;
 $70 = (($69) + ($2))|0;
 $71 = ($70>>>0)<($nb>>>0);
 if ($71) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $72 = (($70) - ($nb))|0;
 $73 = $10 >>> 3;
 $74 = ($10>>>0)<(256);
 do {
  if ($74) {
   $$sum15 = (($2) + 8)|0;
   $75 = (($p) + ($$sum15)|0);
   $76 = HEAP32[$75>>2]|0;
   $$sum16 = (($2) + 12)|0;
   $77 = (($p) + ($$sum16)|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $73 << 1;
   $80 = ((76440 + ($79<<2)|0) + 40|0);
   $81 = ($76|0)==($80|0);
   if (!($81)) {
    $82 = ($76>>>0)<($4>>>0);
    if ($82) {
     _abort();
     // unreachable;
    }
    $83 = (($76) + 12|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = ($84|0)==($3|0);
    if (!($85)) {
     _abort();
     // unreachable;
    }
   }
   $86 = ($78|0)==($76|0);
   if ($86) {
    $87 = 1 << $73;
    $88 = $87 ^ -1;
    $89 = HEAP32[76440>>2]|0;
    $90 = $89 & $88;
    HEAP32[76440>>2] = $90;
    break;
   }
   $91 = ($78|0)==($80|0);
   if ($91) {
    $$pre = (($78) + 8|0);
    $$pre$phiZ2D = $$pre;
   } else {
    $92 = ($78>>>0)<($4>>>0);
    if ($92) {
     _abort();
     // unreachable;
    }
    $93 = (($78) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==($3|0);
    if ($95) {
     $$pre$phiZ2D = $93;
    } else {
     _abort();
     // unreachable;
    }
   }
   $96 = (($76) + 12|0);
   HEAP32[$96>>2] = $78;
   HEAP32[$$pre$phiZ2D>>2] = $76;
  } else {
   $$sum = (($2) + 24)|0;
   $97 = (($p) + ($$sum)|0);
   $98 = HEAP32[$97>>2]|0;
   $$sum2 = (($2) + 12)|0;
   $99 = (($p) + ($$sum2)|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = ($100|0)==($3|0);
   do {
    if ($101) {
     $$sum4 = (($2) + 20)|0;
     $111 = (($p) + ($$sum4)|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = ($112|0)==(0|0);
     if ($113) {
      $$sum3 = (($2) + 16)|0;
      $114 = (($p) + ($$sum3)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = ($115|0)==(0|0);
      if ($116) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $115;$RP$0 = $114;
      }
     } else {
      $R$0 = $112;$RP$0 = $111;
     }
     while(1) {
      $117 = (($R$0) + 20|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = ($118|0)==(0|0);
      if (!($119)) {
       $R$0 = $118;$RP$0 = $117;
       continue;
      }
      $120 = (($R$0) + 16|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ($121|0)==(0|0);
      if ($122) {
       break;
      } else {
       $R$0 = $121;$RP$0 = $120;
      }
     }
     $123 = ($RP$0>>>0)<($4>>>0);
     if ($123) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum14 = (($2) + 8)|0;
     $102 = (($p) + ($$sum14)|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = ($103>>>0)<($4>>>0);
     if ($104) {
      _abort();
      // unreachable;
     }
     $105 = (($103) + 12|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = ($106|0)==($3|0);
     if (!($107)) {
      _abort();
      // unreachable;
     }
     $108 = (($100) + 8|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = ($109|0)==($3|0);
     if ($110) {
      HEAP32[$105>>2] = $100;
      HEAP32[$108>>2] = $103;
      $R$1 = $100;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $124 = ($98|0)==(0|0);
   if (!($124)) {
    $$sum11 = (($2) + 28)|0;
    $125 = (($p) + ($$sum11)|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ((76440 + ($126<<2)|0) + 304|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = ($3|0)==($128|0);
    if ($129) {
     HEAP32[$127>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $130 = 1 << $126;
      $131 = $130 ^ -1;
      $132 = HEAP32[((76440 + 4|0))>>2]|0;
      $133 = $132 & $131;
      HEAP32[((76440 + 4|0))>>2] = $133;
      break;
     }
    } else {
     $134 = HEAP32[((76440 + 16|0))>>2]|0;
     $135 = ($98>>>0)<($134>>>0);
     if ($135) {
      _abort();
      // unreachable;
     }
     $136 = (($98) + 16|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = ($137|0)==($3|0);
     if ($138) {
      HEAP32[$136>>2] = $R$1;
     } else {
      $139 = (($98) + 20|0);
      HEAP32[$139>>2] = $R$1;
     }
     $140 = ($R$1|0)==(0|0);
     if ($140) {
      break;
     }
    }
    $141 = HEAP32[((76440 + 16|0))>>2]|0;
    $142 = ($R$1>>>0)<($141>>>0);
    if ($142) {
     _abort();
     // unreachable;
    }
    $143 = (($R$1) + 24|0);
    HEAP32[$143>>2] = $98;
    $$sum12 = (($2) + 16)|0;
    $144 = (($p) + ($$sum12)|0);
    $145 = HEAP32[$144>>2]|0;
    $146 = ($145|0)==(0|0);
    do {
     if (!($146)) {
      $147 = HEAP32[((76440 + 16|0))>>2]|0;
      $148 = ($145>>>0)<($147>>>0);
      if ($148) {
       _abort();
       // unreachable;
      } else {
       $149 = (($R$1) + 16|0);
       HEAP32[$149>>2] = $145;
       $150 = (($145) + 24|0);
       HEAP32[$150>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum13 = (($2) + 20)|0;
    $151 = (($p) + ($$sum13)|0);
    $152 = HEAP32[$151>>2]|0;
    $153 = ($152|0)==(0|0);
    if (!($153)) {
     $154 = HEAP32[((76440 + 16|0))>>2]|0;
     $155 = ($152>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     } else {
      $156 = (($R$1) + 20|0);
      HEAP32[$156>>2] = $152;
      $157 = (($152) + 24|0);
      HEAP32[$157>>2] = $R$1;
      break;
     }
    }
   }
  }
 } while(0);
 $158 = ($72>>>0)<(16);
 if ($158) {
  $159 = HEAP32[$0>>2]|0;
  $160 = $159 & 1;
  $161 = $70 | $160;
  $162 = $161 | 2;
  HEAP32[$0>>2] = $162;
  $$sum910 = $70 | 4;
  $163 = (($p) + ($$sum910)|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = $164 | 1;
  HEAP32[$163>>2] = $165;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 } else {
  $166 = (($p) + ($nb)|0);
  $167 = HEAP32[$0>>2]|0;
  $168 = $167 & 1;
  $169 = $168 | $nb;
  $170 = $169 | 2;
  HEAP32[$0>>2] = $170;
  $$sum5 = (($nb) + 4)|0;
  $171 = (($p) + ($$sum5)|0);
  $172 = $72 | 3;
  HEAP32[$171>>2] = $172;
  $$sum78 = $70 | 4;
  $173 = (($p) + ($$sum78)|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = $174 | 1;
  HEAP32[$173>>2] = $175;
  _dispose_chunk($166,$72);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 return 0|0;
}
function _memalign($alignment,$bytes) {
 $alignment = $alignment|0;
 $bytes = $bytes|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($alignment>>>0)<(9);
 if ($0) {
  $1 = (_malloc($bytes)|0);
  $$0 = $1;
 } else {
  $2 = (_internal_memalign($alignment,$bytes)|0);
  $$0 = $2;
 }
 STACKTOP = sp;return ($$0|0);
}
function _internal_memalign($alignment,$bytes) {
 $alignment = $alignment|0;
 $bytes = $bytes|0;
 var $$1 = 0, $$alignment = 0, $$sum1 = 0, $$sum2 = 0, $$sum3 = 0, $$sum4 = 0, $$sum6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $a$0 = 0, $mem$0 = 0, $p$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($alignment>>>0)<(16);
 $$alignment = $0 ? 16 : $alignment;
 $1 = (($$alignment) + -1)|0;
 $2 = $1 & $$alignment;
 $3 = ($2|0)==(0);
 if ($3) {
  $$1 = $$alignment;
 } else {
  $a$0 = 16;
  while(1) {
   $4 = ($a$0>>>0)<($$alignment>>>0);
   $5 = $a$0 << 1;
   if ($4) {
    $a$0 = $5;
   } else {
    $$1 = $a$0;
    break;
   }
  }
 }
 $6 = (-64 - ($$1))|0;
 $7 = ($6>>>0)>($bytes>>>0);
 if (!($7)) {
  $8 = (___errno_location()|0);
  HEAP32[$8>>2] = 12;
  $mem$0 = 0;
  STACKTOP = sp;return ($mem$0|0);
 }
 $9 = ($bytes>>>0)<(11);
 if ($9) {
  $14 = 16;
 } else {
  $10 = (($bytes) + 11)|0;
  $11 = $10 & -8;
  $14 = $11;
 }
 $12 = (($$1) + 12)|0;
 $13 = (($12) + ($14))|0;
 $15 = (_malloc($13)|0);
 $16 = ($15|0)==(0|0);
 if ($16) {
  $mem$0 = 0;
  STACKTOP = sp;return ($mem$0|0);
 }
 $17 = (($15) + -8|0);
 $18 = $15;
 $19 = (($$1) + -1)|0;
 $20 = $18 & $19;
 $21 = ($20|0)==(0);
 do {
  if ($21) {
   $p$0 = $17;
  } else {
   $22 = (($15) + ($19)|0);
   $23 = $22;
   $24 = (0 - ($$1))|0;
   $25 = $23 & $24;
   $26 = $25;
   $27 = (($26) + -8|0);
   $28 = $27;
   $29 = $17;
   $30 = (($28) - ($29))|0;
   $31 = ($30>>>0)>(15);
   if ($31) {
    $34 = $27;
   } else {
    $$sum3 = (($$1) + -8)|0;
    $32 = (($26) + ($$sum3)|0);
    $34 = $32;
   }
   $33 = $34;
   $35 = (($33) - ($29))|0;
   $36 = (($15) + -4|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = $37 & -8;
   $39 = (($38) - ($35))|0;
   $40 = $37 & 3;
   $41 = ($40|0)==(0);
   if ($41) {
    $42 = HEAP32[$17>>2]|0;
    $43 = (($42) + ($35))|0;
    HEAP32[$34>>2] = $43;
    $44 = (($34) + 4|0);
    HEAP32[$44>>2] = $39;
    $p$0 = $34;
    break;
   } else {
    $45 = (($34) + 4|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = $46 & 1;
    $48 = $39 | $47;
    $49 = $48 | 2;
    HEAP32[$45>>2] = $49;
    $$sum4 = (($39) + 4)|0;
    $50 = (($34) + ($$sum4)|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = $51 | 1;
    HEAP32[$50>>2] = $52;
    $53 = HEAP32[$36>>2]|0;
    $54 = $53 & 1;
    $55 = $35 | $54;
    $56 = $55 | 2;
    HEAP32[$36>>2] = $56;
    $$sum6 = (($35) + -4)|0;
    $57 = (($15) + ($$sum6)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $58 | 1;
    HEAP32[$57>>2] = $59;
    _dispose_chunk($17,$35);
    $p$0 = $34;
    break;
   }
  }
 } while(0);
 $60 = (($p$0) + 4|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = $61 & 3;
 $63 = ($62|0)==(0);
 if (!($63)) {
  $64 = $61 & -8;
  $65 = (($14) + 16)|0;
  $66 = ($64>>>0)>($65>>>0);
  if ($66) {
   $67 = (($64) - ($14))|0;
   $68 = (($p$0) + ($14)|0);
   $69 = $61 & 1;
   $70 = $14 | $69;
   $71 = $70 | 2;
   HEAP32[$60>>2] = $71;
   $$sum1 = $14 | 4;
   $72 = (($p$0) + ($$sum1)|0);
   $73 = $67 | 3;
   HEAP32[$72>>2] = $73;
   $$sum2 = $64 | 4;
   $74 = (($p$0) + ($$sum2)|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = $75 | 1;
   HEAP32[$74>>2] = $76;
   _dispose_chunk($68,$67);
  }
 }
 $77 = (($p$0) + 8|0);
 $mem$0 = $77;
 STACKTOP = sp;return ($mem$0|0);
}
function _dispose_chunk($p,$psize) {
 $p = $p|0;
 $psize = $psize|0;
 var $$0 = 0, $$02 = 0, $$1 = 0, $$pre = 0, $$pre$phi63Z2D = 0, $$pre$phi65Z2D = 0, $$pre$phiZ2D = 0, $$pre62 = 0, $$pre64 = 0, $$sum = 0, $$sum1 = 0, $$sum12$pre = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0;
 var $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0, $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum4 = 0, $$sum5 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0;
 var $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0;
 var $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0;
 var $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0;
 var $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0;
 var $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0;
 var $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0;
 var $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0;
 var $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0;
 var $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0;
 var $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0;
 var $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0;
 var $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I19$0 = 0, $I19$0$c = 0, $K20$049 = 0, $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$048 = 0, $cond = 0, $cond46 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = (($p) + ($psize)|0);
 $1 = (($p) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = HEAP32[$p>>2]|0;
   $6 = $2 & 3;
   $7 = ($6|0)==(0);
   if ($7) {
    STACKTOP = sp;return;
   }
   $8 = (0 - ($5))|0;
   $9 = (($p) + ($8)|0);
   $10 = (($5) + ($psize))|0;
   $11 = HEAP32[((76440 + 16|0))>>2]|0;
   $12 = ($9>>>0)<($11>>>0);
   if ($12) {
    _abort();
    // unreachable;
   }
   $13 = HEAP32[((76440 + 20|0))>>2]|0;
   $14 = ($9|0)==($13|0);
   if ($14) {
    $$sum = (($psize) + 4)|0;
    $100 = (($p) + ($$sum)|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = $101 & 3;
    $103 = ($102|0)==(3);
    if (!($103)) {
     $$0 = $9;$$02 = $10;
     break;
    }
    HEAP32[((76440 + 8|0))>>2] = $10;
    $104 = HEAP32[$100>>2]|0;
    $105 = $104 & -2;
    HEAP32[$100>>2] = $105;
    $106 = $10 | 1;
    $$sum20 = (4 - ($5))|0;
    $107 = (($p) + ($$sum20)|0);
    HEAP32[$107>>2] = $106;
    HEAP32[$0>>2] = $10;
    STACKTOP = sp;return;
   }
   $15 = $5 >>> 3;
   $16 = ($5>>>0)<(256);
   if ($16) {
    $$sum30 = (8 - ($5))|0;
    $17 = (($p) + ($$sum30)|0);
    $18 = HEAP32[$17>>2]|0;
    $$sum31 = (12 - ($5))|0;
    $19 = (($p) + ($$sum31)|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $15 << 1;
    $22 = ((76440 + ($21<<2)|0) + 40|0);
    $23 = ($18|0)==($22|0);
    if (!($23)) {
     $24 = ($18>>>0)<($11>>>0);
     if ($24) {
      _abort();
      // unreachable;
     }
     $25 = (($18) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==($9|0);
     if (!($27)) {
      _abort();
      // unreachable;
     }
    }
    $28 = ($20|0)==($18|0);
    if ($28) {
     $29 = 1 << $15;
     $30 = $29 ^ -1;
     $31 = HEAP32[76440>>2]|0;
     $32 = $31 & $30;
     HEAP32[76440>>2] = $32;
     $$0 = $9;$$02 = $10;
     break;
    }
    $33 = ($20|0)==($22|0);
    if ($33) {
     $$pre64 = (($20) + 8|0);
     $$pre$phi65Z2D = $$pre64;
    } else {
     $34 = ($20>>>0)<($11>>>0);
     if ($34) {
      _abort();
      // unreachable;
     }
     $35 = (($20) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)==($9|0);
     if ($37) {
      $$pre$phi65Z2D = $35;
     } else {
      _abort();
      // unreachable;
     }
    }
    $38 = (($18) + 12|0);
    HEAP32[$38>>2] = $20;
    HEAP32[$$pre$phi65Z2D>>2] = $18;
    $$0 = $9;$$02 = $10;
    break;
   }
   $$sum22 = (24 - ($5))|0;
   $39 = (($p) + ($$sum22)|0);
   $40 = HEAP32[$39>>2]|0;
   $$sum23 = (12 - ($5))|0;
   $41 = (($p) + ($$sum23)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==($9|0);
   do {
    if ($43) {
     $$sum24 = (16 - ($5))|0;
     $$sum25 = (($$sum24) + 4)|0;
     $53 = (($p) + ($$sum25)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)==(0|0);
     if ($55) {
      $56 = (($p) + ($$sum24)|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==(0|0);
      if ($58) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $57;$RP$0 = $56;
      }
     } else {
      $R$0 = $54;$RP$0 = $53;
     }
     while(1) {
      $59 = (($R$0) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==(0|0);
      if (!($61)) {
       $R$0 = $60;$RP$0 = $59;
       continue;
      }
      $62 = (($R$0) + 16|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       break;
      } else {
       $R$0 = $63;$RP$0 = $62;
      }
     }
     $65 = ($RP$0>>>0)<($11>>>0);
     if ($65) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum29 = (8 - ($5))|0;
     $44 = (($p) + ($$sum29)|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($45>>>0)<($11>>>0);
     if ($46) {
      _abort();
      // unreachable;
     }
     $47 = (($45) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)==($9|0);
     if (!($49)) {
      _abort();
      // unreachable;
     }
     $50 = (($42) + 8|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)==($9|0);
     if ($52) {
      HEAP32[$47>>2] = $42;
      HEAP32[$50>>2] = $45;
      $R$1 = $42;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $66 = ($40|0)==(0|0);
   if ($66) {
    $$0 = $9;$$02 = $10;
   } else {
    $$sum26 = (28 - ($5))|0;
    $67 = (($p) + ($$sum26)|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ((76440 + ($68<<2)|0) + 304|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($9|0)==($70|0);
    if ($71) {
     HEAP32[$69>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $72 = 1 << $68;
      $73 = $72 ^ -1;
      $74 = HEAP32[((76440 + 4|0))>>2]|0;
      $75 = $74 & $73;
      HEAP32[((76440 + 4|0))>>2] = $75;
      $$0 = $9;$$02 = $10;
      break;
     }
    } else {
     $76 = HEAP32[((76440 + 16|0))>>2]|0;
     $77 = ($40>>>0)<($76>>>0);
     if ($77) {
      _abort();
      // unreachable;
     }
     $78 = (($40) + 16|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)==($9|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
     } else {
      $81 = (($40) + 20|0);
      HEAP32[$81>>2] = $R$1;
     }
     $82 = ($R$1|0)==(0|0);
     if ($82) {
      $$0 = $9;$$02 = $10;
      break;
     }
    }
    $83 = HEAP32[((76440 + 16|0))>>2]|0;
    $84 = ($R$1>>>0)<($83>>>0);
    if ($84) {
     _abort();
     // unreachable;
    }
    $85 = (($R$1) + 24|0);
    HEAP32[$85>>2] = $40;
    $$sum27 = (16 - ($5))|0;
    $86 = (($p) + ($$sum27)|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($87|0)==(0|0);
    do {
     if (!($88)) {
      $89 = HEAP32[((76440 + 16|0))>>2]|0;
      $90 = ($87>>>0)<($89>>>0);
      if ($90) {
       _abort();
       // unreachable;
      } else {
       $91 = (($R$1) + 16|0);
       HEAP32[$91>>2] = $87;
       $92 = (($87) + 24|0);
       HEAP32[$92>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum27) + 4)|0;
    $93 = (($p) + ($$sum28)|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==(0|0);
    if ($95) {
     $$0 = $9;$$02 = $10;
    } else {
     $96 = HEAP32[((76440 + 16|0))>>2]|0;
     $97 = ($94>>>0)<($96>>>0);
     if ($97) {
      _abort();
      // unreachable;
     } else {
      $98 = (($R$1) + 20|0);
      HEAP32[$98>>2] = $94;
      $99 = (($94) + 24|0);
      HEAP32[$99>>2] = $R$1;
      $$0 = $9;$$02 = $10;
      break;
     }
    }
   }
  } else {
   $$0 = $p;$$02 = $psize;
  }
 } while(0);
 $108 = HEAP32[((76440 + 16|0))>>2]|0;
 $109 = ($0>>>0)<($108>>>0);
 if ($109) {
  _abort();
  // unreachable;
 }
 $$sum1 = (($psize) + 4)|0;
 $110 = (($p) + ($$sum1)|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = $111 & 2;
 $113 = ($112|0)==(0);
 if ($113) {
  $114 = HEAP32[((76440 + 24|0))>>2]|0;
  $115 = ($0|0)==($114|0);
  if ($115) {
   $116 = HEAP32[((76440 + 12|0))>>2]|0;
   $117 = (($116) + ($$02))|0;
   HEAP32[((76440 + 12|0))>>2] = $117;
   HEAP32[((76440 + 24|0))>>2] = $$0;
   $118 = $117 | 1;
   $119 = (($$0) + 4|0);
   HEAP32[$119>>2] = $118;
   $120 = HEAP32[((76440 + 20|0))>>2]|0;
   $121 = ($$0|0)==($120|0);
   if (!($121)) {
    STACKTOP = sp;return;
   }
   HEAP32[((76440 + 20|0))>>2] = 0;
   HEAP32[((76440 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $122 = HEAP32[((76440 + 20|0))>>2]|0;
  $123 = ($0|0)==($122|0);
  if ($123) {
   $124 = HEAP32[((76440 + 8|0))>>2]|0;
   $125 = (($124) + ($$02))|0;
   HEAP32[((76440 + 8|0))>>2] = $125;
   HEAP32[((76440 + 20|0))>>2] = $$0;
   $126 = $125 | 1;
   $127 = (($$0) + 4|0);
   HEAP32[$127>>2] = $126;
   $128 = (($$0) + ($125)|0);
   HEAP32[$128>>2] = $125;
   STACKTOP = sp;return;
  }
  $129 = $111 & -8;
  $130 = (($129) + ($$02))|0;
  $131 = $111 >>> 3;
  $132 = ($111>>>0)<(256);
  do {
   if ($132) {
    $$sum18 = (($psize) + 8)|0;
    $133 = (($p) + ($$sum18)|0);
    $134 = HEAP32[$133>>2]|0;
    $$sum19 = (($psize) + 12)|0;
    $135 = (($p) + ($$sum19)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = $131 << 1;
    $138 = ((76440 + ($137<<2)|0) + 40|0);
    $139 = ($134|0)==($138|0);
    if (!($139)) {
     $140 = ($134>>>0)<($108>>>0);
     if ($140) {
      _abort();
      // unreachable;
     }
     $141 = (($134) + 12|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = ($142|0)==($0|0);
     if (!($143)) {
      _abort();
      // unreachable;
     }
    }
    $144 = ($136|0)==($134|0);
    if ($144) {
     $145 = 1 << $131;
     $146 = $145 ^ -1;
     $147 = HEAP32[76440>>2]|0;
     $148 = $147 & $146;
     HEAP32[76440>>2] = $148;
     break;
    }
    $149 = ($136|0)==($138|0);
    if ($149) {
     $$pre62 = (($136) + 8|0);
     $$pre$phi63Z2D = $$pre62;
    } else {
     $150 = ($136>>>0)<($108>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($136) + 8|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = ($152|0)==($0|0);
     if ($153) {
      $$pre$phi63Z2D = $151;
     } else {
      _abort();
      // unreachable;
     }
    }
    $154 = (($134) + 12|0);
    HEAP32[$154>>2] = $136;
    HEAP32[$$pre$phi63Z2D>>2] = $134;
   } else {
    $$sum2 = (($psize) + 24)|0;
    $155 = (($p) + ($$sum2)|0);
    $156 = HEAP32[$155>>2]|0;
    $$sum3 = (($psize) + 12)|0;
    $157 = (($p) + ($$sum3)|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = ($158|0)==($0|0);
    do {
     if ($159) {
      $$sum5 = (($psize) + 20)|0;
      $169 = (($p) + ($$sum5)|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==(0|0);
      if ($171) {
       $$sum4 = (($psize) + 16)|0;
       $172 = (($p) + ($$sum4)|0);
       $173 = HEAP32[$172>>2]|0;
       $174 = ($173|0)==(0|0);
       if ($174) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $173;$RP9$0 = $172;
       }
      } else {
       $R7$0 = $170;$RP9$0 = $169;
      }
      while(1) {
       $175 = (($R7$0) + 20|0);
       $176 = HEAP32[$175>>2]|0;
       $177 = ($176|0)==(0|0);
       if (!($177)) {
        $R7$0 = $176;$RP9$0 = $175;
        continue;
       }
       $178 = (($R7$0) + 16|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      }
      $181 = ($RP9$0>>>0)<($108>>>0);
      if ($181) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $$sum17 = (($psize) + 8)|0;
      $160 = (($p) + ($$sum17)|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = ($161>>>0)<($108>>>0);
      if ($162) {
       _abort();
       // unreachable;
      }
      $163 = (($161) + 12|0);
      $164 = HEAP32[$163>>2]|0;
      $165 = ($164|0)==($0|0);
      if (!($165)) {
       _abort();
       // unreachable;
      }
      $166 = (($158) + 8|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = ($167|0)==($0|0);
      if ($168) {
       HEAP32[$163>>2] = $158;
       HEAP32[$166>>2] = $161;
       $R7$1 = $158;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $182 = ($156|0)==(0|0);
    if (!($182)) {
     $$sum14 = (($psize) + 28)|0;
     $183 = (($p) + ($$sum14)|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = ((76440 + ($184<<2)|0) + 304|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = ($0|0)==($186|0);
     if ($187) {
      HEAP32[$185>>2] = $R7$1;
      $cond46 = ($R7$1|0)==(0|0);
      if ($cond46) {
       $188 = 1 << $184;
       $189 = $188 ^ -1;
       $190 = HEAP32[((76440 + 4|0))>>2]|0;
       $191 = $190 & $189;
       HEAP32[((76440 + 4|0))>>2] = $191;
       break;
      }
     } else {
      $192 = HEAP32[((76440 + 16|0))>>2]|0;
      $193 = ($156>>>0)<($192>>>0);
      if ($193) {
       _abort();
       // unreachable;
      }
      $194 = (($156) + 16|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = ($195|0)==($0|0);
      if ($196) {
       HEAP32[$194>>2] = $R7$1;
      } else {
       $197 = (($156) + 20|0);
       HEAP32[$197>>2] = $R7$1;
      }
      $198 = ($R7$1|0)==(0|0);
      if ($198) {
       break;
      }
     }
     $199 = HEAP32[((76440 + 16|0))>>2]|0;
     $200 = ($R7$1>>>0)<($199>>>0);
     if ($200) {
      _abort();
      // unreachable;
     }
     $201 = (($R7$1) + 24|0);
     HEAP32[$201>>2] = $156;
     $$sum15 = (($psize) + 16)|0;
     $202 = (($p) + ($$sum15)|0);
     $203 = HEAP32[$202>>2]|0;
     $204 = ($203|0)==(0|0);
     do {
      if (!($204)) {
       $205 = HEAP32[((76440 + 16|0))>>2]|0;
       $206 = ($203>>>0)<($205>>>0);
       if ($206) {
        _abort();
        // unreachable;
       } else {
        $207 = (($R7$1) + 16|0);
        HEAP32[$207>>2] = $203;
        $208 = (($203) + 24|0);
        HEAP32[$208>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum16 = (($psize) + 20)|0;
     $209 = (($p) + ($$sum16)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     if (!($211)) {
      $212 = HEAP32[((76440 + 16|0))>>2]|0;
      $213 = ($210>>>0)<($212>>>0);
      if ($213) {
       _abort();
       // unreachable;
      } else {
       $214 = (($R7$1) + 20|0);
       HEAP32[$214>>2] = $210;
       $215 = (($210) + 24|0);
       HEAP32[$215>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $216 = $130 | 1;
  $217 = (($$0) + 4|0);
  HEAP32[$217>>2] = $216;
  $218 = (($$0) + ($130)|0);
  HEAP32[$218>>2] = $130;
  $219 = HEAP32[((76440 + 20|0))>>2]|0;
  $220 = ($$0|0)==($219|0);
  if ($220) {
   HEAP32[((76440 + 8|0))>>2] = $130;
   STACKTOP = sp;return;
  } else {
   $$1 = $130;
  }
 } else {
  $221 = $111 & -2;
  HEAP32[$110>>2] = $221;
  $222 = $$02 | 1;
  $223 = (($$0) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($$0) + ($$02)|0);
  HEAP32[$224>>2] = $$02;
  $$1 = $$02;
 }
 $225 = $$1 >>> 3;
 $226 = ($$1>>>0)<(256);
 if ($226) {
  $227 = $225 << 1;
  $228 = ((76440 + ($227<<2)|0) + 40|0);
  $229 = HEAP32[76440>>2]|0;
  $230 = 1 << $225;
  $231 = $229 & $230;
  $232 = ($231|0)==(0);
  if ($232) {
   $233 = $229 | $230;
   HEAP32[76440>>2] = $233;
   $$sum12$pre = (($227) + 2)|0;
   $$pre = ((76440 + ($$sum12$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $228;
  } else {
   $$sum13 = (($227) + 2)|0;
   $234 = ((76440 + ($$sum13<<2)|0) + 40|0);
   $235 = HEAP32[$234>>2]|0;
   $236 = HEAP32[((76440 + 16|0))>>2]|0;
   $237 = ($235>>>0)<($236>>>0);
   if ($237) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $234;$F16$0 = $235;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $$0;
  $238 = (($F16$0) + 12|0);
  HEAP32[$238>>2] = $$0;
  $239 = (($$0) + 8|0);
  HEAP32[$239>>2] = $F16$0;
  $240 = (($$0) + 12|0);
  HEAP32[$240>>2] = $228;
  STACKTOP = sp;return;
 }
 $241 = $$1 >>> 8;
 $242 = ($241|0)==(0);
 if ($242) {
  $I19$0 = 0;
 } else {
  $243 = ($$1>>>0)>(16777215);
  if ($243) {
   $I19$0 = 31;
  } else {
   $244 = (($241) + 1048320)|0;
   $245 = $244 >>> 16;
   $246 = $245 & 8;
   $247 = $241 << $246;
   $248 = (($247) + 520192)|0;
   $249 = $248 >>> 16;
   $250 = $249 & 4;
   $251 = $250 | $246;
   $252 = $247 << $250;
   $253 = (($252) + 245760)|0;
   $254 = $253 >>> 16;
   $255 = $254 & 2;
   $256 = $251 | $255;
   $257 = (14 - ($256))|0;
   $258 = $252 << $255;
   $259 = $258 >>> 15;
   $260 = (($257) + ($259))|0;
   $261 = $260 << 1;
   $262 = (($260) + 7)|0;
   $263 = $$1 >>> $262;
   $264 = $263 & 1;
   $265 = $264 | $261;
   $I19$0 = $265;
  }
 }
 $266 = ((76440 + ($I19$0<<2)|0) + 304|0);
 $267 = (($$0) + 28|0);
 $I19$0$c = $I19$0;
 HEAP32[$267>>2] = $I19$0$c;
 $268 = (($$0) + 20|0);
 HEAP32[$268>>2] = 0;
 $269 = (($$0) + 16|0);
 HEAP32[$269>>2] = 0;
 $270 = HEAP32[((76440 + 4|0))>>2]|0;
 $271 = 1 << $I19$0;
 $272 = $270 & $271;
 $273 = ($272|0)==(0);
 if ($273) {
  $274 = $270 | $271;
  HEAP32[((76440 + 4|0))>>2] = $274;
  HEAP32[$266>>2] = $$0;
  $275 = (($$0) + 24|0);
  HEAP32[$275>>2] = $266;
  $276 = (($$0) + 12|0);
  HEAP32[$276>>2] = $$0;
  $277 = (($$0) + 8|0);
  HEAP32[$277>>2] = $$0;
  STACKTOP = sp;return;
 }
 $278 = HEAP32[$266>>2]|0;
 $279 = ($I19$0|0)==(31);
 if ($279) {
  $287 = 0;
 } else {
  $280 = $I19$0 >>> 1;
  $281 = (25 - ($280))|0;
  $287 = $281;
 }
 $282 = (($278) + 4|0);
 $283 = HEAP32[$282>>2]|0;
 $284 = $283 & -8;
 $285 = ($284|0)==($$1|0);
 L194: do {
  if ($285) {
   $T$0$lcssa = $278;
  } else {
   $286 = $$1 << $287;
   $K20$049 = $286;$T$048 = $278;
   while(1) {
    $294 = $K20$049 >>> 31;
    $295 = ((($T$048) + ($294<<2)|0) + 16|0);
    $290 = HEAP32[$295>>2]|0;
    $296 = ($290|0)==(0|0);
    if ($296) {
     break;
    }
    $288 = $K20$049 << 1;
    $289 = (($290) + 4|0);
    $291 = HEAP32[$289>>2]|0;
    $292 = $291 & -8;
    $293 = ($292|0)==($$1|0);
    if ($293) {
     $T$0$lcssa = $290;
     break L194;
    } else {
     $K20$049 = $288;$T$048 = $290;
    }
   }
   $297 = HEAP32[((76440 + 16|0))>>2]|0;
   $298 = ($295>>>0)<($297>>>0);
   if ($298) {
    _abort();
    // unreachable;
   }
   HEAP32[$295>>2] = $$0;
   $299 = (($$0) + 24|0);
   HEAP32[$299>>2] = $T$048;
   $300 = (($$0) + 12|0);
   HEAP32[$300>>2] = $$0;
   $301 = (($$0) + 8|0);
   HEAP32[$301>>2] = $$0;
   STACKTOP = sp;return;
  }
 } while(0);
 $302 = (($T$0$lcssa) + 8|0);
 $303 = HEAP32[$302>>2]|0;
 $304 = HEAP32[((76440 + 16|0))>>2]|0;
 $305 = ($T$0$lcssa>>>0)<($304>>>0);
 if ($305) {
  _abort();
  // unreachable;
 }
 $306 = ($303>>>0)<($304>>>0);
 if ($306) {
  _abort();
  // unreachable;
 }
 $307 = (($303) + 12|0);
 HEAP32[$307>>2] = $$0;
 HEAP32[$302>>2] = $$0;
 $308 = (($$0) + 8|0);
 HEAP32[$308>>2] = $303;
 $309 = (($$0) + 12|0);
 HEAP32[$309>>2] = $T$0$lcssa;
 $310 = (($$0) + 24|0);
 HEAP32[$310>>2] = 0;
 STACKTOP = sp;return;
}
function _isdigit($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -48)|0;
 $1 = ($0>>>0)<(10);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _isspace($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($c|0)==(32);
 if ($0) {
  $4 = 1;
 } else {
  $1 = (($c) + -9)|0;
  $2 = ($1>>>0)<(5);
  $4 = $2;
 }
 $3 = $4&1;
 STACKTOP = sp;return ($3|0);
}
function _isupper($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -65)|0;
 $1 = ($0>>>0)<(26);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _tolower($c) {
 $c = $c|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_isupper($c)|0);
 $1 = ($0|0)==(0);
 $2 = $c | 32;
 $$0 = $1 ? $c : $2;
 STACKTOP = sp;return ($$0|0);
}
function ___intscan($f,$base,$pok,$0,$1) {
 $f = $f|0;
 $base = $base|0;
 $pok = $pok|0;
 $0 = $0|0;
 $1 = $1|0;
 var $$ = 0, $$1 = 0, $$121 = 0, $$122 = 0, $$18 = 0, $$19 = 0, $$base = 0, $$base16 = 0, $$lcssa = 0, $$sum = 0, $$sum10 = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum1352 = 0, $$sum14 = 0, $$sum15 = 0, $$sum9 = 0, $$sum934 = 0, $10 = 0;
 var $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0;
 var $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0;
 var $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0;
 var $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0;
 var $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0;
 var $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0;
 var $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c$0 = 0, $c$1 = 0, $c$123 = 0, $c$2$be = 0, $c$2$lcssa = 0, $c$3$be = 0, $c$3$lcssa = 0, $c$359 = 0, $c$4$be = 0, $c$4$lcssa = 0, $c$5$be = 0, $c$6$be = 0, $c$6$lcssa = 0, $c$7$be = 0, $c$744 = 0;
 var $c$8 = 0, $c$9$be = 0, $neg$0 = 0, $or$cond = 0, $or$cond17 = 0, $or$cond20 = 0, $or$cond26 = 0, $or$cond2628 = 0, $phitmp93 = 0, $x$068 = 0, $x$135 = 0, $x$253 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $2 = ($base>>>0)>(36);
 if ($2) {
  $5 = (___errno_location()|0);
  HEAP32[$5>>2] = 22;
  $278 = 0;$279 = 0;
  tempRet0 = $278;
  STACKTOP = sp;return ($279|0);
 }
 $3 = (($f) + 4|0);
 $4 = (($f) + 100|0);
 while(1) {
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$4>>2]|0;
  $8 = ($6>>>0)<($7>>>0);
  if ($8) {
   $9 = (($6) + 1|0);
   HEAP32[$3>>2] = $9;
   $10 = HEAP8[$6>>0]|0;
   $11 = $10&255;
   $13 = $11;
  } else {
   $12 = (___shgetc($f)|0);
   $13 = $12;
  }
  $14 = (_isspace($13)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   break;
  }
 }
 $16 = ($13|0)==(45);
 do {
  if ((($13|0) == 43) | (($13|0) == 45)) {
   $17 = $16 << 31 >> 31;
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$4>>2]|0;
   $20 = ($18>>>0)<($19>>>0);
   if ($20) {
    $21 = (($18) + 1|0);
    HEAP32[$3>>2] = $21;
    $22 = HEAP8[$18>>0]|0;
    $23 = $22&255;
    $c$0 = $23;$neg$0 = $17;
    break;
   } else {
    $24 = (___shgetc($f)|0);
    $c$0 = $24;$neg$0 = $17;
    break;
   }
  } else {
   $c$0 = $13;$neg$0 = 0;
  }
 } while(0);
 $25 = ($base|0)==(0);
 $26 = $base & -17;
 $27 = ($26|0)==(0);
 $28 = ($c$0|0)==(48);
 $or$cond = $27 & $28;
 do {
  if ($or$cond) {
   $29 = HEAP32[$3>>2]|0;
   $30 = HEAP32[$4>>2]|0;
   $31 = ($29>>>0)<($30>>>0);
   if ($31) {
    $32 = (($29) + 1|0);
    HEAP32[$3>>2] = $32;
    $33 = HEAP8[$29>>0]|0;
    $34 = $33&255;
    $37 = $34;
   } else {
    $35 = (___shgetc($f)|0);
    $37 = $35;
   }
   $36 = $37 | 32;
   $38 = ($36|0)==(120);
   if (!($38)) {
    $$base = $25 ? 8 : $base;
    $$1 = $$base;$c$1 = $37;
    label = 32;
    break;
   }
   $39 = HEAP32[$3>>2]|0;
   $40 = HEAP32[$4>>2]|0;
   $41 = ($39>>>0)<($40>>>0);
   if ($41) {
    $42 = (($39) + 1|0);
    HEAP32[$3>>2] = $42;
    $43 = HEAP8[$39>>0]|0;
    $44 = $43&255;
    $46 = $44;
   } else {
    $45 = (___shgetc($f)|0);
    $46 = $45;
   }
   $$sum15 = (($46) + 1)|0;
   $47 = (76936 + ($$sum15)|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = ($48&255)>(15);
   if ($49) {
    $50 = HEAP32[$4>>2]|0;
    $51 = ($50|0)==(0|0);
    if (!($51)) {
     $52 = HEAP32[$3>>2]|0;
     $53 = (($52) + -1|0);
     HEAP32[$3>>2] = $53;
    }
    $54 = ($pok|0)==(0);
    if ($54) {
     ___shlim($f,0);
     $278 = 0;$279 = 0;
     tempRet0 = $278;
     STACKTOP = sp;return ($279|0);
    }
    if ($51) {
     $278 = 0;$279 = 0;
     tempRet0 = $278;
     STACKTOP = sp;return ($279|0);
    }
    $55 = HEAP32[$3>>2]|0;
    $56 = (($55) + -1|0);
    HEAP32[$3>>2] = $56;
    $278 = 0;$279 = 0;
    tempRet0 = $278;
    STACKTOP = sp;return ($279|0);
   } else {
    $$122 = 16;$c$123 = $46;
    label = 47;
   }
  } else {
   $$base16 = $25 ? 10 : $base;
   $$sum = (($c$0) + 1)|0;
   $57 = (76936 + ($$sum)|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = ($59>>>0)<($$base16>>>0);
   if ($60) {
    $$1 = $$base16;$c$1 = $c$0;
    label = 32;
   } else {
    $61 = HEAP32[$4>>2]|0;
    $62 = ($61|0)==(0|0);
    if (!($62)) {
     $63 = HEAP32[$3>>2]|0;
     $64 = (($63) + -1|0);
     HEAP32[$3>>2] = $64;
    }
    ___shlim($f,0);
    $65 = (___errno_location()|0);
    HEAP32[$65>>2] = 22;
    $278 = 0;$279 = 0;
    tempRet0 = $278;
    STACKTOP = sp;return ($279|0);
   }
  }
 } while(0);
 if ((label|0) == 32) {
  $66 = ($$1|0)==(10);
  if ($66) {
   $67 = (($c$1) + -48)|0;
   $68 = ($67>>>0)<(10);
   if ($68) {
    $70 = $67;$x$068 = 0;
    while(1) {
     $69 = (($x$068) + ($70))|0;
     $71 = HEAP32[$3>>2]|0;
     $72 = HEAP32[$4>>2]|0;
     $73 = ($71>>>0)<($72>>>0);
     if ($73) {
      $74 = (($71) + 1|0);
      HEAP32[$3>>2] = $74;
      $75 = HEAP8[$71>>0]|0;
      $76 = $75&255;
      $c$2$be = $76;
     } else {
      $77 = (___shgetc($f)|0);
      $c$2$be = $77;
     }
     $78 = (($c$2$be) + -48)|0;
     $79 = ($78>>>0)<(10);
     $80 = ($69>>>0)<(429496729);
     $$ = $79 & $80;
     if (!($$)) {
      break;
     }
     $phitmp93 = ($69*10)|0;
     $70 = $78;$x$068 = $phitmp93;
    }
    $280 = $69;$281 = 0;$c$2$lcssa = $c$2$be;
   } else {
    $280 = 0;$281 = 0;$c$2$lcssa = $c$1;
   }
   $81 = (($c$2$lcssa) + -48)|0;
   $82 = ($81>>>0)<(10);
   if ($82) {
    $83 = $280;$84 = $281;$88 = $81;$c$359 = $c$2$lcssa;
    while(1) {
     $85 = (___muldi3(($83|0),($84|0),10,0)|0);
     $86 = tempRet0;
     $87 = ($88|0)<(0);
     $89 = $87 << 31 >> 31;
     $90 = $88 ^ -1;
     $91 = $89 ^ -1;
     $92 = ($86>>>0)>($91>>>0);
     $93 = ($86|0)==($91|0);
     $94 = ($85>>>0)>($90>>>0);
     $95 = $93 & $94;
     $96 = $92 | $95;
     if ($96) {
      $$lcssa = $88;$282 = $83;$283 = $84;$c$3$lcssa = $c$359;
      break;
     }
     $97 = (_i64Add(($85|0),($86|0),($88|0),($89|0))|0);
     $98 = tempRet0;
     $99 = HEAP32[$3>>2]|0;
     $100 = HEAP32[$4>>2]|0;
     $101 = ($99>>>0)<($100>>>0);
     if ($101) {
      $102 = (($99) + 1|0);
      HEAP32[$3>>2] = $102;
      $103 = HEAP8[$99>>0]|0;
      $104 = $103&255;
      $c$3$be = $104;
     } else {
      $105 = (___shgetc($f)|0);
      $c$3$be = $105;
     }
     $106 = (($c$3$be) + -48)|0;
     $107 = ($106>>>0)<(10);
     $108 = ($98>>>0)<(429496729);
     $109 = ($98|0)==(429496729);
     $110 = ($97>>>0)<(2576980378);
     $111 = $109 & $110;
     $112 = $108 | $111;
     $or$cond17 = $107 & $112;
     if ($or$cond17) {
      $83 = $97;$84 = $98;$88 = $106;$c$359 = $c$3$be;
     } else {
      $$lcssa = $106;$282 = $97;$283 = $98;$c$3$lcssa = $c$3$be;
      break;
     }
    }
    $113 = ($$lcssa>>>0)>(9);
    if ($113) {
     $252 = $283;$255 = $282;
    } else {
     $$121 = 10;$284 = $282;$285 = $283;$c$8 = $c$3$lcssa;
     label = 73;
    }
   } else {
    $252 = $281;$255 = $280;
   }
  } else {
   $$122 = $$1;$c$123 = $c$1;
   label = 47;
  }
 }
 L70: do {
  if ((label|0) == 47) {
   $114 = (($$122) + -1)|0;
   $115 = $114 & $$122;
   $116 = ($115|0)==(0);
   if ($116) {
    $121 = ($$122*23)|0;
    $122 = $121 >>> 5;
    $123 = $122 & 7;
    $124 = (77200 + ($123)|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = $125 << 24 >> 24;
    $$sum934 = (($c$123) + 1)|0;
    $127 = (76936 + ($$sum934)|0);
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = ($129>>>0)<($$122>>>0);
    if ($130) {
     $133 = $129;$x$135 = 0;
     while(1) {
      $131 = $x$135 << $126;
      $132 = $133 | $131;
      $134 = HEAP32[$3>>2]|0;
      $135 = HEAP32[$4>>2]|0;
      $136 = ($134>>>0)<($135>>>0);
      if ($136) {
       $137 = (($134) + 1|0);
       HEAP32[$3>>2] = $137;
       $138 = HEAP8[$134>>0]|0;
       $139 = $138&255;
       $c$4$be = $139;
      } else {
       $140 = (___shgetc($f)|0);
       $c$4$be = $140;
      }
      $$sum9 = (($c$4$be) + 1)|0;
      $141 = (76936 + ($$sum9)|0);
      $142 = HEAP8[$141>>0]|0;
      $143 = $142&255;
      $144 = ($143>>>0)<($$122>>>0);
      $145 = ($132>>>0)<(134217728);
      $$18 = $144 & $145;
      if ($$18) {
       $133 = $143;$x$135 = $132;
      } else {
       break;
      }
     }
     $149 = $142;$152 = 0;$155 = $132;$c$4$lcssa = $c$4$be;
    } else {
     $149 = $128;$152 = 0;$155 = 0;$c$4$lcssa = $c$123;
    }
    $146 = (_bitshift64Lshr(-1,-1,($126|0))|0);
    $147 = tempRet0;
    $148 = $149&255;
    $150 = ($148>>>0)>=($$122>>>0);
    $151 = ($152>>>0)>($147>>>0);
    $153 = ($152|0)==($147|0);
    $154 = ($155>>>0)>($146>>>0);
    $156 = $153 & $154;
    $157 = $151 | $156;
    $or$cond2628 = $150 | $157;
    if ($or$cond2628) {
     $$121 = $$122;$284 = $155;$285 = $152;$c$8 = $c$4$lcssa;
     label = 73;
     break;
    } else {
     $158 = $155;$159 = $152;$163 = $149;
    }
    while(1) {
     $160 = (_bitshift64Shl(($158|0),($159|0),($126|0))|0);
     $161 = tempRet0;
     $162 = $163&255;
     $164 = $162 | $160;
     $165 = HEAP32[$3>>2]|0;
     $166 = HEAP32[$4>>2]|0;
     $167 = ($165>>>0)<($166>>>0);
     if ($167) {
      $168 = (($165) + 1|0);
      HEAP32[$3>>2] = $168;
      $169 = HEAP8[$165>>0]|0;
      $170 = $169&255;
      $c$5$be = $170;
     } else {
      $171 = (___shgetc($f)|0);
      $c$5$be = $171;
     }
     $$sum10 = (($c$5$be) + 1)|0;
     $172 = (76936 + ($$sum10)|0);
     $173 = HEAP8[$172>>0]|0;
     $174 = $173&255;
     $175 = ($174>>>0)>=($$122>>>0);
     $176 = ($161>>>0)>($147>>>0);
     $177 = ($161|0)==($147|0);
     $178 = ($164>>>0)>($146>>>0);
     $179 = $177 & $178;
     $180 = $176 | $179;
     $or$cond26 = $175 | $180;
     if ($or$cond26) {
      $$121 = $$122;$284 = $164;$285 = $161;$c$8 = $c$5$be;
      label = 73;
      break L70;
     } else {
      $158 = $164;$159 = $161;$163 = $173;
     }
    }
   }
   $$sum1352 = (($c$123) + 1)|0;
   $117 = (76936 + ($$sum1352)|0);
   $118 = HEAP8[$117>>0]|0;
   $119 = $118&255;
   $120 = ($119>>>0)<($$122>>>0);
   if ($120) {
    $183 = $119;$x$253 = 0;
    while(1) {
     $181 = Math_imul($x$253, $$122)|0;
     $182 = (($183) + ($181))|0;
     $184 = HEAP32[$3>>2]|0;
     $185 = HEAP32[$4>>2]|0;
     $186 = ($184>>>0)<($185>>>0);
     if ($186) {
      $187 = (($184) + 1|0);
      HEAP32[$3>>2] = $187;
      $188 = HEAP8[$184>>0]|0;
      $189 = $188&255;
      $c$6$be = $189;
     } else {
      $190 = (___shgetc($f)|0);
      $c$6$be = $190;
     }
     $$sum13 = (($c$6$be) + 1)|0;
     $191 = (76936 + ($$sum13)|0);
     $192 = HEAP8[$191>>0]|0;
     $193 = $192&255;
     $194 = ($193>>>0)<($$122>>>0);
     $195 = ($182>>>0)<(119304647);
     $$19 = $194 & $195;
     if ($$19) {
      $183 = $193;$x$253 = $182;
     } else {
      break;
     }
    }
    $197 = $192;$286 = $182;$287 = 0;$c$6$lcssa = $c$6$be;
   } else {
    $197 = $118;$286 = 0;$287 = 0;$c$6$lcssa = $c$123;
   }
   $196 = $197&255;
   $198 = ($196>>>0)<($$122>>>0);
   if ($198) {
    $199 = (___udivdi3(-1,-1,($$122|0),0)|0);
    $200 = tempRet0;
    $202 = $287;$205 = $286;$211 = $197;$c$744 = $c$6$lcssa;
    while(1) {
     $201 = ($202>>>0)>($200>>>0);
     $203 = ($202|0)==($200|0);
     $204 = ($205>>>0)>($199>>>0);
     $206 = $203 & $204;
     $207 = $201 | $206;
     if ($207) {
      $$121 = $$122;$284 = $205;$285 = $202;$c$8 = $c$744;
      label = 73;
      break L70;
     }
     $208 = (___muldi3(($205|0),($202|0),($$122|0),0)|0);
     $209 = tempRet0;
     $210 = $211&255;
     $212 = $210 ^ -1;
     $213 = ($209>>>0)>(4294967295);
     $214 = ($209|0)==(-1);
     $215 = ($208>>>0)>($212>>>0);
     $216 = $214 & $215;
     $217 = $213 | $216;
     if ($217) {
      $$121 = $$122;$284 = $205;$285 = $202;$c$8 = $c$744;
      label = 73;
      break L70;
     }
     $218 = (_i64Add(($210|0),0,($208|0),($209|0))|0);
     $219 = tempRet0;
     $220 = HEAP32[$3>>2]|0;
     $221 = HEAP32[$4>>2]|0;
     $222 = ($220>>>0)<($221>>>0);
     if ($222) {
      $223 = (($220) + 1|0);
      HEAP32[$3>>2] = $223;
      $224 = HEAP8[$220>>0]|0;
      $225 = $224&255;
      $c$7$be = $225;
     } else {
      $226 = (___shgetc($f)|0);
      $c$7$be = $226;
     }
     $$sum14 = (($c$7$be) + 1)|0;
     $227 = (76936 + ($$sum14)|0);
     $228 = HEAP8[$227>>0]|0;
     $229 = $228&255;
     $230 = ($229>>>0)<($$122>>>0);
     if ($230) {
      $202 = $219;$205 = $218;$211 = $228;$c$744 = $c$7$be;
     } else {
      $$121 = $$122;$284 = $218;$285 = $219;$c$8 = $c$7$be;
      label = 73;
      break;
     }
    }
   } else {
    $$121 = $$122;$284 = $286;$285 = $287;$c$8 = $c$6$lcssa;
    label = 73;
   }
  }
 } while(0);
 if ((label|0) == 73) {
  $$sum11 = (($c$8) + 1)|0;
  $231 = (76936 + ($$sum11)|0);
  $232 = HEAP8[$231>>0]|0;
  $233 = $232&255;
  $234 = ($233>>>0)<($$121>>>0);
  if ($234) {
   while(1) {
    $235 = HEAP32[$3>>2]|0;
    $236 = HEAP32[$4>>2]|0;
    $237 = ($235>>>0)<($236>>>0);
    if ($237) {
     $238 = (($235) + 1|0);
     HEAP32[$3>>2] = $238;
     $239 = HEAP8[$235>>0]|0;
     $240 = $239&255;
     $c$9$be = $240;
    } else {
     $241 = (___shgetc($f)|0);
     $c$9$be = $241;
    }
    $$sum12 = (($c$9$be) + 1)|0;
    $242 = (76936 + ($$sum12)|0);
    $243 = HEAP8[$242>>0]|0;
    $244 = $243&255;
    $245 = ($244>>>0)<($$121>>>0);
    if (!($245)) {
     break;
    }
   }
   $246 = (___errno_location()|0);
   HEAP32[$246>>2] = 34;
   $252 = $1;$255 = $0;
  } else {
   $252 = $285;$255 = $284;
  }
 }
 $247 = HEAP32[$4>>2]|0;
 $248 = ($247|0)==(0|0);
 if (!($248)) {
  $249 = HEAP32[$3>>2]|0;
  $250 = (($249) + -1|0);
  HEAP32[$3>>2] = $250;
 }
 $251 = ($252>>>0)<($1>>>0);
 $253 = ($252|0)==($1|0);
 $254 = ($255>>>0)<($0>>>0);
 $256 = $253 & $254;
 $257 = $251 | $256;
 if (!($257)) {
  $258 = $0 & 1;
  $259 = ($258|0)==(0);
  $260 = (0)==(0);
  $261 = $259 & $260;
  $262 = ($neg$0|0)==(0);
  $or$cond20 = $261 & $262;
  if ($or$cond20) {
   $263 = (___errno_location()|0);
   HEAP32[$263>>2] = 34;
   $264 = (_i64Add(($0|0),($1|0),-1,-1)|0);
   $265 = tempRet0;
   $278 = $265;$279 = $264;
   tempRet0 = $278;
   STACKTOP = sp;return ($279|0);
  }
  $266 = ($252>>>0)>($1>>>0);
  $267 = ($252|0)==($1|0);
  $268 = ($255>>>0)>($0>>>0);
  $269 = $267 & $268;
  $270 = $266 | $269;
  if ($270) {
   $271 = (___errno_location()|0);
   HEAP32[$271>>2] = 34;
   $278 = $1;$279 = $0;
   tempRet0 = $278;
   STACKTOP = sp;return ($279|0);
  }
 }
 $272 = ($neg$0|0)<(0);
 $273 = $272 << 31 >> 31;
 $274 = $255 ^ $neg$0;
 $275 = $252 ^ $273;
 $276 = (_i64Subtract(($274|0),($275|0),($neg$0|0),($273|0))|0);
 $277 = tempRet0;
 $278 = $277;$279 = $276;
 tempRet0 = $278;
 STACKTOP = sp;return ($279|0);
}
function ___floatscan($f,$prec,$pok) {
 $f = $f|0;
 $prec = $prec|0;
 $pok = $pok|0;
 var $$$i = 0, $$$i22 = 0, $$0 = 0.0, $$02$i = 0, $$08$i = 0, $$09$i = 0, $$1$be$i = 0, $$1$ph$i = 0, $$10$i = 0, $$14$i = 0, $$2$i = 0, $$3$be$i = 0, $$3$lcssa$i = 0, $$38$i = 0, $$388$i = 0, $$397$i = 0, $$in = 0, $$k$0$i = 0, $$lcssa43$i = 0, $$lnz$0$i = 0;
 var $$neg20$i = 0, $$pn$i = 0.0, $$pre$i = 0.0, $$pre$i17 = 0, $$pre$phi$iZ2D = 0.0, $$promoted$i = 0, $$sink$off0$us$i = 0, $$sink$off0$us53$i = 0, $$x$4$i = 0, $$y$3$i = 0.0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0;
 var $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0;
 var $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0;
 var $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0;
 var $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0;
 var $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0, $187 = 0, $188 = 0.0, $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0;
 var $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0;
 var $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0;
 var $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0;
 var $251 = 0, $252 = 0, $253 = 0, $254 = 0.0, $255 = 0.0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0.0, $266 = 0, $267 = 0, $268 = 0, $269 = 0;
 var $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0.0, $277 = 0.0, $278 = 0.0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0.0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0;
 var $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0.0;
 var $305 = 0.0, $306 = 0.0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0.0, $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0.0, $318 = 0, $319 = 0, $32 = 0, $320 = 0.0, $321 = 0, $322 = 0;
 var $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0;
 var $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0;
 var $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0;
 var $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0;
 var $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0;
 var $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0.0, $418 = 0.0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0;
 var $431 = 0, $432 = 0, $433 = 0, $434 = 0.0, $435 = 0.0, $436 = 0.0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0.0, $447 = 0.0, $448 = 0.0, $449 = 0;
 var $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0.0, $459 = 0.0, $46 = 0, $460 = 0.0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0;
 var $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0.0, $473 = 0, $474 = 0.0, $475 = 0.0, $476 = 0, $477 = 0.0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0, $482 = 0, $483 = 0, $484 = 0.0, $485 = 0.0;
 var $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0.0, $490 = 0, $491 = 0.0, $492 = 0.0, $493 = 0.0, $494 = 0, $495 = 0, $496 = 0, $497 = 0.0, $498 = 0.0, $499 = 0, $5 = 0, $50 = 0.0, $500 = 0, $501 = 0, $502 = 0;
 var $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0.0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0;
 var $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0;
 var $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0;
 var $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0;
 var $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0;
 var $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0;
 var $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0;
 var $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0.0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0;
 var $648 = 0.0, $649 = 0.0, $65 = 0, $650 = 0.0, $651 = 0, $652 = 0.0, $653 = 0.0, $654 = 0.0, $655 = 0.0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0;
 var $666 = 0.0, $667 = 0.0, $668 = 0.0, $669 = 0, $67 = 0, $670 = 0.0, $671 = 0.0, $672 = 0, $673 = 0, $674 = 0, $675 = 0.0, $676 = 0.0, $677 = 0.0, $678 = 0.0, $679 = 0, $68 = 0, $680 = 0, $681 = 0.0, $682 = 0, $683 = 0.0;
 var $684 = 0.0, $685 = 0.0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0.0, $691 = 0, $692 = 0, $693 = 0, $694 = 0.0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0;
 var $701 = 0.0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0.0, $707 = 0, $708 = 0, $709 = 0.0, $71 = 0, $710 = 0.0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0;
 var $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a$0$lcssa161$i = 0, $a$061$i = 0, $a$1$i = 0, $a$2$ph40$i = 0;
 var $a$3$i = 0, $a$3$i$ph = 0, $a$3$ph$i = 0, $a$427$i = 0, $a$5$i = 0, $bias$0$i = 0.0, $bias$07$i = 0.0, $bits$0$ph = 0, $brmerge = 0, $brmerge107 = 0, $c$0 = 0, $c$0$i = 0, $c$1$lcssa = 0, $c$1$ph$i = 0, $c$169 = 0, $c$2 = 0, $c$2$i = 0, $c$2$lcssa$i = 0, $c$364 = 0, $c$4 = 0;
 var $c$5 = 0, $c$6 = 0, $carry$063$i = 0, $carry1$0$us$i = 0, $carry1$0$us49$i = 0, $carry1$1$lcssa$lcssa$i = 0, $carry1$1$us$i = 0, $carry1$1$us54$i = 0, $carry3$030$i = 0, $cond$i = 0, $d$0$i = 0, $denormal$0$i = 0, $denormal$1$i = 0, $denormal$2$i = 0, $e2$0$ph$i = 0, $e2$0$us$i = 0, $e2$0$us44$i = 0, $e2$1$i = 0, $e2$1$i$ph = 0, $e2$1$ph$i = 0;
 var $e2$2$i = 0, $e2$3$i = 0, $emin$0$ph = 0, $exitcond$i = 0, $frac$0$i = 0.0, $frac$1$i = 0.0, $frac$2$i = 0.0, $gotdig$0$i = 0, $gotdig$0$i9 = 0, $gotdig$2$i = 0, $gotdig$2$i11 = 0, $gotdig$3$i = 0, $gotdig$3$lcssa$i = 0, $gotdig$381$i = 0, $gotdig$393$i = 0, $gotdig$4$i = 0, $gotrad$0$i = 0, $gotrad$0$i12 = 0, $gotrad$1$i = 0, $gotrad$1$lcssa$i = 0;
 var $gotrad$194$i = 0, $gotrad$2$i = 0, $gottail$0$i = 0, $gottail$1$i = 0, $gottail$2$i = 0, $i$0$lcssa = 0, $i$025$i = 0, $i$068 = 0, $i$1$i = 0, $i$166 = 0, $i$263 = 0, $i$3 = 0, $i$4 = 0, $j$0$lcssa$i = 0, $j$086$i = 0, $j$096$i = 0, $j$2$i = 0, $j$371$i = 0, $k$0$lcssa$i = 0, $k$084$i = 0;
 var $k$095$i = 0, $k$2$i = 0, $k$3$i = 0, $k$462$i = 0, $k$5$in$us$i = 0, $k$5$in$us48$i = 0, $k$5$us$i = 0, $k$5$us50$i = 0, $k$5$z$2$us$i = 0, $k$5$z$2$us56$i = 0, $k$628$i = 0, $lnz$0$lcssa$i = 0, $lnz$079$i = 0, $lnz$092$i = 0, $lnz$2$i = 0, $notlhs = 0, $notrhs = 0, $or$cond = 0, $or$cond$i = 0, $or$cond$i15 = 0;
 var $or$cond106$not = 0, $or$cond11$i = 0, $or$cond12$us$i = 0, $or$cond12$us55$i = 0, $or$cond13$i = 0, $or$cond15$i = 0, $or$cond16$i = 0, $or$cond17$i = 0, $or$cond3$i = 0, $or$cond3$i16 = 0, $or$cond4$i = 0, $or$cond5 = 0, $or$cond7$i = 0, $or$cond789$i = 0, $or$cond8 = 0, $rp$0$lcssa162$i = 0, $rp$060$i = 0, $rp$1$i18 = 0, $rp$2$ph38$i = 0, $rp$3$i$ph = 0;
 var $rp$3$ph33$i = 0, $rp$426$i = 0, $rp$5$i = 0, $scale$0$i = 0.0, $scale$1$i = 0.0, $scale$2$i = 0.0, $sign$0 = 0, $storemerge$i = 0, $sum$i = 0, $x$0$i = 0, $x$1$i = 0, $x$2$i = 0, $x$3$lcssa$i = 0, $x$313$i = 0, $x$4$lcssa$i = 0, $x$48$i = 0, $x$5$i = 0, $x$6$i = 0, $x$i = 0, $y$0$i = 0.0;
 var $y$1$i = 0.0, $y$1$i23 = 0.0, $y$2$i = 0.0, $y$2$i24 = 0.0, $y$3$i = 0.0, $y$3$lcssa$i = 0.0, $y$39$i = 0.0, $y$4$i = 0.0, $y$5$i = 0.0, $z$0$i = 0, $z$1$ph39$i = 0, $z$1$us$i = 0, $z$1$us45$i = 0, $z$2$us$i = 0, $z$2$us47$i = 0, $z$3$lcssa$lcssa$i = 0, $z$3$us$i = 0, $z$3$us57$i = 0, $z$4$i = 0, $z$5$ph$i = 0;
 var $z$7$1$i = 0, $z$7$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 512|0;
 $x$i = sp;
 if ((($prec|0) == 0)) {
  $bits$0$ph = 24;$emin$0$ph = -149;
 } else if ((($prec|0) == 2)) {
  $bits$0$ph = 53;$emin$0$ph = -1074;
 } else if ((($prec|0) == 1)) {
  $bits$0$ph = 53;$emin$0$ph = -1074;
 } else {
  $$0 = 0.0;
  STACKTOP = sp;return (+$$0);
 }
 $0 = (($f) + 4|0);
 $1 = (($f) + 100|0);
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = ($2>>>0)<($3>>>0);
  if ($4) {
   $5 = (($2) + 1|0);
   HEAP32[$0>>2] = $5;
   $6 = HEAP8[$2>>0]|0;
   $7 = $6&255;
   $9 = $7;
  } else {
   $8 = (___shgetc($f)|0);
   $9 = $8;
  }
  $10 = (_isspace($9)|0);
  $11 = ($10|0)==(0);
  if ($11) {
   break;
  }
 }
 $12 = ($9|0)==(45);
 do {
  if ((($9|0) == 43) | (($9|0) == 45)) {
   $13 = $12&1;
   $14 = $13 << 1;
   $15 = (1 - ($14))|0;
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ($16>>>0)<($17>>>0);
   if ($18) {
    $19 = (($16) + 1|0);
    HEAP32[$0>>2] = $19;
    $20 = HEAP8[$16>>0]|0;
    $21 = $20&255;
    $c$0 = $21;$sign$0 = $15;
    break;
   } else {
    $22 = (___shgetc($f)|0);
    $c$0 = $22;$sign$0 = $15;
    break;
   }
  } else {
   $c$0 = $9;$sign$0 = 1;
  }
 } while(0);
 $c$169 = $c$0;$i$068 = 0;
 while(1) {
  $23 = $c$169 | 32;
  $24 = (77216 + ($i$068)|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($23|0)==($26|0);
  if (!($27)) {
   $c$1$lcssa = $c$169;$i$0$lcssa = $i$068;
   break;
  }
  $28 = ($i$068>>>0)<(7);
  do {
   if ($28) {
    $29 = HEAP32[$0>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = ($29>>>0)<($30>>>0);
    if ($31) {
     $32 = (($29) + 1|0);
     HEAP32[$0>>2] = $32;
     $33 = HEAP8[$29>>0]|0;
     $34 = $33&255;
     $c$2 = $34;
     break;
    } else {
     $35 = (___shgetc($f)|0);
     $c$2 = $35;
     break;
    }
   } else {
    $c$2 = $c$169;
   }
  } while(0);
  $36 = (($i$068) + 1)|0;
  $37 = ($36>>>0)<(8);
  if ($37) {
   $c$169 = $c$2;$i$068 = $36;
  } else {
   $c$1$lcssa = $c$2;$i$0$lcssa = $36;
   break;
  }
 }
 do {
  if ((($i$0$lcssa|0) == 3)) {
   label = 23;
  } else if (!((($i$0$lcssa|0) == 8))) {
   $38 = ($i$0$lcssa>>>0)<(4);
   $39 = ($pok|0)==(0);
   $or$cond = $38 | $39;
   if (!($or$cond)) {
    $40 = ($i$0$lcssa|0)==(8);
    if ($40) {
     break;
    } else {
     label = 23;
     break;
    }
   }
   $52 = ($i$0$lcssa|0)==(0);
   L34: do {
    if ($52) {
     $c$364 = $c$1$lcssa;$i$263 = 0;
     while(1) {
      $53 = $c$364 | 32;
      $54 = (77232 + ($i$263)|0);
      $55 = HEAP8[$54>>0]|0;
      $56 = $55 << 24 >> 24;
      $57 = ($53|0)==($56|0);
      if (!($57)) {
       $c$5 = $c$364;$i$3 = $i$263;
       break L34;
      }
      $58 = ($i$263>>>0)<(2);
      do {
       if ($58) {
        $59 = HEAP32[$0>>2]|0;
        $60 = HEAP32[$1>>2]|0;
        $61 = ($59>>>0)<($60>>>0);
        if ($61) {
         $62 = (($59) + 1|0);
         HEAP32[$0>>2] = $62;
         $63 = HEAP8[$59>>0]|0;
         $64 = $63&255;
         $c$4 = $64;
         break;
        } else {
         $65 = (___shgetc($f)|0);
         $c$4 = $65;
         break;
        }
       } else {
        $c$4 = $c$364;
       }
      } while(0);
      $66 = (($i$263) + 1)|0;
      $67 = ($66>>>0)<(3);
      if ($67) {
       $c$364 = $c$4;$i$263 = $66;
      } else {
       $c$5 = $c$4;$i$3 = $66;
       break;
      }
     }
    } else {
     $c$5 = $c$1$lcssa;$i$3 = $i$0$lcssa;
    }
   } while(0);
   if ((($i$3|0) == 0)) {
    $113 = ($c$5|0)==(48);
    do {
     if ($113) {
      $114 = HEAP32[$0>>2]|0;
      $115 = HEAP32[$1>>2]|0;
      $116 = ($114>>>0)<($115>>>0);
      if ($116) {
       $117 = (($114) + 1|0);
       HEAP32[$0>>2] = $117;
       $118 = HEAP8[$114>>0]|0;
       $119 = $118&255;
       $122 = $119;
      } else {
       $120 = (___shgetc($f)|0);
       $122 = $120;
      }
      $121 = $122 | 32;
      $123 = ($121|0)==(120);
      if (!($123)) {
       $321 = HEAP32[$1>>2]|0;
       $322 = ($321|0)==(0|0);
       if ($322) {
        $c$6 = 48;
        break;
       }
       $323 = HEAP32[$0>>2]|0;
       $324 = (($323) + -1|0);
       HEAP32[$0>>2] = $324;
       $c$6 = 48;
       break;
      }
      $124 = HEAP32[$0>>2]|0;
      $125 = HEAP32[$1>>2]|0;
      $126 = ($124>>>0)<($125>>>0);
      if ($126) {
       $127 = (($124) + 1|0);
       HEAP32[$0>>2] = $127;
       $128 = HEAP8[$124>>0]|0;
       $129 = $128&255;
       $c$0$i = $129;$gotdig$0$i = 0;
      } else {
       $130 = (___shgetc($f)|0);
       $c$0$i = $130;$gotdig$0$i = 0;
      }
      while(1) {
       if ((($c$0$i|0) == 46)) {
        label = 70;
        break;
       } else if (!((($c$0$i|0) == 48))) {
        $169 = 0;$172 = 0;$212 = 0;$214 = 0;$c$2$i = $c$0$i;$gotdig$2$i = $gotdig$0$i;$gotrad$0$i = 0;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
        break;
       }
       $131 = HEAP32[$0>>2]|0;
       $132 = HEAP32[$1>>2]|0;
       $133 = ($131>>>0)<($132>>>0);
       if ($133) {
        $134 = (($131) + 1|0);
        HEAP32[$0>>2] = $134;
        $135 = HEAP8[$131>>0]|0;
        $136 = $135&255;
        $c$0$i = $136;$gotdig$0$i = 1;
        continue;
       } else {
        $137 = (___shgetc($f)|0);
        $c$0$i = $137;$gotdig$0$i = 1;
        continue;
       }
      }
      L66: do {
       if ((label|0) == 70) {
        $138 = HEAP32[$0>>2]|0;
        $139 = HEAP32[$1>>2]|0;
        $140 = ($138>>>0)<($139>>>0);
        if ($140) {
         $141 = (($138) + 1|0);
         HEAP32[$0>>2] = $141;
         $142 = HEAP8[$138>>0]|0;
         $143 = $142&255;
         $c$1$ph$i = $143;
        } else {
         $144 = (___shgetc($f)|0);
         $c$1$ph$i = $144;
        }
        $145 = ($c$1$ph$i|0)==(48);
        if ($145) {
         $155 = -1;$156 = -1;
         while(1) {
          $146 = HEAP32[$0>>2]|0;
          $147 = HEAP32[$1>>2]|0;
          $148 = ($146>>>0)<($147>>>0);
          if ($148) {
           $149 = (($146) + 1|0);
           HEAP32[$0>>2] = $149;
           $150 = HEAP8[$146>>0]|0;
           $151 = $150&255;
           $154 = $151;
          } else {
           $152 = (___shgetc($f)|0);
           $154 = $152;
          }
          $153 = ($154|0)==(48);
          if (!($153)) {
           $169 = 0;$172 = 0;$212 = $155;$214 = $156;$c$2$i = $154;$gotdig$2$i = 1;$gotrad$0$i = 1;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
           break L66;
          }
          $157 = (_i64Add(($155|0),($156|0),-1,-1)|0);
          $158 = tempRet0;
          $155 = $157;$156 = $158;
         }
        } else {
         $169 = 0;$172 = 0;$212 = 0;$214 = 0;$c$2$i = $c$1$ph$i;$gotdig$2$i = $gotdig$0$i;$gotrad$0$i = 1;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
        }
       }
      } while(0);
      L79: while(1) {
       $159 = (($c$2$i) + -48)|0;
       $160 = ($159>>>0)<(10);
       do {
        if ($160) {
         $d$0$i = $159;
         label = 84;
        } else {
         $161 = $c$2$i | 32;
         $162 = (($161) + -97)|0;
         $163 = ($162>>>0)<(6);
         $164 = ($c$2$i|0)==(46);
         $or$cond$i = $163 | $164;
         if (!($or$cond$i)) {
          $c$2$lcssa$i = $c$2$i;
          break L79;
         }
         if ($164) {
          $165 = ($gotrad$0$i|0)==(0);
          if ($165) {
           $712 = $172;$713 = $169;$714 = $172;$715 = $169;$gotdig$3$i = $gotdig$2$i;$gotrad$1$i = 1;$gottail$2$i = $gottail$0$i;$scale$2$i = $scale$0$i;$x$2$i = $x$0$i;$y$2$i = $y$0$i;
           break;
          } else {
           $c$2$lcssa$i = 46;
           break L79;
          }
         } else {
          $166 = ($c$2$i|0)>(57);
          $167 = (($161) + -87)|0;
          $$38$i = $166 ? $167 : $159;
          $d$0$i = $$38$i;
          label = 84;
          break;
         }
        }
       } while(0);
       if ((label|0) == 84) {
        label = 0;
        $168 = ($169|0)<(0);
        $170 = ($169|0)==(0);
        $171 = ($172>>>0)<(8);
        $173 = $170 & $171;
        $174 = $168 | $173;
        do {
         if ($174) {
          $175 = $x$0$i << 4;
          $176 = (($d$0$i) + ($175))|0;
          $gottail$1$i = $gottail$0$i;$scale$1$i = $scale$0$i;$x$1$i = $176;$y$1$i = $y$0$i;
         } else {
          $177 = ($169|0)<(0);
          $178 = ($169|0)==(0);
          $179 = ($172>>>0)<(14);
          $180 = $178 & $179;
          $181 = $177 | $180;
          if ($181) {
           $182 = (+($d$0$i|0));
           $183 = $scale$0$i * 0.0625;
           $184 = $183 * $182;
           $185 = $y$0$i + $184;
           $gottail$1$i = $gottail$0$i;$scale$1$i = $183;$x$1$i = $x$0$i;$y$1$i = $185;
           break;
          }
          $186 = ($d$0$i|0)!=(0);
          $187 = ($gottail$0$i|0)==(0);
          $or$cond3$i = $186 & $187;
          if ($or$cond3$i) {
           $188 = $scale$0$i * 0.5;
           $189 = $y$0$i + $188;
           $gottail$1$i = 1;$scale$1$i = $scale$0$i;$x$1$i = $x$0$i;$y$1$i = $189;
          } else {
           $gottail$1$i = $gottail$0$i;$scale$1$i = $scale$0$i;$x$1$i = $x$0$i;$y$1$i = $y$0$i;
          }
         }
        } while(0);
        $190 = (_i64Add(($172|0),($169|0),1,0)|0);
        $191 = tempRet0;
        $712 = $212;$713 = $214;$714 = $190;$715 = $191;$gotdig$3$i = 1;$gotrad$1$i = $gotrad$0$i;$gottail$2$i = $gottail$1$i;$scale$2$i = $scale$1$i;$x$2$i = $x$1$i;$y$2$i = $y$1$i;
       }
       $192 = HEAP32[$0>>2]|0;
       $193 = HEAP32[$1>>2]|0;
       $194 = ($192>>>0)<($193>>>0);
       if ($194) {
        $195 = (($192) + 1|0);
        HEAP32[$0>>2] = $195;
        $196 = HEAP8[$192>>0]|0;
        $197 = $196&255;
        $169 = $715;$172 = $714;$212 = $712;$214 = $713;$c$2$i = $197;$gotdig$2$i = $gotdig$3$i;$gotrad$0$i = $gotrad$1$i;$gottail$0$i = $gottail$2$i;$scale$0$i = $scale$2$i;$x$0$i = $x$2$i;$y$0$i = $y$2$i;
        continue;
       } else {
        $198 = (___shgetc($f)|0);
        $169 = $715;$172 = $714;$212 = $712;$214 = $713;$c$2$i = $198;$gotdig$2$i = $gotdig$3$i;$gotrad$0$i = $gotrad$1$i;$gottail$0$i = $gottail$2$i;$scale$0$i = $scale$2$i;$x$0$i = $x$2$i;$y$0$i = $y$2$i;
        continue;
       }
      }
      $199 = ($gotdig$2$i|0)==(0);
      if ($199) {
       $200 = HEAP32[$1>>2]|0;
       $201 = ($200|0)==(0|0);
       if (!($201)) {
        $202 = HEAP32[$0>>2]|0;
        $203 = (($202) + -1|0);
        HEAP32[$0>>2] = $203;
       }
       if ($39) {
        ___shlim($f,0);
       } else {
        if (!($201)) {
         $204 = HEAP32[$0>>2]|0;
         $205 = (($204) + -1|0);
         HEAP32[$0>>2] = $205;
         $206 = ($gotrad$0$i|0)==(0);
         if (!($206)) {
          $207 = (($204) + -2|0);
          HEAP32[$0>>2] = $207;
         }
        }
       }
       $208 = (+($sign$0|0));
       $209 = $208 * 0.0;
       $$0 = $209;
       STACKTOP = sp;return (+$$0);
      }
      $210 = ($gotrad$0$i|0)==(0);
      $211 = $210 ? $172 : $212;
      $213 = $210 ? $169 : $214;
      $215 = ($169|0)<(0);
      $216 = ($169|0)==(0);
      $217 = ($172>>>0)<(8);
      $218 = $216 & $217;
      $219 = $215 | $218;
      if ($219) {
       $221 = $172;$222 = $169;$x$313$i = $x$0$i;
       while(1) {
        $220 = $x$313$i << 4;
        $223 = (_i64Add(($221|0),($222|0),1,0)|0);
        $224 = tempRet0;
        $225 = ($224|0)<(0);
        $226 = ($224|0)==(0);
        $227 = ($223>>>0)<(8);
        $228 = $226 & $227;
        $229 = $225 | $228;
        if ($229) {
         $221 = $223;$222 = $224;$x$313$i = $220;
        } else {
         $x$3$lcssa$i = $220;
         break;
        }
       }
      } else {
       $x$3$lcssa$i = $x$0$i;
      }
      $230 = $c$2$lcssa$i | 32;
      $231 = ($230|0)==(112);
      do {
       if ($231) {
        $232 = (_scanexp($f,$pok)|0);
        $233 = tempRet0;
        $234 = ($232|0)==(0);
        $235 = ($233|0)==(-2147483648);
        $236 = $234 & $235;
        if ($236) {
         if ($39) {
          ___shlim($f,0);
          $$0 = 0.0;
          STACKTOP = sp;return (+$$0);
         } else {
          $237 = HEAP32[$1>>2]|0;
          $238 = ($237|0)==(0|0);
          if ($238) {
           $249 = 0;$250 = 0;
           break;
          }
          $239 = HEAP32[$0>>2]|0;
          $240 = (($239) + -1|0);
          HEAP32[$0>>2] = $240;
          $249 = 0;$250 = 0;
          break;
         }
        } else {
         $249 = $232;$250 = $233;
        }
       } else {
        $241 = HEAP32[$1>>2]|0;
        $242 = ($241|0)==(0|0);
        if ($242) {
         $249 = 0;$250 = 0;
        } else {
         $243 = HEAP32[$0>>2]|0;
         $244 = (($243) + -1|0);
         HEAP32[$0>>2] = $244;
         $249 = 0;$250 = 0;
        }
       }
      } while(0);
      $245 = (_bitshift64Shl(($211|0),($213|0),2)|0);
      $246 = tempRet0;
      $247 = (_i64Add(($245|0),($246|0),-32,-1)|0);
      $248 = tempRet0;
      $251 = (_i64Add(($247|0),($248|0),($249|0),($250|0))|0);
      $252 = tempRet0;
      $253 = ($x$3$lcssa$i|0)==(0);
      if ($253) {
       $254 = (+($sign$0|0));
       $255 = $254 * 0.0;
       $$0 = $255;
       STACKTOP = sp;return (+$$0);
      }
      $256 = (0 - ($emin$0$ph))|0;
      $257 = ($252|0)>(0);
      $258 = ($252|0)==(0);
      $259 = ($251>>>0)>($256>>>0);
      $260 = $258 & $259;
      $261 = $257 | $260;
      if ($261) {
       $262 = (___errno_location()|0);
       HEAP32[$262>>2] = 34;
       $263 = (+($sign$0|0));
       $264 = $263 * 1.79769313486231570815E+308;
       $265 = $264 * 1.79769313486231570815E+308;
       $$0 = $265;
       STACKTOP = sp;return (+$$0);
      }
      $266 = (($emin$0$ph) + -106)|0;
      $267 = ($266|0)<(0);
      $268 = $267 << 31 >> 31;
      $269 = ($252|0)<($268|0);
      $270 = ($252|0)==($268|0);
      $271 = ($251>>>0)<($266>>>0);
      $272 = $270 & $271;
      $273 = $269 | $272;
      if ($273) {
       $275 = (___errno_location()|0);
       HEAP32[$275>>2] = 34;
       $276 = (+($sign$0|0));
       $277 = $276 * 2.22507385850720138309E-308;
       $278 = $277 * 2.22507385850720138309E-308;
       $$0 = $278;
       STACKTOP = sp;return (+$$0);
      }
      $274 = ($x$3$lcssa$i|0)>(-1);
      if ($274) {
       $283 = $251;$284 = $252;$x$48$i = $x$3$lcssa$i;$y$39$i = $y$0$i;
       while(1) {
        $279 = !($y$39$i >= 0.5);
        $280 = $x$48$i << 1;
        if ($279) {
         $$pn$i = $y$39$i;$x$5$i = $280;
        } else {
         $281 = $280 | 1;
         $282 = $y$39$i + -1.0;
         $$pn$i = $282;$x$5$i = $281;
        }
        $y$4$i = $y$39$i + $$pn$i;
        $285 = (_i64Add(($283|0),($284|0),-1,-1)|0);
        $286 = tempRet0;
        $287 = ($x$5$i|0)>(-1);
        if ($287) {
         $283 = $285;$284 = $286;$x$48$i = $x$5$i;$y$39$i = $y$4$i;
        } else {
         $292 = $285;$293 = $286;$x$4$lcssa$i = $x$5$i;$y$3$lcssa$i = $y$4$i;
         break;
        }
       }
      } else {
       $292 = $251;$293 = $252;$x$4$lcssa$i = $x$3$lcssa$i;$y$3$lcssa$i = $y$0$i;
      }
      $288 = ($emin$0$ph|0)<(0);
      $289 = $288 << 31 >> 31;
      $290 = (_i64Subtract(32,0,($emin$0$ph|0),($289|0))|0);
      $291 = tempRet0;
      $294 = (_i64Add(($292|0),($293|0),($290|0),($291|0))|0);
      $295 = tempRet0;
      $296 = (0)>($295|0);
      $297 = (0)==($295|0);
      $298 = ($bits$0$ph>>>0)>($294>>>0);
      $299 = $297 & $298;
      $300 = $296 | $299;
      if ($300) {
       $301 = ($294|0)<(0);
       $$$i = $301 ? 0 : $294;
       $$02$i = $$$i;
      } else {
       $$02$i = $bits$0$ph;
      }
      $302 = ($$02$i|0)<(53);
      do {
       if ($302) {
        $303 = (84 - ($$02$i))|0;
        $304 = (+_scalbn(1.0,$303));
        $305 = (+($sign$0|0));
        $306 = (+_copysignl((+$304),(+$305)));
        $307 = ($$02$i|0)<(32);
        $308 = $y$3$lcssa$i != 0.0;
        $or$cond4$i = $307 & $308;
        if (!($or$cond4$i)) {
         $$pre$phi$iZ2D = $305;$bias$07$i = $306;$x$6$i = $x$4$lcssa$i;$y$5$i = $y$3$lcssa$i;
         break;
        }
        $309 = $x$4$lcssa$i & 1;
        $310 = ($309|0)==(0);
        $311 = $309 ^ 1;
        $$x$4$i = (($311) + ($x$4$lcssa$i))|0;
        $$y$3$i = $310 ? 0.0 : $y$3$lcssa$i;
        $$pre$phi$iZ2D = $305;$bias$07$i = $306;$x$6$i = $$x$4$i;$y$5$i = $$y$3$i;
       } else {
        $$pre$i = (+($sign$0|0));
        $$pre$phi$iZ2D = $$pre$i;$bias$07$i = 0.0;$x$6$i = $x$4$lcssa$i;$y$5$i = $y$3$lcssa$i;
       }
      } while(0);
      $312 = (+($x$6$i>>>0));
      $313 = $$pre$phi$iZ2D * $312;
      $314 = $bias$07$i + $313;
      $315 = $$pre$phi$iZ2D * $y$5$i;
      $316 = $315 + $314;
      $317 = $316 - $bias$07$i;
      $318 = $317 != 0.0;
      if (!($318)) {
       $319 = (___errno_location()|0);
       HEAP32[$319>>2] = 34;
      }
      $320 = (+_scalbnl($317,$292));
      $$0 = $320;
      STACKTOP = sp;return (+$$0);
     } else {
      $c$6 = $c$5;
     }
    } while(0);
    $sum$i = (($emin$0$ph) + ($bits$0$ph))|0;
    $325 = (0 - ($sum$i))|0;
    $$08$i = $c$6;$gotdig$0$i9 = 0;
    while(1) {
     if ((($$08$i|0) == 46)) {
      label = 139;
      break;
     } else if (!((($$08$i|0) == 48))) {
      $$2$i = $$08$i;$716 = 0;$717 = 0;$gotdig$2$i11 = $gotdig$0$i9;$gotrad$0$i12 = 0;
      break;
     }
     $326 = HEAP32[$0>>2]|0;
     $327 = HEAP32[$1>>2]|0;
     $328 = ($326>>>0)<($327>>>0);
     if ($328) {
      $329 = (($326) + 1|0);
      HEAP32[$0>>2] = $329;
      $330 = HEAP8[$326>>0]|0;
      $331 = $330&255;
      $$08$i = $331;$gotdig$0$i9 = 1;
      continue;
     } else {
      $332 = (___shgetc($f)|0);
      $$08$i = $332;$gotdig$0$i9 = 1;
      continue;
     }
    }
    L168: do {
     if ((label|0) == 139) {
      $333 = HEAP32[$0>>2]|0;
      $334 = HEAP32[$1>>2]|0;
      $335 = ($333>>>0)<($334>>>0);
      if ($335) {
       $336 = (($333) + 1|0);
       HEAP32[$0>>2] = $336;
       $337 = HEAP8[$333>>0]|0;
       $338 = $337&255;
       $$1$ph$i = $338;
      } else {
       $339 = (___shgetc($f)|0);
       $$1$ph$i = $339;
      }
      $340 = ($$1$ph$i|0)==(48);
      if ($340) {
       $349 = -1;$350 = -1;
       while(1) {
        $341 = HEAP32[$0>>2]|0;
        $342 = HEAP32[$1>>2]|0;
        $343 = ($341>>>0)<($342>>>0);
        if ($343) {
         $344 = (($341) + 1|0);
         HEAP32[$0>>2] = $344;
         $345 = HEAP8[$341>>0]|0;
         $346 = $345&255;
         $$1$be$i = $346;
        } else {
         $347 = (___shgetc($f)|0);
         $$1$be$i = $347;
        }
        $348 = ($$1$be$i|0)==(48);
        if (!($348)) {
         $$2$i = $$1$be$i;$716 = $349;$717 = $350;$gotdig$2$i11 = 1;$gotrad$0$i12 = 1;
         break L168;
        }
        $351 = (_i64Add(($349|0),($350|0),-1,-1)|0);
        $352 = tempRet0;
        $349 = $351;$350 = $352;
       }
      } else {
       $$2$i = $$1$ph$i;$716 = 0;$717 = 0;$gotdig$2$i11 = $gotdig$0$i9;$gotrad$0$i12 = 1;
      }
     }
    } while(0);
    HEAP32[$x$i>>2] = 0;
    $353 = (($$2$i) + -48)|0;
    $354 = ($353>>>0)<(10);
    $355 = ($$2$i|0)==(46);
    $or$cond789$i = $354 | $355;
    L182: do {
     if ($or$cond789$i) {
      $356 = (($x$i) + 496|0);
      $$397$i = $$2$i;$358 = 0;$359 = 0;$718 = $355;$719 = $353;$720 = $716;$721 = $717;$gotdig$393$i = $gotdig$2$i11;$gotrad$194$i = $gotrad$0$i12;$j$096$i = 0;$k$095$i = 0;$lnz$092$i = 0;
      while(1) {
       do {
        if ($718) {
         $cond$i = ($gotrad$194$i|0)==(0);
         if ($cond$i) {
          $722 = $358;$723 = $359;$724 = $358;$725 = $359;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = 1;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
         } else {
          $$388$i = $$397$i;$405 = $720;$406 = $721;$421 = $358;$424 = $359;$gotdig$381$i = $gotdig$393$i;$j$086$i = $j$096$i;$k$084$i = $k$095$i;$lnz$079$i = $lnz$092$i;
          break L182;
         }
        } else {
         $357 = ($k$095$i|0)<(125);
         $360 = (_i64Add(($358|0),($359|0),1,0)|0);
         $361 = tempRet0;
         $362 = ($$397$i|0)!=(48);
         if (!($357)) {
          if (!($362)) {
           $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = $gotrad$194$i;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
           break;
          }
          $372 = HEAP32[$356>>2]|0;
          $373 = $372 | 1;
          HEAP32[$356>>2] = $373;
          $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = $gotrad$194$i;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
          break;
         }
         $$lnz$0$i = $362 ? $360 : $lnz$092$i;
         $363 = ($j$096$i|0)==(0);
         $364 = (($x$i) + ($k$095$i<<2)|0);
         if ($363) {
          $storemerge$i = $719;
         } else {
          $365 = HEAP32[$364>>2]|0;
          $366 = ($365*10)|0;
          $367 = (($$397$i) + -48)|0;
          $368 = (($367) + ($366))|0;
          $storemerge$i = $368;
         }
         HEAP32[$364>>2] = $storemerge$i;
         $369 = (($j$096$i) + 1)|0;
         $370 = ($369|0)==(9);
         $371 = $370&1;
         $$k$0$i = (($371) + ($k$095$i))|0;
         $$10$i = $370 ? 0 : $369;
         $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = 1;$gotrad$2$i = $gotrad$194$i;$j$2$i = $$10$i;$k$2$i = $$k$0$i;$lnz$2$i = $$lnz$0$i;
        }
       } while(0);
       $374 = HEAP32[$0>>2]|0;
       $375 = HEAP32[$1>>2]|0;
       $376 = ($374>>>0)<($375>>>0);
       if ($376) {
        $377 = (($374) + 1|0);
        HEAP32[$0>>2] = $377;
        $378 = HEAP8[$374>>0]|0;
        $379 = $378&255;
        $$3$be$i = $379;
       } else {
        $380 = (___shgetc($f)|0);
        $$3$be$i = $380;
       }
       $381 = (($$3$be$i) + -48)|0;
       $382 = ($381>>>0)<(10);
       $383 = ($$3$be$i|0)==(46);
       $or$cond7$i = $382 | $383;
       if ($or$cond7$i) {
        $$397$i = $$3$be$i;$358 = $724;$359 = $725;$718 = $383;$719 = $381;$720 = $722;$721 = $723;$gotdig$393$i = $gotdig$4$i;$gotrad$194$i = $gotrad$2$i;$j$096$i = $j$2$i;$k$095$i = $k$2$i;$lnz$092$i = $lnz$2$i;
       } else {
        $$3$lcssa$i = $$3$be$i;$386 = $724;$387 = $722;$389 = $725;$390 = $723;$gotdig$3$lcssa$i = $gotdig$4$i;$gotrad$1$lcssa$i = $gotrad$2$i;$j$0$lcssa$i = $j$2$i;$k$0$lcssa$i = $k$2$i;$lnz$0$lcssa$i = $lnz$2$i;
        label = 162;
        break;
       }
      }
     } else {
      $$3$lcssa$i = $$2$i;$386 = 0;$387 = $716;$389 = 0;$390 = $717;$gotdig$3$lcssa$i = $gotdig$2$i11;$gotrad$1$lcssa$i = $gotrad$0$i12;$j$0$lcssa$i = 0;$k$0$lcssa$i = 0;$lnz$0$lcssa$i = 0;
      label = 162;
     }
    } while(0);
    if ((label|0) == 162) {
     $384 = ($gotrad$1$lcssa$i|0)==(0);
     $385 = $384 ? $386 : $387;
     $388 = $384 ? $389 : $390;
     $$388$i = $$3$lcssa$i;$405 = $385;$406 = $388;$421 = $386;$424 = $389;$gotdig$381$i = $gotdig$3$lcssa$i;$j$086$i = $j$0$lcssa$i;$k$084$i = $k$0$lcssa$i;$lnz$079$i = $lnz$0$lcssa$i;
    }
    $391 = ($gotdig$381$i|0)!=(0);
    if ($391) {
     $392 = $$388$i | 32;
     $393 = ($392|0)==(101);
     if ($393) {
      $394 = (_scanexp($f,$pok)|0);
      $395 = tempRet0;
      $396 = ($394|0)==(0);
      $397 = ($395|0)==(-2147483648);
      $398 = $396 & $397;
      do {
       if ($398) {
        if ($39) {
         ___shlim($f,0);
         $$0 = 0.0;
         STACKTOP = sp;return (+$$0);
        } else {
         $399 = HEAP32[$1>>2]|0;
         $400 = ($399|0)==(0|0);
         if ($400) {
          $403 = 0;$404 = 0;
          break;
         }
         $401 = HEAP32[$0>>2]|0;
         $402 = (($401) + -1|0);
         HEAP32[$0>>2] = $402;
         $403 = 0;$404 = 0;
         break;
        }
       } else {
        $403 = $394;$404 = $395;
       }
      } while(0);
      $407 = (_i64Add(($403|0),($404|0),($405|0),($406|0))|0);
      $408 = tempRet0;
      $420 = $407;$423 = $408;
     } else {
      label = 171;
     }
    } else {
     label = 171;
    }
    if ((label|0) == 171) {
     $409 = ($$388$i|0)>(-1);
     if ($409) {
      $410 = HEAP32[$1>>2]|0;
      $411 = ($410|0)==(0|0);
      if ($411) {
       $420 = $405;$423 = $406;
      } else {
       $412 = HEAP32[$0>>2]|0;
       $413 = (($412) + -1|0);
       HEAP32[$0>>2] = $413;
       $420 = $405;$423 = $406;
      }
     } else {
      $420 = $405;$423 = $406;
     }
    }
    if (!($391)) {
     $414 = (___errno_location()|0);
     HEAP32[$414>>2] = 22;
     ___shlim($f,0);
     $$0 = 0.0;
     STACKTOP = sp;return (+$$0);
    }
    $415 = HEAP32[$x$i>>2]|0;
    $416 = ($415|0)==(0);
    if ($416) {
     $417 = (+($sign$0|0));
     $418 = $417 * 0.0;
     $$0 = $418;
     STACKTOP = sp;return (+$$0);
    }
    $419 = ($420|0)==($421|0);
    $422 = ($423|0)==($424|0);
    $425 = $419 & $422;
    $426 = ($424|0)<(0);
    $427 = ($424|0)==(0);
    $428 = ($421>>>0)<(10);
    $429 = $427 & $428;
    $430 = $426 | $429;
    $or$cond$i15 = $425 & $430;
    do {
     if ($or$cond$i15) {
      $431 = ($bits$0$ph>>>0)>(30);
      if (!($431)) {
       $432 = $415 >>> $bits$0$ph;
       $433 = ($432|0)==(0);
       if (!($433)) {
        break;
       }
      }
      $434 = (+($sign$0|0));
      $435 = (+($415>>>0));
      $436 = $434 * $435;
      $$0 = $436;
      STACKTOP = sp;return (+$$0);
     }
    } while(0);
    $437 = (($emin$0$ph|0) / -2)&-1;
    $438 = ($437|0)<(0);
    $439 = $438 << 31 >> 31;
    $440 = ($423|0)>($439|0);
    $441 = ($423|0)==($439|0);
    $442 = ($420>>>0)>($437>>>0);
    $443 = $441 & $442;
    $444 = $440 | $443;
    if ($444) {
     $445 = (___errno_location()|0);
     HEAP32[$445>>2] = 34;
     $446 = (+($sign$0|0));
     $447 = $446 * 1.79769313486231570815E+308;
     $448 = $447 * 1.79769313486231570815E+308;
     $$0 = $448;
     STACKTOP = sp;return (+$$0);
    }
    $449 = (($emin$0$ph) + -106)|0;
    $450 = ($449|0)<(0);
    $451 = $450 << 31 >> 31;
    $452 = ($423|0)<($451|0);
    $453 = ($423|0)==($451|0);
    $454 = ($420>>>0)<($449>>>0);
    $455 = $453 & $454;
    $456 = $452 | $455;
    if ($456) {
     $457 = (___errno_location()|0);
     HEAP32[$457>>2] = 34;
     $458 = (+($sign$0|0));
     $459 = $458 * 2.22507385850720138309E-308;
     $460 = $459 * 2.22507385850720138309E-308;
     $$0 = $460;
     STACKTOP = sp;return (+$$0);
    }
    $461 = ($j$086$i|0)==(0);
    if ($461) {
     $k$3$i = $k$084$i;
    } else {
     $462 = ($j$086$i|0)<(9);
     if ($462) {
      $463 = (($x$i) + ($k$084$i<<2)|0);
      $$promoted$i = HEAP32[$463>>2]|0;
      $465 = $$promoted$i;$j$371$i = $j$086$i;
      while(1) {
       $464 = ($465*10)|0;
       $466 = (($j$371$i) + 1)|0;
       $exitcond$i = ($466|0)==(9);
       if ($exitcond$i) {
        break;
       } else {
        $465 = $464;$j$371$i = $466;
       }
      }
      HEAP32[$463>>2] = $464;
     }
     $467 = (($k$084$i) + 1)|0;
     $k$3$i = $467;
    }
    $468 = ($lnz$079$i|0)<(9);
    do {
     if ($468) {
      $469 = ($lnz$079$i|0)<=($420|0);
      $470 = ($420|0)<(18);
      $or$cond3$i16 = $469 & $470;
      if ($or$cond3$i16) {
       $471 = ($420|0)==(9);
       if ($471) {
        $472 = (+($sign$0|0));
        $473 = HEAP32[$x$i>>2]|0;
        $474 = (+($473>>>0));
        $475 = $472 * $474;
        $$0 = $475;
        STACKTOP = sp;return (+$$0);
       }
       $476 = ($420|0)<(9);
       if ($476) {
        $477 = (+($sign$0|0));
        $478 = HEAP32[$x$i>>2]|0;
        $479 = (+($478>>>0));
        $480 = $477 * $479;
        $481 = (8 - ($420))|0;
        $482 = (77248 + ($481<<2)|0);
        $483 = HEAP32[$482>>2]|0;
        $484 = (+($483|0));
        $485 = $480 / $484;
        $$0 = $485;
        STACKTOP = sp;return (+$$0);
       }
       $486 = Math_imul($420, -3)|0;
       $$neg20$i = (($bits$0$ph) + 27)|0;
       $487 = (($$neg20$i) + ($486))|0;
       $488 = ($487|0)>(30);
       $$pre$i17 = HEAP32[$x$i>>2]|0;
       if (!($488)) {
        $489 = $$pre$i17 >>> $487;
        $490 = ($489|0)==(0);
        if (!($490)) {
         break;
        }
       }
       $491 = (+($sign$0|0));
       $492 = (+($$pre$i17>>>0));
       $493 = $491 * $492;
       $494 = (($420) + -10)|0;
       $495 = (77248 + ($494<<2)|0);
       $496 = HEAP32[$495>>2]|0;
       $497 = (+($496|0));
       $498 = $493 * $497;
       $$0 = $498;
       STACKTOP = sp;return (+$$0);
      }
     }
    } while(0);
    $499 = (($420|0) % 9)&-1;
    $500 = ($499|0)==(0);
    if ($500) {
     $a$2$ph40$i = 0;$e2$0$ph$i = 0;$rp$2$ph38$i = $420;$z$1$ph39$i = $k$3$i;
    } else {
     $501 = ($420|0)>(-1);
     $502 = (($499) + 9)|0;
     $503 = $501 ? $499 : $502;
     $504 = (8 - ($503))|0;
     $505 = (77248 + ($504<<2)|0);
     $506 = HEAP32[$505>>2]|0;
     $507 = ($k$3$i|0)==(0);
     if ($507) {
      $a$0$lcssa161$i = 0;$rp$0$lcssa162$i = $420;$z$0$i = 0;
     } else {
      $508 = (1000000000 / ($506|0))&-1;
      $a$061$i = 0;$carry$063$i = 0;$k$462$i = 0;$rp$060$i = $420;
      while(1) {
       $509 = (($x$i) + ($k$462$i<<2)|0);
       $510 = HEAP32[$509>>2]|0;
       $511 = (($510>>>0) % ($506>>>0))&-1;
       $512 = (($510>>>0) / ($506>>>0))&-1;
       $513 = (($512) + ($carry$063$i))|0;
       HEAP32[$509>>2] = $513;
       $514 = Math_imul($511, $508)|0;
       $515 = ($k$462$i|0)==($a$061$i|0);
       $516 = ($513|0)==(0);
       $or$cond11$i = $515 & $516;
       $517 = (($k$462$i) + 1)|0;
       if ($or$cond11$i) {
        $518 = $517 & 127;
        $519 = (($rp$060$i) + -9)|0;
        $a$1$i = $518;$rp$1$i18 = $519;
       } else {
        $a$1$i = $a$061$i;$rp$1$i18 = $rp$060$i;
       }
       $520 = ($517|0)==($k$3$i|0);
       if ($520) {
        break;
       } else {
        $a$061$i = $a$1$i;$carry$063$i = $514;$k$462$i = $517;$rp$060$i = $rp$1$i18;
       }
      }
      $521 = ($514|0)==(0);
      if ($521) {
       $a$0$lcssa161$i = $a$1$i;$rp$0$lcssa162$i = $rp$1$i18;$z$0$i = $k$3$i;
      } else {
       $522 = (($k$3$i) + 1)|0;
       $523 = (($x$i) + ($k$3$i<<2)|0);
       HEAP32[$523>>2] = $514;
       $a$0$lcssa161$i = $a$1$i;$rp$0$lcssa162$i = $rp$1$i18;$z$0$i = $522;
      }
     }
     $524 = (9 - ($503))|0;
     $525 = (($524) + ($rp$0$lcssa162$i))|0;
     $a$2$ph40$i = $a$0$lcssa161$i;$e2$0$ph$i = 0;$rp$2$ph38$i = $525;$z$1$ph39$i = $z$0$i;
    }
    L280: while(1) {
     $526 = ($rp$2$ph38$i|0)<(18);
     $527 = (($x$i) + ($a$2$ph40$i<<2)|0);
     if ($526) {
      $e2$0$us$i = $e2$0$ph$i;$z$1$us$i = $z$1$ph39$i;
      while(1) {
       $529 = (($z$1$us$i) + 127)|0;
       $carry1$0$us$i = 0;$k$5$in$us$i = $529;$z$2$us$i = $z$1$us$i;
       while(1) {
        $k$5$us$i = $k$5$in$us$i & 127;
        $530 = (($x$i) + ($k$5$us$i<<2)|0);
        $531 = HEAP32[$530>>2]|0;
        $532 = (_bitshift64Shl(($531|0),0,29)|0);
        $533 = tempRet0;
        $534 = (_i64Add(($532|0),($533|0),($carry1$0$us$i|0),0)|0);
        $535 = tempRet0;
        $536 = ($535>>>0)>(0);
        $537 = ($535|0)==(0);
        $538 = ($534>>>0)>(1000000000);
        $539 = $537 & $538;
        $540 = $536 | $539;
        if ($540) {
         $541 = (___udivdi3(($534|0),($535|0),1000000000,0)|0);
         $542 = tempRet0;
         $543 = (___uremdi3(($534|0),($535|0),1000000000,0)|0);
         $544 = tempRet0;
         $$sink$off0$us$i = $543;$carry1$1$us$i = $541;
        } else {
         $$sink$off0$us$i = $534;$carry1$1$us$i = 0;
        }
        HEAP32[$530>>2] = $$sink$off0$us$i;
        $545 = (($z$2$us$i) + 127)|0;
        $546 = $545 & 127;
        $547 = ($k$5$us$i|0)!=($546|0);
        $548 = ($k$5$us$i|0)==($a$2$ph40$i|0);
        $or$cond12$us$i = $547 | $548;
        if ($or$cond12$us$i) {
         $z$3$us$i = $z$2$us$i;
        } else {
         $549 = ($$sink$off0$us$i|0)==(0);
         $k$5$z$2$us$i = $549 ? $k$5$us$i : $z$2$us$i;
         $z$3$us$i = $k$5$z$2$us$i;
        }
        $550 = (($k$5$us$i) + -1)|0;
        if ($548) {
         break;
        } else {
         $carry1$0$us$i = $carry1$1$us$i;$k$5$in$us$i = $550;$z$2$us$i = $z$3$us$i;
        }
       }
       $551 = (($e2$0$us$i) + -29)|0;
       $552 = ($carry1$1$us$i|0)==(0);
       if ($552) {
        $e2$0$us$i = $551;$z$1$us$i = $z$3$us$i;
       } else {
        $$lcssa43$i = $551;$carry1$1$lcssa$lcssa$i = $carry1$1$us$i;$z$3$lcssa$lcssa$i = $z$3$us$i;
        break;
       }
      }
     } else {
      $528 = ($rp$2$ph38$i|0)==(18);
      if ($528) {
       $e2$0$us44$i = $e2$0$ph$i;$z$1$us45$i = $z$1$ph39$i;
      } else {
       $a$3$ph$i = $a$2$ph40$i;$e2$1$ph$i = $e2$0$ph$i;$rp$3$ph33$i = $rp$2$ph38$i;$z$5$ph$i = $z$1$ph39$i;
       break;
      }
      while(1) {
       $553 = HEAP32[$527>>2]|0;
       $554 = ($553>>>0)<(9007199);
       if (!($554)) {
        $a$3$ph$i = $a$2$ph40$i;$e2$1$ph$i = $e2$0$us44$i;$rp$3$ph33$i = 18;$z$5$ph$i = $z$1$us45$i;
        break L280;
       }
       $555 = (($z$1$us45$i) + 127)|0;
       $carry1$0$us49$i = 0;$k$5$in$us48$i = $555;$z$2$us47$i = $z$1$us45$i;
       while(1) {
        $k$5$us50$i = $k$5$in$us48$i & 127;
        $556 = (($x$i) + ($k$5$us50$i<<2)|0);
        $557 = HEAP32[$556>>2]|0;
        $558 = (_bitshift64Shl(($557|0),0,29)|0);
        $559 = tempRet0;
        $560 = (_i64Add(($558|0),($559|0),($carry1$0$us49$i|0),0)|0);
        $561 = tempRet0;
        $562 = ($561>>>0)>(0);
        $563 = ($561|0)==(0);
        $564 = ($560>>>0)>(1000000000);
        $565 = $563 & $564;
        $566 = $562 | $565;
        if ($566) {
         $567 = (___udivdi3(($560|0),($561|0),1000000000,0)|0);
         $568 = tempRet0;
         $569 = (___uremdi3(($560|0),($561|0),1000000000,0)|0);
         $570 = tempRet0;
         $$sink$off0$us53$i = $569;$carry1$1$us54$i = $567;
        } else {
         $$sink$off0$us53$i = $560;$carry1$1$us54$i = 0;
        }
        HEAP32[$556>>2] = $$sink$off0$us53$i;
        $571 = (($z$2$us47$i) + 127)|0;
        $572 = $571 & 127;
        $573 = ($k$5$us50$i|0)!=($572|0);
        $574 = ($k$5$us50$i|0)==($a$2$ph40$i|0);
        $or$cond12$us55$i = $573 | $574;
        if ($or$cond12$us55$i) {
         $z$3$us57$i = $z$2$us47$i;
        } else {
         $575 = ($$sink$off0$us53$i|0)==(0);
         $k$5$z$2$us56$i = $575 ? $k$5$us50$i : $z$2$us47$i;
         $z$3$us57$i = $k$5$z$2$us56$i;
        }
        $576 = (($k$5$us50$i) + -1)|0;
        if ($574) {
         break;
        } else {
         $carry1$0$us49$i = $carry1$1$us54$i;$k$5$in$us48$i = $576;$z$2$us47$i = $z$3$us57$i;
        }
       }
       $577 = (($e2$0$us44$i) + -29)|0;
       $578 = ($carry1$1$us54$i|0)==(0);
       if ($578) {
        $e2$0$us44$i = $577;$z$1$us45$i = $z$3$us57$i;
       } else {
        $$lcssa43$i = $577;$carry1$1$lcssa$lcssa$i = $carry1$1$us54$i;$z$3$lcssa$lcssa$i = $z$3$us57$i;
        break;
       }
      }
     }
     $579 = (($rp$2$ph38$i) + 9)|0;
     $580 = (($a$2$ph40$i) + 127)|0;
     $581 = $580 & 127;
     $582 = ($581|0)==($z$3$lcssa$lcssa$i|0);
     if ($582) {
      $583 = (($z$3$lcssa$lcssa$i) + 127)|0;
      $584 = $583 & 127;
      $585 = (($x$i) + ($584<<2)|0);
      $586 = HEAP32[$585>>2]|0;
      $587 = (($z$3$lcssa$lcssa$i) + 126)|0;
      $588 = $587 & 127;
      $589 = (($x$i) + ($588<<2)|0);
      $590 = HEAP32[$589>>2]|0;
      $591 = $590 | $586;
      HEAP32[$589>>2] = $591;
      $z$4$i = $584;
     } else {
      $z$4$i = $z$3$lcssa$lcssa$i;
     }
     $592 = (($x$i) + ($581<<2)|0);
     HEAP32[$592>>2] = $carry1$1$lcssa$lcssa$i;
     $a$2$ph40$i = $581;$e2$0$ph$i = $$lcssa43$i;$rp$2$ph38$i = $579;$z$1$ph39$i = $z$4$i;
    }
    L311: while(1) {
     $629 = (($z$5$ph$i) + 1)|0;
     $627 = $629 & 127;
     $630 = (($z$5$ph$i) + 127)|0;
     $631 = $630 & 127;
     $632 = (($x$i) + ($631<<2)|0);
     $a$3$i$ph = $a$3$ph$i;$e2$1$i$ph = $e2$1$ph$i;$rp$3$i$ph = $rp$3$ph33$i;
     while(1) {
      $605 = ($rp$3$i$ph|0)==(18);
      $633 = ($rp$3$i$ph|0)>(27);
      $$14$i = $633 ? 9 : 1;
      $a$3$i = $a$3$i$ph;$e2$1$i = $e2$1$i$ph;
      while(1) {
       $i$025$i = 0;
       while(1) {
        $595 = (($i$025$i) + ($a$3$i))|0;
        $596 = $595 & 127;
        $597 = ($596|0)==($z$5$ph$i|0);
        if ($597) {
         $i$1$i = 2;
         break;
        }
        $598 = (($x$i) + ($596<<2)|0);
        $599 = HEAP32[$598>>2]|0;
        $600 = (77240 + ($i$025$i<<2)|0);
        $601 = HEAP32[$600>>2]|0;
        $602 = ($599>>>0)<($601>>>0);
        if ($602) {
         $i$1$i = 2;
         break;
        }
        $603 = ($599>>>0)>($601>>>0);
        $594 = (($i$025$i) + 1)|0;
        if ($603) {
         $i$1$i = $i$025$i;
         break;
        }
        $593 = ($594|0)<(2);
        if ($593) {
         $i$025$i = $594;
        } else {
         $i$1$i = $594;
         break;
        }
       }
       $604 = ($i$1$i|0)==(2);
       $or$cond13$i = $604 & $605;
       if ($or$cond13$i) {
        break L311;
       }
       $608 = (($$14$i) + ($e2$1$i))|0;
       $609 = ($a$3$i|0)==($z$5$ph$i|0);
       if ($609) {
        $a$3$i = $z$5$ph$i;$e2$1$i = $608;
       } else {
        break;
       }
      }
      $610 = 1 << $$14$i;
      $611 = (($610) + -1)|0;
      $612 = 1000000000 >>> $$14$i;
      $a$427$i = $a$3$i;$carry3$030$i = 0;$k$628$i = $a$3$i;$rp$426$i = $rp$3$i$ph;
      while(1) {
       $613 = (($x$i) + ($k$628$i<<2)|0);
       $614 = HEAP32[$613>>2]|0;
       $615 = $614 & $611;
       $616 = $614 >>> $$14$i;
       $617 = (($616) + ($carry3$030$i))|0;
       HEAP32[$613>>2] = $617;
       $618 = Math_imul($615, $612)|0;
       $619 = ($k$628$i|0)==($a$427$i|0);
       $620 = ($617|0)==(0);
       $or$cond15$i = $619 & $620;
       $621 = (($k$628$i) + 1)|0;
       $622 = $621 & 127;
       $623 = (($rp$426$i) + -9)|0;
       $rp$5$i = $or$cond15$i ? $623 : $rp$426$i;
       $a$5$i = $or$cond15$i ? $622 : $a$427$i;
       $624 = ($622|0)==($z$5$ph$i|0);
       if ($624) {
        break;
       } else {
        $a$427$i = $a$5$i;$carry3$030$i = $618;$k$628$i = $622;$rp$426$i = $rp$5$i;
       }
      }
      $625 = ($618|0)==(0);
      if ($625) {
       $a$3$i$ph = $a$5$i;$e2$1$i$ph = $608;$rp$3$i$ph = $rp$5$i;
       continue;
      }
      $626 = ($627|0)==($a$5$i|0);
      if (!($626)) {
       break;
      }
      $634 = HEAP32[$632>>2]|0;
      $635 = $634 | 1;
      HEAP32[$632>>2] = $635;
      $a$3$i$ph = $a$5$i;$e2$1$i$ph = $608;$rp$3$i$ph = $rp$5$i;
     }
     $628 = (($x$i) + ($z$5$ph$i<<2)|0);
     HEAP32[$628>>2] = $618;
     $a$3$ph$i = $a$5$i;$e2$1$ph$i = $608;$rp$3$ph33$i = $rp$5$i;$z$5$ph$i = $627;
    }
    $606 = $a$3$i & 127;
    $607 = ($606|0)==($z$5$ph$i|0);
    if ($607) {
     $636 = (($627) + -1)|0;
     $637 = (($x$i) + ($636<<2)|0);
     HEAP32[$637>>2] = 0;
     $z$7$i = $627;
    } else {
     $z$7$i = $z$5$ph$i;
    }
    $638 = (($x$i) + ($606<<2)|0);
    $639 = HEAP32[$638>>2]|0;
    $640 = (+($639>>>0));
    $641 = (($a$3$i) + 1)|0;
    $642 = $641 & 127;
    $643 = ($642|0)==($z$7$i|0);
    if ($643) {
     $702 = (($z$7$i) + 1)|0;
     $703 = $702 & 127;
     $704 = (($703) + -1)|0;
     $705 = (($x$i) + ($704<<2)|0);
     HEAP32[$705>>2] = 0;
     $z$7$1$i = $703;
    } else {
     $z$7$1$i = $z$7$i;
    }
    $706 = $640 * 1.0E+9;
    $707 = (($x$i) + ($642<<2)|0);
    $708 = HEAP32[$707>>2]|0;
    $709 = (+($708>>>0));
    $710 = $706 + $709;
    $667 = (+($sign$0|0));
    $649 = $667 * $710;
    $687 = (($e2$1$i) + 53)|0;
    $645 = (($687) - ($emin$0$ph))|0;
    $711 = ($645|0)<($bits$0$ph|0);
    if ($711) {
     $644 = ($645|0)<(0);
     $$$i22 = $644 ? 0 : $645;
     $$09$i = $$$i22;$denormal$0$i = 1;
    } else {
     $$09$i = $bits$0$ph;$denormal$0$i = 0;
    }
    $646 = ($$09$i|0)<(53);
    if ($646) {
     $647 = (105 - ($$09$i))|0;
     $648 = (+_scalbn(1.0,$647));
     $650 = (+_copysignl((+$648),(+$649)));
     $651 = (53 - ($$09$i))|0;
     $652 = (+_scalbn(1.0,$651));
     $653 = (+_fmodl((+$649),(+$652)));
     $654 = $649 - $653;
     $655 = $650 + $654;
     $bias$0$i = $650;$frac$0$i = $653;$y$1$i23 = $655;
    } else {
     $bias$0$i = 0.0;$frac$0$i = 0.0;$y$1$i23 = $649;
    }
    $656 = (($a$3$i) + 2)|0;
    $657 = $656 & 127;
    $658 = ($657|0)==($z$7$1$i|0);
    do {
     if ($658) {
      $frac$2$i = $frac$0$i;
     } else {
      $659 = (($x$i) + ($657<<2)|0);
      $660 = HEAP32[$659>>2]|0;
      $661 = ($660>>>0)<(500000000);
      do {
       if ($661) {
        $662 = ($660|0)==(0);
        if ($662) {
         $663 = (($a$3$i) + 3)|0;
         $664 = $663 & 127;
         $665 = ($664|0)==($z$7$1$i|0);
         if ($665) {
          $frac$1$i = $frac$0$i;
          break;
         }
        }
        $666 = $667 * 0.25;
        $668 = $666 + $frac$0$i;
        $frac$1$i = $668;
       } else {
        $669 = ($660>>>0)>(500000000);
        if ($669) {
         $670 = $667 * 0.75;
         $671 = $670 + $frac$0$i;
         $frac$1$i = $671;
         break;
        }
        $672 = (($a$3$i) + 3)|0;
        $673 = $672 & 127;
        $674 = ($673|0)==($z$7$1$i|0);
        if ($674) {
         $675 = $667 * 0.5;
         $676 = $675 + $frac$0$i;
         $frac$1$i = $676;
         break;
        } else {
         $677 = $667 * 0.75;
         $678 = $677 + $frac$0$i;
         $frac$1$i = $678;
         break;
        }
       }
      } while(0);
      $679 = (53 - ($$09$i))|0;
      $680 = ($679|0)>(1);
      if (!($680)) {
       $frac$2$i = $frac$1$i;
       break;
      }
      $681 = (+_fmodl((+$frac$1$i),1.0));
      $682 = $681 != 0.0;
      if ($682) {
       $frac$2$i = $frac$1$i;
       break;
      }
      $683 = $frac$1$i + 1.0;
      $frac$2$i = $683;
     }
    } while(0);
    $684 = $y$1$i23 + $frac$2$i;
    $685 = $684 - $bias$0$i;
    $686 = $687 & 2147483647;
    $688 = (-2 - ($sum$i))|0;
    $689 = ($686|0)>($688|0);
    do {
     if ($689) {
      $690 = (+Math_abs((+$685)));
      $691 = !($690 >= 9007199254740992.0);
      if ($691) {
       $denormal$2$i = $denormal$0$i;$e2$2$i = $e2$1$i;$y$2$i24 = $685;
      } else {
       $692 = ($denormal$0$i|0)!=(0);
       $693 = ($$09$i|0)==($645|0);
       $or$cond16$i = $692 & $693;
       $denormal$1$i = $or$cond16$i ? 0 : $denormal$0$i;
       $694 = $685 * 0.5;
       $695 = (($e2$1$i) + 1)|0;
       $denormal$2$i = $denormal$1$i;$e2$2$i = $695;$y$2$i24 = $694;
      }
      $696 = (($e2$2$i) + 50)|0;
      $697 = ($696|0)>($325|0);
      if (!($697)) {
       $698 = ($denormal$2$i|0)!=(0);
       $699 = $frac$2$i != 0.0;
       $or$cond17$i = $698 & $699;
       if (!($or$cond17$i)) {
        $e2$3$i = $e2$2$i;$y$3$i = $y$2$i24;
        break;
       }
      }
      $700 = (___errno_location()|0);
      HEAP32[$700>>2] = 34;
      $e2$3$i = $e2$2$i;$y$3$i = $y$2$i24;
     } else {
      $e2$3$i = $e2$1$i;$y$3$i = $685;
     }
    } while(0);
    $701 = (+_scalbnl($y$3$i,$e2$3$i));
    $$0 = $701;
    STACKTOP = sp;return (+$$0);
   } else if ((($i$3|0) == 3)) {
    $68 = HEAP32[$0>>2]|0;
    $69 = HEAP32[$1>>2]|0;
    $70 = ($68>>>0)<($69>>>0);
    if ($70) {
     $71 = (($68) + 1|0);
     HEAP32[$0>>2] = $71;
     $72 = HEAP8[$68>>0]|0;
     $73 = $72&255;
     $76 = $73;
    } else {
     $74 = (___shgetc($f)|0);
     $76 = $74;
    }
    $75 = ($76|0)==(40);
    if ($75) {
     $i$4 = 1;
    } else {
     $77 = HEAP32[$1>>2]|0;
     $78 = ($77|0)==(0|0);
     if ($78) {
      $$0 = nan;
      STACKTOP = sp;return (+$$0);
     }
     $79 = HEAP32[$0>>2]|0;
     $80 = (($79) + -1|0);
     HEAP32[$0>>2] = $80;
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    }
    while(1) {
     $81 = HEAP32[$0>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = ($81>>>0)<($82>>>0);
     if ($83) {
      $84 = (($81) + 1|0);
      HEAP32[$0>>2] = $84;
      $85 = HEAP8[$81>>0]|0;
      $86 = $85&255;
      $89 = $86;
     } else {
      $87 = (___shgetc($f)|0);
      $89 = $87;
     }
     $88 = (($89) + -48)|0;
     $90 = ($88>>>0)<(10);
     $91 = (($89) + -65)|0;
     $92 = ($91>>>0)<(26);
     $or$cond8 = $90 | $92;
     if (!($or$cond8)) {
      $93 = (($89) + -97)|0;
      $94 = ($93>>>0)<(26);
      $95 = ($89|0)==(95);
      $or$cond5 = $94 | $95;
      if (!($or$cond5)) {
       break;
      }
     }
     $107 = (($i$4) + 1)|0;
     $i$4 = $107;
    }
    $96 = ($89|0)==(41);
    if ($96) {
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    }
    $97 = HEAP32[$1>>2]|0;
    $98 = ($97|0)==(0|0);
    if (!($98)) {
     $99 = HEAP32[$0>>2]|0;
     $100 = (($99) + -1|0);
     HEAP32[$0>>2] = $100;
    }
    if ($39) {
     $102 = (___errno_location()|0);
     HEAP32[$102>>2] = 22;
     ___shlim($f,0);
     $$0 = 0.0;
     STACKTOP = sp;return (+$$0);
    }
    $101 = ($i$4|0)==(0);
    $brmerge107 = $101 | $98;
    if ($brmerge107) {
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    } else {
     $$in = $i$4;
    }
    while(1) {
     $103 = (($$in) + -1)|0;
     $104 = HEAP32[$0>>2]|0;
     $105 = (($104) + -1|0);
     HEAP32[$0>>2] = $105;
     $106 = ($103|0)==(0);
     if ($106) {
      $$0 = nan;
      break;
     } else {
      $$in = $103;
     }
    }
    STACKTOP = sp;return (+$$0);
   } else {
    $108 = HEAP32[$1>>2]|0;
    $109 = ($108|0)==(0|0);
    if (!($109)) {
     $110 = HEAP32[$0>>2]|0;
     $111 = (($110) + -1|0);
     HEAP32[$0>>2] = $111;
    }
    $112 = (___errno_location()|0);
    HEAP32[$112>>2] = 22;
    ___shlim($f,0);
    $$0 = 0.0;
    STACKTOP = sp;return (+$$0);
   }
  }
 } while(0);
 if ((label|0) == 23) {
  $41 = HEAP32[$1>>2]|0;
  $42 = ($41|0)==(0|0);
  if (!($42)) {
   $43 = HEAP32[$0>>2]|0;
   $44 = (($43) + -1|0);
   HEAP32[$0>>2] = $44;
  }
  $notlhs = ($pok|0)==(0);
  $notrhs = ($i$0$lcssa>>>0)<(4);
  $or$cond106$not = $notrhs | $notlhs;
  $brmerge = $or$cond106$not | $42;
  if (!($brmerge)) {
   $i$166 = $i$0$lcssa;
   while(1) {
    $45 = HEAP32[$0>>2]|0;
    $46 = (($45) + -1|0);
    HEAP32[$0>>2] = $46;
    $47 = (($i$166) + -1)|0;
    $48 = ($47>>>0)>(3);
    if ($48) {
     $i$166 = $47;
    } else {
     break;
    }
   }
  }
 }
 $49 = (+($sign$0|0));
 $50 = $49 * inf;
 $51 = $50;
 $$0 = $51;
 STACKTOP = sp;return (+$$0);
}
function _scanexp($f,$pok) {
 $f = $f|0;
 $pok = $pok|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $c$0 = 0, $c$1$be = 0;
 var $c$18 = 0, $c$2$be = 0, $c$2$lcssa = 0, $c$23 = 0, $c$3$be = 0, $neg$0 = 0, $or$cond = 0, $or$cond2 = 0, $phitmp16 = 0, $x$09 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($f) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($1>>>0)<($3>>>0);
 if ($4) {
  $5 = (($1) + 1|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP8[$1>>0]|0;
  $7 = $6&255;
  $10 = $7;
 } else {
  $8 = (___shgetc($f)|0);
  $10 = $8;
 }
 $9 = ($10|0)==(45);
 if ((($10|0) == 43) | (($10|0) == 45)) {
  $11 = $9&1;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if ($14) {
   $15 = (($12) + 1|0);
   HEAP32[$0>>2] = $15;
   $16 = HEAP8[$12>>0]|0;
   $17 = $16&255;
   $20 = $17;
  } else {
   $18 = (___shgetc($f)|0);
   $20 = $18;
  }
  $19 = (($20) + -48)|0;
  $21 = ($19>>>0)<(10);
  $22 = ($pok|0)==(0);
  $or$cond = $21 | $22;
  if ($or$cond) {
   $c$0 = $20;$neg$0 = $11;
  } else {
   $23 = HEAP32[$2>>2]|0;
   $24 = ($23|0)==(0|0);
   if ($24) {
    $c$0 = $20;$neg$0 = $11;
   } else {
    $25 = HEAP32[$0>>2]|0;
    $26 = (($25) + -1|0);
    HEAP32[$0>>2] = $26;
    $c$0 = $20;$neg$0 = $11;
   }
  }
 } else {
  $c$0 = $10;$neg$0 = 0;
 }
 $27 = (($c$0) + -48)|0;
 $28 = ($27>>>0)>(9);
 if ($28) {
  $29 = HEAP32[$2>>2]|0;
  $30 = ($29|0)==(0|0);
  if ($30) {
   $95 = -2147483648;$96 = 0;
   tempRet0 = $95;
   STACKTOP = sp;return ($96|0);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + -1|0);
  HEAP32[$0>>2] = $32;
  $95 = -2147483648;$96 = 0;
  tempRet0 = $95;
  STACKTOP = sp;return ($96|0);
 } else {
  $c$18 = $c$0;$x$09 = 0;
 }
 while(1) {
  $33 = (($c$18) + -48)|0;
  $34 = (($33) + ($x$09))|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ($35>>>0)<($36>>>0);
  if ($37) {
   $38 = (($35) + 1|0);
   HEAP32[$0>>2] = $38;
   $39 = HEAP8[$35>>0]|0;
   $40 = $39&255;
   $c$1$be = $40;
  } else {
   $41 = (___shgetc($f)|0);
   $c$1$be = $41;
  }
  $42 = (($c$1$be) + -48)|0;
  $43 = ($42>>>0)<(10);
  $44 = ($34|0)<(214748364);
  $$ = $43 & $44;
  if (!($$)) {
   break;
  }
  $phitmp16 = ($34*10)|0;
  $c$18 = $c$1$be;$x$09 = $phitmp16;
 }
 $45 = ($34|0)<(0);
 $46 = $45 << 31 >> 31;
 $47 = (($c$1$be) + -48)|0;
 $48 = ($47>>>0)<(10);
 if ($48) {
  $51 = $34;$52 = $46;$c$23 = $c$1$be;
  while(1) {
   $53 = (___muldi3(($51|0),($52|0),10,0)|0);
   $54 = tempRet0;
   $55 = ($c$23|0)<(0);
   $56 = $55 << 31 >> 31;
   $57 = (_i64Add(($c$23|0),($56|0),-48,-1)|0);
   $58 = tempRet0;
   $59 = (_i64Add(($57|0),($58|0),($53|0),($54|0))|0);
   $60 = tempRet0;
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$2>>2]|0;
   $63 = ($61>>>0)<($62>>>0);
   if ($63) {
    $64 = (($61) + 1|0);
    HEAP32[$0>>2] = $64;
    $65 = HEAP8[$61>>0]|0;
    $66 = $65&255;
    $c$2$be = $66;
   } else {
    $67 = (___shgetc($f)|0);
    $c$2$be = $67;
   }
   $68 = (($c$2$be) + -48)|0;
   $69 = ($68>>>0)<(10);
   $70 = ($60|0)<(21474836);
   $71 = ($60|0)==(21474836);
   $72 = ($59>>>0)<(2061584302);
   $73 = $71 & $72;
   $74 = $70 | $73;
   $or$cond2 = $69 & $74;
   if ($or$cond2) {
    $51 = $59;$52 = $60;$c$23 = $c$2$be;
   } else {
    $89 = $59;$90 = $60;$c$2$lcssa = $c$2$be;
    break;
   }
  }
 } else {
  $89 = $34;$90 = $46;$c$2$lcssa = $c$1$be;
 }
 $49 = (($c$2$lcssa) + -48)|0;
 $50 = ($49>>>0)<(10);
 if ($50) {
  while(1) {
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ($75>>>0)<($76>>>0);
   if ($77) {
    $78 = (($75) + 1|0);
    HEAP32[$0>>2] = $78;
    $79 = HEAP8[$75>>0]|0;
    $80 = $79&255;
    $c$3$be = $80;
   } else {
    $81 = (___shgetc($f)|0);
    $c$3$be = $81;
   }
   $82 = (($c$3$be) + -48)|0;
   $83 = ($82>>>0)<(10);
   if (!($83)) {
    break;
   }
  }
 }
 $84 = HEAP32[$2>>2]|0;
 $85 = ($84|0)==(0|0);
 if (!($85)) {
  $86 = HEAP32[$0>>2]|0;
  $87 = (($86) + -1|0);
  HEAP32[$0>>2] = $87;
 }
 $88 = ($neg$0|0)!=(0);
 $91 = (_i64Subtract(0,0,($89|0),($90|0))|0);
 $92 = tempRet0;
 $93 = $88 ? $91 : $89;
 $94 = $88 ? $92 : $90;
 $95 = $94;$96 = $93;
 tempRet0 = $95;
 STACKTOP = sp;return ($96|0);
}
function ___shlim($f,$lim) {
 $f = $f|0;
 $lim = $lim|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 104|0);
 HEAP32[$0>>2] = $lim;
 $1 = (($f) + 8|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = (($f) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $2;
 $6 = $4;
 $7 = (($5) - ($6))|0;
 $8 = (($f) + 108|0);
 HEAP32[$8>>2] = $7;
 $9 = ($lim|0)!=(0);
 $10 = ($7|0)>($lim|0);
 $or$cond = $9 & $10;
 if ($or$cond) {
  $11 = (($4) + ($lim)|0);
  $12 = (($f) + 100|0);
  HEAP32[$12>>2] = $11;
  STACKTOP = sp;return;
 } else {
  $13 = (($f) + 100|0);
  HEAP32[$13>>2] = $2;
  STACKTOP = sp;return;
 }
}
function ___shgetc($f) {
 $f = $f|0;
 var $$0 = 0, $$phi$trans$insert = 0, $$phi$trans$insert2 = 0, $$pre = 0, $$pre3 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 104|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  label = 3;
 } else {
  $3 = (($f) + 108|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)<($1|0);
  if ($5) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $6 = (___uflow($f)|0);
  $7 = ($6|0)<(0);
  if (!($7)) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)==(0);
   $$phi$trans$insert = (($f) + 8|0);
   $$pre = HEAP32[$$phi$trans$insert>>2]|0;
   if ($10) {
    label = 8;
   } else {
    $11 = (($f) + 4|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = $$pre;
    $14 = $12;
    $15 = (($13) - ($14))|0;
    $16 = (($f) + 108|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($9) - ($17))|0;
    $19 = (($18) + -1)|0;
    $20 = ($15|0)>($19|0);
    if ($20) {
     $21 = (($12) + ($19)|0);
     $22 = (($f) + 100|0);
     HEAP32[$22>>2] = $21;
    } else {
     label = 8;
    }
   }
   if ((label|0) == 8) {
    $23 = (($f) + 100|0);
    HEAP32[$23>>2] = $$pre;
   }
   $24 = ($$pre|0)==(0|0);
   $$phi$trans$insert2 = (($f) + 4|0);
   $$pre3 = HEAP32[$$phi$trans$insert2>>2]|0;
   if (!($24)) {
    $25 = $$pre;
    $26 = $$pre3;
    $27 = (($f) + 108|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (($25) + 1)|0;
    $30 = (($29) - ($26))|0;
    $31 = (($30) + ($28))|0;
    HEAP32[$27>>2] = $31;
   }
   $32 = (($$pre3) + -1|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = ($34|0)==($6|0);
   if ($35) {
    $$0 = $6;
    STACKTOP = sp;return ($$0|0);
   }
   $36 = $6&255;
   HEAP8[$32>>0] = $36;
   $$0 = $6;
   STACKTOP = sp;return ($$0|0);
  }
 }
 $8 = (($f) + 100|0);
 HEAP32[$8>>2] = 0;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function _frexp($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $$0 = 0.0, $$01 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0, $storemerge = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 if ((($4|0) == 0)) {
  $5 = $x != 0.0;
  if ($5) {
   $6 = $x * 18446744073709551616.0;
   $7 = (+_frexp($6,$e));
   $8 = HEAP32[$e>>2]|0;
   $9 = (($8) + -64)|0;
   $$01 = $7;$storemerge = $9;
  } else {
   $$01 = $x;$storemerge = 0;
  }
  HEAP32[$e>>2] = $storemerge;
  $$0 = $$01;
  STACKTOP = sp;return (+$$0);
 } else if ((($4|0) == 2047)) {
  $$0 = $x;
  STACKTOP = sp;return (+$$0);
 } else {
  $10 = (($4) + -1022)|0;
  HEAP32[$e>>2] = $10;
  $11 = $1 & -2146435073;
  $12 = $11 | 1071644672;
  HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $12;$13 = +HEAPF64[tempDoublePtr>>3];
  $$0 = $13;
  STACKTOP = sp;return (+$$0);
 }
 return +0;
}
function _frexpl($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_frexp($x,$e));
 STACKTOP = sp;return (+$0);
}
function _scalbn($x,$n) {
 $x = +$x;
 $n = $n|0;
 var $$ = 0, $$0 = 0, $$1 = 0, $0 = 0, $1 = 0.0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $2 = 0, $3 = 0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0.0, $9 = 0, $y$0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)>(1023);
 if ($0) {
  $1 = $x * 8.98846567431157953864E+307;
  $2 = (($n) + -1023)|0;
  $3 = ($2|0)>(1023);
  if ($3) {
   $4 = $1 * 8.98846567431157953864E+307;
   $5 = (($n) + -2046)|0;
   $6 = ($5|0)>(1023);
   $$ = $6 ? 1023 : $5;
   $$0 = $$;$y$0 = $4;
  } else {
   $$0 = $2;$y$0 = $1;
  }
 } else {
  $7 = ($n|0)<(-1022);
  if ($7) {
   $8 = $x * 2.22507385850720138309E-308;
   $9 = (($n) + 1022)|0;
   $10 = ($9|0)<(-1022);
   if ($10) {
    $11 = $8 * 2.22507385850720138309E-308;
    $12 = (($n) + 2044)|0;
    $13 = ($12|0)<(-1022);
    $$1 = $13 ? -1022 : $12;
    $$0 = $$1;$y$0 = $11;
   } else {
    $$0 = $9;$y$0 = $8;
   }
  } else {
   $$0 = $n;$y$0 = $x;
  }
 }
 $14 = (($$0) + 1023)|0;
 $15 = (_bitshift64Shl(($14|0),0,52)|0);
 $16 = tempRet0;
 HEAP32[tempDoublePtr>>2] = $15;HEAP32[tempDoublePtr+4>>2] = $16;$17 = +HEAPF64[tempDoublePtr>>3];
 $18 = $y$0 * $17;
 STACKTOP = sp;return (+$18);
}
function _scalbnl($x,$n) {
 $x = +$x;
 $n = $n|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_scalbn($x,$n));
 STACKTOP = sp;return (+$0);
}
function _wctomb($s,$wc) {
 $s = $s|0;
 $wc = $wc|0;
 var $$0 = 0, $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 0;
 } else {
  $1 = (_wcrtomb($s,$wc,0)|0);
  $$0 = $1;
 }
 STACKTOP = sp;return ($$0|0);
}
function _wcrtomb($s,$wc,$st) {
 $s = $s|0;
 $wc = $wc|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 1;
  STACKTOP = sp;return ($$0|0);
 }
 $1 = ($wc>>>0)<(128);
 if ($1) {
  $2 = $wc&255;
  HEAP8[$s>>0] = $2;
  $$0 = 1;
  STACKTOP = sp;return ($$0|0);
 }
 $3 = ($wc>>>0)<(2048);
 if ($3) {
  $4 = $wc >>> 6;
  $5 = $4 | 192;
  $6 = $5&255;
  $7 = (($s) + 1|0);
  HEAP8[$s>>0] = $6;
  $8 = $wc & 63;
  $9 = $8 | 128;
  $10 = $9&255;
  HEAP8[$7>>0] = $10;
  $$0 = 2;
  STACKTOP = sp;return ($$0|0);
 }
 $11 = ($wc>>>0)<(55296);
 $12 = (($wc) + -57344)|0;
 $13 = ($12>>>0)<(8192);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = $wc >>> 12;
  $15 = $14 | 224;
  $16 = $15&255;
  $17 = (($s) + 1|0);
  HEAP8[$s>>0] = $16;
  $18 = $wc >>> 6;
  $19 = $18 & 63;
  $20 = $19 | 128;
  $21 = $20&255;
  $22 = (($s) + 2|0);
  HEAP8[$17>>0] = $21;
  $23 = $wc & 63;
  $24 = $23 | 128;
  $25 = $24&255;
  HEAP8[$22>>0] = $25;
  $$0 = 3;
  STACKTOP = sp;return ($$0|0);
 }
 $26 = (($wc) + -65536)|0;
 $27 = ($26>>>0)<(1048576);
 if ($27) {
  $28 = $wc >>> 18;
  $29 = $28 | 240;
  $30 = $29&255;
  $31 = (($s) + 1|0);
  HEAP8[$s>>0] = $30;
  $32 = $wc >>> 12;
  $33 = $32 & 63;
  $34 = $33 | 128;
  $35 = $34&255;
  $36 = (($s) + 2|0);
  HEAP8[$31>>0] = $35;
  $37 = $wc >>> 6;
  $38 = $37 & 63;
  $39 = $38 | 128;
  $40 = $39&255;
  $41 = (($s) + 3|0);
  HEAP8[$36>>0] = $40;
  $42 = $wc & 63;
  $43 = $42 | 128;
  $44 = $43&255;
  HEAP8[$41>>0] = $44;
  $$0 = 4;
  STACKTOP = sp;return ($$0|0);
 } else {
  $45 = (___errno_location()|0);
  HEAP32[$45>>2] = 84;
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function ___toread($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = (($f) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($f) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($7>>>0)>($9>>>0);
 if ($10) {
  $11 = (($f) + 36|0);
  $12 = HEAP32[$11>>2]|0;
  (FUNCTION_TABLE_iiii[$12 & 255]($f,0,0)|0);
 }
 $13 = (($f) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = (($f) + 28|0);
 HEAP32[$14>>2] = 0;
 HEAP32[$6>>2] = 0;
 $15 = HEAP32[$f>>2]|0;
 $16 = $15 & 20;
 $17 = ($16|0)==(0);
 if ($17) {
  $21 = HEAP32[$8>>2]|0;
  $22 = (($f) + 8|0);
  HEAP32[$22>>2] = $21;
  $23 = (($f) + 4|0);
  HEAP32[$23>>2] = $21;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 $18 = $15 & 4;
 $19 = ($18|0)==(0);
 if ($19) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $20 = $15 | 32;
 HEAP32[$f>>2] = $20;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function ___towrite($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = HEAP32[$f>>2]|0;
 $7 = $6 & 8;
 $8 = ($7|0)==(0);
 if ($8) {
  $10 = (($f) + 8|0);
  HEAP32[$10>>2] = 0;
  $11 = (($f) + 4|0);
  HEAP32[$11>>2] = 0;
  $12 = (($f) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($f) + 28|0);
  HEAP32[$14>>2] = $13;
  $15 = (($f) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = (($f) + 48|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($13) + ($17)|0);
  $19 = (($f) + 16|0);
  HEAP32[$19>>2] = $18;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 } else {
  $9 = $6 | 32;
  HEAP32[$f>>2] = $9;
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function ___uflow($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $c = sp;
 $0 = (($f) + 8|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  $3 = (___toread($f)|0);
  $4 = ($3|0)==(0);
  if ($4) {
   label = 3;
  } else {
   $$0 = -1;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $5 = (($f) + 32|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = (FUNCTION_TABLE_iiii[$6 & 255]($f,$c,1)|0);
  $8 = ($7|0)==(1);
  if ($8) {
   $9 = HEAP8[$c>>0]|0;
   $10 = $9&255;
   $$0 = $10;
  } else {
   $$0 = -1;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___fwritex($s,$l,$f) {
 $s = $s|0;
 $l = $l|0;
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$pre = 0, $$pre6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$0 = 0, $i$1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 do {
  if ($2) {
   $3 = (___towrite($f)|0);
   $4 = ($3|0)==(0);
   if ($4) {
    $$pre6 = HEAP32[$0>>2]|0;
    $8 = $$pre6;
    break;
   } else {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  } else {
   $8 = $1;
  }
 } while(0);
 $5 = (($f) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $8;
 $9 = $6;
 $10 = (($7) - ($9))|0;
 $11 = ($10>>>0)<($l>>>0);
 if ($11) {
  $12 = (($f) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (FUNCTION_TABLE_iiii[$13 & 255]($f,$s,$l)|0);
  $$0 = $14;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = (($f) + 75|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)>(-1);
 L11: do {
  if ($17) {
   $i$0 = $l;
   while(1) {
    $18 = ($i$0|0)==(0);
    if ($18) {
     $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
     break L11;
    }
    $19 = (($i$0) + -1)|0;
    $20 = (($s) + ($19)|0);
    $21 = HEAP8[$20>>0]|0;
    $22 = ($21<<24>>24)==(10);
    if ($22) {
     break;
    } else {
     $i$0 = $19;
    }
   }
   $23 = (($f) + 36|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (FUNCTION_TABLE_iiii[$24 & 255]($f,$s,$i$0)|0);
   $26 = ($25>>>0)<($i$0>>>0);
   if ($26) {
    $$0 = $i$0;
    STACKTOP = sp;return ($$0|0);
   } else {
    $27 = (($s) + ($i$0)|0);
    $28 = (($l) - ($i$0))|0;
    $$pre = HEAP32[$5>>2]|0;
    $$01 = $28;$$02 = $27;$29 = $$pre;$i$1 = $i$0;
    break;
   }
  } else {
   $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
  }
 } while(0);
 _memcpy(($29|0),($$02|0),($$01|0))|0;
 $30 = HEAP32[$5>>2]|0;
 $31 = (($30) + ($$01)|0);
 HEAP32[$5>>2] = $31;
 $32 = (($i$1) + ($$01))|0;
 $$0 = $32;
 STACKTOP = sp;return ($$0|0);
}
function _snprintf($s,$n,$fmt,$varargs) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsnprintf($s,$n,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _sprintf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsprintf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfprintf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap2 = 0, $internal_buf = 0, $nl_arg = 0, $nl_type = 0, $ret$1 = 0, $vacopy_currentptr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $ap2 = sp + 120|0;
 $nl_type = sp + 80|0;
 $nl_arg = sp;
 $internal_buf = sp + 136|0;
 dest=$nl_type+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $vacopy_currentptr = HEAP32[$ap>>2]|0;
 HEAP32[$ap2>>2] = $vacopy_currentptr;
 $0 = (_printf_core(0,$fmt,$ap2,$nl_arg,$nl_type)|0);
 $1 = ($0|0)<(0);
 if ($1) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $2 = (($f) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $6 = (($f) + 44|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$6>>2] = $internal_buf;
  $8 = (($f) + 28|0);
  HEAP32[$8>>2] = $internal_buf;
  $9 = (($f) + 20|0);
  HEAP32[$9>>2] = $internal_buf;
  HEAP32[$2>>2] = 80;
  $10 = (($internal_buf) + 80|0);
  $11 = (($f) + 16|0);
  HEAP32[$11>>2] = $10;
  $12 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $13 = ($7|0)==(0|0);
  if ($13) {
   $ret$1 = $12;
  } else {
   $14 = (($f) + 36|0);
   $15 = HEAP32[$14>>2]|0;
   (FUNCTION_TABLE_iiii[$15 & 255]($f,0,0)|0);
   $16 = HEAP32[$9>>2]|0;
   $17 = ($16|0)==(0|0);
   $$ = $17 ? -1 : $12;
   HEAP32[$6>>2] = $7;
   HEAP32[$2>>2] = 0;
   HEAP32[$11>>2] = 0;
   HEAP32[$8>>2] = 0;
   HEAP32[$9>>2] = 0;
   $ret$1 = $$;
  }
 } else {
  $5 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $ret$1 = $5;
 }
 $$0 = $ret$1;
 STACKTOP = sp;return ($$0|0);
}
function _printf_core($f,$fmt,$ap,$nl_arg,$nl_type) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 $nl_arg = $nl_arg|0;
 $nl_type = $nl_type|0;
 var $$ = 0, $$$5$i = 0, $$$i = 0, $$$p$i = 0, $$0 = 0, $$0$lcssa$i = 0, $$0$lcssa$i$i = 0, $$0$lcssa$i104$i = 0, $$0$lcssa$i128$i = 0, $$0$lcssa$i143$i = 0, $$0$lcssa$i38 = 0, $$0$lcssa$i39$i = 0, $$0$lcssa$i44 = 0, $$0$lcssa$i46 = 0, $$0$lcssa$i46$i = 0, $$0$lcssa$i49$i = 0, $$0$lcssa$i53 = 0, $$0$lcssa$i56$i = 0, $$0$lcssa$i60 = 0, $$0$lcssa$i63$i = 0;
 var $$0$lcssa$i67 = 0, $$0$lcssa$i69$i = 0, $$0$lcssa$i76$i = 0, $$0$lcssa$i78 = 0, $$0$lcssa$i85$i = 0, $$0$lcssa$i97$i = 0, $$01$i = 0, $$01$i$i = 0, $$01$i102$i = 0, $$01$i126$i = 0, $$01$i141$i = 0, $$01$i36 = 0, $$01$i37$i = 0, $$01$i44$i = 0, $$01$i51 = 0, $$01$i54$i = 0, $$01$i58 = 0, $$01$i61$i = 0, $$01$i65 = 0, $$01$i67$i = 0;
 var $$01$i74$i = 0, $$01$i76 = 0, $$01$i95$i = 0, $$01$lcssa$off0$i = 0, $$01$lcssa$off0$i$i = 0, $$01$lcssa$off0$i86$i = 0, $$012$i = 0, $$012$i73 = 0, $$03$i41 = 0, $$05$i = 0, $$05$i$i = 0, $$05$i80$i = 0, $$06$i = 0.0, $$1$i = 0.0, $$1$lcssa$i$i = 0, $$1$lcssa$i113$i = 0, $$113$i = 0, $$12$i = 0, $$12$i$i = 0, $$12$i111$i = 0;
 var $$12$i120$i = 0, $$12$i135$i = 0, $$12$i88$i = 0, $$14 = 0, $$15 = 0, $$16 = 0, $$18 = 0, $$2$i = 0.0, $$2$us$i = 0.0, $$2$us$us$i = 0.0, $$2$us160$i = 0.0, $$20$i = 0, $$20$us$i = 0, $$21$i = 0, $$22$i = 0.0, $$24$i = 0, $$26$i = 0, $$29$$24$i = 0, $$29$$26$i = 0, $$29$i = 0;
 var $$3$i = 0.0, $$310$i = 0, $$32$i = 0, $$4$i = 0.0, $$411$lcssa$i = 0, $$411176$i = 0, $$5194$i = 0, $$a$3$i = 0, $$a$3$us$i = 0, $$a$3$us319$i = 0, $$a$3$us320$i = 0, $$a$3321$i = 0, $$a$3322$i = 0, $$fl$4 = 0, $$lcssa94 = 0, $$mask$i = 0, $$mask$i31 = 0, $$mask1$i = 0, $$mask1$i30 = 0, $$neg152$i = 0;
 var $$neg153$i = 0, $$not$i = 0, $$p$5 = 0, $$p$i = 0, $$pn$i = 0, $$pr$i = 0, $$pr147$i = 0, $$pre = 0, $$pre$i = 0, $$pre291 = 0, $$pre292 = 0, $$pre312$i = 0, $$sink = 0, $$sink300$off32 = 0, $$sum$i = 0, $$sum14$i = 0, $$sum15$i = 0, $$z$3$i = 0, $$z$4$us$i = 0, $0 = 0;
 var $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0;
 var $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0;
 var $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0.0, $104 = 0, $1040 = 0.0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1048$phi = 0, $1049 = 0, $1049$phi = 0, $105 = 0;
 var $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0.0, $369 = 0, $37 = 0, $370 = 0.0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0.0, $422 = 0, $423 = 0, $424 = 0, $425 = 0.0, $426 = 0.0, $427 = 0.0, $428 = 0.0, $429 = 0.0, $43 = 0, $430 = 0.0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0.0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0.0, $495 = 0.0, $496 = 0.0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0.0, $51 = 0, $510 = 0.0;
 var $511 = 0.0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0.0, $525 = 0.0, $526 = 0.0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0.0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0.0, $585 = 0.0, $586 = 0.0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0;
 var $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0;
 var $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0;
 var $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0.0;
 var $728 = 0.0, $729 = 0, $73 = 0, $730 = 0.0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0;
 var $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0;
 var $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0;
 var $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0;
 var $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0;
 var $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0;
 var $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0;
 var $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0;
 var $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0;
 var $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0;
 var $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0;
 var $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0;
 var $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0;
 var $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0;
 var $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0;
 var $999 = 0, $a$0 = 0, $a$1 = 0, $a$1$lcssa$i = 0, $a$1257$i = 0, $a$2 = 0, $a$2$i = 0, $a$3$lcssa$i = 0, $a$3245$i = 0, $a$3245$us$i = 0, $a$6$i = 0, $a$7$i = 0, $arglist_current = 0, $arglist_current11 = 0, $arglist_current14 = 0, $arglist_current17 = 0, $arglist_current2 = 0, $arglist_current20 = 0, $arglist_current23 = 0, $arglist_current26 = 0;
 var $arglist_current29 = 0, $arglist_current32 = 0, $arglist_current35 = 0, $arglist_current38 = 0, $arglist_current41 = 0, $arglist_current44 = 0, $arglist_current47 = 0, $arglist_current5 = 0, $arglist_current50 = 0, $arglist_current53 = 0, $arglist_current56 = 0, $arglist_current59 = 0, $arglist_current62 = 0, $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next15 = 0, $arglist_next18 = 0, $arglist_next21 = 0, $arglist_next24 = 0;
 var $arglist_next27 = 0, $arglist_next3 = 0, $arglist_next30 = 0, $arglist_next33 = 0, $arglist_next36 = 0, $arglist_next39 = 0, $arglist_next42 = 0, $arglist_next45 = 0, $arglist_next48 = 0, $arglist_next51 = 0, $arglist_next54 = 0, $arglist_next57 = 0, $arglist_next6 = 0, $arglist_next60 = 0, $arglist_next63 = 0, $arglist_next9 = 0, $argpos$0 = 0, $big$i = 0, $brmerge$i = 0, $buf = 0;
 var $buf$i = 0, $carry$0250$i = 0, $carry3$0239$i = 0, $carry3$0239$us$i = 0, $cnt$0 = 0, $cnt$1 = 0, $d$0251$i = 0, $d$1238$i = 0, $d$1238$us$i = 0, $d$2$a$3$i = 0, $d$2$lcssa$i = 0, $d$2220$i = 0, $d$3$i = 0, $d$4184$i = 0, $d$5175$i = 0, $d$6193$i = 0, $e$0233$i = 0, $e$1$i = 0, $e$2215$i = 0, $e$3$i = 0;
 var $e$4$i = 0, $e2$i = 0, $ebuf0$i = 0, $estr$0$i = 0, $estr$1$lcssa$i = 0, $estr$1$ph$i = 0, $estr$1201$i = 0, $estr$2$i = 0, $exitcond$i = 0, $fl$0102 = 0, $fl$0106 = 0, $fl$1 = 0, $fl$1$ = 0, $fl$3 = 0, $fl$4 = 0, $fl$6 = 0, $i$0$lcssa = 0, $i$0167 = 0, $i$0169 = 0, $i$0234$i = 0;
 var $i$03$i = 0, $i$03$i23 = 0, $i$1$lcssa$i = 0, $i$1175 = 0, $i$1226$i = 0, $i$2216$i = 0, $i$291 = 0, $i$3210$i = 0, $i$390 = 0, $isdigit = 0, $isdigit$i = 0, $isdigit$i25 = 0, $isdigit10 = 0, $isdigit12 = 0, $isdigit2$i = 0, $isdigit2$i22 = 0, $isdigittmp = 0, $isdigittmp$i = 0, $isdigittmp$i24 = 0, $isdigittmp1$i = 0;
 var $isdigittmp1$i21 = 0, $isdigittmp11 = 0, $isdigittmp9 = 0, $j$0$i = 0, $j$0225$i = 0, $j$0227$i = 0, $j$1211$i = 0, $j$2$i = 0, $l$0 = 0, $l$0$i = 0, $l$1$i = 0, $l$1$lcssa = 0, $l$1168 = 0, $l10n$0 = 0, $l10n$0$phi = 0, $l10n$1 = 0, $l10n$2 = 0, $l10n$3 = 0, $mb = 0, $or$cond = 0;
 var $or$cond$i = 0, $or$cond$i$i = 0, $or$cond$i101$i = 0, $or$cond$i36$i = 0, $or$cond$i43$i = 0, $or$cond$i50 = 0, $or$cond$i53$i = 0, $or$cond$i57 = 0, $or$cond$i60$i = 0, $or$cond$i64 = 0, $or$cond$i72 = 0, $or$cond$i73$i = 0, $or$cond$i75 = 0, $or$cond$i94$i = 0, $or$cond19$i = 0, $or$cond19315$i = 0, $or$cond23$i = 0, $or$cond29$i = 0, $or$cond29174$i = 0, $or$cond3$i = 0;
 var $or$cond30$i = 0, $p$0 = 0, $p$1 = 0, $p$2 = 0, $p$2$ = 0, $p$4296 = 0, $p$5 = 0, $pad$i = 0, $pl$0 = 0, $pl$0$i = 0, $pl$1 = 0, $pl$1$i = 0, $pl$2 = 0, $prefix$0 = 0, $prefix$0$$i = 0, $prefix$0$i = 0, $prefix$1 = 0, $prefix$2 = 0, $r$0$a$7$i = 0, $re$0$i = 0;
 var $re$1166$i = 0, $round$0165$i = 0.0, $round6$1$i = 0.0, $s$0$i = 0, $s$0$us$i = 0, $s$0$us$us$i = 0, $s$0$us159$i = 0, $s$1$i = 0, $s$1$lcssa$i = 0, $s$1$us$i = 0, $s$1$us$us$i = 0, $s$1$us161$i = 0, $s1$0$i = 0, $s7$0181$i = 0, $s7$1$i = 0, $s8$0$lcssa$i = 0, $s8$0170$i = 0, $s9$0$i = 0, $s9$1189$i = 0, $s9$2$i = 0;
 var $sext = 0, $sext86 = 0, $small$0$i = 0.0, $small$1$i = 0.0, $st$0 = 0, $storemerge = 0, $storemerge13 = 0, $storemerge8100 = 0, $storemerge8105 = 0, $t$0 = 0, $t$1 = 0, $w$$i = 0, $w$0 = 0, $w$1 = 0, $w$18$i = 0, $w$2 = 0, $w$31$i = 0, $wc = 0, $ws$0170 = 0, $ws$1176 = 0;
 var $y$03$i = 0, $y$03$i$i = 0, $y$03$i110$i = 0, $y$03$i119$i = 0, $y$03$i134$i = 0, $y$03$i87$i = 0, $z$0$i = 0, $z$0$lcssa = 0, $z$095 = 0, $z$1$lcssa$i = 0, $z$1256$i = 0, $z$2 = 0, $z$2$i = 0, $z$2316$i = 0, $z$2317$i = 0, $z$3$lcssa$i = 0, $z$3244$i = 0, $z$3244$us$i = 0, $z$4$i = 0, $z$4$us$i = 0;
 var $z$5$i = 0, $z$6$i = 0, $z$7$$i = 0, $z$7$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 848|0;
 $big$i = sp + 16|0;
 $e2$i = sp + 8|0;
 $buf$i = sp + 812|0;
 $0 = $buf$i;
 $ebuf0$i = sp + 800|0;
 $pad$i = sp + 544|0;
 $buf = sp + 504|0;
 $wc = sp;
 $mb = sp + 496|0;
 $1 = ($f|0)!=(0|0);
 $2 = (($buf) + 40|0);
 $3 = $2;
 $4 = (($buf) + 39|0);
 $5 = (($wc) + 4|0);
 $6 = (($ebuf0$i) + 12|0);
 $7 = (($ebuf0$i) + 11|0);
 $8 = $6;
 $9 = (($8) - ($0))|0;
 $10 = (-2 - ($0))|0;
 $11 = (($8) + 2)|0;
 $12 = (($big$i) + 264|0);
 $13 = (($buf$i) + 9|0);
 $14 = $13;
 $15 = (($buf$i) + 8|0);
 $1048 = 0;$1049 = 0;$22 = $fmt;$cnt$0 = 0;$l$0 = 0;$l10n$0 = 0;
 L1: while(1) {
  $16 = ($cnt$0|0)>(-1);
  do {
   if ($16) {
    $17 = (2147483647 - ($cnt$0))|0;
    $18 = ($l$0|0)>($17|0);
    if ($18) {
     $19 = (___errno_location()|0);
     HEAP32[$19>>2] = 75;
     $cnt$1 = -1;
     break;
    } else {
     $20 = (($l$0) + ($cnt$0))|0;
     $cnt$1 = $20;
     break;
    }
   } else {
    $cnt$1 = $cnt$0;
   }
  } while(0);
  $21 = HEAP8[$22>>0]|0;
  $23 = ($21<<24>>24)==(0);
  if ($23) {
   label = 339;
   break;
  } else {
   $1050 = $21;$25 = $22;
  }
  while(1) {
   if ((($1050<<24>>24) == 0)) {
    $$lcssa94 = $25;$z$0$lcssa = $25;
    break;
   } else if ((($1050<<24>>24) == 37)) {
    $27 = $25;$z$095 = $25;
    label = 9;
    break;
   }
   $24 = (($25) + 1|0);
   $$pre = HEAP8[$24>>0]|0;
   $1050 = $$pre;$25 = $24;
  }
  L12: do {
   if ((label|0) == 9) {
    while(1) {
     label = 0;
     $26 = (($27) + 1|0);
     $28 = HEAP8[$26>>0]|0;
     $29 = ($28<<24>>24)==(37);
     if (!($29)) {
      $$lcssa94 = $27;$z$0$lcssa = $z$095;
      break L12;
     }
     $30 = (($z$095) + 1|0);
     $31 = (($27) + 2|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = ($32<<24>>24)==(37);
     if ($33) {
      $27 = $31;$z$095 = $30;
      label = 9;
     } else {
      $$lcssa94 = $31;$z$0$lcssa = $30;
      break;
     }
    }
   }
  } while(0);
  $34 = $z$0$lcssa;
  $35 = $22;
  $36 = (($34) - ($35))|0;
  if ($1) {
   (___fwritex($22,$36,$f)|0);
  }
  $37 = ($z$0$lcssa|0)==($22|0);
  if (!($37)) {
   $l10n$0$phi = $l10n$0;$1049$phi = $1049;$1048$phi = $1048;$22 = $$lcssa94;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$0$phi;$1049 = $1049$phi;$1048 = $1048$phi;
   continue;
  }
  $38 = (($$lcssa94) + 1|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39 << 24 >> 24;
  $isdigittmp = (($40) + -48)|0;
  $isdigit = ($isdigittmp>>>0)<(10);
  if ($isdigit) {
   $41 = (($$lcssa94) + 2|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = ($42<<24>>24)==(36);
   if ($43) {
    $44 = (($$lcssa94) + 3|0);
    $$pre291 = HEAP8[$44>>0]|0;
    $46 = $$pre291;$argpos$0 = $isdigittmp;$l10n$1 = 1;$storemerge = $44;
   } else {
    $46 = $39;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $38;
   }
  } else {
   $46 = $39;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $38;
  }
  $45 = $46 << 24 >> 24;
  $47 = (($45) + -32)|0;
  $48 = ($47>>>0)<(32);
  L25: do {
   if ($48) {
    $50 = $45;$55 = $46;$fl$0106 = 0;$storemerge8105 = $storemerge;
    while(1) {
     $49 = (($50) + -32)|0;
     $51 = 1 << $49;
     $52 = $51 & 75913;
     $53 = ($52|0)==(0);
     if ($53) {
      $65 = $55;$fl$0102 = $fl$0106;$storemerge8100 = $storemerge8105;
      break L25;
     }
     $54 = $55 << 24 >> 24;
     $56 = (($54) + -32)|0;
     $57 = 1 << $56;
     $58 = $57 | $fl$0106;
     $59 = (($storemerge8105) + 1|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60 << 24 >> 24;
     $62 = (($61) + -32)|0;
     $63 = ($62>>>0)<(32);
     if ($63) {
      $50 = $61;$55 = $60;$fl$0106 = $58;$storemerge8105 = $59;
     } else {
      $65 = $60;$fl$0102 = $58;$storemerge8100 = $59;
      break;
     }
    }
   } else {
    $65 = $46;$fl$0102 = 0;$storemerge8100 = $storemerge;
   }
  } while(0);
  $64 = ($65<<24>>24)==(42);
  do {
   if ($64) {
    $66 = (($storemerge8100) + 1|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67 << 24 >> 24;
    $isdigittmp11 = (($68) + -48)|0;
    $isdigit12 = ($isdigittmp11>>>0)<(10);
    if ($isdigit12) {
     $69 = (($storemerge8100) + 2|0);
     $70 = HEAP8[$69>>0]|0;
     $71 = ($70<<24>>24)==(36);
     if ($71) {
      $72 = (($nl_type) + ($isdigittmp11<<2)|0);
      HEAP32[$72>>2] = 10;
      $73 = HEAP8[$66>>0]|0;
      $74 = $73 << 24 >> 24;
      $75 = (($74) + -48)|0;
      $76 = (($nl_arg) + ($75<<3)|0);
      $77 = $76;
      $78 = $77;
      $79 = HEAP32[$78>>2]|0;
      $80 = (($77) + 4)|0;
      $81 = $80;
      $82 = HEAP32[$81>>2]|0;
      $83 = (($storemerge8100) + 3|0);
      $l10n$2 = 1;$storemerge13 = $83;$w$0 = $79;
     } else {
      label = 24;
     }
    } else {
     label = 24;
    }
    if ((label|0) == 24) {
     label = 0;
     $84 = ($l10n$1|0)==(0);
     if (!($84)) {
      $$0 = -1;
      label = 357;
      break L1;
     }
     if (!($1)) {
      $100 = $66;$fl$1 = $fl$0102;$l10n$3 = 0;$w$1 = 0;
      break;
     }
     $arglist_current = HEAP32[$ap>>2]|0;
     $85 = HEAP32[$arglist_current>>2]|0;
     $arglist_next = (($arglist_current) + 4|0);
     HEAP32[$ap>>2] = $arglist_next;
     $l10n$2 = 0;$storemerge13 = $66;$w$0 = $85;
    }
    $86 = ($w$0|0)<(0);
    if ($86) {
     $87 = $fl$0102 | 8192;
     $88 = (0 - ($w$0))|0;
     $100 = $storemerge13;$fl$1 = $87;$l10n$3 = $l10n$2;$w$1 = $88;
    } else {
     $100 = $storemerge13;$fl$1 = $fl$0102;$l10n$3 = $l10n$2;$w$1 = $w$0;
    }
   } else {
    $89 = $65 << 24 >> 24;
    $isdigittmp1$i = (($89) + -48)|0;
    $isdigit2$i = ($isdigittmp1$i>>>0)<(10);
    if ($isdigit2$i) {
     $92 = $89;$95 = $storemerge8100;$i$03$i = 0;
     while(1) {
      $90 = ($i$03$i*10)|0;
      $91 = (($92) + -48)|0;
      $93 = (($91) + ($90))|0;
      $94 = (($95) + 1|0);
      $96 = HEAP8[$94>>0]|0;
      $97 = $96 << 24 >> 24;
      $isdigittmp$i = (($97) + -48)|0;
      $isdigit$i = ($isdigittmp$i>>>0)<(10);
      if ($isdigit$i) {
       $92 = $97;$95 = $94;$i$03$i = $93;
      } else {
       break;
      }
     }
     $98 = ($93|0)<(0);
     if ($98) {
      $$0 = -1;
      label = 357;
      break L1;
     } else {
      $100 = $94;$fl$1 = $fl$0102;$l10n$3 = $l10n$1;$w$1 = $93;
     }
    } else {
     $100 = $storemerge8100;$fl$1 = $fl$0102;$l10n$3 = $l10n$1;$w$1 = 0;
    }
   }
  } while(0);
  $99 = HEAP8[$100>>0]|0;
  $101 = ($99<<24>>24)==(46);
  L46: do {
   if ($101) {
    $102 = (($100) + 1|0);
    $103 = HEAP8[$102>>0]|0;
    $104 = ($103<<24>>24)==(42);
    if (!($104)) {
     $125 = $103 << 24 >> 24;
     $isdigittmp1$i21 = (($125) + -48)|0;
     $isdigit2$i22 = ($isdigittmp1$i21>>>0)<(10);
     if ($isdigit2$i22) {
      $128 = $125;$131 = $102;$i$03$i23 = 0;
     } else {
      $1051 = $102;$p$0 = 0;
      break;
     }
     while(1) {
      $126 = ($i$03$i23*10)|0;
      $127 = (($128) + -48)|0;
      $129 = (($127) + ($126))|0;
      $130 = (($131) + 1|0);
      $132 = HEAP8[$130>>0]|0;
      $133 = $132 << 24 >> 24;
      $isdigittmp$i24 = (($133) + -48)|0;
      $isdigit$i25 = ($isdigittmp$i24>>>0)<(10);
      if ($isdigit$i25) {
       $128 = $133;$131 = $130;$i$03$i23 = $129;
      } else {
       $1051 = $130;$p$0 = $129;
       break L46;
      }
     }
    }
    $105 = (($100) + 2|0);
    $106 = HEAP8[$105>>0]|0;
    $107 = $106 << 24 >> 24;
    $isdigittmp9 = (($107) + -48)|0;
    $isdigit10 = ($isdigittmp9>>>0)<(10);
    if ($isdigit10) {
     $108 = (($100) + 3|0);
     $109 = HEAP8[$108>>0]|0;
     $110 = ($109<<24>>24)==(36);
     if ($110) {
      $111 = (($nl_type) + ($isdigittmp9<<2)|0);
      HEAP32[$111>>2] = 10;
      $112 = HEAP8[$105>>0]|0;
      $113 = $112 << 24 >> 24;
      $114 = (($113) + -48)|0;
      $115 = (($nl_arg) + ($114<<3)|0);
      $116 = $115;
      $117 = $116;
      $118 = HEAP32[$117>>2]|0;
      $119 = (($116) + 4)|0;
      $120 = $119;
      $121 = HEAP32[$120>>2]|0;
      $122 = (($100) + 4|0);
      $1051 = $122;$p$0 = $118;
      break;
     }
    }
    $123 = ($l10n$3|0)==(0);
    if (!($123)) {
     $$0 = -1;
     label = 357;
     break L1;
    }
    if ($1) {
     $arglist_current2 = HEAP32[$ap>>2]|0;
     $124 = HEAP32[$arglist_current2>>2]|0;
     $arglist_next3 = (($arglist_current2) + 4|0);
     HEAP32[$ap>>2] = $arglist_next3;
     $1051 = $105;$p$0 = $124;
    } else {
     $1051 = $105;$p$0 = 0;
    }
   } else {
    $1051 = $100;$p$0 = -1;
   }
  } while(0);
  $135 = $1051;$st$0 = 0;
  while(1) {
   $134 = HEAP8[$135>>0]|0;
   $136 = $134 << 24 >> 24;
   $137 = (($136) + -65)|0;
   $138 = ($137>>>0)>(57);
   if ($138) {
    $$0 = -1;
    label = 357;
    break L1;
   }
   $139 = (($135) + 1|0);
   $140 = ((77280 + (($st$0*58)|0)|0) + ($137)|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141&255;
   $143 = (($142) + -1)|0;
   $144 = ($143>>>0)<(8);
   if ($144) {
    $135 = $139;$st$0 = $142;
   } else {
    break;
   }
  }
  $145 = ($141<<24>>24)==(0);
  if ($145) {
   $$0 = -1;
   label = 357;
   break;
  }
  $146 = ($141<<24>>24)==(19);
  $147 = ($argpos$0|0)>(-1);
  L65: do {
   if ($146) {
    if ($147) {
     $$0 = -1;
     label = 357;
     break L1;
    }
    if ($1) {
     $198 = $134;$206 = $1049;$228 = $1048;
    } else {
     $1049$phi = $1049;$1048$phi = $1048;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;$1049 = $1049$phi;$1048 = $1048$phi;
     continue L1;
    }
   } else {
    L69: do {
     if ($147) {
      $148 = (($nl_type) + ($argpos$0<<2)|0);
      HEAP32[$148>>2] = $142;
      $149 = (($nl_arg) + ($argpos$0<<3)|0);
      $150 = $149;
      $151 = $150;
      $152 = HEAP32[$151>>2]|0;
      $153 = (($150) + 4)|0;
      $154 = $153;
      $155 = HEAP32[$154>>2]|0;
      $$sink = $152;$$sink300$off32 = $155;
      label = 64;
     } else {
      if (!($1)) {
       $$0 = 0;
       label = 357;
       break L1;
      }
      $156 = ($141&255)>(20);
      if ($156) {
       $198 = $134;$206 = $1049;$228 = $1048;
       break L65;
      }
      do {
       switch ($142|0) {
       case 14:  {
        $arglist_current20 = HEAP32[$ap>>2]|0;
        $178 = HEAP32[$arglist_current20>>2]|0;
        $arglist_next21 = (($arglist_current20) + 4|0);
        HEAP32[$ap>>2] = $arglist_next21;
        $$mask1$i30 = $178 & 65535;
        $179 = $$mask1$i30;
        $1052 = $179;$1053 = 0;
        break L69;
        break;
       }
       case 17:  {
        $arglist_current29 = HEAP32[$ap>>2]|0;
        HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current29>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current29+4>>2];$189 = +HEAPF64[tempDoublePtr>>3];
        $arglist_next30 = (($arglist_current29) + 8|0);
        HEAP32[$ap>>2] = $arglist_next30;
        HEAPF64[tempDoublePtr>>3] = $189;$190 = HEAP32[tempDoublePtr>>2]|0;
        $191 = HEAP32[tempDoublePtr+4>>2]|0;
        $192 = $190;
        $1052 = $192;$1053 = $191;
        break L69;
        break;
       }
       case 9:  {
        $arglist_current5 = HEAP32[$ap>>2]|0;
        $157 = HEAP32[$arglist_current5>>2]|0;
        $arglist_next6 = (($arglist_current5) + 4|0);
        HEAP32[$ap>>2] = $arglist_next6;
        $1052 = $157;$1053 = $1048;
        break L69;
        break;
       }
       case 15:  {
        $arglist_current23 = HEAP32[$ap>>2]|0;
        $180 = HEAP32[$arglist_current23>>2]|0;
        $arglist_next24 = (($arglist_current23) + 4|0);
        HEAP32[$ap>>2] = $arglist_next24;
        $181 = $180&255;
        $182 = $181 << 24 >> 24;
        $183 = ($182|0)<(0);
        $184 = $183 << 31 >> 31;
        $sext = $180 << 24;
        $185 = $sext >> 24;
        $186 = $185;
        $1052 = $186;$1053 = $184;
        break L69;
        break;
       }
       case 12:  {
        $arglist_current14 = HEAP32[$ap>>2]|0;
        $164 = $arglist_current14;
        $165 = $164;
        $166 = HEAP32[$165>>2]|0;
        $167 = (($164) + 4)|0;
        $168 = $167;
        $169 = HEAP32[$168>>2]|0;
        $arglist_next15 = (($arglist_current14) + 8|0);
        HEAP32[$ap>>2] = $arglist_next15;
        $170 = $166;
        $1052 = $170;$1053 = $169;
        break L69;
        break;
       }
       case 18:  {
        $arglist_current32 = HEAP32[$ap>>2]|0;
        HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current32>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current32+4>>2];$193 = +HEAPF64[tempDoublePtr>>3];
        $arglist_next33 = (($arglist_current32) + 8|0);
        HEAP32[$ap>>2] = $arglist_next33;
        HEAPF64[tempDoublePtr>>3] = $193;$194 = HEAP32[tempDoublePtr>>2]|0;
        $195 = HEAP32[tempDoublePtr+4>>2]|0;
        $$sink = $194;$$sink300$off32 = $195;
        label = 64;
        break L69;
        break;
       }
       case 11:  {
        $arglist_current11 = HEAP32[$ap>>2]|0;
        $162 = HEAP32[$arglist_current11>>2]|0;
        $arglist_next12 = (($arglist_current11) + 4|0);
        HEAP32[$ap>>2] = $arglist_next12;
        $163 = $162;
        $1052 = $163;$1053 = 0;
        break L69;
        break;
       }
       case 13:  {
        $arglist_current17 = HEAP32[$ap>>2]|0;
        $171 = HEAP32[$arglist_current17>>2]|0;
        $arglist_next18 = (($arglist_current17) + 4|0);
        HEAP32[$ap>>2] = $arglist_next18;
        $172 = $171&65535;
        $173 = $172 << 16 >> 16;
        $174 = ($173|0)<(0);
        $175 = $174 << 31 >> 31;
        $sext86 = $171 << 16;
        $176 = $sext86 >> 16;
        $177 = $176;
        $1052 = $177;$1053 = $175;
        break L69;
        break;
       }
       case 16:  {
        $arglist_current26 = HEAP32[$ap>>2]|0;
        $187 = HEAP32[$arglist_current26>>2]|0;
        $arglist_next27 = (($arglist_current26) + 4|0);
        HEAP32[$ap>>2] = $arglist_next27;
        $$mask$i31 = $187 & 255;
        $188 = $$mask$i31;
        $1052 = $188;$1053 = 0;
        break L69;
        break;
       }
       case 10:  {
        $arglist_current8 = HEAP32[$ap>>2]|0;
        $158 = HEAP32[$arglist_current8>>2]|0;
        $arglist_next9 = (($arglist_current8) + 4|0);
        HEAP32[$ap>>2] = $arglist_next9;
        $159 = ($158|0)<(0);
        $160 = $159 << 31 >> 31;
        $161 = $158;
        $1052 = $161;$1053 = $160;
        break L69;
        break;
       }
       default: {
        $1052 = $1049;$1053 = $1048;
        break L69;
       }
       }
      } while(0);
     }
    } while(0);
    if ((label|0) == 64) {
     label = 0;
     $196 = $$sink;
     if ($1) {
      $1052 = $196;$1053 = $$sink300$off32;
     } else {
      $1048 = $$sink300$off32;$1049 = $196;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
      continue L1;
     }
    }
    $$pre292 = HEAP8[$135>>0]|0;
    $198 = $$pre292;$206 = $1052;$228 = $1053;
   }
  } while(0);
  $197 = $198 << 24 >> 24;
  $199 = ($st$0|0)==(0);
  if ($199) {
   $t$0 = $197;
  } else {
   $200 = $197 & 15;
   $201 = ($200|0)==(3);
   $202 = $197 & -33;
   $$ = $201 ? $202 : $197;
   $t$0 = $$;
  }
  $203 = $fl$1 & 8192;
  $204 = ($203|0)==(0);
  $205 = $fl$1 & -65537;
  $fl$1$ = $204 ? $fl$1 : $205;
  L92: do {
   switch ($t$0|0) {
   case 88: case 120:  {
    $fl$3 = $fl$1$;$p$1 = $p$0;$t$1 = $t$0;
    label = 78;
    break;
   }
   case 115:  {
    $323 = ($206|0)==(0|0);
    $$18 = $323 ? 77760 : $206;
    $a$1 = $$18;
    label = 100;
    break;
   }
   case 99:  {
    $318 = $206;
    $319 = $318&255;
    HEAP8[$4>>0] = $319;
    $1054 = $228;$1055 = $206;$a$2 = $4;$fl$6 = $205;$p$5 = 1;$pl$2 = 0;$prefix$2 = 77744;$z$2 = $2;
    break;
   }
   case 111:  {
    $248 = $206;
    $249 = ($248|0)==(0);
    $250 = ($228|0)==(0);
    $251 = $249 & $250;
    if ($251) {
     $$0$lcssa$i44 = $2;
    } else {
     $$03$i41 = $2;$253 = $248;$257 = $228;
     while(1) {
      $252 = $253 & 7;
      $254 = $252 | 48;
      $255 = $254&255;
      $256 = (($$03$i41) + -1|0);
      HEAP8[$256>>0] = $255;
      $258 = (_bitshift64Lshr(($253|0),($257|0),3)|0);
      $259 = tempRet0;
      $260 = ($258|0)==(0);
      $261 = ($259|0)==(0);
      $262 = $260 & $261;
      if ($262) {
       $$0$lcssa$i44 = $256;
       break;
      } else {
       $$03$i41 = $256;$253 = $258;$257 = $259;
      }
     }
    }
    $263 = $fl$1$ & 8;
    $264 = ($263|0)==(0);
    if ($264) {
     $307 = $206;$310 = $228;$a$0 = $$0$lcssa$i44;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = 0;$prefix$1 = 77744;
     label = 95;
    } else {
     $$14 = $251 ? 77744 : ((77744 + 5|0));
     $265 = $251&1;
     $$15 = $265 ^ 1;
     $307 = $206;$310 = $228;$a$0 = $$0$lcssa$i44;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $$15;$prefix$1 = $$14;
     label = 95;
    }
    break;
   }
   case 112:  {
    $221 = ($p$0>>>0)>(8);
    $222 = $221 ? $p$0 : 8;
    $223 = $fl$1$ | 8;
    $fl$3 = $223;$p$1 = $222;$t$1 = 120;
    label = 78;
    break;
   }
   case 83:  {
    $331 = ($p$0|0)==(0);
    if ($331) {
     $1058 = $206;$1059 = $206;$i$0167 = 0;
     label = 111;
    } else {
     $1056 = $206;$1057 = $206;$p$4296 = $p$0;
     label = 105;
    }
    break;
   }
   case 110:  {
    switch ($st$0|0) {
    case 2:  {
     $207 = ($cnt$1|0)<(0);
     $208 = $207 << 31 >> 31;
     $209 = $206;
     $210 = $209;
     HEAP32[$210>>2] = $cnt$1;
     $211 = (($209) + 4)|0;
     $212 = $211;
     HEAP32[$212>>2] = $208;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 3:  {
     $213 = $cnt$1&65535;
     HEAP16[$206>>1] = $213;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 4:  {
     $214 = $cnt$1&255;
     HEAP8[$206>>0] = $214;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 6:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 7:  {
     $215 = ($cnt$1|0)<(0);
     $216 = $215 << 31 >> 31;
     $217 = $206;
     $218 = $217;
     HEAP32[$218>>2] = $cnt$1;
     $219 = (($217) + 4)|0;
     $220 = $219;
     HEAP32[$220>>2] = $216;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 0:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 1:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    default: {
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
    }
    }
    break;
   }
   case 67:  {
    $330 = $206;
    HEAP32[$wc>>2] = $330;
    HEAP32[$5>>2] = 0;
    $1056 = $wc;$1057 = $wc;$p$4296 = -1;
    label = 105;
    break;
   }
   case 65: case 71: case 70: case 69: case 97: case 103: case 102: case 101:  {
    $367 = $206;
    HEAP32[tempDoublePtr>>2] = $367;HEAP32[tempDoublePtr+4>>2] = $228;$368 = +HEAPF64[tempDoublePtr>>3];
    HEAP32[$e2$i>>2] = 0;
    $369 = ($228|0)<(0);
    if ($369) {
     $370 = -$368;
     $$06$i = $370;$pl$0$i = 1;$prefix$0$i = 77768;
    } else {
     $371 = $fl$1$ & 2048;
     $372 = ($371|0)==(0);
     if ($372) {
      $373 = $fl$1$ & 1;
      $374 = ($373|0)==(0);
      $$$i = $374 ? ((77768 + 1|0)) : ((77768 + 6|0));
      $$06$i = $368;$pl$0$i = $373;$prefix$0$i = $$$i;
     } else {
      $$06$i = $368;$pl$0$i = 1;$prefix$0$i = ((77768 + 3|0));
     }
    }
    HEAPF64[tempDoublePtr>>3] = $$06$i;$375 = HEAP32[tempDoublePtr>>2]|0;
    $376 = HEAP32[tempDoublePtr+4>>2]|0;
    $377 = $376 & 2146435072;
    $378 = ($377>>>0)<(2146435072);
    $379 = ($377|0)==(2146435072);
    $380 = (0)<(0);
    $381 = $379 & $380;
    $382 = $378 | $381;
    if (!($382)) {
     $383 = $t$0 & 32;
     $384 = ($383|0)!=(0);
     $385 = $384 ? 77792 : 77800;
     $386 = ($$06$i != $$06$i) | (0.0 != 0.0);
     if ($386) {
      $387 = $384 ? 77808 : 77816;
      $pl$1$i = 0;$s1$0$i = $387;
     } else {
      $pl$1$i = $pl$0$i;$s1$0$i = $385;
     }
     $388 = (($pl$1$i) + 3)|0;
     $389 = $fl$1$ & 8192;
     $390 = ($389|0)==(0);
     $391 = ($388|0)<($w$1|0);
     $or$cond$i36$i = $390 & $391;
     if ($or$cond$i36$i) {
      $392 = (($w$1) - ($388))|0;
      $393 = ($392>>>0)>(256);
      $394 = $393 ? 256 : $392;
      _memset(($pad$i|0),32,($394|0))|0;
      $395 = ($392>>>0)>(255);
      if ($395) {
       $$01$i37$i = $392;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $396 = (($$01$i37$i) + -256)|0;
        $397 = ($396>>>0)>(255);
        if ($397) {
         $$01$i37$i = $396;
        } else {
         $$0$lcssa$i39$i = $396;
         break;
        }
       }
      } else {
       $$0$lcssa$i39$i = $392;
      }
      (___fwritex($pad$i,$$0$lcssa$i39$i,$f)|0);
     }
     (___fwritex($prefix$0$i,$pl$1$i,$f)|0);
     (___fwritex($s1$0$i,3,$f)|0);
     $398 = $fl$1$ & 73728;
     $399 = ($398|0)==(8192);
     $or$cond$i43$i = $399 & $391;
     if ($or$cond$i43$i) {
      $400 = (($w$1) - ($388))|0;
      $401 = ($400>>>0)>(256);
      $402 = $401 ? 256 : $400;
      _memset(($pad$i|0),32,($402|0))|0;
      $403 = ($400>>>0)>(255);
      if ($403) {
       $$01$i44$i = $400;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $404 = (($$01$i44$i) + -256)|0;
        $405 = ($404>>>0)>(255);
        if ($405) {
         $$01$i44$i = $404;
        } else {
         $$0$lcssa$i46$i = $404;
         break;
        }
       }
      } else {
       $$0$lcssa$i46$i = $400;
      }
      (___fwritex($pad$i,$$0$lcssa$i46$i,$f)|0);
     }
     $w$$i = $391 ? $w$1 : $388;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$$i;$l10n$0 = $l10n$3;
     continue L1;
    }
    $406 = (+_frexpl($$06$i,$e2$i));
    $407 = $406 * 2.0;
    $408 = $407 != 0.0;
    if ($408) {
     $409 = HEAP32[$e2$i>>2]|0;
     $410 = (($409) + -1)|0;
     HEAP32[$e2$i>>2] = $410;
    }
    $411 = $t$0 | 32;
    $412 = ($411|0)==(97);
    if ($412) {
     $413 = $t$0 & 32;
     $414 = ($413|0)==(0);
     $415 = (($prefix$0$i) + 9|0);
     $prefix$0$$i = $414 ? $prefix$0$i : $415;
     $416 = $pl$0$i | 2;
     $417 = ($p$0>>>0)>(11);
     $418 = (12 - ($p$0))|0;
     $re$0$i = $417 ? 0 : $418;
     $419 = ($re$0$i|0)==(0);
     do {
      if ($419) {
       $$1$i = $407;
      } else {
       $re$1166$i = $re$0$i;$round$0165$i = 8.0;
       while(1) {
        $420 = (($re$1166$i) + -1)|0;
        $421 = $round$0165$i * 16.0;
        $422 = ($420|0)==(0);
        if ($422) {
         break;
        } else {
         $re$1166$i = $420;$round$0165$i = $421;
        }
       }
       $423 = HEAP8[$prefix$0$$i>>0]|0;
       $424 = ($423<<24>>24)==(45);
       if ($424) {
        $425 = -$407;
        $426 = $425 - $421;
        $427 = $421 + $426;
        $428 = -$427;
        $$1$i = $428;
        break;
       } else {
        $429 = $407 + $421;
        $430 = $429 - $421;
        $$1$i = $430;
        break;
       }
      }
     } while(0);
     $431 = HEAP32[$e2$i>>2]|0;
     $432 = ($431|0)<(0);
     $433 = (0 - ($431))|0;
     $434 = $432 ? $433 : $431;
     $435 = ($434|0)<(0);
     if ($435) {
      $436 = ($434|0)<(0);
      $437 = $436 << 31 >> 31;
      $$05$i$i = $6;$438 = $434;$439 = $437;
      while(1) {
       $440 = (___uremdi3(($438|0),($439|0),10,0)|0);
       $441 = tempRet0;
       $442 = $440 | 48;
       $443 = $442&255;
       $444 = (($$05$i$i) + -1|0);
       HEAP8[$444>>0] = $443;
       $445 = (___udivdi3(($438|0),($439|0),10,0)|0);
       $446 = tempRet0;
       $447 = ($439>>>0)>(9);
       $448 = ($439|0)==(9);
       $449 = ($438>>>0)>(4294967295);
       $450 = $448 & $449;
       $451 = $447 | $450;
       if ($451) {
        $$05$i$i = $444;$438 = $445;$439 = $446;
       } else {
        break;
       }
      }
      $$0$lcssa$i49$i = $444;$$01$lcssa$off0$i$i = $445;
     } else {
      $$0$lcssa$i49$i = $6;$$01$lcssa$off0$i$i = $434;
     }
     $452 = ($$01$lcssa$off0$i$i|0)==(0);
     if ($452) {
      $$1$lcssa$i$i = $$0$lcssa$i49$i;
     } else {
      $$12$i$i = $$0$lcssa$i49$i;$y$03$i$i = $$01$lcssa$off0$i$i;
      while(1) {
       $453 = (($y$03$i$i>>>0) % 10)&-1;
       $454 = $453 | 48;
       $455 = $454&255;
       $456 = (($$12$i$i) + -1|0);
       HEAP8[$456>>0] = $455;
       $457 = (($y$03$i$i>>>0) / 10)&-1;
       $458 = ($y$03$i$i>>>0)<(10);
       if ($458) {
        $$1$lcssa$i$i = $456;
        break;
       } else {
        $$12$i$i = $456;$y$03$i$i = $457;
       }
      }
     }
     $459 = ($$1$lcssa$i$i|0)==($6|0);
     if ($459) {
      HEAP8[$7>>0] = 48;
      $estr$0$i = $7;
     } else {
      $estr$0$i = $$1$lcssa$i$i;
     }
     $460 = HEAP32[$e2$i>>2]|0;
     $461 = $460 >> 31;
     $462 = $461 & 2;
     $463 = (($462) + 43)|0;
     $464 = $463&255;
     $465 = (($estr$0$i) + -1|0);
     HEAP8[$465>>0] = $464;
     $466 = (($t$0) + 15)|0;
     $467 = $466&255;
     $468 = (($estr$0$i) + -2|0);
     HEAP8[$468>>0] = $467;
     $469 = ($p$0|0)>(0);
     $470 = $fl$1$ & 8;
     $471 = ($470|0)==(0);
     if ($469) {
      if ($471) {
       $$2$us$us$i = $$1$i;$s$0$us$us$i = $buf$i;
       while(1) {
        $472 = (~~(($$2$us$us$i)));
        $473 = (77824 + ($472)|0);
        $474 = HEAP8[$473>>0]|0;
        $475 = $474&255;
        $476 = $475 | $413;
        $477 = $476&255;
        $478 = (($s$0$us$us$i) + 1|0);
        HEAP8[$s$0$us$us$i>>0] = $477;
        $479 = (+($472|0));
        $480 = $$2$us$us$i - $479;
        $481 = $480 * 16.0;
        $482 = $478;
        $483 = (($482) - ($0))|0;
        $484 = ($483|0)==(1);
        if ($484) {
         $485 = (($s$0$us$us$i) + 2|0);
         HEAP8[$478>>0] = 46;
         $s$1$us$us$i = $485;
        } else {
         $s$1$us$us$i = $478;
        }
        $486 = $481 != 0.0;
        if ($486) {
         $$2$us$us$i = $481;$s$0$us$us$i = $s$1$us$us$i;
        } else {
         $s$1$lcssa$i = $s$1$us$us$i;
         break;
        }
       }
      } else {
       $$2$us$i = $$1$i;$s$0$us$i = $buf$i;
       while(1) {
        $487 = (~~(($$2$us$i)));
        $488 = (77824 + ($487)|0);
        $489 = HEAP8[$488>>0]|0;
        $490 = $489&255;
        $491 = $490 | $413;
        $492 = $491&255;
        $493 = (($s$0$us$i) + 1|0);
        HEAP8[$s$0$us$i>>0] = $492;
        $494 = (+($487|0));
        $495 = $$2$us$i - $494;
        $496 = $495 * 16.0;
        $497 = $493;
        $498 = (($497) - ($0))|0;
        $499 = ($498|0)==(1);
        if ($499) {
         $500 = (($s$0$us$i) + 2|0);
         HEAP8[$493>>0] = 46;
         $s$1$us$i = $500;
        } else {
         $s$1$us$i = $493;
        }
        $501 = $496 != 0.0;
        if ($501) {
         $$2$us$i = $496;$s$0$us$i = $s$1$us$i;
        } else {
         $s$1$lcssa$i = $s$1$us$i;
         break;
        }
       }
      }
     } else {
      if ($471) {
       $$2$us160$i = $$1$i;$s$0$us159$i = $buf$i;
       while(1) {
        $502 = (~~(($$2$us160$i)));
        $503 = (77824 + ($502)|0);
        $504 = HEAP8[$503>>0]|0;
        $505 = $504&255;
        $506 = $505 | $413;
        $507 = $506&255;
        $508 = (($s$0$us159$i) + 1|0);
        HEAP8[$s$0$us159$i>>0] = $507;
        $509 = (+($502|0));
        $510 = $$2$us160$i - $509;
        $511 = $510 * 16.0;
        $512 = $508;
        $513 = (($512) - ($0))|0;
        $514 = ($513|0)==(1);
        $515 = $511 != 0.0;
        $or$cond$i72 = $514 & $515;
        if ($or$cond$i72) {
         $516 = (($s$0$us159$i) + 2|0);
         HEAP8[$508>>0] = 46;
         $s$1$us161$i = $516;
        } else {
         $s$1$us161$i = $508;
        }
        if ($515) {
         $$2$us160$i = $511;$s$0$us159$i = $s$1$us161$i;
        } else {
         $s$1$lcssa$i = $s$1$us161$i;
         break;
        }
       }
      } else {
       $$2$i = $$1$i;$s$0$i = $buf$i;
       while(1) {
        $517 = (~~(($$2$i)));
        $518 = (77824 + ($517)|0);
        $519 = HEAP8[$518>>0]|0;
        $520 = $519&255;
        $521 = $520 | $413;
        $522 = $521&255;
        $523 = (($s$0$i) + 1|0);
        HEAP8[$s$0$i>>0] = $522;
        $524 = (+($517|0));
        $525 = $$2$i - $524;
        $526 = $525 * 16.0;
        $527 = $523;
        $528 = (($527) - ($0))|0;
        $529 = ($528|0)==(1);
        if ($529) {
         $530 = (($s$0$i) + 2|0);
         HEAP8[$523>>0] = 46;
         $s$1$i = $530;
        } else {
         $s$1$i = $523;
        }
        $531 = $526 != 0.0;
        if ($531) {
         $$2$i = $526;$s$0$i = $s$1$i;
        } else {
         $s$1$lcssa$i = $s$1$i;
         break;
        }
       }
      }
     }
     $532 = ($p$0|0)==(0);
     $$pre312$i = $s$1$lcssa$i;
     do {
      if ($532) {
       label = 174;
      } else {
       $533 = (($10) + ($$pre312$i))|0;
       $534 = ($533|0)<($p$0|0);
       if (!($534)) {
        label = 174;
        break;
       }
       $535 = $468;
       $536 = (($11) + ($p$0))|0;
       $537 = (($536) - ($535))|0;
       $l$0$i = $537;
      }
     } while(0);
     if ((label|0) == 174) {
      label = 0;
      $538 = $468;
      $539 = (($9) - ($538))|0;
      $540 = (($539) + ($$pre312$i))|0;
      $l$0$i = $540;
     }
     $541 = (($l$0$i) + ($416))|0;
     $542 = $fl$1$ & 73728;
     $543 = ($542|0)==(0);
     $544 = ($541|0)<($w$1|0);
     $or$cond$i53$i = $543 & $544;
     if ($or$cond$i53$i) {
      $545 = (($w$1) - ($541))|0;
      $546 = ($545>>>0)>(256);
      $547 = $546 ? 256 : $545;
      _memset(($pad$i|0),32,($547|0))|0;
      $548 = ($545>>>0)>(255);
      if ($548) {
       $$01$i54$i = $545;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $549 = (($$01$i54$i) + -256)|0;
        $550 = ($549>>>0)>(255);
        if ($550) {
         $$01$i54$i = $549;
        } else {
         $$0$lcssa$i56$i = $549;
         break;
        }
       }
      } else {
       $$0$lcssa$i56$i = $545;
      }
      (___fwritex($pad$i,$$0$lcssa$i56$i,$f)|0);
     }
     (___fwritex($prefix$0$$i,$416,$f)|0);
     $551 = ($542|0)==(65536);
     $or$cond$i60$i = $551 & $544;
     if ($or$cond$i60$i) {
      $552 = (($w$1) - ($541))|0;
      $553 = ($552>>>0)>(256);
      $554 = $553 ? 256 : $552;
      _memset(($pad$i|0),48,($554|0))|0;
      $555 = ($552>>>0)>(255);
      if ($555) {
       $$01$i61$i = $552;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $556 = (($$01$i61$i) + -256)|0;
        $557 = ($556>>>0)>(255);
        if ($557) {
         $$01$i61$i = $556;
        } else {
         $$0$lcssa$i63$i = $556;
         break;
        }
       }
      } else {
       $$0$lcssa$i63$i = $552;
      }
      (___fwritex($pad$i,$$0$lcssa$i63$i,$f)|0);
     }
     $558 = (($$pre312$i) - ($0))|0;
     (___fwritex($buf$i,$558,$f)|0);
     $559 = $468;
     $560 = (($8) - ($559))|0;
     $561 = (($l$0$i) - ($560))|0;
     $562 = (($561) - ($558))|0;
     $563 = ($562|0)>(0);
     if ($563) {
      $564 = ($562>>>0)>(256);
      $565 = $564 ? 256 : $562;
      _memset(($pad$i|0),48,($565|0))|0;
      $566 = ($562>>>0)>(255);
      if ($566) {
       $$01$i67$i = $562;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $567 = (($$01$i67$i) + -256)|0;
        $568 = ($567>>>0)>(255);
        if ($568) {
         $$01$i67$i = $567;
        } else {
         $$0$lcssa$i69$i = $567;
         break;
        }
       }
      } else {
       $$0$lcssa$i69$i = $562;
      }
      (___fwritex($pad$i,$$0$lcssa$i69$i,$f)|0);
     }
     (___fwritex($468,$560,$f)|0);
     $569 = ($542|0)==(8192);
     $or$cond$i73$i = $569 & $544;
     if ($or$cond$i73$i) {
      $570 = (($w$1) - ($541))|0;
      $571 = ($570>>>0)>(256);
      $572 = $571 ? 256 : $570;
      _memset(($pad$i|0),32,($572|0))|0;
      $573 = ($570>>>0)>(255);
      if ($573) {
       $$01$i74$i = $570;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $574 = (($$01$i74$i) + -256)|0;
        $575 = ($574>>>0)>(255);
        if ($575) {
         $$01$i74$i = $574;
        } else {
         $$0$lcssa$i76$i = $574;
         break;
        }
       }
      } else {
       $$0$lcssa$i76$i = $570;
      }
      (___fwritex($pad$i,$$0$lcssa$i76$i,$f)|0);
     }
     $w$18$i = $544 ? $w$1 : $541;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$18$i;$l10n$0 = $l10n$3;
     continue L1;
    }
    $576 = ($p$0|0)<(0);
    $$p$i = $576 ? 6 : $p$0;
    if ($408) {
     $577 = $407 * 268435456.0;
     $578 = HEAP32[$e2$i>>2]|0;
     $579 = (($578) + -28)|0;
     HEAP32[$e2$i>>2] = $579;
     $$3$i = $577;$581 = $579;
    } else {
     $$pre$i = HEAP32[$e2$i>>2]|0;
     $$3$i = $407;$581 = $$pre$i;
    }
    $580 = ($581|0)<(0);
    $$32$i = $580 ? $big$i : $12;
    $$4$i = $$3$i;$z$0$i = $$32$i;
    while(1) {
     $582 = (~~(($$4$i))>>>0);
     HEAP32[$z$0$i>>2] = $582;
     $583 = (($z$0$i) + 4|0);
     $584 = (+($582>>>0));
     $585 = $$4$i - $584;
     $586 = $585 * 1.0E+9;
     $587 = $586 != 0.0;
     if ($587) {
      $$4$i = $586;$z$0$i = $583;
     } else {
      break;
     }
    }
    $$pr$i = HEAP32[$e2$i>>2]|0;
    $588 = ($$pr$i|0)>(0);
    if ($588) {
     $590 = $$pr$i;$a$1257$i = $$32$i;$z$1256$i = $583;
     while(1) {
      $589 = ($590|0)>(29);
      $591 = $589 ? 29 : $590;
      $592 = (($z$1256$i) + -4|0);
      $593 = ($592>>>0)<($a$1257$i>>>0);
      do {
       if ($593) {
        $594 = HEAP32[$592>>2]|0;
        $595 = ($594|0)==(0);
        $596 = ($z$1256$i>>>0)>($a$1257$i>>>0);
        $or$cond19315$i = $595 & $596;
        $z$2316$i = $or$cond19315$i ? $592 : $z$1256$i;
        $a$2$i = $a$1257$i;$z$2317$i = $z$2316$i;
       } else {
        $carry$0250$i = 0;$d$0251$i = $592;
        while(1) {
         $597 = HEAP32[$d$0251$i>>2]|0;
         $598 = (_bitshift64Shl(($597|0),0,($591|0))|0);
         $599 = tempRet0;
         $600 = (_i64Add(($598|0),($599|0),($carry$0250$i|0),0)|0);
         $601 = tempRet0;
         $602 = (___uremdi3(($600|0),($601|0),1000000000,0)|0);
         $603 = tempRet0;
         HEAP32[$d$0251$i>>2] = $602;
         $604 = (___udivdi3(($600|0),($601|0),1000000000,0)|0);
         $605 = tempRet0;
         $606 = (($d$0251$i) + -4|0);
         $607 = ($606>>>0)<($a$1257$i>>>0);
         if ($607) {
          break;
         } else {
          $carry$0250$i = $604;$d$0251$i = $606;
         }
        }
        $608 = HEAP32[$592>>2]|0;
        $609 = ($608|0)==(0);
        $610 = ($z$1256$i>>>0)>($a$1257$i>>>0);
        $or$cond19$i = $609 & $610;
        $z$2$i = $or$cond19$i ? $592 : $z$1256$i;
        $611 = ($604|0)==(0);
        if ($611) {
         $a$2$i = $a$1257$i;$z$2317$i = $z$2$i;
         break;
        }
        $612 = (($a$1257$i) + -4|0);
        HEAP32[$612>>2] = $604;
        $a$2$i = $612;$z$2317$i = $z$2$i;
       }
      } while(0);
      $613 = HEAP32[$e2$i>>2]|0;
      $614 = (($613) - ($591))|0;
      HEAP32[$e2$i>>2] = $614;
      $615 = ($614|0)>(0);
      if ($615) {
       $590 = $614;$a$1257$i = $a$2$i;$z$1256$i = $z$2317$i;
      } else {
       $$pr147$i = $614;$a$1$lcssa$i = $a$2$i;$z$1$lcssa$i = $z$2317$i;
       break;
      }
     }
    } else {
     $$pr147$i = $$pr$i;$a$1$lcssa$i = $$32$i;$z$1$lcssa$i = $583;
    }
    $616 = ($$pr147$i|0)<(0);
    L245: do {
     if ($616) {
      $617 = ($411|0)==(102);
      $618 = (($$p$i|0) / 9)&-1;
      $619 = (($618) + 2)|0;
      if ($617) {
       $620 = $$32$i;
       $621 = (($$32$i) + ($619<<2)|0);
       $623 = $$pr147$i;$a$3245$us$i = $a$1$lcssa$i;$z$3244$us$i = $z$1$lcssa$i;
       while(1) {
        $622 = (0 - ($623))|0;
        $624 = ($622|0)>(9);
        $$20$us$i = $624 ? 9 : $622;
        $625 = ($a$3245$us$i>>>0)<($z$3244$us$i>>>0);
        do {
         if ($625) {
          $650 = 1 << $$20$us$i;
          $644 = (($650) + -1)|0;
          $647 = 1000000000 >>> $$20$us$i;
          $carry3$0239$us$i = 0;$d$1238$us$i = $a$3245$us$i;
          while(1) {
           $642 = HEAP32[$d$1238$us$i>>2]|0;
           $643 = $642 & $644;
           $645 = $642 >>> $$20$us$i;
           $646 = (($645) + ($carry3$0239$us$i))|0;
           HEAP32[$d$1238$us$i>>2] = $646;
           $633 = Math_imul($643, $647)|0;
           $648 = (($d$1238$us$i) + 4|0);
           $649 = ($648>>>0)<($z$3244$us$i>>>0);
           if ($649) {
            $carry3$0239$us$i = $633;$d$1238$us$i = $648;
           } else {
            break;
           }
          }
          $629 = HEAP32[$a$3245$us$i>>2]|0;
          $630 = ($629|0)==(0);
          $631 = (($a$3245$us$i) + 4|0);
          $$a$3$us$i = $630 ? $631 : $a$3245$us$i;
          $632 = ($633|0)==(0);
          if ($632) {
           $$a$3$us320$i = $$a$3$us$i;$z$4$us$i = $z$3244$us$i;
           break;
          }
          $634 = (($z$3244$us$i) + 4|0);
          HEAP32[$z$3244$us$i>>2] = $633;
          $$a$3$us320$i = $$a$3$us$i;$z$4$us$i = $634;
         } else {
          $626 = HEAP32[$a$3245$us$i>>2]|0;
          $627 = ($626|0)==(0);
          $628 = (($a$3245$us$i) + 4|0);
          $$a$3$us319$i = $627 ? $628 : $a$3245$us$i;
          $$a$3$us320$i = $$a$3$us319$i;$z$4$us$i = $z$3244$us$i;
         }
        } while(0);
        $635 = $z$4$us$i;
        $636 = (($635) - ($620))|0;
        $637 = $636 >> 2;
        $638 = ($637|0)>($619|0);
        $$z$4$us$i = $638 ? $621 : $z$4$us$i;
        $639 = HEAP32[$e2$i>>2]|0;
        $640 = (($639) + ($$20$us$i))|0;
        HEAP32[$e2$i>>2] = $640;
        $641 = ($640|0)<(0);
        if ($641) {
         $623 = $640;$a$3245$us$i = $$a$3$us320$i;$z$3244$us$i = $$z$4$us$i;
        } else {
         $a$3$lcssa$i = $$a$3$us320$i;$z$3$lcssa$i = $$z$4$us$i;
         break L245;
        }
       }
      } else {
       $652 = $$pr147$i;$a$3245$i = $a$1$lcssa$i;$z$3244$i = $z$1$lcssa$i;
      }
      while(1) {
       $651 = (0 - ($652))|0;
       $653 = ($651|0)>(9);
       $$20$i = $653 ? 9 : $651;
       $654 = ($a$3245$i>>>0)<($z$3244$i>>>0);
       do {
        if ($654) {
         $658 = 1 << $$20$i;
         $659 = (($658) + -1)|0;
         $660 = 1000000000 >>> $$20$i;
         $carry3$0239$i = 0;$d$1238$i = $a$3245$i;
         while(1) {
          $661 = HEAP32[$d$1238$i>>2]|0;
          $662 = $661 & $659;
          $663 = $661 >>> $$20$i;
          $664 = (($663) + ($carry3$0239$i))|0;
          HEAP32[$d$1238$i>>2] = $664;
          $665 = Math_imul($662, $660)|0;
          $666 = (($d$1238$i) + 4|0);
          $667 = ($666>>>0)<($z$3244$i>>>0);
          if ($667) {
           $carry3$0239$i = $665;$d$1238$i = $666;
          } else {
           break;
          }
         }
         $668 = HEAP32[$a$3245$i>>2]|0;
         $669 = ($668|0)==(0);
         $670 = (($a$3245$i) + 4|0);
         $$a$3$i = $669 ? $670 : $a$3245$i;
         $671 = ($665|0)==(0);
         if ($671) {
          $$a$3322$i = $$a$3$i;$z$4$i = $z$3244$i;
          break;
         }
         $672 = (($z$3244$i) + 4|0);
         HEAP32[$z$3244$i>>2] = $665;
         $$a$3322$i = $$a$3$i;$z$4$i = $672;
        } else {
         $655 = HEAP32[$a$3245$i>>2]|0;
         $656 = ($655|0)==(0);
         $657 = (($a$3245$i) + 4|0);
         $$a$3321$i = $656 ? $657 : $a$3245$i;
         $$a$3322$i = $$a$3321$i;$z$4$i = $z$3244$i;
        }
       } while(0);
       $673 = $z$4$i;
       $674 = $$a$3322$i;
       $675 = (($673) - ($674))|0;
       $676 = $675 >> 2;
       $677 = ($676|0)>($619|0);
       if ($677) {
        $678 = (($$a$3322$i) + ($619<<2)|0);
        $z$5$i = $678;
       } else {
        $z$5$i = $z$4$i;
       }
       $679 = HEAP32[$e2$i>>2]|0;
       $680 = (($679) + ($$20$i))|0;
       HEAP32[$e2$i>>2] = $680;
       $681 = ($680|0)<(0);
       if ($681) {
        $652 = $680;$a$3245$i = $$a$3322$i;$z$3244$i = $z$5$i;
       } else {
        $a$3$lcssa$i = $$a$3322$i;$z$3$lcssa$i = $z$5$i;
        break;
       }
      }
     } else {
      $a$3$lcssa$i = $a$1$lcssa$i;$z$3$lcssa$i = $z$1$lcssa$i;
     }
    } while(0);
    $682 = ($a$3$lcssa$i>>>0)<($z$3$lcssa$i>>>0);
    $683 = $$32$i;
    do {
     if ($682) {
      $684 = $a$3$lcssa$i;
      $685 = (($683) - ($684))|0;
      $686 = $685 >> 2;
      $687 = ($686*9)|0;
      $688 = HEAP32[$a$3$lcssa$i>>2]|0;
      $689 = ($688>>>0)<(10);
      if ($689) {
       $e$1$i = $687;
       break;
      } else {
       $e$0233$i = $687;$i$0234$i = 10;
      }
      while(1) {
       $690 = ($i$0234$i*10)|0;
       $691 = (($e$0233$i) + 1)|0;
       $692 = ($688>>>0)<($690>>>0);
       if ($692) {
        $e$1$i = $691;
        break;
       } else {
        $e$0233$i = $691;$i$0234$i = $690;
       }
      }
     } else {
      $e$1$i = 0;
     }
    } while(0);
    $693 = ($411|0)!=(102);
    $694 = $693 ? $e$1$i : 0;
    $695 = (($$p$i) - ($694))|0;
    $696 = ($411|0)==(103);
    $697 = ($$p$i|0)!=(0);
    $$21$i = $696 & $697;
    $$neg152$i = $$21$i << 31 >> 31;
    $698 = (($695) + ($$neg152$i))|0;
    $699 = $z$3$lcssa$i;
    $700 = (($699) - ($683))|0;
    $701 = $700 >> 2;
    $702 = ($701*9)|0;
    $703 = (($702) + -9)|0;
    $704 = ($698|0)<($703|0);
    if ($704) {
     $705 = (($698) + 9216)|0;
     $706 = (($705|0) / 9)&-1;
     $$sum$i = (($706) + -1023)|0;
     $707 = (($$32$i) + ($$sum$i<<2)|0);
     $708 = (($705|0) % 9)&-1;
     $j$0225$i = (($708) + 1)|0;
     $709 = ($j$0225$i|0)<(9);
     if ($709) {
      $i$1226$i = 10;$j$0227$i = $j$0225$i;
      while(1) {
       $710 = ($i$1226$i*10)|0;
       $j$0$i = (($j$0227$i) + 1)|0;
       $exitcond$i = ($j$0$i|0)==(9);
       if ($exitcond$i) {
        $i$1$lcssa$i = $710;
        break;
       } else {
        $i$1226$i = $710;$j$0227$i = $j$0$i;
       }
      }
     } else {
      $i$1$lcssa$i = 10;
     }
     $711 = HEAP32[$707>>2]|0;
     $712 = (($711>>>0) % ($i$1$lcssa$i>>>0))&-1;
     $713 = ($712|0)==(0);
     if ($713) {
      $$sum14$i = (($706) + -1022)|0;
      $714 = (($$32$i) + ($$sum14$i<<2)|0);
      $715 = ($714|0)==($z$3$lcssa$i|0);
      if ($715) {
       $a$6$i = $a$3$lcssa$i;$d$3$i = $707;$e$3$i = $e$1$i;
      } else {
       label = 232;
      }
     } else {
      label = 232;
     }
     do {
      if ((label|0) == 232) {
       label = 0;
       $716 = (($711>>>0) / ($i$1$lcssa$i>>>0))&-1;
       $717 = $716 & 1;
       $718 = ($717|0)==(0);
       $$22$i = $718 ? 9007199254740992.0 : 9007199254740994.0;
       $719 = (($i$1$lcssa$i|0) / 2)&-1;
       $720 = ($712>>>0)<($719>>>0);
       do {
        if ($720) {
         $small$0$i = 0.5;
        } else {
         $721 = ($712|0)==($719|0);
         if ($721) {
          $$sum15$i = (($706) + -1022)|0;
          $722 = (($$32$i) + ($$sum15$i<<2)|0);
          $723 = ($722|0)==($z$3$lcssa$i|0);
          if ($723) {
           $small$0$i = 1.0;
           break;
          }
         }
         $small$0$i = 1.5;
        }
       } while(0);
       $724 = ($pl$0$i|0)==(0);
       do {
        if ($724) {
         $round6$1$i = $$22$i;$small$1$i = $small$0$i;
        } else {
         $725 = HEAP8[$prefix$0$i>>0]|0;
         $726 = ($725<<24>>24)==(45);
         if (!($726)) {
          $round6$1$i = $$22$i;$small$1$i = $small$0$i;
          break;
         }
         $727 = $$22$i * -1.0;
         $728 = $small$0$i * -1.0;
         $round6$1$i = $727;$small$1$i = $728;
        }
       } while(0);
       $729 = (($711) - ($712))|0;
       HEAP32[$707>>2] = $729;
       $730 = $round6$1$i + $small$1$i;
       $731 = $730 != $round6$1$i;
       if (!($731)) {
        $a$6$i = $a$3$lcssa$i;$d$3$i = $707;$e$3$i = $e$1$i;
        break;
       }
       $732 = (($729) + ($i$1$lcssa$i))|0;
       HEAP32[$707>>2] = $732;
       $733 = ($732>>>0)>(999999999);
       if ($733) {
        $d$2220$i = $707;
        while(1) {
         $734 = (($d$2220$i) + -4|0);
         HEAP32[$d$2220$i>>2] = 0;
         $735 = HEAP32[$734>>2]|0;
         $736 = (($735) + 1)|0;
         HEAP32[$734>>2] = $736;
         $737 = ($736>>>0)>(999999999);
         if ($737) {
          $d$2220$i = $734;
         } else {
          $d$2$lcssa$i = $734;
          break;
         }
        }
       } else {
        $d$2$lcssa$i = $707;
       }
       $738 = ($d$2$lcssa$i>>>0)<($a$3$lcssa$i>>>0);
       $d$2$a$3$i = $738 ? $d$2$lcssa$i : $a$3$lcssa$i;
       $739 = $d$2$a$3$i;
       $740 = (($683) - ($739))|0;
       $741 = $740 >> 2;
       $742 = ($741*9)|0;
       $743 = HEAP32[$d$2$a$3$i>>2]|0;
       $744 = ($743>>>0)<(10);
       if ($744) {
        $a$6$i = $d$2$a$3$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $742;
        break;
       } else {
        $e$2215$i = $742;$i$2216$i = 10;
       }
       while(1) {
        $745 = ($i$2216$i*10)|0;
        $746 = (($e$2215$i) + 1)|0;
        $747 = ($743>>>0)<($745>>>0);
        if ($747) {
         $a$6$i = $d$2$a$3$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $746;
         break;
        } else {
         $e$2215$i = $746;$i$2216$i = $745;
        }
       }
      }
     } while(0);
     $748 = (($d$3$i) + 4|0);
     $749 = ($z$3$lcssa$i>>>0)>($748>>>0);
     $$z$3$i = $749 ? $748 : $z$3$lcssa$i;
     $z$6$i = $$z$3$i;
     while(1) {
      $750 = (($z$6$i) + -4|0);
      $751 = HEAP32[$750>>2]|0;
      $752 = ($751|0)==(0);
      $753 = ($z$6$i>>>0)>($a$6$i>>>0);
      $or$cond23$i = $752 & $753;
      if ($or$cond23$i) {
       $z$6$i = $750;
      } else {
       $a$7$i = $a$6$i;$e$4$i = $e$3$i;$z$7$i = $z$6$i;
       break;
      }
     }
    } else {
     $a$7$i = $a$3$lcssa$i;$e$4$i = $e$1$i;$z$7$i = $z$3$lcssa$i;
    }
    $754 = (0 - ($e$4$i))|0;
    do {
     if ($696) {
      $755 = ($$p$i|0)==(0);
      $756 = $755&1;
      $$$p$i = (($756) + ($$p$i))|0;
      $757 = ($$$p$i|0)>($e$4$i|0);
      $758 = ($e$4$i|0)>(-5);
      $or$cond3$i = $757 & $758;
      if ($or$cond3$i) {
       $759 = (($t$0) + -1)|0;
       $$neg153$i = (($$$p$i) + -1)|0;
       $760 = (($$neg153$i) - ($e$4$i))|0;
       $$012$i73 = $759;$$29$i = $760;
      } else {
       $761 = (($t$0) + -2)|0;
       $762 = (($$$p$i) + -1)|0;
       $$012$i73 = $761;$$29$i = $762;
      }
      $763 = $fl$1$ & 8;
      $764 = ($763|0)==(0);
      if (!($764)) {
       $$113$i = $$012$i73;$$310$i = $$29$i;
       break;
      }
      $765 = ($z$7$i>>>0)>($a$7$i>>>0);
      do {
       if ($765) {
        $766 = (($z$7$i) + -4|0);
        $767 = HEAP32[$766>>2]|0;
        $768 = ($767|0)==(0);
        if ($768) {
         $j$2$i = 9;
         break;
        }
        $769 = (($767>>>0) % 10)&-1;
        $770 = ($769|0)==(0);
        if ($770) {
         $i$3210$i = 10;$j$1211$i = 0;
        } else {
         $j$2$i = 0;
         break;
        }
        while(1) {
         $771 = ($i$3210$i*10)|0;
         $772 = (($j$1211$i) + 1)|0;
         $773 = (($767>>>0) % ($771>>>0))&-1;
         $774 = ($773|0)==(0);
         if ($774) {
          $i$3210$i = $771;$j$1211$i = $772;
         } else {
          $j$2$i = $772;
          break;
         }
        }
       } else {
        $j$2$i = 9;
       }
      } while(0);
      $775 = $$012$i73 | 32;
      $776 = ($775|0)==(102);
      $777 = $z$7$i;
      $778 = (($777) - ($683))|0;
      $779 = $778 >> 2;
      $780 = ($779*9)|0;
      $781 = (($780) + -9)|0;
      if ($776) {
       $782 = (($781) - ($j$2$i))|0;
       $783 = ($782|0)<(0);
       $$24$i = $783 ? 0 : $782;
       $784 = ($$29$i|0)<($$24$i|0);
       $$29$$24$i = $784 ? $$29$i : $$24$i;
       $$113$i = $$012$i73;$$310$i = $$29$$24$i;
       break;
      } else {
       $785 = (($781) + ($e$4$i))|0;
       $786 = (($785) - ($j$2$i))|0;
       $787 = ($786|0)<(0);
       $$26$i = $787 ? 0 : $786;
       $788 = ($$29$i|0)<($$26$i|0);
       $$29$$26$i = $788 ? $$29$i : $$26$i;
       $$113$i = $$012$i73;$$310$i = $$29$$26$i;
       break;
      }
     } else {
      $$113$i = $t$0;$$310$i = $$p$i;
     }
    } while(0);
    $789 = ($$310$i|0)!=(0);
    if ($789) {
     $793 = 1;
    } else {
     $790 = $fl$1$ & 8;
     $791 = ($790|0)!=(0);
     $793 = $791;
    }
    $792 = $793&1;
    $794 = $$113$i | 32;
    $795 = ($794|0)==(102);
    if ($795) {
     $796 = ($e$4$i|0)>(0);
     $797 = $796 ? $e$4$i : 0;
     $$pn$i = $797;$estr$2$i = 0;
    } else {
     $798 = ($e$4$i|0)<(0);
     $799 = $798 ? $754 : $e$4$i;
     $800 = ($799|0)<(0);
     if ($800) {
      $801 = ($799|0)<(0);
      $802 = $801 << 31 >> 31;
      $$05$i80$i = $6;$803 = $799;$804 = $802;
      while(1) {
       $805 = (___uremdi3(($803|0),($804|0),10,0)|0);
       $806 = tempRet0;
       $807 = $805 | 48;
       $808 = $807&255;
       $809 = (($$05$i80$i) + -1|0);
       HEAP8[$809>>0] = $808;
       $810 = (___udivdi3(($803|0),($804|0),10,0)|0);
       $811 = tempRet0;
       $812 = ($804>>>0)>(9);
       $813 = ($804|0)==(9);
       $814 = ($803>>>0)>(4294967295);
       $815 = $813 & $814;
       $816 = $812 | $815;
       if ($816) {
        $$05$i80$i = $809;$803 = $810;$804 = $811;
       } else {
        break;
       }
      }
      $$0$lcssa$i85$i = $809;$$01$lcssa$off0$i86$i = $810;
     } else {
      $$0$lcssa$i85$i = $6;$$01$lcssa$off0$i86$i = $799;
     }
     $817 = ($$01$lcssa$off0$i86$i|0)==(0);
     if ($817) {
      $estr$1$ph$i = $$0$lcssa$i85$i;
     } else {
      $$12$i88$i = $$0$lcssa$i85$i;$y$03$i87$i = $$01$lcssa$off0$i86$i;
      while(1) {
       $818 = (($y$03$i87$i>>>0) % 10)&-1;
       $819 = $818 | 48;
       $820 = $819&255;
       $821 = (($$12$i88$i) + -1|0);
       HEAP8[$821>>0] = $820;
       $822 = (($y$03$i87$i>>>0) / 10)&-1;
       $823 = ($y$03$i87$i>>>0)<(10);
       if ($823) {
        $estr$1$ph$i = $821;
        break;
       } else {
        $$12$i88$i = $821;$y$03$i87$i = $822;
       }
      }
     }
     $824 = $estr$1$ph$i;
     $825 = (($8) - ($824))|0;
     $826 = ($825|0)<(2);
     if ($826) {
      $estr$1201$i = $estr$1$ph$i;
      while(1) {
       $827 = (($estr$1201$i) + -1|0);
       HEAP8[$827>>0] = 48;
       $828 = $827;
       $829 = (($8) - ($828))|0;
       $830 = ($829|0)<(2);
       if ($830) {
        $estr$1201$i = $827;
       } else {
        $estr$1$lcssa$i = $827;
        break;
       }
      }
     } else {
      $estr$1$lcssa$i = $estr$1$ph$i;
     }
     $831 = $e$4$i >> 31;
     $832 = $831 & 2;
     $833 = (($832) + 43)|0;
     $834 = $833&255;
     $835 = (($estr$1$lcssa$i) + -1|0);
     HEAP8[$835>>0] = $834;
     $836 = $$113$i&255;
     $837 = (($estr$1$lcssa$i) + -2|0);
     HEAP8[$837>>0] = $836;
     $838 = $837;
     $839 = (($8) - ($838))|0;
     $$pn$i = $839;$estr$2$i = $837;
    }
    $840 = (($pl$0$i) + 1)|0;
    $841 = (($840) + ($$310$i))|0;
    $l$1$i = (($841) + ($792))|0;
    $842 = (($l$1$i) + ($$pn$i))|0;
    $843 = $fl$1$ & 73728;
    $844 = ($843|0)==(0);
    $845 = ($842|0)<($w$1|0);
    $or$cond$i94$i = $844 & $845;
    if ($or$cond$i94$i) {
     $846 = (($w$1) - ($842))|0;
     $847 = ($846>>>0)>(256);
     $848 = $847 ? 256 : $846;
     _memset(($pad$i|0),32,($848|0))|0;
     $849 = ($846>>>0)>(255);
     if ($849) {
      $$01$i95$i = $846;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $850 = (($$01$i95$i) + -256)|0;
       $851 = ($850>>>0)>(255);
       if ($851) {
        $$01$i95$i = $850;
       } else {
        $$0$lcssa$i97$i = $850;
        break;
       }
      }
     } else {
      $$0$lcssa$i97$i = $846;
     }
     (___fwritex($pad$i,$$0$lcssa$i97$i,$f)|0);
    }
    (___fwritex($prefix$0$i,$pl$0$i,$f)|0);
    $852 = ($843|0)==(65536);
    $or$cond$i101$i = $852 & $845;
    if ($or$cond$i101$i) {
     $853 = (($w$1) - ($842))|0;
     $854 = ($853>>>0)>(256);
     $855 = $854 ? 256 : $853;
     _memset(($pad$i|0),48,($855|0))|0;
     $856 = ($853>>>0)>(255);
     if ($856) {
      $$01$i102$i = $853;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $857 = (($$01$i102$i) + -256)|0;
       $858 = ($857>>>0)>(255);
       if ($858) {
        $$01$i102$i = $857;
       } else {
        $$0$lcssa$i104$i = $857;
        break;
       }
      }
     } else {
      $$0$lcssa$i104$i = $853;
     }
     (___fwritex($pad$i,$$0$lcssa$i104$i,$f)|0);
    }
    do {
     if ($795) {
      $859 = ($a$7$i>>>0)>($$32$i>>>0);
      $r$0$a$7$i = $859 ? $$32$i : $a$7$i;
      $d$4184$i = $r$0$a$7$i;
      while(1) {
       $860 = HEAP32[$d$4184$i>>2]|0;
       $861 = ($860|0)==(0);
       if ($861) {
        $$1$lcssa$i113$i = $13;
       } else {
        $$12$i111$i = $13;$y$03$i110$i = $860;
        while(1) {
         $862 = (($y$03$i110$i>>>0) % 10)&-1;
         $863 = $862 | 48;
         $864 = $863&255;
         $865 = (($$12$i111$i) + -1|0);
         HEAP8[$865>>0] = $864;
         $866 = (($y$03$i110$i>>>0) / 10)&-1;
         $867 = ($y$03$i110$i>>>0)<(10);
         if ($867) {
          $$1$lcssa$i113$i = $865;
          break;
         } else {
          $$12$i111$i = $865;$y$03$i110$i = $866;
         }
        }
       }
       $868 = ($d$4184$i|0)==($r$0$a$7$i|0);
       do {
        if ($868) {
         $872 = ($$1$lcssa$i113$i|0)==($13|0);
         if (!($872)) {
          $s7$1$i = $$1$lcssa$i113$i;
          break;
         }
         HEAP8[$15>>0] = 48;
         $s7$1$i = $15;
        } else {
         $869 = ($$1$lcssa$i113$i>>>0)>($buf$i>>>0);
         if ($869) {
          $s7$0181$i = $$1$lcssa$i113$i;
         } else {
          $s7$1$i = $$1$lcssa$i113$i;
          break;
         }
         while(1) {
          $870 = (($s7$0181$i) + -1|0);
          HEAP8[$870>>0] = 48;
          $871 = ($870>>>0)>($buf$i>>>0);
          if ($871) {
           $s7$0181$i = $870;
          } else {
           $s7$1$i = $870;
           break;
          }
         }
        }
       } while(0);
       $873 = $s7$1$i;
       $874 = (($14) - ($873))|0;
       (___fwritex($s7$1$i,$874,$f)|0);
       $875 = (($d$4184$i) + 4|0);
       $876 = ($875>>>0)>($$32$i>>>0);
       if ($876) {
        break;
       } else {
        $d$4184$i = $875;
       }
      }
      if (!($789)) {
       $877 = $fl$1$ & 8;
       $878 = ($877|0)==(0);
       if ($878) {
        break;
       }
      }
      (___fwritex(77840,1,$f)|0);
      $879 = ($875>>>0)<($z$7$i>>>0);
      $880 = ($$310$i|0)>(0);
      $or$cond29174$i = $879 & $880;
      if ($or$cond29174$i) {
       $$411176$i = $$310$i;$d$5175$i = $875;
       while(1) {
        $881 = HEAP32[$d$5175$i>>2]|0;
        $882 = ($881|0)==(0);
        if ($882) {
         $s8$0170$i = $13;
         label = 295;
        } else {
         $$12$i120$i = $13;$y$03$i119$i = $881;
         while(1) {
          $883 = (($y$03$i119$i>>>0) % 10)&-1;
          $884 = $883 | 48;
          $885 = $884&255;
          $886 = (($$12$i120$i) + -1|0);
          HEAP8[$886>>0] = $885;
          $887 = (($y$03$i119$i>>>0) / 10)&-1;
          $888 = ($y$03$i119$i>>>0)<(10);
          if ($888) {
           break;
          } else {
           $$12$i120$i = $886;$y$03$i119$i = $887;
          }
         }
         $889 = ($886>>>0)>($buf$i>>>0);
         if ($889) {
          $s8$0170$i = $886;
          label = 295;
         } else {
          $s8$0$lcssa$i = $886;
         }
        }
        if ((label|0) == 295) {
         while(1) {
          label = 0;
          $890 = (($s8$0170$i) + -1|0);
          HEAP8[$890>>0] = 48;
          $891 = ($890>>>0)>($buf$i>>>0);
          if ($891) {
           $s8$0170$i = $890;
           label = 295;
          } else {
           $s8$0$lcssa$i = $890;
           break;
          }
         }
        }
        $892 = ($$411176$i|0)>(9);
        $893 = $892 ? 9 : $$411176$i;
        (___fwritex($s8$0$lcssa$i,$893,$f)|0);
        $894 = (($d$5175$i) + 4|0);
        $895 = (($$411176$i) + -9)|0;
        $896 = ($894>>>0)<($z$7$i>>>0);
        $897 = ($895|0)>(0);
        $or$cond29$i = $896 & $897;
        if ($or$cond29$i) {
         $$411176$i = $895;$d$5175$i = $894;
        } else {
         $$411$lcssa$i = $895;
         break;
        }
       }
      } else {
       $$411$lcssa$i = $$310$i;
      }
      $898 = ($$411$lcssa$i|0)>(0);
      if (!($898)) {
       break;
      }
      $899 = ($$411$lcssa$i>>>0)>(256);
      $900 = $899 ? 256 : $$411$lcssa$i;
      _memset(($pad$i|0),48,($900|0))|0;
      $901 = ($$411$lcssa$i>>>0)>(255);
      if ($901) {
       $$01$i126$i = $$411$lcssa$i;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $902 = (($$01$i126$i) + -256)|0;
        $903 = ($902>>>0)>(255);
        if ($903) {
         $$01$i126$i = $902;
        } else {
         $$0$lcssa$i128$i = $902;
         break;
        }
       }
      } else {
       $$0$lcssa$i128$i = $$411$lcssa$i;
      }
      (___fwritex($pad$i,$$0$lcssa$i128$i,$f)|0);
     } else {
      $904 = ($z$7$i>>>0)>($a$7$i>>>0);
      $905 = (($a$7$i) + 4|0);
      $z$7$$i = $904 ? $z$7$i : $905;
      $906 = ($$310$i|0)>(-1);
      do {
       if ($906) {
        $907 = $fl$1$ & 8;
        $$not$i = ($907|0)!=(0);
        $$5194$i = $$310$i;$d$6193$i = $a$7$i;
        while(1) {
         $908 = HEAP32[$d$6193$i>>2]|0;
         $909 = ($908|0)==(0);
         if ($909) {
          label = 306;
         } else {
          $$12$i135$i = $13;$y$03$i134$i = $908;
          while(1) {
           $910 = (($y$03$i134$i>>>0) % 10)&-1;
           $911 = $910 | 48;
           $912 = $911&255;
           $913 = (($$12$i135$i) + -1|0);
           HEAP8[$913>>0] = $912;
           $914 = (($y$03$i134$i>>>0) / 10)&-1;
           $915 = ($y$03$i134$i>>>0)<(10);
           if ($915) {
            break;
           } else {
            $$12$i135$i = $913;$y$03$i134$i = $914;
           }
          }
          $916 = ($913|0)==($13|0);
          if ($916) {
           label = 306;
          } else {
           $s9$0$i = $913;
          }
         }
         if ((label|0) == 306) {
          label = 0;
          HEAP8[$15>>0] = 48;
          $s9$0$i = $15;
         }
         $917 = ($d$6193$i|0)==($a$7$i|0);
         do {
          if ($917) {
           $921 = (($s9$0$i) + 1|0);
           (___fwritex($s9$0$i,1,$f)|0);
           $922 = ($$5194$i|0)>(0);
           $brmerge$i = $922 | $$not$i;
           if (!($brmerge$i)) {
            $s9$2$i = $921;
            break;
           }
           (___fwritex(77840,1,$f)|0);
           $s9$2$i = $921;
          } else {
           $918 = ($s9$0$i>>>0)>($buf$i>>>0);
           if ($918) {
            $s9$1189$i = $s9$0$i;
           } else {
            $s9$2$i = $s9$0$i;
            break;
           }
           while(1) {
            $919 = (($s9$1189$i) + -1|0);
            HEAP8[$919>>0] = 48;
            $920 = ($919>>>0)>($buf$i>>>0);
            if ($920) {
             $s9$1189$i = $919;
            } else {
             $s9$2$i = $919;
             break;
            }
           }
          }
         } while(0);
         $923 = $s9$2$i;
         $924 = (($14) - ($923))|0;
         $925 = ($924|0)<($$5194$i|0);
         $$$5$i = $925 ? $924 : $$5194$i;
         (___fwritex($s9$2$i,$$$5$i,$f)|0);
         $926 = (($$5194$i) - ($924))|0;
         $927 = (($d$6193$i) + 4|0);
         $928 = ($927>>>0)<($z$7$$i>>>0);
         $929 = ($926|0)>(-1);
         $or$cond30$i = $928 & $929;
         if ($or$cond30$i) {
          $$5194$i = $926;$d$6193$i = $927;
         } else {
          break;
         }
        }
        $930 = ($926|0)>(0);
        if (!($930)) {
         break;
        }
        $931 = ($926>>>0)>(256);
        $932 = $931 ? 256 : $926;
        _memset(($pad$i|0),48,($932|0))|0;
        $933 = ($926>>>0)>(255);
        if ($933) {
         $$01$i141$i = $926;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $934 = (($$01$i141$i) + -256)|0;
          $935 = ($934>>>0)>(255);
          if ($935) {
           $$01$i141$i = $934;
          } else {
           $$0$lcssa$i143$i = $934;
           break;
          }
         }
        } else {
         $$0$lcssa$i143$i = $926;
        }
        (___fwritex($pad$i,$$0$lcssa$i143$i,$f)|0);
       }
      } while(0);
      $936 = $estr$2$i;
      $937 = (($8) - ($936))|0;
      (___fwritex($estr$2$i,$937,$f)|0);
     }
    } while(0);
    $938 = ($843|0)==(8192);
    $or$cond$i$i = $938 & $845;
    if ($or$cond$i$i) {
     $939 = (($w$1) - ($842))|0;
     $940 = ($939>>>0)>(256);
     $941 = $940 ? 256 : $939;
     _memset(($pad$i|0),32,($941|0))|0;
     $942 = ($939>>>0)>(255);
     if ($942) {
      $$01$i$i = $939;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $943 = (($$01$i$i) + -256)|0;
       $944 = ($943>>>0)>(255);
       if ($944) {
        $$01$i$i = $943;
       } else {
        $$0$lcssa$i$i = $943;
        break;
       }
      }
     } else {
      $$0$lcssa$i$i = $939;
     }
     (___fwritex($pad$i,$$0$lcssa$i$i,$f)|0);
    }
    $w$31$i = $845 ? $w$1 : $842;
    $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$31$i;$l10n$0 = $l10n$3;
    continue L1;
    break;
   }
   case 109:  {
    $320 = (___errno_location()|0);
    $321 = HEAP32[$320>>2]|0;
    $322 = (_strerror(($321|0))|0);
    $a$1 = $322;
    label = 100;
    break;
   }
   case 105: case 100:  {
    $266 = $206;
    $267 = ($228|0)<(0);
    if ($267) {
     $268 = (_i64Subtract(0,0,($266|0),($228|0))|0);
     $269 = tempRet0;
     $270 = $268;
     $276 = $270;$278 = $269;$pl$0 = 1;$prefix$0 = 77744;
     label = 90;
     break L92;
    }
    $271 = $fl$1$ & 2048;
    $272 = ($271|0)==(0);
    if ($272) {
     $273 = $fl$1$ & 1;
     $274 = ($273|0)==(0);
     $$16 = $274 ? 77744 : ((77744 + 2|0));
     $276 = $206;$278 = $228;$pl$0 = $273;$prefix$0 = $$16;
     label = 90;
    } else {
     $276 = $206;$278 = $228;$pl$0 = 1;$prefix$0 = ((77744 + 1|0));
     label = 90;
    }
    break;
   }
   case 117:  {
    $276 = $206;$278 = $228;$pl$0 = 0;$prefix$0 = 77744;
    label = 90;
    break;
   }
   default: {
    $1054 = $228;$1055 = $206;$a$2 = $22;$fl$6 = $fl$1$;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 77744;$z$2 = $2;
   }
   }
  } while(0);
  L437: do {
   if ((label|0) == 78) {
    label = 0;
    $224 = $206;
    $225 = $t$1 & 32;
    $226 = ($224|0)==(0);
    $227 = ($228|0)==(0);
    $229 = $226 & $227;
    if ($229) {
     $307 = $206;$310 = $228;$a$0 = $2;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 77744;
     label = 95;
    } else {
     $$012$i = $2;$231 = $224;$238 = $228;
     while(1) {
      $230 = $231 & 15;
      $232 = (77824 + ($230)|0);
      $233 = HEAP8[$232>>0]|0;
      $234 = $233&255;
      $235 = $234 | $225;
      $236 = $235&255;
      $237 = (($$012$i) + -1|0);
      HEAP8[$237>>0] = $236;
      $239 = (_bitshift64Lshr(($231|0),($238|0),4)|0);
      $240 = tempRet0;
      $241 = ($239|0)==(0);
      $242 = ($240|0)==(0);
      $243 = $241 & $242;
      if ($243) {
       break;
      } else {
       $$012$i = $237;$231 = $239;$238 = $240;
      }
     }
     $244 = $fl$3 & 8;
     $245 = ($244|0)==(0);
     if ($245) {
      $307 = $206;$310 = $228;$a$0 = $237;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 77744;
      label = 95;
     } else {
      $246 = $t$1 >> 4;
      $247 = (77744 + ($246)|0);
      $307 = $206;$310 = $228;$a$0 = $237;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 2;$prefix$1 = $247;
      label = 95;
     }
    }
   }
   else if ((label|0) == 90) {
    label = 0;
    $275 = $276;
    $277 = ($278>>>0)>(0);
    $279 = ($278|0)==(0);
    $280 = ($275>>>0)>(4294967295);
    $281 = $279 & $280;
    $282 = $277 | $281;
    if ($282) {
     $$05$i = $2;$283 = $275;$284 = $278;
     while(1) {
      $285 = (___uremdi3(($283|0),($284|0),10,0)|0);
      $286 = tempRet0;
      $287 = $285 | 48;
      $288 = $287&255;
      $289 = (($$05$i) + -1|0);
      HEAP8[$289>>0] = $288;
      $290 = (___udivdi3(($283|0),($284|0),10,0)|0);
      $291 = tempRet0;
      $292 = ($284>>>0)>(9);
      $293 = ($284|0)==(9);
      $294 = ($283>>>0)>(4294967295);
      $295 = $293 & $294;
      $296 = $292 | $295;
      if ($296) {
       $$05$i = $289;$283 = $290;$284 = $291;
      } else {
       break;
      }
     }
     $$0$lcssa$i46 = $289;$$01$lcssa$off0$i = $290;
    } else {
     $$0$lcssa$i46 = $2;$$01$lcssa$off0$i = $275;
    }
    $297 = ($$01$lcssa$off0$i|0)==(0);
    if ($297) {
     $307 = $276;$310 = $278;$a$0 = $$0$lcssa$i46;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
     label = 95;
    } else {
     $$12$i = $$0$lcssa$i46;$y$03$i = $$01$lcssa$off0$i;
     while(1) {
      $298 = (($y$03$i>>>0) % 10)&-1;
      $299 = $298 | 48;
      $300 = $299&255;
      $301 = (($$12$i) + -1|0);
      HEAP8[$301>>0] = $300;
      $302 = (($y$03$i>>>0) / 10)&-1;
      $303 = ($y$03$i>>>0)<(10);
      if ($303) {
       $307 = $276;$310 = $278;$a$0 = $301;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
       label = 95;
       break;
      } else {
       $$12$i = $301;$y$03$i = $302;
      }
     }
    }
   }
   else if ((label|0) == 100) {
    label = 0;
    $324 = (_memchr($a$1,0,$p$0)|0);
    $325 = ($324|0)==(0|0);
    if ($325) {
     $326 = (($a$1) + ($p$0)|0);
     $1054 = $228;$1055 = $206;$a$2 = $a$1;$fl$6 = $205;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 77744;$z$2 = $326;
     break;
    } else {
     $327 = $324;
     $328 = $a$1;
     $329 = (($327) - ($328))|0;
     $1054 = $228;$1055 = $206;$a$2 = $a$1;$fl$6 = $205;$p$5 = $329;$pl$2 = 0;$prefix$2 = 77744;$z$2 = $324;
     break;
    }
   }
   else if ((label|0) == 105) {
    label = 0;
    $i$0169 = 0;$l$1168 = 0;$ws$0170 = $1056;
    while(1) {
     $335 = HEAP32[$ws$0170>>2]|0;
     $336 = ($335|0)==(0);
     if ($336) {
      $i$0$lcssa = $i$0169;$l$1$lcssa = $l$1168;
      break;
     }
     $337 = (_wctomb($mb,$335)|0);
     $338 = ($337|0)>(-1);
     if (!($338)) {
      $$0 = -1;
      label = 357;
      break L1;
     }
     $339 = (($p$4296) - ($i$0169))|0;
     $340 = ($337>>>0)>($339>>>0);
     $334 = (($337) + ($i$0169))|0;
     if ($340) {
      $1058 = $1056;$1059 = $1057;$i$0167 = $i$0169;
      label = 111;
      break L437;
     }
     $332 = (($ws$0170) + 4|0);
     $333 = ($334>>>0)<($p$4296>>>0);
     if ($333) {
      $i$0169 = $334;$l$1168 = $337;$ws$0170 = $332;
     } else {
      $i$0$lcssa = $334;$l$1$lcssa = $337;
      break;
     }
    }
    $341 = ($l$1$lcssa|0)<(0);
    if ($341) {
     $$0 = -1;
     label = 357;
     break L1;
    } else {
     $1058 = $1056;$1059 = $1057;$i$0167 = $i$0$lcssa;
     label = 111;
    }
   }
  } while(0);
  if ((label|0) == 95) {
   label = 0;
   $304 = ($p$2|0)>(-1);
   $305 = $fl$4 & -65537;
   $$fl$4 = $304 ? $305 : $fl$4;
   $306 = $307;
   $308 = ($306|0)==(0);
   $309 = ($310|0)==(0);
   $311 = $308 & $309;
   $312 = ($p$2|0)==(0);
   $or$cond = $311 & $312;
   if ($or$cond) {
    $1054 = $310;$1055 = $307;$a$2 = $2;$fl$6 = $$fl$4;$p$5 = 0;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   } else {
    $313 = $a$0;
    $314 = (($3) - ($313))|0;
    $315 = $311&1;
    $316 = (($315) + ($314))|0;
    $317 = ($p$2|0)>($316|0);
    $p$2$ = $317 ? $p$2 : $316;
    $1054 = $310;$1055 = $307;$a$2 = $a$0;$fl$6 = $$fl$4;$p$5 = $p$2$;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   }
  }
  else if ((label|0) == 111) {
   label = 0;
   $342 = $fl$1$ & 73728;
   $343 = ($342|0)==(0);
   $344 = ($i$0167|0)<($w$1|0);
   $or$cond$i57 = $343 & $344;
   if ($or$cond$i57) {
    $345 = (($w$1) - ($i$0167))|0;
    $346 = ($345>>>0)>(256);
    $347 = $346 ? 256 : $345;
    _memset(($pad$i|0),32,($347|0))|0;
    $348 = ($345>>>0)>(255);
    if ($348) {
     $$01$i58 = $345;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $349 = (($$01$i58) + -256)|0;
      $350 = ($349>>>0)>(255);
      if ($350) {
       $$01$i58 = $349;
      } else {
       $$0$lcssa$i60 = $349;
       break;
      }
     }
    } else {
     $$0$lcssa$i60 = $345;
    }
    (___fwritex($pad$i,$$0$lcssa$i60,$f)|0);
   }
   $351 = ($i$0167|0)==(0);
   L473: do {
    if (!($351)) {
     $i$1175 = 0;$ws$1176 = $1058;
     while(1) {
      $352 = HEAP32[$ws$1176>>2]|0;
      $353 = ($352|0)==(0);
      if ($353) {
       break L473;
      }
      $354 = (_wctomb($mb,$352)|0);
      $355 = (($354) + ($i$1175))|0;
      $356 = ($355|0)>($i$0167|0);
      if ($356) {
       break L473;
      }
      $357 = (($ws$1176) + 4|0);
      (___fwritex($mb,$354,$f)|0);
      $358 = ($355>>>0)<($i$0167>>>0);
      if ($358) {
       $i$1175 = $355;$ws$1176 = $357;
      } else {
       break;
      }
     }
    }
   } while(0);
   $359 = ($342|0)==(8192);
   $or$cond$i64 = $359 & $344;
   if ($or$cond$i64) {
    $360 = (($w$1) - ($i$0167))|0;
    $361 = ($360>>>0)>(256);
    $362 = $361 ? 256 : $360;
    _memset(($pad$i|0),32,($362|0))|0;
    $363 = ($360>>>0)>(255);
    if ($363) {
     $$01$i65 = $360;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $364 = (($$01$i65) + -256)|0;
      $365 = ($364>>>0)>(255);
      if ($365) {
       $$01$i65 = $364;
      } else {
       $$0$lcssa$i67 = $364;
       break;
      }
     }
    } else {
     $$0$lcssa$i67 = $360;
    }
    (___fwritex($pad$i,$$0$lcssa$i67,$f)|0);
   }
   $366 = $344 ? $w$1 : $i$0167;
   $1048 = $228;$1049 = $1059;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $366;$l10n$0 = $l10n$3;
   continue;
  }
  $945 = $z$2;
  $946 = $a$2;
  $947 = (($945) - ($946))|0;
  $948 = ($p$5|0)<($947|0);
  $$p$5 = $948 ? $947 : $p$5;
  $949 = (($pl$2) + ($$p$5))|0;
  $950 = ($w$1|0)<($949|0);
  $w$2 = $950 ? $949 : $w$1;
  $951 = $fl$6 & 73728;
  $952 = ($951|0)==(0);
  $953 = ($949|0)<($w$2|0);
  $or$cond$i75 = $952 & $953;
  if ($or$cond$i75) {
   $954 = (($w$2) - ($949))|0;
   $955 = ($954>>>0)>(256);
   $956 = $955 ? 256 : $954;
   _memset(($pad$i|0),32,($956|0))|0;
   $957 = ($954>>>0)>(255);
   if ($957) {
    $$01$i76 = $954;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $958 = (($$01$i76) + -256)|0;
     $959 = ($958>>>0)>(255);
     if ($959) {
      $$01$i76 = $958;
     } else {
      $$0$lcssa$i78 = $958;
      break;
     }
    }
   } else {
    $$0$lcssa$i78 = $954;
   }
   (___fwritex($pad$i,$$0$lcssa$i78,$f)|0);
  }
  (___fwritex($prefix$2,$pl$2,$f)|0);
  $960 = ($951|0)==(65536);
  $or$cond$i50 = $960 & $953;
  if ($or$cond$i50) {
   $961 = (($w$2) - ($949))|0;
   $962 = ($961>>>0)>(256);
   $963 = $962 ? 256 : $961;
   _memset(($pad$i|0),48,($963|0))|0;
   $964 = ($961>>>0)>(255);
   if ($964) {
    $$01$i51 = $961;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $965 = (($$01$i51) + -256)|0;
     $966 = ($965>>>0)>(255);
     if ($966) {
      $$01$i51 = $965;
     } else {
      $$0$lcssa$i53 = $965;
      break;
     }
    }
   } else {
    $$0$lcssa$i53 = $961;
   }
   (___fwritex($pad$i,$$0$lcssa$i53,$f)|0);
  }
  $967 = ($947|0)<($$p$5|0);
  if ($967) {
   $968 = (($$p$5) - ($947))|0;
   $969 = ($968>>>0)>(256);
   $970 = $969 ? 256 : $968;
   _memset(($pad$i|0),48,($970|0))|0;
   $971 = ($968>>>0)>(255);
   if ($971) {
    $$01$i36 = $968;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $972 = (($$01$i36) + -256)|0;
     $973 = ($972>>>0)>(255);
     if ($973) {
      $$01$i36 = $972;
     } else {
      $$0$lcssa$i38 = $972;
      break;
     }
    }
   } else {
    $$0$lcssa$i38 = $968;
   }
   (___fwritex($pad$i,$$0$lcssa$i38,$f)|0);
  }
  (___fwritex($a$2,$947,$f)|0);
  $974 = ($951|0)==(8192);
  $or$cond$i = $974 & $953;
  if (!($or$cond$i)) {
   $1048 = $1054;$1049 = $1055;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
   continue;
  }
  $975 = (($w$2) - ($949))|0;
  $976 = ($975>>>0)>(256);
  $977 = $976 ? 256 : $975;
  _memset(($pad$i|0),32,($977|0))|0;
  $978 = ($975>>>0)>(255);
  if ($978) {
   $$01$i = $975;
   while(1) {
    (___fwritex($pad$i,256,$f)|0);
    $979 = (($$01$i) + -256)|0;
    $980 = ($979>>>0)>(255);
    if ($980) {
     $$01$i = $979;
    } else {
     $$0$lcssa$i = $979;
     break;
    }
   }
  } else {
   $$0$lcssa$i = $975;
  }
  (___fwritex($pad$i,$$0$lcssa$i,$f)|0);
  $1048 = $1054;$1049 = $1055;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
 }
 if ((label|0) == 339) {
  $981 = ($f|0)==(0|0);
  if (!($981)) {
   $$0 = $cnt$1;
   STACKTOP = sp;return ($$0|0);
  }
  $982 = ($l10n$0|0)==(0);
  if ($982) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  } else {
   $i$291 = 1;
  }
  while(1) {
   $983 = (($nl_type) + ($i$291<<2)|0);
   $984 = HEAP32[$983>>2]|0;
   $985 = ($984|0)==(0);
   if ($985) {
    $i$390 = $i$291;
    break;
   }
   $986 = (($nl_arg) + ($i$291<<3)|0);
   $987 = ($984>>>0)>(20);
   L532: do {
    if (!($987)) {
     do {
      switch ($984|0) {
      case 9:  {
       $arglist_current35 = HEAP32[$ap>>2]|0;
       $988 = HEAP32[$arglist_current35>>2]|0;
       $arglist_next36 = (($arglist_current35) + 4|0);
       HEAP32[$ap>>2] = $arglist_next36;
       HEAP32[$986>>2] = $988;
       break L532;
       break;
      }
      case 10:  {
       $arglist_current38 = HEAP32[$ap>>2]|0;
       $989 = HEAP32[$arglist_current38>>2]|0;
       $arglist_next39 = (($arglist_current38) + 4|0);
       HEAP32[$ap>>2] = $arglist_next39;
       $990 = ($989|0)<(0);
       $991 = $990 << 31 >> 31;
       $992 = $986;
       $993 = $992;
       HEAP32[$993>>2] = $989;
       $994 = (($992) + 4)|0;
       $995 = $994;
       HEAP32[$995>>2] = $991;
       break L532;
       break;
      }
      case 11:  {
       $arglist_current41 = HEAP32[$ap>>2]|0;
       $996 = HEAP32[$arglist_current41>>2]|0;
       $arglist_next42 = (($arglist_current41) + 4|0);
       HEAP32[$ap>>2] = $arglist_next42;
       $997 = $986;
       $998 = $997;
       HEAP32[$998>>2] = $996;
       $999 = (($997) + 4)|0;
       $1000 = $999;
       HEAP32[$1000>>2] = 0;
       break L532;
       break;
      }
      case 12:  {
       $arglist_current44 = HEAP32[$ap>>2]|0;
       $1001 = $arglist_current44;
       $1002 = $1001;
       $1003 = HEAP32[$1002>>2]|0;
       $1004 = (($1001) + 4)|0;
       $1005 = $1004;
       $1006 = HEAP32[$1005>>2]|0;
       $arglist_next45 = (($arglist_current44) + 8|0);
       HEAP32[$ap>>2] = $arglist_next45;
       $1007 = $986;
       $1008 = $1007;
       HEAP32[$1008>>2] = $1003;
       $1009 = (($1007) + 4)|0;
       $1010 = $1009;
       HEAP32[$1010>>2] = $1006;
       break L532;
       break;
      }
      case 13:  {
       $arglist_current47 = HEAP32[$ap>>2]|0;
       $1011 = HEAP32[$arglist_current47>>2]|0;
       $arglist_next48 = (($arglist_current47) + 4|0);
       HEAP32[$ap>>2] = $arglist_next48;
       $1012 = $1011&65535;
       $1013 = $1012 << 16 >> 16;
       $1014 = ($1013|0)<(0);
       $1015 = $1014 << 31 >> 31;
       $1016 = $986;
       $1017 = $1016;
       HEAP32[$1017>>2] = $1013;
       $1018 = (($1016) + 4)|0;
       $1019 = $1018;
       HEAP32[$1019>>2] = $1015;
       break L532;
       break;
      }
      case 14:  {
       $arglist_current50 = HEAP32[$ap>>2]|0;
       $1020 = HEAP32[$arglist_current50>>2]|0;
       $arglist_next51 = (($arglist_current50) + 4|0);
       HEAP32[$ap>>2] = $arglist_next51;
       $$mask1$i = $1020 & 65535;
       $1021 = $986;
       $1022 = $1021;
       HEAP32[$1022>>2] = $$mask1$i;
       $1023 = (($1021) + 4)|0;
       $1024 = $1023;
       HEAP32[$1024>>2] = 0;
       break L532;
       break;
      }
      case 15:  {
       $arglist_current53 = HEAP32[$ap>>2]|0;
       $1025 = HEAP32[$arglist_current53>>2]|0;
       $arglist_next54 = (($arglist_current53) + 4|0);
       HEAP32[$ap>>2] = $arglist_next54;
       $1026 = $1025&255;
       $1027 = $1026 << 24 >> 24;
       $1028 = ($1027|0)<(0);
       $1029 = $1028 << 31 >> 31;
       $1030 = $986;
       $1031 = $1030;
       HEAP32[$1031>>2] = $1027;
       $1032 = (($1030) + 4)|0;
       $1033 = $1032;
       HEAP32[$1033>>2] = $1029;
       break L532;
       break;
      }
      case 16:  {
       $arglist_current56 = HEAP32[$ap>>2]|0;
       $1034 = HEAP32[$arglist_current56>>2]|0;
       $arglist_next57 = (($arglist_current56) + 4|0);
       HEAP32[$ap>>2] = $arglist_next57;
       $$mask$i = $1034 & 255;
       $1035 = $986;
       $1036 = $1035;
       HEAP32[$1036>>2] = $$mask$i;
       $1037 = (($1035) + 4)|0;
       $1038 = $1037;
       HEAP32[$1038>>2] = 0;
       break L532;
       break;
      }
      case 17:  {
       $arglist_current59 = HEAP32[$ap>>2]|0;
       HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current59>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current59+4>>2];$1039 = +HEAPF64[tempDoublePtr>>3];
       $arglist_next60 = (($arglist_current59) + 8|0);
       HEAP32[$ap>>2] = $arglist_next60;
       HEAPF64[$986>>3] = $1039;
       break L532;
       break;
      }
      case 18:  {
       $arglist_current62 = HEAP32[$ap>>2]|0;
       HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current62>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current62+4>>2];$1040 = +HEAPF64[tempDoublePtr>>3];
       $arglist_next63 = (($arglist_current62) + 8|0);
       HEAP32[$ap>>2] = $arglist_next63;
       HEAPF64[$986>>3] = $1040;
       break L532;
       break;
      }
      default: {
       break L532;
      }
      }
     } while(0);
    }
   } while(0);
   $1041 = (($i$291) + 1)|0;
   $1042 = ($1041|0)<(10);
   if ($1042) {
    $i$291 = $1041;
   } else {
    $$0 = 1;
    label = 357;
    break;
   }
  }
  if ((label|0) == 357) {
   STACKTOP = sp;return ($$0|0);
  }
  while(1) {
   $1045 = (($nl_type) + ($i$390<<2)|0);
   $1046 = HEAP32[$1045>>2]|0;
   $1047 = ($1046|0)==(0);
   $1044 = (($i$390) + 1)|0;
   if (!($1047)) {
    $$0 = -1;
    label = 357;
    break;
   }
   $1043 = ($1044|0)<(10);
   if ($1043) {
    $i$390 = $1044;
   } else {
    $$0 = 1;
    label = 357;
    break;
   }
  }
  if ((label|0) == 357) {
   STACKTOP = sp;return ($$0|0);
  }
 }
 else if ((label|0) == 357) {
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _vsnprintf($s,$n,$fmt,$ap) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$$02 = 0, $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $f = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $b = sp + 112|0;
 $f = sp;
 dest=$f+0|0; src=77848+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $0 = (($n) + -1)|0;
 $1 = ($0>>>0)>(2147483646);
 if ($1) {
  $2 = ($n|0)==(0);
  if ($2) {
   $$01 = $b;$$02 = 1;
  } else {
   $3 = (___errno_location()|0);
   HEAP32[$3>>2] = 75;
   $$0 = -1;
   STACKTOP = sp;return ($$0|0);
  }
 } else {
  $$01 = $s;$$02 = $n;
 }
 $4 = $$01;
 $5 = (-2 - ($4))|0;
 $6 = ($$02>>>0)>($5>>>0);
 $$$02 = $6 ? $5 : $$02;
 $7 = (($f) + 48|0);
 HEAP32[$7>>2] = $$$02;
 $8 = (($f) + 20|0);
 HEAP32[$8>>2] = $$01;
 $9 = (($f) + 44|0);
 HEAP32[$9>>2] = $$01;
 $10 = (($$01) + ($$$02)|0);
 $11 = (($f) + 16|0);
 HEAP32[$11>>2] = $10;
 $12 = (($f) + 28|0);
 HEAP32[$12>>2] = $10;
 $13 = (_MUSL_vfprintf($f,$fmt,$ap)|0);
 $14 = ($$$02|0)==(0);
 if ($14) {
  $$0 = $13;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = HEAP32[$8>>2]|0;
 $16 = HEAP32[$11>>2]|0;
 $17 = ($15|0)==($16|0);
 $18 = $17 << 31 >> 31;
 $19 = (($15) + ($18)|0);
 HEAP8[$19>>0] = 0;
 $$0 = $13;
 STACKTOP = sp;return ($$0|0);
}
function _sn_write($f,$s,$l) {
 $f = $f|0;
 $s = $s|0;
 $l = $l|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($f) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $1;
 $5 = $3;
 $6 = (($4) - ($5))|0;
 $7 = ($6>>>0)>($l>>>0);
 $l$ = $7 ? $l : $6;
 _memcpy(($3|0),($s|0),($l$|0))|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($l$)|0);
 HEAP32[$2>>2] = $9;
 STACKTOP = sp;return ($l|0);
}
function _vsprintf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_vsnprintf($s,2147483647,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _atoi($s) {
 $s = $s|0;
 var $$0 = 0, $$1$ph = 0, $$12 = 0, $$neg1 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n$0$lcssa = 0, $n$03 = 0, $neg$0 = 0, $neg$1$ph = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $$0 = $s;
 while(1) {
  $0 = HEAP8[$$0>>0]|0;
  $1 = $0 << 24 >> 24;
  $2 = (_isspace($1)|0);
  $3 = ($2|0)==(0);
  $4 = (($$0) + 1|0);
  if ($3) {
   break;
  } else {
   $$0 = $4;
  }
 }
 $5 = HEAP8[$$0>>0]|0;
 $6 = $5 << 24 >> 24;
 if ((($6|0) == 43)) {
  $neg$0 = 0;
  label = 5;
 } else if ((($6|0) == 45)) {
  $neg$0 = 1;
  label = 5;
 } else {
  $$1$ph = $$0;$8 = $5;$neg$1$ph = 0;
 }
 if ((label|0) == 5) {
  $$pre = HEAP8[$4>>0]|0;
  $$1$ph = $4;$8 = $$pre;$neg$1$ph = $neg$0;
 }
 $7 = $8 << 24 >> 24;
 $9 = (_isdigit($7)|0);
 $10 = ($9|0)==(0);
 if ($10) {
  $n$0$lcssa = 0;
  $20 = ($neg$1$ph|0)!=(0);
  $21 = (0 - ($n$0$lcssa))|0;
  $22 = $20 ? $n$0$lcssa : $21;
  STACKTOP = sp;return ($22|0);
 } else {
  $$12 = $$1$ph;$n$03 = 0;
 }
 while(1) {
  $11 = ($n$03*10)|0;
  $12 = (($$12) + 1|0);
  $13 = HEAP8[$$12>>0]|0;
  $14 = $13 << 24 >> 24;
  $$neg1 = (($11) + 48)|0;
  $15 = (($$neg1) - ($14))|0;
  $16 = HEAP8[$12>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = (_isdigit($17)|0);
  $19 = ($18|0)==(0);
  if ($19) {
   $n$0$lcssa = $15;
   break;
  } else {
   $$12 = $12;$n$03 = $15;
  }
 }
 $20 = ($neg$1$ph|0)!=(0);
 $21 = (0 - ($n$0$lcssa))|0;
 $22 = $20 ? $n$0$lcssa : $21;
 STACKTOP = sp;return ($22|0);
}
function _strtod($s,$p) {
 $s = $s|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $f$i = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $f$i = sp;
 dest=$f$i+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $0 = (($f$i) + 4|0);
 HEAP32[$0>>2] = $s;
 $1 = (($f$i) + 8|0);
 HEAP32[$1>>2] = (-1);
 $2 = (($f$i) + 44|0);
 HEAP32[$2>>2] = $s;
 $3 = (($f$i) + 76|0);
 HEAP32[$3>>2] = -1;
 ___shlim($f$i,0);
 $4 = (+___floatscan($f$i,1,1));
 $5 = (($f$i) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = $7;
 $10 = $8;
 $11 = (($9) - ($10))|0;
 $12 = (($11) + ($6))|0;
 $13 = ($p|0)==(0|0);
 if ($13) {
  STACKTOP = sp;return (+$4);
 }
 $14 = ($12|0)==(0);
 if ($14) {
  $16 = $s;
 } else {
  $15 = (($s) + ($12)|0);
  $16 = $15;
 }
 HEAP32[$p>>2] = $16;
 STACKTOP = sp;return (+$4);
}
function _memchr($src,$c,$n) {
 $src = $src|0;
 $c = $c|0;
 $n = $n|0;
 var $$0$lcssa = 0, $$0$lcssa34 = 0, $$013 = 0, $$1$lcssa = 0, $$17 = 0, $$24 = 0, $$3 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond12 = 0, $s$0$lcssa = 0, $s$0$lcssa33 = 0, $s$014 = 0, $s$15 = 0, $s$2 = 0, $w$0$lcssa = 0, $w$08 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = $src;
 $2 = $1 & 3;
 $3 = ($2|0)==(0);
 $4 = ($n|0)==(0);
 $or$cond12 = $3 | $4;
 L1: do {
  if ($or$cond12) {
   $$0$lcssa = $n;$$lcssa = $4;$s$0$lcssa = $src;
   label = 5;
  } else {
   $5 = $c&255;
   $$013 = $n;$s$014 = $src;
   while(1) {
    $6 = HEAP8[$s$014>>0]|0;
    $7 = ($6<<24>>24)==($5<<24>>24);
    if ($7) {
     $$0$lcssa34 = $$013;$s$0$lcssa33 = $s$014;
     label = 6;
     break L1;
    }
    $8 = (($s$014) + 1|0);
    $9 = (($$013) + -1)|0;
    $10 = $8;
    $11 = $10 & 3;
    $12 = ($11|0)==(0);
    $13 = ($9|0)==(0);
    $or$cond = $12 | $13;
    if ($or$cond) {
     $$0$lcssa = $9;$$lcssa = $13;$s$0$lcssa = $8;
     label = 5;
     break;
    } else {
     $$013 = $9;$s$014 = $8;
    }
   }
  }
 } while(0);
 if ((label|0) == 5) {
  if ($$lcssa) {
   $$3 = 0;$s$2 = $s$0$lcssa;
  } else {
   $$0$lcssa34 = $$0$lcssa;$s$0$lcssa33 = $s$0$lcssa;
   label = 6;
  }
 }
 L8: do {
  if ((label|0) == 6) {
   $14 = HEAP8[$s$0$lcssa33>>0]|0;
   $15 = $c&255;
   $16 = ($14<<24>>24)==($15<<24>>24);
   if ($16) {
    $$3 = $$0$lcssa34;$s$2 = $s$0$lcssa33;
   } else {
    $17 = Math_imul($0, 16843009)|0;
    $18 = ($$0$lcssa34>>>0)>(3);
    L11: do {
     if ($18) {
      $$17 = $$0$lcssa34;$w$08 = $s$0$lcssa33;
      while(1) {
       $19 = HEAP32[$w$08>>2]|0;
       $20 = $19 ^ $17;
       $21 = (($20) + -16843009)|0;
       $22 = $20 & -2139062144;
       $23 = $22 ^ -2139062144;
       $24 = $23 & $21;
       $25 = ($24|0)==(0);
       if (!($25)) {
        $$1$lcssa = $$17;$w$0$lcssa = $w$08;
        break L11;
       }
       $26 = (($w$08) + 4|0);
       $27 = (($$17) + -4)|0;
       $28 = ($27>>>0)>(3);
       if ($28) {
        $$17 = $27;$w$08 = $26;
       } else {
        $$1$lcssa = $27;$w$0$lcssa = $26;
        break;
       }
      }
     } else {
      $$1$lcssa = $$0$lcssa34;$w$0$lcssa = $s$0$lcssa33;
     }
    } while(0);
    $29 = ($$1$lcssa|0)==(0);
    if ($29) {
     $$3 = 0;$s$2 = $w$0$lcssa;
    } else {
     $$24 = $$1$lcssa;$s$15 = $w$0$lcssa;
     while(1) {
      $30 = HEAP8[$s$15>>0]|0;
      $31 = ($30<<24>>24)==($15<<24>>24);
      if ($31) {
       $$3 = $$24;$s$2 = $s$15;
       break L8;
      }
      $32 = (($s$15) + 1|0);
      $33 = (($$24) + -1)|0;
      $34 = ($33|0)==(0);
      if ($34) {
       $$3 = 0;$s$2 = $32;
       break;
      } else {
       $$24 = $33;$s$15 = $32;
      }
     }
    }
   }
  }
 } while(0);
 $35 = ($$3|0)!=(0);
 $36 = $35 ? $s$2 : 0;
 STACKTOP = sp;return ($36|0);
}
function _memcmp($vl,$vr,$n) {
 $vl = $vl|0;
 $vr = $vr|0;
 $n = $n|0;
 var $$03 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$04 = 0, $r$05 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 L1: do {
  if ($0) {
   $11 = 0;
  } else {
   $$03 = $n;$l$04 = $vl;$r$05 = $vr;
   while(1) {
    $1 = HEAP8[$l$04>>0]|0;
    $2 = HEAP8[$r$05>>0]|0;
    $3 = ($1<<24>>24)==($2<<24>>24);
    if (!($3)) {
     break;
    }
    $4 = (($$03) + -1)|0;
    $5 = (($l$04) + 1|0);
    $6 = (($r$05) + 1|0);
    $7 = ($4|0)==(0);
    if ($7) {
     $11 = 0;
     break L1;
    } else {
     $$03 = $4;$l$04 = $5;$r$05 = $6;
    }
   }
   $8 = $1&255;
   $9 = $2&255;
   $10 = (($8) - ($9))|0;
   $11 = $10;
  }
 } while(0);
 STACKTOP = sp;return ($11|0);
}
function _strcmp($l,$r) {
 $l = $l|0;
 $r = $r|0;
 var $$027 = 0, $$08 = 0, $$lcssa = 0, $$lcssa4 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond36 = 0, $or$cond5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP8[$l>>0]|0;
 $1 = HEAP8[$r>>0]|0;
 $2 = ($0<<24>>24)!=($1<<24>>24);
 $3 = ($0<<24>>24)==(0);
 $or$cond5 = $2 | $3;
 $4 = ($1<<24>>24)==(0);
 $or$cond36 = $or$cond5 | $4;
 if ($or$cond36) {
  $$lcssa = $0;$$lcssa4 = $1;
  $12 = $$lcssa&255;
  $13 = $$lcssa4&255;
  $14 = (($12) - ($13))|0;
  STACKTOP = sp;return ($14|0);
 } else {
  $$027 = $l;$$08 = $r;
 }
 while(1) {
  $5 = (($$027) + 1|0);
  $6 = (($$08) + 1|0);
  $7 = HEAP8[$5>>0]|0;
  $8 = HEAP8[$6>>0]|0;
  $9 = ($7<<24>>24)!=($8<<24>>24);
  $10 = ($7<<24>>24)==(0);
  $or$cond = $9 | $10;
  $11 = ($8<<24>>24)==(0);
  $or$cond3 = $or$cond | $11;
  if ($or$cond3) {
   $$lcssa = $7;$$lcssa4 = $8;
   break;
  } else {
   $$027 = $5;$$08 = $6;
  }
 }
 $12 = $$lcssa&255;
 $13 = $$lcssa4&255;
 $14 = (($12) - ($13))|0;
 STACKTOP = sp;return ($14|0);
}
function _strncasecmp($_l,$_r,$n) {
 $_l = $_l|0;
 $_r = $_r|0;
 $n = $n|0;
 var $$04 = 0, $$08 = 0, $$08$in = 0, $$pre$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$06 = 0, $or$cond = 0, $r$0$lcssa = 0, $r$07 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 if ($0) {
  $$04 = 0;
  STACKTOP = sp;return ($$04|0);
 }
 $1 = HEAP8[$_l>>0]|0;
 $2 = ($1<<24>>24)==(0);
 L4: do {
  if ($2) {
   $21 = 0;$r$0$lcssa = $_r;
  } else {
   $3 = $1&255;
   $$08$in = $n;$8 = $1;$9 = $3;$l$06 = $_l;$r$07 = $_r;
   while(1) {
    $$08 = (($$08$in) + -1)|0;
    $4 = HEAP8[$r$07>>0]|0;
    $5 = ($4<<24>>24)==(0);
    $6 = ($$08|0)==(0);
    $or$cond = $5 | $6;
    if ($or$cond) {
     $21 = $8;$r$0$lcssa = $r$07;
     break L4;
    }
    $7 = ($8<<24>>24)==($4<<24>>24);
    if (!($7)) {
     $10 = (_tolower($9)|0);
     $11 = HEAP8[$r$07>>0]|0;
     $12 = $11&255;
     $13 = (_tolower($12)|0);
     $14 = ($10|0)==($13|0);
     if (!($14)) {
      break;
     }
    }
    $15 = (($l$06) + 1|0);
    $16 = (($r$07) + 1|0);
    $17 = HEAP8[$15>>0]|0;
    $18 = $17&255;
    $19 = ($17<<24>>24)==(0);
    if ($19) {
     $21 = 0;$r$0$lcssa = $16;
     break L4;
    } else {
     $$08$in = $$08;$8 = $17;$9 = $18;$l$06 = $15;$r$07 = $16;
    }
   }
   $$pre$pre = HEAP8[$l$06>>0]|0;
   $21 = $$pre$pre;$r$0$lcssa = $r$07;
  }
 } while(0);
 $20 = $21&255;
 $22 = (_tolower($20)|0);
 $23 = HEAP8[$r$0$lcssa>>0]|0;
 $24 = $23&255;
 $25 = (_tolower($24)|0);
 $26 = (($22) - ($25))|0;
 $$04 = $26;
 STACKTOP = sp;return ($$04|0);
}
function _strncmp($_l,$_r,$n) {
 $_l = $_l|0;
 $_r = $_r|0;
 $n = $n|0;
 var $$03 = 0, $$08 = 0, $$08$in = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$06 = 0, $notlhs = 0;
 var $notrhs = 0, $or$cond$not = 0, $or$cond4 = 0, $r$0$lcssa = 0, $r$07 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 if ($0) {
  $$03 = 0;
  STACKTOP = sp;return ($$03|0);
 }
 $1 = HEAP8[$_l>>0]|0;
 $2 = ($1<<24>>24)==(0);
 L4: do {
  if ($2) {
   $11 = 0;$r$0$lcssa = $_r;
  } else {
   $$08$in = $n;$5 = $1;$l$06 = $_l;$r$07 = $_r;
   while(1) {
    $$08 = (($$08$in) + -1)|0;
    $3 = HEAP8[$r$07>>0]|0;
    $notlhs = ($3<<24>>24)!=(0);
    $notrhs = ($$08|0)!=(0);
    $or$cond$not = $notrhs & $notlhs;
    $4 = ($5<<24>>24)==($3<<24>>24);
    $or$cond4 = $or$cond$not & $4;
    if (!($or$cond4)) {
     $11 = $5;$r$0$lcssa = $r$07;
     break L4;
    }
    $6 = (($l$06) + 1|0);
    $7 = (($r$07) + 1|0);
    $8 = HEAP8[$6>>0]|0;
    $9 = ($8<<24>>24)==(0);
    if ($9) {
     $11 = 0;$r$0$lcssa = $7;
     break;
    } else {
     $$08$in = $$08;$5 = $8;$l$06 = $6;$r$07 = $7;
    }
   }
  }
 } while(0);
 $10 = $11&255;
 $12 = HEAP8[$r$0$lcssa>>0]|0;
 $13 = $12&255;
 $14 = (($10) - ($13))|0;
 $$03 = $14;
 STACKTOP = sp;return ($$03|0);
}
function runPostSets() {
 
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _strlen(ptr) {
    ptr = ptr|0;
    var curr = 0;
    curr = ptr;
    while (((HEAP8[((curr)>>0)])|0)) {
      curr = (curr + 1)|0;
    }
    return (curr - ptr)|0;
}
function _strcat(pdest, psrc) {
    pdest = pdest|0; psrc = psrc|0;
    var i = 0;
    var pdestEnd = 0;
    pdestEnd = (pdest + (_strlen(pdest)|0))|0;
    do {
      HEAP8[((pdestEnd+i)>>0)]=HEAP8[((psrc+i)>>0)];
      i = (i+1)|0;
    } while (((HEAP8[(((psrc)+(i-1))>>0)])|0));
    return pdest|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _saveSetjmp(env, label, table) {
    // Not particularly fast: slow table lookup of setjmpId to label. But setjmp
    // prevents relooping anyhow, so slowness is to be expected. And typical case
    // is 1 setjmp per invocation, or less.
    env = env|0;
    label = label|0;
    table = table|0;
    var i = 0;
    setjmpId = (setjmpId+1)|0;
    HEAP32[((env)>>2)]=setjmpId;
    while ((i|0) < 20) {
      if (((HEAP32[(((table)+((i<<3)))>>2)])|0) == 0) {
        HEAP32[(((table)+((i<<3)))>>2)]=setjmpId;
        HEAP32[(((table)+((i<<3)+4))>>2)]=label;
        // prepare next slot
        HEAP32[(((table)+((i<<3)+8))>>2)]=0;
        return 0;
      }
      i = i+1|0;
    }
    _putchar(116);_putchar(111);_putchar(111);_putchar(32);_putchar(109);_putchar(97);_putchar(110);_putchar(121);_putchar(32);_putchar(115);_putchar(101);_putchar(116);_putchar(106);_putchar(109);_putchar(112);_putchar(115);_putchar(32);_putchar(105);_putchar(110);_putchar(32);_putchar(97);_putchar(32);_putchar(102);_putchar(117);_putchar(110);_putchar(99);_putchar(116);_putchar(105);_putchar(111);_putchar(110);_putchar(32);_putchar(99);_putchar(97);_putchar(108);_putchar(108);_putchar(44);_putchar(32);_putchar(98);_putchar(117);_putchar(105);_putchar(108);_putchar(100);_putchar(32);_putchar(119);_putchar(105);_putchar(116);_putchar(104);_putchar(32);_putchar(97);_putchar(32);_putchar(104);_putchar(105);_putchar(103);_putchar(104);_putchar(101);_putchar(114);_putchar(32);_putchar(118);_putchar(97);_putchar(108);_putchar(117);_putchar(101);_putchar(32);_putchar(102);_putchar(111);_putchar(114);_putchar(32);_putchar(77);_putchar(65);_putchar(88);_putchar(95);_putchar(83);_putchar(69);_putchar(84);_putchar(74);_putchar(77);_putchar(80);_putchar(83);_putchar(10);
    abort(0);
    return 0;
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _testSetjmp(id, table) {
    id = id|0;
    table = table|0;
    var i = 0, curr = 0;
    while ((i|0) < 20) {
      curr = ((HEAP32[(((table)+((i<<3)))>>2)])|0);
      if ((curr|0) == 0) break;
      if ((curr|0) == (id|0)) {
        return ((HEAP32[(((table)+((i<<3)+4))>>2)])|0);
      }
      i = i+1|0;
    }
    return 0;
}
function _memcpy(dest, src, num) {

    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _strcpy(pdest, psrc) {
    pdest = pdest|0; psrc = psrc|0;
    var i = 0;
    do {
      HEAP8[(((pdest+i)|0)>>0)]=HEAP8[(((psrc+i)|0)>>0)];
      i = (i+1)|0;
    } while (((HEAP8[(((psrc)+(i-1))>>0)])|0));
    return pdest|0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_ctlz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((ctlz_i8)+(x >>> 24))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((ctlz_i8)+(x&0xff))>>0)])|0) + 24)|0;
  }

function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return (tempRet0 = tempRet0, $10$0) | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return (tempRet0 = tempRet0, $1$0) | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (_llvm_ctlz_i32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0, $d_sroa_0_0_insert_insert99$1, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



// EMSCRIPTEN_END_FUNCS

    
    function dynCall_iiii(index,a1,a2,a3) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0;
      return FUNCTION_TABLE_iiii[index&255](a1|0,a2|0,a3|0)|0;
    }
  

    function dynCall_viiiiiii(index,a1,a2,a3,a4,a5,a6,a7) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0; a7=a7|0;
      FUNCTION_TABLE_viiiiiii[index&63](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0,a7|0);
    }
  

    function dynCall_viiiii(index,a1,a2,a3,a4,a5) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
      FUNCTION_TABLE_viiiii[index&127](a1|0,a2|0,a3|0,a4|0,a5|0);
    }
  

    function dynCall_vi(index,a1) {
      index = index|0;
      a1=a1|0;
      FUNCTION_TABLE_vi[index&255](a1|0);
    }
  

    function dynCall_vii(index,a1,a2) {
      index = index|0;
      a1=a1|0; a2=a2|0;
      FUNCTION_TABLE_vii[index&255](a1|0,a2|0);
    }
  

    function dynCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
      return FUNCTION_TABLE_iiiiiii[index&127](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
    }
  

    function dynCall_ii(index,a1) {
      index = index|0;
      a1=a1|0;
      return FUNCTION_TABLE_ii[index&127](a1|0)|0;
    }
  

    function dynCall_viii(index,a1,a2,a3) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0;
      FUNCTION_TABLE_viii[index&255](a1|0,a2|0,a3|0);
    }
  

    function dynCall_v(index) {
      index = index|0;
      
      FUNCTION_TABLE_v[index&255]();
    }
  

    function dynCall_iiiii(index,a1,a2,a3,a4) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
      return FUNCTION_TABLE_iiiii[index&255](a1|0,a2|0,a3|0,a4|0)|0;
    }
  

    function dynCall_iii(index,a1,a2) {
      index = index|0;
      a1=a1|0; a2=a2|0;
      return FUNCTION_TABLE_iii[index&255](a1|0,a2|0)|0;
    }
  

    function dynCall_iiiiii(index,a1,a2,a3,a4,a5) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
      return FUNCTION_TABLE_iiiiii[index&127](a1|0,a2|0,a3|0,a4|0,a5|0)|0;
    }
  

    function dynCall_viiii(index,a1,a2,a3,a4) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
      FUNCTION_TABLE_viiii[index&255](a1|0,a2|0,a3|0,a4|0);
    }
  
function b0(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_iiii(0);return 0; }
  function _fprintf__wrapper(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; return _fprintf(p0|0,p1|0,p2|0)|0; }
  function b1(p0,p1,p2,p3,p4,p5,p6) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0;p6 = p6|0; nullFunc_viiiiiii(1); }
  function b2(p0,p1,p2,p3,p4) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; nullFunc_viiiii(2); }
  function b3(p0) { p0 = p0|0; nullFunc_vi(3); }
  function b4(p0,p1) { p0 = p0|0;p1 = p1|0; nullFunc_vii(4); }
  function b5(p0,p1,p2,p3,p4,p5) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; nullFunc_iiiiiii(5);return 0; }
  function b6(p0) { p0 = p0|0; nullFunc_ii(6);return 0; }
  function b7(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_viii(7); }
  function b8() { ; nullFunc_v(8); }
  function _abort__wrapper() { ; _abort(); }
  function b9(p0,p1,p2,p3) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; nullFunc_iiiii(9);return 0; }
  function b10(p0,p1) { p0 = p0|0;p1 = p1|0; nullFunc_iii(10);return 0; }
  function b11(p0,p1,p2,p3,p4) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; nullFunc_iiiiii(11);return 0; }
  function b12(p0,p1,p2,p3) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; nullFunc_viiii(12); }
  // EMSCRIPTEN_END_FUNCS
  var FUNCTION_TABLE_iiii = [b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,_sn_write,b0,b0,b0,b0,b0,_read_png_stream,b0,b0
  ,b0,_fprintf__wrapper,b0,b0,_get_raw_image,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,_alloc_small,_alloc_large,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,_png_create_struct_2,b0,b0,b0,b0,b0,b0,b0,b0,_png_zalloc,b0,_inflateInit_,b0,b0,_zcalloc,b0,_do_read
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0];
  var FUNCTION_TABLE_viiiiiii = [b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
  ,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,_merged_2v_upsample,b1,_merged_1v_upsample,b1,b1,b1,b1,b1,b1,b1,_post_process_1pass,_post_process_prepass
  ,_post_process_2pass,b1,_sep_upsample,b1,b1];
  var FUNCTION_TABLE_viiiii = [b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,_grayscale_convert,_ycc_rgb_convert,_gray_rgb_convert,_null_convert,_ycck_cmyk_convert,b2
  ,_jpeg_idct_1x1,_jpeg_idct_2x2,_jpeg_idct_4x4,_jpeg_idct_islow,_jpeg_idct_ifast,_jpeg_idct_float,b2,b2,b2];
  var FUNCTION_TABLE_vi = [b3,_j2k_read_soc,_j2k_read_sot,_j2k_read_sod,_j2k_read_eoc,_j2k_read_siz,_j2k_read_cod,_j2k_read_coc,_j2k_read_rgn,_j2k_read_qcd,_j2k_read_qcc,_j2k_read_poc,_j2k_read_tlm,_j2k_read_plm,_j2k_read_plt,_j2k_read_ppm,_j2k_read_ppt,_j2k_read_crg,_j2k_read_com,_j2k_read_unk,b3,b3,b3,b3,b3,b3,b3,b3,b3
  ,b3,b3,b3,b3,b3,_free,b3,_reset_input_controller,_start_input_pass,_finish_input_pass,_reset_marker_reader,b3,b3,b3,b3,_prepare_for_output_pass,_finish_output_pass,_start_pass_merged_upsample,b3,b3,b3,b3,_start_pass_phuff_decoder,b3,b3,b3,b3,b3,b3,b3
  ,b3,_start_pass_upsample,b3,b3,b3,b3,b3,b3,b3,b3,_error_exit,b3,_output_message,b3,_reset_error_mgr,b3,b3,b3,b3,b3,b3,_realize_virt_arrays,b3,b3,b3,_self_destruct,b3,_finish_pass_1_quant,_new_color_map_1_quant,b3
  ,b3,b3,b3,b3,b3,_new_color_map_2_quant,b3,_finish_pass1,b3,b3,_finish_pass2,_init_source1874,b3,b3,b3,_term_source1877,_start_input_pass1892,_start_output_pass,b3,b3,b3,b3,b3,_start_pass_dcolor,b3,b3,b3,b3,b3,_start_pass
  ,b3,b3,b3,b3,b3,b3,_start_pass_huff_decoder,b3,b3,b3,b3,b3,_dwt_decode_1,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
  ,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
  ,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
  ,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
  ,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3];
  var FUNCTION_TABLE_vii = [b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_png_read_info,b4
  ,_png_read_update_info,b4,b4,_png_read_image,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_start_pass_dpost,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_emit_message,b4,_format_message,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,_free_pool,b4,_start_pass_1_quant,b4,b4,b4
  ,b4,b4,b4,b4,_start_pass_2_quant,b4,b4,b4,b4,b4,b4,b4,b4,_skip_input_data1876,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,_start_pass_main,b4,b4,b4,b4,b4,_png_free,b4,b4,b4,b4,_png_warning,_png_error,b4,b4,_png_zfree,b4,b4,b4,b4,_zcfree,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
  ,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4];
  var FUNCTION_TABLE_iiiiiii = [b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,_request_virt_sarray,_request_virt_barray,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5,b5,b5,b5,b5,b5,b5];
  var FUNCTION_TABLE_ii = [b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,_png_create_info_struct,b6,b6,b6,b6,b6,_png_set_interlace_handling
  ,b6,b6,_malloc,b6,b6,b6,_consume_markers,b6,b6,b6,b6,_read_markers,_read_restart_marker,_skip_variable,_get_interesting_appn,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,_fill_input_buffer1875,b6,b6,b6,b6,b6,_consume_data,b6,_dummy_consume_data,b6,b6,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,b6,b6,b6,b6,b6];
  var FUNCTION_TABLE_viii = [b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,_png_destroy_read_struct,_png_set_read_fn,_png_mem_read_data,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,_png_destroy_struct_2,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,_png_default_read_data,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
  ,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7];
  var FUNCTION_TABLE_v = [b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,_abort__wrapper,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
  ,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8];
  var FUNCTION_TABLE_iiiii = [b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,_png_create_read_struct,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,_alloc_sarray,_alloc_barray,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,_snprintf,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
  ,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9];
  var FUNCTION_TABLE_iii = [b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,_decode_mcu_DC_first,_decode_mcu_AC_first,_decode_mcu_DC_refine,_decode_mcu_AC_refine,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,_jpeg_resync_to_restart,b10,b10,b10,b10,_decompress_data,b10,_decompress_onepass,_decompress_smooth_data,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,_decode_mcu,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,_png_malloc,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10];
  var FUNCTION_TABLE_iiiiii = [b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
  ,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
  ,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,_access_virt_sarray,_access_virt_barray,b11,b11,b11,b11,b11,b11
  ,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
  ,b11,b11,b11,b11,b11,b11,b11,b11,b11];
  var FUNCTION_TABLE_viiii = [b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,_h2v2_merged_upsample,b12,_h2v1_merged_upsample,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,_noop_upsample,_fullsize_upsample,_h2v1_fancy_upsample,_h2v1_upsample,_h2v2_fancy_upsample,_h2v2_upsample,_int_upsample,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,_color_quantize3
  ,_color_quantize,_quantize3_ord_dither,_quantize_ord_dither,_quantize_fs_dither,b12,b12,_prescan_quantize,b12,_pass2_fs_dither,_pass2_no_dither,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,b12,b12,b12,b12,b12,b12,_process_data_context_main,_process_data_simple_main,_process_data_crank_post,b12,b12,b12,b12,_png_set_mem_fn,_png_set_error_fn,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
  ,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12];

    return { _testSetjmp: _testSetjmp, _i64Subtract: _i64Subtract, _strcat: _strcat, _free: _free, _main: _main, _realloc: _realloc, _i64Add: _i64Add, _strlen: _strlen, _memset: _memset, _malloc: _malloc, _saveSetjmp: _saveSetjmp, _memcpy: _memcpy, _bitshift64Lshr: _bitshift64Lshr, _strcpy: _strcpy, _calloc: _calloc, _bitshift64Shl: _bitshift64Shl, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii, dynCall_viiiiiii: dynCall_viiiiiii, dynCall_viiiii: dynCall_viiiii, dynCall_vi: dynCall_vi, dynCall_vii: dynCall_vii, dynCall_iiiiiii: dynCall_iiiiiii, dynCall_ii: dynCall_ii, dynCall_viii: dynCall_viii, dynCall_v: dynCall_v, dynCall_iiiii: dynCall_iiiii, dynCall_iii: dynCall_iii, dynCall_iiiiii: dynCall_iiiiii, dynCall_viiii: dynCall_viiii };
  })
  // EMSCRIPTEN_END_ASM
  ({ "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array }, { "abort": abort, "assert": assert, "asmPrintInt": asmPrintInt, "asmPrintFloat": asmPrintFloat, "min": Math_min, "nullFunc_iiii": nullFunc_iiii, "nullFunc_viiiiiii": nullFunc_viiiiiii, "nullFunc_viiiii": nullFunc_viiiii, "nullFunc_vi": nullFunc_vi, "nullFunc_vii": nullFunc_vii, "nullFunc_iiiiiii": nullFunc_iiiiiii, "nullFunc_ii": nullFunc_ii, "nullFunc_viii": nullFunc_viii, "nullFunc_v": nullFunc_v, "nullFunc_iiiii": nullFunc_iiiii, "nullFunc_iii": nullFunc_iii, "nullFunc_iiiiii": nullFunc_iiiiii, "nullFunc_viiii": nullFunc_viiii, "invoke_iiii": invoke_iiii, "invoke_viiiiiii": invoke_viiiiiii, "invoke_viiiii": invoke_viiiii, "invoke_vi": invoke_vi, "invoke_vii": invoke_vii, "invoke_iiiiiii": invoke_iiiiiii, "invoke_ii": invoke_ii, "invoke_viii": invoke_viii, "invoke_v": invoke_v, "invoke_iiiii": invoke_iiiii, "invoke_iii": invoke_iii, "invoke_iiiiii": invoke_iiiiii, "invoke_viiii": invoke_viiii, "_fabs": _fabs, "_sin": _sin, "_printf": _printf, "_llvm_pow_f64": _llvm_pow_f64, "_send": _send, "_fread": _fread, "_ctime": _ctime, "_fmod": _fmod, "_lseek": _lseek, "_open": _open, "_ceil": _ceil, "_asctime": _asctime, "_fputc": _fputc, "_fsync": _fsync, "___buildEnvironment": ___buildEnvironment, "_longjmp": _longjmp, "_fflush": _fflush, "_pwrite": _pwrite, "_strerror_r": _strerror_r, "_localtime_r": _localtime_r, "_tzset": _tzset, "__reallyNegative": __reallyNegative, "_sbrk": _sbrk, "_atan2": _atan2, "_emscripten_memcpy_big": _emscripten_memcpy_big, "_fileno": _fileno, "_sysconf": _sysconf, "___setErrNo": ___setErrNo, "_ferror": _ferror, "_putchar": _putchar, "_cos": _cos, "_fseek": _fseek, "_asctime_r": _asctime_r, "_lrintf": _lrintf, "_pread": _pread, "_getenv": _getenv, "_fclose": _fclose, "_localtime": _localtime, "_write": _write, "_ftell": _ftell, "___errno_location": ___errno_location, "_stat": _stat, "_recv": _recv, "_copysign": _copysign, "_fgetc": _fgetc, "_mkport": _mkport, "_getrusage": _getrusage, "__exit": __exit, "_read": _read, "_copysignl": _copysignl, "_abort": _abort, "_fwrite": _fwrite, "_time": _time, "_fprintf": _fprintf, "_strerror": _strerror, "_emscripten_longjmp": _emscripten_longjmp, "_abs": _abs, "__formatString": __formatString, "_rint": _rint, "_sqrt": _sqrt, "_fopen": _fopen, "_exit": _exit, "_close": _close, "_mktime": _mktime, "_fmodl": _fmodl, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "ctlz_i8": ctlz_i8, "NaN": NaN, "Infinity": Infinity, "_stderr": _stderr, "_stdout": _stdout }, buffer);
  var _testSetjmp = Module["_testSetjmp"] = asm["_testSetjmp"];
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _strcat = Module["_strcat"] = asm["_strcat"];
var _free = Module["_free"] = asm["_free"];
var _main = Module["_main"] = asm["_main"];
var _realloc = Module["_realloc"] = asm["_realloc"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _strlen = Module["_strlen"] = asm["_strlen"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _saveSetjmp = Module["_saveSetjmp"] = asm["_saveSetjmp"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _strcpy = Module["_strcpy"] = asm["_strcpy"];
var _calloc = Module["_calloc"] = asm["_calloc"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_viiiiiii = Module["dynCall_viiiiiii"] = asm["dynCall_viiiiiii"];
var dynCall_viiiii = Module["dynCall_viiiii"] = asm["dynCall_viiiii"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_vii = Module["dynCall_vii"] = asm["dynCall_vii"];
var dynCall_iiiiiii = Module["dynCall_iiiiiii"] = asm["dynCall_iiiiiii"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_viii = Module["dynCall_viii"] = asm["dynCall_viii"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];
var dynCall_iiiiii = Module["dynCall_iiiiii"] = asm["dynCall_iiiiii"];
var dynCall_viiii = Module["dynCall_viiii"] = asm["dynCall_viiii"];
  
  Runtime.stackAlloc = asm['stackAlloc'];
  Runtime.stackSave = asm['stackSave'];
  Runtime.stackRestore = asm['stackRestore'];
  Runtime.setTempRet0 = asm['setTempRet0'];
  Runtime.getTempRet0 = asm['getTempRet0'];
  

// TODO: strip out parts of this we do not need

//======= begin closure i64 code =======

// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */

var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };


  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.

    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };


  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.


  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};


  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }

    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };


  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };


  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };


  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }

    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }

    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));

    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };


  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.


  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;


  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);


  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);


  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);


  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);


  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);


  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);


  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };


  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };


  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (this.isZero()) {
      return '0';
    }

    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }

    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));

    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);

      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };


  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };


  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };


  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };


  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };


  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };


  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };


  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };


  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }

    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }

    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };


  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };


  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };


  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }

    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }

    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }

    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));

      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);

      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }

      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }

      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };


  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };


  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };


  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };


  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };


  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };


  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };

  //======= begin jsbn =======

  var navigator = { appName: 'Modern Browser' }; // polyfill a little

  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/

  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */

  // Basic JavaScript BN library - subset useful for RSA encryption.

  // Bits per digit
  var dbits;

  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);

  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }

  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }

  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.

  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }

  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);

  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;

  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }

  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }

  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }

  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }

  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }

  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }

  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }

  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }

  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }

  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }

  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }

  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }

  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }

  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }

  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }

  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }

  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }

  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }

  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;

  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }

  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }

  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }

  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }

  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }

  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;

  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }

  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }

  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;

  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;

  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);

  // jsbn2 stuff

  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }

  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }

  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }

  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }

  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }

  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }

  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }

  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;

  //======= end jsbn =======

  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();

//======= end closure i64 code =======



// === Auto-generated postamble setup entry stuff ===

if (memoryInitializer) {
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    var data = Module['readBinary'](memoryInitializer);
    HEAPU8.set(data, STATIC_BASE);
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      for (var i = 0; i < data.length; i++) {
        assert(HEAPU8[STATIC_BASE + i] === 0, "area for memory initializer should not have been touched before it's loaded");
      }
      HEAPU8.set(data, STATIC_BASE);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}

function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun'] && shouldRunNow) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram'] || '/bin/this.program'), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    if (!Module['noExitRuntime']) {
      exit(ret);
    }
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['_main'] && shouldRunNow) {
      Module['callMain'](args);
    }

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      if (!ABORT) doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status) {
  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;

  // exit the runtime
  exitRuntime();

  // TODO We should handle this differently based on environment.
  // In the browser, the best we can do is throw an exception
  // to halt execution, but in node we could process.exit and
  // I'd imagine SM shell would have something equivalent.
  // This would let us set a proper exit status (which
  // would be great for checking test exit statuses).
  // https://github.com/kripken/emscripten/issues/1371

  // throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

function abort(text) {
  if (text) {
    Module.print(text);
    Module.printErr(text);
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '';

  throw 'abort() at ' + stackTrace() + extra;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



//# sourceMappingURL=mindtct.js.map